import ctypes
import ctypes.wintypes as wintypes
from typing import Union

from pymhf.core._types import FUNCDEF
import nmspy.data.common as common
import nmspy.data.structs as structs

FUNC_CALL_SIGS: dict[str, Union[FUNCDEF, dict[str, FUNCDEF]]] = {
    "packed_store_active___un_3C_unf_3E_vyf": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_ulonglong,  # const __m128i *
            ctypes.c_ulonglong,  # const __m128i *
        ]
    ),
    "dot___vyf_3C_3_3E_vyf_3C_3_3E_": FUNCDEF(
        restype=ctypes.c_char * 0x10,  # __m128
        argtypes=[
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "length___vyf_3C_3_3E_": FUNCDEF(
        restype=ctypes.c_char * 0x10,  # __m128
        argtypes=[
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "cross___vyf_3C_3_3E_vyf_3C_3_3E_": FUNCDEF(
        restype=ctypes.c_char * 0x10,  # __m128
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "ClosestPointOnTriangle___vyf_3C_3_3E_vyf_3C_3_3E_vyf_3C_3_3E_vyf_3C_3_3E_": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "IntersectRaySphere___vyf_3C_3_3E_vyf_3C_3_3E_vyf_3C_3_3E_vyf": FUNCDEF(
        restype=ctypes.c_char * 0x10,  # __m128
        argtypes=[
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "sub_14001D810": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # const __m128i *
        ]
    ),
    "Module_InitializeHelper": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            wintypes.HMODULE,  # HMODULE
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "LoadModule": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "ValidateModuleHandle": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "NvAPI_Initialize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "sub_14002A2F0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_ulonglong,  # _WORD *
        ]
    ),
    "sub_14002A3F0": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # _WORD *
        ]
    ),
    "sub_14002A450": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int16 *
        ]
    ),
    "sub_14002A4B0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[]
    ),
    "nvGetSystemDirectoryW": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[]
    ),
    "sub_14002A620": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # wchar_t *
            ctypes.c_ulonglong,  # wchar_t *
        ]
    ),
    "sub_14002A790": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # _QWORD *
            ctypes.c_int64,  # __int64
        ]
    ),
    "StringCchCopyW": {
        "STRSAFE_LPWSTR, size_t, STRSAFE_LPCWSTR": FUNCDEF(
            restype=wintypes.LONG,  # HRESULT
            argtypes=[
                ctypes.c_int64,  # STRSAFE_LPWSTR
                ctypes.c_int32,  # size_t
                ctypes.c_int64,  # STRSAFE_LPCWSTR
            ]
        ),
        "wchar_t *, unsigned __int64, const wchar_t *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # wchar_t *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # const wchar_t *
            ]
        ),
    },
    "StringCbCopyW": FUNCDEF(
        restype=wintypes.LONG,  # HRESULT
        argtypes=[
            ctypes.c_int64,  # STRSAFE_LPWSTR
            ctypes.c_int32,  # size_t
            ctypes.c_int64,  # STRSAFE_LPCWSTR
        ]
    ),
    "StringCchCopyNW": FUNCDEF(
        restype=wintypes.LONG,  # HRESULT
        argtypes=[
            ctypes.c_int64,  # STRSAFE_LPWSTR
            ctypes.c_int32,  # size_t
            ctypes.c_int64,  # STRSAFE_PCNZWCH
            ctypes.c_int32,  # size_t
        ]
    ),
    "StringCchCatW": {
        "STRSAFE_LPWSTR, size_t, STRSAFE_LPCWSTR": FUNCDEF(
            restype=wintypes.LONG,  # HRESULT
            argtypes=[
                ctypes.c_int64,  # STRSAFE_LPWSTR
                ctypes.c_int32,  # size_t
                ctypes.c_int64,  # STRSAFE_LPCWSTR
            ]
        ),
        "wchar_t *, unsigned __int64, const wchar_t *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # wchar_t *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # const wchar_t *
            ]
        ),
    },
    "StringCbCatW": FUNCDEF(
        restype=wintypes.LONG,  # HRESULT
        argtypes=[
            ctypes.c_int64,  # STRSAFE_LPWSTR
            ctypes.c_int32,  # size_t
            ctypes.c_int64,  # STRSAFE_LPCWSTR
        ]
    ),
    "sub_14002AC40": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "sub_14002ADB0": FUNCDEF(
        restype=wintypes.HMODULE,  # HMODULE
        argtypes=[
            ctypes.c_ulonglong,  # wchar_t *
            ctypes.c_int32,  # int
        ]
    ),
    "sub_14002AEC0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "sub_14002B010": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            wintypes.LPCWSTR,  # LPCWSTR
        ]
    ),
    "sub_14002B1D0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # __int16 *
        argtypes=[]
    ),
    "sub_14002B480": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_int64,  # STRSAFE_LPCWSTR
        ]
    ),
    "sub_14002B510": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_int64,  # STRSAFE_PCNZWCH
            ctypes.c_int32,  # size_t
        ]
    ),
    "sub_14002B5B0": FUNCDEF(
        restype=ctypes.c_uint32,  # DWORD
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # HLOCAL *
        ]
    ),
    "sub_14002B6F0": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "sub_14002B920": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # __int64 *
            ctypes.c_ulonglong,  # _QWORD *
        ]
    ),
    "sub_14002BAF0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_int64,  # STRSAFE_PCNZWCH
        ]
    ),
    "sub_14002BC20": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_ulonglong,  # wchar_t *
            ctypes.c_int64,  # __int64
        ]
    ),
    "sub_14002BE10": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_ulonglong,  # wchar_t *
            ctypes.c_int64,  # __int64
        ]
    ),
    "sub_14002C0C0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_ulonglong,  # wchar_t *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # __int64 *
        ]
    ),
    "sub_14002C290": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_ulonglong,  # wchar_t *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # _QWORD *
        ]
    ),
    "nvLoadLibraryFromTrustedLocationExW": FUNCDEF(
        restype=wintypes.HMODULE,  # HMODULE
        argtypes=[
            wintypes.LPCWSTR,  # LPCWSTR
            ctypes.c_int32,  # int
        ]
    ),
    "nvGetDriverStoreW": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_int64,  # __int64
        ]
    ),
    "sub_14002C820": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_int64,  # STRSAFE_LPCWSTR
            ctypes.c_int64,  # STRSAFE_LPCWSTR
        ]
    ),
    "nvGetDriverRegkeyW": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_int64,  # __int64
        ]
    ),
    "nvDetectDisplayDriverModuleRegistrationW": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_int64,  # STRSAFE_LPCWSTR
        ]
    ),
    "nvDetectDisplayDriverModuleLocationW": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_int64,  # STRSAFE_LPCWSTR
        ]
    ),
    "nvLoadSystemLibraryExW": FUNCDEF(
        restype=wintypes.HMODULE,  # HMODULE
        argtypes=[
            ctypes.c_int64,  # STRSAFE_LPCWSTR
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAnimVectorBlendNode::ClassPointerRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureComponent::sAccessoryInfo::sLightInfo::sLightInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent::sAccessoryInfo::sLightInfo *
        ]
    ),
    "TkHandle::Root": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cTkVector3::cTkVector3": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavChunkInfo *
        ]
    ),
    "cTkColour::cTkColour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # NVGcontext *
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
        ]
    ),
    "cTkSeed::cTkSeed": {
        "cTkSeed *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkSeed *
            ]
        ),
        "cTkSeed *, const cTkSeed *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkSeed *
                ctypes.c_ulonglong,  # const cTkSeed *
            ]
        ),
        "cTkSeed *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkSeed *
                ctypes.c_int32,  # int
            ]
        ),
        "cTkSeed *, unsigned __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkSeed *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
        "cTkSeed *, const cTkVector3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkSeed *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
    },
    "cTkMatrix34::cTkMatrix34": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcScanner::TerrainResourcePosition::TerrainResourcePosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner::TerrainResourcePosition *
        ]
    ),
    "cTkResourceDescriptor::cTkResourceDescriptor": {
        "cTkResourceDescriptor *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ]
        ),
        "cTkResourceDescriptor *, const cTkResourceDescriptor *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkResourceDescriptor *
                ctypes.c_ulonglong,  # const cTkResourceDescriptor *
            ]
        ),
    },
    "cTkSmartResHandle::cTkSmartResHandle": {
        "cTkSmartResHandle *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkSmartResHandle *
            ]
        ),
        "cTkSmartResHandle *, const cTkSmartResHandle *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkSmartResHandle *
                ctypes.c_ulonglong,  # const cTkSmartResHandle *
            ]
        ),
    },
    "cTkTypedSmartResHandle<cTkResource>::~cTkTypedSmartResHandle<cTkResource>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTypedSmartResHandle<cTkResource> *
        ]
    ),
    "cTkSmartResHandle::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmartResHandle *
        ]
    ),
    "cTkRigidBody::GetIndexOfProxyOfBody": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "sTkInputDeviceIcons::~sTkInputDeviceIcons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sTkInputDeviceIcons *
        ]
    ),
    "BaseIndex::BaseIndex": {
        "BaseIndex *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # BaseIndex *
            ]
        ),
        "BaseIndex *, unsigned __int16": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # BaseIndex *
                ctypes.c_uint16,  # unsigned __int16
            ]
        ),
        "BaseIndex *, const BaseIndex *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # BaseIndex *
                ctypes.c_ulonglong,  # const BaseIndex *
            ]
        ),
    },
    "cTkReplicatedVariableBase::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableBase *
        ]
    ),
    "cGcNotificationSequence::ShowFutureObjectives": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkAtomicLinearMemoryPool *
        ]
    ),
    "cTkComponent::IsLocalPlayerEntityOwner": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdBase *
        ]
    ),
    "CAkParameterNodeBase::SetAdvSettingsParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkComponent::GetUpdatePeriod": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cTkComponentPool::cTkComponentPool": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentPool *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # {'return': 'cTkComponent *', 'args': ['__int16', 'cTkComponent **', 'const char *']}
            ctypes.c_ulonglong,  # {'return': 'cTkComponent *', 'args': ['cTkComponent *']}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['cTkComponent *']}
            ctypes.c_ulonglong,  # cTkComponentUpdater *
            ctypes.c_int16,  # __int16
            ctypes.c_int16,  # __int16
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkComponentManager::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponentManager *
        argtypes=[]
    ),
    "cTkComponentManager::GetTypeIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentManager *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcDiscoveryPayload::cGcDiscoveryPayload": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPayload *
        ]
    ),
    "cGcDiscoveryData::cGcDiscoveryData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryData *
        ]
    ),
    "cGcDiscoveryData::Invalidate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryData *
        ]
    ),
    "cGcResourceElement::~cGcResourceElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceElement *
        ]
    ),
    "cTkAnimationComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
        ]
    ),
    "cTkAnimationComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkDynamicChainComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRotationComponent *
        ]
    ),
    "ComponentInstancercTkAnimationComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercTkAnimationComponent *
        ]
    ),
    "ComponentInstancercTkAnimationComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercTkAnimationComponent::ConstructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkAnimationComponent::cTkAnimationComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
        ]
    ),
    "ComponentInstancercTkAnimationComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkNodeAnimationController::~cTkNodeAnimationController": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
        ]
    ),
    "StackContainer<std::vector<cGcNavMeshHandle,StackAllocator<cGcNavMeshHandle,32,-1>>,32,-1>::~StackContainer<std::vector<cGcNavMeshHandle,StackAllocator<cGcNavMeshHandle,32,-1>>,32,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cGcNavMeshHandle,StackAllocator<cGcNavMeshHandle,32,-1> >,32,-1> *
        ]
    ),
    "ComponentLoadInstancercTkAnimationComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentLoadInstancercTkAnimationComponent *
        ]
    ),
    "cGcCustomInventoryComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomInventoryComponent *
        ]
    ),
    "cGcCustomInventoryComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomInventoryComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcCustomInventoryComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcCustomInventoryComponent *
        ]
    ),
    "ComponentInstancercGcFleetHologramComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcCustomInventoryComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcCustomInventoryComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcFoliageComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFoliageComponent *
        ]
    ),
    "cGcFoliageComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFoliageComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFoliageComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpringAttachmentComponent *
        ]
    ),
    "ComponentInstancercGcFoliageComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcFoliageComponent *
        ]
    ),
    "ComponentInstancercGcNPCInteractiveObjectComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcFoliageComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcSentinelRobotComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkRandomComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRandomComponent *
        ]
    ),
    "cTkRandomComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRandomComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercTkRandomComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercTkRandomComponent *
        ]
    ),
    "ComponentInstancercGcWiringSocketComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercTkRandomComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcWiringSocketComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkNetReplicatedEntityComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponent *
        ]
    ),
    "cTkNetReplicatedEntityComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercTkNetReplicatedEntityComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercTkNetReplicatedEntityComponent *
        ]
    ),
    "ComponentInstancercTkNetReplicatedEntityComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercTkNetReplicatedEntityComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcShipOwnershipComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkReplicatedVariableTemplated<eCamouflageState>::~cTkReplicatedVariableTemplated<eCamouflageState>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<float> *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cGcProjectileImpact::cGcProjectileImpact": {
        "cGcProjectileImpact *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcProjectileImpact *
            ]
        ),
        "cGcProjectileImpact *, const cGcProjectileImpact *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcProjectileImpact *
                ctypes.c_ulonglong,  # const cGcProjectileImpact *
            ]
        ),
    },
    "cGcLaserBeam::cGcLaserBeam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeam *
        ]
    ),
    "cGcProjectileData::~cGcProjectileData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileData *
        ]
    ),
    "cTkReplicatedVariableTemplated<cTkVector3>::~cTkReplicatedVariableTemplated<cTkVector3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cTkVector3> *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cGcWeapon::GetActiveInventoryItem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcInventoryElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeapon *
        ]
    ),
    "AkDevice::PluginContext<AK::IAkAudioDeviceEffectPluginContext>::GetDownstreamGain": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cEgSceneNode::GetClassOverride": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAtomicLinearMemoryPool *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcVehicleWeapon::GetNumModes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # WaitSeconds *
        ]
    ),
    "cGcPlayerEffectsComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEffectsComponent *
        ]
    ),
    "cGcPlayerEffectsComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEffectsComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerEffectsComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEffectsComponent *
        ]
    ),
    "ComponentInstancercGcPlayerEffectsComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcPlayerEffectsComponent *
        ]
    ),
    "ComponentInstancercGcPlayerEffectsComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcPlayerEffectsComponent::ConstructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcPlayerEffectsComponent::cGcPlayerEffectsComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEffectsComponent *
        ]
    ),
    "ComponentInstancercGcPlayerEffectsComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcScannableComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannableComponent *
        ]
    ),
    "cGcScannableComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannableComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcScannableComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcScannableComponent *
        ]
    ),
    "ComponentInstancercGcScannableComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcScannableComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcScannableComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcDestructableComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponent *
        ]
    ),
    "cGcDestructableComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcDestructableComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcDestructableComponent *
        ]
    ),
    "ComponentInstancercGcDestructableComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcDestructableComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcDestructableComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentLoadInstancercGcDestructableComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentLoadInstancercGcDestructableComponent *
        ]
    ),
    "cGcGrabbableComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabbableComponent *
        ]
    ),
    "cGcGrabbableComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabbableComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcGrabbableComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcGrabbableComponent *
        ]
    ),
    "ComponentInstancercGcGrabbableComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcGrabbableComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcGrabbableComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcLaserBeam::~cGcLaserBeam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeam *
        ]
    ),
    "cTkFixedString<127,char>::cTkFixedString<127,char>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcSpaceshipEngineEffects::Settings::Settings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipEngineEffects::Settings *
        ]
    ),
    "cGcAISpaceshipComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAISpaceshipComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcAISpaceshipComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcAISpaceshipComponent *
        ]
    ),
    "ComponentInstancercGcAISpaceshipComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcAISpaceshipComponent::ConstructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcAISpaceshipComponent::cGcAISpaceshipComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcSpaceshipEngineEffects::cGcSpaceshipEngineEffects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipEngineEffects *
        ]
    ),
    "cGcStagedJetEngine::cGcStagedJetEngine": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStagedJetEngine *
        ]
    ),
    "cGcAudioEmitterScaffold::cGcAudioEmitterScaffold": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioEmitterScaffold *
        ]
    ),
    "cTkRigidBody::cTkRigidBody": {
        "cTkRigidBody *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkRigidBody *
            ]
        ),
        "cTkRigidBody *, const cTkRigidBody *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkRigidBody *
                ctypes.c_ulonglong,  # const cTkRigidBody *
            ]
        ),
    },
    "ComponentInstancercGcAISpaceshipComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkCollision::~cTkCollision": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
        ]
    ),
    "cGcStagedJetEngine::~cGcStagedJetEngine": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStagedJetEngine *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcSpaceshipEngineEffects::~cGcSpaceshipEngineEffects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipEngineEffects *
        ]
    ),
    "cGcAudioEmitterScaffold::~cGcAudioEmitterScaffold": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioEmitterScaffold *
        ]
    ),
    "cGcInventoryStore::~cGcInventoryStore": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
        ]
    ),
    "ComponentLoadInstancercGcAISpaceshipComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentLoadInstancercGcAISpaceshipComponent *
        ]
    ),
    "cGcSpaceshipWeapons::UsingAmmo": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "hknpLodShapeFunctions::estimateNumShapeKeys": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipWeapons::IsReloading": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::GetHeatFactor": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::HasOverheated": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::GetLastFireTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::GetLastModeChangeTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::GetModeIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::SetModeIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSpaceshipWeapons::GetPrimaryStat": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::IsCriticalShot": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::GetListOfModesIfActiveOrNot": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
            ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
        ]
    ),
    "cGcSpaceshipComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcSpaceshipComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcSpaceshipComponent *
        ]
    ),
    "ComponentInstancercGcSpaceshipComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcSpaceshipComponent::ConstructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcSpaceshipComponent::cGcSpaceshipComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipWeapons::cGcSpaceshipWeapons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcVehicleCockpit::cGcVehicleCockpit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
        ]
    ),
    "ComponentInstancercGcSpaceshipComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcSpaceshipWeapons::~cGcSpaceshipWeapons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcProjectileImpact::~cGcProjectileImpact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileImpact *
        ]
    ),
    "cGcVehicleCockpit::~cGcVehicleCockpit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cTkReplicatedVariableTemplated<cTkPhysRelMat34>::~cTkReplicatedVariableTemplated<cTkPhysRelMat34>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cTkPhysRelMat34> *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cGcCustomisationComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationComponent *
        ]
    ),
    "cGcCustomisationComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcCustomisationComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcCustomisationComponent *
        ]
    ),
    "ComponentInstancercGcCustomisationComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentLoadInstancercGcCustomisationComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentLoadInstancercGcCustomisationComponent *
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcMissileComponent>::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cTkRotationComponent> *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcSpaceshipComponent>::UpdateComponentsRender": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcSpaceshipComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcSpaceshipComponent>::UpdateComponentsPostPhysics": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcSpaceshipComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcHologramComponent>::UpdateComponents": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cTkRotationComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkReplicatedVariable<cTkPhysRelMat34>::IsOwned": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariable<cTkPhysRelMat34> *
        ]
    ),
    "cTkReplicatedVariable<cTkPhysRelMat34>::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariable<cTkPhysRelMat34> *
        ]
    ),
    "cTkReplicatedVariableTemplated<cTkPhysRelMat34>::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cTkPhysRelMat34> *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cTkQuadVertex::cTkQuadVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuadVertex *
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcAISpaceshipComponent>::UpdateComponentsRender": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcAISpaceshipComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcAISpaceshipComponent>::UpdateComponentsPostPhysics": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcAISpaceshipComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkReplicatedVariable<cTkVector3>::IsOwned": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariable<cTkVector3> *
        ]
    ),
    "cTkReplicatedVariable<cTkVector3>::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariable<cTkVector3> *
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcPlayerEffectsComponent>::UpdateComponentsRender": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcPlayerEffectsComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkReplicatedVariableTemplated<TkID<128>>::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<TkID<128> > *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cTkReplicatedVariable<uchar>::IsOwned": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariable<float> *
        ]
    ),
    "cTkReplicatedVariable<eCamouflageState>::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariable<float> *
        ]
    ),
    "cTkReplicatedVariableTemplated<bool>::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<bool> *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cTkDynamicArray<cTkSketchNodeConnections>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkSketchNodeConnections> *
        ]
    ),
    "cTkDynamicArray<cTkProceduralTextureChosenOptionSampler>::~cTkDynamicArray<cTkProceduralTextureChosenOptionSampler>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkProceduralTextureChosenOptionSampler> *
        ]
    ),
    "cTkDynamicArray<cGcPersistentBaseTypes>::cTkDynamicArray<cGcPersistentBaseTypes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTechnologyRequirement> *
        ]
    ),
    "cTkUserIdBase<cTkFixedString<64,char>>::IsValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cTkLinearHashTable<void const *,cTkFixedString<256,char>,-1,0,cTkLinearHashTableHash<void const *>>::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<void const *,cTkFixedString<256,char>,-1,0,cTkLinearHashTableHash<void const *> > *
        ]
    ),
    "cTkStringUtilities<char>::CopyIntoBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkLinearHashTable<TkID<128>,int,-1,0,cTkLinearHashTableHash<TkID<128>>>::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<TkID<128>,int,-1,0,cTkLinearHashTableHash<TkID<128> > > *
        ]
    ),
    "cTkLinearHashTable<TkID<128>,int,-1,0,cTkLinearHashTableHash<TkID<128>>>::ChangeNodeCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<TkID<128>,int,-1,0,cTkLinearHashTableHash<TkID<128> > > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAsteroidTypeMask::cGcAsteroidTypeMask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidTypeMask *
        ]
    ),
    "cTkLinearHashTable<cTkFixedString<256,char>,cTkResourceFilterList *,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char>>>::ChangeNodeCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<cTkFixedString<256,char>,cTkResourceFilterList *,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char> > > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkReplicatedVariable<bool>::IsLocalPlayerEntityOwner": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariable<float> *
        ]
    ),
    "cTkReplicatedVariable<cTkVector3>::IsLocalPlayerEntityOwner": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariable<cTkVector3> *
        ]
    ),
    "cTkReplicatedVariable<cTkPhysRelMat34>::IsLocalPlayerEntityOwner": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariable<cTkPhysRelMat34> *
        ]
    ),
    "cGcDebugEditor::IsActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugEditor *
        ]
    ),
    "cGcPortalComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalComponent *
        ]
    ),
    "cGcPortalComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcPortalComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcPortalComponent *
        ]
    ),
    "ComponentInstancercGcTriggerActionComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcPortalComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcSimpleIkRecoilComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkDebugRenderer::ClearBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDebugRenderer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcGraphicsManager::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGraphicsManager *
        ]
    ),
    "cGcMaintenanceComponent::~cGcMaintenanceComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::RefreshPersistency": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcMaintenanceComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcMaintenanceComponent *
        ]
    ),
    "ComponentInstancercGcMaintenanceComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcMaintenanceComponent::ConstructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcMaintenanceComponent::cGcMaintenanceComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "ComponentInstancercGcMaintenanceComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcEggMachineComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "cGcEggMachineComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEggMachineComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "ComponentInstancercGcEggMachineComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcEggMachineComponent *
        ]
    ),
    "ComponentInstancercGcEggMachineComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcEggMachineComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcEggMachineComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "sOwnedCreatureInfo::~sOwnedCreatureInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sOwnedCreatureInfo *
        ]
    ),
    "cTkModelResourceRenderer::~cTkModelResourceRenderer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelResourceRenderer *
        ]
    ),
    "cGcFrontendManager::RequestPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_uint32,  # eFrontendPage
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendManager::RequestPageWithHint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_uint32,  # eFrontendPage
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendManager::MessageBoxData::MessageBoxData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager::MessageBoxData *
        ]
    ),
    "cGcSlotIcon::cGcSlotIcon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSlotIcon *
        ]
    ),
    "cGcSlotIcon::Invalidate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSlotIcon *
        ]
    ),
    "MenuAction::MenuAction": {
        "MenuAction *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # MenuAction *
            ]
        ),
        "MenuAction *, const MenuAction *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # MenuAction *
                ctypes.c_ulonglong,  # const MenuAction *
            ]
        ),
        "MenuAction *, eQuickMenuActions, const cGcInventoryElement *, const bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # MenuAction *
                ctypes.c_uint32,  # eQuickMenuActions
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_ubyte,  # const bool
            ]
        ),
        "MenuAction *, eQuickMenuActions, const cGcInventoryElement *, int, const bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # MenuAction *
                ctypes.c_uint32,  # eQuickMenuActions
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_int32,  # int
                ctypes.c_ubyte,  # const bool
            ]
        ),
        "MenuAction *, eQuickMenuActions, const cGcInventoryElement *, const eWeaponMode, const bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # MenuAction *
                ctypes.c_uint32,  # eQuickMenuActions
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_uint32,  # const eWeaponMode
                ctypes.c_ubyte,  # const bool
            ]
        ),
    },
    "cGcBuildMenu::HasParentMenu": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
        ]
    ),
    "cGcGalaxyPathMenu::UpdateControlsString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyPathMenu *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "MessageBase::MessageBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # MessageBase *
            ctypes.c_float,  # float
        ]
    ),
    "OSDMessage::~OSDMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # OSDMessage *
        ]
    ),
    "cGcHUDManager::GetQuickMenu": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcQuickMenu *
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcPlayerExperienceDirector::GetFrigateFlyby": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFrigateFlyby *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcApplication::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::Data::Data": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication::Data *
        ]
    ),
    "cGcRealityManager::cGcRealityManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
        ]
    ),
    "cGcGameState::cGcGameState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
        ]
    ),
    "cGcPlayerShipOwnership::cGcPlayerShipOwnership": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
        ]
    ),
    "cGcPlacementArc::cGcPlacementArc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlacementArc *
        ]
    ),
    "cGcPlayerFreighterOwnership::cGcPlayerFreighterOwnership": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_uint32,  # eGcNetworkMessageType
        ]
    ),
    "cGcWonderManager::cGcWonderManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
        ]
    ),
    "cGcUserSettings::cGcUserSettings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
        ]
    ),
    "cGcPersistentInteractionsManager::cGcPersistentInteractionsManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionsManager *
        ]
    ),
    "cGcPlayerState::cGcPlayerState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcResourceCustomisation::cGcResourceCustomisation": {
        "cGcResourceCustomisation *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ]
        ),
        "cGcResourceCustomisation *, const cGcResourceCustomisation *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcResourceCustomisation *
                ctypes.c_ulonglong,  # const cGcResourceCustomisation *
            ]
        ),
    },
    "cGcPlayerVehicleOwnership::cGcPlayerVehicleOwnership": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
        ]
    ),
    "cGcSeasonalData::cGcSeasonalData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
        ]
    ),
    "cGcSimulation::cGcSimulation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulation *
        ]
    ),
    "cGcSceneManager::cGcSceneManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneManager *
        ]
    ),
    "cGcParticleManager::cGcParticleManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleManager *
        ]
    ),
    "cGcEnvironment::cGcEnvironment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
        ]
    ),
    "cGcSky::cGcSky": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
        ]
    ),
    "cGcProjectileManager::cGcProjectileManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileManager *
        ]
    ),
    "cGcAISpaceshipManager::cGcAISpaceshipManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
        ]
    ),
    "cGcPlayerExperienceDirector::cGcPlayerExperienceDirector": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayer::cGcPlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayerCommon::cGcPlayerCommon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommon *
        ]
    ),
    "cGcPlayerWeapon::cGcPlayerWeapon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cTkFixedArray<cGcMuzzleFlash,19>::cTkFixedArray<cGcMuzzleFlash,19>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcMuzzleFlash,19> *
        ]
    ),
    "cGcTerrainEditorBeam::cGcTerrainEditorBeam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
        ]
    ),
    "sGrabbedObjectInfo::sGrabbedObjectInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sGrabbedObjectInfo *
        ]
    ),
    "cGcPlayerInteract::cGcPlayerInteract": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcTargetHelper::cGcTargetHelper": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetHelper *
        ]
    ),
    "cGcHUDManager::cGcHUDManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cGcPlayerHUD::cGcPlayerHUD": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcHUD::cGcHUD": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUD *
        ]
    ),
    "cGcPlayerNotifications::cGcPlayerNotifications": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
        ]
    ),
    "cGcQuickMenu::cGcQuickMenu": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenu *
        ]
    ),
    "cGcQuickActionMenu::cGcQuickActionMenu": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
        ]
    ),
    "cGcTerrainEditMenu::cGcTerrainEditMenu": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditMenu *
        ]
    ),
    "cGcShipHUD::cGcShipHUD": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUD *
        ]
    ),
    "cGcFrontendManager::cGcFrontendManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
        ]
    ),
    "cGcFrontendPage::cGcFrontendPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcPurchaseableItem::cGcPurchaseableItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
        ]
    ),
    "cGcFrontendModelRenderer::cGcFrontendModelRenderer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
        ]
    ),
    "cTkModelResourceRenderer::cTkModelResourceRenderer": {
        "cTkModelResourceRenderer *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkModelResourceRenderer *
            ]
        ),
        "cTkModelResourceRenderer *, const cTkModelResourceRenderer *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkModelResourceRenderer *
                ctypes.c_ulonglong,  # const cTkModelResourceRenderer *
            ]
        ),
    },
    "cGcInWorldUIManager::cGcInWorldUIManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
        ]
    ),
    "cGcInWorldScreen::cGcInWorldScreen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldScreen *
        ]
    ),
    "cGcCameraManager::cGcCameraManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
        ]
    ),
    "cGcCameraBehaviourThirdPerson::cGcCameraBehaviourThirdPerson": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
        ]
    ),
    "cGcGraphicsManager::cGcGraphicsManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGraphicsManager *
        ]
    ),
    "cGcAudioManager::cGcAudioManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
        ]
    ),
    "cGcNetworkManager::cGcNetworkManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkManager *
        ]
    ),
    "cGcNetworkBandwidthMonitor::sBandwidthData::sBandwidthData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBandwidthMonitor::sBandwidthData *
        ]
    ),
    "cGcNetworkFreighterOwnershipSyncMessage::~cGcNetworkFreighterOwnershipSyncMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkFreighterOwnershipSyncMessage *
        ]
    ),
    "cGcUserSettingsData::~cGcUserSettingsData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettingsData *
        ]
    ),
    "cGcResourceCustomisation::~cGcResourceCustomisation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
        ]
    ),
    "cGcSentinelInfo::Sentinels::~Sentinels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelInfo::Sentinels *
        ]
    ),
    "cGcTerrainEditorBeam::~cGcTerrainEditorBeam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
        ]
    ),
    "cGcHUDMarker::cGcHUDMarker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
        ]
    ),
    "cGcHUDTrackArrow::cGcHUDTrackArrow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDTrackArrow *
        ]
    ),
    "cGcAtlasCommunityData::~cGcAtlasCommunityData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasCommunityData *
        ]
    ),
    "cGcPurchaseableItem::~cGcPurchaseableItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
        ]
    ),
    "InventoryTransferData::~InventoryTransferData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # InventoryTransferData *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<InventoryData,8,-1> *
        ]
    ),
    "cGcCameraManager::~cGcCameraManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
        ]
    ),
    "cGcCameraBehaviourBuildingMode::~cGcCameraBehaviourBuildingMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourBuildingMode *
        ]
    ),
    "cGcCameraBehaviourGalacticNavigation::~cGcCameraBehaviourGalacticNavigation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticNavigation *
        ]
    ),
    "cGcCameraBehaviourScreenshot::~cGcCameraBehaviourScreenshot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourScreenshot *
        ]
    ),
    "cGcCameraBehaviourPhotoMode::~cGcCameraBehaviourPhotoMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourPhotoMode *
        ]
    ),
    "cGcCameraBehaviourThirdPerson::~cGcCameraBehaviourThirdPerson": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
        ]
    ),
    "cGcCameraBehaviourAnimation::~cGcCameraBehaviourAnimation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAnimation *
        ]
    ),
    "cGcCameraBehaviourFollowTarget::~cGcCameraBehaviourFollowTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFollowTarget *
        ]
    ),
    "cGcGraphicsManager::~cGcGraphicsManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGraphicsManager *
        ]
    ),
    "cGcProceduralTextureManager::~cGcProceduralTextureManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureManager *
        ]
    ),
    "cGcCreatureNode::cGcCreatureNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNode *
        ]
    ),
    "cGcFleetExpedition::~cGcFleetExpedition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcExpeditionEventResult,8,-1> *
        ]
    ),
    "cTkCharDescr::~cTkCharDescr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCharDescr *
        ]
    ),
    "cGcApplication::NotifyResolutionChangedCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkIntTuple2<int,0> *
        ]
    ),
    "cGcApplication::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::DrainFileLoadsAndPollableTasks": {
        "bool": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ubyte,  # bool
            ]
        ),
        "bool, int": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ubyte,  # bool
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcApplication::HideSimulationNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcApplication::ShowSimulationNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::AllowPause": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::ChangeAppGameMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
            ctypes.c_uint32,  # ePresetGameMode
            ctypes.c_int32,  # int
        ]
    ),
    "cGcApplication::UpdateInputManagers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
            ctypes.c_float,  # float
        ]
    ),
    "cGcApplication::GetSimulation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSimulation *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetRealityManager": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcRealityManager *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetGameState": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGameState *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetSeasonalData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSeasonalData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetHUDManager": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcHUDManager *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetCameraManager": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCameraManager *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetInputManager": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkInputManager *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetVibrationManager": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcVibrationManager *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetTimeManager": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkTimeManager *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetGraphicsManager": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGraphicsManager *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetAudioManager": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAudioManager *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetAtlasManager": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAtlasManager *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetDebugRenderer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkDebugRenderer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetDebugEditor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDebugEditor *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetNGuiManager": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiManager *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetFrontendManager": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFrontendManager *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetInWorldUIManager": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInWorldUIManager *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetGalaxyMap": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGalaxyMap *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetFirstBootContext": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcApplication::Data *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::GetNetworkManager": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNetworkManager *
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplication::VRAssertCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char * 0x40,  # cTkAssertData
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcApplication::UpdateVRAssertMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcGameConfigCache::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameConfigCache *
        ]
    ),
    "cTkGraphicsAPI::IsRunningVR": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcApplication::IsAccessibleUIEnabled": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcFirstBootContext::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFirstBootContext *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
        ]
    ),
    "cGcOutpostComponent::NetSerializeOnInit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cGcOutpostComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
        ]
    ),
    "cGcOutpostComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcOutpostComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcOutpostComponent *
        ]
    ),
    "ComponentInstancercGcOutpostComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcOutpostComponent::ConstructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcOutpostComponent::cGcOutpostComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
        ]
    ),
    "ComponentInstancercGcOutpostComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentLoadInstancercGcOutpostComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentLoadInstancercGcOutpostComponent *
        ]
    ),
    "cGcSolarSystemTour::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSolarSystemTour *
        argtypes=[]
    ),
    "cGcAmbientModeUI::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAmbientModeUI *
            ctypes.c_float,  # float
        ]
    ),
    "cGcApplicationAmbientGameState::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationAmbientGameState *
        ]
    ),
    "cGcApplicationAmbientGameState::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationAmbientGameState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationAmbientGameState::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationAmbientGameState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationAmbientGameState::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationAmbientGameState *
            ctypes.c_float,  # float
        ]
    ),
    "cGcApplicationAmbientGameState::ThreadedUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationAmbientGameState *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_double,  # double
        ]
    ),
    "cGcApplicationAmbientGameState::ThreadSyncPoint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationAmbientGameState *
        ]
    ),
    "cGcApplicationAmbientGameState::Event": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationSmokeTestState *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cTkEntitlementManager::cTkEntitlementManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEntitlementManager *
        ]
    ),
    "cGcApplicationBootState::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationBootState *
        ]
    ),
    "cGcApplicationBootState::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationBootState *
            ctypes.c_float,  # float
        ]
    ),
    "cGcApplicationBootState::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationBootState *
            ctypes.c_uint32,  # EgRenderParity::List
        ]
    ),
    "cTkTrophyManagerBase::~cTkTrophyManagerBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrophyManagerBase *
        ]
    ),
    "cTkTrophyManager::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkTrophyManager *
        argtypes=[]
    ),
    "cGcApplicationCoreServicesState::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationCoreServicesState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationCoreServicesState::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationCoreServicesState *
            ctypes.c_float,  # float
        ]
    ),
    "cGcApplicationCoreServicesState::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationCoreServicesState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationCoreServicesState::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationCoreServicesState *
        ]
    ),
    "cGcEncounterComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcEncounterComponent::GetTypedComponent": {
        "int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_int32,  # int
            ]
        ),
        "cTkAttachment *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
    },
    "cGcEncounterComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
        ]
    ),
    "cGcEncounterComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcEncounterComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcEncounterComponent *
        ]
    ),
    "ComponentInstancercGcEncounterComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcEncounterComponent::ConstructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcEncounterComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcEncounterComponent::cGcEncounterComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
        ]
    ),
    "cGcEncounterComponent::~cGcEncounterComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcLootComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLootComponent *
        ]
    ),
    "cGcLootComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLootComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcLootComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcLootComponent *
        ]
    ),
    "ComponentInstancercGcLootComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcLootComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcLootComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcVehicleWeapon::GetLastFireTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVoxelGeneratorData *
        ]
    ),
    "cTkDynamicArray<cGcMechDebugSpawnData>::GetSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegionMap<16,16,32,-1,cGcRegionAsteroid> *
        ]
    ),
    "cGcVehicleWeapon::GetLastModeChangeTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleWeapon *
        ]
    ),
    "cGcVehicleWeapon::SetModeIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleWeapon *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcVehicleWeapon::GetPrimaryStat": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleWeapon *
        ]
    ),
    "cGcVehicleWeapon::GetListOfModesIfActiveOrNot": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleWeapon *
            ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
        ]
    ),
    "cGcSentinelMech::GetOwnerConcept": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcOwnerConcept *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
        ]
    ),
    "cGcVehicleWeapon::GetTerrainEditMode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleWeapon *
        ]
    ),
    "cGcVehicleWeapon::UsingTerrainAdvancedMode": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleWeapon *
        ]
    ),
    "cGcVehicleWeapon::ShowingTerrainEditNotifies": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleWeapon *
        ]
    ),
    "cGcVehicleWeapon::ResetTerrainMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleWeapon *
        ]
    ),
    "cGcVehicleComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcVehicleComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcVehicleComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcVehicleComponent *
        ]
    ),
    "ComponentInstancercGcVehicleComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcVehicleComponent::ConstructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcVehicleComponent::cGcVehicleComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "ComponentInstancercGcVehicleComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentLoadInstancercGcVehicleComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentLoadInstancercGcVehicleComponent *
        ]
    ),
    "cGcApplicationDeathState::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationDeathState *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcApplicationDeathState::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationDeathState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationDeathState::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationDeathState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationDeathState::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationDeathState *
            ctypes.c_double,  # double
        ]
    ),
    "cGcApplicationDeathState::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationDeathState *
            ctypes.c_uint32,  # EgRenderParity::List
        ]
    ),
    "cGcApplicationDeathState::RestoreRenderPipeline": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationDeathState *
        ]
    ),
    "cGcApplicationGalacticMapState::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGalacticMapState *
        ]
    ),
    "cGcApplicationGalacticMapState::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGalacticMapState *
        ]
    ),
    "cGcApplicationGalacticMapState::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGalacticMapState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationGalacticMapState::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGalacticMapState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationGalacticMapState::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGalacticMapState *
            ctypes.c_double,  # double
            ctypes.c_int64,  # __int64
            ctypes.c_double,  # double
        ]
    ),
    "cGcApplicationGalacticMapState::Event": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGalacticMapState *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationGalacticMapState::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGalacticMapState *
            ctypes.c_uint32,  # EgRenderParity::List
        ]
    ),
    "cGcFrontendPageFunctions::AcceptButtonPressed": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcFrontendPageFunctions::BackButtonPressed": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcCommandLine::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCommandLine *
        argtypes=[]
    ),
    "cGcCommandLine::ParseArguments": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCommandLine *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # char **
        ]
    ),
    "cGcUniqueIdComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniqueIdComponent *
        ]
    ),
    "cGcUniqueIdComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniqueIdComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUniqueIdComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniqueIdComponent *
        ]
    ),
    "ComponentInstancercGcUniqueIdComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcUniqueIdComponent *
        ]
    ),
    "ComponentInstancercGcUniqueIdComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcUniqueIdComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcUniqueIdComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcFrontendPageOptions::RequestOptionsPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # eOptionsMenu
        ]
    ),
    "cGcFrontendPageOptions::OnSwitchToRemapControls": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcApplicationGameModeSelectorState::SystemCleanUp_PreparePhase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "GetLOCGameModeTitle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_uint32,  # ePresetGameMode
        ]
    ),
    "cGcApplicationGameModeSelectorState::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationGameModeSelectorState::SlotComponents::Resolve": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState::SlotComponents *
            ctypes.c_uint32,  # cGcApplicationGameModeSelectorState::Selecting
        ]
    ),
    "cGcApplicationGameModeSelectorState::WarningDlgComponents::Resolve": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState::WarningDlgComponents *
        ]
    ),
    "cGcApplicationGameModeSelectorState::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationGameModeSelectorState::LoadAndSetupUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::SetupSlotUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cGcApplicationGameModeSelectorState::RefreshSlotUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::SetupModeUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::FormatSeasonTimeString": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_int64,  # __int64
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcApplicationGameModeSelectorState::ShouldDisplaySeasonHint": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::UpdateSeasonalGameModeUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::SetupMultiplayerUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::SetupCloudSavesUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::SetupStartUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::SetupCustomDifficultyUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::SetupOptionsUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::SetupCreditsUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::SetupRemapControlsUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::UnloadAndDiscardUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::FetchSaveDataState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::RenderCursor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::UpdateMultiplayerFriendsUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::UpdateManageCloudSavesUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::DownloadCloudData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::DoPatchNotes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcLayerInteractControl *
            ctypes.c_ulonglong,  # const std::vector<GcGameStatePersistence::StorageSlotInformation,TkSTLAllocatorShim<GcGameStatePersistence::StorageSlotInformation,4,-1> > *
        ]
    ),
    "cGcApplicationGameModeSelectorState::DoOKCancelDialog": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::OnSeasonWarningClosed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
            ctypes.c_ubyte,  # _BOOL8
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcApplicationGameModeSelectorState::UpdateStartUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcDifficultySettingsConfirmHandlerBootMenu::DoConfirm": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsConfirmHandlerBootMenu *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcApplicationGameModeSelectorState::UpdateCustomDifficultyUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::SwitchToInLobbyState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
            ctypes.c_uint32,  # cGcApplicationGameModeSelectorState::MultiplayerState
        ]
    ),
    "cGcApplicationGameModeSelectorState::LeaveMultiplayerState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::CustomIconFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcApplicationGameModeSelectorState::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
            ctypes.c_uint32,  # EgRenderParity::List
        ]
    ),
    "cGcApplicationGameModeSelectorState::DoUpdateLogicDuringRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::UpdateLogicSlotSelect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::UpdateSeasonalSlotHover": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
            ctypes.c_ulonglong,  # const GcGameStatePersistence::StorageSlotInformation *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcApplicationGameModeSelectorState::RestoreRenderPipeline": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::UpdateLogicModeSelect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::OnSeasonalDataBecameAvailable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcApplicationGameModeSelectorState::SwitchToOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcApplicationGameModeSelectorState::LeaveOptionsScreen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::UpdateDeletionWarningDlg": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
            ctypes.c_float,  # float
        ]
    ),
    "cGcApplicationGameModeSelectorState::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
        ]
    ),
    "cGcApplicationGameModeSelectorState::RenderWarningMessages": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::RenderWarning": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_float,  # float
        ]
    ),
    "cGcApplicationGameModeSelectorState::HasPressedBackOrQuit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
            ctypes.c_uint32,  # eInputValidation
        ]
    ),
    "cGcApplicationGameModeSelectorState::Event": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcVehicleComponent>::UpdateComponentsRender": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcVehicleComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcVehicleComponent>::UpdateComponentsPostPhysics": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcVehicleComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkInplaceLockFreeQueue<cGcSimulation::DestructionPair,1024>::cTkInplaceLockFreeQueue<cGcSimulation::DestructionPair,1024>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInplaceLockFreeQueue<cGcSimulation::DestructionPair,1024> *
        ]
    ),
    "nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>::~GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        ]
    ),
    "cTkDynamicArray<cGcPlayerStatsGroup>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerStatsGroup> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPlayerStat>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerStat> *
            ctypes.c_int32,  # int
        ]
    ),
    "TkID<256>::SetFromFormat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkID<256> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcExplosion::cGcDebris::cGcDebris": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExplosion::cGcDebris *
        ]
    ),
    "cTkLinearHashTable<unsigned __int64,TkID<256>,-1,0,cTkLinearHashTableHash<unsigned __int64>>::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<unsigned __int64,TkID<256>,-1,0,cTkLinearHashTableHash<unsigned __int64> > *
        ]
    ),
    "cTkLinearHashTable<unsigned __int64,TkID<256>,-1,0,cTkLinearHashTableHash<unsigned __int64>>::ChangeNodeCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<unsigned __int64,TkID<256>,-1,0,cTkLinearHashTableHash<unsigned __int64> > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcPlayerStatsGroup>::AllocateAndCopyFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerStatsGroup> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcPlayerStatsGroup> *
        ]
    ),
    "cTkUserIdBase<cTkFixedString<64,char>>::FromString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkStringUtilities<char>::InsertCharacter": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_char,  # char
        ]
    ),
    "cTkStringUtilities<char>::Remove": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkStringUtilities<char>::ReplaceAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkStringUtilities<char>::Replace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkStringUtilities<char>::InsertPrefix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNetworkBaseImageManager::BaseImageRequest::~BaseImageRequest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseImageManager::BaseImageRequest *
        ]
    ),
    "cTkFixedString<512,char>::Format<char,0>": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkStringUtilities<char>::FormatNumberLocalised<int>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkDebugHandler::OutputStyled<4>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkDebugHandler::OutputStyled<2>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcDefaultSaveData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDefaultSaveData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cppcodec::detail::codec<cppcodec::detail::base32<cppcodec::detail::base32_crockford>>::encode<std::string,std::vector<uchar>>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::string *
        argtypes=[
            ctypes.c_ulonglong,  # std::string *
            ctypes.c_ulonglong,  # const std::vector<unsigned char> *
        ]
    ),
    "cppcodec::detail::codec<cppcodec::detail::hex<cppcodec::detail::hex_upper>>::encode<std::vector<uchar>>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const std::vector<unsigned char> *
        ]
    ),
    "cTkStringUtilities<char>::ReplaceLazyEvaluate<std::function<char const * ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const std::function<char const * __cdecl(void)> *
        ]
    ),
    "cppcodec::detail::codec<cppcodec::detail::base32<cppcodec::detail::base32_crockford>>::decode<std::vector<uchar>>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<unsigned char> *
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<unsigned char> *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cppcodec::detail::codec<cppcodec::detail::hex<cppcodec::detail::hex_upper>>::decode<std::vector<uchar>>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<unsigned char> *
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<unsigned char> *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cppcodec::detail::stream_codec<cppcodec::detail::base32<cppcodec::detail::base32_crockford>,cppcodec::detail::base32_crockford>::decode<std::vector<uchar>,cppcodec::data::direct_data_access_result_state<std::vector<uchar>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<unsigned char> *
            ctypes.c_ulonglong,  # cppcodec::data::direct_data_access_result_state<std::vector<unsigned char> > *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cppcodec::detail::stream_codec<cppcodec::detail::base32<cppcodec::detail::base32_crockford>,cppcodec::detail::base32_crockford>::encode<std::string,cppcodec::data::direct_data_access_result_state<std::string>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::string *
            ctypes.c_ulonglong,  # cppcodec::data::direct_data_access_result_state<std::string > *
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "OSDMessage::OSDMessage": {
        "OSDMessage *, const OSDMessage *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # OSDMessage *
                ctypes.c_ulonglong,  # const OSDMessage *
            ]
        ),
        "OSDMessage *, OSDMessage *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # OSDMessage *
                ctypes.c_ulonglong,  # OSDMessage *
            ]
        ),
    },
    "sPlayerTitleStatWatcher::StatChanged": {
        "sPlayerTitleStatWatcher *, const TkID<128> *, __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # sPlayerTitleStatWatcher *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_int64,  # __int64
            ]
        ),
        "sPlayerTitleStatWatcher *, const TkID<128> *, long double": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # sPlayerTitleStatWatcher *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_double,  # long double
            ]
        ),
    },
    "cGcFiendManagerStandard::CanVirtualise": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerStandard *
        ]
    ),
    "cGcFiendManagerStandard::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerStandard *
        ]
    ),
    "cGcPlayerWeapon::ShowingTerrainEditNotifies": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::UsingTerrainAdvancedMode": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetTerrainEditorBeam": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetTerrainEditMode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::IsCharging": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::CanCharge": {
        "cGcPlayerWeapon *, eWeaponMode": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerWeapon *
                ctypes.c_uint32,  # eWeaponMode
            ]
        ),
        "cGcPlayerWeapon *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ]
        ),
    },
    "cGcPlayerWeapon::GetChargeFactor": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetListOfModesIfActiveOrNot": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
        ]
    ),
    "cGcPlayerWeapon::GetPrimaryStat": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetLastModeChangeTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetModeIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetAmmo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_uint32,  # eWeaponMode
        ]
    ),
    "cGcPlayerWeapon::GetAmmoInClip": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_ubyte,  # bool
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcPlayerWeapon::GetCooldownProgress": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::NeedsCooldownAfterFiring": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::IsOnCooldown": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetOverheatProgress": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::CanOverheat": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::HasOverheated": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetLastFireTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetReloadProgress": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::IsReloading": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetAvailableAmmo": {
        "cGcPlayerWeapon *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ]
        ),
        "cGcPlayerWeapon *, eWeaponMode": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerWeapon *
                ctypes.c_uint32,  # eWeaponMode
            ]
        ),
    },
    "cGcPlayerWeapon::UsesInventoryAmmo": {
        "cGcPlayerWeapon *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # const cGcInventoryElement *
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ]
        ),
        "cGcPlayerWeapon *, __int64": FUNCDEF(
            restype=ctypes.c_ulonglong,  # const cGcInventoryElement *
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerWeapon *
                ctypes.c_int64,  # __int64
            ]
        ),
    },
    "cGcPlayerWeapon::UsingAmmo": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcCameraBehaviourInteraction::GetDof": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCameraDepthOfField *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourInteraction *
        ]
    ),
    "cGcPlayer::VirtualGetMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayer::GetNetworkIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::IsInSpaceship": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "hkSignal3<hknpWorld *,hknpBodyId *,int>::FunctionSlot<void ": {
        "cTkHavokProxyCharacterController *, const hknpBodyId **, int *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkHavokProxyCharacterController *
                ctypes.c_ulonglong,  # const hknpBodyId **
                ctypes.c_ulonglong,  # int *
            ]
        ),
        "hkSignal3<hknpWorld *,hknpBodyId *,int>::FunctionSlot<void (__cdecl*)(hknpWorld *,hknpBodyId *,int)> *, hknpWorld *, hknpBodyId *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal3<hknpWorld *,hknpBodyId *,int>::FunctionSlot<void (__cdecl*)(hknpWorld *,hknpBodyId *,int)> *
                ctypes.c_ulonglong,  # hknpWorld *
                ctypes.c_ulonglong,  # hknpBodyId *
                ctypes.c_int32,  # int
            ]
        ),
        "hkSignal3<hknpWorld *,hknpBodyId *,int>::FunctionSlot<void (__cdecl*)(hknpWorld *,hknpBodyId *,int)> *, const void *, int": FUNCDEF(
            restype=ctypes.c_ubyte,  # _BOOL8
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal3<hknpWorld *,hknpBodyId *,int>::FunctionSlot<void (__cdecl*)(hknpWorld *,hknpBodyId *,int)> *
                ctypes.c_ulonglong,  # const void *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcCameraBehaviourFly::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFly *
        ]
    ),
    "cGcCameraManager::AssignBehaviours": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
        ]
    ),
    "cTkCameraManagerTemplate<22>::GetBehaviourIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraManagerTemplate<22> *
            ctypes.c_ulonglong,  # cTkCameraBehaviour *
        ]
    ),
    "cTkCameraManagerTemplate<22>::GetBehaviour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkCameraBehaviour *
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraManagerTemplate<22> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkCameraManagerTemplate<22>::StartBehaviourUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraManagerTemplate<22> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkCameraManagerTemplate<22>::StopBehaviourUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraManagerTemplate<22> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkCameraManagerTemplate<22>::SetActiveBehaviour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraManagerTemplate<22> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkCameraManagerTemplate<22>::DeactivateBehaviour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraManagerTemplate<22> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkCameraManagerTemplate<22>::ActivateBehaviour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraManagerTemplate<22> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkCameraManagerTemplate<22>::CopyDebugToGame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraManagerTemplate<22> *
        ]
    ),
    "cTkCameraManagerTemplate<22>::IsDebugActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraManagerTemplate<22> *
        ]
    ),
    "cTkCameraManagerTemplate<22>::SetDebugActive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraManagerTemplate<22> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCameraManagerTemplate<22>::IsBehaviourActive": FUNCDEF(
        restype=ctypes.c_uint8,  # unsigned __int8
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraManagerTemplate<22> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFleetExpedition::GetUA": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int64 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cTkCameraManagerTemplate<22>::GetActiveCamera": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkCamera *
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraManagerTemplate<22> *
        ]
    ),
    "cTkCameraManagerTemplate<22>::GetCameraMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraManagerTemplate<22> *
        ]
    ),
    "cTkCameraManagerTemplate<22>::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraManagerTemplate<22> *
        ]
    ),
    "cTkGraphicsManager::SetMotionBlur": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManager *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkGraphicsManager::SetShadowCascadeMask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManager *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkGraphicsManager::CalculateNewFov": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManager *
            ctypes.c_float,  # float
        ]
    ),
    "cTkGraphicsManager::IsConstructed": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManager *
        ]
    ),
    "cTkGraphicsManager::EnqueueRender": FUNCDEF(
        restype=ctypes.c_uint64,  # EgRenderQueueHandle
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManagerBase *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_float,  # float
        ]
    ),
    "cTk2dImageEx::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dImageEx *
        ]
    ),
    "cTkCamera::Transform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "NetReplicatedCustomisationData::~NetReplicatedCustomisationData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NetReplicatedCustomisationData *
        ]
    ),
    "cTkFixedString<256,char>::Format<char,0>": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkStringUtilities<char>::ShiftLeft": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAIShipSpawn::cGcAIShipSpawn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSpawn *
        ]
    ),
    "cTkBitArray<uint,1,32>::SetElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned int,1,32> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCurveFunction::SmoothCD<float,float>": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkFileSystem::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFileSystem *
        argtypes=[]
    ),
    "cTkTimeManager::GetLocalTimeStep": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[]
    ),
    "cTkGraphicsAPI::SetTest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
        ]
    ),
    "cTkInputPort::GetCurrentInputDeviceType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachedDevice *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
        ]
    ),
    "cGcMatchmaking::Session::IsOwner": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking::Session *
        ]
    ),
    "cGcNGuiLayer::FindTextSpecialRecursive": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "ModeSelectorPhase::GetNext": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # ModeSelectorPhase::Enum
        ]
    ),
    "cGcNGuiLayer::FindGraphicRecursive": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcNetworkConstants::GetOnlinePlatformIconName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_uint32,  # cGcNetworkConstants::OnlinePlatformType
        ]
    ),
    "cGcNetworkConstants::GetOnlinePlatformName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_uint32,  # cGcNetworkConstants::OnlinePlatformType
        ]
    ),
    "cGcNGuiLayer::AddElementAt": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiElement *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNGuiLayer::FindTextRecursive": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkCamera::SetMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcNGuiLayer::GetGraphic": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiGraphic *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLinearHashTable<cTkHashedNGuiElement,cGcNGuiElement *,-1,0,cTkHashedNGuiElement>::insert": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<cTkHashedNGuiElement,cGcNGuiElement *,-1,0,cTkHashedNGuiElement> *
            ctypes.c_ulonglong,  # const cTkHashedNGuiElement *
            ctypes.c_ulonglong,  # cGcNGuiElement *const *
        ]
    ),
    "cGcNGuiLayer::FindElementRecursive": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cTkHashedNGuiElement *
            ctypes.c_uint32,  # eNGuiGameElementType
        ]
    ),
    "cTk2dRectangle::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dRectangle *
        ]
    ),
    "cTk2dRectangle::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dRectangle *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cTkTrig<float>::Sin": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "cTkGraphicsAPI::SetBlendMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcNGuiManager::ScopedScaleChange::~ScopedScaleChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiManager::ScopedScaleChange *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcNGuiManager::ScopedScaleChange::ScopedScaleChange": {
        "cGcNGuiManager::ScopedScaleChange *, float, __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiManager::ScopedScaleChange *
                ctypes.c_float,  # float
                ctypes.c_int64,  # __int64
            ]
        ),
        "cGcNGuiManager::ScopedScaleChange *, const cTkVector2 *, __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiManager::ScopedScaleChange *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_int64,  # __int64
            ]
        ),
    },
    "cTkFontFile::Size": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcTerrainRegionMap::AreUpgradeListsEmpty": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
        ]
    ),
    "cTkCameraManagerTemplate<22>::GetActiveCameraMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraManagerTemplate<22> *
        ]
    ),
    "cTkLockFreeQueue<cTkJob>::IsEmpty": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkLockFreeQueue<cTkJob> *
        ]
    ),
    "cTkDependencyCounter::Increment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDependencyCounter *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkLockFreeQueue<cTkJob>::Pop": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkLockFreeQueue<cTkJob> *
            ctypes.c_ulonglong,  # cTkJob *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # std::atomic<int> *
        ]
    ),
    "cTkDependencyCounter::Decrement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDependencyCounter *
        ]
    ),
    "cTkJobManager::RunAllJobsInQueue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "cGcScanEventManager::GetFullEventStack": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cGcScanEventSave,TkSTLAllocatorShim<cGcScanEventSave,16,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
        ]
    ),
    "cGcSolarSystem::SetLoading": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiLayer::GetLayer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiLayer::GetText": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiText *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsManager::GetStatDefinition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcStatDefinition *
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcStatsManager::GetStatValue<int>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcPlayer::GetVehicle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachmentPtr *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkAttachmentPtr *
        ]
    ),
    "cGcSolarSystem::InPlanetRange": {
        "cGcSolarSystem *": FUNCDEF(
            restype=ctypes.c_ubyte,  # _BOOL8
            argtypes=[
                ctypes.c_ulonglong,  # cGcSolarSystem *
            ]
        ),
        "cGcSolarSystem *, const cTkVector3 *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcSolarSystem *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
    },
    "cGcPlayer::IsInLandVehicle": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayerState::ResetTime": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::GetSeasonState": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSeasonState *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcSimulation::GetWarp": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSpaceshipWarp *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulation *
        ]
    ),
    "TkID<128>::SetFromFormat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "sGcTelemetryStatVariant::GetVariant": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # sGcTelemetryStatVariant *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "sGcTelemetryStatVariant::sGcTelemetryStatVariant": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sGcTelemetryStatVariant *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkDynamicArray<cTkVector4>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkVector4> *
        ]
    ),
    "cTkDynamicArray<cGcPlayerStatsGroup>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerStatsGroup> *
        ]
    ),
    "cGcInteractionBuffer::cGcInteractionBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionBuffer *
        ]
    ),
    "cTkFixedArray<cGcInteractionBuffer,11>::cTkFixedArray<cGcInteractionBuffer,11>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcInteractionBuffer,11> *
        ]
    ),
    "cGcInteractionBuffer::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionBuffer *
        ]
    ),
    "cTkFixedArray<cGcInteractionBuffer,11>::~cTkFixedArray<cGcInteractionBuffer,11>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcInteractionBuffer,11> *
        ]
    ),
    "cTkDynamicArray<cGcMaintenanceContainer>::~cTkDynamicArray<cGcMaintenanceContainer>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMaintenanceContainer> *
        ]
    ),
    "cTkMetaDataManager::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetaDataManager *
        argtypes=[]
    ),
    "CoreSrvLoadPhase::GetNext": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # CoreSrvLoadPhase::Enum
        ]
    ),
    "cTkTimeManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTimeManager *
        ]
    ),
    "cTkGraphicsAPI::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkGnmxContext::_GetCommandList": FUNCDEF(
        restype=ctypes.c_ulonglong,  # struct VkCommandBuffer_T *
        argtypes=[
            ctypes.c_ulonglong,  # cTkGnmxContext *
        ]
    ),
    "cTkImmediateMode::~cTkImmediateMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImmediateMode *
        ]
    ),
    "cTkImmediateMode::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkImmediateMode *
        argtypes=[]
    ),
    "cTkDynamicGravityControl::cTkDynamicGravityControl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicGravityControl *
        ]
    ),
    "cTkHashedNGuiElement::cTkHashedNGuiElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHashedNGuiElement *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNGuiLayer::GetTextSpecial": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiTextSpecial *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBitArray<uint,1,32>::GetElement": FUNCDEF(
        restype=ctypes.c_uint8,  # unsigned __int8
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned int,1,32> *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpShapeProperties::Entry::Entry": {
        "cGcUniverseAddressLayout::Bitfield *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcUniverseAddressLayout::Bitfield *
            ]
        ),
        "hknpShapeProperties::Entry *, hknpShapeProperties::Entry *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpShapeProperties::Entry *
                ctypes.c_ulonglong,  # hknpShapeProperties::Entry *
            ]
        ),
        "hknpShapeProperties::Entry *, const hknpShapeProperties::Entry *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpShapeProperties::Entry *
                ctypes.c_ulonglong,  # const hknpShapeProperties::Entry *
            ]
        ),
    },
    "cGcGalacticVoxelCoordinate::cGcGalacticVoxelCoordinate": {
        "cGcGalacticVoxelCoordinate *, const cGcGalacticAddressData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcGalacticVoxelCoordinate *
                ctypes.c_ulonglong,  # const cGcGalacticAddressData *
            ]
        ),
        "cGcGalacticVoxelCoordinate *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcGalacticVoxelCoordinate *
            ]
        ),
        "cGcGalacticVoxelCoordinate *, __int16, __int16, __int16": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcGalacticVoxelCoordinate *
                ctypes.c_int16,  # __int16
                ctypes.c_int16,  # __int16
                ctypes.c_int16,  # __int16
            ]
        ),
    },
    "cGcNetworkManager::GetMatchmaking": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMatchmaking *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkManager *
        ]
    ),
    "vr::VROverlay": FUNCDEF(
        restype=ctypes.c_ulonglong,  # vr::IVROverlay *
        argtypes=[]
    ),
    "vr::COpenVRContext::CheckClear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # vr::COpenVRContext *
        ]
    ),
    "vr::VRSystem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # vr::IVRSystem *
        argtypes=[]
    ),
    "cTkFixedString<64,char>::Format<char,0>": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkFixedString<32,char>::Format<char,0>": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkStopwatch::GetDurationInSeconds": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkStopwatch *
        ]
    ),
    "cTkMultiContextManager::cTkMultiContextManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMultiContextManager *
        ]
    ),
    "cTkJobManager::WakeAllThreadsInGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "cTkTimeManager::GetGlobalTimeStep": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[]
    ),
    "cTkSystemBase::cTkSystemBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystemBase *
        ]
    ),
    "cTkSystem::GetWindowMode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
        ]
    ),
    "cTkSystemBase::GetSKUID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystemBase *
        ]
    ),
    "cTkSystemBase::SetSKUID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystemBase *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkSystemBase::DetectAvailableMonitors": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # ValueAsBool *
        ]
    ),
    "cGcCharacterInterface::GetHitReactDir": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterInterface *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "CAkMatrixAwareCtx::SeekPercent": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISetPlayerShooter *
            ctypes.c_float,  # float
        ]
    ),
    "cTkSystemBase::IsRetailAppId": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystemBase *
        ]
    ),
    "cTkSystemBase::~cTkSystemBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystemBase *
        ]
    ),
    "cTkSystem::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSystem *
        argtypes=[]
    ),
    "cGcObjectSpawnData::~cGcObjectSpawnData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnData *
        ]
    ),
    "cTkDynamicArray<cGcObjectSpawnData>::~cTkDynamicArray<cGcObjectSpawnData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcObjectSpawnData> *
        ]
    ),
    "cTkDynamicArray<cGcSelectableObjectSpawnData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSelectableObjectSpawnData> *
        ]
    ),
    "cTkDynamicArray<cGcSelectableObjectSpawnList>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSelectableObjectSpawnList> *
        ]
    ),
    "cTkDynamicArray<cGcSelectableObjectSpawnList>::~cTkDynamicArray<cGcSelectableObjectSpawnList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSelectableObjectSpawnList> *
        ]
    ),
    "cGcEnvironmentSpawnData::~cGcEnvironmentSpawnData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentSpawnData *
        ]
    ),
    "cTkDynamicArray<cGcBuildingSpawnData>::~cTkDynamicArray<cGcBuildingSpawnData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBuildingSpawnData> *
        ]
    ),
    "cGcPlanetBuildingData::~cGcPlanetBuildingData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetBuildingData *
        ]
    ),
    "cGcPlanetGenerationIntermediateData::~cGcPlanetGenerationIntermediateData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerationIntermediateData *
        ]
    ),
    "cGcPlanetData::~cGcPlanetData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cTkInputManager::~cTkInputManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cGcPlanetObjectSpawnData::~cGcPlanetObjectSpawnData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetObjectSpawnData *
        ]
    ),
    "cGcSolarSystemData::~cGcSolarSystemData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemData *
        ]
    ),
    "cGcSolarSystemAsteroidFields::AsteroidGenerators::~AsteroidGenerators": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemAsteroidFields::AsteroidGenerators *
        ]
    ),
    "cTkUiDataMap<cTkNGuiElementData>::~cTkUiDataMap<cTkNGuiElementData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgUpdateTransformsWorkBuffer *
        ]
    ),
    "cGcSolarSystemQuery::~cGcSolarSystemQuery": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemQuery *
        ]
    ),
    "cTkEngineSettings::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkEngineSettings *
        argtypes=[]
    ),
    "cTkInputManager::GetDeviceVRViewResetLayerName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "cTkInputManager::GetActionButton": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkInputManager::GetActionAxis": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkInputManager::GetMappedIcon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const sTkInputDeviceIcons *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eInputIconType
        ]
    ),
    "cTkInputManager::GetChordIcons": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachedDevice *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "cTkInputDeviceManager::LockMouse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
        ]
    ),
    "cTkInputDeviceManager::UnlockMouse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
        ]
    ),
    "cTkHmdOpenVR::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkHmdOpenVR *
        argtypes=[]
    ),
    "cTkInputDeviceManager::IsControllerPositionTracking": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
        ]
    ),
    "cTkInputDeviceManager::cTkInputDeviceManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
        ]
    ),
    "cTkInputDeviceManagerBase::LockMouse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManagerBase *
        ]
    ),
    "cTkInputDeviceManagerBase::UnlockMouse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManagerBase *
        ]
    ),
    "cTkInputDeviceManagerBase::LocaliseButton": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManagerBase *
            ctypes.c_uint32,  # eInputButton
        ]
    ),
    "cTkInputDeviceManagerBase::~cTkInputDeviceManagerBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManagerBase *
        ]
    ),
    "cTkInputDeviceManager::~cTkInputDeviceManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
        ]
    ),
    "cTkInputPort::~cTkInputPort": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
        ]
    ),
    "cTkInputManager::cTkInputManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "cTkTimeManager::SetBaseUTCTimeMilliseconds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNetworkBufferSyncManager::cGcNetworkBufferSyncManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBufferSyncManager *
            ctypes.c_uint32,  # eGcNetworkMessageType
            ctypes.c_uint32,  # eGcNetworkMessageType
        ]
    ),
    "cGcNetworkPlayerFullSim::GetNetworkIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcNetworkPlayerFullSim::IsVRPlayer": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcNetworkPlayerFullSim::GetDominantHand": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcNetworkPlayerFullSim::IsActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcNetworkPlayerFullSim::IsHandTracking": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cTkFixedArray<cGcLaserBeam,6>::~cTkFixedArray<cGcLaserBeam,6>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcLaserBeam,6> *
        ]
    ),
    "cGcNetworkWeapon::LaserSnapshot::~LaserSnapshot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkWeapon::LaserSnapshot *
        ]
    ),
    "cGcNetworkWeapon::ProjectileSnapshot::~ProjectileSnapshot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sNetProjectileImpact *
        ]
    ),
    "cGcNetworkWeapon::~cGcNetworkWeapon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkWeapon *
        ]
    ),
    "cGcEnvironment::GetSky": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSeed *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cTkReplicatedVariableTemplated<cTkUserIdBase<cTkFixedString<64,char>>>::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cTkUserIdBase<cTkFixedString<64,char> > > *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cTkReplicatedVariablePlayer<cTkFixedString<64,char>>::IsLocalPlayerEntityOwner": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariablePlayer<cTkFixedString<64,char> > *
        ]
    ),
    "cTkReplicatedVariableTemplated<signed char>::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<signed char> *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cTkReplicatedVariablePlayer<uint>::IsLocalPlayerEntityOwner": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariablePlayer<bool> *
        ]
    ),
    "cTkReplicatedVariableTemplated<eCamouflageState>::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<int> *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cTkReplicatedVariableTemplated<cTkFixedString<128,char>>::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cTkFixedString<128,char> > *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cTkReplicatedVariablePlayer<cTkFixedString<128,char>>::IsLocalPlayerEntityOwner": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariablePlayer<cTkFixedString<128,char> > *
        ]
    ),
    "cTkReplicatedVariableTemplated<cGcLobbyId>::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cGcLobbyId> *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cTkReplicatedVariablePlayer<cGcLobbyId>::IsLocalPlayerEntityOwner": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariablePlayer<cGcLobbyId> *
        ]
    ),
    "cTkReplicatedVariablePlayer<TkID<128>>::IsLocalPlayerEntityOwner": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariablePlayer<TkID<128> > *
        ]
    ),
    "cTkReplicatedVariableTemplated<unsigned __int64>::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<unsigned __int64> *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cTkReplicatedVariableTemplated<cTkFixedString<4,char>>::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cTkFixedString<4,char> > *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cTkBaseMath::Log2Floor<unsigned __int64>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkReplicatedVariableTemplated<cTkFixedString<64,char>>::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cTkFixedString<64,char> > *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cGcSettlementState::cGcSettlementState": {
        "cGcSettlementState *, const cGcSettlementState *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcSettlementState *
                ctypes.c_ulonglong,  # const cGcSettlementState *
            ]
        ),
        "cGcSettlementState *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcSettlementState *
            ]
        ),
    },
    "cTkReplicatedVariableTemplated<cGcSettlementState>::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cGcSettlementState> *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cTkReplicatedVariablePlayer<cGcSettlementState>::IsLocalPlayerEntityOwner": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariablePlayer<cGcSettlementState> *
        ]
    ),
    "cTkReplicatedVariableTemplated<cGcDifficultySettingsReplicatedState>::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cGcDifficultySettingsReplicatedState> *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cTkReplicatedVariablePlayer<cGcDifficultySettingsReplicatedState>::IsLocalPlayerEntityOwner": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariablePlayer<cGcDifficultySettingsReplicatedState> *
        ]
    ),
    "cGcNetworkPlayerReplicatedState::cGcNetworkPlayerReplicatedState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerReplicatedState *
        ]
    ),
    "cGcNetworkPlayer::IsInSpaceship": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkPlayer::IsInLandVehicle": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkPlayer::GetEnvironmentLocation": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcPlayerCommon::GetNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommon *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcPlayerCommon::NotifyBeingScanned": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommon *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcNetworkPlayer::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkPlayer::IsActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkPlayer::GetPlayerName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cEgResource::GetBoundingBox": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAABB *
        argtypes=[
            ctypes.c_ulonglong,  # cEgResource *
            ctypes.c_ulonglong,  # cTkAABB *
        ]
    ),
    "cGcNetworkPlayer::GetMapPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegionMap<16,16,32,-1,cGcRegionAsteroid> *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcNetworkPlayer::~cGcNetworkPlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cTkReplicatedVariableTemplated<cTkUserIdBase<cTkFixedString<64,char>>>::~cTkReplicatedVariableTemplated<cTkUserIdBase<cTkFixedString<64,char>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cTkFixedString<64,char> > *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cTkReplicatedVariableTemplated<cTkFixedString<128,char>>::~cTkReplicatedVariableTemplated<cTkFixedString<128,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cTkFixedString<128,char> > *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cTkReplicatedVariableTemplated<cGcLobbyId>::~cTkReplicatedVariableTemplated<cGcLobbyId>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cGcLobbyId> *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cTkReplicatedVariableTemplated<cGcSettlementState>::~cTkReplicatedVariableTemplated<cGcSettlementState>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cGcSettlementState> *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cTkReplicatedVariableTemplated<cGcDifficultySettingsReplicatedState>::~cTkReplicatedVariableTemplated<cGcDifficultySettingsReplicatedState>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cGcDifficultySettingsReplicatedState> *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cGcNetworkPlayerReplicatedState::~cGcNetworkPlayerReplicatedState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerReplicatedState *
        ]
    ),
    "cTkAudioParamCache::~cTkAudioParamCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioParamCache *
        ]
    ),
    "cTkGraphicsManagerBase::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManagerBase *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkGraphicsManagerBase::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManagerBase *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkGraphicsManagerBase::NextViewMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManagerBase *
        ]
    ),
    "cTkGraphicsManagerBase::EnqueueRenderViewport": FUNCDEF(
        restype=ctypes.c_uint64,  # EgRenderQueueHandle
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManagerBase *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cTkGraphicsManagerBase::GetViewMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManagerBase *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cTkGraphicsManagerBase::GetProjectionMatrix": {
        "cTkGraphicsManagerBase *, cTkMatrix44 *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkMatrix44 *
            argtypes=[
                ctypes.c_ulonglong,  # cTkGraphicsManagerBase *
                ctypes.c_ulonglong,  # cTkMatrix44 *
            ]
        ),
        "cTkGraphicsManagerBase *, cTkMatrix44 *, float": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkMatrix44 *
            argtypes=[
                ctypes.c_ulonglong,  # cTkGraphicsManagerBase *
                ctypes.c_ulonglong,  # cTkMatrix44 *
                ctypes.c_float,  # float
            ]
        ),
    },
    "cTkGraphicsManagerBase::Release": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManagerBase *
        ]
    ),
    "cTkGraphicsManagerBase::CalculateNewFov": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManagerBase *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCountInteraction::IsActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCountInteraction *
        ]
    ),
    "cGcCountInteraction::EndInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCountInteraction *
        ]
    ),
    "cGcHoldInteraction::IsActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageDataSearchTask *
        ]
    ),
    "cGcHoldInteraction::NeedsWarning": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcHoldInteraction *
        ]
    ),
    "InventoryData::~InventoryData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # InventoryData *
        ]
    ),
    "cGcSpaceshipWarp::cGcSpaceshipWarp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
        ]
    ),
    "DiscoveryResolver::NameAndOwnerResult::NameAndOwnerResult": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DiscoveryResolver::NameAndOwnerResult *
        ]
    ),
    "DiscoveryResolver::DiscoveryInfo::DiscoveryInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DiscoveryResolver::DiscoveryInfo *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcBinoculars::cGcBinoculars": {
        "cGcBinoculars *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcBinoculars *
            ]
        ),
        "cGcBinoculars *, const cGcBinoculars *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcBinoculars *
                ctypes.c_ulonglong,  # const cGcBinoculars *
            ]
        ),
    },
    "cGcScanner::cGcScanner": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
        ]
    ),
    "cGcBinoculars::~cGcBinoculars": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcPlayerEmoteProp::cGcPlayerEmoteProp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEmoteProp *
        ]
    ),
    "cGcWarpJumpTarget::cGcWarpJumpTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWarpJumpTarget *
        ]
    ),
    "cGcPlayerCommon::~cGcPlayerCommon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommon *
        ]
    ),
    "cGcMarkerRenderData::cGcMarkerRenderData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerRenderData *
        ]
    ),
    "cTkPlane::cTkPlane": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPlane *
        ]
    ),
    "cGcPlayerWanted::~cGcPlayerWanted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWanted *
        ]
    ),
    "cGcFadeNode::SetNodeActivation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFadeNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFadeNode::SetNodeParamF": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFadeNode *
            ctypes.c_float,  # float
        ]
    ),
    "cGcHeavyAirSettingValues::cGcHeavyAirSettingValues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeavyAirSettingValues *
        ]
    ),
    "cGcSeasonalGameModeData::~cGcSeasonalGameModeData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalGameModeData *
        ]
    ),
    "cGcPetData::cGcPetData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetData *
        ]
    ),
    "cTkFixedArray<cGcPetData,18>::cTkFixedArray<cGcPetData,18>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcPetData,18> *
        ]
    ),
    "cTkFixedArray<cGcPetData,18>::~cTkFixedArray<cGcPetData,18>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcPetData,18> *
        ]
    ),
    "cGcSeasonalMilestoneEncryption::~cGcSeasonalMilestoneEncryption": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalMilestoneEncryption *
        ]
    ),
    "cGcSeasonalMilestone::~cGcSeasonalMilestone": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalMilestone *
        ]
    ),
    "cGcSeasonalStage::~cGcSeasonalStage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalStage *
        ]
    ),
    "cTkDynamicArray<cGcSeasonalStage>::~cTkDynamicArray<cGcSeasonalStage>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSeasonalStage> *
        ]
    ),
    "cGcPlayerBanner::cGcPlayerBanner": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBanner *
        ]
    ),
    "cTkFixedArray<cGcStoryPageSeenDataArray,8>::~cTkFixedArray<cGcStoryPageSeenDataArray,8>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcStoryPageSeenDataArray,8> *
        ]
    ),
    "cGcSyncBufferSaveDataArray::cGcSyncBufferSaveDataArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSyncBufferSaveDataArray *
        ]
    ),
    "cTkFixedArray<cGcStoryPageSeenDataArray,8>::cTkFixedArray<cGcStoryPageSeenDataArray,8>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcStoryPageSeenDataArray,8> *
        ]
    ),
    "cTkFixedArray<cGcSavedInteractionRaceData,131>::cTkFixedArray<cGcSavedInteractionRaceData,131>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcSavedInteractionRaceData,131> *
        ]
    ),
    "cTkFixedArray<cGcColourPaletteData,58>::cTkFixedArray<cGcColourPaletteData,58>": {
        "cTkFixedArray<cGcColourPaletteData,58> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkFixedArray<cGcColourPaletteData,58> *
            ]
        ),
        "cTkFixedArray<cGcColourPaletteData,58> *, const cTkFixedArray<cGcColourPaletteData,58> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkFixedArray<cGcColourPaletteData,58> *
                ctypes.c_ulonglong,  # const cTkFixedArray<cGcColourPaletteData,58> *
            ]
        ),
    },
    "cGcMaintenanceBuffer::GenerateChangeHash": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMaintenanceContainer *
        ]
    ),
    "cGcMaintenanceBuffer::GenerateHashValue": FUNCDEF(
        restype=ctypes.c_int16,  # __int16
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceBuffer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNetworkInteractionBufferSyncMessage::cGcNetworkInteractionBufferSyncMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInteractionBufferSyncMessage *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer>::GenerateSyncMessage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkSyncMessage> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkSyncMessage> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNetworkVariableInteractionBufferSyncMessage<cGcMaintenanceContainer>::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkVariableInteractionBufferSyncMessage<cGcMaintenanceContainer> *
        ]
    ),
    "cGcNetworkVariableInteractionBufferSyncMessage<cGcMaintenanceContainer>::cGcNetworkVariableInteractionBufferSyncMessage<cGcMaintenanceContainer>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkVariableInteractionBufferSyncMessage<cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # const cGcNetworkInteractionBufferSyncMessage *
        ]
    ),
    "cGcNetworkVariableInteractionBufferSyncMessage<cGcMaintenanceContainer>::~cGcNetworkVariableInteractionBufferSyncMessage<cGcMaintenanceContainer>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkVariableInteractionBufferSyncMessage<cGcMaintenanceContainer> *
        ]
    ),
    "cGcMaintenanceContainer::cGcMaintenanceContainer": {
        "cGcMaintenanceContainer *, cGcMaintenanceContainer *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcMaintenanceContainer *
                ctypes.c_ulonglong,  # cGcMaintenanceContainer *
            ]
        ),
        "cGcMaintenanceContainer *, const cGcMaintenanceContainer *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcMaintenanceContainer *
                ctypes.c_ulonglong,  # const cGcMaintenanceContainer *
            ]
        ),
    },
    "cGcInventoryContainer::~cGcInventoryContainer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryContainer *
        ]
    ),
    "cGcMaintenanceContainer::~cGcMaintenanceContainer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceContainer *
        ]
    ),
    "cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer>::ApplySyncMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # const cGcNetworkSyncMessage *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcTerrainEditBlockBuffer::GenerateHashValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainEditBlockBuffer::~cGcTerrainEditBlockBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
        ]
    ),
    "cGcPersistentInteractionBuffer::~cGcPersistentInteractionBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
        ]
    ),
    "cTkVector3MetaKD<int,-1>::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3MetaKD<unsigned __int64,23> *
        ]
    ),
    "cTkVector3KD<-1>::reserve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3KD<-1> *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkVector3MetaKD<int,-1>::reserve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3MetaKD<int,-1> *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkVector3MetaKD<float,-1>::~cTkVector3MetaKD<float,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3MetaKD<float,-1> *
        ]
    ),
    "nanoflann::PooledAllocator::free_all": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nanoflann::PooledAllocator *
        ]
    ),
    "cGcPersistentInteractionBuffer::cGcPersistentInteractionBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNetworkBufferHash::GetHashValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sHashValue *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBufferHash *
            ctypes.c_ulonglong,  # sHashValue *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcBaseBuildingPersistentBuffer::BaseBuildingPersistentData::IsValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer::PlayerBasePersistentData *
        ]
    ),
    "cGcBaseBuildingPersistentBuffer::GenerateHashValue": FUNCDEF(
        restype=ctypes.c_uint16,  # unsigned __int16
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPersistentBuffer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBaseBuildingPersistentBuffer::~cGcBaseBuildingPersistentBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPersistentBuffer *
        ]
    ),
    "cGcBaseBuildingGlobalBuffer::cGcBaseBuildingGlobalBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGlobalBuffer *
        ]
    ),
    "cGcWonderRecordCustomData::cGcWonderRecordCustomData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderRecordCustomData *
        ]
    ),
    "cGcWonderData::sDiscoveryData::sDiscoveryData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderData::sDiscoveryData *
        ]
    ),
    "cTkFixedArray<cGcHeavyAirColourData,5>::cTkFixedArray<cGcHeavyAirColourData,5>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcHeavyAirColourData,5> *
        ]
    ),
    "cGcPlanetGenerationWonderQuery::cGcPlanetGenerationWonderQuery": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerationWonderQuery *
        ]
    ),
    "cGcWonderData::sPlanetData::sPlanetData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderData::sPlanetData *
        ]
    ),
    "cGcWonderData::sModelResData::sModelResData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderData::sModelResData *
        ]
    ),
    "cGcWonderData::sCreatureData::sCreatureData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderData::sCreatureData *
        ]
    ),
    "cGcWonderData::cGcWonderData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderData *
        ]
    ),
    "cGcCreatureSpawnData::~cGcCreatureSpawnData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSpawnData *
        ]
    ),
    "cTkDynamicArray<cGcCreatureSpawnData>::~cTkDynamicArray<cGcCreatureSpawnData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureSpawnData> *
        ]
    ),
    "cGcPlanetGenerationWonderQuery::~cGcPlanetGenerationWonderQuery": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerationWonderQuery *
        ]
    ),
    "cGcApplication::~cGcApplication": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cGcApplicationGlobalLoadState::~cGcApplicationGlobalLoadState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGlobalLoadState *
        ]
    ),
    "cGcApplicationBaseLoadingState::cGcApplicationBaseLoadingState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationBaseLoadingState *
        ]
    ),
    "cGcApplicationSmokeTestState::~cGcApplicationSmokeTestState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationSmokeTestState *
        ]
    ),
    "cTk2dText::SetSize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dText *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cGcApplicationSimulationState::cGcApplicationSimulationState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationSimulationState *
        ]
    ),
    "cGcApplicationDeathState::~cGcApplicationDeathState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationDeathState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::cGcApplicationGameModeSelectorState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationGameModeSelectorState::EnumIconResources<eModeSelectPreset,7>::~EnumIconResources<eModeSelectPreset,7>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState::EnumIconResources<enum ePresetGameMode,7> *
        ]
    ),
    "cGcApplicationGameModeSelectorState::~cGcApplicationGameModeSelectorState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGameModeSelectorState *
        ]
    ),
    "cGcApplicationTitleScreenState::~cGcApplicationTitleScreenState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationTitleScreenState *
        ]
    ),
    "cGcApplicationState::cGcApplicationState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationState *
        ]
    ),
    "cGcApplicationUGCBaseViewerState::cGcApplicationUGCBaseViewerState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState *
        ]
    ),
    "cGcApplicationState::~cGcApplicationState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationState *
        ]
    ),
    "cGcApplicationUGCBaseViewerState::IconResources::~IconResources": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState::IconResources *
        ]
    ),
    "cGcApplicationUGCBaseViewerState::~cGcApplicationUGCBaseViewerState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState *
        ]
    ),
    "cGcApplication::cGcApplication": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplication *
        ]
    ),
    "cTkFixedString<128,char>::Format<char,0>": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcCreatureGenerationWeightedList::~cGcCreatureGenerationWeightedList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationData *
        ]
    ),
    "cGcScanEventData::~cGcScanEventData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventData *
        ]
    ),
    "cTkDynamicArray<cGcScanEventData>::~cTkDynamicArray<cGcScanEventData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcScanEventData> *
        ]
    ),
    "cTkBaseMath::USplitBits::USplitBits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBaseMath::USplitBits *
            ctypes.c_float,  # const float
        ]
    ),
    "cTkMetaDataEnumLookup::Lookup": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkLinearHashTable<cTkHashedNGuiElement,cGcNGuiElement *,-1,0,cTkHashedNGuiElement>::ChangeNodeCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<cTkHashedNGuiElement,cGcNGuiElement *,-1,0,cTkHashedNGuiElement> *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiLayer::cGcNGuiLayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcNGuiLayer::SelectableInEditor": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcNGuiLayer::Deselect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcNGuiLayer::Resized": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcNGuiLayer::AttachMetadata": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiLayer::DeallocateLayerChildrenRecursive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcNGuiLayer::DetachMetadata": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiLayer::GetSceneTreeIcon": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcNGuiLayer::GetSelected": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcNGuiElement::Resized": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
        ]
    ),
    "cGcNGuiLayer::RemoveAllElements": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcNGuiLayerData::~cGcNGuiLayerData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayerData *
        ]
    ),
    "cTkLinearHashTable<cTkHashedNGuiElement,cGcNGuiElement *,-1,0,cTkHashedNGuiElement>::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<cTkHashedNGuiElement,cGcNGuiElement *,-1,0,cTkHashedNGuiElement> *
        ]
    ),
    "cGcNGuiLayer::~cGcNGuiLayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cTkNGuiInput::~cTkNGuiInput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiInput *
        ]
    ),
    "cTk3dLayer::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk3dLayer *
        ]
    ),
    "cTk3dLayer::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk3dLayer *
            ctypes.c_float,  # float
        ]
    ),
    "cTk2dLayer::SetPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk3dLayer *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cTk3dLayer::ConstructDynamicSize": {
        "cTk3dLayer *, const cTkVector2 *, const cTkVector2 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTk3dLayer *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector2 *
            ]
        ),
        "cTk3dLayer *, const cTkVector3 *, const cTkVector2 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTk3dLayer *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector2 *
            ]
        ),
    },
    "cTk3dLayer::Construct": {
        "cTk3dLayer *, const cTkVector2 *, const cTkVector2 *, const cTkVector2 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTk3dLayer *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector2 *
            ]
        ),
        "cTk3dLayer *, const cTkVector3 *, const cTkVector2 *, const cTkVector2 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTk3dLayer *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector2 *
            ]
        ),
    },
    "cTk3dLayer::SetPosition": {
        "cTk3dLayer *, const cTkVector3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTk3dLayer *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
        "cTk3dLayer *, const cTkPhysRelVec3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTk3dLayer *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ]
        ),
    },
    "cGcNGui::cGcNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGui *
        ]
    ),
    "abs": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "cTk2dLayer::GetPosition": {
        "cTk2dLayer *, cTkVector2 *, const cTkVector2 *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkVector2 *
            argtypes=[
                ctypes.c_ulonglong,  # cTk2dLayer *
                ctypes.c_ulonglong,  # cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector2 *
            ]
        ),
        "cTk2dObject *, cTkVector2 *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkVector2 *
            argtypes=[
                ctypes.c_ulonglong,  # cTk2dObject *
                ctypes.c_ulonglong,  # cTkVector2 *
            ]
        ),
    },
    "cTk2dLayer::SetSize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dLayer *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cTk2dLayer::GetSize": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dLayer *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "cTk2dLayer::SetScale": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dLayer *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cTk2dLayer::SetAngle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dLayer *
            ctypes.c_float,  # float
        ]
    ),
    "cTk2dLayer::GetAngle": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dLayer *
        ]
    ),
    "cTk2dObject::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dObject *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cTk2dObject::GetWorldTopLeft": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dObject *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "cTk2dObject::GetPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dObject *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cTk2dObject::SetPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dObject *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cTk2dObject::SetSize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dObject *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cTk2dObject::GetSize": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dObject *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "cTk2dObject::SetAlignment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dObject *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cTk2dObject::GetAlignment": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dObject *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "cTkSmartResHandle::GetHandle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        ]
    ),
    "cTk2dObject::SetColour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dObject *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cTkTimerTemplate<0>::IsFinished": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkTimerTemplate<0> *
        ]
    ),
    "cTkTimerTemplate<0>::GetTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkTimerTemplate<0> *
        ]
    ),
    "cTkTimerTemplate<0>::GetDuration": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkTimerTemplate<0> *
        ]
    ),
    "cTkTimerTemplate<0>::GetUnitTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkTimerTemplate<0> *
            ctypes.c_uint32,  # eCurve
        ]
    ),
    "cTkLanguageManager::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkLanguageManager *
        argtypes=[]
    ),
    "cTkClock::GetTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkClock *
        ]
    ),
    "cTkStringAPI::Format": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTk2dObject::GetColour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dObject *
            ctypes.c_ulonglong,  # cTkColour *
        ]
    ),
    "cTkBase2DRenderer::cTkBase2DRenderer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBase2DRenderer *
        ]
    ),
    "cTk2dRenderer::SetShader": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dRenderer *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkShader::cTkShader": {
        "cTkShader *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkShader *
                ctypes.c_int32,  # int
            ]
        ),
        "cTkShader *, const cTkShader *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkShader *
                ctypes.c_ulonglong,  # const cTkShader *
            ]
        ),
    },
    "cTk2dRenderer::cTk2dRenderer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dRenderer *
        ]
    ),
    "cTkBase2DRenderer::~cTkBase2DRenderer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBase2DRenderer *
        ]
    ),
    "cTk2dRenderer::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTk2dRenderer *
        argtypes=[]
    ),
    "cTkDynamicArray<cTkClassPointer>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkClassPointer> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<TkID<128>>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<TkID<128> > *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<TkID<128>>::cTkDynamicArray<TkID<128>>": {
        "cTkDynamicArray<TkID<128> > *, const cTkDynamicArray<TkID<128> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDynamicArray<TkID<128> > *
                ctypes.c_ulonglong,  # const cTkDynamicArray<TkID<128> > *
            ]
        ),
        "cTkDynamicArray<TkID<128> > *, cTkDynamicArray<TkID<128> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDynamicArray<TkID<128> > *
                ctypes.c_ulonglong,  # cTkDynamicArray<TkID<128> > *
            ]
        ),
    },
    "cTkDynamicArray<cTkClassPointer>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkClassPointer> *
        ]
    ),
    "cTkDynamicArray<cTkClassPointer>::~cTkDynamicArray<cTkClassPointer>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkClassPointer> *
        ]
    ),
    "hknpShapeProperties::hknpShapeProperties": {
        "std::initializer_list<char const *> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # std::initializer_list<char const *> *
            ]
        ),
        "hknpShapeProperties *, const hknpShapeProperties *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpShapeProperties *
                ctypes.c_ulonglong,  # const hknpShapeProperties *
            ]
        ),
    },
    "cTkLinearHashTable<TkHandle,std::vector<cTkAttachment *,TkSTLAllocatorShim<cTkAttachment *,8,-1>>,-1,0,cTkLinearHashTableHash<TkHandle>>::ChangeNodeCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<TkHandle,std::vector<cTkAttachment *,TkSTLAllocatorShim<cTkAttachment *,8,-1> >,-1,0,cTkLinearHashTableHash<TkHandle> > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAttachmentManager::cTkAttachmentManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachmentManager *
        ]
    ),
    "cTkLinearHashTable<TkHandle,std::vector<cTkAttachment *,TkSTLAllocatorShim<cTkAttachment *,8,-1>>,-1,0,cTkLinearHashTableHash<TkHandle>>::~cTkLinearHashTable<TkHandle,std::vector<cTkAttachment *,TkSTLAllocatorShim<cTkAttachment *,8,-1>>,-1,0,cTkLinearHashTableHash<TkHandle>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<TkHandle,std::vector<cTkAttachment *,TkSTLAllocatorShim<cTkAttachment *,8,-1> >,-1,0,cTkLinearHashTableHash<TkHandle> > *
        ]
    ),
    "cTkComponentPool::GetComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentPool *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkComponentManager::GetPool": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponentPool *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentManager *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcProductData::~cGcProductData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductData *
        ]
    ),
    "cGcRealitySubstanceData::~cGcRealitySubstanceData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceData *
        ]
    ),
    "cTkDynamicArray<cGcRealitySubstanceData>::~cTkDynamicArray<cGcRealitySubstanceData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRealitySubstanceData> *
        ]
    ),
    "cGcTechnology::~cGcTechnology": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnology *
        ]
    ),
    "cTkInputManager::GetPrimaryPort": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkInputPort *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "cTkLSystemManager::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkLSystemManager *
        argtypes=[]
    ),
    "cTkModelDressroom::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkModelDressroom *
        argtypes=[]
    ),
    "cGcApplicationGlobalLoadState::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGlobalLoadState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationGlobalLoadState::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGlobalLoadState *
            ctypes.c_float,  # float
        ]
    ),
    "cGcApplicationGlobalLoadState::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationGlobalLoadState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcURIEncoder::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcURIEncoder *
        argtypes=[]
    ),
    "cTkTextStyle::RenderBatchPlain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkTextStyle::BuildPlainCharacter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cGcQuickMenu::GetMenuState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenu *
        ]
    ),
    "cGcHUDManager::GetNotifications": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerNotifications *
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcNPCPlacementComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementComponent *
        ]
    ),
    "cGcNPCPlacementComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCPlacementComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementComponent *
        ]
    ),
    "ComponentInstancercGcNPCPlacementComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcNPCPlacementComponent *
        ]
    ),
    "ComponentInstancercGcNPCPlacementComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcNPCPlacementComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcPlayerControlComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcApplicationLocalLoadState::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationLocalLoadState *
        ]
    ),
    "cGcApplicationLocalLoadState::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationLocalLoadState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationLocalLoadState::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationLocalLoadState *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
            ctypes.c_double,  # double
        ]
    ),
    "cGcApplicationLocalLoadState::GetRespawnReason": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationLocalLoadState *
        ]
    ),
    "cGcApplicationLocalLoadState::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationLocalLoadState *
            ctypes.c_uint32,  # EgRenderParity::List
        ]
    ),
    "cGcApplicationLocalLoadState::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationLocalLoadState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationLocalLoadState::Event": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationLocalLoadState *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcPresetDualPMap::cGcPresetDualPMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPresetDualPMap *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcPresetDualPMap::Apply": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPresetDualPMap *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "StackContainer<std::vector<cGcShipAIFreighterTargeting::cGcLaunchPoint,StackAllocator<cGcShipAIFreighterTargeting::cGcLaunchPoint,16,-1>>,16,-1>::~StackContainer<std::vector<cGcShipAIFreighterTargeting::cGcLaunchPoint,StackAllocator<cGcShipAIFreighterTargeting::cGcLaunchPoint,16,-1>>,16,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cGcShipAIFreighterTargeting::cGcLaunchPoint,StackAllocator<cGcShipAIFreighterTargeting::cGcLaunchPoint,16,-1> >,16,-1> *
        ]
    ),
    "cGcPresetTerrainTexture::Apply": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPresetTerrainTexture *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcApplicationScratchpadState::CustomData::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationScratchpadState::CustomData *
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
        ]
    ),
    "cGcApplicationScratchpadState::CustomData::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationScratchpadState::CustomData *
        ]
    ),
    "cGcApplicationScratchpadState::StateData::StateData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationScratchpadState::StateData *
        ]
    ),
    "cGcApplicationScratchpadState::~cGcApplicationScratchpadState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationScratchpadState *
        ]
    ),
    "cGcApplicationShutdownState::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationShutdownState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationScratchpadState::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationScratchpadState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationScratchpadState::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationScratchpadState *
            ctypes.c_float,  # float
        ]
    ),
    "cGcApplicationScratchpadState::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationScratchpadState *
        ]
    ),
    "cGcApplicationScratchpadState::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationScratchpadState *
            ctypes.c_uint32,  # EgRenderParity::List
        ]
    ),
    "cTkInteractivityManager::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkInteractivityManager *
        argtypes=[]
    ),
    "cGcApplicationShutdownState::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationShutdownState *
            ctypes.c_float,  # float
        ]
    ),
    "cTkTextStyleOutline::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTextStyleOutline *
            ctypes.c_ulonglong,  # cTk2dText *
        ]
    ),
    "cGcBaseMiniPortalComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponent *
        ]
    ),
    "cGcBaseMiniPortalComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcBaseMiniPortalComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcBaseMiniPortalComponent *
        ]
    ),
    "ComponentInstancercTkInstanceWindComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcBaseMiniPortalComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcLadderComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkScreenshot::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkScreenshot *
        argtypes=[]
    ),
    "cTkWindManager::UpdateVector": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkWindManager *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # std::vector<cTkWindManager::WindTypeInfo,TkSTLAllocatorShim<cTkWindManager::WindTypeInfo,8,-1> > *
        ]
    ),
    "cTkWindManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkWindManager *
            ctypes.c_float,  # float
        ]
    ),
    "cTkWindManager::cTkWindManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkWindManager *
        ]
    ),
    "cGcApplicationSimulationState::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationSimulationState *
        ]
    ),
    "cGcApplicationSimulationState::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationSimulationState *
        ]
    ),
    "cGcApplicationSimulationState::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationSimulationState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationSmokeTestState::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationSmokeTestState *
            ctypes.c_float,  # float
        ]
    ),
    "cGcApplicationSimulationState::DoThreadedUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcApplicationSimulationState::ThreadedUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationSimulationState *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_double,  # double
        ]
    ),
    "cGcApplicationSimulationState::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationSimulationState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationSimulationState::ThreadSyncPoint": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationSimulationState *
        ]
    ),
    "EnqueueApplyShadowMask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "SetGlobalJitterValues": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "EnableLightShafts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "EnqueueTerrainStageSettings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGraphicsManager *
        ]
    ),
    "cGcApplicationSimulationState::BuildRenderQueue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationSimulationState *
        ]
    ),
    "RenderNGuiUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_double,  # double
        ]
    ),
    "cGcApplicationSimulationState::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationSimulationState *
            ctypes.c_uint32,  # EgRenderParity::List
            ctypes.c_double,  # double
        ]
    ),
    "GetCameraFovAdjust": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[]
    ),
    "cGcApplicationSmokeTestState::AssertCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char * 0x40,  # cTkAssertData
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcApplicationSmokeTestState::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationSmokeTestState *
        ]
    ),
    "cGcApplicationSmokeTestState::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationSmokeTestState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationSmokeTestState::ThreadedUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationSmokeTestState *
        ]
    ),
    "cGcApplicationSmokeTestState::ThreadSyncPoint": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationSmokeTestState *
        ]
    ),
    "cGcApplicationSmokeTestState::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationSmokeTestState *
            ctypes.c_uint32,  # EgRenderParity::List
            ctypes.c_double,  # double
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcBaseMiniPortalComponent>::UpdateComponentsRender": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcBaseMiniPortalComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkFixedString<128,char>>::end": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkArraySTLIterator<cTkFixedString<128,char> >::InternalConstIterator *
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkFixedString<128,char> > *
            ctypes.c_ulonglong,  # cTkArraySTLIterator<cTkFixedString<128,char> >::InternalConstIterator *
        ]
    ),
    "cTkDynamicArray<cGcRealitySubstanceData>::begin": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkArraySTLIterator<cGcPulseEncounterInfo>::InternalConstIterator *
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPulseEncounterInfo> *
            ctypes.c_ulonglong,  # cTkArraySTLIterator<cGcPulseEncounterInfo>::InternalConstIterator *
        ]
    ),
    "cTkTimerTemplate<0>::IsActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkTimerTemplate<0> *
        ]
    ),
    "cGcApplicationScratchpadState::StateData::~StateData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationScratchpadState::StateData *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcTexturePrefetchData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEntitlementRewardsTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedString<2048,char>::Format<char,0>": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkScreenshot::cTkScreenshotData::~cTkScreenshotData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkScreenshot::cTkScreenshotData *
        ]
    ),
    "cTkLSystemManager::LSystem::~LSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager::LSystem *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cTkLSystemManager::LSystemEntry::~LSystemEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager::LSystemEntry *
        ]
    ),
    "TkSTLAllocatorShim<cGcPresetTerrainTexture,16,-1>::construct<cGcPresetTerrainTexture,char *,char *,cTkSmartResHandle &>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcPresetTerrainTexture,16,-1> *
            ctypes.c_ulonglong,  # cGcPresetTerrainTexture *
            ctypes.c_ulonglong,  # char **
            ctypes.c_ulonglong,  # char **
            ctypes.c_ulonglong,  # cTkSmartResHandle *
        ]
    ),
    "cGcNPCPlacementInfo::~cGcNPCPlacementInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementInfo *
        ]
    ),
    "cTkAttachment::GetAttachment": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachment *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkMatrix44::DeterminantPrecise": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix44 *
        ]
    ),
    "cTkMatrix44::InvertedPrecise": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix44 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix44 *
            ctypes.c_ulonglong,  # cTkMatrix44 *
        ]
    ),
    "cGcPlanet::GetPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcPlayerState::HasPortalInterference": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcSimulation::GetUp": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcSolarSystem::GetNexusExterior": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcPlayer::GetPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cTkMultipleReadersLockWriteJanitor::cTkMultipleReadersLockWriteJanitor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMultipleReadersLockWriteJanitor *
            ctypes.c_ulonglong,  # cTkMultipleReadersLock *
        ]
    ),
    "cTkNGuiEditor::DoCategoryEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
        ]
    ),
    "cTkNGuiEditor::DoCategoryBegin": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcApplicationScratchpadState::RenderNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkNGuiEditor::DoIndentEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
        ]
    ),
    "cTkNGuiEditor::DoIndentBegin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
        ]
    ),
    "FloatLimits::FloatLimits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # FloatLimits *
        ]
    ),
    "FloatEditOptions::FloatEditOptions": {
        "FloatEditOptions *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # FloatEditOptions *
            ]
        ),
        "FloatEditOptions *, float, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # FloatEditOptions *
                ctypes.c_float,  # float
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cTkNGuiEditor::DoText": {
        "cTkNGuiEditor *, const char *, eNGuiSizeType, float, float": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiEditor *
                ctypes.c_char_p,  # const char *
                ctypes.c_uint32,  # eNGuiSizeType
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
        "cTkNGuiEditor *, const char *, const char *, eNGuiSizeType": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiEditor *
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char *
                ctypes.c_uint32,  # eNGuiSizeType
            ]
        ),
    },
    "TkCalcUniformNameHashNoInl": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkHmdBase::GetProjectionMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix44 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdBase *
            ctypes.c_ulonglong,  # cTkMatrix44 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkCamera::GetFovAdjustDistance": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcPersistentTerrainEdits::~cGcPersistentTerrainEdits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentTerrainEdits *
        ]
    ),
    "cGcUGCBasesManager::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcUGCBasesManager *
        argtypes=[]
    ),
    "cGcSolarSystem::GetPlanetWritable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlanet *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSpaceshipWarp::IsUsingCommunityPortal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
        ]
    ),
    "LocalLoadPhase::GetNext": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # LocalLoadPhase::Enum
        ]
    ),
    "cGcGameState::GetMPMissionTracker": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMPMissionTracker *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
        ]
    ),
    "cTkAnimationComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcNavMeshManager::HasPendingOps": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshManager *
        ]
    ),
    "cGcSimulation::SetScene": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulation *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ScopedTimingBlock::ScopedTimingBlock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ScopedTimingBlock *
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const char *
        ]
    ),
    "GlobalLoadPhase::GetNext": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # GlobalLoadPhase::Enum
        ]
    ),
    "TkHandle::GetLookup": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcInWorldScreen::SetVisible": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldScreen *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcApplicationBaseLoadingState::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationBaseLoadingState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationBaseLoadingState::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationBaseLoadingState *
            ctypes.c_float,  # float
        ]
    ),
    "cGcApplicationBaseLoadingState::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationBaseLoadingState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationBaseLoadingState::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationBaseLoadingState *
            ctypes.c_uint32,  # EgRenderParity::List
        ]
    ),
    "cGcAsyncLoadOps::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsyncLoadOps *
        ]
    ),
    "cGcApplicationTitleScreenState::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationTitleScreenState *
        ]
    ),
    "cGcApplicationTitleScreenState::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationTitleScreenState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationTitleScreenState::SetupRenderPipeline": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationTitleScreenState *
        ]
    ),
    "cGcApplicationTitleScreenState::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationTitleScreenState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationTitleScreenState::LoadAndSetupUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationTitleScreenState *
        ]
    ),
    "cGcApplicationTitleScreenState::UpdateUserDetails": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationTitleScreenState *
        ]
    ),
    "cGcApplicationTitleScreenState::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationTitleScreenState *
            ctypes.c_uint32,  # EgRenderParity::List
        ]
    ),
    "cGcApplicationTitleScreenState::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationTitleScreenState *
            ctypes.c_float,  # float
        ]
    ),
    "cGcApplicationTitleScreenState::ServiceStateChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationTitleScreenState *
        ]
    ),
    "cGcApplicationUGCBaseViewerState::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState *
        ]
    ),
    "cGcApplicationUGCBaseViewerState::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationUGCBaseViewerState::SlotComponents::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState::SlotComponents *
        ]
    ),
    "cGcApplicationUGCBaseViewerState::SlotComponents::Resolve": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState::SlotComponents *
        ]
    ),
    "cGcApplicationUGCBaseViewerState::SlotComponents::SetPreviewImage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState::SlotComponents *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcApplicationUGCBaseViewerState::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcApplicationUGCBaseViewerState::LoadAndSetupUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState *
        ]
    ),
    "cGcApplicationUGCBaseViewerState::UnloadAndDiscardUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState *
        ]
    ),
    "cGcApplicationUGCBaseViewerState::RenderAnimatedBackground": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcApplicationUGCBaseViewerState::RenderCursor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcApplicationUGCBaseViewerState::DoPageDots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState *
        ]
    ),
    "cGcApplicationUGCBaseViewerState::CustomIconFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcApplicationUGCBaseViewerState::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState *
            ctypes.c_uint32,  # EgRenderParity::List
        ]
    ),
    "cGcApplicationUGCBaseViewerState::DoUpdateLogicDuringRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState *
        ]
    ),
    "cGcApplicationUGCBaseViewerState::UpdateUGCDetailsDownloading": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState *
        ]
    ),
    "cGcApplicationUGCBaseViewerState::UpdateDeletionWarningDlg": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState *
            ctypes.c_float,  # float
        ]
    ),
    "cGcApplicationUGCBaseViewerState::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcApplicationUGCBaseViewerState *
            ctypes.c_float,  # float
        ]
    ),
    "cTkFixedArray<cGcAtlasDiscovery,600>::cTkFixedArray<cGcAtlasDiscovery,600>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcAtlasDiscovery,600> *
        ]
    ),
    "cGcAtlasBroker::Complete": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBroker *
            ctypes.c_ulonglong,  # const AtlasRequestTask *
            ctypes.c_ulonglong,  # AtlasResponseTask *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "ITkUserServiceNotifications::AuthToken::~AuthToken": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ITkUserServiceNotifications::AuthToken *
        ]
    ),
    "cGcAtlasBrokerAuth::Construct": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAuth *
            ctypes.c_ulonglong,  # cGcAtlasTransport *
        ]
    ),
    "cGcAtlasBrokerAuth::GenerateAuthServerName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAuth *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasBrokerAuth::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAuth *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAtlasBrokerAuth::HttpHandleAuth": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAuth *
            ctypes.c_ulonglong,  # const Http::RequestResult *
        ]
    ),
    "cGcAtlasBrokerAuth::ProcessPendingNotifications": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAuth *
        ]
    ),
    "cGcAtlasBrokerAuth::AuthenticationTokenArrived": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAuth *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # const ITkUserServiceNotifications::AuthToken *
        ]
    ),
    "cGcAtlasBrokerAuth::AuthenticationTokenFailed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAuth *
            ctypes.c_uint64,  # const unsigned __int64
        ]
    ),
    "cGcAtlasBrokerAzure::GetReadyStateCounter": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
        ]
    ),
    "ServerDataFormat::WriteDiscoveryData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtlasDiscoveryData *
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
        ]
    ),
    "ServerDataFormat::ReadDiscoveryData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_ulonglong,  # cGcAtlasDiscoveryData *
        ]
    ),
    "ServerDataFormat::WriteVector3": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ServerDataFormat::ReadVector3": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ServerDataFormat::WriteBaseRequestData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
            ctypes.c_ulonglong,  # const cGcAtlasBasesRequest *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ServerDataFormat::ReadBaseObject": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_ulonglong,  # cGcPersistentBaseEntry *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ServerDataFormat::ReadBaseDifficultyData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_ulonglong,  # cGcPersistentBaseDifficultyData *
        ]
    ),
    "ServerDataFormat::ReadBaseData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_ulonglong,  # cGcPersistentBase *
        ]
    ),
    "ServerDataFormat::WriteBaseData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
            ctypes.c_ulonglong,  # const cGcPersistentBase *
        ]
    ),
    "ServerDataFormat::WriteBaseQueriesArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcAtlasBaseQuery> *
        ]
    ),
    "ServerDataFormat::ReadTerrainData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_ulonglong,  # cGcPersistentTerrainEdits *
        ]
    ),
    "ServerDataFormat::WriteTerrainData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
            ctypes.c_ulonglong,  # const cGcPersistentTerrainEdits *
        ]
    ),
    "ServerDataFormat::ReadMessageData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_ulonglong,  # cGcAtlasMessage *
        ]
    ),
    "ServerDataFormat::ReadDiscovery": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_ulonglong,  # cGcAtlasDiscovery *
        ]
    ),
    "ServerDataFormat::ReadMonumentData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_ulonglong,  # cGcAtlasMonument *
        ]
    ),
    "ServerDataFormat::ReadActiveFeaturedBaseHeaderData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_ulonglong,  # cGcAtlasActiveFeaturedBaseHeader *
        ]
    ),
    "ServerDataFormat::WriteFeaturesBasesStateChangeArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcAtlasFeaturedBaseStateChange> *
        ]
    ),
    "ServerDataFormat::ReadSettlementData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_ulonglong,  # cGcSettlementState *
        ]
    ),
    "cGcAtlasBrokerAzure::Construct": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
            ctypes.c_ulonglong,  # cGcAtlasTransport *
        ]
    ),
    "cGcAtlasBrokerAzure::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
        ]
    ),
    "cGcAtlasBrokerAzure::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAtlasBrokerAzure::IsCallRateLimited": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
            ctypes.c_ulonglong,  # AtlasTaskCommon *
        ]
    ),
    "cGcAtlasBrokerAzure::ConvertTaskDataForPOST": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
            ctypes.c_ulonglong,  # AtlasTaskCommon *
            ctypes.c_ulonglong,  # cGcURL *
            ctypes.c_ulonglong,  # cTkStringAssembler *
        ]
    ),
    "cGcAtlasBrokerAzure::GetTaskBinaryData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcAtlasShared *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
            ctypes.c_ulonglong,  # AtlasTaskCommon *
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcAtlasBrokerAzure::Service": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
            ctypes.c_ulonglong,  # AtlasTaskCommon *
        ]
    ),
    "cGcAtlasBrokerAzure::Abandon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
            ctypes.c_ulonglong,  # AtlasTaskCommon *
        ]
    ),
    "cGcAtlasBrokerAzure::DeleteTaskData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
            ctypes.c_ulonglong,  # cGcIBrokerData *
        ]
    ),
    "cGcAtlasBrokerAzure::HttpHandleService": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
            ctypes.c_ulonglong,  # const Http::RequestResult *
        ]
    ),
    "cGcAtlasBrokerAzure::DoCompletion": {
        "cGcAtlasBrokerAzure *, cGcIBrokerData *, const cGcAtlasSendRequestDiscoveryExact *, cGcAtlasRecvDiscoveryExact *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
                ctypes.c_ulonglong,  # cGcIBrokerData *
                ctypes.c_ulonglong,  # const cGcAtlasSendRequestDiscoveryExact *
                ctypes.c_ulonglong,  # cGcAtlasRecvDiscoveryExact *
            ]
        ),
        "cGcAtlasBrokerAzure *, cGcIBrokerData *, const cGcAtlasSendRequestDiscoveryCategory *, cGcAtlasRecvDiscoveryList *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
                ctypes.c_ulonglong,  # cGcIBrokerData *
                ctypes.c_ulonglong,  # const cGcAtlasSendRequestDiscoveryCategory *
                ctypes.c_ulonglong,  # cGcAtlasRecvDiscoveryList *
            ]
        ),
        "cGcAtlasBrokerAzure *, cGcIBrokerData *, const cGcAtlasSendRequestDiscoveryAllOnPlanet *, cGcAtlasRecvDiscoveryList *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
                ctypes.c_ulonglong,  # cGcIBrokerData *
                ctypes.c_ulonglong,  # const cGcAtlasSendRequestDiscoveryAllOnPlanet *
                ctypes.c_ulonglong,  # cGcAtlasRecvDiscoveryList *
            ]
        ),
        "cGcAtlasBrokerAzure *, cGcIBrokerData *, const cGcAtlasSendRequestBases *, cGcAtlasRecvBaseList *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
                ctypes.c_ulonglong,  # cGcIBrokerData *
                ctypes.c_ulonglong,  # const cGcAtlasSendRequestBases *
                ctypes.c_ulonglong,  # cGcAtlasRecvBaseList *
            ]
        ),
        "cGcAtlasBrokerAzure *, cGcIBrokerData *, const cGcAtlasSendQueryFeaturedBases *, cGcAtlasRecvFeaturedBasesQueryList *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
                ctypes.c_ulonglong,  # cGcIBrokerData *
                ctypes.c_ulonglong,  # const cGcAtlasSendQueryFeaturedBases *
                ctypes.c_ulonglong,  # cGcAtlasRecvFeaturedBasesQueryList *
            ]
        ),
        "cGcAtlasBrokerAzure *, cGcIBrokerData *, const cGcAtlasSendSubmitMonument *, cGcAtlasRecvBasic *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
                ctypes.c_ulonglong,  # cGcIBrokerData *
                ctypes.c_ulonglong,  # const cGcAtlasSendSubmitMonument *
                ctypes.c_ulonglong,  # cGcAtlasRecvBasic *
            ]
        ),
        "cGcAtlasBrokerAzure *, cGcIBrokerData *, const cGcAtlasSendQueryActiveFeaturedBases *, cGcAtlasRecvActiveFeaturedBasesQuery *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
                ctypes.c_ulonglong,  # cGcIBrokerData *
                ctypes.c_ulonglong,  # const cGcAtlasSendQueryActiveFeaturedBases *
                ctypes.c_ulonglong,  # cGcAtlasRecvActiveFeaturedBasesQuery *
            ]
        ),
        "cGcAtlasBrokerAzure *, cGcIBrokerData *, const cGcAtlasSendQueryBaseImages *, cGcAtlasRecvBaseImages *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
                ctypes.c_ulonglong,  # cGcIBrokerData *
                ctypes.c_ulonglong,  # const cGcAtlasSendQueryBaseImages *
                ctypes.c_ulonglong,  # cGcAtlasRecvBaseImages *
            ]
        ),
        "cGcAtlasBrokerAzure *, cGcIBrokerData *, const cGcAtlasSendRequestVoxel *, cGcAtlasRecvVoxel *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
                ctypes.c_ulonglong,  # cGcIBrokerData *
                ctypes.c_ulonglong,  # const cGcAtlasSendRequestVoxel *
                ctypes.c_ulonglong,  # cGcAtlasRecvVoxel *
            ]
        ),
        "cGcAtlasBrokerAzure *, cGcIBrokerData *, const cGcAtlasSendRequestBlob *, cGcAtlasRecvBlob *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
                ctypes.c_ulonglong,  # cGcIBrokerData *
                ctypes.c_ulonglong,  # const cGcAtlasSendRequestBlob *
                ctypes.c_ulonglong,  # cGcAtlasRecvBlob *
            ]
        ),
        "cGcAtlasBrokerAzure *, cGcIBrokerData *, const cGcAtlasSendRequestMonument *, cGcAtlasRecvMonumentList *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
                ctypes.c_ulonglong,  # cGcIBrokerData *
                ctypes.c_ulonglong,  # const cGcAtlasSendRequestMonument *
                ctypes.c_ulonglong,  # cGcAtlasRecvMonumentList *
            ]
        ),
        "cGcAtlasBrokerAzure *, cGcIBrokerData *, const cGcAtlasSendRequestTotalContribution *, cGcAtlasRecvTotalContribution *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
                ctypes.c_ulonglong,  # cGcIBrokerData *
                ctypes.c_ulonglong,  # const cGcAtlasSendRequestTotalContribution *
                ctypes.c_ulonglong,  # cGcAtlasRecvTotalContribution *
            ]
        ),
        "cGcAtlasBrokerAzure *, cGcIBrokerData *, const cGcAtlasSendQuerySeasonData *, cGcAtlasRecvBasic *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
                ctypes.c_ulonglong,  # cGcIBrokerData *
                ctypes.c_ulonglong,  # const cGcAtlasSendQuerySeasonData *
                ctypes.c_ulonglong,  # cGcAtlasRecvBasic *
            ]
        ),
        "cGcAtlasBrokerAzure *, cGcIBrokerData *, const cGcAtlasSendSubmitSettlement *, cGcAtlasRecvBasic *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
                ctypes.c_ulonglong,  # cGcIBrokerData *
                ctypes.c_ulonglong,  # const cGcAtlasSendSubmitSettlement *
                ctypes.c_ulonglong,  # cGcAtlasRecvBasic *
            ]
        ),
        "cGcAtlasBrokerAzure *, cGcIBrokerData *, const cGcAtlasSendRequestSettlements *, cGcAtlasRecvSettlementList *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
                ctypes.c_ulonglong,  # cGcIBrokerData *
                ctypes.c_ulonglong,  # const cGcAtlasSendRequestSettlements *
                ctypes.c_ulonglong,  # cGcAtlasRecvSettlementList *
            ]
        ),
    },
    "cTkDynamicArray<cGcTerrainEdit>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTerrainEdit> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPersistentBase>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPersistentBase> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPersistentBaseEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPersistentBaseEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<float>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<float> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkVector3>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkVector3> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDebugHandler::OutputStyled<5>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ITkDocumentReader::ReadValue<128>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "ServerDataFormat::ReadDynamicArray<TkID<128>>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_ulonglong,  # cTkDynamicArray<TkID<128> > *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ITkDocumentReader::ReadValue<512>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "ITkDocumentReader::ReadValue<32>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        ]
    ),
    "ITkDocumentReader::ReadValue<64>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        ]
    ),
    "cGcUniverseAddressCoderU64::Decode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcUniverseAddressData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniverseAddressData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcURL::cGcURL": {
        "cGcURL *, const cGcURL *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcURL *
                ctypes.c_ulonglong,  # const cGcURL *
            ]
        ),
        "cGcURL *, unsigned __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcURL *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
    },
    "cTkDynamicArray<uchar>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<unsigned char> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<char>::cTkDynamicArray<char>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<unsigned char> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<unsigned char> *
        ]
    ),
    "Http::Request::~Request": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Http::Request *
        ]
    ),
    "Http::Request::POST": FUNCDEF(
        restype=ctypes.c_ulonglong,  # Http::Request *
        argtypes=[
            ctypes.c_ulonglong,  # Http::Request *
            ctypes.c_ulonglong,  # const cGcURL *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "StackContainer<std::vector<cTkFixedString<160,char>,StackAllocator<cTkFixedString<160,char>,4,-1>>,4,-1>::~StackContainer<std::vector<cTkFixedString<160,char>,StackAllocator<cTkFixedString<160,char>,4,-1>>,4,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cTkFixedString<160,char>,StackAllocator<cTkFixedString<160,char>,4,-1> >,4,-1> *
        ]
    ),
    "cTkFixedArray<cGcSettlementState,100>::cTkFixedArray<cGcSettlementState,100>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcSettlementState,100> *
        ]
    ),
    "cTkFixedArray<cGcSettlementState,100>::~cTkFixedArray<cGcSettlementState,100>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcSettlementState,100> *
        ]
    ),
    "cTkDynamicArray<cGcPersistentBase>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPersistentBase> *
        ]
    ),
    "cTkUgcManager::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkUgcManager *
        argtypes=[]
    ),
    "cGcNGuiLayer::AddElement": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiElement *
            ctypes.c_int64,  # __int64
        ]
    ),
    "TitleScreenPhase::GetNext": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # TitleScreenPhase::Enum
        ]
    ),
    "cTkUserService::GetUserNameA": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkUserService::UserIdentity *
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserService *
        ]
    ),
    "cTk2dImage::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dImage *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcAtlasTransportWinHttp::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasTransportWinHttp *
        ]
    ),
    "cGcAtlasBrokerAuth::cGcAtlasBrokerAuth": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAuth *
        ]
    ),
    "cGcAtlasBrokerAuth::~cGcAtlasBrokerAuth": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAuth *
        ]
    ),
    "cGcAtlasBrokerAuth::State_AuthenticateWithServer::~State_AuthenticateWithServer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAuth::State_AuthenticateWithServer *
        ]
    ),
    "cGcAtlasBrokerAzure::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
        ]
    ),
    "cGcAtlasBrokerAzure::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
        ]
    ),
    "cGcAtlasBrokerAzure::GetMessagesEnabled": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
        ]
    ),
    "cGcAtlasBrokerAzure::GetOpenIdToken": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::string *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
        ]
    ),
    "cGcAtlasBrokerAzure::GetAtlasCommunityData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAtlasCommunityData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
        ]
    ),
    "cGcAtlasBrokerAzure::GetAtlasBaseImagesRoute": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcURL *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
        ]
    ),
    "cGcAtlasBrokerAzure::GetAtlasFeaturedBaseImagesRoute": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcURL *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
        ]
    ),
    "cGcAtlasBrokerAzure::GetAtlasSeasonDataRoute": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcURL *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBrokerAzure *
        ]
    ),
    "cGcAtlasSendSubmitBaseWithImage::~cGcAtlasSendSubmitBaseWithImage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendSubmitBaseWithImage *
        ]
    ),
    "cGcAtlasManager::Data::Data": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasManager::Data *
        ]
    ),
    "cGcAtlasManager::Data::UpdateInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasManager::Data *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAtlasManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasManager *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcAtlasManager::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasManager *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcAtlasManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAtlasManager::RenderActivityIcon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasManager *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_ubyte,  # const bool
            ctypes.c_float,  # const float
        ]
    ),
    "cGcAtlasManager::GetConnectionState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasManager *
        ]
    ),
    "cGcAtlasManager::GetDisplayConnectionState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasManager *
        ]
    ),
    "cGcAtlasManager::InternalEnqueueRequest": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasManager *
            ctypes.c_ulonglong,  # const cTkMetaMessageWrapperTemplated<void> *
            ctypes.c_float,  # float
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcAtlasSendSubmitBase::cGcAtlasSendSubmitBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendSubmitBase *
            ctypes.c_ulonglong,  # const cGcAtlasSendSubmitBase *
        ]
    ),
    "cGcAtlasSendReport::cGcAtlasSendReport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendReport *
            ctypes.c_ulonglong,  # const cGcAtlasSendReport *
        ]
    ),
    "cGcAtlasManager::CancelRequest": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasManager *
            ctypes.c_uint64,  # const unsigned __int64
        ]
    ),
    "cGcAtlasManager::PollForRequestCompletion": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasManager *
            ctypes.c_uint64,  # const unsigned __int64
        ]
    ),
    "cGcAtlasManager::FetchRequestResult": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasManager *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkMetaMessageWrapperTemplated<cGcAtlasShared> *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcAtlasManager::FreeRequestResult": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasManager *
            ctypes.c_uint64,  # const unsigned __int64
        ]
    ),
    "cGcAtlasManager::GetAtlasCommunityData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAtlasCommunityData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasManager *
        ]
    ),
    "cGcAtlasManager::DoThreadedUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcAtlasCommunityData::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasCommunityData *
        ]
    ),
    "WideStringFromUTF8": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # std::basic_string<wchar_t,std::char_traits<wchar_t>,TkSTLAllocatorShim<wchar_t,2,5> > *
        ]
    ),
    "cGcAtlasTransportWinHttp::Data::ConstructTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasTransportWinHttp::Data *
            ctypes.c_ulonglong,  # WinHttpTask *
        ]
    ),
    "cGcAtlasTransportWinHttp::Data::TickTask": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasTransportWinHttp::Data *
            ctypes.c_ulonglong,  # WinHttpTask *
        ]
    ),
    "cGcAtlasTransportWinHttp::Data::WinHttpAsyncCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcAtlasTransportWinHttp::Data::Construct": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasTransportWinHttp::Data *
        ]
    ),
    "cGcAtlasTransportWinHttp::Construct": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasTransportWinHttp *
        ]
    ),
    "cGcAtlasTransportWinHttp::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasTransportWinHttp *
        ]
    ),
    "cGcAtlasTransportWinHttp::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasTransportWinHttp *
        ]
    ),
    "cGcAtlasTransportWinHttp::GetNetworkState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasTransportWinHttp *
        ]
    ),
    "cGcAtlasTransportWinHttp::IsTrafficInFlight": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasTransportWinHttp *
        ]
    ),
    "cGcAtlasTransportWinHttp::RequestHttpAsync": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasTransportWinHttp *
            ctypes.c_ulonglong,  # const Http::Request *
            ctypes.c_ulonglong,  # const std::function<void __cdecl(Http::RequestResult const &)> *
        ]
    ),
    "cGcAudioEmitterScaffold::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioEmitterScaffold *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudioEmitterScaffold::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioEmitterScaffold *
        ]
    ),
    "cGcAudioEmitterScaffold::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioEmitterScaffold *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcAudioEmitterDistanceSquish::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioEmitterDistanceSquish *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcVehicleCheckpointComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointComponent *
        ]
    ),
    "cGcVehicleCheckpointComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcVehicleCheckpointComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcVehicleCheckpointComponent *
        ]
    ),
    "ComponentInstancercGcVehicleCheckpointComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcVehicleCheckpointComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcVehicleCheckpointComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentLoadInstancercGcVehicleCheckpointComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentLoadInstancercGcVehicleCheckpointComponent *
        ]
    ),
    "cGcSpaceshipComponent::GetData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearMemoryPool *
        ]
    ),
    "VmaBlockMetadata_Linear::GetSumFreeSize": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeDecorator *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerCharacterComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcPlayerCharacterComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcPlayerCharacterComponent *
        ]
    ),
    "ComponentInstancercGcPlayerCharacterComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcPlayerCharacterComponent::ConstructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::cGcPlayerCharacterComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcAnimationLayerQueue::cGcAnimationLayerQueue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAnimationLayerQueue *
        ]
    ),
    "ComponentInstancercGcPlayerCharacterComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcPlayerBackpackBars::~cGcPlayerBackpackBars": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBackpackBars *
        ]
    ),
    "cGcAudioManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
        ]
    ),
    "cGcAudioManager::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
        ]
    ),
    "cGcAudioManager::BeginLoadingMusic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
        ]
    ),
    "cGcAudioManager::StopLoadingMusic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
        ]
    ),
    "cGcAudioManager::Release": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcAudioManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAudioManager::OnSimulationStateEnter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudioManager::NextPulseVariant": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
            ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
        ]
    ),
    "cGcAudioManager::ChangeUIState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAudioManager::UpdateAuxSendWithFalloffConcrete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint64,  # TkAudioObject
            ctypes.c_float,  # const float
        ]
    ),
    "cGcAudioManager::UpdateAuxSendFull": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
            ctypes.c_uint64,  # TkAudioObject
        ]
    ),
    "cGcAudioManager::RunDopplerRequests": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAudioManager::SortAndCueFootsteps": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
        ]
    ),
    "cGcAudioManager::RunObstructionQueries": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAudioManager::UpdateAudioPulseState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
            ctypes.c_float,  # const float
        ]
    ),
    "UpdateCommsChatterRTPCS": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
            ctypes.c_ulonglong,  # TkAudioObject *
            ctypes.c_ulonglong,  # const cGcPlayerEnvironment::SpaceState *
        ]
    ),
    "cGcAudioManager::UpdateAudioEnvironment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcAudioManager::UpdateAudioEnvironment_LocationAndInterest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcAudioManager::UpdateAudioEnvironment_FreighterTriggers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcAudioManager::UpdateAudioEnvironment_InteriorTriggers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioManager *
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "ShorelineAudio::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ShorelineAudio *
            ctypes.c_float,  # float
        ]
    ),
    "PulseItemList<cGcAudioPulseMusic::SoundScape::SoundScapeIndex::EventGenerator::ModulatorLink>::Load": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # PulseItemList<cGcAudioPulseMusic::SoundScape::SoundScapeIndex::EventGenerator::ModulatorLink> *
            ctypes.c_ulonglong,  # const XMLNode *
        ]
    ),
    "PulseItemList<cGcAudioPulseMusic::SoundScape::SoundScapeIndex::EventGenerator>::Load": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # PulseItemList<cGcAudioPulseMusic::SoundScape::SoundScapeIndex::EventGenerator> *
            ctypes.c_ulonglong,  # const XMLNode *
        ]
    ),
    "PulseItemList<cGcAudioPulseMusic::SoundScape>::Load": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # PulseItemList<cGcAudioPulseMusic::SoundScape> *
            ctypes.c_ulonglong,  # const XMLNode *
        ]
    ),
    "PulseItemList<cGcAudioPulseMusic::Instrument::InstrumentIndex::InstrumentNote>::Load": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # PulseItemList<cGcAudioPulseMusic::Instrument::InstrumentIndex::InstrumentNote> *
            ctypes.c_ulonglong,  # const XMLNode *
        ]
    ),
    "PulseItemList<cGcAudioPulseMusic::Instrument>::Load": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # PulseItemList<cGcAudioPulseMusic::Instrument> *
            ctypes.c_ulonglong,  # const XMLNode *
        ]
    ),
    "cGcAudioPulseMusic::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseMusic *
        ]
    ),
    "cGcAudioPulseMusic::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseMusic *
        ]
    ),
    "cGcAudioPulseMusic::ReleaseProject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseMusic *
        ]
    ),
    "cGcAudioPulseMusic::StopMusic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseMusic *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAudioPulseMusic::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseMusic *
        ]
    ),
    "cGcAudioPulseMusic::SetVariantIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseMusic *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
        ]
    ),
    "cGcAudioPulseMusic::RefreshSoundScapeType": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseMusic *
        ]
    ),
    "cGcAudioPulseMusic::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseMusic *
        ]
    ),
    "cGcAudioPulseMusic::SoundScape::Load": {
        "cGcAudioPulseMusic::SoundScape *, const XMLNode *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAudioPulseMusic::SoundScape *
                ctypes.c_ulonglong,  # const XMLNode *
            ]
        ),
        "cGcAudioPulseMusic::SoundScape *, const cTkFixedString<256,char> *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAudioPulseMusic::SoundScape *
                ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
            ]
        ),
    },
    "cGcAudioPulseMusic::LoadProject": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseMusic *
            ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
        ]
    ),
    "cGcAudioPulseMusic::Instrument::Load": {
        "cGcAudioPulseMusic::Instrument *, const XMLNode *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAudioPulseMusic::Instrument *
                ctypes.c_ulonglong,  # const XMLNode *
            ]
        ),
        "cGcAudioPulseMusic::Instrument *, const cTkFixedString<256,char> *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcAudioPulseMusic::Instrument *
                ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
            ]
        ),
    },
    "cGcAudioPulseMusic::SoundScape::SoundScapeIndex::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseMusic::SoundScape::SoundScapeIndex *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAudioPulseMusic::GetRNDRange": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAudioPulseMusic::SoundScape::SoundScapeIndex::EventGenerator::GenerateSubEvents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseMusic::SoundScape::SoundScapeIndex::EventGenerator *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcAudioPulseMusic::SoundScape::SoundScapeIndex *
        ]
    ),
    "cGcAudioPulseMusic::SoundScape::SoundScapeIndex::EventGenerator::Update": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseMusic::SoundScape::SoundScapeIndex::EventGenerator *
            ctypes.c_ulonglong,  # cGcAudioPulseMusic::SoundScape::SoundScapeIndex *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudioPulseMusic::SoundScape::SoundScapeIndex::EventGenerator::AudioEvent::Play": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseMusic::SoundScape::SoundScapeIndex::EventGenerator::AudioEvent *
        ]
    ),
    "cGcAudioPulseMusic::SoundScape::SoundScapeIndex::EventGenerator::AudioEvent::EndOfEventCallbackFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # AkCallbackType
            ctypes.c_ulonglong,  # AkCallbackInfo *
        ]
    ),
    "cGcAudioPulseMusic::SoundScape::SoundScapeIndex::EventGenerator::AudioEventCreate": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseMusic::SoundScape::SoundScapeIndex::EventGenerator *
            ctypes.c_ulonglong,  # const cGcAudioPulseMusic::PulseEvent *
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAudioPulseMusic::SoundScape::SoundScapeIndex::EventGenerator::AudioEventRelease": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseMusic::SoundScape::SoundScapeIndex::EventGenerator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAudioPulseMusic::SoundScape::SoundScapeIndex::EventGenerator::GetDistanceAttenuation": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseMusic::SoundScape::SoundScapeIndex::EventGenerator *
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcPlayerCharacterComponent>::UpdateComponentsRender": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcPlayerCharacterComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcPlayerCharacterComponent>::UpdateComponentsPostPhysics": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcPlayerCharacterComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "TransportTaskContainer<WinHttpTask>::~TransportTaskContainer<WinHttpTask>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TransportTaskContainer<WinHttpTask> *
        ]
    ),
    "TransportTaskContainer<WinHttpTask>::TransportTaskContainer<WinHttpTask>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TransportTaskContainer<WinHttpTask> *
        ]
    ),
    "cTkLockFreeQueue<AtlasRequestTask *>::Push": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkLockFreeQueue<AtlasRequestTask *> *
            ctypes.c_ulonglong,  # AtlasRequestTask *const *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcAtlasFeaturedBaseStateChange>::cTkDynamicArray<cGcAtlasFeaturedBaseStateChange>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAtlasFeaturedBaseStateChange> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcAtlasFeaturedBaseStateChange> *
        ]
    ),
    "cGcAudioPulseMusic::SoundScape::SoundScape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseMusic::SoundScape *
        ]
    ),
    "PulseItemList<cGcAudioPulseMusic::SoundScape>::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PulseItemList<cGcAudioPulseMusic::SoundScape> *
        ]
    ),
    "cGcAudioPulseMusic::Instrument::Instrument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseMusic::Instrument *
        ]
    ),
    "cTkDynamicArray<float>::cTkDynamicArray<float>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<float> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<float> *
        ]
    ),
    "cTkDynamicArray<cTkVector3>::cTkDynamicArray<cTkVector3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkVector3> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkVector3> *
        ]
    ),
    "cTkDynamicArray<cGcAtlasFeaturedBaseStateChange>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAtlasFeaturedBaseStateChange> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcAtlasBaseQuery>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAtlasBaseQuery> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkFixedString<64,wchar_t>::Format<wchar_t,0>": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<64,wchar_t> *
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcAudioPulseDemo>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAudioPulseDemo *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "TkTraits::detail::CallIfTrivial<WinHttpTask,0>::destructor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # WinHttpTask *
        ]
    ),
    "TkTraits::detail::CallIfTrivial<WinHttpTask,0>::constructor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # WinHttpTask *
        ]
    ),
    "cGcAudioObstructionRequest::cGcAudioObstructionRequest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioObstructionRequest *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAudioPulseXmlHelper::Open": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseXmlHelper *
            ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
        ]
    ),
    "cGcAudioPulseMusic::RenderNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "getNodeData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # const XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "getNodeData_0": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # const XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "getNodeData_1": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # const XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcPlayer::GetThirdPerson": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerThirdPerson *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcCameraManager::IsBehaviourActiveOrInterpolating": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_uint32,  # eCameraBehaviours
        ]
    ),
    "cTkMemoryPoolBlock::~cTkMemoryPoolBlock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMemoryPoolBlock *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkTrig<float>::Cos": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "cGcByteBeatTree::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTree *
        ]
    ),
    "cGcByteBeatSequencer::cGcByteBeatSequencer": {
        "cGcByteBeatSequencer *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcByteBeatSequencer *
            ]
        ),
        "cGcByteBeatSequencer *, const cGcByteBeatSequencer *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcByteBeatSequencer *
                ctypes.c_ulonglong,  # const cGcByteBeatSequencer *
            ]
        ),
    },
    "cGcByteBeatSequencer::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSequencer *
        ]
    ),
    "cGcByteBeatSequencer::PostSerialise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSequencer *
        ]
    ),
    "cGcByteBeatSequencer::ScaleDegreeToNote": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # eByteBeatKey
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcByteBeatSequencer::ScaleDegreeToOctaveOffset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # eByteBeatKey
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcByteBeatSequencer::cGcByteBeatRhythmTrack::cGcByteBeatRhythmTrack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTree *
        ]
    ),
    "cGcByteBeatGroupData::cGcByteBeatGroupData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatGroupData *
        ]
    ),
    "cGcByteBeat::GetCurrentSample": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
        ]
    ),
    "cGcByteBeat::GetSample": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
        ]
    ),
    "cGcByteBeat::DoesMessageHaveAudibleContent": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        ]
    ),
    "cGcTriggerActionComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
        ]
    ),
    "cGcTriggerActionComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcTriggerActionComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTriggerActionComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTriggerActionComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
        ]
    ),
    "cGcTriggerActionComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcTriggerActionComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcTriggerActionComponent *
        ]
    ),
    "ComponentInstancercGcTriggerActionComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkAudioComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioComponent *
        ]
    ),
    "cTkAudioComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioComponent *
        ]
    ),
    "cTkAudioComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercTkAudioComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercTkAudioComponent *
        ]
    ),
    "ComponentInstancercTkAudioComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercTkAudioComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercTkAudioComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcCharacterInterface::GetRidingAttachment": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkUserIdBase<unsigned __int64> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
            ctypes.c_ulonglong,  # cTkUserIdBase<unsigned __int64> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCharacterInterface::UpdateIdle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterInterface *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCInteractiveObjectComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectComponent *
        ]
    ),
    "cGcNPCInteractiveObjectComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcNPCInteractiveObjectComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcNPCInteractiveObjectComponent *
        ]
    ),
    "ComponentInstancercGcNPCInteractiveObjectComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcNPCNavigationAreaComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
        ]
    ),
    "cGcNPCNavigationAreaComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcNPCNavigationAreaComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcNPCNavigationAreaComponent *
        ]
    ),
    "ComponentInstancercGcNPCNavigationAreaComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcNPCNavigationAreaComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcNPCNavigationAreaComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "StackContainer<std::vector<cGcNavGraphNodeHandle,StackAllocator<cGcNavGraphNodeHandle,64,20>>,64,20>::~StackContainer<std::vector<cGcNavGraphNodeHandle,StackAllocator<cGcNavGraphNodeHandle,64,20>>,64,20>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cGcNavGraphNodeHandle,StackAllocator<cGcNavGraphNodeHandle,64,20> >,64,20> *
        ]
    ),
    "cGcNPCComponent::IsRunning": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::GetRace": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachment_vtbl *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::IsMech": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::GetMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcNPCComponent::GetCharacterSit": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcNPCComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcNPCComponent *
        ]
    ),
    "ComponentInstancercGcNPCComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcNPCComponent::ConstructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcNPCComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcNPCComponent::cGcNPCComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcInteractionComponent::CanTransferOwnership": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetTypedComponent": {
        "cTkAttachment *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
        "int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcInteractionComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcInteractionComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcInteractionComponent *
        ]
    ),
    "ComponentInstancercGcInteractionComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcInteractionComponent::ConstructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcInteractionComponent::cGcInteractionComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "ComponentInstancercGcInteractionComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcInteractionComponent::~cGcInteractionComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcAlienPuzzleEntry::~cGcAlienPuzzleEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleEntry *
        ]
    ),
    "cGcInteractionInterface::~cGcInteractionInterface": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionInterface *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<std::vector<bool,TkSTLAllocatorShim<bool,1,-1> >,8,-1> *
        ]
    ),
    "ComponentLoadInstancercGcInteractionComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentLoadInstancercGcInteractionComponent *
        ]
    ),
    "cGcFrontendPageByteBeat::PrepareToOpen": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcByteBeatPlayerComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcByteBeatPlayerComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatPlayerComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatPlayerComponent *
        ]
    ),
    "cGcByteBeatPlayerComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatPlayerComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcByteBeatPlayerComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatPlayerComponent *
        ]
    ),
    "ComponentInstancercGcByteBeatPlayerComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcByteBeatPlayerComponent *
        ]
    ),
    "ComponentInstancercGcByteBeatPlayerComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcByteBeatPlayerComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcByteBeatPlayerComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcByteBeat::SetScreenDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatSequencer::InitStaticData": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcByteBeat::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcByteBeat::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcByteBeat::SortForCulling": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
        ]
    ),
    "cGcByteBeat::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
            ctypes.c_float,  # float
        ]
    ),
    "cGcByteBeat::RenderScreen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcByteBeat::SetPowered": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeat::GetConnectectSequencers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
            ctypes.c_ulonglong,  # cGcByteBeatGroupData *
        ]
    ),
    "cGcByteBeat::GetAllSequencers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
            ctypes.c_ulonglong,  # cGcByteBeatGroupData *
        ]
    ),
    "cGcByteBeat::SetConnectectPartSequence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcByteBeat::SetConnectectMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcByteBeat::WriteTreeAndSequenceToMessage": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
            ctypes.c_ulonglong,  # cGcByteBeatTree *
            ctypes.c_ulonglong,  # cGcByteBeatSequencer *
        ]
    ),
    "cGcByteBeat::ReadTreeAndSequenceFromMessage": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTree *
            ctypes.c_ulonglong,  # cGcByteBeatSequencer *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # XMLError *
        ]
    ),
    "cGcByteBeat::LoadFromBaseBuildingData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
        ]
    ),
    "cGcByteBeat::SaveToBaseBuildingData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeat::GetAsString": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
            ctypes.c_uint32,  # eByteBeatCopyType
        ]
    ),
    "cGcByteBeat::SetFromString": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
            ctypes.c_uint32,  # eByteBeatCopyType
        ]
    ),
    "cGcByteBeat::GetByteBeatSwitchFlags": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcByteBeat::GetAudioStreamFormat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatPlayerAudioStream *
            ctypes.c_ulonglong,  # cTkAudioStreamFormat *
        ]
    ),
    "cGcByteBeat::GetAudioStreamData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "cGcByteBeat::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
        ]
    ),
    "cGcByteBeatSequencer::SetDrumFromTemplate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSequencer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcByteBeatTree::Tokenize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTree *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # std::vector<cGcByteBeatTokenInfo,TkSTLAllocatorShim<cGcByteBeatTokenInfo,4,-1> > *
        ]
    ),
    "cGcByteBeatTree::Parse": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTree *
            ctypes.c_ulonglong,  # std::vector<cGcByteBeatTokenInfo,TkSTLAllocatorShim<cGcByteBeatTokenInfo,4,-1> > *
        ]
    ),
    "cGcByteBeatTree::Compute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTree *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcByteBeatTree::SetMessage": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTree *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcByteBeatTree::Serialise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTree *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "cGcByteBeatSequencer::GetRhythmEnvelope": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSequencer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # long double *
        ]
    ),
    "cGcByteBeatSequencer::AugmentRhythmFloatSample": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSequencer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_double,  # long double
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_double,  # long double
        ]
    ),
    "cGcByteBeatSequencer::ComputeArpeggioSteps": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSequencer *
        ]
    ),
    "cGcByteBeatSequencer::Randomise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSequencer *
            ctypes.c_uint32,  # eByteBeatRandomiseType
        ]
    ),
    "cGcByteBeatSequencer::GetByteBeatSwitchStatus": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSequencer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcByteBeatSequencer::Serialise": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSequencer *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcAsyncOpsTempBase::ProfanityCheckBase": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # std::shared_ptr<cGcAsyncOpsTempBase> *
        ]
    ),
    "cGcAsyncOpsTempBase::RetrieveOwnerDisplayName": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # std::shared_ptr<cGcAsyncOpsTempBase> *
        ]
    ),
    "cGcAsyncOpsTempBase::SetBaseName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsyncOpsTempBase *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcAsyncOpsTempBase::GetOwnerData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDiscoveryOwner *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsyncOpsTempBase *
        ]
    ),
    "cGcBaseBuildingBaseLayout::GenerateLayout": {
        "cGcBaseBuildingBaseLayout *, cGcPlayerBasePersistentBuffer *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcBaseBuildingBaseLayout *
                ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ]
        ),
        "cGcBaseBuildingBaseLayout *, unsigned __int64, const cTkVector3 *, const cTkMatrix34 *, const cGcPersistentBaseEntry *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcBaseBuildingBaseLayout *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkMatrix34 *
                ctypes.c_ulonglong,  # const cGcPersistentBaseEntry *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcBaseBuildingBaseLayout::AddObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingBaseLayout *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcBaseBuildingBaseLayout::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingBaseLayout *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBaseBuildingBaseLayout::ForceGenerateLayout": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingBaseLayout *
        ]
    ),
    "cGcBaseBuildingBaseLayout::TestSphere": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingBaseLayout *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBaseBuildingBaseLayout::TestBox": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingBaseLayout *
            ctypes.c_ulonglong,  # const cTkAABB *
        ]
    ),
    "cGcBaseBuildingBaseLayout::TestLayout": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingBaseLayout *
            ctypes.c_ulonglong,  # const cGcBaseBuildingBaseLayout *
        ]
    ),
    "cGcAtlasManager::IsConnectionDown": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasManager *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBaseBuildingDownloadsManager::UpdateBaseExistsBatches": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::DownloadBases": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::UploadDeleteBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
            ctypes.c_ulonglong,  # _WORD *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::BaseUploadingCooldown": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::UploadReportBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
            ctypes.c_ulonglong,  # _WORD *
            ctypes.c_ulonglong,  # const std::vector<char,TkSTLAllocatorShim<char,1,-1> > *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::BackgroundUploadBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::ValidateBaseForUpload": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
            ctypes.c_ulonglong,  # unsigned __int16 *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::ValidateDownloadedBase": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
            ctypes.c_ulonglong,  # const cGcPersistentBase *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cGcBaseBuildingBaseLayout *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::ProcessAtlasResponse": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # const cTkMetaMessageWrapperTemplated<cGcAtlasShared> *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_uint32,  # const AtlasRequestResult
        ]
    ),
    "cGcBaseBuildingDownloadsManager::ProccessBaseQueryResponse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
            ctypes.c_ulonglong,  # const cGcAtlasRecvBaseQueryList *
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager::BaseQueryData *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::ProccessFeaturedBasesQueryResponse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
            ctypes.c_ulonglong,  # const cGcAtlasRecvFeaturedBasesQueryList *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::BaseData::BaseData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager::BaseData *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::SendBaseExistsQuery": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::GetBaseExistsQueryData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcBaseBuildingDownloadsManager::BaseQueryData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
            ctypes.c_ulonglong,  # const cGcBaseBuildingDownloadsManager::BaseQueryData *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::ResetDownloadBaseRequestData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::GetFeaturedBasesData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcBaseBuildingDownloadsManager::FeaturedBasesQueryData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
            ctypes.c_ulonglong,  # const cGcBaseBuildingDownloadsManager::FeaturedBasesQueryData *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::HasFeaturedBaseInUA": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingDownloadsManager::IsFeaturedBase": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
            ctypes.c_ulonglong,  # const cGcUniverseAddressData *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::MarkBaseAsFeatured": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
            ctypes.c_ulonglong,  # const BaseIndex *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::BackgroundMarkFeaturedBaseWithScreenshot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::SendActiveFeaturedBasesQuery": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
        ]
    ),
    "cGcBaseBuildingEffectsManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEffectsManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBaseBuildingEffectsManager::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEffectsManager *
        ]
    ),
    "cGcBaseBuildingEffectsManager::AddScanEffectToNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEffectsManager *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cGcScanEffectData *
        ]
    ),
    "cGcBaseBuildingEffectsManager::HideScanEffectOnChildren": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEffectsManager *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcBaseBuildingEffectsManager::RestoreScanEffectOnChildren": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEffectsManager *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cGcScanEffectData *
        ]
    ),
    "cGcBaseBuildingEffectsManager::ChangeScanEffectOnNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEffectsManager *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cGcScanEffectData *
        ]
    ),
    "cGcBaseBuildingEffectsManager::ChangeScanEffectOnChildren": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEffectsManager *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cGcScanEffectData *
        ]
    ),
    "cGcBaseBuildingEffectsManager::RemoveScanEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEffectsManager *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcBaseBuildingEffectsManager::PlayBuildEffectOnNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEffectsManager *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcBaseBuildingEffectsManager::RemoveLightsTransparentDecalsAndShadowBoxNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcBaseBuildingEffectsManager::DisableLODs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcBaseBuildingEffectsManager::FinishBuildEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEffectsManager *
            ctypes.c_ulonglong,  # cGcBaseBuildingEffectsManager::sBuildEffect *
        ]
    ),
    "cGcBaseBuildingEffectsManager::CloneNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # TkHandle
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcBaseBuildingGenerator::GenerateDungeon": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::Room *
        ]
    ),
    "cGcBaseBuildingGenerator::PollToLoadTemplatesTable": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingGenerator::PrepareGrid": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::GeneratedCell *
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
        ]
    ),
    "cGcBaseBuildingGenerator::ShuffleRoomSequence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::Room *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcBaseBuildingGenerator::GenerateDungeonQuestItemPlacement": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::Room *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcBaseBuildingGenerator::CheckDungeonQuestItemRules": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::Room *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcBaseBuildingGenerator::TryGenerateSnake": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::GeneratedCell *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::Room *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcBaseBuildingGenerator::TryAddBranchesToSnake": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::GeneratedCell *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::Room *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcBaseBuildingGenerator::GenerateRoomSizes": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator *
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::GeneratedCell *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::Room *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcBaseBuildingGenerator::GenerateRoomLayouts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::GeneratedCell *
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::Room *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::CellVec *
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::CellVec *
        ]
    ),
    "cGcBaseBuildingGenerator::CheckRoomRules": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator *
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::GeneratedCell *
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::Room *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcBaseBuildingGenerator::AddRoomsToSnake": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::GeneratedCell *
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::Room *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcBaseBuildingGenerator::TryAddQuestItemSpawn": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator *
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::GeneratedCell *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::Room *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const cGcBaseBuildingGenerator::CellVec *
            ctypes.c_ulonglong,  # const cGcBaseBuildingGenerator::CellVec *
            ctypes.c_ulonglong,  # std::vector<cGcBaseBuildingGenerator::sDecorationSpawn,TkSTLAllocatorShim<cGcBaseBuildingGenerator::sDecorationSpawn,4,-1> > *
        ]
    ),
    "cGcBaseBuildingGenerator::AddDecorationSpawns": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator *
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::GeneratedCell *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::Room *
            ctypes.c_uint8,  # const unsigned __int8
            ctypes.c_ulonglong,  # std::vector<cGcBaseBuildingGenerator::sDecorationSpawn,TkSTLAllocatorShim<cGcBaseBuildingGenerator::sDecorationSpawn,4,-1> > *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cGcBaseBuildingGenerator::CellVec *
            ctypes.c_ulonglong,  # const cGcBaseBuildingGenerator::CellVec *
        ]
    ),
    "cGcBaseBuildingGenerator::PatternMatch": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator *
            ctypes.c_ulonglong,  # const cGcBaseBuildingGenerator::CellVec *
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::GeneratedCell *
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # const cGcBaseBuildingGenerator::sPattern *
            ctypes.c_ulonglong,  # const cGcBaseBuildingGenerator::CellVec *
            ctypes.c_uint8,  # const unsigned __int8
            ctypes.c_ulonglong,  # const cGcBaseBuildingGenerator::CellVec *
            ctypes.c_ulonglong,  # const cGcBaseBuildingGenerator::CellVec *
        ]
    ),
    "cGcBaseBuildingGenerator::CacheRoomDecorations": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # eDecorationLayer
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcBaseBuildingGenerator::CacheTemplatesForId": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # eDecorationLayer
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcBaseBuildingGenerator::IncrementCoord": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::CellVec *
            ctypes.c_uint32,  # cGcBaseBuildingGenerator::Direction
            ctypes.c_ulonglong,  # const cGcBaseBuildingGenerator::CellVec *
        ]
    ),
    "cGcBaseBuildingGenerator::CheckNeighbouringCells": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::GeneratedCell *
            ctypes.c_ulonglong,  # const cGcBaseBuildingGenerator::CellVec *
            ctypes.c_uint32,  # cGcBaseBuildingGenerator::Direction
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
        ]
    ),
    "cGcBaseBuildingGenerator::GetLocalMatrixForCoord": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # const cGcBaseBuildingGenerator::CellVec *
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcBaseBuildingGenerator::SpawnDecorations": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator *
            ctypes.c_ulonglong,  # std::vector<cGcBaseBuildingGenerator::sDecorationSpawn,TkSTLAllocatorShim<cGcBaseBuildingGenerator::sDecorationSpawn,4,-1> > *
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # std::vector<cGcBaseBuildingGenerator::sPruningOption,TkSTLAllocatorShim<cGcBaseBuildingGenerator::sPruningOption,8,-1> > *
        ]
    ),
    "cGcBaseBuildingGenerator::PruneDecorations": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # std::vector<cGcBaseBuildingGenerator::sPruningOption,TkSTLAllocatorShim<cGcBaseBuildingGenerator::sPruningOption,8,-1> > *
        ]
    ),
    "cGcBaseBuildingGenerator::MarkDecorationSpawnOnGrid": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator *
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::GeneratedCell *
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # cGcBaseBuildingGenerator::CellVec *
            ctypes.c_ulonglong,  # const cGcBaseBuildingGenerator::sDecorationTemplate *
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcInteractionComponent>::UpdateComponentsRender": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcInteractionComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcNPCComponent>::UpdateComponentsRender": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcNPCComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcNPCComponent>::UpdateComponentsPostPhysics": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcNPCComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcAlienPuzzleOption>::~cTkDynamicArray<cGcAlienPuzzleOption>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAlienPuzzleOption> *
        ]
    ),
    "nanoflann::KDTreeSingleIndexAdaptor<nanoflann::L2_Simple_Adaptor<float,cTkVector3MetaKD<float,-1>,float>,cTkVector3MetaKD<float,-1>,3,unsigned __int64,-1>::divideTree": FUNCDEF(
        restype=ctypes.c_ulonglong,  # nanoflann::KDTreeSingleIndexAdaptor<nanoflann::L2_Simple_Adaptor<float,cTkVector3MetaKD<unsigned __int64,23>,float>,cTkVector3MetaKD<unsigned __int64,23>,3,unsigned __int64,23>::Node *
        argtypes=[
            ctypes.c_ulonglong,  # nanoflann::KDTreeSingleIndexAdaptor<nanoflann::L2_Simple_Adaptor<float,cTkVector3MetaKD<unsigned __int64,23>,float>,cTkVector3MetaKD<unsigned __int64,23>,3,unsigned __int64,23> *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # std::array<nanoflann::KDTreeSingleIndexAdaptor<nanoflann::L2_Simple_Adaptor<float,cTkVector3MetaKD<unsigned __int64,23>,float>,cTkVector3MetaKD<unsigned __int64,23>,3,unsigned __int64,23>::Interval,3> *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcGeneratedBaseTemplatesTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGeneratedBaseTemplatesTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSimpleAnimationQueue::SimpleAnimNode::~SimpleAnimNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleAnimationQueue::SimpleAnimNode *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "StackContainer<std::vector<cGcFiendBlackboard::cGcThreatInfo,StackAllocator<cGcFiendBlackboard::cGcThreatInfo,8,-1>>,8,-1>::~StackContainer<std::vector<cGcFiendBlackboard::cGcThreatInfo,StackAllocator<cGcFiendBlackboard::cGcThreatInfo,8,-1>>,8,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cEgRenderActionQueue *,StackAllocator<cEgRenderActionQueue *,16,-1> >,16,-1> *
        ]
    ),
    "nanoflann::KNNResultSet<float,unsigned __int64,unsigned __int64>::addPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nanoflann::KNNResultSet<float,unsigned __int64,unsigned __int64> *
            ctypes.c_float,  # float
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "nanoflann::PooledAllocator::doAllocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # nanoflann::PooledAllocator *
            ctypes.c_uint64,  # const unsigned __int64
        ]
    ),
    "nanoflann::SearchParams::SearchParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nanoflann::SearchParams *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "nanoflann::KNNResultSet<float,unsigned __int64,unsigned __int64>::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nanoflann::KNNResultSet<float,unsigned __int64,unsigned __int64> *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "nanoflann::KNNResultSet<float,unsigned __int64,unsigned __int64>::KNNResultSet<float,unsigned __int64,unsigned __int64>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nanoflann::KNNResultSet<float,unsigned __int64,unsigned __int64> *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkSpinMutex::Acquire": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSpinMutex *
        ]
    ),
    "cTkLinearPoolJanitor::cTkLinearPoolJanitor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearPoolJanitor *
            ctypes.c_ulonglong,  # cTkLinearMemoryPool *
        ]
    ),
    "cTkComponentManager::PushComponentMask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentManager *
            ctypes.c_uint32,  # eTkComponentMask
        ]
    ),
    "cTkVector3MetaKD<float,-1>::reserve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3MetaKD<float,-1> *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingBaseLayout::cGcBaseBuildingBaseLayout": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingBaseLayout *
        ]
    ),
    "cTkBitArray<unsigned __int64,1,128>::SetElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,128> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBitArray<unsigned __int64,1,128>::SetElementTrue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBuildingList::FindNearest": {
        "cGcBuildingList *, const cTkVector3 *, eBuildingClass, float *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcBuilding *
            argtypes=[
                ctypes.c_ulonglong,  # cGcBuildingList *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_uint32,  # eBuildingClass
                ctypes.c_ulonglong,  # float *
            ]
        ),
        "cGcBuildingList *, const cTkVector3 *, float *, bool": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcBuilding *
            argtypes=[
                ctypes.c_ulonglong,  # cGcBuildingList *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcBuildingList *, const cTkVector3 *, cTkBitArray<unsigned __int64,1,128> *, float *, bool": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcBuilding *
            argtypes=[
                ctypes.c_ulonglong,  # cGcBuildingList *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,128> *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "GcPersistencyHandle::IsValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # GcPersistencyHandle *
        ]
    ),
    "VecIntrinsics::Splatter<0>::operator<<0>": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # VecIntrinsics::Splatter<0> *
            ctypes.c_ulonglong,  # const VecIntrinsics::Splatter<0> *
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::IsValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::GetCurrentID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::HasTwinPosition": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
        ]
    ),
    "cGcBaseBuildingManager::cGcBaseBuildingManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::cGcBaseBuildingPlayerPlacement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
        ]
    ),
    "cGcBaseBuildingManager::SetNodeColour": {
        "cGcBaseBuildingManager *, TkHandle, unsigned __int8": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcBaseBuildingManager *
                ctypes.c_uint32,  # TkHandle
                ctypes.c_uint8,  # unsigned __int8
            ]
        ),
        "TkHandle, const cTkColour *, const cTkColour *, const cTkColour *, const cTkColour *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_uint32,  # TkHandle
                ctypes.c_ulonglong,  # const cTkColour *
                ctypes.c_ulonglong,  # const cTkColour *
                ctypes.c_ulonglong,  # const cTkColour *
                ctypes.c_ulonglong,  # const cTkColour *
            ]
        ),
    },
    "cGcBaseBuildingManager::IsBaseSpawned": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # unsigned __int16 *
        ]
    ),
    "BaseBuildingTemplateCache::~BaseBuildingTemplateCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # BaseBuildingTemplateCache *
        ]
    ),
    "cGcProjectorEffect::~cGcProjectorEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectorEffect *
        ]
    ),
    "cGcBaseBuildingNavigationManager::~cGcBaseBuildingNavigationManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingNavigationManager *
        ]
    ),
    "cGcBaseBuildingEffectsManager::~cGcBaseBuildingEffectsManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEffectsManager *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcBaseBuildingEffectsManager::sBuildEffect,16,-1> *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::FeaturedBaseManagerListData::~FeaturedBaseManagerListData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager::FeaturedBaseManagerListData *
        ]
    ),
    "cGcBaseBuildingDownloadsManager::~cGcBaseBuildingDownloadsManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingDownloadsManager *
        ]
    ),
    "cGcExpeditionHologramComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionHologramComponent *
        ]
    ),
    "cGcExpeditionHologramComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionHologramComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcExpeditionHologramComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcExpeditionHologramComponent *
        ]
    ),
    "ComponentInstancercGcExpeditionHologramComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentLoadInstancercGcExpeditionHologramComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentLoadInstancercGcExpeditionHologramComponent *
        ]
    ),
    "cGcInteractionComponent::GetMessageBeacon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::shared_ptr<cGcMsgBeacon> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # std::shared_ptr<cGcMsgBeacon> *
        ]
    ),
    "cGcInteractionComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInteractionComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSimpleInteractionComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
        ]
    ),
    "cGcSimpleInteractionComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcSimpleInteractionComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcSimpleInteractionComponent *
        ]
    ),
    "ComponentInstancercGcSimpleInteractionComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcSimpleInteractionComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcSimpleInteractionComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcRefinerUnitComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::RefreshPersistency": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::ItemAmountAccumulatorActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFeederComponent *
        ]
    ),
    "cGcRefinerUnitComponent::IsCooker": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcRefinerUnitComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRefinerUnitComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "ComponentInstancercGcRefinerUnitComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcRefinerUnitComponent *
        ]
    ),
    "ComponentInstancercGcRefinerUnitComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcRefinerUnitComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcRefinerUnitComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcObjectSpawnerComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcObjectSpawnerComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectSpawnerComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnerComponent *
        ]
    ),
    "cGcObjectSpawnerComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnerComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcObjectSpawnerComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcObjectSpawnerComponent *
        ]
    ),
    "ComponentInstancercGcSquadronHologramComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcObjectSpawnerComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcVehicleRaceInviteComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcBaseDefenceComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceComponent *
        ]
    ),
    "cGcBaseDefenceComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcBaseDefenceComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcBaseDefenceComponent *
        ]
    ),
    "ComponentInstancercGcBaseDefenceComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcBaseDefenceComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcBaseDefenceComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcByteBeatSwitchComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSwitchComponent *
        ]
    ),
    "cGcByteBeatSwitchComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSwitchComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcByteBeatSwitchComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcByteBeatSwitchComponent *
        ]
    ),
    "ComponentInstancercGcNPCHabitationComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcByteBeatSwitchComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcCustomiserCharacterComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcNPCHabitationComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcNPCHabitationComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCHabitationComponent *
        ]
    ),
    "cGcNPCHabitationComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCHabitationComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCHabitationComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCHabitationComponent *
        ]
    ),
    "ComponentInstancercGcNPCHabitationComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcNPCHabitationComponent *
        ]
    ),
    "ComponentInstancercGcNPCHabitationComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentLoadInstancercGcNPCHabitationComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentLoadInstancercGcNPCHabitationComponent *
        ]
    ),
    "cGcOutpostDockNode::GetDockNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostDockNode *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcOutpostComponent::FindEmptyDockPointLocation": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcOutpostComponent::GetTypedComponent": {
        "cTkAttachment *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
        "int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcOutpostComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcOutpostComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcQuickMenuState::IsPointedAt": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenuState *
        ]
    ),
    "cGcQuickMenu::IsPointedAt": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenu *
        ]
    ),
    "cGcQuickMenu::GetActiveState": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcQuickActionMenu *
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenu *
        ]
    ),
    "cGcHologramComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcHologramComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHologramComponent *
        ]
    ),
    "cGcHologramComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHologramComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHologramComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeightAdjustComponent *
        ]
    ),
    "ComponentInstancercGcHologramComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcHologramComponent *
        ]
    ),
    "ComponentInstancercGcHologramComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcHologramComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcHologramComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentLoadInstancercGcHologramComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentLoadInstancercGcHologramComponent *
        ]
    ),
    "cGcVehicleGarageComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcVehicleGarageComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleGarageComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleGarageComponent *
        ]
    ),
    "cGcVehicleGarageComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleGarageComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcVehicleGarageComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcVehicleGarageComponent *
        ]
    ),
    "ComponentInstancercGcVehicleGarageComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcFreighterBaseComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponent *
        ]
    ),
    "cGcFreighterBaseComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFreighterBaseComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponent *
        ]
    ),
    "ComponentInstancercGcFreighterBaseComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcFreighterBaseComponent *
        ]
    ),
    "ComponentInstancercGcFreighterBaseComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcFreighterBaseComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcFreighterBaseComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcFreighterBaseNPCManager::~cGcFreighterBaseNPCManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseNPCManager *
        ]
    ),
    "cGcAbandonedFreighterComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
        ]
    ),
    "cGcAbandonedFreighterComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAbandonedFreighterComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
        ]
    ),
    "ComponentInstancercGcAbandonedFreighterComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcAbandonedFreighterComponent *
        ]
    ),
    "ComponentInstancercGcAbandonedFreighterComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcAbandonedFreighterComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcAbandonedFreighterComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcColouriseComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcColouriseComponent *
        ]
    ),
    "cGcColouriseComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcColouriseComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcColouriseComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcColouriseComponent *
        ]
    ),
    "ComponentInstancercGcColouriseComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcBaseBuildingManager::~cGcBaseBuildingManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingManager::PollToPrepareTemplates": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingManager::UpdatePreComponents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcBaseBuildingManager::UpdateProtectedLocations": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::UpdateBaseBuildingNodeOffset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::UpdateSelectionMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBaseBuildingManager::UpdatePlacementMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_float,  # float
        ]
    ),
    "StackContainer<std::vector<GhostOption,StackAllocator<GhostOption,64,-1>>,64,-1>::~StackContainer<std::vector<GhostOption,StackAllocator<GhostOption,64,-1>>,64,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cGcSpaceMapAveragedObject,StackAllocator<cGcSpaceMapAveragedObject,128,-1> >,128,-1> *
        ]
    ),
    "cGcBaseBuildingManager::GetCubeGroupFromPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcBaseBuildingManager::GetSnappingOptionsForPlaceholder": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkStackVector<GhostOption,64,-1> *
        ]
    ),
    "cGcBaseBuildingManager::CheckGhostOptionValidAndScore": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            common.Quaternion,  # cTkQuaternion
            ctypes.c_ulonglong,  # const BaseBuildingObjectTemplate *
            ctypes.c_ulonglong,  # GhostOption *
        ]
    ),
    "cGcBaseBuildingManager::AddUniqueGhostOption": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GhostOption *
            ctypes.c_ulonglong,  # cTkStackVector<GhostOption,64,-1> *
        ]
    ),
    "cGcBaseBuildingManager::GetDistanceToSnappingCone": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcBaseBuildingManager::GetTargetOrientation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkQuaternion *
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuaternion *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcBaseBuildingManager::GetPlacementContactForTemplate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkContactPoint *
        argtypes=[
            ctypes.c_ulonglong,  # cTkContactPoint *
            ctypes.c_ulonglong,  # const BaseBuildingObjectTemplate *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int16,  # __int16
        ]
    ),
    "cGcBaseBuildingManager::SetSurfacePlacementForPlaceholder": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_ulonglong,  # const cTkContactPoint *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcBaseBuildingManager::CheckBaseObjectCollision": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # const cGcBaseBuildingObject *
        ]
    ),
    "StackContainer<std::vector<cTkRigidBody *,StackAllocator<cTkRigidBody *,128,-1>>,128,-1>::~StackContainer<std::vector<cTkRigidBody *,StackAllocator<cTkRigidBody *,128,-1>>,128,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cTkRigidBody *,StackAllocator<cTkRigidBody *,128,-1> >,128,-1> *
        ]
    ),
    "cGcBaseBuildingManager::SetBestSnappingOptionsOnPlaceholder": FUNCDEF(
        restype=ctypes.c_ulonglong,  # BaseBuildingObjectTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkStackVector<GhostOption,64,-1> *
        ]
    ),
    "cGcBaseBuildingManager::CheckSnappingPlacementRules": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_ulonglong,  # GhostOption *
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
        ]
    ),
    "cGcBaseBuildingManager::CheckSurfacePlacementRules": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_ulonglong,  # const cTkContactPoint *
        ]
    ),
    "cGcBaseBuildingManager::CheckPlacementAllowed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
        ]
    ),
    "cGcBaseBuildingManager::BroadphaseProbeTemplateCollision": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const BaseBuildingObjectTemplate *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # cTkStackVector<cTkRigidBody *,128,-1> *
        ]
    ),
    "cGcBaseBuildingManager::UpdatePlaceholderParts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::CanBuild": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::CanGenerallyBuildObject": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntry *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcBaseBuildingManager::UpdateModeChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBaseBuildingManager::UpdateActions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBaseBuildingManager::UpdateAutoPower": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::BroadcastPlaceholder": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBaseBuildingManager::UpdateNodeRemoval": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::SetMagicModelsBudgets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcBaseBuildingObject,16,-1> *
        ]
    ),
    "cGcBaseBuildingManager::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::BuildObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBaseBuildingObject *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # BuildingUserData *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBaseBuildingManager::BuildRemoteObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBaseBuildingObject *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # sBaseObjectBufferData *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBaseBuildingManager::BuildWFCObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcBaseBuildingObject *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # BuildingUserData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingManager::RemovePlaceholderNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcBaseBuildingManager::AddNodesFromRegion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcBaseBuildingManager::AddObjectMarker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint64,  # sBaseObjectHandle
        ]
    ),
    "cGcBaseBuildingManager::AddHUDMarker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_uint32,  # cGcMarkerPoint::eType
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcBaseBuildingManager::RemoveHUDMarker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_uint32,  # cGcMarkerPoint::eType
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cGcBaseBuildingManager::AddPlayerBaseNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingManager::ForceAddPlayerBaseNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint64,  # sBaseObjectHandle
        ]
    ),
    "cGcBaseBuildingManager::RefreshPersistentBufferObject": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_uint64,  # sBaseObjectHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingManager::RemoveObjectsByPersistentIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # _WORD *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBaseBuildingManager::ClearPlayerBaseObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # _WORD *
        ]
    ),
    "cGcBaseBuildingManager::ClearBaseCubeNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # unsigned __int16 *
        ]
    ),
    "cGcBaseBuildingManager::DeleteBuildingObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_uint64,  # sBaseObjectHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingManager::RemoveNodesFromRegion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcBaseBuildingManager::DeleteBaseBuildingObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_ulonglong,  # std::vector<cGcBaseBuildingObject *,TkSTLAllocatorShim<cGcBaseBuildingObject *,8,-1> > *
        ]
    ),
    "cGcBaseBuildingManager::RemoveInteriorDecoration": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # const cTkAABB *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # _WORD *
            ctypes.c_ulonglong,  # std::vector<cGcBaseBuildingObject *,TkSTLAllocatorShim<cGcBaseBuildingObject *,8,-1> > *
        ]
    ),
    "cGcBaseBuildingManager::CreateObjectTemplate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::_List_node<std::pair<TkID<128> const ,BaseBuildingObjectTemplate>,void *> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # cGcBaseBuildingEntry *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "StackContainer<std::vector<cTkVector4,StackAllocator<cTkVector4,8,-1>>,8,-1>::~StackContainer<std::vector<cTkVector4,StackAllocator<cTkVector4,8,-1>>,8,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cTkVector4,StackAllocator<cTkVector4,8,-1> >,8,-1> *
        ]
    ),
    "cGcBaseBuildingManager::CreateSnapPointTemplate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # BaseBuildingObjectTemplate *
            ctypes.c_ulonglong,  # robin_hood::detail::Table<0,80,TkHandle,SnapHandle,robin_hood::hash<TkHandle,void>,std::equal_to<TkHandle>,-1> *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcBaseBuildingManager::EditTerrainAroundBuilding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # const cTkAABB *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_uint32,  # eBaseTerrainEditShape
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingManager::LoadModel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # cTkModelResource *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
        ]
    ),
    "cGcBaseBuildingManager::LoadTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # cTkTextureResource *
        ]
    ),
    "cGcBaseBuildingManager::GetNearestPlayersDistanceToObject": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingObject *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcBaseBuildingManager::IsShipInsideObject": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingObject *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingManager::CheckObjectIsIllegal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # const cGcBaseBuildingObject *
        ]
    ),
    "cGcBaseBuildingManager::CheckPositionIsProtected": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcBaseBuildingManager::CheckPriviledgeForObject": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint64,  # sBaseObjectHandle
            ctypes.c_uint32,  # ePrivilegeType
        ]
    ),
    "cGcBaseBuildingManager::CheckPriviledgeForBase": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_uint32,  # ePrivilegeType
        ]
    ),
    "cGcBaseBuildingManager::GetBasePartAudioLocation": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcBaseBuildingManager::ClaimBase": FUNCDEF(
        restype=ctypes.c_ulonglong,  # BaseIndex *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # BaseIndex *
            ctypes.c_uint32,  # ePersistentBaseTypes
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint64,  # sBaseObjectHandle
        ]
    ),
    "cGcBaseBuildingManager::GetBaseRootNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_ulonglong,  # _WORD *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingManager::ClearBaseRootNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # unsigned __int16 *
        ]
    ),
    "cGcBaseBuildingManager::MoveBaseMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # _WORD *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cGcBaseBuildingManager::AreResourcesAwaitingLoad": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::DoPlace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_uint32,  # cGcBaseBuildingManager::eShowEffects
        ]
    ),
    "cGcBaseBuildingManager::DoReplace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::DoDelete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::CheckCanDeleteBuildingObject": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # const cGcBaseBuildingObject *
        ]
    ),
    "cGcBaseBuildingManager::DeleteBuildingNode": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # cGcBaseBuildingManager::eDoCollectEffects
        ]
    ),
    "cGcBaseBuildingManager::CanPickUp": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcBaseBuildingManager::PlayBuiltEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # const cGcBaseBuildingObject *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBaseBuildingManager::GetTerrainEditsForBuilding": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # const cTkAABB *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_uint32,  # eBaseTerrainEditShape
            ctypes.c_ulonglong,  # sRegionEdit *
        ]
    ),
    "cGcBaseBuildingManager::DoSetColour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::DoSetMaterial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::DoRotateSnapping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingManager::DoDeleteBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::GetBaseObjectLocalMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcBaseBuildingManager::GetWorldMatrixFromBufferData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # sBaseObjectBufferData *
            ctypes.c_ulonglong,  # _WORD *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcBaseBuildingManager::GetBaseBuildingRootMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingManager::AddPersistentObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sBaseObjectHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # sBaseObjectHandle *
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_ulonglong,  # sBaseObjectBufferData *
            ctypes.c_int32,  # int
            ctypes.c_int16,  # __int16
        ]
    ),
    "cGcBaseBuildingManager::DeletePersistentObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_uint64,  # sBaseObjectHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingManager::GetMatrixScale": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcBaseBuildingManager::AddBaseTeleporterLocation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # unsigned __int16 *
        ]
    ),
    "cGcBaseBuildingManager::FindBaseTeleporterLocator": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # _WORD *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcBaseBuildingManager::IsPartCore": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcBaseBuildingManager::GetDefaultPaletteIndexForPart": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcBaseBuildingManager::GetDefaultMaterialIndexForPart": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcBaseBuildingManager::SetNodeMaterial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcBaseBuildingManager::GetMaxAllowableParts": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cGcBaseBuildingManager::FindEntryByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBaseBuildingEntry *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcBaseBuildingManager::FindTemplateByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # BaseBuildingObjectTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcBaseBuildingManager::UpdateFreighterBoundingBox": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcBaseBuildingManager::IsPlayerOverlappingPlayerOwnedBase": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingManager::GetBaseTypeFromIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int16 *
        ]
    ),
    "cGcBaseBuildingManager::FindRegionId": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingManager::GetRegionIDForObject": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_uint64,  # sBaseObjectHandle
        ]
    ),
    "cGcBaseBuildingManager::PrepareBasePart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingManager::ColourBasePart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingManager::FindBaseBuildingObject": {
        "cGcBaseBuildingManager *, TkHandle": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            argtypes=[
                ctypes.c_ulonglong,  # cGcBaseBuildingManager *
                ctypes.c_uint32,  # TkHandle
            ]
        ),
        "cGcBaseBuildingManager *, sBaseObjectHandle": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            argtypes=[
                ctypes.c_ulonglong,  # cGcBaseBuildingManager *
                ctypes.c_uint64,  # sBaseObjectHandle
            ]
        ),
    },
    "cGcBaseBuildingManager::AddBaseBuildingObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBaseBuildingObject *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_uint64,  # sBaseObjectHandle
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingManager::AddTemporaryBaseBuildingObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBaseBuildingObject *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # sBaseBuildingMaterial *
        ]
    ),
    "cGcBaseBuildingManager::AddGhostObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_ulonglong,  # sBaseBuildingMaterial *
            ctypes.c_int16,  # __int16
        ]
    ),
    "cGcBaseBuildingManager::GeneratePartData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # const cGcBaseBuildingPart *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcBaseBuildingPartData *
        ]
    ),
    "cGcBaseBuildingManager::RequestDeferredRemovalOfPlayerBaseObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cGcBaseBuildingManager::sRemoveBase,8,-1> *
        ]
    ),
    "cGcBaseBuildingManager::IsPositionOverlappingBase": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcPersistentBase>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPersistentBase *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcPersistentBase> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcPersistentBase>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cGcPersistentBase> *
        argtypes=[]
    ),
    "cTkTemplatedComponentUpdater<cGcVehicleGarageComponent>::UpdateComponentsRender": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcVehicleGarageComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcObjectSpawnerComponent>::UpdateComponentsRender": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcObjectSpawnerComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkLinearHashTable<cTkFixedString<256,char>,std::vector<cTkMetadataCache<cGcPersistentBase>::cDataRequest,TkSTLAllocatorShim<cTkMetadataCache<cGcPersistentBase>::cDataRequest,8,-1>>,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char>>>::~cTkLinearHashTable<cTkFixedString<256,char>,std::vector<cTkMetadataCache<cGcPersistentBase>::cDataRequest,TkSTLAllocatorShim<cTkMetadataCache<cGcPersistentBase>::cDataRequest,8,-1>>,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<cTkFixedString<256,char>,std::vector<cTkMetadataCache<cGcWeatherTable>::cDataRequest,TkSTLAllocatorShim<cTkMetadataCache<cGcWeatherTable>::cDataRequest,8,-1> >,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char> > > *
        ]
    ),
    "cGcBaseBuildingEntry::~cGcBaseBuildingEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEntry *
        ]
    ),
    "cTkLinearHashTable<cTkFixedString<256,char>,cTkMetadataCache<cGcWeatherEffectTable>::cTkMetadataLookup,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char>>>::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<cTkFixedString<256,char>,cTkMetadataCache<cGcWeatherTable>::cTkMetadataLookup,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char> > > *
        ]
    ),
    "cTkLinearHashTable<cTkFixedString<256,char>,cTkMetadataCache<cGcCustomisationBannerGroup>::cTkMetadataLookup,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char>>>::ChangeNodeCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<cTkFixedString<256,char>,cTkMetadataCache<cGcWeatherTable>::cTkMetadataLookup,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char> > > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcBaseBuildingTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBaseBuildingTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcBaseBuildingPartsTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBaseBuildingPartsTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcBaseBuildingPartsDataTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBaseBuildingPartsDataTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcBaseBuildingPartAudioLocationTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBaseBuildingPartAudioLocationTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcPersistentBase>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPersistentBase *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProjectileData::cGcProjectileData": {
        "cGcProjectileData *, const cGcProjectileData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcProjectileData *
                ctypes.c_ulonglong,  # const cGcProjectileData *
            ]
        ),
        "cGcProjectileData *, cGcProjectileData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcProjectileData *
                ctypes.c_ulonglong,  # cGcProjectileData *
            ]
        ),
    },
    "cTkDynamicArray<cGcProjectileImpactData>::cTkDynamicArray<cGcProjectileImpactData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcProjectileImpactData> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcProjectileImpactData> *
        ]
    ),
    "cTkDynamicArray<cGcProjectileImpactData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcProjectileImpactData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcImpactCombatEffectData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcImpactCombatEffectData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcImpactCombatEffectData>::cTkDynamicArray<cGcImpactCombatEffectData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcImpactCombatEffectData> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcImpactCombatEffectData> *
        ]
    ),
    "cTkDynamicArray<cTkProceduralTextureChosenOptionSampler>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkProceduralTextureChosenOptionSampler> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkProceduralTextureChosenOptionSampler>::AllocateAndCopyFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkProceduralTextureChosenOptionSampler> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkProceduralTextureChosenOptionSampler> *
        ]
    ),
    "cTkProceduralTextureChosenOptionList::cTkProceduralTextureChosenOptionList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOptionList *
            ctypes.c_ulonglong,  # const cTkProceduralTextureChosenOptionList *
        ]
    ),
    "cGcResourceElement::cGcResourceElement": {
        "cGcResourceElement *, const cGcResourceElement *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcResourceElement *
                ctypes.c_ulonglong,  # const cGcResourceElement *
            ]
        ),
        "cGcResourceElement *, cGcResourceElement *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcResourceElement *
                ctypes.c_ulonglong,  # cGcResourceElement *
            ]
        ),
    },
    "cGcNetworkMessage::cGcNetworkMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkMessage *
            ctypes.c_uint32,  # eGcNetworkMessageType
            ctypes.c_uint32,  # cGcNetworkConstants::TransmissionChannels
        ]
    ),
    "cGcBaseLinkGridConnectionDependency::~cGcBaseLinkGridConnectionDependency": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridConnectionDependency *
        ]
    ),
    "cTkDynamicArray<cGcBaseLinkGridConnectionDependency>::~cTkDynamicArray<cGcBaseLinkGridConnectionDependency>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseLinkGridConnectionDependency> *
        ]
    ),
    "cGcBaseLinkGridData::~cGcBaseLinkGridData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridData *
        ]
    ),
    "cTkLinearHashTable<void const *,cTkFixedString<256,char>,-1,0,cTkLinearHashTableHash<void const *>>::ChangeNodeCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<void const *,cTkFixedString<256,char>,-1,0,cTkLinearHashTableHash<void const *> > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLinearHashTable<void const *,cTkFixedString<256,char>,-1,0,cTkLinearHashTableHash<void const *>>::insert": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<void const *,cTkFixedString<256,char>,-1,0,cTkLinearHashTableHash<void const *> > *
            ctypes.c_ulonglong,  # const void *const *
            ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
        ]
    ),
    "cTkAttachment::PrepareComponents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkAttachment::PrepareNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcTerrainRegionMap::IsRegionPointerTemporary": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRegionTerrain *
        ]
    ),
    "cTkSphere::GetCubeUp": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkSphere::ProjectSphereToCube": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcProjectileManager::SpawnResourceBlobs": {
        "cGcProjectileManager *, const cTkMatrix34 *, int, const TkID<128> *, eResourceBlobSpawnStyle, bool, cTkAttachment *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcProjectileManager *
                ctypes.c_ulonglong,  # const cTkMatrix34 *
                ctypes.c_int32,  # int
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_uint32,  # eResourceBlobSpawnStyle
                ctypes.c_ubyte,  # bool
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
        "cGcProjectileManager *, const cTkPhysRelMat34 *, int, const TkID<128> *, eResourceBlobSpawnStyle, bool, cTkAttachment *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcProjectileManager *
                ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
                ctypes.c_int32,  # int
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_uint32,  # eResourceBlobSpawnStyle
                ctypes.c_ubyte,  # bool
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
    },
    "cGcNetworkLockManager::sKey::sKey": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkLockManager::sKey *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkComponent::SetWantsUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOption<cTkIntTuple2<int,0>>::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<cTkIntTuple2<int,0> > *
        ]
    ),
    "cGcSpaceshipComponent::GetTypedComponent": {
        "int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_int32,  # int
            ]
        ),
        "cTkAttachment *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
    },
    "cGcSpaceshipComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcSolarSystem::GetNearestPlanetWritable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlanet *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkPhysicsComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcScanManager::GetMarkerList": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMarkerList *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanManager *
        ]
    ),
    "BuildingUserData::GetColourIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # BuildingUserData *
        ]
    ),
    "BuildingUserData::BuildingUserData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # BuildingUserData *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "BuildingUserData::GetObjectType": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcSolarSystem::GetNearestPlanet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlanet *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cTkRigidBody::GetCollisionFilterGroup": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkPhysicsComponent::GetNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "sqrt": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "cGcBaseBuildingObject::GetNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cTkRigidBodyPtr::operator->": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRigidBody *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBodyPtr *
        ]
    ),
    "cTkRigidBody::GetNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cTkContactPoint::cTkContactPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkContactPoint *
        ]
    ),
    "cGcGenericSectionConditionSettlementsEnabled::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSettlementsEnabled *
        ]
    ),
    "cTkPhysicsComponent::AllowComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "VecIntrinsics::Splatter<2>::operator<<2>": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # VecIntrinsics::Splatter<2> *
            ctypes.c_ulonglong,  # const VecIntrinsics::Splatter<2> *
        ]
    ),
    "VecIntrinsics::Splatter<1>::operator<<1>": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # VecIntrinsics::Splatter<1> *
            ctypes.c_ulonglong,  # const VecIntrinsics::Splatter<1> *
        ]
    ),
    "VecIntrinsics::Splatter<0>::operator<<1>": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # VecIntrinsics::Splatter<0> *
            ctypes.c_ulonglong,  # const VecIntrinsics::Splatter<1> *
        ]
    ),
    "VecIntrinsics::Splatter<0>::operator<<2>": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # VecIntrinsics::Splatter<0> *
            ctypes.c_ulonglong,  # const VecIntrinsics::Splatter<2> *
        ]
    ),
    "VecIntrinsics::Splatter<1>::operator<<2>": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # VecIntrinsics::Splatter<1> *
            ctypes.c_ulonglong,  # const VecIntrinsics::Splatter<2> *
        ]
    ),
    "cTkDynamicArray<cTkClassPointer>::cTkDynamicArray<cTkClassPointer>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkClassPointer> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkClassPointer> *
        ]
    ),
    "cTkComponentManager::SetComponentMask": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentManager *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingObject::GetID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "StackContainer<std::vector<cGcWFCBuildingState *,StackAllocator<cGcWFCBuildingState *,16,20>>,16,20>::~StackContainer<std::vector<cGcWFCBuildingState *,StackAllocator<cGcWFCBuildingState *,16,20>>,16,20>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cGcWFCBuildingState *,StackAllocator<cGcWFCBuildingState *,16,20> >,16,20> *
        ]
    ),
    "cGcBaseBuildingManager::GetBaseBuildingObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcBaseBuildingObject *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "BaseObjectLimits::UpdateLimits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # BaseObjectLimits *
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntry *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # unsigned __int16 *
        ]
    ),
    "sComponentBaseObjectHandle::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sComponentBaseObjectHandle *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "sBaseObjectBufferData::ReadBufferData": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint64,  # sBaseObjectHandle
            ctypes.c_ulonglong,  # sBaseObjectBufferData *
        ]
    ),
    "sBaseObjectBufferData::EraseBufferData": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint64,  # sBaseObjectHandle
        ]
    ),
    "ModelPlacementRule::CacheConditionsData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ModelPlacementRule *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkFixedString<128,char> *
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkClassPointer> *
        ]
    ),
    "BaseBuildingObjectTemplate::SetOBB": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # BaseBuildingObjectTemplate *
            ctypes.c_ulonglong,  # const cTkAABB *
        ]
    ),
    "cGcNPCInteractiveObjectComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcNPCNavigationAreaComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcNPCComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcBaseBuildingNavigationManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingNavigationManager *
            ctypes.c_ulonglong,  # std::vector<cGcBaseBuildingObject,TkSTLAllocatorShim<cGcBaseBuildingObject,16,-1> > *
        ]
    ),
    "cGcBaseBuildingNavigationManager::sObjectNavData::sObjectNavData": {
        "cGcBaseBuildingNavigationManager::sObjectNavData *, const cGcBaseBuildingNavigationManager::sObjectNavData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcBaseBuildingNavigationManager::sObjectNavData *
                ctypes.c_ulonglong,  # const cGcBaseBuildingNavigationManager::sObjectNavData *
            ]
        ),
        "cGcBaseBuildingNavigationManager::sObjectNavData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcBaseBuildingNavigationManager::sObjectNavData *
            ]
        ),
    },
    "cGcBaseBuildingNavigationManager::GetAllNavBaseObjects": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingNavigationManager *
            ctypes.c_ulonglong,  # std::vector<std::pair<sBaseObjectHandle,TkID<128> >,TkSTLAllocatorShim<std::pair<sBaseObjectHandle,TkID<128> >,8,-1> > *
            ctypes.c_ulonglong,  # _WORD *
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['sBaseObjectHandle const &', 'TkID<128>'], 'pointer_depth': 1}
        ]
    ),
    "cGcBaseBuildingNavigationManager::GetRandomSubgraphNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNavGraphNode *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingNavigationManager *
            ctypes.c_uint64,  # sBaseObjectHandle
            common.GcSeed,  # cTkSeed
            ctypes.c_uint32,  # eNPCNavSubgraphNodeType
        ]
    ),
    "cGcBaseBuildingNavigationManager::CreateNavArea": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingNavigationManager *
            ctypes.c_ulonglong,  # const BaseIndex *
        ]
    ),
    "cGcBaseBuildingNavigationManager::AddObjectSubgraphs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingNavigationManager *
            ctypes.c_uint64,  # sBaseObjectHandle
        ]
    ),
    "cGcBaseBuildingNavigationManager::RemoveObjectSubgraphs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingNavigationManager *
            ctypes.c_uint64,  # sBaseObjectHandle
        ]
    ),
    "cGcBaseBuildingNavigationManager::DestroyNPCsOverlappingObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingNavigationManager *
            ctypes.c_uint64,  # sBaseObjectHandle
        ]
    ),
    "cGcBaseBuildingNavigationManager::InstantiatePartNav": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingNavigationManager *
            ctypes.c_ulonglong,  # cGcBaseBuildingNavigationManager::sPartNavData *
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartNavData *
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cTkStaticPhysicsComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkStaticPhysicsComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingObject::sSnappingState::~sSnappingState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject::sSnappingState *
        ]
    ),
    "cGcBaseBuildingObject::sSnappingState::IsSnappedToObject": {
        "cGcBaseBuildingObject::sSnappingState *, const TkID<128> *, int": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcBaseBuildingObject::sSnappingState *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_int32,  # int
            ]
        ),
        "cGcBaseBuildingObject::sSnappingState *, const TkID<128> *, int, unsigned __int8": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcBaseBuildingObject::sSnappingState *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_int32,  # int
                ctypes.c_uint8,  # unsigned __int8
            ]
        ),
    },
    "cGcBaseBuildingObject::cGcBaseBuildingObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_ulonglong,  # BaseBuildingObjectTemplate *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint64,  # sBaseObjectHandle
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingObject::SetMaterial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_ulonglong,  # sBaseBuildingMaterial *
        ]
    ),
    "cGcBaseBuildingObject::IsSnapped": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
        ]
    ),
    "cGcBaseBuildingObject::ChangeObjectBufferHandle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_uint64,  # sBaseObjectHandle
        ]
    ),
    "cGcBaseBuildingObject::SnapObjects": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
        ]
    ),
    "cGcBaseBuildingObject::UnSnapAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
        ]
    ),
    "cGcBaseBuildingObject::GetNeighbours": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_ulonglong,  # cTkStackVector<TkHandle,16,-1> *
        ]
    ),
    "cGcBaseBuildingObject::UpdateParts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingObject::GetSnappingPlacementMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # const BaseBuildingObjectTemplate *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "cGcBaseBuildingObject::GetAvailableSnappingOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_ulonglong,  # const SnapSocketTemplate *
            ctypes.c_ulonglong,  # cTkStackVector<SnapHandle,32,-1> *
        ]
    ),
    "cGcBaseBuildingObject::IsSnapPointAvailable": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_ulonglong,  # const SnapPointTemplate *
        ]
    ),
    "cGcBaseBuildingObject::TrySnapObjects": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingObject::UnSnapFromNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcBaseBuildingObject::UpdatePart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_ulonglong,  # const ModelPlacementRule *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingObject::CheckConditions": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkClassPointer> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingObject::CheckCondition": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObject *
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcHUDManager::StartFlash": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            common.Colour,  # cTkColour
        ]
    ),
    "cGcBaseBuildingOffenseReporter::RenderScreenshot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "OnConfirmDialogClose": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingOffenseReporter::OnRenderScreenshotFinished": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcPlayerCharacterComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcBaseBuildingPlayerPlacement::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::GetInvalidPositionDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::RotatePlaceholder": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # cGcBaseBuildingPlayerPlacement::RotationAxis
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::UpdatePosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_double,  # double
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::RestorePlaceholderSettings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::IsSnapped": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::SetValidPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eInvalidPlacementReason
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement::sInvalidPlacementMetadata *
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::SetSelectedNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::SetCurrentBaseIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_ulonglong,  # unsigned __int16 *
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::SetColourIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::SetMaterialIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::DuplicateSelectedNode": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
        ]
    ),
    "cGcProjectorEffect::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectorEffect *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntry *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "StackContainer<std::vector<cGcCreatureRoutine::PathCell,StackAllocator<cGcCreatureRoutine::PathCell,16,-1>>,16,-1>::~StackContainer<std::vector<cGcCreatureRoutine::PathCell,StackAllocator<cGcCreatureRoutine::PathCell,16,-1>>,16,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cGcCreatureRoutine::PathCell,StackAllocator<cGcCreatureRoutine::PathCell,16,-1> >,16,-1> *
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::GetBaseIndexForCurrentPart": FUNCDEF(
        restype=ctypes.c_ulonglong,  # BaseIndex *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_ulonglong,  # BaseIndex *
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::SetPreviewMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::SetPlaceholderHeartSize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_double,  # double
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::SetGhostHeartSize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_int32,  # int
            ctypes.c_double,  # double
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::GetMinPlacementDistanceForCurrentObject": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::DoesPartExistAsProductInInventories": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::PreviewMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::UpdateLoadPlaceholder": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::GetScanEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcScanEffectData *
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::SetNodeState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcBaseBuildingPlayerPlacement::AddGhostHeartToObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcLinkGrid::AddExistingObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLinkGrid *
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcRegionHotspots::sHotspot::sHotspot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspots::sHotspot *
        ]
    ),
    "cGcLinkGrid::AddObjectCopy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLinkGrid *
            ctypes.c_ulonglong,  # const cGcLinkGrid::sLinkObjectInstance *
        ]
    ),
    "cGcLinkGrid::UpdateEstimates": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLinkGrid *
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcLinkGrid::Simulate": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLinkGrid *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLinkGrid::AddAmountToObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLinkGrid *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLinkGrid::DeleteObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLinkGrid *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcLinkGrid::GetDayNightEvents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # std::vector<cGcLinkGrid::sEvent,TkSTLAllocatorShim<cGcLinkGrid::sEvent,8,-1> > *
        ]
    ),
    "cGcLinkGrid::GetStormEvents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # std::vector<cGcLinkGrid::sEvent,TkSTLAllocatorShim<cGcLinkGrid::sEvent,8,-1> > *
        ]
    ),
    "cGcLinkGrid::CopyOutRepeatingEvents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # cGcLinkGrid::sEvent *
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # std::vector<cGcLinkGrid::sEvent,TkSTLAllocatorShim<cGcLinkGrid::sEvent,8,-1> > *
        ]
    ),
    "cGcLinkGrid::AssignGroups": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLinkGrid *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcLinkGrid::UpdateObjectState": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLinkGrid *
            ctypes.c_ulonglong,  # cGcLinkGrid::sLinkObjectInstance *
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcLinkGrid::GetNextDueGroupStateChangeTimestep": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLinkGrid *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcLinkGrid::DistributeGroupAmountsAmongstObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLinkGrid *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # std::vector<cGcLinkGrid::sGroupProduceAmount,TkSTLAllocatorShim<cGcLinkGrid::sGroupProduceAmount,4,-1> > *
        ]
    ),
    "cGcLinkGrid::SaveObjectsCurrentStateToBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLinkGrid *
            ctypes.c_ulonglong,  # cGcLinkGrid::sLinkObjectInstance *
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
        ]
    ),
    "cGcBaseLinks::SetNodeVisualState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntry *
            ctypes.c_uint64,  # sBaseObjectHandle
        ]
    ),
    "sLinkObjectSnapshot::sLinkObjectSnapshot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sLinkObjectSnapshot *
        ]
    ),
    "cGcBaseLinks::IsPowerPreventingObjectInteraction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint64,  # sBaseObjectHandle
        ]
    ),
    "cGcBaseLinkMap::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkMap *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<std::vector<cGcSlot *,TkSTLAllocatorShim<cGcSlot *,8,-1> >,8,-1> *
        ]
    ),
    "cGcBaseLinkMap::AddLinkObject": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkMap *
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBaseLinkMap::RemoveLinkObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkMap *
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # _QWORD *
        ]
    ),
    "cGcBaseLinkMap::LinkSockets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkMap *
            ctypes.c_ulonglong,  # const cGcBaseLinkMap::sLinkSocket *
            ctypes.c_ulonglong,  # const cGcBaseLinkMap::sLinkSocket *
        ]
    ),
    "cTkBitArray<unsigned __int64,1,64>::GetElement": FUNCDEF(
        restype=ctypes.c_uint8,  # unsigned __int8
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,64> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcBaseBuildingPartsNavDataTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBaseBuildingPartsNavDataTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingNavigationManager::sPartNavData::sPartNavData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingNavigationManager::sPartNavData *
            ctypes.c_ulonglong,  # const cGcBaseBuildingNavigationManager::sPartNavData *
        ]
    ),
    "cGcBaseLinkMap::sObjectLinkData::~sObjectLinkData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkMap::sObjectLinkData *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<std::vector<cGcSlot *,TkSTLAllocatorShim<cGcSlot *,8,-1> >,8,-1> *
        ]
    ),
    "cTkFixedArray<cTkNoiseUberLayerData,8>::cTkFixedArray<cTkNoiseUberLayerData,8>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkNoiseUberLayerData,8> *
        ]
    ),
    "cTkNoiseUberLayerData::cTkNoiseUberLayerData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseUberLayerData *
        ]
    ),
    "cTkNoiseGridData::cTkNoiseGridData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseGridData *
        ]
    ),
    "cTkFixedArray<cTkNoiseGridData,9>::cTkFixedArray<cTkNoiseGridData,9>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkNoiseGridData,9> *
        ]
    ),
    "cTkFixedArray<cTkNoiseFeatureData,7>::cTkFixedArray<cTkNoiseFeatureData,7>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkNoiseFeatureData,7> *
        ]
    ),
    "cTkNoiseFeatureData::cTkNoiseFeatureData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFeatureData *
        ]
    ),
    "cTkFixedArray<cTkNoiseCaveData,1>::cTkFixedArray<cTkNoiseCaveData,1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseCaveData *
        ]
    ),
    "cGcSolarSystem::GetPlanetGenerator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlanetGenerator *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcInventoryStore::CreateProductElement": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcInventoryElement *
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ulonglong,  # const cGcProductData *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
        ]
    ),
    "cTkInputManager::IsKBMControlActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "cTkInputManager::IsControllerPositionTracking": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "SnapSocketsMask::IsCompatible": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # SnapSocketsMask *
            ctypes.c_ulonglong,  # const SnapSocketsMask *
        ]
    ),
    "cGcBaseSearch::FindFirstBase": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBaseSearch::sBaseSearchResult *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseSearch::sBaseSearchResult *
            ctypes.c_ulonglong,  # const cGcBaseSearchFilter *
            ctypes.c_ulonglong,  # const cGcBaseSearch::sBaseSortCriteria *
        ]
    ),
    "cGcBaseSearch::FindBases": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseSearchFilter *
            ctypes.c_ulonglong,  # std::vector<cGcBaseSearch::sBaseSearchResult,TkSTLAllocatorShim<cGcBaseSearch::sBaseSearchResult,8,-1> > *
            ctypes.c_ulonglong,  # const cGcBaseSearch::sBaseSortCriteria *
        ]
    ),
    "cGcBaseSearch::FindBaseParts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # const cGcBasePartSearchFilter *
            ctypes.c_ulonglong,  # std::vector<cGcBaseSearch::sBasePartSearchResult,TkSTLAllocatorShim<cGcBaseSearch::sBasePartSearchResult,8,-1> > *
            ctypes.c_ulonglong,  # const cGcBaseSearch::sBasePartSortCriteria *
        ]
    ),
    "cGcBaseSearch::FindParts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBasePartSearchFilter *
            ctypes.c_ulonglong,  # const cGcBaseSearchFilter *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # std::vector<cGcBaseSearch::sBasePartSearchResult,TkSTLAllocatorShim<cGcBaseSearch::sBasePartSearchResult,8,-1> > *
            ctypes.c_ulonglong,  # const cGcBaseSearch::sBasePartSortCriteria *
        ]
    ),
    "cGcBaseSearch::FindNearestBaseInCurrentSystem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # BaseIndex *
        argtypes=[
            ctypes.c_ulonglong,  # BaseIndex *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint32,  # ePersistentBaseTypes
        ]
    ),
    "cGcBaseSearchFilter::~cGcBaseSearchFilter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseSearchFilter *
        ]
    ),
    "cGcBaseSearch::FindFreighterBase": FUNCDEF(
        restype=ctypes.c_ulonglong,  # BaseIndex *
        argtypes=[
            ctypes.c_ulonglong,  # BaseIndex *
        ]
    ),
    "cGcBaseSearch::Match": {
        "const cGcBaseSearchFilter *, cGcPlayerBasePersistentBuffer *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # const cGcBaseSearchFilter *
                ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ]
        ),
        "const cGcBasePartSearchFilter *, GcPersistencyHandle, const cGcBaseBuildingEntry *, cGcPlayerBasePersistentBuffer *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # const cGcBasePartSearchFilter *
                ctypes.c_uint32,  # GcPersistencyHandle
                ctypes.c_ulonglong,  # const cGcBaseBuildingEntry *
                ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ]
        ),
        "const cGcBasePartSearchFilter *, const cGcBaseSearchFilter *, const cGcPersistentBBObjectData *, const cGcBaseBuildingEntry *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # const cGcBasePartSearchFilter *
                ctypes.c_ulonglong,  # const cGcBaseSearchFilter *
                ctypes.c_ulonglong,  # const cGcPersistentBBObjectData *
                ctypes.c_ulonglong,  # const cGcBaseBuildingEntry *
            ]
        ),
    },
    "cGcBaseSearch::Score": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseSearch::sBaseSortCriteria *
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
        ]
    ),
    "cGcBaseSearch::GetInterstellarDistanceToBase": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCNavigationAreaComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcNPCComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterBaseNPCManager::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseNPCManager *
        ]
    ),
    "cGcFreighterBaseNPCManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseNPCManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFreighterBaseNPCManager::TryAddSquadronPilotNPC": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseNPCManager *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFreighterBaseNPCManager::TryAddFrigateCaptainNPC": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseNPCManager *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFreighterBaseNPCManager::TryAddWorkerNPC": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseNPCManager *
            ctypes.c_uint32,  # eFreighterNPCType
        ]
    ),
    "cGcFreighterBaseNPCManager::GenerateSpawnMatrix": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseNPCManager *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcSettlementBuildingState::Get": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSettlementBuildingState *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementBuildingState *
            ctypes.c_ulonglong,  # const cGcBuilding *
        ]
    ),
    "cGcSettlementBuildingState::StoreInPlayerState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementBuildingState *
            ctypes.c_ulonglong,  # cGcBuilding *
        ]
    ),
    "cGcSettlementBuildingState::GetNextAdvancement": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementBuildingState *
        ]
    ),
    "cGcSettlementStateManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
        ]
    ),
    "cGcSettlementStateManager::UpdateOwnedSettlementBuildingsClasses": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
        ]
    ),
    "cGcSettlementStateManager::UpdateOwnedSettlementBuildingsConstruction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
        ]
    ),
    "cGcSettlementStateManager::SetOwnedSettlementState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_ulonglong,  # cGcSettlementState *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcSettlementStateManager::TryStoreOwnedSettlementTeleportEndpoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
        ]
    ),
    "cGcSettlementStateManager::RefreshOwnedSettlementNextBuildingUpragde": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
        ]
    ),
    "cGcSettlementStateManager::ClearOwnedSettlement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
        ]
    ),
    "cGcSettlementStateManager::OwnsSettlement": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
        ]
    ),
    "cGcSettlementStateManager::GetOwnedSettlementState": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSettlementState *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
        ]
    ),
    "cGcSettlementStateManager::HasOwner": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_ulonglong,  # const cGcSettlementState *
        ]
    ),
    "cGcSettlementStateManager::IsOwnedByLocalUser": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_ulonglong,  # const cGcSettlementState *
        ]
    ),
    "cGcSettlementStateManager::ClaimSettlement": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_ulonglong,  # cGcSettlementState *
            ctypes.c_ulonglong,  # const cGcSolarSystemData *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcSettlementStateManager::ClearSettlementState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_ulonglong,  # cGcSettlementState *
        ]
    ),
    "cGcSettlementStateManager::ResetOwnedSettlementProduction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_uint32,  # const eAlienRace
        ]
    ),
    "cGcSettlementStateManager::UpdateOwnedSettlementProductionSlotProduct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_uint64,  # const unsigned __int64
        ]
    ),
    "cGcSettlementStateManager::GetSettlementName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
            ctypes.c_ulonglong,  # const cGcSettlementState *
        ]
    ),
    "cGcSettlementStateManager::GetOwnedSettlementHubBuilding": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBuilding *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
        ]
    ),
    "cGcSettlementStateManager::GetHubBuilding": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBuilding *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_ulonglong,  # const cGcSettlementState *
        ]
    ),
    "cGcSettlementStateManager::OwnedSettlementHasAnyBuildingsInProgress": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "cGcSettlementStateManager::GetOwnedSettlementNumUnstartedBuildings": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
        ]
    ),
    "cGcSettlementStateManager::AddPerkToOwnedSettlement": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint64,  # const unsigned __int64
        ]
    ),
    "cGcSettlementStateManager::AddPerk": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_ulonglong,  # cGcSettlementState *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint64,  # const unsigned __int64
        ]
    ),
    "cGcSettlementStateManager::GenerateProcPerkId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint64,  # const unsigned __int64
        ]
    ),
    "cGcSettlementStateManager::RemovePerkFromOwnedSettlement": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcSettlementStateManager::ApplyStatChangeToOwnedSettlement": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_ulonglong,  # const cGcSettlementStatChange *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcSettlementStateManager::GetTotalStatValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_ulonglong,  # cGcSettlementState *
            ctypes.c_uint32,  # const eSettlementStatType
        ]
    ),
    "cGcSettlementStateManager::GetNormalisedStatValue": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_ulonglong,  # cGcSettlementState *
            ctypes.c_uint32,  # const eSettlementStatType
        ]
    ),
    "cGcSettlementStateManager::GetStatLevel": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_ulonglong,  # cGcSettlementState *
            ctypes.c_uint32,  # const eSettlementStatType
        ]
    ),
    "cGcSettlementStateManager::GetOwnedSettlementNextJudgementTypeAndTime": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_ulonglong,  # eSettlementJudgementType *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "cGcSettlementStateManager::UpdateOwnedSettlementJudgementTrigger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
        ]
    ),
    "cGcSettlementStateManager::GetOwnedSettlementCycleProductivityFactor": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
        ]
    ),
    "cGcSettlementStateManager::GetOwnedSettlementCycleItemProductionRate": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcSettlementStateManager::GetOwnedSettlementDebtPaymentRatePerSecond": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
        ]
    ),
    "cGcSettlementStateManager::IsOwnedSettlementOfficeComplete": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
        ]
    ),
    "cGcSettlementStateManager::ReportSettlement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_ulonglong,  # cGcSettlementState *
        ]
    ),
    "cGcSettlementStateManager::UploadSettlement": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementState *
        ]
    ),
    "cGcSettlementStateManager::SendReportSettlement": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementState *
        ]
    ),
    "cGcSettlementStateManager::GetProcPerkSeed": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementStateManager::AddNewStatsMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_uint32,  # const eSettlementStatType
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcSettlementStateManager::GetSettlementBuiltPercentage": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
            ctypes.c_ulonglong,  # const cGcSettlementState *
        ]
    ),
    "cGcSettlementStateManager::UpdateMpSettlementState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStateManager *
        ]
    ),
    "cGcWFCBaseGenerator::CheckWFCBaseClaimed": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcWFCBaseGenerator::CreateWFCBase": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWFCBuildingState *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ulonglong,  # BaseIndex *
        ]
    ),
    "GetLineIntersectsBuilding2D": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWFCBaseGenerator::ClusterNavGraphConstructionData::sBuildingData *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "GetDistance": {
        "const cTkAABB *, const cTkVector3 *": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # const cTkAABB *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
        "const cGcWFCBaseGenerator::ClusterNavGraphConstructionData::sBuildingData *, const cTkVector3 *": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # const cGcWFCBaseGenerator::ClusterNavGraphConstructionData::sBuildingData *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
    },
    "GetOffset": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_ulonglong,  # std::vector<cGcWFCBaseGenerator::ClusterNavGraphConstructionData::sBuildingData,TkSTLAllocatorShim<cGcWFCBaseGenerator::ClusterNavGraphConstructionData::sBuildingData,16,-1> > *
            ctypes.c_int32,  # int
        ]
    ),
    "AddNodesOnALine": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<std::pair<cTkVector3,float>,TkSTLAllocatorShim<std::pair<cTkVector3,float>,16,-1> > *
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_float,  # float
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_ulonglong,  # std::vector<cGcWFCBaseGenerator::ClusterNavGraphConstructionData::sBuildingData,TkSTLAllocatorShim<cGcWFCBaseGenerator::ClusterNavGraphConstructionData::sBuildingData,16,-1> > *
            ctypes.c_int32,  # int
        ]
    ),
    "AddPerimeterNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavGraph *
            ctypes.c_ulonglong,  # std::vector<cGcWFCBaseGenerator::ClusterNavGraphConstructionData::sBuildingData,TkSTLAllocatorShim<cGcWFCBaseGenerator::ClusterNavGraphConstructionData::sBuildingData,16,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "MergeNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavGraph *
            ctypes.c_float,  # float
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_ulonglong,  # std::deque<cGcNavGraphNodeHandle,TkSTLAllocatorShim<cGcNavGraphNodeHandle,2,20> > *
        ]
    ),
    "RemoveRedundantNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavGraph *
        ]
    ),
    "CreateShortcuts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavGraph *
            ctypes.c_ulonglong,  # const std::vector<cGcWFCBaseGenerator::ClusterNavGraphConstructionData::sBuildingData,TkSTLAllocatorShim<cGcWFCBaseGenerator::ClusterNavGraphConstructionData::sBuildingData,16,-1> > *
        ]
    ),
    "ConnectEntranceNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<cGcWFCBaseGenerator::ClusterNavGraphConstructionData::sBuildingData,TkSTLAllocatorShim<cGcWFCBaseGenerator::ClusterNavGraphConstructionData::sBuildingData,16,-1> > *
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
            ctypes.c_ulonglong,  # cGcNPCNavGraph *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcWFCBaseGenerator::ConstructClusterNavigationGraph": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_uint32,  # eRegionStatus
        ]
    ),
    "cGcWFCBaseGenerator::ReconnectWFCBuildingToClusterGraph": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBuildingState *
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
        ]
    ),
    "cGcWFCBaseGenerator::NavigationAreaResHandle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSmartResHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmartResHandle *
        ]
    ),
    "cGcWFCBaseGenerator::ClusterNavGraphConstructionData::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBaseGenerator::ClusterNavGraphConstructionData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWFCBuildingState::OnBuildingUpgraded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBuildingState *
        ]
    ),
    "cGcWFCBuildingState::PopulateBase": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSettlementBuildingState *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBuildingState *
            ctypes.c_ulonglong,  # cGcSettlementBuildingState *
        ]
    ),
    "cGcWFCBuildingState::UpdateNPCs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBuildingState *
            ctypes.c_ulonglong,  # const cGcSettlementState *
            ctypes.c_int32,  # const cGcSettlementBuildingState
        ]
    ),
    "cGcWFCBuildingState::CreateNavigationGraphs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBuildingState *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcWFCBuildingState::GetClusterHubState": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWFCBuildingState *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBuildingState *
        ]
    ),
    "cGcWFCBuildingState::SetBuilding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBuildingState *
            ctypes.c_ulonglong,  # cGcBuilding *
        ]
    ),
    "cGcWFCBuildingState::NotifyElevationChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBuildingState *
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCBuildingState::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWFCBuildingState *
        argtypes=[
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcWFCBuildingState::OnStateChanged": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementState *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcWFCBuildingState::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cGcWFCBuildingState::StartBuildingConstruction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # eBuildingClass
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcWFCBuildingState::StartSpecificBuildingConstruction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuilding *
        ]
    ),
    "cGcWFCBuildingState::AdvanceBuilding": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementState *
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcWFCBuildingState::GetSecondsUntilConstructionEnds": {
        "const cGcBuilding *, unsigned __int64 *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # const cGcBuilding *
                ctypes.c_ulonglong,  # unsigned __int64 *
            ]
        ),
        "const cGcSettlementState *, const int, const eBuildingClass, unsigned __int64 *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # const cGcSettlementState *
                ctypes.c_int32,  # const int
                ctypes.c_uint32,  # const eBuildingClass
                ctypes.c_ulonglong,  # unsigned __int64 *
            ]
        ),
    },
    "cGcWFCBuildingState::UpdateParts": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBuildingState *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_int32,  # const int
        ]
    ),
    "DecideColourPaletteFromArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcWeightedColourId> *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # sBaseBuildingMaterial *
            ctypes.c_ulonglong,  # sBaseBuildingMaterial *
        ]
    ),
    "cGcWFCBuildingState::DecideColourPalette": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBuildingState *
        ]
    ),
    "cGcWFCBuildingState::ClearFoliage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBuildingState *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBiomeSubType>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInputActions> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkReplicatedVariableTemplated<cGcSettlementState>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSettlementState *
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cGcSettlementState> *
            ctypes.c_ulonglong,  # const cGcSettlementState *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkStringUtilities<char>::InsertString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkStringFunctions::SplitString<32,char>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cTkFixedString<32,char>,TkSTLAllocatorShim<cTkFixedString<32,char>,1,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<cTkFixedString<32,char>,TkSTLAllocatorShim<cTkFixedString<32,char>,1,-1> > *
            ctypes.c_char_p,  # const char *
            ctypes.c_char,  # const char
        ]
    ),
    "cGcNavGraphNode::cGcNavGraphNode": {
        "cGcNavGraphNode *, const cGcNavGraphNode *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNavGraphNode *
                ctypes.c_ulonglong,  # const cGcNavGraphNode *
            ]
        ),
        "cGcNavGraphNode *, const eNPCNavGraphNodeType, TkHandle, cGcNPCInteractiveObjectComponent *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNavGraphNode *
                ctypes.c_uint32,  # const eNPCNavGraphNodeType
                ctypes.c_uint32,  # TkHandle
                ctypes.c_ulonglong,  # cGcNPCInteractiveObjectComponent *
            ]
        ),
        "cGcNavGraphNode *, const eNPCNavGraphNodeType, TkHandle, const cTkVector3 *, cGcNPCInteractiveObjectComponent *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNavGraphNode *
                ctypes.c_uint32,  # const eNPCNavGraphNodeType
                ctypes.c_uint32,  # TkHandle
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # cGcNPCInteractiveObjectComponent *
            ]
        ),
    },
    "cTkPhysRelVec3::Averager::Get": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysRelVec3::Averager *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cTkPhysRelVec3::Averager::Add": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysRelVec3::Averager *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcInventoryStore::CreateSubstanceElement": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcInventoryElement *
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ulonglong,  # const cGcRealitySubstanceData *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcBuilding::cGcBuilding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuilding *
        ]
    ),
    "cGcBuildingList::FindBuilding": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBuilding *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingList *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcScanEventManager::GetActiveEvent": {
        "cGcScanEventManager *, const TkID<256> *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcScanEvent *
            argtypes=[
                ctypes.c_ulonglong,  # cGcScanEventManager *
                ctypes.c_char_p,  # const TkID<256> *
            ]
        ),
        "cGcScanEventManager *, int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcScanEvent *
            argtypes=[
                ctypes.c_ulonglong,  # cGcScanEventManager *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcBaseBuildingManager::GetBaseBuildingTable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBaseBuildingTable *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcBaseBuildingGlobalBuffer::PopulateBufferData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGlobalBuffer *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPersistentBBObjectData> *
        ]
    ),
    "cGcBaseBuildingGlobalBuffer::SetBufferOwnerId": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGlobalBuffer *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcBaseBuildingGlobalBuffer::FindBaseBuildingObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # GcPersistencyHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGlobalBuffer *
            ctypes.c_ulonglong,  # GcPersistencyHandle *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBaseBuildingGlobalBuffer::UpdateBaseBuildingObjectTimestamp": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGlobalBuffer *
            ctypes.c_uint32,  # GcPersistencyHandle
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingGlobalBuffer::GetBaseBuildingObjectData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPersistentBBObjectData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGlobalBuffer *
            ctypes.c_uint32,  # GcPersistencyHandle
        ]
    ),
    "cGcBaseBuildingPersistentBuffer::GenerateHashMessage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkHashMessage> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPersistentBuffer *
            ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkHashMessage> *
        ]
    ),
    "cGcBaseBuildingPersistentBuffer::GenerateSyncMessage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkSyncMessage> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPersistentBuffer *
            ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkSyncMessage> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBaseBuildingPersistentBuffer::ApplySyncMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPersistentBuffer *
            ctypes.c_ulonglong,  # const cGcNetworkSyncMessage *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcBaseBuildingPersistentBuffer::InitializeFromData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPersistentBuffer *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcPersistentBBObjectData> *
        ]
    ),
    "cGcBaseBuildingPersistentBuffer::RefundBlacklistedBuildingObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPersistentBuffer *
        ]
    ),
    "cGcBaseBuildingPersistentBuffer::StoreBaseBuildingObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # GcPersistencyHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPersistentBuffer *
            ctypes.c_ulonglong,  # GcPersistencyHandle *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPersistentBuffer::UpdateBaseBuildingObject": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPersistentBuffer *
            ctypes.c_uint32,  # GcPersistencyHandle
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPersistentBuffer::GetObjectCountsById": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPersistentBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcBaseBuildingPersistentBuffer::HasNetworkOwner": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPersistentBuffer *
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcBaseBuildingPersistentBuffer::RemoveDataFromRegion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPersistentBuffer *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcBlockList::Data::IsOnBlockList": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockList::Data *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcBlockList::Data::IsMessageOnBlockList": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockList::Data *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcBlockList::AddMessageToBlockList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockList *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "TkJsonFileReadStreamBase::Read": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkJsonFileReadStreamBase *
        ]
    ),
    "TkJsonTempFileReadStream::ReadData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # TkJsonTempFileReadStream *
        ]
    ),
    "cGcCloudSaveManager::GetObjectFromJsonReadStream": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCloudSaveManager *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcCloudSaveManager::ExportSaveToCloudThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcCloudSaveManager::CombineSaveData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCloudSaveManager *
        ]
    ),
    "cGcCloudSaveManager::CompressSaveData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCloudSaveManager *
        ]
    ),
    "cGcCloudSaveManager::ImportSaveFromCloudThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcCloudSaveManager::DecompressSaveData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCloudSaveManager *
        ]
    ),
    "cGcCloudSaveManager::ExtractSaveData": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCloudSaveManager *
        ]
    ),
    "cGcFrontendManager::ShowMessageBox": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcDifficultySettings::Save": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
            ctypes.c_ulonglong,  # cGcDifficultyStateData *
        ]
    ),
    "cGcDifficultySettings::GetPresetForGameMode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # ePresetGameMode
        ]
    ),
    "cGcDifficultySettings::SetSettingsData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
            ctypes.c_ulonglong,  # const cGcDifficultySettingsData *
        ]
    ),
    "cGcDifficultySettings::GetReplicatedState": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDifficultySettingsReplicatedState *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
            ctypes.c_ulonglong,  # cGcDifficultySettingsReplicatedState *
        ]
    ),
    "cGcDifficultySettings::GetSettingOptionLocIDs": FUNCDEF(
        restype=ctypes.c_int64,  # signed __int64
        argtypes=[
            ctypes.c_uint32,  # eDifficultySetting
            ctypes.c_ulonglong,  # std::vector<TkID<256>,TkSTLAllocatorShim<TkID<256>,8,-1> > *
        ]
    ),
    "cGcDifficultySettings::GetOptionListSettingAsIntWritable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # int *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsData *
            ctypes.c_uint32,  # eDifficultySetting
        ]
    ),
    "cGcDifficultySettings::GetToggleSettingWritable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDifficultySettingsData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsData *
            ctypes.c_uint32,  # eDifficultySetting
        ]
    ),
    "cGcDifficultySettings::CompareSettingDifficulty": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultySettingsData *
            ctypes.c_ulonglong,  # const cGcDifficultySettingsData *
            ctypes.c_uint32,  # eDifficultySetting
        ]
    ),
    "cGcDifficultySettings::GetDifficultyIconStringInsert": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
            ctypes.c_ulonglong,  # const cGcNetworkPlayer *
        ]
    ),
    "cGcDifficultySettings::FillPersistentBaseDifficultyData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
            ctypes.c_ulonglong,  # cGcPersistentBaseDifficultyData *
        ]
    ),
    "cGcDifficultySettings::HasDifficultyAlwaysBeenAtPresetOrHarder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
            ctypes.c_uint32,  # eDifficultyPresetType
        ]
    ),
    "cGcDifficultySettings::FindMatchingPreset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultySettingsData *
            ctypes.c_uint32,  # cGcDifficultySettings::ePresetMatchType
        ]
    ),
    "cGcDifficultySettings::GetPresetSettingsClampedByEditability": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDifficultySettingsData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsData *
            ctypes.c_ulonglong,  # const cGcDifficultySettingsData *
            ctypes.c_uint32,  # eDifficultyPresetType
        ]
    ),
    "TkInputDevicesGLFW::cTkPadXInput::IsAvailable": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
        ]
    ),
    "cGcDifficultySettings::IsItemGraveSpawnedOnDeath": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
        ]
    ),
    "cGcDifficultySettings::AreItemsDestroyedOnDeath": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
        ]
    ),
    "cGcDifficultySettings::IsSaveDestroyedOnDeath": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
        ]
    ),
    "cGcDifficultySettings::IsSprintingUnlimited": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
        ]
    ),
    "cGcDifficultySettings::DoesSprintingDrainEnergy": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
        ]
    ),
    "cGcDifficultySettings::GetSprintingRateMultiplier": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
        ]
    ),
    "cGcDifficultySettings::IsEnergyBarEnabled": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
        ]
    ),
    "cGcDifficultySettings::GetCurrencyCostMultiplier": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
            ctypes.c_uint32,  # eTypeOfCost
            ctypes.c_uint32,  # eCurrency
            ctypes.c_uint32,  # cGcDifficultySettings::eCurrencyScalingMode
        ]
    ),
    "cGcDifficultySettings::ScaleCurrencyCost": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # cGcDifficultySettings::eCurrencyScalingMode
            ctypes.c_uint32,  # eTypeOfCost
            ctypes.c_uint32,  # eCurrency
        ]
    ),
    "cGcDifficultySettings::ArePurchasesWithCostTypeFree": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
            ctypes.c_uint32,  # eTypeOfCost
            ctypes.c_uint32,  # eCurrency
        ]
    ),
    "cGcEncyclopediaComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcEncyclopediaComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncyclopediaComponent *
        ]
    ),
    "cGcEncyclopediaComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncyclopediaComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcEncyclopediaComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcEncyclopediaComponent *
        ]
    ),
    "ComponentInstancercGcEncyclopediaComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcSimpleInteractionComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcSimpleInteractionComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionComponent::GetSeed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSeed *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # cTkSeed *
        ]
    ),
    "cGcDroneComponent::GetTypedComponent": {
        "cTkAttachment *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
        "int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcDroneComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDroneComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryRecord::SavePersistent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryRecord *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcDiscoveryRecord::LoadPersistent": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryRecord *
            ctypes.c_ulonglong,  # ITkDocumentReader *
        ]
    ),
    "cGcDiscoveryMetadata::LoadPersistent": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryMetadata *
            ctypes.c_ulonglong,  # ITkDocumentReader *
        ]
    ),
    "cGcDiscoveryData::SavePersistent": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcDiscoveryData::LoadPersistent": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
        ]
    ),
    "cGcDiscoveryRecord::StartAsyncChecks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryRecord *
        ]
    ),
    "DiscoveryUtils::PopulateDiscoveryDataFromNode": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # const TkHandle
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cGcDiscoveryData *
        ]
    ),
    "DiscoveryUtils::InitialiseResourceDescriptorFromDiscoveryData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "DiscoveryUtils::DoesPlanetGeneratedCreatureMatchDiscoveryData": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRoleData *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "ConvertAtlasToInternalDiscoveryData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtlasDiscoveryData *
            ctypes.c_ulonglong,  # cGcDiscoveryData *
        ]
    ),
    "ConvertInternalToAtlasDiscoveryData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # cGcAtlasDiscoveryData *
        ]
    ),
    "cGcManagedDiscovery::LoadAndResolve": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcManagedDiscovery *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_ulonglong,  # cGcDiscoveryStoreImpl<DefaultDiscoveryDataHashing,19> *
        ]
    ),
    "Session::SavePersistent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Session *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "Session::LoadPersistent": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # Session *
            ctypes.c_ulonglong,  # ITkDocumentReader *
        ]
    ),
    "Session::RemoveRecord": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # Session *
            ctypes.c_ulonglong,  # const cGcDiscoveryRecord *
        ]
    ),
    "DiscoverySearchContext::DiscoverySearchContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DiscoverySearchContext *
        ]
    ),
    "cGcDiscoveryManager::Data::UpdateProcessRequestsInFlight": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
        ]
    ),
    "cGcDiscoveryManager::Data::UpdateProcessRequestsAsyncOps": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
        ]
    ),
    "cGcDiscoveryManager::Data::UpdateProcessSubmitted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
        ]
    ),
    "cGcDiscoveryManager::Data::UpdateCheckSubmissionQueue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDiscoveryManager::Data::UpdateCheckReportQueue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDiscoveryManager::Data::ProcessAtlasResponse": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # const cTkMetaMessageWrapperTemplated<cGcAtlasShared> *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_uint32,  # const AtlasRequestResult
        ]
    ),
    "cGcDiscoveryManager::Data::RetrieveOwnerDisplayName": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
            ctypes.c_ulonglong,  # const std::shared_ptr<cGcAsyncOpsTempDiscovery> *
        ]
    ),
    "cGcDiscoveryManager::Data::ConvertAndUpsertAtlasData": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
            ctypes.c_ulonglong,  # const cGcAtlasDiscovery *
            ctypes.c_ulonglong,  # cGcDiscoveryData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryManager::Data::NotifyDiscoveryQueryCompleted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ubyte,  # const bool
            ctypes.c_uint64,  # const unsigned __int64
        ]
    ),
    "cGcDiscoveryManager::Data::NotifyDiscovery": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # const cTkMetaMessageWrapperTemplated<cGcAtlasShared> *
            ctypes.c_uint64,  # const unsigned __int64
        ]
    ),
    "cGcDiscoveryManager::Data::LoadPersistent": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
            ctypes.c_ulonglong,  # ITkDocumentReader *
        ]
    ),
    "cGcDiscoveryManager::Data::SubmitDiscoveryData": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcDiscoveryManager::Data::SubbitDiscoveryReport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
            ctypes.c_ulonglong,  # cGcDiscoveryRecord *
        ]
    ),
    "cGcDiscoveryManager::Data::DeleteSearchInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
            ctypes.c_ulonglong,  # cGcDiscoverySearch *
        ]
    ),
    "cGcDiscoveryManager::Data::RunSearches": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
        ]
    ),
    "DbTrimFromPlanet": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryExport::Planet *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # std::vector<cGcDiscoveryRecord const *,TkSTLAllocatorShim<cGcDiscoveryRecord const *,8,19> > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const cGcWonderDiscoveryExport *
        ]
    ),
    "cGcDiscoveryManager::Data::SynchronousTrimDb": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
            ctypes.c_ulonglong,  # cGcDiscoveryManager *
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
        ]
    ),
    "StackContainer<std::vector<cGcWonderDiscoveryExport::sBasicDiscoveryData,StackAllocator<cGcWonderDiscoveryExport::sBasicDiscoveryData,32,-1>>,32,-1>::~StackContainer<std::vector<cGcWonderDiscoveryExport::sBasicDiscoveryData,StackAllocator<cGcWonderDiscoveryExport::sBasicDiscoveryData,32,-1>>,32,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<std::pair<unsigned __int64,unsigned __int64>,StackAllocator<std::pair<unsigned __int64,unsigned __int64>,32,-1> >,32,-1> *
        ]
    ),
    "cGcDiscoveryManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager *
        ]
    ),
    "cGcDiscoveryManager::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager *
        ]
    ),
    "cGcDiscoveryManager::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager *
        ]
    ),
    "cGcDiscoveryManager::SubmitDiscoveryData": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcDiscoveryManager::LookupDiscoveryWithStatus": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # const cGcDiscoveryRecord **
        ]
    ),
    "cGcDiscoveryManager::SubmitCustomNameForDiscovery": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager *
            ctypes.c_ulonglong,  # const cGcDiscoveryRecord *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcDiscoveryManager::Data::IsDiscoveryOwnedByPlayer": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
            ctypes.c_ulonglong,  # const cGcDiscoveryRecord *
        ]
    ),
    "cGcDiscoveryManager::GetDiscoveryRecord": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcDiscoveryRecord *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcDiscoveryManager::BeginRemoteQueryExact": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_uint64,  # const unsigned __int64
        ]
    ),
    "cGcDiscoveryManager::BeginRemoteQueryCategory": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_uint32,  # const eDiscoveryType
            ctypes.c_uint64,  # const unsigned __int64
        ]
    ),
    "cGcDiscoveryManager::BeginRemoteQueryAllOnPlanet": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_uint64,  # const unsigned __int64
        ]
    ),
    "cGcDiscoveryManager::CreateSearchInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDiscoverySearch *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager *
        ]
    ),
    "SortImmutableRecordsNewestFirst_SS": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryExport::SolarSystem *
            ctypes.c_ulonglong,  # const cGcDiscoveryExport::SolarSystem *
        ]
    ),
    "SortImmutableRecordsNewestFirst": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryRecord *
            ctypes.c_ulonglong,  # const cGcDiscoveryRecord *
        ]
    ),
    "SortImmutableRecordsOldestFirst_SS": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryExport::SolarSystem *
            ctypes.c_ulonglong,  # const cGcDiscoveryExport::SolarSystem *
        ]
    ),
    "SortImmutableRecordsOldestFirst": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryRecord *
            ctypes.c_ulonglong,  # const cGcDiscoveryRecord *
        ]
    ),
    "cGcDiscoveryManager::GenerateDiscoveryExport": FUNCDEF(
        restype=ctypes.c_ulonglong,  # GcDiscoverySearchConstraints::ConstraintBase *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager *
            ctypes.c_uint32,  # const cGcDiscoveryManager::ExportSortOption
            ctypes.c_ulonglong,  # std::vector<cGcDiscoveryRecord const *,TkSTLAllocatorShim<cGcDiscoveryRecord const *,8,19> > *
            ctypes.c_ulonglong,  # std::vector<GcDiscoverySearchConstraints::ConstraintBase *,TkSTLAllocatorShim<GcDiscoverySearchConstraints::ConstraintBase *,8,-1> > *
        ]
    ),
    "cGcDiscoveryManager::UnRegisterForEvents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager *
            ctypes.c_ulonglong,  # IDiscoveryManagerEventHandler *
        ]
    ),
    "cGcDiscoveryManager::Data::HideDiscoveryIfOwnerBlocked": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
            ctypes.c_ulonglong,  # const cGcDiscoveryRecord *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcDiscoveryManager::Data::HideAllDiscoveriesByBlockedUsers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryManager::Data *
        ]
    ),
    "cGcRewardManager::cGcRewardManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
        ]
    ),
    "cGcEntitlementManager::HasEntitlementReward": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcEntitlementManager *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcEntitlementManager::HasUsedEntitlementReward": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcEntitlementManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcEntitlementManager::MarkEntitlementUsed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEntitlementManager *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_char * 0x10,  # const TkID<128>
        ]
    ),
    "cGcDiscoveryStoreImpl<DefaultDiscoveryDataHashing,19>::ConstIterator::Next": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcDiscoveryRecord *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryStoreImpl<DefaultDiscoveryDataHashing,19>::ConstIterator *
        ]
    ),
    "cTkDynamicArray<cGcRewardTableItem>::~cTkDynamicArray<cGcRewardTableItem>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRewardTableItem> *
        ]
    ),
    "nms_rapidjson::GenericValue<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>>::HasMember": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericValue<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator> > *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "PersistentData<cGcPersistentBBObjectData>::Remove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PersistentData<cGcPersistentBBObjectData> *
        ]
    ),
    "cTkReplicatedVariableTemplated<cGcDifficultySettingsReplicatedState>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDifficultySettingsReplicatedState *
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cGcDifficultySettingsReplicatedState> *
            ctypes.c_ulonglong,  # const cGcDifficultySettingsReplicatedState *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcPersistentBBObjectData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPersistentBBObjectData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcDiscoveryDataRing<6>::HashDiscovery": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cTkDynamicArray<cGcRewardTableItem>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRewardTableItem> *
        ]
    ),
    "nms_rapidjson::GenericValue<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>>::FindMember": FUNCDEF(
        restype=ctypes.c_ulonglong,  # nms_rapidjson::GenericMemberIterator<0,nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator> > *
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericValue<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator> > *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericMemberIterator<0,nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator> > *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "nms_rapidjson::GenericValue<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>>::Accept<nms_rapidjson::Writer<StringHolder,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0>>": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericValue<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator> > *
            ctypes.c_ulonglong,  # nms_rapidjson::Writer<StringHolder,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0> *
            ctypes.c_ulonglong,  # StringHolder *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcEntitlementRewardsTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEntitlementRewardsTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "nms_rapidjson::GenericValue<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>>::FindMember<nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # nms_rapidjson::GenericMemberIterator<0,nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator> > *
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericValue<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator> > *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericMemberIterator<0,nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator> > *
            ctypes.c_ulonglong,  # const nms_rapidjson::GenericValue<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator> > *
        ]
    ),
    "nms_rapidjson::Writer<StringHolder,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0>::String": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::Writer<StringHolder,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0> *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>::ParseStream<16,nms_rapidjson::UTF8<char>,TkJsonTempFileReadStream>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # TkJsonTempFileReadStream *
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::SkipWhitespaceAndComments<16,TkJsonTempFileReadStream>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # TkJsonTempFileReadStream *
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseValue<16,TkJsonTempFileReadStream,nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # TkJsonTempFileReadStream *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        ]
    ),
    "nms_rapidjson::SkipWhitespace<TkJsonTempFileReadStream>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkJsonTempFileReadStream *
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseString<16,TkJsonTempFileReadStream,nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # TkJsonTempFileReadStream *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseNumber<16,TkJsonTempFileReadStream,nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # TkJsonTempFileReadStream *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        ]
    ),
    "nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>::EndArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>::StartArray": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        ]
    ),
    "nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>::EndObject": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>::String": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>::Double": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
            ctypes.c_double,  # long double
        ]
    ),
    "nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>::Uint64": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>::Int64": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
            ctypes.c_int64,  # __int64
        ]
    ),
    "nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>::Uint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>::Int": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
            ctypes.c_int32,  # int
        ]
    ),
    "nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>::Bool": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>::Null": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        ]
    ),
    "nms_rapidjson::GenericValue<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>>::GenericValue<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericValue<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator> > *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator> *
        ]
    ),
    "nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>::Malloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator> *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseHex4<TkJsonFileReadStream>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # TkJsonTempFileReadStream *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "nms_rapidjson::UTF8<char>::Encode<nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::StackStream<char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::StackStream<char> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "nms_rapidjson::internal::Stack<nms_rapidjson::CrtAllocator>::Expand<nms_rapidjson::Writer<TkJsonFileWriteStream,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0>::Level>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::internal::Stack<nms_rapidjson::CrtAllocator> *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "nms_rapidjson::internal::Stack<nms_rapidjson::CrtAllocator>::Expand<char>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::internal::Stack<nms_rapidjson::CrtAllocator> *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "nms_rapidjson::internal::StrtodNormalPrecision": FUNCDEF(
        restype=ctypes.c_double,  # long double
        argtypes=[
            ctypes.c_double,  # long double
            ctypes.c_int32,  # int
        ]
    ),
    "nms_rapidjson::internal::StrtodDiyFp": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # long double *
        ]
    ),
    "nms_rapidjson::internal::BigInteger::MultiplyPow5": FUNCDEF(
        restype=ctypes.c_ulonglong,  # nms_rapidjson::internal::BigInteger *
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::internal::BigInteger *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "nms_rapidjson::internal::CheckWithinHalfULP": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_double,  # long double
            ctypes.c_ulonglong,  # const nms_rapidjson::internal::BigInteger *
            ctypes.c_int32,  # int
        ]
    ),
    "nms_rapidjson::internal::StrtodBigInteger": FUNCDEF(
        restype=ctypes.c_double,  # long double
        argtypes=[
            ctypes.c_double,  # long double
            ctypes.c_char_p,  # const char *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int32,  # int
        ]
    ),
    "nms_rapidjson::CrtAllocator::Malloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::CrtAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "nms_rapidjson::internal::Stack<nms_rapidjson::CrtAllocator>::Resize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::internal::Stack<nms_rapidjson::CrtAllocator> *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDiscoveryRecord::cGcDiscoveryRecord": {
        "cGcDiscoveryRecord *, const cGcDiscoveryData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcDiscoveryRecord *
                ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ]
        ),
        "cGcDiscoveryRecord *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcDiscoveryRecord *
            ]
        ),
    },
    "nms_rapidjson::internal::u32toa": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "nms_rapidjson::internal::u64toa": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "nms_rapidjson::internal::DigitGen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const nms_rapidjson::internal::DiyFp *
            ctypes.c_ulonglong,  # const nms_rapidjson::internal::DiyFp *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "nms_rapidjson::internal::Grisu2": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_double,  # long double
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "nms_rapidjson::internal::WriteExponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "nms_rapidjson::internal::Prettify": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcDiscoveryRecord::CloneFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryRecord *
            ctypes.c_ulonglong,  # const cGcDiscoveryRecord *
        ]
    ),
    "cTkUserService::UserIdentity::UserIdentity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserService::UserIdentity *
            ctypes.c_ulonglong,  # const cTkUserService::UserIdentity *
        ]
    ),
    "cGcNetworkDelayedMessage::IsReadyToProcess": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkDelayedMessage *
        ]
    ),
    "cGcDiscoveryData::HashSpooky": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryData *
            ctypes.c_ulonglong,  # SpookyHash *
        ]
    ),
    "cGcCreatureComponent::GetExtraPartNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcCreatureComponent::GetType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::GetTypedComponent": {
        "cTkAttachment *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
        "int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcCreatureComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkResourceDescriptor::GetSeed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkSeed *
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcAISpaceshipComponent::GetTypedComponent": {
        "cTkAttachment *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
        "int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcAISpaceshipComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryMetadata::SetCustomName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryMetadata *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcMultitoolData::cGcMultitoolData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultitoolData *
        ]
    ),
    "cTkFixedArray<cGcMultitoolData,6>::cTkFixedArray<cGcMultitoolData,6>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcMultitoolData,6> *
        ]
    ),
    "cGcMultitoolData::~cGcMultitoolData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultitoolData *
        ]
    ),
    "cTkFixedArray<cGcMultitoolData,6>::~cTkFixedArray<cGcMultitoolData,6>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcMultitoolData,6> *
        ]
    ),
    "cGcByteBeatSong::cGcByteBeatSong": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSong *
        ]
    ),
    "cTkFixedArray<cGcByteBeatSong,8>::cTkFixedArray<cGcByteBeatSong,8>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcByteBeatSong,8> *
        ]
    ),
    "cTkFixedArray<cGcCharacterCustomisationSaveData,3>::cTkFixedArray<cGcCharacterCustomisationSaveData,3>": {
        "cTkFixedArray<cGcCharacterCustomisationSaveData,3> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkFixedArray<cGcCharacterCustomisationSaveData,3> *
            ]
        ),
        "cTkFixedArray<cGcCharacterCustomisationSaveData,3> *, const cTkFixedArray<cGcCharacterCustomisationSaveData,3> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkFixedArray<cGcCharacterCustomisationSaveData,3> *
                ctypes.c_ulonglong,  # const cTkFixedArray<cGcCharacterCustomisationSaveData,3> *
            ]
        ),
    },
    "cGcPetCustomisationData::cGcPetCustomisationData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetCustomisationData *
        ]
    ),
    "cTkFixedArray<cTkVector3,16>::cTkFixedArray<cTkVector3,16>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::array<cTkVector3,16> *
        ]
    ),
    "cTkFixedArray<cTkSeed,16>::cTkFixedArray<cTkSeed,16>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkSeed,16> *
        ]
    ),
    "cGcSquadronPilotData::cGcSquadronPilotData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronPilotData *
        ]
    ),
    "cTkFixedArray<cGcSquadronPilotData,4>::cTkFixedArray<cGcSquadronPilotData,4>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcSquadronPilotData,4> *
        ]
    ),
    "cGcSquadronPilotData::~cGcSquadronPilotData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronPilotData *
        ]
    ),
    "cTkFixedArray<cGcSquadronPilotData,4>::~cTkFixedArray<cGcSquadronPilotData,4>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcSquadronPilotData,4> *
        ]
    ),
    "cTkFixedArray<cTkVector3,256>::cTkFixedArray<cTkVector3,256>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkVector3,256> *
        ]
    ),
    "cTkFixedArray<cGcTerrainEdit,30000>::cTkFixedArray<cGcTerrainEdit,30000>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcTerrainEdit,30000> *
        ]
    ),
    "cGcNPCWorkerData::cGcNPCWorkerData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWorkerData *
        ]
    ),
    "cTkFixedArray<cGcNPCWorkerData,5>::cTkFixedArray<cGcNPCWorkerData,5>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcNPCWorkerData,5> *
        ]
    ),
    "cTkFixedArray<cGcNPCWorkerData,5>::~cTkFixedArray<cGcNPCWorkerData,5>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcNPCWorkerData,5> *
        ]
    ),
    "cTkFixedArray<cGcPlayerOwnershipData,7>::cTkFixedArray<cGcPlayerOwnershipData,7>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcPlayerOwnershipData,7> *
        ]
    ),
    "cTkFixedArray<cGcPlayerOwnershipData,7>::~cTkFixedArray<cGcPlayerOwnershipData,7>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcPlayerOwnershipData,7> *
        ]
    ),
    "cGcPlayerOwnershipData::cGcPlayerOwnershipData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerOwnershipData *
        ]
    ),
    "cTkFixedArray<cGcPlayerOwnershipData,12>::cTkFixedArray<cGcPlayerOwnershipData,12>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcPlayerOwnershipData,12> *
        ]
    ),
    "cTkFixedArray<cGcPlayerOwnershipData,12>::~cTkFixedArray<cGcPlayerOwnershipData,12>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcPlayerOwnershipData,12> *
        ]
    ),
    "cGcCharacterCustomisationSaveData::cGcCharacterCustomisationSaveData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationSaveData *
        ]
    ),
    "cTkFixedArray<cGcCharacterCustomisationSaveData,23>::cTkFixedArray<cGcCharacterCustomisationSaveData,23>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcCharacterCustomisationSaveData,23> *
        ]
    ),
    "cGcCharacterCustomisationSaveData::~cGcCharacterCustomisationSaveData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationSaveData *
        ]
    ),
    "cGcCharacterCustomisationData::cGcCharacterCustomisationData": {
        "cGcCharacterCustomisationData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcCharacterCustomisationData *
            ]
        ),
        "cGcCharacterCustomisationData *, const cGcCharacterCustomisationData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcCharacterCustomisationData *
                ctypes.c_ulonglong,  # const cGcCharacterCustomisationData *
            ]
        ),
    },
    "cTkFixedArray<cGcCharacterCustomisationData,3>::cTkFixedArray<cGcCharacterCustomisationData,3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcCharacterCustomisationData,3> *
        ]
    ),
    "cTkFixedArray<cGcCharacterCustomisationData,3>::~cTkFixedArray<cGcCharacterCustomisationData,3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcCharacterCustomisationData,3> *
        ]
    ),
    "cGcFleetFrigateSaveData::~cGcFleetFrigateSaveData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigateSaveData *
        ]
    ),
    "cGcExpeditionEventSaveData::~cGcExpeditionEventSaveData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventSaveData *
        ]
    ),
    "cGcFleetExpeditionSaveData::~cGcFleetExpeditionSaveData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpeditionSaveData *
        ]
    ),
    "cTkDynamicArray<cGcRepairTechData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRepairTechData> *
        ]
    ),
    "cTkDynamicArray<cGcRepairTechData>::~cTkDynamicArray<cGcRepairTechData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRepairTechData> *
        ]
    ),
    "cTkFixedArray<cGcQuickMenuActionSaveData,10>::cTkFixedArray<cGcQuickMenuActionSaveData,10>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcQuickMenuActionSaveData,10> *
        ]
    ),
    "cGcHotActionsSaveData::cGcHotActionsSaveData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHotActionsSaveData *
        ]
    ),
    "cTkFixedArray<cGcHotActionsSaveData,3>::cTkFixedArray<cGcHotActionsSaveData,3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcHotActionsSaveData,3> *
        ]
    ),
    "cTkDynamicArray<cGcDungeonQuestParams>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDungeonQuestParams> *
        ]
    ),
    "cGcSeasonStateData::~cGcSeasonStateData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonStateData *
        ]
    ),
    "cTkFixedArray<cGcWonderRecord,15>::cTkFixedArray<cGcWonderRecord,15>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcWonderRecord,15> *
        ]
    ),
    "cTkFixedArray<cGcWonderRecord,8>::cTkFixedArray<cGcWonderRecord,8>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcWonderRecord,8> *
        ]
    ),
    "cTkFixedArray<cGcWonderRecord,13>::cTkFixedArray<cGcWonderRecord,13>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcWonderRecord,13> *
        ]
    ),
    "cTkFixedArray<cGcWonderRecord,11>::cTkFixedArray<cGcWonderRecord,11>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcWonderRecord,11> *
        ]
    ),
    "cTkFixedArray<cGcWonderRecord,12>::cTkFixedArray<cGcWonderRecord,12>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcWonderRecord,12> *
        ]
    ),
    "cTkFixedArray<cGcWonderRecordCustomData,12>::cTkFixedArray<cGcWonderRecordCustomData,12>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcWonderRecordCustomData,12> *
        ]
    ),
    "cTkFixedArray<cGcSyncBufferSaveDataArray,4>::cTkFixedArray<cGcSyncBufferSaveDataArray,4>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcSyncBufferSaveDataArray,4> *
        ]
    ),
    "cTkFixedArray<cGcSyncBufferSaveDataArray,4>::~cTkFixedArray<cGcSyncBufferSaveDataArray,4>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcSyncBufferSaveDataArray,4> *
        ]
    ),
    "cGcBaseBuildingPersistentBuffer::BaseBuildingPersistentData::BaseBuildingPersistentData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPersistentBuffer::BaseBuildingPersistentData *
        ]
    ),
    "cGcNetworkBufferHash::EnsureMinimumChunks": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBufferHash *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRealityManager::LookupNameByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureSwarm *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityManager::LookupSubstanceCategoryByID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::IsProceduralItem": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char,  # char
        ]
    ),
    "cGcExpeditionEventResult::Save": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventResult *
            ctypes.c_ulonglong,  # cGcExpeditionEventSaveData *
            ctypes.c_ulonglong,  # const cGcPlayerFleetManager *
        ]
    ),
    "cGcExpeditionEventResult::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventResult *
            ctypes.c_ulonglong,  # const cGcExpeditionEventSaveData *
            ctypes.c_ulonglong,  # const cGcFleetExpedition *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionEventResult::OverrideReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventResult *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcExpeditionEventResult::GetDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventResult *
            ctypes.c_ulonglong,  # const cGcFleetExpedition *
            ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
        ]
    ),
    "cGcExpeditionEventResult::GetEventDifficulty": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventResult *
            ctypes.c_ulonglong,  # const cGcFleetExpedition *
        ]
    ),
    "cGcExpeditionEventResult::GetRewardDescription": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventResult *
            ctypes.c_ulonglong,  # const cGcFleetExpedition *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcExpeditionEventResult::GetInterventionDialog": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventResult *
            ctypes.c_ulonglong,  # TkID<256> *
        ]
    ),
    "cGcExpeditionEventResult::OnInterventionDeclined": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventResult *
        ]
    ),
    "cGcExpeditionEventResult::OnInterventionSucceeded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventResult *
            ctypes.c_ulonglong,  # const cGcFleetExpedition *
        ]
    ),
    "cGcExpeditionEventResult::OnInterventionFailed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventResult *
            ctypes.c_ulonglong,  # const cGcFleetExpedition *
        ]
    ),
    "cGcExpeditionEventResult::AddAffectedFrigate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventResult *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # eFrigateDamageResponse
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcExpeditionEventResult::GetChanceOfFrigateDamage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventResult *
        ]
    ),
    "cGcExpeditionEventResult::AttemptToRepairFrigates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventResult *
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcExpeditionEventResult::DamageFrigates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventResult *
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcExpeditionEventResult::CalculateNumberOfDamagedFrigates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventResult *
        ]
    ),
    "cGcExpeditionEventResult::ChooseDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventResult *
            ctypes.c_ulonglong,  # const cGcFleetExpedition *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcExpeditionEventResult::ChooseRewards": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventResult *
            ctypes.c_ulonglong,  # const cGcFleetExpedition *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcMaintenanceComponent::GetTypedComponent": {
        "cTkAttachment *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
        "int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcPlayerNotifications::RequestNotifyMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkSmartResHandle *
            ctypes.c_uint32,  # eIconStyle
            ctypes.c_uint32,  # eFrontendPage
        ]
    ),
    "cGcPlayerNotifications::HasActiveCriticalMissionSequences": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
        ]
    ),
    "cGcExpeditionHologramComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcExpeditionHologramComponent::GetTypedComponent": {
        "int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_int32,  # int
            ]
        ),
        "cTkAttachment *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
    },
    "cGcFreighterBaseComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcFleetExpedition::cGcFleetExpedition": {
        "cGcFleetExpedition *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcFleetExpedition *
            ]
        ),
        "cGcFleetExpedition *, const cGcFleetExpedition *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcFleetExpedition *
                ctypes.c_ulonglong,  # const cGcFleetExpedition *
            ]
        ),
    },
    "cGcFleetExpedition::GetOwnerId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcFleetExpedition::GetPlayerFleetManager": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFleetExpedition ***
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcFleetExpedition::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_uint32,  # eExpeditionCategory
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcExpeditionEventResult,8,-1> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cGcPlayerFleetManager *
        ]
    ),
    "cGcFleetExpedition::SetTerminalPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcFleetExpedition::Save": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_ulonglong,  # const cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetExpeditionSaveData *
        ]
    ),
    "cGcFleetExpedition::Load": {
        "cGcFleetExpedition *, cGcPlayerFleetManager *, const cGcFleetExpeditionSaveData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcFleetExpedition *
                ctypes.c_ulonglong,  # cGcPlayerFleetManager *
                ctypes.c_ulonglong,  # const cGcFleetExpeditionSaveData *
            ]
        ),
        "cGcFleetExpedition *, const cGcPlayerFleetManager *, const cGcNetworkFleetExpeditionsSyncMessage *, int, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcFleetExpedition *
                ctypes.c_ulonglong,  # const cGcPlayerFleetManager *
                ctypes.c_ulonglong,  # const cGcNetworkFleetExpeditionsSyncMessage *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcFleetExpedition::Invalidate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcFleetExpedition::GetPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcFleetExpedition::AddFrigate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetExpedition::RemoveFrigate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetExpedition::IsFrigateDamaged": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_ulonglong,  # const cGcFleetFrigate *
        ]
    ),
    "cGcFleetExpedition::OnFrigateDamaged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetExpedition::SendDamagedFrigateHome": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetExpedition::SetPowerup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcFleetExpedition::GetName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcFleetExpedition::SetUA": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFleetExpedition::GetTimeUntilUAChange": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcFleetExpedition::CanAffordToStart": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_ulonglong,  # const cGcProductData **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcFleetExpedition::GetAverageEventDifficulty": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcFleetExpedition::GetStat": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_uint32,  # eFrigateStatType
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFleetExpedition::GetExpeditionPredictedDistance": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcFleetExpedition::ProcessNextEvent": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFleetExpedition::StartExpedition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFleetExpedition::GiveExpeditionUpgrades": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetExpedition::EndExpedition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcFleetExpedition::HasExpeditionStarted": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcFleetExpedition::IsExpeditionActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcFleetExpedition::GetTimeRemainingInSeconds": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcFleetExpedition::GetTimeRemainingUntilNextExpeditionEventInSeconds": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcFleetExpedition::GetTimeRemainingUntilExpeditionEventInSeconds": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFleetExpedition::GetNumberOfEventsNeedingProcessed": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcFleetExpedition::IsWaitingForIntervention": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcFleetExpedition::GetInterventionEventSeed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSeed *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_ulonglong,  # cTkSeed *
        ]
    ),
    "hknpLodShapeFunctions::getMutationSignals": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcFleetExpedition::IsInterventionEvent": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFleetExpedition::GetEventDescription": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
        ]
    ),
    "cGcFleetExpedition::GetEventIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_ulonglong,  # const cGcExpeditionEventResult *
        ]
    ),
    "cGcFleetExpedition::GetExpeditionDurationInSeconds": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcFleetExpedition::GiveRewards": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFleetExpedition::SetExpeditionTimeRemaining": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFleetExpedition::StopWaitingForIntervention": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cTkLODComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkLODComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkLODComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLODComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODComponent *
        ]
    ),
    "cTkLODComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMarkerComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerComponent *
        ]
    ),
    "ComponentInstancercTkLODComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercTkLODComponent *
        ]
    ),
    "ComponentInstancercTkLODComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentLoadInstancercTkLODComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentLoadInstancercTkLODComponent *
        ]
    ),
    "cGcFleetFrigate::cGcFleetFrigate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetFrigate::GetOwnerId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcFleetFrigate::GetPlayerFleetManager": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFleetFrigate ***
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetFrigate::Save": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_ulonglong,  # const cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetFrigateSaveData *
        ]
    ),
    "cGcFleetFrigate::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_ulonglong,  # const cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const cGcFleetFrigateSaveData *
        ]
    ),
    "cGcFleetFrigate::Invalidate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetBasePosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
        ]
    ),
    "cGcFleetFrigate::GetNameFromSeed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # const eFrigateClass
        ]
    ),
    "cGcFleetFrigate::GetName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcFleetFrigate::GetCustomName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        ]
    ),
    "cGcFleetFrigate::GetPrimaryTraitOverride": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFleetFrigate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetPageData *
        ]
    ),
    "cGcFleetFrigate::GetCaptainName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFleetFrigate::SetAIShipSpawn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_ulonglong,  # cGcAIShipSpawn *
        ]
    ),
    "cGcFleetFrigate::GetAIShipSpawn": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAIShipSpawn *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetFrigate::UpdateSpawnedShipDamagedMaintenanceComponents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetFrigate::GetFrigateClassIconString": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetFrigate::GetFrigateClass": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetFrigate::GetRace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetFrigate::GetCost": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetFrigate::GetStat": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_uint32,  # eFrigateStatType
        ]
    ),
    "cGcFleetFrigate::GetTrait": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcFrigateTraitData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFleetFrigate::AddTrait": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_ulonglong,  # const cGcFrigateTraitData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFleetFrigate::SetDamaged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFleetFrigate::SetDamageTaken": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_char,  # char
        ]
    ),
    "cGcFleetFrigate::IsDamaged": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetFrigate::RepairDamagedBit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFleetFrigate::GetDamagedMaintenanceComponents": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cGcMaintenanceComponent *,TkSTLAllocatorShim<cGcMaintenanceComponent *,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetFrigate::SetPirateDefencePlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_ulonglong,  # const cGcPlayerCommon *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcFleetFrigate::GetPirateDefencePlayer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcPlayerCommon *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetFrigate::GetExpedition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFleetExpedition *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetFrigate::OnExpeditionComplete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFleetFrigate::GetCanLevelUp": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcCreatureComponent::GetRole": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcFleetFrigate::GetNumberOfExpeditionsCompleted": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetFrigate::GetNumberOfTimesDamaged": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetFrigate::AwardLevelUpStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcFleetFrigate::GetCargo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_ulonglong,  # cGcInventoryElement *
        ]
    ),
    "cGcFleetFrigate::OnCargoCollected": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int32,  # int
        ]
    ),
    "base64_encode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "sha256_transf": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sha256_ctx *
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "sha256_update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sha256_ctx *
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "sha256_final": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sha256_ctx *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "hmac_sha256_2": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "cTkCommandLine::GetArgumentString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const TryStoreMode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkCommandLine *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkCommandLine::GetArgumentBool": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkCommandLine *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcGameAnalytics::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameAnalytics *
            ctypes.c_ulonglong,  # cGcAtlasTransport *
        ]
    ),
    "cGcGameAnalytics::StartSession": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameAnalytics *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGameAnalytics::EndSession": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameAnalytics *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGameAnalytics::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameAnalytics *
            ctypes.c_float,  # float
        ]
    ),
    "cGcGameAnalytics::AddRequiredKeyValues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameAnalytics *
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGameAnalytics::HttpHandleInitRequestResult": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameAnalytics *
            ctypes.c_ulonglong,  # const Http::RequestResult *
        ]
    ),
    "cGcGameAnalytics::HttpHandleSessionStartRequestResult": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameAnalytics *
            ctypes.c_ulonglong,  # const Http::RequestResult *
        ]
    ),
    "cGcGameAnalytics::StartEvents": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameAnalytics *
        ]
    ),
    "cGcGameAnalytics::AddCustomEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameAnalytics *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGameAnalytics::AddProgressionEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameAnalytics *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGameAnalytics::UploadEventsJSON": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameAnalytics *
        ]
    ),
    "cGcGameAnalytics::CheckJsonBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameAnalytics *
        ]
    ),
    "cGcGameAnalytics::BuildHeirachicalEventId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameAnalytics *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcGameAnalytics::OnMultiplayerSessionStarting": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameAnalytics *
            ctypes.c_uint32,  # cGcNetworkConstants::LobbyType
        ]
    ),
    "cGcGameAnalytics::OnMultiplayerSessionFinished": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameAnalytics *
            ctypes.c_uint32,  # cGcNetworkConstants::LobbyType
        ]
    ),
    "cGcGameAnalytics::OnNetworkPlayerStateEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameAnalytics *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint32,  # eGcNetworkPlayerStateEvent
        ]
    ),
    "cGcGameAnalytics::UpdateFireteamSession": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameAnalytics *
        ]
    ),
    "StackContainer<std::vector<cGcGalaxyWaypoint,StackAllocator<cGcGalaxyWaypoint,8,-1>>,8,-1>::~StackContainer<std::vector<cGcGalaxyWaypoint,StackAllocator<cGcGalaxyWaypoint,8,-1>>,8,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cGcGalaxyWaypoint,StackAllocator<cGcGalaxyWaypoint,8,-1> >,8,-1> *
        ]
    ),
    "cGcGameKnowledge::LoadPersistent": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameKnowledge *
            ctypes.c_ulonglong,  # ITkDocumentReader *
        ]
    ),
    "cGcGameKnowledge::AddGalacticWaypoint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameKnowledge *
            ctypes.c_ulonglong,  # const cGcGalaxyWaypoint *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcGameKnowledge::RemoveGalacticWaypoint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameKnowledge *
            ctypes.c_ulonglong,  # const cGcGalaxyWaypoint *
        ]
    ),
    "cGcGameKnowledge::FindExactGalacticWaypoint": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameKnowledge *
            ctypes.c_ulonglong,  # const cGcGalaxyWaypoint *
        ]
    ),
    "cGcGameKnowledge::GetGalacticWaypointWithAddress": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGalaxyWaypoint *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameKnowledge *
            ctypes.c_ulonglong,  # const cGcGalacticAddressData *
        ]
    ),
    "cGcGameKnowledge::ClearAllWaypoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameKnowledge *
        ]
    ),
    "cGcGameKnowledge::ClearAllSpecificWaypoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameKnowledge *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerStateData::~cGcPlayerStateData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcNetworkSynchronisedStateDataMessage::CustomSerializeHeader": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkSynchronisedStateDataMessage *
        ]
    ),
    "SynchronisedStateBufferTypes::sBufferFormat<0>::ConstructBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSynchronisedStateBufferBase *
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcGameState::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
        ]
    ),
    "cGcGameState::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
        ]
    ),
    "cGcGameState::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_double,  # double
        ]
    ),
    "cGcGameState::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
        ]
    ),
    "cGcGameState::ComputeWarpCapability": FUNCDEF(
        restype=ctypes.c_ulonglong,  # WarpCapabilityResult *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # WarpCapabilityResult *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cGcGalaxyAttributeGenerator::StarSystemKeyAttributes *
        ]
    ),
    "cGcGameState::ComputeWarpEngineJumpDistanceInLightyears": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_uint32,  # ItemLookupType
        ]
    ),
    "cGcGameState::SendRemoteEgg": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_uint64,  # cGcInventoryIndex
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcProductData *
        ]
    ),
    "cGcGameState::OnReceiveRemoteByteBeatSong": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_ulonglong,  # cGcByteBeatSong *
        ]
    ),
    "cGcGameState::OnReceiveRemoteItems": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcGameState::OnReceiveRemoteCreatureEgg": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_ulonglong,  # sOwnedCreatureInfo *
        ]
    ),
    "cGcGameState::LoadState": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcExpeditionEventResult,8,-1> *
        ]
    ),
    "cGcGameState::SetInventoryStoreBalance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
        ]
    ),
    "cGcGameState::LoadSpecificSave": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_uint32,  # const GcGameStatePersistence::SpecificSave::Type
        ]
    ),
    "sGcTelemetryMultiplayerCountVariant::GetVariant": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # sGcTelemetryMultiplayerCountVariant *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcGameState::LoadFromPersistentStorage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_uint32,  # const cTkStoragePersistent::Slot
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGameState::GetDefaultSaveDataFilename": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcGameState::SetFreshStartSpwanData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcDefaultSaveData *
        ]
    ),
    "cGcGameState::UpgradePatch0To1": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<StatListener,8,-1> *
        ]
    ),
    "cGcGameState::UpgradePatch1ToUpdate1": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<StatListener,8,-1> *
        ]
    ),
    "cGcGameState::UpgradeUpdate1ToUpdate1_1": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<StatListener,8,-1> *
        ]
    ),
    "cGcGameState::UpgradeUpdate1_2ToUpdate1_3": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<StatListener,8,-1> *
        ]
    ),
    "cGcGameState::UpgradeUpdate1_3ToUpdate1_33": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate1_33ToUpdate1_35": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate1_39ToUpdate1_4Internal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<StatListener,8,-1> *
        ]
    ),
    "cGcGameState::UpgradeUpdate1_4InternalToUpdate1_4": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate1_4ToUpdate1_5": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate1_5ToUpdate1_51": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate1_62ToUpdate1_8": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate1_8ToUpdate1_81": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate1_82ToUpdate1_83": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # const cGcDefaultSaveData *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate1_83ToUpdate1_84": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate1_84ToUpdate1_85": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate1_86ToUpdate1_87": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate1_91ToUpdate3_30": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate3_30ToUpdate3_32": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate3_32ToUpdate3_34": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate3_34ToUpdate3_35": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate3_36ToUpdate3_52": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate3_53ToUpdate3_80": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate3_80ToUpdate3_81": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate3_81ToUpdate3_85": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate3_85ToUpdate3_94": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate3_94ToUpdate4_00": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # const cGcDefaultSaveData *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate4_00ToUpdate4_04": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # const cGcDefaultSaveData *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate4_04ToUpdate4_05": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # const cGcDefaultSaveData *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::UpgradeUpdate4_05ToUpdate4_09": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # const cGcDefaultSaveData *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::SaveSpawnLocation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerSpawnStateData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGameState::SaveProgress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_uint32,  # GcGameStatePersistence::SpecificSave::Type
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcPlayerSpawnStateData *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGameState::SaveThreadData::SaveThreadData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState::SaveThreadData *
        ]
    ),
    "cGcGameState::SaveProgressThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcGameState::OnSaveProgressCompleted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # GcGameStatePersistence::SpecificSave::Type
        ]
    ),
    "cGcGameState::WriteStateToJSON": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_uint32,  # const cTkStoragePersistent::Slot
            ctypes.c_ulonglong,  # const cGcPlayerStateData *
            ctypes.c_ulonglong,  # const cGcPlayerSpawnStateData *
        ]
    ),
    "cGcGameState::OverwriteDataFromJSON": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
            ctypes.c_ulonglong,  # cGcPlayerSpawnStateData *
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcGameState::CopyNewElements": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cTkDynamicArray<TkID<128> > *
            ctypes.c_ulonglong,  # const cTkDynamicArray<TkID<128> > *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcGameState::CleanUpSuitInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # cGcDefaultSaveData *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcGameState::GetPlayerSpawnMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcGameState::GetPlayerDeathRespawnMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcGameState::GetShipSpawnMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcGameState::GetPlayerFreighterOwnershipForCurrentSystem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
        ]
    ),
    "cGcGameState::GetPlayerFreighterOwnership": {
        "cGcGameState *, int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            argtypes=[
                ctypes.c_ulonglong,  # cGcGameState *
                ctypes.c_int32,  # int
            ]
        ),
        "cGcGameState *, const cTkUserIdBase<cTkFixedString<64,char> > *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            argtypes=[
                ctypes.c_ulonglong,  # cGcGameState *
                ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
            ]
        ),
    },
    "cGcGameState::GetPlayerFleetManager": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcGameState::GetPlayerFleetManagerWriteable": {
        "cGcGameState *, int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            argtypes=[
                ctypes.c_ulonglong,  # cGcGameState *
                ctypes.c_int32,  # int
            ]
        ),
        "cGcGameState *, const cTkUserIdBase<cTkFixedString<64,char> > *, bool": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            argtypes=[
                ctypes.c_ulonglong,  # cGcGameState *
                ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "DeleteAllSavesForPlayerSlot": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
        ]
    ),
    "DeleteManualSaveForPlayerSlot": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
        ]
    ),
    "cGcGameState::UpdateSummary": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
        ]
    ),
    "cTkMetadataCache<cGcBiomeListPerStarType>::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cTkTrailData> *
            ctypes.c_ulonglong,  # cTkTrailData *
        ]
    ),
    "cTkMetadataCache<cGcInventoryStoreBalance>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkTrailData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcInventoryStoreBalance> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcInventoryStoreBalance>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cGcInventoryStoreBalance> *
        argtypes=[]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::TryApplyCoherentEditMessage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # const cGcNetworkSynchronisedStateEditMessage *
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::ApplyEditMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # const cGcNetworkSynchronisedStateEditMessage *
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::ApplyDataMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # const cGcNetworkSynchronisedStateDataMessage *
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::CreateDataMessage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkSynchronisedStateDataMessage> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkSynchronisedStateDataMessage> *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::UpdateHashOffsetVersion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::IsHashOffsetCompatible": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::SetHashOffset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::GetHashOffsetVersion": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::GetHashOffset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::GetChunkVersion": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::GetChunkHashValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::GetChunksCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
        ]
    ),
    "cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer>::PopulateVariableData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # cGcInteractionBuffer *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMaintenanceContainer> *
        ]
    ),
    "cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer>::InitializeFromVariableData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # const cGcInteractionBuffer *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcMaintenanceContainer> *
        ]
    ),
    "cTkBitArray<unsigned __int64,1,8>::GetNumTrue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,8> *
        ]
    ),
    "cTkDynamicArray<cGcTradingSupplyData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTradingSupplyData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcTeleportEndpoint>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTeleportEndpoint> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcScanEventSave>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcScanEventSave> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPlayerMissionProgress>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerMissionProgress> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcWordGroupKnowledge>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWordGroupKnowledge> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcExpeditionEventSaveData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExpeditionEventSaveData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcInteractionData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInteractionData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcGenericMissionVersionProgress>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInventoryIndex> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcInventoryElement>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInventoryElement> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcInventoryElement>::cTkDynamicArray<cGcInventoryElement>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInventoryElement> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcInventoryElement> *
        ]
    ),
    "cGcNetworkBuffer<std::vector<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,TkSTLAllocatorShim<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,16,-1>>>::ResizeBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBuffer<std::vector<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,TkSTLAllocatorShim<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,16,-1> > > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::SerialiseChunk": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::StoreData": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # const sMaintenanceSaveKey *
            ctypes.c_ulonglong,  # const cGcMaintenanceContainer *
            ctypes.c_ulonglong,  # cGcMaintenanceContainer *
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::TryApplyCoherentEdit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # const sMaintenanceSaveKey *
            ctypes.c_ulonglong,  # const cGcMaintenanceContainer *
            ctypes.c_ulonglong,  # const cGcMaintenanceContainer *
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::TryApplyCoherentClear": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # const sMaintenanceSaveKey *
            ctypes.c_ulonglong,  # const cGcMaintenanceContainer *
        ]
    ),
    "cTkDynamicArray<cGcMaintenanceContainer>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMaintenanceContainer> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkLinearHashTable<void const *,cTkFixedString<256,char>,-1,0,cTkLinearHashTableHash<void const *>>::erase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<void const *,cTkFixedString<256,char>,-1,0,cTkLinearHashTableHash<void const *> > *
            ctypes.c_ulonglong,  # const cTkLinearHashTable<void const *,cTkFixedString<256,char>,-1,0,cTkLinearHashTableHash<void const *> >::iterator *
        ]
    ),
    "sOwnedCreatureInfo::sOwnedCreatureInfo": {
        "sOwnedCreatureInfo *, const sOwnedCreatureInfo *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # sOwnedCreatureInfo *
                ctypes.c_ulonglong,  # const sOwnedCreatureInfo *
            ]
        ),
        "sOwnedCreatureInfo *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # sOwnedCreatureInfo *
            ]
        ),
    },
    "cGcInventoryContainer::cGcInventoryContainer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryContainer *
            ctypes.c_ulonglong,  # const cGcInventoryContainer *
        ]
    ),
    "cGcNetworkBuffer<std::vector<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,TkSTLAllocatorShim<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,16,-1>>>::PushBack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBuffer<std::vector<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,TkSTLAllocatorShim<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,16,-1> > > *
            ctypes.c_ulonglong,  # const std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer> *
        ]
    ),
    "cGcNetworkBuffer<std::vector<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,TkSTLAllocatorShim<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,16,-1>>>::RemoveHash": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBuffer<std::vector<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,TkSTLAllocatorShim<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,16,-1> > > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cTkDynamicArray<cGcInventorySpecialSlot>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInventorySpecialSlot> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCharacterCustomisationBoneScaleData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCharacterCustomisationBoneScaleData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcDiscoveryRewardLookupTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEntitlementRewardsTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcPlayerMissionUpgradeMapTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerMissionUpgradeMapTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLinearHashTable<cTkFixedString<256,char>,cTkMetadataCache<cGcPaletteList>::cTkMetadataLookup,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char>>>::find<cTkFixedString<256,char>>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkLinearHashTable<cTkFixedString<256,char>,cTkMetadataCache<cTkTrailData>::cTkMetadataLookup,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char> > >::iterator *
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<cTkFixedString<256,char>,cTkMetadataCache<cTkTrailData>::cTkMetadataLookup,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char> > > *
            ctypes.c_ulonglong,  # cTkLinearHashTable<cTkFixedString<256,char>,cTkMetadataCache<cTkTrailData>::cTkMetadataLookup,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char> > >::iterator *
            ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcPaletteList>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkTrailData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "GenerateHashValue<sMaintenanceSaveKey,cGcMaintenanceContainer>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer> *
        ]
    ),
    "cGcNetworkBuffer<std::vector<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,TkSTLAllocatorShim<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,16,-1>>>::ShiftChunkHashesAdd<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBuffer<std::vector<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,TkSTLAllocatorShim<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,16,-1> > > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcNetworkBuffer<std::vector<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,TkSTLAllocatorShim<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,16,-1>>>::ShiftChunkHashesAdd<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBuffer<std::vector<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,TkSTLAllocatorShim<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,16,-1> > > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcNetworkBuffer<std::vector<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,TkSTLAllocatorShim<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,16,-1>>>::ShiftChunkHashesRemove<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBuffer<std::vector<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,TkSTLAllocatorShim<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,16,-1> > > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcNetworkBuffer<std::vector<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,TkSTLAllocatorShim<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,16,-1>>>::ShiftChunkHashesRemove<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBuffer<std::vector<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,TkSTLAllocatorShim<std::pair<sMaintenanceSaveKey,cGcMaintenanceContainer>,16,-1> > > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcAISpaceshipComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkNetReplicatedEntityComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkFixedArray<cGcPlayerMissionParticipant,13>::cTkFixedArray<cGcPlayerMissionParticipant,13>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcPlayerMissionParticipant,13> *
        ]
    ),
    "TkCountUTF8<127>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFixedString<127,char> *
        ]
    ),
    "DiscoveryResolver::NameAndOwnerResult::GetLocalisedName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DiscoveryResolver::NameAndOwnerResult *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyVoxelData::~cGcGalaxyVoxelData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelData *
        ]
    ),
    "cTkVector3MetaKD<unsigned __int64,23>::reserve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3MetaKD<unsigned __int64,23> *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkVector3MetaKD<unsigned __int64,23>::~cTkVector3MetaKD<unsigned __int64,23>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3MetaKD<unsigned __int64,23> *
        ]
    ),
    "cGcGalaxyVoxelData::cGcGalaxyVoxelData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelData *
        ]
    ),
    "cGcInventoryStore::GetElement": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
        ]
    ),
    "cGcPlayerState::TryRemoveFromInventories": {
        "cGcPlayerState *, const std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *, const cGcInventoryElement *, TryStoreMode": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_ulonglong,  # const std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_uint32,  # TryStoreMode
            ]
        ),
        "cGcPlayerState *, const std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *, const cGcInventoryElement *, std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_ulonglong,  # const std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_ulonglong,  # std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
            ]
        ),
    },
    "cGcDifficultySettings::IsFuelFree": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
        ]
    ),
    "cGcDifficultySettings::IsFuelCheap": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
        ]
    ),
    "GcGameStatePersistence::ExtractVersion": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const TkStorage::Blob *
        ]
    ),
    "GcGameStatePersistence::Restructuring::ApplyGameMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkStorage::Blob *
            ctypes.c_uint32,  # ePresetGameMode
        ]
    ),
    "GcGameStatePersistence::MakeSaveVersion": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # ePresetGameMode
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # GcGameStatePersistence::eSaveVersion
        ]
    ),
    "GcGameStatePersistence::GetAllPlayerSlotsInformation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<GcGameStatePersistence::StorageSlotInformation,TkSTLAllocatorShim<GcGameStatePersistence::StorageSlotInformation,4,-1> > *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "GcGameStatePersistence::GetSlotValidity": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # cTkStoragePersistent::Slot
            ctypes.c_ulonglong,  # const cTkStoragePersistent::SlotState *
            ctypes.c_ulonglong,  # GcGameStatePersistence::StorageSlotInformation *
        ]
    ),
    "GcGameStatePersistence::GetStorageSlotInformation": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # cTkStoragePersistent::Slot
            ctypes.c_ulonglong,  # GcGameStatePersistence::StorageSlotInformation *
        ]
    ),
    "GcGameStatePersistence::HasSpecificSave": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # const GcGameStatePersistence::SpecificSave::Type
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # cTkStoragePersistent::Slot *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "GcGameStatePersistence::IsAutosaveSlot": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # cTkStoragePersistent::Slot
        ]
    ),
    "GcGameStatePersistence::Restructuring::IsRestructureRequired": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "GcGameStatePersistence::Restructuring::RestructureSaveSlots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcFrontendPageFunctions::IsTechOnlySubInventory": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcInteractionComponent::SetDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcInteractionComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcPlayerNotifications::GetNotificationSequencesWaiting": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # std::vector<cGcNotificationSequence *,TkSTLAllocatorShim<cGcNotificationSequence *,8,-1> > *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerNotifications::GetNotificationSequence": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNotificationSequence *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcInventoryStore::cGcInventoryStore": {
        "cGcInventoryStore *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcInventoryStore *
            ]
        ),
        "cGcInventoryStore *, cGcInventoryStore *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcInventoryStore *
                ctypes.c_ulonglong,  # cGcInventoryStore *
            ]
        ),
        "cGcInventoryStore *, const cGcInventoryStore *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcInventoryStore *
                ctypes.c_ulonglong,  # const cGcInventoryStore *
            ]
        ),
    },
    "cGcInventoryStore::GenerateLayoutSimple": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eInventoryStackSizeGroup
        ]
    ),
    "cGcInventoryStore::GeneratePersonalLayout": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcInventoryStore::GenerateProceduralInventory": {
        "cGcInventoryStore *, unsigned int, const cTkSeed *, int, eSizeType, eWeaponStatClass, LayoutGenerationMode, LayoutCompareGenerationType, eInventoryClass, std::vector<cGcInventoryTechProbability,TkSTLAllocatorShim<cGcInventoryTechProbability,8,-1> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcInventoryStore *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # const cTkSeed *
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # eSizeType
                ctypes.c_uint32,  # eWeaponStatClass
                ctypes.c_uint32,  # LayoutGenerationMode
                ctypes.c_uint32,  # LayoutCompareGenerationType
                ctypes.c_uint32,  # eInventoryClass
                ctypes.c_ulonglong,  # std::vector<cGcInventoryTechProbability,TkSTLAllocatorShim<cGcInventoryTechProbability,8,-1> > *
            ]
        ),
        "cGcInventoryStore *, std::vector<cGcInventoryTechProbability,TkSTLAllocatorShim<cGcInventoryTechProbability,8,-1> > *, TkHandle, const cTkSeed *, int, LayoutGenerationMode, LayoutCompareGenerationType, eInventoryClass": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcInventoryStore *
                ctypes.c_ulonglong,  # std::vector<cGcInventoryTechProbability,TkSTLAllocatorShim<cGcInventoryTechProbability,8,-1> > *
                ctypes.c_uint32,  # TkHandle
                ctypes.c_ulonglong,  # const cTkSeed *
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # LayoutGenerationMode
                ctypes.c_uint32,  # LayoutCompareGenerationType
                ctypes.c_uint32,  # eInventoryClass
            ]
        ),
    },
    "cGcInventoryStore::GenerateProceduralLayoutLegacyVersion0": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eSizeType
            ctypes.c_uint32,  # LayoutGenerationMode
            ctypes.c_uint32,  # LayoutCompareGenerationType
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryStore::GenerateProceduralLayout": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # eSizeType
        ]
    ),
    "cGcInventoryStore::SetCapacityFromGenerationData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryLayoutGenerationDataEntry *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcInventoryStore::SetDimensionsFromGenerationData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryLayoutGenerationDataEntry *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcInventoryStore::GenerateFreighterSubstances": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcInventoryStore::GetBaseStatsData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryGenerationBaseStatData *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eShipClass
            ctypes.c_uint32,  # eWeaponStatClass
        ]
    ),
    "cGcInventoryStore::GenerateProceduralBaseStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # eInventoryClass
            ctypes.c_uint32,  # eShipClass
            ctypes.c_uint32,  # eWeaponStatClass
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryStore::CheckAndUpdateBaseStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # eInventoryClass
            ctypes.c_uint32,  # eShipClass
        ]
    ),
    "cGcInventoryStore::GenerateProceduralTechnology": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # eShipClass
            ctypes.c_ulonglong,  # std::vector<cGcInventoryTechProbability,TkSTLAllocatorShim<cGcInventoryTechProbability,8,-1> > *
        ]
    ),
    "cGcInventoryStore::GenerateProceduralClass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcInventoryStore::GenerateLayoutByInventoryType": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcInventoryStore::Add": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryIndex *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryIndex *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
        ]
    ),
    "cGcInventoryStore::IsAtMaxCapacity": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_uint32,  # const InventoryChoice
            ctypes.c_uint32,  # const eShipClass
        ]
    ),
    "cGcInventoryStore::GetMaxCapacity": FUNCDEF(
        restype=ctypes.c_int16,  # __int16
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # const eShipClass
        ]
    ),
    "cGcInventoryStore::IncreaseDimsToSupportCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # const eShipClass
        ]
    ),
    "cGcInventoryStore::IncreaseCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # const eShipClass
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcInventoryStore::AddSlotsWithCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # const int
            ctypes.c_ubyte,  # const bool
            ctypes.c_uint32,  # const eShipClass
        ]
    ),
    "cGcInventoryStore::GetTechGroupCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcInventoryStore::Remove": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_uint64,  # cGcInventoryIndex
            ctypes.c_int32,  # int
        ]
    ),
    "cGcInventoryStore::CopyInventoryStore": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
        ]
    ),
    "cGcInventoryStore::PopulateInventoryContainer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryContainer *
        ]
    ),
    "cGcInventoryStore::LoadFromContainer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryContainer *
            ctypes.c_int32,  # signed int
        ]
    ),
    "cGcInventoryStore::EnforceStackSizeLimitsOnAllItems": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
        ]
    ),
    "cGcInventoryStore::CheckAndEnableTechBonusSlotsFromDefaultContainer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryContainer *
        ]
    ),
    "cGcInventoryStore::CheckAndEnableTechBonusSlots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcInventoryStore::PackageAllTech": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
        ]
    ),
    "cGcInventoryStore::DamageRandomTech": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcInventoryStore::BreakShipItems": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_float,  # float
        ]
    ),
    "cGcInventoryStore::BreakCrashedShipSlots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
        ]
    ),
    "cGcInventoryStore::BreakAndBlockSlotsWithTech": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
        ]
    ),
    "cGcInventoryStore::DoesSlotGiveTechBonus": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
        ]
    ),
    "cGcInventoryStore::RepairEntireInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
        ]
    ),
    "cGcInventoryStore::GetCost": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eShipClass
            ctypes.c_uint32,  # eWeaponStatClass
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryStore::GetCostRepairSlot": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eShipClass
            ctypes.c_uint32,  # eWeaponStatClass
        ]
    ),
    "cGcInventoryStore::GetCostTradeIn": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eShipClass
            ctypes.c_uint32,  # eWeaponStatClass
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryStore::ValidateBrokenSlots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcInventoryStore::GetSubstanceSlotMaxStorage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcInventoryStore::GetProductSlotMaxStorage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcInventoryStore::GetNodeSlotsAndTechs": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # std::vector<cGcInventoryTechProbability,TkSTLAllocatorShim<cGcInventoryTechProbability,8,-1> > *
            ctypes.c_ulonglong,  # eSizeType *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcInventoryStore::CanInventoryStoreTechnology": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcTechnology *
            ctypes.c_uint32,  # eShipClass
        ]
    ),
    "cGcInventoryStore::GetShipClassFromSizeType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # eSizeType
        ]
    ),
    "cGcInventoryStore::GetStatModifierFromInventory": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_uint32,  # eStatsType
        ]
    ),
    "cGcInventoryStore::GetWeaponClassFromNode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcInventoryStore::CreateTechElement": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcInventoryElement *
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ulonglong,  # const cGcTechnology *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcInventoryStore::UpgradeInventoryChestContainerCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryContainer *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcMPMissionSelectionHelper::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionSelectionHelper *
        ]
    ),
    "cGcMPMissionSelectionHelper::OnMissionStarted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionSelectionHelper *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcMPMissionSelectionHelper::OnMissionHandedIn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionSelectionHelper *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcMPMissionSelectionHelper::HasMissionStarted": {
        "cGcMPMissionSelectionHelper *, const cTkUserIdBase<cTkFixedString<64,char> > *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcMPMissionSelectionHelper *
                ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
            ]
        ),
        "cGcMPMissionSelectionHelper *, const TkID<128> *, const cTkSeed *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcMPMissionSelectionHelper *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ulonglong,  # const cTkSeed *
            ]
        ),
    },
    "cGcMPMissionSelectionHelper::HasMissionBeenHandedIn": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionSelectionHelper *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcMPMissionSelectionHelper::GetAllParticipcants": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cGcMPMissionSelectionHelper::sGcOfferedMPMissionParticipant,TkSTLAllocatorShim<cGcMPMissionSelectionHelper::sGcOfferedMPMissionParticipant,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionSelectionHelper *
            ctypes.c_ulonglong,  # std::vector<cGcMPMissionSelectionHelper::sGcOfferedMPMissionParticipant,TkSTLAllocatorShim<cGcMPMissionSelectionHelper::sGcOfferedMPMissionParticipant,8,-1> > *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcMPMissionSelectionHelper::GetMissionTargetUA": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcScanEventManager::InterstellarSearchResult *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionSelectionHelper *
            ctypes.c_ulonglong,  # cGcScanEventManager::InterstellarSearchResult *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcMPMissionSelectionHelper::JoinMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionSelectionHelper *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMPMissionSelectionHelper::Ready": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionSelectionHelper *
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcMPMissionSelectionHelper::IsAvailableToCurrentPlayer": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionSelectionHelper *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcMPMissionSelectionHelper::StartMission": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionSelectionHelper *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcMPMissionSelectionHelper::LeaveMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionSelectionHelper *
        ]
    ),
    "cGcMPMissionSelectionHelper::HasMissionOffer": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionSelectionHelper *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # cTkSeed *
        ]
    ),
    "cGcMPMissionSelectionHelper::RemoveParticipant": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionSelectionHelper *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcMPMissionSelectionHelper::RPCOnMissionOffered": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_char * 0x10,  # TkID<128>
            common.GcSeed,  # cTkSeed
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMPMissionSelectionHelper::RPCOnMissionOfferRescinded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcMPMissionSelectionHelper::RPCOnRemotePlayerLeftMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcMPMissionSelectionHelper::RPCOnParticipantReadyStatus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_char * 0x10,  # TkID<128>
            common.GcSeed,  # cTkSeed
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMPMissionSelectionHelper::RPCOnMissionStartedProper": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcMPMissionTracker::cGcMPMissionTracker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
        ]
    ),
    "cGcMPMissionTracker::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
        ]
    ),
    "cGcMPMissionTracker::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
            ctypes.c_double,  # double
        ]
    ),
    "cGcMPMissionTracker::sGcSyncedMPMissionPlayerData::sGcSyncedMPMissionPlayerData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker::sGcSyncedMPMissionPlayerData *
            ctypes.c_ulonglong,  # const cGcMPMissionTracker::sGcSyncedMPMissionPlayerData *
        ]
    ),
    "cGcMPMissionTracker::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcMPMissionSelectionHelper::sGcOfferedMPMission,8,-1> *
        ]
    ),
    "cGcMPMissionTracker::OnMissionStarted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
            ctypes.c_char * 0x10,  # TkID<128>
            common.GcSeed,  # cTkSeed
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMPMissionTracker::IsMissionActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
        ]
    ),
    "cGcMPMissionTracker::IsActiveMission": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcMPMissionTracker::GetInvolvedNetworkPlayers": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cGcNetworkPlayer *,TkSTLAllocatorShim<cGcNetworkPlayer *,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
            ctypes.c_ulonglong,  # std::vector<cGcNetworkPlayer *,TkSTLAllocatorShim<cGcNetworkPlayer *,8,-1> > *
        ]
    ),
    "cGcMPMissionTracker::GetInvolvedNetworkPlayerIds": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cTkUserIdBase<cTkFixedString<64,char> >,TkSTLAllocatorShim<cTkUserIdBase<cTkFixedString<64,char> >,1,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
            ctypes.c_ulonglong,  # std::vector<cTkUserIdBase<cTkFixedString<64,char> >,TkSTLAllocatorShim<cTkUserIdBase<cTkFixedString<64,char> >,1,-1> > *
        ]
    ),
    "cGcMPMissionTracker::QueryAmountInRemoteInventories": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
        ]
    ),
    "cGcMPMissionTracker::QueryRemoteStat": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcMPMissionTracker::ClearMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcMPMissionTracker::sGcSyncedMPMissionPlayerData,8,-1> *
        ]
    ),
    "cGcMPMissionTracker::OnPlayerLeftMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
            ctypes.c_uint32,  # const cGcNetworkConstants::LobbyType
        ]
    ),
    "cGcMPMissionTracker::RemovePlayerFromMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcMPMissionTracker::QueryRemoteDiscoveredDiscoveries": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBitArray<unsigned int,1,32> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
            ctypes.c_ulonglong,  # cTkBitArray<unsigned int,1,32> *
            ctypes.c_uint32,  # const eDiscoveryType
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcMPMissionTracker::GetAccountIdWhoDiscovered": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
            ctypes.c_uint32,  # const eDiscoveryType
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcMPMissionTracker::DiscoveryDataSubmitted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcMPMissionTracker::ReplicatePlayerStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
        ]
    ),
    "cGcMPMissionTracker::ReplicateDiscoveries": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMPMissionTracker::GetStatValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcMPMissionTracker::RPCReceiveDiscoveries": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint32,  # const eDiscoveryType
            ctypes.c_ulonglong,  # _DWORD *
        ]
    ),
    "cGcMPMissionTracker::RPCReceiveExistingMissionOffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_char * 0x10,  # TkID<128>
            common.GcSeed,  # cTkSeed
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMPMissionTracker::RPCOnMissionStarted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcMPMissionTracker::RPCPlayerLeftMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcMPMissionTracker::OnNetworkPlayerStateEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint32,  # eGcNetworkPlayerStateEvent
        ]
    ),
    "cGcMsgBeacon::RemoveMarker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMsgBeacon *
        ]
    ),
    "cGcMsgBeaconManager::Data::UploadBeacon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMsgBeaconManager::Data *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cGcUniverseAddressData *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcMsgBeaconManager::Data::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMsgBeaconManager::Data *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMsgBeaconManager::Data::QueueBeaconForAsyncChecks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMsgBeaconManager::Data *
            ctypes.c_ulonglong,  # const cGcAtlasDiscovery *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcMsgBeaconManager::Data::SpawnBeacon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMsgBeaconManager::Data *
            ctypes.c_ulonglong,  # const std::shared_ptr<cGcMsgBeacon> *
        ]
    ),
    "cGcMsgBeaconManager::Data::SubmitContentReport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMsgBeaconManager::Data *
            ctypes.c_ulonglong,  # const std::shared_ptr<cGcMsgBeacon> *
        ]
    ),
    "cGcMsgBeaconManager::Data::FindBeaconsInRange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMsgBeaconManager::Data *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # std::vector<TkHandle,TkSTLAllocatorShim<TkHandle,4,-1> > *
        ]
    ),
    "cGcRepairTechComponent::GetSavePosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcRepairTechComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechComponent *
        ]
    ),
    "ComponentInstancercGcRepairTechComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcRepairTechComponent *
        ]
    ),
    "ComponentInstancercGcRepairTechComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcRepairTechComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcRepairTechComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcGeneratorUnitComponent::Seed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratorUnitComponent *
        ]
    ),
    "cGcGeneratorUnitComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratorUnitComponent *
        ]
    ),
    "cGcGeneratorUnitComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratorUnitComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGeneratorUnitComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratorUnitComponent *
        ]
    ),
    "ComponentInstancercGcGeneratorUnitComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcGeneratorUnitComponent *
        ]
    ),
    "ComponentInstancercGcCreatureHarvesterComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcGeneratorUnitComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcGeneratorUnitComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcSimpleInteractionComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcPersistentInteractionBuffer::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPersistentInteractionBuffer::InitializeFromData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_ulonglong,  # const cGcInteractionBuffer *
        ]
    ),
    "cGcPersistentInteractionBuffer::PopulateBufferData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_ulonglong,  # cGcInteractionBuffer *
        ]
    ),
    "cGcPersistentInteractionBuffer::SaveInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cGcInteractionData *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcPersistentInteractionBuffer::DeleteAllInteractions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
        ]
    ),
    "cGcPersistentInteractionBuffer::StoreInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_ulonglong,  # const cGcInteractionData *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcPersistentInteractionBuffer::GetInteraction": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcInteractionData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPersistentInteractionBuffer::GetInteractionOnPlanet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcInteractionData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPersistentInteractionBuffer::GetInteractionCheckingFurthest": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcInteractionData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPersistentInteractionBuffer::GetNearestInteraction": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcInteractionData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcPersistentInteractionBuffer::LoadGalacticAddressBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcPersistentInteractionBuffer::RegionHasInteractionsPlanetPosition": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcPersistentInteractionBuffer::InternalDoTreeRebuildNoLock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
        ]
    ),
    "cGcPersistentInteractionBuffer::GenerateHashMessage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkHashMessage> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkHashMessage> *
        ]
    ),
    "cGcPersistentInteractionBuffer::GenerateSyncMessage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkSyncMessage> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkSyncMessage> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPersistentInteractionBuffer::ApplySyncMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_ulonglong,  # const cGcNetworkSyncMessage *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcPersistentInteractionBuffer::GenerateHashValue": FUNCDEF(
        restype=ctypes.c_uint16,  # unsigned __int16
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPersistentInteractionBuffer::GetMatchingInteraction": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcInteractionData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
        ]
    ),
    "cGcPersistentInteractionBuffer::ClearInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPersistentInteractionBuffer::ShouldSyncWithPlayer": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionBuffer *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cTkDynamicArray<cGcProductData>::end": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkArraySTLIterator<cGcProductData>::InternalConstIterator *
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcProductData> *
            ctypes.c_ulonglong,  # cTkArraySTLIterator<cGcProductData>::InternalConstIterator *
        ]
    ),
    "HashStringForJSON<3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcStatsManager::GetStatValue<float>": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "RefreshComponentType<cGcSimpleInteractionComponent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "RefreshComponentType<cGcInteractionComponent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "RefreshComponentType<cGcDestructableComponent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "RefreshComponentType<cGcMaintenanceComponent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "RefreshComponentType<cGcRepairTechComponent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "RefreshComponentType<cGcRefinerUnitComponent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "RefreshComponentType<cGcGeneratorUnitComponent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcDestructableComponent::GetTypedComponent": {
        "int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_int32,  # int
            ]
        ),
        "cTkAttachment *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
    },
    "cGcDestructableComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkNetReplicatedEntityComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkRigidBody::GetMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cTkRigidBody::GetPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcMPMissionTracker::ClearPreMissionUA": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
        ]
    ),
    "cGcMPMissionSelectionHelper::sGcOfferedMPMission::sGcOfferedMPMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionSelectionHelper::sGcOfferedMPMission *
            ctypes.c_ulonglong,  # const cGcMPMissionSelectionHelper::sGcOfferedMPMission *
        ]
    ),
    "cGcScanEventManager::GetEventData": {
        "cGcScanEventManager *, const TkID<256> *, eScanTable": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcScanEventData *
            argtypes=[
                ctypes.c_ulonglong,  # cGcScanEventManager *
                ctypes.c_char_p,  # const TkID<256> *
                ctypes.c_uint32,  # eScanTable
            ]
        ),
        "cGcScanEventManager *, const TkID<256> *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcScanEventData *
            argtypes=[
                ctypes.c_ulonglong,  # cGcScanEventManager *
                ctypes.c_char_p,  # const TkID<256> *
            ]
        ),
    },
    "cGcDifficultySettings::IsChargingFree": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettings *
        ]
    ),
    "cGcCustomInventoryComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkAttachment::FindAttachment": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgNodeAttachment *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAttachment::FindNumAttachments": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcInventoryStore::RepairSlot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_uint64,  # cGcInventoryIndex
        ]
    ),
    "cGcInventoryStore::GetNumSpecialSlots": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_uint32,  # eInventorySpecialSlotType
        ]
    ),
    "cTkTrig<float>::Log2": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "cTkTrig<float>::Log": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "cGcInventoryStore::IsSlotBlockedByBrokenTech": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
        ]
    ),
    "cGcInventoryStore::GetClass": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
        ]
    ),
    "cGcInventoryStore::IsSlotBroken": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
        ]
    ),
    "cGcInventoryStore::GetNumSlotsFree": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
        ]
    ),
    "cGcPlayerState::IsTechKnown": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcInventoryStore::ClearItems": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
        ]
    ),
    "cGcInventoryStore::IsValidLayoutPos": FUNCDEF(
        restype=ctypes.c_uint8,  # unsigned __int8
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
        ]
    ),
    "cGcInventoryStore::GetNumSlots": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
        ]
    ),
    "cGcPersistentInteractionsManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionsManager *
        ]
    ),
    "cGcPersistentInteractionsManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionsManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPersistentInteractionsManager::InitializeFromData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionsManager *
            ctypes.c_ulonglong,  # const cTkFixedArray<cGcInteractionBuffer,11> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcPersistentBase> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<unsigned __int64> *
        ]
    ),
    "cGcPersistentInteractionsManager::PopulateArrays": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionsManager *
            ctypes.c_ulonglong,  # cTkFixedArray<cGcInteractionBuffer,11> *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPersistentBase> *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # cTkDynamicArray<unsigned __int64> *
        ]
    ),
    "cGcPersistentInteractionsManager::FindPlayerBaseBufferByBasePosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # BaseIndex *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionsManager *
            ctypes.c_ulonglong,  # BaseIndex *
            ctypes.c_uint32,  # ePersistentBaseTypes
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcPersistentInteractionsManager::GetPlayerBuffersInRange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionsManager *
            ctypes.c_ulonglong,  # std::vector<BaseIndex,TkSTLAllocatorShim<BaseIndex,2,-1> > *
            ctypes.c_uint32,  # ePersistentBaseTypes
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
        ]
    ),
    "cGcPersistentInteractionsManager::GetInteractionBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMaintenanceBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionsManager *
            ctypes.c_uint32,  # eInteractionBufferType
        ]
    ),
    "cGcPersistentInteractionsManager::LoadGalacticAddressBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentInteractionsManager *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::cGcPlayerBasePersistentBuffer": {
        "cGcPlayerBasePersistentBuffer *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ]
        ),
        "cGcPlayerBasePersistentBuffer *, cGcPlayerBasePersistentBuffer *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
                ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ]
        ),
    },
    "cGcPlayerBasePersistentBuffer::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcLinkGrid,8,-1> *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::PopulateBufferData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # cGcPersistentBase *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetBaseBuildingObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # std::vector<GcPersistencyHandle,TkSTLAllocatorShim<GcPersistencyHandle,4,-1> > *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetFirstBaseBuildingObjectWithID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # GcPersistencyHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # GcPersistencyHandle *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetBaseBuildingObjectsWithID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # std::vector<GcPersistencyHandle,TkSTLAllocatorShim<GcPersistencyHandle,4,-1> > *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetBaseBuildingObjectsForRegion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # const cGcRegionTerrain *
            ctypes.c_ulonglong,  # std::vector<GcPersistencyHandle,TkSTLAllocatorShim<GcPersistencyHandle,4,-1> > *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::InitializeFromData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # const cGcPersistentBase *
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::AddBaseObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # const std::vector<cGcPlayerBasePersistentBuffer::PlayerBasePersistentData,TkSTLAllocatorShim<cGcPlayerBasePersistentBuffer::PlayerBasePersistentData,16,-1> > *
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcLinkGrid,8,-1> *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::RefundBlacklistedBuildingObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GenerateLinkGridConnections": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcLinkGrid,8,-1> *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GenerateBaseLayout": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::ProcessObjects": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::CheckDiscoveryOwnerData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::LoadGalacticAddress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_ulonglong,  # unsigned __int16 *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::StoreBaseBuildingObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # GcPersistencyHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # GcPersistencyHandle *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerBasePersistentBuffer::FindBaseBuildingObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # GcPersistencyHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # GcPersistencyHandle *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::UpdateBaseBuildingObject": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint32,  # GcPersistencyHandle
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetBaseBuildingObjectData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcPersistentBaseEntry *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint32,  # GcPersistencyHandle
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetLinkedMatchingHandle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # GcPersistencyHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # GcPersistencyHandle *
            ctypes.c_uint32,  # GcPersistencyHandle
        ]
    ),
    "cGcPlayerBasePersistentBuffer::SetLinkObjectState": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint32,  # GcPersistencyHandle
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerBasePersistentBuffer::AddAmountToGrid": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint32,  # GcPersistencyHandle
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetDecayFactor": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # const GcPersistencyHandle *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetLinkObjectSnapshot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint32,  # GcPersistencyHandle
            ctypes.c_ulonglong,  # sLinkObjectSnapshot *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetLinkObjectDependentSnapshot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint32,  # GcPersistencyHandle
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # sLinkObjectDependentSnapshot *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::AreObjectsOnSameGrid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint32,  # GcPersistencyHandle
            ctypes.c_uint32,  # GcPersistencyHandle
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetBaseBuildingObjectsOnSameGrid": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint32,  # GcPersistencyHandle
            ctypes.c_ulonglong,  # std::vector<GcPersistencyHandle,TkSTLAllocatorShim<GcPersistencyHandle,4,-1> > *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetDependentBaseBuildingObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint32,  # GcPersistencyHandle
            ctypes.c_ulonglong,  # std::vector<GcPersistencyHandle,TkSTLAllocatorShim<GcPersistencyHandle,4,-1> > *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetScreenshotMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::ForceAddNodesAroundPlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerBasePersistentBuffer::ClearPlayerBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcLinkGrid,8,-1> *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::ClearAndUnClaimBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcLinkGrid,8,-1> *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetBaseComputerPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetLastEditedByUserName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetBaseStats": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcPlayerBasePersistentBuffer::sStats *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::IsPlayerBaseOnSystem": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerBasePersistentBuffer::IsPlayerBaseOnCurrentPlanet": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::CountObjectsById": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::ValidateValues": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetPersistentData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer::PlayerBasePersistentData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint32,  # GcPersistencyHandle
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetGridSnapshotData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # sLinkObjectGridSnapshot *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::HasNetworkOwner": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GenerateHashMessage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkHashMessage> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkHashMessage> *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GenerateSyncMessage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkSyncMessage> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkSyncMessage> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerBasePersistentBuffer::ApplySyncMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # const cGcNetworkSyncMessage *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::ApplySyncDataToObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcPersistentBaseEntry *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerBasePersistentBuffer::ApplySyncRemoveObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerBasePersistentBuffer::StartAsyncChecks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::RefundObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::OnHashOffsetChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetNearestSqrDistanceForObjects": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntry *
            ctypes.c_ulonglong,  # const cGcBaseLinkGridConnectionData *
            ctypes.c_ulonglong,  # const cGcPersistentBaseEntry *
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntry *
            ctypes.c_ulonglong,  # const cGcBaseLinkGridConnectionData *
            ctypes.c_ulonglong,  # const cGcPersistentBaseEntry *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GetNearestSqrDistanceForConnections": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseLinkGridConnectionData *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cGcBaseLinkGridConnectionData *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GatherDirectlyLinkedSockets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_int32,  # int
            ctypes.c_char,  # char
            ctypes.c_ulonglong,  # cTkStackVector<cGcBaseLinkMap::sLinkSocket,32,-1> *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GatherDirectlyLinkedDependantSockets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkStackVector<cGcBaseLinkMap::sLinkSocket,32,-1> *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::AreObjectsTransferLinked": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerBasePersistentBuffer::OnLinkObjectAdded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcLinkGrid,8,-1> *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::OnLinkObjectRemoved": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcLinkGrid,8,-1> *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::RegenerateLinkGrids": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcLinkGrid,8,-1> *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::MergeLinkGrids": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # cTkStackVector<unsigned short,8,-1> *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerBasePersistentBuffer::CreateNewLinkGrid": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::UpdateVisualStateOfGrid": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcPlayerBasePersistentBuffer::PlayerBasePersistentData::GetEntryData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBaseBuildingEntry *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer::PlayerBasePersistentData *
        ]
    ),
    "cGcPlayerByteBeatLibrary::AddToPlaylist": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerByteBeatLibrary *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerByteBeatLibrary::RemoveFromPlaylist": {
        "cGcPlayerByteBeatLibrary *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerByteBeatLibrary *
                ctypes.c_int32,  # int
            ]
        ),
        "cGcPlayerByteBeatLibrary *, const TkID<128> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerByteBeatLibrary *
                ctypes.c_char_p,  # const TkID<128> *
            ]
        ),
    },
    "cGcPlayerByteBeatLibrary::MoveUpPlayList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerByteBeatLibrary *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerByteBeatLibrary::MoveDownPlayList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerByteBeatLibrary *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerByteBeatLibrary::GetPlaylistSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerByteBeatLibrary *
        ]
    ),
    "cGcFrontendManager::RequestDeactivate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageFunctions::Option2ButtonPressed": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcFrontendPageFunctions::SetImage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyle *
            ctypes.c_ulonglong,  # const cTkTextureResource *
        ]
    ),
    "cGcPlayerNotifications::ClearTimedMessages": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcPlayerNotifications::TimedMessage,16,-1> *
        ]
    ),
    "cGcPlayerByteBeatLibrary::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerByteBeatLibrary *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerByteBeatLibrary::GetSongFromId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcByteBeatSong *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerByteBeatLibrary *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerByteBeatLibrary::ActivateFrontEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerByteBeatLibrary *
        ]
    ),
    "cGcPlayerByteBeatLibrary::IsSongValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcByteBeatSong *
        ]
    ),
    "cGcPlayerByteBeatLibrary::ClearSong": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSong *
        ]
    ),
    "cGcPlayerByteBeatLibrary::RenderFrontEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerByteBeatLibrary *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcPlayerByteBeatLibrary::DoTransferPopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerByteBeatLibrary *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcFrontendPopup *
        ]
    ),
    "cTkAnimPoseComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkAnimPoseComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimPoseComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureEggComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcPlayerCreatureOwnership::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcPlayerCreatureOwnership::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
        ]
    ),
    "cGcPlayerCreatureOwnership::GenerateTraits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # eCreatureRole
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcPlayerCreatureOwnership::GetNumberOfCreatures": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCreatureOwnership::GetNumberOfUnlockedPetSlots": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
        ]
    ),
    "cGcPlayerCreatureOwnership::AdoptCreature": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcPlayerCreatureOwnership::RemoveCreatureFromCollection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCreatureOwnership::CanConsiderSummoning": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerCreatureOwnership::ShouldUseIndoorNavigation": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "cGcPlayerCreatureOwnership::GetNumCreaturesOwned": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcPlayerCreatureOwnership::AreAnyEggsReadyToHatch": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCreatureOwnership::CanSummon": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerCreatureOwnership::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerCreatureOwnership::GetEmoteArcTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcPlayerCreatureOwnership::ShouldShowEmoteArc": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerCreatureOwnership::GetPlacementOrigin": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcPlayerCreatureOwnership::GetSpawnPostion": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerCreatureOwnership::GetModifiedBehaviourWeight": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # ePetBehaviour
        ]
    ),
    "cGcPlayerCreatureOwnership::GetModifiedBehaviourCooldown": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # ePetBehaviour
        ]
    ),
    "cGcPlayerCreatureOwnership::UpdateMoodsOnActionComplete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_uint32,  # ePetBehaviour
        ]
    ),
    "cGcPlayerCreatureOwnership::UpdateMoods": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_uint32,  # ePetBehaviour
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerCreatureOwnership::GetPlayerCreatureName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerCreatureOwnership::GetDominantMood": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerCreatureOwnership::GetPlayerCreatureMoodString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_uint32,  # ePetMood
            ctypes.c_ulonglong,  # float *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eSpecialPetChatType
        ]
    ),
    "cGcPlayerCreatureOwnership::FormatTraitValueString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # ePetTrait
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cGcPlayerCreatureOwnership::GetPlayerCreatureTraitInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_ulonglong,  # std::vector<DiscoveryResolver::DiscoveryInfo::DiscoveryInfoValue,TkSTLAllocatorShim<DiscoveryResolver::DiscoveryInfo::DiscoveryInfoValue,8,-1> > *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerCreatureOwnership::GetPlayerCreatureInfoBinocs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_ulonglong,  # std::vector<DiscoveryResolver::DiscoveryInfo::DiscoveryInfoValue,TkSTLAllocatorShim<DiscoveryResolver::DiscoveryInfo::DiscoveryInfoValue,8,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCreatureOwnership::SetCustomCreatureName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerCreatureOwnership::HatchPet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerCreatureOwnership::AddEggToColection": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_ulonglong,  # const sOwnedCreatureInfo *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcPlayerCreatureOwnership::GetEggInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sOwnedCreatureInfo *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerCreatureOwnership::IsPetInNativeBiome": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_ulonglong,  # const sOwnedCreatureInfo *
        ]
    ),
    "cGcPlayerCreatureOwnership::PassedEnoughTimeSinceLastEgg": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_ulonglong,  # const sOwnedCreatureInfo *
        ]
    ),
    "cGcPlayerCreatureOwnership::GetTimeUntilNextEgg": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_ulonglong,  # const sOwnedCreatureInfo *
        ]
    ),
    "cGcPlayerCreatureOwnership::HasPendingEggInTheWorld": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
        ]
    ),
    "cGcPlayerCreatureOwnership::GetStaminaModifiers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcPlayerCreatureOwnership::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
        ]
    ),
    "cGcPlayerDiscoveryHelper::TriggerEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDiscoveryHelper *
            ctypes.c_ulonglong,  # const cGcPlayerDiscoveryHelper::DiscoveryHelperEvent *
        ]
    ),
    "cGcPlayerDiscoveryHelper::DiscoverPlanet": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDiscoveryHelper *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcPlayerDiscoveryHelper::RecordPlanetStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDiscoveryHelper *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerDiscoveryHelper::DiscoverSystem": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDiscoveryHelper *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcPlayerDiscoveryHelper::DiscoveryUnitReward": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDiscoveryHelper *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerDiscoveryHelper::GetDiscoveryWorth": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDiscoveryHelper *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcPlayerDiscoveryHelper::GenerateCreatureMissions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDiscoveryHelper *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # std::vector<int,TkSTLAllocatorShim<int,4,-1> > *
        ]
    ),
    "cGcPlayerDiscoveryHelper::CheckEncyclopediaData": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cGcResourceElement *
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['cGcEncyclopediaComponentData const *'], 'pointer_depth': 1}
        ]
    ),
    "cGcPlayerDiscoveryHelper::CalculateCompletedDiscoveriesOnPlanet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBitArray<unsigned int,1,32> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned int,1,32> *
            ctypes.c_uint32,  # const eDiscoveryType
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # cGcDiscoverySearch **
        ]
    ),
    "cGcPlayerDiscoveryHelper::SubmitCurrentSolarSystemDiscoveryData": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDiscoveryHelper *
        ]
    ),
    "cGcPlayerDiscoveryHelper::ShowPlanetDiscoveryMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDiscoveryHelper *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayerDiscoveryHelper::ShowPlanetInfoPanel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDiscoveryHelper *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayerDiscoveryHelper::ShowSystemDiscoveryMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDiscoveryHelper *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayerDiscoveryHelper::ShowPlayerBaseDiscoveryMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDiscoveryHelper *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayerDiscoveryHelper::ShowBlackHoleDiscoveryMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDiscoveryHelper *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayerDiscoveryHelper::ShowAtlasStationDiscoveryMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDiscoveryHelper *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayerDiscoveryHelper::ShowPostBlackHoleWarpDiscoveryMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDiscoveryHelper *
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "DiscoveryResolver::ComputeDisplayNameAndOwnerForDiscovery": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # DiscoveryResolver::NameAndOwnerResult *
        ]
    ),
    "DiscoveryResolver::NameAndOwnerResult::GetOwnershipDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DiscoveryResolver::NameAndOwnerResult *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cTkStringUtilities<char>::FormatNumberLocalised": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cTkStringUtilities<char>::IsEmpty": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkNGuiEditor::DoEditInt<unsigned __int64>": {
        "cTkNGuiEditor *, const char *, unsigned __int64 *, bool, float": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiEditor *
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # unsigned __int64 *
                ctypes.c_ubyte,  # bool
                ctypes.c_float,  # float
            ]
        ),
        "cTkNGuiEditor *, unsigned __int64 *, bool, float, bool": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiEditor *
                ctypes.c_ulonglong,  # unsigned __int64 *
                ctypes.c_ubyte,  # bool
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cTkMetaData::ReadFromFile<cGcPetAccessoryTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPetAccessoryTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcPetBehaviourTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPetBehaviourTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcPetEggSpeciesOverrideTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEntitlementRewardsTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditor::DoComboBox<eCreatureRole>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eCreatureRole *
            ctypes.c_float,  # float
        ]
    ),
    "DiscoveryResolver::DiscoveryInfo::DiscoveryInfoValue::DiscoveryInfoValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DiscoveryResolver::DiscoveryInfo::DiscoveryInfoValue *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkComboBoxPopup::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComboBoxPopup *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char **
            ctypes.c_ulonglong,  # __int64 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "GcDiscoverySearchConstraints::FilterOnType::Accept": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # GcDiscoverySearchConstraints::FilterOnType *
            ctypes.c_ulonglong,  # const cGcDiscoveryRecord *
        ]
    ),
    "GcDiscoverySearchConstraints::FilterPlanet::Accept": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # GcDiscoverySearchConstraints::FilterPlanet *
            ctypes.c_ulonglong,  # const cGcDiscoveryRecord *
        ]
    ),
    "cGcSolarSystem::GetAnomalyType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcPlayerCreatureOwnership::GetSpecialChatType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerCreatureOwnership::GetCurrentCreatureStableTimeSeed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSeed *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcEcosystem::GetCreatureData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystem *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerCreatureOwnership::GetMoodValue": {
        "cGcPlayerCreatureOwnership *, int, ePetMood, bool": FUNCDEF(
            restype=ctypes.c_double,  # double
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # ePetMood
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcPlayerCreatureOwnership *, ePetMood, bool": FUNCDEF(
            restype=ctypes.c_double,  # double
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
                ctypes.c_uint32,  # ePetMood
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "sOwnedCreatureInfo::GetGrowthProgress": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # sOwnedCreatureInfo *
        ]
    ),
    "cTkModelResourceRenderer::SetFurAllowed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelResourceRenderer *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "sOwnedCreatureInfo::GetSeed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSeed *
        argtypes=[
            ctypes.c_ulonglong,  # sOwnedCreatureInfo *
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureComponent::HasFur": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::IsBaby": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::GetScaler": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcFrontendPage::DebounceHoverPopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cTkInputManager::IsHandControlActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "cGcFrontendPage::Delete": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # FrontendHold
        ]
    ),
    "cGcPlayerBasePersistentBuffer::PlayerBasePersistentData::PlayerBasePersistentData": {
        "cGcPlayerBasePersistentBuffer::PlayerBasePersistentData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer::PlayerBasePersistentData *
            ]
        ),
        "cGcPlayerBasePersistentBuffer::PlayerBasePersistentData *, const cGcPersistentBaseEntry *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer::PlayerBasePersistentData *
                ctypes.c_ulonglong,  # const cGcPersistentBaseEntry *
            ]
        ),
    },
    "cGcPlayerBasePersistentBuffer::PlayerBasePersistentData::ClearObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer::PlayerBasePersistentData *
        ]
    ),
    "cGcMaintenanceBuffer::CleanUpValidSlotIndexes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceBuffer *
        ]
    ),
    "cGcPlayerBasePersistentBuffer::GenerateHashValue": FUNCDEF(
        restype=ctypes.c_uint16,  # unsigned __int16
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerBasePersistentBuffer::~cGcPlayerBasePersistentBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcLinkGrid,8,-1> *
        ]
    ),
    "cGcFreighterBaseComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcTriggerActionComponent::Broadcast": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcNetworkFleetSyncRequestMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkFleetSyncRequestMessage *
        ]
    ),
    "cGcPurchasableFrigateTracker::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchasableFrigateTracker *
        ]
    ),
    "cGcFrigateOccupationTracker::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateOccupationTracker *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerFleetManager::SubstituteUADetailsInString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFixedString<2048,char> *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
        ]
    ),
    "cGcPlayerFleetManager::GetClosestExpeditionDurationFromNumberOfEvents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerFleetManager::cGcPlayerFleetManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::OnLocalPlayerDied": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::IsRemotelyOwned": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::GenerateFleetDebug": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::SortFrigateList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_uint32,  # eExpeditionCategory
        ]
    ),
    "cGcPlayerFleetManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcPlayerFleetManager::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcExpeditionEventResult,8,-1> *
        ]
    ),
    "cGcPlayerFleetManager::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::CacheShips": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::Save": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcPlayerFleetManager::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const cGcPlayerStateData *
        ]
    ),
    "cGcPlayerFleetManager::UpdateFrigateSpawnLocations": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::SetShouldAlertPlayerToPurchasableFrigates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerFleetManager::GetShouldAlertPlayerToPurchasableFrigates": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::CanRecruitNewFrigates": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::GetNumberOfFrigates": FUNCDEF(
        restype=ctypes.c_int64,  # signed __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::GetNumberOfFrigatesNotOnActiveExpedition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::AddNewFrigateToFleet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::array<cGcFleetFrigate,30> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # eFrigateClass
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_ulonglong,  # const cTkFixedString<32,char> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerFleetManager::AddPurchasedFrigateToFleet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::array<cGcFleetFrigate,30> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # eFrigateClass
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_ulonglong,  # const cTkFixedString<32,char> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerFleetManager::RequestSetFrigateExpedition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcPlayerFleetManager::RequestRemoveFrigateFromFleet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcPlayerFleetManager::GetFrigateIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const cGcFleetFrigate *
        ]
    ),
    "cGcPlayerFleetManager::GetFrigate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFleetFrigate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerFleetManager::GetFrigateFromSeed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFleetFrigate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcPlayerFleetManager::GetFrigateOccupiedByLocalPlayer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFleetFrigate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::GetFrigateAssignedToPirateDefence": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFleetFrigate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const cGcPlayerCommon *
        ]
    ),
    "cGcPlayerFleetManager::IsPointInsideFrigate": {
        "cGcPlayerFleetManager *, const cTkVector3 *, const cGcFleetFrigate *, float": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerFleetManager *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cGcFleetFrigate *
                ctypes.c_float,  # float
            ]
        ),
        "cGcPlayerFleetManager *, const cTkVector3 *, const cGcFleetFrigate **": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerFleetManager *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cGcFleetFrigate **
            ]
        ),
    },
    "cGcPlayerFleetManager::GetFrigateFromAIShipSpawn": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFleetFrigate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const cGcAIShipSpawn *
        ]
    ),
    "cGcPlayerFleetManager::GetFrigateFromNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFleetFrigate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcPlayerFleetManager::GetAvailableFrigateWithHighestStat": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFleetFrigate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_uint32,  # eFrigateClass
            ctypes.c_uint32,  # eFrigateStatType
        ]
    ),
    "cGcPlayerFleetManager::RequestAssignFrigateToPirateDefence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint32,  # eFrigateStatType
        ]
    ),
    "cGcPlayerFleetManager::RequestCollectCargo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerFleetManager::GetStoredIncome": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_uint32,  # eFrigateClass
        ]
    ),
    "cGcPlayerFleetManager::ClearStoredIncome": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_uint32,  # eFrigateClass
        ]
    ),
    "cGcPlayerFleetManager::GetTrait": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcFrigateTraitData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerFleetManager::CreateExpeditionInternal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgModelNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_uint32,  # eExpeditionCategory
            ctypes.c_uint32,  # eExpeditionDuration
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcPlayerFleetManager::ChooseNewTraitForFrigate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcFrigateTraitData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const cGcFleetFrigate *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerFleetManager::RequestSelectExpedition": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcPlayerFleetManager::RequestStartExpedition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerFleetManager::RequestEndExpedition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcPlayerFleetManager::GetNumberOfActiveExpeditions": FUNCDEF(
        restype=ctypes.c_int64,  # signed __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::GetActiveExpedition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFleetExpedition *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerFleetManager::GetAvailableExpeditionTerminal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcPlayerFleetManager::GetExpeditionTerminalFromPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcExpeditionHologramComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcPlayerFleetManager::GetExpeditionFromTerminalPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # __m128 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcPlayerFleetManager::GetExpeditionFromSeed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFleetExpedition *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcPlayerFleetManager::GetExpeditionInUA": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFleetExpedition *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcPlayerFleetManager::ChooseNewUAForExpedition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcPlayerFleetManager::RequestRemoveExpedition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcPlayerFleetManager::RequestContinueWithDamagedFrigate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcPlayerFleetManager::RequestSendDamagedFrigateHome": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcPlayerFleetManager::AlertPlayerToInterventionEvent": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_ulonglong,  # cGcExpeditionEventResult *
        ]
    ),
    "cGcPlayerFleetManager::OnPlayerDeclinedIntervention": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::OnPlayerAcceptedIntervention": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerFleetManager::GetExpeditionInterventionEvent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcExpeditionInterventionEventData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcPlayerFleetManager::GetExpeditionEvent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcExpeditionEventData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcPlayerFleetManager::ChooseExpeditionEvent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcExpeditionEventData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_uint32,  # eExpeditionCategory
        ]
    ),
    "cGcPlayerFleetManager::SpawnShipForFrigate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerFleetManager::DespawnShipForFrigate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcPlayerFleetManager::SpawnFleet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcExpeditionEventResult,8,-1> *
        ]
    ),
    "cGcPlayerFleetManager::SpawnFleetImmediately": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::DespawnFleet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcExpeditionEventResult,8,-1> *
        ]
    ),
    "cGcPlayerFleetManager::CanPurchaseFrigate": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # eFrigateClass
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerFleetManager::IsPrimaryCategoryForExpeditionEvent": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const cGcExpeditionEventData *
            ctypes.c_uint32,  # eExpeditionCategory
        ]
    ),
    "cGcPlayerFleetManager::ApplyFleetFrigatesSyncMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const std::shared_ptr<cGcNetworkFleetFrigatesSyncMessage const > *
        ]
    ),
    "cGcPlayerFleetManager::OnRemotePlayerRequestedFleetSync": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcPlayerFleetManager::AddFrigateToFleet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::array<cGcFleetFrigate,30> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # eFrigateClass
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_ulonglong,  # const cTkFixedString<32,char> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerFleetManager::ShuffleFrigateTraitsWithProduct": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
        ]
    ),
    "cGcPlayerFleetManager::CreateFrigate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::array<cGcFleetFrigate,30> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::UpdateAvailableExpeditions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcExpeditionEventResult,8,-1> *
        ]
    ),
    "cGcPlayerFleetManager::SpawnShipForFrigateImmediately": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerFleetManager::DespawnShipForFrigateImmediately": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetFrigate *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerFleetManager::RemoveFrigateFromFleetImmediately": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcPlayerFleetManager::SpawnFrigateForPirateDefence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetFrigate *
        ]
    ),
    "cGcPlayerFleetManager::SelectExpedition": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcPlayerFleetManager::RemoveExpedition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcPlayerFleetManager::GetSequentialExpeditionEvent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcExpeditionEventData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_uint32,  # eExpeditionCategory
        ]
    ),
    "cGcPlayerFleetManager::GetExpeditionSpawnPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcPlayerFleetManager::UpdateActiveExpeditions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcFleetExpedition **
        ]
    ),
    "cGcPlayerFleetManager::SetupAsteroidInvalidArea": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
        ]
    ),
    "cGcPlayerFleetManager::UpdateForCurrentUAAndDay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcExpeditionEventResult,8,-1> *
        ]
    ),
    "cGcPlayerFleetManager::OnNetworkPlayerStateEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint32,  # eGcNetworkPlayerStateEvent
        ]
    ),
    "cGcPlayerFleetManager::OnRemotePlayerRepairedFrigate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            common.GcSeed,  # cTkSeed
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerFleetManager::OnRemotePlayerRequestContinueWithDamagedFrigate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcPlayerFleetManager::OnRemotePlayerRequestSendDamagedFrigateHome": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcPlayerFleetManager::OnRemotePlayerCollectedCargo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcPlayerFleetManager::OnRemotePlayerStartExpedition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            common.GcSeed,  # cTkSeed
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerFleetManager::OnRemotePlayerEndExpedition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcPlayerFleetManager::OnRemoteInterventionResponse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cGcPlayerFleetManager::OnRemotePlayerRemovedFrigate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcPlayerFleetManager::OnRemotePlayerSelectedExpedition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            common.GcSeed,  # cTkSeed
            ctypes.c_char * 0x10,  # cTkVector3
        ]
    ),
    "cGcPlayerFleetManager::OnRemotePlayerRemovedExpedition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcPlayerFleetManager::OnRemotePlayerSetFrigateExpedition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            common.GcSeed,  # cTkSeed
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcPlayerFleetManager::OnExpeditionChangedUA": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            common.GcSeed,  # cTkSeed
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerFleetManager::OnRemotePlayerAssignedFrigateToPirateDefence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_char * 0x10,  # cTkVector3
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcPlayerFleetManager::OnRemotePlayerSpawnedPirateDefence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcPlayerFleetManager::OnRemotePlayerUnassignedFrigateFromPirateDefence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcPlayerFleetManager::GetRandomEventLocID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFleetManager *
            ctypes.c_ulonglong,  # TkID<256> *
            common.GcSeed,  # cTkSeed
            ctypes.c_uint32,  # eExpeditionCategory
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOutpostComponent::IsDockEmpty": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerFreighterOwnership::SetFreighterOwnershipSyncMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ulonglong,  # const cGcNetworkFreighterOwnershipSyncMessage *
        ]
    ),
    "cGcPlayerFreighterOwnership::IsRemotelyOwned": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcPlayerFreighterOwnership::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerFreighterOwnership::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::CacheFreighter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::FixNPCResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ulonglong,  # const cGcPlayerStateData *
        ]
    ),
    "cGcPlayerFreighterOwnership::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayerFreighterOwnership::RequestFreighterSpawn": {
        "cGcPlayerFreighterOwnership *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ]
        ),
        "cGcPlayerFreighterOwnership *, const cTkMatrix34 *, const cGcUniverseAddressData *, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
                ctypes.c_ulonglong,  # const cTkMatrix34 *
                ctypes.c_ulonglong,  # const cGcUniverseAddressData *
                ctypes.c_float,  # float
            ]
        ),
    },
    "cGcPlayerFreighterOwnership::CallFreighter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::CreateTeleporterEndpoint": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ulonglong,  # cGcTeleportEndpoint *
        ]
    ),
    "cGcPlayerFreighterOwnership::SetTeleporterEndpoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::DespawnFreighter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerFreighterOwnership::GetFreighterSpawnPostion": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcPlayerFreighterOwnership::SpawnNewFreighter": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ulonglong,  # cGcUniverseAddressData *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcPlayerFreighterOwnership::DestroyFreighter": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerFreighterOwnership::SetCustomFreighterName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcPlayerFreighterOwnership::GetFreighterName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcPlayerFreighterOwnership::GetFreighterNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcPlayerFreighterOwnership::SetFreighterLocation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ulonglong,  # const cGcUniverseAddressData *
        ]
    ),
    "cGcPlayerFreighterOwnership::GetFreighterResourceWriteable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcResourceElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcCameraManager::GetGalacticCamera": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticNavigation *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
        ]
    ),
    "cGcPlayerFreighterOwnership::GetFreighterComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::GetFreighterBaseComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::SetFreighterHomeSystemSeed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcPlayerFreighterOwnership::GetFreighterSeed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSeed *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ulonglong,  # cTkSeed *
        ]
    ),
    "cGcPlayerFreighterOwnership::GetOwnerId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::OnRemotePlayerSwitchedFreighter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_char * 0x10,  # cTkVector3
        ]
    ),
    "cGcExperienceSpawn::cGcExperienceSpawn": {
        "cGcExperienceSpawn *, cGcExperienceSpawn *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcExperienceSpawn *
                ctypes.c_ulonglong,  # cGcExperienceSpawn *
            ]
        ),
        "cGcExperienceSpawn *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcExperienceSpawn *
            ]
        ),
    },
    "cGcPlayerFreighterOwnership::SwitchPlayerFreighter": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerFreighterOwnership::ResetPlayerFreighterBase": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::SetFreighterMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcPlayerFreighterOwnership::GetFreighterLocation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcUniverseAddressData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::ClaimFreighterBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::OwnsFreighter": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::ExistsInCurrentSystem": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::IsInCurrentSystem": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::IsFreighterInventoryAccessible": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerFreighterOwnership::CanConsiderSummoningFreighter": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::CanConsiderDismissingFreighter": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::CanSummonFreighter": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcPlayerFreighterOwnership::RequestFreighterDespawn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::UpdateRaceAndNPCResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::CreatePlayerFreighterGenerationTask": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcProceduralTextureManager::TaskContext *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::ApplyFreighterOwnershipSyncMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::OnNetworkPlayerStateEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint32,  # eGcNetworkPlayerStateEvent
        ]
    ),
    "cGcPlayerFreighterOwnership::OnAmbientFreighterBaseRestructure": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_char * 0x10,  # cTkVector3
        ]
    ),
    "cGcPlayerFreighterOwnership::GetFreighterInteriorMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerFreighterOwnership::UpdateOwnershipsActiveInSystems": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcPlayerFreighterOwnership::DespawnFreighterWithFleet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerFreighterOwnership::GenerateFreighterReplicatedCustomColourOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_ulonglong,  # cGcNetReplicatedFreighterColourData *
        ]
    ),
    "cTkDynamicArray<cGcFleetExpeditionSaveData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFleetExpeditionSaveData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcFleetFrigateSaveData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFleetFrigateSaveData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<__int64>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<unsigned __int64> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcFrigateTraitTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFrigateTraitTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcExpeditionEventTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcExpeditionEventTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcExpeditionRewardTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcExpeditionRewardTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditor::DoComboBox<int>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
            ctypes.c_float,  # float
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcNPCSpawnTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCSpawnTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "nanoflann::KDTreeSingleIndexAdaptor<nanoflann::L2_Simple_Adaptor<float,cTkVector3MetaKD<unsigned __int64,23>,float>,cTkVector3MetaKD<unsigned __int64,23>,3,unsigned __int64,23>::findNeighbors<nanoflann::KNNResultSet<float,unsigned __int64,unsigned __int64>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nanoflann::KDTreeSingleIndexAdaptor<nanoflann::L2_Simple_Adaptor<float,cTkVector3MetaKD<unsigned __int64,23>,float>,cTkVector3MetaKD<unsigned __int64,23>,3,unsigned __int64,23> *
            ctypes.c_ulonglong,  # nanoflann::KNNResultSet<float,unsigned __int64,unsigned __int64> *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const nanoflann::SearchParams *
        ]
    ),
    "cTkResourceDescriptor::FromString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_char_p,  # const char *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cTkResourceDescriptor::GetSize": FUNCDEF(
        restype=ctypes.c_int64,  # signed __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "cTkResourceDescriptor::GetDescriptor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_ulonglong,  # TkID<256> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcResourceCustomisation::SetBaseColours": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_ulonglong,  # const cGcPlanetColourData *
        ]
    ),
    "cTkRosterPool<cGcAIShipSpawn,128>::Get": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAIShipSpawn *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRosterPool<cGcAIShipSpawn,128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkRosterPool<cGcAIShipSpawn,128>::Size": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRosterPool<cGcAIShipSpawn,128> *
        ]
    ),
    "CAkMidiDeviceCtx::GetMidiMgr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcScannableComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannableComponent *
        ]
    ),
    "cTkSphere::GetCubeMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkNetReplicatedEntityComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPhysicsComponent::GetWorldUp": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cTkPhysicsComponent::GetGraphicsMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcAISpaceshipComponent::IsAboutToWarp": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAsteroidManager::RemoveInvalidArea": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidManager *
            ctypes.c_uint64,  # cTkClassPoolHandle
        ]
    ),
    "cGcSolarSystem::GetSpaceStationSphere": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSphere *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # cTkSphere *
        ]
    ),
    "cTkNetReplicatedEntityComponent::AllowComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipManager::CreateSquad": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPoolHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # cTkClassPoolHandle *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcPurchaseableItem::GetNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cTkFibonacciSphere::Generate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cTkVector3,TkSTLAllocatorShim<cTkVector3,16,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<cTkVector3,TkSTLAllocatorShim<cTkVector3,16,-1> > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_double,  # double
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSpaceshipWarp::GetTargetTeleportEndpoint": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
            ctypes.c_ulonglong,  # cGcTeleportEndpoint *
        ]
    ),
    "cGcFrontendPage::GetPurchaseableItem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPurchaseableItem *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcPlayerCommunicator::HasSignal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicator *
        ]
    ),
    "cGcPortalComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcPortalComponent::GetTypedComponent": {
        "int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_int32,  # int
            ]
        ),
        "cTkAttachment *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
    },
    "cGcRealityManager::LookUpNumberOfAlienInteractions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_uint32,  # eInteractionType
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRealityManager::GetMaxProgressionIndexOfGenericAlienInteractions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_uint32,  # eInteractionType
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRealityManager::GetPlayerWeaponPropertiesData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcPlayerWeaponPropertiesData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_uint32,  # eWeaponMode
        ]
    ),
    "cGcRealityManager::LoadTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTextureResource *
        ]
    ),
    "cGcRealityManager::LoadModel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelResource *
        ]
    ),
    "cGcRealityManager::LookupAlienInteractions_Internal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cGcAlienPuzzleEntry const *,TkSTLAllocatorShim<cGcAlienPuzzleEntry const *,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_ulonglong,  # std::vector<cGcAlienPuzzleEntry const *,TkSTLAllocatorShim<cGcAlienPuzzleEntry const *,8,-1> > *
            ctypes.c_uint32,  # eInteractionType
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_ulonglong,  # std::vector<enum eAlienPuzzleCategory,TkSTLAllocatorShim<enum eAlienPuzzleCategory,4,-1> > *
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcPlayerLogBook::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerLogBook *
        ]
    ),
    "cGcPlayerMultitoolOwnership::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMultitoolOwnership *
        ]
    ),
    "cGcPlayerMultitoolOwnership::LoadMultitool": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMultitoolOwnership *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcMultitoolData *
        ]
    ),
    "cGcPlayerMultitoolOwnership::SaveMultitool": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMultitoolOwnership *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcMultitoolData *
        ]
    ),
    "cGcPlayerMultitoolOwnership::GetPrimaryMultitoolIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMultitoolOwnership *
        ]
    ),
    "cGcPlayerMultitoolOwnership::GetNumberOfMultitools": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMultitoolOwnership *
        ]
    ),
    "cGcPlayerMultitoolOwnership::AddMultitoolToCollection": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMultitoolOwnership *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerMultitoolOwnership::UpdateInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMultitoolOwnership *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcPlayerMultitoolOwnership::GetThumbnail": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMultitoolOwnership *
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerMultitoolOwnership::GetName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMultitoolOwnership *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcPlayerMultitoolOwnership::SetPrimaryMultitoolIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMultitoolOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerMultitoolOwnership::sMultitoolInfo::sMultitoolInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMultitoolOwnership::sMultitoolInfo *
        ]
    ),
    "cGcPlayerNPCWorkers::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNPCWorkers *
        ]
    ),
    "cGcPlayerNPCWorkers::TrySetWokerBaseIndexFromSaveData": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNPCWorkers *
            ctypes.c_ulonglong,  # const cGcNPCWorkerData *
            ctypes.c_uint32,  # const eNPCHabitationType
        ]
    ),
    "cGcPlayerNPCWorkers::GetHabitationTypeFromInteractionType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNPCWorkers *
            ctypes.c_uint32,  # eInteractionType
        ]
    ),
    "cGcPlayerNPCWorkers::FindClosestBaseForWorker": FUNCDEF(
        restype=ctypes.c_ulonglong,  # BaseIndex *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNPCWorkers *
            ctypes.c_ulonglong,  # BaseIndex *
            ctypes.c_uint32,  # eNPCHabitationType
        ]
    ),
    "cGcPlayerNPCWorkers::HasWorkerStation": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNPCWorkers *
            ctypes.c_uint32,  # eNPCHabitationType
        ]
    ),
    "cGcPlayerNPCWorkers::HireWorker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNPCWorkers *
            ctypes.c_uint32,  # eNPCHabitationType
            ctypes.c_ulonglong,  # const cGcResourceElement *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcPlayerNPCWorkers::SetWorkerBaseIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNPCWorkers *
            ctypes.c_uint32,  # eNPCHabitationType
            ctypes.c_ulonglong,  # unsigned __int16 *
        ]
    ),
    "cGcShipOwnershipComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcShipOwnershipComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcShipOwnershipComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipOwnershipComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipOwnershipComponent *
        ]
    ),
    "cGcShipOwnershipComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipOwnershipComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcShipOwnershipComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcShipOwnershipComponent *
        ]
    ),
    "ComponentInstancercGcShipOwnershipComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcShipOwnershipComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentLoadInstancercGcShipOwnershipComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentLoadInstancercGcShipOwnershipComponent *
        ]
    ),
    "cGcOutpostComponent::GetClosestDock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcOutpostComponent::GetDockingShipData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcOutpostDockNode::DockingShipData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcOutpostComponent::GetDockState": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcOutpostDockNode::DockingShipData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcPlayerNotifications::HasNotificationSequenceInProgress": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcPlayerNotifications::GetPreviousNotificationSequence": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNotificationSequence *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcPlayerNotifications::IsMissionCompleted": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcPlayerShipOwnership::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcPlayerShipOwnership::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerShipOwnership::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
        ]
    ),
    "cGcPlayerShipOwnership::GetPrimaryShip": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
        ]
    ),
    "cGcPlayerShipOwnership::SpawnNewShip": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_uint32,  # const cGcPlayerShipOwnership::ShipSpawnLandingGearState
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerShipOwnership::DestroyShip": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerShipOwnership::SwitchPlayerShip": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerShipOwnership::TakeOwnership": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership::sGcShipData *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerShipOwnership::GetShipIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_ulonglong,  # const cTkAttachment *
        ]
    ),
    "cGcPlayerShipOwnership::GetShipComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerShipOwnership::GetShipData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcShipOwnershipComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerShipOwnership::GetNumShipsWithWorkingEngines": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerShipOwnership::ShipHasWorkingEngineIgnoreEverythingIsFree": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerShipOwnership::CanSalvageShip": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerShipOwnership::GetLivingShipInventoryIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
        ]
    ),
    "cGcPlayerShipOwnership::GetShipMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerShipOwnership::CacheShipFreighterDockIndexes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_ulonglong,  # const cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcPlayerShipOwnership::SetNextSummonIsFree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerShipOwnership::CreatePlayerShipGenerationTask": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcProceduralTextureManager::TaskContext *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerShipOwnership::GetPlayerShipName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerShipOwnership::RefreshSpaceshipComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerShipOwnership::UpdateMeshRefresh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
        ]
    ),
    "cGcPlayerShipOwnership::GetShipIndexFromNode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcPlayerShipOwnership::CanConsiderSummoningShip": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerShipOwnership::CanSummonShip": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerShipOwnership::RequestShipSpawn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
        ]
    ),
    "cGcPlayerShipOwnership::GetShipSummonPostion": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcShootableComponent::GetTypedComponent": {
        "cTkAttachment *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
        "int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcShootableComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
        ]
    ),
    "cGcShootableComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShootableComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
        ]
    ),
    "ComponentInstancercGcShootableComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcShootableComponent *
        ]
    ),
    "ComponentInstancercGcShootableComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcShootableComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcNetworkComponent::cGcNetworkComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkComponent *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cGcNetworkComponent::sSurvey,4,-1> *
        ]
    ),
    "ComponentInstancercGcShootableComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcPlayerSquadronOwnership::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
        ]
    ),
    "cGcPlayerSquadronOwnership::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
        ]
    ),
    "cGcPlayerSquadronOwnership::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayerSquadronOwnership::IsSquadronAvailable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
        ]
    ),
    "cGcPlayerSquadronOwnership::IsSquadronActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
        ]
    ),
    "cGcPlayerSquadronOwnership::TrySummonSquadron": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
        ]
    ),
    "cGcPlayerSquadronOwnership::RequestDismissSquadron": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSquadronOwnership::SummonSquadron": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
        ]
    ),
    "cGcPlayerSquadronOwnership::HasPilotInSlot": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerSquadronOwnership::SetPilotSlotUnlocked": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSquadronOwnership::RecruitPilot": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_ulonglong,  # cGcSquadronPilotData *
        ]
    ),
    "cGcPlayerSquadronOwnership::RetirePilot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerSquadronOwnership::HasUnusedPilotSlot": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
        ]
    ),
    "cGcPlayerSquadronOwnership::IsPilotInSquad": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcPlayerSquadronOwnership::GetPilotRace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerSquadronOwnership::GetSpaceshipName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerSquadronOwnership::GetPilotName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerSquadronOwnership::GetPilotRank": FUNCDEF(
        restype=ctypes.c_uint16,  # unsigned __int16
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerSquadronOwnership::GeneratePilotRank": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerSquadronOwnership::GetPilotTraitInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_ulonglong,  # std::vector<DiscoveryResolver::DiscoveryInfo::DiscoveryInfoValue,TkSTLAllocatorShim<DiscoveryResolver::DiscoveryInfo::DiscoveryInfoValue,8,-1> > *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerSquadronOwnership::GeneratePilotTraitInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<DiscoveryResolver::DiscoveryInfo::DiscoveryInfoValue,TkSTLAllocatorShim<DiscoveryResolver::DiscoveryInfo::DiscoveryInfoValue,8,-1> > *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerSquadronOwnership::GetSquadronSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
        ]
    ),
    "cGcPlayerSquadronOwnership::GetShipFollowOffset": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerSquadronOwnership::GetShipFollowPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerSquadronOwnership::SpawnSquadronShip": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerSquadronOwnership::GetSpawnMatrix": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcPlayerSquadronOwnership::UpdateSquadronSummoning": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayerSquadronOwnership::UpdateSquadronShipSpawning": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerSquadronOwnership::GetNumActiveShips": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
        ]
    ),
    "cGcPlayerSquadronOwnership::ReleaseSquadron": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronOwnership *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSquadronOwnership::GeneratePilotTraitGrade": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcPlayerSquadronOwnership::GeneratePilotTraitsMasterSeed": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcLootComponent::GetTypedComponent": {
        "cTkAttachment *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
        "int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcRepairTechComponent::QueryHasRepaired": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechComponent *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # cGcInventoryIndex
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRepairTechComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcRefinerUnitComponent::IsPersonalRefiner": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcPlayerState::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::LoadFromData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcPlayerStateData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerState::FindLastDifferentGalacticAddress": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerState *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcPlayerStateData *
        ]
    ),
    "cGcPlayerState::SetTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerState::AdjustStatFromInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # eStatsType
            ctypes.c_uint32,  # eStatsType
            ctypes.c_ulonglong,  # const cGcInventoryStore *
            ctypes.c_uint32,  # ItemLookupType
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerState::CalculateChoiceMap": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cGcPlayerState::InventoryChoiceMap,TkSTLAllocatorShim<cGcPlayerState::InventoryChoiceMap,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # std::vector<cGcPlayerState::InventoryChoiceMap,TkSTLAllocatorShim<cGcPlayerState::InventoryChoiceMap,8,-1> > *
        ]
    ),
    "cGcPlayerState::SaveToData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcPlayerState::ValidateReturnPortalMarkerPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::SetupInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # const eSizeType
            ctypes.c_ulonglong,  # const cGcInventoryLayout *
            ctypes.c_ulonglong,  # const cGcInventoryContainer *
            ctypes.c_uint32,  # const eTechnologyCategory
        ]
    ),
    "cGcPlayerState::GetPlayerNameWithTitle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcPlayerState::UnlockTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerState::UnlockTrophy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerState::AddItemsToInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryContainer *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcPlayerState::IsLocationSurveyed": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcPlayerState::FindExistingSettlementState": {
        "cGcPlayerState *, const unsigned __int64, const cTkVector3 *, const float": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkVector3 *
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_uint64,  # const unsigned __int64
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_float,  # const float
            ]
        ),
        "cGcPlayerState *, const unsigned __int64, const unsigned __int64": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcSettlementState *
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_uint64,  # const unsigned __int64
                ctypes.c_uint64,  # const unsigned __int64
            ]
        ),
    },
    "cGcPlayerState::FindOrAssignSettlementState": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcPlayerState::GetNextSettlementState": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSettlementState *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::GetKnownSettlementStatesInPlanet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const std::vector<cGcSettlementState const *,TkSTLAllocatorShim<cGcSettlementState const *,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const std::vector<cGcSettlementState const *,TkSTLAllocatorShim<cGcSettlementState const *,8,-1> > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerState::UpdateLocalSettlementStates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # const cTkFixedArray<cGcSettlementState,100> *
        ]
    ),
    "cGcPlayerState::UpdateLocalSettlementState": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSettlementState *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcSettlementState *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerState::IsUAInSameSystemAsPlayer": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerState::GetPlayerUniverseAddress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # cGcUniverseAddressData *
        ]
    ),
    "cGcPlayerState::GetPlayerUniverseAddressPacked": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::GetPlayerMatchmakingUniverseAddress": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::ChangeCurrentGalacticLocation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcGalacticAddressData *
        ]
    ),
    "cGcPlayerState::GoToRealityIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::MoveAndStack": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::MoveAndStackAllowed": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcPlayerState::PackageTechnology": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_uint32,  # TryStoreMode
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        ]
    ),
    "cGcPlayerState::DeployProduct": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_uint32,  # TryStoreMode
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
            ctypes.c_uint32,  # eShipClass
        ]
    ),
    "cGcPlayerState::GetRelevantProcTechTierFromInventory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcTechnology *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
        ]
    ),
    "cGcPlayerState::GetDeployTechFromProduct": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcTechnology *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerState::TryStoreInCurrentInventory": {
        "cGcPlayerState *, const cGcRealitySubstanceData *, int, TryStoreMode": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_ulonglong,  # const cGcRealitySubstanceData *
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # TryStoreMode
            ]
        ),
        "cGcPlayerState *, const cGcProductData *, int, TryStoreMode": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_ulonglong,  # const cGcProductData *
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # TryStoreMode
            ]
        ),
        "cGcPlayerState *, const cGcInventoryElement *, int, TryStoreMode": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # TryStoreMode
            ]
        ),
    },
    "cGcPlayerState::GetCurrentInventories": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
        ]
    ),
    "cGcPlayerState::TryStoreInInventory_Internal": {
        "cGcPlayerState *, int, const cGcInventoryElement *, cGcInventoryIndex, TryStoreMode": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_uint64,  # cGcInventoryIndex
                ctypes.c_uint32,  # TryStoreMode
            ]
        ),
        "cGcPlayerState *, cGcInventoryStore *, const cGcInventoryElement *, cGcInventoryIndex, TryStoreMode, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_ulonglong,  # cGcInventoryStore *
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_uint64,  # cGcInventoryIndex
                ctypes.c_uint32,  # TryStoreMode
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cGcPlayerState::TryStoreInInventory": {
        "cGcPlayerState *, int, const cGcInventoryElement *, cGcInventoryIndex, TryStoreMode": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcTechnology *
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_uint64,  # cGcInventoryIndex
                ctypes.c_uint32,  # TryStoreMode
            ]
        ),
        "cGcPlayerState *, int, const cGcRealitySubstanceData *, int, TryStoreMode": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # const cGcRealitySubstanceData *
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # TryStoreMode
            ]
        ),
        "cGcPlayerState *, int, const cGcProductData *, int, TryStoreMode": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # const cGcProductData *
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # TryStoreMode
            ]
        ),
    },
    "cGcPlayerState::TryStoreInMultipleInventories": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_uint32,  # TryStoreMode
        ]
    ),
    "cGcPlayerState::TryToRemoveFromSpecificInventory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_uint64,  # cGcInventoryIndex
            ctypes.c_uint32,  # TryStoreMode
        ]
    ),
    "cGcPlayerState::TryRemoveFromInventory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_uint64,  # cGcInventoryIndex
            ctypes.c_uint32,  # TryStoreMode
        ]
    ),
    "cGcPlayerState::TryRemoveFromAllInventories": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_uint32,  # TryStoreMode
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerState::TryRemoveIllegalItemsFromInventories": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
            ctypes.c_uint32,  # TryStoreMode
        ]
    ),
    "cGcPlayerState::IsInventoryFull": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::ClearInventoryHistories": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::GetNumFilledSlots": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcPlayerState::GetHealth": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::GetHealthPips": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::GetCurrentMaxHealthPips": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::GetHealthPipSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::GetApproxDefenseUpgradeLevel": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::GetApproxCombatUpgradeLevelShip": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::GetApproxCombatUpgradeLevel": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::GiveProductResources": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # TryStoreMode
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerState::GetEnergy": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::ModifyAndReturnHealth": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::RestoreStatus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerState::RestoreShipStatus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerState::GetMaximumHealth": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcShipOwnershipComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::GetMaximumPersonalHealth": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::GetMaximumShipHealth": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcShipOwnershipComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::GetMaximumShield": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::FindFirstRogueBoxItem": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcPlayerState::FindFirstCreatureEgg": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcPlayerState::QueryAnyInventoryHasDeployableTech": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::QueryInventoryHasProductOfType": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eProductCategory
        ]
    ),
    "cGcPlayerState::QueryInventoryHasEggModifierIngredient": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::QueryAmountInInventory": {
        "cGcPlayerState *, const InventoryChoice, const cGcInventoryElement *, bool, bool, int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_uint32,  # const InventoryChoice
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
                ctypes.c_int32,  # int
            ]
        ),
        "cGcPlayerState *, int, const eProceduralProductCategory, const eRarity": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # const eProceduralProductCategory
                ctypes.c_uint32,  # const eRarity
            ]
        ),
    },
    "cGcPlayerState::QueryAmountInInventories": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerState::QueryAmountInAllInventories": {
        "cGcPlayerState *, const cGcInventoryElement *, bool, bool, bool, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcPlayerState *, const eProceduralProductCategory, const eRarity": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_uint32,  # const eProceduralProductCategory
                ctypes.c_uint32,  # const eRarity
            ]
        ),
        "cGcPlayerState *, const TkID<128> *, bool, bool, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cGcPlayerState::QueryAnyInventoryHasItem": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcPlayerState::HasIllegalProductInAnyInventory": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcPlayerState::HasIllegalProduct": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcPlayerState::CollectAllMatchingInventoryElements": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<InventoryLookup,TkSTLAllocatorShim<InventoryLookup,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # std::vector<InventoryLookup,TkSTLAllocatorShim<InventoryLookup,8,-1> > *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerState::CollectAllMatchingInventoryElementsInInventories": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<InventoryLookup,TkSTLAllocatorShim<InventoryLookup,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # std::vector<InventoryLookup,TkSTLAllocatorShim<InventoryLookup,8,-1> > *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
        ]
    ),
    "cGcPlayerState::IsItemValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
        ]
    ),
    "cGcPlayerState::IsTechInstalled": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcTechnology *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcPlayerState::MatchesSpecialSeasonThreeTech": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerState::CanBuildTechnology": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcTechnology *
        ]
    ),
    "cGcPlayerState::PostTechInstalled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcTechnology *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcPlayerState::CanRepairTechnology": {
        "cGcPlayerState *, const cGcInventoryStore *, const InventoryChoice, const cGcInventoryIndex *, const bool": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcInventoryElement *
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_ulonglong,  # const cGcInventoryStore *
                ctypes.c_uint32,  # const InventoryChoice
                ctypes.c_ulonglong,  # const cGcInventoryIndex *
                ctypes.c_ubyte,  # const bool
            ]
        ),
        "cGcPlayerState *, const cGcInventoryElement *, const InventoryChoice, const bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_uint32,  # const InventoryChoice
                ctypes.c_ubyte,  # const bool
            ]
        ),
    },
    "cGcPlayerState::RepairTechnology": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcPlayerState::ChargeForFree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
        ]
    ),
    "cGcPlayerState::ChargeTechnology": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::TrashItem": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # cGcInventoryElement *
        ]
    ),
    "cGcPlayerState::TrashInventory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcLootComponent *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # std::vector<cGcInventoryElement,TkSTLAllocatorShim<cGcInventoryElement,8,-1> > *
        ]
    ),
    "cGcPlayerState::DismantleTechnology": {
        "cGcPlayerState *, cGcInventoryStore *, cGcInventoryStore *, const cGcInventoryElement *, const cGcInventoryIndex *, int": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_ulonglong,  # cGcInventoryStore *
                ctypes.c_ulonglong,  # cGcInventoryStore *
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_ulonglong,  # const cGcInventoryIndex *
                ctypes.c_int32,  # int
            ]
        ),
        "cGcPlayerState *, int, const cGcInventoryElement *, const cGcInventoryIndex *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ]
        ),
    },
    "cGcPlayerState::DismantleResults": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cGcDismantleResult,TkSTLAllocatorShim<cGcDismantleResult,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # std::vector<cGcDismantleResult,TkSTLAllocatorShim<cGcDismantleResult,8,-1> > *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcPlayerState::CanBuildProduct": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcProductData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_ulonglong,  # std::vector<cGcTechnologyRequirement,TkSTLAllocatorShim<cGcTechnologyRequirement,8,-1> > *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerState::CanBuildOne": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcTechnologyRequirement> *
            ctypes.c_ulonglong,  # const std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
            ctypes.c_ulonglong,  # std::unordered_map<TkID<128>,int,std::hash<TkID<128> >,std::equal_to<TkID<128> >,std::allocator<std::pair<TkID<128> const ,int> > > *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerState::GetNumBuildable": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcProductData *
        ]
    ),
    "cGcPlayerState::BuildProduct": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcProductData *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::TryBuildOne": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcTechnologyRequirement> *
            ctypes.c_ulonglong,  # const std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerState::BuildBuildingProduct": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcProductData *
        ]
    ),
    "cGcPlayerState::GetShipCargoValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # cGcInventoryElement *
        ]
    ),
    "cGcPlayerState::GetShipCargoIllegalItemsValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # cGcInventoryElement *
        ]
    ),
    "cGcPlayerState::GetNumSuitStartingSlots": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint32,  # PersonalInventoryIndex
        ]
    ),
    "cGcPlayerState::GetAssociatedInventories": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint32,  # InventoryChoice
            ctypes.c_ulonglong,  # std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
        ]
    ),
    "cGcPlayerState::GetInventoriesFor": {
        "cGcPlayerState *, int, std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcPlayerState *, InventoryChoice, std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_uint32,  # InventoryChoice
                ctypes.c_ulonglong,  # std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
            ]
        ),
    },
    "cGcPlayerState::GetStatBonusText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eStatsType
            ctypes.c_uint32,  # eStatsType
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcPlayerState::GetBaseStatAmount": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint32,  # eStatsType
            ctypes.c_uint32,  # eStatsType
        ]
    ),
    "cTkTextureResource::cTkTextureResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelResource *
            ctypes.c_ulonglong,  # const cTkModelResource *
        ]
    ),
    "cGcPlayerState::IsTechCoreShipWeapon": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTechnology *
        ]
    ),
    "cGcPlayerState::GetNumIntactCoreShipWeapons": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::CanDestroyPlayerTechItem": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
        ]
    ),
    "cGcPlayerState::CanPackagePlayerTechItem": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
        ]
    ),
    "cGcPlayerState::PlayerJumpedToNextGalaxy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::HasUAAtlasStation": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerState::IsUsingThirdPersonCharacterCamera": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::UseEntitlement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcPlayerState::CanStartSpaceBattle": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint32,  # cGcPlayerRespawn::RespawnReason
        ]
    ),
    "cGcPlayerState::StartSpaceBattle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::SetCustomWeaponName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcPlayerState::SetCustomShipName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::SetCustomVehicleName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::GetCurrentWeaponClass": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::GetCurrentShipClass": FUNCDEF(
        restype=ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'cTkComponent*', 'name': 'this'}]}
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::ChargeAllowed": {
        "cGcPlayerState *, const cGcTechnology *, const cGcRealitySubstanceData *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_ulonglong,  # const cGcTechnology *
                ctypes.c_ulonglong,  # const cGcRealitySubstanceData *
            ]
        ),
        "cGcPlayerState *, const cGcTechnology *, const cGcProductData *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_ulonglong,  # const cGcTechnology *
                ctypes.c_ulonglong,  # const cGcProductData *
            ]
        ),
    },
    "cGcPlayerState::CanChargeTechWithItem": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
        ]
    ),
    "cGcPlayerState::RemoveUnits": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::RemoveNanites": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::AwardUnits": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::AwardNanites": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::ApplyInitialKnownThingsPreset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcKnownThingsPreset *
        ]
    ),
    "cGcPlayerState::GetAdjacencyBonus": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ulonglong,  # const cGcStatsBonus *
        ]
    ),
    "cGcPlayerState::GetElementAdjacencyBonusAmount": {
        "cGcPlayerState *, const cGcInventoryStore *, const cGcInventoryElement *, AdjacencyBonus": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_ulonglong,  # const cGcInventoryStore *
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_uint32,  # AdjacencyBonus
            ]
        ),
        "cGcPlayerState *, const cGcInventoryStore *, const cGcInventoryElement *, const cGcStatsBonus *, AdjacencyBonus": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_ulonglong,  # const cGcInventoryStore *
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_ulonglong,  # const cGcStatsBonus *
                ctypes.c_uint32,  # AdjacencyBonus
            ]
        ),
    },
    "cGcPlayerState::IsElementStatChildOf": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
        ]
    ),
    "cGcPlayerState::GetTechColourFromInventory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
            ctypes.c_uint32,  # const eStatsType
            ctypes.c_ulonglong,  # cTkColour *
        ]
    ),
    "cGcPlayerState::GetElementLinkColour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcPlayerState::GetStatValue": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint32,  # eStatsType
            ctypes.c_uint32,  # ItemLookupType
            ctypes.c_uint32,  # eStatsType
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
        ]
    ),
    "cGcPlayerState::GetPrimaryItemIndexForStat": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryIndex *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # cGcInventoryIndex *
            ctypes.c_uint32,  # eStatsType
            ctypes.c_ulonglong,  # const cGcInventoryStore *
            ctypes.c_uint32,  # ItemLookupType
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcPlayerState::GetPrimaryItemForStatFromInventorySelection_Internal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryIndex *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # cGcInventoryIndex *
            ctypes.c_uint32,  # eStatsType
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # ItemLookupType
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # InventoryChoice *
        ]
    ),
    "cGcPlayerState::GetPrimaryItemForStat": {
        "cGcPlayerState *, eStatsType, InventoryChoice, ItemLookupType, int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # const cGcAtlasSendSubmitContribution *
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_uint32,  # eStatsType
                ctypes.c_uint32,  # InventoryChoice
                ctypes.c_uint32,  # ItemLookupType
                ctypes.c_int32,  # int
            ]
        ),
        "cGcPlayerState *, eStatsType, const cGcInventoryStore *, ItemLookupType": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcInventoryElement *
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_uint32,  # eStatsType
                ctypes.c_ulonglong,  # const cGcInventoryStore *
                ctypes.c_uint32,  # ItemLookupType
            ]
        ),
    },
    "cGcPlayerState::AwardTechnology": {
        "cGcPlayerState *, const cGcTechnology *, bool, bool": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_ulonglong,  # const cGcTechnology *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcPlayerState *, const TkID<128> *, bool, bool": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cGcPlayerState::AwardInventorySlot": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_uint64,  # cGcInventoryIndex
        ]
    ),
    "cGcPlayerState::AwardSuitSlot": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # cGcInventoryIndex
        ]
    ),
    "cGcPlayerState::AwardShipSlot": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # cGcInventoryIndex
        ]
    ),
    "cGcPlayerState::AwardWeaponSlot": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # cGcInventoryIndex
        ]
    ),
    "cGcPlayerState::AwardFreighterSlot": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # cGcInventoryIndex
        ]
    ),
    "cGcPlayerState::IsWordKnown": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # eAlienRace
        ]
    ),
    "cGcPlayerState::CanConverseWith": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcPlayerState::SeeBaseBuildingObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerState::LearnProductRecipe": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerState::UpdateRedeemedSpecialsAndRewards": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerState::RedeemSpecial": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerState::RedeemTwitchReward": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerState::RedeemPlatformReward": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerState::PromoteProductToFrontOfList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerState::SpawnPlayerGrave": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # std::vector<cGcInventoryElement,TkSTLAllocatorShim<cGcInventoryElement,8,-1> > *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerState::LearnWordGroup": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_uint32,  # eAlienRace
        ]
    ),
    "cGcPlayerState::RecordRaceWordStat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint32,  # eAlienRace
        ]
    ),
    "cGcPlayerState::ChangeFactionStanding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint32,  # eMissionFaction
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcPlayerState::GetRaceRankInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint32,  # eAlienRace
        ]
    ),
    "cGcPlayerState::GetFactionRankInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint32,  # eMissionFaction
        ]
    ),
    "cGcPlayerState::GetFactionStanding": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint32,  # eMissionFaction
        ]
    ),
    "cGcPlayerState::GetPlayerRaceRankString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const TryStoreMode *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::GetRaceRankString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const TryStoreMode *
        argtypes=[
            ctypes.c_uint32,  # const eAlienRace
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::GetFactionRankString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const TryStoreMode *
        argtypes=[
            ctypes.c_uint32,  # eMissionFaction
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::GetFactionMissionStat": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_uint32,  # eMissionFaction
        ]
    ),
    "cGcPlayerState::IncrementInteractionIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint32,  # eInteractionType
            ctypes.c_uint32,  # eAlienRace
        ]
    ),
    "cGcPlayerState::GetInventoryStoreFromChoice": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedArray<cGcInventoryStore,28> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::GetInventoryStoreFromChoiceWriteable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryStore *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint32,  # const InventoryChoice
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::GetTechElement": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # InventoryChoice *
        ]
    ),
    "cGcPlayerState::GetFreeShipIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::UpdateVehicleLocations": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::UpdateFinalisePurchaseState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::GetInventoryStoresFromBase": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
        ]
    ),
    "cGcPlayerState::GetCookingIngredientStoresFromBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
        ]
    ),
    "cGcPlayerState::UsesThirdPersonShipCam": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::NumRequiredForRepairOrInstall": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # cGcInventoryIndex
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcPlayerState::CalculateInventoryChoiceFromStore": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
        ]
    ),
    "cGcPlayerState::GetCurrentInventorySubIndexForChoice": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::HasInteractionDialog": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerState::ReleaseInteractionDialog": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerState::UpdateInteractionDialog": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcVehicleLocation::GetSpawnTransform": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleLocation *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcPlayerState::StoreCurrentSystemSpaceStationEndpoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::StoreTeleportEndpoint": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcTeleportEndpoint *
        ]
    ),
    "cGcPlayerState::RemoveTeleportEndpoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcTeleportEndpoint *
        ]
    ),
    "cGcPlayerState::GetTeleportEndpoint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint32,  # eTeleporterType
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcTeleportEndpoint *
            ctypes.c_ulonglong,  # cGcTeleportEndpoint *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerState::GetTeleportEndpointsVector": {
        "cGcPlayerState *, eTeleporterType, const cGcTeleportEndpoint *, std::vector<cGcTeleportEndpoint,TkSTLAllocatorShim<cGcTeleportEndpoint,16,-1> > *, float, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_uint32,  # eTeleporterType
                ctypes.c_ulonglong,  # const cGcTeleportEndpoint *
                ctypes.c_ulonglong,  # std::vector<cGcTeleportEndpoint,TkSTLAllocatorShim<cGcTeleportEndpoint,16,-1> > *
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcPlayerState *, const std::vector<enum eTeleporterType,TkSTLAllocatorShim<enum eTeleporterType,4,-1> > *, const cGcTeleportEndpoint *, std::vector<cGcTeleportEndpoint,TkSTLAllocatorShim<cGcTeleportEndpoint,16,-1> > *, float, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerState *
                ctypes.c_ulonglong,  # const std::vector<enum eTeleporterType,TkSTLAllocatorShim<enum eTeleporterType,4,-1> > *
                ctypes.c_ulonglong,  # const cGcTeleportEndpoint *
                ctypes.c_ulonglong,  # std::vector<cGcTeleportEndpoint,TkSTLAllocatorShim<cGcTeleportEndpoint,16,-1> > *
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cGcPlayerState::GetNumTeleportEndpoints": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint32,  # eTeleporterType
            ctypes.c_ulonglong,  # const cGcTeleportEndpoint *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerState::HasEndpointForEvent": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_uint32,  # eTeleporterType
            ctypes.c_ulonglong,  # const cGcTeleportEndpoint *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerState::CanTeleportToExternalBases": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::DoesTeleporterEndpointMatch": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcTeleportEndpoint *
            ctypes.c_ulonglong,  # const cGcTeleportEndpoint *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerState::ValidateSavedInventoryItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # cGcInventoryElement *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
        ]
    ),
    "cGcPlayerState::ValidateSavedTechItem": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # cGcInventoryElement *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcPlayerState::ShelveInventories": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::RestoreShelvedInventories": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::ClearIsOnOtherSideOfPortal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::IsOnOtherSideOfStoryPortal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::IsOnOtherSideOfCommunityPortal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::HasHyperdriveAccess": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eGalaxyStarType
        ]
    ),
    "cGcPlayerState::FindIllegalProductInInventory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # TkID<128> *
            common.GcSeed,  # const cTkSeed
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcPlayerState::FindCookingProductInInventory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # TkID<128> *
            common.GcSeed,  # const cTkSeed
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcPlayerState::FindProcProductInInventory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_uint32,  # const eProceduralProductCategory
            ctypes.c_uint32,  # const eRarity
        ]
    ),
    "cGcPlayerState::IsRewardRedeemedOrSpecialKnown": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerState::GetPendingRewardId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::CompleteClaimPendingReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::IsBlockedFromShipPurchase": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcPlayerState::SetStoryPageSeenIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
        ]
    ),
    "sPlayerTitleStatWatcher::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sPlayerTitleStatWatcher *
        ]
    ),
    "cTkMetadataCache<cGcCustomisationShipBobbleHeads>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCustomisationShipBobbleHeads *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcCustomisationShipBobbleHeads> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcCustomisationThrusterEffects>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCustomisationThrusterEffects *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcCustomisationThrusterEffects> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcCustomisationBannerGroup>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCustomisationBannerGroup *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcCustomisationBannerGroup> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcCustomisationColourPalettes>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCustomisationColourPalettes *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcCustomisationColourPalettes> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcCustomisationPresets>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCustomisationPresets *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcCustomisationPresets> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcCustomisationPresets>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cGcCustomisationPresets> *
        argtypes=[]
    ),
    "cTkReplicatedVariableTemplated<cTkVector3>::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cTkVector3> *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cTkDynamicArray<cGcPlayerWeaponBiomeProperties>::end": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkArraySTLIterator<cGcPlayerWeaponBiomeProperties>::InternalConstIterator *
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerWeaponBiomeProperties> *
            ctypes.c_ulonglong,  # cTkArraySTLIterator<cGcPlayerWeaponBiomeProperties>::InternalConstIterator *
        ]
    ),
    "cTkDynamicArray<cGcProtectedLocation>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcProtectedLocation> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcRepairTechData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRepairTechData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSavedEntitlement>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSavedEntitlement> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcUniverseAddressData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUniverseAddressData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSavedInteractionDialogData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSavedInteractionDialogData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSyncBufferSaveData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSyncBufferSaveData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkLSystemRestrictionData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcVROverride_Layout> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<TkID<256>>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<TkID<256> > *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcStatsEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStatsEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcCustomisationDescriptorGroups>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroups *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcCustomisationTextureOptions>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCustomisationTextureOptions *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSynchronisedStateManager::LoadBuffer<0>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager *
            ctypes.c_ulonglong,  # const cGcSyncBufferSaveData *
            ctypes.c_ulonglong,  # const sMaintenanceSaveKey *
            ctypes.c_ulonglong,  # const cGcMaintenanceContainer *
        ]
    ),
    "cTkLinearHashTable<unsigned __int64,TkID<256>,-1,0,cTkLinearHashTableHash<unsigned __int64>>::find<unsigned __int64>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkLinearHashTable<unsigned __int64,TkID<256>,-1,0,cTkLinearHashTableHash<unsigned __int64> >::const_iterator *
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<unsigned __int64,TkID<256>,-1,0,cTkLinearHashTableHash<unsigned __int64> > *
            ctypes.c_ulonglong,  # cTkLinearHashTable<unsigned __int64,TkID<256>,-1,0,cTkLinearHashTableHash<unsigned __int64> >::const_iterator *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcCustomisationShipBobbleHeads>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCustomisationShipBobbleHeads *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcCustomisationThrusterEffects>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCustomisationThrusterEffects *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcCustomisationBannerGroup>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCustomisationBannerGroup *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcCustomisationColourPalettes>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCustomisationColourPalettes *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcCustomisationPresets>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCustomisationPresets *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::SaveData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # std::vector<sMaintenanceSaveKey,TkSTLAllocatorShim<sMaintenanceSaveKey,16,-1> > *
            ctypes.c_ulonglong,  # std::vector<cGcMaintenanceContainer,TkSTLAllocatorShim<cGcMaintenanceContainer,8,-1> > *
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::Initialise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # const sMaintenanceSaveKey *
            ctypes.c_ulonglong,  # const cGcMaintenanceContainer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "LogBookMessage::LogBookMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # LogBookMessage *
            ctypes.c_int32,  # const int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcInventoryStore::GetFirstFreeSlot": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryIndex *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryIndex *
        ]
    ),
    "cGcScanEventManager::GetEventUA": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcPlayerState::IsWordGroupKnown": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_uint32,  # eAlienRace
        ]
    ),
    "cGcInventoryStore::GetMinMaxInventoryIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryIndex *
            ctypes.c_ulonglong,  # cGcInventoryIndex *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcInventoryStore::GetNextInvalidSlot": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcInventoryIndex *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
        ]
    ),
    "cTkColour::SetFromHSV": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkColour::GetAsHSV": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcStatsGroup::~cGcStatsGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsGroup *
        ]
    ),
    "cGcInventoryStore::CreateTechnologyElement": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcInventoryElement *
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ulonglong,  # const cGcTechnology *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcPlayerState::GetVehicleTypeFromInventory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
        ]
    ),
    "cGcInventoryStore::ClearEmptyItems": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
        ]
    ),
    "cGcInventoryStore::GetElementForEditing": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
        ]
    ),
    "cGcPlayerWeapon::SetAmmo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_uint32,  # eWeaponMode
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::IsProductRecipeKnown": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerState::SetStatusRestores": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # const cGcPlayerStateData *
        ]
    ),
    "cGcAISpaceshipComponent::IsWarping": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcPlayerExperienceDirector::ValidShipSpawn": {
        "cGcPlayerExperienceDirector *, const cTkVector3 *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
        "cGcPlayerExperienceDirector *, const cTkVector3 *, float *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # float *
            ]
        ),
    },
    "cTkMatrix34::VectorTransformMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkFibonacciSphere::GenerateCylinder": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cTkVector3,TkSTLAllocatorShim<cTkVector3,16,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<cTkVector3,TkSTLAllocatorShim<cTkVector3,16,-1> > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkAnimationComponent::SetAnimSpeed": {
        "cTkAnimationComponent *, float, const TkID<128> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkAnimationComponent *
                ctypes.c_float,  # float
                ctypes.c_char_p,  # const TkID<128> *
            ]
        ),
        "cTkAnimationComponent *, float, const cTkAnimInstanceHandle *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkAnimationComponent *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
            ]
        ),
    },
    "cTkAnimLayerHandle::IsValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimLayerHandle *
        ]
    ),
    "cTkAnimationComponent::SetAnimTime": {
        "cTkAnimationComponent *, float, const TkID<128> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkAnimationComponent *
                ctypes.c_float,  # float
                ctypes.c_char_p,  # const TkID<128> *
            ]
        ),
        "cTkAnimationComponent *, float, const cTkAnimInstanceHandle *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkAnimationComponent *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
            ]
        ),
    },
    "cTkAnimationComponent::HasAnim": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcSquadronPilotInfo::RequestLoadResources": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronPilotInfo *
        ]
    ),
    "cGcPlayerShipOwnership::ClearShipDataAtIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerShipOwnership::GetValidShipIndicies": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<int,TkSTLAllocatorShim<int,4,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_ulonglong,  # std::vector<int,TkSTLAllocatorShim<int,4,-1> > *
        ]
    ),
    "cGcPlayerState::SetPrimaryShip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAttachmentManager::FindFirstAttachmentInScene": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachment **
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcPlayerNPCWorkers::GetStationIDFromType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNPCWorkers *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_uint32,  # eNPCHabitationType
        ]
    ),
    "cTkComponentManager::Allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentManager *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcVehicleComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcVehicleComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequence::GetCurrentGroup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNotificationSequenceGroup *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcPlayerNotifications::SetCurrentNotificationSequence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # const cGcNotificationSequence *
        ]
    ),
    "cGcHUDManager::ClearMissionUpUpdateMessages": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<MissionMessage,16,-1> *
        ]
    ),
    "cGcTechnologyAttachmentComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyAttachmentComponent *
        ]
    ),
    "cGcTechnologyAttachmentComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyAttachmentComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcTechnologyAttachmentComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcTechnologyAttachmentComponent *
        ]
    ),
    "ComponentInstancercGcTechnologyAttachmentComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcRocketLockerComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcRocketLockerComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRocketLockerComponent *
        ]
    ),
    "cGcRocketLockerComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRocketLockerComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcRocketLockerComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcRocketLockerComponent *
        ]
    ),
    "ComponentInstancercGcRocketLockerComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcRocketLockerComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcRocketLockerComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcPlayerVehicleOwnership::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcPlayerVehicleOwnership::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
            ctypes.c_float,  # const float
            ctypes.c_int64,  # __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerVehicleOwnership::ClearPreviewNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
        ]
    ),
    "cGcPlayerVehicleOwnership::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
        ]
    ),
    "cGcPlayerVehicleOwnership::GetSpawnPostion": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
            ctypes.c_uint32,  # eVehicleType
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerVehicleOwnership::SpawnNewVehicle": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
            ctypes.c_uint32,  # eVehicleType
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerVehicleOwnership::DestroyVehicle": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
            ctypes.c_uint32,  # eVehicleType
        ]
    ),
    "cGcPlayerVehicleOwnership::IsSummonedAnywhere": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
            ctypes.c_uint32,  # eVehicleType
        ]
    ),
    "cGcPlayerVehicleOwnership::GetCurrentVehicleStatusText": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcPlayerVehicleOwnership::CanConsiderSummoning": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
            ctypes.c_uint32,  # eVehicleType
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerVehicleOwnership::CanSummon": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
            ctypes.c_uint32,  # eVehicleType
        ]
    ),
    "cGcPlayerVehicleOwnership::GetPlayerVehicleName": {
        "cGcPlayerVehicleOwnership *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
            ]
        ),
        "cGcPlayerVehicleOwnership *, eVehicleType": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
                ctypes.c_uint32,  # eVehicleType
            ]
        ),
    },
    "cGcPlayerVehicleOwnership::GetPlayerVehicleDefaultName": FUNCDEF(
        restype=ctypes.c_char_p,  # const TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
        ]
    ),
    "cGcPlayerVehicleOwnership::GetPrimary": FUNCDEF(
        restype=ctypes.c_ulonglong,  # VehicleInstance *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
        ]
    ),
    "cGcPlayerVehicleOwnership::CreatePlayerVehicleGenerationTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
            ctypes.c_uint32,  # eVehicleType
        ]
    ),
    "cGcPlayerVehicleOwnership::UpdateMeshRefresh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
        ]
    ),
    "cGcSeasonalData::OnSeasonSettingsDownloaded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSeasonalData::RequestCurrentSeasonData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
        ]
    ),
    "cGcSeasonalData::GenerateRewardDescriptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
        ]
    ),
    "cGcSeasonalData::RestoreServerSeasonalData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
        ]
    ),
    "cGcSeasonalData::SetSeasonalGameModeData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
            ctypes.c_ulonglong,  # const cGcSeasonalGameModeData *
        ]
    ),
    "cGcSeasonalData::IsCompatibleWithState": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
        ]
    ),
    "cGcSeasonalData::UpdateSaveForSeason": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
            ctypes.c_ulonglong,  # cGcDefaultSaveData *
        ]
    ),
    "cGcSeasonalData::GetResetSaveOnDeath": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
        ]
    ),
    "cGcSeasonalData::GetStatsToPersistOnReset": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkDynamicArray<cGcPersistedStatData> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
        ]
    ),
    "cGcSeasonalData::ShouldForceTernaryStar": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcSeasonalData::DoesSystemHaveOverridenSandworms": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcSeasonalData::ShouldSandwormSpawnGroundWorms": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcSeasonalData::MustCraftInBases": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
        ]
    ),
    "cGcSeasonalData::GetTechCostMultiplier": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
        ]
    ),
    "cGcSeasonalData::IsProductRemovedFromDefaultKnownProductsBySeason": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcSeasonalData::IsSeasonActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
        ]
    ),
    "cGcSeasonalData::GetSecondsRemainingToStartNewGame": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
        ]
    ),
    "cGcSeasonalData::HasUAOverride": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
        ]
    ),
    "cGcSeasonalData::GetSeasonalMainIcon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
            ctypes.c_ulonglong,  # const TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        ]
    ),
    "cGcCreatureBehaviourTree::GetEatAnim": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcSeasonalData::GetMilestone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcSeasonalMilestone *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSeasonalData::GetStageControlMilestoneIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSeasonalData::GetMilestoneWithMission": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSeasonalMilestone *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcSeasonalData::GetMilestoneWithRecipeReward": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcSeasonalMilestone *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcSeasonalData::GetMilestoneIndicesForMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcSeasonalData::GenerateRewardDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcSeasonalData::GenerateSeasonStartSeed": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
        ]
    ),
    "cGcSeasonalData::LoadServerSeasonalDataFromFile": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalData *
        ]
    ),
    "cGcSeasonState::sSeasonalMilestoneProgress::GetMilestone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcSeasonalMilestone *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState::sSeasonalMilestoneProgress *
        ]
    ),
    "cGcSeasonState::OnSeasonCompletionAcknowledged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonState::GetPinnedMilestoneSeed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSeed *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_ulonglong,  # cTkSeed *
        ]
    ),
    "cGcSeasonState::GetMissionStageAndMilestoneFromSeed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcSeasonState::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<std::vector<bool,TkSTLAllocatorShim<bool,1,-1> >,8,-1> *
        ]
    ),
    "cGcSeasonState::Save": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_ulonglong,  # cGcSeasonStateData *
        ]
    ),
    "cGcSeasonState::Load": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_ulonglong,  # const cGcSeasonStateData *
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<std::vector<bool,TkSTLAllocatorShim<bool,1,-1> >,8,-1> *
        ]
    ),
    "cGcSeasonState::OnMissionsLoaded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
        ]
    ),
    "cGcSeasonState::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSeasonState::WasSeasonEverActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_uint32,  # ePresetGameMode
        ]
    ),
    "cGcSeasonState::ApplyTableModifications": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
        ]
    ),
    "cGcSeasonState::RevertTableModifications": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
        ]
    ),
    "cGcSeasonState::GetSeasonProgress": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
        ]
    ),
    "cGcSeasonState::GetNumberOfCompleteStages": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
        ]
    ),
    "cGcSeasonState::GetMilestoneProgress": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSeasonState::sSeasonalMilestoneProgress *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_ulonglong,  # const cGcSeasonalMilestone *
        ]
    ),
    "cGcSeasonState::StartStage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSeasonState::RegisterRendezvousUA": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSeasonState::GetPreviousRendezvousUA": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSeasonState::GetFirstIncompleteRendezvousUA": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
        ]
    ),
    "cGcSeasonState::GetFirstIncompleteRendezvousIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
        ]
    ),
    "cGcSeasonState::CompleteMilestone": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcSeasonState::OnMissionComplete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcSeasonState::GetPinnedMilestone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcSeasonalMilestone *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonState::GetPinnedMilestoneIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
        ]
    ),
    "cGcSeasonState::SetPinnedMilestone": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonState::ClearPinnedMilestone": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
        ]
    ),
    "cGcSeasonState::GetStageWithUnclaimedReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcSeasonState::GetFirstIncompleteMilestoneWithMission": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcSeasonalMilestone *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonState::HasReward": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSeasonState::HasCollectedReward": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSeasonState::CollectFinalReward": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
        ]
    ),
    "cGcSeasonState::CollectReward": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSeasonState::IsPlayerInSeasonalUA": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
        ]
    ),
    "cGcSeasonState::IsUASeasonal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSeasonState::IsMaskedUASeasonal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcSeasonState::RegisterBuildingPreventionArea": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSeasonState::UpdateStageCompletion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSeasonState::OnMilestoneComplete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_ulonglong,  # const cGcSeasonState::sSeasonalMilestoneProgress *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonState::OnStageComplete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSeasonState::AddRewardToUnlockedList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcSeasonState::HasPendingMilestoneReward": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonState *
        ]
    ),
    "cGcSeasonState::OnUnlockedSeasonRewardsLoaded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcInteractionComponent::AllowComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSquadronPilotInfo::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronPilotInfo *
            ctypes.c_ulonglong,  # const cGcSquadronPilotData *
        ]
    ),
    "cGcSquadronPilotInfo::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronPilotInfo *
        ]
    ),
    "cGcSquadronPilotInfo::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronPilotInfo *
        ]
    ),
    "cGcSquadronPilotInfo::UpdateResourceLoading": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronPilotInfo *
            ctypes.c_int32,  # int
        ]
    ),
    "StatListener::GetStatValue<float>": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # StatListener *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cGcStatDefinition *
        ]
    ),
    "cGcStatsManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsManager::SetupFromData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcPlayerStatsGroup> *
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<StatListener,8,-1> *
        ]
    ),
    "cGcStatsManager::WriteToData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerStatsGroup> *
        ]
    ),
    "cGcStatsManager::ActivateListener": FUNCDEF(
        restype=ctypes.c_ulonglong,  # StatListener *
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcStatsManager::DeactivateListener": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcStatsManager::OnReceiveStatRecordInt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsManager::OnReceiveStatRecordFloat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsManager::GetStatAsString": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcStatsManager::GetStatNextLevelAsString": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcLeveledStatData *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcStatsManager::CreateListener": FUNCDEF(
        restype=ctypes.c_ulonglong,  # StatListener *
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcStatsManager::GlobalStatChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cGcStatDefinition *
            ctypes.c_ulonglong,  # const Stat *
            ctypes.c_ulonglong,  # const Stat *
        ]
    ),
    "cGcStatsManager::TriggerLevelChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # const cGcLeveledStatData *
        ]
    ),
    "cGcStatsManager::FormatSols": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcStatsManager::FormatDistance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcStatsManager::GetJourneyStatsTotal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
        ]
    ),
    "cGcStatsManager::GetJourneyStatsValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcStatsManager::GetLeveledStatDataIfExists": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcLeveledStatData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcStatsManager::GetJourneyMilestoneName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_ulonglong,  # TkID<256> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcStatsManager::GetJourneyMilestoneRequirement": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcStatsManager::GetNextJourneyMilestoneLevel": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcJourneyMilestoneData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
        ]
    ),
    "cGcStatsManager::GetCurrentJourneyMilestoneRank": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
        ]
    ),
    "cGcStatsManager::WatchGlobalStat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # IStatWatcher *
        ]
    ),
    "cGcStatsManager::UnWatchGlobalStat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # IStatWatcher *
        ]
    ),
    "cGcSynchronisedStateManager::sBufferTypeSet::sBufferTypeSet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager::sBufferTypeSet *
        ]
    ),
    "cGcSynchronisedStateManager::sBufferTypeSet::~sBufferTypeSet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # robin_hood::detail::Table<1,80,cGcSlot *,std::vector<cGcNavGraphNodeHandle,TkSTLAllocatorShim<cGcNavGraphNodeHandle,2,-1> >,robin_hood::hash<cGcSlot *,void>,std::equal_to<cGcSlot *>,-1>::DataNode<robin_hood::detail::Table<1,80,cGcSlot *,std::vector<cGcNavGraphNodeHandle,TkSTLAllocatorShim<cGcNavGraphNodeHandle,2,-1> >,robin_hood::hash<cGcSlot *,void>,std::equal_to<cGcSlot *>,-1>,1> *
        ]
    ),
    "cGcSynchronisedStateManager::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager *
        ]
    ),
    "cGcSynchronisedStateManager::HandleHashMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager *
            ctypes.c_ulonglong,  # const cGcNetworkSynchronisedStateHashMessage *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcSynchronisedStateManager::HandleDataMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager *
            ctypes.c_ulonglong,  # cGcNetworkSynchronisedStateDataMessage *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcSynchronisedStateManager::HandleEditMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager *
            ctypes.c_ulonglong,  # cGcNetworkSynchronisedStateEditMessage *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcSynchronisedStateManager::SendHashMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager *
        ]
    ),
    "cGcSynchronisedStateManager::SendChunkDataMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcSynchronisedStateManager::SendDeleteBufferMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # eSyncBufferType
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcSynchronisedStateManager::CreateBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # eSyncBufferType
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcSynchronisedStateManager::DestroyBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager *
            ctypes.c_ulonglong,  # unsigned __int16 *
        ]
    ),
    "cGcSynchronisedStateManager::GetOrLoadAddress": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSynchronisedStateManager::sLoadedAddress *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSynchronisedStateManager::LoadAddress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager::sLoadedAddress *
        ]
    ),
    "cGcSynchronisedStateManager::FindExternalBufferIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager *
            ctypes.c_ulonglong,  # const cGcSynchronisedStateManager::sLoadedAddress *
            ctypes.c_uint32,  # eSyncBufferType
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcSynchronisedStateManager::GetAllPlayers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<cGcNetworkPlayer *,TkSTLAllocatorShim<cGcNetworkPlayer *,8,-1> > *
        ]
    ),
    "cGcTelemetryManager::SetupFromData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTelemetryManager *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcTelemetryStat> *
        ]
    ),
    "cGcTelemetryManager::WriteToData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTelemetryManager *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTelemetryStat> *
        ]
    ),
    "cGcTelemetryManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTelemetryManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcTelemetryManager::VariantChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTelemetryManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cGcStatDefinition *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTelemetryManager::NotifyBeginSession": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTelemetryManager *
        ]
    ),
    "cGcTelemetryManager::OnPlayerCountChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTelemetryManager *
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTelemetryManager::OnGameLoaded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTelemetryManager *
            ctypes.c_ulonglong,  # const cGcPlayerStateData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTelemetryManager::BuildTelemetryUploadData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTelemetryManager *
            ctypes.c_ulonglong,  # const cGcPlayerStateData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainEditBlockBuffer::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cGcTerrainEdit *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcTerrainEditBlockBuffer::CopyEditsToBuffer": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
            ctypes.c_ulonglong,  # cGcTerrainEdit *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcTerrainEditBlockBuffer::TryAddEdit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int8,  # sTerrainEditData
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainEditBlockBuffer::TryRemoveEdits": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcTerrainEditBlockBuffer::TryUndoEdit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int8,  # sTerrainEditData
        ]
    ),
    "cGcTerrainEditBlockBuffer::TryUndoEdits": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # sRegionEdit *
        ]
    ),
    "cGcTerrainEditBlockBuffer::ClearBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
        ]
    ),
    "cGcTerrainEditBlockBuffer::ApplyEditsToRegion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcTerrainEditBlockBuffer::TestEditsAgainstPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcTerrainEditBlockBuffer::GenerateHashMessage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkHashMessage> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
            ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkHashMessage> *
        ]
    ),
    "cGcTerrainEditBlockBuffer::GenerateSyncMessage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkSyncMessage> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
            ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkSyncMessage> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainEditBlockBuffer::ApplySyncMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
            ctypes.c_ulonglong,  # const cGcNetworkSyncMessage *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcTerrainEditBlockBuffer::HasNetworkOwner": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcTerrainEditBlockBuffer::AccomodateNewEdit": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcTerrainEditBlockBuffer::CleanupBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
            ctypes.c_ulonglong,  # const cGcTerrainEdit *
        ]
    ),
    "cGcTerrainEditBlockBuffer::ApplyBoundsSync": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcTerrainEditBlockBuffer::OnHashOffsetChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCameraAmbientSpaceData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTelemetryStat> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcUAProtectedLocations>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUAProtectedLocations> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<ushort>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<unsigned short> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcRealitySubstanceData>::end": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkArraySTLIterator<cGcRealitySubstanceData>::InternalConstIterator *
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRealitySubstanceData> *
            ctypes.c_ulonglong,  # cTkArraySTLIterator<cGcRealitySubstanceData>::InternalConstIterator *
        ]
    ),
    "cGcStatsManager::Record<int>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # const int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const IStatVariant *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcAISpaceshipManagerData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAISpaceshipManagerData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcStatDefinitionTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEntitlementRewardsTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcStatGroupTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcStatGroupTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcJourneyMilestoneTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEntitlementRewardsTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcLeveledStatTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcLeveledStatTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsManager::Record<float>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const IStatVariant *
        ]
    ),
    "cGcRealitySubstanceData::cGcRealitySubstanceData": {
        "cGcRealitySubstanceData *, const cGcRealitySubstanceData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcRealitySubstanceData *
                ctypes.c_ulonglong,  # const cGcRealitySubstanceData *
            ]
        ),
        "cGcRealitySubstanceData *, cGcRealitySubstanceData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcRealitySubstanceData *
                ctypes.c_ulonglong,  # cGcRealitySubstanceData *
            ]
        ),
    },
    "cGcProductData::cGcProductData": {
        "cGcProductData *, const cGcProductData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcProductData *
                ctypes.c_ulonglong,  # const cGcProductData *
            ]
        ),
        "cGcProductData *, cGcProductData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcProductData *
                ctypes.c_ulonglong,  # cGcProductData *
            ]
        ),
    },
    "cGcTechnology::cGcTechnology": {
        "cGcTechnology *, const cGcTechnology *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcTechnology *
                ctypes.c_ulonglong,  # const cGcTechnology *
            ]
        ),
        "cGcTechnology *, cGcTechnology *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcTechnology *
                ctypes.c_ulonglong,  # cGcTechnology *
            ]
        ),
    },
    "cTkDynamicArray<cGcStatsBonus>::cTkDynamicArray<cGcStatsBonus>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStatsBonus> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcStatsBonus> *
        ]
    ),
    "cTkDynamicArray<cGcTechnologyRequirement>::cTkDynamicArray<cGcTechnologyRequirement>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTechnologyRequirement> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcTechnologyRequirement> *
        ]
    ),
    "cTkDynamicArray<cGcStatsBonus>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStatsBonus> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcTechnologyRequirement>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTechnologyRequirement> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkModelResource::cTkModelResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTextureResource *
            ctypes.c_ulonglong,  # cTkTextureResource *
        ]
    ),
    "cGcTerrainEditBlockBuffer::CompressTerrainEditPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcStatsManager::GetStatLevel": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsManager *
            ctypes.c_ulonglong,  # const cGcLeveledStatData *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcPlayerVehicleOwnership::IsInSummoningRange": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
            ctypes.c_uint32,  # eVehicleType
        ]
    ),
    "cGcPlanet::IsPositionUnderwater": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cTkPhysicsComponent::GetRigidBody": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRigidBody *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cTkPhysicsComponent::SetRelMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cTkPhysicsComponent::SetMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::InitializeFromData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
            ctypes.c_ulonglong,  # const cGcTerrainEditsBuffer *
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::PopulateBufferData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
            ctypes.c_ulonglong,  # cGcTerrainEditsBuffer *
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::RemoveProtectedEditsAroundBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
            ctypes.c_ulonglong,  # _WORD *
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::GetOverlappingTerrainEditBlocksNoLock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # std::vector<cGcTerrainEditBlockBuffer *,TkSTLAllocatorShim<cGcTerrainEditBlockBuffer *,8,-1> > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::ApplyUndoneEdits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
            ctypes.c_ulonglong,  # const cGcNetworkTerrainEditMessage *
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::ApplyEditsUndo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
            ctypes.c_ulonglong,  # const cGcNetworkTerrainEditMessage *
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::UndoTerrainEdits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_char,  # char
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::GetNearestOverdueBlock": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # float *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::FindMatchingBlockBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
            ctypes.c_ulonglong,  # const cGcUniverseAddressData *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::DeleteBlockBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::QueueEditJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const sTerrainEditData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_char,  # char
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::KickBlockJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::ApplyEditJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::ApplyBlockJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::SaveEditInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
            ctypes.c_ulonglong,  # const cGcTerrainEditsPersistentBuffer::sPendingEdit *
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::UndoEditsInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
            ctypes.c_ulonglong,  # const cGcTerrainEditsPersistentBuffer::sPendingEdit *
            ctypes.c_ulonglong,  # cTkAABB *
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::InternalUpdateYoungest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::InternalFreeBlock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::FindBlockIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcTerrainEditsPersistentBuffer::GetProtectedBlocksForBaseLayoutNoLock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsPersistentBuffer *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cGcBaseBuildingBaseLayout *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcTradingSupplyBuffer::GetAverageSupplyInUA": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingSupplyBuffer *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # eTradeCategory
        ]
    ),
    "cGcUserSeenItemsState::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSeenItemsState *
        ]
    ),
    "cGcUserSeenItemsState::IsProductSeen": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSeenItemsState *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcUserSeenItemsState::IsSpecialUnlocked": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSeenItemsState *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcUserSeenItemsState::GetUnlockedTwitchRewardsInLinkedGroup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSeenItemsState *
            ctypes.c_ulonglong,  # const std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> > *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcUserSeenItemsState::SeeSubstance": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSeenItemsState *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcUserSeenItemsState::SeeProduct": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSeenItemsState *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcUserSeenItemsState::SeeTechnology": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSeenItemsState *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcUserSeenItemsState::SeeTopic": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSeenItemsState *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcUserSeenItemsState::UnlockTopic": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSeenItemsState *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcUserSeenItemsState::UnlockTitle": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSeenItemsState *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcUserSeenItemsState::AddUnlockedSpecial": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSeenItemsState *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcUserSeenItemsState::SaveState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSeenItemsState *
        ]
    ),
    "cGcUserSeenItemsState::LoadState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSeenItemsState *
        ]
    ),
    "cGcUserSeenItemsState::UpdateWikiUnlocks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSeenItemsState *
            ctypes.c_ulonglong,  # cGcWiki *
        ]
    ),
    "cGcUserSettings::GetShipHUDVROffset": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
        ]
    ),
    "cGcUserSettings::InitDefaultUserSettingsForThisDevice": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcUserSettings::GetUseShipAutoControlVignette": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
        ]
    ),
    "cGcUserSettings::IsFlightInverted": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
        ]
    ),
    "cGcUserSettings::LoadState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
        ]
    ),
    "cGcUserSettings::GetGyroMeleeHackEnabled": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
        ]
    ),
    "cGcUserSettings::LoadPlatformSettingsStateFromStorage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
        ]
    ),
    "cGcUserSettings::LoadSettingsAccountDataStateFromStorage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
        ]
    ),
    "cGcUserSettingsData::cGcUserSettingsData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettingsData *
            ctypes.c_ulonglong,  # const cGcUserSettingsData *
        ]
    ),
    "cGcUserSettings::WriteStateToStorage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUserSettings::WriteStateToStorageThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcUserSettings::WriteSettingsAccountDataStateToStorage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
        ]
    ),
    "cGcUserSettings::UpdateAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
        ]
    ),
    "cGcUserSettings::IsVoiceChatEnabled": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
        ]
    ),
    "cGcUserSettings::SetLookSensitivity": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUserSettings::SetFlightSensitivity": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUserSettings::SetCursorSensitivity": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUserSettings::GetLookSensitivity": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
        ]
    ),
    "cGcVisitedSystemsBuffer::VisitNewGalacticAddress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVisitedSystemsBuffer *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcVisitedSystemsBuffer::GetVisitedSystemsAroundVoxel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVisitedSystemsBuffer *
            ctypes.c_ulonglong,  # std::vector<sVisit,TkSTLAllocatorShim<sVisit,2,-1> > *
            ctypes.c_ulonglong,  # cGcGalacticVoxelCoordinate *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcVisitedSystemsBuffer::GetVisitedData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVisitedSystemsBuffer *
            ctypes.c_ulonglong,  # const cGcGalacticVoxelCoordinate *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcWonderData::SetRecord": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderData *
            ctypes.c_ulonglong,  # const cGcWonderRecord *
            ctypes.c_ulonglong,  # const cGcWonderRecordCustomData *
        ]
    ),
    "cGcWonderData::sDiscoveryData::SetData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderData::sDiscoveryData *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcWonderData::UpdateDiscoverySearch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderData *
        ]
    ),
    "cGcWonderData::UpdateGenerateResourceData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderData *
        ]
    ),
    "cGcWonderData::UpdateResourceLoad": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderData *
        ]
    ),
    "cGcWonderData::CacheResourceRenderParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderData *
        ]
    ),
    "cGcWonderData::FillWikiItemData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderData *
            ctypes.c_ulonglong,  # WikiItemData *
        ]
    ),
    "cGcWonderData::GeneratePlanetDescriptionString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderData *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcWonderData::GenerateCreatureDescriptionString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderData *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcWonderData::GenerateFloraDescriptionString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderData *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcWonderData::GenerateMineralDescriptionString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderData *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcWonderData::GetCustomNameString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderData *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcWonderManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
        ]
    ),
    "cGcWonderManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcWonderManager::GetWonderData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcWonderData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
            ctypes.c_uint32,  # eWonderType
            ctypes.c_int32,  # int
        ]
    ),
    "cGcWonderManager::GetWonderDataWritable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWonderManager::sWonderData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
            ctypes.c_uint32,  # eWonderType
            ctypes.c_int32,  # int
        ]
    ),
    "cGcWonderManager::TriggerOSDMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
            ctypes.c_uint32,  # eWonderType
            ctypes.c_int32,  # int
        ]
    ),
    "cGcWonderManager::OnProcProductReceived": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char,  # char
        ]
    ),
    "cGcWonderManager::ClearRecord": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
            ctypes.c_ulonglong,  # const cGcWonderHandle *
        ]
    ),
    "cGcWonderManager::DiscoveryDataSubmitted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcWonderManager::GenerateDiscoveryExport": FUNCDEF(
        restype=ctypes.c_ulonglong,  # GcDiscoverySearchConstraints::ConstraintBase *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
        ]
    ),
    "cGcWonderManager::SetWondersForRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
            ctypes.c_uint32,  # eWonderType
        ]
    ),
    "cGcWonderManager::UpdatePlayerEnvironmentRecords": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
        ]
    ),
    "cGcWonderManager::TrySetRecords": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
            ctypes.c_uint32,  # eWonderType
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkStackVector<float,8,-1> *
        ]
    ),
    "cGcWonderManager::TrySetRecord": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
            ctypes.c_uint32,  # eWonderType
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcWonderRecord *
        ]
    ),
    "cGcWonderManager::ProcessDiscoveryExport": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
            ctypes.c_ulonglong,  # const cGcDiscoveryExport *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcWonderManager::ProcessPlanetDiscovery": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # const cGcPlanetGenerationWonderQuery *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderManager::ProcessCreatureDiscovery": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # const cGcPlanetGenerationWonderQuery *
        ]
    ),
    "cGcWonderManager::ProcessFloraDiscovery": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # const cGcPlanetGenerationWonderQuery *
        ]
    ),
    "cGcWonderManager::ProcessMineralDiscovery": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # const cGcPlanetGenerationWonderQuery *
        ]
    ),
    "cGcWonderManager::CreateWonderDiscoveryExport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderManager *
            ctypes.c_ulonglong,  # cGcWonderDiscoveryExport *
        ]
    ),
    "cGcWonderDiscoveryExport::ContainsRecord": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderDiscoveryExport *
            ctypes.c_ulonglong,  # const cGcDiscoveryRecord *
        ]
    ),
    "cGcWonderUtils::GetWikiTopicForWonderType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # eWonderType
        ]
    ),
    "cGcWonderUtils::GetWonderTypeForWikiGridTopic": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # eWikiTopicType
        ]
    ),
    "cGcWonderUtils::GenerateCreatureIntelligenceStat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # eCreatureType
            ctypes.c_uint32,  # eCreatureRole
        ]
    ),
    "cGcWonderUtils::GenerateCreatureViciousnessStat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # eCreatureType
            ctypes.c_uint32,  # eCreatureRole
            ctypes.c_uint32,  # eDiet
        ]
    ),
    "cGcWonderUtils::GenerateSystemDataFromDiscovery": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_ulonglong,  # cGcSolarSystemData *
            ctypes.c_ulonglong,  # cGcGalaxyAttributesAtAddress *
        ]
    ),
    "cGcWonderUtils::GetAdjustedPlanetRadius": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
        ]
    ),
    "cGcWonderUtils::GenerateRandomGenericWonderStat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_uint32,  # eWonderType
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcWonderUtils::GetCategoryConfig": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedArray<cGcWonderCategoryConfig,11> *
        argtypes=[
            ctypes.c_uint32,  # eWonderType
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<TkID<256>>::cTkDynamicArray<TkID<256>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<TkID<256> > *
            ctypes.c_ulonglong,  # const cTkDynamicArray<TkID<256> > *
        ]
    ),
    "cTkDynamicArray<cTkFixedString<128,char>>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkFixedString<128,char> > *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkFixedString<128,char>>::cTkDynamicArray<cTkFixedString<128,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkFixedString<128,char> > *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkFixedString<128,char> > *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcUserSettingsData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcUserSettingsData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUserSettings::MergeDataArrays<TkID<128>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
            ctypes.c_ulonglong,  # cTkDynamicArray<TkID<128> > *
            ctypes.c_ulonglong,  # const cTkDynamicArray<TkID<128> > *
        ]
    ),
    "cGcUserSettings::MergeDataArrays<TkID<256>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
            ctypes.c_ulonglong,  # cTkDynamicArray<TkID<256> > *
            ctypes.c_ulonglong,  # const cTkDynamicArray<TkID<256> > *
        ]
    ),
    "cTkMetaDataXML::ReadFromFile<cGcUserSettingsData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcUserSettingsData *
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUserSettingsData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettingsData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "GcDiscoverySearchConstraints::FilterGroupOR::Accept": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # GcDiscoverySearchConstraints::FilterGroupOR *
            ctypes.c_ulonglong,  # const cGcDiscoveryRecord *
        ]
    ),
    "cGcSolarSystem::GetSystemGenerator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSolarSystemGenerator *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSolarSystemAsteroidFields::cGcSolarSystemAsteroidFields": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemAsteroidFields *
        ]
    ),
    "cGcPlayerEnvironment::IsOnPlanet": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEnvironment *
        ]
    ),
    "cGcGalaxyAttributesAtAddress::cGcGalaxyAttributesAtAddress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyAttributesAtAddress *
        ]
    ),
    "cTkFixedArray<cGcPlanetGenerationInputData,8>::cTkFixedArray<cGcPlanetGenerationInputData,8>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcPlanetGenerationInputData,8> *
        ]
    ),
    "cTkModelResourceRenderer::SetData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelResourceRenderer *
            ctypes.c_ulonglong,  # cTkModelRendererData *
        ]
    ),
    "cGcWikiPageData::GetPageLayout": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # eWikiTopicType
        ]
    ),
    "sRegionEdit::sRegionEdit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sRegionEdit *
        ]
    ),
    "cGcCommandLine::CreateRegistryKeys": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCommandLine *
        ]
    ),
    "MinidumpTimeoutThread": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "MiniDumpFunction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # _EXCEPTION_POINTERS *
        ]
    ),
    "main": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char **
            ctypes.c_char_p,  # const char **
        ]
    ),
    "Main": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # char **
        ]
    ),
    "Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcColourPalette::Construct": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcColourPalette *
        ]
    ),
    "cGcColourPalette::GeneratePalettes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcColourPalette *
            ctypes.c_uint32,  # eBiome
            ctypes.c_uint32,  # eBiomeSubType
            common.GcSeed,  # cTkSeed
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcColourPaletteData *
        ]
    ),
    "cGcColourPalette::GetVariantOfColour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cGcColourPalette *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_uint32,  # eColourVariantType
        ]
    ),
    "cGcColourPalette::GenerateBaseColours": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcColourPalette *
            ctypes.c_uint32,  # eBiome
            ctypes.c_uint32,  # eBiomeSubType
            ctypes.c_uint32,  # ePalette
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcColourPalette::PickRandomColour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cGcColourPalette *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_uint32,  # eBiome
            ctypes.c_uint32,  # eBiomeSubType
            ctypes.c_uint32,  # ePalette
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcColourPalette::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcColourPalette *
        ]
    ),
    "cGcLegacyColourPalette::GeneratePalettes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLegacyColourPalette *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # eBiomeSubType
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcColourPaletteData *
        ]
    ),
    "cGcLegacyColourPalette::GenerateBaseColours": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLegacyColourPalette *
            ctypes.c_uint32,  # eBiome
            ctypes.c_uint32,  # eBiomeSubType
            ctypes.c_uint32,  # ePalette
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcProceduralTextureDescriptor::~cGcProceduralTextureDescriptor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureDescriptor *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcProceduralMaterialMapping,8,-1> *
        ]
    ),
    "cGcProceduralTextureDescriptor::IsReady": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureDescriptor *
        ]
    ),
    "cGcProceduralTextureDescriptor::GenerateInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureDescriptor *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_ulonglong,  # cGcPlanetColourData *
            ctypes.c_ulonglong,  # TkID<256> *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "ProcTexRegisterWithMetadataCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcProceduralTextureDescriptor::ChooseTextureOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureDescriptor *
            ctypes.c_ulonglong,  # std::vector<cGcProceduralTextureDescriptor::cGcProceduralTextureOptionGroup,TkSTLAllocatorShim<cGcProceduralTextureDescriptor::cGcProceduralTextureOptionGroup,8,-1> > *
            common.GcSeed,  # cTkSeed
            ctypes.c_ulonglong,  # cGcPlanetColourData *
            ctypes.c_uint32,  # eSamplerToReplace
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ubyte,  # bool
            ctypes.c_char * 0x20,  # TkID<256>
        ]
    ),
    "cGcProceduralTextureDescriptor::ChooseTextureOptionsGuided": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureDescriptor *
            ctypes.c_ulonglong,  # std::vector<cGcProceduralTextureDescriptor::cGcProceduralTextureOptionGroup,TkSTLAllocatorShim<cGcProceduralTextureDescriptor::cGcProceduralTextureOptionGroup,8,-1> > *
            ctypes.c_ulonglong,  # cGcPlanetColourData *
            ctypes.c_uint32,  # eSamplerToReplace
        ]
    ),
    "cGcProceduralTextureManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureManager *
        ]
    ),
    "cGcProceduralTextureManager::CheckSourceTexturesResident": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
        ]
    ),
    "cGcProceduralTextureManager::ContextDeleteFunc": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcProceduralTextureManager::AbortBakeFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcProceduralTextureManager::BakeTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcProceduralTextureManager::StreamingBakeTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcProceduralTextureManager::ApplyTextureOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureManager *
            ctypes.c_ulonglong,  # cGcProceduralTextureDescriptor *
            ctypes.c_ulonglong,  # std::vector<cGcProceduralTextureManager::DeferredAverageColourTask,TkSTLAllocatorShim<cGcProceduralTextureManager::DeferredAverageColourTask,8,4> > *
            ctypes.c_ulonglong,  # std::vector<cGcProceduralTextureManager::DeferredTextureSetupTask,TkSTLAllocatorShim<cGcProceduralTextureManager::DeferredTextureSetupTask,8,4> > *
            ctypes.c_ulonglong,  # const cTkSmartResHandle *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_char * 0x20,  # TkID<256>
        ]
    ),
    "cGcProceduralTextureManager::DeferredTextureSetupTask::~DeferredTextureSetupTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureManager::DeferredTextureSetupTask *
        ]
    ),
    "cGcProceduralTextureManager::GenerateAndLoadResource": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSmartResHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureManager *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOptionList *
            ctypes.c_ulonglong,  # const cGcPlanetColourData *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ubyte,  # bool
            ctypes.c_char * 0x20,  # TkID<256>
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralTextureManager::TaskContext::~TaskContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureManager::TaskContext *
        ]
    ),
    "cGcProceduralTextureManager::LoadResourceUniqueColourSeed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSmartResHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureManager *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOptionList *
            ctypes.c_ulonglong,  # TkID<256> *
        ]
    ),
    "cGcProceduralTextureManager::CreateGenerationTask": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcProceduralTextureManager::TaskContext *
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureManager *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOptionList *
            ctypes.c_ulonglong,  # const cGcPlanetColourData *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ubyte,  # bool
            ctypes.c_char * 0x20,  # TkID<256>
            common.GcSeed,  # cTkSeed
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_char * 0x20,  # TkID<256>
        ]
    ),
    "cGcProceduralTextureManager::DestroyTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureManager *
            ctypes.c_ulonglong,  # cGcProceduralTextureManager::TaskContext *
        ]
    ),
    "cGcProceduralTextureManager::AreTaskResourcesLoaded": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureManager *
            ctypes.c_ulonglong,  # cGcProceduralTextureManager::TaskContext *
        ]
    ),
    "cGcProceduralTextureManager::PollForTaskCompletion": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureManager *
            ctypes.c_ulonglong,  # cGcProceduralTextureManager::TaskContext *
            ctypes.c_int64,  # __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProceduralTextureManager::DeferredAverageColourTask::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureManager::DeferredAverageColourTask *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRendertargetTextureCache::GetTarget": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSmartResHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRendertargetTextureCache *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMetadataCache<cTkProceduralTextureList>::FinishAsyncLoad": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkProceduralTextureList *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cTkProceduralTextureList> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMetadataCache<cTkProceduralTextureList>::LoadAsync": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cTkProceduralTextureList> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cCallbackData<cTkProceduralTextureList> *
        ]
    ),
    "cTkMetadataCache<cTkProceduralTextureList>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cTkProceduralTextureList> *
        argtypes=[]
    ),
    "cTkMetadataCache<cGcOverlayTexture>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkTrailData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cTkTrailData> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcPaletteList>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cGcPaletteList> *
        argtypes=[]
    ),
    "cTkDynamicArray<cTkProceduralTextureChosenOption>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkProceduralTextureChosenOption> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcWFCScenes::cGcWFCScenes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<cTkMetadataCache<cTkAttachmentData>::cDataRequest,TkSTLAllocatorShim<cTkMetadataCache<cTkAttachmentData>::cDataRequest,8,-1> > *
            ctypes.c_ulonglong,  # const std::vector<cTkMetadataCache<cTkAttachmentData>::cDataRequest,TkSTLAllocatorShim<cTkMetadataCache<cTkAttachmentData>::cDataRequest,8,-1> > *
        ]
    ),
    "cTkLinearHashTable<cTkFixedString<256,char>,cTkMetadataCache<cTkProceduralTextureList>::cTkMetadataLookup,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char>>>::insert": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<cTkFixedString<256,char>,cTkMetadataCache<cTkProceduralTextureList>::cTkMetadataLookup,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char> > > *
            ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # const cTkMetadataCache<cTkProceduralTextureList>::cTkMetadataLookup *
        ]
    ),
    "cTkMetadataCache<cTkProceduralTextureList>::DefaulOnReceiveMetadataCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkMetadataCache<cTkProceduralTextureList>::OnFinishLoadCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkLinearHashTable<cTkFixedString<256,char>,std::vector<cTkMetadataCache<cTkAttachmentData>::cDataRequest,TkSTLAllocatorShim<cTkMetadataCache<cTkAttachmentData>::cDataRequest,8,-1>>,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char>>>::ChangeNodeCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<cTkFixedString<256,char>,std::vector<cTkMetadataCache<cTkAttachmentData>::cDataRequest,TkSTLAllocatorShim<cTkMetadataCache<cTkAttachmentData>::cDataRequest,8,-1> >,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char> > > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLinearHashTable<cTkFixedString<256,char>,std::vector<cTkMetadataCache<cTkAttachmentData>::cDataRequest,TkSTLAllocatorShim<cTkMetadataCache<cTkAttachmentData>::cDataRequest,8,-1>>,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char>>>::cTkListNode::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<cTkFixedString<256,char>,std::vector<cTkMetadataCache<cTkAttachmentData>::cDataRequest,TkSTLAllocatorShim<cTkMetadataCache<cTkAttachmentData>::cDataRequest,8,-1> >,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char> > >::cTkListNode *
        ]
    ),
    "cGcProceduralMaterialMapping::cGcProceduralMaterialMapping": {
        "cGcProceduralMaterialMapping *, const cGcProceduralMaterialMapping *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcProceduralMaterialMapping *
                ctypes.c_ulonglong,  # const cGcProceduralMaterialMapping *
            ]
        ),
        "cGcProceduralMaterialMapping *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcProceduralMaterialMapping *
            ]
        ),
    },
    "cTkMetaData::Async::StartReadFromFileAsync": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['void *', 'const char *', 'int', '__int64', 'void *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkResourceDescriptor::Equals": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_ulonglong,  # const cTkResourceDescriptor *
        ]
    ),
    "cGcProceduralTextureManager::DeferredTextureSetupSampler::DeferredTextureSetupSampler": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureManager::DeferredTextureSetupSampler *
        ]
    ),
    "cTkResourceDescriptor::AddDescriptor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_char * 0x20,  # TkID<256>
        ]
    ),
    "cGcProceduralTextureDescriptor::GetAsID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTextureDescriptor *
            ctypes.c_ulonglong,  # TkID<256> *
        ]
    ),
    "cGcPlanetGenerator::ValidateSubBiome": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_uint32,  # eBiome
            ctypes.c_uint32,  # eBiomeSubType
        ]
    ),
    "cGcColourPalette::GetColourWriteable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cGcColourPalette *
            ctypes.c_uint32,  # eBiome
            ctypes.c_uint32,  # eBiomeSubType
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cGcColourPalette::RenderPaletteCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkRandom::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRandom *
        argtypes=[]
    ),
    "cGcPlayerHUD::GetWantsTargetCrosshair": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcCameraBehaviourAerialView::ActivateEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAerialView *
            ctypes.c_ulonglong,  # cGcCameraBehaviourAerialView::cGcStartData *
        ]
    ),
    "cGcCameraBehaviourAerialView::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAerialView *
        ]
    ),
    "cGcCameraBehaviourAerialView::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAerialView *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourAerialView::Release": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAerialView *
        ]
    ),
    "cGcCameraBehaviourAerialView::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAerialView *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourAmbient::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient *
        ]
    ),
    "cGcCameraBehaviourAmbient::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourAmbient::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient *
        ]
    ),
    "cGcCameraBehaviourAmbient::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourAmbient::Transform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcCameraBehaviourAmbient::cGcStraightMovementLayer::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::cGcStraightMovementLayer *
        ]
    ),
    "cGcCameraBehaviourAmbient::cGcStraightMovementLayer::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::cGcStraightMovementLayer *
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::CameraState *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCameraBehaviourAmbient::cGcStraightMovementLayer::UpdateSamples": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::cGcStraightMovementLayer *
            ctypes.c_ulonglong,  # const cGcCameraBehaviourAmbient::CameraState *
        ]
    ),
    "cGcCameraBehaviourAmbient::cGcStraightMovementLayer::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::cGcStraightMovementLayer *
        ]
    ),
    "cGcCameraBehaviourAmbient::cGcLookAroundLayer::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::cGcLookAroundLayer *
        ]
    ),
    "cGcCameraBehaviourAmbient::cGcLookAroundLayer::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::cGcLookAroundLayer *
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::CameraState *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCameraBehaviourAmbient::cGcLookAroundLayer::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::cGcLookAroundLayer *
        ]
    ),
    "cGcCameraBehaviourAmbient::cGcMoveAroundLayer::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::cGcMoveAroundLayer *
        ]
    ),
    "cGcCameraBehaviourAmbient::cGcMoveAroundLayer::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::cGcMoveAroundLayer *
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::CameraState *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCameraBehaviourAmbient::cGcMoveAroundLayer::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::cGcMoveAroundLayer *
        ]
    ),
    "cGcCameraBehaviourAmbient::cGcAmbientCameraBehaviourManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::cGcAmbientCameraBehaviourManager *
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::CameraState *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCameraBehaviourAmbient::cGcCurveMovementLayer::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::cGcCurveMovementLayer *
        ]
    ),
    "cGcCameraBehaviourAmbient::cGcCurveMovementLayer::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::cGcCurveMovementLayer *
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::CameraState *
            ctypes.c_double,  # double
        ]
    ),
    "cGcCameraBehaviourAmbient::cGcCurveMovementLayer::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAmbient::cGcCurveMovementLayer *
        ]
    ),
    "cGcCameraBehaviourAnimation::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAnimation *
        ]
    ),
    "cGcCameraBehaviourAnimation::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAnimation *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourAnimation::Setup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAnimation *
        ]
    ),
    "cGcCameraBehaviourAnimation::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAnimation *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourFollowTarget::Transform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFollowTarget *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcCameraBehaviourAnimation::Release": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourAnimation *
        ]
    ),
    "cGcFrontendManager::IsShopPage": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_uint32,  # const eFrontendPage
        ]
    ),
    "cGcCameraBehaviourBuildingMode::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourBuildingMode *
        ]
    ),
    "cGcCameraBehaviourBuildingMode::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourBuildingMode *
        ]
    ),
    "cGcCameraBehaviourBuildingMode::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourBuildingMode *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourBuildingMode::Release": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourBuildingMode *
        ]
    ),
    "cGcCameraBehaviourBuildingMode::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourBuildingMode *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourBuildingMode::UpdateLoadBoundaryResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourBuildingMode *
        ]
    ),
    "cGcCameraBehaviourCharacter::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourCharacter *
        ]
    ),
    "cTkHavokCharacterController::cTkHavokCharacterController": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
        ]
    ),
    "cGcCameraBehaviourCharacter::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourCharacter *
        ]
    ),
    "cGcCameraBehaviourCharacter::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourCharacter *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourCharacter::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourCharacter *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourCharacter::GetDeflectedVector": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourCharacter *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcVehicleComponent::GetVelocity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcCameraBehaviourFirstPerson::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFirstPerson *
        ]
    ),
    "cGcCameraBehaviourFirstPerson::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFirstPerson *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourFirstPerson::Release": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
        ]
    ),
    "cGcCameraBehaviourFirstPerson::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFirstPerson *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourFly::Prepare": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFly *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourFly::ResetStartMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFly *
        ]
    ),
    "cGcCameraBehaviourFly::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFly *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourFly::Transform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFly *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcCameraBehaviourFollowTarget::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFollowTarget *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourFollowTarget::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFollowTarget *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourFollowTarget::MoveToTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFollowTarget *
        ]
    ),
    "cGcCameraBehaviourFreighterWarp::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFreighterWarp *
        ]
    ),
    "cGcCameraBehaviourFreighterWarp::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFreighterWarp *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourFreighterWarp::Release": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFreighterWarp *
        ]
    ),
    "cGcCameraBehaviourFreighterWarp::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFreighterWarp *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourFreighterWarp::ComputeWarpMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFreighterWarp *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcCameraBehaviourGalacticLookAt::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionEndScissorTest *
            ctypes.c_uint32,  # eEgRenderParity
        ]
    ),
    "cGcCameraBehaviourGalacticLookAt::Prepare": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticLookAt *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourGalacticLookAt::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticLookAt *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourGalacticNavigation::Prepare": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticNavigation *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourGalacticNavigation::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticNavigation *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourGalacticNavigation::UpdateInput": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticNavigation *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraBehaviourGalacticNavigation::UpdateHandControl": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticNavigation *
            ctypes.c_double,  # double
        ]
    ),
    "cGcCameraBehaviourGalacticNavigation::DrawGizmo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticNavigation *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcCameraBehaviourGalacticNavigation::LockToOrbitPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticNavigation *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCameraBehaviourGalacticNavigation::TriggerFlyToPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticNavigation *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCameraBehaviourGalacticNavigation::GetEye": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticNavigation *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraBehaviourGalacticNavigation::GetTarget": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticNavigation *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraBehaviourGalacticNavigation::CalculateMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticNavigation *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraBehaviourGalacticNavigation::RunSequenceZoomOut": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticNavigation *
        ]
    ),
    "cGcCameraBehaviourGalacticNavigation::UsePadVRControls": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticNavigation *
        ]
    ),
    "cGcCameraBehaviourGalacticTransition::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticTransition *
        ]
    ),
    "cGcCameraBehaviourGalacticTransition::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticTransition *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourGalacticTransition::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticTransition *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcNPCComponent::IsAllowedToMove": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::IsSeated": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcInteractionComponent::IsCustomisationInteraction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::UsesExtraShopOffsetVR": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::IsFreighterNPCInteraction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # eInteractionType
        ]
    ),
    "cGcInteractionComponent::IsNormalNPCInteraction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInteractionComponent::IsInteractionWithACreature": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::IsInteractionWithAnAlien": {
        "cGcInteractionComponent *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcInteractionComponent *
            ]
        ),
        "unsigned __int64": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
    },
    "cGcInteractionComponent::IsNPCInteraction": {
        "unsigned __int64": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
        "cGcInteractionComponent *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcInteractionComponent *
            ]
        ),
    },
    "cGcInteractionComponent::IsInteractionWithFleetManagement": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetModelRenderDataBlendAmount": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetInteractLocator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcInteractionComponent::HasComponent": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcPlayerFullBodyIKComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcCameraBehaviourInteraction::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourInteraction *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourInteraction::GetInteraction": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcInteractionComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourInteraction *
        ]
    ),
    "cGcCameraBehaviourInteraction::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourInteraction *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourInteraction::ReadDofParameters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourInteraction *
        ]
    ),
    "cGcCameraBehaviourInteraction::UsesFocus": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourInteraction *
        ]
    ),
    "cGcCameraBehaviourInteraction::CalculateModelRendererCameraMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourInteraction *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkModelRendererData *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcCameraBehaviourLookAt::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourLookAt *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourModelView::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourModelView *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourModelView::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourModelView *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourModelView::ReadDofParameters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourModelView *
        ]
    ),
    "cGcCameraBehaviourOffset::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourOffset *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourOffset::GetObjectMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourOffset *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcCameraBehaviourOffset::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourOffset *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourPhotoMode::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourPhotoMode *
        ]
    ),
    "cGcCameraBehaviourPhotoMode::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourPhotoMode *
        ]
    ),
    "cGcCameraBehaviourPhotoMode::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourPhotoMode *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourPhotoMode::Release": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourPhotoMode *
        ]
    ),
    "cGcCameraBehaviourPhotoMode::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourPhotoMode *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourPhotoMode::GetCameraMovement": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourPhotoMode *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcCameraBehaviourPhotoMode::GetCameraTurnVR": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourPhotoMode *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcCameraBehaviourPhotoMode::RepositionCamera": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourPhotoMode *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcCameraBehaviourPhotoMode::UpdateLoadBoundaryResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourPhotoMode *
        ]
    ),
    "cGcPlayerControlComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlComponent *
        ]
    ),
    "cGcPlayerControlComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerControlComponent::GetCurrentState": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlComponent *
        ]
    ),
    "ComponentInstancercGcPlayerControlComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcPlayerControlComponent *
        ]
    ),
    "ComponentInstancercGcSpringAttachmentComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcPlayerControlComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkBlackboard::cTkBlackboard": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboard *
        ]
    ),
    "cGcCameraBehaviourPlayerThirdPerson::GetEnableAutoRotate": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourPlayerThirdPerson *
        ]
    ),
    "cGcCameraBehaviourPlayerThirdPerson::ApplyCollisions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourPlayerThirdPerson *
            ctypes.c_ulonglong,  # cGcCameraFollowSettings *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cGcCameraBehaviourPlayerThirdPerson::StartCollisionJobs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourPlayerThirdPerson *
            ctypes.c_ulonglong,  # cGcCameraFollowSettings *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourPlayerThirdPerson::GetDesiredFollowSettingsData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCameraFollowSettings *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourPlayerThirdPerson *
        ]
    ),
    "cGcCameraBehaviourPlayerThirdPerson::GetCloseFactor": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourPlayerThirdPerson *
        ]
    ),
    "cGcCameraBehaviourScreenshot::ActivateEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourScreenshot *
            ctypes.c_ulonglong,  # cGcCameraBehaviourScreenshot::cGcStartData *
        ]
    ),
    "cGcCameraBehaviourScreenshot::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourScreenshot *
        ]
    ),
    "cGcCameraBehaviourScreenshot::Setup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourScreenshot *
        ]
    ),
    "cGcCameraBehaviourScreenshot::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourScreenshot *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourScreenshot::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourScreenshot *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourScreenshot::RenderScreenshot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcCameraBehaviourShipWarp::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourShipWarp *
        ]
    ),
    "cGcCameraBehaviourShipWarp::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourShipWarp *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourShipWarp::Release": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourShipWarp *
        ]
    ),
    "cGcCameraBehaviourShipWarp::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourShipWarp *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourShipWarp::UpdateSpin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourShipWarp *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCameraBehaviourShipWarp::UpdateShipMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourShipWarp *
        ]
    ),
    "cGcCameraBehaviourShipWarp::ComputeWarpMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourShipWarp *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcCameraBehaviourThirdPerson::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cGcCameraBehaviourThirdPerson::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourThirdPerson::UpdateCreatureInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cGcCameraBehaviourThirdPerson::UpdateLookSpring": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
            ctypes.c_ulonglong,  # cGcCameraFollowSettings *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcCameraBehaviourThirdPerson::UpdateFollowCenterAndVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cGcCameraFollowSettings *
        ]
    ),
    "cGcCameraBehaviourThirdPerson::CalculateBaseElevation": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCameraBehaviourThirdPerson::ApplyCollisions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
            ctypes.c_ulonglong,  # cGcCameraFollowSettings *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcCameraBehaviourThirdPerson::StartCollisionJobs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
            ctypes.c_ulonglong,  # cGcCameraFollowSettings *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cGcCameraBehaviourThirdPerson::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
        ]
    ),
    "cGcCameraBehaviourThirdPerson::Teleport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cGcCameraBehaviourThirdPerson::GetDesiredFollowSettingsData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCameraFollowSettings *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
        ]
    ),
    "cGcCameraBehaviourThirdPerson::UpdateCurrentFollowSettings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcCameraBehaviourThirdPerson::BuildFollowVector": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcCameraBehaviourThirdPerson::GetFollowingOffsets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cGcCameraFollowSettings *
        ]
    ),
    "cGcCameraBehaviourThirdPerson::UpdateFOVSpring": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCameraBehaviourThirdPerson::AnticipateFollowVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cGcCameraFollowSettings *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkHavokCharacterController::getBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ulonglong,  # const hknpBodyId **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkCurveFunction::SmoothCD<cTkVector3,cTkVector3>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerWeapon::IsWeaponScopeActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcCameraBehaviourThirdPerson::SplitRemap": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourThirdPerson *
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
        ]
    ),
    "cGcCreatureComponent::GetRelMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcCreatureComponent::GetHead": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcCreatureComponent::GetNavigation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureNavigation *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcPlayerInteract::IsInteracting": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cTkPhysicsComponent::GetRelMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcPlayerWeapon::GetShieldActiveFactor": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::IsShieldActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcCharacterSit::IsSitting": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterSit *
        ]
    ),
    "cTkPhysicsComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysicsComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputManager::IsPadControlActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "cGcAISpaceshipComponent::GetClass": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "GPN::SearchContext::GetGoalUA": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # GPN::SearchContext *
        ]
    ),
    "cGcPlayerCommunicator::IsActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicator *
        ]
    ),
    "cGcCameraBehaviourGalacticNavigation::SetTargetFacing": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticNavigation *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCameraBehaviourGalacticNavigation::SetFacing": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticNavigation *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCameraBehaviourGalacticNavigation::GetFacing": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticNavigation *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcCameraBehaviourGalacticNavigation::SetTargetPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourGalacticNavigation *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcPlayerVehicleOwnership::GetVehicleSpawnPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerVehicleOwnership::DestroyAllVehicles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
        ]
    ),
    "cGcSolarSystem::GetSpaceStationEntranceTM": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcPlayer::GetPersonalTeleporter": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPersonalTeleporter *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayerWeapon::IsAggressive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::IsMeleeBoosted": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerStick::GetBaseStick": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStick *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "cGcGameCamera::UpdateAndApplyShake": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cTkHavokCharacterController::SetVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkHavokCharacterController::OnGround": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
        ]
    ),
    "cTkComponentPool::HasComponent": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentPool *
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkAnimInstanceHandle::IsValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimInstanceHandle *
        ]
    ),
    "cTkAnimationComponent::GetAnimLength": {
        "cTkAnimationComponent *, const TkID<128> *": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # cTkAnimationComponent *
                ctypes.c_char_p,  # const TkID<128> *
            ]
        ),
        "cTkAnimationComponent *, const cTkAnimInstanceHandle *": FUNCDEF(
            restype=ctypes.c_double,  # double
            argtypes=[
                ctypes.c_ulonglong,  # cTkAnimationComponent *
                ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
            ]
        ),
    },
    "cTkAnimationComponent::IsAnimControl": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkAnimationComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimationComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureComponent::GetSizeClass": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkBitArray<unsigned __int64,1,128>::ClearPositive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,128> *
        ]
    ),
    "cGcUpdateOptionalArguments::cGcUpdateOptionalArguments": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUpdateOptionalArguments *
        ]
    ),
    "cGcCameraBehaviourAmbient::RenderNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcCameraDepthOfField::GetParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcCameraDepthOfField::ReadParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraDepthOfField *
            ctypes.c_ulonglong,  # const cGcInteractionDof *
        ]
    ),
    "cGcFrontendManager::GetPhotoModeUI": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPhotoModeUI *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
        ]
    ),
    "cGcCameraManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
        ]
    ),
    "cGcCameraManager::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_uint32,  # eCameraBehaviours
        ]
    ),
    "cGcCameraManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCameraManager::RepositionPlayerFromCamera": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCameraManager::ApplyCameraEvent": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_uint32,  # cGcCameraManager::eCameraEvent
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcCameraManager::Release": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
        ]
    ),
    "cGcCameraManager::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
        ]
    ),
    "cGcCameraManager::SetDebugActive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
        ]
    ),
    "cGcCameraManager::GetActiveHeadMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcCameraManager::GetActiveEyeMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcCameraManager::GetEyeOffsetAngles": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "cGcCameraManager::GetPercievedAngleFromCamera": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcCameraManager::BlendToBehaviour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_uint32,  # eCameraBehaviours
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eTkCameraTimeCurve
            ctypes.c_float,  # float
        ]
    ),
    "cGcCameraManager::GetNormalisedScreenPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCameraManager::GetScreenPos": {
        "cGcCameraManager *, cTkVector3 *, const cTkVector3 *, const cTkVector2 *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkVector3 *
            argtypes=[
                ctypes.c_ulonglong,  # cGcCameraManager *
                ctypes.c_ulonglong,  # cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector2 *
            ]
        ),
        "cGcCameraManager *, cTkVector3 *, const cTkPhysRelVec3 *, const cTkVector2 *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkVector3 *
            argtypes=[
                ctypes.c_ulonglong,  # cGcCameraManager *
                ctypes.c_ulonglong,  # cTkVector3 *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkVector2 *
            ]
        ),
    },
    "cGcCameraManager::GetScreenPosVec4": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector4 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_ulonglong,  # cTkVector4 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cGcCameraManager::IsPositionOnScreen": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCameraManager::ShakeFromPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcCameraManager::Shake": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPoolHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_ulonglong,  # cTkClassPoolHandle *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCameraManager::IsControlStealingBehaviourActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
        ]
    ),
    "cGcCameraManager::CalculateModelRendererCameraMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkModelRendererData *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCameraManager::GetCurrentDesiredFOV": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
        ]
    ),
    "cGcCameraManager::IsInFreeCameraMode": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
        ]
    ),
    "cGcGameCamera::DebugRenderShakes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameCamera *
        ]
    ),
    "cGcCameraShake::FindData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCameraShakeData *
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcCameraShake::GetStrength": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShake *
        ]
    ),
    "cGcCameraShake::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShake *
        ]
    ),
    "cGcCameraShake::Apply": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShake *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "ConvertUtf16ToByteLimitedUtf8": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcFrontendTextInput::Start": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcFrontendTextInput::GetResult": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcFrontendTextInput::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
        ]
    ),
    "cGcFrontendTextInput::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
        ]
    ),
    "cGcFrontendTextInput::GetInputString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcUIOptionListElement::IsHovering": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcUIOptionListElement *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcIntegerOption::UpdateUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIntegerOption *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFloatOption::CreateElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIntegerOption *
        ]
    ),
    "cGcIntegerOption::GetTranslatedDefaultValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcIntegerOption *
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        ]
    ),
    "cGcBooleanOption::UpdateUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBooleanOption *
        ]
    ),
    "cGcBooleanOption::CreateElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBooleanOption *
        ]
    ),
    "cGcBooleanOption::GetTranslatedDefaultValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBooleanOption *
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        ]
    ),
    "cGcButtonOption::UpdateUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcButtonOption *
        ]
    ),
    "cGcButtonOption::Initialize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcButtonOption *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcOptionsInteractionContext *
        ]
    ),
    "cGcUIOptionListElement::SetEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUIOptionListElement *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFloatOption::UpdateUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFloatOption *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFloatOption::GetTranslatedDefaultValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFloatOption *
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        ]
    ),
    "cGcFloatOption::FormatValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFloatOption *
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
            ctypes.c_float,  # float
        ]
    ),
    "cGcQualityOption::CreateElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQualityOption *
        ]
    ),
    "cGcQualityOption::UpdateUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQualityOption *
        ]
    ),
    "cGcOptionListHeader::Initialize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOptionListHeader *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcOptionsInteractionContext *
        ]
    ),
    "cGcOptionsInteractionContext::Set": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOptionsInteractionContext *
            ctypes.c_ulonglong,  # cGcLayerInteractControl *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcOptionsInteractionContext::Confirm": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcOptionsInteractionContext *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCraftingTreePageData::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCraftingTreePageData *
        ]
    ),
    "cGcDiscoveryPageData::DiscoveryPageInfoBase::UpdateName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData::DiscoveryPageInfoBase *
        ]
    ),
    "cGcDiscoveryPageData::DiscoveryPagePlanetInfo::RefreshDiscoveryRenders": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData::DiscoveryPagePlanetInfo *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcDiscoveryPageData::DiscoveryPagePlanetInfo::InitPlanetDiscoveriesPreview": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData::DiscoveryPagePlanetInfo *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcDiscoveryPagePlanetTask::Start": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPagePlanetTask *
        ]
    ),
    "cGcDiscoveryPagePlanetTask::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPagePlanetTask *
        ]
    ),
    "cGcDiscoveryPagePlanetTask::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPagePlanetTask *
        ]
    ),
    "cGcDiscoveryPageData::cGcDiscoveryPageData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
        ]
    ),
    "cGcDiscoveryPageData::DiscoveryPagePlanetInfo::DiscoveryPagePlanetInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData::DiscoveryPagePlanetInfo *
        ]
    ),
    "cGcDiscoveryPageData::StartFetch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
        ]
    ),
    "cGcDiscoveryPageData::CleanUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcDiscoveryExport::SolarSystem,8,19> *
        ]
    ),
    "cGcDiscoveryPageData::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcDiscoveryExport::SolarSystem,8,19> *
        ]
    ),
    "cGcDiscoveryPageData::FetchPlanetData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryPageData::RequestViewSpecificDiscoveryOnOpen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcDiscoveryPageData::ClearViewOnOpenRequest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
        ]
    ),
    "cGcDiscoveryPageData::PopulateFromDiscoveryExport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcDiscoveryPageData::GetNumItemsInSystemsList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
        ]
    ),
    "cGcDiscoveryPageData::SetSelectedSolarSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDiscoveryPageData::NextSystemPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcDiscoveryPageData::PreviousSystemPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
        ]
    ),
    "cGcDiscoveryPageData::UpdateRenameDiscovery": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
        ]
    ),
    "cGcDiscoveryPageData::UpdateWaypointStates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
        ]
    ),
    "cGcFleetPageData::MoveToNextDebriefLogEntryStage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetPageData *
        ]
    ),
    "cGcFleetPageData::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetPageData *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFleetPageData::SetFocusNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetPageData *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcFleetPageData::SetupPurchasableFrigate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetPageData *
            ctypes.c_uint32,  # TkHandle
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcFleetPageData::GetTraitLineDirection": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetPageData *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_uint32,  # eFrigateClass
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFleetPageData::StopDebriefAudio": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetPageData *
        ]
    ),
    "cGcMissionPageData::UpdateSelectedMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionPageData *
        ]
    ),
    "ScrollBarState::ScrollBarRenderFunction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "TextScrollBarState::TextScrollBarRenderFunction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcSeasonPageData::CacheStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonPageData *
        ]
    ),
    "cGcSeasonPageData::SetStageIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonPageData *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSeasonPageData::GetTransitionProgress": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonPageData *
        ]
    ),
    "cGcSeasonPageData::GetAlpha": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonPageData *
        ]
    ),
    "cGcSeasonPageData::SetupInitialPositions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonPageData *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcStoriesPageData::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoriesPageData *
        ]
    ),
    "cGcStoriesPageData::SelectCategory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoriesPageData *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcStoriesPageData::UpdateCategory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoriesPageData *
        ]
    ),
    "cGcWikiPageData::cGcWikiPageData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiPageData *
        ]
    ),
    "cGcWikiPageData::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiPageData *
        ]
    ),
    "cGcCustomWonderTextEntryHandler::EndTextEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomWonderTextEntryHandler *
        ]
    ),
    "cGcCustomWonderTextEntryHandler::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomWonderTextEntryHandler *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcWikiPageData *
        ]
    ),
    "cGcListOption<eInstantUIMode>::GetTranslatedDefaultValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcListOption<enum eYawAxisDirection> *
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        ]
    ),
    "cGcOption<eTurnMode>::GetTranslatedDefaultValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<bool> *
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        ]
    ),
    "cGcOption<ePitchAxisDirection>::TranslateDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<bool> *
        ]
    ),
    "cGcOption<eMovementMode>::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<bool> *
        ]
    ),
    "cGcOption<eTurnMode>::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<bool> *
        ]
    ),
    "cGcOption<eAnisotropyLevel>::IsNonDefault": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<enum eYawAxisDirection> *
        ]
    ),
    "cGcOption<eLookStickEnabled>::HasChanges": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<enum eYawAxisDirection> *
        ]
    ),
    "cGcOption<int>::DiscardChanges": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<enum eYawAxisDirection> *
        ]
    ),
    "cGcOption<eActiveModeOnFoot>::ResetToDefault": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<enum eYawAxisDirection> *
        ]
    ),
    "cGcOption<eTemperatureUnit>::Apply": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<float> *
        ]
    ),
    "cGcOption<eInstantUIMode>::Initialize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<bool> *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcOptionsInteractionContext *
        ]
    ),
    "cGcListOption<eHand>::cGcListOption<eHand>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcListOption<enum eHand> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # eHand *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcListOption<eMovementDirection>::cGcListOption<eMovementDirection>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcListOption<enum eMovementDirection> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # eMovementDirection *
            ctypes.c_uint32,  # eMovementDirection
            ctypes.c_uint32,  # eListOptionStyle
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcListOption<eMovementMode>::UpdateOption": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcListOption<enum eMovementMode> *
            ctypes.c_uint32,  # eMovementMode
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
        ]
    ),
    "cGcListOption<cTkIntTuple2<int,0>>::GetTranslatedDefaultValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcListOption<cTkIntTuple2<int,0> > *
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        ]
    ),
    "cGcOption<cTkIntTuple2<int,0>>::TranslateDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<cTkIntTuple2<int,0> > *
        ]
    ),
    "cGcOption<cTkIntTuple2<int,0>>::GetName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryStore *
        argtypes=[
            ctypes.c_ulonglong,  # ItemPicking *
        ]
    ),
    "cGcOption<cTkIntTuple2<int,0>>::IsNonDefault": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<cTkIntTuple2<int,0> > *
        ]
    ),
    "cGcOption<cTkIntTuple2<int,0>>::HasChanges": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<cTkIntTuple2<int,0> > *
        ]
    ),
    "cGcOption<cTkIntTuple2<int,0>>::DiscardChanges": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<cTkIntTuple2<int,0> > *
        ]
    ),
    "cGcOption<cTkIntTuple2<int,0>>::ResetToDefault": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<cTkIntTuple2<int,0> > *
        ]
    ),
    "cGcOption<cTkIntTuple2<int,0>>::Apply": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<cTkIntTuple2<int,0> > *
        ]
    ),
    "cGcOption<cTkIntTuple2<int,0>>::Initialize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<cTkIntTuple2<int,0> > *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcOptionsInteractionContext *
        ]
    ),
    "cGcListOption<int>::cGcListOption<int>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcListOption<int> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eListOptionStyle
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcOption<eGyroRotationSpace>::SetOptionsMenuValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<enum eYawAxisDirection> *
            ctypes.c_uint32,  # eYawAxisDirection
        ]
    ),
    "cGcOption<bool>::SetOptionsMenuValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<bool> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOption<float>::SetOptionsMenuValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<float> *
            ctypes.c_float,  # float
        ]
    ),
    "cGcListOption<cTkIntTuple2<int,0>>::UpdateUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcListOption<cTkIntTuple2<int,0> > *
        ]
    ),
    "cGcListOption<cTkIntTuple2<int,0>>::CreateElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcListOption<cTkIntTuple2<int,0> > *
        ]
    ),
    "cGcListOption<eAntiAliasing>::UpdateUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcListOption<enum eYawAxisDirection> *
        ]
    ),
    "cGcListOption<eGraphicDetail>::CreateElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcListOption<enum eYawAxisDirection> *
        ]
    ),
    "cGcDiscoveryPageData::DiscoveryPageInfoBase::DiscoveryPageInfoBase": {
        "cGcDiscoveryPageData::DiscoveryPageInfoBase *, const cGcDiscoveryPageData::DiscoveryPageInfoBase *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcDiscoveryPageData::DiscoveryPageInfoBase *
                ctypes.c_ulonglong,  # const cGcDiscoveryPageData::DiscoveryPageInfoBase *
            ]
        ),
        "cGcDiscoveryPageData::DiscoveryPageInfoBase *, const char *, const char *, const cTkFixedString<3,char> *, const cGcDiscoveryData *, const int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcDiscoveryPageData::DiscoveryPageInfoBase *
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # const cTkFixedString<3,char> *
                ctypes.c_ulonglong,  # const cGcDiscoveryData *
                ctypes.c_int32,  # const int
            ]
        ),
    },
    "cGcSolarSystemQuery::cGcSolarSystemQuery": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemQuery *
        ]
    ),
    "cGcDiscoveryPageData::DiscoveryPagePlanetInfo::SetDiscoveriesFilter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData::DiscoveryPagePlanetInfo *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_uint32,  # eDiscoveryType
        ]
    ),
    "cGcDiscoveryPageData::DiscoveryPagePlanetInfo::SetSelectedDiscovery": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData::DiscoveryPagePlanetInfo *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcCameraShake::GetMechanical": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShake *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcCameraShake::SetDirection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShake *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcGameCamera::Shake": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPoolHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPoolHandle *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkHmdBase::GetGazeMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdBase *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcCameraBehaviourModelView::GetBlendTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourModelView *
        ]
    ),
    "cGcCameraBehaviourModelView::GetBlendOffset": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourModelView *
        ]
    ),
    "cGcPlayer::SetFacing": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCameraManager::IsBehaviourActiveOrInterpolatingTo": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_uint32,  # eCameraBehaviours
        ]
    ),
    "cGcPlayer::RequestFallenCheck": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcPlayer::AdjustPositionForRespawn": {
        "cGcPlayer *, cTkVector3 *, const cTkVector3 *, bool": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkVector3 *
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayer *
                ctypes.c_ulonglong,  # cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcPlayer *, cTkPhysRelVec3 *, const cTkPhysRelVec3 *, bool": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayer *
                ctypes.c_ulonglong,  # cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cGcCameraManager::RenderNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFEElementLayout::cGcFEElementLayout": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFEElementLayout *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFEElementLayout::~cGcFEElementLayout": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFEElementLayout *
        ]
    ),
    "cGcFEElementLayout::GenerateEventsRecursive": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFEElementLayout *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # const cGcFEElementLayout *
        ]
    ),
    "cGcFrontendPageFunctions::SelectLayer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageFunctions::SetActiveSlotType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # eInventorySlotType
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPageFunctions::SetInventoryIcon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # eInventoryIcon
        ]
    ),
    "cGcFrontendPageFunctions::SetIconImage": {
        "cGcNGuiLayer *, const cTkTextureResource *, const cTkColour *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_ulonglong,  # const cTkTextureResource *
                ctypes.c_ulonglong,  # const cTkColour *
            ]
        ),
        "cGcNGuiLayer *, const TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *, const cTkColour *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_ulonglong,  # const TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
                ctypes.c_ulonglong,  # const cTkColour *
            ]
        ),
    },
    "cGcFrontendPageFunctions::SetPopupChemical": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcRealitySubstanceData *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcFrontendPageFunctions::TouchscreenPressButtonPressed": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcFrontendPageFunctions::SetColour": {
        "cTkNGuiGraphicStyle *, const cTkColour *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiGraphicStyle *
                ctypes.c_ulonglong,  # const cTkColour *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cTkNGuiTextStyle *, const cTkColour *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiTextStyle *
                ctypes.c_ulonglong,  # const cTkColour *
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cGcBaseBuildingManager::RequestAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_uint32,  # cGcBaseBuildingManager::BaseBuildingActions
        ]
    ),
    "cGcBaseBuildingManager::GetPlayerPlacement": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBaseBuildingPlayerPlacement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
        ]
    ),
    "cGcPlayerNotifications::HasBlockOnPinning": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
        ]
    ),
    "cGcFrontendPageBasePartPalette::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageBasePartPalette *
        ]
    ),
    "cGcFrontendPageBasePartPalette::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageBasePartPalette *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageBasePartPalette::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageBasePartPalette *
        ]
    ),
    "cGcByteBeat::SetSequence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
            ctypes.c_ulonglong,  # cGcByteBeatSequencer *
        ]
    ),
    "cGcInteractionComponent::AddInteractionEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_uint32,  # eInteractionEvent
        ]
    ),
    "cGcFrontendPageByteBeat::LoadIcons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Compute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTreeEntry *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcFrontendPageByteBeat::DoByteBeat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageByteBeat::RebuildGraphicalTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageByteBeat::SetOptionLabel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const cTkFixedString<32,char> *
        ]
    ),
    "cGcFrontendPageByteBeat::DoByteBeatTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcByteBeat *
        ]
    ),
    "cGcFrontendPageByteBeat::DoByteBeatSequencer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcByteBeat *
        ]
    ),
    "cGcFrontendPageByteBeat::HandleInput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeat *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "GetMaxTemplateDepth": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTemplate *
            ctypes.c_int32,  # int
        ]
    ),
    "CountTemplateBits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTemplate *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcFrontendPageByteBeat::GenerateNonSilentRandomTree": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcByteBeatTreeEntry *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # cGcByteBeatTreeEntry *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # cGcByteBeatTemplate *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageByteBeat::GenerateRandomTree": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcByteBeatTreeEntry *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # cGcByteBeatTreeEntry *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # cGcByteBeatTemplate *
        ]
    ),
    "cGcFrontendPageByteBeat::BuildInternalTree": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcByteBeatTreeEntry *
        argtypes=[
            ctypes.c_ulonglong,  # const cGcByteBeatTokenInfo *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # cGcByteBeatTreeEntry *
        ]
    ),
    "cGcFrontendPageByteBeat::BuildTokenList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTreeEntry *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # cGcByteBeatTokenInfo *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "cGcFrontendPageByteBeat::BuildGraphicalTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTreeEntry *
            ctypes.c_ulonglong,  # cGcFrontendPageTree::TreeBox *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageTree::TreeBox::TreeBox": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageTree::TreeBox *
        ]
    ),
    "cGcFrontendPageByteBeat::SetNodeFromToken": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageTree::TreeBox *
            ctypes.c_ulonglong,  # cGcByteBeatTokenInfo *
        ]
    ),
    "CalculateDepth": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # cGcByteBeatTreeEntry *
        ]
    ),
    "CountTreeBits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # cGcByteBeatTreeEntry *
        ]
    ),
    "cGcFrontendPageByteBeat::UpdateTreeNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcByteBeatTreeEntry *
        ]
    ),
    "cGcFrontendPageByteBeat::DoTooltip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcFrontendPageByteBeat::ClearRecursive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTreeEntry *
        ]
    ),
    "cGcFrontendPageByteBeat::DoHoverButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # cGcFrontendPage::ButtonIcon *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageByteBeat::RenderEmbeddedVisualisation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderMainVisualisation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderVisualisation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcByteBeatTreeEntry *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcFrontendPageByteBeat::RenderTreeLines": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::DrawLines": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTreeEntry *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderSequencer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderHighlight": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderSequencerStep": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cGcFrontendPageByteBeat::RenderSequencerArpStep": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # cTkVector2 *
            common.Colour,  # cTkColour
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cGcFrontendPageByteBeat::RenderThisPartSequencer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderConnectedPartSequencer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderPartSequencer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageByteBeat::RenderPlayHead": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageByteBeat::RenderRhythmSequencer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderRhythmStep": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcFrontendPageByteBeat::RenderArpeggio": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcFrontendPageByteBeat::RenderDrumMixSlider": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderArpRangeSlider": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderArpStepSlider": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderArpLengthSlider": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderArpTypeSlider": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderWaveTypeSlider": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderEnvFadeIn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderEnvFadeOut": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderEnvLength": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderNumStepsSlider": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "RenderPowerIcon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBBox2d *
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageByteBeat::RenderArpPowerButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderMelodyPowerButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderRhythmPowerButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderNoteOnButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderCopyButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderPasteButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderClearSequenceButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderClearRhythmButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "DrawRandomiseIcon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBBox2d *
            ctypes.c_ulonglong,  # const NVGcolor *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcFrontendPageByteBeat::RenderRandomiseSequenceButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderRandomiseRhythmButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderRandomiseArpeggioButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderRandomiseEnvelopeButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderRandomiseEverythingButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderSoloButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderRandomiseKickButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderRandomiseSnareButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderRandomiseHiHatButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderPlayButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageByteBeat::RenderEnvelope": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcFrontendPageByteBeat::RenderSlider": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cTkFixedString<32,char> *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['unsigned int'], 'pointer_depth': 1}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['unsigned int', 'cTkBBox2d const &', 'NVGcolor const &', 'bool'], 'pointer_depth': 1}
        ]
    ),
    "cGcFrontendPageByteBeat::GetRhythmTrackcolour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageByteBeat::GetMelodyTrackcolour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageClaimBase::ShouldDoSeason9CustomPuzzle": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "cGcFrontendPageClaimBase::DoBaseClaimOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # std::array<cGcNGuiLayer *,5> *
            ctypes.c_uint32,  # ePersistentBaseTypes
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # _WORD *
        ]
    ),
    "cGcFrontendPageClaimBase::IsPositionOverlappingPlanetBase": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # BaseIndex *
        ]
    ),
    "cGcFrontendPageCraftingTree::ShouldRequestCraftingTree": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcFrontendPageCraftingTree::DoCraftingTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageCraftingTree::FindNodeToHighlight": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFrontendPageTree::TreeBox *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageTree::TreeBox *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPageCraftingTree::BuildTreeItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageTree::TreeBox *
            ctypes.c_ubyte,  # const bool
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageCraftingTree::UpdateTreeNode": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcFrontendPageTree::TreeBox *
            ctypes.c_ulonglong,  # const std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
        ]
    ),
    "cGcFrontendPageCraftingTree::DoPopUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcFrontendPageTree::TreeBox *
            ctypes.c_ulonglong,  # const std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFeederComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFeederComponent *
        ]
    ),
    "cGcCreatureFeederComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcCreatureFeederComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFeederComponent *
        ]
    ),
    "cGcCreatureFeederComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFeederComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureFeederComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFeederComponent *
        ]
    ),
    "ComponentInstancercGcCreatureFeederComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcCreatureFeederComponent *
        ]
    ),
    "ComponentInstancercGcCreatureFeederComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcCreatureFeederComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcCreatureFeederComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcFrontendPageCreatureFood::GetFeeder": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageCreatureFood *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageCreatureFood::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageCreatureFood *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageCreatureFood::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageCreatureFood *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcCreatureHarvesterComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHarvesterComponent *
        ]
    ),
    "cGcCreatureHarvesterComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcCreatureHarvesterComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHarvesterComponent *
        ]
    ),
    "cGcCreatureHarvesterComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHarvesterComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureHarvesterComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHarvesterComponent *
        ]
    ),
    "ComponentInstancercGcCreatureHarvesterComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcCreatureHarvesterComponent *
        ]
    ),
    "ComponentInstancercGcCreatureHarvesterComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcFrontendPageCreatureHarvester::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageCreatureHarvester *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageCreaturePet::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
        ]
    ),
    "cGcDifficultySettingsConfirmHandlerFrontend::DoConfirm": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsConfirmHandlerFrontend *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageDifficultySettings::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageDifficultySettings::DoDifficultySetting": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsPageContext *
            ctypes.c_ulonglong,  # const cGcDifficultySettingsData *
            ctypes.c_ulonglong,  # cGcDifficultySettingsData *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_ulonglong,  # const eDifficultySetting *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcDifficultySettingEnum> *
            ctypes.c_uint32,  # eDifficultySettingEditability
        ]
    ),
    "cGcFrontendPageDifficultySettings::ValidateSubOption": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const eDifficultySetting *
            ctypes.c_ulonglong,  # const eDifficultySetting *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcFrontendPageDifficultySettings::DoPresets": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsPageContext *
            ctypes.c_ulonglong,  # eDifficultyPresetType *
        ]
    ),
    "cGcFrontendPageDifficultySettings::DoSettings": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsPageContext *
            ctypes.c_ulonglong,  # cGcDifficultySettingsData *
        ]
    ),
    "cGcFrontendPageDifficultySettings::DoSettingOptions": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsPageContext *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eDifficultySetting
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_uint32,  # eDifficultySettingEditability
        ]
    ),
    "cGcFrontendPageDifficultySettings::DoDifficultyDetailDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsPageContext *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcFrontendPageDifficultySettings::DoApplySettingsWarningPopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBitArray<unsigned __int64,1,2> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageDifficultySettings::HasChangesThatWouldDisableAchievements": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "cGcFrontendPageDifficultySettings::GetApplySettingsWarnings": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,2> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,2> *
        ]
    ),
    "cGcFrontendPageDifficultySettings::OnDifficultyConfirmClosed_Exit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageDifficultySettings::OnDifficultyConfirmClosed_Apply": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageDifficultySettings::cGcErrorMessageHandler::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageDifficultySettings::cGcErrorMessageHandler *
            ctypes.c_ulonglong,  # cGcDifficultySettingsPageContext *
        ]
    ),
    "cTkBitArray<unsigned __int64,1,128>::GetNumTrue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,128> *
        ]
    ),
    "cTkBitArray<uint,1,32>::GetNumTrue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned int,1,32> *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcByteBeatIcons>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcByteBeatIcons *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcByteBeatTemplates>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcByteBeatTemplates *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkStringUtilities<char>::FormatNumberLocalised<uint>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNGuiElement::SetPosition": {
        "cGcNGuiElement *, float, float, cGcNGuiElement::PositionType": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiElement *
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_uint32,  # cGcNGuiElement::PositionType
            ]
        ),
        "cGcNGuiElement *, const cTkVector2 *, cGcNGuiElement::PositionType": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiElement *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_uint32,  # cGcNGuiElement::PositionType
            ]
        ),
    },
    "cGcFrontendPage::PickFinished": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "hknpClosestHitCollector::getHits": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryIndex *
        argtypes=[
            ctypes.c_ulonglong,  # ItemPicking *
        ]
    ),
    "cGcFrontendPage::GetPickedItem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # ItemPicking *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cTkBBox2d::GetCenter": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkBBox2d *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "cTkNGui::IsMouseInside": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkBBox2d *
        ]
    ),
    "cGcNGuiText::SetText": {
        "cGcNGuiText *, const TkID<256> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiText *
                ctypes.c_char_p,  # const TkID<256> *
            ]
        ),
        "cGcNGuiText *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiText *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "cTkCollisionFilter::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,128> *
        ]
    ),
    "cGcFrontendPage::IsPicking": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcAlienPuzzleOption::cGcAlienPuzzleOption": {
        "cGcAlienPuzzleOption *, const cGcAlienPuzzleOption *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcAlienPuzzleOption *
                ctypes.c_ulonglong,  # const cGcAlienPuzzleOption *
            ]
        ),
        "cGcAlienPuzzleOption *, cGcAlienPuzzleOption *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcAlienPuzzleOption *
                ctypes.c_ulonglong,  # cGcAlienPuzzleOption *
            ]
        ),
    },
    "cGcEggMachineComponent::GetTypedComponent": {
        "cTkAttachment *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
        "int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcFrontendPageFunctions::SetPageTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageFunctions::SetColourNoAlpha": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyle *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageDiscovery::UpdateDiscoveries": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
        ]
    ),
    "cGcFrontendPageDiscovery::DoDiscoveries": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
        ]
    ),
    "cGcFrontendPageDiscovery::UpdateDisplayState": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
        ]
    ),
    "cGcFrontendPageDiscovery::DoDiscoveryPopupTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # const cGcDiscoveryPageData::DiscoveryPageInfoBase *
        ]
    ),
    "cGcFrontendPageDiscovery::SetAllPopupButtonsEmpty": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageDiscovery::DoUploadDiscoveryButtonAndInput": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cGcDiscoveryPageData::DiscoveryPageInfoBase *
        ]
    ),
    "cGcFrontendPageDiscovery::DoRenameDiscoveryButtonAndInput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
            ctypes.c_ulonglong,  # const cGcDiscoveryPageData::DiscoveryPageInfoBase *
        ]
    ),
    "cGcFrontendPageDiscovery::DoReportDiscoveryButtonAndInput": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cGcDiscoveryPageData::DiscoveryPageInfoBase *
        ]
    ),
    "cGcFrontendPageDiscovery::DoSectorPopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
            ctypes.c_ulonglong,  # const cGcDiscoveryPageData::DiscoveryPageSector *
        ]
    ),
    "cGcFrontendPageDiscovery::DoSystemPopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
            ctypes.c_ulonglong,  # const cGcDiscoveryPageData::DiscoveryPageSystem *
        ]
    ),
    "cGcFrontendPageDiscovery::DoPlanetPopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
            ctypes.c_ulonglong,  # const cGcDiscoveryPageData::DiscoveryPagePlanet *
        ]
    ),
    "cGcFrontendPageDiscovery::DoPlanetDiscoveryPreviewPopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_ulonglong,  # cGcDiscoveryPageData::DiscoveryPagePlanetInfo::DiscoveryPreview *
            ctypes.c_uint32,  # eDiscoveryType
        ]
    ),
    "cGcFrontendPageDiscovery::DoDiscoveryPopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
            ctypes.c_ulonglong,  # const cGcDiscoveryPageData::DiscoveryPageDiscovery *
            ctypes.c_uint32,  # eDiscoveryType
            ctypes.c_uint32,  # eRarity
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageDiscovery::DoDiscoveryPopupBaseInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFrontendPageDiscovery::DoDiscoveryUploadNotify": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageDiscovery::DiscoveryProgressGauge": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcFrontendPageDiscovery::DoSectorPanel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
        ]
    ),
    "cGcFrontendPageDiscovery::UploadAndReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryRecord *
            ctypes.c_uint32,  # const eRarity
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
        ]
    ),
    "cGcFrontendPageDiscovery::UploadAndRewardAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
        ]
    ),
    "cGcFrontendPageDiscovery::CanRename": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcFrontendPageDiscovery::GetDisplayStatus": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcFrontendPageDiscovery::SetDiscoveryDisplayStatus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # const cGcDiscoveryPageData::eDiscoveryDisplayStatus
            ctypes.c_ulonglong,  # const cGcDiscoveryPageData::DiscoveryPageInfoBase *
        ]
    ),
    "cGcFrontendPageDiscovery::DoSystemView": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
        ]
    ),
    "cGcFrontendPageDiscovery::DoSystemInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # const cGcDiscoveryPageData::DiscoveryPageSystem *
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
        ]
    ),
    "cGcFrontendPageDiscovery::DoSystemIcons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcDiscoveryPageData::DiscoveryPageSystem *
        ]
    ),
    "cGcFrontendPageDiscovery::DoPlanetView": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
        ]
    ),
    "cGcFrontendPageDiscovery::DoPlanetInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
            ctypes.c_ulonglong,  # const cGcDiscoveryPageData::DiscoveryPagePlanet *
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
        ]
    ),
    "cGcFrontendPageDiscovery::DoDiscoveryView": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
        ]
    ),
    "cGcFrontendPageDiscovery::GetDiscoveryHintString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
            ctypes.c_uint32,  # eTileType
            ctypes.c_uint32,  # eCreatureType
            ctypes.c_uint32,  # eRarity
            ctypes.c_uint32,  # eCreatureActiveTime
            ctypes.c_uint32,  # eCreatureHemiSphere
        ]
    ),
    "cGcFrontendPageDiscovery::DoDiscoveryInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
        ]
    ),
    "cGcCreatureRoleData::cGcCreatureRoleData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleData *
            ctypes.c_ulonglong,  # const cGcCreatureRoleData *
        ]
    ),
    "cGcFrontendPageDiscovery::DoDiscoveryTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcDiscoveryPageData::DiscoveryPageInfoBase *
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
        ]
    ),
    "cGcFrontendPageDiscovery::DoPlanetsDisplay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
            ctypes.c_uint32,  # ePlanetDisplayTargetDisplayState
        ]
    ),
    "cGcFrontendPageDiscovery::DoPlanetaryMappingView": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
        ]
    ),
    "cGcFrontendPageEggMachine::GetEggMachine": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageEggMachine::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageEggMachine *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageEggMachine::DoSubtitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageEggMachine *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "cGcFrontendPageEggMachine::GetInputSlotsLayers": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const std::vector<cGcNGuiLayer *,TkSTLAllocatorShim<cGcNGuiLayer *,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageEggMachine *
            ctypes.c_ulonglong,  # const std::vector<cGcNGuiLayer *,TkSTLAllocatorShim<cGcNGuiLayer *,8,-1> > *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "cGcFrontendPageEggMachine::DoSlot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageEggMachine *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
            ctypes.c_ulonglong,  # const std::vector<cGcNGuiLayer *,TkSTLAllocatorShim<cGcNGuiLayer *,8,-1> > *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcFrontendPageEggMachine::DoSlotDosage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageEggMachine *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
            ctypes.c_ulonglong,  # const std::vector<cGcNGuiLayer *,TkSTLAllocatorShim<cGcNGuiLayer *,8,-1> > *
        ]
    ),
    "cGcFrontendPageEggMachine::DoButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageEggMachine *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "cGcFrontendPageEggMachine::DoEggInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageEggMachine *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "cGcFrontendPageEggMachine::DoModifyingInfoSlotAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageEggMachine *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageEggMachine::GiveBackItemsOnClose": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageEggMachine *
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "cGcFrontendPageInfoPortal::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageInfoPortal *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageInfoPortal::DoGuideContent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageInfoPortal *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageInfoPortal::DoMilestonesContent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageInfoPortal *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageInfoPortal::DoCataloguesContent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageInfoPortal *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageInfoPortal::LoadStoriesPageData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageInfoPortal *
        ]
    ),
    "cGcFrontendPageInfoPortal::FindWikiHint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageInfoPortal *
            ctypes.c_ulonglong,  # const cGcWiki *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcFrontendPageInfoPortal::GetRandomJourneyStatIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageInfoPortal *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcFrontendPageInfoPortal::PrepareWikiItems": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageInfoPortal *
            ctypes.c_ulonglong,  # const cGcWiki *
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcStoryEntry,8,-1> *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcFrontendPageInteractions::CountListElements": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcFrontendPageInteractions::GetListElement": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageInteractions::ProcessDescription": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
        ]
    ),
    "cGcInteractionComponent::GetCustomisationDataTypeForInteraction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::UsesIntermediateInteraction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::ClearDialogueOverrides": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetMissionSeedOverride": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSeed *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # cTkSeed *
        ]
    ),
    "cGcInteractionComponent::GetPuzzle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcAlienPuzzleEntry *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetCustomFreighterLogText": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcByteBeatSwitchComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcFrontendPageInteractions::TeleporterNexusEndpoint::TeleporterNexusEndpoint": {
        "cGcFrontendPageInteractions::TeleporterNexusEndpoint *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcFrontendPageInteractions::TeleporterNexusEndpoint *
            ]
        ),
        "cGcFrontendPageInteractions::TeleporterNexusEndpoint *, const cGcFrontendPageInteractions::TeleporterNexusEndpoint *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcFrontendPageInteractions::TeleporterNexusEndpoint *
                ctypes.c_ulonglong,  # const cGcFrontendPageInteractions::TeleporterNexusEndpoint *
            ]
        ),
    },
    "cGcFrontendPageInteractions::CurrentLanguageRequiresFullWidthCharacterSupport": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "cGcFrontendPageInteractions::DoInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageInteractions::UpdateIntermediateInteractionPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageInteractions::DoIntermediateInteractionPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageInteractions::GetPetRadialColour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcFrontendPageInteractions::DoRadialInteractionPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "GetSelectedRadialElement": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcFrontendPageInteractions::RadialInteractionPageRenderFunction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcFrontendPageInteractions::DoTitleBar": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageInteractions::DoOption": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::array<cGcNGuiLayer *,5> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcAlienPuzzleOption *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageInteractions::DoRadialOption": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::array<cGcNGuiLayer *,5> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcAlienPuzzleOption *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageInteractions::DoTeleporterOption": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcTeleportEndpoint *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageInteractions::DoConsolePanel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageInteractions::SubstituteFleetDetailsIntoString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
        ]
    ),
    "cGcFrontendPageInteractions::SubstituteSpacePOIDetailsIntoString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
        ]
    ),
    "cGcFrontendPageInteractions::SubtituteMissionTimeIntoString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
        ]
    ),
    "cGcFrontendPageInteractions::SubtituteFoodDetailsIntoString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
        ]
    ),
    "cGcFrontendPageInteractions::SubstituteShipNameIntoString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
        ]
    ),
    "cGcFrontendPageInteractions::SubstituteWeaponNameIntoString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
        ]
    ),
    "cGcFrontendPageInteractions::SubstitutePlanetNameIntoString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
        ]
    ),
    "cGcFrontendPageInteractions::SubstituteCrewNamesIntoString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # const eAlienRace
        ]
    ),
    "cGcFrontendPageInteractions::SubtituteTechShopTypeIntoString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
        ]
    ),
    "cGcFrontendPageInteractions::SubstituteMessageBeaconMessageIntoString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
        ]
    ),
    "cGcFrontendPageInteractions::DoTalkPanel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eBracketsOverride
        ]
    ),
    "cGcFrontendPageInteractions::DoDescriptionPanel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageInteractions::DoTextFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageInteractions::DoInteractionFromEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # const cGcAlienPuzzleEntry *
        ]
    ),
    "cGcFrontendPageInteractions::DoOptionsFromEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # std::array<cGcNGuiLayer *,5> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageInteractions::UpdateOptionHighlighting": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageInteractions::AreAllOptionsBlocked": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcFrontendPageInteractions::DoResponseTextPanel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageInteractions::DoAdvancedInteractionFlow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # const cGcAlienPuzzleEntry *
        ]
    ),
    "cGcFrontendPageInteractions::DoNewTech": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageInteractions::DoNewProduct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageInteractions::DoPatchNotes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageInteractions::TidyUpTextureOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_uint32,  # const eCustomisationDataType
        ]
    ),
    "cGcFrontendPageInteractions::DoCustomisationDescriptorGroupOptions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_uint32,  # const eCustomisationDataType
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcCustomisationDescriptorGroupOptions *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcFrontendPageInteractions::DoCustomisationTextureGroupOptions": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_uint32,  # const eCustomisationDataType
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcCustomisationTextureGroup *
        ]
    ),
    "cGcFrontendPageInteractions::DoCustomisationPresetsOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_uint32,  # const eCustomisationDataType
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cTkDynamicArray<TkID<128> > *
        ]
    ),
    "cGcFrontendPageInteractions::DoCustomisationBoneScaleOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_uint32,  # const eCustomisationDataType
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcCustomisationBoneScales *
        ]
    ),
    "cGcFrontendPageInteractions::DoCustomisationColourGroupOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # const cGcCustomisationUI *
            ctypes.c_uint32,  # const eCustomisationDataType
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcCustomisationColourGroup *
        ]
    ),
    "cGcFrontendPageInteractions::DoCustomisationBannerGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcFrontendPageInteractions::DoCustomisationHoverTip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcFrontendPageInteractions::DoCustomisationPlayerTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageInteractions::DoCustomisationJetpack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
        ]
    ),
    "cGcFrontendPageInteractions::DoCustomisationFreighterEngine": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
        ]
    ),
    "cGcFrontendPageInteractions::RefreshCustomisationResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_uint32,  # const eCustomisationDataType
        ]
    ),
    "cGcFrontendPageInteractions::ChangeRaceDialog": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageInteractions::ChangePresetDialog": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageInteractions::AcceptChangesDialog": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageInteractions::ChangesOutfit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageInteractions::DoCharacterCustomisation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageInteractions::DoTeleporter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageInteractions::DoTeleporterNexus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageInteractions::FormatGridStatValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eLinkNetworkType
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        ]
    ),
    "cGcFrontendPageInteractions::DoBaseGridInputElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint32,  # GcPersistencyHandle
        ]
    ),
    "cGcFrontendPageInteractions::DoBaseGridPowerOutputElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageInteractions::DoHotspotInfoElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # char *
            ctypes.c_uint32,  # eHotspotType
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eInventoryClass
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageInteractions::DoBaseGridInfoElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const char *
            common.Colour,  # cTkColour
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcFrontendPageInteractions::DoBaseGridResourceOutputElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_uint32,  # GcPersistencyHandle
        ]
    ),
    "cGcFrontendPageInteractions::DoCapacityWheel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageInteractions::DoAnimatedWheelRate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFrontendPageInteractions::DoAnimateAndSetColour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_float,  # float
            common.Colour,  # cTkColour
        ]
    ),
    "cGcFrontendPageInteractions::DoAnimatedArrows": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_float,  # float
            common.Colour,  # cTkColour
        ]
    ),
    "cGcFrontendPageInteractions::DoByteBeatSwitchPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageInteractions::ByteBeatSwitchBaseNoteRenderFunction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcFrontendPageInteractions::ByteBeatSwitchArpNoteRenderFunction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcFrontendPageInteractions::ByteBeatSwitchRhythmRenderFunction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcFrontendPageInteractions::ByteBeatSwitchBeatRenderFunction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcFrontendPageInteractions::ByteBeatSwitchPanelRenderFunction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cTkBBox2d *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['cTkBBox2d const &', 'unsigned int', 'bool', 'bool'], 'pointer_depth': 1}
        ]
    ),
    "cGcFrontendPageInteractions::DoBaseGridPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageInteractions::DoGridInputOutputSlot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # GcPersistencyHandle
            ctypes.c_uint32,  # eTileType
        ]
    ),
    "cGcFrontendPageInteractions::OnConfirmReportMessageBeaconClosed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageInteractions::DoMessageModule": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageInteractions::DoByteBeat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageInteractions::DoScrollControls": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageInteractions::SetupOptionAndListForQuickPress": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageInteractions::SetupOptionsArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # std::array<cGcNGuiLayer *,5> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageInteractions::DoTeleperporterNexusThumbnailPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcFrontendPageInteractions::TeleporterNexusEndpoint *
            ctypes.c_ulonglong,  # cGcFrontendPageInteractions::eTeleporterNexusDestType *
            ctypes.c_ulonglong,  # int *
            ctypes.c_uint32,  # const eTeleportOfflineReason
        ]
    ),
    "cGcFrontendPageInteractions::DoTeleperporterNexusDescriptionPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcFrontendPageInteractions::TeleporterNexusEndpoint *
            ctypes.c_ulonglong,  # const cGcFrontendPageInteractions::eTeleporterNexusDestType *
        ]
    ),
    "cGcFrontendPageInteractions::TeleportWhileOnMPMissionDialog": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageInteractions::UpdateTeleportNexusFilter": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcFrontendPageInteractions::eTeleporterNexusDestType *
            ctypes.c_ulonglong,  # robin_hood::detail::Table<1,80,enum cGcFrontendPageInteractions::eTeleporterNexusDestType,unsigned short,robin_hood::hash<enum cGcFrontendPageInteractions::eTeleporterNexusDestType,void>,std::equal_to<enum cGcFrontendPageInteractions::eTeleporterNexusDestType>,-1> *
        ]
    ),
    "cGcFrontendPageInteractions::UpdateTeleportesNexusEndpoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrontendPageInteractions::eTeleporterNexusDestType *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # std::vector<cGcFrontendPageInteractions::TeleporterNexusEndpoint,TkSTLAllocatorShim<cGcFrontendPageInteractions::TeleporterNexusEndpoint,16,-1> > *
        ]
    ),
    "cGcFrontendPageInteractions::GenerateBaseEndpoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrontendPageInteractions::eTeleporterNexusDestType *
            ctypes.c_ulonglong,  # const cGcFrontendPageInteractions::eTeleporterNexusDestType *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # std::vector<cGcFrontendPageInteractions::TeleporterNexusEndpoint,TkSTLAllocatorShim<cGcFrontendPageInteractions::TeleporterNexusEndpoint,16,-1> > *
        ]
    ),
    "cGcFrontendPageInteractions::GenerateFreighterEndpoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrontendPageInteractions::eTeleporterNexusDestType *
            ctypes.c_ulonglong,  # const cGcFrontendPageInteractions::eTeleporterNexusDestType *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # std::vector<cGcFrontendPageInteractions::TeleporterNexusEndpoint,TkSTLAllocatorShim<cGcFrontendPageInteractions::TeleporterNexusEndpoint,16,-1> > *
        ]
    ),
    "cGcFrontendPageInteractions::ComposeBaseTeleporterEndpoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrontendPageInteractions::eTeleporterNexusDestType *
            ctypes.c_ulonglong,  # const cGcFrontendPageInteractions::eTeleporterNexusDestType *
            ctypes.c_ulonglong,  # const BaseIndex *
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # std::vector<cGcFrontendPageInteractions::TeleporterNexusEndpoint,TkSTLAllocatorShim<cGcFrontendPageInteractions::TeleporterNexusEndpoint,16,-1> > *
        ]
    ),
    "cGcFrontendPageInteractions::ComposeStationTeleporterEndpoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # std::vector<cGcFrontendPageInteractions::TeleporterNexusEndpoint,TkSTLAllocatorShim<cGcFrontendPageInteractions::TeleporterNexusEndpoint,16,-1> > *
        ]
    ),
    "cGcFrontendPageInteractions::UpdateFilterEntriesCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # robin_hood::detail::Table<1,80,enum cGcFrontendPageInteractions::eTeleporterNexusDestType,unsigned short,robin_hood::hash<enum cGcFrontendPageInteractions::eTeleporterNexusDestType,void>,std::equal_to<enum cGcFrontendPageInteractions::eTeleporterNexusDestType>,-1> *
        ]
    ),
    "cGcFrontendPageInteractions::ComposeMostRecentTeleporterEndpoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # std::vector<cGcFrontendPageInteractions::TeleporterNexusEndpoint,TkSTLAllocatorShim<cGcFrontendPageInteractions::TeleporterNexusEndpoint,16,-1> > *
            ctypes.c_ulonglong,  # const std::vector<enum eTeleporterType,TkSTLAllocatorShim<enum eTeleporterType,4,-1> > *
        ]
    ),
    "cGcFrontendPageInteractions::ComposeFeaturedTeleporterEndpoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # std::vector<cGcFrontendPageInteractions::TeleporterNexusEndpoint,TkSTLAllocatorShim<cGcFrontendPageInteractions::TeleporterNexusEndpoint,16,-1> > *
        ]
    ),
    "cGcFrontendPageInteractions::EnpointExistsInNexusEndpoints": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const std::vector<cGcFrontendPageInteractions::TeleporterNexusEndpoint,TkSTLAllocatorShim<cGcFrontendPageInteractions::TeleporterNexusEndpoint,16,-1> > *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
        ]
    ),
    "cGcFrontendPageInteractions::UpdateTeleporterNexusElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcTeleportEndpoint *
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPersistentBaseDifficultyData *
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_uint32,  # const eTeleportOfflineReason
        ]
    ),
    "cGcFrontendPageInteractions::UpdateTeleporterNexusElementGraphics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcTeleportEndpoint *
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # const eTeleportOfflineReason
        ]
    ),
    "cGcFrontendPageInteractions::UpdateTeleporterNexusElementIconsAndSubtitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcTeleportEndpoint *
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPersistentBaseDifficultyData *
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_uint32,  # const eTeleportOfflineReason
        ]
    ),
    "cGcFrontendPageInteractions::UpdateTeleporterNexusBaseDownloadStatus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrontendPageInteractions::TeleporterNexusEndpoint *
            ctypes.c_ulonglong,  # cGcFrontendPageInteractions::eTeleporterNexusBaseDownloadState *
            ctypes.c_ulonglong,  # BaseIndex *
        ]
    ),
    "cGcFrontendPageInteractions::UpdateSystemInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcFrontendPageInteractions::TeleporterNexusEndpoint *
        ]
    ),
    "cGcFrontendPageInteractions::UpdateTeleporterNexusBaseInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrontendPageInteractions::TeleporterNexusEndpoint *
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageInteractions::UpdateTeleporterNexusTeleportButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrontendPageInteractions::TeleporterNexusEndpoint *
            ctypes.c_ulonglong,  # const cGcFrontendPageInteractions::eTeleporterNexusBaseDownloadState *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageInteractions::DetectButtonLayerConfirmation": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageInteractions::DoTeleportAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # const cGcFrontendPageInteractions::TeleporterNexusEndpoint *
        ]
    ),
    "cGcFrontendPageInteractions::NetworkIconRenderFn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkCurveFunction::SmoothCD<cTkVector2,cTkVector2>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "WikiItemData::~WikiItemData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # WikiItemData *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcStoryEntry,8,-1> *
        ]
    ),
    "cGcSky::GetTimeOfDay": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
        ]
    ),
    "cGcScanEventManager::CollectInterstellarEvents": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cGcInterstellarEvent,TkSTLAllocatorShim<cGcInterstellarEvent,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # std::vector<cGcInterstellarEvent,TkSTLAllocatorShim<cGcInterstellarEvent,8,-1> > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureComponent::GetIsDead": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cTkRigidBody::GetAabb": {
        "cTkRigidBody *, cTkVector3 *, cTkVector3 *, const cTkMatrix34 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkRigidBody *
                ctypes.c_ulonglong,  # cTkVector3 *
                ctypes.c_ulonglong,  # cTkVector3 *
                ctypes.c_ulonglong,  # const cTkMatrix34 *
            ]
        ),
        "cTkRigidBody *, cTkPhysRelVec3 *, cTkPhysRelVec3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkRigidBody *
                ctypes.c_ulonglong,  # cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ]
        ),
        "cTkRigidBody *, cTkVector3 *, cTkVector3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkRigidBody *
                ctypes.c_ulonglong,  # cTkVector3 *
                ctypes.c_ulonglong,  # cTkVector3 *
            ]
        ),
    },
    "cGcFrontendPage::SetSelectedWikiPageType": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_uint32,  # eWikiPageType
        ]
    ),
    "hknpWorld::getNumBodies": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # ItemPicking *
        ]
    ),
    "cGcDiscoveryPageData::DiscoveryPagePlanetInfo::JumpToDiscoveriesPageBy01Pct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData::DiscoveryPagePlanetInfo *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcDiscoveryPageData::DiscoveryPagePlanetInfo::PreviousDiscoveriesPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData::DiscoveryPagePlanetInfo *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcDiscoveryPageData::DiscoveryPagePlanetInfo::NextDiscoveriesPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData::DiscoveryPagePlanetInfo *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "hknpConvexShapeFunctions::getNumberOfFaces": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData::DiscoveryPagePlanetInfo *
        ]
    ),
    "cGcDiscoveryPageData::GetAnimWaitTimer": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
        ]
    ),
    "cGcDiscoveryPageData::RequestCreatureAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
        ]
    ),
    "cGcDiscoveryPageData::PlayedFirstCreatureAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
        ]
    ),
    "cGcDiscoveryPageData::HasPlayedFirstCreatureAnim": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
        ]
    ),
    "cGcDiscoveryPageData::HasDiscoveryNode": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryPageData *
        ]
    ),
    "cGcSolarSystemQuery::GetPlanetInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlanetSize *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemQuery *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUserSettings::GetUIColourScheme": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
        ]
    ),
    "cGcFrontendPage::DiscoveryUpload": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcRealityManager::GetAlienPuzzle": {
        "cGcRealityManager *, const TkID<256> *, eInteractionType, eAlienRace, cTkSeed": FUNCDEF(
            restype=ctypes.c_ulonglong,  # const cGcAlienPuzzleEntry *
            argtypes=[
                ctypes.c_ulonglong,  # cGcRealityManager *
                ctypes.c_char_p,  # const TkID<256> *
                ctypes.c_uint32,  # eInteractionType
                ctypes.c_uint32,  # eAlienRace
                common.GcSeed,  # cTkSeed
            ]
        ),
        "cGcRealityManager *, const TkID<256> *, eInteractionType, eAlienRace, std::vector<enum eAlienPuzzleCategory,TkSTLAllocatorShim<enum eAlienPuzzleCategory,4,-1> > *, cTkSeed": FUNCDEF(
            restype=ctypes.c_ulonglong,  # const cGcAlienPuzzleEntry *
            argtypes=[
                ctypes.c_ulonglong,  # cGcRealityManager *
                ctypes.c_char_p,  # const TkID<256> *
                ctypes.c_uint32,  # eInteractionType
                ctypes.c_uint32,  # eAlienRace
                ctypes.c_ulonglong,  # std::vector<enum eAlienPuzzleCategory,TkSTLAllocatorShim<enum eAlienPuzzleCategory,4,-1> > *
                common.GcSeed,  # cTkSeed
            ]
        ),
        "cGcRealityManager *, const TkID<256> *, cTkSeed": FUNCDEF(
            restype=ctypes.c_ulonglong,  # const cGcAlienPuzzleEntry *
            argtypes=[
                ctypes.c_ulonglong,  # cGcRealityManager *
                ctypes.c_char_p,  # const TkID<256> *
                common.GcSeed,  # cTkSeed
            ]
        ),
    },
    "cGcPlayerNotifications::GetNotificationSequencesRenounceable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # std::vector<cGcNotificationSequence *,TkSTLAllocatorShim<cGcNotificationSequence *,8,-1> > *
        ]
    ),
    "cGcPlayerNotifications::HasMission": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcPlayerNotifications::HasMissionActive": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcFrontendPageJourney::DoJourney": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageJourney::DoPinnedOverallJourneyMedalBlock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcJourneyMedal *
        ]
    ),
    "cGcFrontendPageJourney::DoPinnedMedalBlock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # const cGcJourneyCategory *
        ]
    ),
    "cGcFrontendPageJourney::DoFactionMedalsBlock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcJourneyCategory *
        ]
    ),
    "cGcFrontendPageJourney::DoOverallJourneyMedal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcJourneyCategory *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageJourney::DoMedalBox": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcJourneyCategory *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageJourney::GetMedalIcon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkTextureResource *
        argtypes=[
            ctypes.c_ulonglong,  # const cGcJourneyMedal *
            ctypes.c_int32,  # const int
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPageJourney::GetProgressText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcJourneyMedal *
            ctypes.c_ulonglong,  # const cGcJourneyCategory *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcInteractionComponent::GetRaceName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_uint32,  # const eAlienRace
        ]
    ),
    "cGcWaypointComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcWaypointComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaypointComponent *
        ]
    ),
    "cGcWaypointComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaypointComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcWaypointComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcWaypointComponent *
        ]
    ),
    "ComponentInstancercGcAtmosphereEntryComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcWaypointComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcCreatureBaitComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentLoadInstancercGcWaypointComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentLoadInstancercGcWaypointComponent *
        ]
    ),
    "cGcRepairTechComponent::GetReturnPage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechComponent *
        ]
    ),
    "cGcFrontendPageMaintenance::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageMaintenance *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFrontendPageMaintenance::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageMaintenance *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageMaintenance::DoMaintenanceSlots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageMaintenance *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageManageFleet::GetTimeAsString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcFrontendPageManageFleet::OnFleetManagerDismissFrigateDialogClosed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageManageFleet::OnFrigateDetailsDismissFrigateDialogClosed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageManageFleet::DoManageFleet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageManageFleet::DoExpeditionDetails": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageManageFleet::DoExpeditionSelection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageManageFleet::DoPowerupSelection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcFleetExpedition *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageManageFleet::DoExpeditionDebrief": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageManageFleet::DoFrigateDetails": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageManageFleet::DoFrigateCaptain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageManageFleet::DoFrigateTraitDots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcFleetFrigate *
        ]
    ),
    "cGcFrontendPageManageFleet::DoFrigateTraits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcFleetFrigate *
        ]
    ),
    "cGcFrontendPageManageFleet::GetFeedAnimationProgress": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageManageFleet::DoExpeditionStatList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # eExpeditionCategory
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcFrontendPageManageFleet::GetFrigateClassName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_ulonglong,  # const cGcFleetFrigate *
        ]
    ),
    "cGcFrontendPageManageFleet::DoStatList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcFrontendPageManageFleet::DoExpeditionFrigateCountBox": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageManageFleet::DoFleetListEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcFleetFrigate *
            ctypes.c_uint32,  # eExpeditionCategory
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageManageFleet::DoExpeditionListEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cGcFleetExpedition *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageManageFleet::DoExpeditionStatus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiTextSpecial *
            ctypes.c_ulonglong,  # const cGcFleetExpedition *
            ctypes.c_ulonglong,  # const cGcFleetFrigate *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageManageFleet::DoFrigateClassBox": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # eFrigateClass
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcFrontendPageManageFleet::DoGridFlicker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageManageFleet::HideFleetNameAndLocation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageManageFleet::DoExpeditionTypeAndName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcFrontendPageManageFleet::DoExpeditionTypeDetailsPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # eExpeditionCategory
        ]
    ),
    "cGcFrontendPageManageFleet::DoExpeditionDifficulty": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageManageFleet::LeavePage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageManageFleet::CountTextSpecialListElements": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcFrontendPageManageFleet::ResetFeedFrigateInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageManageSettlement::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageManageSettlement *
        ]
    ),
    "cGcFrontendPageManageSettlement::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageManageSettlement *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageManageSettlement::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageManageSettlement *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageManageSettlement::Setup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageManageSettlement *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageManageSettlement::CleanUpAndClose": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageManageSettlement *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageManageSettlement::RollInitialStatsAndPerks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageManageSettlement *
        ]
    ),
    "cGcFrontendPageManageSettlement::SetupProductionInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageManageSettlement *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageManageSettlement::RenderHeader": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageManageSettlement *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageManageSettlement::RenderStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageManageSettlement *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageManageSettlement::RenderPerks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageManageSettlement *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageManageSettlement::RenderProduction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageManageSettlement *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageManageSettlement::RenderProductionSlot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageManageSettlement *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcFrontendPageManageSettlement::RenderDebt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageManageSettlement *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageManageSettlement::RenderUnowned": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageManageSettlement *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageManageSettlement::RenderReport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageManageSettlement *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageManageSettlement::OnConfirmAbandonSettlementClosed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageManageSettlement::GetStatFillColour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageManageSettlement *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eSettlementStatType
        ]
    ),
    "cGcFrontendPageMissionGiver::UpdateMissionPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageMissionGiver::UpdateMissionDescriptionPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageMissionGiver::UpdateMissionHandInPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageMissionGiver::HasLocalMissionInProgress": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcMissionItem::cGcMissionItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionItem *
            ctypes.c_ulonglong,  # const cGcMissionItem *
        ]
    ),
    "cGcFrontendPageMissionGiver::HasHandInWaitingFromOtherBoard": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcFrontendPageMissionGiver::DoMissionPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageMissionGiver::DoMissionListItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionItem *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcFEElementLayout *
        ]
    ),
    "cGcFrontendPageMissionGiver::LayoutReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFEElementLayout *
            ctypes.c_ulonglong,  # const cGcMissionItem::Reward *
            ctypes.c_uint32,  # const eMissionFaction
        ]
    ),
    "cGcFrontendPageMissionGiver::DoMissionHandInPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageMissionGiver::DoMissionRenouncePage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageMissionGiver::DoMissionDescriptionPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageMissionGiver::DoMissionItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcFrontendPageMissionGiver::DoScreenPanel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageMissionGiver::GenerateMissionRewardInfo": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # eMissionRewardType *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcFrontendPageMissionGiver::GetRewardInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # eMissionRewardType *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcFrontendPageMissionGiver::SelectMissionBoardMissionPrimaryRewardID": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcFrontendPageMissionGiver::FindRecursiveMissionStandingInfo": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcGenericMissionStage> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcFrontendPageMissionGiver::GetStandingInfo": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcFrontendPageMissionGiver::GetAllowedMissionsListForStandardBoard": FUNCDEF(
        restype=None,  # void
        argtypes=[
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcFrontendPageMissionGiver::GetAllowedMissionsList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # const eAlienRace
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcMissionItem,8,-1> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # std::vector<cGcMissionItem,TkSTLAllocatorShim<cGcMissionItem,8,-1> > *
            ctypes.c_uint32,  # eMissionBoardType
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageMissionGiver::CanUseMissionType": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # const eMissionType
            ctypes.c_uint32,  # const eMissionTable
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,11> *
        ]
    ),
    "cGcFrontendPageMissionGiver::IsAllowed": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionItem *
            ctypes.c_ulonglong,  # const std::vector<cGcMissionItem,TkSTLAllocatorShim<cGcMissionItem,8,-1> > *
            ctypes.c_uint32,  # const eAlienRace
            ctypes.c_ubyte,  # const bool
            ctypes.c_uint32,  # const eMissionTable
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,11> *
        ]
    ),
    "cGcFrontendPageMissionGiver::HasUnusedGuildFaction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionItem *
            ctypes.c_ulonglong,  # std::vector<cGcMissionItem,TkSTLAllocatorShim<cGcMissionItem,8,-1> > *
            ctypes.c_uint32,  # const eAlienRace
        ]
    ),
    "cTkStringUtilities<char>::TrimRightPreserveTags": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerExperienceDirector::PiratesIncoming": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPurchaseableItem::GetType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
        ]
    ),
    "cGcNGuiStyledString::StyledStringLayout::~StyledStringLayout": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyledString::StyledStringLayout *
        ]
    ),
    "cTkNGuiStyles::Blend": {
        "cTkNGuiTextStyleData *, cTkNGuiTextStyleData *, cTkNGuiTextStyleData *, float": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkNGuiTextStyleData *
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiTextStyleData *
                ctypes.c_ulonglong,  # cTkNGuiTextStyleData *
                ctypes.c_ulonglong,  # cTkNGuiTextStyleData *
                ctypes.c_float,  # float
            ]
        ),
        "cTkNGuiGraphicStyleData *, cTkNGuiGraphicStyleData *, cTkNGuiGraphicStyleData *, float": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkNGuiGraphicStyleData *
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiGraphicStyleData *
                ctypes.c_ulonglong,  # cTkNGuiGraphicStyleData *
                ctypes.c_ulonglong,  # cTkNGuiGraphicStyleData *
                ctypes.c_float,  # float
            ]
        ),
    },
    "cTkNGuiStyles::RenderText": {
        "cTkNGui *, void *, const char *, cTkBBox2d *, eNGuiInputType, cTkNGuiElementData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGui *
                ctypes.c_ulonglong,  # void *
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # cTkBBox2d *
                ctypes.c_uint32,  # eNGuiInputType
                ctypes.c_ulonglong,  # cTkNGuiElementData *
            ]
        ),
        "cTkNGui *, cTkNGuiElementData *, cTkNGuiTextStyleData *, const char *, float, float, float, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGui *
                ctypes.c_ulonglong,  # cTkNGuiElementData *
                ctypes.c_ulonglong,  # cTkNGuiTextStyleData *
                ctypes.c_char_p,  # const char *
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
        "cTkNGui *, cTkNGuiTextStyleData *, const char *, cTkBBox2d *, cTkNGuiElementData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGui *
                ctypes.c_ulonglong,  # cTkNGuiTextStyleData *
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # cTkBBox2d *
                ctypes.c_ulonglong,  # cTkNGuiElementData *
            ]
        ),
    },
    "cGcNGuiGame::PushStyle": {
        "cGcNGuiGame *, eNGuiGameTextType, cTkNGuiTextStyle *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiGame *
                ctypes.c_uint32,  # eNGuiGameTextType
                ctypes.c_ulonglong,  # cTkNGuiTextStyle *
            ]
        ),
        "cGcNGuiGame *, eNGuiGameGraphicType, cTkNGuiGraphicStyle *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiGame *
                ctypes.c_uint32,  # eNGuiGameGraphicType
                ctypes.c_ulonglong,  # cTkNGuiGraphicStyle *
            ]
        ),
    },
    "cGcMarkerRenderData::Render": {
        "void *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # void *
            ]
        ),
        "void *, cTkVector2": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # void *
                ctypes.c_char * 0x8,  # cTkVector2
            ]
        ),
    },
    "cGcNotifyGenericSection::CheckConditions": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotifyGenericSection *
        ]
    ),
    "cGcNotificationSequence::GetSectionToHideFromLogForIndex": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNotifyGenericSection *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcPlayerNotifications::HasAnyMissionActive": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcFrontendPageMissions::DoMissionLog": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageMultiplayerMissionGiver::DoMissionPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageMultiplayerMissionGiver::DoMultiplayerMissionItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionItem *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcFEElementLayout *
            ctypes.c_uint32,  # eMultiplayerMissionType
        ]
    ),
    "cGcFrontendPageMultiplayerMissionGiver::DoMissionDescriptionPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageMultiplayerMissionGiver::MakeParticipantsText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcFrontendPageMultiplayerMissionGiver::LayoutReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFEElementLayout *
            ctypes.c_ulonglong,  # const cGcMissionItem::Reward *
        ]
    ),
    "cGcFrontendPageMultiplayerMissionGiver::UpdateMissionDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageMultiplayerMissionGiver::HasSpecialMissionTypes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcFrontendPageMultiplayerMissionGiver::LeaveCurrentFireteamDialog": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageMultiplayerMissionGiver::StartMissionAloneDialog": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageMultiplayerMissionGiver::GetMultiplayerMissionsList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const eAlienRace
            ctypes.c_ulonglong,  # std::vector<cGcMissionItem,TkSTLAllocatorShim<cGcMissionItem,8,-1> > *
        ]
    ),
    "cGcFrontendPageMultiplayerMissionGiver::GetMultiplayerWeekendMissionsList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<cGcMissionItem,TkSTLAllocatorShim<cGcMissionItem,8,-1> > *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcMissionItem,8,-1> *
        ]
    ),
    "cGcFrontendPageNetworkSettings::LoadPage": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageNetworkSettings::DoOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageNetworkSettings::DoPlayers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageNetworkSettings::DoManageFriends": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageNetworkSettings::DoFriendCodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageNetworkSettings::DoBlockedPlayers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageNetworkSettings::DoPopupOption": {
        "cGcFrontendPage *, cGcNGuiLayer *, cGcNGuiLayer *, int, const TkID<256> *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcFrontendPage *
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_int32,  # int
                ctypes.c_char_p,  # const TkID<256> *
            ]
        ),
        "cGcFrontendPage *, cGcNGuiLayer *, cGcNGuiLayer *, int, cTkFixedString<64,char> *, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcFrontendPage *
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # cTkFixedString<64,char> *
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cGcFrontendPageNetworkSettings::DoPlayerList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_uint32,  # cGcNetworkConstants::LobbyType
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
        ]
    ),
    "AkMeterCtx::GetTruePeak": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearMemoryPool *
        ]
    ),
    "cGcFrontendPageOptions::IsUsingMode1Sensitivity": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[]
    ),
    "cGcFrontendPageOptions::IsUsingMode2Sensitivity": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "cGcFrontendPageOptions::ShowPopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcFrontendPageOptions::HidePopup": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::UpdateSensitivityMode": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::DoGeneralOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::OnSwitchToOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageOptions::DoSwitcher": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageOptions::PrepareGeneralOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::PrepareAccessibilityOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::PrepareControlsOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::PrepareCameraOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::PrepareVideoOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::PrepareGyroOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::PrepareSwitcherMenu": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::PrepareBootScreenNetworkOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::Apply": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::Exit": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::DoCredits": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::DoCreditsLine": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_float,  # const float
        ]
    ),
    "cGcFrontendPageOptions::DoRedeem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageOptions::DoControlsPC": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "sTkInputDeviceIcon::sTkInputDeviceIcon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sTkInputDeviceIcon *
            ctypes.c_ulonglong,  # const sTkInputDeviceIcon *
        ]
    ),
    "cGcFrontendPageOptions::ShowCredits": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::UnstuckOnFreighter": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::ShowControlsPage": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::ShowGyroOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::UpdateAvailableResolutions": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcVideoOptionsInterface::UpdatePresetOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVideoOptionsInterface *
        ]
    ),
    "cGcFrontendPageOptions::ApplyPreset": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::UpdateAntialiasingOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::UpdateAntialiasingSubOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::UpdateDLSSSubOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFrontendPageOptions::HasEnabledAdvancedGyroOptions": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[]
    ),
    "cGcFrontendPageOptions::GyroIsNotAttachedToScreen": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "cGcFrontendPageOptions::HasEnabledGyroGameplayOptions": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "cGcFrontendPageOptions::HasEnabledGyroCursorOptions": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[]
    ),
    "cGcFrontendPageOptions::ApplyPreviewAudioVolume": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcOption<bool>::IsNonDefault": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<bool> *
        ]
    ),
    "cGcOption<bool>::HasChanges": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<bool> *
        ]
    ),
    "cGcOption<bool>::DiscardChanges": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<bool> *
        ]
    ),
    "cGcOption<bool>::ResetToDefault": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<bool> *
        ]
    ),
    "cGcOption<bool>::Apply": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<bool> *
        ]
    ),
    "cGcOption<float>::IsNonDefault": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<float> *
        ]
    ),
    "cGcOption<float>::HasChanges": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<float> *
        ]
    ),
    "cGcOption<float>::DiscardChanges": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<float> *
        ]
    ),
    "cGcOption<float>::ResetToDefault": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOption<float> *
        ]
    ),
    "cTkInputManager::GetActionSetLOC": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_ulonglong,  # TkID<256> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcMPMissionTracker::GetPreMissionUA": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcUniverseAddressData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionTracker *
            ctypes.c_ulonglong,  # cGcUniverseAddressData *
        ]
    ),
    "ShopItemData::Set": {
        "ShopItemData *, const cGcRealitySubstanceData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # ShopItemData *
                ctypes.c_ulonglong,  # const cGcRealitySubstanceData *
            ]
        ),
        "ShopItemData *, const cGcProductData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # ShopItemData *
                ctypes.c_ulonglong,  # const cGcProductData *
            ]
        ),
        "ShopItemData *, const cGcTechnology *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # ShopItemData *
                ctypes.c_ulonglong,  # const cGcTechnology *
            ]
        ),
    },
    "ShopItemData::GeneratePrice": {
        "ShopItemData *, const bool, const bool, const cTkSeed, const ShopItemData::eShopItemPriceType, const bool": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # ShopItemData *
                ctypes.c_ubyte,  # const bool
                ctypes.c_ubyte,  # const bool
                common.GcSeed,  # const cTkSeed
                ctypes.c_uint32,  # const ShopItemData::eShopItemPriceType
                ctypes.c_ubyte,  # const bool
            ]
        ),
        "ShopItemData *, const bool, const bool, const float, const ShopItemData::eShopItemPriceType, const bool, const bool": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # ShopItemData *
                ctypes.c_ubyte,  # const bool
                ctypes.c_ubyte,  # const bool
                ctypes.c_float,  # const float
                ctypes.c_uint32,  # const ShopItemData::eShopItemPriceType
                ctypes.c_ubyte,  # const bool
                ctypes.c_ubyte,  # const bool
            ]
        ),
    },
    "cGcRealityManager::GetTradingClassData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcTradingClassData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_uint32,  # eTradingClass
        ]
    ),
    "cGcFrontendManager::HasQueuedPage": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
        ]
    ),
    "cGcFrontendInventorySearch::cGcFrontendInventorySearch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendInventorySearch *
        ]
    ),
    "TradeInventoryItem::TradeInventoryItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TradeInventoryItem *
        ]
    ),
    "cGcFrontendPageFunctions::AnyButtonPressed": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "cGcFrontendPagePortal::DoInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPagePortal *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPagePortalDisplayUa::DoInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPagePortalDisplayUa *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcNPCComponent::SetHoloEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcInteractionComponent::SetReinteractionQueued": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionComponent::IsBasicNPCInteraction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::IsShopInteraction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::CanTrade": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::CanTradeWithShipInventory": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::ShopNotifyBought": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_uint32,  # const eInventoryType
            ctypes.c_char * 0x10,  # const TkID<128>
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcInteractionComponent::ShopAmountBought": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
        ]
    ),
    "cGcHUDManager::GetChargingInventory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryStore *
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcFrontendPagePortalRunes::DoInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPagePortalRunes *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPagePortalRunes::CheckUAIsValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPagePortalRunes::TurnOffInventoryRunes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPagePortalRunes *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPagePortalRunes::ConvertPortalCodeToUA": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # const std::vector<signed char,TkSTLAllocatorShim<signed char,1,-1> > *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcFrontendPagePortalRunes::ConvertUAToPortalCode": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # std::vector<signed char,TkSTLAllocatorShim<signed char,1,-1> > *
        ]
    ),
    "cGcFrontendPagePortalRunes::ConvertPortalCodeHexToUA": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcFrontendPagePortalRunes::ConvertUAToPortalCodeHex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        ]
    ),
    "cGcRefinerUnitComponent::GetCurrentOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::GetCurrentMaxOutput": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRefinerRecipe *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::TimeToComplete": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::GetRequestedOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::SetRequestedOutput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcRefinerUnitComponent::StoreInitialProgress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::GetOutputSlotIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::GetFuelSlotIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcFrontendPageRefiner::GetRefiner": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageRefiner::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageRefiner *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageRefiner::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageRefiner *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcRewardManager::GiveGenericReward": {
        "cGcRewardManager *, const TkID<128> *, const TkID<128> *, const cTkSeed *, const bool, const bool, const bool, InventoryChoice, const bool": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcRewardManager *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ulonglong,  # const cTkSeed *
                ctypes.c_ubyte,  # const bool
                ctypes.c_ubyte,  # const bool
                ctypes.c_ubyte,  # const bool
                ctypes.c_uint32,  # InventoryChoice
                ctypes.c_ubyte,  # const bool
            ]
        ),
        "cGcRewardManager *, const TkID<128> *, const TkID<128> *, const cTkSeed *, const bool, const bool, int *, const bool, int, const bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcRewardManager *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ulonglong,  # const cTkSeed *
                ctypes.c_ubyte,  # const bool
                ctypes.c_ubyte,  # const bool
                ctypes.c_ulonglong,  # int *
                ctypes.c_ubyte,  # const bool
                ctypes.c_int32,  # int
                ctypes.c_ubyte,  # const bool
            ]
        ),
    },
    "cGcFrontendPageSeason::DoSeason": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageSeason::DoStagePanel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageSeason::UpdateStageSelectStar": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cGcFrontendPageSeason::DoFinalStagePanel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageSeason::DoMilestonesPanel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageSeason::DoMilestonePatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageSeason::DoLargePinnedMilestonePatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageSeason::DoFinalMilestonePatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageSeason::DoOptionalMilestonesReadout": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcFrontendPageSeason::DoStatPanel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageSeason::GetProgressText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_ulonglong,  # const cGcSeasonalMilestone *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageSeason::GetLongDescriptionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ulonglong,  # const cGcSeasonalMilestone *
            ctypes.c_ulonglong,  # const cGcNotificationSequence *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageSeason::FormatFinalObjectives": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_char * 0x20,  # TkID<256>
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerThirdPerson::ForceHideMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerThirdPerson *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageSettlementJudgement::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageSettlementJudgement::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageSettlementJudgement::DoRewardsText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_ulonglong,  # cGcFEElementLayout *
            ctypes.c_ulonglong,  # std::vector<cTkFixedString<128,char>,TkSTLAllocatorShim<cTkFixedString<128,char>,1,-1> > *
        ]
    ),
    "cGcFrontendPageSettlementJudgement::CleanUpAndClose": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
        ]
    ),
    "cGcFrontendPageSettlementJudgement::DeterminePerksToAward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
        ]
    ),
    "cGcFrontendPageSettlementJudgement::DeterminePerksToRemove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
        ]
    ),
    "cGcFrontendPageSettlementJudgement::GenerateRewardsText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_ulonglong,  # std::vector<cTkFixedString<128,char>,TkSTLAllocatorShim<cTkFixedString<128,char>,1,-1> > *
            ctypes.c_ulonglong,  # const cGcSettlementJudgementOption *
            ctypes.c_uint32,  # const cGcFrontendPageSettlementJudgement::eJudgementDecision
        ]
    ),
    "cGcFrontendPageSettlementJudgement::GetStatString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eSettlementStatType
        ]
    ),
    "cGcFrontendPageSettlementJudgement::ApplyDecisionEffects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
        ]
    ),
    "cGcFrontendPageSettlementJudgement::TriggerPendingJudgement": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPageSettlementJudgement::TriggerJudgement": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_uint32,  # const eSettlementJudgementType
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPageSettlementJudgement::GeneratePolicyBanSubject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_uint32,  # eSettlementJudgementType
        ]
    ),
    "cGcFrontendPageSettlementJudgement::ProcessDilemmaText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_char * 0x20,  # const TkID<256>
            ctypes.c_uint32,  # const eSettlementJudgementType
            ctypes.c_uint32,  # const eSettlementStatType
            ctypes.c_uint32,  # const eSettlementStatType
        ]
    ),
    "cGcFrontendPageSettlementJudgement::GetJudgementNPCNames": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcFrontendPageSettlementJudgement::GenerateGiftRewardDetails": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # TkID<256> *
        ]
    ),
    "cGcFrontendPageSettlementJudgement::GeneratePolicyPerkStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_ulonglong,  # eSettlementStatType *
            ctypes.c_ulonglong,  # eSettlementStatType *
        ]
    ),
    "cGcFrontendPageSettlementJudgement::GetPerkJobName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_ulonglong,  # TkID<256> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcFrontendPageSettlementJudgement::GenerateJudgementNPCName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_uint32,  # const eAlienRace
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcFrontendPageSettlementJudgement::CreateNPCs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
        ]
    ),
    "cGcFrontendPageSettlementJudgement::CreateNPC": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cGcNPCPlacementInfo *
            common.GcSeed,  # cTkSeed
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcFrontendPageSettlementJudgement::DestroyNPCs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
        ]
    ),
    "cGcFrontendPageSettlementJudgement::DoNPCReactions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPageSettlementJudgement::AddNewPerkMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_char * 0x10,  # const TkID<128>
        ]
    ),
    "cGcFrontendPageSettlementJudgement::AddFakePerkMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_char * 0x10,  # const TkID<128>
            ctypes.c_uint32,  # const eSettlementStatType
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcFrontendPageSettlementJudgement::ExtractKeyPerkData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSettlementPerkUsefulData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSettlementJudgement *
            ctypes.c_ulonglong,  # cGcSettlementPerkUsefulData *
            ctypes.c_char * 0x10,  # const TkID<128>
        ]
    ),
    "DoTradePageBottomBanner": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "DoBottomBanner": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageShop::DoTechTrade": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "ShopItemData::ShopItemData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ShopItemData *
        ]
    ),
    "cGcFrontendPageShop::DoBuildingTrade": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageShop::DoSpecialsTrade": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageShop::DoTrade": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageShop::PickBuyString": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcFrontendPageShop::DoShopShowItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageShop::DoShopCompareItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageShop::DoFreighterShipTransfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageShop::DoSquadronRecruitment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFrontendPageShop::DoInventoryTransfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageShop::DoAcceptButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageShop::DoPurchaseButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageShop::DoDeclineButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageShop::OnDeclineConfirmClosed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageShop::DoAddNewItemButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageShop::PopulateScrollableItemInventories": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageShop::DoRocketLockerTransfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageShop::DoCompareScreenStatsWindow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_uint32,  # eShipClass
            ctypes.c_uint32,  # eWeaponStatClass
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageShop::DoFrigateStatsWindow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcPurchaseableItem *
        ]
    ),
    "cGcFrontendPageShop::DoItemRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const TkStrongType<int,TkStrongTypeIDs::cTkNGuiFontHandleID> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageShop::ConvertToFullWidth": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFixedString<32,char> *
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        ]
    ),
    "cGcFrontendPageShop::GeneratePlayerInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::unordered_map<TkID<128>,TradeInventoryItem,TkIDUnorderedMap::Hash128,std::equal_to<TkID<128> >,TkSTLAllocatorShim<std::pair<TkID<128> const ,TradeInventoryItem>,8,-1> > *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageShop::GetTradeSettings": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTradeSettings *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageShop::GenerateSellerInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::unordered_map<TkID<128>,TradeInventoryItem,TkIDUnorderedMap::Hash128,std::equal_to<TkID<128> >,TkSTLAllocatorShim<std::pair<TkID<128> const ,TradeInventoryItem>,8,-1> > *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # const TradeOptions *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPageShop::GeneratePlanetTechShopInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::unordered_map<TkID<128>,TradeInventoryItem,TkIDUnorderedMap::Hash128,std::equal_to<TkID<128> >,TkSTLAllocatorShim<std::pair<TkID<128> const ,TradeInventoryItem>,8,-1> > *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcFrontendPageShop::GenerateGeneralTechShopInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::unordered_map<TkID<128>,TradeInventoryItem,TkIDUnorderedMap::Hash128,std::equal_to<TkID<128> >,TkSTLAllocatorShim<std::pair<TkID<128> const ,TradeInventoryItem>,8,-1> > *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_uint32,  # eTechnologyCategory
        ]
    ),
    "cGcFrontendPageShop::GenerateSpecialsShopInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::unordered_map<TkID<128>,TradeInventoryItem,TkIDUnorderedMap::Hash128,std::equal_to<TkID<128> >,TkSTLAllocatorShim<std::pair<TkID<128> const ,TradeInventoryItem>,8,-1> > *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcFrontendPageShop::GenerateSeasonRewardsInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::unordered_map<TkID<128>,TradeInventoryItem,TkIDUnorderedMap::Hash128,std::equal_to<TkID<128> >,TkSTLAllocatorShim<std::pair<TkID<128> const ,TradeInventoryItem>,8,-1> > *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcFrontendPageShop::GenerateTwitchRewardsInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::unordered_map<TkID<128>,TradeInventoryItem,TkIDUnorderedMap::Hash128,std::equal_to<TkID<128> >,TkSTLAllocatorShim<std::pair<TkID<128> const ,TradeInventoryItem>,8,-1> > *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcFrontendPageShop::GeneratePlatformRewardsInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::unordered_map<TkID<128>,TradeInventoryItem,TkIDUnorderedMap::Hash128,std::equal_to<TkID<128> >,TkSTLAllocatorShim<std::pair<TkID<128> const ,TradeInventoryItem>,8,-1> > *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcFrontendPageShop::GenerateShopElement": {
        "cGcInventoryElement *, const TkID<128> *, const eInventoryType, const cGcTradeData *, cGcInteractionComponent *, const int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcInventoryElement *
            argtypes=[
                ctypes.c_ulonglong,  # cGcInventoryElement *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_uint32,  # const eInventoryType
                ctypes.c_ulonglong,  # const cGcTradeData *
                ctypes.c_ulonglong,  # cGcInteractionComponent *
                ctypes.c_int32,  # const int
            ]
        ),
        "cGcInventoryElement *, const TkID<128> *, cGcTradingSupplyBuffer::TradingUA *, const eInventoryType, const cGcTradeData *, const int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcInventoryElement *
            argtypes=[
                ctypes.c_ulonglong,  # cGcInventoryElement *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ulonglong,  # cGcTradingSupplyBuffer::TradingUA *
                ctypes.c_uint32,  # const eInventoryType
                ctypes.c_ulonglong,  # const cGcTradeData *
                ctypes.c_int32,  # const int
            ]
        ),
    },
    "cGcFrontendPageShop::IsEverOfferedForSale": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "SetupRankLayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_uint32,  # const eAlienRace
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "DoDescriptionString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiTextSpecial *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # eInventoryType
            ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "DoPriceString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const TradeInventoryItem *
            ctypes.c_ubyte,  # const bool
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "DoPriceStringMaps": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const TradeInventoryItem *
            ctypes.c_int32,  # const int
        ]
    ),
    "DoPriceStringScrap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const TradeInventoryItem *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcFrontendPageShop::DoShopItem": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const TradeInventoryItem *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcFrontendPageShop::PopulateShopItems": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # std::unordered_map<TkID<128>,TradeInventoryItem,TkIDUnorderedMap::Hash128,std::equal_to<TkID<128> >,TkSTLAllocatorShim<std::pair<TkID<128> const ,TradeInventoryItem>,8,-1> > *
            ctypes.c_ulonglong,  # TradeInventoryItem *
            ctypes.c_ulonglong,  # cGcFrontendInventorySearch *
            ctypes.c_ulonglong,  # std::unordered_map<TkID<128>,int,TkIDUnorderedMap::Hash128,std::equal_to<TkID<128> >,TkSTLAllocatorShim<std::pair<TkID<128> const ,int>,8,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageShop::DoShopScrollBar": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageShop::DoCommunityMissionProgress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageShop::DoBuy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageShop::PurchaseCurrentPurchasableItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageShop::CheckTransferItemsDialog": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageShop::CheckTransferBaseDialog": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageShop::DoStorageContainerNameChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # InventoryData *
        ]
    ),
    "cGcFrontendPageShop::DoSwitchSpecialsShopFilter": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const std::pair<bool (__cdecl*)(cGcProductData const *),TkID<256> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcFrontendPageShop::SetPendingReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcRefinerRecipeElement>::end": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkArraySTLIterator<cGcRefinerRecipeElement>::InternalConstIterator *
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRefinerRecipeElement> *
            ctypes.c_ulonglong,  # cTkArraySTLIterator<cGcRefinerRecipeElement>::InternalConstIterator *
        ]
    ),
    "cGcFrontendPageSeason::DoCompleteTextSubstitutions<cTkFixedString<128,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_ulonglong,  # const cGcSeasonalMilestone *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPageSeason::DoCompleteTextSubstitutions<cTkFixedString<256,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # const cGcSeasonalMilestone *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cTkFixedString<127,char>::Format<char,0>": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkBehaviourTree::Broadcast<cGcMessageNPCBehaviourEvent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_ulonglong,  # const cGcMessageNPCBehaviourEvent *
        ]
    ),
    "cTkStringUtilities<char>::FormatNumberLocalised<unsigned __int64>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcPlayerState::QueryProductAmountInAllInventories": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPurchaseableItem::GetItemCargoInventory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryStore *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
        ]
    ),
    "cGcPurchaseableItem::GetCostId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcFrontendPage::SetSlotActionAutoScrollToBottomDone": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPurchaseableItem::GetTransferCargoInventory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryStore *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
        ]
    ),
    "cGcPurchaseableItem::GetTransferPlayerInventory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryStore *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
        ]
    ),
    "cGcPurchaseableItem::GetItemTechInventory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryStore *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
        ]
    ),
    "cGcPurchaseableItem::GetItemInventory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryStore *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
        ]
    ),
    "cGcFrontendPage::SetPurchasbaleItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # const cGcPurchaseableItem *
        ]
    ),
    "cGcFrontendPage::GetTradeOptionsWritable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TradeOptions *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPage::Count": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRefinerRecipeElement::cGcRefinerRecipeElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerRecipeElement *
        ]
    ),
    "cGcUnlockableItemTreeNode::~cGcUnlockableItemTreeNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTreeNode *
        ]
    ),
    "cGcStoneRuneData::cGcStoneRuneData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoneRuneData *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendManager::QueuedFrontendPages::PushPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager::QueuedFrontendPages *
            ctypes.c_uint32,  # const eFrontendPage
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcFrontendManager::QueueFrontendPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_uint32,  # eFrontendPage
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcFrontendPageSquadronPilotDetails::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
        ]
    ),
    "cGcPlayerNotifications::IsBasePartUnlockedBySilentMission": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcFrontendPageSquadronPilots::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
        ]
    ),
    "cGcGeneratorUnitComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcFrontendPageSystemHoover::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSystemHoover *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageSystemHoover::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageSystemHoover *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageSystemHoover::GetSystemHoover": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageTree::LinesRenderFunction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageTree::DrawLines": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageTree::TreeBox *
        ]
    ),
    "cGcFrontendPageTree::GetSubLayerPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcFrontendPageUnlockableTree::UpdateTreeNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcFrontendPageTree::TreeBox *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFrontendPageUnlockableTree::CurrencyData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageUnlockableTree::GetCantAffordString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFixedString<32,char> *
            ctypes.c_ulonglong,  # const cGcUnlockableTreeCostType *
        ]
    ),
    "cGcFrontendPageUnlockableTree::DoUnlockableTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageUnlockableTree::GenerateCurrencyDataFromCostID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcFrontendPageUnlockableTree::CurrencyData *
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrontendPageUnlockableTree::CurrencyData *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcFrontendPageUnlockableTree::DoPopUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # const cGcFrontendPageTree::TreeBox *
            ctypes.c_ulonglong,  # const ShopItemData *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ulonglong,  # const cGcFrontendPageUnlockableTree::CurrencyData *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPageUnlockableTree::DoTopBar": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # const cGcFrontendPageUnlockableTree::CurrencyData *
        ]
    ),
    "cGcFrontendPageUnlockableTree::GenerateCurrencyData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcFrontendPageUnlockableTree::CurrencyData *
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrontendPageUnlockableTree::CurrencyData *
            ctypes.c_ulonglong,  # const cGcUnlockableTreeCostType *
        ]
    ),
    "cGcFrontendPageUnlockableTree::Buy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrontendPageUnlockableTree::CurrencyData *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const cGcInventoryElement *
        ]
    ),
    "cGcFrontendPageTree::TreeBox::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageTree::TreeBox *
        ]
    ),
    "cGcFrontendPageUnlockableTree::Build": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageTree::TreeBox *
            ctypes.c_ulonglong,  # const cGcUnlockableItemTreeNode *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageTree::TreeBox::Draw": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageTree::TreeBox *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageTree::TreeBox::~TreeBox": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageTree::TreeBox *
        ]
    ),
    "cGcVehicleRaceInviteComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleRaceInviteComponent *
        ]
    ),
    "cGcVehicleRaceInviteComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcVehicleRaceInviteComponent::GetTypedComponent": {
        "int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_int32,  # int
            ]
        ),
        "cTkAttachment *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
    },
    "cGcVehicleRaceInviteComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleRaceInviteComponent *
        ]
    ),
    "cGcVehicleRaceInviteComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleRaceInviteComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcVehicleRaceInviteComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcVehicleRaceInviteComponent *
        ]
    ),
    "ComponentInstancercGcVehicleRaceInviteComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcVehicleCheckpointManager::RequestRaceStart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
            ctypes.c_ulonglong,  # cGcVehicleCheckpointComponent *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleCheckpointManager::HasValidRaceTrack": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
        ]
    ),
    "cGcFrontendPageVehicleRace::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageVehicleRace *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageVehicleRace::RenderResultsScreen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageVehicleRace *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageVehicleRace::RenderInviteScreen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageVehicleRace *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageVehicleRace::RenderMainScreen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPageVehicleRace *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageWiki::GetWiki": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWiki *
        argtypes=[
            ctypes.c_uint32,  # eWikiPageType
        ]
    ),
    "cGcFrontendPageWiki::GetWikiWritable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWiki *
        argtypes=[
            ctypes.c_uint32,  # eWikiPageType
        ]
    ),
    "cGcFrontendPageWiki::GetWikiPageData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWikiPageData *
        argtypes=[
            ctypes.c_uint32,  # const eWikiPageType
        ]
    ),
    "cGcFrontendPageWiki::DoWiki": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageWiki::DoStoriesWiki": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageWiki::GetListPageItemsData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<WikiItemData,TkSTLAllocatorShim<WikiItemData,16,-1> > *
            ctypes.c_uint32,  # const eWikiPageType
            ctypes.c_uint32,  # eWikiTopicType
        ]
    ),
    "cGcFrontendPageWiki::DoWikiListPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiGraphic *
            ctypes.c_ulonglong,  # std::vector<WikiItemData,TkSTLAllocatorShim<WikiItemData,16,-1> > *
        ]
    ),
    "cGcFrontendPageWiki::DoStoriesListPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiGraphic *
            ctypes.c_ulonglong,  # std::vector<WikiItemData,TkSTLAllocatorShim<WikiItemData,16,-1> > *
        ]
    ),
    "cGcFrontendPageWiki::DoGridPageSwitch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_uint32,  # eWikiTopicType
            ctypes.c_uint32,  # eWikiPageLayout
            ctypes.c_ulonglong,  # const cGcWikiCategory *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiGraphic *
        ]
    ),
    "cGcFrontendPageWiki::GetWikiGridItemsData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const eWikiPageType
            ctypes.c_uint32,  # const eWikiTopicType
            ctypes.c_ulonglong,  # const cGcWikiCategory *
            ctypes.c_uint32,  # const eAlienRace
            ctypes.c_ulonglong,  # std::vector<WikiItemData,TkSTLAllocatorShim<WikiItemData,16,-1> > *
        ]
    ),
    "cGcFrontendPageWiki::DoGridItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # WikiItemData *
            ctypes.c_uint32,  # eWikiPageLayout
        ]
    ),
    "cGcFrontendPageWiki::DoGridItemIcon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # WikiItemData *
        ]
    ),
    "cGcFrontendPageWiki::DoGridItemPlanet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # WikiItemData *
        ]
    ),
    "cGcFrontendPageWiki::DoGuideContentPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiGraphic *
        ]
    ),
    "cGcFrontendPageWiki::DoStoriesContentPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiGraphic *
            ctypes.c_ulonglong,  # const std::vector<WikiItemData,TkSTLAllocatorShim<WikiItemData,16,-1> > *
        ]
    ),
    "cGcFrontendPageWiki::MarkItemAsSeen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # WikiItemData *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageWiki::DoCustomWonderPopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const WikiItemData *
            ctypes.c_ulonglong,  # const int *
        ]
    ),
    "cGcFrontendPageWiki::DoConfirmViewWonderInDiscoveries": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcWonderHandle *
        ]
    ),
    "QSortRecipes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcFrontendPageWiki::FetchRecipesForCatalogue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<cGcRecipeWikiData,TkSTLAllocatorShim<cGcRecipeWikiData,16,20> > *
            ctypes.c_uint32,  # eWikiTopicType
        ]
    ),
    "cTkDynamicArray<cGcByteBeatTemplate>::~cTkDynamicArray<cGcByteBeatTemplate>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUnlockableItemTreeNode> *
        ]
    ),
    "cTkDynamicArray<cGcUnlockableItemTreeNode>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUnlockableItemTreeNode> *
        ]
    ),
    "cTkDynamicArray<cGcUnlockableItemTreeNode>::AllocateAndCopyFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUnlockableItemTreeNode> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcUnlockableItemTreeNode> *
        ]
    ),
    "cTkDynamicArray<cGcUnlockableItemTreeNode>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUnlockableItemTreeNode> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcRefinerRecipeElement>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRefinerRecipeElement> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageWiki::DoGridPage<WikiItemData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiGraphic *
            ctypes.c_ulonglong,  # std::vector<WikiItemData,TkSTLAllocatorShim<WikiItemData,16,-1> > *
            ctypes.c_uint32,  # eWikiPageLayout
        ]
    ),
    "cGcRecipeWikiData::cGcRecipeWikiData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRecipeWikiData *
            ctypes.c_ulonglong,  # const cGcRecipeWikiData *
        ]
    ),
    "WikiItemData::WikiItemData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # WikiItemData *
            ctypes.c_ulonglong,  # const WikiItemData *
        ]
    ),
    "cGcStoryEntry::cGcStoryEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryEntry *
            ctypes.c_ulonglong,  # const cGcStoryEntry *
        ]
    ),
    "cTkDynamicArray<cGcStoryEntryBranch>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStoryEntryBranch> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerState::QueryTechAmountInAllInventories": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPage::GetSelectedWikiPageType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcNGuiElement::GetAlignment": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
        ]
    ),
    "cGcRealityManager::LookupIconByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTechnology *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcEggMachineComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcFrontendManager::IsMenuPage": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_uint32,  # const eFrontendPage
        ]
    ),
    "cGcFrontendManager::PageGroup::InGroup": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager::PageGroup *
            ctypes.c_uint32,  # const eFrontendPage
        ]
    ),
    "cGcFrontendPageFunctions::SetSlotOutline": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcFrontendPageFunctions::CheckMoveAndTransfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcFrontendPage::SetPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_uint32,  # eFrontendPage
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPurchaseableItem::SetupSpecific": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInventoryLayout *
            ctypes.c_ulonglong,  # const cGcInventoryContainer *
            ctypes.c_uint32,  # cGcPurchaseableItem::ePurchaseableItem
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # const eSizeType
        ]
    ),
    "cGcPurchaseableItem::CalculateCost": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPurchaseableItem::FormatSuitSlotItemName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
        ]
    ),
    "cGcPurchaseableItem::Setup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # cGcPurchaseableItem::ePurchaseableItem
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPurchaseableItem::SetupSquadronPilot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPurchaseableItem::MakeInventoryDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
        ]
    ),
    "cGcPurchaseableItem::CancelPurchase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
        ]
    ),
    "cGcPurchaseableItem::SetTransferInventoryToPlayerInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
            ctypes.c_uint32,  # cGcPurchaseableItem::ePurchaseableItem
        ]
    ),
    "cGcPurchaseableItem::SetTransferInventoryToCargoInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
            ctypes.c_uint32,  # cGcPurchaseableItem::ePurchaseableItem
        ]
    ),
    "cGcPurchaseableItem::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventorySlotAction::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventorySlotAction *
        ]
    ),
    "cGcFrontendPage::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPage::ActivateHover": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # ePopupType
            ctypes.c_uint32,  # ePopupScreenAdjust
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ubyte,  # BOOL
        ]
    ),
    "cGcFrontendPage::ActivateAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # ePopupAction
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_ulonglong,  # cGcInventoryStore *
        ]
    ),
    "cGcFrontendPage::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFrontendPage::CanChangeStackSize": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPage::SetSelectedSubInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPage::SetSelectedInventoryItemIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_uint64,  # cGcInventoryIndex
        ]
    ),
    "cGcFrontendPage::RefreshSaveSlots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPage::GetPrimaryInventoryStore": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryStore *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPage::GetInventoryForPage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # eFrontendPage
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPage::CanCompletePicking": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPage::OnHover": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPage::DoPopups": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPage::SetInteractionImmediate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcFrontendPage::GetInteraction": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInteractionComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPage::GetMostRecentInteraction": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInteractionComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPage::SetMaintenanceImmediate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcFrontendPage::GetMaintenance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPage::GetMaintenanceIncludingPrevious": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPage::DeactivatePopups": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPage::ActivateInventorySlotAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # cGcInventorySlotAction::Style
        ]
    ),
    "cGcFrontendPage::Confirm": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # TkAudioID *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPage::ConfirmWithWarning": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPage::Transfer": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # FrontendHold
        ]
    ),
    "cGcFrontendPage::Upload": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # FrontendHold
        ]
    ),
    "cGcHoldInteraction::GetConfirmValue": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcHoldInteraction *
        ]
    ),
    "cGcHoldInteraction::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHoldInteraction *
        ]
    ),
    "cGcHoldInteraction::Confirm": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcHoldInteraction *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # FrontendHold
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # TkAudioID *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHoldInteraction::EndInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHoldInteraction *
        ]
    ),
    "cGcCountInteraction::Count": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCountInteraction *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "ItemPicking::GetSourceItem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryElement *
        argtypes=[
            ctypes.c_ulonglong,  # ItemPicking *
        ]
    ),
    "ItemPicking::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ItemPicking *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRepairTechComponent::ValidTech": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTechnology *
        ]
    ),
    "cGcRefinerUnitComponent::GetOutputId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "SendToInventoryOption::GetDestinationSlots": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SendToInventoryOption *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
        ]
    ),
    "SendToInventoryOption::GetDestinationInventory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedArray<cGcInventoryStore,28> *
        argtypes=[
            ctypes.c_ulonglong,  # SendToInventoryOption *
        ]
    ),
    "SendToInventoryOption::DoSend": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # SendToInventoryOption *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcFrontendPageFunctions::SetInventorySlot": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryStore *
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPageFunctions::SetItemSlot": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # eItemSlotTypes
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_uint32,  # eRarity
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcFrontendPageFunctions::SetItemSlotBorder": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcFrontendPageFunctions::SetItemSlotBorderRarity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # eRarity
        ]
    ),
    "cGcFrontendPageFunctions::SetPopupBasics": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # const int
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcFrontendPageFunctions::SetPopupDiscoveryTitleBar": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkFixedString<3,char> *
        ]
    ),
    "cGcFrontendPageFunctions::GetTradeCategoryString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_uint32,  # eTradeCategory
        ]
    ),
    "cGcFrontendPageFunctions::SetPopupIcon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkTextureResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # const int
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcFrontendPageFunctions::SetPopupTransfer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcFrontendPageFunctions::SetQuantityBar": {
        "cGcNGuiLayer *, const cGcInventoryElement *, eQuantityBarType, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_uint32,  # eQuantityBarType
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcNGuiLayer *, float, const cTkColour *, const cTkTextureResource *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # const cTkColour *
                ctypes.c_ulonglong,  # const cTkTextureResource *
            ]
        ),
        "cGcNGuiLayer *, const char *, int, int, const cTkColour *, eQuantityBarType, const char *, const cTkTextureResource *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_char_p,  # const char *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # const cTkColour *
                ctypes.c_uint32,  # eQuantityBarType
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # const cTkTextureResource *
            ]
        ),
    },
    "cGcFrontendPageFunctions::SetRequirements": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcTechnologyRequirement> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcTechnologyRequirement> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcRefinerRecipeElement> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageFunctions::DoSlot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const cGcInventoryStore *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcFrontendPageFunctions::DoChemicalSlot": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPageFunctions::GetFreighterStorageInventories": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
        ]
    ),
    "cGcFrontendPageFunctions::DoProductSlot": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPageFunctions::DoTechnologySlot": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ulonglong,  # const cGcTechnology *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPageFunctions::AllowedToMoveTech": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
        ]
    ),
    "cGcFrontendPageFunctions::CanTechEvolve": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTechnology *
        ]
    ),
    "cGcFrontendPageFunctions::GetEvolveQuality": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTechnology *
        ]
    ),
    "cGcFrontendPageFunctions::DoTechStatSection": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTechnology *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # eItemPopupLayer
            ctypes.c_uint64,  # const cGcInventoryIndex
            ctypes.c_ulonglong,  # const cGcInventoryStore *
        ]
    ),
    "cGcFrontendPageFunctions::HideUsedItemPopupElementsForWiki": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageFunctions::DoItemPopup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_uint32,  # eItemPopupLayer
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
        ]
    ),
    "cGcFrontendPageFunctions::IsItemProcUpgradeProduct": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cGcFrontendPageFunctions::LayerOverlapsPopup": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageFunctions::DoBaseStatsWindow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_uint32,  # eShipClass
            ctypes.c_uint32,  # eWeaponStatClass
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkFixedString<128,char> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageFunctions::DoLiveStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # eShipClass
            ctypes.c_uint32,  # eWeaponStatClass
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_uint32,  # eShipClass
        ]
    ),
    "cGcFrontendPageFunctions::CalculateShipStatWeapons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryStore *
        ]
    ),
    "cGcFrontendPageFunctions::CalculateShipStatShield": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryStore *
        ]
    ),
    "cGcFrontendPageFunctions::CalculateShipStatWarp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageFunctions::CalculateShipStatEngines": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # eShipClass
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryStore *
        ]
    ),
    "cGcFrontendPageFunctions::CalculateStatFleetBoosts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryStore *
        ]
    ),
    "cGcFrontendPageFunctions::DoClassBox": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # eInventoryClass
        ]
    ),
    "cGcFrontendPageFunctions::DoIterateLayers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFrontendPageFunctions::DoIterateAndScaleLayers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcFrontendPageFunctions::CanExitViaBackButton": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPageFunctions::SetStyleAlpha": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyle *
            ctypes.c_double,  # double
            ctypes.c_double,  # double
        ]
    ),
    "cGcFrontendPageFunctions::PopulateInventoryPageInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # std::vector<enum ePopupAction,TkSTLAllocatorShim<enum ePopupAction,4,-1> > *
            ctypes.c_ulonglong,  # std::vector<enum ePopupAction,TkSTLAllocatorShim<enum ePopupAction,4,-1> > *
        ]
    ),
    "cGcFrontendPageFunctions::DoInventoryExpandButton": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # TkID<256> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # const bool
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcFrontendPageFunctions::DoScrollingInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # std::vector<enum ePopupAction,TkSTLAllocatorShim<enum ePopupAction,4,-1> > *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # bool *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPageFunctions::DoInventoryPageTabs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
        ]
    ),
    "cGcFrontendPageFunctions::PopulateDynamicInventorySlots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # std::vector<enum ePopupAction,TkSTLAllocatorShim<enum ePopupAction,4,-1> > *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageFunctions::CanPickInteract": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
        ]
    ),
    "cGcFrontendPageFunctions::DoCrossplayPlatform": {
        "cGcNGuiLayer *, const cTkFixedString<64,char> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ]
        ),
        "cGcNGuiLayer *, cGcNetworkConstants::OnlinePlatformType": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_uint32,  # cGcNetworkConstants::OnlinePlatformType
            ]
        ),
    },
    "cGcFrontendPageFunctions::DoCrossplayDifficulty": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcDifficultyIconData *
        ]
    ),
    "cGcFrontendPageFunctions::CanInstallTech": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # const cGcTechnology *
        ]
    ),
    "cGcFrontendPageFunctions::DoInventorySlots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # std::vector<enum ePopupAction,TkSTLAllocatorShim<enum ePopupAction,4,-1> > *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_ubyte,  # const bool
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_ubyte,  # const bool
            ctypes.c_float,  # const float
        ]
    ),
    "cGcFrontendPageFunctions::DoSingleInventorySlot": {
        "cGcFrontendPage *, cGcNGuiLayer *, cGcInventoryStore *, const cGcInventoryIndex *, int, std::vector<enum ePopupAction,TkSTLAllocatorShim<enum ePopupAction,4,-1> > *, bool, bool, bool": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcFrontendPage *
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_ulonglong,  # cGcInventoryStore *
                ctypes.c_ulonglong,  # const cGcInventoryIndex *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # std::vector<enum ePopupAction,TkSTLAllocatorShim<enum ePopupAction,4,-1> > *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcFrontendPage *, cGcNGuiLayer *, cGcInventoryStore *, const cGcInventoryIndex *, const cGcInventoryElement *, int, std::vector<enum ePopupAction,TkSTLAllocatorShim<enum ePopupAction,4,-1> > *, bool, bool, bool": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcFrontendPage *
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_ulonglong,  # cGcInventoryStore *
                ctypes.c_ulonglong,  # const cGcInventoryIndex *
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # std::vector<enum ePopupAction,TkSTLAllocatorShim<enum ePopupAction,4,-1> > *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cGcFrontendPageFunctions::DoTechSlotPopupButtons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageFunctions::DoTransferPopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcFrontendPopup *
        ]
    ),
    "cGcFrontendPageFunctions::DoPopupCategories": FUNCDEF(
        restype=ctypes.c_ulonglong,  # int *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcFrontendPopup *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::cTkNGuiFontHandleID> *
        ]
    ),
    "cGcFrontendPageFunctions::ShowInventoryOutOfRangeMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcPlayerNotifications::TimedMessage,16,-1> *
        ]
    ),
    "cGcFrontendPageFunctions::DoTransferSendPopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcFrontendPopup *
        ]
    ),
    "cGcFrontendPageFunctions::TransferItem": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageFunctions::DoSlotToInventoryTransfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
        ]
    ),
    "cGcFrontendPageFunctions::GetHotSpotTypeForPage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPageFunctions::DoTechChargePopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcFrontendPopup *
        ]
    ),
    "cTkStackVector<cGcRealitySubstanceData const *,10,-1>::~cTkStackVector<cGcRealitySubstanceData const *,10,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStackVector<cGcRealitySubstanceData const *,10,-1> *
        ]
    ),
    "cGcFrontendPageFunctions::DoToolbar": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcFrontendManager::PageGroup *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageFunctions::PopulateCatagorisedItemInventory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # std::vector<cGcInventoryStore *,TkSTLAllocatorShim<cGcInventoryStore *,8,-1> > *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # cGcInventoryIndex *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # cGcFrontendInventorySearch *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageFunctions::CheckSlotHighlight": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPageFunctions::SetEmptySlotBackground": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPageFunctions::DoEmptySlotPopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # std::vector<enum ePopupAction,TkSTLAllocatorShim<enum ePopupAction,4,-1> > *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_ulonglong,  # cGcInventoryStore *
        ]
    ),
    "cGcFrontendPageFunctions::DoRepairSlotPopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # std::vector<enum ePopupAction,TkSTLAllocatorShim<enum ePopupAction,4,-1> > *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_ulonglong,  # cGcInventoryStore *
        ]
    ),
    "cGcFrontendPageFunctions::DoBuildPopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcFrontendPopup *
            ctypes.c_uint32,  # ePopupAction
        ]
    ),
    "cGcFrontendPageFunctions::DoScrollDots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageFunctions::DoScrolling_NoPage": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # const bool
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcFrontendPageFunctions::BuildSelectedItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPageFunctions::DoBuildInfoPopup": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPageFunctions::DoInventoryTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_char_p,  # const char *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcFrontendPageFunctions::DoModelLocators": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager::sModelLocator *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cGcFrontendPageFunctions::RenderLocatorArms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFrontendPageFunctions::DoPlayerBanner": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcFrontendPageFunctions::DoBanner": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcPlayerBanner *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcFrontendPageFunctions::IsCookingIngredient": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryElement *
        ]
    ),
    "cGcFrontendPageFunctions::GetCreatureEggIndexFromProduct": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProductData *
        ]
    ),
    "cGcFrontendPageFunctions::DoCreatureEggPopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcProductData *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcFrontendPageFunctions::GenerateCreatureEggText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const eFrontendPage *
            ctypes.c_ulonglong,  # const sOwnedCreatureInfo *
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # cTkDynamicString *
            ctypes.c_ulonglong,  # cTkDynamicString *
        ]
    ),
    "cGcFrontendPageFunctions::GetExtraEggDescriptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const sOwnedCreatureInfo *
            ctypes.c_ulonglong,  # TkID<256> *
            ctypes.c_ulonglong,  # TkID<256> *
        ]
    ),
    "cGcFrontendPageFunctions::HandleEggHatchingSuccess": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProductData *
        ]
    ),
    "cGcFrontendPageFunctions::HandleEggHatchingFailure": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProductData *
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
        ]
    ),
    "AtlasIconRenderFn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcFrontendPageFunctions::DoOnlineStatusBar": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPageFunctions::DoManualTextScrollbox": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # TextScrollBarState *
        ]
    ),
    "cTkAttachment::FindFirstAttachmentWithComponent<cGcAISpaceshipComponent>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachment *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkBehaviourTree::Broadcast<cGcMessageRequestTakeOff>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_ulonglong,  # const cGcMessageRequestTakeOff *
        ]
    ),
    "cTkDynamicString::AllocateFromString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicString *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcPlayerState::QuerySubstanceAmountInAllInventories": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcFrontendPage::PickItem": {
        "cGcFrontendPage *, const cGcInventoryIndex *, cGcInventoryStore *, ItemPicking::ActivationType": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcFrontendPage *
                ctypes.c_ulonglong,  # const cGcInventoryIndex *
                ctypes.c_ulonglong,  # cGcInventoryStore *
                ctypes.c_uint32,  # ItemPicking::ActivationType
            ]
        ),
        "cGcFrontendPage *, const cGcInventoryIndex *, int, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcFrontendPage *
                ctypes.c_ulonglong,  # const cGcInventoryIndex *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcFrontendPage::UpdatePick": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendPage::IsPickPackingTech": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPage::IsPickDeploy": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPage::GetPrimaryTechInventory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "ItemPicking::GetSplitSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # ItemPicking *
        ]
    ),
    "cGcFrontendPage::GetPrimaryInventory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPage::GetCachedFreighterInventories": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcPurchaseableItem::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
        ]
    ),
    "cGcSpaceshipWeapons::ClearLockTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcPlayerShipOwnership::GetThumbnail": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerShipOwnership *
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFrontendManager::PageGroup::GetNextPage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager::PageGroup *
            ctypes.c_uint32,  # const eFrontendPage
        ]
    ),
    "cGcFrontendManager::PageGroup::GetPreviousPage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager::PageGroup *
            ctypes.c_uint32,  # const eFrontendPage
        ]
    ),
    "cGcFrontendPopup::IsRightAligned": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPopup *
        ]
    ),
    "cGcFrontendPopup::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPopup *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_uint32,  # ePopupType
            ctypes.c_ubyte,  # BOOL
            ctypes.c_uint32,  # ePopupScreenAdjust
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcFrontendPopup::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPopup *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cGcFrontendPopup::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPopup *
        ]
    ),
    "cGcHUDManager::HasActiveDetailMessage": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cTkTextStyle::RenderPlain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # cTkTexture *
        ]
    ),
    "cTkTextStylePlain::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTextStylePlain *
            ctypes.c_ulonglong,  # cTk2dText *
        ]
    ),
    "cGcInteractionComponent::GetOverridePage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcFrontendPageCreaturePet::SetNeedsModelRefresh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendPageSquadronPilots::SetNeedsModelRefresh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
        ]
    ),
    "cGcFrontendManager::Preload": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendManager::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
        ]
    ),
    "cGcFrontendManager::AssignButtonIcons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
        ]
    ),
    "cGcFrontendManager::AddFreighterToInventoryPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
        ]
    ),
    "cGcFrontendManager::FinaliseDeactivation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
        ]
    ),
    "cGcFrontendManager::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFrontendManager::GetCurrentInitialPage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
        ]
    ),
    "cGcFrontendManager::Activate": {
        "cGcFrontendManager *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcFrontendManager *
            ]
        ),
        "cGcFrontendManager *, eFrontendPage, bool": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcFrontendManager *
                ctypes.c_uint32,  # eFrontendPage
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cGcFrontendManager::LoadPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcDiscoveryExport::SolarSystem,8,19> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendManager::PreloadDirectory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcFrontendManager::RebuildInventoryGroups": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
        ]
    ),
    "cGcFrontendManager::RebuildPauseGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendManager::UpdateAllowedInteractions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
        ]
    ),
    "cGcFrontendManager::ShouldRenderCursor": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
        ]
    ),
    "cGcFrontendManager::InterpretPageLocID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_uint32,  # const eFrontendPage
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcFrontendManager::UpdateSubInventorySelection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # std::vector<enum InventoryChoice,TkSTLAllocatorShim<enum InventoryChoice,4,-1> > *
        ]
    ),
    "cGcFrontendManager::RequestInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_uint32,  # eFrontendPage
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendManager::RequestMaintenancePage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcFrontendManager::DisplayTechMulti": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_ulonglong,  # const cTkDynamicArray<TkID<128> > *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcFrontendManager::DisplayTech": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_ubyte,  # const bool
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcFrontendManager::DisplaySpecialProduct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcFrontendManager::DisplayMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcFrontendManager::TransferBetweenInventories": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_ulonglong,  # const InventoryTransferData *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<InventoryData,8,-1> *
        ]
    ),
    "cGcFrontendManager::HandleInteractionsReset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
        ]
    ),
    "cGcFrontendManager::TerminatePendingInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcFrontendManager::Deactivate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendManager::IsShowingMissionHandInRewards": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
        ]
    ),
    "cGcFrontendManager::IsCurrentPageInventory": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
        ]
    ),
    "cGcFrontendManager::AddDiscoveryThumbnailRequest": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcFrontendManager::EnqueueRender3D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendManager::Render2D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcFrontendManager::DrawCursor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcFrontendManager::DrawFrontendCursor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcFrontendManager::RenderPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_int64,  # __int64
            ctypes.c_double,  # double
        ]
    ),
    "cGcFrontendManager::RenderPCOSK": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcFrontendTextInput *
        ]
    ),
    "cGcFrontendManager::AreShipOperationsAllowed": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
        ]
    ),
    "cGcFrontendManager::AllowedToOpenFrontendPages": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
        ]
    ),
    "cGcFrontendManager::UpdateDialogBox": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcFrontendManager::ShowDialogBox": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['void *', 'bool', 'bool']}
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendManager::ShowQuitDialog": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['void *', 'bool', 'bool']}
        ]
    ),
    "cGcFrontendManager::IsInteractingWithCreature": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendManager *
        ]
    ),
    "lookup_ACESmin": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "lookup_ACESmax": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "init_TsParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TsParams *
        argtypes=[
            ctypes.c_ulonglong,  # TsParams *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "inv_ssts": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # TsParams *
        ]
    ),
    "cGcFullscreenEffectManager::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFullscreenEffectManager *
        ]
    ),
    "cGcFullscreenEffectManager::UpdateLightingParams": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFullscreenEffectManager *
        ]
    ),
    "cGcFullscreenEffectManager::UpdateDoF": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFullscreenEffectManager *
        ]
    ),
    "SSRActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "cGcFullscreenEffectManager::IsSSROn": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcFullscreenEffectManager *
        ]
    ),
    "cGcFullscreenEffectManager::UpdateHDR": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFullscreenEffectManager *
        ]
    ),
    "cGcFullscreenEffectManager::UpdateScreenFilter": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFullscreenEffectManager *
        ]
    ),
    "cGcFullscreenEffectManager::SetScreenFilter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFullscreenEffectManager *
            ctypes.c_uint32,  # eScreenFilter
            ctypes.c_uint32,  # eFilterSlot
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "GetLensDistortionVectors": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector4 *
            ctypes.c_ulonglong,  # cTkVector4 *
        ]
    ),
    "cGcCharacterInterfaceComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterInterfaceComponent *
        ]
    ),
    "cGcCharacterInterfaceComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterInterfaceComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcCharacterInterfaceComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcCharacterInterfaceComponent *
        ]
    ),
    "ComponentInstancercGcCharacterInterfaceComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcLogoSequence::Unload": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLogoSequence *
        ]
    ),
    "cGcLogoSequence::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLogoSequence *
        ]
    ),
    "cGcGraphicsManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGraphicsManager *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGraphicsManager::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGraphicsManager *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cTkTypedSmartResHandle<cTkResource>,8,-1> *
        ]
    ),
    "cGcGraphicsManager::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGraphicsManager *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcGraphicsManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGraphicsManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcGraphicsManager::SetShaderUniformDefault": {
        "cGcGraphicsManager *, const char *, double, double, float, float": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcGraphicsManager *
                ctypes.c_char_p,  # const char *
                ctypes.c_double,  # double
                ctypes.c_double,  # double
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
        "cGcGraphicsManager *, unsigned __int64, double, double, float, float": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcGraphicsManager *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_double,  # double
                ctypes.c_double,  # double
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
    },
    "cGcGraphicsManager::SetShaderUniformArrayDefault": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGraphicsManager *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # float *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcGraphicsManager::LoadSequenceNvgOverlay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGraphicsManager *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcGraphicsManager::RenderMemoryPoolOverlayNVG": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "cGcGraphicsManager::UpdateAntiAliasingSettings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGraphicsManager *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGraphicsManager::ModelViewAllowedComponentsCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcGraphicsManager::ModelViewAttachmentPrepareCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkMetadataCache<cGcNGuiLayerData>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayerData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcNGuiLayerData> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcNGuiLayerData>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cGcNGuiLayerData> *
        argtypes=[]
    ),
    "cTkMetaData::ReadFromFile<cGcCustomisationUIData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCustomisationUIData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcScreenFilterTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcScreenFilterTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcBootLogoData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBootLogoData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcFontTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFontTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cTkAnimDetailSettingsTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimDetailSettingsTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcNGuiLayerData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayerData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFullBodyIKComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcTargetHelper::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetHelper *
        ]
    ),
    "cGcPlayerInteract::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcFrontendPage::IsPickMovingTech": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendModelRenderer::ShowScanEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFrontendPage::IsPickCharging": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcFrontendPage::AddNewItemPopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # const cGcFrontendPage::sNewItemPopupData *
        ]
    ),
    "cGcFrontendModelRenderer::SetPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_uint32,  # const eFrontendPage
        ]
    ),
    "cGcFrontendModelRenderer::ClearActivePage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
        ]
    ),
    "cGcFrontendModelRenderer::SetFade": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_float,  # float
        ]
    ),
    "cTkBase2DRenderer::PushMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBase2DRenderer *
        ]
    ),
    "cGcQuickMenu::Open": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenu *
        ]
    ),
    "cGcHUDManager::RenderNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcHUDManager::CollectedResource::CollectedResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager::CollectedResource *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_float,  # const float
            ctypes.c_char_p,  # const char *
            ctypes.c_float,  # const float
            ctypes.c_int32,  # const int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # cTkSmartResHandle *
        ]
    ),
    "cGcHUDManager::CollectedResource::CreateAmountString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager::CollectedResource *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcPhotoModeUI::IsActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeUI *
        ]
    ),
    "cGcInteractionComponent::GetInteractInWorldLocator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcInteractionComponent::GetOverrideInWorldLocator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcInWorldUIManager::SetScreenVisible": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_uint64,  # cTkClassPoolHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInWorldUIManager::ApplyTransforms": {
        "cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cGcInWorldUIScreenData *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            argtypes=[
                ctypes.c_ulonglong,  # cTkPhysRelMat34 *
                ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
                ctypes.c_ulonglong,  # const cGcInWorldUIScreenData *
            ]
        ),
        "cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkVector3 *, double, const cTkVector3 *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            argtypes=[
                ctypes.c_ulonglong,  # cTkPhysRelMat34 *
                ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_double,  # double
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
    },
    "cGcHUDManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_double,  # double
        ]
    ),
    "cGcHUDManager::UpdateSolidOverlayTransition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_double,  # double
        ]
    ),
    "cGcHUDManager::UpdateSolidOverlayUniforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDManager::IsStageActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_uint32,  # cGcHUDManager::RenderStage
        ]
    ),
    "cGcHUDManager::EnqueueRender3D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::RenderPostScene": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::RenderPostSceneTrackArrows": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::RenderPostSceneStatMessages": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::RenderPostSceneOSDMessages": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::RenderPostSceneSkipMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDManager::RenderMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_ulonglong,  # OSDMessage *
        ]
    ),
    "cGcHUDManager::RenderMessage_Internal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # OSDMessage *
        ]
    ),
    "cGcHUDManager::RenderMissionMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_ulonglong,  # MissionMessage *
        ]
    ),
    "cGcHUDManager::RenderMissionDetailMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_ulonglong,  # MissionDetailMessage *
        ]
    ),
    "cGcHUDManager::RenderNotifications": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::UpdateThumbnailQueue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcHUDManager::GetModelResourceRenderer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkModelResourceRenderer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::EnqueueRenderThumbnailQueue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::RenderOffscreen2D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::RenderImmediate2D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcHUDManager::RenderEffectLayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::IsRenderingInFront": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::SetBooting": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::CancelIntro": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDManager::IsDisabled": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::RenderDiscoveryInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDManager::UpdateIntroSequence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcHUDManager::Release": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::SetCinematicMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcHUDManager::AddOSDMessageResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_int32,  # const int
            ctypes.c_ubyte,  # const bool
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # cTkSmartResHandle *
        ]
    ),
    "cGcHUDManager::GenerationMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcHUDManager::PrepareAndAddCreatureMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcHUDManager::AddSeasonMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_ulonglong,  # const OSDMessage *
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcPlayerNotifications::TimedMessage,16,-1> *
        ]
    ),
    "cGcHUDManager::AddStoryMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_char_p,  # const char *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcHUDManager::AddCommunityTierCompleteMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_uint32,  # const eCommunityMissionTierMessage
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcHUDManager::AdjustPositionForHUDScaling": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cGcHUDManager::ShowShipOutOfRangeMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "cGcHUDManager::ShowFreighterOutOfRangeMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "cGcHUDManager::ShowVehicleOutOfRangeMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "cGcHUDManager::ShowShipBrokenMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcHUDManager::ShowInventoryFullMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcHUDManager::ShowInsufficientFundsMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcHUDManager::ShowTwitchRewardLockedMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcHUDManager::ShowTwitchOwnedMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcHUDManager::ShowTechnologyAwardedMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcHUDManager::ShowTechnologyAlreadyKnownMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcHUDManager::GetLocalisedTemperature": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcHUDManager::RemoveActiveMarker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "OSDMessageEffectReward::RareReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcVehicleComponent::GetInWorldLocator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcVehicleComponent::RequestScan": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcInWorldUIManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
        ]
    ),
    "cGcInWorldUIManager::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
        ]
    ),
    "cGcInWorldUIManager::PrepareGameModeSelect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
        ]
    ),
    "cGcInWorldUIManager::ActivateScreenAndClear": {
        "cGcInWorldUIManager *, eInWorldScreens": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcInWorldUIManager *
                ctypes.c_uint32,  # eInWorldScreens
            ]
        ),
        "cGcInWorldUIManager *, cTkClassPoolHandle, __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcInWorldUIManager *
                ctypes.c_uint64,  # cTkClassPoolHandle
                ctypes.c_int64,  # __int64
            ]
        ),
    },
    "cGcInWorldUIManager::ActivateScreen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_uint32,  # eInWorldScreens
        ]
    ),
    "cGcInWorldUIManager::SetScreenPixelSizeOverride": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_uint32,  # eInWorldScreens
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cGcInWorldUIManager::SetScreenPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_uint64,  # cTkClassPoolHandle
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcInWorldUIManager::SetScreenTransforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_uint64,  # cTkClassPoolHandle
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cGcInWorldUIScreenData *
        ]
    ),
    "cGcInWorldUIManager::SetScreenAsObjectLabel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_uint32,  # eInWorldScreens
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcInWorldUIManager::ClampPointToOnScreen": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcInWorldUIManager::BeginGameGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcInWorldUIManager::DeactivateScreen": {
        "cGcInWorldUIManager *, eInWorldScreens": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcInWorldUIManager *
                ctypes.c_uint32,  # eInWorldScreens
            ]
        ),
        "cGcInWorldUIManager *, cTkClassPoolHandle": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcInWorldUIManager *
                ctypes.c_uint64,  # cTkClassPoolHandle
            ]
        ),
    },
    "cGcInWorldUIManager::UpdateUIOffsetScale": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_double,  # double
            ctypes.c_double,  # double
            ctypes.c_double,  # double
        ]
    ),
    "cGcInWorldUIManager::UpdateHUDOffsetScale": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_float,  # float
            ctypes.c_double,  # double
        ]
    ),
    "cGcInWorldUIManager::GetCompassValueFromWorld": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_float,  # const float
            ctypes.c_int32,  # int
        ]
    ),
    "cGcInWorldUIManager::GetRayUIQuadIntersectionPoint": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cGcInWorldUIManager::GetRayUIQuadIntersectionPointAsScreenPos": {
        "cGcInWorldUIManager *, eInWorldScreens, const cTkPhysRelVec3 *, const cTkPhysRelVec3 *, const cTkVector2 *, cTkVector2 *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcInWorldUIManager *
                ctypes.c_uint32,  # eInWorldScreens
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # cTkVector2 *
            ]
        ),
        "cGcInWorldUIManager *, const cTkPhysRelMat34 *, const cTkVector2 *, const cTkVector2 *, const cTkVector2 *, const cTkPhysRelVec3 *, const cTkPhysRelVec3 *, const cTkVector2 *, cTkVector2 *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcInWorldUIManager *
                ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # cTkVector2 *
            ]
        ),
    },
    "cGcInWorldUIManager::GetScreenMatrix": {
        "cGcInWorldUIManager *, cTkPhysRelMat34 *, eInWorldScreens": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            argtypes=[
                ctypes.c_ulonglong,  # cGcInWorldUIManager *
                ctypes.c_ulonglong,  # cTkPhysRelMat34 *
                ctypes.c_uint32,  # eInWorldScreens
            ]
        ),
        "cGcInWorldUIManager *, cTkPhysRelMat34 *, cTkClassPoolHandle": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            argtypes=[
                ctypes.c_ulonglong,  # cGcInWorldUIManager *
                ctypes.c_ulonglong,  # cTkPhysRelMat34 *
                ctypes.c_uint64,  # cTkClassPoolHandle
            ]
        ),
    },
    "cGcInWorldUIManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcInWorldUIManager::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_int64,  # __int64
            ctypes.c_double,  # double
        ]
    ),
    "cGcInWorldUIManager::UpdateFaceLockedVR": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
        ]
    ),
    "cGcInWorldUIManager::RenderWeaponMenu": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
        ]
    ),
    "cGcInWorldUIManager::RenderQuickMenuLauncher": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
        ]
    ),
    "cGcInWorldUIManager::UpdateInWorldUISimulation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcInWorldUIManager::UpdateQuickMenu": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
        ]
    ),
    "cGcInWorldUIManager::UpdateProjectorDrone": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcInWorldUIManager::UpdateHeadCoherentUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcInWorldUIManager::UpdateInWorldUILocalLoad": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
        ]
    ),
    "cGcInWorldUIManager::UpdateInWorldUIGalaxyMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
        ]
    ),
    "cGcInWorldUIManager::GetMenuTransform": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_uint32,  # eMenuPositioning
        ]
    ),
    "cGcInWorldUIManager::CanShowScreen": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_uint32,  # eInWorldScreens
        ]
    ),
    "cGcInWorldUIManager::ApplyActiveProjectorTransforms": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkVector4 *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcInWorldUIManager::UpdateCursorControl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
        ]
    ),
    "cGcInWorldUIManager::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
        ]
    ),
    "cGcInWorldUIManager::IsRepositioningScreenWithHand": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcInWorldUIManager::CanRepositionScreenWithHand": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcInWorldUIManager::UpdateScreenMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_ulonglong,  # cGcInWorldScreen *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcInWorldUIManager::GetInputTransform": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_uint32,  # eHand
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcInWorldUIManager::CreateScreen": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPoolHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_ulonglong,  # cTkClassPoolHandle *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInWorldUIManager::CacheScreenMats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
        ]
    ),
    "cGcInWorldUIManager::HideScreen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_uint32,  # eInWorldScreens
        ]
    ),
    "cGcInWorldUIManager::HideScreens": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
        ]
    ),
    "cGcInWorldUIManager::NotifyLanguageChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
        ]
    ),
    "cGcInWorldUIManager::cGcScreenProjector::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager::cGcScreenProjector *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eHand
            ctypes.c_ulonglong,  # const cGcProjectorOffsetData *
            ctypes.c_ulonglong,  # const cGcInWorldUIScreenData *
            ctypes.c_uint32,  # cGcInWorldUIManager::cGcScreenProjector::Status
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cGcInWorldUIManager::cGcScreenProjector::GetRepositionedScreenData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager::cGcScreenProjector *
            ctypes.c_ulonglong,  # cGcInWorldUIScreenData *
            ctypes.c_uint32,  # eHand
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcInWorldUIManager::cGcScreenProjector::UpdateModel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager::cGcScreenProjector *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_char * 0x10,  # cTkVector3
        ]
    ),
    "cGcInWorldUIManager::cGcScreenProjector::StartProjectorAudio": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager::cGcScreenProjector *
            ctypes.c_float,  # float
        ]
    ),
    "cGcLayerInteractControl::GetConfirmValue": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcLayerInteractControl *
        ]
    ),
    "cGcLayerInteractControl::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLayerInteractControl *
        ]
    ),
    "cGcLayerInteractControl::Confirm": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLayerInteractControl *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "sUIScalingValues::sUIScalingValues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sUIScalingValues *
        ]
    ),
    "cTkMicroSmoother<20,10>::Sync": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMicroSmoother<20,10> *
        ]
    ),
    "cTkMicroSmoother<20,10>::Update": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMicroSmoother<20,10> *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cTkDynamicArray<cGcMissionSequenceDetailMessagePoint>::cTkDynamicArray<cGcMissionSequenceDetailMessagePoint>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMissionSequenceDetailMessagePoint> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcMissionSequenceDetailMessagePoint> *
        ]
    ),
    "cTkDynamicArray<cGcMissionSequenceDetailMessagePoint>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMissionSequenceDetailMessagePoint> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcHUDManagerData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcHUDManagerData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcScannerIcons>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcScannerIcons *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeapon::IsHolstered": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPersonalTeleporter::IsActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersonalTeleporter *
        ]
    ),
    "cGcCameraManager::GetBehaviourActiveWeight": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_uint32,  # eCameraBehaviours
        ]
    ),
    "cGcPlayerInteract::HasInteracted": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcSpaceshipComponent::GetInWorldLocator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcPlayerWeapon::Reload": {
        "cGcPlayerWeapon *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ]
        ),
        "cGcPlayerWeapon *, eWeaponMode, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerWeapon *
                ctypes.c_uint32,  # eWeaponMode
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cTkRigidBody::GetLocalAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcPlayerCreatureOwnership::GetSummonedIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
        ]
    ),
    "cGcHUDMarker::Deactivate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
        ]
    ),
    "cGcHUDMarker::BumpToTagged": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
        ]
    ),
    "cGcHUDMarker::CanEverBeTagged": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
        ]
    ),
    "cGcMarkerComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerComponent *
        ]
    ),
    "cGcMarkerComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcMarkerComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcMarkerComponent *
        ]
    ),
    "ComponentInstancercGcMarkerComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcMarkerComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcMarkerComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcInteractionComponent::IsAbandonedFreighterInteraction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcAbandonedFreighterComponent::IsRoomBranch": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAbandonedFreighterComponent::GetCurrentRoomIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
        ]
    ),
    "cTkSketchComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "cTkSketchComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkSketchComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "ComponentInstancercTkSketchComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercTkSketchComponent *
        ]
    ),
    "ComponentInstancercTkSketchComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercTkSketchComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cGcNetworkComponent::sSurvey,4,-1> *
        ]
    ),
    "ComponentInstancercTkSketchComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcInWorldUIManager::ReleaseScreen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIManager *
            ctypes.c_uint64,  # cTkClassPoolHandle
        ]
    ),
    "cGcHUDMarker::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
        ]
    ),
    "cGcHUDMarker::LoadGuiFiles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
        ]
    ),
    "RenderTag": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char * 0x8,  # cTkVector2
            ctypes.c_double,  # double
            ctypes.c_float,  # float
        ]
    ),
    "cGcHUDMarker::RenderNGuiElements": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
        ]
    ),
    "cGcHUDMarker::CanDisplay": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
        ]
    ),
    "cGcHUDMarker::CanDisplayOnCompass": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
        ]
    ),
    "cGcHUDMarker::ShowTagLockRing": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
        ]
    ),
    "cGcHUDMarker::ClearLockedMarkerOnNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcHUDMarker::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
            ctypes.c_ulonglong,  # const cGcMarkerPoint *
        ]
    ),
    "cGcHUDMarker::GenerateName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
        ]
    ),
    "cGcHUDMarker::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
            ctypes.c_ulonglong,  # const cGcMarkerPoint *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDMarker::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
            ctypes.c_ulonglong,  # cGcHUDScreensPool *
            ctypes.c_double,  # double
        ]
    ),
    "cGcHUDMarker::UpdateRenderables": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDMarker::UpdateDescriptionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
        ]
    ),
    "cGcHUDMarker::UpdateDistanceText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDMarker::UpdateCompassIcons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDMarker::UpdateIcons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDMarker::Animate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcHUDMarker::CalculateHoverState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDMarker::CalculateShowLargeState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDMarker::CalculateFade": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
        ]
    ),
    "cGcHUDMarker::SetArrowType": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
            ctypes.c_uint32,  # cGcHUDMarker::eArrowType
        ]
    ),
    "cGcSentinelRobotComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcSentinelRobotComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelRobotComponent *
        ]
    ),
    "cGcSentinelRobotComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelRobotComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcSentinelRobotComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcSentinelRobotComponent *
        ]
    ),
    "ComponentInstancercGcSentinelRobotComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcHUDTrackArrow::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDTrackArrow *
        ]
    ),
    "ComputeVRScreenPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector4 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector4 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "TrackArrowFadeVR": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcHUDTrackArrow::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDTrackArrow *
        ]
    ),
    "cGcHUDTrackArrow::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDTrackArrow *
        ]
    ),
    "cGcHUDTrackArrow::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDTrackArrow *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcHUDTrackArrow::CalculateTargetSizeAndPos": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcHUDTrackArrow::GetBorderPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # float *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBasePartsMenuUI::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartsMenuUI *
        ]
    ),
    "cGcBasePartsMenuUI::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartsMenuUI *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcBasePartsMenuUI::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartsMenuUI *
        ]
    ),
    "cGcBasePartsMenuUI::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartsMenuUI *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcBasePartsMenuUI::UpdateAvailableParts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartsMenuUI *
        ]
    ),
    "cGcBasePartsMenuUI::SetBreadcrumb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartsMenuUI *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcBasePartsMenuUI::SetCursorRestorationPartId": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartsMenuUI *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcBasePartsMenuUI::GetNextPartInSubgroup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartsMenuUI *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cGcBasePartsMenuUI::RenderItemDetails": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartsMenuUI *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcProductData *
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBasePartsMenuUI::IsObjectAvailable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartsMenuUI *
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntry *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcBasePartsMenuUI::IsObjectGreyedOut": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartsMenuUI *
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntry *
        ]
    ),
    "cGcBasePartsMenuUI::UpdateAvailablePartsBrowse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartsMenuUI *
        ]
    ),
    "cGcBasePartsMenuUI::UpdateAvailablePartsRelatives": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartsMenuUI *
        ]
    ),
    "GetScreenshotName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcBaseUploadUI::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseUploadUI *
            ctypes.c_float,  # const float
        ]
    ),
    "OnConfirmDeleteClose": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseUploadUI::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseUploadUI *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcBaseUploadUI::BeginBaseUploadUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcBaseUploadUI::HidePage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseUploadUI *
        ]
    ),
    "cGcBaseUploadUI::OnPhotoTakenCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # ePhotoModeReturnResult
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcBaseUploadUI::OnSnapshotTakenCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # ePhotoModeReturnResult
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcBaseUploadUI::OnPhotoTakenCallbackInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseUploadUI *
            ctypes.c_uint32,  # ePhotoModeReturnResult
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcBaseUploadUI::OnSnapshotTakenCallbackInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseUploadUI *
            ctypes.c_uint32,  # ePhotoModeReturnResult
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcBaseUploadUI::GetDefaultCameraMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseUploadUI *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cGcBaseUploadUI::HandleFeaturedBaseUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseUploadUI *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcBaseUploadUI::HideBaseComputerButtons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseUploadUI *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcHUD::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUD *
        ]
    ),
    "cGcHUDScreensPool::GetScreen": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPoolHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDScreensPool *
            ctypes.c_ulonglong,  # cTkClassPoolHandle *
            ctypes.c_uint32,  # cGcHUDScreensPool::eType
        ]
    ),
    "cGcHUDScreensPool::ResetPool": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDScreensPool *
        ]
    ),
    "cGcPhotoBaseForUploadUI::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoBaseForUploadUI *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPhotoBaseForUploadUI::DisableFreeCameraCapture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoBaseForUploadUI *
        ]
    ),
    "cGcPhotoBaseForUploadUI::RenderScreenshot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoBaseForUploadUI::OnRenderScreenshotFinished": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcMissionPhotoListener::NotifySuccess": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionPhotoListener *
        ]
    ),
    "cGcMissionPhotoListener::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPhotoListener *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionPhotoListener *
        ]
    ),
    "cGcPhotoManager::AnalysePhotoScene": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoManager *
            ctypes.c_ulonglong,  # const cTkCamera *
            ctypes.c_ulonglong,  # cGcPhoto *
        ]
    ),
    "cGcPhoto::GetPhotoBiomeName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_uint32,  # eBiome
        ]
    ),
    "cGcPhotoManager::AcknowledgePhotoRequests": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoManager *
            ctypes.c_ulonglong,  # const cGcPhoto *
            ctypes.c_ulonglong,  # std::vector<cTkFixedString<256,char>,TkSTLAllocatorShim<cTkFixedString<256,char>,1,-1> > *
        ]
    ),
    "cGcScannableComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkMatrix44::Determinant": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix44 *
        ]
    ),
    "cTkMatrix44::Inverted": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix44 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix44 *
            ctypes.c_ulonglong,  # cTkMatrix44 *
        ]
    ),
    "ScanInfo::IsValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # ScanInfo *
        ]
    ),
    "cGcPlayerCreatureOwnership::GetSummonedCreatureSpecialChatType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
        ]
    ),
    "cGcPlayerCreatureOwnership::GetCurrentMoodString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEvent::IsFromMission": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEvent *
            ctypes.c_ulonglong,  # const std::pair<TkID<128>,cTkSeed> *
        ]
    ),
    "cGcScanEventManager::HasEvent": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_ulonglong,  # std::pair<TkID<128>,cTkSeed> *
        ]
    ),
    "cGcScannableComponent::ShowInFreighterBranchRoom": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannableComponent *
        ]
    ),
    "cGcBinoculars::HasHighlightWithScanInfo": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcScannableComponent::GetCompassRange": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannableComponent *
        ]
    ),
    "cGcBinoculars::IsMissionSurveying": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcMarkerPoint::SetUseInteractionPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerPoint *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMarkerPoint::GetUseInteractionPosition": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerPoint *
        ]
    ),
    "cGcScanManager::IsMarkedPrimaryMarkerType": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanManager *
            ctypes.c_uint32,  # cGcMarkerPoint::eType
        ]
    ),
    "cTk2dLayer::SetVisible": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dLayer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannableComponent::HasComponent": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "PhotoModeControl::IsValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl *
        ]
    ),
    "PhotoModeControl_List::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_List *
        ]
    ),
    "cGcDistanceScaleComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcDistanceScaleComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDistanceScaleComponent *
        ]
    ),
    "cGcDistanceScaleComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDistanceScaleComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcDistanceScaleComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcDistanceScaleComponent *
        ]
    ),
    "ComponentInstancercGcDistanceScaleComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcHUDMarker::HideNGuiTextWhileHUDDisabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
        ]
    ),
    "cGcHUDMarker::HasCompassDistanceText": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarker *
        ]
    ),
    "cGcHUDPulseEffect::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcHUDPulseManager::Add": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDPulseManager *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cTkNGuiRectanglePulseEffect *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcPlayerHUD::ClearHelmentLineVisibility": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::RenderNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcHUDManager::ClearMissionDetailMessages": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<MissionDetailMessage,8,-1> *
        ]
    ),
    "PhotoModeControl_List::PhotoModeControl_List": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_List *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # {'return': 'int', 'args': ['void'], 'pointer_depth': 1}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['int'], 'pointer_depth': 1}
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['int'], 'pointer_depth': 1}
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['void'], 'pointer_depth': 1}
        ]
    ),
    "PhotoModeControl_List::GetValueAsString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_List *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "PhotoModeControl_List::Increase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_List *
        ]
    ),
    "PhotoModeControl_List::Decrease": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_List *
        ]
    ),
    "PhotoModeControl_TimeOfDay::GetValueAsString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_TimeOfDay *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "PhotoModeControl_TimeOfDay::Increase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_TimeOfDay *
        ]
    ),
    "PhotoModeControl_TimeOfDay::Decrease": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_TimeOfDay *
        ]
    ),
    "PhotoModeControl_TimeOfDay::IsValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_TimeOfDay *
        ]
    ),
    "cGcPhotoModeUI::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeUI *
        ]
    ),
    "cGcPhotoModeUI::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeUI *
        ]
    ),
    "cGcPhotoModeUI::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeUI *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPhotoModeUI::ForceHideUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeUI *
        ]
    ),
    "cGcPhotoModeUI::SetPhotoMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeUI *
            ctypes.c_uint32,  # ePhotoMode
        ]
    ),
    "cGcPhotoModeUI::CanActivate": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeUI *
        ]
    ),
    "cGcPhotoModeUI::Deactivate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeUI *
        ]
    ),
    "cGcPhotoModeUI::ResetPhotoModeState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeUI *
        ]
    ),
    "cGcPhotoModeUI::SetGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeUI *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPhotoModeUI::SetOption": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeUI *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPhotoModeUI::RenderCamMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeUI *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcPhotoModeUI::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeUI *
            ctypes.c_ulonglong,  # cGcFrontendPage *
        ]
    ),
    "cGcPhotoModeUI::EnterPhotoMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeUI *
        ]
    ),
    "cGcPhotoModeUI::RenderScreenshot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "Rec2390Rolloff": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # const float
        ]
    ),
    "TonemapHdrToSdr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPhotoModeUI::OnRenderScreenshotFinished": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcPhotoModeUI::CallPhotoManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeUI *
        ]
    ),
    "cGcPhotoModeUI::SystemUIOverlayChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeUI *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoModeUI::ShareMenuOpened": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeUI *
        ]
    ),
    "cGcShootableComponent::GetHealth": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
        ]
    ),
    "cGcShootableComponent::GetHealthPercent": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
        ]
    ),
    "cGcShootableComponent::TakesDefaultObjectDamage": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
        ]
    ),
    "cGcShootableComponent::GetInterceptPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShootableComponent::GetPhysicsMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcCreatureSpawnComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSpawnComponent *
        ]
    ),
    "cGcCreatureSpawnComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSpawnComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcCreatureSpawnComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcCreatureSpawnComponent *
        ]
    ),
    "ComponentInstancercGcCreatureSpawnComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcCreatureSpawnComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcCreatureSpawnComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcVehicleComponent::GetWeapon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcVehicleWeapon *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcDroneComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcPlayerHUD::LoadData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerHUD::IsAnyMarkerTagInteractionActive": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerHUD::GetBuildingEntryMessageText": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcPlayerHUD::UpdateTrackArrows": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSimpleScanEffect::cGcSimpleScanEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleScanEffect *
            ctypes.c_ulonglong,  # const cGcSimpleScanEffect *
        ]
    ),
    "cGcPlayerHUD::UpdatePanelShow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::SelectCrosshairStyle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::ShouldShowDroneIcon": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "GetImpactFactor": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerImpact *
        ]
    ),
    "cGcPlayerHUD::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::SetVRMovementVignetteValues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcPlayerHUD::RenderOffscreen2D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::RenderDamageNumbers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcPlayerHUD::RenderHeadsUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::RenderTrackArrows": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::RenderCrosshair": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcPlayerHUD::RenderShieldPanel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::RenderIndicatorPanel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::RenderHazardBarArrow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # cGcPlayerHUD::eHazardArrow
        ]
    ),
    "cGcPlayerHUD::RenderWeaponPanel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::RenderHazardPanel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::RenderHazardBar": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eHazard
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerHUD::RenderHUDBar": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cTkTextureResource *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_uint32,  # cGcPlayerHUD::eHazardArrow
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerHUD::CanRenderMarkers": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::RenderBaseGridObjectInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcProductData *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcPlayerHUD::RenderBaseBuildingObjectInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcProductData *
        ]
    ),
    "cGcPlayerHUD::RenderBuildInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::RenderBasePower": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::RenderChatPanel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
        ]
    ),
    "cGcPlayerHUD::Render2D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcPlayerHUD::SetHUDIcon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cTkTextureResource *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerHUD::SetHUDBar": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHUD *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcHUDPulseEffect::RenderInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDPulseEffect *
        ]
    ),
    "cGcPositionMarker::SetTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPositionMarker *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # cGcPositionMarker::eType
            ctypes.c_float,  # float
        ]
    ),
    "cGcPositionMarker::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPositionMarker *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "PhotoModeControl_Value<float>::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_Value<float> *
        ]
    ),
    "PhotoModeControl_Value<int>::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_Value<int> *
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcDistanceScaleComponent>::UpdateComponentsPostPhysics": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcDistanceScaleComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "PhotoModeControl_Value<int>::~PhotoModeControl_Value<int>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_Value<int> *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "PhotoModeControl_Value<int>::GetValueAsString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_Value<int> *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "PhotoModeControl_Value<int>::Increase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_Value<int> *
        ]
    ),
    "PhotoModeControl_Value<int>::Decrease": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_Value<int> *
        ]
    ),
    "PhotoModeControl_Value<int>::EnsureInBounds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_Value<int> *
        ]
    ),
    "PhotoModeControl_Value<float>::GetValueAsString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_Value<float> *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "PhotoModeControl_Value<float>::Increase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_Value<float> *
        ]
    ),
    "PhotoModeControl_Value<float>::Decrease": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_Value<float> *
        ]
    ),
    "PhotoModeControl_Value<float>::EnsureInBounds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PhotoModeControl_Value<float> *
        ]
    ),
    "cTkCurveFunction::SmoothCD<cTkVector4,cTkVector4>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector4 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector4 *
            ctypes.c_ulonglong,  # cTkVector4 *
            ctypes.c_ulonglong,  # const cTkVector4 *
            ctypes.c_ulonglong,  # const cTkVector4 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcSky::IsWeatherExtreme": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetData *
        ]
    ),
    "cGcBinoculars::HasValidMissionSurveyTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcPlayerHazard::GetSecondaryHazard": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazard *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcCameraBehaviourFirstPerson::IsShipFreeLookActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraBehaviourFirstPerson *
        ]
    ),
    "cGcPlayerImpact::GetPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerImpact *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cTkDynamicArray<cGcScreenFilterOption>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCombatEffectDamageMultiplier> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerImpact::cGcPlayerImpact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerImpact *
            ctypes.c_ulonglong,  # const cGcPlayerImpact *
        ]
    ),
    "cGcUserSettings::UseOldMouseFlight": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
        ]
    ),
    "cGcProjectileManager::GetProjectileData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcProjectileData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerHazard::IsInHazard": FUNCDEF(
        restype=ctypes.c_uint8,  # unsigned __int8
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazard *
            ctypes.c_uint32,  # eHazard
        ]
    ),
    "cGcPlayerHazard::GetPrimaryHazard": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazard *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcCreatureComponent::GetUsingIndoorNavigation": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcSpaceshipComponent::IsFollowModeActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcWeapon::GetCurrentRange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeapon *
        ]
    ),
    "cGcWeapon::GetCurrentSpeed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeapon *
        ]
    ),
    "cGcBuildingList::GetNearestBuilding": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBuilding *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingList *
        ]
    ),
    "cGcPlayerCommon::IsBeingScanned": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommon *
        ]
    ),
    "cGcPlayerExperienceDirector::GetRobots": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cTkAttachmentPtr,TkSTLAllocatorShim<cTkAttachmentPtr,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcAISpaceshipComponent::IsPositionInHangar": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkPhysicsComponent::CalcRelMatFromChildNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkPhysicsComponent::GetVelocity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cTkBaseMath::FloatToHalfI": FUNCDEF(
        restype=ctypes.c_uint16,  # unsigned __int16
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBaseMath::FloatToHalf": FUNCDEF(
        restype=ctypes.c_uint16,  # unsigned __int16
        argtypes=[
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraManager::GetPickRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcSky::PhotoModeChangeTimeOfDay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipHUD::RenderNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcNotificationSequence::GetCurrentSectionWithTargetID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNotifyGenericSection *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotificationSequence::Begin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcPlayerNotifications::DisableNotifyMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_uint32,  # eConstantMessage
        ]
    ),
    "cGcPlayerNotifications::SetConstantMessage": {
        "cGcPlayerNotifications *, int, bool, const cGcSlotIcon *, eFrontendPage, const TkID<256> *, const char *, const char *, unsigned int, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerNotifications *
                ctypes.c_int32,  # int
                ctypes.c_ubyte,  # bool
                ctypes.c_ulonglong,  # const cGcSlotIcon *
                ctypes.c_uint32,  # eFrontendPage
                ctypes.c_char_p,  # const TkID<256> *
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcPlayerNotifications *, int, bool, const cGcInventoryElement *, eFrontendPage, const TkID<256> *, const char *, const char *, unsigned int, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerNotifications *
                ctypes.c_int32,  # int
                ctypes.c_ubyte,  # bool
                ctypes.c_ulonglong,  # const cGcInventoryElement *
                ctypes.c_uint32,  # eFrontendPage
                ctypes.c_char_p,  # const TkID<256> *
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcPlayerNotifications *, int, bool, const cTkSmartResHandle *, eFrontendPage, const TkID<256> *, const char *, const char *, unsigned int, bool, eMessageCategory": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerNotifications *
                ctypes.c_int32,  # int
                ctypes.c_ubyte,  # bool
                ctypes.c_ulonglong,  # const cTkSmartResHandle *
                ctypes.c_uint32,  # eFrontendPage
                ctypes.c_char_p,  # const TkID<256> *
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ubyte,  # bool
                ctypes.c_uint32,  # eMessageCategory
            ]
        ),
        "cGcPlayerNotifications *, int, bool, eFrontendPage, const TkID<256> *, const char *, const char *, unsigned int, bool, eMessageCategory": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerNotifications *
                ctypes.c_int32,  # int
                ctypes.c_ubyte,  # bool
                ctypes.c_uint32,  # eFrontendPage
                ctypes.c_char_p,  # const TkID<256> *
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ubyte,  # bool
                ctypes.c_uint32,  # eMessageCategory
            ]
        ),
    },
    "cGcPlayerNotifications::ConstantMessage::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications::ConstantMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_uint32,  # eMessageCategory
        ]
    ),
    "cGcPlayerNotifications::ConstantMessage::UpdateTimeTracker": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications::ConstantMessage *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerNotifications::IsCurrentlyAppropriate": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # const cGcPlayerNotifications::ConstantMessage *
        ]
    ),
    "cGcPlayerNotifications::RenderImGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcPlayerNotifications::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
        ]
    ),
    "cGcPlayerNotifications::LoadData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerNotifications::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerNotifications::UpdateVehicle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
        ]
    ),
    "cGcPlayerNotifications::UpdateMissionSequences": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerNotifications::UpdateNotifyWeapons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
        ]
    ),
    "cGcPlayerNotifications::UpdateNotifySpaceship": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerNotifications::UpdateNotifyEnergy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcPlayerNotifications::TimedMessage,16,-1> *
        ]
    ),
    "cGcPlayerNotifications::UpdateNotifyHealthAndShield": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
        ]
    ),
    "cGcPlayerNotifications::UpdateNotifyHazards": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
        ]
    ),
    "cGcPlayerNotifications::UpdateRequests": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerNotifications::RestartMissionWithNewSeed": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerNotifications::RequestAddOrRestartMission": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerNotifications::RenounceAllMilestoneMissions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
        ]
    ),
    "cGcPlayerNotifications::RenounceAllSubstanceMissions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
        ]
    ),
    "cGcPlayerNotifications::SendMissionMessage": {
        "cGcPlayerNotifications *, const TkID<128> *, bool, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerNotifications *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcPlayerNotifications *, const TkID<128> *, const TkID<128> *, const cTkSeed *, bool, bool": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerNotifications *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ulonglong,  # const cTkSeed *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcPlayerNotifications *, const TkID<128> *, const cTkSeed *, bool, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerNotifications *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ulonglong,  # const cTkSeed *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cGcPlayerNotifications::RewardProduct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerNotifications::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
        ]
    ),
    "cGcPlayerNotifications::ClearAllMessagesAndHide": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcPlayerNotifications::TimedMessage,16,-1> *
        ]
    ),
    "cGcPlayerNotifications::GetTargetMissionSurveyScanInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerNotifications::sMissionTargetSurveyScanInfo *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # cGcPlayerNotifications::sMissionTargetSurveyScanInfo *
        ]
    ),
    "cGcPlayerNotifications::GetInventoryHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPlayerNotifications::CanShowLowerMessages": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
        ]
    ),
    "cGcPlayerNotifications::ClearSequences": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcMPMissionTracker::sGcSyncedMPMissionPlayerData,8,-1> *
        ]
    ),
    "cGcPlayerNotifications::ReadMissions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcPlayerMissionProgress> *
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcMPMissionTracker::sGcSyncedMPMissionPlayerData,8,-1> *
        ]
    ),
    "cGcPlayerNotifications::RecordMissionStat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcPlayerNotifications::AddRecipeHint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerNotifications::RemovePinnedRecipe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
        ]
    ),
    "cGcPlayerNotifications::GetGenericMissionData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerNotifications::GetDefaultSubstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # eDefaultSubstanceType
        ]
    ),
    "cGcPlayerNotifications::GetDefaultProduct": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # eDefaultProductType
        ]
    ),
    "cGcPlayerNotifications::AddTimedMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_char_p,  # const char *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const cTkSmartResHandle *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerNotifications::GetTechChargeString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # const cGcTechnology *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerNotifications::GetTechChargeMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcPlayerNotifications::GetCurrentNotificationSequence": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNotificationSequence *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcPlayerNotifications::IsCurrentNotificationSequence": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # const std::pair<TkID<128>,cTkSeed> *
        ]
    ),
    "cGcPlayerNotifications::GetFirstNotificationSequence": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const BaseBuildingObjectTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcPlayerNotifications::GetLastNotificationSequence": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const BaseBuildingObjectTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcPlayerNotifications::GetNotificationSequenceAfter": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const BaseBuildingObjectTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcPlayerNotifications::GetNotificationSequenceBefore": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const BaseBuildingObjectTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcPlayerNotifications::RequestRenounceMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcPlayerNotifications::RenounceMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcPlayerNotifications::RenderMissionNotificationPanel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcPlayerNotifications::InDanger": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
        ]
    ),
    "cGcPlayerNotifications::RenderImGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
        ]
    ),
    "cGcPlayerNotifications::DisplayTimedMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_ulonglong,  # const cGcPlayerNotifications::SimpleMessage *
        ]
    ),
    "cGcPlayerNotifications::RPCStartMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_ulonglong,  # std::vector<cTkUserIdBase<cTkFixedString<64,char> >,TkSTLAllocatorShim<cTkUserIdBase<cTkFixedString<64,char> >,1,-1> > *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # cTkSeed *
        ]
    ),
    "cGcPlayerNotifications::RPCBroadcastMissionMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cGcPlayerNotifications::RPCBroadcastMissionMessageSeeded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_char * 0x10,  # TkID<128>
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcTargetScreen::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetScreen *
        ]
    ),
    "cGcRenderableScreen::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRenderableScreen *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRenderableScreen::EnqueueRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRenderableScreen *
        ]
    ),
    "cGcRenderableScreen::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRenderableScreen *
        ]
    ),
    "cGcRenderableScreen::ShouldRenderLiveInShipScreen": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRenderableScreen *
        ]
    ),
    "cGcReportBaseUI::EnterReportBaseMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int16 *
        ]
    ),
    "cGcReportBaseUI::ExitReportBaseMode": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcReportBaseUI::OnDialogClose": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShootableComponent::IsDead": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
        ]
    ),
    "cGcShootableComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcTurretComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcTurretComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTurretComponent *
        ]
    ),
    "cGcTurretComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTurretComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcTurretComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcTurretComponent *
        ]
    ),
    "ComponentInstancercGcTurretComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcTurretComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcTurretComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcMissileComponent::AttackingPlayer": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissileComponent *
        ]
    ),
    "cGcMissileComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcMissileComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissileComponent *
        ]
    ),
    "cGcMissileComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissileComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcMissileComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcMissileComponent *
        ]
    ),
    "ComponentInstancercGcMissileComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcMissileComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcPlayerWeaponComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcSpaceshipShieldComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipShieldComponent *
        ]
    ),
    "cGcSpaceshipShieldComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipShieldComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcSpaceshipShieldComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcSpaceshipShieldComponent *
        ]
    ),
    "ComponentInstancercGcSpaceshipShieldComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcSpaceshipShieldComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcSpaceshipShieldComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcShipHUD::LoadData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUD *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipHUD::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUD *
        ]
    ),
    "cGcSpaceMap::cGcSpaceMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMap *
        ]
    ),
    "cGcSpaceMap::~cGcSpaceMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMap *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cTkFixedArray<cGcSpaceMapObject,6>::~cTkFixedArray<cGcSpaceMapObject,6>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcSpaceMapObject,6> *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cTkFixedArray<cGcSpaceMapLine,64>::~cTkFixedArray<cGcSpaceMapLine,64>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcSpaceMapLine,64> *
        ]
    ),
    "cTkFixedArray<cGcSpaceMapObject,32>::~cTkFixedArray<cGcSpaceMapObject,32>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcSpaceMapObject,32> *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cTkFixedArray<cGcSpaceMapObject,8>::~cTkFixedArray<cGcSpaceMapObject,8>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcSpaceMapObject,8> *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcShipHUD::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUD *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipHUD::UpdateTrackArrows": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUD *
            ctypes.c_ulonglong,  # cTkAttachmentPtr *
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim **
        ]
    ),
    "cGcShipHUD::UpdateMarkerLockOn": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMarkerPoint *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUD *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipHUD::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUD *
        ]
    ),
    "cGcShipHUD::ReadPlanetStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUD *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcShipHUD::RenderOffscreen2D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUD *
        ]
    ),
    "cGcShipHUD::Render2D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUD *
        ]
    ),
    "cGcShipHUD::RenderHeadsUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUD *
        ]
    ),
    "cTkFixedString<128,wchar_t>::Format<wchar_t,0>": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<128,wchar_t> *
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "cGcPlayerNotifications::SimpleMessage::SimpleMessage": {
        "cGcPlayerNotifications::SimpleMessage *, const cGcPlayerNotifications::SimpleMessage *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerNotifications::SimpleMessage *
                ctypes.c_ulonglong,  # const cGcPlayerNotifications::SimpleMessage *
            ]
        ),
        "cGcPlayerNotifications::SimpleMessage *, cGcPlayerNotifications::SimpleMessage *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerNotifications::SimpleMessage *
                ctypes.c_ulonglong,  # cGcPlayerNotifications::SimpleMessage *
            ]
        ),
    },
    "cGcPlayerCommunicator::HasHUDMessage": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicator *
        ]
    ),
    "cGcAISpaceshipManager::LookupCombatDefinition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcShipAICombatDefinition *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcNGuiNodeInfo::~cGcNGuiNodeInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiNodeInfo *
        ]
    ),
    "cGcRenderableScreen::~cGcRenderableScreen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRenderableScreen *
        ]
    ),
    "cGcGameState::GetAtlasPathProgress": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameState *
        ]
    ),
    "cGcPlayerState::GetNumAtlasStationsVisited": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcScanEventManager::ClearMissionEvents": {
        "cGcScanEventManager *, const std::vector<std::pair<TkID<128>,cTkSeed>,TkSTLAllocatorShim<std::pair<TkID<128>,cTkSeed>,8,-1> > *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcScanEventManager *
                ctypes.c_ulonglong,  # const std::vector<std::pair<TkID<128>,cTkSeed>,TkSTLAllocatorShim<std::pair<TkID<128>,cTkSeed>,8,-1> > *
            ]
        ),
        "cGcScanEventManager *, const std::pair<TkID<128>,cTkSeed> *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcScanEventManager *
                ctypes.c_ulonglong,  # const std::pair<TkID<128>,cTkSeed> *
            ]
        ),
    },
    "cGcLootComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcSpaceMapObjectUtils::ColourMapObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSmartResHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcSpaceMapLine::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMapLine *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcSpaceMap::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMap *
        ]
    ),
    "cGcSpaceMap::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMap *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSpaceMap::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMap *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceMap::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMap *
        ]
    ),
    "cTkRichPresence::NotifyPlayerStat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMap *
        ]
    ),
    "cGcSpaceMap::CleanupResources": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMap *
        ]
    ),
    "cGcSpaceMap::GetLogDistance": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceMap::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMap *
        ]
    ),
    "cGcSpaceMapObject::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMapObject *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcSpaceMapObject::ChangeResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMapObject *
            ctypes.c_ulonglong,  # const cTkSmartResHandle *
        ]
    ),
    "cGcSpaceMapLine::UpdatePlane": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMapLine *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceMapAveragedObject::ShouldCoalesce": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMapAveragedObject *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_uint32,  # eSpaceMapObjectType
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcTargetScreen::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetScreen *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTargetScreen::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetScreen *
            ctypes.c_float,  # float
        ]
    ),
    "cGcTargetScreen::EnqueueRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetScreen *
        ]
    ),
    "cGcTargetScreen::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetScreen *
        ]
    ),
    "cGcTargetScreen::CleanupResources": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetScreen *
        ]
    ),
    "cGcTargetScreen::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetScreen *
        ]
    ),
    "cGcTargetScreen::HideOtherNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetScreen *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # std::vector<std::pair<TkHandle,bool>,TkSTLAllocatorShim<std::pair<TkHandle,bool>,4,-1> > *
        ]
    ),
    "cGcTargetScreen::RestoreOtherNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetScreen *
            ctypes.c_ulonglong,  # std::vector<std::pair<TkHandle,bool>,TkSTLAllocatorShim<std::pair<TkHandle,bool>,4,-1> > *
        ]
    ),
    "cGcGenericSectionCondition::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionCondition *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionCondition::GetBuildMenuHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionCondition *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcGenericSectionCondition::EqualityText": {
        "cGcGenericSectionCondition *, eEqualityEnum, __int64, cTkFixedString<1024,char> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcGenericSectionCondition *
                ctypes.c_uint32,  # eEqualityEnum
                ctypes.c_int64,  # __int64
                ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ]
        ),
        "cGcGenericSectionCondition *, eEqualityEnum, float, cTkFixedString<1024,char> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcGenericSectionCondition *
                ctypes.c_uint32,  # eEqualityEnum
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ]
        ),
    },
    "cGcGenericSectionCondition::DebugText": {
        "eConditionTest, std::vector<cGcGenericSectionCondition *,TkSTLAllocatorShim<cGcGenericSectionCondition *,8,-1> > *, eRepeatLogic, cTkFixedString<1024,char> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_uint32,  # eConditionTest
                ctypes.c_ulonglong,  # std::vector<cGcGenericSectionCondition *,TkSTLAllocatorShim<cGcGenericSectionCondition *,8,-1> > *
                ctypes.c_uint32,  # eRepeatLogic
                ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ]
        ),
        "eConditionTest, std::vector<cGcGenericSectionCondition *,TkSTLAllocatorShim<cGcGenericSectionCondition *,8,-1> > *, eRepeatLogic, cTkFixedString<1024,char> *, std::vector<cTkFixedString<256,char>,TkSTLAllocatorShim<cTkFixedString<256,char>,1,-1> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_uint32,  # eConditionTest
                ctypes.c_ulonglong,  # std::vector<cGcGenericSectionCondition *,TkSTLAllocatorShim<cGcGenericSectionCondition *,8,-1> > *
                ctypes.c_uint32,  # eRepeatLogic
                ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
                ctypes.c_ulonglong,  # std::vector<cTkFixedString<256,char>,TkSTLAllocatorShim<cTkFixedString<256,char>,1,-1> > *
            ]
        ),
    },
    "cGcGenericSectionConditionIsScanEventLocal::cGcGenericSectionConditionIsScanEventLocal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsScanEventLocal *
            ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventLocal *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionWaitForTime::ResetsUserDataOnActivation": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWaitForTime *
        ]
    ),
    "cGcAbandonedFreighterComponent::IsAbandonedFreighterNearby": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "cGcAbandonedFreighterComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcGenericSectionConditionBaseClaimed::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBaseClaimed *
        ]
    ),
    "cGcGenericSectionConditionBaseClaimed::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBaseClaimed *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionBaseClaimed::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBaseClaimed *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionBaseQuery::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBaseQuery *
        ]
    ),
    "cGcGenericSectionConditionBaseQuery::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBaseQuery *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionBaseQuery::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBaseQuery *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionBasePartsQuery::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBasePartsQuery *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntry *
        ]
    ),
    "cGcGenericSectionConditionMessageBeaconsQuery::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionMessageBeaconsQuery *
        ]
    ),
    "cGcGenericSectionConditionCriticalMissionsDone::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCriticalMissionsDone *
        ]
    ),
    "cGcGenericSectionConditionCriticalMissionsDone::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCriticalMissionsDone *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionLocation::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionLocation *
        ]
    ),
    "cGcGenericSectionConditionLocation::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionLocation *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionLocation::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionLocation *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionWarping::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWarping *
        ]
    ),
    "cGcGenericSectionConditionWarping::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWarping *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionPlanetResourceHint::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPlanetResourceHint *
        ]
    ),
    "cGcGenericSectionConditionPlanetResourceHint::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPlanetResourceHint *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionPlanetCreatureRoles::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPlanetCreatureRoles *
        ]
    ),
    "cGcGenericSectionConditionPlanetCreatureRoles::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPlanetCreatureRoles *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionSquadronSlots::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSquadronSlots *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionBasePartNear::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBasePartNear *
        ]
    ),
    "cGcGenericSectionConditionBasePartNear::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBasePartNear *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionBasePartNear::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBasePartNear *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionBasePartBuilt::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBasePartBuilt *
        ]
    ),
    "cGcGenericSectionConditionBasePartBuilt::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBasePartBuilt *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionBasePartBuilt::GetCurrentCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBasePartBuilt *
        ]
    ),
    "cGcGenericSectionConditionBasePartBuilt::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBasePartBuilt *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionBaseRequiresPower::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBaseRequiresPower *
        ]
    ),
    "cGcGenericSectionConditionBaseRequiresPower::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBaseRequiresPower *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionRefinerActive::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionRefinerActive *
        ]
    ),
    "cGcGenericSectionConditionRefinerActive::GetRefinerComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionRefinerActive *
        ]
    ),
    "cGcGenericSectionConditionRefinerActive::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionRefinerActive *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionRefinerActive::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionRefinerActive *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionRefinerActive::GetConditionInventoryHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionRefinerActive *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcGenericSectionConditionRefinerHasOutput::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionRefinerHasOutput *
        ]
    ),
    "cGcGenericSectionConditionRefinerHasOutput::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionRefinerHasOutput *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionGameMode::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionGameMode *
        ]
    ),
    "cGcGenericSectionConditionGameMode::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionGameMode *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionMissionCompleted::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionMissionCompleted *
        ]
    ),
    "cGcGenericSectionConditionMissionCompleted::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionMissionCompleted *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionMissionCompleted::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionMissionCompleted *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionBiomeType::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBiomeType *
        ]
    ),
    "cGcGenericSectionConditionBiomeType::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBiomeType *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionBiomeType::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBiomeType *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionWaterPlanet::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWaterPlanet *
        ]
    ),
    "cGcGenericSectionConditionWaterPlanet::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWaterPlanet *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionWaterInSystem::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWaterInSystem *
        ]
    ),
    "cGcGenericSectionConditionWaterInSystem::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWaterInSystem *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionSentinelLevel::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSentinelLevel *
        ]
    ),
    "cGcGenericSectionConditionSentinelLevel::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSentinelLevel *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionSentinelsDisabled::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSentinelsDisabled *
        ]
    ),
    "cGcGenericSectionConditionSentinelsDisabled::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSentinelsDisabled *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionSpecialKnown::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSpecialKnown *
        ]
    ),
    "cGcGenericSectionConditionSpecialKnown::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSpecialKnown *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionSeasonRewardUnlocked::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSeasonRewardUnlocked *
        ]
    ),
    "cGcGenericSectionConditionSeasonRewardUnlocked::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSeasonRewardUnlocked *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionProductKnown::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionProductKnown *
        ]
    ),
    "cGcGenericSectionConditionProductKnown::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionProductKnown *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionProductKnown::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionProductKnown *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionTechnologyKnown::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionTechnologyKnown *
        ]
    ),
    "cGcGenericSectionConditionTechnologyKnown::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionTechnologyKnown *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionTechnologyKnown::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionTechnologyKnown *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionNPCHabitationHasWorker::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNPCHabitationHasWorker *
        ]
    ),
    "cGcGenericSectionConditionNPCHabitationHasWorker::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNPCHabitationHasWorker *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionFactionRank::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionFactionRank *
        ]
    ),
    "cGcGenericSectionConditionFactionRank::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionFactionRank *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionSystemRace::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSystemRace *
        ]
    ),
    "cGcGenericSectionConditionSystemRace::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSystemRace *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionSystemRace::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSystemRace *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionPirateSystem::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPirateSystem *
        ]
    ),
    "cGcGenericSectionConditionPirateSystem::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPirateSystem *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasIllegalGoods::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasIllegalGoods *
        ]
    ),
    "cGcGenericSectionConditionHasIllegalGoods::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasIllegalGoods *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasIngredientsForItem::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasIngredientsForItem *
        ]
    ),
    "cGcGenericSectionConditionHasIngredientsForItem::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasIngredientsForItem *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasIngredientsForItem::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasIngredientsForItem *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionForceHideMultiplayer::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionForceHideMultiplayer *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionPlanetAttackPiratesActive::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPlanetAttackPiratesActive *
        ]
    ),
    "cGcGenericSectionConditionPlanetAttackPiratesActive::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPlanetAttackPiratesActive *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionSystemHasInfestedPlanet::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSystemHasInfestedPlanet *
        ]
    ),
    "cGcGenericSectionConditionSystemHasInfestedPlanet::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSystemHasInfestedPlanet *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionSystemHasInfestedPlanet::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSystemHasInfestedPlanet *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionSystemPlanetTest::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSystemPlanetTest *
        ]
    ),
    "cGcGenericSectionConditionSystemPlanetTest::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSystemPlanetTest *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionSystemStarClass::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSystemStarClass *
        ]
    ),
    "cGcGenericSectionConditionSystemStarClass::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSystemStarClass *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasSubstance::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasSubstance *
        ]
    ),
    "cGcGenericSectionConditionHasSubstance::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasSubstance *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionHasProduct::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasSubstance *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasProduct::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasProduct *
        ]
    ),
    "cGcGenericSectionConditionHasProduct::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasProduct *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionHasProduct::GetConditionInventoryHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasProduct *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcGenericSectionConditionHasProcProduct::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasProcProduct *
        ]
    ),
    "cGcGenericSectionConditionHasProcProduct::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasProcProduct *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionHasProcProduct::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasProcProduct *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasMoney::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasMoney *
        ]
    ),
    "cGcGenericSectionConditionHasMoney::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasMoney *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasMoney::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasMoney *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionHasMoney::CalculateRequiredAmount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasMoney *
        ]
    ),
    "cGcGenericSectionConditionInMultiplayer::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInMultiplayer *
        ]
    ),
    "cGcGenericSectionConditionInMultiplayer::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInMultiplayer *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasLegacyBasePending::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasLegacyBasePending *
        ]
    ),
    "cGcGenericSectionConditionHasLegacyBasePending::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasLegacyBasePending *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionWeaponMode::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWeaponMode *
        ]
    ),
    "cGcGenericSectionConditionWeaponMode::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWeaponMode *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasTechnology::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasTechnology *
        ]
    ),
    "cGcGenericSectionConditionHasTechnology::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasTechnology *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionHasTechnology::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasTechnology *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasTechnology::ShouldRefreshInventoryHint": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasTechnology *
        ]
    ),
    "cGcGenericSectionConditionHasTechnology::GetConditionInventoryHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasTechnology *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcGenericSectionConditionHasProcTechnology::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasProcTechnology *
        ]
    ),
    "cGcGenericSectionConditionHasProcTechnology::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasProcTechnology *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionHasProcTechnology::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasProcTechnology *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionIsTechnologyRepaired::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsTechnologyRepaired *
        ]
    ),
    "cGcGenericSectionConditionIsTechnologyRepaired::GetConditionInventoryHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBasePartNear *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcGenericSectionConditionIsTechnologyRepaired::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsTechnologyRepaired *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionIsTechnologyRepaired::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsTechnologyRepaired *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionHasFreighter::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasFreighter *
        ]
    ),
    "cGcGenericSectionConditionHasFreighter::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasFreighter *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionMultiplayerFreighterAvailable::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionMultiplayerFreighterAvailable *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionCanSummonExocraft::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCanSummonExocraft *
        ]
    ),
    "cGcGenericSectionConditionCanSummonExocraft::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCanSummonExocraft *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasExocraft::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasExocraft *
        ]
    ),
    "cGcGenericSectionConditionHasExocraft::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasExocraft *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionPrimaryExocraft::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPrimaryExocraft *
        ]
    ),
    "cGcGenericSectionConditionPrimaryExocraft::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPrimaryExocraft *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionBasePowerGenerated::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBasePowerGenerated *
        ]
    ),
    "cGcGenericSectionConditionBasePowerGenerated::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBasePowerGenerated *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionBasePowerGenerated::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBasePowerGenerated *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionNexusEnabled::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNexusEnabled *
        ]
    ),
    "cGcGenericSectionConditionNexusEnabled::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNexusEnabled *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionInVR::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInVR *
        ]
    ),
    "cGcGenericSectionConditionInVR::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInVR *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionLocalScanActive::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionLocalScanActive *
        ]
    ),
    "cGcGenericSectionConditionLocalScanActive::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionLocalScanActive *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionWristMenuOpen::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWristMenuOpen *
        ]
    ),
    "cGcGenericSectionConditionWristMenuOpen::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWristMenuOpen *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionPadActive::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPadActive *
        ]
    ),
    "cGcGenericSectionConditionPadActive::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPadActive *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionOnPlanetWithSandwormsOverriden::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionOnPlanetWithSandwormsOverriden *
        ]
    ),
    "cGcGenericSectionConditionOnPlanetWithSandwormsOverriden::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionOnPlanetWithSandwormsOverriden *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionOnPlanetWithSandwormsOverriden::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionOnPlanetWithSandwormsOverriden *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionGunOut::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcGenericSectionConditionGunOut::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionGunOut *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionIsGrabbed::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcGenericSectionConditionIsGrabbed::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsGrabbed *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionEggMachinePageOpen::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionEggMachinePageOpen *
        ]
    ),
    "cGcGenericSectionConditionEggMachinePageOpen::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionEggMachinePageOpen *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasCreatureEggItem::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasCreatureEggItem *
        ]
    ),
    "cGcGenericSectionConditionHasCreatureEggItem::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasCreatureEggItem *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionSeasonNumber::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSeasonNumber *
        ]
    ),
    "cGcGenericSectionConditionSeasonNumber::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSeasonNumber *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionInSeasonalUA::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInSeasonalUA *
        ]
    ),
    "cGcGenericSectionConditionIsPartyPlanetUnlocked::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsPartyPlanetUnlocked *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionInSeasonalUA::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInSeasonalUA *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionIsPartyPlanetUnlocked::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsPartyPlanetUnlocked *
        ]
    ),
    "cGcGenericSectionConditionIsPartyPlanetUnlocked::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsPartyPlanetUnlocked *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionTakingDamage::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionTakingDamage *
        ]
    ),
    "cGcGenericSectionConditionTakingDamage::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionTakingDamage *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionShieldDown::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionShieldDown *
        ]
    ),
    "cGcGenericSectionConditionShieldDown::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionShieldDown *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionShieldDown::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionShieldDown *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionConvertedFromSeason::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionConvertedFromSeason *
        ]
    ),
    "cGcGenericSectionConditionConvertedFromSeason::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionConvertedFromSeason *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionInteractionIndexChanged::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInteractionIndexChanged *
        ]
    ),
    "cGcGenericSectionConditionInteractionIndexChanged::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInteractionIndexChanged *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionAllMilestonesComplete::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAllMilestonesComplete *
        ]
    ),
    "cGcGenericSectionConditionAllMilestonesComplete::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAllMilestonesComplete *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionAllMilestonesComplete::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAllMilestonesComplete *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionInventoryOpen::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInventoryOpen *
        ]
    ),
    "cGcGenericSectionConditionInventoryOpen::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInventoryOpen *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionAbandonedOrEmptySystem::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAbandonedOrEmptySystem *
        ]
    ),
    "cGcGenericSectionConditionAbandonedOrEmptySystem::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAbandonedOrEmptySystem *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionBuildMenuOpen::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBuildMenuOpen *
        ]
    ),
    "cGcGenericSectionConditionBuildMenuOpen::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBuildMenuOpen *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasActiveDetailMessage::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasActiveDetailMessage *
        ]
    ),
    "cGcGenericSectionConditionHasActiveDetailMessage::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasActiveDetailMessage *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasCommunicatorSignal::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasCommunicatorSignal *
        ]
    ),
    "cGcGenericSectionConditionHasCommunicatorSignal::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasCommunicatorSignal *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasActiveStatsMessage::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasActiveStatsMessage *
        ]
    ),
    "cGcGenericSectionConditionHasActiveStatsMessage::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasActiveStatsMessage *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasSpareProcTech::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasSpareProcTech *
        ]
    ),
    "cGcGenericSectionConditionHasSpareProcTech::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasSpareProcTech *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionIsSurveying::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsSurveying *
        ]
    ),
    "cGcGenericSectionConditionIsSurveying::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsSurveying *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionOnMultiplayerMission::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionOnMultiplayerMission *
        ]
    ),
    "cGcGenericSectionConditionOnMultiplayerMission::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionOnMultiplayerMission *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionRidingCreature::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcGenericSectionConditionRidingCreature::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionRidingCreature *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionFeedingCreatures::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionFeedingCreatures *
        ]
    ),
    "cGcGenericSectionConditionFeedingCreatures::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionFeedingCreatures *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionFeedingCreatures::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionFeedingCreatures *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionFeedingCreatures::GetNumFollowing": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionFeedingCreatures *
        ]
    ),
    "cGcGenericSectionConditionHasGrabbableTarget::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasGrabbableTarget *
        ]
    ),
    "cGcGenericSectionConditionHasGrabbableTarget::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasGrabbableTarget *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionAimingTeleporter::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAimingTeleporter *
        ]
    ),
    "cGcGenericSectionConditionAimingTeleporter::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAimingTeleporter *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionIsFrigateFlybyActive::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsFrigateFlybyActive *
        ]
    ),
    "cGcGenericSectionConditionIsFrigateFlybyActive::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsFrigateFlybyActive *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionExocraftMoving::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExocraftMoving *
        ]
    ),
    "cGcGenericSectionConditionExocraftMoving::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExocraftMoving *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionIsAbandFreighterDoorOpen::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsAbandFreighterDoorOpen *
        ]
    ),
    "cGcGenericSectionConditionIsAbandFreighterDoorOpen::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsAbandFreighterDoorOpen *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionIsAbandFreighterDoorOpen::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsAbandFreighterDoorOpen *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionShipEngineStatus::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionShipEngineStatus *
        ]
    ),
    "cGcGenericSectionConditionShipEngineStatus::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionShipEngineStatus *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionUsingThirdPersonCamera::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionUsingThirdPersonCamera *
        ]
    ),
    "cGcGenericSectionConditionUsingThirdPersonCamera::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionUsingThirdPersonCamera *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionCameraControlStealing::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCameraControlStealing *
        ]
    ),
    "cGcGenericSectionConditionCameraControlStealing::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCameraControlStealing *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionIsPlayerWeak::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsPlayerWeak *
        ]
    ),
    "cGcGenericSectionConditionIsPlayerWeak::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsPlayerWeak *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionDefaultItem::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionDefaultItem *
        ]
    ),
    "cGcGenericSectionConditionDefaultItem::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionDefaultItem *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionMissionMessage::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionMissionMessage *
        ]
    ),
    "cGcGenericSectionConditionTotalWarpsNumber::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionTotalWarpsNumber *
        ]
    ),
    "cGcGenericSectionConditionIsPlayerWanted::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsPlayerWanted *
        ]
    ),
    "cGcGenericSectionConditionIsPlayerWanted::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsPlayerWanted *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionHasFuel::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasFuel *
        ]
    ),
    "cGcGenericSectionConditionHasFuel::GetConditionInventoryHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasFuel *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcGenericSectionConditionHasFuel::GetItem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasFuel *
        ]
    ),
    "cGcGenericSectionConditionHasFuel::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasFuel *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionHasFuelForTakeoff::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasFuelForTakeoff *
        ]
    ),
    "cGcGenericSectionConditionHasFuelForTakeoff::GetConditionInventoryHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasFuelForTakeoff *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcGenericSectionConditionHasFuelForTakeoff::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasFuelForTakeoff *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionInventorySlots::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInventorySlots *
        ]
    ),
    "cGcGenericSectionConditionInventorySlots::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInventorySlots *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionInventorySlots::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInventorySlots *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionFrigateCount::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionFrigateCount *
        ]
    ),
    "cGcGenericSectionConditionFrigateCount::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionFrigateCount *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionExpeditionCount::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExpeditionCount *
        ]
    ),
    "cGcGenericSectionConditionExpeditionCount::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExpeditionCount *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionExpeditionContainsReward::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExpeditionContainsReward *
        ]
    ),
    "cGcGenericSectionConditionExpeditionContainsReward::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExpeditionContainsReward *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionDamagedFrigateAtHome::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionDamagedFrigateAtHome *
        ]
    ),
    "cGcGenericSectionConditionDamagedFrigateAtHome::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionDamagedFrigateAtHome *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionDamagedFrigateAtHome::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionDamagedFrigateAtHome *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionExpeditionProgress::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExpeditionProgress *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionExpeditionProgress::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExpeditionProgress *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionPlanetStatLevel::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPlanetStatLevel *
        ]
    ),
    "cGcGenericSectionConditionPlanetStatLevel::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPlanetStatLevel *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionPlanetStatLevel::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPlanetStatLevel *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionStatLevel::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionStatLevel *
        ]
    ),
    "cGcGenericSectionConditionStatLevel::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionStatLevel *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionStatLevel::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionStatLevel *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionStatDiff::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionStatDiff *
        ]
    ),
    "cGcGenericSectionConditionStatDiff::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionStatDiff *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionStatDiff::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionStatDiff *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionStatChange::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionStatChange *
        ]
    ),
    "cGcGenericSectionConditionStatChange::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionStatChange *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionIsAnomalyLoaded::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsAnomalyLoaded *
        ]
    ),
    "cGcGenericSectionConditionIsLookingAtAnomalySpawn::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsLookingAtAnomalySpawn *
        ]
    ),
    "cGcGenericSectionConditionIsCurrentMission::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsCurrentMission *
        ]
    ),
    "cGcGenericSectionConditionIsCurrentMission::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsCurrentMission *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionMissionSelected::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionMissionSelected *
        ]
    ),
    "cGcGenericSectionConditionMissionSelected::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionMissionSelected *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasGalacticFeature::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasGalacticFeature *
        ]
    ),
    "cGcGenericSectionConditionHasGalacticFeature::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasGalacticFeature *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionGlobalMission::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionGlobalMission *
        ]
    ),
    "cGcGenericSectionConditionGlobalMission::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionGlobalMission *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionFreighterBattle::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionFreighterBattle *
        ]
    ),
    "cGcGenericSectionConditionFreighterBattle::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionFreighterBattle *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionExtraSuitSlots::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExtraSuitSlots *
        ]
    ),
    "cGcGenericSectionConditionExtraSuitSlots::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExtraSuitSlots *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionCanReceiveReward::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCanReceiveReward *
        ]
    ),
    "cGcGenericSectionConditionCanReceiveReward::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCanReceiveReward *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionPercentChance::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPercentChance *
        ]
    ),
    "cGcGenericSectionConditionPercentChance::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPercentChance *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionGroup::cGcGenericSectionConditionGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionGroup *
            ctypes.c_ulonglong,  # cGcMissionConditionGroup *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionGroup::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionGroup *
        ]
    ),
    "cGcGenericSectionConditionGroup::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionGroup *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionGroup::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionGroup *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionAIShipCount::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAIShipCount *
        ]
    ),
    "cGcGenericSectionConditionAIShipCount::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAIShipCount *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionNumberOfShipsOwned::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNumberOfShipsOwned *
        ]
    ),
    "cGcGenericSectionConditionNumberOfShipsOwned::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNumberOfShipsOwned *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionNumAtlasStationsVisited::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNumAtlasStationsVisited *
        ]
    ),
    "cGcGenericSectionConditionNumAtlasStationsVisited::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNumAtlasStationsVisited *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasEndpointForEvent::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasEndpointForEvent *
        ]
    ),
    "cGcGenericSectionConditionHasEndpointForEvent::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasEndpointForEvent *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionInUA::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInUA *
        ]
    ),
    "cGcGenericSectionConditionInUA::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInUA *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionEventRequiresRGB::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionEventRequiresRGB *
        ]
    ),
    "cGcGenericSectionConditionEventRequiresRGB::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionEventRequiresRGB *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionPulseEncounterOverriden::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPulseEncounterOverriden *
        ]
    ),
    "cGcGenericSectionConditionPulseEncounterOverriden::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPulseEncounterOverriden *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionPulseEncounterActive::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPulseEncounterActive *
        ]
    ),
    "cGcGenericSectionConditionPulseEncounterActive::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPulseEncounterActive *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionAbandonedFreighterExplored::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAbandonedFreighterExplored *
        ]
    ),
    "cGcGenericSectionConditionAbandonedFreighterExplored::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAbandonedFreighterExplored *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionAbandonedFreighterExplored::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAbandonedFreighterExplored *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionAlienPodAggroed::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAlienPodAggroed *
        ]
    ),
    "cGcGenericSectionConditionAlienPodAggroed::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAlienPodAggroed *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasEntitlement::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasEntitlement *
        ]
    ),
    "cGcGenericSectionConditionHasEntitlement::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasEntitlement *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionUnclaimedStageReward::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionUnclaimedStageReward *
        ]
    ),
    "cGcGenericSectionConditionUnclaimedStageReward::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionUnclaimedStageReward *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionExpeditionNearlyOver::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExpeditionNearlyOver *
        ]
    ),
    "cGcGenericSectionConditionExpeditionNearlyOver::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExpeditionNearlyOver *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionExpeditionNearlyOver::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExpeditionNearlyOver *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionHasSeasonalReward::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasSeasonalReward *
        ]
    ),
    "cGcGenericSectionConditionHasSeasonalReward::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasSeasonalReward *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionSeasonRewardsRecipe::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSeasonRewardsRecipe *
        ]
    ),
    "cGcGenericSectionConditionSeasonRewardsRecipe::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSeasonRewardsRecipe *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasTwitchReward::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasTwitchReward *
        ]
    ),
    "cGcGenericSectionConditionHasTwitchReward::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasTwitchReward *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasPlatformReward::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasPlatformReward *
        ]
    ),
    "cGcGenericSectionConditionHasPlatformReward::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasPlatformReward *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasMultiTool::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasMultiTool *
        ]
    ),
    "cGcGenericSectionConditionHasShip::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasShip *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionHasMultiTool::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasMultiTool *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasShip::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasShip *
        ]
    ),
    "cGcGenericSectionConditionHasShip::DoesShipMatch": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasShip *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcGenericSectionConditionHasShip::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasShip *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionCreatureOwned::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureOwned *
        ]
    ),
    "cGcGenericSectionConditionCreatureOwned::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureOwned *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionCreatureSlots::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureSlots *
        ]
    ),
    "cGcGenericSectionConditionCreatureSlots::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureSlots *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionCreatureTrust::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureTrust *
        ]
    ),
    "cGcGenericSectionConditionCreatureTrust::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureTrust *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionCreatureTrust::GetHighestTrust": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureTrust *
        ]
    ),
    "cGcGenericSectionConditionCreatureTrust::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureTrust *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionCreatureSummoned::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureSummoned *
        ]
    ),
    "cGcGenericSectionConditionCreatureSummoned::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureSummoned *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionCreatureReadyToHatch::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureReadyToHatch *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionCreatureReadyToHatch::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureReadyToHatch *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionCreatureReadyToLay::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureReadyToLay *
        ]
    ),
    "cGcGenericSectionConditionCreatureReadyToLay::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureReadyToLay *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionSquadronPilotsOwned::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSquadronPilotsOwned *
        ]
    ),
    "cGcGenericSectionConditionSquadronPilotsOwned::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSquadronPilotsOwned *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionSquadronSlots::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSquadronSlots *
        ]
    ),
    "cGcGenericSectionConditionSquadronSlots::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSquadronSlots *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionCurrentSlope::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCurrentSlope *
        ]
    ),
    "cGcGenericSectionConditionCurrentSlope::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCurrentSlope *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionElevation::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionElevation *
        ]
    ),
    "cGcGenericSectionConditionElevation::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionElevation *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionElevation::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionElevation *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionUnderwaterDepth::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionUnderwaterDepth *
        ]
    ),
    "cGcGenericSectionConditionUnderwaterDepth::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionUnderwaterDepth *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionUnderwaterDepth::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionUnderwaterDepth *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionNearPole::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNearPole *
        ]
    ),
    "cGcGenericSectionConditionNearPole::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNearPole *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionIsDepotDestroyed::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsDepotDestroyed *
        ]
    ),
    "cGcGenericSectionConditionAreDroneHivePartsDestroyed::AllowMissionRestart": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsDepotDestroyed *
        ]
    ),
    "cGcGenericSectionConditionIsDepotDestroyed::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsDepotDestroyed *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionIsDepotDestroyed::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsDepotDestroyed *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionAreDroneHivePartsDestroyed::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAreDroneHivePartsDestroyed *
        ]
    ),
    "cGcGenericSectionConditionAreDroneHivePartsDestroyed::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAreDroneHivePartsDestroyed *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionIsScanEventLocal::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsScanEventLocal *
        ]
    ),
    "cGcGenericSectionConditionIsScanEventOnCurrentPlanet::AllowMissionRestart": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsScanEventRepaired *
        ]
    ),
    "cGcGenericSectionConditionIsScanEventLocal::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsScanEventLocal *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionNearestBuilding::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNearestBuilding *
        ]
    ),
    "cGcGenericSectionConditionNearestBuilding::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNearestBuilding *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionIsScanEventRepaired::cGcGenericSectionConditionIsScanEventRepaired": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsScanEventRepaired *
            ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventRepaired *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionIsScanEventRepaired::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsScanEventRepaired *
        ]
    ),
    "cGcGenericSectionConditionIsScanEventRepaired::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsScanEventRepaired *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionIsScanEventRepaired::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsScanEventRepaired *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionIsScanEventOnCurrentPlanet::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsScanEventOnCurrentPlanet *
        ]
    ),
    "cGcGenericSectionConditionIsScanEventOnCurrentPlanet::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsScanEventOnCurrentPlanet *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionIsScanEventActive::cGcGenericSectionConditionIsScanEventActive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsScanEventActive *
            ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventActive *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionIsScanEventActive::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsScanEventActive *
        ]
    ),
    "cGcGenericSectionConditionIsScanEventActive::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsScanEventActive *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionIsMissionInProgress::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsMissionInProgress *
        ]
    ),
    "cGcGenericSectionConditionIsMissionInProgress::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsMissionInProgress *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionNearObject::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNearObject *
        ]
    ),
    "cGcGenericSectionConditionNearObject::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNearObject *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionNearScanEvent::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNearScanEvent *
        ]
    ),
    "cGcGenericSectionConditionNearScanEvent::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNearScanEvent *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionUsingPortal::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionUsingPortal *
        ]
    ),
    "cGcGenericSectionConditionUsingPortal::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionUsingPortal *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHazard::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHazard *
        ]
    ),
    "cGcGenericSectionConditionHazard::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHazard *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionWeather::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWeather *
        ]
    ),
    "cGcGenericSectionConditionWeather::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWeather *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionRequestedPhoto::cGcGenericSectionConditionRequestedPhoto": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionRequestedPhoto *
            ctypes.c_ulonglong,  # cGcMissionConditionRequestedPhoto *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionRequestedPhoto::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionRequestedPhoto *
        ]
    ),
    "cGcGenericSectionConditionRequestedPhoto::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionRequestedPhoto *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionInCombat::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInCombat *
        ]
    ),
    "cGcGenericSectionConditionInCombat::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInCombat *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionInCombat::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInCombat *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionInCombat::EncouragesFightingSentinels": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInCombat *
        ]
    ),
    "cGcGenericSectionConditionWaitForTime::cGcGenericSectionConditionWaitForTime": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWaitForTime *
            ctypes.c_ulonglong,  # cGcMissionConditionWaitForTime *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionWaitForTime::ResetUserData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWaitForTime *
        ]
    ),
    "cGcGenericSectionConditionWaitForTime::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWaitForTime *
        ]
    ),
    "cGcGenericSectionConditionWaitForTime::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWaitForTime *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionOnOtherSideOfPortal::cGcGenericSectionConditionOnOtherSideOfPortal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionOnOtherSideOfPortal *
            ctypes.c_ulonglong,  # cGcMissionConditionOnOtherSideOfPortal *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionOnOtherSideOfPortal::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionOnOtherSideOfPortal *
        ]
    ),
    "cGcGenericSectionConditionOnOtherSideOfPortal::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionOnOtherSideOfPortal *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionIsScanEventLocalOrNear::cGcGenericSectionConditionIsScanEventLocalOrNear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsScanEventLocalOrNear *
            ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventLocalOrNear *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionIsScanEventLocalOrNear::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsScanEventLocalOrNear *
        ]
    ),
    "cGcGenericSectionConditionIsScanEventLocalOrNear::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsScanEventLocalOrNear *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionWaitForPirates::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWaitForPirates *
        ]
    ),
    "cGcGenericSectionConditionWaitForPirates::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWaitForPirates *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionWaitForPirates::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWaitForPirates *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionReadyToSpawnPirates::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionReadyToSpawnPirates *
        ]
    ),
    "cGcGenericSectionConditionReadyToSpawnPirates::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionReadyToSpawnPirates *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasSettlement::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasSettlement *
        ]
    ),
    "cGcGenericSectionConditionHasSettlement::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasSettlement *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasSettlementLocal::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasSettlementLocal *
        ]
    ),
    "cGcGenericSectionConditionHasSettlementLocal::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasSettlementLocal *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionSettlementStatLevel::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSettlementStatLevel *
        ]
    ),
    "cGcGenericSectionConditionSettlementStatLevel::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSettlementStatLevel *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasSettlementBuilding::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasSettlementBuilding *
        ]
    ),
    "cGcGenericSectionConditionHasSettlementBuilding::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasSettlementBuilding *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasAnySettlementBuildingInProgress::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasAnySettlementBuildingInProgress *
        ]
    ),
    "cGcGenericSectionConditionHasAnySettlementBuildingInProgress::FormatConditionText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasAnySettlementBuildingInProgress *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericSectionConditionHasAnySettlementBuildingInProgress::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasAnySettlementBuildingInProgress *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasSettlementProductPending::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasSettlementProductPending *
        ]
    ),
    "cGcGenericSectionConditionHasSettlementProductPending::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasSettlementProductPending *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionSettlementBuildingsAllStarted::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSettlementBuildingsAllStarted *
        ]
    ),
    "cGcGenericSectionConditionSettlementBuildingsAllStarted::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSettlementBuildingsAllStarted *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasPendingSettlementJudgement::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasPendingSettlementJudgement *
        ]
    ),
    "cGcGenericSectionConditionHasPendingSettlementJudgement::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasPendingSettlementJudgement *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionSettlementsEnabled::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSettlementsEnabled *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionStartWithAllPartsKnown::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionStartWithAllPartsKnown *
        ]
    ),
    "cGcGenericSectionConditionStartWithAllPartsKnown::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionStartWithAllPartsKnown *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionItemCostsEnabled::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionItemCostsEnabled *
        ]
    ),
    "cGcGenericSectionConditionItemCostsEnabled::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionItemCostsEnabled *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHazardsEnabled::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHazardsEnabled *
        ]
    ),
    "cGcGenericSectionConditionHazardsEnabled::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHazardsEnabled *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionLifeSupportEnabled::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionLifeSupportEnabled *
        ]
    ),
    "cGcGenericSectionConditionLifeSupportEnabled::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionLifeSupportEnabled *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionTutorialEnabled::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionTutorialEnabled *
        ]
    ),
    "cGcGenericSectionConditionTutorialEnabled::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionTutorialEnabled *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionOnFootCombatEnabled::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionOnFootCombatEnabled *
        ]
    ),
    "cGcGenericSectionConditionOnFootCombatEnabled::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionOnFootCombatEnabled *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionSpaceCombatEnabled::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSpaceCombatEnabled *
        ]
    ),
    "cGcGenericSectionConditionSpaceCombatEnabled::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSpaceCombatEnabled *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionAutoPowerEnabled::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAutoPowerEnabled *
        ]
    ),
    "cGcGenericSectionConditionAutoPowerEnabled::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAutoPowerEnabled *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionCommunityResearchTier::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCommunityResearchTier *
        ]
    ),
    "cGcGenericSectionConditionCommunityResearchTier::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCommunityResearchTier *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasProcMissionForFaction::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasProcMissionForFaction *
        ]
    ),
    "cGcGenericSectionConditionHasProcMissionForFaction::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasProcMissionForFaction *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionVisorActive::IsConditionTrue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
        ]
    ),
    "cGcGenericSectionConditionVisorActive::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionVisorActive *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cTkFixedString<1024,char>::Format<char,0>": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcScanEventManager::GetEventUAFromMission": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcSpaceshipComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceMapObject::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMapObject *
        ]
    ),
    "cGcSpaceMapObject::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMapObject *
        ]
    ),
    "cGcSolarSystem::GetAnomaly": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcSpaceMapObject::Show": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMapObject *
        ]
    ),
    "cGcScannableComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcSpaceMapObject::Hide": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMapObject *
        ]
    ),
    "cGcNotificationSequenceStage::~cGcNotificationSequenceStage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceStage::SetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceStage::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequence::GetMissionDataTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceStage::GetCurrentScanEventTarget": FUNCDEF(
        restype=ctypes.c_char_p,  # const TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcNotificationSequence::UseCommunityMissionForLog": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "OnNodeActivated::GetSaveSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # StopParticles *
        ]
    ),
    "cGcNotificationSequenceWaitRealTime::WaitedRealTime": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceWaitRealTimeCombat::WaitedRealTime": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceMessage::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceMessage::DisableIcon": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceMessage *
        ]
    ),
    "cGcNotificationSequenceMessage::DisableTitlePrefix": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceMessage *
        ]
    ),
    "cGcNotificationSequenceGatherFor::AllowAutoEndPinnedMission": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGatherFor *
        ]
    ),
    "cGcNotificationSequenceGatherFor::GetTitleForCodeGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGatherFor *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceRepairFinal::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceRepairFinal *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceGetToExpedition::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceCompleteMission::Run": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotifyGenericSection::GetTargetMissionSurveyId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotifyGenericSection *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcNotificationSequenceGroup::GetTargetMissionSurveyDefinitelyExists": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceGroup *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGroup *
        ]
    ),
    "cGcNotificationSequence::GetCurrentCategory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotificationSequence::GetCurrentIcon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSlotIcon *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotificationSequence::GetIconAtStage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNotificationSequence::GetCurrentFrontendPageHint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotificationSequence::GetCurrentInventoryHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcNotificationSequence::GetCurrentFrontendPageLocID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_ulonglong,  # TkID<256> *
        ]
    ),
    "cGcNotificationSequence::Restart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotificationSequence::GoBackToStage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNotificationSequence::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_ulonglong,  # const cGcPlayerMissionProgress *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNotificationSequence::Save": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_ulonglong,  # cGcPlayerMissionProgress *
        ]
    ),
    "cGcNotificationSequenceEditor::RenderImGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcQuickMenu::IsQuickMenuOpen": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenu *
        ]
    ),
    "cGcGenericSectionConsequenceRemoveScanEvent::DoConsequence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConsequenceRemoveScanEvent *
        ]
    ),
    "cGcGenericSectionConsequenceRemoveScanEvent::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConsequenceRemoveScanEvent *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcGenericSectionConsequenceRemoveCommunicatorMessage::DoConsequence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConsequenceRemoveCommunicatorMessage *
        ]
    ),
    "cGcGenericSectionConsequenceRemoveCommunicatorMessage::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConsequenceRemoveCommunicatorMessage *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcGenericSectionConsequenceRemoveCommunicatorTakeOffMessage::DoConsequence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConsequenceRemoveCommunicatorTakeOffMessage *
        ]
    ),
    "cGcGenericSectionConsequenceRemoveCommunicatorTakeOffMessage::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConsequenceRemoveCommunicatorTakeOffMessage *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcGenericSectionConsequenceResetPulseEncounterOverride::DoConsequence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConsequenceResetPulseEncounterOverride *
        ]
    ),
    "cGcGenericSectionConsequenceResetPulseEncounterOverride::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConsequenceResetPulseEncounterOverride *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcRealityManager::LookupPinObjectiveByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_ulonglong,  # TkID<256> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcNotificationScheduler::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationScheduler *
        ]
    ),
    "cGcNotificationScheduler::GetTimeToDeadline": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationScheduler *
            ctypes.c_ulonglong,  # const cGcNotificationSequence *
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationScheduler::GetDeadline": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationScheduler *
            ctypes.c_ulonglong,  # const cGcNotificationSequence *
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        ]
    ),
    "CalculateUpcomingRecurrenceDeadline": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSchedulingData *
        ]
    ),
    "GetServerTime": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[]
    ),
    "IsRecurrenceExpired": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "IsLeapYear": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNotificationScheduler::GetCurrentCommunityMissionID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationScheduler *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # const cGcMissionCommunityData *
        ]
    ),
    "cGcNotificationScheduler::UpdateCachedAtlasCommunityData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationScheduler *
        ]
    ),
    "cGcNotificationScheduler::HandleCommunityMissions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationScheduler *
        ]
    ),
    "cGcNotificationScheduler::HandleRecurringMissions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationScheduler *
        ]
    ),
    "cGcNotificationSequenceDataRenderer::SetSelectedSequence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotificationSequenceFactory::Create": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcNotificationSequence::cGcNotificationSequence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcNotificationSequence::~cGcNotificationSequence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotificationSequence::Update": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotificationSequence::HasText": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotificationSequence::GetText": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequence::CanBeRestarted": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotificationSequence::IsBasePartUnlockedByMission": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcNotificationSequence::GetSubtitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcNotificationSequence::GetDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequence::ShowCommunityProgress": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotificationSequence::GetCurrentColour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_ulonglong,  # cTkColour *
        ]
    ),
    "cGcNotificationSequence::NGuiRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotificationSequence::ShouldBeInLog": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotificationSequence::CanBeRenounced": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotificationSequence::RenounceMission": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequence::SelectWeekendEventMissionID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcNotificationSequence::GetMultiplayerMissionType": {
        "const TkID<128> *, const cTkSeed *, const bool, const cGcGenericMissionSequence *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ulonglong,  # const cTkSeed *
                ctypes.c_ubyte,  # const bool
                ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            ]
        ),
        "cGcNotificationSequence *, const bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcNotificationSequence *
                ctypes.c_ubyte,  # const bool
            ]
        ),
    },
    "cGcNotificationSequence::OnSequenceComplete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotifyGenericSection::~cGcNotifyGenericSection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotifyGenericSection *
        ]
    ),
    "cGcNotifyGenericSection::GetIcon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotifyGenericSection *
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        ]
    ),
    "cGcNotificationSequence::GetAppropriateMissionSurveyHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_ulonglong,  # TkID<256> *
        ]
    ),
    "cGcNotificationSequence::GetMissionPhotoTargetPhotoBuilding": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # cTkSeed *
        ]
    ),
    "cGcNotificationSequence::GetMissionPhotoTargetBuilding": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # cTkSeed *
        ]
    ),
    "cGcNotificationSequenceDataRenderer::RenderDataBrowser": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcNotificationSequenceDataRenderer::RenderSelectMissionFromData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcNotificationSequenceDataRenderer::RenderMissionSequence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotificationSequenceEditor::RenderConditions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceEditor *
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkClassPointer> *
        ]
    ),
    "cGcNotificationSequenceEditor::RenderSequences": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceEditor *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGenericMissionStage> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNotificationSequenceGroup::GetObjective": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGroup *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceGroup::GetObjectiveTip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGroup *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceGroup::GetFormattableObjective": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGroup *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceGroup::ObjectivesCanBeFormatted": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGroup *
        ]
    ),
    "cGcEncounterComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEncounterComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequenceGatherFor::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGatherFor *
        ]
    ),
    "cGcNotificationSequenceGatherFor::GetRequirements": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGatherFor *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequenceGatherFor::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceGatherFor::GetRemaining": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGatherFor *
        ]
    ),
    "cGcNotificationSequenceGatherFor::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGatherFor *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequenceGatherFor::GetSurveyHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGatherFor *
            ctypes.c_ulonglong,  # TkID<256> *
        ]
    ),
    "cGcNotificationSequenceGatherForCrafting::GatherTargetAlreadyAcheived": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGatherForCrafting *
        ]
    ),
    "cGcNotificationSequenceGatherForCrafting::DoesTargetDamagedTechStillNeedReq": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGatherForCrafting *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequenceWaitForLearnedProduct::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForLearnedProduct *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForLearnedProduct::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceWaitForLearnedTech::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForLearnedTech *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForLearnedTech::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceProductAmountNeeded::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceProductAmountNeeded *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceProductAmountNeeded::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceUnitsNeeded::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceUnitsNeeded *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceUnitsNeeded::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceGetUnits::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGetUnits *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceGetUnits::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceRepairFinal::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceBuildFinal::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceBuildFinal *
        ]
    ),
    "cGcNotificationSequenceBuildFinal::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceBuildFinal::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceBuildFinal *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceCraftFinal::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCraftFinal *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceCraftFinal::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceCraftFinal::GetBuildMenuHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCraftFinal *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcNotificationSequenceCleanUpPinnedRecipe::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceRefuelFinal::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceStartEvent::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceStartEvent::DeepInterstellarSearch": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceEndEvent::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceScan::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceVehicleScan::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceGetToScanEvent::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGetToScanEvent *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceGetToScanEvent::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGetToScanEvent *
        ]
    ),
    "cGcNotificationSequenceGetToScanEvent::RunInternal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGetToScanEvent *
        ]
    ),
    "cGcNotificationSequenceWaitForScanEvent::AllowMissionRestart": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForScanEvent *
        ]
    ),
    "cGcNotificationSequenceGetToScanEvent::TimeRemaining": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGetToScanEvent *
        ]
    ),
    "cGcNotificationSequenceGetToScanEvent::NeedsGPSHint": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGetToScanEvent *
        ]
    ),
    "cGcNotificationSequenceGetToScanEvent::OverridesMissionSurveyText": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForScanEvent *
        ]
    ),
    "cGcNotificationSequenceWaitForScanEvent::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForScanEvent *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForScanEvent::NeedsGPSHint": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForScanEvent *
        ]
    ),
    "cGcNotificationSequenceWaitForScanEvent::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForScanEvent *
        ]
    ),
    "cGcNotificationSequenceWaitForScanEvent::RunInternal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForScanEvent *
        ]
    ),
    "cGcNotificationSequenceWaitForScanEvent::TimeRemaining": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForScanEvent *
        ]
    ),
    "cGcNotificationSequenceWaitForFriendlyDroneScanEvent::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForFriendlyDroneScanEvent *
        ]
    ),
    "cGcNotificationSequenceWaitForFriendlyDroneScanEvent::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForFriendlyDroneScanEvent *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForFriendlyDroneScanEvent::RunInternal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForFriendlyDroneScanEvent *
        ]
    ),
    "cGcNotificationSequenceGetInShip::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceGatherForBuild::GatherTargetAlreadyAcheived": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGatherForBuild *
        ]
    ),
    "cGcNotificationSequenceGatherForRefuel::GatherTargetAlreadyAcheived": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGatherForRefuel *
        ]
    ),
    "cGcNotificationSequenceGatherForRepair::GatherTargetAlreadyAcheived": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGatherForRepair *
        ]
    ),
    "cGcNotificationSequenceGoToGalacticPoint::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGoToGalacticPoint *
        ]
    ),
    "cGcNotificationSequenceGoToGalacticPoint::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceSignalGalacticPoint::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceSignalGalacticPoint *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceSignalGalacticPoint::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceSignalGalacticPoint *
        ]
    ),
    "cGcNotificationSequenceSignalGalacticPoint::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceBaseBuildingBuildPart::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceBaseBuildingBuildPart::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceBaseBuildingBuildPart *
        ]
    ),
    "cGcNotificationSequenceBaseBuildingBuildPart::GetCurrentCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceBaseBuildingBuildPart *
        ]
    ),
    "cGcNotificationSequenceWaitForMessage::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceWaitForMessage::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForMessage *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForCompletionMessage::IsComplete": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceReward::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceBroadcastMessage::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequenceMissionUpdateMessage::RunInternal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceMissionUpdateMessage *
        ]
    ),
    "cGcNotificationSequenceMessage::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceMessage *
        ]
    ),
    "cGcNotificationSequenceMessage::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceMessage *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceMessage::RunInternal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceMessage *
        ]
    ),
    "cGcNotificationSequencePodMessage::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequencePodMessage *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceShowSeasonTimeWarning::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceShowSeasonTimeWarning *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceDetailMessage::RunInternal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceDetailMessage *
        ]
    ),
    "cGcNotificationSequenceDetailMessage::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceDetailMessage *
        ]
    ),
    "cGcNotificationSequenceSetCurrentMission::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceStartMission::DefaultCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceClearInventoryHistory::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceCloseMenu::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<MissionDetailMessage,8,-1> *
        ]
    ),
    "cGcNotificationSequenceRepairTech::RunInternal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceRepairTech *
        ]
    ),
    "cGcNotificationSequenceRepairTech::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceRepairTech *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceCreateSpecificPulseEncounter::RunInternal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCreateSpecificPulseEncounter *
        ]
    ),
    "cGcNotificationSequenceCreateSpecificPulseEncounter::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCreateSpecificPulseEncounter *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForAbandFreighterDoorOpen::RunInternal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForAbandFreighterDoorOpen *
        ]
    ),
    "cGcNotificationSequenceWaitForAbandFreighterDoorOpen::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForAbandFreighterDoorOpen *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceConstruct::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceConstruct *
        ]
    ),
    "cGcNotificationSequenceConstruct::RunInternal": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceConstruct *
        ]
    ),
    "cGcNotificationSequenceConstruct::GetBuildMenuHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceConstruct *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcNotificationSequenceConstruct::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceConstruct *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceLeaveNexusMP::RunInternal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceLeaveNexusMP *
        ]
    ),
    "cGcNotificationSequenceLeaveNexusMP::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceLeaveNexusMP *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceVisitPlanets::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceVisitPlanets *
        ]
    ),
    "cGcNotificationSequenceVisitPlanets::RunInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceVisitPlanets *
        ]
    ),
    "cGcNotificationSequenceVisitPlanets::GetPlanetStatFromBiome": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceVisitPlanets *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_uint32,  # eBiome
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequenceWaitForPhoto::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForPhoto *
        ]
    ),
    "cGcNotificationSequenceWaitForPhoto::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForPhoto *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceCollectMoney::CalculateRequiredAmount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollectMoney *
        ]
    ),
    "cGcNotificationSequenceGetToExpedition::cGcNotificationSequenceGetToExpedition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGetToExpedition *
            ctypes.c_ulonglong,  # cGcMissionSequenceGetToExpedition *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcNotificationSequenceGetToExpedition::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGetToExpedition *
        ]
    ),
    "cGcNotificationSequenceGetToExpedition::RunInternal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGetToExpedition *
        ]
    ),
    "cGcNotificationSequenceCompleteMission::cGcNotificationSequenceCompleteMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCompleteMission *
            ctypes.c_ulonglong,  # cGcMissionSequenceCompleteMission *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcNotificationSequenceWaitRealTime::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitRealTime *
        ]
    ),
    "cGcNotificationSequenceDisplaySeasonRewardReminder::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceDisplaySeasonRewardReminder *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceWaitRealTime::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitRealTime *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceWaitRealTimeCombat::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitRealTimeCombat *
        ]
    ),
    "cGcNotificationSequenceWaitRealTimeCombat::OverrideWithDangerColour": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitRealTimeCombat *
        ]
    ),
    "cGcNotificationSequenceWaitRealTimeCombat::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitRealTimeCombat *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForSuitUpgrade::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceModifyStat::RunInternal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceModifyStat *
        ]
    ),
    "cGcNotificationSequenceCompleteSeasonalMilestone::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceWaitForSettlementActivity::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForSettlementActivity *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceEnsureBarrelsAtPlayerSettlement::RunInternal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceEnsureBarrelsAtPlayerSettlement *
        ]
    ),
    "cGcNotificationSequenceConstructSettlementBuildingWithScanEvent::RunInternal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceConstructSettlementBuildingWithScanEvent *
        ]
    ),
    "cGcNotificationSequenceConstructSettlementBuildingWithScanEvent::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceConstructSettlementBuildingWithScanEvent *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceConstructSettlementBuildingWithScanEvent::GetFormattedObjectiveTip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceConstructSettlementBuildingWithScanEvent *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequenceCompleteSettlementJudgement::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCompleteSettlementJudgement *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceExploreAbandonedFreighter::RunInternal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExploreAbandonedFreighter *
        ]
    ),
    "cGcNotificationSequenceExploreAbandonedFreighter::GetTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExploreAbandonedFreighter *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceExplorationLogSpecial::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExplorationLogSpecial *
        ]
    ),
    "cGcNotificationSequenceExplorationLogSpecial::RunInternal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExplorationLogSpecial *
        ]
    ),
    "cGcNotificationSequenceExplorationLogSpecial::UpdateSystemConstants": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExplorationLogSpecial *
        ]
    ),
    "cGcNotificationSequenceExplorationLogSpecial::UpdatePlanetConstants": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExplorationLogSpecial *
        ]
    ),
    "cGcNotificationSequenceExplorationLogSpecial::GetFormattedObjectiveTip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExplorationLogSpecial *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceExplorationLogSpecial::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExplorationLogSpecial *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceExplorationLogSpecial::GetRatingFormat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExplorationLogSpecial *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequenceExplorationLogSpecial::GenerateStarClassText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExplorationLogSpecial *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceExplorationLogSpecial::GeneratePlanetFact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExplorationLogSpecial *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcNotificationSequenceExplorationLogSpecial::GeneratePlanetAtmosFact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExplorationLogSpecial *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcNotificationSequenceExplorationLogSpecial::GeneratePlanetTerrainFact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExplorationLogSpecial *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcNotificationSequenceExplorationLogSpecial::GeneratePlanetCoreFact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExplorationLogSpecial *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcNotificationSequenceExplorationLogSpecial::GetTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExplorationLogSpecial *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceExplorationLogSpecial::GetFormattedObjective": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExplorationLogSpecial *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceExplorationLogSpecial::FormatTitleDetails": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExplorationLogSpecial *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequenceExplorationLogSpecial::DiscoveryDataSubmitted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExplorationLogSpecial *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcNotificationSequenceKillEncounter::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceKillEncounter *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceStage::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cTkMetadataCache<cGcMissionTable>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cGcMissionTable> *
        argtypes=[]
    ),
    "cTkDynamicArray<cGcWordKnowledge>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMissionIDEpochPair> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNotificationSequenceCollectMoney::FormatText<cTkFixedString<512,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollectMoney *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceCollectMoney::FormatText<cTkFixedString<128,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollectMoney *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceCollectMoney::FormatText<cTkFixedString<1024,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollectMoney *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceBaseBuildingBuildPart::FormatText<cTkFixedString<512,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceBaseBuildingBuildPart *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceBaseBuildingBuildPart::FormatText<cTkFixedString<128,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceBaseBuildingBuildPart *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceBaseBuildingBuildPart::FormatText<cTkFixedString<1024,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceBaseBuildingBuildPart *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForPhoto::FormatText<cTkFixedString<128,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForPhoto *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForPhoto::FormatText<cTkFixedString<1024,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForPhoto *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceVisitPlanets::FormatLongText<cTkFixedString<512,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceVisitPlanets *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceVisitPlanets::FormatText<cTkFixedString<128,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceVisitPlanets *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceVisitPlanets::FormatLongText<cTkFixedString<1024,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceVisitPlanets *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceExploreAbandonedFreighter::FormatText<cTkFixedString<512,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExploreAbandonedFreighter *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceExploreAbandonedFreighter::FormatText<cTkFixedString<1024,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExploreAbandonedFreighter *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequence::DoCommunityMissionTextSubstitution<cTkFixedString<512,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequence::DoCommunityMissionTextSubstitution<cTkFixedString<1024,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGenericMissionSequence>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericMissionSequence *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditor::DoComboBox<eAutoStart>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eAutoStart *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNotifyGenericSection::cGcNotifyGenericSection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotifyGenericSection *
            ctypes.c_ulonglong,  # const cGcNotifyGenericSection *
        ]
    ),
    "GcDiscoverySearchConstraints::FilterSolarSystem::Accept": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # GcDiscoverySearchConstraints::FilterSolarSystem *
            ctypes.c_ulonglong,  # const cGcDiscoveryRecord *
        ]
    ),
    "cGcUniverseAddressCoderU64::InSameSystem": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNGuiEditor::DoEditID": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcPlayerExperienceDirector::SetPulseEncounterOverrideChance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_float,  # float
        ]
    ),
    "cGcRealityManager::AmountOfItemNeededToBuildTech": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::AmountOfItemNeededToBuildProduct": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcNotificationTextHelper::AmountMoreString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNotificationTextHelper::GetMarkedUpItemString": {
        "const TkID<128> *, cTkFixedString<256,char> *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ulonglong,  # cTkFixedString<256,char> *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const TkID<128> *, int, cTkFixedString<256,char> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ]
        ),
    },
    "cGcNotificationTextHelper::GetMarkedUpItemStringWithIcon": {
        "const TkID<128> *, cTkFixedString<256,char> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ]
        ),
        "const TkID<128> *, int, cTkFixedString<256,char> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ]
        ),
    },
    "cGcNotificationTextHelper::GetRequirementsString": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationTextHelper::GetGPSString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # eScanEventGPSHint
            ctypes.c_char * 0x20,  # TkID<256>
            common.GcSeed,  # cTkSeed
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcNotificationTextHelper::GetSettlementConstructionLevelLocID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # TkID<256> *
            ctypes.c_uint32,  # eSettlementConstructionLevel
        ]
    ),
    "cGcNotificationHelper::GetBuildingPartCountFromType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # cGcBuildingPartSearchType *
        ]
    ),
    "cGcNotificationSequenceWait::Waited": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceWait::SuppressMessages": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWait *
        ]
    ),
    "cGcNotificationSequenceDisplaySeasonRewardReminder::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceCollectMoney::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceCollectMoney::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollectMoney *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceCollectMoney::GetFormattedObjective": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollectMoney *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceCollectMoney::GetFormattedObjectiveTip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollectMoney *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceMissionUpdateMessage::cGcNotificationSequenceMissionUpdateMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceMissionUpdateMessage *
            ctypes.c_uint32,  # eMissionUpdateMessage
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # ePlayMusicSting
            ctypes.c_char * 0x20,  # TkID<256>
            ctypes.c_char * 0x20,  # TkID<256>
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceMissionUpdateMessage::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequencePodMessage::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceShowSeasonTimeWarning::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceDetailMessage::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceStop::cGcNotificationSequenceStop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStop *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceCloseMenu::cGcNotificationSequenceCloseMenu": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCloseMenu *
            ctypes.c_uint32,  # eMenuToClose
            ctypes.c_float,  # float
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceCreateSpecificPulseEncounter::cGcNotificationSequenceCreateSpecificPulseEncounter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCreateSpecificPulseEncounter *
            ctypes.c_ulonglong,  # cGcMissionSequenceCreateSpecificPulseEncounter *
        ]
    ),
    "cGcNotificationSequenceCreateSpecificPulseEncounter::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceWaitForAbandFreighterDoorOpen::cGcNotificationSequenceWaitForAbandFreighterDoorOpen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForAbandFreighterDoorOpen *
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForAbandFreighterDoorOpen *
        ]
    ),
    "cGcNotificationSequenceWaitForAbandFreighterDoorOpen::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceRepairTech::cGcNotificationSequenceRepairTech": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceRepairTech *
            ctypes.c_ulonglong,  # cGcMissionSequenceRepairTech *
        ]
    ),
    "cGcNotificationSequenceRepairTech::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceGatherForCrafting::cGcNotificationSequenceGatherForCrafting": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGatherForCrafting *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceGatherForRepair::cGcNotificationSequenceGatherForRepair": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGatherForRepair *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # eStatsType
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceGatherForBuild::cGcNotificationSequenceGatherForBuild": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGatherForBuild *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceWaitForLearnedProduct::cGcNotificationSequenceWaitForLearnedProduct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForLearnedProduct *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceWaitForLearnedProduct::GetTitleForCodeGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForLearnedTech *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceGatherForRefuel::cGcNotificationSequenceGatherForRefuel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGatherForRefuel *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # eStatsType
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceProductAmountNeeded::cGcNotificationSequenceProductAmountNeeded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceProductAmountNeeded *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceUnitsNeeded::cGcNotificationSequenceUnitsNeeded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceUnitsNeeded *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceGetUnits::cGcNotificationSequenceGetUnits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGetUnits *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceRepairFinal::cGcNotificationSequenceRepairFinal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceRepairFinal *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # eStatsType
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceBuildFinal::cGcNotificationSequenceBuildFinal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceBuildFinal *
            ctypes.c_char_p,  # const char *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceCleanUpPinnedRecipe::cGcNotificationSequenceCleanUpPinnedRecipe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCleanUpPinnedRecipe *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceRefuelFinal::cGcNotificationSequenceRefuelFinal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceRefuelFinal *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # eStatsType
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceStartEvent::cGcNotificationSequenceStartEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStartEvent *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eScanTable
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_ulonglong,  # cGcPlayerMissionParticipant *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceEndEvent::cGcNotificationSequenceEndEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceEndEvent *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceScan::cGcNotificationSequenceScan": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceScan *
            ctypes.c_char_p,  # const char *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceVehicleScan::BlocksVehicleScans": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceVehicleScan *
            ctypes.c_char * 0x20,  # TkID<256>
        ]
    ),
    "cGcNotificationSequenceGetToScanEvent::cGcNotificationSequenceGetToScanEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGetToScanEvent *
            ctypes.c_ulonglong,  # cGcMissionSequenceGetToScanEvent *
        ]
    ),
    "cGcNotificationSequenceGetToScanEvent::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceWaitForScanEvent::GetCurrentScanEventTarget": FUNCDEF(
        restype=ctypes.c_char_p,  # const TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForScanEvent *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcNotificationSequenceWaitForScanEvent::cGcNotificationSequenceWaitForScanEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForScanEvent *
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForScanEvent *
        ]
    ),
    "cGcNotificationSequenceWaitForScanEvent::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceWaitForFriendlyDroneScanEvent::cGcNotificationSequenceWaitForFriendlyDroneScanEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForFriendlyDroneScanEvent *
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForFriendlyDroneScanEvent *
        ]
    ),
    "cGcNotificationSequenceWaitForFriendlyDroneScanEvent::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceCompleteSettlementJudgement::cGcNotificationSequenceCompleteSettlementJudgement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCompleteSettlementJudgement *
            ctypes.c_ulonglong,  # cGcMissionSequenceCompleteSettlementJudgement *
        ]
    ),
    "cGcNotificationSequenceCompleteSettlementJudgement::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceConstructSettlementBuildingWithScanEvent::cGcNotificationSequenceConstructSettlementBuildingWithScanEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceConstructSettlementBuildingWithScanEvent *
            ctypes.c_ulonglong,  # cGcMissionSequenceConstructSettlementBuildingWithScanEvent *
        ]
    ),
    "cGcNotificationSequenceConstructSettlementBuildingWithScanEvent::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceConstructSettlementBuildingWithScanEvent::FormatsObjectiveText": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceConstructSettlementBuildingWithScanEvent *
        ]
    ),
    "cGcNotificationSequenceWaitForSettlementActivity::cGcNotificationSequenceWaitForSettlementActivity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForSettlementActivity *
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForSettlementActivity *
        ]
    ),
    "cGcNotificationSequenceWaitForSettlementActivity::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceEnsureBarrelsAtPlayerSettlement::cGcNotificationSequenceEnsureBarrelsAtPlayerSettlement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceEnsureBarrelsAtPlayerSettlement *
            ctypes.c_ulonglong,  # cGcMissionSequenceEnsureBarrelsAtPlayerSettlement *
        ]
    ),
    "cGcNotificationSequenceEnsureBarrelsAtPlayerSettlement::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceBaseBuildingBuildPart::GetBuildMenuHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceBaseBuildingBuildPart *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcNotificationSequenceBaseBuildingBuildPart::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceBaseBuildingBuildPart *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceBaseBuildingBuildPart::GetFormattedObjective": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceBaseBuildingBuildPart *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceBaseBuildingBuildPart::GetFormattedObjectiveTip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceBaseBuildingBuildPart *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcBuildingPartSearchType::cGcBuildingPartSearchType": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingPartSearchType *
            ctypes.c_ulonglong,  # const cGcBuildingPartSearchType *
        ]
    ),
    "cGcNotificationSequenceGetInShip::cGcNotificationSequenceGetInShip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGetInShip *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceSignalGalacticPoint::cGcNotificationSequenceSignalGalacticPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceSignalGalacticPoint *
            ctypes.c_char_p,  # const char *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eGalacticMapRequest
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceGoToGalacticPoint::cGcNotificationSequenceGoToGalacticPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGoToGalacticPoint *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # eGalacticMapRequest
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceWaitForMessage::cGcNotificationSequenceWaitForMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceWaitForCompletionMessage::cGcNotificationSequenceWaitForCompletionMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForCompletionMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceWaitForSuitUpgrade::cGcNotificationSequenceWaitForSuitUpgrade": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForSuitUpgrade *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceWaitForSuitUpgrade::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForSuitUpgrade *
        ]
    ),
    "cGcNotificationSequenceSetCurrentMission::cGcNotificationSequenceSetCurrentMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceSetCurrentMission *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceClearInventoryHistory::cGcNotificationSequenceClearInventoryHistory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceClearInventoryHistory *
        ]
    ),
    "cGcNotificationSequenceAudioEvent::Run": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceConstruct::GetStatusMessageMarkup": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceConstruct *
        ]
    ),
    "cGcNotificationSequenceConstruct::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceSetGlobalMission::cGcNotificationSequenceSetGlobalMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceSetGlobalMission *
            ctypes.c_uint32,  # eGlobalMission
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceSetGlobalMission::Run": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceWaitForPhoto::cGcNotificationSequenceWaitForPhoto": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForPhoto *
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForPhoto *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcNotificationSequenceWaitForPhoto::GetFormattedObjective": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForPhoto *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForPhoto::GetFormattedObjectiveTip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForPhoto *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForPhoto::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceLeaveNexusMP::cGcNotificationSequenceLeaveNexusMP": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceLeaveNexusMP *
            ctypes.c_ulonglong,  # cGcMissionSequenceLeaveNexusMP *
        ]
    ),
    "cGcNotificationSequenceLeaveNexusMP::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceVisitPlanets::cGcNotificationSequenceVisitPlanets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceVisitPlanets *
            ctypes.c_ulonglong,  # cGcMissionSequenceVisitPlanets *
        ]
    ),
    "cGcNotificationSequenceVisitPlanets::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceVisitPlanets::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceVisitPlanets *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceVisitPlanets::GetFormattedObjective": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceVisitPlanets *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceVisitPlanets::GetFormattedObjectiveTip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceVisitPlanets *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceSendChatMessage::cGcNotificationSequenceSendChatMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceSendChatMessage *
            ctypes.c_ulonglong,  # cGcMissionSequenceSendChatMessage *
        ]
    ),
    "cGcNotificationSequenceSendChatMessage::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceSendChatMessage::GetStatusMessageMarkup": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceSendChatMessage *
        ]
    ),
    "cGcNotificationSequenceModifyStat::cGcNotificationSequenceModifyStat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceModifyStat *
            ctypes.c_ulonglong,  # cGcMissionSequenceModifyStat *
        ]
    ),
    "cGcNotificationSequenceModifyStat::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceCompleteSeasonalMilestone::cGcNotificationSequenceCompleteSeasonalMilestone": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCompleteSeasonalMilestone *
            ctypes.c_ulonglong,  # cGcMissionSequenceCompleteSeasonalMilestone *
        ]
    ),
    "cGcNotificationSequenceExploreAbandonedFreighter::cGcNotificationSequenceExploreAbandonedFreighter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExploreAbandonedFreighter *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # const bool
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcNotificationSequenceExploreAbandonedFreighter::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceExploreAbandonedFreighter::OverridesIcon": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExploreAbandonedFreighter *
        ]
    ),
    "cGcNotificationSequenceExploreAbandonedFreighter::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExploreAbandonedFreighter *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceExploreAbandonedFreighter::GetFormattedObjectiveTip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExploreAbandonedFreighter *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceExplorationLogSpecial::cGcNotificationSequenceExplorationLogSpecial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceExplorationLogSpecial *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceExplorationLogSpecial::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceKillEncounter::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcGenericSectionConditionBaseQuery::cGcGenericSectionConditionBaseQuery": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBaseQuery *
            ctypes.c_ulonglong,  # cGcBaseSearchFilter *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionBasePartsQuery::cGcGenericSectionConditionBasePartsQuery": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBasePartsQuery *
            ctypes.c_ulonglong,  # cGcBasePartSearchFilter *
            ctypes.c_ulonglong,  # cGcBaseSearchFilter *
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionPlanetResourceHint::cGcGenericSectionConditionPlanetResourceHint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPlanetResourceHint *
            ctypes.c_ulonglong,  # cGcMissionConditionPlanetResourceHint *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionPlanetCreatureRoles::cGcGenericSectionConditionPlanetCreatureRoles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPlanetCreatureRoles *
            ctypes.c_ulonglong,  # cGcMissionConditionPlanetCreatureRoles *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasIngredientsForItem::cGcGenericSectionConditionHasIngredientsForItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasIngredientsForItem *
            ctypes.c_ulonglong,  # cGcMissionConditionHasIngredientsForItem *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionInSeasonalUA::cGcGenericSectionConditionInSeasonalUA": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInSeasonalUA *
            ctypes.c_ulonglong,  # cGcMissionConditionInSeasonalUA *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionIsPartyPlanetUnlocked::cGcGenericSectionConditionIsPartyPlanetUnlocked": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsPartyPlanetUnlocked *
            ctypes.c_ulonglong,  # cGcMissionConditionIsPartyPlanetUnlocked *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionTakingDamage::cGcGenericSectionConditionTakingDamage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionTakingDamage *
            ctypes.c_ulonglong,  # cGcMissionConditionTakingDamage *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionShieldDown::cGcGenericSectionConditionShieldDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionShieldDown *
            ctypes.c_ulonglong,  # cGcMissionConditionShieldDown *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasLegacyBasePending::cGcGenericSectionConditionHasLegacyBasePending": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasLegacyBasePending *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionInMultiplayer::cGcGenericSectionConditionInMultiplayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInMultiplayer *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionWaitForPirates::cGcGenericSectionConditionWaitForPirates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWaitForPirates *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eEqualityEnum
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionReadyToSpawnPirates::cGcGenericSectionConditionReadyToSpawnPirates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionReadyToSpawnPirates *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionBasePowerGenerated::cGcGenericSectionConditionBasePowerGenerated": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBasePowerGenerated *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionMissionMessage::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionMissionMessage *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionMissionMessageWarp::cGcGenericSectionConditionMissionMessageWarp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionMissionMessageWarp *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionMissionMessagePortal::cGcGenericSectionConditionMissionMessagePortal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionMissionMessagePortal *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionTotalWarpsNumber::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionTotalWarpsNumber *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionIsPlayerWanted::cGcGenericSectionConditionIsPlayerWanted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsPlayerWanted *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eEqualityEnum
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionIsPlayerWanted::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsPlayerWanted *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasFuel::cGcGenericSectionConditionHasFuel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasFuel *
            ctypes.c_uint32,  # eStatsType
            ctypes.c_int32,  # int
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasFuel::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasFuel *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionHasFuelForTakeoff::cGcGenericSectionConditionHasFuelForTakeoff": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasFuelForTakeoff *
            ctypes.c_ulonglong,  # cGcMissionConditionHasFuelForTakeoff *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasFuelForTakeoff::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasFuelForTakeoff *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcGenericSectionConditionInventorySlots::cGcGenericSectionConditionInventorySlots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInventorySlots *
            ctypes.c_ulonglong,  # cGcMissionConditionInventorySlots *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionFrigateCount::cGcGenericSectionConditionFrigateCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionFrigateCount *
            ctypes.c_ulonglong,  # cGcMissionConditionFrigateCount *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionExpeditionCount::cGcGenericSectionConditionExpeditionCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExpeditionCount *
            ctypes.c_ulonglong,  # cGcMissionConditionExpeditionCount *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionExpeditionContainsReward::cGcGenericSectionConditionExpeditionContainsReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExpeditionContainsReward *
            ctypes.c_ulonglong,  # cGcMissionConditionExpeditionContainsReward *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionDamagedFrigateAtHome::cGcGenericSectionConditionDamagedFrigateAtHome": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionDamagedFrigateAtHome *
            ctypes.c_ulonglong,  # cGcMissionConditionDamagedFrigateAtHome *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionExpeditionProgress::cGcGenericSectionConditionExpeditionProgress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExpeditionProgress *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionStatLevel::cGcGenericSectionConditionStatLevel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionStatLevel *
            ctypes.c_ulonglong,  # cGcMissionConditionStatLevel *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionPlanetStatLevel::cGcGenericSectionConditionPlanetStatLevel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPlanetStatLevel *
            ctypes.c_ulonglong,  # cGcMissionConditionPlanetStatLevel *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionStatDiff::cGcGenericSectionConditionStatDiff": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionStatDiff *
            ctypes.c_ulonglong,  # cGcMissionConditionStatDiff *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionStatChange::cGcGenericSectionConditionStatChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionStatChange *
            ctypes.c_ulonglong,  # cGcMissionConditionStatChange *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionIsAnomalyLoaded::cGcGenericSectionConditionIsAnomalyLoaded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsAnomalyLoaded *
            ctypes.c_uint32,  # eGalaxyStarAnomaly
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionIsLookingAtAnomalySpawn::cGcGenericSectionConditionIsLookingAtAnomalySpawn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsLookingAtAnomalySpawn *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionIsCurrentMission::cGcGenericSectionConditionIsCurrentMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsCurrentMission *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionMissionSelected::cGcGenericSectionConditionMissionSelected": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionMissionSelected *
            ctypes.c_ulonglong,  # cGcMissionConditionMissionSelected *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasGalacticFeature::cGcGenericSectionConditionHasGalacticFeature": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasGalacticFeature *
            ctypes.c_ulonglong,  # cGcMissionConditionHasGalacticFeature *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionGlobalMission::cGcGenericSectionConditionGlobalMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionGlobalMission *
            ctypes.c_ulonglong,  # cGcMissionConditionGlobalMission *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionFreighterBattle::cGcGenericSectionConditionFreighterBattle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionFreighterBattle *
            ctypes.c_ulonglong,  # cGcMissionConditionFreighterBattle *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionExtraSuitSlots::cGcGenericSectionConditionExtraSuitSlots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExtraSuitSlots *
            ctypes.c_ulonglong,  # cGcMissionConditionExtraSuitSlots *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionCanReceiveReward::cGcGenericSectionConditionCanReceiveReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCanReceiveReward *
            ctypes.c_ulonglong,  # cGcMissionConditionCanReceiveReward *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionPercentChance::cGcGenericSectionConditionPercentChance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPercentChance *
            ctypes.c_int32,  # int
            common.GcSeed,  # cTkSeed
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionAIShipCount::cGcGenericSectionConditionAIShipCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAIShipCount *
            ctypes.c_ulonglong,  # cGcMissionConditionAIShipCount *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionNumberOfShipsOwned::cGcGenericSectionConditionNumberOfShipsOwned": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNumberOfShipsOwned *
            ctypes.c_ulonglong,  # cGcMissionConditionNumberOfShipsOwned *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionNumAtlasStationsVisited::cGcGenericSectionConditionNumAtlasStationsVisited": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNumAtlasStationsVisited *
            ctypes.c_ulonglong,  # cGcMissionConditionNumAtlasStationsVisited *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionIsDepotDestroyed::cGcGenericSectionConditionIsDepotDestroyed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsDepotDestroyed *
            ctypes.c_ulonglong,  # cGcMissionConditionIsDepotDestroyed *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionAreDroneHivePartsDestroyed::cGcGenericSectionConditionAreDroneHivePartsDestroyed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAreDroneHivePartsDestroyed *
            ctypes.c_ulonglong,  # cGcMissionConditionAreDroneHivePartsDestroyed *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionIsScanEventOnCurrentPlanet::cGcGenericSectionConditionIsScanEventOnCurrentPlanet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsScanEventOnCurrentPlanet *
            ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventOnCurrentPlanet *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasEndpointForEvent::cGcGenericSectionConditionHasEndpointForEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasEndpointForEvent *
            ctypes.c_ulonglong,  # cGcMissionConditionHasEndpointForEvent *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionNearestBuilding::cGcGenericSectionConditionNearestBuilding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNearestBuilding *
            ctypes.c_ulonglong,  # cGcMissionConditionNearestBuilding *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionInUA::cGcGenericSectionConditionInUA": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInUA *
            ctypes.c_ulonglong,  # cGcMissionConditionInUA *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionEventRequiresRGB::cGcGenericSectionConditionEventRequiresRGB": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionEventRequiresRGB *
            ctypes.c_ulonglong,  # cGcMissionConditionEventRequiresRGB *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionPulseEncounterOverriden::cGcGenericSectionConditionPulseEncounterOverriden": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPulseEncounterOverriden *
            ctypes.c_ulonglong,  # cGcMissionConditionPulseEncounterOverriden *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionPulseEncounterActive::cGcGenericSectionConditionPulseEncounterActive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPulseEncounterActive *
            ctypes.c_ulonglong,  # cGcMissionConditionPulseEncounterActive *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionAbandonedFreighterExplored::cGcGenericSectionConditionAbandonedFreighterExplored": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAbandonedFreighterExplored *
            ctypes.c_ulonglong,  # cGcMissionConditionAbandonedFreighterExplored *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionAlienPodAggroed::cGcGenericSectionConditionAlienPodAggroed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAlienPodAggroed *
            ctypes.c_ulonglong,  # cGcMissionConditionAlienPodAggroed *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasEntitlement::cGcGenericSectionConditionHasEntitlement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasEntitlement *
            ctypes.c_ulonglong,  # cGcMissionConditionHasEntitlement *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionUnclaimedStageReward::cGcGenericSectionConditionUnclaimedStageReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionUnclaimedStageReward *
            ctypes.c_ulonglong,  # cGcMissionConditionUnclaimedStageReward *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionExpeditionNearlyOver::cGcGenericSectionConditionExpeditionNearlyOver": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExpeditionNearlyOver *
            ctypes.c_ulonglong,  # cGcMissionConditionExpeditionNearlyOver *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasSeasonalReward::cGcGenericSectionConditionHasSeasonalReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasSeasonalReward *
            ctypes.c_ulonglong,  # cGcMissionConditionHasSeasonalReward *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionSeasonRewardsRecipe::cGcGenericSectionConditionSeasonRewardsRecipe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSeasonRewardsRecipe *
            ctypes.c_ulonglong,  # cGcMissionConditionSeasonRewardsRecipe *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasTwitchReward::cGcGenericSectionConditionHasTwitchReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasTwitchReward *
            ctypes.c_ulonglong,  # cGcMissionConditionHasTwitchReward *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasPlatformReward::cGcGenericSectionConditionHasPlatformReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasPlatformReward *
            ctypes.c_ulonglong,  # cGcMissionConditionHasPlatformReward *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasShip::cGcGenericSectionConditionHasShip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasShip *
            ctypes.c_ulonglong,  # cGcMissionConditionHasShip *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasMultiTool::cGcGenericSectionConditionHasMultiTool": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasMultiTool *
            ctypes.c_ulonglong,  # cGcMissionConditionHasMultiTool *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionCreatureOwned::cGcGenericSectionConditionCreatureOwned": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureOwned *
            ctypes.c_ulonglong,  # cGcMissionConditionCreatureOwned *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionCreatureSlots::cGcGenericSectionConditionCreatureSlots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureSlots *
            ctypes.c_ulonglong,  # cGcMissionConditionCreatureSlots *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionCreatureTrust::cGcGenericSectionConditionCreatureTrust": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureTrust *
            ctypes.c_ulonglong,  # cGcMissionConditionCreatureTrust *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionCreatureSummoned::cGcGenericSectionConditionCreatureSummoned": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureSummoned *
            ctypes.c_ulonglong,  # cGcMissionConditionCreatureSummoned *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionCreatureReadyToHatch::cGcGenericSectionConditionCreatureReadyToHatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureReadyToHatch *
            ctypes.c_ulonglong,  # cGcMissionConditionCreatureReadyToHatch *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionCreatureReadyToLay::cGcGenericSectionConditionCreatureReadyToLay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCreatureReadyToLay *
            ctypes.c_ulonglong,  # cGcMissionConditionCreatureReadyToLay *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionSquadronPilotsOwned::cGcGenericSectionConditionSquadronPilotsOwned": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSquadronPilotsOwned *
            ctypes.c_ulonglong,  # cGcMissionConditionSquadronPilotsOwned *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionSquadronSlots::cGcGenericSectionConditionSquadronSlots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSquadronSlots *
            ctypes.c_ulonglong,  # cGcMissionConditionSquadronSlots *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionElevation::cGcGenericSectionConditionElevation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionElevation *
            ctypes.c_ulonglong,  # cGcMissionConditionElevation *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionUnderwaterDepth::cGcGenericSectionConditionUnderwaterDepth": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionUnderwaterDepth *
            ctypes.c_ulonglong,  # cGcMissionConditionUnderwaterDepth *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionCurrentSlope::cGcGenericSectionConditionCurrentSlope": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCurrentSlope *
            ctypes.c_ulonglong,  # cGcMissionConditionCurrentSlope *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionNearPole::cGcGenericSectionConditionNearPole": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNearPole *
            ctypes.c_ulonglong,  # cGcMissionConditionNearPole *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionIsMissionInProgress::cGcGenericSectionConditionIsMissionInProgress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsMissionInProgress *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionNearObject::cGcGenericSectionConditionNearObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNearObject *
            ctypes.c_ulonglong,  # cGcMissionConditionNearObject *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionNearScanEvent::cGcGenericSectionConditionNearScanEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNearScanEvent *
            ctypes.c_ulonglong,  # cGcMissionConditionNearScanEvent *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionUsingPortal::cGcGenericSectionConditionUsingPortal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionUsingPortal *
            ctypes.c_ulonglong,  # cGcMissionConditionUsingPortal *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionWeather::cGcGenericSectionConditionWeather": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWeather *
            ctypes.c_ulonglong,  # cGcMissionConditionWeather *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHazard::cGcGenericSectionConditionHazard": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHazard *
            ctypes.c_ulonglong,  # cGcMissionConditionHazard *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionInCombat::cGcGenericSectionConditionInCombat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInCombat *
            ctypes.c_ulonglong,  # cGcMissionConditionInCombat *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionNexusEnabled::cGcGenericSectionConditionNexusEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionNexusEnabled *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionInVR::cGcGenericSectionConditionInVR": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInVR *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionLocalScanActive::cGcGenericSectionConditionLocalScanActive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionLocalScanActive *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionWristMenuOpen::cGcGenericSectionConditionWristMenuOpen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionWristMenuOpen *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionPadActive::cGcGenericSectionConditionPadActive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionPadActive *
            ctypes.c_ulonglong,  # cGcMissionConditionPadActive *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionOnPlanetWithSandwormsOverriden::cGcGenericSectionConditionOnPlanetWithSandwormsOverriden": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionOnPlanetWithSandwormsOverriden *
            ctypes.c_ulonglong,  # cGcMissionConditionOnPlanetWithSandwormsOverriden *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionGunOut::cGcGenericSectionConditionGunOut": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionGunOut *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionIsGrabbed::cGcGenericSectionConditionIsGrabbed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsGrabbed *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionEggMachinePageOpen::cGcGenericSectionConditionEggMachinePageOpen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionEggMachinePageOpen *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasCreatureEggItem::cGcGenericSectionConditionHasCreatureEggItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasCreatureEggItem *
            ctypes.c_ulonglong,  # cGcMissionConditionHasCreatureEggItem *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionAllMilestonesComplete::cGcGenericSectionConditionAllMilestonesComplete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAllMilestonesComplete *
            ctypes.c_ulonglong,  # cGcMissionConditionAllMilestonesComplete *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionSeasonNumber::cGcGenericSectionConditionSeasonNumber": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSeasonNumber *
            ctypes.c_ulonglong,  # cGcMissionConditionSeasonNumber *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionConvertedFromSeason::cGcGenericSectionConditionConvertedFromSeason": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionConvertedFromSeason *
            ctypes.c_ulonglong,  # cGcMissionConditionConvertedFromSeason *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionInteractionIndexChanged::cGcGenericSectionConditionInteractionIndexChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInteractionIndexChanged *
            ctypes.c_ulonglong,  # cGcMissionConditionInteractionIndexChanged *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionInventoryOpen::cGcGenericSectionConditionInventoryOpen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionInventoryOpen *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionAbandonedOrEmptySystem::cGcGenericSectionConditionAbandonedOrEmptySystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAbandonedOrEmptySystem *
            ctypes.c_uint32,  # eMissionSystemType
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasActiveDetailMessage::cGcGenericSectionConditionHasActiveDetailMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasActiveDetailMessage *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasCommunicatorSignal::cGcGenericSectionConditionHasCommunicatorSignal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasCommunicatorSignal *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionBuildMenuOpen::cGcGenericSectionConditionBuildMenuOpen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionBuildMenuOpen *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasActiveStatsMessage::cGcGenericSectionConditionHasActiveStatsMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasActiveStatsMessage *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasSpareProcTech::cGcGenericSectionConditionHasSpareProcTech": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasSpareProcTech *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionIsSurveying::cGcGenericSectionConditionIsSurveying": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsSurveying *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eForHotspotType
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionOnMultiplayerMission::cGcGenericSectionConditionOnMultiplayerMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionOnMultiplayerMission *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionRidingCreature::cGcGenericSectionConditionRidingCreature": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionRidingCreature *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionFeedingCreatures::cGcGenericSectionConditionFeedingCreatures": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionFeedingCreatures *
            ctypes.c_ulonglong,  # cGcMissionConditionFeedingCreatures *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasGrabbableTarget::cGcGenericSectionConditionHasGrabbableTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasGrabbableTarget *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionIsFrigateFlybyActive::cGcGenericSectionConditionIsFrigateFlybyActive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsFrigateFlybyActive *
            ctypes.c_ulonglong,  # cGcMissionConditionIsFrigateFlybyActive *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionAimingTeleporter::cGcGenericSectionConditionAimingTeleporter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAimingTeleporter *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionExocraftMoving::cGcGenericSectionConditionExocraftMoving": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionExocraftMoving *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionIsAbandFreighterDoorOpen::cGcGenericSectionConditionIsAbandFreighterDoorOpen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsAbandFreighterDoorOpen *
            ctypes.c_uint32,  # eAbandonedFreighterDoorStatus
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionShipEngineStatus::cGcGenericSectionConditionShipEngineStatus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionShipEngineStatus *
            ctypes.c_uint32,  # eEngineStatus
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionUsingThirdPersonCamera::cGcGenericSectionConditionUsingThirdPersonCamera": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionUsingThirdPersonCamera *
            ctypes.c_uint32,  # eUsingCameraMode
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionCameraControlStealing::cGcGenericSectionConditionCameraControlStealing": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionCameraControlStealing *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionIsPlayerWeak::cGcGenericSectionConditionIsPlayerWeak": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionIsPlayerWeak *
            ctypes.c_uint32,  # eProgressType
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionDefaultItem::cGcGenericSectionConditionDefaultItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionDefaultItem *
            ctypes.c_ulonglong,  # cGcMissionConditionDefaultItem *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionSettlementStatLevel::cGcGenericSectionConditionSettlementStatLevel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSettlementStatLevel *
            ctypes.c_ulonglong,  # cGcMissionConditionSettlementStatLevel *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasSettlementProductPending::cGcGenericSectionConditionHasSettlementProductPending": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasSettlementProductPending *
            ctypes.c_ulonglong,  # cGcMissionConditionHasSettlementProductPending *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasAnySettlementBuildingInProgress::cGcGenericSectionConditionHasAnySettlementBuildingInProgress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasAnySettlementBuildingInProgress *
            ctypes.c_ulonglong,  # cGcMissionConditionHasAnySettlementBuildingInProgress *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionSettlementBuildingsAllStarted::cGcGenericSectionConditionSettlementBuildingsAllStarted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSettlementBuildingsAllStarted *
            ctypes.c_ulonglong,  # cGcMissionConditionSettlementBuildingsAllStarted *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHasProcMissionForFaction::cGcGenericSectionConditionHasProcMissionForFaction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHasProcMissionForFaction *
            ctypes.c_ulonglong,  # cGcMissionConditionHasProcMissionForFaction *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionHazardsEnabled::cGcGenericSectionConditionHazardsEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionHazardsEnabled *
            ctypes.c_ulonglong,  # cGcMissionConditionHazardsEnabled *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionLifeSupportEnabled::cGcGenericSectionConditionLifeSupportEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionLifeSupportEnabled *
            ctypes.c_ulonglong,  # cGcMissionConditionLifeSupportEnabled *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionItemCostsEnabled::cGcGenericSectionConditionItemCostsEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionItemCostsEnabled *
            ctypes.c_ulonglong,  # cGcMissionConditionItemCostsEnabled *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionStartWithAllPartsKnown::cGcGenericSectionConditionStartWithAllPartsKnown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionStartWithAllPartsKnown *
            ctypes.c_ulonglong,  # cGcMissionConditionStartWithAllPartsKnown *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionTutorialEnabled::cGcGenericSectionConditionTutorialEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionTutorialEnabled *
            ctypes.c_ulonglong,  # cGcMissionConditionTutorialEnabled *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionOnFootCombatEnabled::cGcGenericSectionConditionOnFootCombatEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionOnFootCombatEnabled *
            ctypes.c_ulonglong,  # cGcMissionConditionOnFootCombatEnabled *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionSpaceCombatEnabled::cGcGenericSectionConditionSpaceCombatEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionSpaceCombatEnabled *
            ctypes.c_ulonglong,  # cGcMissionConditionSpaceCombatEnabled *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionAutoPowerEnabled::cGcGenericSectionConditionAutoPowerEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionAutoPowerEnabled *
            ctypes.c_ulonglong,  # cGcMissionConditionAutoPowerEnabled *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcGenericSectionConditionVisorActive::cGcGenericSectionConditionVisorActive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericSectionConditionVisorActive *
            ctypes.c_ulonglong,  # cGcMissionConditionVisorActive *
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotificationSequence::GetCurrentSection": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNotifyGenericSection *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotificationSequence::GetCurrentSectionWithIcon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNotifyGenericSection *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
        ]
    ),
    "cGcNotificationSequence::AddStage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
            ctypes.c_ulonglong,  # cGcNotificationSequenceGroup *
        ]
    ),
    "cGcNotificationSequenceCommon::GetGlobalMissionState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcNotificationSequenceCommon::SetGlobalMissionState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # eGlobalMission
        ]
    ),
    "cGcNotificationSequenceGeneric::GetParticipantTypeFromPointer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ulonglong,  # cGcPlayerMissionParticipant *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetParticipantPointerFromType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerMissionParticipant *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_uint32,  # eParticipantType
        ]
    ),
    "cGcNotificationSequenceGeneric::GetNumberOfParticipants": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollect *
        ]
    ),
    "cGcNotificationSequenceGeneric::Restart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::DebugNguiMissionInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::IsPrimary": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetClass": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::IsLegacy": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::IsCritical": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::IsProcedural": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::DataAllowsRenounce": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::UseCommunityMissionForLog": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcNotificationSequenceGeneric::HasBeenHandedIn": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::IsForcingPageHint": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::IsForcingBuildMenuHint": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::IsTelemetryUpload": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGroup::GetTargetMissionSurveyId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGroup *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcNotificationSequenceGroup::GetPinnedMissionBuildTarget": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGroup *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcNotificationSequenceCollectMultiProducts::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollectMultiProducts *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceCollectMultiProducts::GetFormattedObjective": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollectMultiProducts *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceCollectMultiProducts::GetFormattedObjectiveTip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollectMultiProducts *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceCollectMultiProducts::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceBounty::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequencePirates::cGcNotificationSequencePirates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequencePirates *
            ctypes.c_ulonglong,  # cGcMissionSequencePirates *
        ]
    ),
    "cGcNotificationSequencePirates::Run": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceWaitForStat::cGcNotificationSequenceWaitForStat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStat *
            ctypes.c_char_p,  # const char *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # const eMissionMarkup
        ]
    ),
    "cGcNotificationSequenceWaitForStat::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStat *
        ]
    ),
    "cGcNotificationSequenceWaitForStat::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceWaitForStat::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStat *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForStat::GetStatusMessageMarkup": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStat *
        ]
    ),
    "cGcNotificationSequenceWaitForStat::GetFormattedObjective": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStat *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForStat::GetFormattedObjectiveTip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStat *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForStatSeasonal::cGcNotificationSequenceWaitForStatSeasonal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStatSeasonal *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNotificationSequenceWaitForStatSeasonal::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceWaitForStatSeasonal::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStatSeasonal *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForStatSeasonal::GetFormattedObjective": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStatSeasonal *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForStatSeasonal::GetFormattedObjectiveTip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStatSeasonal *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForStatMilestone::cGcNotificationSequenceWaitForStatMilestone": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStatMilestone *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_char,  # char
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequenceWaitForStatMilestone::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStatMilestone *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForStatMilestone::GetFormattedObjective": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStatMilestone *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForStatMilestone::GetFormattedObjectiveTip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStatMilestone *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForWarps::cGcNotificationSequenceWaitForWarps": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForWarps *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceWaitForDepots::cGcNotificationSequenceWaitForDepots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForDepots *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceDoMissionsForFaction::cGcNotificationSequenceDoMissionsForFaction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceDoMissionsForFaction *
            ctypes.c_ulonglong,  # cGcMissionSequenceDoMissionsForFaction *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcNotificationSequenceWaitForFactionStanding::cGcNotificationSequenceWaitForFactionStanding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForFactionStanding *
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForFactionStanding *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcNotificationSequenceWaitForShips::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "AkQueuedMsg::Sizeof_AddOutputCaptureMarker": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SetHazard *
        ]
    ),
    "cGcNotificationSequenceDiscoverOnPlanet::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceLearnWords::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceLearnWords::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceLearnWords *
        ]
    ),
    "cGcNotificationSequenceWaitForPortalWarp::cGcNotificationSequenceWaitForPortalWarp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForPortalWarp *
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForPortalWarp *
        ]
    ),
    "cGcNotificationSequenceTeleport::cGcNotificationSequenceTeleport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceTeleport *
            ctypes.c_ulonglong,  # cGcMissionSequenceTeleport *
        ]
    ),
    "cGcNotificationSequenceFreighterEngage::cGcNotificationSequenceFreighterEngage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceFreighterEngage *
            ctypes.c_ulonglong,  # cGcMissionSequenceFreighterEngage *
        ]
    ),
    "cGcNotificationSequenceFreighterEngage::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceFreighterDefend::cGcNotificationSequenceFreighterDefend": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceFreighterDefend *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNotificationSequenceFreighterDefend::Run": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceWaitForFreighterPods::cGcNotificationSequenceWaitForFreighterPods": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForFreighterPods *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequenceStartSummonAnomaly::cGcNotificationSequenceStartSummonAnomaly": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStartSummonAnomaly *
            ctypes.c_ulonglong,  # cGcMissionSequenceStartSummonAnomaly *
        ]
    ),
    "cGcNotificationSequenceStartSummonAnomaly::DefaultCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceFinishSummonAnomaly::cGcNotificationSequenceFinishSummonAnomaly": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceFinishSummonAnomaly *
            ctypes.c_ulonglong,  # cGcMissionSequenceFinishSummonAnomaly *
        ]
    ),
    "cGcNotificationSequenceFinishSummonAnomaly::DefaultCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequeneSummonNexus::cGcNotificationSequeneSummonNexus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequeneSummonNexus *
            ctypes.c_ulonglong,  # cGcMissionSequenceSummonNexus *
        ]
    ),
    "cGcNotificationSequenceSuppressMarkers::cGcNotificationSequenceSuppressMarkers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceSuppressMarkers *
            ctypes.c_ulonglong,  # cGcMissionSequenceSuppressMarkers *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetCurrentCategory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::BlocksPinning": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::CheckOptionCondition": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetMissionTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            common.GcSeed,  # cTkSeed
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetMissionSubtitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            common.GcSeed,  # cTkSeed
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetMissionDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            common.GcSeed,  # cTkSeed
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceGeneric::RefreshLogInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetMissionObjective": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            common.GcSeed,  # cTkSeed
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetPirateMissionTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNotificationSequenceGeneric::HasEnoughFactionRank": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcNotificationSequenceGeneric::cGcNotificationSequenceGeneric": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcNotificationSequenceGeneric::CreateAndAddAutoStartMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::~cGcNotificationSequenceGeneric": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ulonglong,  # const cGcPlayerMissionProgress *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNotificationSequenceGeneric::UpgradeProgress": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcGenericMissionStage> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcNotificationSequenceGeneric::Save": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ulonglong,  # cGcPlayerMissionProgress *
        ]
    ),
    "cGcNotificationSequenceGeneric::Update": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::CanBegin": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetSubtitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetMissionIcon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkTextureResource *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetMissionIconSelected": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkTextureResource *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetMissionIconNotSelected": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkTextureResource *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetMissionDataTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetText": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetCurrentIcon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSlotIcon *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetCurrentColour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ulonglong,  # cTkColour *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetIconAtStage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNotificationSequenceGeneric::GetCurrentFrontendPageHint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetCurrentInventoryHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetCurrentFrontendPageLocID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ulonglong,  # TkID<256> *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetCurrentBuildMenuHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcNotificationSequenceGeneric::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
        ]
    ),
    "cGcNotificationSequenceGeneric::GetMultiplayerMissionType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcNotificationSequenceGeneric::CountTotalMissionSequences": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcGenericMissionStage> *
        ]
    ),
    "cGcNotificationSequenceGeneric::CreateAndAddMissionSequences": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcGenericMissionStage> *
            ctypes.c_ulonglong,  # cGcNotificationSequenceGroup *
        ]
    ),
    "cGcNotificationSequenceGeneric::CreateAndAddConditions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequence *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkClassPointer> *
            ctypes.c_ulonglong,  # std::vector<cGcGenericSectionCondition *,TkSTLAllocatorShim<cGcGenericSectionCondition *,8,-1> > *
        ]
    ),
    "cGcNotificationSequenceGeneric::WriteItemText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequenceGeneric::CreateAndAddConsequences": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGeneric *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkClassPointer> *
            ctypes.c_ulonglong,  # std::vector<cGcGenericSectionConsequence *,TkSTLAllocatorShim<cGcGenericSectionConsequence *,8,-1> > *
        ]
    ),
    "cGcNotificationSequenceGroupCode::cGcNotificationSequenceGroupCode": {
        "cGcNotificationSequenceGroupCode *, const char *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNotificationSequenceGroupCode *
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cGcNotificationSequenceGroupCode *, const char *, const TkID<128> *, const TkID<128> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNotificationSequenceGroupCode *
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_char_p,  # const TkID<128> *
            ]
        ),
    },
    "cGcNotificationSequenceGroupCode::GetTargetMissionSurveyId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGroupCode *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcNotificationSequenceGroupCode::GetPinnedMissionBuildTarget": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGroupCode *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcNotificationSequenceCode::Update": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCode *
        ]
    ),
    "cGcNotificationSequenceCode::GetMissionIconSelected": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkTextureResource *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCode *
        ]
    ),
    "cGcNotificationSequenceCode::IsForcingBuildMenuHint": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCode *
        ]
    ),
    "cGcNotificationSequenceUpgradeTech::GetSubtitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceUpgradeTech *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcNotificationSequenceGroupCode::GetObjective": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGroupCode *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceGroupCode::GetObjectiveTip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceGroupCode *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceUpgradeTech::IsAppropriate": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceUpgradeTech *
        ]
    ),
    "cGcNotificationSequenceUpgradeTech::GetTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceUpgradeTech *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceUpgradeTech::GetMissionDataTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceUpgradeTech *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceUpgradeTech::GetDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceUpgradeTech *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceUpgradeTech::GetMissionIcon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTechnology *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceUpgradeTech *
        ]
    ),
    "cGcNotificationSequenceUpgradeTech::cGcNotificationSequenceUpgradeTech": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceUpgradeTech *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcNotificationSequenceUpgradeTech::Update": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceUpgradeTech *
        ]
    ),
    "cGcNotificationSequenceCraftDevice::GetMissionDataTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCraftDevice *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceCraftDevice::GetSubtitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCraftDevice *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcNotificationSequenceCraftDevice::GetTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCraftDevice *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceCraftDevice::GetDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCraftDevice *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceCraftDevice::GetMissionIcon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureSwarm *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCraftDevice *
        ]
    ),
    "cGcNotificationSequenceCraftDevice::cGcNotificationSequenceCraftDevice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCraftDevice *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcNotificationSequenceCraftDevice::Update": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCraftDevice *
        ]
    ),
    "cGcNotificationSequenceCode::~cGcNotificationSequenceCode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCode *
        ]
    ),
    "cGcNotificationSequenceCode::GetCurrentBuildMenuHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCode *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcNotificationSequenceCode::GenerateDetailMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCode *
            ctypes.c_ulonglong,  # cGcMissionSequenceDetailMessage *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationTextHelper::SubstituteFlightTipIntoString<cTkFixedString<128,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationTextHelper::SubstituteFlightTipIntoString<cTkFixedString<256,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcNotificationTextHelper::SubstituteFlightTipIntoString<cTkFixedString<512,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationTextHelper::SubstituteFlightTipIntoString<cTkFixedString<1024,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceGeneric::SubstitutePirateDetailsIntoString<cTkFixedString<128,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceGeneric::SubstitutePirateDetailsIntoString<cTkFixedString<512,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceGeneric::SubstitutePirateDetailsIntoString<cTkFixedString<1024,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceBounty::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceBounty *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceBounty::RunInternal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceBounty *
        ]
    ),
    "cGcNotificationSequenceCollect::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceCollect::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollect *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceCollect::GetFormattedObjective": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollect *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceCollect::GetFormattedObjectiveTip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollect *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceCollect::cGcNotificationSequenceCollect": {
        "cGcNotificationSequenceCollect *, cGcMissionSequenceCollectSubstance *, const TkID<128> *, const cTkSeed *, const int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNotificationSequenceCollect *
                ctypes.c_ulonglong,  # cGcMissionSequenceCollectSubstance *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ulonglong,  # const cTkSeed *
                ctypes.c_int32,  # const int
            ]
        ),
        "cGcNotificationSequenceCollect *, cGcMissionSequenceCollectProduct *, const TkID<128> *, const cTkSeed *, const int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNotificationSequenceCollect *
                ctypes.c_ulonglong,  # cGcMissionSequenceCollectProduct *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ulonglong,  # const cTkSeed *
                ctypes.c_int32,  # const int
            ]
        ),
    },
    "cGcNotificationSequenceCollect::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollect *
        ]
    ),
    "cGcNotificationSequenceCollect::RunInternal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollect *
        ]
    ),
    "cGcNotificationSequenceCollect::GetLocalAmount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollect *
        ]
    ),
    "cGcNotificationSequenceCollect::GetCurrent": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollect *
        ]
    ),
    "cGcNotificationSequenceCollect::CalculateDependantRequirements": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollect *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcNotificationSequenceCollect::OverridesMissionSurveyText": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollect *
        ]
    ),
    "cGcNotificationSequenceCollectMultiProducts::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollectMultiProducts *
        ]
    ),
    "cGcNotificationSequenceCollectMultiProducts::RunInternal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollectMultiProducts *
        ]
    ),
    "cGcNotificationSequenceCollectMultiProducts::GetLocalAmount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollectMultiProducts *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNotificationSequenceCollectMultiProducts::GetCurrent": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollectMultiProducts *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNotificationSequenceCommunicator::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceCommunicatorOnTakeOff::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceCommunicator::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCommunicator *
        ]
    ),
    "cGcNotificationSequenceCommunicator::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCommunicator *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceCommunicator::RunInternal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCommunicator *
        ]
    ),
    "cGcNotificationSequenceCommunicatorOnTakeOff::RunInternal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCommunicatorOnTakeOff *
        ]
    ),
    "cGcNotificationSequenceWaitForConditions::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceWaitForConditions::FormatsObjectiveText": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForConditions *
        ]
    ),
    "cGcNotificationSequenceWaitForConditions::GetFormattedObjective": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForConditions *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cGcNotificationSequenceWaitForConditions::GetFormattedObjectiveTip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForConditions *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceConditionalReward::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceDiscover::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceDiscover *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceDiscover::RunInternal": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceDiscover *
        ]
    ),
    "cGcNotificationSequenceDiscover::GetStatValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceDiscover *
        ]
    ),
    "cGcNotificationSequenceDiscover::GetOtherPlayersCounts": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceDiscover *
        ]
    ),
    "cGcNotificationSequenceDiscoverOnPlanet::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceDiscoverOnPlanet *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceDiscoverOnPlanet::DiscoveryDataSubmitted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceDiscoverOnPlanet *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcNotificationSequenceDiscoverOnPlanet::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceDiscoverOnPlanet *
        ]
    ),
    "cGcNotificationSequenceDiscoverOnPlanet::RunInternal": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceDiscoverOnPlanet *
        ]
    ),
    "cGcNotificationSequenceFreighterEngage::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceFreighterEngage *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceFreighterEngage::RunInternal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceFreighterEngage *
        ]
    ),
    "cGcNotificationSequenceWaitForFreighterPods::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForFreighterPods *
        ]
    ),
    "cGcNotificationSequenceFeed::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceKill *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceKill::GetStatusMessageMarkup": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceKill *
        ]
    ),
    "cGcNotificationSequenceKill::EncouragesFightingSentinels": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceKill *
        ]
    ),
    "cGcNotificationSequenceKill::GetStatValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceKill *
        ]
    ),
    "cGcNotificationSequenceKill::GetOtherPlayersCounts": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceKill *
        ]
    ),
    "CAkMusicSwitchCntr::NodeCategory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceFeed *
        ]
    ),
    "cGcNotificationSequenceFeed::GetStatValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceFeed *
        ]
    ),
    "cGcNotificationSequenceFeed::GetOtherPlayersCounts": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceFeed *
        ]
    ),
    "cGcNotificationSequenceLearnWords::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceLearnWords *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceLearnWords::GetGlobalAmount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceLearnWords *
        ]
    ),
    "cGcNotificationSequenceMultiplayerMissionPlayerRoundUp::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceMultiplayerMissionPlayerRoundUp *
        ]
    ),
    "cGcNotificationSequenceMultiplayerMissionPlayerRoundUp::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceMultiplayerMissionPlayerRoundUp::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceMultiplayerMissionPlayerRoundUp *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceStartSummonAnomaly::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStartSummonAnomaly *
        ]
    ),
    "cGcNotificationSequenceStartSummonAnomaly::GetValidSpawn": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStartSummonAnomaly *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcNotificationSequenceFinishSummonAnomaly::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceFinishSummonAnomaly *
        ]
    ),
    "cGcNotificationSequeneSummonNexus::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceSuppressMarkers::Run": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcClothComponent::RequestTeleport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothComponent *
        ]
    ),
    "cGcClothComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcSpringAttachmentComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcNotificationSequenceTeleport::Run": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcNotificationSequenceWaitForConditions::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForConditions *
        ]
    ),
    "cGcNotificationSequenceWaitForConditions::AllowMissionRestart": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForConditions *
        ]
    ),
    "cGcNotificationSequenceWaitForConditions::CheckConditions": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForConditions *
        ]
    ),
    "cGcNotificationSequenceWaitForConditions::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForConditions *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForConditions::BlockPirateMessage": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForConditions *
        ]
    ),
    "cGcNotificationSequenceWaitForConditions::OverridesMissionSurveyText": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForConditions *
        ]
    ),
    "cGcNotificationSequenceWaitForConditions::GetBuildMenuHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForConditions *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcNotificationSequenceWaitForConditions::GetStatusMessageMarkup": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForConditions *
        ]
    ),
    "cGcNotificationSequenceWaitForConditions::EncouragesFightingSentinels": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForConditions *
        ]
    ),
    "cGcNotificationSequenceConditionalReward::CheckConditions": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceConditionalReward *
        ]
    ),
    "cGcNotificationSequenceWaitForStat::RunInternal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStat *
        ]
    ),
    "cGcNotificationSequenceWaitForStat::GetStatValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStat *
        ]
    ),
    "cGcNotificationSequenceWaitForStat::GetOtherPlayersCounts": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStat *
        ]
    ),
    "cGcNotificationSequenceWaitForStat::SetChangeValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStat *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNotificationSequenceWaitForStatSeasonal::RunInternal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStatSeasonal *
        ]
    ),
    "cGcNotificationSequenceWaitForStatMilestone::RunInternal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStatMilestone *
        ]
    ),
    "cGcNotificationSequenceWaitForStatMilestone::GetRequiredLevel": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStatMilestone *
        ]
    ),
    "cGcNotificationSequenceWaitForMissionSelected::Wait": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcNotificationSequenceDoMissionsForFaction::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceDoMissionsForFaction *
        ]
    ),
    "cGcNotificationSequenceDoMissionsForFaction::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForFactionStanding *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForFactionStanding::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForFactionStanding *
        ]
    ),
    "cGcNotificationSequenceWaitForShips::GetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForShips *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForShips::GetCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForShips *
        ]
    ),
    "cGcNotificationSequenceWaitForPortalWarp::Run": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceStage *
        ]
    ),
    "cGcInWorldButton::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldButton *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cGcInWorldButton::RenderCursor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldButton *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cGcInWorldScreen::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldScreen *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcInWorldScreen::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldScreen *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcInWorldScreen::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldScreen *
        ]
    ),
    "cGcInWorldScreen::SetSize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldScreen *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcInWorldScreen::SetOffsetAndScale": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldScreen *
            ctypes.c_double,  # double
            ctypes.c_double,  # double
            ctypes.c_double,  # double
        ]
    ),
    "cGcInWorldScreen::GetMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldScreen *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcInWorldScreen::GetCursorWorldTransform": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldScreen *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcInWorldScreen::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldScreen *
            ctypes.c_float,  # float
        ]
    ),
    "cGcInWorldScreen::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldScreen *
        ]
    ),
    "cGcInWorldScreen::GetScreenPixelSize": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldScreen *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "cGcInWorldScreen::GetScreenWorldSize": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldScreen *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "GcNGuiFx::LoadingDistractionFullscreenNoPhase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkIntTuple2<int,0> *
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
        ]
    ),
    "cGcFrontendModelRenderer::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const std::_Value_init_tag *
        ]
    ),
    "cGcFrontendModelRenderer::PlayAnim": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendModelRenderer::GetDiscoveryThumbnailNvgTextureIndex": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::cTkNGuiFontHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::cTkNGuiFontHandleID> *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcFrontendModelRenderer::GetMainRendererNvgTextureIndex": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::cTkNGuiFontHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::cTkNGuiFontHandleID> *
        ]
    ),
    "cGcFrontendModelRenderer::HasMainDiscoveryModelRendered": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
        ]
    ),
    "cGcFrontendModelRenderer::IsDiscoveryThumbnailRendererRefreshed": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcFrontendModelRenderer::cDiscoveryLoadRequest::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer::cDiscoveryLoadRequest *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # cTkModelResourceRenderer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFrontendModelRenderer::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcFrontendModelRenderer::cDiscoveryLoadRequest::CreateDiscoveryLoadTask": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcProceduralTextureManager::TaskContext *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer::cDiscoveryLoadRequest *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_uint32,  # eBiome
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcFrontendModelRenderer::ClearDiscoveryRecords": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
        ]
    ),
    "cGcFrontendModelRenderer::HasDiscovery": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcFrontendModelRenderer::EnqueueRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrontendModelRenderer::SetRelativeToScreenCenterParallaxOffsets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcFrontendModelRenderer::RequestRenderDiscoveryAs3dModel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcFrontendModelRenderer::UpdateVehicleNodeVisibility": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcFrontendModelRenderer::UpdateLogic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcFrontendModelRenderer::HandleInput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrontendModelRenderer *
            ctypes.c_float,  # const float
        ]
    ),
    "cTkNGuiEditor::~cTkNGuiEditor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
        ]
    ),
    "cGcNGuiElement::CommitIdToMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNGuiElement::CommonRenderPositioning": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
        ]
    ),
    "cGcNGuiElement::RenderTreeControls": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
        ]
    ),
    "cGcNGuiElement::IsParentHiddenRecursive": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
            ctypes.c_ulonglong,  # const cGcNGuiLayer *
        ]
    ),
    "cGcNGuiElement::SelectInEditor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiElement::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcNGuiElement::GetPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_uint32,  # cGcNGuiElement::PositionType
        ]
    ),
    "cGcNGuiElement::ForceAspect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # eNGuiSizeType *
        ]
    ),
    "cGcNGuiElement::ApplyVROverrides_Layout_Position": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "cGcNGuiElement::ApplyAccessibleOverrides_Layout_Position": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "cGcNGuiElement::IsMouseInside": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcNGuiLayer::GetSceneTreeText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNGuiLayer::RenderAdditionalOverlayIcons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cGcNGuiLayer::SetNeedsUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcNGuiLayer::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcNGuiLayer::EditElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcNGuiLayer::TogglePinChildren": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcNGuiLayer::LoadFromMetadata": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiElement::EditLayout": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
            ctypes.c_ulonglong,  # cGcNGuiLayoutData *
        ]
    ),
    "cGcNGuiElement::ElementScale": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_uint32,  # cGcNGuiElement::ElementClass
        ]
    ),
    "cGcNGuiElement::EditElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
        ]
    ),
    "cGcNGuiElement::EditGraphicPreset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
            ctypes.c_ulonglong,  # cGcNGuiPresetGraphic *
        ]
    ),
    "cGcNGuiElement::EditPresets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
            ctypes.c_ulonglong,  # cGcNGuiPreset *
        ]
    ),
    "cGcNGuiElement::RenderUIEditorToolbar": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
            ctypes.c_ulonglong,  # cGcNGuiPreset *
        ]
    ),
    "cGcNGuiElement::SetImage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyle *
        ]
    ),
    "cGcNGuiElement::Copy": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiText *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElement *
        ]
    ),
    "cGcNGuiTextSpecial::SetNeedsUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiTextSpecial *
        ]
    ),
    "cGcNGuiTextSpecial::SetText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiTextSpecial *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNGuiTextSpecial::Convert": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiTextSpecial *
        ]
    ),
    "cGcNGuiTextSpecial::EditElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiTextSpecial *
        ]
    ),
    "CommonFontHeightRescalingToFit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGui *
            ctypes.c_ulonglong,  # const cGcNGuiElementData *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # cTkNGuiTextStyle *
        ]
    ),
    "cGcNGuiTextSpecial::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiTextSpecial *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcNGuiText::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiText *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcNGuiText::ApplyVROverrides_Text": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiText *
            ctypes.c_ulonglong,  # cTkNGuiTextStyle *
        ]
    ),
    "cGcNGuiText::ApplyAccessibleOverrides_Text": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiText *
            ctypes.c_ulonglong,  # cTkNGuiTextStyle *
        ]
    ),
    "cGcNGuiText::Convert": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiText *
        ]
    ),
    "cGcNGuiText::GetSceneTreeText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiText *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNGuiText::EditElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiText *
        ]
    ),
    "cGcNGuiGraphic::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGraphic *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcNGuiGraphic::EditElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGraphic *
        ]
    ),
    "cGcNGuiGraphic::GetSceneTreeText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGraphic *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNGuiLayer::MoveElementUndoable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkNGuiUndoable *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiElement *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNGuiLayer::SetData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # cGcNGuiLayerData *
        ]
    ),
    "cGcNGuiSpacing::GetSceneTreeText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpacing *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcUndoableNGuiElementOperation::CreateDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcNGuiElement *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiGraphicData>::Do": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiGraphicData> *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiGraphicData>::Undo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiGraphicData> *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiGraphicData>::GetData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiGraphicData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiGraphicData> *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiGraphicData>::Compare": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiGraphicData> *
            ctypes.c_ulonglong,  # const cGcNGuiGraphicData *
            ctypes.c_ulonglong,  # const cGcNGuiGraphicData *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiGraphicData>::End": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiGraphicData> *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiTextData>::Do": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiTextData> *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiTextData>::Undo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiTextData> *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiTextData>::GetData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiTextData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiTextData> *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiTextData>::Compare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiTextData> *
            ctypes.c_ulonglong,  # const cGcNGuiTextData *
            ctypes.c_ulonglong,  # const cGcNGuiTextData *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiTextData>::End": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiTextData> *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiLayoutData>::Do": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiLayoutData> *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiLayoutData>::Undo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiLayoutData> *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiLayoutData>::GetData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayoutData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiLayoutData> *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiLayoutData>::Compare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiLayoutData> *
            ctypes.c_ulonglong,  # const cGcNGuiLayoutData *
            ctypes.c_ulonglong,  # const cGcNGuiLayoutData *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiLayoutData>::End": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiLayoutData> *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiLayoutData>::cTkNGuiTypeUndoable<cGcNGuiLayoutData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiLayoutData> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cGcNGuiLayoutData *
        ]
    ),
    "cGcNGuiGraphicData::cGcNGuiGraphicData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGraphicData *
            ctypes.c_ulonglong,  # const cGcNGuiGraphicData *
        ]
    ),
    "cGcNGuiTextData::cGcNGuiTextData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiTextData *
            ctypes.c_ulonglong,  # const cGcNGuiTextData *
        ]
    ),
    "cTkDynamicArray<cGcVROverride_Text>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWFCTerrainConstraint> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNotificationSequenceCollect::FormatText<cTkFixedString<512,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollect *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequenceCollect::FormatText<cTkFixedString<128,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollect *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequenceCollect::FormatText<cTkFixedString<1024,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollect *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequenceCollectMultiProducts::FormatText<cTkFixedString<512,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollectMultiProducts *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceCollectMultiProducts::FormatText<cTkFixedString<128,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollectMultiProducts *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceCollectMultiProducts::FormatText<cTkFixedString<1024,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceCollectMultiProducts *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForStat::FormatText<cTkFixedString<512,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStat *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForStat::FormatText<cTkFixedString<128,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStat *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForStat::FormatText<cTkFixedString<1024,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStat *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForStatSeasonal::FormatText<cTkFixedString<512,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStatSeasonal *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequenceWaitForStatSeasonal::FormatText<cTkFixedString<128,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStatSeasonal *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequenceWaitForStatSeasonal::FormatText<cTkFixedString<1024,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStatSeasonal *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNotificationSequenceWaitForStatMilestone::FormatText<cTkFixedString<512,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStatMilestone *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForStatMilestone::FormatText<cTkFixedString<128,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStatMilestone *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNotificationSequenceWaitForStatMilestone::FormatText<cTkFixedString<1024,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNotificationSequenceWaitForStatMilestone *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
        ]
    ),
    "cTkAttachment::FindFirstAttachmentWithComponent<cTkAnimationComponent>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachment *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkNGuiEditor::DoComboBox<eAltMode>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eAltMode *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNGuiGame::IsActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGame *
        ]
    ),
    "cGcNGuiGame::GetFontSpacing": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGame *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNGuiGame::GetFontID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGame *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNGuiLayer::ConvertText": {
        "cGcNGuiLayer *, cGcNGuiText *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcNGuiText *
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_ulonglong,  # cGcNGuiText *
            ]
        ),
        "cGcNGuiLayer *, cGcNGuiTextSpecial *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcNGuiText *
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_ulonglong,  # cGcNGuiTextSpecial *
            ]
        ),
    },
    "cTkNGuiTextStyle::cTkNGuiTextStyle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextStyle *
            ctypes.c_ulonglong,  # cTkNGuiTextStyle *
        ]
    ),
    "cGcNGuiLayer::AddAfterElement": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const cGcNGuiElement *
            ctypes.c_ulonglong,  # cGcNGuiElement *
        ]
    ),
    "cGcNGuiSpacing::SetData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpacing *
            ctypes.c_ulonglong,  # cGcNGuiSpacingData *
        ]
    ),
    "cGcNGuiGraphic::SetData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGraphic *
            ctypes.c_ulonglong,  # cGcNGuiGraphicData *
        ]
    ),
    "cGcNGuiText::SetData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiText *
            ctypes.c_ulonglong,  # cGcNGuiTextData *
        ]
    ),
    "cGcNGuiLayer::GatherExternals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # std::vector<cTkFixedString<128,char>,TkSTLAllocatorShim<cTkFixedString<128,char>,1,-1> > *
        ]
    ),
    "cGcNGuiText::SetFromPreset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiText *
            ctypes.c_ulonglong,  # cGcNGuiPresetText *
        ]
    ),
    "cTkNGuiEditor::DoComboBox<uchar>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eCurve *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNGuiLayoutData::cGcNGuiLayoutData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayoutData *
            ctypes.c_ulonglong,  # const cGcNGuiLayoutData *
        ]
    ),
    "cTkNGuiGraphicStyle::cTkNGuiGraphicStyle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyle *
            ctypes.c_ulonglong,  # const cTkNGuiGraphicStyle *
        ]
    ),
    "cGcNGuiLayerData::cGcNGuiLayerData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayerData *
            ctypes.c_ulonglong,  # const cGcNGuiLayerData *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiLayerData>::Undo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiLayerData> *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiLayerData>::Do": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiLayerData> *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiLayerData>::Compare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiLayerData> *
            ctypes.c_ulonglong,  # const cGcNGuiLayerData *
            ctypes.c_ulonglong,  # const cGcNGuiLayerData *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiLayerData>::End": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiLayerData> *
        ]
    ),
    "cTkNGuiTypeUndoable<cGcNGuiLayerData>::GetData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayerData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTypeUndoable<cGcNGuiLayerData> *
        ]
    ),
    "cGcGuiTypeUndoableLayerData::Compare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGuiTypeUndoableLayerData *
            ctypes.c_ulonglong,  # const cGcNGuiLayerData *
            ctypes.c_ulonglong,  # const cGcNGuiLayerData *
        ]
    ),
    "cGcNGuiGame::DoPanelBegin": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGame *
            ctypes.c_uint32,  # eNGuiSizeType
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiStyles::GetGraphicMargins": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcUndoableNGuiElementOperation::cGcUndoableNGuiElementOperation": {
        "cGcUndoableNGuiElementOperation *, cGcNGuiLayer *, cGcNGuiElement *, int, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcUndoableNGuiElementOperation *
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_ulonglong,  # cGcNGuiElement *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
            ]
        ),
        "cGcUndoableNGuiElementOperation *, cGcNGuiLayer *, cGcNGuiElement *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcUndoableNGuiElementOperation *
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_ulonglong,  # cGcNGuiElement *
                ctypes.c_int32,  # int
            ]
        ),
        "cGcUndoableNGuiElementOperation *, cGcNGuiLayer *, cGcNGuiLayer *, cGcNGuiElement *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcUndoableNGuiElementOperation *
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_ulonglong,  # cGcNGuiLayer *
                ctypes.c_ulonglong,  # cGcNGuiElement *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcNGuiSpacing::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpacing *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcNGuiSpacing::EditElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpacing *
        ]
    ),
    "cGcNGuiSpacing::AttachMetadata": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpacing *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiSpacing::DetachMetadata": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiText *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiSpacing::GetSceneTreeIcon": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpacing *
        ]
    ),
    "cGcNGuiSpacing::cGcNGuiSpacing": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpacing *
        ]
    ),
    "cTkNGuiEditor::BeginUndo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_ulonglong,  # cTkNGuiUndoable *
        ]
    ),
    "cTkNGuiEditor::EndUndo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
        ]
    ),
    "cGcUndoableNGuiElementOperation::Undo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUndoableNGuiElementOperation *
        ]
    ),
    "cGcUndoableNGuiElementOperation::Do": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUndoableNGuiElementOperation *
        ]
    ),
    "cTkNGuiUndoable::Icon": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiUndoable *
        ]
    ),
    "cTkNGui::cTkNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkNGui::~cTkNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
        ]
    ),
    "cTkNGuiEditor::GetFontID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNGuiEditor::IsActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
        ]
    ),
    "cTkNGuiEditor::cTkNGuiEditor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcNGuiGraphic::~cGcNGuiGraphic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGraphic *
        ]
    ),
    "cEgModelNode::GetLodMeshType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
        ]
    ),
    "cGcNGuiGraphic::SelectableInEditor": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGraphic *
        ]
    ),
    "cGcNGuiGraphic::AttachMetadata": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGraphic *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiGraphic::GetSceneTreeIcon": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGraphic *
        ]
    ),
    "cGcNGuiGraphic::cGcNGuiGraphic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGraphic *
        ]
    ),
    "cGcNGuiTextSpecial::~cGcNGuiTextSpecial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiTextSpecial *
        ]
    ),
    "cGcNGuiTextSpecial::GetSceneTreeIcon": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiTextSpecial *
        ]
    ),
    "cGcNGuiStyledString::StyledStringLayout::StyledStringLayout": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyledString::StyledStringLayout *
        ]
    ),
    "cGcNGuiTextSpecial::cGcNGuiTextSpecial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiTextSpecial *
        ]
    ),
    "cGcNGuiTextData::~cGcNGuiTextData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiTextData *
        ]
    ),
    "cGcNGuiText::~cGcNGuiText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiText *
        ]
    ),
    "cGcNGuiText::AttachMetadata": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiText *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiText::GetSceneTreeIcon": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiText *
        ]
    ),
    "cGcNGuiText::cGcNGuiText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiText *
        ]
    ),
    "cTkModelResourceRenderer::SetModelNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelResourceRenderer *
            ctypes.c_uint32,  # const TkHandle
            ctypes.c_ulonglong,  # const cTkAABB *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetGenerator::GetSeedForColourGeneration": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSeed *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSeed *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcGameCamera::SetShakeStrength": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPoolHandle *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCameraManager::SetShakeStrength": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_ulonglong,  # const cTkClassPoolHandle *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerCommunicator::Open": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicator *
        ]
    ),
    "NGuiRenderEmitterFloatProperty": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkEmitterFloatProperty *
        ]
    ),
    "NGuiRenderFloatRange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkFloatRange *
        ]
    ),
    "cGcNGuiManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNGuiManager::Render2D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiManager *
        ]
    ),
    "cGcNGuiManager::BeginFrame": {
        "cGcNGuiManager *, cTkNGuiInput *, bool, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiManager *
                ctypes.c_ulonglong,  # cTkNGuiInput *
                ctypes.c_ubyte,  # bool
                ctypes.c_float,  # float
            ]
        ),
        "cGcNGuiManager *, cTkNGuiInput *, const cTkVector2 *, bool, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiManager *
                ctypes.c_ulonglong,  # cTkNGuiInput *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ubyte,  # bool
                ctypes.c_float,  # float
            ]
        ),
    },
    "cGcNGuiManager::EndFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiManager *
            ctypes.c_ulonglong,  # cTkNGuiInput *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiManager::CentreCursor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiInput *
        ]
    ),
    "cGcNGuiManager::DoColourUtils": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiManager *
        ]
    ),
    "cGcNGuiManager::BeginInput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiManager *
            ctypes.c_ulonglong,  # cTkNGuiInput *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cGcNGuiManager::EndInput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiManager *
            ctypes.c_ulonglong,  # cTkNGuiInput *
        ]
    ),
    "cGcNGui::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGui *
        ]
    ),
    "cGcNGui::RenderWithHandControlCursor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGui *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_uint32,  # eInputDeviceTransform
        ]
    ),
    "cGcNGui::Render3d": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGui *
            ctypes.c_uint64,  # cTkClassPoolHandle
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['void'], 'pointer_depth': 1}
        ]
    ),
    "GetUIPlatformForFile": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "cGcNGuiStyledString::GenerateLayout": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiStyledString::StyledStringLayout *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyledString::StyledStringLayout *
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_float,  # const float
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const cTkNGuiTextStyle *
            ctypes.c_uint32,  # const ePunctuationDelays
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcNGuiStyledString::StyledStringLayout::DoText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyledString::StyledStringLayout *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNGuiStyledString::StyledStringLayout::TryAndAddToLayout": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyledString::StyledStringLayout *
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcNGuiStyledString::StyledStringLayout::DoImage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyledString::StyledStringLayout *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcNGuiStyledString::StyledStringLayout::Duplicate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyledString::StyledStringLayout *
        ]
    ),
    "cGcNGuiStyledString::StyledStringLayout::SetStylePos": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyledString::StyledStringLayout *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcNGuiStyledString::ApplyPreRenderAnims": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextStyle *
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # cGcNGuiStyledString::StyledStringLayout *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcNGuiStyledString::RenderLayout": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyledString::StyledStringLayout *
            ctypes.c_ulonglong,  # char *
            ctypes.c_uint32,  # const eNGuiSizeType
        ]
    ),
    "cGcNGuiStyledString::RenderLayout_Internal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyledString::StyledStringLayout *
            ctypes.c_ulonglong,  # char *
            ctypes.c_uint32,  # const eNGuiSizeType
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
        ]
    ),
    "cGcNGuiStyledString::StyledStringLayout::PushStyle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyledString::StyledStringLayout *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcNGuiStyledString::StyledStringLayout::AlignRow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyledString::StyledStringLayout *
        ]
    ),
    "cGcNGuiStyledString::BlendBetweenKeyframes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextStyle *
            ctypes.c_ulonglong,  # const cGcNGuiStyleAnimationKeyframeData *
            ctypes.c_ulonglong,  # const cGcNGuiStyleAnimationKeyframeData *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcResourceManager::SanitiseSceneFilename": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFilename *
        ]
    ),
    "cGcNGuiEffectViewer::LoadEffectFromTable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiEffectViewer *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcNGuiEffectViewer::ClearEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiEffectViewer *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcNGuiEffectViewer::LoadEmitter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiEffectViewer *
            ctypes.c_ulonglong,  # const cTkFilename *
        ]
    ),
    "cGcNGuiEffectViewer::EmitterData::~EmitterData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiEffectViewer::EmitterData *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cTkSceneNodeData::~cTkSceneNodeData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSceneNodeData *
        ]
    ),
    "cGcNGuiEffectViewer::EmitterData::EmitterData": {
        "cGcNGuiEffectViewer::EmitterData *, const cGcNGuiEffectViewer::EmitterData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiEffectViewer::EmitterData *
                ctypes.c_ulonglong,  # const cGcNGuiEffectViewer::EmitterData *
            ]
        ),
        "cGcNGuiEffectViewer::EmitterData *, cGcNGuiEffectViewer::EmitterData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNGuiEffectViewer::EmitterData *
                ctypes.c_ulonglong,  # cGcNGuiEffectViewer::EmitterData *
            ]
        ),
    },
    "cGcNGuiGame::Initialise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGame *
        ]
    ),
    "cGcNGuiGame::LoadFonts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGame *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiGame::BuildSpecialImageMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGame *
        ]
    ),
    "cTkInstanceWindComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInstanceWindComponent *
        ]
    ),
    "cTkInstanceWindComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInstanceWindComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkInstanceWindComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
        ]
    ),
    "ComponentInstancercTkInstanceWindComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercTkInstanceWindComponent *
        ]
    ),
    "ComponentInstancercTkInstanceWindComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcModelViewerCustomisationWindow::CustomisationUIStatic::CustomisationUIStatic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModelViewerCustomisationWindow::CustomisationUIStatic *
        ]
    ),
    "cGcModelViewerCustomisationWindow::CustomisationUIStatic::~CustomisationUIStatic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModelViewerCustomisationWindow::CustomisationUIStatic *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcProceduralTextureDescriptor::cGcProceduralTextureOptionGroup,8,-1> *
        ]
    ),
    "cTkTemplatedComponentUpdater<cTkInstanceWindComponent>::UpdateComponentsRender": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cTkInstanceWindComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkMetadataCache<cTkHeavyAirData>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cTkHeavyAirData> *
        argtypes=[]
    ),
    "cTkMetadataCache<cTkAttachmentData>::GetFilename": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cTkParticleData> *
            ctypes.c_ulonglong,  # const cTkParticleData *
        ]
    ),
    "cTkMetadataCache<cTkParticleData>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cTkParticleData> *
        argtypes=[]
    ),
    "cTkDynamicArray<cTkSceneNodeData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkSceneNodeData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkSceneNodeAttributeData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkSceneNodeData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkSceneNodeData> *
        ]
    ),
    "cTkAttachment::FindFirstAttachmentWithComponent<cGcCreatureComponent>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachment *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkMetaData::ReadFromFile<cTkMaterialData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMaterialData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcNGuiSpecialTextStyles>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiSpecialTextStyles *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcNGuiSpecialTextImages>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEntitlementRewardsTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcPunctuationDelayTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPunctuationDelayTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cTkLanguageFontTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEntitlementRewardsTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcObjectSpawnData>::AllocateAndCopyFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcObjectSpawnData> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcObjectSpawnData> *
        ]
    ),
    "cTkDynamicArray<cGcVehicleWeaponMuzzleData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlanetDataResourceHint> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureRoleData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureRoleData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBuildingOverrideData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBuildingOverrideData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBuildingSpawnData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBuildingSpawnData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBuildingSpawnSlot>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBuildingSpawnSlot> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSelectableObjectSpawnList>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSelectableObjectSpawnList> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcObjectSpawnData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcObjectSpawnData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureSpawnData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureSpawnData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcExternalObjectListOptions>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExternalObjectListOptions> *
            ctypes.c_int32,  # int
        ]
    ),
    "SpecialTextImageData::SpecialTextImageData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SpecialTextImageData *
            ctypes.c_ulonglong,  # SpecialTextImageData *
        ]
    ),
    "cGcNGuiStyledString::SpecialStyle::SpecialStyle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyledString::SpecialStyle *
        ]
    ),
    "cTkDynamicArray<cTkFixedString<256,char>>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkFixedString<256,char> > *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkResourceDescriptor::ToString": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcParticleManager::AddPermanentEffect": {
        "cGcParticleManager *, EffectInstance *, const TkID<128> *, const cTkMatrix34 *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # EffectInstance *
            argtypes=[
                ctypes.c_ulonglong,  # cGcParticleManager *
                ctypes.c_ulonglong,  # EffectInstance *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ulonglong,  # const cTkMatrix34 *
            ]
        ),
        "cGcParticleManager *, EffectInstance *, const TkID<128> *, TkHandle, const cTkMatrix34 *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # EffectInstance *
            argtypes=[
                ctypes.c_ulonglong,  # cGcParticleManager *
                ctypes.c_ulonglong,  # EffectInstance *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_uint32,  # TkHandle
                ctypes.c_ulonglong,  # const cTkMatrix34 *
            ]
        ),
    },
    "cGcEcosystem::GetSpawnData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureFilename *
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystem *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcEcosystem::GetNumCreatures": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystem *
        ]
    ),
    "cGcNGuiStyledString::StyledStringLayout::PushTextInstruction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyledString::StyledStringLayout *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_float,  # const float
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNGuiStyledString::StyledStringLayout::MoveCursor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyledString::StyledStringLayout *
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
        ]
    ),
    "cTkFixedArray<cGcNGuiPresetGraphic,10>::cTkFixedArray<cGcNGuiPresetGraphic,10>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcNGuiPresetGraphic,10> *
        ]
    ),
    "cTkFixedArray<cGcNGuiPresetGraphic,10>::~cTkFixedArray<cGcNGuiPresetGraphic,10>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcNGuiPresetGraphic,10> *
        ]
    ),
    "cTkFixedArray<cGcNGuiPresetText,10>::cTkFixedArray<cGcNGuiPresetText,10>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcNGuiPresetText,10> *
        ]
    ),
    "cTkFixedArray<cGcNGuiPresetText,10>::~cTkFixedArray<cGcNGuiPresetText,10>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcNGuiPresetText,10> *
        ]
    ),
    "cTkMetadataCache<cTkAttachmentData>::Save": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cTkAttachmentData> *
            ctypes.c_ulonglong,  # const cTkAttachmentData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cTkAttachmentData>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cTkAttachmentData> *
        argtypes=[]
    ),
    "cTkAttachment::SaveData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkUiDataMap<cTkNGuiElementData>::Clean": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUiDataMap<cTkNGuiElementData> *
        ]
    ),
    "PCBoolParam::Set": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PCBoolParam *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::WriteBytesCount::SetCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PCIntParam *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNGuiProceduralTextureViewer::cGcProceduralTextureRecord::~cGcProceduralTextureRecord": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiProceduralTextureViewer::cGcProceduralTextureRecord *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcNGuiWindow::SetActive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiWindow *
        ]
    ),
    "cGcNGuiWindow::DoNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_ulonglong,  # const cGcNGuiNodeInfo *
        ]
    ),
    "cGcNGuiNodeInfo::Get": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiNodeInfo *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcNetworkComponent::SendAll": {
        "cGcNetworkComponent *, unsigned __int8 *, unsigned int, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNetworkComponent *
                ctypes.c_ulonglong,  # unsigned __int8 *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_float,  # float
            ]
        ),
        "cGcNetworkComponent *, unsigned __int8 *, unsigned int, cGcNetworkComponentMessage::eType, unsigned __int16, float": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcNetworkComponent *
                ctypes.c_ulonglong,  # unsigned __int8 *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # cGcNetworkComponentMessage::eType
                ctypes.c_uint16,  # unsigned __int16
                ctypes.c_float,  # float
            ]
        ),
    },
    "cGcNetworkComponent::SurveyAll": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkComponent *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkComponent::GetSurveyState": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkComponent *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcNetworkComponent::ProcessSurveyResponses": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkComponent *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['unsigned char const *', 'unsigned short'], 'pointer_depth': 1}
        ]
    ),
    "cGcNetworkComponent::ClearSurvey": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkComponent *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcNetworkComponent::HandleComponentMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const std::shared_ptr<cGcNetworkMessage const > *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkComponent::CreateMessage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const std::shared_ptr<cGcNetworkComponentMessage> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkComponent *
            ctypes.c_ulonglong,  # const std::shared_ptr<cGcNetworkComponentMessage> *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # cGcNetworkComponentMessage::eType
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkComponent::CreateSurvey": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkComponent *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcNetworkInterpolationComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInterpolationComponent *
        ]
    ),
    "cGcNetworkInterpolationComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInterpolationComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcNetworkInterpolationComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcNetworkInterpolationComponent *
        ]
    ),
    "ComponentInstancercGcNetworkInterpolationComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcNetworkInterpolationComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcNetworkInterpolationComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "GeneratePhysLocationMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkQuaternion *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcNetworkInterpolationComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInterpolationComponent *
        ]
    ),
    "cGcNetworkInterpolationComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInterpolationComponent *
        ]
    ),
    "cGcNetworkInterpolationComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInterpolationComponent *
        ]
    ),
    "cGcNetworkInterpolationComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInterpolationComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkInterpolationComponent::SnapToNewPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInterpolationComponent *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkQuaternion *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNetworkInterpolationComponent::NetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInterpolationComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNetworkInterpolationComponent::NetSerializeOnInit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInterpolationComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cGcNetworkPlayerMarkerComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerMarkerComponent *
        ]
    ),
    "cGcNetworkPlayerMarkerComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerMarkerComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcNetworkPlayerMarkerComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcNetworkPlayerMarkerComponent *
        ]
    ),
    "ComponentInstancercGcNetworkPlayerMarkerComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcNetworkPlayerMarkerComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcNetworkPlayerMarkerComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcNetworkPlayerMarkerComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerMarkerComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMatchmaking::cGcMatchmaking": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
        ]
    ),
    "cGcMatchmaking::~cGcMatchmaking": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
        ]
    ),
    "cGcMatchmaking::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
        ]
    ),
    "cGcMatchmaking::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMatchmaking::ProcessInvites": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
        ]
    ),
    "cGcMatchmaking::GetNotificationMessage": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
            ctypes.c_ulonglong,  # TkID<256> *
        ]
    ),
    "cGcMatchmaking::IsInGame": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "cGcMatchmaking::OnInviteQuitDialog": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMatchmaking::ResetAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
        ]
    ),
    "cGcMatchmaking::Session::GetPlayer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking::Session *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcMatchmaking::Connect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
        ]
    ),
    "cGcMatchmaking::JoinInvite": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcMatchmaking::GetFullSimLobbyIDHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
        ]
    ),
    "cGcMatchmaking::Session::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking::Session *
        ]
    ),
    "cGcMatchmaking::Session::GetPlayerUa": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking::Session *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcMatchmaking::Session::GetMaxPlayerCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking::Session *
        ]
    ),
    "cGcMatchmaking::Session::LeaveSession": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking::Session *
        ]
    ),
    "cGcMatchmaking::AllowGameplayLobby": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
        ]
    ),
    "cGcMatchmaking::IsReadyToLoad": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
        ]
    ),
    "cGcMatchmaking::OnNetworkPlayerStateEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint32,  # eGcNetworkPlayerStateEvent
        ]
    ),
    "cGcMatchmaking::OnRequestPermissionCreateLobby": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMatchmaking::SendRequestPermissionCreateLobby": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMatchmaking::Session::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking::Session *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMatchmaking::JoinFireteam": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcMatchmaking::AddBadLobby": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
            ctypes.c_ulonglong,  # const cGcLobbyId *
        ]
    ),
    "cGcMatchmaking::OnRemoteKickRequest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcMatchmaking::KickPlayer": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcMatchmaking::OnRemoteInviteToFireteamRequest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcMatchmaking::InviteToFireteam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcMatchmaking::OnRemoteRefuseInviteToFireteam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcMatchmaking::RefuseInviteToFireteam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcMatchmaking::OnRemoteRequestAddFriend": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcMatchmaking::ReplyToRequestAddFriend": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcMatchmaking::OnRemoteReplyToRequestAddFriend": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcMatchmaking::UpdateFullSimPrivileges": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
        ]
    ),
    "cGcMonumentManager::SpawnMonument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMonumentManager *
            ctypes.c_ulonglong,  # const cGcAtlasMonument *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMonumentManager::UpdateRequests": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMonumentManager *
        ]
    ),
    "cGcMonumentManager::DoInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMonumentManager *
            ctypes.c_ulonglong,  # cGcFrontendPage *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcNetworkAbandonedFreightersManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkAbandonedFreightersManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkAbandonedFreightersManager::SpawnAbandonedFreighter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkAbandonedFreightersManager *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcNetworkAbandonedFreightersManager::GetAbandonedFreighter": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkAbandonedFreightersManager *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcNetworkAbandonedFreightersManager::CleanAll": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkAbandonedFreightersManager *
        ]
    ),
    "cGcNetworkAbandonedFreightersManager::FindSpawn": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkAbandonedFreightersManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcNetworkAbandonedFreightersManager::SpawnFreighter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkAbandonedFreightersManager *
            ctypes.c_ulonglong,  # cGcNetworkAbandonedFreightersManager::sAbandonedFreighterSpawn *
        ]
    ),
    "cGcNetworkAbandonedFreightersManager::DespawnFreighter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkAbandonedFreightersManager *
            ctypes.c_ulonglong,  # cGcNetworkAbandonedFreightersManager::sAbandonedFreighterSpawn *
        ]
    ),
    "cGcNetworkBaseBuildingManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseBuildingManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkBaseBuildingManager::SelectBufferToSynchronise": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseBuildingManager *
        ]
    ),
    "cGcNetworkBaseBuildingManager::SelectBufferFromHashMessage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPersistentInteractionsManager *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseBuildingManager *
            ctypes.c_ulonglong,  # const cGcNetworkHashMessage *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkBaseBuildingManager::SelectBufferFromSyncMessage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPersistentInteractionsManager *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseBuildingManager *
            ctypes.c_ulonglong,  # const cGcNetworkSyncMessage *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkBaseBuildingManager::OnPlayerLoaded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseBuildingManager *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNetworkBaseBuildingManager::OnPlayerLeft": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseBuildingManager *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNetworkBaseBuildingManager::BroadcastBaseHeaderMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseBuildingManager *
            ctypes.c_ulonglong,  # _WORD *
        ]
    ),
    "cGcNetworkBaseBuildingManager::BroadcastBaseEdit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseBuildingManager *
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
        ]
    ),
    "cGcNetworkBaseBuildingManager::BroadcastBaseDataChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseBuildingManager *
            ctypes.c_uint64,  # sBaseObjectHandle
        ]
    ),
    "cGcNetworkBaseBuildingManager::BroadcastPlaceholderUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseBuildingManager *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_double,  # double
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eBaseBuildingMode
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNetworkBaseBuildingManager::BroadcastGridAmountChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseBuildingManager *
            ctypes.c_uint64,  # sBaseObjectHandle
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNetworkBaseBuildingManager::BroadcastDifficultyChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseBuildingManager *
            ctypes.c_ulonglong,  # __int16 *
        ]
    ),
    "cGcNetworkBaseBuildingManager::UpdateSendBaseHeaderMessages": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseBuildingManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkBaseBuildingManager::SelectOrRequestBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPersistentInteractionsManager *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseBuildingManager *
            ctypes.c_uint32,  # ePersistentBaseTypes
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkBaseBuildingManager::GetBaseLocationForEditMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseBuildingManager *
            ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkBaseEditMessage> *
            ctypes.c_ulonglong,  # _WORD *
        ]
    ),
    "cGcNetworkBaseImageManager::LoadPendingTextures": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseImageManager *
        ]
    ),
    "cGcNetworkBaseImageManager::FetchBaseImageData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseImageManager *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNetworkBaseImageManager::CheckAndRenewRequest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseImageManager *
            ctypes.c_ulonglong,  # cGcNetworkBaseImageManager::BaseImageRequest *
        ]
    ),
    "cGcNetworkBaseImageManager::HttpSendRequest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseImageManager *
            ctypes.c_ulonglong,  # cGcNetworkBaseImageManager::BaseImageRequest *
        ]
    ),
    "cGcNetworkBaseImageManager::HttpHandleRequestResult": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseImageManager *
            ctypes.c_ulonglong,  # const Http::RequestResult *
        ]
    ),
    "cGcNetworkBaseImageManager::GetBaseImageTexture": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkTextureResource *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseImageManager *
            ctypes.c_ulonglong,  # const unsigned int *
        ]
    ),
    "cGcNetworkBaseImageManager::ResetNotFoundAndFailedStates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseImageManager *
        ]
    ),
    "cGcNetworkBaseImageManager::sSnapshotHandle::WriteEncodedImageCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNetworkBaseImageManager::RequestSaveTextureToSnapshot": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseImageManager *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
        ]
    ),
    "cGcNetworkBaseImageManager::ClearSnapshot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseImageManager *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkReplicatedVariableTemplated<bool>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<signed char> *
            ctypes.c_char_p,  # const char *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkReplicatedVariableTemplated<cTkUserIdBase<cTkFixedString<64,char>>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cTkUserIdBase<cTkFixedString<64,char> > > *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkUserIdBase<cTkFixedString<64,char>>::Invalidate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cTkNGuiEditor::DoEditInt<int>": {
        "cTkNGuiEditor *, const char *, int *, bool, float": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiEditor *
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # int *
                ctypes.c_ubyte,  # bool
                ctypes.c_float,  # float
            ]
        ),
        "cTkNGuiEditor *, int *, bool, float, bool": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiEditor *
                ctypes.c_ulonglong,  # int *
                ctypes.c_ubyte,  # bool
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cGcMatchmaking::IsInFullSimLobby": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
        ]
    ),
    "cTkRegion::GetResource": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const TryStoreMode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
        ]
    ),
    "cGcMatchmaking::GetExtraFriendsString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcMatchmaking::Release": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
        ]
    ),
    "cGcNetworkBufferSyncManager::GetPlayerIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBufferSyncManager *
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcAsyncOpsTempMonument::SetCocreatorName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsyncOpsTempMonument *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcAsyncOpsTempMonument::SetCreatorName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsyncOpsTempMonument *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkRigidBody::SetCollisionFilterGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkRigidBody::GetCollisionFilterMask": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::SetCollisionFilterMask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkScopedCriticalSectionLock::cTkScopedCriticalSectionLock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkScopedCriticalSectionLock *
            ctypes.c_ulonglong,  # _RTL_CRITICAL_SECTION *
        ]
    ),
    "cTkScopedCriticalSectionLock::~cTkScopedCriticalSectionLock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkScopedCriticalSectionLock *
        ]
    ),
    "cGcMatchmaking::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking *
        ]
    ),
    "cGcMatchmaking::Session::RegisterOtherSession": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking::Session *
            ctypes.c_ulonglong,  # cGcMatchmaking::Session *
        ]
    ),
    "cGcMatchmaking::Session::Leave": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking::Session *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcParticleManager::AddManagedEffect": {
        "cGcParticleManager *, EffectInstance *, const TkID<128> *, const cTkMatrix34 *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # EffectInstance *
            argtypes=[
                ctypes.c_ulonglong,  # cGcParticleManager *
                ctypes.c_ulonglong,  # EffectInstance *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ulonglong,  # const cTkMatrix34 *
            ]
        ),
        "cGcParticleManager *, EffectInstance *, const TkID<128> *, TkHandle, const cTkMatrix34 *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # EffectInstance *
            argtypes=[
                ctypes.c_ulonglong,  # cGcParticleManager *
                ctypes.c_ulonglong,  # EffectInstance *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_uint32,  # TkHandle
                ctypes.c_ulonglong,  # const cTkMatrix34 *
            ]
        ),
    },
    "cGcPlayerEffectsComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkAtomicLinearMemoryPool::GetBaseAddress": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAtomicLinearMemoryPool *
        ]
    ),
    "cGcNetworkBufferHash::DebugValidateChunkHashValues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBufferHash *
        ]
    ),
    "cGcNetworkBufferHash::SyncHashOffsetValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBufferHash *
            ctypes.c_int16,  # __int16
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNetworkBufferHash::Initialise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBufferHash *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNetworkBufferHash::ClearHashes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBufferHash *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNetworkBufferHash::OnBufferRangeAdded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBufferHash *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNetworkBufferHash::OnBufferRangeRemoved": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBufferHash *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNetworkBufferHash::OnBufferRangeCleared": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBufferHash *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNetworkBufferHash::OnBufferRangeSet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBufferHash *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNetworkBufferSyncManager::Initialise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBufferSyncManager *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkBufferSyncManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBufferSyncManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkBufferSyncManager::OnNetworkPlayerStateEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBufferSyncManager *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint32,  # eGcNetworkPlayerStateEvent
        ]
    ),
    "cGcNetworkBufferSyncManager::UpdateSendHashMessages": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBufferSyncManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkCreatureManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkCreatureManager *
        ]
    ),
    "cGcNetworkCreatureManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkCreatureManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkCreatureManager::OnNetworkPlayerStateEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkCreatureManager *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint32,  # eGcNetworkPlayerStateEvent
        ]
    ),
    "cGcNetworkIdManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkIdManager *
        ]
    ),
    "cGcNetworkIdManager::GenerateId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkNetworkId *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkIdManager *
            ctypes.c_ulonglong,  # cTkNetworkId *
        ]
    ),
    "cGcNetworkIdManager::SetRemoteOwner": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcNetworkIdManager::OnRemoteEntitySpawned": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkIdManager *
            ctypes.c_ulonglong,  # const cTkNetworkId *
        ]
    ),
    "cGcNetworkIdManager::OnNetworkPlayerStateEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkIdManager *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint32,  # eGcNetworkPlayerStateEvent
        ]
    ),
    "cGcNetworkIdManager::UpdateMaxEntityUsed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcNetworkIdManager::OnNewClient": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkIdManager *
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcNetworkInteractionBufferManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInteractionBufferManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkInteractionBufferManager::SelectBufferToSynchronise": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMaintenanceBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInteractionBufferManager *
        ]
    ),
    "cGcNetworkInteractionBufferManager::SelectBufferFromHashMessage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMaintenanceBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInteractionBufferManager *
            ctypes.c_ulonglong,  # const cGcNetworkHashMessage *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkInteractionBufferManager::SelectBufferFromSyncMessage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMaintenanceBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInteractionBufferManager *
            ctypes.c_ulonglong,  # const cGcNetworkSyncMessage *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cTkAudioComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcInteractionComponent::SetName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNetworkInteractionManager::InviteToFireteam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInteractionManager *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkInteractionManager::CanInvite": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInteractionManager *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcNetworkInteractionManager::RequestAddFriend": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInteractionManager *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkInteractionManager::OnHover": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInteractionManager *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcNetworkInteractionManager::GenerateDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInteractionManager *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcNetworkInteractionManager::CanInteract": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInteractionManager *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcNetworkInteractionManager::IsInteractionDisabled": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInteractionManager *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcNetworkInteractionManager::GetInteractionDisabledReason": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInteractionManager *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        ]
    ),
    "cGcNetworkLockManager::RequestLock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkLockManager *
            ctypes.c_ulonglong,  # const cGcNetworkLockManager::sLock *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkLockManager::CheckHaveLock": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkLockManager *
            ctypes.c_ulonglong,  # const cGcNetworkLockManager::sKey *
        ]
    ),
    "cGcNetworkLockManager::TryKeyAndGetOwner": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkLockManager *
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
            ctypes.c_ulonglong,  # const cGcNetworkLockManager::sKey *
        ]
    ),
    "cGcNetworkLockManager::FindNetworkLock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkLockManager *
            ctypes.c_ulonglong,  # const cGcNetworkLockManager::sLock *
        ]
    ),
    "cGcNetworkLockManager::SendUnlockMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkLockManager *
            ctypes.c_ulonglong,  # const cGcNetworkLockManager::sKey *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcNetworkLockManager::CheckLockBlocks": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNetworkLockManager::sLock *
            ctypes.c_ulonglong,  # const cGcNetworkLockManager::sLock *
        ]
    ),
    "cGcNetworkManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkManager *
        ]
    ),
    "cGcNetworkManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkManager::GetLocalUserId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkManager *
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcNetworkPlayer::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkPlayer::HandleMessage": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_ulonglong,  # const std::shared_ptr<cGcNetworkMessage const > *
        ]
    ),
    "cGcNetworkPlayer::MatchesLocalPlayerLocation": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcNetworkPlayer::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNetworkPlayer::OnConnected": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkPlayer::SetPlayerName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcNetworkPlayer::CheckPendingState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkPlayer::VirtualGetMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcNetworkPlayer::ReceiveData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcNetworkPlayer::ProcessMessage": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkMessage const > *
        ]
    ),
    "cGcNetworkPlayer::IsHost": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkPlayer::GetPlatformType": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkPlayerFireteam::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFireteam *
        ]
    ),
    "cGcNetworkPlayerFireteam::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFireteam *
        ]
    ),
    "cGcNetworkPlayerFireteam::HandleMessage": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFireteam *
            ctypes.c_ulonglong,  # const std::shared_ptr<cGcNetworkMessage const > *
        ]
    ),
    "cGcNetworkPlayerFireteam::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFireteam *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNetworkPlayerFireteam::GetNetworkIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFireteam *
            ctypes.c_uint32,  # cGcNetworkConstants::LobbyType
        ]
    ),
    "cGcVehicleComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcPlayerCharacterComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerFullBodyIKComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNetworkPlayerFullSim::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcNetworkPlayerFullSim::IsFullSimLobbyHost": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcNetworkPlayerFullSim::OnConnected": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
            ctypes.c_uint32,  # eGcNetworkPlayerState
        ]
    ),
    "cGcNetworkPlayerFullSim::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcNetworkPlayerFullSim::HandleMessage": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
            ctypes.c_ulonglong,  # const std::shared_ptr<cGcNetworkMessage const > *
        ]
    ),
    "cGcNetworkPlayerFullSim::OnServerUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
            ctypes.c_ulonglong,  # const std::shared_ptr<cGcNetworkFullSimUpdateMessage const > *
        ]
    ),
    "cGcNetworkPlayerFullSim::GetPlayerNameWithTitle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcNetworkPlayerFullSim::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
            ctypes.c_double,  # double
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNetworkPlayerFullSim::UpdateHandTracking": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
            ctypes.c_double,  # double
        ]
    ),
    "cGcNetworkPlayerFullSim::DisableHandTracking": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcNetworkPlayerFullSim::GetTimeUntilNextExpectedHandsUpdate": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcNetworkPlayerFullSim::OnNetworkPlayerStateEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
            ctypes.c_uint32,  # eGcNetworkPlayerStateEvent
        ]
    ),
    "cGcNetworkPlayerFullSim::GetVehicle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcVehicleComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
            ctypes.c_uint32,  # eVehicleType
        ]
    ),
    "cGcNetworkPlayerFullSim::GetOccupiedVehicle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcNetworkPlayerFullSim::SetPlayerState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
            ctypes.c_uint32,  # eGcNetworkPlayerState
        ]
    ),
    "cGcNetworkPlayerFullSim::SendPlayerExperienceSpawn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
            ctypes.c_ulonglong,  # const cGcSpawnRequest *
        ]
    ),
    "cGcNetworkPlayerFullSim::GetNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcNetworkPlayerFullSim::VirtualGetAttachment": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachment *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcNetworkPlayerFullSim::GetPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcNetworkPlayerFullSim::GetMapPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcNetworkPlayerFullSim::VirtualGetMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcNetworkPlayerFullSim::IsOnMyPlanet": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcNetworkPlayerFullSim::IsInUnknownFireteam": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcNetworkPlayerFullSim::IsStealthActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcNetworkPlayerReplicatedState::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerReplicatedState *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint32,  # cGcNetworkConstants::LobbyType
        ]
    ),
    "cGcNetworkPlayerReplicatedState::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerReplicatedState *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkPrivilegesManager::GetRemotePrivilegeForLocalPlayer": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPrivilegesManager *
            ctypes.c_uint32,  # ePrivilegeType
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcNetworkPrivilegesManager::SaveLocalPrivilegesToUInt64": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPrivilegesManager *
        ]
    ),
    "cGcNetworkPrivilegesManager::OnNetworkPlayerStateEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPrivilegesManager *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint32,  # eGcNetworkPlayerStateEvent
        ]
    ),
    "cGcNetworkPrivilegesManager::RPCPrivilegeUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "ComponentInstancercGcPlayerComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcPlayerComponent *
        ]
    ),
    "ComponentInstancercGcPlayerComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcNetworkReplicatedEntityManager::FindMasterEntityNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_ulonglong,  # const cTkNetworkId *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNetworkReplicatedEntityManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkReplicatedEntityManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkReplicatedEntityManager::SerializeEntity": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkReplicatedEntityManager *
            ctypes.c_ulonglong,  # sGcNetworkReplicatedEntitiesUpdateMessage *
        ]
    ),
    "cGcNetworkReplicatedEntityManager::CheckInterestRange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkReplicatedEntityManager *
            ctypes.c_uint32,  # eReplicationRange
            ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponent *
        ]
    ),
    "cGcNetworkReplicatedEntityManager::IsInValidRepicationRange": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkReplicatedEntityManager *
            ctypes.c_uint32,  # eReplicationRange
            ctypes.c_uint32,  # EnvironmentLocation::Enum
        ]
    ),
    "cGcNetworkReplicatedEntityManager::HandleNetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkReplicatedEntityManager *
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
            ctypes.c_ulonglong,  # const std::shared_ptr<cGcNetworkMessage const > *
        ]
    ),
    "cGcNetworkReplicatedEntityManager::RemoveEntitiesFromPlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkReplicatedEntityManager *
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
            ctypes.c_uint32,  # eReplicationRange
        ]
    ),
    "cGcNetworkReplicatedEntityManager::OnNetworkPlayerStateEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkReplicatedEntityManager *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint32,  # eGcNetworkPlayerStateEvent
        ]
    ),
    "cGcNetworkReplicatedEntityManager::RemoveEntity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkReplicatedEntityManager *
            ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponent *
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcNetworkReplicatedEntityManager::HandleRemoveEntity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkReplicatedEntityManager *
            ctypes.c_ulonglong,  # const std::shared_ptr<sGcNetworkReplicatedEntityDestroyedMessage const > *
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcNetworkReplicatedEntityManager::ReplicateEntityAcrossNetwork": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkReplicatedEntityManager *
            ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponent *
        ]
    ),
    "cGcNetworkReplicatedEntityManager::DestroyReplicatedEntity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkReplicatedEntityManager *
            ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNetworkReplicatedEntityManager::SendReplicatedEntityToPlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkReplicatedEntityManager *
            ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponent *
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcNetworkReplicatedEntityManager::SendEntityListToPlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkReplicatedEntityManager *
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
            ctypes.c_uint32,  # eReplicationRange
        ]
    ),
    "cGcNetworkReplicatedEntityManager::ChangeOwnership": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkReplicatedEntityManager *
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
            ctypes.c_ulonglong,  # const cTkNetworkId *
        ]
    ),
    "cGcNetworkReplicatedEntityManager::RemoveEntityFromReplication": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkReplicatedEntityManager *
            ctypes.c_ulonglong,  # const cTkNetworkId *
            ctypes.c_ulonglong,  # cGcNetworkPlayerFullSim *
        ]
    ),
    "cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer>::DeleteVariableInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint32,  # cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer>::eLocation
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkPlayer::cGcNetworkPlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkWeapon::cGcNetworkWeapon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkWeapon *
        ]
    ),
    "cTkReplicatedVariableTemplated<cTkFixedString<4,char>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<4,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cTkFixedString<4,char> > *
            ctypes.c_ulonglong,  # const cTkFixedString<4,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkReplicatedVariableTemplated<cTkFixedString<128,char>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cTkFixedString<128,char> > *
            ctypes.c_ulonglong,  # const cTkFixedString<128,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkReplicatedVariableTemplated<eCamouflageState>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # eCamouflageState *
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<enum eCamouflageState> *
            ctypes.c_ulonglong,  # const eCamouflageState *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkReplicatedVariableTemplated<TkID<128>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<TkID<128> > *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkReplicatedVariableTemplated<unsigned __int64>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int64 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<unsigned __int64> *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkReplicatedVariableTemplated<cGcLobbyId>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcLobbyId *
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<cGcLobbyId> *
            ctypes.c_ulonglong,  # const cGcLobbyId *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNetworkMaintenanceBufferEditMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkMaintenanceBufferEditMessage *
        ]
    ),
    "cGcDestructableComponent::Destroy": {
        "cGcDestructableComponent *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcDestructableComponent *
            ]
        ),
        "cGcDestructableComponent *, eDestroyedBy, const cTkUserIdBase<cTkFixedString<64,char> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcDestructableComponent *
                ctypes.c_uint32,  # eDestroyedBy
                ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
            ]
        ),
        "cGcDestructableComponent *, const cGcProjectileImpact *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcDestructableComponent *
                ctypes.c_ulonglong,  # const cGcProjectileImpact *
            ]
        ),
    },
    "sGcNetworkReplicatedPetEntityData::sGcNetworkReplicatedPetEntityData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sGcNetworkReplicatedPetEntityData *
        ]
    ),
    "cGcPlayer::TakeDamage": {
        "cGcPlayer *, const TkID<128> *, const TkID<128> *, const cTkVector3 *, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayer *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_float,  # float
            ]
        ),
        "cGcPlayer *, const TkID<128> *, const TkID<128> *, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayer *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_float,  # float
            ]
        ),
        "cGcPlayer *, const TkID<128> *, const TkID<128> *, const cTkVector3 *, cGcOwnerConcept *, float, const std::vector<cGcCombatEffectDamageMultiplier,TkSTLAllocatorShim<cGcCombatEffectDamageMultiplier,4,-1> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayer *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # cGcOwnerConcept *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # const std::vector<cGcCombatEffectDamageMultiplier,TkSTLAllocatorShim<cGcCombatEffectDamageMultiplier,4,-1> > *
            ]
        ),
        "cGcPlayer *, float, eDamageType, const TkID<128> *, const cTkVector3 *, cGcOwnerConcept *, const std::vector<cGcCombatEffectDamageMultiplier,TkSTLAllocatorShim<cGcCombatEffectDamageMultiplier,4,-1> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayer *
                ctypes.c_float,  # float
                ctypes.c_uint32,  # eDamageType
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # cGcOwnerConcept *
                ctypes.c_ulonglong,  # const std::vector<cGcCombatEffectDamageMultiplier,TkSTLAllocatorShim<cGcCombatEffectDamageMultiplier,4,-1> > *
            ]
        ),
    },
    "cTkSpinMutexScoped::cTkSpinMutexScoped": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSpinMutexScoped *
            ctypes.c_ulonglong,  # cTkSpinMutex *
        ]
    ),
    "NetReplicatedCustomisationData::NetReplicatedCustomisationData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NetReplicatedCustomisationData *
        ]
    ),
    "sGcNetworkReplicatedEntitiesWelcomeMessage::sGcNetworkReplicatedEntitiesWelcomeMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sGcNetworkReplicatedEntitiesWelcomeMessage *
            ctypes.c_uint32,  # eGcNetworkMessageType
        ]
    ),
    "cTkAudioManager::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAudioManager *
        argtypes=[]
    ),
    "cGcNetworkRpcManager::RegisterRpc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkRpcManager *
            ctypes.c_ulonglong,  # cGcRpcCallBase *
        ]
    ),
    "cGcNetworkRpcManager::UnregisterRpc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkRpcManager *
            ctypes.c_ulonglong,  # cGcRpcCallBase *
        ]
    ),
    "cGcNetworkSynchronisedBuffer::GetHashValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sHashValue *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkSynchronisedBuffer *
            ctypes.c_ulonglong,  # sHashValue *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcNetworkSynchronisedBuffer::GetHashTimestamp": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkSynchronisedBuffer *
        ]
    ),
    "cGcNetworkSynchronisedBuffer::AddHashValues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkSynchronisedBuffer *
            ctypes.c_ulonglong,  # cGcNetworkHashMessage *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNetworkSynchronisedBuffer::AddSyncValues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkSynchronisedBuffer *
            ctypes.c_ulonglong,  # cGcNetworkSyncMessage *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNetworkSynchronisedBuffer::HasNetworkOwner": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkSynchronisedBuffer *
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcNetworkSynchronizedClock::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkSynchronizedClock *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkSynchronizedClock::OnMultiplayerSessionStarting": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkSynchronizedClock *
            ctypes.c_uint32,  # cGcNetworkConstants::LobbyType
        ]
    ),
    "cGcNetworkSynchronizedClock::HandleNetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkSynchronizedClock *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_ulonglong,  # std::shared_ptr<cGcNetworkMessage const > *
        ]
    ),
    "cGcNetworkSynchronizedClock::OnHandleFullRoundTimeUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkSynchronizedClock *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNetworkSynchronizedClock::RenderImGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkSynchronizedClock *
        ]
    ),
    "cGcNetworkTerrainEditsManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkTerrainEditsManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkTerrainEditsManager::OnPlayerLeft": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkTerrainEditsManager *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNetworkTerrainEditsManager::BroadcastEdits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkTerrainEditsManager *
            ctypes.c_ulonglong,  # const sRegionEdit *
            ctypes.c_int32,  # const int
            ctypes.c_uint32,  # cGcNetworkTerrainEditMessage::eDataType
        ]
    ),
    "cGcNetworkTerrainEditsManager::UpdateSendEditMessages": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkTerrainEditsManager *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcNetworkTerrainEditsManager::UpdateSendNewBlockMessages": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkTerrainEditsManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkTerrainEditsManager::SelectBufferToSynchronise": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkTerrainEditsManager *
        ]
    ),
    "cGcNetworkTerrainEditsManager::SelectBufferFromHashMessage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkTerrainEditsManager *
            ctypes.c_ulonglong,  # const cGcNetworkHashMessage *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkTerrainEditsManager::SelectBufferFromSyncMessage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTerrainEditBlockBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkTerrainEditsManager *
            ctypes.c_ulonglong,  # const cGcNetworkSyncMessage *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkTerrainEditsManager::OnHashMessageSent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkTerrainEditsManager *
            ctypes.c_ulonglong,  # cGcNetworkSynchronisedBuffer *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkTerrainEditsManager::OnHashMessageReceived": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkTerrainEditsManager *
            ctypes.c_ulonglong,  # cGcNetworkSynchronisedBuffer *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint32,  # cGcNetworkBufferSyncManager::eHashSyncError
        ]
    ),
    "cGcNetworkTerrainEditsManager::OnSyncMessageReceived": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkTerrainEditsManager *
            ctypes.c_ulonglong,  # const cGcNetworkSyncMessage *
            ctypes.c_ulonglong,  # cGcNetworkSynchronisedBuffer *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint32,  # cGcNetworkBufferSyncManager::eHashSyncError
        ]
    ),
    "cGcNetworkTerrainEditsManager::CheckPlayerCanReceiveEdits": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkTerrainEditsManager *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerWeaponComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponComponent *
        ]
    ),
    "cGcPlayerWeaponComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcPlayerWeaponComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcPlayerWeaponComponent *
        ]
    ),
    "ComponentInstancercGcPlayerWeaponComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcNetworkWeapon::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkWeapon *
        ]
    ),
    "cGcNetworkWeapon::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkWeapon *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkWeapon::UpdateLaserSnapshots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkWeapon *
            ctypes.c_float,  # float
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNetworkWeapon::GetProjectileOrEffectID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkWeapon *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_uint32,  # eRemoteWeaponType
        ]
    ),
    "cGcNetworkWeapon::GetBarrelPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkWeapon *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_uint32,  # eRemoteWeaponType
        ]
    ),
    "cGcNetworkWeapon::RefreshShipBarrels": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkWeapon *
        ]
    ),
    "cGcNetworkWeapon::UpdateWeaponMuzzle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkWeapon *
            ctypes.c_uint32,  # eRemoteWeaponType
        ]
    ),
    "cGcNetworkWeapon::OnHolsterStatusChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNetworkWeapon::OnReload": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkWeapon::FireRemote": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # eRemoteWeaponType
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const std::vector<cGcImpactCombatEffectData,TkSTLAllocatorShim<cGcImpactCombatEffectData,4,-1> > *
            ctypes.c_ulonglong,  # const std::vector<cGcCombatEffectDamageMultiplier,TkSTLAllocatorShim<cGcCombatEffectDamageMultiplier,4,-1> > *
        ]
    ),
    "cGcNetworkWeapon::RPCHolsterStatusChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNetworkWeapon::RPCPlayerWeaponMelee": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcNetworkWeapon::RPCPlayerWeaponReload": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcTextChatInput::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextChatInput *
        ]
    ),
    "cGcTextChatInput::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextChatInput *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcTextChatInput::GetCommandFromName": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextChatInput *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char **
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eTextChatCommands
        ]
    ),
    "cGcTextChatInput::AutoCompleteNextPlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextChatInput *
            ctypes.c_uint32,  # cGcNetworkConstants::LobbyType
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTextChatInput::AutoComplete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextChatInput *
        ]
    ),
    "cGcTextChatInput::TokeniseInput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextChatInput *
            ctypes.c_ulonglong,  # const cTkFixedString<1023,char> *
            ctypes.c_ulonglong,  # std::vector<cTkFixedString<64,char>,TkSTLAllocatorShim<cTkFixedString<64,char>,1,-1> > *
        ]
    ),
    "cGcTextChatInput::ParseTextForCommands": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextChatInput *
            ctypes.c_ulonglong,  # const cTkFixedString<1023,char> *
        ]
    ),
    "cGcTextChatManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextChatManager *
        ]
    ),
    "cGcTextChatManager::Say": {
        "cGcTextChatManager *, const cTkFixedString<1023,char> *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcTextChatManager *
                ctypes.c_ulonglong,  # const cTkFixedString<1023,char> *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcTextChatManager *, const cTkFixedString<1023,char> *, const cTkUserIdBase<cTkFixedString<64,char> > *, const std::vector<cTkUserIdBase<cTkFixedString<64,char> >,TkSTLAllocatorShim<cTkUserIdBase<cTkFixedString<64,char> >,1,-1> > *, bool, bool, bool, TextChatMessageType, const cTkColour *, int, float": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcTextChatManager *
                ctypes.c_ulonglong,  # const cTkFixedString<1023,char> *
                ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
                ctypes.c_ulonglong,  # const std::vector<cTkUserIdBase<cTkFixedString<64,char> >,TkSTLAllocatorShim<cTkUserIdBase<cTkFixedString<64,char> >,1,-1> > *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
                ctypes.c_uint32,  # TextChatMessageType
                ctypes.c_ulonglong,  # const cTkColour *
                ctypes.c_int32,  # int
                ctypes.c_float,  # float
            ]
        ),
    },
    "cGcTextChatManager::PostLocalMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextChatManager *
            ctypes.c_ulonglong,  # const cTkFixedString<1121,char> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cGcTextChatManager::AddPendingMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextChatManager *
            ctypes.c_ulonglong,  # const PendingMessage *
        ]
    ),
    "cGcTextChatManager::AddStatusMessageCustomText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextChatManager *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # cGcTextChatManager::ePrefixPlayerName
            ctypes.c_ulonglong,  # std::initializer_list<char const *> *
        ]
    ),
    "cGcTextChatManager::GetRandomLocId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextChatManager *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcTextChatManager::GeneratePetChatText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextChatManager *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_uint32,  # ePetChatType
            ctypes.c_uint32,  # eSpecialPetChatType
        ]
    ),
    "cGcTextChatManager::GenerateFriendlyDroneChatText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextChatManager *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_uint32,  # eFriendlyDroneChatType
        ]
    ),
    "cGcTextChatManager::AddPetChat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextChatManager *
            ctypes.c_uint32,  # ePetChatType
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVoiceChat::GetVoiceState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVoiceChat *
            ctypes.c_ulonglong,  # const cGcNetworkPlayer *
        ]
    ),
    "cGcVoiceChat::MutePlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVoiceChat *
            ctypes.c_ulonglong,  # const cGcNetworkPlayer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVoiceChat::IsMuted": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcVoiceChat *
            ctypes.c_ulonglong,  # const cGcNetworkPlayer *
        ]
    ),
    "cGcVoiceChat::CheckStartStop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVoiceChat *
        ]
    ),
    "cGcVoiceChat::OnMultiplayerSessionStarting": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVoiceChat *
            ctypes.c_uint32,  # cGcNetworkConstants::LobbyType
        ]
    ),
    "cGcVoiceChat::OnMultiplayerSessionFinished": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVoiceChat *
            ctypes.c_uint32,  # cGcNetworkConstants::LobbyType
        ]
    ),
    "cGcVoiceChat::OnNetworkPlayerStateEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVoiceChat *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint32,  # eGcNetworkPlayerStateEvent
        ]
    ),
    "cTkMetadataCache<cGcStatusMessageDefinitions>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcStatusMessageDefinitions *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcStatusMessageDefinitions> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcPlayerWeaponComponent>::UpdateComponentsPostPhysics": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcPlayerWeaponComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkFixedString<1121,char>::Format<char,0>": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<1121,char> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcStatusMessageDefinitions>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcStatusMessageDefinitions *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCreatureOwnership::GetTraitValue": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_uint32,  # ePetTrait
        ]
    ),
    "TextChatMessageBuffer::AddMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TextChatMessageBuffer *
            ctypes.c_ulonglong,  # const cTkFixedString<1121,char> *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cGcTextChatManager::RenderImGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcProjectileManager::GetLaserData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcLaserBeamData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcNetworkSynchronizedClock::RenderImGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcNetworkBaseEditMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseEditMessage *
        ]
    ),
    "cGcNetworkBaseHashMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseHashMessage *
        ]
    ),
    "cGcNetworkBaseHeaderMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseHeaderMessage *
        ]
    ),
    "cGcNetworkBaseSyncMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkBaseSyncMessage *
        ]
    ),
    "sGcNetworkClientTimeSyncMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # sGcNetworkClientTimeSyncMessage *
        ]
    ),
    "cGcNetworkComponentMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkComponentMessage *
        ]
    ),
    "cGcNetworkFleetExpeditionsSyncMessage::GatherData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkFleetExpeditionsSyncMessage *
        ]
    ),
    "cGcNetworkFleetExpeditionsSyncMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkFleetExpeditionsSyncMessage *
        ]
    ),
    "cGcNetworkFleetExpeditionsSyncMessage::SerializeOrDeserializeExpedition": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkFleetExpeditionsSyncMessage *
            ctypes.c_ulonglong,  # cGcExpeditionSyncData *
        ]
    ),
    "cGcNetworkFleetFrigateNamesSyncMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkFleetFrigateNamesSyncMessage *
        ]
    ),
    "cGcNetworkFleetFrigatesSyncMessage::GatherData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkFleetFrigatesSyncMessage *
        ]
    ),
    "cGcNetworkFleetFrigatesSyncMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkFleetFrigatesSyncMessage *
        ]
    ),
    "cGcNetworkFleetFrigatesSyncMessage::SerializeOrDeserializeFrigate": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkFleetFrigatesSyncMessage *
            ctypes.c_ulonglong,  # cGcNetworkFleetFrigatesSyncMessage::cGcFrigateSyncData *
        ]
    ),
    "cGcNetworkFreighterOwnershipSyncMessage::cGcNetworkFreighterOwnershipSyncMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkFreighterOwnershipSyncMessage *
            ctypes.c_uint32,  # eGcNetworkMessageType
        ]
    ),
    "cGcNetworkFreighterOwnershipSyncMessage::GatherData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkFreighterOwnershipSyncMessage *
        ]
    ),
    "cGcNetworkFreighterOwnershipSyncMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkFreighterOwnershipSyncMessage *
        ]
    ),
    "cGcNetworkFullSimHandsUpdateMessage::SetPlayerData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkFullSimHandsUpdateMessage *
            ctypes.c_ulonglong,  # const cGcPlayer *
        ]
    ),
    "cGcNetworkFullSimHandsUpdateMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkFullSimHandsUpdateMessage *
        ]
    ),
    "cGcNetworkFullSimUpdateMessage::SetPlayerData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkFullSimUpdateMessage *
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcNetworkFullSimUpdateMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkFullSimUpdateMessage *
        ]
    ),
    "cGcNetworkInteractionBufferSyncMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInteractionBufferSyncMessage *
        ]
    ),
    "cGcNetworkInteractionBufferHashMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInteractionBufferHashMessage *
        ]
    ),
    "cGcNetworkInteractionBufferEditMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInteractionBufferEditMessage *
        ]
    ),
    "cGcNetworkLockMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkLockMessage *
        ]
    ),
    "cGcNetworkMessage::Serialize": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkMessage *
        ]
    ),
    "cGcNetworkMonumentSpawnedMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkMonumentSpawnedMessage *
        ]
    ),
    "sGcNetworkOrbUpdateMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # sGcNetworkOrbUpdateMessage *
        ]
    ),
    "cGcNetworkPlayerExperienceSpawnMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerExperienceSpawnMessage *
        ]
    ),
    "sGcNetworkPlayerReplicatedVariablesMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # sGcNetworkPlayerReplicatedVariablesMessage *
        ]
    ),
    "sGcNetworkPlayerStateMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # sGcNetworkPlayerStateMessage *
        ]
    ),
    "sGcNetworkReplicatedCustomisedEntitiesWelcomeMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # sGcNetworkReplicatedCustomisedEntitiesWelcomeMessage *
        ]
    ),
    "sGcNetworkReplicatedCustomisedEntitiesWelcomeMessage::SerializeCustomisedEntity": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # sGcNetworkReplicatedCustomisedEntitiesWelcomeMessage *
            ctypes.c_ulonglong,  # NetReplicatedCustomisationData *
        ]
    ),
    "sGcNetworkReplicatedEntitiesReplicatedVariablesMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # sGcNetworkReplicatedEntitiesReplicatedVariablesMessage *
        ]
    ),
    "sGcNetworkReplicatedEntitiesUpdateMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # sGcNetworkReplicatedEntitiesUpdateMessage *
        ]
    ),
    "sGcNetworkReplicatedEntitiesWelcomeMessage::AddEntity": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # sGcNetworkReplicatedEntitiesWelcomeMessage *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponent *
        ]
    ),
    "sGcNetworkReplicatedEntitiesWelcomeMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # sGcNetworkReplicatedEntitiesWelcomeMessage *
        ]
    ),
    "sGcNetworkReplicatedEntitiesWelcomeMessage::SerializeEntity": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # sGcNetworkReplicatedEntitiesWelcomeMessage *
            ctypes.c_ulonglong,  # SyncedEntityInfo *
        ]
    ),
    "sGcNetworkReplicatedEntityDestroyedMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # sGcNetworkReplicatedEntityDestroyedMessage *
        ]
    ),
    "sGcNetworkReplicatedPetEntitiesWelcomeMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # sGcNetworkReplicatedPetEntitiesWelcomeMessage *
        ]
    ),
    "cGcNetworkRpcMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkRpcMessage *
        ]
    ),
    "cGcNetworkRpcResponseMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkRpcResponseMessage *
        ]
    ),
    "sGcNetworkServerTimeSyncMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # sGcNetworkServerTimeSyncMessage *
        ]
    ),
    "cGcNetworkSynchronisedStateHashMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkSynchronisedStateHashMessage *
        ]
    ),
    "cGcNetworkSyncMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkSyncMessage *
        ]
    ),
    "cGcNetworkHashMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkHashMessage *
        ]
    ),
    "cGcNetworkTerrainEditMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkTerrainEditMessage *
        ]
    ),
    "cGcNetworkTerrainHashMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkTerrainHashMessage *
        ]
    ),
    "cGcNetworkTerrainSyncMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkTerrainSyncMessage *
        ]
    ),
    "cGcNetworkVoiceUpdateMessage::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkVoiceUpdateMessage *
        ]
    ),
    "cGcConnectionPlayfab::SendMessageA": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcConnectionPlayfab *
            ctypes.c_ulonglong,  # const std::shared_ptr<cGcNetworkMessage> *
            ctypes.c_ulonglong,  # std::array<std::shared_ptr<cGcNetworkAddress>,32> *
            ctypes.c_uint32,  # cGcNetworkConstants::TransmissionChannels
            ctypes.c_uint32,  # cGcNetworkConstants::LobbyType
        ]
    ),
    "cGcLobbyId::cGcLobbyId": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLobbyId *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "PartyGetWindowsDllExports": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const PARTY_FUNCTIONS **
        ]
    ),
    "Party::PartyNetwork::LeaveNetwork": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # Party::PartyNetwork *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "Party::PartyNetwork::SetCustomContext": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # Party::PartyNetwork *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "Party::PartyLocalChatControl::SetPermissions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # Party::PartyLocalChatControl *
            ctypes.c_ulonglong,  # const Party::PartyChatControl *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "Party::PartyLocalChatControl::SetAudioInputMuted": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # Party::PartyLocalChatControl *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "Party::PartyManager::GetSingleton": FUNCDEF(
        restype=ctypes.c_ulonglong,  # Party::PartyManager *
        argtypes=[]
    ),
    "Party::PartyManager::Initialize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # Party::PartyManager *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcMatchmaking::Session::SetOwner": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMatchmaking::Session *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcCreatureComponent::IsFemale": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::GetId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcVoiceChatPlayfab::OnStartVoice": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcVoiceChatPlayfab *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cGcVoiceChatPlayfab::OnStopVoice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVoiceChatPlayfab *
        ]
    ),
    "cGcVoiceChatPlayfab::OnPlayerJoined": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVoiceChatPlayfab *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcVoiceChatPlayfab::OnPlayerLeaving": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVoiceChatPlayfab *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "cGcVoiceChatPlayfab::OnMute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVoiceChatPlayfab *
            ctypes.c_ulonglong,  # const cGcNetworkPlayer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVoiceChatPlayfab::IsTalking": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcVoiceChatPlayfab *
            ctypes.c_ulonglong,  # const cGcNetworkPlayer *
        ]
    ),
    "cGcVoiceChatPlayfab::EnableVoicePermissions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVoiceChatPlayfab *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNetworkPlayer *
        ]
    ),
    "cGcActivitiesTracker::OnGameImported": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActivitiesTracker *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcPlayerStatsGroup> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRichPresence::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRichPresence *
        ]
    ),
    "cGcRichPresence::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRichPresence *
        ]
    ),
    "cGcRichPresence::NotifyPlayerOutside": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRichPresence *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRichPresence::NotifyGameMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRichPresence *
            ctypes.c_uint32,  # ePresetGameMode
        ]
    ),
    "cGcRichPresence::StatChanged": {
        "cGcRichPresence *, const TkID<128> *, __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcRichPresence *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_int64,  # __int64
            ]
        ),
        "cGcRichPresence *, const TkID<128> *, long double": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcRichPresence *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_double,  # long double
            ]
        ),
    },
    "cGcCostManager::CanAfford": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcCostManager::CostAsString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostManager *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostManager::DisplayCost": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcCostManager::GetCostHiddenInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcCostManager::ApplyCost": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostManager::CalcCurrentPirateTribute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostManager *
            ctypes.c_ulonglong,  # const cGcCostPirateTribute *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostManager::GetMoneyListCost": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostManager *
            ctypes.c_ulonglong,  # const cGcCostMoneyList *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostManager::IsCostCurrency": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcCostManager::IsCostProdOrSubstance": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcCostManager::GetItemFromIndexedList": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostManager *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # const cGcCostItemListIndexed *
        ]
    ),
    "cGcCostManager::GetProductFromCost": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostManager *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcCostManager::GetSettlementBuildingCostInfoAndStringForBuilding": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostManager *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # const cGcBuilding *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcIntroFeed::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIntroFeed *
        ]
    ),
    "cGcIntroFeed::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIntroFeed *
        ]
    ),
    "TrigramString::TrigramString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TrigramString *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcMarkovAssembler::Assemble": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkovAssembler *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # const cGcMarkovAssembler::Request *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcPhonotactics::CheckConsonantPacking": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFixedString<127,char> *
        ]
    ),
    "ScanAndDiffuseVulgarityAt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcPhonotactics::VulgaritySweep": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcPhonotactics::MakePetNameCuter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
        ]
    ),
    "cGcNameGenerator::InitWordData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGeneratorWord *
        ]
    ),
    "cGcNameGenerator::GenerateGalacticRegionName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateSolarSystemName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateSpaceStationName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # const cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GeneratePlanetName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateBeaconName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateStructureName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint32,  # const cGcNameGenerator::StructureFlavour
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateLibraryName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint32,  # const eAlienRace
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateNPCName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint32,  # const eAlienRace
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateNPCNameWithTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint32,  # const eAlienRace
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateSettlementNPCName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint32,  # const eAlienRace
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateShipPilotName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint32,  # const eAlienRace
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GeneratePirateFaction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GeneratePirateName": {
        "cGcNameGenerator *, const unsigned __int64, cTkFixedString<127,char> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNameGenerator *
                ctypes.c_uint64,  # const unsigned __int64
                ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ]
        ),
        "cGcNameGenerator *, const eAlienRace, const unsigned __int64, cTkFixedString<127,char> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNameGenerator *
                ctypes.c_uint32,  # const eAlienRace
                ctypes.c_uint64,  # const unsigned __int64
                ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ]
        ),
    },
    "cGcNameGenerator::GeneratePetName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cGcNameGenerator::AddPetNameSuffix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cGcNameGenerator::GenerateWeirdShipName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateShipName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_uint32,  # eShipClass
        ]
    ),
    "cGcNameGenerator::GenerateFreighterName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateFrigateName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateWeaponName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterCaptainName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterOfficerName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNameGenerator::GetSeededPasswordOfficerIndex": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_uint32,  # eAlienRace
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterPassengerName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNameGenerator::GenerateArtifactName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # const cGcProceduralProductData *
            ctypes.c_uint32,  # eRarity
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateExpeditionName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_uint32,  # eExpeditionCategory
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateExpeditionNameGeneral": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateExpeditionNameCombat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateExpeditionNameExplore": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateExpeditionNameMining": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateExpeditionNameTrade": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateCreatureName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateFloraName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateMineralName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateMissionName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_uint32,  # eMissionType
            ctypes.c_uint32,  # eMissionFaction
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GeneratePetProcNoise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GeneratePetFiendProcSentence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateFriendlyDroneBeep": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::AppendAutoLocalisedAlphanumeric": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GetAbandonedFreighterNameFromUA": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterLogSnippet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_uint32,  # eAlienRace
            common.GcSeed,  # cTkSeed
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterDoorText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterCrewManifestIntro": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_uint32,  # eAlienRace
            common.GcSeed,  # cTkSeed
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterCrewManifestScan": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_uint32,  # eAlienRace
            common.GcSeed,  # cTkSeed
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterCrewManifest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_uint32,  # eAlienRace
            common.GcSeed,  # cTkSeed
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterCargoList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterLogFragment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            common.GcSeed,  # cTkSeed
            common.GcSeed,  # cTkSeed
            ctypes.c_uint32,  # eAlienRace
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterCrewApprasial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_uint32,  # eAlienRace
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterSecurityLog": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterCaptainLogLogin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_uint32,  # eAlienRace
            common.GcSeed,  # cTkSeed
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterCaptainLog": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_uint32,  # eAlienRace
            common.GcSeed,  # cTkSeed
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterCaptainLogIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterShipAIOutput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            common.GcSeed,  # cTkSeed
            ctypes.c_uint32,  # eAlienRace
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterResearchNote": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterPersonalItemList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            common.GcSeed,  # cTkSeed
            ctypes.c_uint32,  # eAlienRace
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterMedicalReport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_uint32,  # eAlienRace
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterDisciplineRecord": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_uint32,  # eAlienRace
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterLocationReport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_uint32,  # eAlienRace
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcNameGenerator::GenerateAbandonedFreighterPersonalRecording": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_uint32,  # eAlienRace
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcNameGenerator::GenerateFreighterRoomName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            common.GcSeed,  # cTkSeed
            ctypes.c_char * 0x20,  # TkID<256>
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNameGenerator::GenerateSettlementName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateSettlementStructureArchetypeNameForBuildingClass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint32,  # const eBuildingClass
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateSettlementStructureName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_uint32,  # const eBuildingClass
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::GenerateSettlementPerkName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # const cGcSettlementPerkData *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcNameGenerator::SubstituteDateDetailsIntoString<cTkFixedString<1024,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            common.GcSeed,  # const cTkSeed
        ]
    ),
    "cGcNameGenerator::SubstituteDateDetailsIntoString<cTkFixedString<2048,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
            common.GcSeed,  # const cTkSeed
        ]
    ),
    "cTkLocalisationEntry::~cTkLocalisationEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocalisationEntry *
        ]
    ),
    "cTkStringAssembler::Appendf": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkStringAssembler *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcDebugTextLine::Format": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugTextLine *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "GcAppendConvertedToJapanese": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "GcAppendConvertedToJapaneseAlphanumeric": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "GcConvertFullWidthToHalfWidth": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcRealityManager::IsItemProcFreighterDeployableProduct": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_uint32,  # eProceduralProductCategory
        ]
    ),
    "cGcRealityManager::RenderNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcRepairTechComponent::GenerateTechID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcRealityManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
        ]
    ),
    "cGcRealityManager::GetCurrentRealityIteration": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
        ]
    ),
    "cGcRealityManager::SetHomeRealityIteration": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcRealityManager::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
        ]
    ),
    "cGcRealityManager::LoadMissionSectionIcons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGenericMissionStage> *
        ]
    ),
    "cGcRealityManager::LookupSubstanceByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureSwarm *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupLegacyItemByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcLegacyItem *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupConsumableItemByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcConsumableItem *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupDamageMultiplier": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDamageMultiplierLookup *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupEmoteByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerEmote *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::CollectMatchingSubstances": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_uint32,  # eSubstanceCategory
            ctypes.c_uint32,  # eLegality
            ctypes.c_uint32,  # eRarity
            ctypes.c_ulonglong,  # cTkStackVector<cGcRealitySubstanceData const *,10,-1> *
        ]
    ),
    "cGcRealityManager::LookupDiscoveryWorth": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcDiscoveryWorth *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_uint32,  # eDiscoveryType
        ]
    ),
    "cGcRealityManager::ReplaceTechnology": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_ulonglong,  # const cGcTechnology *
            ctypes.c_ulonglong,  # cGcTechnology *
        ]
    ),
    "cGcRealityManager::IsTitleValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::GetPlayerTitle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerTitle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::GetNextTechnology": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTechnology *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_uint32,  # eTechnologyCategory
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupAlienInteractionByIndex": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcAlienPuzzleEntry *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_int32,  # const int
            ctypes.c_uint32,  # eInteractionType
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_ulonglong,  # std::vector<enum eAlienPuzzleCategory,TkSTLAllocatorShim<enum eAlienPuzzleCategory,4,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eIndexType
            ctypes.c_ulonglong,  # cTkSeed *
        ]
    ),
    "cGcRealityManager::GetAlienPuzzleCategories": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_ulonglong,  # const cGcInteractionComponent *
            ctypes.c_ulonglong,  # std::vector<enum eAlienPuzzleCategory,TkSTLAllocatorShim<enum eAlienPuzzleCategory,4,-1> > *
        ]
    ),
    "cGcRealityManager::GetAlienRaceName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_uint32,  # const eAlienRace
        ]
    ),
    "cGcRealityManager::RegisterBuyTradingItemById": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityManager::RegisterSellTradingItemById": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityManager::ReplenishTradingItemById": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_ulonglong,  # cGcTradingSupplyData *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRealityManager::GetResourceIconFromLookup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkTextureResource *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_ulonglong,  # cTkTextureResource *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::GetPickupIconFromLookup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkTextureResource *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_ulonglong,  # cTkTextureResource *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # eIcon
        ]
    ),
    "cGcRealityManager::GenerateProceduralProduct": {
        "cGcRealityManager *, eProceduralProductCategory, const cTkSeed *, eRarity, eQuality": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcProductData *
            argtypes=[
                ctypes.c_ulonglong,  # cGcRealityManager *
                ctypes.c_uint32,  # eProceduralProductCategory
                ctypes.c_ulonglong,  # const cTkSeed *
                ctypes.c_uint32,  # eRarity
                ctypes.c_uint32,  # eQuality
            ]
        ),
        "cGcRealityManager *, const TkID<128> *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcProductData *
            argtypes=[
                ctypes.c_ulonglong,  # cGcRealityManager *
                ctypes.c_char_p,  # const TkID<128> *
            ]
        ),
    },
    "cGcRealityManager::GetProceduralProductCategory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char,  # char
        ]
    ),
    "cGcRealityManager::IsItemProcFreighterProduct": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char,  # char
        ]
    ),
    "cGcRealityManager::IsItemProcDismantleProduct": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char,  # char
        ]
    ),
    "cGcRealityManager::GetTechIDFromHash": FUNCDEF(
        restype=ctypes.c_char_p,  # const TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::GetHashedIDForTech": FUNCDEF(
        restype=ctypes.c_char_p,  # const TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::GenerateTechPackProduct": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcProductData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char,  # char
        ]
    ),
    "cGcRealityManager::GenerateProceduralTechnologyID": {
        "cGcRealityManager *, TkID<128> *, eProceduralTechnologyCategory, const cTkSeed *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # TkID<128> *
            argtypes=[
                ctypes.c_ulonglong,  # cGcRealityManager *
                ctypes.c_ulonglong,  # TkID<128> *
                ctypes.c_uint32,  # eProceduralTechnologyCategory
                ctypes.c_ulonglong,  # const cTkSeed *
            ]
        ),
        "cGcRealityManager *, TkID<128> *, const TkID<128> *, const cTkSeed *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # TkID<128> *
            argtypes=[
                ctypes.c_ulonglong,  # cGcRealityManager *
                ctypes.c_ulonglong,  # TkID<128> *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ulonglong,  # const cTkSeed *
            ]
        ),
    },
    "cGcRealityManager::GenerateProceduralTechnology": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTechnology *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRealityManager::ClearCachedTechnologiesWithTranslatedText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
        ]
    ),
    "cGcRealityManager::GetAbandonedFreighterSeedFromUA": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSeed *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRealityManager::GetAbandonedFreighterRaceFromUA": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRealityManager::GenerateFreighterPasswordDesc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_uint32,  # eAlienRace
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcRealityManager::GenerateFreighterCaptLogDesc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_uint32,  # eAlienRace
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcRealityManager::GenerateFreighterCrewListDesc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_uint32,  # eAlienRace
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcRealityManager::GenerateRarity": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_uint32,  # eProceduralProductCategory
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcRealityManager::GenerateFreighterTechQuality": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcRealityManager::ConstructRepairTechForTech": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_ulonglong,  # cGcTechnology *
            ctypes.c_ulonglong,  # const cGcTechnology *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRealityManager::ExtractInfoFromProcID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupTechnologyByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTechnology *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupProceduralTechDataByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcProceduralTechnologyData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupProductByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureSwarm *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupRecipeByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRefinerRecipe *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcRealityManager::GetChargeValue": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cGcRealityManager::IsProductEasyToBuy": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupDeployTechnologyFromProduct": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcTechnology *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
            common.GcSeed,  # cTkSeed
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityManager::LookupDeployTechnologyFromProcTechID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTechnology *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # cTkSeed *
        ]
    ),
    "cGcRealityManager::LookupDeployTechCategoryFromProduct": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupDeployTechCategoryFromDeployID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupRewardByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcRewardTableEntry *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupGenericRewardByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcRewardTableItemList *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupInventoryEntryByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryTableEntry *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupPurchasableSpecialByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPurchaseableSpecial *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupUnlockableSeasonRewardByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcUnlockableSeasonReward *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupUnlockableTwitchRewardByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcUnlockableTwitchReward *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupUnlockableTwitchRewardByProductID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcUnlockableTwitchReward *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupUnlockablePlatformRewardByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcUnlockablePlatformReward *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupUnlockablePlatformRewardByProductID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcUnlockablePlatformReward *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupMission": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGenericMissionSequence *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupMissionTable": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupMissionByRewardID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGenericMissionSequence *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupMissionByCostID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGenericMissionSequence *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupMissionByDialogID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcRealityManager::LookupMissionByScanEvent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcRealityManager::LookupBaseStatbyID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryBaseStat *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupItemAverageSellPriceById": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRealityManager::LookUpCostByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCostTableEntry *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupDamageByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerDamageData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupSettlementPerkByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSettlementPerkData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRealityManager::LookupSubstanceBySeed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRealitySubstanceData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            common.GcSeed,  # cTkSeed
            ctypes.c_uint32,  # eSubstanceCategory
            ctypes.c_uint32,  # eRarity
        ]
    ),
    "cGcRealityManager::LookupSubstanceSecondary": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcSubstanceSecondary *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            common.GcSeed,  # cTkSeed
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "CopyToNameStringUtf8LengthLimited": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "WikiItemData::Set": {
        "WikiItemData *, const cGcRealitySubstanceData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # WikiItemData *
                ctypes.c_ulonglong,  # const cGcRealitySubstanceData *
            ]
        ),
        "WikiItemData *, const cGcProductData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # WikiItemData *
                ctypes.c_ulonglong,  # const cGcProductData *
            ]
        ),
        "WikiItemData *, const cGcTechnology *, char": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # WikiItemData *
                ctypes.c_ulonglong,  # const cGcTechnology *
                ctypes.c_char,  # char
            ]
        ),
        "WikiItemData *, const cGcStoneRuneData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # WikiItemData *
                ctypes.c_ulonglong,  # const cGcStoneRuneData *
            ]
        ),
        "WikiItemData *, const cGcRecipeWikiData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # WikiItemData *
                ctypes.c_ulonglong,  # const cGcRecipeWikiData *
            ]
        ),
        "WikiItemData *, const cGcWikiTopic *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # WikiItemData *
                ctypes.c_ulonglong,  # const cGcWikiTopic *
            ]
        ),
        "WikiItemData *, const cGcAlienSpeechEntry *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # WikiItemData *
                ctypes.c_ulonglong,  # const cGcAlienSpeechEntry *
            ]
        ),
    },
    "ShopItemData::GetGalacticAveragePrice": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # ShopItemData *
        ]
    ),
    "cTkMetaData::ValidateData<TkID<128>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkMetadataCache<cGcMissionTable>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionTable *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcMissionTable> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcRealityManagerData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcRealityManagerData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcTechnologyTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTechnologyTypes *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcSubstanceTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSubstanceTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcProductTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcProductTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcProceduralProductTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcProceduralProductTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcProceduralTechnologyTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcProceduralTechnologyTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcLegacyItemTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcLegacyItemTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcTechBoxTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTechBoxTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcConsumableItemTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcConsumableItemTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcRecipeTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcRecipeTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcStoriesTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcStoriesTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcAlienSpeechTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAlienSpeechTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcRewardTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcRewardTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcInventoryTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcCostTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCostTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcPlayerDamageTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerDamageTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcPurchaseableBuildingBlueprints>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPurchaseableBuildingBlueprints *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcUnlockableSeasonRewards>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcUnlockableSeasonRewards *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcItemCostTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcItemCostTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcTradingClassTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTradingClassTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcMaintenanceGroupsTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMaintenanceGroupsTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcUnlockableTrees>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcUnlockableTrees *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcSettlementPerksTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSettlementPerksTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcPlayerWeaponPropertiesTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerWeaponPropertiesTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcCombatEffectsTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCombatEffectsTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcAlienPuzzleTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAlienPuzzleTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcMissionSchedulesTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSchedulesTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcMissionCommunityData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSentinelWaveGroup *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcPlayerTitleData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerTitleData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcGalaxyInfoIcons>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGalaxyInfoIcons *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcPlayerEmoteList>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerEmoteList *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcWiki>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWiki *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcJourney>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcJourney *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityManager::ParseAlienSpeechSpecial<cTkFixedString<1024,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityManager::ParseAlienSpeechSpecial<cTkFixedString<2048,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcMissionTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralTechnologyData::cGcProceduralTechnologyData": {
        "cGcProceduralTechnologyData *, cGcProceduralTechnologyData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcProceduralTechnologyData *
                ctypes.c_ulonglong,  # cGcProceduralTechnologyData *
            ]
        ),
        "cGcProceduralTechnologyData *, const cGcProceduralTechnologyData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcProceduralTechnologyData *
                ctypes.c_ulonglong,  # const cGcProceduralTechnologyData *
            ]
        ),
    },
    "cTkDynamicArray<cGcProceduralTechnologyStatLevel>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcProceduralTechnologyStatLevel> *
            ctypes.c_int32,  # int
        ]
    ),
    "cpToUTF8": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "cTkFixedArray<cGcProductProceduralOnlyData,3>::~cTkFixedArray<cGcProductProceduralOnlyData,3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcProductProceduralOnlyData,3> *
        ]
    ),
    "cTkFixedArray<cGcProceduralProductData,26>::~cTkFixedArray<cGcProceduralProductData,26>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcProceduralProductData,26> *
        ]
    ),
    "cGcNPCPlacementComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcRewardManager::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
        ]
    ),
    "cGcRewardManager::DoDeferredReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            common.GcSeed,  # const cTkSeed
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::StripModelBasedOnRarityOfReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkFixedArray<TkID<128>,3> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::ChooseExpeditionRewards": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardTableItemList *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # std::vector<cGcRewardTableItem const *,TkSTLAllocatorShim<cGcRewardTableItem const *,8,-1> > *
        ]
    ),
    "cGcRewardManager::GiveExpeditionRewards": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # std::vector<cGcRewardTableItem const *,TkSTLAllocatorShim<cGcRewardTableItem const *,8,-1> > *
            ctypes.c_ulonglong,  # std::vector<cTkSeed,TkSTLAllocatorShim<cTkSeed,8,-1> > *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardManager::GetRewardDescription": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardTableItem *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRewardManager::GiveSeasonReward": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardManager::GiveEntitlementReward": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveReward": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # const eRarity
            ctypes.c_uint32,  # const eSizeIndicator
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveCargoReward": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
        ]
    ),
    "cGcRewardManager::GiveFrigateDamageResponseReward": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardFrigateDamageResponse *
        ]
    ),
    "cGcRewardManager::GetSubstanceFromReward": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureSwarm *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # const eRarity
            ctypes.c_uint32,  # const eSizeIndicator
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardManager::GetMultiRewardDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # std::vector<cTkFixedString<128,char>,TkSTLAllocatorShim<cTkFixedString<128,char>,1,-1> > *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::FormatProcProductRewardString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_ulonglong,  # const cGcProductData *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcRewardManager::GetProductFromReward": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureSwarm *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardManager::GetUnitsFromReward": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::GetStandingFromReward": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::WouldNotifyPlayerOfShieldRecharge": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::GetNumberFromReward": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::GetCurrencyInfoFromRewardItem": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardTableItem *
            ctypes.c_ulonglong,  # eCurrency *
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        ]
    ),
    "cGcRewardManager::GiveRewardsFromList": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardTableItemList *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # std::vector<cGcRewardTableItem const *,TkSTLAllocatorShim<cGcRewardTableItem const *,8,-1> > *
            ctypes.c_ulonglong,  # int *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardFromPointer": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cTkClassPointer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::ChooseRewardSubstance": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSubstance *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # eLegality
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const cGcRealitySubstanceData **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRewardManager::GiveRewardSubstance": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSubstance *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const TkAudioObject *
        ]
    ),
    "cGcRewardManager::ChooseRewardSpecificSubstance": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificSubstance *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const cGcRealitySubstanceData **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRewardManager::ChooseRewardSpecificSeasonalSubstance": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificSeasonalSubstance *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const cGcRealitySubstanceData **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRewardManager::ChooseRewardPlanetSubstance": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardPlanetSubstance *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const cGcRealitySubstanceData **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRewardManager::GiveRewardSpecificSubstance": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificSubstance *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const TkAudioObject *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRewardManager::GiveRewardSpecificSeasonalSubstance": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificSeasonalSubstance *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const TkAudioObject *
        ]
    ),
    "cGcRewardManager::GiveRewardPlanetSubstance": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardPlanetSubstance *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const TkAudioObject *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRewardManager::GiveRewardSecondarySubstance": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSecondarySubstance *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const TkAudioObject *
        ]
    ),
    "cGcRewardManager::ChooseRewardProduct": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardProduct *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cGcProductData **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRewardManager::GiveRewardProduct": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardProduct *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const TkAudioObject *
        ]
    ),
    "cGcRewardManager::ChooseRewardSpecificProduct": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificProduct *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cGcProductData **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRewardManager::ChooseRewardSystemSpecificProductFromList": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSystemSpecificProductFromList *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cGcProductData **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRewardManager::ChooseRewardDisguisedProduct": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardDisguisedProduct *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cGcProductData **
            ctypes.c_ulonglong,  # const cGcRealitySubstanceData **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRewardManager::ChooseRewardExchangeProduct": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardExchangeProduct *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cGcProductData **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRewardManager::GetCommunityTierProductFromList": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<TkID<128> > *
        ]
    ),
    "cGcRewardManager::ChooseRewardSpecificCommunityTierProduct": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificCommunityTierProduct *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cGcProductData **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRewardManager::ChooseRewardMultiItemsProcTechProduct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcMultiSpecificItemEntry *
            ctypes.c_ulonglong,  # const cGcProductData **
        ]
    ),
    "cGcRewardManager::ChooseRewardProcTechProduct": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardProcTechProduct *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cGcProductData **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRewardManager::GiveRewardSpecificProduct": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificProduct *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const TkAudioObject *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardRequirementsForRecipe": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardRequirementsForRecipe *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardSpecificCommunityTierProduct": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificCommunityTierProduct *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const TkAudioObject *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardSystemSpecificProductFromList": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSystemSpecificProductFromList *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const TkAudioObject *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardExchangeProduct": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardExchangeProduct *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const TkAudioObject *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardDisguisedProduct": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardDisguisedProduct *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const TkAudioObject *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRewardManager::GiveRewardProcTechProduct": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardProcTechProduct *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const TkAudioObject *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardProceduralProduct": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_uint32,  # const eProceduralProductCategory
            ctypes.c_ubyte,  # const bool
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const TkAudioObject *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # const eRarity
        ]
    ),
    "cGcRewardManager::SendProcProductMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_uint32,  # const eProceduralProductCategory
            ctypes.c_uint32,  # const eRarity
        ]
    ),
    "cGcRewardManager::GiveRewardProceduralTechnology": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_uint32,  # const eProceduralTechnologyCategory
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const TkAudioObject *
        ]
    ),
    "cGcRewardManager::GiveRewardMultiSpecificProducts": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardMultiSpecificProducts *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardMultiSpecificItems": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardMultiSpecificItems *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardStatDiff": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardStatDiff *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const TkAudioObject *
        ]
    ),
    "cGcRewardManager::GiveRewardWorker": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardWorker *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardCrashSiteFly": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardCrashSiteFly *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardCrashSiteRepair": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardCrashSiteRepair *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardShield": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardShield *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::GiveRewardSpecificShip": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificShip *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardSpecificFrigate": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificFrigate *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardAtlasPathProgress": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardAtlasPathProgress *
        ]
    ),
    "cGcRewardManager::GiveRewardModifyStat": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardModifyStat *
        ]
    ),
    "cGcRewardManager::GiveRewardIncrementStat": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardIncrementStat *
        ]
    ),
    "cGcRewardManager::GiveRewardStatCompareAndSet": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardStatCompareAndSet *
        ]
    ),
    "cGcRewardManager::GiveRewardInteractionSeenBitmask": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSetInteractionSeenBitmask *
        ]
    ),
    "cGcRewardManager::GiveRewardShipAssistance": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardShipAssistance *
        ]
    ),
    "cGcRewardManager::GiveRewardTrigger": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardTrigger *
        ]
    ),
    "cGcRewardManager::GiveRewardSetCurrentMission": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSetCurrentMission *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::GiveRewardMission": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardMission *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardCompleteMission": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardCompleteMission *
        ]
    ),
    "cGcRewardManager::GiveRewardCompleteMultiMission": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardCompleteMultiMission *
        ]
    ),
    "cGcRewardManager::GiveRewardMissionSeeded": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardMissionSeeded *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::GiveRewardMissionMessageSeeded": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardMissionMessageSeeded *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRewardManager::GiveRewardSetAbandonedFreighterMissionState": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSetAbandonedFreighterMissionState *
        ]
    ),
    "cGcRewardManager::GiveRewardHealth": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardHealth *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::GiveRewardEnergy": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardEnergy *
        ]
    ),
    "cGcRewardManager::GiveRewardFreeStamina": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardFreeStamina *
        ]
    ),
    "cGcRewardManager::GiveRewardMoney": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardMoney *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::GiveRewardAssessCookedProduct": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardAssessCookedProduct *
        ]
    ),
    "cGcRewardManager::GiveRewardHazard": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardHazard *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::GiveRewardDamage": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardDamage *
        ]
    ),
    "cGcRewardManager::GiveRewardDeath": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardDeath *
        ]
    ),
    "cGcRewardManager::GiveRewardWantedLevel": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardWantedLevel *
        ]
    ),
    "cGcRewardManager::GiveRewardDisableSentinels": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardDisableSentinels *
        ]
    ),
    "cGcRewardManager::GiveRewardEnableSentinels": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardEnableSentinels *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardFrigateFlyby": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardFrigateFlyby *
        ]
    ),
    "cGcRewardManager::GiveRewardCustomPlayerControl": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardCustomPlayerControl *
        ]
    ),
    "cGcRewardManager::GiveRewardStanding": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardStanding *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::GiveRewardFactionStanding": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardFactionStanding *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRewardManager::GiveRewardSecondaryInteractionOptions": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSecondaryInteractionOptions *
        ]
    ),
    "cGcRewardManager::GiveRewardScanEventNearestBuilding": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardScanEventNearestBuilding *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardManager::GiveRewardWeapon": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardWeapon *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::GiveRewardSpecificWeapon": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificWeapon *
        ]
    ),
    "cGcRewardManager::GiveRewardStartPurchase": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
        ]
    ),
    "cGcRewardManager::GiveRewardOpenPage": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardOpenPage *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardManager::GiveRewardOpenFreeFreighter": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardOpenFreeFreighter *
        ]
    ),
    "cGcRewardManager::GiveRewardWikiTopic": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardWikiTopic *
        ]
    ),
    "cGcRewardManager::GiveRewardScan": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardScan *
        ]
    ),
    "cGcRewardManager::GiveRewardSignalScan": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSignalScan *
        ]
    ),
    "cGcRewardManager::GiveRewardTeachWord": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardTeachWord *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::GiveRewardDisplayTechWindow": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardDisplayTechWindow *
        ]
    ),
    "cGcRewardManager::GiveRewardDamageTech": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardDamageTech *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::GiveRewardRepairTech": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardRepairTech *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::GiveRewardRechargeTech": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardRechargeTech *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::ChooseRewardProductRecipe": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardProductRecipe *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cGcProductData **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRewardManager::GiveRewardProductRecipe": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardProductRecipe *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const TkAudioObject *
        ]
    ),
    "cGcRewardManager::GiveRewardSpecificProductRecipe": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificProductRecipe *
            ctypes.c_ulonglong,  # const TkAudioObject *
        ]
    ),
    "cGcRewardManager::GiveRewardForgetSpecificProductRecipe": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardForgetSpecificProductRecipe *
        ]
    ),
    "cGcRewardManager::GiveRewardForgetSpecificTechRecipe": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardForgetSpecificTechRecipe *
        ]
    ),
    "cGcRewardManager::ChooseRewardProductRecipeFromList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificProductRecipeFromList *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cGcProductData **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRewardManager::GiveRewardProductRecipeFromList": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificProductRecipeFromList *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const TkAudioObject *
        ]
    ),
    "cGcRewardManager::GiveRewardMultiSpecificProductRecipe": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardMultiSpecificProductRecipes *
            ctypes.c_ulonglong,  # const TkAudioObject *
        ]
    ),
    "cGcRewardManager::GiveRewardMultiSpecificTechRecipe": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardMultiSpecificTechRecipes *
            ctypes.c_ulonglong,  # const TkAudioObject *
        ]
    ),
    "cGcRewardManager::GiveRewardTechRecipe": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardTechRecipe *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const TkAudioObject *
        ]
    ),
    "cGcRewardManager::GiveRewardSpecificTech": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificTech *
            ctypes.c_ulonglong,  # const TkAudioObject *
        ]
    ),
    "cGcRewardManager::GiveRewardInstallTech": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardInstallTech *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const TkAudioObject *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::ChooseRewardSpecificTechFromList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificTechFromList *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cGcTechnology **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRewardManager::GiveRewardSpecificTechFromList": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificTechFromList *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const TkAudioObject *
        ]
    ),
    "cGcRewardManager::GiveRewardTimeWarp": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
        ]
    ),
    "cGcRewardManager::GiveRewardPortalState": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardAdvancePortalState *
        ]
    ),
    "cGcRewardManager::GiveRewardClosePortal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
        ]
    ),
    "cGcRewardManager::GiveRewardDiscoverRune": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardManager::GiveRewardTeleport": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardTeleport *
        ]
    ),
    "cGcRewardManager::RPCGiveGenericRewardToNetPlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_char * 0x10,  # TkID<128>
            common.GcSeed,  # const cTkSeed
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardCommunityContribution": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardCommunityContribution *
        ]
    ),
    "cGcRewardManager::GiveRewardInterventionResponse": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardInterventionResponse *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::GiveRewardSpecificSpecial": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificSpecial *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardCustomExpeditionLogEntry": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardCustomExpeditionLogEntry *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::GiveRewardShipMessage": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardShipMessage *
        ]
    ),
    "cGcRewardManager::GiveRewardSalvageShip": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSalvageShip *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardManager::GiveRewardJetpackBoost": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardJetpackBoost *
        ]
    ),
    "cGcRewardManager::GiveRewardRefreshHazProt": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardRefreshHazProt *
        ]
    ),
    "cGcRewardManager::GiveRewardUploadBase": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardUploadBase *
        ]
    ),
    "cGcRewardManager::GiveRewardSendChatMessage": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSendChatMessage *
        ]
    ),
    "cGcRewardManager::GiveRewardShipSlot": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardShipSlot *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardWeaponSlot": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardWeaponSlot *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardFreighterSlot": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardFreighterSlot *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardEnableInteractionClass": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardEnableInteractionClass *
        ]
    ),
    "cGcRewardManager::GiveRewardUpgradeShipClass": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardUpgradeShipClass *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardUpgradeWeaponClass": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardUpgradeWeaponClass *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardActivateEncounterSentinels": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardActivateEncounterSentinels *
        ]
    ),
    "cGcRewardManager::GiveRewardSetNexusExitWarpTargetToFireteamMemberUA": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSetNexusExitWarpTargetToFireteamMemberUA *
        ]
    ),
    "cGcRewardManager::GiveRewardAbortTakeoff": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardAbortTakeoff *
        ]
    ),
    "cGcRewardManager::GiveRewardPetAction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardPetAction *
            ctypes.c_ulonglong,  # const TkAudioObject *
        ]
    ),
    "cGcRewardManager::GiveRewardPetEgg": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardPetEgg *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardPetEggHatch": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardPetEggHatch *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardBeginSettlementBuilding": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardBeginSettlementBuilding *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GetAvailableSettlementBuildingForReward": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgModelNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardBeginSettlementBuilding *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cGcSettlementState *
            ctypes.c_ulonglong,  # const cGcBuilding *
        ]
    ),
    "cGcRewardManager::GetBuildingFromReward": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgModelNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::GiveRewardSpecificPetEgg": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSpecificPetEgg *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardOSDMessage": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardOSDMessage *
        ]
    ),
    "cGcRewardManager::GiveRewardRepairWholeInventory": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardRepairWholeInventory *
        ]
    ),
    "cGcRewardManager::GiveRewardUnlockSeasonReward": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardUnlockSeasonReward *
        ]
    ),
    "cGcRewardManager::GiveRewardSettlementProgress": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSettlementProgress *
        ]
    ),
    "cGcRewardManager::GiveRewardSettlementJudgement": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSettlementJudgement *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::GiveRewardSettlementCustomJudgement": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSettlementCustomJudgement *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcRewardManager::DisplaySettlementJudgementAlert": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_uint32,  # const eSettlementJudgementType
        ]
    ),
    "cGcRewardManager::GiveRewardTriggerSettlementJudgement": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardTriggerSettlementJudgement *
        ]
    ),
    "cGcRewardManager::GiveRewardStartSettlementExpedition": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardStartSettlementExpedition *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardEndSettlementExpedition": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardEndSettlementExpedition *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::GiveRewardSwapMultiTool": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardSwapMultiTool *
        ]
    ),
    "cGcRewardManager::GetTechList": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> > *
            ctypes.c_ulonglong,  # const cGcRewardSpecificTechFromList *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcRewardManager::GetProductRecipeList": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> > *
            ctypes.c_ulonglong,  # const cGcRewardSpecificProductRecipeFromList *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcRewardManager::CouldRewardGiveInventoryItem": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ulonglong,  # const cGcRewardTableItem *
        ]
    ),
    "cGcRewardManager::DisplayStandingMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_uint32,  # eMissionFaction
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardManager::GenerateProceduralProductCategoryFromBiome": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_uint32,  # eBiome
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcProceduralProductData::cGcProceduralProductData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductData *
            ctypes.c_ulonglong,  # const cGcProceduralProductData *
        ]
    ),
    "cGcRewardManager::RewardTriggersExchange": {
        "cGcRewardManager *, const TkID<128> *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcRewardManager *
                ctypes.c_char_p,  # const TkID<128> *
            ]
        ),
        "cGcRewardManager *, const cGcRewardTableItemList *, int *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcRewardManager *
                ctypes.c_ulonglong,  # const cGcRewardTableItemList *
                ctypes.c_ulonglong,  # int *
            ]
        ),
    },
    "cGcRewardManager::OnExchangeFinished": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRewardManager::DoesRewardGiveProductRecipe": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcRewardManager::DoesRewardGiveTechRecipe": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkDynamicArray<cGcRewardTableItem>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRewardTableItem> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcAlienPuzzleOption>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAlienPuzzleOption> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcProceduralProductWord>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcProceduralProductWord> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNGuiEditor::DoComboBox<eRarity>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eRarity *
            ctypes.c_float,  # float
        ]
    ),
    "cTkBehaviourTree::Broadcast<cGcDoShipReceiveMessage>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_ulonglong,  # const cGcDoShipReceiveMessage *
        ]
    ),
    "cTkBehaviourTree::Broadcast<cGcMessagePetBehaviourEvent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_ulonglong,  # const cGcMessagePetBehaviourEvent *
        ]
    ),
    "cTkMetaMessageMap<2>::Broadcast<cGcMessagePetBehaviourEvent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
            ctypes.c_ulonglong,  # const cGcMessagePetBehaviourEvent *
        ]
    ),
    "cGcCreatureComponent::GetTree": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcScanEventManager::AddEvent": {
        "cGcScanEventManager *, cGcScanEventManager::AddEventResult *, eScanTable, const TkID<256> *, float, bool": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcScanEventManager::AddEventResult *
            argtypes=[
                ctypes.c_ulonglong,  # cGcScanEventManager *
                ctypes.c_ulonglong,  # cGcScanEventManager::AddEventResult *
                ctypes.c_uint32,  # eScanTable
                ctypes.c_char_p,  # const TkID<256> *
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcScanEventManager *, eSignalScanType": FUNCDEF(
            restype=ctypes.c_ulonglong,  # TkID<256> *
            argtypes=[
                ctypes.c_ulonglong,  # cGcScanEventManager *
                ctypes.c_uint32,  # eSignalScanType
            ]
        ),
        "cGcScanEventManager *, cGcScanEventManager::AddEventResult *, eScanTable, cGcScanEventData *, cGcPlayerMissionParticipant *, bool, float, std::pair<TkID<128>,cTkSeed> *, unsigned __int64 *, bool, bool": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cGcScanEventManager::AddEventResult *
            argtypes=[
                ctypes.c_ulonglong,  # cGcScanEventManager *
                ctypes.c_ulonglong,  # cGcScanEventManager::AddEventResult *
                ctypes.c_uint32,  # eScanTable
                ctypes.c_ulonglong,  # cGcScanEventData *
                ctypes.c_ulonglong,  # cGcPlayerMissionParticipant *
                ctypes.c_ubyte,  # bool
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # std::pair<TkID<128>,cTkSeed> *
                ctypes.c_ulonglong,  # unsigned __int64 *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cGcPurchaseableItem::SetItemName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableItem *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcResourceManager::AddResourceForceReload": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSmartResHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceManager *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::WeaponIsLarge": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::IsHitReacting": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetHitReactDir": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::IsOnGround": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::IsInSpace": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::ShouldUseAbandonedFreighterWalk": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetPhysicsMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetFootPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetVelocity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetAimPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetCurrentMoveForce": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetFacingDir": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetWorldUp": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetHeight": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetRace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::IsDying": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetDeathReason": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetEnergy": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetHazard": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::IsJetpacking": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::IsJetpackBoosted": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::IsSliding": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetRidingAttachment": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachmentPtr *
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkAttachmentPtr *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetHandPose": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_uint32,  # eHand
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::IsMeleeBoostActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetDominantHand": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::IsMeleeActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcLocalPlayerCharacterInterface::IsPointing": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcLocalPlayerCharacterInterface::IsInteracting": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcLocalPlayerCharacterInterface::IsRocketBootsActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::IsWaitingForRocketBootsActivation": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetTimeLastUsedJetpack": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetLocalTeleportDestination": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::IsRunning": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::IsUnderwater": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::HasMoveForce": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::HasFired": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::IsPilotingShip": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::IsInVehicle": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetSlopeState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::IsHandTracking": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcLocalPlayerCharacterInterface::SetToPosition": {
        "cGcLocalPlayerCharacterInterface *, const cTkVector3 *, const cTkVector3 *, const cTkVector3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
        "cGcLocalPlayerCharacterInterface *, const cTkPhysRelVec3 *, const cTkVector3 *, const cTkVector3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
    },
    "cGcLocalPlayerCharacterInterface::EnableCharacterPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_ubyte,  # bool
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcLocalPlayerCharacterInterface::EnableCharacterCollision": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_uint32,  # eCharacterPhysicsDisabledReasons
        ]
    ),
    "cGcLocalPlayerCharacterInterface::DisableCharacterCollision": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_uint32,  # eCharacterPhysicsDisabledReasons
        ]
    ),
    "cGcLocalPlayerCharacterInterface::AttachControlMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cGcPlayerControlComponent *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::EnableCharacterGravity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_uint32,  # eCharacterPhysicsDisabledReasons
        ]
    ),
    "cGcLocalPlayerCharacterInterface::DisableCharacterGravity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_uint32,  # eCharacterPhysicsDisabledReasons
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetRigidBody": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRigidBody *
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetCharacterSit": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCharacterSit *
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::TakeDamage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eDamageType
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cGcOwnerConcept *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::SetPitch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_float,  # float
        ]
    ),
    "cGcLocalPlayerCharacterInterface::SetRoll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_float,  # float
        ]
    ),
    "cGcLocalPlayerCharacterInterface::CanLookAt": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::AllowHeadMovement": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
        ]
    ),
    "cGcLocalPlayerCharacterInterface::GetLookAtPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcLocalPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcNetworkInterpolationComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcNetworkInterpolationComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNetworkInterpolationComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hknpDecoratorBroadPhase::addBodies": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerCharacterInterface *
        ]
    ),
    "cGcNetworkPlayerCharacterInterface::GetEnergy": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerCharacterInterface *
        ]
    ),
    "cGcNetworkPlayerCharacterInterface::GetHazard": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerCharacterInterface *
        ]
    ),
    "cGcNetworkPlayerCharacterInterface::IsJetpacking": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerCharacterInterface *
        ]
    ),
    "cGcNetworkPlayerCharacterInterface::IsJetpackBoosted": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerCharacterInterface *
        ]
    ),
    "cGcNetworkPlayerCharacterInterface::IsSliding": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerCharacterInterface *
        ]
    ),
    "cGcNetworkPlayerCharacterInterface::IsUnderwater": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerCharacterInterface *
        ]
    ),
    "cGcNetworkPlayerCharacterInterface::IsHandTracking": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerCharacterInterface *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcNetworkPlayerCharacterInterface::GetMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcNetworkPlayerCharacterInterface::GetPhysicsMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcNetworkPlayerCharacterInterface::GetVelocity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcNetworkPlayerCharacterInterface::GetFacingDir": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcNetworkPlayerCharacterInterface::GetWorldUp": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcNetworkPlayerCharacterInterface::GetRace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISquadronData *
        ]
    ),
    "cGcNetworkPlayerCharacterInterface::GetCurrentMoveForce": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcNetworkPlayerCharacterInterface::GetDominantHand": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerCharacterInterface *
        ]
    ),
    "cGcNetworkPlayerCharacterInterface::GetLookAtPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerCharacterInterface *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcNPCBehaviourFlyAway::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourWithTurning *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cGcNPCBehaviourIlde::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourIlde *
        ]
    ),
    "cGcNPCBehaviourReturnToShip::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourReturnToShip *
        ]
    ),
    "cGcNPCBehaviourChooseInteraction::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourChooseInteraction *
        ]
    ),
    "cGcNPCBehaviourMoveToDestination::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourMoveToDestination *
        ]
    ),
    "cGcNPCBehaviourMoveToDestination::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourMoveToDestination *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cGcNPCBehaviourPerformInteraction::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourPerformInteraction *
        ]
    ),
    "cGcNPCBehaviourPerformInteraction::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourPerformInteraction *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cGcNPCBehaviourLookAtInteresting::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourLookAtInteresting *
        ]
    ),
    "cGcNPCBehaviourLookAtInteresting::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourLookAtInteresting *
        ]
    ),
    "cGcNPCBehaviourReactToPlayer::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourReactToPlayer *
        ]
    ),
    "cGcNPCBehaviourReactToJudgement::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourReactToPlayer *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cGcNPCBehaviourReactToJudgement::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourReactToJudgement *
        ]
    ),
    "cGcNPCBehaviourGreet::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourGreet *
        ]
    ),
    "cGcNPCBehaviourGreet::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourGreet *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cGcNPCBehaviourInteraction::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourInteraction *
        ]
    ),
    "cGcNPCBehaviourInteraction::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourInteraction *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cGcNPCBehaviourFlyAway::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourFlyAway *
        ]
    ),
    "cGcNPCBehaviourFlyAway::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourFlyAway *
        ]
    ),
    "cGcChairComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcChairComponent *
        ]
    ),
    "cGcChairComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcChairComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcChairComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcChairComponent *
        ]
    ),
    "ComponentInstancercGcChairComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcAnimationLayerQueue::GetCurrentAnimation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAnimationLayerQueue *
        ]
    ),
    "cGcNPCPlacementComponent::GetPlacements": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const std::vector<cGcNPCPlacementComponent::NPCPlacement,TkSTLAllocatorShim<cGcNPCPlacementComponent::NPCPlacement,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementComponent *
            ctypes.c_ulonglong,  # const std::vector<cGcNPCPlacementComponent::NPCPlacement,TkSTLAllocatorShim<cGcNPCPlacementComponent::NPCPlacement,8,-1> > *
        ]
    ),
    "cGcNPCBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcNPCBehaviourTree::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
        ]
    ),
    "cGcNPCBehaviourTree::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCBehaviourTree::CalcNavSubDestOffset": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cGcNPCNavGraph::NavGraphPath *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcNPCBehaviourTree::SetDestinationNode": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
            ctypes.c_ulonglong,  # const cGcNavGraphNodeHandle *
        ]
    ),
    "cGcNPCBehaviourTree::GetNumOtherNPCsInNavArea": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
        ]
    ),
    "cGcNPCBehaviourTree::NavigateToNode": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cGcNavGraphNodeHandle *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCBehaviourTree::TerminateNavigation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
        ]
    ),
    "cGcNPCBehaviourTree::NavigationWantsStaticTurn": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcNPCBehaviourTree::BeginNavigating": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
        ]
    ),
    "cGcNPCBehaviourTree::UpdateNavigation": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCBehaviourTree::IsSeated": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
        ]
    ),
    "cGcNPCBehaviourTree::GetNPCSettlementBehaviourState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
        ]
    ),
    "cGcNPCBehaviourTree::DoesNavAreaSatisfySettlementBehaviourAreaProperty": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
            ctypes.c_uint32,  # eNPCSettlementBehaviourAreaProperty
        ]
    ),
    "cGcNPCBehaviourTree::GetPOINodeSelectionWeight": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
            ctypes.c_ulonglong,  # const cGcNavGraphNode *
            ctypes.c_ulonglong,  # const robin_hood::detail::Table<1,80,cGcNavGraphNodeHandle,int,robin_hood::hash<cGcNavGraphNodeHandle,void>,std::equal_to<cGcNavGraphNodeHandle>,-1> *
            ctypes.c_uint32,  # eFreighterNPCType
        ]
    ),
    "GetBuildingClassWeightEntry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourBuildingClassWeightEntry *
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourData *
            ctypes.c_uint32,  # eBuildingClass
            ctypes.c_uint32,  # eNPCSettlementBehaviourState
        ]
    ),
    "cGcNPCBehaviourTree::GetPOITypeSelectionWeight": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eNPCInteractiveObjectType
            ctypes.c_uint32,  # eNPCSettlementBehaviourState
        ]
    ),
    "cGcNPCBehaviourTree::GetNavAreaTransferSelectionWeight": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
            ctypes.c_uint32,  # eNPCSettlementBehaviourState
        ]
    ),
    "cGcNPCBehaviourTree::GetRunProbability": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
            ctypes.c_uint32,  # eNPCSettlementBehaviourState
        ]
    ),
    "cGcNPCBehaviourTree::GetNavNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNavGraphNode *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
            ctypes.c_ulonglong,  # const cGcNavGraphNodeHandle *
        ]
    ),
    "cGcNPCBehaviour::PlayAnimation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviour *
            ctypes.c_ulonglong,  # cGcSimpleAnimationQueue::SimpleAnimNode *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['float'], 'pointer_depth': 1}
        ]
    ),
    "cGcNPCBehaviour::StopAnimation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviour *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCBehaviour::IsSeated": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviour *
        ]
    ),
    "cGcNPCBehaviour::SetPropA": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviour *
            ctypes.c_uint32,  # eNPCProp
        ]
    ),
    "cGcNPCBehaviour::GetGenericAnimation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviour *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cGcNPCBehaviourWithTurning::UpdateTurning": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourWithTurning *
            ctypes.c_float,  # float
        ]
    ),
    "StackContainer<std::vector<cGcNPCBehaviourChooseInteraction::cGcUpdateQueue::cGcQueueEntry,StackAllocator<cGcNPCBehaviourChooseInteraction::cGcUpdateQueue::cGcQueueEntry,32,-1>>,32,-1>::~StackContainer<std::vector<cGcNPCBehaviourChooseInteraction::cGcUpdateQueue::cGcQueueEntry,StackAllocator<cGcNPCBehaviourChooseInteraction::cGcUpdateQueue::cGcQueueEntry,32,-1>>,32,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cGcNPCBehaviourChooseInteraction::cGcUpdateQueue::cGcQueueEntry,StackAllocator<cGcNPCBehaviourChooseInteraction::cGcUpdateQueue::cGcQueueEntry,32,-1> >,32,-1> *
        ]
    ),
    "cGcNPCBehaviourChooseInteraction::cGcUpdateQueue::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourChooseInteraction::cGcUpdateQueue *
        ]
    ),
    "cGcNPCBehaviourChooseInteraction::cGcUpdateQueue::GetQueueEntry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCBehaviourChooseInteraction::cGcUpdateQueue::cGcQueueEntry *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourChooseInteraction::cGcUpdateQueue *
            ctypes.c_uint64,  # cTkAttachmentPtr
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcNPCBehaviourChooseInteraction::cGcNPCBehaviourChooseInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourChooseInteraction *
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
        ]
    ),
    "StackContainer<std::vector<std::pair<cGcNavGraphNodeHandle,float>,StackAllocator<std::pair<cGcNavGraphNodeHandle,float>,512,-1>>,512,-1>::~StackContainer<std::vector<std::pair<cGcNavGraphNodeHandle,float>,StackAllocator<std::pair<cGcNavGraphNodeHandle,float>,512,-1>>,512,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<std::pair<cGcNavGraphNodeHandle,float>,StackAllocator<std::pair<cGcNavGraphNodeHandle,float>,512,-1> >,512,-1> *
        ]
    ),
    "cGcNPCBehaviourChooseInteraction::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourChooseInteraction *
        ]
    ),
    "cGcNPCBehaviourChooseInteraction::OnStop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourChooseInteraction *
        ]
    ),
    "cGcNPCBehaviourChooseInteraction::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourChooseInteraction *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCBehaviourChooseInteraction::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourChooseInteraction *
        ]
    ),
    "cGcNPCBehaviourChooseInteraction::GatherValidDestinations": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourChooseInteraction *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCBehaviourChooseInteraction::WeightDestinations": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourChooseInteraction *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCBehaviourChooseInteraction::SelectDestination": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourChooseInteraction *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCBehaviourChooseInteraction::GeneratePath": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourChooseInteraction *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCBehaviourMoveToDestination::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourMoveToDestination *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCBehaviourMoveToDestination::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourMoveToDestination *
        ]
    ),
    "cGcNPCBehaviourPerformInteraction::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourPerformInteraction *
            ctypes.c_float,  # float
            ctypes.c_double,  # double
        ]
    ),
    "cGcNPCBehaviourPerformInteraction::ChooseNewState": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourPerformInteraction *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectComponent *
        ]
    ),
    "cGcNPCBehaviourPerformInteraction::UpdateConversing": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourPerformInteraction *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCBehaviourPerformInteraction::PlayStateAnimation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourPerformInteraction *
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCBehaviourPerformInteraction::BeginAnimationState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourPerformInteraction *
            ctypes.c_ulonglong,  # const cGcNPCInteractiveObjectComponent *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNPCBehaviourPerformInteraction::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourPerformInteraction *
        ]
    ),
    "cGcNPCBehaviourPerformInteraction::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourPerformInteraction *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcNPCBehaviourGreet::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourGreet *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcNPCBehaviourReactToPlayer::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourReactToPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCBehaviourReactToPlayer::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourReactToPlayer *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcNPCBehaviourReactToJudgement::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourReactToJudgement *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCBehaviourReactToJudgement::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourReactToJudgement *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcNPCBehaviourInteraction::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourInteraction *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCBehaviourInteraction::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourInteraction *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcNPCBehaviourInteraction::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourInteraction *
        ]
    ),
    "cGcNPCBehaviourLookAtInteresting::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourLookAtInteresting *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCBehaviourLookAtInteresting::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourLookAtInteresting *
        ]
    ),
    "cGcNPCBehaviourFlyAway::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourFlyAway *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCBehaviourFlyAway::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourFlyAway *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcNPCBehaviourReturnToShip::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourReturnToShip *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCBehaviourIlde::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourIlde *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCBehaviourIlde::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourIlde *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcNPCSettlementBehaviourData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBehaviourTreeBehaviour::Update": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeBehaviour *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIHail<4,2>::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIHail<4,2> *
        ]
    ),
    "cTkBehaviourTreeBehaviour::ToggleDebugBreak": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeBehaviour *
        ]
    ),
    "cTkBehaviourTreeBehaviour::IsDebugBreakSet": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeBehaviour *
        ]
    ),
    "cGcSky::GetTimeOfDayAtPosition": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkBehaviourTree::AddNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        ]
    ),
    "cTkBehaviourTreeNode::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        ]
    ),
    "cGcPlayer::HasMoveForce": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::IsClenchingFist": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcAudioAreaTriggerComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioAreaTriggerComponent *
        ]
    ),
    "cGcAudioAreaTriggerComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioAreaTriggerComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkGravityComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGravityComponent *
        ]
    ),
    "ComponentInstancercGcAudioAreaTriggerComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcAudioAreaTriggerComponent *
        ]
    ),
    "ComponentInstancercGcAudioAreaTriggerComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcAudioAreaTriggerComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioAreaTriggerComponent *
        ]
    ),
    "cGcAudioAreaTriggerComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioAreaTriggerComponent *
        ]
    ),
    "cGcAudioAreaTriggerComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioAreaTriggerComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAudioAreaTriggerComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioAreaTriggerComponent *
        ]
    ),
    "cGcByteBeatPlayerComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatPlayerComponent *
        ]
    ),
    "cGcByteBeatPlayerComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatPlayerComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcByteBeatPlayerComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatPlayerComponent *
        ]
    ),
    "cGcByteBeatPlayerAudioStream::GetAudioStreamData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatPlayerAudioStream *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "sSketchValue::sSketchValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "sSketchValue::GetAsBool": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "cGcHologramComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHologramComponent *
        ]
    ),
    "cGcHologramComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHologramComponent *
        ]
    ),
    "cGcHologramComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHologramComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcHologramComponent::TrySpawnHologram": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcHologramComponent *
        ]
    ),
    "cGcHologramComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHologramComponent *
        ]
    ),
    "cGcHologramComponent::DoRandomAnimation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHologramComponent *
        ]
    ),
    "cGcHologramComponent::UpdateCreatureState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHologramComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCHabitationComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCHabitationComponent *
        ]
    ),
    "cGcNPCHabitationComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCHabitationComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCHabitationComponent::TrySpawnWorker": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCHabitationComponent *
        ]
    ),
    "cGcNPCHabitationComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCHabitationComponent *
        ]
    ),
    "cGcSpawnedObjectComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnedObjectComponent *
        ]
    ),
    "cGcSpawnedObjectComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnedObjectComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpawnedObjectComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnedObjectComponent *
        ]
    ),
    "ComponentInstancercGcSpawnedObjectComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcSpawnedObjectComponent *
        ]
    ),
    "ComponentInstancercGcShipFlareComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcSpawnedObjectComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcObjectSpawnerComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnerComponent *
        ]
    ),
    "cGcObjectSpawnerComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnerComponent *
        ]
    ),
    "cGcObjectSpawnerComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnerComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcObjectSpawnerComponent::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnerComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcObjectSpawnerComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnerComponent *
        ]
    ),
    "cGcObjectSpawnerComponent::SpawnObject": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnerComponent *
        ]
    ),
    "cGcObjectSpawnerComponent::FindSpawnedObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSpawnedObjectComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnerComponent *
        ]
    ),
    "cGcSpawnedObjectComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnedObjectComponent *
        ]
    ),
    "cGcSpawnedObjectComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnedObjectComponent *
        ]
    ),
    "cGcSpawnedObjectComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnedObjectComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpawnedObjectComponent::NetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnedObjectComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSpawnedObjectComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnedObjectComponent *
        ]
    ),
    "cGcDissolveEffectComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDissolveEffectComponent *
        ]
    ),
    "cGcDissolveEffectComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDissolveEffectComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcDissolveEffectComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcDissolveEffectComponent *
        ]
    ),
    "ComponentInstancercGcDissolveEffectComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcDissolveEffectComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcDissolveEffectComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcDissolveEffectComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDissolveEffectComponent *
        ]
    ),
    "cGcDissolveEffectComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDissolveEffectComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDissolveEffectComponent::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDissolveEffectComponent *
            ctypes.c_float,  # float
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDissolveEffectComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDissolveEffectComponent *
        ]
    ),
    "cGcPlayerEffectsComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEffectsComponent *
        ]
    ),
    "cGcPlayerEffectsComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEffectsComponent *
        ]
    ),
    "cGcPlayerEffectsComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEffectsComponent *
        ]
    ),
    "cGcPlayerEffectsComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEffectsComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerEffectsComponent::UpdateEffects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEffectsComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerEffectsComponent::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEffectsComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerEffectsComponent::TriggerEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEffectsComponent *
            ctypes.c_uint32,  # const ePlayerEffects
            ctypes.c_ubyte,  # const bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerEffectsComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEffectsComponent *
        ]
    ),
    "cGcPlayerEffectsComponent::ShouldFreezePlayerMovement": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEffectsComponent *
        ]
    ),
    "cGcPlayerEffectsComponent::IsEffectRunning": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEffectsComponent *
            ctypes.c_uint32,  # ePlayerEffects
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerEffectsComponent::ReplicateEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcPlayerEffectsComponent::Dissolve::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEffectsComponent::Dissolve *
        ]
    ),
    "cGcPlayerDissapearEffect::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDissapearEffect *
        ]
    ),
    "cGcPlayerDissapearEffect::Start": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDissapearEffect *
        ]
    ),
    "cGcPlayerDissapearEffect::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDissapearEffect *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayerDissapearEffect::End": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDissapearEffect *
        ]
    ),
    "cGcPlayerDissapearEffect::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDissapearEffect *
        ]
    ),
    "cGcPlayerDissapearEffect::FreezePlayerMovement": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDissapearEffect *
        ]
    ),
    "cGcAlienPodComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPodComponent *
        ]
    ),
    "cGcAlienPodComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPodComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcAlienPodComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcAlienPodComponent *
        ]
    ),
    "ComponentInstancercTkCreatureTailComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcAlienPodComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercTkCreatureTailComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcAlienPodComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPodComponent *
        ]
    ),
    "cGcAlienPodComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPodComponent *
        ]
    ),
    "cGcAlienPodComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPodComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAntagonistComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcAntagonistComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
        ]
    ),
    "cGcAntagonistComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcAntagonistComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcAntagonistComponent *
        ]
    ),
    "ComponentInstancercGcAntagonistComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcAntagonistComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcAntagonistComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcAntagonistComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
        ]
    ),
    "cGcAntagonistComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
        ]
    ),
    "cGcAntagonistComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
        ]
    ),
    "cGcAntagonistComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAntagonistComponent::SetPerceptionEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAntagonistComponent::TryAddThreat": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAntagonistComponent::GetBounds": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAABB *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
            ctypes.c_ulonglong,  # cTkAABB *
        ]
    ),
    "cGcAntagonistComponent::GetRigidBody": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRigidBody *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
        ]
    ),
    "cGcAntagonistComponent::CheckPerception": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
            ctypes.c_ulonglong,  # cGcAntagonistComponent::sPerception *
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
        ]
    ),
    "cGcAntagonistComponent::AddThreat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
            ctypes.c_ulonglong,  # cGcAntagonistComponent::sThreat *
        ]
    ),
    "cGcAntagonistComponent::CommunicateThreat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
            ctypes.c_ulonglong,  # cGcAntagonistComponent::sThreat *
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
            ctypes.c_ulonglong,  # const cGcAntagonistFriend *
        ]
    ),
    "cGcAntagonistComponent::GetClampedRayForPerceptionFOV": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cGcAntagonistPerception *
        ]
    ),
    "cGcAtmosphereEntryComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtmosphereEntryComponent *
        ]
    ),
    "cGcAtmosphereEntryComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtmosphereEntryComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcAtmosphereEntryComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcAtmosphereEntryComponent *
        ]
    ),
    "ComponentInstancercGcAtmosphereEntryComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcAtmosphereEntryComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtmosphereEntryComponent *
        ]
    ),
    "cGcAtmosphereEntryComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtmosphereEntryComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAtmosphereEntryComponent::ActiveAtmosphereEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtmosphereEntryComponent *
        ]
    ),
    "cGcTurretComponent::SetTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTurretComponent *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkSketchComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkSketchComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseDefenceComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceComponent *
        ]
    ),
    "cGcBaseDefenceComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBaseDefenceComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceComponent *
        ]
    ),
    "cGcBaseDefenceComponent::GetSearchingUnitTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_uint32,  # eAntagonistGroup
            ctypes.c_ulonglong,  # _WORD *
        ]
    ),
    "cGcBaseDefenceComponent::UpdateBehaviours": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceComponent *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseDefenceComponent::UpdateLasers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBaseDefenceComponent::UpdateTurretReset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceComponent *
            ctypes.c_ulonglong,  # cGcTurretComponent *
        ]
    ),
    "cGcBaseMiniPortalComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcBaseMiniPortalComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBaseMiniPortalComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAbandonedFreighterComponent::GetTypedComponent": {
        "cTkAttachment *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
        "int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkComponent *
            argtypes=[
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcBaseMiniPortalComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponent *
        ]
    ),
    "cGcBaseMiniPortalComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponent *
        ]
    ),
    "cGcBaseMiniPortalComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBaseMiniPortalComponent::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBaseMiniPortalComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponent *
        ]
    ),
    "cGcBaseMiniPortalComponent::FindNextPortalDestination": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponent *
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponent::sPortalDestination *
        ]
    ),
    "cGcBaseMiniPortalComponent::TeleportToNextPortal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponent *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cGcBaseMiniPortalComponent::TeleportObject": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponent *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # const cGcBaseMiniPortalComponent::sPortalDestination *
        ]
    ),
    "cGcBaseMiniPortalComponent::CheckEnabled": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseMiniPortalComponent::GetAbandonedFreighterComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponent *
        ]
    ),
    "cGcPortalMod::PrepareJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcPortalMod::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalMod *
        ]
    ),
    "cGcPlayerComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerComponent *
        ]
    ),
    "cGcBuildingComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcBuildingComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBuildingComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingComponent *
        ]
    ),
    "cGcBuildingComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcBuildingComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcBuildingComponent *
        ]
    ),
    "ComponentInstancercGcBuildingComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcByteBeatSwitchComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSwitchComponent *
        ]
    ),
    "cGcByteBeatSwitchComponent::GatherConnectedByteBeats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSwitchComponent *
            ctypes.c_ulonglong,  # cGcPlayerBasePersistentBuffer *
            ctypes.c_ulonglong,  # std::vector<char const *,TkSTLAllocatorShim<char const *,8,-1> > *
        ]
    ),
    "cGcChairComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcChairComponent *
        ]
    ),
    "cGcChairComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcChairComponent *
        ]
    ),
    "cGcChairComponent::GetReservedLocation": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcChairComponent *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcChairComponent::FindAvailableLocation": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcChairComponent *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcChairComponent::VacateLocation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcChairComponent *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcChairComponent::Sit": {
        "cGcChairComponent *, cGcCharacterSit *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcChairComponent *
                ctypes.c_ulonglong,  # cGcCharacterSit *
            ]
        ),
        "cGcChairComponent *, cGcCharacterSit *, TkHandle": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcChairComponent *
                ctypes.c_ulonglong,  # cGcCharacterSit *
                ctypes.c_uint32,  # TkHandle
            ]
        ),
    },
    "cGcCombatEffectsComponent::RenderNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcCombatEffectsComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsComponent *
        ]
    ),
    "cGcCombatEffectsComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcCombatEffectsComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcCombatEffectsComponent *
        ]
    ),
    "ComponentInstancercGcCombatEffectsComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcCombatEffectsComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcScanEffectComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcCombatEffectsComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsComponent *
        ]
    ),
    "cGcCombatEffectsComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsComponent *
        ]
    ),
    "cGcCombatEffectsComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCombatEffectsComponent::RemoveExpiredEffects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsComponent *
            ctypes.c_ulonglong,  # const std::vector<int,TkSTLAllocatorShim<int,4,-1> > *
        ]
    ),
    "cGcCombatEffectsComponent::HandleCombatEffectsOnImpact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsComponent *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcCombatEffectsComponent::RenderImGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsComponent *
        ]
    ),
    "cGcCreatureAttractorComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureAttractorComponent *
        ]
    ),
    "cGcDistanceScaleComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDistanceScaleComponent *
        ]
    ),
    "cGcDistanceScaleComponent::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDistanceScaleComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDistanceScaleComponent::CalculateScale": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcDistanceScaleComponent *
        ]
    ),
    "cGcEncounterStateComponent::IsAllowedToCreateNewGuards": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterStateComponent *
        ]
    ),
    "cGcEncounterStateComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcEncounterStateComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterStateComponent *
        ]
    ),
    "cGcEncounterStateComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterStateComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEncounterComponent::IsScriptedEncounter": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
        ]
    ),
    "cGcSentinelRobotComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSentinelRobotComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGroundWormComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcGroundWormComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundWormComponent *
        ]
    ),
    "cGcGroundWormComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundWormComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcGroundWormComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcGroundWormComponent *
        ]
    ),
    "ComponentInstancercGcGroundWormComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcGroundWormComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcGroundWormComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcEncounterComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
        ]
    ),
    "cGcEncounterComponent::OnPartDestroyed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
        ]
    ),
    "cGcEncounterComponent::GetNearestEncounterOfType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_uint32,  # eEncounterType
        ]
    ),
    "cGcEncounterComponent::IsGroundWormEncounterActive": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[]
    ),
    "cGcEncounterComponent::IsScriptedEncounterActive": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[]
    ),
    "cGcEncounterComponent::IsEncounterDefeated": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
        ]
    ),
    "cGcEncounterComponent::GetRemainingGuards": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
        ]
    ),
    "cGcEncounterComponent::ApplyScriptedOverride": {
        "cGcEncounterComponent *, const TkID<128> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcEncounterComponent *
                ctypes.c_char_p,  # const TkID<128> *
            ]
        ),
        "cGcEncounterComponent *, const cGcSentinelEncounterOverride *, StackAllocator<cTkSmartResHandle,4,-1> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcEncounterComponent *
                ctypes.c_ulonglong,  # const cGcSentinelEncounterOverride *
                ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
            ]
        ),
    },
    "cGcEncounterComponent::GetWave": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcSentinelSpawnWave *
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcEncounterComponent::AddWaveToSpawn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcEncounterComponent::PickNextWaveFromSequence": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcEncounterComponent::GenerateGuards": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_char * 0x20,  # TkID<256>
        ]
    ),
    "cGcEncounterComponent::SpawnGuard": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
        ]
    ),
    "cGcEncounterComponent::AddSpawnedSentinels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
            ctypes.c_ulonglong,  # std::vector<cTkAttachmentPtr,TkSTLAllocatorShim<cTkAttachmentPtr,8,-1> > *
        ]
    ),
    "cGcEncounterComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcEncounterComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcEncounterComponent::ClearGuards": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcEncounterComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
        ]
    ),
    "cGcEncounterComponent::IsInRange": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
        ]
    ),
    "cGcEncounterComponent::GetOrCreateStateComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponent *
        ]
    ),
    "ComponentInstancercGcEncounterStateComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcEncounterStateComponent *
        ]
    ),
    "ComponentInstancercGcEncounterStateComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcEncounterStateComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterStateComponent *
        ]
    ),
    "cGcEncounterStateComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterStateComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcEncounterStateComponent::NetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterStateComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "sEncounterGuard::FindNodeHandle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sEncounterGuard *
        ]
    ),
    "cGcExpeditionHologramComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionHologramComponent *
        ]
    ),
    "cGcExpeditionHologramComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionHologramComponent *
        ]
    ),
    "cGcExpeditionHologramComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionHologramComponent *
        ]
    ),
    "cGcExpeditionHologramComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionHologramComponent *
            ctypes.c_double,  # double
        ]
    ),
    "cGcExpeditionHologramComponent::GetBaseBuildingData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPersistentBaseEntry *
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionHologramComponent *
        ]
    ),
    "cGcExpeditionHologramComponent::SetExpedition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionHologramComponent *
            ctypes.c_ulonglong,  # cGcFleetExpedition *
        ]
    ),
    "cGcExpeditionHologramComponent::TryToAssignExpedition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionHologramComponent *
        ]
    ),
    "cGcExpeditionHologramComponent::UpdateCaptainHologram": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionHologramComponent *
        ]
    ),
    "cGcExpeditionHologramComponent::UpdateFrigateHologramsInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionHologramComponent *
        ]
    ),
    "cGcExpeditionHologramComponent::UpdateFrigateHologramRotation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionHologramComponent *
            ctypes.c_double,  # double
        ]
    ),
    "cGcExpeditionHologramComponent::UpdateFrigateHologramColours": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionHologramComponent *
        ]
    ),
    "cGcExpeditionHologramComponent::SetFrigateNodeUserData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionHologramComponent *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcFleetHologramComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetHologramComponent *
        ]
    ),
    "cGcFleetHologramComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetHologramComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcFleetHologramComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcFleetHologramComponent *
        ]
    ),
    "ComponentInstancercGcFleetHologramComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentLoadInstancercGcFleetHologramComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentLoadInstancercGcFleetHologramComponent *
        ]
    ),
    "cGcGrabbableComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabbableComponent *
        ]
    ),
    "cGcGrabbableComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabbableComponent *
        ]
    ),
    "cGcGrabbableComponent::GetGrabbable": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabbableComponent *
            ctypes.c_uint32,  # eHand
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const sGrabbedObjectInfo *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # sGrabbedObjectInfo *
        ]
    ),
    "cGcGrabbableComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabbableComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcGrabbableComponent::GetGrabbableData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcGrabbableData *
        argtypes=[
            ctypes.c_ulonglong,  # const sGrabbedObjectInfo *
        ]
    ),
    "cGcGrabPlayerComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabPlayerComponent *
        ]
    ),
    "cGcGrabPlayerComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabPlayerComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcGrabPlayerComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcGrabPlayerComponent *
        ]
    ),
    "ComponentInstancercGcGrabPlayerComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcGrabPlayerComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcGrabPlayerComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcGrabPlayerComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabPlayerComponent *
        ]
    ),
    "cGcGrabPlayerComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabPlayerComponent *
        ]
    ),
    "cGcGrabPlayerComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabPlayerComponent *
        ]
    ),
    "cGcGrabPlayerComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabPlayerComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcGrabPlayerComponent::ReleasePlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabPlayerComponent *
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcDissolveEffectComponent>::UpdateComponentsRender": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcDissolveEffectComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cGcBeamEffect::cGcBeamEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBeamEffect *
            ctypes.c_ulonglong,  # cGcBeamEffect *
        ]
    ),
    "cTkOnPostAnimCallbackForwarder<cTkIKFullBody>::OnPostAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkOnPostAnimCallbackForwarder<cTkIKFullBody> *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cTkPreAdditiveCallbackForwarder<cTkIKFullBody>::PreAdditive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPreAdditiveCallbackForwarder<cTkIKFullBody> *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerEffectsComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkRigidBody::ClosestPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_float,  # float
        ]
    ),
    "cTkRigidBody::GetCenterOfMassWorld": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cTkIKFullBody::DetachConstraint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # cTkIKConstraint *
        ]
    ),
    "cTkIKFullBody::GetBoneIndexFromNode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcHeadConstraint::OnAttach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeadConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
        ]
    ),
    "cGcHeadConstraint::OnDetach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeadConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
        ]
    ),
    "cGcHeadConstraint::PreAdditive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeadConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcHeadConstraint::PreSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeadConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkEulerVector *
        ]
    ),
    "cGcHeadConstraint::GetNumRows": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeadConstraint *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcHeadConstraint::BuildRows": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeadConstraint *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # const cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkEulerVector *
        ]
    ),
    "cGcGrabbableComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "cGcGrabbableComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkIKFullBody::~cTkIKFullBody": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
        ]
    ),
    "cTkPhysicsComponent::SetGraphicsToPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cGcCreatureComponent::GetAudioObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkAudioObject *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # TkAudioObject *
        ]
    ),
    "cGcCreatureComponent::GetFwd": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cTkAnimationComponent::SetBlendWeight": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkAnimBlendHandle *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimationComponent::GetAnimTime": {
        "cTkAnimationComponent *, const cTkAnimInstanceHandle *": FUNCDEF(
            restype=ctypes.c_double,  # double
            argtypes=[
                ctypes.c_ulonglong,  # cTkAnimationComponent *
                ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
            ]
        ),
        "cTkAnimationComponent *, const TkID<128> *": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # cTkAnimationComponent *
                ctypes.c_char_p,  # const TkID<128> *
            ]
        ),
    },
    "cGcPlayerNotifications::IsMissionSelected": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcHUDManager::RemoveInteractionOffScreenRenderer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcGroundWormComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundWormComponent *
        ]
    ),
    "cGcGroundWormComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundWormComponent *
        ]
    ),
    "cGcGroundWormComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundWormComponent *
        ]
    ),
    "cGcGroundWormComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundWormComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcGroundWormComponent::UpdateState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundWormComponent *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcGroundWormComponent::TriggerEmerge": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundWormComponent *
        ]
    ),
    "cGcGroundWormComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundWormComponent *
        ]
    ),
    "cGcHeightAdjustComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeightAdjustComponent *
        ]
    ),
    "cGcHeightAdjustComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeightAdjustComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcHeightAdjustComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcHeightAdjustComponent *
        ]
    ),
    "ComponentInstancercGcHeightAdjustComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcHeightAdjustComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcHeightAdjustComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeightAdjustComponent *
        ]
    ),
    "cGcHeightAdjustComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeightAdjustComponent *
        ]
    ),
    "cGcHeightAdjustComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeightAdjustComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcInteraction::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteraction *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "CAkSpatialAudioVoice::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomiserInteraction *
        ]
    ),
    "cGcDebugEditor::GetSelectedNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugEditor *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcInteractionComponent::IsFreighterCaptainNPCInteraction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::IsFreighterStaffNPCInteraction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::CanRepeatInteraction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::IsSettlementNPCInteraction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::ShouldHide3rdPersonPlayerDuringInteraction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::ScansAsType": {
        "cGcInteractionComponent *, eInteractionType": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcInteractionComponent *
                ctypes.c_uint32,  # eInteractionType
            ]
        ),
        "cGcInteractionComponent *, eInteractionType, eTechnologyCategory": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcInteractionComponent *
                ctypes.c_uint32,  # eInteractionType
                ctypes.c_uint32,  # eTechnologyCategory
            ]
        ),
    },
    "cGcInteractionComponent::GetAudioObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkAudioObject *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::ClearAdditionalOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::AddNPCTrigger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_uint32,  # eNPCTrigger
        ]
    ),
    "cGcShipOwnershipComponent::GetShipClass": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipOwnershipComponent *
        ]
    ),
    "cGcVehicleCheckpointComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcInteractionComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::SetChainedInteractionAndClearIfNecessary": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::UpdatePuzzleOverridingControllingMission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetUpdatePeriod": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcInteractionComponent::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_double,  # double
        ]
    ),
    "cGcInteractionComponent::GetInteractionDialogHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cGcInteractionComponent::GenerateNewRewardSeed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::UpdateRequests": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetModelRendererData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkModelRendererData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcInteractionComponent::GetInteractionIndexRace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_uint32,  # eInteractionType
        ]
    ),
    "cGcInteractionComponent::NetSerializeOnInit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cGcInteractionComponent::GetViewTargetNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcInteractionComponent::GetFocusJoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetNextFocusJoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::DoInteractionEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_uint32,  # eInteractionEvent
        ]
    ),
    "cGcInteractionComponent::CouldGiveRecruit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_uint32,  # eNPCHabitationType
        ]
    ),
    "cGcInteractionComponent::GiveReward": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # const cGcAlienPuzzleOption *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcInteractionComponent::GetInWorldUIScale": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetInWorldUIMinDistOverride": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetInWorldUIForcedOffset": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::UseInWorldUICameraUp": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInteractionComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::CanGiveTheseRewards": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # const cTkDynamicArray<TkID<128> > *
        ]
    ),
    "cGcInteractionComponent::GetBaseBuildingType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::CanInteract": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetInteractDistSq": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::CanExit": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetBackOutToLeaveOption": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcAlienPuzzleOption *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::Attract": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::TriggerOnMaster": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionComponent::IsPersistentInteract": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::SaveInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # cGcInteractionData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionComponent::GetInteractionData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInteractionData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::UpdateInteractionPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::Interact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::BeginHold": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::CancelHold": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::InteractStarted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::SetupCommandPostInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::SetupFrigateCaptainInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::InteractComplete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionComponent::GetModelRes": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        ]
    ),
    "cGcInteractionComponent::OverrideNPCInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_uint32,  # eInteractionType
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcInteractionComponent::CalculateCurrentPuzzle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcAlienPuzzleEntry *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetCurrentModelRendererData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkModelRendererData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetModelViewerRenderData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkModelRendererData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::UsePlayerCam": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::UseAlignToCamera": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::ChangeInteractionCamera": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_int32,  # const int
            ctypes.c_float,  # const float
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcInteractionComponent::IsInteractionNetworkLocking": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::UsesProgressiveDialog": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # const cGcAlienPuzzleEntry *
        ]
    ),
    "cGcInteractionComponent::GetCameraTimeIn": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GivesWeapon": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::PickRandomAnim": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # TkID<128> *
            common.GcSeed,  # cTkSeed
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkAnimationComponent *
        ]
    ),
    "cGcInteractionComponent::GenerateName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GenerateGridPartNameAndDesc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcInteractionComponent::GeneratePlatformIcon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetTitleOverrideNameInfo": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcInteractionComponent::GetOverridenTitle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const TryStoreMode *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GenerateInteractionIcon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GenerateDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetShipPilotName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcInteractionComponent::ClearCost": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::SwitchToSecondaryInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::SwitchToPrimaryInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::UpdateShipInteractionData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::MarkInteractionComplete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const cGcInteractionData *
        ]
    ),
    "cGcInteractionComponent::CreateSecondaryMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::DestroySecondaryMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::SetInteractionTypeOverride": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_uint32,  # const eInteractionType
        ]
    ),
    "cGcInteractionComponent::GetRace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::RefreshPersistency": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::SwitchWeaponMaterials": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionComponent::DoOption": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const cGcInteractionData *
        ]
    ),
    "cGcInteractionComponent::TriggerMoodResponse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_uint32,  # eMood
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionComponent::TriggerPropChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_uint32,  # eNPCProp
        ]
    ),
    "cGcInteractionComponent::GetAdditionalOptions": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cGcAlienPuzzleOption,TkSTLAllocatorShim<cGcAlienPuzzleOption,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::ShouldDoSeason2MysteryOptions": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::ShouldDoSeason4AtlasOption": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::ShouldDoSeason4AFOption": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::ShouldDoSeason7MemoryMilkOption": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetTechShopCategory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetType": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::UpdateBaseBuildingMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcInteractionComponent::DisplayMessageModuleMesage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::SetForcedInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcInteractionComponent::GetPuzzleTableOverride": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # TkID<256> *
        ]
    ),
    "cGcInteractionComponent::GetNewRewardSeed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSeed *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
            ctypes.c_ulonglong,  # cTkSeed *
        ]
    ),
    "cGcInteractionComponent::OnHover": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetFoodProductResultText": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # TkID<256> *
        ]
    ),
    "cGcInteractionComponent::GetStorageContainerType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GenerateCustomFreighterLogText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcInteractionComponent::FreighterTerminalComplete": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetPersistentBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMaintenanceBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcInteractionComponent::GetArticleLocIDForBuildingClass": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # TkID<256> *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcInteractionComponent::GetNPCDeliveryItemName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcNetworkConstants::GetOnlinePlatform": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
        ]
    ),
    "cGcPlayer::IsAttackingPredator": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "ActivationCost::HasCost": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # ActivationCost *
        ]
    ),
    "ActivationCost::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ActivationCost *
        ]
    ),
    "cTkEulerVector::cTkEulerVector": {
        "cTkEulerVector *, const cTkQuaternion *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkEulerVector *
                ctypes.c_ulonglong,  # const cTkQuaternion *
            ]
        ),
        "cTkEulerVector *, const cTkMatrix34 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkEulerVector *
                ctypes.c_ulonglong,  # const cTkMatrix34 *
            ]
        ),
    },
    "cGcCameraManager::ShakeStop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_uint64,  # cTkClassPoolHandle
        ]
    ),
    "cTkIKRotModifyConstraint::OnAttach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKRotModifyConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
        ]
    ),
    "cTkIKRotModifyConstraint::OnDetach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKRotModifyConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
        ]
    ),
    "cTkIKRotModifyConstraint::PreSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKRotModifyConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkEulerVector *
        ]
    ),
    "cTkIKRotModifyConstraint::GetNumRows": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKCenterOfGravityConstraint *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkIKRotModifyConstraint::BuildRows": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKRotModifyConstraint *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # const cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkEulerVector *
        ]
    ),
    "cGcLadderComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLadderComponent *
        ]
    ),
    "cGcLadderComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLadderComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcLadderComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcLadderComponent *
        ]
    ),
    "ComponentInstancercGcLadderComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcLadderComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLadderComponent *
        ]
    ),
    "cGcLadderComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLadderComponent *
        ]
    ),
    "cGcLadderComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLadderComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcLadderComponent::MountLadder": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLadderComponent *
        ]
    ),
    "cGcLandingHelperComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLandingHelperComponent *
        ]
    ),
    "cGcLandingHelperComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLandingHelperComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcLandingHelperComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcLandingHelperComponent *
        ]
    ),
    "ComponentInstancercGcLandingHelperComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkReferenceComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkReferenceComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcLandingHelperComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLandingHelperComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcLookAtComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLookAtComponent *
        ]
    ),
    "cGcLookAtComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLookAtComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcLookAtComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcLookAtComponent *
        ]
    ),
    "ComponentInstancercGcLookAtComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcLookAtComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLookAtComponent *
        ]
    ),
    "cGcLookAtComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLookAtComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPortalComponent::PortalNguiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcLootComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLootComponent *
        ]
    ),
    "cGcLootComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLootComponent *
        ]
    ),
    "cGcLootComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLootComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcLootComponent::Collect": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcLootComponent *
        ]
    ),
    "cGcLootComponent::ActiveDeathPoint": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLootComponent *
        ]
    ),
    "cGcLootComponent::CanTakeLoot": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcLootComponent *
        ]
    ),
    "cGcMaintenanceComponent::SetModelNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcObjectPlacementComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcObjectPlacementComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectPlacementComponent *
        ]
    ),
    "cGcObjectPlacementComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectPlacementComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcObjectPlacementComponent::DefaultRenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectPlacementComponent *
        ]
    ),
    "ComponentInstancercGcObjectPlacementComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcObjectPlacementComponent *
        ]
    ),
    "ComponentInstancercGcObjectPlacementComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcObjectPlacementComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcObjectPlacementComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcMaintenanceComponent::GetInheritedTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcMaintenanceComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMaintenanceComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::CanInteract": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::Interact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::GetName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::GetDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::GetInWorldUIScale": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::GetInWorldUIMinDistOverride": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::GetInWorldUIForcedOffset": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::UseInWorldUICameraUp": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMaintenanceComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::GetMaintenanceSlotPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceComponent::GetModelViewerRenderData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkModelRendererData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::UsePlayerCam": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::UseAlignToCamera": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::RevertAllSlots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::CompleteAllSlots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::Save": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceComponent::DeleteSavedState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::LoadInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
            ctypes.c_ulonglong,  # const cGcMaintenanceContainer *
            ctypes.c_ulonglong,  # cGcInventoryStore *
        ]
    ),
    "cGcMaintenanceComponent::Setup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::Seed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::UpdateRequiresMaintenance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceComponent::GenerateInventoryElement": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
            ctypes.c_ulonglong,  # cGcInventoryElement *
            ctypes.c_ulonglong,  # const cGcMaintenanceElement *
        ]
    ),
    "cGcMaintenanceComponent::CompleteItemRequirements": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
            ctypes.c_uint64,  # cGcInventoryIndex
            ctypes.c_ulonglong,  # const cGcMaintenanceElement *
        ]
    ),
    "cGcMaintenanceComponent::StartDamageTimers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
        ]
    ),
    "cGcMaintenanceComponent::GetSavePosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcMaintenanceComponent::GetPersistancyData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
            ctypes.c_ulonglong,  # cGcMaintenanceContainer *
        ]
    ),
    "StackContainer<std::vector<sStateObjectsSet<cGcMaintenanceContainer>::sPlayerId,StackAllocator<sStateObjectsSet<cGcMaintenanceContainer>::sPlayerId,16,-1>>,16,-1>::~StackContainer<std::vector<sStateObjectsSet<cGcMaintenanceContainer>::sPlayerId,StackAllocator<sStateObjectsSet<cGcMaintenanceContainer>::sPlayerId,16,-1>>,16,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<sStateObjectsSet<cGcMaintenanceContainer>::sPlayerId,StackAllocator<sStateObjectsSet<cGcMaintenanceContainer>::sPlayerId,16,-1> >,16,-1> *
        ]
    ),
    "cGcMaintenanceComponent::PopulateMaintenanceInventoryContainer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponent *
            ctypes.c_ulonglong,  # cGcInventoryContainer *
        ]
    ),
    "cGcChairComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcScanEffectComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEffectComponent *
        ]
    ),
    "cGcScanEffectComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEffectComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcScanEffectComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcScanEffectComponent *
        ]
    ),
    "ComponentInstancercGcScanEffectComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcNPCComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcNPCComponent::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_double,  # double
        ]
    ),
    "cGcNPCBehaviourTree::cGcNPCBehaviourTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCBehaviourTree *
        ]
    ),
    "cGcNPCComponent::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCComponent::GetUpdatePeriod": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::SetIKBodyMass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::ReadyForPlayerInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # cGcPlayerCommon *
        ]
    ),
    "cGcNPCComponent::OnOwnershipTransfer": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcNPCComponent::InitialiseNPCAfterTransfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::Trigger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_uint32,  # const eNPCTrigger
            ctypes.c_int32,  # const int
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcNPCComponent::AllowMovement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
        ]
    ),
    "cGcNPCComponent::SetState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCComponent::SetPropNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_uint32,  # eNPCProp
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcNPCComponent::SetActiveProp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_uint32,  # eNPCProp
        ]
    ),
    "cGcNPCComponent::PlayAnimation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # cGcSimpleAnimationQueue::SimpleAnimNode *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['float'], 'pointer_depth': 1}
        ]
    ),
    "cGcNPCComponent::SetAudioSwitchForRace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::UpdateHologramEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::UpdateMovement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcNPCComponent::UpdateSteeringForces": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_char * 0x10,  # cTkVector3
        ]
    ),
    "cGcNPCComponent::GetNextAnimID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # int *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # cTkAnimationComponent *
        ]
    ),
    "cGcNPCComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::UpdateGraphics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCComponent::HasMoveForce": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::GetRidingAttachment": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachmentPtr *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # cTkAttachmentPtr *
        ]
    ),
    "cGcNPCComponent::GetPhysicsMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcNPCComponent::GetVelocity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcNPCComponent::GetWorldUp": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcNPCComponent::GetHeight": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::GetFacingDir": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcNPCComponent::SetToPosition": {
        "cGcNPCComponent *, const cTkVector3 *, const cTkVector3 *, const cTkVector3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNPCComponent *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
        "cGcNPCComponent *, const cTkPhysRelVec3 *, const cTkVector3 *, const cTkVector3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNPCComponent *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
        "cGcNPCComponent *, bool, const cTkPhysRelVec3 *, const cTkVector3 *, const cTkVector3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNPCComponent *
                ctypes.c_ubyte,  # bool
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
    },
    "cGcNPCComponent::EnableCharacterGravity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_uint32,  # eCharacterPhysicsDisabledReasons
        ]
    ),
    "cGcNPCComponent::DisableCharacterGravity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_uint32,  # eCharacterPhysicsDisabledReasons
        ]
    ),
    "cGcNPCComponent::EnableCharacterPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ubyte,  # bool
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcNPCComponent::EnableCharacterCollision": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_uint32,  # eCharacterPhysicsDisabledReasons
        ]
    ),
    "cGcNPCComponent::DisableCharacterCollision": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_uint32,  # eCharacterPhysicsDisabledReasons
        ]
    ),
    "cGcNPCComponent::GetCurrentMoveForce": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcNPCComponent::UpdateIdle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCComponent::AllowHeadMovement": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::GetLookAtPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcNPCComponent::GetMoveSpeed": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::ZeroSteering": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCComponent::UpdatePerception": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcNPCComponent::GetLookAtNodeForAttachment": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcNPCComponent::GetConversationPartners": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # cTkStackVector<cGcNPCComponent *,16,-1> *
        ]
    ),
    "cGcNPCComponent::GetSomethingToLookAt": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcNPCComponent::NetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCComponent::NetSerializeOnInit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cGcNPCComponent::ShouldKeepAlive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::SwitchToCharacterPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::UpdateDissolveProgress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # cGcNPCComponent::eDissolveMode
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCComponent::GetSettlementState": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCComponent::SetNavigationArea": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
        ]
    ),
    "cGcNPCComponent::GetNavigationArea": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
        ]
    ),
    "cGcNPCPlacementComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementComponent *
        ]
    ),
    "cGcNPCPlacementComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementComponent *
        ]
    ),
    "cGcNPCPlacementComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCPlacementComponent::OverrideAndRevertHireableNPCs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementComponent *
        ]
    ),
    "cGcNPCPlacementComponent::GenerateNPCID": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementComponent *
            ctypes.c_ulonglong,  # const cGcNPCPlacementComponent::NPCPlacement *
        ]
    ),
    "cGcNPCPlacementComponent::GeneratePlacements": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementComponent *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cGcNPCPlacementInfo *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcNPCPlacementComponent::IsControlledByThisSpawner": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementComponent *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "GetOtherAlienRace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCPlacementComponent::AddNPC": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementComponent *
            ctypes.c_ulonglong,  # cGcNPCPlacementComponent::NPCPlacement *
        ]
    ),
    "cGcScanEvent::cGcScanEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEvent *
            ctypes.c_ulonglong,  # const cGcScanEvent *
        ]
    ),
    "cGcMarkerPoint::cGcMarkerPoint": {
        "cGcMarkerPoint *, cGcMarkerPoint *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcMarkerPoint *
                ctypes.c_ulonglong,  # cGcMarkerPoint *
            ]
        ),
        "cGcMarkerPoint *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcMarkerPoint *
            ]
        ),
    },
    "cGcNPCPlacementComponent::SpawnNPC": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # const TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # const cGcNPCManager::NPCResourceRequest *
            ctypes.c_ulonglong,  # const cGcNPCPlacementInfo *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCPlacementComponent::SpawnShipNPC": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachmentPtr *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementComponent *
            ctypes.c_ulonglong,  # cTkAttachmentPtr *
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCPlacementComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementComponent *
        ]
    ),
    "cGcNPCPlacementComponent::RemovePlaced": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementComponent *
        ]
    ),
    "cGcObjectPlacementComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectPlacementComponent *
        ]
    ),
    "cGcObjectPlacementComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectPlacementComponent *
        ]
    ),
    "cGcObjectPlacementComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectPlacementComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcObjectPlacementComponent::RepositionObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectPlacementComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcObjectPlacementComponent::PlaceObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectPlacementComponent *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_float,  # float
        ]
    ),
    "cGcObjectPlacementComponent::IsControlledByThisSpawner": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectPlacementComponent *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcObjectPlacementComponent::PlaceObjectsInGroups": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectPlacementComponent *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_float,  # float
        ]
    ),
    "cGcObjectPlacementComponent::AddObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectPlacementComponent *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cGcSelectableObjectSpawnList *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_float,  # float
        ]
    ),
    "cGcObjectPlacementComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectPlacementComponent *
        ]
    ),
    "cGcPortalComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalComponent *
        ]
    ),
    "cGcPortalComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalComponent *
        ]
    ),
    "cGcPortalComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPortalComponent::WarpPlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalComponent *
        ]
    ),
    "cGcPortalComponent::ActivateRune": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalComponent *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPortalComponent::ActivateRuneInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalComponent *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPortalComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalComponent *
        ]
    ),
    "cGcPortalComponent::IsCommunityPortal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalComponent *
        ]
    ),
    "cGcPortalComponent::ResetTerminal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalComponent *
        ]
    ),
    "cGcPortalComponent::ResetEverythingExceptTerminal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalComponent *
        ]
    ),
    "cGcPortalComponent::DisablePortalInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalComponent *
        ]
    ),
    "cGcSentinelCoverComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelCoverComponent *
        ]
    ),
    "cGcSentinelCoverComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelCoverComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcSentinelCoverComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcSentinelCoverComponent *
        ]
    ),
    "ComponentInstancercGcSentinelCoverComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcSentinelCoverComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcShootableComponent::SetStartingHealth": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSentinelCoverComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelCoverComponent *
        ]
    ),
    "cGcSentinelCoverComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelCoverComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSentinelCoverComponent::ShouldDestroy": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelCoverComponent *
        ]
    ),
    "cGcSentinelCoverComponent::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelCoverComponent *
        ]
    ),
    "cGcSentinelCoverComponent::OnHitCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcSentinelCoverComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelCoverComponent *
        ]
    ),
    "cTkReplicatedVariableTemplated<float>::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableTemplated<float> *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cTkTimerTemplate<0>::Start": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTimerTemplate<0> *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcSynchronisedStateManager::StoreData<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const sMaintenanceSaveKey *
            ctypes.c_ulonglong,  # const cGcMaintenanceContainer *
        ]
    ),
    "cGcSynchronisedStateManager::ClearData<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const sMaintenanceSaveKey *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSynchronisedStateManager::QueryData<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const sMaintenanceSaveKey *
            ctypes.c_ulonglong,  # sStateObjectsSet<cGcMaintenanceContainer> *
        ]
    ),
    "cTkComponent::RegisterReplicatedVariable<bool>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
            ctypes.c_ulonglong,  # cTkReplicatedVariable<bool> *
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::LookupData": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # const sMaintenanceSaveKey *
            ctypes.c_ulonglong,  # sStateObjectsSet<cGcMaintenanceContainer> *
        ]
    ),
    "cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer>::ClearData": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateBuffer<sMaintenanceSaveKey,cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # const sMaintenanceSaveKey *
            ctypes.c_ulonglong,  # cGcMaintenanceContainer *
        ]
    ),
    "cGcSynchronisedStateManager::SendEditMessage<sMaintenanceSaveKey,cGcMaintenanceContainer>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedStateManager *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # eSyncBufferType
            ctypes.c_ulonglong,  # const sMaintenanceSaveKey *
            ctypes.c_ulonglong,  # cGcMaintenanceContainer *
            ctypes.c_ulonglong,  # const cGcMaintenanceContainer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCPlacementInfo::cGcNPCPlacementInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementInfo *
            ctypes.c_ulonglong,  # const cGcNPCPlacementInfo *
        ]
    ),
    "cTkDynamicArray<cGcAlienPuzzleMissionOverride>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAlienPuzzleMissionOverride> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcScanEventManager::LookupBuildingEvent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcScanEvent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # cGcBuilding *
        ]
    ),
    "cGcScanEvent::~cGcScanEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEvent *
        ]
    ),
    "cTkHavokCharacterController::SetClingToTerrain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkHavokCharacterController::SetCollisionFilters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcCreatureSteering::GetMaxForwardInverseDanger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSteering *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcCreatureSteering::AddDanger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSteering *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eSteeringInput
        ]
    ),
    "cGcCreatureSteering::AddEvadeDanger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSteering *
            ctypes.c_uint32,  # eSteeringInput
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureSteering::AddInterest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSteering *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eSteeringInput
        ]
    ),
    "cGcCreatureSteering::GetDirection": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSteering *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCreatureSteering::GetDirection_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSteering *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureSteering::PickDirection_": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSteering *
        ]
    ),
    "cGcCreatureSteering::UpdateSpring": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSteering *
            ctypes.c_float,  # float
        ]
    ),
    "cTkHavokCharacterController::Stepping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkHavokCharacterController::GetUp": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcCreatureComponent::RegisterPersistentGrassPusher": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::GrassPusher *
        ]
    ),
    "cGcCreatureComponent::UnregisterPersistentGrassPusher": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::GrassPusher *
        ]
    ),
    "cGcCreatureSteering::Zero": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSteering *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSteering::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSteering *
        ]
    ),
    "cTkRigidBody::GetMass": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::DisableContactResponse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::AlignAroundAxis": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "HkVectorToTkVector": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "cTkRigidBody::GetAngularFactor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cTkRigidBody::SetAngularFactor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkRigidBody::Align": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerLadderClimb::IsOnLadder": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerLadderClimb *
        ]
    ),
    "cGcRewardManager::GiveGenericRewardFromPosition": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcRealityManager::LookupAlienWordBySeed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcAlienSpeechEntry *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcLadderComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcSimpleInteractionComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
        ]
    ),
    "cGcSimpleInteractionComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
        ]
    ),
    "cGcSimpleInteractionComponent::PreparePersistency": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcSimpleInteractionComponent::ActivateOnPrepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
        ]
    ),
    "cGcSimpleInteractionComponent::PrepareRewards": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcSimpleInteractionComponent::PrepareName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
        ]
    ),
    "cGcSimpleInteractionComponent::PrepareCost": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcSimpleInteractionComponent::BaseBuildingChangeState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSimpleInteractionComponent::GiveResourceHarvesterReward": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSimpleInteractionComponent::GiveBiomeHarvesterReward": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSimpleInteractionComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSimpleInteractionComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
        ]
    ),
    "cGcSimpleInteractionComponent::CanInteract": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
        ]
    ),
    "cGcSimpleInteractionComponent::CanReward": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
        ]
    ),
    "cGcSimpleInteractionComponent::Interact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
        ]
    ),
    "cGcSimpleInteractionComponent::BeginHold": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSimpleInteractionComponent::DoAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
        ]
    ),
    "cGcSimpleInteractionComponent::GetRewardIDWithOverride": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcSimpleInteractionComponent::SaveEmptyInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
            ctypes.c_char * 0x10,  # cTkVector3
        ]
    ),
    "cGcSimpleInteractionComponent::GetName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
        ]
    ),
    "cGcSimpleInteractionComponent::GetInteractionCategoryData": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # cTkColour *
        ]
    ),
    "cGcSimpleInteractionComponent::GetPersistentInteractionPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcSimpleInteractionComponent::GetPersistentBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMaintenanceBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
        ]
    ),
    "cGcSimpleInteractionComponent::GetBuildingProgress": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcSimpleInteractionComponent::UpdateBuildingTimestamp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
        ]
    ),
    "cGcSimpleInteractionComponent::GetInteractDistance": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
        ]
    ),
    "cGcSimpleInteractionComponent::CreateAndResetCosts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
        ]
    ),
    "cGcSimpleInteractionComponent::TimeToFinalBaseBuildingStage": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
        ]
    ),
    "cGcSquadronHologramComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronHologramComponent *
        ]
    ),
    "cGcSquadronHologramComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronHologramComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcSquadronHologramComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcSquadronHologramComponent *
        ]
    ),
    "ComponentInstancercGcSquadronHologramComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcSquadronHologramComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronHologramComponent *
        ]
    ),
    "cGcSquadronHologramComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronHologramComponent *
        ]
    ),
    "cGcSquadronHologramComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronHologramComponent *
        ]
    ),
    "cGcSquadronHologramComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronHologramComponent *
            ctypes.c_double,  # double
        ]
    ),
    "cGcSquadronHologramComponent::SpawnPilotHologram": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronHologramComponent *
        ]
    ),
    "cGcTechnologyAttachmentComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyAttachmentComponent *
        ]
    ),
    "cGcTechnologyAttachmentComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyAttachmentComponent *
        ]
    ),
    "cGcTechnologyAttachmentComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyAttachmentComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcTechnologyAttachmentComponent::IsActive": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyAttachmentComponent *
        ]
    ),
    "cGcThereminComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcThereminComponent *
        ]
    ),
    "cGcThereminComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcThereminComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcThereminComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcThereminComponent *
        ]
    ),
    "ComponentInstancercGcThereminComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcThereminComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcThereminComponent *
        ]
    ),
    "cGcThereminComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcThereminComponent *
        ]
    ),
    "cGcThereminComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcThereminComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcTurretComponent::MoveToFaceTarget": {
        "cGcTurretComponent *, cTkAttachment *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcTurretComponent *
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
        "cGcTurretComponent *, const cTkVector3 *, cTkAttachment *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcTurretComponent *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
    },
    "cGcTurretComponent::OnHit": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcTurretComponent::GetOrigin": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTurretComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcMissileComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcFreighterSyncComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcFreighterSyncComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterSyncComponent *
        ]
    ),
    "cGcFreighterSyncComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterSyncComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcFreighterSyncComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcFreighterSyncComponent *
        ]
    ),
    "ComponentInstancercGcFreighterSyncComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcTurretComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTurretComponent *
        ]
    ),
    "cGcTurretComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTurretComponent *
        ]
    ),
    "cGcTurretComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTurretComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcTurretComponent::CanTarget": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTurretComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcTurretComponent::FireProjectiles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTurretComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcTurretComponent::GetGunData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcProjectileData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTurretComponent *
        ]
    ),
    "cGcTurretComponent::Deactivate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTurretComponent *
        ]
    ),
    "cGcTurretComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTurretComponent *
        ]
    ),
    "cGcTurretComponent::GetTargetCentrePosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTurretComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcWiringSocketComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWiringSocketComponent *
        ]
    ),
    "cGcWiringSocketComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWiringSocketComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcWiringSocketComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcWiringSocketComponent *
        ]
    ),
    "ComponentInstancercGcWiringSocketComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcWiringSocketComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWiringSocketComponent *
        ]
    ),
    "cGcInteractionInterface::InteractionSequence::GotoNextStage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionInterface::InteractionSequence *
        ]
    ),
    "cGcLoopingHelper::Start": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLoopingHelper *
        ]
    ),
    "cGcLoopingHelper::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLoopingHelper *
            ctypes.c_ulonglong,  # cTkAnimationComponent *
        ]
    ),
    "cGcLoopingHelper::Stop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLoopingHelper *
        ]
    ),
    "cGcInteractionInterface::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionInterface *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<std::vector<bool,TkSTLAllocatorShim<bool,1,-1> >,8,-1> *
        ]
    ),
    "cGcInteractionInterface::SetupSequences": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionInterface *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<std::vector<bool,TkSTLAllocatorShim<bool,1,-1> >,8,-1> *
        ]
    ),
    "cGcInteractionInterface::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionInterface *
            ctypes.c_float,  # float
        ]
    ),
    "cGcInteractionInterface::NumOptionsVisible": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionInterface *
        ]
    ),
    "cGcInteractionInterface::ApplyInteractionEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionInterface *
            ctypes.c_uint32,  # eInteractionEvent
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcInteractionInterface::InteractionStage,8,-1> *
        ]
    ),
    "cGcInteractionInterface::RefreshInteractionTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionInterface *
        ]
    ),
    "cGcInteractionInterface::InteractStarted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionInterface *
        ]
    ),
    "cGcInteractionInterface::StopTextTyping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionInterface *
        ]
    ),
    "cGcInteractionInterface::OptionSelected": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionInterface *
            ctypes.c_ulonglong,  # const cGcAlienPuzzleOption *
        ]
    ),
    "cGcInteractionInterface::NextTextStage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionInterface *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "ComponentInstancercGcCustomiserCharacterComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcCustomiserCharacterComponent *
        ]
    ),
    "ComponentInstancercGcCustomiserCharacterComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcPlayerThirdPerson::GetCharacterNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerThirdPerson *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcClothComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcClothComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomiserInteraction::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomiserInteraction *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcCustomiserInteraction::Interact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomiserInteraction *
        ]
    ),
    "cGcCustomiserInteraction::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomiserInteraction *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcCustomiserInteraction::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomiserInteraction *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcCustomiserInteraction::SendMessageA": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomiserInteraction *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cGcCustomiserInteraction::DoInteractionEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomiserInteraction *
            ctypes.c_uint32,  # eInteractionEvent
        ]
    ),
    "cGcCustomiserInteraction::GetViewTargetNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomiserInteraction *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcMPMissionGiverInteraction::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionGiverInteraction *
        ]
    ),
    "cGcMPMissionGiverInteraction::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMPMissionGiverInteraction *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcMPMissionGiverInteraction::UpdateFlag": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # const bool
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcMPMissionGiverInteraction::MissionFlag *
        ]
    ),
    "cGcShipSalvageInteraction::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipSalvageInteraction *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipSalvageInteraction::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipSalvageInteraction *
        ]
    ),
    "cGcShipSalvageInteraction::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipSalvageInteraction *
        ]
    ),
    "cGcShipSalvageInteraction::Explode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipSalvageInteraction *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcShipSalvageInteraction::ClearSalvageShipHologram": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipSalvageInteraction *
        ]
    ),
    "cGcShipSalvageInteraction::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipSalvageInteraction *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcSpecialsShopInteraction::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpecialsShopInteraction *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcSpecialsShopInteraction::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpecialsShopInteraction *
        ]
    ),
    "cGcSpecialsShopInteraction::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpecialsShopInteraction *
        ]
    ),
    "cGcSpecialsShopInteraction::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpecialsShopInteraction *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcSpecialsShopInteraction::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpecialsShopInteraction *
            ctypes.c_float,  # const float
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpecialsShopInteraction::RenderOffscreen2D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpecialsShopInteraction *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkMetadataCache<cGcModelExplosionRules>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEntitlementRewardsTable *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcModelExplosionRules> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcModelExplosionRules>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEntitlementRewardsTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSceneManager::AddSpotlightToNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneManager *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkPhysicsComponent::FindModelOwner": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysicsComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcCreatureAttractorComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcCreatureFeederComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFeederComponent *
        ]
    ),
    "cGcCreatureFeederComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFeederComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureFeederComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFeederComponent *
        ]
    ),
    "cGcCreatureFeederComponent::UpdateAttractor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFeederComponent *
        ]
    ),
    "cGcCreatureFeederComponent::RemoveMeal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFeederComponent *
        ]
    ),
    "cGcCreatureFeederComponent::DispenseBait": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFeederComponent *
        ]
    ),
    "cGcRealityManager::GetBaseTechFromProcId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcCreatureHarvesterComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHarvesterComponent *
        ]
    ),
    "cGcCreatureHarvesterComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHarvesterComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureHarvesterComponent::CanHarvest": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHarvesterComponent *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureHarvesterComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHarvesterComponent *
        ]
    ),
    "cGcCreatureHarvesterComponent::PlayHarvestAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHarvesterComponent *
        ]
    ),
    "cGcCreatureHarvesterComponent::Harvest": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHarvesterComponent *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "sSketchValue::GetAsFloat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_float,  # float
        ]
    ),
    "cGcEggMachineComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "cGcEggMachineComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "cGcEggMachineComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcEggMachineComponent::UpdateEggMachineState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "cGcEggMachineComponent::GetAnimationStateValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sSketchValue *
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # const sSketchValue *
        ]
    ),
    "cGcEggMachineComponent::Interact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "cGcEggMachineComponent::PageOpened": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "cGcEggMachineComponent::HasAnythingInInputs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "cGcEggMachineComponent::HasValidInputInSlot": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
            ctypes.c_uint32,  # const eEggMachineInputs
        ]
    ),
    "cGcEggMachineComponent::GetItemInSlot": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
            ctypes.c_uint32,  # const eEggMachineInputs
        ]
    ),
    "cGcEggMachineComponent::PeekSlotMaxDosageForItem": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
            ctypes.c_uint32,  # eEggMachineInputs
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcEggMachineComponent::ValidateAndUpdateInputCreatureInfo": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "cGcEggMachineComponent::UpdateOuputEgg": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "cGcEggMachineComponent::UpdateGrowthDosage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "cGcEggMachineComponent::UpdateTraitDosage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "cGcEggMachineComponent::UpdateAccessoriesDosage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "cGcEggMachineComponent::UpdateColourDosage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
        ]
    ),
    "cGcEggMachineComponent::UpdateOutputEggScale": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
            ctypes.c_ulonglong,  # const cGcCreatureData *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcEggMachineComponent::UpdateOutputEggTraits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
            ctypes.c_ulonglong,  # const cGcCreatureData *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcEggMachineComponent::UpdateOutputEggAccessories": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponent *
            ctypes.c_ulonglong,  # const cGcCreatureData *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcGeneratorUnitComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratorUnitComponent *
        ]
    ),
    "cGcGeneratorUnitComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratorUnitComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcGeneratorUnitComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratorUnitComponent *
        ]
    ),
    "cGcGeneratorUnitComponent::LookupResourceId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratorUnitComponent *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcGeneratorUnitComponent::GetSystemHooverActiveOutputSlotIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratorUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcRefinerUnitComponent::Setup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::UpdatePlayerInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::CustomSlotUpdate": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cGcRefinerUnitComponent::CustomSlotRequirementsMet": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::IsRefinerFueled": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::GetFuelAmount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::TimeToMakeOne": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
            ctypes.c_ulonglong,  # const cGcRefinerRecipe *
        ]
    ),
    "cGcRefinerUnitComponent::HasAnythingInInputs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::GetCurrentActiveRecipe": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRefinerRecipe *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::GetBestPossibleRecipe": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRefinerRecipe *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
            ctypes.c_ulonglong,  # eRefinerStatus *
        ]
    ),
    "cGcRefinerUnitComponent::GetNumInputSlots": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::GetItemInSlot": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRefinerUnitComponent::Start": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::Stop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::GetProgress": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::StartPickup": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::CompletePickup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::GetDefaultOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
            ctypes.c_ulonglong,  # const cGcRefinerRecipe *
        ]
    ),
    "cGcRefinerUnitComponent::CanMakeRecipe": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
            ctypes.c_ulonglong,  # const cGcRefinerRecipe *
        ]
    ),
    "cGcRefinerUnitComponent::ValidateOutputAndInputSlots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRefinerUnitComponent::IsInInventorySlot": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
        ]
    ),
    "cGcRefinerUnitComponent::CalculateCurrentRecipeOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
            ctypes.c_ulonglong,  # const cGcRefinerRecipe *
        ]
    ),
    "cGcRefinerUnitComponent::FindRawElement": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkFixedArray<int,3> *
        ]
    ),
    "cGcRefinerUnitComponent::GetUsedInputs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponent *
        ]
    ),
    "cGcRepairTechComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcRepairTechComponent::RepairItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechComponent *
        ]
    ),
    "cGcRepairTechComponent::ClearBlockedSlots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechComponent *
        ]
    ),
    "cGcRepairTechComponent::GetCurrentTech": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTechnology *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechComponent *
        ]
    ),
    "cGcRepairTechComponent::IsInstallation": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechComponent *
        ]
    ),
    "cGcRepairTechComponent::DeleteSavedState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechComponent *
        ]
    ),
    "cGcRepairTechComponent::ValidateRepairTechBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechComponent *
        ]
    ),
    "cGcRepairTechComponent::Save": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechComponent *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRepairTechComponent::GetPersistancyData": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechComponent *
            ctypes.c_ulonglong,  # cGcMaintenanceContainer *
        ]
    ),
    "cGcRepairTechComponent::LaunchRepairInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechComponent *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # cGcInventoryIndex
        ]
    ),
    "cGcRepairTechComponent::GetModelViewerRenderData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkModelRendererData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechComponent *
        ]
    ),
    "cGcRocketLockerComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRocketLockerComponent *
        ]
    ),
    "cGcRocketLockerComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRocketLockerComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcOutpostDockNode::ReleaseAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostDockNode *
        ]
    ),
    "cGcOutpostComponent::IsFrigate": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
        ]
    ),
    "cGcOutpostComponent::GetDockIndexAndShipDockingData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcOutpostDockNode::DockingShipData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcOutpostComponent::IsTemporary": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
        ]
    ),
    "cGcOutpostComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcOutpostComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
        ]
    ),
    "cGcOutpostComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
        ]
    ),
    "cGcOutpostComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcOutpostComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOutpostComponent::GetHoldingPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcOutpostComponent::GetBestDockInfo": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcDockingInfo *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOutpostComponent::SetAllDockedShipsVisibility": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOutpostComponent::RequestLanding": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOutpostComponent::ForceJoinOutpost": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOutpostComponent::StartDocked": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcOutpostComponent::TransferDock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcOutpostComponent::OutpostsNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcOutpostComponent::CanAIShipLandHere": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
        ]
    ),
    "cGcOutpostComponent::SummonShip": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcOutpostComponent::GetCurrentOrNextFreeDockShipData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcOutpostDockNode::DockingShipData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcOutpostComponent::LaunchPolice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
        ]
    ),
    "cGcOutpostComponent::GetClosestEmptyDock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcOutpostComponent::IsLandingAreaClear": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
        ]
    ),
    "cGcOutpostComponent::RequestTakeOffAI": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcOutpostComponent::RequestTakeOffPlayer": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcOutpostComponent::GenerateInteriorStep": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcOutpostComponent::LoadInteriorStep": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcOutpostComponent::GetEntrancePoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcOutpostComponent::HasHigherRankingPlayer": FUNCDEF(
        restype=ctypes.c_uint8,  # unsigned __int8
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcOutpostComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
        ]
    ),
    "cGcOutpostDockNode::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostDockNode *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cGcOutpostComponent *
        ]
    ),
    "cGcOutpostDockNode::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostDockNode *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cGcOutpostDockNode::DockingShipData *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eDockState
        ]
    ),
    "cGcOutpostDockNode::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostDockNode *
            ctypes.c_ulonglong,  # cGcOutpostDockNode::DockingShipData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOutpostDockNode::GetDoorMat": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostDockNode *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcOutpostDockNode::IsBuyLocked": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostDockNode *
        ]
    ),
    "cGcOutpostDockNode::DockingShipData::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostDockNode::DockingShipData *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcOutpostDockNode *
        ]
    ),
    "cGcOutpostDockNode::DockingShipData::TakeOff": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostDockNode::DockingShipData *
            ctypes.c_ulonglong,  # cGcOutpostDockNode *
        ]
    ),
    "cGcOutpostDockNode::DockingShipData::PlayTakeoffEffects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostDockNode::DockingShipData *
            ctypes.c_ulonglong,  # cGcOutpostDockNode *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcPetEggTraitModifierOverrideTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPetEggTraitModifierOverrideTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystem::GetNexusExteriorTeleportNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cTkLockFreeQueue<cGcSimulation::DestructionPair>::Push": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkLockFreeQueue<cGcSimulation::DestructionPair> *
            ctypes.c_ulonglong,  # const cGcSimulation::DestructionPair *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystem::GetNexus": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcSolarSystem::GetNexusEntranceTM": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSpaceshipComponent::DisableAlignWaitingForFrosting": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSolarSystem::IsNexusLoaded": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cTkRigidBody::EnableContactResponse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::SetMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cTkPhysRelVec3::Averager::Averager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysRelVec3::Averager *
        ]
    ),
    "cTkClassPool<cTkDynamicGravityControl::cTkGravityOBB,64>::Allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPoolHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPool<cTkDynamicGravityControl::cTkGravityOBB,64> *
            ctypes.c_ulonglong,  # cTkClassPoolHandle *
        ]
    ),
    "cTrackedNode::cTrackedNode": {
        "cTrackedNode *, TkHandle, int, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTrackedNode *
                ctypes.c_uint32,  # TkHandle
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
            ]
        ),
        "cTrackedNode *, const cTrackedNode *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTrackedNode *
                ctypes.c_ulonglong,  # const cTrackedNode *
            ]
        ),
    },
    "FindSubNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_ulonglong,  # const TkHandle *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
        ]
    ),
    "TrackAnimatedNodeAndParents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<cTrackedNode,TkSTLAllocatorShim<cTrackedNode,16,0> > *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cClothPiece::SimP": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
            ctypes.c_int32,  # int
        ]
    ),
    "cClothPiece::RenderP": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcClothComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothComponent *
        ]
    ),
    "cGcClothComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAnimationLayerQueue::GetCurrentAnimationName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimBlendTree *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAnimationLayerQueue *
        ]
    ),
    "cGcPlayerCharacterComponent::GetFullBodyCurrentAnimationName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimBlendTree *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcCharacterInterfaceComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkAudioComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAudioComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkWindManager::WindInfo::WindInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkWindManager::WindInfo *
        ]
    ),
    "ComponentInstancercGcClothComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcClothComponent *
        ]
    ),
    "ComponentInstancercGcClothComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcClothComponent::ConstructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcClothComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcClothComponent::cGcClothComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothComponent *
        ]
    ),
    "GetAxis": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cAxisSpecification *
        ]
    ),
    "RegisterClothComponentForNgui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothComponent *
        ]
    ),
    "cClothPiece::DrawCamAlignedPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cClothPiece::DrawCamAlignedXCross": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcClothComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothComponent *
        ]
    ),
    "cGcClothComponent::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cClothPiece::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
        ]
    ),
    "cGcClothComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothComponent *
        ]
    ),
    "cClothPiece::cClothPiece": {
        "cClothPiece *, cGcClothComponent *, cGcClothPiece *, TkVector_BoundChecked<cTkVector2> *, TkVector_BoundChecked<cTkVector3> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cClothPiece *
                ctypes.c_ulonglong,  # cGcClothComponent *
                ctypes.c_ulonglong,  # cGcClothPiece *
                ctypes.c_ulonglong,  # TkVector_BoundChecked<cTkVector2> *
                ctypes.c_ulonglong,  # TkVector_BoundChecked<cTkVector3> *
            ]
        ),
        "cClothPiece *, const cClothPiece *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cClothPiece *
                ctypes.c_ulonglong,  # const cClothPiece *
            ]
        ),
    },
    "cClothPiece::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
        ]
    ),
    "cGcClothComponent::DebugDrawPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothComponent *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cClothPiece::DebugDrawPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cClothPiece::DebugDraw": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
        ]
    ),
    "cGcClothComponent::ShiftWorld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cClothPiece::CollectTrackedNodes": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
        ]
    ),
    "cClothPiece::GetInitialPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_double,  # double
            ctypes.c_int32,  # int
        ]
    ),
    "cClothPiece::GetPointOfParameter": {
        "cClothPiece *, cTkVector3 *, const cTrackedNodeWithData<cGcClothAttachmentCirlce> *, float, bool": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkVector3 *
            argtypes=[
                ctypes.c_ulonglong,  # cClothPiece *
                ctypes.c_ulonglong,  # cTkVector3 *
                ctypes.c_ulonglong,  # const cTrackedNodeWithData<cGcClothAttachmentCirlce> *
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cClothPiece *, cTkVector3 *, const cTrackedNodeWithData<cGcClothAttachmentLine> *, double": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkVector3 *
            argtypes=[
                ctypes.c_ulonglong,  # cClothPiece *
                ctypes.c_ulonglong,  # cTkVector3 *
                ctypes.c_ulonglong,  # const cTrackedNodeWithData<cGcClothAttachmentLine> *
                ctypes.c_double,  # double
            ]
        ),
    },
    "cClothPiece::AddFixedPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cClothPiece::StoreFixedPointsAndAttractors": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
        ]
    ),
    "cClothPiece::ApplyFixedPoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
            ctypes.c_float,  # float
        ]
    ),
    "cClothPiece::ApplyAttractors": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
            ctypes.c_float,  # float
        ]
    ),
    "cClothPiece::ApplyWindAndExplosions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
            ctypes.c_float,  # float
        ]
    ),
    "cClothPiece::CreateSplines": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const std::_Value_init_tag *
        ]
    ),
    "cClothPiece::UpdateSplines": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
        ]
    ),
    "cClothPiece::ApplyClothDeletionOnSimulation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
        ]
    ),
    "cClothPiece::ApplyClothDeletionOnGraphics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
        ]
    ),
    "cClothPiece::CheckForBetterRemappingPermutation": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
        ]
    ),
    "cClothPiece::RecalculateMayaIndicesMapping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
        ]
    ),
    "cClothPiece::UpdateCollisionObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
            ctypes.c_float,  # float
        ]
    ),
    "cClothPiece::CalcClothMatrix": {
        "cClothPiece *, cTkMatrix34 *, int, int": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cClothPiece *
                ctypes.c_ulonglong,  # cTkMatrix34 *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
            ]
        ),
        "cClothPiece *, cTkMatrix34 *, const cAttachedNode *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cClothPiece *
                ctypes.c_ulonglong,  # cTkMatrix34 *
                ctypes.c_ulonglong,  # const cAttachedNode *
            ]
        ),
    },
    "cClothPiece::SyncSimulationToRenderBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
        ]
    ),
    "cClothPiece::SendRenderBufferToRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cClothPiece::CalculateOneFrameMovement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
        ]
    ),
    "cClothPiece::HeavySimulate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
            ctypes.c_float,  # float
        ]
    ),
    "cGcClothComponent::HeavySimulate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothComponent *
            ctypes.c_float,  # float
        ]
    ),
    "HeavySimulateTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcClothComponent::KickHeavySimulate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cClothPiece::NonSimUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
            ctypes.c_float,  # float
        ]
    ),
    "cGcClothComponent::OnPostAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothComponent *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cClothPiece::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
            ctypes.c_float,  # float
        ]
    ),
    "cGcClothComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerFullBodyIKComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
        ]
    ),
    "cGcPlayerFullBodyIKComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFootConstraint::cGcFootConstraint": {
        "cGcFootConstraint *, const cGcFootConstraint *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcFootConstraint *
                ctypes.c_ulonglong,  # const cGcFootConstraint *
            ]
        ),
        "cGcFootConstraint *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcFootConstraint *
            ]
        ),
    },
    "cGcPlayerFullBodyIKComponentUpdater::UpdateComponentsPostPhysics": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponentUpdater *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "ComponentInstancercGcPlayerFullBodyIKComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcPlayerFullBodyIKComponent *
        ]
    ),
    "ComponentInstancercGcPlayerFullBodyIKComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcPlayerFullBodyIKComponent::ConstructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcPlayerFullBodyIKComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcPlayerFullBodyIKComponent::cGcPlayerFullBodyIKComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcHandTrackingConstraint::PreSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHandTrackingConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkEulerVector *
        ]
    ),
    "cGcHandTrackingConstraint::PostSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHandTrackingConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcHandTrackingConstraint::PostAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHandTrackingConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # AnimTransform *
        ]
    ),
    "cGcHandTrackingConstraint::PostAnimInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHandTrackingConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # AnimTransform *
        ]
    ),
    "cGcPlayerFullBodyIKComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
        ]
    ),
    "cGcPlayerFullBodyIKComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
        ]
    ),
    "cGcPlayerFullBodyIKComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
        ]
    ),
    "cGcPlayerFullBodyIKComponent::NetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerFullBodyIKComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerFullBodyIKComponent::UpdateIKBasedOnCurrentCharacterState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerFullBodyIKComponent::UpdateCurrentSlopeAngle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerFullBodyIKComponent::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerFullBodyIKComponent::SetDesiredWristMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
            ctypes.c_uint32,  # eHand
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerFullBodyIKComponent::BlendToDesiredWristMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
            ctypes.c_uint32,  # eHand
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerFullBodyIKComponent::BlendToDisableHandIk": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
            ctypes.c_uint32,  # eHand
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerFullBodyIKComponent::GetDesiredWristMatrix": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
            ctypes.c_uint32,  # eHand
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerFullBodyIKComponent::GetActualWristMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcPlayerFullBodyIKComponent::DisableHandIK": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcPlayerFullBodyIKComponent::LockBody": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
        ]
    ),
    "cGcPlayerFullBodyIKComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
        ]
    ),
    "cGcPlayerWeaponComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponComponent *
        ]
    ),
    "cGcPlayerWeaponComponent::FindAndSetOwningNetPlayerAttachment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponComponent *
        ]
    ),
    "cGcPlayerWeaponComponent::GatherHoldingLocators": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponComponent *
        ]
    ),
    "cGcPlayerWeaponComponent::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpringAttachmentComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpringAttachmentComponent *
        ]
    ),
    "cGcSpringAttachmentComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpringAttachmentComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcSpringAttachmentComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcSpringAttachmentComponent *
        ]
    ),
    "ComponentInstancercGcSpringAttachmentComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cSpringLink::cSpringLink": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cSpringLink *
            ctypes.c_ulonglong,  # const cSpringLink *
        ]
    ),
    "cSpringLink::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cSpringLink *
        ]
    ),
    "cGcSpringAttachmentComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpringAttachmentComponent *
        ]
    ),
    "cGcSpringAttachmentComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpringAttachmentComponent *
        ]
    ),
    "cGcSpringAttachmentComponent::ShiftWorld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpringAttachmentComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cSpringLink::CollectTrackedNodes": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cSpringLink *
        ]
    ),
    "cSpringLink::ApplyPositionalSpring": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cSpringLink *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_int32,  # int
        ]
    ),
    "cSpringLink::ApplyAngularSpring": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cSpringLink *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSpringAttachmentComponent::OnPostAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpringAttachmentComponent *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cGcSpringAttachmentComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpringAttachmentComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cTkOnPostAnimCallbackForwarder<cGcSpringAttachmentComponent>::OnPostAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkOnPostAnimCallbackForwarder<cGcSpringAttachmentComponent> *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cTkOnPostAnimCallbackForwarder<cGcPlayerFullBodyIKComponent>::OnPostAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkOnPostAnimCallbackForwarder<cGcPlayerFullBodyIKComponent> *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkPreAdditiveCallbackForwarder<cGcPlayerFullBodyIKComponent>::PreAdditive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPreAdditiveCallbackForwarder<cGcPlayerFullBodyIKComponent> *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPlayerCharacterIKStateData>::cTkDynamicArray<cGcPlayerCharacterIKStateData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerCharacterIKStateData> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcPlayerCharacterIKStateData> *
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcClothComponent>::UpdateComponentsRender": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcClothComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkOnPostAnimCallbackForwarder<cGcClothComponent>::OnPostAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkOnPostAnimCallbackForwarder<cGcClothComponent> *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cTkDynamicArray<cGcPlayerCharacterIKStateData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerCharacterIKStateData> *
        ]
    ),
    "cTkDynamicArray<cGcPlayerCharacterIKStateData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerCharacterIKStateData> *
            ctypes.c_int32,  # int
        ]
    ),
    "DampedSpringGeneralSolution<cTkVector3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cClothPiece::~cClothPiece": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cClothPiece *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<std::vector<cGcSlot *,TkSTLAllocatorShim<cGcSlot *,8,-1> >,8,-1> *
        ]
    ),
    "cGcHandTrackingConstraint::cGcHandTrackingConstraint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHandTrackingConstraint *
            ctypes.c_ulonglong,  # const cGcHandTrackingConstraint *
        ]
    ),
    "ToAngularVelocity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkQuaternion *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFootConstraint::UpdateFootHold": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFootConstraint *
        ]
    ),
    "cTkIKJointXZPushConstraint::OnAttach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKJointXZPushConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
        ]
    ),
    "cTkIKJointXZPushConstraint::OnDetach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKJointXZPushConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
        ]
    ),
    "cTkIKJointXZPushConstraint::PreSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKJointXZPushConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cTkIKJointHeightConstraint::GetNumRows": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKJointXZPushConstraint *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkIKJointXZPushConstraint::BuildRows": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKJointXZPushConstraint *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # const cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkEulerVector *
        ]
    ),
    "cTkIKCenterOfGravityConstraint::BuildRows": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKCenterOfGravityConstraint *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # const cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcFootConstraint::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFootConstraint *
        ]
    ),
    "cTkIKJointHeightConstraint::OnAttach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKJointHeightConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
        ]
    ),
    "cTkIKJointHeightConstraint::PreSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKJointHeightConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkEulerVector *
        ]
    ),
    "cTkIKJointHeightConstraint::BuildRows": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKJointHeightConstraint *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # const cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkEulerVector *
        ]
    ),
    "Cross": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "Dot": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "LengthSquared": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "Normalise": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkMultipleReadersLock::UnlockForRead": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMultipleReadersLock *
        ]
    ),
    "cTkMultipleReadersLock::LockForRead": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMultipleReadersLock *
        ]
    ),
    "cTkMultipleReadersLockReadJanitor::cTkMultipleReadersLockReadJanitor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMultipleReadersLockReadJanitor *
            ctypes.c_ulonglong,  # cTkMultipleReadersLock *
        ]
    ),
    "cGcScanEffectComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEffectComponent *
        ]
    ),
    "cGcScanEffectComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEffectComponent *
        ]
    ),
    "cGcScanEffectComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEffectComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcScannableComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannableComponent *
        ]
    ),
    "cGcScannableComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannableComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcScannableComponent::GetPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannableComponent *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcDungeonGenerationParams::~cGcDungeonGenerationParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDungeonGenerationParams *
        ]
    ),
    "cGcAbandonedFreighterComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
        ]
    ),
    "cGcAbandonedFreighterComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAbandonedFreighterComponent::GetDungeonDoorVariable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcAbandonedFreighterComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
        ]
    ),
    "cGcAbandonedFreighterComponent::GenerateFreighterDungeonJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcDungeonGenerationParams::cGcDungeonGenerationParams": {
        "cGcDungeonGenerationParams *, const cGcDungeonGenerationParams *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcDungeonGenerationParams *
                ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ]
        ),
        "cGcDungeonGenerationParams *, cGcDungeonGenerationParams *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcDungeonGenerationParams *
                ctypes.c_ulonglong,  # cGcDungeonGenerationParams *
            ]
        ),
    },
    "cGcAbandonedFreighterComponent::IsDungeonUnlocked": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
        ]
    ),
    "cGcAbandonedFreighterComponent::SetDungeonUnlocked": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAbandonedFreighterComponent::IsInsideFreighterAirLock": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcAbandonedFreighterComponent::IsInsideFreighterDungeonBoundingBox": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcAbandonedFreighterComponent::GetPlayerLocalAbandonedFreighter": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[]
    ),
    "cGcAbandonedFreighterComponent::GetNumberOfRoomsVisited": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
        ]
    ),
    "cGcAbandonedFreighterComponent::GetRoomIndexForPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcAbandonedFreighterComponent::GetRoomLocId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
            ctypes.c_ulonglong,  # TkID<256> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAbandonedFreighterComponent::IsWorldPositionInsideAnyAbandonedFreighter": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcAbandonedFreighterComponent::PickDungeon": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAbandonedFreighterComponent::IsWorldPositionInOBB": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkAABB *
        ]
    ),
    "cGcAbandonedFreighterComponent::GenerateFreighterDungeon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAbandonedFreighterComponent::IsQuestItemInCurrentRoom": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponent *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcCameraShakeComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeComponent *
        ]
    ),
    "cGcCameraShakeComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcCameraShakeComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcCameraShakeComponent *
        ]
    ),
    "ComponentInstancercGcCameraShakeComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcCameraShakeComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeComponent *
        ]
    ),
    "cGcCameraShakeComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCameraShakeComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeComponent *
        ]
    ),
    "cGcCharacterInterfaceComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcCharacterInterfaceComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCharacterInterfaceComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNetworkPlayerCharacterInterface::IsReady": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerCharacterInterface *
        ]
    ),
    "cGcCharacterInterfaceComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterInterfaceComponent *
        ]
    ),
    "cGcCharacterInterfaceComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterInterfaceComponent *
        ]
    ),
    "cGcCharacterInterfaceComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterInterfaceComponent *
        ]
    ),
    "cGcColouriseComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcColouriseComponent *
        ]
    ),
    "cGcColouriseComponent::SetPackColourVecToUserData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkVector4 *
        ]
    ),
    "cGcColouriseComponent::GammaCorrectRGBQuartet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # cTkColour *
        ]
    ),
    "cGcColouriseComponent::PackRGBQuartetToVec4": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector4 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector4 *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcCustomInventoryComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomInventoryComponent *
        ]
    ),
    "cGcDecorationComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDecorationComponent *
        ]
    ),
    "cGcDecorationComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDecorationComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcDecorationComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcDecorationComponent *
        ]
    ),
    "ComponentInstancercGcDecorationComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcDecorationComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDecorationComponent *
        ]
    ),
    "cGcFreighterBaseComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponent *
        ]
    ),
    "cGcFreighterBaseComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponent *
        ]
    ),
    "cGcFreighterBaseComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFreighterBaseComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponent *
        ]
    ),
    "cGcFreighterBaseComponent::GenerateFreighterBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponent *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint32,  # eInventoryClass
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcFreighterBaseComponent::ResetAndLoadFreighterBaseFromFile": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponent *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcFreighterBaseComponent::SpawnFreighterBaseNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponent *
        ]
    ),
    "cGcFreighterBaseComponent::DespawnFreighterBaseNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponent *
        ]
    ),
    "cGcFreighterBaseComponent::ClaimFreighterBase": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterBaseComponent::IsInsideFreighterBaseBoundingBox": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "GetFreighterBaseData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPersistentBase *
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFreighterBaseOptions *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcFreighterBaseComponent::GenerateCivilianFreighterBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponent *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # eInventoryClass
            common.GcSeed,  # const cTkSeed
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcFreighterBaseComponent::PollToGenerateBase": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponent *
            ctypes.c_uint32,  # eInventoryClass
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcMarkerComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerComponent *
        ]
    ),
    "cGcMarkerComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerComponent *
        ]
    ),
    "cGcMarkerComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAnimationLayerQueue::SetCurrentAnimTime": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAnimationLayerQueue *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerCharacterComponent::IsMech": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::OnHit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcPlayerFullBodyIKComponent::GetGroundSlopeAngle": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
        ]
    ),
    "cGcPlayerBackpackBars::SetThirdPersonNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBackpackBars *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcPlayerBackpackBars::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBackpackBars *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerBackpackBars::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBackpackBars *
        ]
    ),
    "cGcPlayerBackpackBars::CalculateBackpackMaterialValue": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerBackpackBars *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerCharacterComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cGcPlayerCharacterComponent::~cGcPlayerCharacterComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::GetPrefixedAnimationName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCharacterComponent::GetAltWeaponAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcPlayerCharacterComponent::ShouldAllowWeaponAimAdjust": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::PlayMeleeAnimation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcPlayerCharacterComponent::PlayAnimation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_ulonglong,  # cTkAnimLayerHandle *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerCharacterComponent::HandleLocomotionAnimationStates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cGcPlayerCharacterComponent::OnAnimationComplete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # cGcAnimationLayerQueue *
        ]
    ),
    "cGcPlayerCharacterComponent::PerformDeathRagdoll": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::SetBackpackCamLookAt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcPlayerCharacterComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerCharacterComponent::GetUpdatePeriod": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerCharacterComponent::UpdateAirborne": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cGcPlayerCharacterComponent::UpdateSwimming": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerCharacterComponent::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerCharacterComponent::ApplyRootMotion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcPlayerCharacterComponent::ApplyCharacterBanking": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::UpdateLookAt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayerCharacterComponent::UpdateBlink": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayerCharacterComponent::UpdateAirborneDetection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayerCharacterComponent::UpdateSteepSlopeDetection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayerCharacterComponent::UpdateFootstepAudioExtertion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayerCharacterComponent::UpdateJetpackEffects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayerCharacterComponent::SetDeathState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::SetCharacterState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_uint32,  # eCharacterState
        ]
    ),
    "cGcPlayerCharacterComponent::MakeSplash": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::DetermineCharacterState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::ClearJetpackEffects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::RefreshJetpackEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::GetCamoEffectStrength": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::GetCamoDissolveTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCamouflageData *
        ]
    ),
    "cGcPlayerCharacterComponent::NetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerCharacterComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::GetCustomPlayerControl": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerControlComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcPlayerControlComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlComponent *
        ]
    ),
    "cGcPlayerControlComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlComponent *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cGcPlayerControlComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlComponent *
        ]
    ),
    "cGcPlayerControlComponent::BeginState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlComponent *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerControlComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerControlComponent::ModifyCameraStick": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcPlayerControlComponent::UpdateControl": {
        "cGcPlayerControlComponent *, float, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerControlComponent *
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
        "cGcPlayerControlComponent *, const cGcCharacterMove *, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerControlComponent *
                ctypes.c_ulonglong,  # const cGcCharacterMove *
                ctypes.c_float,  # float
            ]
        ),
        "cGcPlayerControlComponent *, const cGcCharacterRotate *, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerControlComponent *
                ctypes.c_ulonglong,  # const cGcCharacterRotate *
                ctypes.c_float,  # float
            ]
        ),
    },
    "cGcPlayerControlComponent::UpdateInput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlComponent *
            ctypes.c_ulonglong,  # const cGcPlayerControlInput *
        ]
    ),
    "cGcPlayerControlComponent::InterceptAnalogInput": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlComponent *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcPlayerControlComponent::InterceptButtonInput": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlComponent *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eInputValidation
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcPlayerControlComponent::SetVariable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSimpleIkRecoilComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleIkRecoilComponent *
        ]
    ),
    "cGcSimpleIkRecoilComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleIkRecoilComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcSimpleIkRecoilComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcSimpleIkRecoilComponent *
        ]
    ),
    "ComponentInstancercGcSimpleIkRecoilComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcSimpleIkRecoilComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleIkRecoilComponent *
        ]
    ),
    "cGcSimpleIkRecoilComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleIkRecoilComponent *
        ]
    ),
    "cGcSimpleIkRecoilComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleIkRecoilComponent *
        ]
    ),
    "cGcSimpleIkRecoilComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleIkRecoilComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcTriggerActionComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
        ]
    ),
    "cGcTriggerActionComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcTriggerActionComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
        ]
    ),
    "cGcTriggerActionComponent::ChangeState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
            ctypes.c_ulonglong,  # cGcActionTriggerState *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTriggerActionComponent::UpdateAnimFrameTrigger": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
            ctypes.c_ulonglong,  # cGcAnimFrameEvent *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcTriggerActionComponent::UpdatePlayerNearbyTrigger": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
            ctypes.c_ulonglong,  # cGcPlayerNearbyEvent *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcTriggerActionComponent::HandleTrigger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
            ctypes.c_ulonglong,  # cGcActionTrigger *
        ]
    ),
    "cGcTriggerActionComponent::HandleAudioTrigger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
            ctypes.c_ulonglong,  # cGcPlayAudioAction *
        ]
    ),
    "cGcTriggerActionComponent::HandleParticleTrigger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
            ctypes.c_ulonglong,  # cGcParticleAction *
        ]
    ),
    "cGcTriggerActionComponent::HandleChangeStateTrigger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
            ctypes.c_ulonglong,  # cGcGoToStateAction *
        ]
    ),
    "cGcTriggerActionComponent::RequestChangeState": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTriggerActionComponent::HandleFrigateInteractionTrigger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
            ctypes.c_ulonglong,  # cGcFrigateInteractionAction *
        ]
    ),
    "cGcTriggerActionComponent::HandleDisplayTextTrigger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
            ctypes.c_ulonglong,  # cGcDisplayText *
        ]
    ),
    "cGcTriggerActionComponent::HandlePainTrigger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
            ctypes.c_ulonglong,  # cGcPainAction *
        ]
    ),
    "cGcTriggerActionComponent::HandleHazardTrigger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
            ctypes.c_ulonglong,  # cGcHazardAction *
        ]
    ),
    "cGcTriggerActionComponent::HandleDestroyTrigger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
            ctypes.c_ulonglong,  # cGcDestroyAction *
        ]
    ),
    "cGcTriggerActionComponent::HandleRewardTrigger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
            ctypes.c_ulonglong,  # cGcRewardAction *
        ]
    ),
    "cGcTriggerActionComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
        ]
    ),
    "cGcTriggerActionComponent::GetGlobalAlert": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureAlertData *
        argtypes=[
            ctypes.c_uint32,  # eCreatureType
            ctypes.c_uint32,  # eCreatureType
        ]
    ),
    "cTkMetadataCache<cGcPlayerCharacterStateTable>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkTrailData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcPlayerCharacterStateTable> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcPlayerCharacterStateTable>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cGcPlayerCharacterStateTable> *
        argtypes=[]
    ),
    "cTkDynamicArray<cGcInputActions>::cTkDynamicArray<cGcInputActions>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInputActions> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcInputActions> *
        ]
    ),
    "cTkMetadataCache<cGcWFCBuilding>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWFCBuilding *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcWFCBuilding> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcWFCBuilding>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cGcWFCBuilding> *
        argtypes=[]
    ),
    "cTkDynamicArray<cGcDungeonQuestParams>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDungeonQuestParams> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcDungeonRoomParams>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDungeonRoomParams> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkComponent::RegisterReplicatedVariable<int>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
            ctypes.c_ulonglong,  # cTkReplicatedVariable<int> *
            ctypes.c_ulonglong,  # const int *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcFreighterDungeonsTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFreighterDungeonsTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkComponent::RegisterReplicatedVariable<TkID<128>>": {
        "cTkComponent *, cTkReplicatedVariable<TkID<128> > *, const TkID<128> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkComponent *
                ctypes.c_ulonglong,  # cTkReplicatedVariable<TkID<128> > *
                ctypes.c_char_p,  # const TkID<128> *
            ]
        ),
        "cTkComponent *, cTkReplicatedVariable<cTkVector3> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkComponent *
                ctypes.c_ulonglong,  # cTkReplicatedVariable<cTkVector3> *
            ]
        ),
    },
    "cTkBlackboard::GetValue<cTkVector3>": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboard *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cTkBlackboard::SetValue<cTkVector3 const &>": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboard *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkBlackboard::SetValue<float>": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboard *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const float *
        ]
    ),
    "cGcTriggerActionComponent::CheckTrigger<0>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
            ctypes.c_ulonglong,  # cGcPlayerNearbyEvent *
            ctypes.c_ulonglong,  # cGcCharacterInterface *
        ]
    ),
    "cGcTriggerActionComponent::CheckTrigger<1>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponent *
            ctypes.c_ulonglong,  # cGcPlayerNearbyEvent *
            ctypes.c_ulonglong,  # cGcCharacterInterface *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcWFCBuilding>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWFCBuilding *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimationComponent::GetNumAnimFrames": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkAnimationComponent::GetAnimFrameInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcCameraManager::IsBehaviourDesiredCurrent": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraManager *
            ctypes.c_uint32,  # eCameraBehaviours
        ]
    ),
    "cTkAnimationComponent::SetMirrored": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventManager::HasActiveMarkerType": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_uint32,  # cGcMarkerPoint::eType
        ]
    ),
    "TryGetLocalUserOnlineId": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        ]
    ),
    "IsLocalUser": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcCustomisationComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationComponent *
        ]
    ),
    "cGcCustomiserCharacterComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomiserCharacterComponent *
        ]
    ),
    "cGcCustomiserCharacterComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomiserCharacterComponent *
        ]
    ),
    "cGcCustomiserCharacterComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomiserCharacterComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCustomiserCharacterComponent::SpawnWeapon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomiserCharacterComponent *
        ]
    ),
    "cGcResourceCustomisation::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationData *
            ctypes.c_uint32,  # const eCustomisationDataType
            ctypes.c_ubyte,  # const bool
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcResourceCustomisation::Save": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_ulonglong,  # cGcCharacterCustomisationData *
        ]
    ),
    "cGcResourceCustomisation::LoadFromSaveData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationSaveData *
            ctypes.c_uint32,  # const eCustomisationDataType
            common.GcSeed,  # cTkSeed
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcResourceCustomisation::SaveToSaveData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_ulonglong,  # cGcCharacterCustomisationSaveData *
        ]
    ),
    "cGcResourceCustomisation::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
        ]
    ),
    "cGcResourceCustomisation::SwapDescriptor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcResourceCustomisation::RemoveDescriptorGroupPrefix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcResourceCustomisation::UsingDescriptorGroup": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcResourceCustomisation::SetTextureOption": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcResourceCustomisation::SetTextureOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_ulonglong,  # const std::vector<cTkProceduralTextureChosenOption,TkSTLAllocatorShim<cTkProceduralTextureChosenOption,16,-1> > *
        ]
    ),
    "cGcResourceCustomisation::SetBoneScalePosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcResourceCustomisation::RebuildDescriptorsFromGroups": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
        ]
    ),
    "cGcResourceCustomisation::RebuildTextureOptionsFromChoices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
        ]
    ),
    "cGcResourceCustomisation::CreateGenerationTask": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcProceduralTextureManager::TaskContext *
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcResourceCustomisation::GenerateReplicationData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # NetReplicatedCustomisationData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_ulonglong,  # NetReplicatedCustomisationData *
            ctypes.c_uint32,  # const eCustomisationDataType
        ]
    ),
    "cGcResourceCustomisation::SetFromReplicationData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_ulonglong,  # const NetReplicatedCustomisationData *
        ]
    ),
    "cGcResourceCustomisation::ApplyMeshModifications": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const std::map<TkID<128>,float,std::less<TkID<128> >,std::allocator<std::pair<TkID<128> const ,float> > > *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcResourceCustomisation::SetFromPreset": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # const eCustomisationDataType
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureNavigation::RequestTurnInPlace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureNavigation::IsTurningInPlace": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cGcCreatureNavigation::ClearSteeringMovement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcCreatureNavigation::GetAxis": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcAdjustHeight::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourWait *
        ]
    ),
    "cGcAdjustHeight::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAdjustHeight *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAppear::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAppear *
        ]
    ),
    "cGcAppear::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAppear *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAttracted::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttracted *
        ]
    ),
    "cGcAttracted::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttracted *
        ]
    ),
    "cGcAttracted::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttracted *
        ]
    ),
    "cGcAttracted::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttracted *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAttracted::DoEat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttracted *
        ]
    ),
    "cGcCanFollowPlayer::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCanFollowPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCanSeePlayer::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCanSeePlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCanSeePrey::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCanSeePrey *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCanSenseAttraction::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCanSenseAttraction *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCheckDeath::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCheckDeath *
        ]
    ),
    "cGcShootableComponent::GetDeathHit": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcProjectileImpact *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
        ]
    ),
    "cGcCheckDeath::cGcCheckDeath": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCheckDeath *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # const cGcCreatureHealthData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCheckDeath::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCheckDeath *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCheckDeath::FinalDestruction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCheckDeath *
        ]
    ),
    "cGcCheckTimer::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCheckTimer *
        ]
    ),
    "cGcCheckTimer::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCheckTimer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCombatEffectsComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcCombatEffectsComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCombatEffectsComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureStunned::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureStunned *
        ]
    ),
    "cGcCreatureStunned::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureStunned *
        ]
    ),
    "cGcCreatureStunned::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureStunned *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureStunned::EndStun": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureStunned *
        ]
    ),
    "cGcDamageNearby::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageNearby *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMetadataBehaviourTree::GetBlackboard": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBlackboard *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_uint32,  # eBlackboardCategory
        ]
    ),
    "cGcMetadataBehaviour::GetBlackboard": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBlackboard *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMetadataBehaviour *
            ctypes.c_uint32,  # eBlackboardCategory
        ]
    ),
    "cGcMetadataBehaviour::GetBlackboardOrDefaultValue": {
        "cGcMetadataBehaviour *, cTkBlackboardDefaultValueBool *": FUNCDEF(
            restype=ctypes.c_ubyte,  # _BOOL8
            argtypes=[
                ctypes.c_ulonglong,  # cGcMetadataBehaviour *
                ctypes.c_ulonglong,  # cTkBlackboardDefaultValueBool *
            ]
        ),
        "cGcMetadataBehaviour *, cTkBlackboardDefaultValueFloat *": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # cGcMetadataBehaviour *
                ctypes.c_ulonglong,  # cTkBlackboardDefaultValueFloat *
            ]
        ),
    },
    "cGcBehaviourDetailAnims::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourDetailAnims *
        ]
    ),
    "cGcBehaviourDetailAnims::NGuiRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourDetailAnims *
        ]
    ),
    "cGcBehaviourDetailAnims::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourDetailAnims *
        ]
    ),
    "BehaviourAllocatorcGcBehaviourDetailAnims::Allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourDetailAnimsData *
        argtypes=[
            ctypes.c_ulonglong,  # BehaviourAllocatorcGcBehaviourDetailAnims *
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDetailAnimList::GetRandomAnim": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDetailAnimList *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcDetailAnims::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDetailAnims *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBehaviourDetailAnims::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourDetailAnims *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFollowPlayer::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFollowPlayer *
        ]
    ),
    "cGcFollowPlayer::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFollowPlayer *
        ]
    ),
    "cGcFollowPlayer::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFollowPlayer *
        ]
    ),
    "cGcFollowPlayer::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFollowPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFollowPlayer::DoPoop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFollowPlayer *
        ]
    ),
    "cGcFollowPlayer::MoveToPlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFollowPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "GcFollowRoutineGround::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GcFollowRoutineGround *
        ]
    ),
    "GcFollowRoutineGround::StartStep": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GcFollowRoutineGround *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "GcFollowRoutineGround::PickSpotToEat": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # GcGrazeFollowingRoutine *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "GcFollowRoutineGround::IsFoodNearby": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # GcFollowRoutineGround *
            ctypes.c_float,  # float
        ]
    ),
    "GcFollowRoutineGround::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # GcFollowRoutineGround *
            ctypes.c_float,  # float
        ]
    ),
    "cGcGetNearestPlayer::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGetNearestPlayer *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
        ]
    ),
    "GcGrazeFollowingRoutine::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GcGrazeFollowingRoutine *
        ]
    ),
    "GcGrazeFollowingRoutine::PickSpotToEatAndTestValidity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GcGrazeFollowingRoutine *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "GcGrazeFollowingRoutine::MoveToNextRoutinePositionIfRequired": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # GcGrazeFollowingRoutine *
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
        ]
    ),
    "GcGrazeFollowingRoutine::StartStep": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GcGrazeFollowingRoutine *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "GcGrazeFollowingRoutine::DoEat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GcGrazeFollowingRoutine *
        ]
    ),
    "GcGrazeFollowingRoutine::DoLookAround": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GcGrazeFollowingRoutine *
        ]
    ),
    "GcGrazeFollowingRoutine::DoIdle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GcGrazeFollowingRoutine *
        ]
    ),
    "GcGrazeFollowingRoutine::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # GcGrazeFollowingRoutine *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBehaviourLook::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourLook *
        ]
    ),
    "cGcBehaviourLook::NGuiRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourLook *
        ]
    ),
    "BehaviourAllocatorcGcBehaviourLook::Allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourLookData *
        argtypes=[
            ctypes.c_ulonglong,  # BehaviourAllocatorcGcBehaviourLook *
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLookAtStuff::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLookAtStuff *
        ]
    ),
    "cGcLookAtStuff::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLookAtStuff *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBehaviourLook::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourLook *
        ]
    ),
    "cGcBehaviourLook::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourLook *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMoveToTarget::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMoveToTarget *
        ]
    ),
    "cGcMoveToTarget::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMoveToTarget *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMoveToTarget::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMoveToTarget *
        ]
    ),
    "cGcRidden::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRidden *
        ]
    ),
    "cGcRidden::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRidden *
        ]
    ),
    "cGcRidden::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRidden *
        ]
    ),
    "cGcRidden::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRidden *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShouldAvoidPlayer::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShouldAvoidPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "GcSimpleFollowRoutine::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GcSimpleFollowRoutine *
        ]
    ),
    "GcSimpleFollowRoutine::MoveToNextRoutinePositionIfRequired": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # GcSimpleFollowRoutine *
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
            ctypes.c_float,  # float
        ]
    ),
    "GcSimpleFollowRoutine::StartStep": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # GcSimpleFollowRoutine *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "GcSimpleFollowRoutine::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # GcSimpleFollowRoutine *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkDynamicArray<cGcSettlementGiftDetails>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCharacterCustomisationTextureOptionData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCharacterCustomisationColourData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCharacterCustomisationColourData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkBlackboard::GetValue<cTkAttachmentPtr>": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboard *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # cTkAttachmentPtr *
        ]
    ),
    "cTkBlackboard::GetValue<bool>": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboard *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcCreatureComponent::Error": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_float,  # float
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcCreatureComponent::GetRoutine": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureRoutinePtr *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cGcCreatureRoutinePtr *
        ]
    ),
    "cGcCreatureRoutinePtr::operator->": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureRoutine *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutinePtr *
        ]
    ),
    "cTkPhysicsComponent::FindPhysRelMatrix": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcScannableComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcScannableComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimationComponent::FreezeAllAnims": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
        ]
    ),
    "cTkPhysicsComponent::HasRagdoll": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cGcCreatureComponent::GetObb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cTkAABB *
        ]
    ),
    "cTkRandomComponent::RandomFloat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkRandomComponent *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # cTkRandomComponent::RandomType
        ]
    ),
    "cGcCreaturePerception::GetLeaderCommand": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreaturePerception::LeaderCommand *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePerception *
        ]
    ),
    "cGcScaryThingsList::GetClosestScaryThing": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScaryThingsList *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcScaryThingsList::Thing *
        ]
    ),
    "cGcCreatureComponent::GetPerception": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreaturePerception *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::GetPerceptionDistance": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::GetKnowledge": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNavPolyHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cTkRandomComponent::GetRNG": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPersonalRNG *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRandomComponent *
            ctypes.c_uint32,  # cTkRandomComponent::RandomType
        ]
    ),
    "cGcCreatureSteering::AddImpassable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSteering *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eSteeringInput
        ]
    ),
    "cGcCreatureBehaviour::GetAxis": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviour *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cTkAnimationComponent::GetAnimUnscaledLength": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkRandomComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcCreatureBehaviour::cGcCreatureBehaviour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviour *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cGcCreatureBaitComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcCreatureNavigation::GetSwarm": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureSwarm *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cGcCreatureNavigation::GetSwarmIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cGcIdleAnimList::BuildIdleList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIdleAnimList *
            ctypes.c_ulonglong,  # cTkAnimationComponent *
        ]
    ),
    "cGcIdleAnimList::GetRandomIdle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcIdleAnimList *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcIdleAnimList::OkayToLook": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcIdleAnimList *
        ]
    ),
    "cGcCreatureBehaviour::PlayAnimation": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviour *
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureBehaviour::MoveToPos": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviour *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureBehaviour::FacePos": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviour *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcHerbivoreFeed::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHerbivoreFeed *
        ]
    ),
    "cGcHerbivoreFeed::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHerbivoreFeed *
            ctypes.c_float,  # float
        ]
    ),
    "cGcHerbivoreFeed::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHerbivoreFeed *
        ]
    ),
    "cGcHerbivoreFlee::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHerbivoreFlee *
        ]
    ),
    "cGcHerbivoreFlee::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHerbivoreFlee *
            ctypes.c_float,  # float
        ]
    ),
    "cGcHerbivoreStop::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHerbivoreStop *
            ctypes.c_float,  # float
        ]
    ),
    "cTkBlackboard::ClearValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboard *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcMetadataBehaviour::BehaviourError": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMetadataBehaviour *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcMetadataBehaviour::NGuiDrawProgressBar": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMetadataBehaviour *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcBehaviourGetTarget::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourGetTarget *
        ]
    ),
    "cGcBehaviourGetTarget::NGuiRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourGetTarget *
        ]
    ),
    "BehaviourAllocatorcGcBehaviourGetTarget::Allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourGetTargetData *
        argtypes=[
            ctypes.c_ulonglong,  # BehaviourAllocatorcGcBehaviourGetTarget *
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourRegisterAttacker::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourRegisterAttacker *
        ]
    ),
    "cGcBehaviourRegisterAttacker::NGuiRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourRegisterAttacker *
        ]
    ),
    "BehaviourAllocatorcGcBehaviourRegisterAttacker::Allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourRegisterAttackerData *
        argtypes=[
            ctypes.c_ulonglong,  # BehaviourAllocatorcGcBehaviourRegisterAttacker *
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourMoveToTarget::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourMoveToTarget *
        ]
    ),
    "cGcBehaviourMoveToTarget::NGuiRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourMoveToTarget *
        ]
    ),
    "BehaviourAllocatorcGcBehaviourMoveToTarget::Allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourMoveToTargetData *
        argtypes=[
            ctypes.c_ulonglong,  # BehaviourAllocatorcGcBehaviourMoveToTarget *
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourMaintainRangeFromTarget::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourMaintainRangeFromTarget *
        ]
    ),
    "cGcBehaviourMaintainRangeFromTarget::NGuiRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourMaintainRangeFromTarget *
        ]
    ),
    "BehaviourAllocatorcGcBehaviourMaintainRangeFromTarget::Allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourMaintainRangeFromTargetData *
        argtypes=[
            ctypes.c_ulonglong,  # BehaviourAllocatorcGcBehaviourMaintainRangeFromTarget *
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourFaceTarget::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourFaceTarget *
        ]
    ),
    "cGcBehaviourFaceTarget::NGuiRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourFaceTarget *
        ]
    ),
    "BehaviourAllocatorcGcBehaviourFaceTarget::Allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourFaceTargetData *
        argtypes=[
            ctypes.c_ulonglong,  # BehaviourAllocatorcGcBehaviourFaceTarget *
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourPlayAnim::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sPlayAnimType *
        ]
    ),
    "cGcBehaviourPlayAnim::NGuiRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourPlayAnim *
        ]
    ),
    "BehaviourAllocatorcGcBehaviourPlayAnim::Allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourPlayAnimData *
        argtypes=[
            ctypes.c_ulonglong,  # BehaviourAllocatorcGcBehaviourPlayAnim *
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourApplyDamage::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourApplyDamage *
        ]
    ),
    "cGcBehaviourApplyDamage::NGuiRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourApplyDamage *
        ]
    ),
    "BehaviourAllocatorcGcBehaviourApplyDamage::Allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourApplyDamageData *
        argtypes=[
            ctypes.c_ulonglong,  # BehaviourAllocatorcGcBehaviourApplyDamage *
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourLaunchProjectile::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourLaunchProjectile *
        ]
    ),
    "cGcBehaviourLaunchProjectile::NGuiRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourLaunchProjectile *
        ]
    ),
    "BehaviourAllocatorcGcBehaviourLaunchProjectile::Allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourLaunchProjectileData *
        argtypes=[
            ctypes.c_ulonglong,  # BehaviourAllocatorcGcBehaviourLaunchProjectile *
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourWait::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourWait *
        ]
    ),
    "cGcBehaviourWait::NGuiRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourWait *
        ]
    ),
    "BehaviourAllocatorcGcBehaviourWait::Allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourWaitData *
        argtypes=[
            ctypes.c_ulonglong,  # BehaviourAllocatorcGcBehaviourWait *
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourIdle::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourIdle *
        ]
    ),
    "cGcBehaviourIdle::NGuiRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourIdle *
        ]
    ),
    "BehaviourAllocatorcGcBehaviourIdle::Allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourIdleData *
        argtypes=[
            ctypes.c_ulonglong,  # BehaviourAllocatorcGcBehaviourIdle *
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourGetTarget::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourGetTarget *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBehaviourRegisterAttacker::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourRegisterAttacker *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBehaviourMoveToTarget::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourMoveToTarget *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
        ]
    ),
    "cGcBehaviourMaintainRangeFromTarget::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourMoveToTarget *
        ]
    ),
    "cGcBehaviourMaintainRangeFromTarget::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourMaintainRangeFromTarget *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBehaviourFaceTarget::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourFaceTarget *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBehaviourFaceTarget::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourFaceTarget *
        ]
    ),
    "cGcBehaviourWait::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourWait *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBehaviourWait::NGuiRenderImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourWait *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcBehaviourApplyDamage::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourApplyDamage *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBehaviourLaunchProjectile::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourLaunchProjectile *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBehaviourPlayAnim::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourPlayAnim *
        ]
    ),
    "cGcBehaviourPlayAnim::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourPlayAnim *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBehaviourIdle::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourIdle *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCanFiendEngage::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCanFiendEngage *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFiendHunt::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendHunt *
        ]
    ),
    "cGcFiendHunt::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendHunt *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFiendHunt::ResignActiveAttacker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendHunt *
            ctypes.c_ulonglong,  # cGcPlayerCommon *
        ]
    ),
    "cGcFiendHunt::DoMoveToSurround": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendHunt *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcFiendHunt::PlayAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendHunt *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # cGcFiendHunt::HuntState
            ctypes.c_float,  # float
        ]
    ),
    "cGcFiendHunt::DoPounce": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendHunt *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcFiendHunt::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendHunt *
        ]
    ),
    "cGcFiendHunt::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendHunt *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcFiendPatrol::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendPatrol *
        ]
    ),
    "cGcFiendPatrol::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendPatrol *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFiendPatrol::DoPatrol": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendPatrol *
        ]
    ),
    "cGcFiendPatrol::DoBurrow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendPatrol *
        ]
    ),
    "cGcFiendStunned::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendStunned *
        ]
    ),
    "cGcFiendStunned::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendStunned *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMiniDroneHunt::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMiniDroneHunt *
        ]
    ),
    "cGcMiniDroneHunt::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMiniDroneHunt *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMiniDroneHunt::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMiniDroneHunt *
        ]
    ),
    "cGcPredatorHunt::GenAttackWeights": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPredatorHunt *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPetBehaviourTree::SetWantsLaserOn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTree *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetBehaviourTree::SetWantsLaserAim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTree *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetBehaviourTree::SetLaserAim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTree *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcPredatorHunt::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPredatorHunt *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPredatorHunt::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPredatorHunt *
        ]
    ),
    "cGcScuttlerHunt::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScuttlerHunt *
        ]
    ),
    "cGcScuttlerHunt::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScuttlerHunt *
            ctypes.c_float,  # float
        ]
    ),
    "cGcScuttlerHunt::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScuttlerHunt *
        ]
    ),
    "cGcScuttlerHunt::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScuttlerHunt *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcSharkHunt::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSharkHunt *
        ]
    ),
    "cGcSharkHunt::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSharkHunt *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSharkHunt::UpdateDefault": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSharkHunt *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cGcPlayerCommon *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcSharkHunt::UpdateFishFiendBig": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSharkHunt *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cGcPlayerCommon *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcSharkHunt::UpdateJellyfish": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSharkHunt *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cGcPlayerCommon *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcSharkHunt::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSharkHunt *
        ]
    ),
    "cGcLandDolphin::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcLandDolphin *
        ]
    ),
    "cTkCreatureTailComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkCreatureTailComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailComponent *
        ]
    ),
    "cTkCreatureTailComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercTkCreatureTailComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercTkCreatureTailComponent *
        ]
    ),
    "ComponentInstancercTkCreatureTailComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcLandDolphin::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLandDolphin *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cGcLandDolphin::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLandDolphin *
        ]
    ),
    "cGcLandDolphin::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLandDolphin *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSnakePatrol::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSnakePatrol *
        ]
    ),
    "cGcSnakePatrol::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSnakePatrol *
        ]
    ),
    "cGcSnakePatrol::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSnakePatrol *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSnakePatrol::UpdateEffects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSnakePatrol *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSnakePatrol::SpawnGroundWorms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSnakePatrol *
        ]
    ),
    "cTkTemplatedComponentUpdater<cTkCreatureTailComponent>::UpdateComponentsPostPhysics": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cTkCreatureTailComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkBlackboard::SetValue<bool>": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboard *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const bool *
        ]
    ),
    "cGcProjectileImpact::Init": {
        "cGcProjectileImpact *, cGcProjectileData *, const cTkVector3 *, const cTkVector3 *, const cTkVector3 *, float, cTkRigidBody *, cGcOwnerConcept *, int, eDamageType, const std::vector<cGcImpactCombatEffectData,TkSTLAllocatorShim<cGcImpactCombatEffectData,4,-1> > *, const std::vector<cGcCombatEffectDamageMultiplier,TkSTLAllocatorShim<cGcCombatEffectDamageMultiplier,4,-1> > *, float, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcProjectileImpact *
                ctypes.c_ulonglong,  # cGcProjectileData *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # cTkRigidBody *
                ctypes.c_ulonglong,  # cGcOwnerConcept *
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # eDamageType
                ctypes.c_ulonglong,  # const std::vector<cGcImpactCombatEffectData,TkSTLAllocatorShim<cGcImpactCombatEffectData,4,-1> > *
                ctypes.c_ulonglong,  # const std::vector<cGcCombatEffectDamageMultiplier,TkSTLAllocatorShim<cGcCombatEffectDamageMultiplier,4,-1> > *
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
        "cGcProjectileImpact *, cGcProjectileData *, const cTkVector3 *, const cTkVector3 *, const cTkVector3 *, float, cTkRigidBody *, cGcOwnerConcept *, int, eDamageType, const cTkDynamicArray<cGcImpactCombatEffectData> *, const std::vector<cGcCombatEffectDamageMultiplier,TkSTLAllocatorShim<cGcCombatEffectDamageMultiplier,4,-1> > *, float, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcProjectileImpact *
                ctypes.c_ulonglong,  # cGcProjectileData *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # cTkRigidBody *
                ctypes.c_ulonglong,  # cGcOwnerConcept *
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # eDamageType
                ctypes.c_ulonglong,  # const cTkDynamicArray<cGcImpactCombatEffectData> *
                ctypes.c_ulonglong,  # const std::vector<cGcCombatEffectDamageMultiplier,TkSTLAllocatorShim<cGcCombatEffectDamageMultiplier,4,-1> > *
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
    },
    "cGcCreatureSwarm::AddAlign": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarm *
            ctypes.c_ulonglong,  # cGcCreatureSteering *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcCreatureSwarm::AddCohere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarm *
            ctypes.c_ulonglong,  # cGcCreatureSteering *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureSwarm::AddSeparate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarm *
            ctypes.c_ulonglong,  # cGcCreatureSteering *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureFullBodyIKComponent::GetHeadJointNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcCreatureComponent::GetEnergy": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::ModifyEnergy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureComponent::SetWantsToDespawn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerCommon::RemoveEngagedPredator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommon *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcPlayerCommon::RemoveActiveAttacker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommon *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcPlayerCommon::AddActiveAttacker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommon *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcPlayerCommon::IsActiveAttacker": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommon *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkRandomComponent::RandomInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRandomComponent *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # cTkRandomComponent::RandomType
        ]
    ),
    "cGcCreatureComponent::IsBeingFed": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cTkRigidBody::QueueForPrePhysUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBodyProxy *
        ]
    ),
    "cTkRigidBody::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureComponent::GetTypeID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcSwarmFlee::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSwarmFlee *
        ]
    ),
    "cGcSwarmFlee::cGcSwarmFlee": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSwarmFlee *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_uint32,  # cGcCreatureMoveSpeedRequest
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcSwarmFlee::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSwarmFlee *
        ]
    ),
    "cGcSwarmFlee::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSwarmFlee *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSwarmFlee::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSwarmFlee *
        ]
    ),
    "cGcSwarmMotion::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSwarmMotion *
        ]
    ),
    "cGcSwarmLand::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSwarmLand *
        ]
    ),
    "cGcSwarmMotion::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSwarmMotion *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSwarmLand::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSwarmLand *
        ]
    ),
    "cGcSwarmLand::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSwarmLand *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFiendStunned::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendStunned *
        ]
    ),
    "cGcFiendHunt::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendHunt *
        ]
    ),
    "cGcScuttlerHunt::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScuttlerHunt *
        ]
    ),
    "cGcScuttlerHunt::cGcScuttlerHunt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScuttlerHunt *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # const cGcCreatureFiendAttackData *
            ctypes.c_ulonglong,  # cGcFiendBlackboard *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcMiniDroneHunt::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMiniDroneHunt *
        ]
    ),
    "cGcLookAtStuff::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcLookAtStuff *
        ]
    ),
    "cGcLookAtStuff::cGcLookAtStuff": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLookAtStuff *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcFiendPatrol::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendPatrol *
        ]
    ),
    "cGcCanFiendEngage::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCanFiendEngage *
        ]
    ),
    "cGcAppear::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAppear *
        ]
    ),
    "cGcAppear::cGcAppear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAppear *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcDetailAnims::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDetailAnims *
        ]
    ),
    "cGcDetailAnims::cGcDetailAnims": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDetailAnims *
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcDetailAnims::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDetailAnims *
        ]
    ),
    "cGcDamageNearby::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageNearby *
        ]
    ),
    "cGcFiendBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcFiendBehaviourTree::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendBehaviourTree *
        ]
    ),
    "cGcFiendBehaviourTree::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendBehaviourTree *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFiendBlackboard::RegisterThreat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendBlackboard *
            ctypes.c_uint64,  # cTkAttachmentPtr
            ctypes.c_float,  # float
        ]
    ),
    "cGcFiendBlackboard::GetThreatLevel": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendBlackboard *
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcGetNearestPlayer::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGetNearestPlayer *
        ]
    ),
    "cGcMoveToTarget::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMoveToTarget *
        ]
    ),
    "cGcHerbivoreFlee::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcHerbivoreFlee *
        ]
    ),
    "cGcRockCreatureBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRockCreatureBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcMetadataBehaviourTree::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRockCreatureBehaviourTree *
        ]
    ),
    "cGcRockCreatureBehaviourTree::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRockCreatureBehaviourTree *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBeetleBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBeetleBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcSnakeBehaviourTree::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipBehaviourTree *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAdjustHeight::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAdjustHeight *
        ]
    ),
    "cGcCanSeePlayer::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCanSeePlayer *
        ]
    ),
    "cGcCanSeePlayer::cGcCanSeePlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCanSeePlayer *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcPetBehaviourTree::SetHasBeenCustomised": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTree *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetBehaviourTree::GetHasBeenCustomised": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTree *
        ]
    ),
    "cGcPetBehaviour::GetRNG": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPersonalRNG *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviour *
        ]
    ),
    "cGcPetInteraction::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetInteraction *
        ]
    ),
    "cGcPetInteraction::cGcPetInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetInteraction *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcPetRoam::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetRoam *
        ]
    ),
    "cGcPetRoam::MoveToPlayer": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetRoam *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcPetRoam::BeginBehaviourSubState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetRoam *
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cGcPetRoam::GetActiveBehaviour": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetRoam *
        ]
    ),
    "cGcBirdBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBirdBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcBirdBehaviourTree::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBirdBehaviourTree *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCheckTimer::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCheckTimer *
        ]
    ),
    "cGcHerbivoreStop::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcHerbivoreStop *
        ]
    ),
    "cGcHerbivoreFeed::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcHerbivoreFeed *
        ]
    ),
    "cGcCanFollowPlayer::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCanFollowPlayer *
        ]
    ),
    "cGcCanSenseAttraction::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCanSenseAttraction *
        ]
    ),
    "cGcCanSeePrey::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCanSeePrey *
        ]
    ),
    "cGcPredatorHunt::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPredatorHunt *
        ]
    ),
    "cGcPredatorHunt::cGcPredatorHunt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPredatorHunt *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcPredatorHunt::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPredatorHunt *
        ]
    ),
    "GcGrazeFollowingRoutine::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # GcGrazeFollowingRoutine *
        ]
    ),
    "GcGrazeFollowingRoutine::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GcGrazeFollowingRoutine *
        ]
    ),
    "GcSimpleFollowRoutine::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # GcSimpleFollowRoutine *
        ]
    ),
    "GcSimpleFollowRoutine::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GcSimpleFollowRoutine *
        ]
    ),
    "cGcShouldAvoidPlayer::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShouldAvoidPlayer *
        ]
    ),
    "cGcShouldAvoidPlayer::cGcShouldAvoidPlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShouldAvoidPlayer *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcCreatureBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcCreatureBehaviourTree::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
        ]
    ),
    "cGcCreatureBehaviourTree::GetMoodAnim": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_uint32,  # eMood
        ]
    ),
    "cGcCreatureBehaviourTree::AddInteractionBehaviour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
        ]
    ),
    "cGcCreatureBehaviourTree::AddRidingBehaviour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
        ]
    ),
    "cGcCreatureBehaviourTree::AddFollowPlayerBehaviour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
        ]
    ),
    "cGcCreatureBehaviourTree::AddAttractedBehaviour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
            ctypes.c_ulonglong,  # cTkAnimationComponent *
        ]
    ),
    "cGcCreatureBehaviourTree::AddStandardFleeBehavior": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
        ]
    ),
    "cGcCreatureBehaviourTree::AddDefendBehaviour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
            ctypes.c_ulonglong,  # cTkAnimationComponent *
        ]
    ),
    "cGcCreatureBehaviourTree::AddAvoidPlayerBehaviour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
            ctypes.c_ulonglong,  # cGcCreatureMovementData *
        ]
    ),
    "cGcCreatureBehaviourTree::AddNoticePlayerBehaviour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
        ]
    ),
    "cGcCreatureBehaviourTree::AddFollowRoutineBehaviour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
            ctypes.c_ulonglong,  # cGcCreatureMovementData *
            ctypes.c_ulonglong,  # cTkAnimationComponent *
        ]
    ),
    "cGcCreatureBehaviourTree::AddPredatorBehaviour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
            ctypes.c_ulonglong,  # cTkAnimationComponent *
        ]
    ),
    "cGcCreatureBehaviourTree::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
        ]
    ),
    "cGcFishBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFishBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcJellyFishBehaviourTree::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJellyFishBehaviourTree *
            ctypes.c_float,  # float
        ]
    ),
    "cGcJellyFishBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJellyFishBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcBlackboardValueDecorator::NGuiRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlackboardValueDecorator *
        ]
    ),
    "cGcBlackboardValueDecorator::Update": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlackboardValueDecorator *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMetadataBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcMetadataBehaviourTree::SetBlackboardValuesFromArchetype": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
        ]
    ),
    "cGcMetadataBehaviourTree::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
        ]
    ),
    "cGcMetadataBehaviourTree::AddNodesFromClassPointers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkClassPointer> *
        ]
    ),
    "cGcMetadataBehaviourTree::AddNodeFromClassPointer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMetadataBehaviourTree::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSharkHunt::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSharkHunt *
        ]
    ),
    "cGcSharkHunt::cGcSharkHunt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSharkHunt *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcPetBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcPetBehaviourTree::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTree *
        ]
    ),
    "cGcPetBehaviourTree::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTree *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPetBehaviourTree::GetAttackAnim": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTree *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcPetBehaviour::FacePlayer": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviour *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcPetBehaviour::FacePos": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviour *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcPetBehaviour::MoveToPlayer": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviour *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcPetFollowTrail *
        ]
    ),
    "cGcPetBehaviour::MoveToPos": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviour *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetInteraction::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetInteraction *
        ]
    ),
    "cGcPetInteraction::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetInteraction *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPetInteraction::MoveToInteractPos": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetInteraction *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcPetInteraction::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetInteraction *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcPetInteraction::EventCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetInteraction *
            ctypes.c_ulonglong,  # const cGcMessagePetBehaviourEvent *
        ]
    ),
    "cGcPetInteraction::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetInteraction *
        ]
    ),
    "cGcPetRoam::cGcPetRoam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetRoam *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcPetBehaviourTree *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcPetRoam::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetRoam *
        ]
    ),
    "cGcPetRoam::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetRoam *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcPetRoam::EventCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetRoam *
            ctypes.c_ulonglong,  # const cGcMessagePetBehaviourEvent *
        ]
    ),
    "cGcPetRoam::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetRoam *
        ]
    ),
    "cGcPetRoam::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetRoam *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPetRoam::CanFinishCurrentBehaviour": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetRoam *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcPetRoam::DecideWhatToDo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetRoam *
        ]
    ),
    "cGcPetRoam::FindUsefulThings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetRoam *
        ]
    ),
    "cGcPetRoam::CalculateBehaviourValidity": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetRoam *
            ctypes.c_uint32,  # ePetBehaviour
        ]
    ),
    "cGcPetRoam::BeginBehaviour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetRoam *
            ctypes.c_uint32,  # ePetBehaviour
        ]
    ),
    "cGcPetRoam::DoPoop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetRoam *
        ]
    ),
    "cGcPetRoam::CanPostChatMessage": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetRoam *
        ]
    ),
    "cGcPetFollowTrail::ShouldAddPoint": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetFollowTrail *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcPetFollowTrail::AddPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetFollowTrail *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcPetBlackboard::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBlackboard *
        ]
    ),
    "cGcCreatureStunned::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureStunned *
        ]
    ),
    "cGcUngulateBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUngulateBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcUngulateBehaviourTree::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUngulateBehaviourTree *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoComboBox<ePetBehaviour>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # ePetBehaviour *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDestructableComponent::GivesResource": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponent *
        ]
    ),
    "cTkAnimationComponent::GetAnimActionFrame": {
        "cTkAnimationComponent *, const TkID<128> *": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # cTkAnimationComponent *
                ctypes.c_char_p,  # const TkID<128> *
            ]
        ),
        "cTkAnimationComponent *, const cTkAnimInstanceHandle *": FUNCDEF(
            restype=ctypes.c_double,  # double
            argtypes=[
                ctypes.c_ulonglong,  # cTkAnimationComponent *
                ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
            ]
        ),
    },
    "cTkAnimationComponent::GetAnimFrame": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcCreatureComponent::GetPetInteractionAttachment": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachmentPtr *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cTkAttachmentPtr *
        ]
    ),
    "cTkBehaviourTreeAlwaysSucceedDecorator::Update": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeAlwaysSucceedDecorator *
            ctypes.c_float,  # float
        ]
    ),
    "cTkBehaviourTreeDecorator::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeDecorator *
            ctypes.c_ulonglong,  # AK::SrcMedia::Stream::State *
        ]
    ),
    "cTkBehaviourTreeDecorator::AddNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeDecorator *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        ]
    ),
    "cTkPhysicsComponent::SetRagdollLinearDamping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cTkPhysicsComponent::SetRagdollGravityFactor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureSwarm::GetComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCreatureNode *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarm *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcShootableComponent::SetNumHits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureNavTrail::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavTrail *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cGcCreatureNavTrail::PushElt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavTrail *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureNavTrail::AdjustFootPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavTrail *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint32,  # ePrimaryAxis
        ]
    ),
    "cGcCreatureNavTrail::AdjustCoGPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavTrail *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint32,  # ePrimaryAxis
        ]
    ),
    "cGcCreatureNavTrail::GetHipAngle": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavTrail *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint32,  # ePrimaryAxis
        ]
    ),
    "cGcCreatureNavigation::~cGcCreatureNavigation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cGcCreatureNavigation::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cGcCreatureNavigation::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cGcCreatureNavigation::SetMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_uint32,  # eNavigationMode
        ]
    ),
    "cGcCreatureNavigation::GetFrontNavMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcCreatureNavigation::GetTurnPivot": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcRegionKnowledgeChunkMask::cGcRegionKnowledgeChunkMask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledgeChunkMask *
        ]
    ),
    "cGcCreatureNavigation::NavigateInRoughDirection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
        ]
    ),
    "cGcCreatureNavigation::NavigateToPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkStackVector<cGcNavPolyHandle,64,-1> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
        ]
    ),
    "cGcCreatureNavigation::SetMoveType": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_uint32,  # cGcCreatureMoveSpeedRequest
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureNavigation::RequestIdle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimLayerHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
            ctypes.c_ulonglong,  # cTkAnimLayerHandle *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureNavigation::GetIdleLength": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcCreatureNavigation::GetMoveType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cGcCreatureNavigation::GetAttackAnimCandidates": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcCreatureNavigation::GetDesiredRidingAnim": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcCreatureNavigation::GetRidingLean": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cGcCreatureNavigation::GetBaseRegistrationLocator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkQuaternion *
        ]
    ),
    "cGcCreatureNavigation::UpdateRegistrationLocators": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cTkStackVector<cGcNavPolyHandle,64,-1>::~cTkStackVector<cGcNavPolyHandle,64,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStackVector<cGcNavPolyHandle,64,-1> *
        ]
    ),
    "StackContainer<std::vector<cGcNavBorderSegment,StackAllocator<cGcNavBorderSegment,32,-1>>,32,-1>::~StackContainer<std::vector<cGcNavBorderSegment,StackAllocator<cGcNavBorderSegment,32,-1>>,32,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cGcNavBorderSegment,StackAllocator<cGcNavBorderSegment,32,-1> >,32,-1> *
        ]
    ),
    "GcFollowRoutineGround::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # GcFollowRoutineGround *
        ]
    ),
    "GcFollowRoutineGround::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GcFollowRoutineGround *
        ]
    ),
    "cGcPredatorBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPredatorBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcPredatorBehaviourTree::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPredatorBehaviourTree *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCheckEnergy::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCheckEnergy *
        ]
    ),
    "cGcCheckEnergy::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCheckEnergy *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSharkBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSharkBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcSnakePatrol::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSnakePatrol *
        ]
    ),
    "cGcSnakeBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSnakeBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcSnakeBehaviourTree::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSnakeBehaviourTree *
        ]
    ),
    "cGcRobotCheckAlert::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotCheckAlert *
        ]
    ),
    "cGcRobotBehaviourTree::SetAlertState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotBehaviourTree *
            ctypes.c_uint32,  # cGcRobotBehaviourTree::eAlertness
        ]
    ),
    "cGcRobotBehaviourTree::SetLightIntensity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotBehaviourTree *
            ctypes.c_float,  # float
        ]
    ),
    "cGcRobotBehaviourTree::SetLightEndPos": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotBehaviourTree *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcQuadGuard::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadGuard *
        ]
    ),
    "cGcQuadCheckAlert::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadCheckAlert *
        ]
    ),
    "cGcQuadPounce::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadPounce *
        ]
    ),
    "cGcQuadEvade::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadEvade *
        ]
    ),
    "cGcQuadAttackMovement::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadAttackMovement *
        ]
    ),
    "cGcQuadAttack::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadAttack *
        ]
    ),
    "cGcQuadSearch::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadSearch *
        ]
    ),
    "cGcQuadBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcQuadGuard::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadGuard *
        ]
    ),
    "cGcQuadGuard::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadGuard *
            ctypes.c_float,  # float
        ]
    ),
    "cGcQuadCheckAlert::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadCheckAlert *
            ctypes.c_float,  # float
        ]
    ),
    "cGcQuadAttack::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadAttack *
            ctypes.c_float,  # float
        ]
    ),
    "cGcQuadAttack::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadAttack *
        ]
    ),
    "cGcQuadPounce::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadPounce *
            ctypes.c_float,  # float
        ]
    ),
    "cGcQuadEvade::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadEvade *
            ctypes.c_float,  # float
        ]
    ),
    "cGcQuadAttackMovement::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadAttackMovement *
            ctypes.c_float,  # float
        ]
    ),
    "cGcQuadAttackMovement::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadAttackMovement *
        ]
    ),
    "cGcQuadAttackMovement::FaceTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadAttackMovement *
        ]
    ),
    "cGcQuadSearch::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuadSearch *
            ctypes.c_float,  # float
        ]
    ),
    "cGcRobotCheckAlert::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotCheckAlert *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcRobotBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcRobotBehaviourTree::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotBehaviourTree *
        ]
    ),
    "cGcRobotBehaviourTree::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotBehaviourTree *
            ctypes.c_float,  # float
        ]
    ),
    "cGcRobotBehaviourTree::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotBehaviourTree *
        ]
    ),
    "cGcRobotBehaviourTree::FireLasers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotBehaviourTree *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcRobotBehaviourTree::FireGuns": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotBehaviourTree *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcRobotBehaviourTree::FireGrenade": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotBehaviourTree *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcRobotBehaviourTree::FireGrenade2": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotBehaviourTree *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcRobotBehaviourTree::HasLostArmour": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotBehaviourTree *
        ]
    ),
    "cGcRobotBehaviourTree::NotifyNetworkOfAttack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotBehaviourTree *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint32,  # SentinelAttackType
        ]
    ),
    "cGcRobotBehaviourTree::OnCriticalHit": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcRobotBehaviourTree::OnArmourHit": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcWalkerLegArmourDestroyed::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerLegArmourDestroyed *
        ]
    ),
    "cGcWalkerTitanFall::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerTitanFall *
        ]
    ),
    "cGcWalkerGuard::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerGuard *
        ]
    ),
    "cGcWalkerAttack::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerAttack *
        ]
    ),
    "cGcWalkerEnrage::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerEnrage *
        ]
    ),
    "cGcWalkerSearch::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerSearch *
        ]
    ),
    "cGcWalkerBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcWalkerBehaviourTree::SetLookAtPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerBehaviourTree *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcWalkerBehaviourTree::UpdateLookingInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerBehaviourTree *
            ctypes.c_float,  # float
        ]
    ),
    "cGcWalkerBehaviour::ForceSingleAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerBehaviour *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcWalkerAttack::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerAttack *
        ]
    ),
    "cGcWalkerAttack::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerAttack *
            ctypes.c_float,  # float
        ]
    ),
    "cGcWalkerAttack::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerAttack *
        ]
    ),
    "cGcWalkerTitanFall::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerTitanFall *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcWalkerTitanFall::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerTitanFall *
            ctypes.c_float,  # float
        ]
    ),
    "cGcWalkerGuard::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerGuard *
            ctypes.c_float,  # float
        ]
    ),
    "cGcWalkerEnrage::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerEnrage *
            ctypes.c_float,  # float
        ]
    ),
    "cGcWalkerLegArmourDestroyed::ReturnToIdle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerLegArmourDestroyed *
        ]
    ),
    "cGcWalkerSearch::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerSearch *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureFullBodyIKComponentUpdater::UpdateComponentsPostPhysics": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponentUpdater *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "ComponentInstancercGcCreatureFullBodyIKComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcCreatureFullBodyIKComponent *
        ]
    ),
    "ComponentInstancercGcCreatureFullBodyIKComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcCreatureFullBodyIKComponent::ConstructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcCreatureFullBodyIKComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcCreatureFullBodyIKComponent::cGcCreatureFullBodyIKComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
        ]
    ),
    "RegisterCreatureIkComponentForNgui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
        ]
    ),
    "cGcFootConstraint::PreAdditive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFootConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkEulerVector *
        ]
    ),
    "cGcFootConstraint::PreSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFootConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkEulerVector *
        ]
    ),
    "cGcPistonConstraint::PostSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPistonConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcCreatureCogConstraint::PreSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureCogConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcCreaturePelvisConstraint::PreSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePelvisConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkEulerVector *
        ]
    ),
    "cGcFootConstraint::PostSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFootConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcCreatureFullBodyIKComponent::OnPostAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureFullBodyIKComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const std::_Value_init_tag *
        ]
    ),
    "cGcCreatureFullBodyIKComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
        ]
    ),
    "cGcCreatureFullBodyIKComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
        ]
    ),
    "cGcCreatureFullBodyIKComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureFullBodyIKComponent::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureFullBodyIKComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
        ]
    ),
    "cGcHeadConstraint::PostSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeadConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "RefCountedJob::ShouldAbort": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # RefCountedJob *
        ]
    ),
    "RefCountedJob::Increment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RefCountedJob *
        ]
    ),
    "RefCountedJob::Destroy": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # RefCountedJob *
        ]
    ),
    "cGcCreatureGroundNavigation::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::SetMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_uint32,  # eNavigationMode
        ]
    ),
    "cGcNavMap<16>::cGcNavMap<16>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
        ]
    ),
    "cGcCreatureGroundNavigation::HasPath": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::CopyPathFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::AddAvoidTurning": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::GetTurnPivot": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcCreatureGroundNavigation::GetBestMoveAnim": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureMoveAnimData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcCreatureGroundNavigation::ForceSingleAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureGroundNavigation::GetForcedSingleAnimHandle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimLayerHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_ulonglong,  # cTkAnimLayerHandle *
        ]
    ),
    "cGcCreatureGroundNavigation::HasForcedSingleAnim": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::CancelForcedSingleAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::PlaySubAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGroundNavigation::SetStealthy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureGroundNavigation::ForceTurn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGroundNavigation::GetMoveType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::GetStealthy": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::GetRunAnimSpeed": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::ShouldDespawn": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::DebugRecreateNav": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
        ]
    ),
    "cGcCreatureGroundNavigation::RequestRefreshNavMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureGroundNavigation::RenderNGuiVis": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::RenderNGuiControl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::GetRight": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcCreatureGroundNavigation::RefreshMoveSpeeds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::SetMoveSpeeds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureGroundNavigation::SetSpeedFactor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_uint32,  # cGcCreatureMoveSpeedRequest
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureGroundNavigation::GetDesiredRidingAnim": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcCreatureGroundNavigation::GetRidingLean": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::IsBeingRidden": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::SetMinObstacleAabbSize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureSwarmNavigation::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
        ]
    ),
    "cGcCreatureSwarmNavigation::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
        ]
    ),
    "cGcCreatureSwarmNavigation::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
        ]
    ),
    "cGcCreatureSwarmNavigation::NavigateToPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkStackVector<cGcNavPolyHandle,64,-1> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
        ]
    ),
    "cGcCreatureSwarmNavigation::GetMoveType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
        ]
    ),
    "cGcCreatureSwarmNavigation::SetSpeedFactor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
            ctypes.c_uint32,  # cGcCreatureMoveSpeedRequest
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureSwarmNavigation::GetDesiredRidingAnim": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcCreatureSwarmNavigation::GetLandingProgress": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
        ]
    ),
    "cGcCreatureSwarmNavigation::SetWantsToLand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSwarmNavigation::NotifyOfPredator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
            ctypes.c_ulonglong,  # cGcPathfindingObstacle *
        ]
    ),
    "cGcCreatureSwarmNavigation::AddBoost": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureSwarmNavigation::IsBoosting": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
        ]
    ),
    "cGcCreatureSwarmNavigation::SetClampToGround": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureNavigation::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
        ]
    ),
    "cGcCreatureNavigation::ZeroSteering": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureGroundNavigation::ZeroSteering": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureNavigation::AddAvoidCreatures": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_ulonglong,  # cGcPathfindingObstacle *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcCreatureNavigation::AddAvoidDanger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cGcCreatureNavigation::AddSteerToFollow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::AddSteerToAvoidImpassable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureNavigation::AddAvoidTerrainEditsAndBaseParts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::GetFrontNavMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcCreatureNavigation::IsAtTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureNavigation::IsPointAccessible": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
        ]
    ),
    "cGcCreatureNavigation::UpdatePath": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureNavigation::UpdateHeadPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cGcCreatureNavigation::UpdateHeadDamage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureNavigation::RenderNGuiControl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cGcCreatureNavigation::DebugRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
        ]
    ),
    "cGcCreatureNavigation::RenderNGuiVis": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
        ]
    ),
    "cGcCreatureGroundNavigation::GetIdleLength": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcCreatureGroundNavigation::SetFakeAnimSpeed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_ulonglong,  # cGcCreatureMoveAnimData *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
        ]
    ),
    "cGcCreatureGroundNavigation::GetRecipTurnRadiusForVector": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureGroundNavigation::GetAdjustedSpeed": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureGroundNavigation::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcPathfindingObstacle *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcCreatureGroundNavigation::DoRayCastJob": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcCreatureGroundNavigation::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureGroundNavigation::InitialAdjustToKnowledge": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
        ]
    ),
    "cGcCreatureGroundNavigation::AdjustToKnowledge": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
            ctypes.c_double,  # double
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGroundNavigation::GetSteeringDirections": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcCreatureGroundNavigation::RequestIdle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimLayerHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_ulonglong,  # cTkAnimLayerHandle *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # cGcCreatureMoveSpeedRequest
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureGroundNavigation::SetMoveType": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_uint32,  # cGcCreatureMoveSpeedRequest
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureNavigation::GetHeadSize": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cGcCreatureNavigation::CanPounce": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcCreatureNavigation::StartPounce": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimInstanceHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_ulonglong,  # cTkAnimInstanceHandle *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGroundNavigation::StartPounce": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimInstanceHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_ulonglong,  # cTkAnimInstanceHandle *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureNavigation::UpdatePounce": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_ulonglong,  # cTkAnimInstanceHandle *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureNavigation::StartLocomotionDamage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cGcCreatureNavigation::UpdateLocomotionDamage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureNavigation::DoLocomotionDamageAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcCreatureGroundNavigation::GetAttackAnimCandidates": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcCreatureGroundNavigation::GetTurnRadius": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureSwarmNavigation::GetSteeringDirections": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcCreatureSwarmNavigation::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcPathfindingObstacle *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcCreatureSwarmNavigation::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureSwarmNavigation::UpdateHoverRayJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureSwarmNavigation::UpdateFloaterRayJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureSwarmNavigation::UpdateFishRayJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
        ]
    ),
    "cGcCreatureSwarmNavigation::UpdateLandJellyfishRayJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureSwarmNavigation::UpdateWallClimberRayJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcCreatureSwarmNavigation::UpdatePounce": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmNavigation *
            ctypes.c_ulonglong,  # cTkAnimInstanceHandle *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cGcAntagonistComponent *
        ]
    ),
    "CreateCreatureGroundNavigation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureNavigation *
        argtypes=[]
    ),
    "cGcCreatureGroundNavigation::cGcCreatureGroundNavigation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroundNavigation *
        ]
    ),
    "cGcCreatureNavigation::cGcCreatureNavigation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
        ]
    ),
    "cGcRegionSegmentCache::cGcRegionSegmentCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionSegmentCache *
        ]
    ),
    "CreateCreatureGroundIndoorNavigation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureNavigation *
        argtypes=[]
    ),
    "CreateRobotGroundNavigation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureNavigation *
        argtypes=[]
    ),
    "CreateCreatureSwarmNavigation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureNavigation *
        argtypes=[
            ctypes.c_uint32,  # eNavigationMode
            ctypes.c_ulonglong,  # cGcBirdData *
        ]
    ),
    "StackContainer<std::vector<cGcPathfindingObstacle,StackAllocator<cGcPathfindingObstacle,4,-1>>,4,-1>::~StackContainer<std::vector<cGcPathfindingObstacle,StackAllocator<cGcPathfindingObstacle,4,-1>>,4,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cGcPathfindingObstacle,StackAllocator<cGcPathfindingObstacle,4,-1> >,4,-1> *
        ]
    ),
    "cGcCreatureNavTrail::DebugDraw": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavTrail *
        ]
    ),
    "cGcCreatureNavigation::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNavigation *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMap<16>::DebugRecreate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
        ]
    ),
    "cGcNavMap<16>::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
        ]
    ),
    "cGcNavMap<16>::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
        ]
    ),
    "cGcNavMap<16>::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
        ]
    ),
    "cGcNavMap<16>::UpdateOwnerPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNavMapPathfindJob<16>::cGcNavMapPathfindJob<16>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapPathfindJob<16> *
        ]
    ),
    "cGcNavMap<16>::AddFlowSteering": {
        "cGcNavMap<16> *, float, cGcCreatureSteering *, const cTkPhysRelVec3 *, const cTkVector3 *, const cTkVector3 *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNavMap<16> *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # cGcCreatureSteering *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcNavMap<16> *, float *, cGcNavSteering *, const cTkPhysRelVec3 *, const cTkVector3 *, const cTkVector3 *, float, bool, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNavMap<16> *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # cGcNavSteering *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
                ctypes.c_float,  # float
            ]
        ),
    },
    "cGcNavMap<16>::PointIsInsideMapFast": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNavMapData<16>::cGcNavMapData<16>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapData<16> *
        ]
    ),
    "cGcNavMapData<16>::CopyDistanceArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapData<16> *
            ctypes.c_ulonglong,  # const cGcNavMapData<16> *
        ]
    ),
    "cGcNavMap<16>::CreateNewPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMap<16>::CreateNewSquare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMap<16>::ShiftMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMap<16>::RebuildMapPortion": {
        "cGcNavMap<16> *, const cTkPhysRelVec3 *, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNavMap<16> *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_float,  # float
            ]
        ),
        "cGcNavMap<16> *, int, int, int, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNavMap<16> *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcNavMap<16>::KickProbeTerrainJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNavMap<16>::PickupProbeTerrainJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
        ]
    ),
    "cGcNavMap<16>::PointIsInsideMapNoHeight": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcNavMap<16>::PointIsReachable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNavMap<16>::GetSquaredDistanceBetweenSquares": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcNavMap<16>::IsSquareInMapHalfCentre": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMapData<16>::IsSquareNavigable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapData<16> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMapData<16>::IsSquareBlocked": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapData<16> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMapData<16>::SetFourNeighborsReachable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapData<16> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMapPathfindJob<16>::CopyObstacleList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapPathfindJob<16> *
            ctypes.c_ulonglong,  # const cGcPathfindingObstacle *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMapPathfindJob<16>::CopyAvoidBodyList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapPathfindJob<16> *
            ctypes.c_ulonglong,  # const cTkRigidBody **
            ctypes.c_int32,  # int
        ]
    ),
    "PathJob<16>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcNavMap<16>::InvalidatePoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMap<16>::WorkOutNeighborReachability": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMap<16>::PointIsInsideMap": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNavMapData<16>::CopyPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapData<16> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMapData<16>::CopySquare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapData<16> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMapData<16>::CopyProbeResults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapData<16> *
            ctypes.c_ulonglong,  # const cGcNavMapData<16> *
        ]
    ),
    "ProbeAndPathJob<16>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcNavMapData<16>::ClearDynamicBlocked": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapData<16> *
        ]
    ),
    "cGcNavMapProbeJob<16>::ProbeSomeTerrain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapProbeJob<16> *
        ]
    ),
    "cGcNavMapPathfindJob<16>::FindClosestSquare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapPathfindJob<16> *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcNavMapPathfindJob<16>::FindClosestNavigableSquare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapPathfindJob<16> *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcNavMapPathfindJob<16>::FindClosestReachableNavigableSquare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapPathfindJob<16> *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcNavMapPathfindJob<16>::ExpandBlockedAreas": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapPathfindJob<16> *
        ]
    ),
    "cGcNavMapPathfindJob<16>::FloodFillReachable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapPathfindJob<16> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMapPathfindJob<16>::FloodFillDistances": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapPathfindJob<16> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMapProbeJob<16>::ProbePointHeight": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapProbeJob<16> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMapProbeJob<16>::ProbeTerrain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapProbeJob<16> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMapData<16>::GetPointLocal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMapData<16> *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkIKFullBody::cTkIKFullBody": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkIKFullBody *
        ]
    ),
    "cTkOnPostAnimCallbackForwarder<cGcCreatureFullBodyIKComponent>::OnPostAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkOnPostAnimCallbackForwarder<cGcCreatureFullBodyIKComponent> *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkPreAdditiveCallbackForwarder<cGcCreatureFullBodyIKComponent>::PreAdditive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPreAdditiveCallbackForwarder<cGcCreatureFullBodyIKComponent> *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkJobManager::WakeAllThreadsForQueue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureFullBodyIKComponent::GetHeadSize": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
        ]
    ),
    "cTkAnimationComponent::GetAnimSpeed": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
        ]
    ),
    "cTkAnimationComponent::GetAnimMirrored": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
        ]
    ),
    "cTkAnimationComponent::SetAnimMirrored": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
        ]
    ),
    "cGcCreatureComponent::GetMovementMode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureMovementBase *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureFullBodyIKComponent::GetFrontFootPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcCreatureFullBodyIKComponent::SetNavTrail": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
            ctypes.c_ulonglong,  # cGcCreatureNavTrail *
            ctypes.c_uint32,  # ePrimaryAxis
        ]
    ),
    "cGcCreatureComponent::GetCreatureSizeValue": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cTkIKFullBody::SetJointAxes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # cTkIKFullBody::ActiveAxes
        ]
    ),
    "cGcPistonConstraint::OnAttach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPistonConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
        ]
    ),
    "cGcPistonConstraint::GetNumRows": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPistonConstraint *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkIKRotYModifyConstraint::OnAttach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKRotYModifyConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
        ]
    ),
    "cTkIKRotYModifyConstraint::OnDetach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKRotYModifyConstraint *
            ctypes.c_ulonglong,  # cTkIKFullBody *
        ]
    ),
    "cTkIKRotYModifyConstraint::GetNumRows": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKRotYModifyConstraint *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkIKRotYModifyConstraint::BuildRows": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKRotYModifyConstraint *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # const cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkEulerVector *
        ]
    ),
    "cGcCreatureFullBodyIKComponent::~cGcCreatureFullBodyIKComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
        ]
    ),
    "cGcCreatureFullBodyIKComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
        ]
    ),
    "cGcCreatureFullBodyIKComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureFullBodyIKComponent::SetEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWanted::RegisterWitness": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWanted *
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # ePrimaryAxis
        ]
    ),
    "cTkMatrix44::Decompose": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix44 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkQuaternion *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cTkPhysicsComponent::ApplyPositionKeyframe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_double,  # double
            ctypes.c_float,  # float
        ]
    ),
    "cTkPhysicsComponent::EstimateNextPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_double,  # double
        ]
    ),
    "cGcNavMeshTile::Utils::GetOffset": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcCreatureComponent::GetPreferredLod": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcNavPolyHandle::cGcNavPolyHandle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavPolyHandle *
        ]
    ),
    "cTkTrig<float>::Exp2": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "cTkTrig<float>::Exp": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "StackContainer<std::vector<cGcPathfindingObstacle,StackAllocator<cGcPathfindingObstacle,32,-1>>,32,-1>::~StackContainer<std::vector<cGcPathfindingObstacle,StackAllocator<cGcPathfindingObstacle,32,-1>>,32,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cGcNPCNavGraphPhysProbeJob::sNodePairData,StackAllocator<cGcNPCNavGraphPhysProbeJob::sNodePairData,32,-1> >,32,-1> *
        ]
    ),
    "cGcCreatureAttractorComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureAttractorComponent *
        ]
    ),
    "cGcCreatureAttractorComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureAttractorComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "StackContainer<std::vector<cGcScaryThingsList::Thing,StackAllocator<cGcScaryThingsList::Thing,1024,-1>>,1024,-1>::~StackContainer<std::vector<cGcScaryThingsList::Thing,StackAllocator<cGcScaryThingsList::Thing,1024,-1>>,1024,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cGcScaryThingsList::Thing,StackAllocator<cGcScaryThingsList::Thing,1024,-1> >,1024,-1> *
        ]
    ),
    "cGcScaryThingsList::AddThing": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScaryThingsList *
            ctypes.c_uint32,  # eScaryThingMask
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcScaryThingsList::GetAnyScaryThings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScaryThingsList *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkStackVector<cTkPhysRelVec3,64,-1> *
            ctypes.c_ulonglong,  # cTkStackVector<float,64,-1> *
        ]
    ),
    "cGcCreaturePerception::InitThingFromAttach": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePerception *
            ctypes.c_ulonglong,  # cGcCreaturePerception::TrackedThing *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcCreaturePerception::UpdatePlayerAvoidRadius": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePerception *
            ctypes.c_ulonglong,  # cGcCreaturePerception::TrackedThing *
        ]
    ),
    "cGcCreaturePerception::UpdateLeader": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePerception *
        ]
    ),
    "cGcCreaturePerception::UpdateThing": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePerception *
            ctypes.c_ulonglong,  # cGcCreaturePerception::TrackedThing *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreaturePerception::WantVisionCast": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePerception *
            ctypes.c_ulonglong,  # cGcCreaturePerception::TrackedThing *
        ]
    ),
    "cGcCreaturePerception::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePerception *
        ]
    ),
    "cGcCreaturePerception::GetSomethingToLookAt": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePerception *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcCreaturePerception::GetThingsToAvoid": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePerception *
            ctypes.c_ulonglong,  # cTkStackVector<cGcPathfindingObstacle,32,-1> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePerception::GetThingsOfType": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePerception *
            ctypes.c_uint32,  # cGcCreaturePerception::ThingType
            ctypes.c_ulonglong,  # cTkStackVector<cGcCreaturePerception::TrackedThing *,128,-1> *
        ]
    ),
    "cGcCreaturePerception::NotifyPreyOfPresence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePerception *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreaturePerception::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePerception *
        ]
    ),
    "PickHeight": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanet *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcCreatureRoutine::GenerateRoutine": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
            ctypes.c_ulonglong,  # const cGcRegionKnowledge *
            ctypes.c_ulonglong,  # const cGcRegionKnowledge *const *
            ctypes.c_ulonglong,  # const cGcRegionDecoratorCreatures *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cGcCreatureStats *
            ctypes.c_ulonglong,  # const cGcCreatureStats *
            ctypes.c_ulonglong,  # const cGcCreatureRoutine *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eTileType
        ]
    ),
    "StackContainer<std::vector<std::pair<cGcRegionKnowledge const *,short>,StackAllocator<std::pair<cGcRegionKnowledge const *,short>,600,-1>>,600,-1>::~StackContainer<std::vector<std::pair<cGcRegionKnowledge const *,short>,StackAllocator<std::pair<cGcRegionKnowledge const *,short>,600,-1>>,600,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<std::pair<cGcRegionKnowledge const *,short>,StackAllocator<std::pair<cGcRegionKnowledge const *,short>,600,-1> >,600,-1> *
        ]
    ),
    "cGcCreatureRoutine::BuildActualPaths": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
            ctypes.c_ulonglong,  # const cGcRegionKnowledge *const *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcCreatureRoutine::PathCell *
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eRegionKnowledgeInterest
        ]
    ),
    "cGcCreatureRoutine::RemoveRedundantCells": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureRoutine::PickSomeInterestingChunks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkStackVector<std::pair<cGcRegionKnowledge const *,short>,600,-1> *
            ctypes.c_ulonglong,  # const cGcCreatureRoutine *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eTileType
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_ulonglong,  # cTkStackVector<cGcCreatureRoutine::PathCell,16,-1> *
        ]
    ),
    "cGcCreatureRoutine::GetNeighboringArea": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
            ctypes.c_ulonglong,  # const cGcRegionKnowledge *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_ulonglong,  # const cGcRegionKnowledge *const *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
            ctypes.c_ulonglong,  # cTkStackVector<std::pair<cGcRegionKnowledge const *,unsigned short>,64,-1> *
        ]
    ),
    "cGcCreatureRoutine::GetRoutineStepForPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCreatureRoutine::GetRoutineStepForTime": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcCreatureRoutine::GetStepDestination": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureRoutine::GetAdjustedStepDestination": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureRoutine::GetCreatureMatrixForTime": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcCreatureRoutine::GetRoutineTime": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcCreatureRoutine::RenderNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcCreatureRoutine::SetCreatureHandle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureRoutine::RepackCreature": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
        ]
    ),
    "cGcCreatureRoutine::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
        ]
    ),
    "cGcCreatureRoutine::GetRoutineByUniqueId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureRoutine *
        argtypes=[
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcCreatureRoutine::PathCell::SetPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine::PathCell *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "ConvertCreatureIDToAudioClass": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcCreatureVocals::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureVocals *
            ctypes.c_ulonglong,  # cGcCreatureVocalData *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkRandomComponent *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureVocals::ForcePlayCurrentVocal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureVocals *
        ]
    ),
    "cGcCreatureVocals::Play": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureVocals *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcCreatureVocals::PlaySnore": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureVocals *
        ]
    ),
    "cGcCreatureVocals::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureVocals *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcCreatureVocals::SetVocalType": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureVocals *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureVocals::PickCooldown": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureVocals *
        ]
    ),
    "cGcCreatureVocals::RenderNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "PlayCreatureVocal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "PlayCreatureSnore": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "sOwnedCreatureInfo::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sOwnedCreatureInfo *
            ctypes.c_ulonglong,  # const cGcPetData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "sOwnedCreatureInfo::Save": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sOwnedCreatureInfo *
            ctypes.c_ulonglong,  # cGcPetData *
        ]
    ),
    "sOwnedCreatureInfo::UpdateResourceLoading": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sOwnedCreatureInfo *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRagdollLimiter::GetRagdollForDespawn": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRagdollLimiter *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcCreatureAttractorComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureAttractorComponent *
        ]
    ),
    "cGcCreatureAttractorComponent::IsAttracted": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureAttractorComponent *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureAttractorComponent::IsBasicBait": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureAttractorComponent *
        ]
    ),
    "cGcCreatureAttractorComponent::NetSerializeOnInit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureAttractorComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "ComponentInstancercGcCreatureAttractorComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcCreatureAttractorComponent *
        ]
    ),
    "ComponentInstancercGcCreatureAttractorComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcCreatureBaitComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBaitComponent *
        ]
    ),
    "cGcCreatureBaitComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBaitComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureBaitComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBaitComponent *
        ]
    ),
    "cGcCreatureBaitComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBaitComponent *
            ctypes.c_float,  # float
        ]
    ),
    "ComponentInstancercGcCreatureBaitComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcCreatureBaitComponent *
        ]
    ),
    "ComponentInstancercGcCreatureBaitComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcShipAIHailData::GetIdentifier": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIHailData *
        ]
    ),
    "cGcPetBehaviourTree::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTree *
        ]
    ),
    "cGcMetadataBehaviourTree::cGcMetadataBehaviourTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcCreatureBehaviourTree::cGcCreatureBehaviourTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
        ]
    ),
    "cGcMetadataBehaviourTree::GetErrorString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMetadataBehaviourTree *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        ]
    ),
    "cGcCreatureBehaviourTree::~cGcCreatureBehaviourTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTree *
        ]
    ),
    "cGcShootableComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcShootableComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWalkerBehaviourTree::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWalkerBehaviourTree *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureComponentUpdater::UpdateComponents": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponentUpdater *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "ComponentInstancercGcCreatureComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcCreatureComponent *
        ]
    ),
    "ComponentInstancercGcCreatureComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcCreatureComponent::ConstructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcCreatureComponent::cGcCreatureComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "ComponentInstancercGcCreatureComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcCreatureComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::SetUpBehaviourTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_uint32,  # eCreatureType
            ctypes.c_uint32,  # eCreatureRole
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFiendBehaviourTree::cGcFiendBehaviourTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendBehaviourTree *
        ]
    ),
    "cGcPredatorBehaviourTree::cGcPredatorBehaviourTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPredatorBehaviourTree *
        ]
    ),
    "cGcUngulateBehaviourTree::cGcUngulateBehaviourTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUngulateBehaviourTree *
        ]
    ),
    "cGcSharkBehaviourTree::cGcSharkBehaviourTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSharkBehaviourTree *
        ]
    ),
    "cGcRobotBehaviourTree::cGcRobotBehaviourTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotBehaviourTree *
        ]
    ),
    "cGcCreatureComponent::AllocatePerception": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::FreePerception": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::ClassifyCreatureRidingSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureMovementWeird::cGcCreatureMovementWeird": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementWeird *
        ]
    ),
    "cGcCreatureMovementFlock::cGcCreatureMovementFlock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementFlock *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureMovementCrystal::cGcCreatureMovementCrystal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementCrystal *
        ]
    ),
    "cGcCreatureComponent::LookInfo::LookInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent::LookInfo *
        ]
    ),
    "cGcCreatureComponent::SetLookPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureComponent::SetSwarm": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cGcCreatureSwarm *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureComponent::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureComponent::GiveDeathResources": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::PlayDeathEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcCreatureComponent::GetCreatureDiscoveryConstructionData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "cGcCreatureComponent::GetFeedingProduct": {
        "cGcCreatureComponent *, TkID<128> *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # TkID<128> *
            argtypes=[
                ctypes.c_ulonglong,  # cGcCreatureComponent *
                ctypes.c_ulonglong,  # TkID<128> *
            ]
        ),
        "TkID<128> *, unsigned __int64, unsigned __int64, bool": FUNCDEF(
            restype=ctypes.c_ulonglong,  # TkID<128> *
            argtypes=[
                ctypes.c_ulonglong,  # TkID<128> *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cGcCreatureComponent::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::NetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureComponent::IsInView": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcCreatureComponent::DoEffectDeath": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::Ragdoll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::GetLeader": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::IncreaseFeedingTasksDone": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::ResetFeedingTasksToDo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureComponent::StartFeeding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::DoStartFeeding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcCreatureComponent::Feed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::DoFeed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::Harvest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureComponent::DoHarvest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_uint64,  # cTkAttachmentPtr
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureComponent::GetBumMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_float,  # const float
            ctypes.c_double,  # double
        ]
    ),
    "cGcCreatureComponent::GiveHarvestResources": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureComponent::DoHarvestingCooldown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::GetHarvestingDesc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcCreatureComponent::GetHarvestingProduct": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcCreatureComponent::GetMinHarvestResourceBlobs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::NetUpdateAttitude": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::SetFriendlyAttitude": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_uint32,  # eFriendlyAttitude
        ]
    ),
    "cGcCreatureComponent::SetCreatureMarker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_uint32,  # cGcMarkerPoint::eType
        ]
    ),
    "cGcCreatureComponent::Ride": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::GetNavRadiusAndHeight": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcCreatureComponent::GetRoughRadiusAndHeight": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcCreatureComponent::GetRoutineMoveSpeed": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::GetRidingVignetteStrength": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::GetDamageMultiplier": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcCreatureComponent::GetStartingHealth": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::UpdateDiscoveredStatus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::GetPlayerDamage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcCreatureComponent::GetDiscoryThumbnailParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureComponentData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcCreatureComponent::CanRide": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # TkID<256> *
        ]
    ),
    "cGcCreatureComponent::IsBeingRidden": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::GetRidingMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureComponent::CalcRidingMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::GetRidingLegSpread": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcCreatureComponent::OnHit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcCreatureComponent::CloneAndAttachNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcCreatureComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::GetUpdatePeriod": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::~cGcCreatureComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::NetSerializeOnInit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cGcCreatureComponent::DoPetSetUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureComponent::RefreshAccessories": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::HasLaserAccessory": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::IsDissolving": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::DoDissolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_uint32,  # cGcCreatureComponent::eDissolveMode
        ]
    ),
    "cGcCreatureComponent::UpdateDissolveProgress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_float,  # float
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureComponent::SetPetAccessoryState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_uint32,  # ePetAccessoryState
        ]
    ),
    "cGcCreatureComponent::sAccessoryInfo::UpdateLaser": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent::sAccessoryInfo *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcPetBehaviourTree *
        ]
    ),
    "cGcCreatureComponent::sAccessoryInfo::UpdateLights": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent::sAccessoryInfo *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureComponent::sAccessoryInfo::UpdateMeshRefresh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent::sAccessoryInfo *
            ctypes.c_ulonglong,  # cGcOwnerConcept *
        ]
    ),
    "cGcCreatureComponent::GetNumAccessories": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::GetAccessoryCustomisation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcResourceCustomisation *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureComponent::CreateNetworkedParticleEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcCreatureComponent::SetSketchVariableLocal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_uint32,  # eCreatureSketchVariable
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureComponent::DoCreatureNetworkEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_uint32,  # eCreatureNetworkedEvent
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCreatureComponent::DoCreatureNetworkEventLocal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_uint32,  # eCreatureNetworkedEvent
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCreatureComponent::LaunchProjectile": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcCreatureComponent::AmInterestedInOwnership": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::OnOwnershipTransfer": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcCreatureComponent::ShouldAbandon": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::UpdateInteractionTitle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureComponent::SpawnPetEgg": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureComponent::GetInitialAttitudeTowardsPlayer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureMovementBase::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementBase *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureMovementBase::SetSoundState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementBase *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureMovementWeird::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementWeird *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cGcCreatureWeirdMovementData *
        ]
    ),
    "cGcCreatureMovementWeird::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementWeird *
        ]
    ),
    "cGcCreatureMovementWeird::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementWeird *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureMovementCrystal::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementCrystal *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cGcCreatureCrystalMovementDataParams *
        ]
    ),
    "cGcCreatureMovementCrystal::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementCrystal *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcCreatureMovementCrystal::cGcCrystalMovementElement,16,-1> *
        ]
    ),
    "cGcCreatureMovementCrystal::TriggerDeathAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementCrystal *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcCreatureMovementCrystal::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementCrystal *
            ctypes.c_double,  # double
        ]
    ),
    "cGcCreatureMovementCrystal::SetSoundState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementCrystal *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureMovementCrystal::cGcCrystalMovementElement::SetState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementCrystal::cGcCrystalMovementElement *
            ctypes.c_uint32,  # cGcCreatureMovementCrystal::eState
            ctypes.c_ulonglong,  # cGcCreatureCrystalMovementDataParams *
            ctypes.c_ulonglong,  # cGcCreatureMovementCrystal *
        ]
    ),
    "cGcCreatureMovementCrystal::cGcCrystalMovementElement::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementCrystal::cGcCrystalMovementElement *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcCreatureCrystalMovementDataParams *
            ctypes.c_ulonglong,  # cGcCreatureMovementCrystal *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureMovementCrystal::cGcCrystalMovementElement::cGcTentacleChain::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementCrystal::cGcCrystalMovementElement::cGcTentacleChain *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cGcCreatureCrystalMovementDataParams *
        ]
    ),
    "cGcCreatureMovementCrystal::cGcCrystalMovementElement::cGcTentacleChain::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementCrystal::cGcCrystalMovementElement::cGcTentacleChain *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcCreatureCrystalMovementDataParams *
        ]
    ),
    "cGcCreatureMovementFlock::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementFlock *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cGcCreatureFlockMovementData *
        ]
    ),
    "cGcCreatureMovementFlock::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementFlock *
        ]
    ),
    "cGcCreatureMovementFlock::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementFlock *
        ]
    ),
    "cGcCreatureMovementFlock::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementFlock *
            ctypes.c_float,  # float
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcCreatureComponent>::UpdateComponentsRender": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcCreatureComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcCreatureComponent>::UpdateComponentsPostPhysics": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcCreatureComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkBehaviourTree::Broadcast<cGcMessageTrackTargetAlert>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_ulonglong,  # const cGcMessageTrackTargetAlert *
        ]
    ),
    "cGcCreatureMovementCrystal::GetExtraPartNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementCrystal *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcCreatureMovementFlock::NotifyOfPredator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementFlock *
            ctypes.c_ulonglong,  # cGcPathfindingObstacle *
        ]
    ),
    "cGcCreatureMovementFlock::GetExtraPartNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementFlock *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcCreatureComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcFadeNode::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFadeNode *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFullBodyIKComponent::TransferReset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
        ]
    ),
    "cGcCreatureComponent::sAccessoryInfo::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent::sAccessoryInfo *
        ]
    ),
    "cGcCreatureFullBodyIKComponent::DoIkPush": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkPhysicsComponent::AddRagdollImpact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCreatureSwarm::GetRoutine": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureRoutinePtr *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarm *
            ctypes.c_ulonglong,  # cGcCreatureRoutinePtr *
        ]
    ),
    "cGcRagdollLimiter::RegisterRagdoll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRagdollLimiter *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cTkRigidBody::ResetHasContacts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkPhysicsComponent::GetSubBody": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRigidBody *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkPhysicsComponent::GetNumSubBodies": FUNCDEF(
        restype=ctypes.c_int64,  # signed __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cGcCreatureComponent::GetTargetPointSize": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcRegionKnowledge::IsValid": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
        ]
    ),
    "cGcRegionKnowledge::EstimateCellSize": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
        ]
    ),
    "cGcRegionTerrainFar::GetRegionKnowledge": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcRegionKnowledge *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainFar *
        ]
    ),
    "cTkPhysicsComponent::ForceGraphicsToFollowPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cGcCreatureComponent::ShouldDespawnOnRagdoll": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureFullBodyIKComponent::SetNeckAngles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkPhysicsComponent::RemoveFromWorld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkPhysicsComponent::AddJointIndexToUpdateTransformForInRagdoll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkRandomComponent::AddComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRandomComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_ulonglong,  # cTkRandomComponentData *
        ]
    ),
    "cTkRigidBody::UseCharacterPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureFullBodyIKComponent::SetFootShake": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureMovementBase::TriggerDeathAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementBase *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcCreatureComponent::DestroyTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcCreatureRoutine::GetPosFacingMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint32,  # ePrimaryAxis
        ]
    ),
    "cGcCreatureRoutine::GetFacingForStep": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionKnowledge::ChunkIsAccessible": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNavChunkInfo *
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
        ]
    ),
    "cGcCreatureRoutine::ComputeTotalTime": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
            ctypes.c_ulonglong,  # _DWORD *
        ]
    ),
    "cGcCreatureRoutine::AssignAndDistributeExtraTime": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
        ]
    ),
    "cGcCreatureRoutine::ComputeTravelTimes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoutine *
            ctypes.c_float,  # float
        ]
    ),
    "cGcRegionKnowledge::RoughHeightEstimate": {
        "cGcRegionKnowledge *, const cTkVector3 *, int": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # cGcRegionKnowledge *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_int32,  # int
            ]
        ),
        "cGcRegionKnowledge *, int, int, const cTkVector3 *, int": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # cGcRegionKnowledge *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cGcRegionKnowledge::GetFirstSurfaceBelow": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cGcRegionKnowledge **
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_float,  # float
        ]
    ),
    "cGcRegionKnowledge::GetChunkForPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cGcRegionKnowledge **
            ctypes.c_ulonglong,  # unsigned __int16 *
        ]
    ),
    "cGcPathfindingObstacle::cGcPathfindingObstacle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPathfindingObstacle *
        ]
    ),
    "cGcCreatureComponent::GetSeed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkSeed *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcCreatureEffectComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEffectComponent *
        ]
    ),
    "cGcCreatureEffectComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEffectComponent *
        ]
    ),
    "cGcCreatureEffectComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEffectComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcCreatureEffectComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcCreatureEffectComponent *
        ]
    ),
    "ComponentInstancercGcCreatureEffectComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcCreatureEffectComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcCreatureEffectComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcCreatureEffectComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEffectComponent *
        ]
    ),
    "cGcCreatureEffectComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEffectComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureEffectComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEffectComponent *
        ]
    ),
    "cGcCreatureEggComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEggComponent *
        ]
    ),
    "cGcCreatureEggComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEggComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureEggComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEggComponent *
        ]
    ),
    "cGcCreatureEggComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEggComponent *
        ]
    ),
    "cGcCreatureEggComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEggComponent *
            ctypes.c_float,  # float
        ]
    ),
    "ComponentInstancercGcCreatureEggComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcCreatureEggComponent *
        ]
    ),
    "ComponentInstancercGcCreatureEggComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcCreatureEggComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcCreatureEggComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcCreatureLegIKComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureLegIKComponent *
        ]
    ),
    "cGcCreatureLegIKComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureLegIKComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcCreatureLegIKComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcCreatureLegIKComponent *
        ]
    ),
    "ComponentInstancercGcCreatureLegIKComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcCreatureLegIKComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcCreatureLegIKComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcCreatureLegIKComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureLegIKComponent *
        ]
    ),
    "cGcCreatureLegIKComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureLegIKComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureLegIK::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureLegIK *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcCreatureLegIK::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureLegIK *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCreatureSwarm::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarm *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCreatureSwarm::Remove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarm *
            ctypes.c_ulonglong,  # cGcCreatureNode *
        ]
    ),
    "cGcCreatureSwarm::Release": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarm *
        ]
    ),
    "cGcCreatureSwarm::GetFollowSwarmPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarm *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcEcosystem::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystem *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEcosystem::Release": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystem *
        ]
    ),
    "cGcEcosystem::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystem *
            ctypes.c_float,  # float
        ]
    ),
    "cGcEcosystem::GetPlayerExperienceSwarmToJoin": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureSwarm *
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystem *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcEcosystem::GetFreePetSwarm": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureSwarm *
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystem *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # cGcCreatureData *
            ctypes.c_ulonglong,  # cGcCreatureSwarmData *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcEcosystem::CanSpawnCreatures": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystem *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eCreatureRole
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcEcosystem::RegisterCreature": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystem *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eCreatureRole
            ctypes.c_ulonglong,  # cGcCreatureGUID *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcEcosystem::UnregisterCreature": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystem *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcEcosystem::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystem *
        ]
    ),
    "cGcEcosystem::RenderCreatureInspectorNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystem *
        ]
    ),
    "cGcEcosystem::RemoveNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystem *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcEcosystem::RemoveCreaturesInArea": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystem *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcEcosystem::Remove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystem *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEcosystem::GetBehaviourTreeData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureBehaviourTreeData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystem *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcEcosystem::LoadAndGetCreatureData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureData *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcFoliageComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFoliageComponent *
        ]
    ),
    "cGcFoliageComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFoliageComponent *
        ]
    ),
    "cGcFreighterBaseComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFreighterBaseComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseMiniPortalComponent::TriggerTeleport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponent *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cGcBaseBuildingManager::IsInsideSealedBuildingNode": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcEnvironment::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
        ]
    ),
    "cGcEnvironment::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
        ]
    ),
    "cGcEnvironment::Render2D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcEnvironment::ActivateInstance_Internal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_ulonglong,  # cGcProjectileImpact *
        ]
    ),
    "cGcEnvironment::ActivateInstanceCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcEnvironment::UpdateShadowingState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "GeomProbeTask::GeomProbeTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GeomProbeTask *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "SetMotionBlurStartThresholdPixels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "cGcEnvironment::UpdatePlayerEnvironmentState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
            ctypes.c_float,  # float
        ]
    ),
    "cGcEnvironment::CalculateHazardFactor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # const cTkFixedArray<float,5> *
            ctypes.c_float,  # float
        ]
    ),
    "cGcEnvironment::NormaliseHazardFactor": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eHazard
        ]
    ),
    "cGcEnvironment::CalcTemperature": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEnvironment::CalcRadiation": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEnvironment::CalcToxicity": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEnvironment::CalcLifeDrain": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcEnvironment::CalculatePlanetHazardValue": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcEnvironment::PollToConstructTextures": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
        ]
    ),
    "cGcEnvironment::EnqueueRenderReflections": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEnvironment::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
        ]
    ),
    "cGcPlayerEnvironment::IsOnboardOwnFreighter": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEnvironment *
        ]
    ),
    "cGcPlayerEnvironment::IsOnboardFreighter": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEnvironment *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcEnvironment::AssignShaders": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
        ]
    ),
    "cGcEnvironment::GetTerrainFadeTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
        ]
    ),
    "cGcEnvironment::IsPointInsideFreighterInternals": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcEnvironment::IsPointInsideSpaceBase": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcEnvironment::IsPointInsideAbandonedFreighter": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcEnvironment::IsPlayerIndoorsOnAbandonedFreighter": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
        ]
    ),
    "cGcEnvironment::SetCloudsEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironment *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcPersistencyMissionOverride>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureTagAndRarity> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcCreatureDataTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureDataTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcCreatureFilenameTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureFilenameTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcCreatureAudioTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcLegacyItemTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcCreatureBehaviourTrees>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureBehaviourTrees *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaMessageMap<2>::Broadcast<cGcMessageProjectileImpact>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
            ctypes.c_ulonglong,  # const cGcMessageProjectileImpact *
        ]
    ),
    "cGcSky::GetNightFactor": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
        ]
    ),
    "cGcRegionTerrainFar::EmergencyUpgradeBuildings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainFar *
        ]
    ),
    "cTkPhysicsManager::GetActiveTriggerList": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkStackVector<ActiveTrigger,32,-1> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
        ]
    ),
    "cGcCreatureSwarm::Add": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarm *
            ctypes.c_ulonglong,  # cGcCreatureNode *
        ]
    ),
    "cGcEcosystem::RenderCreatureInspectorNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcEcosystem::RenderNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcReflectionProbesManager::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcReflectionProbesManager *
        ]
    ),
    "cGcReflectionProbesManager::EnqueueRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcReflectionProbesManager *
        ]
    ),
    "cGcReflectionProbesManager::ConstructTextures": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcReflectionProbesManager *
        ]
    ),
    "cGcReflectionProbesManager::ClosestProbeCheck": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcReflectionProbesManager *
        ]
    ),
    "cGcReflectionProbesManager::SetupProbes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcReflectionProbesManager *
        ]
    ),
    "cGcReflectionProbesManager::EnqueueRenderTile": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcReflectionProbesManager *
        ]
    ),
    "cGcReflectionProbesManager::ClearEnvironment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcReflectionProbesManager *
        ]
    ),
    "cGcReflectionProbesManager::SwitchEnvironment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcReflectionProbesManager *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcReflectionProbesManager::ProbeReflectionsActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcReflectionProbesManager *
        ]
    ),
    "DistanceFromProbe": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "DisableDynamicNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<sNodeCache,TkSTLAllocatorShim<sNodeCache,4,-1> > *
        ]
    ),
    "cGcRegionBase::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionBase *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkRegionMapBase *
            ctypes.c_ulonglong,  # cTkVoxelArray *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionBase::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionBase *
        ]
    ),
    "cGcRegionBase::GenerateVoxels": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionBase *
        ]
    ),
    "cGcRegionBase::Polygonise": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionBase *
        ]
    ),
    "cGcRegionBase::BuildKnowledge": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionBase *
        ]
    ),
    "cGcRegionBase::PostPolygonise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionBase *
        ]
    ),
    "cGcRegionBase::PostPolygonisePopulate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionAsteroid::PostPolygonisePopulate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionAsteroid *
        ]
    ),
    "cGcRegionBase::CancelPolygonise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionBase *
        ]
    ),
    "cGcRegionBase::PolygoniserThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcRegionBase::GeneratorThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcRegionBase::KnowledgeBuilderThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcRegionBase::ThreadedGenerateVoxels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionBase *
            ctypes.c_ulonglong,  # cTkRazorLocalContext *
        ]
    ),
    "cGcRegionBase::ThreadedPolygonise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionBase *
            ctypes.c_ulonglong,  # cTkRazorLocalContext *
        ]
    ),
    "cGcRegionBase::ThreadedKnowledgeBuild": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionBase *
            ctypes.c_ulonglong,  # cTkRazorLocalContext *
        ]
    ),
    "cGcBasicNoiseData2D::GenerateNoise2D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasicNoiseData2D *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # cGcPlanetBuildingData *
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # cTkRazorLocalContext *
        ]
    ),
    "LargeRegionVoronoi": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # GPU::sVoronoiResults *
            ctypes.c_ulonglong,  # cTkLinearPoolJanitor *
        ]
    ),
    "cGcBasicNoiseData2D::GenerateBasicHeightNoise": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
        ]
    ),
    "cGcBasicNoiseData2D::LargeRegionNoise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasicNoiseData2D *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # cGcPlanetBuildingData *
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # GPU::sVoronoiResults *
        ]
    ),
    "cGcBasicNoiseData2D::CalculateMaxGradientForBuilding": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # cGcPlanetBuildingData *
            ctypes.c_double,  # double
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcBasicNoiseData2D::GenerateElevation": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_double,  # double
        ]
    ),
    "cGcGridNoise2DWrapper::Generate2D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGridNoise2DWrapper *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cTkNoiseGridData *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # cTkBitArray<unsigned int,1,32> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkBasicNoiseHelper::NoisePositionOutput *
        ]
    ),
    "cGcFeatureNoise2DWrapper::Generate2D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFeatureNoise2DWrapper *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cTkNoiseFeatureData *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # cTkBitArray<unsigned int,1,32> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionTerrain::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkRegionMapBase *
            ctypes.c_ulonglong,  # cTkVoxelArray *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionTerrain::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionTerrain::TryToClear": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionTerrain::PolygoniseTerrain": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # cTkTerrainVertexData *
        ]
    ),
    "cGcRegionTerrain::ThreadedPolygonise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # cTkRazorLocalContext *
        ]
    ),
    "cGcRegionTerrain::PostPolygoniseThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionTerrain::ThreadedGenerateVoxels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # cTkRazorLocalContext *
        ]
    ),
    "cGcRegionTerrain::PollToMapStreams": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionTerrain::Polygonise": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionTerrain::PostPolygonise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionTerrain::PartitionTrianglesByTile": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionTerrain::PartitionTriangles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionTerrain::PollToUnmapStreams": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionTerrain::CancelPolygonise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionTerrain::EditRegion": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # sRegionEdit *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # std::array<unsigned int,11> *
        ]
    ),
    "cGcRegionTerrain::ApplyEdit": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # sRegionEdit *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # std::array<unsigned int,11> *
        ]
    ),
    "cGcRegionTerrain::CleanUpLonersAfterEdit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # sRegionEdit *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # std::array<unsigned int,11> *
        ]
    ),
    "cGcRegionTerrain::IsVoxelLoner": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionTerrain::Invalidate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionTerrain::CalculateSurfaceDistanceFromWorldOffset": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcRegionTerrain::RegisterTerrainResourcePositions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionTerrainFar::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainFar *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkRegionMapBase *
            ctypes.c_ulonglong,  # cTkVoxelArray *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionTerrainFar::Polygonise": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainFar *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcRegionTerrainVeryFar::PostPolygonise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainVeryFar *
        ]
    ),
    "cGcRegionTerrainFar::PostPolygonisePopulate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainFar *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcRegionTerrainFar::KnowledgeBuilt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainFar *
        ]
    ),
    "cGcRegionTerrainFar::OnDistanceChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainFar *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionTerrainFar::Invalidate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainFar *
        ]
    ),
    "cGcRegionTerrainFar::SetHidden": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainFar *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
        ]
    ),
    "cGcRegionTerrainFar::ThreadedPolygonise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainFar *
            ctypes.c_ulonglong,  # cTkRazorLocalContext *
        ]
    ),
    "cGcRegionTerrainFar::InvalidateKnowledge": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainFar *
        ]
    ),
    "cGcRegionTerrainFar::PreAsyncKnowledgeBuild": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainFar *
        ]
    ),
    "cGcRegionTerrainNear::ThreadedKnowledgeBuild": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainNear *
            ctypes.c_ulonglong,  # cTkRazorLocalContext *
        ]
    ),
    "cGcRegionTerrainFar::GetHeightmapBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcRegionTerrainFar::sHeightmapBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainFar *
        ]
    ),
    "cGcRegionTerrainMid::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkRegionMapBase *
            ctypes.c_ulonglong,  # cTkVoxelArray *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionTerrainMid::CreatureUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionTerrainMid::Polygonise": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcRegionTerrainMid::PostPolygonise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
        ]
    ),
    "cGcRegionTerrainNear::PostPolygonisePopulate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainNear *
        ]
    ),
    "cGcRegionTerrainMid::KnowledgeBuilt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
        ]
    ),
    "cGcRegionTerrainMid::SetHidden": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcRegionTerrainMid::Invalidate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionTerrainMid::AreResourcesLoaded": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
        ]
    ),
    "cGcRegionTerrainMid::ThreadedPolygonise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
            ctypes.c_ulonglong,  # cTkRazorLocalContext *
        ]
    ),
    "cGcRegionTerrainMid::PostPolygoniseThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
        ]
    ),
    "cGcRegionTerrainMid::ThreadedKnowledgeBuild": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
            ctypes.c_ulonglong,  # cTkRazorLocalContext *
        ]
    ),
    "cGcRegionTerrainMid::ClearKnowledge": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
        ]
    ),
    "cGcRegionTerrainMid::InvalidateKnowledge": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionTerrainMid::PartitionTriangles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
        ]
    ),
    "cGcTerrainTextureData::cGcTerrainTextureData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainTextureData *
        ]
    ),
    "cGcRegionTerrainNear::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainNear *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkRegionMapBase *
            ctypes.c_ulonglong,  # cTkVoxelArray *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionTerrainNear::ThreadedPolygonise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainNear *
            ctypes.c_ulonglong,  # cTkRazorLocalContext *
        ]
    ),
    "cGcRegionTerrainNear::PollToMapStreams": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainNear *
        ]
    ),
    "cGcRegionTerrainNear::AreResourcesLoaded": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainVeryFar *
        ]
    ),
    "cGcRegionTerrainNear::Polygonise": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainNear *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcRegionTerrainNear::PostPolygonise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainNear *
        ]
    ),
    "cGcRegionTerrainNear::UpdateMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainNear *
        ]
    ),
    "cGcRegionTerrainVeryFar::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainVeryFar *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkRegionMapBase *
            ctypes.c_ulonglong,  # cTkVoxelArray *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionTerrainVeryFar::Polygonise": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainVeryFar *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcRegionTerrainVeryFar::OnDistanceChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainVeryFar *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionTerrainVeryFar::Invalidate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainVeryFar *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcRegionTerrainVeryFar::SetHidden": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainVeryFar *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
        ]
    ),
    "cGcRegionTerrainVeryFar::RemoveInstancesWithinArea": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainVeryFar *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionTerrainVeryFar::SetInstanceActivating": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainVeryFar *
            ctypes.c_ulonglong,  # cTkRigidBodyPtr *
        ]
    ),
    "cGcRegionTerrainVeryFar::ThreadedPolygonise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainVeryFar *
            ctypes.c_ulonglong,  # cTkRazorLocalContext *
        ]
    ),
    "CanApplyTerrainEdit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_int8,  # const sTerrainEditData
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "EditTerrain": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
            ctypes.c_ulonglong,  # sRegionEdit *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # std::array<unsigned int,11> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::cGcTerrainRegionMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
        ]
    ),
    "cGcTerrainRegionMap::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # cGcPlanetBuildingData *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::GetHeightAtPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTerrainRegionMapOcttree::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkRegionHeightResult *
        ]
    ),
    "cGcTerrainRegionMap::HasLodReachedState": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eRegionStatus
        ]
    ),
    "cGcTerrainRegionMap::IsLodFinishedHiding": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::RefreshRegions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # const cTkAABB *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::SearchOctreeForRegions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # const cTkAABB *
            ctypes.c_uint32,  # eRegionStatus
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # std::vector<cGcRegionTerrain *,TkSTLAllocatorShim<cGcRegionTerrain *,8,-1> > *
        ]
    ),
    "cGcTerrainRegionMap::SearchOctree": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTerrainRegionMapOcttree::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint32,  # eRegionStatus
            ctypes.c_uint32,  # eRegionStatus
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::AreAnyRegionsProcesing": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
        ]
    ),
    "cGcTerrainRegionMap::ClearAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainRegionMap::SetActiveLods": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # const std::array<bool,6> *
        ]
    ),
    "cGcTerrainRegionMap::PollToClearRegionsForLOD": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::CalculateRegionNormal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcTerrainRegionMap::SubdivideNode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # std::array<std::array<cGcTerrainRegionMap::cTkRegionStub,1500>,6> *
            ctypes.c_ulonglong,  # std::array<int,6> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::ClearOctree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
        ]
    ),
    "cGcTerrainRegionMap::BuildOctree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # std::array<std::array<cGcTerrainRegionMap::cTkRegionStub,1500>,6> *
        ]
    ),
    "cGcTerrainRegionMap::FindPersistingRegionsOnThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # std::array<std::array<cGcTerrainRegionMap::cTkRegionStub,1500>,6> *
            ctypes.c_ulonglong,  # cGcRegionTerrain **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcTerrainRegionMap::FindPersistingRegionsJobFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcTerrainRegionMap::AssignNewRegions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # std::array<std::array<cGcTerrainRegionMap::cTkRegionStub,1500>,6> *
            ctypes.c_ulonglong,  # std::array<std::array<cGcTerrainRegionMap::cTkRegionStub,1500>,6> *
        ]
    ),
    "cGcTerrainRegionMap::UpdateOctreeFromStubs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # std::array<std::array<cGcTerrainRegionMap::cTkRegionStub,1500>,6> *
            ctypes.c_ulonglong,  # std::array<std::array<cGcTerrainRegionMap::cTkRegionStub,1500>,6> *
        ]
    ),
    "cGcTerrainRegionMap::CopyRegionStubs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # std::array<std::array<cGcTerrainRegionMap::cTkRegionStub,1500>,6> *
        ]
    ),
    "cGcTerrainRegionMap::CalculateRegionDistances": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # std::array<std::array<cGcTerrainRegionMap::cTkRegionStub,1500>,6> *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcTerrainRegionMap::CreateRegionList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcRegionTerrain::SpherifyVertices<cTkQuadVertex>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAABB *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # cTkAABB *
            ctypes.c_ulonglong,  # cTkQuadVertex *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::GetCentre": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcTerrainRegionMap::cTkRegionStub::cTkRegionStub": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap::cTkRegionStub *
        ]
    ),
    "cGcRegionTerrain::UpdateMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionDecoratorObjects::FadeOut": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_float,  # float
        ]
    ),
    "cGcRegionKnowledge::BuildInterest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
        ]
    ),
    "cTkVoxel::GetTileBlend": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxel *
        ]
    ),
    "cTkVoxel::SetMaterial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxel *
            ctypes.c_char,  # char
        ]
    ),
    "cTkVoxel::SetSecondaryMaterial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxel *
            ctypes.c_char,  # char
        ]
    ),
    "cTkVoxel::SetTexturePatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxel *
            ctypes.c_float,  # float
        ]
    ),
    "cTkVoxel::ShortFromFloat": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_float,  # const float
        ]
    ),
    "cTkVoxel::SetTileBlend": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxel *
            ctypes.c_float,  # float
        ]
    ),
    "cGcRegionTerrain::SetHidden": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ubyte,  # _BOOL8
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cTkRegion::Invalidate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
        ]
    ),
    "cGcRegionBase::UnmapStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkRegion::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
        ]
    ),
    "cTkRegion::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkRegionMapBase *
            ctypes.c_ulonglong,  # cTkVoxelArray *
        ]
    ),
    "IsPlayerInFastMovingShip": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "GetSmoothTransitionsForGeneration": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "cGcTerrainRegionMap::UpdateGenerator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "cGcTerrainRegionMap::PauseGeneration": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
        ]
    ),
    "cGcTerrainRegionMap::CalculateRegionHeights": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::GenerateRegionStateMachine": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "cGcTerrainRegionMapOcttree::FindRootNodeIdx": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMapOcttree *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcTerrainRegionMapOcttree::FindNodeIdx": FUNCDEF(
        restype=ctypes.c_uint16,  # unsigned __int16
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMapOcttree *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcTerrainRegionMapOcttree::FindOverlappingNodeIndices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMapOcttree *
            ctypes.c_ulonglong,  # unsigned __int16 **
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # cTkLinearPoolJanitor *
        ]
    ),
    "cGcTerrainRegionMapOcttree::FindDeepestNodeIdx": FUNCDEF(
        restype=ctypes.c_uint16,  # unsigned __int16
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMapOcttree *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcTerrainRegionMapOcttree::SubdivideNode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMapOcttree *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcTerrainRegionMap::KickUpdateJobs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainRegionMap::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
        ]
    ),
    "cGcTerrainRegionMap::AreChildrenRenderable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::AreSiblingsRenderable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcTerrainRegionMap::AreChildrenRenderableJobFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcTerrainRegionMap::AreSiblingsRenderableJobFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcTextureBlockAllocator::cGcTextureBlockAllocator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextureBlockAllocator *
        ]
    ),
    "cGcTextureBlockAllocator::CreateTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextureBlockAllocator *
        ]
    ),
    "cGcTextureBlockAllocator::FlushBlockQueue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextureBlockAllocator *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTextureBlockAllocator::FreeBlock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextureBlockAllocator *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcTerrainTextureData *
        ]
    ),
    "cGcTerrainTextureData::GetMinMip": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcTerrainTextureData::MarkUsedPages": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainTextureData *
            ctypes.c_ulonglong,  # cTkTerrainVertex *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainTextureData::CreateTextures": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcTerrainTextureData::TextureContextDeleteFunc": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcTerrainTextureData::TextureAbortBakeFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "ApplyFeedbackTex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "ClearFeedbackTex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcTerrainTextureData::TextureCompleteProbeFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGnmxContext *
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "ClearStatusTex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcTerrainTextureData::TextureEvictFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainTextureData::GetSourceTextures": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainTextureData *
        ]
    ),
    "cGcTerrainTextureData::CheckSourceTexturesResident": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainTextureData *
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainTextureData::DebugRenderTextures": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "UberPrimitiveFunction": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_double,  # double
            ctypes.c_double,  # double
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_float,  # float
        ]
    ),
    "SuperPrimitiveFunction": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkNoiseSuperPrimitiveData *
        ]
    ),
    "BlendSuperPrimitiveData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkNoiseSuperPrimitiveData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseSuperPrimitiveData *
            ctypes.c_ulonglong,  # cTkNoiseSuperPrimitiveData *
            ctypes.c_ulonglong,  # cTkNoiseSuperPrimitiveData *
            ctypes.c_float,  # float
        ]
    ),
    "GetSuperPrimitiveData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # eNoiseGridType
            ctypes.c_ulonglong,  # cTkNoiseSuperPrimitiveData *
        ]
    ),
    "BlendSuperFormulaData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkNoiseSuperFormulaData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseSuperFormulaData *
            ctypes.c_ulonglong,  # cTkNoiseSuperFormulaData *
            ctypes.c_ulonglong,  # cTkNoiseSuperFormulaData *
            ctypes.c_float,  # float
        ]
    ),
    "GetSuperFormulaData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # eNoiseGridType
            ctypes.c_ulonglong,  # cTkNoiseSuperFormulaData *
        ]
    ),
    "SphereDensityFunction": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "SuperShape": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "SuperFunction3D": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkNoiseSuperFormulaData *
            ctypes.c_ulonglong,  # cTkNoiseSuperFormulaData *
        ]
    ),
    "Generate_InitPositions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # cGcBasicNoiseData2D *
            ctypes.c_ulonglong,  # const sGenerateRegionParams *
        ]
    ),
    "Generate_NormalizePositions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # cGcBasicNoiseData2D *
            ctypes.c_ulonglong,  # const sGenerateRegionParams *
        ]
    ),
    "Generate_BuildVoxelColumn_2D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sGenVoxelShared *
            ctypes.c_ulonglong,  # sGenVoxelDebug *
            ctypes.c_ulonglong,  # sGenVoxelTemp *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # cGcBasicNoiseData2D *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "Generate_BuildVoxelColumn_Grid": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sGenVoxelShared *
            ctypes.c_ulonglong,  # sGenVoxelDebug *
            ctypes.c_ulonglong,  # sGenVoxelTemp *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # cGcBasicNoiseData2D *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "Generate_BuildVoxelColumn_Buildings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sGenVoxelShared *
            ctypes.c_ulonglong,  # sGenVoxelDebug *
            ctypes.c_ulonglong,  # sGenVoxelTemp *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # cGcBasicNoiseData2D *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "Generate_BuildVoxelColumn_FillVoxels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # const sGenerateRegionParams *
            ctypes.c_ulonglong,  # sGenVoxelShared *
            ctypes.c_ulonglong,  # sGenVoxelDebug *
            ctypes.c_ulonglong,  # sGenVoxelTemp *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # cGcBasicNoiseData2D *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "Generate_BuildVoxels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # cGcBasicNoiseData2D *
            ctypes.c_ulonglong,  # const sGenerateRegionParams *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkRazorLocalContext *
        ]
    ),
    "Generate_MaxHeights": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # cGcBasicNoiseData2D *
            ctypes.c_ulonglong,  # const sGenerateRegionParams *
        ]
    ),
    "Generate_Edits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # const sGenerateRegionParams *
        ]
    ),
    "cGcVoxelGeneratorTerrain::Generate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # cGcPlanetBuildingData *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkRazorLocalContext *
        ]
    ),
    "cGcVoxelGeneratorTerrain::GenerateApplyEdits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # cGcPlanetBuildingData *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcVoxelGeneratorTerrain::GenerateBasic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkVoxelArray *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
        ]
    ),
    "cTkTexture::PropagateStateChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcUberNoise2DWrapper::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUberNoise2DWrapper *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkLinearPoolJanitor *
        ]
    ),
    "cGcBasicNoiseData2D::AllocFromTemp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasicNoiseData2D *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkLinearPoolJanitor *
        ]
    ),
    "cTkTexture::RefreshAniso": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
        ]
    ),
    "cTkTexture::SaveFeedbackTracking": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBuildingPlacement::GetBuildingMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcRegionDecoratorBuildings::IsLocationPlayerInitialStart": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            common.GcSeed,  # cTkSeed
            ctypes.c_ulonglong,  # const cGcPlanet *
        ]
    ),
    "cGcRegionDecoratorBuildings::GetBuildingClusterForPoint": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_ulonglong,  # const cGcPlanetBuildingData *
            ctypes.c_ulonglong,  # const GPU::sVoronoiResults *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionDecoratorBuildings::UpdateDistance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorBuildings *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcRegionDecoratorBuildings::TryToAddBuildingCluster": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorBuildings *
            ctypes.c_ulonglong,  # const cGcRegionDecoratorBuildings::sAddBuildingParams *
        ]
    ),
    "cGcRegionDecoratorBuildings::DecideInitialSettlementBuildingUnlocks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetBuildingData *
            common.GcSeed,  # cTkSeed
            ctypes.c_ulonglong,  # const int *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcRegionDecoratorBuildings::CheckBuildingValid": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorBuildings *
            ctypes.c_ulonglong,  # const cGcRegionDecoratorBuildings::sAddBuildingParams *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionDecoratorBuildings::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorBuildings *
        ]
    ),
    "cGcRegionDecoratorBuildings::SpawnBuildings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorBuildings *
        ]
    ),
    "cGcRegionDecoratorBuildings::SetupBuildingComponents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cGcBuilding *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionDecoratorBuildings::SpawnWFCBaseParts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorBuildings *
        ]
    ),
    "cGcRegionDecoratorBuildings::DespawnBuildings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorBuildings *
        ]
    ),
    "cGcRegionDecoratorBuildings::PollToUpgradeLandmarks": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorBuildings *
        ]
    ),
    "cGcRegionDecoratorBuildings::SpawnProceduralShipwreck": FUNCDEF(
        restype=None,  # void
        argtypes=[
            common.GcSeed,  # const cTkSeed
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # cGcBuilding *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcRegionDecoratorBuildings::GenerateWFCBuildings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorBuildings *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRegionDecoratorBuildings::SetUpgradeState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorBuildings *
            ctypes.c_uint32,  # cGcRegionDecoratorBuildings::eBuildingUpgradeState
        ]
    ),
    "cGcRegionDecoratorBuildings::Remove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorBuildings *
        ]
    ),
    "cGcRegionDecoratorCreatures::GenRoutinesUpdate": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorCreatures *
        ]
    ),
    "cGcRegionDecoratorCreatures::Add": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorCreatures *
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
        ]
    ),
    "cGcRegionDecoratorCreatures::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorCreatures *
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionDecoratorCreatures::Remove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorCreatures *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcRegionDecoratorCreatures::cGcCreatureNodeData,8,20> *
        ]
    ),
    "cGcRegionDecoratorCreatures::GetLocalTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorCreatures *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcRegionDecoratorCreatures::GetRegionForCreature": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcRegionDecoratorCreatures *
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRegionDecoratorCreatures::AddNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorCreatures *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcCreatureSpawnData *
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionDecoratorCreatures::CreateCreatureRoutines": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorCreatures *
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_ulonglong,  # const cGcRegionKnowledge *const *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionDecoratorCreatures::DestroyCreatureRoutines": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorCreatures *
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
        ]
    ),
    "cGcRegionDecoratorCreatures::CollectRoutinePositions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorCreatures *
            ctypes.c_ulonglong,  # std::vector<cTkPhysRelVec3,TkSTLAllocatorShim<cTkPhysRelVec3,16,-1> > *
        ]
    ),
    "cGcRegionDecoratorCreatures::AddExistingCreature": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorCreatures *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionDecoratorObjects::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_uint32,  # eSpawnGroup
        ]
    ),
    "cGcRegionDecoratorObjects::UpdateDistance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionDecoratorObjects::DoUpgrading": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
        ]
    ),
    "GetNumPositionsForTriangles": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkTerrainVertex *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "GetPositionsForTriangles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkTerrainVertex *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_ulonglong,  # GPU::sTerrainVertexUnpacked *
            ctypes.c_ulonglong,  # cTkRegionHeightResult *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_int32,  # int
        ]
    ),
    "BuildObjectTypePerPositionList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const GPU::sObjectTypePriorityMap *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_ulonglong,  # GPU::sAddObjectPosIndex *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionDecoratorObjects::FillSpawnParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_ulonglong,  # GPU::sSpawnObjectGlobalParams *
            ctypes.c_ulonglong,  # GPU::sSpawnObjectPerTypeParams *
        ]
    ),
    "AddToGlobalCounterAndGetLimitedRegionCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # std::atomic<__int64> *
            ctypes.c_ulonglong,  # std::atomic<__int64> *
        ]
    ),
    "SubtractFromGlobalCounter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # std::atomic<__int64> *
        ]
    ),
    "cGcRegionDecoratorObjects::GetObjectCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_ulonglong,  # {'return': 'int', 'args': ['cGcRegionDecoratorObjects::cGcFoliageNodeData const &'], 'pointer_depth': 1}
        ]
    ),
    "cGcRegionDecoratorObjects::Add": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_ulonglong,  # cTkTerrainVertex *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionDecoratorObjects::SpawnObject": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # cGcObjectSpawnData *
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects::cGcFoliageNodeData *
            ctypes.c_ulonglong,  # const cTkRegionHeightResult *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkAABB *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionDecoratorObjects::AreFoliageResourcesLoaded": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
        ]
    ),
    "cGcRegionDecoratorObjects::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcRegionDecoratorObjects::Populate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_ulonglong,  # cGcRegionDecoratorPhysics *
        ]
    ),
    "FindClosestRayContactPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkContactPoint *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcRegionDecoratorObjects::PollToUpgradeFoliage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_ulonglong,  # cGcRegionDecoratorPhysics *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionDecoratorObjects::Downgrade": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
        ]
    ),
    "cGcRegionDecoratorObjects::SetUpgradeState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_uint32,  # cGcRegionDecoratorObjects::eFoliageUpgradeState
        ]
    ),
    "cGcRegionDecoratorObjects::Remove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcRegionDecoratorObjects::CheckLargeObjectBounds": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # eLargeObjectCoverage
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkAABB *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionDecoratorObjects::GetObjectPosition": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_ulonglong,  # cGcObjectSpawnData *
            ctypes.c_uint32,  # eTileType
            ctypes.c_ulonglong,  # const cTkRegionHeightResult *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcRegionDecoratorObjects::GetObjectMatrix": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # cGcObjectSpawnData *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcRegionDecoratorObjects::AddInstanceNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcPlanetObjectSpawnData *
        ]
    ),
    "cGcRegionDecoratorObjects::CountObjsWithinArea": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # const float
            ctypes.c_uint32,  # eType
        ]
    ),
    "cGcRegionDecoratorObjects::PickNearbyEdibleInstance": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkRigidBody **
        ]
    ),
    "cGcRegionDecoratorObjects::RemoveInstancesWithinArea": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # cGcRegionDecoratorPhysics *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionDecoratorObjects::AddNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcPlanetObjectSpawnData *
        ]
    ),
    "cGcRegionDecoratorObjects::GetInstanceCollisionNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcObjectSpawnData *
        ]
    ),
    "cGcRegionDecoratorObjects::AllocateObjectCollisionIndices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_ulonglong,  # cGcRegionDecoratorPhysics *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcRegionDecoratorObjects::AllocateInstanceCollision": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_ulonglong,  # cGcRegionDecoratorPhysics *
        ]
    ),
    "cGcRegionDecoratorObjects::FoliageRigidBody_SetRemoved": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionDecoratorObjects::FoliageRigidBody_CreatePhysicsBody": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionDecoratorObjects::ApplyPhysicsProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_ulonglong,  # cGcObjectSpawnData *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcRegionDecoratorObjects::FoliageRigidBody_StartRefreshingActiveBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
        ]
    ),
    "cGcRegionDecoratorObjects::FoliageRigidBody_RefreshActiveBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "cGcFadeNodeInstance::cGcFadeNodeInstance": {
        "cGcFadeNodeInstance *, const cGcFadeNodeInstance *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcFadeNodeInstance *
                ctypes.c_ulonglong,  # const cGcFadeNodeInstance *
            ]
        ),
        "cGcFadeNodeInstance *, cGcFadeNodeInstance *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcFadeNodeInstance *
                ctypes.c_ulonglong,  # cGcFadeNodeInstance *
            ]
        ),
        "cGcFadeNodeInstance *, unsigned __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcFadeNodeInstance *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
    },
    "cGcFadeNodeInstance::SetNodeParamF": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFadeNodeInstance *
            ctypes.c_float,  # float
        ]
    ),
    "cGcRegionDecoratorBuildings::sAddBuildingParams::sAddBuildingParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorBuildings::sAddBuildingParams *
            ctypes.c_ulonglong,  # const cGcRegionDecoratorBuildings::sAddBuildingParams *
        ]
    ),
    "cGcFadeNodeInstance::~cGcFadeNodeInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFadeNodeInstance *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcFadeNodeInstance::SetNodeActivation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFadeNodeInstance *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFadeNodeInstance::AddNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFadeNodeInstance *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkMatrix44::cTkMatrix44": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix44 *
            ctypes.c_ulonglong,  # const float *
        ]
    ),
    "cGcRegionDecoratorObjects::CountPredicate_Objects": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRegionDecoratorObjects::cGcFoliageNodeData *
        ]
    ),
    "cGcRegionDecoratorObjects::CountPredicate_Physics": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRegionDecoratorObjects::cGcFoliageNodeData *
        ]
    ),
    "cGcRegionKnowledge::GetRandomSpawnPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
            ctypes.c_char,  # char
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcRegionKnowledge::GetWalkableArea": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
            ctypes.c_char,  # char
        ]
    ),
    "cGcRegionDecoratorBuildings::cGcUpgradeMarker::cGcUpgradeMarker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorBuildings::cGcUpgradeMarker *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcPlanet::GetShipStartBuilding": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBuilding *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcRegionDecoratorPhysics::Add": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorPhysics *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionDecoratorPhysics::Generate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorPhysics *
            ctypes.c_ulonglong,  # cTkTerrainVertex *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cGcRegionDecoratorPhysics::Remove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorPhysics *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcRegionDecoratorPhysics::AllocCollision": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorPhysics *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionDecoratorPhysics::ConstructCollision": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorPhysics *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionDecoratorPhysics::DestructCollision": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorPhysics *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionDecoratorPhysics::AllocRigidBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorPhysics *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionDecoratorPhysics::ConstructRigidBody": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorPhysics *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionDecoratorPhysics::DestructRigidBody": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorPhysics *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcRegionDecoratorPhysics::BatchDestructBodiesAndCollision": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorPhysics *
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int16,  # __int16
        ]
    ),
    "cGcAtmosphereEntryComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcPlayerNotifications::GetCurrentMissionScanEventTarget": FUNCDEF(
        restype=ctypes.c_char_p,  # const TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcBinoculars::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcBinoculars::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcBinoculars::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBinoculars::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcBinoculars::UpdateDiscoveryUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcBinoculars::UpdateScanUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcBinoculars::UpdateModeUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcBinoculars::UpdateMissionSurveyUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcBinoculars::IsScannableNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # ScanInfo *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
            ctypes.c_ulonglong,  # ScanInfo *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcBinoculars::GetScopeMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcBinoculars::UpdateTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBinoculars::UpdateRayCasts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
            ctypes.c_ulonglong,  # cTkContactPoint *
        ]
    ),
    "cGcBinoculars::UpdateTargetScanMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
            ctypes.c_uint32,  # cGcBinoculars::eTargetScan
        ]
    ),
    "cGcBinoculars::IsMissionSurveyTargetDistant": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcBinoculars::IsCurrentClosestScannableOutsideNormalIconRange": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcBinoculars::StartZoom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcBinoculars::UpdateScanBarProgress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBinoculars::UpdateTargetScanData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
            ctypes.c_ulonglong,  # DiscoveryResolver::DiscoveryInfo *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBinoculars::UpdateDataView": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # DiscoveryResolver::DiscoveryInfo *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBinoculars::SetSystemScanData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcBinoculars::SetPlanetScanData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcBinoculars::SetStationScanData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcBinoculars::SetNexusScanData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcBinoculars::SetAbandonedFreighterScanData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcBinoculars::PopulateDiscoveryInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
            ctypes.c_ulonglong,  # DiscoveryResolver::DiscoveryInfo *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint64,  # cTkAttachmentPtr
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBinoculars::LocIdVariation": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcBinoculars::LoadGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBinoculars::SetMarker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcBinoculars::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcBinoculars::GetMissionSurveyScannableIconType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcBinoculars::GetRange": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcBinoculars::UpdateHotspotsUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcBinoculars::GetNearestCivilianFreighterNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcBinoculars::IsModeAvailable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
            ctypes.c_uint32,  # cGcBinoculars::eMode
        ]
    ),
    "cGcBinoculars::sMissionSurvey::GetMissionSurveyClosestEvent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcScanEvent *
        argtypes=[]
    ),
    "cGcBinoculars::sMissionSurvey::GetMissionSurveyClosestScannable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcScannableComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars::sMissionSurvey *
        ]
    ),
    "cGcBinoculars::sMissionSurvey::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars::sMissionSurvey *
        ]
    ),
    "cGcBinoculars::sMissionSurvey::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars::sMissionSurvey *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMarkerPoint::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerPoint *
        ]
    ),
    "cGcMarkerPoint::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerPoint *
        ]
    ),
    "cGcMarkerPoint::IsEqual": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerPoint *
            ctypes.c_ulonglong,  # const cGcMarkerPoint *
        ]
    ),
    "cGcMarkerPoint::CanDisplay": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerPoint *
        ]
    ),
    "cGcMarkerPoint::IsSpaceStationMarker": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerPoint *
        ]
    ),
    "cGcMarkerPoint::IsOtherPlayerStuffMarker": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerPoint *
        ]
    ),
    "cGcMarkerPoint::IsSpaceAtlasMarker": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerPoint *
        ]
    ),
    "cGcMarkerPoint::IsNexusMarker": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerPoint *
        ]
    ),
    "cGcMarkerPoint::IsInteriorMarker": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerPoint *
        ]
    ),
    "cGcMarkerPoint::GetCurrentPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerPoint *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcMarkerPoint::UpdateInteractPosition": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerPoint *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcMarkerPoint::GetHorizonPosition": {
        "cGcMarkerPoint *, cTkPhysRelVec3 *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            argtypes=[
                ctypes.c_ulonglong,  # cGcMarkerPoint *
                ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ]
        ),
        "cGcMarkerPoint *, cTkPhysRelVec3 *, const cTkPhysRelVec3 *, int *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            argtypes=[
                ctypes.c_ulonglong,  # cGcMarkerPoint *
                ctypes.c_ulonglong,  # cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # int *
            ]
        ),
    },
    "cGcMarkerPoint::GetBuildingIconType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerPoint *
        ]
    ),
    "cGcMarkerPoint::GetScannerIcon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedArray<cGcScannerIcon,4> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerPoint *
        ]
    ),
    "cGcMarkerPoint::GetIcon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerPoint *
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            ctypes.c_ulonglong,  # cTkFixedArray<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,3> *
        ]
    ),
    "cGcMarkerPoint::RenderNGui": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerPoint *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMarkerPoint::GetIconTypeForBuildingClass": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # eBuildingClass
        ]
    ),
    "cGcMarkerList::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerList *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMarkerList::CheckTimers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerList *
        ]
    ),
    "cGcMarkerList::RemoveMarker": {
        "cGcMarkerList *, int": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcMarkerList *
                ctypes.c_int32,  # int
            ]
        ),
        "cGcMarkerList *, cGcMarkerPoint *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcMarkerList *
                ctypes.c_ulonglong,  # cGcMarkerPoint *
            ]
        ),
    },
    "cGcMarkerList::RemoveMarkerWithNode": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerList *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcMarkerList::RemoveMarkers": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerList *
            ctypes.c_uint32,  # cGcMarkerPoint::eType
        ]
    ),
    "cGcMarkerList::TryAddMarker": {
        "cGcMarkerList *, const cGcMarkerPoint *, bool, float, float, float, float, const cTkVector3 *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcMarkerList *
                ctypes.c_ulonglong,  # const cGcMarkerPoint *
                ctypes.c_ubyte,  # bool
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
        "cGcMarkerList *, const cGcMarkerPoint *, bool": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcMarkerList *
                ctypes.c_ulonglong,  # const cGcMarkerPoint *
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cGcRegionHotspots::sHotspot::GetDistanceToPosition": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspots::sHotspot *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcRegionHotspots::sHotspot::GetDistanceToRay": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspots::sHotspot *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcRegionHotspots::sHotspot::GetStrength": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspots::sHotspot *
            ctypes.c_float,  # float
        ]
    ),
    "cGcRegionHotspots::GetNearbyHotspots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkStackVector<cGcRegionHotspots::sHotspot,16,-1> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionHotspots::GetNearestHotspotToPosition": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcRegionHotspots::sHotspot *
        ]
    ),
    "cGcRegionHotspots::GetHotspotFromHandle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # cGcRegionHotspots::sHotspotHandle *
            ctypes.c_ulonglong,  # cGcRegionHotspots::sHotspot *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionHotspots::GetSubstanceData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRegionHotspotSubstance *
        argtypes=[
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionHotspots::GetSubstanceYieldFromAmount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionHotspots::GetSubstanceProgressFromAmount": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRegionHotspotSubstance *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cGcRegionHotspots::GetHotspotsTable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcRegionHotspotsTable *
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionHotspots::GetCategory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # eHotspotType
        ]
    ),
    "cGcRegionHotspots::GetSphericalPolarCoord": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcRegionHotspots::GetWorldOffsetPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcRegionHotspots::GetHotpotSubstanceID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # int *
            ctypes.c_uint32,  # eHotspotType
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRegionHotspots::GetSubstanceRate": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionHotspots::GenerateHotspotsForPole": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_double,  # double
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkStackVector<cGcRegionHotspots::sHotspot,16,-1> *
        ]
    ),
    "cGcRegionHotspots::GenerateHotspot": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # cGcRegionHotspots::sHotspot *
        ]
    ),
    "cGcScanEvent::ShowArrow": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEvent *
        ]
    ),
    "cGcScanEvent::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEvent *
            ctypes.c_uint32,  # eScanTable
            ctypes.c_ulonglong,  # cGcScanEventData *
            ctypes.c_ulonglong,  # cGcBuilding *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # std::pair<TkID<128>,cTkSeed> *
        ]
    ),
    "cGcScanEvent::CalculateMarkerPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEvent *
        ]
    ),
    "cGcScanEvent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEvent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcScanEvent::ShowMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEvent *
        ]
    ),
    "cGcScanEvent::ShowIcons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEvent *
        ]
    ),
    "cGcScanEvent::ShowTooltip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEvent *
        ]
    ),
    "cGcScanEvent::CheckForCompletion": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEvent *
        ]
    ),
    "cGcScanEvent::EndEventWithMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEvent *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcScanEvent::UpdateInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEvent *
        ]
    ),
    "cGcScanEvent::UpdateSpaceStationLocation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEvent *
        ]
    ),
    "cGcScanEvent::GetForcedInteraction": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInteractionComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEvent *
        ]
    ),
    "cGcScanEvent::MarkCompleted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEvent *
        ]
    ),
    "cGcScanEvent::CollectAllInteractions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # std::vector<cGcInteractionComponent *,TkSTLAllocatorShim<cGcInteractionComponent *,8,-1> > *
        ]
    ),
    "cGcScanEvent::CollectAllDestructables": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # std::vector<cGcDestructableComponent *,TkSTLAllocatorShim<cGcDestructableComponent *,8,-1> > *
        ]
    ),
    "cGcScanEvent::CheckForAllDestroyed": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEvent *
        ]
    ),
    "cGcScanEvent::GetDestructableCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEvent *
        ]
    ),
    "cGcScanEvent::CollectAllMaintenanceComponents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # std::vector<cGcMaintenanceComponent *,TkSTLAllocatorShim<cGcMaintenanceComponent *,8,-1> > *
        ]
    ),
    "cGcInterstellarEvent::IsInSeasonalTable": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcInterstellarEvent *
        ]
    ),
    "cGcSimpleScanEffectPool<3>::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleScanEffectPool<3> *
        ]
    ),
    "cGcSimpleScanEffectPool<3>::ScanNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleScanEffectPool<3> *
            ctypes.c_ulonglong,  # const cGcScanEffectData *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcRegionHotspotsTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcRegionHotspotsTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBaseMath::Log2Floor<int>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "cGcDestructableComponent::GetDiscoveryResource": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponent *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcAISpaceshipManager::GetSquad": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAIShipSpawn *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcCreatureComponent::GetCreatureRoleData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureRoleData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcBinoculars::ResetTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcPlayerNotifications::HasBlockOnTimedScans": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNotifications *
        ]
    ),
    "cGcScanEventManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
        ]
    ),
    "cGcScanEventManager::LoadMarkerIcons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
        ]
    ),
    "cGcScanEventManager::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
        ]
    ),
    "cGcScanEventManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcScanEventManager::CheckForBuildingEvent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBuilding *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # cGcBuilding *
            ctypes.c_uint32,  # eScanTable
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_float,  # float
        ]
    ),
    "cGcScanEventManager::CheckForInteractionEvent": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_ulonglong,  # const std::pair<TkID<128>,cTkSeed> *
        ]
    ),
    "cGcScanEventManager::CheckForTimedScanEvent": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
        ]
    ),
    "cGcScanEventManager::AddVehicleScanEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcScanEventManager::GetVehicleScanTable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cGcVehicleScanTableEntry *,TkSTLAllocatorShim<cGcVehicleScanTableEntry *,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # std::vector<cGcVehicleScanTableEntry *,TkSTLAllocatorShim<cGcVehicleScanTableEntry *,8,-1> > *
        ]
    ),
    "cGcScanEventManager::IsBuildingUsedInEvent": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # const cGcBuilding *
        ]
    ),
    "cGcScanEventManager::FindInterstellarEvent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcScanEventManager::InterstellarSearchResult *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager::InterstellarSearchResult *
            ctypes.c_ulonglong,  # cGcScanEventData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const std::pair<TkID<128>,cTkSeed> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventManager::FindWeekendInterstellarEvent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcScanEventManager::InterstellarSearchResult *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager::InterstellarSearchResult *
            ctypes.c_ulonglong,  # cGcScanEventData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventManager::FindInterstellarEventStepped": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcScanEventManager::InterstellarSearchResult *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager::InterstellarSearchResult *
            ctypes.c_ulonglong,  # cGcScanEventManager::SteppedInterstellarSearch *
            ctypes.c_ulonglong,  # cGcScanEventData *
            ctypes.c_ulonglong,  # const std::pair<TkID<128>,cTkSeed> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventManager::FindAndAddInterstellarEvent": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # cGcScanEventData *
            ctypes.c_uint32,  # eScanTable
            ctypes.c_ulonglong,  # std::pair<TkID<128>,cTkSeed> *
            ctypes.c_ulonglong,  # cGcPlayerMissionParticipant *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "cGcScanEventManager::FindAndAddInterstellarEventStepped": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # cGcScanEventManager::SteppedInterstellarSearch *
            ctypes.c_ulonglong,  # cGcScanEventData *
            ctypes.c_uint32,  # eScanTable
            ctypes.c_ulonglong,  # std::pair<TkID<128>,cTkSeed> *
            ctypes.c_ulonglong,  # cGcPlayerMissionParticipant *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventManager::RestartEventsWithForcedInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_uint32,  # eInteractionType
        ]
    ),
    "cGcScanEventManager::RestartEventsWithBuildingType": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_uint32,  # eBuildingType
        ]
    ),
    "cGcScanEventManager::RestartFreighterEvents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
        ]
    ),
    "cGcScanEventManager::AddInterstellarEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # cGcScanEventData *
            ctypes.c_uint32,  # eScanTable
            ctypes.c_ulonglong,  # std::pair<TkID<128>,cTkSeed> *
            ctypes.c_ulonglong,  # cGcPlayerMissionParticipant *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScanEventManager::CheckInterstellarEvent": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const std::pair<TkID<128>,cTkSeed> *
            ctypes.c_uint32,  # eGalaxyStarAnomaly
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventManager::GetBuilding": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBuilding *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # cGcScanEventData *
            ctypes.c_ulonglong,  # cGcPlayerMissionParticipant *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # std::pair<TkID<128>,cTkSeed> *
        ]
    ),
    "cGcScanEventManager::PassesPlanetChecks": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanet *
            ctypes.c_ulonglong,  # const cGcScanEventSolarSystemLookup *
            ctypes.c_ubyte,  # const bool
            ctypes.c_uint32,  # const eBuildingClass
        ]
    ),
    "cGcScanEventManager::PassesPlanetInfoChecks": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const PlanetGenerationQuery *
            ctypes.c_ulonglong,  # const cGcScanEventSolarSystemLookup *
            ctypes.c_ubyte,  # const bool
            ctypes.c_uint32,  # const eBuildingClass
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcScanEventManager::WriteMarkerStack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcScanEventSave> *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcScanEventSave> *
        ]
    ),
    "cGcScanEventManager::WriteEventToSave": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # cGcScanEvent *
            ctypes.c_ulonglong,  # cGcScanEventSave *
            ctypes.c_uint32,  # eParticipantType
        ]
    ),
    "cGcScanEventManager::RemovingEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # const cGcScanEvent *
        ]
    ),
    "cGcScanEventManager::RemovingFullStackEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # const cGcScanEventSave *
        ]
    ),
    "cGcScanEventManager::SyncEvents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
        ]
    ),
    "cGcScanEventManager::AddScanEventBuilding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # const cGcScanEventSave *
        ]
    ),
    "cGcScanEventManager::SendSyncScanEventRequest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # cGcScanEventSave *
        ]
    ),
    "cGcScanEventManager::IsPending": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # const cGcScanEventSave *
        ]
    ),
    "cGcScanEventManager::ReadMarkerStack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcScanEventSave> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcScanEventSave> *
        ]
    ),
    "cGcScanEventManager::RerollMarkerStack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
        ]
    ),
    "cGcScanEventManager::TryRewriteParticipant": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # const cGcScanEventSave *
        ]
    ),
    "cGcScanEventManager::AddRerolledEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # const cGcScanEventSave *
        ]
    ),
    "cGcScanEventManager::AddEventFromFullStack": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcScanEventManager::_AddEventResult *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # cGcScanEventManager::_AddEventResult *
            ctypes.c_ulonglong,  # cGcScanEventSave *
        ]
    ),
    "cGcScanEventManager::GetDistanceFromEvent": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcScanEventManager::EndEvent": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_ulonglong,  # const std::pair<TkID<128>,cTkSeed> *
        ]
    ),
    "cGcScanEventManager::CanStartTimeScanEvent": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
        ]
    ),
    "cGcScanEventManager::IsValidAbandonedSystemInteraction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # eInteractionType
            ctypes.c_uint32,  # eBuildingType
        ]
    ),
    "cGcScanEventManager::SteppedInterstellarSearch::Step": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager::SteppedInterstellarSearch *
        ]
    ),
    "cGcVehicleCheckpointManager::GetActiveCheckpoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcVehicleCheckpointComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
        ]
    ),
    "cGcScanManager::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanManager *
        ]
    ),
    "cGcScanManager::CanDisplayCompass": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanManager *
        ]
    ),
    "cGcScanManager::GetScanEventManager": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcScanEventManager *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanManager *
        ]
    ),
    "cGcScanManager::ActivateAerialCam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanManager *
            ctypes.c_ulonglong,  # cGcCameraAerialViewData *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcScanManager::ActivateAerialCamStraight": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanManager *
            ctypes.c_ulonglong,  # cGcCameraAerialViewData *
        ]
    ),
    "cGcScanManager::UpdateConstantMarkers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanManager *
        ]
    ),
    "cGcScanManager::UpdateScannableMarkers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanManager *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScanManager::SetNPCFreighterMarker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanManager *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcScanManager::SetPlayerFreighterMarker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanManager *
            ctypes.c_ulonglong,  # const cGcPlayerFreighterOwnership *
        ]
    ),
    "cGcScanner::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
        ]
    ),
    "cGcScanner::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
            ctypes.c_float,  # float
        ]
    ),
    "cGcScanner::GetTerrainResourceString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cGcScanner::UpdateToolScan": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
        ]
    ),
    "cGcScanner::AddMarkerToShipPilot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcScanner::UpdateZoom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanner::UpdateScanEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
            ctypes.c_float,  # float
        ]
    ),
    "cGcScanner::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
        ]
    ),
    "cGcScanner::RenderVR": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcScanner::RenderHeadsUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcScanner::TriggerAt": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cGcScanData *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcBuilding *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanner::IsScannerCharged": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
        ]
    ),
    "cGcScanner::IsHotspotSurveyingActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
        ]
    ),
    "cGcScanner::CreateAndAddNodeSpecificScan": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
            ctypes.c_uint32,  # const TkHandle
            ctypes.c_ulonglong,  # cGcScanEffectData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanner::UpdateScannedAsteroids": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
        ]
    ),
    "cGcScanner::FadeInAndOutAlphaCallback": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const ScannedNode *
        ]
    ),
    "cGcScanner::ScannedMarkersAlphaCallback": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const ScannedNode *
        ]
    ),
    "cGcScanner::ScannedMarkersBaseColourCallback": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const ScannedNode *
        ]
    ),
    "cGcScanner::ScannedMarkersFresnelIntensityCallback": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const ScannedNode *
        ]
    ),
    "cGcScanner::GetBuildingNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcScanner::CreateAndAddSimpleTimedScan": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcScanEffectData *
        ]
    ),
    "cGcScanner::RemoveTimedScan": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcScanner::UpdateScannedNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cGcScanEffectData *
        ]
    ),
    "cGcScanner::GetScanConfig": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannedNodesList::AddNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannedNodesList *
            ctypes.c_ulonglong,  # const ScannedNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannedNodesList::RemoveNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannedNodesList *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcScannedNodesList::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannedNodesList *
            ctypes.c_float,  # float
        ]
    ),
    "cGcScannedNodesList::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannedNodesList *
        ]
    ),
    "cGcScannedNodesList::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannedNodesList *
        ]
    ),
    "cGcScanner::FuelAsteroidInfo::ConstructMarker": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMarkerPoint *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner::FuelAsteroidInfo *
            ctypes.c_ulonglong,  # cGcMarkerPoint *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcScanEventTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcScanEventTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcVehicleScanTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcVehicleScanTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventManager::MissionHasAnyActiveEvents": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # std::pair<TkID<128>,cTkSeed> *
        ]
    ),
    "cGcPlayerExperienceDirector::BountyTarget": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachmentPtr *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cTkAttachmentPtr *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanner::GetLastScanTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanner *
        ]
    ),
    "DiscoveryUtils::AssembleDiscoveryDataForSolarSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cGcDiscoveryData *
        ]
    ),
    "cGcScanEventManager::RemoveAllEventsByTable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_uint32,  # eScanTable
        ]
    ),
    "cGcScanEventManager::CheckForNextEvent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcScanEvent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventManager *
            ctypes.c_ulonglong,  # cGcScanEventData *
            ctypes.c_uint32,  # eScanTable
        ]
    ),
    "cGcSimpleScanEffect::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleScanEffect *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSimpleScanEffect::ScanNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleScanEffect *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcSimpleScanEffect::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleScanEffect *
        ]
    ),
    "sSolarDay::GetSunDirection": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # sSolarDay *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "sSolarDay::GetSolarSunAngle": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # sSolarDay *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "sSolarDay::GetRelativeSunAngle": {
        "sSolarDay *, const cTkVector3 *, float": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # sSolarDay *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_float,  # float
            ]
        ),
        "float": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_float,  # float
            ]
        ),
    },
    "sSolarDay::GetSolarSunAngles": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # sSolarDay *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "sSolarDay::GetTimeOfDay": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # sSolarDay *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSky::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
        ]
    ),
    "cGcSky::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
        ]
    ),
    "cGcSky::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSky::UpdateSkyNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
        ]
    ),
    "cGcSky::UpdateFlight": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSky::SetStormState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_uint32,  # cGcSky::eStormState
        ]
    ),
    "cGcSky::UpdateWind": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
        ]
    ),
    "cGcSky::UpdateLightDirection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
        ]
    ),
    "cGcSky::UpdateLightNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
        ]
    ),
    "cGcSky::UpdateScreenFilter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSky::UpdateHeavyAir": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
        ]
    ),
    "cGcSky::UpdateSunPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
        ]
    ),
    "cGcSky::UpdateRainbow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
        ]
    ),
    "cGcSky::CleanUpWeatherEffectsAndResources": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcSky::RefreshWeatherEffectRegionCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # eRegionStatus
        ]
    ),
    "cGcSky::CanSpawn": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_ulonglong,  # const cGcWeatherEffect *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSky::SpawnWeatherEffects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcWeatherEffect *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSky::SpawnWeatherHazards": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_ulonglong,  # const cGcWeatherEffect *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSky::UpdateWeatherEffects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
            ctypes.c_ulonglong,  # cGcWeatherEffectInstance **
        ]
    ),
    "cGcSky::GetWeatherBloomOverrides": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "cGcSky::ImpulsetWeatherBloomInterpFactor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
        ]
    ),
    "cGcSky::GetStorms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cGcPlanetWeatherData *
            ctypes.c_ulonglong,  # std::vector<cGcSky::sStorm,TkSTLAllocatorShim<cGcSky::sStorm,8,-1> > *
        ]
    ),
    "cGcSky::GetNumWeatherEffectsOfTypeActive": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcSky::GetWeatherSpawnChance": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_ulonglong,  # const cGcWeatherEffect *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSky::SetSunAngle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSky::SetTimeOfDay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSky::DisablePhotoMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
        ]
    ),
    "cGcSky::CalculateColours": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlanetWeatherColourData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_ulonglong,  # cGcPlanetWeatherColourData *
            ctypes.c_uint32,  # eBiome
            ctypes.c_ulonglong,  # const cGcPlanetWeatherData *
        ]
    ),
    "cGcSky::ModifyColours": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlanetWeatherColourData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_ulonglong,  # cGcPlanetWeatherColourData *
            ctypes.c_ulonglong,  # const cGcPlanetWeatherColourData *
            ctypes.c_ulonglong,  # const cGcWeatherColourModifiers *
        ]
    ),
    "cGcSky::BlendSkyColours": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlanetWeatherColourData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_ulonglong,  # cGcPlanetWeatherColourData *
            ctypes.c_ulonglong,  # const cGcPlanetWeatherColourData *
            ctypes.c_ulonglong,  # const cGcPlanetWeatherColourData *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSky::CalculateSkyProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSkyProperties *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_ulonglong,  # cGcSkyProperties *
            ctypes.c_ulonglong,  # const cGcPlanetWeatherData *
        ]
    ),
    "cGcSky::CalculateFogProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFogProperties *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_ulonglong,  # cGcFogProperties *
            ctypes.c_ulonglong,  # const cGcPlanetWeatherData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSky::UpdateWetEffects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_ulonglong,  # const cGcFogProperties *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSky::BlendFogProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFogProperties *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_ulonglong,  # cGcFogProperties *
            ctypes.c_ulonglong,  # const cGcFogProperties *
            ctypes.c_ulonglong,  # const cGcFogProperties *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSky::CalculateHeavyAirProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlanetHeavyAirData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_ulonglong,  # cGcPlanetHeavyAirData *
            ctypes.c_ulonglong,  # const cGcPlanetWeatherData *
        ]
    ),
    "cGcWeatherColourModifiers::cGcWeatherColourModifiers": {
        "cGcWeatherColourModifiers *, const cGcWeatherColourModifiers *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcWeatherColourModifiers *
                ctypes.c_ulonglong,  # const cGcWeatherColourModifiers *
            ]
        ),
        "cGcWeatherColourModifiers *, cGcWeatherColourModifiers *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcWeatherColourModifiers *
                ctypes.c_ulonglong,  # cGcWeatherColourModifiers *
            ]
        ),
    },
    "cTkFixedArray<cGcColourModifier,5>::cTkFixedArray<cGcColourModifier,5>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcColourModifier,5> *
            ctypes.c_ulonglong,  # const cTkFixedArray<cGcColourModifier,5> *
        ]
    ),
    "cGcSky::CalculateLightShaftProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcLightShaftProperties *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_ulonglong,  # cGcLightShaftProperties *
            ctypes.c_ulonglong,  # const cGcPlanetWeatherData *
        ]
    ),
    "cGcSky::BlendHeavyAirColours": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlanetHeavyAirData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_ulonglong,  # cGcPlanetHeavyAirData *
            ctypes.c_ulonglong,  # const cGcPlanetHeavyAirData *
            ctypes.c_ulonglong,  # const cGcPlanetHeavyAirData *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSky::SetCloudProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSky *
            ctypes.c_ulonglong,  # cGcPlanetWeatherColourData *
        ]
    ),
    "cGcSky::ChooseWeightedFilenameIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcWeightedFilename> *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcWeatherEffectInstance::~cGcWeatherEffectInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherEffectInstance *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcWeatherEffectInstance::AddAudioObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherEffectInstance *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcWeatherEffectInstance::Update": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherEffectInstance *
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherEffectInstance::UpdateAudioAndFade": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherEffectInstance *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cGcWeatherEffectInstance::UpdateWander": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherEffectInstance *
        ]
    ),
    "cGcWeatherEffectUpdate::GenerateResourceAsync": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            common.GcSeed,  # const cTkSeed
            ctypes.c_ulonglong,  # cTkSmartResHandle *
        ]
    ),
    "cGcWeatherEffectUpdate::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherEffect *
            ctypes.c_ulonglong,  # cGcWeatherEffectInstance *
        ]
    ),
    "cGcWeatherEffectUpdate::UpdateMeteorHazard": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherEffect *
            ctypes.c_ulonglong,  # const cGcWeatherHazardMeteorData *
            ctypes.c_ulonglong,  # cGcWeatherEffectInstance *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherEffectUpdate::UpdateTornadoHazard": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherEffect *
            ctypes.c_ulonglong,  # const cGcWeatherHazardTornadoData *
            ctypes.c_ulonglong,  # cGcWeatherEffectInstance *
        ]
    ),
    "cGcWeatherEffectUpdate::InitLightningHazard": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherEffect *
            ctypes.c_ulonglong,  # const cGcWeatherHazardLightningData *
            ctypes.c_ulonglong,  # cGcWeatherEffectInstance *
        ]
    ),
    "cGcWeatherEffectUpdate::UpdateLightningHazard": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherEffect *
            ctypes.c_ulonglong,  # const cGcWeatherHazardLightningData *
            ctypes.c_ulonglong,  # cGcWeatherEffectInstance *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherEffectUpdate::UpdateDynamicIndicator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcWeatherEffectUpdate::UpdateStaticIndicator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcWeatherEffectUpdate::SpawnImpactGift": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherEffect *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkStackVector<cTkVector3,32,-1> *
        ]
    ),
    "cGcWeatherEffectUpdate::GetNearbyNetworkPlayerPositions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStackVector<cTkVector3,32,-1> *
        ]
    ),
    "cGcWeatherHazardGiftManager::SpawnGift": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherHazardGiftManager *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcWeatherHazardGiftManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherHazardGiftManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcWeatherHazardGiftManager::RemoveFurthestGift": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherHazardGiftManager *
        ]
    ),
    "cGcPlanetGenerator::GetBiomeData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBiomeData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # const cGcPlanetData *
        ]
    ),
    "cGcGalaxyAttributeGenerator::ClassifyVoxel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalacticVoxelCoordinate *
            ctypes.c_ulonglong,  # cGcGalaxyVoxelAttributesData *
        ]
    ),
    "cGcGalaxyAttributeGenerator::ClassifyStarKeyAttributes": {
        "const unsigned __int64, cGcGalaxyAttributeGenerator::StarSystemKeyAttributes *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_uint64,  # const unsigned __int64
                ctypes.c_ulonglong,  # cGcGalaxyAttributeGenerator::StarSystemKeyAttributes *
            ]
        ),
        "cTkParallelRNG *, unsigned __int16, cGcGalaxyVoxelAttributesData *, cGcGalaxyAttributeGenerator::StarSystemKeyAttributes *, unsigned __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkParallelRNG *
                ctypes.c_uint16,  # unsigned __int16
                ctypes.c_ulonglong,  # cGcGalaxyVoxelAttributesData *
                ctypes.c_ulonglong,  # cGcGalaxyAttributeGenerator::StarSystemKeyAttributes *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
    },
    "cGcGalaxyAttributeGenerator::ClassifyStarSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # cGcGalaxyStarAttributesData *
        ]
    ),
    "cGcGalaxyAttributeGenerator::GetStarSystemSunRotationAxis": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "LookupCLUT": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # const cTkFixedArray<cTkColour,10> *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcGalaxyAttributeGenerator::GetPrimaryVoxelColour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcGalaxyAttributeGenerator::GetSecondaryVoxelColour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcGalaxyAttributesAtAddress::Classify": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyAttributesAtAddress *
            ctypes.c_uint64,  # const unsigned __int64
        ]
    ),
    "cGcGalaxyMapUI::SolarSelectionData::SolarSelectionData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI::SolarSelectionData *
        ]
    ),
    "cGcGalaxyMapUI::NavigationStateData::NavigationStateData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI::NavigationStateData *
        ]
    ),
    "cGcGalaxyMapUI::GetSolarSelectionPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "IntroCameraLook::SmoothTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # IntroCameraLook *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
        ]
    ),
    "cGcGalaxyMap::Data::Data": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
        ]
    ),
    "cGcGalaxyMapUI::cGcGalaxyMapUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
        ]
    ),
    "cGcGalaxyMap::Data::HasWaypointForNavMode": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_uint32,  # GalaxyNavigationMode
        ]
    ),
    "cGcGalaxyMap::Data::GetPathForNavMode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # PathFindingTask *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_uint32,  # GalaxyNavigationMode
        ]
    ),
    "cGcGalaxyMap::Data::StartingIntro": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
        ]
    ),
    "cGcGalaxyMap::Data::ResetScreens": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
        ]
    ),
    "cGcGalaxyMap::Data::GetScreenSpacePosition": {
        "cGcGalaxyMap::Data *, cTkVector2 *, const cTkVector3 *, bool *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkVector2 *
            argtypes=[
                ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
                ctypes.c_ulonglong,  # cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # bool *
            ]
        ),
        "cGcGalaxyMap::Data *, cTkVector2 *, const cTkPhysRelVec3 *, bool *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkVector2 *
            argtypes=[
                ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
                ctypes.c_ulonglong,  # cTkVector2 *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # bool *
            ]
        ),
    },
    "cGcGalaxyMap::Data::DrawLine": {
        "cGcGalaxyMap::Data *, const cTkPhysRelVec3 *, const cTkPhysRelVec3 *, const cTkColour *, float, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkColour *
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcGalaxyMap::Data *, const cTkVector3 *, const cTkVector3 *, const cTkColour *, float, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkColour *
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cGcGalaxyMap::PreloadResources": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcGalaxyMap::DrawCircle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcGalaxyMap::Data::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcGalaxyMap::Data::CommonReset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
        ]
    ),
    "cGcGalaxyMap::Data::StateEnter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
        ]
    ),
    "cGcGalaxyMap::Data::StateExit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
        ]
    ),
    "PathFindingTask::~PathFindingTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PathFindingTask *
        ]
    ),
    "cGcGalaxyMap::Data::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_double,  # double
            ctypes.c_int64,  # __int64
            ctypes.c_double,  # double
        ]
    ),
    "cGcGalaxyMap::Data::EnqueueRender3D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
        ]
    ),
    "MarkerSortIndex::SortMarkersOnUA": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const MarkerSortIndex *
            ctypes.c_ulonglong,  # const MarkerSortIndex *
        ]
    ),
    "cGcGalaxyMap::Data::RenderNVG": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ulonglong,  # cTkNGuiInput *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyMap::Data::RenderIntroTransition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcGalaxyMap::Data::AddSingleVoxelCube": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcGalaxyMap::Data::DoSolarSelection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ulonglong,  # const SolarQueryResult *
            ctypes.c_ubyte,  # const bool
            ctypes.c_float,  # const float
        ]
    ),
    "cGcGalaxyMap::Data::DoSolarDeselection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyMap::Data::DoSolarPopup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ulonglong,  # const SolarQueryResult *
        ]
    ),
    "cGcGalaxyMap::Data::CreateLocalLinksFromNewSelection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
        ]
    ),
    "cGcGalaxyMap::Data::InstigateFocusPullOutForPending": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
        ]
    ),
    "cGcGalaxyMap::Data::RenderSolarSelection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ulonglong,  # cGcGalaxySolarSelection *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyMap::Data::RenderSolarSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ulonglong,  # cGcGalaxySolarSelection *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcGalaxyMap::Data::RenderSolarSystemOrbit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # NVGcolor *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "cGcGalaxyMap::Data::RenderStarTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ulonglong,  # const StarTree *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcGalaxyMap::Data::RenderGeneratedPath": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ulonglong,  # const PathFindingTask *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcGalaxyMap::Data::RenderVisitedPath": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ulonglong,  # const std::vector<sVisit,TkSTLAllocatorShim<sVisit,2,-1> > *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcGalaxyMap::Data::RequestVoxelForCoordinate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGalaxyVoxel *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ulonglong,  # const cGcGalacticVoxelCoordinate *
        ]
    ),
    "cGcGalaxyMap::Data::CleanPrefetchAroundCoordinate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ulonglong,  # const cGcGalacticVoxelCoordinate *
        ]
    ),
    "cGcGalaxyMap::Data::CheckAndWrapCamera": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcGalaxyMap::Data::TryAcceptDestination": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyMap::Data::TransitionBackToGame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
        ]
    ),
    "cGcGalaxyMap::Data::InitiateWarpToNextGalaxy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
        ]
    ),
    "cGcGalaxyMap::Data::FindVoxelAlongDirectionAtGenerationCubeBoundary": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ulonglong,  # const cGcGalacticVoxelCoordinate *
            ctypes.c_ulonglong,  # const cGcGalacticVoxelCoordinate *
            ctypes.c_ulonglong,  # cGcGalacticVoxelCoordinate *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcGalaxyMap::Data::DoInputHandlingCommon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcGalaxyMap::Data::DoInputHandlingWaypoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcGalaxyMap::Data::DoInputHandlingPushToNavigate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcGalaxyMap::Data::DoInputNavigationModeSwitch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcGalaxyMap::Data::DoInputFilterModeSwitch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcGalaxyMap::Data::MoveAlongCurrentNavPath": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcGalaxyMap::Data::TransitionNavigationMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_uint32,  # const GalaxyNavigationMode
        ]
    ),
    "cGcGalaxyMap::Data::UpdateNavigationIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
        ]
    ),
    "cGcGalaxyMap::Data::TriggerSelectionTransition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ulonglong,  # cGcUniverseAddressData *
        ]
    ),
    "cGcGalaxyMap::Data::UpdateCamera": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CameraState *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ulonglong,  # CameraState *
            ctypes.c_double,  # double
        ]
    ),
    "cGcGalaxyMap::Data::UpdateCameraDependant": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcGalaxyMap::Data::UpdateLocalStarFieldEffects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const CameraState *
        ]
    ),
    "cGcGalaxyMap::Data::UpdateWaypoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcGalaxyMap::Data::UpdatePathFindingTasks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcGalaxyMap::Data::SetMapFilter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcGalaxyMap::Data::IsFilterAvailable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcGalaxyMap::Data::GetCurrentPathGoalDistanceLightYears": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
        ]
    ),
    "cGcGalaxyMap::Data::EnsurePathToCenterTaskIsValid": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
        ]
    ),
    "cGcGalaxyMap::Data::RemovePathFindingTask": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ulonglong,  # const PathFindingTask *
        ]
    ),
    "cGcGalaxyMap::Data::DiscoveryQueryCompleted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
            ctypes.c_ulonglong,  # const cGcUniverseAddressData *
            ctypes.c_uint32,  # const eDiscoveryType
            ctypes.c_ubyte,  # const bool
            ctypes.c_uint64,  # const unsigned __int64
        ]
    ),
    "cGcGalaxyMap::Data::KnowledgeRevisionWaypointsChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
        ]
    ),
    "cGcGalaxyMap::Data::IsGalaxyCenterReachable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
        ]
    ),
    "cGcGalaxyMap::TriggerGalacticMapLocationSyncToGameState": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcGalaxyMap::DiscoveryQueryCompleted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap *
            ctypes.c_ulonglong,  # const cGcUniverseAddressData *
            ctypes.c_uint32,  # const eDiscoveryType
            ctypes.c_ubyte,  # const bool
            ctypes.c_uint64,  # const unsigned __int64
        ]
    ),
    "cGcGalaxyMap::DiscoveryDataSubmitted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcGalaxyMap::KnowledgeRevisionWaypointsChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap *
        ]
    ),
    "cGcGalaxyMap::ComputeInitialSpawnVoxel": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGalacticVoxelCoordinate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalacticVoxelCoordinate *
            ctypes.c_ulonglong,  # const SystemSeed *
        ]
    ),
    "cGcGalaxyMap::ComputeBlackHoleDestinyVoxel": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGalacticVoxelCoordinate *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalacticVoxelCoordinate *
            ctypes.c_ulonglong,  # const cGcGalacticVoxelCoordinate *
            common.GcSeed,  # const cTkSeed
            ctypes.c_float,  # const float
        ]
    ),
    "cGcGalaxyMap::RenderSetup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap *
        ]
    ),
    "cGcGalaxyMap::EnqueueRender3D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyMap::Data::CanScan": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap::Data *
        ]
    ),
    "cGcGalaxyMap::PollToCompleteCurrentVoxelGeneration": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap *
        ]
    ),
    "cGcGalaxyMap::GetGalaxyVoxelCache": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGalaxyVoxelCache *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMap *
        ]
    ),
    "cTkMetaMessageWrapperTemplated<cGcAtlasShared>::Get<cGcAtlasRecvVoxel>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcAtlasShared *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetaMessageWrapperTemplated<cGcAtlasShared> *
        ]
    ),
    "cGcAtlasManager::EnqueueRequest<cGcAtlasSendRequestVoxel>": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasManager *
            ctypes.c_ulonglong,  # const cGcAtlasSendRequestVoxel *
            ctypes.c_float,  # float
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkHmdBase::GetProjectionRaw": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdBase *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcPlayer::HasWarpJumpTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "TkGalactic::ParticleInstanceData::ParticleInstanceData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkGalactic::ParticleInstanceData *
        ]
    ),
    "StarTree::LinkNode::LinkNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StarTree::LinkNode *
        ]
    ),
    "TkTraits::detail::CallIfTrivial<cGcGalaxyVoxel,0>::constructor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxel *
        ]
    ),
    "cTkStackVector<cGcGalaxyVoxelCache::VoxelRetirement,4,23>::~cTkStackVector<cGcGalaxyVoxelCache::VoxelRetirement,4,23>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStackVector<cGcGalaxyVoxelCache::VoxelRetirement,4,23> *
        ]
    ),
    "cGcGalaxyVoxelCache::cGcGalaxyVoxelCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelCache *
        ]
    ),
    "GPN::SearchContext::GetOrCreateGraphNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # GPN::GraphNode *
        argtypes=[
            ctypes.c_ulonglong,  # GPN::SearchContext *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "GPN::SearchContext::GetGoalPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # GPN::SearchContext *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "GPN::SearchContext::HasLevel1Upgrade": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
        ]
    ),
    "GPN::SearchContext::HasLevel2Upgrade": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # GPN::SearchContext *
        ]
    ),
    "GPN::SearchContext::HasLevel3Upgrade": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # GPN::SearchContext *
        ]
    ),
    "GPN::SearchContext::GetMaximumJumpDistance": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # GPN::SearchContext *
        ]
    ),
    "GPN::GraphNode::GraphNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GPN::GraphNode *
        ]
    ),
    "GPN::GraphNode::~GraphNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GPN::GraphNode *
        ]
    ),
    "cGcGalaxyMapUI::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
            ctypes.c_uint64,  # TkAudioObject
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyMapUI::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
            ctypes.c_float,  # float
        ]
    ),
    "cGcGalaxyMapUI::UpdateMessageData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
            ctypes.c_float,  # float
        ]
    ),
    "cGcGalaxyMapUI::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
            ctypes.c_ubyte,  # bool
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcGalaxyMapUI::UpdateNavigationStateData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
            ctypes.c_ulonglong,  # const cGcGalaxyMapUI::NavigationStateData *
        ]
    ),
    "cGcGalaxyMapUI::SelectSolarSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
            ctypes.c_ulonglong,  # const cGcGalaxyMapUI::SolarSelectionData *
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyMapUI::GetCurrentSolarSelectionLocation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGalaxyMapUI::SolarSelectionLocation *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
        ]
    ),
    "cGcGalaxyMapUI::GetSolarPanelMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cGcGalaxyMapUI::HasSelection": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
        ]
    ),
    "cGcGalaxyMapUI::UpdateScanningState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
            ctypes.c_uint32,  # cGcGalaxyMapUI::ScanState
        ]
    ),
    "cGcGalaxyMapUI::UpdateSolarInfoPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cGcGalaxyMapUI::UpdateSolarDiscoveryInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcGalaxyMapUI::ShowCinematicBars": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcGalaxyMapUI::RenderTextLabel": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
            ctypes.c_uint32,  # eInputAction
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcGalaxyMapUI::GenerateGalaxyName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcGalaxyMapUI::SolarInfoPanel::GatherMissionTooltips": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cGcGalaxyMapUI::SolarInfoPanel::MissionTooltip,TkSTLAllocatorShim<cGcGalaxyMapUI::SolarInfoPanel::MissionTooltip,4,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI::SolarInfoPanel *
            ctypes.c_ulonglong,  # std::vector<cGcGalaxyMapUI::SolarInfoPanel::MissionTooltip,TkSTLAllocatorShim<cGcGalaxyMapUI::SolarInfoPanel::MissionTooltip,4,-1> > *
        ]
    ),
    "cGcGalaxyMapUI::UpdateMinimap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcGalaxyMapUI::UpdateNavigationHelperText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
            ctypes.c_float,  # float
        ]
    ),
    "cGcGalaxyMapUI::UpdateFilterHelperText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
            ctypes.c_float,  # float
        ]
    ),
    "cGcGalaxyMapUI::CustomPathSelectionRenderFunction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcGalaxyMapUI::CustomScanRenderFunction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcGalaxyMapUI::RenderPathSelector": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI *
        ]
    ),
    "cGcGalaxyMapUI::SolarInfoPanel::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI::SolarInfoPanel *
        ]
    ),
    "cGcGalaxyMapUI::SolarInfoPanel::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI::SolarInfoPanel *
            ctypes.c_float,  # float
        ]
    ),
    "cGcGalaxyMapUI::SolarInfoPanel::GetHyperdriveErrorMessage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI::SolarInfoPanel *
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cGcGalaxyMapUI::SolarInfoPanel::UpdatePanelUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI::SolarInfoPanel *
        ]
    ),
    "cGcGalaxyMapUI::SolarInfoPanel::UpdateWarpLabel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI::SolarInfoPanel *
            ctypes.c_ulonglong,  # cGcNGuiTextSpecial *
        ]
    ),
    "cGcGalaxyMapUI::SolarInfoPanel::SetupDiscoveryBlob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMapUI::SolarInfoPanel *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "GcGalaxyFx::nvgMultiEdgeRadial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
        ]
    ),
    "GcGalaxyFx::nvgPulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
        ]
    ),
    "cGcGalaxySolarSelection::IncTimers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxySolarSelection *
            ctypes.c_float,  # float
        ]
    ),
    "cGcGalaxySolarSelection::DecTimers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxySolarSelection *
            ctypes.c_float,  # float
        ]
    ),
    "StarTree::LinkNode::RecurseAndRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StarTree::LinkNode *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcGalacticSolarSystemAddress::ComputeVoxelLightyearDistanceToSystem": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalacticSolarSystemAddress *
            ctypes.c_ulonglong,  # cGcGalacticSolarSystemAddress *
        ]
    ),
    "cGcGalacticVoxelCoordinate::ClampToLimits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalacticVoxelCoordinate *
        ]
    ),
    "SolarQueryResult::ComputeLightyearDistanceBetweenSolarSystems": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const SolarQueryResult *
            ctypes.c_ulonglong,  # const SolarQueryResult *
        ]
    ),
    "VoxelTraversal::WalkPathBresenham": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalacticVoxelCoordinate *
            ctypes.c_ulonglong,  # const cGcGalacticVoxelCoordinate *
            ctypes.c_ulonglong,  # std::vector<cGcGalacticVoxelCoordinate,TkSTLAllocatorShim<cGcGalacticVoxelCoordinate,2,23> > *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcGalaxyVoxelCache::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelCache *
            ctypes.c_float,  # float
        ]
    ),
    "cGcGalaxyVoxelCache::SyncSpatial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelCache *
            ctypes.c_ulonglong,  # const cGcGalacticVoxelCoordinate *
        ]
    ),
    "cGcGalaxyVoxelCache::RunSyncSpatialJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcGalaxyVoxelCache::RunVoxelGenerationJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcGalaxyVoxelCache::FindOrGenerateVoxelAtCoordinate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGalaxyVoxel *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelCache *
            ctypes.c_uint8,  # const unsigned __int8
            ctypes.c_ulonglong,  # const cGcGalacticVoxelCoordinate *
        ]
    ),
    "cGcGalaxyVoxelCache::FindVoxelAtCoordinate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGalaxyVoxel *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelCache *
            ctypes.c_ulonglong,  # const cGcGalacticVoxelCoordinate *
        ]
    ),
    "cGcGalaxyVoxelCache::SpatialQueryFindNearest": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelCache *
            ctypes.c_ulonglong,  # const cGcGalacticVoxelCoordinate *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # SolarQueryResult *
        ]
    ),
    "cGcGalaxyVoxelCache::SpatialQueryFindNearestAlongDir": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelCache *
            ctypes.c_ulonglong,  # const cGcGalacticVoxelCoordinate *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # SolarQueryResult *
        ]
    ),
    "cGcGalaxyVoxelCache::RecurseSystemLinkages": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelCache *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # robin_hood::detail::Table<1,80,unsigned __int64,int,robin_hood::hash<unsigned __int64,void>,std::equal_to<unsigned __int64>,23> *
            ctypes.c_ulonglong,  # StarTree::LinkNode *
            ctypes.c_ulonglong,  # cTkTransientHeap *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "cGcGalaxyVoxelCache::FindSpecificSolarSystemByUAIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelCache *
            ctypes.c_ulonglong,  # const cGcGalacticVoxelCoordinate *
            ctypes.c_uint16,  # const unsigned __int16
            ctypes.c_ulonglong,  # SolarQueryResult *
        ]
    ),
    "cGcGalaxyVoxelCache::BeginPointRouteSearch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelCache *
            ctypes.c_ulonglong,  # const SolarQueryResult *
            ctypes.c_ulonglong,  # const SolarQueryResult *
            ctypes.c_ulonglong,  # GPN::SearchContext *
        ]
    ),
    "cGcGalaxyVoxelCache::RunPointRouteSearch": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelCache *
            ctypes.c_ulonglong,  # GPN::SearchContext *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcGalaxyVoxelCache::GPNPopulateGraphNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelCache *
            ctypes.c_ulonglong,  # GPN::SearchInterface *
            ctypes.c_ulonglong,  # GPN::GraphNode *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcGalaxyVoxelGenerator::Populate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # cGcGalaxyVoxelData *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcGalaxyGenerationSetupData::cGcGalaxyGenerationSetupData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyGenerationSetupData *
            ctypes.c_ulonglong,  # const cGcGalaxyGenerationSetupData *
        ]
    ),
    "cGcGalaxyVoxelTransitMesh::Generate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelTransitMesh *
            ctypes.c_uint8,  # const unsigned __int8
            ctypes.c_ulonglong,  # const cGcGalacticVoxelCoordinate *
            ctypes.c_ulonglong,  # robin_hood::detail::Table<1,80,unsigned int,void,robin_hood::hash<unsigned int,void>,std::equal_to<unsigned int>,23> *
        ]
    ),
    "cGcSimulation::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulation *
        ]
    ),
    "cGcSolarSystem::cGcSolarSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcAsteroidManager::cGcAsteroidManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidManager *
        ]
    ),
    "cGcPlanetGenerator::cGcPlanetGenerator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
        ]
    ),
    "cGcSimulation::PollToPrepare": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulation *
        ]
    ),
    "cGcSimulation::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulation *
            ctypes.c_uint32,  # SimulationUpdateMode
            ctypes.c_double,  # double
            ctypes.c_double,  # double
        ]
    ),
    "cGcSimulation::PollToRelease": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulation *
        ]
    ),
    "cGcSimulation::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulation *
        ]
    ),
    "cGcSimulation::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulation *
        ]
    ),
    "cGcSimulation::StoreAndSetGroupNodeActivation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulation *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSimulation::RestoreNodeActivation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulation *
        ]
    ),
    "cGcSimulation::OnViewRecentre": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcSimulation::CanTransitionToGalacticMap": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulation *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSimulation::WarpToSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulation *
            ctypes.c_ulonglong,  # const cGcGalacticSolarSystemAddress *
        ]
    ),
    "cGcSimulation::UpdateDestructionQueues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulation *
        ]
    ),
    "cGcTradeRoute::~cGcTradeRoute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeRoute *
        ]
    ),
    "cGcSimulation::GetUpParticles": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcPlanet::cGcPlanet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcCreatureSteering::RenderDebug": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSteering *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcNavTarget::IsValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavTarget *
        ]
    ),
    "cGcNavigationQuery::IsValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavigationQuery *
        ]
    ),
    "cGcNavigation::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavigation *
            ctypes.c_ulonglong,  # std::vector<cGcNavObstacleTracker *,TkSTLAllocatorShim<cGcNavObstacleTracker *,8,-1> > *
            ctypes.c_ulonglong,  # cGcEnvironmentNavigator *
            ctypes.c_ulonglong,  # cGcNavSteering *
        ]
    ),
    "cGcNavigation::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavigation *
        ]
    ),
    "cGcNavigation::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavigation *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNavigation::CancelNavigation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavigation *
        ]
    ),
    "cGcNavigation::NavigateToPoint": {
        "cGcNavigation *, const cTkPhysRelVec3 *, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNavigation *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_float,  # float
            ]
        ),
        "cGcNavigation *, const cTkVector3 *, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNavigation *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_float,  # float
            ]
        ),
    },
    "cGcNavigation::IsAtTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavigation *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNavigation::UpdateNavTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavigation *
        ]
    ),
    "cGcNavBorderSegment::cGcNavBorderSegment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavBorderSegment *
            ctypes.c_ulonglong,  # const cGcNavEdgeHandle *
        ]
    ),
    "AirNavPhysProbeJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcAirNavigator::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAirNavigator *
        ]
    ),
    "cGcAirNavigator::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAirNavigator *
        ]
    ),
    "cGcAirNavigator::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAirNavigator *
            ctypes.c_ulonglong,  # const cGcNavigationConfig *
        ]
    ),
    "cGcAirNavigator::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAirNavigator *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcNavigationQuery *
            ctypes.c_ulonglong,  # cGcNavigationWorkingData *
        ]
    ),
    "cGcAirNavigator::ApplyToSteering": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAirNavigator *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcNavigationQuery *
            ctypes.c_ulonglong,  # cGcNavSteering *
        ]
    ),
    "cGcAirNavigator::GetNearestAccessiblePoint": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAirNavigator *
            ctypes.c_ulonglong,  # const cGcNavigationConfig *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcAirNavigator::GeneratePath": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAirNavigator *
            ctypes.c_ulonglong,  # const cGcNavigationQuery *
        ]
    ),
    "cGcAirNavigator::UpdatePath": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAirNavigator *
            ctypes.c_ulonglong,  # const cGcNavigationQuery *
        ]
    ),
    "cGcAirNavigator::GetBestSubTarget": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAirNavigator *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cGcNavigationQuery *
        ]
    ),
    "cGcAirNavigator::UpdatePhysProbes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAirNavigator *
            ctypes.c_ulonglong,  # const cGcNavigationQuery *
        ]
    ),
    "cGcAirNavigator::IsStuck": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAirNavigator *
        ]
    ),
    "cGcAirNavPhysProbeJob::ProbePhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAirNavPhysProbeJob *
        ]
    ),
    "cGcGroundNavigator::sPathData::Invalidate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundNavigator::sPathData *
        ]
    ),
    "cGcGroundNavigator::sPathData::IsValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundNavigator::sPathData *
        ]
    ),
    "cGcGroundNavigator::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundNavigator *
        ]
    ),
    "cGcGroundNavigator::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundNavigator *
        ]
    ),
    "cGcGroundNavigator::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundNavigator *
            ctypes.c_ulonglong,  # const cGcNavigationConfig *
        ]
    ),
    "cGcGroundNavigator::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundNavigator *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcNavigationQuery *
            ctypes.c_ulonglong,  # cGcNavigationWorkingData *
        ]
    ),
    "cGcGroundNavigator::ApplyToSteering": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundNavigator *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcNavigationQuery *
            ctypes.c_ulonglong,  # cGcNavSteering *
        ]
    ),
    "cGcGroundNavigator::IsPointNavigable": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundNavigator *
            ctypes.c_ulonglong,  # const cGcNavigationConfig *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcGroundNavigator::CanPathToPoint": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundNavigator *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGroundNavigator::GetNearestAccessiblePoint": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundNavigator *
            ctypes.c_ulonglong,  # const cGcNavigationConfig *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcGroundNavigator::InitKnowledge": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundNavigator *
            ctypes.c_ulonglong,  # const cGcNavigationQuery *
        ]
    ),
    "cGcGroundNavigator::GeneratePath": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundNavigator *
            ctypes.c_ulonglong,  # const cGcNavigationQuery *
        ]
    ),
    "cGcGroundNavigator::UpdatePath": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundNavigator *
            ctypes.c_ulonglong,  # const cGcNavigationQuery *
        ]
    ),
    "cGcGroundNavigator::GetBestSubTarget": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundNavigator *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cGcNavigationQuery *
        ]
    ),
    "cGcNavMeshManager::IsEdgeNavigable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshManager *
            ctypes.c_ulonglong,  # cGcNavEdgeHandle *
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
        ]
    ),
    "cGcNavMeshManager::MeshesAreAdjacent": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshManager *
            ctypes.c_ulonglong,  # const cGcNavMeshHandle *
            ctypes.c_ulonglong,  # const cGcNavMeshHandle *
        ]
    ),
    "cGcNavMeshManager::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshManager *
        ]
    ),
    "cGcNavMeshTile::cGcNavMeshTile": {
        "cGcNavMeshTile *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNavMeshTile *
            ]
        ),
        "cGcNavMeshTile *, cGcNavMeshTile *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNavMeshTile *
                ctypes.c_ulonglong,  # cGcNavMeshTile *
            ]
        ),
    },
    "cGcNavMeshManager::FreeNavMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshManager *
            ctypes.c_ulonglong,  # const cGcNavMeshHandle *
        ]
    ),
    "cGcNavMeshManager::IsSquareAllPassable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshManager *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
        ]
    ),
    "cGcNavMeshManager::CanTraverse": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshManager *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
        ]
    ),
    "cGcNavMeshManager::GetPolyNeighbors": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshManager *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
            ctypes.c_ulonglong,  # cGcNavPolyHandle *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
            ctypes.c_ulonglong,  # const cGcRegionKnowledge *const *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMeshManager::PlotPathInRoughDirection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshManager *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkStackVector<cGcNavPolyHandle,64,-1> *
        ]
    ),
    "cGcNavMeshManager::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNavMeshManager *
        argtypes=[]
    ),
    "cGcNavMeshManager::cGcNavMeshManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshManager *
        ]
    ),
    "cGcNavMeshUpdater::~cGcNavMeshUpdater": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshUpdater *
        ]
    ),
    "cGcNavMeshSpatialMap::KeysFromTile": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshSpatialMap *
            ctypes.c_ulonglong,  # const cGcNavMeshTile *
            ctypes.c_ulonglong,  # robin_hood::detail::Table<1,80,cTkIntTuple3<int,0>,void,robin_hood::hash<cTkIntTuple3<int,0>,void>,std::equal_to<cTkIntTuple3<int,0> >,-1> *
        ]
    ),
    "cGcNavMeshSpatialMap::Insert": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshSpatialMap *
            ctypes.c_ulonglong,  # const cGcNavMeshTile *
        ]
    ),
    "cGcNavMeshSpatialMap::Remove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshSpatialMap *
            ctypes.c_ulonglong,  # const cGcNavMeshTile *
        ]
    ),
    "cGcNavMeshUpdater::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshUpdater *
            ctypes.c_ulonglong,  # cGcNavMeshManager *
        ]
    ),
    "cGcNavMeshTile::CalcAabb": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkQuantizedAabb<float> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshTile *
            ctypes.c_ulonglong,  # cTkQuantizedAabb<float> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMeshTile::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshTile *
        ]
    ),
    "cGcNavMeshTile::AddPoly": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshTile *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cGcRegionKnowledge *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cGcNavChunkInfo *
        ]
    ),
    "cGcNavMeshTile::RayIntersectsPoly": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshTile *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcNavMeshTile::GetIntersectingEdges": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshTile *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkStackVector<cGcNavEdgeHandle,16,-1> *
        ]
    ),
    "cGcNavMeshTile::GetPolyNeighbors": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshTile *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
            ctypes.c_ulonglong,  # cGcNavPolyHandle *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
            ctypes.c_ulonglong,  # const cGcRegionKnowledge *const *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNavMeshTile::BuildAabbs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshTile *
        ]
    ),
    "cGcNavMeshTile::LinkEdges": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshTile *
            ctypes.c_ulonglong,  # cGcNavMeshTile *
        ]
    ),
    "cGcNavMeshTile::LinkSelfEdges": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshTile *
        ]
    ),
    "cGcNavMeshTile::DebugDraw": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshTile *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcRegionKnowledge::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
        ]
    ),
    "cGcRegionKnowledge::PrepareAsync": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcRegionDecoratorObjects *
        ]
    ),
    "cGcRegionKnowledge::LocalPointToPlanet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcRegionKnowledge::PlanetPointToLocal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcRegionKnowledge::GetWorldUp": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcRegionKnowledge::BlockOutsideCells": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
        ]
    ),
    "cGcRegionKnowledge::DecideInternalPassability": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_float,  # float
        ]
    ),
    "cGcRegionKnowledge::BuildChunkInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcRegionDecoratorObjects *
        ]
    ),
    "cGcRegionKnowledge::BuildChunkLinks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkStackVector<unsigned short,128,-1> *
            ctypes.c_ulonglong,  # cTkStackVector<enum eRegionKnowledgeDirection,128,-1> *
        ]
    ),
    "cGcRegionKnowledge::FindChunkInterest": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cGcRegionKnowledge::MarkBuildings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_ulonglong,  # const cGcRegionDecoratorObjects *
        ]
    ),
    "cGcRegionKnowledge::Layer::CanMergeRect": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge::Layer *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcRegionKnowledge::Layer::TryMergeChunks": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge::Layer *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcRegionKnowledge::Layer::Chunkify": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge::Layer *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcRegionKnowledge::AddInstances": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
        ]
    ),
    "cGcRegionKnowledge::LinkToNeighbors": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
        ]
    ),
    "cGcRegionKnowledge::MatchNewLayerAcrossBoundary": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRegionKnowledge *
            ctypes.c_ulonglong,  # const cGcRegionKnowledge **
            ctypes.c_ulonglong,  # const cGcRegionKnowledge *
            ctypes.c_ulonglong,  # const cGcRegionKnowledge *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionKnowledge::Walk": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # eRegionKnowledgeDirection
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # const cGcRegionKnowledge **
        ]
    ),
    "cGcRegionKnowledge::GetLoadedNeighboursWithinDistance": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_ulonglong,  # cGcRegionKnowledge **
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cGcRegionKnowledge::GetConnectedChunks": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkStackVector<std::pair<cGcRegionKnowledge const *,short>,600,-1> *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
            ctypes.c_ulonglong,  # const cGcRegionKnowledge *const *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionKnowledge::IsChunkPassageWideEnough": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # const cGcRegionKnowledge *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint32,  # eRegionKnowledgeDirection
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
        ]
    ),
    "cGcRegionKnowledge::GetChunkNeighbors": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # const cGcRegionKnowledge **
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # eRegionKnowledgeDirection *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
            ctypes.c_ulonglong,  # const cGcRegionKnowledge *const *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionKnowledge::PlotPathToPoint": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cGcRegionKnowledgeChunkMask *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkStackVector<std::pair<cGcRegionKnowledge const *,unsigned short>,64,-1> *
            ctypes.c_ulonglong,  # cTkStackVector<std::pair<cGcRegionKnowledge const *,unsigned short>,64,-1> *
            ctypes.c_ulonglong,  # const cGcRegionKnowledge *const *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcRegionKnowledge *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint32,  # eRegionKnowledgeInterest
        ]
    ),
    "cGcRegionKnowledge::CopyToNavMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
        ]
    ),
    "cGcRegionKnowledge::DebugRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcRegionKnowledge::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcRegionKnowledge::Remove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
        ]
    ),
    "cGcNavMeshManager::FindPolyForPoint<0>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNavPolyHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshManager *
            ctypes.c_ulonglong,  # cGcNavPolyHandle *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcNavMeshManager::FindPolyForPoint<1>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNavPolyHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshManager *
            ctypes.c_ulonglong,  # cGcNavPolyHandle *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcNavMap<16>::PointIsBlocked": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMap<16> *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSubdivBuilder<float,cGcNavMeshTile,0>::Subdivide": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSubdivBuilder<float,cGcNavMeshTile,0> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcNavMeshTile::FindPolyForPoint<0>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNavPolyHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshTile *
            ctypes.c_ulonglong,  # cGcNavPolyHandle *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cGcNavPolyHandle *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_float,  # float
        ]
    ),
    "cTkMultiVector<float,6,0,8>::Reserve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMultiVector<float,6,0,8> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMultiVector<int,2,0,8>::Reserve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMultiVector<int,2,0,8> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSubdivBuilder<float,EgInstancedModelExtension::cEgInstancedModelNode::AabbBuilderHelper,6>::Subdivide": {
        "cTkSubdivBuilder<float,EgInstancedModelExtension::cEgInstancedModelNode::AabbBuilderHelper,6> *, __int64, unsigned int, const cTkQuantizedAabb<float> *, cTkQuantizedAabb<float> *, cTkQuantizedAabb<float> *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkSubdivBuilder<float,EgInstancedModelExtension::cEgInstancedModelNode::AabbBuilderHelper,6> *
                ctypes.c_int64,  # __int64
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # const cTkQuantizedAabb<float> *
                ctypes.c_ulonglong,  # cTkQuantizedAabb<float> *
                ctypes.c_ulonglong,  # cTkQuantizedAabb<float> *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cTkSubdivBuilder<float,EgInstancedModelExtension::cEgInstancedModelNode::AabbBuilderHelper,6> *, unsigned int, unsigned int, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkSubdivBuilder<float,EgInstancedModelExtension::cEgInstancedModelNode::AabbBuilderHelper,6> *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "cTkQuantizedAabbList<float,0>::AddAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuantizedAabbList<float,0> *
            ctypes.c_ulonglong,  # const cTkQuantizedAabb<float> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "AlignUp<int>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const int *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionKnowledge::GetClosestSurface": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cGcRegionKnowledge **
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_float,  # float
        ]
    ),
    "cGcRegionKnowledge::RecomputeInterestOnEdge": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_uint32,  # eRegionKnowledgeDirection
        ]
    ),
    "cGcRegionKnowledge::GetLocalSurface": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkSphere::ProjectHemisphereToPlane": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcRegionKnowledge::Layer::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge::Layer *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionKnowledge::CheckCreateLayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkLinearPoolJanitor *
        ]
    ),
    "cGcRegionKnowledge::GetLocalCenterFlat": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcRegionKnowledge::SetupPlaneNorms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionKnowledge *
        ]
    ),
    "cGcNavMeshTile::~cGcNavMeshTile": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavMeshTile *
        ]
    ),
    "cGcNavSteering::PreUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavSteering *
            ctypes.c_ulonglong,  # const cGcNavigationQuery *
        ]
    ),
    "cGcNavSteering::GenerateRecipTurnRadii": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavSteering *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNavSteering::PickBestDirection": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavSteering *
        ]
    ),
    "cGcWeightedSpringSteering::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedSpringSteering *
            ctypes.c_ulonglong,  # const cGcNavigationConfig *
        ]
    ),
    "cGcWeightedSpringSteering::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedSpringSteering *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcNavigationQuery *
            ctypes.c_ulonglong,  # cGcNavigationWorkingData *
        ]
    ),
    "cGcWeightedSpringSteering::AddInterest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedSpringSteering *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eNavSteeringInterest
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eNavSteeringInput
        ]
    ),
    "cGcShootableComponent::AllowComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExplosion::SetMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExplosion *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cGcExplosion::CalculateDistanceScaling": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cGcExplosionData *
        ]
    ),
    "cGcExplosion::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExplosion *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cGcExplosionData *
        ]
    ),
    "cGcExplosion::DoAreaDamage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cGcAreaDamageData *
        ]
    ),
    "cGcExplosion::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExplosion *
            ctypes.c_float,  # float
        ]
    ),
    "cGcExplosion::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExplosion *
        ]
    ),
    "cGcExplosion::IsAlive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcExplosion *
        ]
    ),
    "cGcParticleManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleManager *
        ]
    ),
    "cGcParticleManager::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleManager *
        ]
    ),
    "cGcParticleManager::LoadData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleManager *
            ctypes.c_ulonglong,  # cGcExplosionData *
        ]
    ),
    "cGcParticleManager::BeginUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcParticleManager::EndUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcParticleManager::Sync": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleManager *
        ]
    ),
    "cGcParticleManager::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleManager *
        ]
    ),
    "cGcParticleManager::SetHeavyAir": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleManager *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # eHeavyAirType
        ]
    ),
    "cGcParticleManager::SetHeavyAirProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleManager *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_uint32,  # eHeavyAirType
        ]
    ),
    "cGcParticleManager::AddEffect_Internal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # EffectInstance *
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleManager *
            ctypes.c_ulonglong,  # EffectInstance *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcParticleManager::GetExplosionData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcExplosionData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcParticleManager::Save": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcParticleManager::RenderImGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleManager *
        ]
    ),
    "cGcParticleManager::CanSpawnEffect": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleManager *
            ctypes.c_ulonglong,  # const cGcExplosionData *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcParticleManager::IsActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleManager *
            ctypes.c_ulonglong,  # const EffectInstance *
        ]
    ),
    "cGcParticleManager::RetireEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleManager *
            ctypes.c_ulonglong,  # EffectInstance *
        ]
    ),
    "cGcParticleManager::RetireEffectsAndDestroyNodeOnCompletion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleManager *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcPlayerControlComponent::ControlsMovement": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::SetCamo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcPlayerAim::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerAim *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerAim::UpdateAutoAim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerAim *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerAim::UpdateComponentTargeting": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerAim *
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcPlayerAimTarget *
        ]
    ),
    "cGcPlayerAim::GetTargetPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerAim *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_uint32,  # eTargetPoint
        ]
    ),
    "cGcPlayerAim::GetTargetNormal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerAim *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_uint32,  # eTargetPoint
        ]
    ),
    "cGcPlayerAim::GetTargetBody": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRigidBody *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerAim *
            ctypes.c_uint32,  # eTargetPoint
        ]
    ),
    "cGcPlayerAim::GetAutoAimTarget": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachment *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerAim *
        ]
    ),
    "cGcPlayerAim::GetCrosshairAimPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerAim *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerAim::GetCrosshairPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerAim *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcHUDManager::GetLastDetailMessageTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManager *
        ]
    ),
    "cGcNPCComponent::DeactivateHoloEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCommunicator::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicator *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerCommunicator::SetHologramVisible": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicator *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcPlayerCommunicator::IsHologramVisible": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicator *
        ]
    ),
    "cGcPlayerCommunicator::SetSignal": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicator *
            ctypes.c_ulonglong,  # const cGcPlayerCommunicatorMessage *
            ctypes.c_ulonglong,  # const std::pair<TkID<128>,cTkSeed> *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcPlayerCommunicator::CanUseSignal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicator *
        ]
    ),
    "cGcPlayerCommunicator::RemoveSignal": {
        "cGcPlayerCommunicator *, const TkID<256>": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerCommunicator *
                ctypes.c_char * 0x20,  # const TkID<256>
            ]
        ),
        "cGcPlayerCommunicator *, const cTkAttachment *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerCommunicator *
                ctypes.c_ulonglong,  # const cTkAttachment *
            ]
        ),
        "cGcPlayerCommunicator *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerCommunicator *
            ]
        ),
    },
    "cGcPlayerCommunicator::GetResource": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicator *
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        ]
    ),
    "cGcPlayerCommunicator::CalcSpawnMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicator *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "GetAnimationId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cGcPlayerWeapon::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::AsyncPrepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_ulonglong,  # cGcResourceCustomisation *
            ctypes.c_uint64,  # TkAudioObject
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cGcOwnerConcept *
        ]
    ),
    "cGcPlayerWeapon::GetOverHeatTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetCooldownDuration": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetHeatFactor": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetLaserAmmoUseTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::FinishPrepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::UpdateVisibility": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerWeapon::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerWeapon::ForceRemoveShield": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::CheckReloadOrHolster": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerWeapon::Holster": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::InstantHolster": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::PostPhysicsUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerWeapon::RenderScreen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::Release": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::SetModeIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerWeapon::GetNumModes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetNumSecondaryModes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetWeaponRange": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerWeapon::GetWeaponSpeed": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerWeapon::GetProjectileID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_uint32,  # eWeaponMode
        ]
    ),
    "cGcPlayerWeapon::GetInventoryItemWriteable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcAtlasSendSubmitContribution *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_uint32,  # eWeaponMode
        ]
    ),
    "cGcPlayerWeapon::GetActiveInventoryItem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcInventoryElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetActiveSecondaryInventoryItem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcInventoryElement *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetBarrelPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcPlayerWeapon::GetBarrelMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcPlayerWeapon::GetDominantHandOffset": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_uint32,  # eHand
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerWeapon::GetNonDominantHandOffset": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_uint32,  # eHand
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerWeapon::GetDominantHandAttachMatrix": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_uint32,  # eHand
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcPlayerWeapon::GetNonDominantHandAttachMatrix": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_uint32,  # eHand
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcPlayerWeapon::RenderWeaponImGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetClipSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetWeaponName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcPlayerWeapon::CycleAlt": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::SetAlt": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_uint32,  # eWeaponMode
        ]
    ),
    "cGcPlayerWeapon::ResetTerrainMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::Cycle": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetClass": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # eWeaponMode
        ]
    ),
    "cGcPlayerWeapon::GetStatForWeaponMode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # eWeaponMode
        ]
    ),
    "cGcPlayerWeapon::GetModeFromStat": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # eStatsType
        ]
    ),
    "cGcPlayerWeapon::GetMaxDamagePerSecond": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryStore *
        ]
    ),
    "cGcPlayerWeapon::CancelCharging": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetChargeTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::Fire": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_uint32,  # eWeaponMode
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeapon::GetLaserColour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
        ]
    ),
    "cGcPlayerWeapon::GetHeatWarningFactor": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetAlertPoint": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetCooldownFactor": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::Melee": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeapon::IsActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetOffset": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerWeapon::FullyControlledByHands": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::AimControlledByHands": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::UseThirdPersonAnimations": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::CalculateDamagePerHit": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_uint32,  # eWeaponMode
            ctypes.c_uint32,  # ItemLookupType
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryStore *
        ]
    ),
    "cGcPlayerWeapon::CheckAmmoLevels": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_uint32,  # eWeaponMode
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeapon::ReadyWeapon": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::FireProjectile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_uint32,  # eWeaponMode
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkRigidBodyPtr *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayerWeapon::MeleeHit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcPlayerWeapon::GetHolsterPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcPlayerWeapon::DoImpact": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_ulonglong,  # cTkRigidBodyPtr *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerWeapon::PlayOneShot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerWeapon::PlayAnimation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eCurve
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeapon::StopAllAnimations": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eCurve
        ]
    ),
    "cGcPlayerWeapon::CalculateDPS": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_uint32,  # eWeaponMode
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryStore *
        ]
    ),
    "cGcPlayerWeapon::UpdateMeshRefresh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::IsMeleeActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetWeaponFireAnimId": FUNCDEF(
        restype=ctypes.c_char_p,  # const TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # eWeaponMode
        ]
    ),
    "cGcPlayerWeapon::UpdateMuzzleNodesVisibility": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetWeaponCombatEffects": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cGcImpactCombatEffectData,TkSTLAllocatorShim<cGcImpactCombatEffectData,4,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_ulonglong,  # std::vector<cGcImpactCombatEffectData,TkSTLAllocatorShim<cGcImpactCombatEffectData,4,-1> > *
            ctypes.c_uint32,  # eWeaponMode
        ]
    ),
    "cGcPlayerWeapon::GetWeaponCombatEffectsDamageMultipliers": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cGcCombatEffectDamageMultiplier,TkSTLAllocatorShim<cGcCombatEffectDamageMultiplier,4,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_ulonglong,  # std::vector<cGcCombatEffectDamageMultiplier,TkSTLAllocatorShim<cGcCombatEffectDamageMultiplier,4,-1> > *
            ctypes.c_uint32,  # eWeaponMode
        ]
    ),
    "cGcPlayerWeapon::UpdateLasersCombatEffects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetStealthChargeAmount": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::UpdateStealthCharge": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeapon::DisabledUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerWeapon::RenderWeaponTableImGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::RenderWeaponProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
            ctypes.c_uint32,  # eWeaponMode
            ctypes.c_ulonglong,  # cGcPlayerWeaponPropertiesData *
        ]
    ),
    "cTkMetadataCache<cTkSpeedLineData>::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cTkSpeedLineData> *
        ]
    ),
    "cTkMetadataCache<cTkSpeedLineData>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cTkSpeedLineData> *
        argtypes=[]
    ),
    "cTkMetadataCache<cTkTrailData>::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cTkTrailData> *
        ]
    ),
    "cTkMetadataCache<cTkTrailData>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cTkTrailData> *
        argtypes=[]
    ),
    "cTkMetadataCache<cGcExplosionDataTable>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcExplosionDataTable *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcExplosionDataTable> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcExplosionDataTable>::Save": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcExplosionDataTable> *
            ctypes.c_ulonglong,  # const cGcExplosionDataTable *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcExplosionDataTable>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cGcExplosionDataTable> *
        argtypes=[]
    ),
    "cGcEffectList<cGcExplosion,512>::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEffectList<cGcExplosion,512> *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcAreaDamageDataTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEntitlementRewardsTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditor::DoComboBox<eCommunicatorType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eCommunicatorType *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoComboBox<eWeaponClass>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eWeaponClass *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoComboBox<eStatsType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eStatsType *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoComboBox<eRemoteWeaponType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eRemoteWeaponType *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoComboBox<eBiome>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eBiome *
            ctypes.c_float,  # float
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcExplosionDataTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcExplosionDataTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDestructableComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDestructableComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLaserBeam::IsTracerMode": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeam *
        ]
    ),
    "cGcBinoculars::GetLastUsedTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcBinoculars *
        ]
    ),
    "cGcPlayerWeapon::IsStealthActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::RenderWeaponTableNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcPlayerWeapon::RenderWeaponNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcPlayerAim::SetAutoAimPos": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerAim *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcPlayerAim::GetAutoAimPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerAim *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "TkWorldMatrixToHkTransform": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkTransformf *
        argtypes=[
            ctypes.c_ulonglong,  # hkTransformf *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cGcParticleManager::RenderImGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcHumanController::GetButtonInput": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcHumanController *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcHumanController::GetAnalogInput": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcHumanController *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcQuickMenuState::UpdateErrorLabel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenuState *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # QuickMenuRenderState *
        ]
    ),
    "cGcBuildMenu::BuildGroups::NavigateStart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu::BuildGroups *
        ]
    ),
    "cGcBuildMenu::BuildGroups::NavigateUp": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu::BuildGroups *
        ]
    ),
    "cGcBuildMenu::BuildGroups::EnsureGroup": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu::BuildGroups *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # cGcBuildMenu::BuildGroups::eGroupNodeType
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcBuildMenu::BuildGroups::SetObjectColourMaterialLink": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu::BuildGroups *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildMenu::BuildGroups::GetChildNodeIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu::BuildGroups *
            ctypes.c_ulonglong,  # const cGcBuildMenu::BuildGroups::GroupNode *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBuildMenu::sBuildOptionsGui::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu::sBuildOptionsGui *
        ]
    ),
    "cGcPlayerStick::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStick *
            ctypes.c_ulonglong,  # cGcPlayerController *
            ctypes.c_ulonglong,  # cGcPlayerStickData *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # cGcUserSettings::Sensitivity
        ]
    ),
    "cGcPlayerStick::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStick *
        ]
    ),
    "cGcPlayerStick::GetSensitivityMultiplier": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_uint32,  # cGcUserSettings::Sensitivity
        ]
    ),
    "cGcPlayerStick::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStick *
        ]
    ),
    "cGcPlayerCameraLook::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCameraLook *
        ]
    ),
    "cGcBuildMenu::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildMenu::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # QuickMenuRenderState *
        ]
    ),
    "cGcBuildMenu::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
        ]
    ),
    "cGcBuildMenu::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
        ]
    ),
    "cGcBuildMenu::ShouldRenderCursor": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
        ]
    ),
    "cGcBuildMenu::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
        ]
    ),
    "cGcBuildMenu::Close": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
            ctypes.c_uint32,  # eInWorldScreens
        ]
    ),
    "cGcBuildMenu::TriggerClose": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
        ]
    ),
    "cGcBuildMenu::ShouldClose": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
        ]
    ),
    "cGcBuildMenu::UpdateControlsString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcBuildMenu::AllowedToOpenFrontendPages": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
        ]
    ),
    "cGcBuildMenu::SetBreadcrumb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcBuildMenu::UpdateBaseIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
        ]
    ),
    "cGcBuildMenu::UpdateContextualControls": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
            ctypes.c_float,  # float
        ]
    ),
    "cGcBuildMenu::UpdateSelectionMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # QuickMenuRenderState *
        ]
    ),
    "cGcBuildMenu::UpdatePlacementMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # QuickMenuRenderState *
        ]
    ),
    "cGcBuildMenu::UpdateBrowseMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # QuickMenuRenderState *
        ]
    ),
    "cGcBuildMenu::BuildBuildingGroups": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcBuildMenu::BuildGroups::GroupNode,8,-1> *
        ]
    ),
    "cGcBuildMenu::IsPointedAt": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
        ]
    ),
    "cGcBuildMenu::SetPathToBreadcrumb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
        ]
    ),
    "cGcBuildMenu::UpdateControlOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu *
        ]
    ),
    "cGcBuildMenu::BuildGroups::SetObjectAvailable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenu::BuildGroups *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcChairComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcChairComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerFullBodyIKComponent::GetCurrentSlopeNormal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponent *
        ]
    ),
    "cGcPlayerCharacterComponent::GetPhysicsMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcPlayerCharacterComponent::GetFooting": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponent *
        ]
    ),
    "cGcCharacterSit::Sit": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterSit *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterSit::TerminateSitting": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterSit *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterSit::VacateChair": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterSit *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterSit::UpdateSlideIntoPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterSit *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_double,  # double
        ]
    ),
    "cGcCharacterSit::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterSit *
            ctypes.c_double,  # double
        ]
    ),
    "cGcCharacterSit::ChangePosture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterSit *
            ctypes.c_uint32,  # eNPCSeatedPosture
        ]
    ),
    "cGcCharacterSit::Stand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterSit *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterSit::GetSitConstraintStrength": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterSit *
        ]
    ),
    "cGcCharacterSuitAudio::SetCharacterNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterSuitAudio *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcCharacterSuitAudio::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterSuitAudio *
        ]
    ),
    "cGcCharacterSuitAudio::UpdateAudioObjectPositions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterSuitAudio *
            ctypes.c_float,  # float
        ]
    ),
    "sFrigateFlybyRequest::sFrigateFlybyRequest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sFrigateFlybyRequest *
        ]
    ),
    "cGcFrigateFlyby::CreateFlyby": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlyby *
            ctypes.c_ulonglong,  # const sFrigateFlybyRequest *
        ]
    ),
    "cGcFrigateFlyby::DestroyFlyby": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlyby *
        ]
    ),
    "cGcFrigateFlyby::EndFlyby": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlyby *
        ]
    ),
    "cGcFrigateFlyby::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlyby *
        ]
    ),
    "cGcFrigateFlyby::IsSquadLeader": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlyby *
            ctypes.c_ulonglong,  # const cGcAISpaceshipComponent *
        ]
    ),
    "cGcFrigateFlyby::GetFlybyType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlyby *
        ]
    ),
    "cGcFrigateFlyby::UpdateSpawnMatrixImmediatelyPriorToSpawn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlyby *
            ctypes.c_ulonglong,  # cGcExperienceSpawn *
        ]
    ),
    "cGcFrigateFlyby::CalculateSingleShipFlybySpawnMatrix": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlyby *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcFrigateFlyby::CalculateFlybySpawnMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlyby *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateFlyby::ChooseFrigateClasses": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcFrigateFlybyLayout *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlyby *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # eFrigateFlybyType
        ]
    ),
    "cGcGalaxyPathMenu::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyPathMenu *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # QuickMenuRenderState *
        ]
    ),
    "cGcGalaxyPathMenu::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyPathMenu *
        ]
    ),
    "cGcGalaxyPathMenu::TriggerClose": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyPathMenu *
        ]
    ),
    "cGcNetworkPlayerTeleportEffect::cGcNetworkPlayerTeleportEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerTeleportEffect *
            ctypes.c_ulonglong,  # const cGcNetworkPlayerFullSim *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcNetworkPlayerTeleportEffect::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerTeleportEffect *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNetworkPlayerTeleportEffect::UpdateEffectNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayerTeleportEffect *
        ]
    ),
    "cGcPersonalTeleporter::CanUseTeleport": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersonalTeleporter *
        ]
    ),
    "cGcPersonalTeleporter::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersonalTeleporter *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPersonalTeleporter::IsManualMoveActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersonalTeleporter *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcPersonalTeleporter::IsUsedForMovement": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersonalTeleporter *
        ]
    ),
    "cGcPersonalTeleporter::GetTargetMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersonalTeleporter *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcPersonalTeleporter::UseStickForTeleport": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersonalTeleporter *
        ]
    ),
    "cGcPersonalTeleporter::GetStick": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersonalTeleporter *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "cGcPersonalTeleporter::GetDirection": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersonalTeleporter *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcPersonalTeleporter::UpdateCharging": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersonalTeleporter *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPersonalTeleporter::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersonalTeleporter *
        ]
    ),
    "cGcPersonalTeleporter::InitiateTeleport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersonalTeleporter *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersonalTeleporter::DoTeleport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersonalTeleporter *
        ]
    ),
    "cGcPersonalTeleporter::GetChargeFactor": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersonalTeleporter *
        ]
    ),
    "cGcPersonalTeleporter::GetTeleportMotionOffset": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersonalTeleporter *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcPlacementArc::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlacementArc *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcPlacementArc::Render": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlacementArc *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlacementArc::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlacementArc *
        ]
    ),
    "cGcPlacementArc::GetArcPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlacementArc *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlacementArc::CalculateAimMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlacementArc *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcPlayerThirdPerson::GetWristToHoldingPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerThirdPerson *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcPlayerThirdPerson::GetFingerPointMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerThirdPerson *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcPlayer::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cGcPlayerController *
        ]
    ),
    "cGcPlayerThirdPerson::cGcPlayerThirdPerson": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerThirdPerson *
        ]
    ),
    "cGcPlayer::GetVehicleMatrix": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcPlayer::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcPlayer::IsStealthActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetJetpackAmount": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetWaterDepth": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetWaterSurfaceFactor": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetMiningBonus": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetJetpackTankSize": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetJetpackRefillBonus": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::CanJetpackRefillMidair": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetSprintRecoveryBonus": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::UseLifeSupportForOxygenShield": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetJetpackIgnitionTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetHyperdriveFuelUse": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryStore *
        ]
    ),
    "cGcPlayer::CanEmergencyWarp": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::HasCargoShieldAvailable": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcPlayer::GetAutoTranslatorCapacity": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::CanDiscoverSecondary": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::AddRecoil": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_uint32,  # eWeaponMode
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayer::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_double,  # double
            ctypes.c_int64,  # __int64
            ctypes.c_double,  # double
        ]
    ),
    "cGcPlayer::UpdateAutoSave": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::UpdateBodyTurning": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayer::UpdateShieldsAndFuel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayer::UpdateJetpackBoosts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayer::UpdateSprintBoosts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayer::DelayThirdPersonPostPhysUpdate": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::UpdatePointing": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetPointingUnitWristMotion": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcPlayer::UpdateHandTracking": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayer::UpdateHandTrackingPostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetHandTrackingBodyMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcPlayer::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::IsFullSimLobbyHost": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetWristToSocketTransform": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcPlayer::GetWeaponAttachMatrix": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcPlayer::GetThirdPersonCharacterNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcPlayer::UpdateVRCameraOffset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcPlayer::GetVRCameraOffset": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcPlayer::GetPhysicsMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcPlayer::GetControllerMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcPlayer::GetRigidBody": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRigidBody *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetCustomPlayerControl": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerControlComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetEnvironmentLocation": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::IsWaitingForRocketBootsActivation": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::IsStillJetpackIgniting": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::HasJetpackTankAtLeastMinimumFuelForRocketBoots": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::IsUsingJetpack": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::RenderInventoryEditor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::ChangingMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_uint32,  # const cGcPlayer::PlayerMode
            ctypes.c_uint32,  # const cGcPlayer::PlayerMode
        ]
    ),
    "cGcPlayer::CalculatePlayerForce": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayer::GetDesiredControllerDir": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcPlayer::CanUseWeapon": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_uint32,  # eHand
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayer::UpdateAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayer::UpdateWarpRequests": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayer::DisablePlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayer::EnablePlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::BreakTechFromDamageType": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # const cGcPlayerDamageData *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayer::SelectTechElementToDamage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcAtlasSendSubmitContribution *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # const cGcPlayerDamageData *
        ]
    ),
    "cGcPlayer::SetToPosition": {
        "cGcPlayer *, const cTkVector3 *, const cTkVector3 *, const cTkVector3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayer *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
        "cGcPlayer *, const cTkPhysRelVec3 *, const cTkVector3 *, const cTkVector3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayer *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
    },
    "cGcPlayer::PlayHeadAnimation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_uint32,  # cGcPlayer::PlayerHeadAnim
        ]
    ),
    "cGcPlayer::IsPlayingHeadAnim": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::UpdateScanning": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayer::DoShipScan": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::DoToolScan": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::UpdateControl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_double,  # double
        ]
    ),
    "cGcPlayer::CheckForLanding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayer::UpdateUnderwaterControl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_double,  # double
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcPlayer::UpdateSteepSlopePlayerMovement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcPlayer::CanRun": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::CanRechargePersonalShield": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::FinishRidingCreature": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::CanThrowCreatureFood": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::ThrowCreatureFood": {
        "cGcPlayer *, TkID<128>, cGcCreatureComponent *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayer *
                ctypes.c_char * 0x10,  # TkID<128>
                ctypes.c_ulonglong,  # cGcCreatureComponent *
            ]
        ),
        "cGcPlayer *, TkID<128> *, cTkVector3 *, cTkVector3 *, cGcCreatureComponent *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayer *
                ctypes.c_ulonglong,  # TkID<128> *
                ctypes.c_ulonglong,  # cTkVector3 *
                ctypes.c_ulonglong,  # cTkVector3 *
                ctypes.c_ulonglong,  # cGcCreatureComponent *
            ]
        ),
    },
    "cGcPlayer::CanConsiderSummoningPet": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPlayer::GetSummonedPet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetCurrentPetBehaviour": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetPetBehaviourText": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcPlayer::ClaimPet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcPlayer::DismissSummonedPet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::SummonPet": {
        "cGcPlayer *, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayer *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "cGcPlayer *, unsigned int, const cTkMatrix34 *, eCreatureType, float, float, cTkSmartResHandle *, cTkSeed *, TkHandle": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayer *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # const cTkMatrix34 *
                ctypes.c_uint32,  # eCreatureType
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # cTkSmartResHandle *
                ctypes.c_ulonglong,  # cTkSeed *
                ctypes.c_uint32,  # TkHandle
            ]
        ),
    },
    "cGcPlayer::OwnsFriendlyDrone": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::SummonFriendlyDrone": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::ShouldDismissFriendlyDrone": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::DismissFriendlyDrone": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcPlayer::GetSummonedFriendlyDrone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSentinelRobotComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::CanBeGrabbed": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::SetGrabbedBy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayer::DoEmote": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # const cGcPlayerEmote *
        ]
    ),
    "cGcPlayer::ReleaseGrabbedBy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcPlayer::TrashShipInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::UpdateLooking": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayer::UpdateGraphics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayer::IsInControlOfShip": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::PostPhysicsUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayer::PostEngineUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayer::GetEyeLevelFromPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcPlayer::Respawn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcPlayer::TogglePerspective": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::IsOnSteepSlope": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::SetCurrentSlopeState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayer::GetActiveWeapon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcWeapon *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetVelocity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcPlayer::IsTorchAllowed": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::IsTorchLightNeeded": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::UpdateTorch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayer::NotifyPredatorAttacking": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcPlayer::SetExternalControl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcPlayer::GetVehicleComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcVehicleComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::OnEnteredCockpit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetEnergyDischargeRate": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetSpaceship": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::IsUnderwater": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::IsInSpace": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::ShouldUseAbandonedFreighterWalk": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::IsHandTracking": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcPlayer::ShouldUseEnterVehicleEffect": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayer::IsRidingCreature": {
        "cGcPlayer *, bool": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayer *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcPlayer *, cGcCreatureComponent *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayer *
                ctypes.c_ulonglong,  # cGcCreatureComponent *
            ]
        ),
    },
    "cGcPlayer::FootPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcPlayer::PhysRelHeadPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcPlayer::BodyPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcPlayer::HasRandomWarpTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cGcWarpJumpTarget *
            ctypes.c_ulonglong,  # cGcSpaceshipWarp::eWarpEffectType *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayer::GetCurrentPerspectiveBehaviour": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::IsCameraInCurrentPerspective": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::VirtualGetAttachment": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachment *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::CheckFallenThroughFloor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::UpdatePlayerGravity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::WantToShowVirtualPad": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetDominantHand": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
        ]
    ),
    "cGcPlayer::GetHandMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcPlayer::GetSmoothHandMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcPlayer::GetFingerPointMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcPlayer::GetActualWristMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_uint32,  # eHand
        ]
    ),
    "cGcPlayer::IsHoldingGrabType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_uint32,  # eGrabType
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcPlayer::CheckGesture": FUNCDEF(
        restype=ctypes.c_uint8,  # unsigned __int8
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_uint32,  # eHandGesture
        ]
    ),
    "cGcPlayer::CanUseActionButtonNow": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_uint32,  # eInputAction
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcFrigateFlybyTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFrigateFlybyTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBehaviourTree::Broadcast<cGcMessageRequestWarp>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_ulonglong,  # const cGcMessageRequestWarp *
        ]
    ),
    "cTkNGuiEditor::DoComboBox<InventoryChoice>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # InventoryChoice *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoComboBox<eInventoryType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eInventoryType *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoComboBox<eProceduralProductCategory>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eProceduralProductCategory *
            ctypes.c_float,  # float
        ]
    ),
    "cTkBehaviourTree::Broadcast<cGcMessageSummonAndDismiss>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_ulonglong,  # const cGcMessageSummonAndDismiss *
        ]
    ),
    "cGcPlayerWeapon::IsMeleeCompleted": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcPlayerWeapon::GetTimeSinceMeleeInitiated": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapon *
        ]
    ),
    "cGcUserSettings::GetVRVehiclesUseWorldControls": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettings *
        ]
    ),
    "cGcPlayer::RenderSpaceshipNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcPlayer::RenderNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcPlayer::OverrideCharacterMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_uint32,  # cGcPlayer::eCharacterMode
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerAim::GetAutoAimWeight": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerAim *
        ]
    ),
    "cGcPlayerCommon::UpdateBeingScanned": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommon *
        ]
    ),
    "cGcPlayerCommon::GetPotentialCriminal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerCommon *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ubyte,  # bool
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerCommon::AddEngagedPredator": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommon *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcPlayerCommon::CanBecomeActiveAttacker": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommon *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcPlayerCommon::CanEngage": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommon *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcPlayerCommon::TakeDamage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommon *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcPlayerCommon::IsValidCreatureAttackTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommon *
        ]
    ),
    "cGcPlayerEmoteProp::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEmoteProp *
            ctypes.c_ulonglong,  # const cGcPlayerEmotePropData *
            ctypes.c_ulonglong,  # const TkHandle *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_uint64,  # const cTkAttachmentPtr
        ]
    ),
    "cGcPlayerEmoteProp::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEmoteProp *
        ]
    ),
    "cGcPlayerEmoteProp::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEmoteProp *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcPlayerEmoteProp::CalcMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEmoteProp *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcDroneComponent::IsFriendly": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponent *
        ]
    ),
    "cGcNPCPlacementComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCPlacementComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityManager::GetEmotesList": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerEmoteList *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManager *
        ]
    ),
    "cGcExperienceSpawn::IsActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceSpawn *
        ]
    ),
    "cGcExperienceSpawn::IsInteresting": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceSpawn *
        ]
    ),
    "cGcExperienceSpawn::UpdateInterestTime": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceSpawn *
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcExperienceSpawn::UpdateMarker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceSpawn *
            ctypes.c_float,  # float
        ]
    ),
    "cGcExperienceSpawn::UpdateShipMarker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceSpawn *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcAIShipSpawn *
        ]
    ),
    "cGcExperienceSpawn::HasFinished": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceSpawn *
        ]
    ),
    "cGcExperienceSpawn::GetPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceSpawn *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcExperienceSpawn::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceSpawn *
        ]
    ),
    "cGcExperienceSpawn::CreateInvalidArea": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceSpawn *
        ]
    ),
    "cGcPlayerExperienceDirector::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::LoadCreatureResources": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::IsDebugSceneRiddenPet": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcPlayerExperienceDirector::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpawnRequest::cGcSpawnRequest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnRequest *
            ctypes.c_ulonglong,  # const cGcSpawnRequest *
        ]
    ),
    "cGcPlayerExperienceDirector::UpdateTimerAccelerators": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::SpawnTrailerNPC": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_uint64,  # cTkAttachmentPtr
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcPlayerExperienceDirector::AllowedToSpawnTraders": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::WarpSquadAway": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cGcAIShipSquad *
        ]
    ),
    "cGcPlayerExperienceDirector::UpdateNotificationMessages": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::IsPulseEncounterSpaceEgg": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::StartPirates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceDirector::StartClosestFreighterBattle": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::StartPulseEncounter": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceDirector::StartBackgroundSpaceEncounter": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::IsPulseEncounterWithShip": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::IsPulseEncounterAbandonedFreighter": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::IsAnyPlayerWithinRange": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerExperienceDirector::StartSandWorm": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::AddPirateKiller": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcPlayerExperienceDirector::UpdatePirates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerExperienceDirector::UpdatePirateRaidBuildingTarget": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::UpdatePulseEncounters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerExperienceDirector::CleanUpEncounter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::PrepareEncounter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::UpdateBackgroundSpaceEncounters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerExperienceDirector::CleanUpBackgroundSpaceEncounter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::PrepareBackgroundSpaceEncounter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::IsShipClearOfSolarSystemObjects": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerExperienceDirector::UpdateSeason": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerExperienceDirector::ShowBountyOSDMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcPlayerExperienceDirector::IsPlayerInSpace": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::BeginPirateAttack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::BeginPlanetaryPirateAttack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceDirector::ValidFreighterSpawn": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcPlayerExperienceDirector::TrySpawnAmbientFreighters": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::GenerateAmbientBountyInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBountySpawnInfo *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::RenderNGuiMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::RequestPirateBounty": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerExperienceDirector::RequestStartPirates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_uint32,  # ePirateSpawn
            ctypes.c_int32,  # int
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_char * 0x20,  # TkID<256>
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceDirector::SetSpacePirateTimer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::SetPlanetPirateTimer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::UpdatePlanetExperience": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerExperienceDirector::AddToCombatSquad": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # std::vector<cTkAttachmentPtr,TkSTLAllocatorShim<cTkAttachmentPtr,8,-1> > *
        ]
    ),
    "cGcPlayerExperienceDirector::PickRandomSentinelWave": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # const cGcSentinelWaveGroup *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceDirector::PickRandomSentinelSequence": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSentinelSpawnNamedSequence *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_uint32,  # eEncounterType
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceDirector::CanSpawnSentinelWave": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcPlayerExperienceDirector::GetPendingSentinelSummon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcPlayerExperienceDirector::ClearPendingSentinelSummon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcPlayerExperienceDirector::GetMaxSpawnableSentinels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cGcSentinelInfo::SentinelCounts *
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcPlayerExperienceDirector::GetSentinelShootCooldownModifier": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::IsCreatureInActiveFiendCombat": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcPlayerExperienceDirector::ActivateFiendsAsReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # const cGcRewardActivateFiends *
        ]
    ),
    "cGcPlayerExperienceDirector::CreatureRemoved": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcPlayerExperienceDirector::GetSpawnArchetypeData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcPlayerExperienceSpawnArchetypeData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerExperienceDirector::DoFlybyFrigateSpawn": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cGcExperienceSpawn *
            ctypes.c_uint32,  # eFrigateFlybyType
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerExperienceDirector::GetRewardFreighter": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcPlayerExperienceDirector::NotifyDroneDeploy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerExperienceDirector::DeactivateDrones": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::UpdateFreighterExperience": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerExperienceDirector::UpdateSpaceExperience": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerExperienceDirector::UpdateSpawnRequests": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerExperienceDirector::UpdateAssistanceSpawns": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerExperienceDirector::UpdateSpacePolice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerExperienceDirector::StartPolice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceDirector::EscapedPolice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::SpawnPoliceForWanted": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cGcPlayerCommon *
        ]
    ),
    "cGcPlayerExperienceDirector::SpawnPolice": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerExperienceDirector::GetPirateInterestLevel": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cGcInventoryElement *
        ]
    ),
    "cGcPlayerExperienceDirector::GetPoliceCargoInterestLevel": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cGcInventoryElement *
        ]
    ),
    "cGcPlayerExperienceDirector::RequestTraders": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cGcOutpostComponent *
        ]
    ),
    "cGcPlayerExperienceDirector::CheckForSpaceBattle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_uint32,  # cGcPlayerRespawn::RespawnReason
        ]
    ),
    "cGcPlayerExperienceDirector::GenerateSpaceBattlePosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcPlayerExperienceDirector::HasActiveSpaceBattleInCurrentSystem": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::IsPlayerCloseToSpaceBattle": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::IsPositionCloseToSpaceBattle": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceDirector::GetClosestFreighterLeader": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # eAIFaction
        ]
    ),
    "cGcPlayerExperienceDirector::GetClosestFreighter": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcPlayerExperienceDirector::GetClosestFreighterSquad": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAIShipSquad *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcPlayerExperienceDirector::GetClosestShipOfType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # const eShipType
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcPlayerExperienceDirector::PoliceIncomingTimer": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::ResetPoliceEntrance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::DoRequestShipSpawn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # const cGcExperienceSpawn *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerExperienceDirector::DoDirectShipSpawn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # const cGcExperienceSpawn *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceDirector::StartFreighters": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::IsHost": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::SpawnFreighters": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceDirector::StartSpaceBattleMainFleet": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::StartSpaceBattleFlyBy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::CalculateRandomViewableSpawn": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_double,  # double
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerExperienceDirector::SpawnInterceptFlyby": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cGcAIShipSpawnData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceDirector::ActiveSquad": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_uint64,  # cTkClassPoolHandle
        ]
    ),
    "cGcPlayerExperienceDirector::IsTradingInProgress": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::NotifyTradingEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::MessageCrimeReport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # const cGcMessageCrime *
        ]
    ),
    "cGcPlayerExperienceDirector::ReportFiendCrime": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # const cGcMessageFiendCrime *
            ctypes.c_ulonglong,  # const cGcRewardActivateFiends *
        ]
    ),
    "cGcPlayerExperienceDirector::ClearSpawns": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceDirector::ClearPirateSpawns": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::DespawnDefenceForce": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::DespawnFleetContainingFreighter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcPlayerExperienceDirector::DeployCargoShield": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::IsCargoProbeActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::IsHostileProbeActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcPlayerExperienceDirector::IsPulseEncounterAvailable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcPlayerExperienceDirector::ActiveFiendCombat": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceDirector::FreighterBattleActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcPlayerExperienceDirector::GetBountyAmount": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        ]
    ),
    "cGcPlayerExperienceDirector::GetLastPirateRaidBuildingTarget": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::GetActivePirateCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceDirector::DebugFlyby": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceDirector::SpawnTrackedShip": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachmentPtr *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cTkAttachmentPtr *
            ctypes.c_ulonglong,  # const cGcExperienceSpawn *
        ]
    ),
    "cGcPlayerExperienceDirector::GetExperienceSpawnForShip": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcExperienceSpawn *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # const cGcAISpaceshipComponent *
        ]
    ),
    "cGcPlayerExperienceDirector::NotifyShipPurchase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcPlayerExperienceDirector::SpawnSentinels": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cTkAttachmentPtr,TkSTLAllocatorShim<cTkAttachmentPtr,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # std::vector<cTkAttachmentPtr,TkSTLAllocatorShim<cTkAttachmentPtr,8,-1> > *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcSentinelInfo::SentinelCounts *
        ]
    ),
    "cGcPlayerExperienceDirector::SpawnDestroyedAsteroidCreatures": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_uint32,  # eAsteroidType
            ctypes.c_ulonglong,  # const cTkRigidBody *
        ]
    ),
    "cGcPlayerExperienceDirector::DebugSpawnSentinelMech": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachmentPtr *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cTkAttachmentPtr *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerExperienceDirector::GetCreatureResource": {
        "cGcPlayerExperienceDirector *, TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *, eCreatureType": FUNCDEF(
            restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
                ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
                ctypes.c_uint32,  # eCreatureType
            ]
        ),
        "cGcPlayerExperienceDirector *, TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *, const TkID<128> *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
                ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
                ctypes.c_char_p,  # const TkID<128> *
            ]
        ),
    },
    "cGcPlayerExperienceDirector::SpawnFiends": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cGcFiendManagerBase *
            ctypes.c_ulonglong,  # const std::vector<cGcFiendSpawnRequest,TkSTLAllocatorShim<cGcFiendSpawnRequest,16,-1> > *
        ]
    ),
    "cGcPlayerExperienceDirector::SpawnFishFiends": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cTkAttachmentPtr,TkSTLAllocatorShim<cTkAttachmentPtr,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # std::vector<cTkAttachmentPtr,TkSTLAllocatorShim<cTkAttachmentPtr,8,-1> > *
            ctypes.c_ulonglong,  # const std::vector<cGcFiendSpawnRequest,TkSTLAllocatorShim<cGcFiendSpawnRequest,16,-1> > *
        ]
    ),
    "cGcPlayerExperienceDirector::StartPatrolDrones": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerExperienceDirector::StartFrigateFlyby": {
        "cGcPlayerExperienceDirector *, const sFrigateFlybyRequest *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
                ctypes.c_ulonglong,  # const sFrigateFlybyRequest *
            ]
        ),
        "cGcPlayerExperienceDirector *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ]
        ),
    },
    "cGcPlayerExperienceDirector::StartFlyby": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAIShipSpawnData> *
            ctypes.c_uint32,  # eAIFaction
            ctypes.c_uint32,  # ePirateSpawn
        ]
    ),
    "cGcPlayerExperienceDirector::SelectPirateAttackDefinition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcPlayerExperienceDirector::StartTraderMissionSpaceFlyby": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerExperienceDirector::StartSpaceFlyby": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcSentinelInfo::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelInfo *
            ctypes.c_ulonglong,  # std::vector<cTkAttachmentPtr,TkSTLAllocatorShim<cTkAttachmentPtr,8,-1> > *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSentinelInfo::AddSpawnsForCurrentWave": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelInfo *
        ]
    ),
    "cGcSentinelInfo::Add": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelInfo *
            ctypes.c_ulonglong,  # std::vector<cTkAttachmentPtr,TkSTLAllocatorShim<cTkAttachmentPtr,8,-1> > *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelInfo::IsDefeated": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelInfo *
        ]
    ),
    "cGcSentinelInfo::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelInfo *
        ]
    ),
    "cGcSentinelInfo::CalculateNeeded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelInfo *
            ctypes.c_ulonglong,  # std::vector<cTkAttachmentPtr,TkSTLAllocatorShim<cTkAttachmentPtr,8,-1> > *
            ctypes.c_ulonglong,  # cGcSentinelInfo::SentinelCounts *
        ]
    ),
    "cGcFiendManagerBase::Add": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerBase *
            ctypes.c_ulonglong,  # std::vector<cTkAttachmentPtr,TkSTLAllocatorShim<cTkAttachmentPtr,8,-1> > *
        ]
    ),
    "cGcFiendManagerBase::GetDespawnDistance": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerBase *
        ]
    ),
    "cGcFiendManagerBase::GetNumLocalAndRemoteCreaturesOfType": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerBase *
            ctypes.c_uint32,  # eCreatureType
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "cGcFiendManagerBase::CanTakeControlOfFiend": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerBase *
            ctypes.c_ulonglong,  # const cGcCreatureComponent *
        ]
    ),
    "cGcFiendManagerBase::TakeControlOfFiend": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerBase *
            ctypes.c_ulonglong,  # cGcCreatureComponent *
        ]
    ),
    "cGcFiendManagerBase::ReportCrime": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerBase *
            ctypes.c_ulonglong,  # const cGcMessageFiendCrime *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # const cGcRewardActivateFiends *
        ]
    ),
    "cGcFiendManagerBase::GetSpawnRequests": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerBase *
            ctypes.c_ulonglong,  # std::vector<cGcFiendSpawnRequest,TkSTLAllocatorShim<cGcFiendSpawnRequest,16,-1> > *
        ]
    ),
    "cGcFiendManagerBase::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerBase *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFiendManagerBase::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerBase *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cGcExperienceSpawnTable *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcFiendManagerBase::DebugDraw": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerBase *
        ]
    ),
    "cGcFiendManagerBase::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerBase *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcVirtualisedCreature,16,-1> *
        ]
    ),
    "cGcFiendManagerStandard::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerStandard *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cGcExperienceSpawnTable *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_uint32,  # eExperienceSpawnType
        ]
    ),
    "cGcFiendManagerStandard::ActiveFiendCombat": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerStandard *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFiendManagerStandard::OnTakeControlOfFiend": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerStandard *
        ]
    ),
    "cGcFiendManagerStandard::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerStandard *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcVirtualisedCreature,16,-1> *
        ]
    ),
    "cGcFiendManagerStandard::HandleSpawnEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerStandard *
            ctypes.c_uint64,  # cTkAttachmentPtr
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cGcPlayerExperienceSpawnTable *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcFiendManagerStandard::ReportCrime": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerStandard *
            ctypes.c_ulonglong,  # const cGcMessageFiendCrime *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # const cGcRewardActivateFiends *
        ]
    ),
    "cGcFiendManagerStandard::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerStandard *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFiendManagerUnderwater::ReportCrime": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerUnderwater *
            ctypes.c_ulonglong,  # const cGcMessageFiendCrime *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # const cGcRewardActivateFiends *
        ]
    ),
    "cGcFiendManagerUnderwater::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerUnderwater *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFiendManagerUnderwater::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerUnderwater *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cGcExperienceSpawnTable *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcFiendManagerUnderwater::GetNumBigFishFiends": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendManagerUnderwater *
        ]
    ),
    "cGcPlayerExperienceDirector::GetFreighterSpawnsCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
        ]
    ),
    "cGcPlayerExperienceDirector::OnNetworkPlayerStateEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_uint32,  # eGcNetworkPlayerStateEvent
        ]
    ),
    "cGcPlayerExperienceDirector::OnWantedLevelNotify": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceDirector::RPCRemotePlayerJoinedBattle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceDirector::RPCRemoteGiveBountyReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_char * 0x20,  # TkID<256>
            ctypes.c_char * 0x10,  # TkID<128>
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cTkDynamicArray<cGcAIShipDebugSpawnData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAIShipDebugSpawnData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcChooser<ePirateSpawn>::GenChoice": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcChooser<enum ePirateSpawn> *
        ]
    ),
    "cGcChooser<ePirateSpawn>::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcChooser<enum ePirateSpawn> *
            common.GcSeed,  # cTkSeed
            ctypes.c_ulonglong,  # ePirateSpawn *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBackgroundSpaceEncounterInfo>::end": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkArraySTLIterator<cGcBackgroundSpaceEncounterInfo>::InternalConstIterator *
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBackgroundSpaceEncounterInfo> *
            ctypes.c_ulonglong,  # cTkArraySTLIterator<cGcBackgroundSpaceEncounterInfo>::InternalConstIterator *
        ]
    ),
    "cTkDynamicArray<cGcPulseEncounterInfo>::end": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkArraySTLIterator<cGcPulseEncounterInfo>::InternalConstIterator *
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPulseEncounterInfo> *
            ctypes.c_ulonglong,  # cTkArraySTLIterator<cGcPulseEncounterInfo>::InternalConstIterator *
        ]
    ),
    "cTkRosterPool<cGcAIShipSpawn,128>::Free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRosterPool<cGcAIShipSpawn,128> *
            ctypes.c_ulonglong,  # cGcAIShipSpawn *
        ]
    ),
    "cTkDynamicArray<cTkInteractiveControlData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkInteractiveControlData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcExperienceSpawnTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcExperienceSpawnTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAttachment::FindFirstAttachmentWithComponent<cTkPhysicsComponent>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachment *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkBehaviourTree::Broadcast<cGcDoShipClearCommunication>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_ulonglong,  # const cGcDoShipClearCommunication *
        ]
    ),
    "cTkMetaMessageMap<2>::Broadcast<cGcDoShipClearCommunication>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
            ctypes.c_ulonglong,  # const cGcDoShipClearCommunication *
        ]
    ),
    "cGcSentinelInfo::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelInfo *
        ]
    ),
    "cGcVirtualisedCreature::cGcVirtualisedCreature": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVirtualisedCreature *
            ctypes.c_ulonglong,  # const cGcPlayerExperienceSpawnData *
            ctypes.c_uint32,  # eCreatureType
            ctypes.c_uint32,  # eCreatureRole
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # eMoveArea
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelInfo::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelInfo *
        ]
    ),
    "cGcPlayerExperienceDirector::SetPoliceArriveState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_uint32,  # ePoliceEntrance
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditor::HasActivePopup": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
        ]
    ),
    "cTkPhysicsComponent::SetGraphicsOffsetMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcAISpaceshipComponent::GetResourceSeed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSeed *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_ulonglong,  # cTkSeed *
        ]
    ),
    "cGcPlayerExperienceDirector::SetPirateState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_uint32,  # ePirateSpawn
        ]
    ),
    "TkAddNodesHandle::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkAddNodesHandle *
        ]
    ),
    "sTerrainEditData::sTerrainEditData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sTerrainEditData *
            ctypes.c_uint32,  # eVoxelType
            ctypes.c_uint32,  # eTerrainEditShape
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerHazard::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazard *
        ]
    ),
    "cGcPlayerHazard::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazard *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerHazard::GetHazardLevel": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazard *
            ctypes.c_uint32,  # eHazard
        ]
    ),
    "cGcPlayerHazard::ModifyHazardValues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazard *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerHazard::GetProtectionTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazard *
            ctypes.c_uint32,  # eHazard
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerHazard::GetRechargeTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazard *
            ctypes.c_uint32,  # eHazard
        ]
    ),
    "cGcPlayerHazard::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazard *
        ]
    ),
    "cGcPlayerHazard::UpdateShielding": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazard *
            ctypes.c_uint32,  # eHazard
        ]
    ),
    "cGcInteractionComponent::SetOverridePage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
            ctypes.c_uint32,  # eFrontendPage
        ]
    ),
    "cGcInteractionComponent::DisplayDescriptionOnInteractPrompt": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcSimpleInteractionComponent::GetVRInteractSubtitle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
            ctypes.c_ulonglong,  # TkID<256> *
        ]
    ),
    "cGcSimpleInteractionComponent::IsFeedingInteraction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponent *
        ]
    ),
    "cGcGrabPlayerComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGrabPlayerComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWiringSocketComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "ActivationCost::Set": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ActivationCost *
            ctypes.c_ulonglong,  # const cGcInteractionActivationCost *
        ]
    ),
    "ActivationCost::CostRemaining": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # ActivationCost *
        ]
    ),
    "cGcPlayerInteract::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerInteract::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerInteract::DoCollectEffects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::HasCollectableHoldTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::HasWiringTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::HasCollectableOnlyHoldTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::PassInteract": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
            ctypes.c_ulonglong,  # cGcInteractionComponent *
        ]
    ),
    "cGcPlayerInteract::StealControls": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::GetInteractionComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInteractionComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::ReleaseInteractionNetworkLock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::FixOnScreen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cGcPlayerInteract::CanRender": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcPlayerInteract::ResetActivationCost": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::GetActivationCost": FUNCDEF(
        restype=ctypes.c_ulonglong,  # ActivationCost *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::HasActivationCost": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::NextActivationCost": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::CostRemaining": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::CanPayForElement": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerInteract::CanPayActivationCost": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::PayCostForElement": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcPlayerInteract::PayActivationCost": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::InteractError": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::CanTrigger": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::CanInteract": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::UpdateAllowedToInteractBasedOnVehicle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcTargetHelper::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetHelper *
            ctypes.c_uint32,  # ePlayerInteraction
        ]
    ),
    "cGcTargetHelper::Update": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetHelper *
            ctypes.c_float,  # float
        ]
    ),
    "cGcTargetHelper::UpdateSync": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetHelper *
        ]
    ),
    "cGcTargetHelper::GenerateTargetInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetHelper *
        ]
    ),
    "cGcTargetHelper::SetDestructableSubstanceSubtitles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetHelper *
            ctypes.c_ulonglong,  # cGcDestructableComponent *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # cTkColour *
        ]
    ),
    "cGcTargetHelper::GetInteractiveAttachment": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachment *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetHelper *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_uint32,  # ePlayerInteraction
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcPlayerInteract::SaveEmptyInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcTargetHelper::InteractLabelMarker::GetLabelPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetHelper::InteractLabelMarker *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcTargetHelper::InteractLabelMarker::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetHelper::InteractLabelMarker *
        ]
    ),
    "cGcTargetHelper::ClearHighlight": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetHelper *
        ]
    ),
    "cGcTargetHelper::IsOnScreen": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetHelper *
        ]
    ),
    "cGcPlayerInteract::UpdateViewTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerInteract::AcquireInteractionNetworkLock": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerLadderClimb::StartLadderClimb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerLadderClimb *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cGcPlayerLadderClimb::StopLadderClimb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerLadderClimb *
        ]
    ),
    "cGcPlayerLadderClimb::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerLadderClimb *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerLadderClimb::OnStateComplete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerLadderClimb *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcOutpostComponent::FindNearestEmptyDockPointLocation": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcOutpostComponent::GetExitPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerRespawn::DoPlayerRespawn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_uint32,  # cGcPlayerRespawn::RespawnReason
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::UpdateShipInventories": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_uint32,  # eLastKnownPlayerState
            ctypes.c_uint32,  # cGcPlayerRespawn::SpawnLocation
        ]
    ),
    "cGcPlayerRespawn::SpawnAndPositionFreighter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
        ]
    ),
    "cGcPlayerRespawn::GetMultiplayerOffsetRotation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerRespawn::GetShipPositionStartScene": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuilding *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerRespawn::GetSpawnPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_uint32,  # eLastKnownPlayerState
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerRespawn::AdjustPositionOnFreighter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_uint32,  # eLastKnownPlayerState
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::UpdateSolarSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::UpdateTeleportLocations": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerRespawn::DoPlayerPositioningRaycast": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerRespawn::DoShipPositioningRaycast": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::ChooseSpawnLocation": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_uint32,  # cGcPlayerRespawn::RespawnReason
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetGameStartSpawnLocation": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
        ]
    ),
    "cGcPlayerRespawn::GetPositionInSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionLookingAtSpaceBattle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionLookingAtPlanet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionLookingAtSpaceStation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionLookingAtAnomaly": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionLookingInGeneratedDirection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionInSpaceStation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionOnGameStartPlanet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionOnNearestPlanet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionNextToCrashedShip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionNextToPortal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerRespawn::GetWarpDestinationOnPlayerFreighter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetDestinationOnGameStartFreighter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetTeleportDestinationOnPlayerFreighter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionOnNexus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionOnNearestAbandonedFreighter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionNextToBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionAwayFromShip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # cGcPlayerRespawn::RespawnReason
        ]
    ),
    "cGcPlayerRespawn::GetSeasonalStartPositionNearBuilding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuilding *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionNextToBuilding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cGcBuilding *
        ]
    ),
    "cGcPlayerRespawn::GetPrecisePositionNextToBuilding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # cGcBuilding *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerRespawn::GetPositionForTeleportEndpointInStation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionForTeleportEndpointInBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionForTeleportEndpointInSettlement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerRespawn::GetPositionForTeleportEndpointOnFrigate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerRespawn::GetLastFreighterTransform": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionForTeleportEndpointInAtlas": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionFromSave": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerRespawn::GetLocationFromDebugInput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
        ]
    ),
    "cGcPlayerRespawn::GetPositionFromDebugInput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionFromCurrentUGCBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcPlayerRespawn::GetPositionOnPlanetSurface": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcPlayerHazardTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerHazardTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerRespawn::GetLastKnownPlayerState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerRespawn *
            ctypes.c_uint32,  # const cGcPlayerRespawn::SpawnLocation
            ctypes.c_uint32,  # const eLastKnownPlayerState
        ]
    ),
    "cGcSolarSystem::GetAnomalySphere": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSphere *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # cTkSphere *
        ]
    ),
    "RayCastInfo::AddLookRays": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RayCastInfo *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_double,  # double
        ]
    ),
    "RayCastInfo::AddRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RayCastInfo *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "RayCastInfo::AddShootRays": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RayCastInfo *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_double,  # double
        ]
    ),
    "cGcPlayerInteract::AbleToInteract": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerInteract *
        ]
    ),
    "cGcPlayerHazard::RenderImGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcPlayerThirdPerson::SpawnThirdPersonModel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerThirdPerson *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cGcPlayerThirdPerson::PostPhysicsUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerThirdPerson *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerThirdPerson::DetermineCurrentVisibility": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerThirdPerson *
        ]
    ),
    "cGcPlayerThirdPerson::RefreshThirdPersonMeshVisibility": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerThirdPerson *
        ]
    ),
    "cGcPlayerThirdPerson::BlockPlayerMovement": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerThirdPerson *
        ]
    ),
    "cGcPlayerThirdPerson::BlockPlayerWeapon": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerThirdPerson *
        ]
    ),
    "cGcPlayerThirdPerson::UpdateMeshRefresh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerThirdPerson *
        ]
    ),
    "cGcPlayerThirdPerson::GenerateResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerThirdPerson *
        ]
    ),
    "cGcPlayerThirdPerson::UpdateLookAtPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerThirdPerson *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcPlayerThirdPerson::UpdateHandNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerThirdPerson *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AnimateItems::GetPos": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # AnimateItems *
        ]
    ),
    "cGcQuickMenuState::SetErrorMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenuState *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcQuickActionMenu::ActionTable::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu::ActionTable *
        ]
    ),
    "cGcQuickActionMenu::ActionTable::FixUpIndicies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu::ActionTable *
        ]
    ),
    "cGcQuickActionMenu::ActionTable::AddAction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu::ActionTable *
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # const MenuAction *
        ]
    ),
    "cGcQuickActionMenu::ActionTable::GetActionAtDepth": FUNCDEF(
        restype=ctypes.c_ulonglong,  # MenuAction *
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu::ActionTable *
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcQuickActionMenu::ActionTable::SetActionIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu::ActionTable *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
        ]
    ),
    "cGcQuickActionMenu::HotAction::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu::HotAction *
        ]
    ),
    "cGcQuickActionMenu::HotAction::GetMenuDepth": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu::HotAction *
        ]
    ),
    "cGcPlayerWanted::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWanted *
            ctypes.c_ulonglong,  # cGcPlayerCommon *
        ]
    ),
    "cGcPlayerWanted::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWanted *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerWanted::SetWantedLevel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWanted *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWanted::GetLastKnownPlayerPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWanted *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcPlayerWanted::GetTimeoutTimer": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWanted *
        ]
    ),
    "cGcPlayerWanted::IsCombatStealthing": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWanted *
        ]
    ),
    "cGcPlayerWanted::GetSentinelCrimeResponse": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWanted *
        ]
    ),
    "cGcWitness::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWitness *
        ]
    ),
    "cGcPlayerByteBeatLibrary::Play": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerByteBeatLibrary *
        ]
    ),
    "cGcPlayerByteBeatLibrary::Stop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerByteBeatLibrary *
        ]
    ),
    "cGcQuickActionMenu::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
        ]
    ),
    "cGcQuickActionMenu::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # QuickMenuRenderState *
        ]
    ),
    "cGcQuickActionMenu::LoadHotActions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
            ctypes.c_ulonglong,  # const cTkFixedArray<cGcHotActionsSaveData,3> *
        ]
    ),
    "cGcQuickActionMenu::IsSlotSelected": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
            ctypes.c_uint32,  # eQuickMenuSlot
        ]
    ),
    "cGcQuickActionMenu::IsInWorldAction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
        ]
    ),
    "cGcQuickActionMenu::CanEmote": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
        ]
    ),
    "cGcQuickActionMenu::UpdateActions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
            ctypes.c_ulonglong,  # QuickMenuRenderState *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_char,  # char
        ]
    ),
    "cGcQuickActionMenu::UpdateActionString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cGcQuickActionMenu::UpdateInWorldUISelection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
            ctypes.c_ulonglong,  # QuickMenuRenderState *
        ]
    ),
    "cGcQuickActionMenu::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
        ]
    ),
    "cGcQuickActionMenu::Close": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcQuickActionMenu::AddActionsForInventories": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # std::initializer_list<enum InventoryChoice const > *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcQuickActionMenu::IsUpgradeAppropriateForEnvironment": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
            ctypes.c_uint32,  # const eHazard
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcStatsBonus> *
        ]
    ),
    "cGcQuickActionMenu::IsPlayerEmoteAllowedAtLocation": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
            ctypes.c_ulonglong,  # const cGcPlayerEmote *
        ]
    ),
    "cGcQuickActionMenu::ShouldShowSwitchShipOption": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
        ]
    ),
    "cGcQuickActionMenu::TriggerCurrentAction": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
        ]
    ),
    "cGcQuickActionMenu::TriggerAction": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
            ctypes.c_ulonglong,  # const MenuAction *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcQuickActionMenu::TryOpenToHotAction": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
            ctypes.c_ulonglong,  # QuickMenuRenderState *
        ]
    ),
    "cGcQuickActionMenu::TryOpenToOverrideAction": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
            ctypes.c_ulonglong,  # QuickMenuRenderState *
        ]
    ),
    "cGcQuickActionMenu::StepIntoSubMenu": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
        ]
    ),
    "cGcQuickActionMenu::TriggerClose": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
        ]
    ),
    "cGcQuickActionMenu::UpdateControlsString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcQuickActionMenu::ShouldClose": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
        ]
    ),
    "cGcQuickActionMenu::ShouldTriggerAction": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
        ]
    ),
    "cGcQuickActionMenu::HasParentMenu": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickActionMenu *
        ]
    ),
    "cGcAnalogueMenuNavigator::Update": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAnalogueMenuNavigator *
        ]
    ),
    "cGcQuickMenuState::IsSlotHighlighted": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenuState *
            ctypes.c_uint32,  # eQuickMenuSlot
        ]
    ),
    "cGcQuickMenuState::GetIconTextForActionPair": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcQuickMenuState::ConstructTipString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcQuickMenu::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenu *
        ]
    ),
    "cGcQuickMenu::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenu *
        ]
    ),
    "cGcQuickMenu::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenu *
        ]
    ),
    "cGcQuickMenu::CloseMenu": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenu *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcQuickMenu::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenu *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcQuickMenu::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenu *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
        ]
    ),
    "cGcQuickMenu::CanOpenBuildMenu": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenu *
        ]
    ),
    "cGcQuickMenu::OpenBuildMenu": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenu *
        ]
    ),
    "cGcQuickMenu::Close": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenu *
        ]
    ),
    "cGcQuickMenu::DoNextActionItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenu *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const MenuAction *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcQuickMenu::DoNextActions": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenu *
            ctypes.c_ulonglong,  # cGcNGuiLayer *
            ctypes.c_ulonglong,  # const std::vector<MenuAction,TkSTLAllocatorShim<MenuAction,16,-1> > *
            ctypes.c_ulonglong,  # const AnimateItems *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcQuickMenu::GetActionItemName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # const MenuAction *
        ]
    ),
    "cGcQuickMenu::CanOpen": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenu *
        ]
    ),
    "cTkBehaviourTree::Broadcast<cGcDoShipReceiveHail>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_ulonglong,  # const cGcDoShipReceiveHail *
        ]
    ),
    "cGcPlayerCreatureOwnership::ShowEmoteArc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCreatureOwnership *
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlayerVehicleOwnership::RequestVehicleSpawn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerVehicleOwnership *
            ctypes.c_uint32,  # eVehicleType
        ]
    ),
    "cGcPlayerState::ResetShipTerrainPositioning": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
        ]
    ),
    "cGcSpaceshipWarp::RequestExitPulseDrive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
        ]
    ),
    "cGcTerrainEditMenu::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditMenu *
        ]
    ),
    "cGcTerrainEditMenu::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditMenu *
        ]
    ),
    "cGcTerrainEditMenu::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditMenu *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # QuickMenuRenderState *
        ]
    ),
    "cGcTerrainEditMenu::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditMenu *
        ]
    ),
    "cGcTerrainEditMenu::Close": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditMenu *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcTerrainEditMenu::TriggerClose": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditMenu *
        ]
    ),
    "cGcTerrainEditMenu::UpdateControlsString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditMenu *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcMuzzleFlash::Construct": {
        "cGcMuzzleFlash *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcMuzzleFlash *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cGcMuzzleFlash *, const char *, const eRemoteWeaponType": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcMuzzleFlash *
                ctypes.c_char_p,  # const char *
                ctypes.c_uint32,  # const eRemoteWeaponType
            ]
        ),
        "cGcMuzzleFlash *, cTkSmartResHandle *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcMuzzleFlash *
                ctypes.c_ulonglong,  # cTkSmartResHandle *
            ]
        ),
    },
    "cGcMuzzleFlash::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMuzzleFlash *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcMuzzleFlash::SetColour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMuzzleFlash *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcMuzzleFlash::SetScale": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMuzzleFlash *
            ctypes.c_double,  # double
        ]
    ),
    "cGcMuzzleFlash::SetParticlesEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMuzzleFlash *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
        ]
    ),
    "cGcLootComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcLootComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDestructableComponent::CleanUpDestructionNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcDestructableComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponent *
        ]
    ),
    "cGcDestructableComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponent *
        ]
    ),
    "cGcDestructableComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDestructableComponent::DoGiveReward": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponent *
        ]
    ),
    "cGcDestructableComponent::DoLootReward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponent *
        ]
    ),
    "cGcDestructableComponent::DoGiveSubstances": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponent *
        ]
    ),
    "cGcDestructableComponent::GetDiscoveryReward": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponent *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcDestructableComponent::PlayDestroyEffects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponent *
        ]
    ),
    "cGcDestructableComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponent *
        ]
    ),
    "cGcDestructableComponent::CalculateDestroyEffectMatrices": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cTkMatrix34,TkSTLAllocatorShim<cTkMatrix34,16,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<cTkMatrix34,TkSTLAllocatorShim<cTkMatrix34,16,-1> > *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcDestructableComponent::RefreshPersistency": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponent *
        ]
    ),
    "cGcDestructableComponent::GiveResourceFromDamage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponent *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # eResourceBlobSpawnStyle
        ]
    ),
    "cGcDestructableComponent::ResetHidden": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponent *
        ]
    ),
    "cGcDestructableComponent::GetPersistentInteractionBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDestructableComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponent *
        ]
    ),
    "cGcBeamEffect::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBeamEffect *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcBeamEffect::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBeamEffect *
        ]
    ),
    "cGcBeamEffect::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBeamEffect *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLaserBeam::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeam *
            ctypes.c_ulonglong,  # const cGcLaserBeamData *
            ctypes.c_ulonglong,  # cGcOwnerConcept *
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcLaserBeam::Fire": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeam *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLaserBeam::Overheat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeam *
        ]
    ),
    "cGcLaserBeam::Deactivate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeam *
        ]
    ),
    "cGcLaserBeam::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeam *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLaserBeam::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeam *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcLaserBeam::HitTerrain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeam *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcLaserBeam::UpdateBeam": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeam *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cGcBeamEffect *
            ctypes.c_ulonglong,  # cGcBeamEffect *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcLaserBeam::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeam *
        ]
    ),
    "cGcLaserBeam::IsFromPlayer": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeam *
        ]
    ),
    "cGcProjectileImpact::IsFromLocalPlayer": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileImpact *
        ]
    ),
    "cGcProjectileImpact::IsFromRemotePlayer": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileImpact *
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcProjectileImpact::IsFromVehicle": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileImpact *
        ]
    ),
    "cGcProjectileImpact::IsFromSpaceship": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileImpact *
        ]
    ),
    "cGcProjectileImpact::FromFaction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileImpact *
        ]
    ),
    "cGcProjectile::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectile *
            ctypes.c_double,  # double
        ]
    ),
    "cGcProjectile::GetDamage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectile *
        ]
    ),
    "cGcSentinelCoverComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcSpaceshipShieldComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcProjectileManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileManager *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProjectileManager::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileManager *
        ]
    ),
    "cGcProjectileManager::Launch": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPoolHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileManager *
            ctypes.c_ulonglong,  # cTkClassPoolHandle *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cGcOwnerConcept *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcProjectileManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileManager *
            ctypes.c_double,  # double
        ]
    ),
    "cGcProjectileManager::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileManager *
        ]
    ),
    "cGcProjectileManager::Impact": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileManager *
            ctypes.c_ulonglong,  # cGcProjectileImpact *
        ]
    ),
    "sNetProjectileImpact::sNetProjectileImpact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sNetProjectileImpact *
            ctypes.c_ulonglong,  # const sNetProjectileImpact *
        ]
    ),
    "cGcProjectileManager::OnResourceBlobSpawn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_int32,  # int
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cGcProjectileManager::HitEditTerrain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileManager *
            ctypes.c_ulonglong,  # const cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
            ctypes.c_int8,  # const sTerrainEditData
        ]
    ),
    "cGcProjectileManager::SpawnResources": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileManager *
            ctypes.c_ulonglong,  # const std::array<unsigned int,11> *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcProjectileManager::SpawnSpecialQuestResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const std::array<unsigned int,11> *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcProjectileManager::GetImpactType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileManager *
            ctypes.c_ulonglong,  # const cTkRigidBody *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcProjectileManager::HandleImpactExplosion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileManager *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
            ctypes.c_ulonglong,  # const cGcProjectileData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcShootableComponent *
            ctypes.c_ulonglong,  # const cGcCombatEffectsComponent *
        ]
    ),
    "cGcProjectileManager::AddImpactParticleEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileManager *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
            ctypes.c_ulonglong,  # const cGcProjectileData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShootableComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
        ]
    ),
    "cGcShootableComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
        ]
    ),
    "cGcShootableComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShootableComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
        ]
    ),
    "cGcShootableComponent::OnHit": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcShootableComponent::AddHealth": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcShootableComponent::RegisterCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['void *', 'const cGcProjectileImpact *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcShootableComponent::IsAutoAimTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
        ]
    ),
    "cGcShootableComponent::CalculateInterceptFraction": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShootableComponent::GetCriticalHitInfo": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcShootableComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
        ]
    ),
    "cGcShootableComponent::CanBeDamaged": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
        ]
    ),
    "cGcShootableComponent::IsCriticalHit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcShootableComponent::ApplyDamageMultiplier": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # eDamageType
            ctypes.c_float,  # float
        ]
    ),
    "cGcShootableComponent::UpdateAttackerInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcShootableComponent::HighlightArmour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
        ]
    ),
    "cGcShootableComponent::Receive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponent *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcScanEffectComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcTerrainEditorBeam::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
            ctypes.c_ulonglong,  # cGcWeapon *
            ctypes.c_uint64,  # TkAudioObject
        ]
    ),
    "cGcTerrainEditorBeam::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainEditorBeam::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcTerrainEditorBeam::SetMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
            ctypes.c_uint32,  # cGcTerrainEditorBeam::eEditMode
        ]
    ),
    "cGcTerrainEditorBeam::Fire": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cGcOwnerConcept *
            ctypes.c_uint32,  # eStatsType
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainEditorBeam::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcTerrainEditorBeam::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
        ]
    ),
    "cGcTerrainEditorBeam::Deactivate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
        ]
    ),
    "cGcTerrainEditorBeam::IsResourcesLoaded": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
        ]
    ),
    "cGcTerrainEditorBeam::Deselected": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
        ]
    ),
    "cGcTerrainEditorBeam::GetEditRadius": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
        ]
    ),
    "cGcTerrainEditorBeam::SetCurrentEditPlaneState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
            ctypes.c_uint32,  # eEditPlaneState
        ]
    ),
    "cGcTerrainEditorBeam::StartEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
        ]
    ),
    "cGcTerrainEditorBeam::ApplyTerrainEditStroke": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
            ctypes.c_int8,  # sTerrainEditData
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcTerrainEditorBeam::ApplyTerrainEditFlatten": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
            ctypes.c_int8,  # sTerrainEditData
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcTerrainEditorBeam::ApplyTerrainEditUndo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcTerrainEditorBeam::GetEditData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sTerrainEditData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
            ctypes.c_ulonglong,  # sTerrainEditData *
        ]
    ),
    "cGcTerrainEditorBeam::RotateEditPlane": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditorBeam *
            ctypes.c_float,  # float
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcProjectileDataTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcProjectileDataTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaMessageMap<2>::Broadcast<cGcMessageSubstanceMined>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
            ctypes.c_ulonglong,  # const cGcMessageSubstanceMined *
        ]
    ),
    "cGcBeamEffect::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBeamEffect *
        ]
    ),
    "cGcBeamEffect::Deactivate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBeamEffect *
        ]
    ),
    "cGcShootableComponent::CalculateInterceptPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShootableComponent::AdjustforGravity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcMissileComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissileComponent *
        ]
    ),
    "cGcMissileComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissileComponent *
        ]
    ),
    "cGcMissileComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissileComponent *
            ctypes.c_double,  # double
        ]
    ),
    "cGcMissileComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissileComponent *
        ]
    ),
    "cGcMissileComponent::Launch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissileComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cGcOwnerConcept *
        ]
    ),
    "cGcDroneAttackMovement::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneAttackMovement *
        ]
    ),
    "cGcDroneAttackMovement::cGcDroneAttackMovement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneAttackMovement *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcDroneBlackboard *
        ]
    ),
    "cGcDroneComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponent *
        ]
    ),
    "cGcDroneComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDroneAttackMovement::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneAttackMovement *
        ]
    ),
    "cGcDroneAttackMovement::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneAttackMovement *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneAttackMovement::UpdatePositioning": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneAttackMovement *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneAttackMovement::ChooseAttackPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneAttackMovement *
        ]
    ),
    "cGcDroneAttackShooting::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneAttackShooting *
        ]
    ),
    "cGcDroneAttackShooting::cGcDroneAttackShooting": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneAttackShooting *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcDroneBlackboard *
        ]
    ),
    "cGcDroneAttackShooting::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneAttackShooting *
        ]
    ),
    "cGcDroneAttackShooting::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneAttackShooting *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneCheckAlert::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneCheckAlert *
        ]
    ),
    "cGcDroneCheckAlert::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneCheckAlert *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcDroneGetAntagonistTarget::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneGetAntagonistTarget *
        ]
    ),
    "cGcDroneGetAntagonistTarget::cGcDroneGetAntagonistTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneGetAntagonistTarget *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcDroneBlackboard *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneCheckAlert::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneCheckAlert *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneGetAntagonistTarget::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneGetAntagonistTarget *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneGetAntagonistTarget::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneGetAntagonistTarget *
        ]
    ),
    "cGcDroneCoverMovement::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneCoverMovement *
        ]
    ),
    "cGcSentinelCoverComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcDroneCoverMovement::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneCoverMovement *
        ]
    ),
    "cGcDroneCoverMovement::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneCoverMovement *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneDoRepairs::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneDoRepairs *
        ]
    ),
    "cGcDroneDoRepairs::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneDoRepairs *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneFindPlayer::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneFindPlayer *
        ]
    ),
    "cGcDroneFindPlayer::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneFindPlayer *
        ]
    ),
    "cGcDroneFindPlayer::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneFindPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDronePatrol::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePatrol *
        ]
    ),
    "cGcDronePatrol::cGcDronePatrol": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePatrol *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcDroneBlackboard *
        ]
    ),
    "cGcDronePatrol::AlreadyScanned": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePatrol *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcDronePatrol::~cGcDronePatrol": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePatrol *
        ]
    ),
    "cGcDroneIsOwnerWanted::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneIsOwnerWanted *
        ]
    ),
    "cGcDroneFollowPlayer::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneFollowPlayer *
        ]
    ),
    "cGcDronePlayerFocusBehaviour::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePlayerFocusBehaviour *
        ]
    ),
    "cGcDronePlayerFocusBehaviour::cGcDronePlayerFocusBehaviour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePlayerFocusBehaviour *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcDroneBlackboard *
        ]
    ),
    "cGcDroneInteract::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneInteract *
        ]
    ),
    "cGcDroneMoodResponse::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneMoodResponse *
        ]
    ),
    "cGcDroneIsOwnerWanted::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneIsOwnerWanted *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneFollowPlayer::cGcDroneFollowPlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneFollowPlayer *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcDroneBlackboard *
        ]
    ),
    "cGcDroneFollowPlayer::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneFollowPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneFollowPlayer::GetSearchCentrePos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneFollowPlayer *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcDroneFollowPlayer::GetSearchRadius": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneFollowPlayer *
        ]
    ),
    "cGcDroneFollowPlayer::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneFollowPlayer *
        ]
    ),
    "cGcDronePlayerFocusBehaviour::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePlayerFocusBehaviour *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcDronePlayerFocusBehaviour::ShowMood": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePlayerFocusBehaviour *
        ]
    ),
    "cGcDronePlayerFocusBehaviour::FocusOnPlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePlayerFocusBehaviour *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneInteract::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneInteract *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneMoodResponse::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneMoodResponse *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneInvestigate::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneInvestigate *
        ]
    ),
    "cGcDroneInvestigate::AngleFromPlayer": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneInvestigate *
        ]
    ),
    "cGcDroneInvestigate::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneInvestigate *
        ]
    ),
    "cGcDroneInvestigate::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneInvestigate *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneInvestigate::NGuiRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneInvestigate *
        ]
    ),
    "cGcDroneInvestigate::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneInvestigate *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcDroneInvestigate::AttackCriminal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneInvestigate *
        ]
    ),
    "cGcDroneInvestigate::ScanCriminal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneInvestigate *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcDronePatrol::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePatrol *
        ]
    ),
    "cGcDronePatrol::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePatrol *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcDronePatrol::GetSearchCentrePos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePatrol *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcDronePatrol::GetSearchRadius": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePatrol *
        ]
    ),
    "cGcDronePatrol::ChooseTargetArea": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePatrol *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDronePatrol::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePatrol *
            ctypes.c_double,  # double
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcDronePatrol::GetToTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePatrol *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDronePatrol::ChooseScanTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePatrol *
        ]
    ),
    "cGcDronePatrol::InspectTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePatrol *
        ]
    ),
    "cGcDronePatrol::GetScanRadius": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePatrol *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcDronePatrol::InRange": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePatrol *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcDronePlaceCover::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePlaceCover *
        ]
    ),
    "cGcDronePlaceCover::cGcDronePlaceCover": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePlaceCover *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcDroneBlackboard *
        ]
    ),
    "cGcDronePlaceCover::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDronePlaceCover *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneStunned::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneStunned *
        ]
    ),
    "cGcDroneStunned::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneStunned *
        ]
    ),
    "cGcDroneStunned::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneStunned *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneStunned::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneStunned *
        ]
    ),
    "cGcDroneStunned::EndStun": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneStunned *
        ]
    ),
    "cGcDroneSummon::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneSummon *
        ]
    ),
    "cGcDroneSummonMovement::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneSummonMovement *
        ]
    ),
    "cGcDissolveEffectComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcDissolveEffectComponent::FindFirstTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDissolveEffectComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneSummonMovement::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneSummonMovement *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneSummon::cGcDroneSummon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneSummon *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcDroneBlackboard *
        ]
    ),
    "cGcDroneSummon::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneSummon *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneSummon::ClearPreviews": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneSummon *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcDroneSummon::cGcSummonPreview,16,-1> *
        ]
    ),
    "cGcDroneSummon::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneSummon *
        ]
    ),
    "cGcDroneSummonAndDismiss::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneSummonAndDismiss *
        ]
    ),
    "cGcDroneSummonAndDismiss::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneSummonAndDismiss *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneSummonAndDismiss::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneSummonAndDismiss *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcDroneBehaviour::cGcDroneBehaviour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviour *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcDroneBlackboard *
        ]
    ),
    "cGcDroneBehaviour::GetPerception": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviour *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcDroneBehaviour::HasLineOfSight": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviour *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneBehaviour::NotifyWanted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviour *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcDroneBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcDroneBehaviourTree::CreateInvestigateBehaviours": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        ]
    ),
    "cGcDroneBehaviourTree::CreateStunnedBehaviours": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCombatEffectsComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        ]
    ),
    "cGcDroneBehaviourTree::CreateAttackBehaviours": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        ]
    ),
    "cGcDroneBehaviourTree::CreatePatrolBehaviours": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        ]
    ),
    "cGcDroneBehaviourTree::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
        ]
    ),
    "cGcDroneBehaviourTree::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
        ]
    ),
    "cGcDroneBehaviourTree::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneBehaviourTree::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
        ]
    ),
    "cGcDroneBehaviourTree::SetScanActive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcDroneBehaviourTree::Fire": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcDroneBehaviourTree::GetWeaponData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcDroneWeaponData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
        ]
    ),
    "cGcDroneBehaviourTree::IsTargetPlayerMech": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
        ]
    ),
    "cGcDroneBehaviourTree::GetWeaponMoveDistanceMax": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
        ]
    ),
    "cGcDroneBehaviourTree::GetWeaponMoveDistanceMin": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
        ]
    ),
    "cGcDroneBehaviourTree::WitnessingCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcDroneBehaviourTree::SetVisualAlertness": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
            ctypes.c_uint32,  # cGcDroneBlackboard::eAlertness
        ]
    ),
    "cGcDroneBlackboard::SetAlertState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBlackboard *
            ctypes.c_uint32,  # cGcDroneBlackboard::eAlertness
        ]
    ),
    "cGcDroneBlackboard::GetTargetPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBlackboard *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcDroneBlackboard::GetTargetRigidBody": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRigidBody *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBlackboard *
        ]
    ),
    "cGcDroneBlackboard::GetTargetRadius": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBlackboard *
        ]
    ),
    "cGcDroneBlackboard::UseHostilePerceptionRanges": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBlackboard *
        ]
    ),
    "cGcDroneBlackboard::CalculateTargetMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "ComponentInstancercGcDroneComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcDroneComponent *
        ]
    ),
    "ComponentInstancercGcDroneComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcDroneComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcDroneBehaviourTree::cGcDroneBehaviourTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
        ]
    ),
    "cGcNavigation::cGcNavigation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavigation *
        ]
    ),
    "ComponentInstancercGcDroneComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcDroneBehaviourTree::~cGcDroneBehaviourTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
        ]
    ),
    "cGcNavigation::~cGcNavigation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavigation *
        ]
    ),
    "cGcDroneComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponent *
        ]
    ),
    "cGcDroneComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponent *
        ]
    ),
    "cGcDroneComponent::GetUpdatePeriod": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponent *
        ]
    ),
    "cGcDroneComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneComponent::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneComponent::NetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDroneComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponent *
        ]
    ),
    "cGcDroneComponent::AmInterestedInOwnership": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponent *
        ]
    ),
    "cGcDroneComponent::OnOwnershipTransfer": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponent *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcDroneComponent::PrepareNavigation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponent *
        ]
    ),
    "cGcWeightedSpringSteering::cGcWeightedSpringSteering": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedSpringSteering *
        ]
    ),
    "cGcDroneComponent::UpdateNavigation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDroneComponent::GetSentinelType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponent *
        ]
    ),
    "cGcDroneComponent::PostChatMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponent *
            ctypes.c_uint32,  # eFriendlyDroneChatType
            ctypes.c_uint32,  # eAkEvent
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneControl::RepelFromNeighbours": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cGcDroneControlData *
        ]
    ),
    "cGcDroneControl::RepelFromGround": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cGcDroneControlData *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcDroneControl::MoveWithVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cGcDroneControlData *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcDroneControl::LookAt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cGcDroneControlData *
        ]
    ),
    "cGcDroneControl::GetPerception": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneControl::HasLineOfSight": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcDroneControl::NearestGroundPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_double,  # double
            ctypes.c_float,  # float
        ]
    ),
    "cGcSentinelDrone::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
            ctypes.c_uint32,  # eSentinelType
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcSentinelDrone::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSentinelDrone::FireWeapon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
            ctypes.c_uint32,  # SentinelAttackType
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcSentinelDrone::GetTargetPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcSentinelDrone::GetForward": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcSentinelDrone::GetLabelPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcSentinelDrone::GetTargetPointSize": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
        ]
    ),
    "cGcSentinelDrone::GetSightPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcSentinelDrone::GetObb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
            ctypes.c_ulonglong,  # cTkAABB *
        ]
    ),
    "cGcSentinelDrone::GetEffectAttachLocator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcSentinelDrone::IsAttacking": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
        ]
    ),
    "cGcSentinelDrone::IsAlert": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
        ]
    ),
    "cGcSentinelDrone::GetAttackTarget": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachment *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
        ]
    ),
    "cGcSentinelDrone::GetAlertRange": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
        ]
    ),
    "cGcSentinelDrone::GetSightRange": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
        ]
    ),
    "cGcSentinelDrone::GetSightAngle": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
        ]
    ),
    "cGcSentinelDrone::GetOwnerConcept": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcOwnerConcept *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
        ]
    ),
    "cGcSentinelDrone::GetTree": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDroneBehaviourTree *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
        ]
    ),
    "cGcSentinelDrone::OnHit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcSentinelDrone::OnDeath": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelDrone *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcSentinelDrone::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelRobot *
        ]
    ),
    "cTkTemplatedComponentUpdater<cGcDroneComponent>::UpdateComponentsPostPhysics": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cGcDroneComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerExperienceDirector::ClearRepairer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcPlayerExperienceDirector::CheckAllowedToRepair": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceDirector *
            ctypes.c_uint64,  # cTkAttachmentPtr
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcSentinelWalker::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
            ctypes.c_uint32,  # eSentinelType
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcSentinelQuad::FireWeapon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
            ctypes.c_uint32,  # SentinelAttackType
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcSentinelWalker::GetTargetPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcSentinelWalker::GetForward": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcSentinelWalker::GetLabelPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcSentinelWalker::GetTargetPointSize": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
        ]
    ),
    "cGcSentinelWalker::GetSightPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcSentinelWalker::GetObb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
            ctypes.c_ulonglong,  # cTkAABB *
        ]
    ),
    "cGcSentinelQuad::GetEffectAttachLocator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcSentinelQuad::IsAttacking": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelQuad *
        ]
    ),
    "cGcSentinelQuad::IsAlert": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelQuad *
        ]
    ),
    "cGcSentinelQuad::IsDead": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
        ]
    ),
    "cGcSentinelWalker::GetAttackTarget": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachment *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
        ]
    ),
    "cGcSentinelQuad::GetAlertRange": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelQuad *
        ]
    ),
    "cGcSentinelQuad::GetSightRange": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelQuad *
        ]
    ),
    "cGcSentinelQuad::GetSightAngle": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelQuad *
        ]
    ),
    "cGcSentinelWalker::GetOwnerConcept": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcOwnerConcept *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
        ]
    ),
    "cGcSentinelQuad::GetTree": FUNCDEF(
        restype=ctypes.c_ulonglong,  # robin_hood::pair<unsigned __int64 const ,cTkStackVector<std::function<void __cdecl(cTkMetaMessageWrapperTemplated<void> const &)>,2,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
        ]
    ),
    "cGcSentinelQuad::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
        ]
    ),
    "cGcSentinelRobot::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelRobot *
            ctypes.c_uint32,  # eSentinelType
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcSentinelRobot::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelRobot *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSentinelRobot::OnHitCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcSentinelRobot::OnHit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelRobot *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcSentinelRobot::OnArmourHitCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcSentinelRobot::OnArmourHit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelRobot *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcSentinelRobot::OnDeathCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcSentinelMech::OnDeath": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelRobot *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcSentinelRobot::IsLocalPlayerEntityOwner": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelRobot *
        ]
    ),
    "cGcSentinelRobot::IsFromScriptedEncounter": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelRobot *
        ]
    ),
    "hknpBreakableConstraintData::getBreakingThreshold": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeapon *
        ]
    ),
    "hkMemoryMeshMaterial::getNumTextures": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeapon *
        ]
    ),
    "cGcSentinelMechWeapon::GetLastModeChangeTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeapon *
        ]
    ),
    "cGcSentinelMechWeapon::GetPrimaryStat": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeapon *
        ]
    ),
    "cGcSentinelMech::GetWeapon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSentinelMechWeapon *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
        ]
    ),
    "cGcSentinelWalker::GetCreatureType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
        ]
    ),
    "cGcSentinelRobotComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelRobotComponent *
        ]
    ),
    "cGcSentinelRobotComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelRobotComponent *
        ]
    ),
    "cGcSentinelRobotComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelRobotComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSentinelRobotComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelRobotComponent *
        ]
    ),
    "cGcSentinelRobotComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelRobotComponent *
        ]
    ),
    "cGcSentinelRobotComponent::NetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelRobotComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSentinelWalker::IsAttacking": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
        ]
    ),
    "cGcSentinelWalker::IsAlert": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
        ]
    ),
    "cGcSentinelWalker::GetAlertRange": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
        ]
    ),
    "cGcSentinelWalker::GetSightRange": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
        ]
    ),
    "cGcSentinelWalker::GetSightAngle": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWalker *
        ]
    ),
    "cGcExoMechAttachShooting::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcExoMechAttachShooting *
        ]
    ),
    "cGcExoMechAttachShooting::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExoMechAttachShooting *
            ctypes.c_float,  # float
        ]
    ),
    "cGcExoMechAttachShooting::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExoMechAttachShooting *
        ]
    ),
    "cGcExoMechAttachShooting::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExoMechAttachShooting *
        ]
    ),
    "cGcExoMechAttachShooting::BeginFiring": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExoMechAttachShooting *
            ctypes.c_uint32,  # eVehicleWeaponMode
        ]
    ),
    "cGcExoMechAttachShooting::EndFiring": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExoMechAttachShooting *
        ]
    ),
    "cGcExoMechAttachShooting::HaveValidShot": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcExoMechAttachShooting *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_uint32,  # eVehicleWeaponMode
        ]
    ),
    "cGcMechAttackMove::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAttackMove *
        ]
    ),
    "cGcMechAttackMove::cGcMechAttackMove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAttackMove *
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_ulonglong,  # cGcNavigation *
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ulonglong,  # cGcMechBlackboard *
        ]
    ),
    "cGcMechAttackMove::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAttackMove *
        ]
    ),
    "cGcMechAttackMove::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAttackMove *
        ]
    ),
    "cGcMechAttackMove::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAttackMove *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMechCheckAlert::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechCheckAlert *
        ]
    ),
    "cGcMechCheckAlert::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechCheckAlert *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMechCheckCanAttack::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechCheckCanAttack *
        ]
    ),
    "cGcMechCheckCanAttack::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechCheckCanAttack *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMechFollowPlayer::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechFollowPlayer *
        ]
    ),
    "CAkActionExcept::ClearExceptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechFollowPlayer *
        ]
    ),
    "cGcMechFollowPlayer::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechFollowPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMechGuard::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechGuard *
        ]
    ),
    "cGcMechGuard::cGcMechGuard": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechGuard *
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_ulonglong,  # cGcNavigation *
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ulonglong,  # cGcMechBlackboard *
        ]
    ),
    "cGcMechGuard::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechGuard *
        ]
    ),
    "cGcMechGuard::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechGuard *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMechGuard::FindBuildingToGuard": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechGuard *
        ]
    ),
    "cGcMechSearchForTarget::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechSearchForTarget *
        ]
    ),
    "cGcMechSearchForTarget::cGcMechSearchForTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechSearchForTarget *
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_ulonglong,  # cGcNavigation *
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ulonglong,  # cGcMechBlackboard *
        ]
    ),
    "cGcMechSearchForTarget::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechSearchForTarget *
        ]
    ),
    "cGcMechSearchForTarget::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechSearchForTarget *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMechSearchForTarget::BeginMoveToLocation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechSearchForTarget *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcMechSelectAttackTarget::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechSelectAttackTarget *
        ]
    ),
    "cGcMechSelectAttackTarget::cGcMechSelectAttackTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechSelectAttackTarget *
            ctypes.c_uint32,  # eAIMechFaction
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_ulonglong,  # cGcNavigation *
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ulonglong,  # cGcMechBlackboard *
        ]
    ),
    "cGcMechSelectAttackTarget::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechSelectAttackTarget *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMechSelectAttackTarget::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechSelectAttackTarget *
        ]
    ),
    "cGcMechSelectAttackTarget::IsValidTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechSelectAttackTarget *
            ctypes.c_uint64,  # cTkAttachmentPtr
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcMechSelectAttackTarget::HasLineOfSightToTarget": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechSelectAttackTarget *
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcMechSelectAttackTarget::GetSightPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechSelectAttackTarget *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcMechStunned::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechStunned *
        ]
    ),
    "cGcMechStunned::cGcMechStunned": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechStunned *
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_ulonglong,  # cGcNavigation *
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ulonglong,  # cGcMechBlackboard *
        ]
    ),
    "cGcMechStunned::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechStunned *
        ]
    ),
    "cGcMechStunned::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechStunned *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSentinelMechAttackShooting::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechAttackShooting *
        ]
    ),
    "cGcSentinelMechAttackShooting::EndFiring": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechAttackShooting *
        ]
    ),
    "cGcSentinelMechAttackShooting::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechAttackShooting *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSentinelMechAttackShooting::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechAttackShooting *
        ]
    ),
    "cGcSentinelMechAttackShooting::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechAttackShooting *
        ]
    ),
    "cGcMechAIController::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAIController *
            ctypes.c_uint32,  # eAIMechFaction
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_ulonglong,  # cGcWeapon *
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
        ]
    ),
    "cGcMechAIController::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAIController *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcMechAIController::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAIController *
        ]
    ),
    "cGcMechAIController::NetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAIController *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cGcMechAIController::RelinquishControl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAIController *
        ]
    ),
    "cGcMechAIController::IsAIPilotAvailable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAIController *
        ]
    ),
    "cGcMechAIController::IsAIPilotInstalled": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAIController *
        ]
    ),
    "cGcMechAIController::NeedsResummon": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAIController *
        ]
    ),
    "cGcMechAIController::TryResummon": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAIController *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechBehaviourTree::Construct": {
        "cGcMechBehaviourTree *, cTkAttachment *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcMechBehaviourTree *
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
        "cGcMechBehaviourTree *, cTkAttachment *, cGcMechControl *, cGcMechAIController *, cGcWeapon *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcMechBehaviourTree *
                ctypes.c_ulonglong,  # cTkAttachment *
                ctypes.c_ulonglong,  # cGcMechControl *
                ctypes.c_ulonglong,  # cGcMechAIController *
                ctypes.c_ulonglong,  # cGcWeapon *
            ]
        ),
    },
    "cGcMechBehaviourTree::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechBehaviourTree *
        ]
    ),
    "sMechTargetData::GetTargetBody": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRigidBody *
        argtypes=[
            ctypes.c_ulonglong,  # sMechTargetData *
        ]
    ),
    "sMechTargetData::GetTargetPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # sMechTargetData *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcSentinelMech::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
            ctypes.c_uint32,  # eSentinelType
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcMechControl::cGcMechControl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
        ]
    ),
    "cGcSentinelMech::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
        ]
    ),
    "cGcSentinelMech::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
            ctypes.c_double,  # double
        ]
    ),
    "cGcSentinelMech::BeginTitanFall": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
        ]
    ),
    "cGcSentinelMech::GetTree": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMechAIController *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
        ]
    ),
    "cGcSentinelMech::FireWeapon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
            ctypes.c_uint32,  # SentinelAttackType
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcSentinelMech::GetTargetPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcSentinelMech::GetForward": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcSentinelMech::GetLabelPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcSentinelMech::GetTargetPointSize": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
        ]
    ),
    "cGcSentinelMech::GetSightPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcSentinelMech::GetObb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
            ctypes.c_ulonglong,  # cTkAABB *
        ]
    ),
    "cGcSentinelMech::GetEffectAttachLocator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "cGcSentinelMech::IsAttacking": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
        ]
    ),
    "cGcSentinelMech::IsAlert": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
        ]
    ),
    "cGcSentinelMech::GetAttackTarget": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachment *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
        ]
    ),
    "cGcSentinelMech::GetAlertRange": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
        ]
    ),
    "cGcSentinelMech::GetSightRange": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
        ]
    ),
    "cGcSentinelMech::GetSightAngle": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
        ]
    ),
    "cGcSentinelMech::NetSerialize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cGcSentinelMech::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
        ]
    ),
    "cGcSentinelMech::UpdateVisibility": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSentinelMech::UpdateWeapons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSentinelMech::UpdateAudio": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSentinelMech::OnHit": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcSentinelMech::OnArmourHit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMech *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcSentinelMechWeapon::GetWeaponRange": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeapon *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcSentinelMechWeapon::GetWeaponSpeed": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeapon *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcSentinelMechWeapon::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeapon *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSentinelMechWeapon::OnBeginFiring": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeapon *
        ]
    ),
    "cGcSentinelMechWeapon::OnEndFiring": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeapon *
        ]
    ),
    "cGcSentinelMechWeapon::SetModeIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeapon *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSentinelMechWeapon::GetListOfModesIfActiveOrNot": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeapon *
            ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
        ]
    ),
    "cGcSentinelMechWeapon::GetShootPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeapon *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_uint32,  # eSentinelMechWeaponMode
        ]
    ),
    "GetNumImposterViews": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "GetTextureWidth": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "GetTextureHeight": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "cGcFadeNode::OctahedralImpostersEnabled": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "cGcFadeNodeUpdater::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFadeNodeUpdater *
        ]
    ),
    "cGcFadeNodeUpdater::UpdateStart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFadeNodeUpdater *
        ]
    ),
    "cGcFadeNodeUpdater::UpdateEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFadeNodeUpdater *
        ]
    ),
    "cGcFadeNodeUpdater::AddNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFadeNodeUpdater *
            ctypes.c_ulonglong,  # cGcFadeNodeBase *
        ]
    ),
    "cGcFadeNodeUpdater::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFadeNodeUpdater *
        argtypes=[]
    ),
    "cGcFadeNodeInstance::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFadeNodeInstance *
        ]
    ),
    "cGcFadeNodeInstance::ClearTestMaterialsRendered": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcFadeNodeInstance::ContextDeleteFunc": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFadeNodeInstance::RenderImposterToTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFadeNodeInstance::Update": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFadeNodeInstance *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcFadeNodeBase::Invalidate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFadeNodeBase *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFadeNodeBase::SetHidden": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFadeNodeBase *
            ctypes.c_ubyte,  # _BOOL8
            ctypes.c_float,  # float
        ]
    ),
    "cGcFadeNodeBase::Update": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcFadeNodeBase *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cGcSceneManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneManager *
        ]
    ),
    "cGcSceneManager::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneManager *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSceneManager::PrepareLighting": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneManager *
        ]
    ),
    "cGcSceneManager::SetLightMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneManager *
        ]
    ),
    "cGcSceneManager::LoadScene": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneManager *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSceneManager::LoadPlanetScenes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneManager *
            ctypes.c_ulonglong,  # cGcSceneSettings *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcSceneSettings>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSceneSettings *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMemoryPool::Realloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMemoryPool *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMemoryPool::GetHighWaterUsage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMemoryPool *
        ]
    ),
    "cTkLinearMemoryPool::ContainsAddress": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkAtomicLinearMemoryPool *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "IAnimatableNode::SetLOD": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # IAnimatableNode *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDoubleBufferMemoryPool::GetAlignment": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # IAnimatableNode *
        ]
    ),
    "cTkLinearMemoryPool::GetUsage": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearMemoryPool *
        ]
    ),
    "cTkAtomicLinearMemoryPool::GetSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAtomicLinearMemoryPool *
        ]
    ),
    "cTkMemoryPool::GetAllocInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMemoryPool *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkMemoryPool::FlagError": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMemoryPool *
            ctypes.c_uint32,  # eMemoryError
        ]
    ),
    "cTkMemoryPool::HasError": FUNCDEF(
        restype=ctypes.c_uint8,  # unsigned __int8
        argtypes=[
            ctypes.c_ulonglong,  # cTkMemoryPool *
            ctypes.c_uint32,  # eMemoryError
        ]
    ),
    "cGcSceneManager::RenderSceneManagerNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkPhysicsComponent::InitCollisionAsBox": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkSketchComponent::sNode::sNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_float,  # float
        ]
    ),
    "cTkSketchComponent::sNode::~sNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        ]
    ),
    "DeleteModel::GetSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # StopSounds *
        ]
    ),
    "OnPlayerNear::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # Sequence *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "OnSpawned::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # OnSpawned *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sOnSpawnedType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sOnSpawnedType *
        ]
    ),
    "sOnSpawnedType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sOnSpawnedType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "StopParticles::GetSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # StopParticles *
        ]
    ),
    "StopParticles::GetSaveAddress": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcRegionDecoratorObjects *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainVeryFar *
        ]
    ),
    "OnVariableChanged::Start": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # OnVariableChanged *
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "OnVariableChanged::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # OnVariableChanged *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "OnVariableChanged::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # OnVariableChanged *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sOnVariableChangedType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sOnVariableChangedType *
        ]
    ),
    "sOnVariableChangedType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sOnVariableChangedType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "OnBroadcastReceived::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # OnBroadcastReceived *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sOnBroadcastReceivedType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sOnBroadcastReceivedType *
        ]
    ),
    "sOnBroadcastReceivedType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sOnBroadcastReceivedType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "RotateNode::GetSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # TranslateNode *
        ]
    ),
    "SetVariableValue::GetSaveSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SetVariableValue *
        ]
    ),
    "OnComponentEvent::Initialise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # OnComponentEvent *
            ctypes.c_ulonglong,  # const cTkSketchComponent *
        ]
    ),
    "OnComponentEvent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # OnComponentEvent *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sOnComponentEventType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sOnComponentEventType *
        ]
    ),
    "sOnComponentEventType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sOnComponentEventType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "BroadcastValueDistance::GetSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SurveyVariableValue *
        ]
    ),
    "OnAnimFrame::GetSaveSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SurveyVariableValue *
        ]
    ),
    "IfValueIs::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # IfValueIs *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "IfValueIs::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # IfValueIs *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sIfValueIsType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sIfValueIsType *
        ]
    ),
    "sIfValueIsType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sIfValueIsType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "FadeModel::GetSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # WaitSeconds *
        ]
    ),
    "ValueAsBool::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # ValueAsBool *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "ValueAsBool::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ValueAsBool *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sValueAsBoolType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sValueAsBoolType *
        ]
    ),
    "sValueAsBoolType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sValueAsBoolType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "FixedValue::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # FixedValue *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "FixedValue::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # FixedValue *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sFixedValueType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sFixedValueType *
        ]
    ),
    "sFixedValueType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sFixedValueType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "GetVariableValue::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # GetVariableValue *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "GetVariableValue::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GetVariableValue *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sGetVariableValueType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sGetVariableValueType *
        ]
    ),
    "sGetVariableValueType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sGetVariableValueType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "SetVariableValue::GetSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SetVariableValue *
        ]
    ),
    "SetVariableValue::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SetVariableValue *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "SetVariableValue::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SetVariableValue *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sSetVariableValueType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sSetVariableValueType *
        ]
    ),
    "sSetVariableValueType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sSetVariableValueType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "SurveyVariableValue::Start": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SurveyVariableValue *
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "SurveyVariableValue::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SurveyVariableValue *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "SurveyVariableValue::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SurveyVariableValue *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sSurveyVariableValueType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sSurveyVariableValueType *
        ]
    ),
    "sSurveyVariableValueType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sSurveyVariableValueType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "Sequence::Initialise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Sequence *
            ctypes.c_ulonglong,  # const cTkSketchComponent *
        ]
    ),
    "Sequence::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Sequence *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sSequenceType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sSequenceType *
        ]
    ),
    "sSequenceType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sSequenceType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "WaitSeconds::Start": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # WaitSeconds *
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "WaitSeconds::GetOutput": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # WaitSeconds *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "WaitSeconds::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # WaitSeconds *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sWaitSecondsType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sWaitSecondsType *
        ]
    ),
    "sWaitSecondsType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sWaitSecondsType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "WaitFrames::Start": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # WaitFrames *
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "WaitFrames::GetOutput": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # WaitFrames *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "WaitFrames::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # WaitFrames *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sWaitFramesType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sWaitFramesType *
        ]
    ),
    "sWaitFramesType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sWaitFramesType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "AkQueuedMsg::Sizeof_SetRandomSeed": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RandomNumber *
        ]
    ),
    "RandomNumber::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RandomNumber *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "RandomNumber::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RandomNumber *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sRandomNumberType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sRandomNumberType *
        ]
    ),
    "sRandomNumberType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sRandomNumberType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "SetShaderValue::GetSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SetShaderValue *
        ]
    ),
    "MathsFunction::GetSaveSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SetShaderValue *
        ]
    ),
    "MathsFunction::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # MathsFunction *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "MathsFunction::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # MathsFunction *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sMathsFunctionType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sMathsFunctionType *
        ]
    ),
    "sMathsFunctionType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sMathsFunctionType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "BroadcastValueModel::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # BroadcastValueModel *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "BroadcastValueModel::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # BroadcastValueModel *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sBroadcastValueModelType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sBroadcastValueModelType *
        ]
    ),
    "sBroadcastValueModelType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sBroadcastValueModelType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "BroadcastValueDistance::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # BroadcastValueDistance *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "BroadcastValueDistance::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # BroadcastValueDistance *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sBroadcastValueDistanceType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sBroadcastValueDistanceType *
        ]
    ),
    "sBroadcastValueDistanceType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sBroadcastValueDistanceType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "RaiseEvent::GetSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SpawnParticles *
        ]
    ),
    "SpawnParticles::GetSaveSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SpawnParticles *
        ]
    ),
    "RaiseEvent::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RaiseEvent *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "RaiseEvent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RaiseEvent *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sRaiseEventType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sRaiseEventType *
        ]
    ),
    "sRaiseEventType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sRaiseEventType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "OnNodeActivated::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # OnNodeActivated *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "OnNodeActivated::CheckTrigger": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # OnNodeActivated *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "sOnNodeActivatedType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sOnNodeActivatedType *
        ]
    ),
    "sOnNodeActivatedType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sOnNodeActivatedType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "OnTriggerVolume::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # OnTriggerVolume *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "OnTriggerVolume::CheckTrigger": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # OnTriggerVolume *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "sOnTriggerVolumeType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sOnTriggerVolumeType *
        ]
    ),
    "sOnTriggerVolumeType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sOnTriggerVolumeType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "OnAnimFrame::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # OnAnimFrame *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "OnAnimFrame::CheckTrigger": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # OnAnimFrame *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "sOnAnimFrameType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sOnAnimFrameType *
        ]
    ),
    "sOnAnimFrameType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sOnAnimFrameType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "SetNodeActive::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SetNodeActive *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "SetNodeActive::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SetNodeActive *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sSetNodeActiveType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sSetNodeActiveType *
        ]
    ),
    "sSetNodeActiveType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sSetNodeActiveType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "DeleteModel::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DeleteModel *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "DeleteModel::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DeleteModel *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sDeleteModelType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sDeleteModelType *
        ]
    ),
    "sDeleteModelType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sDeleteModelType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "PlayAnim::Start": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PlayAnim *
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "PlayAnim::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # PlayAnim *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "PlayAnim::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PlayAnim *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sPlayAnimType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sPlayAnimType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "IfAnimIsPlaying::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # IfAnimIsPlaying *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "IfAnimIsPlaying::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # IfAnimIsPlaying *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sIfAnimIsPlayingType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sIfAnimIsPlayingType *
        ]
    ),
    "sIfAnimIsPlayingType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sIfAnimIsPlayingType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "StopAnim::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # StopAnim *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "StopAnim::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StopAnim *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sStopAnimType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sStopAnimType *
        ]
    ),
    "sStopAnimType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sStopAnimType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "PlaySound::EventCallbackFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # AkCallbackType
            ctypes.c_ulonglong,  # AkCallbackInfo *
        ]
    ),
    "PlaySound::Start": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PlaySound *
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "PlaySound::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # PlaySound *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "PlaySound::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PlaySound *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sPlaySoundType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sPlaySoundType *
        ]
    ),
    "sPlaySoundType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sPlaySoundType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "StopSounds::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # StopSounds *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "StopSounds::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StopSounds *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sStopSoundsType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sStopSoundsType *
        ]
    ),
    "sStopSoundsType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sStopSoundsType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "FadeModel::GetOutput": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # FadeModel *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "FadeModel::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # FadeModel *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sFadeModelType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sFadeModelType *
        ]
    ),
    "sFadeModelType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sFadeModelType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "TranslateNode::GetSaveSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # TranslateNode *
        ]
    ),
    "TranslateNode::Start": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TranslateNode *
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "TranslateNode::GetOutput": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # TranslateNode *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "TranslateNode::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TranslateNode *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sTranslateNodeType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sTranslateNodeType *
        ]
    ),
    "sTranslateNodeType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sTranslateNodeType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "RotateNode::GetOutput": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # RotateNode *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "RotateNode::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RotateNode *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sRotateNodeType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sRotateNodeType *
        ]
    ),
    "sRotateNodeType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sRotateNodeType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "ScaleNode::GetOutput": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # ScaleNode *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "ScaleNode::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ScaleNode *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sScaleNodeType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sScaleNodeType *
        ]
    ),
    "sScaleNodeType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sScaleNodeType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "SetLightIntensity::GetOutput": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # SetLightIntensity *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "SetLightIntensity::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SetLightIntensity *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sSetLightIntensityType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sSetLightIntensityType *
        ]
    ),
    "sSetLightIntensityType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sSetLightIntensityType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "SetLightColour::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SetLightColour *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "SetLightColour::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SetLightColour *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sSetLightColourType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sSetLightColourType *
        ]
    ),
    "sSetLightColourType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sSetLightColourType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "OnPlayerNear::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # OnPlayerNear *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "OnPlayerNear::CheckTrigger": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # OnPlayerNear *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "sOnPlayerNearType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sOnPlayerNearType *
        ]
    ),
    "sOnPlayerNearType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sOnPlayerNearType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "SpawnParticles::Start": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SpawnParticles *
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "SpawnParticles::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SpawnParticles *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "SpawnParticles::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SpawnParticles *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sSpawnParticlesType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sSpawnParticlesType *
        ]
    ),
    "sSpawnParticlesType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sSpawnParticlesType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "SetParticlesStage::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SetParticlesStage *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "SetParticlesStage::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SetParticlesStage *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sSetParticlesStageType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sSetParticlesStageType *
        ]
    ),
    "sSetParticlesStageType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sSetParticlesStageType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "StopParticles::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # StopParticles *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "StopParticles::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StopParticles *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sStopParticlesType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sStopParticlesType *
        ]
    ),
    "sStopParticlesType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sStopParticlesType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "DestroyDestructables::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DestroyDestructables *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "DestroyDestructables::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DestroyDestructables *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sDestroyDestructablesType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sDestroyDestructablesType *
        ]
    ),
    "sDestroyDestructablesType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sDestroyDestructablesType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "SetHazard::GetOutput": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # SetHazard *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "SetHazard::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SetHazard *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "sSetHazardType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sSetHazardType *
        ]
    ),
    "sSetHazardType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sSetHazardType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "DamagePlayer::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DamagePlayer *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "DamagePlayer::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DamagePlayer *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sDamagePlayerType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sDamagePlayerType *
        ]
    ),
    "sDamagePlayerType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sDamagePlayerType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "CameraShake::Start": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CameraShake *
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "CameraShake::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CameraShake *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CameraShake::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CameraShake *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sCameraShakeType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sCameraShakeType *
        ]
    ),
    "sCameraShakeType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sCameraShakeType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "StopCameraShake::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # StopCameraShake *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "StopCameraShake::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StopCameraShake *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sStopCameraShakeType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sStopCameraShakeType *
        ]
    ),
    "sStopCameraShakeType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sStopCameraShakeType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "FadeMaterials::GetSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # FadeMaterials *
        ]
    ),
    "FadeMaterials::GetSaveSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # FadeMaterials *
        ]
    ),
    "FadeMaterials::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # FadeMaterials *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "FadeMaterials::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # FadeMaterials *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sFadeMaterialsType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sFadeMaterialsType *
        ]
    ),
    "sFadeMaterialsType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sFadeMaterialsType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "SetMaterialUniform::GetSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SetMaterialUniform *
        ]
    ),
    "hknpAnimatedShapeSurfaceVelocityModifier::getEnabledFunctions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SetMaterialUniform *
        ]
    ),
    "SetMaterialUniform::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SetMaterialUniform *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "SetMaterialUniform::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SetMaterialUniform *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sSetMaterialUniformType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sSetMaterialUniformType *
        ]
    ),
    "sSetMaterialUniformType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sSetMaterialUniformType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "SetShaderValue::GetOutput": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # SetShaderValue *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "SetShaderValue::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SetShaderValue *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sSetShaderValueType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sSetShaderValueType *
        ]
    ),
    "sSetShaderValueType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sSetShaderValueType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "GiveReward::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # GiveReward *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "GiveReward::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GiveReward *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sGiveRewardType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sGiveRewardType *
        ]
    ),
    "sGiveRewardType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sGiveRewardType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "AddShootableHealth::GetSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AddShootableHealth *
        ]
    ),
    "AddShootableHealth::GetSaveSizeBytes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AddShootableHealth *
        ]
    ),
    "AddShootableHealth::GetOutput": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # AddShootableHealth *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "AddShootableHealth::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AddShootableHealth *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sAddShootableHealthType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sAddShootableHealthType *
        ]
    ),
    "sAddShootableHealthType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sAddShootableHealthType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "GetPlanetColour::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # GetPlanetColour *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "GetPlanetColour::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GetPlanetColour *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sGetPlanetColourType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sGetPlanetColourType *
        ]
    ),
    "sGetPlanetColourType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sGetPlanetColourType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "GetTimeOfDay::GetOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # GetTimeOfDay *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "GetTimeOfDay::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GetTimeOfDay *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "sGetTimeOfDayType::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sGetTimeOfDayType *
        ]
    ),
    "sGetTimeOfDayType::ConstructNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        argtypes=[
            ctypes.c_ulonglong,  # sGetTimeOfDayType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "cGcInstancedPhysicsAsteroidGrid::cGcInstancedPhysicsAsteroidGrid": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInstancedPhysicsAsteroidGrid *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcInstancedPhysicsAsteroidGrid::ClearCells": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInstancedPhysicsAsteroidGrid *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcInstancedPhysicsAsteroidGrid::CalculateAsteroidTransform": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInstancedPhysicsAsteroidGrid *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInstancedPhysicsAsteroidGrid::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInstancedPhysicsAsteroidGrid *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcInstancedPhysicsAsteroidGrid::GenerateData": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcInstancedPhysicsAsteroidGrid *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcInstancedPhysicsAsteroidGrid::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInstancedPhysicsAsteroidGrid *
        ]
    ),
    "cTkCollision::cTkCollision": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_ulonglong,  # const cTkCollision *
        ]
    ),
    "cGcInstancedPhysicsAsteroidGrid::NotifyDataChanged": {
        "cGcInstancedPhysicsAsteroidGrid *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcInstancedPhysicsAsteroidGrid *
            ]
        ),
        "cGcInstancedPhysicsAsteroidGrid *, int, int *, cTkMatrix44 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcInstancedPhysicsAsteroidGrid *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # int *
                ctypes.c_ulonglong,  # cTkMatrix44 *
            ]
        ),
    },
    "cGcInstancedPhysicsAsteroidGrid::AutoUpdateForInvalidArea": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInstancedPhysicsAsteroidGrid *
            ctypes.c_uint64,  # cTkClassPoolHandle
        ]
    ),
    "cGcInstancedPhysicsAsteroidGrid::AssignRigidBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInstancedPhysicsAsteroidGrid *
        ]
    ),
    "cGcInstancedPhysicsAsteroidGrid::HitBody": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcInstancedPhysicsAsteroidGrid *
            ctypes.c_ulonglong,  # const cTkRigidBody *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcInstancedPhysicsAsteroidGrid::DisablePhyscis": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInstancedPhysicsAsteroidGrid *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAsteroidManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidManager *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcAsteroidManager::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidManager *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcAsteroidManager::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidManager *
        ]
    ),
    "cGcAsteroidManager::Generate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidManager *
            ctypes.c_ulonglong,  # const cGcAsteroidLayout *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcAsteroidManager::RenderImGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidManager *
        ]
    ),
    "cGcAsteroidManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAsteroidManager::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidManager *
        ]
    ),
    "cGcAsteroidManager::AddInvalidArea": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPoolHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidManager *
            ctypes.c_ulonglong,  # cTkClassPoolHandle *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAsteroidManager::RefreshAsteroids": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidManager *
        ]
    ),
    "cGcAsteroidManager::IsPointWithinAsteroidField": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidManager *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cGcAsteroidTypeMask *
        ]
    ),
    "cGcAsteroidManager::GetFuelAsteroidBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidManager *
            ctypes.c_ulonglong,  # cTkStackVector<cTkRigidBodyPtr,32,-1> *
        ]
    ),
    "cGcAsteroidManager::HitAsteroid": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidManager *
            ctypes.c_ulonglong,  # const cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcAsteroidPatternGenerator::Generate": {
        "const cGcAsteroidGeneratorRing *, cGcAsteroidLayout *, const cTkSphere *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const cGcAsteroidGeneratorRing *
                ctypes.c_ulonglong,  # cGcAsteroidLayout *
                ctypes.c_ulonglong,  # const cTkSphere *
            ]
        ),
        "const cGcAsteroidGeneratorSurround *, cGcAsteroidLayout *, const cTkSphere *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const cGcAsteroidGeneratorSurround *
                ctypes.c_ulonglong,  # cGcAsteroidLayout *
                ctypes.c_ulonglong,  # const cTkSphere *
            ]
        ),
        "const cGcAsteroidGeneratorSlab *, cGcAsteroidLayout *, const cTkSphere *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const cGcAsteroidGeneratorSlab *
                ctypes.c_ulonglong,  # cGcAsteroidLayout *
                ctypes.c_ulonglong,  # const cTkSphere *
            ]
        ),
    },
    "cTkVoxelGeneratorAsteroid::Generate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
            common.GcSeed,  # cTkSeed
            ctypes.c_uint32,  # eVoxelType
        ]
    ),
    "cGcModuleSet::DeleteBuildingPresetsForAllModules": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcBuilding::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuilding *
        ]
    ),
    "cGcBuilding::Setup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuilding *
            ctypes.c_ulonglong,  # cGcRegionTerrainFar *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcBuilding::GetPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuilding *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcBuilding::GetMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuilding *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcBuilding::WholeClusterAdded": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuilding *
        ]
    ),
    "cGcBuilding::GetSurroundingBuilding": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBuilding *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuilding *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBuilding::DestroyIntersectingVolcanoes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuilding *
        ]
    ),
    "cGcBuilding::Scanned": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuilding *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuilding::Visited": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuilding *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuilding::UpdateDistanceFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuilding *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcBuildingList::AddBuilding": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBuilding *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingList *
            ctypes.c_ulonglong,  # cGcBuildingSpawnData *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcBuildingList::SortByDistance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingList *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcBuildingList::RemoveBuilding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingList *
            ctypes.c_ulonglong,  # cGcBuilding *
            ctypes.c_ulonglong,  # cGcRegionTerrainFar *
        ]
    ),
    "cGcBuildingPlacement::GetBuildingsAtPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,128> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # cGcBuildingList *
            ctypes.c_ulonglong,  # std::vector<cTkSeed,TkSTLAllocatorShim<cTkSeed,8,-1> > *
        ]
    ),
    "cGcBuildingPlacement::GetTheorecticalBuildingPosition": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # cGcBuildingList *
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_int32,  # int
        ]
    ),
    "GetClusterLayout": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcBuildingClusterLayout *
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingSpawnData *
        ]
    ),
    "cGcBuildingPlacement::FindClusterBuildings": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetBuildingData *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingPlacement::CheckBuildingCanBePlaced": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cGcPlanetBuildingData *
            ctypes.c_int32,  # int
        ]
    ),
    "RoundDirection": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_char * 0x10,  # cTkVector3
        ]
    ),
    "cGcClusterLayoutPlacement::GetClusterLayout": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetBuildingData *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # const cGcBuildingClusterLayout *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcClusterLayoutPlacement::GetOverlapError": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkAABB *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkAABB *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcClusterLayoutPlacement::TestBox": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcClusterLayoutPlacement::sMapBox *
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cGcClusterLayoutPlacement::sClusterLayoutBox *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcPlanetBuildingGenerator::GenerateBuildings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetBuildingGenerator *
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_uint32,  # eBuildingDensity
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # cGcBuildingList *
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetBuildingGenerator::PollToPlaceInitialBuildings": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetBuildingGenerator *
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanetBuildingGenerator::AddOverrideBuildings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetBuildingGenerator *
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_uint32,  # eBuildingDensity
            ctypes.c_ulonglong,  # cGcBuildingList *
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetBuildingGenerator::AddSurvivalBuildings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetBuildingGenerator *
            ctypes.c_ulonglong,  # cGcBuildingList *
            ctypes.c_ulonglong,  # std::vector<cTkSeed,TkSTLAllocatorShim<cTkSeed,8,-1> > *
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,128> *
        ]
    ),
    "cGcPlanetBuildingGenerator::GenerateBuildingModels": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetBuildingGenerator *
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_uint32,  # eBuildingDensity
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetBuildingGenerator::SetupBuildingFilenames": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetBuildingGenerator *
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ulonglong,  # cGcBuildingSpawnData *
            ctypes.c_ulonglong,  # cGcBuildingDefinitionData *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcPlanetBuildingGenerator::PollCalculateBuildingSizes": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetBuildingGenerator *
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcVoxelAsteroidGrid<5>::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVoxelAsteroidGrid<5> *
        ]
    ),
    "cTkBitArray<unsigned __int64,1,125>::GetNumTrue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,125> *
        ]
    ),
    "cGcAsteroidGrid<5>::Iterate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGrid<5> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkBitArray<unsigned __int64,0,0>::GetFirstZeroBit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,0,0> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkBitArray<unsigned __int64,0,0>::cTkBitArray<unsigned __int64,0,0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,0,0> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkSketchNodeConnections>::~cTkDynamicArray<cTkSketchNodeConnections>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkSketchNodeConnections> *
        ]
    ),
    "cTkRegionMap<16,16,32,-1,cGcRegionAsteroid>::Edit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegionMap<16,16,32,-1,cGcRegionAsteroid> *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int8,  # sTerrainEditData
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # std::array<unsigned int,11> *
        ]
    ),
    "cGcVoxelAsteroidGrid<5>::GenerateData": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcVoxelAsteroidGrid<5> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkSketchComponent::sNode::DoComponentRequiredFlag<cTkAnimationComponent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        ]
    ),
    "cTkSketchComponent::GetComponentForSketch<cTkAnimationComponent>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimationComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "cTkSketchComponent::GetComponentForSketch<cTkAudioComponent>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAudioComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "cTkSketchComponent::sNode::DoComponentRequiredFlag<cTkAudioComponent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        ]
    ),
    "cTkSketchComponent::sNode::DoEditEnum<eHazard>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eHazard *
        ]
    ),
    "cTkSketchComponent::GetComponentForSketch<cGcShootableComponent>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcShootableComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "cTkSketchComponent::sNode::DoComponentRequiredFlag<cGcShootableComponent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        ]
    ),
    "cGcAsteroidGrid<5>::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGrid<5> *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcAsteroidGrid<5>::CalculateAsteroidTransform": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGrid<5> *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditor::DoComboBox<eAsteroidLevel>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eAsteroidLevel *
            ctypes.c_float,  # float
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcBuildingDefinitionTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBuildingDefinitionTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcSettlementColourTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSettlementColourTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditor::DoComboBox<eCondition>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eCondition *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoComboBox<eAkEvent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eAkEvent *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoComboBox<eHazard>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eHazard *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoComboBox<ePalette>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # ePalette *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoComboBox<eColourAlt>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eColourAlt *
            ctypes.c_float,  # float
        ]
    ),
    "hkArrayBase<float>::_ensureOwnsStorage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<int> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "cGcRegionAsteroid::ThreadedPolygonise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionAsteroid *
            ctypes.c_ulonglong,  # cTkRazorLocalContext *
        ]
    ),
    "cGcRegionAsteroid::ThreadedGenerateVoxels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionAsteroid *
            ctypes.c_ulonglong,  # cTkRazorLocalContext *
        ]
    ),
    "cGcRegionAsteroid::CanPolygonise": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionAsteroid *
        ]
    ),
    "cGcRegionAsteroid::PostPolygoniseThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionAsteroid *
        ]
    ),
    "cGcRegionAsteroid::Polygonise": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionAsteroid *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcRegionAsteroid::GetResource": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionAsteroid *
        ]
    ),
    "cGcRegionAsteroid::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionAsteroid *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkRegionMapBase *
            ctypes.c_ulonglong,  # cTkVoxelArray *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRegionAsteroid::PostPolygonise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionAsteroid *
        ]
    ),
    "cGcRegionAsteroid::PollToUnmapStreams": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionAsteroid *
        ]
    ),
    "cTkRegionMap<16,16,32,-1,cGcRegionAsteroid>::GetScaleZ": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegionMap<16,16,32,-1,cGcRegionAsteroid> *
        ]
    ),
    "cGcBuildingList::CompareBuildingDistances": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcBuildingList::CompareBuildingIDs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cGcRegionAsteroid::Invalidate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionAsteroid *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcAsteroidManager::RenderImGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkRegion::PollToUnmapStreams": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
        ]
    ),
    "cTkRegion::Assign": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkRegion::TryToClear": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
        ]
    ),
    "cTkRegion::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
        ]
    ),
    "cTkRegion::RefreshKnowledge": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
        ]
    ),
    "cTkRegion::GetStatusColour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
            ctypes.c_ulonglong,  # cTkColour *
        ]
    ),
    "cGcRegionBase::GetFacingTriangleStart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionBase *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRegionBase::GetTileBlendTriangleStart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionBase *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRegionBase::cGcRegionBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionBase *
        ]
    ),
    "cTkAnimationComponent::GetLayerHandleForAnim": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimLayerHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_ulonglong,  # cTkAnimLayerHandle *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcWFCFace::~cGcWFCFace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCFace *
        ]
    ),
    "sWFCNavigationSubgraphNode::sWFCNavigationSubgraphNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sWFCNavigationSubgraphNode *
            ctypes.c_ulonglong,  # const sWFCNavigationSubgraphNode *
        ]
    ),
    "cGcBuildingPreset::~cGcBuildingPreset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingPreset *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cEgAnimationController::JointLODGroup,8,-1> *
        ]
    ),
    "cGcBuildingPreset::Generate": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingPreset *
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cEgAnimationController::JointLODGroup,8,-1> *
        ]
    ),
    "cGcBuildingPreset::GenerateUsingFallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingPreset *
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBuildingPreset::MergeBounds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingPreset *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMap::cGcMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_ulonglong,  # const cGcMap *
        ]
    ),
    "cGcMap::~cGcMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cEgAnimationController::JointLODGroup,8,-1> *
        ]
    ),
    "cGcMap::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
        ]
    ),
    "cGcMap::OnCollapseSlot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_ulonglong,  # cGcSlot *
        ]
    ),
    "cGcMap::CollapseAll": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMap::Generate": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcMap::ApplyMinimumUseConstraints": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcMap::FullyCollapsed": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
        ]
    ),
    "IsTerrainEditBoxNode": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "CanUseDecorationItem": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSlot *
            ctypes.c_ulonglong,  # const cGcWFCDecorationItem *
            ctypes.c_uint32,  # const eBlockRotation
        ]
    ),
    "cGcMap::PickDecorationItems": {
        "cGcMap *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcMap *
            ]
        ),
        "cGcMap *, eWFCDecorationTheme, std::vector<std::pair<cTkIntTuple3<int,0>,sSlotDecoration>,TkSTLAllocatorShim<std::pair<cTkIntTuple3<int,0>,sSlotDecoration>,8,-1> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcMap *
                ctypes.c_uint32,  # eWFCDecorationTheme
                ctypes.c_ulonglong,  # std::vector<std::pair<cTkIntTuple3<int,0>,sSlotDecoration>,TkSTLAllocatorShim<std::pair<cTkIntTuple3<int,0>,sSlotDecoration>,8,-1> > *
            ]
        ),
    },
    "cGcMap::CreateDecorationPartsList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_uint32,  # eWFCDecorationTheme
            ctypes.c_ulonglong,  # std::vector<sWFCBuildingPart,TkSTLAllocatorShim<sWFCBuildingPart,16,18> > *
        ]
    ),
    "cGcMap::CopyState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_ulonglong,  # const cGcMap *
        ]
    ),
    "cGcMap::ApplyConstraints": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
        ]
    ),
    "cGcMap::PropagateConstraints": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
        ]
    ),
    "cGcMap::ApplyWallThemes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_ulonglong,  # std::vector<sWFCBuildingPart,TkSTLAllocatorShim<sWFCBuildingPart,16,18> > *
        ]
    ),
    "cGcMap::CreatePartsList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_ulonglong,  # std::vector<sWFCBuildingPart,TkSTLAllocatorShim<sWFCBuildingPart,16,18> > *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMap::CreateNPCList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_ulonglong,  # std::vector<sBuildingNPCPlacement,TkSTLAllocatorShim<sBuildingNPCPlacement,16,18> > *
        ]
    ),
    "cGcMap::GetBoundingBox": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAABB *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_ulonglong,  # cTkAABB *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMap::DecideLayout": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcMap::GetGroundLevelBoundingBoxes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_ulonglong,  # std::vector<cTkAABB,TkSTLAllocatorShim<cTkAABB,16,-1> > *
        ]
    ),
    "cGcMap::GetNonApplyingSettlementProgressBits": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
        ]
    ),
    "cGcMap::IsInside": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_char * 0x10,  # cTkVector3
        ]
    ),
    "cGcMap::FindReachableSlots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
        ]
    ),
    "cGcMap::FindRooms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cEgAnimationController::JointLODGroup,8,-1> *
        ]
    ),
    "cGcMap::GetRoom": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sFreighterRoom *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_ulonglong,  # const cGcSlot *
        ]
    ),
    "cGcMap::ImproveConnectedAreas": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
        ]
    ),
    "cGcMap::GetStartSlot": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSlot *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
        ]
    ),
    "cGcMap::FindConnectedAreas": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_ulonglong,  # std::vector<std::vector<cGcSlot *,TkSTLAllocatorShim<cGcSlot *,8,-1> >,TkSTLAllocatorShim<std::vector<cGcSlot *,TkSTLAllocatorShim<cGcSlot *,8,-1> >,8,-1> > *
        ]
    ),
    "cGcMap::AllocateSlots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
        ]
    ),
    "cGcMap::GetLowestEntropySlot": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSlot *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMap *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMapLayout::cGcMapLayout": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMapLayout *
            ctypes.c_ulonglong,  # const cGcModuleSet *
            ctypes.c_ulonglong,  # const cTkFixedString<128,char> *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcMapLayout::PollToParse": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMapLayout *
        ]
    ),
    "GetModule": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcModule *
        argtypes=[
            ctypes.c_uint32,  # const TkHandle
            ctypes.c_ulonglong,  # const cGcModuleSet *
        ]
    ),
    "GetSlotPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkIntTuple3<int,0> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkIntTuple3<int,0> *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_char * 0x10,  # cTkVector3
        ]
    ),
    "cGcMapLayout::Parse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMapLayout *
            ctypes.c_uint32,  # const TkHandle
        ]
    ),
    "cGcModule::cGcModule": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModule *
            ctypes.c_int32,  # const int
            ctypes.c_uint32,  # eBlockRotation
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcModuleSet *
            ctypes.c_ulonglong,  # const cGcWFCScenes *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "IsExcluded": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWFCModulePrototype *
            ctypes.c_uint32,  # const eBlockDirection
            ctypes.c_ulonglong,  # const cGcWFCModulePrototype *
            ctypes.c_uint32,  # const eBlockDirection
            ctypes.c_uint32,  # const eBlockRotation
        ]
    ),
    "cGcModule::GetFace": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWFCFace *
        argtypes=[
            ctypes.c_ulonglong,  # cGcModule *
            ctypes.c_uint32,  # eBlockDirection
        ]
    ),
    "cGcModule::TransferPossibleNeighbourDataFromPreviousModule": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModule *
        ]
    ),
    "cGcModule::BoundingBoxWithoutRotation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAABB *
        argtypes=[
            ctypes.c_ulonglong,  # cGcModule *
            ctypes.c_ulonglong,  # cTkAABB *
        ]
    ),
    "GetFace": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWFCFace *
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWFCModulePrototype *
            ctypes.c_uint32,  # eBlockDirection
        ]
    ),
    "cGcModuleSet::RotationsMatch": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWFCModulePrototype *
            ctypes.c_uint32,  # eBlockRotation
        ]
    ),
    "cGcModuleSet::LoadModuleSet": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWFCBuilding *
        ]
    ),
    "cGcModuleSet::Import": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcModuleSet *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFixedString<128,char> *
            ctypes.c_ulonglong,  # const cGcWFCBuilding *
        ]
    ),
    "cGcModuleSet::GetModulesWithoutConnectors": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcModuleSubset *
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleSet *
            ctypes.c_ulonglong,  # const cGcModuleSubset *
            ctypes.c_ulonglong,  # std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> > *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkFixedString<32,char> > *
        ]
    ),
    "cGcModuleSet::GetDecorationItemGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcWFCScene *
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleSet *
            ctypes.c_ulonglong,  # const cGcWFCDecorationItem *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcModuleSet::CreateNewMapFromPreset": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMap *
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleSet *
            ctypes.c_int32,  # int
            common.GcSeed,  # const cTkSeed
        ]
    ),
    "cGcModuleSet::ConnectorsFit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleSet *
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,128> *
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,128> *
        ]
    ),
    "cGcModuleSet::GetOrCreateBuildingPreset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleSet *
            ctypes.c_ulonglong,  # const cGcPlanet *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcModuleSet::PollToOptimize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleSet *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cEgAnimationController::JointLODGroup,8,-1> *
        ]
    ),
    "cGcModuleSet::PrepareDecoration": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleSet *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcModuleSet::PrepareBoundaryConstraints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleSet *
        ]
    ),
    "Mirrored": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cGcModule *
            ctypes.c_ulonglong,  # const cGcModule *
        ]
    ),
    "cGcModuleSet::PreparePossibleNeighbours": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleSet *
        ]
    ),
    "cGcModuleSet::PrepareCompatibleConnectors": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleSet *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> > *
        ]
    ),
    "cGcModuleSet::CreateMapParameters": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sMapParameters *
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleSet *
            ctypes.c_ulonglong,  # sMapParameters *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "GetConnectorForId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,128> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,128> *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_ulonglong,  # std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> > *
        ]
    ),
    "cGcModuleSet::GetConnector": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkBitArray<unsigned __int64,1,128> *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBitArray<unsigned __int64,1,128> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> > *
        ]
    ),
    "cGcModuleSubset::Intersects": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleSubset *
            ctypes.c_ulonglong,  # const cGcModuleSubset *
        ]
    ),
    "cGcModuleSubset::IsSplitBy": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleSubset *
            ctypes.c_ulonglong,  # const std::vector<cGcModuleSubset,TkSTLAllocatorShim<cGcModuleSubset,16,-1> > *
        ]
    ),
    "cGcModuleSubset::GetEntropyForSplit": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleSubset *
            ctypes.c_ulonglong,  # const std::vector<cGcModuleSubset,TkSTLAllocatorShim<cGcModuleSubset,16,-1> > *
        ]
    ),
    "cGcModuleSubset::PickGroup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcModuleSubset *
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleSubset *
            ctypes.c_ulonglong,  # const std::vector<cGcModuleSubset,TkSTLAllocatorShim<cGcModuleSubset,16,-1> > *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcSlot::Collapse": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSlot *
            ctypes.c_ulonglong,  # cGcModule *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSlot::CollapseRandom": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSlot *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSlot::GetRelativeTransform": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSlot *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_uint32,  # eBlockRotation
        ]
    ),
    "cGcSlot::RemoveModules": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSlot *
            ctypes.c_ulonglong,  # const cGcModuleSubset *
        ]
    ),
    "cGcSlot::RemoveAllBut": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSlot *
            ctypes.c_ulonglong,  # const cGcModuleSubset *
        ]
    ),
    "cGcSlot::UpdateNeighbours": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSlot *
        ]
    ),
    "cGcSlot::GetNeighbour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSlot *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSlot *
            ctypes.c_uint32,  # eBlockDirection
        ]
    ),
    "cGcSlot::CanWalk": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSlot *
            ctypes.c_uint32,  # eBlockDirection
        ]
    ),
    "cGcSlot::GetNavigationSubGraph": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::pair<cGcWFCNavigationSubGraph const &,enum eBlockRotation> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSlot *
            ctypes.c_ulonglong,  # std::pair<cGcWFCNavigationSubGraph const &,enum eBlockRotation> *
        ]
    ),
    "cGcSlot::IsBuildingEntrance": {
        "cGcSlot *, eBlockDirection": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcSlot *
                ctypes.c_uint32,  # eBlockDirection
            ]
        ),
        "cGcSlot *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cGcSlot *
            ]
        ),
    },
    "cGcWFCModulePrototype::~cGcWFCModulePrototype": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCModulePrototype *
        ]
    ),
    "GetRelativeTransform": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const TkHandle *
        ]
    ),
    "cGcWFCScene::PollToParseScenes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcWFCScene::Get": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWFCScene *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFixedString<128,char> *
        ]
    ),
    "cGcWFCScene::BasebuildingTerrainEditBoxes": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const std::vector<std::pair<cTkAABB,cTkMatrix34>,TkSTLAllocatorShim<std::pair<cTkAABB,cTkMatrix34>,16,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCScene *
        ]
    ),
    "cGcWFCScenes::PickGeometryIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCScenes *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "GetNavigationNodeArriveDistance": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcWFCNavigationSubGraph::ParseScene": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCNavigationSubGraph *
            ctypes.c_ulonglong,  # const TkHandle *
            ctypes.c_ulonglong,  # std::vector<sTransformedBasebuildingPart,TkSTLAllocatorShim<sTransformedBasebuildingPart,16,-1> > *
            ctypes.c_ulonglong,  # __int16 *
        ]
    ),
    "FindInteractiveObjectInParents": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCInteractiveObjectComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkMetadataCache<cGcWFCDecorationSet>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWFCDecorationSet *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcWFCDecorationSet> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcWFCModuleSet>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWFCModuleSet *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcWFCModuleSet> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcWFCModuleSet>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cGcWFCModuleSet> *
        argtypes=[]
    ),
    "cTkBitArray<unsigned __int64,1,512>::SetFirstNBitsTrue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,512> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcWeightedResource>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWeightedResource> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcWFCDecorationSet>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWFCDecorationSet *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcWFCModuleSet>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWFCModuleSet *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingManager::SetBasePlanetPositions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingManager *
            ctypes.c_ulonglong,  # const cTkFixedArray<cTkVector3,8> *
        ]
    ),
    "cGcSolarSystem::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSolarSystem::PollToPrepare": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSolarSystem::PreloadCommonResources": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcSolarSystem::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSolarSystem::UpdatePlanets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_float,  # float
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSolarSystem::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSolarSystem::CanRelease": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSolarSystem::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSolarSystem::Generate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ubyte,  # bool
            ctypes.POINTER(common.GcSeed),  # cTkSeed *
        ]
    ),
    "cGcSolarSystem::PollPostPlanetGeneration": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSolarSystem::DeferredCreateSpaceStation": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSolarSystem::DeferredCreateNexusExterior": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSolarSystem::DeferredCreateNexus": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSolarSystem::DeferredCreateBaseBuilding": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSolarSystem::ChangePrimaryPlanet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcSolarSystem::RequestPrimaryPlanetChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystem::LoadAnomaly": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_uint32,  # eGalaxyStarAnomaly
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcSolarSystem::SpawnAnomaly": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcSolarSystem::SetNexusVisibility": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcSolarSystem::IsPointInsideSpaceStationInternals": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcSolarSystem::IsPointInsideAnomalyInternals": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcSolarSystem::IsPointInsideNexusInternals": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystem::IsPointInsideSpaceStationBoundingSphere": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSolarSystem::IsPointInsideNexusExteriorBoundingSphere": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSolarSystem::GetName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcSolarSystem::GetSpaceStationName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # cTkFixedString<127,char> *
        ]
    ),
    "cGcSolarSystem::OnEnterPlanetOrbit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystem::OnLeavePlanetOrbit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystem::DeactivateSpaceObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSolarSystem::ActivateSpaceObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSolarSystem::SetHeavyAir": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_uint32,  # eHeavyAirType
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystem::TryToLockPositionToNearestPlanet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_double,  # double
            ctypes.c_float,  # float
        ]
    ),
    "cGcSolarSystem::QueryLocators": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # const cGcSolarSystemLocatorChoice *
            ctypes.c_ulonglong,  # cTkStackVector<cGcSolarSystemLocator const *,32,-1> *
        ]
    ),
    "cGcSolarSystem::ResolveAsteroidAssignment": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # const cGcAsteroidGeneratorAssignment *
            ctypes.c_ulonglong,  # cTkSphere *
        ]
    ),
    "cGcSolarSystem::GetPlanetWithVoxelData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlanet *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSolarSystem::CheckPlanetForInitialSpawn": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcSolarSystem::GetNexusExteriorSphere": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSphere *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # cTkSphere *
        ]
    ),
    "cGcSolarSystem::GetPlayerSolarSystemInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGalacticSolarSystemAddress *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalacticSolarSystemAddress *
        ]
    ),
    "cGcSolarSystem::GetUA": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSolarSystem::GetNexusSpawnMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cGcSolarSystem::UpdateSSRNodesForModel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcSolarSystem::IsSphereClearOfSoloarSystemObjects": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_ulonglong,  # const cTkSphere *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSolarSystem::ActivateStationAfterWarp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSolarSystem::SummonNexusExteriorSafe": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSolarSystem::CanConsiderSummoningNexus": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
        ]
    ),
    "cGcSolarSystem::DoesBuildingDensityHaveBuildingClass": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystem *
            ctypes.c_uint32,  # eBuildingDensity
            ctypes.c_uint32,  # eBuildingClass
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemDirector::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemDirector *
        ]
    ),
    "cGcSolarSystemDirector::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemDirector *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSolarSystemDirector::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemDirector *
        ]
    ),
    "cGcSolarSystemGenerator::GenerateQueryInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cGcGalaxyAttributesAtAddress *
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator::GenerationData *
        ]
    ),
    "cGcSolarSystemGenerator::GenerateBiomesInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # cGcSolarSystemData *
        ]
    ),
    "cGcSolarSystemGenerator::Generate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cGcGalaxyAttributesAtAddress *
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator::GenerationData *
        ]
    ),
    "cGcSolarSystemGenerator::GenerateBasics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cGcGalaxyAttributesAtAddress *
            ctypes.c_ulonglong,  # cGcGalaxyAttributeGenerator::StarSystemKeyAttributes *
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator::GenerationData *
        ]
    ),
    "cGcSolarSystemGenerator::GeneratePlanetBiomes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator *
            ctypes.c_ulonglong,  # const cGcGalaxyAttributesAtAddress *
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator::GenerationData *
            ctypes.c_ulonglong,  # cGcGalaxyAttributeGenerator::StarSystemKeyAttributes *
        ]
    ),
    "cGcSolarSystemGenerator::GeneratePlanetPositions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator *
            ctypes.c_ulonglong,  # const cGcGalaxyAttributesAtAddress *
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator::GenerationData *
            ctypes.c_ulonglong,  # cGcSolarSystemGeometry *
        ]
    ),
    "cGcSolarSystemGenerator::GenerateSpaceStationAndPlayerSpawn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator *
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator::GenerationData *
        ]
    ),
    "cGcSolarSystemGenerator::GenerateAsteroids": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator *
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator::GenerationData *
        ]
    ),
    "cGcSolarSystemGenerator::GenerateSky": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator *
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator::GenerationData *
        ]
    ),
    "cGcSolarSystemGenerator::GenerateAttributeStrings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # eTradingClass
            ctypes.c_uint32,  # eWealthClass
            ctypes.c_uint32,  # eConflictLevel
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcSolarSystemGenerator::GenerateProceduralStarClass": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
            ctypes.c_uint32,  # eGalaxyStarType
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcSolarSystemGenerator::GenerateCelestialBodyStrings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cGcGalaxyStarAttributesData *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemGenerator::GenerateLocators": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator *
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator::GenerationData *
        ]
    ),
    "cGcSolarSystemGenerator::ShouldReplaceWithSailShip": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ubyte,  # const bool
            common.GcSeed,  # const cTkSeed
            common.GcSeed,  # const cTkSeed
            ctypes.c_uint32,  # const eShipClass
        ]
    ),
    "cGcSolarSystemQuery::Run": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemQuery *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # cGcSolarSystemQuery::Type
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcSpaceSkyColourSettingList>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEntitlementRewardsTable *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcSpaceSkyColourSettingList> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcSolarSystemData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSolarSystemData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradeRoute::cGcTradeRoute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeRoute *
            ctypes.c_ulonglong,  # const cGcTradeRoute *
        ]
    ),
    "cGcTradeRouteSection::cGcTradeRouteSection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeRouteSection *
            ctypes.c_ulonglong,  # const cGcTradeRouteSection *
        ]
    ),
    "cTkDynamicArray<cGcSolarSystemLocator>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSolarSystemLocator> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcAISpaceshipPreloadCacheData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAISpaceshipPreloadCacheData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcWarpEffect::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWarpEffect *
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "NavGraphPhysProbeJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcNPCManager::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCManager *
        ]
    ),
    "cGcNPCManager::GetNPCResource": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCManager *
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            ctypes.c_ulonglong,  # const cGcNPCManager::NPCResourceRequest *
        ]
    ),
    "cGcNPCManager::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCManager *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcNPCManager::ManagedNPCResource,16,-1> *
        ]
    ),
    "cGcNPCManager::StartGenerationTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCManager *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
        ]
    ),
    "cGcNPCManager::SetPoseBonesForRace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const eAlienRace
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcNPCAnimationManager::GetReaction": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCAnimationManager *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # eAlienRace
        ]
    ),
    "cGcNPCAnimationManager::GetMoodAnim": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCAnimationManager *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_uint32,  # eMood
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkDynamicArray<TkID<128> > *
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCAnimationManager::ChooseAnim": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCAnimationManager *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcNPCProbabilityAnimationData> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<TkID<128> > *
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_uint32,  # eAnimationIntensity
            ctypes.c_uint32,  # eAnimationIntensity
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cGcNPCManager::GetRandomHologramProp": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCManager *
            ctypes.c_uint32,  # eTechnologyCategory
        ]
    ),
    "cGcNPCManager::GetNPCWordReaction": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCManager *
            ctypes.c_ulonglong,  # TkID<256> *
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_uint32,  # egcwordcategorytableEnum
        ]
    ),
    "cGcNPCManager::GetTerminalWordReaction": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCManager *
            ctypes.c_ulonglong,  # TkID<256> *
            ctypes.c_uint32,  # eAlienRace
            ctypes.c_uint32,  # egcwordcategorytableEnum
        ]
    ),
    "cGcNavGraphNode::GetConnectedNavArea": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavGraphNode *
        ]
    ),
    "cGcNavGraphNode::AddLink": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNavGraphNode::NodeLink *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavGraphNode *
            ctypes.c_ulonglong,  # cGcNavGraphNode *
        ]
    ),
    "cGcNavGraphNode::InvalidateLink": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavGraphNode *
            ctypes.c_ulonglong,  # const cGcNavGraphNodeHandle *
        ]
    ),
    "cGcNPCNavGraph::cGcNPCNavGraph": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavGraph *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCNavGraph::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavGraph *
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
            ctypes.c_ulonglong,  # const std::_Value_init_tag *
        ]
    ),
    "cGcNPCNavGraph::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavGraph *
        ]
    ),
    "cGcNPCNavGraph::FreeNavNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavGraph *
            ctypes.c_ulonglong,  # const cGcNavGraphNodeHandle *
        ]
    ),
    "cGcNPCNavGraph::Expand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavGraph *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const std::_Value_init_tag *
        ]
    ),
    "cGcNPCNavGraph::IsValidHandle": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavGraph *
            ctypes.c_ulonglong,  # const cGcNavGraphNodeHandle *
        ]
    ),
    "cGcNPCNavGraph::IsValidLink": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavGraph *
            ctypes.c_ulonglong,  # const cGcNavGraphLinkHandle *
        ]
    ),
    "cGcNPCNavGraph::GetNavNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNavGraphNode *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavGraph *
            ctypes.c_ulonglong,  # const cGcNavGraphNodeHandle *
        ]
    ),
    "cGcNPCNavGraph::RebuildSpatialPartitioning": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavGraph *
        ]
    ),
    "cGcNPCNavGraph::AddNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNavGraphNodeHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavGraph *
            ctypes.c_ulonglong,  # cGcNavGraphNodeHandle *
            ctypes.c_ulonglong,  # cGcNavGraphNode *
        ]
    ),
    "cGcNPCNavGraph::RemoveNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavGraph *
            ctypes.c_ulonglong,  # const cGcNavGraphNodeHandle *
        ]
    ),
    "cGcNPCNavGraph::InvalidateLinks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavGraph *
            ctypes.c_ulonglong,  # const cGcNavGraphNodeHandle *
        ]
    ),
    "cGcNPCNavGraph::FindNearestNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNavGraphNodeHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavGraph *
            ctypes.c_ulonglong,  # cGcNavGraphNodeHandle *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "StackContainer<std::vector<cGcNavGraphQuadTree::sNavNodeData,StackAllocator<cGcNavGraphQuadTree::sNavNodeData,4,20>>,4,20>::~StackContainer<std::vector<cGcNavGraphQuadTree::sNavNodeData,StackAllocator<cGcNavGraphQuadTree::sNavNodeData,4,20>>,4,20>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cGcNavGraphQuadTree::sNavNodeData,StackAllocator<cGcNavGraphQuadTree::sNavNodeData,4,20> >,4,20> *
        ]
    ),
    "cGcNavGraphQuadTree::TreeNode::Insert": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavGraphQuadTree::TreeNode *
            ctypes.c_ulonglong,  # const cGcNavGraphQuadTree::sNavNodeData *
        ]
    ),
    "cGcNavGraphQuadTree::TreeNode::Remove": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavGraphQuadTree::TreeNode *
            ctypes.c_ulonglong,  # const cGcNavGraphQuadTree::sNavNodeData *
        ]
    ),
    "cGcNavGraphQuadTree::TreeNode::Subdivide": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavGraphQuadTree::TreeNode *
        ]
    ),
    "cGcNavGraphQuadTree::TreeNode::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavGraphQuadTree::TreeNode **
        ]
    ),
    "cGcNavGraphQuadTree::TreeNode::FindNearestNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavGraphQuadTree::TreeNode *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cGcNavGraphNodeHandle *
            ctypes.c_ulonglong,  # cGcNavGraphQuadTree::TreeNode::NearestNodeSearchData *
        ]
    ),
    "cGcNavGraphQuadTree::TreeNode::FindNodesInRange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavGraphQuadTree::TreeNode *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # std::vector<cGcNavGraphNodeHandle,TkSTLAllocatorShim<cGcNavGraphNodeHandle,2,-1> > *
        ]
    ),
    "cGcNavGraphQuadTree::AABB::ContainsPoint": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavGraphQuadTree::AABB *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcNavGraphQuadTree::AABB::DistanceToPointSq": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavGraphQuadTree::AABB *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcNavGraphQuadTree::AddNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavGraphQuadTree *
            ctypes.c_ulonglong,  # const cGcNavGraphNode *
        ]
    ),
    "cGcNavGraphQuadTree::ExpandToFit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNavGraphQuadTree *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcNPCNavGraphPhysProbeJob::ProbePhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavGraphPhysProbeJob *
        ]
    ),
    "cGcOutpostComponent::GetDockIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcNPCNavigationAreaComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
        ]
    ),
    "cGcNPCNavigationAreaComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
        ]
    ),
    "cGcNPCNavigationAreaComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
        ]
    ),
    "cGcNPCNavigationAreaComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcNPCNavigationAreaComponent::TerminateAllNavigation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
        ]
    ),
    "cGcNPCNavigationAreaComponent::OnExternalNavGraphConstructionComplete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
        ]
    ),
    "cGcNPCNavigationAreaComponent::GatherNodes": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
        ]
    ),
    "cGcNPCNavigationAreaComponent::CalculateNodeFloorPositions": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
        ]
    ),
    "cGcNPCNavigationAreaComponent::HasLinkBeenChecked": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
            ctypes.c_ulonglong,  # const cGcNavGraphLinkHandle *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcNPCNavigationAreaComponent::ValidateLinks": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
        ]
    ),
    "cGcNPCNavigationAreaComponent::SimplifyGraph": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
        ]
    ),
    "cGcNPCNavigationAreaComponent::GenerateNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
            ctypes.c_ulonglong,  # const cGcNavGraphNodeHandle *
        ]
    ),
    "cGcNPCNavigationAreaComponent::GetNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNavGraphNode *
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
            ctypes.c_ulonglong,  # const cGcNavGraphNodeHandle *
        ]
    ),
    "cGcNPCNavigationAreaComponent::GeneratePath": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
            ctypes.c_ulonglong,  # const cGcNavGraphNodeHandle *
            ctypes.c_ulonglong,  # const cGcNavGraphNodeHandle *
            ctypes.c_ulonglong,  # cGcNPCNavGraph::NavGraphPath *
        ]
    ),
    "cGcNPCNavigationAreaComponent::ReservePOI": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
            ctypes.c_ulonglong,  # const cGcNavGraphNode *
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcNPCNavigationAreaComponent::VacatePOI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
            ctypes.c_ulonglong,  # const cGcNavGraphNode *
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcNPCNavigationAreaComponent::UpdateRecurrentPhysicsValidation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent *
        ]
    ),
    "cGcNPCNavigationAreaComponent::GetChairComponentFromNavNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcChairComponent *
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNavGraphNode *
        ]
    ),
    "cGcRegionTerrainMid::cGcRegionTerrainMid": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
        ]
    ),
    "cGcRegionTerrainNear::PollToUnmapStreams": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainNear *
        ]
    ),
    "cGcRegionTerrainNear::Invalidate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainNear *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcRegionTerrainNear::GetQuadResource": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainNear *
        ]
    ),
    "cGcRegionTerrainNear::GetEditsLineResource": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainNear *
        ]
    ),
    "cGcRegionTerrainNear::GetLineResource": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainNear *
        ]
    ),
    "cGcRegionTerrainNear::CancelPolygonise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainNear *
        ]
    ),
    "cGcRegionTerrainNear::GetNodeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainNear *
        ]
    ),
    "cGcRegionTerrainNear::CanPolygonise": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainNear *
        ]
    ),
    "cGcRegionTerrainNear::PostPolygoniseThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainNear *
        ]
    ),
    "cGcRegionTerrainNear::~cGcRegionTerrainNear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainNear *
        ]
    ),
    "cGcRegionTerrainMid::~cGcRegionTerrainMid": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cGcRegionDecoratorCreatures::cGcCreatureNodeData,8,20> *
        ]
    ),
    "cGcPlanet::GetRegionRadiusForSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # ePlanetSize
        ]
    ),
    "cGcPlanet::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlanet::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanet::PreloadCommonResources": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cGcPlanet::Generate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # const cGcPlanetGenerationInputData *
        ]
    ),
    "cGcPlanet::UpdateTextures": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanet::PostPrepareTasks": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanet::UpdatePhase1": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlanet::UpdatePhase2": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "cGcPlanet::UpdateClouds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlanet::UpdateTargetGravityMultiplier": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanet::UpdateGravity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlanet::UpdateWeather": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlanet::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanet::SetSentinelCrimeResponse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_uint32,  # eSentinelCrimeResponse
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlanet::FulfillResourceLoadingRequests": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanet::SetPlanetUniforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanet::SetUniform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ulonglong,  # const std::pair<char const *,unsigned __int64> *
            ctypes.c_ubyte,  # bool
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanet::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcPlanet::ReleaseSpawnData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanet::ReleaseCreaturesSpawnData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanet::RequestLoadSpawnData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanet::RequestLoadCreaturesSpawnData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanet::GetStormProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcStormProperties *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanet::GenerateResourceString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const cGcPlanetData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanet::AppendResourceHintList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ulonglong,  # const cGcPlanetData *
            ctypes.c_uint32,  # ePlanetResourceHintAlignment
            ctypes.c_uint32,  # ePlanetResourceHintColouring
        ]
    ),
    "cGcPlanet::GetParent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcPlanet *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanet::GetVoxelColour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_uint32,  # eVoxelType
            ctypes.c_double,  # double
        ]
    ),
    "cGcPlanet::HasExtremeHazard": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanet::CreateResources": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanet::GetTimeOfDayAtPoint": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcPlanet::GetLightShaftIntensityAtPoint": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcPlanet::GetHeightResult": FUNCDEF(
        restype=ctypes.c_uint8,  # unsigned __int8
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkRegionHeightResult *
            ctypes.c_uint32,  # eHeightTestAccuracy
        ]
    ),
    "cGcPlanet::RegionHeightAtPoint": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkRegionHeightResult *
        ]
    ),
    "cGcPlanet::SetupRegionMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanet::SetupWaterPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanet::GetUA": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanetControls::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetControls *
        ]
    ),
    "cGcPlanetControls::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetControls *
        ]
    ),
    "cGcPlanetControls::FulfillGenerationRequests": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetControls *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcNPCPropTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCPropTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcNPCColourTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCColourTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcNPCWordReactionTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCWordReactionTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcNPCInteractionsDataTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCInteractionsDataTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcNPCAnimationsData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCAnimationsData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcNPCReactionData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCReactionData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "robin_hood__detail__Table_1_80_cGcNavGraphNodeHandle__cGcNPCNavigationAreaComponent__GeneratePath____2___sAStarData_robin_hood__hash_cGcNavGraphNodeHandle_void__std__equal_to_cGcNavGraphNodeHandle___1___operator____cGcNPCNavigationAreaComponent__GeneratePath____2___sAStarData_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponent::GeneratePath::__l2::sAStarData *
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNavGraphNodeHandle *
            ctypes.c_ulonglong,  # __int16 *
        ]
    ),
    "cGcTerrainRegionMap::TransferArray<cGcRegionTerrainNear>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::TransferArray<cGcRegionTerrainMid>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::TransferArray<cGcRegionTerrainFar>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::TransferArray<cGcRegionTerrainVeryFar>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::TransferArray<cGcRegionPlanet>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::AllocateArray<cGcRegionTerrainNear>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::AllocateArray<cGcRegionTerrainMid>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::AllocateArray<cGcRegionTerrainFar>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::AllocateArray<cGcRegionTerrainVeryFar>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::AllocateArray<cGcRegionPlanet>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTerrainRegionMap::DeallocateArray<cGcRegionPlanet>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcTerrainRegionMap::DeallocateArray<cGcRegionTerrainVeryFar>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcTerrainRegionMap::DeallocateArray<cGcRegionTerrainFar>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcTerrainRegionMap::DeallocateArray<cGcRegionTerrainMid>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcTerrainRegionMap::DeallocateArray<cGcRegionTerrainNear>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainRegionMap *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "robin_hood__detail__Table_1_80_cGcNavGraphNodeHandle__cGcNPCNavigationAreaComponent__GeneratePath____2___sAStarData_robin_hood__hash_cGcNavGraphNodeHandle_void__std__equal_to_cGcNavGraphNodeHandle___1___throwOverflowError": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "robin_hood__detail__Table_1_80_cGcNavGraphNodeHandle__cGcNPCNavigationAreaComponent__GeneratePath____2___sAStarData_robin_hood__hash_cGcNavGraphNodeHandle_void__std__equal_to_cGcNavGraphNodeHandle___1___insertKeyPrepareEmptySpot_cGcNavGraphNodeHandle_const___": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcNavGraphNodeHandle *
        argtypes=[
            ctypes.c_ulonglong,  # std::pair<unsigned __int64,enum robin_hood::detail::Table<1,80,cGcNavGraphNodeHandle,`cGcNPCNavigationAreaComponent::GeneratePath'::`2'::sAStarData,robin_hood::hash<cGcNavGraphNodeHandle,void>,std::equal_to<cGcNavGraphNodeHandle>,-1>::InsertionState> *
            ctypes.c_ulonglong,  # const cGcNavGraphNodeHandle *
            ctypes.c_ulonglong,  # __int16 *
        ]
    ),
    "robin_hood__detail__Table_1_80_cGcNavGraphNodeHandle__cGcNPCNavigationAreaComponent__GeneratePath____2___sAStarData_robin_hood__hash_cGcNavGraphNodeHandle_void__std__equal_to_cGcNavGraphNodeHandle___1___try_increase_info": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_int64,  # __int64
        ]
    ),
    "robin_hood__detail__Table_1_80_cGcNavGraphNodeHandle__cGcNPCNavigationAreaComponent__GeneratePath____2___sAStarData_robin_hood__hash_cGcNavGraphNodeHandle_void__std__equal_to_cGcNavGraphNodeHandle___1___initData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "robin_hood__detail__Table_1_80_cGcNavGraphNodeHandle__cGcNPCNavigationAreaComponent__GeneratePath____2___sAStarData_robin_hood__hash_cGcNavGraphNodeHandle_void__std__equal_to_cGcNavGraphNodeHandle___1___rehashPowerOfTwo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionTerrainMid::GetNodeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
        ]
    ),
    "cGcRegionTerrainMid::GetCreatures": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcRegionDecoratorCreatures *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
        ]
    ),
    "cGcRegionTerrainMid::GetGroundPhysics": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRigidBody *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainMid *
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcExternalObjectList>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcExternalObjectList *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcExternalObjectList>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcExternalObjectList *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcExternalObjectList> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCollision::AddSphereShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_uint32,  # eVoxelType
        ]
    ),
    "cGcSolarSystemGenerator::GetSpaceSkyColours": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSolarSystemSkyColourData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemGenerator *
            ctypes.c_ulonglong,  # const cGcPlanetWeatherColourIndex *
        ]
    ),
    "cGcPlanetGenerator::GetWaterColours": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlanetWaterColourData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlanetGenerator::GetTileTypeSet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTileTypeSet *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # const cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::GetOverlayTexture": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcOverlayTexture *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # const cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::GetTerrainTexture": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTerrainTexture *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # const cGcPlanetData *
        ]
    ),
    "cGcRegionTerrain::GetNodeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionTerrain::cGcRegionTerrain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionTerrainVeryFar::GetNodeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainVeryFar *
        ]
    ),
    "cGcRegionTerrainVeryFar::cGcRegionTerrainVeryFar": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainVeryFar *
        ]
    ),
    "cGcRegionTerrain::~cGcRegionTerrain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionTerrainVeryFar::~cGcRegionTerrainVeryFar": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainVeryFar *
        ]
    ),
    "cGcRegionTerrain::GetStatusColour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # cTkColour *
        ]
    ),
    "cGcRegionTerrain::CanPolygonise": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
        ]
    ),
    "cGcRegionTerrain::GetTileBlendTriangleStart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrain *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcRegionTerrainFar::GetBuildings": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcRegionDecoratorBuildings *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainFar *
        ]
    ),
    "cGcRegionTerrainFar::GetNodeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainFar *
        ]
    ),
    "StackContainer<std::vector<cGcRegionDecoratorBuildings::sAddBuildingParams,StackAllocator<cGcRegionDecoratorBuildings::sAddBuildingParams,16,20>>,16,20>::~StackContainer<std::vector<cGcRegionDecoratorBuildings::sAddBuildingParams,StackAllocator<cGcRegionDecoratorBuildings::sAddBuildingParams,16,20>>,16,20>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cGcRegionDecoratorBuildings::sAddBuildingParams,StackAllocator<cGcRegionDecoratorBuildings::sAddBuildingParams,16,20> >,16,20> *
        ]
    ),
    "cGcRegionDecoratorBuildings::~cGcRegionDecoratorBuildings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionDecoratorBuildings *
        ]
    ),
    "StackContainer<std::vector<cGcRegionKnowledge *,StackAllocator<cGcRegionKnowledge *,37,-1>>,37,-1>::~StackContainer<std::vector<cGcRegionKnowledge *,StackAllocator<cGcRegionKnowledge *,37,-1>>,37,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cGcRegionKnowledge *,StackAllocator<cGcRegionKnowledge *,37,-1> >,37,-1> *
        ]
    ),
    "cGcRegionTerrainFar::cGcRegionTerrainFar": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionTerrainFar *
        ]
    ),
    "cTkHeirarchicalRNG::Push": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHeirarchicalRNG *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcPlanetGenerator::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
        ]
    ),
    "cGcPlanetGenerator::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetGenerator::GenerateQueryInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # PlanetGenerationQuery *
            ctypes.c_ulonglong,  # const cGcPlanetGenerationInputData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetGenerator::GenerateWonderQueryData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetGenerationWonderQuery *
            ctypes.c_ulonglong,  # const cGcPlanetGenerationInputData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetGenerator::GenerateWeatherInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::Generate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # const cGcPlanetGenerationInputData *
            ctypes.c_ulonglong,  # cGcPlanet *
        ]
    ),
    "cGcPlanetGenerator::ReseedTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_uint32,  # eGenerationSeed
        ]
    ),
    "cGcPlanetGenerator::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
        ]
    ),
    "cGcPlanetGenerator::GenerateInhabitants": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::GenerateSkyColourPalettes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::GenerateColourPalettes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::GenerateSky": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::GenerateScreenFilter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::GenerateClouds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::GenerateHeavyAir": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # cGcPlanetHeavyAirData *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # ePalette
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlanetGenerator::GetVoxelType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_uint32,  # eNoiseVoxelType
            ctypes.c_ulonglong,  # cGcTerrainControls *
        ]
    ),
    "cGcPlanetGenerator::RandomiseNoiseData": {
        "cGcPlanetGenerator *, cTkNoiseUberLayerData *, cTkNoiseUberLayerData *, cTkNoiseUberLayerData *, cGcTerrainControls *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkNoiseUberLayerData *
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlanetGenerator *
                ctypes.c_ulonglong,  # cTkNoiseUberLayerData *
                ctypes.c_ulonglong,  # cTkNoiseUberLayerData *
                ctypes.c_ulonglong,  # cTkNoiseUberLayerData *
                ctypes.c_ulonglong,  # cGcTerrainControls *
            ]
        ),
        "cGcPlanetGenerator *, cTkNoiseGridData *, cTkNoiseGridData *, cTkNoiseGridData *, cGcTerrainControls *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkNoiseGridData *
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlanetGenerator *
                ctypes.c_ulonglong,  # cTkNoiseGridData *
                ctypes.c_ulonglong,  # cTkNoiseGridData *
                ctypes.c_ulonglong,  # cTkNoiseGridData *
                ctypes.c_ulonglong,  # cGcTerrainControls *
            ]
        ),
        "cGcPlanetGenerator *, cTkNoiseFeatureData *, cTkNoiseFeatureData *, cTkNoiseFeatureData *, cGcTerrainControls *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkNoiseFeatureData *
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlanetGenerator *
                ctypes.c_ulonglong,  # cTkNoiseFeatureData *
                ctypes.c_ulonglong,  # cTkNoiseFeatureData *
                ctypes.c_ulonglong,  # cTkNoiseFeatureData *
                ctypes.c_ulonglong,  # cGcTerrainControls *
            ]
        ),
    },
    "cGcPlanetGenerator::GenerateTerrain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::GenerateTerrainSetting": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::RandomiseWaterHeight": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
        ]
    ),
    "cGcPlanetGenerator::GenerateBasics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetGenerator::GenerateHazards": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::GenerateWeather": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetGenerator::GenerateSubstances": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetGenerator::GenerateExtraResourceHints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::GetLocString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcPlanetGenerator::GenerateInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::GenerateSentinels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::GenerateRings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::GenerateCreatureInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # cGcCreatureRoleData *
        ]
    ),
    "cGcPlanetGenerator::GenerateTerrainTileSet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::GenerateTerrainColours": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::GenerateWater": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetGenerator::GenerateExternalObjectLists": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetGenerator::IsOptionalObjectListActive": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetGenerator::ChooseExternalLists": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExternalObjectListOptions> *
            ctypes.c_ulonglong,  # std::vector<cGcExternalObjectListOptions,TkSTLAllocatorShim<cGcExternalObjectListOptions,8,-1> > *
            ctypes.c_ulonglong,  # std::vector<int,TkSTLAllocatorShim<int,4,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetGenerator::TakeAllValidExternalLists": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExternalObjectListOptions> *
            ctypes.c_ulonglong,  # std::vector<cGcExternalObjectListOptions,TkSTLAllocatorShim<cGcExternalObjectListOptions,8,-1> > *
            ctypes.c_ulonglong,  # std::vector<int,TkSTLAllocatorShim<int,4,-1> > *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cGcPlanetGenerator::IsObjectListDisabled": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExternalObjectListOptions> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlanetGenerator::LoadQualityVariantForObjectList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcExternalObjectList *
        ]
    ),
    "cGcPlanetGenerator::LoadExternalObjectLists": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # std::vector<cGcExternalObjectList *,TkSTLAllocatorShim<cGcExternalObjectList *,8,-1> > *
            ctypes.c_ulonglong,  # std::vector<enum eTileType,TkSTLAllocatorShim<enum eTileType,4,-1> > *
            ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
        ]
    ),
    "IsObjectBPriorityLessThanObjectAPriority": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetObjectSpawnData *
            ctypes.c_ulonglong,  # const cGcPlanetObjectSpawnData *
        ]
    ),
    "cGcPlanetGenerator::GenerateSpawnData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # std::vector<cGcPlanetObjectSpawnData,TkSTLAllocatorShim<cGcPlanetObjectSpawnData,8,-1> > *
            ctypes.c_ulonglong,  # std::vector<cGcPlanetObjectSpawnData,TkSTLAllocatorShim<cGcPlanetObjectSpawnData,8,-1> > *
            ctypes.c_ulonglong,  # std::vector<cGcPlanetObjectSpawnData,TkSTLAllocatorShim<cGcPlanetObjectSpawnData,8,-1> > *
            ctypes.c_ulonglong,  # std::vector<cGcPlanetObjectSpawnData,TkSTLAllocatorShim<cGcPlanetObjectSpawnData,8,-1> > *
        ]
    ),
    "cGcPlanetGenerator::FillObjectArray": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcObjectSpawnData> *
            ctypes.c_ulonglong,  # std::vector<cGcPlanetObjectSpawnData,TkSTLAllocatorShim<cGcPlanetObjectSpawnData,8,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eTileType
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetGenerator::GenerateSelectableObjectData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::LoadSpawnData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::LoadObjectArray": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # std::vector<cGcPlanetObjectSpawnData,TkSTLAllocatorShim<cGcPlanetObjectSpawnData,8,-1> > *
        ]
    ),
    "cGcPlanetGenerator::GetCreatureResourceHint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # TkID<256> *
            ctypes.c_uint32,  # eBiome
            ctypes.c_uint32,  # eBiomeSubType
        ]
    ),
    "cGcPlanetGenerator::GetCreatureResourceFilter": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # TkID<256> *
            ctypes.c_uint32,  # eCreatureRole
        ]
    ),
    "cGcPlanetGenerator::LoadCreatureArray": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureSpawnData> *
        ]
    ),
    "cGcPlanetGenerator::ShouldLoadFemaleCreature": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # const cGcCreatureSpawnData *
        ]
    ),
    "cGcPlanetGenerator::LoadModel": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eTileType
            ctypes.c_uint32,  # eGroundColourIndex
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_ulonglong,  # cGcResourceElement *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cGcPlanetGenerator::GenerateCreatureRoles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcPlanetGenerator::GenerateCreatureSpawnData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::FillCreatureSpawnDataFromDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # const cGcCreatureRoleData *
            ctypes.c_ulonglong,  # cGcCreatureSpawnData *
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerator::UpdateProceduralTextureColours": {
        "cGcPlanetGenerator *, std::vector<cGcPlanetObjectSpawnData,TkSTLAllocatorShim<cGcPlanetObjectSpawnData,8,-1> > *, cGcPlanetData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlanetGenerator *
                ctypes.c_ulonglong,  # std::vector<cGcPlanetObjectSpawnData,TkSTLAllocatorShim<cGcPlanetObjectSpawnData,8,-1> > *
                ctypes.c_ulonglong,  # cGcPlanetData *
            ]
        ),
        "cGcPlanetGenerator *, cTkDynamicArray<cGcCreatureSpawnData> *, cGcPlanetData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlanetGenerator *
                ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureSpawnData> *
                ctypes.c_ulonglong,  # cGcPlanetData *
            ]
        ),
    },
    "cGcPlanetGenerator::SetupCustomObjectColourPalettes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cGcPlanetColourData *
        ]
    ),
    "cGcPlanetGenerator::GetSubBiomeForDiscoveryData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_uint32,  # eBiome
        ]
    ),
    "cGcPlanetGenerator::GetSystemFlagsForDiscoveryData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcPlanetGenerator::GetPlanetInfoFromDiscoveryData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetInfo *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcPlanetGenerator::GetCreatureResourceFilterFromDiscoveryData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<256> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # TkID<256> *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcPlanetGenerator::GetCreatureInfoFromDiscoveryData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcCreatureInfo *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcPlanetGenerator::GetCreatureInfoAndSpawnDataFromDiscoveryData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcCreatureInfo *
            ctypes.c_ulonglong,  # cGcCreatureSpawnData *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcPlanetGenerator::DoesCreatureStillExistOnPlanet": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcPlanetGenerator::GenerateCreatureRolesForPlanet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcPlanetGenerator::GenerateSubstancesForPlanet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerator *
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # const cGcDiscoveryData *
        ]
    ),
    "cGcPlanetLodSphere::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetLodSphere *
        ]
    ),
    "cGcPlanetLodSphere::Build": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetLodSphere::sBuildParams *
        ]
    ),
    "cGcPlanetLodSphere::BuildJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcPlanetLodSphere::GenerateSphere_Begin": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetLodSphere *
        ]
    ),
    "cGcPlanetLodSphere::GenerateSphere_End": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetLodSphere *
        ]
    ),
    "cGcPlanetLodSphere::IsFullyGenerated": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetLodSphere *
        ]
    ),
    "cGcPlanetMappingManager::GetMappingDataFromUAWritable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # MappingDataInternal *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetMappingManager *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "cGcPlanetMappingManager::GetSectionIndexesFromWorldPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::array<int,3> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetMappingManager *
            ctypes.c_ulonglong,  # std::array<int,3> *
            ctypes.c_ulonglong,  # cGcPlanet *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcPlanetMappingManager::GetCurrentCubeIndexes": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::array<unsigned int,3> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetMappingManager *
            ctypes.c_ulonglong,  # std::array<unsigned int,3> *
        ]
    ),
    "cTkMetadataCache<cGcCreatureRoleDescriptionTable>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureRoleDescriptionTable *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcCreatureRoleDescriptionTable> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcWeatherEffectTable>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWeatherEffectTable *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcWeatherEffectTable> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcWeatherEffectTable>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cGcWeatherEffectTable> *
        argtypes=[]
    ),
    "cTkMetadataCache<cGcWaterColourSettingList>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWaterColourSettingList *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcWaterColourSettingList> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcWeatherColourSettings>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWeatherColourSettings *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcWeatherColourSettings> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcWeatherColourSettings>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cGcWeatherColourSettings> *
        argtypes=[]
    ),
    "cTkMetadataCache<cGcCreatureGenerationArchetypes>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureGenerationArchetypes *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcCreatureGenerationArchetypes> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcCreatureGenerationData>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureGenerationData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcCreatureGenerationData> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcSelectableObjectTable>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSelectableObjectTable *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcSelectableObjectTable> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcSpawnDensityList>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEntitlementRewardsTable *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcSpawnDensityList> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcWeatherProperties>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWeatherProperties *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcWeatherProperties> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcWeatherTable>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkTrailData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcWeatherTable> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcTerrainTexture>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTerrainTexture *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcTerrainTexture> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcTileTypeSets>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTileTypeSets *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcTileTypeSets> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcBiomeData>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBiomeData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcBiomeData> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcBiomeData>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,
        argtypes=[],
    ),
    "cTkMetadataCache<cGcBiomeListPerStarType>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBiomeListPerStarType *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcBiomeListPerStarType> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcBiomeListPerStarType>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cGcBiomeListPerStarType> *
        argtypes=[]
    ),
    "cTkMetadataCache<cGcBiomeFileList>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBiomeFileList *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcBiomeFileList> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcBiomeFileList>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cGcBiomeFileList> *
        argtypes=[]
    ),
    "cTkMetadataCache<cGcDeprecatedAssetsTable>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEntitlementRewardsTable *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cGcDeprecatedAssetsTable> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataCache<cGcDeprecatedAssetsTable>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cGcDeprecatedAssetsTable> *
        argtypes=[]
    ),
    "cTkBitArray<unsigned __int64,1,384>::GetNumTrue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,384> *
        ]
    ),
    "cTkDynamicArray<cGcSelectableObjectSpawnData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSelectableObjectSpawnData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPlanetSectionData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlanetSectionData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMetaData::ReadFromFile<cTkVoxelGeneratorSettingsArray>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVoxelGeneratorSettingsArray *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcCreatureRoleDescriptionTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureRoleDescriptionTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcWeatherEffectTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWeatherEffectTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcWaterColourSettingList>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWaterColourSettingList *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcWeatherColourSettings>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWeatherColourSettings *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcCreatureGenerationArchetypes>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureGenerationArchetypes *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcCreatureGenerationData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureGenerationData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcSelectableObjectTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSelectableObjectTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcWeatherProperties>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWeatherProperties *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcTerrainTexture>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTerrainTexture *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcTileTypeSets>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTileTypeSets *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcBiomeData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBiomeData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcBiomeListPerStarType>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBiomeListPerStarType *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcBiomeFileList>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBiomeFileList *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetObjectSpawnData::cGcPlanetObjectSpawnData": {
        "cGcPlanetObjectSpawnData *, cGcPlanetObjectSpawnData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlanetObjectSpawnData *
                ctypes.c_ulonglong,  # cGcPlanetObjectSpawnData *
            ]
        ),
        "cGcPlanetObjectSpawnData *, const cGcPlanetObjectSpawnData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcPlanetObjectSpawnData *
                ctypes.c_ulonglong,  # const cGcPlanetObjectSpawnData *
            ]
        ),
    },
    "cTkDynamicArray<cGcResourceElement>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcResourceElement> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTradeRouteSection::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeRouteSection *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTradeRouteSection::GetClosestPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeRouteSection *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcTradeRouteSection::CalculatePointByDistance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeRouteSection *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcTradeRoute::GetClosestSectionInDirection": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTradeRouteSection *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeRoute *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcTradeRoute::Generate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeRoute *
            common.GcSeed,  # cTkSeed
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTradeRouteSection::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeRouteSection *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcAvailableWeapons::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAvailableWeapons *
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcAvailableWeapons::ChooseBySeed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAvailableWeapons *
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcSolarSystemTour::RenderNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcOutpostComponent::AbortLanding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcSolarSystemTour::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTour *
        ]
    ),
    "cGcSolarSystemTour::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTour *
        ]
    ),
    "cGcSolarSystemTour::RestoreCamera": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTour *
        ]
    ),
    "cGcSolarSystemTour::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTour *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSolarSystemTour::ThreadedUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTour *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
        ]
    ),
    "cGcSolarSystemTour::ThreadSyncPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTour *
        ]
    ),
    "cGcSolarSystemTour::SwitchToFirstPersonMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTour *
        ]
    ),
    "cGcSolarSystemTour::SwitchToPlanet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTour *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSolarSystemTour::SwitchToAnimatedCamera": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTour *
        ]
    ),
    "cGcSolarSystemTour::SelectValidBuildingAnimation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCameraAmbientBuildingData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTour *
        ]
    ),
    "cGcSolarSystemTour::SelectValidSpaceAnimation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCameraAmbientSpaceData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTour *
        ]
    ),
    "cGcSolarSystemTour::SelectValidSpecialAnimation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCameraAmbientSpecialData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTour *
        ]
    ),
    "cGcSolarSystemTour::WaitForCameraAnimation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTour *
        ]
    ),
    "cGcSolarSystemTour::SwitchToCameraTourStage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTour *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSolarSystemTour::SwitchToNextCameraTourStage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTour *
        ]
    ),
    "cGcShipAIAttackBuilding::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackBuilding *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIAttackBuilding::ClampToHeightAboveGround": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackBuilding *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcFrigateAIBase::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateAIBase *
            ctypes.c_double,  # double
        ]
    ),
    "cGcFrigateAIBase::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateAIBase *
        ]
    ),
    "cGcFrigateAIBase::SetPositionOfInterest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateAIBase *
            ctypes.c_char * 0x10,  # cTkVector3
        ]
    ),
    "cGcFrigateAIBase::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateAIBase *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcShipAIFollowTradeRoutes::SetLandTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFollowTradeRoutes *
            ctypes.c_ulonglong,  # cGcOutpostComponent *
        ]
    ),
    "cGcShipAIFollowTradeRoutes::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFollowTradeRoutes *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIFollowTradeRoutes::AttachToClosestTradeRoute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFollowTradeRoutes *
        ]
    ),
    "cGcShipAILand::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAILand *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAILandNextToPlayer::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAILandNextToPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAILandNextToPlayer::PickSearchLocation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcShipAIDocked::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIDocked *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIDocked::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIDocked *
        ]
    ),
    "cGcShipAIDocked::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIDocked *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcShipAIDocked::PutPilotBackInShip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIDocked *
        ]
    ),
    "cGcShipAITakeOff::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAITakeOff *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAITakeOff::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAITakeOff *
        ]
    ),
    "cGcShipAIHasHailReply::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIHasHailReply *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIHasSpecificHailReply::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIHasSpecificHailReply *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcShipAIClearCommunication::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIClearCommunication *
        ]
    ),
    "cGcShipAISendFightPlayerMessage::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISendFightPlayerMessage *
        ]
    ),
    "cGcShipAIFlybyOverhead::StartFlyby": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyOverhead *
            ctypes.c_ulonglong,  # const cGcDoShipFlybyOverhead *
        ]
    ),
    "cGcShipAIFlybyOverhead::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyOverhead *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIFlybyClose::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyClose *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIFlybyClose::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyClose *
        ]
    ),
    "cGcShipAIFlybyClose::StartFlyby": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyClose *
            ctypes.c_ulonglong,  # const cGcDoShipFlybyClose *
        ]
    ),
    "cGcShipAIFlybyClose::UpdateLocked": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyClose *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIFlybyClose::UpdateAttack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyClose *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIFlybyIntercept::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyIntercept *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIStayClose::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIStayClose *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIStayClose::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIStayClose *
        ]
    ),
    "cGcShipAIEscort::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIEscort *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIEscort::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIEscort *
        ]
    ),
    "cGcShipAIEscort::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIEscort *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcShipAIFlybyMineAsteroids::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyMineAsteroids *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<std::pair<cGcShipAIFlybyMineAsteroids::sAsteroidData,float>,128,-1> *
        ]
    ),
    "cGcShipAIFlybyMineAsteroids::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyMineAsteroids *
        ]
    ),
    "cGcShipAIFlybyMineAsteroids::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyMineAsteroids *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcShipAIFlybyMineAsteroids::StartFlyby": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyMineAsteroids *
            ctypes.c_ulonglong,  # const cGcDoShipFlybyMineAsteroids *
        ]
    ),
    "cGcShipAIFlybyMineAsteroids::sAsteroidData::IsValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyMineAsteroids::sAsteroidData *
        ]
    ),
    "cGcShipAICheckShooter::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAICheckShooter *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIAlertData::NewTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAlertData *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcShipAIShooterData::NewShooter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIShooterData *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcShipAISetAlert::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISetAlert *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAICheckPlayerIsShooter::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAICheckPlayerIsShooter *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAISetPlayerShooter::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISetPlayerShooter *
        ]
    ),
    "cGcShipAICheckDeath::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAICheckDeath *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAICheckDeath::DropLoot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAICheckDeath *
        ]
    ),
    "cGcShipAICheckDeath::FinalDestruction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAICheckDeath *
        ]
    ),
    "cGcShipAICheckOrbit::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAICheckOrbit *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAITrackTarget::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAITrackTarget *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIAttack::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttack *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIAttackFreighter::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackFreighter *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIAttackFreighter::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackFreighter *
        ]
    ),
    "cGcShipAIAttackFreighter::GetTargetNormal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackFreighter *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcShipAIBehaviour::GetSpaceshipComponentData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent_vtbl *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIBehaviour *
        ]
    ),
    "cGcShipAIBehaviour::Travel": {
        "cGcShipAIBehaviour *, const cTkVector3 *, const cGcSpaceshipTravelData *, const cTkVector3 *, cGcShipAIAvoidanceTracking *, bool, cTkRigidBody *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcShipAIBehaviour *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cGcSpaceshipTravelData *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # cGcShipAIAvoidanceTracking *
                ctypes.c_ubyte,  # bool
                ctypes.c_ulonglong,  # cTkRigidBody *
            ]
        ),
        "cGcShipAIBehaviour *, const cTkVector3 *, const cGcSpaceshipTravelData *, cGcShipAIAvoidanceTracking *, bool, cTkRigidBody *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcShipAIBehaviour *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cGcSpaceshipTravelData *
                ctypes.c_ulonglong,  # cGcShipAIAvoidanceTracking *
                ctypes.c_ubyte,  # bool
                ctypes.c_ulonglong,  # cTkRigidBody *
            ]
        ),
    },
    "cGcShipAIBehaviour::CanTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIBehaviour *
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcShipAIBehaviour::NewTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIBehaviour *
            ctypes.c_ulonglong,  # cGcShipAIAlertData *
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcShipAIBehaviour::FollowLeader": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIBehaviour *
            ctypes.c_ulonglong,  # cGcSpaceshipTravelData *
            ctypes.c_double,  # double
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
        ]
    ),
    "cGcShipAIBehaviour::CountSquadTargets": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIBehaviour *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcShipAIEscape::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIEscape *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIWarpingIn::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIWarpingIn *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIWarpingOut::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIWarpingOut *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIFreighterTargeting::NotifySquad": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFreighterTargeting *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcShipAIFreighterTargeting::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFreighterTargeting *
        ]
    ),
    "cGcShipAIFreighterTargeting::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFreighterTargeting *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIFreighterTargeting::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFreighterTargeting *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcShipAIFreighterTargeting::UpdateTracking": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFreighterTargeting *
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # ePerception
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcAmbientModeCameras>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAmbientModeCameras *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCommunicator::HasSignalFromAttachment": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicator *
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cGcShipAIBehaviour::GetAxis": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIBehaviour *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcShipAISquadronData::SetFormationState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISquadronData *
            ctypes.c_uint32,  # cGcShipAISquadronData::eFormationState
        ]
    ),
    "cGcShipAIAlertData::cGcShipAIAlertData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAlertData *
        ]
    ),
    "cGcShipAIShooterData::cGcShipAIShooterData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIShooterData *
        ]
    ),
    "cEgGeometryResource::GetDeletionFrameDelay": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
        ]
    ),
    "cGcShipAICheckShooter::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAICheckShooter *
        ]
    ),
    "cGcShipAICheckPlayerIsShooter::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISetPlayerShooter *
        ]
    ),
    "cGcShipAISetPlayerShooter::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISetPlayerShooter *
        ]
    ),
    "cGcShipAISetAlert::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISetAlert *
        ]
    ),
    "cGcShipAISetAlert::cGcShipAISetAlert": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISetAlert *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcSpaceshipCommon *
        ]
    ),
    "cGcShipAISetAlert::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISetAlert *
        ]
    ),
    "cGcShipAICheckPlayerIsShooter::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAICheckPlayerIsShooter *
        ]
    ),
    "cGcShipAISquadronFormationAttack::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISquadronFormationAttack *
        ]
    ),
    "cGcShipAISquadronFormationAttack::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISquadronFormationAttack *
        ]
    ),
    "cGcShipAISquadronFormationAttack::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISquadronFormationAttack *
        ]
    ),
    "cGcShipAISquadronFormationAttack::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISquadronFormationAttack *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAILandingData::cGcShipAILandingData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAILandingData *
        ]
    ),
    "cGcShipAIFollowTradeRoutes::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFollowTradeRoutes *
        ]
    ),
    "cGcShipAIFollowTradeRoutes::cGcShipAIFollowTradeRoutes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFollowTradeRoutes *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcSpaceshipCommon *
        ]
    ),
    "cGcShipAIFollowTradeRoutes::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFollowTradeRoutes *
        ]
    ),
    "cGcShipAIDocked::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIDocked *
        ]
    ),
    "cGcShipAIDocked::cGcShipAIDocked": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIDocked *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcSpaceshipCommon *
        ]
    ),
    "cGcShipAIDocked::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIDocked *
        ]
    ),
    "cGcShipAIDocked::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAITakeOff *
        ]
    ),
    "cGcShipAITakeOff::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAITakeOff *
        ]
    ),
    "cGcShipAITakeOff::cGcShipAITakeOff": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAITakeOff *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcSpaceshipCommon *
        ]
    ),
    "cGcShipAITakeOff::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAITakeOff *
        ]
    ),
    "cGcShipAILand::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAILand *
        ]
    ),
    "cGcShipAILand::cGcShipAILand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAILand *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcSpaceshipCommon *
        ]
    ),
    "cGcShipAILand::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAILand *
        ]
    ),
    "cGcShipAILandNextToPlayer::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAILandNextToPlayer *
        ]
    ),
    "cGcShipAILandNextToPlayer::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAILandNextToPlayer *
        ]
    ),
    "cGcShipAILandNextToPlayer::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAILandNextToPlayer *
        ]
    ),
    "cGcShipAILandNextToPlayer::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAILandNextToPlayer *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcShipAIPlayerSquadronCheckFormation::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPlayerSquadronCheckFormation *
        ]
    ),
    "cGcShipAIPlayerSquadronCheckFormation::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPlayerSquadronCheckFormation *
        ]
    ),
    "cGcShipAIPlayerSquadronUpdateTarget::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPlayerSquadronUpdateTarget *
        ]
    ),
    "cGcShipAIPlayerSquadronUpdateTarget::cGcShipAIPlayerSquadronUpdateTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPlayerSquadronUpdateTarget *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcSpaceshipCommon *
        ]
    ),
    "cGcShipAIPlayerSquadronUpdateTarget::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPoliceChooseTarget *
        ]
    ),
    "cGcShipAIPlayerSquadronLeave::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPlayerSquadronLeave *
        ]
    ),
    "cGcShipAIPlayerSquadronLeave::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPlayerSquadronLeave *
        ]
    ),
    "cGcShipAISquadronFormationMove::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISquadronFormationMove *
        ]
    ),
    "cGcShipAISquadronFormationMove::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISquadronFormationMove *
        ]
    ),
    "cGcShipAISquadronFormationMove::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISquadronFormationMove *
            ctypes.c_double,  # double
        ]
    ),
    "cGcShipAISquadronFormationMove::MaintainFormation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISquadronFormationMove *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcSpaceshipTravelData *
        ]
    ),
    "cGcShipAISquadronFormationMove::UpdateJoiningFormation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISquadronFormationMove *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcSpaceshipTravelData *
        ]
    ),
    "cGcShipAISquadronFormationMove::UpdateBreakingFormation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISquadronFormationMove *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcSpaceshipTravelData *
        ]
    ),
    "cGcFrigateAIBase::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateAIBase *
        ]
    ),
    "cGcFlybyFrigateBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFlybyFrigateBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcShipAIAttackFreighter::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackFreighter *
        ]
    ),
    "cGcShipAIAttackFreighter::cGcShipAIAttackFreighter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackFreighter *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcSpaceshipCommon *
        ]
    ),
    "cGcShipAIAttackFreighter::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackFreighter *
        ]
    ),
    "cGcShipAICheckDeath::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAICheckDeath *
        ]
    ),
    "cGcCrashedShipTakeOff::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAICheckDeath *
        ]
    ),
    "cGcShipAIFlybyIntercept::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyIntercept *
        ]
    ),
    "cGcShipAIFlybyIntercept::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyIntercept *
        ]
    ),
    "cGcShipAIFlybyIntercept::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyIntercept *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcShipAIFlybyOverhead::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyOverhead *
        ]
    ),
    "cGcShipAIFlybyOverhead::cGcShipAIFlybyOverhead": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyOverhead *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcSpaceshipCommon *
        ]
    ),
    "cGcShipAIFlybyOverhead::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyOverhead *
        ]
    ),
    "cGcShipAIFlybyOverhead::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyOverhead *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcShipAIFlybyClose::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyClose *
        ]
    ),
    "cGcShipAIFlybyClose::cGcShipAIFlybyClose": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyClose *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcSpaceshipCommon *
        ]
    ),
    "cGcShipAIFlybyClose::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyClose *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcShipAIFlybyMineAsteroids::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFlybyMineAsteroids *
        ]
    ),
    "cGcShipAIStayClose::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIStayClose *
        ]
    ),
    "cGcShipAIStayClose::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIStayClose *
        ]
    ),
    "cGcShipAIEscort::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIEscort *
        ]
    ),
    "cGcShipAITrackTarget::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAITrackTarget *
        ]
    ),
    "cGcShipAITrackTarget::cGcShipAITrackTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAITrackTarget *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcSpaceshipCommon *
        ]
    ),
    "cGcShipAITrackTarget::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAITrackTarget *
        ]
    ),
    "cGcShipAITrackTarget::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAITrackTarget *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcShipAITrackTarget::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAITrackTarget *
        ]
    ),
    "cGcShipAIAttack::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttack *
        ]
    ),
    "cGcShipAIAttack::cGcShipAIAttack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttack *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcSpaceshipCommon *
        ]
    ),
    "cGcShipAIAttack::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttack *
        ]
    ),
    "cGcShipAIAttack::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttack *
        ]
    ),
    "cGcPlayerSquadronSpaceshipBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronSpaceshipBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcShipAIPlayerSquadronUpdateTarget::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPlayerSquadronUpdateTarget *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIPlayerSquadronCheckFormation::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPlayerSquadronCheckFormation *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIPlayerSquadronLeave::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPlayerSquadronLeave *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCrashedShipTakeOff::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCrashedShipTakeOff *
        ]
    ),
    "cGcSpaceCrashedShipBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceCrashedShipBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcCrashedShipTakeOff::cGcCrashedShipTakeOff": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCrashedShipTakeOff *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cGcSpaceshipCommon *
        ]
    ),
    "cGcCrashedShipTakeOff::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCrashedShipTakeOff *
            ctypes.c_float,  # float
        ]
    ),
    "cGcCrashedShipTakeOff::UpdateWingman": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCrashedShipTakeOff *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # cGcCrashedShipTakeOff::Data *
        ]
    ),
    "cGcShipAIDefenceForceChooseTarget::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIDefenceForceChooseTarget *
        ]
    ),
    "cGcShipAICheckOrbit::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAICheckOrbit *
        ]
    ),
    "cGcShipAIDefenceForceChooseTarget::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIDefenceForceChooseTarget *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceDefenceForceBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceDefenceForceBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcShipAIFreighterTargeting::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIFreighterTargeting *
        ]
    ),
    "cGcSpaceFreighterBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceFreighterBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcShipAIPirateChooseTarget::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPirateChooseTarget *
        ]
    ),
    "cGcShipAIPirateChooseTarget::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPirateChooseTarget *
        ]
    ),
    "cGcShipAIAttackBuilding::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackBuilding *
        ]
    ),
    "cGcShipAIAttackBuilding::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackBuilding *
        ]
    ),
    "cGcShipAIAttackBuilding::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackBuilding *
        ]
    ),
    "cGcSpacePirateBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpacePirateBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcShipAIPirateChooseTarget::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPirateChooseTarget *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIPoliceChooseTarget::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPoliceChooseTarget *
        ]
    ),
    "cGcSpacePoliceBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpacePoliceBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcShipAIPoliceChooseTarget::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPoliceChooseTarget *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipCommon::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipCommon *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcSpaceshipCommon::GetSpaceshipComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipCommon *
        ]
    ),
    "cGcSpaceshipCommon::CheckBeenShot": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachment *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipCommon *
        ]
    ),
    "cGcSpaceshipCommon::GetLeader": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipCommon *
        ]
    ),
    "cGcSpaceshipCommon::GetTravelData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSpaceshipTravelData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipCommon *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcSpaceshipBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcSpaceshipBehaviourTree::IsAttackingPlayer": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipBehaviourTree *
        ]
    ),
    "cGcSpaceshipBehaviourTree::GetAttackingAttachment": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachmentPtr *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachmentPtr *
        ]
    ),
    "cGcShipAIHailData::cGcShipAIHailData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIHailData *
        ]
    ),
    "cGcShipAIHasHailReply::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIHasHailReply *
        ]
    ),
    "cGcShipAIHasHailReply::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIHasHailReply *
        ]
    ),
    "cGcShipAIHasSpecificHailReply::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIHasSpecificHailReply *
        ]
    ),
    "cGcShipAISendFightPlayerMessage::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAISendFightPlayerMessage *
        ]
    ),
    "cGcShipAIClearCommunication::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIClearCommunication *
        ]
    ),
    "cGcShipAIClearShooter::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIClearShooter *
        ]
    ),
    "cGcShipAIClearShooter::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIClearShooter *
        ]
    ),
    "cGcSpaceTraderBehaviourTree::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceTraderBehaviourTree *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcAIShipSquad::HasActiveMembers": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSquad *
        ]
    ),
    "cGcAIShipSquad::CalcAveragePos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSquad *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcAIShipSquad::GetLeader": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSquad *
        ]
    ),
    "cGcShipFlareComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipFlareComponent *
        ]
    ),
    "cGcShipFlareComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipFlareComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcShipFlareComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcShipFlareComponent *
        ]
    ),
    "ComponentInstancercGcShipFlareComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcFreighterSyncComponent::GetNumComponents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cGcAISpaceshipComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAISpaceshipComponent::NetSerializeOnInit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cGcAISpaceshipComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcSpaceshipBehaviourTree::cGcSpaceshipBehaviourTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipBehaviourTree *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSpaceshipCommon::~cGcSpaceshipCommon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipCommon *
        ]
    ),
    "cGcAISpaceshipComponent::DockPlayerShips": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAISpaceshipComponent::IsPlayerShipDockedInFreighter": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAISpaceshipComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAISpaceshipComponent::UpdateVisibility": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAISpaceshipComponent::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAISpaceshipComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAISpaceshipComponent::~cGcAISpaceshipComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcAISpaceshipComponent::IsLeader": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAISpaceshipComponent::FireLasers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_uint32,  # eAIShipWeapon
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcAISpaceshipComponent::SetAttackDefinition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcAISpaceshipComponent::Attack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAISpaceshipComponent::FireProjectiles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAISpaceshipComponent::NotifyNetworkOfAttack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint32,  # eAIShipWeapon
        ]
    ),
    "cGcAISpaceshipComponent::GetFaction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAISpaceshipComponent::GetRole": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAISpaceshipComponent::IsFrigateFlyby": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAISpaceshipComponent::SetInactive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAISpaceshipComponent::GetInventoryStore": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryStore *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAISpaceshipComponent::GetTechOnlyInventoryStore": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryStore *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAISpaceshipComponent::GetCargoInventoryStore": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInventoryStore *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAISpaceshipComponent::GenerateInventory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipComponent::ActivateAlarm": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipComponent::GetCommander": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInteractionComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAISpaceshipComponent::RequestFadeIn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAISpaceshipComponent::RequestWarpIn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAISpaceshipComponent::RequestWarpOut": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipComponent::AmInterestedInOwnership": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAISpaceshipComponent::SetShipVisible": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipComponent::OverrideData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_uint32,  # eShipClass
            ctypes.c_uint32,  # eAIFaction
            ctypes.c_uint32,  # eAIShipRole
            ctypes.c_ulonglong,  # cTkModelResource *
        ]
    ),
    "cGcAISpaceshipComponent::OnOwnershipTransfer": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cGcAISpaceshipComponent::StartDocked": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_ulonglong,  # cGcOutpostComponent *
        ]
    ),
    "cGcAISpaceshipComponent::TakeDamage": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipComponent::TakeOffAndWarpOutCrashedShip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAISpaceshipComponent::SpawnPilotNPC": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_ulonglong,  # cGcOutpostComponent *
        ]
    ),
    "cGcAISpaceshipComponent::RequestNPCSpawn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAISpaceshipComponent::SwitchToComplexCollision": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipComponent::AddNavigationArea": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAISpaceshipComponent::RemoveNavigationArea": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAISpaceshipComponent::UpdateFreighterVisibility": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAISpaceshipComponent::OnHitCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcAISpaceshipComponent::OnHitEnergyShieldCallback": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcAISpaceshipComponent::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAISpaceshipComponent::GetLaserDamageLevel": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcAISpaceshipComponent::IsAbandoned": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cTkGravityComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkGravityComponent *
        ]
    ),
    "cTkGravityComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGravityComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercTkGravityComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercTkGravityComponent *
        ]
    ),
    "ComponentInstancercTkGravityComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcAISpaceshipManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipManager::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # const cGcSolarSystem *
        ]
    ),
    "cGcAISpaceshipManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAISpaceshipManager::ProcessSpawnEvent": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # cGcAIShipSpawnEvent *
        ]
    ),
    "cGcAISpaceshipManager::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
        ]
    ),
    "cGcAISpaceshipManager::RemoveSquadShipNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # cGcAIShipSquad *
        ]
    ),
    "cGcAISpaceshipManager::FindSquadByName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPoolHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # cTkClassPoolHandle *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcAISpaceshipManager::FindOrCreateSquad": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPoolHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # cTkClassPoolHandle *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcAISpaceshipManager::ReleaseSquad": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # const cTkClassPoolHandle *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipManager::GetPerception": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcAISpaceshipManager::LookupAttackData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcShipAICombatDefinition *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcAISpaceshipManager::LookupAttackTravelData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSpaceshipTravelData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcAISpaceshipManager::LookupShieldData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcAISpaceshipManager::LookupAttackBehaviourData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cGcAISpaceshipManager::SpawnShips": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # const cGcAIShipSpawnData *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eAIFaction
            ctypes.c_uint64,  # cTkClassPoolHandle
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcAISpaceshipManager::RemoveSpecificShipFromSquadManagement": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcAISpaceshipManager::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
        ]
    ),
    "cGcAISpaceshipManager::RemoveAllShipNodesFromAllSquads": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
        ]
    ),
    "cGcAISpaceshipManager::UpdateFreighterSpawns": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
        ]
    ),
    "cGcAISpaceshipManager::GetFrigateShipResource": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            ctypes.c_uint32,  # eFrigateClass
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cGcAISpaceshipManager::CheckPlayerBoost": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
        ]
    ),
    "cGcAISpaceshipManager::CacheHangar": {
        "cGcAISpaceshipManager *, cTkModelResource *, const cGcPlanetColourData *, const cTkSeed *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcAISpaceshipManager *
                ctypes.c_ulonglong,  # cTkModelResource *
                ctypes.c_ulonglong,  # const cGcPlanetColourData *
                ctypes.c_ulonglong,  # const cTkSeed *
            ]
        ),
        "cGcAISpaceshipManager *, cGcPlayerFreighterOwnership *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcAISpaceshipManager *
                ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ]
        ),
    },
    "cGcAISpaceshipManager::CacheShips": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcAISpaceshipPreloadCacheData> *
        ]
    ),
    "cGcAISpaceshipManager::GetAppropriateSpaceshipModelData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAISpaceshipModelData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # const cGcAISpaceshipPreloadCacheData *
        ]
    ),
    "cGcAISpaceshipManager::SpawnShip": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAIShipSpawn *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_uint32,  # eAIFaction
            ctypes.c_uint32,  # eAIShipRole
            ctypes.c_uint32,  # eFrigateClass
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            common.GcSeed,  # cTkSeed
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint64,  # cTkClassPoolHandle
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipManager::GetSpawnData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAIShipSpawn *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcAISpaceshipManager::DropLoot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cGcAISpaceshipManager::CacheShipForFrigate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_uint32,  # eAIFaction
            ctypes.c_uint32,  # eAIShipRole
            ctypes.c_uint32,  # eFrigateClass
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cGcPlanetColourData *
        ]
    ),
    "cGcAISpaceshipManager::CreateSpaceshipComponentData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManager *
            ctypes.c_ulonglong,  # cGcAISpaceshipComponentData *
            ctypes.c_uint32,  # eShipClass
            ctypes.c_uint32,  # eAIFaction
            ctypes.c_uint32,  # eAIShipRole
            ctypes.c_ulonglong,  # cTkModelResource *
        ]
    ),
    "cGcShipAIHail<4,2>::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIHail<4,2> *
        ]
    ),
    "cGcShipAIHail<4,2>::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIHail<4,2> *
        ]
    ),
    "cGcShipAIHail<4,2>::Subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIHail<4,2> *
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
        ]
    ),
    "cGcShipAIHail<4,2>::Run": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIHail<4,2> *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipAIHail<4,2>::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIHail<4,2> *
        ]
    ),
    "cTkBitArray<unsigned __int64,1,64>::GetNumTrue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitArray<unsigned __int64,1,64> *
        ]
    ),
    "cGcShipAIHail<4,2>::~cGcShipAIHail<4,2>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIHail<4,2> *
        ]
    ),
    "cGcShipAIHail<4,2>::GetCommunicatorMessage": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIHail<4,2> *
            ctypes.c_ulonglong,  # cGcPlayerCommunicatorMessage *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcShipAIAttackDataTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcShipAIAttackDataTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditor::DoComboBox<eAIFaction>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eAIFaction *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoComboBox<eAIShipRole>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # eAIShipRole *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAIShipSpawnEvent::cGcAIShipSpawnEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSpawnEvent *
            ctypes.c_ulonglong,  # const cGcAIShipSpawnEvent *
        ]
    ),
    "cGcCachedShipResource::cGcCachedShipResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCachedShipResource *
        ]
    ),
    "cGcAISpaceshipManager::RenderNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcAISpaceshipComponent::HasComponent": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcShipAIEscape::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIEscape *
        ]
    ),
    "cGcShipAIEscape::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIEscape *
        ]
    ),
    "cGcShipAIEscape::ShipAIConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIEscape *
        ]
    ),
    "cTkBehaviourTree::AddBranchSelector": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        ]
    ),
    "cGcShipAIWarpingOut::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIWarpingOut *
        ]
    ),
    "cGcShipAIWarpingIn::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIWarpingIn *
        ]
    ),
    "cGcShipAIBehaviour::GetName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIBehaviour *
        ]
    ),
    "cGcFreighterSyncComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterSyncComponent *
        ]
    ),
    "cGcFreighterSyncComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterSyncComponent *
        ]
    ),
    "cGcFreighterSyncComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterSyncComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcFreighterSyncComponent::NotifyNetworkOfAttack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterSyncComponent *
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterSyncComponent::NetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterSyncComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFreighterSyncComponent::IsAttacking": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterSyncComponent *
        ]
    ),
    "cGcEngineComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEngineComponent *
        ]
    ),
    "cGcEngineComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEngineComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercGcEngineComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercGcEngineComponent *
        ]
    ),
    "ComponentInstancercGcEngineComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ComponentInstancercGcEngineComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercGcEngineComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cGcEngineComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEngineComponent *
        ]
    ),
    "cGcEngineComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEngineComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcShipFlareComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipFlareComponent *
        ]
    ),
    "cGcShipFlareComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipFlareComponent *
        ]
    ),
    "cGcShipFlareComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipFlareComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcOutpostComponent::CalcDockingTarget": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponent *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkReplicatedVariable<cTkVector3> *
        ]
    ),
    "cGcShipOwnershipComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipOwnershipComponent *
        ]
    ),
    "cGcShipOwnershipComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipOwnershipComponent *
        ]
    ),
    "cGcShipOwnershipComponent::CreateShipComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipOwnershipComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipOwnershipComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipOwnershipComponent *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcShipOwnershipComponent::UpdateRemoteLanding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipOwnershipComponent *
        ]
    ),
    "cGcShipOwnershipComponent::NetSerializeOnInit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipOwnershipComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cGcShipOwnershipComponent::NetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipOwnershipComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcShipOwnershipComponent::Purchase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipOwnershipComponent *
        ]
    ),
    "cGcShipOwnershipComponent::Abandon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipOwnershipComponent *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ulonglong,  # cGcInventoryStore *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcShipOwnershipComponent::GetActiveDockingOutpost": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcOutpostComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipOwnershipComponent *
        ]
    ),
    "cGcSpaceCombatFollowMode::UpdateFollowTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceCombatFollowMode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceCombatFollowMode::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceCombatFollowMode *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceCombatFollowMode::GetTarget": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceCombatFollowMode *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSpaceCombatFollowMode::IsValidTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceCombatFollowMode *
            ctypes.c_ulonglong,  # cGcAISpaceshipComponent *
        ]
    ),
    "cGcPIDController::Update": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcPIDController *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::CheckForDocked": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_double,  # double
        ]
    ),
    "cGcSpaceshipComponent::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::TakeControl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipComponent::Eject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # cGcPlayer *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipComponent::SetPilotedCollisionFlags": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::CalculateFreighterApproachFactor": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::CheckCrashEffectActivationState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipComponent::UseThirdPersonCamera": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::IsWithinAtmosphericCombatRange": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::GetTakeOffCost": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::HasValidLandingSite": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::GetCockpitCameraLocatorPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcSpaceshipComponent::GetCockpitCameraLocatorRenderSpaceOffsetPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcSpaceshipComponent::GetCockpitPlayerLocatorMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcSpaceshipComponent::NetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSpaceshipComponent::GetVelocity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcSpaceshipComponent::GetWeaponRange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::SetupClassControlBonuses": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::UpdateControlled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipComponent::UpdateHeightSmoothing": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipComponent::UpdateForwardCollisionCasts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipComponent::UpdateCockpit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipComponent::CanBeDestroyedByShip": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcSpaceshipComponent::CalculateSafeLanding": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipComponent::TakeDamage": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcSpaceshipComponent::SetEngineIgnition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipComponent::UseTakeOffFuel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::IsDockingLandingOrTakingOff": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::IsLanding": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::IsOnGround": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::HasWorkingEngine": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::CanTakeOff": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::ReportCannotTakeOff": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::UpdateHover": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::GetDockOutpost": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcOutpostComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::IsWithinDistanceAndAngleRange": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # cGcDockingInfo *
            ctypes.c_ulonglong,  # cGcOutpostComponent *
        ]
    ),
    "cGcSpaceshipComponent::IsOutsideDistanceAndAngleRange": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # cGcDockingInfo *
            ctypes.c_ulonglong,  # cGcOutpostComponent *
        ]
    ),
    "cGcSpaceshipComponent::CanDock": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcOutpostComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cGcDockingInfo *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipComponent::UpdateLanding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_double,  # double
        ]
    ),
    "cGcSpaceshipComponent::OnLandingGroundManeuvreComplete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::SetLanding": {
        "cGcSpaceshipComponent *, cGcOutpostComponent *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcSpaceshipComponent *
                ctypes.c_ulonglong,  # cGcOutpostComponent *
            ]
        ),
        "cGcSpaceshipComponent *, const cTkPhysRelMat34 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcSpaceshipComponent *
                ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ]
        ),
    },
    "cGcSpaceshipComponent::SetTakeOff": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # cGcOutpostComponent *
        ]
    ),
    "cGcSpaceshipComponent::GetFoVSpring": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::GetPhysicsMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcSpaceshipComponent::GetRigidBody": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRigidBody *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcShipTargetDisplay::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipTargetDisplay *
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcShipTargetDisplay::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipTargetDisplay *
        ]
    ),
    "cGcSpaceshipComponent::BecomeActivelyFlying": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::ThrustActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::BrakeActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::UpdateJoysticks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipComponent::ShouldDelayTakeOff": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::IsHoldingControlStick": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::DisplayPulseDriveWarning": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::AllowedToMoveGrabbable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_uint32,  # eGrabType
        ]
    ),
    "cGcSpaceshipComponent::GetSalvageValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::GetVolumetricLightStrength": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::SetVolumetricLightStrength": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipComponent::Salvage": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipComponent::UpdateVisibility": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipComponent::DoThirdPersonWarpWander": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # float *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipComponent::UpdateVolumetricLights": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::UpdateFollowModeState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::IsInverted": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::IsLow": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
        ]
    ),
    "cGcSpaceshipComponent::TriggerTakeOffLandingVibration": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cTkComponent::RegisterReplicatedVariable<cTkPhysRelMat34>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
            ctypes.c_ulonglong,  # cTkReplicatedVariable<cTkPhysRelMat34> *
        ]
    ),
    "cGcShipTargetDisplay::RemoveShipRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipTargetDisplay *
        ]
    ),
    "cGcVehicleCockpit::SetGlassFrosted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipComponent::GetShipCentreMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponent *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cTkBehaviourTreeSelector::GetName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sAutoFollowControlParameters *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceCombatFollowMode *
        ]
    ),
    "cTkRigidBody::AlignPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_double,  # double
            ctypes.c_double,  # double
        ]
    ),
    "cGcPlayerState::ClearShipSlot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerState *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSpaceshipControl::cGcAvoidance::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipControl::cGcAvoidance *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cGcSpaceshipAvoidanceData *
        ]
    ),
    "cGcCreatureFullBodyIKComponent::SetFootStrength": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipComponent::OnHit": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcSpaceshipControl::cGcSpaceshipControl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipControl *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cGcSpaceshipControl::GetPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipControl *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcSpaceshipControl::GetVelocity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipControl *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcSpaceshipControl::GetAxis": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipControl *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcSpaceshipControl::Align": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cGcSpaceshipTravelData *
        ]
    ),
    "cGcSpaceshipControl::Move": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cGcSpaceshipTravelData *
        ]
    ),
    "cGcSpaceshipControl::Hover": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipControl::ApplyPlanetAvoidance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcSpaceshipControl::ApplyHeightForce": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipControl::Notify": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # cTkAttachmentPtr
            ctypes.c_uint32,  # eShipNotification
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipControl::NotifyModel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # eShipNotification
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipControl::IsNotificationActive": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAttachment *
        argtypes=[
            ctypes.c_uint64,  # cTkAttachmentPtr
            ctypes.c_uint32,  # eShipNotification
        ]
    ),
    "cGcSpaceshipEngineEffects::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipEngineEffects *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcSpaceshipEngineEffects::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipEngineEffects *
            ctypes.c_ulonglong,  # const cGcSpaceshipEngineEffects::Settings *
            ctypes.c_int64,  # __int64
            ctypes.c_double,  # double
        ]
    ),
    "cGcSpaceshipEngineEffects::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipEngineEffects *
        ]
    ),
    "cGcSpaceshipEngineEffects::UpdateTrailsResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipEngineEffects *
        ]
    ),
    "cGcSpaceshipEngineEffects::UpdateFreighterEngineEffectResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipEngineEffects *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStagedJetEngine::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStagedJetEngine *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcStagedJetEngine::AddJetStage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStagedJetEngine *
            ctypes.c_uint32,  # cGcStagedJetEngine::eJetStage
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcStagedJetEngine::UpdatePower": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStagedJetEngine *
            ctypes.c_float,  # float
        ]
    ),
    "cGcStagedJetEngine::UpdateStage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStagedJetEngine *
            ctypes.c_uint32,  # cGcStagedJetEngine::eJetStage
            ctypes.c_double,  # double
        ]
    ),
    "cGcStagedJetEngine::UpdateLight": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStagedJetEngine *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipControl::cGcAvoidance::UpdateAndApply": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipControl::cGcAvoidance *
        ]
    ),
    "cGcSpaceshipControl::cGcAvoidance::GetContactByFilter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipControl::cGcAvoidance *
            ctypes.c_ulonglong,  # cTkContactPoint *
            ctypes.c_ulonglong,  # float *
            ctypes.c_int16,  # __int16
        ]
    ),
    "cGcSpaceshipControl::cGcAvoidance::CalculateCentre": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipControl::cGcAvoidance *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcSpaceshipControl::cGcAvoidance::CalculateOffset": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipControl::cGcAvoidance *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSpaceshipEngineAudioController::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipEngineAudioController *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint64,  # TkAudioObject
            ctypes.c_uint32,  # eShipType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkAABB *
        ]
    ),
    "cGcSpaceshipEngineAudioController::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipEngineAudioController *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipEngineAudioController::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipEngineAudioController *
        ]
    ),
    "cGcSpaceshipShieldComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipShieldComponent *
        ]
    ),
    "cGcSpaceshipShieldComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipShieldComponent *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcSpaceshipShieldComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipShieldComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipShieldComponent::OnShieldBeginCharge": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipShieldComponent *
            ctypes.c_uint32,  # cGcSpaceshipShieldComponent::eShieldState
        ]
    ),
    "cGcSpaceshipShieldComponent::OnHit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipShieldComponent *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcSpaceshipShieldComponent::UpdateSketchFadeOpacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipShieldComponent *
        ]
    ),
    "cGcSpaceshipWarp::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
        ]
    ),
    "cGcSpaceshipWarp::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
            ctypes.c_uint32,  # cGcSpaceshipWarp::eWarpEffectType
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcSpaceshipWarp::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
        ]
    ),
    "cGcSpaceshipWarp::WarpToNextGalaxy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
        ]
    ),
    "cGcSpaceshipWarp::WarpToFreighter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
            ctypes.c_ulonglong,  # const cGcWarpJumpTarget *
            ctypes.c_ulonglong,  # cGcPlayerFreighterOwnership *
            ctypes.c_uint32,  # const cGcSpaceshipWarp::eWarpEffectType
        ]
    ),
    "cGcSpaceshipWarp::WarpToDistantSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
            ctypes.c_ulonglong,  # const cGcWarpJumpTarget *
            ctypes.c_uint32,  # cGcSpaceshipWarp::eWarpEffectType
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcSpaceshipWarp::WarpInFromDistantSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcSpaceshipWarp::WarpToNexus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
        ]
    ),
    "cGcSpaceshipWarp::InternalWarpOutSetup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
        ]
    ),
    "cGcSpaceshipWarp::InternalCommonWarpSetup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
        ]
    ),
    "cGcSpaceshipWarp::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
        ]
    ),
    "cGcSpaceshipWarp::CanPulseDrive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
        ]
    ),
    "cGcSpaceshipWarp::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
            ctypes.c_double,  # double
        ]
    ),
    "cGcSpaceshipWarp::SetMaterialsAlpha": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipWarp::ActivatePulseDrive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
        ]
    ),
    "cGcSpaceshipWarp::GetPulseDriveSpeed": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
        ]
    ),
    "cGcSpaceshipWarp::GetPulseDriveCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
        ]
    ),
    "cGcSpaceshipWarp::GetPulseDriveFuelFactor": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
        ]
    ),
    "cGcSpaceshipWarp::GetPulseDriveFactor": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
        ]
    ),
    "cGcSpaceshipWarp::CheckPulseDrive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
        ]
    ),
    "cGcSpaceshipWarp::GetFuelPulseDriveMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cGcSpaceshipWarp::UpdatePulseDrive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
            ctypes.c_double,  # double
        ]
    ),
    "cGcSpaceshipWarp::ShowPulseDriveCollision": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcSpaceshipWarp::CalcPulseDrive": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipWarp::HasPulseDriveCollision": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWarp *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cGcWarpEffect::RequestWarpIn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWarpEffect *
            ctypes.c_float,  # float
        ]
    ),
    "cGcWarpEffect::RequestWarpOut": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWarpEffect *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWarpEffect::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWarpEffect *
        ]
    ),
    "cGcWarpEffect::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWarpEffect *
        ]
    ),
    "cGcWarpEffect::PlayWarpEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWarpEffect *
        ]
    ),
    "cGcWarpEffect::Finish": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWarpEffect *
        ]
    ),
    "cGcWarpEffect::IsNexus": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcWarpEffect *
        ]
    ),
    "cGcWarpEffect::IsFreighter": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcWarpEffect *
        ]
    ),
    "cGcWarpEffect::GetAxisSign": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcWarpEffect *
        ]
    ),
    "cGcWarpEffect::UpdateWarpIn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWarpEffect *
            ctypes.c_float,  # float
        ]
    ),
    "cGcWarpEffect::UpdateWarpOut": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWarpEffect *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipWeapons::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
            ctypes.c_ulonglong,  # cGcOwnerConcept *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcSpaceshipWeapons::GetCurrentShootPoints": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcShootPoint *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipWeapons::UpdateAutoLockTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipWeapons::UpdateManualLockTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSpaceshipWeapons::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::CycleWeapon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::GetReloadProgress": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::GetOverheatProgress": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::GetDamageTypeForMode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
            ctypes.c_uint32,  # eShipWeapon
        ]
    ),
    "cGcSpaceshipWeapons::GetActiveInventoryItem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcAtlasSendSubmitContribution *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::IsProjectile": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::GetWeaponSpeed": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSpaceshipWeapons::GetWeaponRange": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSpaceshipWeapons::RenderImGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::IsPotentialTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cGcSpaceshipWeapons::TurnOffWeapons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::GetDamagePotential": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
            ctypes.c_ubyte,  # bool
            common.GcSeed,  # cTkSeed
            ctypes.c_ulonglong,  # const cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
        ]
    ),
    "cGcSpaceshipWeapons::GetAverageBarrelPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcSpaceshipWeapons::CanFire": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::GetAlertPoint": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::GetOverHeatTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::GetHeatWarningFactor": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
        ]
    ),
    "cGcSpaceshipWeapons::HasWeapon": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
            ctypes.c_uint32,  # eShipWeapon
        ]
    ),
    "cGcSpaceshipWeapons::CalculateDPS": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
            ctypes.c_uint32,  # const eShipWeapon
            ctypes.c_ubyte,  # BOOL
            ctypes.c_ubyte,  # const bool
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
        ]
    ),
    "cGcSpaceshipWeapons::CalculateDamagePerHit": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
            ctypes.c_uint32,  # const eShipWeapon
            ctypes.c_uint32,  # ItemLookupType
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
            ctypes.c_ulonglong,  # const cGcInventoryStore *
        ]
    ),
    "cGcSpaceshipWeapons::GetInventoryItemForShot": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcAtlasSendSubmitContribution *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
            ctypes.c_uint32,  # eShipWeapon
        ]
    ),
    "cGcSpaceshipWeapons::ReportFailureToFire": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipWeapons *
            ctypes.c_uint32,  # eShipWeapon
        ]
    ),
    "cGcShootPoint::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootPoint *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
        ]
    ),
    "cGcShootPoint::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootPoint *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcShootPoint::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootPoint *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cGcStagedJetEngine::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStagedJetEngine *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cGcMechControl::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
        ]
    ),
    "cGcMechControl::OnSummon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
        ]
    ),
    "cGcMechControl::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_ubyte,  # bool
            ctypes.c_double,  # double
        ]
    ),
    "cGcMechControl::SetPhysicsEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_ubyte,  # bool
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcMechControl::DetermineAnimState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMechControl::UpdateAnims": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMechControl::UpdateFootDust": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_ubyte,  # bool
            ctypes.c_double,  # double
        ]
    ),
    "cGcMechControl::DoFootPrint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechControl::PlayMechAudioEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_ulonglong,  # const cGcMechAudioEvent *
        ]
    ),
    "cGcMechControl::GetMoveDir": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcMechControl::UpdateFootprints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMechControl::UpdateJetpack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcMechControl::UpdateWeaponArmAimAnims": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMechControl::NetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cGcMechControl::UpdateContrails": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cGcMechControl::UpdateJumpState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcMechControl::IsJetpackIgnited": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
        ]
    ),
    "cGcMechControl::UpdateSlope": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcMechControl::GetCockpitBob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcMechControl::SetPilotType": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_uint32,  # cGcMechControl::eMechPilotType
        ]
    ),
    "cGcMechControl::DeactivatePilot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
        ]
    ),
    "cGcMechControl::ActivatePilot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
        ]
    ),
    "cGcMechControl::PlayerInMechCombat": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
        ]
    ),
    "cGcMechControl::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechControl *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcVehicleCheckpointComponent::GetBeamPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointComponent *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cGcVehicleCheckpointManager::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<RaceSection,8,-1> *
        ]
    ),
    "cGcVehicleCheckpointManager::GetRaceProgress": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
        ]
    ),
    "cGcVehicleCheckpointManager::GetCheckpointBuilding": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cGcVehicleCheckpointComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointComponent *
        ]
    ),
    "cGcVehicleCheckpointComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointComponent *
        ]
    ),
    "cGcVehicleCheckpointComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointComponent *
        ]
    ),
    "cGcVehicleCheckpointComponent::UpdateRace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointComponent *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcVehicleCheckpointComponent::SetRaceState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointComponent *
            ctypes.c_uint32,  # cGcVehicleCheckpointComponent::eState
        ]
    ),
    "cGcVehicleComponent::SetCollisionUnpiloted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleRaceInviteComponent::GetFromLocation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcVehicleCheckpointManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
        ]
    ),
    "cGcVehicleCheckpointManager::SetupRaceCourse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
            ctypes.c_ulonglong,  # cGcVehicleCheckpointComponent *
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<RaceSection,8,-1> *
        ]
    ),
    "cGcVehicleCheckpointManager::GetStartInteraction": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInteractionComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
        ]
    ),
    "cGcVehicleCheckpointManager::MultiplayerRaceStart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
        ]
    ),
    "cGcVehicleCheckpointManager::LeaveRace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
        ]
    ),
    "cGcVehicleCheckpointManager::IsPlayerLookingAtDeleteableCheckpoint": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
        ]
    ),
    "cGcVehicleCheckpointManager::SpawnPlayerAtStart": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
        ]
    ),
    "cGcVehicleCheckpointManager::RestartCancelledPlayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
        ]
    ),
    "cGcVehicleCheckpointManager::WantsQuitRace": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
        ]
    ),
    "cGcVehicleCheckpointManager::GetStartCheckpointAtLocation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcVehicleCheckpointComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcVehicleCheckpointManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleCheckpointManager::PlaceCheckpoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
        ]
    ),
    "cGcVehicleCheckpointManager::UpdateRaceIndices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
        ]
    ),
    "cGcVehicleCheckpointManager::CalculateStartMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleCheckpointManager::SetStartCheckpoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
            ctypes.c_ulonglong,  # cGcVehicleCheckpointComponent *
        ]
    ),
    "cGcVehicleCheckpointManager::OnVehicleRaceJoin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleCheckpointManager::OnVehicleRaceStart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleCheckpointManager::OnVehicleHitCheckpoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleCheckpointManager::FinishedRace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleCheckpointManager::UpdateRaceMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
            ctypes.c_uint32,  # cGcVehicleCheckpointManager::eRaceMessage
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcVehicleCheckpointManager::ClearCheckpointsInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointManager *
            ctypes.c_ulonglong,  # cGcVehicleCheckpointComponent *
        ]
    ),
    "RaceSection::IsPersistencyValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # RaceSection *
        ]
    ),
    "RaceSection::GetRaceIndex": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # RaceSection *
        ]
    ),
    "RaceSection::GetBestTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # RaceSection *
        ]
    ),
    "RaceSection::SetBestTime": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RaceSection *
            ctypes.c_float,  # float
        ]
    ),
    "RaceSection::GetMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # RaceSection *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "RaceSection::Checkpoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcVehicleCheckpointComponent *
        argtypes=[
            ctypes.c_ulonglong,  # RaceSection *
        ]
    ),
    "cGcVehicleCockpit::GetScaleFromCockpitLocator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcVehicleCockpit::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
        ]
    ),
    "cGcVehicleCockpit::AddCockpit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
        ]
    ),
    "cGcVehicleCockpit::RemoveCockpit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
        ]
    ),
    "cGcVehicleCockpit::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
        ]
    ),
    "cGcVehicleCockpit::SetStoppedVFXNodesEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleCockpit::UpdateLoading": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
        ]
    ),
    "cGcVehicleCockpit::DoEnter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleCockpit::DoExit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleCockpit::RayTestForExitPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcVehicleCockpit::UpdateActiveAnimation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleCockpit::EndActiveAnimation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
        ]
    ),
    "cGcVehicleCockpit::FinaliseExit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
        ]
    ),
    "cGcVehicleCockpit::OnStoppedMoving": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
        ]
    ),
    "cGcVehicleCockpit::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleCockpit::UpdateTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcVehicleCockpit::UpdateSpeedMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleCockpit::UpdateAtmosphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleCockpit::SetVisibility": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleCockpit::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCockpit *
        ]
    ),
    "cGcVehicleControlStick::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleControlStick *
            ctypes.c_uint32,  # eHand
            ctypes.c_uint32,  # cGcVehicleControlStick::Type
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cGcVehicleControlStick::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleControlStick *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cGcVehicleControlStick::UpdateAsStick": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleControlStick *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cGcVehicleControlStick::UpdateAsThrottle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleControlStick *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cGcVehicleControlStick::CalculateHandStickValues": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleControlStick *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cGcAttractor::Attract": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttractor *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcAttractor::Align": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttractor *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCarSuspensionAction::TransformedMotion::CopyFromBody": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCarSuspensionAction::TransformedMotion *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCarSuspensionAction::TransformedMotion::CopyToBody": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCarSuspensionAction::TransformedMotion *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcCarSuspensionAction::TransformedMotion::GetLinearVelocityAtPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCarSuspensionAction::TransformedMotion *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCarSuspensionAction::TransformedMotion::ApplyCentralImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCarSuspensionAction::TransformedMotion *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cGcCarSuspensionAction::TransformedMotion::ApplyImpulseAtPoint": {
        "cGcCarSuspensionAction::TransformedMotion *, const cTkVector3 *, const cTkVector3 *, double": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcCarSuspensionAction::TransformedMotion *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_double,  # double
            ]
        ),
        "cGcCarSuspensionAction::TransformedMotion *, const cTkVector3 *, const cTkVector3 *, const cTkVector3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcCarSuspensionAction::TransformedMotion *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
    },
    "cGcCarSuspensionAction::cGcCarSuspensionAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCarSuspensionAction *
        ]
    ),
    "cGcCarSuspensionAction::GetWheelTurnForce": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcCarSuspensionAction *
        ]
    ),
    "cGcCarSuspensionAction::resetWheels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCarSuspensionAction *
        ]
    ),
    "cGcCarSuspensionAction::getBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCarSuspensionAction *
            ctypes.c_ulonglong,  # const hknpBodyId **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cGcCarSuspensionAction::applyAction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCarSuspensionAction *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_double,  # double
        ]
    ),
    "cGcCarSuspensionAction::TransformedMotion::TransformedMotion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCarSuspensionAction::TransformedMotion *
        ]
    ),
    "GcVehicleInstanceBroadphaseFunc": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "GcVehicleInstanceManifoldFunc": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcVehicleComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::NoFade": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::UseThirdPersonCamera": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::GetCockpitPlayerLocatorMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        ]
    ),
    "cGcVehicleComponent::GetCockpitCameraLocatorPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cGcVehicleComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_double,  # double
        ]
    ),
    "cGcVehicleComponent::UpdateAudio": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleComponent::UpdateWeapons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleComponent::CycleTerrainEditMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::UpdateSteeringWheel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleComponent::ToggleCameraViewPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::UpdateCameraLook": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_ulonglong,  # const cGcPlayerCameraLook *
        ]
    ),
    "cGcVehicleComponent::GetBoosterColour": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_ulonglong,  # cTkColour *
        ]
    ),
    "cGcVehicleComponent::UpdateControl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_double,  # double
        ]
    ),
    "cGcVehicleComponent::UpdateJumping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleComponent::UpdateBoost": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleComponent::UpdateMechControls": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleComponent::UpdateUnderwaterControl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_double,  # double
        ]
    ),
    "cGcVehicleComponent::UpdateHeadlights": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleComponent::UpdateCockpit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleComponent::ReportMalfunction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::UseFuel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcVehicleComponent::CycleWeaponMode": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::GetUnderwaterDepth": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::GetSurfaceFactor": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::ApplyBuoyancy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_double,  # double
        ]
    ),
    "cGcVehicleComponent::UpdateScanning": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleComponent::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcVehicleComponent::Enter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::InstantExit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::GetFuel": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::IsEngineActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::IsBoosting": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::HasDriver": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::NetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcVehicleComponent::IsOnGround": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::NotifyDeathOfDriver": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::HasWorkingScanner": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cGcVehicleComponent::GetRigidBody": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRigidBody *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::DisableEmitters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::IsAIControlAvailable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::IsUnderAIControl": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::ProjectileHitCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcVehicleComponent::OnProjectileHit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_ulonglong,  # const cGcProjectileImpact *
        ]
    ),
    "cGcVehicleComponent::UpdateActiveMechMeshParts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
        ]
    ),
    "cGcVehicleComponent::GetShootPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_uint32,  # eVehicleWeaponMode
        ]
    ),
    "cGcVehicleComponent::TriggerWeaponMuzzleEffects": FUNCDEF(
        restype=ctypes.c_ulonglong,  # EffectInstance *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_ulonglong,  # EffectInstance *
            ctypes.c_uint32,  # eVehicleWeaponMode
        ]
    ),
    "cGcVehicleComponent::UpdateActiveMechWeaponLocations": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcVehicleComponent::FireWeapon": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cGcVehicleComponent::GetCombatEffectsForLaser": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # std::vector<cGcImpactCombatEffectData,TkSTLAllocatorShim<cGcImpactCombatEffectData,4,-1> > *
            ctypes.c_ulonglong,  # std::vector<cGcCombatEffectDamageMultiplier,TkSTLAllocatorShim<cGcCombatEffectDamageMultiplier,4,-1> > *
        ]
    ),
    "cGcVehicleComponent::UpdateWeaponAim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponent *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcVehicleWeapon::GetInventoryItem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcAtlasSendSubmitContribution *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleWeapon *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcVehicleWeapon::GetWeaponRange": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleWeapon *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcVehicleWeapon::GetWeaponSpeed": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleWeapon *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkPerlinNoise<&cTkPerlinNoiseFunctions::ValueNoise3d": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
        ]
    ),
    "RaceSection::RaceSection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RaceSection *
            ctypes.c_ulonglong,  # const RaceSection *
        ]
    ),
    "cTkRigidBody::GetLinearVelocityAtPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
        ]
    ),
    "cGcVehicleGarageComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleGarageComponent *
        ]
    ),
    "cGcVehicleGarageComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleGarageComponent *
        ]
    ),
    "cGcVehicleGarageComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleGarageComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleGarageComponent::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleGarageComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleGarageComponent::SpawnVehicle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleGarageComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleRaceInviteComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleRaceInviteComponent *
        ]
    ),
    "cGcVehicleRaceInviteComponent::RefreshRaceTrack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleRaceInviteComponent *
        ]
    ),
    "cGcVehicleRaceInviteComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleRaceInviteComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcVehicleRaceInviteComponent::NetSerializeOnInit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleRaceInviteComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cGcVehicleRaceInviteComponent::NetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleRaceInviteComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWaypointComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaypointComponent *
        ]
    ),
    "cGcWaypointComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaypointComponent *
        ]
    ),
    "cGcWaypointComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaypointComponent *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWaypointComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaypointComponent *
        ]
    ),
    "cGcWaypointComponent::ShowSectorMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaypointComponent *
        ]
    ),
    "cGcWaypointComponent::ShowDiscoveryMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaypointComponent *
        ]
    ),
    "cGcWaypointComponent::Interact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaypointComponent *
        ]
    ),
    "cGcWaypointComponent::UpdateDiscoveryData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaypointComponent *
        ]
    ),
    "cGcInputRemap::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputRemap *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputRemap::LoadActionSets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputRemap *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputRemap::RegisterBindings": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputRemap *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputRemap::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputRemap *
        ]
    ),
    "cGcInputRemap::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputRemap *
        ]
    ),
    "cGcInputRemap::ResetBindings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputRemap *
            ctypes.c_uint32,  # eActionSetType
        ]
    ),
    "cTkInputPort::cTkInputPort": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_ulonglong,  # const cTkInputPort *
        ]
    ),
    "cGcInputRemap::RemapAction": {
        "cGcInputRemap *, eActionSetType, eInputAction, eInputButton": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcInputRemap *
                ctypes.c_uint32,  # eActionSetType
                ctypes.c_uint32,  # eInputAction
                ctypes.c_uint32,  # eInputButton
            ]
        ),
        "cGcInputRemap *, eActionSetType, eInputAction, eInputAxis": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cGcInputRemap *
                ctypes.c_uint32,  # eActionSetType
                ctypes.c_uint32,  # eInputAction
                ctypes.c_uint32,  # eInputAxis
            ]
        ),
    },
    "FindInputBinding": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sTkInputBinding *
        argtypes=[
            ctypes.c_uint32,  # eActionSetType
            ctypes.c_uint32,  # eInputAction
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eInputButtonType
            ctypes.c_int32,  # int
        ]
    ),
    "cGcInputRemap::ValidateRemapping": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputRemap *
            ctypes.c_uint32,  # eActionSetType
            ctypes.c_ulonglong,  # const sTkInputBinding *
        ]
    ),
    "cGcInputRemap::HasKeyButton": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputRemap *
            ctypes.c_uint32,  # eActionSetType
            ctypes.c_uint32,  # eInputAction
        ]
    ),
    "cGcInputRemap::IsPadActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputRemap *
        ]
    ),
    "cGcInputRemap::GetKeyName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputRemap *
            ctypes.c_uint32,  # eActionSetType
            ctypes.c_uint32,  # eInputAction
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        ]
    ),
    "cGcInputRemap::HasPadButton": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputRemap *
            ctypes.c_uint32,  # eActionSetType
            ctypes.c_uint32,  # eInputAction
        ]
    ),
    "cGcInputRemap::GetPadName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputRemap *
            ctypes.c_uint32,  # eActionSetType
            ctypes.c_uint32,  # eInputAction
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        ]
    ),
    "cGcInputRemap::GetPadIcon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sTkInputDeviceIcons *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputRemap *
            ctypes.c_uint32,  # eActionSetType
            ctypes.c_uint32,  # eInputAction
        ]
    ),
    "cGcInputRemap::GetKeyIcon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sTkInputDeviceIcons *
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputRemap *
            ctypes.c_uint32,  # eActionSetType
            ctypes.c_uint32,  # eInputAction
        ]
    ),
    "BindingHelpers__CollectBindings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcInputActionMapping> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcInputActionMapping2> *
            ctypes.c_ulonglong,  # std::vector<sGcInputBinding,TkSTLAllocatorShim<sGcInputBinding,4,-1> > *
        ]
    ),
    "BindingHelpers__CollectBindings_0": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<cGcInputActionMapping2,TkSTLAllocatorShim<cGcInputActionMapping2,1,-1> > *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcInputRemap::RetrieveCustomBindings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputRemap *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcInputActionMapping> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcInputActionMapping2> *
        ]
    ),
    "cGcInputRemap::StoreCustomBindings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputRemap *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInputActionMapping> *
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInputActionMapping2> *
        ]
    ),
    "cGcVibrationManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVibrationManager *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVibrationManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVibrationManager *
            ctypes.c_float,  # float
        ]
    ),
    "cGcVibrationManager::AddChannel": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcVibrationChannel *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVibrationManager *
            ctypes.c_ulonglong,  # cGcVibrationChannelData *
        ]
    ),
    "cGcVibrationManager::SendValue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcVibrationManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcVibrationManager::SetTriggerFeedback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVibrationManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cGcVibrationManager::GetActionTriggerIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVibrationManager *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcVibrationManager::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVibrationManager *
        ]
    ),
    "cTkDynamicArray<cGcInputActionMapping2>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInputActionMapping2> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcInputActionInfoMap>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInputActionInfoMap *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcActionSets>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcActionSets *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcInputBindings>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInputBindings *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcVibrationDataTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcVibrationDataTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cGcTriggerFeedbackStateTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTriggerFeedbackStateTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAnimationLayerQueue::Play": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAnimationLayerQueue *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # AnimationTransitionType
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eCurve
        ]
    ),
    "cGcAnimationLayerQueue::StartAnimation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAnimationLayerQueue *
            ctypes.c_ulonglong,  # cGcAnimationLayerQueue::AnimEntry *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cGcAnimationLayerQueue::StopAllAnimations": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAnimationLayerQueue *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eCurve
        ]
    ),
    "cGcAnimationLayerQueue::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAnimationLayerQueue *
            ctypes.c_float,  # float
        ]
    ),
    "cGcAnimationLayerQueue::StopAnyCurrentTransitions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAnimationLayerQueue *
        ]
    ),
    "cGcSimpleAnimationQueue::UpdateQueue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleAnimationQueue *
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_float,  # const float
        ]
    ),
    "cGcSimpleAnimationQueue::StopAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleAnimationQueue *
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cGcSimpleAnimationQueue::IsPlaying": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleAnimationQueue *
            ctypes.c_float,  # float
        ]
    ),
    "cTkRotationComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRotationComponent *
        ]
    ),
    "cTkRotationComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRotationComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercTkRotationComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercTkRotationComponent *
        ]
    ),
    "ComponentInstancercTkRotationComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkReferenceComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkReferenceComponent *
        ]
    ),
    "cTkReferenceComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkReferenceComponent *
        ]
    ),
    "cTkReferenceComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkReferenceComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercTkReferenceComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercTkReferenceComponent *
        ]
    ),
    "ComponentInstancercTkReferenceComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentLoadInstancercTkReferenceComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentLoadInstancercTkReferenceComponent *
        ]
    ),
    "cGcDebugEditor::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugEditor *
            ctypes.c_float,  # float
        ]
    ),
    "cGcDebugEditor::Deactivate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugEditor *
        ]
    ),
    "cGcDebugEditor::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugEditor *
        ]
    ),
    "cGcDebugEditor::CalculatePlaceMatrix": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugEditor *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cGcDebugRenderOffsetControl::ApplyCamCenterRenderOffset": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkMetaData::ReadFromFile<cTkLSystemRulesData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkLSystemRulesData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkResource::IsLoaded": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkResource *
        ]
    ),
    "cTkResource::SetDescriptor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResource *
            ctypes.c_ulonglong,  # const cTkResourceDescriptor *
        ]
    ),
    "cTkResource::Equals": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgResource::CloneInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgResource *
            ctypes.c_ulonglong,  # const cTkResource *
        ]
    ),
    "cEgCodeResource::FactoryFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "cEgShaderResource::FactoryFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "cTkTextureBase::ComputeEvictableSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkTextureBase *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkTexture::SetLodClamp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_float,  # float
        ]
    ),
    "cEgTextureResource::FactoryFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgTextureResource *
        argtypes=[
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "cEgTextureResource::ScheduleForEviction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
        ]
    ),
    "cEgTextureResource::Evict": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
        ]
    ),
    "cEgMaterialResource::FactoryFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "cEgPipelineResource::FactoryFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkResource *
        argtypes=[
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "cEgThreadableRenderCall::SetData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_ulonglong,  # const cEgThreadableRenderCall::cRendererData *
            ctypes.c_ulonglong,  # cTkDestructableClass *
        ]
    ),
    "cEgSceneNodeTemplate::cEgSceneNodeTemplate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "cEgSceneNodeTemplate::~cEgSceneNodeTemplate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
        ]
    ),
    "cEgSceneNodeTemplate::SetName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cEgSceneNodeTemplate::GetTransform": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cEgSceneNodeTemplate::SetDescriptor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # const cTkSharedPtr<cTkResourceDescriptor const > *
        ]
    ),
    "cEgSceneNodeTemplate::GatherMaterials": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *
        ]
    ),
    "cEgSceneNodeTemplate::GatherData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # std::vector<cTkAttachmentData *,TkSTLAllocatorShim<cTkAttachmentData *,8,-1> > *
        ]
    ),
    "cEgSceneNodeTemplate::GetBoundingBox": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAABB *
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkAABB *
            ctypes.c_ulonglong,  # const cTkResourceDescriptor *
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgSceneNodeTemplate::IsGeometryStreamedIn": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # cEgResource *
        ]
    ),
    "cEgAnimationResource::FactoryFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "GeometryStreaming::cEgStreamRequests::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GeometryStreaming::cEgStreamRequests *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "GeometryStreaming::cEgStreamRequests::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # GeometryStreaming::cEgStreamRequests *
        argtypes=[]
    ),
    "GeometryStreaming::cEgGeometryStreamer::IsMeshDoubleBuffered": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # GeometryStreaming::cEgGeometryStreamer *
            ctypes.c_int32,  # int
        ]
    ),
    "GeometryStreaming::cEgGeometryStreamer::SyncDataForRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GeometryStreaming::cEgGeometryStreamer *
        ]
    ),
    "cEgGeometryAABBCache::cEgGeometryAABBCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryAABBCache *
            ctypes.c_ulonglong,  # const cEgGeometryAABBCache *
        ]
    ),
    "cEgGeometryResource::FactoryFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "cEgGeometryResource::GetBindTransform": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AnimTransform *
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cEgGeometryResource::SetSkinnedVertexData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkSkinnedVertexData *
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgGeometryResource::CloneOriginalVertDataToIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgGeometryResource::CreateVertexInfoForHash": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cEgGeometryResource::InsertElement": {
        "cEgGeometryResource *, char *, int, const cTkVector2 *, const cTkVertexElement *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cEgGeometryResource *
                ctypes.c_ulonglong,  # char *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVertexElement *
            ]
        ),
        "cEgGeometryResource *, char *, int, const cTkVector3 *, const cTkVertexElement *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cEgGeometryResource *
                ctypes.c_ulonglong,  # char *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVertexElement *
            ]
        ),
        "cEgGeometryResource *, char *, int, const cTkVector4 *, const cTkVertexElement *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cEgGeometryResource *
                ctypes.c_ulonglong,  # char *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # const cTkVector4 *
                ctypes.c_ulonglong,  # const cTkVertexElement *
            ]
        ),
    },
    "cEgGeometryAABBCache::~cEgGeometryAABBCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryAABBCache *
        ]
    ),
    "cEgModelNodeTemplate::cEgModelNodeTemplate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNodeTemplate *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgModelNodeTemplate::~cEgModelNodeTemplate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNodeTemplate *
        ]
    ),
    "cEgModelNodeTemplate::GetBoundingBox": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAABB *
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNodeTemplate *
            ctypes.c_ulonglong,  # cTkAABB *
            ctypes.c_ulonglong,  # const cTkResourceDescriptor *
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgModelNodeTemplate::IsGeometryStreamedIn": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNodeTemplate *
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # cEgResource *
        ]
    ),
    "cEgModelNodeTemplate::IsLoaded": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNodeTemplate *
        ]
    ),
    "cEgModelNode::SetAnimJointsLOD": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgModelNode::AddSkinMatsToBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cEgLightNodeTemplate::cEgLightNodeTemplate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgLightNodeTemplate *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "cEgIndexMappingArray::GetIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgIndexMappingArray *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const cTkFixedString<32,char> *
        ]
    ),
    "cEgSceneManager::ClearQueues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneManager *
        ]
    ),
    "cEgRenderCameraInfo::~cEgRenderCameraInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderCameraInfo *
        ]
    ),
    "cEgCameraNodeTemplate::cEgCameraNodeTemplate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgCameraNodeTemplate *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cEgPipelineResource *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "cEgCameraNodeTemplate::~cEgCameraNodeTemplate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgCameraNodeTemplate *
        ]
    ),
    "cEgRenderActionSetResParam::MoveTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetResParam *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgRenderActionSetMaterialUniform::MoveTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetMaterialUniform *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgRenderActionSetMaterialUniformArray::MoveTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetMaterialUniformArray *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgRenderActionSetMaterialSampler::MoveTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetMaterialSampler *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgRenderActionClearMaterialSamplers::MoveTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionClearMaterialSamplers *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgRenderActionFlushTexture::MoveTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionFlushTexture *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgRenderActionSetAsyncComputeEnable::MoveTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetAsyncComputeEnable *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgRenderActionBlitTexture::MoveTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionBlitTexture *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgRenderActionEndScissorTest::MoveTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionEndScissorTest *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgRenderActionSetupViewport::MoveTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetupViewport *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgRenderActionSetRenderTarget::MoveTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetRenderTarget *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgRenderActionSetSamplerDefaultTexture::MoveTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetSamplerDefaultTexture *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgRenderActionClear::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionClear *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgRenderActionClear::MoveTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionClear *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgRenderActionQueue::MoveActionQueue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionQueue *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgRenderer::EnqueueBlitTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_ulonglong,  # const cTkVector4 *
            ctypes.c_ulonglong,  # const cTkVector4 *
        ]
    ),
    "cEgRenderActionSetMaterialUniformArray::~cEgRenderActionSetMaterialUniformArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetMaterialUniformArray *
        ]
    ),
    "cEgRenderActionSetSamplerDefaultTexture::~cEgRenderActionSetSamplerDefaultTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetSamplerDefaultTexture *
        ]
    ),
    "cEgRenderActionBlitTexture::~cEgRenderActionBlitTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionBlitTexture *
        ]
    ),
    "cEgExtensionManager::InstallExtension": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgExtensionManager *
            ctypes.c_ulonglong,  # {'return': 'const char *', 'args': []}
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': []}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': []}
        ]
    ),
    "cEgMeshNodeTemplate::cEgMeshNodeTemplate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNodeTemplate *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgMeshNodeTemplate::SetName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNodeTemplate *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cEgMeshNodeTemplate::SetDescriptor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNodeTemplate *
            ctypes.c_ulonglong,  # const cTkSharedPtr<cTkResourceDescriptor const > *
        ]
    ),
    "cEgMeshNodeTemplate::GatherMaterials": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNodeTemplate *
            ctypes.c_ulonglong,  # std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *
        ]
    ),
    "cEgMeshNodeTemplate::GetBoundingBox": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAABB *
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNodeTemplate *
            ctypes.c_ulonglong,  # cTkAABB *
            ctypes.c_ulonglong,  # const cTkResourceDescriptor *
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgMeshNodeTemplate::IsGeometryStreamedIn": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNodeTemplate *
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # cEgResource *
        ]
    ),
    "cEgMeshNode::GetANName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNode *
        ]
    ),
    "cEgMeshNode::GetANNameHash": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNode *
        ]
    ),
    "cEgMeshNode::GetParentModel": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgModelNode *
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNode *
        ]
    ),
    "cEgJointNode::GetANName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cEgJointNode *
        ]
    ),
    "cEgJointNode::GetANNameHash": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgJointNode *
        ]
    ),
    "cEgJointNode::GetJointIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgJointNode *
        ]
    ),
    "CAkFxBaseStateAware::GetStateChunks": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgModelNode *
        argtypes=[
            ctypes.c_ulonglong,  # cEgJointNode *
        ]
    ),
    "cEgMeshNode::cEgMeshNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNode *
            ctypes.c_ulonglong,  # const cEgMeshNodeTemplate *
        ]
    ),
    "cEgMeshNode::~cEgMeshNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNode *
        ]
    ),
    "cEgMeshNode::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgMeshNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgMeshNode::Parse": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgMeshNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNodeTemplate *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgMeshNode::FactoryFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "cEgMeshNode::GetANParent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAnimatableNode *
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNode *
        ]
    ),
    "cEgMeshNode::CanAttach": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNode *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "cEgMeshNode::GetParameterInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNode *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgMeshNode::SetParameterInt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNode *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgMeshNode::CheckIntersection": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNode *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cEgMeshNode::OnAttach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNode *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "cEgMeshNode::GetVisibleBBox": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNode *
            ctypes.c_ulonglong,  # cEgBoundingBox *
        ]
    ),
    "cEgMeshNode::GetParameterFloat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNode *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgMeshNode::SetParameterFloat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNode *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_double,  # double
        ]
    ),
    "cEgMeshNode::EnqueueFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
            ctypes.c_ulonglong,  # cEgRenderQueueEntry *
            ctypes.c_ulonglong,  # cEgRenderableSceneNode **
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgJointNode::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSharedPtr<cTkResourceDescriptor const > *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgJointNode::FactoryFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNode *
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "cEgJointNode::GetANParent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAnimatableNode *
        argtypes=[
            ctypes.c_ulonglong,  # cEgJointNode *
        ]
    ),
    "cEgJointNode::GetParameterInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgJointNode *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgJointNode::OnAttach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgJointNode *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "cEgJointNode::OnDetach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgJointNode *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "IAnimatableNode::SetANIgnoreAnimation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # IAnimatableNode *
            ctypes.c_int64,  # __int64
        ]
    ),
    "IAnimatableNode::AddANToIgnoreAnimationResetList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # IAnimatableNode *
        ]
    ),
    "cEgAnimationResource::cEgAnimationResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationResource *
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgAnimationResource::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkResource *
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationResource *
        ]
    ),
    "cEgAnimationResource::CloneInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationResource *
            ctypes.c_ulonglong,  # const cTkResource *
        ]
    ),
    "cEgAnimationResource::InitDefault": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationResource *
        ]
    ),
    "cEgAnimationResource::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationResource *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<AnimResEntity,8,-1> *
        ]
    ),
    "cEgAnimationResource::Load": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgAnimationResource::GetElementCount": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationResource *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgAnimationResource::GetElementParamInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgAnimationResource::FindEntity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimNodeData *
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationResource *
            ctypes.c_uint32,  # const unsigned int
        ]
    ),
    "cEgAnimationController::ResizeNodeList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationController *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgAnimationController::RegisterNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationController *
            ctypes.c_ulonglong,  # IAnimatableNode *
        ]
    ),
    "cEgAnimationController::CreateAnimMapping": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationController *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # cEgAnimationResource *
            ctypes.c_ulonglong,  # const cTkStackVector<unsigned int,1,-1> *
        ]
    ),
    "cEgAnimationController::AnimMapping::~AnimMapping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationController::AnimMapping *
        ]
    ),
    "cEgAnimationController::MapAnimationResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationController *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # cEgAnimationResource *
            ctypes.c_ulonglong,  # const cTkStackVector<unsigned int,1,-1> *
            ctypes.c_ulonglong,  # cEgAnimationController::AnimMapping *
        ]
    ),
    "cEgAnimationController::SetWorldPoseMatrix": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationController *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "ExecuteExtractFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimMetadata *
            ctypes.c_ulonglong,  # cTkAnimNodeData *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "ExtractFrame": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimMetadata *
            ctypes.c_ulonglong,  # cTkAnimNodeData *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # AnimTransform *
        ]
    ),
    "ExtractAllTransformsForFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimMetadata *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkQuaternion *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "ExtractMoveLocatorTransform": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AnimTransform *
        argtypes=[
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_ulonglong,  # cTkAnimMetadata *
            ctypes.c_ulonglong,  # cTkAnimNodeData *
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgAnimationController::RunPreAdditiveCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationController *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # std::vector<cCallbackWithPriority<cPreAdditiveCallback>,TkSTLAllocatorShim<cCallbackWithPriority<cPreAdditiveCallback>,8,-1> > *
        ]
    ),
    "cEgAnimationController::WriteBindPose": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgAnimationController::MirrorPose": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # IAnimatableNode **
            ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgAnimationController::ExtractZeroFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_ulonglong,  # const cEgAnimationController::AnimMapping *
            ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgAnimationController::Subtract": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgAnimationController::Add": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgAnimationController::NlerpAdd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgAnimationController::NlerpComplete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgAnimationController::NlerpMultiply": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgAnimationController::Multiply": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgAnimationController::Blend": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgAnimationController::EvalCommandListInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationController *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_ulonglong,  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # std::vector<cCallbackWithPriority<cPreAdditiveCallback>,TkSTLAllocatorShim<cCallbackWithPriority<cPreAdditiveCallback>,8,-1> > *
        ]
    ),
    "cEgAnimationController::BuildSkinningMatrices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationController *
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgAnimationController::AddAnimToAdditivePose": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationController *
            ctypes.c_ulonglong,  # cEgAnimationResource *
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkHmdBase::GetProjectionRawUnscaled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdBase *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cEgCameraNode::cEgCameraNode": {
        "cEgCameraNode *, const cEgCameraNodeTemplate *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cEgCameraNode *
                ctypes.c_ulonglong,  # const cEgCameraNodeTemplate *
            ]
        ),
        "cEgCameraNode *, const cEgCameraNode *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cEgCameraNode *
                ctypes.c_ulonglong,  # const cEgCameraNode *
            ]
        ),
    },
    "cEgCameraNode::~cEgCameraNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgCameraNode *
        ]
    ),
    "cEgCameraNode::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgCameraNode::FactoryFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "cEgCameraNode::GetParameterInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgCameraNode *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgCameraNode::SetParameterInt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgCameraNode *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgCameraNode::GetParameterFloat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cEgCameraNode *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgCameraNode::SetParameterFloat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgCameraNode *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cEgCameraNode::SetupViewParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgCameraNode *
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector4 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgCameraNode::SetVelFromPrevMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgCameraNode *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cEgCameraNode::UpdateViewFrustum": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgCameraNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgCollisionNode::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSharedPtr<cTkResourceDescriptor const > *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgCollisionNode::FactoryFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNode *
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "cEgCollisionNode::GetParameterInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgCollisionNode *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgCollisionNode::GetParameterFloat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cEgCollisionNode *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgEngineLog::PushMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgEngineLog *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "cEgEngineLog::WriteError": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgEngineLog *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cEgEngineLog::WriteWarning": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgEngineLog *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cEgEngineLog::WriteDebugInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgEngineLog *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cEgExtensionManager::InstallExtensions": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgExtensionManager *
        ]
    ),
    "cEgGeometryResource::cEgGeometryResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkResourceDescriptor *
        ]
    ),
    "cEgGeometryResource::~cEgGeometryResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgGeometryResource::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkResource *
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
        ]
    ),
    "cEgGeometryResource::CloneInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # const cTkResource *
        ]
    ),
    "cEgGeometryResource::InitDefault": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
        ]
    ),
    "cEgGeometryResource::ReleaseVertexBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgGeometryResource::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgGeometryResource::Load": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgGeometryResource::ParseData": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # cTkGeometryData *
        ]
    ),
    "cEgGeometryResource::GetElementCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgGeometryResource::GetElementParamInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgGeometryResource::MapResStream": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgJoint *
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgGeometryResource::GetSkinnedVertexData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # cTkSkinnedVertexData *
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "cEgGeometryResource::UpdateDynamicVertData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cEgGeometryResource::BuildVertexDecl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVertexLayoutRT *
        ]
    ),
    "cEgGeometryResource::GetVertexBufferHandle": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgGeometryResource::GetIndexBufferHandle": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_int32,  # int
        ]
    ),
    "GeometryStreaming::cEgGeometryStreamer::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GeometryStreaming::cEgGeometryStreamer *
            ctypes.c_ulonglong,  # cTkGeometryData *
        ]
    ),
    "GeometryStreaming::cEgGeometryStreamer::UpdateDoubleBufferedMeshList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GeometryStreaming::cEgGeometryStreamer *
        ]
    ),
    "GeometryStreaming::cEgGeometryStreamer::GetVertexBufferByHash": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # GeometryStreaming::cEgGeometryStreamer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "GeometryStreaming::cEgGeometryStreamer::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GeometryStreaming::cEgGeometryStreamer *
        ]
    ),
    "GeometryStreaming::cEgGeometryStreamer::OnBufferLoadFinish": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "GeometryStreaming::cEgGeometryStreamer::RequestStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GeometryStreaming::cEgGeometryStreamer *
            ctypes.c_ulonglong,  # const cTkResourceDescriptor *
        ]
    ),
    "GeometryStreaming::cEgGeometryStreamer::RequestUnstream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GeometryStreaming::cEgGeometryStreamer *
            ctypes.c_ulonglong,  # const cTkResourceDescriptor *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "GeometryStreaming::cEgGeometryStreamer::FindBufferIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # GeometryStreaming::cEgGeometryStreamer *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # std::_Vector_iterator<std::_Vector_val<std::_Simple_types<GeometryStreaming::cEgGeometryStreamer::cBufferData> > > *
        ]
    ),
    "GeometryStreaming::cEgGeometryStreamer::DecrementBufferRefCount": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # GeometryStreaming::cEgGeometryStreamer *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "GeometryStreaming::cEgGeometryStreamer::~cEgGeometryStreamer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GeometryStreaming::cEgGeometryStreamer *
        ]
    ),
    "GeometryStreaming::cEgStreamRequests::ProcessOnlyStreamRequests": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GeometryStreaming::cEgStreamRequests *
            ctypes.c_double,  # long double
        ]
    ),
    "GeometryStreaming::cEgStreamRequests::ProcessRenderDataSyncRequests": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GeometryStreaming::cEgStreamRequests *
            ctypes.c_double,  # long double
        ]
    ),
    "GeometryStreaming::cEgStreamRequests::AddRenderDataSyncRequest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GeometryStreaming::cEgStreamRequests *
            ctypes.c_ulonglong,  # GeometryStreaming::cEgGeometryStreamer *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['GeometryStreaming::cEgGeometryStreamer *']}
        ]
    ),
    "cEgNodeAttachment::GetFromNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgNodeAttachment *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cEgNodeAttachment::AttachToNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgNodeAttachment *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cEgNodeAttachment::~cEgNodeAttachment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgNodeAttachment *
        ]
    ),
    "cEgLightNode::cEgLightNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgLightNode *
            ctypes.c_ulonglong,  # const cEgLightNodeTemplate *
        ]
    ),
    "cEgLightNode::~cEgLightNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgLightNode *
        ]
    ),
    "cEgLightNode::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgLightNode::FactoryFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "cEgLightNode::EnqueueFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
            ctypes.c_ulonglong,  # cEgRenderQueueEntry *
            ctypes.c_ulonglong,  # cEgRenderableSceneNode **
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgLightNode::GetParameterInt": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgLightNode *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgLightNode::SetParameterInt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgLightNode *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgLightNode::GetParameterFloat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cEgLightNode *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgLightNode::SetParameterFloat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgLightNode *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_double,  # double
        ]
    ),
    "cEgLightNode::GetParameterStr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cEgLightNode *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgLightNode::SetParameterStr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgLightNode *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cEgLightNode::UpdatePositionAndDirection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgLightNode *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cEgLightNode::CalculateViewMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cEgLightNode *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cEgLightNode::GetFalloff": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cEgLightNode *
        ]
    ),
    "cEgSceneGraphResource::FactoryFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "cEgSceneGraphResource::GetBoundingBox": {
        "cEgSceneGraphResource *, cTkAABB *, bool": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkAABB *
            argtypes=[
                ctypes.c_ulonglong,  # cEgSceneGraphResource *
                ctypes.c_ulonglong,  # cTkAABB *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cEgSceneGraphResource *, cTkAABB *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkAABB *
            argtypes=[
                ctypes.c_ulonglong,  # cEgSceneGraphResource *
                ctypes.c_ulonglong,  # cTkAABB *
            ]
        ),
    },
    "cEgMetadataResource::FactoryFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkResource *
        argtypes=[
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "Engine::Create3DTexture": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSmartResHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # TkResourceFindType
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "Engine::CreateTexture": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSmartResHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # TkResourceFindType
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "Engine::Initialise": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[]
    ),
    "Engine::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Engine *
        ]
    ),
    "Engine::ForceUpdateNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::CopyThreadData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Engine *
        ]
    ),
    "Engine::SetThreadID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # eThreadType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "Engine::GetThreadID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # eThreadType
        ]
    ),
    "Engine::SetRenderQueueCameras": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # EgRenderQueueHandle
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_ulonglong,  # const cTkMatrix44 *
        ]
    ),
    "Engine::SetRenderQueueStereoCameras": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # EgRenderQueueHandle
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_ulonglong,  # const cTkMatrix44 *
        ]
    ),
    "Engine::EnqueueRender": FUNCDEF(
        restype=ctypes.c_uint64,  # EgRenderQueueHandle
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # EgRenderQueueHandle
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::PatchRenderTargetMatrices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
        ]
    ),
    "Engine::ExecuteQueuedActions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "Engine::EnqueueSetAsyncComputeEnable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Engine::SetGameCameraNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::SetupViewport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Engine::EnqueueSetupViewport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Engine::EnqueueEndScissorTest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Engine *
        ]
    ),
    "Engine::GetMsg": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgLogMessage *
        argtypes=[
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "Engine::SetOption": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # EgOptions::List
            ctypes.c_float,  # float
        ]
    ),
    "Engine::SyncGeometryStreaming": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_double,  # long double
        ]
    ),
    "Engine::FindRes": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkResourceDescriptor *
        ]
    ),
    "Engine::AddResource": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSmartResHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "Engine::GetDefaultTexture": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkResource *
        argtypes=[]
    ),
    "Engine::GetDefaultUavTexture": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkResource *
        argtypes=[]
    ),
    "Engine::IsMeshStreamedIn": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::GetResourceDescriptorFromNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkResourceDescriptor *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::GetResourceHandleForNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::UpdateVertexBufferData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "Engine::PollToReleaseUnusedResources": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Engine::ReleaseUnusedResources": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Engine::CreateShaderCompilationState": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "Engine::cEgShaderCompilationState::cEgShaderCompilationState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Engine::cEgShaderCompilationState *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "StackContainer<std::vector<cEgShaderResource::CombinationCompilationTask,StackAllocator<cEgShaderResource::CombinationCompilationTask,64,-1>>,64,-1>::~StackContainer<std::vector<cEgShaderResource::CombinationCompilationTask,StackAllocator<cEgShaderResource::CombinationCompilationTask,64,-1>>,64,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cEgShaderResource::CombinationCompilationTask,StackAllocator<cEgShaderResource::CombinationCompilationTask,64,-1> >,64,-1> *
        ]
    ),
    "Engine::DestroyShaderCompilationState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Engine::cEgShaderCompilationState *
        ]
    ),
    "Engine::PollToCompileShaderTaskList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkStackVector<cEgShaderResource::CombinationCompilationTask,64,-1> *
            ctypes.c_ulonglong,  # TkIterationState<1> *
        ]
    ),
    "Engine::PollToCompileUncompiledShaders": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # Engine::cEgShaderCompilationState *
        ]
    ),
    "Engine::PollToCompilePreloadedShaders": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # Engine::cEgShaderCompilationState *
        ]
    ),
    "Engine::CompileAnyUncompiledShaders": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "Engine::RefreshAllTexturesLodBias": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "Engine::RefreshAllTexturesAniso": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "Engine::RendererReady": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "Engine::RefreshAllPipelines": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Engine *
        ]
    ),
    "Engine::AnyRenderTargetScaleChange": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_char_p,  # const char *
        ]
    ),
    "Engine::SetShaderPreambles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "Engine::RecalculateRenderIDChildren": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "Engine::RecalculateRenderID": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::SetUniformArrayDefaultMultipleShaders": {
        "TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *, int, const char *, const float *, int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
                ctypes.c_int32,  # int
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # const float *
                ctypes.c_int32,  # int
            ]
        ),
        "TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *, int, unsigned __int64, const float *, int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
                ctypes.c_int32,  # int
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # const float *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "Engine::GetRenderBufferTexture": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkTexture *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "Engine::SetFurAllowed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Engine::GetNodeIsValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::GetNodeType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::GetNodeParent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::GetModelNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::SetNodeParent": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::GetNodeFirstChild": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::GetNodeNextSibling": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::GetNodeNumChildren": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::GetNodeNumDescendants": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::AddNodesStep": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkAddNodesHandle *
            ctypes.c_int32,  # int
        ]
    ),
    "Engine::DeleteRemovedNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Engine *
        ]
    ),
    "Engine::RequestRemoveNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::SetNodeActivation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Engine::SetNodeCastsShadow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Engine::SetNodeActivationRecursive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Engine::SetNodesActivation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_int32,  # int
        ]
    ),
    "Engine::MarkNodeNeedsUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::IsNodeAndParentsActive": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::GetNodeActivation": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::GetNodeTransMats": {
        "TkHandle, cTkMatrix34 *, cTkMatrix34 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_uint32,  # TkHandle
                ctypes.c_ulonglong,  # cTkMatrix34 *
                ctypes.c_ulonglong,  # cTkMatrix34 *
            ]
        ),
        "TkHandle, VecIntrinsics::V128Matrix *, VecIntrinsics::V128Matrix *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_uint32,  # TkHandle
                ctypes.c_ulonglong,  # VecIntrinsics::V128Matrix *
                ctypes.c_ulonglong,  # VecIntrinsics::V128Matrix *
            ]
        ),
    },
    "Engine::GetNodeAbsoluteTransMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "Engine::GetNodeTransPositions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "Engine::SetNodeTransMat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "Engine::ShiftAllTransformsForNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "Engine::GetNodeParamI": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
        ]
    ),
    "Engine::SetNodeParamI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "Engine::GetNodeParamF": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "Engine::SetNodeParamF": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "Engine::GetNodeParamV": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "Engine::GetNodeNameHash": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::SetNodeName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_char_p,  # const char *
        ]
    ),
    "Engine::GetNodeNetworkId": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::SetNodeNetworkId": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "Engine::GetNodeAltId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkResourceDescriptor *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::GetNodeLocalAABB": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkAABB *
        ]
    ),
    "Engine::SetNodeLocalAABB": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkAABB *
        ]
    ),
    "Engine::GetNodeAABB": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkAABB *
        ]
    ),
    "Engine::IterateNode": {
        "TkHandle, cEgSceneNode *, __int64, __int64 *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_uint32,  # TkHandle
                ctypes.c_ulonglong,  # cEgSceneNode *
                ctypes.c_int64,  # __int64
                ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['TkHandle'], 'pointer_depth': 1}
            ]
        ),
        "TkHandle, const char *, __int64, __int64 *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_uint32,  # TkHandle
                ctypes.c_char_p,  # const char *
                ctypes.c_int64,  # __int64
                ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['TkHandle'], 'pointer_depth': 1}
            ]
        ),
    },
    "Engine::FindAttachment": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgNodeAttachment *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
        ]
    ),
    "Engine::AddGroupNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_char_p,  # const char *
        ]
    ),
    "Engine::AddLocatorNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_char_p,  # const char *
        ]
    ),
    "Engine::GetAnimNameFromMapping": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int16,  # __int16
        ]
    ),
    "Engine::ApplyAnimCommandList": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkAnimCommand *
            ctypes.c_int32,  # int
        ]
    ),
    "Engine::UpdateLodding": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::SetAnimLOD": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
        ]
    ),
    "Engine::SetAnimJointsLOD": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
            ctypes.c_int32,  # int
        ]
    ),
    "Engine::ResetModelAnimJoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::SetModelAnimCulledNextUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::ClearModelAdditivePose": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "Engine::SetModelAnimJointPoseMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "Engine::SetModelAnimPoseOffset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "Engine::SubscribeToPreAdditiveCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cPreAdditiveCallback *
            ctypes.c_int32,  # int
        ]
    ),
    "Engine::UnsubscribeFromPreAdditiveCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cPreAdditiveCallback *
        ]
    ),
    "Engine::IsSubscribedToPreAdditiveCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cPreAdditiveCallback *
        ]
    ),
    "Engine::SubscribeToOnPostAnimCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cOnPostAnimCallback *
            ctypes.c_int32,  # int
        ]
    ),
    "Engine::UnsubscribeFromOnPostAnimCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cOnPostAnimCallback *
        ]
    ),
    "Engine::IsSubscribedToOnPostAnimCallback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cOnPostAnimCallback *
        ]
    ),
    "Engine::SwitchToEditedGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::UpdateModelGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::UpdateModelLODsImmediate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "Engine::SetLODsUpdateEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Engine::RequestStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResource *
            ctypes.c_ulonglong,  # const cTkResourceDescriptor *
        ]
    ),
    "Engine::RequestStreamNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "Engine::DoesNodeUseJoint": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::GetWindEigenvectorSet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkWindEigenvectors *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::SetWindEigenvectorSet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkWindEigenvectors *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkWindEigenvectors *
        ]
    ),
    "Engine::DoesHeadNodeUseJoint": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::SetupCameraView": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Engine::SetupCameraViewOrthographic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "Engine::SetCameraCustomViewport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkVector4 *
        ]
    ),
    "Engine::SetCameraCustomScissor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkVector4 *
        ]
    ),
    "Engine::SetCameraJumped": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::SetCameraVelFromPrevMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "Engine::ClearCameraBlurAndJitter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::SetBlurParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_double,  # double
            ctypes.c_double,  # double
        ]
    ),
    "Engine::ClearNodeMotion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::GetCameraProjMat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkMatrix44 *
        ]
    ),
    "Engine::GetCameraViewMat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkMatrix44 *
        ]
    ),
    "Engine::InFrustum": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "Engine::SetRenderSpaceOffset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "Engine::RunMultiThreaded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Engine::CleanupMultithreadedPointers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Engine *
        ]
    ),
    "Engine::TempGetNewRenderContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache **
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Engine::EncodeVertexIntoStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkSkinnedVertexData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "cTkLinearHashTable<int,GeometryStreaming::cEgGeometryStreamer::cMeshDescriptorData,-1,0,cTkLinearHashTableHash<int>>::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<int,GeometryStreaming::cEgGeometryStreamer::cMeshDescriptorData,-1,0,cTkLinearHashTableHash<int> > *
        ]
    ),
    "cTkTypedSmartResHandle<cEgMaterialResource>::cTkTypedSmartResHandle<cEgMaterialResource>": {
        "cTkTypedSmartResHandle<cEgMaterialResource> *, const cEgMaterialResource *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkTypedSmartResHandle<cEgMaterialResource> *
                ctypes.c_ulonglong,  # const cEgMaterialResource *
            ]
        ),
        "cTkTypedSmartResHandle<cEgMaterialResource> *, const cTkTypedSmartResHandle<cEgMaterialResource> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkTypedSmartResHandle<cEgMaterialResource> *
                ctypes.c_ulonglong,  # const cTkTypedSmartResHandle<cEgMaterialResource> *
            ]
        ),
    },
    "cEgRBObjects<cTkBuffer,8>::GetBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # cEgRBObjects<cTkBuffer,8> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkLinearHashTable<unsigned __int64,TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,6,0,cTkLinearHashTableHash<unsigned __int64>>::ChangeNodeCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<unsigned __int64,TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,6,0,cTkLinearHashTableHash<unsigned __int64> > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLinearHashTable<int,GeometryStreaming::cEgGeometryStreamer::cMeshDescriptorData,-1,0,cTkLinearHashTableHash<int>>::ChangeNodeCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<int,GeometryStreaming::cEgGeometryStreamer::cMeshDescriptorData,-1,0,cTkLinearHashTableHash<int> > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cEgRenderActionQueue::EnqueueAction<cEgRenderActionSetSamplerDefaultTexture>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionQueue *
            ctypes.c_ulonglong,  # cEgRenderActionSetSamplerDefaultTexture *
        ]
    ),
    "cEgRenderer::GetRenderSpaceOffset<0>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "Engine::GetNodeRelativeTransMatrix<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "Engine::ComputeNodeTransMatInOffsetSpace<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "Engine::GetNodeName<0>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const TryStoreMode *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Engine::GetRenderSpaceOffset<0>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cTkGraphicsTempBuffer::Malloc<cTkGraphicsThreadTask>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgModelNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsTempBuffer *
        ]
    ),
    "cEgRenderActionSetResParam::cEgRenderActionSetResParam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetResParam *
            ctypes.c_ulonglong,  # const cEgRenderActionSetResParam *
        ]
    ),
    "cEgRenderActionSetMaterialUniform::cEgRenderActionSetMaterialUniform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetMaterialUniform *
            ctypes.c_ulonglong,  # const cEgRenderActionSetMaterialUniform *
        ]
    ),
    "cEgRenderActionSetMaterialUniformArray::cEgRenderActionSetMaterialUniformArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetMaterialUniformArray *
            ctypes.c_ulonglong,  # const cEgRenderActionSetMaterialUniformArray *
        ]
    ),
    "cEgRenderActionSetMaterialSampler::cEgRenderActionSetMaterialSampler": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetMaterialSampler *
            ctypes.c_ulonglong,  # const cEgRenderActionSetMaterialSampler *
        ]
    ),
    "cEgRenderActionBlitTexture::cEgRenderActionBlitTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionBlitTexture *
            ctypes.c_ulonglong,  # const cEgRenderActionBlitTexture *
        ]
    ),
    "cEgRenderActionSetSamplerDefaultTexture::cEgRenderActionSetSamplerDefaultTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetSamplerDefaultTexture *
            ctypes.c_ulonglong,  # const cEgRenderActionSetSamplerDefaultTexture *
        ]
    ),
    "cEgAnimationController::AnimMapping::AnimMapping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationController::AnimMapping *
            ctypes.c_ulonglong,  # const cEgAnimationController::AnimMapping *
        ]
    ),
    "cTkResourceDescriptor::HashAltID": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "cEgMaterialSampler::SetName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialSampler *
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
        ]
    ),
    "PCBoolParam::PCBoolParam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PCBoolParam *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "PCIntParam::PCIntParam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PCUIntParam *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "PCVoidVec4Param::PCVoidVec4Param": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PCVoidVec4Param *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "PCVoidVec4Param::Set": {
        "PCVoidVec4Param *, unsigned int, unsigned int, unsigned int, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # PCVoidVec4Param *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "PCVoidVec4Param *, float, float, float, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # PCVoidVec4Param *
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
    },
    "PCFloatParam::PCFloatParam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PCFloatParam *
            ctypes.c_float,  # float
        ]
    ),
    "PCStringParam::PCStringParam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PCStringParam *
            ctypes.c_ulonglong,  # const std::string *
        ]
    ),
    "PCID128Param::PCID128Param": {
        "PCID128Param *, const TkID<128> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # PCID128Param *
                ctypes.c_char_p,  # const TkID<128> *
            ]
        ),
        "PCID128Param *, const std::string *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # PCID128Param *
                ctypes.c_ulonglong,  # const std::string *
            ]
        ),
    },
    "PCID128HashedParam::PCID128HashedParam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PCID128HashedParam *
            ctypes.c_ulonglong,  # const std::string *
        ]
    ),
    "cEgPipelineCommand::cEgPipelineCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineCommand *
            ctypes.c_uint32,  # PipelineCommands::List
        ]
    ),
    "cEgRenderTarget::cEgRenderTarget": {
        "cEgRenderTarget *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cEgRenderTarget *
            ]
        ),
        "cEgRenderTarget *, const cEgRenderTarget *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cEgRenderTarget *
                ctypes.c_ulonglong,  # const cEgRenderTarget *
            ]
        ),
        "cEgRenderTarget *, cEgRenderTarget *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cEgRenderTarget *
                ctypes.c_ulonglong,  # cEgRenderTarget *
            ]
        ),
    },
    "cTkResourceManager::SetResourceLoaded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceManager *
            ctypes.c_ulonglong,  # cTkResource *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgMaterialResource::cEgMaterialResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "cEgMaterialResource::OnResourceLoaded": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
        ]
    ),
    "cEgMaterialResource::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkResource *
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
        ]
    ),
    "cEgMaterialResource::CloneInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_ulonglong,  # const cTkResource *
        ]
    ),
    "cEgMaterialResource::InitDefault": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
        ]
    ),
    "cEgMaterialResource::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
        ]
    ),
    "cEgMaterialResource::ParseNode": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_ulonglong,  # cTkMaterialData *
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cTkMaterialUniform,16,-1> *
        ]
    ),
    "cEgMaterialSampler::~cEgMaterialSampler": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialSampler *
        ]
    ),
    "cEgMaterialResource::GetFlagsForSampler": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_ulonglong,  # cTkMaterialSampler *
        ]
    ),
    "cEgMaterialResource::Load": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgMaterialResource::SetUniformArray": {
        "cEgMaterialResource *, const char *, const float *, int": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cEgMaterialResource *
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # const float *
                ctypes.c_int32,  # int
            ]
        ),
        "cEgMaterialResource *, unsigned __int64, const float *, int": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cEgMaterialResource *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # const float *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cEgMaterialResource::GetUniform": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cEgMaterialResource::SetSampler": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cEgTextureResource *
        ]
    ),
    "cEgMaterialResource::GetElementCount": FUNCDEF(
        restype=ctypes.c_int64,  # signed __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgMaterialResource::GetElementParamInt": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cEgMaterialResource::SetElementParamInt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cEgMaterialResource::GetElementParamFloat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cEgMaterialResource::SetElementParamFloat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cEgMaterialResource::GetElementParamStr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMaterialUniform *
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgMaterialResource::SetElementParamStr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # cTkLinearHashTable<unsigned __int64,int,-1,0,cTkHashTableHashPassthrough>::cTkListNode **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cEgMaterialResource::AllocateUniformBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
        ]
    ),
    "cEgMaterialResource::CreateStaticMaterialHash": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
        ]
    ),
    "cEgModelNode::cEgModelNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_ulonglong,  # const cEgModelNodeTemplate *
        ]
    ),
    "cEgAnimationController::~cEgAnimationController": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgAnimationController *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cEgAnimationController::JointLODGroup,8,-1> *
        ]
    ),
    "cEgModelNode::~cEgModelNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cEgAnimationController::JointLODGroup,8,-1> *
        ]
    ),
    "cEgModelNode::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgModelNode::ParseRefAttrib": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_ulonglong,  # cTkSceneNodeAttributeData *
        ]
    ),
    "cEgModelNode::FactoryFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "cEgModelNode::RecreateNodeListRec": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
        ]
    ),
    "cEgModelNode::CacheSkeleton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
        ]
    ),
    "cEgModelNode::RecreateNodeList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
        ]
    ),
    "cEgModelNode::GetLocatorForAnim": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_ulonglong,  # cEgAnimationResource *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkQuaternion *
        ]
    ),
    "cEgModelNode::UpdateLocalMeshAABBs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
        ]
    ),
    "cEgModelNode::RecalculateLocalMeshAABB": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_ulonglong,  # cEgGeometryAABBCache *
            ctypes.c_ulonglong,  # cEgMeshNode *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cEgModelNode::GetParameterInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgModelNode::SetParameterInt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgModelNode::GetParameterFloat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgModelNode::SetParameterFloat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cEgModelNode::GetParameterVec3": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgModelNode::GetSkinnedVert": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cEgModelNode::SwitchToEditedGeometry": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
        ]
    ),
    "cEgModelNode::UpdateGeometry": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
        ]
    ),
    "cEgModelNode::CalculateLodLevel": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cEgModelNode::PrepAnimate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
        ]
    ),
    "cEgModelNode::Animate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
        ]
    ),
    "cEgModelNode::BuildMatFromAnimData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_ulonglong,  # const AnimTransform *
        ]
    ),
    "cEgModelNode::BuildSkinningMats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
        ]
    ),
    "cEgModelNode::FinishAnimate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
        ]
    ),
    "cEgModelNode::PreAsyncUpdate": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
        ]
    ),
    "cEgModelNode::UpdateLod_HasLods": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_float,  # float
        ]
    ),
    "cEgModelNode::UpdateLod_HasLodsImmediate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
        ]
    ),
    "cEgModelNode::UpdateLod": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cEgModelNode::AsyncUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
        ]
    ),
    "cEgModelNode::PostAsyncUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
        ]
    ),
    "cEgModelNode::OnAttach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "cEgModelNode::OnSubNodeAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgModelNode *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "cEgLodNodes::AddNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgLodNodes *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cEgRenderableSceneNode *
        ]
    ),
    "cEgLodNodes::GetGenericMeshName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cEgLodNodes::CheckMeshesMatch": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
        ]
    ),
    "cEgModules::Initialise": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cEgModules::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cEgPipelineResource::InitDefault": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineResource *
        ]
    ),
    "cEgPipelineResource::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineResource *
        ]
    ),
    "cEgPipelineResource::RaiseError": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineResource *
            ctypes.c_ulonglong,  # const std::string *
            ctypes.c_int32,  # int
        ]
    ),
    "PlatformCheck": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "GetGroupEnabledFromId": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const std::string *
        ]
    ),
    "cEgPipelineResource::ParseStage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const std::string *
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineResource *
            ctypes.c_ulonglong,  # const std::string *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ulonglong,  # cEgPipelineStage *
        ]
    ),
    "cEgPipelineCommand::~cEgPipelineCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineCommand *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<cTkTypedSmartResHandle<cTkResource>,8,-1> *
        ]
    ),
    "cEgPipelineResource::ParseAndAddRenderTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineResource *
            ctypes.c_ulonglong,  # const XMLNode *
        ]
    ),
    "cEgPipelineResource::ParseRenderTargetGroup": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineResource *
            ctypes.c_ulonglong,  # const XMLNode *
        ]
    ),
    "cEgRenderTarget::~cEgRenderTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderTarget *
        ]
    ),
    "cEgPipelineResource::FindRenderTarget": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgRenderTarget *
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineResource *
            ctypes.c_ulonglong,  # const std::string *
        ]
    ),
    "cEgPipelineResource::EnableRenderTargets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineResource *
        ]
    ),
    "cEgPipelineResource::CreateRenderTargets": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineResource *
        ]
    ),
    "cEgPipelineResource::CreateDelayedTempRenderTargets": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineResource *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cEgPipelineResource::ReleaseRenderTargets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineResource *
        ]
    ),
    "cEgPipelineResource::GetRenderTargetFinalScale": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineResource *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # const cEgRenderTarget *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cEgPipelineResource::Load": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgPipelineResource::GetElementCount": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineResource *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgPipelineResource::GetElementParamInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgPipelineResource::SetElementParamInt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cEgPipelineResource::GetElementParamStr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const TryStoreMode *
        argtypes=[
            ctypes.c_ulonglong,  # cEgPipelineResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgFlowParseStack::Pop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgFlowParseStack *
        ]
    ),
    "cEgFlowParseStack::Push": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgFlowParseStack *
        ]
    ),
    "cEgFlowParseStack::Set": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgFlowParseStack *
            ctypes.c_uint32,  # cEgFlowParseStack::Branch
            ctypes.c_int32,  # int
        ]
    ),
    "cEgFlowParseStack::Bind": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgFlowParseStack *
            ctypes.c_uint32,  # cEgFlowParseStack::Branch
            ctypes.c_ulonglong,  # PCIntParam *
        ]
    ),
    "cEgFlowParseStack::Bound": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cEgFlowParseStack *
            ctypes.c_uint32,  # cEgFlowParseStack::Branch
        ]
    ),
    "ParseIf": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::string *
        argtypes=[
            ctypes.c_ulonglong,  # const std::string *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ulonglong,  # cEgPipelineStage *
        ]
    ),
    "ParseCondition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::string *
        argtypes=[
            ctypes.c_ulonglong,  # const std::string *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cEgPipelineStage *
        ]
    ),
    "cEgFrustum::BuildViewFrustum": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgFrustum *
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cEgFrustum::CalculateFrustumCorners": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cEgFrustum::CalculateShadowFrustum": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgFrustum *
        argtypes=[
            ctypes.c_ulonglong,  # cEgFrustum *
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "TransposeFrustum": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgTransposedFrustum *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTypedSmartResHandle<cTkResource>::cTkTypedSmartResHandle<cTkResource>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTypedSmartResHandle<cTkResource> *
            ctypes.c_ulonglong,  # const cTkSmartResHandle *
        ]
    ),
    "cTkDynamicArray<cTkVector4>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkVector4> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkLinearHashTable<unsigned __int64,int,-1,0,cTkHashTableHashPassthrough>::insert": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<unsigned __int64,int,-1,0,cTkHashTableHashPassthrough> *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_ulonglong,  # const int *
        ]
    ),
    "cTkLinearHashTable<unsigned __int64,int,-1,0,cTkHashTableHashPassthrough>::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<unsigned int,cCachedVertexBufferEntry *,-1,0,cTkLinearHashTableHash<unsigned int> > *
        ]
    ),
    "cTkLinearHashTable<unsigned __int64,int,-1,0,cTkHashTableHashPassthrough>::InsertNodeInBucket": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<unsigned __int64,unsigned int,-1,0,cTkHashTableHashPassthrough> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkLinearHashTable<unsigned __int64,unsigned int,-1,0,cTkHashTableHashPassthrough>::cTkListNode *
        ]
    ),
    "cTkLinearHashTable<unsigned __int64,int,-1,0,cTkHashTableHashPassthrough>::ChangeNodeCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<unsigned __int64,int,-1,0,cTkHashTableHashPassthrough> *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLinearHashTable<unsigned __int64,int,-1,0,cTkHashTableHashPassthrough>::SetupFreeList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<unsigned __int64,int,-1,0,cTkHashTableHashPassthrough> *
        ]
    ),
    "cTkLinearHashTable<unsigned __int64,int,-1,0,cTkHashTableHashPassthrough>::SetBucketTableSize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<unsigned __int64,int,-1,0,cTkHashTableHashPassthrough> *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgShaderCombination::~cEgShaderCombination": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderCombination *
        ]
    ),
    "cEgMaterialSampler::cEgMaterialSampler": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialSampler *
            ctypes.c_ulonglong,  # const cEgMaterialSampler *
        ]
    ),
    "cTkMaterialUniform::cTkMaterialUniform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialUniform *
            ctypes.c_ulonglong,  # const cTkMaterialUniform *
        ]
    ),
    "cEgShaderCombination::cEgShaderCombination": {
        "cEgShaderCombination *, cEgShaderCombination *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cEgShaderCombination *
                ctypes.c_ulonglong,  # cEgShaderCombination *
            ]
        ),
        "cEgShaderCombination *, const cEgShaderCombination *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cEgShaderCombination *
                ctypes.c_ulonglong,  # const cEgShaderCombination *
            ]
        ),
    },
    "TkShaderBufferHandle::TkShaderBufferHandle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkShaderBufferHandle *
        ]
    ),
    "cTkShaderAsyncLoadData::Cleanup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkShaderAsyncLoadData *
        ]
    ),
    "cEgCodeResource::cEgCodeResource": {
        "cEgCodeResource *, const cEgCodeResource *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cEgCodeResource *
                ctypes.c_ulonglong,  # const cEgCodeResource *
            ]
        ),
        "cEgCodeResource *, const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cEgCodeResource *
                ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cEgShaderContext::cEgShaderContext": {
        "cEgShaderContext *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cEgShaderContext *
            ]
        ),
        "cEgShaderContext *, const cEgShaderContext *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cEgShaderContext *
                ctypes.c_ulonglong,  # const cEgShaderContext *
            ]
        ),
    },
    "cTkTexture::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
        ]
    ),
    "cTkTexture::ClearMipStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "cEgTextureResource::GetDataSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
        ]
    ),
    "cEgTextureResource::EvictOneMip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
        ]
    ),
    "cEgTextureResource::AbortEviction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
        ]
    ),
    "cEgCodeResource::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkResource *
        argtypes=[
            ctypes.c_ulonglong,  # cEgCodeResource *
        ]
    ),
    "cEgCodeResource::CloneInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgCodeResource *
            ctypes.c_ulonglong,  # const cTkResource *
        ]
    ),
    "cEgCodeResource::InitDefault": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgCodeResource *
        ]
    ),
    "cEgCodeResource::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgCodeResource *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # TkSTLAllocatorShim<std::pair<cTkTypedSmartResHandle<cEgCodeResource>,cEgCodeResource::cEgShaderSize>,8,-1> *
        ]
    ),
    "cEgCodeResource::Load": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgCodeResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgCodeResource::HasDependency": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgCodeResource *
            ctypes.c_ulonglong,  # cEgCodeResource *
        ]
    ),
    "cEgCodeResource::UpdateShaders": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgCodeResource *
        ]
    ),
    "Tokenizer::seekChar": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # Tokenizer *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "Tokenizer::getNextToken": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tokenizer *
        ]
    ),
    "Tokenizer::checkToken": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # Tokenizer *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "Tokenizer::getToken": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # Tokenizer *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cEgShaderResource::cEgShaderResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderResource *
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgShaderResource::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderResource *
        ]
    ),
    "cEgShaderResource::RaiseError": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderResource *
            ctypes.c_ulonglong,  # const std::string *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgShaderResource::ParseFXSection": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderResource *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cEgShaderSampler::~cEgShaderSampler": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderSampler *
        ]
    ),
    "cEgShaderContext::~cEgShaderContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderContext *
        ]
    ),
    "cTkStackVector<cTkVector4,1,-1>::~cTkStackVector<cTkVector4,1,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStackVector<cTkVector4,1,-1> *
        ]
    ),
    "StackContainer<std::vector<cEgSMShaderContext::Conn,StackAllocator<cEgSMShaderContext::Conn,4,-1>>,4,-1>::~StackContainer<std::vector<cEgSMShaderContext::Conn,StackAllocator<cEgSMShaderContext::Conn,4,-1>>,4,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cEgSMShaderContext::Conn,StackAllocator<cEgSMShaderContext::Conn,4,-1> >,4,-1> *
        ]
    ),
    "cEgShaderResource::Load": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgShaderResource::PreLoadCombination": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderResource *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CreateShaderNames": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # const cEgShaderCombination *
            ctypes.c_ulonglong,  # const cEgShaderContext *
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "cEgShaderResource::CompileShader": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderResource *
            ctypes.c_ulonglong,  # cEgShaderCombination *
        ]
    ),
    "cEgShaderResource::CompileCombination": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderResource *
            ctypes.c_ulonglong,  # cEgShaderContext *
            ctypes.c_ulonglong,  # cEgShaderCombination *
        ]
    ),
    "cEgShaderResource::StartAsyncCompileCombination": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderResource *
            ctypes.c_ulonglong,  # cEgShaderContext *
            ctypes.c_ulonglong,  # cEgShaderCombination *
        ]
    ),
    "cEgShaderResource::FinishAsyncCompileCombination": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderResource *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cEgShaderCombination *
            ctypes.c_ulonglong,  # cTkStackVector<cEgShaderResource::PipelineCompilationTask,64,-1> *
        ]
    ),
    "cEgShaderResource::AddToLoadingList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderResource *
        ]
    ),
    "cEgShaderResource::AppendUncompiledContextsToCompilationTaskList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderResource *
            ctypes.c_ulonglong,  # cTkStackVector<cEgShaderResource::CombinationCompilationTask,64,-1> *
        ]
    ),
    "cEgShaderResource::PollToExecuteCompilationTaskList": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkStackVector<cEgShaderResource::CombinationCompilationTask,64,-1> *
            ctypes.c_ulonglong,  # TkIterationState<1> *
        ]
    ),
    "cEgShaderResource::CompileContextsSynchronous": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderResource *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cEgShaderResource::GetElementCount": FUNCDEF(
        restype=ctypes.c_int64,  # signed __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderResource *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgShaderResource::GetElementParamFloat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cEgShaderResource::SetElementParamFloat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cEgShaderResource::GetElementParamStr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgShaderContext *
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgShaderPreloadList::Read": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderPreloadList *
        ]
    ),
    "cEgTextureResource::InitialisationFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cEgTextureResource::cEgTextureResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "cEgTextureResource::Initialise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgTextureResource::InitDefault": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
        ]
    ),
    "cEgTextureResource::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
        ]
    ),
    "cEgTextureResource::SearchCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
        ]
    ),
    "cEgTextureResource::Load": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgTextureResource::CopyResourceFlagsToTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
        ]
    ),
    "cEgTextureResource::LoadFromDds": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgTextureResource::GetMipCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
        ]
    ),
    "cEgTextureResource::GetElementCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgTextureResource::GetElementParamInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgTextureResource::SetElementParamInt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cEgTextureResource::MapResStream": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgTextureResource::UnmapStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgTextureResource::GetElementParamFloat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cEgTextureResource::CalculateAverageColour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgTextureResource::NotifyResidentToMip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgTextureResource::AddToStreamList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTextureResource *
        ]
    ),
    "cEgTextureResource::PanicEvict": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgTextureResource::CheckForDeletedStreamingTextures": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cEgTextureResource::OutOfMemoryCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
        ]
    ),
    "EvictTex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgTextureResource::UpdateStreaming": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkMipReportBuffer *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgRBObjects<cEgRenderBuffer,8>::GetBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgRenderBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # cEgRBObjects<cEgRenderBuffer,8> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkLinearHashTable<unsigned __int64,int,-1,256,cTkHashTableHashPassthrough>::ChangeNodeCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<unsigned __int64,unsigned int,-1,0,cTkHashTableHashPassthrough> *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cEgShaderPreloadList::ShaderContext::ShaderContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderPreloadList::ShaderContext *
            ctypes.c_ulonglong,  # cEgShaderPreloadList::ShaderContext *
        ]
    ),
    "cEgSMShaderContext::cEgSMShaderContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSMShaderContext *
            ctypes.c_ulonglong,  # const cEgSMShaderContext *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cEgSMShaderContext::Conn,4,-1> *
        ]
    ),
    "cEgShaderUniform::cEgShaderUniform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderUniform *
            ctypes.c_ulonglong,  # const cEgShaderUniform *
        ]
    ),
    "cEgShaderSampler::cEgShaderSampler": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderSampler *
            ctypes.c_ulonglong,  # const cEgShaderSampler *
        ]
    ),
    "cTkGnmxContext::EndRenderPass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGnmxContext *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgDrawScreenQuad::Draw": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgFfxFsr2Pass::Dispatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgFfxFsr2Draw::Draw": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgDrawGeometry::Draw": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgDrawLightGeometry::Draw": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgDrawLightShapes::Draw": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgDrawLightShapesMulti::Draw": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgDrawShadowGeometry::Draw": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgDrawBboxes::Draw": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "AVX_memcpy16_aligned": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "RunGraphicsTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkGraphicsAPI::SetStencilMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_uint8,  # const unsigned __int8
        ]
    ),
    "cTkGraphicsAPI::SetCheckerboard": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkGraphicsAPI::SetSamplingRate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkGraphicsAPI::SetCullMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgSceneManager::AllocRenderQueue": FUNCDEF(
        restype=ctypes.c_uint64,  # EgRenderQueueHandle
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneManager *
            ctypes.c_uint64,  # EgRenderQueueHandle
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgRenderQueueInfo::cEgRenderQueueInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
        ]
    ),
    "cEgRenderCameraInfo::cEgRenderCameraInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderCameraInfo *
        ]
    ),
    "cEgRenderActionRender::MoveTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionRender *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgRenderActionRender::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionRender *
            ctypes.c_uint32,  # eEgRenderParity
        ]
    ),
    "cEgRenderActionSetResParam::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetResParam *
            ctypes.c_uint32,  # eEgRenderParity
        ]
    ),
    "cEgRenderActionSetMaterialUniform::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetMaterialUniform *
            ctypes.c_uint32,  # eEgRenderParity
        ]
    ),
    "cEgRenderActionSetMaterialUniformArray::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetMaterialUniformArray *
            ctypes.c_uint32,  # eEgRenderParity
        ]
    ),
    "cEgRenderActionSetMaterialSampler::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetMaterialSampler *
            ctypes.c_uint32,  # eEgRenderParity
        ]
    ),
    "cEgRenderActionClearMaterialSamplers::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionClearMaterialSamplers *
            ctypes.c_uint32,  # eEgRenderParity
        ]
    ),
    "cEgRenderActionFlushTexture::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionFlushTexture *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgRenderActionSetupViewport::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetupViewport *
            ctypes.c_uint32,  # eEgRenderParity
        ]
    ),
    "cEgRenderActionSetRenderTarget::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetRenderTarget *
            ctypes.c_uint32,  # eEgRenderParity
        ]
    ),
    "cEgRenderActionSetSamplerDefaultTexture::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionSetSamplerDefaultTexture *
            ctypes.c_uint32,  # eEgRenderParity
        ]
    ),
    "cEgRenderer::cEgRenderer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
        ]
    ),
    "StackContainer<std::vector<cEgShaderResource::PipelineCompilationTask,StackAllocator<cEgShaderResource::PipelineCompilationTask,64,-1>>,64,-1>::~StackContainer<std::vector<cEgShaderResource::PipelineCompilationTask,StackAllocator<cEgShaderResource::PipelineCompilationTask,64,-1>>,64,-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<cEgShaderResource::PipelineCompilationTask,StackAllocator<cEgShaderResource::PipelineCompilationTask,64,-1> >,64,-1> *
        ]
    ),
    "cEgRenderer::~cEgRenderer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
        ]
    ),
    "cEgRenderer::UseScratchBuf": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cEgRenderer::Initialise": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cEgRendererBase::Resize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgRenderer::ProcessInFlightShaderCombinations": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
        ]
    ),
    "cEgRenderer::ProcessInFlightMaterials": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
        ]
    ),
    "cEgRenderer::SetMaterialTextures": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderResource *
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_ulonglong,  # cEgShaderCombination *
            ctypes.c_ulonglong,  # cEgShaderUniformBufferBinding *
            ctypes.c_char_p,  # const TkIDHashed<128> *
            ctypes.c_ulonglong,  # const cEgThreadableRenderCall::cRendererData *
        ]
    ),
    "cEgRenderer::SetLightingUniforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cEgShaderCombination *
            ctypes.c_ulonglong,  # cEgShaderUniformBufferBinding *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "cEgRenderer::SetMaterialUniforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_ulonglong,  # cEgShaderResource *
            ctypes.c_ulonglong,  # cEgShaderUniformBufferBinding *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "cEgRenderer::BindPipeBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_char,  # char
            ctypes.c_uint32,  # eTextureAddressMode
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eFrameIdx
        ]
    ),
    "cEgRenderer::DrawRenderables": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueue *
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const std::string *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgRenderer::SetupMeshGeometry": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNode *
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # const cTkVertexLayoutRT **
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkRenderStateCache *
        ]
    ),
    "cEgRenderer::SetupMeshMaterial": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgMeshNode *
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_char_p,  # const TkIDHashed<128> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkVector4 *
            ctypes.c_ulonglong,  # cTkVector4 *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # cEgShaderUniformBufferBinding *
        ]
    ),
    "cEgRenderer::DrawImposterMeshes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # RenderingOrder::List
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgRenderer::DrawMeshes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # RenderingOrder::List
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "cEgRenderer::DrawMeshesBboxes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # cTkResource *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "cEgRenderer::PopulateCameraInfoFromNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_ulonglong,  # cEgRenderCameraInfo *
            ctypes.c_ulonglong,  # cEgCameraNode *
        ]
    ),
    "cEgRenderer::EnqueueRender": FUNCDEF(
        restype=ctypes.c_uint64,  # EgRenderQueueHandle
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_ulonglong,  # cEgCameraNode **
            ctypes.c_ulonglong,  # cEgCameraNode **
            ctypes.c_uint64,  # EgRenderQueueHandle
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cEgRenderer::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_ulonglong,  # const EgRenderQueueHandle *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cEgRenderer::FinalizeFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgRenderer::SetShaderContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cEgShaderContext *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgRenderer::KickUpdateShadowMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
            ctypes.c_ulonglong,  # cEgCameraNode *
            ctypes.c_ulonglong,  # cEgLightNode *
            ctypes.c_ulonglong,  # const cEgSceneNodeFlags *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgSceneNode::cEgSceneNode": {
        "cEgSceneNode *, const cEgSceneNode *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cEgSceneNode *
                ctypes.c_ulonglong,  # const cEgSceneNode *
            ]
        ),
        "cEgSceneNode *, const cEgSceneNodeTemplate *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cEgSceneNode *
                ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ]
        ),
    },
    "cEgRenderer::SetShadowViewMatrices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "cEgRenderer::ExecutePipelineCommand": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_ulonglong,  # cEgPipelineCommand *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cEgFlowRunStack *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # cTkRenderStateCache *
        ]
    ),
    "cEgRenderer::CreatePrimitives": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
        ]
    ),
    "cEgRenderer::DrawSphere_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_ulonglong,  # cTkMatrix44 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_ulonglong,  # cEgShaderUniformBufferBinding *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgRenderer::DrawShape_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_ulonglong,  # cTkMatrix44 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_ulonglong,  # cEgShaderUniformBufferBinding *
        ]
    ),
    "cEgRenderer::SetPerRenderTargetUniforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_ulonglong,  # cEgShaderUniformBufferBinding *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgRenderer::CallPipelineCommand": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_ulonglong,  # cTkGraphicsThreadTask *
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgRenderer::RunStageMultiThreaded": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_ulonglong,  # const cEgPipelineStage *
        ]
    ),
    "cEgRenderer::ReleaseBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cEgRenderer::ReleaseUnusedBuffers": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
        ]
    ),
    "cEgRenderActionBlitTexture::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderActionBlitTexture *
            ctypes.c_uint32,  # eEgRenderParity
        ]
    ),
    "GetCameraInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgRenderCameraInfo *
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
            ctypes.c_int32,  # const int
        ]
    ),
    "cEgRendererBase::GetRenderBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgRenderBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # eFrameIdx
        ]
    ),
    "cEgMaterialResource::GetTextureSortHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
        ]
    ),
    "cEgSceneNodeTemplate::GatherNonReferenceData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # std::vector<cTkAttachmentData *,TkSTLAllocatorShim<cTkAttachmentData *,8,-1> > *
        ]
    ),
    "cTkBitAlloc::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitAlloc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBitAlloc::Alloc": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitAlloc *
        ]
    ),
    "cEgEventPool::~cEgEventPool": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgEventPool *
        ]
    ),
    "cEgEventPool::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void **
        argtypes=[
            ctypes.c_ulonglong,  # cEgEventPool *
        ]
    ),
    "cEgRenderQueueBuffer::DuplicateEntries": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgRenderQueueEntry **
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
        ]
    ),
    "cEgRenderQueueBuffer::CalculateDepthValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cEgPrivateManagedRenderData::cEgPrivateManagedRenderData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgPrivateManagedRenderData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cEgRenderQueue::PostCullUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueue *
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgRenderQueue::Finalise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueue *
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgRenderQueue::FinaliseJobFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgRenderQueue::FinaliseJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueue *
            ctypes.c_ulonglong,  # cEgRenderQueue::FinaliseJobParams *
        ]
    ),
    "cEgRenderQueue::GetSortedRenderQueue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueue *
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cEgFrustum *
        ]
    ),
    "cEgRenderQueue::KickSortsForPipeline": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueue *
            ctypes.c_ulonglong,  # const cEgPipelineResource *
            ctypes.c_ulonglong,  # const cEgFrustum *
        ]
    ),
    "cEgSceneGraphResource::IsLoaded": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneGraphResource *
        ]
    ),
    "cEgSceneGraphResource::GetAttribute": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneGraphResource *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
        ]
    ),
    "cEgSceneGraphResource::SetDescriptor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneGraphResource *
            ctypes.c_ulonglong,  # const cTkResourceDescriptor *
        ]
    ),
    "cEgSceneGraphResource::GatherMaterials": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneGraphResource *
            ctypes.c_ulonglong,  # std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *
        ]
    ),
    "cEgSceneGraphResource::GatherData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneGraphResource *
            ctypes.c_ulonglong,  # std::vector<cTkAttachmentData *,TkSTLAllocatorShim<cTkAttachmentData *,8,-1> > *
        ]
    ),
    "cEgReferenceNodeTemplate::SetDescriptor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgReferenceNodeTemplate *
            ctypes.c_ulonglong,  # const cTkSharedPtr<cTkResourceDescriptor const > *
        ]
    ),
    "cEgReferenceNodeTemplate::GatherMaterials": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgReferenceNodeTemplate *
            ctypes.c_ulonglong,  # std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *
        ]
    ),
    "cEgReferenceNodeTemplate::GetBoundingBox": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAABB *
        argtypes=[
            ctypes.c_ulonglong,  # cEgReferenceNodeTemplate *
            ctypes.c_ulonglong,  # cTkAABB *
            ctypes.c_ulonglong,  # const cTkResourceDescriptor *
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgReferenceNodeTemplate::GatherData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgReferenceNodeTemplate *
            ctypes.c_ulonglong,  # std::vector<cTkAttachmentData *,TkSTLAllocatorShim<cTkAttachmentData *,8,-1> > *
        ]
    ),
    "cEgResource::Load": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgResource::GetElementCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgResource *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgResource::GetElementParamInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgResource::SetElementParamInt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgResource::GetElementParamFloat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cEgResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgResource::SetElementParamFloat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgResource::GetElementParamStr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const TryStoreMode *
        argtypes=[
            ctypes.c_ulonglong,  # cEgResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgResource::SetElementParamStr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgResource::MapResStream": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # cEgResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgResource::UnmapStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgResource *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgRenderableSceneNode::cEgRenderableSceneNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderableSceneNode *
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # const cTkTypedSmartResHandle<cEgMaterialResource> *
        ]
    ),
    "cEgRenderableSceneNode::RecalculateRenderID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderableSceneNode *
        ]
    ),
    "cEgRenderableSceneNode::OnAttach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderableSceneNode *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "cEgRenderableSceneNode::OnDetach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderableSceneNode *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "cEgSceneNode::~cEgSceneNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "cEgSceneNode::SetTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cEgSceneNode::GetParameterInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgSceneNode::SetParameterInt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgSceneNode::GetParameterFloat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgSceneNode::SetParameterFloat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
        ]
    ),
    "cEgSceneNode::GetParameterVec3": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgSceneNode::SetParameterVec3": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cEgSceneNode::GetParameterStr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::string *
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgSceneNode::SetParameterStr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cEgSceneNode::GetVisibleBBox": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_ulonglong,  # cEgBoundingBox *
        ]
    ),
    "cEgSceneNode::SetHasAsyncUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgSceneNode::SetHasLods": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgSceneNode::SetRenderable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgSceneNode::ComputeTransformInOffsetSpace": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cEgSceneNode::ComputePrevTransformInOffsetSpace": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cEgGroupNode::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSharedPtr<cTkResourceDescriptor const > *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgGroupNode::FactoryFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNode *
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "cEgLocatorNode::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSharedPtr<cTkResourceDescriptor const > *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgLocatorNode::FactoryFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNode *
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "cEgLocatorNode::OnAttach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgLocatorNode *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "cEgLocatorNode::OnDetach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgLocatorNode *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "cEgSceneNodeTemplate::GatherModels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # std::vector<cEgModelNodeTemplate *,TkSTLAllocatorShim<cEgModelNodeTemplate *,8,-1> > *
        ]
    ),
    "cEgSceneGraphResource::cEgSceneGraphResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneGraphResource *
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "cEgSceneGraphResource::InitDefault": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneGraphResource *
        ]
    ),
    "cEgSceneGraphResource::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneGraphResource *
        ]
    ),
    "cEgSceneGraphResource::CloneInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneGraphResource *
            ctypes.c_ulonglong,  # const cTkResource *
        ]
    ),
    "cEgSceneGraphResource::ParseData": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneGraphResource *
            ctypes.c_ulonglong,  # cTkSceneNodeData *
            ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
        ]
    ),
    "cEgSceneGraphResource::Load": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneGraphResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "GetNodeCountRecursive": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgSceneGraphResource::SetElementParamFloat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneGraphResource *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
        ]
    ),
    "cEgSceneGraphResource::RegisterWithMetadataCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgSceneManager::cEgSceneManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneManager *
        ]
    ),
    "cEgSceneNodeData::~cEgSceneNodeData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
        ]
    ),
    "cEgPrivateManagedRenderData::~cEgPrivateManagedRenderData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgPrivateManagedRenderData *
        ]
    ),
    "cEgSceneManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneManager *
        ]
    ),
    "cEgSceneManager::RegisterType": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneManager *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # {'return': 'cEgSceneNodeTemplate *', 'args': ['const cTkDynamicArray<cTkSceneNodeAttributeData> *', 'cTkResourceDescriptor *', 'int']}
            ctypes.c_ulonglong,  # {'return': 'cEgSceneNode *', 'args': ['const cEgSceneNodeTemplate *', 'cTkSlotAlloc *']}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['cEgRenderQueueBuffer *', 'const TkID<128> *', 'int', 'bool', 'RenderingOrder::List', 'bool', 'bool', 'const cEgFrustum *', 'cEgThreadableRenderCall *']}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['cEgRenderQueueBuffer *', 'const TkID<128> *', 'int', 'bool', 'RenderingOrder::List', 'bool', 'bool', 'const cEgFrustum *', 'cEgThreadableRenderCall *']}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['cEgRenderQueueBuffer *', 'const TkID<128> *', 'cTkResource *', 'bool', 'float', 'const cEgFrustum *', 'cEgThreadableRenderCall *']}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['cEgRenderQueueInfo *', 'cEgUpdatableQueueEntry *', 'const cEgFrustum *', {'type': 'const', 'name': 'int'}, {'type': 'const', 'name': 'int'}]}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['cEgRenderQueueBuffer *', 'const TkID<128> *', 'int', 'bool', 'RenderingOrder::List', 'bool', 'bool', 'const cEgFrustum *', 'cEgThreadableRenderCall *', 'int']}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['cEgRenderQueueInfo *', 'cEgRenderQueueEntry *', 'cEgRenderableSceneNode **', 'const cEgFrustum *', 'int', 'int', 'cTkDependencyCounter *']}
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgSceneManager::FindType": {
        "cEgSceneManager *, int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cEgNodeRegistryEntry *
            argtypes=[
                ctypes.c_ulonglong,  # cEgSceneManager *
                ctypes.c_int32,  # int
            ]
        ),
        "cEgSceneManager *, const TkID<128> *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cEgNodeRegistryEntry *
            argtypes=[
                ctypes.c_ulonglong,  # cEgSceneManager *
                ctypes.c_char_p,  # const TkID<128> *
            ]
        ),
    },
    "cEgSceneManager::TransferPrivateRenderData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneManager *
        ]
    ),
    "cEgSceneManager::AddNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneManager *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "cEgSceneManager::RelocateNode": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneManager *
            ctypes.c_ulonglong,  # cEgSceneNode *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "cEgSceneManager::FindNodes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneManager *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cEgBaseNodeIterator *
        ]
    ),
    "cEgSceneManager::IterateNode": {
        "cEgSceneManager *, __int64, const char *, int, __int64 *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cEgSceneManager *
                ctypes.c_int64,  # __int64
                ctypes.c_char_p,  # const char *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['TkHandle'], 'pointer_depth': 1}
            ]
        ),
        "cEgSceneManager *, __int64, unsigned int, int, __int64 *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cEgSceneManager *
                ctypes.c_int64,  # __int64
                ctypes.c_uint32,  # unsigned int
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['TkHandle'], 'pointer_depth': 1}
            ]
        ),
    },
    "cEgSceneManager::FindAttachment": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgNodeAttachment *
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneManager *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
        ]
    ),
    "cEgRenderLightInfo::SetFromLight": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderLightInfo *
            ctypes.c_ulonglong,  # cEgLightNode *
        ]
    ),
    "cEgRenderLightInfo::SetFromQueueEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderLightInfo *
            ctypes.c_ulonglong,  # const cEgRenderQueueEntry *
        ]
    ),
    "cEgSceneNodeData::cEgSceneNodeData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
        ]
    ),
    "cEgSceneNodeData::ChangeNodeParent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cEgSceneNodeData::AddNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cEgSceneNodeData::UpdateAsync": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgSceneNodeData::UpdateLod": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgSceneNodeData::UpdateHierarchyFromRoot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cEgSceneNodeData::ForceUpdateNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cEgSceneNodeData::ForceUpdateRecurse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cEgSceneNodeData::UpdateBoundingBoxesAndTransformsFromRoot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgSceneNodeData::UpdateBoundingBoxesAndTransformsRecursive": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cEgBoundingBox *
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const VecIntrinsics::V128Matrix *
        ]
    ),
    "cEgSceneNodeData::FinishAsyncTransformUpdates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgSceneNodeData::SubmitAsyncTransformUpdates": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const VecIntrinsics::V128Matrix *
            ctypes.c_ulonglong,  # cEgUpdateTransformsWorkBuffer *
        ]
    ),
    "cEgSceneNodeData::AsyncTransformUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgSceneNodeData::CacheNodeFlags": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cEgSceneNodeFlags *
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
        ]
    ),
    "cEgSceneNodeData::KickUpdateQueuesJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgRenderQueue *
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # std::atomic<int> *
            ctypes.c_ulonglong,  # const cEgSceneNodeFlags *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cEgSceneNodeData::UpdateQueuesJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgSceneNodeData::SetRelativeTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cEgSceneNodeData::ComputeTransformInOffsetSpace": FUNCDEF(
        restype=ctypes.c_ulonglong,  # VecIntrinsics::V128Matrix *
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_ulonglong,  # VecIntrinsics::V128Matrix *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cEgSceneNodeData::ComputePrevTransformInOffsetSpace": FUNCDEF(
        restype=ctypes.c_ulonglong,  # VecIntrinsics::V128Matrix *
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_ulonglong,  # VecIntrinsics::V128Matrix *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cEgSceneNodeData::UpdateNodeNetworkIdLookup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cEgSceneNodeData::RemoveNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgSceneNodeData::ProcessDeferredRemoveNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
        ]
    ),
    "cEgSceneNodeData::DetachRecurse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgSceneNodeData::RemoveRecurse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgSceneNodeData::DeleteRemovedNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
        ]
    ),
    "cEgSceneNodeData::RequestAsyncUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cEgSceneNodeData::RecalculateAbsoluteTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgSceneNodeData::RecalculateAABBRecurse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const VecIntrinsics::V128Matrix *
        ]
    ),
    "cEgSceneNodeData::MarkSelfOrAncestorTransformDirtyRecurse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgSceneNodeData::MarkSelfOrDescendentsAABBDirty": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_int32,  # int
        ]
    ),
    "SwapIndexListFixup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::vector<int,TkSTLAllocatorShim<int,16,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgSceneNodeData::SwapNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgSceneNodeData::MoveChildrenTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgSceneNodeData::FullDefrag": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
        ]
    ),
    "cEgSceneNodeData::IncrementalDefrag": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
        ]
    ),
    "cEgUpdateTransformsWorkBuffer::CreateNextWorkload": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgUpdateTransformsWorkBuffer *
        ]
    ),
    "cEgUpdateTransformsWorkBuffer::Job": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgUpdateTransformsWorkBuffer *
            ctypes.c_ulonglong,  # cEgSceneNodeData *
        ]
    ),
    "cEgCullNodesWorkBuffer::Begin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgCullNodesWorkBuffer *
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgTransposedFrustum *
            ctypes.c_ulonglong,  # cEgTransposedFrustum *
            ctypes.c_ulonglong,  # const cEgSceneNodeFlags *
            ctypes.c_ulonglong,  # cEgRenderQueue *
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgCullNodesWorkBuffer::Submit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgCullNodesWorkBuffer *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const VecIntrinsics::V128Matrix *
            ctypes.c_ulonglong,  # const VecIntrinsics::V128Matrix *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # cEgFrustum::eCullResult
        ]
    ),
    "cTkMetadataCache<cTkAttachmentData>::LoadAsync": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cTkAttachmentData> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cCallbackData<cTkAttachmentData> *
        ]
    ),
    "cTkMetadataCache<cTkAttachmentData>::OnFinishLoadCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkMetadataCache<cTkAttachmentData>::FinishAsyncLoad": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBasePlacementRule *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cTkAttachmentData> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgSceneNodeData::SubmitCullNodeNoneRecursive<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_ulonglong,  # cEgTransposedFrustum *
            ctypes.c_ulonglong,  # cEgTransposedFrustum *
            ctypes.c_ulonglong,  # const cEgSceneNodeFlags *
            ctypes.c_ulonglong,  # cEgRenderQueue *
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
            ctypes.c_ulonglong,  # const VecIntrinsics::V128Matrix *
            ctypes.c_ulonglong,  # const VecIntrinsics::V128Matrix *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cEgCullNodesWorkBuffer *
        ]
    ),
    "cEgSceneNodeData::SubmitCullNodeNoneRecursive<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_ulonglong,  # cEgTransposedFrustum *
            ctypes.c_ulonglong,  # cEgTransposedFrustum *
            ctypes.c_ulonglong,  # const cEgSceneNodeFlags *
            ctypes.c_ulonglong,  # cEgRenderQueue *
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
            ctypes.c_ulonglong,  # const VecIntrinsics::V128Matrix *
            ctypes.c_ulonglong,  # const VecIntrinsics::V128Matrix *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cEgCullNodesWorkBuffer *
        ]
    ),
    "cEgSceneNodeData::SubmitCullNodeNoneRecursive<2>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_ulonglong,  # cEgTransposedFrustum *
            ctypes.c_ulonglong,  # cEgTransposedFrustum *
            ctypes.c_ulonglong,  # const cEgSceneNodeFlags *
            ctypes.c_ulonglong,  # cEgRenderQueue *
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
            ctypes.c_ulonglong,  # const VecIntrinsics::V128Matrix *
            ctypes.c_ulonglong,  # const VecIntrinsics::V128Matrix *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cEgCullNodesWorkBuffer *
        ]
    ),
    "cEgSceneNodeData::SubmitCullNodeNoneRecursive<-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeData *
            ctypes.c_ulonglong,  # cEgTransposedFrustum *
            ctypes.c_ulonglong,  # cEgTransposedFrustum *
            ctypes.c_ulonglong,  # const cEgSceneNodeFlags *
            ctypes.c_ulonglong,  # cEgRenderQueue *
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
            ctypes.c_ulonglong,  # const VecIntrinsics::V128Matrix *
            ctypes.c_ulonglong,  # const VecIntrinsics::V128Matrix *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cEgCullNodesWorkBuffer *
        ]
    ),
    "cEgSceneNodeData::AsyncCull<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgCullNodesWorkBuffer::Job<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgCullNodesWorkBuffer *
        ]
    ),
    "cEgCullNodesWorkBuffer::AddNodestoQueue<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgCullNodesWorkBuffer *
        ]
    ),
    "cEgSceneNodeData::AsyncCull<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgCullNodesWorkBuffer::Job<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgCullNodesWorkBuffer *
        ]
    ),
    "cEgSceneNodeData::AsyncCull<2>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgCullNodesWorkBuffer::Job<2>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgCullNodesWorkBuffer *
        ]
    ),
    "cEgSceneNodeData::AsyncCull<-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgCullNodesWorkBuffer::Job<-1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgCullNodesWorkBuffer *
        ]
    ),
    "cTkGnmxContext::SetNextRenderPass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGnmxContext *
            ctypes.c_ulonglong,  # TkRenderPassDesc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkRenderStateCache::cTkRenderStateCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
        ]
    ),
    "cEgRenderBuffer::cEgRenderBuffer": {
        "cEgRenderBuffer *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cEgRenderBuffer *
            ]
        ),
        "cEgRenderBuffer *, const cEgRenderBuffer *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cEgRenderBuffer *
                ctypes.c_ulonglong,  # const cEgRenderBuffer *
            ]
        ),
    },
    "cEgThreadableRenderCall::cEgThreadableRenderCall": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cEgRBObjects<cEgRenderBuffer,8>::~cEgRBObjects<cEgRenderBuffer,8>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRBObjects<cTkBuffer,8> *
        ]
    ),
    "cEgRendererBase::Initialise": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cEgRendererBase::CreateVertexBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const cTkVertexLayoutRT *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgRendererBase::CreateIndexBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgRendererBase::DoReleaseBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cEgRendererBase::CloneRenderBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cEgRendererBase::StealRenderBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_ulonglong,  # cFrameBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_ulonglong,  # cEgRenderBuffer **
        ]
    ),
    "cEgRendererBase::CreateRenderBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # RenderBufferFormats::List *
            ctypes.c_uint32,  # RenderBufferFormats::List
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # cEgRenderBuffer::eMipMode
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eTextureAddressMode
            ctypes.c_ulonglong,  # eTkTextureReductionMode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgRendererBase::CreateRenderBufferSamplers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_ulonglong,  # cEgRenderBuffer *
            ctypes.c_ulonglong,  # RenderBufferFormats::List *
            ctypes.c_uint32,  # cEgRenderBuffer::eMipMode
            ctypes.c_uint32,  # eTextureAddressMode
            ctypes.c_ulonglong,  # eTkTextureReductionMode *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cEgRendererBase::ReleaseRenderBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cEgRendererBase::GenerateMipMaps": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkRenderStateCache *
        ]
    ),
    "cEgRendererBase::UpdateMappedBufferData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cEgRendererBase::GetRenderBufferTexture": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkTexture *
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgRendererBase::SetRenderBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # VkFormat
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkVector4 *
            ctypes.c_ulonglong,  # cTkVector4 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgRendererBase::UnSetRenderBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgRendererBase::ApplyVertexLayout": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_ulonglong,  # const cTkVertexLayoutRT *
            ctypes.c_ulonglong,  # const cTkShader *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_int32,  # const int
        ]
    ),
    "cEgRendererBase::ClearVertexLayout": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgRendererBase::BeginPipelineStage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        argtypes=[
            ctypes.c_ulonglong,  # cEgRendererBase *
            ctypes.c_char_p,  # const char *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "cEgRenderer::RunPipeline": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
        ]
    ),
    "cEgRenderer::SetMaterial": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgMaterialResource *
            ctypes.c_char_p,  # const TkIDHashed<128> *
            ctypes.c_ulonglong,  # cEgShaderUniformBufferBinding *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cEgRenderer::SetStereoTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderer *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cEgThreadableRenderCall::cRendererData *
            ctypes.c_ulonglong,  # cTkRenderStateCache *
        ]
    ),
    "cEgRenderer::UpdatePerTargetUniforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgShaderUniformBufferBinding *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_ulonglong,  # cTkRenderStateCache *
        ]
    ),
    "cEgRBObjects<cEgRenderBuffer,8>::Remove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRBObjects<cEgRenderBuffer,8> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cEgRBObjects<cEgRenderBuffer,8>::Add": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgRBObjects<cEgRenderBuffer,8> *
            ctypes.c_ulonglong,  # cEgRenderBuffer *
        ]
    ),
    "cEgRBObjects<cEgRenderBuffer,8>::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRBObjects<cEgRenderBuffer,8> *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cEgRBObjects<cTkBuffer,8>::Alloc": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgRBObjects<cTkBuffer,8> *
        ]
    ),
    "cEgRBObjects<cTkBuffer,8>::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRBObjects<cTkBuffer,8> *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cEgRBObjects<cEgRenderBuffer,8>::Grow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRBObjects<cEgRenderBuffer,8> *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const std::_Value_init_tag *
        ]
    ),
    "cEgRBObjects<cTkBuffer,8>::Grow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRBObjects<cTkBuffer,8> *
        ]
    ),
    "cTkDestroyQueue<VkImageView_T *,16384,0>::Push<void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDestroyQueue<VkImageView_T *,16384,0> *
            ctypes.c_ulonglong,  # struct VkImageView_T **
            ctypes.c_ulonglong,  # VmaAllocation_T *
            ctypes.c_ulonglong,  # struct VkDeviceMemory_T *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['struct VkDevice_T *', 'struct VkImageView_T *', 'const VkAllocationCallbacks *']}
        ]
    ),
    "cTkDestroyQueue<VkImage_T *,16384,0>::Push<void ": {
        "cTkDestroyQueue<VkImage_T *,16384,0> *, struct VkImage_T **, struct VkDeviceMemory_T *, __int64 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDestroyQueue<VkImage_T *,16384,0> *
                ctypes.c_ulonglong,  # struct VkImage_T **
                ctypes.c_ulonglong,  # struct VkDeviceMemory_T *
                ctypes.c_ulonglong,  # {'return': 'void', 'args': ['struct VkDevice_T *', 'struct VkImage_T *', 'const VkAllocationCallbacks *']}
            ]
        ),
        "cTkDestroyQueue<VkImage_T *,16384,0> *, struct VkImage_T **, VmaAllocation_T *, __int64 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDestroyQueue<VkImage_T *,16384,0> *
                ctypes.c_ulonglong,  # struct VkImage_T **
                ctypes.c_ulonglong,  # VmaAllocation_T *
                ctypes.c_ulonglong,  # {'return': 'void', 'args': ['struct VkDevice_T *', 'struct VkImage_T *', 'const VkAllocationCallbacks *']}
            ]
        ),
    },
    "cTkDestroyQueue<VkImage_T *,16384,0>::DestroyObjects<void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDestroyQueue<VkImage_T *,16384,0> *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['struct VkDevice_T *', 'struct VkImage_T *', 'const VkAllocationCallbacks *']}
        ]
    ),
    "EngineUtils::LoadResourcesFromDisk": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # EngineUtils::LoadResourcesBalancing
        ]
    ),
    "EngineUtils::AreResourcesAwaitingLoad": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "EngineUtils::DumpMessages": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "xmlMalloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int64 *
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "xmlRealloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _QWORD *
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "XMLNode::getError": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_uint32,  # XMLError
        ]
    ),
    "XMLNode::writeToFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char,  # char
        ]
    ),
    "stringDup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "toXMLStringUnSafe": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "lengthXMLString": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "fromXMLString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XML *
        ]
    ),
    "myTagCompare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "GetNextToken": FUNCDEF(
        restype=ctypes.c_ulonglong,  # NextToken *
        argtypes=[
            ctypes.c_ulonglong,  # NextToken *
            ctypes.c_ulonglong,  # XML *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # XMLTokenTypeTag *
        ]
    ),
    "XMLNode::XMLNode": {
        "XMLNode *, XMLNode::XMLNodeDataTag *, char *, char": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_ulonglong,  # XMLNode::XMLNodeDataTag *
                ctypes.c_ulonglong,  # char *
                ctypes.c_char,  # char
            ]
        ),
        "XMLNode *, const XMLNode *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_ulonglong,  # const XMLNode *
            ]
        ),
    },
    "XMLNode::createXMLTopNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # XMLNode *
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char,  # char
        ]
    ),
    "XMLNode::removeOrderElement": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode::XMLNodeDataTag *
            ctypes.c_uint32,  # XMLElementType
            ctypes.c_int32,  # int
        ]
    ),
    "XMLNode::addToOrder": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # XMLElementType
        ]
    ),
    "XMLNode::addChild_priv": FUNCDEF(
        restype=ctypes.c_ulonglong,  # XMLNode *
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # char *
            ctypes.c_char,  # char
            ctypes.c_int32,  # int
        ]
    ),
    "XMLNode::addText_priv": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
        ]
    ),
    "XMLNode::parseClearTag": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "XMLNode::exactMemory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode::XMLNodeDataTag *
        ]
    ),
    "XMLNode::maybeAddTxT": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ulonglong,  # void *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "XMLNode::ParseXMLElement": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "XMLNode::parseString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # XMLNode *
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # XMLResults *
        ]
    ),
    "XMLNode::parseFile": FUNCDEF(
        restype=ctypes.c_ulonglong,  # XMLNode *
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # XMLResults *
        ]
    ),
    "XMLNode::CreateXMLStringR": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode::XMLNodeDataTag *
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
        ]
    ),
    "XMLNode::detachFromParent": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode::XMLNodeDataTag *
        ]
    ),
    "XMLNode::~XMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
        ]
    ),
    "XMLNode::deleteNodeContent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
        ]
    ),
    "XMLNode::deleteNodeContent_priv": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char,  # char
            ctypes.c_char,  # char
        ]
    ),
    "XMLNode::deepCopy": FUNCDEF(
        restype=ctypes.c_ulonglong,  # XMLNode *
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ulonglong,  # XMLNode *
        ]
    ),
    "XMLNode::deleteAttribute": {
        "XMLNode *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_int32,  # int
            ]
        ),
        "XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "XMLNode::updateAttribute_WOSD": FUNCDEF(
        restype=ctypes.c_ulonglong,  # XMLAttribute *
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "XMLNode::nChildNode": {
        "XMLNode *, const char *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # XMLNode::XMLNodeDataTag *
            argtypes=[
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "XMLNode *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # XMLNode::XMLNodeDataTag *
            argtypes=[
                ctypes.c_ulonglong,  # XMLNode *
            ]
        ),
    },
    "XMLNode::getChildNode": {
        "XMLNode *, XMLNode *, const char *, int *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # XMLNode *
            argtypes=[
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # int *
            ]
        ),
        "XMLNode *, XMLNode *, const char *, int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # XMLNode *
            argtypes=[
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_int32,  # int
            ]
        ),
        "XMLNode *, XMLNode *, int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # XMLNode *
            argtypes=[
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "XMLNode::getChildNodeWithAttribute": FUNCDEF(
        restype=ctypes.c_ulonglong,  # XMLNode *
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "XMLNode::getAttribute": {
        "XMLNode *, const char *, int *": FUNCDEF(
            restype=ctypes.c_char_p,  # const char *
            argtypes=[
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # int *
            ]
        ),
        "XMLNode *, const char *, const char *": FUNCDEF(
            restype=ctypes.c_char_p,  # const char *
            argtypes=[
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "XMLNode::getName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # XMLNode::XMLNodeDataTag *
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
        ]
    ),
    "XMLNode::nAttribute": FUNCDEF(
        restype=ctypes.c_ulonglong,  # XMLNode::XMLNodeDataTag *
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
        ]
    ),
    "XMLNode::getAttributeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_int32,  # int
        ]
    ),
    "XMLNode::isEmpty": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
        ]
    ),
    "XMLNode::addChild": FUNCDEF(
        restype=ctypes.c_ulonglong,  # XMLNode *
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char,  # char
            ctypes.c_int32,  # int
        ]
    ),
    "XMLNode::addAttribute": FUNCDEF(
        restype=ctypes.c_ulonglong,  # XMLAttribute *
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "XMLNode::addAttribute_WOSD": FUNCDEF(
        restype=ctypes.c_ulonglong,  # XMLAttribute *
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "XMLNode::updateAttribute": FUNCDEF(
        restype=ctypes.c_ulonglong,  # XMLAttribute *
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "XMLParserBase64Tool::encode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # XMLParserBase64Tool *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_char,  # char
        ]
    ),
    "XMLParserBase64Tool::decode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # XMLParserBase64Tool *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # XMLError *
        ]
    ),
    "TkExample::cEgCubeNode::cEgCubeNodeTemplate::GetBoundingBox": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAABB *
        argtypes=[
            ctypes.c_ulonglong,  # TkLineRenderer::cEgLineRendererNode::cEgLineRendererNodeTemplate *
            ctypes.c_ulonglong,  # cTkAABB *
            ctypes.c_ulonglong,  # const cTkResourceDescriptor *
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "TkExample::cEgCubeNode::cEgCubeNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkExample::cEgCubeNode *
            ctypes.c_ulonglong,  # const TkExample::cEgCubeNode::cEgCubeNodeTemplate *
        ]
    ),
    "TkExample::cEgCubeNode::InitialiseVertexLayout": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "TkExample::cEgCubeNode::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "TkExample::cEgCubeNode::FactoryFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "TkExample::cEgCubeNode::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # RenderingOrder::List
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "TkExample::cEgCubeNode::DrawShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_ulonglong,  # cEgShaderUniformBufferBinding *
        ]
    ),
    "TkExample::InitExtension": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[]
    ),
    "TkGalactic::cEgPStreamCacheResource::cEgPStreamCacheResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkGalactic::cEgPStreamCacheResource *
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "TkGalactic::cEgPStreamCacheResource::MapResStream": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # TkGalactic::cEgPStreamCacheResource *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "TkGalactic::cEgPStreamCacheResource::UnmapStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkGalactic::cEgPStreamCacheResource *
            ctypes.c_int32,  # int
        ]
    ),
    "TkGalactic::cEgPStreamNode::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "TkGalactic::cEgPStreamNode::FactoryFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgRenderableSceneNode *
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "TkGalactic::cEgPStreamNode::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # RenderingOrder::List
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "TkGalactic::cEgPStreamNode::ResourceFactoryFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "TkGalactic::cEgPStreamNode::RenderStars": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkGalactic::cEgPStreamNode *
            ctypes.c_ulonglong,  # const cEgRenderQueueEntry *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_ulonglong,  # cEgShaderUniformBufferBinding *
        ]
    ),
    "TkGalactic::cEgPStreamNode::RenderInstanced": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkGalactic::cEgPStreamNode *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_ulonglong,  # cEgShaderUniformBufferBinding *
        ]
    ),
    "TkGalactic::InitParticleStreamVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkGalactic *
        ]
    ),
    "TkGalactic::InitExtension": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "GetScreenPlaneData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cEgRenderQueueEntry *
            ctypes.c_ulonglong,  # cTkAABB2D *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "GetCullingIcosahedron": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cEgRenderQueueEntry *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # cTkIcosahedron *
        ]
    ),
    "GetIcoFaceBox": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const cTkIcosahedron *
            ctypes.c_int32,  # const int
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # cTkFixedArray<cTkVector3,4> *
            ctypes.c_ulonglong,  # cTkFixedArray<cTkVector3,4> *
            ctypes.c_ulonglong,  # cTkFixedArray<cTkVector3,4> *
            ctypes.c_ulonglong,  # cTkAABB2D *
        ]
    ),
    "TkHeavyAir::TkHeavyAirNodeTemplate::GetBoundingBox": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAABB *
        argtypes=[
            ctypes.c_ulonglong,  # TkHeavyAir::TkHeavyAirNodeTemplate *
            ctypes.c_ulonglong,  # cTkAABB *
            ctypes.c_ulonglong,  # const cTkResourceDescriptor *
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "TkAtmospherics::InitExtension": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[]
    ),
    "TkHeavyAir::BeginIncrementUpdateCounter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "TkHeavyAir::TkHeavyAirNodeTemplate::TkHeavyAirNodeTemplate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkHeavyAir::TkHeavyAirNodeTemplate *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "TkHeavyAir::cTkHeavyAirNode::cTkHeavyAirNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkHeavyAir::cTkHeavyAirNode *
            ctypes.c_ulonglong,  # const TkHeavyAir::TkHeavyAirNodeTemplate *
        ]
    ),
    "TkHeavyAir::cTkHeavyAirNode::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkHeavyAir::cTkHeavyAirNode *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "TkHeavyAir::cTkHeavyAirWorkLoad::Run": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkHeavyAir::cTkHeavyAirWorkLoad *
        ]
    ),
    "TkHeavyAir::HeavyAirAsyncUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "TkHeavyAir::cTkHeavyAirNode::BeginUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkHeavyAir::cTkHeavyAirNode *
            ctypes.c_float,  # float
        ]
    ),
    "TkHeavyAir::cTkHeavyAirNode::OnDetach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkHeavyAir::cTkHeavyAirNode *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "TkHeavyAir::cTkHeavyAirNode::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "TkHeavyAir::cTkHeavyAirNode::FactoryFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "TkHeavyAir::cTkHeavyAirNode::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # RenderingOrder::List
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "TkHeavyAir::cTkHeavyAirNode::RenderBatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkHeavyAir::cTkHeavyAirNode *
            ctypes.c_ulonglong,  # cEgShaderCombination *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cEgShaderUniformBufferBinding *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "TkHeavyAir::cTkHeavyAirNode::GetParameterFloat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # TkHeavyAir::cTkHeavyAirNode *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "TkHeavyAir::cTkHeavyAirNode::SetParameterFloat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkHeavyAir::cTkHeavyAirNode *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_double,  # double
        ]
    ),
    "TkHeavyAir::cTkHeavyAirNode::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkHeavyAir::cTkHeavyAirNode *
        ]
    ),
    "TkSpeedLines::cTkSpeedLinesNode::cTkSpeedLinesNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkSpeedLines::cTkSpeedLinesNode *
            ctypes.c_ulonglong,  # const TkSpeedLines::cTkSpeedLinesNode::cTkSpeedLinesNodeTemplate *
        ]
    ),
    "TkSpeedLines::cTkSpeedLinesNode::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkSpeedLines::cTkSpeedLinesNode *
        ]
    ),
    "TkSpeedLines::cTkSpeedLinesNode::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "TkSpeedLines::cTkSpeedLinesNode::FactoryFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "TkSpeedLines::cTkSpeedLinesNode::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # RenderingOrder::List
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "TkSpeedLines::cTkSpeedLinesNode::PreAsyncUpdate": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # TkSpeedLines::cTkSpeedLinesNode *
        ]
    ),
    "TkSpeedLines::cTkSpeedLinesNode::AsyncUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkSpeedLines::cTkSpeedLinesNode *
        ]
    ),
    "TkSpeedLines::cTkSpeedLinesNode::SetParameterFloat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkSpeedLines::cTkSpeedLinesNode *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
        ]
    ),
    "TkSpeedLines::cTkSpeedLinesNode::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkSpeedLines::cTkSpeedLinesNode *
        ]
    ),
    "TkSpeedLines::cTkSpeedLinesNode::UpdateLineBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkSpeedLines::cTkSpeedLinesNode *
        ]
    ),
    "cTkAtomicLinearMemoryPool::GetUsage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAtomicLinearMemoryPool *
        ]
    ),
    "cTkAtomicLinearMemoryPool::GetHighWaterUsage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAtomicLinearMemoryPool *
        ]
    ),
    "cTkAtomicLinearMemoryPool::SetMutex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAtomicLinearMemoryPool *
            ctypes.c_ulonglong,  # cTkCSMutex *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::GetLodScales": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::PreAsyncUpdate": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::AsyncUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::PostAsyncUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "EgInstancedModelExtension::InitExtension": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[]
    ),
    "EgInstancedModelExtension::GetInstanceTransformsMappedOrder": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix44 *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "EgInstancedModelExtension::GetInstanceOrderMap": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgNodeAttachment *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "EgInstancedModelExtension::GetNodeRegionHandle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "EgInstancedModelExtension::GetNodeResHandle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "EgInstancedModelExtension::RemoveOneInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
        ]
    ),
    "EgInstancedModelExtension::IsInstanceModelNode": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::GetLodMeshType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgModelNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedMeshNode::EnqueueFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
            ctypes.c_ulonglong,  # cEgRenderQueueEntry *
            ctypes.c_ulonglong,  # cEgRenderableSceneNode **
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkDependencyCounter *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::FactoryFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgModelNode *
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::GatherConstantParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const EgInstancedModelExtension::cEgInstancedModelNode::sPostCullUpdateConstParams *
        argtypes=[
            ctypes.c_ulonglong,  # const EgInstancedModelExtension::cEgInstancedModelNode::sPostCullUpdateConstParams *
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
            ctypes.c_ulonglong,  # cEgUpdatableQueueEntry *
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::PostCullUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
            ctypes.c_ulonglong,  # cEgUpdatableQueueEntry *
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
        ]
    ),
    "EgInstancedModelExtension::PostCullUpdateJobParams::PostCullUpdateJobParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::PostCullUpdateJobParams *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::PostCullUpdateJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::UpdateLocalMeshAABBs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # cEgMeshNode *
        ]
    ),
    "EgInstancedModelExtension::UpdateAndSearchForRenderables": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::InstanceTransformsUpdated": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::RebuildInstanceAABBs_PreAsyncUpdate": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::RebuildInstanceAABBs_AsyncUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::RebuildInstanceAABBs_PostAsyncUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::ExpandBoundingBoxes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::CheckIntersection": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::RemoveOneInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
            ctypes.c_int32,  # int
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::SetParameterFloat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::SetParameterInt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::GetClassOverride": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedMeshNode::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgMeshNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedMeshNode::FactoryFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgMeshNode *
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedMeshNode::CanAttach": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedMeshNode *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedMeshNode::OnAttach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedMeshNode *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedMeshNode::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # RenderingOrder::List
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedMeshNode::RenderStereo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # RenderingOrder::List
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedMeshNode::StartFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "EgInstancedModelExtension::cEgInstancedMeshNode::BuildMatrixJobPostCull": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedMeshNode::BuildMatrixJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedMeshNode::RenderAsync": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # RenderingOrder::List
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedMeshNode::RenderInstancedNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedMeshNode *
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # cEgRenderQueueEntry *
            ctypes.c_ulonglong,  # cEgMeshNodeQueueEntry *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const cTkVertexLayoutRT **
            ctypes.c_ulonglong,  # cEgGeometryResource **
            ctypes.c_ulonglong,  # cEgMaterialResource **
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cEgShaderUniformBufferBinding *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "EgInstancedModelExtension::CacheGrassPushers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const std::vector<EgInstancedModelExtension::GrassPusher,TkSTLAllocatorShim<EgInstancedModelExtension::GrassPusher,16,-1> > *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "EgInstancedModelExtension::InstanceNguiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkSubdivBuilder<float,EgInstancedModelExtension::cEgInstancedModelNode::AabbBuilderHelper,6>::SwapTris": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSubdivBuilder<float,EgInstancedModelExtension::cEgInstancedModelNode::AabbBuilderHelper,6> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkQuantizedAabbList<float,6>::~cTkQuantizedAabbList<float,6>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuantizedAabbList<float,6> *
        ]
    ),
    "cTkQuantizedAabbList<float,6>::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuantizedAabbList<float,6> *
        ]
    ),
    "cTkQuantizedAabbList<float,6>::AddAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuantizedAabbList<float,6> *
            ctypes.c_ulonglong,  # const cTkQuantizedAabb<float> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkQuantizedAabbList<float,6>::CalcAabb": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkQuantizedAabb<float> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuantizedAabbList<float,6> *
            ctypes.c_ulonglong,  # cTkQuantizedAabb<float> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkQuantizedAabbList<float,6>::SwapElements": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuantizedAabbList<float,6> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::GetHasNoneZeroInterpolationVectors<0>": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::GetInstanceTransforms<0>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix44 *
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::GetInterpolateFromVectors<0>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector4 *
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::GetInstancePositions<0>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::SetHasNoneZeroInterpolationVectors<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::GetInstanceOrderMap<0>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # int *
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::GetNumInstanceTransforms<0>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::GetOverallBox<0>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgBoundingBox *
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
            ctypes.c_ulonglong,  # cEgBoundingBox *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::SetLodLevelMin<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
            ctypes.c_int32,  # int
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::SetLodLevelMax<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
            ctypes.c_int32,  # int
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::SetAabbListDirty<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::GetFadeOutStart<0>": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::GetFadeOutOffsetDistance<0>": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "EgInstancedModelExtension::cEgInstancedModelNode::GetFadeOutRange<0>": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # EgInstancedModelExtension::cEgInstancedModelNode *
        ]
    ),
    "cTkMultiVector<float,6,6,8>::Reserve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMultiVector<float,6,6,8> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMultiVector<int,2,6,8>::Reserve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMultiVector<int,2,6,8> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Json::Value::nullSingleton": FUNCDEF(
        restype=ctypes.c_ulonglong,  # Json::Value *
        argtypes=[]
    ),
    "Json__duplicateStringValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "Json::ValueIteratorBase::deref": FUNCDEF(
        restype=ctypes.c_ulonglong,  # Json::Value *
        argtypes=[
            ctypes.c_ulonglong,  # Json::ValueIteratorBase *
        ]
    ),
    "Json::ValueIteratorBase::increment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::ValueIteratorBase *
        ]
    ),
    "Json::ValueIteratorBase::isEqual": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # Json::ValueIteratorBase *
            ctypes.c_ulonglong,  # const Json::ValueIteratorBase *
        ]
    ),
    "Json::ValueIteratorBase::key": FUNCDEF(
        restype=ctypes.c_ulonglong,  # Json::Value *
        argtypes=[
            ctypes.c_ulonglong,  # Json::ValueIteratorBase *
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "Json::Value::CommentInfo::setComment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value::CommentInfo *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "Json::Value::CZString::CZString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value::CZString *
            ctypes.c_ulonglong,  # const Json::Value::CZString *
        ]
    ),
    "Json::Value::CZString::operator<": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value::CZString *
            ctypes.c_ulonglong,  # const Json::Value::CZString *
        ]
    ),
    "Json::Value::Value": {
        "Json::Value *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # Json::Value *
                ctypes.c_int32,  # int
            ]
        ),
        "Json::Value *, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # Json::Value *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "Json::Value *, long double": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # Json::Value *
                ctypes.c_double,  # long double
            ]
        ),
        "Json::Value *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # Json::Value *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "Json::Value *, const std::string *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # Json::Value *
                ctypes.c_ulonglong,  # const std::string *
            ]
        ),
        "Json::Value *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # Json::Value *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "Json::Value *, const Json::Value *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # Json::Value *
                ctypes.c_ulonglong,  # const Json::Value *
            ]
        ),
    },
    "Json::Value::~Value": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "Json::Value::asCString": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "Json::Value::asString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::string *
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
            ctypes.c_ulonglong,  # std::string *
        ]
    ),
    "Json::Value::asInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "Json::Value::asUInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "Json::Value::asDouble": FUNCDEF(
        restype=ctypes.c_double,  # long double
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "Json::Value::asBool": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "Json::Value::size": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "Json::Value::dupPayload": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
            ctypes.c_ulonglong,  # const Json::Value *
        ]
    ),
    "Json::Value::releasePayload": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "Json::Value::dupMeta": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
            ctypes.c_ulonglong,  # const Json::Value *
        ]
    ),
    "Json::Value::resolveReference": FUNCDEF(
        restype=ctypes.c_ulonglong,  # Json::Value *
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "Json::Value::find": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const Json::Value *
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "Json::Value::get": FUNCDEF(
        restype=ctypes.c_ulonglong,  # Json::Value *
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
            ctypes.c_ulonglong,  # Json::Value *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const Json::Value *
        ]
    ),
    "Json::Value::getMemberNames": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<std::string> *
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
            ctypes.c_ulonglong,  # std::vector<std::string> *
        ]
    ),
    "Json::Value::isInt": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "Json::Value::isUInt": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "Json::Value::isString": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "Json::Value::isArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "Json::Value::isObject": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "Json::Value::toStyledString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::string *
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
            ctypes.c_ulonglong,  # std::string *
        ]
    ),
    "Json::Value::begin": FUNCDEF(
        restype=ctypes.c_ulonglong,  # Json::ValueConstIterator *
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
            ctypes.c_ulonglong,  # Json::ValueConstIterator *
        ]
    ),
    "Json::Value::end": FUNCDEF(
        restype=ctypes.c_ulonglong,  # Json::ValueConstIterator *
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
            ctypes.c_ulonglong,  # Json::ValueConstIterator *
        ]
    ),
    "Json__codePointToUTF8": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::string *
        argtypes=[
            ctypes.c_ulonglong,  # std::string *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "Json::Reader::Reader": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::Reader *
        ]
    ),
    "Json::Reader::parse": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # Json::Reader *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "Json::Reader::readValue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # Json::Reader *
        ]
    ),
    "Json::Reader::readToken": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # Json::Reader *
            ctypes.c_ulonglong,  # Json::Reader::Token *
        ]
    ),
    "Json::Reader::readComment": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # Json::Reader *
        ]
    ),
    "Json::Reader::readObject": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # Json::Reader *
            ctypes.c_ulonglong,  # Json::Reader::Token *
        ]
    ),
    "Json::Reader::readArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # Json::Reader *
            ctypes.c_ulonglong,  # Json::Reader::Token *
        ]
    ),
    "Json::Reader::decodeNumber": {
        "Json::Reader *, Json::Reader::Token *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # Json::Reader *
                ctypes.c_ulonglong,  # Json::Reader::Token *
            ]
        ),
        "Json::Reader *, Json::Reader::Token *, Json::Value *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # Json::Reader *
                ctypes.c_ulonglong,  # Json::Reader::Token *
                ctypes.c_ulonglong,  # Json::Value *
            ]
        ),
    },
    "Json::Reader::decodeDouble": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # Json::Reader *
            ctypes.c_ulonglong,  # Json::Reader::Token *
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "Json::OurReader::decodeString": {
        "Json::Reader *, Json::Reader::Token *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # Json::Reader *
                ctypes.c_ulonglong,  # Json::Reader::Token *
            ]
        ),
        "Json::Reader *, Json::Reader::Token *, std::string *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # Json::Reader *
                ctypes.c_ulonglong,  # Json::Reader::Token *
                ctypes.c_ulonglong,  # std::string *
            ]
        ),
    },
    "Json::Reader::decodeUnicodeCodePoint": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # Json::Reader *
            ctypes.c_ulonglong,  # Json::Reader::Token *
            ctypes.c_char_p,  # const char **
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "Json::OurReader::decodeUnicodeEscapeSequence": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # Json::Reader *
            ctypes.c_ulonglong,  # Json::Reader::Token *
            ctypes.c_char_p,  # const char **
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "Json::Reader::addError": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # Json::Reader *
            ctypes.c_ulonglong,  # const std::string *
            ctypes.c_ulonglong,  # Json::Reader::Token *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "Json::OurReader::parse": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # Json::OurReader *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # Json::Value *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Json::OurReader::readValue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # Json::OurReader *
        ]
    ),
    "Json::OurReader::readToken": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # Json::OurReader *
            ctypes.c_ulonglong,  # Json::OurReader::Token *
        ]
    ),
    "Json::OurReader::readComment": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # Json::OurReader *
        ]
    ),
    "Json::OurReader::readNumber": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # Json::OurReader *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Json::OurReader::readObject": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # Json::OurReader *
            ctypes.c_ulonglong,  # Json::OurReader::Token *
        ]
    ),
    "Json::OurReader::readArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # Json::OurReader *
            ctypes.c_ulonglong,  # Json::OurReader::Token *
        ]
    ),
    "Json::OurReader::decodeNumber": {
        "Json::OurReader *, Json::OurReader::Token *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # Json::OurReader *
                ctypes.c_ulonglong,  # Json::OurReader::Token *
            ]
        ),
        "Json::OurReader *, Json::OurReader::Token *, Json::Value *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # Json::OurReader *
                ctypes.c_ulonglong,  # Json::OurReader::Token *
                ctypes.c_ulonglong,  # Json::Value *
            ]
        ),
    },
    "Json::OurReader::decodeDouble": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # Json::OurReader *
            ctypes.c_ulonglong,  # Json::OurReader::Token *
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "Json::OurReader::getFormattedErrorMessages": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::string *
        argtypes=[
            ctypes.c_ulonglong,  # Json::OurReader *
            ctypes.c_ulonglong,  # std::string *
        ]
    ),
    "Json::OurCharReader::parse": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # Json::OurCharReader *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # Json::Value *
            ctypes.c_ulonglong,  # std::string *
        ]
    ),
    "Json::CharReaderBuilder::CharReaderBuilder": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::CharReaderBuilder *
        ]
    ),
    "Json::CharReaderBuilder::~CharReaderBuilder": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::CharReaderBuilder *
        ]
    ),
    "Json::CharReaderBuilder::newCharReader": FUNCDEF(
        restype=ctypes.c_ulonglong,  # Json::CharReader *
        argtypes=[
            ctypes.c_ulonglong,  # Json::CharReaderBuilder *
        ]
    ),
    "Json::CharReaderBuilder::setDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "Json::valueToString": {
        "std::string *, __int64": FUNCDEF(
            restype=ctypes.c_ulonglong,  # std::string *
            argtypes=[
                ctypes.c_ulonglong,  # std::string *
                ctypes.c_int64,  # __int64
            ]
        ),
        "std::string *, unsigned __int64": FUNCDEF(
            restype=ctypes.c_ulonglong,  # std::string *
            argtypes=[
                ctypes.c_ulonglong,  # std::string *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
    },
    "Json__toHex16Bit": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::string *
        argtypes=[
            ctypes.c_ulonglong,  # std::string *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "Json__valueToQuotedStringN": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::string *
        argtypes=[
            ctypes.c_ulonglong,  # std::string *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "Json::BuiltStyledStreamWriter::write": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # Json::BuiltStyledStreamWriter *
            ctypes.c_ulonglong,  # const Json::Value *
            ctypes.c_ulonglong,  # std::ostream *
        ]
    ),
    "Json::BuiltStyledStreamWriter::writeValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::BuiltStyledStreamWriter *
            ctypes.c_ulonglong,  # const Json::Value *
        ]
    ),
    "Json::BuiltStyledStreamWriter::writeArrayValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::BuiltStyledStreamWriter *
            ctypes.c_ulonglong,  # const Json::Value *
        ]
    ),
    "Json::BuiltStyledStreamWriter::isMultilineArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # Json::BuiltStyledStreamWriter *
            ctypes.c_ulonglong,  # const Json::Value *
        ]
    ),
    "Json::BuiltStyledStreamWriter::writeIndent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::BuiltStyledStreamWriter *
        ]
    ),
    "Json::BuiltStyledStreamWriter::writeWithIndent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::BuiltStyledStreamWriter *
            ctypes.c_ulonglong,  # const std::string *
        ]
    ),
    "Json::BuiltStyledStreamWriter::unindent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::BuiltStyledStreamWriter *
        ]
    ),
    "Json::BuiltStyledStreamWriter::writeCommentBeforeValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::BuiltStyledStreamWriter *
            ctypes.c_ulonglong,  # const Json::Value *
        ]
    ),
    "Json::BuiltStyledStreamWriter::writeCommentAfterValueOnSameLine": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::BuiltStyledStreamWriter *
            ctypes.c_ulonglong,  # const Json::Value *
        ]
    ),
    "Json::StreamWriterBuilder::newStreamWriter": FUNCDEF(
        restype=ctypes.c_ulonglong,  # Json::StreamWriter *
        argtypes=[
            ctypes.c_ulonglong,  # Json::StreamWriterBuilder *
        ]
    ),
    "Json::StreamWriterBuilder::setDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Json::Value *
        ]
    ),
    "TkLineRenderer::cEgLineRendererNode::cEgLineRendererNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkLineRenderer::cEgLineRendererNode *
            ctypes.c_ulonglong,  # const TkLineRenderer::cEgLineRendererNode::cEgLineRendererNodeTemplate *
        ]
    ),
    "TkLineRenderer::cEgLineRendererNode::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "TkLineRenderer::cEgLineRendererNode::FactoryFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "TkLineRenderer::cEgLineRendererNode::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # RenderingOrder::List
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "TkLineRenderer::cEgLineRendererNode::RenderLines": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkLineRenderer::cEgLineRendererNode *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_ulonglong,  # const cEgRenderQueueEntry *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "TkLineRenderer::cEgLineRendererNode::AddLineImmediate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkLineRenderer::cEgLineRendererNode *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "TkLineRenderer::cEgLineRendererNode::AddLineImmediatePhysRel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkLineRenderer::cEgLineRendererNode *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "TkLineRenderer::cEgLineRendererNode::AddCircleImmediate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkLineRenderer::cEgLineRendererNode *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "TkLineRenderer::cEgLineRendererNode::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkLineRenderer::cEgLineRendererNode *
        ]
    ),
    "TkLineRenderer::InitExtension": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[]
    ),
    "TkLineRenderer::AddLine": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "TkLineRenderer::AddLineImmediatePhysRel": {
        "TkHandle, cTkPhysRelVec3 *, const cTkColour *, int, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_uint32,  # TkHandle
                ctypes.c_ulonglong,  # cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkColour *
                ctypes.c_int32,  # int
                ctypes.c_float,  # float
            ]
        ),
        "TkHandle, cTkPhysRelVec3 *, cTkColour *, int, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_uint32,  # TkHandle
                ctypes.c_ulonglong,  # cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # cTkColour *
                ctypes.c_int32,  # int
                ctypes.c_float,  # float
            ]
        ),
    },
    "TkLineRenderer::AddCircleImmediate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "TkLineRenderer::ClearImmediateBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "LZ4_putPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # tableType_t
            ctypes.c_ulonglong,  # const unsigned __int8 *
        ]
    ),
    "LZ4_compress_fast_extState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "LZ4_compress_fast": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "LZ4_renormDictT": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # LZ4_stream_t_internal *
            ctypes.c_ulonglong,  # const unsigned __int8 *
        ]
    ),
    "LZ4_compress_fast_continue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # LZ4_stream_t *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "LZ4_decompress_safe": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "LZ4_decompress_fast": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
        ]
    ),
    "LZ4_decompress_safe_continue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # LZ4_streamDecode_t *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
        ]
    ),
    "EgMagicModels__SetModelFadeValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_float,  # float
        ]
    ),
    "EgMagicModels::cEgMagicModel::cEgMagicModel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgMagicModels::cEgMagicModel *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # TkHandle
            ctypes.c_float,  # float
        ]
    ),
    "EgMagicModels::cEgMagicModel::UpdateMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgMagicModels::cEgMagicModel *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "EgMagicModels::cEgMagicModel::SpawnModel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgMagicModels::cEgMagicModel *
            ctypes.c_ulonglong,  # const EgMagicModels::cEgMagicWish *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['TkHandle', {'type': 'unsigned', 'name': '__int64'}]}
            ctypes.c_ubyte,  # bool
        ]
    ),
    "EgMagicModels::cEgMagicModel::UpdateModel": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # EgMagicModels::cEgMagicModel *
            ctypes.c_ulonglong,  # const EgMagicModels::cEgMagicWish *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['TkHandle', {'type': 'unsigned', 'name': '__int64'}]}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['TkHandle', {'type': 'unsigned', 'name': '__int64'}]}
        ]
    ),
    "EgMagicModels::cEgMagicModel::UpdateFade": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # EgMagicModels::cEgMagicModel *
            ctypes.c_float,  # float
        ]
    ),
    "StackContainer<std::vector<EgMagicModels::cEgMagicVisibilityMap::sModel,StackAllocator<EgMagicModels::cEgMagicVisibilityMap::sModel,8,8>>,8,8>::~StackContainer<std::vector<EgMagicModels::cEgMagicVisibilityMap::sModel,StackAllocator<EgMagicModels::cEgMagicVisibilityMap::sModel,8,8>>,8,8>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StackContainer<std::vector<EgMagicModels::cEgMagicVisibilityMap::sModel,StackAllocator<EgMagicModels::cEgMagicVisibilityMap::sModel,8,8> >,8,8> *
        ]
    ),
    "EgMagicModels::GetExtensionName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "EgMagicModels::InitExtension": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "EgMagicModels__GatherMeshVertsRecursive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkGeometryStreamData *
            ctypes.c_ulonglong,  # const cTkVertexLayoutRT *
            ctypes.c_ulonglong,  # std::vector<hkFloat3,TkSTLAllocatorShim<hkFloat3,4,-1> > *
        ]
    ),
    "EgMagicModels__GatherModelVertsRecursive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # std::vector<hkFloat3,TkSTLAllocatorShim<hkFloat3,4,-1> > *
        ]
    ),
    "EgMagicModels::AddMagicModel": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "EgMagicModels::ForceActivateMagicModel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "EgMagicModels::UpdateModelMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "EgMagicModels::RemoveMagicModel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "EgMagicModels::SetBudgetValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char,  # char
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "EgMagicModels::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_float,  # float
        ]
    ),
    "EgMagicModels::UpdateModelSpawns": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgMagicModels *
        ]
    ),
    "EgMagicModels::SortByVisibility": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const EgMagicModels::sMagicModelVisibility *
            ctypes.c_ulonglong,  # const EgMagicModels::sMagicModelVisibility *
        ]
    ),
    "EgMagicModels::AddModelToSpawnList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "EgMagicModels::BuildVisibilityMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "EgMagicModels::ComputeModelVisibilities": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "EgMagicModels::UpdateModelsForBudget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "EgMagicModels::cEgMagicVisibilityMap::CalculateModelVisibility": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # EgMagicModels::cEgMagicVisibilityMap *
            ctypes.c_ulonglong,  # const EgMagicModels::cEgMagicVisibilityMap::sModel *
        ]
    ),
    "EgMagicModels::cEgMagicVisibilityMap::CouldBeObscuringConeSphereIntersection": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const EgMagicModels::cEgMagicVisibilityMap::sModel *
            ctypes.c_ulonglong,  # const EgMagicModels::cEgMagicVisibilityMap::sModel *
        ]
    ),
    "EgMagicModels::cEgMagicVisibilityMap::QuickHull": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "EgMagicModels::cEgMagicVisibilityMap::ProjectModelToShape": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "EgMagicModels::cEgMagicVisibilityMap::CutShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # const EgMagicModels::cEgMagicVisibilityMap::sShape *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # EgMagicModels::cEgMagicVisibilityMap::sShape *
            ctypes.c_ulonglong,  # EgMagicModels::cEgMagicVisibilityMap::sShape *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "EgMagicModels::cEgMagicVisibilityMap::CalculateArea": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # const EgMagicModels::cEgMagicVisibilityMap::sShape *
        ]
    ),
    "EgMagicModels::sEgMagicBuffer<EgMagicModels::cEgMagicModel>::GetEntry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # EgMagicModels::sEgMagicBuffer<EgMagicModels::cEgMagicModel>::Entry *
        argtypes=[
            ctypes.c_ulonglong,  # EgMagicModels::sEgMagicBuffer<EgMagicModels::cEgMagicModel> *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "EgMagicModels::sEgMagicBuffer<EgMagicModels::cEgMagicWish>::GetEntry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # EgMagicModels::sEgMagicBuffer<EgMagicModels::cEgMagicWish>::Entry *
        argtypes=[
            ctypes.c_ulonglong,  # EgMagicModels::sEgMagicBuffer<EgMagicModels::cEgMagicWish> *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "EgMagicModels::BandSegments<18>::GetSegment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgMagicModels::BandSegments<18> *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # EgMagicModels::sSegment *
        ]
    ),
    "nNanoVG__stbtt__isfont": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int8 *
        ]
    ),
    "nNanoVG__stbtt__find_table": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "stbtt_GetFontOffsetForIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_int32,  # int
        ]
    ),
    "stbtt_InitFont": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbtt_fontinfo *
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_int32,  # int
        ]
    ),
    "stbtt_FindGlyphIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const nNanoVG::stbtt_fontinfo *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbtt__GetGlyfOffset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const nNanoVG::stbtt_fontinfo *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbtt__close_shape": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbtt_vertex *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "stbtt_GetGlyphShape": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const nNanoVG::stbtt_fontinfo *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # nNanoVG::stbtt_vertex **
        ]
    ),
    "stbtt_GetGlyphKernAdvance": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # const nNanoVG::stbtt_fontinfo *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbtt__fill_active_edges": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # nNanoVG::stbtt__active_edge *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbtt__rasterize_sorted_edges": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbtt__bitmap *
            ctypes.c_ulonglong,  # nNanoVG::stbtt__edge *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "nNanoVG__stbtt__edge_compare": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "nNanoVG__stbtt__rasterize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbtt__bitmap *
            ctypes.c_ulonglong,  # nNanoVG::stbtt__point *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "nNanoVG__stbtt__tesselate_curve": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbtt__point *
            ctypes.c_ulonglong,  # int *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG::stbtt_FlattenCurves": FUNCDEF(
        restype=ctypes.c_ulonglong,  # nNanoVG::stbtt__point *
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbtt_vertex *
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # int **
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "stbtt_MakeGlyphBitmapSubpixel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const nNanoVG::stbtt_fontinfo *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__fons__tmpalloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "nNanoVG__fons__atlasInsertNode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONSatlas *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__fons__atlasAddSkylineLevel": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONSatlas *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__fons__atlasAddRect": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONSatlas *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "nNanoVG__fons__addWhiteRect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONScontext *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG::fonsCreateInternal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # nNanoVG::FONScontext *
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONSparams *
        ]
    ),
    "nNanoVG__fons__allocFont": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONScontext *
        ]
    ),
    "nNanoVG::fonsAddFontMem": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONScontext *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG::fonsGetFontByName": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONScontext *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "nNanoVG__fons__allocGlyph": FUNCDEF(
        restype=ctypes.c_ulonglong,  # nNanoVG::FONSglyph *
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONSfont *
        ]
    ),
    "nNanoVG__fons__blurCols": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__fons__blurRows": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__fons__blur": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONScontext *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__fons__getQuad": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONScontext *
            ctypes.c_ulonglong,  # nNanoVG::FONSfont *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # nNanoVG::FONSglyph *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # nNanoVG::FONSquad *
        ]
    ),
    "nNanoVG__fons__flush": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONScontext *
        ]
    ),
    "nNanoVG__fons__getVertAlign": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONScontext *
            ctypes.c_ulonglong,  # nNanoVG::FONSfont *
            ctypes.c_int32,  # int
            ctypes.c_int16,  # __int16
        ]
    ),
    "nNanoVG::fonsTextIterInit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONScontext *
            ctypes.c_ulonglong,  # nNanoVG::FONStextIter *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "nNanoVG::fonsTextIterNext": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONScontext *
            ctypes.c_ulonglong,  # nNanoVG::FONStextIter *
            ctypes.c_ulonglong,  # nNanoVG::FONSquad *
        ]
    ),
    "nNanoVG::fonsTextBounds": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONScontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "nNanoVG::fonsLineBounds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONScontext *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "nNanoVG::fonsDeleteInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONScontext *
        ]
    ),
    "nNanoVG::fonsResetAtlas": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::FONScontext *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbi_load_main": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "nNanoVG__stbi__refill_buffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
        ]
    ),
    "nNanoVG__stbi__skip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbi__getn": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbi__get16be": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
        ]
    ),
    "nNanoVG__stbi__get32be": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
        ]
    ),
    "nNanoVG__stbi__get16le": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
        ]
    ),
    "nNanoVG__stbi__get32le": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
        ]
    ),
    "nNanoVG__stbi__convert_format": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "nNanoVG__stbi__build_huffman": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__huffman *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "nNanoVG__stbi__grow_buffer_unsafe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__jpeg *
        ]
    ),
    "nNanoVG__stbi__jpeg_decode_block": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__jpeg *
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_ulonglong,  # nNanoVG::stbi__huffman *
            ctypes.c_ulonglong,  # nNanoVG::stbi__huffman *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbi__idct_block": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "nNanoVG__stbi__get_marker": FUNCDEF(
        restype=ctypes.c_uint8,  # unsigned __int8
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__jpeg *
        ]
    ),
    "nNanoVG__stbi__parse_entropy_coded_data": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__jpeg *
        ]
    ),
    "nNanoVG__stbi__process_marker": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__jpeg *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbi__process_scan_header": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__jpeg *
        ]
    ),
    "nNanoVG__stbi__process_frame_header": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__jpeg *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__decode_jpeg_header": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__jpeg *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbi__resample_row_v_2": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbi__resample_row_h_2": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbi__resample_row_hv_2": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbi__resample_row_generic": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbi__cleanup_jpeg": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__jpeg *
        ]
    ),
    "nNanoVG__load_jpeg_image": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__jpeg *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "nNanoVG__stbi__jpeg_load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "zbuild_huffman": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # zhuffman *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbi__zexpand": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__zbuf *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbi__parse_huffman_block": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__zbuf *
        ]
    ),
    "nNanoVG__stbi__compute_huffman_codes": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__zbuf *
        ]
    ),
    "nNanoVG__stbi__parse_uncomperssed_block": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__zbuf *
        ]
    ),
    "nNanoVG__stbi__parse_zlib": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__zbuf *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbi__check_png_header": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
        ]
    ),
    "nNanoVG__stbi__create_png_image_raw": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__png *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "nNanoVG__stbi__create_png_image": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__png *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbi__parse_png_file": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__png *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "high_bit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "nNanoVG__stbi__bmp_load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "nNanoVG__stbi__tga_test": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
        ]
    ),
    "nNanoVG__stbi__tga_load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "nNanoVG__stbi__psd_load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "nNanoVG__stbi__pic_is4": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "nNanoVG__stbi__readval": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "nNanoVG__stbi__pic_load_core": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "nNanoVG__stbi__pic_load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "nNanoVG__stbi__gif_test_raw": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
        ]
    ),
    "nNanoVG__stbi__gif_header": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
            ctypes.c_ulonglong,  # nNanoVG::stbi__gif *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbi__out_gif_code": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__gif *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "nNanoVG__stbi__process_gif_raster": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
            ctypes.c_ulonglong,  # nNanoVG::stbi__gif *
        ]
    ),
    "nNanoVG__stbi__gif_load_next": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
            ctypes.c_ulonglong,  # nNanoVG::stbi__gif *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbi__gif_load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "nNanoVG__stbi__hdr_test": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
        ]
    ),
    "nNanoVG__stbi__hdr_gettoken": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "hdr_convert": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbi__hdr_load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # float *
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__context *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "nNanoVG__stbiw__jpg_writeBits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__write_context *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # const unsigned __int16 *
        ]
    ),
    "nNanoVG__stbiw__jpg_DCT": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "nNanoVG__stbi_write_jpg_core": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nNanoVG::stbi__write_context *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int32,  # int
        ]
    ),
    "nvg__deletePathCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGpathCache *
        ]
    ),
    "nvgCreateInternal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # NVGcontext *
        argtypes=[
            ctypes.c_ulonglong,  # NVGparams *
        ]
    ),
    "nvgBeginFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "nvgEndFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
        ]
    ),
    "nvgRGBf": FUNCDEF(
        restype=ctypes.c_ulonglong,  # NVGcolor *
        argtypes=[
            ctypes.c_ulonglong,  # NVGcolor *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "nvgRGBA": FUNCDEF(
        restype=ctypes.c_ulonglong,  # NVGcolor *
        argtypes=[
            ctypes.c_ulonglong,  # NVGcolor *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "nvgRGBAf": FUNCDEF(
        restype=ctypes.c_ulonglong,  # NVGcolor *
        argtypes=[
            ctypes.c_ulonglong,  # NVGcolor *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "nvgTransformScale": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "nvgTransformMultiply": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # const float *
        ]
    ),
    "nvgTransformInverse": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # const float *
        ]
    ),
    "nvgSave": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
        ]
    ),
    "nvgRestore": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
        ]
    ),
    "nvgReset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
        ]
    ),
    "nvgStrokeWidth": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
        ]
    ),
    "nvgLineCap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_int32,  # int
        ]
    ),
    "nvgGlobalAlpha": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
        ]
    ),
    "nvgTranslate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "nvgScale": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "nvgCurrentTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "nvgImagePattern": FUNCDEF(
        restype=ctypes.c_ulonglong,  # NVGpaint *
        argtypes=[
            ctypes.c_ulonglong,  # NVGpaint *
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "nvgScissor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "nvgIntersectScissor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "nvgResetScissor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
        ]
    ),
    "nvg__appendCommands": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_ulonglong,  # float *
            ctypes.c_int32,  # int
        ]
    ),
    "nvg__addPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
        ]
    ),
    "nvg__getAverageScale": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "nvg__allocTempVerts": FUNCDEF(
        restype=ctypes.c_ulonglong,  # NVGvertex *
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_int32,  # int
        ]
    ),
    "nvg__tesselateBezier": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "nvg__flattenPaths": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
        ]
    ),
    "nvg__roundJoin": FUNCDEF(
        restype=ctypes.c_ulonglong,  # NVGvertex *
        argtypes=[
            ctypes.c_ulonglong,  # NVGvertex *
            ctypes.c_ulonglong,  # NVGpoint *
            ctypes.c_ulonglong,  # NVGpoint *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
        ]
    ),
    "nvg__bevelJoin": FUNCDEF(
        restype=ctypes.c_ulonglong,  # NVGvertex *
        argtypes=[
            ctypes.c_ulonglong,  # NVGvertex *
            ctypes.c_ulonglong,  # NVGpoint *
            ctypes.c_ulonglong,  # NVGpoint *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "nvg__buttCapStart": FUNCDEF(
        restype=ctypes.c_ulonglong,  # NVGvertex *
        argtypes=[
            ctypes.c_ulonglong,  # NVGvertex *
            ctypes.c_ulonglong,  # NVGpoint *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "nvg__buttCapEnd": FUNCDEF(
        restype=ctypes.c_ulonglong,  # NVGvertex *
        argtypes=[
            ctypes.c_ulonglong,  # NVGvertex *
            ctypes.c_ulonglong,  # NVGpoint *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "nvg__calculateJoins": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "nvg__expandStroke": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "nvg__expandFill": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "nvgBeginPath": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
        ]
    ),
    "nvgMoveTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "nvgLineTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "nvgBezierTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "nvgArcTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "nvgPathWinding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_int32,  # int
        ]
    ),
    "nvgArc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
        ]
    ),
    "nvgRect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "nvgRoundedRect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "nvgEllipse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "nvgCircle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "nvgFill": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
        ]
    ),
    "nvgStroke": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
        ]
    ),
    "nvgCreateFontMem": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "nvgFontSize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
        ]
    ),
    "nvgFontBlur": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
        ]
    ),
    "nvgTextLetterSpacing": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
        ]
    ),
    "nvgTextAlign": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_int32,  # int
        ]
    ),
    "nvgFontFaceId": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_int32,  # int
        ]
    ),
    "nvgFontFace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "nvg__flushTextTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
        ]
    ),
    "nvg__allocTextAtlas": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
        ]
    ),
    "nvg__renderText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_ulonglong,  # NVGvertex *
            ctypes.c_int32,  # int
        ]
    ),
    "nvgText": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "nvgTextBox": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "nvgTextBreakLines": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # NVGtextRow *
            ctypes.c_int32,  # int
        ]
    ),
    "nvgTextBounds": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "nvgTextBoxBounds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "nvgTextMetrics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "nvg_malloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int64 *
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "nvg_calloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "nvg_free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "nvg_realloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _QWORD *
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgTrailNode::ResourceFactoryFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneGraphResource *
        argtypes=[
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "cEgTrailNode::SetParameterFloat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTrailNode *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_double,  # double
        ]
    ),
    "cEgTrailNode::PreTransferRenderData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTrailNode *
        ]
    ),
    "cEgTrailNode::cTkTrailVertex::cTkTrailVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTrailNode::cTkTrailVertex *
        ]
    ),
    "cEgTrailNode::cEgTrailNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTrailNode *
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # const cTkTypedSmartResHandle<cEgMaterialResource> *
        ]
    ),
    "cEgTrailNode::~cEgTrailNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTrailNode *
        ]
    ),
    "cEgTrailNode::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgTrailNode::FactoryFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "cEgTrailNode::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # RenderingOrder::List
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "cEgTrailNode::GetLinePosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cEgTrailNode *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgTrailNode::CreateTrail": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTrailNode *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgTrailNode::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTrailNode *
        ]
    ),
    "cEgTrailNode::UpdateVertexData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgTrailNode *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint32,  # eEgRenderParity
        ]
    ),
    "cEgTrailNode::EnqueueFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
            ctypes.c_ulonglong,  # cEgRenderQueueEntry *
            ctypes.c_ulonglong,  # cEgRenderableSceneNode **
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "TkParticle::TkParticleNodeTpl::GetBoundingBox": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAABB *
        argtypes=[
            ctypes.c_ulonglong,  # TkParticle::TkParticleNodeTpl *
            ctypes.c_ulonglong,  # cTkAABB *
            ctypes.c_ulonglong,  # const cTkResourceDescriptor *
            ctypes.c_ulonglong,  # cEgGeometryResource *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "TkParticle::InitExtension": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[]
    ),
    "TkParticle::AsyncBeginParticles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "TkParticle::BeginIncrementUpdateCounter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "TkParticle::Sync": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkParticle *
        ]
    ),
    "TkParticle::EnableEmitters": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "TkParticle::SetEmitterFade": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_float,  # float
        ]
    ),
    "TkParticle::SetEmitterStage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_float,  # float
        ]
    ),
    "TkParticle::StopEmitters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "TkParticle::StopEmittersAndAttachToProxy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "TkParticle::HasFinished": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "TkParticle::SetEmitterMaxRenderDistance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_float,  # float
        ]
    ),
    "TkParticle::SetUserColour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "TkParticle::SetColourisation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "TkParticle::SetSpawnOffsetMultiplier": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_float,  # float
        ]
    ),
    "TkParticle::NGuiRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkParticle *
        ]
    ),
    "TkParticle::GetEmitterFloatPropertyMax": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkEmitterFloatProperty *
        ]
    ),
    "TkParticle::GetEmitterFloatPropertyMin": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkEmitterFloatProperty *
        ]
    ),
    "TkParticle::cTkEmitterNode::GetEmitterFloatProperty": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNode *
            ctypes.c_ulonglong,  # const cTkParticleData *
            ctypes.c_ulonglong,  # const cTkEmitterFloatProperty *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "TkParticle::GetEmitterFloatPropertyIntegral": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkParticleData *
            ctypes.c_ulonglong,  # const cTkEmitterFloatProperty *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "TkParticle::CalculateBufferSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkParticleData *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const cTkParticleData *
        ]
    ),
    "TkParticle::cTkEmitterNode::cTkEmitterNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNode *
            ctypes.c_ulonglong,  # const TkParticle::TkParticleNodeTpl *
        ]
    ),
    "TkParticle::cTkEmitterNode::OnAttach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNode *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "TkParticle::cTkEmitterNode::OnDetach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNode *
            ctypes.c_ulonglong,  # cEgSceneNode *
        ]
    ),
    "TkParticle::cTkEmitterNode::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "TkParticle::cTkEmitterNode::FactoryFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "TkParticle::cTkEmitterNode::CreateParticleBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNode *
            ctypes.c_int32,  # int
        ]
    ),
    "TkParticle::cTkEmitterNode::EmitterBeginUpdate": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNode *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNodeAsyncState *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "TkParticle::cTkEmitterNode::SpawnParticleFromParticle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNode *
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNodeAsyncState *
            ctypes.c_ulonglong,  # const TkParticle::cTkParticle *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "TkParticle::cTkEmitterNode::EmitterAsyncUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNode *
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNodeAsyncState *
        ]
    ),
    "TkParticle::cTkEmitterNode::EmitterEndUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNode *
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNodeAsyncState *
        ]
    ),
    "TkParticle::cTkEmitterNode::SpawnParticle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNode *
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNodeAsyncState *
            ctypes.c_ulonglong,  # TkParticle::cTkParticle *
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "TkParticle::cTkEmitterNode::Sync": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNode *
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "TkParticle::cTkEmitterNode::DrawParticles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # RenderingOrder::List
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "TkParticle::cTkEmitterNode::RenderBatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNode *
            ctypes.c_ulonglong,  # cEgShaderCombination *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cEgShaderUniformBufferBinding *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # int *
            ctypes.c_uint32,  # eEgRenderParity
        ]
    ),
    "TkParticle::cTkEmitterNode::UpdateEmitter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNode *
            ctypes.c_float,  # float
        ]
    ),
    "TkParticle::cTkEmitterNode::GetParameterInt": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNode *
            ctypes.c_int32,  # int
        ]
    ),
    "TkParticle::cTkEmitterNode::SetPerDrawUniforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cEgShaderCombination *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
            ctypes.c_ulonglong,  # cEgShaderUniformBufferBinding *
            ctypes.c_ulonglong,  # cEgRenderQueueEntry *
            ctypes.c_ulonglong,  # TkParticle::cTkEmitterNode *
        ]
    ),
    "cTkMetadataCache<cTkParticleData>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkParticleData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cTkParticleData> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cTkParticleData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkParticleData *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "GPU::UberNoiseSOA4": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # GPU::vec3 *
            ctypes.c_int32,  # int
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "GPU::GenerateNoise2D": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # GPU::vec3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # GPU::vec3 *
        ]
    ),
    "GPU::TrySpawnObject_CheckDensity": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const GPU::sSpawnObjectGlobalParams *
            ctypes.c_ulonglong,  # const GPU::sSpawnObjectPerTypeParams *
            ctypes.c_ulonglong,  # const GPU::sSpawnObjectPerInstanceParams *
        ]
    ),
    "GPU::TrySpawnObject_CheckNoise": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # GPU::sSpawnObjectResult *
            ctypes.c_ulonglong,  # const GPU::sSpawnObjectGlobalParams *
            ctypes.c_ulonglong,  # const GPU::sSpawnObjectPerTypeParams *
            ctypes.c_ulonglong,  # const GPU::sSpawnObjectPerInstanceParams *
        ]
    ),
    "GPU::TrySpawnObject": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # GPU::sSpawnObjectResult *
            ctypes.c_ulonglong,  # const GPU::sSpawnObjectGlobalParams *
            ctypes.c_ulonglong,  # const GPU::sSpawnObjectPerTypeParams *
            ctypes.c_ulonglong,  # const GPU::sSpawnObjectPerInstanceParams *
        ]
    ),
    "squish::DecompressAlphaDxt3": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "squish::DecompressAlphaDxt5": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "squish::DecompressColour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgSingleLineNode::cEgSingleLineNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSingleLineNode *
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # const cTkTypedSmartResHandle<cEgMaterialResource> *
        ]
    ),
    "cEgSingleLineNode::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgSingleLineNode::FactoryFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "cEgSingleLineNode::EnqueueFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
            ctypes.c_ulonglong,  # cEgRenderQueueEntry *
            ctypes.c_ulonglong,  # cEgRenderableSceneNode **
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgSingleLineNode::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # RenderingOrder::List
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "cEgSingleLineNode::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSingleLineNode *
        ]
    ),
    "cEgSingleLineNode::SetParameterInt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSingleLineNode *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgSingleLineNode::SetParameterFloat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSingleLineNode *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_double,  # double
        ]
    ),
    "cEgSingleLineNode::UpdateLineBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgSingleLineNode *
        ]
    ),
    "EgDynamicVertexExtension::InitTerrainVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgDynamicVertexExtension *
        ]
    ),
    "EgDynamicVertexExtension::InitLineVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # EgDynamicVertexExtension *
        ]
    ),
    "EgDynamicVertexExtension::InitExtension": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "cEgLinesNodeTemplate<cTkLineVertexData,32,128>::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # RenderingOrder::List
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "cEgLinesNodeTemplate<cTkLineVertexData,32,128>::UpdateVertexData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgLinesNodeTemplate<cTkLineVertexData,32,128> *
            ctypes.c_ulonglong,  # cTkLineVertexData *
        ]
    ),
    "cEgLinesNodeTemplate<cTkLineVertexData,32,128>::ResourceFactoryFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneGraphResource *
        argtypes=[
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "cEgLinesNodeTemplate<cTkLineVertexData,32,128>::FactoryFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgRenderableSceneNode *
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "cEgVertexNode<cTkTerrainVertexData,32,120>::cEgVertexResource::SharedBuffers::SharedBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgVertexNode<cTkTerrainVertexData,32,120>::cEgVertexResource::SharedBuffers *
        ]
    ),
    "cEgVertexNode<cTkTerrainVertexData,32,120>::cEgVertexResource::GetElementParamInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgVertexNode<cTkTerrainVertexData,32,120>::cEgVertexResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgVertexNode<cTkTerrainVertexData,32,120>::cEgVertexResource::UnmapStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgVertexNode<cTkTerrainVertexData,32,120>::cEgVertexResource *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgVertexNode<cTkTerrainVertexData,32,120>::cEgVertexResource::MapResStream": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNode_vtbl *
        argtypes=[
            ctypes.c_ulonglong,  # cEgVertexNode<cTkTerrainVertexData,32,120>::cEgVertexResource *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cEgVertexNode<cTkTerrainVertexData,32,120>::ResourceFactoryFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneGraphResource *
        argtypes=[
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "cEgVertexNode<cTkTerrainVertexData,32,120>::SetParameterInt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgVertexNode<cTkTerrainVertexData,32,120> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgVertexNode<cTkLineVertexData,32,128>::SetParameterFloat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgVertexNode<cTkTerrainVertexData,32,120> *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
        ]
    ),
    "cEgVertexNode<cTkLineVertexData,32,128>::GetParameterInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgVertexNode<cTkTerrainVertexData,32,120> *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgVertexNode<cTkLineVertexData,32,128>::GetParameterFloat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cEgVertexNode<cTkTerrainVertexData,32,120> *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
        ]
    ),
    "cEgVertexNode<cTkTerrainVertexData,32,120>::~cEgVertexNode<cTkTerrainVertexData,32,120>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgVertexNode<cTkTerrainVertexData,32,120> *
        ]
    ),
    "cEgVertexNode<cTkLineVertexData,32,128>::cEgVertexResource::GetElementParamInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cEgVertexNode<cTkLineVertexData,32,128>::cEgVertexResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgVertexNode<cTkLineVertexData,32,128>::cEgVertexResource::UnmapStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgVertexNode<cTkLineVertexData,32,128>::cEgVertexResource *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cEgVertexNode<cTkLineVertexData,32,128>::cEgVertexResource::MapResStream": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNode_vtbl *
        argtypes=[
            ctypes.c_ulonglong,  # cEgVertexNode<cTkLineVertexData,32,128>::cEgVertexResource *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cEgVertexNode<cTkLineVertexData,32,128>::~cEgVertexNode<cTkLineVertexData,32,128>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgVertexNode<cTkLineVertexData,32,128> *
        ]
    ),
    "cEgVertexNode<cTkLineVertexData,32,128>::cEgVertexResource::SharedBuffers::SharedBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgVertexNode<cTkLineVertexData,32,128>::cEgVertexResource::SharedBuffers *
        ]
    ),
    "cEgVertexNode<cTkTerrainVertexData,32,120>::cEgVertexResource::PrintVertexBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgVertexNode<cTkTerrainVertexData,32,120>::cEgVertexResource *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cEgVertexNode<cTkLineVertexData,32,128>::cEgVertexResource::PrintVertexBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgVertexNode<cTkLineVertexData,32,128>::cEgVertexResource *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cEgVertexNode<cTkLineVertexData,32,128>::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgVertexNode<cTkLineVertexData,32,128>::UpdateVertexData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgVertexNode<cTkLineVertexData,32,128> *
            ctypes.c_ulonglong,  # cTkLineVertexData *
        ]
    ),
    "cEgVertexNode<cTkTerrainVertexData,32,120>::ParsingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgSceneNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_int32,  # int
        ]
    ),
    "cEgVertexNode<cTkTerrainVertexData,32,120>::FactoryFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgRenderableSceneNode *
        argtypes=[
            ctypes.c_ulonglong,  # const cEgSceneNodeTemplate *
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "cEgVertexNode<cTkTerrainVertexData,32,120>::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # RenderingOrder::List
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "cEgVertexNode<cTkTerrainVertexData,32,120>::RenderAABBs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueBuffer *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # cTkResource *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_ulonglong,  # cEgThreadableRenderCall *
        ]
    ),
    "cEgVertexNode<cTkTerrainVertexData,32,120>::EnqueueFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgRenderQueueInfo *
            ctypes.c_ulonglong,  # cEgRenderQueueEntry *
            ctypes.c_ulonglong,  # cEgRenderableSceneNode **
            ctypes.c_ulonglong,  # const cEgFrustum *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cEgVertexNode<cTkTerrainVertexData,32,120>::UpdateVertexData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cEgVertexNode<cTkTerrainVertexData,32,120> *
            ctypes.c_ulonglong,  # cTkTerrainVertexData *
        ]
    ),
    "findMapping": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _GLFWmapping *
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "findValidMapping": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _GLFWmapping *
        argtypes=[
            ctypes.c_ulonglong,  # const _GLFWjoystick *
        ]
    ),
    "parseMapping": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWmapping *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "glfwSetInputMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
        ]
    ),
    "glfwGetKeyName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "glfwGetKey": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_int32,  # int
        ]
    ),
    "glfwGetMouseButton": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_int32,  # int
        ]
    ),
    "glfwGetCursorPos": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_ulonglong,  # long double *
            ctypes.c_ulonglong,  # long double *
        ]
    ),
    "glfwSetCursorPos": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_double,  # long double
            ctypes.c_double,  # long double
        ]
    ),
    "glfwCreateStandardCursor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _GLFWcursor *
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "glfwDestroyCursor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GLFWcursor *
        ]
    ),
    "glfwSetCursor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_ulonglong,  # GLFWcursor *
        ]
    ),
    "glfwSetKeyCallback": FUNCDEF(
        restype=ctypes.c_ulonglong,  # {'return': 'void', 'args': ['struct GLFWwindow *', 'int', 'int', 'int', 'int']}
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['struct GLFWwindow *', 'int', 'int', 'int', 'int']}
        ]
    ),
    "glfwSetCharCallback": FUNCDEF(
        restype=ctypes.c_ulonglong,  # {'return': 'void', 'args': ['struct GLFWwindow *', {'type': 'unsigned', 'name': 'int'}]}
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['struct GLFWwindow *', {'type': 'unsigned', 'name': 'int'}]}
        ]
    ),
    "glfwSetScrollCallback": FUNCDEF(
        restype=ctypes.c_ulonglong,  # {'return': 'void', 'args': ['struct GLFWwindow *', {'type': 'long', 'name': 'double'}, {'type': 'long', 'name': 'double'}]}
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['struct GLFWwindow *', {'type': 'long', 'name': 'double'}, {'type': 'long', 'name': 'double'}]}
        ]
    ),
    "glfwUpdateGamepadMappings": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "glfwSwapInterval": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "glfwExtensionSupported": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "terminate_0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[]
    ),
    "glfwInit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "glfwTerminate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[]
    ),
    "compareVideoModes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "refreshVideoModes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWmonitor *
        ]
    ),
    "glfwGetMonitors": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _GLFWmonitor **
        argtypes=[
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "glfwGetPrimaryMonitor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _GLFWmonitor *
        argtypes=[]
    ),
    "glfwGetMonitorPos": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWmonitor *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "glfwGetMonitorName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWmonitor *
        ]
    ),
    "glfwGetVideoModes": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const GLFWvidmode *
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWmonitor *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "glfwGetVideoMode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const GLFWvidmode *
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWmonitor *
        ]
    ),
    "glfwCreateWindow": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _GLFWwindow *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # struct GLFWmonitor *
            ctypes.c_ulonglong,  # struct GLFWwindow *
        ]
    ),
    "glfwWindowHint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "glfwDestroyWindow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
        ]
    ),
    "glfwGetWindowPos": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "glfwSetWindowPos": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
        ]
    ),
    "glfwGetWindowSize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "glfwSetWindowSize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
        ]
    ),
    "glfwGetWindowAttrib": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_int32,  # int
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "glfwSetWindowMonitor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_ulonglong,  # struct GLFWmonitor *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "glfwSetWindowCloseCallback": FUNCDEF(
        restype=ctypes.c_ulonglong,  # {'return': 'void', 'args': ['struct GLFWwindow *']}
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['struct GLFWwindow *']}
        ]
    ),
    "glfwPollEvents": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "glfwGetRequiredInstanceExtensions": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char **
        argtypes=[
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "glfwGetPhysicalDevicePresentationSupport": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "glfwCreateWindowSurface": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_ulonglong,  # const VkAllocationCallbacks *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "monitorCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # HMONITOR__ *
            ctypes.c_ulonglong,  # HDC__ *
            ctypes.c_ulonglong,  # tagRECT *
            ctypes.c_int64,  # __int64
        ]
    ),
    "createMonitor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _GLFWmonitor *
        argtypes=[
            ctypes.c_ulonglong,  # _DISPLAY_DEVICEW *
            ctypes.c_ulonglong,  # _DISPLAY_DEVICEW *
        ]
    ),
    "CAkLFOCtx::GetParamsPtr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const wchar_t *
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWmonitor *
        ]
    ),
    "glfwGetMonitorAdapterName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const wchar_t *
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWmonitor *
        ]
    ),
    "getWindowStyle": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const _GLFWwindow *
        ]
    ),
    "applyAspectRatio": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # tagRECT *
        ]
    ),
    "updateCursorImage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
        ]
    ),
    "enableRawMouseMotion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
        ]
    ),
    "disableRawMouseMotion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
        ]
    ),
    "disableCursor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
        ]
    ),
    "enableCursor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
        ]
    ),
    "cursorInContentArea": FUNCDEF(
        restype=ctypes.c_ubyte,  # BOOL
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
        ]
    ),
    "updateFramebufferTransparency": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const _GLFWwindow *
            ctypes.c_int64,  # __int64
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "getKeyMods": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "getAsyncKeyMods": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "fitToMonitor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
        ]
    ),
    "acquireMonitor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
        ]
    ),
    "releaseMonitor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
        ]
    ),
    "windowProc": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # HWND__ *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "createNativeWindow": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
            ctypes.c_ulonglong,  # const _GLFWwndconfig *
            ctypes.c_ulonglong,  # const _GLFWfbconfig *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "glfwGetWin32Window": FUNCDEF(
        restype=ctypes.c_ulonglong,  # HWND__ *
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
        ]
    ),
    "compareJoystickObjects": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "closeJoystick": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWjoystick *
        ]
    ),
    "deviceObjectCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const DIDEVICEOBJECTINSTANCEW *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "deviceCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const DIDEVICEINSTANCEW *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "loadLibraries": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "createKeyTables": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "findPixelFormatAttribValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const int *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const int *
            ctypes.c_int32,  # int
        ]
    ),
    "choosePixelFormat": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
            ctypes.c_ulonglong,  # const _GLFWctxconfig *
            ctypes.c_ulonglong,  # const _GLFWfbconfig *
        ]
    ),
    "makeContextCurrentWGL": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
        ]
    ),
    "swapBuffersWGL": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
            ctypes.c_int64,  # __int64
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "swapIntervalWGL": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "extensionSupportedWGL": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "getProcAddressWGL": FUNCDEF(
        restype=ctypes.c_ulonglong,  # {'return': 'void', 'args': []}
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "destroyContextWGL": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
        ]
    ),
    "getEGLErrorString": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "chooseEGLConfig": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const _GLFWctxconfig *
            ctypes.c_ulonglong,  # const _GLFWfbconfig *
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "makeContextCurrentEGL": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
        ]
    ),
    "swapBuffersEGL": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
        ]
    ),
    "swapIntervalEGL": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "extensionSupportedEGL": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "getProcAddressEGL": FUNCDEF(
        restype=ctypes.c_ulonglong,  # {'return': 'void', 'args': []}
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "destroyContextEGL": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
        ]
    ),
    "makeContextCurrentOSMesa": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
        ]
    ),
    "getProcAddressOSMesa": FUNCDEF(
        restype=ctypes.c_ulonglong,  # {'return': 'void', 'args': []}
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "destroyContextOSMesa": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _GLFWwindow *
        ]
    ),
    "stbi_image_free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "stbi_load_from_memory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "stbi_loadf_from_memory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # float *
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "stbi_is_hdr_from_memory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
        ]
    ),
    "get8": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[]
    ),
    "get8u": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[]
    ),
    "skip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "get16": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "get16le": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "getn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
        ]
    ),
    "convert_format": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "ldr_to_hdr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # float *
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hdr_to_ldr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "build_huffman": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # huffman *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "grow_buffer_unsafe": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[]
    ),
    "decode_block": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_ulonglong,  # huffman *
            ctypes.c_ulonglong,  # huffman *
            ctypes.c_int32,  # int
        ]
    ),
    "idct_block": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "get_marker": FUNCDEF(
        restype=ctypes.c_uint8,  # unsigned __int8
        argtypes=[]
    ),
    "parse_entropy_coded_data": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "process_marker": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "process_scan_header": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "process_frame_header": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "decode_jpeg_header": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "resample_v_2": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "resample_hv_2": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cleanup_jpeg": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "load_jpeg_image": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "fill_bits": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[]
    ),
    "expand": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "parse_huffman_block": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "compute_huffman_codes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "parse_zlib": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "create_png_image": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
        ]
    ),
    "parse_png_file": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "do_png": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "bmp_test": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "bmp_load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "tga_test": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "tga_load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "hdr_test": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "hdr_gettoken": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "hdr_load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # float *
        argtypes=[
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "dds_test": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "stbi_decode_DXT1_block": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "stbi_decode_DXT45_alpha_block": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "stbi_decode_DXT_color_block": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "dds_load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMetaDataXML::ReadArrayXML<float>": {
        "cTkDynamicArray<float> *, XMLNode *, const char *, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDynamicArray<float> *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "float *, int, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # float *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "cTkMetaDataXML::ReadClassArrayXML<cTkNoiseFlattenPoint>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkNoiseFlattenPoint> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkVoxelGeneratorSettingsElement>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorSettingsElement *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkNoiseVoxelTypeEnum>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseVoxelTypeEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkVoxelGeneratorData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadXML<int>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadXML<float>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadXML<cTkFixedString<128,char>>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<int>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const int *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<float>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const float *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkNoiseCaveData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkNoiseCaveData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkNoiseFeatureData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkNoiseFeatureData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkNoiseFlattenPoint const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkNoiseFlattenPoint *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkNoiseGridData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkNoiseGridData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkNoiseUberLayerData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkNoiseUberLayerData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkVoxelGeneratorSettingsElement const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkVoxelGeneratorSettingsElement *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkNoiseVoxelTypeEnum>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseVoxelTypeEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNoiseVoxelTypeEnum *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkVoxelGeneratorData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkVoxelGeneratorData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNoiseVoxelTypeEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseVoxelTypeEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkVoxelGeneratorData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVoxelGeneratorData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkVoxelGeneratorRegionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVoxelGeneratorRegionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkVoxelGeneratorSettingsArray const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVoxelGeneratorSettingsArray *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkVoxelGeneratorSettingsElement const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVoxelGeneratorSettingsElement *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkVoxelGeneratorSettingsTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVoxelGeneratorSettingsTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cTkVoxelGeneratorSettingsElement,21>::cTkFixedArray<cTkVoxelGeneratorSettingsElement,21>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkVoxelGeneratorSettingsElement,21> *
        ]
    ),
    "cTkDynamicArray<cTkNoiseFlattenPoint>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkNoiseFlattenPoint> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNoiseVoxelTypeEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkVoxelGeneratorData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkVoxelGeneratorRegionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkVoxelGeneratorSettingsArray::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkVoxelGeneratorSettingsElement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkVoxelGeneratorSettingsTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkVoxelGeneratorData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkVoxelGeneratorRegionData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkVoxelGeneratorSettingsArray::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkVoxelGeneratorSettingsElement::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkVoxelGeneratorData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkVoxelGeneratorData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkVoxelGeneratorSettingsArray::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorSettingsArray *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNoiseVoxelTypeEnum::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseVoxelTypeEnum *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVoxelGeneratorData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVoxelGeneratorRegionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorRegionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVoxelGeneratorSettingsElement::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorSettingsElement *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadElementXML": {
        "cTkFixedString<128,char> *, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkFixedString<128,char> *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkSeed *, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkSeed *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "TkID<256> *, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # TkID<256> *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "TkID<128> *, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # TkID<128> *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkFixedString<32,char> *, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkFixedString<32,char> *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkFixedString<64,char> *, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkFixedString<64,char> *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkFixedString<256,char> *, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkFixedString<256,char> *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkFixedString<1024,char> *, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkFixedString<512,char> *, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkFixedString<512,char> *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkFixedString<2048,char> *, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkColour *, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkColour *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkHalfVector4 *, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkHalfVector4 *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkVector2 *, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkVector2 *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkVector3 *, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkVector3 *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkVector4 *, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkVector4 *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "cTkNoiseVoxelTypeEnum::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseVoxelTypeEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVoxelGeneratorData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVoxelGeneratorRegionData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorRegionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVoxelGeneratorSettingsTypes::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorSettingsTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateFlybyType::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDownloadType *
        ]
    ),
    "cTkVoxelGeneratorData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
        ]
    ),
    "cTkNoiseVoxelTypeEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseVoxelTypeEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVoxelGeneratorData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVoxelGeneratorRegionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorRegionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVoxelGeneratorSettingsArray::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorSettingsArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVoxelGeneratorSettingsElement::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorSettingsElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVoxelGeneratorSettingsTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorSettingsTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseVoxelTypeEnum::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseVoxelTypeEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNoiseVoxelTypeEnum *
        ]
    ),
    "cTkVoxelGeneratorData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkVoxelGeneratorData *
        ]
    ),
    "cTkVoxelGeneratorRegionData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorRegionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkVoxelGeneratorRegionData *
        ]
    ),
    "cTkVoxelGeneratorSettingsElement::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorSettingsElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkVoxelGeneratorSettingsElement *
        ]
    ),
    "cTkVoxelGeneratorSettingsTypes::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVoxelGeneratorSettingsTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkVoxelGeneratorSettingsTypes *
        ]
    ),
    "cTkMetaDataXML::WriteXML": {
        "const float *, XMLNode *, const char *, bool, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const float *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const bool *, XMLNode *, const char *, bool, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const bool *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const cTkSeed *, XMLNode *, const char *, bool, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const cTkSeed *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const int *, XMLNode *, const char *, bool, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const int *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const char *, XMLNode *, const char *, bool, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const unsigned __int64 *, XMLNode *, const char *, bool, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const unsigned __int64 *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const cTkColour *, XMLNode *, const char *, bool, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const cTkColour *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const cTkHalfVector4 *, XMLNode *, const char *, bool, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const cTkHalfVector4 *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const cTkVector2 *, XMLNode *, const char *, bool, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const cTkVector3 *, XMLNode *, const char *, bool, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const cTkVector4 *, XMLNode *, const char *, bool, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const cTkVector4 *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cTkMetaDataXML::ReadClassXML<cTkNoiseFeatureData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFeatureData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkNoiseOffsetEnum>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseOffsetEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkNoiseSuperFormulaData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseSuperFormulaData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadXML<bool>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkNoiseFeatureData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseFeatureData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNoiseFeatureData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkNoiseFlattenOptions>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseFlattenOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNoiseFlattenOptions *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkNoiseFlattenPoint>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseFlattenPoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNoiseFlattenPoint *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkNoiseOffsetEnum>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseOffsetEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNoiseOffsetEnum *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkNoiseSuperFormulaData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseSuperFormulaData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNoiseSuperFormulaData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkNoiseSuperPrimitiveData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseSuperPrimitiveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNoiseSuperPrimitiveData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkCavesEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkCavesEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkFeaturesEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFeaturesEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkGridLayersEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkGridLayersEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNoiseCaveData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseCaveData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNoiseFeatureData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseFeatureData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNoiseFlattenFixedPosition const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseFlattenFixedPosition *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNoiseFlattenOptions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseFlattenOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNoiseFlattenPoint const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseFlattenPoint *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNoiseGridData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseGridData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNoiseLayerData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseLayerData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNoiseLayersEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseLayersEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNoiseOffsetEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseOffsetEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNoiseSuperFormulaData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseSuperFormulaData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNoiseSuperPrimitiveData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseSuperPrimitiveData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNoiseUberData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseUberData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNoiseUberLayerData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNoiseUberLayerData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCavesEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkFeaturesEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkGridLayersEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNoiseCaveData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNoiseFeatureData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNoiseFlattenFixedPosition::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNoiseFlattenOptions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNoiseFlattenPoint::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNoiseGridData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNoiseLayerData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNoiseLayersEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNoiseOffsetEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNoiseSuperFormulaData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNoiseSuperPrimitiveData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNoiseUberData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNoiseUberLayerData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNoiseCaveData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNoiseFeatureData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNoiseGridData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNoiseUberLayerData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNoiseCaveData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseCaveData *
        ]
    ),
    "cTkNoiseFeatureData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFeatureData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNoiseGridData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseGridData *
        ]
    ),
    "cTkNoiseUberLayerData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseUberLayerData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNoiseCaveData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseCaveData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNoiseGridData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseGridData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNoiseCaveData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseCaveData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseFeatureData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFeatureData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseFlattenFixedPosition::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFlattenFixedPosition *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseFlattenOptions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFlattenOptions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseFlattenPoint::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFlattenPoint *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseGridData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseGridData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseLayerData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseLayerData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseOffsetEnum::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseOffsetEnum *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseSuperFormulaData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseSuperFormulaData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseSuperPrimitiveData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseSuperPrimitiveData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseUberData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseUberData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseUberLayerData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseUberLayerData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseCaveData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseCaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseFeatureData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFeatureData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseFlattenFixedPosition::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFlattenFixedPosition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseFlattenOptions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFlattenOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseFlattenPoint::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFlattenPoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseGridData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseGridData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseOffsetEnum::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseOffsetEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseSuperPrimitiveData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseSuperPrimitiveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseUberData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseUberData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseUberLayerData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseUberLayerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseCaveData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseCaveData *
        ]
    ),
    "cTkNoiseFeatureData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFeatureData *
        ]
    ),
    "cTkNoiseFlattenOptions::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFlattenOptions *
        ]
    ),
    "cTkNoiseFlattenPoint::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFlattenPoint *
        ]
    ),
    "cTkNoiseGridData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseGridData *
        ]
    ),
    "cTkNoiseUberLayerData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseUberLayerData *
        ]
    ),
    "cTkCavesEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCavesEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFeaturesEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFeaturesEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGridLayersEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGridLayersEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseCaveData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseCaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseFeatureData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFeatureData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseFlattenFixedPosition::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFlattenFixedPosition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseFlattenOptions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFlattenOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseFlattenPoint::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFlattenPoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseGridData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseGridData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseLayerData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseLayerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseLayersEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseLayersEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseOffsetEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseOffsetEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseSuperFormulaData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseSuperFormulaData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseSuperPrimitiveData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseSuperPrimitiveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseUberData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseUberData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseUberLayerData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseUberLayerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNoiseFeatureData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFeatureData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNoiseFeatureData *
        ]
    ),
    "cTkNoiseFlattenOptions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFlattenOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNoiseFlattenOptions *
        ]
    ),
    "cTkNoiseFlattenPoint::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFlattenPoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNoiseFlattenPoint *
        ]
    ),
    "cTkNoiseGridData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseGridData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNoiseGridData *
        ]
    ),
    "cTkNoiseLayerData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseLayerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNoiseLayerData *
        ]
    ),
    "cTkNoiseOffsetEnum::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseOffsetEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNoiseOffsetEnum *
        ]
    ),
    "cTkNoiseSuperFormulaData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseSuperFormulaData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNoiseSuperFormulaData *
        ]
    ),
    "cTkNoiseSuperPrimitiveData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseSuperPrimitiveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNoiseSuperPrimitiveData *
        ]
    ),
    "cTkNoiseUberData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseUberData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNoiseUberData *
        ]
    ),
    "cTkNoiseUberLayerData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseUberLayerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNoiseUberLayerData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkActionButtonMap>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkActionButtonMap> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkAxisPathMapping>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAxisPathMapping> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkButtonPathMapping>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkButtonPathMapping> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkChordPathMapping>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkChordPathMapping> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkControllerButtonLookup>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkControllerButtonLookup> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkInputEnum>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkInputEnum> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkOpenVRControllerLookup>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkOpenVRControllerLookup> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkPlatformButtonPair>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkPlatformButtonPair> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkInputEnum>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkInputHandEnum>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputHandEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadXML<TkID<256>>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # TkID<256> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadXML<TkID<128>>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadXML<cTkFixedString<32,char>>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkActionButtonMap const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkActionButtonMap *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkAxisPathMapping const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkAxisPathMapping *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkButtonPathMapping const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkButtonPathMapping *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkChordPathMapping const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkChordPathMapping *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkControllerButtonLookup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkControllerButtonLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkInputEnum const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkInputEnum *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkOpenVRControllerLookup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkOpenVRControllerLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkPlatformButtonPair const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkPlatformButtonPair *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkChordsImageLookup>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkChordsImageLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkChordsImageLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkInputEnum>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkInputEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkInputEnum *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkInputHandEnum>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkInputHandEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkInputHandEnum *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkActionButtonLookup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkActionButtonLookup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkActionButtonMap const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkActionButtonMap *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAxisImageLookup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAxisImageLookup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAxisPathMapping const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAxisPathMapping *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkButtonImageLookup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkButtonImageLookup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkButtonPathMapping const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkButtonPathMapping *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkChordPathMapping const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkChordPathMapping *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkChordsImageLookup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkChordsImageLookup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkControllerButtonLookup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkControllerButtonLookup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkControllerList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkControllerList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkControllerSpecification const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkControllerSpecification *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkInputValidation const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkInputValidation *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkOpenVRControllerList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkOpenVRControllerList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkOpenVRControllerLookup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkOpenVRControllerLookup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkPlatformButtonPair const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPlatformButtonPair *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkTriggerFeedbackData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkTriggerFeedbackData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cTkActionButtonMap>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkActionButtonMap> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkAxisPathMapping>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkButtonPathMapping> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkChordPathMapping>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkChordPathMapping> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkControllerButtonLookup>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkControllerButtonLookup> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkOpenVRControllerLookup>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkOpenVRControllerLookup> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkPlatformButtonPair>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkPlatformButtonPair> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkActionButtonLookup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkActionButtonMap::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAxisImageLookup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAxisPathMapping::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkButtonImageLookup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkButtonPathMapping::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkChordPathMapping::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkChordsImageLookup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkControllerButtonLookup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkControllerList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkControllerSpecification::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkInputValidation::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkOpenVRControllerList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkOpenVRControllerLookup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkPlatformButtonPair::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkTriggerFeedbackData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkActionButtonLookup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkActionButtonMap::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAxisImageLookup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAxisPathMapping::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkButtonImageLookup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkButtonPathMapping::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkChordPathMapping::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkChordsImageLookup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkControllerList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkControllerSpecification::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkOpenVRControllerList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcNPCProbabilityReactionData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingFamily> *
        ]
    ),
    "cTkDynamicArray<cTkChordPathMapping>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkChordPathMapping> *
        ]
    ),
    "cTkButtonImageLookup::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkButtonImageLookup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkChordsImageLookup::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkChordsImageLookup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAxisImageLookup::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkButtonImageLookup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkChordsImageLookup::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkChordsImageLookup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkControllerSpecification::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkControllerSpecification *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcHeavyAirList::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEntitlementRewardsTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkActionButtonMap::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkActionButtonMap *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAxisImageLookup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAxisImageLookup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAxisPathMapping::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAxisPathMapping *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkButtonImageLookup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkButtonImageLookup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkButtonPathMapping::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkButtonPathMapping *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkChordPathMapping::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkChordPathMapping *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkChordsImageLookup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkChordsImageLookup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkControllerButtonLookup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkControllerButtonLookup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkControllerSpecification::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkControllerSpecification *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkOpenVRControllerLookup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkOpenVRControllerLookup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPlatformButtonPair::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkPlatformButtonPair *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTriggerFeedbackData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkTriggerFeedbackData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkActionButtonMap::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkActionButtonMap *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAxisPathMapping::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAxisPathMapping *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkButtonPathMapping::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkButtonPathMapping *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkChordPathMapping::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkChordPathMapping *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkControllerSpecification::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkControllerSpecification *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPlatformButtonPair::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPlatformButtonPair *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTriggerFeedbackData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTriggerFeedbackData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAxisPathMapping::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkButtonPathMapping *
        ]
    ),
    "cTkChordPathMapping::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkChordPathMapping *
        ]
    ),
    "cTkControllerButtonLookup::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIdSceneFilename *
        ]
    ),
    "cTkOpenVRControllerLookup::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkOpenVRControllerLookup *
        ]
    ),
    "cTkActionButtonLookup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkActionButtonLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkActionButtonMap::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkActionButtonMap *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAxisImageLookup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAxisImageLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAxisPathMapping::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAxisPathMapping *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkButtonImageLookup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkButtonImageLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkButtonPathMapping::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkButtonPathMapping *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkChordPathMapping::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkChordPathMapping *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkChordsImageLookup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkChordsImageLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkControllerButtonLookup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkControllerButtonLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkControllerList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkControllerList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkControllerSpecification::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkControllerSpecification *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputValidation::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputValidation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkOpenVRControllerList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkOpenVRControllerList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkOpenVRControllerLookup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkOpenVRControllerLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPlatformButtonPair::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPlatformButtonPair *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTriggerFeedbackData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTriggerFeedbackData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkActionButtonMap::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkActionButtonMap *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkActionButtonMap *
        ]
    ),
    "cTkAxisPathMapping::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAxisPathMapping *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAxisPathMapping *
        ]
    ),
    "cTkButtonPathMapping::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkButtonPathMapping *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkButtonPathMapping *
        ]
    ),
    "cTkChordPathMapping::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkChordPathMapping *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkChordPathMapping *
        ]
    ),
    "cTkControllerButtonLookup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkControllerButtonLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkControllerButtonLookup *
        ]
    ),
    "cTkControllerSpecification::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkControllerSpecification *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkControllerSpecification *
        ]
    ),
    "cTkOpenVRControllerLookup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkOpenVRControllerLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkOpenVRControllerLookup *
        ]
    ),
    "cTkPlatformButtonPair::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPlatformButtonPair *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkPlatformButtonPair *
        ]
    ),
    "cTkTriggerFeedbackData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTriggerFeedbackData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkTriggerFeedbackData *
        ]
    ),
    "cTkMetaDataXML::ReadArrayXML<cTkFixedString<128,char>>": {
        "cTkFixedString<128,char> *, int, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkFixedString<128,char> *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkDynamicArray<cTkFixedString<128,char> > *, XMLNode *, const char *, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDynamicArray<cTkFixedString<128,char> > *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cTkMetaDataXML::ReadXML<cTkColour>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<cTkFixedString<128,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFixedString<128,char> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkImGuiWindowData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkImGuiWindowData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkImGuiWindowData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkImGuiWindowData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkImGuiWindowData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInputActionMapping2 const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInputActionMapping2 *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInputActionMapping const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInputActionMapping *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInputActions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInputActions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkImGuiData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkImGuiData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkImGuiSettings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkImGuiSettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkImGuiWindowData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkImGuiWindowData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkPostProcessData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPostProcessData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkRandomComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkRandomComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkSpeedLineData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkSpeedLineData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkTrailData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkTrailData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRandomComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRandomComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInputActionMapping2::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInputActionMapping::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInputActions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkImGuiData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkImGuiSettings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkImGuiWindowData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkPostProcessData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkRandomComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkSpeedLineData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkTrailData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInputActionMapping2::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActionMapping2 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputActionMapping::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActionMapping *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputActions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkImGuiData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkImGuiData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkImGuiSettings::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkImGuiSettings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkImGuiWindowData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkImGuiWindowData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPostProcessData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkPostProcessData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSpeedLineData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSpeedLineData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTrailData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrailData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputActionMapping2::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActionMapping2 *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputActionMapping::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActionMapping *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputActions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkImGuiData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImGuiData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkImGuiSettings::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImGuiSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkImGuiWindowData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImGuiWindowData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSpeedLineData::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSpeedLineData *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTrailData::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrailData *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputActionMapping2::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActionMapping2 *
        ]
    ),
    "cGcInputActionMapping::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryIndex *
        ]
    ),
    "cTkImGuiData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImGuiData *
        ]
    ),
    "cTkImGuiSettings::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImGuiSettings *
        ]
    ),
    "cTkImGuiWindowData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImGuiWindowData *
        ]
    ),
    "cTkSpeedLineData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSpeedLineData *
        ]
    ),
    "cGcInputActionMapping2::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActionMapping2 *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputActionMapping::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActionMapping *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputActions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkImGuiData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImGuiData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkImGuiSettings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImGuiSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkImGuiWindowData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImGuiWindowData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPostProcessData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPostProcessData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRandomComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRandomComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSpeedLineData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSpeedLineData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTrailData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrailData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputActions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInputActions *
        ]
    ),
    "cTkImGuiData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImGuiData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkImGuiData *
        ]
    ),
    "cTkImGuiSettings::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImGuiSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkImGuiSettings *
        ]
    ),
    "cTkImGuiWindowData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImGuiWindowData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkImGuiWindowData *
        ]
    ),
    "cTkPostProcessData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPostProcessData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkPostProcessData *
        ]
    ),
    "cTkSpeedLineData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSpeedLineData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkSpeedLineData *
        ]
    ),
    "cTkTrailData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrailData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkTrailData *
        ]
    ),
    "cTkMetaDataXML::ReadArrayXML<cTkVector3>": {
        "cTkDynamicArray<cTkVector3> *, XMLNode *, const char *, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDynamicArray<cTkVector3> *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cTkVector3 *, int, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkVector3 *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "cTkMetaDataXML::ReadClassArrayXML<cTkLSystemGlobalRestriction>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLSystemGlobalRestriction> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkLSystemGlobalVariation>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLSystemGlobalVariation> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkLSystemRule>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLSystemRule> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkLSystemRuleTemplate>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLSystemRuleTemplate> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkCoordinateOrientation>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCoordinateOrientation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkCurveType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCurveType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkEmitterFloatProperty>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEmitterFloatProperty *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadXML<cTkVector3>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<cTkVector3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkLSystemGlobalRestriction const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkLSystemGlobalRestriction *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkLSystemGlobalVariation const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkLSystemGlobalVariation *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkLSystemRule const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkLSystemRule *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkLSystemRuleTemplate const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkLSystemRuleTemplate *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkCoordinateOrientation>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkCoordinateOrientation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkCoordinateOrientation *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkCurveType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkCurveType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkCurveType *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkEmitterFloatProperty>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkEmitterFloatProperty *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkEmitterFloatProperty *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkLSystemRuleTemplate const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLSystemRuleTemplate *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkLSystemRulesData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLSystemRulesData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkMagicModelData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMagicModelData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkMaterialMetaData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMaterialMetaData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkParticleBurstData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkParticleBurstData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkParticleData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkParticleData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cTkMaterialShaderMillNode>::~cTkDynamicArray<cTkMaterialShaderMillNode>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMaterialShaderMillNode> *
        ]
    ),
    "cTkLSystemLocatorEntry::~cTkLSystemLocatorEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemLocatorEntry *
        ]
    ),
    "cTkMaterialShaderMillData::~cTkMaterialShaderMillData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillData *
        ]
    ),
    "cTkDynamicArray<cTkLSystemGlobalRestriction>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLSystemGlobalRestriction> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkLSystemGlobalVariation>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLSystemGlobalVariation> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkLSystemRule>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLSystemRule> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkLSystemRuleTemplate>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLSystemRuleTemplate> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkLSystemRuleTemplate::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLSystemRulesData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkMagicModelData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkMaterialMetaData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkParticleBurstData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkParticleData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLSystemRulesData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkMagicModelData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkMaterialMetaData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkParticleBurstData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkParticleData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cTkLSystemInnerRule>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLSystemInnerRule> *
        ]
    ),
    "cTkDynamicArray<cTkLSystemRule>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLSystemRule> *
        ]
    ),
    "cTkMagicModelData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnSequenceStep *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLSystemRulesData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemRulesData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkParticleData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkParticleData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLSystemRuleTemplate::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemRuleTemplate *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMagicModelData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMagicModelData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialMetaData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialMetaData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkParticleBurstData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkParticleBurstData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkParticleData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkParticleData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMagicModelData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMagicModelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialMetaData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialMetaData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkParticleData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkParticleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkParticleData::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkParticleData *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemRuleTemplate::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemRuleTemplate *
        ]
    ),
    "cTkMagicModelData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMagicModelData *
        ]
    ),
    "cTkMaterialMetaData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialMetaData *
        ]
    ),
    "cTkParticleData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkParticleData *
        ]
    ),
    "cTkLSystemRuleTemplate::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemRuleTemplate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemRulesData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemRulesData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMagicModelData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMagicModelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialMetaData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialMetaData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkParticleBurstData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkParticleBurstData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkParticleData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkParticleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemRuleTemplate::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemRuleTemplate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkLSystemRuleTemplate *
        ]
    ),
    "cTkMagicModelData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMagicModelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMagicModelData *
        ]
    ),
    "cTkMaterialMetaData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialMetaData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMaterialMetaData *
        ]
    ),
    "cTkParticleBurstData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkParticleBurstData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkParticleBurstData *
        ]
    ),
    "cTkParticleData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkParticleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkParticleData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkHeavyAirData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkHeavyAirData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkLSystemInnerRule>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLSystemInnerRule> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkLSystemLocatorEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLSystemLocatorEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkLSystemRestrictionData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLSystemRestrictionData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadXML<cTkVector2>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkHeavyAirData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkHeavyAirData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkInputFrame const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkInputFrame *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkLSystemInnerRule const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkLSystemInnerRule *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkLSystemLocatorEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkLSystemLocatorEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkLSystemRestrictionData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkLSystemRestrictionData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkCoordinateOrientation const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkCoordinateOrientation *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkEmitFromParticleInfo const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkEmitFromParticleInfo *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkEmitterData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkEmitterData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkEmitterFloatProperty const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkEmitterFloatProperty *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkFoliageData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFoliageData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkHeavyAirCollection const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkHeavyAirCollection *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkHeavyAirData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkHeavyAirData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkHeavyAirSystem const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkHeavyAirSystem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkIdSceneFilename const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkIdSceneFilename *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkInputAxisEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkInputAxisEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkInputEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkInputEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkInputFrame const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkInputFrame *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkInputFrameArray const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkInputFrameArray *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkInputHandEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkInputHandEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkLSystemGlobalRestriction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLSystemGlobalRestriction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkLSystemGlobalVariation const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLSystemGlobalVariation *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkLSystemInnerRule const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLSystemInnerRule *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkLSystemLocatorEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLSystemLocatorEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkLSystemRestrictionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLSystemRestrictionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkLSystemRule const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLSystemRule *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cTkHeavyAirData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkHeavyAirData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkLSystemInnerRule>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLSystemInnerRule> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkLSystemLocatorEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLSystemLocatorEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkCoordinateOrientation::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkEmitFromParticleInfo::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkEmitterData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkEmitterFloatProperty::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkFoliageData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkHeavyAirCollection::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkHeavyAirData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkHeavyAirSystem::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkIdSceneFilename::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkInputAxisEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkInputEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkInputFrame::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkInputFrameArray::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkInputHandEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLSystemGlobalRestriction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLSystemGlobalVariation::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLSystemInnerRule::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLSystemLocatorEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLSystemRestrictionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLSystemRule::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkEmitterFloatProperty::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkHeavyAirCollection::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLSystemGlobalRestriction::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLSystemInnerRule::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLSystemLocatorEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLSystemRule::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkEmitterFloatProperty::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEmitterFloatProperty *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLSystemGlobalRestriction::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemGlobalRestriction *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLSystemRule::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemRule *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkEmitterFloatProperty::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEmitterFloatProperty *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLSystemGlobalRestriction::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemGlobalRestriction *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLSystemInnerRule::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemInnerRule *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLSystemRule::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemRule *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkCoordinateOrientation::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkCoordinateOrientation *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEmitFromParticleInfo::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkEmitFromParticleInfo *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEmitterFloatProperty::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkEmitterFloatProperty *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFoliageData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkFoliageData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkHeavyAirData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkHeavyAirData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkHeavyAirSystem::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkHeavyAirSystem *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkIdSceneFilename::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkIdSceneFilename *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputAxisEnum::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputAxisEnum *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputEnum::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputEnum *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputFrame::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputFrame *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputHandEnum::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputHandEnum *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemGlobalRestriction::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemGlobalRestriction *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemGlobalVariation::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemGlobalVariation *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemInnerRule::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemInnerRule *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemLocatorEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemLocatorEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemRestrictionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemRestrictionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemRule::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemRule *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCoordinateOrientation::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCoordinateOrientation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEmitFromParticleInfo::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEmitFromParticleInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEmitterFloatProperty::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEmitterFloatProperty *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFoliageData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFoliageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkHeavyAirData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHeavyAirData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputAxisEnum::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputAxisEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputEnum::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputFrame::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputFrame *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputHandEnum::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputHandEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemGlobalRestriction::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemGlobalRestriction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemGlobalVariation::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemGlobalVariation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemInnerRule::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemInnerRule *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemRestrictionData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemRestrictionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemRule::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemRule *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasSendSubmitContribution::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendSubmitContribution *
        ]
    ),
    "cGcMissionSequenceExplorationLogSpecial::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDiscoveryMetadata *
        ]
    ),
    "cTkEmitterFloatProperty::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEmitterFloatProperty *
        ]
    ),
    "cTkFoliageData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFoliageData *
        ]
    ),
    "cTkHeavyAirData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHeavyAirData *
        ]
    ),
    "cTkHeavyAirSystem::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHeavyAirSystem *
        ]
    ),
    "cTkLSystemGlobalRestriction::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemGlobalRestriction *
        ]
    ),
    "cTkLSystemGlobalVariation::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemGlobalVariation *
        ]
    ),
    "cTkLSystemInnerRule::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemInnerRule *
        ]
    ),
    "cTkLSystemLocatorEntry::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemLocatorEntry *
        ]
    ),
    "cTkLSystemRule::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemRule *
        ]
    ),
    "cTkCoordinateOrientation::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCoordinateOrientation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEmitFromParticleInfo::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEmitFromParticleInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEmitterData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEmitterData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEmitterFloatProperty::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEmitterFloatProperty *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFoliageData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFoliageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkHeavyAirCollection::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHeavyAirCollection *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkHeavyAirData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHeavyAirData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkHeavyAirSystem::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHeavyAirSystem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkIdSceneFilename::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIdSceneFilename *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputAxisEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputAxisEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputFrame::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputFrame *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputFrameArray::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputFrameArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputHandEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputHandEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemGlobalRestriction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemGlobalRestriction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemGlobalVariation::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemGlobalVariation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemInnerRule::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemInnerRule *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemLocatorEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemLocatorEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemRestrictionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemRestrictionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemRule::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemRule *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCoordinateOrientation::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCoordinateOrientation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkCoordinateOrientation *
        ]
    ),
    "cTkEmitFromParticleInfo::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEmitFromParticleInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkEmitFromParticleInfo *
        ]
    ),
    "cTkEmitterFloatProperty::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEmitterFloatProperty *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkEmitterFloatProperty *
        ]
    ),
    "cTkFoliageData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFoliageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkFoliageData *
        ]
    ),
    "cTkHeavyAirData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHeavyAirData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkHeavyAirData *
        ]
    ),
    "cTkHeavyAirSystem::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHeavyAirSystem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkHeavyAirSystem *
        ]
    ),
    "cTkInputAxisEnum::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputAxisEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkInputAxisEnum *
        ]
    ),
    "cTkInputEnum::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkInputEnum *
        ]
    ),
    "cTkInputFrame::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputFrame *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkInputFrame *
        ]
    ),
    "cTkInputHandEnum::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputHandEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkInputHandEnum *
        ]
    ),
    "cTkLSystemInnerRule::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemInnerRule *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkLSystemInnerRule *
        ]
    ),
    "cTkLSystemLocatorEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemLocatorEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkLSystemLocatorEntry *
        ]
    ),
    "cTkLSystemRestrictionData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemRestrictionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkLSystemRestrictionData *
        ]
    ),
    "cTkMetaDataXML::ReadArrayXML<uchar>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<unsigned char> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadArrayXML<uint>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<unsigned int> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkProceduralTexture>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkProceduralTexture> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkProceduralTextureChosenOption>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkProceduralTextureChosenOption> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkProceduralTextureChosenOptionSampler>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkProceduralTextureChosenOptionSampler> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkProceduralTextureLayer>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureLayer *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkSketchNodeConnections>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkSketchNodeConnections> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkTrophyEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkTrophyEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkPaletteTexture>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPaletteTexture *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadXML<cTkFixedString<64,char>>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadXML<unsigned __int64>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<uchar>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<uint>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkProceduralTexture const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkProceduralTexture *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkProceduralTextureChosenOption const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkProceduralTextureChosenOption *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkProceduralTextureChosenOptionSampler const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkProceduralTextureChosenOptionSampler *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkProceduralTextureLayer const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkProceduralTextureLayer *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkSketchNodeConnections const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkSketchNodeConnections *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkTrophyEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkTrophyEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkPaletteTexture>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPaletteTexture *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkPaletteTexture *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkMaterialResource const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMaterialResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkPaletteTexture const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPaletteTexture *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkProbability const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkProbability *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkProceduralTexture const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkProceduralTexture *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkProceduralTextureChosenOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkProceduralTextureChosenOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkProceduralTextureChosenOptionList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkProceduralTextureChosenOptionList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkProceduralTextureChosenOptionSampler const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkProceduralTextureChosenOptionSampler *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkProceduralTextureLayer const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkProceduralTextureLayer *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkProceduralTextureList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkProceduralTextureList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkRawID const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkRawID *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkSketchConditions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkSketchConditions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkSketchNodeConnections const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkSketchNodeConnections *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkSketchNodeData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkSketchNodeData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkTrophyData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkTrophyData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkTrophyEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkTrophyEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkTrophyEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkTrophyEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cTkProceduralTexture>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkProceduralTexture> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkSketchNodeConnections>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimPoseExampleData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkTrophyEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkTrophyEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkModelResource::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkPaletteTexture::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkProbability::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkProceduralTexture::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkProceduralTextureChosenOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkProceduralTextureChosenOptionList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkProceduralTextureChosenOptionSampler::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkProceduralTextureLayer::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkProceduralTextureList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkRawID::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkSketchConditions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkSketchNodeConnections::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkSketchNodeData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkTextureResource::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkTrophyData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkTrophyEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkTrophyEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkProceduralTextureChosenOptionList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkProceduralTextureChosenOptionSampler::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkProceduralTextureLayer::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkProceduralTextureList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkSketchNodeConnections::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkSketchNodeData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkTrophyData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkProceduralTextureChosenOptionList::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOptionList *
        ]
    ),
    "cTkProceduralTextureChosenOptionList::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOptionList *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkSketchNodeData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchNodeData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkProceduralTextureChosenOptionList::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOptionList *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkProceduralTextureList::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureList *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkSketchNodeData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchNodeData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkModelResource::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelResource *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPaletteTexture::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkPaletteTexture *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralTexture::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTexture *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralTextureChosenOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralTextureChosenOptionList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOptionList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralTextureChosenOptionSampler::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOptionSampler *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralTextureLayer::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureLayer *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRawID::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRawID *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSketchNodeConnections::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchNodeConnections *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSketchNodeData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchNodeData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTextureResource::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkTextureResource *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTrophyEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrophyEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTextureResource::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkTextureResource *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkPaletteTexture::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkPaletteTexture *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkProceduralTextureChosenOption::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOption *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkProceduralTextureChosenOptionList::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOptionList *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkProceduralTextureChosenOptionSampler::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOptionSampler *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkTextureResource::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialResource *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPaletteTexture::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPaletteTexture *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralTexture::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTexture *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralTextureChosenOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralTextureChosenOptionList::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOptionList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralTextureLayer::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureLayer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRawID::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRawID *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSketchNodeData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchNodeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTextureResource::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialResource *
        ]
    ),
    "cTkPaletteTexture::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPaletteTexture *
        ]
    ),
    "cTkProceduralTexture::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTexture *
        ]
    ),
    "cTkProceduralTextureChosenOption::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOption *
        ]
    ),
    "cTkProceduralTextureList::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureList *
        ]
    ),
    "cTkSketchNodeData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchNodeData *
        ]
    ),
    "cTkTrophyEntry::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrophyEntry *
        ]
    ),
    "cTkModelResource::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTextureResource *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cTkPaletteTexture::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPaletteTexture *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cTkProceduralTextureChosenOptionList::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOptionList *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cTkProceduralTextureChosenOptionSampler::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOptionSampler *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cTkMaterialResource::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialResource *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPaletteTexture::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPaletteTexture *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProbability::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProbability *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralTexture::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTexture *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralTextureChosenOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralTextureChosenOptionList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOptionList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralTextureChosenOptionSampler::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOptionSampler *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralTextureLayer::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureLayer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralTextureList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRawID::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRawID *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSketchConditions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchConditions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSketchNodeConnections::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchNodeConnections *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSketchNodeData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchNodeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTrophyData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrophyData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTrophyEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrophyEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTrophyEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrophyEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSelectableObjectData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSelectableObjectData *
        ]
    ),
    "cTkPaletteTexture::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPaletteTexture *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkPaletteTexture *
        ]
    ),
    "cTkProceduralTexture::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTexture *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkProceduralTexture *
        ]
    ),
    "cTkProceduralTextureChosenOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkProceduralTextureChosenOption *
        ]
    ),
    "cTkProceduralTextureChosenOptionList::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureChosenOptionList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralTextureLayer::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralTextureLayer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkProceduralTextureLayer *
        ]
    ),
    "cTkSketchNodeData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchNodeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkSketchNodeData *
        ]
    ),
    "cTkTrophyEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrophyEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkTrophyEntry *
        ]
    ),
    "cTkMetaDataXML::ReadArrayXML<TkID<128>>": {
        "cTkDynamicArray<TkID<128> > *, XMLNode *, const char *, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDynamicArray<TkID<128> > *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "TkID<128> *, int, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # TkID<128> *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "cTkMetaDataXML::ReadClassArrayXML<cTkInteractiveControlData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkInteractiveControlData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkInteractiveSceneData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkInteractiveSceneData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkLanguageFontTableEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLanguageFontTableEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkLanguages>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLanguages> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkLocalisationEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLocalisationEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkLanguages>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguages *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadDynamicStringXML<char>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<char> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ITkDocumentReader::ReadValue<256>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cTkMetaDataXML::ReadXML<cTkFixedString<256,char>>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<char>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<TkID<128>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkInteractiveControlData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkInteractiveControlData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkInteractiveSceneData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkInteractiveSceneData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkLanguageFontTableEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkLanguageFontTableEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkLanguages const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkLanguages *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkLocalisationEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkLocalisationEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkCameraWanderData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkCameraWanderData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkCameraWanderData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkInteractiveControlData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkInteractiveControlData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkInteractiveControlData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkLanguages>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLanguages *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkLanguages *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkBoundingBoxData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBoundingBoxData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkCameraWanderData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkCameraWanderData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkEqualityEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkEqualityEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkFloatRange const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFloatRange *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkGlobals const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkIdArray const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkIdArray *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkInteractiveControlData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkInteractiveControlData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkInteractiveSceneData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkInteractiveSceneData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkInteractivityData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkInteractivityData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkLODModelResource const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLODModelResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkLanguageFontTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLanguageFontTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkLanguageFontTableEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLanguageFontTableEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkLanguages const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLanguages *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkLanguagesAllowedData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLanguagesAllowedData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkLocalisationEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLocalisationEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkLocalisationTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLocalisationTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkMetadataFilenameList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMetadataFilenameList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkMetadataReadMask const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMetadataReadMask *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkModelRendererCameraData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkModelRendererCameraData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkModelRendererData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkModelRendererData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkUserServiceAuthProvider const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkUserServiceAuthProvider *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLocalisationEntry::cTkLocalisationEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocalisationEntry *
            ctypes.c_ulonglong,  # cTkLocalisationEntry *
        ]
    ),
    "cTkDynamicArray<cTkInteractiveSceneData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkInteractiveSceneData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkLanguageFontTableEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLanguageFontTableEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkLocalisationEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLocalisationEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkBoundingBoxData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkCameraWanderData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkEqualityEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkFloatRange::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkGlobals::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkIdArray::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkInteractiveControlData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkInteractiveSceneData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkInteractivityData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLODModelResource::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLanguageFontTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLanguageFontTableEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLanguages::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLanguagesAllowedData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLocalisationEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLocalisationTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkMaterialResource::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkMetadataFilenameList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkMetadataReadMask::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkModelRendererCameraData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkModelRendererData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkUserServiceAuthProvider::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkIdArray::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkInteractiveSceneData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkInteractivityData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLODModelResource::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLanguageFontTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLanguagesAllowedData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLocalisationEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLocalisationTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkMetadataFilenameList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cTkLocalisationEntry>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLocalisationEntry> *
        ]
    ),
    "cTkLocalisationEntry::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocalisationEntry *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLocalisationTable::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocalisationTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLocalisationEntry::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocalisationEntry *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLocalisationTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocalisationTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkBoundingBoxData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBoundingBoxData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCameraWanderData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraWanderData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEqualityEnum::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkEqualityEnum *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFloatRange::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkFloatRange *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGlobals::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkGlobals *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkIdArray::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkIdArray *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInteractiveControlData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInteractiveControlData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInteractiveSceneData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInteractiveSceneData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLODModelResource::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODModelResource *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLanguageFontTableEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguageFontTableEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLanguages::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguages *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLocalisationEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocalisationEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialResource::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialResource *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkModelRendererCameraData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelRendererCameraData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkModelRendererData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelRendererData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLanguages::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguages *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkEqualityEnum::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEqualityEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFloatRange::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFloatRange *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkIdArray::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIdArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLODModelResource::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODModelResource *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLanguageFontTableEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguageFontTableEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLanguages::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguages *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLocalisationEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocalisationEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkModelRendererCameraData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelRendererCameraData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkModelRendererData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelRendererData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFloatRange::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendCrimeSpawnTable *
        ]
    ),
    "cTkGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGlobals *
        ]
    ),
    "cTkInteractiveControlData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInteractiveControlData *
        ]
    ),
    "cTkInteractiveSceneData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPartTypeData *
        ]
    ),
    "cTkLODModelResource::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODModelResource *
        ]
    ),
    "cTkLanguageFontTableEntry::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguageFontTableEntry *
        ]
    ),
    "cTkLocalisationEntry::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::MemoryMgr::GlobalStats *
        ]
    ),
    "cTkModelRendererCameraData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelRendererCameraData *
        ]
    ),
    "cTkModelRendererData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelRendererData *
        ]
    ),
    "cTkLanguages::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguages *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cTkBoundingBoxData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBoundingBoxData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCameraWanderData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraWanderData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEqualityEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEqualityEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFloatRange::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFloatRange *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGlobals::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkIdArray::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIdArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInteractiveControlData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInteractiveControlData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInteractiveSceneData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInteractiveSceneData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInteractivityData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInteractivityData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLODModelResource::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODModelResource *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLanguageFontTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguageFontTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLanguageFontTableEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguageFontTableEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLanguages::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguages *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLanguagesAllowedData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguagesAllowedData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLocalisationEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocalisationEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLocalisationTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocalisationTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataFilenameList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataFilenameList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetadataReadMask::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataReadMask *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkModelRendererCameraData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelRendererCameraData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkModelRendererData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelRendererData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkUserServiceAuthProvider::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceAuthProvider *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCameraWanderData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraWanderData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkCameraWanderData *
        ]
    ),
    "cTkEqualityEnum::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEqualityEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkEqualityEnum *
        ]
    ),
    "cTkFloatRange::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFloatRange *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkFloatRange *
        ]
    ),
    "cTkGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkGlobals *
        ]
    ),
    "cTkInteractiveControlData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInteractiveControlData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkInteractiveControlData *
        ]
    ),
    "cTkInteractiveSceneData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInteractiveSceneData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkInteractiveSceneData *
        ]
    ),
    "cTkLODModelResource::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODModelResource *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkLODModelResource *
        ]
    ),
    "cTkLanguageFontTableEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguageFontTableEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkLanguageFontTableEntry *
        ]
    ),
    "cTkLanguages::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguages *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkLanguages *
        ]
    ),
    "cTkLocalisationEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocalisationEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkLocalisationEntry *
        ]
    ),
    "cTkModelRendererCameraData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelRendererCameraData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkModelRendererCameraData *
        ]
    ),
    "cTkModelRendererData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelRendererData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkModelRendererData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkSceneBoneRemapping>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkSceneBoneRemapping> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkSceneNodeAttributeData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkSceneNodeAttributeData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkSceneNodeData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkSceneNodeData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkBlackboardCategory>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkSceneBoneRemapping const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkSceneBoneRemapping *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkSceneNodeAttributeData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkSceneNodeAttributeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkSceneNodeData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkSceneNodeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkBlackboardCategory>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkBlackboardCategory *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkBehaviourTreeConcurrentSelectorData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBehaviourTreeConcurrentSelectorData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkBehaviourTreeSucceedDecoratorData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBehaviourTreeSucceedDecoratorData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkBehaviourTreeSequentialSelectorData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBehaviourTreeSequentialSelectorData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkBlackboardCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkBlackboardComparisonTypeEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardComparisonTypeEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkBlackboardDefaultValueBool const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardDefaultValueBool *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkBlackboardDefaultValueFloat const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardDefaultValueFloat *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkBlackboardDefaultValueId const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardDefaultValueId *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkBlackboardDefaultValueInteger const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardDefaultValueInteger *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkBlackboardDefaultValueVector const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardDefaultValueVector *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkBlackboardKey const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardKey *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkBlackboardType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkBlackboardValueBool const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardValueBool *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkBlackboardValueFloat const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardValueFloat *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkBlackboardValueId const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardValueId *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkBlackboardValueInteger const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardValueInteger *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkBlackboardValueVector const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardValueVector *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkPhysRelVectorData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPhysRelVectorData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkSceneBoneRemapping const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkSceneBoneRemapping *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkSceneBoneRemappingTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkSceneBoneRemappingTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkSceneNodeAttributeData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkSceneNodeAttributeData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkSceneNodeData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkSceneNodeData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkTransformData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkTransformData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cTkSceneBoneRemapping>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkSceneBoneRemapping> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkBehaviourTreeConcurrentSelectorData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBehaviourTreeConcurrentSelectorData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBehaviourTreeSequentialSelectorData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBehaviourTreeSequentialSelectorData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBehaviourTreeSucceedDecoratorData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBehaviourTreeSucceedDecoratorData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBlackboardValueBool::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkBlackboardValueBool *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cTkBlackboardValueFloat::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkBlackboardValueFloat *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cTkBlackboardValueId::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkBlackboardValueId *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cTkBlackboardValueInteger::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkBlackboardValueInteger *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cTkBlackboardValueVector::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkBlackboardValueVector *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cTkBehaviourTreeConcurrentSelectorData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBehaviourTreePriorityDecoratorData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBehaviourTreeSequentialSelectorData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBehaviourTreeSucceedDecoratorData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBlackboardCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBlackboardComparisonTypeEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBlackboardDefaultValueBool::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBlackboardDefaultValueFloat::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBlackboardDefaultValueId::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBlackboardDefaultValueInteger::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBlackboardDefaultValueVector::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBlackboardKey::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBlackboardType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBlackboardValueBool::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBlackboardValueFloat::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBlackboardValueId::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBlackboardValueInteger::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBlackboardValueVector::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkPhysRelVectorData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkSceneBoneRemapping::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkSceneBoneRemappingTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkSceneNodeAttributeData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkSceneNodeData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkTransformData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkBehaviourTreeConcurrentSelectorData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkBehaviourTreePriorityDecoratorData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkBehaviourTreeSequentialSelectorData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkBehaviourTreeSucceedDecoratorData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkSceneBoneRemappingTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkSceneNodeData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkSceneNodeData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSceneNodeData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkSceneNodeData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSceneNodeData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkBehaviourTreeConcurrentSelectorData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeConcurrentSelectorData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBehaviourTreeSequentialSelectorData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeSequentialSelectorData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardCategory::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardCategory *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardComparisonTypeEnum::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardComparisonTypeEnum *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardDefaultValueBool::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueBool *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardDefaultValueFloat::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueFloat *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardDefaultValueId::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueId *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardDefaultValueInteger::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueInteger *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardDefaultValueVector::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueVector *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardKey::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardKey *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardValueBool::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardValueBool *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardValueFloat::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardValueFloat *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardValueId::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardValueId *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardValueVector::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardValueVector *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPhysRelVectorData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysRelVectorData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSceneBoneRemapping::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSceneBoneRemapping *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSceneNodeAttributeData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSceneNodeAttributeData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSceneNodeData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSceneNodeData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTransformData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkTransformData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardCategory::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardComparisonTypeEnum::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardComparisonTypeEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardDefaultValueBool::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueBool *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardDefaultValueFloat::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueFloat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardDefaultValueId::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueId *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardDefaultValueInteger::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueInteger *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardDefaultValueVector::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueVector *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardKey::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardKey *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPhysRelVectorData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysRelVectorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSceneNodeData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSceneNodeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTransformData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTransformData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardDefaultValueBool::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueBool *
        ]
    ),
    "cTkBlackboardDefaultValueFloat::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueInteger *
        ]
    ),
    "cTkBlackboardDefaultValueId::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueId *
        ]
    ),
    "cTkBlackboardDefaultValueVector::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueVector *
        ]
    ),
    "cTkBlackboardKey::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageSubstanceMined *
        ]
    ),
    "cTkPhysRelVectorData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysRelVectorData *
        ]
    ),
    "cTkSceneBoneRemapping::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGetInShip *
        ]
    ),
    "cTkSceneNodeAttributeData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSceneNodeAttributeData *
        ]
    ),
    "cTkSceneNodeData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSceneNodeData *
        ]
    ),
    "cTkBehaviourTreeConcurrentSelectorData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeConcurrentSelectorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBehaviourTreePriorityDecoratorData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeSucceedDecoratorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBehaviourTreeSequentialSelectorData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeSequentialSelectorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardComparisonTypeEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardComparisonTypeEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardDefaultValueBool::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueBool *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardDefaultValueFloat::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueFloat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardDefaultValueId::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueId *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardDefaultValueInteger::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueInteger *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardDefaultValueVector::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueVector *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardKey::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardKey *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardValueBool::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardValueBool *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardValueFloat::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardValueFloat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardValueId::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardValueId *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardValueInteger::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardValueInteger *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBlackboardValueVector::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardValueVector *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPhysRelVectorData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysRelVectorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSceneBoneRemapping::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSceneBoneRemapping *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSceneBoneRemappingTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSceneBoneRemappingTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSceneNodeAttributeData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSceneNodeAttributeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSceneNodeData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSceneNodeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTransformData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTransformData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBehaviourTreeConcurrentSelectorData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeConcurrentSelectorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkBehaviourTreeConcurrentSelectorData *
        ]
    ),
    "cTkBlackboardCategory::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkBlackboardCategory *
        ]
    ),
    "cTkBlackboardComparisonTypeEnum::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardComparisonTypeEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkBlackboardComparisonTypeEnum *
        ]
    ),
    "cTkBlackboardDefaultValueBool::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueBool *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkBlackboardDefaultValueBool *
        ]
    ),
    "cTkBlackboardDefaultValueFloat::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueFloat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkBlackboardDefaultValueFloat *
        ]
    ),
    "cTkBlackboardDefaultValueId::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueId *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkBlackboardDefaultValueId *
        ]
    ),
    "cTkBlackboardDefaultValueInteger::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueInteger *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkBlackboardDefaultValueInteger *
        ]
    ),
    "cTkBlackboardDefaultValueVector::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueVector *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkBlackboardDefaultValueVector *
        ]
    ),
    "cTkBlackboardKey::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardKey *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkBlackboardKey *
        ]
    ),
    "cTkSceneBoneRemapping::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSceneBoneRemapping *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkSceneBoneRemapping *
        ]
    ),
    "cTkSceneNodeAttributeData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSceneNodeAttributeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkSceneNodeAttributeData *
        ]
    ),
    "cTkSceneNodeData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSceneNodeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkSceneNodeData *
        ]
    ),
    "cTkTransformData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTransformData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkTransformData *
        ]
    ),
    "cTkMetaDataXML::ReadArrayXML<cTkVector4>": {
        "cTkDynamicArray<cTkVector4> *, XMLNode *, const char *, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDynamicArray<cTkVector4> *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cTkVector4 *, int, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkVector4 *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "cTkMetaDataXML::ReadClassArrayXML<cTkMaterialFlags>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMaterialFlags> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkMaterialSampler>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMaterialSampler> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkMaterialShaderMillComment>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMaterialShaderMillComment> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkMaterialShaderMillConnect>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMaterialShaderMillConnect> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkMaterialShaderMillFlag>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMaterialShaderMillFlag> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkMaterialShaderMillLink>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMaterialShaderMillLink> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkMaterialShaderMillNode>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMaterialShaderMillNode> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkMaterialUniform>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMaterialUniform> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkVertexElement>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkVertexElement> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadXML<cTkFixedString<1024,char>>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<1024,char> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadXML<__int64>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # __int64 *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<cTkVector4>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector4 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkMaterialFlags const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMaterialFlags *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkMaterialSampler const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMaterialSampler *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkMaterialShaderMillComment const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMaterialShaderMillComment *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkMaterialShaderMillConnect const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMaterialShaderMillConnect *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkMaterialShaderMillFlag const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMaterialShaderMillFlag *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkMaterialShaderMillLink const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMaterialShaderMillLink *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkMaterialShaderMillNode const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMaterialShaderMillNode *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkMaterialUniform const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMaterialUniform *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkVertexElement const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkVertexElement *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkMaterialAlternative const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMaterialAlternative *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkMaterialData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMaterialData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkMaterialFlags const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMaterialFlags *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkMaterialSampler const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMaterialSampler *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkMaterialShaderMillComment const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMaterialShaderMillComment *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkMaterialShaderMillConnect const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMaterialShaderMillConnect *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkMaterialShaderMillData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMaterialShaderMillData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkMaterialShaderMillFlag const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMaterialShaderMillFlag *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkMaterialShaderMillLink const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMaterialShaderMillLink *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkMaterialShaderMillNode const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMaterialShaderMillNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkMaterialUniform const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMaterialUniform *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkVertexLayout const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVertexLayout *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkVertexStream const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVertexStream *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cTkMaterialSampler>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMaterialSampler> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkMaterialShaderMillComment>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMaterialShaderMillComment> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkMaterialShaderMillConnect>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMaterialShaderMillConnect> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkMaterialShaderMillFlag>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMaterialShaderMillFlag> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkMaterialShaderMillLink>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMaterialShaderMillLink> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkMaterialShaderMillNode>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMaterialShaderMillNode> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkMaterialUniform>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMaterialUniform> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkVertexElement>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkVertexElement> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMaterialAlternative::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkMaterialData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkMaterialFlags::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkMaterialSampler::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkMaterialShaderMillComment::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkMaterialShaderMillConnect::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkMaterialShaderMillData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkMaterialShaderMillFlag::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkMaterialShaderMillLink::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkMaterialShaderMillNode::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkMaterialUniform::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkVertexLayout::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkVertexStream::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkMaterialData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkMaterialShaderMillData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkMaterialShaderMillNode::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkMaterialUniform::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkVertexLayout::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkVertexStream::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkMaterialShaderMillData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSubstanceSecondaryLookup::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceSecondaryLookup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkMaterialData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkMaterialShaderMillData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSubstanceSecondaryLookup::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceSecondaryLookup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkMaterialAlternative::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialAlternative *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialFlags::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialFlags *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialSampler::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialSampler *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialShaderMillComment::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillComment *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialShaderMillConnect::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillConnect *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialShaderMillData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialShaderMillFlag::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillFlag *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialShaderMillLink::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillLink *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialShaderMillNode::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillNode *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialUniform::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialUniform *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVertexLayout::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkVertexLayout *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialFlags::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialFlags *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialSampler::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialSampler *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialShaderMillComment::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillComment *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialShaderMillData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialShaderMillLink::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillLink *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialShaderMillNode::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialUniform::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialUniform *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVertexLayout::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVertexLayout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialAlternative::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialAlternative *
        ]
    ),
    "cTkMaterialData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialData *
        ]
    ),
    "cTkMaterialSampler::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialSampler *
        ]
    ),
    "cTkMaterialShaderMillComment::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillComment *
        ]
    ),
    "cTkMaterialShaderMillConnect::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillConnect *
        ]
    ),
    "cTkMaterialShaderMillData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillData *
        ]
    ),
    "cTkMaterialShaderMillLink::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillLink *
        ]
    ),
    "cTkMaterialShaderMillNode::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillNode *
        ]
    ),
    "cTkMaterialUniform::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialUniform *
        ]
    ),
    "cTkMaterialAlternative::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialAlternative *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialSampler::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialSampler *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialShaderMillData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialShaderMillFlag::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillFlag *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialShaderMillLink::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillLink *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialShaderMillNode::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialUniform::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialUniform *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVertexLayout::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVertexLayout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVertexStream::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVertexStream *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMaterialData *
        ]
    ),
    "cTkMaterialFlags::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialFlags *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMaterialSampler::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialSampler *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMaterialSampler *
        ]
    ),
    "cTkMaterialShaderMillComment::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillComment *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMaterialShaderMillComment *
        ]
    ),
    "cTkMaterialShaderMillConnect::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillConnect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMaterialShaderMillConnect *
        ]
    ),
    "cTkMaterialShaderMillData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMaterialShaderMillData *
        ]
    ),
    "cTkMaterialShaderMillLink::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillLink *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMaterialShaderMillLink *
        ]
    ),
    "cTkMaterialShaderMillNode::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialShaderMillNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMaterialShaderMillNode *
        ]
    ),
    "cTkMaterialUniform::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMaterialUniform *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMaterialUniform *
        ]
    ),
    "cTkVertexLayout::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVertexLayout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkVertexLayout *
        ]
    ),
    "cTkMetaDataXML::ReadArrayXML<ushort>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<unsigned short> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadArrayXML<int>": {
        "cTkDynamicArray<int> *, XMLNode *, const char *, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDynamicArray<int> *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "int *, int, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # int *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "cTkMetaDataXML::ReadArrayXML<cTkFixedString<32,char>>": {
        "cTkDynamicArray<cTkFixedString<32,char> > *, XMLNode *, const char *, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDynamicArray<cTkFixedString<32,char> > *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cTkFixedString<32,char> *, int, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkFixedString<32,char> *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "cTkMetaDataXML::ReadArrayXML<cTkHalfVector4>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkHalfVector4> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkAnimNodeData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimNodeData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkAnimNodeFrameData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimNodeFrameData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkAnimNodeFrameHalfData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimNodeFrameHalfData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkJointBindingData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkJointBindingData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkJointExtentData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkJointExtentData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkJointMirrorAxis>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkJointMirrorAxis> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkMeshData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMeshData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkMeshMetaData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMeshMetaData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkVertexLayout>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVertexLayout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<ushort>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int16 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<cTkFixedString<32,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<cTkHalfVector4>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkHalfVector4 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkAnimNodeData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkAnimNodeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkAnimNodeFrameData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkAnimNodeFrameData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkAnimNodeFrameHalfData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkAnimNodeFrameHalfData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkJointBindingData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkJointBindingData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkJointExtentData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkJointExtentData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkJointMirrorAxis const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkJointMirrorAxis *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkMeshData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMeshData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkMeshMetaData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkMeshMetaData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkAnimNodeFrameData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimNodeFrameData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimNodeFrameData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkAnimNodeFrameHalfData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimNodeFrameHalfData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimNodeFrameHalfData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimCompactMetadata const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimCompactMetadata *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimMetadata const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimMetadata *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimNodeData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimNodeData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimNodeFrameData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimNodeFrameData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimNodeFrameHalfData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimNodeFrameHalfData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkGeometryData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkGeometryData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkGeometryStreamData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkGeometryStreamData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkIndexStream const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkIndexStream *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkJointBindingData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkJointBindingData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkJointExtentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkJointExtentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkJointMirrorAxis const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkJointMirrorAxis *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkMeshData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMeshData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkMeshMetaData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMeshMetaData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkPhysicsData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPhysicsData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkRagdollData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkRagdollData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkVertexElement const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVertexElement *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimNodeFrameHalfData::~cTkAnimNodeFrameHalfData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimNodeFrameHalfData *
        ]
    ),
    "cTkGeometryData::~cTkGeometryData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGeometryData *
        ]
    ),
    "cTkDynamicArray<cTkFixedString<32,char>>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkFixedString<32,char> > *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkAnimNodeData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimNodeData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkAnimNodeFrameData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimNodeFrameHalfData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkHalfVector4>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkHalfVector4> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkJointBindingData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkJointBindingData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkJointExtentData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkJointExtentData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkJointMirrorAxis>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkJointMirrorAxis> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkMeshData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMeshData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkMeshMetaData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkMeshMetaData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkRagdollData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRagdollData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimCompactMetadata::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimMetadata::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimNodeData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimNodeFrameData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimNodeFrameHalfData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkGeometryData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkGeometryStreamData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkIndexStream::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkJointBindingData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkJointExtentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkJointMirrorAxis::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkMeshData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkMeshMetaData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkPhysicsData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkRagdollData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkVertexElement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimCompactMetadata::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimMetadata::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimNodeFrameData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimNodeFrameHalfData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkGeometryData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkGeometryStreamData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkIndexStream::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkMeshData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkRagdollData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimMetadata::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimMetadata *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkGeometryData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGeometryData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkGeometryStreamData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGeometryStreamData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimCompactMetadata::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimCompactMetadata *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimMetadata::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimMetadata *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimNodeData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimNodeData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimNodeFrameData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimNodeFrameData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimNodeFrameHalfData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimNodeFrameHalfData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGeometryData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkGeometryData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkJointBindingData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkJointBindingData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkJointExtentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkJointExtentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkJointMirrorAxis::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkJointMirrorAxis *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMeshData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMeshData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMeshMetaData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMeshMetaData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPhysicsData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRagdollData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRagdollData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVertexElement::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkVertexElement *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimNodeData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimNodeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGeometryData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGeometryData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkJointBindingData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkJointBindingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkJointExtentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkJointExtentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkJointMirrorAxis::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkJointMirrorAxis *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMeshData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMeshData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMeshMetaData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMeshMetaData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPhysicsData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVertexElement::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVertexElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimNodeData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimNodeData *
        ]
    ),
    "cTkMeshData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMeshData *
        ]
    ),
    "cTkMeshMetaData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMeshMetaData *
        ]
    ),
    "cTkPhysicsData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsData *
        ]
    ),
    "cTkVertexElement::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateClassCost *
        ]
    ),
    "cTkAnimCompactMetadata::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimCompactMetadata *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimMetadata::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimMetadata *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimNodeFrameData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimNodeFrameData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimNodeFrameHalfData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimNodeFrameHalfData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGeometryData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGeometryData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGeometryStreamData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGeometryStreamData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkIndexStream::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIndexStream *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkJointBindingData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkJointBindingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkJointExtentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkJointExtentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkJointMirrorAxis::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkJointMirrorAxis *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMeshData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMeshData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPhysicsData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRagdollData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRagdollData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVertexElement::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVertexElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimCompactMetadata::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimCompactMetadata *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimCompactMetadata *
        ]
    ),
    "cTkAnimMetadata::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimMetadata *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimMetadata *
        ]
    ),
    "cTkAnimNodeData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimNodeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimNodeData *
        ]
    ),
    "cTkAnimNodeFrameData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimNodeFrameData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimNodeFrameHalfData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimNodeFrameHalfData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGeometryData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGeometryData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkGeometryData *
        ]
    ),
    "cTkJointBindingData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkJointBindingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkJointExtentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkJointExtentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkJointMirrorAxis::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkJointMirrorAxis *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkJointMirrorAxis *
        ]
    ),
    "cTkMeshData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMeshData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMeshData *
        ]
    ),
    "cTkMeshMetaData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMeshMetaData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMeshMetaData *
        ]
    ),
    "cTkPhysicsData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkPhysicsData *
        ]
    ),
    "cTkVertexElement::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVertexElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkVertexElement *
        ]
    ),
    "cTkMetaDataXML::ReadArrayXML<cTkFixedString<256,char>>": {
        "cTkFixedString<256,char> *, int, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkFixedString<256,char> *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkDynamicArray<cTkFixedString<256,char> > *, XMLNode *, const char *, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDynamicArray<cTkFixedString<256,char> > *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cTkMetaDataXML::ReadClassArrayXML<cTkEntitlementListData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkEntitlementListData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkNGuiEditorStyleColour>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditorStyleColour *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkNGuiLayoutListData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkNGuiLayoutListData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkNGuiTextStyle>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextStyle *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkNGuiGraphicStyleData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkNGuiTextStyleData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextStyleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<cTkFixedString<256,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkEntitlementListData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkEntitlementListData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkNGuiEditorStyleColour const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkNGuiEditorStyleColour *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkNGuiGraphicStyle const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkNGuiGraphicStyle *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkNGuiLayoutListData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkNGuiLayoutListData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkNGuiTextStyle const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkNGuiTextStyle *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkNGuiWindowLayoutData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkNGuiWindowLayoutData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkNGuiAlignment>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiAlignment *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNGuiAlignment *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkNGuiGraphicStyleData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiGraphicStyleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNGuiGraphicStyleData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkNGuiLayoutListData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiLayoutListData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNGuiLayoutListData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkNGuiTextStyleData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiTextStyleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNGuiTextStyleData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTKNGuiEditorComponentSize const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTKNGuiEditorComponentSize *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTKNGuiEditorTextType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTKNGuiEditorTextType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkEntitlementList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkEntitlementList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNGuiAlignment const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiAlignment *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNGuiEditorGraphicType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiEditorGraphicType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNGuiEditorLayout const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiEditorLayout *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNGuiEditorStyleColour const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiEditorStyleColour *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNGuiEditorStyleData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiEditorStyleData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNGuiForcedStyle const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiForcedStyle *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNGuiGraphicStyle const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiGraphicStyle *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNGuiGraphicStyleData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiGraphicStyleData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNGuiLayoutList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiLayoutList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNGuiLayoutListData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiLayoutListData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNGuiRectanglePulseEffect const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiRectanglePulseEffect *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNGuiTextStyle const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiTextStyle *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNGuiTextStyleData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiTextStyleData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNGuiUserSettings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiUserSettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNGuiWindowLayoutData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiWindowLayoutData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cTkNGuiGraphicStyle,72>::cTkFixedArray<cTkNGuiGraphicStyle,72>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkNGuiGraphicStyle,72> *
        ]
    ),
    "cTkFixedArray<cTkNGuiTextStyle,14>::cTkFixedArray<cTkNGuiTextStyle,14>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkNGuiTextStyle,14> *
        ]
    ),
    "cTkFixedArray<cTkNGuiWindowLayoutData,256>::cTkFixedArray<cTkNGuiWindowLayoutData,256>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkNGuiWindowLayoutData,256> *
        ]
    ),
    "cTkDynamicArray<cTkEntitlementListData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkEntitlementListData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkNGuiLayoutListData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkNGuiLayoutListData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTKNGuiEditorComponentSize::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTKNGuiEditorTextType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkEntitlementList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNGuiAlignment::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNGuiEditorGraphicType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNGuiEditorLayout::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNGuiEditorStyleColour::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNGuiEditorStyleData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNGuiForcedStyle::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNGuiGraphicStyle::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNGuiGraphicStyleData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNGuiLayoutList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNGuiLayoutListData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNGuiRectanglePulseEffect::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNGuiTextStyle::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNGuiTextStyleData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNGuiUserSettings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNGuiWindowLayoutData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkEntitlementList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNGuiEditorStyleData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNGuiGraphicStyle::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNGuiLayoutList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNGuiRectanglePulseEffect::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNGuiGraphicStyle::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyle *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWaterColourSettingList::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaterColourSettingList *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNGuiRectanglePulseEffect::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiRectanglePulseEffect *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNGuiAlignment::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiAlignment *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditorStyleColour::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditorStyleColour *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditorStyleData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditorStyleData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiForcedStyle::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiForcedStyle *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiGraphicStyle::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyle *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiGraphicStyleData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyleData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiLayoutListData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiLayoutListData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiRectanglePulseEffect::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiRectanglePulseEffect *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiTextStyle::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextStyle *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiTextStyleData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextStyleData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiUserSettings::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiUserSettings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiWindowLayoutData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiWindowLayoutData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiAlignment::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiAlignment *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditorStyleData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditorStyleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiForcedStyle::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiForcedStyle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiGraphicStyle::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiGraphicStyleData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiRectanglePulseEffect::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiRectanglePulseEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiTextStyle::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextStyle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiTextStyleData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextStyleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiWindowLayoutData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiWindowLayoutData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditorLayout::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditorLayout *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiAlignment::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiAlignment *
        ]
    ),
    "cTkNGuiEditorStyleColour::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditorStyleColour *
        ]
    ),
    "cTkNGuiEditorStyleData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditorStyleData *
        ]
    ),
    "cTkNGuiGraphicStyle::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyle *
        ]
    ),
    "cTkNGuiGraphicStyleData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyleData *
        ]
    ),
    "cTkNGuiLayoutListData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiLayoutListData *
        ]
    ),
    "cTkNGuiRectanglePulseEffect::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiRectanglePulseEffect *
        ]
    ),
    "cTkNGuiTextStyle::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextStyle *
        ]
    ),
    "cTkNGuiTextStyleData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextStyleData *
        ]
    ),
    "cTkNGuiUserSettings::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiUserSettings *
        ]
    ),
    "cTkNGuiWindowLayoutData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiWindowLayoutData *
        ]
    ),
    "cTKNGuiEditorComponentSize::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTKNGuiEditorComponentSize *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTKNGuiEditorTextType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTKNGuiEditorTextType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEntitlementList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEntitlementList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiAlignment::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiAlignment *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditorGraphicType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditorGraphicType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditorLayout::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditorLayout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditorStyleColour::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditorStyleColour *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditorStyleData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditorStyleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiForcedStyle::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiForcedStyle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiGraphicStyle::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiGraphicStyleData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiLayoutList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiLayoutList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiLayoutListData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiLayoutListData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiRectanglePulseEffect::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiRectanglePulseEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiTextStyle::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextStyle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiTextStyleData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextStyleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiUserSettings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiUserSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiWindowLayoutData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiWindowLayoutData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiAlignment::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiAlignment *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNGuiAlignment *
        ]
    ),
    "cTkNGuiEditorStyleColour::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditorStyleColour *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNGuiEditorStyleColour *
        ]
    ),
    "cTkNGuiGraphicStyle::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNGuiGraphicStyle *
        ]
    ),
    "cTkNGuiGraphicStyleData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNGuiGraphicStyleData *
        ]
    ),
    "cTkNGuiLayoutListData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiLayoutListData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNGuiLayoutListData *
        ]
    ),
    "cTkNGuiRectanglePulseEffect::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiRectanglePulseEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNGuiRectanglePulseEffect *
        ]
    ),
    "cTkNGuiTextStyle::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextStyle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNGuiTextStyle *
        ]
    ),
    "cTkNGuiTextStyleData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextStyleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNGuiTextStyleData *
        ]
    ),
    "cTkNGuiWindowLayoutData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiWindowLayoutData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNGuiWindowLayoutData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcInputActionMapping2>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInputActionMapping2> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcInputActionMapping>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInputActionMapping> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkAnimDetailSettings>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimDetailSettings *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkAnimDetailSettingsData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimDetailSettingsData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkGraphicsDetailTypes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsDetailTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<bool>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const bool *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInputActionMapping2 const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInputActionMapping2 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInputActionMapping const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInputActionMapping *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkAnimDetailSettings const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkAnimDetailSettings *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkAnimDetailSettingsData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkAnimDetailSettingsData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkAnimDetailSettingsData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimDetailSettingsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimDetailSettingsData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkDynamicResScalingSettings>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicResScalingSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkDynamicResScalingSettings *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkGraphicsDetailTypes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkGraphicsDetailTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkGraphicsDetailTypes *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimDetailSettings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimDetailSettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimDetailSettingsData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimDetailSettingsData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimDetailSettingsTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimDetailSettingsTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkDynamicResScalingSettings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicResScalingSettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkEngineSettingTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkEngineSettingTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkEngineSettingsMapping const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkEngineSettingsMapping *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkGameSettings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkGameSettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkGraphicsDetailPreset const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkGraphicsDetailPreset *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkGraphicsDetailTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkGraphicsDetailTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkGraphicsSettings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsSettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkLODDistances const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLODDistances *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkLODSettingsData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLODSettingsData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cTkAnimDetailSettingsData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimDetailSettingsData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAnimDetailSettings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimDetailSettingsData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimDetailSettingsTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkDynamicResScalingSettings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkEngineSettingTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkEngineSettingsMapping::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkGameSettings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkGraphicsDetailPreset::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkGraphicsDetailTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkGraphicsSettings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLODDistances::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLODSettingsData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimDetailSettings::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimDetailSettingsTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkGameSettings::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkGraphicsSettings::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimDetailSettingsTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimDetailSettingsTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkGameSettings::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGameSettings *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkGraphicsSettings::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEffectTrigger *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimDetailSettings::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimDetailSettings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimDetailSettingsData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimDetailSettingsData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicResScalingSettings::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicResScalingSettings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEngineSettingsMapping::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettingsMapping *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGraphicsDetailPreset::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsDetailPreset *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGraphicsDetailTypes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsDetailTypes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGraphicsSettings::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsSettings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLODDistances::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODDistances *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLODSettingsData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODSettingsData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicResScalingSettings::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicResScalingSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEngineSettingsMapping::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettingsMapping *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGameSettings::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGameSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGraphicsDetailPreset::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsDetailPreset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGraphicsDetailTypes::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsDetailTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGraphicsSettings::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLODDistances::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODDistances *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLODSettingsData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODSettingsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGraphicsDetailPreset::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsDetailPreset *
        ]
    ),
    "cGcWealthClass::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityCommonFactions *
        ]
    ),
    "cTkGraphicsSettings::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsSettings *
        ]
    ),
    "cTkLODDistances::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODDistances *
        ]
    ),
    "cTkLODSettingsData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODSettingsData *
        ]
    ),
    "cTkAnimDetailSettings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimDetailSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimDetailSettingsData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimDetailSettingsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimDetailSettingsTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimDetailSettingsTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicResScalingSettings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicResScalingSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEngineSettingTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettingTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEngineSettingsMapping::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettingsMapping *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGameSettings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGameSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGraphicsDetailPreset::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsDetailPreset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGraphicsDetailTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsDetailTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGraphicsSettings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLODDistances::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODDistances *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLODSettingsData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODSettingsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimDetailSettings::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimDetailSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimDetailSettings *
        ]
    ),
    "cTkAnimDetailSettingsData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimDetailSettingsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimDetailSettingsData *
        ]
    ),
    "cTkDynamicResScalingSettings::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicResScalingSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkDynamicResScalingSettings *
        ]
    ),
    "cTkGraphicsDetailPreset::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsDetailPreset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkGraphicsDetailPreset *
        ]
    ),
    "cTkGraphicsDetailTypes::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsDetailTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkGraphicsDetailTypes *
        ]
    ),
    "cTkGraphicsSettings::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkGraphicsSettings *
        ]
    ),
    "cTkLODSettingsData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODSettingsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkLODSettingsData *
        ]
    ),
    "cTkMetaDataXML::ReadArrayXML<TkID<256>>": {
        "cTkDynamicArray<TkID<256> > *, XMLNode *, const char *, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDynamicArray<TkID<256> > *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "TkID<256> *, int, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # TkID<256> *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "cTkMetaDataXML::ReadClassArrayXML<cTkLODModelResource>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLODModelResource> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkProceduralInstanceData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralInstanceData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkResourceDescriptorData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkResourceDescriptorData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkResourceDescriptorList>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkResourceDescriptorList> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkResourceFilterData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkResourceFilterData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkSketchNodeData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkSketchNodeData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkShearWindOctaveData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkShearWindOctaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkLODModelResource const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkLODModelResource *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkProceduralInstanceData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkProceduralInstanceData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkResourceDescriptorData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkResourceDescriptorData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkResourceDescriptorList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkResourceDescriptorList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkResourceFilterData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkResourceFilterData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkSketchNodeData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkSketchNodeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkShearWindOctaveData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkShearWindOctaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkShearWindOctaveData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkGravityComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkGravityComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkLODComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLODComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkModelDescriptorList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkModelDescriptorList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkProceduralInstance const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkProceduralInstance *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkProceduralInstanceData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkProceduralInstanceData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkProceduralModelComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkProceduralModelComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkProceduralModelList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkProceduralModelList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkReferenceComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkReferenceComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkResourceDescriptorData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkResourceDescriptorData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkResourceDescriptorList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkResourceDescriptorList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkResourceFilterData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkResourceFilterData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkResourceFilterList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkResourceFilterList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkRotationComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkRotationComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkShearWindData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkShearWindData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkShearWindOctaveData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkShearWindOctaveData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkSketchComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkSketchComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cTkLODModelResource>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLODModelResource> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkResourceDescriptorData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkResourceDescriptorData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkResourceDescriptorList>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkResourceDescriptorList> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcGeneratedBaseThemeTemplate>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGeneratedBaseThemeTemplate> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkSketchNodeData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkSketchNodeData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkGravityComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkGravityComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLODComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkLODComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkModelDescriptorList::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkModelDescriptorList *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cTkProceduralModelComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkProceduralModelComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkReferenceComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkReferenceComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkRotationComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRotationComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkSketchComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkGravityComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLODComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkModelDescriptorList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkProceduralInstance::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkProceduralInstanceData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkProceduralModelComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkProceduralModelList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkReferenceComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkResourceDescriptorData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkResourceDescriptorList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkResourceFilterData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkResourceFilterList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkRotationComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkShearWindData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkShearWindOctaveData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkSketchComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkLODComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkModelDescriptorList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkProceduralModelComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkProceduralModelList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkResourceDescriptorData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkResourceDescriptorList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkResourceFilterData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkResourceFilterList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkSketchComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cTkLODModelResource>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkLODModelResource> *
        ]
    ),
    "cTkResourceDescriptorData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptorData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkResourceDescriptorList::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptorList *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkModelDescriptorList::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelDescriptorList *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkResourceDescriptorData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptorData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkResourceDescriptorList::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptorList *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkResourceFilterList::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatGroupTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLODComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralInstanceData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralInstanceData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralModelList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralModelList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkResourceDescriptorData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptorData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkResourceDescriptorList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptorList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkResourceFilterData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceFilterData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRotationComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRotationComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkShearWindData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkShearWindData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkShearWindOctaveData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkShearWindOctaveData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSketchComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkResourceDescriptorData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRotationComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRotationComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkShearWindData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkShearWindData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkShearWindOctaveData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkShearWindOctaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralInstance::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralInstance *
        ]
    ),
    "cTkResourceDescriptorData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptorData *
        ]
    ),
    "cTkShearWindData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkShearWindData *
        ]
    ),
    "cTkGravityComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGravityComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLODComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkModelDescriptorList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelDescriptorList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralInstance::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralInstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralInstanceData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralInstanceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralModelComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralModelComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralModelList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralModelList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkReferenceComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkReferenceComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkResourceDescriptorData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkResourceDescriptorList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptorList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkResourceFilterData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceFilterData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkResourceFilterList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceFilterList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRotationComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRotationComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkShearWindData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkShearWindData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkShearWindOctaveData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkShearWindOctaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSketchComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProceduralInstanceData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralInstanceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkProceduralInstanceData *
        ]
    ),
    "cTkResourceDescriptorData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkResourceDescriptorData *
        ]
    ),
    "cTkResourceDescriptorList::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptorList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkResourceDescriptorList *
        ]
    ),
    "cTkResourceFilterData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceFilterData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkResourceFilterData *
        ]
    ),
    "cTkRotationComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRotationComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkRotationComponentData *
        ]
    ),
    "cTkShearWindData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkShearWindData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkShearWindData *
        ]
    ),
    "cTkShearWindOctaveData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkShearWindOctaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkShearWindOctaveData *
        ]
    ),
    "cTkSketchComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkSketchComponentData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkAudioAnimTrigger>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAudioAnimTrigger> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkCreatureTailJoints>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkCreatureTailJoints> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkCreatureTailParams>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkCreatureTailParams> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkPhysicsData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkVolumeTriggerType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVolumeTriggerType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkAudioAnimTrigger const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkAudioAnimTrigger *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkCreatureTailJoints const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkCreatureTailJoints *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkCreatureTailParams const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkCreatureTailParams *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkCreatureTailParams>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkCreatureTailParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkCreatureTailParams *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkPhysicsData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPhysicsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkPhysicsData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkVolumeTriggerType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVolumeTriggerType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkVolumeTriggerType *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAudioAnimTrigger const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAudioAnimTrigger *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAudioComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAudioComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkCreatureTailComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkCreatureTailComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkCreatureTailJoints const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkCreatureTailJoints *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkCreatureTailParams const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkCreatureTailParams *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkDynamicChainComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkDynamicChainComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkInstanceWindComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkInstanceWindComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkNetReplicatedEntityComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNetReplicatedEntityComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkPhysicsComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkPhysicsComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkStaticPhysicsComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkStaticPhysicsComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkVolumeTriggerType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVolumeTriggerType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cTkAudioAnimTrigger>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAudioAnimTrigger> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkCreatureTailJoints>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkCreatureTailJoints> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkCreatureTailParams>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkCreatureTailParams> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAudioComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAudioComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkCreatureTailComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkCreatureTailComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkDynamicChainComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkDynamicChainComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkInstanceWindComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkInstanceWindComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNetReplicatedEntityComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkPhysicsComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysicsComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkStaticPhysicsComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkStaticPhysicsComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAudioAnimTrigger::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAudioComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkCreatureTailComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkCreatureTailJoints::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkCreatureTailParams::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkDynamicChainComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkInstanceWindComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkNetReplicatedEntityComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkPhysicsComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkStaticPhysicsComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkVolumeTriggerType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAudioComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkCreatureTailComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkCreatureTailParams::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicChainComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNetReplicatedEntityComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkPhysicsComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkStaticPhysicsComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAudioAnimTrigger::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioAnimTrigger *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAudioComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCreatureTailJoints::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailJoints *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCreatureTailParams::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailParams *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicChainComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicChainComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInstanceWindComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInstanceWindComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNetReplicatedEntityComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPhysicsComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkStaticPhysicsComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkStaticPhysicsComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVolumeTriggerType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkVolumeTriggerType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCreatureTailJoints::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailJoints *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCreatureTailParams::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPhysicsComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVolumeTriggerType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVolumeTriggerType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAudioAnimTrigger::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioAnimTrigger *
        ]
    ),
    "cTkAudioComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioComponentData *
        ]
    ),
    "cTkCreatureTailComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailComponentData *
        ]
    ),
    "cTkCreatureTailJoints::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailJoints *
        ]
    ),
    "cTkAudioAnimTrigger::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioAnimTrigger *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAudioComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCreatureTailComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCreatureTailJoints::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailJoints *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCreatureTailParams::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicChainComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicChainComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInstanceWindComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInstanceWindComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNetReplicatedEntityComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPhysicsComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkStaticPhysicsComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStaticPhysicsComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkVolumeTriggerType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVolumeTriggerType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAudioAnimTrigger::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioAnimTrigger *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAudioAnimTrigger *
        ]
    ),
    "cTkAudioComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAudioComponentData *
        ]
    ),
    "cTkCreatureTailComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkCreatureTailComponentData *
        ]
    ),
    "cTkCreatureTailJoints::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailJoints *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkCreatureTailJoints *
        ]
    ),
    "cTkCreatureTailParams::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkCreatureTailParams *
        ]
    ),
    "cTkDynamicChainComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicChainComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkDynamicChainComponentData *
        ]
    ),
    "cTkInstanceWindComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInstanceWindComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkInstanceWindComponentData *
        ]
    ),
    "cTkNetReplicatedEntityComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNetReplicatedEntityComponentData *
        ]
    ),
    "cTkPhysicsComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkPhysicsComponentData *
        ]
    ),
    "cTkStaticPhysicsComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStaticPhysicsComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkStaticPhysicsComponentData *
        ]
    ),
    "cTkVolumeTriggerType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVolumeTriggerType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkVolumeTriggerType *
        ]
    ),
    "cTkMetaDataXML::ReadArrayXML<cTkFixedString<64,char>>": {
        "cTkDynamicArray<cTkFixedString<64,char> > *, XMLNode *, const char *, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDynamicArray<cTkFixedString<64,char> > *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cTkFixedString<64,char> *, int, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkFixedString<64,char> *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "cTkMetaDataXML::ReadClassArrayXML<cTkAnimBlendTree>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimBlendTree> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkAnimJointLODData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimJointLODData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkAnimPoseBabyModifier>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimPoseBabyModifier> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkAnimPoseCorrelationData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimPoseCorrelationData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkAnimPoseData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimPoseData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkAnimPoseExampleData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimPoseExampleData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkAnimPoseExampleElement>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimPoseExampleElement> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkAnimationData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimationData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkAnimationGameData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationGameData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<cTkFixedString<64,char>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkAnimBlendTree const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkAnimBlendTree *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkAnimJointLODData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkAnimJointLODData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkAnimPoseBabyModifier const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkAnimPoseBabyModifier *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkAnimPoseCorrelationData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkAnimPoseCorrelationData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkAnimPoseData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkAnimPoseData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkAnimPoseExampleData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkAnimPoseExampleData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkAnimPoseExampleElement const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkAnimPoseExampleElement *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkAnimationData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkAnimationData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkAnimPoseBabyModifier>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimPoseBabyModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimPoseBabyModifier *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkAnimPoseCorrelationData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimPoseCorrelationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimPoseCorrelationData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkAnimPoseExampleElement>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimPoseExampleElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimPoseExampleElement *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkAnimationData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimationData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkAnimationGameData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimationGameData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimationGameData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkInOutCurve>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkInOutCurve *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkInOutCurve *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimBlendTree const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimBlendTree *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimJointLODData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimJointLODData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimPoseBabyModifier const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimPoseBabyModifier *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimPoseComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimPoseComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimPoseCorrelationData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimPoseCorrelationData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimPoseData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimPoseData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimPoseExampleData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimPoseExampleData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimPoseExampleElement const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimPoseExampleElement *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimationComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimationComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimationData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimationData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimationDataTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimationDataTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimationGameData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimationGameData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAttachmentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAttachmentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkCurveType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkCurveType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkHitCurveData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkHitCurveData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkInOutCurve const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkInOutCurve *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cTkAnimBlendTree>::~cTkDynamicArray<cTkAnimBlendTree>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimBlendTree> *
        ]
    ),
    "cTkDynamicArray<cTkFixedString<64,char>>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkFixedString<64,char> > *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkAnimBlendTree>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimBlendTree> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkAnimJointLODData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInputBindingSet> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkAnimPoseData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimPoseData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBaseBuildingEntryGroup>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingEntryGroup> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkAnimPoseExampleElement>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDifficultyFuelUseTechOverride> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkAnimationData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimationData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAnimPoseComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimPoseComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimationComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimationComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimBlendTree::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimJointLODData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimPoseBabyModifier::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimPoseComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimPoseCorrelationData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimPoseData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimPoseExampleData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimPoseExampleElement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimationComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimationData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimationDataTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimationGameData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAttachmentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkCurveType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkHitCurveData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkInOutCurve::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimBlendTree::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimJointLODData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimPoseComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimPoseExampleData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimationComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimationData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimationDataTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAttachmentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cTkAnimBlendTree>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimBlendTree> *
        ]
    ),
    "cTkDynamicArray<cGcFiendCrimeSpawnTable>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInputBindingSet> *
        ]
    ),
    "cTkAttachmentData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePlacementRule *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBasePlacementRule::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePlacementRule *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimBlendTree::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimBlendTree *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimJointLODData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimJointLODData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimPoseBabyModifier::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseBabyModifier *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimPoseComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimPoseCorrelationData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseCorrelationData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimPoseData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimPoseExampleData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseExampleData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimPoseExampleElement::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseExampleElement *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimationComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimationData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimationGameData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationGameData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAttachmentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachmentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCurveType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkCurveType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInOutCurve::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInOutCurve *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimationData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimationGameData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationGameData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAttachmentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachmentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCurveType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCurveType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkHitCurveData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHitCurveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAttachmentData::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachmentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimPoseComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponentData *
        ]
    ),
    "cGcCameraAmbientSpaceData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTelemetryStat *
        ]
    ),
    "cGcCharacterCustomisationBoneScaleData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionBaseBuildingState *
        ]
    ),
    "cTkAnimationData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationData *
        ]
    ),
    "cTkAttachmentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachmentData *
        ]
    ),
    "cGcPlayerControlInput::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlInput *
        ]
    ),
    "cTkHitCurveData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHitCurveData *
        ]
    ),
    "cTkAnimBlendTree::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimBlendTree *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimJointLODData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimJointLODData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimPoseBabyModifier::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseBabyModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimPoseComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimPoseCorrelationData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseCorrelationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimPoseData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimPoseExampleData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseExampleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimPoseExampleElement::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseExampleElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimationComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimationData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimationDataTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationDataTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimationGameData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationGameData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAttachmentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachmentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCurveType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCurveType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkHitCurveData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHitCurveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInOutCurve::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInOutCurve *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimBlendTree::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimBlendTree *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimBlendTree *
        ]
    ),
    "cTkAnimJointLODData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimJointLODData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimJointLODData *
        ]
    ),
    "cTkAnimPoseBabyModifier::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseBabyModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimPoseBabyModifier *
        ]
    ),
    "cTkAnimPoseCorrelationData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseCorrelationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimPoseCorrelationData *
        ]
    ),
    "cTkAnimPoseData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimPoseData *
        ]
    ),
    "cTkAnimPoseExampleElement::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseExampleElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimPoseExampleElement *
        ]
    ),
    "cTkAnimationData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimationData *
        ]
    ),
    "cTkAnimationGameData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationGameData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimationGameData *
        ]
    ),
    "cTkAttachmentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachmentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCurveType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCurveType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkCurveType *
        ]
    ),
    "cTkHitCurveData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHitCurveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkHitCurveData *
        ]
    ),
    "cTkInOutCurve::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInOutCurve *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkInOutCurve *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkAnim2dBlendNodeData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnim2dBlendNodeData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkAnimVectorBlendNodeData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimVectorBlendNodeData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<unsigned __int64>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkAnim2dBlendNodeData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkAnim2dBlendNodeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkAnimVectorBlendNodeData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkAnimVectorBlendNodeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcWonderType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWonderType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWonderType *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWikiTopicType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWikiTopicType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWonderCategoryConfig const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWonderCategoryConfig *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWonderCreatureCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWonderCreatureCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWonderCustomCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWonderCustomCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWonderFloraCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWonderFloraCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWonderMineralCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWonderMineralCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWonderPlanetCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWonderPlanetCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWonderRecord const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWonderRecord *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWonderRecordCustomData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWonderRecordCustomData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWonderTreasureCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWonderTreasureCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWonderType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWonderType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWonderWeirdBasePartCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWonderWeirdBasePartCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnim2dBlendNode const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnim2dBlendNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnim2dBlendNodeData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnim2dBlendNodeData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimAnimNode const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimAnimNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimBlendNode const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimBlendNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimVectorBlendNode const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimVectorBlendNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkAnimVectorBlendNodeData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkAnimVectorBlendNodeData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cTkAnim2dBlendNodeData>::~cTkDynamicArray<cTkAnim2dBlendNodeData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnim2dBlendNodeData> *
        ]
    ),
    "cTkDynamicArray<cTkAnim2dBlendNodeData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnim2dBlendNodeData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkAnimVectorBlendNodeData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimVectorBlendNodeData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcWikiTopicType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWonderCategoryConfig::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWonderCreatureCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWonderCustomCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWonderFloraCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWonderMineralCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWonderPlanetCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWonderRecord::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWonderRecordCustomData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWonderTreasureCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWonderType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWonderWeirdBasePartCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnim2dBlendNode::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnim2dBlendNodeData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimAnimNode::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimBlendNode::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimVectorBlendNode::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimVectorBlendNodeData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnim2dBlendNode::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnim2dBlendNodeData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimAnimNode::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimBlendNode::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimVectorBlendNode::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimVectorBlendNodeData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cTkAnim2dBlendNodeData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnim2dBlendNodeData> *
        ]
    ),
    "cTkDynamicArray<cTkAnimVectorBlendNodeData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkAnimVectorBlendNodeData> *
        ]
    ),
    "cGcWikiTopicType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiTopicType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderCategoryConfig::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderCategoryConfig *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderRecord::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderRecord *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderRecordCustomData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderRecordCustomData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnim2dBlendNode::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnim2dBlendNode *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnim2dBlendNodeData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnim2dBlendNodeData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimAnimNode::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimAnimNode *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimBlendNode::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimBlendNode *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimVectorBlendNodeData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimVectorBlendNodeData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderRecord::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderRecord *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcWonderRecordCustomData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderRecordCustomData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcWikiTopicType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiTopicType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderCategoryConfig::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderCategoryConfig *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderRecord::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderRecord *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderRecordCustomData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderRecordCustomData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnim2dBlendNodeData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnim2dBlendNodeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimVectorBlendNodeData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimVectorBlendNodeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderCategoryConfig::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderCategoryConfig *
        ]
    ),
    "cGcWonderRecord::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderRecord *
        ]
    ),
    "cGcWonderRecordCustomData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderRecordCustomData *
        ]
    ),
    "cTkAnim2dBlendNode::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnim2dBlendNode *
        ]
    ),
    "cTkAnimAnimNode::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimAnimNode *
        ]
    ),
    "cTkAnimVectorBlendNodeData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimVectorBlendNodeData *
        ]
    ),
    "cGcWonderRecord::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderRecord *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcWonderRecordCustomData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderRecordCustomData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcWikiTopicType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiTopicType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderCategoryConfig::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderCategoryConfig *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderCreatureCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderCreatureCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderCustomCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderCustomCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderFloraCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderFloraCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderMineralCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderMineralCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderPlanetCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderPlanetCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderRecord::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderRecord *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderRecordCustomData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderRecordCustomData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderTreasureCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderTreasureCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWonderWeirdBasePartCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderWeirdBasePartCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnim2dBlendNode::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnim2dBlendNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnim2dBlendNodeData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnim2dBlendNodeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimAnimNode::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimAnimNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimBlendNode::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimBlendNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimVectorBlendNode::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimVectorBlendNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimVectorBlendNodeData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimVectorBlendNodeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWikiTopicType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiTopicType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWikiTopicType *
        ]
    ),
    "cGcWonderCategoryConfig::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderCategoryConfig *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWonderCategoryConfig *
        ]
    ),
    "cGcWonderType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWonderType *
        ]
    ),
    "cTkAnim2dBlendNode::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnim2dBlendNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnim2dBlendNode *
        ]
    ),
    "cTkAnim2dBlendNodeData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnim2dBlendNodeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnim2dBlendNodeData *
        ]
    ),
    "cTkAnimAnimNode::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimAnimNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimAnimNode *
        ]
    ),
    "cTkAnimBlendNode::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimBlendNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimBlendNode *
        ]
    ),
    "cTkAnimVectorBlendNodeData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimVectorBlendNodeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimVectorBlendNodeData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcIDLookupPath>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcIDLookupPath> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcStoryCategory>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryCategory *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcStoryEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStoryEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcStoryEntryBranch>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStoryEntryBranch> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcStoryPage>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStoryPage> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcStoryPageSeenData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStoryPageSeenData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcVibrationChannelData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcVibrationChannelData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcWikiCategory>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWikiCategory> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcWikiPage>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWikiPage> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcWikiTopic>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWikiTopic> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcWikiTopicType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiTopicType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkTextureResource>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelResource *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadXML<cTkFixedString<512,char>>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadXML<cTkFixedString<2048,char>>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<2048,char> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadXML<cTkSeed>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcIDLookupPath const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcIDLookupPath *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcStoryCategory const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcStoryCategory *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcStoryEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcStoryEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcStoryEntryBranch const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcStoryEntryBranch *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcStoryPage const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcStoryPage *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcStoryPageSeenData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcStoryPageSeenData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcVibrationChannelData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcVibrationChannelData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcVibrationData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcVibrationData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcWikiCategory const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcWikiCategory *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcWikiPage const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcWikiPage *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcWikiTopic const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcWikiTopic *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcStoryEntryBranch>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStoryEntryBranch *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStoryEntryBranch *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcWikiTopicType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWikiTopicType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWikiTopicType *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkTextureResource>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkModelResource *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkModelResource *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcIDLookupPath const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcIDLookupPath *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcIDLookupPaths const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcIDLookupPaths *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcResourceElement const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcResourceElement *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStoriesTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStoriesTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStoryCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStoryCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStoryEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStoryEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStoryEntryBranch const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStoryEntryBranch *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStoryPage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStoryPage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStoryPageSeenData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStoryPageSeenData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStoryPageSeenDataArray const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStoryPageSeenDataArray *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTestMetadata const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTestMetadata *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcVibrationChannelData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcVibrationChannelData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcVibrationData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcVibrationData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcVibrationDataTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcVibrationDataTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWiki const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWiki *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWikiCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWikiCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWikiPage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWikiPage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWikiTopic const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWikiTopic *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcStoryCategory,8>::cTkFixedArray<cGcStoryCategory,8>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcStoryCategory,8> *
        ]
    ),
    "cGcVibrationData::cGcVibrationData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVibrationData *
        ]
    ),
    "cTkFixedArray<cGcStoryCategory,8>::~cTkFixedArray<cGcStoryCategory,8>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcStoryCategory,8> *
        ]
    ),
    "cGcStoryCategory::~cGcStoryCategory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryCategory *
        ]
    ),
    "cGcWikiCategory::~cGcWikiCategory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiCategory *
        ]
    ),
    "cGcWikiPage::~cGcWikiPage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiPage *
        ]
    ),
    "cGcWikiTopic::~cGcWikiTopic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiTopic *
        ]
    ),
    "cTkDynamicArray<cGcIDLookupPath>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcIDLookupPath> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcStoryEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStoryEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcStoryPage>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStoryPage> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcVibrationChannelData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcVibrationChannelData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcWikiCategory>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWikiCategory> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcWikiPage>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWikiPage> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcWikiTopic>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWikiTopic> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcIDLookupPath::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcIDLookupPaths::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcResourceElement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStoriesTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStoryCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStoryEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStoryEntryBranch::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStoryPage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStoryPageSeenData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStoryPageSeenDataArray::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTestMetadata::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVibrationChannelData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVibrationData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVibrationDataTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWiki::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWikiCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWikiPage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWikiTopic::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcIDLookupPaths::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcResourceElement::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStoriesTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStoryCategory::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStoryEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStoryPage::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStoryPageSeenDataArray::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTestMetadata::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcVibrationChannelData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcVibrationData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcVibrationDataTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWiki::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWikiCategory::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWikiPage::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWikiTopic::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcStoryEntry>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStoryEntry> *
        ]
    ),
    "cGcResourceElement::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceElement *
        ]
    ),
    "cGcResourceElement::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceElement *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStoryCategory::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryCategory *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStoryPage::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryPage *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWikiCategory::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiCategory *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWikiTopic::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiTopic *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcResourceElement::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceElement *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStoriesTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoriesTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStoryCategory::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryCategory *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStoryPage::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryPage *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcVibrationDataTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVibrationDataTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWiki::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWiki *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWikiCategory::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiCategory *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWikiTopic::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiTopic *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcIDLookupPath::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcIDLookupPath *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcResourceElement::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceElement *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryCategory::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryCategory *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryEntryBranch::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryEntryBranch *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryPage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryPage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryPageSeenData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryPageSeenData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryPageSeenDataArray::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryPageSeenDataArray *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTestMetadata::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTestMetadata *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVibrationChannelData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVibrationChannelData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVibrationData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVibrationData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWikiCategory::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiCategory *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWikiPage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiPage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWikiTopic::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiTopic *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcResourceElement::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceElement *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcStoryPageSeenDataArray::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryPageSeenDataArray *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcResourceElement::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryCategory::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryPage::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryPage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryPageSeenDataArray::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryPageSeenDataArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTestMetadata::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTestMetadata *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVibrationChannelData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVibrationChannelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVibrationData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVibrationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWikiCategory::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWikiPage::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiPage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWikiTopic::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiTopic *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcIDLookupPath::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIDLookupPath *
        ]
    ),
    "cGcResourceElement::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceElement *
        ]
    ),
    "cGcStoryEntryBranch::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryEntryBranch *
        ]
    ),
    "cGcTestMetadata::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTestMetadata *
        ]
    ),
    "cGcWikiPage::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiPage *
        ]
    ),
    "cGcWikiTopic::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiTopic *
        ]
    ),
    "cGcResourceElement::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceElement *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcStoryPageSeenDataArray::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryPageSeenDataArray *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcIDLookupPath::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIDLookupPath *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcIDLookupPaths::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIDLookupPaths *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcResourceElement::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoriesTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoriesTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryEntryBranch::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryEntryBranch *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryPage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryPage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryPageSeenData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryPageSeenData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryPageSeenDataArray::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryPageSeenDataArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTestMetadata::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTestMetadata *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVibrationChannelData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVibrationChannelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVibrationData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVibrationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVibrationDataTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVibrationDataTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWiki::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWiki *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWikiCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWikiPage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiPage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWikiTopic::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiTopic *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcIDLookupPath::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIDLookupPath *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcIDLookupPath *
        ]
    ),
    "cGcResourceElement::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcResourceElement *
        ]
    ),
    "cGcStoryCategory::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStoryCategory *
        ]
    ),
    "cGcStoryEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStoryEntry *
        ]
    ),
    "cGcStoryEntryBranch::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryEntryBranch *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStoryEntryBranch *
        ]
    ),
    "cGcStoryPage::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryPage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStoryPage *
        ]
    ),
    "cGcStoryPageSeenData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryPageSeenData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStoryPageSeenData *
        ]
    ),
    "cGcTestMetadata::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTestMetadata *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTestMetadata *
        ]
    ),
    "cGcVibrationChannelData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVibrationChannelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcVibrationChannelData *
        ]
    ),
    "cGcVibrationData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVibrationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcVibrationData *
        ]
    ),
    "cGcWikiCategory::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWikiCategory *
        ]
    ),
    "cGcWikiPage::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiPage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWikiPage *
        ]
    ),
    "cGcWikiTopic::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWikiTopic *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWikiTopic *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcTriggerFeedbackState>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTriggerFeedbackState> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcInWorldUIScreenData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIScreenData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcMovementDirection>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMovementDirection *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcTriggerFeedbackState const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcTriggerFeedbackState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcInWorldUIScreenData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInWorldUIScreenData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInWorldUIScreenData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcMovementDirection>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMovementDirection *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMovementDirection *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcLaunchFuelCostDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcLaunchFuelCostDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcOptionsUIHeaderIcons const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcOptionsUIHeaderIcons *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPersistentBaseDifficultyData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPersistentBaseDifficultyData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPhotoModeAdjustData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPhotoModeAdjustData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPhotoModeSettings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPhotoModeSettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcReputationGainDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcReputationGainDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScannerRechargeDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScannerRechargeDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSprintingCostDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSprintingCostDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSubstanceCollectionDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSubstanceCollectionDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSurvivalBarBoolArray const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSurvivalBarBoolArray *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTriggerFeedbackState const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTriggerFeedbackState *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTriggerFeedbackStateTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTriggerFeedbackStateTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUserSettingsData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUserSettingsData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcTriggerFeedbackState>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTriggerFeedbackState> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcLaunchFuelCostDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcOptionsUIHeaderIcons::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPersistentBaseDifficultyData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPhotoModeAdjustData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPhotoModeSettings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcReputationGainDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScannerRechargeDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSprintingCostDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSubstanceCollectionDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSurvivalBarBoolArray::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTriggerFeedbackState::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTriggerFeedbackStateTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUserSettingsData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPersistentBaseDifficultyData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPhotoModeAdjustData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTriggerFeedbackState::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTriggerFeedbackStateTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUserSettingsData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPhotoModeAdjustData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipWarpData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTriggerFeedbackStateTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerFeedbackStateTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUserSettingsData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettingsData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcLaunchFuelCostDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaunchFuelCostDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistentBaseDifficultyData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseDifficultyData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoModeAdjustData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeAdjustData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoModeSettings::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeSettings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcReputationGainDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcReputationGainDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannerRechargeDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerRechargeDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSprintingCostDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSprintingCostDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSubstanceCollectionDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceCollectionDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSurvivalBarBoolArray::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSurvivalBarBoolArray *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTriggerFeedbackState::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerFeedbackState *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUserSettingsData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettingsData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLaunchFuelCostDifficultyOption::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaunchFuelCostDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPersistentBaseDifficultyData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseDifficultyData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPhotoModeSettings::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeSettings *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcReputationGainDifficultyOption::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcReputationGainDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcScannerRechargeDifficultyOption::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerRechargeDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSprintingCostDifficultyOption::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSprintingCostDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSubstanceCollectionDifficultyOption::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceCollectionDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcUserSettingsData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettingsData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcLaunchFuelCostDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaunchFuelCostDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistentBaseDifficultyData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseDifficultyData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoModeAdjustData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeAdjustData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoModeSettings::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcReputationGainDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcReputationGainDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannerRechargeDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerRechargeDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSprintingCostDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSprintingCostDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSubstanceCollectionDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceCollectionDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTriggerFeedbackState::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerFeedbackState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUserSettingsData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettingsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetSize::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionDuration *
        ]
    ),
    "cGcPersistentBaseDifficultyData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseDifficultyData *
        ]
    ),
    "cGcPhotoModeAdjustData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeAdjustData *
        ]
    ),
    "cGcPhotoModeSettings::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeSettings *
        ]
    ),
    "cGcUserSettingsData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettingsData *
        ]
    ),
    "cGcLaunchFuelCostDifficultyOption::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaunchFuelCostDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcPersistentBaseDifficultyData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseDifficultyData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcPhotoModeSettings::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeSettings *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcReputationGainDifficultyOption::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcReputationGainDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcScannerRechargeDifficultyOption::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerRechargeDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcSprintingCostDifficultyOption::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSprintingCostDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcSubstanceCollectionDifficultyOption::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceCollectionDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcUserSettingsData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettingsData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcLaunchFuelCostDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaunchFuelCostDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOptionsUIHeaderIcons::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOptionsUIHeaderIcons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistentBaseDifficultyData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseDifficultyData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoModeAdjustData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeAdjustData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoModeSettings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcReputationGainDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcReputationGainDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannerRechargeDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerRechargeDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSprintingCostDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSprintingCostDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSubstanceCollectionDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceCollectionDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSurvivalBarBoolArray::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSurvivalBarBoolArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTriggerFeedbackState::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerFeedbackState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTriggerFeedbackStateTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerFeedbackStateTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUserSettingsData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettingsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLaunchFuelCostDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaunchFuelCostDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcLaunchFuelCostDifficultyOption *
        ]
    ),
    "cGcPersistentBaseDifficultyData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseDifficultyData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPersistentBaseDifficultyData *
        ]
    ),
    "cGcPhotoModeAdjustData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeAdjustData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPhotoModeAdjustData *
        ]
    ),
    "cGcPhotoModeSettings::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPhotoModeSettings *
        ]
    ),
    "cGcReputationGainDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcReputationGainDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcReputationGainDifficultyOption *
        ]
    ),
    "cGcScannerRechargeDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerRechargeDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScannerRechargeDifficultyOption *
        ]
    ),
    "cGcSprintingCostDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSprintingCostDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSprintingCostDifficultyOption *
        ]
    ),
    "cGcSubstanceCollectionDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceCollectionDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSubstanceCollectionDifficultyOption *
        ]
    ),
    "cGcTriggerFeedbackState::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerFeedbackState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTriggerFeedbackState *
        ]
    ),
    "cGcUserSettingsData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUserSettingsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcUserSettingsData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDifficultySettingEnum>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDifficultySettingEnum> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCombatTimerDifficultyOption>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatTimerDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcDifficultyPresetType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyPresetType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcInventoryContainer>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryContainer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDifficultySettingEnum const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDifficultySettingEnum *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCombatTimerDifficultyOption>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCombatTimerDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCombatTimerDifficultyOption *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcDifficultyPresetType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultyPresetType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultyPresetType *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcDifficultySettingEnum>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultySettingEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultySettingEnum *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcEnergyDrainDifficultyOption>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcEnergyDrainDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcEnergyDrainDifficultyOption *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcFuelUseDifficultyOption>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFuelUseDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFuelUseDifficultyOption *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcHazardDrainDifficultyOption>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHazardDrainDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcHazardDrainDifficultyOption *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcInventoryContainer>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryContainer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryContainer *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcInventoryStackLimitsDifficultyOption>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryStackLimitsDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryStackLimitsDifficultyOption *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcItemShopAvailabilityDifficultyOption>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcItemShopAvailabilityDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcItemShopAvailabilityDifficultyOption *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDifficultySettingEditability const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultySettingEditability *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDifficultySettingEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultySettingEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDifficultySettingLocData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultySettingLocData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDifficultySettingType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultySettingType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDifficultySettingUIOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultySettingUIOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDifficultySettingsData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultySettingsData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDifficultySettingsReplicatedState const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultySettingsReplicatedState *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDifficultyStartWithAllItemsKnownOptionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultyStartWithAllItemsKnownOptionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDifficultyStateData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultyStateData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcEnergyDrainDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcEnergyDrainDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFuelUseDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFuelUseDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGyroSettingsData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGyroSettingsData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHazardDrainDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHazardDrainDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryStackLimitsDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryStackLimitsDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcItemShopAvailabilityDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcItemShopAvailabilityDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcItemShopAvailabilityDifficultyOptionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcItemShopAvailabilityDifficultyOptionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcKnownThingsPreset::~cGcKnownThingsPreset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcKnownThingsPreset *
        ]
    ),
    "cTkDynamicArray<cGcDifficultySettingEnum>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDifficultySettingEnum> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcDifficultySettingEditability::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultySettingEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultySettingLocData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultySettingType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultySettingUIOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultySettingsData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultySettingsReplicatedState::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultyStartWithAllItemsKnownOptionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultyStateData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEnergyDrainDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFuelUseDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGyroSettingsData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHazardDrainDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryStackLimitsDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcItemShopAvailabilityDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcItemShopAvailabilityDifficultyOptionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultySettingUIOption::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDifficultySettingsData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDifficultySettingsReplicatedState::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDifficultyStartWithAllItemsKnownOptionData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDifficultyStateData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcItemShopAvailabilityDifficultyOptionData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDifficultySettingsData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsData *
        ]
    ),
    "cGcDifficultyStateData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyStateData *
        ]
    ),
    "cGcExperienceDebugTriggerInput::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistFriend *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDifficultySettingsData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDifficultyStartWithAllItemsKnownOptionData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyStartWithAllItemsKnownOptionData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDifficultyStateData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyStateData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDebugCamera::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistFriend *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDifficultyStartWithAllItemsKnownOptionData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyStartWithAllItemsKnownOptionData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDifficultySettingEditability::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingEditability *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingEnum::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingEnum *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingLocData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingLocData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingUIOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingUIOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingsData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingsReplicatedState::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsReplicatedState *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyStartWithAllItemsKnownOptionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyStartWithAllItemsKnownOptionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyStateData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyStateData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEnergyDrainDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnergyDrainDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFuelUseDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFuelUseDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGyroSettingsData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGyroSettingsData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHazardDrainDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHazardDrainDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryStackLimitsDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStackLimitsDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemShopAvailabilityDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemShopAvailabilityDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemShopAvailabilityDifficultyOptionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemShopAvailabilityDifficultyOptionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingsData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcDifficultyStateData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyStateData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcGyroSettingsData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGyroSettingsData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcDifficultySettingEditability::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingEditability *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingEnum::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingUIOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingUIOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingsData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyStartWithAllItemsKnownOptionData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyStartWithAllItemsKnownOptionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyStateData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEnergyDrainDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnergyDrainDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFuelUseDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFuelUseDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGyroSettingsData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGyroSettingsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHazardDrainDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHazardDrainDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryStackLimitsDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStackLimitsDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemShopAvailabilityDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemShopAvailabilityDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemShopAvailabilityDifficultyOptionData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemShopAvailabilityDifficultyOptionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingsData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsData *
        ]
    ),
    "cGcDifficultyStartWithAllItemsKnownOptionData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyStartWithAllItemsKnownOptionData *
        ]
    ),
    "cGcDifficultyStateData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyStateData *
        ]
    ),
    "cGcGyroSettingsData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGyroSettingsData *
        ]
    ),
    "cGcDifficultySettingsData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcDifficultyStateData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyStateData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcGyroSettingsData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGyroSettingsData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcDifficultySettingEditability::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingEditability *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingLocData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingLocData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingUIOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingUIOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingsData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingsReplicatedState::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsReplicatedState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyStartWithAllItemsKnownOptionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyStartWithAllItemsKnownOptionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyStateData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEnergyDrainDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnergyDrainDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFuelUseDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFuelUseDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGyroSettingsData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGyroSettingsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHazardDrainDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHazardDrainDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryStackLimitsDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStackLimitsDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemShopAvailabilityDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemShopAvailabilityDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemShopAvailabilityDifficultyOptionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemShopAvailabilityDifficultyOptionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingEditability::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingEditability *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultySettingEditability *
        ]
    ),
    "cGcDifficultySettingEnum::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultySettingEnum *
        ]
    ),
    "cGcDifficultySettingType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultySettingType *
        ]
    ),
    "cGcDifficultySettingsData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultySettingsData *
        ]
    ),
    "cGcDifficultyStartWithAllItemsKnownOptionData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyStartWithAllItemsKnownOptionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultyStartWithAllItemsKnownOptionData *
        ]
    ),
    "cGcDifficultyStateData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultyStateData *
        ]
    ),
    "cGcEnergyDrainDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnergyDrainDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcEnergyDrainDifficultyOption *
        ]
    ),
    "cGcFuelUseDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFuelUseDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFuelUseDifficultyOption *
        ]
    ),
    "cGcGyroSettingsData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGyroSettingsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGyroSettingsData *
        ]
    ),
    "cGcHazardDrainDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHazardDrainDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcHazardDrainDifficultyOption *
        ]
    ),
    "cGcInventoryStackLimitsDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStackLimitsDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryStackLimitsDifficultyOption *
        ]
    ),
    "cGcItemShopAvailabilityDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemShopAvailabilityDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcItemShopAvailabilityDifficultyOption *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDifficultyFuelUseOptionData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyFuelUseOptionData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDifficultyFuelUseTechOverride>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDifficultyFuelUseTechOverride> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDifficultyInventoryStackSizeOptionData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyInventoryStackSizeOptionData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDifficultyOptionUIGroup>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyOptionUIGroup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDifficultySettingUIOption>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDifficultySettingUIOption> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcDifficultyStartWithAllItemsKnownOptionData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyStartWithAllItemsKnownOptionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDifficultyCurrencyCostOptionData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDifficultyCurrencyCostOptionData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDifficultyFuelUseOptionData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDifficultyFuelUseOptionData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDifficultyFuelUseTechOverride const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDifficultyFuelUseTechOverride *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDifficultyInventoryStackSizeOptionData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDifficultyInventoryStackSizeOptionData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDifficultyOptionUIGroup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDifficultyOptionUIGroup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDifficultySettingCommonData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDifficultySettingCommonData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDifficultySettingUIOption const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDifficultySettingUIOption *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDifficultySettingsData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDifficultySettingsData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcItemShopAvailabilityDifficultyOptionData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcItemShopAvailabilityDifficultyOptionData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcDifficultyFuelUseOptionData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultyFuelUseOptionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultyFuelUseOptionData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcDifficultyFuelUseTechOverride>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultyFuelUseTechOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultyFuelUseTechOverride *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcDifficultySettingsData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultySettingsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultySettingsData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcDifficultyStartWithAllItemsKnownOptionData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultyStartWithAllItemsKnownOptionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultyStartWithAllItemsKnownOptionData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDeathConsequencesDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDeathConsequencesDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDifficultyConfig const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultyConfig *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDifficultyCurrencyCostOptionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultyCurrencyCostOptionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDifficultyFuelUseOptionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultyFuelUseOptionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDifficultyFuelUseTechOverride const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultyFuelUseTechOverride *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDifficultyInventoryStackSizeOptionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultyInventoryStackSizeOptionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDifficultyOptionGroups const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultyOptionGroups *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDifficultyOptionUIGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultyOptionUIGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDifficultyPresetType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultyPresetType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDifficultySettingCommonData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDifficultySettingCommonData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcDifficultySettingsData,7>::cTkFixedArray<cGcDifficultySettingsData,7>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcDifficultySettingsData,7> *
        ]
    ),
    "cTkDynamicArray<cGcDifficultySettingUIOption>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDifficultySettingUIOption> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcDeathConsequencesDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultyConfig::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultyCurrencyCostOptionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultyFuelUseOptionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultyFuelUseTechOverride::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultyInventoryStackSizeOptionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultyOptionGroups::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultyOptionUIGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultyPresetType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultySettingCommonData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDifficultyConfig::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDifficultyFuelUseOptionData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDifficultyOptionUIGroup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDifficultySettingCommonData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDifficultyConfig::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyConfig *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDeathConsequencesDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDeathConsequencesDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyConfig::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyConfig *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyCurrencyCostOptionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyCurrencyCostOptionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyFuelUseOptionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyFuelUseOptionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyFuelUseTechOverride::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyFuelUseTechOverride *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyInventoryStackSizeOptionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyInventoryStackSizeOptionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyOptionUIGroup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyOptionUIGroup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyPresetType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyPresetType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingCommonData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingCommonData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDeathConsequencesDifficultyOption::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDeathConsequencesDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcDifficultyPresetType::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyPresetType *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcDeathConsequencesDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDeathConsequencesDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyConfig::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyConfig *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyCurrencyCostOptionData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyCurrencyCostOptionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyPresetType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyPresetType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingCommonData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingCommonData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyConfig::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyConfig *
        ]
    ),
    "cGcDifficultyCurrencyCostOptionData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyCurrencyCostOptionData *
        ]
    ),
    "cGcDifficultyInventoryStackSizeOptionData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyInventoryStackSizeOptionData *
        ]
    ),
    "cGcDifficultySettingCommonData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingCommonData *
        ]
    ),
    "cGcDeathConsequencesDifficultyOption::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDeathConsequencesDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcDifficultyPresetType::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyPresetType *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcDeathConsequencesDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDeathConsequencesDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyConfig::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyConfig *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyCurrencyCostOptionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyCurrencyCostOptionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyFuelUseOptionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyFuelUseOptionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyFuelUseTechOverride::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyFuelUseTechOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyInventoryStackSizeOptionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyInventoryStackSizeOptionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyOptionGroups::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyOptionGroups *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyOptionUIGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyOptionUIGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultyPresetType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyPresetType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDifficultySettingCommonData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingCommonData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDeathConsequencesDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDeathConsequencesDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDeathConsequencesDifficultyOption *
        ]
    ),
    "cGcDifficultyConfig::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyConfig *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultyConfig *
        ]
    ),
    "cGcDifficultyCurrencyCostOptionData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyCurrencyCostOptionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultyCurrencyCostOptionData *
        ]
    ),
    "cGcDifficultyFuelUseOptionData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyFuelUseOptionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultyFuelUseOptionData *
        ]
    ),
    "cGcDifficultyFuelUseTechOverride::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyFuelUseTechOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultyFuelUseTechOverride *
        ]
    ),
    "cGcDifficultyInventoryStackSizeOptionData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyInventoryStackSizeOptionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultyInventoryStackSizeOptionData *
        ]
    ),
    "cGcDifficultyOptionUIGroup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyOptionUIGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultyOptionUIGroup *
        ]
    ),
    "cGcDifficultyPresetType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyPresetType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultyPresetType *
        ]
    ),
    "cGcDifficultySettingCommonData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultySettingCommonData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDifficultySettingCommonData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBlockedUser>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockedUser *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBlockedMessage const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBlockedMessage *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBlockedUser const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBlockedUser *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcQuickMenuActionSaveData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcQuickMenuActionSaveData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcQuickMenuActions>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcQuickMenuActions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcQuickMenuActions *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcRemoteWeapons>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRemoteWeapons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRemoteWeapons *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcActiveSurvivalBarsDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcActiveSurvivalBarsDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBlockListPersistence const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBlockListPersistence *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBlockedMessage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBlockedMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBlockedUser const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBlockedUser *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBreakTechOnDamageDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBreakTechOnDamageDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcChargingRequirementsDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcChargingRequirementsDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCombatTimerDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCombatTimerDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureHostilityDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureHostilityDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCurrencyCostDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCurrencyCostDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDamageGivenDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDamageGivenDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDamageReceivedDifficultyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDamageReceivedDifficultyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHotActionMenuTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHotActionMenuTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHotActionsSaveData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHotActionsSaveData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInWorldUIScreenData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInWorldUIScreenData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcQuickMenuActionSaveData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcQuickMenuActionSaveData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcQuickMenuActions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcQuickMenuActions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRemoteWeapons const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRemoteWeapons *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcShipWeaponData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShipWeaponData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcShipWeapons const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShipWeapons *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpringLink const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpringLink *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActiveSurvivalBarsDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBlockListPersistence::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBlockedMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBlockedUser::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBreakTechOnDamageDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcChargingRequirementsDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCombatTimerDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureHostilityDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCurrencyCostDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDamageGivenDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDamageReceivedDifficultyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHotActionMenuTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHotActionsSaveData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInWorldUIScreenData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcQuickMenuActionSaveData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcQuickMenuActions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRemoteWeapons::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShipWeaponData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShipWeapons::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpringLink::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHotActionsSaveData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcQuickMenuActionSaveData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcShipWeaponData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSpringLink::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcHotActionsSaveData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHotActionsSaveData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkInputManager::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "cGcSpringLink::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpringLink *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSpringLink::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpringLink *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcActiveSurvivalBarsDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcActiveSurvivalBarsDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBlockListPersistence::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockListPersistence *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBlockedMessage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockedMessage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBlockedUser::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockedUser *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBreakTechOnDamageDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBreakTechOnDamageDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcChargingRequirementsDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcChargingRequirementsDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatTimerDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatTimerDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHostilityDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHostilityDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCurrencyCostDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCurrencyCostDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDamageGivenDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageGivenDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDamageReceivedDifficultyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageReceivedDifficultyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHotActionsSaveData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHotActionsSaveData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInWorldUIScreenData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIScreenData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcQuickMenuActionSaveData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenuActionSaveData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcQuickMenuActions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenuActions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRemoteWeapons::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRemoteWeapons *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipWeaponData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipWeaponData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpringLink::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpringLink *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActiveSurvivalBarsDifficultyOption::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcActiveSurvivalBarsDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcBlockListPersistence::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockListPersistence *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcBlockedUser::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockedUser *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcBreakTechOnDamageDifficultyOption::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBreakTechOnDamageDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcChargingRequirementsDifficultyOption::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcChargingRequirementsDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcCombatTimerDifficultyOption::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatTimerDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcCreatureHostilityDifficultyOption::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHostilityDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcCurrencyCostDifficultyOption::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCurrencyCostDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcDamageGivenDifficultyOption::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageGivenDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcDamageReceivedDifficultyOption::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageReceivedDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcHotActionsSaveData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHotActionsSaveData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcInWorldUIScreenData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIScreenData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcQuickMenuActionSaveData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenuActionSaveData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcActiveSurvivalBarsDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActiveSurvivalBarsDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBlockListPersistence::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockListPersistence *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBreakTechOnDamageDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBreakTechOnDamageDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcChargingRequirementsDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcChargingRequirementsDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatTimerDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatTimerDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHostilityDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHostilityDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCurrencyCostDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCurrencyCostDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDamageGivenDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageGivenDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDamageReceivedDifficultyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageReceivedDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHotActionsSaveData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHotActionsSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInWorldUIScreenData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIScreenData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcQuickMenuActionSaveData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenuActionSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcQuickMenuActions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenuActions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRemoteWeapons::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRemoteWeapons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipWeaponData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipWeaponData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpringLink::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpringLink *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardChangeGlobalMission::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardChangeGlobalMission *
        ]
    ),
    "cGcBlockListPersistence::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockListPersistence *
        ]
    ),
    "cGcBlockedUser::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockedUser *
        ]
    ),
    "cGcHotActionsSaveData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHotActionsSaveData *
        ]
    ),
    "cGcInWorldUIScreenData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIScreenData *
        ]
    ),
    "cGcQuickMenuActionSaveData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenuActionSaveData *
        ]
    ),
    "cGcRemoteWeapons::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRemoteWeapons *
        ]
    ),
    "cGcShipWeaponData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipWeaponData *
        ]
    ),
    "cGcSpringLink::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpringLink *
        ]
    ),
    "cGcActiveSurvivalBarsDifficultyOption::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActiveSurvivalBarsDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcBlockListPersistence::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockListPersistence *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcBreakTechOnDamageDifficultyOption::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBreakTechOnDamageDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcChargingRequirementsDifficultyOption::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcChargingRequirementsDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcCombatTimerDifficultyOption::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatTimerDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcCreatureHostilityDifficultyOption::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHostilityDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcCurrencyCostDifficultyOption::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCurrencyCostDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcDamageGivenDifficultyOption::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageGivenDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcDamageReceivedDifficultyOption::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageReceivedDifficultyOption *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcHotActionsSaveData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHotActionsSaveData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcInWorldUIScreenData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIScreenData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcActiveSurvivalBarsDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActiveSurvivalBarsDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBlockListPersistence::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockListPersistence *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBlockedMessage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockedMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBlockedUser::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockedUser *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBreakTechOnDamageDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBreakTechOnDamageDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcChargingRequirementsDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcChargingRequirementsDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatTimerDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatTimerDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHostilityDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHostilityDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCurrencyCostDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCurrencyCostDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDamageGivenDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageGivenDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDamageReceivedDifficultyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageReceivedDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHotActionMenuTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHotActionMenuTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHotActionsSaveData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHotActionsSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInWorldUIScreenData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIScreenData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcQuickMenuActionSaveData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenuActionSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcQuickMenuActions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenuActions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRemoteWeapons::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRemoteWeapons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipWeaponData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipWeaponData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipWeapons::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipWeapons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpringLink::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpringLink *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActiveSurvivalBarsDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActiveSurvivalBarsDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcActiveSurvivalBarsDifficultyOption *
        ]
    ),
    "cGcBlockListPersistence::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockListPersistence *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBlockListPersistence *
        ]
    ),
    "cGcBlockedMessage::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockedMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBlockedMessage *
        ]
    ),
    "cGcBlockedUser::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlockedUser *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBlockedUser *
        ]
    ),
    "cGcBreakTechOnDamageDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBreakTechOnDamageDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBreakTechOnDamageDifficultyOption *
        ]
    ),
    "cGcChargingRequirementsDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcChargingRequirementsDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcChargingRequirementsDifficultyOption *
        ]
    ),
    "cGcCombatTimerDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatTimerDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCombatTimerDifficultyOption *
        ]
    ),
    "cGcCreatureHostilityDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHostilityDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureHostilityDifficultyOption *
        ]
    ),
    "cGcCurrencyCostDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCurrencyCostDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCurrencyCostDifficultyOption *
        ]
    ),
    "cGcDamageGivenDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageGivenDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDamageGivenDifficultyOption *
        ]
    ),
    "cGcDamageReceivedDifficultyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageReceivedDifficultyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDamageReceivedDifficultyOption *
        ]
    ),
    "cGcInWorldUIScreenData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInWorldUIScreenData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInWorldUIScreenData *
        ]
    ),
    "cGcQuickMenuActionSaveData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenuActionSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcQuickMenuActionSaveData *
        ]
    ),
    "cGcQuickMenuActions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuickMenuActions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcQuickMenuActions *
        ]
    ),
    "cGcRemoteWeapons::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRemoteWeapons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRemoteWeapons *
        ]
    ),
    "cGcShipWeaponData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipWeaponData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcShipWeaponData *
        ]
    ),
    "cGcSpringLink::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpringLink *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSpringLink *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerWeaponBiomeProperties>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerWeaponBiomeProperties> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcStatsBonus>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStatsBonus> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcVehicleScanTableEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcVehicleScanTableEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcVehicleType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerWeaponBiomeProperties const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerWeaponBiomeProperties *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerWeaponPropertiesData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerWeaponPropertiesData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcStatsBonus const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcStatsBonus *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcVehicleScanTableEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcVehicleScanTableEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlayerWeaponClass>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerWeaponClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerWeaponClass *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcVehicleType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcVehicleType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcVehicleType *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAIShipWeapons const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAIShipWeapons *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCamouflageData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCamouflageData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerWeaponBiomeProperties const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerWeaponBiomeProperties *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerWeaponClass const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerWeaponClass *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerWeaponData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerWeaponData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerWeaponPropertiesData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerWeaponPropertiesData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerWeaponPropertiesTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerWeaponPropertiesTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerWeapons const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerWeapons *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcVehicleCheckpointComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcVehicleCheckpointComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcVehicleComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcVehicleComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcVehicleGarageComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcVehicleGarageComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcVehicleMuzzleData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcVehicleMuzzleData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcVehicleRaceInviteComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcVehicleRaceInviteComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcVehicleScanTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcVehicleScanTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcVehicleScanTableEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcVehicleScanTableEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcVehicleType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcVehicleType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcVehicleWeaponMode const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcVehicleWeaponMode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcVehicleWeaponMuzzleData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcVehicleWeaponMuzzleData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWaypointComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWaypointComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcPlayerWeaponPropertiesData,19>::cTkFixedArray<cGcPlayerWeaponPropertiesData,19>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcPlayerWeaponPropertiesData,19> *
        ]
    ),
    "cTkFixedArray<cGcPlayerWeaponPropertiesData,19>::~cTkFixedArray<cGcPlayerWeaponPropertiesData,19>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcPlayerWeaponPropertiesData,19> *
        ]
    ),
    "cGcVehicleScanTableEntry::~cGcVehicleScanTableEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleScanTableEntry *
        ]
    ),
    "cTkDynamicArray<cGcPlayerWeaponBiomeProperties>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerWeaponBiomeProperties> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcVehicleScanTableEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcVehicleScanTableEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcVehicleCheckpointComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcVehicleCheckpointComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVehicleComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcVehicleComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVehicleGarageComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcVehicleGarageComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVehicleRaceInviteComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcVehicleRaceInviteComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWaypointComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWaypointComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAIShipWeapons::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCamouflageData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerWeaponBiomeProperties::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerWeaponClass::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerWeaponData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerWeaponPropertiesData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerWeaponPropertiesTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerWeapons::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVehicleCheckpointComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVehicleComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVehicleGarageComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVehicleMuzzleData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVehicleRaceInviteComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVehicleScanTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVehicleScanTableEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVehicleType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVehicleWeaponMode::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVehicleWeaponMuzzleData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWaypointComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCamouflageData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerWeaponBiomeProperties::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerWeaponPropertiesData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerWeaponPropertiesTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcVehicleComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcVehicleScanTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcVehicleScanTableEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWaypointComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerWeaponBiomeProperties::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponBiomeProperties *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerWeaponPropertiesData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponPropertiesData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerWeaponBiomeProperties::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponBiomeProperties *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerWeaponPropertiesData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponPropertiesData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerWeaponPropertiesTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponPropertiesTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcVehicleScanTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleScanTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCamouflageData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCamouflageData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeaponBiomeProperties::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponBiomeProperties *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeaponClass::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponClass *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeaponData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeaponPropertiesData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponPropertiesData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeapons::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapons *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleScanTableEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleScanTableEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleWeaponMuzzleData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleWeaponMuzzleData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeapons::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapons *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcCamouflageData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCamouflageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeaponBiomeProperties::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponBiomeProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeaponClass::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeaponData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeaponPropertiesData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponPropertiesData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeaponPropertiesTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponPropertiesTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeapons::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleMuzzleData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleMuzzleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleWeaponMuzzleData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleWeaponMuzzleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeaponData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponData *
        ]
    ),
    "cGcPlayerWeaponPropertiesData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponPropertiesData *
        ]
    ),
    "cGcPlayerWeaponPropertiesTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponPropertiesTable *
        ]
    ),
    "cGcPlayerWeapons::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceCategory *
        ]
    ),
    "cGcVehicleComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponentData *
        ]
    ),
    "cGcVehicleMuzzleData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleMuzzleData *
        ]
    ),
    "cGcIDPair::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistedStatData *
        ]
    ),
    "cGcPlayerWeapons::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapons *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcAIShipWeapons::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipWeapons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCamouflageData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCamouflageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeaponBiomeProperties::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponBiomeProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeaponClass::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeaponData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeaponPropertiesData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponPropertiesData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeaponPropertiesTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponPropertiesTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerWeapons::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleCheckpointComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleCheckpointComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleGarageComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleGarageComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleMuzzleData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleMuzzleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleRaceInviteComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleRaceInviteComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleScanTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleScanTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleScanTableEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleScanTableEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleWeaponMode::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleWeaponMode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleWeaponMuzzleData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleWeaponMuzzleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWaypointComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaypointComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCamouflageData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCamouflageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCamouflageData *
        ]
    ),
    "cGcPlayerWeaponBiomeProperties::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponBiomeProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerWeaponBiomeProperties *
        ]
    ),
    "cGcPlayerWeaponClass::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerWeaponClass *
        ]
    ),
    "cGcPlayerWeaponPropertiesData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponPropertiesData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerWeaponPropertiesData *
        ]
    ),
    "cGcPlayerWeaponPropertiesTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeaponPropertiesTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerWeaponPropertiesTable *
        ]
    ),
    "cGcPlayerWeapons::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerWeapons *
        ]
    ),
    "cGcVehicleComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcVehicleComponentData *
        ]
    ),
    "cGcVehicleScanTableEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleScanTableEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcVehicleScanTableEntry *
        ]
    ),
    "cGcVehicleType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcVehicleType *
        ]
    ),
    "cGcVehicleWeaponMuzzleData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleWeaponMuzzleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcVehicleWeaponMuzzleData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcMechMeshPartTypeData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPartTypeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcMechWeaponLocation>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMechWeaponLocation> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMechMeshPartData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMechMeshPartData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMechMeshPartTypeData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMechMeshPartTypeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMechWeaponLocation const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMechWeaponLocation *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcMechMeshPart>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMechMeshPart *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMechMeshPart *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcMechMeshType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMechMeshType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMechMeshType *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMechMeshPart const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMechMeshPart *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMechMeshPartData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMechMeshPartData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMechMeshPartTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMechMeshPartTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMechMeshPartTypeData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMechMeshPartTypeData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMechMeshType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMechMeshType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMechPartAudioEventOverride const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMechPartAudioEventOverride *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMechWeaponLocation const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMechWeaponLocation *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMechWeaponLocationPriority const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMechWeaponLocationPriority *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcVehicleData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcVehicleData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcMechMeshPartData,5>::cTkFixedArray<cGcMechMeshPartData,5>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcMechMeshPartData,5> *
        ]
    ),
    "cGcMechMeshPartTypeData::cGcMechMeshPartTypeData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPartTypeData *
        ]
    ),
    "cGcMechMeshPartTypeData::~cGcMechMeshPartTypeData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPartTypeData *
        ]
    ),
    "cGcMechMeshPart::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMechMeshPartData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMechMeshPartTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMechMeshPartTypeData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMechMeshType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMechPartAudioEventOverride::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMechWeaponLocation::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMechWeaponLocationPriority::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVehicleData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMechMeshPartData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMechMeshPartTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMechMeshPartTypeData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMechPartAudioEventOverride::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMechWeaponLocationPriority::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMechPartAudioEventOverride::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechPartAudioEventOverride *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMechMeshPartTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPartTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMechMeshPart::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPart *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechMeshPartData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPartData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechMeshPartTypeData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPartTypeData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechMeshType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechPartAudioEventOverride::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechPartAudioEventOverride *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechWeaponLocation::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechWeaponLocation *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechMeshPart::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPart *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechMeshPartTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPartTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechMeshType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechPartAudioEventOverride::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechPartAudioEventOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechWeaponLocation::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechWeaponLocation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechMeshPartData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPartData *
        ]
    ),
    "cGcMechMeshPartTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPartTable *
        ]
    ),
    "cGcMechPartAudioEventOverride::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechPartAudioEventOverride *
        ]
    ),
    "cGcVehicleData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleData *
        ]
    ),
    "cGcMechMeshPart::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPart *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechMeshPartData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPartData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechMeshPartTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPartTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechMeshPartTypeData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPartTypeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechMeshType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechPartAudioEventOverride::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechPartAudioEventOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechWeaponLocation::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechWeaponLocation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechWeaponLocationPriority::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechWeaponLocationPriority *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechMeshPart::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPart *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMechMeshPart *
        ]
    ),
    "cGcMechMeshPartTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPartTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechMeshPartTypeData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshPartTypeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMechMeshPartTypeData *
        ]
    ),
    "cGcMechMeshType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechMeshType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMechMeshType *
        ]
    ),
    "cGcMechPartAudioEventOverride::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechPartAudioEventOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMechPartAudioEventOverride *
        ]
    ),
    "cGcMechWeaponLocation::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechWeaponLocation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMechWeaponLocation *
        ]
    ),
    "cGcVehicleData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcVehicleData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDebugCameraEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDebugCameraEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcMechPartAudioEventOverride>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMechPartAudioEventOverride> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcAudioWwiseEvents>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioWwiseEvents *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcMechAudioEvent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAudioEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcPlayerCommunicatorMessage>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicatorMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcResourceElement>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDebugCameraEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDebugCameraEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMechPartAudioEventOverride const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMechPartAudioEventOverride *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcAudioWwiseEvents>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAudioWwiseEvents *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAudioWwiseEvents *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcBackgroundSpaceEncounterSpawnConditions>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBackgroundSpaceEncounterSpawnConditions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBackgroundSpaceEncounterSpawnConditions *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcMechAudioEvent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMechAudioEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMechAudioEvent *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlayerCommunicatorMessage>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerCommunicatorMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerCommunicatorMessage *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPulseEncounterSpawnConditions>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPulseEncounterSpawnConditions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPulseEncounterSpawnConditions *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcResourceElement>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcResourceElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcResourceElement *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBackgroundSpaceEncounterInfo const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBackgroundSpaceEncounterInfo *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBackgroundSpaceEncounterSpawnConditions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBackgroundSpaceEncounterSpawnConditions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDebugCamera const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDebugCamera *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDebugCameraEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDebugCameraEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDebugShipTravelLine const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDebugShipTravelLine *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExoMechWeaponData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExoMechWeaponData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMechAudioEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMechAudioEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMechAudioEventTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMechAudioEventTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMechDebugSpawnData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMechDebugSpawnData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPulseEncounterInfo const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPulseEncounterInfo *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPulseEncounterSpawnAbandonedFreighter const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPulseEncounterSpawnAbandonedFreighter *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPulseEncounterSpawnAlienFreighter const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPulseEncounterSpawnAlienFreighter *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPulseEncounterSpawnConditions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPulseEncounterSpawnConditions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPulseEncounterSpawnFrigateFlyby const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPulseEncounterSpawnFrigateFlyby *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPulseEncounterSpawnObject const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPulseEncounterSpawnObject *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostSpecificCreatureBait const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPulseEncounterSpawnSpaceHostiles const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPulseEncounterSpawnSpaceHostiles *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPulseEncounterSpawnTrader const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPulseEncounterSpawnTrader *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechAudioEventTable::~cGcMechAudioEventTable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAudioEventTable *
        ]
    ),
    "cTkDynamicArray<cGcDebugCameraEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDebugCameraEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPulseEncounterSpawnAbandonedFreighter::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcPulseEncounterSpawnAbandonedFreighter *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcPulseEncounterSpawnFrigateFlyby::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcPulseEncounterSpawnFrigateFlyby *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcPulseEncounterSpawnObject::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcPulseEncounterSpawnObject *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcPulseEncounterSpawnPirates::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcPulseEncounterSpawnPirates *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcPulseEncounterSpawnSpaceHostiles::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcPulseEncounterSpawnSpaceHostiles *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcPulseEncounterSpawnTrader::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcPulseEncounterSpawnTrader *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcBackgroundSpaceEncounterInfo::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBackgroundSpaceEncounterSpawnConditions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDebugCamera::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDebugCameraEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDebugShipTravelLine::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExoMechWeaponData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMechAudioEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMechAudioEventTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMechDebugSpawnData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPulseEncounterInfo::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPulseEncounterSpawnAbandonedFreighter::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPulseEncounterSpawnAlienFreighter::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPulseEncounterSpawnConditions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPulseEncounterSpawnFrigateFlyby::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPulseEncounterSpawnObject::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPulseEncounterSpawnPirates::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPulseEncounterSpawnSpaceHostiles::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPulseEncounterSpawnTrader::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBackgroundSpaceEncounterInfo::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDebugCamera::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExoMechWeaponData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMechAudioEvent::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMechAudioEventTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMechDebugSpawnData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPulseEncounterInfo::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPulseEncounterSpawnAbandonedFreighter::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPulseEncounterSpawnFrigateFlyby::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPulseEncounterSpawnObject::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPulseEncounterSpawnSpaceHostiles::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPulseEncounterSpawnTrader::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBackgroundSpaceEncounterInfo::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicatorMessageWeighted *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMechAudioEvent::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAudioEvent *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMechDebugSpawnData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechDebugSpawnData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPulseEncounterInfo::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterInfo *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExoMechWeaponData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExoMechWeaponData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMechAudioEventTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAudioEventTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMechDebugSpawnData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechDebugSpawnData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPulseEncounterInfo::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterInfo *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBackgroundSpaceEncounterInfo::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBackgroundSpaceEncounterInfo *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBackgroundSpaceEncounterSpawnConditions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBackgroundSpaceEncounterSpawnConditions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugCamera::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugCamera *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugCameraEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugCameraEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugShipTravelLine::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugShipTravelLine *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExoMechWeaponData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExoMechWeaponData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechAudioEvent::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAudioEvent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechDebugSpawnData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechDebugSpawnData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPulseEncounterInfo::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterInfo *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPulseEncounterSpawnAlienFreighter::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnAlienFreighter *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPulseEncounterSpawnConditions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnConditions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPulseEncounterSpawnFrigateFlyby::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnFrigateFlyby *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPulseEncounterSpawnObject::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnObject *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPulseEncounterSpawnSpaceHostiles::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnSpaceHostiles *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPulseEncounterSpawnTrader::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnTrader *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBackgroundSpaceEncounterInfo::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBackgroundSpaceEncounterInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugCamera::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugCamera *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugCameraEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugCameraEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugShipTravelLine::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugShipTravelLine *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExoMechWeaponData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExoMechWeaponData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechAudioEventTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAudioEventTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechDebugSpawnData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechDebugSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPulseEncounterInfo::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPulseEncounterSpawnConditions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnConditions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPulseEncounterSpawnObject::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnObject *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBackgroundSpaceEncounterInfo::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBackgroundSpaceEncounterInfo *
        ]
    ),
    "cGcDebugCamera::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugCamera *
        ]
    ),
    "cGcDebugShipTravelLine::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugShipTravelLine *
        ]
    ),
    "cGcExoMechWeaponData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExoMechWeaponData *
        ]
    ),
    "cGcMechAudioEvent::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponentData *
        ]
    ),
    "cGcMechAudioEventTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAudioEventTable *
        ]
    ),
    "cGcMechDebugSpawnData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechDebugSpawnData *
        ]
    ),
    "cGcPulseEncounterInfo::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterInfo *
        ]
    ),
    "cGcPulseEncounterSpawnConditions::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnConditions *
        ]
    ),
    "cGcPulseEncounterSpawnObject::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnObject *
        ]
    ),
    "cGcPulseEncounterSpawnSpaceHostiles::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnSpaceHostiles *
        ]
    ),
    "cGcBackgroundSpaceEncounterInfo::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBackgroundSpaceEncounterInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBackgroundSpaceEncounterSpawnConditions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBackgroundSpaceEncounterSpawnConditions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugCamera::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugCamera *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugCameraEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugCameraEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugShipTravelLine::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugShipTravelLine *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExoMechWeaponData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExoMechWeaponData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechAudioEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAudioEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechAudioEventTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAudioEventTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechDebugSpawnData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechDebugSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPulseEncounterInfo::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPulseEncounterSpawnAbandonedFreighter::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnAbandonedFreighter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPulseEncounterSpawnAlienFreighter::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnAlienFreighter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPulseEncounterSpawnConditions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnConditions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPulseEncounterSpawnFrigateFlyby::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnFrigateFlyby *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPulseEncounterSpawnObject::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnObject *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPulseEncounterSpawnSpaceHostiles::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnSpaceHostiles *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPulseEncounterSpawnTrader::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnTrader *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBackgroundSpaceEncounterInfo::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBackgroundSpaceEncounterInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBackgroundSpaceEncounterInfo *
        ]
    ),
    "cGcBackgroundSpaceEncounterSpawnConditions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBackgroundSpaceEncounterSpawnConditions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBackgroundSpaceEncounterSpawnConditions *
        ]
    ),
    "cGcDebugCamera::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugCamera *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDebugCamera *
        ]
    ),
    "cGcDebugCameraEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugCameraEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDebugCameraEntry *
        ]
    ),
    "cGcDebugShipTravelLine::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugShipTravelLine *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDebugShipTravelLine *
        ]
    ),
    "cGcExoMechWeaponData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExoMechWeaponData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcExoMechWeaponData *
        ]
    ),
    "cGcMechAudioEvent::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAudioEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMechAudioEvent *
        ]
    ),
    "cGcMechAudioEventTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechAudioEventTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMechAudioEventTable *
        ]
    ),
    "cGcMechDebugSpawnData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechDebugSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMechDebugSpawnData *
        ]
    ),
    "cGcPulseEncounterInfo::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPulseEncounterInfo *
        ]
    ),
    "cGcPulseEncounterSpawnConditions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnConditions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPulseEncounterSpawnConditions *
        ]
    ),
    "cGcPulseEncounterSpawnFrigateFlyby::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnFrigateFlyby *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPulseEncounterSpawnFrigateFlyby *
        ]
    ),
    "cGcPulseEncounterSpawnObject::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPulseEncounterSpawnObject *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPulseEncounterSpawnObject *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerCommunicatorMessageWeighted const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerCommunicatorMessageWeighted *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcShipMessage>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShipMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcShipMessage *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDoShipEscort const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDoShipEscort *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDoShipFlybyClose const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDoShipFlybyClose *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDoShipFlybyIntercept const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDoShipFlybyIntercept *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDoShipFlybyMineAsteroids const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDoShipFlybyMineAsteroids *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDoShipFlybyOverhead const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDoShipLandNextToPlayer *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDoShipReceiveMessage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDoShipReceiveMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcShipAICombatDefinition const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShipAICombatDefinition *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcShipAIDeathData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShipAIDeathData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcShipAIPerformanceArray const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShipAIPerformanceArray *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcShipAIPlanetPatrolData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShipAIPlanetPatrolData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcShipDialogue const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShipDialogue *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcShipDialogueTreeEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShipDialogueTreeEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcShipMessage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShipMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcShipOwnershipComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShipOwnershipComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpaceshipAvoidanceData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpaceshipAvoidanceData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpaceshipComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpaceshipComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDoShipClearCommunication::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcDoShipClearCommunication *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcDoShipEscort::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcDoShipEscort *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcDoShipFlybyClose::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcDoShipFlybyClose *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcDoShipFlybyIntercept::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcDoShipFlybyIntercept *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcDoShipFlybyMineAsteroids::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcDoShipFlybyMineAsteroids *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcDoShipFlybyOverhead::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcDoShipFlybyOverhead *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcDoShipReceiveHail::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcDoShipReceiveHail *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcDoShipReceiveMessage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcDoShipReceiveMessage *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcShipOwnershipComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcShipOwnershipComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcSpaceshipComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcSpaceshipComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcSpaceshipShieldComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSpaceshipShieldComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDoShipClearCommunication::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDoShipEscort::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDoShipFlybyClose::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDoShipFlybyIntercept::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDoShipFlybyMineAsteroids::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDoShipFlybyOverhead::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDoShipLandNextToPlayer::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDoShipReceiveHail::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDoShipReceiveMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShipAICombatDefinition::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShipAIDeathData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShipAIPerformanceArray::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShipAIPlanetPatrolData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShipDialogue::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShipDialogueTreeEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShipMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShipOwnershipComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpaceshipAvoidanceData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpaceshipComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpaceshipShieldComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDoShipEscort::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcShipAICombatDefinition::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcShipAIPerformanceArray::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcShipDialogue::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcShipOwnershipComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSpaceshipComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcShipAICombatDefinition::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAICombatDefinition *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcShipDialogue::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipDialogue *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDoShipEscort::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDoShipEscort *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDoShipFlybyClose::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDoShipFlybyClose *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAICombatDefinition::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAICombatDefinition *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAIDeathData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIDeathData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAIPerformanceArray::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPerformanceArray *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAIPlanetPatrolData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPlanetPatrolData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipMessage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipMessage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipAvoidanceData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipAvoidanceData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDoShipEscort::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDoShipEscort *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAICombatDefinition::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAICombatDefinition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAIDeathData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIDeathData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAIPerformanceArray::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPerformanceArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAIPlanetPatrolData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPlanetPatrolData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipDialogue::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipDialogue *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipDialogueTreeEnum::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipDialogueTreeEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipMessage::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipAvoidanceData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipAvoidanceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAICombatDefinition::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAICombatDefinition *
        ]
    ),
    "cGcShipAIDeathData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIDeathData *
        ]
    ),
    "cGcShipAIPlanetPatrolData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPlanetPatrolData *
        ]
    ),
    "cGcShipDialogue::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipDialogue *
        ]
    ),
    "cGcSpaceshipAvoidanceData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipAvoidanceData *
        ]
    ),
    "cGcSpaceshipComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponentData *
        ]
    ),
    "cGcDoShipEscort::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDoShipEscort *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDoShipFlybyClose::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDoShipFlybyClose *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDoShipFlybyIntercept::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDoShipFlybyIntercept *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDoShipFlybyMineAsteroids::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDoShipFlybyMineAsteroids *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDoShipFlybyOverhead::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDoShipLandNextToPlayer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDoShipReceiveMessage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDoShipReceiveMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAICombatDefinition::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAICombatDefinition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAIDeathData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIDeathData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAIPerformanceArray::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPerformanceArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAIPlanetPatrolData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPlanetPatrolData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipDialogue::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipDialogue *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipDialogueTreeEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipDialogueTreeEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipMessage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipOwnershipComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipOwnershipComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipAvoidanceData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipAvoidanceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDoShipEscort::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDoShipEscort *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDoShipEscort *
        ]
    ),
    "cGcDoShipFlybyClose::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDoShipFlybyClose *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDoShipFlybyClose *
        ]
    ),
    "cGcShipAICombatDefinition::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAICombatDefinition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcShipAICombatDefinition *
        ]
    ),
    "cGcShipAIDeathData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIDeathData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcShipAIDeathData *
        ]
    ),
    "cGcShipAIPerformanceArray::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPerformanceArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAIPlanetPatrolData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIPlanetPatrolData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcShipAIPlanetPatrolData *
        ]
    ),
    "cGcShipDialogue::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipDialogue *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipMessage::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcShipMessage *
        ]
    ),
    "cGcSpaceshipAvoidanceData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipAvoidanceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSpaceshipAvoidanceData *
        ]
    ),
    "cGcSpaceshipComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSpaceshipComponentData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcShipAIAttackData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcShipAIAttackData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcShipAICombatDefinition>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcShipAICombatDefinition> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSpaceshipShieldData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSpaceshipShieldData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSpaceshipTravelData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSpaceshipTravelData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcPlayerSpaceshipEngineData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipEngineData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcShipAIAttackData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcShipAIAttackData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcShipAICombatDefinition const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcShipAICombatDefinition *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSpaceshipShieldData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSpaceshipShieldData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSpaceshipTravelData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSpaceshipTravelData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlayerSpaceshipEngineData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipEngineData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipEngineData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcShieldComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShieldComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerSpaceshipAim const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipAim *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerSpaceshipClassBonuses const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipClassBonuses *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerSpaceshipControlData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipControlData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerSpaceshipEngineData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipEngineData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerSpaceshipWarpData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipWarpData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcShipAIAttackData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShipAIAttackData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcShipAIAttackDataTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShipAIAttackDataTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpaceshipTravelData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpaceshipTravelData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcShipAIAttackData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcShipAIAttackData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcShipAICombatDefinition>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcShipAICombatDefinition> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSpaceshipShieldData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSpaceshipShieldData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSpaceshipTravelData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSpaceshipTravelData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcEngineComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEngineComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEngineComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerSpaceshipAim::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerSpaceshipClassBonuses::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerSpaceshipControlData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerSpaceshipEngineData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerSpaceshipWarpData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShieldComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShipAIAttackData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShipAIAttackDataTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpaceshipTravelData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerSpaceshipControlData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerSpaceshipWarpData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcShipAIAttackDataTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerSpaceshipControlData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipControlData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcShipAIAttackDataTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackDataTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerSpaceshipAim::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipAim *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpaceshipClassBonuses::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipClassBonuses *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpaceshipControlData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipControlData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpaceshipEngineData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipEngineData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpaceshipWarpData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipWarpData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAIAttackData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAIAttackDataTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackDataTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipTravelData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipTravelData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpaceshipEngineData::Interpolate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipEngineData *
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipEngineData *
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipEngineData *
            ctypes.c_float,  # float
        ]
    ),
    "cGcPlayerSpaceshipAim::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipAim *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpaceshipClassBonuses::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipClassBonuses *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpaceshipControlData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipControlData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpaceshipEngineData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipEngineData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpaceshipWarpData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipWarpData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAIAttackData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAIAttackDataTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackDataTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipTravelData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipTravelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAIAttackDataTable::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackDataTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpaceshipAim::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipAim *
        ]
    ),
    "cGcPlayerSpaceshipClassBonuses::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipClassBonuses *
        ]
    ),
    "cGcPlayerSpaceshipControlData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipControlData *
        ]
    ),
    "cGcPlayerSpaceshipEngineData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipEngineData *
        ]
    ),
    "cGcPlayerSpaceshipWarpData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipWarpData *
        ]
    ),
    "cGcShipAIAttackData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackData *
        ]
    ),
    "cGcSpaceshipTravelData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipTravelData *
        ]
    ),
    "cGcShieldComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShieldComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpaceshipAim::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipAim *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpaceshipClassBonuses::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipClassBonuses *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpaceshipControlData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipControlData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpaceshipEngineData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipEngineData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpaceshipWarpData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipWarpData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAIAttackData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipAIAttackDataTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackDataTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipTravelData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipTravelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpaceshipAim::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipAim *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipAim *
        ]
    ),
    "cGcPlayerSpaceshipClassBonuses::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipClassBonuses *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipClassBonuses *
        ]
    ),
    "cGcPlayerSpaceshipControlData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipControlData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipControlData *
        ]
    ),
    "cGcPlayerSpaceshipEngineData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipEngineData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipEngineData *
        ]
    ),
    "cGcPlayerSpaceshipWarpData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipWarpData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipWarpData *
        ]
    ),
    "cGcShipAIAttackData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIAttackData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcShipAIAttackData *
        ]
    ),
    "cGcSpaceshipTravelData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipTravelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSpaceshipTravelData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcAISpaceshipModelData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAISpaceshipModelData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcAISpaceshipPreloadCacheData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAISpaceshipPreloadCacheData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcRealityCommonFactions>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityCommonFactions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<cTkVector2>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAISpaceshipModelData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAISpaceshipModelData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAISpaceshipPreloadCacheData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAISpaceshipPreloadCacheData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcAISpaceshipRoles>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAISpaceshipRoles *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAISpaceshipRoles *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcRealityCommonFactions>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRealityCommonFactions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRealityCommonFactions *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAISpaceshipModelDataArray const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAISpaceshipModelDataArray *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAISpaceshipPreloadCacheData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAISpaceshipPreloadCacheData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAISpaceshipPreloadList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAISpaceshipPreloadList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAISpaceshipRoles const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAISpaceshipRoles *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAISpaceshipTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAISpaceshipTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAISpaceshipWeightingData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAISpaceshipWeightingData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBountySpawnInfo const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBountySpawnInfo *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDebugPlanetPos const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDebugPlanetPos *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFreighterSyncComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFreighterSyncComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerSquadronConfig const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerSquadronConfig *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcShipFlareComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShipFlareComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpaceshipShieldData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpaceshipShieldData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcAIShipSpawnData>::~cTkDynamicArray<cGcAIShipSpawnData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAIShipSpawnData> *
        ]
    ),
    "cGcAIShipSpawnData::~cGcAIShipSpawnData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSpawnData *
        ]
    ),
    "cTkDynamicArray<cGcAISpaceshipModelData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAISpaceshipModelData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFreighterSyncComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFreighterSyncComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShipFlareComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcShipFlareComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAISpaceshipModelDataArray::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAISpaceshipPreloadCacheData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAISpaceshipPreloadList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAISpaceshipRoles::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAISpaceshipTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAISpaceshipWeightingData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBountySpawnInfo::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDebugPlanetPos::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFreighterSyncComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerSquadronConfig::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShipFlareComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpaceshipShieldData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAISpaceshipModelDataArray::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAISpaceshipPreloadCacheData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAISpaceshipPreloadList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBountySpawnInfo::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerSquadronConfig::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAISpaceshipModelDataArray::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipModelDataArray *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAISpaceshipPreloadCacheData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipPreloadCacheData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBountySpawnInfo::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBountySpawnInfo *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAISpaceshipModelDataArray::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipModelDataArray *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBountySpawnInfo::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBountySpawnInfo *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerSquadronConfig::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronConfig *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAISpaceshipModelDataArray::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipModelDataArray *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipPreloadCacheData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipPreloadCacheData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipRoles::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipRoles *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipTypes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipTypes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipWeightingData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipWeightingData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBountySpawnInfo::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBountySpawnInfo *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugPlanetPos::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugPlanetPos *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSquadronConfig::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronConfig *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipShieldData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipShieldData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipModelDataArray::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipModelDataArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipPreloadCacheData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipPreloadCacheData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipRoles::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipRoles *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipTypes::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipWeightingData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipWeightingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBountySpawnInfo::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBountySpawnInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugPlanetPos::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugPlanetPos *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSquadronConfig::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronConfig *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipShieldData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipShieldData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipPreloadCacheData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipPreloadCacheData *
        ]
    ),
    "cGcAISpaceshipWeightingData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipWeightingData *
        ]
    ),
    "cGcBountySpawnInfo::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBountySpawnInfo *
        ]
    ),
    "cGcDebugPlanetPos::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugPlanetPos *
        ]
    ),
    "cGcPlayerSquadronConfig::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronConfig *
        ]
    ),
    "cGcSpaceshipShieldData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipShieldData *
        ]
    ),
    "cGcAISpaceshipModelDataArray::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipModelDataArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipPreloadCacheData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipPreloadCacheData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipPreloadList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipPreloadList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipRoles::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipRoles *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipWeightingData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipWeightingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBountySpawnInfo::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBountySpawnInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugPlanetPos::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugPlanetPos *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSquadronConfig::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronConfig *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipFlareComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipFlareComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipShieldData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipShieldData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipPreloadCacheData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipPreloadCacheData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAISpaceshipPreloadCacheData *
        ]
    ),
    "cGcAISpaceshipRoles::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipRoles *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAISpaceshipRoles *
        ]
    ),
    "cGcAISpaceshipTypes::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAISpaceshipTypes *
        ]
    ),
    "cGcBountySpawnInfo::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBountySpawnInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBountySpawnInfo *
        ]
    ),
    "cGcPlayerSquadronConfig::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronConfig *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerSquadronConfig *
        ]
    ),
    "cGcSpaceshipShieldData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipShieldData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSpaceshipShieldData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcAIShipSpawnData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAIShipSpawnData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlanetWaterColourData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlanetWaterColourData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlanetWeatherColourData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlanetWeatherColourData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcTileTypeSet>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTileTypeSet> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcAISpaceshipRoles>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipRoles *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcSpaceshipClasses>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipClasses *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAIShipSpawnData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAIShipSpawnData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAISpaceshipInstanceData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAISpaceshipInstanceData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAISpaceshipModelDataArray const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAISpaceshipModelDataArray *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlanetWaterColourData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlanetWaterColourData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlanetWeatherColourData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlanetWeatherColourData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcTerrainTextureSettings const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcTerrainTextureSettings *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcTileTypeSet const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcTileTypeSet *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcWeatherColourSettingList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcWeatherColourSettingList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcSpaceshipClasses>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpaceshipClasses *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSpaceshipClasses *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcWeatherColourSettingList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherColourSettingList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWeatherColourSettingList *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAIShipDebugSpawnData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAIShipDebugSpawnData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAIShipSpawnData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAIShipSpawnData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAIShipSpawnMarkerData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAIShipSpawnMarkerData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAISpaceshipComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAISpaceshipComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAISpaceshipInstanceData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAISpaceshipInstanceData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAISpaceshipManagerData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAISpaceshipManagerData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAISpaceshipMappingData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAISpaceshipMappingData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAISpaceshipModelData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAISpaceshipModelData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTerrainEditing const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTerrainEditing *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTerrainTexture const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTerrainTexture *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTileTypeSets const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTileTypeSets *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTradingClass const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTradingClass *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWaterColourSettingList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWaterColourSettingList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWealthClass const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWealthClass *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeatherColourSettingList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherColourSettingList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeatherColourSettings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherColourSettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeatherWeightings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherWeightings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcAIShipSpawnData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAIShipSpawnData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPlanetWaterColourData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlanetWaterColourData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPlanetWeatherColourData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlanetWeatherColourData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcTileTypeSet>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTileTypeSet> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAISpaceshipComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcAISpaceshipComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcAIShipDebugSpawnData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAIShipSpawnData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAIShipSpawnMarkerData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAISpaceshipComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAISpaceshipInstanceData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAISpaceshipManagerData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAISpaceshipMappingData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAISpaceshipModelData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTerrainEditing::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTerrainTexture::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTileTypeSets::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTradingClass::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWaterColourSettingList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWealthClass::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeatherColourSettingList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeatherColourSettings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeatherWeightings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAIShipSpawnData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAIShipSpawnMarkerData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAISpaceshipComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAISpaceshipManagerData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAISpaceshipModelData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTerrainTexture::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTileTypeSets::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWaterColourSettingList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWeatherColourSettingList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWeatherColourSettings::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcAIShipSpawnData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAIShipSpawnData> *
        ]
    ),
    "cGcAIShipSpawnData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSpawnData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAISpaceshipModelData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipModelData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAIShipSpawnData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSpawnData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAISpaceshipManagerData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManagerData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTerrainTexture::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainTexture *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTileTypeSets::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTileTypeSets *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWeatherColourSettings::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherColourSettings *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAIShipDebugSpawnData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipDebugSpawnData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAIShipSpawnData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSpawnData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAIShipSpawnMarkerData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSpawnMarkerData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipInstanceData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipInstanceData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipModelData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipModelData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainEditing::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditing *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradingClass::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingClass *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWealthClass::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWealthClass *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherColourSettingList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherColourSettingList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherWeightings::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherWeightings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradingClass::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingClass *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcWealthClass::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWealthClass *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcAIShipDebugSpawnData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipDebugSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAIShipSpawnData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAIShipSpawnMarkerData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSpawnMarkerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipModelData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipModelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainEditing::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditing *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainTexture::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainTexture *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradingClass::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWealthClass::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWealthClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherColourSettings::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherColourSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherWeightings::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherWeightings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAIShipDebugSpawnData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipDebugSpawnData *
        ]
    ),
    "cGcAIShipSpawnData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSpawnData *
        ]
    ),
    "cGcAIShipSpawnMarkerData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSpawnMarkerData *
        ]
    ),
    "cGcAISpaceshipComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponentData *
        ]
    ),
    "cGcAISpaceshipMappingData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipMappingData *
        ]
    ),
    "cGcAISpaceshipModelData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipModelData *
        ]
    ),
    "cGcTerrainEditing::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditing *
        ]
    ),
    "cGcTerrainTexture::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainTexture *
        ]
    ),
    "cGcWeatherWeightings::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherWeightings *
        ]
    ),
    "cGcTradingClass::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingClass *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcWealthClass::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWealthClass *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcAIShipDebugSpawnData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipDebugSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAIShipSpawnData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAIShipSpawnMarkerData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSpawnMarkerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipInstanceData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipInstanceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipManagerData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipManagerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipMappingData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipMappingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipModelData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipModelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainEditing::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditing *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainTexture::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainTexture *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTileTypeSets::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTileTypeSets *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradingClass::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWaterColourSettingList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaterColourSettingList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWealthClass::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWealthClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherColourSettingList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherColourSettingList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherColourSettings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherColourSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherWeightings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherWeightings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAIShipDebugSpawnData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipDebugSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAIShipDebugSpawnData *
        ]
    ),
    "cGcAIShipSpawnData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAIShipSpawnData *
        ]
    ),
    "cGcAIShipSpawnMarkerData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSpawnMarkerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAIShipSpawnMarkerData *
        ]
    ),
    "cGcAISpaceshipComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAISpaceshipComponentData *
        ]
    ),
    "cGcAISpaceshipInstanceData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipInstanceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAISpaceshipInstanceData *
        ]
    ),
    "cGcAISpaceshipModelData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipModelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAISpaceshipModelData *
        ]
    ),
    "cGcTerrainEditing::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditing *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTerrainEditing *
        ]
    ),
    "cGcTradingClass::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTradingClass *
        ]
    ),
    "cGcWealthClass::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWealthClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWealthClass *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSolarSystemSkyColourData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSolarSystemSkyColourData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSpawnDensity>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSpawnDensity> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcPlanetHeavyAirData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetHeavyAirData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcScreenFilters>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScreenFilters *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSolarSystemSkyColourData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSolarSystemSkyColourData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSpawnDensity const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSpawnDensity *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlanetHeavyAirData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetHeavyAirData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetHeavyAirData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcRainbowType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRainbowType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRainbowType *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcScreenFilters>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScreenFilters *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScreenFilters *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetSectionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetSectionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetSentinelData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetSentinelData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetSize const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetSize *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetTerrainColour const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetTerrainColour *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetTradingData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetTradingData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetWaterColourData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetWaterColourData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetWaterData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetWaterData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetWeatherColourData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetWeatherColourData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetWeatherColourIndex const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetWeatherColourIndex *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetWeatherData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetWeatherData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerConflictData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerConflictData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRainbowType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRainbowType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScreenFilterOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScreenFilterOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSolarSystemClass const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSolarSystemClass *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSolarSystemSkyColourData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSolarSystemSkyColourData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpaceSkyColourSettingList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpaceSkyColourSettingList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpawnDensity const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpawnDensity *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpawnDensityList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpawnDensityList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTerrainControls const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTerrainControls *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemSkyColourData::cGcSolarSystemSkyColourData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSolarSystemSkyColourData *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemSkyColourData *
            ctypes.c_ulonglong,  # const cGcSolarSystemSkyColourData *
        ]
    ),
    "cTkDynamicArray<cGcSolarSystemSkyColourData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSolarSystemSkyColourData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSpawnDensity>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSpawnDensity> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlanetSectionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetSentinelData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetSize::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetTerrainColour::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetTradingData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetWaterColourData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetWaterData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetWeatherColourData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetWeatherColourIndex::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetWeatherData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerConflictData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRainbowType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScreenFilterOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSolarSystemClass::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSolarSystemSkyColourData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpaceSkyColourSettingList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpawnDensity::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpawnDensityList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTerrainControls::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetTerrainColour::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetTradingData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetWaterData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetWeatherData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScreenFilterOption::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSpaceSkyColourSettingList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSpawnDensityList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetTradingData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetTradingData *
        ]
    ),
    "cGcPlanetWaterData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWaterData *
        ]
    ),
    "cGcPlanetWeatherData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWeatherData *
        ]
    ),
    "cGcCreatureAlertData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureAlertData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetWaterData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWaterData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetWeatherData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWeatherData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetSectionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSectionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetSentinelData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSentinelData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetSize::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSize *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetTradingData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetTradingData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetWaterColourData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWaterColourData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetWaterData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWaterData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetWeatherColourData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWeatherColourData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetWeatherColourIndex::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWeatherColourIndex *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetWeatherData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWeatherData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerConflictData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerConflictData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRainbowType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRainbowType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScreenFilterOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScreenFilterOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemClass::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemClass *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemSkyColourData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemSkyColourData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpawnDensity::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnDensity *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainControls::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainControls *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetTradingData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetTradingData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPlanetWeatherColourData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWeatherColourData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPlayerConflictData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerConflictData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPlanetSectionData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSectionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetSentinelData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSentinelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetSize::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSize *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetTradingData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetTradingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetWaterColourData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWaterColourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetWaterData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWaterData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetWeatherColourData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWeatherColourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetWeatherColourIndex::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWeatherColourIndex *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetWeatherData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWeatherData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerConflictData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerConflictData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRainbowType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRainbowType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScreenFilterOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScreenFilterOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemClass::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemSkyColourData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemSkyColourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpawnDensity::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnDensity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainControls::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainControls *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetSectionData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSectionData *
        ]
    ),
    "cGcPlanetSentinelData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSentinelData *
        ]
    ),
    "cGcPlanetTradingData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetTradingData *
        ]
    ),
    "cGcPlanetWaterColourData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWaterColourData *
        ]
    ),
    "cGcPlanetWaterData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWaterData *
        ]
    ),
    "cGcPlanetWeatherColourData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWeatherColourData *
        ]
    ),
    "cGcAccessibleOverride_Text::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAccessibleOverride_Text *
        ]
    ),
    "cGcPlanetWeatherData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWeatherData *
        ]
    ),
    "cGcScreenFilterOption::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScreenFilterOption *
        ]
    ),
    "cGcTerrainControls::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainControls *
        ]
    ),
    "cGcPlanetTradingData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetTradingData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcPlayerConflictData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerConflictData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcPlanetSectionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSectionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetSentinelData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSentinelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetSize::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSize *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetTerrainColour::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetTerrainColour *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetTradingData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetTradingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetWaterColourData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWaterColourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetWaterData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWaterData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetWeatherColourData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWeatherColourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetWeatherColourIndex::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWeatherColourIndex *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetWeatherData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWeatherData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerConflictData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerConflictData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRainbowType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRainbowType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScreenFilterOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScreenFilterOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemClass::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemSkyColourData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemSkyColourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceSkyColourSettingList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceSkyColourSettingList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpawnDensity::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnDensity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpawnDensityList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnDensityList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainControls::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainControls *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetSentinelData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSentinelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetSentinelData *
        ]
    ),
    "cGcPlanetSize::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSize *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetSize *
        ]
    ),
    "cGcPlanetTerrainColour::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetTerrainColour *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetTerrainColour *
        ]
    ),
    "cGcPlanetTradingData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetTradingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetTradingData *
        ]
    ),
    "cGcPlanetWaterColourData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWaterColourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetWaterColourData *
        ]
    ),
    "cGcPlanetWaterData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWaterData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetWaterData *
        ]
    ),
    "cGcPlanetWeatherColourData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWeatherColourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetWeatherColourData *
        ]
    ),
    "cGcPlanetWeatherColourIndex::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWeatherColourIndex *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetWeatherColourIndex *
        ]
    ),
    "cGcPlanetWeatherData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetWeatherData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetWeatherData *
        ]
    ),
    "cGcPlayerConflictData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerConflictData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerConflictData *
        ]
    ),
    "cGcRainbowType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRainbowType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRainbowType *
        ]
    ),
    "cGcSolarSystemClass::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSolarSystemClass *
        ]
    ),
    "cGcSolarSystemSkyColourData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemSkyColourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSolarSystemSkyColourData *
        ]
    ),
    "cGcSpawnDensity::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnDensity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSpawnDensity *
        ]
    ),
    "cGcTerrainControls::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainControls *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTerrainControls *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcExternalObjectListOptions>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExternalObjectListOptions> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlanetDataResourceHint>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlanetDataResourceHint> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlanetSectionData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlanetSectionData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlanetaryMappingValues>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetaryMappingValues *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcBiomeSubType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeSubType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcBiomeType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcGalaxyStarTypes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcPlanetClass>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcPlanetLife>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetLife *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcPlanetSize>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSize *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadXML<ushort>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<cTkColour>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcColourPaletteData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcColourPaletteData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcExternalObjectListOptions const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcExternalObjectListOptions *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcHeavyAirColourData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcHeavyAirColourData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlanetDataResourceHint const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlanetDataResourceHint *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlanetSectionData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlanetSectionData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlanetaryMappingValues const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlanetaryMappingValues *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcBiomeSubType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBiomeSubType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBiomeSubType *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcBiomeType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBiomeType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBiomeType *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcGalaxyStarTypes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyStarTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyStarTypes *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlanetClass>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetClass *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlanetHazardData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetHazardData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetHazardData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlanetLife>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetLife *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetLife *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlanetSize>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetSize *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetSize *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHeavyAirSettingValues const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHeavyAirSettingValues *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMiningSubstanceData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMiningSubstanceData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcOverlayTexture const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcOverlayTexture *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetClass const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetClass *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetColourData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetColourData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetDataResourceHint const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetDataResourceHint *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetGenerationInputData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetGenerationInputData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetGenerationIntermediateData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetGenerationIntermediateData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetHazardData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetHazardData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetHeavyAirData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetHeavyAirData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetInfo const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetInfo *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetRingData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetRingData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetaryMappingData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetaryMappingData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetaryMappingTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetaryMappingTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetaryMappingValues const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetaryMappingValues *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHeavyAirSettingValues::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMiningSubstanceData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcOverlayTexture::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetClass::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetColourData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetDataResourceHint::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetGenerationInputData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetGenerationIntermediateData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetHazardData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetHeavyAirData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetInfo::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetRingData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetaryMappingData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetaryMappingTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetaryMappingValues::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHeavyAirSettingValues::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMiningSubstanceData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetColourData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetGenerationInputData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetGenerationIntermediateData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetHeavyAirData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetaryMappingData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetaryMappingTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetaryMappingValues::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcHeavyAirSettingValues::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeavyAirSettingValues *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMiningSubstanceData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMiningSubstanceData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetColourData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetColourData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetGenerationInputData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerationInputData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetGenerationIntermediateData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerationIntermediateData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetHeavyAirData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetHeavyAirData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetGenerationIntermediateData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerationIntermediateData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcHeavyAirSettingValues::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeavyAirSettingValues *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMiningSubstanceData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMiningSubstanceData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOverlayTexture::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcOverlayTexture *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetClass::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetClass *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetColourData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetColourData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetDataResourceHint::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetDataResourceHint *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetGenerationInputData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerationInputData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetGenerationIntermediateData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerationIntermediateData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetHazardData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetHazardData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetHeavyAirData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetHeavyAirData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetInfo::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetInfo *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetRingData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetRingData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetaryMappingValues::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetaryMappingValues *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHeavyAirSettingValues::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeavyAirSettingValues *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMiningSubstanceData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMiningSubstanceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetClass::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetColourData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetColourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetGenerationInputData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerationInputData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetGenerationIntermediateData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerationIntermediateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetHazardData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetHazardData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetHeavyAirData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetHeavyAirData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetInfo::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetRingData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetRingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHeavyAirSettingValues::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeavyAirSettingValues *
        ]
    ),
    "cGcMiningSubstanceData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMiningSubstanceData *
        ]
    ),
    "cGcOverlayTexture::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOverlayTexture *
        ]
    ),
    "cGcPlanetColourData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetColourData *
        ]
    ),
    "cGcPlanetData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetData *
        ]
    ),
    "cGcPlanetGenerationInputData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerationInputData *
        ]
    ),
    "cGcPlanetGenerationIntermediateData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerationIntermediateData *
        ]
    ),
    "cGcPlanetHazardData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetHazardData *
        ]
    ),
    "cGcPlanetHeavyAirData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetHeavyAirData *
        ]
    ),
    "cGcPlanetInfo::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetInfo *
        ]
    ),
    "cGcPlanetRingData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetRingData *
        ]
    ),
    "cGcHeavyAirSettingValues::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeavyAirSettingValues *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMiningSubstanceData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMiningSubstanceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOverlayTexture::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOverlayTexture *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetClass::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetColourData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetColourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetDataResourceHint::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetDataResourceHint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetGenerationInputData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerationInputData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetGenerationIntermediateData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerationIntermediateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetHazardData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetHazardData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetHeavyAirData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetHeavyAirData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetInfo::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetRingData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetRingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetaryMappingData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetaryMappingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetaryMappingTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetaryMappingTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetaryMappingValues::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetaryMappingValues *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHeavyAirSettingValues::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeavyAirSettingValues *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcHeavyAirSettingValues *
        ]
    ),
    "cGcMiningSubstanceData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMiningSubstanceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMiningSubstanceData *
        ]
    ),
    "cGcPlanetClass::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetClass *
        ]
    ),
    "cGcPlanetColourData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetColourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetData *
        ]
    ),
    "cGcPlanetDataResourceHint::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetDataResourceHint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetDataResourceHint *
        ]
    ),
    "cGcPlanetGenerationInputData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerationInputData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetGenerationInputData *
        ]
    ),
    "cGcPlanetGenerationIntermediateData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetGenerationIntermediateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetGenerationIntermediateData *
        ]
    ),
    "cGcPlanetHazardData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetHazardData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetHeavyAirData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetHeavyAirData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetHeavyAirData *
        ]
    ),
    "cGcPlanetInfo::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetInfo *
        ]
    ),
    "cGcPlanetRingData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetRingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetRingData *
        ]
    ),
    "cGcPlanetaryMappingValues::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetaryMappingValues *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetaryMappingValues *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBiomeFileListOption>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBiomeFileListOption> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBiomeType>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBiomeType> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcExternalObjectFileList>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExternalObjectFileList> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcScreenFilterOption>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcScreenFilterOption> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcMiningSubstanceData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMiningSubstanceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcSolarSystemLocatorTypes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemLocatorTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBiomeFileListOption const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBiomeFileListOption *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBiomeFileListOptions const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBiomeFileListOptions *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBiomeType const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBiomeType *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcExternalObjectFileList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcExternalObjectFileList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcHeavyAirSettingValues const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcHeavyAirSettingValues *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcScreenFilterOption const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcScreenFilterOption *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcWeatherWeightings const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcWeatherWeightings *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcBiomeFileListOption>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBiomeFileListOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBiomeFileListOption *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcMiningSubstanceData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMiningSubstanceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMiningSubstanceData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcSolarSystemLocatorTypes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSolarSystemLocatorTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSolarSystemLocatorTypes *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBiomeCloudSettings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBiomeCloudSettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBiomeData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBiomeData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBiomeFileList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBiomeFileList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBiomeFileListOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBiomeFileListOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBiomeFileListOptions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBiomeFileListOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBiomeList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBiomeList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBiomeSubType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBiomeSubType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBiomeType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBiomeType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExternalObjectFileList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExternalObjectFileList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExternalObjectList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExternalObjectList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHeavyAirColourData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHeavyAirColourData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHeavyAirList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHeavyAirList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHeavyAirSetting const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHeavyAirSetting *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCNavigationAreaComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCNavigationAreaComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSolarSystemLocator const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSolarSystemLocator *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSolarSystemLocatorChoice const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSolarSystemLocatorChoice *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSolarSystemLocatorTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSolarSystemLocatorTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSolarSystemTraderSpawnData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSolarSystemTraderSpawnData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpaceStationSpawnData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpaceStationSpawnData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcBiomeFileListOption>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBiomeFileListOption> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcExternalObjectFileList>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExternalObjectFileList> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNPCNavigationAreaComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBiomeCloudSettings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBiomeData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBiomeFileList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBiomeFileListOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBiomeFileListOptions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBiomeList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBiomeSubType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBiomeType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExternalObjectFileList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExternalObjectList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHeavyAirColourData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHeavyAirList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHeavyAirSetting::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCNavigationAreaComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSolarSystemLocator::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSolarSystemLocatorChoice::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSolarSystemLocatorTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSolarSystemTraderSpawnData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpaceStationSpawnData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBiomeData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBiomeFileList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBiomeFileListOptions::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExternalObjectFileList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExternalObjectList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcHeavyAirList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcHeavyAirSetting::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcExternalObjectFileList>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExternalObjectFileList> *
        ]
    ),
    "cGcHeavyAirSetting::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeavyAirSetting *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBiomeData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBiomeFileList::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeFileList *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExternalObjectList::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExternalObjectList *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBiomeCloudSettings::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeCloudSettings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeFileListOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeFileListOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeFileListOptions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeFileListOptions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeSubType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeSubType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExternalObjectFileList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExternalObjectFileList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHeavyAirColourData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeavyAirColourData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHeavyAirSetting::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeavyAirSetting *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCNavigationAreaComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemLocator::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemLocator *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemLocatorChoice::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemLocatorChoice *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemLocatorTypes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemLocatorTypes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemTraderSpawnData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTraderSpawnData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceStationSpawnData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceStationSpawnData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeSubType::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeSubType *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcBiomeType::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeType *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcBiomeCloudSettings::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeCloudSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeFileList::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeFileList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeFileListOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeFileListOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeList::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeSubType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeSubType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExternalObjectFileList::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExternalObjectFileList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHeavyAirColourData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeavyAirColourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHeavyAirSetting::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeavyAirSetting *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemLocator::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemLocator *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemLocatorChoice::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemLocatorChoice *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemLocatorTypes::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemLocatorTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemTraderSpawnData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTraderSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceStationSpawnData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceStationSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeData *
        ]
    ),
    "cGcBiomeFileListOption::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeFileListOption *
        ]
    ),
    "cGcBiomeList::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeList *
        ]
    ),
    "cGcHeavyAirColourData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeavyAirColourData *
        ]
    ),
    "cGcHeavyAirSetting::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeavyAirSetting *
        ]
    ),
    "cGcSolarSystemLocator::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemLocator *
        ]
    ),
    "cGcSolarSystemLocatorChoice::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemLocatorChoice *
        ]
    ),
    "cGcSolarSystemTraderSpawnData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTraderSpawnData *
        ]
    ),
    "cGcSpaceStationSpawnData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceStationSpawnData *
        ]
    ),
    "cGcBiomeSubType::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeSubType *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcBiomeType::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeType *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcBiomeCloudSettings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeCloudSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeFileList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeFileList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeFileListOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeFileListOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeFileListOptions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeFileListOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeSubType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeSubType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExternalObjectFileList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExternalObjectFileList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExternalObjectList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExternalObjectList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHeavyAirColourData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeavyAirColourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHeavyAirList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeavyAirList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHeavyAirSetting::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeavyAirSetting *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCNavigationAreaComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemLocator::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemLocator *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemLocatorChoice::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemLocatorChoice *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemLocatorTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemLocatorTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemTraderSpawnData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTraderSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceStationSpawnData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceStationSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeCloudSettings::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeCloudSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBiomeCloudSettings *
        ]
    ),
    "cGcBiomeData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBiomeData *
        ]
    ),
    "cGcBiomeFileListOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeFileListOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBiomeFileListOption *
        ]
    ),
    "cGcBiomeList::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeSubType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeSubType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBiomeSubType *
        ]
    ),
    "cGcBiomeType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBiomeType *
        ]
    ),
    "cGcExternalObjectFileList::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExternalObjectFileList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcExternalObjectFileList *
        ]
    ),
    "cGcHeavyAirSetting::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeavyAirSetting *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCNavigationAreaComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavigationAreaComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCNavigationAreaComponentData *
        ]
    ),
    "cGcSolarSystemLocator::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemLocator *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSolarSystemLocator *
        ]
    ),
    "cGcSolarSystemLocatorChoice::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemLocatorChoice *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSolarSystemLocatorChoice *
        ]
    ),
    "cGcSolarSystemLocatorTypes::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemLocatorTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSolarSystemLocatorTypes *
        ]
    ),
    "cGcSolarSystemTraderSpawnData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTraderSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSolarSystemTraderSpawnData *
        ]
    ),
    "cGcSpaceStationSpawnData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceStationSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSpaceStationSpawnData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSmokeBotPlanetReport>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSmokeBotPlanetReport> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSmokeBotSystemReport>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSmokeBotSystemReport> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSolarSystemLocator>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSolarSystemLocator> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcAsteroidGeneratorAssignment>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGeneratorAssignment *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcBiomeList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcSmokeBotStats>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSmokeBotStats *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcSolarSystemLocatorChoice>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemLocatorChoice *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcSolarSystemTraderSpawnData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemTraderSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBiomeList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBiomeList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlanetGenerationInputData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlanetGenerationInputData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSmokeBotPlanetReport const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSmokeBotPlanetReport *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSmokeBotSystemReport const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSmokeBotSystemReport *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSolarSystemLocator const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSolarSystemLocator *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcAsteroidGeneratorAssignment>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAsteroidGeneratorAssignment *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAsteroidGeneratorAssignment *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcBiomeList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBiomeList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBiomeList *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcSmokeBotStats>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSmokeBotStats *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSmokeBotStats *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcSolarSystemLocatorChoice>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSolarSystemLocatorChoice *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSolarSystemLocatorChoice *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcSolarSystemTraderSpawnData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSolarSystemTraderSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSolarSystemTraderSpawnData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAsteroidGeneratorAssignment const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAsteroidGeneratorAssignment *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAsteroidGeneratorRing const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAsteroidGeneratorRing *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAsteroidGeneratorSlab const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAsteroidGeneratorSlab *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAsteroidGeneratorSurround const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAsteroidGeneratorSurround *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAtmosphereList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtmosphereList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBiomeListPerStarType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBiomeListPerStarType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExperienceTimers const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExperienceTimers *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGeneratedShipCounts const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGeneratedShipCounts *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSmokeBotPlanetReport const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSmokeBotPlanetReport *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSmokeBotReport const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSmokeBotReport *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSmokeBotStats const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSmokeBotStats *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSmokeBotSystemReport const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSmokeBotSystemReport *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSolarGenerationData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSolarGenerationData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSolarSystemData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSolarSystemData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSolarSystemEventWarpIn const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSolarSystemEventWarpIn *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSolarSystemEventWarpOut const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSolarSystemEventWarpOut *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSolarSystemEventWarpPlayer const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSolarSystemEventWarpPlayer *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcSmokeBotPlanetReport>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSmokeBotPlanetReport> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSmokeBotSystemReport>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSmokeBotSystemReport> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAsteroidGeneratorRing::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcAsteroidGeneratorRing *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcAsteroidGeneratorSlab::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcAsteroidGeneratorSlab *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcAsteroidGeneratorSurround::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcAsteroidGeneratorSurround *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcAsteroidGeneratorAssignment::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAsteroidGeneratorRing::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAsteroidGeneratorSlab::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAsteroidGeneratorSurround::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAtmosphereList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBiomeListPerStarType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExperienceTimers::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGeneratedShipCounts::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSmokeBotPlanetReport::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSmokeBotReport::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSmokeBotStats::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSmokeBotSystemReport::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSolarGenerationData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSolarSystemData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSolarSystemEventWarpIn::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSolarSystemEventWarpOut::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSolarSystemEventWarpPlayer::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAsteroidGeneratorAssignment::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAsteroidGeneratorRing::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAsteroidGeneratorSlab::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAsteroidGeneratorSurround::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAtmosphereList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBiomeListPerStarType::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSmokeBotReport::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSmokeBotSystemReport::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSolarSystemData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSolarSystemEventWarpIn::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSolarSystemEventWarpPlayer::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSolarSystemData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBiomeListPerStarType::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeListPerStarType *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSolarSystemData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAsteroidGeneratorAssignment::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGeneratorAssignment *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAsteroidGeneratorRing::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGeneratorRing *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAsteroidGeneratorSlab::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGeneratorSlab *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAsteroidGeneratorSurround::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGeneratorSurround *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeListPerStarType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeListPerStarType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExperienceTimers::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceTimers *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratedShipCounts::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedShipCounts *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSmokeBotPlanetReport::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSmokeBotPlanetReport *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSmokeBotStats::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSmokeBotStats *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSmokeBotSystemReport::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSmokeBotSystemReport *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemEventWarpIn::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemEventWarpIn *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemEventWarpOut::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemEventWarpOut *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeListPerStarType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeListPerStarType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExperienceTimers::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceTimers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSmokeBotStats::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSmokeBotStats *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSmokeBotSystemReport::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSmokeBotSystemReport *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemEventWarpIn::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemEventWarpIn *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAsteroidGeneratorAssignment::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGeneratorAssignment *
        ]
    ),
    "cGcAsteroidGeneratorRing::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGeneratorRing *
        ]
    ),
    "cGcAsteroidGeneratorSlab::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGeneratorSlab *
        ]
    ),
    "cGcAsteroidGeneratorSurround::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGeneratorSurround *
        ]
    ),
    "cGcBiomeListPerStarType::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeListPerStarType *
        ]
    ),
    "cGcExperienceTimers::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceTimers *
        ]
    ),
    "cGcSolarSystemData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemData *
        ]
    ),
    "cGcSolarSystemEventWarpIn::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemEventWarpIn *
        ]
    ),
    "cGcSolarSystemEventWarpOut::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemEventWarpOut *
        ]
    ),
    "cGcAsteroidGeneratorAssignment::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGeneratorAssignment *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAsteroidGeneratorRing::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGeneratorRing *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAsteroidGeneratorSlab::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGeneratorSlab *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAsteroidGeneratorSurround::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGeneratorSurround *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtmosphereList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtmosphereList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeListPerStarType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeListPerStarType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExperienceTimers::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceTimers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratedShipCounts::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedShipCounts *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSmokeBotPlanetReport::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSmokeBotPlanetReport *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSmokeBotReport::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSmokeBotReport *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSmokeBotStats::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSmokeBotStats *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSmokeBotSystemReport::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSmokeBotSystemReport *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarGenerationData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarGenerationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemEventWarpIn::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemEventWarpIn *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemEventWarpOut::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemEventWarpOut *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarSystemEventWarpPlayer::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemEventWarpPlayer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAsteroidGeneratorAssignment::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGeneratorAssignment *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAsteroidGeneratorAssignment *
        ]
    ),
    "cGcAsteroidGeneratorRing::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGeneratorRing *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAsteroidGeneratorRing *
        ]
    ),
    "cGcAsteroidGeneratorSlab::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGeneratorSlab *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAsteroidGeneratorSlab *
        ]
    ),
    "cGcAsteroidGeneratorSurround::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGeneratorSurround *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAsteroidGeneratorSurround *
        ]
    ),
    "cGcExperienceTimers::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceTimers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcExperienceTimers *
        ]
    ),
    "cGcSmokeBotPlanetReport::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSmokeBotPlanetReport *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSmokeBotPlanetReport *
        ]
    ),
    "cGcSmokeBotStats::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSmokeBotStats *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSmokeBotStats *
        ]
    ),
    "cGcSmokeBotSystemReport::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSmokeBotSystemReport *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSmokeBotSystemReport *
        ]
    ),
    "cGcSolarSystemData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSolarSystemData *
        ]
    ),
    "cGcSolarSystemEventWarpIn::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarSystemEventWarpIn *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSolarSystemEventWarpIn *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcButtonSpawn>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcButtonSpawn> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcButtonSpawn const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcButtonSpawn *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcButtonSpawnOffset>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcButtonSpawnOffset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcButtonSpawnOffset *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcSentinelTypes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSentinelTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSentinelTypes *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcButtonSpawn const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcButtonSpawn *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcButtonSpawnOffset const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcButtonSpawnOffset *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcButtonSpawnTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcButtonSpawnTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMechTargetSelectionWeightingSettings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMechTargetSelectionWeightingSettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRobotLaserData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRobotLaserData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSentinelEncounterOverride const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSentinelEncounterOverride *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSentinelMechWeaponData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSentinelMechWeaponData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSentinelMechWeaponMode const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSentinelMechWeaponMode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSentinelResource const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSentinelResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSentinelRobotComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSentinelRobotComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSentinelTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSentinelTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcButtonSpawn>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcButtonSpawn> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSentinelRobotComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSentinelRobotComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcButtonSpawn::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcButtonSpawnOffset::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcButtonSpawnTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMechTargetSelectionWeightingSettings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRobotLaserData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSentinelEncounterOverride::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSentinelMechWeaponData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSentinelMechWeaponMode::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSentinelResource::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSentinelRobotComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSentinelTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcButtonSpawn::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcButtonSpawnOffset::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcButtonSpawnTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSentinelEncounterOverride::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSentinelMechWeaponData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSentinelEncounterOverride::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelEncounterOverride *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSentinelMechWeaponData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeaponData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcButtonSpawn::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcButtonSpawn *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcButtonSpawnOffset::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcButtonSpawnOffset *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechTargetSelectionWeightingSettings::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechTargetSelectionWeightingSettings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRobotLaserData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotLaserData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelEncounterOverride::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelEncounterOverride *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelMechWeaponData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeaponData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelResource::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelResource *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelTypes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelTypes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcButtonSpawn::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcButtonSpawn *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcButtonSpawnOffset::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcButtonSpawnOffset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechTargetSelectionWeightingSettings::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechTargetSelectionWeightingSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRobotLaserData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotLaserData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelEncounterOverride::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelEncounterOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelMechWeaponData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeaponData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelResource::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelResource *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelTypes::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechTargetSelectionWeightingSettings::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechTargetSelectionWeightingSettings *
        ]
    ),
    "cGcRobotLaserData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotLaserData *
        ]
    ),
    "cGcSentinelEncounterOverride::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelEncounterOverride *
        ]
    ),
    "cGcSentinelMechWeaponData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeaponData *
        ]
    ),
    "cGcSentinelResource::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelResource *
        ]
    ),
    "cGcButtonSpawn::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcButtonSpawn *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcButtonSpawnOffset::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcButtonSpawnOffset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcButtonSpawnTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcButtonSpawnTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMechTargetSelectionWeightingSettings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechTargetSelectionWeightingSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRobotLaserData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotLaserData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelEncounterOverride::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelEncounterOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelMechWeaponData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeaponData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelMechWeaponMode::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeaponMode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelResource::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelResource *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelRobotComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelRobotComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcButtonSpawn::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcButtonSpawn *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcButtonSpawn *
        ]
    ),
    "cGcButtonSpawnOffset::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcButtonSpawnOffset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcButtonSpawnOffset *
        ]
    ),
    "cGcMechTargetSelectionWeightingSettings::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMechTargetSelectionWeightingSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMechTargetSelectionWeightingSettings *
        ]
    ),
    "cGcRobotLaserData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotLaserData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRobotLaserData *
        ]
    ),
    "cGcSentinelEncounterOverride::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelEncounterOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSentinelEncounterOverride *
        ]
    ),
    "cGcSentinelMechWeaponData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelMechWeaponData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSentinelMechWeaponData *
        ]
    ),
    "cGcSentinelResource::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelResource *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSentinelResource *
        ]
    ),
    "cGcSentinelTypes::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSentinelTypes *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDroneGun>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDroneGun> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcDroneControlData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneControlData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcProjectileImpactType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileImpactType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDroneGun const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDroneGun *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcDroneControlData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDroneControlData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDroneControlData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcProjectileImpactType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProjectileImpactType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProjectileImpactType *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDroneComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDroneComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDroneControlData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDroneControlData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDroneData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDroneData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDroneResource const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDroneResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDroneTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDroneTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDroneWeaponData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDroneWeaponData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProjectileImpactData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProjectileImpactData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProjectileImpactType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProjectileImpactType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProjectileLineData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProjectileLineData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcShootableComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShootableComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTracerData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTracerData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcDroneGun>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDroneGun> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcDroneComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDroneComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShootableComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcShootableComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDroneComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDroneControlData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDroneData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDroneResource::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDroneTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDroneWeaponData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProjectileImpactData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProjectileImpactType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProjectileLineData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShootableComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTracerData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDroneComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDroneData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProjectileLineData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcShootableComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDroneData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDroneComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneControlData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneControlData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneWeaponData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneWeaponData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProjectileImpactData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileImpactData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProjectileImpactType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileImpactType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProjectileLineData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileLineData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShootableComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTracerData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTracerData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneControlData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneControlData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneWeaponData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneWeaponData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProjectileImpactData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileImpactData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProjectileImpactType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileImpactType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProjectileLineData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileLineData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShootableComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTracerData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTracerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneData *
        ]
    ),
    "cGcDroneWeaponData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneWeaponData *
        ]
    ),
    "cGcSubstanceSecondary::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementProductionSlotData *
        ]
    ),
    "cGcProjectileLineData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileLineData *
        ]
    ),
    "cGcShootableComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponentData *
        ]
    ),
    "cGcTracerData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTracerData *
        ]
    ),
    "cGcDroneComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneControlData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneControlData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneResource::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneResource *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneWeaponData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneWeaponData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProjectileImpactData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileImpactData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProjectileImpactType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileImpactType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProjectileLineData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileLineData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShootableComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTracerData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTracerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDroneComponentData *
        ]
    ),
    "cGcDroneControlData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneControlData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDroneControlData *
        ]
    ),
    "cGcDroneData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDroneData *
        ]
    ),
    "cGcDroneWeaponData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneWeaponData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDroneWeaponData *
        ]
    ),
    "cGcProjectileImpactData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileImpactData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProjectileImpactData *
        ]
    ),
    "cGcProjectileImpactType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileImpactType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProjectileImpactType *
        ]
    ),
    "cGcProjectileLineData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileLineData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProjectileLineData *
        ]
    ),
    "cGcShootableComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShootableComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcShootableComponentData *
        ]
    ),
    "cGcTracerData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTracerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTracerData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCombatEffectDamageMultiplier>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCombatEffectDamageMultiplier> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDamageMultiplier>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDamageMultiplier> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcImpactCombatEffectData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcImpactCombatEffectData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcLaserBeamData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcLaserBeamData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcLootProbability>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcLootProbability> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcProjectileData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcProjectileData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcProjectileImpactData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcProjectileImpactData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcRewardMissionOverride>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRewardMissionOverride> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSubstanceAmount>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSubstanceAmount> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcDamageType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCombatEffectDamageMultiplier const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCombatEffectDamageMultiplier *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDamageMultiplier const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDamageMultiplier *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcImpactCombatEffectData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcImpactCombatEffectData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcLaserBeamData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcLaserBeamData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcLootProbability const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcLootProbability *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcProjectileData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcProjectileData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcProjectileImpactData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcProjectileImpactData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcRewardMissionOverride const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcRewardMissionOverride *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSubstanceAmount const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSubstanceAmount *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcDamageType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDamageType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDamageType *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBoidData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBoidData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCombatEffectDamageMultiplier const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCombatEffectDamageMultiplier *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDamageMultiplier const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDamageMultiplier *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDamageMultiplierLookup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDamageMultiplierLookup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDamageType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDamageType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDestructableComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDestructableComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcLaserBeamData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcLaserBeamData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissileComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissileComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProjectileData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProjectileData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProjectileDataTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProjectileDataTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLaserBeamData::cGcLaserBeamData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeamData *
            ctypes.c_ulonglong,  # cGcLaserBeamData *
        ]
    ),
    "cGcLaserBeamData::~cGcLaserBeamData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeamData *
        ]
    ),
    "cTkDynamicArray<cGcLaserBeamData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcLaserBeamData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcLootProbability>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcLootProbability> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcProjectileData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcProjectileData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSubstanceAmount>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSubstanceAmount> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcDestructableComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDestructableComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissileComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissileComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBoidData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCombatEffectDamageMultiplier::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDamageMultiplier::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDamageMultiplierLookup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDamageType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDestructableComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLaserBeamData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissileComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProjectileData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProjectileDataTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCombatEffectDamageMultiplier::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDamageMultiplierLookup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDestructableComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcLaserBeamData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProjectileData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProjectileDataTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcLootProbability>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcLootProbability> *
        ]
    ),
    "cGcPlayerStatsGroup::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingFamily *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcLaserBeamData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeamData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProjectileData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingFamily::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingFamily *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcLaserBeamData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeamData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProjectileData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProjectileDataTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileDataTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBoidData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBoidData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatEffectDamageMultiplier::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectDamageMultiplier *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDamageMultiplier::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageMultiplier *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDamageMultiplierLookup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageMultiplierLookup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDamageType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDestructableComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLaserBeamData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeamData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissileComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissileComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProjectileData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBoidData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBoidData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatEffectDamageMultiplier::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectDamageMultiplier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDamageMultiplierLookup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageMultiplierLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDamageType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDestructableComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLaserBeamData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeamData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProjectileData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBoidData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBoidData *
        ]
    ),
    "cGcFreighterDungeonChoice::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterDungeonChoice *
        ]
    ),
    "cGcDestructableComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponentData *
        ]
    ),
    "cGcLaserBeamData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeamData *
        ]
    ),
    "cGcMissileComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissileComponentData *
        ]
    ),
    "cGcProjectileData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileData *
        ]
    ),
    "cGcBoidData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBoidData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatEffectDamageMultiplier::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectDamageMultiplier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDamageMultiplier::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageMultiplier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDamageMultiplierLookup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageMultiplierLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDamageType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDestructableComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLaserBeamData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeamData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissileComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissileComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProjectileData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProjectileDataTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileDataTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBoidData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBoidData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBoidData *
        ]
    ),
    "cGcCombatEffectDamageMultiplier::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectDamageMultiplier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCombatEffectDamageMultiplier *
        ]
    ),
    "cGcDamageMultiplier::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageMultiplier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDamageMultiplier *
        ]
    ),
    "cGcDamageType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDamageType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDamageType *
        ]
    ),
    "cGcDestructableComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestructableComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDestructableComponentData *
        ]
    ),
    "cGcLaserBeamData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLaserBeamData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcLaserBeamData *
        ]
    ),
    "cGcProjectileData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProjectileData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerEmote>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerEmote> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerEmote const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerEmote *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerHazardData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerHazardData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerCharacterStateType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerCharacterStateType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerCommunicatorMessage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerCommunicatorMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerCommunicatorMessageWeighted const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerCommunicatorMessageWeighted *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerEmote const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerEmote *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerEmoteList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerEmoteList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerEmotePropData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerEmotePropData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerHazardData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerHazardData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerHazardTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerHazardTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerHazardType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerHazardType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerStickData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerStickData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerSurvivalBarType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerSurvivalBarType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProjectorOffsetData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProjectorOffsetData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScanData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScanData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScanEffectData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScanEffectData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcZoomData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcZoomData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerEmote::~cGcPlayerEmote": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEmote *
        ]
    ),
    "cTkDynamicArray<cGcPlayerEmote>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerEmote> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerCharacterStateType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerCommunicatorMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerCommunicatorMessageWeighted::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerEmote::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerEmoteList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerEmotePropData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerHazardData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerHazardTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerHazardType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerStickData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerSurvivalBarType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProjectorOffsetData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScanData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScanEffectData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcZoomData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerCommunicatorMessage::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerCommunicatorMessageWeighted::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerEmote::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerEmoteList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerEmotePropData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerHazardData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerHazardTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProjectorOffsetData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerEmoteList::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEmoteList *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerHazardTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazardTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProjectorOffsetData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectorOffsetData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerCharacterStateType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterStateType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCommunicatorMessage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicatorMessage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCommunicatorMessageWeighted::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicatorMessageWeighted *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerEmote::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEmote *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerEmotePropData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEmotePropData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerHazardData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazardData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerHazardType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazardType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerStickData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStickData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProjectorOffsetData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectorOffsetData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEffectData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEffectData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcZoomData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcZoomData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCharacterStateType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterStateType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCommunicatorMessage::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicatorMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCommunicatorMessageWeighted::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicatorMessageWeighted *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerEmote::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEmote *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerEmotePropData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEmotePropData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerHazardData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazardData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerHazardType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazardType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerStickData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStickData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProjectorOffsetData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectorOffsetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEffectData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEffectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcZoomData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcZoomData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCommunicatorMessage::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicatorMessage *
        ]
    ),
    "cGcPlayerCommunicatorMessageWeighted::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicatorMessageWeighted *
        ]
    ),
    "cGcPlayerEmote::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEmote *
        ]
    ),
    "cGcPlayerEmotePropData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEmotePropData *
        ]
    ),
    "cGcPlayerHazardData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazardData *
        ]
    ),
    "cGcPlayerHazardTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazardTable *
        ]
    ),
    "cGcPlayerStickData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStickData *
        ]
    ),
    "cGcProjectorOffsetData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectorOffsetData *
        ]
    ),
    "cGcScanData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanData *
        ]
    ),
    "cGcScanEffectData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEffectData *
        ]
    ),
    "cGcZoomData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcZoomData *
        ]
    ),
    "cGcPlayerCharacterStateType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterStateType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCommunicatorMessage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicatorMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCommunicatorMessageWeighted::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicatorMessageWeighted *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerEmote::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEmote *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerEmoteList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEmoteList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerEmotePropData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEmotePropData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerHazardData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazardData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerHazardTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazardTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerHazardType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazardType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerStickData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStickData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSurvivalBarType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSurvivalBarType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProjectorOffsetData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectorOffsetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEffectData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEffectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcZoomData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcZoomData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCharacterStateType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterStateType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerCharacterStateType *
        ]
    ),
    "cGcPlayerCommunicatorMessage::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCommunicatorMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerCommunicatorMessage *
        ]
    ),
    "cGcPlayerEmote::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEmote *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerEmote *
        ]
    ),
    "cGcPlayerEmotePropData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEmotePropData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerEmotePropData *
        ]
    ),
    "cGcPlayerHazardData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazardData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerHazardData *
        ]
    ),
    "cGcPlayerHazardType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazardType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerHazardType *
        ]
    ),
    "cGcPlayerStickData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStickData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerStickData *
        ]
    ),
    "cGcProjectorOffsetData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectorOffsetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProjectorOffsetData *
        ]
    ),
    "cGcScanData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScanData *
        ]
    ),
    "cGcScanEffectData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEffectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScanEffectData *
        ]
    ),
    "cGcZoomData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcZoomData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcZoomData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcAreaDamageData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAreaDamageData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDebrisData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDebrisData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcExplosionData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExplosionData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerCharacterAnimationOverrideData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerCharacterAnimationOverrideData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcPlayerCharacterIKOverrideData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterIKOverrideData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAreaDamageData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAreaDamageData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDebrisData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDebrisData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcExplosionData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcExplosionData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerCharacterAnimationOverrideData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerCharacterAnimationOverrideData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerCharacterStateData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerCharacterStateData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcHand>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHand *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcHand *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPhotoBuilding>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPhotoBuilding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPhotoBuilding *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPhotoCreature>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPhotoCreature *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPhotoCreature *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPhotoPlant>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPhotoPlant *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPhotoPlant *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlayerCharacterIKOverrideData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerCharacterIKOverrideData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerCharacterIKOverrideData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAreaDamageData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAreaDamageData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAreaDamageDataTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAreaDamageDataTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureDiscoveryThumbnailOverride const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureDiscoveryThumbnailOverride *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDebrisData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDebrisData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExplosionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExplosionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExplosionDataTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExplosionDataTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGrabbableData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGrabbableData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHand const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHand *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMovementDirection const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMovementDirection *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPhotoBuilding const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPhotoBuilding *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPhotoBuildings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPhotoBuildings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPhotoCreature const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPhotoCreature *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPhotoFauna const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPhotoFauna *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPhotoFlora const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPhotoFlora *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPhotoPlant const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPhotoPlant *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerCharacterAnimationOverrideData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerCharacterAnimationOverrideData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerCharacterIKOverrideData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerCharacterIKOverrideData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerCharacterIKStateData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerCharacterIKStateData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerCharacterStateData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerCharacterStateData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerCharacterStateTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerCharacterStateTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcReplacementEffectData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcReplacementEffectData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcPlayerCharacterStateData,18>::cTkFixedArray<cGcPlayerCharacterStateData,18>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcPlayerCharacterStateData,18> *
        ]
    ),
    "cTkDynamicArray<cGcAreaDamageData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAreaDamageData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcDebrisData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDebrisData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcExplosionData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExplosionData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPlayerCharacterAnimationOverrideData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerCharacterAnimationOverrideData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAreaDamageData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAreaDamageDataTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureDiscoveryThumbnailOverride::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDebrisData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExplosionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExplosionDataTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGrabbableData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHand::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMovementDirection::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPhotoBuilding::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPhotoBuildings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPhotoCreature::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPhotoFauna::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPhotoFlora::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPhotoPlant::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerCharacterAnimationOverrideData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerCharacterIKOverrideData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerCharacterIKStateData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerCharacterStateData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerCharacterStateTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcReplacementEffectData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAreaDamageDataTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDebrisData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExplosionData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExplosionDataTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerCharacterIKStateData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcDebrisData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDebrisData> *
        ]
    ),
    "cGcExplosionData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExplosionData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerCharacterIKStateData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterIKStateData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExplosionData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExplosionData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExplosionDataTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExplosionDataTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerCharacterIKStateData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterIKStateData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAreaDamageData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAreaDamageData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureDiscoveryThumbnailOverride::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDiscoveryThumbnailOverride *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebrisData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebrisData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExplosionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExplosionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGrabbableData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabbableData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHand::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHand *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMovementDirection::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMovementDirection *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoBuilding::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoBuilding *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoBuildings::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoBuildings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoCreature::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoCreature *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoFauna::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoFauna *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoFlora::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoFlora *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoPlant::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoPlant *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCharacterAnimationOverrideData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterAnimationOverrideData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCharacterIKOverrideData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterIKOverrideData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCharacterIKStateData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterIKStateData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCharacterStateData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterStateData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcReplacementEffectData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcReplacementEffectData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHand::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHand *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcMovementDirection::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMovementDirection *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcAreaDamageData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAreaDamageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureDiscoveryThumbnailOverride::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDiscoveryThumbnailOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebrisData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebrisData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExplosionData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExplosionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGrabbableData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabbableData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHand::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHand *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMovementDirection::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMovementDirection *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoBuilding::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoBuilding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoBuildings::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoBuildings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoCreature::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoCreature *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoFauna::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoFauna *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoFlora::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoFlora *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoPlant::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoPlant *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCharacterIKOverrideData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterIKOverrideData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCharacterIKStateData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterIKStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCharacterStateData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcReplacementEffectData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcReplacementEffectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExplosionDataTable::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExplosionDataTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureDiscoveryThumbnailOverride::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDiscoveryThumbnailOverride *
        ]
    ),
    "cGcExplosionData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExplosionData *
        ]
    ),
    "cGcGrabbableData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabbableData *
        ]
    ),
    "cGcPhotoFlora::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoFlora *
        ]
    ),
    "cGcPhotoFauna::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoFauna *
        ]
    ),
    "cGcPlayerCharacterIKOverrideData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterIKOverrideData *
        ]
    ),
    "cGcPlayerCharacterIKStateData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterIKStateData *
        ]
    ),
    "cGcPlayerCharacterStateTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterStateTable *
        ]
    ),
    "cGcHand::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHand *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcMovementDirection::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMovementDirection *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcAreaDamageData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAreaDamageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAreaDamageDataTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAreaDamageDataTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureDiscoveryThumbnailOverride::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDiscoveryThumbnailOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebrisData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebrisData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExplosionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExplosionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExplosionDataTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExplosionDataTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGrabbableData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabbableData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHand::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHand *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMovementDirection::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMovementDirection *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoBuilding::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoBuilding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoBuildings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoBuildings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoCreature::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoCreature *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoFauna::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoFauna *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoFlora::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoFlora *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhotoPlant::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoPlant *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCharacterAnimationOverrideData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterAnimationOverrideData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCharacterIKOverrideData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterIKOverrideData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCharacterIKStateData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterIKStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCharacterStateData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCharacterStateTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterStateTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcReplacementEffectData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcReplacementEffectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAreaDamageData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAreaDamageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAreaDamageData *
        ]
    ),
    "cGcDebrisData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebrisData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDebrisData *
        ]
    ),
    "cGcExplosionData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExplosionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcExplosionData *
        ]
    ),
    "cGcGrabbableData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabbableData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGrabbableData *
        ]
    ),
    "cGcHand::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHand *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcHand *
        ]
    ),
    "cGcMovementDirection::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMovementDirection *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMovementDirection *
        ]
    ),
    "cGcPhotoBuilding::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoBuilding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPhotoBuilding *
        ]
    ),
    "cGcPhotoCreature::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoCreature *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPhotoCreature *
        ]
    ),
    "cGcPhotoPlant::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoPlant *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPhotoPlant *
        ]
    ),
    "cGcPlayerCharacterAnimationOverrideData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterAnimationOverrideData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerCharacterAnimationOverrideData *
        ]
    ),
    "cGcPlayerCharacterIKOverrideData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterIKOverrideData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerCharacterIKOverrideData *
        ]
    ),
    "cGcPlayerCharacterIKStateData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterIKStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerCharacterIKStateData *
        ]
    ),
    "cGcPlayerCharacterStateData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerCharacterStateData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNPCPlacementInfo>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCPlacementInfo> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNPCProbabilityReactionData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCProbabilityReactionData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNPCProbabilityWordReactionData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCProbabilityWordReactionData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNPCReactionEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCReactionEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNPCSettlementBehaviourAreaPropertyWeightEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCSettlementBehaviourAreaPropertyWeightEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNPCSettlementBehaviourBuildingClassCapacityEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCSettlementBehaviourBuildingClassCapacityEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNPCSettlementBehaviourBuildingClassWeightEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCSettlementBehaviourBuildingClassWeightEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNPCSettlementBehaviourObjectTypeWeightEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCSettlementBehaviourObjectTypeWeightEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcUniqueNPCSpawnData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUniqueNPCSpawnData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcBuildingClassification>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingClassification *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCPlacementInfo const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCPlacementInfo *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCProbabilityReactionData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCProbabilityReactionData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCProbabilityWordReactionData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCProbabilityWordReactionData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCReactionEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCReactionEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCSettlementBehaviourAreaPropertyWeightEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourAreaPropertyWeightEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCSettlementBehaviourBuildingClassCapacityEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourBuildingClassCapacityEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCSettlementBehaviourBuildingClassWeightEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourBuildingClassWeightEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCSettlementBehaviourEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCSettlementBehaviourObjectTypeWeightEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourObjectTypeWeightEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCWordReactionCategory const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCWordReactionCategory *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCWordReactionList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCWordReactionList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcUniqueNPCSpawnData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcUniqueNPCSpawnData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcBuildingClassification>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingClassification *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBuildingClassification *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcNPCReactionEntry>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCReactionEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCReactionEntry *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcNPCSettlementBehaviourAreaProperty>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourAreaProperty *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourAreaProperty *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcNPCSettlementBehaviourAreaPropertyWeightEntry>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourAreaPropertyWeightEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourAreaPropertyWeightEntry *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcNPCSettlementBehaviourBuildingClassWeightEntry>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourBuildingClassWeightEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourBuildingClassWeightEntry *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcNPCSettlementBehaviourEntry>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourEntry *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcNPCWordReactionList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCWordReactionList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCWordReactionList *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCRaceProbabilityModifierData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCRaceProbabilityModifierData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCReactionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCReactionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCReactionEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCReactionEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCSettlementBehaviourAreaProperty const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourAreaProperty *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCSettlementBehaviourAreaPropertyWeightEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourAreaPropertyWeightEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCSettlementBehaviourBuildingClassCapacityEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourBuildingClassCapacityEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCSettlementBehaviourBuildingClassWeightEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourBuildingClassWeightEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCSettlementBehaviourData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCSettlementBehaviourEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCSettlementBehaviourObjectTypeWeightEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourObjectTypeWeightEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCSettlementBehaviourState const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourState *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCSpawnTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCSpawnTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCWordReactionCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCWordReactionCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCWordReactionList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCWordReactionList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCWordReactionTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCWordReactionTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcNPCWordReactionCategory,8>::cTkFixedArray<cGcNPCWordReactionCategory,8>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcNPCWordReactionCategory,8> *
        ]
    ),
    "cTkFixedArray<cGcNPCSettlementBehaviourEntry,5>::~cTkFixedArray<cGcNPCSettlementBehaviourEntry,5>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcNPCSettlementBehaviourEntry,5> *
        ]
    ),
    "cTkFixedArray<cGcNPCWordReactionCategory,8>::~cTkFixedArray<cGcNPCWordReactionCategory,8>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcNPCWordReactionCategory,8> *
        ]
    ),
    "cGcNPCSettlementBehaviourEntry::~cGcNPCSettlementBehaviourEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourEntry *
        ]
    ),
    "cGcNPCWordReactionCategory::~cGcNPCWordReactionCategory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWordReactionCategory *
        ]
    ),
    "cTkDynamicArray<cGcNPCPlacementInfo>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCPlacementInfo> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcDamageMultiplierLookup>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDamageMultiplierLookup> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcNPCProbabilityWordReactionData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCProbabilityWordReactionData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcNPCReactionEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCReactionEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcNPCSettlementBehaviourBuildingClassWeightEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCSettlementBehaviourBuildingClassWeightEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcNPCSettlementBehaviourBuildingClassCapacityEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCSettlementBehaviourBuildingClassCapacityEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcNPCSettlementBehaviourObjectTypeWeightEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCRaceProbabilityModifierData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcUniqueNPCSpawnData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUniqueNPCSpawnData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNPCRaceProbabilityModifierData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCReactionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCReactionEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCSettlementBehaviourAreaProperty::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCSettlementBehaviourAreaPropertyWeightEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCSettlementBehaviourBuildingClassCapacityEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCSettlementBehaviourBuildingClassWeightEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCSettlementBehaviourData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCSettlementBehaviourEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCSettlementBehaviourObjectTypeWeightEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCSettlementBehaviourState::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCSpawnTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCWordReactionCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCWordReactionList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCWordReactionTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCRaceProbabilityModifierData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCReactionData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCReactionEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCSettlementBehaviourBuildingClassCapacityEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCSettlementBehaviourBuildingClassWeightEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCSettlementBehaviourData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCSettlementBehaviourEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCSettlementBehaviourObjectTypeWeightEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCSpawnTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCWordReactionCategory::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCWordReactionList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCWordReactionTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcNPCReactionEntry>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCReactionEntry> *
        ]
    ),
    "cGcNPCSettlementBehaviourData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCSettlementBehaviourEntry::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourEntry *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCWordReactionCategory::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWordReactionCategory *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCReactionData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCReactionData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCSettlementBehaviourData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCSettlementBehaviourEntry::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourEntry *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCSpawnTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSpawnTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCWordReactionCategory::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWordReactionCategory *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCWordReactionTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWordReactionTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCRaceProbabilityModifierData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCRaceProbabilityModifierData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCReactionEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCReactionEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourAreaProperty::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourAreaProperty *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourAreaPropertyWeightEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourAreaPropertyWeightEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourBuildingClassCapacityEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourBuildingClassCapacityEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourBuildingClassWeightEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourBuildingClassWeightEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourObjectTypeWeightEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourObjectTypeWeightEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSpawnTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSpawnTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCWordReactionCategory::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWordReactionCategory *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCWordReactionList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWordReactionList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCRaceProbabilityModifierData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCRaceProbabilityModifierData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourAreaProperty::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourAreaProperty *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourAreaPropertyWeightEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourAreaPropertyWeightEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourBuildingClassCapacityEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourBuildingClassCapacityEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourObjectTypeWeightEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourObjectTypeWeightEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCWordReactionCategory::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWordReactionCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCRaceProbabilityModifierData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCRaceProbabilityModifierData *
        ]
    ),
    "cGcNPCSpawnTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSpawnTable *
        ]
    ),
    "cGcNPCRaceProbabilityModifierData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCRaceProbabilityModifierData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCReactionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCReactionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCReactionEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCReactionEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourAreaProperty::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourAreaProperty *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourAreaPropertyWeightEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourAreaPropertyWeightEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourBuildingClassCapacityEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourBuildingClassCapacityEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourBuildingClassWeightEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourBuildingClassWeightEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourObjectTypeWeightEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourObjectTypeWeightEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSettlementBehaviourState::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSpawnTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSpawnTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCWordReactionCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWordReactionCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCWordReactionList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWordReactionList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCWordReactionTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWordReactionTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCRaceProbabilityModifierData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCRaceProbabilityModifierData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCRaceProbabilityModifierData *
        ]
    ),
    "cGcNPCReactionEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCReactionEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCReactionEntry *
        ]
    ),
    "cGcNPCSettlementBehaviourAreaProperty::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourAreaProperty *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourAreaProperty *
        ]
    ),
    "cGcNPCSettlementBehaviourAreaPropertyWeightEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourAreaPropertyWeightEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourAreaPropertyWeightEntry *
        ]
    ),
    "cGcNPCSettlementBehaviourBuildingClassCapacityEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourBuildingClassCapacityEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourBuildingClassCapacityEntry *
        ]
    ),
    "cGcNPCSettlementBehaviourBuildingClassWeightEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourBuildingClassWeightEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourBuildingClassWeightEntry *
        ]
    ),
    "cGcNPCSettlementBehaviourEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourEntry *
        ]
    ),
    "cGcNPCSettlementBehaviourObjectTypeWeightEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSettlementBehaviourObjectTypeWeightEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCSettlementBehaviourObjectTypeWeightEntry *
        ]
    ),
    "cGcNPCWordReactionCategory::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWordReactionCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCWordReactionCategory *
        ]
    ),
    "cTkMetaDataXML::ReadArrayXML<cTkColour>": {
        "cTkDynamicArray<cTkColour> *, XMLNode *, const char *, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDynamicArray<cTkColour> *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cTkColour *, int, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkColour *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "cTkMetaDataXML::ReadClassArrayXML<cGcAlienRace>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAlienRace> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNPCColourGroup>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCColourGroup> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNPCInteractionData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCInteractionData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNPCInteractiveObjectStateTransition>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCInteractiveObjectStateTransition> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNPCProbabilityAnimationData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCProbabilityAnimationData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNPCRaceProbabilityModifierData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCRaceProbabilityModifierData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcNPCAnimationSetData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCAnimationSetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAlienRace const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAlienRace *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCAnimationList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCAnimationList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCColourGroup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCColourGroup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCInteractionData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCInteractionData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCInteractiveObjectStateTransition const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCInteractiveObjectStateTransition *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCProbabilityAnimationData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCProbabilityAnimationData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCPropInfo const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCPropInfo *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCRaceProbabilityModifierData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCRaceProbabilityModifierData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcAlienRace>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAlienRace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAlienRace *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcNPCAnimationSetData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCAnimationSetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCAnimationSetData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCAnimationList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCAnimationList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCAnimationSetData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCAnimationSetData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCAnimationsData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCAnimationsData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCColourGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCColourGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCColourTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCColourTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCDebugSpawnData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCDebugSpawnData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCInteractionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCInteractionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCInteractionsDataTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCInteractionsDataTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCInteractiveObjectState const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCInteractiveObjectState *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCInteractiveObjectStateTransition const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCInteractiveObjectStateTransition *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCInteractiveObjectType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCInteractiveObjectType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCProbabilityAnimationData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCProbabilityAnimationData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCProbabilityReactionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCProbabilityReactionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCProbabilityWordReactionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCProbabilityWordReactionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCPropInfo const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCPropInfo *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCPropTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCPropTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProductToCollect const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProductToCollect *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScanEventGPSHint const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScanEventGPSHint *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcNPCPropInfo,13>::cTkFixedArray<cGcNPCPropInfo,13>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcNPCPropInfo,13> *
        ]
    ),
    "cGcNPCAnimationSetData::~cGcNPCAnimationSetData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCAnimationSetData *
        ]
    ),
    "cTkDynamicArray<cGcAlienRace>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAlienRace> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcNPCColourGroup>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCColourGroup> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcNPCInteractionData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCInteractionData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcNPCInteractiveObjectStateTransition>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCInteractiveObjectStateTransition> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcNPCProbabilityAnimationData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCProbabilityAnimationData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcMissionType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCAnimationList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCAnimationSetData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCAnimationsData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCColourGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCColourTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCDebugSpawnData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCInteractionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCInteractionsDataTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCInteractiveObjectState::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCInteractiveObjectStateTransition::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCInteractiveObjectType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCProbabilityAnimationData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCProbabilityReactionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCProbabilityWordReactionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCPropInfo::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCPropTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProductToCollect::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScanEventGPSHint::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCAnimationList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCAnimationSetData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCAnimationsData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCColourGroup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCColourTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCDebugSpawnData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCInteractionData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCInteractionsDataTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCInteractiveObjectState::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCInteractiveObjectStateTransition::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCProbabilityAnimationData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCProbabilityReactionData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCPropInfo::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCPropTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcNPCInteractionData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCInteractionData> *
        ]
    ),
    "cGcNPCAnimationList::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCAnimationList *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCAnimationSetData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCAnimationSetData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCDebugSpawnData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCDebugSpawnData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCInteractiveObjectState::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectState *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCProbabilityAnimationData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCProbabilityAnimationData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCProbabilityReactionData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCProbabilityReactionData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCAnimationList::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCAnimationList *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCAnimationSetData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCAnimationSetData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCAnimationsData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCAnimationsData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCColourTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCColourTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCDebugSpawnData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCDebugSpawnData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCInteractionsDataTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractionsDataTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCInteractiveObjectState::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectState *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCProbabilityAnimationData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCProbabilityAnimationData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCProbabilityReactionData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCProbabilityReactionData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCPropTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPropTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCAnimationList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCAnimationList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCAnimationSetData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCAnimationSetData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCColourGroup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCColourGroup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCDebugSpawnData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCDebugSpawnData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCInteractionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCInteractiveObjectState::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectState *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCInteractiveObjectStateTransition::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectStateTransition *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCInteractiveObjectType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCProbabilityAnimationData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCProbabilityAnimationData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCProbabilityReactionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCProbabilityReactionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCProbabilityWordReactionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCProbabilityWordReactionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCPropInfo::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPropInfo *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProductToCollect::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductToCollect *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventGPSHint::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventGPSHint *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCAnimationSetData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCAnimationSetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCColourGroup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCColourGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCDebugSpawnData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCDebugSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCInteractionData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCInteractiveObjectState::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCInteractiveObjectStateTransition::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectStateTransition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCInteractiveObjectType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCProbabilityAnimationData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCProbabilityAnimationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCProbabilityReactionData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCProbabilityReactionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCProbabilityWordReactionData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCProbabilityWordReactionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCPropInfo::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPropInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProductToCollect::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductToCollect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventGPSHint::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventGPSHint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCAnimationsData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCAnimationsData *
        ]
    ),
    "cGcNPCDebugSpawnData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCDebugSpawnData *
        ]
    ),
    "cGcNPCInteractiveObjectState::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectState *
        ]
    ),
    "cGcNPCInteractiveObjectType::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienRace *
        ]
    ),
    "cGcNPCProbabilityWordReactionData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCProbabilityWordReactionData *
        ]
    ),
    "cGcNPCPropInfo::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPropInfo *
        ]
    ),
    "cGcNPCPropTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPropTable *
        ]
    ),
    "cGcExpeditionPaymentToken::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemAmountCostPair *
        ]
    ),
    "cGcMissionType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCAnimationList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCAnimationList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCAnimationSetData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCAnimationSetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCAnimationsData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCAnimationsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCColourGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCColourGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCColourTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCColourTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCDebugSpawnData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCDebugSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCInteractionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCInteractionsDataTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractionsDataTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCInteractiveObjectState::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCInteractiveObjectStateTransition::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectStateTransition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCInteractiveObjectType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCProbabilityAnimationData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCProbabilityAnimationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCProbabilityReactionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCProbabilityReactionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCProbabilityWordReactionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCProbabilityWordReactionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCPropInfo::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPropInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCPropTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPropTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProductToCollect::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductToCollect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventGPSHint::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventGPSHint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionType *
        ]
    ),
    "cGcNPCAnimationSetData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCAnimationSetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCColourGroup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCColourGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCColourGroup *
        ]
    ),
    "cGcNPCDebugSpawnData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCDebugSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCDebugSpawnData *
        ]
    ),
    "cGcNPCInteractionData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCInteractionData *
        ]
    ),
    "cGcNPCInteractiveObjectState::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCInteractiveObjectState *
        ]
    ),
    "cGcNPCInteractiveObjectStateTransition::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectStateTransition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCInteractiveObjectStateTransition *
        ]
    ),
    "cGcNPCInteractiveObjectType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCInteractiveObjectType *
        ]
    ),
    "cGcNPCProbabilityAnimationData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCProbabilityAnimationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCProbabilityAnimationData *
        ]
    ),
    "cGcNPCPropInfo::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPropInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCPropInfo *
        ]
    ),
    "cGcScanEventGPSHint::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventGPSHint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScanEventGPSHint *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPersistentBaseTypes>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPersistentBaseTypes> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcStatsTypes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPersistentBaseTypes const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPersistentBaseTypes *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcStatsTypes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatsTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatsTypes *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingPartSearchType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingPartSearchType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcConstructionPart const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcConstructionPart *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcJudgementMessageOptions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcJudgementMessageOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionTest const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionTest *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionDifficulty const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionDifficulty *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionFaction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionFaction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionGalacticFeature const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionGalacticFeature *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionGalacticPoint const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionGalacticPoint *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionPageHint const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionPageHint *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForRefuel const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceGatherForRefuel *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForRepair const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForRepair *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForScanEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForScanEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForSettlementActivity const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForSettlementActivity *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForShips const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForShips *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForStat const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForStat *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForStatMilestone const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForStatMilestone *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForStatSeasonal const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForStatSeasonal *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceExplorationLogSpecial const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceFinishSummonAnomaly *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForWarps const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForWarps *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitRealTime const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitRealTime *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitRealTimeCombat const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitRealTimeCombat *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForRefuel::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForRefuel *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForRepair::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForRepair *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForScanEvent::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForScanEvent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForSettlementActivity::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForSettlementActivity *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForShips::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForShips *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForStat::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForStat *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForStatMilestone::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForStatMilestone *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForStatSeasonal::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForStatSeasonal *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForSuitUpgrade::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForSuitUpgrade *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForWarps::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForWarps *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitRealTime::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitRealTime *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitRealTimeCombat::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitRealTimeCombat *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingPartSearchType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcConstructionPart::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcJudgementMessageOptions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionTest::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionDifficulty::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionFaction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionGalacticFeature::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionGalacticPoint::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionPageHint::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForRefuel::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForRepair::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForScanEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForSettlementActivity::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForShips::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForStat::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForStatMilestone::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForStatSeasonal::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForSuitUpgrade::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForWarps::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitRealTime::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitRealTimeCombat::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingPartSearchType::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceWaitForRefuel::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceWaitForRepair::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceWaitForScanEvent::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceWaitForShips::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBuildingPartSearchType::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingPartSearchType *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBuildingPartSearchType::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingPartSearchType *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBuildingPartSearchType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingPartSearchType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcConstructionPart::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcConstructionPart *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJudgementMessageOptions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcJudgementMessageOptions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionCategory::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionCategory *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionTest::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionTest *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionDifficulty::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionDifficulty *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionFaction::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionFaction *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionGalacticFeature::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionGalacticFeature *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionGalacticPoint::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionGalacticPoint *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionPageHint::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionPageHint *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForRefuel::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForRefuel *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForScanEvent::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForScanEvent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForSettlementActivity::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForSettlementActivity *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForShips::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForShips *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForStat::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForStat *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForStatMilestone::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForStatMilestone *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForStatSeasonal::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForStatSeasonal *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitRealTime::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitRealTime *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitRealTimeCombat::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitRealTimeCombat *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingPartSearchType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingPartSearchType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcConstructionPart::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcConstructionPart *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJudgementMessageOptions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJudgementMessageOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionCategory::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionTest::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionTest *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionDifficulty::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionDifficulty *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionFaction::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionFaction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionGalacticFeature::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionGalacticFeature *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionGalacticPoint::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionGalacticPoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionPageHint::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionPageHint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingPartSearchType::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingPartSearchType *
        ]
    ),
    "cGcJudgementMessageOptions::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJudgementMessageOptions *
        ]
    ),
    "cGcMissionFaction::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionFaction *
        ]
    ),
    "cGcMissionSequenceWaitForRefuel::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGatherForRefuel *
        ]
    ),
    "cGcMissionSequenceSignalGalacticPoint::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGoToGalacticPoint *
        ]
    ),
    "cGcMissionSequenceWaitForScanEvent::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForScanEvent *
        ]
    ),
    "cGcMissionSequenceWaitForSettlementActivity::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForSettlementActivity *
        ]
    ),
    "cGcMissionSequenceWaitForShips::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForShips *
        ]
    ),
    "cGcMissionSequenceWaitForStat::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForStat *
        ]
    ),
    "cGcMissionSequenceWaitForStatMilestone::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForStatMilestone *
        ]
    ),
    "cGcMissionSequenceWaitForStatSeasonal::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForStatSeasonal *
        ]
    ),
    "cGcMissionSequenceWaitForWarps::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForWarps *
        ]
    ),
    "cGcMissionSequenceWaitRealTime::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitRealTime *
        ]
    ),
    "cGcMissionSequenceWaitRealTimeCombat::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitRealTimeCombat *
        ]
    ),
    "cGcBuildingPartSearchType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingPartSearchType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcConstructionPart::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcConstructionPart *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJudgementMessageOptions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJudgementMessageOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionTest::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionTest *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionDifficulty::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionDifficulty *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionFaction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionFaction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionGalacticFeature::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionGalacticFeature *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionGalacticPoint::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionGalacticPoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionPageHint::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionPageHint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceGatherForRefuel::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGatherForRefuel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForRepair::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForRepair *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForScanEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForScanEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForSettlementActivity::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForSettlementActivity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForShips::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForShips *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForStat::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForStat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForStatMilestone::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForStatMilestone *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForStatSeasonal::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForStatSeasonal *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceExplorationLogSpecial::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceFinishSummonAnomaly *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForWarps::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForWarps *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitRealTime::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitRealTime *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitRealTimeCombat::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitRealTimeCombat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingPartSearchType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingPartSearchType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBuildingPartSearchType *
        ]
    ),
    "cGcJudgementMessageOptions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJudgementMessageOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcJudgementMessageOptions *
        ]
    ),
    "cGcMissionCategory::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionCategory *
        ]
    ),
    "cGcMissionConditionTest::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionTest *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionTest *
        ]
    ),
    "cGcMissionDifficulty::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionDifficulty *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionDifficulty *
        ]
    ),
    "cGcMissionFaction::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionFaction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionFaction *
        ]
    ),
    "cGcMissionGalacticFeature::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionGalacticFeature *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionGalacticFeature *
        ]
    ),
    "cGcMissionGalacticPoint::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionGalacticPoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionGalacticPoint *
        ]
    ),
    "cGcMissionPageHint::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionPageHint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionPageHint *
        ]
    ),
    "cGcMissionSequenceWaitForScanEvent::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForScanEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForScanEvent *
        ]
    ),
    "cGcMissionSequenceWaitForSettlementActivity::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForSettlementActivity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForSettlementActivity *
        ]
    ),
    "cGcMissionSequenceWaitForShips::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForShips *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForShips *
        ]
    ),
    "cGcMissionSequenceWaitForStat::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForStat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForStat *
        ]
    ),
    "cGcMissionSequenceWaitForStatMilestone::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForStatMilestone *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForStatMilestone *
        ]
    ),
    "cGcMissionSequenceWaitForStatSeasonal::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForStatSeasonal *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForStatSeasonal *
        ]
    ),
    "cGcMissionSequenceWaitRealTime::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitRealTime *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitRealTime *
        ]
    ),
    "cGcMissionSequenceWaitRealTimeCombat::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitRealTimeCombat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitRealTimeCombat *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPhotoBuildings>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPhotoBuildings> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPhotoFauna>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPhotoFauna> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPhotoFlora>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPhotoFlora> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPhotoBuildings const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPhotoBuildings *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPhotoFauna const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPhotoFauna *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPhotoFlora const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPhotoFlora *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceStartScanEventSpecific const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceStartScanEventSpecific *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceStartSummonAnomaly const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceStartSummonAnomaly *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForCompletionMessage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceGetInShip *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceSuppressMarkers const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceSuppressMarkers *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceTeleport const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceTeleport *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceVehicleScan const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceVehicleScan *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceVisitPlanets const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceVisitPlanets *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWait const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWait *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForAbandFreighterDoorOpen const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForAbandFreighterDoorOpen *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForBuild const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForBuild *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForConditions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForConditions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForDepots const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForDepots *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForFactionStanding const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceDoMissionsForFaction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForFreighterPods const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForFreighterPods *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForFriendlyDroneScanEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForFriendlyDroneScanEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForMessage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForPhoto const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForPhoto *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceWaitForPortalWarp const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForPortalWarp *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcPhotoBuildings>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPhotoFlora> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPhotoFauna>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPhotoFauna> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcMissionSequenceStartScanEventSpecific::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceStartScanEventSpecific *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceStartSummonAnomaly::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceStartSummonAnomaly *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceStop::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceStop *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceSummonNexus::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceSummonNexus *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceSuppressMarkers::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceSuppressMarkers *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceTeleport::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceTeleport *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceVehicleScan::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceVehicleScan *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceVisitPlanets::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceVisitPlanets *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWait::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWait *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForAbandFreighterDoorOpen::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForAbandFreighterDoorOpen *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForBuild::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForBuild *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForCompletionMessage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForCompletionMessage *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForConditions::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForConditions *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForDepots::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForDepots *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForFactionStanding::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForFactionStanding *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForFreighterPods::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForFreighterPods *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForFriendlyDroneScanEvent::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForFriendlyDroneScanEvent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForMessage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForMessage *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForPhoto::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForPhoto *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForPortalWarp::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceWaitForPortalWarp *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceStartScanEventSpecific::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceStartSummonAnomaly::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceStop::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceSummonNexus::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceSuppressMarkers::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceTeleport::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceVehicleScan::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceVisitPlanets::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWait::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForAbandFreighterDoorOpen::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForBuild::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForCompletionMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForConditions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForDepots::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForFactionStanding::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForFreighterPods::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForFriendlyDroneScanEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForPhoto::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceWaitForPortalWarp::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceStartScanEventSpecific::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceStartSummonAnomaly::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceTeleport::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceVisitPlanets::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceWaitForConditions::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceWaitForFactionStanding::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceWaitForPhoto::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceWaitForPhoto::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForPhoto *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceStartScanEventSpecific::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceStartScanEventSpecific *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceTeleport::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceTeleport *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceVehicleScan::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceVehicleScan *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceVisitPlanets::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceVisitPlanets *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWait::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWait *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForAbandFreighterDoorOpen::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForAbandFreighterDoorOpen *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForBuild::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForBuild *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForConditions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForConditions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForDepots::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForDepots *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForFactionStanding::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForFactionStanding *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForFreighterPods::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForFreighterPods *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForFriendlyDroneScanEvent::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForFriendlyDroneScanEvent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForMessage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForMessage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForPhoto::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForPhoto *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForPortalWarp::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForPortalWarp *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForConditions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForConditions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceStartScanEventSpecific::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceStartScanEventSpecific *
        ]
    ),
    "cGcMissionSequenceStartSummonAnomaly::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceStartSummonAnomaly *
        ]
    ),
    "cGcMissionSequenceTeleport::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceTeleport *
        ]
    ),
    "cGcMissionSequenceVehicleScan::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceVehicleScan *
        ]
    ),
    "cGcMissionSequenceVisitPlanets::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceVisitPlanets *
        ]
    ),
    "cGcMissionSequenceWait::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWait *
        ]
    ),
    "cGcMissionSequenceWaitForAbandFreighterDoorOpen::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForAbandFreighterDoorOpen *
        ]
    ),
    "cGcMissionSequenceGetUnitsToBuyItem::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGetUnitsToBuyItem *
        ]
    ),
    "cGcMissionSequenceWaitForConditions::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForConditions *
        ]
    ),
    "cGcMissionSequenceWaitForDepots::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForDepots *
        ]
    ),
    "cGcMissionSequenceWaitForFactionStanding::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDoMissionsForFaction *
        ]
    ),
    "cGcMissionSequenceWaitForFreighterPods::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForFreighterPods *
        ]
    ),
    "cGcMissionSequenceWaitForFriendlyDroneScanEvent::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForFriendlyDroneScanEvent *
        ]
    ),
    "cGcMissionSequenceWaitForPhoto::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForPhoto *
        ]
    ),
    "cGcMissionSequenceWaitForPortalWarp::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForPortalWarp *
        ]
    ),
    "cGcMissionSequenceStartScanEventSpecific::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceStartScanEventSpecific *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceStartSummonAnomaly::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceStartSummonAnomaly *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForCompletionMessage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGetInShip *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceSuppressMarkers::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceSuppressMarkers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceTeleport::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceTeleport *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceVehicleScan::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceVehicleScan *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceVisitPlanets::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceVisitPlanets *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWait::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWait *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForAbandFreighterDoorOpen::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForAbandFreighterDoorOpen *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForBuild::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForBuild *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForConditions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForConditions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForDepots::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForDepots *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceDoMissionsForFaction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDoMissionsForFaction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForFreighterPods::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForFreighterPods *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForFriendlyDroneScanEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForFriendlyDroneScanEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForMessage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForPhoto::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForPhoto *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceWaitForPortalWarp::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForPortalWarp *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceStartScanEventSpecific::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceStartScanEventSpecific *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceStartScanEventSpecific *
        ]
    ),
    "cGcMissionSequenceStartSummonAnomaly::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceStartSummonAnomaly *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceStartSummonAnomaly *
        ]
    ),
    "cGcMissionSequenceTeleport::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceTeleport *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceTeleport *
        ]
    ),
    "cGcMissionSequenceVisitPlanets::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceVisitPlanets *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceVisitPlanets *
        ]
    ),
    "cGcMissionSequenceWaitForConditions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForConditions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForConditions *
        ]
    ),
    "cGcMissionSequenceWaitForDepots::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForDepots *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForDepots *
        ]
    ),
    "cGcMissionSequenceDoMissionsForFaction::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDoMissionsForFaction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceDoMissionsForFaction *
        ]
    ),
    "cGcMissionSequenceWaitForFreighterPods::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForFreighterPods *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForFreighterPods *
        ]
    ),
    "cGcMissionSequenceWaitForFriendlyDroneScanEvent::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForFriendlyDroneScanEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForFriendlyDroneScanEvent *
        ]
    ),
    "cGcMissionSequenceWaitForPhoto::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceWaitForPhoto *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceWaitForPhoto *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceKill const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceKill *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceKillEncounter const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceKillEncounter *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceLearnWords const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceLearnWords *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceLeaveNexusMP const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceLeaveNexusMP *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceModifyStat const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceModifyStat *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequencePirates const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequencePirates *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceProductAmountNeeded const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceProductAmountNeeded *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceRepairTech const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceRepairTech *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceReward const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceReward *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceScan const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceScan *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSendChatMessage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSendChatMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceSetCurrentMission const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceSetCurrentMission *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceSetGlobalMission const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceSetGlobalMission *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceShowMessage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceShowMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceShowMissionUpdateMessage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceShowMissionUpdateMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceShowSeasonTimeWarning const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceShowSeasonTimeWarning *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceGoToGalacticPoint const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceGoToGalacticPoint *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceStartMission const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceStartMission *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceStartScanEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceStartScanEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceKill::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcMissionSequenceKill *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceKillEncounter::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceKillEncounter *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceLearnWords::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceLearnWords *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceLeaveNexusMP::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceLeaveNexusMP *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceModifyStat::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceModifyStat *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequencePirates::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequencePirates *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceProductAmountNeeded::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceProductAmountNeeded *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceRepairTech::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceRepairTech *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceReward::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcMissionSequenceReward *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceScan::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceScan *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceSendChatMessage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceSendChatMessage *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceSetCurrentMission::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceSetCurrentMission *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceSetGlobalMission::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceSetGlobalMission *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceShowMessage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceShowMessage *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceShowMissionUpdateMessage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceShowMissionUpdateMessage *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceShowPodMessage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceShowPodMessage *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceShowSeasonTimeWarning::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceShowSeasonTimeWarning *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceSignalGalacticPoint::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceSignalGalacticPoint *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceStartMission::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceStartMission *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceStartScanEvent::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceStartScanEvent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceKill::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceKillEncounter::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceLearnWords::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceLeaveNexusMP::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceModifyStat::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequencePirates::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceProductAmountNeeded::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceRepairTech::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceReward::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceScan::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceSendChatMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceSetCurrentMission::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceSetGlobalMission::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceShowMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceShowMissionUpdateMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceShowPodMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceShowSeasonTimeWarning::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceSignalGalacticPoint::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceStartMission::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceStartScanEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceModifyStat::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceRepairTech::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceSetGlobalMission::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceShowMessage::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceSignalGalacticPoint::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceStartScanEvent::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceKill::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceKill *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceKillEncounter::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceKillEncounter *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceLeaveNexusMP::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceLeaveNexusMP *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceModifyStat::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceModifyStat *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequencePirates::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequencePirates *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceProductAmountNeeded::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceProductAmountNeeded *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceRepairTech::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceRepairTech *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceReward::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceReward *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceScan::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceScan *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceSendChatMessage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceSendChatMessage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceSetCurrentMission::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceSetCurrentMission *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceShowMessage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceShowMessage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceShowMissionUpdateMessage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceShowMissionUpdateMessage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceShowSeasonTimeWarning::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceShowSeasonTimeWarning *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceStartMission::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceStartMission *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceStartScanEvent::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceStartScanEvent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceLeaveNexusMP::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceLeaveNexusMP *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceShowMessage::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceShowMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceKill::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceKill *
        ]
    ),
    "cGcMissionSequenceKillEncounter::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceKillEncounter *
        ]
    ),
    "cGcMissionSequenceLearnWords::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceLearnWords *
        ]
    ),
    "cGcMissionSequenceLeaveNexusMP::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceLeaveNexusMP *
        ]
    ),
    "cGcMissionSequencePirates::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequencePirates *
        ]
    ),
    "cGcMissionSequenceProductAmountNeeded::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceProductAmountNeeded *
        ]
    ),
    "cGcMissionSequenceRepairTech::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceRepairTech *
        ]
    ),
    "cGcMissionSequenceReward::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceReward *
        ]
    ),
    "cGcMissionSequenceScan::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceScan *
        ]
    ),
    "cGcMissionSequenceCompleteMission::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCompleteMission *
        ]
    ),
    "cGcMissionSequenceSetGlobalMission::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceSetGlobalMission *
        ]
    ),
    "cGcMissionSequenceShowMessage::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceShowMessage *
        ]
    ),
    "cGcMissionSequenceShowMissionUpdateMessage::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceShowMissionUpdateMessage *
        ]
    ),
    "cGcMissionSequenceShowSeasonTimeWarning::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceShowSeasonTimeWarning *
        ]
    ),
    "cGcMissionSequenceStartMission::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceStartMission *
        ]
    ),
    "cGcMissionSequenceStartScanEvent::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceStartScanEvent *
        ]
    ),
    "cGcMissionSequenceKill::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceKill *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceKillEncounter::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceKillEncounter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceLearnWords::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceLearnWords *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceLeaveNexusMP::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceLeaveNexusMP *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceModifyStat::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceModifyStat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequencePirates::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequencePirates *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceProductAmountNeeded::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceProductAmountNeeded *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceRepairTech::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceRepairTech *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceReward::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceReward *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceScan::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceScan *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSendChatMessage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSendChatMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceSetCurrentMission::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceSetCurrentMission *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceSetGlobalMission::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceSetGlobalMission *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceShowMessage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceShowMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceShowMissionUpdateMessage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceShowMissionUpdateMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceShowSeasonTimeWarning::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceShowSeasonTimeWarning *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceSignalGalacticPoint::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGoToGalacticPoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceStartMission::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceStartMission *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceStartScanEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceStartScanEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceKill::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceKill *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceKill *
        ]
    ),
    "cGcMissionSequenceModifyStat::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceModifyStat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceModifyStat *
        ]
    ),
    "cGcMissionSequencePirates::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequencePirates *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequencePirates *
        ]
    ),
    "cGcMissionSequenceProductAmountNeeded::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceProductAmountNeeded *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceProductAmountNeeded *
        ]
    ),
    "cGcMissionSequenceReward::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceReward *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceReward *
        ]
    ),
    "cGcMissionSequenceScan::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceScan *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceScan *
        ]
    ),
    "cGcMissionSequenceSetGlobalMission::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceSetGlobalMission *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceSetGlobalMission *
        ]
    ),
    "cGcMissionSequenceShowMessage::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceShowMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceShowMessage *
        ]
    ),
    "cGcMissionSequenceShowMissionUpdateMessage::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceShowMissionUpdateMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceShowMissionUpdateMessage *
        ]
    ),
    "cGcMissionSequenceSignalGalacticPoint::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGoToGalacticPoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceGoToGalacticPoint *
        ]
    ),
    "cGcMissionSequenceStartMission::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceStartMission *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceStartMission *
        ]
    ),
    "cGcMissionSequenceStartScanEvent::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceStartScanEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceStartScanEvent *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcGenericMissionStage>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGenericMissionStage> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcGenericMissionStage const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcGenericMissionStage *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceEndScanEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceEndScanEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceEnsureBarrelsAtPlayerSettlement const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceEnsureBarrelsAtPlayerSettlement *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceExploreAbandonedFreighter const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceExploreAbandonedFreighter *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceFeed const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceFeed *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceFreighterEngage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceFreighterEngage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceGatherForBuild const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceGatherForBuild *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceGatherForRepair const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceGatherForRepair *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceGetToExpedition const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceGetToExpedition *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceGetToScanEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceGetToScanEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceGetUnits const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceGetUnits *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceGetUnitsToBuyItem const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceGetUnitsToBuyItem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcGenericMissionStage>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGenericMissionStage> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcMissionSequenceEndScanEvent::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceEndScanEvent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceEnsureBarrelsAtPlayerSettlement::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceEnsureBarrelsAtPlayerSettlement *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceExplorationLogSpecial::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceExplorationLogSpecial *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceExploreAbandonedFreighter::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceExploreAbandonedFreighter *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceFeed::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcMissionSequenceFeed *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceFinishSummonAnomaly::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceFinishSummonAnomaly *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceFreighterDefend::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceFreighterDefend *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceFreighterEngage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceFreighterEngage *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGatherForBuild::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceGatherForBuild *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGatherForRefuel::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceGatherForRefuel *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGatherForRepair::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceGatherForRepair *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGetInShip::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceGetInShip *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGetToExpedition::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceGetToExpedition *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGetToScanEvent::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceGetToScanEvent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGetUnits::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceGetUnits *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGetUnitsToBuyItem::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceGetUnitsToBuyItem *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGoToGalacticPoint::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceGoToGalacticPoint *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGroup::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcMissionSequenceGroup *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceEndScanEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceEnsureBarrelsAtPlayerSettlement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceExplorationLogSpecial::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceExploreAbandonedFreighter::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceFeed::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceFinishSummonAnomaly::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceFreighterDefend::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceFreighterEngage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGatherForBuild::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGatherForRefuel::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGatherForRepair::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGetInShip::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGetToExpedition::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGetToScanEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGetUnits::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGetUnitsToBuyItem::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGoToGalacticPoint::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceGatherForRefuel::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceGatherForRepair::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceGetToScanEvent::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceGoToGalacticPoint::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceGroup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcGenericMissionStage>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGenericMissionStage> *
        ]
    ),
    "cGcMissionSequenceGroup::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGroup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceGroup::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGroup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceEndScanEvent::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceEndScanEvent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceEnsureBarrelsAtPlayerSettlement::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceEnsureBarrelsAtPlayerSettlement *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceExploreAbandonedFreighter::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceExploreAbandonedFreighter *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceFeed::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceFeed *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceFreighterEngage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceFreighterEngage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceGatherForBuild::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGatherForBuild *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceGatherForRefuel::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGatherForRefuel *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceGatherForRepair::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGatherForRepair *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceGetToExpedition::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGetToExpedition *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceGetToScanEvent::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGetToScanEvent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceGetUnitsToBuyItem::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGetUnitsToBuyItem *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceGroup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGroup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceGetToScanEvent::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGetToScanEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceGroup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceEndScanEvent::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceEndScanEvent *
        ]
    ),
    "cGcMissionSequenceEnsureBarrelsAtPlayerSettlement::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceEnsureBarrelsAtPlayerSettlement *
        ]
    ),
    "cGcMissionSequenceExploreAbandonedFreighter::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceExploreAbandonedFreighter *
        ]
    ),
    "cGcMissionSequenceFeed::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceFeed *
        ]
    ),
    "cGcMissionSequenceFreighterEngage::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceFreighterEngage *
        ]
    ),
    "cGcMissionSequenceGatherForBuild::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGatherForBuild *
        ]
    ),
    "cGcMissionSequenceGatherForRepair::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGatherForRepair *
        ]
    ),
    "cGcMissionSequenceGetToExpedition::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGetToExpedition *
        ]
    ),
    "cGcMissionSequenceGetToScanEvent::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGetToScanEvent *
        ]
    ),
    "cGcMissionSequenceGetUnits::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGetUnits *
        ]
    ),
    "cGcMissionSequenceGroup::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGroup *
        ]
    ),
    "cGcMissionSequenceEndScanEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceEndScanEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceEnsureBarrelsAtPlayerSettlement::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceEnsureBarrelsAtPlayerSettlement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceExploreAbandonedFreighter::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceExploreAbandonedFreighter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceFeed::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceFeed *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceFreighterEngage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceFreighterEngage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceGatherForBuild::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGatherForBuild *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceGatherForRepair::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGatherForRepair *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceGetToExpedition::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGetToExpedition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceGetToScanEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGetToScanEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceGetUnits::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGetUnits *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceGetUnitsToBuyItem::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGetUnitsToBuyItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceEnsureBarrelsAtPlayerSettlement::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceEnsureBarrelsAtPlayerSettlement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceEnsureBarrelsAtPlayerSettlement *
        ]
    ),
    "cGcMissionSequenceFeed::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceFeed *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceFeed *
        ]
    ),
    "cGcMissionSequenceFreighterEngage::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceFreighterEngage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceFreighterEngage *
        ]
    ),
    "cGcMissionSequenceGetToExpedition::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGetToExpedition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceGetToExpedition *
        ]
    ),
    "cGcMissionSequenceGetToScanEvent::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGetToScanEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceGetToScanEvent *
        ]
    ),
    "cGcMissionSequenceGroup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceGroup *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcConstructionPart>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcConstructionPart> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcMissionSequenceDetailMessagePoint>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMissionSequenceDetailMessagePoint> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcItemNeedPurpose>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemNeedPurpose *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcConstructionPart const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcConstructionPart *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcJudgementMessageOptions const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcJudgementMessageOptions *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMissionSequenceDetailMessagePoint const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMissionSequenceDetailMessagePoint *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcItemNeedPurpose>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcItemNeedPurpose *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcItemNeedPurpose *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcJudgementMessageOptions>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcJudgementMessageOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcJudgementMessageOptions *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceCollectProduct const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceCollectProduct *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceCollectSubstance const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceCollectSubstance *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceCommunicator const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceCommunicator *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceCommunicatorOnTakeOff const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceCommunicatorOnTakeOff *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceCompleteMission const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceCompleteMission *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceCompleteSettlementJudgement const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceCompleteSettlementJudgement *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceConditionalReward const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceConditionalReward *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceConstruct const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceConstruct *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceConstructSettlementBuildingWithScanEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceConstructSettlementBuildingWithScanEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceCreateSpecificPulseEncounter const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceCreateSpecificPulseEncounter *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceDetailMessage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceDetailMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceDetailMessagePoint const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceDetailMessagePoint *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceDiscover const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceDiscover *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceDiscoverOnPlanet const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceDiscoverOnPlanet *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceDisplaySeasonRewardReminder const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceDisplaySeasonRewardReminder *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcExpeditionPaymentToken>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExpeditionPaymentToken> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcMissionSequenceCollectProduct::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcMissionSequenceCollectProduct *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCollectSubstance::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcMissionSequenceCollectSubstance *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCommunicator::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceCommunicator *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCommunicatorOnTakeOff::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceCommunicatorOnTakeOff *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCompleteMission::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceCompleteMission *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCompleteSeasonalMilestone::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceCompleteSeasonalMilestone *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCompleteSettlementJudgement::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceCompleteSettlementJudgement *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceConditionalReward::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceConditionalReward *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceConstruct::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceConstruct *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceConstructSettlementBuildingWithScanEvent::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceConstructSettlementBuildingWithScanEvent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCreateSpecificPulseEncounter::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceCreateSpecificPulseEncounter *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceDetailMessage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceDetailMessage *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceDiscover::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcMissionSequenceDiscover *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceDiscoverOnPlanet::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcMissionSequenceDiscoverOnPlanet *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceDisplaySeasonRewardReminder::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceDisplaySeasonRewardReminder *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceDoMissionsForFaction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceDoMissionsForFaction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCollectProduct::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCollectSubstance::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCommunicator::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCommunicatorOnTakeOff::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCompleteMission::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCompleteSeasonalMilestone::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCompleteSettlementJudgement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceConditionalReward::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceConstruct::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceConstructSettlementBuildingWithScanEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCreateSpecificPulseEncounter::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceDetailMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceDetailMessagePoint::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceDiscover::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceDiscoverOnPlanet::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceDisplaySeasonRewardReminder::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceDoMissionsForFaction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCollectProduct::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceCollectSubstance::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceCommunicator::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceCommunicatorOnTakeOff::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceCompleteSettlementJudgement::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceConditionalReward::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceConstruct::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceDetailMessage::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceDoMissionsForFaction::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceCollectProduct::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCollectProduct *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCollectSubstance::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCollectSubstance *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCommunicator::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCommunicator *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCompleteMission::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCompleteMission *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceConditionalReward::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceConditionalReward *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceConstruct::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceConstruct *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceConstructSettlementBuildingWithScanEvent::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceConstructSettlementBuildingWithScanEvent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCreateSpecificPulseEncounter::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCreateSpecificPulseEncounter *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceDetailMessage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDetailMessage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceDetailMessagePoint::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDetailMessagePoint *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceDiscover::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDiscover *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceDiscoverOnPlanet::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDiscoverOnPlanet *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceDoMissionsForFaction::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDoMissionsForFaction *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCollectProduct::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCollectProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCompleteSettlementJudgement::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCompleteSettlementJudgement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceDetailMessagePoint::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDetailMessagePoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCollectProduct::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCollectProduct *
        ]
    ),
    "cGcMissionSequenceCollectSubstance::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCollectSubstance *
        ]
    ),
    "cGcMissionSequenceCommunicator::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCommunicator *
        ]
    ),
    "cGcMissionSequenceCommunicatorOnTakeOff::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCommunicatorOnTakeOff *
        ]
    ),
    "cGcMissionSequenceCompleteSettlementJudgement::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCompleteSettlementJudgement *
        ]
    ),
    "cGcMissionSequenceConditionalReward::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceConditionalReward *
        ]
    ),
    "cGcMissionSequenceConstruct::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceConstruct *
        ]
    ),
    "cGcMissionSequenceConstructSettlementBuildingWithScanEvent::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceConstructSettlementBuildingWithScanEvent *
        ]
    ),
    "cGcMissionSequenceCreateSpecificPulseEncounter::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCreateSpecificPulseEncounter *
        ]
    ),
    "cGcMissionSequenceDetailMessage::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDetailMessage *
        ]
    ),
    "cGcMissionSequenceDetailMessagePoint::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDetailMessagePoint *
        ]
    ),
    "cGcMissionSequenceDiscover::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDiscover *
        ]
    ),
    "cGcMissionSequenceDiscoverOnPlanet::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDiscoverOnPlanet *
        ]
    ),
    "cGcMissionSequenceDisplaySeasonRewardReminder::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDisplaySeasonRewardReminder *
        ]
    ),
    "cGcMissionSequenceCollectProduct::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCollectProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCollectSubstance::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCollectSubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCommunicator::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCommunicator *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCommunicatorOnTakeOff::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCommunicatorOnTakeOff *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCompleteMission::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCompleteMission *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCompleteSettlementJudgement::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCompleteSettlementJudgement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceConditionalReward::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceConditionalReward *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceConstruct::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceConstruct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceConstructSettlementBuildingWithScanEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceConstructSettlementBuildingWithScanEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCreateSpecificPulseEncounter::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCreateSpecificPulseEncounter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceDetailMessage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDetailMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceDetailMessagePoint::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDetailMessagePoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceDiscover::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDiscover *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceDiscoverOnPlanet::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDiscoverOnPlanet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceDisplaySeasonRewardReminder::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDisplaySeasonRewardReminder *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCollectProduct::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCollectProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceCollectProduct *
        ]
    ),
    "cGcMissionSequenceCollectSubstance::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCollectSubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceCollectSubstance *
        ]
    ),
    "cGcMissionSequenceCommunicator::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCommunicator *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceCommunicator *
        ]
    ),
    "cGcMissionSequenceConditionalReward::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceConditionalReward *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceConditionalReward *
        ]
    ),
    "cGcMissionSequenceConstruct::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceConstruct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceConstruct *
        ]
    ),
    "cGcMissionSequenceConstructSettlementBuildingWithScanEvent::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceConstructSettlementBuildingWithScanEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceConstructSettlementBuildingWithScanEvent *
        ]
    ),
    "cGcMissionSequenceCreateSpecificPulseEncounter::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCreateSpecificPulseEncounter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceCreateSpecificPulseEncounter *
        ]
    ),
    "cGcMissionSequenceDetailMessage::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDetailMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceDetailMessage *
        ]
    ),
    "cGcMissionSequenceDetailMessagePoint::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDetailMessagePoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceDetailMessagePoint *
        ]
    ),
    "cGcMissionSequenceDiscover::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDiscover *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceDiscover *
        ]
    ),
    "cGcMissionSequenceDiscoverOnPlanet::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceDiscoverOnPlanet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceDiscoverOnPlanet *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcMissionCommunityMissionData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMissionCommunityMissionData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcMissionSchedulingData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMissionSchedulingData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcProductToCollect>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcProductToCollect> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcDate>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcMonth>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMonth *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMissionCommunityMissionData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMissionCommunityMissionData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMissionSchedulingData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMissionSchedulingData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcProductToCollect const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcProductToCollect *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcDate>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDate *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcDay>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDay *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDay *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcMonth>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMonth *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMonth *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDailyRecurrence const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDailyRecurrence *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDate const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDate *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDay const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDay *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionCommunityData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionCommunityData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionCommunityMissionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionCommunityMissionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionIDEpochPair const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionIDEpochPair *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSchedulesTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSchedulesTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSchedulingData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSchedulingData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceAudioEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceAudioEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceBounty const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceBounty *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceBroadcastMessage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceBroadcastMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceBuild const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceBuild *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceCloseMenu const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceCloseMenu *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceCollectMoney const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceCollectMoney *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionSequenceCollectMultiProducts const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionSequenceCollectMultiProducts *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMonth const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMonth *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMonthlyRecurrence const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMonthlyRecurrence *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcObjectiveTextFormatOptions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcObjectiveTextFormatOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTargetMissionSurveyOptions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTargetMissionSurveyOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeeklyRecurrence const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeeklyRecurrence *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcYearlyRecurrence const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcYearlyRecurrence *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcMissionCommunityMissionData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMissionCommunityMissionData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcMissionSchedulingData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMissionSchedulingData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcDailyRecurrence::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcDailyRecurrence *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceAudioEvent::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceAudioEvent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceBounty::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcMissionSequenceBounty *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceBroadcastMessage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceBroadcastMessage *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceBuild::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceBuild *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceClearInventoryHistory::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceClearInventoryHistory *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCloseMenu::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceCloseMenu *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCollectMoney::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceCollectMoney *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCollectMultiProducts::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionSequenceCollectMultiProducts *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMonthlyRecurrence::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcMonthlyRecurrence *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcWeeklyRecurrence::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcWeeklyRecurrence *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcYearlyRecurrence::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcYearlyRecurrence *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcDailyRecurrence::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDate::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDay::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionCommunityData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionCommunityMissionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConsequenceClearDetailMessages::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionIDEpochPair::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSchedulesTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSchedulingData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceAudioEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceBounty::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceBroadcastMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceBuild::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceClearInventoryHistory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCloseMenu::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCollectMoney::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionSequenceCollectMultiProducts::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMonth::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMonthlyRecurrence::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcObjectiveTextFormatOptions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTargetMissionSurveyOptions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeeklyRecurrence::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcYearlyRecurrence::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionCommunityData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSchedulesTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSchedulingData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceAudioEvent::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceBuild::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceCollectMoney::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSequenceCollectMultiProducts::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionCommunityData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWaveGroup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionSchedulesTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSchedulesTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDate::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDate *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDay::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDay *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionCommunityData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionCommunityData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionCommunityMissionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionCommunityMissionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionIDEpochPair::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionIDEpochPair *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSchedulingData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSchedulingData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceBounty::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceBounty *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceBroadcastMessage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceBroadcastMessage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceBuild::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceBuild *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCloseMenu::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCloseMenu *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCollectMoney::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCollectMoney *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCollectMultiProducts::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCollectMultiProducts *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMonth::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMonth *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectiveTextFormatOptions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectiveTextFormatOptions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTargetMissionSurveyOptions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetMissionSurveyOptions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcYearlyRecurrence::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcYearlyRecurrence *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionIDEpochPair::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionIDEpochPair *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcDate::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDay::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDay *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionIDEpochPair::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionIDEpochPair *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSchedulingData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSchedulingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCloseMenu::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCloseMenu *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMonth::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMonth *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectiveTextFormatOptions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectiveTextFormatOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTargetMissionSurveyOptions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetMissionSurveyOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDailyRecurrence::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDailyRecurrence *
        ]
    ),
    "cGcPlayerStatsGroup::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWordKnowledge *
        ]
    ),
    "cGcMissionSequenceAudioEvent::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceAudioEvent *
        ]
    ),
    "cGcMissionSequenceBounty::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceBounty *
        ]
    ),
    "cGcMissionSequenceBroadcastMessage::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceBroadcastMessage *
        ]
    ),
    "cGcMissionSequenceBuild::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceBuild *
        ]
    ),
    "cGcMissionSequenceCloseMenu::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCloseMenu *
        ]
    ),
    "cGcMissionSequenceCollectMoney::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCollectMoney *
        ]
    ),
    "cGcMissionSequenceCollectMultiProducts::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCollectMultiProducts *
        ]
    ),
    "cGcMonthlyRecurrence::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeeklyRecurrence *
        ]
    ),
    "cGcObjectiveTextFormatOptions::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectiveTextFormatOptions *
        ]
    ),
    "cGcTargetMissionSurveyOptions::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetMissionSurveyOptions *
        ]
    ),
    "cGcYearlyRecurrence::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcYearlyRecurrence *
        ]
    ),
    "cGcMissionIDEpochPair::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionIDEpochPair *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcDailyRecurrence::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDailyRecurrence *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDate::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDay::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDay *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionCommunityData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionCommunityData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionIDEpochPair::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionIDEpochPair *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSchedulesTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSchedulesTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSchedulingData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSchedulingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceAudioEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceAudioEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceBounty::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceBounty *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceBroadcastMessage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceBroadcastMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceBuild::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceBuild *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCloseMenu::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCloseMenu *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCollectMoney::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCollectMoney *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionSequenceCollectMultiProducts::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCollectMultiProducts *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMonth::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMonth *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMonthlyRecurrence::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMonthlyRecurrence *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectiveTextFormatOptions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectiveTextFormatOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTargetMissionSurveyOptions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetMissionSurveyOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeeklyRecurrence::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeeklyRecurrence *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcYearlyRecurrence::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcYearlyRecurrence *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDate::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDate *
        ]
    ),
    "cGcDay::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDay *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDay *
        ]
    ),
    "cGcMissionSchedulingData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSchedulingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSchedulingData *
        ]
    ),
    "cGcMissionSequenceAudioEvent::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceAudioEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceAudioEvent *
        ]
    ),
    "cGcMissionSequenceBounty::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceBounty *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceBounty *
        ]
    ),
    "cGcMissionSequenceBroadcastMessage::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceBroadcastMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceBroadcastMessage *
        ]
    ),
    "cGcMissionSequenceBuild::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceBuild *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceBuild *
        ]
    ),
    "cGcMissionSequenceCloseMenu::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCloseMenu *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceCloseMenu *
        ]
    ),
    "cGcMissionSequenceCollectMoney::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionSequenceCollectMoney *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionSequenceCollectMoney *
        ]
    ),
    "cGcMonth::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMonth *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMonth *
        ]
    ),
    "cGcMonthlyRecurrence::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMonthlyRecurrence *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMonthlyRecurrence *
        ]
    ),
    "cGcObjectiveTextFormatOptions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectiveTextFormatOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcObjectiveTextFormatOptions *
        ]
    ),
    "cGcTargetMissionSurveyOptions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTargetMissionSurveyOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTargetMissionSurveyOptions *
        ]
    ),
    "cGcYearlyRecurrence::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcYearlyRecurrence *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcYearlyRecurrence *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCostTableEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCostTableEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDefaultMissionProduct>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDefaultMissionProduct> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDefaultMissionSubstance>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDefaultMissionSubstance> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcGenericMissionSequence>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGenericMissionSequence> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcGenericMissionVersionProgress>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGenericMissionVersionProgress> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcGenericRewardTableEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGenericRewardTableEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcMissionFaction>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMissionFaction> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcScanEventData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcScanEventData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcMissionConditionTest>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionTest *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcNumberedTextList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNumberedTextList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCostTableEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCostTableEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDefaultMissionProduct const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDefaultMissionProduct *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDefaultMissionSubstance const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDefaultMissionSubstance *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcGenericMissionSequence const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcGenericMissionVersionProgress const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcGenericMissionVersionProgress *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcGenericRewardTableEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcGenericRewardTableEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMissionFaction const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMissionFaction *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcScanEventData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcScanEventData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcMissionConditionTest>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionTest *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionTest *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcMissionFaction>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionFaction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionFaction *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcNumberedTextList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNumberedTextList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNumberedTextList *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomNotifyTimerOptions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomNotifyTimerOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDefaultMissionItemsTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDefaultMissionItemsTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDefaultMissionProduct const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDefaultMissionProduct *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDefaultMissionProductEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDefaultMissionProductEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDefaultMissionSubstance const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDefaultMissionSubstance *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDefaultMissionSubstanceEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDefaultMissionSubstanceEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFactionSelectOptions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFactionSelectOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGenericMissionSequence const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGenericMissionStage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericMissionStage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGenericMissionVersionProgress const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericMissionVersionProgress *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionBoardOptions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionBoardOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConsequenceRemoveCommunicatorMessage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConsequenceRemoveCommunicatorTakeOffMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConsequenceRemoveScanEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConsequenceRemoveScanEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcDefaultMissionProduct>::cTkDynamicArray<cGcDefaultMissionProduct>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDefaultMissionSubstance> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcDefaultMissionSubstance> *
        ]
    ),
    "cGcGenericMissionSequence::cGcGenericMissionSequence": {
        "cGcGenericMissionSequence *, cGcGenericMissionSequence *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcGenericMissionSequence *
                ctypes.c_ulonglong,  # cGcGenericMissionSequence *
            ]
        ),
        "cGcGenericMissionSequence *, const cGcGenericMissionSequence *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcGenericMissionSequence *
                ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
            ]
        ),
    },
    "cGcScanEventData::cGcScanEventData": {
        "cGcScanEventData *, cGcScanEventData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcScanEventData *
                ctypes.c_ulonglong,  # cGcScanEventData *
            ]
        ),
        "cGcScanEventData *, const cGcScanEventData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcScanEventData *
                ctypes.c_ulonglong,  # const cGcScanEventData *
            ]
        ),
    },
    "cGcGenericMissionSequence::~cGcGenericMissionSequence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericMissionSequence *
        ]
    ),
    "cGcMissionBoardOptions::~cGcMissionBoardOptions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionBoardOptions *
        ]
    ),
    "cTkDynamicArray<cGcCostTableEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCostTableEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcDefaultMissionSubstance>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDefaultMissionSubstance> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcGenericMissionSequence>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGenericMissionSequence> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcGenericRewardTableEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGenericRewardTableEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcMissionFaction>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMissionFaction> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcScanEventData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcScanEventData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcMissionConsequenceRemoveCommunicatorMessage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcMissionConsequenceRemoveCommunicatorMessage *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcMissionConsequenceRemoveCommunicatorTakeOffMessage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcMissionConsequenceRemoveCommunicatorTakeOffMessage *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcMissionConsequenceRemoveScanEvent::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcMissionConsequenceRemoveScanEvent *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcMissionConsequenceResetPulseEncounterOverride::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcMissionConsequenceResetPulseEncounterOverride *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCustomNotifyTimerOptions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDefaultMissionItemsTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDefaultMissionProduct::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDefaultMissionProductEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDefaultMissionSubstance::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDefaultMissionSubstanceEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFactionSelectOptions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGenericMissionSequence::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGenericMissionStage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGenericMissionVersionProgress::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionBoardOptions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConsequenceRemoveCommunicatorMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConsequenceRemoveCommunicatorTakeOffMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConsequenceRemoveScanEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConsequenceResetPulseEncounterOverride::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDefaultMissionItemsTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFactionSelectOptions::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGenericMissionSequence::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGenericMissionStage::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionBoardOptions::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcCostTableEntry>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCostTableEntry> *
        ]
    ),
    "cTkDynamicArray<cGcGenericRewardTableEntry>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGenericRewardTableEntry> *
        ]
    ),
    "cGcUniverseAddressData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniverseAddressData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGenericMissionSequence::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericMissionSequence *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGenericMissionStage::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericMissionStage *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionBoardOptions::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionBoardOptions *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGenericMissionSequence::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericMissionSequence *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGenericMissionStage::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericMissionStage *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionBoardOptions::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionBoardOptions *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomNotifyTimerOptions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomNotifyTimerOptions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDefaultMissionItemsTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDefaultMissionItemsTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDefaultMissionProduct::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDefaultMissionProduct *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDefaultMissionProductEnum::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDefaultMissionProductEnum *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDefaultMissionSubstance::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDefaultMissionSubstance *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDefaultMissionSubstanceEnum::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDefaultMissionSubstanceEnum *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFactionSelectOptions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFactionSelectOptions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericMissionSequence::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericMissionSequence *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericMissionStage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericMissionStage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericMissionVersionProgress::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericMissionVersionProgress *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionBoardOptions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionBoardOptions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConsequenceRemoveCommunicatorMessage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConsequenceRemoveCommunicatorMessage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConsequenceRemoveCommunicatorTakeOffMessage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConsequenceRemoveCommunicatorTakeOffMessage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConsequenceRemoveScanEvent::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConsequenceRemoveScanEvent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomNotifyTimerOptions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomNotifyTimerOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDefaultMissionItemsTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDefaultMissionItemsTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDefaultMissionProductEnum::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDefaultMissionProductEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDefaultMissionSubstanceEnum::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDefaultMissionSubstanceEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFactionSelectOptions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFactionSelectOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericMissionSequence::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericMissionSequence *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericMissionStage::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericMissionStage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionBoardOptions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionBoardOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionTable::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomNotifyTimerOptions::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomNotifyTimerOptions *
        ]
    ),
    "cGcFactionSelectOptions::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFactionSelectOptions *
        ]
    ),
    "cGcGenericMissionSequence::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericMissionSequence *
        ]
    ),
    "cGcCustomNotifyTimerOptions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomNotifyTimerOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDefaultMissionItemsTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDefaultMissionItemsTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDefaultMissionProductEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDefaultMissionProductEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDefaultMissionSubstanceEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDefaultMissionSubstanceEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFactionSelectOptions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFactionSelectOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericMissionSequence::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericMissionSequence *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericMissionStage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericMissionStage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericMissionVersionProgress::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericMissionVersionProgress *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionBoardOptions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionBoardOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConsequenceRemoveCommunicatorTakeOffMessage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConsequenceRemoveCommunicatorTakeOffMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConsequenceRemoveScanEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConsequenceRemoveScanEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomNotifyTimerOptions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomNotifyTimerOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCustomNotifyTimerOptions *
        ]
    ),
    "cGcDefaultMissionItemsTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDefaultMissionItemsTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDefaultMissionProductEnum::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDefaultMissionProductEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDefaultMissionProductEnum *
        ]
    ),
    "cGcDefaultMissionSubstanceEnum::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDefaultMissionSubstanceEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDefaultMissionSubstanceEnum *
        ]
    ),
    "cGcFactionSelectOptions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFactionSelectOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFactionSelectOptions *
        ]
    ),
    "cGcGenericMissionSequence::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericMissionSequence *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGenericMissionSequence *
        ]
    ),
    "cGcGenericMissionVersionProgress::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericMissionVersionProgress *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGenericMissionVersionProgress *
        ]
    ),
    "cGcMissionBoardOptions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionBoardOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionBoardOptions *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcAlienRace>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienRace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcPlayerHazardType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerHazardType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlayerHazardType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerHazardType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerHazardType *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionSystemPlanetTest const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionSystemPlanetTest *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionSystemRace const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionSystemRace *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionSystemStarClass const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionSystemStarClass *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionTakingDamage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionTakingDamage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionTechnologyKnown const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionTechnologyKnown *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionTotalWarpsNumber const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionTotalWarpsNumber *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionAutoPowerEnabled const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionAutoPowerEnabled *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionUnderwaterDepth const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionUnderwaterDepth *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionUsingPortal const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionUsingPortal *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionUsingThirdPersonCamera const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionUsingThirdPersonCamera *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionWaitForPirates const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionWaitForPirates *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionWaitForTime const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionWaitForTime *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionWeaponMode const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionWeaponMode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionWeather const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionWeather *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionWordCategoryKnown const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionWordCategoryKnown *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionWristMenuOpen const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionWristMenuOpen *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionSystemHasInfestedPlanet::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionSystemHasInfestedPlanet *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSystemPlanetTest::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionSystemPlanetTest *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSystemRace::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionSystemRace *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSystemStarClass::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionSystemStarClass *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionTakingDamage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionTakingDamage *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionTechnologyKnown::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionTechnologyKnown *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionTotalWarpsNumber::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionTotalWarpsNumber *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionTutorialEnabled::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionTutorialEnabled *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionUnclaimedStageReward::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionUnclaimedStageReward *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionUnderwaterDepth::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionUnderwaterDepth *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionUsingPortal::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionUsingPortal *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionUsingThirdPersonCamera::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionUsingThirdPersonCamera *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionVisorActive::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionVisorActive *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionWaitForPirates::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionWaitForPirates *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionWaitForTime::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionWaitForTime *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionWarping::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionWarping *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionWaterInSystem::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionWaterInSystem *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionWaterPlanet::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionWaterPlanet *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionWeaponMode::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionWeaponMode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionWeather::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionWeather *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionWristMenuOpen::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionWristMenuOpen *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSystemHasInfestedPlanet::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSystemPlanetTest::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSystemRace::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSystemStarClass::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionTakingDamage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionTechnologyKnown::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionTotalWarpsNumber::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionTutorialEnabled::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionUnclaimedStageReward::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionUnderwaterDepth::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionUsingPortal::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionUsingThirdPersonCamera::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionVisorActive::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionWaitForPirates::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionWaitForTime::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionWarping::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionWaterInSystem::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionWaterPlanet::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionWeaponMode::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionWeather::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionWordCategoryKnown::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionWristMenuOpen::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSystemPlanetTest::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionSystemRace::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionSystemStarClass::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionWaitForPirates::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionWeaponMode::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionWeather::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionWordCategoryKnown::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionTakingDamage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionTakingDamage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionTechnologyKnown::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionTechnologyKnown *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionWaitForPirates::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionWaitForPirates *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionWeather::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionWeather *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionWristMenuOpen::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionWristMenuOpen *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionSystemPlanetTest::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionSystemPlanetTest *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionSystemRace::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionSystemRace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionSystemStarClass::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionSystemStarClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionTakingDamage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionTakingDamage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionTechnologyKnown::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionTechnologyKnown *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionTotalWarpsNumber::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionTotalWarpsNumber *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionUnderwaterDepth::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionUnderwaterDepth *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionUsingPortal::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionUsingPortal *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionUsingThirdPersonCamera::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionUsingThirdPersonCamera *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionWaitForPirates::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionWaitForPirates *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionWaitForTime::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionWaitForTime *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionWeaponMode::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionWeaponMode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionWeather::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionWeather *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionWordCategoryKnown::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionWordCategoryKnown *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionWristMenuOpen::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionWristMenuOpen *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionWaitForPirates::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionWaitForPirates *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionWaitForPirates *
        ]
    ),
    "cGcMissionConditionWordCategoryKnown::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionWordCategoryKnown *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionWordCategoryKnown *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkEqualityEnum>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEqualityEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkEqualityEnum>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkEqualityEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkEqualityEnum *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionProductKnown const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionProductKnown *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionPulseEncounterActive const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionPulseEncounterActive *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionRefinerActive const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionRefinerActive *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionRefinerHasOutput const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionRefinerHasOutput *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionRequestedPhoto const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionRequestedPhoto *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionSeasonNumber const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionSeasonNumber *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionSpecialKnown const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionSpecialKnown *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionSeasonRewardsRecipe const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionSeasonRewardsRecipe *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionSentinelLevel const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionSentinelLevel *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionSettlementStatLevel const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionSettlementStatLevel *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionShieldDown const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionShieldDown *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionShipEngineStatus const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionShipEngineStatus *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionSquadronPilotsOwned const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionSquadronPilotsOwned *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionSquadronSlots const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionSquadronSlots *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionStatChange const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionStatChange *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionStatDiff const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionStatDiff *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionStatLevel const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionStatLevel *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionProductKnown::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionProductKnown *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPulseEncounterActive::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionPulseEncounterActive *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPulseEncounterOverriden::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionPulseEncounterOverriden *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionReadyToSpawnPirates::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionReadyToSpawnPirates *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionRefinerActive::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionRefinerActive *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionRefinerHasOutput::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionRefinerHasOutput *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionRequestedPhoto::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionRequestedPhoto *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionRidingCreature::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionRidingCreature *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSeasonNumber::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionSeasonNumber *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSeasonRewardUnlocked::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionSeasonRewardUnlocked *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSeasonRewardsRecipe::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionSeasonRewardsRecipe *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSentinelLevel::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionSentinelLevel *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSentinelsDisabled::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionSentinelsDisabled *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSettlementBuildingsAllStarted::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionSettlementBuildingsAllStarted *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSettlementStatLevel::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionSettlementStatLevel *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSettlementsEnabled::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionSettlementsEnabled *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionShieldDown::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionShieldDown *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionShipEngineStatus::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionShipEngineStatus *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSpaceCombatEnabled::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionSpaceCombatEnabled *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSpecialKnown::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionSpecialKnown *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSquadronPilotsOwned::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionSquadronPilotsOwned *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSquadronSlots::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionSquadronSlots *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionStartWithAllPartsKnown::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionStartWithAllPartsKnown *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionStatChange::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionStatChange *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionStatDiff::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionStatDiff *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionStatLevel::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionStatLevel *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionProductKnown::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPulseEncounterActive::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPulseEncounterOverriden::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionReadyToSpawnPirates::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionRefinerActive::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionRefinerHasOutput::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionRequestedPhoto::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionRidingCreature::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSeasonNumber::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSeasonRewardUnlocked::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSeasonRewardsRecipe::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSentinelLevel::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSentinelsDisabled::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSettlementBuildingsAllStarted::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSettlementStatLevel::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSettlementsEnabled::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionShieldDown::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionShipEngineStatus::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSpaceCombatEnabled::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSpecialKnown::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSquadronPilotsOwned::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionSquadronSlots::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionStartWithAllPartsKnown::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionStatChange::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionStatDiff::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionStatLevel::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionRequestedPhoto::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionSettlementStatLevel::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionSquadronPilotsOwned::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionSquadronSlots::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionStatDiff::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionStatLevel::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionProductKnown::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionProductKnown *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionPulseEncounterActive::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionPulseEncounterActive *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionRefinerActive::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionRefinerActive *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionSquadronSlots::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionSquadronSlots *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionStatChange::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionStatChange *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionStatDiff::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionStatDiff *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionStatLevel::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionStatLevel *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionSettlementStatLevel::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionSettlementStatLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionStatLevel::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionStatLevel *
        ]
    ),
    "cGcMissionConditionProductKnown::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionProductKnown *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionPulseEncounterActive::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionPulseEncounterActive *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionRefinerActive::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionRefinerActive *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionRefinerHasOutput::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionRefinerHasOutput *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionRequestedPhoto::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionRequestedPhoto *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionSeasonNumber::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionSeasonNumber *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionSentinelLevel::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionSentinelLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionSettlementStatLevel::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionSettlementStatLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionShipEngineStatus::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionShipEngineStatus *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionSquadronPilotsOwned::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionSquadronPilotsOwned *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionSquadronSlots::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionSquadronSlots *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionStatChange::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionStatChange *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionStatDiff::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionStatDiff *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionStatLevel::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionStatLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionSettlementStatLevel::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionSettlementStatLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionSettlementStatLevel *
        ]
    ),
    "cGcMissionConditionStatLevel::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionStatLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionStatLevel *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBuildingClassification>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBuildingClassification> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBuildingClassification const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBuildingClassification *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionMissionCompleted const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionMissionCompleted *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionMissionMessage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionMissionMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionMissionSelected const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionMissionSelected *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionNPCHabitationHasWorker const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionNPCHabitationHasWorker *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionNearObject const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionNearObject *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionNearPole const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionNearPole *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionNearScanEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionNearScanEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionNearestBuilding const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionNearestBuilding *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionNumAtlasStationsVisited const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionNumAtlasStationsVisited *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionNumberOfShipsOwned const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionNumberOfShipsOwned *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionOnOtherSideOfPortal const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionOnOtherSideOfPortal *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionOnPlanetWithSandwormsOverriden const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionOnPlanetWithSandwormsOverriden *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionPercentageChance const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionPercentageChance *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionPlanetCreatureRoles const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionPlanetCreatureRoles *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionPlanetResourceHint const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionPlanetResourceHint *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionPlanetStatLevel const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionPlanetStatLevel *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionPrimaryExocraft const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionPrimaryExocraft *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionMissionCompleted::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionMissionCompleted *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionMissionMessage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionMissionMessage *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionMissionMessagePortal::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionMissionMessagePortal *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionMissionMessageWarp::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionMissionMessageWarp *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionMissionSelected::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionMissionSelected *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionMultiplayerFreighterAvailable::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionMultiplayerFreighterAvailable *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionNPCHabitationHasWorker::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionNPCHabitationHasWorker *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionNearObject::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionNearObject *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionNearPole::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionNearPole *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionNearScanEvent::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionNearScanEvent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionNearestBuilding::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionNearestBuilding *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionNexusEnabled::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionNexusEnabled *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionNumAtlasStationsVisited::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionNumAtlasStationsVisited *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionNumberOfShipsOwned::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionNumberOfShipsOwned *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionOnFootCombatEnabled::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionOnFootCombatEnabled *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionOnMultiplayerMission::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionOnMultiplayerMission *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionOnOtherSideOfPortal::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionOnOtherSideOfPortal *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionOnPlanetWithSandwormsOverriden::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionOnPlanetWithSandwormsOverriden *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPadActive::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionPadActive *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPercentageChance::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionPercentageChance *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPirateSystem::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionPirateSystem *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPlanetAttackPiratesActive::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionPlanetAttackPiratesActive *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPlanetCreatureRoles::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionPlanetCreatureRoles *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPlanetResourceHint::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionPlanetResourceHint *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPlanetStatLevel::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionPlanetStatLevel *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPrimaryExocraft::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionPrimaryExocraft *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionMissionCompleted::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionMissionMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionMissionMessagePortal::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionMissionMessageWarp::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionMissionSelected::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionMultiplayerFreighterAvailable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionNPCHabitationHasWorker::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionNearObject::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionNearPole::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionNearScanEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionNearestBuilding::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionNexusEnabled::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionNumAtlasStationsVisited::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionNumberOfShipsOwned::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionOnFootCombatEnabled::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionOnMultiplayerMission::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionOnOtherSideOfPortal::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionOnPlanetWithSandwormsOverriden::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPadActive::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPercentageChance::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPirateSystem::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPlanetAttackPiratesActive::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPlanetCreatureRoles::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPlanetResourceHint::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPlanetStatLevel::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionPrimaryExocraft::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionNPCHabitationHasWorker::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionNearestBuilding::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionNumAtlasStationsVisited::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionNumberOfShipsOwned::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionPlanetCreatureRoles::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionPlanetStatLevel::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionPrimaryExocraft::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionMissionCompleted::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionMissionCompleted *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionNearScanEvent::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionNearScanEvent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionPercentageChance::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionPercentageChance *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionPlanetStatLevel::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionPlanetStatLevel *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionPlanetStatLevel::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionPlanetStatLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionMissionCompleted::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionMissionCompleted *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionNPCHabitationHasWorker::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionNPCHabitationHasWorker *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionNearObject::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionNearObject *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionNearPole::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionNearPole *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionNearScanEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionNearScanEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionNearestBuilding::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionNearestBuilding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionNumAtlasStationsVisited::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionNumAtlasStationsVisited *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionNumberOfShipsOwned::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionNumberOfShipsOwned *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionOnOtherSideOfPortal::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionOnOtherSideOfPortal *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionOnPlanetWithSandwormsOverriden::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionOnPlanetWithSandwormsOverriden *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionPercentageChance::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionPercentageChance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionPlanetCreatureRoles::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionPlanetCreatureRoles *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionPlanetStatLevel::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionPlanetStatLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionPrimaryExocraft::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionPrimaryExocraft *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionNPCHabitationHasWorker::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionNPCHabitationHasWorker *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionNPCHabitationHasWorker *
        ]
    ),
    "cGcMissionConditionNearScanEvent::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionNearScanEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionNearScanEvent *
        ]
    ),
    "cGcMissionConditionPercentageChance::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionPercentageChance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionPercentageChance *
        ]
    ),
    "cGcMissionConditionPlanetStatLevel::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionPlanetStatLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionPlanetStatLevel *
        ]
    ),
    "cGcMissionConditionPrimaryExocraft::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionPrimaryExocraft *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionPrimaryExocraft *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcMissionConditionIsScanEventLocal>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionIsScanEventLocal *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionIsScanEventLocal *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionInVR const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionInVR *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionInventorySlots const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionInventorySlots *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionIsAbandFreighterDoorOpen const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionIsAbandFreighterDoorOpen *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionIsAnomalyLoaded const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionIsAnomalyLoaded *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionIsDepotDestroyed const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionAreDroneHivePartsDestroyed *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionIsFrigateFlybyActive const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionIsFrigateFlybyActive *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionIsLookingAtAnomaly const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionIsLookingAtAnomaly *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionIsMissionInProgress const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionIsMissionInProgress *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionIsPartyPlanetUnlocked const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionIsPartyPlanetUnlocked *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionIsPlayerWanted const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionIsPlayerWanted *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionIsPlayerWeak const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionIsPlayerWeak *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionIsScanEventActive const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionIsScanEventActive *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionIsScanEventLocal const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionIsScanEventLocal *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionIsScanEventLocalOrNear const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionIsScanEventLocalOrNear *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionIsScanEventOnCurrentPlanet const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionIsScanEventOnCurrentPlanet *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionIsScanEventRepaired const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionIsScanEventRepaired *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionIsSurveying const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionIsSurveying *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionIsTechnologyRepaired const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionIsTechnologyRepaired *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionItemCostsEnabled const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionItemCostsEnabled *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionLocation const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionLocation *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionMessageBeaconsQuery const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionMessageBeaconsQuery *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionInVR::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionInVR *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionInventoryOpen::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionInventoryOpen *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionInventorySlots::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionInventorySlots *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsAbandFreighterDoorOpen::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionIsAbandFreighterDoorOpen *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsAnomalyLoaded::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionIsAnomalyLoaded *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsCurrentMission::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionIsCurrentMission *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsDepotDestroyed::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionIsDepotDestroyed *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsFrigateFlybyActive::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionIsFrigateFlybyActive *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsGrabbed::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionIsGrabbed *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsLookingAtAnomaly::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionIsLookingAtAnomaly *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsMissionInProgress::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionIsMissionInProgress *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsPartyPlanetUnlocked::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionIsPartyPlanetUnlocked *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsPlayerWanted::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionIsPlayerWanted *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsPlayerWeak::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionIsPlayerWeak *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsScanEventActive::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventActive *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsScanEventLocal::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventLocal *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsScanEventLocalOrNear::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventLocalOrNear *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsScanEventOnCurrentPlanet::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventOnCurrentPlanet *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsScanEventRepaired::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventRepaired *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsSurveying::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionIsSurveying *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsTechnologyRepaired::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionIsTechnologyRepaired *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionItemCostsEnabled::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionItemCostsEnabled *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionLifeSupportEnabled::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionLifeSupportEnabled *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionLocalScanActive::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionLocalScanActive *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionLocation::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionLocation *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionMessageBeaconsQuery::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionMessageBeaconsQuery *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionInVR::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionInventoryOpen::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionInventorySlots::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsAbandFreighterDoorOpen::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsAnomalyLoaded::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsCurrentMission::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsDepotDestroyed::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsFrigateFlybyActive::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsGrabbed::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsLookingAtAnomaly::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsMissionInProgress::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsPartyPlanetUnlocked::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsPlayerWanted::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsPlayerWeak::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsScanEventActive::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsScanEventLocal::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsScanEventLocalOrNear::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsScanEventOnCurrentPlanet::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsScanEventRepaired::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsSurveying::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionIsTechnologyRepaired::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionItemCostsEnabled::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionLifeSupportEnabled::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionLocalScanActive::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionLocation::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionMessageBeaconsQuery::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionInventorySlots::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionIsAnomalyLoaded::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionIsFrigateFlybyActive::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionIsPlayerWanted::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionItemCostsEnabled::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionInVR::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionInVR *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionInventorySlots::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionInventorySlots *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsDepotDestroyed::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsDepotDestroyed *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsMissionInProgress::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsMissionInProgress *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsPartyPlanetUnlocked::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsPartyPlanetUnlocked *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsScanEventActive::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventActive *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsScanEventLocal::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventLocal *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsScanEventOnCurrentPlanet::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventOnCurrentPlanet *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsScanEventRepaired::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventRepaired *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsTechnologyRepaired::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsTechnologyRepaired *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionInVR::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionInVR *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionInventorySlots::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionInventorySlots *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsAbandFreighterDoorOpen::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsAbandFreighterDoorOpen *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsAnomalyLoaded::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsAnomalyLoaded *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsDepotDestroyed::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionAreDroneHivePartsDestroyed *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsFrigateFlybyActive::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsFrigateFlybyActive *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsLookingAtAnomaly::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsLookingAtAnomaly *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsMissionInProgress::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsMissionInProgress *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsPartyPlanetUnlocked::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsPartyPlanetUnlocked *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsPlayerWanted::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsPlayerWanted *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsPlayerWeak::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsPlayerWeak *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsScanEventActive::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventActive *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsScanEventLocal::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventLocal *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsScanEventLocalOrNear::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventLocalOrNear *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsScanEventOnCurrentPlanet::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventOnCurrentPlanet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsScanEventRepaired::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventRepaired *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsSurveying::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsSurveying *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionIsTechnologyRepaired::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsTechnologyRepaired *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionItemCostsEnabled::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionItemCostsEnabled *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionLocation::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionLocation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionMessageBeaconsQuery::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionMessageBeaconsQuery *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionInventorySlots::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionInventorySlots *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionInventorySlots *
        ]
    ),
    "cGcMissionConditionIsScanEventLocal::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionIsScanEventLocal *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionIsScanEventLocal *
        ]
    ),
    "cGcMissionConditionItemCostsEnabled::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionItemCostsEnabled *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionItemCostsEnabled *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcInventoryClass>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcInventoryClass>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryClass *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasMultiTool const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasMultiTool *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasPendingSettlementJudgement const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasPendingSettlementJudgement *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasProcMissionForFaction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasProcMissionForFaction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasProcProduct const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasProcProduct *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasProcTechnology const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasProcTechnology *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasProduct const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasProduct *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasSettlementBuilding const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasSettlementBuilding *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasShip const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasShip *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasSubstance const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasSubstance *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasTechnology const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasTechnology *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHazard const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHazard *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionInCombat const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionInCombat *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionInMultiplayer const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionInMultiplayer *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionInSeasonalUA const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionInSeasonalUA *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionInUA const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionInUA *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionInteractionIndexChanged const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionInteractionIndexChanged *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasMultiTool::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasMultiTool *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasPendingSettlementJudgement::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasPendingSettlementJudgement *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasPlatformReward::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasPlatformReward *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasProcMissionForFaction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasProcMissionForFaction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasProcProduct::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasProcProduct *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasProcTechnology::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasProcTechnology *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasProduct::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasProduct *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasSeasonalReward::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasSeasonalReward *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasSettlement::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasSettlement *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasSettlementBuilding::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasSettlementBuilding *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasSettlementLocal::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasSettlementLocal *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasSettlementProductPending::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasSettlementProductPending *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasShip::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasShip *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasSpareProcTech::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasSpareProcTech *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasSubstance::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasSubstance *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasTechnology::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasTechnology *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasTwitchReward::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasTwitchReward *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHazard::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHazard *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHazardsEnabled::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHazardsEnabled *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionInCombat::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionInCombat *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionInMultiplayer::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionInMultiplayer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionInSeasonalUA::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionInSeasonalUA *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionInUA::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionInUA *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionInteractionIndexChanged::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionInteractionIndexChanged *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasMultiTool::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasPendingSettlementJudgement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasPlatformReward::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasProcMissionForFaction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasProcProduct::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasProcTechnology::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasProduct::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasSeasonalReward::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasSettlement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasSettlementBuilding::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasSettlementLocal::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasSettlementProductPending::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasShip::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasSpareProcTech::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasSubstance::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasTechnology::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasTwitchReward::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHazard::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHazardsEnabled::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionInCombat::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionInMultiplayer::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionInSeasonalUA::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionInUA::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionInteractionIndexChanged::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasMultiTool::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionHasProcMissionForFaction::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionHasProcProduct::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionHasProduct::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionHasSettlementBuilding::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionHasShip::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionHasSubstance::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionHazard::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionInteractionIndexChanged::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionHasMultiTool::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasMultiTool *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasProcTechnology::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasProcTechnology *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasProduct::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasProduct *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasShip::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasShip *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasSubstance::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasSubstance *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasTechnology::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasTechnology *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionInCombat::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionInCombat *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionInSeasonalUA::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionInSeasonalUA *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasProduct::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetPartHider::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSavedEntitlement *
        ]
    ),
    "cGcMissionConditionHasMultiTool::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasMultiTool *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasProcMissionForFaction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasProcMissionForFaction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasProcProduct::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasProcProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasProcTechnology::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasProcTechnology *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasProduct::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasSettlementBuilding::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasSettlementBuilding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasShip::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasShip *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasSubstance::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasSubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasTechnology::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasTechnology *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHazard::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHazard *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionInCombat::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionInCombat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionInSeasonalUA::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionInSeasonalUA *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionInUA::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionInUA *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionInteractionIndexChanged::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionInteractionIndexChanged *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasProcProduct::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasProcProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionHasProcProduct *
        ]
    ),
    "cGcMissionConditionHasProduct::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionHasProduct *
        ]
    ),
    "cGcMissionConditionHasSettlementBuilding::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasSettlementBuilding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionHasSettlementBuilding *
        ]
    ),
    "cGcMissionConditionHasShip::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasShip *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionHasShip *
        ]
    ),
    "cGcMissionConditionHasSubstance::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasSubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionHasSubstance *
        ]
    ),
    "cGcMissionConditionInSeasonalUA::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionInSeasonalUA *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionInSeasonalUA *
        ]
    ),
    "cGcMissionConditionInteractionIndexChanged::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionInteractionIndexChanged *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionInteractionIndexChanged *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionExtraSuitSlots const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionExtraSuitSlots *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionFactionRank const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionFactionRank *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionFeedingCreatures const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionFeedingCreatures *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionFreighterBattle const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionFreighterBattle *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionFrigateCount const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionFrigateCount *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionGameMode const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionGameMode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionGlobalMission const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionGlobalMission *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasAnySettlementBuildingInProgress const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasAnySettlementBuildingInProgress *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasCreatureEggItem const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasCreatureEggItem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasEndpointForEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasEndpointForEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasEntitlement const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasEntitlement *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasExocraft const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasExocraft *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasFreighter const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasFreighter *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasFuel const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasFuel *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasFuelForTakeoff const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasFuelForTakeoff *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasGalacticFeature const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasGalacticFeature *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasIllegalGoods const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasIllegalGoods *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasIngredientsForItem const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasIngredientsForItem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionHasMoney const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionHasMoney *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionExtraSuitSlots::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionExtraSuitSlots *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionFactionRank::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionFactionRank *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionFeedingCreatures::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionFeedingCreatures *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionForceHideMultiplayer::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcMissionConditionForceHideMultiplayer *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcMissionConditionFreighterBattle::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionFreighterBattle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionFrigateCount::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionFrigateCount *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionGameMode::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionGameMode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionGlobalMission::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionGlobalMission *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionGroup::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionGroup *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionGunOut::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionGunOut *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasActiveDetailMessage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasActiveDetailMessage *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasActiveStatsMessage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasActiveStatsMessage *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasAnySettlementBuildingInProgress::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasAnySettlementBuildingInProgress *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasCommunicatorSignal::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasCommunicatorSignal *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasCreatureEggItem::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasCreatureEggItem *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasEndpointForEvent::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasEndpointForEvent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasEntitlement::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasEntitlement *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasExocraft::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasExocraft *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasFreighter::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasFreighter *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasFuel::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasFuel *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasFuelForTakeoff::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasFuelForTakeoff *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasGalacticFeature::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasGalacticFeature *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasGrabbableTarget::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasGrabbableTarget *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasIllegalGoods::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasIllegalGoods *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasIngredientsForItem::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasIngredientsForItem *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasLegacyBasePending::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasLegacyBasePending *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasMoney::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionHasMoney *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionExtraSuitSlots::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionFactionRank::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionFeedingCreatures::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionForceHideMultiplayer::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionFreighterBattle::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionFrigateCount::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionGameMode::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionGlobalMission::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionGunOut::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasActiveDetailMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasActiveStatsMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasAnySettlementBuildingInProgress::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasCommunicatorSignal::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasCreatureEggItem::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasEndpointForEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasEntitlement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasExocraft::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasFreighter::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasFuel::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasFuelForTakeoff::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasGalacticFeature::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasGrabbableTarget::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasIllegalGoods::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasIngredientsForItem::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasLegacyBasePending::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionHasMoney::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionFactionRank::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionFreighterBattle::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionFrigateCount::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionGameMode::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionGlobalMission::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionGroup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionHasExocraft::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionHasFuel::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionHasGalacticFeature::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionHasIngredientsForItem::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionHasMoney::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionHasEndpointForEvent::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasEndpointForEvent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasFuel::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasFuel *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasIngredientsForItem::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasIngredientsForItem *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionExtraSuitSlots::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionExtraSuitSlots *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionFactionRank::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionFactionRank *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionFeedingCreatures::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionFeedingCreatures *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionFreighterBattle::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionFreighterBattle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionFrigateCount::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionFrigateCount *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionGameMode::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionGameMode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionGlobalMission::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionGlobalMission *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasCreatureEggItem::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasCreatureEggItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasEndpointForEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasEndpointForEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasExocraft::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasExocraft *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasFreighter::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasFreighter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasFuel::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasFuel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasGalacticFeature::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasGalacticFeature *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasIngredientsForItem::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasIngredientsForItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionHasMoney::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasMoney *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionFactionRank::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionFactionRank *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionFactionRank *
        ]
    ),
    "cGcMissionConditionHasExocraft::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasExocraft *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionHasExocraft *
        ]
    ),
    "cGcMissionConditionHasFuel::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasFuel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionHasFuel *
        ]
    ),
    "cGcMissionConditionHasMoney::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionHasMoney *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionHasMoney *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionBaseRequiresPower const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionBaseRequiresPower *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionBiomeType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionBiomeType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionCanSummonExocraft const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionCanSummonExocraft *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionCommunityResearchTier const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionCommunityResearchTier *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionConvertedFromSeason const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionConvertedFromSeason *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionCreatureOwned const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionCreatureOwned *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionCreatureReadyToLay const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionCreatureReadyToLay *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionCreatureSlots const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionCreatureSlots *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionCreatureTrust const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionCreatureTrust *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionCriticalMissionsDone const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionCriticalMissionsDone *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionCurrentSlope const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionCurrentSlope *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionDefaultItem const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionDefaultItem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionElevation const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionElevation *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionEventRequiresRGB const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionEventRequiresRGB *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionExpeditionContainsReward const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionExpeditionContainsReward *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionExpeditionCount const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionExpeditionCount *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionExpeditionNearlyOver const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionExpeditionNearlyOver *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionBaseRequiresPower::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionBaseRequiresPower *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionBiomeType::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionBiomeType *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionBuildMenuOpen::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionBuildMenuOpen *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCameraControlStealing::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionCameraControlStealing *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCanReceiveReward::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionCanReceiveReward *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCanSummonExocraft::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionCanSummonExocraft *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCommunityResearchTier::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionCommunityResearchTier *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionConvertedFromSeason::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionConvertedFromSeason *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCreatureOwned::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionCreatureOwned *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCreatureReadyToHatch::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionCreatureReadyToHatch *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCreatureReadyToLay::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionCreatureReadyToLay *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCreatureSlots::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionCreatureSlots *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCreatureSummoned::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionCreatureSummoned *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCreatureTrust::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionCreatureTrust *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCriticalMissionsDone::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionCriticalMissionsDone *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCurrentSlope::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionCurrentSlope *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionDamagedFrigateAtHome::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionDamagedFrigateAtHome *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionDefaultItem::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcMissionConditionDefaultItem *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcMissionConditionEggMachinePageOpen::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionEggMachinePageOpen *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionElevation::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionElevation *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionEventRequiresRGB::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionEventRequiresRGB *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionExocraftMoving::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionExocraftMoving *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionExpeditionContainsReward::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionExpeditionContainsReward *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionExpeditionCount::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionExpeditionCount *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionExpeditionNearlyOver::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionExpeditionNearlyOver *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionExpeditionProgress::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionExpeditionProgress *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionBaseRequiresPower::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionBiomeType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionBuildMenuOpen::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCameraControlStealing::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCanReceiveReward::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCanSummonExocraft::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCommunityResearchTier::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionConvertedFromSeason::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCreatureOwned::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCreatureReadyToHatch::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCreatureReadyToLay::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCreatureSlots::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCreatureSummoned::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCreatureTrust::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCriticalMissionsDone::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionCurrentSlope::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionDamagedFrigateAtHome::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionDefaultItem::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionEggMachinePageOpen::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionElevation::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionEventRequiresRGB::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionExocraftMoving::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionExpeditionContainsReward::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionExpeditionCount::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionExpeditionNearlyOver::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionExpeditionProgress::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionBiomeType::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionCanSummonExocraft::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionCommunityResearchTier::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionCreatureOwned::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionCreatureSlots::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionCreatureTrust::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionCurrentSlope::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionDefaultItem::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionEventRequiresRGB::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionExpeditionCount::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionCommunityResearchTier::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionCommunityResearchTier *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionCreatureOwned::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionCreatureOwned *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionElevation::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionElevation *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionEventRequiresRGB::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionEventRequiresRGB *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionExpeditionCount::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionExpeditionCount *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionBaseRequiresPower::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBaseRequiresPower *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionBiomeType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBiomeType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionCanSummonExocraft::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionCanSummonExocraft *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionCommunityResearchTier::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionCommunityResearchTier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionConvertedFromSeason::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionConvertedFromSeason *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionCreatureOwned::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionCreatureOwned *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionCreatureSlots::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionCreatureSlots *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionCreatureTrust::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionCreatureTrust *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionCurrentSlope::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionCurrentSlope *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionDefaultItem::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionDefaultItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionElevation::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionElevation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionEventRequiresRGB::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionEventRequiresRGB *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionExpeditionCount::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionExpeditionCount *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionExpeditionNearlyOver::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionExpeditionNearlyOver *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionBiomeType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBiomeType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionBiomeType *
        ]
    ),
    "cGcMissionConditionCanSummonExocraft::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionCanSummonExocraft *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionCanSummonExocraft *
        ]
    ),
    "cGcMissionConditionDefaultItem::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionDefaultItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionDefaultItem *
        ]
    ),
    "cGcMissionConditionEventRequiresRGB::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionEventRequiresRGB *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionEventRequiresRGB *
        ]
    ),
    "cGcMissionConditionExpeditionCount::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionExpeditionCount *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionExpeditionCount *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcModelExplosionRule>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcModelExplosionRule> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcBaseSearchFilter>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseSearchFilter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcModelExplosionRule const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcModelExplosionRule *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcBaseSearchFilter>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseSearchFilter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseSearchFilter *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionAIShipCount const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionAIShipCount *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionAbandonedFreighterExplored const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionAbandonedFreighterExplored *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionAbandonedOrEmptySystem const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionAbandonedOrEmptySystem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionAlienPodAggroed const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionAlienPodAggroed *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionAllMilestonesComplete const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionAllMilestonesComplete *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionBaseClaimed const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionBaseClaimed *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionBasePartBuilt const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionBasePartBuilt *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionBasePartNear const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionBasePartNear *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionBasePartsQuery const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionBasePartsQuery *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionBasePowerGenerated const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionBasePowerGenerated *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMissionConditionBaseQuery const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMissionConditionBaseQuery *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcModelExplosionRule const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcModelExplosionRule *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcModelExplosionRules const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcModelExplosionRules *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScanEventTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScanEventTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScanEventTableType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScanEventTableType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScanEventTriggers const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScanEventTriggers *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSceneSettings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSceneSettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpaceshipClasses const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpaceshipClasses *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeaponClasses const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeaponClasses *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcModelExplosionRule>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcModelExplosionRule> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcMissionConditionAIShipCount::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionAIShipCount *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionAbandonedFreighterExplored::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionAbandonedFreighterExplored *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionAbandonedOrEmptySystem::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionAbandonedOrEmptySystem *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionAimingTeleporter::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionAimingTeleporter *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionAlienPodAggroed::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionAlienPodAggroed *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionAllMilestonesComplete::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionAllMilestonesComplete *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionAreDroneHivePartsDestroyed::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionAreDroneHivePartsDestroyed *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionAutoPowerEnabled::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionAutoPowerEnabled *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionBaseClaimed::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionBaseClaimed *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionBasePartBuilt::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionBasePartBuilt *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionBasePartNear::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionBasePartNear *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionBasePartsQuery::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionBasePartsQuery *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionBasePowerGenerated::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionBasePowerGenerated *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionBaseQuery::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMissionConditionBaseQuery *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionAIShipCount::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionAbandonedFreighterExplored::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionAbandonedOrEmptySystem::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionAimingTeleporter::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionAlienPodAggroed::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionAllMilestonesComplete::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionAreDroneHivePartsDestroyed::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionAutoPowerEnabled::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionBaseClaimed::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionBasePartBuilt::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionBasePartNear::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionBasePartsQuery::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionBasePowerGenerated::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionBaseQuery::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcModelExplosionRule::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcModelExplosionRules::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScanEventTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScanEventTableType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScanEventTriggers::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSceneSettings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpaceshipClasses::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeaponClasses::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMissionConditionAIShipCount::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionBaseClaimed::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionBasePartBuilt::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionBasePartsQuery::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionBaseQuery::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcModelExplosionRules::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScanEventTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScanEventTriggers::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSceneSettings::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScanEventTable::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTable *
        ]
    ),
    "cGcScanEventTriggers::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTriggers *
        ]
    ),
    "cGcScanEventTable::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScanEventTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSceneSettings::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneSettings *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMissionConditionAreDroneHivePartsDestroyed::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionAreDroneHivePartsDestroyed *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionBaseClaimed::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBaseClaimed *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionBasePartBuilt::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBasePartBuilt *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionBasePartNear::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBasePartNear *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionBasePartsQuery::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBasePartsQuery *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionBasePowerGenerated::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBasePowerGenerated *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionBaseQuery::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBaseQuery *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcModelExplosionRule::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcModelExplosionRule *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcModelExplosionRules::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcModelExplosionRules *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventTableType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTableType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventTriggers::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTriggers *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSceneSettings::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneSettings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipClasses::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipClasses *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeaponClasses::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeaponClasses *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventTable::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTable *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcScanEventTriggers::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTriggers *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSpaceshipClasses::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipClasses *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcModelExplosionRule::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModelExplosionRule *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcModelExplosionRules::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModelExplosionRules *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventTableType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTableType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventTriggers::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTriggers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipClasses::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipClasses *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeaponClasses::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeaponClasses *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSceneSettings::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneSettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcModelExplosionRule::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModelExplosionRule *
        ]
    ),
    "cGcScanEventTriggers::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTriggers *
        ]
    ),
    "cGcSceneSettings::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneSettings *
        ]
    ),
    "cGcScanEventTable::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTable *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcScanEventTriggers::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTriggers *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcSpaceshipClasses::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipClasses *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcMissionConditionAIShipCount::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionAIShipCount *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionAbandonedFreighterExplored::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionAbandonedFreighterExplored *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionAbandonedOrEmptySystem::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionAbandonedOrEmptySystem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionAlienPodAggroed::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionAlienPodAggroed *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionAllMilestonesComplete::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionAllMilestonesComplete *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionBaseClaimed::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBaseClaimed *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionBasePartBuilt::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBasePartBuilt *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionBasePartNear::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBasePartNear *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionBasePartsQuery::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBasePartsQuery *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionBasePowerGenerated::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBasePowerGenerated *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionBaseQuery::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBaseQuery *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcModelExplosionRule::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModelExplosionRule *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcModelExplosionRules::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModelExplosionRules *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventTableType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTableType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventTriggers::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTriggers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSceneSettings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipClasses::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipClasses *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeaponClasses::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeaponClasses *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMissionConditionAIShipCount::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionAIShipCount *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionAIShipCount *
        ]
    ),
    "cGcMissionConditionBaseClaimed::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBaseClaimed *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionBaseClaimed *
        ]
    ),
    "cGcMissionConditionBasePartBuilt::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBasePartBuilt *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionBasePartBuilt *
        ]
    ),
    "cGcMissionConditionBasePartsQuery::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBasePartsQuery *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionBasePartsQuery *
        ]
    ),
    "cGcMissionConditionBaseQuery::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMissionConditionBaseQuery *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMissionConditionBaseQuery *
        ]
    ),
    "cGcModelExplosionRule::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModelExplosionRule *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcModelExplosionRule *
        ]
    ),
    "cGcScanEventTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventTableType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTableType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScanEventTableType *
        ]
    ),
    "cGcScanEventTriggers::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventTriggers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScanEventTriggers *
        ]
    ),
    "cGcSceneSettings::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSceneSettings *
        ]
    ),
    "cGcSpaceshipClasses::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipClasses *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSpaceshipClasses *
        ]
    ),
    "cGcWeaponClasses::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeaponClasses *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWeaponClasses *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcRegionHotspotBiomeGases>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspotBiomeGases *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcRegionHotspotData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspotData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcRegionHotspotSubstance>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRegionHotspotSubstance> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcScanEventSolarSystemLookup>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventSolarSystemLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPassiveFrigateIncome const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPassiveFrigateIncome *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcRegionHotspotBiomeGases const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcRegionHotspotBiomeGases *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcRegionHotspotData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcRegionHotspotData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcRegionHotspotSubstance const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcRegionHotspotSubstance *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcScanEventSolarSystemLookup>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScanEventSolarSystemLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScanEventSolarSystemLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPassiveFrigateIncomeArray const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPassiveFrigateIncomeArray *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPhysicsCollisionGroups const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPhysicsCollisionGroups *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcQuestItemPlacementRule const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcQuestItemPlacementRule *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRegionHotspotBiomeGases const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRegionHotspotBiomeGases *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRegionHotspotData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRegionHotspotData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRegionHotspotSubstance const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRegionHotspotSubstance *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRegionHotspotTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRegionHotspotTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRegionHotspotsTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRegionHotspotsTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRoomCountRule const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRoomCountRule *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRoomSequenceRule const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRoomSequenceRule *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScanEventData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScanEventData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScanEventSave const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScanEventSave *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScanEventSolarSystemLookup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScanEventSolarSystemLookup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcRegionHotspotData,6>::cTkFixedArray<cGcRegionHotspotData,6>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcRegionHotspotData,6> *
        ]
    ),
    "cTkDynamicArray<cGcRegionHotspotSubstance>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRegionHotspotSubstance> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcQuestItemPlacementRule::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcQuestItemPlacementRule *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRoomCountRule::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRoomCountRule *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRoomSequenceRule::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRoomSequenceRule *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcPassiveFrigateIncomeArray::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPhysicsCollisionGroups::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcQuestItemPlacementRule::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRegionHotspotBiomeGases::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRegionHotspotData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRegionHotspotSubstance::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRegionHotspotTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRegionHotspotsTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRoomCountRule::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRoomSequenceRule::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScanEventData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScanEventSave::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScanEventSolarSystemLookup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPassiveFrigateIncomeArray::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcQuestItemPlacementRule::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRegionHotspotsTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScanEventData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScanEventSave::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScanEventSolarSystemLookup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScanEventData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventData *
        ]
    ),
    "cGcScanEventSave::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventSave *
        ]
    ),
    "cGcScanEventData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScanEventSave::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventSave *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScanEventSolarSystemLookup::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventSolarSystemLookup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPassiveFrigateIncomeArray::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPassiveFrigateIncomeArray *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRegionHotspotsTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspotsTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScanEventData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcQuestItemPlacementRule::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuestItemPlacementRule *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionHotspotBiomeGases::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspotBiomeGases *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionHotspotData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspotData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionHotspotSubstance::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspotSubstance *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionHotspotsTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspotsTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRoomCountRule::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRoomCountRule *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRoomSequenceRule::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRoomSequenceRule *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventSave::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventSave *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventSolarSystemLookup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventSolarSystemLookup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcScanEventSave::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventSave *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcScanEventSolarSystemLookup::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventSolarSystemLookup *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPassiveFrigateIncomeArray::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPassiveFrigateIncomeArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionHotspotData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspotData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventSave::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventSave *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventSolarSystemLookup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventSolarSystemLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPassiveFrigateIncomeArray::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPassiveFrigateIncomeArray *
        ]
    ),
    "cGcRegionHotspotsTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspotsTable *
        ]
    ),
    "cGcScanEventData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventData *
        ]
    ),
    "cGcScanEventSave::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventSave *
        ]
    ),
    "cGcScanEventSolarSystemLookup::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventSolarSystemLookup *
        ]
    ),
    "cGcScanEventData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcScanEventSave::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventSave *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcScanEventSolarSystemLookup::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventSolarSystemLookup *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcPassiveFrigateIncomeArray::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPassiveFrigateIncomeArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPhysicsCollisionGroups::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhysicsCollisionGroups *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcQuestItemPlacementRule::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuestItemPlacementRule *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionHotspotBiomeGases::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspotBiomeGases *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionHotspotData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspotData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionHotspotSubstance::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspotSubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionHotspotTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspotTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRegionHotspotsTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspotsTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRoomCountRule::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRoomCountRule *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRoomSequenceRule::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRoomSequenceRule *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventSave::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventSave *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEventSolarSystemLookup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventSolarSystemLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPassiveFrigateIncomeArray::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPassiveFrigateIncomeArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcQuestItemPlacementRule::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcQuestItemPlacementRule *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcQuestItemPlacementRule *
        ]
    ),
    "cGcRegionHotspotBiomeGases::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspotBiomeGases *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRegionHotspotBiomeGases *
        ]
    ),
    "cGcRegionHotspotData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspotData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRegionHotspotData *
        ]
    ),
    "cGcRegionHotspotSubstance::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspotSubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRegionHotspotSubstance *
        ]
    ),
    "cGcRegionHotspotsTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRegionHotspotsTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRegionHotspotsTable *
        ]
    ),
    "cGcScanEventData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScanEventData *
        ]
    ),
    "cGcScanEventSave::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventSave *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScanEventSave *
        ]
    ),
    "cGcScanEventSolarSystemLookup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEventSolarSystemLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScanEventSolarSystemLookup *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcExpeditionEventData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExpeditionEventData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcExpeditionInterventionEventData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExpeditionInterventionEventData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcFreighterBaseOption>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFreighterBaseOption> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcFrigateFlybyLayout>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFrigateFlybyLayout> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcFrigateFlybyOption>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFrigateFlybyOption> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcFrigateStatRange>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateStatRange *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcFrigateStats>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateStats *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcFrigateTraitData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFrigateTraitData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcExpeditionCategoryStrength const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcExpeditionCategoryStrength *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcExpeditionEventData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcExpeditionEventData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcExpeditionInterventionEventData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcExpeditionInterventionEventData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFreighterBaseOption const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFreighterBaseOption *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFrigateFlybyLayout const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFrigateFlybyLayout *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFrigateFlybyOption const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFrigateFlybyOption *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFrigateStatRange const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFrigateStatRange *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFrigateStats const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFrigateStats *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFrigateTraitData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFrigateTraitData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFrigateTraitStrengthValues const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFrigateTraitStrengthValues *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcFreighterBaseOption>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFreighterBaseOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFreighterBaseOption *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcFrigateFlybyType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateFlybyType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFrigateFlybyType *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExpeditionEventOccurrenceRate const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExpeditionEventOccurrenceRate *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExpeditionEventTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExpeditionEventTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExpeditionInterventionEventData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExpeditionInterventionEventData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExpeditionPaymentToken const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExpeditionPaymentToken *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFreighterBaseOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFreighterBaseOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFreighterBaseOptions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFreighterBaseOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFreighterDungeonChoice const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFreighterDungeonChoice *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFreighterDungeonParams const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFreighterDungeonParams *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFrigateClassCost const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateClassCost *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFrigateFlybyLayout const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateFlybyLayout *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFrigateFlybyOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateFlybyOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFrigateFlybyTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateFlybyTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFrigateFlybyType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateFlybyType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFrigateStatRange const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateStatRange *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFrigateStats const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateStats *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFrigateStatsByClass const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateStatsByClass *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFrigateTraitData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateTraitData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFrigateTraitIcons const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateTraitIcons *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFrigateTraitStrengthByType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateTraitStrengthByType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFrigateTraitStrengthValues const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateTraitStrengthValues *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFrigateTraitTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateTraitTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFrigateUITraitLines const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateUITraitLines *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryClassCostMultiplier const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryClassCostMultiplier *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPassiveFrigateIncome const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPassiveFrigateIncome *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcFrigateStats,8>::cTkFixedArray<cGcFrigateStats,8>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcFrigateStats,8> *
        ]
    ),
    "cTkDynamicArray<cGcExpeditionEventData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExpeditionEventData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcExpeditionInterventionEventData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExpeditionInterventionEventData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcWeightedFilename>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWeightedFilename> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcFrigateFlybyLayout>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFrigateFlybyLayout> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcFrigateFlybyOption>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFrigateFlybyOption> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcFrigateTraitData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFrigateTraitData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcExpeditionEventOccurrenceRate::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExpeditionEventTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExpeditionInterventionEventData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExpeditionPaymentToken::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFreighterBaseOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFreighterBaseOptions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFreighterDungeonChoice::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFreighterDungeonParams::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateClassCost::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateFlybyLayout::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateFlybyOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateFlybyTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateFlybyType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateStatRange::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateStats::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateStatsByClass::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateTraitData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateTraitIcons::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateTraitStrengthByType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateTraitStrengthValues::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateTraitTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateUITraitLines::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryClassCostMultiplier::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPassiveFrigateIncome::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExpeditionEventOccurrenceRate::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExpeditionEventTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExpeditionInterventionEventData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFreighterBaseOptions::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFreighterDungeonParams::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFrigateFlybyLayout::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFrigateFlybyOption::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFrigateFlybyTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFrigateTraitData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFrigateTraitTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPassiveFrigateIncome::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFreighterDungeonParams::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterDungeonParams *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExpeditionEventOccurrenceRate::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventOccurrenceRate *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExpeditionEventTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFreighterDungeonParams::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterDungeonParams *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFrigateFlybyTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlybyTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFrigateTraitTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExpeditionInterventionEventData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionInterventionEventData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterBaseOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterBaseOptions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseOptions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterDungeonChoice::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterDungeonChoice *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterDungeonParams::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterDungeonParams *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateClassCost::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateClassCost *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateFlybyLayout::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlybyLayout *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateFlybyOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlybyOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateFlybyType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlybyType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateStatRange::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateStatRange *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateStats::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateStats *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateTraitData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateTraitStrengthValues::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitStrengthValues *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateUITraitLines::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateUITraitLines *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryClassCostMultiplier::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryClassCostMultiplier *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPassiveFrigateIncome::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPassiveFrigateIncome *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionEventOccurrenceRate::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventOccurrenceRate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionInterventionEventData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionInterventionEventData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionPaymentToken::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionPaymentToken *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterBaseOptions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterDungeonChoice::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterDungeonChoice *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterDungeonParams::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterDungeonParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateClassCost::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateClassCost *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateFlybyLayout::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlybyLayout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateFlybyOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlybyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateFlybyType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlybyType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateStatsByClass::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateStatsByClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateTraitData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateTraitIcons::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitIcons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateTraitStrengthByType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitStrengthByType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateTraitStrengthValues::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitStrengthValues *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateUITraitLines::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateUITraitLines *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryClassCostMultiplier::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryClassCostMultiplier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPassiveFrigateIncome::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPassiveFrigateIncome *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionEventOccurrenceRate::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventOccurrenceRate *
        ]
    ),
    "cGcExpeditionInterventionEventData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionInterventionEventData *
        ]
    ),
    "cGcWeightedFilename::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedFilename *
        ]
    ),
    "cGcFreighterDungeonParams::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterDungeonParams *
        ]
    ),
    "cGcFrigateStatsByClass::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateStatsByClass *
        ]
    ),
    "cGcFrigateTraitIcons::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitIcons *
        ]
    ),
    "cGcFrigateTraitStrengthByType::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitStrengthByType *
        ]
    ),
    "cGcFrigateTraitStrengthValues::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitStrengthValues *
        ]
    ),
    "cGcFrigateUITraitLines::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalacticAddressData *
        ]
    ),
    "cGcPassiveFrigateIncome::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPassiveFrigateIncome *
        ]
    ),
    "cGcExpeditionEventOccurrenceRate::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventOccurrenceRate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionEventTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionInterventionEventData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionInterventionEventData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionPaymentToken::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionPaymentToken *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterBaseOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterBaseOptions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterDungeonChoice::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterDungeonChoice *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterDungeonParams::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterDungeonParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateClassCost::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateClassCost *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateFlybyLayout::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlybyLayout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateFlybyOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlybyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateFlybyTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlybyTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateFlybyType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlybyType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateStatRange::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateStatRange *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateStats::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateStats *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateStatsByClass::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateStatsByClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateTraitData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateTraitIcons::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitIcons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateTraitStrengthByType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitStrengthByType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateTraitStrengthValues::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitStrengthValues *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateTraitTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateUITraitLines::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateUITraitLines *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryClassCostMultiplier::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryClassCostMultiplier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPassiveFrigateIncome::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPassiveFrigateIncome *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionEventOccurrenceRate::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventOccurrenceRate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionInterventionEventData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionInterventionEventData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcExpeditionInterventionEventData *
        ]
    ),
    "cGcFreighterBaseOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFreighterBaseOption *
        ]
    ),
    "cGcFreighterDungeonParams::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterDungeonParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFreighterDungeonParams *
        ]
    ),
    "cGcFrigateClassCost::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateClassCost *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateFlybyLayout::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlybyLayout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFrigateFlybyLayout *
        ]
    ),
    "cGcFrigateFlybyOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlybyOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFrigateFlybyOption *
        ]
    ),
    "cGcFrigateFlybyType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateFlybyType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFrigateFlybyType *
        ]
    ),
    "cGcFrigateStatRange::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateStatRange *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFrigateStatRange *
        ]
    ),
    "cGcFrigateStats::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateStats *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFrigateStats *
        ]
    ),
    "cGcFrigateStatsByClass::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateStatsByClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateTraitData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFrigateTraitData *
        ]
    ),
    "cGcFrigateTraitIcons::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitIcons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateTraitStrengthByType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitStrengthByType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateTraitStrengthValues::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitStrengthValues *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFrigateTraitStrengthValues *
        ]
    ),
    "cGcFrigateUITraitLines::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateUITraitLines *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFrigateUITraitLines *
        ]
    ),
    "cGcInventoryClassCostMultiplier::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryClassCostMultiplier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPassiveFrigateIncome::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPassiveFrigateIncome *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPassiveFrigateIncome *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcAIShipDebugSpawnData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAIShipDebugSpawnData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureDebugSpawnData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureDebugSpawnData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDebugCamera>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDebugCamera> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDebugShipTravelLine>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDebugShipTravelLine> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDungeonQuestParams>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDungeonQuestParams> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDungeonRoomParams>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDungeonRoomParams> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcExperienceDebugTriggerInput>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExperienceDebugTriggerInput> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcMechDebugSpawnData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMechDebugSpawnData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNPCDebugSpawnData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCDebugSpawnData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcAsteroidGenerationData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGenerationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadXML<uint>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAIShipDebugSpawnData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAIShipDebugSpawnData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCameraFollowSettings const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCameraFollowSettings *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureDebugSpawnData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureDebugSpawnData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDebugCamera const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDebugCamera *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDebugPlanetPos const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDebugPlanetPos *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDebugShipTravelLine const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDebugShipTravelLine *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDungeonQuestParams const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDungeonQuestParams *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDungeonRoomParams const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDungeonRoomParams *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcExperienceDebugTriggerInput const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcExperienceDebugTriggerInput *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMechDebugSpawnData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMechDebugSpawnData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCDebugSpawnData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCDebugSpawnData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPetCustomisationData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPetCustomisationData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPetData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPetData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcAsteroidGenerationData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAsteroidGenerationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAsteroidGenerationData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcDungeonRoomParams>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDungeonRoomParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDungeonRoomParams *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcGalaxyWaypointTypes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyWaypointTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyWaypointTypes *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAsteroidGenerationData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAsteroidGenerationData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAsteroidSystemGenerationData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAsteroidSystemGenerationData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDebugScene const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDebugScene *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDungeonGenerationParams const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDungeonQuestParams const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDungeonQuestParams *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDungeonRoomParams const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDungeonRoomParams *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExpeditionCategoryStrength const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExpeditionCategoryStrength *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExpeditionDebriefPunctuation const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExpeditionDebriefPunctuation *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExpeditionDifficultyKeyframe const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExpeditionDifficultyKeyframe *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExpeditionDurationValues const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExpeditionDurationValues *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExpeditionEventData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExpeditionEventData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalaxyWaypoint const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyWaypoint *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalaxyWaypointTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyWaypointTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCDebugSpawnData::cGcNPCDebugSpawnData": {
        "cGcNPCDebugSpawnData *, cGcNPCDebugSpawnData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNPCDebugSpawnData *
                ctypes.c_ulonglong,  # cGcNPCDebugSpawnData *
            ]
        ),
        "cGcNPCDebugSpawnData *, const cGcNPCDebugSpawnData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcNPCDebugSpawnData *
                ctypes.c_ulonglong,  # const cGcNPCDebugSpawnData *
            ]
        ),
    },
    "cGcDebugScene::~cGcDebugScene": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugScene *
        ]
    ),
    "cGcNPCDebugSpawnData::~cGcNPCDebugSpawnData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCDebugSpawnData *
        ]
    ),
    "cTkDynamicArray<cGcCreatureDebugSpawnData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureDebugSpawnData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcDebugCamera>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDebugCamera> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcDebugShipTravelLine>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDebugShipTravelLine> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcExperienceDebugTriggerInput>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExperienceDebugTriggerInput> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcMechDebugSpawnData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMechDebugSpawnData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcNPCDebugSpawnData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCDebugSpawnData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAsteroidGenerationData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAsteroidSystemGenerationData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDebugScene::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDungeonGenerationParams::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDungeonQuestParams::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDungeonRoomParams::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExpeditionCategoryStrength::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExpeditionDebriefPunctuation::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExpeditionDifficultyKeyframe::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExpeditionDurationValues::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExpeditionEventData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGalaxyWaypoint::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGalaxyWaypointTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDebugScene::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDungeonGenerationParams::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDungeonQuestParams::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExpeditionEventData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGalaxyWaypoint::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcMechDebugSpawnData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMechDebugSpawnData> *
        ]
    ),
    "cGcDebugScene::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugScene *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDungeonGenerationParams::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDungeonGenerationParams *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExpeditionEventData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDebugScene::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugScene *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDungeonGenerationParams::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDungeonGenerationParams *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAsteroidGenerationData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGenerationData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugScene::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugScene *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDungeonGenerationParams::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDungeonGenerationParams *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDungeonQuestParams::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDungeonQuestParams *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDungeonRoomParams::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDungeonRoomParams *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionCategoryStrength::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionCategoryStrength *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionEventData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyWaypoint::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyWaypoint *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyWaypointTypes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyWaypointTypes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyWaypoint::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyWaypoint *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcAsteroidGenerationData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGenerationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAsteroidSystemGenerationData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidSystemGenerationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugScene::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugScene *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDungeonGenerationParams::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionCategoryStrength::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionCategoryStrength *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionDebriefPunctuation::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionDebriefPunctuation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionDifficultyKeyframe::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionDifficultyKeyframe *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionDurationValues::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionDurationValues *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionEventData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyWaypointTypes::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyWaypointTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugScene::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugScene *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAsteroidGenerationData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGenerationData *
        ]
    ),
    "cGcAsteroidSystemGenerationData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidSystemGenerationData *
        ]
    ),
    "cGcDebugScene::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugScene *
        ]
    ),
    "cGcDungeonGenerationParams::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDungeonGenerationParams *
        ]
    ),
    "cGcDungeonQuestParams::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMusicSettings *
        ]
    ),
    "cGcDungeonRoomParams::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDungeonRoomParams *
        ]
    ),
    "cGcExpeditionCategoryStrength::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionCategoryStrength *
        ]
    ),
    "cGcPunctuationDelay::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPunctuationDelay *
        ]
    ),
    "cGcExpeditionEventData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventData *
        ]
    ),
    "cGcGalaxyWaypoint::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyWaypoint *
        ]
    ),
    "cGcGalaxyWaypoint::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyWaypoint *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcAsteroidGenerationData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGenerationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAsteroidSystemGenerationData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidSystemGenerationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugScene::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugScene *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDungeonGenerationParams::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDungeonQuestParams::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDungeonQuestParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDungeonRoomParams::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDungeonRoomParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionCategoryStrength::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionCategoryStrength *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionDebriefPunctuation::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionDebriefPunctuation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionDifficultyKeyframe::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionDifficultyKeyframe *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionDurationValues::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionDurationValues *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionEventData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyWaypoint::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyWaypoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyWaypointTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyWaypointTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAsteroidGenerationData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAsteroidGenerationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAsteroidGenerationData *
        ]
    ),
    "cGcDebugScene::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugScene *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDebugScene *
        ]
    ),
    "cGcDungeonGenerationParams::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDungeonGenerationParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDungeonGenerationParams *
        ]
    ),
    "cGcDungeonQuestParams::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDungeonQuestParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDungeonQuestParams *
        ]
    ),
    "cGcDungeonRoomParams::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDungeonRoomParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDungeonRoomParams *
        ]
    ),
    "cGcExpeditionDurationValues::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionDurationValues *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionEventData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcExpeditionEventData *
        ]
    ),
    "cGcGalaxyWaypoint::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyWaypoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyWaypoint *
        ]
    ),
    "cGcGalaxyWaypointTypes::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyWaypointTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyWaypointTypes *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkTextureResource>": {
        "cTkTextureResource *, int, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkTextureResource *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkDynamicArray<cTkTextureResource> *, XMLNode *, const char *, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDynamicArray<cTkTextureResource> *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cTkMetaDataXML::ReadClassXML<cGcGalaxySolarSystemOrbitParams>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxySolarSystemOrbitParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadXML<cTkVector4>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector4 *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<cTkSeed>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlanetSize const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlanetSize *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkTextureResource const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkTextureResource *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcGalaxySolarSystemOrbitParams>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxySolarSystemOrbitParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxySolarSystemOrbitParams *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcGalaxyStarAnomaly>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyStarAnomaly *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyStarAnomaly *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalaxyInfoIcons const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyInfoIcons *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalaxyMarkerSettings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyMarkerSettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalaxyMarkerTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyMarkerTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalaxyRenderAnostreakData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyRenderAnostreakData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalaxyRenderSetupData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyRenderSetupData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalaxySolarSystemOrbitParams const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxySolarSystemOrbitParams *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalaxySolarSystemParams const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxySolarSystemParams *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalaxyStarAnomaly const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyStarAnomaly *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalaxyStarAttributesData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyStarAttributesData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalaxyStarColours const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyStarColours *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalaxyStarTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyStarTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalaxyVoxelAttributesData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyVoxelAttributesData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cTkTextureResource,4>::cTkFixedArray<cTkTextureResource,4>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkTextureResource,4> *
        ]
    ),
    "cTkFixedArray<cTkTextureResource,7>::cTkFixedArray<cTkTextureResource,7>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkTextureResource,7> *
        ]
    ),
    "cTkFixedArray<cTkTextureResource,8>::cTkFixedArray<cTkTextureResource,8>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkTextureResource,8> *
        ]
    ),
    "cGcGalaxyInfoIcons::~cGcGalaxyInfoIcons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyInfoIcons *
        ]
    ),
    "cGcGalaxyInfoIcons::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGalaxyMarkerSettings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGalaxyMarkerTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGalaxyRenderAnostreakData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGalaxyRenderSetupData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGalaxySolarSystemOrbitParams::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGalaxySolarSystemParams::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGalaxyStarAnomaly::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGalaxyStarAttributesData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGalaxyStarColours::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGalaxyStarTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGalaxyVoxelAttributesData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGalaxyInfoIcons::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGalaxyStarAttributesData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGalaxyInfoIcons::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyInfoIcons *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGalaxyInfoIcons::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyInfoIcons *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyMarkerSettings::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMarkerSettings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyRenderAnostreakData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyRenderAnostreakData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyRenderSetupData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyRenderSetupData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxySolarSystemOrbitParams::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxySolarSystemOrbitParams *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxySolarSystemParams::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxySolarSystemParams *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyStarAnomaly::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarAnomaly *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyStarAttributesData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarAttributesData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyStarColours::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarColours *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyStarTypes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarTypes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyVoxelAttributesData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelAttributesData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyRenderSetupData::Interpolate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyRenderSetupData *
            ctypes.c_ulonglong,  # const cGcGalaxyRenderSetupData *
            ctypes.c_ulonglong,  # cGcGalaxyRenderSetupData *
            ctypes.c_double,  # double
        ]
    ),
    "cGcGalaxyStarAnomaly::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarAnomaly *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcGalaxyStarTypes::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarTypes *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcGalaxyMarkerSettings::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMarkerSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyRenderAnostreakData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyRenderAnostreakData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyRenderSetupData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyRenderSetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxySolarSystemOrbitParams::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxySolarSystemOrbitParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxySolarSystemParams::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxySolarSystemParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyStarAnomaly::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarAnomaly *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyStarAttributesData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarAttributesData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyStarColours::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarColours *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyStarTypes::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyInfoIcons::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyInfoIcons *
        ]
    ),
    "cGcGalaxyMarkerSettings::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMarkerSettings *
        ]
    ),
    "cGcGalaxyRenderAnostreakData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyRenderAnostreakData *
        ]
    ),
    "cGcGalaxyRenderSetupData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyRenderSetupData *
        ]
    ),
    "cGcGalaxySolarSystemParams::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxySolarSystemParams *
        ]
    ),
    "cGcGalaxyStarAttributesData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarAttributesData *
        ]
    ),
    "cGcGalaxyStarColours::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarColours *
        ]
    ),
    "cGcGalaxyVoxelAttributesData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelAttributesData *
        ]
    ),
    "cGcGalaxyStarAnomaly::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarAnomaly *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcGalaxyStarTypes::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarTypes *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcGalaxyInfoIcons::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyInfoIcons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyMarkerSettings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMarkerSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyMarkerTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMarkerTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyRenderAnostreakData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyRenderAnostreakData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyRenderSetupData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyRenderSetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxySolarSystemOrbitParams::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxySolarSystemOrbitParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxySolarSystemParams::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxySolarSystemParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyStarAnomaly::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarAnomaly *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyStarAttributesData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarAttributesData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyStarColours::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarColours *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyStarTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyVoxelAttributesData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelAttributesData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyMarkerSettings::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyMarkerSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyMarkerSettings *
        ]
    ),
    "cGcGalaxyRenderAnostreakData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyRenderAnostreakData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyRenderAnostreakData *
        ]
    ),
    "cGcGalaxyRenderSetupData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyRenderSetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyRenderSetupData *
        ]
    ),
    "cGcGalaxySolarSystemOrbitParams::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxySolarSystemOrbitParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxySolarSystemOrbitParams *
        ]
    ),
    "cGcGalaxySolarSystemParams::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxySolarSystemParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxySolarSystemParams *
        ]
    ),
    "cGcGalaxyStarAnomaly::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarAnomaly *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyStarAnomaly *
        ]
    ),
    "cGcGalaxyStarAttributesData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarAttributesData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyStarAttributesData *
        ]
    ),
    "cGcGalaxyStarColours::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarColours *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyStarTypes::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyStarTypes *
        ]
    ),
    "cGcGalaxyVoxelAttributesData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyVoxelAttributesData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyVoxelAttributesData *
        ]
    ),
    "cTkMetaDataXML::ReadArrayXML<__int64>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<__int64> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcFreighterBaseRoom>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFreighterBaseRoom> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcIDPair>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcIDPair> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcMinimumUseConstraint>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMinimumUseConstraint> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcModuleOverride>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcModuleOverride> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcWFCModulePrototype>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWFCModulePrototype> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcWFCTerrainConstraint>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWFCTerrainConstraint> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcWeightedBuildingSize>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWeightedBuildingSize> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcWeightedResource>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWeightedResource> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcWFCFace>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCFace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<__int64>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const __int64 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFreighterBaseRoom const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFreighterBaseRoom *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcIDPair const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcIDPair *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMinimumUseConstraint const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMinimumUseConstraint *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcModuleOverride const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcModuleOverride *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcWFCModulePrototype const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcWFCModulePrototype *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcWFCTerrainConstraint const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcWFCTerrainConstraint *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcWeightedBuildingSize const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcWeightedBuildingSize *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcWeightedResource const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcWeightedResource *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcWFCFace>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWFCFace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWFCFace *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalaxyAudioSetupData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyAudioSetupData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalaxyCameraData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyCameraData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalaxyGenerationSetupData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyGenerationSetupData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWFCBuilding const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWFCBuilding *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWFCFace const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWFCFace *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWFCModulePrototype const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWFCModulePrototype *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWFCModuleSet const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWFCModuleSet *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWFCTerrainConstraint const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWFCTerrainConstraint *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCFace::cGcWFCFace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCFace *
            ctypes.c_ulonglong,  # cGcWFCFace *
        ]
    ),
    "cTkDynamicArray<cGcMinimumUseConstraint>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMinimumUseConstraint> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcModuleOverride>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcModuleOverride> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcWFCModulePrototype>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWFCModulePrototype> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcWeightedBuildingSize>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWeightedBuildingSize> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcGalaxyAudioSetupData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGalaxyCameraData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGalaxyGenerationSetupData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWFCBuilding::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWFCFace::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWFCModulePrototype::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWFCModuleSet::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWFCTerrainConstraint::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGalaxyAudioSetupData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWFCBuilding::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWFCFace::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWFCModulePrototype::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWFCModuleSet::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcMinimumUseConstraint>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMinimumUseConstraint> *
        ]
    ),
    "cGcWFCBuilding::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBuilding *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWFCModulePrototype::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCModulePrototype *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGalaxyAudioSetupData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyAudioSetupData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWFCBuilding::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBuilding *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWFCModulePrototype::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCModulePrototype *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWFCModuleSet::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCModuleSet *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGalaxyAudioSetupData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyAudioSetupData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyCameraData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyCameraData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyGenerationSetupData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyGenerationSetupData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCBuilding::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBuilding *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCFace::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCFace *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCModulePrototype::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCModulePrototype *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCModuleSet::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCModuleSet *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCTerrainConstraint::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCTerrainConstraint *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyGenerationSetupData::Interpolate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyGenerationSetupData *
            ctypes.c_ulonglong,  # const cGcGalaxyGenerationSetupData *
            ctypes.c_ulonglong,  # cGcGalaxyGenerationSetupData *
            ctypes.c_double,  # double
        ]
    ),
    "cGcGalaxyAudioSetupData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyAudioSetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyCameraData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyCameraData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyGenerationSetupData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyGenerationSetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCBuilding::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBuilding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCFace::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCFace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCModulePrototype::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCModulePrototype *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCTerrainConstraint::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCTerrainConstraint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyAudioSetupData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyAudioSetupData *
        ]
    ),
    "cGcGalaxyCameraData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyCameraData *
        ]
    ),
    "cGcGalaxyGenerationSetupData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyGenerationSetupData *
        ]
    ),
    "cGcWFCBuilding::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBuilding *
        ]
    ),
    "cGcWFCFace::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCFace *
        ]
    ),
    "cGcWFCModulePrototype::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCModulePrototype *
        ]
    ),
    "cGcWFCModuleSet::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCModuleSet *
        ]
    ),
    "cGcGalaxyAudioSetupData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyAudioSetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyCameraData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyCameraData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyGenerationSetupData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyGenerationSetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCBuilding::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBuilding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCFace::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCFace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCModulePrototype::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCModulePrototype *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCModuleSet::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCModuleSet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCTerrainConstraint::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCTerrainConstraint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyAudioSetupData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyAudioSetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyAudioSetupData *
        ]
    ),
    "cGcGalaxyCameraData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyCameraData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyCameraData *
        ]
    ),
    "cGcGalaxyGenerationSetupData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyGenerationSetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyGenerationSetupData *
        ]
    ),
    "cGcWFCBuilding::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCBuilding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWFCBuilding *
        ]
    ),
    "cGcWFCFace::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCFace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWFCFace *
        ]
    ),
    "cGcWFCModulePrototype::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCModulePrototype *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWFCModulePrototype *
        ]
    ),
    "cGcWFCModuleSet::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCModuleSet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWFCModuleSet *
        ]
    ),
    "cGcWFCTerrainConstraint::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCTerrainConstraint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWFCTerrainConstraint *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBuildingColourPalette>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBuildingColourPalette> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSettlementColourPalette>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementColourPalette> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcWFCDecorationItem>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWFCDecorationItem> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcWeightedColourId>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWeightedColourId> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcWFCDecorationFace>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCDecorationFace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBuildingColourPalette const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBuildingColourPalette *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSettlementColourPalette const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSettlementColourPalette *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcWFCDecorationItem const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcWFCDecorationItem *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcWeightedColourId const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcWeightedColourId *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcWFCDecorationFace>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWFCDecorationFace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWFCDecorationFace *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingColourPalette const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingColourPalette *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFreighterBaseRoom const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFreighterBaseRoom *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcIDPair const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcIDPair *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMinimumUseConstraint const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMinimumUseConstraint *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcModuleOverride const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcModuleOverride *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementColourPalette const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementColourPalette *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementColourTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementColourTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWFCDecorationFace const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWFCDecorationFace *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWFCDecorationItem const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWFCDecorationItem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWFCDecorationSet const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWFCDecorationSet *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWFCDecorationTheme const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWFCDecorationTheme *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeatherHazardLightningData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherHazardLightningData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeatherHazardMeteorData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherHazardMeteorData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeatherHazardTornadoData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherHazardTornadoData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeightedBuildingSize const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeightedBuildingSize *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeightedColourId const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeightedColourId *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeightedResource const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeightedResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCDecorationItem::cGcWFCDecorationItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCDecorationItem *
            ctypes.c_ulonglong,  # cGcWFCDecorationItem *
        ]
    ),
    "cTkDynamicArray<cGcBuildingColourPalette>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBuildingColourPalette> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSettlementColourPalette>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementColourPalette> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcWFCDecorationItem>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWFCDecorationItem> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcWeightedColourId>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWeightedColourId> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcWeatherEffectLightningData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcWeatherEffectLightningData *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcWeatherHazardLightningData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcWeatherHazardLightningData *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcWeatherHazardMeteorData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcWeatherHazardMeteorData *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcWeatherHazardTornadoData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcWeatherHazardTornadoData *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcBuildingColourPalette::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFreighterBaseRoom::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcIDPair::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMinimumUseConstraint::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcModuleOverride::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementColourPalette::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementColourTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWFCDecorationFace::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWFCDecorationItem::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWFCDecorationSet::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWFCDecorationTheme::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeatherEffectLightningData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeatherHazardLightningData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeatherHazardMeteorData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeatherHazardTornadoData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeightedBuildingSize::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeightedColourId::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeightedResource::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingColourPalette::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMinimumUseConstraint::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcModuleOverride::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementColourPalette::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementColourTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWFCDecorationItem::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWFCDecorationSet::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWeatherHazardLightningData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWeatherHazardMeteorData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWeightedResource::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcSettlementColourPalette>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementColourPalette> *
        ]
    ),
    "cGcModuleOverride::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleOverride *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementColourPalette::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementColourPalette *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWFCDecorationItem::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCDecorationItem *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcModuleOverride::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleOverride *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementColourPalette::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementColourPalette *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementColourTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementColourTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWFCDecorationItem::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCDecorationItem *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWFCDecorationSet::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCDecorationSet *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBuildingColourPalette::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingColourPalette *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterBaseRoom::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseRoom *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcIDPair::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcIDPair *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMinimumUseConstraint::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMinimumUseConstraint *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcModuleOverride::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleOverride *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementColourPalette::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementColourPalette *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementColourTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementColourTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCDecorationFace::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCDecorationFace *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCDecorationItem::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCDecorationItem *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherHazardLightningData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherHazardLightningData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherHazardMeteorData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherHazardMeteorData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherHazardTornadoData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherHazardTornadoData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeightedBuildingSize::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedBuildingSize *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeightedColourId::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedColourId *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeightedResource::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedResource *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingColourPalette::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingColourPalette *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterBaseRoom::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseRoom *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcIDPair::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIDPair *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMinimumUseConstraint::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMinimumUseConstraint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcModuleOverride::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementColourPalette::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementColourPalette *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCDecorationFace::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCDecorationFace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCDecorationItem::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCDecorationItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherHazardMeteorData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherHazardMeteorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeightedBuildingSize::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedBuildingSize *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeightedResource::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedResource *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementGiftDetails::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingSubGroup *
        ]
    ),
    "cGcMinimumUseConstraint::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMinimumUseConstraint *
        ]
    ),
    "cGcModuleOverride::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleOverride *
        ]
    ),
    "cGcFontData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFontData *
        ]
    ),
    "cGcWFCDecorationItem::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCDecorationItem *
        ]
    ),
    "cGcWeatherHazardLightningData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherHazardLightningData *
        ]
    ),
    "cGcWeatherHazardMeteorData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherHazardMeteorData *
        ]
    ),
    "cGcWeightedBuildingSize::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedBuildingSize *
        ]
    ),
    "cGcWeightedResource::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedResource *
        ]
    ),
    "cGcBuildingColourPalette::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingColourPalette *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterBaseRoom::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseRoom *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcIDPair::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIDPair *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMinimumUseConstraint::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMinimumUseConstraint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcModuleOverride::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModuleOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementColourPalette::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementColourPalette *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementColourTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementColourTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCDecorationFace::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCDecorationFace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCDecorationItem::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCDecorationItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCDecorationSet::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCDecorationSet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWFCDecorationTheme::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCDecorationTheme *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherHazardLightningData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherHazardLightningData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherHazardMeteorData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherHazardMeteorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherHazardTornadoData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherHazardTornadoData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeightedBuildingSize::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedBuildingSize *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeightedColourId::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedColourId *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeightedResource::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedResource *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingColourPalette::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingColourPalette *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBuildingColourPalette *
        ]
    ),
    "cGcSettlementColourPalette::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementColourPalette *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementColourPalette *
        ]
    ),
    "cGcWFCDecorationFace::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCDecorationFace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWFCDecorationFace *
        ]
    ),
    "cGcWFCDecorationItem::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWFCDecorationItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWFCDecorationItem *
        ]
    ),
    "cGcWeatherHazardLightningData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherHazardLightningData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWeatherHazardLightningData *
        ]
    ),
    "cGcWeatherHazardMeteorData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherHazardMeteorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWeatherHazardMeteorData *
        ]
    ),
    "cGcWeatherHazardTornadoData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherHazardTornadoData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWeatherHazardTornadoData *
        ]
    ),
    "cGcWeightedBuildingSize::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedBuildingSize *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWeightedBuildingSize *
        ]
    ),
    "cGcWeightedColourId::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedColourId *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWeightedColourId *
        ]
    ),
    "cGcWeightedResource::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedResource *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWeightedResource *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcHazardValues>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHazardValues *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcScreenFilters>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcScreenFilters> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSelectableObjectData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSelectableObjectData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSelectableObjectList>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSelectableObjectList> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSelectableObjectSpawnData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSelectableObjectSpawnData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcStormProperties>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStormProperties> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcWeatherEffect>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWeatherEffect> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcWeightedFilename>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWeightedFilename> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcColourModifier>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcColourModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcFogProperties>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFogProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcLightShaftProperties>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLightShaftProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcColourModifier const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcColourModifier *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcHazardValues const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcHazardValues *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcScreenFilters const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcScreenFilters *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSelectableObjectData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSelectableObjectData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSelectableObjectList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSelectableObjectList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSelectableObjectSpawnData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSelectableObjectSpawnData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcStormProperties const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcStormProperties *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcWeatherEffect const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcWeatherEffect *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcWeightedFilename const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcWeightedFilename *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcColourModifier>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcColourModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcColourModifier *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcFogProperties>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFogProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFogProperties *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcHazardValues>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHazardValues *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcHazardValues *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcLightShaftProperties>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcLightShaftProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcLightShaftProperties *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHazardModifiers const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHazardModifiers *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHazardValueTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHazardValueTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHazardValues const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHazardValues *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcObjectPlacementCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcObjectPlacementCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSelectableObjectData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSelectableObjectData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSelectableObjectList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSelectableObjectList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSelectableObjectSpawnData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSelectableObjectSpawnData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSelectableObjectSpawnList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSelectableObjectSpawnList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSelectableObjectTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSelectableObjectTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeatherColourModifiers const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherColourModifiers *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeatherEffect const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherEffect *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeatherEffectTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherEffectTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeatherOptions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeatherProperties const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherProperties *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeatherTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeatherTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcSelectableObjectData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSelectableObjectData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSelectableObjectList>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSelectableObjectList> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcStormProperties>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStormProperties> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcWeatherEffect>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWeatherEffect> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcHazardModifiers::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHazardValueTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHazardValues::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcObjectPlacementCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSelectableObjectData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSelectableObjectList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSelectableObjectSpawnData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSelectableObjectSpawnList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSelectableObjectTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeatherColourModifiers::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeatherEffect::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeatherEffectTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeatherOptions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeatherProperties::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeatherTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSelectableObjectList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSelectableObjectSpawnData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSelectableObjectSpawnList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSelectableObjectTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWeatherColourModifiers::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWeatherEffect::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWeatherEffectTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWeatherProperties::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSelectableObjectSpawnList::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectSpawnList *
        ]
    ),
    "cGcSelectableObjectSpawnList::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectSpawnList *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWeatherColourModifiers::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherColourModifiers *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWeatherEffect::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherEffect *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSelectableObjectSpawnList::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectSpawnList *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSelectableObjectTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWeatherEffect::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherEffect *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWeatherEffectTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherEffectTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcWeatherProperties::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherProperties *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcHazardValues::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHazardValues *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSelectableObjectData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSelectableObjectList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSelectableObjectSpawnData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectSpawnData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSelectableObjectSpawnList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectSpawnList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherColourModifiers::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherColourModifiers *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherEffect::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherEffect *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherOptions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherOptions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherProperties::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherProperties *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSelectableObjectList::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSelectableObjectSpawnData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSelectableObjectSpawnList::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectSpawnList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherColourModifiers::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherColourModifiers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherEffect::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherOptions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherProperties::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSelectableObjectSpawnData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectSpawnData *
        ]
    ),
    "cGcSentinelSpawnNamedSequence::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnNamedSequence *
        ]
    ),
    "cGcWeatherColourModifiers::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherColourModifiers *
        ]
    ),
    "cGcWeatherEffect::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherEffect *
        ]
    ),
    "cGcWeatherProperties::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherProperties *
        ]
    ),
    "cGcWeatherTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherTable *
        ]
    ),
    "cGcHazardModifiers::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHazardModifiers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHazardValueTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHazardValueTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHazardValues::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHazardValues *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectPlacementCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectPlacementCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSelectableObjectData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSelectableObjectList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSelectableObjectSpawnData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSelectableObjectSpawnList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectSpawnList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSelectableObjectTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherColourModifiers::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherColourModifiers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherEffect::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherEffectTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherEffectTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherOptions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherProperties::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeatherTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHazardValues::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHazardValues *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcHazardValues *
        ]
    ),
    "cGcSelectableObjectList::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSelectableObjectList *
        ]
    ),
    "cGcSelectableObjectSpawnData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSelectableObjectSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSelectableObjectSpawnData *
        ]
    ),
    "cGcWeatherColourModifiers::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherColourModifiers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWeatherColourModifiers *
        ]
    ),
    "cGcWeatherEffect::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWeatherEffect *
        ]
    ),
    "cGcWeatherOptions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWeatherOptions *
        ]
    ),
    "cGcWeatherProperties::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeatherProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWeatherProperties *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkPaletteTexture>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkPaletteTexture> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkPaletteTexture const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkPaletteTexture *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetSkyProperties const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetSkyProperties *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcResourceCollectEffect const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcResourceCollectEffect *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcResourceOrigin const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcResourceOrigin *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSkyProperties const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSkyProperties *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpaceSkyColours const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpaceSkyColours *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpaceSkyProperties const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpaceSkyProperties *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStormProperties const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStormProperties *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSubstanceAmount const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSubstanceAmount *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTerrainTextureSettings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTerrainTextureSettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTerrainTileType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTerrainTileType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTileTypeOptions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTileTypeOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTileTypeSet const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTileTypeSet *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cTkPaletteTexture>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkPaletteTexture> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlanetSkyProperties::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcResourceCollectEffect::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcResourceOrigin::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSkyProperties::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpaceSkyColours::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpaceSkyProperties::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStormProperties::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSubstanceAmount::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTerrainTextureSettings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTerrainTileType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTileTypeOptions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTileTypeSet::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetSkyProperties::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSpaceSkyProperties::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStormProperties::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSubstanceAmount::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTileTypeOptions::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTileTypeSet::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStormProperties::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStormProperties *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSubstanceAmount::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceAmount *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTileTypeSet::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTileTypeSet *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetSkyProperties::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSkyProperties *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcResourceCollectEffect::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCollectEffect *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSkyProperties::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSkyProperties *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceSkyColours::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceSkyColours *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceSkyProperties::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceSkyProperties *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStormProperties::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStormProperties *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSubstanceAmount::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceAmount *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainTextureSettings::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainTextureSettings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainTileType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainTileType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTileTypeSet::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTileTypeSet *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetSkyProperties::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSkyProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcResourceCollectEffect::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCollectEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSkyProperties::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSkyProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceSkyColours::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceSkyColours *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceSkyProperties::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceSkyProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStormProperties::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStormProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSubstanceAmount::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceAmount *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainTextureSettings::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainTextureSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainTileType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainTileType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTileTypeSet::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTileTypeSet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetSkyProperties::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSkyProperties *
        ]
    ),
    "cGcResourceCollectEffect::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCollectEffect *
        ]
    ),
    "cGcSkyProperties::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSkyProperties *
        ]
    ),
    "cGcSpaceSkyColours::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceSkyColours *
        ]
    ),
    "cGcSpaceSkyProperties::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceSkyProperties *
        ]
    ),
    "cGcStormProperties::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStormProperties *
        ]
    ),
    "cGcSubstanceAmount::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceAmount *
        ]
    ),
    "cGcTerrainTextureSettings::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainTextureSettings *
        ]
    ),
    "cGcTileTypeSet::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTileTypeSet *
        ]
    ),
    "cGcPlanetSkyProperties::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSkyProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcResourceCollectEffect::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCollectEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcResourceOrigin::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceOrigin *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSkyProperties::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSkyProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceSkyColours::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceSkyColours *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceSkyProperties::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceSkyProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStormProperties::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStormProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSubstanceAmount::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceAmount *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainTextureSettings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainTextureSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainTileType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainTileType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTileTypeOptions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTileTypeOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTileTypeSet::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTileTypeSet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetSkyProperties::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSkyProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetSkyProperties *
        ]
    ),
    "cGcResourceCollectEffect::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCollectEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcResourceCollectEffect *
        ]
    ),
    "cGcSkyProperties::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSkyProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSkyProperties *
        ]
    ),
    "cGcSpaceSkyColours::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceSkyColours *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSpaceSkyColours *
        ]
    ),
    "cGcSpaceSkyProperties::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceSkyProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSpaceSkyProperties *
        ]
    ),
    "cGcStormProperties::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStormProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStormProperties *
        ]
    ),
    "cGcSubstanceAmount::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceAmount *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSubstanceAmount *
        ]
    ),
    "cGcTerrainTileType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainTileType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTerrainTileType *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBuildingOverrideData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBuildingOverrideData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBuildingSpawnData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBuildingSpawnData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBuildingSpawnSlot>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBuildingSpawnSlot> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureSpawnData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureSpawnData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcObjectSpawnData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcObjectSpawnData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcObjectSpawnDataVariant>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcObjectSpawnDataVariant> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcResourceElement>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcResourceElement> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSelectableObjectSpawnList>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSelectableObjectSpawnList> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcTerrainTileType>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTerrainTileType> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcTerrainTileType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainTileType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBuildingOverrideData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBuildingOverrideData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBuildingSpawnData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBuildingSpawnData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBuildingSpawnSlot const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBuildingSpawnSlot *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureSpawnData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureSpawnData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcObjectSpawnData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcObjectSpawnData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcObjectSpawnDataVariant const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcObjectSpawnDataVariant *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcResourceElement const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcResourceElement *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSelectableObjectSpawnList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSelectableObjectSpawnList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcTerrainTileType const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcTerrainTileType *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcTerrainTileType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTerrainTileType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTerrainTileType *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcEnvironmentSpawnData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcEnvironmentSpawnData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExternalObjectListOptions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExternalObjectListOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFogProperties const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFogProperties *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcLightProperties const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcLightProperties *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcLightShaftProperties const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcLightShaftProperties *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcObjectDefinitionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcObjectDefinitionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcObjectSpawnData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcObjectSpawnData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcObjectSpawnDataArray const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcObjectSpawnDataArray *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcObjectSpawnDataVariant const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcObjectSpawnDataVariant *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetBuildingData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetBuildingData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetCloudProperties const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetCloudProperties *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetLife const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetLife *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectSpawnData::cGcObjectSpawnData": {
        "cGcObjectSpawnData *, cGcObjectSpawnData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcObjectSpawnData *
                ctypes.c_ulonglong,  # cGcObjectSpawnData *
            ]
        ),
        "cGcObjectSpawnData *, const cGcObjectSpawnData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcObjectSpawnData *
                ctypes.c_ulonglong,  # const cGcObjectSpawnData *
            ]
        ),
    },
    "cTkDynamicArray<cGcObjectSpawnDataVariant>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcObjectSpawnDataVariant> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcEnvironmentSpawnData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExternalObjectListOptions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFogProperties::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLightProperties::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLightShaftProperties::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcObjectDefinitionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcObjectSpawnData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcObjectSpawnDataArray::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcObjectSpawnDataVariant::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetBuildingData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetCloudProperties::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetLife::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEnvironmentSpawnData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExternalObjectListOptions::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFogProperties::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcObjectSpawnData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcObjectSpawnDataArray::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetBuildingData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcEnvironmentSpawnData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentSpawnData *
        ]
    ),
    "cGcExternalObjectListOptions::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExternalObjectListOptions *
        ]
    ),
    "cGcObjectSpawnData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnData *
        ]
    ),
    "cGcPlanetBuildingData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetBuildingData *
        ]
    ),
    "cGcEnvironmentSpawnData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentSpawnData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExternalObjectListOptions::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExternalObjectListOptions *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFogProperties::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFogProperties *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcObjectSpawnData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetBuildingData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetBuildingData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcEnvironmentSpawnData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentSpawnData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExternalObjectListOptions::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExternalObjectListOptions *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcObjectSpawnData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetBuildingData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetBuildingData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcEnvironmentSpawnData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentSpawnData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExternalObjectListOptions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExternalObjectListOptions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFogProperties::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFogProperties *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLightProperties::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLightProperties *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLightShaftProperties::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLightShaftProperties *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectDefinitionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectDefinitionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectSpawnData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectSpawnDataVariant::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnDataVariant *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetBuildingData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetBuildingData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetCloudProperties::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetCloudProperties *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetLife::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetLife *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEnvironmentSpawnData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExternalObjectListOptions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExternalObjectListOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFogProperties::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFogProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLightProperties::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLightProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLightShaftProperties::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLightShaftProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectDefinitionData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectDefinitionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectSpawnData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectSpawnDataVariant::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnDataVariant *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetBuildingData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetBuildingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetCloudProperties::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetCloudProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetLife::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetLife *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExternalObjectListOptions::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExternalObjectListOptions *
        ]
    ),
    "cGcFogProperties::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFogProperties *
        ]
    ),
    "cGcLightProperties::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLightProperties *
        ]
    ),
    "cGcLightShaftProperties::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLightShaftProperties *
        ]
    ),
    "cGcObjectDefinitionData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectDefinitionData *
        ]
    ),
    "cGcObjectSpawnData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnData *
        ]
    ),
    "cGcObjectSpawnDataVariant::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnDataVariant *
        ]
    ),
    "cGcPlanetBuildingData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetBuildingData *
        ]
    ),
    "cGcPlanetCloudProperties::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetCloudProperties *
        ]
    ),
    "cGcEnvironmentSpawnData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExternalObjectListOptions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExternalObjectListOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFogProperties::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFogProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLightProperties::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLightProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLightShaftProperties::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLightShaftProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectDefinitionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectDefinitionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectSpawnData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectSpawnDataArray::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnDataArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectSpawnDataVariant::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnDataVariant *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetBuildingData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetBuildingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetCloudProperties::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetCloudProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetLife::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetLife *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEnvironmentSpawnData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExternalObjectListOptions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExternalObjectListOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcExternalObjectListOptions *
        ]
    ),
    "cGcFogProperties::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFogProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFogProperties *
        ]
    ),
    "cGcLightProperties::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLightProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcLightProperties *
        ]
    ),
    "cGcLightShaftProperties::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLightShaftProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcLightShaftProperties *
        ]
    ),
    "cGcObjectDefinitionData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectDefinitionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcObjectDefinitionData *
        ]
    ),
    "cGcObjectSpawnData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcObjectSpawnData *
        ]
    ),
    "cGcObjectSpawnDataVariant::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnDataVariant *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcObjectSpawnDataVariant *
        ]
    ),
    "cGcPlanetBuildingData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetBuildingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetBuildingData *
        ]
    ),
    "cGcPlanetCloudProperties::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetCloudProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetCloudProperties *
        ]
    ),
    "cGcPlanetLife::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetLife *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetLife *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBuildingClusterLayout>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBuildingClusterLayout> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBuildingClusterLayoutEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBuildingClusterLayoutEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBuildingFilename>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingFilename *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkNoiseFlattenOptions>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseFlattenOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBuildingClusterLayout const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBuildingClusterLayout *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBuildingClusterLayoutEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBuildingClusterLayoutEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBuildingDefinitionData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBuildingDefinitionData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBuildingFilename const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBuildingFilename *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBuildingFilenameList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBuildingFilenameList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingClassification const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingClassification *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingClusterLayout const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingClusterLayout *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingClusterLayoutEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingClusterLayoutEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingDefinitionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingDefinitionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingDefinitionTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingDefinitionTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingDensity const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingDensity *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingDensityLevels const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingDensityLevels *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingDistribution const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingDistribution *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingFilename const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingFilename *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingFilenameList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingFilenameList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingOverrideData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingOverrideData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingSpawnData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingSpawnData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingSpawnSlot const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingSpawnSlot *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingSystemTypeEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingSystemTypeEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCloudProperties const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCloudProperties *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcEnvironmentProperties const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcEnvironmentProperties *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFoliageComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFoliageComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcBuildingFilename,52>::cTkFixedArray<cGcBuildingFilename,52>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcBuildingFilename,52> *
        ]
    ),
    "cTkFixedArray<cGcBuildingFilenameList,8>::cTkFixedArray<cGcBuildingFilenameList,8>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcBuildingFilenameList,8> *
        ]
    ),
    "cTkDynamicArray<cGcBuildingClusterLayout>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBuildingClusterLayout> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBuildingClusterLayoutEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBuildingClusterLayoutEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFoliageComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFoliageComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingClassification::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingClusterLayout::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingClusterLayoutEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingDefinitionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingDefinitionTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingDensity::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingDensityLevels::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingDistribution::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingFilename::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingFilenameList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingOverrideData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingSpawnData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingSpawnSlot::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingSystemTypeEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCloudProperties::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEnvironmentProperties::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFoliageComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingClusterLayout::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBuildingDefinitionData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBuildingDefinitionTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBuildingSpawnData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBuildingSpawnData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingSpawnData *
        ]
    ),
    "cGcBuildingSpawnData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingSpawnData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBuildingDefinitionTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingDefinitionTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBuildingSpawnData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingSpawnData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBuildingClassification::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingClassification *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingClusterLayout::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingClusterLayout *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingClusterLayoutEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingClusterLayoutEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingDefinitionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingDefinitionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingDensityLevels::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingDensityLevels *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingDistribution::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingDistribution *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingFilename::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingFilename *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingFilenameList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingFilenameList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingOverrideData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingOverrideData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingSpawnData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingSpawnData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingSpawnSlot::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingSpawnSlot *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCloudProperties::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCloudProperties *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEnvironmentProperties::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentProperties *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingClassification::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingClassification *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcBuildingClassification::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingClassification *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingClusterLayout::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingClusterLayout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingClusterLayoutEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingClusterLayoutEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingDefinitionData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingDefinitionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingDefinitionTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingDefinitionTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingDensityLevels::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingDensityLevels *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingOverrideData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingOverrideData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingSpawnData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingSpawnSlot::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingSpawnSlot *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCloudProperties::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCloudProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEnvironmentProperties::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingDefinitionData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingDefinitionData *
        ]
    ),
    "cGcBuildingDefinitionTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingDefinitionTable *
        ]
    ),
    "cGcBuildingFilename::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingFilename *
        ]
    ),
    "cGcBuildingOverrideData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingOverrideData *
        ]
    ),
    "cGcBuildingSpawnData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingSpawnData *
        ]
    ),
    "cGcBuildingSpawnSlot::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingSpawnSlot *
        ]
    ),
    "cGcCloudProperties::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCloudProperties *
        ]
    ),
    "cGcEnvironmentProperties::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentProperties *
        ]
    ),
    "cGcBuildingClassification::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingClassification *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcBuildingClassification::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingClassification *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingClusterLayout::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingClusterLayout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingClusterLayoutEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingClusterLayoutEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingDefinitionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingDefinitionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingDefinitionTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingDefinitionTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingDensity::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingDensity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingDensityLevels::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingDensityLevels *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingDistribution::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingDistribution *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingFilename::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingFilename *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingFilenameList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingFilenameList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingOverrideData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingOverrideData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingSpawnData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingSpawnSlot::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingSpawnSlot *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingSystemTypeEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingSystemTypeEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCloudProperties::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCloudProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEnvironmentProperties::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFoliageComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFoliageComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingClassification::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingClassification *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBuildingClassification *
        ]
    ),
    "cGcBuildingClusterLayout::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingClusterLayout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBuildingClusterLayout *
        ]
    ),
    "cGcBuildingClusterLayoutEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingClusterLayoutEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBuildingClusterLayoutEntry *
        ]
    ),
    "cGcBuildingDefinitionData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingDefinitionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBuildingDefinitionData *
        ]
    ),
    "cGcBuildingDensityLevels::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingDensityLevels *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBuildingDensityLevels *
        ]
    ),
    "cGcBuildingFilename::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingFilename *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingSpawnData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBuildingSpawnData *
        ]
    ),
    "cGcCloudProperties::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCloudProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCloudProperties *
        ]
    ),
    "cGcEnvironmentProperties::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcEnvironmentProperties *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcEcosystemCreatureData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcEcosystemCreatureData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPetAccessoryGroup>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPetAccessoryGroup> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPetAccessoryType>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPetAccessoryType> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPetEggSpeciesOverrideData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPetEggSpeciesOverrideData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPetEggTraitModifierOverrideData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPetEggTraitModifierOverrideData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcEcosystemCreatureData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcEcosystemCreatureData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPetAccessoryGroup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPetAccessoryGroup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPetAccessoryInfo const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPetAccessoryInfo *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPetAccessoryType const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPetAccessoryType *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPetEggSpeciesOverrideData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPetEggSpeciesOverrideData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPetEggTraitModifierOverrideData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPetEggTraitModifierOverrideData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureSpawnEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureSpawnEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDeprecatedAssetsTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDeprecatedAssetsTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcEcosystemCreatureData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcEcosystemCreatureData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcEcosystemSpawnData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcEcosystemSpawnData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcIkPistonData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcIkPistonData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCPropType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCPropType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCSeatedPosture const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCSeatedPosture *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCTriggerTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCTriggerTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetAccessoryGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetAccessoryGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetAccessoryInfo const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetAccessoryInfo *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetAccessoryTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetAccessoryTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetAccessoryType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetAccessoryType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetEggSpeciesOverrideData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetEggSpeciesOverrideData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetEggSpeciesOverrideTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetEggSpeciesOverrideTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetEggTraitModifierOverrideData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetEggTraitModifierOverrideData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetEggTraitModifierOverrideTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetEggTraitModifierOverrideTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSandwormTimerAndFrequencyOverride const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSandwormTimerAndFrequencyOverride *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpawnComponentOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpawnComponentOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcEcosystemCreatureData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcEcosystemCreatureData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPetEggSpeciesOverrideData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPetEggSpeciesOverrideData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPetEggTraitModifierOverrideData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPetEggTraitModifierOverrideData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureSpawnEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDeprecatedAssetsTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEcosystemCreatureData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEcosystemSpawnData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcIkPistonData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCPropType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCSeatedPosture::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCTriggerTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetAccessoryGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetAccessoryInfo::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetAccessoryTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetAccessoryType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetEggSpeciesOverrideData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetEggSpeciesOverrideTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetEggTraitModifierOverrideData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetEggTraitModifierOverrideTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSandwormTimerAndFrequencyOverride::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpawnComponentOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDeprecatedAssetsTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcEcosystemCreatureData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcEcosystemSpawnData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetAccessoryGroup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetAccessoryTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetEggSpeciesOverrideTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetEggTraitModifierOverrideData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetEggTraitModifierOverrideTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSpawnComponentOption::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSpawnComponentOption::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnComponentOption *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetAccessoryTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetAccessoryTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetEggTraitModifierOverrideTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetEggTraitModifierOverrideTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSpawnComponentOption::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnComponentOption *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcEcosystemCreatureData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystemCreatureData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEcosystemSpawnData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystemSpawnData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcIkPistonData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcIkPistonData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCPropType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPropType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSeatedPosture::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSeatedPosture *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCTriggerTypes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCTriggerTypes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetAccessoryGroup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetAccessoryGroup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetAccessoryInfo::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetAccessoryInfo *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetAccessoryType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetAccessoryType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetEggSpeciesOverrideData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetEggSpeciesOverrideData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetEggTraitModifierOverrideData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetEggTraitModifierOverrideData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSandwormTimerAndFrequencyOverride::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSandwormTimerAndFrequencyOverride *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpawnComponentOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnComponentOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSandwormTimerAndFrequencyOverride::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSandwormTimerAndFrequencyOverride *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcEcosystemCreatureData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystemCreatureData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcIkPistonData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIkPistonData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCPropType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPropType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSeatedPosture::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSeatedPosture *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCTriggerTypes::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCTriggerTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetAccessoryType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetAccessoryType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetEggSpeciesOverrideData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetEggSpeciesOverrideData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetEggTraitModifierOverrideData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetEggTraitModifierOverrideData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSandwormTimerAndFrequencyOverride::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSandwormTimerAndFrequencyOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpawnComponentOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnComponentOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcIkPistonData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIkPistonData *
        ]
    ),
    "cGcNPCPropType::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPropType *
        ]
    ),
    "cGcPetAccessoryTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetAccessoryTable *
        ]
    ),
    "cGcSandwormTimerAndFrequencyOverride::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSandwormTimerAndFrequencyOverride *
        ]
    ),
    "cGcSpawnComponentOption::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnComponentOption *
        ]
    ),
    "cGcSandwormTimerAndFrequencyOverride::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSandwormTimerAndFrequencyOverride *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcCreatureSpawnEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSpawnEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDeprecatedAssetsTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDeprecatedAssetsTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEcosystemCreatureData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystemCreatureData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEcosystemSpawnData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystemSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcIkPistonData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIkPistonData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCPropType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPropType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCSeatedPosture::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSeatedPosture *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCTriggerTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCTriggerTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetAccessoryGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetAccessoryGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetAccessoryInfo::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetAccessoryInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetAccessoryTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetAccessoryTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetAccessoryType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetAccessoryType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetEggSpeciesOverrideData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetEggSpeciesOverrideData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetEggSpeciesOverrideTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetEggSpeciesOverrideTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetEggTraitModifierOverrideData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetEggTraitModifierOverrideData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetEggTraitModifierOverrideTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetEggTraitModifierOverrideTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSandwormTimerAndFrequencyOverride::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSandwormTimerAndFrequencyOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpawnComponentOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnComponentOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEcosystemCreatureData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEcosystemCreatureData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcEcosystemCreatureData *
        ]
    ),
    "cGcNPCPropType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPropType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCPropType *
        ]
    ),
    "cGcNPCSeatedPosture::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCSeatedPosture *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCSeatedPosture *
        ]
    ),
    "cGcNPCTriggerTypes::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCTriggerTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCTriggerTypes *
        ]
    ),
    "cGcPetAccessoryGroup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetAccessoryGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPetAccessoryGroup *
        ]
    ),
    "cGcPetAccessoryType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetAccessoryType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPetAccessoryType *
        ]
    ),
    "cGcPetEggSpeciesOverrideData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetEggSpeciesOverrideData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPetEggSpeciesOverrideData *
        ]
    ),
    "cGcPetEggTraitModifierOverrideData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetEggTraitModifierOverrideData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPetEggTraitModifierOverrideData *
        ]
    ),
    "cGcSandwormTimerAndFrequencyOverride::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSandwormTimerAndFrequencyOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSandwormTimerAndFrequencyOverride *
        ]
    ),
    "cGcSpawnComponentOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpawnComponentOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSpawnComponentOption *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureRoleData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureRoleData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureRoleDescription>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureRoleDescription> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureRoleFilename>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureRoleFilename> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureRoleFilenameList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleFilenameList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSpawnComponentOption>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSpawnComponentOption> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCreatureHemiSphere>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHemiSphere *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCreatureRoleFilenameList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleFilenameList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCreatureRoles>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoles *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCreatureSizeClasses>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSizeClasses *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCreatureTypes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureRoleData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureRoleData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureRoleDescription const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureRoleDescription *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureRoleFilename const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureRoleFilename *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureRoleFilenameList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureRoleFilenameList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSpawnComponentOption const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSpawnComponentOption *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCreatureHemiSphere>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureHemiSphere *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureHemiSphere *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCreatureRoleDescription>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRoleDescription *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureRoleDescription *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCreatureRoleFilenameList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRoleFilenameList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureRoleFilenameList *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCreatureRoles>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRoles *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureRoles *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCreatureSizeClasses>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureSizeClasses *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureSizeClasses *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCreatureTypes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureTypes *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureGenerationWeightedListDomainEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureGenerationWeightedListDomainEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureGroupDescription const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureGroupDescription *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureGroupProbability const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureGroupProbability *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureGroups const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureGroups *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureHemiSphere const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureHemiSphere *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureIkData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureIkData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureLegIKComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureLegIKComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureRoleData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRoleData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureRoleDataTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRoleDataTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureRoleDescription const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRoleDescription *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureRoleDescriptionTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRoleDescriptionTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureRoleFilename const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRoleFilename *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureRoleFilenameList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRoleFilenameList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureRoleFilenameTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRoleFilenameTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureRoles const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRoles *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureSpawnComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureSpawnComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureSpawnData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureSpawnData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcCreatureRoleDescription>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureRoleDescription> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureRoleFilename>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureRoleFilename> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSpawnComponentOption>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSpawnComponentOption> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureLegIKComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureLegIKComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureSpawnComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureSpawnComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureGenerationWeightedListDomainEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureGroupDescription::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureGroupProbability::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureGroups::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureHemiSphere::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureIkData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureLegIKComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureRoleData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureRoleDataTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureRoleDescription::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureRoleDescriptionTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureRoleFilename::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureRoleFilenameList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureRoleFilenameTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureRoles::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureSpawnComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureSpawnData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureIkData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureRoleData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureRoleDataTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureRoleDescription::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureRoleDescriptionTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureRoleFilenameList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureRoleFilenameTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureSpawnComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureSpawnData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureRoleDataTable::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleDataTable *
        ]
    ),
    "cGcCreatureSpawnData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSpawnData *
        ]
    ),
    "cGcCreatureRoleData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureRoleDataTable::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleDataTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureSpawnData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSpawnData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureRoleDataTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleDataTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureRoleDescriptionTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleDescriptionTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureSpawnData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSpawnData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureGenerationWeightedListDomainEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationWeightedListDomainEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGroupDescription::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroupDescription *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGroupProbability::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroupProbability *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHemiSphere::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHemiSphere *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureIkData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureIkData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleDataTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleDataTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleDescription::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleDescription *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleDescriptionTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleDescriptionTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleFilename::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleFilename *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleFilenameList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleFilenameList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleFilenameTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleFilenameTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoles::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoles *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSpawnComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSpawnComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSpawnData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSpawnData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationWeightedListDomainEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationWeightedListDomainEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHemiSphere::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHemiSphere *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureIkData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureIkData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleDataTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleDataTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleDescription::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleDescription *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoles::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoles *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSpawnComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSpawnComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSpawnData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationWeightedListDomainEntry::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationWeightedListDomainEntry *
        ]
    ),
    "cGcCreatureIkData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureIkData *
        ]
    ),
    "cGcCreatureRoleData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleData *
        ]
    ),
    "cGcCreatureRoleDataTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleDataTable *
        ]
    ),
    "cGcCreatureRoleDescription::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleDescription *
        ]
    ),
    "cGcCreatureRoleFilename::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleFilename *
        ]
    ),
    "cGcCreatureSpawnComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSpawnComponentData *
        ]
    ),
    "cGcCreatureSpawnData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSpawnData *
        ]
    ),
    "cGcCreatureGenerationWeightedListDomainEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationWeightedListDomainEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGroupDescription::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroupDescription *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGroupProbability::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroupProbability *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGroups::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroups *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHemiSphere::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHemiSphere *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureIkData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureIkData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureLegIKComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureLegIKComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleDataTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleDataTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleDescription::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleDescription *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleDescriptionTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleDescriptionTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleFilename::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleFilename *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleFilenameList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleFilenameList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleFilenameTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleFilenameTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoles::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoles *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSpawnComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSpawnComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSpawnData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGroupDescription::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGroupDescription *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureGroupDescription *
        ]
    ),
    "cGcCreatureHemiSphere::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHemiSphere *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureHemiSphere *
        ]
    ),
    "cGcCreatureIkData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureIkData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureIkData *
        ]
    ),
    "cGcCreatureRoleData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureRoleData *
        ]
    ),
    "cGcCreatureRoleDataTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleDataTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureRoleDataTable *
        ]
    ),
    "cGcCreatureRoleDescription::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleDescription *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureRoleDescription *
        ]
    ),
    "cGcCreatureRoleDescriptionTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleDescriptionTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureRoleDescriptionTable *
        ]
    ),
    "cGcCreatureRoleFilename::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleFilename *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureRoleFilename *
        ]
    ),
    "cGcCreatureRoles::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoles *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureRoles *
        ]
    ),
    "cGcCreatureSpawnComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSpawnComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureSpawnComponentData *
        ]
    ),
    "cGcCreatureSpawnData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureSpawnData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureDebugWaypoint>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureDebugWaypoint> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureDiscoveryThumbnailOverride>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureDiscoveryThumbnailOverride> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureEffectTrigger>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureEffectTrigger> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureGenerationDomainEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureGenerationDomainEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureGenerationDomainTable>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureGenerationDomainTable> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureGenerationOptionalWeightedList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationOptionalWeightedList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureGenerationWeightedListDomainEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureGenerationWeightedListDomainEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcReplacementEffectData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcReplacementEffectData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCreatureGenerationOptionalWeightedList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationOptionalWeightedList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCreatureGenerationWeightedList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationWeightedList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureDebugWaypoint const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureDebugWaypoint *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureDiscoveryThumbnailOverride const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureDiscoveryThumbnailOverride *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureEffectTrigger const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureEffectTrigger *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureGenerationDomainEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureGenerationDomainEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureGenerationDomainTable const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureGenerationDomainTable *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureGenerationOptionalWeightedList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureGenerationOptionalWeightedList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureGenerationWeightedListDomainEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureGenerationWeightedListDomainEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcReplacementEffectData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcReplacementEffectData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCreatureGenerationDensity>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureGenerationDensity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureGenerationDensity *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCreatureGenerationOptionalWeightedList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureGenerationOptionalWeightedList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureGenerationOptionalWeightedList *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCreatureGenerationWeightedList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureGenerationWeightedList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureGenerationWeightedList *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBehaviourPlayAnimTrigger const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBehaviourPlayAnimTrigger *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureActiveTime const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureActiveTime *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureAttractorComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureAttractorComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureBaitComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureBaitComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureDebugSpawnData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureDebugSpawnData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureDebugWaypoint const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureDebugWaypoint *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureDiet const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureDiet *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureEffectComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureEffectComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureEggComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureEggComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureFilename const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureFilename *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureGenerationArchetypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureGenerationArchetypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureGenerationData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureGenerationData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureGenerationDensity const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureGenerationDensity *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureGenerationDomainEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureGenerationDomainEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureGenerationDomainTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureGenerationDomainTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureGenerationOptionalWeightedList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureGenerationOptionalWeightedList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureGenerationWeightedList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureGenerationWeightedList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcCreatureDebugWaypoint>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureDebugWaypoint> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureDiscoveryThumbnailOverride>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureDiscoveryThumbnailOverride> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureEffectTrigger>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureEffectTrigger> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureGenerationDomainEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureGenerationDomainEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureGenerationDomainTable>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureGenerationDomainTable> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureGenerationWeightedListDomainEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureGenerationWeightedListDomainEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureAttractorComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureAttractorComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureBaitComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureBaitComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCreatureComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCreatureEffectComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureEffectComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureEggComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureEggComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourPlayAnimTrigger::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureActiveTime::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureAttractorComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureBaitComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureDebugSpawnData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureDebugWaypoint::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureDiet::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureEffectComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureEggComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureFilename::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureGenerationArchetypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureGenerationData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureGenerationDensity::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureGenerationDomainEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureGenerationDomainTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureGenerationOptionalWeightedList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureGenerationWeightedList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureBaitComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureDebugSpawnData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureEffectComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureGenerationArchetypes::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureGenerationData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureGenerationDomainTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureGenerationOptionalWeightedList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureGenerationWeightedList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcCreatureGenerationDomainTable>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSentinelSpawnWave> *
        ]
    ),
    "cGcCreatureGenerationArchetypes::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationArchetypes *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureGenerationWeightedList::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationWeightedList *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureGenerationArchetypes::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationArchetypes *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureGenerationData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureGenerationWeightedList::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationWeightedList *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBehaviourPlayAnimTrigger::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourPlayAnimTrigger *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureActiveTime::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureActiveTime *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureAttractorComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureAttractorComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureBaitComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBaitComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureDebugSpawnData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDebugSpawnData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureDebugWaypoint::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDebugWaypoint *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureDiet::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDiet *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFilename::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFilename *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationArchetypes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationArchetypes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationDensity::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationDensity *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationDomainEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationDomainEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationDomainTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationDomainTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationOptionalWeightedList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationOptionalWeightedList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationWeightedList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationWeightedList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourPlayAnimTrigger::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourPlayAnimTrigger *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureActiveTime::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureActiveTime *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureDebugSpawnData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDebugSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureDebugWaypoint::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDebugWaypoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureDiet::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDiet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFilename::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFilename *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationDensity::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationDensity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationDomainEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationDomainEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationDomainTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationDomainTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationWeightedList::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationWeightedList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourPlayAnimTrigger::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourPlayAnimTrigger *
        ]
    ),
    "cGcCreatureComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponentData *
        ]
    ),
    "cGcCreatureDebugSpawnData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDebugSpawnData *
        ]
    ),
    "cGcCreatureFilename::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFilename *
        ]
    ),
    "cGcCreatureGenerationData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationData *
        ]
    ),
    "cGcCreatureGenerationDomainEntry::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationDomainEntry *
        ]
    ),
    "cGcBehaviourPlayAnimTrigger::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourPlayAnimTrigger *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureActiveTime::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureActiveTime *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureAttractorComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureAttractorComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureBaitComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBaitComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureDebugSpawnData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDebugSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureDebugWaypoint::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDebugWaypoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureDiet::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDiet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureEffectComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEffectComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFilename::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFilename *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationArchetypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationArchetypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationDensity::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationDensity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationDomainEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationDomainEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationDomainTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationDomainTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationOptionalWeightedList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationOptionalWeightedList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGenerationWeightedList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationWeightedList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureActiveTime::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureActiveTime *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureActiveTime *
        ]
    ),
    "cGcCreatureAttractorComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureAttractorComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureAttractorComponentData *
        ]
    ),
    "cGcCreatureComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureComponentData *
        ]
    ),
    "cGcCreatureDebugSpawnData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDebugSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureDebugSpawnData *
        ]
    ),
    "cGcCreatureDebugWaypoint::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDebugWaypoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureDebugWaypoint *
        ]
    ),
    "cGcCreatureDiet::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDiet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureDiet *
        ]
    ),
    "cGcCreatureGenerationData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureGenerationData *
        ]
    ),
    "cGcCreatureGenerationDensity::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationDensity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureGenerationDensity *
        ]
    ),
    "cGcCreatureGenerationDomainEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationDomainEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureGenerationDomainEntry *
        ]
    ),
    "cGcCreatureGenerationDomainTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationDomainTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureGenerationDomainTable *
        ]
    ),
    "cGcCreatureGenerationOptionalWeightedList::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationOptionalWeightedList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureGenerationOptionalWeightedList *
        ]
    ),
    "cGcCreatureGenerationWeightedList::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGenerationWeightedList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPetBehaviourMoodModifier>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPetBehaviourMoodModifier> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPetBehaviourTraitModifier>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPetBehaviourTraitModifier> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPetFollowUpBehaviour>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPetFollowUpBehaviour> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPetMoodStaminaModifier>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPetMoodStaminaModifier> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPetTraitMoodModifier>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetTraitMoodModifier *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPetTraitStaminaModifier>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPetTraitStaminaModifier> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCreaturePetMood>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetMood *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCreaturePetTraits>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetTraits *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreaturePetTraitRanges const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreaturePetTraitRanges *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPetActionMoodModifier const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPetActionMoodModifier *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPetBehaviourData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPetBehaviourData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPetBehaviourMoodModifier const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPetBehaviourMoodModifier *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPetBehaviourTraitModifier const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPetBehaviourTraitModifier *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPetFollowUpBehaviour const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPetFollowUpBehaviour *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPetMoodStaminaModifier const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPetMoodStaminaModifier *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPetTraitMoodModifier const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPetTraitMoodModifier *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPetTraitMoodModifierList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPetTraitMoodModifierList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPetTraitStaminaModifier const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPetTraitStaminaModifier *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCreaturePetMood>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreaturePetMood *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreaturePetMood *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCreaturePetTraits>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreaturePetTraits *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreaturePetTraits *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPetBehaviours>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetBehaviours *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPetBehaviours *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetBehaviourData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetBehaviourData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetBehaviourMoodModifier const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetBehaviourMoodModifier *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetBehaviourTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetBehaviourTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetBehaviourTraitModifier const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetBehaviourTraitModifier *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetBehaviours const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetBehaviours *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetFollowUpBehaviour const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetFollowUpBehaviour *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetMoodStaminaModifier const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetMoodStaminaModifier *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetTraitMoodModifier const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetTraitMoodModifier *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetTraitMoodModifierList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetTraitMoodModifierList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetTraitStaminaModifier const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetTraitStaminaModifier *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeirdCreatureRewardList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeirdCreatureRewardList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcCreaturePetTraitRanges,11>::cTkFixedArray<cGcCreaturePetTraitRanges,11>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcCreaturePetTraitRanges,11> *
        ]
    ),
    "cTkFixedArray<cGcPetBehaviourData,28>::cTkFixedArray<cGcPetBehaviourData,28>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcPetBehaviourData,28> *
        ]
    ),
    "cTkFixedArray<cGcPetTraitMoodModifierList,3>::cTkFixedArray<cGcPetTraitMoodModifierList,3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcPetTraitMoodModifierList,3> *
        ]
    ),
    "cTkFixedArray<cGcPetBehaviourData,28>::~cTkFixedArray<cGcPetBehaviourData,28>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcPetBehaviourData,28> *
        ]
    ),
    "cGcPetBehaviourData::~cGcPetBehaviourData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourData *
        ]
    ),
    "cTkDynamicArray<cGcPetBehaviourMoodModifier>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPetBehaviourTraitModifier> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPetFollowUpBehaviour>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPetFollowUpBehaviour> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPetMoodStaminaModifier>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPetTraitStaminaModifier> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPetBehaviourData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetBehaviourMoodModifier::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetBehaviourTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetBehaviourTraitModifier::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetBehaviours::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetFollowUpBehaviour::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetMoodStaminaModifier::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetTraitMoodModifier::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetTraitMoodModifierList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetTraitStaminaModifier::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeirdCreatureRewardList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetBehaviourData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetBehaviourMoodModifier::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetBehaviourTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetBehaviourTraitModifier::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetFollowUpBehaviour::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetMoodStaminaModifier::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetTraitStaminaModifier::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetBehaviourData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetBehaviourData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetBehaviourTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetBehaviourData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetBehaviourMoodModifier::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourMoodModifier *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetBehaviourTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetBehaviourTraitModifier::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTraitModifier *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetBehaviours::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviours *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetFollowUpBehaviour::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetFollowUpBehaviour *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetMoodStaminaModifier::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetMoodStaminaModifier *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetTraitMoodModifier::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetTraitMoodModifier *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetTraitMoodModifierList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetTraitMoodModifierList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetTraitStaminaModifier::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetTraitStaminaModifier *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetBehaviourData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetBehaviourMoodModifier::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourMoodModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetBehaviourTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetBehaviourTraitModifier::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTraitModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetBehaviours::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviours *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetFollowUpBehaviour::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetFollowUpBehaviour *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetMoodStaminaModifier::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetMoodStaminaModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetTraitStaminaModifier::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetTraitStaminaModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeirdCreatureRewardList::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeirdCreatureRewardList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetBehaviourTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTable *
        ]
    ),
    "cGcWeirdCreatureRewardList::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeirdCreatureRewardList *
        ]
    ),
    "cGcPetBehaviourData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetBehaviourMoodModifier::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourMoodModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetBehaviourTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetBehaviourTraitModifier::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTraitModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetBehaviours::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviours *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetFollowUpBehaviour::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetFollowUpBehaviour *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetMoodStaminaModifier::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetMoodStaminaModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetTraitMoodModifier::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetTraitMoodModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetTraitMoodModifierList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetTraitMoodModifierList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetTraitStaminaModifier::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetTraitStaminaModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeirdCreatureRewardList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeirdCreatureRewardList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetBehaviourData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPetBehaviourData *
        ]
    ),
    "cGcPetBehaviourMoodModifier::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourMoodModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPetBehaviourMoodModifier *
        ]
    ),
    "cGcPetBehaviourTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPetBehaviourTable *
        ]
    ),
    "cGcPetBehaviourTraitModifier::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviourTraitModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPetBehaviourTraitModifier *
        ]
    ),
    "cGcPetBehaviours::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetBehaviours *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPetBehaviours *
        ]
    ),
    "cGcPetFollowUpBehaviour::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetFollowUpBehaviour *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPetFollowUpBehaviour *
        ]
    ),
    "cGcPetMoodStaminaModifier::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetMoodStaminaModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPetMoodStaminaModifier *
        ]
    ),
    "cGcPetTraitMoodModifier::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetTraitMoodModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPetTraitMoodModifier *
        ]
    ),
    "cGcPetTraitStaminaModifier::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetTraitStaminaModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPetTraitStaminaModifier *
        ]
    ),
    "cGcWeirdCreatureRewardList::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeirdCreatureRewardList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureBehaviourTreeData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureBehaviourTreeData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureFilename>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureFilename> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureIkData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureIkData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureStupidName>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureStupidName> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureTagAndRarity>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureTagAndRarity> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureVocalSoundData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureVocalSoundData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcIkPistonData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcIkPistonData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCreatureRarity>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRarity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCreatureRoleFrequencyModifier>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleFrequencyModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureBehaviourTreeData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureBehaviourTreeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureFilename const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureFilename *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureIkData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureIkData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureStupidName const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureStupidName *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureTagAndRarity const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureTagAndRarity *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureVocalSoundData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureVocalSoundData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcIkPistonData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcIkPistonData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCreatureRarity>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRarity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureRarity *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCreatureRoleFrequencyModifier>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRoleFrequencyModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureRoleFrequencyModifier *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureAudioTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureAudioTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureBehaviourTreeData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureBehaviourTreeData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureBehaviourTrees const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureBehaviourTrees *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureDataTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureDataTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureEffectTrigger const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureEffectTrigger *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureFilenameTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureFilenameTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureFoodList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureFoodList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureFullBodyIKComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureFullBodyIKComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureHarvestSubstanceList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureHarvestSubstanceList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureInfo const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureInfo *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureRoleFrequencyModifier const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRoleFrequencyModifier *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureSizeClasses const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureSizeClasses *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureStupidName const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureStupidName *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureStupidNameTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureStupidNameTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureSubstanceList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureSubstanceList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureTagAndRarity const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureTagAndRarity *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureVocalTestData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureVocalTestData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetActionMoodModifier const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetActionMoodModifier *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcCreatureBehaviourTreeData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureBehaviourTreeData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureFilename>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureFilename> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureIkData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureIkData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureStupidName>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureStupidName> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureVocalSoundData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureVocalSoundData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcIkPistonData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcIkPistonData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureFullBodyIKComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureAudioTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureBehaviourTreeData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureBehaviourTrees::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureDataTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureEffectTrigger::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureFilenameTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureFoodList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureFullBodyIKComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureHarvestSubstanceList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureInfo::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureRoleFrequencyModifier::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureSizeClasses::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureStupidName::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureStupidNameTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureSubstanceList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureTagAndRarity::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureVocalTestData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetActionMoodModifier::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureAudioTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureBehaviourTreeData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureBehaviourTrees::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureDataTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureEffectTrigger::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureFilenameTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureFullBodyIKComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureInfo::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureStupidName::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureStupidNameTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureTagAndRarity::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcCreatureBehaviourTreeData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureBehaviourTreeData> *
        ]
    ),
    "cTkDynamicArray<cGcCreatureData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureData> *
        ]
    ),
    "cGcCreatureInfo::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureInfo *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureBehaviourTreeData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTreeData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureEffectTrigger::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEffectTrigger *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureAudioTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLegacyItemTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureBehaviourTreeData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTreeData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureBehaviourTrees::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTrees *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureDataTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDataTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureFilenameTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFilenameTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureBehaviourTreeData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTreeData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureEffectTrigger::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEffectTrigger *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFoodList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFoodList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFullBodyIKComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHarvestSubstanceList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHarvestSubstanceList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureInfo::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureInfo *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleFrequencyModifier::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleFrequencyModifier *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSizeClasses::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSizeClasses *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureStupidName::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureStupidName *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSubstanceList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSubstanceList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureTagAndRarity::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureTagAndRarity *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureTypes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureTypes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureVocalTestData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureVocalTestData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetActionMoodModifier::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetActionMoodModifier *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureTypes::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureTypes *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcCreatureData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureEffectTrigger::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEffectTrigger *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFoodList::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFoodList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFullBodyIKComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHarvestSubstanceList::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHarvestSubstanceList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureInfo::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleFrequencyModifier::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleFrequencyModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSizeClasses::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSizeClasses *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSubstanceList::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSubstanceList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureTypes::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetActionMoodModifier::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetActionMoodModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureAudioTable::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureAudioTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureBehaviourTrees::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTrees *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureDataTable::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDataTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureData *
        ]
    ),
    "cGcCreatureEffectTrigger::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEffectTrigger *
        ]
    ),
    "cGcCreatureFoodList::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFoodList *
        ]
    ),
    "cGcCreatureHarvestSubstanceList::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHarvestSubstanceList *
        ]
    ),
    "cGcCreatureInfo::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureInfo *
        ]
    ),
    "cGcCreatureStupidNameTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureStupidNameTable *
        ]
    ),
    "cGcCreatureTagAndRarity::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistencyMissionOverride *
        ]
    ),
    "cGcCreatureVocalTestData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureVocalTestData *
        ]
    ),
    "cGcCreatureTypes::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureTypes *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcCreatureAudioTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureAudioTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureBehaviourTreeData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTreeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureBehaviourTrees::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTrees *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureDataTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureDataTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureEffectTrigger::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEffectTrigger *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFilenameTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFilenameTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFoodList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFoodList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFullBodyIKComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHarvestSubstanceList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHarvestSubstanceList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureInfo::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRoleFrequencyModifier::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleFrequencyModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSizeClasses::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSizeClasses *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureStupidName::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureStupidName *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureStupidNameTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureStupidNameTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSubstanceList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSubstanceList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureTagAndRarity::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureTagAndRarity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureVocalTestData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureVocalTestData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetActionMoodModifier::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetActionMoodModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureBehaviourTreeData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureBehaviourTreeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureBehaviourTreeData *
        ]
    ),
    "cGcCreatureData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureData *
        ]
    ),
    "cGcCreatureEffectTrigger::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureEffectTrigger *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureEffectTrigger *
        ]
    ),
    "cGcCreatureFullBodyIKComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFullBodyIKComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureFullBodyIKComponentData *
        ]
    ),
    "cGcCreatureInfo::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureInfo *
        ]
    ),
    "cGcCreatureRoleFrequencyModifier::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRoleFrequencyModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureRoleFrequencyModifier *
        ]
    ),
    "cGcCreatureSizeClasses::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSizeClasses *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureSizeClasses *
        ]
    ),
    "cGcCreatureStupidName::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureStupidName *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureStupidName *
        ]
    ),
    "cGcCreatureTagAndRarity::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureTagAndRarity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureTagAndRarity *
        ]
    ),
    "cGcCreatureTypes::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureTypes *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureRidingAnimation>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureRidingAnimation> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureRidingPartModifier>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureRidingPartModifier> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureSwarmDataParams>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureSwarmDataParams> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCreatureVocalSoundData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureVocalSoundData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureRidingAnimation const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureRidingAnimation *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureRidingPartModifier const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureRidingPartModifier *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureSwarmDataParams const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureSwarmDataParams *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCreatureVocalSoundData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureVocalSoundData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureVocalSoundData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureRidingAnimation const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRidingAnimation *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureRidingData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRidingData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureRidingPartModifier const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRidingPartModifier *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureSwarmData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureSwarmData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureSwarmDataParams const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureSwarmDataParams *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureVocalData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureVocalData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureVocalSoundData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureVocalSoundData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureWeirdMovementData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureWeirdMovementData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFlyingSnakeData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFlyingSnakeData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcUnlockablePlatformReward>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUnlockablePlatformReward> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureRidingPartModifier>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureRidingPartModifier> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureSwarmDataParams>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureSwarmDataParams> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureRidingData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureRidingData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureSwarmData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureSwarmData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureVocalData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureVocalData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureWeirdMovementData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureWeirdMovementData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureRidingAnimation::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureRidingData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureRidingPartModifier::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureSwarmData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureSwarmDataParams::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureVocalData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureVocalSoundData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureWeirdMovementData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFlyingSnakeData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureRidingData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureRidingPartModifier::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureSwarmData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureSwarmDataParams::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureWeirdMovementData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcCreatureSwarmDataParams>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureSwarmDataParams> *
        ]
    ),
    "cGcCreatureRidingAnimation::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRidingAnimation *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRidingData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRidingData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRidingPartModifier::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRidingPartModifier *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSwarmData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSwarmDataParams::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmDataParams *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureVocalSoundData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureVocalSoundData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureWeirdMovementData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureWeirdMovementData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFlyingSnakeData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFlyingSnakeData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRidingPartModifier::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRidingPartModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSwarmDataParams::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmDataParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureVocalSoundData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureVocalSoundData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFlyingSnakeData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFlyingSnakeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRidingData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRidingData *
        ]
    ),
    "cGcCreatureRidingPartModifier::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRidingPartModifier *
        ]
    ),
    "cGcCreatureSwarmDataParams::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmDataParams *
        ]
    ),
    "cGcCreatureVocalSoundData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureVocalSoundData *
        ]
    ),
    "cGcCreatureWeirdMovementData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureWeirdMovementData *
        ]
    ),
    "cGcFlyingSnakeData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFlyingSnakeData *
        ]
    ),
    "cGcCreatureRidingAnimation::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRidingAnimation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRidingData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRidingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRidingPartModifier::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRidingPartModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSwarmData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureSwarmDataParams::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmDataParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureVocalData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureVocalData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureVocalSoundData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureVocalSoundData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureWeirdMovementData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureWeirdMovementData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFlyingSnakeData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFlyingSnakeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRidingAnimation::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRidingAnimation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureRidingAnimation *
        ]
    ),
    "cGcCreatureRidingData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRidingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureRidingData *
        ]
    ),
    "cGcCreatureRidingPartModifier::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRidingPartModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureRidingPartModifier *
        ]
    ),
    "cGcCreatureSwarmData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureSwarmData *
        ]
    ),
    "cGcCreatureSwarmDataParams::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureSwarmDataParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureSwarmDataParams *
        ]
    ),
    "cGcCreatureVocalSoundData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureVocalSoundData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureVocalSoundData *
        ]
    ),
    "cGcCreatureWeirdMovementData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureWeirdMovementData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureWeirdMovementData *
        ]
    ),
    "cGcFlyingSnakeData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFlyingSnakeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFlyingSnakeData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureFootParticleSingleData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureFootParticleSingleData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureHoverMovementDataParams>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureHoverMovementDataParams> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureHoverTintableEffect>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureHoverTintableEffect> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureMoveAnimData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureMoveAnimData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreaturePetAccessory>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreaturePetAccessory> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreaturePetAccessorySlot>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreaturePetAccessorySlot> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreaturePetTraitRange>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetTraitRange *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureFootParticleSingleData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureFootParticleSingleData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureHoverMovementDataParams const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureHoverMovementDataParams *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureHoverTintableEffect const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureHoverTintableEffect *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureMoveAnimData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureMoveAnimData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreaturePetAccessory const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreaturePetAccessory *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreaturePetAccessorySlot const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreaturePetAccessorySlot *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreaturePetTraitRange const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreaturePetTraitRange *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCreaturePetTraitRange>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreaturePetTraitRange *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreaturePetTraitRange *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureFlockMovementData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureFlockMovementData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureFootParticleData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureFootParticleData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureFootParticleSingleData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureFootParticleSingleData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureHealthData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureHealthData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureHoverMovementData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureHoverMovementData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureHoverMovementDataParams const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureHoverMovementDataParams *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureHoverTintableEffect const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureHoverTintableEffect *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureMoveAnimData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureMoveAnimData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureMovementData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureMovementData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreaturePetAccessory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreaturePetAccessory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreaturePetAccessorySlot const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreaturePetAccessorySlot *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreaturePetData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreaturePetData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreaturePetMood const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreaturePetMood *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreaturePetPartHider const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreaturePetPartHider *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreaturePetRewardActions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreaturePetRewardActions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreaturePetTraitRange const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreaturePetTraitRange *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreaturePetTraitRanges const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreaturePetTraitRanges *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreaturePetTraits const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreaturePetTraits *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHoverMovementDataParams::~cGcCreatureHoverMovementDataParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHoverMovementDataParams *
        ]
    ),
    "cTkDynamicArray<cGcCreatureFootParticleSingleData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureFootParticleSingleData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureHoverMovementDataParams>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureHoverMovementDataParams> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureHoverTintableEffect>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureHoverTintableEffect> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureMoveAnimData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureMoveAnimData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreaturePetAccessory>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreaturePetAccessory> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCharacterJetpackEffect>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCharacterJetpackEffect> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureFlockMovementData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureFlockMovementData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureFootParticleData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureFootParticleData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureHealthData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureHealthData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureHoverMovementData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureHoverMovementData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureMovementData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureMovementData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreaturePetData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreaturePetData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureFlockMovementData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureFootParticleData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureFootParticleSingleData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureHealthData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureHoverMovementData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureHoverMovementDataParams::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureHoverTintableEffect::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureMoveAnimData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureMovementData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreaturePetAccessory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreaturePetAccessorySlot::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreaturePetData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreaturePetMood::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreaturePetPartHider::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreaturePetRewardActions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreaturePetTraitRange::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreaturePetTraitRanges::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreaturePetTraits::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureFootParticleData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureHealthData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureHoverMovementData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureHoverMovementDataParams::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureMovementData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreaturePetAccessory::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreaturePetData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreaturePetPartHider::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureHealthData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHealthData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureHealthData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHealthData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureFlockMovementData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFlockMovementData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFootParticleSingleData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFootParticleSingleData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHealthData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHealthData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHoverMovementDataParams::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHoverMovementDataParams *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHoverTintableEffect::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHoverTintableEffect *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureMoveAnimData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMoveAnimData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureMovementData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetAccessory::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetAccessory *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetAccessorySlot::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetAccessorySlot *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetMood::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetMood *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetPartHider::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetPartHider *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetRewardActions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetRewardActions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetTraitRange::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetTraitRange *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetTraitRanges::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetTraitRanges *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetTraits::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetTraits *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFlockMovementData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFlockMovementData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFootParticleSingleData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFootParticleSingleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHealthData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHealthData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHoverMovementDataParams::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHoverMovementDataParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHoverTintableEffect::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHoverTintableEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureMoveAnimData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMoveAnimData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetMood::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetMood *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetRewardActions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetRewardActions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetTraitRanges::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetTraitRanges *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetTraits::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetTraits *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFootParticleSingleData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFootParticleSingleData *
        ]
    ),
    "cGcCreatureHealthData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHealthData *
        ]
    ),
    "cGcCreatureHoverMovementDataParams::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHoverMovementDataParams *
        ]
    ),
    "cGcCreatureHoverTintableEffect::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHoverTintableEffect *
        ]
    ),
    "cGcCreatureMoveAnimData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMoveAnimData *
        ]
    ),
    "cGcCreaturePetAccessorySlot::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterJetpackEffect *
        ]
    ),
    "cGcCreaturePetTraitRanges::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetTraitRanges *
        ]
    ),
    "cGcCreatureFlockMovementData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFlockMovementData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFootParticleData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFootParticleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFootParticleSingleData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFootParticleSingleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHealthData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHealthData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHoverMovementData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHoverMovementData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHoverMovementDataParams::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHoverMovementDataParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHoverTintableEffect::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHoverTintableEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureMoveAnimData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMoveAnimData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureMovementData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetAccessory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetAccessory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetAccessorySlot::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetAccessorySlot *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetMood::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetMood *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetPartHider::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetPartHider *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetRewardActions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetRewardActions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetTraitRange::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetTraitRange *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetTraitRanges::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetTraitRanges *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreaturePetTraits::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetTraits *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFlockMovementData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFlockMovementData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureFlockMovementData *
        ]
    ),
    "cGcCreatureFootParticleSingleData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFootParticleSingleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureFootParticleSingleData *
        ]
    ),
    "cGcCreatureHealthData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHealthData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureHealthData *
        ]
    ),
    "cGcCreatureHoverMovementDataParams::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHoverMovementDataParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureHoverMovementDataParams *
        ]
    ),
    "cGcCreatureHoverTintableEffect::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHoverTintableEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureHoverTintableEffect *
        ]
    ),
    "cGcCreatureMoveAnimData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMoveAnimData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureMoveAnimData *
        ]
    ),
    "cGcCreatureMovementData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureMovementData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureMovementData *
        ]
    ),
    "cGcCreaturePetAccessory::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetAccessory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreaturePetAccessory *
        ]
    ),
    "cGcCreaturePetAccessorySlot::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetAccessorySlot *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreaturePetAccessorySlot *
        ]
    ),
    "cGcCreaturePetMood::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetMood *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreaturePetMood *
        ]
    ),
    "cGcCreaturePetRewardActions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetRewardActions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreaturePetRewardActions *
        ]
    ),
    "cGcCreaturePetTraitRange::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetTraitRange *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreaturePetTraitRange *
        ]
    ),
    "cGcCreaturePetTraits::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreaturePetTraits *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreaturePetTraits *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBehaviourPlayAnimTrigger>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBehaviourPlayAnimTrigger> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBiomeSubType>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBiomeSubType> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureCrystalMovementDataParams>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureCrystalMovementDataParams> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkBlackboardComparisonTypeEnum>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardComparisonTypeEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkBlackboardDefaultValueFloat>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueFloat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBehaviourPlayAnimTrigger const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBehaviourPlayAnimTrigger *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBiomeSubType const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBiomeSubType *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureCrystalMovementDataParams const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureCrystalMovementDataParams *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkBlackboardComparisonTypeEnum>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardComparisonTypeEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkBlackboardComparisonTypeEnum *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkBlackboardDefaultValueFloat>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardDefaultValueFloat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkBlackboardDefaultValueFloat *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBehaviourPlayAnimData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBehaviourPlayAnimData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBehaviourRegisterAttackerData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBehaviourGetTargetData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBehaviourWaitData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBehaviourWaitData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBirdData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBirdData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBlackboardFloatCompareDecoratorData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBlackboardFloatCompareDecoratorData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBlackboardIntCompareDecoratorData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBlackboardIntCompareDecoratorData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBlackboardIntModifyData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBlackboardIntModifyData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBlackboardValueDecoratorData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBlackboardValueDecoratorData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCooldownDecoratorData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCooldownDecoratorData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureAlertData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureAlertData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureCrystalMovementData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureCrystalMovementData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureCrystalMovementDataParams const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureCrystalMovementDataParams *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureFiendAttackData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureFiendAttackData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureIkType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureIkType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPrimaryAxis const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPrimaryAxis *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcBehaviourPlayAnimTrigger>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBehaviourPlayAnimTrigger> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureCrystalMovementDataParams>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureCrystalMovementDataParams> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBehaviourPlayAnimData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourPlayAnimData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourRegisterAttackerData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourRegisterAttackerData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourWaitData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourWaitData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBirdData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBirdData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBlackboardValueDecoratorData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBlackboardValueDecoratorData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureCrystalMovementData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureCrystalMovementData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureFiendAttackData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCreatureFiendAttackData *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcBehaviourPlayAnimData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourRegisterAttackerData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourWaitData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBirdData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBlackboardFloatCompareDecoratorData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBlackboardIntCompareDecoratorData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBlackboardIntModifyData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBlackboardValueDecoratorData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCooldownDecoratorData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureAlertData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureCrystalMovementData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureCrystalMovementDataParams::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureFiendAttackData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureIkType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPrimaryAxis::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourPlayAnimData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBlackboardFloatCompareDecoratorData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBlackboardIntCompareDecoratorData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBlackboardValueDecoratorData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCooldownDecoratorData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureAlertData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureCrystalMovementData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureCrystalMovementDataParams::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureCrystalMovementDataParams::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureCrystalMovementDataParams *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureCrystalMovementDataParams::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureCrystalMovementDataParams *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBehaviourPlayAnimData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourPlayAnimData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBirdData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBirdData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBlackboardIntModifyData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlackboardIntModifyData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBlackboardValueDecoratorData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlackboardValueDecoratorData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureAlertData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureAlertData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureCrystalMovementDataParams::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureCrystalMovementDataParams *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFiendAttackData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFiendAttackData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureIkType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureIkType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPrimaryAxis::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPrimaryAxis *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureAlertData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureAlertData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureCrystalMovementDataParams::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureCrystalMovementDataParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureIkType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureIkType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPrimaryAxis::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPrimaryAxis *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBirdData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBirdData *
        ]
    ),
    "cGcBlackboardFloatCompareDecoratorData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlackboardFloatCompareDecoratorData *
        ]
    ),
    "cGcBlackboardIntCompareDecoratorData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlackboardIntCompareDecoratorData *
        ]
    ),
    "cGcBlackboardIntModifyData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlackboardIntModifyData *
        ]
    ),
    "cGcCooldownDecoratorData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCooldownDecoratorData *
        ]
    ),
    "cGcCreatureAlertData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureAlertData *
        ]
    ),
    "cGcCreatureCrystalMovementDataParams::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureCrystalMovementDataParams *
        ]
    ),
    "cGcCreatureFiendAttackData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFiendAttackData *
        ]
    ),
    "cGcBehaviourPlayAnimData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourPlayAnimData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourGetTargetData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourGetTargetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourWaitData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourWaitData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBirdData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBirdData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBlackboardFloatCompareDecoratorData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlackboardFloatCompareDecoratorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBlackboardIntCompareDecoratorData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlackboardIntCompareDecoratorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBlackboardIntModifyData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlackboardIntModifyData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBlackboardValueDecoratorData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlackboardValueDecoratorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCooldownDecoratorData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCooldownDecoratorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureAlertData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureAlertData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureCrystalMovementData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureCrystalMovementData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureCrystalMovementDataParams::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureCrystalMovementDataParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFiendAttackData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFiendAttackData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureIkType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureIkType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPrimaryAxis::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPrimaryAxis *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBirdData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBirdData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBirdData *
        ]
    ),
    "cGcBlackboardIntCompareDecoratorData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBlackboardIntCompareDecoratorData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBlackboardIntCompareDecoratorData *
        ]
    ),
    "cGcCreatureAlertData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureAlertData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureAlertData *
        ]
    ),
    "cGcCreatureCrystalMovementDataParams::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureCrystalMovementDataParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureCrystalMovementDataParams *
        ]
    ),
    "cGcCreatureFiendAttackData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFiendAttackData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureFiendAttackData *
        ]
    ),
    "cGcCreatureIkType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureIkType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureIkType *
        ]
    ),
    "cGcPrimaryAxis::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPrimaryAxis *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPrimaryAxis *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCustomisationBackpackData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationBackpackData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCustomisationBobbleHead>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationBobbleHead> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCustomisationFreighterEngineEffect>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationFreighterEngineEffect> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCustomisationGroup>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationGroup> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCustomisationRace>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationRace> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCustomisationShipTrails>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationShipTrails> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCustomisationThrusterEffect>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationThrusterEffect> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCustomisationThrusterJet>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationThrusterJet> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkBlackboardDefaultValueBool>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueBool *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkBlackboardDefaultValueId>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardDefaultValueId *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationBackpackData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationBackpackData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationBobbleHead const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationBobbleHead *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationFreighterEngineEffect const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationFreighterEngineEffect *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationGroup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationGroup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationRace const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationRace *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationShipTrails const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationShipTrails *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationThrusterEffect const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationThrusterEffect *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationThrusterJet const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationThrusterJet *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationUI const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationUI *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkBlackboardDefaultValueBool>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardDefaultValueBool *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkBlackboardDefaultValueBool *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkBlackboardDefaultValueId>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardDefaultValueId *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkBlackboardDefaultValueId *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBehaviourAppearData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBehaviourAppearData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBehaviourApplyDamageData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBehaviourApplyDamageData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBehaviourCooldownBeginData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBehaviourCooldownBeginData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBehaviourDetailAnimsData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBehaviourDetailAnimsData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBehaviourFaceTargetData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBehaviourFaceTargetData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBehaviourIncrementCounter const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBehaviourIncrementCounter *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBehaviourLaunchProjectileData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBehaviourLaunchProjectileData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBehaviourLegacyData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBehaviourLegacyData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBehaviourLookData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBehaviourLookData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBehaviourMaintainRangeFromTargetData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBehaviourMaintainRangeFromTargetData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBehaviourMoveToTargetData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBehaviourMoveToTargetData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationRace const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationRace *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationShipBobbleHeads const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationShipBobbleHeads *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationShipTrails const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationShipTrails *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationTextureGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationTextureGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationThrusterEffect const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationThrusterEffect *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationThrusterEffects const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationThrusterEffects *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationThrusterJet const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationThrusterJet *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationUI const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationUI *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationUIData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationUIData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcCustomisationUI,23>::cTkFixedArray<cGcCustomisationUI,23>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcCustomisationUI,23> *
        ]
    ),
    "cTkFixedArray<cGcCustomisationUI,23>::~cTkFixedArray<cGcCustomisationUI,23>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcCustomisationUI,23> *
        ]
    ),
    "cGcCustomisationDescriptorGroupOption::~cGcCustomisationDescriptorGroupOption": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroupOption *
        ]
    ),
    "cGcCustomisationGroup::~cGcCustomisationGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationGroup *
        ]
    ),
    "cGcCustomisationThrusterJet::~cGcCustomisationThrusterJet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationThrusterJet *
        ]
    ),
    "cTkDynamicArray<cGcCustomisationBackpackData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationBackpackData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCustomisationBobbleHead>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerControlModeEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCustomisationFreighterEngineEffect>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationFreighterEngineEffect> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCustomisationGroup>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationGroup> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCustomisationRace>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationRace> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCustomisationShipTrails>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationShipTrails> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCustomisationThrusterEffect>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationThrusterEffect> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCustomisationThrusterJet>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationThrusterJet> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBehaviourAppearData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourAppearData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourApplyDamageData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourApplyDamageData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourCheckDeathData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourCheckDeathData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourDetailAnimsData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourDetailAnimsData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourFaceTargetData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourFaceTargetData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourGetTargetData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourGetTargetData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourIdleData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourIdleData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourLaunchProjectileData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourLaunchProjectileData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourLegacyData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourLegacyData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourLookData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourLookData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourMaintainRangeFromTargetData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourMaintainRangeFromTargetData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourMoveToTargetData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBehaviourMoveToTargetData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourAppearData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourApplyDamageData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourCheckDeathData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourCooldownBeginData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourDetailAnimsData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourFaceTargetData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourGetTargetData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourIdleData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourIncrementCounter::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourLaunchProjectileData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourLegacyData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourLookData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourMaintainRangeFromTargetData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourMoveToTargetData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationRace::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationShipBobbleHeads::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationShipTrails::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationTextureGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationThrusterEffect::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationThrusterEffects::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationThrusterJet::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationUI::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationUIData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBehaviourApplyDamageData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBehaviourDetailAnimsData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBehaviourLaunchProjectileData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBehaviourLookData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBehaviourMaintainRangeFromTargetData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBehaviourMoveToTargetData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationRace::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationShipBobbleHeads::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationShipTrails::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationThrusterEffect::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationThrusterEffects::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationThrusterJet::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationUI::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationUIData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcCustomisationBoneScales>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationBoneScales> *
        ]
    ),
    "cTkDynamicArray<cGcCustomisationDescriptorGroupOptions>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationDescriptorGroupOptions> *
        ]
    ),
    "cTkDynamicArray<cGcCustomisationThrusterEffect>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationThrusterEffect> *
        ]
    ),
    "cGcCustomisationRace::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationRace *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationThrusterEffect::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationThrusterEffect *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationThrusterEffects::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationThrusterEffects *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationUI::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationUI *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationRace::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationRace *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationShipBobbleHeads::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationShipBobbleHeads *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationThrusterEffect::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationThrusterEffect *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationThrusterEffects::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationThrusterEffects *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationUI::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationUI *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationUIData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationUIData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBehaviourFaceTargetData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourFaceTargetData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourLaunchProjectileData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourLaunchProjectileData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourLookData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourLookData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourMaintainRangeFromTargetData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourMaintainRangeFromTargetData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourMoveToTargetData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourMoveToTargetData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationRace::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationRace *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationShipTrails::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationShipTrails *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationTextureGroup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationTextureGroup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationThrusterEffect::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationThrusterEffect *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationThrusterJet::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationThrusterJet *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationUI::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationUI *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourLaunchProjectileData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourLaunchProjectileData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationRace::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationRace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationTextureGroup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationTextureGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationThrusterEffect::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationThrusterEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationUI::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationUI *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourApplyDamageData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourApplyDamageData *
        ]
    ),
    "cGcBehaviourDetailAnimsData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourDetailAnimsData *
        ]
    ),
    "cGcBehaviourLaunchProjectileData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourLaunchProjectileData *
        ]
    ),
    "cGcBehaviourLookData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourLookData *
        ]
    ),
    "cGcBehaviourMaintainRangeFromTargetData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourMaintainRangeFromTargetData *
        ]
    ),
    "cGcBehaviourMoveToTargetData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourMoveToTargetData *
        ]
    ),
    "cGcCustomisationTextureGroup::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationTextureGroup *
        ]
    ),
    "cGcBehaviourApplyDamageData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourApplyDamageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourCooldownBeginData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourCooldownBeginData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourDetailAnimsData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourDetailAnimsData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourFaceTargetData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourFaceTargetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourIncrementCounter::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourIncrementCounter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourLaunchProjectileData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourLaunchProjectileData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourLegacyData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourLegacyData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourLookData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourLookData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourMaintainRangeFromTargetData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourMaintainRangeFromTargetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourMoveToTargetData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourMoveToTargetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationRace::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationRace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationShipBobbleHeads::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationShipBobbleHeads *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationShipTrails::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationShipTrails *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationTextureGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationTextureGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationThrusterEffect::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationThrusterEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationThrusterEffects::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationThrusterEffects *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationThrusterJet::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationThrusterJet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationUI::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationUI *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationUIData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationUIData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBehaviourApplyDamageData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourApplyDamageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBehaviourApplyDamageData *
        ]
    ),
    "cGcBehaviourLaunchProjectileData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourLaunchProjectileData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBehaviourLaunchProjectileData *
        ]
    ),
    "cGcBehaviourMaintainRangeFromTargetData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourMaintainRangeFromTargetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBehaviourMaintainRangeFromTargetData *
        ]
    ),
    "cGcBehaviourMoveToTargetData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBehaviourMoveToTargetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBehaviourMoveToTargetData *
        ]
    ),
    "cGcCustomisationRace::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationRace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCustomisationRace *
        ]
    ),
    "cGcCustomisationShipTrails::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationShipTrails *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCustomisationShipTrails *
        ]
    ),
    "cGcCustomisationThrusterEffect::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationThrusterEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCustomisationThrusterEffect *
        ]
    ),
    "cGcCustomisationThrusterJet::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationThrusterJet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCustomisationThrusterJet *
        ]
    ),
    "cGcCustomisationUI::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationUI *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCustomisationUI *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureAlertData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureAlertData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCustomisationBannerImageData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationBannerImageData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCustomisationBoneScales>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationBoneScales> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCustomisationColourGroup>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationColourGroup> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCustomisationDescriptorGroupOption>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationDescriptorGroupOption> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCustomisationDescriptorGroupOptions>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationDescriptorGroupOptions> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCustomisationTextureGroup>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationTextureGroup> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCustomisationColourPaletteExtraData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationColourPaletteExtraData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcPaletteData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPaletteData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureAlertData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureAlertData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationBannerImageData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationBannerImageData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationBoneScales const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationBoneScales *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationColourGroup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationColourGroup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationDescriptorGroupOption const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationDescriptorGroupOption *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationDescriptorGroupOptions const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationDescriptorGroupOptions *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationTextureGroup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationTextureGroup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCustomisationCameraData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationCameraData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCustomisationCameraData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCustomisationColourPaletteExtraData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationColourPaletteExtraData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCustomisationColourPaletteExtraData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPaletteData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPaletteData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPaletteData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAnimFrameEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAnimFrameEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBeenShotEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBeenShotEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureNearbyEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureNearbyEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationBackpackData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationBackpackData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationBannerGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationBannerGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationBannerImageData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationBannerImageData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationBobbleHead const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationBobbleHead *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationBoneScales const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationBoneScales *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationCameraData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationCameraData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationColourGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationColourGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationDescriptorGroupOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationDescriptorGroupOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationDescriptorGroupOptions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationDescriptorGroupOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationFreighterEngineEffect const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationFreighterEngineEffect *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationGroups const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationGroups *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerNearbyEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerNearbyEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStateTimeEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStateTimeEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStormEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStormEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationDescriptorGroupOption::cGcCustomisationDescriptorGroupOption": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroupOption *
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroupOption *
        ]
    ),
    "cTkDynamicArray<cGcCreatureAlertData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureAlertData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCustomisationBannerImageData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationBannerImageData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCustomisationBoneScales>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationBoneScales> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCustomisationColourGroup>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationColourGroup> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCustomisationDescriptorGroupOption>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationDescriptorGroupOption> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCustomisationDescriptorGroupOptions>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationDescriptorGroupOptions> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCustomisationTextureGroup>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationTextureGroup> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAnimFrameEvent::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAnimFrameEvent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBeenShotEvent::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBeenShotEvent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureNearbyEvent::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureNearbyEvent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCustomisationComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerNearbyEvent::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerNearbyEvent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStateTimeEvent::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcStateTimeEvent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStormEvent::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcStormEvent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAnimFrameEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBeenShotEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureNearbyEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationBackpackData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationBannerGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationBannerImageData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationBobbleHead::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationBoneScales::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationCameraData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationColourGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationDescriptorGroupOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationDescriptorGroupOptions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationFreighterEngineEffect::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationGroups::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerNearbyEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStateTimeEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStormEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureNearbyEvent::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationBannerGroup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationBannerImageData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationBobbleHead::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationBoneScales::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationColourGroup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationDescriptorGroupOption::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationDescriptorGroupOptions::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationFreighterEngineEffect::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationGroup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationGroups::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationDescriptorGroupOption::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroupOption *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationDescriptorGroupOptions::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroupOptions *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationFreighterEngineEffect::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationFreighterEngineEffect *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationGroup::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationGroup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationGroups::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationGroups *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationBannerGroup::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationBannerGroup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationDescriptorGroupOption::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroupOption *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationDescriptorGroupOptions::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroupOptions *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationGroup::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationGroup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationGroups::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationGroups *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAnimFrameEvent::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAnimFrameEvent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationBackpackData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationBackpackData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationBannerImageData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationBannerImageData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationBobbleHead::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationBobbleHead *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationBoneScales::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationBoneScales *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationCameraData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationCameraData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationColourGroup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationColourGroup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationDescriptorGroupOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroupOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationDescriptorGroupOptions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroupOptions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationFreighterEngineEffect::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationFreighterEngineEffect *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationGroup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationGroup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationGroups::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationGroups *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerNearbyEvent::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNearbyEvent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStateTimeEvent::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStateTimeEvent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationBackpackData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationBackpackData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationBannerImageData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationBannerImageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationBobbleHead::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationBobbleHead *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationCameraData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationCameraData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationColourGroup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationColourGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationDescriptorGroupOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroupOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationDescriptorGroupOptions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroupOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationFreighterEngineEffect::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationFreighterEngineEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationGroup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationGroups::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationGroups *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerNearbyEvent::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNearbyEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationBackpackData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationBackpackData *
        ]
    ),
    "cGcPlayerControlModeEntry::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlModeEntry *
        ]
    ),
    "cGcCustomisationCameraData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationCameraData *
        ]
    ),
    "cGcCustomisationDescriptorGroupOption::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroupOption *
        ]
    ),
    "cGcCustomisationFreighterEngineEffect::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationFreighterEngineEffect *
        ]
    ),
    "cGcCustomisationGroup::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationGroup *
        ]
    ),
    "cGcAnimFrameEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAnimFrameEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBeenShotEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBeenShotEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureNearbyEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureNearbyEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationBackpackData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationBackpackData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationBannerGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationBannerGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationBannerImageData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationBannerImageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationBobbleHead::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationBobbleHead *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationBoneScales::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationBoneScales *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationCameraData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationCameraData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationColourGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationColourGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationDescriptorGroupOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroupOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationDescriptorGroupOptions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroupOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationFreighterEngineEffect::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationFreighterEngineEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationGroups::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationGroups *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerNearbyEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNearbyEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStateTimeEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStateTimeEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBeenShotEvent::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBeenShotEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBeenShotEvent *
        ]
    ),
    "cGcCustomisationBannerImageData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationBannerImageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCustomisationBannerImageData *
        ]
    ),
    "cGcCustomisationBoneScales::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationBoneScales *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCustomisationBoneScales *
        ]
    ),
    "cGcCustomisationCameraData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationCameraData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCustomisationCameraData *
        ]
    ),
    "cGcCustomisationColourGroup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationColourGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCustomisationColourGroup *
        ]
    ),
    "cGcCustomisationDescriptorGroupOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroupOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCustomisationDescriptorGroupOption *
        ]
    ),
    "cGcCustomisationDescriptorGroupOptions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroupOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCustomisationDescriptorGroupOptions *
        ]
    ),
    "cGcCustomisationFreighterEngineEffect::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationFreighterEngineEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCustomisationFreighterEngineEffect *
        ]
    ),
    "cGcCustomisationGroup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCustomisationGroup *
        ]
    ),
    "cGcCustomisationGroups::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationGroups *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerNearbyEvent::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerNearbyEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerNearbyEvent *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcBroadcastLevel>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBroadcastLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcBroadcastLevel>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBroadcastLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBroadcastLevel *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingSettingsAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingSettingsAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseDefenceStatusAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseDefenceStatusAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCameraShakeAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraShakeAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDestroyAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDestroyAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDisplayText const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDisplayText *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFiendCrimeAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFiendCrimeAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFrigateInteractionAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateInteractionAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGoToStateAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGoToStateAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHarvestPlantAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHarvestPlantAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHazardAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHazardAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcLodAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcLodAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNodeActivationAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNodeActivationAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPainAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPainAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcParticleAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcParticleAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayAnimAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayAnimAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayAudioAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayAudioAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPowerStateAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPowerStateAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScareCreaturesAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScareCreaturesAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpawnAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpawnAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWarpAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWarpAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingSettingsAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBaseBuildingSettingsAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCameraShakeAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCameraShakeAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDestroyAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDestroyAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDisplayText::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDisplayText *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFiendCrimeAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFiendCrimeAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFireSimpleInteractionAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFireSimpleInteractionAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateInteractionAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFrigateInteractionAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGoToStateAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGoToStateAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHarvestPlantAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcHarvestPlantAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHazardAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcHazardAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLodAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcLodAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNodeActivationAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNodeActivationAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPainAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPainAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcParticleAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcParticleAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayAnimAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayAnimAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayAudioAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayAudioAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPowerStateAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcPowerStateAction *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcResetSimpleInteractionAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcResetSimpleInteractionAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcRewardAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScareCreaturesAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcScareCreaturesAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpawnAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSpawnAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUniqueIdComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcUniqueIdComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWarpAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWarpAction *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingSettingsAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseDefenceStatusAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCameraShakeAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDestroyAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDisplayText::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFiendCrimeAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFireSimpleInteractionAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateInteractionAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGoToStateAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHarvestPlantAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHazardAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLodAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNodeActivationAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPainAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcParticleAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayAnimAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayAudioAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPowerStateAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcResetSimpleInteractionAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScareCreaturesAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpawnAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUniqueIdComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWarpAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingSettingsAction::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseDefenceStatusAction::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDisplayText::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFiendCrimeAction::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGoToStateAction::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcHazardAction::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcParticleAction::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCameraShakeAction::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeAction *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDestroyAction::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestroyAction *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDisplayText::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDisplayText *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateInteractionAction::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateInteractionAction *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGoToStateAction::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGoToStateAction *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHazardAction::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHazardAction *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNodeActivationAction::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNodeActivationAction *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPainAction::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPainAction *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcParticleAction::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleAction *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayAudioAction::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayAudioAction *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraShakeAction::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeAction *
        ]
    ),
    "cGcDisplayText::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDisplayText *
        ]
    ),
    "cGcNodeActivationAction::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNodeActivationAction *
        ]
    ),
    "cGcPainAction::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPainAction *
        ]
    ),
    "cGcParticleAction::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleAction *
        ]
    ),
    "cGcPlayAudioAction::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayAudioAction *
        ]
    ),
    "cGcBaseBuildingSettingsAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingSettingsAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseDefenceStatusAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceStatusAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraShakeAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDestroyAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDestroyAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDisplayText::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDisplayText *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFiendCrimeAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendCrimeAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateInteractionAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateInteractionAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGoToStateAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGoToStateAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHarvestPlantAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHarvestPlantAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHazardAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHazardAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLodAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLodAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNodeActivationAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNodeActivationAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPainAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPainAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcParticleAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcParticleAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayAudioAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayAudioAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPowerStateAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPowerStateAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScareCreaturesAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScareCreaturesAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWarpAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWarpAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingSettingsAction::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingSettingsAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingSettingsAction *
        ]
    ),
    "cGcDisplayText::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDisplayText *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDisplayText *
        ]
    ),
    "cGcFiendCrimeAction::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendCrimeAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFiendCrimeAction *
        ]
    ),
    "cGcHazardAction::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHazardAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcHazardAction *
        ]
    ),
    "cGcNodeActivationAction::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNodeActivationAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNodeActivationAction *
        ]
    ),
    "cGcPainAction::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPainAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPainAction *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcActionTrigger>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcActionTrigger> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcActionTriggerState>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcActionTriggerState> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCharacterJetpackEffect>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCharacterJetpackEffect> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcFreighterDungeonChoice>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFreighterDungeonChoice> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcInventoryTechProbability>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInventoryTechProbability> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerControlModeEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerControlModeEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerControlState>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerControlState> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkBlackboardKey>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardKey *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcActionTrigger const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcActionTrigger *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcActionTriggerState const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcActionTriggerState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCharacterJetpackEffect const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCharacterJetpackEffect *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFreighterBaseOptions const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFreighterBaseOptions *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFreighterDungeonChoice const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFreighterDungeonChoice *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInventoryTechProbability const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInventoryTechProbability *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerControlModeEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerControlModeEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerControlState const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerControlState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkBlackboardKey>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkBlackboardKey *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkBlackboardKey *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAbandonedFreighterComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAbandonedFreighterComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcActionTrigger const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcActionTrigger *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcActionTriggerState const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcActionTriggerState *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseDefenceStatusType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseDefenceStatusType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseDefenceTrigger const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseDefenceTrigger *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBroadcastLevel const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBroadcastLevel *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCameraShakeComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraShakeComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCameraShakeTriggerData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraShakeTriggerData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcChainComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcChainComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcColouriseComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcColouriseComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomInventoryComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomInventoryComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDecorationComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDecorationComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFreighterBaseComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFreighterBaseComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryTechProbability const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryTechProbability *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcLootProbability const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcLootProbability *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMarkerComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMarkerComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMetaBallComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMetaBallComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMultitoolPoolData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMultitoolPoolData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerCharacterComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerCharacterComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerControlComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerControlComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTriggerActionComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTriggerActionComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerControlInput::cGcPlayerControlInput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlInput *
            ctypes.c_ulonglong,  # cGcPlayerControlInput *
        ]
    ),
    "cGcPlayerControlState::~cGcPlayerControlState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlState *
        ]
    ),
    "cTkDynamicArray<cGcActionTrigger>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcActionTrigger> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcActionTriggerState>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcActionTriggerState> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcFreighterDungeonChoice>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFreighterDungeonChoice> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcInventoryTechProbability>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInventoryTechProbability> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPlayerControlState>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerControlState> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAbandonedFreighterComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAbandonedFreighterComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCameraShakeComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCameraShakeComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCharacterInterfaceComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCharacterInterfaceComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcColouriseComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcColouriseComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomInventoryComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCustomInventoryComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDecorationComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDecorationComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFreighterBaseComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFreighterBaseComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMarkerComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMarkerComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerCharacterComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerCharacterComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerControlComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerControlComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTriggerActionComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTriggerActionComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAbandonedFreighterComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcActionTrigger::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcActionTriggerState::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseDefenceStatusType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseDefenceTrigger::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBroadcastLevel::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCameraShakeComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCameraShakeTriggerData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcChainComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCharacterInterfaceComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcColouriseComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomInventoryComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDecorationComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFreighterBaseComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryTechProbability::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLootProbability::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMarkerComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMetaBallComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMultitoolPoolData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerCharacterComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerControlComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTriggerActionComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAbandonedFreighterComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcActionTrigger::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcActionTriggerState::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomInventoryComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFreighterBaseComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcLootProbability::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMarkerComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerCharacterComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerControlComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTriggerActionComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcActionTrigger>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcActionTrigger> *
        ]
    ),
    "cTkDynamicArray<cGcActionTriggerState>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcActionTriggerState> *
        ]
    ),
    "cGcActionTrigger::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionTrigger *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcActionTrigger::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionTrigger *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAbandonedFreighterComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActionTrigger::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionTrigger *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActionTriggerState::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionTriggerState *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseDefenceStatusType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceStatusType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseDefenceTrigger::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceTrigger *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBroadcastLevel::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBroadcastLevel *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraShakeComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraShakeTriggerData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeTriggerData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcColouriseComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcColouriseComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomInventoryComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomInventoryComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterBaseComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryTechProbability::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryTechProbability *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLootProbability::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLootProbability *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMarkerComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMetaBallComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMetaBallComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMultitoolPoolData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultitoolPoolData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCharacterComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerControlComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTriggerActionComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseDefenceStatusType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceStatusType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseDefenceTrigger::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceTrigger *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBroadcastLevel::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBroadcastLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterBaseComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryTechProbability::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryTechProbability *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLootProbability::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLootProbability *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMarkerComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMultitoolPoolData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultitoolPoolData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseDefenceTrigger::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceTrigger *
        ]
    ),
    "cGcCameraShakeComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeComponentData *
        ]
    ),
    "cGcCameraShakeTriggerData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeTriggerData *
        ]
    ),
    "cGcChainComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcChainComponentData *
        ]
    ),
    "cGcFreighterBaseComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponentData *
        ]
    ),
    "cGcLootProbability::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLootProbability *
        ]
    ),
    "cGcMetaBallComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMetaBallComponentData *
        ]
    ),
    "cGcMultitoolPoolData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultitoolPoolData *
        ]
    ),
    "cGcAbandonedFreighterComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActionTrigger::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionTrigger *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActionTriggerState::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionTriggerState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseDefenceStatusType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceStatusType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseDefenceTrigger::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceTrigger *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBroadcastLevel::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBroadcastLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraShakeComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraShakeTriggerData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeTriggerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcChainComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcChainComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcColouriseComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcColouriseComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomInventoryComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomInventoryComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDecorationComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDecorationComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterBaseComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryTechProbability::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryTechProbability *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLootProbability::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLootProbability *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMarkerComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMetaBallComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMetaBallComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMultitoolPoolData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultitoolPoolData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerCharacterComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerCharacterComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerControlComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTriggerActionComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAbandonedFreighterComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAbandonedFreighterComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAbandonedFreighterComponentData *
        ]
    ),
    "cGcActionTriggerState::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionTriggerState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcActionTriggerState *
        ]
    ),
    "cGcBaseDefenceStatusType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceStatusType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseDefenceStatusType *
        ]
    ),
    "cGcBaseDefenceTrigger::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceTrigger *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseDefenceTrigger *
        ]
    ),
    "cGcBroadcastLevel::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBroadcastLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBroadcastLevel *
        ]
    ),
    "cGcColouriseComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcColouriseComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcColouriseComponentData *
        ]
    ),
    "cGcInventoryTechProbability::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryTechProbability *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryTechProbability *
        ]
    ),
    "cGcMarkerComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMarkerComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMarkerComponentData *
        ]
    ),
    "cGcMetaBallComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMetaBallComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMetaBallComponentData *
        ]
    ),
    "cGcMultitoolPoolData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultitoolPoolData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMultitoolPoolData *
        ]
    ),
    "cGcPlayerControlComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerControlComponentData *
        ]
    ),
    "cGcTriggerActionComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTriggerActionComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTriggerActionComponentData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcClothPiece>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcClothPiece> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcIKConstraint>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcIKConstraint> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcOutpostLSystemPair>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcOutpostLSystemPair> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcScanEffectData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcScanEffectData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSpringLink>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSpringLink> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcIKConstraint>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIKConstraint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcClothPiece const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcClothPiece *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcIKConstraint const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcIKConstraint *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcOutpostLSystemPair const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcOutpostLSystemPair *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcScanEffectData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcScanEffectData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSpringLink const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSpringLink *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcIKConstraint>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcIKConstraint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcIKConstraint *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcClothComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcClothComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcEncyclopediaComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcEncyclopediaComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMiningComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMiningComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNetworkInterpolationComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNetworkInterpolationComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcOutpostComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcOutpostComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcOutpostLSystemPair const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcOutpostLSystemPair *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerFullBodyIKComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerFullBodyIKComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScanEffectComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScanEffectComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScannableComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScannableComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpringAttachmentComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpringAttachmentComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcClothPiece::cGcClothPiece": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothPiece *
            ctypes.c_ulonglong,  # cGcClothPiece *
        ]
    ),
    "cGcClothPiece::~cGcClothPiece": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothPiece *
        ]
    ),
    "cTkDynamicArray<cGcClothPiece>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcClothPiece> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcIKConstraint>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcIKConstraint> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcOutpostLSystemPair>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcOutpostLSystemPair> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcScanEffectData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcScanEffectData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSpringLink>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSpringLink> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcClothComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcClothComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEncyclopediaComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcEncyclopediaComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcNetworkInterpolationComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNetworkInterpolationComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNetworkPlayerMarkerComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNetworkPlayerMarkerComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcOutpostComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcOutpostComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerFullBodyIKComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerWeaponComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerWeaponComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScanEffectComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcScanEffectComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScannableComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcScannableComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpringAttachmentComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSpringAttachmentComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcClothComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEncyclopediaComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMiningComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNetworkInterpolationComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNetworkPlayerMarkerComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcOutpostComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcOutpostLSystemPair::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerFullBodyIKComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerWeaponComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScanEffectComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScannableComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpringAttachmentComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcClothComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcEncyclopediaComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcOutpostComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerFullBodyIKComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScanEffectComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScannableComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSpringAttachmentComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcAttachmentPointSet>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAttachmentPointSet> *
        ]
    ),
    "cTkDynamicArray<cGcIKConstraint>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcIKConstraint> *
        ]
    ),
    "cTkDynamicArray<cGcSpringLink>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSpringLink> *
        ]
    ),
    "cGcPlayerFullBodyIKComponentData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponentData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerFullBodyIKComponentData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponentData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcClothComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOutpostComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOutpostLSystemPair::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostLSystemPair *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerFullBodyIKComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannableComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannableComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpringAttachmentComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpringAttachmentComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcClothComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOutpostComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerFullBodyIKComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannableComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannableComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOutpostComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponentData *
        ]
    ),
    "cGcOutpostLSystemPair::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostLSystemPair *
        ]
    ),
    "cGcScanEffectComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEffectComponentData *
        ]
    ),
    "cGcScannableComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannableComponentData *
        ]
    ),
    "cGcSpringAttachmentComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpringAttachmentComponentData *
        ]
    ),
    "cGcClothComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEncyclopediaComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncyclopediaComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMiningComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMiningComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNetworkInterpolationComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNetworkInterpolationComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOutpostComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOutpostLSystemPair::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostLSystemPair *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerFullBodyIKComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScanEffectComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEffectComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannableComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannableComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpringAttachmentComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpringAttachmentComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcClothComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcClothComponentData *
        ]
    ),
    "cGcOutpostComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcOutpostComponentData *
        ]
    ),
    "cGcOutpostLSystemPair::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutpostLSystemPair *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcOutpostLSystemPair *
        ]
    ),
    "cGcPlayerFullBodyIKComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerFullBodyIKComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerFullBodyIKComponentData *
        ]
    ),
    "cGcScannableComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannableComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScannableComponentData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCharacterAlternateAnimation>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCharacterAlternateAnimation> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcMaintenanceElement>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMaintenanceElement> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNPCInteractiveObjectState>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCInteractiveObjectState> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkModelRendererData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelRendererData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCharacterAlternateAnimation const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCharacterAlternateAnimation *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMaintenanceElement const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMaintenanceElement *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCInteractiveObjectState const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCInteractiveObjectState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkModelRendererData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkModelRendererData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkModelRendererData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInteractionDof const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInteractionDof *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInteractionType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInteractionType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcLookAtComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcLookAtComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMaintenanceComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMaintenanceComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCInteractiveObjectComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCInteractiveObjectComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRefinerUnitComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRefinerUnitComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRocketLockerComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRocketLockerComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSentinelCoverComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSentinelCoverComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSquadronHologramComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSquadronHologramComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTechnologyAttachmentComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTechnologyAttachmentComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcMaintenanceElement>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMaintenanceElement> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcNPCInteractiveObjectState>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCInteractiveObjectState> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcLadderComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcLadderComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLookAtComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcLookAtComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMaintenanceComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcMaintenanceComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCInteractiveObjectComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCInteractiveObjectComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRefinerUnitComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcRefinerUnitComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRocketLockerComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcRocketLockerComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSentinelCoverComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSentinelCoverComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSquadronHologramComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSquadronHologramComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTechnologyAttachmentComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTechnologyAttachmentComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInteractionDof::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInteractionType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLadderComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLookAtComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMaintenanceComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCInteractiveObjectComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRefinerUnitComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRocketLockerComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSentinelCoverComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSquadronHologramComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTechnologyAttachmentComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMaintenanceComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCInteractiveObjectComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRefinerUnitComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTechnologyAttachmentComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcNPCInteractiveObjectState>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCInteractiveObjectState> *
        ]
    ),
    "cTkDynamicArray<cGcNPCInteractiveObjectStateTransition>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCInteractiveObjectStateTransition> *
        ]
    ),
    "cTkDynamicArray<cGcNPCProbabilityAnimationData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNPCProbabilityAnimationData> *
        ]
    ),
    "cGcMaintenanceComponentData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponentData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMaintenanceComponentData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponentData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInteractionDof::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionDof *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLookAtComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLookAtComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCInteractiveObjectComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelCoverComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelCoverComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSquadronHologramComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronHologramComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnologyAttachmentComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyAttachmentComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionType::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionType *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcInteractionDof::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionDof *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelCoverComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelCoverComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionDof::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionDof *
        ]
    ),
    "cGcLookAtComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLookAtComponentData *
        ]
    ),
    "cGcMaintenanceComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponentData *
        ]
    ),
    "cGcSentinelCoverComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelCoverComponentData *
        ]
    ),
    "cGcInteractionType::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionType *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcInteractionDof::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionDof *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLookAtComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLookAtComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCInteractiveObjectComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRefinerUnitComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRocketLockerComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRocketLockerComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelCoverComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelCoverComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSquadronHologramComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronHologramComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnologyAttachmentComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyAttachmentComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionDof::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionDof *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInteractionDof *
        ]
    ),
    "cGcInteractionType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInteractionType *
        ]
    ),
    "cGcLookAtComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLookAtComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcLookAtComponentData *
        ]
    ),
    "cGcMaintenanceComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMaintenanceComponentData *
        ]
    ),
    "cGcNPCComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCComponentData *
        ]
    ),
    "cGcNPCInteractiveObjectComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCInteractiveObjectComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCInteractiveObjectComponentData *
        ]
    ),
    "cGcRefinerUnitComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerUnitComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRefinerUnitComponentData *
        ]
    ),
    "cGcSentinelCoverComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelCoverComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSentinelCoverComponentData *
        ]
    ),
    "cGcSquadronHologramComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronHologramComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSquadronHologramComponentData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcAlienPuzzleMissionOverride>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAlienPuzzleMissionOverride> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcAntagonistPerception>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAntagonistPerception> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCombatEffectsProperties>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsProperties *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkModelRendererData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkModelRendererData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCharacterControlInputValidity>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterControlInputValidity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCharacterControlOutputSpace>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterControlOutputSpace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcInputActions>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcInteractionActivationCost>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionActivationCost *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcInteractionType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcMaintenanceComponentData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAlienPuzzleMissionOverride const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAlienPuzzleMissionOverride *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAntagonistEnemy const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAntagonistEnemy *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAntagonistFriend const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAntagonistFriend *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAntagonistPerception const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAntagonistPerception *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCombatEffectsProperties const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCombatEffectsProperties *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkModelRendererData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkModelRendererData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCharacterControlInputValidity>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCharacterControlInputValidity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCharacterControlInputValidity *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCharacterControlOutputSpace>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCharacterControlOutputSpace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCharacterControlOutputSpace *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcInputActions>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInputActions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInputActions *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcInteractionActivationCost>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInteractionActivationCost *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInteractionActivationCost *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcInteractionType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInteractionType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInteractionType *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcMaintenanceComponentData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMaintenanceComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMaintenanceComponentData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAntagonistComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAntagonistComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAudioAreaTriggerComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAudioAreaTriggerComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCombatEffectsComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCombatEffectsComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCombatEffectsProperties const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCombatEffectsProperties *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureFeederComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureFeederComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureHarvesterComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureHarvesterComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDissolveEffectComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDissolveEffectComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcEggMachineComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcEggMachineComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcEncounterComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcEncounterComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExpeditionHologramComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExpeditionHologramComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGeneratorUnitComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGeneratorUnitComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInteractionComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInteractionComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCHabitationComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCHabitationComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerControlInputAxis const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerControlInputAxis *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerControlInputMouse const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerControlInputMouse *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerControlInputRemap const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerControlInputRemap *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerEffectsComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerEffectsComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcAntagonistPerception>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAntagonistPerception> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkModelRendererData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkModelRendererData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAntagonistComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAntagonistComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAudioAreaTriggerComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAudioAreaTriggerComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcByteBeatPlayerComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcByteBeatPlayerComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcChairComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcChairComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCombatEffectsComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCombatEffectsComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureFeederComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureFeederComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureHarvesterComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureHarvesterComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDissolveEffectComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDissolveEffectComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEggMachineComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEggMachineComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEncounterComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEncounterComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEncounterStateComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEncounterStateComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExpeditionHologramComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcExpeditionHologramComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFleetHologramComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcFleetHologramComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGeneratorUnitComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGeneratorUnitComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHologramComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcHologramComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInteractionComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcInteractionComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCHabitationComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCHabitationComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerControlInputAxis::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcPlayerControlInputAxis *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcPlayerControlInputMouse::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcPlayerControlInputMouse *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcPlayerControlInputRemap::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerControlInputRemap *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerEffectsComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPlayerEffectsComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAntagonistComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAudioAreaTriggerComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcByteBeatPlayerComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcChairComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCombatEffectsComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCombatEffectsProperties::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureFeederComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureHarvesterComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDissolveEffectComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEggMachineComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEncounterComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEncounterStateComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExpeditionHologramComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFleetHologramComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGeneratorUnitComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHologramComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInteractionComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCHabitationComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerControlInputAxis::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerControlInputMouse::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerControlInputRemap::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerEffectsComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAntagonistComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAudioAreaTriggerComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureFeederComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCreatureHarvesterComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcEggMachineComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcEncounterComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGeneratorUnitComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInteractionComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCHabitationComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerControlInputAxis::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerControlInputMouse::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerControlInputRemap::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcEncounterComponentData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponentData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInteractionComponentData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponentData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcEncounterComponentData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponentData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAntagonistComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudioAreaTriggerComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioAreaTriggerComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatEffectsProperties::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsProperties *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFeederComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFeederComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEncounterComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionHologramComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionHologramComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratorUnitComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratorUnitComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerControlInputAxis::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlInputAxis *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerControlInputMouse::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlInputMouse *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerEffectsComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEffectsComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAntagonistComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatEffectsComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatEffectsProperties::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEncounterComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratorUnitComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratorUnitComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAntagonistComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistComponentData *
        ]
    ),
    "cGcCombatEffectsComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsComponentData *
        ]
    ),
    "cGcInteractionComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponentData *
        ]
    ),
    "cGcAntagonistComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudioAreaTriggerComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioAreaTriggerComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatEffectsComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatEffectsProperties::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureFeederComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFeederComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureHarvesterComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureHarvesterComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDissolveEffectComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDissolveEffectComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEggMachineComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEggMachineComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEncounterComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionHologramComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionHologramComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratorUnitComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratorUnitComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCHabitationComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCHabitationComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerControlInputAxis::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlInputAxis *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerControlInputMouse::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlInputMouse *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerControlInputRemap::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlInputRemap *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerEffectsComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerEffectsComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAntagonistComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAntagonistComponentData *
        ]
    ),
    "cGcCombatEffectsComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatEffectsProperties::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCombatEffectsProperties *
        ]
    ),
    "cGcCreatureFeederComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureFeederComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureFeederComponentData *
        ]
    ),
    "cGcEncounterComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcEncounterComponentData *
        ]
    ),
    "cGcExpeditionHologramComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionHologramComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcExpeditionHologramComponentData *
        ]
    ),
    "cGcInteractionComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInteractionComponentData *
        ]
    ),
    "cGcNPCHabitationComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCHabitationComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCHabitationComponentData *
        ]
    ),
    "cGcPlayerControlInputMouse::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlInputMouse *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerControlInputMouse *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCameraAmbientBuildingData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCameraAmbientBuildingData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCameraAmbientSpaceData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCameraAmbientSpaceData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCameraAmbientSpecialData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCameraAmbientSpecialData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcInputActions>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInputActions> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcRewardTableItem>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRewardTableItem> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCameraAmbientBuildingData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCameraAmbientBuildingData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCameraAmbientSpaceData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCameraAmbientSpaceData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCameraAmbientSpecialData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCameraAmbientSpecialData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInputActions const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInputActions *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcRewardDestructEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcRewardDestructEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcRewardDestructRarities const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcRewardDestructRarities *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcRewardTableItem const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcRewardTableItem *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlayerControlInput>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerControlInput *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerControlInput *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAmbientModeCameras const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAmbientModeCameras *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCharacterAlternateAnimation const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCharacterAlternateAnimation *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCharacterControlInputValidity const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCharacterControlInputValidity *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCharacterControlOutputSpace const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCharacterControlOutputSpace *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCharacterJetpackEffect const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCharacterJetpackEffect *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCharacterLookAtData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCharacterLookAtData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCharacterMove const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCharacterMove *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCharacterRotate const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCharacterRotate *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExactResource const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExactResource *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryValueData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryValueData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerControlInput const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerControlInput *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerControlModeEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerControlModeEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerControlState const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerControlState *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardDestructEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardDestructEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardDestructRarities const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardDestructRarities *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardDestructTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardDestructTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardTableEntitlementItem const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardTableEntitlementItem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardTableItemList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardTableItemList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatIconTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatIconTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatsBonus const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatsBonus *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatsTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatsTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cgcwordcategorytableEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cgcwordcategorytableEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcCameraAmbientBuildingData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCameraAmbientBuildingData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCameraAmbientSpecialData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCameraAmbientSpecialData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCharacterMove::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCharacterMove *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCharacterRotate::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCharacterRotate *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcAmbientModeCameras::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCharacterAlternateAnimation::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCharacterControlInputValidity::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCharacterControlOutputSpace::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCharacterJetpackEffect::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCharacterLookAtData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCharacterMove::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCharacterRotate::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExactResource::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryValueData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerControlInput::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerControlModeEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerControlState::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardDestructEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardDestructRarities::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardDestructTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardTableEntitlementItem::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardTableItemList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatIconTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatsBonus::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatsTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcValueData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cgcwordcategorytableEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAmbientModeCameras::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCharacterRotate::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerControlInput::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerControlModeEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerControlState::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardTableEntitlementItem::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardTableItemList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerControlInput::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlInput *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerControlState::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlState *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardTableEntitlementItem::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableEntitlementItem *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardTableItemList::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableItemList *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAmbientModeCameras::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAmbientModeCameras *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerControlInput::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlInput *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerControlState::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlState *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardTableEntitlementItem::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableEntitlementItem *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardTableItemList::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableItemList *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCharacterAlternateAnimation::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterAlternateAnimation *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterControlInputValidity::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterControlInputValidity *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterControlOutputSpace::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterControlOutputSpace *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterJetpackEffect::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterJetpackEffect *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterLookAtData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterLookAtData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterMove::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterMove *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterRotate::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterRotate *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExactResource::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExactResource *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryValueData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryValueData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerControlInput::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlInput *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerControlModeEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlModeEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerControlState::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlState *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDestructEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDestructEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDestructRarities::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDestructRarities *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTableEntitlementItem::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableEntitlementItem *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTableItemList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableItemList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsBonus::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsBonus *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsTypes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsTypes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cgcwordcategorytableEnum::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cgcwordcategorytableEnum *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExactResource::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExactResource *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcStatsBonus::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsBonus *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcStatsTypes::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsTypes *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcCharacterAlternateAnimation::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterAlternateAnimation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterControlInputValidity::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterControlInputValidity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterControlOutputSpace::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterControlOutputSpace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterJetpackEffect::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterJetpackEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterLookAtData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterLookAtData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExactResource::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExactResource *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryValueData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryValueData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerControlInput::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlInput *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerControlModeEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlModeEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerControlState::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDestructRarities::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDestructRarities *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTableEntitlementItem::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableEntitlementItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTableItemList::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableItemList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsBonus::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsBonus *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsTypes::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cgcwordcategorytableEnum::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cgcwordcategorytableEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterLookAtData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterLookAtData *
        ]
    ),
    "cGcExactResource::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExactResource *
        ]
    ),
    "cGcInventoryValueData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryValueData *
        ]
    ),
    "cGcPlayerControlState::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlState *
        ]
    ),
    "cGcRewardTableItemList::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableItemList *
        ]
    ),
    "cGcStatIconTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatIconTable *
        ]
    ),
    "cGcStatsBonus::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsBonus *
        ]
    ),
    "cGcExactResource::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExactResource *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcStatsBonus::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsBonus *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcStatsTypes::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsTypes *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcAmbientModeCameras::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAmbientModeCameras *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterAlternateAnimation::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterAlternateAnimation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterControlInputValidity::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterControlInputValidity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterControlOutputSpace::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterControlOutputSpace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterJetpackEffect::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterJetpackEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterLookAtData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterLookAtData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterMove::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterMove *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterRotate::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterRotate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExactResource::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExactResource *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryValueData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryValueData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerControlInput::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlInput *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerControlModeEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlModeEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerControlState::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDestructEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDestructEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDestructRarities::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDestructRarities *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDestructTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDestructTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTableEntitlementItem::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableEntitlementItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTableItemList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableItemList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatIconTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatIconTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsBonus::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsBonus *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cgcwordcategorytableEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cgcwordcategorytableEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterControlInputValidity::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterControlInputValidity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCharacterControlInputValidity *
        ]
    ),
    "cGcCharacterControlOutputSpace::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterControlOutputSpace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCharacterControlOutputSpace *
        ]
    ),
    "cGcCharacterLookAtData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterLookAtData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCharacterLookAtData *
        ]
    ),
    "cGcCharacterRotate::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterRotate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCharacterRotate *
        ]
    ),
    "cGcExactResource::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExactResource *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcExactResource *
        ]
    ),
    "cGcInventoryValueData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryValueData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryValueData *
        ]
    ),
    "cGcPlayerControlInput::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlInput *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerControlInput *
        ]
    ),
    "cGcPlayerControlModeEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerControlModeEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerControlModeEntry *
        ]
    ),
    "cGcRewardTableItemList::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableItemList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardTableItemList *
        ]
    ),
    "cGcStatsTypes::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatsTypes *
        ]
    ),
    "cgcwordcategorytableEnum::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cgcwordcategorytableEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cgcwordcategorytableEnum *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcUnlockableItemTree>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUnlockableItemTree> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcUnlockableItemTreeNode>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUnlockableItemTreeNode> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcUnlockableItemTrees>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTrees *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcUnlockablePlatformReward>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUnlockablePlatformReward> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcUnlockableSeasonReward>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUnlockableSeasonReward> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcUnlockableTreeCostType>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUnlockableTreeCostType> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcUnlockableTwitchReward>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUnlockableTwitchReward> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcTradeCategory>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcTradingCategoryData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcTradingCategoryData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcTradingClassData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcTradingClassData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcUnlockableItemTree const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcUnlockableItemTree *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcUnlockableItemTreeNode const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcUnlockableItemTreeNode *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcUnlockableItemTrees const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcUnlockableItemTrees *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcUnlockablePlatformReward const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcUnlockablePlatformReward *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcUnlockableSeasonReward const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcUnlockableSeasonReward *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcUnlockableTreeCostType const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcUnlockableTreeCostType *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcUnlockableTwitchReward const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcUnlockableTwitchReward *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcTradeCategory>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTradeCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTradeCategory *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcUnlockableItemTreeNode>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUnlockableItemTreeNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcUnlockableItemTreeNode *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTradingClassData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTradingClassData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTradingClassTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTradingClassTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUnlockableItemTree const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUnlockableItemTree *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUnlockableItemTreeGroups const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUnlockableItemTreeGroups *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUnlockableItemTreeNode const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUnlockableItemTreeNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUnlockableItemTrees const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUnlockableItemTrees *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUnlockablePlatformReward const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUnlockablePlatformReward *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUnlockablePlatformRewards const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUnlockablePlatformRewards *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUnlockableSeasonReward const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUnlockableSeasonReward *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUnlockableSeasonRewards const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUnlockableSeasonRewards *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUnlockableTreeCostType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUnlockableTreeCostType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUnlockableTrees const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUnlockableTrees *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUnlockableTwitchReward const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUnlockableTwitchReward *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUnlockableTwitchRewards const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUnlockableTwitchRewards *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeaponInventoryMaxUpgradeCapacity const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeaponInventoryMaxUpgradeCapacity *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeightedFilename const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeightedFilename *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWeightingCurve const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeightingCurve *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcTradingCategoryData,9>::cTkFixedArray<cGcTradingCategoryData,9>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcTradingCategoryData,9> *
        ]
    ),
    "cTkFixedArray<cGcTradingClassData,7>::cTkFixedArray<cGcTradingClassData,7>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcTradingClassData,7> *
        ]
    ),
    "cTkDynamicArray<cGcUnlockableItemTree>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUnlockableItemTree> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcUnlockableSeasonReward>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUnlockableSeasonReward> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcUnlockableTreeCostType>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUnlockableTreeCostType> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcUnlockableTwitchReward>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUnlockableTwitchReward> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcTradingClassData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTradingClassTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUnlockableItemTree::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUnlockableItemTreeGroups::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUnlockableItemTreeNode::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUnlockableItemTrees::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUnlockablePlatformReward::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUnlockablePlatformRewards::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUnlockableSeasonReward::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUnlockableSeasonRewards::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUnlockableTreeCostType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUnlockableTrees::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUnlockableTwitchReward::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUnlockableTwitchRewards::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeaponInventoryMaxUpgradeCapacity::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeightedFilename::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWeightingCurve::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTradingClassData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTradingClassTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUnlockableItemTree::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUnlockableItemTreeNode::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUnlockableItemTrees::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUnlockablePlatformRewards::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUnlockableSeasonReward::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUnlockableSeasonRewards::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUnlockableTreeCostType::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUnlockableTrees::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUnlockableTwitchRewards::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcUnlockableItemTree>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUnlockableItemTree> *
        ]
    ),
    "cGcByteBeatTemplate::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTemplate *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUnlockableItemTrees::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTrees *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTradingClassTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingClassTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUnlockableItemTreeNode::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTreeNode *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUnlockableItemTrees::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTrees *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUnlockableSeasonRewards::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableSeasonRewards *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUnlockableTrees::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableTrees *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTradingClassData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingClassData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradingClassTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingClassTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableItemTree::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTree *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableItemTreeGroups::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTreeGroups *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableItemTreeNode::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTreeNode *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableItemTrees::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTrees *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockablePlatformReward::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockablePlatformReward *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableSeasonReward::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableSeasonReward *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableTreeCostType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableTreeCostType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableTwitchReward::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableTwitchReward *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeaponInventoryMaxUpgradeCapacity::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeaponInventoryMaxUpgradeCapacity *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeightedFilename::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedFilename *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeightingCurve::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightingCurve *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradingClassData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingClassData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradingClassTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingClassTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableItemTree::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTree *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableItemTreeGroups::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTreeGroups *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableSeasonReward::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableSeasonReward *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableTreeCostType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableTreeCostType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeaponInventoryMaxUpgradeCapacity::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeaponInventoryMaxUpgradeCapacity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeightedFilename::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedFilename *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeightingCurve::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightingCurve *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradingClassTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingClassTable *
        ]
    ),
    "cGcUnlockableTrees::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableTrees *
        ]
    ),
    "cGcTradingClassData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingClassData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradingClassTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingClassTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableItemTree::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTree *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableItemTreeGroups::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTreeGroups *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableItemTreeNode::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTreeNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableItemTrees::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTrees *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockablePlatformReward::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockablePlatformReward *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockablePlatformRewards::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockablePlatformRewards *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableSeasonReward::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableSeasonReward *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableSeasonRewards::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableSeasonRewards *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableTreeCostType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableTreeCostType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableTrees::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableTrees *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableTwitchReward::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableTwitchReward *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUnlockableTwitchRewards::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableTwitchRewards *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeaponInventoryMaxUpgradeCapacity::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeaponInventoryMaxUpgradeCapacity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeightedFilename::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightedFilename *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeightingCurve::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightingCurve *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradingClassData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingClassData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTradingClassData *
        ]
    ),
    "cGcUnlockableItemTree::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTree *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcUnlockableItemTree *
        ]
    ),
    "cGcUnlockableItemTreeGroups::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTreeGroups *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcUnlockableItemTreeGroups *
        ]
    ),
    "cGcUnlockableItemTreeNode::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTreeNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcUnlockableItemTreeNode *
        ]
    ),
    "cGcUnlockableItemTrees::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableItemTrees *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcUnlockableItemTrees *
        ]
    ),
    "cGcUnlockablePlatformReward::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockablePlatformReward *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcUnlockablePlatformReward *
        ]
    ),
    "cGcUnlockableSeasonReward::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableSeasonReward *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcUnlockableSeasonReward *
        ]
    ),
    "cGcUnlockableTreeCostType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableTreeCostType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcUnlockableTreeCostType *
        ]
    ),
    "cGcUnlockableTwitchReward::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUnlockableTwitchReward *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcUnlockableTwitchReward *
        ]
    ),
    "cGcWeaponInventoryMaxUpgradeCapacity::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeaponInventoryMaxUpgradeCapacity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWeightingCurve::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightingCurve *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWeightingCurve *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcRealityCraftingRecipeData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRealityCraftingRecipeData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcRealitySubstanceData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRealitySubstanceData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSubstanceSecondary>": {
        "cTkDynamicArray<cGcSubstanceSecondary> *, XMLNode *, const char *, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDynamicArray<cGcSubstanceSecondary> *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcSubstanceSecondary *, int, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcSubstanceSecondary *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "cTkMetaDataXML::ReadClassArrayXML<cGcTechBoxData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTechBoxData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcTechnology>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTechnology> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcTechnologyRequirement>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTechnologyRequirement> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcTechnologyRarity>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyRarity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcTradeData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcRealityCraftingRecipeData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcRealityCraftingRecipeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcRealitySubstanceData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcRealitySubstanceData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSubstanceSecondary const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSubstanceSecondary *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcTechBoxData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcTechBoxData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcTechnology const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcTechnology *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcTechnologyRequirement const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcTechnologyRequirement *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcTechnologyCategory>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTechnologyCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTechnologyCategory *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcTechnologyRarity>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTechnologyRarity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTechnologyRarity *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcTradeData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTradeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTradeData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSubstanceSecondaryLookup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSubstanceSecondaryLookup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSubstanceTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSubstanceTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTechBoxData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTechBoxData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTechBoxTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTechBoxTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTechList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTechList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTechnology const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTechnology *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTechnologyCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTechnologyCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTechnologyRarity const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTechnologyRarity *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTechnologyRequirement const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTechnologyRequirement *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTechnologyTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTechnologyTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTechnologyTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTechnologyTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTradeCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTradeCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTradeData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTradeData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTradeSettings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTradeSettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTradingCategoryData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTradingCategoryData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcRealityCraftingRecipeData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRealityCraftingRecipeData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcRealitySubstanceData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRealitySubstanceData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSubstanceSecondary>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSubstanceSecondary> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcTechBoxData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTechBoxData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcTechnology>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTechnology> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSubstanceSecondaryLookup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSubstanceTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTechBoxData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTechBoxTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTechList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTechnology::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTechnologyCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTechnologyRarity::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTechnologyRequirement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTechnologyTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTechnologyTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTradeCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTradeData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTradeSettings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTradingCategoryData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSubstanceSecondaryLookup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSubstanceTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTechBoxData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTechBoxTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTechList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTechnology::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTechnologyRequirement::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTechnologyTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTechnologyTypes::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTradeData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTradeSettings::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTradingCategoryData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTechnology::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnology *
        ]
    ),
    "cGcTradeData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeData *
        ]
    ),
    "cGcTechnology::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnology *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTechnologyTypes::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyTypes *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTradeData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTradeSettings::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeSettings *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTradingCategoryData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJobDetails *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSubstanceTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTechBoxTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechBoxTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTechnology::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnology *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTechnologyTypes::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyTypes *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTradeData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTradeSettings::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeSettings *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSubstanceSecondaryLookup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceSecondaryLookup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechBoxData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechBoxData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnology::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnology *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnologyCategory::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyCategory *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnologyRarity::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyRarity *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnologyRequirement::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyRequirement *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradeCategory::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeCategory *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradeData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradeSettings::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeSettings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradingCategoryData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingCategoryData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnology::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnology *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcTechnologyCategory::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyCategory *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcTechnologyRarity::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyRarity *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcTechnologyRequirement::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyRequirement *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcTradeCategory::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeCategory *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcTradeData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSubstanceSecondaryLookup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceSecondaryLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechList::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnology::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnology *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnologyCategory::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnologyRarity::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyRarity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnologyRequirement::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyRequirement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradeCategory::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradeData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradeSettings::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradingCategoryData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingCategoryData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnology::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnology *
        ]
    ),
    "cGcTechnologyCategory::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyCategory *
        ]
    ),
    "cGcTechnologyRequirement::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyRequirement *
        ]
    ),
    "cGcTradeData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeData *
        ]
    ),
    "cGcTradeSettings::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeSettings *
        ]
    ),
    "cGcTradingCategoryData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingCategoryData *
        ]
    ),
    "cGcTechnology::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnology *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcTechnologyCategory::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyCategory *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcTechnologyRarity::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyRarity *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcTechnologyRequirement::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyRequirement *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcTradeCategory::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeCategory *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcTradeData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcSubstanceSecondaryLookup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceSecondaryLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSubstanceTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechBoxData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechBoxData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechBoxTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechBoxTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnology::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnology *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnologyCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnologyRarity::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyRarity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnologyRequirement::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyRequirement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnologyTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnologyTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradeCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradeData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradeSettings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradingCategoryData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingCategoryData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechBoxData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechBoxData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTechBoxData *
        ]
    ),
    "cGcTechList::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTechnology::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnology *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTechnology *
        ]
    ),
    "cGcTechnologyCategory::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTechnologyCategory *
        ]
    ),
    "cGcTechnologyRarity::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyRarity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTechnologyRarity *
        ]
    ),
    "cGcTechnologyRequirement::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTechnologyRequirement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTechnologyRequirement *
        ]
    ),
    "cGcTradeCategory::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTradeCategory *
        ]
    ),
    "cGcTradeData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTradeData *
        ]
    ),
    "cGcTradeSettings::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTradeSettings *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSettlementJudgementPerkOption>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementJudgementPerkOption> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSettlementPerkData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementPerkData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSettlementStatChange>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementStatChange> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSettlementStatStrengthRanges>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatStrengthRanges *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcStatsEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStatsEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcStatsGroup>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStatsGroup> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcSettlementJudgementOption>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcSettlementStatType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSettlementJudgementPerkOption const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSettlementJudgementPerkOption *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSettlementPerkData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSettlementPerkData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSettlementStatChange const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSettlementStatChange *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSettlementStatStrengthRanges const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSettlementStatStrengthRanges *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcStatsEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcStatsEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcStatsGroup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcStatsGroup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcSettlementJudgementOption>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementJudgementOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementJudgementOption *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcSettlementJudgementPerkOption>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementJudgementPerkOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementJudgementPerkOption *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcSettlementStatStrength>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementStatStrength *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementStatStrength *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcSettlementStatType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementStatType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementStatType *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementJobDetails const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementJobDetails *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementJudgementData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementJudgementData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementJudgementOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementJudgementOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementJudgementPerkOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementJudgementPerkOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementPerkData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementPerkData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementPerkUsefulData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementPerkUsefulData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementPerksTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementPerksTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementStatChange const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementStatChange *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementStatChangeArray const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementStatChangeArray *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementStatStrength const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementStatStrength *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementStatStrengthData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementStatStrengthData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementStatStrengthRanges const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementStatStrengthRanges *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcShipInventoryMaxUpgradeCapacity const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShipInventoryMaxUpgradeCapacity *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSizeIndicator const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSizeIndicator *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStats const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStats *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatsEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatsEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatsEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatsEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatsGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatsGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStoryUtilityOverride const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStoryUtilityOverride *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSubstanceSecondary const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSubstanceSecondary *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSubstanceSecondaryBiome const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSubstanceSecondaryBiome *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementJudgementOption::~cGcSettlementJudgementOption": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementOption *
        ]
    ),
    "cTkDynamicArray<cGcSettlementJudgementPerkOption>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementJudgementPerkOption> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSettlementPerkData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementPerkData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSettlementStatChange>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementStatChange> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcStatsGroup>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStatsGroup> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSettlementJobDetails::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementJudgementData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementJudgementOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementJudgementPerkOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementPerkData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementPerkUsefulData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementPerksTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementStatChange::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementStatChangeArray::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementStatStrength::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementStatStrengthData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementStatStrengthRanges::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShipInventoryMaxUpgradeCapacity::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSizeIndicator::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStats::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatsEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatsEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatsGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStoryUtilityOverride::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSubstanceSecondary::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSubstanceSecondaryBiome::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementJobDetails::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementJudgementData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementJudgementOption::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementPerkData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementPerkUsefulData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementPerksTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementStatChange::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementStatChangeArray::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStats::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStatsEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStatsGroup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStoryUtilityOverride::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcSettlementPerkData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementPerkData> *
        ]
    ),
    "cGcSettlementJudgementData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementJudgementOption::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementOption *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStats::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStats *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStatsGroup::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsGroup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStoryUtilityOverride::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryUtilityOverride *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementJudgementData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementJudgementOption::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementOption *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementPerksTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementPerksTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementStatChangeArray::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatChangeArray *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStats::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStats *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStatsGroup::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsGroup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStoryUtilityOverride::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryUtilityOverride *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementJobDetails::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJobDetails *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementJudgementData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementJudgementOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementJudgementPerkOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementPerkOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementPerkData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementPerkData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementPerkUsefulData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementPerkUsefulData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementStatChange::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatChange *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementStatStrength::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatStrength *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementStatStrengthRanges::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatStrengthRanges *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipInventoryMaxUpgradeCapacity::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipInventoryMaxUpgradeCapacity *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSizeIndicator::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSizeIndicator *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStats::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStats *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsEnum::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsEnum *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsGroup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsGroup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryUtilityOverride::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryUtilityOverride *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSubstanceSecondary::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceSecondary *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSubstanceSecondaryBiome::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceSecondaryBiome *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementJobDetails::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJobDetails *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementJudgementData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementJudgementOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementPerkData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementPerkData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementPerkUsefulData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementPerkUsefulData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementStatChange::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatChange *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementStatChangeArray::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatChangeArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementStatStrength::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatStrength *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementStatStrengthData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatStrengthData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipInventoryMaxUpgradeCapacity::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipInventoryMaxUpgradeCapacity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSizeIndicator::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSizeIndicator *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStats::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStats *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsEnum::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsGroup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryUtilityOverride::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryUtilityOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSubstanceSecondary::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceSecondary *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSubstanceSecondaryBiome::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceSecondaryBiome *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementJobDetails::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJobDetails *
        ]
    ),
    "cGcSettlementJudgementData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementData *
        ]
    ),
    "cGcSettlementJudgementOption::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementOption *
        ]
    ),
    "cGcSettlementPerkUsefulData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementPerkUsefulData *
        ]
    ),
    "cGcSettlementStatChange::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatChange *
        ]
    ),
    "cGcSettlementStatStrengthData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatStrengthData *
        ]
    ),
    "cGcShipInventoryMaxUpgradeCapacity::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipInventoryMaxUpgradeCapacity *
        ]
    ),
    "cGcStatsEntry::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsEntry *
        ]
    ),
    "cGcStoryUtilityOverride::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryUtilityOverride *
        ]
    ),
    "cGcSubstanceSecondaryBiome::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceSecondaryBiome *
        ]
    ),
    "cGcSettlementJobDetails::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJobDetails *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementJudgementData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementJudgementOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementJudgementPerkOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementPerkOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementPerkData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementPerkData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementPerkUsefulData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementPerkUsefulData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementPerksTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementPerksTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementStatChange::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatChange *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementStatChangeArray::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatChangeArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementStatStrength::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatStrength *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementStatStrengthData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatStrengthData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementStatStrengthRanges::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatStrengthRanges *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipInventoryMaxUpgradeCapacity::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipInventoryMaxUpgradeCapacity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSizeIndicator::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSizeIndicator *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStats::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStats *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStoryUtilityOverride::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryUtilityOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSubstanceSecondary::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceSecondary *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSubstanceSecondaryBiome::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceSecondaryBiome *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementJudgementData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementJudgementData *
        ]
    ),
    "cGcSettlementJudgementOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementJudgementOption *
        ]
    ),
    "cGcSettlementJudgementPerkOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementPerkOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementJudgementPerkOption *
        ]
    ),
    "cGcSettlementPerkData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementPerkData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementPerkData *
        ]
    ),
    "cGcSettlementPerkUsefulData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementPerkUsefulData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementPerkUsefulData *
        ]
    ),
    "cGcSettlementStatChange::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatChange *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementStatChange *
        ]
    ),
    "cGcSettlementStatStrength::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatStrength *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementStatStrength *
        ]
    ),
    "cGcSettlementStatStrengthRanges::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatStrengthRanges *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementStatStrengthRanges *
        ]
    ),
    "cGcSizeIndicator::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSizeIndicator *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSizeIndicator *
        ]
    ),
    "cGcStatsEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatsEntry *
        ]
    ),
    "cGcStatsEnum::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatsEnum *
        ]
    ),
    "cGcStatsGroup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatsGroup *
        ]
    ),
    "cGcStoryUtilityOverride::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStoryUtilityOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStoryUtilityOverride *
        ]
    ),
    "cGcSubstanceSecondary::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceSecondary *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSubstanceSecondary *
        ]
    ),
    "cGcSubstanceSecondaryBiome::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSubstanceSecondaryBiome *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcRewardTableCategory const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcRewardTableCategory *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcRewardTableItemList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcRewardTableItemList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSettlementBuildingCostData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSettlementBuildingCostData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardTableCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardTableCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardTableEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardTableEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardTableItem const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardTableItem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardTeachWord const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardTeachWord *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardTechRecipe const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardTechRecipe *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardTeleport const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardTeleport *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardTraderFlyby const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardTraderFlyby *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardTrigger const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardTrigger *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardTriggerStorm const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardTriggerStorm *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardUnlockSeasonReward const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardUnlockSeasonReward *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardUnlockTitle const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardUnlockTitle *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardUpgradeBase const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardUpgradeBase *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardUpgradeShipClass const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardUpgradeShipClass *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardUpgradeWeaponClass const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardUpgradeWeaponClass *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardWantedLevel const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardWantedLevel *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardWeapon const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardWeapon *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardWeaponSlot const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardWeaponSlot *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardWikiTopic const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardWikiTopic *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardWorker const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardWorker *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementBuildingCost const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementBuildingCost *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementBuildingCostData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementBuildingCostData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementCustomJudgement const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementCustomJudgement *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementGiftDetails const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementGiftDetails *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcRewardTableCategory,3>::cTkFixedArray<cGcRewardTableCategory,3>": {
        "cTkFixedArray<cGcRewardTableCategory,3> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkFixedArray<cGcRewardTableCategory,3> *
            ]
        ),
        "cTkFixedArray<cGcRewardTableCategory,3> *, const cTkFixedArray<cGcRewardTableCategory,3> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkFixedArray<cGcRewardTableCategory,3> *
                ctypes.c_ulonglong,  # const cTkFixedArray<cGcRewardTableCategory,3> *
            ]
        ),
    },
    "cTkFixedArray<cGcSettlementBuildingCostData,6>::cTkFixedArray<cGcSettlementBuildingCostData,6>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcSettlementBuildingCostData,6> *
        ]
    ),
    "cGcRewardTableItemList::cGcRewardTableItemList": {
        "cGcRewardTableItemList *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcRewardTableItemList *
            ]
        ),
        "cGcRewardTableItemList *, const cGcRewardTableItemList *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcRewardTableItemList *
                ctypes.c_ulonglong,  # const cGcRewardTableItemList *
            ]
        ),
    },
    "cTkFixedArray<cGcSettlementBuildingCostData,6>::~cTkFixedArray<cGcSettlementBuildingCostData,6>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcSettlementBuildingCostData,6> *
        ]
    ),
    "cGcRewardTableCategory::~cGcRewardTableCategory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableCategory *
        ]
    ),
    "cGcRewardTableItemList::~cGcRewardTableItemList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableItemList *
        ]
    ),
    "cGcRewardTeachWord::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardTeachWord *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardTechRecipe::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardTechRecipe *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardTeleport::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardTeleport *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardTimeWarp::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardTimeWarp *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardTraderFlyby::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardTraderFlyby *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardTrigger::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardTrigger *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardTriggerSettlementJudgement::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardTriggerSettlementJudgement *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardTriggerStorm::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardTriggerStorm *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardUnlockSeasonReward::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardUnlockSeasonReward *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardUnlockTitle::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardUnlockTitle *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardUpgradeBase::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardUpgradeBase *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardUpgradeShipClass::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardUpgradeShipClass *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardUpgradeWeaponClass::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardUpgradeWeaponClass *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardUploadBase::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardUploadBase *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardWantedLevel::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardWantedLevel *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardWeapon::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardWeapon *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardWeaponSlot::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardWeaponSlot *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardWikiTopic::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardWikiTopic *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardWorker::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardWorker *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardTableCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardTableEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardTableItem::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardTeachWord::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardTechRecipe::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardTeleport::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardTimeWarp::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardTraderFlyby::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardTrigger::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardTriggerSettlementJudgement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardTriggerStorm::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardUnlockSeasonReward::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardUnlockTitle::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardUpgradeBase::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardUpgradeShipClass::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardUpgradeWeaponClass::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardUploadBase::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardWantedLevel::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardWeapon::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardWeaponSlot::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardWikiTopic::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardWorker::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementBuildingCost::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementBuildingCostData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementCustomJudgement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementGiftDetails::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardTableCategory::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardTableEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardTableItem::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardTeachWord::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardTechRecipe::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardUpgradeBase::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardUpgradeShipClass::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardWorker::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementBuildingCost::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementBuildingCostData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementCustomJudgement::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardTableEntry::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableEntry *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardTableItem::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableItem *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardTableEntry::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableEntry *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardTableItem::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableItem *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementBuildingCost::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementBuildingCost *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementCustomJudgement::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementCustomJudgement *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardTableCategory::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableCategory *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTableEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTableItem::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableItem *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardUnlockSeasonReward::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardUnlockSeasonReward *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardUnlockTitle::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardUnlockTitle *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardWantedLevel::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardWantedLevel *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardWeapon::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardWeapon *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardWeaponSlot::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardWeaponSlot *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardWikiTopic::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardWikiTopic *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementBuildingCostData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementBuildingCostData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementCustomJudgement::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementCustomJudgement *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementGiftDetails::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementGiftDetails *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTableCategory::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTableEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTableItem::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementBuildingCost::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementBuildingCost *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementBuildingCostData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementBuildingCostData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementCustomJudgement::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementCustomJudgement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementGiftDetails::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementGiftDetails *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTableEntry::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableEntry *
        ]
    ),
    "cGcRewardTableItem::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableItem *
        ]
    ),
    "cGcRewardWeaponSlot::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardWeaponSlot *
        ]
    ),
    "cGcSettlementBuildingCost::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementBuildingCost *
        ]
    ),
    "cGcSettlementBuildingCostData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementBuildingCostData *
        ]
    ),
    "cGcSettlementCustomJudgement::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementCustomJudgement *
        ]
    ),
    "cGcRewardTableCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTableEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTableItem::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTableItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTeachWord::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTeachWord *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTechRecipe::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTechRecipe *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTeleport::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTeleport *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTraderFlyby::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTraderFlyby *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTrigger::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTrigger *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTriggerStorm::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTriggerStorm *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardUnlockSeasonReward::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardUnlockSeasonReward *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardUnlockTitle::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardUnlockTitle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardUpgradeBase::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardUpgradeBase *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardUpgradeShipClass::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardUpgradeShipClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardUpgradeWeaponClass::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardUpgradeWeaponClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardWantedLevel::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardWantedLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardWeapon::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardWeapon *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardWeaponSlot::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardWeaponSlot *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardWikiTopic::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardWikiTopic *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardWorker::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardWorker *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementBuildingCost::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementBuildingCost *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementBuildingCostData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementBuildingCostData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementCustomJudgement::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementCustomJudgement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementGiftDetails::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementGiftDetails *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTeachWord::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTeachWord *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardTeachWord *
        ]
    ),
    "cGcRewardTechRecipe::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTechRecipe *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardTechRecipe *
        ]
    ),
    "cGcRewardUnlockSeasonReward::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardUnlockSeasonReward *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardUnlockSeasonReward *
        ]
    ),
    "cGcRewardUpgradeShipClass::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardUpgradeShipClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardUpgradeShipClass *
        ]
    ),
    "cGcRewardWeapon::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardWeapon *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardWeapon *
        ]
    ),
    "cGcSettlementBuildingCostData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementBuildingCostData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementBuildingCostData *
        ]
    ),
    "cGcSettlementCustomJudgement::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementCustomJudgement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementCustomJudgement *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcRewardTableEntitlementItem>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRewardTableEntitlementItem> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcRewardTableEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRewardTableEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCurrency>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCurrency *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcInventoryLayout>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcRewardTableEntitlementItem const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcRewardTableEntitlementItem *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcRewardTableEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcRewardTableEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCurrency>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCurrency *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCurrency *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcInventoryLayout>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryLayout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryLayout *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSpecificSeasonalSubstance const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSpecificSeasonalSubstance *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSpecificShip const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSpecificShip *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSpecificSpecial const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSpecificSpecial *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSpecificSubstance const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSpecificSubstance *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSpecificTech const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSpecificTech *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSpecificTechFromList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSpecificTechFromList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSpecificWeapon const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSpecificWeapon *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardStanding const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardStanding *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardStatCompareAndSet const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardStatCompareAndSet *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardStatDiff const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardStatDiff *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSubstance const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSubstance *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSystemSpecificProductFromList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSystemSpecificProductFromList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcRewardTableEntitlementItem>::~cTkDynamicArray<cGcRewardTableEntitlementItem>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRewardTableEntitlementItem> *
        ]
    ),
    "cGcRewardTable::~cGcRewardTable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTable *
        ]
    ),
    "cTkDynamicArray<cGcRewardTableEntitlementItem>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRewardTableEntitlementItem> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcRewardTableEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRewardTableEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcRewardTableItem>::AllocateAndCopyFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRewardTableItem> *
            ctypes.c_ulonglong,  # const cTkDynamicArray<cGcRewardTableItem> *
        ]
    ),
    "cGcRewardSpecificSeasonalSubstance::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSpecificSeasonalSubstance *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificShip::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSpecificShip *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificSpecial::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSpecificSpecial *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificSubstance::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSpecificSubstance *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificTech::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSpecificTech *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificTechFromList::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSpecificTechFromList *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificWeapon::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSpecificWeapon *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardStanding::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardStanding *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardStartPurchase::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardStartPurchase *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardStartSettlementExpedition::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardStartSettlementExpedition *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardStatCompareAndSet::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardStatCompareAndSet *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardStatDiff::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardStatDiff *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSubstance::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSubstance *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSwapMultiTool::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSwapMultiTool *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSystemSpecificProductFromList::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSystemSpecificProductFromList *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificSeasonalSubstance::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificShip::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificSpecial::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificSubstance::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificTech::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificTechFromList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificWeapon::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardStanding::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardStartPurchase::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardStartSettlementExpedition::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardStatCompareAndSet::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardStatDiff::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSubstance::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSwapMultiTool::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSystemSpecificProductFromList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificShip::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardSpecificSubstance::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardSpecificTechFromList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardSpecificWeapon::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardStanding::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardStatDiff::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardSubstance::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardSystemSpecificProductFromList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcRewardTableEntitlementItem>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRewardTableEntitlementItem> *
        ]
    ),
    "cTkDynamicArray<cGcRewardTableEntry>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRewardTableEntry> *
        ]
    ),
    "cGcRewardTable::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardSpecificSeasonalSubstance::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificSeasonalSubstance *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificShip::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificShip *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificSpecial::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificSpecial *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificSubstance::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificSubstance *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificTech::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificTech *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificTechFromList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificTechFromList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificWeapon::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificWeapon *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardStatCompareAndSet::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardStatCompareAndSet *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardStatDiff::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardStatDiff *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSubstance::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSubstance *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSystemSpecificProductFromList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSystemSpecificProductFromList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificSeasonalSubstance::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificSeasonalSubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificShip::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificShip *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificWeapon::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificWeapon *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSubstance::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificShip::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificShip *
        ]
    ),
    "cGcRewardSpecificSeasonalSubstance::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificSeasonalSubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificShip::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificShip *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificSpecial::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificSpecial *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificSubstance::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificSubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificTech::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificTech *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificTechFromList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificTechFromList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificWeapon::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificWeapon *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardStanding::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardStanding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardStatCompareAndSet::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardStatCompareAndSet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardStatDiff::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardStatDiff *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSubstance::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSystemSpecificProductFromList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSystemSpecificProductFromList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificSeasonalSubstance::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificSeasonalSubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardSpecificSeasonalSubstance *
        ]
    ),
    "cGcRewardSpecificShip::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificShip *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardSpecificShip *
        ]
    ),
    "cGcRewardSpecificSpecial::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificSpecial *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardSpecificSpecial *
        ]
    ),
    "cGcRewardSpecificSubstance::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificSubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardSpecificSubstance *
        ]
    ),
    "cGcRewardSpecificWeapon::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificWeapon *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardSpecificWeapon *
        ]
    ),
    "cGcRewardStanding::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardStanding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardStanding *
        ]
    ),
    "cGcRewardStatDiff::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardStatDiff *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardStatDiff *
        ]
    ),
    "cGcRewardSubstance::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardSubstance *
        ]
    ),
    "cGcRewardTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSettlementJudgementType>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementJudgementType> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSettlementJudgementType const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSettlementJudgementType *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSecondarySubstance const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSecondarySubstance *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSetAbandonedFreighterMissionState const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSetAbandonedFreighterMissionState *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSetCurrentMission const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSetCurrentMission *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSetInteractionSeenBitmask const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSetInteractionSeenBitmask *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSetNexusExitWarpTargetToFireteamMemberUA const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSetNexusExitWarpTargetToFireteamMemberUA *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSetWeaponSuppressed const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSetWeaponSuppressed *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSettlementCustomJudgement const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSettlementCustomJudgement *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSettlementJudgement const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSettlementJudgement *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSettlementProgress const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSettlementProgress *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSettlementStat const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSettlementStat *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardShield const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardShield *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardShipAssistance const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardShipAssistance *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardShipMessage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardShipMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardShipSlot const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardShipSlot *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardShowBlackHoles const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardShowBlackHoles *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSignalScan const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSignalScan *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSpecificCommunityTierProduct const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSpecificCommunityTierProduct *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSpecificFrigate const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSpecificFrigate *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSpecificPetEgg const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSpecificPetEgg *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSpecificProduct const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSpecificProduct *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSpecificProductRecipe const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSpecificProductRecipe *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSpecificProductRecipeFromList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSpecificProductRecipeFromList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSecondarySubstance::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSecondarySubstance *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSendChatMessage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSendChatMessage *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSetAbandonedFreighterMissionState::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSetAbandonedFreighterMissionState *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSetCurrentMission::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSetCurrentMission *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSetInteractionSeenBitmask::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSetInteractionSeenBitmask *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSetNexusExitWarpTargetToFireteamMemberUA::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSetNexusExitWarpTargetToFireteamMemberUA *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSetWeaponSuppressed::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSetWeaponSuppressed *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSettlementCustomJudgement::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSettlementCustomJudgement *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSettlementJudgement::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSettlementJudgement *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSettlementProgress::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSettlementProgress *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSettlementStat::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSettlementStat *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardShield::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardShield *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardShipAssistance::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardShipAssistance *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardShipMessage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardShipMessage *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardShipSlot::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardShipSlot *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardShowBlackHoles::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardShowBlackHoles *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSignalScan::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSignalScan *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificCommunityTierProduct::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSpecificCommunityTierProduct *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificFrigate::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSpecificFrigate *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificPetEgg::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSpecificPetEgg *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificProduct::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSpecificProduct *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificProductRecipe::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSpecificProductRecipe *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificProductRecipeFromList::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSpecificProductRecipeFromList *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSecondarySubstance::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSendChatMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSetAbandonedFreighterMissionState::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSetCurrentMission::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSetInteractionSeenBitmask::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSetNexusExitWarpTargetToFireteamMemberUA::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSetWeaponSuppressed::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSettlementCustomJudgement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSettlementJudgement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSettlementProgress::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSettlementStat::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardShield::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardShipAssistance::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardShipMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardShipSlot::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardShowBlackHoles::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSignalScan::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificCommunityTierProduct::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificFrigate::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificPetEgg::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificProduct::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificProductRecipe::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSpecificProductRecipeFromList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSetInteractionSeenBitmask::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardSettlementJudgement::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardSettlementProgress::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardSettlementStat::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardShipMessage::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardSpecificCommunityTierProduct::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardSpecificFrigate::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardSpecificPetEgg::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardSpecificProduct::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardSpecificProductRecipeFromList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardSecondarySubstance::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSecondarySubstance *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSendChatMessage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSendChatMessage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSetCurrentMission::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSetCurrentMission *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSetInteractionSeenBitmask::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSetInteractionSeenBitmask *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSettlementCustomJudgement::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSettlementCustomJudgement *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardShield::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardShield *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardShipSlot::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardShipSlot *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificCommunityTierProduct::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificCommunityTierProduct *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificFrigate::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificFrigate *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificProduct::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificProduct *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificProductRecipe::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificProductRecipe *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificProductRecipeFromList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificProductRecipeFromList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificFrigate::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificFrigate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardShipSlot::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardShipSlot *
        ]
    ),
    "cGcRewardSecondarySubstance::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSecondarySubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSetAbandonedFreighterMissionState::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSetAbandonedFreighterMissionState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSetCurrentMission::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSetCurrentMission *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSetInteractionSeenBitmask::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSetInteractionSeenBitmask *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSetNexusExitWarpTargetToFireteamMemberUA::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSetNexusExitWarpTargetToFireteamMemberUA *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSettlementCustomJudgement::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSettlementCustomJudgement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSettlementJudgement::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSettlementJudgement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSettlementProgress::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSettlementProgress *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSettlementStat::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSettlementStat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardShield::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardShield *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardShipAssistance::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardShipAssistance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardShipMessage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardShipMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardShipSlot::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardShipSlot *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSignalScan::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSignalScan *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificCommunityTierProduct::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificCommunityTierProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificFrigate::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificFrigate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificPetEgg::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificPetEgg *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificProduct::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificProductRecipe::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificProductRecipe *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSpecificProductRecipeFromList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificProductRecipeFromList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSetInteractionSeenBitmask::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSetInteractionSeenBitmask *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardSetInteractionSeenBitmask *
        ]
    ),
    "cGcRewardSettlementProgress::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSettlementProgress *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardSettlementProgress *
        ]
    ),
    "cGcRewardSettlementStat::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSettlementStat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardSettlementStat *
        ]
    ),
    "cGcRewardShield::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardShield *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardShield *
        ]
    ),
    "cGcRewardShipSlot::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardShipSlot *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardShipSlot *
        ]
    ),
    "cGcRewardSpecificCommunityTierProduct::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificCommunityTierProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardSpecificCommunityTierProduct *
        ]
    ),
    "cGcRewardSpecificFrigate::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificFrigate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardSpecificFrigate *
        ]
    ),
    "cGcRewardSpecificPetEgg::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificPetEgg *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardSpecificPetEgg *
        ]
    ),
    "cGcRewardSpecificProduct::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSpecificProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardSpecificProduct *
        ]
    ),
    "cTkMetaDataXML::ReadArrayXML<bool>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # bool *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcAlienPuzzleOption>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAlienPuzzleOption> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcRarity>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRarity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcRealitySubstanceCategory>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAlienPuzzleOption const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAlienPuzzleOption *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcRarity>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRarity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRarity *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcRealitySubstanceCategory>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRealitySubstanceCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRealitySubstanceCategory *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardOverridePulseEncounterChance const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardOverridePulseEncounterChance *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardPetAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardPetAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardPetEggHatch const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardPetEggHatch *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardPirateAttack const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardPirateAttack *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardPirateProbeSignal const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardPoliceScanSignal *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardPlanetSubstance const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardPlanetSubstance *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardProcTechProduct const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardProcTechProduct *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardProceduralProduct const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardProceduralProduct *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardProceduralTechnology const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardProceduralTechnology *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardProduct const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardProduct *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardProductRecipe const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardProductRecipe *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardRechargeTech const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardRechargeTech *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardRefreshHazProt const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardRefreshHazProt *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardRepairTech const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardRepairTech *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardRepairWholeInventory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardRepairWholeInventory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardRequirementsForRecipe const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardRequirementsForRecipe *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardScan const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardScan *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardScanEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardScanEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardScanEventNearestBuilding const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardScanEventNearestBuilding *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardScanEventOutcome const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardScanEventOutcome *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardSecondaryInteractionOptions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardSecondaryInteractionOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardOverridePulseEncounterChance::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardOverridePulseEncounterChance *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardPetAction::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardPetAction *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardPetEgg::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardPetEgg *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardPetEggHatch::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardPetEggHatch *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardPirateAttack::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardPirateAttack *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardPirateProbeSignal::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardPirateProbeSignal *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardPlanetSubstance::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardPlanetSubstance *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardPoliceScanSignal::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardPoliceScanSignal *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardProcTechProduct::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardProcTechProduct *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardProceduralProduct::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardProceduralProduct *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardProceduralProductFromBiome::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardProceduralProductFromBiome *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardProceduralTechnology::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardProceduralTechnology *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardProduct::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardProduct *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardProductRecipe::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardProductRecipe *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardRechargeTech::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardRechargeTech *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardRefreshHazProt::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardRefreshHazProt *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardRepairTech::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardRepairTech *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardRepairWholeInventory::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardRepairWholeInventory *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardRequirementsForRecipe::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardRequirementsForRecipe *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSalvageShip::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSalvageShip *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardScan::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardScan *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardScanEvent::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardScanEvent *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardScanEventNearestBuilding::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardScanEventNearestBuilding *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardSecondaryInteractionOptions::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardSecondaryInteractionOptions *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardOverridePulseEncounterChance::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardPetAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardPetEgg::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardPetEggHatch::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardPirateAttack::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardPirateProbeSignal::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardPlanetSubstance::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardPoliceScanSignal::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardProcTechProduct::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardProceduralProduct::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardProceduralProductFromBiome::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardProceduralTechnology::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardProduct::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardProductRecipe::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardRechargeTech::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardRefreshHazProt::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardRepairTech::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardRepairWholeInventory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardRequirementsForRecipe::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSalvageShip::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardScan::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardScanEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardScanEventNearestBuilding::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardScanEventOutcome::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardSecondaryInteractionOptions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardPetAction::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardProceduralProduct::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardProceduralTechnology::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardProduct::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardProductRecipe::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardRepairTech::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardScan::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardSecondaryInteractionOptions::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardPetAction::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardPetAction *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardPirateAttack::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardPirateAttack *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardPlanetSubstance::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardPlanetSubstance *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardProcTechProduct::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardProcTechProduct *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardProceduralProduct::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardProceduralProduct *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardProduct::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardProduct *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardProductRecipe::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardProductRecipe *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardRefreshHazProt::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardRefreshHazProt *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardRequirementsForRecipe::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardRequirementsForRecipe *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardScanEvent::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardScanEvent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardOverridePulseEncounterChance::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardOverridePulseEncounterChance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardPetAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardPetAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardPetEggHatch::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardPetEggHatch *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardPirateAttack::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardPirateAttack *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardPlanetSubstance::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardPlanetSubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardProcTechProduct::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardProcTechProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardProceduralProduct::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardProceduralProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardProceduralTechnology::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardProceduralTechnology *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardProduct::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardProductRecipe::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardProductRecipe *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardRefreshHazProt::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardRefreshHazProt *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardRepairTech::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardRepairTech *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardRepairWholeInventory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardRepairWholeInventory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardRequirementsForRecipe::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardRequirementsForRecipe *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardScan::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardScan *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardScanEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardScanEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardScanEventNearestBuilding::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardScanEventNearestBuilding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardScanEventOutcome::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardScanEventOutcome *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardSecondaryInteractionOptions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardSecondaryInteractionOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardPetAction::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardPetAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardPetAction *
        ]
    ),
    "cGcRewardPlanetSubstance::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardPlanetSubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardPlanetSubstance *
        ]
    ),
    "cGcRewardProcTechProduct::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardProcTechProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardProcTechProduct *
        ]
    ),
    "cGcRewardProceduralProduct::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardProceduralProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardProceduralProduct *
        ]
    ),
    "cGcRewardProduct::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardProduct *
        ]
    ),
    "cGcRewardScanEvent::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardScanEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardScanEvent *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcMultiSpecificItemEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMultiSpecificItemEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMultiSpecificItemEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMultiSpecificItemEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardIncrementStat const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardIncrementStat *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardInstallTech const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardInstallTech *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardInterventionResponse const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardInterventionResponse *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardInventorySlots const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardInventorySlots *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardJetpackBoost const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardJetpackBoost *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardJourneyThroughCentre const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardJourneyThroughCentre *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardMission const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardMission *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardMissionMessageToMatchingSeeds const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardMissionMessageToMatchingSeeds *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardMissionMessageSeeded const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardMissionMessageSeeded *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardMissionOverride const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardMissionOverride *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardMissionSeeded const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardMissionSeeded *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardModifyStat const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardModifyStat *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardMoney const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardMoney *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardMultiSpecificItems const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardMultiSpecificItems *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardMultiSpecificProductRecipes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardMultiSpecificProductRecipes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardMultiSpecificProducts const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardMultiSpecificProducts *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardMultiSpecificTechRecipes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardMultiSpecificTechRecipes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardNetworkPlayer const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardNetworkPlayer *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardNexus const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardNexus *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardOSDMessage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardOSDMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardOpenFreeFreighter const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardOpenFreeFreighter *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardOpenPage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardOpenPage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardOpenUnlockTree const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardOpenUnlockTree *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcMultiSpecificItemEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMultiSpecificItemEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRewardIncrementStat::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardIncrementStat *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardInstallTech::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardInstallTech *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardInterventionResponse::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardInterventionResponse *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardInventorySlots::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardInventorySlots *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardJetpackBoost::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardJetpackBoost *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardJourneyThroughCentre::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardJourneyThroughCentre *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardMission::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardMission *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardMissionMessage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardMissionMessage *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardMissionMessageSeeded::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardMissionMessageSeeded *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardMissionMessageToMatchingSeeds::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardMissionMessageToMatchingSeeds *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardMissionSeeded::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardMissionSeeded *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardModifyStat::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardModifyStat *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardMoney::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardMoney *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardMultiSpecificItems::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardMultiSpecificItems *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardMultiSpecificProductRecipes::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardMultiSpecificProductRecipes *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardMultiSpecificProducts::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardMultiSpecificProducts *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardMultiSpecificTechRecipes::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardMultiSpecificTechRecipes *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardNetworkPlayer::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardNetworkPlayer *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardNexus::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardNexus *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardOSDMessage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardOSDMessage *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardOpenFreeFreighter::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardOpenFreeFreighter *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardOpenPage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardOpenPage *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardOpenUnlockTree::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardOpenUnlockTree *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardIncrementStat::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardInstallTech::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardInterventionResponse::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardInventorySlots::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardJetpackBoost::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardJourneyThroughCentre::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardMission::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardMissionMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardMissionMessageSeeded::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardMissionMessageToMatchingSeeds::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardMissionOverride::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardMissionSeeded::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardModifyStat::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardMoney::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardMultiSpecificItems::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardMultiSpecificProductRecipes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardMultiSpecificProducts::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardMultiSpecificTechRecipes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardNetworkPlayer::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardNexus::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardOSDMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardOpenFreeFreighter::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardOpenPage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardOpenUnlockTree::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardModifyStat::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardMoney::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardMultiSpecificItems::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardMultiSpecificProductRecipes::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardMultiSpecificProducts::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardMultiSpecificTechRecipes::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardOSDMessage::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardOpenUnlockTree::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcMultiSpecificItemEntry>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMultiSpecificItemEntry> *
        ]
    ),
    "cGcRewardInstallTech::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardInstallTech *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardJetpackBoost::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardJetpackBoost *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMission::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMission *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMissionMessage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMissionMessage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMissionMessageSeeded::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMissionMessageSeeded *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMissionMessageToMatchingSeeds::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMissionMessageToMatchingSeeds *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMissionOverride::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMissionOverride *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMissionSeeded::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMissionSeeded *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardModifyStat::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardModifyStat *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMultiSpecificProductRecipes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMultiSpecificProductRecipes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMultiSpecificProducts::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMultiSpecificProducts *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMultiSpecificTechRecipes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMultiSpecificTechRecipes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardOSDMessage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardOSDMessage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardOpenFreeFreighter::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardOpenFreeFreighter *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMissionOverride::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMissionOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardOSDMessage::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardOSDMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardNetworkPlayer::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardNetworkPlayer *
        ]
    ),
    "cGcRewardIncrementStat::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardIncrementStat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardInstallTech::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardInstallTech *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardInterventionResponse::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardInterventionResponse *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardInventorySlots::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardInventorySlots *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardJetpackBoost::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardJetpackBoost *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardJourneyThroughCentre::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardJourneyThroughCentre *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMission::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMission *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMissionMessageToMatchingSeeds::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMissionMessageToMatchingSeeds *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMissionMessageSeeded::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMissionMessageSeeded *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMissionOverride::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMissionOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMissionSeeded::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMissionSeeded *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardModifyStat::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardModifyStat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMoney::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMoney *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMultiSpecificItems::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMultiSpecificItems *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMultiSpecificProductRecipes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMultiSpecificProductRecipes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMultiSpecificProducts::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMultiSpecificProducts *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardMultiSpecificTechRecipes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMultiSpecificTechRecipes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardNetworkPlayer::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardNetworkPlayer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardOSDMessage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardOSDMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardOpenFreeFreighter::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardOpenFreeFreighter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardOpenPage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardOpenPage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardOpenUnlockTree::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardOpenUnlockTree *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardModifyStat::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardModifyStat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardModifyStat *
        ]
    ),
    "cGcRewardMoney::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardMoney *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardMoney *
        ]
    ),
    "cGcRewardOSDMessage::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardOSDMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardOSDMessage *
        ]
    ),
    "cGcRewardOpenUnlockTree::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardOpenUnlockTree *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardOpenUnlockTree *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardDeath const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardDeath *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardDisableSentinels const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardDisableSentinels *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardDiscoverRune const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardDiscoverRune *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardDisguisedProduct const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardDisguisedProduct *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardDisplayTechWindow const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardDisplayTechWindow *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardDummyLocID const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardDummyLocID *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardEnergy const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardEnergy *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardExchangeProduct const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardExchangeProduct *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardFactionStanding const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardFactionStanding *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardForceDiscoverSystem const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardForceDiscoverSystem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardForgetSpecificProductRecipe const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardForgetSpecificProductRecipe *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardForgetSpecificTechRecipe const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardForgetSpecificTechRecipe *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardFreeStamina const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardFreeStamina *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardFreighterSlot const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardFreighterSlot *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardFrigateDamageResponse const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardFrigateDamageResponse *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardFrigateFlyby const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardFrigateFlyby *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardHazard const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardHazard *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardHealth const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardHealth *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardIncrementInteractionIndex const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardIncrementInteractionIndex *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDeactivateFiends::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardDeactivateFiends *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardDeath::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardDeath *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardDisableSentinels::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardDisableSentinels *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardDiscoverRune::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardDiscoverRune *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardDisguisedProduct::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardDisguisedProduct *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardDisplayTechWindow::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardDisplayTechWindow *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardDummyLocID::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardDummyLocID *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardEnableInteractionClass::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardEnableInteractionClass *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardEnableSentinels::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardEnableSentinels *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardEndFrigateFlyby::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardEndFrigateFlyby *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardEndSettlementExpedition::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardEndSettlementExpedition *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardEnergy::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardEnergy *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardExchangeProduct::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardExchangeProduct *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardFactionStanding::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardFactionStanding *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardForceDiscoverSystem::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardForceDiscoverSystem *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardForgetSpecificProductRecipe::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardForgetSpecificProductRecipe *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardForgetSpecificTechRecipe::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardForgetSpecificTechRecipe *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardFreeStamina::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardFreeStamina *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardFreighterBaseReset::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardFreighterBaseReset *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardFreighterMegaWarp::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardFreighterMegaWarp *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardFreighterSlot::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardFreighterSlot *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardFrigateDamageResponse::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardFrigateDamageResponse *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardFrigateFlyby::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardFrigateFlyby *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardHazard::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardHazard *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardHealth::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardHealth *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardIncrementInteractionIndex::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardIncrementInteractionIndex *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardDeactivateFiends::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardDeath::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardDisableSentinels::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardDiscoverRune::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardDisguisedProduct::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardDisplayTechWindow::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardDummyLocID::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardEnableInteractionClass::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardEnableSentinels::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardEndFrigateFlyby::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardEndSettlementExpedition::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardEnergy::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardExchangeProduct::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardFactionStanding::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardForceDiscoverSystem::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardForgetSpecificProductRecipe::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardForgetSpecificTechRecipe::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardFreeStamina::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardFreighterBaseReset::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardFreighterMegaWarp::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardFreighterSlot::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardFrigateDamageResponse::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardFrigateFlyby::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardHazard::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardHealth::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardIncrementInteractionIndex::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardDeath::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardDisguisedProduct::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardFactionStanding::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardForgetSpecificProductRecipe::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardForgetSpecificTechRecipe::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardFrigateFlyby::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardIncrementInteractionIndex::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardDeath::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDeath *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDisableSentinels::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDisableSentinels *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDisguisedProduct::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDisguisedProduct *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDisplayTechWindow::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDisplayTechWindow *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDummyLocID::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDummyLocID *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardExchangeProduct::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardExchangeProduct *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardFrigateFlyby::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardFrigateFlyby *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDeath::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDeath *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardFrigateFlyby::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardFrigateFlyby *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDeath::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDeath *
        ]
    ),
    "cGcRewardDisableSentinels::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDisableSentinels *
        ]
    ),
    "cGcRewardDeath::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDeath *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDisableSentinels::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDisableSentinels *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDisguisedProduct::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDisguisedProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDisplayTechWindow::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDisplayTechWindow *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDummyLocID::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDummyLocID *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardEnergy::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardEnergy *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardExchangeProduct::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardExchangeProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardFactionStanding::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardFactionStanding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardForgetSpecificProductRecipe::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardForgetSpecificProductRecipe *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardForgetSpecificTechRecipe::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardForgetSpecificTechRecipe *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardFreeStamina::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardFreeStamina *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardFreighterSlot::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardFreighterSlot *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardFrigateDamageResponse::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardFrigateDamageResponse *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardFrigateFlyby::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardFrigateFlyby *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardHazard::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardHazard *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardHealth::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardHealth *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardIncrementInteractionIndex::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardIncrementInteractionIndex *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDeath::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDeath *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardDeath *
        ]
    ),
    "cGcRewardDisguisedProduct::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDisguisedProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardDisguisedProduct *
        ]
    ),
    "cGcRewardExchangeProduct::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardExchangeProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardExchangeProduct *
        ]
    ),
    "cGcRewardFactionStanding::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardFactionStanding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardFactionStanding *
        ]
    ),
    "cGcRewardFrigateFlyby::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardFrigateFlyby *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardFrigateFlyby *
        ]
    ),
    "cGcRewardIncrementInteractionIndex::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardIncrementInteractionIndex *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardIncrementInteractionIndex *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcRefinerRecipe>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRefinerRecipe> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcRefinerRecipeElement>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRefinerRecipeElement> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcRefinerRecipe const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcRefinerRecipe *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcRefinerRecipeElement const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcRefinerRecipeElement *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcRefinerRecipeElement>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRefinerRecipeElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRefinerRecipeElement *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRealitySubstanceData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRealitySubstanceData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRecipeTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRecipeTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRefinerRecipe const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRefinerRecipe *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRefinerRecipeElement const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRefinerRecipeElement *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardActivateEncounterSentinels const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardActivateEncounterSentinels *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardActivateFiends const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardActivateFiends *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardAdvancePortalState const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardAdvancePortalState *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardAssessCookedProduct const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardAssessCookedProduct *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardAtlasPathProgress const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardAtlasPathProgress *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardBeginSettlementBuilding const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardBeginSettlementBuilding *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardChangeGlobalMission const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardChangeGlobalMission *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardCommunicatorMessage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardCommunicatorMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardCommunityContribution const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardCommunityContribution *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardCompleteMission const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardCompleteMission *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardCompleteMultiMission const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardCompleteMultiMission *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardCrashSiteFly const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardCrashSiteFly *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardCustomExpeditionLogEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardCustomExpeditionLogEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardCustomPlayerControl const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardCustomPlayerControl *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardDamage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardDamage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRewardDamageTech const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRewardDamageTech *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcRefinerRecipe>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRefinerRecipe> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRewardAbortTakeoff::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardAbortTakeoff *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardActivateEncounterSentinels::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardActivateEncounterSentinels *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardActivateFiends::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardActivateFiends *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardAdvancePortalState::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardAdvancePortalState *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardAssessCookedProduct::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardAssessCookedProduct *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardAtlasPathProgress::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardAtlasPathProgress *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardBeginSettlementBuilding::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardBeginSettlementBuilding *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardCargo::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardCargo *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardChangeGlobalMission::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardChangeGlobalMission *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardClosePortal::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardClosePortal *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardCommunityContribution::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardCommunityContribution *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardCompleteMission::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardCompleteMission *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardCompleteMultiMission::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardCompleteMultiMission *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardCrashSiteFly::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardCrashSiteFly *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardCrashSiteRepair::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardCrashSiteRepair *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardCustomExpeditionLogEntry::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardCustomExpeditionLogEntry *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardCustomPlayerControl::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardCustomPlayerControl *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardDamage::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardDamage *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRewardDamageTech::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcRewardDamageTech *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcRealitySubstanceData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRecipeTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRefinerRecipe::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRefinerRecipeElement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardAbortTakeoff::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardActivateEncounterSentinels::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardActivateFiends::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardAdvancePortalState::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardAssessCookedProduct::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardAtlasPathProgress::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardBeginSettlementBuilding::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardCargo::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardChangeGlobalMission::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardClosePortal::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardCommunicatorMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardCommunityContribution::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardCompleteMission::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardCompleteMultiMission::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardCrashSiteFly::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardCrashSiteRepair::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardCustomExpeditionLogEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardCustomPlayerControl::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardDamage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRewardDamageTech::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRealitySubstanceData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRecipeTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRefinerRecipe::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRefinerRecipeElement::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardBeginSettlementBuilding::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardCommunicatorMessage::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardCommunityContribution::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardCompleteMultiMission::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRewardDamageTech::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRealitySubstanceData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceData *
        ]
    ),
    "cGcRealitySubstanceData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRefinerRecipe::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerRecipe *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRealitySubstanceData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRecipeTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRecipeTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRefinerRecipe::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerRecipe *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRealitySubstanceData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRefinerRecipe::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerRecipe *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRefinerRecipeElement::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerRecipeElement *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardActivateEncounterSentinels::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardActivateEncounterSentinels *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardAdvancePortalState::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardAdvancePortalState *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardAssessCookedProduct::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardAssessCookedProduct *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardChangeGlobalMission::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardChangeGlobalMission *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardCommunicatorMessage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardCommunicatorMessage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardCommunityContribution::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardCommunityContribution *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardCrashSiteFly::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardCrashSiteFly *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardCustomExpeditionLogEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardCustomExpeditionLogEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDamageTech::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDamageTech *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealitySubstanceData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcRealitySubstanceData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRefinerRecipe::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerRecipe *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRefinerRecipeElement::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerRecipeElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardChangeGlobalMission::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardChangeGlobalMission *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealitySubstanceData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceData *
        ]
    ),
    "cGcRefinerRecipeElement::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerRecipeElement *
        ]
    ),
    "cGcRewardCommunicatorMessage::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardCommunicatorMessage *
        ]
    ),
    "cGcRealitySubstanceData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcRealitySubstanceData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRecipeTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRecipeTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRefinerRecipe::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerRecipe *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRefinerRecipeElement::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerRecipeElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardActivateEncounterSentinels::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardActivateEncounterSentinels *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardAdvancePortalState::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardAdvancePortalState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardAssessCookedProduct::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardAssessCookedProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardAtlasPathProgress::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardAtlasPathProgress *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardBeginSettlementBuilding::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardBeginSettlementBuilding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardChangeGlobalMission::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardChangeGlobalMission *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardCommunicatorMessage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardCommunicatorMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardCommunityContribution::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardCommunityContribution *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardCompleteMultiMission::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardCompleteMultiMission *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardCrashSiteFly::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardCrashSiteFly *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardCustomExpeditionLogEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardCustomExpeditionLogEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardCustomPlayerControl::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardCustomPlayerControl *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDamage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDamage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRewardDamageTech::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDamageTech *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealitySubstanceData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRealitySubstanceData *
        ]
    ),
    "cGcRefinerRecipe::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerRecipe *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRefinerRecipe *
        ]
    ),
    "cGcRefinerRecipeElement::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRefinerRecipeElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRefinerRecipeElement *
        ]
    ),
    "cGcRewardAssessCookedProduct::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardAssessCookedProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardAssessCookedProduct *
        ]
    ),
    "cGcRewardChangeGlobalMission::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardChangeGlobalMission *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardChangeGlobalMission *
        ]
    ),
    "cGcRewardCommunicatorMessage::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardCommunicatorMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardCommunicatorMessage *
        ]
    ),
    "cGcRewardCommunityContribution::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardCommunityContribution *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardCommunityContribution *
        ]
    ),
    "cGcRewardDamageTech::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRewardDamageTech *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRewardDamageTech *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDamageMultiplierLookup>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDamageMultiplierLookup> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcFiendCrimeSpawnTable>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFiendCrimeSpawnTable> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcFreighterCargoOption>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFreighterCargoOption> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNumberedTextList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNumberedTextList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlanetResourceIconLookup>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlanetResourceIconLookup> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcRealityIcon>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRealityIcon> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSubstanceSecondaryLookup>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSubstanceSecondaryLookup> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkIdArray>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIdArray *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkRawID>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkRawID> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAlienPuzzleTableIndex const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAlienPuzzleTableIndex *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDamageMultiplierLookup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDamageMultiplierLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDiscoveryWorth const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDiscoveryWorth *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFiendCrimeSpawnTable const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFiendCrimeSpawnTable *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFreighterCargoOption const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFreighterCargoOption *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNumberedTextList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNumberedTextList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlanetResourceIconLookup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlanetResourceIconLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerWeaponData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerWeaponData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcRealityIcon const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcRealityIcon *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcShipWeaponData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcShipWeaponData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcStats const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcStats *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSubstanceSecondaryLookup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSubstanceSecondaryLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcTechList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcTechList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkCurveType const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkCurveType *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkIdArray const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkIdArray *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkRawID const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkRawID *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcTechList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTechList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTechList *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRealityIcon const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRealityIcon *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRealityIconTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRealityIconTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRealityManagerData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRealityManagerData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRealitySubstanceCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRealitySubstanceCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRealitySubstanceCraftingMix const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRealitySubstanceCraftingMix *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cTkTextureResource,5>::cTkFixedArray<cTkTextureResource,5>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkTextureResource,5> *
        ]
    ),
    "cTkFixedArray<cTkTextureResource,6>::cTkFixedArray<cTkTextureResource,6>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkTextureResource,6> *
        ]
    ),
    "cTkFixedArray<cTkTextureResource,9>::cTkFixedArray<cTkTextureResource,9>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkTextureResource,9> *
        ]
    ),
    "cTkFixedArray<cTkTextureResource,17>::cTkFixedArray<cTkTextureResource,17>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkTextureResource,17> *
        ]
    ),
    "cTkFixedArray<cTkTextureResource,86>::cTkFixedArray<cTkTextureResource,86>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkTextureResource,86> *
        ]
    ),
    "cTkFixedArray<cTkTextureResource,189>::cTkFixedArray<cTkTextureResource,189>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkTextureResource,189> *
        ]
    ),
    "cTkDynamicArray<cGcStatsGroup>::~cTkDynamicArray<cGcStatsGroup>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStatsGroup> *
        ]
    ),
    "cGcPlanetResourceIconLookup::~cGcPlanetResourceIconLookup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetResourceIconLookup *
        ]
    ),
    "cGcRealityIconTable::~cGcRealityIconTable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityIconTable *
        ]
    ),
    "cGcRealityManagerData::~cGcRealityManagerData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManagerData *
        ]
    ),
    "cGcTradeSettings::~cGcTradeSettings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradeSettings *
        ]
    ),
    "cTkDynamicArray<cGcFiendCrimeSpawnTable>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFiendCrimeSpawnTable> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcFreighterCargoOption>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFreighterCargoOption> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPlanetResourceIconLookup>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlanetResourceIconLookup> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcRealityIcon>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRealityIcon> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSubstanceSecondaryLookup>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSubstanceSecondaryLookup> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkRawID>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkRawID> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkTextureResource>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkTextureResource> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRealityIcon::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRealityIconTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRealityManagerData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRealitySubstanceCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRealitySubstanceCraftingMix::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRealityIcon::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRealityIconTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRealityManagerData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cTkTextureResource>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkTextureResource> *
        ]
    ),
    "cGcRealityIconTable::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityIconTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRealityManagerData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManagerData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRealityIconTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityIconTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRealityManagerData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManagerData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRealityIcon::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityIcon *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityIconTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityIconTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityManagerData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManagerData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealitySubstanceCategory::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceCategory *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealitySubstanceCraftingMix::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceCraftingMix *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealitySubstanceCategory::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceCategory *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcRealityIconTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityIconTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityManagerData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManagerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealitySubstanceCategory::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealitySubstanceCraftingMix::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceCraftingMix *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityIconTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityIconTable *
        ]
    ),
    "cGcRealityManagerData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManagerData *
        ]
    ),
    "cGcRealitySubstanceCategory::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceCategory *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcRealityIcon::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityIcon *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityIconTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityIconTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityManagerData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManagerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealitySubstanceCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealitySubstanceCraftingMix::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceCraftingMix *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityIcon::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityIcon *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRealityIcon *
        ]
    ),
    "cGcRealityIconTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityIconTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRealityIconTable *
        ]
    ),
    "cGcRealityManagerData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityManagerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRealityManagerData *
        ]
    ),
    "cGcRealitySubstanceCategory::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealitySubstanceCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRealitySubstanceCategory *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBuildingBlueprint>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBuildingBlueprint> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcProceduralTechnologyData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcProceduralTechnologyData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcProceduralTechnologyStatLevel>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcProceduralTechnologyStatLevel> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcProductData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcProductData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPurchaseableSpecial>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPurchaseableSpecial> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcNameGeneratorWord>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGeneratorWord *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcWeightingCurve>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWeightingCurve *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBuildingBlueprint const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBuildingBlueprint *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcProceduralTechnologyData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcProceduralTechnologyData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcProceduralTechnologyStatLevel const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcProceduralTechnologyStatLevel *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcProductData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcProductData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPurchaseableSpecial const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPurchaseableSpecial *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcRealitySubstanceCraftingMix const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcRealitySubstanceCraftingMix *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcNameGeneratorWord>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNameGeneratorWord *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNameGeneratorWord *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcProceduralTechnologyCategory>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProceduralTechnologyCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProceduralTechnologyCategory *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcProductCategory>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProductCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProductCategory *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcWeightingCurve>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWeightingCurve *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWeightingCurve *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProceduralProductWord const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProceduralProductWord *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProceduralTechnologyCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProceduralTechnologyCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProceduralTechnologyData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProceduralTechnologyData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProceduralTechnologyStatLevel const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProceduralTechnologyStatLevel *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProceduralTechnologyTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProceduralTechnologyTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProductCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProductCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProductData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProductData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProductProceduralOnlyData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProductProceduralOnlyData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProductTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProductTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPurchaseableBuildingBlueprints const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPurchaseableBuildingBlueprints *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPurchaseableSpecial const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPurchaseableSpecial *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPurchaseableSpecials const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPurchaseableSpecials *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPuzzleTextFlow const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPuzzleTextFlow *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRarity const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRarity *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRealityCommonFactions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRealityCommonFactions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRealityCraftingRecipeData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRealityCraftingRecipeData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRealityGameIcons const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRealityGameIcons *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcProceduralTechnologyData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcProceduralTechnologyData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcProductData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcProductData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPurchaseableSpecial>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPurchaseableSpecial> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcProceduralProductWord::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProceduralTechnologyCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProceduralTechnologyData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProceduralTechnologyStatLevel::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProceduralTechnologyTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProductCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProductData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProductProceduralOnlyData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProductTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPurchaseableBuildingBlueprints::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPurchaseableSpecial::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPurchaseableSpecials::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPuzzleTextFlow::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRarity::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRealityCommonFactions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRealityCraftingRecipeData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRealityGameIcons::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProceduralProductWord::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProceduralTechnologyData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProceduralTechnologyStatLevel::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProceduralTechnologyTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProductData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProductProceduralOnlyData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProductTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPurchaseableBuildingBlueprints::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPurchaseableSpecials::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPuzzleTextFlow::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRealityCraftingRecipeData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProductData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductData *
        ]
    ),
    "cGcProceduralProductWord::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductWord *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProceduralTechnologyData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTechnologyData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProductData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProductProceduralOnlyData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductProceduralOnlyData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProductTable::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPuzzleTextFlow::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPuzzleTextFlow *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRealityCraftingRecipeData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityCraftingRecipeData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProceduralTechnologyData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTechnologyData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProceduralTechnologyTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTechnologyTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProductData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProductTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPurchaseableBuildingBlueprints::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableBuildingBlueprints *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProceduralProductWord::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductWord *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralTechnologyCategory::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTechnologyCategory *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralTechnologyData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTechnologyData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralTechnologyStatLevel::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTechnologyStatLevel *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProductCategory::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductCategory *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProductData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProductProceduralOnlyData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductProceduralOnlyData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPurchaseableSpecial::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableSpecial *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPuzzleTextFlow::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPuzzleTextFlow *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRarity::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRarity *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityCommonFactions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityCommonFactions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityCraftingRecipeData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityCraftingRecipeData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityGameIcons::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityGameIcons *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProductData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcRarity::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRarity *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcProceduralProductWord::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductWord *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralTechnologyCategory::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTechnologyCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralTechnologyData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTechnologyData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralTechnologyStatLevel::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTechnologyStatLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProductCategory::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProductData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProductProceduralOnlyData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductProceduralOnlyData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPurchaseableSpecial::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableSpecial *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPuzzleTextFlow::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPuzzleTextFlow *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRarity::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRarity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityCommonFactions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityCommonFactions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityCraftingRecipeData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityCraftingRecipeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityGameIcons::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityGameIcons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralProductWord::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductWord *
        ]
    ),
    "cGcProceduralTechnologyData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTechnologyData *
        ]
    ),
    "cGcProductData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductData *
        ]
    ),
    "cGcProductProceduralOnlyData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductProceduralOnlyData *
        ]
    ),
    "cGcPuzzleTextFlow::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPuzzleTextFlow *
        ]
    ),
    "cGcRealityCraftingRecipeData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityCraftingRecipeData *
        ]
    ),
    "cGcProductData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcRarity::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRarity *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcProceduralProductWord::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductWord *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralTechnologyCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTechnologyCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralTechnologyData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTechnologyData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralTechnologyStatLevel::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTechnologyStatLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralTechnologyTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTechnologyTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProductCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProductData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProductProceduralOnlyData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductProceduralOnlyData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProductTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPurchaseableBuildingBlueprints::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableBuildingBlueprints *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPurchaseableSpecial::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableSpecial *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPurchaseableSpecials::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableSpecials *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPuzzleTextFlow::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPuzzleTextFlow *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRarity::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRarity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityCommonFactions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityCommonFactions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityCraftingRecipeData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityCraftingRecipeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRealityGameIcons::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityGameIcons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralTechnologyCategory::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTechnologyCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProceduralTechnologyCategory *
        ]
    ),
    "cGcProceduralTechnologyData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTechnologyData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProceduralTechnologyData *
        ]
    ),
    "cGcProceduralTechnologyStatLevel::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralTechnologyStatLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProceduralTechnologyStatLevel *
        ]
    ),
    "cGcProductCategory::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProductCategory *
        ]
    ),
    "cGcProductData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProductData *
        ]
    ),
    "cGcProductProceduralOnlyData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProductProceduralOnlyData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProductProceduralOnlyData *
        ]
    ),
    "cGcPurchaseableSpecial::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPurchaseableSpecial *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPurchaseableSpecial *
        ]
    ),
    "cGcPuzzleTextFlow::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPuzzleTextFlow *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPuzzleTextFlow *
        ]
    ),
    "cGcRarity::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRarity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRarity *
        ]
    ),
    "cGcRealityCommonFactions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityCommonFactions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRealityCommonFactions *
        ]
    ),
    "cGcRealityGameIcons::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRealityGameIcons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRealityGameIcons *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBreakTechByStatData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBreakTechByStatData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcLegacyItem>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcLegacyItem> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcMaintenanceGroupEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMaintenanceGroupEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerDamageData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerDamageData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerTitle>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerTitle> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcProceduralProductWord>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcProceduralProductWord> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcInventoryType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBreakTechByStatData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBreakTechByStatData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcLegacyItem const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcLegacyItem *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMaintenanceGroup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMaintenanceGroup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMaintenanceGroupEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMaintenanceGroupEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerDamageData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerDamageData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerTitle const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerTitle *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcProceduralProductData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcProceduralProductData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcProceduralProductWord const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcProceduralProductWord *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcProductProceduralOnlyData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcProductProceduralOnlyData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcInventoryType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryType *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcMaintenanceElementGroups>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMaintenanceElementGroups *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMaintenanceElementGroups *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcMaintenanceGroupEntry>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMaintenanceGroupEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMaintenanceGroupEntry *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcProceduralProductCategory>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProceduralProductCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProceduralProductCategory *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcLegacyItemTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcLegacyItemTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcLegality const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcLegality *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMaintenanceElement const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMaintenanceElement *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMaintenanceElementGroups const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMaintenanceElementGroups *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMaintenanceGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMaintenanceGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMaintenanceGroupEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMaintenanceGroupEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMaintenanceGroupsTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMaintenanceGroupsTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMultiSpecificItemEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMultiSpecificItemEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNameGeneratorSectorTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNameGeneratorSectorTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNameGeneratorTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNameGeneratorTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNameGeneratorWord const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNameGeneratorWord *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNumberedTextList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNumberedTextList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlanetResourceIconLookup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetResourceIconLookup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerDamageData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerDamageData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerDamageTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerDamageTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerTitleData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerTitleData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProceduralProductCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProceduralProductCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProceduralProductData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProceduralProductData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProceduralProductDeployable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProceduralProductDeployable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProceduralProductTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProceduralProductTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcProceduralProductData,26>::cTkFixedArray<cGcProceduralProductData,26>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcProceduralProductData,26> *
        ]
    ),
    "cTkFixedArray<cGcProductProceduralOnlyData,3>::cTkFixedArray<cGcProductProceduralOnlyData,3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcProductProceduralOnlyData,3> *
        ]
    ),
    "cGcPlayerDamageData::~cGcPlayerDamageData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDamageData *
        ]
    ),
    "cTkDynamicArray<cGcBreakTechByStatData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBreakTechByStatData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcLegacyItem>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcLegacyItem> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcMaintenanceGroupEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMaintenanceGroupEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPlayerDamageData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerDamageData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPlayerTitle>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerTitle> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcLegacyItemTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLegality::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMaintenanceElement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMaintenanceElementGroups::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMaintenanceGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMaintenanceGroupEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMaintenanceGroupsTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMultiSpecificItemEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNameGeneratorSectorTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNameGeneratorTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNameGeneratorWord::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNumberedTextList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlanetResourceIconLookup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerDamageData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerDamageTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerTitleData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProceduralProductCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProceduralProductData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProceduralProductDeployable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProceduralProductTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLegacyItemTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMaintenanceElement::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMaintenanceGroup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMaintenanceGroupEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMaintenanceGroupsTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMultiSpecificItemEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetResourceIconLookup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerDamageData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerDamageTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerTitleData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProceduralProductData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProceduralProductTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMultiSpecificItemEntry::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultiSpecificItemEntry *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlanetResourceIconLookup::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetResourceIconLookup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerDamageData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDamageData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProceduralProductData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMaintenanceGroupsTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceGroupsTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMultiSpecificItemEntry::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultiSpecificItemEntry *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerDamageData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDamageData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerDamageTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDamageTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerTitleData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerTitleData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProceduralProductData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcProceduralProductTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcLegality::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLegality *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceElement::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceElement *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceElementGroups::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceElementGroups *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceGroup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceGroup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceGroupEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceGroupEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMultiSpecificItemEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultiSpecificItemEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNameGeneratorWord::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGeneratorWord *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNumberedTextList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNumberedTextList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetResourceIconLookup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetResourceIconLookup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerDamageData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDamageData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralProductCategory::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductCategory *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralProductData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLegality::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLegality *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcLegality::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLegality *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceElement::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceElementGroups::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceElementGroups *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMultiSpecificItemEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultiSpecificItemEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNameGeneratorWord::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGeneratorWord *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNumberedTextList::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNumberedTextList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetResourceIconLookup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetResourceIconLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerDamageData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDamageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralProductCategory::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralProductData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerDamageTable::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDamageTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralProductTable::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceElement::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceElement *
        ]
    ),
    "cGcMultiSpecificItemEntry::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultiSpecificItemEntry *
        ]
    ),
    "cGcNameGeneratorWord::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGeneratorWord *
        ]
    ),
    "cGcNumberedTextList::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNumberedTextList *
        ]
    ),
    "cGcPlanetResourceIconLookup::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetResourceIconLookup *
        ]
    ),
    "cGcPlayerDamageData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDamageData *
        ]
    ),
    "cGcProceduralProductData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductData *
        ]
    ),
    "cGcProceduralProductTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductTable *
        ]
    ),
    "cGcLegality::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLegality *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcLegacyItemTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLegacyItemTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLegality::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLegality *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceElement::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceElementGroups::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceElementGroups *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceGroupEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceGroupEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceGroupsTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceGroupsTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMultiSpecificItemEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultiSpecificItemEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNameGeneratorSectorTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGeneratorSectorTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNameGeneratorTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGeneratorTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNameGeneratorWord::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGeneratorWord *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNumberedTextList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNumberedTextList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlanetResourceIconLookup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetResourceIconLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerDamageData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDamageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerDamageTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDamageTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerTitleData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerTitleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralProductCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralProductData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralProductDeployable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductDeployable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProceduralProductTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLegality::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLegality *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcLegality *
        ]
    ),
    "cGcMaintenanceElement::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMaintenanceElement *
        ]
    ),
    "cGcMaintenanceElementGroups::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceElementGroups *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMaintenanceElementGroups *
        ]
    ),
    "cGcMaintenanceGroupEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceGroupEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMaintenanceGroupEntry *
        ]
    ),
    "cGcMultiSpecificItemEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultiSpecificItemEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMultiSpecificItemEntry *
        ]
    ),
    "cGcNameGeneratorWord::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNameGeneratorWord *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNameGeneratorWord *
        ]
    ),
    "cGcNumberedTextList::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNumberedTextList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNumberedTextList *
        ]
    ),
    "cGcPlayerDamageData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerDamageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerDamageData *
        ]
    ),
    "cGcProceduralProductCategory::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProceduralProductCategory *
        ]
    ),
    "cGcProceduralProductData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProceduralProductData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProceduralProductData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcInventoryBaseStat>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInventoryBaseStat> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcInventoryCostDataEntry>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryCostDataEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcInventoryGenerationBaseStatData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryGenerationBaseStatData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcInventoryGenerationBaseStatDataEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInventoryGenerationBaseStatDataEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcInventoryTableEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInventoryTableEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcInventoryLayoutGenerationBounds>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayoutGenerationBounds *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInventoryBaseStat const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInventoryBaseStat *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInventoryClassProbabilities const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInventoryClassProbabilities *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInventoryCostDataEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInventoryCostDataEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInventoryGenerationBaseStatClassData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInventoryGenerationBaseStatClassData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInventoryGenerationBaseStatData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInventoryGenerationBaseStatData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInventoryGenerationBaseStatDataEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInventoryGenerationBaseStatDataEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInventoryLayoutGenerationDataEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInventoryLayoutGenerationDataEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInventoryTableEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInventoryTableEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcShipInventoryMaxUpgradeCapacity const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcShipInventoryMaxUpgradeCapacity *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcInventoryCostDataEntry>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryCostDataEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryCostDataEntry *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcInventoryGenerationBaseStatData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryGenerationBaseStatData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryGenerationBaseStatData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcInventoryLayoutGenerationBounds>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryLayoutGenerationBounds *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryLayoutGenerationBounds *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcInventoryLayoutGenerationData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryLayoutGenerationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryLayoutGenerationData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcInventoryLayoutSizeType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryLayoutSizeType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryLayoutSizeType *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryClass const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryClass *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryClassProbabilities const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryClassProbabilities *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryCostData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryCostData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryCostDataEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryCostDataEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryGenerationBaseStatClassData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryGenerationBaseStatClassData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryGenerationBaseStatData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryGenerationBaseStatData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryGenerationBaseStatDataEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryGenerationBaseStatDataEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryLayoutGenerationBounds const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryLayoutGenerationBounds *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryLayoutGenerationData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryLayoutGenerationData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryLayoutGenerationDataEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryLayoutGenerationDataEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryLayoutSizeType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryLayoutSizeType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventorySpecialSlotType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventorySpecialSlotType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryStackSizeGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryStackSizeGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryTableEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryTableEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcItemAmountCostPair const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcItemAmountCostPair *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcItemCostData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcItemCostData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcItemNeedPurpose const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcItemNeedPurpose *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcItemPriceModifiers const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcItemPriceModifiers *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcLegacyItem const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcLegacyItem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcInventoryLayoutGenerationDataEntry,37>::cTkFixedArray<cGcInventoryLayoutGenerationDataEntry,37>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcInventoryLayoutGenerationDataEntry,37> *
        ]
    ),
    "cTkDynamicArray<cGcInventoryBaseStat>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInventoryBaseStat> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcInventoryGenerationBaseStatDataEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInventoryGenerationBaseStatDataEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcInventoryTableEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInventoryTableEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcInventoryClass::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryClassProbabilities::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryCostData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryCostDataEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryGenerationBaseStatClassData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryGenerationBaseStatData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryGenerationBaseStatDataEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryLayoutGenerationBounds::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryLayoutGenerationData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryLayoutGenerationDataEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryLayoutSizeType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventorySpecialSlotType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryStackSizeGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryTableEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcItemAmountCostPair::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcItemCostData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcItemNeedPurpose::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcItemPriceModifiers::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLegacyItem::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryGenerationBaseStatClassData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventoryGenerationBaseStatData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventoryLayoutGenerationData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventoryLayoutGenerationDataEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventoryTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventoryTable::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventoryTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventoryClass::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryClass *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryClassProbabilities::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryClassProbabilities *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryCostData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryCostData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryCostDataEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryCostDataEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryGenerationBaseStatClassData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryGenerationBaseStatClassData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryGenerationBaseStatData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryGenerationBaseStatData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryGenerationBaseStatDataEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryGenerationBaseStatDataEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryLayoutGenerationBounds::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayoutGenerationBounds *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryLayoutGenerationData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayoutGenerationData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryLayoutGenerationDataEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayoutGenerationDataEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryLayoutSizeType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayoutSizeType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventorySpecialSlotType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventorySpecialSlotType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryStackSizeGroup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStackSizeGroup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryTableEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryTableEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemAmountCostPair::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemAmountCostPair *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemCostData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemCostData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemNeedPurpose::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemNeedPurpose *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemPriceModifiers::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemPriceModifiers *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLegacyItem::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLegacyItem *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryClass::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryClass *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcInventorySpecialSlotType::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventorySpecialSlotType *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcInventoryStackSizeGroup::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStackSizeGroup *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcInventoryType::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryType *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcItemPriceModifiers::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemPriceModifiers *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcInventoryClass::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryCostDataEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryCostDataEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryGenerationBaseStatData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryGenerationBaseStatData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryGenerationBaseStatDataEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryGenerationBaseStatDataEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryLayoutGenerationBounds::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayoutGenerationBounds *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryLayoutGenerationData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayoutGenerationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryLayoutGenerationDataEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayoutGenerationDataEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryLayoutSizeType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayoutSizeType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventorySpecialSlotType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventorySpecialSlotType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryStackSizeGroup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStackSizeGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryTableEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryTableEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemAmountCostPair::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemAmountCostPair *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemCostData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemCostData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemNeedPurpose::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemNeedPurpose *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemPriceModifiers::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemPriceModifiers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLegacyItem::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLegacyItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryTable::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryLayoutGenerationData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayoutGenerationData *
        ]
    ),
    "cGcInventoryLayoutGenerationDataEntry::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayoutGenerationDataEntry *
        ]
    ),
    "cGcInventoryTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryTable *
        ]
    ),
    "cGcItemCostData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemCostData *
        ]
    ),
    "cGcLegacyItem::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLegacyItem *
        ]
    ),
    "cGcInventoryClass::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryClass *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcInventorySpecialSlotType::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventorySpecialSlotType *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcInventoryStackSizeGroup::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStackSizeGroup *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcInventoryType::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryType *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcItemPriceModifiers::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemPriceModifiers *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcInventoryClass::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryClassProbabilities::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryClassProbabilities *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryCostData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryCostData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryCostDataEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryCostDataEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryGenerationBaseStatClassData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryGenerationBaseStatClassData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryGenerationBaseStatData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryGenerationBaseStatData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryGenerationBaseStatDataEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryGenerationBaseStatDataEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryLayoutGenerationBounds::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayoutGenerationBounds *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryLayoutGenerationData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayoutGenerationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryLayoutGenerationDataEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayoutGenerationDataEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryLayoutSizeType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayoutSizeType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventorySpecialSlotType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventorySpecialSlotType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryStackSizeGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStackSizeGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryTableEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryTableEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemAmountCostPair::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemAmountCostPair *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemCostData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemCostData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemNeedPurpose::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemNeedPurpose *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemPriceModifiers::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemPriceModifiers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLegacyItem::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLegacyItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryClass::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryClass *
        ]
    ),
    "cGcInventoryCostDataEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryCostDataEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryCostDataEntry *
        ]
    ),
    "cGcInventoryGenerationBaseStatDataEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryGenerationBaseStatDataEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryGenerationBaseStatDataEntry *
        ]
    ),
    "cGcInventoryLayoutGenerationBounds::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayoutGenerationBounds *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryLayoutGenerationBounds *
        ]
    ),
    "cGcInventoryLayoutGenerationDataEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayoutGenerationDataEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryLayoutGenerationDataEntry *
        ]
    ),
    "cGcInventoryLayoutSizeType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayoutSizeType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryLayoutSizeType *
        ]
    ),
    "cGcInventorySpecialSlotType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventorySpecialSlotType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventorySpecialSlotType *
        ]
    ),
    "cGcInventoryStackSizeGroup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStackSizeGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryStackSizeGroup *
        ]
    ),
    "cGcInventoryTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryTable *
        ]
    ),
    "cGcInventoryTableEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryTableEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryTableEntry *
        ]
    ),
    "cGcInventoryType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryType *
        ]
    ),
    "cGcItemCostData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemCostData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcItemCostData *
        ]
    ),
    "cGcItemNeedPurpose::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemNeedPurpose *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcItemNeedPurpose *
        ]
    ),
    "cGcItemPriceModifiers::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemPriceModifiers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcItemPriceModifiers *
        ]
    ),
    "cGcLegacyItem::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLegacyItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcLegacyItem *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDeathQuote>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDeathQuote> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDialogClearanceInfo>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDialogClearanceInfo> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDiscoveryRewardLookup>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDiscoveryRewardLookup> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcInventoryBaseStatBonus>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInventoryBaseStatBonus> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcTextPreset>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextPreset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDeathQuote const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDeathQuote *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDialogClearanceInfo const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDialogClearanceInfo *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDiscoveryRewardLookup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDiscoveryRewardLookup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInventoryBaseStatBonus const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInventoryBaseStatBonus *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcFrigateStatType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateStatType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFrigateStatType *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcTextPreset>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTextPreset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTextPreset *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostTableEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostTableEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostWordKnowledge const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostWordKnowledge *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureRarity const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCreatureRarity *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCurrency const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCurrency *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDeathQuote const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDeathQuote *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDeathStateData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDeathStateData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDialogClearanceInfo const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDialogClearanceInfo *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDialogClearanceTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDialogClearanceTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDiscoveryOwner const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryOwner *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDiscoveryRewardLookup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryRewardLookup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDiscoveryRewardLookupTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryRewardLookupTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDiscoveryType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDiscoveryWorth const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryWorth *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExpeditionCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExpeditionCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExpeditionDuration const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExpeditionDuration *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExpeditionPowerup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExpeditionPowerup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExpeditionRewardTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExpeditionRewardTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFreighterCargoOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFreighterCargoOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFrigateClass const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateClass *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFrigateStatType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateStatType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFrigateTraitStrength const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateTraitStrength *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGenericRewardTableEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericRewardTableEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryBaseStat const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryBaseStat *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryBaseStatBonus const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryBaseStatBonus *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryBaseStatEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryBaseStatEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcDeathQuote>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDeathQuote> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcDialogClearanceInfo>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDialogClearanceInfo> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcDiscoveryRewardLookup>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDiscoveryRewardLookup> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcInventoryBaseStatBonus>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInventoryBaseStatBonus> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCostWordKnowledge::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostWordKnowledge *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostTableEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostWordKnowledge::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCreatureRarity::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCurrency::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDeathQuote::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDeathStateData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDialogClearanceInfo::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDialogClearanceTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDiscoveryOwner::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDiscoveryRewardLookup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDiscoveryRewardLookupTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDiscoveryType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDiscoveryWorth::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExpeditionCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExpeditionDuration::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExpeditionPowerup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExpeditionRewardTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFreighterCargoOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateClass::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateStatType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFrigateTraitStrength::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGenericRewardTableEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryBaseStat::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryBaseStatBonus::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryBaseStatEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostTableEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostWordKnowledge::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDeathStateData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDialogClearanceTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDiscoveryRewardLookupTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExpeditionRewardTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGenericRewardTableEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventoryBaseStat::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventoryBaseStatBonus::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostTableEntry::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostTableEntry *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGenericRewardTableEntry::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericRewardTableEntry *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventoryBaseStat::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryBaseStat *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostTableEntry::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostTableEntry *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExpeditionRewardTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionRewardTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGenericRewardTableEntry::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericRewardTableEntry *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventoryBaseStat::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryBaseStat *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostTableEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostTableEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRarity::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRarity *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCurrency::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCurrency *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDeathQuote::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDeathQuote *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDialogClearanceInfo::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDialogClearanceInfo *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryOwner::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryOwner *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryRewardLookup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryRewardLookup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryWorth::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryWorth *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionCategory::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionCategory *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionDuration::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionDuration *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionPowerup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionPowerup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterCargoOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterCargoOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateClass::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateClass *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateStatType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateStatType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateTraitStrength::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitStrength *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericRewardTableEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericRewardTableEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryBaseStat::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryBaseStat *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryBaseStatBonus::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryBaseStatBonus *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryBaseStatEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryBaseStatEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryOwner::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryOwner *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcExpeditionCategory::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionCategory *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcExpeditionDuration::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionDuration *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcFrigateClass::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateClass *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcInventoryBaseStatEntry::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryBaseStatEntry *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcCostTableEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostTableEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRarity::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRarity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCurrency::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCurrency *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryOwner::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryOwner *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryRewardLookup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryRewardLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryWorth::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryWorth *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionCategory::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionDuration::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionDuration *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionPowerup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionPowerup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterCargoOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterCargoOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateClass::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateStatType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateStatType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateTraitStrength::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitStrength *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericRewardTableEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericRewardTableEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryBaseStat::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryBaseStat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryBaseStatBonus::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryBaseStatBonus *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryBaseStatEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryBaseStatEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostTableEntry::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostTableEntry *
        ]
    ),
    "cGcDeathQuote::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDeathQuote *
        ]
    ),
    "cGcDiscoveryOwner::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryOwner *
        ]
    ),
    "cGcDiscoveryWorth::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryWorth *
        ]
    ),
    "cGcExpeditionPowerup::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionPowerup *
        ]
    ),
    "cGcFreighterCargoOption::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterCargoOption *
        ]
    ),
    "cGcGenericRewardTableEntry::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericRewardTableEntry *
        ]
    ),
    "cGcDiscoveryOwner::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryOwner *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcExpeditionCategory::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionCategory *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcExpeditionDuration::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionDuration *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcFrigateClass::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateClass *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcInventoryBaseStatEntry::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryBaseStatEntry *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcCostTableEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostTableEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostWordKnowledge::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostWordKnowledge *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureRarity::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRarity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCurrency::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCurrency *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDeathQuote::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDeathQuote *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDeathStateData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDeathStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDialogClearanceInfo::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDialogClearanceInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDialogClearanceTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDialogClearanceTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryOwner::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryOwner *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryRewardLookup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryRewardLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryRewardLookupTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryRewardLookupTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryWorth::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryWorth *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionDuration::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionDuration *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionPowerup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionPowerup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionRewardTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionRewardTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterCargoOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterCargoOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateClass::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateStatType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateStatType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFrigateTraitStrength::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitStrength *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericRewardTableEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericRewardTableEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryBaseStat::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryBaseStat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryBaseStatBonus::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryBaseStatBonus *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryBaseStatEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryBaseStatEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostTableEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostTableEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCostTableEntry *
        ]
    ),
    "cGcCostWordKnowledge::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostWordKnowledge *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCostWordKnowledge *
        ]
    ),
    "cGcCreatureRarity::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureRarity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureRarity *
        ]
    ),
    "cGcCurrency::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCurrency *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCurrency *
        ]
    ),
    "cGcDeathQuote::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDeathQuote *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDeathQuote *
        ]
    ),
    "cGcDialogClearanceInfo::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDialogClearanceInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDialogClearanceInfo *
        ]
    ),
    "cGcDiscoveryOwner::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryOwner *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDiscoveryOwner *
        ]
    ),
    "cGcDiscoveryRewardLookup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryRewardLookup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDiscoveryRewardLookup *
        ]
    ),
    "cGcDiscoveryType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDiscoveryType *
        ]
    ),
    "cGcExpeditionCategory::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcExpeditionCategory *
        ]
    ),
    "cGcExpeditionDuration::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionDuration *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcExpeditionDuration *
        ]
    ),
    "cGcExpeditionPowerup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionPowerup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcExpeditionPowerup *
        ]
    ),
    "cGcFreighterCargoOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterCargoOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFreighterCargoOption *
        ]
    ),
    "cGcFrigateClass::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateClass *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFrigateClass *
        ]
    ),
    "cGcFrigateStatType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateStatType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFrigateStatType *
        ]
    ),
    "cGcFrigateTraitStrength::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitStrength *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFrigateTraitStrength *
        ]
    ),
    "cGcGenericRewardTableEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericRewardTableEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGenericRewardTableEntry *
        ]
    ),
    "cGcInventoryBaseStatBonus::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryBaseStatBonus *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryBaseStatBonus *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcItemAmountCostPair>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcItemAmountCostPair> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcDefaultMissionProductEnum>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDefaultMissionProductEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcItemAmountCostPair const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcItemAmountCostPair *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcDefaultMissionProductEnum>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDefaultMissionProductEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDefaultMissionProductEnum *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostItemFromList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostItemFromList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostItemListIndexed const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostItemListIndexed *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostJourneyMilestone const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostJourneyMilestone *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostJourneyStatLevel const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostJourneyStatLevel *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostMissionComplete const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostMissionComplete *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostMoney const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostMoney *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostMoneyList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostMoneyList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostMultiItem const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostMultiItem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostNPCHabitation const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostNPCHabitation *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostPirateTribute const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostPirateTribute *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostPoliceCargoBribe const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostPoliceCargoBribe *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostProcProduct const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostProcProduct *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostProduct const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostProduct *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostProductOnlyTakeIfCanAfford const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostProductOnlyTakeIfCanAfford *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostSentinelBlockStatus const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostSentinelBlockStatus *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostStanding const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostStanding *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostStat const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostStat *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostStatCompare const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostStatCompare *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostSubstance const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostSubstance *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostItemFromList::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostItemFromList *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostItemListIndexed::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostItemListIndexed *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostJourneyMilestone::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostJourneyMilestone *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostJourneyStatLevel::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostJourneyStatLevel *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostMissionComplete::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostMissionComplete *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostMoney::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostMoney *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostMoneyList::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostMoneyList *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostMultiItem::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostMultiItem *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostNPCHabitation::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostNPCHabitation *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostOwnSettlement::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostOwnSettlement *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostPendingSettlementJudgement::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostPendingSettlementJudgement *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostPirateTribute::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostPirateTribute *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostPoliceCargoBribe::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostPoliceCargoBribe *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostPoliceCargoComply::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostPoliceCargoComply *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostProcProduct::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostProcProduct *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostProduct::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostProduct *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostProductOnlyTakeIfCanAfford::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostProductOnlyTakeIfCanAfford *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostSalvageShip::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostSalvageShip *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostSentinelBlockStatus::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostSentinelBlockStatus *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostSpecificCreatureBait::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostSpecificCreatureBait *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostStanding::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostStanding *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostStat::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostStat *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostStatCompare::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostStatCompare *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostSubstance::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostSubstance *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostItemFromList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostItemListIndexed::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostJourneyMilestone::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostJourneyStatLevel::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostMissionComplete::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostMoney::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostMoneyList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostMultiItem::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostNPCHabitation::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostOwnSettlement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostPendingSettlementJudgement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostPirateTribute::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostPoliceCargoBribe::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostPoliceCargoComply::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostProcProduct::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostProduct::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostProductOnlyTakeIfCanAfford::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostSalvageShip::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostSentinelBlockStatus::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostShipUpgradeable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostSpecificCreatureBait::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostStanding::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostStat::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostStatCompare::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostSubstance::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostItemFromList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostItemListIndexed::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostMoney::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostMoneyList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostMultiItem::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostNPCHabitation::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostProcProduct::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostProduct::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostProductOnlyTakeIfCanAfford::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostStanding::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostSubstance::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostTable::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostItemFromList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostItemFromList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostItemListIndexed::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostItemListIndexed *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostMissionComplete::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostMissionComplete *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostMoneyList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostMoneyList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostMultiItem::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostMultiItem *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostPoliceCargoBribe::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostPoliceCargoBribe *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostProduct::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostProduct *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostProductOnlyTakeIfCanAfford::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostProductOnlyTakeIfCanAfford *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostStat::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostStat *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostStatCompare::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostStatCompare *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostSubstance::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostSubstance *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostStat::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostStat *
        ]
    ),
    "cGcCostItemFromList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostItemFromList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostItemListIndexed::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostItemListIndexed *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostJourneyStatLevel::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostJourneyStatLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostMissionComplete::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostMissionComplete *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostMoney::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostMoney *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostMoneyList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostMoneyList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostMultiItem::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostMultiItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostNPCHabitation::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostNPCHabitation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostPirateTribute::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostPirateTribute *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostPoliceCargoBribe::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostPoliceCargoBribe *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostProcProduct::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostProcProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostProduct::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostProductOnlyTakeIfCanAfford::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostProductOnlyTakeIfCanAfford *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostStanding::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostStanding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostStat::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostStat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostStatCompare::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostStatCompare *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostSubstance::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostSubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostItemListIndexed::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostItemListIndexed *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCostItemListIndexed *
        ]
    ),
    "cGcCostMoneyList::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostMoneyList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCostMoneyList *
        ]
    ),
    "cGcCostNPCHabitation::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostNPCHabitation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCostNPCHabitation *
        ]
    ),
    "cGcCostProcProduct::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostProcProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCostProcProduct *
        ]
    ),
    "cGcCostStanding::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostStanding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCostStanding *
        ]
    ),
    "cGcCostStat::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostStat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCostStat *
        ]
    ),
    "cGcCostSubstance::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostSubstance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCostSubstance *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcAlienPuzzleEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAlienPuzzleEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcAlienSpeechEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAlienSpeechEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBuildingCostPartCount>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBuildingCostPartCount> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcConsumableItem>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcConsumableItem> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAlienPuzzleEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAlienPuzzleEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAlienSpeechEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAlienSpeechEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBuildingCostPartCount const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBuildingCostPartCount *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcConsumableItem const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcConsumableItem *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAlienPuzzleTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAlienPuzzleTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAlienPuzzleTableIndex const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAlienPuzzleTableIndex *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAlienRace const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAlienRace *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAlienSpeechEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAlienSpeechEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAlienSpeechTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAlienSpeechTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBreakTechByStatData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBreakTechByStatData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingBlueprint const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingBlueprint *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingCostPartCount const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingCostPartCount *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCatalogueGroups const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCatalogueGroups *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcConsumableItem const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcConsumableItem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcConsumableItemTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcConsumableItemTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostAdvanceSettlementBuilding const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostAdvanceSettlementBuilding *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostAnyCookedProduct const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostAnyCookedProduct *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostBuildingParts const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostBuildingParts *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostCargo const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostCargo *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostFleetStoredIncome const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostFleetStoredIncome *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostGameMode const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostGameMode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostHasFireteamMember const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostHasFireteamMember *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostHealth const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostHealth *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostInstalledTech const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostInstalledTech *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCostInteractionIndex const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCostInteractionIndex *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPuzzleEntry::cGcAlienPuzzleEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleEntry *
            ctypes.c_ulonglong,  # cGcAlienPuzzleEntry *
        ]
    ),
    "cTkDynamicArray<cGcAlienPuzzleEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAlienPuzzleEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcAlienSpeechEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAlienSpeechEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBuildingCostPartCount>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBuildingCostPartCount> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcConsumableItem>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcConsumableItem> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCostAdvanceSettlementBuilding::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostAdvanceSettlementBuilding *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostAnyCookedProduct::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostAnyCookedProduct *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostBuildingParts::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostBuildingParts *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostCanAdoptCreature::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostCanAdoptCreature *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostCanCustomiseCreature::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostCanCustomiseCreature *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostCanFreighterMegaWarp::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostCanFreighterMegaWarp *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostCanMilkCreature::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostCanMilkCreature *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostCanRideCreature::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostCanRideCreature *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostCreatureCanLayEggs::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostCreatureCanLayEggs *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostFleetStoredIncome::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostFleetStoredIncome *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostFrigateCargo::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostFrigateCargo *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostHasFireteamMember::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostHasFireteamMember *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostHealth::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostHealth *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostInstalledTech::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostInstalledTech *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcCostInteractionIndex::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcCostInteractionIndex *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcAlienPuzzleTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAlienPuzzleTableIndex::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAlienRace::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAlienSpeechEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAlienSpeechTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBreakTechByStatData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingBlueprint::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingCostPartCount::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCatalogueGroups::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcConsumableItem::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcConsumableItemTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostAdvanceSettlementBuilding::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostAnyCookedProduct::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostBuildingParts::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostCanAdoptCreature::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostCanCustomiseCreature::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostCanFreighterMegaWarp::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostCanMilkCreature::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostCanRideCreature::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostCargo::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostCreatureCanLayEggs::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostFleetStoredIncome::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostFrigateCargo::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostGameMode::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostHasFireteamMember::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostHealth::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostInstalledTech::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCostInteractionIndex::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAlienPuzzleTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAlienSpeechEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAlienSpeechTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBreakTechByStatData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcConsumableItem::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcConsumableItemTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostBuildingParts::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostFleetStoredIncome::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostGameMode::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCostInteractionIndex::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAlienPuzzleTable::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAlienPuzzleTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAlienSpeechTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienSpeechTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcConsumableItemTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcConsumableItemTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAlienPuzzleTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPuzzleTableIndex::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleTableIndex *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienRace::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienRace *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienSpeechEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienSpeechEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBreakTechByStatData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBreakTechByStatData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingBlueprint::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingBlueprint *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingCostPartCount::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingCostPartCount *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcConsumableItem::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcConsumableItem *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostAnyCookedProduct::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostAnyCookedProduct *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostBuildingParts::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostBuildingParts *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostGameMode::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostGameMode *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostInteractionIndex::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostInteractionIndex *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienRace::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienRace *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcAlienPuzzleTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPuzzleTableIndex::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleTableIndex *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienRace::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienRace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienSpeechEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienSpeechEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBreakTechByStatData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBreakTechByStatData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingBlueprint::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingBlueprint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcConsumableItem::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcConsumableItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBreakTechByStatData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBreakTechByStatData *
        ]
    ),
    "cGcConsumableItem::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcConsumableItem *
        ]
    ),
    "cGcCostAnyCookedProduct::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostAnyCookedProduct *
        ]
    ),
    "cGcCostInteractionIndex::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostInteractionIndex *
        ]
    ),
    "cGcAlienRace::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienRace *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcAlienPuzzleTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPuzzleTableIndex::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleTableIndex *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienRace::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienRace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienSpeechEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienSpeechEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienSpeechTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienSpeechTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBreakTechByStatData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBreakTechByStatData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingBlueprint::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingBlueprint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingCostPartCount::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingCostPartCount *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCatalogueGroups::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCatalogueGroups *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcConsumableItem::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcConsumableItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcConsumableItemTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcConsumableItemTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostAdvanceSettlementBuilding::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostAdvanceSettlementBuilding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostAnyCookedProduct::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostAnyCookedProduct *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostBuildingParts::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostBuildingParts *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostCargo::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostCargo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostFleetStoredIncome::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostFleetStoredIncome *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostGameMode::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostGameMode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostHasFireteamMember::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostHasFireteamMember *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostHealth::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostHealth *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostInstalledTech::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostInstalledTech *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCostInteractionIndex::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostInteractionIndex *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPuzzleTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienRace::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienRace *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAlienRace *
        ]
    ),
    "cGcAlienSpeechEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienSpeechEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAlienSpeechEntry *
        ]
    ),
    "cGcBreakTechByStatData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBreakTechByStatData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBreakTechByStatData *
        ]
    ),
    "cGcBuildingCostPartCount::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingCostPartCount *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBuildingCostPartCount *
        ]
    ),
    "cGcConsumableItem::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcConsumableItem *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcConsumableItem *
        ]
    ),
    "cGcCostFleetStoredIncome::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostFleetStoredIncome *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCostFleetStoredIncome *
        ]
    ),
    "cGcCostGameMode::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostGameMode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCostGameMode *
        ]
    ),
    "cGcCostInteractionIndex::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCostInteractionIndex *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCostInteractionIndex *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBackgroundSpaceEncounterInfo>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBackgroundSpaceEncounterInfo> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBountySpawnInfo>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBountySpawnInfo> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcExperienceDebugTriggerAction>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExperienceDebugTriggerAction> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerExperienceSpawnArchetypeData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerExperienceSpawnArchetypeData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerExperienceSpawnTable>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerExperienceSpawnTable> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPulseEncounterInfo>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPulseEncounterInfo> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPuzzleTextFlow>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPuzzleTextFlow> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSentinelEncounterOverride>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSentinelEncounterOverride> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSentinelSpawnNamedSequence>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSentinelSpawnNamedSequence> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSentinelSpawnSequenceGroupList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnSequenceGroupList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSentinelSpawnWave>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSentinelSpawnWave> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcAlienMood>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienMood *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcNPCPropType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPropType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBackgroundSpaceEncounterInfo const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBackgroundSpaceEncounterInfo *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBountySpawnInfo const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBountySpawnInfo *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcExperienceDebugTriggerAction const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcExperienceDebugTriggerAction *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerExperienceSpawnArchetypeData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerExperienceSpawnArchetypeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerExperienceSpawnTable const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerExperienceSpawnTable *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPulseEncounterInfo const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPulseEncounterInfo *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPuzzleTextFlow const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPuzzleTextFlow *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSentinelEncounterOverride const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSentinelEncounterOverride *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSentinelSpawnNamedSequence const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSentinelSpawnNamedSequence *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSentinelSpawnSequenceGroupList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSentinelSpawnSequenceGroupList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSentinelSpawnWave const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSentinelSpawnWave *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcAlienMood>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAlienMood *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAlienMood *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcAlienPuzzleCategory>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAlienPuzzleCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAlienPuzzleCategory *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcExperienceDebugTriggerActionTypes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExperienceDebugTriggerActionTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcExperienceDebugTriggerActionTypes *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcNPCPropType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCPropType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCPropType *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAlienMood const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAlienMood *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAlienPuzzleCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAlienPuzzleCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAlienPuzzleEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAlienPuzzleEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAlienPuzzleMissionOverride const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAlienPuzzleMissionOverride *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAlienPuzzleOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAlienPuzzleOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExperienceDebugTriggerAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExperienceDebugTriggerAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExperienceDebugTriggerActionTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExperienceDebugTriggerActionTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExperienceDebugTriggerInput const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExperienceDebugTriggerInput *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExperienceSpawnTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExperienceSpawnTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPerformanceFlyby const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPerformanceFlyby *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPerformanceGuard const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPerformanceGuard *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayfabMatchmakingAttributes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayfabMatchmakingAttributes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcSentinelSpawnSequenceGroupList,6>::cTkFixedArray<cGcSentinelSpawnSequenceGroupList,6>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcSentinelSpawnSequenceGroupList,6> *
        ]
    ),
    "cTkFixedArray<cGcSentinelSpawnSequenceGroupList,7>::cTkFixedArray<cGcSentinelSpawnSequenceGroupList,7>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcSentinelSpawnSequenceGroupList,7> *
        ]
    ),
    "cGcAIShipSpawnData::cGcAIShipSpawnData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAIShipSpawnData *
            ctypes.c_ulonglong,  # cGcAIShipSpawnData *
        ]
    ),
    "cTkFixedArray<cGcSentinelSpawnSequenceGroupList,6>::~cTkFixedArray<cGcSentinelSpawnSequenceGroupList,6>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcSentinelSpawnSequenceGroupList,6> *
        ]
    ),
    "cTkFixedArray<cGcSentinelSpawnSequenceGroupList,7>::~cTkFixedArray<cGcSentinelSpawnSequenceGroupList,7>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcSentinelSpawnSequenceGroupList,7> *
        ]
    ),
    "cGcBountySpawnInfo::~cGcBountySpawnInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBountySpawnInfo *
        ]
    ),
    "cGcExperienceSpawnTable::~cGcExperienceSpawnTable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceSpawnTable *
        ]
    ),
    "cGcPlayerExperienceAsteroidCreatureSpawnTable::~cGcPlayerExperienceAsteroidCreatureSpawnTable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceAsteroidCreatureSpawnTable *
        ]
    ),
    "cTkDynamicArray<cGcBackgroundSpaceEncounterInfo>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBackgroundSpaceEncounterInfo> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBountySpawnInfo>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBountySpawnInfo> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPlayerExperienceSpawnArchetypeData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerExperienceSpawnArchetypeData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPlayerExperienceSpawnTable>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerExperienceSpawnTable> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPulseEncounterInfo>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPulseEncounterInfo> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPuzzleTextFlow>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPuzzleTextFlow> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSentinelEncounterOverride>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSentinelEncounterOverride> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSentinelSpawnNamedSequence>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSentinelSpawnNamedSequence> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSentinelSpawnWave>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSentinelSpawnWave> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAlienMood::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAlienPuzzleCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAlienPuzzleEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAlienPuzzleMissionOverride::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAlienPuzzleOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExperienceDebugTriggerAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExperienceDebugTriggerActionTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExperienceDebugTriggerInput::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExperienceSpawnTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPerformanceFlyby::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPerformanceGuard::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayfabMatchmakingAttributes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAlienPuzzleEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAlienPuzzleOption::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExperienceDebugTriggerInput::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExperienceSpawnTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPerformanceGuard::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcPlayerExperienceSpawnArchetypeData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerExperienceSpawnArchetypeData> *
        ]
    ),
    "cTkDynamicArray<cGcSentinelSpawnNamedSequence>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSentinelSpawnNamedSequence> *
        ]
    ),
    "cGcAlienPuzzleEntry::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleEntry *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAlienPuzzleOption::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleOption *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExperienceSpawnTable::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceSpawnTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAlienPuzzleEntry::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleEntry *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAlienPuzzleOption::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleOption *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExperienceSpawnTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceSpawnTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAlienMood::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienMood *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPuzzleCategory::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleCategory *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPuzzleEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPuzzleMissionOverride::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleMissionOverride *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPuzzleOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExperienceDebugTriggerAction::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceDebugTriggerAction *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExperienceDebugTriggerActionTypes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceDebugTriggerActionTypes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExperienceDebugTriggerInput::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceDebugTriggerInput *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExperienceSpawnTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceSpawnTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPerformanceFlyby::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPerformanceFlyby *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayfabMatchmakingAttributes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayfabMatchmakingAttributes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayfabMatchmakingAttributes::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayfabMatchmakingAttributes *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcAlienMood::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienMood *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPuzzleCategory::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPuzzleEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPuzzleMissionOverride::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleMissionOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPuzzleOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExperienceDebugTriggerAction::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceDebugTriggerAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExperienceDebugTriggerActionTypes::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceDebugTriggerActionTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExperienceDebugTriggerInput::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceDebugTriggerInput *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExperienceSpawnTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceSpawnTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExperienceSpawnTable::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceSpawnTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPuzzleEntry::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleEntry *
        ]
    ),
    "cGcAlienPuzzleMissionOverride::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleMissionOverride *
        ]
    ),
    "cGcAlienPuzzleOption::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleOption *
        ]
    ),
    "cGcExperienceSpawnTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceSpawnTable *
        ]
    ),
    "cGcPerformanceGuard::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPerformanceGuard *
        ]
    ),
    "cGcPlayfabMatchmakingAttributes::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayfabMatchmakingAttributes *
        ]
    ),
    "cGcPlayfabMatchmakingAttributes::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayfabMatchmakingAttributes *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcAlienMood::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienMood *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPuzzleCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPuzzleEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPuzzleMissionOverride::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleMissionOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPuzzleOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExperienceDebugTriggerAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceDebugTriggerAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExperienceDebugTriggerActionTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceDebugTriggerActionTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExperienceDebugTriggerInput::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceDebugTriggerInput *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExperienceSpawnTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceSpawnTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPerformanceFlyby::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPerformanceFlyby *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPerformanceGuard::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPerformanceGuard *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayfabMatchmakingAttributes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayfabMatchmakingAttributes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienMood::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienMood *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAlienMood *
        ]
    ),
    "cGcAlienPuzzleCategory::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAlienPuzzleCategory *
        ]
    ),
    "cGcAlienPuzzleEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAlienPuzzleEntry *
        ]
    ),
    "cGcAlienPuzzleMissionOverride::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleMissionOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAlienPuzzleMissionOverride *
        ]
    ),
    "cGcAlienPuzzleOption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPuzzleOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAlienPuzzleOption *
        ]
    ),
    "cGcExperienceDebugTriggerAction::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceDebugTriggerAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcExperienceDebugTriggerAction *
        ]
    ),
    "cGcExperienceDebugTriggerActionTypes::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceDebugTriggerActionTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcExperienceDebugTriggerActionTypes *
        ]
    ),
    "cGcExperienceSpawnTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceSpawnTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcExperienceSpawnTable *
        ]
    ),
    "cGcPerformanceFlyby::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPerformanceFlyby *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPerformanceFlyby *
        ]
    ),
    "cGcPerformanceGuard::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPerformanceGuard *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPerformanceGuard *
        ]
    ),
    "cGcPlayfabMatchmakingAttributes::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayfabMatchmakingAttributes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayfabMatchmakingAttributes *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcJourneyCategory>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcJourneyCategory> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcJourneyMedal>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcJourneyMedal> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcJourneyCategory const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcJourneyCategory *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcJourneyMedal const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcJourneyMedal *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcJourneyCategoryType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcJourneyCategoryType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcJourneyCategoryType *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcJourney const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcJourney *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcJourneyCategory const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcJourneyCategory *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcJourneyCategoryType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcJourneyCategoryType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcJourneyMedal const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcJourneyMedal *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcJourneyMedalTiers const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcJourneyMedalTiers *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcJourneyMedalType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcJourneyMedalType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMessageCrime const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMessageCrime *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMessageFiendCrime const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMessageFiendCrime *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMessageNPCBehaviourEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMessageNPCBehaviourEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMessagePetBehaviourEvent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMessagePetBehaviourEvent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMessageProjectileImpact const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMessageProjectileImpact *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMessageRequestTakeOff const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMessageRequestTakeOff *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMessageRequestWarp const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMessageRequestWarp *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMessageSubstanceMined const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMessageSubstanceMined *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMessageSummonAndDismiss const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMessageSummonAndDismiss *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMessageTrackTargetAlert const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMessageTrackTargetAlert *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMessageUpdateFrigateSpeed const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMessageUpdateFrigateSpeed *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJourneyCategory::~cGcJourneyCategory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyCategory *
        ]
    ),
    "cGcJourneyMedal::~cGcJourneyMedal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyMedal *
        ]
    ),
    "cTkDynamicArray<cGcJourneyCategory>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcJourneyCategory> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcJourneyMedal>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcJourneyMedal> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcJourney::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcJourneyCategory::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcJourneyCategoryType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcJourneyMedal::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcJourneyMedalTiers::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcJourneyMedalType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMessageCrime::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMessageFiendCrime::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMessageNPCBehaviourEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMessagePetBehaviourEvent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMessageProjectileImpact::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMessageProjectileLaunch::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMessageRequestTakeOff::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMessageRequestWarp::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMessageSubstanceMined::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMessageSummonAndDismiss::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMessageTitanFall::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMessageTrackTargetAlert::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMessageUpdateFrigateSpeed::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcJourney::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcJourneyCategory::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcJourneyMedal::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMessageFiendCrime::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMessageNPCBehaviourEvent::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMessagePetBehaviourEvent::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMessageProjectileImpact::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcJourneyCategory::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyCategory *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcJourney::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourney *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcJourneyCategory::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyCategory *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcJourneyCategory::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyCategory *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJourneyCategoryType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyCategoryType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJourneyMedal::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyMedal *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMessageCrime::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageCrime *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMessageFiendCrime::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageFiendCrime *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMessageNPCBehaviourEvent::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageNPCBehaviourEvent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMessagePetBehaviourEvent::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessagePetBehaviourEvent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMessageProjectileImpact::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageProjectileImpact *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMessageTrackTargetAlert::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageTrackTargetAlert *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJourneyCategory::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJourneyCategoryType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyCategoryType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJourneyMedal::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyMedal *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJourneyMedalTiers::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyMedalTiers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMessageProjectileImpact::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageProjectileImpact *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJourneyMedal::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyMedal *
        ]
    ),
    "cGcJourneyMedalTiers::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyMedalTiers *
        ]
    ),
    "cGcMessageCrime::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageCrime *
        ]
    ),
    "cGcMessageFiendCrime::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageFiendCrime *
        ]
    ),
    "cGcMessageNPCBehaviourEvent::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageNPCBehaviourEvent *
        ]
    ),
    "cGcMessagePetBehaviourEvent::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessagePetBehaviourEvent *
        ]
    ),
    "cGcMessageProjectileImpact::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageProjectileImpact *
        ]
    ),
    "cGcMessageTrackTargetAlert::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageTrackTargetAlert *
        ]
    ),
    "cGcJourney::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourney *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJourneyCategory::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJourneyCategoryType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyCategoryType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJourneyMedal::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyMedal *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJourneyMedalTiers::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyMedalTiers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJourneyMedalType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyMedalType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMessageCrime::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageCrime *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMessageFiendCrime::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageFiendCrime *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMessageNPCBehaviourEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageNPCBehaviourEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMessagePetBehaviourEvent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessagePetBehaviourEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMessageProjectileImpact::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageProjectileImpact *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMessageRequestTakeOff::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageRequestTakeOff *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMessageRequestWarp::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageRequestWarp *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMessageSubstanceMined::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageSubstanceMined *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMessageSummonAndDismiss::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageSummonAndDismiss *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMessageTrackTargetAlert::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageTrackTargetAlert *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMessageUpdateFrigateSpeed::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageUpdateFrigateSpeed *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJourneyCategory::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyCategory *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcJourneyCategory *
        ]
    ),
    "cGcJourneyCategoryType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyCategoryType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcJourneyCategoryType *
        ]
    ),
    "cGcJourneyMedal::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyMedal *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcJourneyMedal *
        ]
    ),
    "cGcJourneyMedalTiers::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyMedalTiers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcJourneyMedalTiers *
        ]
    ),
    "cGcMessageFiendCrime::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageFiendCrime *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMessageFiendCrime *
        ]
    ),
    "cGcMessageNPCBehaviourEvent::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageNPCBehaviourEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMessageNPCBehaviourEvent *
        ]
    ),
    "cGcMessagePetBehaviourEvent::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessagePetBehaviourEvent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMessagePetBehaviourEvent *
        ]
    ),
    "cGcMessageProjectileImpact::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageProjectileImpact *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMessageProjectileImpact *
        ]
    ),
    "cGcMessageTrackTargetAlert::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMessageTrackTargetAlert *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMessageTrackTargetAlert *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcAccessibleOverride_Text>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAccessibleOverride_Text> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcActionSet>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcActionSet> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcActionSetAction>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcActionSetAction> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcInputBinding>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInputBinding> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcInputBindingSet>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInputBindingSet> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNGuiSpecialTextStyleData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNGuiSpecialTextStyleData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNGuiStyleAnimationKeyframeData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNGuiStyleAnimationKeyframeData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerCharacterIKStateData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerCharacterIKStateData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcVROverride_Text>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcVROverride_Text> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcActionSetType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionSetType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcActionUseType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionUseType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAccessibleOverride_Text const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAccessibleOverride_Text *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcActionSet const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcActionSet *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcActionSetAction const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcActionSetAction *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInputActionInfo const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInputActionInfo *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInputBinding const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInputBinding *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInputBindingSet const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInputBindingSet *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNGuiSpecialTextStyleData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNGuiSpecialTextStyleData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNGuiStyleAnimationKeyframeData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNGuiStyleAnimationKeyframeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerCharacterIKStateData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerCharacterIKStateData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcVROverride_Text const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcVROverride_Text *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcActionSetType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcActionSetType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcActionSetType *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcActionUseType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcActionUseType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcActionUseType *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcActionSet const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcActionSet *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcActionSetAction const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcActionSetAction *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcActionSetType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcActionSetType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcActionSets const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcActionSets *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcActionUseType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcActionUseType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcIKConstraint const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcIKConstraint *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInputActionInfo const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInputActionInfo *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInputActionInfoMap const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInputActionInfoMap *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInputBinding const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInputBinding *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInputBindingSet const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInputBindingSet *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInputBindings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInputBindings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNGuiSpecialTextStyles const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiSpecialTextStyles *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNGuiStyleAnimationData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiStyleAnimationData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNGuiStyleAnimationKeyframeData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiStyleAnimationKeyframeData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNGuiTextData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiTextData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStyleProp_Colour const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStyleProp_Colour *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStyleProp_Font const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStyleProp_Font *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStyleProp_Size const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStyleProp_Size *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcVROverride_Layout const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcVROverride_Layout *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcVROverride_Text const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcVROverride_Text *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcInputActionInfo,291>::cTkFixedArray<cGcInputActionInfo,291>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcInputActionInfo,291> *
        ]
    ),
    "cTkDynamicArray<cGcActionSet>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcActionSet> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcNGuiSpecialTextStyleData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNGuiSpecialTextStyleData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcNGuiStyleAnimationKeyframeData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNGuiStyleAnimationKeyframeData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNGuiTextData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiTextData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStyleProp_Colour::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcStyleProp_Colour *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcStyleProp_Font::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcStyleProp_Font *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcStyleProp_Size::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cGcStyleProp_Size *
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
        ]
    ),
    "cGcActionSet::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcActionSetAction::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcActionSetType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcActionSets::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcActionUseType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcIKConstraint::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInputActionInfo::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInputActionInfoMap::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInputBinding::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInputBindingSet::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInputBindings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiSpecialTextStyles::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiStyleAnimationData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiStyleAnimationKeyframeData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiTextData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStyleProp_Colour::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStyleProp_Font::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStyleProp_Size::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVROverride_Layout::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcVROverride_Text::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcActionSet::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcActionSetAction::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcActionSets::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcIKConstraint::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInputActionInfo::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInputActionInfoMap::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInputBinding::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInputBindingSet::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInputBindings::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiSpecialTextStyles::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiStyleAnimationData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiStyleAnimationKeyframeData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiTextData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcActionSet>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcActionSet> *
        ]
    ),
    "cTkDynamicArray<cGcNGuiSpecialTextStyleData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNGuiSpecialTextStyleData> *
        ]
    ),
    "cTkDynamicArray<cGcNGuiStyleAnimationKeyframeData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNGuiStyleAnimationKeyframeData> *
        ]
    ),
    "cGcIKConstraint::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIKConstraint *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInputBindingSet::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputBindingSet *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiStyleAnimationData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyleAnimationData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiStyleAnimationKeyframeData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyleAnimationKeyframeData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcActionSets::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionSets *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcIKConstraint::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIKConstraint *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInputActionInfoMap::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActionInfoMap *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInputBindingSet::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputBindingSet *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInputBindings::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputBindings *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiSpecialTextStyles::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpecialTextStyles *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiStyleAnimationData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyleAnimationData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiStyleAnimationKeyframeData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyleAnimationKeyframeData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcActionSet::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionSet *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActionSetAction::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionSetAction *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActionSetType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionSetType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActionUseType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionUseType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcIKConstraint::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcIKConstraint *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputActionInfo::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActionInfo *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputBinding::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputBinding *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputBindingSet::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputBindingSet *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiStyleAnimationData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyleAnimationData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiStyleAnimationKeyframeData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyleAnimationKeyframeData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiTextData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiTextData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStyleProp_Colour::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStyleProp_Colour *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVROverride_Layout::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVROverride_Layout *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVROverride_Text::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVROverride_Text *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActionSet::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionSet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActionSetType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionSetType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActionUseType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionUseType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcIKConstraint::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIKConstraint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputActionInfo::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActionInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputBinding::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputBinding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiStyleAnimationData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyleAnimationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiTextData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiTextData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVROverride_Layout::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVROverride_Layout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVROverride_Text::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVROverride_Text *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActionSet::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionSet *
        ]
    ),
    "cGcIKConstraint::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIKConstraint *
        ]
    ),
    "cGcInputActionInfo::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActionInfo *
        ]
    ),
    "cGcInputActionInfoMap::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActionInfoMap *
        ]
    ),
    "cGcNGuiStyleAnimationData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyleAnimationData *
        ]
    ),
    "cGcNGuiTextData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiTextData *
        ]
    ),
    "cGcNGuiTextData::WriteToClassPtr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiTextData *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcActionSet::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionSet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActionSetAction::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionSetAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActionSetType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionSetType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActionSets::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionSets *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActionUseType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionUseType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcIKConstraint::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIKConstraint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputActionInfo::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActionInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputActionInfoMap::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActionInfoMap *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputBinding::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputBinding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputBindingSet::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputBindingSet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInputBindings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputBindings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiSpecialTextStyles::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpecialTextStyles *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiStyleAnimationData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyleAnimationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiStyleAnimationKeyframeData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyleAnimationKeyframeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiTextData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiTextData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStyleProp_Font::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStyleProp_Font *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStyleProp_Size::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStyleProp_Size *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVROverride_Layout::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVROverride_Layout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVROverride_Text::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVROverride_Text *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcActionSet::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionSet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcActionSet *
        ]
    ),
    "cGcActionSetAction::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionSetAction *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcActionSetAction *
        ]
    ),
    "cGcActionSetType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionSetType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcActionSetType *
        ]
    ),
    "cGcActionUseType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcActionUseType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcActionUseType *
        ]
    ),
    "cGcIKConstraint::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcIKConstraint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcIKConstraint *
        ]
    ),
    "cGcInputActionInfo::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputActionInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInputActionInfo *
        ]
    ),
    "cGcInputBinding::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputBinding *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInputBinding *
        ]
    ),
    "cGcInputBindingSet::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInputBindingSet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInputBindingSet *
        ]
    ),
    "cGcNGuiStyleAnimationData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyleAnimationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNGuiStyleAnimationData *
        ]
    ),
    "cGcNGuiStyleAnimationKeyframeData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiStyleAnimationKeyframeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNGuiStyleAnimationKeyframeData *
        ]
    ),
    "cGcNGuiTextData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiTextData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNGuiTextData *
        ]
    ),
    "cGcVROverride_Text::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVROverride_Text *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcVROverride_Text *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcAccessibleOverride_Layout>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAccessibleOverride_Layout> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNGuiPresetGraphic>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiPresetGraphic *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcNGuiSpecialTextImageData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNGuiSpecialTextImageData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcVROverride_Layout>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcVROverride_Layout> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcNGuiElementData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElementData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcNGuiLayoutData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayoutData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkNGuiGraphicStyle>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAccessibleOverride_Layout const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAccessibleOverride_Layout *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNGuiPresetGraphic const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNGuiPresetGraphic *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNGuiPresetText const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNGuiPresetText *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNGuiSpecialTextImageData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNGuiSpecialTextImageData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcVROverride_Layout const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcVROverride_Layout *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcAccessibleOverride_Layout>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAccessibleOverride_Layout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAccessibleOverride_Layout *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcNGuiElementData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiElementData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNGuiElementData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcNGuiLayoutData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiLayoutData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNGuiLayoutData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcShipHUDTargetIconData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShipHUDTargetIconData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcShipHUDTargetIconData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkNGuiGraphicStyle>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiGraphicStyle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNGuiGraphicStyle *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAccessibleOverride_Layout const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAccessibleOverride_Layout *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAccessibleOverride_Text const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAccessibleOverride_Text *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNGuiActionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiActionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNGuiElementData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiElementData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNGuiFileBrowserRecents const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiFileBrowserRecents *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNGuiGraphicData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiGraphicData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNGuiLayerData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiLayerData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNGuiLayoutData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiLayoutData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNGuiPreset const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiPreset *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNGuiPresetGraphic const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiPresetGraphic *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNGuiPresetText const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiPresetText *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNGuiSpacingData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiSpacingData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNGuiSpecialTextImageData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiSpecialTextImageData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNGuiSpecialTextImages const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiSpecialTextImages *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNGuiSpecialTextStyleData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNGuiSpecialTextStyleData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcShipHUDTargetData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShipHUDTargetData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcShipHUDTargetIconData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcShipHUDTargetIconData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTextStyleOutline const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTextStyleOutline *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTextStyleShadow const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTextStyleShadow *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcNGuiStyleAnimationKeyframeData>::~cTkDynamicArray<cGcNGuiStyleAnimationKeyframeData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNGuiStyleAnimationKeyframeData> *
        ]
    ),
    "cTkDynamicArray<cGcNGuiSpecialTextImageData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcNGuiSpecialTextImageData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNGuiGraphicData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiGraphicData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiLayerData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiLayerData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiSpacingData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNGuiSpacingData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAccessibleOverride_Layout::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAccessibleOverride_Text::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiActionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiElementData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiFileBrowserRecents::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiGraphicData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiLayerData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiLayoutData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiPreset::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiPresetGraphic::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiPresetText::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiSpacingData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiSpecialTextImageData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiSpecialTextImages::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiSpecialTextStyleData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShipHUDTargetData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcShipHUDTargetIconData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTextStyleOutline::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTextStylePlain::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTextStyleShadow::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiElementData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiGraphicData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiLayerData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiLayoutData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiPreset::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiPresetGraphic::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiPresetText::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiSpacingData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiSpecialTextImages::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiSpecialTextStyleData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiElementData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElementData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiLayoutData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayoutData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiSpecialTextStyleData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpecialTextStyleData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiElementData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElementData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiLayerData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayerData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiLayoutData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayoutData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNGuiSpecialTextStyleData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpecialTextStyleData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAccessibleOverride_Layout::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAccessibleOverride_Layout *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAccessibleOverride_Text::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAccessibleOverride_Text *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiActionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiActionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiElementData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElementData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiLayerData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayerData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiLayoutData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayoutData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiPresetGraphic::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiPresetGraphic *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiPresetText::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiPresetText *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiSpecialTextImageData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpecialTextImageData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiSpecialTextStyleData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpecialTextStyleData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipHUDTargetData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUDTargetData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipHUDTargetIconData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUDTargetIconData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTextStyleOutline::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextStyleOutline *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTextStyleShadow::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextStyleShadow *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAccessibleOverride_Layout::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAccessibleOverride_Layout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAccessibleOverride_Text::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAccessibleOverride_Text *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiElementData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElementData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiLayoutData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayoutData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiPreset::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiPreset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiPresetText::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiPresetText *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiSpecialTextStyleData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpecialTextStyleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipHUDTargetData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUDTargetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipHUDTargetIconData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUDTargetIconData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiActionData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiActionData *
        ]
    ),
    "cGcNGuiSpacingData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpacingData *
        ]
    ),
    "cGcNGuiFileBrowserRecents::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiFileBrowserRecents *
        ]
    ),
    "cGcNGuiGraphicData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGraphicData *
        ]
    ),
    "cGcNGuiLayerData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayerData *
        ]
    ),
    "cGcNGuiLayoutData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayoutData *
        ]
    ),
    "cGcNGuiPreset::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiPreset *
        ]
    ),
    "cGcNGuiPresetGraphic::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiPresetGraphic *
        ]
    ),
    "cGcNGuiPresetText::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiPresetText *
        ]
    ),
    "cGcNGuiSpecialTextImageData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpecialTextImageData *
        ]
    ),
    "cGcNGuiSpecialTextStyleData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpecialTextStyleData *
        ]
    ),
    "cGcShipHUDTargetData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUDTargetData *
        ]
    ),
    "cGcShipHUDTargetIconData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUDTargetIconData *
        ]
    ),
    "cGcNGuiGraphicData::WriteToClassPtr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGraphicData *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiLayerData::WriteToClassPtr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayerData *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNGuiSpacingData::WriteToClassPtr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpacingData *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAccessibleOverride_Layout::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAccessibleOverride_Layout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAccessibleOverride_Text::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAccessibleOverride_Text *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiActionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiActionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiElementData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElementData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiFileBrowserRecents::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiFileBrowserRecents *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiGraphicData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiGraphicData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiLayerData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiLayoutData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayoutData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiPreset::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiPreset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiPresetGraphic::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiPresetGraphic *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiPresetText::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiPresetText *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiSpacingData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpacingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiSpecialTextImageData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpecialTextImageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiSpecialTextImages::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpecialTextImages *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNGuiSpecialTextStyleData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpecialTextStyleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipHUDTargetData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUDTargetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcShipHUDTargetIconData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUDTargetIconData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTextStyleOutline::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextStyleOutline *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTextStyleShadow::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextStyleShadow *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAccessibleOverride_Layout::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAccessibleOverride_Layout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAccessibleOverride_Layout *
        ]
    ),
    "cGcNGuiElementData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiElementData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNGuiElementData *
        ]
    ),
    "cGcNGuiLayerData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNGuiLayerData *
        ]
    ),
    "cGcNGuiLayoutData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiLayoutData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNGuiLayoutData *
        ]
    ),
    "cGcNGuiPresetGraphic::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiPresetGraphic *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNGuiPresetGraphic *
        ]
    ),
    "cGcNGuiPresetText::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiPresetText *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNGuiPresetText *
        ]
    ),
    "cGcNGuiSpecialTextImageData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpecialTextImageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNGuiSpecialTextImageData *
        ]
    ),
    "cGcNGuiSpecialTextStyleData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNGuiSpecialTextStyleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNGuiSpecialTextStyleData *
        ]
    ),
    "cGcShipHUDTargetData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUDTargetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcShipHUDTargetData *
        ]
    ),
    "cGcShipHUDTargetIconData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipHUDTargetIconData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcShipHUDTargetIconData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcScannerIcon>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIcon *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcTextPreset>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTextPreset> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcScannerIcon>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIcon *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcScannerIcon const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcScannerIcon *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcTextPreset const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcTextPreset *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcScannerIcon>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScannerIcon *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScannerIcon *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScannerIconTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScannerIconTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScannerIcons const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScannerIcons *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpaceMapObjectData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpaceMapObjectData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTextPreset const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTextPreset *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTextPresetTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTextPresetTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcScannerIcon,4>::cTkFixedArray<cGcScannerIcon,4>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcScannerIcon,4> *
        ]
    ),
    "cTkFixedArray<cGcScannerIcon,6>::cTkFixedArray<cGcScannerIcon,6>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcScannerIcon,6> *
        ]
    ),
    "cTkFixedArray<cGcScannerIcon,33>::cTkFixedArray<cGcScannerIcon,33>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcScannerIcon,33> *
        ]
    ),
    "cTkFixedArray<cGcScannerIcon,54>::cTkFixedArray<cGcScannerIcon,54>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcScannerIcon,54> *
        ]
    ),
    "cTkDynamicArray<cGcTextPreset>::~cTkDynamicArray<cGcTextPreset>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTextPreset> *
        ]
    ),
    "cTkFixedArray<cGcScannerIcon,4>::~cTkFixedArray<cGcScannerIcon,4>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcScannerIcon,4> *
        ]
    ),
    "cTkFixedArray<cGcScannerIcon,6>::~cTkFixedArray<cGcScannerIcon,6>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcScannerIcon,6> *
        ]
    ),
    "cTkFixedArray<cGcScannerIcon,33>::~cTkFixedArray<cGcScannerIcon,33>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcScannerIcon,33> *
        ]
    ),
    "cTkFixedArray<cGcScannerIcon,54>::~cTkFixedArray<cGcScannerIcon,54>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcScannerIcon,54> *
        ]
    ),
    "cGcBaseBuildingMaterial::~cGcBaseBuildingMaterial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingMaterial *
        ]
    ),
    "cGcScannerIcons::~cGcScannerIcons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIcons *
        ]
    ),
    "cTkDynamicArray<cGcTextPreset>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTextPreset> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcScannerIconTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScannerIcons::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpaceMapObjectData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTextPreset::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTextPresetTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScannerIcons::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTextPreset::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTextPresetTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcTextPreset>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTextPreset> *
        ]
    ),
    "cGcScannerIcons::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIcons *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTextPreset::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextPreset *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTextPreset::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextPreset *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScannerIconTypes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIconTypes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannerIcons::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIcons *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceMapObjectData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMapObjectData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTextPreset::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextPreset *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannerIconTypes::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIconTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannerIcons::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIcons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceMapObjectData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMapObjectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTextPreset::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextPreset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannerIcons::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIcons *
        ]
    ),
    "cGcSpaceMapObjectData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMapObjectData *
        ]
    ),
    "cGcTextPreset::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextPreset *
        ]
    ),
    "cGcScannerIconTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIconTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannerIcons::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIcons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceMapObjectData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMapObjectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTextPreset::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextPreset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTextPresetTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextPresetTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannerIconTypes::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIconTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScannerIconTypes *
        ]
    ),
    "cGcScannerIcons::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIcons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScannerIcons *
        ]
    ),
    "cGcSpaceMapObjectData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMapObjectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSpaceMapObjectData *
        ]
    ),
    "cGcTextPreset::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTextPreset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTextPreset *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcFontData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFontData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcHUDStartup>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDStartup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPunctuationDelay>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPunctuationDelay> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPunctuationDelayData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPunctuationDelayData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcHUDComponent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDComponent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFontData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFontData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcHUDStartup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcHUDStartup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPunctuationDelay const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPunctuationDelay *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPunctuationDelayData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPunctuationDelayData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcHUDComponent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHUDComponent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcHUDComponent *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcHUDStartup>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHUDStartup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcHUDStartup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPunctuationDelayData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPunctuationDelayData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPunctuationDelayData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcScannerIconHighlightTypes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScannerIconHighlightTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScannerIconHighlightTypes *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcByteBeatIcons const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcByteBeatIcons *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDiscoveryHelperTimings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryHelperTimings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFontData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFontData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFontTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFontTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFontTypesEnum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFontTypesEnum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGenericIconTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGenericIconTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHUDComponent const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHUDComponent *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHUDEffectRewardData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHUDEffectRewardData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHUDImageData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHUDImageData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHUDLayerData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHUDLayerData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHUDMarkerData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHUDMarkerData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHUDStartup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHUDStartup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHUDStartupTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHUDStartupTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHUDTextData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHUDTextData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryFilterOptions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryFilterOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventorySlotActionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventorySlotActionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcModelViewCollection const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcModelViewCollection *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPunctuationDelay const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPunctuationDelay *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPunctuationDelayData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPunctuationDelayData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPunctuationDelayTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPunctuationDelayTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScannerBuildingIconTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScannerBuildingIconTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScannerIcon const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScannerIcon *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScannerIconHighlightTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScannerIconHighlightTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cTkModelRendererData,41>::cTkFixedArray<cTkModelRendererData,41>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkModelRendererData,41> *
        ]
    ),
    "cTkFixedArray<cTkTextureResource,18>::cTkFixedArray<cTkTextureResource,18>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkTextureResource,18> *
        ]
    ),
    "cTkDynamicArray<cGcExpeditionDebriefPunctuation>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExpeditionDebriefPunctuation> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcByteBeatIcons::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDiscoveryHelperTimings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFontData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFontTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFontTypesEnum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGenericIconTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHUDComponent::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHUDEffectRewardData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHUDImageData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHUDLayerData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHUDMarkerData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHUDStartup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHUDStartupTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHUDTextData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryFilterOptions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventorySlotActionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcModelViewCollection::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPunctuationDelay::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPunctuationDelayData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPunctuationDelayTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScannerBuildingIconTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScannerIcon::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScannerIconHighlightTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcByteBeatIcons::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcHUDEffectRewardData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcHUDLayerData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcHUDStartup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcHUDStartupTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcHUDTextData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventorySlotActionData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcModelViewCollection::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPunctuationDelayData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPunctuationDelayTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScannerIcon::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcScannerIcon::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingMaterial *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcByteBeatIcons::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatIcons *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcHUDStartupTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDStartupTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventorySlotActionData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventorySlotActionData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcModelViewCollection::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModelViewCollection *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPunctuationDelayTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPunctuationDelayTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDiscoveryHelperTimings::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryHelperTimings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFontData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFontData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFontTypesEnum::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFontTypesEnum *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericIconTypes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericIconTypes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDComponent::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDComponent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDEffectRewardData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDEffectRewardData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDImageData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDImageData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDMarkerData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarkerData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDStartup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDStartup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDStartupTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDStartupTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventorySlotActionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventorySlotActionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPunctuationDelay::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPunctuationDelay *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPunctuationDelayData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPunctuationDelayData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannerIcon::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIcon *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannerIconHighlightTypes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIconHighlightTypes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannerIconHighlightTypes::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIconHighlightTypes *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcByteBeatIcons::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatIcons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryHelperTimings::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryHelperTimings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFontTypesEnum::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFontTypesEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericIconTypes::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericIconTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDComponent::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDComponent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDEffectRewardData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDEffectRewardData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDStartupTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDStartupTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventorySlotActionData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventorySlotActionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcModelViewCollection::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModelViewCollection *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannerIcon::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIcon *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannerIconHighlightTypes::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIconHighlightTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatIcons::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatIcons *
        ]
    ),
    "cGcDiscoveryHelperTimings::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryHelperTimings *
        ]
    ),
    "cGcFontTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFontTable *
        ]
    ),
    "cGcHUDEffectRewardData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDEffectRewardData *
        ]
    ),
    "cGcHUDImageData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDImageData *
        ]
    ),
    "cGcHUDMarkerData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarkerData *
        ]
    ),
    "cGcHUDStartup::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDStartup *
        ]
    ),
    "cGcHUDStartupTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDStartupTable *
        ]
    ),
    "cGcHUDTextData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDTextData *
        ]
    ),
    "cGcInventorySlotActionData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventorySlotActionData *
        ]
    ),
    "cGcModelViewCollection::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModelViewCollection *
        ]
    ),
    "cGcScannerIcon::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIcon *
        ]
    ),
    "cGcScannerIconHighlightTypes::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIconHighlightTypes *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcByteBeatIcons::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatIcons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryHelperTimings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryHelperTimings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFontData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFontData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFontTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFontTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFontTypesEnum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFontTypesEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGenericIconTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericIconTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDComponent::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDComponent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDEffectRewardData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDEffectRewardData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDImageData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDImageData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDLayerData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDLayerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDMarkerData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDMarkerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDStartup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDStartup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDStartupTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDStartupTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDTextData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDTextData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryFilterOptions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryFilterOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventorySlotActionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventorySlotActionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcModelViewCollection::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModelViewCollection *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPunctuationDelay::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPunctuationDelay *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPunctuationDelayData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPunctuationDelayData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPunctuationDelayTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPunctuationDelayTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannerBuildingIconTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerBuildingIconTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannerIcon::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIcon *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScannerIconHighlightTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIconHighlightTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDiscoveryHelperTimings::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDiscoveryHelperTimings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDiscoveryHelperTimings *
        ]
    ),
    "cGcFontData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFontData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFontData *
        ]
    ),
    "cGcFontTypesEnum::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFontTypesEnum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFontTypesEnum *
        ]
    ),
    "cGcGenericIconTypes::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGenericIconTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGenericIconTypes *
        ]
    ),
    "cGcHUDComponent::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDComponent *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcHUDComponent *
        ]
    ),
    "cGcHUDEffectRewardData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDEffectRewardData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcHUDEffectRewardData *
        ]
    ),
    "cGcHUDStartup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDStartup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcHUDStartup *
        ]
    ),
    "cGcHUDStartupTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDStartupTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcHUDStartupTable *
        ]
    ),
    "cGcHUDTextData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDTextData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcHUDTextData *
        ]
    ),
    "cGcInventorySlotActionData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventorySlotActionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventorySlotActionData *
        ]
    ),
    "cGcModelViewCollection::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModelViewCollection *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPunctuationDelay::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPunctuationDelay *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPunctuationDelay *
        ]
    ),
    "cGcPunctuationDelayData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPunctuationDelayData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPunctuationDelayData *
        ]
    ),
    "cGcScannerIcon::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIcon *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScannerIcon *
        ]
    ),
    "cGcScannerIconHighlightTypes::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScannerIconHighlightTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScannerIconHighlightTypes *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcScreenFilterData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScreenFilterData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcScreenFilterData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcScreenFilterData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCameraShakeCapturedData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraShakeCapturedData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCameraShakeCapturedData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCameraShakeMechanicalData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraShakeMechanicalData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCameraShakeMechanicalData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBootLogoData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBootLogoData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCameraAmbientBuildingData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraAmbientBuildingData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCameraAmbientSpaceData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraAmbientSpaceData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCameraAmbientSpecialData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraAmbientSpecialData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCameraAnimationData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraAnimationData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCameraAnomalySetupData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraAnomalySetupData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCameraShakeCapturedData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraShakeCapturedData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCameraShakeData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraShakeData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCameraShakeMechanicalData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraShakeMechanicalData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCameraSpawnSetupData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraSpawnSetupData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHUDManagerData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHUDManagerData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPresetTextureData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPresetTextureData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScreenFilterData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScreenFilterData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScreenFilterTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScreenFilterTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScreenFilters const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScreenFilters *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTexturePrefetchData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTexturePrefetchData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWorldUISettings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWorldUISettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBootLogoData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCameraAmbientBuildingData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCameraAmbientSpaceData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCameraAmbientSpecialData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCameraAnimationData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCameraAnomalySetupData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCameraShakeCapturedData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCameraShakeData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCameraShakeMechanicalData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCameraSpawnSetupData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHUDManagerData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPresetTextureData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScreenFilterData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScreenFilterTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcScreenFilters::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTexturePrefetchData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWorldUISettings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCameraAnimationData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcHUDManagerData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTexturePrefetchData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcHUDManagerData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManagerData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBootLogoData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBootLogoData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraAmbientBuildingData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAmbientBuildingData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraAmbientSpaceData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAmbientSpaceData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraAmbientSpecialData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAmbientSpecialData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraAnomalySetupData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAnomalySetupData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraShakeCapturedData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeCapturedData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraShakeData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraShakeMechanicalData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeMechanicalData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraSpawnSetupData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraSpawnSetupData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDManagerData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManagerData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScreenFilterData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScreenFilterData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScreenFilters::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScreenFilters *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWorldUISettings::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWorldUISettings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraAmbientBuildingData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAmbientBuildingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraAmbientSpaceData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAmbientSpaceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraAmbientSpecialData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAmbientSpecialData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraAnimationData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAnimationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraAnomalySetupData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAnomalySetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraShakeCapturedData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeCapturedData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraShakeData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraShakeMechanicalData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeMechanicalData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraSpawnSetupData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraSpawnSetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDManagerData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManagerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPresetTextureData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPresetTextureData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScreenFilterData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScreenFilterData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScreenFilters::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScreenFilters *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWorldUISettings::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWorldUISettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDManagerData::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManagerData *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBootLogoData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBootLogoData *
        ]
    ),
    "cGcCameraAmbientBuildingData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAmbientBuildingData *
        ]
    ),
    "cGcCameraAmbientSpecialData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAmbientSpecialData *
        ]
    ),
    "cGcCameraAnimationData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAnimationData *
        ]
    ),
    "cGcCameraAnomalySetupData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAnomalySetupData *
        ]
    ),
    "cGcCameraShakeData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeData *
        ]
    ),
    "cGcCameraShakeMechanicalData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeMechanicalData *
        ]
    ),
    "cGcCameraSpawnSetupData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraSpawnSetupData *
        ]
    ),
    "cGcHUDManagerData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManagerData *
        ]
    ),
    "cGcPresetTextureData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPresetTextureData *
        ]
    ),
    "cGcScreenFilterData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScreenFilterData *
        ]
    ),
    "cGcScreenFilterTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScreenFilterTable *
        ]
    ),
    "cGcWorldUISettings::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWorldUISettings *
        ]
    ),
    "cGcBootLogoData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBootLogoData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraAmbientBuildingData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAmbientBuildingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraAmbientSpaceData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAmbientSpaceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraAmbientSpecialData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAmbientSpecialData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraAnimationData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAnimationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraAnomalySetupData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAnomalySetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraShakeCapturedData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeCapturedData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraShakeData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraShakeMechanicalData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeMechanicalData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraSpawnSetupData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraSpawnSetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHUDManagerData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManagerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPresetTextureData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPresetTextureData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScreenFilterData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScreenFilterData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScreenFilterTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScreenFilterTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcScreenFilters::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScreenFilters *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTexturePrefetchData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTexturePrefetchData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWorldUISettings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWorldUISettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraAmbientBuildingData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAmbientBuildingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCameraAmbientBuildingData *
        ]
    ),
    "cGcCameraAmbientSpecialData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAmbientSpecialData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCameraAmbientSpecialData *
        ]
    ),
    "cGcCameraAnimationData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAnimationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCameraAnimationData *
        ]
    ),
    "cGcCameraAnomalySetupData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAnomalySetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCameraAnomalySetupData *
        ]
    ),
    "cGcCameraShakeCapturedData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeCapturedData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCameraShakeCapturedData *
        ]
    ),
    "cGcCameraShakeData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCameraShakeData *
        ]
    ),
    "cGcCameraShakeMechanicalData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraShakeMechanicalData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCameraShakeMechanicalData *
        ]
    ),
    "cGcHUDManagerData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDManagerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcHUDManagerData *
        ]
    ),
    "cGcScreenFilterData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScreenFilterData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScreenFilterData *
        ]
    ),
    "cGcScreenFilters::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScreenFilters *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScreenFilters *
        ]
    ),
    "cGcWorldUISettings::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWorldUISettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWorldUISettings *
        ]
    ),
    "RegistercGcWaterGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcWaterGlobals *
        ]
    ),
    "RegistercGcWaterGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcWaterGlobals *
        ]
    ),
    "RegistercGcWaterGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcWaterGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcWaterGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaterGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPaletteData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPaletteData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCameraAerialViewData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraAerialViewData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCameraFollowSettings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraFollowSettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCameraWarpSettings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraWarpSettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcColourModifier const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcColourModifier *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcColourPaletteData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcColourPaletteData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPaletteData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPaletteData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPaletteList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPaletteList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTerrainOverlayColours const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTerrainOverlayColours *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWaterGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaterGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraAerialViewData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCameraFollowSettings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCameraWarpSettings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcColourModifier::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcColourPaletteData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPaletteData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPaletteList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTerrainOverlayColours::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCameraAerialViewData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCameraWarpSettings::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCameraAerialViewData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAerialViewData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCameraWarpSettings::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraWarpSettings *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCameraAerialViewData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAerialViewData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraFollowSettings::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraFollowSettings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraWarpSettings::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraWarpSettings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcColourModifier::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcColourModifier *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcColourPaletteData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcColourPaletteData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPaletteData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPaletteData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainOverlayColours::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainOverlayColours *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWaterGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcCameraAerialViewData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAerialViewData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraFollowSettings::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraFollowSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraWarpSettings::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraWarpSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcColourModifier::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcColourModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcColourPaletteData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcColourPaletteData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPaletteData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPaletteData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainOverlayColours::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainOverlayColours *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWaterGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaterGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWaterGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaterGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcPaletteList::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPaletteList *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraAerialViewData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAerialViewData *
        ]
    ),
    "cGcCameraFollowSettings::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraFollowSettings *
        ]
    ),
    "cGcCameraWarpSettings::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraWarpSettings *
        ]
    ),
    "cGcColourModifier::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcColourModifier *
        ]
    ),
    "cGcColourPaletteData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcColourPaletteData *
        ]
    ),
    "cGcPaletteData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPaletteData *
        ]
    ),
    "cGcPaletteList::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPaletteList *
        ]
    ),
    "cGcTerrainOverlayColours::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainOverlayColours *
        ]
    ),
    "cGcWaterGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaterGlobals *
        ]
    ),
    "cGcCameraAerialViewData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAerialViewData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraFollowSettings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraFollowSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraWarpSettings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraWarpSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcColourModifier::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcColourModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcColourPaletteData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcColourPaletteData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPaletteData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPaletteData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPaletteList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPaletteList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainOverlayColours::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainOverlayColours *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraAerialViewData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAerialViewData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCameraAerialViewData *
        ]
    ),
    "cGcCameraFollowSettings::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraFollowSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCameraFollowSettings *
        ]
    ),
    "cGcCameraWarpSettings::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraWarpSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCameraWarpSettings *
        ]
    ),
    "cGcColourModifier::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcColourModifier *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcColourModifier *
        ]
    ),
    "cGcPaletteData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPaletteData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPaletteData *
        ]
    ),
    "cGcTerrainOverlayColours::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainOverlayColours *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTerrainOverlayColours *
        ]
    ),
    "cGcWaterGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWaterGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWaterGlobals *
        ]
    ),
    "RegistercGcVehicleGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcVehicleGlobals *
        ]
    ),
    "RegistercGcVehicleGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcVehicleGlobals *
        ]
    ),
    "RegistercGcVehicleGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcVehicleGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcVehicleData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcVehicleData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcVehicleWeaponMuzzleData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcVehicleWeaponMuzzleData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcScanData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcVehicleGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcExoMechWeaponData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcExoMechWeaponData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcVehicleData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcVehicleData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcVehicleMuzzleData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcVehicleMuzzleData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcVehicleWeaponMuzzleData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcVehicleWeaponMuzzleData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcScanData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScanData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScanData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcVehicleGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleGlobals::~cGcVehicleGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleGlobals *
        ]
    ),
    "cTkDynamicArray<cGcVehicleData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcVehicleData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcVehicleGlobals::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleGlobals *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcVehicleGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcVehicleGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcVehicleGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleGlobals *
        ]
    ),
    "cGcVehicleGlobals::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcVehicleGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcVehicleGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcVehicleGlobals *
        ]
    ),
    "RegistercGcTerrainGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcTerrainGlobals *
        ]
    ),
    "RegistercGcTerrainGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcTerrainGlobals *
        ]
    ),
    "RegistercGcTerrainGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcTerrainGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "RegistercGcUIGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcUIGlobals *
        ]
    ),
    "RegistercGcUIGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcUIGlobals *
        ]
    ),
    "RegistercGcUIGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcUIGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadArrayXML<cTkVector2>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkVector2> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSeasonalRingArray>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSeasonalRingArray> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcWonderCategoryConfig>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderCategoryConfig *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcHUDEffectRewardData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDEffectRewardData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcHUDStartupTable>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHUDStartupTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcInventorySlotActionData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventorySlotActionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcScanEffectData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcScanEffectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcSpaceMapObjectData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceMapObjectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcTerrainOverlayColours>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainOverlayColours *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkHitCurveData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHitCurveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkNGuiRectanglePulseEffect>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiRectanglePulseEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcTerrainGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcUIGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcUIGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSeasonalRingArray const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSeasonalRingArray *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcWonderCategoryConfig const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcWonderCategoryConfig *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcDiscoveryHelperTimings>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDiscoveryHelperTimings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDiscoveryHelperTimings *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcHUDEffectRewardData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHUDEffectRewardData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcHUDEffectRewardData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcHUDStartupTable>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHUDStartupTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcHUDStartupTable *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcInventorySlotActionData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventorySlotActionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventorySlotActionData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcModelViewCollection>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcModelViewCollection *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcModelViewCollection *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcScanEffectData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcScanEffectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcScanEffectData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcSpaceMapObjectData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpaceMapObjectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSpaceMapObjectData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcTerrainOverlayColours>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTerrainOverlayColours *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTerrainOverlayColours *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcWorldUISettings>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWorldUISettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcWorldUISettings *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkHitCurveData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkHitCurveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkHitCurveData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkNGuiRectanglePulseEffect>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkNGuiRectanglePulseEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkNGuiRectanglePulseEffect *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTerrainGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUIGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUIGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainGlobals::~cGcTerrainGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainGlobals *
        ]
    ),
    "cGcUIGlobals::~cGcUIGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUIGlobals *
        ]
    ),
    "cGcUIGlobals::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUIGlobals *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkMetaDataXML::IsDefault": {
        "const float *, const float *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # const float *
                ctypes.c_ulonglong,  # const float *
            ]
        ),
        "const cTkVector2 *, const cTkVector2 *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector2 *
            ]
        ),
        "const TkID<256> *, const TkID<256> *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_char_p,  # const TkID<256> *
                ctypes.c_char_p,  # const TkID<256> *
            ]
        ),
    },
    "cGcTerrainGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcUIGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcTerrainGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUIGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUIGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcUIGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUIGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcTerrainGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainGlobals *
        ]
    ),
    "cGcUIGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUIGlobals *
        ]
    ),
    "cGcTerrainGlobals::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUIGlobals::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUIGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTerrainGlobals *
        ]
    ),
    "cGcUIGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUIGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcUIGlobals *
        ]
    ),
    "RegistercGcSpaceshipGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSpaceshipGlobals *
        ]
    ),
    "RegistercGcSpaceshipGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSpaceshipGlobals *
        ]
    ),
    "RegistercGcSpaceshipGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSpaceshipGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerSpaceshipAim>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerSpaceshipAim> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcPlayerSpaceshipClassBonuses>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipClassBonuses *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcPlayerSpaceshipControlData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpaceshipControlData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcSpaceshipGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerSpaceshipAim const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipAim *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlayerSpaceshipAim>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipAim *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipAim *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlayerSpaceshipClassBonuses>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipClassBonuses *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipClassBonuses *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlayerSpaceshipControlData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipControlData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerSpaceshipControlData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpaceshipGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipGlobals::~cGcSpaceshipGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipGlobals *
        ]
    ),
    "cTkDynamicArray<cGcPlayerSpaceshipAim>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerSpaceshipAim> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSpaceshipGlobals::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipGlobals *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSpaceshipGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcSpaceshipGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcSpaceshipGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipGlobals *
        ]
    ),
    "cGcSpaceshipGlobals::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpaceshipGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSpaceshipGlobals *
        ]
    ),
    "RegistercGcSolarGenerationGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSolarGenerationGlobals *
        ]
    ),
    "RegistercGcSolarGenerationGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSolarGenerationGlobals *
        ]
    ),
    "RegistercGcSolarGenerationGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSolarGenerationGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "RegistercGcSkyGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSkyGlobals *
        ]
    ),
    "RegistercGcSkyGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSkyGlobals *
        ]
    ),
    "RegistercGcSkyGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSkyGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "RegistercGcSmokeTestOptions::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSmokeTestOptions *
        ]
    ),
    "RegistercGcSmokeTestOptions::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSmokeTestOptions *
        ]
    ),
    "RegistercGcSmokeTestOptions::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSmokeTestOptions *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcAISpaceshipWeightingData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAISpaceshipWeightingData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcAsteroidSystemGenerationData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAsteroidSystemGenerationData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSpaceSkyColours>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSpaceSkyColours> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcPhotoModeAdjustData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPhotoModeAdjustData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcPlanetCloudProperties>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetCloudProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcPlanetRingData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetRingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcPlanetSkyProperties>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlanetSkyProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcSpaceSkyProperties>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceSkyProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcSkyGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSkyGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcSmokeTestOptions>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSmokeTestOptions *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcSolarGenerationGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarGenerationGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAISpaceshipWeightingData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAISpaceshipWeightingData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAsteroidSystemGenerationData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAsteroidSystemGenerationData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSpaceSkyColours const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSpaceSkyColours *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPhotoModeAdjustData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPhotoModeAdjustData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPhotoModeAdjustData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlanetCloudProperties>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetCloudProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetCloudProperties *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlanetRingData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetRingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetRingData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlanetSkyProperties>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlanetSkyProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlanetSkyProperties *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcSpaceSkyProperties>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpaceSkyProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSpaceSkyProperties *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSkyGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSkyGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSmokeTestOptions>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSmokeTestOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSolarGenerationGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarGenerationGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarGenerationGlobals::~cGcSolarGenerationGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarGenerationGlobals *
        ]
    ),
    "cTkDynamicArray<cGcAISpaceshipWeightingData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAISpaceshipWeightingData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcAsteroidSystemGenerationData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAsteroidSystemGenerationData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSpaceSkyColours>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSpaceSkyColours> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSkyGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcSmokeTestOptions::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcSolarGenerationGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcSkyGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSkyGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSmokeTestOptions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSmokeTestOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSolarGenerationGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarGenerationGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSkyGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSkyGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcSmokeTestOptions::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSmokeTestOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcSolarGenerationGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarGenerationGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcSkyGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSkyGlobals *
        ]
    ),
    "cGcSolarGenerationGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarGenerationGlobals *
        ]
    ),
    "cGcSkyGlobals::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSkyGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSkyGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSkyGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSkyGlobals *
        ]
    ),
    "cGcSmokeTestOptions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSmokeTestOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSmokeTestOptions *
        ]
    ),
    "cGcSolarGenerationGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSolarGenerationGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSolarGenerationGlobals *
        ]
    ),
    "RegistercGcSceneOptions::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSceneOptions *
        ]
    ),
    "RegistercGcSceneOptions::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSceneOptions *
        ]
    ),
    "RegistercGcSceneOptions::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSceneOptions *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "RegistercGcScratchpadGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcScratchpadGlobals *
        ]
    ),
    "RegistercGcScratchpadGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcScratchpadGlobals *
        ]
    ),
    "RegistercGcScratchpadGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcScratchpadGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "RegistercGcSettlementGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSettlementGlobals *
        ]
    ),
    "RegistercGcSettlementGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSettlementGlobals *
        ]
    ),
    "RegistercGcSettlementGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSettlementGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "RegistercGcSimulationGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSimulationGlobals *
        ]
    ),
    "RegistercGcSimulationGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSimulationGlobals *
        ]
    ),
    "RegistercGcSimulationGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcSimulationGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcMultitoolPoolData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMultitoolPoolData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPresetTextureData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPresetTextureData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSettlementCustomJudgement>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementCustomJudgement> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSettlementGiftDetails>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementGiftDetails> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSettlementJobDetails>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementJobDetails> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSettlementJudgementData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementJudgementData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcSceneOptions>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneOptions *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcScratchpadGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcScratchpadGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcSettlementGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcSimulationGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulationGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMultitoolPoolData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMultitoolPoolData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPresetTextureData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPresetTextureData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSettlementBuildingCost const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSettlementBuildingCost *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSettlementCustomJudgement const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSettlementCustomJudgement *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSettlementGiftDetails const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSettlementGiftDetails *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSettlementJobDetails const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSettlementJobDetails *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSettlementJudgementData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSettlementJudgementData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSettlementStatChangeArray const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSettlementStatChangeArray *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSettlementStatStrengthData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSettlementStatStrengthData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSceneOptions>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcScratchpadGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScratchpadGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSimulationGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulationGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcSettlementStatStrengthData,7>::cTkFixedArray<cGcSettlementStatStrengthData,7>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcSettlementStatStrengthData,7> *
        ]
    ),
    "cGcSettlementJudgementData::cGcSettlementJudgementData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementData *
            ctypes.c_ulonglong,  # cGcSettlementJudgementData *
        ]
    ),
    "cGcSettlementJudgementOption::cGcSettlementJudgementOption": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementOption *
            ctypes.c_ulonglong,  # cGcSettlementJudgementOption *
        ]
    ),
    "cGcSettlementGlobals::~cGcSettlementGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementGlobals *
        ]
    ),
    "cGcSimulationGlobals::~cGcSimulationGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulationGlobals *
        ]
    ),
    "cTkDynamicArray<cGcMultitoolPoolData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMultitoolPoolData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPresetTextureData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPresetTextureData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSettlementCustomJudgement>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementCustomJudgement> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSettlementJobDetails>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementJobDetails> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSettlementJudgementData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementJudgementData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSettlementCustomJudgement>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementCustomJudgement> *
        ]
    ),
    "cTkDynamicArray<cGcSettlementJudgementData>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSettlementJudgementData> *
        ]
    ),
    "cGcSettlementGlobals::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementGlobals *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSceneOptions::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcScratchpadGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcSettlementGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcSimulationGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcSceneOptions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSimulationGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulationGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSceneOptions::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcScratchpadGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcScratchpadGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcSettlementGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcSimulationGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulationGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcSceneOptions::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneOptions *
        ]
    ),
    "cGcSettlementGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementGlobals *
        ]
    ),
    "cGcSimulationGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulationGlobals *
        ]
    ),
    "cGcSettlementGlobals::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSimulationGlobals::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulationGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSceneOptions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSceneOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSceneOptions *
        ]
    ),
    "cGcSettlementGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementGlobals *
        ]
    ),
    "cGcSimulationGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimulationGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSimulationGlobals *
        ]
    ),
    "RegistercGcRichPresenceGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcRichPresenceGlobals *
        ]
    ),
    "RegistercGcRichPresenceGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcRichPresenceGlobals *
        ]
    ),
    "RegistercGcRichPresenceGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcRichPresenceGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "RegistercGcRobotGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcRobotGlobals *
        ]
    ),
    "RegistercGcRobotGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcRobotGlobals *
        ]
    ),
    "RegistercGcRobotGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcRobotGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcDroneWeaponData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDroneWeaponData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcRobotLaserData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotLaserData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcRichPresenceGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRichPresenceGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcRobotGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcDroneWeaponData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcDroneWeaponData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSentinelMechWeaponData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSentinelMechWeaponData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSentinelResource const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSentinelResource *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcRobotLaserData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRobotLaserData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRobotLaserData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRichPresenceGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRichPresenceGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRobotGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRobotGlobals::~cGcRobotGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotGlobals *
        ]
    ),
    "cTkDynamicArray<cGcDroneWeaponData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDroneWeaponData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcRichPresenceGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcRobotGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcRichPresenceGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRichPresenceGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRobotGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRichPresenceGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRichPresenceGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcRobotGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcRobotGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotGlobals *
        ]
    ),
    "cGcRobotGlobals::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRichPresenceGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRichPresenceGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRichPresenceGlobals *
        ]
    ),
    "cGcRobotGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRobotGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRobotGlobals *
        ]
    ),
    "RegistercGcMultiplayerGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcMultiplayerGlobals *
        ]
    ),
    "RegistercGcMultiplayerGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcMultiplayerGlobals *
        ]
    ),
    "RegistercGcMultiplayerGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcMultiplayerGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "RegistercGcPlacementGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcPlacementGlobals *
        ]
    ),
    "RegistercGcPlacementGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcPlacementGlobals *
        ]
    ),
    "RegistercGcPlacementGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcPlacementGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "RegistercGcPlayerGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcPlayerGlobals *
        ]
    ),
    "RegistercGcPlayerGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcPlayerGlobals *
        ]
    ),
    "RegistercGcPlayerGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcPlayerGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCameraSpawnSetupData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCameraSpawnSetupData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcShipDialogueTreeEnum>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipDialogueTreeEnum *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCameraAnomalySetupData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAnomalySetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcInventoryValueData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryValueData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcPlayerStickData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStickData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcProjectileLineData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectileLineData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcProjectorOffsetData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProjectorOffsetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcResourceCollectEffect>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcResourceCollectEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcMultiplayerGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultiplayerGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcPlacementGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlacementGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcPlayerGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCameraSpawnSetupData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCameraSpawnSetupData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcShipDialogueTreeEnum const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcShipDialogueTreeEnum *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCameraAnomalySetupData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraAnomalySetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCameraAnomalySetupData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcInventoryValueData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryValueData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryValueData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlayerStickData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerStickData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerStickData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcProjectileLineData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProjectileLineData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProjectileLineData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcProjectorOffsetData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProjectorOffsetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcProjectorOffsetData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcResourceCollectEffect>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcResourceCollectEffect *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcResourceCollectEffect *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMultiplayerGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultiplayerGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlacementGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlacementGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMultiplayerGlobals::~cGcMultiplayerGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultiplayerGlobals *
        ]
    ),
    "cGcPlayerGlobals::~cGcPlayerGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerGlobals *
        ]
    ),
    "cTkDynamicArray<cGcCameraSpawnSetupData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCameraSpawnSetupData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerGlobals::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerGlobals *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMultiplayerGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcPlacementGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcPlayerGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcMultiplayerGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultiplayerGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlacementGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlacementGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMultiplayerGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultiplayerGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcPlacementGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlacementGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcPlayerGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcMultiplayerGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultiplayerGlobals *
        ]
    ),
    "cGcPlacementGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlacementGlobals *
        ]
    ),
    "cGcPlayerGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerGlobals *
        ]
    ),
    "cGcPlayerGlobals::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMultiplayerGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultiplayerGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMultiplayerGlobals *
        ]
    ),
    "cGcPlacementGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlacementGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlacementGlobals *
        ]
    ),
    "cGcPlayerGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerGlobals *
        ]
    ),
    "RegistercGcGraphicsGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcGraphicsGlobals *
        ]
    ),
    "RegistercGcGraphicsGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcGraphicsGlobals *
        ]
    ),
    "RegistercGcGraphicsGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcGraphicsGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cTkGraphicsDetailPreset>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsDetailPreset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcGraphicsGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGraphicsGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkGraphicsDetailPreset const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkGraphicsDetailPreset *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cTkGraphicsDetailPreset>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkGraphicsDetailPreset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkGraphicsDetailPreset *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGraphicsGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGraphicsGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cTkGraphicsDetailPreset,4>::cTkFixedArray<cTkGraphicsDetailPreset,4>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cTkGraphicsDetailPreset,4> *
        ]
    ),
    "cGcGraphicsGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcGraphicsGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGraphicsGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGraphicsGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGraphicsGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcGraphicsGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGraphicsGlobals *
        ]
    ),
    "cGcGraphicsGlobals::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGraphicsGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGraphicsGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGraphicsGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGraphicsGlobals *
        ]
    ),
    "RegistercGcGameplayGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcGameplayGlobals *
        ]
    ),
    "RegistercGcGameplayGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcGameplayGlobals *
        ]
    ),
    "RegistercGcGameplayGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcGameplayGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcZoomData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcZoomData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcExperienceTimers>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExperienceTimers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcTracerData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTracerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcGameplayGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameplayGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcZoomData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcZoomData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcExperienceTimers>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExperienceTimers *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcExperienceTimers *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcTracerData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTracerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTracerData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGameplayGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameplayGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcDifficultySettingUIOption>::~cTkDynamicArray<cGcDifficultySettingUIOption>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcDifficultySettingUIOption> *
        ]
    ),
    "cGcDifficultyConfig::~cGcDifficultyConfig": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDifficultyConfig *
        ]
    ),
    "cGcGameplayGlobals::~cGcGameplayGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameplayGlobals *
        ]
    ),
    "cTkDynamicArray<cGcZoomData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcZoomData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcGameplayGlobals::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameplayGlobals *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGameplayGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcGameplayGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameplayGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGameplayGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameplayGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcGameplayGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameplayGlobals *
        ]
    ),
    "cGcGameplayGlobals::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameplayGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGameplayGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameplayGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGameplayGlobals *
        ]
    ),
    "RegistercGcFreighterBaseGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcFreighterBaseGlobals *
        ]
    ),
    "RegistercGcFreighterBaseGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcFreighterBaseGlobals *
        ]
    ),
    "RegistercGcFreighterBaseGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcFreighterBaseGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "RegistercGcGalaxyGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcGalaxyGlobals *
        ]
    ),
    "RegistercGcGalaxyGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcGalaxyGlobals *
        ]
    ),
    "RegistercGcGalaxyGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcGalaxyGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcFreighterRoomNPCData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFreighterRoomNPCData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcGalaxyGenerationSetupData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyGenerationSetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcGalaxyRenderAnostreakData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyRenderAnostreakData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcGalaxyRenderSetupData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyRenderSetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcGalaxyStarColours>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyStarColours *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcFreighterBaseGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcGalaxyGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFreighterNPCSpawnPriority const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFreighterNPCSpawnPriority *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFreighterRoomNPCData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFreighterRoomNPCData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcGalaxyMarkerSettings const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcGalaxyMarkerSettings *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcGalaxyGenerationSetupData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyGenerationSetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyGenerationSetupData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcGalaxyRenderAnostreakData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyRenderAnostreakData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyRenderAnostreakData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcGalaxyRenderSetupData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyRenderSetupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyRenderSetupData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcGalaxyStarColours>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalaxyStarColours *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyStarColours *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFreighterBaseGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalaxyGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterBaseGlobals::~cGcFreighterBaseGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseGlobals *
        ]
    ),
    "cTkDynamicArray<cGcFreighterRoomNPCData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFreighterRoomNPCData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFreighterBaseGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcGalaxyGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFreighterBaseGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalaxyGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterBaseGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcGalaxyGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcFreighterBaseGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseGlobals *
        ]
    ),
    "cGcGalaxyGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyGlobals *
        ]
    ),
    "cGcGalaxyGlobals::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterBaseGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterBaseGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFreighterBaseGlobals *
        ]
    ),
    "cGcGalaxyGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalaxyGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalaxyGlobals *
        ]
    ),
    "RegistercGcEffectsGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcEffectsGlobals *
        ]
    ),
    "RegistercGcEffectsGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcEffectsGlobals *
        ]
    ),
    "RegistercGcEffectsGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcEffectsGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "RegistercGcEnvironmentGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcEnvironmentGlobals *
        ]
    ),
    "RegistercGcEnvironmentGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcEnvironmentGlobals *
        ]
    ),
    "RegistercGcEnvironmentGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcEnvironmentGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "RegistercGcFleetGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcFleetGlobals *
        ]
    ),
    "RegistercGcFleetGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcFleetGlobals *
        ]
    ),
    "RegistercGcFleetGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcFleetGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcExpeditionDebriefPunctuation>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExpeditionDebriefPunctuation> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcExpeditionDifficultyKeyframe>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExpeditionDifficultyKeyframe> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcExpeditionPaymentToken>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExpeditionPaymentToken> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcExpeditionPowerup>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExpeditionPowerup> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcFrigateUITraitLines>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFrigateUITraitLines> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkShearWindData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkShearWindData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcEnvironmentProperties>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcFrigateClassCost>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateClassCost *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcFrigateTraitIcons>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFrigateTraitIcons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcEffectsGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcEffectsGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcEnvironmentGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcFleetGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcExpeditionDebriefPunctuation const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcExpeditionDebriefPunctuation *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcExpeditionDifficultyKeyframe const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcExpeditionDifficultyKeyframe *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcExpeditionPaymentToken const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcExpeditionPaymentToken *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcExpeditionPowerup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcExpeditionPowerup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFrigateUITraitLines const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFrigateUITraitLines *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkLODSettingsData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkLODSettingsData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkShearWindData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkShearWindData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcEnvironmentProperties>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcEnvironmentProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcEnvironmentProperties *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcFrigateClassCost>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateClassCost *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFrigateClassCost *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcFrigateTraitIcons>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFrigateTraitIcons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFrigateTraitIcons *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcEffectsGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEffectsGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcEnvironmentGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFleetGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEnvironmentGlobals::~cGcEnvironmentGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentGlobals *
        ]
    ),
    "cGcFleetGlobals::~cGcFleetGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetGlobals *
        ]
    ),
    "cTkDynamicArray<cGcExpeditionPowerup>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExpeditionPowerup> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcFrigateUITraitLines>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFrigateUITraitLines> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkShearWindData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkShearWindData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFleetGlobals::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetGlobals *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcEffectsGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcEnvironmentGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcFleetGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcEffectsGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEffectsGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEnvironmentGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFleetGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEffectsGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEffectsGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcEnvironmentGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcFleetGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcEnvironmentGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentGlobals *
        ]
    ),
    "cGcFleetGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetGlobals *
        ]
    ),
    "cGcEnvironmentGlobals::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFleetGlobals::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEffectsGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEffectsGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcEffectsGlobals *
        ]
    ),
    "cGcEnvironmentGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEnvironmentGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcEnvironmentGlobals *
        ]
    ),
    "cGcFleetGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFleetGlobals *
        ]
    ),
    "RegistercGcDebugOptions::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcDebugOptions *
        ]
    ),
    "RegistercGcDebugOptions::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcDebugOptions *
        ]
    ),
    "RegistercGcDebugOptions::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcDebugOptions *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcDebugOptions>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugOptions *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDebugOptions>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugOptions::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcDebugOptions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugOptions::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcDebugOptions::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugOptions *
        ]
    ),
    "cGcDebugOptions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDebugOptions::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDebugOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDebugOptions *
        ]
    ),
    "RegistercGcCreatureGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcCreatureGlobals *
        ]
    ),
    "RegistercGcCreatureGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcCreatureGlobals *
        ]
    ),
    "RegistercGcCreatureGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcCreatureGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "RegistercGcCharacterGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcCharacterGlobals *
        ]
    ),
    "RegistercGcCharacterGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcCharacterGlobals *
        ]
    ),
    "RegistercGcCharacterGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcCharacterGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureFoodList>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureFoodList> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureHarvestSubstanceList>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureHarvestSubstanceList> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCreatureSubstanceList>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureSubstanceList> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcCharacterGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcCreatureGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureFoodList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureFoodList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureHarvestSubstanceList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureHarvestSubstanceList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCreatureSubstanceList const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCreatureSubstanceList *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCharacterGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCreatureGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGlobals::~cGcCreatureGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGlobals *
        ]
    ),
    "cTkDynamicArray<cGcCreatureFoodList>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureFoodList> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCreatureHarvestSubstanceList>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCreatureHarvestSubstanceList> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCreatureGlobals::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGlobals *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCharacterGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcCreatureGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcCharacterGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCreatureGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcCreatureGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcCharacterGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterGlobals *
        ]
    ),
    "cGcCreatureGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGlobals *
        ]
    ),
    "cGcCreatureGlobals::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCharacterGlobals *
        ]
    ),
    "cGcCreatureGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCreatureGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCreatureGlobals *
        ]
    ),
    "RegistercGcCameraGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcCameraGlobals *
        ]
    ),
    "RegistercGcCameraGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcCameraGlobals *
        ]
    ),
    "RegistercGcCameraGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcCameraGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCameraFollowSettings>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCameraFollowSettings> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCameraShakeData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCameraShakeData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkPhysRelVectorData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkPhysRelVectorData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCameraAerialViewData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraAerialViewData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCameraFollowSettings>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraFollowSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcCameraWarpSettings>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraWarpSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcCameraGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCameraShakeData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCameraShakeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkPhysRelVectorData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkPhysRelVectorData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCameraAerialViewData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraAerialViewData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCameraAerialViewData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCameraFollowSettings>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraFollowSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCameraFollowSettings *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCameraWarpSettings>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCameraWarpSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCameraWarpSettings *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCameraGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraGlobals::~cGcCameraGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraGlobals *
        ]
    ),
    "cTkDynamicArray<cGcCameraFollowSettings>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCameraFollowSettings> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCameraShakeData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCameraShakeData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cTkPhysRelVectorData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkPhysRelVectorData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCameraGlobals::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraGlobals *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCameraGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcCameraGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcCameraGlobals::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraGlobals *
        ]
    ),
    "cGcCameraGlobals::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCameraGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCameraGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCameraGlobals *
        ]
    ),
    "RegistercGcBuildingGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcBuildingGlobals *
        ]
    ),
    "RegistercGcBuildingGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcBuildingGlobals *
        ]
    ),
    "RegistercGcBuildingGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcBuildingGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "RegistercGcAtlasGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcAtlasGlobals *
        ]
    ),
    "RegistercGcAtlasGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcAtlasGlobals *
        ]
    ),
    "RegistercGcAtlasGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcAtlasGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "RegistercGcAudioGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcAudioGlobals *
        ]
    ),
    "RegistercGcAudioGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcAudioGlobals *
        ]
    ),
    "RegistercGcAudioGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcAudioGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cTkLODDistances>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODDistances *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcAtlasGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcAudioGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcBuildingGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBuildMenuIconSet const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBuildMenuIconSet *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cTkLODDistances const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkLODDistances *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAtlasGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAudioGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcBuildMenuIconSet,16>::cTkFixedArray<cGcBuildMenuIconSet,16>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcBuildMenuIconSet,16> *
        ]
    ),
    "cTkFixedArray<cGcBuildMenuIconSet,16>::~cTkFixedArray<cGcBuildMenuIconSet,16>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcBuildMenuIconSet,16> *
        ]
    ),
    "cGcBuildingGlobals::~cGcBuildingGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingGlobals *
        ]
    ),
    "cGcAtlasGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcAudioGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcBuildingGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcAudioGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcAudioGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcBuildingGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcAudioGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioGlobals *
        ]
    ),
    "cGcBuildingGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingGlobals *
        ]
    ),
    "cGcAtlasGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAtlasGlobals *
        ]
    ),
    "cGcAudioGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAudioGlobals *
        ]
    ),
    "cGcBuildingGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBuildingGlobals *
        ]
    ),
    "RegistercGcAISpaceshipGlobals::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcAISpaceshipGlobals *
        ]
    ),
    "RegistercGcAISpaceshipGlobals::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcAISpaceshipGlobals *
        ]
    ),
    "RegistercGcAISpaceshipGlobals::LoadModFromFolder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RegistercGcAISpaceshipGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcItemCostData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcItemCostData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcSpaceshipTravelData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpaceshipTravelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadGlobalFromFile<cGcAISpaceshipGlobals>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipGlobals *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcItemCostData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcItemCostData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcSpaceshipTravelData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpaceshipTravelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSpaceshipTravelData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcItemCostTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcItemCostTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTelemetryStat const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTelemetryStat *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAISpaceshipGlobals>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipGlobals::~cGcAISpaceshipGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipGlobals *
        ]
    ),
    "cGcPlayerSquadronConfig::~cGcPlayerSquadronConfig": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSquadronConfig *
        ]
    ),
    "cTkDynamicArray<cGcItemCostData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcItemCostData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcItemCostTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTelemetryStat::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcItemCostTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcItemCostTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemCostTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTelemetryStat::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTelemetryStat *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipGlobals::NGuiRenderCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cGcTelemetryStat::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTelemetryStat *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcAISpaceshipGlobals::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTelemetryStat::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTelemetryStat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipGlobals::ReadModFromFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipGlobals *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cGcAISpaceshipGlobals::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipGlobals::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipGlobals *
        ]
    ),
    "cGcTelemetryStat::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTelemetryStat *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcAISpaceshipGlobals::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcItemCostTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcItemCostTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTelemetryStat::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTelemetryStat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAISpaceshipGlobals::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAISpaceshipGlobals *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAISpaceshipGlobals *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcJourneyMilestoneData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcJourneyMilestoneData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcLeveledStatData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcLeveledStatData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerStat>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerStat> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcStatDefinition>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStatDefinition> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcStatGroupData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStatGroupData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcStatLevelData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatLevelData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcStatValueData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatValueData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcJourneyMilestoneData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcJourneyMilestoneData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcLeveledStatData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcLeveledStatData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerStat const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerStat *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcStatDefinition const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcStatDefinition *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcStatGroupData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcStatGroupData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcStatLevelData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcStatLevelData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcStatDisplayType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatDisplayType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatDisplayType *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcStatTrackType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatTrackType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatTrackType *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcStatType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatType *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcStatValueData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatValueData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatValueData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcJourneyMilestoneData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcJourneyMilestoneData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcJourneyMilestoneTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcJourneyMilestoneTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcLeveledStatData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcLeveledStatData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcLeveledStatTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcLeveledStatTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerStat const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerStat *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerStatsGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerStatsGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerTitle const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerTitle *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatDefinition const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatDefinition *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatDefinitionTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatDefinitionTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatDisplayType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatDisplayType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatGroupData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatGroupData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatGroupTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatGroupTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatLevelData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatLevelData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatModifyType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatModifyType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatTrackType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatTrackType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatValueData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatValueData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatsAchievements const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatsAchievements *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatsOneShotTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatsOneShotTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatsValueTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatsValueTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcStatLevelData,11>::cTkFixedArray<cGcStatLevelData,11>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcStatLevelData,11> *
        ]
    ),
    "cTkDynamicArray<cGcJourneyMilestoneData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcJourneyMilestoneData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcLeveledStatData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcLeveledStatData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcStatDefinition>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStatDefinition> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcJourneyMilestoneData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcJourneyMilestoneTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLeveledStatData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLeveledStatTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerStat::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerStatsGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerTitle::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatDefinition::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatDefinitionTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatDisplayType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatGroupData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatGroupTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatLevelData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatModifyType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatTrackType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatValueData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatsAchievements::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatsOneShotTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatsValueTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcJourneyMilestoneTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcLeveledStatData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcLeveledStatTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerStatsGroup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerTitle::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStatDefinitionTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStatGroupData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStatGroupTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerStatsGroup::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStatsGroup *
        ]
    ),
    "cGcPlayerTitle::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerTitle *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcLeveledStatTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLeveledStatTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcJourneyMilestoneData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyMilestoneData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLeveledStatData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLeveledStatData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerStat::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStat *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerStatsGroup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStatsGroup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerTitle::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerTitle *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatDefinition::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatDefinition *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatDisplayType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatDisplayType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatGroupData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatGroupData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatLevelData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatLevelData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatModifyType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatModifyType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatTrackType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatTrackType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatValueData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatValueData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerStatsGroup::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStatsGroup *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcStatValueData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatValueData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcLeveledStatData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLeveledStatData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerStatsGroup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStatsGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerTitle::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerTitle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatDefinition::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatDefinition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatDisplayType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatDisplayType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatLevelData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatLevelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatModifyType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatModifyType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatTrackType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatTrackType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLeveledStatData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLeveledStatData *
        ]
    ),
    "cGcPlayerTitle::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerTitle *
        ]
    ),
    "cGcPlayerStatsGroup::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStatsGroup *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcStatValueData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatValueData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcJourneyMilestoneData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyMilestoneData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJourneyMilestoneTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyMilestoneTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLeveledStatData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLeveledStatData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLeveledStatTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLeveledStatTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerStat::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerStatsGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStatsGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerTitle::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerTitle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatDefinition::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatDefinition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatDefinitionTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatDefinitionTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatDisplayType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatDisplayType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatGroupData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatGroupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatGroupTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatGroupTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatLevelData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatLevelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatModifyType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatModifyType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatTrackType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatTrackType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatValueData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatValueData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsAchievements::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsAchievements *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsOneShotTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsOneShotTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatsValueTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatsValueTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcJourneyMilestoneData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcJourneyMilestoneData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcJourneyMilestoneData *
        ]
    ),
    "cGcLeveledStatData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLeveledStatData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcLeveledStatData *
        ]
    ),
    "cGcPlayerStat::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStat *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerStat *
        ]
    ),
    "cGcPlayerTitle::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerTitle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerTitle *
        ]
    ),
    "cGcStatDefinition::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatDefinition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatDefinition *
        ]
    ),
    "cGcStatDisplayType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatDisplayType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatDisplayType *
        ]
    ),
    "cGcStatGroupData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatGroupData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatGroupData *
        ]
    ),
    "cGcStatLevelData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatLevelData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatLevelData *
        ]
    ),
    "cGcStatModifyType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatModifyType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatModifyType *
        ]
    ),
    "cGcStatTrackType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatTrackType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatTrackType *
        ]
    ),
    "cGcStatType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatType *
        ]
    ),
    "cGcStatValueData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatValueData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatValueData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcInteractionData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInteractionData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPetVocabularyEntry>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetVocabularyEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPetVocabularyTraitEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPetVocabularyTraitEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcStatusMessageDefinition>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStatusMessageDefinition> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcTerrainEdit>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTerrainEdit> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFriendlyDroneVocabularyEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFriendlyDroneVocabularyEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInteractionData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInteractionData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPetVocabularyEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPetVocabularyEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPetVocabularyTraitEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPetVocabularyTraitEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcStatusMessageDefinition const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcStatusMessageDefinition *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcTerrainEdit const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcTerrainEdit *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcStatusMessageMissionMarkup>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatusMessageMissionMarkup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatusMessageMissionMarkup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFriendlyDroneChatType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFriendlyDroneChatType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFriendlyDroneVocabularyEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFriendlyDroneVocabularyEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInteractionBuffer const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInteractionBuffer *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInteractionBufferType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInteractionBufferType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInteractionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInteractionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPersistentTerrainEdits const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPersistentTerrainEdits *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetChatType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetChatType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetVocabularyEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetVocabularyEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetVocabularyTraitEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetVocabularyTraitEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetVocabularyWords const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetVocabularyWords *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSavedInteractionDialogData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSavedInteractionDialogData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSavedInteractionRaceData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSavedInteractionRaceData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpecialPetChatType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpecialPetChatType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatusMessageDefinition const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatusMessageDefinition *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatusMessageDefinitions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatusMessageDefinitions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcStatusMessageMissionMarkup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcStatusMessageMissionMarkup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTerrainEdit const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTerrainEdit *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTerrainEditsBuffer const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTerrainEditsBuffer *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcPetVocabularyTraitEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPetVocabularyTraitEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcStatusMessageDefinition>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcStatusMessageDefinition> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcFriendlyDroneChatType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFriendlyDroneVocabularyEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInteractionBuffer::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInteractionBufferType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInteractionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPersistentTerrainEdits::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetChatType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetVocabularyEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetVocabularyTraitEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetVocabularyWords::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSavedInteractionDialogData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSavedInteractionRaceData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpecialPetChatType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatusMessageDefinition::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatusMessageDefinitions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcStatusMessageMissionMarkup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTerrainEdit::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTerrainEditsBuffer::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInteractionBuffer::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPersistentTerrainEdits::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetVocabularyEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetVocabularyTraitEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStatusMessageDefinitions::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcStatusMessageDefinitions::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatusMessageDefinitions *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFriendlyDroneVocabularyEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFriendlyDroneVocabularyEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionBuffer::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionBuffer *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionBufferType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionBufferType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistentTerrainEdits::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentTerrainEdits *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetVocabularyEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetVocabularyEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetVocabularyTraitEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetVocabularyTraitEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSavedInteractionDialogData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSavedInteractionDialogData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSavedInteractionRaceData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSavedInteractionRaceData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatusMessageDefinition::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatusMessageDefinition *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatusMessageDefinitions::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatusMessageDefinitions *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatusMessageMissionMarkup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatusMessageMissionMarkup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainEdit::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEdit *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainEditsBuffer::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsBuffer *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionBuffer::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionBuffer *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcInteractionData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSavedInteractionDialogData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSavedInteractionDialogData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSavedInteractionRaceData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSavedInteractionRaceData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcTerrainEdit::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEdit *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcTerrainEditsBuffer::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsBuffer *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcInteractionBuffer::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionBuffer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionBufferType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionBufferType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetVocabularyTraitEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetVocabularyTraitEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSavedInteractionDialogData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSavedInteractionDialogData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSavedInteractionRaceData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSavedInteractionRaceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatusMessageDefinition::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatusMessageDefinition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatusMessageDefinitions::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatusMessageDefinitions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatusMessageMissionMarkup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatusMessageMissionMarkup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainEdit::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEdit *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainEditsBuffer::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsBuffer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatusMessageDefinitions::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatusMessageDefinitions *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionBuffer::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionBuffer *
        ]
    ),
    "cGcInteractionData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionData *
        ]
    ),
    "cGcUAProtectedLocations::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUAProtectedLocations *
        ]
    ),
    "cGcSavedInteractionDialogData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSavedInteractionDialogData *
        ]
    ),
    "cGcSavedInteractionRaceData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSavedInteractionRaceData *
        ]
    ),
    "cGcStatusMessageDefinitions::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatusMessageDefinitions *
        ]
    ),
    "cGcStatusMessageMissionMarkup::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatusMessageMissionMarkup *
        ]
    ),
    "cGcTerrainEdit::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEdit *
        ]
    ),
    "cGcTerrainEditsBuffer::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsBuffer *
        ]
    ),
    "cGcInteractionBuffer::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionBuffer *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcInteractionData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcSavedInteractionDialogData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSavedInteractionDialogData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcSavedInteractionRaceData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSavedInteractionRaceData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcTerrainEditsBuffer::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsBuffer *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcFriendlyDroneChatType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFriendlyDroneChatType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFriendlyDroneVocabularyEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFriendlyDroneVocabularyEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionBuffer::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionBuffer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionBufferType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionBufferType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistentTerrainEdits::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentTerrainEdits *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetChatType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetChatType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetVocabularyEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetVocabularyEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetVocabularyTraitEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetVocabularyTraitEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetVocabularyWords::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetVocabularyWords *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSavedInteractionDialogData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSavedInteractionDialogData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSavedInteractionRaceData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSavedInteractionRaceData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSpecialPetChatType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSpecialPetChatType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatusMessageDefinition::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatusMessageDefinition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatusMessageDefinitions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatusMessageDefinitions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcStatusMessageMissionMarkup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatusMessageMissionMarkup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainEdit::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEdit *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTerrainEditsBuffer::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsBuffer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionBufferType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionBufferType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInteractionBufferType *
        ]
    ),
    "cGcInteractionData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInteractionData *
        ]
    ),
    "cGcPetVocabularyEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetVocabularyEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPetVocabularyEntry *
        ]
    ),
    "cGcPetVocabularyTraitEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetVocabularyTraitEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPetVocabularyTraitEntry *
        ]
    ),
    "cGcStatusMessageDefinition::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatusMessageDefinition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatusMessageDefinition *
        ]
    ),
    "cGcStatusMessageMissionMarkup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcStatusMessageMissionMarkup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcStatusMessageMissionMarkup *
        ]
    ),
    "cGcTerrainEdit::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEdit *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTerrainEdit *
        ]
    ),
    "cGcTerrainEditsBuffer::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTerrainEditsBuffer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadArrayXML<short>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<short> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadArrayXML<unsigned __int64>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<unsigned __int64> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcProtectedLocation>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcProtectedLocation> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSettlementProductionSlotData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementProductionSlotData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSyncBufferSaveData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSyncBufferSaveData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcUAProtectedLocations>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcUAProtectedLocations> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteArrayXML<short>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const __int16 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcProtectedLocation const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcProtectedLocation *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSyncBufferSaveData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSyncBufferSaveData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcUAProtectedLocations const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcUAProtectedLocations *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcSeasonSaveStateOnDeath>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSeasonSaveStateOnDeath *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSeasonSaveStateOnDeath *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcUniverseAddressData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUniverseAddressData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcUniverseAddressData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSeasonPetConstraints const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSeasonPetConstraints *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSeasonSaveStateOnDeath const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSeasonSaveStateOnDeath *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSeasonStateData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSeasonStateData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementProductionSlotData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementProductionSlotData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementState const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementState *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSquadronPilotData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSquadronPilotData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSyncBufferSaveData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSyncBufferSaveData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSyncBufferSaveDataArray const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSyncBufferSaveDataArray *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSynchronisedBufferData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSynchronisedBufferData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSynchronisedBufferType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSynchronisedBufferType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTeleportEndpoint const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTeleportEndpoint *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTeleporterType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTeleporterType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTradingSupplyData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTradingSupplyData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUAProtectedLocations const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUAProtectedLocations *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUniqueIdData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUniqueIdData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUniverseAddressData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUniverseAddressData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWordGroupKnowledge const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWordGroupKnowledge *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWordKnowledge const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWordKnowledge *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonPetConstraints::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSeasonSaveStateOnDeath::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSeasonStateData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementProductionSlotData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementState::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSquadronPilotData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSyncBufferSaveData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSyncBufferSaveDataArray::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSynchronisedBufferData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSynchronisedBufferType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTeleportEndpoint::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTeleporterType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTradingSupplyData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUAProtectedLocations::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUniqueIdData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUniverseAddressData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWordGroupKnowledge::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWordKnowledge::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSeasonPetConstraints::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSeasonStateData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementState::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSquadronPilotData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSyncBufferSaveDataArray::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSynchronisedBufferData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcTeleportEndpoint::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUAProtectedLocations::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUniverseAddressData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSeasonPetConstraints::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonPetConstraints *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSeasonStateData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonStateData *
        ]
    ),
    "cGcSeasonStateData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonStateData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementState::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementState *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSquadronPilotData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronPilotData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSeasonStateData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonStateData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSettlementState::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementState *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSquadronPilotData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronPilotData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSeasonPetConstraints::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonPetConstraints *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonSaveStateOnDeath::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonSaveStateOnDeath *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonStateData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonStateData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementProductionSlotData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementProductionSlotData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementState::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementState *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSquadronPilotData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronPilotData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSyncBufferSaveData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSyncBufferSaveData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSyncBufferSaveDataArray::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSyncBufferSaveDataArray *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTeleportEndpoint::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTeleportEndpoint *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTeleporterType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTeleporterType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradingSupplyData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingSupplyData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUAProtectedLocations::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUAProtectedLocations *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUniqueIdData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniqueIdData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUniverseAddressData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniverseAddressData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWordGroupKnowledge::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWordGroupKnowledge *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWordKnowledge::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWordKnowledge *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonPetConstraints::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonPetConstraints *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSeasonStateData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonStateData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSettlementProductionSlotData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementProductionSlotData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSettlementState::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementState *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSquadronPilotData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronPilotData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSyncBufferSaveData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSyncBufferSaveData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSyncBufferSaveDataArray::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSyncBufferSaveDataArray *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcTeleportEndpoint::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTeleportEndpoint *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcTradingSupplyData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingSupplyData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcUAProtectedLocations::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUAProtectedLocations *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcUniverseAddressData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniverseAddressData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcWordGroupKnowledge::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWordGroupKnowledge *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcWordKnowledge::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcWordKnowledge *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSeasonPetConstraints::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonPetConstraints *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonSaveStateOnDeath::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonSaveStateOnDeath *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonStateData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementState::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSquadronPilotData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronPilotData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSyncBufferSaveData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSyncBufferSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSyncBufferSaveDataArray::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSyncBufferSaveDataArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTeleportEndpoint::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTeleportEndpoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTeleporterType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTeleporterType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradingSupplyData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingSupplyData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUniverseAddressData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniverseAddressData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWordGroupKnowledge::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWordGroupKnowledge *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWordKnowledge::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWordKnowledge *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonPetConstraints::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonPetConstraints *
        ]
    ),
    "cGcSeasonStateData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonStateData *
        ]
    ),
    "cGcSettlementState::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementState *
        ]
    ),
    "cGcSquadronPilotData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronPilotData *
        ]
    ),
    "cGcSyncBufferSaveData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSyncBufferSaveData *
        ]
    ),
    "cGcTeleportEndpoint::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTeleportEndpoint *
        ]
    ),
    "cGcTradingSupplyData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingSupplyData *
        ]
    ),
    "cGcUniqueIdData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniqueIdData *
        ]
    ),
    "cGcUniverseAddressData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniverseAddressData *
        ]
    ),
    "cGcWordGroupKnowledge::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWordGroupKnowledge *
        ]
    ),
    "cGcSeasonPetConstraints::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonPetConstraints *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcSeasonStateData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonStateData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcSettlementState::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementState *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcSquadronPilotData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronPilotData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcSyncBufferSaveDataArray::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSyncBufferSaveDataArray *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcTeleportEndpoint::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTeleportEndpoint *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcTradingSupplyData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingSupplyData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcUniverseAddressData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniverseAddressData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcWordGroupKnowledge::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWordGroupKnowledge *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcWordKnowledge::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWordKnowledge *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcSeasonPetConstraints::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonPetConstraints *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonSaveStateOnDeath::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonSaveStateOnDeath *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonStateData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementProductionSlotData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementProductionSlotData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementState::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSquadronPilotData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSquadronPilotData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSyncBufferSaveData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSyncBufferSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSyncBufferSaveDataArray::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSyncBufferSaveDataArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSynchronisedBufferData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedBufferData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSynchronisedBufferType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSynchronisedBufferType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTeleportEndpoint::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTeleportEndpoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTeleporterType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTeleporterType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTradingSupplyData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingSupplyData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUAProtectedLocations::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUAProtectedLocations *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUniqueIdData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniqueIdData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUniverseAddressData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniverseAddressData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWordGroupKnowledge::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWordGroupKnowledge *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcWordKnowledge::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWordKnowledge *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonPetConstraints::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonPetConstraints *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSeasonPetConstraints *
        ]
    ),
    "cGcSeasonSaveStateOnDeath::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonSaveStateOnDeath *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSeasonSaveStateOnDeath *
        ]
    ),
    "cGcSeasonStateData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSeasonStateData *
        ]
    ),
    "cGcSettlementState::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementState *
        ]
    ),
    "cGcSyncBufferSaveData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSyncBufferSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSyncBufferSaveData *
        ]
    ),
    "cGcTeleportEndpoint::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTeleportEndpoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTeleportEndpoint *
        ]
    ),
    "cGcTeleporterType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTeleporterType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTeleporterType *
        ]
    ),
    "cGcTradingSupplyData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTradingSupplyData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTradingSupplyData *
        ]
    ),
    "cGcUAProtectedLocations::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUAProtectedLocations *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcUAProtectedLocations *
        ]
    ),
    "cGcUniqueIdData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniqueIdData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcUniqueIdData *
        ]
    ),
    "cGcUniverseAddressData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniverseAddressData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcUniverseAddressData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPersistedStatData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPersistedStatData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSandwormTimerAndFrequencyOverride>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSandwormTimerAndFrequencyOverride> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSeasonPetConstraints>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSeasonPetConstraints> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSeasonalMilestone>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSeasonalMilestone> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSeasonalRingData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSeasonalRingData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSeasonalStage>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSeasonalStage> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPersistedStatData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPersistedStatData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSandwormTimerAndFrequencyOverride const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSandwormTimerAndFrequencyOverride *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSeasonPetConstraints const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSeasonPetConstraints *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSeasonalMilestone const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSeasonalMilestone *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSeasonalRingData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSeasonalRingData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSeasonalStage const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSeasonalStage *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcSeasonalRingData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSeasonalRingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSeasonalRingData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPortalSaveData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPortalSaveData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcProtectedLocation const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcProtectedLocation *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRepairTechData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRepairTechData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSavedEntitlement const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSavedEntitlement *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSeasonalGameModeData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSeasonalGameModeData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSeasonalMilestone const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSeasonalMilestone *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSeasonalMilestoneEncryption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSeasonalMilestoneEncryption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSeasonalRingArray const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSeasonalRingArray *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSeasonalRingData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSeasonalRingData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSeasonalStage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSeasonalStage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcSandwormTimerAndFrequencyOverride>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSandwormTimerAndFrequencyOverride> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSeasonPetConstraints>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSeasonPetConstraints> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSeasonalMilestone>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSeasonalMilestone> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSeasonalRingData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSeasonalRingData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcSeasonalStage>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSeasonalStage> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPortalSaveData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcProtectedLocation::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRepairTechData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSavedEntitlement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSeasonalGameModeData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSeasonalMilestone::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSeasonalMilestoneEncryption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSeasonalRingArray::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSeasonalRingData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSeasonalStage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRepairTechData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSeasonalGameModeData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSeasonalMilestone::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSeasonalMilestoneEncryption::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSeasonalRingArray::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSeasonalStage::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRepairTechData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechData *
        ]
    ),
    "cGcSeasonalGameModeData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalGameModeData *
        ]
    ),
    "cGcRepairTechData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSeasonalGameModeData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalGameModeData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSeasonalMilestone::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalMilestone *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSeasonalStage::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalStage *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcRepairTechData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSeasonalGameModeData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalGameModeData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSeasonalStage::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalStage *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPortalSaveData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalSaveData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProtectedLocation::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProtectedLocation *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRepairTechData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSavedEntitlement::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSavedEntitlement *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonalGameModeData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalGameModeData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonalMilestone::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalMilestone *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonalMilestoneEncryption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalMilestoneEncryption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonalRingData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalRingData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonalStage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalStage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPortalSaveData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalSaveData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcProtectedLocation::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcProtectedLocation *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcRepairTechData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSavedEntitlement::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSavedEntitlement *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSeasonalGameModeData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalGameModeData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSeasonalMilestone::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalMilestone *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSeasonalMilestoneEncryption::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalMilestoneEncryption *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSeasonalStage::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalStage *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPortalSaveData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProtectedLocation::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProtectedLocation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRepairTechData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSavedEntitlement::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSavedEntitlement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonalGameModeData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalGameModeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonalMilestone::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalMilestone *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonalMilestoneEncryption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalMilestoneEncryption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonalRingArray::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalRingArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPortalSaveData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalSaveData *
        ]
    ),
    "cGcProtectedLocation::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceSaveKey *
        ]
    ),
    "cGcRepairTechData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechData *
        ]
    ),
    "cGcSeasonalGameModeData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalGameModeData *
        ]
    ),
    "cGcSeasonalMilestone::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalMilestone *
        ]
    ),
    "cGcPortalSaveData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalSaveData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcProtectedLocation::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProtectedLocation *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcRepairTechData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcSavedEntitlement::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSavedEntitlement *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcSeasonalGameModeData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalGameModeData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcSeasonalMilestone::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalMilestone *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcSeasonalMilestoneEncryption::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalMilestoneEncryption *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcSeasonalStage::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalStage *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcPortalSaveData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcProtectedLocation::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcProtectedLocation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRepairTechData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSavedEntitlement::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSavedEntitlement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonalGameModeData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalGameModeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonalMilestone::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalMilestone *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonalMilestoneEncryption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalMilestoneEncryption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonalRingArray::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalRingArray *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonalRingData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalRingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSeasonalStage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalStage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPortalSaveData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPortalSaveData *
        ]
    ),
    "cGcRepairTechData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRepairTechData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRepairTechData *
        ]
    ),
    "cGcSeasonalGameModeData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalGameModeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSeasonalGameModeData *
        ]
    ),
    "cGcSeasonalMilestone::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalMilestone *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSeasonalMilestone *
        ]
    ),
    "cGcSeasonalMilestoneEncryption::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalMilestoneEncryption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSeasonalMilestoneEncryption *
        ]
    ),
    "cGcSeasonalRingData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalRingData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSeasonalRingData *
        ]
    ),
    "cGcSeasonalStage::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSeasonalStage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSeasonalStage *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcFleetExpeditionSaveData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFleetExpeditionSaveData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcFleetFrigateSaveData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFleetFrigateSaveData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcMaintenanceContainer>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMaintenanceContainer> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcMaintenanceSaveKey>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMaintenanceSaveKey> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcMissionIDEpochPair>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcMissionIDEpochPair> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPersistentBBObjectData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPersistentBBObjectData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPersistentBase>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPersistentBase> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPetData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerMissionProgress>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerMissionProgress> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerOwnershipData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerOwnershipData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerStatsGroup>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerStatsGroup> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPortalSaveData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPortalSaveData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcRepairTechData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcRepairTechData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSavedEntitlement>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSavedEntitlement> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSavedInteractionDialogData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSavedInteractionDialogData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcScanEventSave>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcScanEventSave> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcTelemetryStat>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTelemetryStat> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcTeleportEndpoint>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTeleportEndpoint> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcTradingSupplyData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcTradingSupplyData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcUniverseAddressData>": {
        "cTkDynamicArray<cGcUniverseAddressData> *, XMLNode *, const char *, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDynamicArray<cGcUniverseAddressData> *
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cGcUniverseAddressData *, int, XMLNode *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cGcUniverseAddressData *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # XMLNode *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "cTkMetaDataXML::ReadClassArrayXML<cGcWonderRecord>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcWonderRecord *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcWordGroupKnowledge>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWordGroupKnowledge> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcWordKnowledge>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcWordKnowledge> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcInteractionData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcPlayerWeapons>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerWeapons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcUniverseAddressData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniverseAddressData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCharacterCustomisationData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCharacterCustomisationSaveData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationSaveData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFleetExpeditionSaveData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFleetExpeditionSaveData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFleetFrigateSaveData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFleetFrigateSaveData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcHotActionsSaveData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcHotActionsSaveData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInteractionBuffer const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInteractionBuffer *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMaintenanceContainer const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMaintenanceContainer *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMaintenanceSaveKey const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMaintenanceSaveKey *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMissionIDEpochPair const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMissionIDEpochPair *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcMultitoolData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcMultitoolData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcNPCWorkerData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcNPCWorkerData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPersistentBBObjectData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPersistentBBObjectData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPersistentBase const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPersistentBase *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerMissionProgress const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerMissionProgress *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerOwnershipData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerOwnershipData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerStatsGroup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerStatsGroup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPortalSaveData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPortalSaveData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcRepairTechData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcRepairTechData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSavedEntitlement const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSavedEntitlement *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSavedInteractionDialogData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSavedInteractionDialogData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSavedInteractionRaceData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSavedInteractionRaceData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcScanEventSave const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcScanEventSave *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSettlementState const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSettlementState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSquadronPilotData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSquadronPilotData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcStoryPageSeenDataArray const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcStoryPageSeenDataArray *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSyncBufferSaveDataArray const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSyncBufferSaveDataArray *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcTelemetryStat const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcTelemetryStat *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcTeleportEndpoint const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcTeleportEndpoint *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcTradingSupplyData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcTradingSupplyData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcUniverseAddressData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcUniverseAddressData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcWonderRecord const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcWonderRecord *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcWonderRecordCustomData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcWonderRecordCustomData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcWordGroupKnowledge const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcWordGroupKnowledge *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcWordKnowledge const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcWordKnowledge *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcInteractionData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInteractionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInteractionData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlayerSpawnStateData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerSpawnStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerSpawnStateData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPlayerWeapons>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerWeapons *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerWeapons *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcPortalSaveData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPortalSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPortalSaveData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcTeleportEndpoint>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTeleportEndpoint *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTeleportEndpoint *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerSpawnStateData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerSpawnStateData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerStateData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerStateData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFleetExpeditionSaveData::cGcFleetExpeditionSaveData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpeditionSaveData *
            ctypes.c_ulonglong,  # cGcFleetExpeditionSaveData *
        ]
    ),
    "cTkDynamicArray<cGcPortalSaveData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPortalSaveData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcPlayerSpawnStateData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerStateData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerStateData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerStateData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStateData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerStateData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStateData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerStateData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStateData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerSpawnStateData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpawnStateData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerStateData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStateData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpawnStateData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpawnStateData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPlayerStateData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStateData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPlayerSpawnStateData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpawnStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerStateData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpawnStateData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpawnStateData *
        ]
    ),
    "cGcPlayerStateData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStateData *
        ]
    ),
    "cGcPlayerSpawnStateData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpawnStateData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcPlayerStateData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStateData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcPlayerSpawnStateData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpawnStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerStateData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerSpawnStateData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerSpawnStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerSpawnStateData *
        ]
    ),
    "cGcPlayerStateData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerStateData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerStateData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcModSettingsInfo>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcModSettingsInfo> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerMissionProgressMapEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerMissionProgressMapEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerMissionUpgradeMapEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerMissionUpgradeMapEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcModSettingsInfo const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcModSettingsInfo *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerMissionParticipant const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerMissionParticipant *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerMissionProgressMapEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerMissionProgressMapEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerMissionUpgradeMapEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerMissionUpgradeMapEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryStoreBalance const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryStoreBalance *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcKnownThingsPreset const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcKnownThingsPreset *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMaintenanceContainer const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMaintenanceContainer *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMaintenanceSaveKey const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMaintenanceSaveKey *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcModSettings const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcModSettings *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcModSettingsInfo const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcModSettingsInfo *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcMultitoolData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcMultitoolData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPersistedStatData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPersistedStatData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetCustomisationData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetCustomisationData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPetData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPetData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerMissionParticipant const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerMissionParticipant *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerMissionParticipantType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerMissionParticipantType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerMissionProgress const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerMissionProgress *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerMissionProgressMapEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerMissionProgressMapEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerMissionProgressMapTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerMissionProgressMapTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerMissionUpgradeMapEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerMissionUpgradeMapEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerMissionUpgradeMapTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerMissionUpgradeMapTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerOwnershipData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerOwnershipData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcModSettingsInfo>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcModSettingsInfo> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPlayerMissionProgressMapEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerMissionProgressMapEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPlayerMissionUpgradeMapEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerMissionUpgradeMapEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcInventoryStoreBalance::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcKnownThingsPreset::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMaintenanceContainer::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMaintenanceSaveKey::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcModSettings::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcModSettingsInfo::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcMultitoolData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPersistedStatData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetCustomisationData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPetData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerMissionParticipant::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerMissionParticipantType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerMissionProgress::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerMissionProgressMapEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerMissionProgressMapTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerMissionUpgradeMapEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerMissionUpgradeMapTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerOwnershipData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcKnownThingsPreset::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMaintenanceContainer::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcModSettings::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcModSettingsInfo::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMultitoolData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetCustomisationData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerMissionProgressMapTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerMissionUpgradeMapEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerMissionUpgradeMapTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerOwnershipData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcPlayerMissionUpgradeMapEntry>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGeneratedBasePruningRule> *
        ]
    ),
    "cGcMaintenanceContainer::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceContainer *
        ]
    ),
    "cGcPlayerOwnershipData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerOwnershipData *
        ]
    ),
    "cGcKnownThingsPreset::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcKnownThingsPreset *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMaintenanceContainer::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceContainer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMultitoolData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultitoolData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetCustomisationData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetCustomisationData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerOwnershipData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerOwnershipData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcKnownThingsPreset::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcKnownThingsPreset *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMaintenanceContainer::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceContainer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcMultitoolData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultitoolData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetCustomisationData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetCustomisationData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPetData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerMissionUpgradeMapTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionUpgradeMapTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerOwnershipData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerOwnershipData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventoryStoreBalance::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStoreBalance *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcKnownThingsPreset::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcKnownThingsPreset *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceContainer::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceContainer *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceSaveKey::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceSaveKey *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcModSettingsInfo::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcModSettingsInfo *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMultitoolData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultitoolData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistedStatData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistedStatData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetCustomisationData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetCustomisationData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerMissionParticipant::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionParticipant *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerMissionParticipantType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionParticipantType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerMissionProgress::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionProgress *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerMissionProgressMapEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionProgressMapEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerMissionUpgradeMapEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionUpgradeMapEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerOwnershipData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerOwnershipData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceContainer::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceContainer *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcMaintenanceSaveKey::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceSaveKey *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcMultitoolData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultitoolData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPersistedStatData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistedStatData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPetCustomisationData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetCustomisationData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPetData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPlayerMissionParticipant::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionParticipant *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPlayerMissionParticipantType::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionParticipantType *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPlayerMissionProgress::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionProgress *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPlayerOwnershipData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerOwnershipData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcKnownThingsPreset::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcKnownThingsPreset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceContainer::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceContainer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceSaveKey::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceSaveKey *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcModSettingsInfo::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModSettingsInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMultitoolData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultitoolData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistedStatData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistedStatData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetCustomisationData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetCustomisationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerMissionParticipant::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionParticipant *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerMissionParticipantType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionParticipantType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerMissionProgress::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionProgress *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerMissionUpgradeMapEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionUpgradeMapEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerOwnershipData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerOwnershipData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceContainer::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceContainer *
        ]
    ),
    "cGcModSettingsInfo::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModSettingsInfo *
        ]
    ),
    "cGcMultitoolData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultitoolData *
        ]
    ),
    "cGcPetCustomisationData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetCustomisationData *
        ]
    ),
    "cGcPetData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetData *
        ]
    ),
    "cGcPlayerMissionParticipant::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionParticipant *
        ]
    ),
    "cGcPlayerMissionProgress::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionProgress *
        ]
    ),
    "cGcPlayerOwnershipData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerOwnershipData *
        ]
    ),
    "cGcMaintenanceContainer::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceContainer *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcMaintenanceSaveKey::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceSaveKey *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcMultitoolData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultitoolData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcPersistedStatData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistedStatData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcPetCustomisationData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetCustomisationData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcPetData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcPlayerMissionParticipantType::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionParticipantType *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcPlayerMissionProgress::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionProgress *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcPlayerOwnershipData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerOwnershipData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcInventoryStoreBalance::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStoreBalance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcKnownThingsPreset::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcKnownThingsPreset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceContainer::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceContainer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceSaveKey::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceSaveKey *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcModSettings::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModSettings *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcModSettingsInfo::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModSettingsInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMultitoolData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultitoolData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistedStatData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistedStatData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetCustomisationData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetCustomisationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerMissionParticipant::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionParticipant *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerMissionParticipantType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionParticipantType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerMissionProgress::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionProgress *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerMissionProgressMapEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionProgressMapEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerMissionProgressMapTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionProgressMapTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerMissionUpgradeMapEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionUpgradeMapEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerMissionUpgradeMapTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionUpgradeMapTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerOwnershipData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerOwnershipData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryStoreBalance::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryStoreBalance *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryStoreBalance *
        ]
    ),
    "cGcKnownThingsPreset::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcKnownThingsPreset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcMaintenanceContainer::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMaintenanceContainer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMaintenanceContainer *
        ]
    ),
    "cGcModSettingsInfo::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModSettingsInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcModSettingsInfo *
        ]
    ),
    "cGcMultitoolData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcMultitoolData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcMultitoolData *
        ]
    ),
    "cGcPetCustomisationData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetCustomisationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPetData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPetData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPetData *
        ]
    ),
    "cGcPlayerMissionParticipant::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionParticipant *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerMissionParticipant *
        ]
    ),
    "cGcPlayerMissionParticipantType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionParticipantType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerMissionParticipantType *
        ]
    ),
    "cGcPlayerMissionProgress::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionProgress *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerMissionProgress *
        ]
    ),
    "cGcPlayerMissionProgressMapEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionProgressMapEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerMissionProgressMapEntry *
        ]
    ),
    "cGcPlayerMissionUpgradeMapEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerMissionUpgradeMapEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerMissionUpgradeMapEntry *
        ]
    ),
    "cGcPlayerOwnershipData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerOwnershipData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerOwnershipData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCustomisationColourPaletteExtraData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationColourPaletteExtraData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCustomisationDescriptorGroup>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationDescriptorGroup> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCustomisationPreset>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationPreset> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCustomisationTextureOption>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationTextureOption> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcExpeditionEventSaveData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcExpeditionEventSaveData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcInventoryBaseStatEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInventoryBaseStatEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcInventoryElement>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInventoryElement> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcInventoryIndex>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInventoryIndex> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcInventorySpecialSlot>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInventorySpecialSlot> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcInventoryIndex>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryIndex *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationColourPaletteExtraData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationColourPaletteExtraData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationDescriptorGroup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationDescriptorGroup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationPreset const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationPreset *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCustomisationTextureOption const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCustomisationTextureOption *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcExpeditionEventSaveData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcExpeditionEventSaveData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInventoryBaseStatEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInventoryBaseStatEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInventoryElement const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInventoryIndex const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInventorySpecialSlot const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInventorySpecialSlot *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcInventoryIndex>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationColourPaletteExtraData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationColourPaletteExtraData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationColourPalettes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationColourPalettes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationDescriptorGroups const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationDescriptorGroups *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationPreset const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationPreset *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationPresets const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationPresets *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationTextureOptions const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationTextureOptions *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDefaultSaveData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDefaultSaveData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcExpeditionEventSaveData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcExpeditionEventSaveData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFleetExpeditionSaveData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFleetExpeditionSaveData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFleetFrigateSaveData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFleetFrigateSaveData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGalacticAddressData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGalacticAddressData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGameMode const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGameMode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryContainer const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryContainer *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryElement const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryElement *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryIndex const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventoryLayout const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventoryLayout *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInventorySpecialSlot const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInventorySpecialSlot *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcCustomisationPreset>::~cTkDynamicArray<cGcCustomisationPreset>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationPreset> *
        ]
    ),
    "cTkFixedArray<cGcCustomisationColourPaletteExtraData,23>::~cTkFixedArray<cGcCustomisationColourPaletteExtraData,23>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcCustomisationColourPaletteExtraData,23> *
        ]
    ),
    "cGcCustomisationDescriptorGroup::~cGcCustomisationDescriptorGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroup *
        ]
    ),
    "cTkDynamicArray<cGcCustomisationDescriptorGroup>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationDescriptorGroup> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCustomisationPreset>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationPreset> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcCustomisationTextureOption>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationTextureOption> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCustomisationColourPaletteExtraData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationColourPalettes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationDescriptorGroups::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationPreset::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationPresets::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationTextureOptions::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDefaultSaveData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcExpeditionEventSaveData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFleetExpeditionSaveData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFleetFrigateSaveData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGalacticAddressData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGameMode::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryContainer::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryElement::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryIndex::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventoryLayout::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInventorySpecialSlot::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationColourPaletteExtraData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationColourPalettes::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationDescriptorGroups::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationPreset::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationPresets::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationTextureOptions::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDefaultSaveData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcExpeditionEventSaveData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFleetExpeditionSaveData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFleetFrigateSaveData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventoryContainer::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventoryElement::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventorySpecialSlot::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcCustomisationPreset>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCustomisationPreset> *
        ]
    ),
    "cGcFleetExpeditionSaveData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpeditionSaveData *
        ]
    ),
    "cGcFleetFrigateSaveData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigateSaveData *
        ]
    ),
    "cGcInventoryContainer::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryContainer *
        ]
    ),
    "cGcSentinelSpawnSequenceGroupList::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWaveGroup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFleetExpeditionSaveData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpeditionSaveData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFleetFrigateSaveData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigateSaveData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventoryContainer::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryContainer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationColourPalettes::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationColourPalettes *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationDescriptorGroups::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroups *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationPresets::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationPresets *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationTextureOptions::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationTextureOptions *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDefaultSaveData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDefaultSaveData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFleetExpeditionSaveData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpeditionSaveData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFleetFrigateSaveData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigateSaveData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInventoryContainer::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryContainer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationColourPaletteExtraData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationColourPaletteExtraData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationPreset::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationPreset *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionEventSaveData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventSaveData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFleetExpeditionSaveData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpeditionSaveData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFleetFrigateSaveData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigateSaveData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalacticAddressData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalacticAddressData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGameMode::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameMode *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryContainer::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryContainer *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryElement::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryElement *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryIndex::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryIndex *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryLayout::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayout *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventorySpecialSlot::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventorySpecialSlot *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionEventSaveData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventSaveData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcFleetExpeditionSaveData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpeditionSaveData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcFleetFrigateSaveData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigateSaveData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcGalacticAddressData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalacticAddressData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcGameMode::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameMode *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcInventoryContainer::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryContainer *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcInventoryElement::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryElement *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcInventoryIndex::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryIndex *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcInventoryLayout::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayout *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcCustomisationColourPaletteExtraData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationColourPaletteExtraData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationPreset::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationPreset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionEventSaveData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFleetExpeditionSaveData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpeditionSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFleetFrigateSaveData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigateSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalacticAddressData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalacticAddressData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGameMode::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameMode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryContainer::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryContainer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryElement::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryIndex::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryIndex *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryLayout::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventorySpecialSlot::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventorySpecialSlot *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionEventSaveData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventSaveData *
        ]
    ),
    "cGcFleetExpeditionSaveData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpeditionSaveData *
        ]
    ),
    "cGcFleetFrigateSaveData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigateSaveData *
        ]
    ),
    "cGcInventoryContainer::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryContainer *
        ]
    ),
    "cGcInventoryElement::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryElement *
        ]
    ),
    "cGcInventoryLayout::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayout *
        ]
    ),
    "cGcExpeditionEventSaveData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventSaveData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcFleetExpeditionSaveData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpeditionSaveData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcFleetFrigateSaveData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigateSaveData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcGalacticAddressData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalacticAddressData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcGameMode::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameMode *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcInventoryContainer::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryContainer *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcInventoryIndex::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryIndex *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcInventoryLayout::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayout *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcCustomisationColourPaletteExtraData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationColourPaletteExtraData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationColourPalettes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationColourPalettes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationDescriptorGroups::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroups *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationPreset::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationPreset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationPresets::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationPresets *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationTextureOptions::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationTextureOptions *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDefaultSaveData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDefaultSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcExpeditionEventSaveData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFleetExpeditionSaveData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpeditionSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFleetFrigateSaveData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigateSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGalacticAddressData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalacticAddressData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGameMode::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameMode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryContainer::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryContainer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryElement::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryIndex::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryIndex *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventoryLayout::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInventorySpecialSlot::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventorySpecialSlot *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationColourPaletteExtraData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationColourPaletteExtraData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationPreset::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationPreset *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCustomisationPreset *
        ]
    ),
    "cGcDefaultSaveData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDefaultSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDefaultSaveData *
        ]
    ),
    "cGcExpeditionEventSaveData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcExpeditionEventSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcExpeditionEventSaveData *
        ]
    ),
    "cGcFleetExpeditionSaveData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetExpeditionSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFleetExpeditionSaveData *
        ]
    ),
    "cGcFleetFrigateSaveData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFleetFrigateSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFleetFrigateSaveData *
        ]
    ),
    "cGcGalacticAddressData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGalacticAddressData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGalacticAddressData *
        ]
    ),
    "cGcGameMode::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameMode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGameMode *
        ]
    ),
    "cGcInventoryContainer::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryContainer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryContainer *
        ]
    ),
    "cGcInventoryElement::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryElement *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryElement *
        ]
    ),
    "cGcInventoryIndex::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryIndex *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryIndex *
        ]
    ),
    "cGcInventoryLayout::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventoryLayout *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventoryLayout *
        ]
    ),
    "cGcInventorySpecialSlot::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInventorySpecialSlot *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInventorySpecialSlot *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCharacterCustomisationBoneScaleData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCharacterCustomisationBoneScaleData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCharacterCustomisationColourData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCharacterCustomisationColourData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcCharacterCustomisationTextureOptionData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcCharacterCustomisationTextureOptionData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcFreighterRoomNPCSpawnCapacityEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFreighterRoomNPCSpawnCapacityEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCharacterCustomisationBoneScaleData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationBoneScaleData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCharacterCustomisationColourData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationColourData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCharacterCustomisationTextureOptionData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationTextureOptionData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFreighterRoomNPCSpawnCapacityEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFreighterRoomNPCSpawnCapacityEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcCharacterCustomisationBoneScaleData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationBoneScaleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationBoneScaleData *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCharacterCustomisationBoneScaleData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationBoneScaleData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCharacterCustomisationColourData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationColourData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCharacterCustomisationData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCharacterCustomisationSaveData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationSaveData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCharacterCustomisationTextureOptionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationTextureOptionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationDescriptorGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationDescriptorGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCustomisationTextureOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCustomisationTextureOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFreighterNPCSpawnPriority const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFreighterNPCSpawnPriority *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFreighterNPCType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFreighterNPCType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFreighterRoomNPCData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFreighterRoomNPCData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFreighterRoomNPCSpawnCapacities const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFreighterRoomNPCSpawnCapacities *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFreighterRoomNPCSpawnCapacityEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFreighterRoomNPCSpawnCapacityEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCNavSubgraphNodeType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCNavSubgraphNodeType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCNavSubgraphNodeTypeConnectivity const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCNavSubgraphNodeTypeConnectivity *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPersistentBBObjectData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPersistentBBObjectData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPersistentBaseEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPersistentBaseEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPersistentBaseTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPersistentBaseTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSnapPointCondition const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSnapPointCondition *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcFreighterRoomNPCSpawnCapacityEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFreighterRoomNPCSpawnCapacityEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcCharacterCustomisationBoneScaleData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCharacterCustomisationColourData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCharacterCustomisationData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCharacterCustomisationSaveData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCharacterCustomisationTextureOptionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationDescriptorGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCustomisationTextureOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFreighterNPCSpawnPriority::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFreighterNPCType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFreighterRoomNPCData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFreighterRoomNPCSpawnCapacities::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFreighterRoomNPCSpawnCapacityEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCNavSubgraphNodeType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCNavSubgraphNodeTypeConnectivity::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPersistentBBObjectData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPersistentBaseEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPersistentBaseTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSnapPointCondition::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCharacterCustomisationColourData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCharacterCustomisationData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCharacterCustomisationSaveData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationDescriptorGroup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationTextureOption::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFreighterNPCSpawnPriority::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFreighterRoomNPCSpawnCapacities::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSnapPointCondition::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCharacterCustomisationSaveData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationSaveData *
        ]
    ),
    "cGcCharacterCustomisationData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCharacterCustomisationSaveData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationSaveData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationDescriptorGroup::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationTextureOption::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationTextureOption *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCharacterCustomisationData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCharacterCustomisationSaveData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationSaveData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationDescriptorGroup::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCustomisationTextureOption::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationTextureOption *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCharacterCustomisationBoneScaleData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationBoneScaleData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterCustomisationColourData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationColourData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterCustomisationData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterCustomisationSaveData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationSaveData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterCustomisationTextureOptionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationTextureOptionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationDescriptorGroup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationTextureOption::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationTextureOption *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterRoomNPCData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterRoomNPCData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterRoomNPCSpawnCapacityEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterRoomNPCSpawnCapacityEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCNavSubgraphNodeType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavSubgraphNodeType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCNavSubgraphNodeTypeConnectivity::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavSubgraphNodeTypeConnectivity *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistentBBObjectData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBBObjectData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistentBaseEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistentBaseTypes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseTypes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSnapPointCondition::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSnapPointCondition *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterCustomisationData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcCharacterCustomisationSaveData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationSaveData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPersistentBBObjectData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBBObjectData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPersistentBaseEntry::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseEntry *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPersistentBaseTypes::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseTypes *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcCharacterCustomisationColourData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationColourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterCustomisationData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterCustomisationSaveData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationDescriptorGroup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationTextureOption::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationTextureOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterNPCSpawnPriority::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterNPCSpawnPriority *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterRoomNPCData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterRoomNPCData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCNavSubgraphNodeType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavSubgraphNodeType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCNavSubgraphNodeTypeConnectivity::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavSubgraphNodeTypeConnectivity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistentBBObjectData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBBObjectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistentBaseEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistentBaseTypes::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSnapPointCondition::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSnapPointCondition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterCustomisationColourData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationColourData *
        ]
    ),
    "cGcCharacterCustomisationData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationData *
        ]
    ),
    "cGcCharacterCustomisationSaveData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationSaveData *
        ]
    ),
    "cGcCustomisationDescriptorGroup::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroup *
        ]
    ),
    "cGcCustomisationTextureOption::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationTextureOption *
        ]
    ),
    "cGcFreighterRoomNPCData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterRoomNPCData *
        ]
    ),
    "cGcNPCNavSubgraphNodeTypeConnectivity::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavSubgraphNodeTypeConnectivity *
        ]
    ),
    "cGcPersistentBBObjectData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBBObjectData *
        ]
    ),
    "cGcPersistentBaseEntry::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseEntry *
        ]
    ),
    "cGcSnapPointCondition::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSnapPointCondition *
        ]
    ),
    "cGcCharacterCustomisationData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcCharacterCustomisationSaveData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationSaveData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcPersistentBBObjectData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBBObjectData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcPersistentBaseEntry::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseEntry *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcPersistentBaseTypes::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseTypes *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcCharacterCustomisationBoneScaleData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationBoneScaleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterCustomisationColourData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationColourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterCustomisationData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterCustomisationSaveData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterCustomisationTextureOptionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationTextureOptionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationDescriptorGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCustomisationTextureOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationTextureOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterNPCSpawnPriority::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterNPCSpawnPriority *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterNPCType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterNPCType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterRoomNPCData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterRoomNPCData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterRoomNPCSpawnCapacities::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterRoomNPCSpawnCapacities *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterRoomNPCSpawnCapacityEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterRoomNPCSpawnCapacityEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCNavSubgraphNodeType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavSubgraphNodeType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCNavSubgraphNodeTypeConnectivity::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavSubgraphNodeTypeConnectivity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistentBBObjectData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBBObjectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistentBaseEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistentBaseTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSnapPointCondition::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSnapPointCondition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCharacterCustomisationBoneScaleData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationBoneScaleData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationBoneScaleData *
        ]
    ),
    "cGcCharacterCustomisationColourData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationColourData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationColourData *
        ]
    ),
    "cGcCharacterCustomisationData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationData *
        ]
    ),
    "cGcCharacterCustomisationSaveData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationSaveData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationSaveData *
        ]
    ),
    "cGcCharacterCustomisationTextureOptionData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCharacterCustomisationTextureOptionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCharacterCustomisationTextureOptionData *
        ]
    ),
    "cGcCustomisationDescriptorGroup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCustomisationDescriptorGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCustomisationDescriptorGroup *
        ]
    ),
    "cGcFreighterRoomNPCSpawnCapacityEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterRoomNPCSpawnCapacityEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFreighterRoomNPCSpawnCapacityEntry *
        ]
    ),
    "cGcNPCNavSubgraphNodeType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavSubgraphNodeType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCNavSubgraphNodeType *
        ]
    ),
    "cGcNPCNavSubgraphNodeTypeConnectivity::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCNavSubgraphNodeTypeConnectivity *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCNavSubgraphNodeTypeConnectivity *
        ]
    ),
    "cGcPersistentBBObjectData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBBObjectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPersistentBBObjectData *
        ]
    ),
    "cGcPersistentBaseEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPersistentBaseEntry *
        ]
    ),
    "cGcPersistentBaseTypes::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBaseTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPersistentBaseTypes *
        ]
    ),
    "cGcSnapPointCondition::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSnapPointCondition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSnapPointCondition *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcFreighterDungeonParams>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFreighterDungeonParams> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcGeneratedBaseDecorationTemplate>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGeneratedBaseDecorationTemplate> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcGeneratedBasePruningRule>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGeneratedBasePruningRule> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcGeneratedBaseRoomTemplate>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGeneratedBaseRoomTemplate> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcGeneratedBaseThemeTemplate>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGeneratedBaseThemeTemplate> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPersistentBaseEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPersistentBaseEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFreighterDungeonParams const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFreighterDungeonParams *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcGeneratedBaseDecorationTemplate const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcGeneratedBaseDecorationTemplate *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcGeneratedBasePruningRule const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcGeneratedBasePruningRule *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcGeneratedBaseRoomTemplate const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcGeneratedBaseRoomTemplate *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcGeneratedBaseThemeTemplate const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcGeneratedBaseThemeTemplate *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPersistentBaseEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPersistentBaseEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcBaseSnapState>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseSnapState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseSnapState *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseSnapState const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseSnapState *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseStatCondition const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseStatCondition *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBiomeCondition const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBiomeCondition *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildMenuIconSet const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildMenuIconSet *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildMenuOption const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildMenuOption *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingModeCondition const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingModeCondition *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBuildingPlacementErrorTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBuildingPlacementErrorTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFreighterDungeonsTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFreighterDungeonsTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGeneratedBaseDecorationTemplate const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGeneratedBaseDecorationTemplate *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGeneratedBaseLockDoorPair const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGeneratedBaseLockDoorPair *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGeneratedBasePruningRule const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGeneratedBasePruningRule *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGeneratedBaseRoomTemplate const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGeneratedBaseRoomTemplate *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGeneratedBaseStructuralTemplate const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGeneratedBaseStructuralTemplate *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGeneratedBaseTemplatesTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGeneratedBaseTemplatesTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGeneratedBaseThemeTemplate const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGeneratedBaseThemeTemplate *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGroupCondition const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGroupCondition *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcId256List const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcId256List *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcLinkNetworkTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcLinkNetworkTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcModBasePart const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcModBasePart *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCHabitationType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCHabitationType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCWorkerData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCWorkerData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcOutSnapSocketCondition const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcOutSnapSocketCondition *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPersistentBase const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPersistentBase *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratedBaseRoomTemplate::cGcGeneratedBaseRoomTemplate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseRoomTemplate *
            ctypes.c_ulonglong,  # const cGcGeneratedBaseRoomTemplate *
        ]
    ),
    "cGcGeneratedBaseDecorationTemplate::~cGcGeneratedBaseDecorationTemplate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseDecorationTemplate *
        ]
    ),
    "cTkDynamicArray<cGcFreighterDungeonParams>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFreighterDungeonParams> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcGeneratedBaseDecorationTemplate>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGeneratedBaseDecorationTemplate> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcGeneratedBasePruningRule>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGeneratedBasePruningRule> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcGeneratedBaseRoomTemplate>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGeneratedBaseRoomTemplate> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBaseSnapState::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseStatCondition::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBiomeCondition::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildMenuIconSet::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildMenuOption::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingModeCondition::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingPlacementErrorTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFreighterDungeonsTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGeneratedBaseDecorationTemplate::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGeneratedBaseLockDoorPair::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGeneratedBasePruningRule::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGeneratedBaseRoomTemplate::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGeneratedBaseStructuralTemplate::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGeneratedBaseTemplatesTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGeneratedBaseThemeTemplate::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGroupCondition::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcId256List::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLinkNetworkTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcModBasePart::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCHabitationType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCWorkerData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcOutSnapSocketCondition::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPersistentBase::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBiomeCondition::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBuildMenuIconSet::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFreighterDungeonsTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGeneratedBaseDecorationTemplate::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGeneratedBasePruningRule::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGeneratedBaseRoomTemplate::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGeneratedBaseStructuralTemplate::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGeneratedBaseTemplatesTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGeneratedBaseThemeTemplate::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGroupCondition::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcId256List::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcModBasePart::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCWorkerData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPersistentBase::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcFreighterDungeonParams>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFreighterDungeonParams> *
        ]
    ),
    "cGcNPCWorkerData::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWorkerData *
        ]
    ),
    "cGcPersistentBase::DeallocateDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBase *
        ]
    ),
    "cGcGeneratedBaseTemplatesTable::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseTemplatesTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerExperienceSpawnTable::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceSpawnTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCWorkerData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWorkerData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPersistentBase::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFreighterDungeonsTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterDungeonsTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGeneratedBaseTemplatesTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseTemplatesTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcId256List::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceSpawnTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCWorkerData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWorkerData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPersistentBase::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseSnapState::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseSnapState *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratedBaseDecorationTemplate::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseDecorationTemplate *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratedBaseLockDoorPair::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseLockDoorPair *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratedBasePruningRule::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBasePruningRule *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratedBaseRoomTemplate::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseRoomTemplate *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratedBaseThemeTemplate::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseThemeTemplate *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcId256List::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcId256List *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLinkNetworkTypes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLinkNetworkTypes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCHabitationType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCHabitationType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCWorkerData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWorkerData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOutSnapSocketCondition::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutSnapSocketCondition *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistentBase::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBase *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCWorkerData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWorkerData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcPersistentBase::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBase *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcBaseSnapState::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseSnapState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildMenuIconSet::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenuIconSet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratedBaseDecorationTemplate::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseDecorationTemplate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratedBasePruningRule::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBasePruningRule *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratedBaseRoomTemplate::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseRoomTemplate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcId256List::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcId256List *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLinkNetworkTypes::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLinkNetworkTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCHabitationType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCHabitationType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCWorkerData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWorkerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistentBase::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBase *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildMenuIconSet::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenuIconSet *
        ]
    ),
    "cGcModBasePart::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModBasePart *
        ]
    ),
    "cGcNPCWorkerData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWorkerData *
        ]
    ),
    "cGcOutSnapSocketCondition::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutSnapSocketCondition *
        ]
    ),
    "cGcPersistentBase::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBase *
        ]
    ),
    "cGcNPCWorkerData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWorkerData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcPersistentBase::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBase *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcBaseSnapState::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseSnapState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseStatCondition::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseStatCondition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBiomeCondition::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBiomeCondition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildMenuIconSet::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenuIconSet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildMenuOption::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildMenuOption *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingModeCondition::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingModeCondition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBuildingPlacementErrorTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBuildingPlacementErrorTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFreighterDungeonsTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFreighterDungeonsTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratedBaseDecorationTemplate::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseDecorationTemplate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratedBaseLockDoorPair::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseLockDoorPair *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratedBasePruningRule::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBasePruningRule *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratedBaseRoomTemplate::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseRoomTemplate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratedBaseStructuralTemplate::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseStructuralTemplate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratedBaseTemplatesTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseTemplatesTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGeneratedBaseThemeTemplate::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseThemeTemplate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGroupCondition::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroupCondition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcId256List::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcId256List *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLinkNetworkTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLinkNetworkTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcModBasePart::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModBasePart *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCHabitationType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCHabitationType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCWorkerData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWorkerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcOutSnapSocketCondition::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcOutSnapSocketCondition *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistentBase::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBase *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseSnapState::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseSnapState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseSnapState *
        ]
    ),
    "cGcGeneratedBaseDecorationTemplate::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseDecorationTemplate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGeneratedBaseDecorationTemplate *
        ]
    ),
    "cGcGeneratedBasePruningRule::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBasePruningRule *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGeneratedBasePruningRule *
        ]
    ),
    "cGcGeneratedBaseRoomTemplate::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseRoomTemplate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGeneratedBaseRoomTemplate *
        ]
    ),
    "cGcGeneratedBaseThemeTemplate::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGeneratedBaseThemeTemplate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGeneratedBaseThemeTemplate *
        ]
    ),
    "cGcLinkNetworkTypes::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLinkNetworkTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcLinkNetworkTypes *
        ]
    ),
    "cGcModBasePart::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcModBasePart *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcModBasePart *
        ]
    ),
    "cGcNPCHabitationType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCHabitationType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCHabitationType *
        ]
    ),
    "cGcNPCWorkerData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCWorkerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCWorkerData *
        ]
    ),
    "cGcPersistentBase::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistentBase *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPersistentBase *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBaseBuildingEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBaseBuildingFamily>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingFamily> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBaseBuildingGroup>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingGroup> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBaseBuildingMaterial>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingMaterial> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBaseBuildingPalette>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingPalette> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBaseBuildingPart>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingPart> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBaseBuildingPartData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingPartData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBaseBuildingPartInteractionData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingPartInteractionData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBaseBuildingPartNavData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingPartNavData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBaseBuildingPartNavNodeData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingPartNavNodeData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBaseLinkGridConnectionDependency>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseLinkGridConnectionDependency> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcId256List>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcId256List> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcBaseLinkGridConnectionData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridConnectionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcLinkNetworkTypes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLinkNetworkTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBaseBuildingEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBaseBuildingFamily const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBaseBuildingFamily *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBaseBuildingGroup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBaseBuildingGroup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBaseBuildingMaterial const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBaseBuildingMaterial *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBaseBuildingPalette const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBaseBuildingPalette *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBaseBuildingPart const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBaseBuildingPart *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBaseBuildingPartData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBaseBuildingPartInteractionData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartInteractionData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBaseBuildingPartNavData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartNavData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBaseBuildingPartNavNodeData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartNavNodeData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBaseLinkGridConnectionDependency const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBaseLinkGridConnectionDependency *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcId256List const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcId256List *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcBaseBuildingGroup>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingGroup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcBaseBuildingPartStyle>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartStyle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartStyle *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcBaseBuildingProperties>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingProperties *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcBaseLinkGridConnectionData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseLinkGridConnectionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseLinkGridConnectionData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcLinkNetworkTypes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcLinkNetworkTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcLinkNetworkTypes *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingPartNavData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartNavData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingPartNavNodeData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartNavNodeData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingPartStyle const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartStyle *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingPartStyleModel const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartStyleModel *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingPartsDataTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartsDataTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingPartsNavDataTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartsNavDataTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingPartsTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartsTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingProperties const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingProperties *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingSubGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingSubGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseGridSearchFilter const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseGridSearchFilter *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseLinkGridConnectionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseLinkGridConnectionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseLinkGridConnectionDependency const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseLinkGridConnectionDependency *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseLinkGridData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseLinkGridData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBasePartSearchFilter const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBasePartSearchFilter *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBasePlacementRule const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBasePlacementRule *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseSearchFilter const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseSearchFilter *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingEntry::cGcBaseBuildingEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEntry *
            ctypes.c_ulonglong,  # cGcBaseBuildingEntry *
        ]
    ),
    "cGcBaseLinkGridConnectionData::cGcBaseLinkGridConnectionData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridConnectionData *
            ctypes.c_ulonglong,  # cGcBaseLinkGridConnectionData *
        ]
    ),
    "cGcBaseBuildingGroup::~cGcBaseBuildingGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGroup *
        ]
    ),
    "cGcBaseBuildingPartStyleModel::~cGcBaseBuildingPartStyleModel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartStyleModel *
        ]
    ),
    "cGcBaseBuildingTable::~cGcBaseBuildingTable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingTable *
        ]
    ),
    "cTkDynamicArray<cGcBaseBuildingEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBaseBuildingFamily>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingFamily> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBaseBuildingGroup>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingGroup> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBaseBuildingMaterial>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingMaterial> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBaseBuildingPalette>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingPalette> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBaseBuildingPart>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingPart> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBaseBuildingPartData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingPartData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBaseBuildingPartInteractionData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingPartInteractionData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBaseBuildingPartNavData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingPartNavData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBaseBuildingPartNavNodeData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingPartNavNodeData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBaseLinkGridConnectionDependency>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseLinkGridConnectionDependency> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcId256List>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcId256List> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBaseBuildingPartNavData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingPartNavNodeData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingPartStyle::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingPartStyleModel::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingPartsDataTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingPartsNavDataTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingPartsTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingProperties::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingSubGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseGridSearchFilter::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseLinkGridConnectionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseLinkGridConnectionDependency::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseLinkGridData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBasePartSearchFilter::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBasePlacementRule::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseSearchFilter::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingPartNavData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPartNavNodeData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPartStyleModel::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPartsDataTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPartsNavDataTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPartsTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseGridSearchFilter::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseLinkGridConnectionData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseLinkGridConnectionDependency::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseLinkGridData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBasePartSearchFilter::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBasePlacementRule::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseSearchFilter::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcBaseBuildingPart>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingPart> *
        ]
    ),
    "cTkDynamicArray<cGcBaseBuildingPartStyleModel>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingPartStyleModel> *
        ]
    ),
    "cGcBaseBuildingPartNavData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartNavData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPartStyleModel::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartStyleModel *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingTable::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseLinkGridData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBasePartSearchFilter::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartSearchFilter *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseSearchFilter::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseSearchFilter *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPartNavData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartNavData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPartsDataTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartsDataTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPartsNavDataTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartsNavDataTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPartsTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartsTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseLinkGridData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseSearchFilter::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseSearchFilter *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPartNavData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartNavData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartNavNodeData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartNavNodeData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartStyle::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartStyle *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartStyleModel::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartStyleModel *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingProperties::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingProperties *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingSubGroup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingSubGroup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseGridSearchFilter::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseGridSearchFilter *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseLinkGridConnectionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridConnectionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseLinkGridConnectionDependency::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridConnectionDependency *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseLinkGridData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBasePartSearchFilter::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartSearchFilter *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBasePlacementRule::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePlacementRule *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseSearchFilter::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseSearchFilter *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartNavNodeData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartNavNodeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartStyle::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartStyle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartStyleModel::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartStyleModel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingSubGroup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingSubGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseGridSearchFilter::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseGridSearchFilter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseLinkGridConnectionData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridConnectionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseLinkGridConnectionDependency::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridConnectionDependency *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseLinkGridData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBasePartSearchFilter::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartSearchFilter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBasePlacementRule::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePlacementRule *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseSearchFilter::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseSearchFilter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartStyleModel::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartStyleModel *
        ]
    ),
    "cGcBaseBuildingTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingTable *
        ]
    ),
    "cGcBaseLinkGridConnectionData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridConnectionData *
        ]
    ),
    "cGcBaseLinkGridData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridData *
        ]
    ),
    "cGcBasePartSearchFilter::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartSearchFilter *
        ]
    ),
    "cGcBasePlacementRule::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePlacementRule *
        ]
    ),
    "cGcBaseSearchFilter::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseSearchFilter *
        ]
    ),
    "cGcBaseBuildingPartNavData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartNavData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartNavNodeData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartNavNodeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartStyle::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartStyle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartStyleModel::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartStyleModel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartsDataTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartsDataTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartsNavDataTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartsNavDataTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartsTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartsTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingProperties::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingSubGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingSubGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseGridSearchFilter::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseGridSearchFilter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseLinkGridConnectionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridConnectionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseLinkGridConnectionDependency::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridConnectionDependency *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseLinkGridData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBasePartSearchFilter::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartSearchFilter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBasePlacementRule::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePlacementRule *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseSearchFilter::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseSearchFilter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartNavData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartNavData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartNavData *
        ]
    ),
    "cGcBaseBuildingPartNavNodeData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartNavNodeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartNavNodeData *
        ]
    ),
    "cGcBaseBuildingPartStyle::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartStyle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartStyle *
        ]
    ),
    "cGcBaseBuildingProperties::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingProperties *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingProperties *
        ]
    ),
    "cGcBaseBuildingTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingTable *
        ]
    ),
    "cGcBaseGridSearchFilter::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseGridSearchFilter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseGridSearchFilter *
        ]
    ),
    "cGcBaseLinkGridConnectionData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridConnectionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseLinkGridConnectionData *
        ]
    ),
    "cGcBaseLinkGridConnectionDependency::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridConnectionDependency *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseLinkGridConnectionDependency *
        ]
    ),
    "cGcBaseLinkGridData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseLinkGridData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseLinkGridData *
        ]
    ),
    "cGcBasePartSearchFilter::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartSearchFilter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBasePartSearchFilter *
        ]
    ),
    "cGcBasePlacementRule::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePlacementRule *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBasePlacementRule *
        ]
    ),
    "cGcBaseSearchFilter::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseSearchFilter *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseSearchFilter *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBaseBuildingEntryCosts>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingEntryCosts> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBaseBuildingEntryGroup>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingEntryGroup> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBaseBuildingPartAudioLocationEntry>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingPartAudioLocationEntry> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBaseBuildingPartStyleModel>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingPartStyleModel> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBaseBuildingSubGroup>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingSubGroup> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcBaseBuildingPartStyle>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartStyle *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBaseBuildingEntryCosts const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntryCosts *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBaseBuildingEntryGroup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntryGroup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBaseBuildingPartAudioLocationEntry const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartAudioLocationEntry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBaseBuildingPartStyleModel const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartStyleModel *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBaseBuildingSubGroup const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBaseBuildingSubGroup *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcBaseBuildingObjectDecorationTypes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingObjectDecorationTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingObjectDecorationTypes *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseAutoPowerSetting const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseAutoPowerSetting *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingCostsTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingCostsTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingEntryCosts const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntryCosts *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingEntryGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntryGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingFamily const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingFamily *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingMaterial const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingMaterial *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingMode const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingMode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingObjectDecorationTypes const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingObjectDecorationTypes *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingPalette const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingPalette *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingPart const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingPart *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingPartAudioLocationEntry const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartAudioLocationEntry *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingPartAudioLocationTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartAudioLocationTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingPartData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseBuildingPartInteractionData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartInteractionData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcBaseBuildingEntryCosts>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingEntryCosts> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBaseBuildingPartAudioLocationEntry>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingPartAudioLocationEntry> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBaseBuildingPartStyleModel>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseBuildingPartStyleModel> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcBaseAutoPowerSetting::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingCostsTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingEntryCosts::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingEntryGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingFamily::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingMaterial::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingMode::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingObjectDecorationTypes::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingPalette::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingPart::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingPartAudioLocationEntry::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingPartAudioLocationTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingPartData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingPartInteractionData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseBuildingCostsTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingFamily::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingGroup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingMaterial::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPart::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPartAudioLocationEntry::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPartAudioLocationTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPartData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingEntry::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEntry *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingGroup::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGroup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPart::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPart *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPartData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingEntry::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEntry *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingGroup::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGroup *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPart::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPart *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPartAudioLocationTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartAudioLocationTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseBuildingPartData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseAutoPowerSetting::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseAutoPowerSetting *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingEntryCosts::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEntryCosts *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingEntryGroup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEntryGroup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingFamily::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingFamily *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingGroup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGroup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingMaterial::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingMaterial *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingObjectDecorationTypes::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObjectDecorationTypes *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPalette::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPalette *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPart::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPart *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartAudioLocationEntry::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartAudioLocationEntry *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartInteractionData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartInteractionData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseAutoPowerSetting::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseAutoPowerSetting *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcBaseAutoPowerSetting::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseAutoPowerSetting *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingEntryCosts::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEntryCosts *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingFamily::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingFamily *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingGroup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingMaterial::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingMaterial *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingObjectDecorationTypes::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObjectDecorationTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPalette::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPalette *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPart::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPart *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartAudioLocationEntry::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartAudioLocationEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartInteractionData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartInteractionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingEntry::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEntry *
        ]
    ),
    "cGcBaseBuildingGroup::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGroup *
        ]
    ),
    "cGcBaseBuildingMaterial::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingMaterial *
        ]
    ),
    "cGcBaseBuildingPalette::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPalette *
        ]
    ),
    "cGcBaseBuildingPartData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartData *
        ]
    ),
    "cGcBaseBuildingPartInteractionData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartInteractionData *
        ]
    ),
    "cGcBaseAutoPowerSetting::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseAutoPowerSetting *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcBaseAutoPowerSetting::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseAutoPowerSetting *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingCostsTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingCostsTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingEntryCosts::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEntryCosts *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingEntryGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEntryGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingFamily::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingFamily *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingMaterial::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingMaterial *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingMode::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingMode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingObjectDecorationTypes::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObjectDecorationTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPalette::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPalette *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPart::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPart *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartAudioLocationEntry::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartAudioLocationEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartAudioLocationTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartAudioLocationTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseBuildingPartInteractionData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartInteractionData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseAutoPowerSetting::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseAutoPowerSetting *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseAutoPowerSetting *
        ]
    ),
    "cGcBaseBuildingEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntry *
        ]
    ),
    "cGcBaseBuildingEntryCosts::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEntryCosts *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntryCosts *
        ]
    ),
    "cGcBaseBuildingEntryGroup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingEntryGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingEntryGroup *
        ]
    ),
    "cGcBaseBuildingGroup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingGroup *
        ]
    ),
    "cGcBaseBuildingMaterial::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingMaterial *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingMaterial *
        ]
    ),
    "cGcBaseBuildingObjectDecorationTypes::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingObjectDecorationTypes *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingObjectDecorationTypes *
        ]
    ),
    "cGcBaseBuildingPalette::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPalette *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingPalette *
        ]
    ),
    "cGcBaseBuildingPartAudioLocationEntry::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartAudioLocationEntry *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartAudioLocationEntry *
        ]
    ),
    "cGcBaseBuildingPartData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseBuildingPartData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseBuildingPartData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcInteractionBaseBuildingState>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcInteractionBaseBuildingState> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPersistencyMissionOverride>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPersistencyMissionOverride> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSentinelSpawnData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSentinelSpawnData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcInteractionBaseBuildingState const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcInteractionBaseBuildingState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPersistencyMissionOverride const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPersistencyMissionOverride *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSentinelSpawnData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSentinelSpawnData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSentinelSpawnSequenceGroupList const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSentinelSpawnSequenceGroupList *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSentinelSpawnSequenceStep const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSentinelSpawnSequenceStep *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSentinelSpawnWave const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSentinelSpawnWave *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSentinelWaveGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSentinelWaveGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementConstructionLevel const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementConstructionLevel *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementJudgementType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementJudgementType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSettlementStatType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSettlementStatType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSimpleIkRecoilComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSimpleIkRecoilComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSimpleInteractionComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSimpleInteractionComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSpawnedObjectComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSpawnedObjectComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcThereminComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcThereminComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcTurretComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcTurretComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcUniqueNPCSpawnData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcUniqueNPCSpawnData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcWiringSocketComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcWiringSocketComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcSentinelSpawnData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSentinelSpawnData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcSimpleIkRecoilComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSimpleIkRecoilComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSimpleInteractionComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSimpleInteractionComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpawnedObjectComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcSpawnedObjectComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcThereminComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcThereminComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTurretComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcTurretComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWiringSocketComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWiringSocketComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSentinelSpawnSequenceGroupList::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSentinelSpawnSequenceStep::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSentinelSpawnWave::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSentinelWaveGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementConstructionLevel::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementJudgementType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSettlementStatType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSimpleIkRecoilComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSimpleInteractionComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSpawnedObjectComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcThereminComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcTurretComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcUniqueNPCSpawnData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcWiringSocketComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSentinelSpawnSequenceGroupList::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSentinelSpawnSequenceStep::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSentinelSpawnWave::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSentinelWaveGroup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSimpleInteractionComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUniqueNPCSpawnData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSentinelSpawnWave::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnWave *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSimpleInteractionComponentData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponentData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUniqueNPCSpawnData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniqueNPCSpawnData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSentinelSpawnWave::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnWave *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcUniqueNPCSpawnData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniqueNPCSpawnData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSentinelSpawnSequenceGroupList::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnSequenceGroupList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelSpawnSequenceStep::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnSequenceStep *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelSpawnWave::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnWave *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelWaveGroup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWaveGroup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementJudgementType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementStatType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSimpleIkRecoilComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleIkRecoilComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSimpleInteractionComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcThereminComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcThereminComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTurretComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcTurretComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUniqueNPCSpawnData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniqueNPCSpawnData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementJudgementType::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementType *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcSentinelSpawnSequenceGroupList::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnSequenceGroupList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelSpawnSequenceStep::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnSequenceStep *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelSpawnWave::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnWave *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelWaveGroup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWaveGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementJudgementType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementStatType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSimpleInteractionComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTurretComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTurretComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUniqueNPCSpawnData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniqueNPCSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelSpawnWave::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnWave *
        ]
    ),
    "cGcSimpleIkRecoilComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleIkRecoilComponentData *
        ]
    ),
    "cGcSimpleInteractionComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponentData *
        ]
    ),
    "cGcThereminComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcThereminComponentData *
        ]
    ),
    "cGcTurretComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTurretComponentData *
        ]
    ),
    "cGcUniqueNPCSpawnData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniqueNPCSpawnData *
        ]
    ),
    "cGcSettlementJudgementType::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementType *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcSentinelSpawnSequenceGroupList::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnSequenceGroupList *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelSpawnSequenceStep::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnSequenceStep *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelSpawnWave::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnWave *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelWaveGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWaveGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementConstructionLevel::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementConstructionLevel *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementJudgementType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementStatType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSimpleIkRecoilComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleIkRecoilComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSimpleInteractionComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcThereminComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcThereminComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcTurretComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTurretComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcUniqueNPCSpawnData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniqueNPCSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelWaveGroup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelWaveGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSettlementJudgementType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementJudgementType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementJudgementType *
        ]
    ),
    "cGcSettlementStatType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSettlementStatType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSettlementStatType *
        ]
    ),
    "cGcSimpleIkRecoilComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleIkRecoilComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSimpleIkRecoilComponentData *
        ]
    ),
    "cGcSimpleInteractionComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSimpleInteractionComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSimpleInteractionComponentData *
        ]
    ),
    "cGcThereminComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcThereminComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcThereminComponentData *
        ]
    ),
    "cGcTurretComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcTurretComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcTurretComponentData *
        ]
    ),
    "cGcUniqueNPCSpawnData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcUniqueNPCSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcUniqueNPCSpawnData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerExperienceAsteroidCreatureSpawnData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerExperienceAsteroidCreatureSpawnData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcPlayerExperienceSpawnData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerExperienceSpawnData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcSentinelSpawnSequenceStep>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcSentinelSpawnSequenceStep> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcSentinelSpawnSequence>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnSequence *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerExperienceAsteroidCreatureSpawnData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerExperienceAsteroidCreatureSpawnData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcPlayerExperienceSpawnData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcPlayerExperienceSpawnData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcSentinelSpawnSequenceStep const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcSentinelSpawnSequenceStep *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcSentinelSpawnSequence>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSentinelSpawnSequence *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSentinelSpawnSequence *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCPlacementComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCPlacementComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcNPCPlacementInfo const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcNPCPlacementInfo *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcObjectPlacementComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcObjectPlacementComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcObjectSpawnerComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcObjectSpawnerComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPersistencyMissionOverride const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPersistencyMissionOverride *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerExperienceAsteroidCreatureSpawnData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerExperienceAsteroidCreatureSpawnData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerExperienceAsteroidCreatureSpawnTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerExperienceAsteroidCreatureSpawnTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerExperienceSpawnArchetypeData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerExperienceSpawnArchetypeData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerExperienceSpawnData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerExperienceSpawnData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPlayerExperienceSpawnTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPlayerExperienceSpawnTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPortalComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcByteBeatSwitchComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcPortalData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcPortalData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSentinelCoverState const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSentinelCoverState *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSentinelSpawnData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSentinelSpawnData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSentinelSpawnNamedSequence const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSentinelSpawnNamedSequence *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSentinelSpawnSequence const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSentinelSpawnSequence *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcSentinelSpawnSequenceGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcSentinelSpawnSequenceGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcPlayerExperienceAsteroidCreatureSpawnData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerExperienceAsteroidCreatureSpawnData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcPlayerExperienceSpawnData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcPlayerExperienceSpawnData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcNPCPlacementComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcNPCPlacementComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcObjectPlacementComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcObjectPlacementComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcObjectSpawnerComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcObjectSpawnerComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPortalComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcPortalComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCPlacementComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCPlacementInfo::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcObjectPlacementComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcObjectSpawnerComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPersistencyMissionOverride::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerExperienceAsteroidCreatureSpawnData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerExperienceAsteroidCreatureSpawnTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerExperienceSpawnArchetypeData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerExperienceSpawnData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPlayerExperienceSpawnTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPortalComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcPortalData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSentinelCoverState::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSentinelSpawnData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSentinelSpawnNamedSequence::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSentinelSpawnSequence::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcSentinelSpawnSequenceGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcNPCPlacementComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCPlacementInfo::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcObjectSpawnerComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPersistencyMissionOverride::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerExperienceAsteroidCreatureSpawnTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerExperienceSpawnArchetypeData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerExperienceSpawnTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSentinelSpawnData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSentinelSpawnNamedSequence::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSentinelSpawnSequence::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSentinelSpawnSequenceGroup::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCPlacementInfo::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementInfo *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerExperienceAsteroidCreatureSpawnTable::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceAsteroidCreatureSpawnTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerExperienceSpawnArchetypeData::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceSpawnArchetypeData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSentinelSpawnNamedSequence::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnNamedSequence *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCPlacementInfo::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementInfo *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerExperienceAsteroidCreatureSpawnTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceAsteroidCreatureSpawnTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcPlayerExperienceSpawnArchetypeData::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceSpawnArchetypeData *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcSentinelSpawnNamedSequence::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnNamedSequence *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcNPCPlacementComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCPlacementInfo::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementInfo *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectPlacementComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectPlacementComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistencyMissionOverride::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistencyMissionOverride *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceAsteroidCreatureSpawnData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceAsteroidCreatureSpawnData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceAsteroidCreatureSpawnTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceAsteroidCreatureSpawnTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceSpawnArchetypeData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceSpawnArchetypeData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceSpawnData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceSpawnData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceSpawnTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceSpawnTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPortalData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelSpawnData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelSpawnNamedSequence::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnNamedSequence *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelSpawnSequence::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnSequence *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCPlacementInfo::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistencyMissionOverride::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistencyMissionOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceAsteroidCreatureSpawnData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceAsteroidCreatureSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceAsteroidCreatureSpawnTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceAsteroidCreatureSpawnTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceSpawnArchetypeData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceSpawnArchetypeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceSpawnData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceSpawnTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceSpawnTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPortalData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelSpawnData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelSpawnNamedSequence::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnNamedSequence *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCPlacementInfo::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementInfo *
        ]
    ),
    "cGcObjectPlacementComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectPlacementComponentData *
        ]
    ),
    "cGcPlayerExperienceAsteroidCreatureSpawnTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceAsteroidCreatureSpawnTable *
        ]
    ),
    "cGcPlayerExperienceSpawnArchetypeData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceSpawnArchetypeData *
        ]
    ),
    "cGcPlayerExperienceSpawnTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceSpawnTable *
        ]
    ),
    "cGcPortalData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalData *
        ]
    ),
    "cGcSentinelSpawnData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnData *
        ]
    ),
    "cGcNPCPlacementComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCPlacementInfo::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectPlacementComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectPlacementComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcObjectSpawnerComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnerComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPersistencyMissionOverride::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistencyMissionOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceAsteroidCreatureSpawnData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceAsteroidCreatureSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceAsteroidCreatureSpawnTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceAsteroidCreatureSpawnTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceSpawnArchetypeData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceSpawnArchetypeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceSpawnData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPlayerExperienceSpawnTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceSpawnTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPortalComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSwitchComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcPortalData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelCoverState::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelCoverState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelSpawnData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelSpawnNamedSequence::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnNamedSequence *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelSpawnSequence::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnSequence *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcSentinelSpawnSequenceGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnSequenceGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcNPCPlacementInfo::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcNPCPlacementInfo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcNPCPlacementInfo *
        ]
    ),
    "cGcObjectPlacementComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectPlacementComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcObjectPlacementComponentData *
        ]
    ),
    "cGcObjectSpawnerComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcObjectSpawnerComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcObjectSpawnerComponentData *
        ]
    ),
    "cGcPersistencyMissionOverride::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPersistencyMissionOverride *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPersistencyMissionOverride *
        ]
    ),
    "cGcPlayerExperienceAsteroidCreatureSpawnData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceAsteroidCreatureSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerExperienceAsteroidCreatureSpawnData *
        ]
    ),
    "cGcPlayerExperienceAsteroidCreatureSpawnTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceAsteroidCreatureSpawnTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerExperienceAsteroidCreatureSpawnTable *
        ]
    ),
    "cGcPlayerExperienceSpawnArchetypeData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceSpawnArchetypeData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerExperienceSpawnArchetypeData *
        ]
    ),
    "cGcPlayerExperienceSpawnData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerExperienceSpawnData *
        ]
    ),
    "cGcPlayerExperienceSpawnTable::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPlayerExperienceSpawnTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPlayerExperienceSpawnTable *
        ]
    ),
    "cGcPortalData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcPortalData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcPortalData *
        ]
    ),
    "cGcSentinelSpawnData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcSentinelSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcSentinelSpawnData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcFiendCrimeSpawnData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFiendCrimeSpawnData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcGrabbableData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGrabbableData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcFiendCrimeSpawnData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcFiendCrimeSpawnData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcGrabbableData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcGrabbableData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcFiendCrime>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFiendCrime *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFiendCrime *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDistanceScaleComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDistanceScaleComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcDroneGun const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcDroneGun *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcEncounterType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcEncounterType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFiendCrime const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFiendCrime *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFiendCrimeSpawnData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFiendCrimeSpawnData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcFiendCrimeSpawnTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcFiendCrimeSpawnTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGrabPlayerComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGrabPlayerComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGrabbableComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGrabbableComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcGroundWormComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGroundWormComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcHeightAdjustComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcHeightAdjustComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcImpactCombatEffectData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcImpactCombatEffectData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInteractionActivationCost const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInteractionActivationCost *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcInteractionBaseBuildingState const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcInteractionBaseBuildingState *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcLandingHelperComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcLandingHelperComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcLootComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcLootComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcFiendCrimeSpawnData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcFiendCrimeSpawnData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcGrabbableData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcGrabbableData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcDistanceScaleComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcDistanceScaleComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGrabPlayerComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGrabPlayerComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGrabbableComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGrabbableComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGroundWormComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcGroundWormComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHeightAdjustComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcHeightAdjustComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLandingHelperComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcLandingHelperComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLootComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcLootComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDistanceScaleComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcDroneGun::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEncounterType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFiendCrime::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFiendCrimeSpawnData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFiendCrimeSpawnTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGrabPlayerComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGrabbableComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcGroundWormComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcHeightAdjustComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcImpactCombatEffectData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInteractionActivationCost::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcInteractionBaseBuildingState::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLandingHelperComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcLootComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcFiendCrimeSpawnData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFiendCrimeSpawnTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGrabPlayerComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcGrabbableComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcImpactCombatEffectData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInteractionActivationCost::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFiendCrimeSpawnTable::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendCrimeSpawnTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInteractionActivationCost::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionActivationCost *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcFiendCrimeSpawnTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendCrimeSpawnTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcInteractionActivationCost::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionActivationCost *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcDistanceScaleComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDistanceScaleComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneGun::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneGun *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEncounterType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFiendCrime::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendCrime *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFiendCrimeSpawnData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendCrimeSpawnData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFiendCrimeSpawnTable::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendCrimeSpawnTable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGrabPlayerComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabPlayerComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGroundWormComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundWormComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcImpactCombatEffectData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcImpactCombatEffectData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionActivationCost::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionActivationCost *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionBaseBuildingState::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionBaseBuildingState *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLandingHelperComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLandingHelperComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLootComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcLootComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneGun::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneGun *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEncounterType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFiendCrime::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendCrime *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFiendCrimeSpawnData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendCrimeSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFiendCrimeSpawnTable::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendCrimeSpawnTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGrabPlayerComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabPlayerComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGroundWormComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundWormComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcImpactCombatEffectData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcImpactCombatEffectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionActivationCost::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionActivationCost *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionBaseBuildingState::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionBaseBuildingState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneGun::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneGun *
        ]
    ),
    "cGcGrabPlayerComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabPlayerComponentData *
        ]
    ),
    "cGcGroundWormComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundWormComponentData *
        ]
    ),
    "cGcImpactCombatEffectData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcImpactCombatEffectData *
        ]
    ),
    "cGcInteractionActivationCost::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionActivationCost *
        ]
    ),
    "cGcDistanceScaleComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDistanceScaleComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDroneGun::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDroneGun *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEncounterType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFiendCrime::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendCrime *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFiendCrimeSpawnData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendCrimeSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcFiendCrimeSpawnTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendCrimeSpawnTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGrabPlayerComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabPlayerComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGrabbableComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabbableComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcGroundWormComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundWormComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcHeightAdjustComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcHeightAdjustComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcImpactCombatEffectData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcImpactCombatEffectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionActivationCost::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionActivationCost *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcInteractionBaseBuildingState::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionBaseBuildingState *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLandingHelperComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLandingHelperComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcLootComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLootComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcDistanceScaleComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcDistanceScaleComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcDistanceScaleComponentData *
        ]
    ),
    "cGcEncounterType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEncounterType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcEncounterType *
        ]
    ),
    "cGcFiendCrime::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendCrime *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFiendCrime *
        ]
    ),
    "cGcFiendCrimeSpawnData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcFiendCrimeSpawnData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcFiendCrimeSpawnData *
        ]
    ),
    "cGcGrabPlayerComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGrabPlayerComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGrabPlayerComponentData *
        ]
    ),
    "cGcGroundWormComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGroundWormComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGroundWormComponentData *
        ]
    ),
    "cGcImpactCombatEffectData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcImpactCombatEffectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcImpactCombatEffectData *
        ]
    ),
    "cGcInteractionActivationCost::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcInteractionActivationCost *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcInteractionActivationCost *
        ]
    ),
    "cGcLootComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcLootComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcLootComponentData *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBaseDefenceTrigger>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseDefenceTrigger> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcBasePlacementRule>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBasePlacementRule> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcEntitlementRewardData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcEntitlementRewardData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBaseDefenceTrigger const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBaseDefenceTrigger *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcBasePlacementRule const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcBasePlacementRule *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcCombatEffectData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcCombatEffectData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcEntitlementRewardData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcEntitlementRewardData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAlienPodComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAlienPodComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAntagonistEnemy const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAntagonistEnemy *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAntagonistFriend const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAntagonistFriend *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAntagonistGroup const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAntagonistGroup *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAntagonistPerception const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAntagonistPerception *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAtmosphereEntryComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtmosphereEntryComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseDefenceComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseDefenceComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBaseMiniPortalComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBaseMiniPortalComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBasePlacementComponentData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBasePlacementComponentData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCombatEffectData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCombatEffectData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCombatEffectType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCombatEffectType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcCombatEffectsTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcCombatEffectsTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcConstraintsToCreateSpec const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcConstraintsToCreateSpec *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcEntitlementRewardData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcEntitlementRewardData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcEntitlementRewardsTable const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcEntitlementRewardsTable *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcRectangularInitialShape const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcRectangularInitialShape *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cTkEntitlementListData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkEntitlementListData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFixedArray<cGcCombatEffectData,4>::cTkFixedArray<cGcCombatEffectData,4>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedArray<cGcCombatEffectData,4> *
        ]
    ),
    "cTkDynamicArray<cGcBaseDefenceTrigger>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBaseDefenceTrigger> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcBasePlacementRule>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBasePlacementRule> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcEntitlementRewardData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcEntitlementRewardData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cGcAlienPodComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAlienPodComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAtmosphereEntryComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcAtmosphereEntryComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseDefenceComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBaseDefenceComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseMiniPortalComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBaseMiniPortalComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBasePlacementComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBasePlacementComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcBuildingComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcByteBeatSwitchComponentData::Cast": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcByteBeatSwitchComponentData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAlienPodComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAntagonistEnemy::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAntagonistFriend::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAntagonistGroup::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAntagonistPerception::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAtmosphereEntryComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseDefenceComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBaseMiniPortalComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBasePlacementComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBuildingComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcByteBeatSwitchComponentData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCombatEffectData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCombatEffectType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcCombatEffectsTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcConstraintsToCreateSpec::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEntitlementRewardData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcEntitlementRewardsTable::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcRectangularInitialShape::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkEntitlementListData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAntagonistEnemy::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAntagonistFriend::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBaseDefenceComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcBasePlacementComponentData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCombatEffectData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcCombatEffectsTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcEntitlementRewardsTable::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicArray<cGcBasePlacementRule>::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcBasePlacementRule> *
        ]
    ),
    "cGcCombatEffectsTable::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsTable *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAlienPodComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPodComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAntagonistEnemy::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistEnemy *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAntagonistFriend::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistFriend *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAntagonistGroup::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistGroup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAntagonistPerception::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistPerception *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtmosphereEntryComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtmosphereEntryComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseDefenceComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseMiniPortalComponentData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponentData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatEffectData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatEffectType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcConstraintsToCreateSpec::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcConstraintsToCreateSpec *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEntitlementRewardData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcEntitlementRewardData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRectangularInitialShape::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcRectangularInitialShape *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEntitlementListData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkEntitlementListData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAntagonistEnemy::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistEnemy *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAntagonistFriend::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistFriend *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAntagonistGroup::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAntagonistPerception::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistPerception *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseMiniPortalComponentData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatEffectData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatEffectType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcConstraintsToCreateSpec::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcConstraintsToCreateSpec *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRectangularInitialShape::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRectangularInitialShape *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEntitlementListData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEntitlementListData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAntagonistEnemy::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistEnemy *
        ]
    ),
    "cGcAntagonistFriend::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistFriend *
        ]
    ),
    "cGcAntagonistPerception::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistPerception *
        ]
    ),
    "cGcAtmosphereEntryComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtmosphereEntryComponentData *
        ]
    ),
    "cGcBaseMiniPortalComponentData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponentData *
        ]
    ),
    "cGcCombatEffectsTable::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsTable *
        ]
    ),
    "cGcConstraintsToCreateSpec::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcConstraintsToCreateSpec *
        ]
    ),
    "cGcRectangularInitialShape::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRectangularInitialShape *
        ]
    ),
    "cTkEntitlementListData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEntitlementListData *
        ]
    ),
    "cGcAlienPodComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPodComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAntagonistEnemy::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistEnemy *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAntagonistFriend::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistFriend *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAntagonistGroup::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAntagonistPerception::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistPerception *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtmosphereEntryComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtmosphereEntryComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseDefenceComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBaseMiniPortalComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBasePlacementComponentData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePlacementComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatEffectData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatEffectType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcCombatEffectsTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectsTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcConstraintsToCreateSpec::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcConstraintsToCreateSpec *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEntitlementRewardData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEntitlementRewardData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcEntitlementRewardsTable::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEntitlementRewardsTable *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcRectangularInitialShape::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRectangularInitialShape *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEntitlementListData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEntitlementListData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPodComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPodComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAlienPodComponentData *
        ]
    ),
    "cGcAntagonistGroup::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistGroup *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAntagonistGroup *
        ]
    ),
    "cGcAntagonistPerception::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAntagonistPerception *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAntagonistPerception *
        ]
    ),
    "cGcAtmosphereEntryComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtmosphereEntryComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAtmosphereEntryComponentData *
        ]
    ),
    "cGcBaseDefenceComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseDefenceComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseDefenceComponentData *
        ]
    ),
    "cGcBaseMiniPortalComponentData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBaseMiniPortalComponentData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBaseMiniPortalComponentData *
        ]
    ),
    "cGcCombatEffectData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCombatEffectData *
        ]
    ),
    "cGcCombatEffectType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcCombatEffectType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcCombatEffectType *
        ]
    ),
    "cGcConstraintsToCreateSpec::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcConstraintsToCreateSpec *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcConstraintsToCreateSpec *
        ]
    ),
    "cGcEntitlementRewardData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcEntitlementRewardData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcEntitlementRewardData *
        ]
    ),
    "cGcRectangularInitialShape::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcRectangularInitialShape *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcRectangularInitialShape *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcAttachedNode>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAttachedNode> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcAttachmentPointData>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAttachmentPointData> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcAttachmentPointSet>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAttachmentPointSet> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcByteBeatDrum>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcByteBeatDrum> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcByteBeatSong>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcByteBeatSong> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcByteBeatTemplate>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcByteBeatTemplate> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcClothAttachmentCirlce>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcClothAttachmentCirlce> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcClothAttachmentLine>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcClothAttachmentLine> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcClothCollisionBone>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcClothCollisionBone> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cAxisSpecification>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cAxisSpecification *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAttachedNode const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAttachedNode *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAttachmentPointData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAttachmentPointData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAttachmentPointSet const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAttachmentPointSet *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcByteBeatDrum const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcByteBeatDrum *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcByteBeatSong const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcByteBeatSong *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcByteBeatTemplate const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcByteBeatTemplate *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcClothAttachmentCirlce const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcClothAttachmentCirlce *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcClothAttachmentLine const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcClothAttachmentLine *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcClothCollisionBone const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcClothCollisionBone *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cAxisSpecification>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cAxisSpecification *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cAxisSpecification *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcAdvancedTweaks>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAdvancedTweaks *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAdvancedTweaks *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcByteBeatToken>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcByteBeatToken *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcByteBeatToken *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cAxisSpecification const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cAxisSpecification *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAdvancedTweaks const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAdvancedTweaks *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAttachedNode const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAttachedNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAttachmentPointData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAttachmentPointData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAttachmentPointSet const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAttachmentPointSet *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcByteBeatEnvelope const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcByteBeatEnvelope *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcByteBeatLibraryData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcByteBeatLibraryData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcByteBeatSong const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcByteBeatSong *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcByteBeatTemplate const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcByteBeatTemplate *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcByteBeatTemplates const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcByteBeatTemplates *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcByteBeatToken const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcByteBeatToken *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcByteBeatWave const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcByteBeatWave *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcClothAttachmentCirlce const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcClothAttachmentCirlce *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcClothAttachmentLine const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcClothAttachmentLine *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcClothCollisionBone const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcClothCollisionBone *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcClothPiece const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcClothPiece *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicArray<cGcAttachedNode>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAttachedNode> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcAttachmentPointData>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAttachmentPointData> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcAttachmentPointSet>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcAttachmentPointSet> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcByteBeatDrum>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcByteBeatDrum> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcByteBeatSong>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcByteBeatSong> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcByteBeatTemplate>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcByteBeatTemplate> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcClothAttachmentCirlce>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcClothAttachmentCirlce> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcClothAttachmentLine>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcClothAttachmentLine> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicArray<cGcClothCollisionBone>::Allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cGcClothCollisionBone> *
            ctypes.c_int32,  # int
        ]
    ),
    "cAxisSpecification::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAdvancedTweaks::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAttachedNode::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAttachmentPointData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAttachmentPointSet::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcByteBeatEnvelope::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcByteBeatLibraryData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcByteBeatSong::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcByteBeatTemplate::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcByteBeatTemplates::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcByteBeatToken::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcByteBeatWave::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcClothAttachmentCirlce::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcClothAttachmentLine::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcClothCollisionBone::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcClothPiece::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAttachmentPointSet::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcByteBeatTemplate::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcByteBeatTemplates::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcClothPiece::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcByteBeatTemplates::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTemplates *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcClothPiece::FixDown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothPiece *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcByteBeatTemplate::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTemplate *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcByteBeatTemplates::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTemplates *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcClothPiece::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothPiece *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cAxisSpecification::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cAxisSpecification *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAdvancedTweaks::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAdvancedTweaks *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAttachedNode::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttachedNode *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAttachmentPointData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttachmentPointData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAttachmentPointSet::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttachmentPointSet *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatEnvelope::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatEnvelope *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatLibraryData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatLibraryData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatSong::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSong *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatTemplate::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTemplate *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatTemplates::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTemplates *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatToken::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatToken *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatWave::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatWave *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcClothAttachmentCirlce::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothAttachmentCirlce *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcClothAttachmentLine::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothAttachmentLine *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcClothCollisionBone::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothCollisionBone *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcClothPiece::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothPiece *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatLibraryData::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatLibraryData *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcByteBeatSong::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSong *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cAxisSpecification::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cAxisSpecification *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAdvancedTweaks::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAdvancedTweaks *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAttachedNode::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttachedNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAttachmentPointSet::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttachmentPointSet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatEnvelope::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatEnvelope *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatLibraryData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatLibraryData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatSong::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSong *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatTemplate::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTemplate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatTemplates::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTemplates *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatToken::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatToken *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatWave::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatWave *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcClothAttachmentCirlce::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothAttachmentCirlce *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcClothAttachmentLine::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothAttachmentLine *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcClothCollisionBone::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothCollisionBone *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcClothPiece::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothPiece *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatTemplates::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTemplates *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAttachedNode::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttachedNode *
        ]
    ),
    "cGcAttachmentPointSet::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttachmentPointSet *
        ]
    ),
    "cGcByteBeatLibraryData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatLibraryData *
        ]
    ),
    "cGcByteBeatSong::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSong *
        ]
    ),
    "cGcClothAttachmentCirlce::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothAttachmentCirlce *
        ]
    ),
    "cGcClothAttachmentLine::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothAttachmentLine *
        ]
    ),
    "cGcClothCollisionBone::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothCollisionBone *
        ]
    ),
    "cGcClothPiece::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothPiece *
        ]
    ),
    "cGcByteBeatLibraryData::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatLibraryData *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcByteBeatSong::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSong *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cAxisSpecification::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cAxisSpecification *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAdvancedTweaks::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAdvancedTweaks *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAttachedNode::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttachedNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAttachmentPointData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttachmentPointData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAttachmentPointSet::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttachmentPointSet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatEnvelope::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatEnvelope *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatLibraryData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatLibraryData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatSong::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSong *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatTemplate::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTemplate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatTemplates::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTemplates *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatToken::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatToken *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatWave::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatWave *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcClothAttachmentCirlce::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothAttachmentCirlce *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcClothAttachmentLine::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothAttachmentLine *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcClothCollisionBone::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothCollisionBone *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcClothPiece::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothPiece *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cAxisSpecification::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cAxisSpecification *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cAxisSpecification *
        ]
    ),
    "cGcAdvancedTweaks::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAdvancedTweaks *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAdvancedTweaks *
        ]
    ),
    "cGcAttachedNode::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttachedNode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAttachedNode *
        ]
    ),
    "cGcAttachmentPointData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttachmentPointData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAttachmentPointData *
        ]
    ),
    "cGcAttachmentPointSet::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAttachmentPointSet *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAttachmentPointSet *
        ]
    ),
    "cGcByteBeatEnvelope::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatEnvelope *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcByteBeatEnvelope *
        ]
    ),
    "cGcByteBeatLibraryData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatLibraryData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcByteBeatLibraryData *
        ]
    ),
    "cGcByteBeatSong::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatSong *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcByteBeatSong *
        ]
    ),
    "cGcByteBeatTemplate::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTemplate *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcByteBeatTemplate *
        ]
    ),
    "cGcByteBeatTemplates::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatTemplates *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcByteBeatTemplates *
        ]
    ),
    "cGcByteBeatToken::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatToken *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcByteBeatToken *
        ]
    ),
    "cGcByteBeatWave::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatWave *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcByteBeatWave *
        ]
    ),
    "cGcClothAttachmentCirlce::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothAttachmentCirlce *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcClothAttachmentCirlce *
        ]
    ),
    "cGcClothAttachmentLine::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothAttachmentLine *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcClothAttachmentLine *
        ]
    ),
    "cGcClothCollisionBone::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothCollisionBone *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcClothCollisionBone *
        ]
    ),
    "cGcClothPiece::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcClothPiece *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcClothPiece *
        ]
    ),
    "cTkMetaDataXML::ReadClassArrayXML<cGcAudio3PointDopplerData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudio3PointDopplerData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcByteBeatEnvelope>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatEnvelope *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassArrayXML<cGcAudio3PointDopplerData const>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cGcAudio3PointDopplerData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcAudio3PointDopplerData>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAudio3PointDopplerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAudio3PointDopplerData *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcByteBeatEnvelope>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcByteBeatEnvelope *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcByteBeatEnvelope *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAudio3PointDopplerData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAudio3PointDopplerData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAudioNPCDoppler const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAudioNPCDoppler *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAudioPulseDemo const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAudioPulseDemo *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAudioWwiseEvents const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAudioWwiseEvents *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAudioWwiseRTPCs const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAudioWwiseRTPCs *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcBasePartAudioLocation const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcBasePartAudioLocation *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcByteBeatDrum const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcByteBeatDrum *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudio3PointDopplerData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAudioNPCDoppler::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAudioPulseDemo::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAudioWwiseEvents::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAudioWwiseRTPCs::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcBasePartAudioLocation::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcByteBeatDrum::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcByteBeatDrum::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcByteBeatDrum::FixUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatDrum *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAudio3PointDopplerData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudio3PointDopplerData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudioPulseDemo::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseDemo *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudioWwiseEvents::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioWwiseEvents *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudioWwiseRTPCs::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioWwiseRTPCs *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBasePartAudioLocation::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartAudioLocation *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatDrum::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatDrum *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudioWwiseEvents::ReadFromDocument": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioWwiseEvents *
            ctypes.c_ulonglong,  # ITkDocumentReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cGcAudio3PointDopplerData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudio3PointDopplerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudioNPCDoppler::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioNPCDoppler *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudioWwiseEvents::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioWwiseEvents *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudioWwiseRTPCs::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioWwiseRTPCs *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBasePartAudioLocation::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartAudioLocation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatDrum::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatDrum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasSendSubmitDiscoveryExact::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendSubmitDiscoveryExact *
        ]
    ),
    "cGcAtlasSendSubmitMessage::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendSubmitMessage *
        ]
    ),
    "cGcAtlasSendSubmitSettlement::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendSubmitSettlement *
        ]
    ),
    "cGcAudio3PointDopplerData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudio3PointDopplerData *
        ]
    ),
    "cGcAudioNPCDoppler::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioNPCDoppler *
        ]
    ),
    "cGcByteBeatDrum::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatDrum *
        ]
    ),
    "cGcAudioWwiseEvents::WriteToDocument": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioWwiseEvents *
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cGcAudio3PointDopplerData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudio3PointDopplerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudioNPCDoppler::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioNPCDoppler *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudioPulseDemo::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseDemo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudioWwiseEvents::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioWwiseEvents *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudioWwiseRTPCs::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioWwiseRTPCs *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcBasePartAudioLocation::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartAudioLocation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcByteBeatDrum::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatDrum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudio3PointDopplerData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudio3PointDopplerData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAudio3PointDopplerData *
        ]
    ),
    "cGcAudioNPCDoppler::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioNPCDoppler *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAudioPulseDemo::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioPulseDemo *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAudioPulseDemo *
        ]
    ),
    "cGcAudioWwiseEvents::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioWwiseEvents *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAudioWwiseEvents *
        ]
    ),
    "cGcAudioWwiseRTPCs::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAudioWwiseRTPCs *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAudioWwiseRTPCs *
        ]
    ),
    "cGcBasePartAudioLocation::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcBasePartAudioLocation *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcBasePartAudioLocation *
        ]
    ),
    "cGcByteBeatDrum::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcByteBeatDrum *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcByteBeatDrum *
        ]
    ),
    "cTkMetaDataXML::ReadClassXML<cGcGameMode>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcGameMode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcGameMode>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcGameMode *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcGameMode *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAtlasSendSubmitContribution const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtlasSendSubmitContribution *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasSendSubmitContribution::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAtlasSendSubmitContribution::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendSubmitContribution *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasSendSubmitContribution::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendSubmitContribution *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasRecvMonumentList::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasRecvMonumentList *
        ]
    ),
    "cGcAtlasRecvSettlementList::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasRecvSettlementList *
        ]
    ),
    "cGcAtlasRecvTotalContribution::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasRecvTotalContribution *
        ]
    ),
    "cGcAtlasRecvVoxel::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasRecvVoxel *
        ]
    ),
    "cGcAtlasSendMarkFeaturedBase::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendMarkFeaturedBaseWithImage *
        ]
    ),
    "cGcAtlasRecvActiveFeaturedBasesQuery::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasRecvActiveFeaturedBasesQuery *
        ]
    ),
    "cGcAtlasSendReport::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendReport *
        ]
    ),
    "cGcAtlasSendReportBase::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendReportBase *
        ]
    ),
    "cGcAtlasSendRequestBases::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendRequestBases *
        ]
    ),
    "cGcAtlasSendRequestBlob::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendRequestBlob *
        ]
    ),
    "cGcAtlasSendRequestDiscoveryExact::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendRequestDiscoveryExact *
        ]
    ),
    "cGcAtlasSendSubmitBase::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendSubmitBase *
        ]
    ),
    "cGcAtlasSendSubmitBaseWithImage::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendSubmitBaseWithImage *
        ]
    ),
    "cGcAtlasSendSubmitContribution::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendSubmitContribution *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasSendSubmitContribution::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasSendSubmitContribution *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAtlasSendSubmitContribution *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcAtlasDiscoveryMetadata>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtlasDiscoveryMetadata *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAtlasDiscoveryMetadata *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcAtlasDownloadType>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtlasDownloadType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAtlasDownloadType *
        ]
    ),
    "cTkMetaDataXML::WriteClassXML<cGcAtlasMessage>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtlasMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAtlasMessage *
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAlienPodAnimParams const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAlienPodAnimParams *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAtlasActiveFeaturedBaseHeader const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtlasActiveFeaturedBaseHeader *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAtlasBase const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtlasBase *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAtlasBaseQuery const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtlasBaseQuery *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAtlasBasesRequest const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtlasBasesRequest *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAtlasDiscovery const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtlasDiscovery *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAtlasDiscoveryData const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtlasDiscoveryData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAtlasDiscoveryMetadata const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtlasDiscoveryMetadata *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAtlasDownloadType const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtlasDownloadType *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAtlasFeaturedBaseStateChange const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtlasFeaturedBaseStateChange *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAtlasMessage const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtlasMessage *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcAtlasMonument const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcAtlasMonument *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteToXMLFile<cGcThirdPersonAnimParams const>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cGcThirdPersonAnimParams *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAlienPodAnimParams::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAtlasActiveFeaturedBaseHeader::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAtlasBase::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAtlasBaseQuery::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAtlasBasesRequest::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAtlasDiscovery::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAtlasDiscoveryData::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAtlasDiscoveryMetadata::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAtlasDownloadType::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAtlasFeaturedBaseStateChange::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAtlasMessage::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAtlasMonument::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcThirdPersonAnimParams::ClassPointerCreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkClassPointer *
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cGcAtlasBase::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAtlasBaseQuery::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAtlasBasesRequest::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAtlasDiscovery::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAtlasDiscoveryData::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAtlasMessage::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAtlasMonument::ClassPointerFix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cGcAtlasActiveFeaturedBaseHeader::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasActiveFeaturedBaseHeader *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasBaseQuery::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBaseQuery *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasBasesRequest::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBasesRequest *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasDiscovery::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDiscovery *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasDiscoveryData::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDiscoveryData *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasDiscoveryMetadata::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDiscoveryMetadata *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasDownloadType::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDownloadType *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasFeaturedBaseStateChange::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasFeaturedBaseStateChange *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasMessage::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasMessage *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasMonument::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasMonument *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcThirdPersonAnimParams::GenerateHash": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cGcThirdPersonAnimParams *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasActiveFeaturedBaseHeader::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasActiveFeaturedBaseHeader *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasDiscovery::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDiscovery *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasDiscoveryData::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDiscoveryData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasDownloadType::ReadFromXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDownloadType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasActiveFeaturedBaseHeader::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasActiveFeaturedBaseHeader *
        ]
    ),
    "cGcAtlasBase::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBase *
        ]
    ),
    "cGcAtlasBaseQuery::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBaseQuery *
        ]
    ),
    "cGcAtlasBasesRequest::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBasesRequest *
        ]
    ),
    "cGcAtlasDiscovery::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDiscovery *
        ]
    ),
    "cGcAtlasDiscoveryData::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDiscoveryData *
        ]
    ),
    "cGcAtlasFeaturedBaseStateChange::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasFeaturedBaseStateChange *
        ]
    ),
    "cGcAtlasMessage::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasMessage *
        ]
    ),
    "cGcAtlasMonument::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasMonument *
        ]
    ),
    "cGcAtlasRecvBaseImages::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasRecvBaseImages *
        ]
    ),
    "cGcAtlasRecvBaseQueryList::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasRecvFeaturedBasesQueryList *
        ]
    ),
    "cGcAtlasRecvBasic::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasRecvBasic *
        ]
    ),
    "cGcAtlasRecvBlob::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasRecvBlob *
        ]
    ),
    "cGcAtlasRecvDiscoveryExact::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasRecvDiscoveryExact *
        ]
    ),
    "cGcAtlasRecvDiscoveryList::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasRecvDiscoveryList *
        ]
    ),
    "cGcThirdPersonAnimParams::SetDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcThirdPersonAnimParams *
        ]
    ),
    "cGcAlienPodAnimParams::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAlienPodAnimParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasActiveFeaturedBaseHeader::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasActiveFeaturedBaseHeader *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasBase::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBase *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasBaseQuery::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBaseQuery *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasBasesRequest::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBasesRequest *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasDiscovery::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDiscovery *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasDiscoveryData::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDiscoveryData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasDiscoveryMetadata::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDiscoveryMetadata *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasDownloadType::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDownloadType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasFeaturedBaseStateChange::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasFeaturedBaseStateChange *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasMessage::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasMonument::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasMonument *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcThirdPersonAnimParams::WriteToXMLNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcThirdPersonAnimParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cGcAtlasActiveFeaturedBaseHeader::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasActiveFeaturedBaseHeader *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAtlasActiveFeaturedBaseHeader *
        ]
    ),
    "cGcAtlasBasesRequest::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasBasesRequest *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAtlasBasesRequest *
        ]
    ),
    "cGcAtlasDiscovery::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDiscovery *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAtlasDiscovery *
        ]
    ),
    "cGcAtlasDiscoveryData::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDiscoveryData *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAtlasDiscoveryData *
        ]
    ),
    "cGcAtlasDiscoveryMetadata::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDiscoveryMetadata *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAtlasDiscoveryMetadata *
        ]
    ),
    "cGcAtlasDownloadType::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasDownloadType *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAtlasDownloadType *
        ]
    ),
    "cGcAtlasFeaturedBaseStateChange::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasFeaturedBaseStateChange *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAtlasFeaturedBaseStateChange *
        ]
    ),
    "cGcAtlasMessage::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasMessage *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAtlasMessage *
        ]
    ),
    "cGcAtlasMonument::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcAtlasMonument *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcAtlasMonument *
        ]
    ),
    "cGcThirdPersonAnimParams::WriteToXMLNodeExplicit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcThirdPersonAnimParams *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cGcThirdPersonAnimParams *
        ]
    ),
    "hkContainerAllocator::malloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkBuiltinContainerAllocatorBase::malloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "hkBuiltinContainerAllocatorBase::free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkBuiltinContainerAllocatorBase *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemory::Allocator::batchAllocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::Allocator *
            ctypes.c_ulonglong,  # void **
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "hkMemory::Allocator::batchDeallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::Allocator *
            ctypes.c_ulonglong,  # void **
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkTaskGraph::Dependency>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<float> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpDeactivatedIsland::ConstraintStateInfo>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkSerialize::HffWriter::Scope> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkReferencedObject::deleteThisReferencedObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkReferencedObject *
        ]
    ),
    "hkSingletonUtil__init_hkReferencedObjectLock_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hkReferencedObjectLock> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hkReferencedObjectLock> *
            ctypes.c_ulonglong,  # volatile signed __int64 **
        ]
    ),
    "hkSingletonUtil__quit_hkReferencedObjectLock_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hkReferencedObjectLock> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hkReferencedObjectLock> *
            ctypes.c_ulonglong,  # __int64 *
        ]
    ),
    "hkReferencedObject::addReference": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkReferencedObject *
        ]
    ),
    "hkReferencedObject::removeReference": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkReferencedObject *
        ]
    ),
    "hkReferencedObject::setReferenceCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkReferencedObject *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkViewport::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkViewport *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkRelArray_ArrayImpl<__int64>::getValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # hkRelArray_ArrayImpl<__int64> *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkReflect::ArrayType *
            ctypes.c_ulonglong,  # hkReflect::ArrayValue *
            ctypes.c_ulonglong,  # _QWORD *
        ]
    ),
    "hkRelArray_ArrayImpl<__int64>::setNumElements": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # hkRelArray_ArrayImpl<__int64> *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const hkReflect::ArrayType *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkRelArray_ArrayImpl<__int64>::inplaceFixup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # hkRelArray_ArrayImpl<__int64> *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const hkReflect::Type *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const hkReflect::Type *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkMpRational::toLogString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # const hkStringView *
        ]
    ),
    "hkQuaternionf::mul": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkQuaternionf *
            ctypes.c_ulonglong,  # const hkQuaternionf *
        ]
    ),
    "hkQuaternionf::normalize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkQuaternionf *
        ]
    ),
    "hkTransformf::set": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTransformf *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkArrayBase<hknpConvexHull::Face>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpSimdTreeBroadPhase::PointId> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkRefCountedProperties::Entry>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkRefCountedProperties::Entry> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMpRational::setNumeratorAndDemominator<int,int>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMpRational *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkOffsetBitFieldStorage<hkArray<uint,hkBuiltinContainerAllocator<0,0>>>::~hkOffsetBitFieldStorage<hkArray<uint,hkBuiltinContainerAllocator<0,0>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSet<int,hkBuiltinContainerAllocator<0,0>,hkMapOperations<int> > *
        ]
    ),
    "hkBaseObject::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkBaseObject *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkDefaultTaskQueue::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkGeometry::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkLocalFrameGroup::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkLocalFrameGroup *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkMonitorStreamColorTable::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStreamColorTable *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkMonitorStreamContainer::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStreamContainer *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkMonitorStreamStringMap::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStreamStringMap *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkMonitorStreamTypeMap::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStreamTypeMap *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkRefCountedProperties::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefCountedProperties *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkReferencedObject::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkReferencedObject *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkReferencedTask::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkReferencedTask *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkSimpleLocalFrame::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkSimpleLocalFrame *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkTask::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkTask *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkTaskQueue::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskQueue *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "set": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMpUint *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemory::memCheck4": {
        "const void *, const int, unsigned __int64, unsigned __int64, unsigned __int64 *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # const void *
                ctypes.c_int32,  # const int
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # unsigned __int64 *
            ]
        ),
        "const void *, const int, unsigned __int64, unsigned __int64 *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # const void *
                ctypes.c_int32,  # const int
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # unsigned __int64 *
            ]
        ),
    },
    "hkMemory::memCpyBackwards": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemory::memMove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemory::memSet4": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # const int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemory::memSet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # const int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkCriticalSection::hkCriticalSection": {
        "hkCriticalSection *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkCriticalSection *
            ]
        ),
        "hkCriticalSection *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkCriticalSection *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "hkHash::appendCrc64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkHash::computeCrc64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkTransformf::isApproximatelyEqual": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const float *
        ]
    ),
    "hkTransformf::setInverse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
        ]
    ),
    "hkTransformf::setMul": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
        ]
    ),
    "hkTransformf::setMulInverseMul": {
        "hkTransformf *, const hkTransformf *, const hkTransformf *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkTransformf *
                ctypes.c_ulonglong,  # const hkTransformf *
                ctypes.c_ulonglong,  # const hkTransformf *
            ]
        ),
        "hkTransformf *, const hkPreciseTransform *, const hkPreciseTransform *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkTransformf *
                ctypes.c_ulonglong,  # const hkPreciseTransform *
                ctypes.c_ulonglong,  # const hkPreciseTransform *
            ]
        ),
    },
    "hkTransformf::setMulMulInverse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
        ]
    ),
    "hkErrorFwd::messageAssert": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "hkErrorFwd::messageError": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "hkErrorFwd::messageFatalError": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkErrorFwd::messageWarning": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "hkErrorFwd::setEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkArrayBase<hknpParticleSimulationThreadContext *>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpStreamContactJacobian *> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkBaseSystem::accessLogOutput": FUNCDEF(
        restype=ctypes.c_ulonglong,  # BaseSystemOutput *
        argtypes=[]
    ),
    "hkBaseSystem::error": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hkBaseSystem::init": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkBaseSystem::Cinfo *
        argtypes=[
            ctypes.c_ulonglong,  # const hkBaseSystem::Cinfo *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkBaseSystem::initNodes": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBaseSystem::InitNode **
        argtypes=[
            ctypes.c_ulonglong,  # hkBaseSystem::InitNode **
            ctypes.c_ulonglong,  # hknpStreamContactJacobian **
        ]
    ),
    "hkBaseSystem::InitNode::internalConstruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkBaseSystem::InitNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # {'return': 'hkResult', 'args': ['void *']}
            ctypes.c_ulonglong,  # {'return': 'hkResult', 'args': ['void *']}
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkBaseSystem::isInitialized": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkBool *
        ]
    ),
    "hkBaseSystem::outputDebugString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkBaseSystem::quit": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBaseSystem::Detail *
        argtypes=[
            ctypes.c_ulonglong,  # hkBaseSystem::Detail *
        ]
    ),
    "hkBaseSystem::quitNodes": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBaseSystem::InitNode **
        argtypes=[
            ctypes.c_ulonglong,  # hkBaseSystem::InitNode **
            ctypes.c_ulonglong,  # hknpStreamContactJacobian **
        ]
    ),
    "hkMultiThreadCheck::enableChecks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMultiThreadCheck *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<void *,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkArrayBase<signed char>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkBool> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkInetAddr::hkInetAddr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkInetAddr *
        ]
    ),
    "hkInetAddr::toLogString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # const hkStringView *
        ]
    ),
    "hkArrayBase<hkPair<uint,hkArray<hkReflect::Type const *,hkBuiltinContainerAllocator<0,0>>>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<unsigned int,hkArray<hkReflect::Type const *,hkBuiltinContainerAllocator<0,0> > > > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkPair<hkReflect::Type const *,hkArray<hkReflect::Detail::AfterCache::AfterInfo,hkBuiltinContainerAllocator<0,0>>>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<hkReflect::Type const *,hkArray<hkReflect::Detail::VtableCache::VtableInfo,hkBuiltinContainerAllocator<0,0> > > > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkSurfaceConstraintInteraction>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkTuple<hkReflect::Type *,hkReflect::Type const *,void,void,void,void,void,void> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkHashMap<hkReflect::Type const *,hkArray<hkReflect::Detail::AfterCache::AfterInfo,hkBuiltinContainerAllocator<0,0>>,hkBuiltinContainerAllocator<0,0>>::findOrInsertKey<hkReflect::Type const * &>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHashBase<hkPair<hkReflect::Type const *,hkArray<hkReflect::Detail::VtableCache::VtableInfo,hkBuiltinContainerAllocator<0,0> > >,hkBuiltinContainerAllocator<0,0> >::Iterator *
        argtypes=[
            ctypes.c_ulonglong,  # hkHashMap<hkReflect::Type const *,hkArray<hkReflect::Detail::VtableCache::VtableInfo,hkBuiltinContainerAllocator<0,0> >,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkHashBase<hkPair<hkReflect::Type const *,hkArray<hkReflect::Detail::VtableCache::VtableInfo,hkBuiltinContainerAllocator<0,0> > >,hkBuiltinContainerAllocator<0,0> >::Iterator *
            ctypes.c_ulonglong,  # const hkReflect::Type **
            ctypes.c_ulonglong,  # const hkArray<hkReflect::Detail::VtableCache::VtableInfo,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkSingletonUtil__init_hkReflect__Detail__DerivedDataCache_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hkReflect::Detail::DerivedDataCache> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hkReflect::Detail::DerivedDataCache> *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hkSingletonUtil__quit_ReflectStrings_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<ReflectStrings> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<ReflectStrings> *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hkArrayBase<unsigned __int64>::_ensureOwnsStorage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkReflect::Type const * *> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hkArrayBase<hkTuple<unsigned __int64,unsigned __int64,void,void,void,void,void,void>>::_ensureOwnsStorage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkTuple<unsigned __int64,unsigned __int64,void,void,void,void,void,void> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hkHashMapDetail::Index<hkBuiltinContainerAllocator<0,0>>::reserve2": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkHashMapDetail::Index<hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_int32,  # int
        ]
    ),
    "hkSimdFloat32::ToString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkStringView *
        argtypes=[
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkSimdDouble64_FloatImpl::getValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # hkSimdDouble64_FloatImpl *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkReflect::FloatType *
            ctypes.c_ulonglong,  # hkReflect::FloatValue *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "hkArrayBase<hkReflect::Var>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkReflect::Var> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpParticleSimulationThreadContext *>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkcdSimdTree const *> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkLinearBuffer::blockAllocAligned": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # hkLinearBuffer *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkResult,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkTupleDetail::reflectToString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # const hkStringView *
        ]
    ),
    "hkRelArray_ArrayImpl<int>::getValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # hkRelArray_ArrayImpl<int> *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkReflect::ArrayType *
            ctypes.c_ulonglong,  # hkReflect::ArrayValue *
            ctypes.c_ulonglong,  # _QWORD *
        ]
    ),
    "hkRelArray_ArrayImpl<int>::setNumElements": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # hkRelArray_ArrayImpl<int> *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const hkReflect::ArrayType *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkRelArray_ArrayImpl<int>::inplaceFixup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # hkRelArray_ArrayImpl<int> *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const hkReflect::Type *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const hkReflect::Type *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<char [21],void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "assign": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # char **
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "hkQuaternionf::getAngleSr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSimdFloat32 *
        argtypes=[
            ctypes.c_ulonglong,  # hkQuaternionf *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hkQuaternionf::set": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkQuaternionf *
            ctypes.c_ulonglong,  # const hkRotationImpl<float> *
        ]
    ),
    "hkQuaternionf::setAxisAngle": {
        "hkQuaternionf *, const hkVector4f *, const hkSimdFloat32 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkQuaternionf *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ]
        ),
        "hkQuaternionf *, const hkVector4f *, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkQuaternionf *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_float,  # float
            ]
        ),
    },
    "hkQuaternionf::setFlippedRotation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkQuaternionf *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkQuaternionf::setSlerp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkQuaternionf *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
        ]
    ),
    "hkQuaternionf::setUsingEulerAngles": {
        "hkQuaternionf *, float, float, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkQuaternionf *
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
        "hkQuaternionf *, hkQuaternionAxisOrder, const hkSimdFloat32 *, const hkSimdFloat32 *, const hkSimdFloat32 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkQuaternionf *
                ctypes.c_uint32,  # hkQuaternionAxisOrder
                ctypes.c_ulonglong,  # const hkSimdFloat32 *
                ctypes.c_ulonglong,  # const hkSimdFloat32 *
                ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ]
        ),
    },
    "hkResult::ToString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkStringView *
        argtypes=[
            ctypes.c_ulonglong,  # hkResult *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkResult::reflectToString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # const hkStringView *
        ]
    ),
    "hkResult::toString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkResult *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkStreamWriter::seek": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplayHandler *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkQuaternionf *
        ]
    ),
    "hk::IsValueArray::Impl::getValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # hk::IsValueArray::Impl *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkReflect::ArrayType *
            ctypes.c_ulonglong,  # hkReflect::ArrayValue *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "hkArrayBase<hkStringView>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkStringView> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hk::Presets::assign": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Var *
        argtypes=[
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # const hkStringView *
        ]
    ),
    "hk::Presets::assignFlags": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Var *
        argtypes=[
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # const hkStringView *
        ]
    ),
    "hk::Presets::createFromParsed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Var *
        argtypes=[
            ctypes.c_ulonglong,  # hkReflect::Var *
            ctypes.c_ulonglong,  # const hkAttrData *
            ctypes.c_ulonglong,  # const hkReflect::Type *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hk::DeleteTypeInfo::deleteType": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # hkReflect::Type *
        ]
    ),
    "hk::Presets::getNameByPreset": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hk::Presets *
            ctypes.c_ulonglong,  # const hkReflect::Var *
        ]
    ),
    "hk::Presets::getNameByPresetFlags": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # hk::Presets *
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hk::Presets::getPresetByName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Var *
        argtypes=[
            ctypes.c_ulonglong,  # hk::Presets *
            ctypes.c_ulonglong,  # hkReflect::Var *
            ctypes.c_ulonglong,  # const hkStringView *
        ]
    ),
    "hk::Presets::makePresetFlagsByName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Var *
        argtypes=[
            ctypes.c_ulonglong,  # hk::Presets *
            ctypes.c_ulonglong,  # hkReflect::Var *
            ctypes.c_ulonglong,  # const hkStringView *
        ]
    ),
    "hk::Presets::reflectToString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # const hkStringView *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkStringView,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkArrayBase<uchar>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<unsigned char> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkBool>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<bool> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "parseAndAppendSinglePrintable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkStringView *
        argtypes=[
            ctypes.c_ulonglong,  # hkStringView *
            ctypes.c_ulonglong,  # const hkPrintableArgs::Vector *
            ctypes.c_ulonglong,  # _QWORD *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_int32,  # int
        ]
    ),
    "parseSignedDecimal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkStringView *
        argtypes=[
            ctypes.c_ulonglong,  # hkStringView *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "parseUnsignedDecimal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkStringView *
        argtypes=[
            ctypes.c_ulonglong,  # hkStringView *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "hkQTransformf::isApproximatelyEqual": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkQTransformf *
            ctypes.c_ulonglong,  # const hkQTransformf *
            ctypes.c_float,  # float
        ]
    ),
    "hkQTransformf::setMul": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkQTransformf *
            ctypes.c_ulonglong,  # const hkQTransformf *
            ctypes.c_ulonglong,  # const hkQTransformf *
        ]
    ),
    "hkQTransformf::setMulInverseMul": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkQTransformf *
            ctypes.c_ulonglong,  # const hkQTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
        ]
    ),
    "hkSingletonUtil__init_hkUuidPseudoRandomGenerator_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hkUuidPseudoRandomGenerator> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hkUuidPseudoRandomGenerator> *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hkUuid::reflectionToString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # const hkStringView *
        ]
    ),
    "hkRotationImpl<float>::isOrthonormal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkRotationImpl<float> *
            ctypes.c_float,  # float
        ]
    ),
    "hkRotationImpl<float>::orthonormalDeviation": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hkRotationImpl<float> *
        ]
    ),
    "hkRotationImpl<float>::renormalize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkRotationImpl<float> *
        ]
    ),
    "hkRotationImpl<float>::set": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkRotationImpl<float> *
            ctypes.c_ulonglong,  # const hkQuaternionf *
        ]
    ),
    "hkRotationImpl<float>::setAxisAngle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkRotationImpl<float> *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
        ]
    ),
    "hkVariantArray::hkVariantArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkVariantArray *
            ctypes.c_ulonglong,  # const hkVariantArray *
        ]
    ),
    "hkVariantArray::clearAndDeallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkVariantArray *
        ]
    ),
    "div": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkMpUint *
            ctypes.c_ulonglong,  # const hkMpUint *
            ctypes.c_ulonglong,  # hkMpUint *
            ctypes.c_ulonglong,  # hkMpUint *
        ]
    ),
    "toString": {
        "const hkMpRational *, unsigned int, hkStringBuf *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hkMpRational *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # hkStringBuf *
            ]
        ),
        "const hkMpUint *, unsigned int, hkStringBuf *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hkMpUint *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # hkStringBuf *
            ]
        ),
    },
    "hkArrayBase<hkStringPtr>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkStringPtr> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkStringPtr>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkStringPtr> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMonitorStreamColorTable::hkMonitorStreamColorTable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStreamColorTable *
        ]
    ),
    "hkMonitorStreamStringMap::hkMonitorStreamStringMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStreamStringMap *
        ]
    ),
    "hkMonitorStreamTypeMap::hkMonitorStreamTypeMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStreamTypeMap *
        ]
    ),
    "hkMonitorStreamStringMap::~hkMonitorStreamStringMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStreamStringMap *
        ]
    ),
    "hkMonitorStreamTypeMap::~hkMonitorStreamTypeMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStreamTypeMap *
        ]
    ),
    "hkMonitorStreamFrameInfo::hkMonitorStreamFrameInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStreamFrameInfo *
        ]
    ),
    "hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64>>::_findEmptyPositionInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMapBase<unsigned __int64,hkReflect::Type const *,hkMapOperations<unsigned __int64> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64>>::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64> > *
        ]
    ),
    "hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64>>::clearAndDeallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64>>::findKey": FUNCDEF(
        restype=ctypes.c_ulonglong,  # Dummy *
        argtypes=[
            ctypes.c_ulonglong,  # hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64> > *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64>>::get": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const unsigned __int64 *
        argtypes=[
            ctypes.c_ulonglong,  # hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64> > *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64>>::getWithDefault": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int64 *
        argtypes=[
            ctypes.c_ulonglong,  # hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64> > *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64>>::insert": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_ulonglong,  # const unsigned __int64 *
        ]
    ),
    "hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64>>::reserve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64>>::resizeTable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkContainerAllocator *
        argtypes=[
            ctypes.c_ulonglong,  # hkMapBase<unsigned __int64,hkReflect::Type const *,hkMapOperations<unsigned __int64> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_ulonglong,  # __int64 *
            ctypes.c_int32,  # int
        ]
    ),
    "hkArrayBase<hkPair<hkReflect::Type const *,unsigned __int64>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<hkReflect::Type const *,hkReflect::Type *> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMonitorStreamContainer::hkMonitorStreamContainer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStreamContainer *
        ]
    ),
    "hkMonitorStreamContainer::~hkMonitorStreamContainer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStreamContainer *
        ]
    ),
    "hkAsyncThreadPool::Cinfo::Cinfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkAsyncThreadPool::Cinfo *
        ]
    ),
    "hkAsyncThreadPool::acquireThreadSemaphore": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkAsyncThreadPool *
        ]
    ),
    "hkAsyncThreadPool::driveToForeground": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkAsyncThreadPool *
        ]
    ),
    "hkAsyncThreadPool::getCurrentThreadIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "hkAsyncThreadPool::getCurrentThreadPool": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkAsyncThreadPool *
        argtypes=[]
    ),
    "hkAsyncThreadPool::releaseThreadSemaphore": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkAsyncThreadPool *
            ctypes.c_int32,  # int
        ]
    ),
    "hkAsyncThreadPool::tendToCurrentThreadPool": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkAsyncThreadPool::tendToThreadPool": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkAsyncThreadPool *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkArrayBase<hkVector4f>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpBodyIdAndIndex> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkGeometry::Triangle>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkVector4f> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkGeometry::hkGeometry": {
        "hkGeometry *, const hkGeometry *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkGeometry *
                ctypes.c_ulonglong,  # const hkGeometry *
            ]
        ),
        "hkGeometry *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkGeometry *
            ]
        ),
    },
    "hkGeometry::~hkGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkGeometry *
        ]
    ),
    "hkGeometry::appendGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkGeometry *
        argtypes=[
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # const hkGeometry *
            ctypes.c_ulonglong,  # const hkMatrix4Impl<float> *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "hkGeometry::appendVertex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkGeometry::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkGeometry *
        ]
    ),
    "hkLocalFrameGroup::hkLocalFrameGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkLocalFrameGroup *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkSimpleLocalFrame::hkSimpleLocalFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSimpleLocalFrame *
        ]
    ),
    "hkSimpleLocalFrame::getChildFrame": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkLocalFrame *
        argtypes=[
            ctypes.c_ulonglong,  # hkSimpleLocalFrame *
            ctypes.c_int32,  # int
        ]
    ),
    "hkLocalFrame::getDescendants": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkLocalFrame *
            ctypes.c_ulonglong,  # hkArrayBase<hkLocalFrame const *> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hkSimpleLocalFrame::getLocalPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSimpleLocalFrame *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkMemoryMeshBody::getTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshBody *
            ctypes.c_ulonglong,  # hkMatrix4Impl<float> *
        ]
    ),
    "hkMemoryMeshBody::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshBody *
        ]
    ),
    "hkSimpleLocalFrame::getNearbyFrames": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSimpleLocalFrame *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkLocalFrameCollector *
        ]
    ),
    "hkSimpleLocalFrame::getNumChildFrames": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkSimpleLocalFrame *
        ]
    ),
    "hkSimpleLocalFrame::setGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSimpleLocalFrame *
            ctypes.c_ulonglong,  # const hkLocalFrameGroup *
        ]
    ),
    "hkSimpleLocalFrame::setLocalTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshBody *
            ctypes.c_ulonglong,  # const hkMatrix4Impl<float> *
        ]
    ),
    "hkSimpleLocalFrame::setParentFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkRefCountedProperties::Entry>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkRefCountedProperties::Entry> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkRefCountedProperties::hkRefCountedProperties": {
        "hkRefCountedProperties *, const hkRefCountedProperties *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkRefCountedProperties *
                ctypes.c_ulonglong,  # const hkRefCountedProperties *
            ]
        ),
        "hkRefCountedProperties *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkRefCountedProperties *
            ]
        ),
    },
    "hkMemory::AllocatorStack::AllocatorStack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::AllocatorStack *
            ctypes.c_ulonglong,  # hkMemory::Allocator *
        ]
    ),
    "hkMemory::AllocatorStack::pop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::AllocatorStack *
        ]
    ),
    "hkMemory::AllocatorStack::push": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkMemory::AllocatorStackFrame *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::AllocatorStack *
        ]
    ),
    "hkMemory::Detail::FwdToSharedAllocator::malloc": {
        "hkMemory::Detail::FwdToSharedAllocator *, unsigned __int64": FUNCDEF(
            restype=ctypes.c_ulonglong,  # void *
            argtypes=[
                ctypes.c_ulonglong,  # hkMemory::Detail::FwdToSharedAllocator *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
        "hkMemory::Detail::FwdToSharedAllocator *, unsigned __int64, unsigned __int64 *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # void *
            argtypes=[
                ctypes.c_ulonglong,  # hkMemory::Detail::FwdToSharedAllocator *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # unsigned __int64 *
            ]
        ),
    },
    "hkMemory::Detail::FwdToSharedAllocator::free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::Detail::FwdToSharedAllocator *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemory::Group::purgeDeallocationQueuesInternal<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkMemory::Group::purgeDeallocationQueuesInternal<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkMemory::Detail::DebugDataFuncs::checkForMemoryBudgetRestored": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::Detail::DebugDataFuncs *
            ctypes.c_ulonglong,  # const hkMemory::Allocator *
            ctypes.c_uint32,  # hkMemoryType
        ]
    ),
    "hkQueue<hkTuple<hkMemoryType,void *,void,void,void,void,void,void>,hkMemory::Detail::FwdToSharedAllocator>::clearAndDeallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkQueue<hkTuple<enum hkMemoryType,void *,void,void,void,void,void,void>,hkMemory::Detail::FwdToSharedAllocator> *
        ]
    ),
    "hkMemory::Detail::deleteDebugThreadData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::Allocator *
            ctypes.c_ulonglong,  # hkMemory::Detail::DebugThreadData *
        ]
    ),
    "hkMemory::Detail::DebugCrossThreadData::deleteThisReferencedObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::Detail::DebugCrossThreadData *
        ]
    ),
    "hkMemory::Detail::AllocationDebugInfo::destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::Detail::AllocationDebugInfo *
        ]
    ),
    "hkMemory::Detail::AllocationDebugInfo::Data::TrackingData::destroyBookmark": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::Detail::AllocationDebugInfo::Data::TrackingData *
        ]
    ),
    "hkMemory::Detail::destroyMetaAndDeallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::Allocator *
            ctypes.c_ulonglong,  # hkMemory::Allocator::DebugInterface::DeallocParams *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMemory::ForwardingContainerAllocator::free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::ForwardingContainerAllocator *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemory::Detail::freeDebugCrossThreadData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::Detail *
        ]
    ),
    "hkMemory::Detail::freeWithDebugHeader": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::Allocator *
            ctypes.c_ulonglong,  # hkMemory::Allocator::DebugInterface::DeallocParams *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMemory::Detail::getDebugThreadDataStats": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkMemory::Detail::ThreadStatsStorage *
        argtypes=[
            ctypes.c_ulonglong,  # const hkMemory::Detail::DebugThreadData *
            ctypes.c_uint32,  # hkMemory::GroupId
        ]
    ),
    "hkMemory::Detail::initDebugCrossThreadData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::Detail *
        ]
    ),
    "hkMemory::ForwardingContainerAllocator::malloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::ForwardingContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "hkMemory::Detail::newDebugThreadData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkMemory::Detail::DebugThreadData *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::Allocator *
        ]
    ),
    "hkMemory::Detail::DeallocationsQueue<hkMemory::Detail::AllocationDebugInfo::Data::TrackingData *,&hkMemory::Detail::PurgeDeallocationTraces": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::Detail::DeallocationsQueue<hkMemory::Detail::AllocationDebugInfo::Data::TrackingData *,&hkMemory::Detail::PurgeDeallocationTraces> *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkMemory::Detail::DeallocationsQueue<hkTuple<hkMemoryType,void *,void,void,void,void,void,void>,&hkMemory::Detail::PurgeDelayedDeallocations": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::Detail::DeallocationsQueue<hkTuple<enum hkMemoryType,void *,void,void,void,void,void,void>,&hkMemory::Detail::PurgeDelayedDeallocations> *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkMemory::Detail::purgeAllDeallocationTraces": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkMemory::Group::purgeAllDelayedFrees": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkMemory::Detail::AllocationDebugInfo::Data::TrackingData::untrack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::Detail::AllocationDebugInfo::Data::TrackingData *
            ctypes.c_ulonglong,  # hkMemory::Detail::DebugData *
            ctypes.c_uint32,  # hkMemoryType
        ]
    ),
    "hkThreadLocalDataBase::hkThreadLocalDataBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkThreadLocalDataBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkBaseSystem::ThreadContext::ThreadContextContainer::~ThreadContextContainer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkBaseSystem::ThreadContext::ThreadContextContainer *
        ]
    ),
    "hkBaseSystem::Detail::allocateTlsTableDataIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkBaseSystem::ThreadContext::createAndStoreNewContext": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBaseSystem::ThreadContextImpl *
        argtypes=[]
    ),
    "hkBaseSystem::ThreadContextImpl::freeContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkBaseSystem::ThreadContextImpl *
        ]
    ),
    "hkBaseSystem::Detail::getThreadContextSystemSharedAllocator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkMemory::Allocator *
        argtypes=[]
    ),
    "hkBaseSystem::ThreadContext::getThreadContexts": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBaseSystem::ThreadContext::ThreadContextContainer *
        argtypes=[
            ctypes.c_ulonglong,  # hkBaseSystem::ThreadContext::ThreadContextContainer *
        ]
    ),
    "hkBaseSystem::Detail::initThreadContextSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::Allocator *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['hkBaseSystem::ThreadContext *', 'void *']}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['hkBaseSystem::ThreadContext *']}
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkBaseSystem::Detail::quitThreadContextSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkBaseSystem::Detail *
        ]
    ),
    "hkBaseSystem::ThreadContextImpl::quitThreadContextSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "hkBaseSystem::Detail::setImplCb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkBaseSystem::ThreadContext *
        ]
    ),
    "hkSignal::destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal *
        ]
    ),
    "hkSignal::getNumSubscriptions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal *
        ]
    ),
    "hkSignal::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal *
        ]
    ),
    "hkSignal::unsubscribeAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hkSignal::unsubscribeInternal": {
        "hkSignal *, hkBool *, void *, const void *, int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkBool *
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal *
                ctypes.c_ulonglong,  # hkBool *
                ctypes.c_ulonglong,  # void *
                ctypes.c_ulonglong,  # const void *
                ctypes.c_int32,  # int
            ]
        ),
        "hkSignal *, hkBool *, void *, const void *, int, const char *, int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkBool *
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal *
                ctypes.c_ulonglong,  # hkBool *
                ctypes.c_ulonglong,  # void *
                ctypes.c_ulonglong,  # const void *
                ctypes.c_int32,  # int
                ctypes.c_char_p,  # const char *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "SetThreadNameUsingException": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkThread::getMyThreadId": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[]
    ),
    "hkThread::setPriority": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkThread *
            ctypes.c_int32,  # int
        ]
    ),
    "hkHashMapDetail::Entry::getSentinel": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHashMapDetail::Entry *
        argtypes=[]
    ),
    "hkHashMapDetail::setSentinel_OOL": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hkSingletonUtil__init_hkHardwareInfo_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hkHardwareInfo> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hkHardwareInfo> *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hkHardwareInfo::hkHardwareInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkHardwareInfo *
        ]
    ),
    "hkHardwareInfo::~hkHardwareInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkHardwareInfo *
        ]
    ),
    "hkHardwareInfo::calcNumHardwareThreads": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hkMatrix3Impl<float>::changeBasis": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # const hkRotationImpl<float> *
        ]
    ),
    "hkMatrix3Impl<float>::diagonalizeSymmetric": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRotationImpl<float> *
        argtypes=[
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # hkRotationImpl<float> *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkMatrix3Impl<float>::diagonalizeSymmetricApproximation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # hkRotationImpl<float> *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMatrix3Impl<float>::isApproximatelyEqual": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # const hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # const float *
        ]
    ),
    "hkMatrix3Impl<float>::mul": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # const hkMatrix3Impl<float> *
        ]
    ),
    "hkMatrix3Impl<float>::setCrossSkewSymmetric": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkMatrix3Impl<float>::setInverse": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkMatrix3Impl<float> *
        argtypes=[
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # const hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "hkMatrix3Impl<float>::setMul": {
        "hkMatrix3Impl<float> *, const hkMatrix3Impl<float> *, const hkMatrix3Impl<float> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
                ctypes.c_ulonglong,  # const hkMatrix3Impl<float> *
                ctypes.c_ulonglong,  # const hkMatrix3Impl<float> *
            ]
        ),
        "hkMatrix3Impl<float> *, const hkQuaternionf *, const hkMatrix3Impl<float> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
                ctypes.c_ulonglong,  # const hkQuaternionf *
                ctypes.c_ulonglong,  # const hkMatrix3Impl<float> *
            ]
        ),
    },
    "hkMatrix3Impl<float>::setMulInverse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # const hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # const hkRotationImpl<float> *
        ]
    ),
    "hkMatrix3Impl<float>::setTranspose": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # const hkMatrix3Impl<float> *
        ]
    ),
    "hkMatrix3Impl<float>::transpose": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
        ]
    ),
    "hkVector4f::ToString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkStringView *
        argtypes=[
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkVector4f::setRotatedDir": {
        "hkVector4f *, const hkMatrix3Impl<float> *, const hkVector4f *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkVector4f *
                ctypes.c_ulonglong,  # const hkMatrix3Impl<float> *
                ctypes.c_ulonglong,  # const hkVector4f *
            ]
        ),
        "hkVector4f *, const hkQuaternionf *, const hkVector4f *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkVector4f *
                ctypes.c_ulonglong,  # const hkQuaternionf *
                ctypes.c_ulonglong,  # const hkVector4f *
            ]
        ),
    },
    "hkVector4f::setRotatedInverseDir": {
        "hkVector4f *, const hkMatrix3Impl<float> *, const hkVector4f *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkVector4f *
                ctypes.c_ulonglong,  # const hkMatrix3Impl<float> *
                ctypes.c_ulonglong,  # const hkVector4f *
            ]
        ),
        "hkVector4f *, const hkQuaternionf *, const hkVector4f *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkVector4f *
                ctypes.c_ulonglong,  # const hkQuaternionf *
                ctypes.c_ulonglong,  # const hkVector4f *
            ]
        ),
    },
    "hkVector4f::setTransformedInversePos": {
        "hkVector4f *, const hkQTransformf *, const hkVector4f *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkVector4f *
                ctypes.c_ulonglong,  # const hkQTransformf *
                ctypes.c_ulonglong,  # const hkVector4f *
            ]
        ),
        "hkVector4f *, const hkTransformf *, const hkVector4f *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkVector4f *
                ctypes.c_ulonglong,  # const hkTransformf *
                ctypes.c_ulonglong,  # const hkVector4f *
            ]
        ),
    },
    "hkVector4f::setTransformedPos": {
        "hkVector4f *, const hkPreciseTransform *, const hkVector4f *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkVector4f *
                ctypes.c_ulonglong,  # const hkPreciseTransform *
                ctypes.c_ulonglong,  # const hkVector4f *
            ]
        ),
        "hkVector4f *, const hkQTransformf *, const hkVector4f *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkVector4f *
                ctypes.c_ulonglong,  # const hkQTransformf *
                ctypes.c_ulonglong,  # const hkVector4f *
            ]
        ),
        "hkVector4f *, const hkQsTransformf *, const hkVector4f *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkVector4f *
                ctypes.c_ulonglong,  # const hkQsTransformf *
                ctypes.c_ulonglong,  # const hkVector4f *
            ]
        ),
        "hkVector4f *, const hkTransformf *, const hkVector4f *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkVector4f *
                ctypes.c_ulonglong,  # const hkTransformf *
                ctypes.c_ulonglong,  # const hkVector4f *
            ]
        ),
    },
    "hkArrayBase<hkLog::Origin::OutputLevelPair>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkLog::Origin::OutputLevelPair> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkRefPtr<hkLog::Output>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkRefPtr<hknpConstraintManager::ConstraintName> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkSingletonUtil__init__anonymous_namespace___Registry_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<Registry> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<Registry> *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hkArrayUtil::reduce<hkLog::Origin::OutputLevelPair>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # hkLog::Origin::OutputLevelPair *
            ctypes.c_int32,  # int
        ]
    ),
    "hkHashSet<int,hkBuiltinContainerAllocator<0,0>>::tryInsert<int const &>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHashSet<int,hkBuiltinContainerAllocator<0,0> >::TryInsertResult *
        argtypes=[
            ctypes.c_ulonglong,  # hkHashSet<int,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkHashSet<int,hkBuiltinContainerAllocator<0,0> >::TryInsertResult *
            ctypes.c_ulonglong,  # const int *
        ]
    ),
    "hkLog::Detail::AutoMessage::AutoMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Message *
            ctypes.c_ulonglong,  # const hkLog::Origin *
            ctypes.c_uint32,  # hkLog::Level
        ]
    ),
    "hkLog::Detail::AutoScope::AutoScope": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Detail::AutoScope *
            ctypes.c_ulonglong,  # const hkLog::Origin *
            ctypes.c_ulonglong,  # hkPrintableArgs::Vector *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
        ]
    ),
    "hkLog::Detail::DebugOutputMessage::DebugOutputMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Detail::DebugOutputMessage *
            ctypes.c_ulonglong,  # const hkLog::Origin *
            ctypes.c_uint32,  # hkLog::Level
        ]
    ),
    "hkLog::Output::Output": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Output *
        ]
    ),
    "hkLog::RegisteredOrigin::RegisteredOrigin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::RegisteredOrigin *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkLog::Detail::AutoMessage::~AutoMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Detail::AutoMessage *
        ]
    ),
    "hkLog::Detail::AutoScope::~AutoScope": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Detail::AutoScope *
        ]
    ),
    "hkLog::Detail::DebugOutputMessage::~DebugOutputMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Detail::DebugOutputMessage *
        ]
    ),
    "hkLog::Message::~Message": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Message *
        ]
    ),
    "hkLog::Origin::~Origin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Origin *
        ]
    ),
    "hkStreamWriter::~hkStreamWriter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkBlockStreamAllocator *
        ]
    ),
    "hkLog::Message::addFormatV": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkLog::Message *
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Message *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const hkPrintableArgs::Vector *
            ctypes.c_int32,  # int
        ]
    ),
    "hkLog::Detail::addModuleLocalList": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkLog::Detail::ModuleLocalList *
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Detail::ModuleLocalList *
            ctypes.c_ulonglong,  # hkLog::Origin::OutputLevelPair *
        ]
    ),
    "hkLog::Origin::beginOutput": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkLog::Origin::OutputIterator *
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Origin *
            ctypes.c_ulonglong,  # hkLog::Origin::OutputIterator *
        ]
    ),
    "hkLog::Origin::connectOutput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Origin *
            ctypes.c_ulonglong,  # hkLog::Output *
            ctypes.c_uint32,  # hkLog::Level
        ]
    ),
    "hkLog::Origin::disconnectAllOutputs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Origin *
        ]
    ),
    "hkLog::Detail::dispatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Message *
        ]
    ),
    "hkLog::Origin::endOutput": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkLog::Origin::OutputIterator *
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Origin *
            ctypes.c_ulonglong,  # hkLog::Origin::OutputIterator *
        ]
    ),
    "hkSignal1<hknpAction *>::fire": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal1<hknpAction *> *
            ctypes.c_ulonglong,  # hknpAction *
        ]
    ),
    "hkLog::flush": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkLog *
        ]
    ),
    "hkLog::formatLogMessage": {
        "const hkLog::Message *, hkIo::WriteBuffer *, int, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hkLog::Message *
                ctypes.c_ulonglong,  # hkIo::WriteBuffer *
                ctypes.c_int32,  # int
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const hkLog::Message *, hkStringBuf *, int, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hkLog::Message *
                ctypes.c_ulonglong,  # hkStringBuf *
                ctypes.c_int32,  # int
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "hkLog::Detail::getGlobalInitNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBaseSystem::InitNode *
        argtypes=[]
    ),
    "hkLog::Detail::removeModuleLocalList": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkLog::Detail::ModuleLocalList *
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Detail::ModuleLocalList *
            ctypes.c_ulonglong,  # hkLog::Detail::ModuleLocalList *
        ]
    ),
    "hkLog::Detail::rerouteToDebugOutput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkStringBuf,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkLog::Message::setText": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkLog::Message *
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Message *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkLog::Detail::shouldRerouteToDebugOutput": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[]
    ),
    "hkLog::Detail::AutoScope::show": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkLog::Detail::AutoScope *
            ctypes.c_ulonglong,  # hkLog::Output *
            ctypes.c_uint32,  # hkLog::Level
        ]
    ),
    "hkIo::WriteBuffer::write8": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::WriteBuffer *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "hkIo::WriteBuffer::writeRaw": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::WriteBuffer *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkSingletonUtil__initWith_hkError_hkDefaultError_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hkError> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hkError> *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hkDisableError::hkDisableError": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDisableError *
            ctypes.c_int32,  # int
        ]
    ),
    "hkErrStream::hkErrStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkErrStream *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
        ]
    ),
    "hkError::hkError": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkError *
        ]
    ),
    "hkDisableError::~hkDisableError": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDisableError *
        ]
    ),
    "hkErrStream::~hkErrStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkErrStream *
        ]
    ),
    "hkErrorMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkSingletonUtil__init_hkErrorRouter_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hkErrorRouter> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hkErrorRouter> *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hkBufferedStreamWriter::isBuffered": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkWin32StreamWriter *
            ctypes.c_ulonglong,  # hkBool *
        ]
    ),
    "hkErrorRouter::message": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkErrorRouter *
            ctypes.c_uint32,  # hkError::Message
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "hkArrayBase<hkMemory::AllocatorStats>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpConstraintGroupDirectSolver::LimitsEnforcedForSolving> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemory::FreeListAllocator::FreeListAllocator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::FreeListAllocator *
            ctypes.c_ulonglong,  # hkMemory::Allocator *
        ]
    ),
    "hkMemory::FreeListAllocator::~FreeListAllocator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::FreeListAllocator *
        ]
    ),
    "hkMemory::FreeListAllocatorImpl::FixedSizeAllocator::allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::FreeListAllocatorImpl::FixedSizeAllocator *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "hkMemory::FreeListAllocator::allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::FreeListAllocator *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "hkMemory::FreeListAllocatorImpl::FixedSizeAllocator::batchAllocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::FreeListAllocatorImpl::FixedSizeAllocator *
            ctypes.c_ulonglong,  # void **
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "hkMemory::FreeListAllocator::batchAllocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::FreeListAllocator *
            ctypes.c_ulonglong,  # void **
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "hkMemory::FreeListAllocatorImpl::FixedSizeAllocator::batchDeallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::FreeListAllocatorImpl::FixedSizeAllocator *
            ctypes.c_ulonglong,  # void **
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemory::FreeListAllocator::batchDeallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::FreeListAllocator *
            ctypes.c_ulonglong,  # void **
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemory::FreeListAllocator::collectGarbage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::FreeListAllocator *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMemory::FreeListAllocatorImpl::FixedSizeAllocator::deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::FreeListAllocatorImpl::FixedSizeAllocator *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemory::FreeListAllocator::deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::FreeListAllocator *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemory::FreeListAllocator::getAllocatorStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::FreeListAllocator *
            ctypes.c_ulonglong,  # hkArray<hkMemory::AllocatorStats,hkBuiltinContainerAllocator<2,0> > *
        ]
    ),
    "hkMemory::FreeListAllocator::getSystemStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::FreeListAllocator *
            ctypes.c_ulonglong,  # hkMemory::SystemStats *
        ]
    ),
    "hkMemory::FreeListAllocator::resetPeakInUse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::FreeListAllocator *
        ]
    ),
    "hkMemory::FreeListAllocatorImpl::setFixedSizeCinfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # hkMemory::FreeListAllocatorImpl::Cinfo *
        ]
    ),
    "hkMath::getFlushDenormalsInitNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBaseSystem::InitNode *
        argtypes=[]
    ),
    "hkMath::setFlushDenormals": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_char,  # char
        ]
    ),
    "hkSystemClock::getTickCounter": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[]
    ),
    "hkSystemClock::getTicksPerSecond": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[]
    ),
    "hkMemory::MallocAllocator::~MallocAllocator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::MallocAllocator *
        ]
    ),
    "hkMemory::MallocAllocator::allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::MallocAllocator *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "hkMemory::MallocAllocator::deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::MallocAllocator *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemory::MallocAllocator::getAllocatorStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::MallocAllocator *
            ctypes.c_ulonglong,  # hkArray<hkMemory::AllocatorStats,hkBuiltinContainerAllocator<2,0> > *
        ]
    ),
    "hkMemory::MallocAllocator::getSystemStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::MallocAllocator *
            ctypes.c_ulonglong,  # hkMemory::SystemStats *
        ]
    ),
    "hkMemory::MallocAllocator::resetPeakInUse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::MallocAllocator *
        ]
    ),
    "hkFileSystem::mkdirRecursive": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # hkFileSystem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkSingletonUtil__initWith_hkFileSystem_hkWin32FileSystem_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hkFileSystem> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hkFileSystem> *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hkHashSet<unsigned __int64,hkBuiltinContainerAllocator<0,0>>::tryInsert<unsigned __int64 const &>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHashSet<hkReflect::Type const *,hkBuiltinContainerAllocator<0,0> >::TryInsertResult *
        argtypes=[
            ctypes.c_ulonglong,  # hkHashSet<hkReflect::Type const *,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkHashSet<hkReflect::Type const *,hkBuiltinContainerAllocator<0,0> >::TryInsertResult *
            ctypes.c_ulonglong,  # const hkReflect::Type *const *
        ]
    ),
    "hkFileSystem::Entry::Entry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkFileSystem::Entry *
        ]
    ),
    "hkFileSystem::Watcher::Watcher": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkFileSystem::Watcher *
        ]
    ),
    "hkFileSystem::Entry::~Entry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkFileSystem::Entry *
        ]
    ),
    "hkFileSystem::~hkFileSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkFileSystem *
        ]
    ),
    "hkFileSystem::_handleFlags": {
        "hkFileSystem *, hkStreamReader *, hkFileSystem::OpenFlags": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkStreamReader *
            argtypes=[
                ctypes.c_ulonglong,  # hkFileSystem *
                ctypes.c_ulonglong,  # hkStreamReader *
                ctypes.c_uint32,  # hkFileSystem::OpenFlags
            ]
        ),
        "hkFileSystem *, hkStreamWriter *, hkFileSystem::OpenFlags": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkStreamWriter *
            argtypes=[
                ctypes.c_ulonglong,  # hkFileSystem *
                ctypes.c_ulonglong,  # hkStreamWriter *
                ctypes.c_uint32,  # hkFileSystem::OpenFlags
            ]
        ),
    },
    "hkFileSystem::Watcher::combineMatchingChanges": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArray<hkFileSystem::Watcher::Change,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkFileSystem::copy": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkFileSystem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkFileSystem::remove": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkFileSystem *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkFileSystem::Iterator::nameAcceptable": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkFileSystem::openReader": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefNew<hkStreamReader> *
        argtypes=[
            ctypes.c_ulonglong,  # hkFileSystem *
            ctypes.c_ulonglong,  # hkRefNew<hkStreamReader> *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # hkFileSystem::OpenFlags
        ]
    ),
    "hkFileSystem::openWriter": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefNew<hkStreamWriter> *
        argtypes=[
            ctypes.c_ulonglong,  # hkFileSystem *
            ctypes.c_ulonglong,  # hkRefNew<hkStreamWriter> *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # hkFileSystem::OpenFlags
        ]
    ),
    "hkSocketCreate": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "hkSocket::hkSocket": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSocket *
        ]
    ),
    "hkSocket::~hkSocket": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSocket *
        ]
    ),
    "hkSocket::getAddress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSocket *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hkSocket::WriterAdapter::isOk": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkSocket::WriterAdapter *
            ctypes.c_ulonglong,  # hkBool *
        ]
    ),
    "hkSocket::ReaderAdapter::read": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkSocket::ReaderAdapter *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkSocket::WriterAdapter::write": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkSocket::WriterAdapter *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase__anonymous_namespace___IsaInfo____growCapacity_hkBuiltinContainerAllocatorBase_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase_hkReflect__Detail___anonymous_namespace___Entry____growCapacity_hkContainerAllocator_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkReflect::Detail::BuiltinTypeReg::SubscriptionImpl *>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkBlockStream::Block *> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkSingletonUtil__init_hkReflect__Detail__BuiltinTypeReg_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hkReflect::Detail::BuiltinTypeReg> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hkReflect::Detail::BuiltinTypeReg> *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hkHashSet<hkReflect::TypeName,hkBuiltinContainerAllocator<0,0>>::tryInsert<hkReflect::TypeName const &>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHashSet<hkReflect::TypeName,hkBuiltinContainerAllocator<0,0> >::TryInsertResult *
        argtypes=[
            ctypes.c_ulonglong,  # hkHashSet<hkReflect::TypeName,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkHashSet<hkReflect::TypeName,hkBuiltinContainerAllocator<0,0> >::TryInsertResult *
            ctypes.c_ulonglong,  # const hkReflect::TypeName *
        ]
    ),
    "hkMap<unsigned __int64,hkReflect::Type const *,hkMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0>>::~hkMap<unsigned __int64,hkReflect::Type const *,hkMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMap<unsigned __int64,hkReflect::Type const *,hkMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkMap<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<2,0>>::~hkMap<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<2,0>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMap<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<2,0> > *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::TypeName,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "s_updateAlignment": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "hkStackTracer::hkStackTracer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkStackTracer *
        ]
    ),
    "hkStackTracer::~hkStackTracer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkStackTracer *
        ]
    ),
    "hkStackTracer::dumpStackTrace": {
        "hkStackTracer *, const unsigned __int64 *, int, __int64 *, void *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkStackTracer *
                ctypes.c_ulonglong,  # const unsigned __int64 *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # {'return': 'void', 'args': ['const char *', 'void *']}
                ctypes.c_ulonglong,  # void *
            ]
        ),
        "hkStackTracer *, const unsigned __int64 *, int, char *, unsigned __int64": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # hkStackTracer *
                ctypes.c_ulonglong,  # const unsigned __int64 *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # char *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
    },
    "hkStackTracerImpl::ensureInitialized": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkStackTracerImpl *
        ]
    ),
    "hkStackTracer::getStackTrace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkStackTracer *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hkHash::computeFNV1Hash32": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<char [2],void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<short,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<ushort,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<int,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<uint,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<float,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<char const *,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<void const *,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<__int64,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<unsigned __int64,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkMath::Detail::countBitsSetGeneric": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkMath::ceil": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "hkMath__checkPopCntSupport": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hkMath___anonymous_namespace___countBitsSetPopCnt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkMath::cubeRootApprox22Bit": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # const float
        ]
    ),
    "hkMath::doubleToFloat": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkMath::quadAsin": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkMath::quadAtan2": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkMath::quadCos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkMath::quadExp": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkMath::quadLog": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkMath::quadPow": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkMath::quadSin": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkMath::quadSinCos": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkArrayBase<hkMemory::LifoAllocatorImpl::Data::NonLifoFree>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkAabbFloat3> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemory::LifoAllocator::LifoAllocator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::LifoAllocator *
            ctypes.c_ulonglong,  # hkMemory::Allocator *
            ctypes.c_ulonglong,  # hkMemory::Allocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemory::LifoAllocator::allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::LifoAllocator *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "hkMemory::LifoAllocator::allocateFromNewSlab": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::LifoAllocator *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "hkMemory::LifoAllocator::deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::LifoAllocator *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemory::LifoAllocator::getSystemStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::LifoAllocator *
            ctypes.c_ulonglong,  # hkMemory::SystemStats *
        ]
    ),
    "hkMemory::LifoAllocator::insertNonLifoFree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::LifoAllocator *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemory::LifoAllocator::popNonLifoFrees": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::LifoAllocator *
        ]
    ),
    "hkMemory::LifoAllocator::resetPeakInUse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::LifoAllocator *
        ]
    ),
    "hkMemory::LifoAllocator::slowBlockFree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::LifoAllocator *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkAssert::defaultAssertHandler": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkAssert::defaultDisableAssert": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "hkAssert::assertFailed": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hkAssert::Args *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkOstream::hkOstream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkOstream *
            ctypes.c_ulonglong,  # hkStreamWriter *
        ]
    ),
    "hkOstream::~hkOstream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hkOstream::operator<<": {
        "hkOstream *, const hkMatrix3Impl<float> *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkOstream *
            argtypes=[
                ctypes.c_ulonglong,  # hkOstream *
                ctypes.c_ulonglong,  # const hkMatrix3Impl<float> *
            ]
        ),
        "hkOstream *, const hkQuaternionf *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkOstream *
            argtypes=[
                ctypes.c_ulonglong,  # hkOstream *
                ctypes.c_ulonglong,  # const hkQuaternionf *
            ]
        ),
        "hkOstream *, const hkStringPtr *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkOstream *
            argtypes=[
                ctypes.c_ulonglong,  # hkOstream *
                ctypes.c_ulonglong,  # const hkStringPtr *
            ]
        ),
        "hkOstream *, const hkVector4d *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkOstream *
            argtypes=[
                ctypes.c_ulonglong,  # hkOstream *
                ctypes.c_ulonglong,  # const hkVector4d *
            ]
        ),
        "hkOstream *, const hkVector4f *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkOstream *
            argtypes=[
                ctypes.c_ulonglong,  # hkOstream *
                ctypes.c_ulonglong,  # const hkVector4f *
            ]
        ),
        "hkOstream *, unsigned __int16": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkOstream *
            argtypes=[
                ctypes.c_ulonglong,  # hkOstream *
                ctypes.c_uint16,  # unsigned __int16
            ]
        ),
        "hkOstream *, int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkOstream *
            argtypes=[
                ctypes.c_ulonglong,  # hkOstream *
                ctypes.c_int32,  # int
            ]
        ),
        "hkOstream *, unsigned int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkOstream *
            argtypes=[
                ctypes.c_ulonglong,  # hkOstream *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "hkOstream *, float": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkOstream *
            argtypes=[
                ctypes.c_ulonglong,  # hkOstream *
                ctypes.c_float,  # float
            ]
        ),
        "hkOstream *, const char *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkOstream *
            argtypes=[
                ctypes.c_ulonglong,  # hkOstream *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "hkOstream *, const void *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkOstream *
            argtypes=[
                ctypes.c_ulonglong,  # hkOstream *
                ctypes.c_ulonglong,  # const void *
            ]
        ),
        "hkOstream *, hkBool": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkOstream *
            argtypes=[
                ctypes.c_ulonglong,  # hkOstream *
                ctypes.c_ubyte,  # hkBool
            ]
        ),
        "hkOstream *, unsigned __int64": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkOstream *
            argtypes=[
                ctypes.c_ulonglong,  # hkOstream *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
    },
    "hkOstream::printf": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkOstream *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkArrayBase<hkTypeVm::Detail::DeferredString>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkTypeVm::Detail::DeferredString> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkReflect::VarN>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkReflect::VarN> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase__anonymous_namespace___Pair_hkReflect__PointerVar_hkReflect__PointerVar______growCapacity_hkContainerAllocator_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase__anonymous_namespace___Pair_hkReflect__Var_hkReflect__Var______growCapacity_hkContainerAllocator_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkPair<hkReflect::Var,hkReflect::Var>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<hkReflect::Var,hkReflect::Var> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkSingletonUtil__init__anonymous_namespace___AfterReflectNewLock_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<AfterReflectNewLock> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<AfterReflectNewLock> *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::BoolVar,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::FloatVar,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::IntVar,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::StringVar,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::Var,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::QualifiedType<hkReflect::ArrayType>,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::QualifiedType<hkReflect::BoolType>,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::QualifiedType<hkReflect::FloatType>,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::QualifiedType<hkReflect::IntType>,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::QualifiedType<hkReflect::OpaqueType>,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::QualifiedType<hkReflect::PointerType>,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::QualifiedType<hkReflect::RecordType>,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::QualifiedType<hkReflect::StringType>,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::QualifiedType<hkReflect::Type>,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::QualifiedType<hkReflect::VoidType>,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::Decl,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::FieldDecl,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::Kind,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkSerialize::Detail::FlatTypeRelocs::FlatTypeRelocs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::FlatTypeRelocs *
        ]
    ),
    "hkMemory::BufferedAllocatorNoCleanup::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::BufferedAllocatorNoCleanup *
        ]
    ),
    "hkMemory::BufferedAllocatorNoCleanup::clearAndDeallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::BufferedAllocatorNoCleanup *
        ]
    ),
    "hkMemory::BufferedAllocatorNoCleanup::free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::BufferedAllocatorNoCleanup *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemory::BufferedAllocatorNoCleanup::malloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::BufferedAllocatorNoCleanup *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "hkRelPtr_PointerImpl::getValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # hkRelPtr_PointerImpl *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkReflect::PointerType *
            ctypes.c_ulonglong,  # hkReflect::Var *
            ctypes.c_ulonglong,  # _QWORD *
        ]
    ),
    "hkRelPtr_PointerImpl::setValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # hkRelPtr_PointerImpl *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const hkReflect::PointerType *
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hkRelPtr_PointerImpl::inplaceFixup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # hkRelPtr_PointerImpl *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const hkReflect::Type *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "hkRelArrayUtil::hkRelArrayUtil": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkRelArrayUtil *
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "hkRelArrayUtil::allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReferencedObject *
        argtypes=[
            ctypes.c_ulonglong,  # hkRelArrayUtil *
        ]
    ),
    "hkFloatParseUtil::parseFloat": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkStringView *
        argtypes=[
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # long double *
            ctypes.c_ulonglong,  # double *
        ]
    ),
    "s_typeFromValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hkStringView *
        ]
    ),
    "hkVarArgs::Detail::pointersFromVaList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const void **
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "hkAttributeParser::hkAttributeParser": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkAttributeParser *
            ctypes.c_ulonglong,  # const hkStringView *
        ]
    ),
    "hkAttributeParser::advance": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # hkAttributeParser *
        ]
    ),
    "hkAttributeParser::currentAttrName": {
        "hkAttributeParser *, hkStringBuf *, hkStringBuf *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkStringBuf *
            argtypes=[
                ctypes.c_ulonglong,  # hkAttributeParser *
                ctypes.c_ulonglong,  # hkStringBuf *
                ctypes.c_ulonglong,  # hkStringBuf *
            ]
        ),
        "hkAttributeParser *, hkStringView *, hkStringBuf *, hkStringBuf *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkStringView *
            argtypes=[
                ctypes.c_ulonglong,  # hkAttributeParser *
                ctypes.c_ulonglong,  # hkStringView *
                ctypes.c_ulonglong,  # hkStringBuf *
                ctypes.c_ulonglong,  # hkStringBuf *
            ]
        ),
    },
    "hkAttributeParser::currentAttrValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkAttrData *
        argtypes=[
            ctypes.c_ulonglong,  # hkAttributeParser *
            ctypes.c_ulonglong,  # hkAttrData *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkAttrData::getField": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkAttrData::FieldBase<hkAttrData const > *
        argtypes=[
            ctypes.c_ulonglong,  # hkAttrData *
            ctypes.c_ulonglong,  # hkAttrData::FieldBase<hkAttrData const > *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkAttrData::WritableField::setValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkAttrData::WritableField *
            ctypes.c_ulonglong,  # const hkStringView *
        ]
    ),
    "hkAttrData::WritableField::setValuesArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkAttrData::WritableField *
            ctypes.c_ulonglong,  # const hkArrayView<hkStringView const > *
        ]
    ),
    "hkArrayBase<wchar_t>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<wchar_t> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayUtil::reduce<char>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
        ]
    ),
    "hkUtf8::Utf8FromWide::Utf8FromWide": {
        "hkUtf8::Utf8FromWide *, const wchar_t *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkUtf8::Utf8FromWide *
                ctypes.c_ulonglong,  # const wchar_t *
            ]
        ),
        "hkUtf8::Utf8FromWide *, const wchar_t *, unsigned __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkUtf8::Utf8FromWide *
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
    },
    "hkUtf8::WideFromUtf8::WideFromUtf8": {
        "hkUtf8::WideFromUtf8 *, const hkStringBuf *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkUtf8::WideFromUtf8 *
                ctypes.c_ulonglong,  # const hkStringBuf *
            ]
        ),
        "hkUtf8::WideFromUtf8 *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkUtf8::WideFromUtf8 *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "hkUtf8::WideFromUtf8 *, const char *, unsigned __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkUtf8::WideFromUtf8 *
                ctypes.c_char_p,  # const char *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
    },
    "hkUtf8::WideFromUtf8::~WideFromUtf8": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkUtf8::WideFromUtf8 *
        ]
    ),
    "hkUtf8::utf8FromValidWide": FUNCDEF(
        restype=ctypes.c_int64,  # signed __int64
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkUtf8::wideFromValidUtf8": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # wchar_t *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMT19937RandomGenerator::hkMT19937RandomGenerator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMT19937RandomGenerator *
        ]
    ),
    "hkMT19937RandomGenerator::~hkMT19937RandomGenerator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMT19937RandomGenerator *
        ]
    ),
    "hkMT19937RandomGenerator::initialize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMT19937RandomGenerator *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkStopwatch::getTickCounter": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[]
    ),
    "hkMonitorStream::~hkMonitorStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStream *
        ]
    ),
    "hkMonitorStream::addTag": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStream *
        ]
    ),
    "hkMonitorStream::addValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStream *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "hkMonitorStream::init": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "hkMonitorStream::monitorAddStruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStream *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # void **
            ctypes.c_ulonglong,  # const hkReflect::Type *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMonitorStream::monitorPopDir": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStream *
        ]
    ),
    "hkMonitorStream::monitorPushDir": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStream *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkMonitorStream::multiTimerBegin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStream *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMonitorStream::multiTimerEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStream *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const hkMonitorStream::MultiTimerCommand *
        ]
    ),
    "hkMonitorStream::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStream *
        ]
    ),
    "hkMonitorStream::TimerCommand::setTime": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStream::TimerCommand *
        ]
    ),
    "hkMonitorStream::timerBegin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStream *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMonitorStream::timerBeginList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStream *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMonitorStream::timerBeginObjectName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStream *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkMonitorStream::timerEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStream *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkMonitorStream::timerEndList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStream *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkMonitorStream::timerSplitList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMonitorStream *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0>>::~hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0>>::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0>>::findKey": FUNCDEF(
        restype=ctypes.c_ulonglong,  # Dummy *
        argtypes=[
            ctypes.c_ulonglong,  # hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0>>::getIterator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # Dummy *
        argtypes=[
            ctypes.c_ulonglong,  # hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0>>::getKey": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # Dummy *
        ]
    ),
    "hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0>>::getNext": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # Dummy *
        ]
    ),
    "hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0>>::getValue": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # Dummy *
        ]
    ),
    "hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0>>::getWithDefault": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0>>::insert": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0>>::isValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # Dummy *
        ]
    ),
    "hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0>>::resizeTable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkCachedHashMap<hkStringMapOperations,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_int32,  # int
        ]
    ),
    "hkColor::rgbFromFloats": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
        ]
    ),
    "hkDetail::Func::FuncBase::FuncBase": {
        "hkDetail::Func::FuncBase *, hkDetail::Func::FuncBase *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkDetail::Func::FuncBase *
                ctypes.c_ulonglong,  # hkDetail::Func::FuncBase *
            ]
        ),
        "hkDetail::Func::FuncBase *, unsigned __int64, unsigned __int64, void *, __int64 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkDetail::Func::FuncBase *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # void *
                ctypes.c_ulonglong,  # {'return': 'unsigned __int64', 'args': ['void *', 'void *']}
            ]
        ),
    },
    "hkDetail::Func::FuncBase::~FuncBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDetail::Func::FuncBase *
        ]
    ),
    "hkDetail::Func::removeReferenceFromExternalFunctionalObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hkThread::Cinfo::Cinfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkThread::Cinfo *
        ]
    ),
    "hkThread::hkThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkThread *
            ctypes.c_int32,  # int
        ]
    ),
    "hkThread::~hkThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkThread *
        ]
    ),
    "hkArrayUtil::reduce<hkDefaultTaskQueue::HandleImpl *>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const hknpMotionWeldTOICommand **
            ctypes.c_int32,  # int
        ]
    ),
    "hkDefaultTaskQueue::HandleAllocator::Block::Block": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue::HandleAllocator::Block *
        ]
    ),
    "hkDefaultTaskQueue::hkDefaultTaskQueue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue *
            ctypes.c_ulonglong,  # const hkDefaultTaskQueue::Cinfo *
        ]
    ),
    "hkMinHeap<hkDefaultTaskQueue::QueueEntry,hkMinHeapDefaultOperations<hkDefaultTaskQueue::QueueEntry>,hkBuiltinContainerAllocator<0,0>>::~hkMinHeap<hkDefaultTaskQueue::QueueEntry,hkMinHeapDefaultOperations<hkDefaultTaskQueue::QueueEntry>,hkBuiltinContainerAllocator<0,0>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::TagfileTypeRelocs *
        ]
    ),
    "hkDefaultTaskQueue::ThreadContext::~ThreadContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue::ThreadContext *
        ]
    ),
    "hkDefaultTaskQueue::~hkDefaultTaskQueue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue *
        ]
    ),
    "hkDefaultTaskQueue::addAndSubmitTask": FUNCDEF(
        restype=ctypes.c_ulonglong,  # struct hkTaskQueue::_Handle *
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskQueue *
            ctypes.c_ulonglong,  # hkTask *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # hkTask::MultiplicityMode::Enum
            ctypes.c_uint32,  # hkTask::Priority::Enum
        ]
    ),
    "hkDefaultTaskQueue::addDependencies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue *
            ctypes.c_ulonglong,  # hkTaskQueue::Dependency *
            ctypes.c_int32,  # int
        ]
    ),
    "hkDefaultTaskQueue::addDependency": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue *
            ctypes.c_ulonglong,  # struct hkTaskQueue::_Handle *
            ctypes.c_ulonglong,  # struct hkTaskQueue::_Handle *
        ]
    ),
    "hkDefaultTaskQueue::HandleAllocator::allocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue::HandleAllocator *
            ctypes.c_ulonglong,  # hkDefaultTaskQueue::HandleImpl **
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkDefaultTaskQueue::allocateHandles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue *
            ctypes.c_ulonglong,  # struct hkTaskQueue::_Handle **
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkDefaultTaskQueue::clearTimerData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue *
        ]
    ),
    "hkDefaultTaskQueue::close": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue *
        ]
    ),
    "hkTaskQueue::close": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskQueue *
        ]
    ),
    "hkDefaultTaskQueue::HandleAllocator::free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue::HandleAllocator *
            ctypes.c_ulonglong,  # hkDefaultTaskQueue::HandleImpl **
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkDefaultTaskQueue::freeHandles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue *
            ctypes.c_ulonglong,  # struct hkTaskQueue::_Handle **
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hknpCollideSyncTask::getDebugAttributes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPrepareNarrowPhasePairsTask *
            ctypes.c_ulonglong,  # hkTask::DebugAttributes *
        ]
    ),
    "hkDefaultTaskQueue::EmptyTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue::EmptyTask *
        ]
    ),
    "hkDefaultTaskQueue::getNumThreadsHint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue *
        ]
    ),
    "hkDefaultTaskQueue::getTimerData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue *
            ctypes.c_ulonglong,  # hkArray<hkTimerData,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkDefaultTaskQueue::initHandle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue *
            ctypes.c_ulonglong,  # struct hkTaskQueue::_Handle *
            ctypes.c_ulonglong,  # hkTask *
            ctypes.c_uint32,  # hkTask::Priority::Enum
        ]
    ),
    "hkDefaultTaskQueue::isHandleFinished": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue *
            ctypes.c_ulonglong,  # struct hkTaskQueue::_Handle *
        ]
    ),
    "CAkSrcMediaCodecVorbis::ReleaseBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecVorbis *
            ctypes.c_ulonglong,  # AK::SrcMedia::Stream::State *
        ]
    ),
    "hkDefaultTaskQueue::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue *
        ]
    ),
    "hkTaskQueue::processAndFree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskQueue *
            ctypes.c_ulonglong,  # struct hkTaskQueue::_Handle *
        ]
    ),
    "hkDefaultTaskQueue::processOnce": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue *
        ]
    ),
    "hkDefaultTaskQueue::processUntilFinished": {
        "hkDefaultTaskQueue *, struct hkTaskQueue::_Handle **, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkDefaultTaskQueue *
                ctypes.c_ulonglong,  # struct hkTaskQueue::_Handle **
                ctypes.c_int32,  # int
            ]
        ),
        "hkDefaultTaskQueue *, struct hkTaskQueue::_Handle *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkDefaultTaskQueue *
                ctypes.c_ulonglong,  # struct hkTaskQueue::_Handle *
            ]
        ),
    },
    "hkMinHeap<hkDefaultTaskQueue::QueueEntry,hkMinHeapDefaultOperations<hkDefaultTaskQueue::QueueEntry>,hkBuiltinContainerAllocator<0,0>>::removeEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMinHeap<hkDefaultTaskQueue::QueueEntry,hkMinHeapDefaultOperations<hkDefaultTaskQueue::QueueEntry>,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_int32,  # int
        ]
    ),
    "hkDefaultTaskQueue::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue *
        ]
    ),
    "hkTaskQueue::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskQueue *
        ]
    ),
    "hkTaskQueue::setAffinity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskQueue *
            ctypes.c_ulonglong,  # struct hkTaskQueue::_Handle *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkDefaultTaskQueue::setMultiplicity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue *
            ctypes.c_ulonglong,  # struct hkTaskQueue::_Handle *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # hkTask::MultiplicityMode::Enum
        ]
    ),
    "hkDefaultTaskQueue::submitHandles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultTaskQueue *
            ctypes.c_ulonglong,  # struct hkTaskQueue::_Handle **
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # hkTaskQueue::Order::Enum
            ctypes.c_int32,  # int
        ]
    ),
    "hkSemaphore::hkSemaphore": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSemaphore *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkSemaphore::~hkSemaphore": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "TkThread::WaitOnSemaphore": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "hkSemaphore::release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSemaphore *
            ctypes.c_int32,  # int
        ]
    ),
    "hkAsyncThreadPoolDetail::ToBackgroundQueue::pushAllSemaphoresToBackground": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkAsyncThreadPoolDetail::ToBackgroundQueue *
        ]
    ),
    "hkAsyncThreadPoolDetail::ToBackgroundQueue::pushToBackground": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkAsyncThreadPoolDetail::ToBackgroundQueue *
            ctypes.c_int32,  # int
        ]
    ),
    "hkAsyncHeartbeat::beat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkMemory::Detail::outOfMemory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemory::Detail *
        ]
    ),
    "hkHash::appendCrc32": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkHash::computeCrc32": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkArrayUtil::checkReduce": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "hkIo::WriteBuffer::WriteBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::WriteBuffer *
            ctypes.c_ulonglong,  # const hkIo::Detail::WriteBufferAdapter *
        ]
    ),
    "hkIo::WriteBuffer::~WriteBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::WriteBuffer *
        ]
    ),
    "hkIo::Detail::ArrayWriteBufferImpl::_attachTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::Detail::ArrayWriteBufferImpl *
            ctypes.c_ulonglong,  # hkIo::Detail::WriteBufferState *
        ]
    ),
    "hkIo::Detail::ArrayWriteBufferImpl::_detachFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::Detail::ArrayWriteBufferImpl *
            ctypes.c_ulonglong,  # hkIo::Detail::WriteBufferState *
        ]
    ),
    "hkIo::Detail::ArrayWriteBufferImpl::_expandBy": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::Detail::ArrayWriteBufferImpl *
            ctypes.c_ulonglong,  # hkIo::Detail::WriteBufferState *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkIo::Detail::ArrayWriteBufferImpl::_writeRaw": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::Detail::ArrayWriteBufferImpl *
            ctypes.c_ulonglong,  # hkIo::Detail::WriteBufferState *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkIo::Detail::ArrayWriteBufferImpl::_writeRawAt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::Detail::ArrayWriteBufferImpl *
            ctypes.c_ulonglong,  # hkIo::Detail::WriteBufferState *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkIo::Detail::createWriterImpl": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReferencedObject *
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<char> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hkIo::WriteBuffer::detach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::WriteBuffer *
        ]
    ),
    "hkIo::WriteBuffer::implExpandBy": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::WriteBuffer *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkIo::WriteBuffer::implWriteRaw": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::WriteBuffer *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkIo::WriteBuffer::implWriteRawAt": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::WriteBuffer *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkIo::WriteBuffer::writeZero": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::WriteBuffer *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkBufferedStreamWriter::hkBufferedStreamWriter": {
        "hkBufferedStreamWriter *, hkStreamWriter *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkBufferedStreamWriter *
                ctypes.c_ulonglong,  # hkStreamWriter *
                ctypes.c_int32,  # int
            ]
        ),
        "hkBufferedStreamWriter *, void *, int, hkBool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkBufferedStreamWriter *
                ctypes.c_ulonglong,  # void *
                ctypes.c_int32,  # int
                ctypes.c_ubyte,  # hkBool
            ]
        ),
    },
    "hkBufferedStreamWriter::flush": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkBufferedStreamWriter *
        ]
    ),
    "hkBufferedStreamWriter::flushBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkBufferedStreamWriter *
        ]
    ),
    "hkBufferedStreamWriter::isOk": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkBufferedStreamWriter *
            ctypes.c_ulonglong,  # hkBool *
        ]
    ),
    "hkBufferedStreamWriter::seek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkBufferedStreamWriter *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkBufferedStreamWriter::seekTellSupported": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkBufferedStreamWriter *
            ctypes.c_ulonglong,  # hkBool *
        ]
    ),
    "hkBufferedStreamWriter::tell": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkBufferedStreamWriter *
        ]
    ),
    "hkBufferedStreamWriter::write": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkBufferedStreamWriter *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkDefaultError::hkDefaultError": {
        "hkDefaultError *, __int64 *, void *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkDefaultError *
                ctypes.c_ulonglong,  # {'return': 'void', 'args': ['const char *', 'void *']}
                ctypes.c_ulonglong,  # void *
            ]
        ),
        "hkDefaultError *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkDefaultError *
            ]
        ),
    },
    "hkDefaultError::~hkDefaultError": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultError *
        ]
    ),
    "hkDefaultError::enableAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultError *
        ]
    ),
    "hkDefaultError::formMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultError *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkDefaultError::getMessageTypeString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultError *
            ctypes.c_uint32,  # hkError::Message
            ctypes.c_char_p,  # const char **
        ]
    ),
    "hkDefaultError::getMinimumMessageLevel": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultError *
        ]
    ),
    "hkDefaultError::isEnabled": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultError *
            ctypes.c_ulonglong,  # hkBool *
            ctypes.c_int32,  # int
        ]
    ),
    "hkDefaultError::message": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultError *
            ctypes.c_uint32,  # hkError::Message
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "hkDefaultError::sectionBegin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultError *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkDefaultError::sectionEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultError *
        ]
    ),
    "hkDefaultError::setEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultError *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # hkBool
        ]
    ),
    "hkDefaultError::setMinimumMessageLevel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultError *
            ctypes.c_uint32,  # hkError::Message
        ]
    ),
    "hkDefaultError::shouldReportStackTrace": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultError *
            ctypes.c_uint32,  # hkError::Message
        ]
    ),
    "hkDefaultError::showMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultError *
            ctypes.c_uint32,  # hkError::Message
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "hkFreeList::hkFreeList": {
        "hkFreeList *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkFreeList *
            ]
        ),
        "hkFreeList *, unsigned __int64, unsigned __int64, unsigned __int64, const hkContainerAllocator *, const hkContainerAllocator *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkFreeList *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # const hkContainerAllocator *
                ctypes.c_ulonglong,  # const hkContainerAllocator *
            ]
        ),
    },
    "hkFreeList::_calcBlockFree": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkFreeList *
            ctypes.c_ulonglong,  # const hkFreeList::Block *
        ]
    ),
    "hkFreeList::_compareElements": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkBool *
            ctypes.c_ulonglong,  # const hkFreeList::Element *
            ctypes.c_ulonglong,  # const hkFreeList::Element *
        ]
    ),
    "hkFreeList::_freeBlocks": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkFreeList *
            ctypes.c_ulonglong,  # hkFreeList::Block *
        ]
    ),
    "hkFreeList::addSpace": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # hkFreeList *
        ]
    ),
    "hkFreeList::findGarbage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkFreeList *
        ]
    ),
    "hkFreeList::freeAllFreeBlocks": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkFreeList *
        ]
    ),
    "hkFreeList::freeAllMemory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkFreeList *
        ]
    ),
    "hkFreeList::getMemoryStatistics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkFreeList *
            ctypes.c_ulonglong,  # hkMemory::AllocatorStats *
        ]
    ),
    "hkFreeList::incrementalFindGarbage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkFreeList *
            ctypes.c_ulonglong,  # hkBool *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hkFreeList::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkFreeList *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hkArrayBase<hkFileSystem::Watcher::Change>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkFileSystem::Watcher::Change> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkFileSystem::Watcher::Change>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkFileSystem::Watcher::Change> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkHashSet<hkRefPtr<Win32FileSystemWatcher>,hkBuiltinContainerAllocator<0,0>>::tryInsert<hkRefPtr<Win32FileSystemWatcher> const &>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHashSet<hkRefPtr<Win32FileSystemWatcher>,hkBuiltinContainerAllocator<0,0> >::TryInsertResult *
        argtypes=[
            ctypes.c_ulonglong,  # hkHashSet<hkRefPtr<Win32FileSystemWatcher>,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkHashSet<hkRefPtr<Win32FileSystemWatcher>,hkBuiltinContainerAllocator<0,0> >::TryInsertResult *
            ctypes.c_ulonglong,  # const hkRefPtr<Win32FileSystemWatcher> *
        ]
    ),
    "hkWin32FileSystem::WatcherImpl::NotificationCompletion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # _OVERLAPPED *
        ]
    ),
    "hkWin32FileSystem::_openReader": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefNew<hkStreamReader> *
        argtypes=[
            ctypes.c_ulonglong,  # hkWin32FileSystem *
            ctypes.c_ulonglong,  # hkRefNew<hkStreamReader> *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # hkFileSystem::OpenFlags
        ]
    ),
    "hkWin32FileSystem::_openWriter": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefNew<hkStreamWriter> *
        argtypes=[
            ctypes.c_ulonglong,  # hkWin32FileSystem *
            ctypes.c_ulonglong,  # hkRefNew<hkStreamWriter> *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # hkFileSystem::OpenFlags
        ]
    ),
    "hkWin32FileSystem::WatcherImpl::addDirectoryProc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkWin32FileSystem::WatcherImpl::addDirectoryWatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkWin32FileSystem::WatcherImpl *
            ctypes.c_ulonglong,  # Win32FileSystemWatcher *
        ]
    ),
    "hkWin32FileSystem::WatcherImpl::addTrackedDirectory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefNew<hkFileSystem::Watcher> *
        argtypes=[
            ctypes.c_ulonglong,  # hkWin32FileSystem::WatcherImpl *
            ctypes.c_ulonglong,  # hkRefNew<hkFileSystem::Watcher> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkWin32FileSystem::copy": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkWin32FileSystem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkWin32FileSystem::createIterator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefNew<hkFileSystem::Iterator::Impl> *
        argtypes=[
            ctypes.c_ulonglong,  # hkWin32FileSystem *
            ctypes.c_ulonglong,  # hkRefNew<hkFileSystem::Iterator::Impl> *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkWin32FileSystem::createWatcher": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefNew<hkFileSystem::Watcher> *
        argtypes=[
            ctypes.c_ulonglong,  # hkWin32FileSystem *
            ctypes.c_ulonglong,  # hkRefNew<hkFileSystem::Watcher> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "Win32FileSystemWatcher::getCompletedChanges": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkArray<hkFileSystem::Watcher::Change,hkBuiltinContainerAllocator<0,0> > *
        argtypes=[
            ctypes.c_ulonglong,  # Win32FileSystemWatcher *
            ctypes.c_ulonglong,  # hkArray<hkFileSystem::Watcher::Change,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArray<hkFileSystem::Watcher::Change,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkWin32FileSystem::getOperatingSystemPath": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # hkWin32FileSystem *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_uint32,  # hkFileSystem::OSPathFlags
        ]
    ),
    "hkWin32FileSystem::mkdir": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkWin32FileSystem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "hkWin32FileSystem::moveFile": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkWin32FileSystem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkWin32FileSystem::remove": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkWin32FileSystem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "s_entryFromFindData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkFileSystem::Entry *
            ctypes.c_ulonglong,  # hkFileSystem *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const _WIN32_FIND_DATAW *
        ]
    ),
    "hkWin32FileSystem::stat": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkWin32FileSystem *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # hkFileSystem::Entry *
            ctypes.c_ulonglong,  # hkFileSystem::Entry *
        ]
    ),
    "hkWin32FileSystem::WatcherImpl::stopWatching": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "Win32FileSystemWatcher::stopWatching": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # Win32FileSystemWatcher *
            ctypes.c_ulonglong,  # _DWORD *
        ]
    ),
    "hkWin32FileSystem::WatcherImpl::terminateProc2": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkWin32FileSystem::WatcherImpl::terminateProc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "Win32FileSystemWatcher::updatePendingCompletionQueue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Win32FileSystemWatcher *
        ]
    ),
    "hkBufferedStreamReader::hkBufferedStreamReader": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkBufferedStreamReader *
            ctypes.c_ulonglong,  # hkStreamReader *
            ctypes.c_int32,  # int
        ]
    ),
    "hkBufferedStreamReader::isOk": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkBufferedStreamReader *
            ctypes.c_ulonglong,  # hkBool *
        ]
    ),
    "hkBufferedStreamReader::isSeekTellSupported": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBufferedStreamReader *
        argtypes=[
            ctypes.c_ulonglong,  # hkBufferedStreamReader *
        ]
    ),
    "hkBufferedStreamReader::peek": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkBufferedStreamReader *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkBufferedStreamReader::read": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkBufferedStreamReader *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "refillBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkStreamReader *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkBufferedStreamReader::seek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkBufferedStreamReader *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "hkBufferedStreamReader::skip": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkBufferedStreamReader *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkBufferedStreamReader::tell": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkBufferedStreamReader *
        ]
    ),
    "hkIo::copyStream": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # hkStreamReader *
            ctypes.c_ulonglong,  # hkStreamWriter *
        ]
    ),
    "hkBsdSocket::hkBsdSocket": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkBsdSocket *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkBsdSocket::canRead": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkBsdSocket *
        ]
    ),
    "hkBsdSocket::canWrite": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkBsdSocket *
        ]
    ),
    "hkBsdSocket::close": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkBsdSocket *
        ]
    ),
    "hkBsdSocket::connect": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkBsdSocket *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkBsdSocket::createDatagramSocket": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkBsdSocket *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkBsdSocket::createSocket": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkBsdSocket *
            ctypes.c_ulonglong,  # _DWORD *
        ]
    ),
    "hkBsdSocket::getAddress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkBsdSocket *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hkBsdSocket::getInetAddr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkInetAddr *
        argtypes=[
            ctypes.c_ulonglong,  # hkBsdSocket *
            ctypes.c_ulonglong,  # hkInetAddr *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkBsdSocket::getLastError": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkBsdSocket *
        ]
    ),
    "hkBsdGetAddressString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkBsdNetworkInit": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "hkBsdSocket::isOk": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkBsdSocket *
            ctypes.c_ulonglong,  # hkBool *
        ]
    ),
    "hkBsdSocket::listen": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkBsdSocket *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_uint32,  # hkSocket::ListenOptions
            ctypes.c_char,  # char
        ]
    ),
    "hkBsdSocket::pollForNewClient": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSocket *
        argtypes=[
            ctypes.c_ulonglong,  # hkBsdSocket *
            ctypes.c_ulonglong,  # hkInetAddr *
        ]
    ),
    "hkBsdSocket::read": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkBsdSocket *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkBsdSocket::receiveFrom": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkBsdSocket *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # hkInetAddr *
        ]
    ),
    "hkBsdSocket::sendTo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkBsdSocket *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const hkInetAddr *
        ]
    ),
    "hkBsdSocket::setBlocking": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkBsdSocket *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_char,  # char
        ]
    ),
    "hkBsdSocket::write": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkBsdSocket *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkStreamWriter::hkStreamWriter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkStreamWriter *
        ]
    ),
    "hkSeekableStreamReader::hkSeekableStreamReader": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSeekableStreamReader *
        ]
    ),
    "hkStreamReader::hkStreamReader": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkStreamReader *
        ]
    ),
    "hkStreamReader::skip": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkStreamReader *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::ArrayType,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::DataFieldDecl,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::PointerType,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::Type,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkReflect::Opt::Values,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkMultiMap<unsigned __int64,unsigned __int64,hkMultiMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0>>::hkMultiMap<unsigned __int64,unsigned __int64,hkMultiMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMultiMap<unsigned __int64,unsigned __int64,hkMultiMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkMultiMap<unsigned __int64,unsigned __int64,hkMultiMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0>>::~hkMultiMap<unsigned __int64,unsigned __int64,hkMultiMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMultiMap<unsigned __int64,unsigned __int64,hkMultiMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkMultiMap<unsigned __int64,unsigned __int64,hkMultiMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0>>::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMultiMap<unsigned __int64,unsigned __int64,hkMultiMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkMultiMap<unsigned __int64,unsigned __int64,hkMultiMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0>>::findKey": FUNCDEF(
        restype=ctypes.c_ulonglong,  # Dummy *
        argtypes=[
            ctypes.c_ulonglong,  # hkMultiMap<unsigned __int64,unsigned __int64,hkMultiMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMultiMap<unsigned __int64,unsigned __int64,hkMultiMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0>>::findKeyWithValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # Dummy *
        argtypes=[
            ctypes.c_ulonglong,  # hkMultiMap<unsigned __int64,unsigned __int64,hkMultiMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMultiMap<unsigned __int64,unsigned __int64,hkMultiMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0>>::insert": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMultiMap<unsigned __int64,unsigned __int64,hkMultiMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMultiMap<unsigned __int64,unsigned __int64,hkMultiMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0>>::resizeTable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkMultiMap<unsigned __int64,unsigned __int64,hkMultiMapOperations<unsigned __int64>,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_int32,  # int
        ]
    ),
    "hkArrayBase<hkTrait::AlignedStorage<56,1>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkTrait::AlignedStorage<56,1> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkPair<hkTuple<hkReflect::Type const *,hkReflect::Type const *,void,void,void,void,void,void>,hkTypeVm::CompiledProgram *>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<hkTuple<hkReflect::Type const *,hkReflect::Type const *,void,void,void,void,void,void>,hkTypeVm::CompiledProgram *> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkScopedPtr<hkTypeVm::Pass>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkScopedPtr<hkTypeVm::Pass> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkTypeVm::AbstractInstruction>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkTypeVm::AbstractInstruction> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkHashMap<hkReflect::Type const *,hkTypeVm::CompiledProgram *,hkBuiltinContainerAllocator<0,0>>::insert": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # hkHashMap<hkReferencedObject const *,hkReferencedObject *,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # const hkReferencedObject *const *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hkHashMap<hkTuple<hkReflect::Type const *,hkReflect::Type const *,void,void,void,void,void,void>,hkTypeVm::CompiledProgram *,hkBuiltinContainerAllocator<0,0>>::insert": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # hkHashMap<hkTuple<hkReflect::Type const *,hkReflect::Type const *,void,void,void,void,void,void>,hkTypeVm::CompiledProgram *,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # const hkTuple<hkReflect::Type const *,hkReflect::Type const *,void,void,void,void,void,void> *
            ctypes.c_ulonglong,  # hkTypeVm::CompiledProgram **
        ]
    ),
    "hkArrayBase<hkTypeVm::AbstractInstruction>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkTypeVm::AbstractInstruction> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkTypeVm::AbstractInstruction>::_ensureOwnsStorage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkTypeVm::AbstractInstruction> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hkThreadLocalBlockStreamAllocator::hkThreadLocalBlockStreamAllocator": {
        "hkThreadLocalBlockStreamAllocator *, hkBlockStreamAllocator *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
                ctypes.c_ulonglong,  # hkBlockStreamAllocator *
            ]
        ),
        "hkThreadLocalBlockStreamAllocator *, hkBlockStreamAllocator *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
                ctypes.c_ulonglong,  # hkBlockStreamAllocator *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "hkThreadLocalBlockStreamAllocator::~hkThreadLocalBlockStreamAllocator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
        ]
    ),
    "hkThreadLocalBlockStreamAllocator::blockAlloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBlockStream::Block *
        argtypes=[
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
        ]
    ),
    "hkThreadLocalBlockStreamAllocator::blockFree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
            ctypes.c_ulonglong,  # hkBlockStream::Block *
        ]
    ),
    "hkThreadLocalBlockStreamAllocator::blockFreeBatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
            ctypes.c_ulonglong,  # hkBlockStream::Block **
            ctypes.c_int32,  # int
        ]
    ),
    "hkThreadLocalBlockStreamAllocator::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
        ]
    ),
    "hkTask::~hkTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTask *
        ]
    ),
    "hkTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkTask *
        ]
    ),
    "hkTaskQueue::addDependencies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskQueue *
            ctypes.c_ulonglong,  # hkTaskQueue::Dependency *
            ctypes.c_int32,  # int
        ]
    ),
    "hkTaskQueue::forEach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskQueue *
            ctypes.c_ulonglong,  # hkTaskBatchProcessor *
            ctypes.c_ulonglong,  # const hkTaskBatchProcessor::BatchInfo *
        ]
    ),
    "hkTaskQueue::isFinished": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskQueue *
            ctypes.c_ulonglong,  # struct hkTaskQueue::_Handle *
        ]
    ),
    "hkTaskQueue_BatchingTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskQueue_BatchingTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hkTaskQueue::processUntilFinished": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskQueue *
            ctypes.c_ulonglong,  # struct hkTaskQueue::_Handle **
            ctypes.c_int32,  # int
        ]
    ),
    "hkTaskQueue::submitGraphAndWait": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskQueue *
            ctypes.c_ulonglong,  # hkTaskGraph *
            ctypes.c_uint32,  # hkTaskQueue::Order::Enum
        ]
    ),
    "hkAsyncThreadPoolSimpleSemaphore::hkAsyncThreadPoolSimpleSemaphore": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkAsyncThreadPoolSimpleSemaphore *
            ctypes.c_ulonglong,  # hkAsyncThreadPool *
            ctypes.c_int32,  # int
        ]
    ),
    "hkAsyncThreadPoolMutex::~hkAsyncThreadPoolMutex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkAsyncThreadPoolMutex *
        ]
    ),
    "hkAsyncThreadPoolSimpleSemaphore::acquire": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkAsyncThreadPoolSimpleSemaphore *
        ]
    ),
    "hkAsyncThreadPoolMutex::enter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkAsyncThreadPoolMutex *
        ]
    ),
    "hkAsyncThreadPoolMutex::leave": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkAsyncThreadPoolMutex *
        ]
    ),
    "hkAsyncThreadPoolMutex::AsyncWaiter::release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkAsyncThreadPoolMutex::AsyncWaiter *
        ]
    ),
    "hkAsyncThreadPoolMutex::OtherWaiter::release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkAsyncThreadPoolMutex::OtherWaiter *
        ]
    ),
    "hkAsyncThreadPoolSimpleSemaphore::release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkAsyncThreadPoolSimpleSemaphore *
        ]
    ),
    "hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64>>::remove": {
        "hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64> > *, const unsigned __int64 *, const unsigned __int64 *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # const unsigned __int64 *
            argtypes=[
                ctypes.c_ulonglong,  # hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64> > *
                ctypes.c_ulonglong,  # const unsigned __int64 *
                ctypes.c_ulonglong,  # const unsigned __int64 *
            ]
        ),
        "hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64> > *, Dummy *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkMapBase<unsigned __int64,unsigned __int64,hkMapOperations<unsigned __int64> > *
                ctypes.c_ulonglong,  # Dummy *
            ]
        ),
    },
    "hkStdioStreamReader::isOk": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkStdioStreamReader *
            ctypes.c_ulonglong,  # hkBool *
        ]
    ),
    "hkStdioStreamReader::open": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkStdioStreamReader *
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkStdioStreamReader::peek": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkStdioStreamReader *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkStdioStreamReader::read": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkStdioStreamReader *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkStdioStreamReader::seek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkStdioStreamReader *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # hkSeekableStreamReader::SeekWhence
            ctypes.c_int32,  # int
        ]
    ),
    "hkStdioStreamReader::tell": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkStdioStreamReader *
        ]
    ),
    "hkWin32StreamWriter::flush": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkWin32StreamWriter *
        ]
    ),
    "hkWin32StreamWriter::isOk": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkWin32StreamWriter *
            ctypes.c_ulonglong,  # hkBool *
        ]
    ),
    "hkWin32StreamWriter::open": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkStreamWriter *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkWin32StreamWriter::seek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkWin32StreamWriter *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # hkStreamWriter::SeekWhence
            ctypes.c_uint32,  # DWORD
        ]
    ),
    "hkWin32StreamWriter::tell": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkWin32StreamWriter *
        ]
    ),
    "hkWin32StreamWriter::write": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkWin32StreamWriter *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkTaskGraph::Node>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkTaskGraph::Node> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkRefPtr<hknpAction>>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkRefPtr<hknpAction> > *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkTaskGraph::hkTaskGraph": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskGraph *
        ]
    ),
    "hkTaskGraph::~hkTaskGraph": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskGraph *
        ]
    ),
    "hkTaskGraphEx::_ensureSingleLeafTask": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskGraphEx *
        ]
    ),
    "hkTaskGraph::addReferencedTask": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHandle<unsigned short,65535,hkTaskGraph::TaskIdDiscriminant> *
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskGraph *
            ctypes.c_ulonglong,  # hkHandle<unsigned short,65535,hkTaskGraph::TaskIdDiscriminant> *
            ctypes.c_ulonglong,  # hkReferencedTask *
            ctypes.c_uint32,  # hkTask::Priority::Enum
        ]
    ),
    "hkTaskGraph::addReferencedTaskWithMultiplicity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHandle<unsigned short,65535,hkTaskGraph::TaskIdDiscriminant> *
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskGraph *
            ctypes.c_ulonglong,  # hkHandle<unsigned short,65535,hkTaskGraph::TaskIdDiscriminant> *
            ctypes.c_ulonglong,  # hkReferencedTask *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # hkTask::MultiplicityMode::Enum
            ctypes.c_uint32,  # hkTask::Priority::Enum
        ]
    ),
    "hkTaskGraph::addTask": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHandle<unsigned short,65535,hkTaskGraph::TaskIdDiscriminant> *
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskGraph *
            ctypes.c_ulonglong,  # hkHandle<unsigned short,65535,hkTaskGraph::TaskIdDiscriminant> *
            ctypes.c_ulonglong,  # hkTask *
            ctypes.c_uint32,  # hkTask::Priority::Enum
        ]
    ),
    "hkTaskGraph::addTaskWithMultiplicity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHandle<unsigned short,65535,hkTaskGraph::TaskIdDiscriminant> *
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskGraph *
            ctypes.c_ulonglong,  # hkHandle<unsigned short,65535,hkTaskGraph::TaskIdDiscriminant> *
            ctypes.c_ulonglong,  # hkTask *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # hkTask::MultiplicityMode::Enum
            ctypes.c_uint32,  # hkTask::Priority::Enum
        ]
    ),
    "hkTaskGraph::append": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkTaskGraph::TaskIdMapping *
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskGraph *
            ctypes.c_ulonglong,  # hkTaskGraph::TaskIdMapping *
            ctypes.c_ulonglong,  # const hkTaskGraph *
        ]
    ),
    "hkTaskGraph::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskGraph *
        ]
    ),
    "hkTaskGraph::freeTaskQueueHandles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskGraph *
            ctypes.c_ulonglong,  # hkTaskQueue *
        ]
    ),
    "hkTaskGraph::reserve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskGraph *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkTaskGraph::submitToTaskQueue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # struct hkTaskQueue::_Handle *
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskGraph *
            ctypes.c_ulonglong,  # hkTaskQueue *
            ctypes.c_uint32,  # hkTaskQueue::Order::Enum
        ]
    ),
    "hkTaskGraph::submitToTaskQueueAndWait": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTaskGraph *
            ctypes.c_ulonglong,  # hkTaskQueue *
            ctypes.c_uint32,  # hkTaskQueue::Order::Enum
        ]
    ),
    "hknpBinaryAction::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpBinaryAction *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpBinaryAction::getBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBinaryAction *
            ctypes.c_ulonglong,  # const hknpBodyId **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hknpBridgeConstraintAtomBase::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpBridgeConstraintAtom::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpSpringAction::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpSpringAction *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpVehicleDefaultSteering::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultSteering *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpCompoundShape::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpShapeKeyMask::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeKeyMask *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpVehicleWheelCollide::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleWheelCollide *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpPointToPathConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpPointToPathConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpMaterialLibrary::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpVehicleSuspension::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleSuspension *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpExternMeshShapeGeometry::getQuadVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShapeGeometry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpExternMeshShapeGeometry::getQuadShapeTag": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShapeGeometry *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpExternMeshShapeGeometry::getNumPrimitives": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShapeGeometry *
        ]
    ),
    "hknpExternMeshShapeGeometry::getPrimitiveFlags": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShapeGeometry *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpExternMeshShapeGeometry::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpExternMeshShapeGeometry> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShapeGeometry *
            ctypes.c_ulonglong,  # hkRefPtr<hknpExternMeshShapeGeometry> *
        ]
    ),
    "hknpFirstPersonGun::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpFirstPersonGun *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpBodyIntegrator::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyIntegrator *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpConvexShape::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpFirstPersonGun::createClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefNew<hknpFirstPersonGun> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpFirstPersonGun *
            ctypes.c_ulonglong,  # hkRefNew<hknpFirstPersonGun> *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpFirstPersonGun::syncCloneFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFirstPersonGun *
            ctypes.c_ulonglong,  # const hknpFirstPersonGun *
        ]
    ),
    "hknpPointToPlaneConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpPointToPlaneConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknp6DofConstraintData::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpCompressedMeshShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpPrismaticConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpPrismaticConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpVehicleInstance::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpVehicleDefaultSuspension::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultSuspension *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpDefaultExternMeshShapeGeometry::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpDefaultExternMeshShapeGeometry *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpPulleyConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpPulleyConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpExtendedExternMeshShapeGeometry::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpExtendedExternMeshShapeGeometry *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpBallGun::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpBallGun *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpExtendedExternMeshShapeGeometry::getTriangleFlags": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpExtendedExternMeshShapeGeometry *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpExtendedExternMeshShapeGeometry::getQuadFlags": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpExtendedExternMeshShapeGeometry *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpRackAndPinionConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpRackAndPinionConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpGravityGun::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpGravityGun *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpVehicleDefaultTransmission::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultTransmission *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpTyremarksWheel::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpTyremarksWheel *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpRagdollConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpRagdollConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpProjectileGun::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpProjectileGun *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpRotationalConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpRotationalConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpWorldSnapshot::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldSnapshot *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpCompoundShapeData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpTyremarksInfo::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpTyremarksInfo *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpLodShape::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpStiffSpringConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpStiffSpringConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpExternMeshShapeData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShapeData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpFirstPersonCharacter::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpFirstPersonCharacter *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpWheelFrictionConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWheelFrictionConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpVehicleDefaultVelocityDamper::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultVelocityDamper *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpCharacterProxyCinfo::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyCinfo *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpWheelConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWheelConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpVehicleLinearCastWheelCollide::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleLinearCastWheelCollide *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpPhysicsSystemData::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpBallGun::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpWrappedConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWrappedConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpCharacterProxy::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxy *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpConnectivityGraph::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConnectivityGraph *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpConstraintMotor::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintMotor *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpVehicleData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpLimitedForceConstraintMotor::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpLimitedForceConstraintMotor *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpCallbackConstraintMotor::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCallbackConstraintMotor *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpCompoundShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpWheelFrictionConstraintAtom::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpFirstPersonGun::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpVehicleInstance::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpMaterialLibrary::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpMaterialLibrary *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpVehicleLinearCastWheelCollide::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpVehicleRayCastWheelCollide::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleRayCastWheelCollide *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpCharacterRigidBodyCinfo::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterRigidBodyCinfo *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpShapeProperty<61697>::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeProperty<61697> *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpMaterialPalette::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpMaterialPalette *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpPhysicsSystemData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpPhysicsSystemData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknp6DofConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknp6DofConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpBodyQuality::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyQuality *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpVehicleDefaultAerodynamics::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultAerodynamics *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpCircularSurfaceVelocity::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCircularSurfaceVelocity *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpVehicleRayCastWheelCollide::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpLinearSurfaceVelocity::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearSurfaceVelocity *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpLodShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpLodShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpDisableCollisionFilter::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpDisableCollisionFilter *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpAngularFrictionConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpAngularFrictionConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpScaledConvexShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpScaledConvexShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpVehicleWheelCollide::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpExternMeshShape::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpShapePropertyBase::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapePropertyBase *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpGravityGun::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpBallAndSocketConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpBallAndSocketConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpCollisionFilter::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionFilter *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpCollisionFilter::getLodInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpLodCollisionInfo *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionFilter *
            ctypes.c_ulonglong,  # hknpLodCollisionInfo *
            ctypes.c_ulonglong,  # const hknpBody *
            ctypes.c_ulonglong,  # const hknpBody *
        ]
    ),
    "hknpCollisionFilter::copyFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionFilter *
            ctypes.c_ulonglong,  # const hknpCollisionFilter *
        ]
    ),
    "hknpCollisionFilter::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpCollisionFilter> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionFilter *
            ctypes.c_ulonglong,  # hkRefPtr<hknpCollisionFilter> *
        ]
    ),
    "hknpTransformedShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpTransformedShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpVehicleDefaultBrake::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultBrake *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpLinearClearanceConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearClearanceConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpTriangleShape::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpMotionPropertiesLibrary::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotionPropertiesLibrary *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpShapeProperty<61461>::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeProperty<61461> *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpShapeConnectivityGraph::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeConnectivityGraph *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<5,5,5,16>>::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<5,5,5,16> > *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpParticlesColliderShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesColliderShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpCogWheelConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCogWheelConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpRefMaterial::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpRefMaterial *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpConnectivityGraphMask::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConnectivityGraphMask *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpShapeProperty<61696>::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeProperty<61696> *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpShapeMassProperties::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeMassProperties *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpShapeProperty<61699>::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeProperty<61699> *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpDeformableFixedConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeformableFixedConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpShapeMassConfigProperty::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeMassConfigProperty *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpShapeProperty<61460>::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeProperty<61460> *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpRefDragProperties::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpRefDragProperties *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpBodyQualityLibrary::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyQualityLibrary *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpRefMassDistribution::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpRefMassDistribution *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpExternMeshShapeData::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpMountedBallGun::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpFixedConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpFixedConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpVehicleDriverInputAnalogStatus::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDriverInputAnalogStatus *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpShape::MassConfig::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpShape::MassConfig *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpMassPropertiesTree::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpMassPropertiesTree *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpHingeConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpHingeConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpVehicleDefaultAnalogDriverInput::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultAnalogDriverInput *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpRefWorldCinfo::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpRefWorldCinfo *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpPhysicsSceneData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpPhysicsSceneData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpLimitedHingeConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpLimitedHingeConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpMaskedShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpMaskedShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpShapeProperty<61462>::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeProperty<61462> *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpVehicleDefaultEngine::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultEngine *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpShapeMassPropertiesTree::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeMassPropertiesTree *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpVehicleFrictionDescription::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleFrictionDescription *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpAction::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpAction *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpDefaultBroadPhaseConfig::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpDefaultBroadPhaseConfig *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpLinearParametricCurve::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearParametricCurve *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpUnaryAction::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpUnaryAction *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpUnaryAction::getBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpUnaryAction *
            ctypes.c_ulonglong,  # const hknpBodyId **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hknpThreadSafeObjectPool<hknpMaterial>::_copyFrom<hkTrait::IsPodType<hknpMaterial>,hknpThreadSafeObjectPool<hknpMaterial>::EmptyCopyFunctor>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpMaterial> *
            ctypes.c_ulonglong,  # const hknpThreadSafeObjectPool<hknpMaterial> *
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpMaterial>::EmptyCopyFunctor *
        ]
    ),
    "hknpThreadSafeObjectPool<hknpMotionProperties>::_copyFrom<hkTrait::IsPodType<hknpMotionProperties>,hknpThreadSafeObjectPool<hknpMotionProperties>::EmptyCopyFunctor>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpMotionProperties> *
            ctypes.c_ulonglong,  # const hknpThreadSafeObjectPool<hknpMotionProperties> *
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpMotionProperties>::EmptyCopyFunctor *
        ]
    ),
    "hkArrayBase<float>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpSimdTreeBroadPhase::PointId> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkPair<uint,hkRefPtr<hknpShape const>>>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<unsigned int,hkRefPtr<hknpShape const > > > *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpCharacterProxy::Contact>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpCharacterProxy::Contact> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpVehicleSuspension::SuspensionWheelParameters>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpCharacterProxy::TriggerInteraction> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpVehicleData::WheelComponentParams>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpConstraintGroupCinfo> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpVehicleInstance::WheelInfo>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpVehicleInstance::WheelInfo> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpParticleTransformsAndVelocities>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpParticleTransformsAndVelocities> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpStorageParticleSystem>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpStorageParticleSystem> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpTyremarkPoint>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpTyremarkPoint> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkRefPtr<hknpFirstPersonGunBulletListener>>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkRefPtr<hknpWorldSnapshot> > *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkStringPtr>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkStringPtr> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpBody>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpBody> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpConstraintCinfo>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpConstraintCinfo> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpMotion>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpMotion> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkcdDynamicTree::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::Codec32>::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::Codec32>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::Codec32> *
            ctypes.c_ulonglong,  # const hkcdDynamicTree::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::Codec32> *
        ]
    ),
    "hkFreeListRelArrayStorage<hknpShapeInstance>::hkFreeListRelArrayStorage<hknpShapeInstance>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkFreeListRelArrayStorage<hknpShapeInstance> *
            ctypes.c_ulonglong,  # const hkFreeListRelArrayStorage<hknpShapeInstance> *
        ]
    ),
    "hknpThreadSafeObjectPool<hknpMaterial>::hknpThreadSafeObjectPool<hknpMaterial>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpMaterial> *
            ctypes.c_ulonglong,  # const hknpThreadSafeObjectPool<hknpMaterial> *
        ]
    ),
    "hknpThreadSafeObjectPool<hknpMotionProperties>::hknpThreadSafeObjectPool<hknpMotionProperties>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpMotionProperties> *
            ctypes.c_ulonglong,  # const hknpThreadSafeObjectPool<hknpMotionProperties> *
        ]
    ),
    "hknp6DofConstraintData::Blueprints::Blueprints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknp6DofConstraintData::Blueprints *
        ]
    ),
    "hknpConnectivityGraph::CompressedIntArray::CompressedIntArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConnectivityGraph::CompressedIntArray *
            ctypes.c_ulonglong,  # const hknpConnectivityGraph::CompressedIntArray *
        ]
    ),
    "hkRelBitField::hkRelBitField": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkRelBitField *
            ctypes.c_ulonglong,  # const hkRelBitField *
        ]
    ),
    "hkcdSimdTree::hkcdSimdTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdSimdTree *
        ]
    ),
    "hknp6DofConstraintData::hknp6DofConstraintData": {
        "hknp6DofConstraintData *, const hknp6DofConstraintData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknp6DofConstraintData *
                ctypes.c_ulonglong,  # const hknp6DofConstraintData *
            ]
        ),
        "hknp6DofConstraintData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknp6DofConstraintData *
            ]
        ),
    },
    "hknpBodyQualityLibrary::hknpBodyQualityLibrary": {
        "hknpBodyQualityLibrary *, const hknpBodyQualityLibrary *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpBodyQualityLibrary *
                ctypes.c_ulonglong,  # const hknpBodyQualityLibrary *
            ]
        ),
        "hknpBodyQualityLibrary *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpBodyQualityLibrary *
            ]
        ),
    },
    "hknpCharacterProxy::hknpCharacterProxy": {
        "hknpCharacterProxy *, const hknpCharacterProxy *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCharacterProxy *
                ctypes.c_ulonglong,  # const hknpCharacterProxy *
            ]
        ),
        "hknpCharacterProxy *, const hknpCharacterProxyCinfo *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCharacterProxy *
                ctypes.c_ulonglong,  # const hknpCharacterProxyCinfo *
            ]
        ),
    },
    "hknpCompoundShapeData::hknpCompoundShapeData": {
        "hknpCompoundShapeData *, const hknpCompoundShapeData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCompoundShapeData *
                ctypes.c_ulonglong,  # const hknpCompoundShapeData *
            ]
        ),
        "hknpCompoundShapeData *, hknpCompoundShapeBoundingVolumeType::Enum": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCompoundShapeData *
                ctypes.c_uint32,  # hknpCompoundShapeBoundingVolumeType::Enum
            ]
        ),
    },
    "hknpCompressedMeshShape::hknpCompressedMeshShape": {
        "hknpCompressedMeshShape *, const hknpCompressedMeshShape *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCompressedMeshShape *
                ctypes.c_ulonglong,  # const hknpCompressedMeshShape *
            ]
        ),
        "hknpCompressedMeshShape *, const hknpCompressedMeshShape *, hknpShape::CloneMode, hknpShapeMap *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCompressedMeshShape *
                ctypes.c_ulonglong,  # const hknpCompressedMeshShape *
                ctypes.c_uint32,  # hknpShape::CloneMode
                ctypes.c_ulonglong,  # hknpShapeMap *
            ]
        ),
        "hknpCompressedMeshShape *, const hknpCompressedMeshShapeCinfo *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCompressedMeshShape *
                ctypes.c_ulonglong,  # const hknpCompressedMeshShapeCinfo *
            ]
        ),
    },
    "hknpConnectivityGraphMask::hknpConnectivityGraphMask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConnectivityGraphMask *
            ctypes.c_ulonglong,  # const hknpConnectivityGraphMask *
        ]
    ),
    "hknpFirstPersonGun::hknpFirstPersonGun": {
        "hknpFirstPersonGun *, const hknpFirstPersonGun *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpFirstPersonGun *
                ctypes.c_ulonglong,  # const hknpFirstPersonGun *
            ]
        ),
        "hknpFirstPersonGun *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpFirstPersonGun *
            ]
        ),
    },
    "hknpParticleTransformsAndVelocities::hknpParticleTransformsAndVelocities": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleTransformsAndVelocities *
            ctypes.c_ulonglong,  # const hknpParticleTransformsAndVelocities *
        ]
    ),
    "hknpRagdollMotorConstraintAtom::hknpRagdollMotorConstraintAtom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRagdollMotorConstraintAtom *
            ctypes.c_ulonglong,  # const hknpRagdollMotorConstraintAtom *
        ]
    ),
    "hknpStorageParticleSystem::hknpStorageParticleSystem": {
        "hknpStorageParticleSystem *, hknpStorageParticleSystem *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpStorageParticleSystem *
                ctypes.c_ulonglong,  # hknpStorageParticleSystem *
            ]
        ),
        "hknpStorageParticleSystem *, const hknpStorageParticleSystem *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpStorageParticleSystem *
                ctypes.c_ulonglong,  # const hknpStorageParticleSystem *
            ]
        ),
    },
    "hknpVehicleFrictionDescription::hknpVehicleFrictionDescription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleFrictionDescription *
            ctypes.c_ulonglong,  # const hknpVehicleFrictionDescription *
        ]
    ),
    "hknpVehicleInstance::hknpVehicleInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
        ]
    ),
    "hknpWorldCinfo::hknpWorldCinfo": {
        "hknpWorldCinfo *, const hknpWorldCinfo *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpWorldCinfo *
                ctypes.c_ulonglong,  # const hknpWorldCinfo *
            ]
        ),
        "hknpWorldCinfo *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpWorldCinfo *
            ]
        ),
    },
    "hknpWorldSnapshot::hknpWorldSnapshot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldSnapshot *
            ctypes.c_ulonglong,  # const hknpWorldSnapshot *
        ]
    ),
    "hknpThreadSafeObjectPool<hknpMaterial>::~hknpThreadSafeObjectPool<hknpMaterial>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpMaterial> *
        ]
    ),
    "hknpThreadSafeObjectPool<hknpMotionProperties>::~hknpThreadSafeObjectPool<hknpMotionProperties>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpMotionProperties> *
        ]
    ),
    "hknpCompoundShapeData::~hknpCompoundShapeData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeData *
        ]
    ),
    "hknpFirstPersonGun::~hknpFirstPersonGun": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFirstPersonGun *
        ]
    ),
    "hknpMassPropertiesTree::~hknpMassPropertiesTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMassPropertiesTree *
        ]
    ),
    "hknpShapeProperties::~hknpShapeProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeProperties *
        ]
    ),
    "hknpStorageParticleSystem::~hknpStorageParticleSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpStorageParticleSystem *
        ]
    ),
    "hknpWorldCinfo::~hknpWorldCinfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldCinfo *
        ]
    ),
    "hkArrayBase<hkPair<uint,hkRefPtr<hknpShape const>>>::_ensureOwnsStorage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<unsigned int,hkRefPtr<hknpShape const > > > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hkArrayBase<hkRefPtr<hknpConvexShape const>>::_ensureOwnsStorage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkRefPtr<hknpConvexShape const > > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hkArrayBase<hkBool>::_ensureOwnsStorage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkBool> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hkArrayBase<hkVector4f>::_ensureOwnsStorage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkGeometry::Triangle> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hkHashMapDetail::Index<hkBuiltinContainerAllocator<0,0>>::dupIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkHashMapDetail::Index<hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # const hkHashMapDetail::Index<hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpBoxShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpBoxShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpBreakableCompoundShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpBreakableCompoundShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpBreakableCompoundShapeKeyMask::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpBreakableCompoundShapeKeyMask *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpBreakableConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpBreakableConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpCapsuleShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCapsuleShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpCompositeShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompositeShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpCompoundShapeInternalsCdDynamicTreeKeyMask::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeInternalsCdDynamicTreeKeyMask *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpCompoundShapeInternalsSimdTreeKeyMask::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeInternalsSimdTreeKeyMask *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpCompoundShapeKeyMask::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeKeyMask *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpConstraintCollisionFilter::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintCollisionFilter *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpConvexShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpCylinderShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCylinderShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpDecoratorShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpDecoratorShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpDefaultHeightFieldGeometry::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpDefaultHeightFieldGeometry *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpDummyShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpDummyShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpExternMeshShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpGroupCollisionFilter::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpGroupCollisionFilter *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpHeightFieldBoundingVolume::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpHeightFieldBoundingVolume *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpHeightFieldGeometry::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpHeightFieldGeometry *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpHeightFieldShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpHeightFieldShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpMalleableConstraintData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpMalleableConstraintData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpMaskedCompoundShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpMaskedCompoundShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpMeshShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpMeshShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpMountedBallGun::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpMountedBallGun *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpPairCollisionFilter::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpPairCollisionFilter *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpPositionConstraintMotor::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpPositionConstraintMotor *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpRagdollData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpRagdollData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpScaledConvexShapeBase::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpScaledConvexShapeBase *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpShapeTagCodec::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeTagCodec *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpSphereShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpSphereShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpSpringDamperConstraintMotor::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpSpringDamperConstraintMotor *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpTriangleShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpTriangleShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpVelocityConstraintMotor::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVelocityConstraintMotor *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpThreadSafeObjectPool<hknpMaterial>::relocateBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpMaterial *
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpMaterial> *
            ctypes.c_ulonglong,  # hknpMaterial *
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPoolElement<hknpMaterial> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hknpThreadSafeObjectPool<hknpMotionProperties>::relocateBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpMotionProperties *
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpMotionProperties> *
            ctypes.c_ulonglong,  # hknpMotionProperties *
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPoolElement<hknpMotionProperties> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkArrayBase<hkPair<hknpShape const *,hkRefPtr<hknpShape>>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<hknpShape const *,hkRefPtr<hknpShape> > > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpScaledConvexShapeBase::hknpScaledConvexShapeBase": {
        "hknpScaledConvexShapeBase *, const hknpScaledConvexShapeBase *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpScaledConvexShapeBase *
                ctypes.c_ulonglong,  # const hknpScaledConvexShapeBase *
            ]
        ),
        "hknpScaledConvexShapeBase *, const hknpConvexShape *, const hkVector4f *, hknpShape::ScaleMode": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpScaledConvexShapeBase *
                ctypes.c_ulonglong,  # const hknpConvexShape *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_uint32,  # hknpShape::ScaleMode
            ]
        ),
    },
    "hknpScaledConvexShape::~hknpScaledConvexShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpScaledConvexShape *
        ]
    ),
    "hknpDecoratorShape::~hknpDecoratorShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDecoratorShape *
        ]
    ),
    "hknpScaledConvexShapeFunctions::buildMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpScaledConvexShapeFunctions::buildSurfaceGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::BuildSurfaceGeometryConfig *
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpScaledConvexShapeFunctions::calcAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpScaledConvexShapeFunctions::calcBoundingRadius": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpScaledConvexShapeFunctions::calcBoundingSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpScaledConvexShapeFunctions::calcInnerRadius": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpScaledConvexShapeFunctions::calcInnerSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpScaledConvexShapeFunctions::calcMinAngleBetweenFaces": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpScaledConvexShapeFunctions::calcSurfaceVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::SurfaceVelocityInput *
            ctypes.c_ulonglong,  # hknpShape::SurfaceVelocityOutput *
        ]
    ),
    "hknpScaledConvexShapeFunctions::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpScaledConvexShapeFunctions::convertVertexIdsToVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpScaledConvexShapeBase::createInPlace": {
        "hknpShapeBuffer *, const hknpConvexShape *, const hknpShapeQueryScalingData *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hknpScaledConvexShapeBase *
            argtypes=[
                ctypes.c_ulonglong,  # hknpShapeBuffer *
                ctypes.c_ulonglong,  # const hknpConvexShape *
                ctypes.c_ulonglong,  # const hknpShapeQueryScalingData *
            ]
        ),
        "hknpShapeBuffer *, const hknpConvexShape *, const hkVector4f *, hknpShape::ScaleMode": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpShapeBuffer *
                ctypes.c_ulonglong,  # const hknpConvexShape *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_uint32,  # hknpShape::ScaleMode
            ]
        ),
    },
    "hkSignal2<hknpShape const *,uchar>::fire": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpShape const *,unsigned char> *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "hknpScaledConvexShapeFunctions::getCircularSymmetryAxis": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCircularSymmetryAxis *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "hknpScaledConvexShapeFunctions::getConvexVertices": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpConvexVertexShape *
            ctypes.c_ulonglong,  # hkArrayView<hkFloat3> *
            ctypes.c_ulonglong,  # __int128 *
        ]
    ),
    "hknpScaledConvexShapeFunctions::getFaceInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hknpAngularTimType *
        ]
    ),
    "hknpScaledConvexShapeFunctions::getFaceVertices": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpConvexShapeFunctions::getLeafShapes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpShapeCollector *
        ]
    ),
    "hknpScaledConvexShapeFunctions::getMemoryFootprint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpShapeSet *
        ]
    ),
    "hknpScaledConvexShapeFunctions::getMutationSignals": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpShapeProperties *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpScaledConvexShapeFunctions::getNumberOfFaces": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpScaledConvexShapeFunctions::getNumberOfSupportSpheres": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpScaledConvexShapeFunctions::getSupportSpheres": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkFloat3 *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkFloat3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpScaledConvexShapeFunctions::getSupportingFace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkcdGsk::Cache *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hknpAngularTimType *
        ]
    ),
    "hknpScaledConvexShapeFunctions::getSupportingVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpScaledConvexShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hkHashMap<hknpShape const *,hkRefPtr<hknpShape>,hkBuiltinContainerAllocator<0,0>>::insert": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # hkHashMap<hknpShape const *,hkRefPtr<hknpShape>,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # const hknpShape *const *
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
        ]
    ),
    "hknpScaledConvexShapeFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpScaledConvexShape::makeCloneImpl": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpScaledConvexShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpScaledConvexShape *
            ctypes.c_ulonglong,  # hkRefPtr<hknpScaledConvexShape> *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpScaledConvexShapeFunctions::onCoreShapeMutated": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShape *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "hknpScaledConvexShapeFunctions::queryAabb": {
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hknpCollisionQueryCollector *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
    },
    "hknpShapeNullFunctions::queryPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCollisionQueryCollector::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpScaledConvexShapeBase::setScaleNoSignals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpScaledConvexShapeBase *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_uint32,  # hknpShape::ScaleMode
        ]
    ),
    "AkQueuedMsg::Sizeof_OpenDynamicSequence": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "hkSingletonUtil__init_hknpShapeFunctionsSingleton_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShapeFunctionsSingleton> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShapeFunctionsSingleton> *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hknpShapeFunctions::hknpShapeFunctions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeFunctions *
        ]
    ),
    "hknpShapeFunctionsSingleton::hknpShapeFunctionsSingleton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeFunctionsSingleton *
        ]
    ),
    "hknpCompressedMeshShapeInternals::KeyMaskFunctions::commitChanges": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeKeyMask *
        ]
    ),
    "hknpCompressedMeshShapeInternals::KeyMaskFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpBoxShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpBreakableCompoundShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpBreakableCompoundShapeKeyMaskFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpCapsuleShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpCompoundShapeCdDynamicTreeKeyMaskFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpCompoundShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpCompoundShapeSimdTreeKeyMaskFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpCompressedMeshShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpConvexShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpCylinderShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpDummyShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpExternMeshShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpHeightFieldShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpLodShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpMaskedCompoundShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpMaskedShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpMeshShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpParticlesColliderShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpScaledConvexShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpSphereShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpTransformedShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpTriangleShapeFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpCompressedMeshShapeInternals::KeyMaskFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
            ctypes.c_ulonglong,  # const hknpShapeKeyMask *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpShapeFunctions::registerDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeFunctions *
        ]
    ),
    "hknpCapsuleShapeFunctions::buildMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_float,  # float
        ]
    ),
    "hknpCapsuleShapeFunctions::buildSurfaceGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::BuildSurfaceGeometryConfig *
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<hkReflect::Type const *,hkReflect::Type *> > *
        ]
    ),
    "hknpCapsuleShapeFunctions::calcAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpCapsuleShapeFunctions::calcBoundingSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpCapsuleShapeFunctions::calcInnerSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpBoxShapeFunctions::calcMinAngleBetweenFaces": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpCapsuleShapeFunctions::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCapsuleShapeFunctions::getConvexVertices": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpConvexVertexShape *
            ctypes.c_ulonglong,  # hkArrayView<hkFloat3> *
        ]
    ),
    "hknpCapsuleShapeFunctions::getFaceInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hknpAngularTimType *
        ]
    ),
    "hknpCapsuleShapeFunctions::getFaceVertices": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpCapsuleShapeFunctions::getSignedDistances": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::SdfQuery *
            ctypes.c_ulonglong,  # hknpShape::SdfContactPoint *
        ]
    ),
    "hknpCapsuleShapeFunctions::getSupportSpheres": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkFloat3 *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkFloat3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpCapsuleShapeFunctions::getSupportingFace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkcdGsk::Cache *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hknpAngularTimType *
        ]
    ),
    "hknpCapsuleShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hknpCapsuleShapeFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpVehicleDefaultSteering::hknpVehicleDefaultSteering": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultSteering *
        ]
    ),
    "hknpVehicleDefaultSteering::calcMainSteeringAngle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultSteering *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # const hknpVehicleDriverInput::FilteredDriverInputOutput *
            ctypes.c_ulonglong,  # hknpVehicleSteering::SteeringAnglesOutput *
        ]
    ),
    "hknpVehicleDefaultSteering::calcSteering": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultSteering *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # const hknpVehicleDriverInput::FilteredDriverInputOutput *
            ctypes.c_ulonglong,  # hknpVehicleSteering::SteeringAnglesOutput *
        ]
    ),
    "hknpSpringAction::applyAction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpSpringAction *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_float,  # float
        ]
    ),
    "hknpSpringAction::onWorldShifted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSpringAction *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpShapeKeyMask::_getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
            ctypes.c_ulonglong,  # const hknpShapeKeyMask *
        ]
    ),
    "hkArrayBase<hknpParticle4Faces>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpParticle4Faces> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkcdSimdTree::aabbCast<hknpExternMeshShapeInternals::ShapeCastQuery>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdRay *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hknpExternMeshShapeInternals::ShapeCastQuery *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTreeUtils::aabbOverlaps_Trampoline<0,hknpExternMeshShapeInternals::AabbOverlaps<hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdSimdTree::TransposedAabb *
            ctypes.c_ulonglong,  # hknpExternMeshShapeInternals::AabbOverlaps<hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTreeUtils::aabbOverlaps_Trampoline<0,hknpExternMeshShapeInternals::AabbOverlaps<hknpCollisionQueryCollector>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdSimdTree::TransposedAabb *
            ctypes.c_ulonglong,  # hknpExternMeshShapeInternals::AabbOverlaps<hknpCollisionQueryCollector> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb6BytesCodec>,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbCastWrapperExternal<hknpExternMeshShapeInternals::ShapeCastQuery>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb6BytesCodec> *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbCastWrapperExternal<hknpExternMeshShapeInternals::ShapeCastQuery> *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb6BytesCodec>,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpExternMeshShapeInternals::AabbOverlaps<hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb6BytesCodec> *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpExternMeshShapeInternals::AabbOverlaps<hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > > > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb6BytesCodec>,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpExternMeshShapeInternals::AabbOverlaps<hknpCollisionQueryCollector>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb6BytesCodec> *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpExternMeshShapeInternals::AabbOverlaps<hknpCollisionQueryCollector> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb6BytesCodec>,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpExternMeshShapeInternals::AabbOverlaps<hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb6BytesCodec> *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpExternMeshShapeInternals::AabbOverlaps<hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > > > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb6BytesCodec>,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpExternMeshShapeInternals::AabbOverlaps<hknpCollisionQueryCollector>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb6BytesCodec> *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpExternMeshShapeInternals::AabbOverlaps<hknpCollisionQueryCollector> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb6BytesCodec>,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::RayCastWrapperExternal<hknpExternMeshShapeInternals::RayCastQuery>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb6BytesCodec> *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::RayCastWrapperExternal<hknpExternMeshShapeInternals::RayCastQuery> *
        ]
    ),
    "hknpExternMeshShape::~hknpExternMeshShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShape *
        ]
    ),
    "hknpExternMeshShape::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShape *
        ]
    ),
    "hknpExternMeshShapeFunctions::buildSurfaceGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::BuildSurfaceGeometryConfig *
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<hkReflect::Type const *,hkReflect::Type *> > *
        ]
    ),
    "hknpExternMeshShapeFunctions::calcAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpExternMeshShapeFunctions::calcSurfaceVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::SurfaceVelocityInput *
            ctypes.c_ulonglong,  # hknpShape::SurfaceVelocityOutput *
        ]
    ),
    "hknpExternMeshShapeFunctions::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpExternMeshShapeFunctions::castShapeImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpExternMeshShapeGeometry::generateGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShapeGeometry *
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hknpExternMeshShape::getAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShape *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpExternMeshShapeFunctions::getLeafShapes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpShapeCollector *
        ]
    ),
    "hknpExternMeshShapeFunctions::getMemoryFootprint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpShapeSet *
        ]
    ),
    "hknpExternMeshShapeFunctions::getMutationSignals": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpShapeSignals *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpExternMeshShapeGeometry::getPrimitiveShapeTag": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShapeGeometry *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpExternMeshShapeGeometry::getPrimitiveVertices": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShapeGeometry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpExternMeshShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hknpExternMeshShape::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShape *
            ctypes.c_ulonglong,  # const hknpExternMeshShapeGeometry *
            ctypes.c_ulonglong,  # const hknpExternMeshShapeData *
        ]
    ),
    "hknpExternMeshShapeFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpExternMeshShapeFunctions::queryAabb": {
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hknpCollisionQueryCollector *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
    },
    "hkArrayBase<hknpMotionId>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpConstraintGroupDirectSolver::BlockWFIInfo> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpBodyIntegrator::hknpBodyIntegrator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyIntegrator *
        ]
    ),
    "hknpDeflectedLinearCast::Output::~Output": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeflectedLinearCast::Output *
        ]
    ),
    "hknpBodyIntegrator::calcMotionWeldManifoldType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyIntegrator *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpManifold *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const hknpWeldingUtil::MotionWeldConfig *
        ]
    ),
    "hknpWorld::getBodyQualityLibrary": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpBodyQualityLibrary *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorld::getCollisionFilter": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpParticlesColliderManager *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorld::getMaterialLibrary": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpMaterialLibrary *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorld::getShapeTagCodec": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShapeTagCodec *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpBodyIntegrator::motionRangeBreached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyIntegrator *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpMotion *
            ctypes.c_ulonglong,  # hknpBodyIntegrator::MotionData *
        ]
    ),
    "hknpBodyIntegrator::motionWeldToiPlaneBreached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyIntegrator *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpMotionWeldTOICommand **
            ctypes.c_int32,  # int
        ]
    ),
    "hknpBodyIntegrator::processMotionRangeBreach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyIntegrator *
            ctypes.c_ulonglong,  # const hknpMotion *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
        ]
    ),
    "hknpFirstPersonGun::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFirstPersonGun *
        ]
    ),
    "hknpFirstPersonGun::calcVelocityToTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpFirstPersonGun::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpFirstPersonGun *
        ]
    ),
    "hknpFirstPersonGun::quitGun": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRichPresence *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_double,  # const long double
        ]
    ),
    "hknpClosestHitCollector::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpClosestHitCollector *
        ]
    ),
    "hknpFirstPersonGun::sweepSphere": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpWorld *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hknpFirstPersonGun::SweepSphereOut *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "hknpVehicleDefaultSuspension::calcSuspension": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultSuspension *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # const hknpVehicleWheelCollide::CollisionDetectionWheelOutput *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpDefaultExternMeshShapeGeometry::~hknpDefaultExternMeshShapeGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDefaultExternMeshShapeGeometry *
        ]
    ),
    "hknpDefaultExternMeshShapeGeometry::getNumTriangles": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpDefaultExternMeshShapeGeometry *
        ]
    ),
    "hknpDefaultExternMeshShapeGeometry::getTriangleShapeTag": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpDefaultExternMeshShapeGeometry *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpDefaultExternMeshShapeGeometry::getTriangleVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDefaultExternMeshShapeGeometry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpDefaultExternMeshShapeGeometry::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpExternMeshShapeGeometry> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpDefaultExternMeshShapeGeometry *
            ctypes.c_ulonglong,  # hkRefPtr<hknpExternMeshShapeGeometry> *
        ]
    ),
    "hknpExtendedExternMeshShapeGeometry::getNumQuads": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceContainer *
        ]
    ),
    "hkMemoryMeshShape::getNumSections": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshShape *
        ]
    ),
    "hknpExtendedExternMeshShapeGeometry::getQuadShapeTag": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpExtendedExternMeshShapeGeometry *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpExtendedExternMeshShapeGeometry::getQuadVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpExtendedExternMeshShapeGeometry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpExtendedExternMeshShapeGeometry::getTriangleShapeTag": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpExtendedExternMeshShapeGeometry *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpExtendedExternMeshShapeGeometry::getTriangleVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpExtendedExternMeshShapeGeometry *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpBallGun::~hknpBallGun": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBallGun *
        ]
    ),
    "hknpBallGun::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBallGun *
        ]
    ),
    "hknpBallGun::fireGun": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBallGun *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hkTransformf *
        ]
    ),
    "hknpBallGun::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBallGun *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hkQueue<hknpBodyId,hkBuiltinContainerAllocator<0,0>>::setCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkQueue<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpGravityGun::hknpGravityGun": {
        "hknpGravityGun *, const hknpGravityGun *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpGravityGun *
                ctypes.c_ulonglong,  # const hknpGravityGun *
            ]
        ),
        "hknpGravityGun *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpGravityGun *
            ]
        ),
    },
    "hknpGravityGun::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGravityGun *
        ]
    ),
    "hknpGravityGun::createClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefNew<hknpFirstPersonGun> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpGravityGun *
            ctypes.c_ulonglong,  # hkRefNew<hknpFirstPersonGun> *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpGravityGun::dropObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpBodyId *
        argtypes=[
            ctypes.c_ulonglong,  # hknpGravityGun *
            ctypes.c_ulonglong,  # hknpBodyId *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpGravityGun::gravityGunPull": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGravityGun *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpGravityGun::gravityGunPush": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGravityGun *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hknpMotion *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpGravityGun::pickBody": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGravityGun *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpGravityGun::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGravityGun *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpGravityGun::syncCloneFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGravityGun *
            ctypes.c_ulonglong,  # const hknpFirstPersonGun *
        ]
    ),
    "hknpVehicleDefaultTransmission::hknpVehicleDefaultTransmission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultTransmission *
        ]
    ),
    "hknpVehicleDefaultTransmission::calcIsReversing": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultTransmission *
            ctypes.c_ulonglong,  # hkBool *
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # hknpVehicleTransmission::TransmissionOutput *
        ]
    ),
    "hknpVehicleDefaultTransmission::calcMainTransmittedTorque": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultTransmission *
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # hknpVehicleTransmission::TransmissionOutput *
        ]
    ),
    "hknpVehicleDefaultTransmission::calcTransmission": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultTransmission *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # hknpVehicleTransmission::TransmissionOutput *
        ]
    ),
    "hknpVehicleDefaultTransmission::calcTransmissionRPM": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultTransmission *
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # hknpVehicleTransmission::TransmissionOutput *
        ]
    ),
    "hknpVehicleDefaultTransmission::getCurrentRatio": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultTransmission *
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # hknpVehicleTransmission::TransmissionOutput *
        ]
    ),
    "hknpVehicleDefaultTransmission::updateCurrentGear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultTransmission *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # hknpVehicleTransmission::TransmissionOutput *
        ]
    ),
    "hknpHeightFieldBoundingVolume::hknpHeightFieldBoundingVolume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpHeightFieldBoundingVolume *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpHeightFieldGeometry::hknpHeightFieldGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpHeightFieldGeometry *
        ]
    ),
    "hknpMinMaxQuadTree::hknpMinMaxQuadTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMinMaxQuadTree *
            ctypes.c_ulonglong,  # const hknpMinMaxQuadTree *
        ]
    ),
    "hknpHeightFieldBoundingVolume::~hknpHeightFieldBoundingVolume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpHeightFieldBoundingVolume *
        ]
    ),
    "hknpHeightFieldShape::~hknpHeightFieldShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpHeightFieldShape *
        ]
    ),
    "hknpMinMaxQuadTree::~hknpMinMaxQuadTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMinMaxQuadTree *
        ]
    ),
    "hknpHeightFieldBoundingVolume::_getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
            ctypes.c_ulonglong,  # const hknpHeightFieldBoundingVolume *
        ]
    ),
    "hknpHeightFieldShapeFunctions::calcAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpHeightFieldBoundingVolume::getCoarseMinMax": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpHeightFieldBoundingVolume *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpHeightFieldShapeFunctions::getMutationSignals": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpShapeProperties *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpHeightFieldShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hknpHeightFieldGeometry::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpHeightFieldGeometry> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpHeightFieldGeometry *
            ctypes.c_ulonglong,  # hkRefPtr<hknpHeightFieldGeometry> *
        ]
    ),
    "hknpHeightFieldShapeFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpProjectileGun::~hknpProjectileGun": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpProjectileGun *
        ]
    ),
    "hknpProjectileGun::clearProjectiles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpProjectileGun *
        ]
    ),
    "hknpProjectileGun::destroyAgedProjectiles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpProjectileGun *
            ctypes.c_float,  # float
        ]
    ),
    "hknpProjectileGun::fireGun": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpProjectileGun *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hkTransformf *
        ]
    ),
    "hknpProjectileGun::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpProjectileGun *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpTyremarkPoint::hknpTyremarkPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpTyremarkPoint *
        ]
    ),
    "hknpTyremarksWheel::hknpTyremarksWheel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpTyremarksWheel *
        ]
    ),
    "hknpTyremarksInfo::~hknpTyremarksInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpTyremarksInfo *
        ]
    ),
    "hknpTyremarksInfo::getWheelTyremarksStrips": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpTyremarksInfo *
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpTyremarksInfo::updateTyremarksInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpTyremarksInfo *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
        ]
    ),
    "hknpVehicleDefaultVelocityDamper::hknpVehicleDefaultVelocityDamper": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultVelocityDamper *
        ]
    ),
    "hkArrayBase<hknpVehicleLinearCastWheelCollide::WheelState>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpVehicleLinearCastWheelCollide::WheelState> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpVehicleLinearCastWheelCollide::hknpVehicleLinearCastWheelCollide": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleLinearCastWheelCollide *
        ]
    ),
    "hknpVehicleLinearCastWheelCollide::~hknpVehicleLinearCastWheelCollide": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleLinearCastWheelCollide *
        ]
    ),
    "hknpVehicleLinearCastWheelCollide::castSingleWheel": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleLinearCastWheelCollide *
            ctypes.c_ulonglong,  # hkBool *
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const hknpWorldReader *
            ctypes.c_ulonglong,  # hknpShapeCastQuery *
            ctypes.c_ulonglong,  # hknpShapeCastQueryResult *
        ]
    ),
    "hknpVehicleLinearCastWheelCollide::collideWheels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleLinearCastWheelCollide *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # hknpVehicleWheelCollide::CollisionDetectionWheelOutput *
            ctypes.c_ulonglong,  # const hknpWorldReader *
        ]
    ),
    "hknpVehicleLinearCastWheelCollide::getCollisionOutputFromCastResult": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleLinearCastWheelCollide *
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeCastQueryResult *
            ctypes.c_ulonglong,  # const hknpWorldReader *
            ctypes.c_ulonglong,  # hknpVehicleWheelCollide::CollisionDetectionWheelOutput *
        ]
    ),
    "hknpVehicleLinearCastWheelCollide::getCollisionOutputWithoutHit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleRayCastWheelCollide *
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # hknpVehicleWheelCollide::CollisionDetectionWheelOutput *
        ]
    ),
    "hknpVehicleLinearCastWheelCollide::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleLinearCastWheelCollide *
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
        ]
    ),
    "hknpVehicleLinearCastWheelCollide::makeWheelShape": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConvexShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleLinearCastWheelCollide *
            ctypes.c_ulonglong,  # hkRefPtr<hknpConvexShape> *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "hknpVehicleLinearCastWheelCollide::setCollisionFilterInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleLinearCastWheelCollide *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hknpVehicleLinearCastWheelCollide::updateBeforeCollisionDetection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleLinearCastWheelCollide *
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
        ]
    ),
    "hknpVehicleLinearCastWheelCollide::updateWheelState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleLinearCastWheelCollide *
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "hknpVehicleLinearCastWheelCollide::wheelCollideCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleLinearCastWheelCollide *
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # hknpVehicleWheelCollide::CollisionDetectionWheelOutput *
        ]
    ),
    "hknpBodyCinfo::hknpBodyCinfo": {
        "hknpBodyCinfo *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpBodyCinfo *
            ]
        ),
        "hknpBodyCinfo *, const hknpBodyCinfo *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpBodyCinfo *
                ctypes.c_ulonglong,  # const hknpBodyCinfo *
            ]
        ),
    },
    "hknpBodyCinfo::~hknpBodyCinfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyCinfo *
        ]
    ),
    "hknpConstraintData::_getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpConstraintData::enableDirectSolverImpl": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_char,  # char
        ]
    ),
    "hknpConstraintData::getConstraintInfoUtil": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintAtom *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpConstraintData::ConstraintInfo *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpBody::setShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBody *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpBody::syncStaticBodyFromMotionTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBody *
        ]
    ),
    "hknpBody::updateBodyFromMotionTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBody *
            ctypes.c_ulonglong,  # const hknpMotion *
            ctypes.c_ulonglong,  # const hkQuaternionf *
        ]
    ),
    "hknpBody::updateComCenteredBoundingRadius": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBody *
        ]
    ),
    "hknpVehicleData::hknpVehicleData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleData *
        ]
    ),
    "hknpVehicleData::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleData *
            ctypes.c_ulonglong,  # const hkArray<hknpVehicleSuspension::SuspensionWheelParameters,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hknpMotion *
        ]
    ),
    "hkArrayBase<hknpVehicleInstance::WheelInfo>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpVehicleInstance::WheelInfo> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpVehicleInstance::~hknpVehicleInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
        ]
    ),
    "hknpVehicleInstance::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
        ]
    ),
    "hknpVehicleInstance::applyAction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_double,  # double
        ]
    ),
    "hknpVehicleInstance::applyAerodynamicDrag": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hknpVehicleAerodynamics::AerodynamicsDragOutput *
            ctypes.c_ulonglong,  # hknpVehicleFrictionSolverParams *
        ]
    ),
    "hknpVehicleInstance::applyDampingToAxleAccumulators": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hknpBodyId *
            ctypes.c_ulonglong,  # hknpVehicleFrictionSolverParams *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpVehicleInstance::applyResultsToWheelInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hknpBodyId *
            ctypes.c_ulonglong,  # const hknpVehicleFrictionSolverParams *
        ]
    ),
    "hknpVehicleInstance::applySuspensionForces": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hkArray<float,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hknpBodyId *
            ctypes.c_ulonglong,  # hknpVehicleFrictionSolverParams *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpVehicleInstance::buildAccumulator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
            ctypes.c_ulonglong,  # const hknpMotion *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hknpVelAccumulatorJacobianBuilding *
        ]
    ),
    "hknpVehicleInstance::calcCurrentPositionAndRotation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hknpVehicleSuspension *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkQuaternionf *
        ]
    ),
    "hknpVehicleInstance::calcGroundBodyVelocities": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # const hknpVehicleFrictionSolverAxleParams *
            ctypes.c_ulonglong,  # hknpBodyId *
            ctypes.c_ulonglong,  # hknpVehicleInstance::StepResults *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpVehicleInstance::calcKMPH": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
        ]
    ),
    "hknpVehicleInstance::calcMPH": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
        ]
    ),
    "hknpVehicleInstance::calcRPM": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
        ]
    ),
    "hknpVehicleInstance::getAxleParamsFromWheel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hknpVehicleFrictionSolverAxleParams *
        ]
    ),
    "hknpVehicleInstance::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
        ]
    ),
    "hknpVehicleInstance::onActionAdded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
            ctypes.c_ulonglong,  # hknpActionManager *
        ]
    ),
    "hknpVehicleInstance::onActionRemoved": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
            ctypes.c_ulonglong,  # hknpActionManager *
        ]
    ),
    "hknpVehicleInstance::prepareAxleParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hkArray<float,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # const hkArray<float,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hknpBodyId *
            ctypes.c_ulonglong,  # hknpVehicleFrictionSolverParams *
            ctypes.c_ulonglong,  # hknpVelAccumulatorJacobianBuilding *
            ctypes.c_ulonglong,  # hknpVelAccumulatorJacobianBuilding *
        ]
    ),
    "hknpVehicleInstance::simulateVehicle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # const hknpVehicleAerodynamics::AerodynamicsDragOutput *
            ctypes.c_ulonglong,  # const hkArray<float,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # const hkArray<float,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hknpVehicleInstance::StepResults *
        ]
    ),
    "hknpVehicleInstance::stepVehicleUsingWheelCollideOutput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # const hknpVehicleWheelCollide::CollisionDetectionWheelOutput *
            ctypes.c_ulonglong,  # hknpVehicleInstance::StepResults *
        ]
    ),
    "hknpVehicleInstance::updateBeforeCollisionDetection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
        ]
    ),
    "hknpVehicleInstance::updateComponents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hknpVehicleWheelCollide::CollisionDetectionWheelOutput *
            ctypes.c_ulonglong,  # hknpVehicleAerodynamics::AerodynamicsDragOutput *
            ctypes.c_ulonglong,  # hkArray<float,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArray<float,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpVehicleInstance::updateWheels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleInstance *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hknpVehicleWheelCollide::CollisionDetectionWheelOutput *
        ]
    ),
    "hknpVehicleRayCastWheelCollide::hknpVehicleRayCastWheelCollide": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleRayCastWheelCollide *
        ]
    ),
    "hknpVehicleRayCastWheelCollide::calcWheelsAABB": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleRayCastWheelCollide *
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpVehicleRayCastWheelCollide::collideWheels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleRayCastWheelCollide *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # hknpVehicleWheelCollide::CollisionDetectionWheelOutput *
            ctypes.c_ulonglong,  # const hknpWorldReader *
        ]
    ),
    "hknpVehicleRayCastWheelCollide::getCollisionOutputFromCastResult": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleRayCastWheelCollide *
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const hknpClosestHitCollector *
            ctypes.c_ulonglong,  # hknpVehicleWheelCollide::CollisionDetectionWheelOutput *
            ctypes.c_ulonglong,  # const hknpWorldReader *
        ]
    ),
    "hknpVehicleRayCastWheelCollide::setCollisionFilterInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleRayCastWheelCollide *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkArrayBase<hknpMaterial>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpMaterial> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpMotionProperties>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpMotionProperties> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpPhysicsSystemData::~hknpPhysicsSystemData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPhysicsSystemData *
        ]
    ),
    "hknpPhysicsSystemData::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPhysicsSystemData *
        ]
    ),
    "hkArrayBase<ushort>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<unsigned short> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpVehicleDefaultAerodynamics::hknpVehicleDefaultAerodynamics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultAerodynamics *
        ]
    ),
    "hknpVehicleDefaultAerodynamics::calcAerodynamicDrag": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultAerodynamics *
            ctypes.c_double,  # double
        ]
    ),
    "hknpVehicleDefaultAerodynamics::calcAerodynamicLift": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultAerodynamics *
            ctypes.c_float,  # float
        ]
    ),
    "hknpVehicleDefaultAerodynamics::calcAerodynamics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultAerodynamics *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # hknpVehicleAerodynamics::AerodynamicsDragOutput *
        ]
    ),
    "hknpCircularSurfaceVelocity::hknpCircularSurfaceVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCircularSurfaceVelocity *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpCircularSurfaceVelocity::calcSurfaceVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCircularSurfaceVelocity *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpLinearSurfaceVelocity::hknpLinearSurfaceVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearSurfaceVelocity *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_uint32,  # hknpSurfaceVelocity::Space
            ctypes.c_uint32,  # hknpLinearSurfaceVelocity::ProjectMethod
        ]
    ),
    "hknpLinearSurfaceVelocity::calcSurfaceVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearSurfaceVelocity *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpDisableCollisionFilter::hknpDisableCollisionFilter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDisableCollisionFilter *
        ]
    ),
    "hknpDisableCollisionFilter::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpCollisionFilter> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpDisableCollisionFilter *
            ctypes.c_ulonglong,  # hkRefPtr<hknpCollisionFilter> *
        ]
    ),
    "hknpShapeProperties::Entry::~Entry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeProperties::Entry *
        ]
    ),
    "hknpShapePropertyBase::_getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
            ctypes.c_ulonglong,  # const hknpShapePropertyBase *
        ]
    ),
    "hknpShapeProperties::addProperty": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeProperties *
            ctypes.c_ulonglong,  # hknpShapePropertyBase *
        ]
    ),
    "hknpShapeProperties::getMemoryFootprint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeProperties *
        ]
    ),
    "hknpShapeProperties::getProperty": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpShapePropertyBase *
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeProperties *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "hknpVehicleDefaultBrake::hknpVehicleDefaultBrake": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultBrake *
        ]
    ),
    "hknpVehicleDefaultBrake::calcBreakingInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultBrake *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # const hknpVehicleDriverInput::FilteredDriverInputOutput *
            ctypes.c_ulonglong,  # hknpVehicleBrake::WheelBreakingOutput *
        ]
    ),
    "hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<5,5,5,16>>::hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<5,5,5,16>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<5,5,5,16> > *
        ]
    ),
    "hknpGroupCollisionFilter::hknpGroupCollisionFilter": {
        "hknpGroupCollisionFilter *, const hknpGroupCollisionFilter *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpGroupCollisionFilter *
                ctypes.c_ulonglong,  # const hknpGroupCollisionFilter *
            ]
        ),
        "hknpGroupCollisionFilter *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpGroupCollisionFilter *
            ]
        ),
    },
    "hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<5,5,5,16>>::copyFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<5,5,5,16> > *
            ctypes.c_ulonglong,  # const hknpCollisionFilter *
        ]
    ),
    "hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<6,5,5,16>>::copyFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<6,5,5,16> > *
            ctypes.c_ulonglong,  # const hknpCollisionFilter *
        ]
    ),
    "hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<5,5,5,16>>::filterBodyPairs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<5,5,5,16> > *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hknpBodyIndexPair *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<6,5,5,16>>::filterBodyPairs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<6,5,5,16> > *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hknpBodyIndexPair *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<6,5,5,16>>::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<6,5,5,16> > *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<5,5,5,16>>::isCollisionEnabled": {
        "hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<5,5,5,16> > *, hknpCollisionQueryType::Enum, const hknpQueryFilterData *, const hknpBody *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<5,5,5,16> > *
                ctypes.c_uint32,  # hknpCollisionQueryType::Enum
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpBody *
            ]
        ),
        "hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<5,5,5,16> > *, hknpCollisionQueryType::Enum, bool, const hknpCollisionFilter::FilterInput *, const hknpCollisionFilter::FilterInput *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<5,5,5,16> > *
                ctypes.c_uint32,  # hknpCollisionQueryType::Enum
                ctypes.c_ubyte,  # bool
                ctypes.c_ulonglong,  # const hknpCollisionFilter::FilterInput *
                ctypes.c_ulonglong,  # const hknpCollisionFilter::FilterInput *
            ]
        ),
    },
    "hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<6,5,5,16>>::isCollisionEnabled": {
        "hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<6,5,5,16> > *, hknpCollisionQueryType::Enum, const hknpQueryFilterData *, const hknpBody *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<6,5,5,16> > *
                ctypes.c_uint32,  # hknpCollisionQueryType::Enum
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpBody *
            ]
        ),
        "hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<6,5,5,16> > *, hknpCollisionQueryType::Enum, bool, const hknpCollisionFilter::FilterInput *, const hknpCollisionFilter::FilterInput *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # hknpGroupCollisionFilterBase<hknpGroupCollisionFilterTypes::Config<6,5,5,16> > *
                ctypes.c_uint32,  # hknpCollisionQueryType::Enum
                ctypes.c_ubyte,  # bool
                ctypes.c_ulonglong,  # const hknpCollisionFilter::FilterInput *
                ctypes.c_ulonglong,  # const hknpCollisionFilter::FilterInput *
            ]
        ),
    },
    "hknpGroupCollisionFilter::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpCollisionFilter> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpGroupCollisionFilter *
            ctypes.c_ulonglong,  # hkRefPtr<hknpCollisionFilter> *
        ]
    ),
    "hknpVehicleDefaultAnalogDriverInput::hknpVehicleDefaultAnalogDriverInput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultAnalogDriverInput *
        ]
    ),
    "hknpVehicleDefaultAnalogDriverInput::calcAcceleratorInput": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultAnalogDriverInput *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # const hknpVehicleDriverInputAnalogStatus *
        ]
    ),
    "hknpVehicleDefaultAnalogDriverInput::calcBrakeInput": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultAnalogDriverInput *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # const hknpVehicleDriverInputAnalogStatus *
        ]
    ),
    "hknpVehicleDefaultAnalogDriverInput::calcDriverInput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultAnalogDriverInput *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # const hknpVehicleDriverInputStatus *
            ctypes.c_ulonglong,  # hknpVehicleDriverInput::FilteredDriverInputOutput *
        ]
    ),
    "hknpVehicleDefaultAnalogDriverInput::calcSteeringInput": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultAnalogDriverInput *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # const hknpVehicleDriverInputAnalogStatus *
        ]
    ),
    "hknpVehicleDefaultAnalogDriverInput::calcTryingToReverse": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultAnalogDriverInput *
            ctypes.c_ulonglong,  # hkBool *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # const hknpVehicleDriverInputAnalogStatus *
        ]
    ),
    "hknpVehicleDriverInputAnalogStatus::clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpVehicleDriverInputStatus *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDriverInputAnalogStatus *
        ]
    ),
    "hknpVehicleDefaultEngine::hknpVehicleDefaultEngine": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultEngine *
        ]
    ),
    "hknpVehicleDefaultEngine::calcEngineInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleDefaultEngine *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const hknpVehicleInstance *
            ctypes.c_ulonglong,  # const hknpVehicleDriverInput::FilteredDriverInputOutput *
            ctypes.c_ulonglong,  # const hknpVehicleTransmission::TransmissionOutput *
            ctypes.c_ulonglong,  # hknpVehicleEngine::EngineOutput *
        ]
    ),
    "hknpShape::hknpShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShape *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpShape::_getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpShape::setMassProperties": {
        "hknpShape *, const hknpShape::MassConfig *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpShape *
                ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ]
        ),
        "hknpShape *, const hkDiagonalizedMassProperties *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpShape *
                ctypes.c_ulonglong,  # const hkDiagonalizedMassProperties *
            ]
        ),
    },
    "hknpShape::SurfaceVelocityOutput::setVelocityLs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShape::SurfaceVelocityOutput *
            ctypes.c_ulonglong,  # const hknpShape::SurfaceVelocityInput *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpDefaultBroadPhaseConfig::getLayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDefaultBroadPhaseConfig *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # hknpBroadPhaseConfig::Layer *
        ]
    ),
    "hknpDefaultBroadPhaseConfig::getLayerIndices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDefaultBroadPhaseConfig *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hknpBody *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "hknpLinearParametricCurve::hknpLinearParametricCurve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearParametricCurve *
        ]
    ),
    "hknpLinearParametricCurve::addPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearParametricCurve *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpLinearParametricCurve::clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpParametricCurve *
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearParametricCurve *
        ]
    ),
    "hknpLinearParametricCurve::getBinormal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearParametricCurve *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpLinearParametricCurve::getEnd": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearParametricCurve *
        ]
    ),
    "hknpLinearParametricCurve::getLengthFromStart": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearParametricCurve *
            ctypes.c_float,  # float
        ]
    ),
    "hknpLinearParametricCurve::getNearestPoint": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearParametricCurve *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpLinearParametricCurve::getPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearParametricCurve *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpLinearParametricCurve::getPointsToDraw": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearParametricCurve *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpLinearParametricCurve::getTangent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearParametricCurve *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpLinearParametricCurve::isClosedLoop": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearParametricCurve *
            ctypes.c_ulonglong,  # hkBool *
        ]
    ),
    "hknpLinearParametricCurve::transformPoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearParametricCurve *
            ctypes.c_ulonglong,  # const hkTransformf *
        ]
    ),
    "hknpMotionProperties::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotionProperties *
            ctypes.c_uint32,  # hknpMotionPropertiesId::Preset
            ctypes.c_double,  # double
        ]
    ),
    "hknpMotion::buildEffMassMatrixAt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotion *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
        ]
    ),
    "hknpMotion::getAngularVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotion *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpMotion::getInertiaWorld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotion *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
        ]
    ),
    "hknpMotion::getInverseInertiaWorld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotion *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
        ]
    ),
    "hknpMotion::getPointVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotion *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpMotion::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotion *
            ctypes.c_ulonglong,  # const hknpMotionCinfo *
        ]
    ),
    "hknpMotion::reintegrate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotion *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
        ]
    ),
    "hknpMotion::setAngularVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotion *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpMotion::setMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotion *
            ctypes.c_ulonglong,  # const hkDiagonalizedMassProperties *
            ctypes.c_ulonglong,  # const hkQTransform *
        ]
    ),
    "hknpMotion::setPointVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotion *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpMotion::syncSpeedLimits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotion *
            ctypes.c_ulonglong,  # const hknpMotionPropertiesLibrary *
        ]
    ),
    "hknpConstraint::hknpConstraint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraint *
            ctypes.c_ulonglong,  # const hknpConstraint *
        ]
    ),
    "hknpConstraint::allocateRuntime": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraint *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hknpConstraint::transform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraint *
            ctypes.c_ulonglong,  # const hkTransformf *
        ]
    ),
    "hknpShapeTagCodec::copyFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeTagCodec *
            ctypes.c_ulonglong,  # const hknpShapeTagCodec *
        ]
    ),
    "hknpShapeTagCodec::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShapeTagCodec> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeTagCodec *
            ctypes.c_ulonglong,  # hkRefPtr<hknpShapeTagCodec> *
        ]
    ),
    "hkArrayBase<hkFloat3>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpConstraintAtomSolverSetup::ConstraintState> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkFloat3>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpConstraintAtomSolverSetup::ConstraintState> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpConvexShapeBuildConfig::hknpConvexShapeBuildConfig": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexShapeBuildConfig *
        ]
    ),
    "hknpConvexShape::~hknpConvexShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexShape *
        ]
    ),
    "hknpConvexShape::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexShape *
        ]
    ),
    "hknpConvexShape::buildConnectivity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexShape *
            ctypes.c_ulonglong,  # _DWORD *
        ]
    ),
    "hknpConvexShapeFunctions::buildMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpConvexShapeFunctions::buildSurfaceGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::BuildSurfaceGeometryConfig *
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<hkReflect::Type const *,hkReflect::Type *> > *
        ]
    ),
    "hknpConvexShapeFunctions::calcAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpConvexShape::calcAabbNoRadius": {
        "hknpConvexShape *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpConvexShape *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "hknpConvexShape *, const hkTransformf *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpConvexShape *
                ctypes.c_ulonglong,  # const hkTransformf *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
    },
    "hknpConvexShapeFunctions::calcBoundingRadius": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpConvexShapeFunctions::calcBoundingSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkSphere *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpConvexShapeFunctions::calcInnerRadius": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpConvexShapeFunctions::calcInnerSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpConvexShapeFunctions::calcMinAngleBetweenFaces": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpConvexShapeFunctions::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpConvexShape::combineMaximumAllowedPenetration": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSimdFloat32 *
        argtypes=[
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpConvexShapeFunctions::convertVertexIdsToVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpConvexShape::createHullFromVertices": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkStridedVertices *
        argtypes=[
            ctypes.c_ulonglong,  # const hkStridedVertices *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # const hknpConvexShapeBuildConfig *
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_ulonglong,  # hkRefPtr<hknpShapeMassProperties> *
            ctypes.c_ulonglong,  # hkCompressedMassProperties **
        ]
    ),
    "hknpConvexShapeFunctions::getConvexVertices": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpConvexVertexShape *
            ctypes.c_ulonglong,  # hkArrayView<hkFloat3> *
        ]
    ),
    "hknpConvexShapeFunctions::getFaceInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hknpAngularTimType *
        ]
    ),
    "hknpConvexShapeFunctions::getFaceVertices": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpConvexShapeFunctions::getMemoryFootprint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpShapeSet *
        ]
    ),
    "hknpConvexShapeFunctions::getMutationSignals": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpShapeSignals *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpConvexShapeFunctions::getNumberOfSupportSpheres": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpConvexShapeFunctions::getSignedDistances": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::SdfQuery *
            ctypes.c_ulonglong,  # hknpShape::SdfContactPoint *
        ]
    ),
    "hknpConvexShapeFunctions::getSupportSpheres": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkFloat3 *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkFloat3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpConvexShapeFunctions::getSupportingFace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkcdGsk::Cache *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hknpAngularTimType *
        ]
    ),
    "hknpConvexShapeFunctions::getSupportingVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpConvexShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hknpConvexShape::makeCloneImpl": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConvexShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexShape *
            ctypes.c_ulonglong,  # hkRefPtr<hknpConvexShape> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpConvexShape::makeFromAabb": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConvexShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConvexShape> *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hknpConvexShapeBuildConfig *
        ]
    ),
    "hknpConvexShape::makeFromVertices": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConvexShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConvexShape> *
            ctypes.c_ulonglong,  # const hkStridedVertices *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hknpConvexShapeBuildConfig *
        ]
    ),
    "hknpConvexShapeFunctions::queryAabb": {
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hknpCollisionQueryCollector *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
    },
    "hknpConvexShapeFunctions::queryPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpConvexShape::setMaximumPenetration": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexShape *
            ctypes.c_float,  # float
        ]
    ),
    "hknpTriangleShape::~hknpTriangleShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpTriangleShape *
        ]
    ),
    "hknpTriangleShape::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpTriangleShape *
        ]
    ),
    "hknpTriangleShapeFunctions::buildMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpTriangleShapeFunctions::buildSurfaceGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::BuildSurfaceGeometryConfig *
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<hkReflect::Type const *,hkReflect::Type *> > *
        ]
    ),
    "hknpTriangleShapeFunctions::calcInnerSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpTriangleShapeFunctions::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpTriangleShapeFunctions::getConvexVertices": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpConvexVertexShape *
            ctypes.c_ulonglong,  # hkArrayView<hkFloat3> *
        ]
    ),
    "hknpTriangleShapeFunctions::getSupportingFace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkcdGsk::Cache *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hknpAngularTimType *
        ]
    ),
    "hknpTriangleShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hknpTriangleShape::makeEmptyTriangleShape": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpTriangleShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpTriangleShape> *
            ctypes.c_float,  # float
        ]
    ),
    "hknpTriangleShape::setVertices": {
        "hknpTriangleShape *, const hkVector4f *, const hkVector4f *, const hkVector4f *, const hkVector4f *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpTriangleShape *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_ulonglong,  # const hkVector4f *
            ]
        ),
        "hknpTriangleShape *, const hkVector4f *, const hkVector4f *, const hkVector4f *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpTriangleShape *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_ulonglong,  # const hkVector4f *
            ]
        ),
    },
    "hkSignal2_hknpShape_const___unsigned_char___subscribe__anonymous_namespace___hknpLodShapeEx_void____cdecl_A0xec73bb4d__hknpLodShapeEx_____hknpShape_const___unsigned_char__": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal2<hknpShape const *,unsigned char>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hknpLodShapeEx *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpLodShapeEx*', 'name': 'this'}, 'const hknpShape *', {'type': 'unsigned', 'name': '__int8'}]}
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkSignal3_hknpWorld___hknpBodyId_const___int___unsubscribe__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld___hknpBodyId_const___int__": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpActionManagerEx *
        argtypes=[
            ctypes.c_ulonglong,  # hkBool *
            ctypes.c_ulonglong,  # hknpActionManagerEx *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpActionManagerEx*', 'name': 'this'}, 'hknpWorld *', 'const hknpBodyId *', 'int']}
        ]
    ),
    "hknpLodShape::hknpLodShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLodShape *
            ctypes.c_ulonglong,  # const hknpLodShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpLodShape::~hknpLodShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLodShape *
        ]
    ),
    "hkSignal3_hknpWorld___hknpBodyId_const___int___MemberSlot__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld___hknpBodyId_const___int_____scalar_deleting_destructor_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _QWORD *
        argtypes=[
            ctypes.c_ulonglong,  # _QWORD *
            ctypes.c_char,  # char
        ]
    ),
    "hknpLodShape::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLodShape *
        ]
    ),
    "hknpLodShapeFunctions::buildMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpLodShapeFunctions::buildSurfaceGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::BuildSurfaceGeometryConfig *
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpLodShapeFunctions::calcAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpLodShapeFunctions::calcBoundingRadius": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpLodShapeFunctions::calcBoundingSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpLodShapeFunctions::calcMinAngleBetweenFaces": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hkSignal1<hknpWorld *>::MemberSlot<hknpCharacterRigidBody,void ": {
        "hkSignal1<hknpWorld *>::MemberSlot<cTkPhysicsManager,void (__cdecl cTkPhysicsManager::*)(hknpWorld *)> *, hknpWorld *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal1<hknpWorld *>::MemberSlot<cTkPhysicsManager,void (__cdecl cTkPhysicsManager::*)(hknpWorld *)> *
                ctypes.c_ulonglong,  # hknpWorld *
            ]
        ),
        "hkSignal1<hknpWorld *>::MemberSlot<cTkPhysicsManager,void (__cdecl cTkPhysicsManager::*)(hknpWorld *)> *, const void *, int": FUNCDEF(
            restype=ctypes.c_ubyte,  # _BOOL8
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal1<hknpWorld *>::MemberSlot<cTkPhysicsManager,void (__cdecl cTkPhysicsManager::*)(hknpWorld *)> *
                ctypes.c_ulonglong,  # const void *
                ctypes.c_int32,  # int
            ]
        ),
        "hkSignal1<hknpWorld *>::MemberSlot<cTkPhysicsManager,void (__cdecl cTkPhysicsManager::*)(hknpWorld *)> *, hkSignal1<hknpWorld *> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal1<hknpWorld *>::MemberSlot<cTkPhysicsManager,void (__cdecl cTkPhysicsManager::*)(hknpWorld *)> *
                ctypes.c_ulonglong,  # hkSignal1<hknpWorld *> *
            ]
        ),
        "hkSignal1<hknpWorld *>::MemberSlot<hknpCharacterRigidBody,void (__cdecl hknpCharacterRigidBody::*)(hknpWorld *)> *, hkSignal1<hknpWorld *> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal1<hknpWorld *>::MemberSlot<hknpCharacterRigidBody,void (__cdecl hknpCharacterRigidBody::*)(hknpWorld *)> *
                ctypes.c_ulonglong,  # hkSignal1<hknpWorld *> *
            ]
        ),
    },
    "hknpLodShapeFunctions::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpLodShapeFunctions::getLeafShapes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpShapeCollector *
        ]
    ),
    "hknpLodShapeFunctions::getMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hkSignal2<hknpEventHandlerInput const &,hknpEvent const &>::GlobalSlot<void ": {
        "hkSignal1<hknpWorld *>::MemberSlot<cTkPhysicsManager,void (__cdecl cTkPhysicsManager::*)(hknpWorld *)> *, const void **, int *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal1<hknpWorld *>::MemberSlot<cTkPhysicsManager,void (__cdecl cTkPhysicsManager::*)(hknpWorld *)> *
                ctypes.c_ulonglong,  # const void **
                ctypes.c_ulonglong,  # int *
            ]
        ),
        "hkSignal2<hknpEventHandlerInput const &,hknpEvent const &>::GlobalSlot<void (__cdecl*)(hknpEventHandlerInput const &,hknpEvent const &)> *, const hknpEventHandlerInput *, const hknpEvent *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal2<hknpEventHandlerInput const &,hknpEvent const &>::GlobalSlot<void (__cdecl*)(hknpEventHandlerInput const &,hknpEvent const &)> *
                ctypes.c_ulonglong,  # const hknpEventHandlerInput *
                ctypes.c_ulonglong,  # const hknpEvent *
            ]
        ),
        "hkSignal2<hknpEventHandlerInput const &,hknpEvent const &>::GlobalSlot<void (__cdecl*)(hknpEventHandlerInput const &,hknpEvent const &)> *, hkSignal2<hknpEventHandlerInput const &,hknpEvent const &> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal2<hknpEventHandlerInput const &,hknpEvent const &>::GlobalSlot<void (__cdecl*)(hknpEventHandlerInput const &,hknpEvent const &)> *
                ctypes.c_ulonglong,  # hkSignal2<hknpEventHandlerInput const &,hknpEvent const &> *
            ]
        ),
    },
    "hknpLodShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hknpLodShapeFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpLodShapeFunctions::queryAabb": {
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hknpCollisionQueryCollector *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
    },
    "hkSignal2_hknpShape_const___unsigned_char___MemberSlot__anonymous_namespace___hknpLodShapeEx_void____cdecl_A0xec73bb4d__hknpLodShapeEx_____hknpShape_const___unsigned_char____subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpShape const *,unsigned char> *
            ctypes.c_ulonglong,  # hknpLodShapeEx *
        ]
    ),
    "hkSignal3_hknpWorld___hknpBodyId_const___int___MemberSlot__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld___hknpBodyId_const___int____unsubscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal3<hknpWorld *,hknpBodyId const *,int> *
            ctypes.c_ulonglong,  # hkBool *
        ]
    ),
    "hkSignal2<hknpShape const *,uchar>::subscribe<hknpDecoratorShape,void ": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal2<hknpShape const *,unsigned char>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpShape const *,unsigned char> *
            ctypes.c_ulonglong,  # hknpDecoratorShape *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpDecoratorShape*', 'name': 'this'}, 'const hknpShape *', {'type': 'unsigned', 'name': '__int8'}]}
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkSignal2<hknpEventHandlerInput const &,hknpEvent const &>::unsubscribe<hknpCharacterRigidBody,void ": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal1<hknpWorld *> *
            ctypes.c_ulonglong,  # hkBool *
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'cTkPhysicsManager*', 'name': 'this'}, 'hknpWorld *']}
        ]
    ),
    "hknpDecoratorShape::hknpDecoratorShape": {
        "hknpDecoratorShape *, const hknpDecoratorShape *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpDecoratorShape *
                ctypes.c_ulonglong,  # const hknpDecoratorShape *
            ]
        ),
        "hknpDecoratorShape *, hknpShapeType::Enum, const hknpShape *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpDecoratorShape *
                ctypes.c_uint32,  # hknpShapeType::Enum
                ctypes.c_ulonglong,  # const hknpShape *
            ]
        ),
    },
    "hknpDecoratorShapeFunctions::buildMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpDecoratorShapeFunctions::calcAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpDecoratorShapeFunctions::calcInnerRadius": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpDecoratorShapeFunctions::calcInnerSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpDecoratorShapeFunctions::calcMinAngleBetweenFaces": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpDecoratorShapeFunctions::calcSurfaceVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::SurfaceVelocityInput *
            ctypes.c_ulonglong,  # hknpShape::SurfaceVelocityOutput *
        ]
    ),
    "hknpDecoratorShapeFunctions::convertVertexIdsToVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpDecoratorShapeFunctions::estimateNumShapeKeys": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpDecoratorShapeFunctions::getCircularSymmetryAxis": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCircularSymmetryAxis *
        ]
    ),
    "hknpDecoratorShapeFunctions::getFaceInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hknpAngularTimType *
        ]
    ),
    "hknpDecoratorShapeFunctions::getFaceVertices": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpDecoratorShapeFunctions::getShapeKeys": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const hknpShape::GetShapeKeysConfig *
        ]
    ),
    "hknpDecoratorShapeFunctions::getSignedDistanceContacts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpShape::GetSignedDistanceInput *
            ctypes.c_ulonglong,  # hknpShape::SdfContactPointOutput *
        ]
    ),
    "hknpDecoratorShapeFunctions::getSignedDistances": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::SdfQuery *
            ctypes.c_ulonglong,  # hknpShape::SdfContactPoint *
        ]
    ),
    "hknpDecoratorShapeFunctions::getSupportSpheres": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkFloat3 *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkFloat3 *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpDecoratorShapeFunctions::getSupportingFace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkcdGsk::Cache *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hknpAngularTimType *
        ]
    ),
    "hknpDecoratorShapeFunctions::getSupportingVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpDecoratorShape::makeCloneCoreShape": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShape const > *
        argtypes=[
            ctypes.c_ulonglong,  # hknpDecoratorShape *
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape const > *
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpDecoratorShapeFunctions::makeShapeKeyMask": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpDecoratorShape::onCoreShapeMutated": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDecoratorShape *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpDecoratorShapeFunctions::queryPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hkSignal2<hknpShape const *,uchar>::MemberSlot<hknpDecoratorShape,void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpShape const *,unsigned char>::MemberSlot<hknpDecoratorShape,void (__cdecl hknpDecoratorShape::*)(hknpShape const *,unsigned char)> *
            ctypes.c_ulonglong,  # hkSignal2<hknpShape const *,unsigned char> *
        ]
    ),
    "hknpDecoratorShape::subscribeToCoreShapeMutation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDecoratorShape *
        ]
    ),
    "hknpDecoratorShape::unsubscribeFromCoreShapeMutation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDecoratorShape *
        ]
    ),
    "hknpMaskedShape::hknpMaskedShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMaskedShape *
            ctypes.c_ulonglong,  # const hknpMaskedShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
            ctypes.c_uint32,  # hknpMaskedShape::MaskInitMode
        ]
    ),
    "hknpMaskedShape::~hknpMaskedShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMaskedShape *
        ]
    ),
    "hknpMaskedShapeFunctions::buildSurfaceGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::BuildSurfaceGeometryConfig *
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpMaskedShapeFunctions::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpMaskedShapeFunctions::getLeafShapes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # hknpShapeCollector *
        ]
    ),
    "hknpMaskedShapeFunctions::getMutationSignals": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkEnum<enum hknpShapeType::Enum,unsigned char> *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpMaskedShapeFunctions::getShapeKeys": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const hknpShape::GetShapeKeysConfig *
        ]
    ),
    "hknpMaskedShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hknpMaskedShapeFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpMaskedShape::makeCloneImpl": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpMaskedShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpMaskedShape *
            ctypes.c_ulonglong,  # hkRefPtr<hknpMaskedShape> *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpMaskedShapeFunctions::onCoreShapeMutated": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShape *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "hknpMaskedShapeFunctions::onFinishEditingMask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShape *
        ]
    ),
    "hknpMaskedShapeFunctions::queryAabb": {
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hknpCollisionQueryCollector *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
    },
    "hknpCompositeShape::hknpCompositeShape": {
        "hknpCompositeShape *, const hknpCompositeShape *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCompositeShape *
                ctypes.c_ulonglong,  # const hknpCompositeShape *
            ]
        ),
        "hknpCompositeShape *, hknpShapeType::Enum": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCompositeShape *
                ctypes.c_uint32,  # hknpShapeType::Enum
            ]
        ),
    },
    "hknpShapeInstance::~hknpShapeInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeInstance *
        ]
    ),
    "hknpShapeInstance::_setShapeNoReduceCurrentRefCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeInstance *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpShapeInstance::calculateAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeInstance *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpShapeInstance::getFullTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeInstance *
            ctypes.c_ulonglong,  # hkTransformf *
        ]
    ),
    "hknpShapeInstance::getFullTransformInverse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeInstance *
            ctypes.c_ulonglong,  # hkTransformf *
        ]
    ),
    "hknpShapeInstance::setScale": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeInstance *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_uint32,  # hknpShape::ScaleMode
        ]
    ),
    "hknpShapeInstance::setShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeInstance *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpShapeInstance::setTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeInstance *
            ctypes.c_ulonglong,  # const hkQTransform *
        ]
    ),
    "hkArrayBase<hkHandle<ushort,65535,hkTaskGraph::TaskIdDiscriminant>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkHandle<unsigned short,65535,hkTaskGraph::TaskIdDiscriminant> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkMassElement>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpDeflectedLinearCast::Contact> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkcdSimdTree::aabbCast<hknpCompoundShapeEx::ShapeCast<2>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdRay *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ShapeCast<2> *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::aabbCast<hknpCompoundShapeEx::ShapeCast<8>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdRay *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ShapeCast<8> *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::aabbCast<hknpCompoundShapeEx::ShapeCast<0>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdRay *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ShapeCast<0> *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromAabb<hknpCompoundShapeEx::ClosestPointsToConvexQuery<1>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ClosestPointsToConvexQuery<1> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromAabb<hknpCompoundShapeEx::ClosestPointsToConvexQuery<2>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ClosestPointsToConvexQuery<2> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromAabb<hknpCompoundShapeEx::ClosestPointsToConvexQuery<8>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ClosestPointsToConvexQuery<8> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromAabb<hknpCompoundShapeEx::ClosestPointsToConvexQuery<0>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ClosestPointsToConvexQuery<0> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromAabbOrdered<hknpCompoundShapeEx::ClosestPointsToConvexQuery<1>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ClosestPointsToConvexQuery<1> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromAabbOrdered<hknpCompoundShapeEx::ClosestPointsToConvexQuery<2>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ClosestPointsToConvexQuery<2> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromAabbOrdered<hknpCompoundShapeEx::ClosestPointsToConvexQuery<8>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ClosestPointsToConvexQuery<8> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromAabbOrdered<hknpCompoundShapeEx::ClosestPointsToConvexQuery<0>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ClosestPointsToConvexQuery<0> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPoint<hknpCompoundShapeEx::PointDistanceQuery<1>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::PointDistanceQuery<1> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPoint<hknpCompoundShapeEx::PointDistanceQuery<2>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::PointDistanceQuery<2> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPoint<hknpCompoundShapeEx::PointDistanceQuery<8>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::PointDistanceQuery<8> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPoint<hknpCompoundShapeEx::PointDistanceQuery<0>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::PointDistanceQuery<0> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPointOrdered<hknpCompoundShapeEx::ClosestPointsToConvexQuery<1>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ClosestPointsToConvexQuery<1> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPointOrdered<hknpCompoundShapeEx::ClosestPointsToConvexQuery<2>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ClosestPointsToConvexQuery<2> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPointOrdered<hknpCompoundShapeEx::ClosestPointsToConvexQuery<8>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ClosestPointsToConvexQuery<8> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPointOrdered<hknpCompoundShapeEx::ClosestPointsToConvexQuery<0>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ClosestPointsToConvexQuery<0> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPointOrdered<hknpCompoundShapeEx::PointDistanceQuery<1>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::PointDistanceQuery<1> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPointOrdered<hknpCompoundShapeEx::PointDistanceQuery<2>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::PointDistanceQuery<2> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPointOrdered<hknpCompoundShapeEx::PointDistanceQuery<8>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::PointDistanceQuery<8> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPointOrdered<hknpCompoundShapeEx::PointDistanceQuery<0>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::PointDistanceQuery<0> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPointParticles<hknpCompoundShapeEx::ClosestPointsToConvexQuery<1>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ClosestPointsToConvexQuery<1> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPointParticles<hknpCompoundShapeEx::ClosestPointsToConvexQuery<2>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ClosestPointsToConvexQuery<2> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPointParticles<hknpCompoundShapeEx::ClosestPointsToConvexQuery<8>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ClosestPointsToConvexQuery<8> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPointParticles<hknpCompoundShapeEx::ClosestPointsToConvexQuery<0>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ClosestPointsToConvexQuery<0> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hknpCompoundShapeEx::getClosestPointsImpl<1,0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompoundShapeEx::getClosestPointsImpl<2,0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompoundShapeEx::getClosestPointsImpl<8,0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompoundShapeEx::getClosestPointsImpl<0,0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompoundShapeEx::queryPointImpl<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # const hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompoundShapeEx::queryPointImpl<2>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # const hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompoundShapeEx::queryPointImpl<8>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # const hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompoundShapeEx::queryPointImpl<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # const hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hkSignal2<hknpShape const *,uchar>::subscribe<hknpCompoundShapeEx,void ": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal2<hknpShape const *,unsigned char>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpShape const *,unsigned char> *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpCompoundShapeEx*', 'name': 'this'}, 'const hknpShape *', {'type': 'unsigned', 'name': '__int8'}, {'type': 'unsigned', 'name': '__int16'}]}
            ctypes.c_char_p,  # const char *
            ctypes.c_uint16,  # const unsigned __int16
        ]
    ),
    "hkSignal1<hknpShape const *>::subscribe<hknpCompoundShapeEx,void ": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal1<hknpShape const *>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal1<hknpShape const *> *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpCompoundShapeEx*', 'name': 'this'}, 'const hknpShape *', {'type': 'unsigned', 'name': '__int16'}]}
            ctypes.c_char_p,  # const char *
            ctypes.c_uint16,  # const unsigned __int16
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbCastWrapperExternal<hknpCompoundShapeEx::ShapeCast<1>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbCastWrapperExternal<hknpCompoundShapeEx::ShapeCast<1> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbCastWrapperExternal<hknpCompoundShapeEx::ShapeCast<8>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbCastWrapperExternal<hknpCompoundShapeEx::ShapeCast<8> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpCompoundShapeEx::AabbOverlaps<1,hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpCompoundShapeEx::AabbOverlaps<1,hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > > > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpCompoundShapeEx::AabbOverlaps<1,hknpCollisionQueryCollector>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpCompoundShapeEx::AabbOverlaps<1,hknpCollisionQueryCollector> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpCompoundShapeEx::AabbOverlaps<8,hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpCompoundShapeEx::AabbOverlaps<8,hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > > > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpCompoundShapeEx::AabbOverlaps<8,hknpCollisionQueryCollector>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpCompoundShapeEx::AabbOverlaps<8,hknpCollisionQueryCollector> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpCompoundShapeEx::AabbOverlaps<1,hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpCompoundShapeEx::AabbOverlaps<1,hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > > > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpCompoundShapeEx::AabbOverlaps<1,hknpCollisionQueryCollector>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpCompoundShapeEx::AabbOverlaps<1,hknpCollisionQueryCollector> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpCompoundShapeEx::AabbOverlaps<8,hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpCompoundShapeEx::AabbOverlaps<8,hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > > > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpCompoundShapeEx::AabbOverlaps<8,hknpCollisionQueryCollector>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpCompoundShapeEx::AabbOverlaps<8,hknpCollisionQueryCollector> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromAabbWrapper<hknpCompoundShapeEx::ClosestPointsToConvexQuery<1>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromAabbWrapper<hknpCompoundShapeEx::ClosestPointsToConvexQuery<1> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromAabbWrapper<hknpCompoundShapeEx::ClosestPointsToConvexQuery<2>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromAabbWrapper<hknpCompoundShapeEx::ClosestPointsToConvexQuery<8> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromAabbWrapper<hknpCompoundShapeEx::ClosestPointsToConvexQuery<0>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromAabbWrapper<hknpCompoundShapeEx::ClosestPointsToConvexQuery<0> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromPointWrapper<hknpCompoundShapeEx::ClosestPointsToConvexQuery<1>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromPointWrapper<hknpCompoundShapeEx::ClosestPointsToConvexQuery<1> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromPointWrapper<hknpCompoundShapeEx::ClosestPointsToConvexQuery<8>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromPointWrapper<hknpCompoundShapeEx::ClosestPointsToConvexQuery<8> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromPointWrapper<hknpCompoundShapeEx::ClosestPointsToConvexQuery<0>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromPointWrapper<hknpCompoundShapeEx::ClosestPointsToConvexQuery<0> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromPointWrapper<hknpCompoundShapeEx::PointDistanceQuery<1>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromPointWrapper<hknpCompoundShapeEx::PointDistanceQuery<1> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromPointWrapper<hknpCompoundShapeEx::PointDistanceQuery<8>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromPointWrapper<hknpCompoundShapeEx::PointDistanceQuery<8> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromPointWrapper<hknpCompoundShapeEx::PointDistanceQuery<0>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromPointWrapper<hknpCompoundShapeEx::PointDistanceQuery<0> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::RayCastWrapperExternal<hknpCompoundShapeEx::RayCast<8>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::RayCastWrapperExternal<hknpCompoundShapeEx::RayCast<8> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompoundShapeCdDynamicTree,hkcdTreeQueriesStacks::Dynamic<128,ushort>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::VisibilityWrapper<hknpCompoundShapeEx::RayCast<8>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,unsigned short> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::VisibilityWrapper<hknpCompoundShapeEx::RayCast<8> > *
        ]
    ),
    "hkSignal1<hknpShape const *>::unsubscribe<hknpCompoundShapeEx,void ": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal1<hknpShape const *> *
            ctypes.c_ulonglong,  # hkBool *
            ctypes.c_ulonglong,  # hknpCompoundShapeEx *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpCompoundShapeEx*', 'name': 'this'}, 'const hknpShape *', {'type': 'unsigned', 'name': '__int16'}]}
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "hknpCompoundShape::hknpCompoundShape": {
        "hknpCompoundShape *, const hknpCompoundShapeCinfo *, const hknpCompoundShape::Layout *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCompoundShape *
                ctypes.c_ulonglong,  # const hknpCompoundShapeCinfo *
                ctypes.c_ulonglong,  # const hknpCompoundShape::Layout *
            ]
        ),
        "hknpCompoundShape *, const hknpCompoundShape *, hknpShape::CloneMode, hknpShapeMap *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCompoundShape *
                ctypes.c_ulonglong,  # const hknpCompoundShape *
                ctypes.c_uint32,  # hknpShape::CloneMode
                ctypes.c_ulonglong,  # hknpShapeMap *
            ]
        ),
    },
    "hkFreeListRelArray<hknpShapeInstance,hkHandle<ushort,65535,hknpShapeInstanceIdDiscriminant>,8,hknpShapeInstance>::~hkFreeListRelArray<hknpShapeInstance,hkHandle<ushort,65535,hknpShapeInstanceIdDiscriminant>,8,hknpShapeInstance>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkFreeListRelArray<hknpShapeInstance,hkHandle<unsigned short,65535,hknpShapeInstanceIdDiscriminant>,8,hknpShapeInstance> *
        ]
    ),
    "hknpCompoundShape::~hknpCompoundShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShape *
        ]
    ),
    "hknpHitScalingCollector::addHit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpHitScalingCollector *
            ctypes.c_ulonglong,  # const hknpCollisionResult *
        ]
    ),
    "hknpCompoundShapeEx::AabbOverlaps<1,hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>::addLeaves": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::AabbOverlaps<1,hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpCompoundShapeEx::AabbOverlaps<1,hknpCollisionQueryCollector>::addLeaves": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::AabbOverlaps<1,hknpCollisionQueryCollector> *
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpCompoundShapeEx::AabbOverlaps<8,hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>::addLeaves": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::AabbOverlaps<8,hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpCompoundShapeEx::AabbOverlaps<8,hknpCollisionQueryCollector>::addLeaves": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::AabbOverlaps<8,hknpCollisionQueryCollector> *
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpCompoundShapeEx::AabbOverlaps<0,hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>::addLeaves": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::AabbOverlaps<0,hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpCompoundShapeEx::AabbOverlaps<0,hknpCollisionQueryCollector>::addLeaves": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::AabbOverlaps<0,hknpCollisionQueryCollector> *
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpCompoundShape::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShape *
        ]
    ),
    "hknpCompoundShapeFunctions::buildMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpCompoundShape::buildMassPropertiesMasked": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # const hknpShapeKeyMask *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpCompoundShapeFunctions::buildSurfaceGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::BuildSurfaceGeometryConfig *
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<hkReflect::Type const *,hkReflect::Type *> > *
        ]
    ),
    "hknpCompoundShapeFunctions::calcAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpCompoundShapeFunctions::calcBoundingRadius": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpCompoundShapeFunctions::calcBoundingSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpCompoundShapeEx::calcLocalRayAndQueryAabbHalfExtents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hkcdRay *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpCompoundShapeEx::calcQueryToTargetTransformWithScale": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hkTransformf *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpCompoundShapeFunctions::calcSurfaceVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::SurfaceVelocityInput *
            ctypes.c_ulonglong,  # hknpShape::SurfaceVelocityOutput *
        ]
    ),
    "hkSignal2<hknpShape const *,uchar>::MemberSlotWithData<hknpCompoundShapeEx,void ": {
        "hkSignal2<hknpShape const *,unsigned char>::MemberSlotWithData<hknpCompoundShapeEx,void (__cdecl hknpCompoundShapeEx::*)(hknpShape const *,unsigned char,unsigned short),unsigned short> *, const hknpShape *, __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal2<hknpShape const *,unsigned char>::MemberSlotWithData<hknpCompoundShapeEx,void (__cdecl hknpCompoundShapeEx::*)(hknpShape const *,unsigned char,unsigned short),unsigned short> *
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_int64,  # __int64
            ]
        ),
        "hkSignal2<hknpShape const *,unsigned char>::MemberSlotWithData<hknpCompoundShapeEx,void (__cdecl hknpCompoundShapeEx::*)(hknpShape const *,unsigned char,unsigned short),unsigned short> *, hkSignal2<hknpShape const *,unsigned char> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal2<hknpShape const *,unsigned char>::MemberSlotWithData<hknpCompoundShapeEx,void (__cdecl hknpCompoundShapeEx::*)(hknpShape const *,unsigned char,unsigned short),unsigned short> *
                ctypes.c_ulonglong,  # hkSignal2<hknpShape const *,unsigned char> *
            ]
        ),
    },
    "hkSignal1<hknpShape const *>::MemberSlotWithData<hknpCompoundShapeEx,void ": {
        "hkSignal1<hknpShape const *>::MemberSlotWithData<hknpCompoundShapeEx,void (__cdecl hknpCompoundShapeEx::*)(hknpShape const *,unsigned short),unsigned short> *, const hknpShape *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal1<hknpShape const *>::MemberSlotWithData<hknpCompoundShapeEx,void (__cdecl hknpCompoundShapeEx::*)(hknpShape const *,unsigned short),unsigned short> *
                ctypes.c_ulonglong,  # const hknpShape *
            ]
        ),
        "hkSignal1<hknpShape const *>::MemberSlotWithData<hknpCompoundShapeEx,void (__cdecl hknpCompoundShapeEx::*)(hknpShape const *,unsigned short),unsigned short> *, const char *, int": FUNCDEF(
            restype=ctypes.c_ubyte,  # _BOOL8
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal1<hknpShape const *>::MemberSlotWithData<hknpCompoundShapeEx,void (__cdecl hknpCompoundShapeEx::*)(hknpShape const *,unsigned short),unsigned short> *
                ctypes.c_char_p,  # const char *
                ctypes.c_int32,  # int
            ]
        ),
        "hkSignal1<hknpShape const *>::MemberSlotWithData<hknpCompoundShapeEx,void (__cdecl hknpCompoundShapeEx::*)(hknpShape const *,unsigned short),unsigned short> *, hkSignal1<hknpShape const *> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal1<hknpShape const *>::MemberSlotWithData<hknpCompoundShapeEx,void (__cdecl hknpCompoundShapeEx::*)(hknpShape const *,unsigned short),unsigned short> *
                ctypes.c_ulonglong,  # hkSignal1<hknpShape const *> *
            ]
        ),
    },
    "hknpCompoundShapeFunctions::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompoundShapeFunctions::castShapeImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompoundShapeFunctions::getClosestPointsImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompoundShapeFunctions::getLeafShapes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpShapeCollector *
        ]
    ),
    "hknpCompoundShapeFunctions::getMemoryFootprint": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpShapeSet *
        ]
    ),
    "hknpCompoundShapeFunctions::getMutationSignals": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpShapeSignals *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpCompoundShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hkFreeListArray<hknpShapeInstance,hkHandle<ushort,65535,hknpShapeInstanceIdDiscriminant>,8,hknpShapeInstance,hkFreeListRelArrayStorage<hknpShapeInstance>>::grow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkFreeListArray<hknpShapeInstance,hkHandle<unsigned short,65535,hknpShapeInstanceIdDiscriminant>,8,hknpShapeInstance,hkFreeListRelArrayStorage<hknpShapeInstance> > *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpCompoundShapeFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpCompoundShape::makeCompoundShape": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpCompoundShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpCompoundShape> *
            ctypes.c_ulonglong,  # const hknpCompoundShapeCinfo *
        ]
    ),
    "hknpCompoundShapeFunctions::makeShapeKeyMask": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpCompoundShapeEx::onChildShapeDestroyed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "hknpCompoundShapeEx::onChildShapeMutated": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "hknpCompoundShapeEx::CastRayImpl<1,2>::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompoundShapeEx::CastRayImpl<8,1>::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompoundShapeEx::CastRayImpl<8,2>::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompoundShapeEx::CastRayImpl<0,2>::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompoundShapeEx::ClosestPointsToConvexQuery<0>::processInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ClosestPointsToConvexQuery<0> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpCompoundShapeEx::ClosestPointsToConvexQuery<2>::processInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ClosestPointsToConvexQuery<8> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpCompoundShapeEx::PointDistanceQuery<1>::processInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::PointDistanceQuery<0> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpCompoundShapeEx::PointDistanceQuery<2>::processInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::PointDistanceQuery<8> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpCompoundShapeEx::ShapeCast<1>::processInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ShapeCast<0> *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpCompoundShapeEx::ShapeCast<2>::processInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::ShapeCast<8> *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpCompoundShapeFunctions::queryAabb": {
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hknpCollisionQueryCollector *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
    },
    "hknpCompoundShapeFunctions::queryPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpHitScalingCollector::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpHitScalingCollector *
        ]
    ),
    "hknpMaterial::hknpMaterial": {
        "hknpMaterial *, const hknpMaterial *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpMaterial *
                ctypes.c_ulonglong,  # const hknpMaterial *
            ]
        ),
        "hknpMaterial *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpMaterial *
            ]
        ),
    },
    "hknpMaterial::synchronizeFlags": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMaterial *
        ]
    ),
    "hkcdSimdTree::aabbCast<hknpCompressedMeshShapeInternals::ShapeCastQuery>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdRay *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::ShapeCastQuery *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticMeshTree::Section,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbCastWrapperExternal<hknpCompressedMeshShapeInternals::ShapeCastQuery>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Section *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbCastWrapperExternal<hknpCompressedMeshShapeInternals::ShapeCastQuery> *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticMeshTree::Section,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpCompressedMeshShapeInternals::AabbOverlaps<hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Section *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpCompressedMeshShapeInternals::AabbOverlaps<hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > > > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticMeshTree::Section,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpCompressedMeshShapeInternals::AabbOverlaps<hknpCollisionQueryCollector>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Section *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpCompressedMeshShapeInternals::AabbOverlaps<hknpCollisionQueryCollector> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticMeshTree::Section,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpCompressedMeshShapeInternals::AabbOverlaps<hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Section *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpCompressedMeshShapeInternals::AabbOverlaps<hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > > > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticMeshTree::Section,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpCompressedMeshShapeInternals::AabbOverlaps<hknpCollisionQueryCollector>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Section *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpCompressedMeshShapeInternals::AabbOverlaps<hknpCollisionQueryCollector> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticMeshTree::Section,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hknpCompressedMeshShapeInternals::ShapeCastQueryScaled>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Section *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::ShapeCastQueryScaled *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompressedMeshShapeTree,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbCastWrapperExternal<hknpCompressedMeshShapeInternals::ShapeCastQuery>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbCastWrapperExternal<hknpCompressedMeshShapeInternals::ShapeCastQuery> *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompressedMeshShapeTree,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpCompressedMeshShapeInternals::AabbOverlaps<hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpCompressedMeshShapeInternals::AabbOverlaps<hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > > > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompressedMeshShapeTree,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpCompressedMeshShapeInternals::AabbOverlaps<hknpCollisionQueryCollector>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNearMissAabbWrapper<hknpCompressedMeshShapeInternals::AabbOverlaps<hknpCollisionQueryCollector> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompressedMeshShapeTree,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpCompressedMeshShapeInternals::AabbOverlaps<hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpCompressedMeshShapeInternals::AabbOverlaps<hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > > > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompressedMeshShapeTree,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpCompressedMeshShapeInternals::AabbOverlaps<hknpCollisionQueryCollector>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::AabbOverlapsNoEarlyExitWrapper<hknpCompressedMeshShapeInternals::AabbOverlaps<hknpCollisionQueryCollector> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompressedMeshShapeTree,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hknpCompressedMeshShapeInternals::ShapeCastQueryScaled>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::ShapeCastQueryScaled *
        ]
    ),
    "hkcdStaticMeshTree::Base::Base": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::Base *
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Base *
        ]
    ),
    "hkcdStaticMeshTree::Connectivity::Connectivity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::Connectivity *
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Connectivity *
        ]
    ),
    "hkcdDefaultStaticMeshTree::hkcdDefaultStaticMeshTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDefaultStaticMeshTree *
            ctypes.c_ulonglong,  # const hkcdDefaultStaticMeshTree *
        ]
    ),
    "hkcdStaticMeshTree::Base::~Base": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::Base *
        ]
    ),
    "hkcdStaticMeshTree::Connectivity::~Connectivity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::Connectivity *
        ]
    ),
    "hkcdDefaultStaticMeshTree::~hkcdDefaultStaticMeshTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDefaultStaticMeshTree *
        ]
    ),
    "hknpCompressedMeshShape::~hknpCompressedMeshShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShape *
        ]
    ),
    "hknpCompressedMeshShapeInternals::AabbOverlaps<hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>::addLeaves": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::AabbOverlaps<hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpCompressedMeshShapeInternals::AabbOverlaps<hknpCollisionQueryCollector>::addLeaves": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::AabbOverlaps<hknpCollisionQueryCollector> *
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpCompressedMeshShapeFunctions::calcAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpCompressedMeshShapeFunctions::castShapeImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hkSet<hknpShape const *,hkBuiltinContainerAllocator<0,0>,hkMapOperations<hknpShape const *>>::findElement": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkSet<hknpShape const *,hkBuiltinContainerAllocator<0,0>,hkMapOperations<hknpShape const *> > *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb4BytesCodec>::findLeafByValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHandle<int,-1,hkcdStaticTree::AabbTreeBase::NodeIdDiscriminant> *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb4BytesCodec> *
            ctypes.c_ulonglong,  # hkHandle<int,-1,hkcdStaticTree::AabbTreeBase::NodeIdDiscriminant> *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpCompressedMeshShapeFunctions::getMemoryFootprint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpShapeSet *
        ]
    ),
    "hknpCompressedMeshShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hknpCompressedMeshShapeFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpCompressedMeshShapeFunctions::queryAabb": {
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hknpCollisionQueryCollector *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
    },
    "hkSet<hknpShape const *,hkBuiltinContainerAllocator<0,0>,hkMapOperations<hknpShape const *>>::resizeTable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkSet<hknpShape const *,hkBuiltinContainerAllocator<0,0>,hkMapOperations<hknpShape const *> > *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpShapeCastQueryEx::transformRotation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeCastQueryEx *
            ctypes.c_ulonglong,  # const hkTransformf *
        ]
    ),
    "hknpSphereShape::~hknpSphereShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSphereShape *
        ]
    ),
    "hknpSphereShapeFunctions::buildMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpSphereShapeFunctions::buildSurfaceGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::BuildSurfaceGeometryConfig *
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpSphereShapeFunctions::calcBoundingRadius": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpSphereShapeFunctions::calcBoundingSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpSphereShapeFunctions::calcInnerSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpSphereShapeFunctions::calcMinAngleBetweenFaces": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # _DWORD *
        ]
    ),
    "hknpSphereShapeFunctions::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpSphereShapeFunctions::getConvexVertices": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpConvexVertexShape *
            ctypes.c_ulonglong,  # hkArrayView<hkFloat3> *
        ]
    ),
    "hknpSphereShapeFunctions::getSupportingVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpSphereShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hknpSphereShapeFunctions::queryPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpDragProperties::hknpDragProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDragProperties *
        ]
    ),
    "hkArrayBase<hkMassProperties>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpSimpleSimplexSolver::Contact> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpMassPropertiesTree::hknpMassPropertiesTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMassPropertiesTree *
        ]
    ),
    "hknpMassPropertiesTree::calcMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMassPropertiesTree *
            ctypes.c_ulonglong,  # const hknpMassPropertiesTree::MassProvider *
            ctypes.c_ulonglong,  # const hkRelOffsetBitField *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpShapeMassPropertiesTree::getMemoryFootPrint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShapePropertyBase *
        ]
    ),
    "hknpPairCollisionFilter::hknpPairCollisionFilter": {
        "hknpPairCollisionFilter *, const hknpPairCollisionFilter *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpPairCollisionFilter *
                ctypes.c_ulonglong,  # const hknpPairCollisionFilter *
            ]
        ),
        "hknpPairCollisionFilter *, const hknpCollisionFilter *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpPairCollisionFilter *
                ctypes.c_ulonglong,  # const hknpCollisionFilter *
            ]
        ),
    },
    "hkMapBase<hknpPairCollisionFilter::Key,uint,hknpPairCollisionFilter::MapOperations>::clearAndDeallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMapBase<hknpPairCollisionFilter::Key,unsigned int,hknpPairCollisionFilter::MapOperations> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hknpPairCollisionFilter::copyFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPairCollisionFilter *
            ctypes.c_ulonglong,  # const hknpCollisionFilter *
        ]
    ),
    "hknpPairCollisionFilter::filterBodyPairs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpPairCollisionFilter *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hknpBodyIndexPair *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpPairCollisionFilter::getLodInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpLodCollisionInfo *
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesCollisionFilter *
            ctypes.c_ulonglong,  # hknpLodCollisionInfo *
            ctypes.c_ulonglong,  # const hknpBody *
            ctypes.c_ulonglong,  # const hknpBody *
        ]
    ),
    "hkMapBase<hknpPairCollisionFilter::Key,uint,hknpPairCollisionFilter::MapOperations>::insert": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMapBase<hknpPairCollisionFilter::Key,unsigned int,hknpPairCollisionFilter::MapOperations> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_ulonglong,  # const hknpPairCollisionFilter::Key *
            ctypes.c_ulonglong,  # const unsigned int *
        ]
    ),
    "hknpPairCollisionFilter::isCollisionEnabled": {
        "hknpPairCollisionFilter *, __int64, const hknpQueryFilterData *, const hknpBody *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # hknpPairCollisionFilter *
                ctypes.c_int64,  # __int64
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpBody *
            ]
        ),
        "hknpPairCollisionFilter *, __int64, __int64, unsigned __int64": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # hknpPairCollisionFilter *
                ctypes.c_int64,  # __int64
                ctypes.c_int64,  # __int64
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
        "hknpPairCollisionFilter *, __int64, _BOOL8, const hknpCollisionFilter::FilterInput *, const hknpCollisionFilter::FilterInput *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # hknpPairCollisionFilter *
                ctypes.c_int64,  # __int64
                ctypes.c_ubyte,  # _BOOL8
                ctypes.c_ulonglong,  # const hknpCollisionFilter::FilterInput *
                ctypes.c_ulonglong,  # const hknpCollisionFilter::FilterInput *
            ]
        ),
    },
    "hknpPairCollisionFilter::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpCollisionFilter> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpPairCollisionFilter *
            ctypes.c_ulonglong,  # hkRefPtr<hknpCollisionFilter> *
        ]
    ),
    "hkMapBase<hknpPairCollisionFilter::Key,uint,hknpPairCollisionFilter::MapOperations>::remove": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpPairCollisionFilter::Key *
        argtypes=[
            ctypes.c_ulonglong,  # hkMapBase<hknpPairCollisionFilter::Key,unsigned int,hknpPairCollisionFilter::MapOperations> *
            ctypes.c_ulonglong,  # const hknpPairCollisionFilter::Key *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkMapBase<hknpPairCollisionFilter::Key,uint,hknpPairCollisionFilter::MapOperations>::resizeTable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkContainerAllocator *
        argtypes=[
            ctypes.c_ulonglong,  # hkMapBase<hknpPairCollisionFilter::Key,unsigned int,hknpPairCollisionFilter::MapOperations> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "hkSignal1<hknpWorld *>::subscribe<hknpConstraintCollisionFilter,void ": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal1<hknpWorld *>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal1<hknpWorld *> *
            ctypes.c_ulonglong,  # hknpConstraintCollisionFilter *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpConstraintCollisionFilter*', 'name': 'this'}, 'hknpWorld *']}
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkSignal2<hknpWorld *,hknpConstraintId>::subscribe<hknpConstraintCollisionFilter,void ": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hknpConstraintId>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hknpConstraintId> *
            ctypes.c_ulonglong,  # hknpConstraintCollisionFilter *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpConstraintCollisionFilter*', 'name': 'this'}, 'hknpWorld *', 'hknpConstraintId']}
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hknpConstraintCollisionFilter::hknpConstraintCollisionFilter": {
        "hknpConstraintCollisionFilter *, const hknpConstraintCollisionFilter *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpConstraintCollisionFilter *
                ctypes.c_ulonglong,  # const hknpConstraintCollisionFilter *
            ]
        ),
        "hknpConstraintCollisionFilter *, const hknpCollisionFilter *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpConstraintCollisionFilter *
                ctypes.c_ulonglong,  # const hknpCollisionFilter *
            ]
        ),
    },
    "hknpConstraintCollisionFilter::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpCollisionFilter> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintCollisionFilter *
            ctypes.c_ulonglong,  # hkRefPtr<hknpCollisionFilter> *
        ]
    ),
    "hknpConstraintCollisionFilter::onWorldDestroyedSignal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintCollisionFilter *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hkSignal1<hknpWorld *>::MemberSlot<hknpConstraintCollisionFilter,void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal1<hknpWorld *>::MemberSlot<hknpConstraintCollisionFilter,void (__cdecl hknpConstraintCollisionFilter::*)(hknpWorld *)> *
            ctypes.c_ulonglong,  # hkSignal1<hknpWorld *> *
        ]
    ),
    "hkSignal2<hknpWorld *,hknpConstraintId>::MemberSlot<hknpConstraintCollisionFilter,void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hknpConstraintId>::MemberSlot<hknpConstraintCollisionFilter,void (__cdecl hknpConstraintCollisionFilter::*)(hknpWorld *,hknpConstraintId)> *
            ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hknpConstraintId> *
        ]
    ),
    "hknpConstraintCollisionFilter::subscribeToWorld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintCollisionFilter *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpConstraintCollisionFilter::unsubscribeFromWorld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintCollisionFilter *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpBreakableCompoundShapeKeyMask::hknpBreakableCompoundShapeKeyMask": {
        "hknpBreakableCompoundShapeKeyMask *, const hknpBreakableCompoundShapeKeyMask *, const hknpCompoundShape *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpBreakableCompoundShapeKeyMask *
                ctypes.c_ulonglong,  # const hknpBreakableCompoundShapeKeyMask *
                ctypes.c_ulonglong,  # const hknpCompoundShape *
            ]
        ),
        "hknpBreakableCompoundShapeKeyMask *, const hknpBreakableCompoundShape *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpBreakableCompoundShapeKeyMask *
                ctypes.c_ulonglong,  # const hknpBreakableCompoundShape *
            ]
        ),
        "hknpBreakableCompoundShapeKeyMask *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpBreakableCompoundShapeKeyMask *
            ]
        ),
    },
    "hknpBreakableCompoundShapeKeyMask::~hknpBreakableCompoundShapeKeyMask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBreakableCompoundShapeKeyMask *
        ]
    ),
    "hknpBreakableCompoundShapeKeyMaskFunctions::commitChanges": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeKeyMask *
        ]
    ),
    "hknpBreakableCompoundShapeKeyMaskFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
            ctypes.c_ulonglong,  # const hknpShapeKeyMask *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpBreakableCompoundShape::hknpBreakableCompoundShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBreakableCompoundShape *
            ctypes.c_ulonglong,  # const hknpBreakableCompoundShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpConnectivityGraph::hknpConnectivityGraph": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConnectivityGraph *
            ctypes.c_ulonglong,  # const hknpConnectivityGraph *
        ]
    ),
    "hknpBreakableCompoundShape::~hknpBreakableCompoundShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBreakableCompoundShape *
        ]
    ),
    "hknpCompoundShapeEx::AabbOverlaps<2,hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>::addLeaves": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::AabbOverlaps<2,hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpCompoundShapeEx::AabbOverlaps<2,hknpCollisionQueryCollector>::addLeaves": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::AabbOverlaps<2,hknpCollisionQueryCollector> *
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpBreakableCompoundShapeFunctions::buildMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpBreakableCompoundShapeFunctions::calcAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpBreakableCompoundShapeFunctions::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpAllHitsCollector::getHits": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmMemory *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "hknpBreakableCompoundShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hknpBreakableCompoundShapeFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpBreakableCompoundShapeFunctions::makeShapeKeyMask": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpBreakableCompoundShapeFunctions::onCoreShapeMutated": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShape *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "hknpCompoundShapeEx::CastRayImpl<2,2>::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpBreakableCompoundShapeFunctions::queryAabb": {
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hknpCollisionQueryCollector *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
    },
    "hknpAllHitsCollector::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpAllHitsCollector *
        ]
    ),
    "hkcdTreeFilterUtils::computeValidBranchesMask<hknpCompoundShapeCdDynamicTree>": {
        "const hknpCompoundShapeCdDynamicTree *, const hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::Codec32> >::NodeContext *, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
                ctypes.c_ulonglong,  # const hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::Codec32> >::NodeContext *
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
        "const hknpCompoundShapeCdDynamicTree *, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const hknpCompoundShapeCdDynamicTree *
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
    },
    "hknpCompoundShapeInternalsCdDynamicTreeKeyMask::hknpCompoundShapeInternalsCdDynamicTreeKeyMask": {
        "hknpCompoundShapeInternalsCdDynamicTreeKeyMask *, const hknpCompoundShapeInternalsCdDynamicTreeKeyMask *, const hknpCompoundShape *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCompoundShapeInternalsCdDynamicTreeKeyMask *
                ctypes.c_ulonglong,  # const hknpCompoundShapeInternalsCdDynamicTreeKeyMask *
                ctypes.c_ulonglong,  # const hknpCompoundShape *
            ]
        ),
        "hknpCompoundShapeInternalsCdDynamicTreeKeyMask *, const hknpCompoundShape *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCompoundShapeInternalsCdDynamicTreeKeyMask *
                ctypes.c_ulonglong,  # const hknpCompoundShape *
            ]
        ),
    },
    "hknpCompoundShapeInternalsSimdTreeKeyMask::hknpCompoundShapeInternalsSimdTreeKeyMask": {
        "hknpCompoundShapeInternalsSimdTreeKeyMask *, const hknpCompoundShapeInternalsSimdTreeKeyMask *, const hknpCompoundShape *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCompoundShapeInternalsSimdTreeKeyMask *
                ctypes.c_ulonglong,  # const hknpCompoundShapeInternalsSimdTreeKeyMask *
                ctypes.c_ulonglong,  # const hknpCompoundShape *
            ]
        ),
        "hknpCompoundShapeInternalsSimdTreeKeyMask *, const hknpCompoundShape *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCompoundShapeInternalsSimdTreeKeyMask *
                ctypes.c_ulonglong,  # const hknpCompoundShape *
            ]
        ),
    },
    "hknpCompoundShapeInternalsCdDynamicTreeKeyMask::~hknpCompoundShapeInternalsCdDynamicTreeKeyMask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeInternalsCdDynamicTreeKeyMask *
        ]
    ),
    "hknpCompoundShapeInternalsSimdTreeKeyMask::~hknpCompoundShapeInternalsSimdTreeKeyMask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeInternalsSimdTreeKeyMask *
        ]
    ),
    "hknpCompoundShapeCdDynamicTreeKeyMaskFunctions::commitChanges": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeKeyMask *
        ]
    ),
    "hknpCompoundShapeSimdTreeKeyMaskFunctions::commitChanges": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeKeyMask *
        ]
    ),
    "hknpCompoundShapeCdDynamicTreeKeyMaskFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
            ctypes.c_ulonglong,  # const hknpShapeKeyMask *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpCompoundShapeSimdTreeKeyMaskFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
            ctypes.c_ulonglong,  # const hknpShapeKeyMask *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hkBitFieldBase<hkBitFieldStorage<hkRelArray<uint>>>::setSizeAndFill": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkBitFieldBase<hkBitFieldStorage<hkRelArray<unsigned int> > > *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkBitFieldStorage<hkArray<uint,hkBuiltinContainerAllocator<0,0>>>::hkBitFieldStorage<hkArray<uint,hkBuiltinContainerAllocator<0,0>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkBitFieldStorage<hkArray<unsigned int,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_ulonglong,  # const hkBitFieldStorage<hkArray<unsigned int,hkBuiltinContainerAllocator<0,0> > > *
        ]
    ),
    "hknpDefaultHeightFieldGeometry::hknpDefaultHeightFieldGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDefaultHeightFieldGeometry *
            ctypes.c_ulonglong,  # const hknpDefaultHeightFieldGeometry *
        ]
    ),
    "hknpDefaultHeightFieldGeometry::~hknpDefaultHeightFieldGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDefaultHeightFieldGeometry *
        ]
    ),
    "hknpDefaultHeightFieldGeometry::getNumHeights": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDefaultHeightFieldGeometry *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hknpDefaultHeightFieldGeometry::getQuadInfoAt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDefaultHeightFieldGeometry *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # hknpHeightFieldGeometry::QuadType *
        ]
    ),
    "hknpDefaultHeightFieldGeometry::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpHeightFieldGeometry> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpDefaultHeightFieldGeometry *
            ctypes.c_ulonglong,  # hkRefPtr<hknpHeightFieldGeometry> *
        ]
    ),
    "hkBitFieldBase<hkBitFieldStorage<hkArray<uint,hkBuiltinContainerAllocator<0,0>>>>::setSizeAndFill": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkBitFieldBase<hkBitFieldStorage<hkArray<unsigned int,hkBuiltinContainerAllocator<0,0> > > > *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkArrayBase<hknpCollisionCacheRange>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpCollisionCacheRange> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<uint>>>::SAHTask>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned int> > >::SAHTask> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkcdSimdTree::Point>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkBlockStream::Range> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<uint>>>::SAHBin>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned int> > >::SAHBin> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkcdStaticTree::convertDynamicTreeToStaticTree<hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb6BytesCodec>,hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<uint>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned int> > > *
            ctypes.c_ulonglong,  # hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb6BytesCodec> *
        ]
    ),
    "hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<uint>>>::copyTo<hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<uint>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned int> > > *
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned int> > > *
        ]
    ),
    "hknpExternMeshShapeBuildContext::hknpExternMeshShapeBuildContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShapeBuildContext *
        ]
    ),
    "hknpExternMeshShapeData::hknpExternMeshShapeData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShapeData *
            ctypes.c_ulonglong,  # const hknpExternMeshShapeData *
        ]
    ),
    "hkcdSimdTree::BuildContext::~BuildContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree::BuildContext *
        ]
    ),
    "hknpExternMeshShapeData::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShapeData *
        ]
    ),
    "hknpExternMeshShape::buildBoundingVolume": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpExternMeshShapeData *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpExternMeshShapeGeometry *
            ctypes.c_ulonglong,  # const hknpExternMeshShape::BuildConfig *
            ctypes.c_ulonglong,  # hkArray<int,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArray<int,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<uint>>>::compactIndices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned int> > > *
        ]
    ),
    "hknpExternMeshShapeData::getMemoryFootprint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShapeData *
        ]
    ),
    "hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<uint>>>::rebuildBranchSAH": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned int> > > *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdDynamicTree::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::CodecRaw<uint>>::reserveNodes": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::CodecRaw<unsigned int> > *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpBoxShape::hknpBoxShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBoxShape *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hknpConvexShapeBuildConfig *
        ]
    ),
    "hknpBoxShape::~hknpBoxShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBoxShape *
        ]
    ),
    "hknpBoxShapeFunctions::buildMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpBoxShapeFunctions::calcAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpBoxShapeFunctions::calcBoundingSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpBoxShapeFunctions::calcInnerSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpBoxShapeFunctions::getConvexVertices": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpConvexVertexShape *
            ctypes.c_ulonglong,  # hkArrayView<hkFloat3> *
        ]
    ),
    "hknpBoxShapeFunctions::getMemoryFootprint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpShapeSet *
        ]
    ),
    "hknpBoxShapeFunctions::getSupportingFace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkcdGsk::Cache *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hknpAngularTimType *
        ]
    ),
    "hknpBoxShapeFunctions::getSupportingVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpBoxShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hknpBoxShape::initialize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBoxShape *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "hknpBoxShape::makeBoxShape": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpBoxShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpBoxShape> *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hknpConvexShapeBuildConfig *
        ]
    ),
    "hknpBoxShapeFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpBoxShapeFunctions::queryPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCylinderShape::~hknpCylinderShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCylinderShape *
        ]
    ),
    "hknpCylinderShapeFunctions::buildMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_float,  # float
        ]
    ),
    "hknpCylinderShapeFunctions::calcBoundingSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpCylinderShapeFunctions::calcInnerSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpCylinderShapeFunctions::calcMinAngleBetweenFaces": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpCylinderShapeFunctions::getCircularSymmetryAxis": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCircularSymmetryAxis *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpCylinderShapeFunctions::getMemoryFootprint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpShapeSet *
        ]
    ),
    "hknpCylinderShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hknpCylinderShape::initialize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCylinderShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_double,  # double
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "hknpCylinderShapeFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "queryAabbCompositeImpl<hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpTransformedShape *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "queryAabbCompositeImpl<hknpCollisionQueryCollector>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpTransformedShape *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "queryAabbConvexImpl<hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpTransformedShape *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "queryAabbConvexImpl<hknpCollisionQueryCollector>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpTransformedShape *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpTransformedShape::~hknpTransformedShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpTransformedShape *
        ]
    ),
    "hknpTransformedShapeEx::applyScaleAndTransformToMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpTransformedShapeEx *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpTransformedShapeFunctions::buildMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpTransformedShapeFunctions::buildSurfaceGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::BuildSurfaceGeometryConfig *
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpTransformedShapeFunctions::calcAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpTransformedShapeFunctions::calcBoundingRadius": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpTransformedShapeFunctions::calcBoundingSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpTransformedShapeFunctions::calcInnerRadius": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpTransformedShapeFunctions::calcInnerSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpTransformedShapeFunctions::calcMinAngleBetweenFaces": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpTransformedShapeFunctions::calcSurfaceVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::SurfaceVelocityInput *
            ctypes.c_ulonglong,  # hknpShape::SurfaceVelocityOutput *
        ]
    ),
    "hknpTransformedShapeFunctions::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpTransformedShapeFunctions::convertVertexIdsToVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpTransformedShapeFunctions::getCircularSymmetryAxis": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCircularSymmetryAxis *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "hknpTransformedShapeFunctions::getConvexVertices": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpConvexVertexShape *
            ctypes.c_ulonglong,  # hkArrayView<hkFloat3> *
            ctypes.c_ulonglong,  # __int128 *
        ]
    ),
    "hknpTransformedShapeFunctions::getFaceInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hknpAngularTimType *
        ]
    ),
    "hknpTransformedShapeFunctions::getFaceVertices": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpTransformedShapeFunctions::getLeafShapes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpShapeCollector *
        ]
    ),
    "hknpTransformedShapeFunctions::getMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpTransformedShapeFunctions::getMemoryFootprint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpShapeSet *
        ]
    ),
    "hknpTransformedShapeFunctions::getMutationSignals": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::atomic<unsigned __int64> *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpTransformedShapeFunctions::getSupportSpheres": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkFloat3 *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkFloat3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpTransformedShapeFunctions::getSupportingFace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkcdGsk::Cache *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hknpAngularTimType *
        ]
    ),
    "hknpTransformedShapeFunctions::getSupportingVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpTransformedShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hknpTransformedShapeFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpTransformedShape::makeCloneImpl": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpTransformedShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpTransformedShape *
            ctypes.c_ulonglong,  # hkRefPtr<hknpTransformedShape> *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpTransformedShapeFunctions::queryAabb": {
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hknpCollisionQueryCollector *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
    },
    "hknpTransformedShapeFunctions::queryPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpDummyShape::~hknpDummyShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDummyShape *
        ]
    ),
    "hknpDummyShapeFunctions::calcAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpDummyShapeFunctions::getMutationSignals": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpShapeSignals *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpDummyShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hknpDummyShapeFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hkArrayBase<hkGeometry>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkGeometry> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkGeometry>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkGeometry> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkcdSimdTree::aabbCast<hknpParticlesColliderShapeQueryHelperStructs::hknpParticlesShapeCastQuery>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdRay *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hknpParticlesColliderShapeQueryHelperStructs::hknpParticlesShapeCastQuery *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromAabb<hknpParticlesColliderShapeQueryHelperStructs::hknpParticlesClosestPointsToConvexQuery>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpParticlesColliderShapeQueryHelperStructs::hknpParticlesClosestPointsToConvexQuery *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromAabbOrdered<hknpParticlesColliderShapeQueryHelperStructs::hknpParticlesClosestPointsToConvexQuery>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpParticlesColliderShapeQueryHelperStructs::hknpParticlesClosestPointsToConvexQuery *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hknpParticlesColliderShape::hknpParticlesColliderShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesColliderShape *
            ctypes.c_ulonglong,  # hknpParticlesCollider *
        ]
    ),
    "hknpParticlesColliderShape::~hknpParticlesColliderShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesColliderShape *
        ]
    ),
    "hknpParticlesColliderShapeFunctions::buildSurfaceGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::BuildSurfaceGeometryConfig *
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpParticlesColliderShapeFunctions::calcAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpParticlesColliderShapeFunctions::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpParticlesColliderShapeFunctions::castShapeImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpParticlesColliderShapeFunctions::getClosestPointsFromConvex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpParticlesColliderShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpParticlesColliderShapeFunctions::getLeafShapes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpShapeCollector *
        ]
    ),
    "cTkRigidBody::GetProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysicsProperties *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "hknpParticlesColliderShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hknpParticlesColliderShapeFunctions::queryAabb": {
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hknpCollisionQueryCollector *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
    },
    "hknpParticlesColliderShapeFunctions::queryPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpBridgeConstraintAtom::hknpBridgeConstraintAtom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBridgeConstraintAtom *
        ]
    ),
    "hknpBridgeConstraintAtom::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBridgeConstraintAtom *
        ]
    ),
    "hknpBridgeConstraintAtomBase::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBridgeConstraintAtomBase *
        ]
    ),
    "hkBridgeConstraintAtom_callData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_ulonglong,  # const hknpConstraintQueryIn *
            ctypes.c_ulonglong,  # hknpConstraintQueryOut *
        ]
    ),
    "hknpBridgeConstraintAtom::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBridgeConstraintAtom *
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hknpBridgeConstraintAtomBase::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBridgeConstraintAtomBase *
            ctypes.c_ulonglong,  # hknpConstraintData *
        ]
    ),
    "hknp6DofConstraintData::~hknp6DofConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknp6DofConstraintData *
        ]
    ),
    "hknp6DofConstraintData::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknp6DofConstraintData *
        ]
    ),
    "hknp6DofConstraintDataFunctions::getAppliedImpulsesInSolverSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hknpConstraintData::AppliedImpulses *
        ]
    ),
    "hknp6DofConstraintDataFunctions::getAppliedLinearImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknp6DofConstraintDataFunctions::getConstraintInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintData::ConstraintInfo *
        ]
    ),
    "hknp6DofConstraintDataFunctions::getInertiaStabilizationFactor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpConstraintData *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknp6DofConstraintDataFunctions::getRuntimeInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpConstraintData::RuntimeInfo *
        ]
    ),
    "hknp6DofConstraintDataFunctions::isValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknp6DofConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpEllipticalLimitConstraintAtom::makeStableByUsingConeAngle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpEllipticalLimitConstraintAtom *
            ctypes.c_float,  # float
        ]
    ),
    "hknp6DofConstraintData::setInBodySpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknp6DofConstraintData *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknp6DofConstraintDataFunctions::setInertiaStabilizationFactor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpConstraintData *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "hknp6DofConstraintDataFunctions::updateDirtyAtoms": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
        ]
    ),
    "hknpAngularFrictionConstraintData::hknpAngularFrictionConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpAngularFrictionConstraintData *
        ]
    ),
    "hknpCogWheelConstraintDataFunctions::getAppliedImpulsesInSolverSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hknpConstraintData::AppliedImpulses *
        ]
    ),
    "hknpRotationalConstraintDataFunctions::getAppliedLinearImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpAngularFrictionConstraintDataFunctions::getConstraintInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintData::ConstraintInfo *
        ]
    ),
    "hknpAngularFrictionConstraintDataFunctions::getRuntimeInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpConstraintData::RuntimeInfo *
        ]
    ),
    "hknpPointToPlaneConstraintDataFunctions::isValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpAngularFrictionConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpBallAndSocketConstraintData::hknpBallAndSocketConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBallAndSocketConstraintData *
        ]
    ),
    "hknpStiffSpringConstraintDataFunctions::getAppliedImpulsesInSolverSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hknpConstraintData::AppliedImpulses *
        ]
    ),
    "hknpDeformableFixedConstraintDataFunctions::getAppliedLinearImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpBallAndSocketConstraintDataFunctions::getConstraintInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintData::ConstraintInfo *
        ]
    ),
    "hknpBallAndSocketConstraintDataFunctions::getInertiaStabilizationFactor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpConstraintData *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpBallAndSocketConstraintDataFunctions::getMaximumAngularImpulse": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpBallAndSocketConstraintDataFunctions::getMaximumLinearImpulse": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpRotationalConstraintDataFunctions::getRuntimeInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpConstraintData::RuntimeInfo *
        ]
    ),
    "hknpBallAndSocketConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpBallAndSocketConstraintData::setInBodySpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBallAndSocketConstraintData *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpBallAndSocketConstraintDataFunctions::setInertiaStabilizationFactor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpConstraintData *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "hknpBallAndSocketConstraintDataFunctions::setMaximumAngularImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_float,  # float
        ]
    ),
    "hknpBallAndSocketConstraintDataFunctions::setMaximumLinearImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_float,  # float
        ]
    ),
    "hknpLinearClearanceConstraintData::hknpLinearClearanceConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearClearanceConstraintData *
        ]
    ),
    "hknpLinearClearanceConstraintData::~hknpLinearClearanceConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearClearanceConstraintData *
        ]
    ),
    "hknpLinearClearanceConstraintDataFunctions::getAppliedImpulsesInSolverSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hknpConstraintData::AppliedImpulses *
        ]
    ),
    "hknpLinearClearanceConstraintDataFunctions::getAppliedLinearImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpLinearClearanceConstraintDataFunctions::getConstraintInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintData::ConstraintInfo *
        ]
    ),
    "hknpLinearClearanceConstraintDataFunctions::getRuntimeInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpConstraintData::RuntimeInfo *
        ]
    ),
    "hknpLinearClearanceConstraintDataFunctions::isValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpLinearClearanceConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpCogWheelConstraintData::hknpCogWheelConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCogWheelConstraintData *
        ]
    ),
    "hknpCogWheelConstraintDataFunctions::getConstraintInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintData::ConstraintInfo *
        ]
    ),
    "hknpCogWheelConstraintDataFunctions::getRuntimeInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpConstraintData::RuntimeInfo *
        ]
    ),
    "hknpCogWheelConstraintDataFunctions::isValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpCogWheelConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpDeformableFixedConstraintData::hknpDeformableFixedConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeformableFixedConstraintData *
        ]
    ),
    "hknpDeformableFixedConstraintDataFunctions::getAppliedImpulsesInSolverSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hknpConstraintData::AppliedImpulses *
        ]
    ),
    "hknpFixedConstraintDataFunctions::getRuntimeInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpConstraintData::RuntimeInfo *
        ]
    ),
    "hknpDeformableFixedConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpFixedConstraintData::setInBodySpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFixedConstraintData *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
        ]
    ),
    "hknpFixedConstraintData::hknpFixedConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFixedConstraintData *
        ]
    ),
    "hknpFixedConstraintDataFunctions::getAppliedImpulsesInSolverSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hknpConstraintData::AppliedImpulses *
        ]
    ),
    "hknpHingeConstraintDataFunctions::getConstraintInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintData::ConstraintInfo *
        ]
    ),
    "hknpFixedConstraintDataFunctions::getInertiaStabilizationFactor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpConstraintData *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpFixedConstraintDataFunctions::getMaximumAngularImpulse": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpFixedConstraintDataFunctions::getMaximumLinearImpulse": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpFixedConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpFixedConstraintDataFunctions::setInertiaStabilizationFactor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpConstraintData *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "hknpFixedConstraintDataFunctions::setMaximumAngularImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_float,  # float
        ]
    ),
    "hknpFixedConstraintDataFunctions::setMaximumLinearImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_float,  # float
        ]
    ),
    "hknpHingeConstraintData::hknpHingeConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpHingeConstraintData *
        ]
    ),
    "hknpHingeConstraintDataFunctions::getAppliedImpulsesInSolverSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hknpConstraintData::AppliedImpulses *
        ]
    ),
    "hknpHingeConstraintDataFunctions::getAppliedLinearImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpHingeConstraintDataFunctions::getInertiaStabilizationFactor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpConstraintData *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpHingeConstraintDataFunctions::getRuntimeInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpConstraintData::RuntimeInfo *
        ]
    ),
    "hknpHingeConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpHingeConstraintData::setInBodySpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpHingeConstraintData *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpHingeConstraintDataFunctions::setInertiaStabilizationFactor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpConstraintData *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "hknpLimitedHingeConstraintData::hknpLimitedHingeConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLimitedHingeConstraintData *
        ]
    ),
    "hknpLimitedHingeConstraintData::~hknpLimitedHingeConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLimitedHingeConstraintData *
        ]
    ),
    "hknpLimitedHingeConstraintDataFunctions::getAppliedImpulsesInSolverSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hknpConstraintData::AppliedImpulses *
        ]
    ),
    "hknpLimitedHingeConstraintDataFunctions::getAppliedLinearImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpLimitedHingeConstraintDataFunctions::getConstraintInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintData::ConstraintInfo *
        ]
    ),
    "hknpLimitedHingeConstraintDataFunctions::getInertiaStabilizationFactor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpConstraintData *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpLimitedHingeConstraintDataFunctions::getRuntimeInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpConstraintData::RuntimeInfo *
        ]
    ),
    "hknpLimitedHingeConstraintDataFunctions::isValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpLimitedHingeConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpLimitedHingeConstraintData::setInBodySpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLimitedHingeConstraintData *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpLimitedHingeConstraintDataFunctions::setInertiaStabilizationFactor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpConstraintData *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "hknpPointToPathConstraintData::hknpPointToPathConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPointToPathConstraintData *
        ]
    ),
    "hknpPointToPathConstraintData::~hknpPointToPathConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPointToPathConstraintData *
        ]
    ),
    "hknpPointToPathConstraintDataFunctions::buildJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_ulonglong,  # const hknpConstraintQueryIn *
            ctypes.c_ulonglong,  # hknpConstraintQueryOut *
        ]
    ),
    "hknpPointToPathConstraintDataFunctions::getConstraintInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintData::ConstraintInfo *
        ]
    ),
    "hknpPointToPathConstraintDataFunctions::getRuntimeInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpConstraintData::RuntimeInfo *
        ]
    ),
    "hknpPointToPathConstraintDataFunctions::isValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpPointToPathConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpPointToPathConstraintData::makeCloneImpl": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpPointToPathConstraintData *
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpPointToPathConstraintData::setPath": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPointToPathConstraintData *
            ctypes.c_ulonglong,  # hknpParametricCurve *
        ]
    ),
    "hknpPointToPlaneConstraintData::hknpPointToPlaneConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPointToPlaneConstraintData *
        ]
    ),
    "hknpPointToPlaneConstraintDataFunctions::getAppliedLinearImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpStiffSpringConstraintDataFunctions::getRuntimeInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpConstraintData::RuntimeInfo *
        ]
    ),
    "hknpPointToPlaneConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpPointToPlaneConstraintData::setInBodySpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPointToPlaneConstraintData *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpPrismaticConstraintData::hknpPrismaticConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPrismaticConstraintData *
        ]
    ),
    "hknpPrismaticConstraintData::~hknpPrismaticConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPrismaticConstraintData *
        ]
    ),
    "hknpPrismaticConstraintDataFunctions::getAppliedImpulsesInSolverSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hknpConstraintData::AppliedImpulses *
        ]
    ),
    "hknpPrismaticConstraintDataFunctions::getAppliedLinearImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpPrismaticConstraintDataFunctions::getRuntimeInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpConstraintData::RuntimeInfo *
        ]
    ),
    "hknpPrismaticConstraintDataFunctions::isValid": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpPrismaticConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpPrismaticConstraintData::setInBodySpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPrismaticConstraintData *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpPulleyConstraintData::hknpPulleyConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPulleyConstraintData *
        ]
    ),
    "hknpPulleyConstraintDataFunctions::getAppliedImpulsesInSolverSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hknpConstraintData::AppliedImpulses *
        ]
    ),
    "hknpPulleyConstraintDataFunctions::getAppliedLinearImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpPulleyConstraintDataFunctions::getConstraintInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintData::ConstraintInfo *
        ]
    ),
    "hknpPulleyConstraintDataFunctions::isValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpPulleyConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpRackAndPinionConstraintData::hknpRackAndPinionConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRackAndPinionConstraintData *
        ]
    ),
    "hknpRackAndPinionConstraintDataFunctions::getRuntimeInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpConstraintData::RuntimeInfo *
        ]
    ),
    "hknpRackAndPinionConstraintDataFunctions::isValid": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpRackAndPinionConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpRagdollConstraintData::hknpRagdollConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRagdollConstraintData *
        ]
    ),
    "hknpRagdollConstraintData::~hknpRagdollConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRagdollConstraintData *
        ]
    ),
    "hknpRagdollConstraintDataFunctions::getAppliedImpulsesInSolverSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hknpConstraintData::AppliedImpulses *
        ]
    ),
    "hknpRagdollConstraintDataFunctions::getAppliedLinearImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpRagdollConstraintData::getConstraintFrameA": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRagdollConstraintData *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
        ]
    ),
    "hknpRagdollConstraintData::getConstraintFrameB": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRagdollConstraintData *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
        ]
    ),
    "hknpRagdollConstraintDataFunctions::getConstraintInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintData::ConstraintInfo *
        ]
    ),
    "hknpRagdollConstraintDataFunctions::getInertiaStabilizationFactor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpConstraintData *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpRagdollConstraintData::getPlaneMotor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpConstraintMotor *
        argtypes=[
            ctypes.c_ulonglong,  # hknpRagdollConstraintData *
        ]
    ),
    "hknpRagdollConstraintDataFunctions::getRuntimeInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpConstraintData::RuntimeInfo *
        ]
    ),
    "hknpRagdollConstraintDataFunctions::isValid": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpRagdollConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpRagdollConstraintData::setConeMotor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRagdollConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintMotor *
        ]
    ),
    "hknpRagdollConstraintData::setInBodySpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRagdollConstraintData *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpRagdollConstraintData::setInWorldSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRagdollConstraintData *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpRagdollConstraintDataFunctions::setInertiaStabilizationFactor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpConstraintData *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "hknpRagdollConstraintData::setMaxFrictionTorque": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRagdollConstraintData *
            ctypes.c_float,  # float
        ]
    ),
    "hknpRagdollConstraintData::setMotor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRagdollConstraintData *
            ctypes.c_uint32,  # hknpRagdollConstraintData::MotorIndex
            ctypes.c_ulonglong,  # hknpConstraintMotor *
        ]
    ),
    "hknpRagdollConstraintData::setPlaneMotor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRagdollConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintMotor *
        ]
    ),
    "hknpRagdollConstraintData::setTwistMotor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRagdollConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintMotor *
        ]
    ),
    "hknpRotationalConstraintData::hknpRotationalConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRotationalConstraintData *
        ]
    ),
    "hknpRotationalConstraintDataFunctions::getAppliedImpulsesInSolverSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hknpConstraintData::AppliedImpulses *
        ]
    ),
    "hknpRotationalConstraintDataFunctions::getConstraintInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintData::ConstraintInfo *
        ]
    ),
    "hknpRotationalConstraintDataFunctions::isValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpRotationalConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpStiffSpringConstraintData::hknpStiffSpringConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpStiffSpringConstraintData *
        ]
    ),
    "hknpStiffSpringConstraintDataFunctions::getAppliedLinearImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpStiffSpringConstraintDataFunctions::isValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpStiffSpringConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpWheelFrictionConstraintData::hknpWheelFrictionConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWheelFrictionConstraintData *
        ]
    ),
    "hknpWheelFrictionConstraintDataFunctions::getAppliedImpulsesInSolverSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hknpConstraintData::AppliedImpulses *
        ]
    ),
    "hknpWheelFrictionConstraintDataFunctions::getAppliedLinearImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpWheelFrictionConstraintDataFunctions::getRuntimeInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpConstraintData::RuntimeInfo *
        ]
    ),
    "hknpWheelFrictionConstraintDataFunctions::isValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpWheelFrictionConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpWheelConstraintData::hknpWheelConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWheelConstraintData *
        ]
    ),
    "hknpWheelConstraintDataFunctions::getAppliedImpulsesInSolverSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hknpConstraintData::AppliedImpulses *
        ]
    ),
    "hknpWheelConstraintDataFunctions::getAppliedLinearImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpWheelConstraintDataFunctions::getConstraintInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintData::ConstraintInfo *
        ]
    ),
    "hknpWheelConstraintDataFunctions::isValid": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpWheelConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpWheelConstraintData::setInBodySpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWheelConstraintData *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpWrappedConstraintData::hknpWrappedConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWrappedConstraintData *
            ctypes.c_ulonglong,  # const hknpWrappedConstraintData *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpWrappedConstraintData::~hknpWrappedConstraintData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWrappedConstraintData *
        ]
    ),
    "hknpWrappedConstraintDataFunctions::enableDirectSolver": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpConstraintData *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_int64,  # __int64
            ctypes.c_char,  # char
        ]
    ),
    "hknpWrappedConstraintDataFunctions::getAppliedImpulsesInSolverSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hknpConstraintData::AppliedImpulses *
        ]
    ),
    "hknpWrappedConstraintDataFunctions::getAppliedLinearImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpWrappedConstraintDataFunctions::getConstraintInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintData::ConstraintInfo *
        ]
    ),
    "hknpWrappedConstraintDataFunctions::getRuntimeInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ubyte,  # _BOOL8
            ctypes.c_ulonglong,  # hknpConstraintData::RuntimeInfo *
        ]
    ),
    "hknpWrappedConstraintData::getWrappedConstraintData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpConstraintData *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWrappedConstraintData *
        ]
    ),
    "hknpWrappedConstraintDataFunctions::updateDirtyAtoms": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
        ]
    ),
    "hknpPositionConstraintMotor::hknpPositionConstraintMotor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPositionConstraintMotor *
        ]
    ),
    "hknpPositionConstraintMotor::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintMotor> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpPositionConstraintMotor *
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintMotor> *
        ]
    ),
    "hknpSpringDamperConstraintMotor::hknpSpringDamperConstraintMotor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSpringDamperConstraintMotor *
        ]
    ),
    "hknpSpringDamperConstraintMotor::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintMotor> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpSpringDamperConstraintMotor *
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintMotor> *
        ]
    ),
    "hknpVelocityConstraintMotor::hknpVelocityConstraintMotor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVelocityConstraintMotor *
        ]
    ),
    "hknpVelocityConstraintMotor::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintMotor> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpVelocityConstraintMotor *
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintMotor> *
        ]
    ),
    "hknpMaterialLibrary::hknpMaterialLibrary": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMaterialLibrary *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpMaterialLibrary::~hknpMaterialLibrary": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMaterialLibrary *
        ]
    ),
    "hknpMaterialLibrary::addEntry": {
        "hknpMaterialLibrary *, hknpMaterialId *, const hknpMaterialDescriptor *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hknpMaterialId *
            argtypes=[
                ctypes.c_ulonglong,  # hknpMaterialLibrary *
                ctypes.c_ulonglong,  # hknpMaterialId *
                ctypes.c_ulonglong,  # const hknpMaterialDescriptor *
            ]
        ),
        "hknpMaterialLibrary *, hknpMaterialId *, const hknpMaterial *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hknpMaterialId *
            argtypes=[
                ctypes.c_ulonglong,  # hknpMaterialLibrary *
                ctypes.c_ulonglong,  # hknpMaterialId *
                ctypes.c_ulonglong,  # const hknpMaterial *
            ]
        ),
    },
    "hknpMaterialLibrary::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMaterialLibrary *
        ]
    ),
    "hknpMaterialLibrary::findEntryByName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpMaterialId *
        argtypes=[
            ctypes.c_ulonglong,  # hknpMaterialLibrary *
            ctypes.c_ulonglong,  # hknpMaterialId *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkSignal2<hknpWorld *,int>::fire": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,int> *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpMotionPropertiesLibrary::hknpMotionPropertiesLibrary": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotionPropertiesLibrary *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpMotionPropertiesLibrary::addEntry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpMotionPropertiesId *
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotionPropertiesLibrary *
            ctypes.c_ulonglong,  # hknpMotionPropertiesId *
            ctypes.c_ulonglong,  # const hknpMotionProperties *
        ]
    ),
    "hknpThreadSafeObjectPool<hknpMotionProperties>::free": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpMotionProperties> *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpMotionPropertiesLibrary::initialize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotionPropertiesLibrary *
            ctypes.c_double,  # double
        ]
    ),
    "hknpMotionPropertiesLibrary::setCapacity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotionPropertiesLibrary *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpParticlesColliderCinfo::hknpParticlesColliderCinfo": {
        "hknpParticlesColliderCinfo *, const hknpParticlesColliderCinfo *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpParticlesColliderCinfo *
                ctypes.c_ulonglong,  # const hknpParticlesColliderCinfo *
            ]
        ),
        "hknpParticlesColliderCinfo *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpParticlesColliderCinfo *
            ]
        ),
    },
    "hknpParticlesColliderCinfo::~hknpParticlesColliderCinfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesColliderCinfo *
        ]
    ),
    "hkQueue<hknpBodyId,hkBuiltinContainerAllocator<0,0>>::enqueue<hknpBodyId const &>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkQueue<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # const hknpBodyId *
        ]
    ),
    "hknpMountedBallGun::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMountedBallGun *
        ]
    ),
    "hknpMountedBallGun::fireGun": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMountedBallGun *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hkTransformf *
        ]
    ),
    "hknpFirstPersonCharacterCinfo::hknpFirstPersonCharacterCinfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFirstPersonCharacterCinfo *
        ]
    ),
    "hknpFirstPersonCharacter::~hknpFirstPersonCharacter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFirstPersonCharacter *
        ]
    ),
    "hknpFirstPersonCharacterCinfo::~hknpFirstPersonCharacterCinfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFirstPersonCharacterCinfo *
        ]
    ),
    "hkArrayBase<hknpCharacterProxy::Contact>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpCharacterProxy::Contact> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpCharacterProxy::TriggerInteraction>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpParticlesCollider::DirtyAabb> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpCsJacRange>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpProcessFullCastsTask::SubTask> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkParallel::IteratorContext::ThreadData>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkParallel::IteratorContext::ThreadData> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkSurfaceConstraintInfo>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkSurfaceConstraintInfo> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkSignal1<hknpWorld *>::subscribe<hknpCharacterProxy,void ": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal1<hknpWorld *>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal1<hknpWorld *> *
            ctypes.c_ulonglong,  # hknpCharacterProxy *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpCharacterProxy*', 'name': 'this'}, 'hknpWorld *']}
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hknpCharacterProxyInternals::QueryCollector::QueryCollector": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals::QueryCollector *
            ctypes.c_ulonglong,  # hknpAllHitsCollector *
            ctypes.c_ulonglong,  # const hknpMaterialLibrary *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hknpCharacterProxy::~hknpCharacterProxy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxy *
        ]
    ),
    "hknpCharacterProxyInternals::QueryCollector::addHit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals::QueryCollector *
            ctypes.c_ulonglong,  # const hknpCollisionResult *
        ]
    ),
    "hknpCharacterProxy::checkSupportWithCollector": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxy *
            ctypes.c_ulonglong,  # const hkStepInfo *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hknpCharacterSurfaceInfo *
            ctypes.c_ulonglong,  # hknpAllHitsCollector *
        ]
    ),
    "hknpCharacterProxyInternals::QueryCollector::getHits": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpCollisionResult *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals::QueryCollector *
        ]
    ),
    "hknpCharacterProxy::integrateWithCollectors": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxy *
            ctypes.c_ulonglong,  # const hkStepInfo *
            ctypes.c_ulonglong,  # const hknpCharacterSurfaceInfo *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hknpAllHitsCollector *
            ctypes.c_ulonglong,  # hknpAllHitsCollector *
            ctypes.c_ulonglong,  # hkArray<hknpCharacterProxy::ObjectInteraction,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpCharacterProxyInternals::QueryCollector::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals::QueryCollector *
        ]
    ),
    "hknpCharacterProxyInternals::resolveContacts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals *
            ctypes.c_ulonglong,  # const hkStepInfo *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkArray<hknpCharacterProxy::ObjectInteraction,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpCharacterProxy::shift": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxy *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkSignal1<hknpWorld *>::MemberSlot<hknpCharacterProxy,void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal1<hknpWorld *>::MemberSlot<hknpCharacterProxy,void (__cdecl hknpCharacterProxy::*)(hknpWorld *)> *
            ctypes.c_ulonglong,  # hkSignal1<hknpWorld *> *
        ]
    ),
    "hknpCharacterProxyInternals::updateOverlappingTriggers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals *
            ctypes.c_ulonglong,  # const hkLocalArray<hknpCharacterProxy::TriggerInteraction,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpCharacterProxyInternals::updateTriggersSeen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCharacterProxyInternals::QueryCollector *
            ctypes.c_ulonglong,  # const hknpCharacterProxyInternals::QueryCollector *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkLocalArray<hknpCharacterProxy::TriggerInteraction,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkArrayBase<hknpCharacterRigidBody::ContactPointInfo>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpCharacterRigidBody::ContactPointInfo> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkSignal2<hknpEventHandlerInput const &,hknpEvent const &>::subscribe<hknpCharacterRigidBody,void ": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal2<hknpEventHandlerInput const &,hknpEvent const &>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpEventHandlerInput const &,hknpEvent const &> *
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpCharacterRigidBody*', 'name': 'this'}, 'const hknpEventHandlerInput *', 'const hknpEvent *']}
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkSignal1<hknpWorld *>::subscribe<hknpCharacterRigidBody,void ": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal1<hknpWorld *>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal1<hknpWorld *> *
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpCharacterRigidBody*', 'name': 'this'}, 'hknpWorld *']}
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkSignal2<hknpWorld *,hknpStepInput const &>::subscribe<hknpCharacterRigidBody,void ": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hknpStepInput const &>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hknpStepInput const &> *
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpCharacterRigidBody*', 'name': 'this'}, 'hknpWorld *', 'const hknpStepInput *']}
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hknpCharacterRigidBody::hknpCharacterRigidBody": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
            ctypes.c_ulonglong,  # const hknpCharacterRigidBodyCinfo *
        ]
    ),
    "hknpCharacterRigidBodyCinfo::hknpCharacterRigidBodyCinfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterRigidBodyCinfo *
        ]
    ),
    "hknpCharacterRigidBody::~hknpCharacterRigidBody": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
        ]
    ),
    "hknpCharacterRigidBody::addImmediateContactConstraint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # const hknpCharacterRigidBody::ContactPointInfo *
            ctypes.c_ulonglong,  # const hknpMaterial *
            ctypes.c_ulonglong,  # hknpSolverData *
        ]
    ),
    "hknpCharacterRigidBody::checkSupport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
            ctypes.c_ulonglong,  # const hkStepInfo *
            ctypes.c_ulonglong,  # hknpCharacterSurfaceInfo *
        ]
    ),
    "hknpCharacterRigidBody::findSupportingContacts": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
            ctypes.c_ulonglong,  # const hkStepInfo *
            ctypes.c_ulonglong,  # hkArray<int,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpCharacterRigidBody::getGround": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
            ctypes.c_ulonglong,  # const hkArray<int,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hknpCharacterSurfaceInfo *
            ctypes.c_ulonglong,  # const hkStepInfo *
        ]
    ),
    "hknpCharacterRigidBody::onCollisionDetected": {
        "hknpCharacterRigidBody *, const hknpEventHandlerInput *, const hknpBinaryBodyEvent *, hknpManifoldCollisionCache *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkVector4f *, const hkVector4f *, const hkSimdFloat32 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCharacterRigidBody *
                ctypes.c_ulonglong,  # const hknpEventHandlerInput *
                ctypes.c_ulonglong,  # const hknpBinaryBodyEvent *
                ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
                ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ]
        ),
        "hknpCharacterRigidBody *, const hknpEventHandlerInput *, const hknpEvent *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCharacterRigidBody *
                ctypes.c_ulonglong,  # const hknpEventHandlerInput *
                ctypes.c_ulonglong,  # const hknpEvent *
            ]
        ),
    },
    "hknpCharacterRigidBody::onCollisionDetectedForwarder": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
            ctypes.c_ulonglong,  # const hknpEventHandlerInput *
            ctypes.c_ulonglong,  # const hknpEvent *
        ]
    ),
    "hknpCharacterRigidBody::onPostCollideSignal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpCharacterRigidBody::onPotentialManifold": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
            ctypes.c_ulonglong,  # const hknpEventHandlerInput *
            ctypes.c_ulonglong,  # const hknpEvent *
        ]
    ),
    "FileCaptureWriter::PassAudioData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # FileCaptureWriter *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpCharacterRigidBody::onPreCollideSignal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpStepInput *
        ]
    ),
    "hknpCharacterRigidBody::setAngularVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpCharacterRigidBody::setLinearVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ubyte,  # hkBool
        ]
    ),
    "hknpCharacterRigidBody::setListener": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
            ctypes.c_ulonglong,  # hknpCharacterRigidBodyListener *
        ]
    ),
    "hknpCharacterRigidBody::setTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
            ctypes.c_ulonglong,  # const hkTransformf *
        ]
    ),
    "hkSignal2<hknpEventHandlerInput const &,hknpEvent const &>::MemberSlot<hknpCharacterRigidBody,void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpEventHandlerInput const &,hknpEvent const &>::MemberSlot<hknpCharacterRigidBody,void (__cdecl hknpCharacterRigidBody::*)(hknpEventHandlerInput const &,hknpEvent const &)> *
            ctypes.c_ulonglong,  # hkSignal2<hknpEventHandlerInput const &,hknpEvent const &> *
        ]
    ),
    "hkSignal2<hknpWorld *,hknpStepInput const &>::MemberSlot<hknpCharacterRigidBody,void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hknpStepInput const &>::MemberSlot<hknpCharacterRigidBody,void (__cdecl hknpCharacterRigidBody::*)(hknpWorld *,hknpStepInput const &)> *
            ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hknpStepInput const &> *
        ]
    ),
    "hknpCharacterRigidBody::subscribeWorldSignals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
        ]
    ),
    "hknpPhysicsSceneData::hknpPhysicsSceneData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPhysicsSceneData *
        ]
    ),
    "hknpPhysicsSceneData::~hknpPhysicsSceneData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPhysicsSceneData *
        ]
    ),
    "hkArrayBase<hknpPhysicsSystemData::ExtendedBodyCinfo>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpPhysicsSystemData::ExtendedBodyCinfo> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpRagdollData::hknpRagdollData": {
        "hknpRagdollData *, const hknpRagdollData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpRagdollData *
                ctypes.c_ulonglong,  # const hknpRagdollData *
            ]
        ),
        "hknpRagdollData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpRagdollData *
            ]
        ),
    },
    "hknpRagdollData::copyPhysicsSystemData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRagdollData *
            ctypes.c_ulonglong,  # const hknpPhysicsSystemData *
        ]
    ),
    "hknp1dAngularFollowCamCinfo::CameraSet::CameraSet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknp1dAngularFollowCamCinfo::CameraSet *
        ]
    ),
    "hknp1dAngularFollowCamCinfo::hknp1dAngularFollowCamCinfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknp1dAngularFollowCamCinfo *
        ]
    ),
    "hkArrayBase<hknpConstraintGroupDirectSolver::BlockWFIInfo>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpConstraintGroupDirectSolver::OffDiagonalBlockInfoInput> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpMultiFrameWorldSnapshot::hknpMultiFrameWorldSnapshot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMultiFrameWorldSnapshot *
        ]
    ),
    "hknpMultiFrameWorldSnapshot::~hknpMultiFrameWorldSnapshot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMultiFrameWorldSnapshot *
        ]
    ),
    "hknpWorldSnapshot::~hknpWorldSnapshot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldSnapshot *
        ]
    ),
    "hkArrayBase<hknpShape::SdfContactPoint>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpShape::SdfContactPoint> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpShapeUtil::buildAabbMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape::MassConfig *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "hknpShapeUtil::buildBoxMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape::MassConfig *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_ulonglong,  # hkQuaternionf *
        ]
    ),
    "hknpShapeUtil::buildCapsuleMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape::MassConfig *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpShapeUtil::buildCylinderMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape::MassConfig *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpShapeUtil::buildHullMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape::MassConfig *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # const hknpConvexHull *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpShapeUtil::buildHullMassPropertiesFromVertices": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape::MassConfig *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpShapeUtil::buildPolygonMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape::MassConfig *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpShapeUtil::buildQuadMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape::MassConfig *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpShapeUtil::buildShapeDisplayGeometries": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint32,  # hknpShape::ConvexRadiusDisplayMode
            ctypes.c_ulonglong,  # hkArrayBase<hkDisplayGeometry *> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint32,  # hknpLevelOfDetail
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const float *
        ]
    ),
    "hknpShapeUtil::buildShapeMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpShapeUtil::buildSphereMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape::MassConfig *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpShapeUtil::buildTriangleMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape::MassConfig *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpShapeUtil::calcAabbWithScale": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_uint32,  # const hknpShape::ScaleMode
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpShapeUtil::calcInnerSphereAabb": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkAabb *
        argtypes=[
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # hkSphere *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "hknpShapeUtil::calcInnerSphereConvex": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkArrayView<hkFloat3 const > *
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayView<hkFloat3 const > *
            ctypes.c_ulonglong,  # hkArrayView<hkVector4f const > *
            ctypes.c_ulonglong,  # hkSphere *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "hknpShapeUtil::calcScalingParameters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # hknpShape::ScaleMode
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpShapeUtil::createConvexHullEdges": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<hkReflect::Type const *,hkReflect::Type *> > *
        ]
    ),
    "hknpShapeUtil::createConvexHullGeometry": {
        "const hknpConvexHull *, float, __int64, const hkVector4f *, const hkVector4f *, hkGeometry *, hkGeometry *, int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # const hknpConvexHull *
            argtypes=[
                ctypes.c_ulonglong,  # const hknpConvexHull *
                ctypes.c_float,  # float
                ctypes.c_int64,  # __int64
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_ulonglong,  # hkGeometry *
                ctypes.c_ulonglong,  # hkGeometry *
                ctypes.c_int32,  # int
            ]
        ),
        "const hkFloat3 *, unsigned __int64 *, float, hknpShape::ConvexRadiusDisplayMode, const hkVector4f *, const hkVector4f *, hkGeometry *, hkGeometry *, int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hkFloat3 *
            argtypes=[
                ctypes.c_ulonglong,  # const hkFloat3 *
                ctypes.c_ulonglong,  # unsigned __int64 *
                ctypes.c_float,  # float
                ctypes.c_uint32,  # hknpShape::ConvexRadiusDisplayMode
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_ulonglong,  # hkGeometry *
                ctypes.c_ulonglong,  # hkGeometry *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "getEdgeSdfContactPoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::SdfContactPoint *
            ctypes.c_ulonglong,  # const hkcdVertex *
            ctypes.c_ulonglong,  # const hknpShape::GetSignedDistanceInput *
            ctypes.c_ulonglong,  # hkInplaceArray<hknpShape::SdfContactPoint,32,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpShapeUtil::getSignedDistanceContactsImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpShape::GetSignedDistanceInput *
            ctypes.c_ulonglong,  # hknpShape::SdfContactPointOutput *
        ]
    ),
    "hknpShapeUtil::queryPointUsingSphereShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpForwardClosestHitCollector::addHit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpForwardClosestHitCollector *
            ctypes.c_ulonglong,  # const hknpCollisionResult *
        ]
    ),
    "hknpShapeNullFunctions::buildMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpShapeNullFunctions::buildSurfaceGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCircularSymmetryAxis *
        ]
    ),
    "hknpShapeNullFunctions::calcAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpShapeNullFunctions::calcBoundingRadius": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpShapeNullFunctions::calcBoundingSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpShapeNullFunctions::calcInnerRadius": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpShapeNullFunctions::calcInnerSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkSphere *
        ]
    ),
    "hknpShapeNullFunctions::calcMinAngleBetweenFaces": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # _DWORD *
        ]
    ),
    "hknpShapeNullFunctions::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
        ]
    ),
    "hknpShapeKeyMaskNullFunctions::commitChanges": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeKeyMask *
        ]
    ),
    "hknpShapeNullFunctions::convertVertexIdsToVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpShapeNullFunctions::estimateNumShapeKeys": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpShapeNullFunctions::getConvexVertices": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpConvexVertexShape *
            ctypes.c_ulonglong,  # hkArrayView<hkFloat3> *
        ]
    ),
    "hknpShapeNullFunctions::getFaceInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hknpAngularTimType *
        ]
    ),
    "hknpShapeNullFunctions::getFaceVertices": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpShapeNullFunctions::getLeafShapes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpShapeCollector *
        ]
    ),
    "hknpShapeNullFunctions::getMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpShapeNullFunctions::getNumberOfFaces": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpShapeNullFunctions::getNumberOfSupportSpheres": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpShapeNullFunctions::getSignedDistanceContacts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpShape::GetSignedDistanceInput *
            ctypes.c_ulonglong,  # hknpShape::SdfContactPointOutput *
        ]
    ),
    "hknpShapeNullFunctions::getSignedDistances": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::SdfQuery *
            ctypes.c_ulonglong,  # hknpShape::SdfContactPoint *
        ]
    ),
    "hknpShapeNullFunctions::getSupportSpheres": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkFloat3 *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkFloat3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpShapeNullFunctions::getSupportingFace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkcdGsk::Cache *
        ]
    ),
    "hknpShapeNullFunctions::getSupportingVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpShapeKeyMaskNullFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
            ctypes.c_ulonglong,  # const hknpShapeKeyMask *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpShapeNullFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpShapeNullFunctions::makeShapeKeyMask": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpShapeNullFunctions::onCoreShapeMutated": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShape *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "hknpShapeNullFunctions::onFinishEditingMask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShape *
        ]
    ),
    "hknpShapeNullFunctions::queryAabb": {
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ]
        ),
    },
    "hknpShapeKeyMaskNullFunctions::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeKeyMask *
        ]
    ),
    "hknpMeshShapeFunctions::buildSurfaceGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::BuildSurfaceGeometryConfig *
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<hkReflect::Type const *,hkReflect::Type *> > *
        ]
    ),
    "hknpMeshShapeFunctions::calcAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpMeshShapeFunctions::getLeafShapes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpShapeCollector *
        ]
    ),
    "hknpMeshShapeFunctions::getMemoryFootprint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpShapeSet *
        ]
    ),
    "hknpMeshShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hknpMeshShapeFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hkcdSimdTree::aabbCast<ShapeCastSelectSectionQuery>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdRay *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # ShapeCastSelectSectionQuery *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTreeUtils::aabbOverlaps_Trampoline<0,AabbSelectSectionQuery<hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdSimdTree::TransposedAabb *
            ctypes.c_ulonglong,  # AabbSelectSectionQuery<hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTreeUtils::aabbOverlaps_Trampoline<0,AabbSelectSectionQuery<hknpCollisionQueryCollector>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdSimdTree::TransposedAabb *
            ctypes.c_ulonglong,  # AabbSelectSectionQuery<hknpCollisionQueryCollector> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromAabb<GetClosestPointsToConvexSelectQuery>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # GetClosestPointsToConvexSelectQuery *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPoint<QueryPointsSelectSectionQuery>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # QueryPointsSelectSectionQuery *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hknpCollisionResult::BodyInfo::set<hknpRayCastQuery,hknpShapeQueryInfo>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionResult::BodyInfo *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
        ]
    ),
    "hknpMeshShapeFunctions::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpMeshShape::castShapeImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpMeshShape::getClosestPointsToConvex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpMeshShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "RayCastSelectSectionQuery<hkAabb8TreeUtils::RayCastDescriptorSSE,1>::isCollisionEnabled": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # RayCastSelectSectionQuery<hkAabb8TreeUtils::RayCastDescriptorSSE,0> *
            ctypes.c_ulonglong,  # hknpTriangleShape *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_ulonglong,  # hknpQueryFilterData *
            ctypes.c_int32,  # int
        ]
    ),
    "GetClosestPointsToConvexSelectQuery::isCollisionEnabled": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # GetClosestPointsToConvexSelectQuery *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_ulonglong,  # hknpQueryFilterData *
        ]
    ),
    "QueryPointsSelectSectionQuery::isCollisionEnabled": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # QueryPointsSelectSectionQuery *
            ctypes.c_ulonglong,  # const hknpTriangleShape *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_ulonglong,  # hknpQueryFilterData *
            ctypes.c_int32,  # int
        ]
    ),
    "ShapeCastSelectSectionQuery::isCollisionEnabled": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # ShapeCastSelectSectionQuery *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_ulonglong,  # const hknpTriangleShape *
            ctypes.c_ulonglong,  # hknpQueryFilterData *
        ]
    ),
    "hknpMeshShapeFunctions::queryAabb": {
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hknpCollisionQueryCollector *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
    },
    "hknpMeshShapeFunctions::queryPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpHeightFieldShape_queryAabb_hkArray_hkHandle_unsigned_int_4294967295_hknpShapeKeyDiscriminant__hkBuiltinContainerAllocator_0_0____0_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hknpHeightFieldShape *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpHeightFieldShape_queryAabb_hknpCollisionQueryCollector_1_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hknpHeightFieldShape *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpHeightFieldShape_queryAabb_hknpCollisionQueryCollector_0_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hknpHeightFieldShape *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpFlippedGetClosestPointsQueryCollector::addHit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFlippedGetClosestPointsQueryCollector *
            ctypes.c_ulonglong,  # const hknpCollisionResult *
        ]
    ),
    "hknpHeightFieldShapeFunctions::buildSurfaceGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::BuildSurfaceGeometryConfig *
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<hkReflect::Type const *,hkReflect::Type *> > *
        ]
    ),
    "hknpHeightFieldShape::calcClosestDistanceSquaredLowerEstimate": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpHeightFieldShape *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpHeightFieldShape::calcClosestDistanceSquaredLowerEstimateWithScale": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpHeightFieldShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpHeightFieldShapeFunctions::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpHeightFieldShapeFunctions::castShapeImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpHeightFieldShapeFunctions::getClosestPointsImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpHeightFieldShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpHeightFieldShapeFunctions::getClosestPointsToHeightfieldImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpHeightFieldShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpHeightFieldShapeFunctions::getLeafShapes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpShapeCollector *
        ]
    ),
    "hknpHeightFieldShapeFunctions::getSignedDistances": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::SdfQuery *
            ctypes.c_ulonglong,  # hknpShape::SdfContactPoint *
        ]
    ),
    "hknpHeightFieldShape_calcClosestDistanceSquaredLowerEstimate": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const hknpHeightFieldShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpHeightFieldShape_castShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkcdRay *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkcdRay *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hknpHeightFieldShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpHeightFieldShape_getClosestPoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hknpHeightFieldShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpHeightFieldShape_getClosestPointsToHeightfield": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hknpHeightFieldShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpHeightFieldShapeFunctions::queryAabb": {
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hknpCollisionQueryCollector *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
    },
    "hknpFlippedGetClosestPointsQueryCollector::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFlippedGetClosestPointsQueryCollector *
        ]
    ),
    "hknpMaskedCompoundShape::hknpMaskedCompoundShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMaskedCompoundShape *
            ctypes.c_ulonglong,  # const hknpMaskedCompoundShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpMaskedCompoundShapeFunctions::buildMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpMaskedCompoundShapeFunctions::calcAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpMaskedCompoundShapeFunctions::getTypeName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[]
    ),
    "hknpMaskedCompoundShapeFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape> *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint32,  # hknpShape::CloneMode
            ctypes.c_ulonglong,  # hknpShapeMap *
        ]
    ),
    "hknpMaskedCompoundShapeFunctions::onCoreShapeMutated": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShape *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "hknpMaskedCompoundShape::onCoreShapeMutatedImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMaskedCompoundShape *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "hknpMaskedCompoundShapeFunctions::onFinishEditingMask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShape *
        ]
    ),
    "hknpMaskedCompoundShape::onFinishEditingMaskImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMaskedCompoundShape *
        ]
    ),
    "hknpShapeConnectivityGraph::getMemoryFootPrint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShapePropertyBase *
        ]
    ),
    "hknpCompoundShapeFunctions_castShapeImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpShapeCastFuncs::flip<&hknpShapeCastFuncs::castShapeFunc<hknpCompoundShapeFunctions>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpShapeCastFuncs::flip<&hknpCompressedMeshShapeFunctions::castShapeImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpShapeCastFuncs::flip<&hknpExternMeshShapeFunctions::castShapeImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpShapeCastFuncs::flip<&hknpHeightFieldShapeFunctions::castShapeImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpShapeCastFuncs::flip<&hknpMeshShape::castShapeImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpShapeCastFuncs::flip<&hknpParticlesColliderShapeFunctions::castShapeImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::flip<&hknpClosestPointFuncs::compositeVsShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::flip<&hknpClosestPointFuncs::compoundVsShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::flip<&hknpClosestPointFuncs::convexVsCompound": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::flip<&hknpClosestPointFuncs::convexVsCompressedMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::flip<&hknpClosestPointFuncs::convexVsExternMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::flip<&hknpClosestPointFuncs::convexVsHeightField": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::flip<&hknpClosestPointFuncs::convexVsMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::flip<&hknpClosestPointFuncs::convexVsParticleSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::flip<&hknpClosestPointFuncs::externMeshVsShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::flip<&hknpClosestPointFuncs::heightFieldVsCompressedMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::flip<&hknpClosestPointFuncs::particlesVsShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpFlippedShapeCastQueryCollector::addHit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFlippedShapeCastQueryCollector *
            ctypes.c_ulonglong,  # const hknpCollisionResult *
        ]
    ),
    "hknpClosestPointFuncs::compositeVsShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::compoundVsCompound": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::compoundVsShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::compressedMeshVsCompressedMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::convexVsCompound": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::convexVsCompressedMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::convexVsConvex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::convexVsExternMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::convexVsHeightField": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::convexVsMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::convexVsParticleSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::externMeshVsShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::heightFieldVsCompressedMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::heightFieldVsHeightfield": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpClosestPointFuncs::particlesVsShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpFlippedShapeCastQueryCollector::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFlippedShapeCastQueryCollector *
        ]
    ),
    "hknpSpatialQueryUtil::calcQueryToTargetTransformWithScale": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hkTransformf *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpWorld::addCollisionCaches": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpBodyIndexPair *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # hknpWorldWriter::ActivationMode
        ]
    ),
    "hknpThreadSafeQueue<hknpBodyIndex,256,hknpBodyIndex::QueueOperations>::addItem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpThreadSafeQueue<hknpBodyIndex,256,hknpBodyIndex::QueueOperations>::ItemStorage *
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeQueue<hknpBodyIndex,256,hknpBodyIndex::QueueOperations> *
            ctypes.c_ulonglong,  # hknpThreadSafeQueue<hknpBodyIndex,256,hknpBodyIndex::QueueOperations>::ItemStorage *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
        ]
    ),
    "hk::swap<hknpSimpleSimplexSolver::Contact>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimpleSimplexSolver::Contact *
            ctypes.c_ulonglong,  # hknpSimpleSimplexSolver::Contact *
        ]
    ),
    "hknpDeflectedLinearCast::CheckStartCollector::addHit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeflectedLinearCast::CheckStartCollector *
            ctypes.c_ulonglong,  # const hknpCollisionResult *
        ]
    ),
    "hknpDeflectedLinearCast::FilteredQueryCollector::addHit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeflectedLinearCast::FilteredQueryCollector *
            ctypes.c_ulonglong,  # const hknpCollisionResult *
        ]
    ),
    "hknpDeflectedLinearCast::QueryFilter::getLodInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpLodCollisionInfo *
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeflectedLinearCast::QueryFilter *
            ctypes.c_ulonglong,  # hknpLodCollisionInfo *
            ctypes.c_ulonglong,  # const hknpBody *
            ctypes.c_ulonglong,  # const hknpBody *
        ]
    ),
    "hknpDeflectedLinearCast::QueryFilter::isCollisionEnabled": {
        "hknpDeflectedLinearCast::QueryFilter *, __int64, const hknpQueryFilterData *, const hknpBody *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # hknpDeflectedLinearCast::QueryFilter *
                ctypes.c_int64,  # __int64
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpBody *
            ]
        ),
        "hknpParticlesCollisionFilter *, __int64, __int64, unsigned __int64": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # hknpParticlesCollisionFilter *
                ctypes.c_int64,  # __int64
                ctypes.c_int64,  # __int64
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
        "hknpDeflectedLinearCast::QueryFilter *, __int64, _BOOL8, const hknpCollisionFilter::FilterInput *, const hknpCollisionFilter::FilterInput *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # hknpDeflectedLinearCast::QueryFilter *
                ctypes.c_int64,  # __int64
                ctypes.c_ubyte,  # _BOOL8
                ctypes.c_ulonglong,  # const hknpCollisionFilter::FilterInput *
                ctypes.c_ulonglong,  # const hknpCollisionFilter::FilterInput *
            ]
        ),
    },
    "hknpDeflectedLinearCast::linearCast": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpWorldEx *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpDeflectedLinearCast::Input *
            ctypes.c_ulonglong,  # hknpDeflectedLinearCast::Output *
        ]
    ),
    "hknpContactSolverEvent::calculateManifold": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpContactSolverEvent *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkcdManifold4 *
        ]
    ),
    "hknpContactSolverEvent::calculateToi": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSimdFloat32 *
        argtypes=[
            ctypes.c_ulonglong,  # hknpContactSolverEvent *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hknpConstraintForceEvent::printCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintForceEvent *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpConstraintForceExceededEvent::printCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintForceExceededEvent *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpContactImpulseClippedEvent::printCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpContactImpulseClippedEvent *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpContactImpulseEvent::printCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpContactImpulseEvent *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpDeflectedLinearCastCollisionEvent::printCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeflectedLinearCastCollisionEvent *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpFullCastCollisionEvent::printCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFullCastCollisionEvent *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpLinearIntegrationClippedEvent::printCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLinearIntegrationClippedEvent *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpClosestHitCollector::addHit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpClosestHitCollector *
            ctypes.c_ulonglong,  # const hknpCollisionResult *
        ]
    ),
    "hkArrayBase__hknpWorldEx__updateMotionAndAttachedBodiesAfterModifyingTransform____2___BodyAabbAndId____growCapacity_hkBuiltinContainerAllocatorBase_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpParticleLinearInfo>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpParticleLinearInfo> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpSolverVelocity>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpConstraintGroupDirectSolver::BodyPositionInformation> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpThreadSafeObjectPoolRange<hknpBodyId>::hknpThreadSafeObjectPoolRange<hknpBodyId>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPoolRange<hknpBodyId> *
            ctypes.c_ulonglong,  # const hknpThreadSafeObjectPool<hknpBody> *
        ]
    ),
    "hknpThreadSafeObjectPoolRange<hknpConstraintId>::hknpThreadSafeObjectPoolRange<hknpConstraintId>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPoolRange<hknpConstraintId> *
            ctypes.c_ulonglong,  # const hknpThreadSafeObjectPool<hknpConstraint> *
        ]
    ),
    "hknpThreadSafeObjectPoolRange<hknpConstraintGroup const &>::hknpThreadSafeObjectPoolRange<hknpConstraintGroup const &>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPoolRange<hkHandle<unsigned int,2147483647,hknpConstraintGroupIdDiscriminant> > *
            ctypes.c_ulonglong,  # const hknpThreadSafeObjectPool<hknpConstraintGroup> *
        ]
    ),
    "hknpWorld::hknpWorld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpWorldCinfo *
        ]
    ),
    "hknpWorldSignals::hknpWorldSignals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldSignals *
        ]
    ),
    "hknpCollisionCacheStream::~hknpCollisionCacheStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCdPairStream *
        ]
    ),
    "hknpLockedBodyIdArrayView::~hknpLockedBodyIdArrayView": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLockedBodyIdArrayView *
        ]
    ),
    "hknpTimerVolumeTags::~hknpTimerVolumeTags": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpTimerVolumeTags *
        ]
    ),
    "hknpWorld::~hknpWorld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorldSignals::~hknpWorldSignals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldSignals *
        ]
    ),
    "hknpWorld::accessSolverData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpSolverData *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorld::addBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpBodyId *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # hknpWorldWriter::AdditionMode
            ctypes.c_uint32,  # hknpWorldWriter::ActivationMode
        ]
    ),
    "hknpWorld::allocateConstraintGroup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHandle<unsigned int,2147483647,hknpConstraintGroupIdDiscriminant> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hkHandle<unsigned int,2147483647,hknpConstraintGroupIdDiscriminant> *
            ctypes.c_ulonglong,  # const hknpConstraintGroupCinfo *
        ]
    ),
    "hknpWorld::allocateMaterial": {
        "hknpWorld *, hknpMaterialId *, const hknpMaterialDescriptor *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hknpMaterialId *
            argtypes=[
                ctypes.c_ulonglong,  # hknpWorld *
                ctypes.c_ulonglong,  # hknpMaterialId *
                ctypes.c_ulonglong,  # const hknpMaterialDescriptor *
            ]
        ),
        "hknpWorld *, hknpMaterialId *, const hknpMaterial *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hknpMaterialId *
            argtypes=[
                ctypes.c_ulonglong,  # hknpWorld *
                ctypes.c_ulonglong,  # hknpMaterialId *
                ctypes.c_ulonglong,  # const hknpMaterial *
            ]
        ),
    },
    "hknpWorld::allocateMotion": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpMotionId *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hknpMotionId *
            ctypes.c_ulonglong,  # const hknpMotionCinfo *
        ]
    ),
    "hknpWorld::allocateMotionProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpMotionPropertiesId *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hknpMotionPropertiesId *
            ctypes.c_ulonglong,  # const hknpMotionProperties *
        ]
    ),
    "hknpWorld::allocateParticlesCollider": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpParticlesColliderId *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hknpParticlesColliderId *
            ctypes.c_ulonglong,  # const hknpParticlesColliderCinfo *
        ]
    ),
    "hkSignal2<hknpEventHandlerInput const &,hknpEvent const &>::MemberSlot<hknpWorldEx,void ": {
        "hkSignal2<hknpWorld *,hknpConstraintManager *>::MemberSlot<hknpWorldEx,void (__cdecl hknpWorldEx::*)(hknpWorld *,hknpConstraintManager *)> *, hknpWorld *, hknpConstraintManager *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hknpConstraintManager *>::MemberSlot<hknpWorldEx,void (__cdecl hknpWorldEx::*)(hknpWorld *,hknpConstraintManager *)> *
                ctypes.c_ulonglong,  # hknpWorld *
                ctypes.c_ulonglong,  # hknpConstraintManager *
            ]
        ),
        "hkSignal2<hknpWorld *,hknpConstraintManager *>::MemberSlot<hknpWorldEx,void (__cdecl hknpWorldEx::*)(hknpWorld *,hknpConstraintManager *)> *, const void *, int": FUNCDEF(
            restype=ctypes.c_ubyte,  # _BOOL8
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hknpConstraintManager *>::MemberSlot<hknpWorldEx,void (__cdecl hknpWorldEx::*)(hknpWorld *,hknpConstraintManager *)> *
                ctypes.c_ulonglong,  # const void *
                ctypes.c_int32,  # int
            ]
        ),
        "hkSignal2<hknpEventHandlerInput const &,hknpEvent const &>::MemberSlot<hknpWorldEx,void (__cdecl hknpWorldEx::*)(hknpEventHandlerInput const &,hknpEvent const &)> *, hkSignal2<hknpEventHandlerInput const &,hknpEvent const &> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal2<hknpEventHandlerInput const &,hknpEvent const &>::MemberSlot<hknpWorldEx,void (__cdecl hknpWorldEx::*)(hknpEventHandlerInput const &,hknpEvent const &)> *
                ctypes.c_ulonglong,  # hkSignal2<hknpEventHandlerInput const &,hknpEvent const &> *
            ]
        ),
    },
    "hknpThreadSafeQueue<hknpBodyIndex,256,hknpBodyIndex::QueueOperations>::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeQueue<hknpBodyIndex,256,hknpBodyIndex::QueueOperations> *
        ]
    ),
    "hknpTaskGraph::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpTaskGraph *
        ]
    ),
    "hknpWorldEx::commitBroadphaseAdditions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_uint32,  # hknpBroadPhase::AdditionModeHint
        ]
    ),
    "hknpWorldEx::constraintCollisionFlagsChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_ulonglong,  # hknpConstraint *
            ctypes.c_uint32,  # hknpWorldWriter::ActivationMode
        ]
    ),
    "hknpWorldEx::defaultBodyExitedSafeRegionBehavior": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_ulonglong,  # const hknpEventHandlerInput *
            ctypes.c_ulonglong,  # const hknpEvent *
        ]
    ),
    "hknpWorld::destroyBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpBodyId *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # hknpWorldWriter::ActivationMode
        ]
    ),
    "hknpWorld::destroyConstraints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpConstraintId *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # hknpWorldWriter::ActivationMode
        ]
    ),
    "hknpWorld::destroyMaterials": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpMaterialId *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpWorld::destroyMotionProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpMotionPropertiesId *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpWorld::destroyMotions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpMotionId *
            ctypes.c_ulonglong,  # hkBaseSystem::ThreadContextImpl *
        ]
    ),
    "hknpWorld::detachBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpBodyId *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # hknpWorldWriter::UpdateMotionMode
        ]
    ),
    "hknpWorld::executeUserCommandImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpUserCommand *
        ]
    ),
    "hkSignal2<hknpWorld *,hknpMotionManager *>::fire": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hkVector4f const &> *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkSignal3<hknpWorld *,hknpBodyId const *,int>::fire": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal3<hknpWorld *,hknpBodyId const *,int> *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpBodyId *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpWorld::generateCollideTasks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpStepInput *
            ctypes.c_ulonglong,  # hknpTaskGraph *
        ]
    ),
    "hknpWorldEx::generateSolveTasks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_ulonglong,  # hknpTaskGraph *
            ctypes.c_ulonglong,  # hkTaskQueue *
        ]
    ),
    "hknpWorld::getActionManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorld::getActiveBodies": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpLockedBodyIdArrayView *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hknpLockedBodyIdArrayView *
        ]
    ),
    "hknpWorld::getAirDensity": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorld::getBodies": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpThreadSafeObjectPoolRange<hknpBodyId> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPoolRange<hknpBodyId> *
        ]
    ),
    "hknpWorld::getCharacterProxyManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorld::getCollisionQueryDispatcher": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpCollisionQueryDispatcher *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorld::getConstraintCapacity": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorld::getConstraintGroupCapacity": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorld::getConstraintGroupIds": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpThreadSafeObjectPoolRange<hknpConstraintGroup const &> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPoolRange<hknpConstraintGroup const &> *
        ]
    ),
    "hknpWorld::getConstraints": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpThreadSafeObjectPoolRange<hknpConstraint const &> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPoolRange<hknpConstraint const &> *
        ]
    ),
    "hknpWorld::getGravity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkArray<hkVdbCmdTag,hkBuiltinContainerAllocator<3,0> > *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hkSignal2<hknpMaterialLibrary *,uint>::MemberSlot<hknpWorldEx,void ": {
        "hkSignal2<hknpWorld *,hknpConstraintManager *>::MemberSlot<hknpWorldEx,void (__cdecl hknpWorldEx::*)(hknpWorld *,hknpConstraintManager *)> *, const void **, int *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hknpConstraintManager *>::MemberSlot<hknpWorldEx,void (__cdecl hknpWorldEx::*)(hknpWorld *,hknpConstraintManager *)> *
                ctypes.c_ulonglong,  # const void **
                ctypes.c_ulonglong,  # int *
            ]
        ),
        "hkSignal2<hknpMaterialLibrary *,unsigned int>::MemberSlot<hknpWorldEx,void (__cdecl hknpWorldEx::*)(hknpMaterialLibrary *,unsigned int)> *, hkSignal2<hknpMaterialLibrary *,unsigned int> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkSignal2<hknpMaterialLibrary *,unsigned int>::MemberSlot<hknpWorldEx,void (__cdecl hknpWorldEx::*)(hknpMaterialLibrary *,unsigned int)> *
                ctypes.c_ulonglong,  # hkSignal2<hknpMaterialLibrary *,unsigned int> *
            ]
        ),
    },
    "hknpWorld::getMotionPropertiesLibrary": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpMotionPropertiesLibrary *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorld::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorld::getNumConstraintGroups": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorld::getNumConstraints": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorld::getNumParticlesColliders": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorld::getParticlesColliderCapacity": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorld::getSolverInfoPtr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpCollisionQueryDispatcher> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorldEx::onBodyBufferChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hknpBodyManager *
        ]
    ),
    "hknpWorldEx::onConstraintBufferChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hknpConstraintManager *
        ]
    ),
    "hknpWorldEx::onMaterialCapacityChangedSignal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_ulonglong,  # hknpMaterialLibrary *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hknpWorldEx::postCollide": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
        ]
    ),
    "hknpWorldEx::postSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
        ]
    ),
    "hknpWorldEx::preCollide": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_ulonglong,  # const hknpStepInput *
        ]
    ),
    "hknpWorldEx::preSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
        ]
    ),
    "hknpWorld::removeBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpBodyId *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # hknpWorldWriter::ActivationMode
        ]
    ),
    "hknpWorldEx::removeBodyFromMotion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_ulonglong,  # hknpBody *
        ]
    ),
    "hknpWorld::setAirDensity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_float,  # float
        ]
    ),
    "hknpWorld::setCollisionFilter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hknpCollisionFilter *
        ]
    ),
    "hknpWorld::setEventDispatcher": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hknpEventDispatcher *
        ]
    ),
    "hknpWorld::setGravity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpWorld::setName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hknpPropertyManager::setProperty": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # hknpPropertyManager *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpWorld::setSafeRegionAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # hknpWorldWriter::SetSafeRegionAabbResult *
        ]
    ),
    "hkBaseSystem::Detail::setTlsData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpWorld::shiftWorld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpWorld::stepCollide": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpStepInput *
            ctypes.c_ulonglong,  # hkTaskQueue *
        ]
    ),
    "hknpWorld::stepSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hkTaskQueue *
        ]
    ),
    "hkSignal2<hknpBodyQualityLibrary *,hknpBodyQualityId>::MemberSlot<hknpWorldEx,void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpBodyQualityLibrary *,hknpBodyQualityId>::MemberSlot<hknpWorldEx,void (__cdecl hknpWorldEx::*)(hknpBodyQualityLibrary *,hknpBodyQualityId)> *
            ctypes.c_ulonglong,  # hkSignal2<hknpBodyQualityLibrary *,hknpBodyQualityId> *
        ]
    ),
    "hkSignal2<hknpMaterialLibrary *,hknpMaterialId>::MemberSlot<hknpWorldEx,void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpMaterialLibrary *,hknpMaterialId>::MemberSlot<hknpWorldEx,void (__cdecl hknpWorldEx::*)(hknpMaterialLibrary *,hknpMaterialId)> *
            ctypes.c_ulonglong,  # hkSignal2<hknpMaterialLibrary *,hknpMaterialId> *
        ]
    ),
    "hkSignal3<hknpMotionPropertiesLibrary *,hknpMotionPropertiesId,hknpMotionProperties const &>::MemberSlot<hknpWorldEx,void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal3<hknpMotionPropertiesLibrary *,hknpMotionPropertiesId,hknpMotionProperties const &>::MemberSlot<hknpWorldEx,void (__cdecl hknpWorldEx::*)(hknpMotionPropertiesLibrary *,hknpMotionPropertiesId,hknpMotionProperties const &)> *
            ctypes.c_ulonglong,  # hkSignal3<hknpMotionPropertiesLibrary *,hknpMotionPropertiesId,hknpMotionProperties const &> *
        ]
    ),
    "hkSignal2<hknpWorld *,hknpBodyManager *>::MemberSlot<hknpWorldEx,void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hknpBodyManager *>::MemberSlot<hknpWorldEx,void (__cdecl hknpWorldEx::*)(hknpWorld *,hknpBodyManager *)> *
            ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hknpBodyManager *> *
        ]
    ),
    "hkSignal2<hknpWorld *,hknpConstraintManager *>::MemberSlot<hknpWorldEx,void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hknpConstraintManager *>::MemberSlot<hknpWorldEx,void (__cdecl hknpWorldEx::*)(hknpWorld *,hknpConstraintManager *)> *
            ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hknpConstraintManager *> *
        ]
    ),
    "hknpWorldEx::synchronizeConstraintBodyIds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_ulonglong,  # hknpConstraint *
        ]
    ),
    "hknpWorldEx::unregisterBodyAtActiveList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_ulonglong,  # hknpBody *
        ]
    ),
    "hknpWorld::updateBroadPhase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpWorldEx::updateMotionAndAttachedBodiesAfterModifyingTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_ulonglong,  # hknpBody *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_uint32,  # hknpWorldWriter::PivotLocation
            ctypes.c_uint32,  # hknpWorldWriter::ActivationMode
        ]
    ),
    "hknpWorldEx::updateMotionMaxRotationPerStep": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_ulonglong,  # hknpMotion *
        ]
    ),
    "hknpVehicleWheelCollide::WheelHitCollector::WheelHitCollector": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleWheelCollide::WheelHitCollector *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
        ]
    ),
    "hknpVehicleWheelCollide::WheelHitCollector::addHit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVehicleWheelCollide::WheelHitCollector *
            ctypes.c_ulonglong,  # const hknpCollisionResult *
        ]
    ),
    "hknpConstraintDataUtils::getPivotIntern<hknpConstraintData const>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const __m128 *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpConstraintDataUtils::getConstraintPivots": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpConstraintData *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "hknpConstraintDataUtils::getSolverImpulsesInSolverSpaceFromAtom": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpConstraintAtom *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintAtom *
            ctypes.c_ulonglong,  # const hknpSolverResults *
            ctypes.c_ulonglong,  # hknpConstraintData::AppliedImpulses *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # _BYTE *
        ]
    ),
    "hknpConstraintDataUtils::isSupportedByDirectSolver": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hknpConstraintDataUtils::transformPivot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_uint32,  # hknpConstraintDataUtils::Pivot
        ]
    ),
    "hkSingletonUtil__init_hknpConstraintDataFunctionsSingleton_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintDataFunctionsSingleton> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintDataFunctionsSingleton> *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hknpConstraintDataFunctions::hknpConstraintDataFunctions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintDataFunctions *
        ]
    ),
    "hknp6DofConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpAngularFrictionConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpBallAndSocketConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpBreakableConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpCogWheelConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpDeformableFixedConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpFixedConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpHingeConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpLimitedHingeConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpLinearClearanceConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpMalleableConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpPointToPathConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpPointToPlaneConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpPrismaticConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpPulleyConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpRackAndPinionConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpRagdollConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpRotationalConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpStiffSpringConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpWheelConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpWheelFrictionConstraintDataFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpConstraintDataFunctions::registerDefaults": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintDataFunctions *
        ]
    ),
    "hknpMotionUtil::buildSolverVelocities": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hkArray<hknpMotionId,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpMotionUtil::calcFutureAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpBody *
            ctypes.c_ulonglong,  # const hknpMotion *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpMotionUtil::calcMotionMaxRotationPerStep": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const hknpMotion *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # const hknpBodyQualityLibrary *
        ]
    ),
    "hknpMotionUtil::gatherSolverVelocities": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpMotion *
            ctypes.c_ulonglong,  # const hknpMotionId *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpSolverVelocity *
            ctypes.c_ulonglong,  # hknpSolverSumVelocity *
        ]
    ),
    "hkBaseSystem::Detail::getTlsData": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "hknpMotionUtil::integrateMotionTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpMotion *
        ]
    ),
    "hknpMotionUtil::updateBodyAfterMotionIntegration": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpBodyQuality *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # hknpBody *
            ctypes.c_ulonglong,  # const hknpMotion *
        ]
    ),
    "hknpMotionUtil::updateBodyAfterMotionIntegrationInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpBodyQuality *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # hknpBody *
            ctypes.c_ulonglong,  # const hknpMotion *
        ]
    ),
    "hknpMotionUtil::updateBodyTransformFromMotion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBody *
            ctypes.c_ulonglong,  # const hknpMotion *
        ]
    ),
    "hknpMotionUtil::updateSweptBodyAabbWithMargin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBody *
            ctypes.c_ulonglong,  # const hknpMotion *
            ctypes.c_ulonglong,  # const hknpBodyQuality *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_uint32,  # hknpMotionUtil::AabbUpdateMode
            ctypes.c_uint32,  # hknpMotionUtil::AabbSweepMode
        ]
    ),
    "hkCalculateFrictionalEllipsoidScale": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpVehicleFrictionDescription::AxisDescription *
            ctypes.c_float,  # float
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpVehicleFrictionSolver_buildJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpVelAccumulatorJacobianBuilding *
            ctypes.c_ulonglong,  # const hknpVelAccumulatorJacobianBuilding *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hknp1Lin2AngJacobian *
        ]
    ),
    "hknpVehicleFrictionApplyVehicleFriction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpVehicleStepInfo *
            ctypes.c_ulonglong,  # const hknpVehicleFrictionDescription *
            ctypes.c_ulonglong,  # hknpVehicleFrictionSolverParams *
            ctypes.c_ulonglong,  # hknpVehicleFrictionStatus *
        ]
    ),
    "hknpVehicleFrictionDescriptionInitValues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpVehicleFrictionDescription::Cinfo *
            ctypes.c_ulonglong,  # hknpVehicleFrictionDescription *
        ]
    ),
    "hkArrayBase<hkAabbFloat3>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkAabbFloat3> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayUtil::reduce<hknpConvexHull::Edge>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpBodyManager::hknpBodyManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyManager *
        ]
    ),
    "hknpThreadSafeObjectPool<hknpBody>::~hknpThreadSafeObjectPool<hknpBody>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpBody> *
        ]
    ),
    "hknpBodyManager::~hknpBodyManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyManager *
        ]
    ),
    "hknpBodyManager::allocateBody": {
        "hknpBodyManager *, _DWORD *, unsigned int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # _DWORD *
            argtypes=[
                ctypes.c_ulonglong,  # hknpBodyManager *
                ctypes.c_ulonglong,  # _DWORD *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "hknpBodyManager *, hknpBodyIndex *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # hknpBodyIndex *
            argtypes=[
                ctypes.c_ulonglong,  # hknpBodyManager *
                ctypes.c_ulonglong,  # hknpBodyIndex *
            ]
        ),
    },
    "hknpBodyManager::buildMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpMassDistribution *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hknpBodyManager::initialize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyManager *
            ctypes.c_ulonglong,  # const hknpBodyManagerCinfo *
        ]
    ),
    "hknpThreadSafeObjectPool<hknpBody>::rebuildFreeList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpBody> *
        ]
    ),
    "hknpBodyManager::relocateBodyBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpBody *
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyManager *
            ctypes.c_ulonglong,  # hknpBody *
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPoolElement<hknpBody> *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpThreadSafeObjectPool<hknpBody>::relocateBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpBody *
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpBody> *
            ctypes.c_ulonglong,  # hknpBody *
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPoolElement<hknpBody> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hknpBodyManager::setBodyDragProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyManager *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const hknpDragProperties *
        ]
    ),
    "hknpBodyManager::setBodyMassDistribution": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyManager *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const hknpMassDistribution *
        ]
    ),
    "hknpBodyManager::updatePreviousAabbsOfActiveBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyManager *
        ]
    ),
    "hkRelArrayView<uchar,int>::operator<uchar const> hkArrayView<uchar const>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkArrayView<unsigned char const > *
        argtypes=[
            ctypes.c_ulonglong,  # hkRelArrayView<unsigned char,int> *
            ctypes.c_ulonglong,  # hkArrayView<unsigned char const > *
        ]
    ),
    "hkRelArrayView<hknpConvexHull::Face,int>::operator<hknpConvexHull::Face const> hkArrayView<hknpConvexHull::Face const>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkArrayView<hknpConvexHull::Face const > *
        argtypes=[
            ctypes.c_ulonglong,  # hkRelArrayView<hknpConvexHull::Face,int> *
            ctypes.c_ulonglong,  # hkArrayView<hknpConvexHull::Face const > *
        ]
    ),
    "hkRelArrayView<hkVector4f,int>::operator<hkVector4f const> hkArrayView<hkVector4f const>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkArrayView<hkVector4f const > *
        argtypes=[
            ctypes.c_ulonglong,  # hkRelArrayView<hkVector4f,int> *
            ctypes.c_ulonglong,  # hkArrayView<hkVector4f const > *
        ]
    ),
    "hkArrayBase__anonymous_namespace___HalfEdge____growCapacity_hkBuiltinContainerAllocatorBase_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpConvexHull::calcAabb": {
        "hknpConvexHull *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpConvexHull *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "hknpConvexHull *, const hkTransformf *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpConvexHull *
                ctypes.c_ulonglong,  # const hkTransformf *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
    },
    "hknpConvexHull::calcBoundingRadius": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexHull *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpConvexHull::calcConnectivity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkArrayView<hkFloat3 const > *
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayView<hkFloat3 const > *
            ctypes.c_ulonglong,  # hkArrayView<hknpConvexHull::Face const > *
            ctypes.c_ulonglong,  # hkArrayView<unsigned char const > *
            ctypes.c_ulonglong,  # hkArray<hknpConvexHull::Edge,hkBuiltinContainerAllocator<2,0> > *
            ctypes.c_ulonglong,  # hkArray<hknpConvexHull::Edge,hkBuiltinContainerAllocator<2,0> > *
            ctypes.c_ulonglong,  # hkArrayBase<hknpSimdTreeBroadPhase::PointId> *
        ]
    ),
    "hknpConvexHull::calcFaceSupportHalfAngle": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayView<hknpConvexHull::Face const > *
            ctypes.c_ulonglong,  # hkArrayView<unsigned char const > *
            ctypes.c_ulonglong,  # hkArrayView<hkFloat3 const > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpConvexHull::calcMinAngleBetweenFaces": FUNCDEF(
        restype=ctypes.c_ulonglong,  # float *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexHull *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpConvexHull::convertVertexIdsToVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexHull *
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpConvexHull::getDimensions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexHull *
        ]
    ),
    "hknpConvexHull::getFaceVertices": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexHull *
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpConvexHull::getSupportingVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexHull *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hkSingletonUtil__init_hknpGlobals_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpGlobals> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpGlobals> *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hknpGlobals::hknpGlobals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGlobals *
        ]
    ),
    "hkSignal0::fire": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal0 *
        ]
    ),
    "hkParallel::IteratorContext::~IteratorContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkParallel::IteratorContext *
        ]
    ),
    "hkArrayBase<hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::Codec32>>::SAHTask>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::Codec32> >::SAHTask> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::Codec32>>::SAHBin>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::Codec32> >::SAHBin> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::Codec32>>::copyTo<hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::Codec32>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::Codec32> > *
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::Codec32> > *
        ]
    ),
    "hkMeshBoneIndexMapping::~hkMeshBoneIndexMapping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMeshBoneIndexMapping *
        ]
    ),
    "hkArrayBase<ushort>::_ensureOwnsStorage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<unsigned short> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hknpCompoundShape::buildBoundingVolume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShape *
            ctypes.c_uint32,  # hknpCompoundShapeBoundingVolumeType::Enum
        ]
    ),
    "hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::Codec32>>::compactIndices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::Codec32> > *
        ]
    ),
    "hknpCompoundShapeEx::MassProvider::getMass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::MassProvider *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hknpShape::MassConfig *
            ctypes.c_ulonglong,  # hkMassElement *
        ]
    ),
    "hknpCompoundShapeEx::getMemoryFootprintImpl": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx *
            ctypes.c_ulonglong,  # hknpShapeSet *
        ]
    ),
    "hknpCompoundShapeEx::MassProvider::getNumMasses": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShapeEx::MassProvider *
        ]
    ),
    "hknpCompoundShape::rebuild": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShape *
        ]
    ),
    "hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::Codec32>>::rebuildBranchSAH": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::Codec32> > *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdDynamicTree::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::Codec32>::reserveNodes": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::Codec32> *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpCompoundShape::updateCachedBounds": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShape *
        ]
    ),
    "hknpCompoundShape::updateInstancesInBoundingVolume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompoundShape *
            ctypes.c_ulonglong,  # const hkHandle<unsigned short,65535,hknpShapeInstanceIdDiscriminant> *
            ctypes.c_int32,  # int
        ]
    ),
    "hkArrayBase<hknpCollisionResult>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpCollisionResult> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpAllHitsCollector::addHit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpAllHitsCollector *
            ctypes.c_ulonglong,  # const hknpCollisionResult *
        ]
    ),
    "hknpAllHitsCollector::sortHits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpAllHitsCollector *
        ]
    ),
    "hknpShapeTransform::transformVertices<hkFloat3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeTransform *
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_ulonglong,  # hkFloat3 *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpShapeTransform::transformVertices<hkVector4f>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeTransform *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdSimdTreeUtils::aabbOverlaps_Trampoline<0,hknpParticlesColliderQueryHelperStructs::hknpParticlesAabbOverlapsQuery>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdSimdTree::TransposedAabb *
            ctypes.c_ulonglong,  # hknpParticlesColliderQueryHelperStructs::hknpParticlesAabbOverlapsQuery *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPoint<hknpParticlesColliderQueryHelperStructs::hknpParticlesPointQuery>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpParticlesColliderQueryHelperStructs::hknpParticlesPointQuery *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hknpParticlesCollider::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesCollider *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpParticlesCollider::queryAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesCollider *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
            ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpParticlesCollider::queryPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesCollider *
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hkArrayBase<hkAabb>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkAabb> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpParticleEarlyOutCache>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpParticleEarlyOutCache> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpPartitionTreeBuildContext::hknpPartitionTreeBuildContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPartitionTreeBuildContext *
        ]
    ),
    "hkcdSimdTree::TaskBuildContext::~TaskBuildContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree::TaskBuildContext *
        ]
    ),
    "hknpParticleParticleCollider::~hknpParticleParticleCollider": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleParticleCollider *
        ]
    ),
    "hknpParticlesCollider::~hknpParticlesCollider": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesCollider *
        ]
    ),
    "hknpParticlesTasks::~hknpParticlesTasks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesTasks *
        ]
    ),
    "hkArrayBase<hkcdSimdTree::Point>::_ensureOwnsStorage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkcdSimdTree::Point> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hknpParticlesCollider::addParticles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesCollider *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpParticlesCollider::disableParticles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesCollider *
            ctypes.c_ulonglong,  # const hkArrayView<int const > *
        ]
    ),
    "hknpParticlesCollider::enableParticles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesCollider *
            ctypes.c_ulonglong,  # const hkArrayView<int const > *
        ]
    ),
    "hknpParticlesColliderEx::getDomainAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesColliderEx *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpParticlesCollider::getParticlesOutsideTheSafeRegion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesCollider *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # hkArray<int,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpParticlesColliderEx::getPartitionTree": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkcdSimdTree *
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesColliderEx *
        ]
    ),
    "hknpParticlesCollider::rebuildParticleCollisionCaches": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesCollider *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hknpParticlesCollider::removeParticles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesCollider *
            ctypes.c_ulonglong,  # const hkArrayView<int const > *
            ctypes.c_ulonglong,  # hkArray<hknpParticleCacheMoveInfo,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpParticlesCollider::shiftParticleCaches": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesCollider *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpConstraintMotor::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintMotor> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintMotor> *
            ctypes.c_ulonglong,  # const hknpConstraintMotor *
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknp1dAngularBilateralConstraintBuildJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknp1dAngularBilateralConstraintInfo *
            ctypes.c_ulonglong,  # const hknpConstraintQueryIn *
            ctypes.c_ulonglong,  # hknpConstraintQueryOut *
        ]
    ),
    "hknp1dAngularBilateralConstraintUserTauBuildJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknp1dAngularBilateralUserTauConstraintInfo *
            ctypes.c_ulonglong,  # const hknpConstraintQueryIn *
            ctypes.c_ulonglong,  # hknpConstraintQueryOut *
        ]
    ),
    "hknp1dLinearBilateralConstraintBuildJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknp1dLinearBilateralConstraintInfo *
            ctypes.c_ulonglong,  # const hknpConstraintQueryIn *
            ctypes.c_ulonglong,  # hknpConstraintQueryOut *
        ]
    ),
    "hknp1dLinearBilateralConstraintUserTauBuildJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknp1dLinearBilateralUserTauConstraintInfo *
            ctypes.c_ulonglong,  # const hknpConstraintQueryIn *
            ctypes.c_ulonglong,  # hknpConstraintQueryOut *
        ]
    ),
    "hknp1dLinearFrictionBuildJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknp1dLinearFrictionInfo *
            ctypes.c_ulonglong,  # const hknpConstraintQueryIn *
            ctypes.c_ulonglong,  # hknpConstraintQueryOut *
        ]
    ),
    "hknp1dLinearLimitBuildJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknp1dLinearLimitInfo *
            ctypes.c_ulonglong,  # const hknpConstraintQueryIn *
            ctypes.c_ulonglong,  # hknpConstraintQueryOut *
        ]
    ),
    "hknpSetInvMassBuildJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hknpConstraintQueryOut *
        ]
    ),
    "hkArrayBase<hkSurfaceConstraintInfo>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkSurfaceConstraintInfo> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpCharacterProxy::Contact>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpCharacterProxy::Contact> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpCharacterProxyInternals::addMaxSlopePlane": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals *
            ctypes.c_ulonglong,  # hkBool *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkArray<hkSurfaceConstraintInfo,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_float,  # float
        ]
    ),
    "hknpCharacterProxyInternals::compareContacts": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals *
            ctypes.c_ulonglong,  # const hknpCharacterProxy::Contact *
            ctypes.c_ulonglong,  # const hknpCharacterProxy::Contact *
        ]
    ),
    "hknpCharacterProxyInternals::createConstraintsFromManifold": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals *
            ctypes.c_ulonglong,  # const hkStepInfo *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkArray<hkSurfaceConstraintInfo,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpCharacterProxyInternals::createSurfaceConstraint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals *
            ctypes.c_ulonglong,  # const hknpCharacterProxy::Contact *
            ctypes.c_ulonglong,  # const hkStepInfo *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkSurfaceConstraintInfo *
        ]
    ),
    "hknpCharacterProxyInternals::findContact": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals *
            ctypes.c_ulonglong,  # const hknpCharacterProxy::Contact *
            ctypes.c_ulonglong,  # const hkArrayBase<hknpCharacterProxy::Contact> *
            ctypes.c_float,  # float
        ]
    ),
    "hknpCharacterProxy::refreshManifold": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxy *
            ctypes.c_ulonglong,  # hknpAllHitsCollector *
        ]
    ),
    "hknpCharacterProxyInternals::updateManifold": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals *
            ctypes.c_ulonglong,  # const hknpAllHitsCollector *
            ctypes.c_ulonglong,  # const hknpAllHitsCollector *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpCharacterProxyInternals::validateManifold": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals *
        ]
    ),
    "hknpCharacterProxyInternals::fireCharacterInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals *
            ctypes.c_ulonglong,  # hknpCharacterProxy *
            ctypes.c_ulonglong,  # const hkContactPoint *
        ]
    ),
    "hknpCharacterProxyInternals::fireConstraintsProcessed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals *
            ctypes.c_ulonglong,  # const hkArray<hknpCharacterProxy::Contact,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkSimplexSolverInput *
        ]
    ),
    "hknpCharacterProxyInternals::fireContactAdded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals *
            ctypes.c_ulonglong,  # const hknpCharacterProxy::Contact *
        ]
    ),
    "hknpCharacterProxyInternals::fireContactRemoved": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals *
            ctypes.c_ulonglong,  # const hknpCharacterProxy::Contact *
        ]
    ),
    "hknpCharacterProxyInternals::fireObjectInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals *
            ctypes.c_ulonglong,  # const hknpCharacterObjectInteractionEvent *
            ctypes.c_ulonglong,  # hknpCharacterObjectInteractionResult *
        ]
    ),
    "hknpCharacterProxyInternals::fireShapeChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpCharacterProxyInternals::fireTriggerVolumeInteraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyInternals *
            ctypes.c_ulonglong,  # const hknpCharacterProxy::TriggerInteraction *
            ctypes.c_uint32,  # hknpTriggerEvent::Status
        ]
    ),
    "hknpCharacterProxy::onPostSolveSignal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxy *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hkSignal2<hknpWorld *,hkVector4f const &>::subscribe<hknpCharacterProxyManager,void ": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hkVector4f const &>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hkVector4f const &> *
            ctypes.c_ulonglong,  # hknpCharacterProxyManager *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpCharacterProxyManager*', 'name': 'this'}, 'hknpWorld *', 'const hkVector4f *']}
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hknpCharacterProxyManager::hknpCharacterProxyManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyManager *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpCharacterProxyManager::~hknpCharacterProxyManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyManager *
        ]
    ),
    "hknpCharacterProxyManager::addCharacterProxy": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpCharacterProxy *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyManager *
            ctypes.c_ulonglong,  # hknpCharacterProxy *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpCharacterProxyManager::characterProxyDestroyed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpCharacterProxy *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyManager *
            ctypes.c_ulonglong,  # hknpCharacterProxy *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpCharacterProxyManager::onWorldShifted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCharacterProxyManager *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkSignal2<hknpWorld *,hkVector4f const &>::MemberSlot<hknpCharacterProxyManager,void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hkVector4f const &>::MemberSlot<hknpCharacterProxyManager,void (__cdecl hknpCharacterProxyManager::*)(hknpWorld *,hkVector4f const &)> *
            ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hkVector4f const &> *
        ]
    ),
    "hknpMassDistribution::setFromShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMassDistribution *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
        ]
    ),
    "hknpMassDistribution::setFromShapes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMassDistribution *
            ctypes.c_ulonglong,  # const hknpShape **
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpManifold::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpManifold *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_double,  # double
        ]
    ),
    "hknpContactJacobianUtil::accessPointEffectiveMass": FUNCDEF(
        restype=ctypes.c_ulonglong,  # float *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpContactJacobianUtil::accessPointRhs": FUNCDEF(
        restype=ctypes.c_ulonglong,  # float *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpContactJacobianUtil::calculateManifold": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpMotion *
            ctypes.c_uint32,  # hknpContactJacobianUtil::BackstepMode
            ctypes.c_ulonglong,  # hkcdManifold4 *
        ]
    ),
    "hknpContactJacobianUtil::disableContacts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
        ]
    ),
    "hknpContactJacobianUtil::getBodyIdA": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpBodyId *
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyId *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
        ]
    ),
    "hknpContactJacobianUtil::getBodyIdB": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpBodyId *
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyId *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
        ]
    ),
    "hknpContactJacobianUtil::getBodyIndexA": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpBodyIndex *
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyIndex *
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
        ]
    ),
    "hknpContactJacobianUtil::getBodyIndexB": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpBodyIndex *
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyIndex *
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
        ]
    ),
    "hknpContactJacobianUtil::getManifoldType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
        ]
    ),
    "hknpContactJacobianUtil::getNumContactPoints": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
        ]
    ),
    "hknpContactJacobianUtil::getShapeKeyA": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
        argtypes=[
            ctypes.c_ulonglong,  # hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
        ]
    ),
    "hknpContactJacobianUtil::getShapeKeyB": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
        argtypes=[
            ctypes.c_ulonglong,  # hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
        ]
    ),
    "hknpContactJacobianUtil::scaleEffectiveMass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_float,  # float
        ]
    ),
    "hknpContactJacobianUtil::scaleFriction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "hknpContactJacobianUtil::scalePenetrations": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_float,  # float
        ]
    ),
    "CAkState::InitNotificationSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkState *
            ctypes.c_ulonglong,  # CAkStateAware *
        ]
    ),
    "hknpContactJacobianUtil::setMaxImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # hknpImpulseClipMode::Enum
        ]
    ),
    "hknpContactJacobianUtil::setSurfaceVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkArrayBase<hknpSolverData::ThreadData>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpSolverData::ThreadData> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpSolverData::SimpleMaterial::SimpleMaterial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverData::SimpleMaterial *
        ]
    ),
    "hknpSingleThreadedSolverData::hknpSingleThreadedSolverData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSingleThreadedSolverData *
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
        ]
    ),
    "hknpSolverData::hknpSolverData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverData *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hknpGrid<hknpCsJacRange>::~hknpGrid<hknpCsJacRange>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGrid<hknpCsJacRange> *
        ]
    ),
    "hknpSolverData::ThreadData::~ThreadData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverData::ThreadData *
        ]
    ),
    "hknpCsGrid::~hknpCsGrid": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCsGrid *
        ]
    ),
    "hknpSolverData::~hknpSolverData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverData *
        ]
    ),
    "hknpSolverData::addImmediateContactConstraintUsingJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverData *
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
        ]
    ),
    "hknpSolverData::beginStep": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverData *
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpSolverData::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverData *
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
        ]
    ),
    "hknpSolverData::endStep": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverData *
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
        ]
    ),
    "hknpManifoldEvent::getNumContactPoints": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
        argtypes=[
            ctypes.c_ulonglong,  # hknpManifoldEvent *
        ]
    ),
    "hknpManifoldEvent::printCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpManifoldEvent *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpParticleBodyImpulseAppliedEvent::printCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleBodyImpulseAppliedEvent *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpParticleParticleImpulseAppliedEvent::printCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleParticleImpulseAppliedEvent *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpParticlesCollidedWithBodiesEvent::printCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesCollidedWithBodiesEvent *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpParticlesCollidedWithParticlesEvent::printCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesCollidedWithParticlesEvent *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpParticlesExitedSafeRegionEvent::printCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesExitedSafeRegionEvent *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpReserved0Event::printCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpReserved1Event *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpTriggerEvent::printCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpTriggerEvent *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpConstraintManager::hknpConstraintManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintManager *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hknpConstraint *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # hknpConstraintGroup *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hknpThreadSafeObjectPool<hknpConstraint>::~hknpThreadSafeObjectPool<hknpConstraint>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpConstraint> *
        ]
    ),
    "hknpThreadSafeObjectPool<hknpConstraintGroup>::~hknpThreadSafeObjectPool<hknpConstraintGroup>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpConstraintGroup> *
        ]
    ),
    "hknpConstraintManager::~hknpConstraintManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintManager *
        ]
    ),
    "hknpConstraintManager::allocateConstraintGroup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHandle<unsigned int,2147483647,hknpConstraintGroupIdDiscriminant> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintManager *
            ctypes.c_ulonglong,  # hkHandle<unsigned int,2147483647,hknpConstraintGroupIdDiscriminant> *
            ctypes.c_ulonglong,  # const hknpConstraintGroupCinfo *
        ]
    ),
    "hknpConstraintManager::checkGroupMotionCells": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # const hknpConstraintGroup *
        ]
    ),
    "hknpConstraintManager::forceGroupToOneMotionCell": {
        "hknpConstraintManager *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpConstraintManager *
            ]
        ),
        "hknpWorld *, const hknpConstraintGroup *, unsigned __int8": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpWorld *
                ctypes.c_ulonglong,  # const hknpConstraintGroup *
                ctypes.c_uint8,  # unsigned __int8
            ]
        ),
    },
    "hknpThreadSafeObjectPool<hknpConstraint>::rebuildFreeList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpConstraint> *
        ]
    ),
    "hknpThreadSafeObjectPool<hknpConstraintGroup>::rebuildFreeList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpConstraintGroup> *
        ]
    ),
    "hknpThreadSafeObjectPool<hknpConstraint>::relocateBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpConstraint *
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpConstraint> *
            ctypes.c_ulonglong,  # hknpConstraint *
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPoolElement<hknpConstraint> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hknpThreadSafeObjectPool<hknpConstraintGroup>::relocateBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpConstraintGroup *
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpConstraintGroup> *
            ctypes.c_ulonglong,  # hknpConstraintGroup *
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPoolElement<hknpConstraintGroup> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hknpConstraintManager::relocateConstraintBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpConstraint *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintManager *
            ctypes.c_ulonglong,  # hknpConstraint *
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPoolElement<hknpConstraint> *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpConstraintManager::removeFromGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintManager *
            ctypes.c_ulonglong,  # hknpConstraint *
        ]
    ),
    "hkArrayBase<hkScopedPtr<hknpParticleShapeProperties>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkScopedPtr<hknpParticleShapeProperties> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpInplaceTriangleShape::hknpInplaceTriangleShape": {
        "hknpInplaceTriangleShape *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpInplaceTriangleShape *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "hknpInplaceTriangleShape *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpInplaceTriangleShape *
            ]
        ),
    },
    "hknpShapeCollector::getTriangleShape": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpTriangleShape *
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeCollector *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpTriangleShape *
        ]
    ),
    "hknpShapeQueryInterface::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpShapeQueryInterface::castShape": {
        "hknpCollisionQueryContext *, const hknpShapeCastQuery *, const hknpShapeQueryInfo *, const hknpShape *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hknpCollisionQueryCollector *, hknpCollisionQueryCollector *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpShapeCastQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
                ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ]
        ),
        "hknpCollisionQueryContext *, const hknpShapeCastQuery *, const hkRotationImpl<float> *, const hknpShape *, const hkTransformf *, hknpCollisionQueryCollector *, hknpCollisionQueryCollector *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpShapeCastQuery *
                ctypes.c_ulonglong,  # const hkRotationImpl<float> *
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # const hkTransformf *
                ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
                ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ]
        ),
    },
    "hknpShapeQueryInterface::getClosestPoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpMeshShapeEx::getNextMeshKey": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpMeshShapeEx *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hknpMeshShapeEx::getShapeTagForPrimitiveKey": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpMeshShapeEx *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hknpConvexShapeUtil::getSupportingFace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkcdSimdTree::closestFromAabb<hknpExternMeshShapeInternals::GetClosestPointsToConvexQuery>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpExternMeshShapeInternals::GetClosestPointsToConvexQuery *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromAabbOrdered<hknpExternMeshShapeInternals::GetClosestPointsToConvexQuery>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpExternMeshShapeInternals::GetClosestPointsToConvexQuery *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb6BytesCodec>,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromAabbWrapper<hknpExternMeshShapeInternals::GetClosestPointsToConvexQuery>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb6BytesCodec> *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromAabbWrapper<hknpExternMeshShapeInternals::GetClosestPointsToConvexQuery> *
        ]
    ),
    "hknpExternMeshShapeUtil::getClosestPointsToConvex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpExternMeshShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpExternMeshShapeInternals::GetClosestPointsToConvexQuery::processLeaf": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpExternMeshShapeInternals::GetClosestPointsToConvexQuery *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpShapeCastFuncs::flip<&hknpShapeCastFuncs::castConvexConvexCalculateToi": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCollisionResult::BodyInfo::setNoLod<hknpShapeQueryInfo>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionResult::BodyInfo *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
        ]
    ),
    "hknpShapeCastFuncs::castBoxQuad": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpShapeCastFuncs::castConvexConvexCalculateToi": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpShapeCastFuncs::castConvexConvexConservativeAdvancement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpShapeCastFuncs::castConvexConvexGsk": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpShapeCastFuncs::castSphereQuad": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hkArrayBase<hknpParticleIndicesArray>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpParticleIndicesArray> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpParticleIndicesArray::hknpParticleIndicesArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleIndicesArray *
            ctypes.c_ulonglong,  # hknpParticleIndicesArray *
        ]
    ),
    "hknpParticlesColliderManager::hknpParticlesColliderManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesColliderManager *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hknpThreadSafeObjectPool<hknpParticlesColliderManager::ParticlesColliderContainer>::~hknpThreadSafeObjectPool<hknpParticlesColliderManager::ParticlesColliderContainer>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpParticlesColliderManager::ParticlesColliderContainer> *
        ]
    ),
    "hknpParticlesColliderManager::~hknpParticlesColliderManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesColliderManager *
        ]
    ),
    "hknpParticlesColliderManager::allocateParticlesCollider": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpParticlesColliderId *
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesColliderManager *
            ctypes.c_ulonglong,  # hknpParticlesColliderId *
            ctypes.c_ulonglong,  # const hknpParticlesColliderCinfo *
        ]
    ),
    "hknpParticlesColliderManager::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesColliderManager *
        ]
    ),
    "PostStepTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # PostStepTask *
        ]
    ),
    "PreStepTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # PreStepTask *
        ]
    ),
    "hknpParticlesColliderManager::postStepParticles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesColliderManager *
        ]
    ),
    "PostStepTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PostStepTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "PreStepTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PreStepTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpParticlesColliderManager::queryParticlesOutsideTheAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesColliderManager *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # hkArray<hknpParticleIndicesArray,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpThreadSafeObjectPool<hknpParticlesColliderManager::ParticlesColliderContainer>::relocateBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpParticlesColliderManager::ParticlesColliderContainer *
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpParticlesColliderManager::ParticlesColliderContainer> *
            ctypes.c_ulonglong,  # hknpParticlesColliderManager::ParticlesColliderContainer *
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPoolElement<hknpParticlesColliderManager::ParticlesColliderContainer> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "shapeContainsMesh": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpParticlesColliderManager::shiftParticleCaches": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesColliderManager *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpParticlesColliderManager::updateParticleCachesOnBodyChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesColliderManager *
            ctypes.c_ulonglong,  # const hknpBody *
            ctypes.c_ulonglong,  # const hkAabbFloat3 *
            ctypes.c_ulonglong,  # const hkAabbFloat3 *
        ]
    ),
    "hkArrayBase<hkTuple<int,int,int,void,void,void,void,void>>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkTuple<int,int,int,void,void,void,void,void> > *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpProcessNarrowPhaseGridTask::GridRangeSubTask>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpSortFinalizeNarrowPhasePairsTask::sortRange> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpBodyActivationCommand>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpBodyActivationCommand> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpDeactivationThreadData>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpDeactivationThreadData> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpDeactivationManager::hknpDeactivationManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationManager *
        ]
    ),
    "hknpDeactivationStepInfo::hknpDeactivationStepInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationStepInfo *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hknpDeactivationManager::~hknpDeactivationManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationManager *
        ]
    ),
    "hknpDeactivationStepInfo::~hknpDeactivationStepInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationStepInfo *
        ]
    ),
    "hknpDeactivationStepInfo::addActiveBodyPairs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationStepInfo *
            ctypes.c_ulonglong,  # const hknpCdPairStream *
            ctypes.c_ulonglong,  # const hknpGrid<hknpIdxRange> *
            ctypes.c_ulonglong,  # const hknpMotionManager *
        ]
    ),
    "hknpDeactivationStepInfo::addActiveBodyPairsBegin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationStepInfo *
        ]
    ),
    "hknpDeactivationStepInfo::addActiveBodyPairsEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationStepInfo *
        ]
    ),
    "hknpDeactivationManager::addDeactivatedCaches": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationManager *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hknpCollisionCacheStreams *
        ]
    ),
    "hknpDeactivationStepInfo::combineActivityBitFields": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationStepInfo *
            ctypes.c_ulonglong,  # hkBitField *
        ]
    ),
    "hknpDeactivationManager::deactivateBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationManager *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hknpDeactivationStepInfo *
            ctypes.c_ulonglong,  # const hkBitField *
            ctypes.c_ulonglong,  # const hknpGrid<hknpIdxRange> *
            ctypes.c_ulonglong,  # const hkArray<hknpMotionId,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkMonitorStream *
        ]
    ),
    "hknpDeactivationManager::defragmentInactiveStreamsIncremental": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationManager *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpDeactivationManager::flushActivationEvents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationManager *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
        ]
    ),
    "hknpDeactivationStepInfo::getThreadData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpDeactivationThreadData *
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationStepInfo *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpDeactivationManager::mergeIslands": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationManager *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hknpDeactivationManager::moveActivatedCaches": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationManager *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hknpCollisionCacheStreams *
            ctypes.c_ulonglong,  # hkTypedBlockStream<hknpBodyIndexPair> *
            ctypes.c_ulonglong,  # hkTypedBlockStream<hknpCollisionCacheWorkItem>::Writer *
            ctypes.c_ulonglong,  # hknpStreamWorkItemWriter *
            ctypes.c_ulonglong,  # hknpConstraintAtomSolverSetup::ConstraintState *
            ctypes.c_ulonglong,  # hknpConstraintAtomSolverSetup::ConstraintStateGroups *
        ]
    ),
    "hknpDeactivationManager::postCollideMaintenance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationManager *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # hknpCollisionCacheStream *
        ]
    ),
    "hknpDeactivationManager::postSolveMaintenance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationManager *
        ]
    ),
    "hkSet<int,hkBuiltinContainerAllocator<2,0>,hkMapOperations<int>>::resizeTable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkSet<int,hkBuiltinContainerAllocator<2,0>,hkMapOperations<int> > *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpDeactivationManager::splitActivateIsland": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationManager *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
            ctypes.c_ulonglong,  # hknpCollisionCacheBatchConsumer *
            ctypes.c_ulonglong,  # hknpModifierSharedData *
            ctypes.c_ulonglong,  # hkTypedBlockStream<hknpBodyIndexPair>::Writer *
            ctypes.c_ulonglong,  # hkTypedBlockStream<hknpCollisionCacheWorkItem>::Writer *
            ctypes.c_ulonglong,  # hknpStreamWorkItemWriter *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
        ]
    ),
    "hkSignal1<hknpShape const *>::subscribe<hknpShapeManager::MutableShapeInfo,void ": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal1<hknpShape const *>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal1<hknpShape const *> *
            ctypes.c_ulonglong,  # hknpShapeManager::MutableShapeInfo *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpShapeManager::MutableShapeInfo*', 'name': 'this'}, 'const hknpShape *']}
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkSignal2<hknpShape const *,uchar>::subscribe<hknpShapeManager::MutableShapeInfo,void ": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal2<hknpShape const *,unsigned char>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpShape const *,unsigned char> *
            ctypes.c_ulonglong,  # hknpShapeManager::MutableShapeInfo *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpShapeManager::MutableShapeInfo*', 'name': 'this'}, 'const hknpShape *', {'type': 'unsigned', 'name': '__int8'}]}
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hknpShapeManager::hknpShapeManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeManager *
        ]
    ),
    "hknpShapeManager::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeManager *
        ]
    ),
    "hknpShapeManager::deregisterBodyWithMutableShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeManager *
            ctypes.c_ulonglong,  # hknpBody *
        ]
    ),
    "hkSignal5<hknpWorld *,hkRefPtr<hknpShape const> &,uchar,hknpBodyId const *,int>::fire": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal5<hknpWorld *,hkRefPtr<hknpShape const > &,unsigned char,hknpBodyId const *,int> *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hkRefPtr<hknpShape const > *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const hknpBodyId *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpShapeManager::MutableShapeInfo::onShapeDestroyed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeManager::MutableShapeInfo *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpShapeManager::MutableShapeInfo::onShapeMutated": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeManager::MutableShapeInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "hknpShapeManager::processMutatedShapes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeManager *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpShapeManager::registerBodyWithMutableShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpShapeManager *
            ctypes.c_ulonglong,  # hknpBody *
        ]
    ),
    "hkSet<uint,hkBuiltinContainerAllocator<0,0>,hkMapOperations<uint>>::remove": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkSet<int,hkBuiltinContainerAllocator<0,0>,hkMapOperations<int> > *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_int32,  # int
        ]
    ),
    "hkSet<uint,hkBuiltinContainerAllocator<0,0>,hkMapOperations<uint>>::resizeTable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkSet<int,hkBuiltinContainerAllocator<0,0>,hkMapOperations<int> > *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_int32,  # int
        ]
    ),
    "hkSignal1<hknpShape const *>::MemberSlot<hknpShapeManager::MutableShapeInfo,void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal1<hknpShape const *>::MemberSlot<hknpShapeManager::MutableShapeInfo,void (__cdecl hknpShapeManager::MutableShapeInfo::*)(hknpShape const *)> *
            ctypes.c_ulonglong,  # hkSignal1<hknpShape const *> *
        ]
    ),
    "hkSignal2<hknpShape const *,uchar>::MemberSlot<hknpShapeManager::MutableShapeInfo,void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpShape const *,unsigned char>::MemberSlot<hknpShapeManager::MutableShapeInfo,void (__cdecl hknpShapeManager::MutableShapeInfo::*)(hknpShape const *,unsigned char)> *
            ctypes.c_ulonglong,  # hkSignal2<hknpShape const *,unsigned char> *
        ]
    ),
    "hknpWorld::castAabb": {
        "hknpWorld *, const hknpAabbCastQuery *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpWorld *
                ctypes.c_ulonglong,  # const hknpAabbCastQuery *
                ctypes.c_ulonglong,  # hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *
            ]
        ),
        "hknpWorld *, const hknpAabbCastQuery *, hknpBroadPhaseQueryCollector *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpWorld *
                ctypes.c_ulonglong,  # const hknpAabbCastQuery *
                ctypes.c_ulonglong,  # hknpBroadPhaseQueryCollector *
            ]
        ),
    },
    "hknpWorld::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpWorld::castShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hkRotationImpl<float> *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpWorld::getClosestPoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpWorld::queryAabb": {
        "hknpWorld *, const hknpAabbQuery *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpWorld *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *
            ]
        ),
        "hknpWorld *, const hknpAabbQuery *, hknpBroadPhaseQueryCollector *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpWorld *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # hknpBroadPhaseQueryCollector *
            ]
        ),
        "hknpWorld *, const hknpAabbQuery *, hknpCollisionQueryCollector *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpWorld *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ]
        ),
    },
    "hknpWorld::queryAabbNmp": {
        "hknpWorld *, const hknpAabbQuery *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpWorld *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "hknpWorld *, const hknpAabbQuery *, hknpBroadPhaseQueryCollector *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpWorld *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # hknpBroadPhaseQueryCollector *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
    },
    "hknpWorld::queryPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpWorld::setShapeTagCodec": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hknpShapeTagCodec *
        ]
    ),
    "hknpSimpleSimplexSolver::examineActivePlanes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimpleSimplexSolver::Data *
        ]
    ),
    "hknpSimpleSimplexSolver__hkSimpleSimplexSolverSolve2d": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimpleSimplexSolver::Data *
            ctypes.c_ulonglong,  # hknpSimpleSimplexSolver::Contact *
            ctypes.c_ulonglong,  # hknpSimpleSimplexSolver::Contact *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpSimpleSimplexSolver__hkSimpleSimplexSolverSolve3d": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimpleSimplexSolver::Data *
            ctypes.c_ulonglong,  # hknpSimpleSimplexSolver::Contact *
            ctypes.c_ulonglong,  # hknpSimpleSimplexSolver::Contact *
            ctypes.c_ulonglong,  # hknpSimpleSimplexSolver::Contact *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpSimpleSimplexSolver::Contact::setContact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimpleSimplexSolver::Contact *
            ctypes.c_ulonglong,  # const hknpShapeCastQueryResult *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
        ]
    ),
    "hknpSolverUtil::calculateProjectedPointVelocitiesUsingIntegratedVelocities": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkcdManifold4 *
            ctypes.c_ulonglong,  # const hknpMotion *
            ctypes.c_ulonglong,  # const hknpMotion *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpSolverUtil::solveSt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hknpCsGrid **
            ctypes.c_ulonglong,  # hknpSolverVelocity *
            ctypes.c_ulonglong,  # hknpSolverSumVelocity *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpCollisionCacheStreams::append": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionCacheStreams *
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
            ctypes.c_ulonglong,  # hknpCollisionCacheStreams *
        ]
    ),
    "hknpCollisionCacheStreams::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionCacheStreams *
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
        ]
    ),
    "hknpCollisionCacheStreams::clearAndSteal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionCacheStreams *
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
            ctypes.c_ulonglong,  # hknpCollisionCacheStreams *
        ]
    ),
    "hknpCollisionShapeKeyArrayCache::construct": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpCollisionShapeKeyArrayCache *
        argtypes=[
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
        ]
    ),
    "hknpCollisionCacheStreams::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionCacheStreams *
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hknpCollisionCacheStreams::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionCacheStreams *
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
        ]
    ),
    "hknpApiCommandProcessor::exec": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpApiCommandProcessor *
            ctypes.c_ulonglong,  # const hkCommand *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpApiCommandProcessor::print": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpApiCommandProcessor *
            ctypes.c_ulonglong,  # const hkCommand *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpBodyToConstraintsMap::hknpBodyToConstraintsMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyToConstraintsMap *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpBodyToConstraintsMap::~hknpBodyToConstraintsMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyToConstraintsMap *
        ]
    ),
    "hkMapBase<ushort,hknpPropertyManager::PropertyBuffer *,hkMapOperations<ushort>>::_findEmptyPositionInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMapBase<unsigned short,hknpPropertyManager::PropertyBuffer *,hkMapOperations<unsigned short> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_ulonglong,  # const unsigned __int16 *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "hknpPropertyManager::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPropertyManager *
        ]
    ),
    "hknpPropertyManager::clearAllPropertiesFromItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPropertyManager *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkMapBase<ushort,hknpPropertyManager::PropertyBuffer *,hkMapOperations<ushort>>::clearAndDeallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMapBase<unsigned short,hknpPropertyManager::PropertyBuffer *,hkMapOperations<unsigned short> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hknpPropertyManager::clearProperty": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpPropertyManager *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "hknpPropertyManager::PropertyBuffer::construct": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpPropertyManager::PropertyBuffer *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hknpPropertyManager::PropertyBuffer *
        ]
    ),
    "hknpPropertyManager::PropertyBuffer::destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPropertyManager::PropertyBuffer *
        ]
    ),
    "hkMapBase<ushort,hknpPropertyManager::PropertyBuffer *,hkMapOperations<ushort>>::getWithDefault": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpPropertyManager::PropertyBuffer **
        argtypes=[
            ctypes.c_ulonglong,  # hkMapBase<unsigned short,hknpPropertyManager::PropertyBuffer *,hkMapOperations<unsigned short> > *
            ctypes.c_ulonglong,  # const unsigned __int16 *
            ctypes.c_ulonglong,  # hknpPropertyManager::PropertyBuffer *const *
        ]
    ),
    "hkMapBase<ushort,hknpPropertyManager::PropertyBuffer *,hkMapOperations<ushort>>::insert": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMapBase<unsigned short,hknpPropertyManager::PropertyBuffer *,hkMapOperations<unsigned short> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_ulonglong,  # const unsigned __int16 *
            ctypes.c_ulonglong,  # hknpPropertyManager::PropertyBuffer *const *
        ]
    ),
    "hkMapBase<ushort,hknpPropertyManager::PropertyBuffer *,hkMapOperations<ushort>>::resizeTable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkContainerAllocator *
        argtypes=[
            ctypes.c_ulonglong,  # hkMapBase<unsigned short,hknpPropertyManager::PropertyBuffer *,hkMapOperations<unsigned short> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "hkArrayBase<hknpMotionId>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpMotionId> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpMotionManager::hknpMotionManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotionManager *
        ]
    ),
    "hknpThreadSafeObjectPool<hknpMotion>::~hknpThreadSafeObjectPool<hknpMotion>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpMotion> *
        ]
    ),
    "hknpMotionManager::~hknpMotionManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotionManager *
        ]
    ),
    "hknpMotionManager::allocateMotion": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpMotionId *
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotionManager *
            ctypes.c_ulonglong,  # hknpMotionId *
            ctypes.c_ulonglong,  # const hknpMotionCinfo *
        ]
    ),
    "hknpMotionManager::buildCellIndexToSolverIdMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotionManager *
            ctypes.c_ulonglong,  # hknpGrid<hknpIdxRange> *
        ]
    ),
    "hknpMotionManager::buildSolverIdToMotionIdMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotionManager *
            ctypes.c_ulonglong,  # hkArray<hknpMotionId,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpMotionManager::initialize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotionManager *
            ctypes.c_ulonglong,  # hknpMotion *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpBodyManager *
            ctypes.c_ulonglong,  # hknpDeactivationManager *
        ]
    ),
    "hknpThreadSafeObjectPool<hknpMotion>::rebuildFreeList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpMotion> *
        ]
    ),
    "hknpThreadSafeObjectPool<hknpMotion>::relocateBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpMotion *
        argtypes=[
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPool<hknpMotion> *
            ctypes.c_ulonglong,  # hknpMotion *
            ctypes.c_ulonglong,  # hknpThreadSafeObjectPoolElement<hknpMotion> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hknpSolverInfo::hknpSolverInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverInfo *
        ]
    ),
    "hknpSolverInfo::setStepInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverInfo *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hknpSolverInfo::setTauAndDamping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverInfo *
            ctypes.c_double,  # double
            ctypes.c_double,  # double
        ]
    ),
    "hknpSolverVelocityBuffer::CacheAlignedBuffer::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverVelocityBuffer::CacheAlignedBuffer *
        ]
    ),
    "hknpSolverVelocityBuffer::reserve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverVelocityBuffer *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpSolverVelocityBuffer::setSize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverVelocityBuffer *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpSolverVelocityBuffer::CacheAlignedBuffer::setSizeBytes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverVelocityBuffer::CacheAlignedBuffer *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpMotionUtil::applyDrag": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpMotion *
            ctypes.c_ulonglong,  # const hknpDragProperties *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpMotionUtil::predictBodyTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpBody *
            ctypes.c_ulonglong,  # const hknpMotion *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hkPreciseTransform *
        ]
    ),
    "hknpBroadPhase::create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpBroadPhase *
        argtypes=[
            ctypes.c_uint32,  # hknpBroadPhaseType::Enum
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpBroadPhaseConfig *
        ]
    ),
    "hknpBroadPhase::getClosestPointsParticles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBroadPhase *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hknpParticlesStaticCollector *
        ]
    ),
    "hknpCheckDeterminismUtilities::baseQueryFilter<hknpAabbQuery>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkCheckDeterminism::Filter<96> *
        argtypes=[
            ctypes.c_ulonglong,  # hkCheckDeterminism::Filter<96> *
        ]
    ),
    "hknpCheckDeterminismUtilities::checkAabbCastQuery<751829168>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkCheckDeterminismThreadData *
            ctypes.c_ulonglong,  # const hknpAabbCastQuery *
        ]
    ),
    "hknpCheckDeterminismUtilities::checkAabbCastQuery<751829184>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkCheckDeterminismThreadData *
            ctypes.c_ulonglong,  # const hknpAabbCastQuery *
        ]
    ),
    "hknpCheckDeterminismUtilities::checkAabbQuery<751829056>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkCheckDeterminismThreadData *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
        ]
    ),
    "hknpCheckDeterminismUtilities::checkAabbQuery<751829072>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkCheckDeterminismThreadData *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
        ]
    ),
    "hknpCheckDeterminismUtilities::checkAabbQuery<751829073>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkCheckDeterminismThreadData *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
        ]
    ),
    "hknpCheckDeterminismUtilities::checkAabbQuery<751829088>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkCheckDeterminismThreadData *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
        ]
    ),
    "hknpCheckDeterminismUtilities::checkAabbQuery<751829104>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkCheckDeterminismThreadData *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
        ]
    ),
    "hknpCheckDeterminismUtilities::checkAabbQuery<751829120>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkCheckDeterminismThreadData *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
        ]
    ),
    "hknpCheckDeterminismUtilities::checkAabbQuery<751829136>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkCheckDeterminismThreadData *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
        ]
    ),
    "hknpCheckDeterminismUtilities::checkAabbQuery<751829152>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkCheckDeterminismThreadData *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
        ]
    ),
    "hknpCheckDeterminismUtilities::checkClosestPointsQuery<751829040>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkCheckDeterminismThreadData *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
        ]
    ),
    "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase::QueryCheckingBroadPhase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
            ctypes.c_ulonglong,  # const hknpWorld *
        ]
    ),
    "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase::appendHitsImpl": {
        "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *, hkCheckDeterminismThreadData *, const hknpCollisionQueryCollector *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
                ctypes.c_ulonglong,  # hkCheckDeterminismThreadData *
                ctypes.c_ulonglong,  # const hknpCollisionQueryCollector *
            ]
        ),
        "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *, hkCheckDeterminismThreadData *, const hknpParticlesStaticCollector *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
                ctypes.c_ulonglong,  # hkCheckDeterminismThreadData *
                ctypes.c_ulonglong,  # const hknpParticlesStaticCollector *
            ]
        ),
    },
    "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase::castAabb": {
        "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *, const hknpAabbCastQuery *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbCastQuery *
                ctypes.c_ulonglong,  # hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *
            ]
        ),
        "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *, const hknpAabbCastQuery *, hknpBroadPhaseQueryCollector *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbCastQuery *
                ctypes.c_ulonglong,  # hknpBroadPhaseQueryCollector *
            ]
        ),
    },
    "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
        ]
    ),
    "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase::castShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hkRotationImpl<float> *
            ctypes.c_ulonglong,  # const hknpCollisionQueryDispatcher *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
        ]
    ),
    "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase::getClosestPoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hknpCollisionQueryDispatcher *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase::getClosestPointsAdvanced": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase::getClosestPointsParticles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hknpParticlesStaticCollector *
        ]
    ),
    "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase::queryAabb": {
        "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *, const hknpAabbQuery *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *
            ]
        ),
        "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *, const hknpAabbQuery *, hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *
            ]
        ),
        "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *, const hknpAabbQuery *, hknpBroadPhaseQueryCollector *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # hknpBroadPhaseQueryCollector *
            ]
        ),
    },
    "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase::queryAabbNmp": {
        "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *, const hknpAabbQuery *, const hkAabb *, hkAabb *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hkAabb *
                ctypes.c_ulonglong,  # hkAabb *
                ctypes.c_ulonglong,  # hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *
            ]
        ),
        "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *, const hknpAabbQuery *, const hkAabb *, hkAabb *, hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hkAabb *
                ctypes.c_ulonglong,  # hkAabb *
                ctypes.c_ulonglong,  # hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *
            ]
        ),
        "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *, const hknpAabbQuery *, const hkAabb *, hkAabb *, hknpBroadPhaseQueryCollector *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hkAabb *
                ctypes.c_ulonglong,  # hkAabb *
                ctypes.c_ulonglong,  # hknpBroadPhaseQueryCollector *
            ]
        ),
    },
    "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase::queryOutsideOfAabb": {
        "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *, const hknpAabbQuery *, hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *
            ]
        ),
        "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *, const hknpAabbQuery *, hknpBroadPhaseQueryCollector *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # hknpBroadPhaseQueryCollector *
            ]
        ),
    },
    "hknpCheckDeterminismUtilities::QueryCheckingBroadPhase::queryPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCheckDeterminismUtilities::QueryCheckingBroadPhase *
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # const hknpCollisionQueryDispatcher *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hkArrayBase<hknpCollisionCacheRange>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpCollisionCacheRange> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "constructFlipped__hknpConvexCompositeCollisionCache__construct_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpCollisionCache *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCreateCollisionCacheInput *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
        ]
    ),
    "constructFlipped__hknpSignedDistanceFieldCollisionCache__construct_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpCollisionCache *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCreateCollisionCacheInput *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
        ]
    ),
    "hknpCollisionDispatcher::hknpCollisionDispatcher": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionDispatcher *
        ]
    ),
    "hknpCollisionDispatcher::dispatchBodyPairs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionDispatcher *
            ctypes.c_ulonglong,  # const hknpWorldEx *
            ctypes.c_ulonglong,  # const hkBlockStream::Range *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
            ctypes.c_ulonglong,  # hkArray<hknpCollisionCacheRange,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpNullShapeTagCodec::hknpNullShapeTagCodec": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpNullShapeTagCodec *
        ]
    ),
    "hknpNullShapeTagCodec::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShapeTagCodec> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpNullShapeTagCodec *
            ctypes.c_ulonglong,  # hkRefPtr<hknpShapeTagCodec> *
        ]
    ),
    "hknpCollisionCacheManager::hknpCollisionCacheManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionCacheManager *
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpGrid<hkBlockStream::LinkedRange>::~hknpGrid<hkBlockStream::LinkedRange>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGrid<hkBlockStream::LinkedRange> *
        ]
    ),
    "hknpGrid<hkBlockStream::Range>::~hknpGrid<hkBlockStream::Range>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGrid<hkBlockStream::Range> *
        ]
    ),
    "hknpCollisionCacheManager::~hknpCollisionCacheManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionCacheManager *
        ]
    ),
    "hknpCollisionCacheManager::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionCacheManager *
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
        ]
    ),
    "hknpCollisionCacheManager::consumeForcedNewBodyPairs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionCacheManager *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hkTypedBlockStream<hknpBodyIndexPair> *
        ]
    ),
    "hknpCollisionCacheManager::dispatchBodyPairs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionCacheManager *
            ctypes.c_ulonglong,  # const hknpWorldEx *
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
            ctypes.c_ulonglong,  # const hkBlockStream::Range *
            ctypes.c_ulonglong,  # hkArray<hknpCollisionCacheRange,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpCollisionCacheManager::getLastFrameContactData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBlockStream *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionCacheManager *
        ]
    ),
    "hknpBodyQualityLibraryCinfo::hknpBodyQualityLibraryCinfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyQualityLibraryCinfo *
        ]
    ),
    "hknpBodyQualityLibrary::initialize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyQualityLibrary *
            ctypes.c_ulonglong,  # const hknpBodyQualityLibraryCinfo *
        ]
    ),
    "hknpDeactivationState::checkMotionForDeactivation": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationState *
            ctypes.c_ulonglong,  # const hknpMotion *
            ctypes.c_ulonglong,  # const hknpMotionProperties *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpDeactivationState::initDeactivationState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationState *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # hknpActivationControl::Enum
        ]
    ),
    "hknpInternalCollideSharedData::hknpInternalCollideSharedData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpInternalCollideSharedData *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpModifierSharedData::initSt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpModifierSharedData *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpModifierManager::hknpModifierManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpModifierManager *
        ]
    ),
    "hknpModifierManager::~hknpModifierManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpModifierManager *
        ]
    ),
    "hknpModifierManager::setCollisionDetector": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpModifierManager *
            ctypes.c_uint32,  # hknpCollisionCacheType::Enum
            ctypes.c_ulonglong,  # hknpCollisionDetector *
        ]
    ),
    "hknpSimulationThreadContext::hknpSimulationThreadContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpSimulationThreadContext::~hknpSimulationThreadContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
        ]
    ),
    "hknpCommandGrid::dispatchCommands": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCommandGrid *
            ctypes.c_ulonglong,  # hkPrimaryCommandDispatcher *
            ctypes.c_ulonglong,  # hkCheckDeterminismThreadData *
        ]
    ),
    "hknpSimulationThreadContext::finishStep": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
        ]
    ),
    "hknpSimulationThreadContext::initThreadContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hkBlockStreamAllocator *
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_ulonglong,  # hknpCommandGrid *
        ]
    ),
    "hknpSimulationThreadContext::initializeSharedBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
        ]
    ),
    "hknpSimulationThreadContext::resetCommandStreamAndWriter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
        ]
    ),
    "hknpSimulationThreadContext::shutdownThreadContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hkThreadLocalBlockStreamAllocator *
        ]
    ),
    "hknpCsGrid::allocateTemps": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCsGrid *
        ]
    ),
    "hknpCsGrid::freeTemps": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCsGrid *
        ]
    ),
    "hknpSolverStep::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverStep *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hkMonitorStream *
        ]
    ),
    "hkArrayBase<hknpInternalCommandProcessor::EventWithCount<hknpBodyActivationCommand>>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpInternalCommandProcessor::EventWithCount<hknpBodyActivationCommand> > *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpInternalCommandProcessor::exec": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpInternalCommandProcessor *
            ctypes.c_ulonglong,  # const hkCommand *
        ]
    ),
    "hknpInternalCommandProcessor::flushRemainingEvents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpInternalCommandProcessor *
        ]
    ),
    "hknpInternalCommandProcessor::print": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpInternalCommandProcessor *
            ctypes.c_ulonglong,  # const hkCommand *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpUserCommandDispatcher::hknpUserCommandDispatcher": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpUserCommandDispatcher *
        ]
    ),
    "executeLambdaCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpUserCommand *
        ]
    ),
    "hknpUserCommandDispatcher::getFunctionForCmd": FUNCDEF(
        restype=ctypes.c_ulonglong,  # {'return': 'void', 'args': ['hknpWorld *', 'const hknpUserCommand *']}
        argtypes=[
            ctypes.c_ulonglong,  # hknpUserCommandDispatcher *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkArrayBase<hknpEventDispatcher::Entry>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpEventDispatcher::Entry> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpEventDispatcher::Entry>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpEventDispatcher::Entry> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpEventDispatcher::hknpEventDispatcher": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpEventDispatcher *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpEventDispatcher::exec": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpEventDispatcher *
            ctypes.c_ulonglong,  # const hkCommand *
        ]
    ),
    "hknpEventDispatcher::freeEntries": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpEventDispatcher *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "hknpEventDispatcher::getSignal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpEventSignal *
        argtypes=[
            ctypes.c_ulonglong,  # hknpEventDispatcher *
            ctypes.c_uint32,  # hknpEventType::Enum
        ]
    ),
    "hknpEventDispatcher::print": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpEventDispatcher *
            ctypes.c_ulonglong,  # const hkCommand *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hkArrayBase<hknpEventMergeAndDispatcher::EventWithCount<hknpTriggerEvent>>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpConstraintGroupDirectSolver::MotorSolverAtomInfo> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpEventMergeAndDispatcher::hknpEventMergeAndDispatcher": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpEventMergeAndDispatcher *
            ctypes.c_ulonglong,  # hknpWorldEx *
        ]
    ),
    "hknpEventDispatcher::~hknpEventDispatcher": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpEventDispatcher *
        ]
    ),
    "hknpEventMergeAndDispatcher::exec": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpEventMergeAndDispatcher *
            ctypes.c_ulonglong,  # const hkCommand *
        ]
    ),
    "hknpEventMergeAndDispatcher::flushRemainingEvents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpEventMergeAndDispatcher *
        ]
    ),
    "hknpSimulationContext::~hknpSimulationContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationContext *
        ]
    ),
    "hknpSimulationContext::allocateThreadContext": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpSimulationThreadContext *
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationContext *
        ]
    ),
    "hknpSimulationContext::beginStep": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpSimulationContext::dispatchCommandRange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hkPrimaryCommandDispatcher *
            ctypes.c_ulonglong,  # hkCheckDeterminismThreadData *
            ctypes.c_ulonglong,  # const hkBlockStream::LinkedRange *
            ctypes.c_ulonglong,  # hkTypedBlockStream<hkCommand>::Writer *
        ]
    ),
    "hknpSimulationContext::dispatchCommands": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hkPrimaryCommandDispatcher *
        ]
    ),
    "hknpSimulationContext::dispatchHighPriorityCommands": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hkPrimaryCommandDispatcher *
        ]
    ),
    "hknpSimulationContext::dispatchPostCollideCommands": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hknpEventDispatcher *
        ]
    ),
    "hknpSimulationContext::finishStep": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationContext *
        ]
    ),
    "hknpSimulationContext::freeThreadContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
        ]
    ),
    "hknpSimulationContext::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_int32,  # int
        ]
    ),
    "hkArrayBase<hknpSortFinalizeNarrowPhasePairsTask::sortRange>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpConstraintGroupDirectSolver::ConstraintFillInComputations> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkHashSet<hknpBodyIndexPair,hkBuiltinContainerAllocator<0,0>>::tryInsert<hknpBodyIndexPair const &>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHashSet<hknpBodyIndexPair,hkBuiltinContainerAllocator<0,0> >::TryInsertResult *
        argtypes=[
            ctypes.c_ulonglong,  # hkHashSet<hknpBodyIndexPair,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkHashSet<hknpBodyIndexPair,hkBuiltinContainerAllocator<0,0> >::TryInsertResult *
            ctypes.c_ulonglong,  # const hknpBodyIndexPair *
        ]
    ),
    "hknpSingleThreadedSimulation::collide": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSingleThreadedSimulation *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hknpTaskGraph *
        ]
    ),
    "hknpSingleThreadedSimulation::solve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSingleThreadedSimulation *
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hknpTaskGraph *
            ctypes.c_ulonglong,  # hkTaskQueue *
        ]
    ),
    "hkArrayBase<hknpSpaceSplitter::Link>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpSpaceSplitter::Link> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpSpaceSplitter::Link>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpSpaceSplitter::Link> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpDynamicSpaceSplitter::hknpDynamicSpaceSplitter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDynamicSpaceSplitter *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpSingleCellSpaceSplitter::hknpSingleCellSpaceSplitter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSingleCellSpaceSplitter *
        ]
    ),
    "hknpSpaceSplitter::~hknpSpaceSplitter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSpaceSplitter *
        ]
    ),
    "hknpDynamicSpaceSplitter::applyThreadData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDynamicSpaceSplitter *
            ctypes.c_ulonglong,  # const hknpSpaceSplitterData *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpDynamicSpaceSplitter::calcInitialCellPositions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDynamicSpaceSplitter *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpDynamicSpaceSplitter::copyFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDynamicSpaceSplitter *
            ctypes.c_ulonglong,  # const hknpSpaceSplitter *
        ]
    ),
    "hknpSingleCellSpaceSplitter::copyFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSingleCellSpaceSplitter *
            ctypes.c_ulonglong,  # const hknpSpaceSplitter *
        ]
    ),
    "hknpDynamicSpaceSplitter::getCellIdx": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpDynamicSpaceSplitter *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpDynamicSpaceSplitter_updateCenters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDynamicSpaceSplitter *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpSpaceSplitter::initSortedLinks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSpaceSplitter *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpDynamicSpaceSplitter::onShiftWorld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDynamicSpaceSplitter *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpSpaceSplitter::scheduleLinks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hknpSpaceSplitter::Link *
            ctypes.c_ulonglong,  # hknpSpaceSplitter::Link *
        ]
    ),
    "hkBitFieldBase<hkBitFieldInplaceStorage<33,hkBuiltinContainerAllocator<0,0>>>::setSizeAndFill": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkBitFieldBase<hkBitFieldInplaceStorage<33,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkArrayBase<hknpSortFinalizeNarrowPhasePairsTask::ThreadOutput>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpSortFinalizeNarrowPhasePairsTask::ThreadOutput> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpFinalizeNarrowPhasePairsTask::hknpFinalizeNarrowPhasePairsTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFinalizeNarrowPhasePairsTask *
            ctypes.c_ulonglong,  # hknpMultithreadedSimulation *
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hknpSortFinalizeNarrowPhasePairsTask *
            ctypes.c_uint32,  # hknpFinalizeNarrowPhasePairsTask::WorkStage
        ]
    ),
    "hknpMultithreadedSimulation::hknpMultithreadedSimulation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMultithreadedSimulation *
        ]
    ),
    "hknpSortFinalizeNarrowPhasePairsTask::hknpSortFinalizeNarrowPhasePairsTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSortFinalizeNarrowPhasePairsTask *
            ctypes.c_ulonglong,  # hknpMultithreadedSimulation *
            ctypes.c_ulonglong,  # hknpSimulationContext *
        ]
    ),
    "hknpGrid<hknpIdxRange>::~hknpGrid<hknpIdxRange>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGrid<hknpIdxRange> *
        ]
    ),
    "hknpConstraintAtomSolverSetup::SortedConstraintStates::~SortedConstraintStates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintAtomSolverSetup::SortedConstraintStates *
        ]
    ),
    "hknpMultithreadedSimulation::~hknpMultithreadedSimulation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMultithreadedSimulation *
        ]
    ),
    "hknpProcessNarrowPhaseGridTask::~hknpProcessNarrowPhaseGridTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpProcessNarrowPhaseGridTask *
        ]
    ),
    "hknpSortFinalizeNarrowPhasePairsTask::~hknpSortFinalizeNarrowPhasePairsTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSortFinalizeNarrowPhasePairsTask *
        ]
    ),
    "hknpMultithreadedSimulation::checkConsistency": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMultithreadedSimulation *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpMultithreadedSimulation::collide": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMultithreadedSimulation *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hknpTaskGraph *
        ]
    ),
    "hknpMultithreadedSimulation::exportStatisticsToMonitorStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMultithreadedSimulation *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpFinalizeNarrowPhasePairsTask::getDebugAttributes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFinalizeNarrowPhasePairsTask *
            ctypes.c_ulonglong,  # hkTask::DebugAttributes *
        ]
    ),
    "hknpPostSolveTask::getDebugAttributes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPreSolveTask *
            ctypes.c_ulonglong,  # hkTask::DebugAttributes *
        ]
    ),
    "hknpAddActiveBodyPairsTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpAddActiveBodyPairsTask *
        ]
    ),
    "hknpCollideSyncTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollideSyncTask *
        ]
    ),
    "hknpDeactivationMaintenanceTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationMaintenanceTask *
        ]
    ),
    "hknpFinalizeNarrowPhaseGridTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpFinalizeNarrowPhaseGridTask *
        ]
    ),
    "hknpFinalizeNarrowPhasePairsTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpFinalizeNarrowPhasePairsTask *
        ]
    ),
    "hknpGenSolverTasksTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpGenSolverTasksTask *
        ]
    ),
    "hknpPostCollideTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpPostCollideTask *
        ]
    ),
    "hknpPostSolveTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpPostSolveTask *
        ]
    ),
    "hknpPreSolveTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpPreSolveTask *
        ]
    ),
    "hknpProcessFullCastsTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpProcessFullCastsTask *
        ]
    ),
    "hknpProcessNarrowPhaseGridTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpProcessNarrowPhaseGridTask *
        ]
    ),
    "hknpSolveFinalizeTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolveFinalizeTask *
        ]
    ),
    "hknpSolverFactoryTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverFactoryTask *
        ]
    ),
    "hknpSortFinalizeNarrowPhasePairsTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpSortFinalizeNarrowPhasePairsTask *
        ]
    ),
    "hknpProcessFullCastsTask::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpProcessFullCastsTask *
            ctypes.c_ulonglong,  # hknpSimulationContext *
        ]
    ),
    "hknpAddActiveBodyPairsTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpAddActiveBodyPairsTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpCollideSyncTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollideSyncTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpDeactivationMaintenanceTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDeactivationMaintenanceTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpFinalizeNarrowPhaseGridTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFinalizeNarrowPhaseGridTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpGenSolverTasksTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGenSolverTasksTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpPostCollideTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPostCollideTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpPostSolveTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPostSolveTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpPreSolveTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPreSolveTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpProcessFullCastsTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpProcessFullCastsTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpSolveFinalizeTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolveFinalizeTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpSolverFactoryTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverFactoryTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpMultithreadedSimulation::solve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMultithreadedSimulation *
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hknpTaskGraph *
            ctypes.c_ulonglong,  # hkTaskQueue *
        ]
    ),
    "hknpDefaultModifierSet::hknpDefaultModifierSet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDefaultModifierSet *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hknpCollisionDetector::collide": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionDetector *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # hknpCdBody *
        ]
    ),
    "hknpContactImpulseEventCreator::getEnabledFunctions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpContactImpulseEventCreator *
        ]
    ),
    "hknpManifoldEventCreator::getEnabledFunctions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpManifoldEventCreator *
        ]
    ),
    "hknpRestitutionModifier::getEnabledFunctions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpRestitutionModifier *
        ]
    ),
    "hknpModifier::postCompositeQueryAabb": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpDefaultModifierSet::setupModifierManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDefaultModifierSet *
            ctypes.c_ulonglong,  # hknpModifierManager *
        ]
    ),
    "hkSet<unsigned __int64,hkBuiltinContainerAllocator<2,0>,hkMapOperations<unsigned __int64>>::resizeTable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkSet<unsigned __int64,hkBuiltinContainerAllocator<2,0>,hkMapOperations<unsigned __int64> > *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpWorldShiftUtil::shiftWorld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpVariableTimeStepUtil::hknpVariableTimeStepUtil": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVariableTimeStepUtil *
        ]
    ),
    "hknpVariableTimeStepUtil::calcSolverInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVariableTimeStepUtil *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hknpSolverInfo *
        ]
    ),
    "hknpVariableTimeStepUtil::setConfig": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVariableTimeStepUtil *
            ctypes.c_ulonglong,  # const hknpVariableTimeStepUtil::Config *
        ]
    ),
    "hknpAtomSolverFunctions::generateJacobianMatrices<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
        ]
    ),
    "hknpAtomSolverFunctions::generateJacobianMatrices<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
        ]
    ),
    "hknpAtomSolverFunctions::solveQuatAngular3D<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpAtomSolverFunctions::solveQuatAngular3D<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpAtomSolverFunctions::computeInverseVirtualMass3D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # const hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
        ]
    ),
    "hknpAtomSolverFunctions::computeInverseVirtualMass3D_2": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkSymmetricMatrix3Impl<float> *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
        ]
    ),
    "hknpAtomSolverFunctions::estimateFutureOrientations": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # const hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # hkQuaternionf *
            ctypes.c_ulonglong,  # hkQuaternionf *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpAtomSolverFunctions___anonymous_namespace___estimateFutureTransforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # const hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # hkQTransform *
            ctypes.c_ulonglong,  # hkQTransform *
        ]
    ),
    "hknpAtomSolverFunctions___anonymous_namespace___estimateFutureTransformsAndVelocities": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # const hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # hkQTransform *
            ctypes.c_ulonglong,  # hkQTransform *
            ctypes.c_ulonglong,  # hknpAtomSolverFunctions::hknpFuturePosVelInfo *
        ]
    ),
    "hknpConstraintAtomSolver::exportImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpCsJacRange *
            ctypes.c_ulonglong,  # const hknpSolverElemTemp *
            ctypes.c_ulonglong,  # const hknpSolverSumVelocity *
            ctypes.c_ulonglong,  # hknpSolverVelocity *
            ctypes.c_ulonglong,  # const hknpSolverSumVelocity *
            ctypes.c_ulonglong,  # hknpSolverVelocity *
        ]
    ),
    "hknpConstraintAtomSolver::setupConstraints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintAtomSolver *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hknpCsJacWriter *
            ctypes.c_ulonglong,  # hknpCdPairWriter *
        ]
    ),
    "hknpAtomSolverFunctions::solve1dBilateralUserTau": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpJacobian1dBilateralUserTauSchema *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpSolverElemTemp *
        ]
    ),
    "hknpAtomSolverFunctions::solve1dBilateralWithImpulseLimit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpJacobian1dBilateralSchema *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpSolverElemTemp *
        ]
    ),
    "hknpAtomSolverFunctions::solve1dLinearMotor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpSolverStep *
            ctypes.c_ulonglong,  # const hknpJacobian1dLinearMotorSchema *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpSolverElemTemp *
        ]
    ),
    "hknpAtomSolverFunctions::solveDeformableAngConstraint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkQTransform *
            ctypes.c_ulonglong,  # const hkQTransform *
            ctypes.c_ulonglong,  # const hknpDeformableAngSchema *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpSolverElemTemp *
        ]
    ),
    "hknpAtomSolverFunctions::solveDeformableLinConstraint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkQTransform *
            ctypes.c_ulonglong,  # const hkQTransform *
            ctypes.c_ulonglong,  # const hknpDeformableLinSchema *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpSolverElemTemp *
        ]
    ),
    "hknpAtomSolverFunctions::solveEllipticalLimit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpEllipticalLimitSchema *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpSolverElemTemp *
        ]
    ),
    "hknpConstraintAtomSolver::solveJacobians": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpCsJacRange *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintAtomSolver *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverStepInfo *
            ctypes.c_ulonglong,  # const hknpSolverStep *
            ctypes.c_ulonglong,  # const hknpCsJacRange *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # const hknpIdxRange *
            ctypes.c_ulonglong,  # const hknpIdxRange *
        ]
    ),
    "hknpConstraintAtomSolver::solveJacobiansImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpSolverStep *
            ctypes.c_ulonglong,  # const hknpCsJacRange *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # const hknpSolverSumVelocity *
            ctypes.c_ulonglong,  # hknpSolverVelocity *
            ctypes.c_ulonglong,  # const hknpSolverSumVelocity *
            ctypes.c_ulonglong,  # hknpSolverVelocity *
        ]
    ),
    "hknpAtomSolverFunctions::solveLinear3D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # const hkQuaternionf *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpSolverElemTemp *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
        ]
    ),
    "hknpAtomSolverFunctions::solveStableAngularConstraint3d": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpStableAngularSchema3d *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpSolverElemTemp *
        ]
    ),
    "hknpAtomSolverFunctions::solveStableBallSocket": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpStableBallSocketSchema *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpSolverElemTemp *
        ]
    ),
    "hknpAtomSolverFunctions::solveStablePulley": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpStablePulleySchema *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpSolverElemTemp *
        ]
    ),
    "hknpAtomSolverFunctions::solveStableStiffSpring": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpStableStiffSpringSchema *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpSolverElemTemp *
        ]
    ),
    "StreamContactSolver::convergeContact<1,0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_ulonglong,  # StreamContactSolver::VelocityAccumulator *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # StreamContactSolver::ImpulseClipper *
            ctypes.c_ulonglong,  # StreamContactSolver::SolveStateHeader *
        ]
    ),
    "StreamContactSolver::convergeContact<1,256>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_ulonglong,  # StreamContactSolver::VelocityAccumulator *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # StreamContactSolver::ImpulseClipper *
            ctypes.c_ulonglong,  # StreamContactSolver::SolveStateHeader *
        ]
    ),
    "StreamContactSolver::convergeContact<0,0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_ulonglong,  # StreamContactSolver::VelocityAccumulator *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # StreamContactSolver::ImpulseClipper *
            ctypes.c_ulonglong,  # StreamContactSolver::SolveStateHeader *
        ]
    ),
    "StreamContactSolver::convergeContact<0,256>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_ulonglong,  # StreamContactSolver::VelocityAccumulator *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # StreamContactSolver::ImpulseClipper *
            ctypes.c_ulonglong,  # StreamContactSolver::SolveStateHeader *
        ]
    ),
    "StreamContactSolver::convergeContact<0,512>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_ulonglong,  # StreamContactSolver::VelocityAccumulator *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # StreamContactSolver::ImpulseClipper *
            ctypes.c_ulonglong,  # StreamContactSolver::SolveStateHeader *
        ]
    ),
    "hkSimdFloat32::setDiv<1,3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
        ]
    ),
    "hkSimdFloat32::setDiv<2,3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
        ]
    ),
    "hknpLiveWelding::LeafPair::~LeafPair": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLiveWelding::LeafPair *
        ]
    ),
    "hknpStreamContactSolver::bindInjectors": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpStreamContactSolver *
            ctypes.c_ulonglong,  # hknpCsContactJacInjector **
            ctypes.c_ulonglong,  # hknpCsJacWriter **
            ctypes.c_int32,  # int
        ]
    ),
    "hknpStreamContactSolver::buildAndSolveLiveWeldingJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverStepInfo *
            ctypes.c_ulonglong,  # const hknpSolverStep *
            ctypes.c_ulonglong,  # hknpSolverSumVelocity **
            ctypes.c_ulonglong,  # StreamContactSolver::ImpulseClipper *
            ctypes.c_ulonglong,  # StreamContactSolver::Stats *
            ctypes.c_ulonglong,  # StreamContactSolver::VelocityAccumulator *
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
            ctypes.c_ulonglong,  # StreamContactSolver::SolveStateHeader *
            ctypes.c_ulonglong,  # hknpConvexConvexManifoldCollisionCache *
            ctypes.c_ulonglong,  # const hknpManifold *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hkQTransform *
            ctypes.c_ulonglong,  # const hkQTransform *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
        ]
    ),
    "StreamContactSolver::buildContact": FUNCDEF(
        restype=ctypes.c_ulonglong,  # __m128 *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpManifold *
            ctypes.c_ulonglong,  # const hknpStreamContactSolverTypes::BuildConfig *
            ctypes.c_ulonglong,  # hknpStreamContactSolverTypes::BuildState *
            ctypes.c_ulonglong,  # hknpStreamContactSolverTypes::ConstraintState *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpCsJacWriter *
        ]
    ),
    "hknpStreamContactSolver::buildJacobianV": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpStreamContactSolver *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpManifold *
            ctypes.c_ulonglong,  # const hknpStreamContactSolverTypes::BuildConfig *
            ctypes.c_ulonglong,  # hknpStreamContactSolverTypes::ConstraintState *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpCsJacWriter *
        ]
    ),
    "hknpStreamContactSolver::buildLiveWeldingJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpSolverStep *
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
            ctypes.c_ulonglong,  # StreamContactSolver::SolveStateHeader *
            ctypes.c_ulonglong,  # hknpConvexConvexManifoldCollisionCache *
            ctypes.c_ulonglong,  # const hknpManifold *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hkQTransform *
            ctypes.c_ulonglong,  # const hkQTransform *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
        ]
    ),
    "hknpStreamContactSolver::cloneJacobianIntoStream": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpStreamContactJacobian *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # hknpCsJacWriter *
            ctypes.c_ulonglong,  # hkBlockStream::Block **
        ]
    ),
    "hknpStreamContactSolver::createInjectors": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpStreamContactSolver *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpCsContactJacInjector **
        ]
    ),
    "hknpStreamContactSolver::destroyInjectors": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpStreamContactSolver *
            ctypes.c_ulonglong,  # hknpCsContactJacInjector **
            ctypes.c_int32,  # int
        ]
    ),
    "hknpStreamContactSolver::initMotionTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpStreamContactSolver *
            ctypes.c_ulonglong,  # hknpCsContactJacInjector *
            ctypes.c_ulonglong,  # const hknpMotion *
            ctypes.c_ulonglong,  # const hknpMotion *
        ]
    ),
    "hknpStreamContactSolver::injectJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpStreamContactSolver *
            ctypes.c_ulonglong,  # hknpCsContactJacInjector *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # const hknpManifold *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hknpStreamContactSolver::reuseJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpStreamContactSolver *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpJacobianConsumer *
            ctypes.c_ulonglong,  # hknpCsContactJacInjector *
        ]
    ),
    "hknpStreamContactSolver::setJacRangeEndPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpStreamContactSolver *
            ctypes.c_ulonglong,  # hknpCsContactJacInjector *
            ctypes.c_ulonglong,  # hkBlockStream::Range *
        ]
    ),
    "hknpStreamContactSolver::setJacRangeStartPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpStreamContactSolver *
            ctypes.c_ulonglong,  # hknpCsContactJacInjector *
            ctypes.c_ulonglong,  # hkBlockStream::Range *
        ]
    ),
    "hknpStreamContactSolver::solveJacobians": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpCsJacRange *
        argtypes=[
            ctypes.c_ulonglong,  # hknpStreamContactSolver *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverStepInfo *
            ctypes.c_ulonglong,  # const hknpSolverStep *
            ctypes.c_ulonglong,  # const hknpCsJacRange *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # const hknpIdxRange *
            ctypes.c_ulonglong,  # const hknpIdxRange *
        ]
    ),
    "hkSignal1_hknpWorld_____subscribe__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld____": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal1<hknpWorld *>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hknpActionManagerEx *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpActionManagerEx*', 'name': 'this'}, 'hknpWorld *']}
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkSignal2_hknpWorld___hknpStepInput_const_____subscribe__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld___hknpStepInput_const____": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hknpStepInput const &>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hknpActionManagerEx *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpActionManagerEx*', 'name': 'this'}, 'hknpWorld *', 'const hknpStepInput *']}
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkSignal2_hknpWorld___hkVector4f_const_____subscribe__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld___hkVector4f_const____": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hkVector4f const &>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hknpActionManagerEx *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpActionManagerEx*', 'name': 'this'}, 'hknpWorld *', 'const hkVector4f *']}
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkSignal3_hknpWorld___hknpBodyId_const___int___subscribe__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld___hknpBodyId_const___int__": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal3<hknpWorld *,hknpBodyId const *,int>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hknpActionManagerEx *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'hknpActionManagerEx*', 'name': 'this'}, 'hknpWorld *', 'const hknpBodyId *', 'int']}
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hknpActionManager::hknpActionManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpActionManager *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpActionManager::~hknpActionManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpActionManager *
        ]
    ),
    "hknpActionManager::addAction": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpAction *
        argtypes=[
            ctypes.c_ulonglong,  # hknpActionManager *
            ctypes.c_ulonglong,  # hknpAction *
            ctypes.c_ulonglong,  # hkReferencedObject *
        ]
    ),
    "hknpActionManager::applyActions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpActionManager *
            ctypes.c_float,  # float
        ]
    ),
    "hknpActionManager::removeAction": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpAction *
        argtypes=[
            ctypes.c_ulonglong,  # hknpActionManager *
            ctypes.c_ulonglong,  # hknpAction *
            ctypes.c_ulonglong,  # hknpAction *
        ]
    ),
    "hkSignal1_hknpWorld_____MemberSlot__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld______subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal1<hknpWorld *> *
            ctypes.c_ulonglong,  # hknpActionManagerEx *
        ]
    ),
    "hkSignal2_hknpWorld___hknpStepInput_const_____MemberSlot__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld___hknpStepInput_const______subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hknpStepInput const &> *
            ctypes.c_ulonglong,  # hknpActionManagerEx *
        ]
    ),
    "hkSignal2_hknpWorld___hkVector4f_const_____MemberSlot__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld___hkVector4f_const______subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpWorld *,hkVector4f const &> *
            ctypes.c_ulonglong,  # hknpActionManagerEx *
        ]
    ),
    "hkSignal3_hknpWorld___hknpBodyId_const___int___MemberSlot__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld___hknpBodyId_const___int____subscribe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal3<hknpWorld *,hknpBodyId const *,int> *
            ctypes.c_ulonglong,  # hknpActionManagerEx *
        ]
    ),
    "hknpWorldUtil::addBroadPhaseDecorator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hknpDecoratorBroadPhase *
        ]
    ),
    "hknpConstraintDataNullFunctions::buildJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_ulonglong,  # const hknpConstraintQueryIn *
            ctypes.c_ulonglong,  # hknpConstraintQueryOut *
        ]
    ),
    "hknpConstraintDataNullFunctions::enableDirectSolver": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpConstraintData *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_char,  # char
        ]
    ),
    "hknpConstraintDataNullFunctions::getAppliedImpulsesInSolverSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
            ctypes.c_ulonglong,  # hknpConstraintData::AppliedImpulses *
        ]
    ),
    "hknpConstraintDataNullFunctions::getAppliedLinearImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const struct hknpConstraintRuntime *
        ]
    ),
    "hknpConstraintDataNullFunctions::getConstraintInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintData::ConstraintInfo *
        ]
    ),
    "hknpConstraintDataNullFunctions::getInertiaStabilizationFactor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpConstraintData *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpConstraintDataNullFunctions::getMaximumAngularImpulse": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpConstraintDataNullFunctions::getMaximumLinearImpulse": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpConstraintDataNullFunctions::getReflectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[]
    ),
    "hknpConstraintDataNullFunctions::getRuntimeInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpConstraintData::RuntimeInfo *
        ]
    ),
    "hknpConstraintDataNullFunctions::isValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpConstraintDataNullFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpConstraintDataNullFunctions::setInertiaStabilizationFactor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpConstraintData *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_float,  # float
        ]
    ),
    "hknpConstraintDataNullFunctions::setMaximumAngularImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_float,  # float
        ]
    ),
    "hknpConstraintDataNullFunctions::setMaximumLinearImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_float,  # float
        ]
    ),
    "hknpBreakableConstraintDataFunctions::isValid": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
        ]
    ),
    "hknpBreakableConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hknpMalleableConstraintDataFunctions::buildJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintData *
            ctypes.c_ulonglong,  # const hknpConstraintQueryIn *
            ctypes.c_ulonglong,  # hknpConstraintQueryOut *
        ]
    ),
    "hknpMalleableConstraintDataFunctions::getConstraintInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ulonglong,  # hknpConstraintData::ConstraintInfo *
        ]
    ),
    "hknpMalleableConstraintDataFunctions::getRuntimeInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpConstraintData::RuntimeInfo *
        ]
    ),
    "hknpMalleableConstraintDataFunctions::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintData> *
            ctypes.c_ulonglong,  # const hknpConstraintData *
            ctypes.c_uint32,  # hknpConstraintData::CloneMode
            ctypes.c_ulonglong,  # hknpConstraintDataClonedObjectMap *
        ]
    ),
    "hkArrayBase<hknpParticleParticleCollisionsCacheHeader>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpParticleParticleCollisionsCacheHeader> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpParticleParticleCollider::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleParticleCollider *
            ctypes.c_ulonglong,  # hknpParticlesColliderEx *
        ]
    ),
    "hknpParticleParticleCollider::invalidateCaches": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleParticleCollider *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkArrayBase<hkThreadLocalTimeStampHash<unsigned __int64,hknpLandscapeQuadProvider::HashValue>::LocalValue>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpParticleFaceVerticesWithEffMass> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkTimeStampHash<unsigned __int64,hknpLandscapeQuadProvider::HashValue>::Value>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkTimeStampHash<unsigned __int64,hknpLandscapeQuadProvider::HashValue>::Value> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkThreadLocalTimeStampHash<unsigned __int64,hknpLandscapeQuadProvider::HashValue>::hkThreadLocalTimeStampHash<unsigned __int64,hknpLandscapeQuadProvider::HashValue>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkThreadLocalTimeStampHash<unsigned __int64,hknpLandscapeQuadProvider::HashValue> *
            ctypes.c_ulonglong,  # hkTimeStampHash<unsigned __int64,hknpLandscapeQuadProvider::HashValue> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkThreadLocalTimeStampHash<unsigned __int64,hknpLandscapeQuadProvider::HashValue>::LocalValue *
        ]
    ),
    "hkTimeStampHash<unsigned __int64,hknpLandscapeQuadProvider::HashValue>::hkTimeStampHash<unsigned __int64,hknpLandscapeQuadProvider::HashValue>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTimeStampHash<unsigned __int64,hknpLandscapeQuadProvider::HashValue> *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpParticleSimulationThreadContext::hknpParticleSimulationThreadContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleSimulationThreadContext *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkTimeStampHash<unsigned __int64,hknpLandscapeQuadProvider::HashValue> *
        ]
    ),
    "hkThreadLocalTimeStampHash<unsigned __int64,hknpLandscapeQuadProvider::HashValue>::~hkThreadLocalTimeStampHash<unsigned __int64,hknpLandscapeQuadProvider::HashValue>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkThreadLocalTimeStampHash<unsigned __int64,hknpLandscapeQuadProvider::HashValue> *
        ]
    ),
    "hknpParticleSimulationThreadContext::ParticleInfo::~ParticleInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleSimulationThreadContext::ParticleInfo *
        ]
    ),
    "hknpParticleSimulationContext::~hknpParticleSimulationContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleSimulationContext *
        ]
    ),
    "hknpParticleSimulationContext::commitLocalChanges": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleSimulationContext *
        ]
    ),
    "hknpParticleSimulationContext::dispatchCommands": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleSimulationContext *
            ctypes.c_ulonglong,  # hkPrimaryCommandDispatcher *
        ]
    ),
    "hknpParticleSimulationThreadContext::execCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleSimulationThreadContext *
            ctypes.c_ulonglong,  # const hkCommand *
        ]
    ),
    "hknpParticleSimulationContext::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleSimulationContext *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkArrayBase<hknpParticleCollisionCache>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpParticleCollisionCache> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpParticleDynamicNeighborsCache>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpParticleDynamicNeighborsCache> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpParticleNmpCacheBase>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpParticleNmpCacheBase> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpParticlesBodyCollider::hknpParticlesBodyCollider": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesBodyCollider *
            ctypes.c_ulonglong,  # hknpParticlesColliderEx *
            ctypes.c_ulonglong,  # const hkArray<hknpParticleLinearInfo,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpParticlesBodyCollider::~hknpParticlesBodyCollider": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesBodyCollider *
        ]
    ),
    "hknpParticlesBodyCollider::invalidateCaches": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesBodyCollider *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hknpParticlesBodyCollider::shiftCaches": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesBodyCollider *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpCallbackConstraintMotor::makeClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintMotor> *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCallbackConstraintMotor *
            ctypes.c_ulonglong,  # hkRefPtr<hknpConstraintMotor> *
        ]
    ),
    "hknpStreamContactJacobianUtil::calcDistances": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpStreamContactJacobianUtil::calcMinDistance": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
        ]
    ),
    "hknpStreamContactJacobianUtil::calculateManifold": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_ulonglong,  # const hknpMotion *
            ctypes.c_uint32,  # hknpContactJacobianUtil::BackstepMode
            ctypes.c_ulonglong,  # hkcdManifold4 *
        ]
    ),
    "hknpStreamContactJacobianUtil::disableContact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
        ]
    ),
    "hknpStreamContactJacobianUtil::getContactVelocities": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpStreamContactJacobianUtil::getDv": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSimdFloat32 *
        argtypes=[
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpStreamContactJacobianUtil::getJacModHdr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpStreamContactSolverTypes::JacModHdr *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
        ]
    ),
    "hknpStreamContactJacobianUtil::getJacModMfoldData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpStreamContactSolverTypes::JacModMfoldData *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactSolverTypes::JacModHdr *
        ]
    ),
    "hknpStreamContactJacobianUtil::getJacModNormalVel": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpStreamContactSolverTypes::JacModNormalVel *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactSolverTypes::JacModHdr *
        ]
    ),
    "hknpStreamContactJacobianUtil::getPointEffMass": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpBodyIndex *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpStreamContactJacobianUtil::getPointRhs": FUNCDEF(
        restype=ctypes.c_ulonglong,  # float *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpStreamContactJacobianUtil::scaleEffectiveMass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_float,  # float
        ]
    ),
    "hknpStreamContactJacobianUtil::scaleFriction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "hknpStreamContactJacobianUtil::scalePenetrations": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_float,  # float
        ]
    ),
    "hknpStreamContactJacobianUtil::setDistances": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
        ]
    ),
    "hknpStreamContactJacobianUtil::setDynamic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
        ]
    ),
    "hknpStreamContactJacobianUtil::setMaxImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # hknpImpulseClipMode::Enum
        ]
    ),
    "hknpStreamContactJacobianUtil::setSurfaceVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkFixedSizeBuffer_hkRadixSort__SortDataT_unsigned_int__hknpConstraintAtomSolverSetup__SortedConstraintStates__init____2___TwoInts__hkFixedStackStorage_hkRadixSort__SortDataT_unsigned_int__hknpConstraintAtomSolverSetup__SortedConstraintStates__init____2___TwoInts_______hkFixedSizeBuffer_hkRadixSort__SortDataT_unsigned_int__hknpConstraintAtomSolverSetup__SortedConstraintStates__init____2___TwoInts__hkFixedStackStorage_hkRadixSort__SortDataT_unsigned_int__hknpConstraintAtomSolverSetup__SortedConstraintStates__init____2___TwoInts_____": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "hknpConstraintGroupDirectSolver::SolverSetupData::SolverSetupData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::SolverSetupData *
            ctypes.c_ulonglong,  # const hknpConstraintGroupDirectSolverSettings *
        ]
    ),
    "hknpConstraintAtomSolverSetup::SortedConstraintStates::SortedConstraintStates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintAtomSolverSetup::SortedConstraintStates *
        ]
    ),
    "hknpConstraintGroupDirectSolver::MatrixInitializationStructure::~MatrixInitializationStructure": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::MatrixInitializationStructure *
        ]
    ),
    "hknpConstraintGroupDirectSolver::SolverSetupData::~SolverSetupData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::SolverSetupData *
        ]
    ),
    "hknpConstraintAtomSolverSetup::buildJacobianMt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpWorldEx *
            ctypes.c_ulonglong,  # hknpConstraint *
            ctypes.c_ulonglong,  # hknpCsGrid *
            ctypes.c_ulonglong,  # hkBlockStream *
            ctypes.c_ulonglong,  # hknpCdPairStream *
        ]
    ),
    "hknpConstraintAtomSolverSetup::buildJacobiansMt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpWorldEx *
            ctypes.c_ulonglong,  # const hknpConstraint *
            ctypes.c_ulonglong,  # const hknpConstraintGroup *
            ctypes.c_ulonglong,  # const hkArray<hknpConstraintAtomSolverSetup::ConstraintState,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpCsGrid *
            ctypes.c_ulonglong,  # hkBlockStream *
            ctypes.c_ulonglong,  # hknpCdPairStream *
        ]
    ),
    "hknpConstraintAtomSolverSetup::buildJacobiansSt": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpWorldEx *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # hknpConstraint *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hknpConstraintGroup *
            ctypes.c_ulonglong,  # hknpCsJacWriter *
            ctypes.c_ulonglong,  # hknpCdPairWriter *
        ]
    ),
    "hknpConstraintAtomSolverSetup::SortedConstraintStates::clearAndDeallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintAtomSolverSetup::SortedConstraintStates *
        ]
    ),
    "hknpConstraintAtomSolverSetup::SortedConstraintStates::constraintGroupsEnforceSameCell": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintAtomSolverSetup::SortedConstraintStates *
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_ulonglong,  # const hknpConstraint *
            ctypes.c_ulonglong,  # hkArray<hknpConstraintAtomSolverSetup::ConstraintStateGroup,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpConstraintAtomSolverSetup::ConstraintStateGroups::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintAtomSolverSetup::ConstraintStateGroups *
            ctypes.c_ulonglong,  # const hknpWorldEx *
            ctypes.c_ulonglong,  # const hknpConstraintAtomSolverSetup::SortedConstraintStates *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hknpConstraint *
        ]
    ),
    "hknpConstraintAtomSolverSetup::SortedConstraintStates::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintAtomSolverSetup::SortedConstraintStates *
            ctypes.c_ulonglong,  # const hknpWorldEx *
            ctypes.c_ulonglong,  # hknpConstraint *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkHashSet<hknpBodyIndexPair,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpConstraintAtomSolverSetup::SortedConstraintStates::sortReactivated": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintAtomSolverSetup::SortedConstraintStates *
            ctypes.c_ulonglong,  # const hknpWorldEx *
            ctypes.c_ulonglong,  # const hknpConstraint *
            ctypes.c_ulonglong,  # hkArray<hknpConstraintAtomSolverSetup::ConstraintStateGroup,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkArrayBase<hknpParticle4Faces>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpParticle4Faces> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpParticleFaceVerticesWithEffMass>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpParticleFaceVerticesWithEffMass> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpParticleShapeProperties::hknpParticleShapeProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleShapeProperties *
            ctypes.c_ulonglong,  # const hknpConvexShape *
        ]
    ),
    "hknpContinuousGsk::calculateToi": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpContinuousGsk *
            ctypes.c_ulonglong,  # const hknpContinuousGsk::Input *
            ctypes.c_ulonglong,  # hkcdGsk::Cache *
            ctypes.c_ulonglong,  # hknpContinuousGsk::Output *
        ]
    ),
    "hknpContinuousGsk::calculateToiCast": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpContinuousGsk *
            ctypes.c_ulonglong,  # const hknpContinuousGsk::Input *
            ctypes.c_ulonglong,  # hkcdGsk::Cache *
            ctypes.c_ulonglong,  # hknpContinuousGsk::Output *
        ]
    ),
    "hknpContinuousGsk::calculateToiStaticB": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpContinuousGsk *
            ctypes.c_ulonglong,  # const hknpContinuousGsk::Input *
            ctypes.c_ulonglong,  # hkcdGsk::Cache *
            ctypes.c_ulonglong,  # hknpContinuousGsk::Output *
        ]
    ),
    "hknpCompoundShapeEx::getClosestPointsImpl<1,1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompoundShapeEx::getClosestPointsImpl<2,1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompoundShapeEx::getClosestPointsImpl<8,1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompoundShapeEx::getClosestPointsImpl<0,1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpCompoundShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpParticlesClosestHitOnMeshCollector::hknpParticlesClosestHitOnMeshCollector": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesClosestHitOnMeshCollector *
            ctypes.c_ulonglong,  # hknpParticlesCollisionQueryCollector *
        ]
    ),
    "hknpParticlesCollisionFilter::hknpParticlesCollisionFilter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesCollisionFilter *
        ]
    ),
    "hknpParticlesCollisionFilter::filterBodyPairs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesCollisionFilter *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hknpBodyIndexPair *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpParticlesCollisionQueryCollector::getClosestPointToConvex": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesCollisionQueryCollector *
            ctypes.c_ulonglong,  # const hknpCollisionQueryDispatcher *
            ctypes.c_ulonglong,  # const hknpConvexVertexShape *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpConvexVertexShape *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # hkcdCollisionResult *
        ]
    ),
    "hknpParticleWorldQuery::hknpParticleCompoundClosest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpParticleWorldQuery::hknpParticleCompressedMeshClosest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpParticleWorldQuery::hknpParticleExternMeshClosest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpParticleWorldQuery::hknpParticleHeightFieldClosest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpParticleWorldQuery::hknpParticleMeshClosest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpParticleWorldQuery::hknpParticleSDFClosest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpParticlesCollisionFilter::isCollisionEnabled": {
        "hknpParticlesCollisionFilter *, hknpCollisionQueryType::Enum, const hknpQueryFilterData *, const hknpBody *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # hknpParticlesCollisionFilter *
                ctypes.c_uint32,  # hknpCollisionQueryType::Enum
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpBody *
            ]
        ),
        "hknpParticlesCollisionFilter *, __int64, __int64, const hknpCollisionFilter::FilterInput *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # hknpParticlesCollisionFilter *
                ctypes.c_int64,  # __int64
                ctypes.c_int64,  # __int64
                ctypes.c_ulonglong,  # const hknpCollisionFilter::FilterInput *
            ]
        ),
    },
    "hknpParticlesClosestHitOnMeshCollector::processClosestPointsCvxCvx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticlesClosestHitOnMeshCollector *
            ctypes.c_ulonglong,  # const hknpCollisionQueryDispatcher *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpConvexVertexShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpConvexVertexShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
        ]
    ),
    "hkTimeStampHashBase::hkTimeStampHashBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTimeStampHashBase *
        ]
    ),
    "hkTimeStampHashBase::~hkTimeStampHashBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkTimeStampHashBase *
        ]
    ),
    "hkArrayBase<hknpProcessNarrowPhasePairsTask::ThreadOutput>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpProcessNarrowPhasePairsTask::ThreadOutput> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpProcessNarrowPhasePairsTask::ThreadOutput::ThreadOutput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpProcessNarrowPhasePairsTask::ThreadOutput *
            ctypes.c_ulonglong,  # hknpProcessNarrowPhasePairsTask::ThreadOutput *
        ]
    ),
    "hknpCollisionCacheStreams::hknpCollisionCacheStreams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionCacheStreams *
            ctypes.c_ulonglong,  # hknpCollisionCacheStreams *
        ]
    ),
    "hknpProcessNarrowPhasePairsTask::hknpProcessNarrowPhasePairsTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpProcessNarrowPhasePairsTask *
            ctypes.c_ulonglong,  # hkCriticalSection *
        ]
    ),
    "hknpProcessNarrowPhasePairsTask::~hknpProcessNarrowPhasePairsTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpProcessNarrowPhasePairsTask *
        ]
    ),
    "hknpProcessNarrowPhasePairsTask::addWorkItems": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpProcessNarrowPhasePairsTask *
            ctypes.c_ulonglong,  # const hknpBlockStreamBlockOffsetPair *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpProcessNarrowPhasePairsTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpProcessNarrowPhasePairsTask *
        ]
    ),
    "hknpProcessNarrowPhasePairsTask::getNextWorkItem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpBlockStreamBlockOffsetPair *
        argtypes=[
            ctypes.c_ulonglong,  # hknpProcessNarrowPhasePairsTask *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpProcessNarrowPhasePairsTask::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpProcessNarrowPhasePairsTask *
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hknpSolverData *
            ctypes.c_ulonglong,  # hknpCollisionCacheManager *
        ]
    ),
    "hknpProcessNarrowPhasePairsTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpProcessNarrowPhasePairsTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpSubIntegrateLastStepProcess": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const hknpSolverStepInfo *
            ctypes.c_ulonglong,  # hkMonitorStream *
        ]
    ),
    "hknpSubIntegrateProcess": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const hknpSolverStepInfo *
            ctypes.c_ulonglong,  # hkMonitorStream *
        ]
    ),
    "hknpSolverIntegrator::subIntegrate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverStepInfo *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hknpMotionProperties *
            ctypes.c_ulonglong,  # hkMonitorStream *
        ]
    ),
    "hknpSolverIntegrator::subIntegrateLastStep": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverStepInfo *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkMonitorStream *
        ]
    ),
    "hknpCollisionCacheUtils::deepCopyJacobianCachePointer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionCache *
            ctypes.c_ulonglong,  # hkBlockStream::Writer *
        ]
    ),
    "hknpDecoratorBroadPhase::buildTaskGraph": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDecoratorBroadPhase *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # hkTypedBlockStream<hknpBodyIndexPair> *
            ctypes.c_ulonglong,  # hknpTaskGraph *
        ]
    ),
    "hknpDecoratorBroadPhase::copyFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDecoratorBroadPhase *
            ctypes.c_ulonglong,  # const hknpBroadPhase *
        ]
    ),
    "hknpDecoratorBroadPhase::findAllPairs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDecoratorBroadPhase *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hkTypedBlockStream<hknpBodyIndexPair>::Writer *
        ]
    ),
    "hknpDecoratorBroadPhase::findNewPairs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDecoratorBroadPhase *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hkAabbFloat3 *
            ctypes.c_ulonglong,  # hkTypedBlockStream<hknpBodyIndexPair>::Writer *
        ]
    ),
    "hknpDecoratorBroadPhase::getConfig": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpBroadPhaseConfig *
        argtypes=[
            ctypes.c_ulonglong,  # hknpDecoratorBroadPhase *
        ]
    ),
    "hknpDecoratorBroadPhase::getExtents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDecoratorBroadPhase *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpDecoratorBroadPhase::markBodiesDirty": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDecoratorBroadPhase *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # hknpBroadPhase::UpdateModeHint
        ]
    ),
    "hknpDecoratorBroadPhase::removeBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDecoratorBroadPhase *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpDecoratorBroadPhase::setBodyBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLVolAutmNode *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "hknpDecoratorBroadPhase::translate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDecoratorBroadPhase *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpDecoratorBroadPhase::update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDecoratorBroadPhase *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpDecoratorBroadPhase::updateBodyAabbs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDecoratorBroadPhase *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpConvexCompositeCollisionCache::construct": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpCollisionCache *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCreateCollisionCacheInput *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
        ]
    ),
    "hknpCompositeCompositeCollisionCache::construct": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpCollisionCache *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCreateCollisionCacheInput *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
        ]
    ),
    "hknpManifoldCollisionCache::allocateProperty": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
            ctypes.c_uint32,  # hknpManifoldCollisionCache::PropertyKey
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hknpGskCollisionCache::construct": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpCollisionCache *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCreateCollisionCacheInput *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
        ]
    ),
    "hknpManifoldCollisionCache::fireManifoldDestroyed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
        ]
    ),
    "hknpGskCollisionCache::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGskCollisionCache *
            ctypes.c_ulonglong,  # const hknpCreateCollisionCacheInput *
        ]
    ),
    "hknpManifoldCollisionCache::initMaterialAndShapeSettings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # const hknpMaterial *
            ctypes.c_ulonglong,  # const hknpMaterial *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpManifoldCollisionCache::setCircularSymmetryAxes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
        ]
    ),
    "hknpSignedDistanceFieldCollisionCache::construct": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpCollisionCache *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCreateCollisionCacheInput *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
        ]
    ),
    "hknpBroadphaseTriggerCollisionCache::construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # const hknpBody *
            ctypes.c_ulonglong,  # const hknpBody *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
        ]
    ),
    "hknpManifoldBuffer::hknpManifoldBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpManifoldBuffer *
        ]
    ),
    "hknpManifoldBuffer::flushImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpManifoldBuffer *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
        ]
    ),
    "hknpBinaryBodyDebugEvent::printCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBinaryBodyDebugEvent *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpBodyExitedSafeRegionEvent::printCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBodyExitedSafeRegionEvent *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpUnaryBodyDebugEvent::printCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpUnaryBodyDebugEvent *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hknpCollisionFilter::removeDisabledConstraintPairsFromStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hkHashSet<hknpBodyIndexPair,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkTypedBlockStream<hknpBodyIndexPair> *
        ]
    ),
    "hknpCollideTimers::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollideTimers *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hknpCollideTimers::exportTimers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollideTimers *
            ctypes.c_ulonglong,  # hkMonitorStream *
        ]
    ),
    "hknpCompositeCompositeTimers::exportTimers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompositeCompositeTimers *
            ctypes.c_ulonglong,  # hkMonitorStream *
        ]
    ),
    "hknpCvxCompositeKeysTimers::exportTimers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCvxCompositeKeysTimers *
            ctypes.c_ulonglong,  # hkMonitorStream *
        ]
    ),
    "hknpCvxCompositeTimers::exportTimers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCvxCompositeTimers *
            ctypes.c_ulonglong,  # hkMonitorStream *
        ]
    ),
    "hknpCvxCvxDetectorTimers::exportTimers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCvxCvxDetectorTimers *
            ctypes.c_ulonglong,  # hkMonitorStream *
        ]
    ),
    "hknpDebrisDetectorTimers::exportTimers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpDebrisDetectorTimers *
            ctypes.c_ulonglong,  # hkMonitorStream *
        ]
    ),
    "mergeAndCollide2StreamsIntern__anonymous_namespace___cacheConsumerWrapper_A0xc686054b__hknpCacheStarConsumers_1_A0xc686054b__hknpCollideTimersAccessor_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpInternalCollideSharedData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # cacheConsumerWrapper *
            ctypes.c_uint32,  # hknpCollisionCache::RebuildMode
            ctypes.c_ulonglong,  # hknpCacheStarConsumers *
            ctypes.c_uint32,  # hknpCollisionCache::RebuildMode
            ctypes.c_ulonglong,  # hkTypedBlockStream<hknpCollisionCacheWorkItem>::Writer *
            ctypes.c_ulonglong,  # hknpStreamWorkItemWriter *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriters *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriters *
            ctypes.c_ulonglong,  # hknpCdPairStream *
            ctypes.c_ulonglong,  # struct hknpCsContactJacInjector *
            ctypes.c_ulonglong,  # struct hknpCsContactJacInjector *
            ctypes.c_ulonglong,  # hknpCollideTimersAccessor *
        ]
    ),
    "mergeAndCollide2StreamsIntern__anonymous_namespace___cacheConsumerWrapper_A0xc686054b__hknpContigCacheConsumers_1_A0xc686054b__hknpCollideTimersAccessor_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpInternalCollideSharedData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # cacheConsumerWrapper *
            ctypes.c_uint32,  # hknpCollisionCache::RebuildMode
            ctypes.c_ulonglong,  # hknpContigCacheConsumers *
            ctypes.c_uint32,  # hknpCollisionCache::RebuildMode
            ctypes.c_ulonglong,  # hkTypedBlockStream<hknpCollisionCacheWorkItem>::Writer *
            ctypes.c_ulonglong,  # hknpStreamWorkItemWriter *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriters *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriters *
            ctypes.c_ulonglong,  # hknpCdPairStream *
            ctypes.c_ulonglong,  # struct hknpCsContactJacInjector *
            ctypes.c_ulonglong,  # struct hknpCsContactJacInjector *
            ctypes.c_ulonglong,  # hknpCollideTimersAccessor *
        ]
    ),
    "mergeAndCollide2StreamsIntern_hknpNarrowPhaseWorkStealingConsumer__anonymous_namespace___hknpContigCacheConsumers_0_hknpNarrowPhaseWorkStealingConsumer_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpInternalCollideSharedData *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # hknpNarrowPhaseWorkStealingConsumer *
            ctypes.c_uint32,  # hknpCollisionCache::RebuildMode
            ctypes.c_ulonglong,  # hknpContigCacheConsumers *
            ctypes.c_uint32,  # hknpCollisionCache::RebuildMode
            ctypes.c_ulonglong,  # hkTypedBlockStream<hknpCollisionCacheWorkItem>::Writer *
            ctypes.c_ulonglong,  # hknpStreamWorkItemWriter *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriters *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriters *
            ctypes.c_ulonglong,  # hknpCdPairStream *
            ctypes.c_ulonglong,  # struct hknpCsContactJacInjector *
            ctypes.c_ulonglong,  # struct hknpCsContactJacInjector *
            ctypes.c_ulonglong,  # hknpNarrowPhaseWorkStealingConsumer *
        ]
    ),
    "buildJacobianForManifold": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # hknpManifold *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpJacobianConsumer *
            ctypes.c_ulonglong,  # hknpCsContactJacInjector *
        ]
    ),
    "hknpCollidePipeline::buildJacobiansForManifoldRange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hkBlockStream *
            ctypes.c_ulonglong,  # hkBlockStream::LinkedRange *
            ctypes.c_ulonglong,  # hknpCsContactJacInjector *
        ]
    ),
    "hknpCollidePipeline::collideStealingWork": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpInternalCollideSharedData *
            ctypes.c_ulonglong,  # hknpNarrowPhaseWorkStealingConsumer *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriters *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriters *
            ctypes.c_ulonglong,  # hknpCdPairStream *
            ctypes.c_ulonglong,  # hknpCsContactJacInjector *
            ctypes.c_ulonglong,  # hknpCsContactJacInjector *
        ]
    ),
    "hknpCollidePipeline::mergeAndCollide2Streams": {
        "const hknpSimulationThreadContext *, const hknpInternalCollideSharedData *, int, hknpCollisionCacheConsumers *, hknpCollisionCache::RebuildMode, hknpCollisionCacheConsumers *, hknpCollisionCache::RebuildMode, hknpCollisionCacheWriters *, hknpCollisionCacheWriters *, hknpCdPairStream *, hknpCsContactJacInjector *, hknpCsContactJacInjector *, hkBlockStream::Writer *, hknpCollideTimers *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
                ctypes.c_ulonglong,  # const hknpInternalCollideSharedData *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # hknpCollisionCacheConsumers *
                ctypes.c_uint32,  # hknpCollisionCache::RebuildMode
                ctypes.c_ulonglong,  # hknpCollisionCacheConsumers *
                ctypes.c_uint32,  # hknpCollisionCache::RebuildMode
                ctypes.c_ulonglong,  # hknpCollisionCacheWriters *
                ctypes.c_ulonglong,  # hknpCollisionCacheWriters *
                ctypes.c_ulonglong,  # hknpCdPairStream *
                ctypes.c_ulonglong,  # hknpCsContactJacInjector *
                ctypes.c_ulonglong,  # hknpCsContactJacInjector *
                ctypes.c_ulonglong,  # hkBlockStream::Writer *
                ctypes.c_ulonglong,  # hknpCollideTimers *
            ]
        ),
        "const hknpSimulationThreadContext *, const hknpInternalCollideSharedData *, int, hknpCollisionCache::RebuildMode, bool *, hknpCollisionCacheConsumers *, hkTypedBlockStream<hknpBlockStreamBlockOffsetPair>::Consumer *, hkBlockStream::RandomAccessConsumer *, hknpCollisionCacheBatchConsumer *, hkTypedBlockStream<hknpCollisionCacheWorkItem>::Writer *, hknpStreamWorkItemWriter *, hknpCollisionCacheWriters *, hknpCollisionCacheWriters *, hknpCdPairStream *, hkBlockStream::Writer *, hkBlockStream::Writer *, hknpCsContactJacInjector *, hknpCsContactJacInjector *, hknpCollideTimers *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
                ctypes.c_ulonglong,  # const hknpInternalCollideSharedData *
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # hknpCollisionCache::RebuildMode
                ctypes.c_ulonglong,  # bool *
                ctypes.c_ulonglong,  # hknpCollisionCacheConsumers *
                ctypes.c_ulonglong,  # hkTypedBlockStream<hknpBlockStreamBlockOffsetPair>::Consumer *
                ctypes.c_ulonglong,  # hkBlockStream::RandomAccessConsumer *
                ctypes.c_ulonglong,  # hknpCollisionCacheBatchConsumer *
                ctypes.c_ulonglong,  # hkTypedBlockStream<hknpCollisionCacheWorkItem>::Writer *
                ctypes.c_ulonglong,  # hknpStreamWorkItemWriter *
                ctypes.c_ulonglong,  # hknpCollisionCacheWriters *
                ctypes.c_ulonglong,  # hknpCollisionCacheWriters *
                ctypes.c_ulonglong,  # hknpCdPairStream *
                ctypes.c_ulonglong,  # hkBlockStream::Writer *
                ctypes.c_ulonglong,  # hkBlockStream::Writer *
                ctypes.c_ulonglong,  # hknpCsContactJacInjector *
                ctypes.c_ulonglong,  # hknpCsContactJacInjector *
                ctypes.c_ulonglong,  # hknpCollideTimers *
            ]
        ),
    },
    "hknpManifoldStreamIterator<hkBlockStream::Modifier>::setupNextHeader": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpManifoldStreamIterator<hkBlockStream::Modifier> *
        ]
    ),
    "hknpFullCast::ManifoldGenerator::collide": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpFullCast::ManifoldGenerator *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpConvexConvexManifoldGenerator::Input *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpGskCollisionCache *
            ctypes.c_ulonglong,  # hknpManifold *
            ctypes.c_ulonglong,  # hkMonitorStream::MultiTimerCommand **
        ]
    ),
    "hkArrayBase<hknpFullCast::BodyData>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpFullCast::BodyData> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpJacobianConsumer::~hknpJacobianConsumer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpJacobianConsumer *
        ]
    ),
    "hknpFullCast::collideComposite": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFullCast::Context *
            ctypes.c_ulonglong,  # hknpCdBody *
            ctypes.c_ulonglong,  # hknpCdBody *
            ctypes.c_ulonglong,  # hknpCompositeCollisionCache *
            ctypes.c_ulonglong,  # hknpCollisionCacheBatchConsumer *
        ]
    ),
    "hknpFullCast::collideConvex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFullCast::Context *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpGskCollisionCache *
        ]
    ),
    "hknpFullCast::collideExisting": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFullCast::Context *
            ctypes.c_ulonglong,  # const hknpCollisionCache *
            ctypes.c_ulonglong,  # const hknpBody *
            ctypes.c_ulonglong,  # const hknpBody *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hknpFullCast::collideMain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFullCast::Context *
            ctypes.c_ulonglong,  # const hknpCollisionCache *
        ]
    ),
    "hknpFullCast::collideNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFullCast::Context *
            ctypes.c_ulonglong,  # const hknpBody *
            ctypes.c_ulonglong,  # const hknpBody *
        ]
    ),
    "hknpFullCast::integrate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkVector4d *
            ctypes.c_ulonglong,  # hkQuaternionf *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
        ]
    ),
    "hknpFullCast::setupBody": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFullCast::Context *
            ctypes.c_ulonglong,  # const hknpBody *
            ctypes.c_uint32,  # hknpLevelOfDetail
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hkQuaternionf *
            ctypes.c_ulonglong,  # hkVector4d *
            ctypes.c_ulonglong,  # hkPreciseTransform *
            ctypes.c_ulonglong,  # hknpCdBody *
        ]
    ),
    "hknpFullCast::setupSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFullCast::Context *
            ctypes.c_ulonglong,  # const hknpManifoldStreamHeader *
            ctypes.c_ulonglong,  # hknpManifold *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpShapeCollector *
            ctypes.c_ulonglong,  # hknpShapeCollector *
            ctypes.c_ulonglong,  # hkPreciseTransform *
            ctypes.c_ulonglong,  # hknpCdBody *
            ctypes.c_ulonglong,  # hknpCdBody *
            ctypes.c_ulonglong,  # hknpMotion *
        ]
    ),
    "hknpFullCast::updateManifold": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hknpManifold *
        ]
    ),
    "hknpCollisionCacheSorter::sortCaches": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hknpCollisionCacheStreams *
        ]
    ),
    "hknpApplyDragTask::hknpApplyDragTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpApplyDragTask *
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "hknpApplyDragTask::doesWorldHaveDrag": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hknpWorldEx *
        ]
    ),
    "hknpApplyDragTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpApplyDragTask *
        ]
    ),
    "hknpApplyDragTask::process": {
        "hknpApplyDragTask *, hknpSimulationThreadContext *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpApplyDragTask *
                ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ]
        ),
        "hknpApplyDragTask *, const hkTask::Input *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpApplyDragTask *
                ctypes.c_ulonglong,  # const hkTask::Input *
            ]
        ),
    },
    "hkArrayBase__anonymous_namespace___CellInfo____growCapacity_hkBuiltinContainerAllocatorBase_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase__anonymous_namespace___CellInfo____growCapacity_hkContainerAllocator_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpSolverScheduler::createFullStepTask": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpSolverSchedulerTask *
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # const hknpSpaceSplitter *
            ctypes.c_ulonglong,  # hkArray<hknpSolverSchedulerTask,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpSolverScheduler::estimateNumTasks": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSpaceSplitter *
            ctypes.c_ulonglong,  # const hknpCsGrid *const *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpSolverSchedulerTask::getDebugAttributes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverSchedulerTask *
            ctypes.c_ulonglong,  # hkTask::DebugAttributes *
        ]
    ),
    "hknpSolverSchedulerTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverSchedulerTask *
        ]
    ),
    "hknpStepConstraintJacobianProcess": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # hknpJacobianGridType::Enum
            ctypes.c_int32,  # int
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const hknpSolverStepInfo *
            ctypes.c_ulonglong,  # hkMonitorStream *
        ]
    ),
    "hknpSolverSchedulerTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSolverSchedulerTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpPrepareNarrowPhaseTask::hknpPrepareNarrowPhaseTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPrepareNarrowPhaseTask *
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hknpMultithreadedSimulation *
        ]
    ),
    "hknpPrepareNarrowPhaseTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpPrepareNarrowPhaseTask *
        ]
    ),
    "hknpPrepareNarrowPhaseTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPrepareNarrowPhaseTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpPrepareNarrowPhasePairsTask::hknpPrepareNarrowPhasePairsTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPrepareNarrowPhasePairsTask *
            ctypes.c_ulonglong,  # hknpMultithreadedSimulation *
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hkTypedBlockStream<hknpBodyIndexPair> *
            ctypes.c_ulonglong,  # hknpConstraintAtomSolverSetup::SortedConstraintStates *
            ctypes.c_ulonglong,  # hknpConstraintAtomSolverSetup::ConstraintStateGroups *
            ctypes.c_ulonglong,  # hkHashSet<hknpBodyIndexPair,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpPrepareNarrowPhasePairsTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpPrepareNarrowPhasePairsTask *
        ]
    ),
    "hknpPrepareNarrowPhasePairsTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPrepareNarrowPhasePairsTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpPrepareNarrowPhasePairsTask::processImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPrepareNarrowPhasePairsTask *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
        ]
    ),
    "hknpBuildConstraintJacobiansTask::hknpBuildConstraintJacobiansTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBuildConstraintJacobiansTask *
            ctypes.c_uint32,  # hknpBuildConstraintJacobiansTask::InputType
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hknpMultithreadedSimulation *
            ctypes.c_ulonglong,  # const hknpConstraintAtomSolverSetup::ConstraintStateGroups *
        ]
    ),
    "hknpPrepareConstraintsTask::hknpPrepareConstraintsTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPrepareConstraintsTask *
            ctypes.c_ulonglong,  # hknpWorldEx *
            ctypes.c_ulonglong,  # hknpConstraintAtomSolverSetup::SortedConstraintStates *
            ctypes.c_ulonglong,  # hknpConstraintAtomSolverSetup::ConstraintStateGroups *
            ctypes.c_ulonglong,  # hkHashSet<hknpBodyIndexPair,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpBuildConstraintJacobiansTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpBuildConstraintJacobiansTask *
        ]
    ),
    "hknpPrepareConstraintsTask::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpPrepareConstraintsTask *
        ]
    ),
    "hknpBuildConstraintJacobiansTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBuildConstraintJacobiansTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpPrepareConstraintsTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPrepareConstraintsTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hkArrayBase<hknpSolverSchedulerTask>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpSolverSchedulerTask> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpPrepareSolverData::hknpPrepareSolverData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPrepareSolverData *
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hknpSolverData *
        ]
    ),
    "hknpPrepareSolverData::BuildTaskGraphHelper::~BuildTaskGraphHelper": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPrepareSolverData::BuildTaskGraphHelper *
        ]
    ),
    "hknpPrepareSolverData::initPreSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPrepareSolverData *
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hknpSolverData *
        ]
    ),
    "hkArrayBase<hknpProcessNarrowPhaseGridTask::ThreadOutput>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpProcessNarrowPhaseGridTask::ThreadOutput> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpProcessNarrowPhaseGridTask::ThreadOutput::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpProcessNarrowPhaseGridTask::ThreadOutput *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
        ]
    ),
    "hknpProcessNarrowPhaseGridTask::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpProcessNarrowPhaseGridTask *
            ctypes.c_ulonglong,  # hknpSimulationContext *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hknpSolverData *
            ctypes.c_ulonglong,  # hknpCollisionCacheManager *
        ]
    ),
    "hknpProcessNarrowPhaseGridTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpProcessNarrowPhaseGridTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hkFixedSizeBuffer<int,hkFixedStackStorage<int>>::hkFixedSizeBuffer<int,hkFixedStackStorage<int>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkFixedSizeBuffer<int,hkFixedStackStorage<int> > *
            ctypes.c_int32,  # int
        ]
    ),
    "gridEntryBuilder::gridTuple::gridTuple": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # gridEntryBuilder::gridTuple *
        ]
    ),
    "hknpFinalizeNarrowPhasePairsTaskEx::buildJacobiansForDeferedManifolds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFinalizeNarrowPhasePairsTaskEx *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
        ]
    ),
    "iterateBuildGrid": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hknpModifierSharedData *
            ctypes.c_ulonglong,  # hknpGrid<hkBlockStream::LinkedRange> *
            ctypes.c_ulonglong,  # hkBlockStream *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCsGrid *
            ctypes.c_ulonglong,  # hknpCsJacWriter *
            ctypes.c_ulonglong,  # hknpCsContactJacInjector *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # std::atomic<int> *
        ]
    ),
    "hknpFinalizeNarrowPhasePairsTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFinalizeNarrowPhasePairsTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpSortFinalizeNarrowPhasePairsTask::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSortFinalizeNarrowPhasePairsTask *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpContactImpulseClippedEventCreator::getEnabledFunctions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpContactImpulseClippedEventCreator *
        ]
    ),
    "hknpContactImpulseClippedEventCreator::manifoldCreatedCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpContactImpulseClippedEventCreator *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpModifier::ManifoldCreatedCallbackInput *
        ]
    ),
    "hknpContactImpulseClippedEventCreator::postContactImpulseClipped": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpContactImpulseClippedEventCreator *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifier::ContactSolverCallbackInput *
            ctypes.c_ulonglong,  # const float *
        ]
    ),
    "hknpContactImpulseEventCreator::manifoldCreatedCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpContactImpulseEventCreator *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpModifier::ManifoldCreatedCallbackInput *
        ]
    ),
    "hknpContactImpulseEventCreator::manifoldDestroyedCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpContactImpulseEventCreator *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_ulonglong,  # const hknpManifoldCollisionCache *
        ]
    ),
    "hknpContactImpulseEventCreator::postContactSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpContactImpulseEventCreator *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifier::ContactSolverCallbackInput *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
        ]
    ),
    "hknpMassChangerModifier::manifoldProcessCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMassChangerModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifold *
        ]
    ),
    "hknpTriggerModifier::getEnabledFunctions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpTriggerModifier *
        ]
    ),
    "hknpTriggerModifier::manifoldCreatedCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpTriggerModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpModifier::ManifoldCreatedCallbackInput *
        ]
    ),
    "hknpTriggerModifier::manifoldDestroyedCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpTriggerModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_ulonglong,  # const hknpManifoldCollisionCache *
        ]
    ),
    "hknpTriggerModifier::manifoldProcessCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpTriggerModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifold *
        ]
    ),
    "hknpTriggerModifier::postContactJacobianSetup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpTriggerModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # const hknpManifold *
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
        ]
    ),
    "hknpTriggerModifier::postContactSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpTriggerModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifier::ContactSolverCallbackInput *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "checkContactsForRestitution": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpManifoldCollisionCache *
        ]
    ),
    "hknpRestitutionModifier::manifoldCreatedCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRestitutionModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpModifier::ManifoldCreatedCallbackInput *
        ]
    ),
    "hknpRestitutionModifier::postContactJacobianReused": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRestitutionModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
        ]
    ),
    "hknpRestitutionModifier::postContactJacobianSetup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRestitutionModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # const hknpManifold *
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
        ]
    ),
    "hknpSoftContactModifier::getEnabledFunctions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpSoftContactModifier *
        ]
    ),
    "hknpSoftContactModifier::manifoldCreatedCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSoftContactModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpModifier::ManifoldCreatedCallbackInput *
        ]
    ),
    "hknpSoftContactModifier::manifoldFullCastCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSoftContactModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpModifier::ManifoldFullCastCallbackInput *
        ]
    ),
    "hknpSoftContactModifier::manifoldProcessCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSoftContactModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifold *
        ]
    ),
    "modifyCacheAndJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # const hknpSoftContactModifier::SoftContactProperties *
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
        ]
    ),
    "hknpSoftContactModifier::postContactJacobianReused": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSoftContactModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
        ]
    ),
    "hknpSoftContactModifier::postContactJacobianSetup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSoftContactModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # const hknpManifold *
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
        ]
    ),
    "applySurfaceVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
        ]
    ),
    "hknpAnimatedShapeSurfaceVelocityModifier::postContactJacobianReused": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpAnimatedShapeSurfaceVelocityModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
        ]
    ),
    "hknpSurfaceVelocityModifier::postContactJacobianReused": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSurfaceVelocityModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
        ]
    ),
    "hknpAnimatedShapeSurfaceVelocityModifier::postContactJacobianSetup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpAnimatedShapeSurfaceVelocityModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # const hknpManifold *
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
        ]
    ),
    "hknpSurfaceVelocityModifier::postContactJacobianSetup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSurfaceVelocityModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # const hknpManifold *
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
        ]
    ),
    "hknpPotentialManifoldEventCreator::potentialManifoldCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpPotentialManifoldEventCreator *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpModifier::PotentialManifoldCallbackInput *
        ]
    ),
    "hknpManifoldEventCreator::manifoldCreatedCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpManifoldEventCreator *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpModifier::ManifoldCreatedCallbackInput *
        ]
    ),
    "hknpManifoldEventCreator::manifoldDestroyedCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpManifoldEventCreator *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_ulonglong,  # const hknpManifoldCollisionCache *
        ]
    ),
    "hknpManifoldEventCreator::manifoldFullCastCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpManifoldEventCreator *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpModifier::ManifoldFullCastCallbackInput *
        ]
    ),
    "hknpManifoldEventCreator::manifoldProcessCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpManifoldEventCreator *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifold *
        ]
    ),
    "hknpManifoldEventCreator::postContactJacobianReused": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpManifoldEventCreator *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
        ]
    ),
    "hknpFlipBackfaceCollisionModifier::manifoldFullCastCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFlipBackfaceCollisionModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpModifier::ManifoldFullCastCallbackInput *
        ]
    ),
    "hknpFlipBackfaceCollisionModifier::manifoldProcessCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpFlipBackfaceCollisionModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifold *
        ]
    ),
    "hknpMotionWeldingModifier_getClosestPoint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkcdGsk::GetClosestPointInput *
            ctypes.c_ulonglong,  # hkcdGsk::Cache *
            ctypes.c_ulonglong,  # hkcdGsk::GetClosestPointOutput *
        ]
    ),
    "hknpMotionWeldingModifier_getSupportingVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkcdGsk::GetSupportingVertexOut *
        ]
    ),
    "hknpMotionWeldingModifier::postCompositeCollideCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpMotionWeldingModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpWeldingModifier::WeldingInfo *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hkBlockStream::Range *
        ]
    ),
    "hknpTriangleWeldingModifier::postCompositeCollideCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpTriangleWeldingModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpWeldingModifier::WeldingInfo *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hkBlockStream::Range *
        ]
    ),
    "hknpLiveWeldingModifier::weldManifolds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLiveWeldingModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpWeldingModifier::WeldingInfo *
            ctypes.c_ulonglong,  # hkArrayView<hknpManifoldHeaderPair> *
        ]
    ),
    "hknpCompositeCollisionDetector::copyChildCaches": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompositeCollisionDetector *
            ctypes.c_ulonglong,  # hknpCompositeCollisionCache *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
            ctypes.c_ulonglong,  # hkBlockStream::Writer *
        ]
    ),
    "hknpCompositeCollisionDetector::destructCollisionCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompositeCollisionDetector *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCollisionCache *
            ctypes.c_ulonglong,  # hknpCollisionCacheBatchConsumer *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
        ]
    ),
    "hknpCompositeCollisionDetector::moveAndConsumeChildCachesAndClearJacobianPointers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompositeCollisionDetector *
            ctypes.c_ulonglong,  # hknpCompositeCollisionCache *
            ctypes.c_ulonglong,  # hknpCollisionCacheBatchConsumer *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
        ]
    ),
    "hknpConvexCompositeCollisionDetector::collideConvexWithCompositeKeys": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpInternalCollideSharedData *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCompositeCollisionCache *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpCollisionShapeKeyArrayCache *
            ctypes.c_ulonglong,  # hknpCollisionCacheBatchConsumer *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
            ctypes.c_ulonglong,  # hknpManifoldBuffer *
            ctypes.c_ulonglong,  # hknpJacobianConsumer *
            ctypes.c_ulonglong,  # struct hknpCsContactJacInjector *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # hknpCvxCompositeKeysTimers *
            ctypes.c_ulonglong,  # hknpConvexConvexManifoldGenerator *
        ]
    ),
    "hknpConvexCompositeCollisionDetector::collideWithChildren": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexCompositeCollisionDetector *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpInternalCollideSharedData *
            ctypes.c_ulonglong,  # hknpCdBody *
            ctypes.c_ulonglong,  # hknpCdBody *
            ctypes.c_ulonglong,  # hknpCompositeCollisionCache *
            ctypes.c_ulonglong,  # hknpCollisionCacheBatchConsumer *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
            ctypes.c_ulonglong,  # hknpJacobianConsumer *
            ctypes.c_ulonglong,  # struct hknpCsContactJacInjector *
            ctypes.c_ulonglong,  # hknpManifoldBuffer *
            ctypes.c_ulonglong,  # hknpCollideTimers *
            ctypes.c_ulonglong,  # hknpConvexConvexManifoldGenerator *
        ]
    ),
    "hknpConvexCompositeCollisionDetector::getMaximumLinearTim": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
        ]
    ),
    "hknpConvexCompositeCollisionDetector_midphase": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpCollisionShapeKeyArrayCache *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpInternalCollideSharedData *
            ctypes.c_ulonglong,  # hknpCompositeCollisionCache *
            ctypes.c_ulonglong,  # hknpCdBody *
            ctypes.c_ulonglong,  # hknpCdBody *
            ctypes.c_uint32,  # hknpLevelOfDetail
            ctypes.c_ulonglong,  # hknpCollisionCacheBatchConsumer *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
            ctypes.c_ulonglong,  # hkInplaceArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,1024,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hknpCollideTimers *
        ]
    ),
    "hknpConvexCompositeCollisionDetector::queryAabbWithNmp": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkAabb *
            ctypes.c_ulonglong,  # hknpQueryAabbTimers *
        ]
    ),
    "hknpConvexCompositeCollisionDetector::reuseConvexWithComposite": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpInternalCollideSharedData *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpCdBody *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpCollisionCacheBatchConsumer *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
            ctypes.c_ulonglong,  # hknpManifoldBuffer *
            ctypes.c_ulonglong,  # hknpReuseContactCacheTimers *
        ]
    ),
    "hknpConvexConvexCollisionDetector::hknpConvexConvexCollisionDetector": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexConvexCollisionDetector *
        ]
    ),
    "hknpCapsuleCapsuleConvexConvexManifoldGenerator::collide": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpCapsuleCapsuleConvexConvexManifoldGenerator *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpConvexConvexManifoldGenerator::Input *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpGskCollisionCache *
            ctypes.c_ulonglong,  # hknpManifold *
            ctypes.c_ulonglong,  # hkMonitorStream::MultiTimerCommand **
        ]
    ),
    "hknpCapsuleCapsuleConvexConvexManifoldGenerator::collideImpl": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpConvexConvexManifoldGenerator::Input *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_ulonglong,  # hknpGskCollisionCache *
            ctypes.c_ulonglong,  # hknpManifold *
            ctypes.c_ulonglong,  # hkMonitorStream::MultiTimerCommand **
        ]
    ),
    "hknpConvexConvexCollisionDetector::destructCollisionCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexConvexCollisionDetector *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCollisionCache *
            ctypes.c_ulonglong,  # hknpCollisionCacheBatchConsumer *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
        ]
    ),
    "hknpCompositeCompositeCollisionDetector::collideWithChildren": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompositeCompositeCollisionDetector *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpInternalCollideSharedData *
            ctypes.c_ulonglong,  # hknpCdBody *
            ctypes.c_ulonglong,  # hknpCdBody *
            ctypes.c_ulonglong,  # hknpCompositeCollisionCache *
            ctypes.c_ulonglong,  # hknpCollisionCacheBatchConsumer *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
            ctypes.c_ulonglong,  # hknpJacobianConsumer *
            ctypes.c_ulonglong,  # struct hknpCsContactJacInjector *
            ctypes.c_ulonglong,  # hknpManifoldBuffer *
            ctypes.c_ulonglong,  # hknpCollideTimers *
            ctypes.c_ulonglong,  # hknpConvexConvexManifoldGenerator *
        ]
    ),
    "buildSdfJacobians": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # hknpCompositeCollisionCache *
            ctypes.c_ulonglong,  # hknpCdBody *
            ctypes.c_ulonglong,  # hknpCdBody *
            ctypes.c_ulonglong,  # hkTypedBlockStream<hknpShape::SdfContactPoint>::Reader *
            ctypes.c_ulonglong,  # hknpCollisionCacheBatchConsumer *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
            ctypes.c_ulonglong,  # hknpManifoldBuffer *
            ctypes.c_ulonglong,  # struct hknpCsContactJacInjector *
        ]
    ),
    "hknpSignedDistanceFieldCollisionDetector::collideWithChildren": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSignedDistanceFieldCollisionDetector *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpInternalCollideSharedData *
            ctypes.c_ulonglong,  # hknpCdBody *
            ctypes.c_ulonglong,  # hknpCdBody *
            ctypes.c_ulonglong,  # hknpCompositeCollisionCache *
            ctypes.c_ulonglong,  # hknpCollisionCacheBatchConsumer *
            ctypes.c_ulonglong,  # hknpCollisionCacheWriter *
            ctypes.c_ulonglong,  # hknpJacobianConsumer *
            ctypes.c_ulonglong,  # struct hknpCsContactJacInjector *
            ctypes.c_ulonglong,  # hknpManifoldBuffer *
            ctypes.c_ulonglong,  # hknpCollideTimers *
        ]
    ),
    "hknpBroadphaseTriggerCollisionDetector::collide": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBroadphaseTriggerCollisionDetector *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # hknpCdBody *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCdBody *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionCache *
        ]
    ),
    "hknpBroadphaseTriggerCollisionDetector::destructCollisionCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpBroadphaseTriggerCollisionDetector *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCollisionCache *
        ]
    ),
    "hknpConstraintForceExceededEventCreator::getEnabledFunctions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintForceExceededEventCreator *
        ]
    ),
    "hknpConstraintForceEventCreator::postConstraintExport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintForceEventCreator *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifier::ConstraintSolverCallbackInput *
            ctypes.c_ulonglong,  # hknpSolverResults *
        ]
    ),
    "hknpConstraintForceExceededEventCreator::postConstraintExport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintForceExceededEventCreator *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifier::ConstraintSolverCallbackInput *
            ctypes.c_ulonglong,  # hknpSolverResults *
        ]
    ),
    "hknpParticleEventCreator::getEnabledFunctions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleEventCreator *
        ]
    ),
    "hknpParticleEventCreator::postParticleBodyImpulseApplied": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleEventCreator *
            ctypes.c_ulonglong,  # const hknpParticleSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpParticlesCollider *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkArrayView<hknpParticleBodyContact> *
        ]
    ),
    "hknpParticleEventCreator::postParticleParticleImpulseApplied": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleEventCreator *
            ctypes.c_ulonglong,  # const hknpParticleSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpParticlesCollider *
            ctypes.c_ulonglong,  # const hknpParticleParticleContact *
        ]
    ),
    "hknpParticleEventCreator::postParticlesCollidedWithBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleEventCreator *
            ctypes.c_ulonglong,  # const hknpParticleSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpParticlesCollider *
            ctypes.c_ulonglong,  # const hkArrayView<int> *
        ]
    ),
    "hknpParticleEventCreator::postParticlesCollidedWithParticles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleEventCreator *
            ctypes.c_ulonglong,  # const hknpParticleSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpParticlesCollider *
            ctypes.c_ulonglong,  # const hkArrayView<hknpParticleCollisionPair> *
        ]
    ),
    "hknpParticleEventCreator::postParticlesExitedSafeRegion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpParticleEventCreator *
            ctypes.c_ulonglong,  # const hknpParticleSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpParticlesCollider *
            ctypes.c_ulonglong,  # const hkArrayView<int> *
        ]
    ),
    "hkArrayBase<hknpConstraintGroupDirectSolver::AtomLimitInfo>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpConstraintGroupDirectSolver::AtomLimitInfo> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpConstraintGroupDirectSolver::ConstraintFrameInformation>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpConstraintGroupDirectSolver::ConstraintFrameInformation> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpConstraintGroupDirectSolver::OneConstraintInformation>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpConstraintGroupDirectSolver::OneConstraintInformation> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpConstraintGroupDirectSolver::AtomLimitInfo>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpConstraintGroupDirectSolver::AtomLimitInfo> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpConstraintGroupDirectSolver::BodyPositionEstimationInformation>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpConstraintGroupDirectSolver::BodyPositionEstimationInformation> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpConstraintGroupDirectSolver::MotorSolverAtomInfo>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpConstraintGroupDirectSolver::MotorSolverAtomInfo> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpConstraintGroupDirectSolver::OneConstraintInformation>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpConstraintGroupDirectSolver::OneConstraintInformation> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpConstraintGroupDirectSolver::OneConstraintInformation::addAtom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::OneConstraintInformation *
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::SolverSetupData *
            ctypes.c_uint32,  # hknpConstraintAtom::AtomType
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
        ]
    ),
    "hknpConstraintGroupDirectSolver::OneConstraintInformation::addMotorInfoToTheLastAddedAtom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::OneConstraintInformation *
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::SolverSetupData *
            ctypes.c_ulonglong,  # const hknp1dConstraintMotorInfo *
            ctypes.c_ulonglong,  # const hknpConstraintAtom *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "hknpConstraintGroupDirectSolver::SolverSetupData::addNewConstraint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::OneConstraintInformation *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::SolverSetupData *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "buildJacobiansAndRHSFromAllConstraints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpSolverStep *
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::SolverSetupData *
            ctypes.c_ulonglong,  # const hknpSolverElemTemp *
            ctypes.c_ulonglong,  # const hknpSolverElemTemp *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "hknpConstraintGroupDirectSolver::SolverSetupData::clearCashedData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::SolverSetupData *
        ]
    ),
    "hknpDirectSolveConstraintGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpSolverStep *
            ctypes.c_ulonglong,  # const hknpSolverSumVelocity **
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::SolverSetupData *
            ctypes.c_ulonglong,  # hknpSolverVelocity **
            ctypes.c_ulonglong,  # const hknpSolverElemTemp *
        ]
    ),
    "hknpConstraintGroupDirectSolver::LimitsEnforcedForSolving::initInFirstStepMotor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::LimitsEnforcedForSolving *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpConstraintGroupDirectSolverSettings *
            ctypes.c_ulonglong,  # const hknpConstraintGroupDirectSolver::MotorSolverAtomInfo *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hknpConstraintGroupDirectSolver::CompressedAtomInformation::isMotor": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::CompressedAtomInformation *
        ]
    ),
    "hknpConstraintGroupDirectSolver::SolverSetupData::restartConstraintCollection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::SolverSetupData *
        ]
    ),
    "hknpAtomSolverFunctions__applyImpulses": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const hknp1Lin2AngJacobian *
            ctypes.c_ulonglong,  # const hknpSingleConstraintSolverSchema *
            ctypes.c_ulonglong,  # hknpSolverElemTemp *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
        ]
    ),
    "hknpAtomSolverFunctions__buildDenseMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # const hknp1Lin2AngJacobian *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpAtomSolverFunctions__buildJacobians": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknp1Lin2AngJacobian *
            ctypes.c_ulonglong,  # const hknpSingleConstraintSolverSchema *
            ctypes.c_ulonglong,  # hknpSolverElemTemp *
            ctypes.c_ulonglong,  # float *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "hknpAtomSolverFunctions__doFixedNumberSORIteration": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpAtomSolverFunctions__factorizeA": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "hknpAtomSolverFunctions__isSolutionOK": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const bool *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # const hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # const hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # const hknp1Lin2AngJacobian *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const unsigned __int8 *
        ]
    ),
    "hknpAtomSolverFunctions::solveDirectlySingleConstraint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpSingleConstraintSolverSchema *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # hknpSolverElemTemp *
        ]
    ),
    "hknpAtomSolverFunctions__solveLinearProblem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpAtomSolverFunctions__solveMLCProblem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_ulonglong,  # const hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # const hknpVelocityAccumulator *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const hknp1Lin2AngJacobian *
            ctypes.c_ulonglong,  # const hknpSingleConstraintSolverSchema *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpCircularSymmetry::getCircularSymmetryAxisPair": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpCdBody *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpShape **
        ]
    ),
    "hknpLiveWelding::LeafPair::LeafPair": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLiveWelding::LeafPair *
        ]
    ),
    "hknpLiveWelding::buildManifold": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hknpLiveWelding::Contact *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hknpConvexConvexManifoldCollisionCache *
            ctypes.c_ulonglong,  # hknpManifold *
        ]
    ),
    "hknpLiveWelding::buildWeldedContact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConvexVertexShape *
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpLiveWelding::Contact *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hknpLiveWelding::castPair": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConvexVertexShape *
            ctypes.c_ulonglong,  # const hknpConvexVertexShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkcdGsk::Cache *
            ctypes.c_ulonglong,  # hknpLiveWelding::Contact *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpLiveWelding::collidePair": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hknpLiveWelding::Contact *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpConvexConvexManifoldCollisionCache *
            ctypes.c_ulonglong,  # hknpManifold *
        ]
    ),
    "hknpLiveWelding::drawLeaf": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkPreciseTransform *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdGsk::ShapeInterface::getPlanes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # IConvexOverlapImpl::ShapeInterface *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkVector4f **
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # const hkVector4f **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hknpGsk2DShapeInterface::getPlanes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGsk2DShapeInterface *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkVector4f **
        ]
    ),
    "hknpGsk2DShapeInterface::getShapeAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGsk2DShapeInterface *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hkcdGsk_Float3Shape::getSupportingVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdGsk_Float3Shape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkcdVertex *
        ]
    ),
    "hknpGsk2DShapeInterface::getSupportingVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpGsk2DShapeInterface *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkcdGsk::GetSupportingVertexOut *
        ]
    ),
    "hknpLiveWelding::handlePenetration": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpLiveWelding::LeafPair *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hknpLiveWelding::Contact *
            ctypes.c_ulonglong,  # hkcdGsk::Cache *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpLiveWelding::NeighborData::initSource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLiveWelding::NeighborData *
            ctypes.c_ulonglong,  # const hknpLiveWelding::LeafPair *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpLiveWelding::initTimJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpManifold *
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCsJacWriter *
        ]
    ),
    "hknpLiveWelding::neighborWeld": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkStridedArrayView<hknpLiveWelding::NeighborData> *
            ctypes.c_ulonglong,  # hkArray<int,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hknpConvexVertexShape *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # hkArray<int,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpLiveWelding::neighborWeldPair": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpLiveWelding::WeldResult *
        argtypes=[
            ctypes.c_ulonglong,  # hknpLiveWelding::WeldResult *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkcdGsk::Cache *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hknpWeldFeature *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hknpLiveWelding::reuseTimJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
        ]
    ),
    "hknpLiveWelding::LeafPair::setPair": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLiveWelding::LeafPair *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpBody *
            ctypes.c_ulonglong,  # hknpConvexConvexManifoldCollisionCache *
        ]
    ),
    "hknpLiveWelding::LeafPair::setPreviousDisplacement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLiveWelding::LeafPair *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpLiveWelding::solveTimJacobian<1>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_ulonglong,  # const hknpLiveWelding::LeafPair *
            ctypes.c_ulonglong,  # const hkVector4d *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
        ]
    ),
    "hknpLiveWelding::solveTimJacobian<0>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const hknpStreamContactJacobian *
            ctypes.c_ulonglong,  # const hknpLiveWelding::LeafPair *
            ctypes.c_ulonglong,  # const hkVector4d *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
        ]
    ),
    "NpJacobianBuilder__buildSchemaFromBallSocketAtomNewPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpBallSocketConstraintAtom *
            ctypes.c_ulonglong,  # const hknpConstraintQueryIn *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpConstraintQueryOut *
        ]
    ),
    "hknpSolverBuildJacobianFromAtomsNotContact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintAtom *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkTransformf *
            ctypes.c_ulonglong,  # hkTransformf *
            ctypes.c_ulonglong,  # hknpConstraintAtomSolverSetup::MotorsInfoOneConstraint *
            ctypes.c_ulonglong,  # const hknpConstraintQueryIn *
            ctypes.c_ulonglong,  # hknpConstraintQueryOut *
            ctypes.c_ulonglong,  # const hknpConstraint *
            ctypes.c_ulonglong,  # const hkAabb *
        ]
    ),
    "hknpLandscapeQuadProvider::commitLocalChanges": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpLandscapeQuadProvider *
        ]
    ),
    "hkThreadLocalTimeStampHash<unsigned __int64,hknpLandscapeQuadProvider::HashValue>::commitLocalChangesNotThreadSafe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkThreadLocalTimeStampHash<unsigned __int64,hknpLandscapeQuadProvider::HashValue> *
        ]
    ),
    "hknpConstraintGroupDirectSolver::SolverSetupData::setReferencePoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::SolverSetupData *
            ctypes.c_ulonglong,  # const hkVector4d *
        ]
    ),
    "hknpNarrowPhaseData::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpNarrowPhaseData *
            ctypes.c_ulonglong,  # hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hknpSimulationContext *
        ]
    ),
    "hknpNarrowPhaseWriters::~hknpNarrowPhaseWriters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpNarrowPhaseWriters *
        ]
    ),
    "hknpNarrowPhaseWriters::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpNarrowPhaseWriters *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hknpSolverData::ThreadData *
            ctypes.c_ulonglong,  # hknpProcessNarrowPhasePairsTask::ThreadOutput *
        ]
    ),
    "hknpWeldingUtil::applyModifiedNormal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hkcdManifold4 *
        ]
    ),
    "hknpWeldingUtil::motionWeldCvxVsCvx": FUNCDEF(
        restype=ctypes.c_ulonglong,  # {'return': 'hkcdGsk::GetClosestPointStatus', 'args': ['const void *', 'const void *', 'const hkcdGsk::GetClosestPointInput *', 'hkcdGsk::Cache *', 'hkcdGsk::GetClosestPointOutput *']}
        argtypes=[
            ctypes.c_ulonglong,  # {'return': 'hkcdGsk::GetClosestPointStatus', 'args': ['const void *', 'const void *', 'const hkcdGsk::GetClosestPointInput *', 'hkcdGsk::Cache *', 'hkcdGsk::GetClosestPointOutput *']}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['const void *', 'const hkVector4f *', 'const void *', 'const hkTransformf *', 'hkcdGsk::GetSupportingVertexOut *']}
            ctypes.c_ulonglong,  # const hknpWeldingUtil::MotionWeldConfig *
            ctypes.c_ulonglong,  # const hkcdGsk::Cache *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "hknpNeighborWeldingUtil::neighborWeld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpBody *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # hknpNeighborWeldingUtil::ManifoldData *
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # const hknpBodyIntegrator *
        ]
    ),
    "getSignedDistances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpSignedDistanceFieldChildCollisionCache *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSignedDistanceFieldChildCollisionCache *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hkcdGsk::GetClosestPointOutput *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpMotionWeldingUtil::motionWeldCvxVsSdf": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpCdBody *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpWeldingUtil::MotionWeldConfig *
            ctypes.c_ulonglong,  # const hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "hknpMotionWeldingUtil::setupMotionWeldConfigBodyA": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpWeldingConfig *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpWeldingConfig *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpWeldingUtil::MotionWeldConfig *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "hknpMotionWeldingUtil::setupMotionWeldConfigBodyB": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpManifold *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hknpWeldingUtil::MotionWeldConfig *
        ]
    ),
    "hkArrayBase<hknpConstraintGroupDirectSolver::DiagonalBlockInfoInput>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpConstraintGroupDirectSolver::DiagonalBlockInfoInput> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hknpConstraintGroupDirectSolver::BaseBlockSquaredMatrix::~BaseBlockSquaredMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::BaseBlockSquaredMatrix *
        ]
    ),
    "hknpConstraintGroupDirectSolver::DenselockSquaredMatrix::allocateMatrixEntries": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::DenselockSquaredMatrix *
            ctypes.c_ulonglong,  # const hkArray<hknpConstraintGroupDirectSolver::DiagonalBlockInfoInput,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # const hkArray<hknpConstraintGroupDirectSolver::OffDiagonalBlockInfoInput,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpConstraintGroupDirectSolver::SparseSquaredMatrixWithLUFillIns::allocateMatrixEntries": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::SparseSquaredMatrixWithLUFillIns *
            ctypes.c_ulonglong,  # const hkArray<hknpConstraintGroupDirectSolver::DiagonalBlockInfoInput,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # const hkArray<hknpConstraintGroupDirectSolver::OffDiagonalBlockInfoInput,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpConstraintGroupDirectSolver::buildMatrixWithLimitsAndStoreJacobians": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::SolverSetupData *
            ctypes.c_float,  # float
            ctypes.c_int32,  # const int
        ]
    ),
    "hknpConstraintGroupDirectSolver::DenselockSquaredMatrix::doLUDecomposition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::DenselockSquaredMatrix *
        ]
    ),
    "hknpConstraintGroupDirectSolver::SparseSquaredMatrixWithLUFillIns::doLUDecomposition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::SparseSquaredMatrixWithLUFillIns *
        ]
    ),
    "hknpConstraintGroupDirectSolver::initMatrixFromConstraintGroup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::SolverSetupData *
        argtypes=[
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::SolverSetupData *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpConstraintGroupDirectSolver::predictActiveStateForLimit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # const bool
            ctypes.c_int32,  # const int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hknpConstraintGroupDirectSolver::LimitsEnforcedForSolving *
        ]
    ),
    "hknp1dLinearVelocityMotorBeginJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hknpConstraintQueryIn *
            ctypes.c_ulonglong,  # hknp1Lin2AngJacobian *
            ctypes.c_ulonglong,  # hknp1dBilateralConstraintStatus *
        ]
    ),
    "hknp1dLinearVelocityMotorCommitJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknp1dConstraintMotorInfo *
            ctypes.c_ulonglong,  # const hknpConstraintQueryIn *
            ctypes.c_ulonglong,  # hknp1Lin2AngJacobian *
            ctypes.c_ulonglong,  # hknpConstraintQueryOut *
        ]
    ),
    "hknpCalcMotorData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintMotor *
            ctypes.c_ulonglong,  # const hknpConstraintMotorInput *
            ctypes.c_ulonglong,  # hknpConstraintMotorOutput *
        ]
    ),
    "hknpSingleConstraintDirectSolverSchemaSetup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConstraintAtom *
            ctypes.c_ulonglong,  # const hknpConstraintAtom *
            ctypes.c_ulonglong,  # const hknpConstraintQueryIn *
            ctypes.c_ulonglong,  # hknpConstraintQueryOut *
            ctypes.c_ulonglong,  # const hknpConstraint *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "cTkDoubleBufferMemoryPool::cTkDoubleBufferMemoryPool": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDoubleBufferMemoryPool *
        ]
    ),
    "cTkLocklessMultiPoolAllocator::Alloc": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocklessMultiPoolAllocator *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkLocklessSlotAlloc::Alloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocklessSlotAlloc *
        ]
    ),
    "cTkDynamicMemoryPool::AllocLargeBlockPages": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicMemoryPool *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkLocklessMultiPoolAllocator::CalcRequiredSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkLocklessMultiPoolAllocator::sPoolConfig *
        ]
    ),
    "cTkMemoryManager::Calloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMemoryManager *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkSmallBlockMemoryPool::Cleanup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmallBlockMemoryPool *
        ]
    ),
    "cTkLocklessSlotAlloc::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocklessSlotAlloc *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMemoryManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMemoryManager *
            ctypes.c_ulonglong,  # const cTkMemoryPoolDefinition *
            ctypes.c_ulonglong,  # const cTkMemoryBackingPoolDefinition *
            ctypes.c_ulonglong,  # const cTkMemoryPoolDefinition *
            ctypes.c_ulonglong,  # const cTkMemoryBackingPoolDefinition *
            ctypes.c_ulonglong,  # const cTkMemoryBackingPoolDefinition *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDoubleBufferMemoryPool::ContainsAddress": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkDoubleBufferMemoryPool *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkDynamicMemoryPool::ContainsAddress": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicMemoryPool *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkSingleBufferMemoryPool::ContainsAddress": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkSingleBufferMemoryPool *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkSmallBlockMemoryPool::ContainsAddress": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmallBlockMemoryPool *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkAtomicLinearMemoryPool::Create": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAtomicLinearMemoryPool *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDoubleBufferMemoryPool::Create": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDoubleBufferMemoryPool *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicMemoryPool::Create": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicMemoryPool *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cTkLinearMemoryPool::Create": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearMemoryPool *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cTkSingleBufferMemoryPool::Create": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSingleBufferMemoryPool *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cTkSmallBlockMemoryPool::Create": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmallBlockMemoryPool *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "CreateLocalAllocator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # ELocalAllocatorSize
        ]
    ),
    "DestroyLocalAllocator": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "DestroyLocalAllocatorAtEndOfThread": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkLocklessSlotAlloc::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocklessSlotAlloc *
        ]
    ),
    "DisableMainLocalAllocator": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkDynamicMemoryPool::DoAllocation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicMemoryPool *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "EnableMainLocalAllocator": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkSmallBlockMemoryPool::FallbackMalloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmallBlockMemoryPool *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkDoubleBufferMemoryPool::FrameSyncEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDoubleBufferMemoryPool *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMemoryManager::FrameSyncEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMemoryManager *
        ]
    ),
    "cTkSingleBufferMemoryPool::FrameSyncEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSingleBufferMemoryPool *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkDynamicMemoryPool::Free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicMemoryPool *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkLocklessMultiPoolAllocator::Free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocklessMultiPoolAllocator *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkLocklessSlotAlloc::Free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocklessSlotAlloc *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkMemoryManager::Free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMemoryManager *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkSmallBlockMemoryPool::Free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmallBlockMemoryPool *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkDynamicMemoryPool::FreeLargeBlockPages": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicMemoryPool *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkDynamicMemoryPool::GetAllocInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicMemoryPool *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkSmallBlockMemoryPool::GetAllocInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmallBlockMemoryPool *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkLocklessMultiPoolAllocator::GetAllocSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocklessMultiPoolAllocator *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkDynamicMemoryPool::GetAllocationSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicMemoryPool *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkMemoryManager::GetAllocationSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMemoryManager *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkSmallBlockMemoryPool::GetAllocationSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmallBlockMemoryPool *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkDoubleBufferMemoryPool::GetBaseAddress": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # cTkDoubleBufferMemoryPool *
        ]
    ),
    "cTkDoubleBufferMemoryPool::GetHighWaterUsage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDoubleBufferMemoryPool *
        ]
    ),
    "cTkDynamicMemoryPool::GetHighWaterUsage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicMemoryPool *
        ]
    ),
    "cTkSingleBufferMemoryPool::GetHighWaterUsage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSingleBufferMemoryPool *
        ]
    ),
    "cTkSmallBlockMemoryPool::GetHighWaterUsage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmallBlockMemoryPool *
        ]
    ),
    "GetLocalAlloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkLinearMemoryPool *
        argtypes=[
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cTkSmallBlockMemoryPool::GetOverflowPool": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSmallBlockMemoryPool *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmallBlockMemoryPool *
        ]
    ),
    "cTkDoubleBufferMemoryPool::GetSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDoubleBufferMemoryPool *
        ]
    ),
    "cTkDoubleBufferMemoryPool::GetUsage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDoubleBufferMemoryPool *
        ]
    ),
    "cTkDynamicMemoryPool::GetUsage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicMemoryPool *
        ]
    ),
    "cTkSingleBufferMemoryPool::GetUsage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSingleBufferMemoryPool *
        ]
    ),
    "cTkSmallBlockMemoryPool::GetUsage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmallBlockMemoryPool *
        ]
    ),
    "cTkMemoryManager::InPausableArea": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkMemoryManager *
        ]
    ),
    "cTkLocklessMultiPoolAllocator::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLocklessMultiPoolAllocator *
            ctypes.c_ulonglong,  # const cTkLocklessMultiPoolAllocator::sPoolConfig *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
        ]
    ),
    "InitLocalAllocPerThreadData": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "IsUsingMainLocalAllocator": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "cTkAtomicLinearMemoryPool::Malloc": {
        "cTkAtomicLinearMemoryPool *, int, int, int *": FUNCDEF(
            restype=ctypes.c_uint64,  # unsigned __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkAtomicLinearMemoryPool *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # int *
            ]
        ),
        "cTkSmallBlockMemoryPool *, __int64, int *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # void *
            argtypes=[
                ctypes.c_ulonglong,  # cTkSmallBlockMemoryPool *
                ctypes.c_int64,  # __int64
                ctypes.c_ulonglong,  # int *
            ]
        ),
    },
    "cTkDoubleBufferMemoryPool::Malloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # cTkDoubleBufferMemoryPool *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkDynamicMemoryPool::Malloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicMemoryPool *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkLinearMemoryPool::Malloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearMemoryPool *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkMemoryManager::Malloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMemoryManager *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkSingleBufferMemoryPool::Malloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSingleBufferMemoryPool *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkSmallBlockMemoryPool::Malloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmallBlockMemoryPool *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkDoubleBufferMemoryPool::NeedsMutex": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkDoubleBufferMemoryPool *
        ]
    ),
    "PopLocalAllocator": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "PushLocalAllocator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearMemoryPool *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkLinearMemoryPool::Realloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearMemoryPool *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMemoryManager::Realloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMemoryManager *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAtomicLinearMemoryPool::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAtomicLinearMemoryPool *
        ]
    ),
    "cTkDoubleBufferMemoryPool::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDoubleBufferMemoryPool *
        ]
    ),
    "cTkLinearMemoryPool::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearMemoryPool *
        ]
    ),
    "cTkSingleBufferMemoryPool::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSingleBufferMemoryPool *
        ]
    ),
    "cTkLinearMemoryPool::ResetToAddress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearMemoryPool *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkDoubleBufferMemoryPool::SetAlignment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDoubleBufferMemoryPool *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkSingleBufferMemoryPool::SetAlignment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSingleBufferMemoryPool *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicMemoryPool::SetAllocInfo": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicMemoryPool *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkSmallBlockMemoryPool::SetAllocInfo": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmallBlockMemoryPool *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkMemoryManager::SetInPausableArea": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMemoryManager *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDoubleBufferMemoryPool::SetMutex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDoubleBufferMemoryPool *
            ctypes.c_ulonglong,  # cTkCSMutex *
        ]
    ),
    "cTkSingleBufferMemoryPool::SetMutex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSingleBufferMemoryPool *
            ctypes.c_ulonglong,  # cTkCSMutex *
        ]
    ),
    "cTkSmallBlockMemoryPool::SetMutex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmallBlockMemoryPool *
            ctypes.c_ulonglong,  # cTkCSMutex *
        ]
    ),
    "cTkRotationComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRotationComponent *
        ]
    ),
    "cTkRotationComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRotationComponent *
        ]
    ),
    "cTkRotationComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRotationComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cTkRotationComponent::GetUpdatePeriod": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkRotationComponent *
        ]
    ),
    "cTkRotationComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRotationComponent *
        ]
    ),
    "cTkRandomComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRandomComponent *
        ]
    ),
    "cTkRandomComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRandomComponent *
        ]
    ),
    "cTkAudioComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioComponent *
        ]
    ),
    "cTkAudioComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioComponent *
        ]
    ),
    "cTkAudioComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cTkAudioComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioComponent *
        ]
    ),
    "cTkSketchComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "cTkSketchComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "cTkSketchComponent::Receive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkSketchComponent::GetUpdatePeriod": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "cTkSketchComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cTkSketchComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "cTkGravityComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGravityComponent *
        ]
    ),
    "cTkGravityComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGravityComponent *
        ]
    ),
    "cTkGravityComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGravityComponent *
        ]
    ),
    "cTkLODComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODComponent *
        ]
    ),
    "cTkLODComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODComponent *
        ]
    ),
    "cTkLODComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODComponent *
        ]
    ),
    "cTkLODComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cTkProceduralModelComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralModelComponent *
        ]
    ),
    "cTkProceduralModelComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralModelComponent *
        ]
    ),
    "cTkProceduralModelComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralModelComponent *
        ]
    ),
    "cTkProceduralModelComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProceduralModelComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercTkProceduralModelComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercTkProceduralModelComponent *
        ]
    ),
    "cTkMetaMessageMap<2>::Broadcast<cGcMessageProjectileLaunch>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetaMessageMap<2> *
            ctypes.c_ulonglong,  # const cGcMessageProjectileLaunch *
        ]
    ),
    "cTkLSystemManager::CachedLSystem::CachedLSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager::CachedLSystem *
            ctypes.c_ulonglong,  # const cTkLSystemManager::CachedLSystem *
        ]
    ),
    "cTkLSystemManager::LSystem::LSystem": {
        "cTkLSystemManager::LSystem *, cTkLSystemManager::LSystem *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkLSystemManager::LSystem *
                ctypes.c_ulonglong,  # cTkLSystemManager::LSystem *
            ]
        ),
        "cTkLSystemManager::LSystem *, const cTkLSystemManager::LSystem *, __int64, __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkLSystemManager::LSystem *
                ctypes.c_ulonglong,  # const cTkLSystemManager::LSystem *
                ctypes.c_int64,  # __int64
                ctypes.c_int64,  # __int64
            ]
        ),
        "cTkLSystemManager::LSystem *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkLSystemManager::LSystem *
            ]
        ),
    },
    "cTkLSystemManager::LSystemEntry::LSystemEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager::LSystemEntry *
            ctypes.c_ulonglong,  # const cTkLSystemManager::LSystemEntry *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkLSystemManager::LSystemGlobalRestriction::LSystemGlobalRestriction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager::LSystemGlobalRestriction *
            ctypes.c_ulonglong,  # const cTkLSystemManager::LSystemGlobalRestriction *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkLSystemManager::LSystemInnerRule::LSystemInnerRule": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager::LSystemInnerRule *
            ctypes.c_ulonglong,  # const cTkLSystemManager::LSystemInnerRule *
        ]
    ),
    "cTkLSystemManager::LSystemRule::LSystemRule": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager::LSystemRule *
            ctypes.c_ulonglong,  # const cTkLSystemManager::LSystemRule *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkAttachmentPtr::cTkAttachmentPtr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachmentPtr *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkComponentInstancerBase::cTkComponentInstancerBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentInstancerBase *
        ]
    ),
    "cTkComponentLoadInstancerBase::cTkComponentLoadInstancerBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentLoadInstancerBase *
        ]
    ),
    "cTkNGuiNodeGraph::cTkNGuiNodeGraph": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "cTkSketchComponent::sBranch::sBranch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sBranch *
            ctypes.c_ulonglong,  # cTkSketchComponent::sPendingBranch *
        ]
    ),
    "cTkGravityComponent::~cTkGravityComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGravityComponent *
        ]
    ),
    "cTkLODComponent::~cTkLODComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODComponent *
        ]
    ),
    "cTkSketchComponent::~cTkSketchComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "cTkAttachmentManager::AddAttachment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkAttachmentData *
        ]
    ),
    "cTkAttachmentManager::AddComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkComponent::AddComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkLSystemManager::AddLSystem": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # cTkLSystemManager::LoadType
            ctypes.c_ulonglong,  # const cGcPlanetColourData *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cTkAttachmentManager::AddNodeLookup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachmentManager *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkSketchComponent::AddPendingBranches": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "cTkLinearHashTable<TkHandle,std::vector<cTkAttachment *,TkSTLAllocatorShim<cTkAttachment *,8,-1>>,-1,0,cTkLinearHashTableHash<TkHandle>>::AllocateNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkLinearHashTable<TkHandle,std::vector<cTkAttachment *,TkSTLAllocatorShim<cTkAttachment *,8,-1> >,-1,0,cTkLinearHashTableHash<TkHandle> >::cTkListNode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<TkHandle,std::vector<cTkAttachment *,TkSTLAllocatorShim<cTkAttachment *,8,-1> >,-1,0,cTkLinearHashTableHash<TkHandle> > *
            ctypes.c_ulonglong,  # const TkHandle *
            ctypes.c_ulonglong,  # const std::vector<cTkAttachment *,TkSTLAllocatorShim<cTkAttachment *,8,-1> > *
        ]
    ),
    "cTkSketchComponent::sNode::AnimateOutputLinks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkLSystemManager::AreResourcesAwaitingLoad": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
        ]
    ),
    "cTkSketchComponent::BroadcastValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # const sSketchValue *
        ]
    ),
    "ColourPickerCallbackNoAlpha": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkComponentManager::CompareComponentPools": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cTkAttachmentManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachmentManager *
        ]
    ),
    "cTkComponentManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentManager *
        ]
    ),
    "cTkLSystemManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
        ]
    ),
    "ComponentInstancercTkProceduralModelComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkLSystemManager::CreateAtLeastRestrictionChain": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkSketchComponent::DestructNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkSketchComponent::sNodeType::DoAddButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNodeType *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "cTkSketchComponent::sNode::DoEditBool": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cTkSketchComponent::sNode::DoEditEnum": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char **
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkSketchComponent::sNode::DoEditFloat32": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cTkSketchComponent::sNode::DoEditInt32": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkSketchComponent::sNode::DoEditNodeName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkSketchComponent::sNode::DoEditSketchCurve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # sSketchCurve *
        ]
    ),
    "cTkSketchComponent::sNode::DoEditSketchValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "cTkSketchComponent::sNode::DoEditString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # char *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkSketchComponent::sNode::DoEditUInt32": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "cTkSketchComponent::sNode::DoErrorMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkSketchComponent::DoGroupSeparator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkAttachmentManager::Free": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachmentManager *
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkComponentManager::Free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentManager *
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkComponentPool::Free": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentPool *
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkLSystemManager::Generate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLSystemManager::GenerateLSystemSampleByID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cTkLSystemManager::GenerateProceduralLSystem": {
        "cTkLSystemManager *, TkHandle, int, const cTkSeed *, const cTkMatrix34 *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkLSystemManager *
                ctypes.c_uint32,  # TkHandle
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # const cTkSeed *
                ctypes.c_ulonglong,  # const cTkMatrix34 *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cTkLSystemManager *, TkHandle, cTkLSystemRulesData *, const cTkSeed *, const cTkMatrix34 *, cTkLSystemManager::LoadType, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkLSystemManager *
                ctypes.c_uint32,  # TkHandle
                ctypes.c_ulonglong,  # cTkLSystemRulesData *
                ctypes.c_ulonglong,  # const cTkSeed *
                ctypes.c_ulonglong,  # const cTkMatrix34 *
                ctypes.c_uint32,  # cTkLSystemManager::LoadType
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cTkLSystemManager::GenerateStep": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkSeed *
        ]
    ),
    "cTkLSystemManager::GenerateVariants": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
        ]
    ),
    "cTkComponent::GetComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkSketchComponent::sNode::GetCurrentContext": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        ]
    ),
    "cTkComponent::GetEntityOwnerPlayerId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
            ctypes.c_ulonglong,  # cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cTkMetadataCache<cTkLSystemRulesData>::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetadataCache<cTkLSystemRulesData> *
        argtypes=[]
    ),
    "cTkLODComponent::GetLODForDistance": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODComponent *
            ctypes.c_float,  # float
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cTkLSystemManager::GetLSystemSampleAABBByID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkAABB *
        ]
    ),
    "cTkLSystemManager::GetLSystemSampleSizeByID": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkLSystemManager::GetLocatorNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
            ctypes.c_uint32,  # cTkLSystemManager::LocatorType
        ]
    ),
    "cTkLSystemManager::GetLocatorType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkComponent::GetMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cTkComponent::GetNetReplicateComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkComponent::GetNetworkId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkNetworkId *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkSketchComponent::GetNextSketchVisibleNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkLSystemManager::LSystemRule::GetResHandle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSmartResHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager::LSystemRule *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cTkRotationComponent::GetTypedComponent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRotationComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkSketchComponent::GetVariableValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sSketchValue *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # sSketchValue *
            ctypes.c_ulonglong,  # const sSketchValue *
        ]
    ),
    "cTkLSystemManager::GetVariantAmount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        ]
    ),
    "cTkComponent::HasComponent": FUNCDEF(
        restype=ctypes.c_uint8,  # unsigned __int8
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkLSystemManager::IncreaseNodeRuleCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
        ]
    ),
    "cTkComponent::IsNetReplicated": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkComponentManager::IsRegisteredComponent": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentManager *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetadataCache<cTkLSystemRulesData>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkLSystemRulesData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cTkLSystemRulesData> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSketchComponent::sNode::LoadFromMetadata": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
            ctypes.c_ulonglong,  # const cTkSketchComponent *
            ctypes.c_ulonglong,  # const cTkSketchNodeData *
        ]
    ),
    "cTkLSystemManager::LoadLSystemSamples": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkLSystemManager::LoadRuleSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # cTkLSystemManager::LoadType
            ctypes.c_ulonglong,  # const cGcPlanetColourData *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cTkLSystemManager::MakeInnerRule": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_ulonglong,  # const cTkLSystemInnerRule *
            ctypes.c_ulonglong,  # cTkLSystemManager::LSystemInnerRule *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # cTkLSystemManager::LoadType
            ctypes.c_ulonglong,  # const cGcPlanetColourData *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cTkLSystemManager::MakeLocatorEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_ulonglong,  # const cTkLSystemLocatorEntry *
            ctypes.c_ulonglong,  # cTkLSystemManager::LSystemEntry *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # cTkLSystemManager::LoadType
            ctypes.c_ulonglong,  # const cGcPlanetColourData *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cTkLSystemManager::MakeRule": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_ulonglong,  # const cTkLSystemRule *
            ctypes.c_ulonglong,  # cTkLSystemManager::LSystemRule *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # cTkLSystemManager::LoadType
            ctypes.c_ulonglong,  # const cGcPlanetColourData *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cTkSketchComponent::MatchNodeName": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkLSystemManager::MergeRuleSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_ulonglong,  # const cTkLSystemRulesData *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_uint32,  # cTkLSystemManager::LoadType
            ctypes.c_uint32,  # cTkLSystemManager::MergeType
            ctypes.c_ulonglong,  # const cGcPlanetColourData *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cTkAudioComponent::Mute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioComponent *
        ]
    ),
    "cTkSketchComponent::OnVariableValueChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # const sSketchValue *
        ]
    ),
    "cTkSketchComponent::PollSurveyVariableValue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # sSketchValue *
        ]
    ),
    "cTkLSystemManager::PrepareGeneration": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cTkLSystemManager::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # StackAllocator<cTkSmartResHandle,4,-1> *
        ]
    ),
    "cTkSketchComponent::Reload": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "cTkAttachmentManager::RemoveAttachment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkAttachmentManager::RemoveComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachment *
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkAttachmentManager::RemoveNodeLookup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachmentManager *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkAttachmentManager::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachmentManager *
        ]
    ),
    "cTkComponentManager::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentManager *
        ]
    ),
    "cTkLSystemManager::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
        ]
    ),
    "cTkSketchComponent::RenderNGuiAddNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "cTkLSystemManager::RenderNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkSketchComponent::RenderNGuiEditor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "cTkSketchComponent::RenderNGuiVariables": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "cTkSketchComponent::sNode::RenderNGuiVisualDebugInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
        ]
    ),
    "cTkSketchComponent::sNode::RenderVisualDebugLastOutput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cTkSketchComponent::Save": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "cTkSketchComponent::sNode::SaveToMetadata": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
            ctypes.c_ulonglong,  # const cTkSketchComponent *
            ctypes.c_ulonglong,  # cTkSketchNodeData *
        ]
    ),
    "cTkLSystemManager::SelectEntry": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_ulonglong,  # cTkLSystemManager::LSystemInnerRule *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkGravityComponent::SetEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGravityComponent *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSketchComponent::SetVariableValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkSketchComponent::sNode::SetVisualDebugProgress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sNode *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "cTkAudioComponent::Shutdown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioComponent *
        ]
    ),
    "cTkComponentManager::SortComponentPools": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentManager *
        ]
    ),
    "cTkSketchComponent::SurveyVariableValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent *
            ctypes.c_ulonglong,  # const sSketchValue *
        ]
    ),
    "cTkLODComponent::SwitchToLOD": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLODComponent *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cTkAudioComponent::Unmute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioComponent *
        ]
    ),
    "cTkComponentManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentManager *
            ctypes.c_float,  # float
        ]
    ),
    "cTkLSystemManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
            ctypes.c_float,  # float
        ]
    ),
    "cTkSketchComponent::sBranch::Update": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkSketchComponent::sBranch *
            ctypes.c_ulonglong,  # cTkSketchComponent *
        ]
    ),
    "cTkAudioComponent::UpdateDistanceCulling": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioComponent *
        ]
    ),
    "cTkComponentManager::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentManager *
            ctypes.c_float,  # float
        ]
    ),
    "cTkComponentManager::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponentManager *
            ctypes.c_float,  # float
        ]
    ),
    "cTkLSystemManager::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLSystemManager *
        ]
    ),
    "cTkSpaceshipReportCollisionCallback::manifoldCreatedCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSpaceshipReportCollisionCallback *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpModifier::ManifoldCreatedCallbackInput *
        ]
    ),
    "cTkVehicleReportDestructionCallback::getEnabledFunctions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkVehicleReportDestructionCallback *
        ]
    ),
    "cTkVehicleReportDestructionCallback::manifoldCreatedCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVehicleReportDestructionCallback *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpModifier::ManifoldCreatedCallbackInput *
        ]
    ),
    "cTkVehicleReportDestructionCallback::manifoldProcessCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVehicleReportDestructionCallback *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifold *
        ]
    ),
    "cTkVehicleReportDestructionCallback::postContactJacobianSetup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkVehicleReportDestructionCallback *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifoldCollisionCache *
            ctypes.c_ulonglong,  # const hknpManifold *
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
        ]
    ),
    "hkSignal2<hknpEventHandlerInput const &,hknpEvent const &>::subscribe<void ": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal2<hknpEventHandlerInput const &,hknpEvent const &>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal2<hknpEventHandlerInput const &,hknpEvent const &> *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['const hknpEventHandlerInput *', 'const hknpEvent *']}
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkSignal3<hknpWorld *,hknpBodyId *,int>::subscribe<void ": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal3<hknpWorld *,hknpBodyId *,int> *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['hknpWorld *', 'hknpBodyId *', 'int']}
        ]
    ),
    "hkSignal1<hknpWorld *>::subscribe<cTkPhysicsManager,void ": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSignal1<hknpWorld *>::Slot *
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal1<hknpWorld *> *
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'cTkPhysicsManager*', 'name': 'this'}, 'hknpWorld *']}
            ctypes.c_char_p,  # const char *
        ]
    ),
    "TkClosestRayResultCallback::TkClosestRayResultCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkClosestRayResultCallback *
        ]
    ),
    "cTkPhysReadJanitor::cTkPhysReadJanitor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysReadJanitor *
        ]
    ),
    "cTkPhysWriteJanitor::cTkPhysWriteJanitor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysWriteJanitor *
        ]
    ),
    "cTkPhysicsManager::cTkPhysicsManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkRaycastJob::cTkRaycastJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
        ]
    ),
    "cTkThresholdedAnyHitCollector::cTkThresholdedAnyHitCollector": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkThresholdedAnyHitCollector *
        ]
    ),
    "hkVdbCmdTag::hkVdbCmdTag": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkVdbCmdTag *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkHashTrie<uint,uint,TempMemAllocShim>::~cTkHashTrie<uint,uint,TempMemAllocShim>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHashTrie<unsigned int,unsigned int,TempMemAllocShim> *
        ]
    ),
    "hknpWorldWriter::SetSafeRegionAabbResult::~SetSafeRegionAabbResult": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorldWriter::SetSafeRegionAabbResult *
        ]
    ),
    "TkAllRayResultCallback::~TkAllRayResultCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkAllRayResultCallback *
        ]
    ),
    "cTkPhysReadJanitor::~cTkPhysReadJanitor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysReadJanitor *
        ]
    ),
    "cTkPhysWriteJanitor::~cTkPhysWriteJanitor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysWriteJanitor *
        ]
    ),
    "cTkRaycastJob::~cTkRaycastJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
        ]
    ),
    "cTkPhysicsManager::AddAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # hknpAction *
        ]
    ),
    "cTkPhysicsManager::AddConstraint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # cTkPhysicsConstraint *
        ]
    ),
    "cTkPhysicsManager::AddRigidBody": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::ApplyAngularKeyFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRigidBody::ApplyAngularVelocityDelta": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkRigidBody::ApplyAngularVelocityDeltaAtPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkRigidBody::ApplyCentralForce": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkRigidBody::ApplyCentralForceLimited": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_double,  # double
        ]
    ),
    "cTkRigidBody::ApplyCentralImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkRigidBody::ApplyCentralImpulseLimited": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_double,  # double
        ]
    ),
    "cTkRigidBody::ApplyLinearKeyFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRigidBody::ApplyLinearVelocityDelta": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkRigidBody::ApplyTorque": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkRigidBody::ApplyTorqueImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkRigidBody::ApplyTorqueImpulseLimited": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_double,  # double
        ]
    ),
    "cTkRigidBody::AssertOnCheckSupport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::AttachCompoundRigidBody": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkPhysicsManager::BroadphaseConvexShapeProbe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const hknpScaledConvexShape *
            ctypes.c_ulonglong,  # cTkStackVector<cTkRigidBody *,128,-1> *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkPhysicsManager::BroadphaseProbe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # const cTkAABB *
            ctypes.c_ulonglong,  # cTkStackVector<cTkRigidBody *,128,-1> *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkPhysicsManager::BroadphaseProbeRawIds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # const cTkAABB *
            ctypes.c_ulonglong,  # cTkStackVector<hknpBodyId,128,-1> *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cTkPhysicsManager::BroadphaseSphereProbe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # cTkStackVector<cTkRigidBody *,128,-1> *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkPhysicsManager::CastLocalRaySingle": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const hknpConvexShape *
        ]
    ),
    "cTkPhysicsManager::CastRay": {
        "cTkPhysicsManager *, const cTkPhysRelVec3 *, const cTkPhysRelVec3 *, cTkContactPoint *, unsigned __int16, cTkRigidBody *": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # cTkPhysicsManager *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # cTkContactPoint *
                ctypes.c_uint16,  # unsigned __int16
                ctypes.c_ulonglong,  # cTkRigidBody *
            ]
        ),
        "cTkPhysicsManager *, const cTkVector3 *, const cTkVector3 *, cTkContactPoint *, unsigned __int16, cTkRigidBody *": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # cTkPhysicsManager *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # cTkContactPoint *
                ctypes.c_uint16,  # unsigned __int16
                ctypes.c_ulonglong,  # cTkRigidBody *
            ]
        ),
    },
    "cTkPhysicsManager::CastRaySingle": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkContactPoint *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRaycastJob::CastRays": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob::Status *
        ]
    ),
    "cTkPhysicsManager::CastShape": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # const cTkCollision *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkContactPoint *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPhysicsManager::CastSphere": {
        "cTkPhysicsManager *, float, const cTkVector3 *, const cTkVector3 *, cTkContactPoint *, int, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkPhysicsManager *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # cTkContactPoint *
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "cTkPhysicsManager *, float, const cTkPhysRelVec3 *, const cTkPhysRelVec3 *, cTkContactPoint *, unsigned __int16, cTkRigidBody *": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # cTkPhysicsManager *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # cTkContactPoint *
                ctypes.c_uint16,  # unsigned __int16
                ctypes.c_ulonglong,  # cTkRigidBody *
            ]
        ),
        "cTkPhysicsManager *, float, const cTkVector3 *, const cTkVector3 *, cTkContactPoint *, unsigned __int16, cTkRigidBody *": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # cTkPhysicsManager *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # cTkContactPoint *
                ctypes.c_uint16,  # unsigned __int16
                ctypes.c_ulonglong,  # cTkRigidBody *
            ]
        ),
    },
    "cTkRaycastJob::Close": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
        ]
    ),
    "CollectVerticesRecursive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # std::vector<hkFloat3,TkSTLAllocatorShim<hkFloat3,4,-1> > *
        ]
    ),
    "cTkPhysicsManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkRaycastJob::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
        ]
    ),
    "cTkRigidBody::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # eBodyOwnerType
        ]
    ),
    "cTkPhysicsManager::ConvexProbe": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkContactPoint *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cTkPhysicsManager::CreateSimpleConvexShape": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpConvexShape> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpConvexShape> *
            ctypes.c_ulonglong,  # const hknpShape **
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_char * 0x10,  # cTkVector3
        ]
    ),
    "cTkPhysicsManager::DebugRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
        ]
    ),
    "cTkPhysicsManager::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
        ]
    ),
    "cTkRigidBody::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkPhysicsManager::DisableCollisionsBetween": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::DisableGravity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::DisableKinematic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkRigidBody::EnableGravity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::EnableKinematic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkPhysExplosion::Explode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysExplosion *
        ]
    ),
    "cTkRigidBody::ForceInertiaOfBox": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkHashTrieNode<16,uint,uint,TempMemAllocShim>::Get": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned int *
        argtypes=[
            ctypes.c_ulonglong,  # cTkHashTrieNode<16,unsigned int,unsigned int,TempMemAllocShim> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # TempMemAllocShim *
        ]
    ),
    "cTkHashTrieNode<16,uint,char const *,TempMemAllocShim>::Get": FUNCDEF(
        restype=ctypes.c_char_p,  # const char **
        argtypes=[
            ctypes.c_ulonglong,  # cTkHashTrieNode<16,unsigned int,char const *,TempMemAllocShim> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # TempMemAllocShim *
        ]
    ),
    "cTkRigidBody::GetAngularVelocity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cTkRaycastJob::GetContact": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # cTkContactPoint *
        ]
    ),
    "cTkRaycastJob::GetContactBody": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRigidBody *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkRaycastJob::GetContactFraction": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkRaycastJob::GetContactNormal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkRaycastJob::GetContactPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkRaycastJob::GetContactRayHitFlags": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkRigidBody::GetGroundNormal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cTkRigidBody::GetHavokBody": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpBody *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::GetInertiaModelSpace": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cTkRigidBody::GetLinearVelocity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cTkHashTrieNode<28,uint,uint,TempMemAllocShim>::GetNext": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkHashTrieNode<28,unsigned int,unsigned int,TempMemAllocShim> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRaycastJob::GetNumContacts": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkRaycastJob::GetNumRays": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::ReadBytesSkip *
        ]
    ),
    "cTkRaycastJob::GetRayEndPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkRaycastJob::GetRayStartPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkRigidBodyProxy::GetTkBody": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRigidBody *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBodyProxy *
        ]
    ),
    "cTkRaycastJob::HasContact": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "HavokWorkerThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkRigidBodyProxy::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBodyProxy *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # cTkCharacterRigidBody *
            ctypes.c_ulonglong,  # const hknpWorld *
        ]
    ),
    "cTkRigidBody::IsCreated": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::IsLocallyOwned": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::IsOnClimbable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::IsOnGround": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRigidBodyProxy *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRaycastJob::IsValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
        ]
    ),
    "cTkBitAlloc::IterateNext": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkBitAlloc *
            ctypes.c_ulonglong,  # cTkBitAlloc::IteratorData *
        ]
    ),
    "cTkRaycastJob::Kick": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRigidBody::MarkAsClimbable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::MarkAsClimbsClimbable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::MarkAsInstances": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::MarkAsNotCollidingWithPlayerShip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::MarkAsPlayerShip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::MarkAsPlayerVehicle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::MarkAsTerrain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkPhysicsManager::OnPostSolveSignal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "cTkRaycastJob::Open": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # eTkMemType
        ]
    ),
    "cTkPhysicsManager::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
        ]
    ),
    "cTkRigidBody::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_ulonglong,  # const cTkPhysicsProperties *
        ]
    ),
    "cTkPhysicsManager::PrintObjectsList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "RaycastJobThreadFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkPhysicsManager::RemoveAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # hknpAction *
        ]
    ),
    "cTkPhysicsManager::RemoveConstraint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # cTkPhysicsConstraint *
        ]
    ),
    "cTkPhysicsManager::RemoveRigidBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # cTkRigidBody **
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkRigidBody::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRigidBody::ResetBroadphase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRaycastJob::RunImmediately": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
        ]
    ),
    "cTkRaycastJob::SetAllAllowBackfaces": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRaycastJob::SetAllAllowInsideHits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRaycastJob::SetAllCollideFilters": {
        "cTkRaycastJob *, unsigned __int16": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkRaycastJob *
                ctypes.c_uint16,  # unsigned __int16
            ]
        ),
        "cTkRaycastJob *, unsigned __int16, unsigned __int16": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkRaycastJob *
                ctypes.c_uint16,  # unsigned __int16
                ctypes.c_uint16,  # unsigned __int16
            ]
        ),
    },
    "cTkRaycastJob::SetAllIgnoreBody": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkRaycastJob::SetAllIgnoreCameraInvisible": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRaycastJob::SetAllIgnoreGameplayFlags": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_uint32,  # eTkRigidBodyGameplayFlags
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRaycastJob::SetAllMaxContacts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cTkRaycastJob::SetAllSphereRadius": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_float,  # float
        ]
    ),
    "cTkRaycastJob::SetAllowBackfaces": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRigidBody::SetAngularVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkRigidBody::SetCharacterPhysicsMaxSlopeAngle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_float,  # float
        ]
    ),
    "cTkRigidBody::SetCharacterPhysicsUpVector": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkRaycastJob::SetCollideFilter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cTkRigidBody::SetCollisionFilters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkRigidBody::SetDamping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkRigidBody::SetDebugRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPhysicsManager::SetDebugRenderFlags": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "cTkRigidBody::SetGravityFactor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_float,  # float
        ]
    ),
    "cTkRaycastJob::SetIgnoreCameraInvisible": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRigidBody::SetLinearDamping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_float,  # float
        ]
    ),
    "cTkRigidBody::SetLinearVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkRaycastJob::SetMaxContacts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cTkRaycastJob::SetNumRays": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkRigidBody::SetQuality": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_uint32,  # eTkQualityType
        ]
    ),
    "cTkRigidBody::SetRelMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cTkRigidBody::SetRelMatrixFromWorld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cTkRigidBody::SetRelMatrixFromWorldPreNormalizedV128": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const VecIntrinsics::V128Matrix *
        ]
    ),
    "cTkPhysicsManager::SetSpaceshipManiforldCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['cTkRigidBody *', 'cTkContactPoint']}
        ]
    ),
    "cTkRaycastJob::SetSphereRadius": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "cTkRaycastJob::SetUsingDelta": {
        "cTkRaycastJob *, unsigned int, const cTkPhysRelVec3 *, const cTkVector3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkRaycastJob *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
        "cTkRaycastJob *, unsigned int, const cTkVector3 *, const cTkVector3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkRaycastJob *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
    },
    "cTkRaycastJob::SetUsingExtents": {
        "cTkRaycastJob *, unsigned int, const cTkPhysRelVec3 *, const cTkPhysRelVec3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkRaycastJob *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ]
        ),
        "cTkRaycastJob *, unsigned int, const cTkVector3 *, const cTkVector3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkRaycastJob *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
    },
    "cTkPhysicsManager::SetVehicleCreatureResponse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_float,  # float
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cTkPhysicsManager::SetVehicleManifoldCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['cTkRigidBody *', 'cTkRigidBody *']}
        ]
    ),
    "cTkPhysicsManager::SetVehicleWheelGuard": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkPhysicsManager::SpherePusher": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "TkBodyActivationEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hknpBodyId *
            ctypes.c_int32,  # int
        ]
    ),
    "TkBodyDeactivationEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # hknpBodyId *
            ctypes.c_int32,  # int
        ]
    ),
    "TkBroadphaseExitFunction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpEventHandlerInput *
            ctypes.c_ulonglong,  # const hknpEvent *
        ]
    ),
    "TkContactCollectorFunction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpEventHandlerInput *
            ctypes.c_ulonglong,  # const hknpEvent *
        ]
    ),
    "TkTriggerHandler": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpEventHandlerInput *
            ctypes.c_ulonglong,  # const hknpEvent *
        ]
    ),
    "cTkPhysicsManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPhysicsManager::UpdateContactFlagsImmediately": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsManager *
        ]
    ),
    "cTkRigidBodyProxy::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBodyProxy *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkRigidBodyProxy::UpdatePrePhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRigidBodyProxy *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkRigidBody::UpdateProxiesContactEnable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
        ]
    ),
    "cTkRaycastJob::Wait": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRaycastJob *
        ]
    ),
    "TkAllRayResultCallback::addHit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkAllRayResultCallback *
            ctypes.c_ulonglong,  # const hknpCollisionResult *
        ]
    ),
    "TkClosestRayResultCallback::addHit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkClosestRayResultCallback *
            ctypes.c_ulonglong,  # const hknpCollisionResult *
        ]
    ),
    "cTkCharacterBodyAction::applyAction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkCharacterBodyAction *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_float,  # float
        ]
    ),
    "cTkCharacterRigidBody::checkSupport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCharacterRigidBody *
            ctypes.c_ulonglong,  # const hkStepInfo *
            ctypes.c_ulonglong,  # hknpCharacterSurfaceInfo *
        ]
    ),
    "cTkHavokAabbSizeFilter::filterBodyPairs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GeometryProvider *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "cTkCharacterBodyListener::filterContactPoint": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkCharacterBodyListener *
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
            ctypes.c_ulonglong,  # hknpCharacterRigidBody::ContactPointInfo *
        ]
    ),
    "cTkCharacterRigidBody::findSupportingContacts": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkCharacterRigidBody *
            ctypes.c_ulonglong,  # const hkStepInfo *
            ctypes.c_ulonglong,  # hkArray<int,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "freeForHavok": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # hkMemoryType
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkCharacterBodyAction::getBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCharacterBodyAction *
            ctypes.c_ulonglong,  # const hknpBodyId **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkAngularFactorCallback::getEnabledFunctions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAngularFactorCallback *
        ]
    ),
    "cTkHavokAabbSizeFilter::isCollisionEnabled": {
        "cTkHavokAabbSizeFilter *, __int64, const hknpQueryFilterData *, const hknpBody *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cTkHavokAabbSizeFilter *
                ctypes.c_int64,  # __int64
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpBody *
            ]
        ),
        "cTkHavokAabbSizeFilter *, __int64, __int64, unsigned __int64": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkHavokAabbSizeFilter *
                ctypes.c_int64,  # __int64
                ctypes.c_int64,  # __int64
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
        "cTkHavokAabbSizeFilter *, __int64, __int64, const hknpCollisionFilter::FilterInput *, const hknpCollisionFilter::FilterInput *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkHavokAabbSizeFilter *
                ctypes.c_int64,  # __int64
                ctypes.c_int64,  # __int64
                ctypes.c_ulonglong,  # const hknpCollisionFilter::FilterInput *
                ctypes.c_ulonglong,  # const hknpCollisionFilter::FilterInput *
            ]
        ),
    },
    "mallocForHavok": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_uint32,  # hkMemoryType
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkAngularFactorCallback::manifoldProcessCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAngularFactorCallback *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifold *
        ]
    ),
    "cTkIgnoreBackfacesModifier::manifoldProcessCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIgnoreBackfacesModifier *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # const hknpCdBodyBase *
            ctypes.c_ulonglong,  # hknpManifold *
        ]
    ),
    "cTkCharacterBodyListener::onPostSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCharacterBodyListener *
            ctypes.c_ulonglong,  # hknpCharacterRigidBody *
        ]
    ),
    "hkKeycodeManager::setKeycodes": FUNCDEF(
        restype=ctypes.c_ulonglong,  # int *
        argtypes=[
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hkSignal1<hknpWorld *>::MemberSlot<cTkPhysicsManager,void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSignal1<hknpWorld *>::MemberSlot<cTkPhysicsManager,void (__cdecl cTkPhysicsManager::*)(hknpWorld *)> *
            ctypes.c_ulonglong,  # hkSignal1<hknpWorld *> *
        ]
    ),
    "cTkMetaData::ReadFromFile<cTkModelDescriptorList>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkModelDescriptorList *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cTkResourceFilterList>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcStatGroupTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMcQmcLFSR::cTkMcQmcLFSR": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMcQmcLFSR *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkPersonalRNG::cTkPersonalRNG": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cTkResource::cTkResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResource *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkResourceManager::cTkResourceManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceManager *
        ]
    ),
    "cTkLinearHashTable<unsigned __int64,TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,6,0,cTkLinearHashTableHash<unsigned __int64>>::~cTkLinearHashTable<unsigned __int64,TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,6,0,cTkLinearHashTableHash<unsigned __int64>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<unsigned __int64,TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,6,0,cTkLinearHashTableHash<unsigned __int64> > *
        ]
    ),
    "cTkResource::~cTkResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResource *
        ]
    ),
    "cTkResourceManager::~cTkResourceManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceManager *
        ]
    ),
    "cTkResourceManager::AddNonExistingResource": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSmartResHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceManager *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_ulonglong,  # cTkResource *
        ]
    ),
    "cTkResourceManager::AddResource": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSmartResHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceManager *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "cTkResourceManager::AddResourceInternal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceManager *
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            ctypes.c_ulonglong,  # cTkResource *
        ]
    ),
    "cTkResourceManager::AreAllResourcesKillableThisFrame": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceManager *
        ]
    ),
    "CapturedNoise1D": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "CapturedNoise3D": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "Wombat::Cellular3D_Deriv": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector4 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector4 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkLayeredNoise::CheckerboardNoise": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_double,  # double
            ctypes.c_double,  # double
            ctypes.c_double,  # double
        ]
    ),
    "cTkResource::CloneInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResource *
            ctypes.c_ulonglong,  # const cTkResource *
        ]
    ),
    "cTkResourceManager::CloneResource": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSmartResHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceManager *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_ulonglong,  # cTkResource *
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "cTkMcQmcLFSRStore::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMcQmcLFSRStore *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkSmartResHandle::DecRef": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmartResHandle *
        ]
    ),
    "cTkNodeNetworkAddress::Deserialise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeNetworkAddress *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cTkMcQmcLFSRStore::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMcQmcLFSRStore *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkResourceDescriptor::DistortInstance": {
        "cTkResourceDescriptor *, const char *, const cTkSeed *, const cTkSeed *, const TkID<256> *, const TkID<256> *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cTkResourceDescriptor *
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # const cTkSeed *
                ctypes.c_ulonglong,  # const cTkSeed *
                ctypes.c_char_p,  # const TkID<256> *
                ctypes.c_char_p,  # const TkID<256> *
            ]
        ),
        "cTkResourceDescriptor *, const char *, const float, const float, const cTkSeed *, const cTkSeed *, const TkID<256> *, const TkID<256> *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cTkResourceDescriptor *
                ctypes.c_char_p,  # const char *
                ctypes.c_float,  # const float
                ctypes.c_float,  # const float
                ctypes.c_ulonglong,  # const cTkSeed *
                ctypes.c_ulonglong,  # const cTkSeed *
                ctypes.c_char_p,  # const TkID<256> *
                ctypes.c_char_p,  # const TkID<256> *
            ]
        ),
    },
    "SpookyHash::Final": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SpookyHash *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "cTkResourceManager::FindResourceA": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkResource *
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceManager *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkResourceDescriptor *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkResourceDescriptor::GenerateDistortedInstanceFromList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_ulonglong,  # cTkModelDescriptorList *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_ulonglong,  # cTkResourceFilterData *
        ]
    ),
    "cTkResourceDescriptor::GenerateInstance": {
        "cTkResourceDescriptor *, const char *, const cTkSeed *, const TkID<256> *, const TkID<256> *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkModelDescriptorList *
            argtypes=[
                ctypes.c_ulonglong,  # cTkResourceDescriptor *
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # const cTkSeed *
                ctypes.c_char_p,  # const TkID<256> *
                ctypes.c_char_p,  # const TkID<256> *
            ]
        ),
        "cTkResourceDescriptor *, const char *, const std::vector<TkID<256>,TkSTLAllocatorShim<TkID<256>,8,-1> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkResourceDescriptor *
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # const std::vector<TkID<256>,TkSTLAllocatorShim<TkID<256>,8,-1> > *
            ]
        ),
    },
    "cTkResourceDescriptor::GenerateInstanceFromGuide": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_ulonglong,  # const cTkModelDescriptorList *
            ctypes.c_ulonglong,  # const std::vector<TkID<256>,TkSTLAllocatorShim<TkID<256>,8,-1> > *
        ]
    ),
    "cTkResourceDescriptor::GenerateInstanceFromList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_ulonglong,  # cTkModelDescriptorList *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_ulonglong,  # cTkResourceFilterData *
        ]
    ),
    "cTkResourceDescriptor::GenerateInstanceFromModel": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkModelDescriptorList *
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkSeed *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cTkLayeredNoise::GenerateSmoothNoiseLayer3D": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkLayeredNoise::GenerateUberNoiseLayer3D_Vector": {
        "const float *, const float *, const float *, float, cTkNoiseUberData *, int, float *, cTkVector4 *, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const float *
                ctypes.c_ulonglong,  # const float *
                ctypes.c_ulonglong,  # const float *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # cTkNoiseUberData *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # cTkVector4 *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "const GPU::vec3 *, float, cTkNoiseUberData *, int, float *, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const GPU::vec3 *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # cTkNoiseUberData *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # float *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "cTkResourceManager::GetAllResourceHandlesOfType": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceManager *
            ctypes.c_ulonglong,  # std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkResourceManager::GetDefaultResource": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkResource *
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceManager *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkRegionNoise::GetFaceIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkRegionNoise::GetFaceRight": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkRegionNoise::GetFaceUp": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkResourceDescriptor::GetModelDescriptorList": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkModelDescriptorList *
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "GetNullSelectionResDescriptor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSharedPtr<cTkResourceDescriptor const > *
        argtypes=[]
    ),
    "cTkSmartResHandle::GetPResource": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkResource *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmartResHandle *
        ]
    ),
    "cTkResourceDescriptor::GetResourceFilter": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkResourceFilterData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cTkResourceManager::GetResourceTypeName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkResourceRegistryEntry *
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceManager *
            ctypes.c_int32,  # int
        ]
    ),
    "SpookyHash::Hash128": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "cTkSmartResHandle::IncRef": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmartResHandle *
        ]
    ),
    "SpookyHash::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SpookyHash *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLayeredNoise::IqTurbulenceNoise": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkMetadataCache<cTkModelDescriptorList>::Load": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkModelDescriptorList *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMetadataCache<cTkModelDescriptorList> *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNodeNetworkAddress::LookupNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeNetworkAddress *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_float,  # float
        ]
    ),
    "cTkResource::MakePersistent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResource *
        ]
    ),
    "cTkSmartResHandle::MakePersistent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSmartResHandle *
        ]
    ),
    "cTkSimplexNoise::Noise2d": {
        "const float, const float": FUNCDEF(
            restype=ctypes.c_double,  # double
            argtypes=[
                ctypes.c_float,  # const float
                ctypes.c_float,  # const float
            ]
        ),
        "const float, const float, float *, float *": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_float,  # const float
                ctypes.c_float,  # const float
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
            ]
        ),
    },
    "cTkSimplexNoise::Noise3d": {
        "const float, const float, const float": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_float,  # const float
                ctypes.c_float,  # const float
                ctypes.c_float,  # const float
            ]
        ),
        "const float, const float, const float, float *, float *, float *": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_float,  # const float
                ctypes.c_float,  # const float
                ctypes.c_float,  # const float
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
            ]
        ),
    },
    "cTkSimplexNoise::Noise3dVectorised": {
        "double, double, double": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_double,  # double
                ctypes.c_double,  # double
                ctypes.c_double,  # double
            ]
        ),
        "double, double, double, float *, float *, float *": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_double,  # double
                ctypes.c_double,  # double
                ctypes.c_double,  # double
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
            ]
        ),
    },
    "cTkLayeredNoise::OctaveNoise": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_int32,  # const int
        ]
    ),
    "cTkLayeredNoise::PlateauNoise": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # const int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkResourceManager::PollToReleaseUnusedResources": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceManager *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkResourceManager::QueryAllUnloadedResources": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceManager *
            ctypes.c_ulonglong,  # std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *
        ]
    ),
    "cTkResourceManager::QueryUnloadedResource": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceManager *
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkLayeredNoise::Random3f": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkResourceManager::RegisterType": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceManager *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': []}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': []}
            ctypes.c_ulonglong,  # {'return': 'cTkResource *', 'args': ['const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *', 'int', 'cTkResourceDescriptor *']}
        ]
    ),
    "cTkPersonalRNG::Reseed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPersonalRNG *
        ]
    ),
    "cTkResourceDescriptor::SearchForID": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_ulonglong,  # cTkModelDescriptorList *
        ]
    ),
    "cTkResourceDescriptor::SelectPart": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAsyncIOManagerImp::cTkAsyncOp *
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_ulonglong,  # cTkModelDescriptorList *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_ulonglong,  # cTkPersonalRNG *
            ctypes.c_ulonglong,  # cTkResourceFilterData *
        ]
    ),
    "cTkNodeNetworkAddress::Serialise": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeNetworkAddress *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "cTkResourceManager::SetDefaultResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceManager *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkNodeNetworkAddress::SetToNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeNetworkAddress *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "SpookyHash::Short": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "cTkResourceDescriptor::ShouldLoadMesh": {
        "cTkResourceDescriptor *, const TkID<256> *, const TkID<256> *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cTkResourceDescriptor *
                ctypes.c_char_p,  # const TkID<256> *
                ctypes.c_char_p,  # const TkID<256> *
            ]
        ),
        "cTkResourceDescriptor *, const TkID<256> *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cTkResourceDescriptor *
                ctypes.c_char_p,  # const TkID<256> *
            ]
        ),
        "cTkResourceDescriptor *, const char *, TkID<256> *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cTkResourceDescriptor *
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # TkID<256> *
            ]
        ),
    },
    "SpookyHash::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SpookyHash *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkResourceManager::UpdateStreamRequests": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceManager *
        ]
    ),
    "cTkResourceDescriptor::Validate": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "Wombat::Value3D": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkRegionNoise::VoronoiDistanceOnCube": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cTkRegionNoise::VoronoiDistanceOnCubeBatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const GPU::vec3 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # GPU::sVoronoiResults *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkRegionNoise::VoronoiID": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "Sha256_Final": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CSha256 *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "Sha256_Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CSha256 *
        ]
    ),
    "Sha256_Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CSha256 *
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "Sha256_WriteByteBlock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CSha256 *
        ]
    ),
    "cTkNetReplicatedEntityComponent::NetSerializeOnInit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
        ]
    ),
    "cTkNetReplicatedEntityComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponent *
        ]
    ),
    "cTkNetReplicatedEntityComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponent *
        ]
    ),
    "cTkNetReplicatedEntityComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponent *
        ]
    ),
    "CopyAndPadChunks<cTkQuantizedAabbList<short,0>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkQuantizedAabbList<short,0> *
            ctypes.c_ulonglong,  # cTkQuantizedAabbList<short,0> *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CopyAndPadChunks<cTkQuantizedTriList>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkQuantizedTriList *
            ctypes.c_ulonglong,  # cTkQuantizedTriList *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkBvMesh::PerformAabbQuery<hkArray<hkHandle<uint,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBvMesh *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "cTkBvMesh::PerformAabbQuery<hknpCollisionQueryCollector>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBvMesh *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpAabbQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "Tk::cNapkinSimulator::SatisfyFourConstraints<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkinSimulator *
            ctypes.c_ulonglong,  # Tk::cNapkin *
            ctypes.c_ulonglong,  # const Tk::cDistanceConstraint *
            ctypes.c_double,  # double
        ]
    ),
    "Tk::cNapkinSimulator::SatisfyFourConstraints<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkinSimulator *
            ctypes.c_ulonglong,  # Tk::cNapkin *
            ctypes.c_ulonglong,  # const Tk::cDistanceConstraint *
            ctypes.c_double,  # double
        ]
    ),
    "Tk::cNapkin::cNapkin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkin *
            ctypes.c_ulonglong,  # Tk::cNapkinSimulator *
        ]
    ),
    "cTkBvMesh::cTkBvMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBvMesh *
        ]
    ),
    "cTkPhysicsConstraint::cTkPhysicsConstraint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsConstraint *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkQuantizedAabbList<short,0>::~cTkQuantizedAabbList<short,0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuantizedAabbList<short,0> *
        ]
    ),
    "Tk::cNapkin::~cNapkin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkin *
        ]
    ),
    "cTkBvMesh::~cTkBvMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBvMesh *
        ]
    ),
    "cTkPhysicsConstraint::Activate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsConstraint *
        ]
    ),
    "Tk::cNapkin::AddConstraint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkin *
            ctypes.c_uint32,  # Tk::eEdgeType
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCollision::AddFromNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkCollision::AddFromNodeStep": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_ulonglong,  # cTkStackVector<TkHandle,16,-1> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkCollision::AddFromTerrain": {
        "cTkCollision *, cTkQuadElement *, int, bool, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkCollision *
                ctypes.c_ulonglong,  # cTkQuadElement *
                ctypes.c_int32,  # int
                ctypes.c_ubyte,  # bool
                ctypes.c_float,  # float
            ]
        ),
        "cTkCollision *, cTkTerrainVertex *, int, bool, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkCollision *
                ctypes.c_ulonglong,  # cTkTerrainVertex *
                ctypes.c_int32,  # int
                ctypes.c_ubyte,  # bool
                ctypes.c_float,  # float
            ]
        ),
    },
    "cTkCollision::AddHavokShapes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_ulonglong,  # hknpShape **
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkCollision::AddStaticFromNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkBvMesh::AddTriangle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBvMesh *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkEigenFinder::Alloc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEigenFinder *
            ctypes.c_int32,  # int
        ]
    ),
    "Tk::cNapkin::ApplyFrictionLimits": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkin *
        ]
    ),
    "BestAlignBasisAroundY": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cOrthonormalBasis *
        argtypes=[
            ctypes.c_ulonglong,  # cOrthonormalBasis *
            ctypes.c_ulonglong,  # const hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # const cOrthonormalBasis *
        ]
    ),
    "Tk::cDistanceConstraint::BothParticlesAreFixed": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cDistanceConstraint *
            ctypes.c_ulonglong,  # const Tk::cNapkin *
        ]
    ),
    "cTkBvMesh::Build": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkTransformf *
        argtypes=[
            ctypes.c_ulonglong,  # cTkBvMesh *
            ctypes.c_ulonglong,  # hkTransformf *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCollision::BuildBoxShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_ulonglong,  # hknpShape **
            ctypes.c_ulonglong,  # hkTransformf *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cTkCollision::BuildCapsuleShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_ulonglong,  # hknpShape **
            ctypes.c_ulonglong,  # hkTransformf *
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cTkCollision::BuildConvexPointsShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_ulonglong,  # hknpShape **
            ctypes.c_ulonglong,  # hkTransformf *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cTkCollision::BuildFromChildNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_uint32,  # const TkHandle
            ctypes.c_uint32,  # const TkHandle
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkStackVector<hknpShape *,128,-1> *
            ctypes.c_ulonglong,  # cTkStackVector<hkTransformf,128,-1> *
        ]
    ),
    "cTkCollision::BuildFromMesh": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpCompoundShape *
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hkTransformf *
        ]
    ),
    "cTkIndexedQuantizedTriList::BuildFromTris": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIndexedQuantizedTriList *
            ctypes.c_ulonglong,  # const cTkQuantizedTriList *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkStackVector<cTkQuantVector3<short>,256,0> *
            ctypes.c_ulonglong,  # cTkStackVector<int,256,0> *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkCollision::BuildSphereShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_ulonglong,  # hknpShape **
            ctypes.c_ulonglong,  # hkTransformf *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cTkCollision::BuildStaticFromChildNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkStackVector<hknpShape *,128,-1> *
            ctypes.c_ulonglong,  # cTkStackVector<hkTransformf,128,-1> *
        ]
    ),
    "cTkQuantizedAabbList<short,0>::CalcAabb": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkQuantizedAabb<short> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuantizedAabbList<short,0> *
            ctypes.c_ulonglong,  # cTkQuantizedAabb<short> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkQuantizedTriList::CalcAabb": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkQuantizedAabb<short> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuantizedTriList *
            ctypes.c_ulonglong,  # cTkQuantizedAabb<short> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkCollision::CastShape": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkContactPoint *
        ]
    ),
    "cTkQuantizedAabbList<short,0>::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuantizedAabbList<short,0> *
        ]
    ),
    "cTkReplicatedVariableArray::ClearDirtyFlags": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableArray *
        ]
    ),
    "cTkCollision::ClosestPoint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_float,  # float
        ]
    ),
    "cTkEigenFinder::Compute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEigenFinder *
        ]
    ),
    "cTkCollision::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
        ]
    ),
    "cTkNetworkId::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNetworkId *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkCollision::ConvexHullClosestPoint": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkContactPoint *
            ctypes.c_int16,  # __int16
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkIndexedQuantizedTriList::CopyTrisOut": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIndexedQuantizedTriList *
            ctypes.c_ulonglong,  # cTkQuantizedTriBlock *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
        ]
    ),
    "Tk::cCubicSpline3d::Create": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cCubicSpline3d *
            ctypes.c_ulonglong,  # const std::vector<cTkVector3,TkSTLAllocatorShim<cTkVector3,16,-1> > *
            ctypes.c_ulonglong,  # const std::vector<float,TkSTLAllocatorShim<float,4,-1> > *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkCollision::CreateHull": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpConvexShape *
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkPhysicsConstraint::CreateMotors": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsConstraint *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "Tk::cNapkinSimulator::CreateNapkin": FUNCDEF(
        restype=ctypes.c_ulonglong,  # Tk::cNapkin *
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkinSimulator *
            ctypes.c_ulonglong,  # const std::vector<cTkVector4,TkSTLAllocatorShim<cTkVector4,16,0> > *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const Tk::cNapkinConstructionInfo *
        ]
    ),
    "cTkCollision::CreateShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_ulonglong,  # hknpShape **
            ctypes.c_ulonglong,  # hkTransformf *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCollision::CreateSingleShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkCollision::CreateTriMesh": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpCompositeShape *
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # hkTransformf *
        ]
    ),
    "cTkPhysicsConstraint::Deactivate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsConstraint *
        ]
    ),
    "Tk::cNapkinSimulator::DestroyNapkin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkinSimulator *
            ctypes.c_ulonglong,  # Tk::cNapkin *
        ]
    ),
    "cTkCollision::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
        ]
    ),
    "Tk::cNapkinSimulator::ExtractConstraint": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkinSimulator *
            ctypes.c_ulonglong,  # const Tk::cCapsuleShape *
            ctypes.c_ulonglong,  # const Tk::cDistanceConstraint *
            ctypes.c_ulonglong,  # Tk::cNapkin *
            ctypes.c_float,  # float
        ]
    ),
    "Tk::cCapsuleShape::ExtractFourParticles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cCapsuleShape *
            ctypes.c_ulonglong,  # std::vector<cTkVector4,TkSTLAllocatorShim<cTkVector4,16,0> > *
            ctypes.c_ulonglong,  # int *
            ctypes.c_double,  # double
            ctypes.c_float,  # float
        ]
    ),
    "Tk::cCapsuleShape::ExtractParticle": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cCapsuleShape *
            ctypes.c_ulonglong,  # cTkVector4 *
            ctypes.c_double,  # double
            ctypes.c_float,  # float
        ]
    ),
    "FindFrameForTightBounds": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cOrientedBb *
        argtypes=[
            ctypes.c_ulonglong,  # cOrientedBb *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # const hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "cTkPhysicsConstraint::GetHkFrameA": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkTransformf *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsConstraint *
            ctypes.c_ulonglong,  # hkTransformf *
        ]
    ),
    "cTkPhysicsConstraint::GetHkFrameB": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkTransformf *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsConstraint *
            ctypes.c_ulonglong,  # hkTransformf *
        ]
    ),
    "cTkCollision::GetOBB": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAABB *
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_ulonglong,  # cTkAABB *
        ]
    ),
    "Tk::cNapkin::GetParticlePosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector4 *
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkin *
            ctypes.c_int32,  # int
        ]
    ),
    "Tk::cNapkin::GetParticleVelocity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector4 *
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkin *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkCollision::GetRadius": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
        ]
    ),
    "Tk::cCubicSpline3d::GetValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cCubicSpline3d *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cTkCollision::GetVoxelTypeFromTri": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkTerrainVertex *
            ctypes.c_ulonglong,  # const cTkTerrainVertex *
            ctypes.c_ulonglong,  # const cTkTerrainVertex *
        ]
    ),
    "Tk::cNapkin::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkin *
            ctypes.c_ulonglong,  # const std::vector<cTkVector4,TkSTLAllocatorShim<cTkVector4,16,0> > *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "IsCapsuleOrSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "Tk::cNapkin::IsConstraintEnabled": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkin *
            ctypes.c_int32,  # int
        ]
    ),
    "Tk::cNapkin::IsParticleFixed": FUNCDEF(
        restype=ctypes.c_uint8,  # unsigned __int8
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkin *
            ctypes.c_int32,  # int
        ]
    ),
    "Tk::cDistanceConstraint::IsSuitableForResolving_Internal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cDistanceConstraint *
            ctypes.c_ulonglong,  # const Tk::cNapkin *
        ]
    ),
    "MakeScaledMeshShape": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpCompoundShape *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompositeShape *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkEigenFinder::NormaliseEigenvectors": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEigenFinder *
        ]
    ),
    "cTkReplicatedVariableArray::NumChangedReplicatedVariables": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableArray *
        ]
    ),
    "Tk::cDistanceConstraint::OnlyOnePointIsFixed_Internal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cDistanceConstraint *
            ctypes.c_ulonglong,  # const Tk::cNapkin *
        ]
    ),
    "cTkBvMesh::PerformClosestPoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBvMesh *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
        ]
    ),
    "cTkBvMesh::PerformShapecast": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBvMesh *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
        ]
    ),
    "cTkQuantizedAabbList<short,0>::PushBackElement": {
        "cTkQuantizedAabbList<short,0> *, const cTkQuantizedAabbList<short,0> *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkQuantizedAabbList<short,0> *
                ctypes.c_ulonglong,  # const cTkQuantizedAabbList<short,0> *
                ctypes.c_int32,  # int
            ]
        ),
        "cTkQuantizedAabbList<short,0> *, int, int, __int16, __int16, __int16, __int16, __int16, __int16, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkQuantizedAabbList<short,0> *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_int16,  # __int16
                ctypes.c_int16,  # __int16
                ctypes.c_int16,  # __int16
                ctypes.c_int16,  # __int16
                ctypes.c_int16,  # __int16
                ctypes.c_int16,  # __int16
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cTkPhysicsBvMeshShapeFunctions::RegisterShapeFunctions": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkMultiVector<short,6,0,8>::Reserve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMultiVector<short,6,0,8> *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkQuantizedTriList::Reserve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuantizedTriList *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBvMesh::ReserveTriangles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBvMesh *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "Tk::cNapkinSimulator::SatisfyConstraint_Unoptimized": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkinSimulator *
            ctypes.c_ulonglong,  # Tk::cNapkin *
            ctypes.c_ulonglong,  # const Tk::cDistanceConstraint *
            ctypes.c_double,  # double
        ]
    ),
    "Tk::SegmentSegmentShortestDistance": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # const cTkVector4 *
            ctypes.c_ulonglong,  # const cTkVector4 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector4 *
            ctypes.c_ulonglong,  # const cTkVector4 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkReplicatedVariableArray::SerializeOrDeserialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableArray *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPhysicsConstraint::SetConeLimit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsConstraint *
            ctypes.c_double,  # double
        ]
    ),
    "Tk::cNapkin::SetConstraintEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkin *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPhysicsConstraint::SetJointFriction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsConstraint *
            ctypes.c_float,  # float
        ]
    ),
    "cTkPhysicsConstraint::SetMotorParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsConstraint *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "Tk::cNapkin::SetParticleEnabled": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkin *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Tk::cNapkin::SetParticleFixed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkin *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "Tk::cNapkin::SetParticlePosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkin *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkVector4 *
        ]
    ),
    "Tk::cNapkin::SetParticleVelocity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkin *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkVector4 *
        ]
    ),
    "cTkPhysicsConstraint::SetTwistLimit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsConstraint *
            ctypes.c_float,  # float
        ]
    ),
    "cTkCollision::SetVolumeAndInertiaToThatOfBox": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollision *
            ctypes.c_ulonglong,  # const cTkAABB *
            ctypes.c_ulonglong,  # hkMassProperties *
        ]
    ),
    "Tk::cNapkin::ShiftWorld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkin *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkReplicatedVariableArray::ShouldReplicateToPlayer": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkReplicatedVariableArray *
            ctypes.c_ulonglong,  # cGcNetworkPlayer *
        ]
    ),
    "Tk::cNapkin::SortConstraints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkin *
        ]
    ),
    "Tk::cNapkin::Step": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkin *
        ]
    ),
    "cTkSubdivBuilder<short,cTkQuantizedTriList,0>::Subdivide": {
        "cTkSubdivBuilder<short,cTkQuantizedTriList,0> *, unsigned int, unsigned int, const cTkQuantizedAabb<short> *, cTkQuantizedAabb<short> *, cTkQuantizedAabb<short> *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkSubdivBuilder<short,cTkQuantizedTriList,0> *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # const cTkQuantizedAabb<short> *
                ctypes.c_ulonglong,  # cTkQuantizedAabb<short> *
                ctypes.c_ulonglong,  # cTkQuantizedAabb<short> *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cTkSubdivBuilder<short,cTkQuantizedTriList,0> *, unsigned int, unsigned int, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkSubdivBuilder<short,cTkQuantizedTriList,0> *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "cTkSubdivBuilder<short,cTkQuantizedAabbList<short,0>,0>::Subdivide": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSubdivBuilder<short,cTkQuantizedAabbList<short,0>,0> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkQuantizedAabbList<short,0>::SwapElements": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuantizedAabbList<short,0> *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "Tk::cNapkin::SwapParticles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Tk::cNapkin *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cOrthonormalBasis::ToLocal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
        argtypes=[
            ctypes.c_ulonglong,  # cOrthonormalBasis *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # const hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "cTkNetReplicatedEntityComponent::UpdateRemoteActive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNetReplicatedEntityComponent *
        ]
    ),
    "cTkIndexedQuantizedTriList::WriteIndBuf": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIndexedQuantizedTriList *
            ctypes.c_ulonglong,  # const cTkStackVector<cTkQuantVector3<short>,256,0> *
            ctypes.c_ulonglong,  # const cTkStackVector<int,256,0> *
            ctypes.c_ulonglong,  # const unsigned __int8 *
        ]
    ),
    "cTkPhysicsBvMeshShape::buildSurfaceGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape::BuildSurfaceGeometryConfig *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsBvMeshShape *
            ctypes.c_ulonglong,  # const hknpShape::BuildSurfaceGeometryConfig *
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "cTkPhysicsBvMeshShapeFunctions::buildSurfaceGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::BuildSurfaceGeometryConfig *
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "cTkPhysicsBvMeshShapeFunctions::calcAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "cTkPhysicsBvMeshShapeFunctions::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "cTkPhysicsBvMeshShape::castShapeImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "cTkPhysicsBvMeshShape::getClosestPointsImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "cTkIndexedMeshShapeCInfo::getConvexShape": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpShapeInstance *
        argtypes=[
            ctypes.c_ulonglong,  # cTkIndexedMeshShapeCInfo *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkQuadMeshShapeCInfo::getConvexShape": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpShapeInstance *
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuadMeshShapeCInfo *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkTerrainMeshShapeCInfo::getConvexShape": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hknpShapeInstance *
        argtypes=[
            ctypes.c_ulonglong,  # cTkTerrainMeshShapeCInfo *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkIndexedMeshShapeCInfo::getIndices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIndexedMeshShapeCInfo *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkQuadMeshShapeCInfo::getIndices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuadMeshShapeCInfo *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkTerrainMeshShapeCInfo::getIndices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTerrainMeshShapeCInfo *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkPhysicsBvMeshShape::getLeafShapes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsBvMeshShape *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpShapeCollector *
        ]
    ),
    "cTkPhysicsBvMeshShapeFunctions::getLeafShapes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpShapeCollector *
        ]
    ),
    "cTkIndexedMeshShapeCInfo::getNumTriangles": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkIndexedMeshShapeCInfo *
        ]
    ),
    "cTkQuadMeshShapeCInfo::getNumTriangles": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuadMeshShapeCInfo *
        ]
    ),
    "cTkTerrainMeshShapeCInfo::getNumTriangles": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkTerrainMeshShapeCInfo *
        ]
    ),
    "cTkIndexedMeshShapeCInfo::getNumVertices": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkIndexedMeshShapeCInfo *
        ]
    ),
    "cTkTerrainMeshShapeCInfo::getNumVertices": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkStorageSkinnedMeshShape *
        ]
    ),
    "cTkQuadMeshShapeCInfo::getTriangleShapeTag": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuadMeshShapeCInfo *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkTerrainMeshShapeCInfo::getTriangleShapeTag": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkTerrainMeshShapeCInfo *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkIndexedMeshShapeCInfo::getVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIndexedMeshShapeCInfo *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "cTkQuadMeshShapeCInfo::getVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuadMeshShapeCInfo *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "cTkTerrainMeshShapeCInfo::getVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTerrainMeshShapeCInfo *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "cTkPhysicsBvMeshShapeFunctions::queryAabb": {
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "const hknpShape *, hknpCollisionQueryContext *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpQueryFilterData *, const hknpShapeQueryInfo *, hknpCollisionQueryCollector *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hknpShape *
                ctypes.c_ulonglong,  # hknpCollisionQueryContext *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
                ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
    },
    "Tk__spline": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_int32,  # int
        ]
    ),
    "Tk__splint": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFont::cTkFont": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFont *
        ]
    ),
    "cTkDynamicTexture::ActivateTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicTexture *
            ctypes.c_ulonglong,  # cTkRenderStateCache *
        ]
    ),
    "cTk2dLayer::AddObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dLayer *
            ctypes.c_ulonglong,  # cTk2dObject *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFont::AdjustForKerningPairs": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkFont *
            ctypes.c_wchar,  # wchar_t
            ctypes.c_wchar,  # wchar_t
        ]
    ),
    "cTkMultiContextManager::BeginFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMultiContextManager *
        ]
    ),
    "cTk2dLayer::CalculateSize": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dLayer *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "cTkTextureBase::CalculateTextureSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # cTkTextureBase::eTexFormat
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkFont::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFont *
        ]
    ),
    "cTkFontFile::Close": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFontFile *
        ]
    ),
    "cTk2dImageEx::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dImageEx *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cTk2dLayer::Construct": {
        "cTk2dLayer *, const cTkVector2 *, const cTkVector2 *, const cTkVector2 *, const cTkColour *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTk2dLayer *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkColour *
            ]
        ),
        "cTk2dLayer *, const cTkVector2 *, const cTkVector2 *, const cTkVector2 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTk2dLayer *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector2 *
            ]
        ),
    },
    "cTk2dText::Construct": {
        "cTk2dText *, const cTkVector2 *, const cTkVector2 *, const cTkVector2 *, const cTk2dTextPreset *, const TkID<256> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTk2dText *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTk2dTextPreset *
                ctypes.c_char_p,  # const TkID<256> *
            ]
        ),
        "cTk2dText *, const cTkVector2 *, const cTkVector2 *, const cTkVector2 *, const cTk2dTextPreset *, const wchar_t *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTk2dText *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTk2dTextPreset *
                ctypes.c_ulonglong,  # const wchar_t *
            ]
        ),
        "cTk2dText *, const cTkVector2 *, const cTkVector2 *, const cTk2dTextPreset *, const TkID<256> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTk2dText *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTk2dTextPreset *
                ctypes.c_char_p,  # const TkID<256> *
            ]
        ),
        "cTk2dText *, const cTkVector2 *, const cTkVector2 *, const cTk2dTextPreset *, const wchar_t *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTk2dText *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTkVector2 *
                ctypes.c_ulonglong,  # const cTk2dTextPreset *
                ctypes.c_ulonglong,  # const wchar_t *
            ]
        ),
    },
    "cTkDynamicTexture::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicTexture *
        ]
    ),
    "cTkFontFile::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFontFile *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkFontLoader::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFontLoader *
            ctypes.c_ulonglong,  # cTkFontFile *
            ctypes.c_ulonglong,  # cTkFont *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkTexture *
        ]
    ),
    "cTkMultiContextManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMultiContextManager *
        ]
    ),
    "cTk2dLayer::ConstructDynamicSize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dLayer *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cTkMultiContextManager::CreateContext": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkGraphicsContext *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMultiContextManager *
        ]
    ),
    "cTkDynamicTexture::DeactivateTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicTexture *
            ctypes.c_ulonglong,  # cTkRenderStateCache *
        ]
    ),
    "cTk2dLayer::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dLayer *
        ]
    ),
    "cTkTextStyle::GetTextWidth": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkFont *
            ctypes.c_float,  # float
        ]
    ),
    "cTkMultiContextManager::GetUnusedContext": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAsyncIOManagerImp::cTkAsyncOp *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMultiContextManager *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFont::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkFont *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkSmartResHandle *
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_int16,  # __int16
            ctypes.c_int16,  # __int16
        ]
    ),
    "cTkFontLoaderText::InterpretChar": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFontLoaderText *
            ctypes.c_ulonglong,  # std::string *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkFontLoaderText::InterpretCommon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFontLoaderText *
            ctypes.c_ulonglong,  # std::string *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkFontLoaderText::InterpretInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFontLoaderText *
            ctypes.c_ulonglong,  # std::string *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkFontLoaderText::InterpretKerning": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFontLoaderText *
            ctypes.c_ulonglong,  # std::string *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkFontFile::IsBinary": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # TkInputDevicesGLFW::cTkDS4Input *
        ]
    ),
    "cTkFontLoaderText::Load": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkFontLoaderText *
        ]
    ),
    "cTkColour::NmzInterpolate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkColour *
        argtypes=[
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_float,  # float
        ]
    ),
    "cTk2dLayer::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dLayer *
        ]
    ),
    "cTkDynamicTexture::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicTexture *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # TkResourceFindType
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFontFile::ReachedEndOfFile": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkFontFile *
        ]
    ),
    "cTkFontFile::Read": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkFontFile *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
        ]
    ),
    "cTk2dLayer::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dLayer *
        ]
    ),
    "cTkDynamicTexture::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicTexture *
        ]
    ),
    "cTk2dLayer::RemoveAllObjects": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dLayer *
        ]
    ),
    "cTk2dImage::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dImage *
        ]
    ),
    "cTk2dImageEx::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dImageEx *
        ]
    ),
    "cTk2dLayer::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dLayer *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTk2dText::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dText *
        ]
    ),
    "cTk3dLayer::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk3dLayer *
        ]
    ),
    "cTk2dText::SetTextHeight": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dText *
            ctypes.c_float,  # float
        ]
    ),
    "cTkMultiContextManager::Submit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMultiContextManager *
        ]
    ),
    "cTkMultiContextManager::SubmitAllWeCan": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMultiContextManager *
        ]
    ),
    "cTkGraphicsContext::SubmitMultiple": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsContext **
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGraphicsContext::TaskComplete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsContext *
        ]
    ),
    "cTk2dLayer::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dLayer *
            ctypes.c_float,  # float
        ]
    ),
    "cTkTextStyle::WriteParagraph": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dText *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['const cTkVector2 *', 'const cTkVector2 *', 'const cTkVector2 *', 'const cTkVector2 *', 'const cTkVector2 *', 'void *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkTextStyle::WriteSentence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_int32,  # int
            ctypes.c_char * 0x8,  # cTkVector2
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkFont *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['const cTkVector2 *', 'const cTkVector2 *', 'const cTkVector2 *', 'const cTkVector2 *', 'const cTkVector2 *', 'void *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkTextStyle::WriteText": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dText *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['const cTkVector2 *', 'const cTkVector2 *', 'const cTkVector2 *', 'const cTkVector2 *', 'const cTkVector2 *', 'void *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkTemplatedComponentUpdater<cTkAnimPoseComponent>::UpdateComponentsRender": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTemplatedComponentUpdater<cTkAnimPoseComponent> *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "ComponentInstancercTkAnimPoseComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercTkAnimPoseComponent *
        ]
    ),
    "ComponentLoadInstancercTkAnimPoseComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentLoadInstancercTkAnimPoseComponent *
        ]
    ),
    "cTkAnimationComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
        ]
    ),
    "cTkAnimationComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
        ]
    ),
    "cTkAnimationComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cTkAnimationComponent::NetSerialize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_ulonglong,  # cGcNetworkSerializer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAnimationComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
        ]
    ),
    "cTkAnimPoseComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
        ]
    ),
    "cTkAnimPoseComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
        ]
    ),
    "cTkAnimPoseComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cTkAnimPoseComponent::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cTkAnimPoseComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
        ]
    ),
    "cTkAnimPoseComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
        ]
    ),
    "cTkAnimPoseComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkAnimTreeBuilder::BlendInfo::BlendInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimTreeBuilder::BlendInfo *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkAnimTreeBuilder::BlendInfo::~BlendInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimTreeBuilder::BlendInfo *
        ]
    ),
    "cTkAnimPoseComponent::~cTkAnimPoseComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
        ]
    ),
    "cTkAnimationComponent::~cTkAnimationComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
        ]
    ),
    "cTkNodeAnimationController::Activate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimLayerHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # cTkAnimLayerHandle *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
        ]
    ),
    "cTkBehaviourTree::AddAlwaysSucceedDecorator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        ]
    ),
    "cTkBehaviourTree::AddConcurrentSelector": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBehaviourTree::AddInverseDecorator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        ]
    ),
    "cTkBehaviourTree::AddNeverFailDecorator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        ]
    ),
    "cTkBehaviourTreeSelector::AddNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeSelector *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        ]
    ),
    "cTkBehaviourTree::AddPrioritySelector": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        ]
    ),
    "cTkBehaviourTree::AddSelectionDecorator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkBehaviourTree::AddSequentialSelector": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBehaviourTree::AddSwitchDecorator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "ComponentInstancercTkAnimPoseComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkAnimationLayer::AnimationIsNull": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationLayer *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
        ]
    ),
    "cTkAnimationLayer::ApplyMemberInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationLayer *
            ctypes.c_ulonglong,  # const cTkMetaDataMember *
        ]
    ),
    "cTkAnimPoseComponent::Babify": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
            ctypes.c_float,  # float
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cTkNodeAnimationController::BlendTo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimLayerHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # cTkAnimLayerHandle *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eCurve
            ctypes.c_uint32,  # eCurve
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimLayerHandle *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAnimTreeBuilder::BuildTreeInfo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimTreeBuilder *
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ulonglong,  # cTkAnimationData **
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # std::unordered_map<TkID<128>,TkID<128>,TkIDUnorderedMap::Hash128,std::equal_to<TkID<128> >,TkSTLAllocatorShim<std::pair<TkID<128> const ,TkID<128> >,8,-1> > *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cTkBehaviourTreeSelector::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeSelector *
        ]
    ),
    "cTkNodeAnimationController::ClearAnimTargetDelta": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
        ]
    ),
    "cTkAnimPoseComponent::ComputeCorrelationsFromExamples": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
        ]
    ),
    "cTkBehaviourTree::Construct": {
        "cTkBehaviourTree *, cTkAttachment *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkBehaviourTree *
                ctypes.c_ulonglong,  # cTkAttachment *
            ]
        ),
        "cTkBehaviourTree *, TkHandle": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkBehaviourTree *
                ctypes.c_uint32,  # TkHandle
            ]
        ),
    },
    "cTkNodeAnimationController::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkAnimationData **
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkAnimBlendTree *
            ctypes.c_int32,  # int
        ]
    ),
    "ComponentInstancercTkAnimPoseComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkAnimPoseComponent::CopyCorrelationsFromData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
        ]
    ),
    "cTkAnimPoseComponent::CopyCorrelationsToData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
        ]
    ),
    "cTkBehaviourTree::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
        ]
    ),
    "ComponentInstancercTkAnimPoseComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkNodeAnimationController::FreezeAllAnims": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
        ]
    ),
    "cTkAnimPoseComponent::GenGaussian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cTkNodeAnimationController::GenerateCommandList": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # cTkStackVector<cTkAnimCommand,16,-1> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimPoseComponent::GenerateExamples": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkAnimationComponent::GetActiveAnimIndices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_ulonglong,  # cTkStackVector<int,16,-1> *
        ]
    ),
    "cTkNodeAnimationController::GetAnimActionFrame": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
        ]
    ),
    "cTkNodeAnimationController::GetAnimActionStartFrame": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
        ]
    ),
    "cTkNodeAnimationController::GetAnimBlendedLocator": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkQuaternion *
        ]
    ),
    "cTkNodeAnimationController::GetAnimDuration": {
        "cTkNodeAnimationController *, const cTkAnimInstanceHandle *": FUNCDEF(
            restype=ctypes.c_double,  # double
            argtypes=[
                ctypes.c_ulonglong,  # cTkNodeAnimationController *
                ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
            ]
        ),
        "cTkNodeAnimationController *, int": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # cTkNodeAnimationController *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cTkNodeAnimationController::GetAnimFrame": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
        ]
    ),
    "cTkNodeAnimationController::GetAnimFrameInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
        ]
    ),
    "cTkNodeAnimationController::GetAnimFromLayerById": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimInstanceHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # cTkAnimInstanceHandle *
            ctypes.c_ulonglong,  # const cTkAnimLayerHandle *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkAnimationComponent::GetAnimIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkNodeAnimationController::GetAnimLocator": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkQuaternion *
        ]
    ),
    "cTkNodeAnimationController::GetAnimMirrored": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
        ]
    ),
    "cTkNodeAnimationController::GetAnimMoveLocatorSpeed": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAnimationLayer::GetAnimMovement": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationLayer *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNodeAnimationController::GetAnimName": {
        "cTkNodeAnimationController *, const cTkAnimInstanceHandle *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkAnimBlendTree *
            argtypes=[
                ctypes.c_ulonglong,  # cTkNodeAnimationController *
                ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
            ]
        ),
        "cTkNodeAnimationController *, int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # cTkAnimBlendTree *
            argtypes=[
                ctypes.c_ulonglong,  # cTkNodeAnimationController *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cTkNodeAnimationController::GetAnimSpeed": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
        ]
    ),
    "cTkNodeAnimationController::GetAnimTime": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
        ]
    ),
    "cTkNodeAnimationController::GetAnimType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNodeAnimationController::GetAnimUnscaledDuration": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAnimationComponent::GetAnimationUnitTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimationComponent::GetAnimsWithPrefix": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAnimationLayer::GetBlendById": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimBlendHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationLayer *
            ctypes.c_ulonglong,  # cTkAnimBlendHandle *
            ctypes.c_ulonglong,  # const cTkAnimLayerHandle *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkNodeAnimationController::GetBlendFromLayerById": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimBlendHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # cTkAnimBlendHandle *
            ctypes.c_ulonglong,  # const cTkAnimLayerHandle *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkNodeAnimationController::GetBlendNodeFromLayer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkAnimBlendNode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimLayerHandle *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkAnimationComponent::GetBlendWeight": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_ulonglong,  # const cTkAnimBlendHandle *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBehaviourTreeSelector::GetChild": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBehaviourTreeNode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeSelector *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAnimationInstance::GetDuration": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationInstance *
        ]
    ),
    "cTkNodeAnimationController::GetFirstAnim": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimInstanceHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # cTkAnimInstanceHandle *
            ctypes.c_ulonglong,  # const cTkAnimLayerHandle *
        ]
    ),
    "cTkAnimationLayer::GetFrame": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationLayer *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
        ]
    ),
    "cTkAnimationComponent::GetGameData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimationGameData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkNodeAnimationController::GetHandleForAnim": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimLayerHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # cTkAnimLayerHandle *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNodeAnimationController::GetHandleFromLayer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimLayerHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # cTkAnimLayerHandle *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAnimationLayer::GetIsPartial": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationLayer *
        ]
    ),
    "cTkNodeAnimationController::GetLayerFromHandle": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimLayerHandle *
        ]
    ),
    "cTkNodeAnimationController::GetLayerFromId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimLayerHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # cTkAnimLayerHandle *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkNodeAnimationController::GetLayerState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimLayerHandle *
        ]
    ),
    "cTkAnimationComponent::GetLocatorForAnim": {
        "cTkAnimationComponent *, const TkID<128> *, const TkID<128> *, float, bool, cTkVector3 *, cTkQuaternion *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cTkAnimationComponent *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
                ctypes.c_ulonglong,  # cTkVector3 *
                ctypes.c_ulonglong,  # cTkQuaternion *
            ]
        ),
        "cTkAnimationComponent *, const cTkAnimInstanceHandle *, const TkID<128> *, float, bool, cTkVector3 *, cTkQuaternion *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cTkAnimationComponent *
                ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
                ctypes.c_ulonglong,  # cTkVector3 *
                ctypes.c_ulonglong,  # cTkQuaternion *
            ]
        ),
    },
    "cTkNodeAnimationController::GetLocatorForAnim": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkQuaternion *
        ]
    ),
    "cTkAnimationComponent::GetMoveLocatorForAnim": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkQuaternion *
        ]
    ),
    "cTkAnimPoseComponent::GetNameOfStateVar": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNodeAnimationController::GetNextAnim": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimInstanceHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # cTkAnimInstanceHandle *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
        ]
    ),
    "cTkNodeAnimationController::GetNumAnimFrames": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkBehaviourTreeSelector::GetNumChildren": FUNCDEF(
        restype=ctypes.c_int64,  # signed __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeSelector *
        ]
    ),
    "cTkAnimationInstance::GetUnitTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationInstance *
            ctypes.c_ulonglong,  # cTkClock *
        ]
    ),
    "cTkAnimationLayer::GetUnitTime": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationLayer *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
        ]
    ),
    "cTkAnimationWeight::GetWeightUncurved": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationWeight *
        ]
    ),
    "cTkAnimationComponent::IsAnimActive": {
        "cTkAnimationComponent *, const TkID<128> *, bool": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cTkAnimationComponent *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cTkAnimationComponent *, const cTkAnimLayerHandle *, bool, bool": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cTkAnimationComponent *
                ctypes.c_ulonglong,  # const cTkAnimLayerHandle *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cTkAnimationComponent::IsAnimActiveByIndex": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimationComponent::IsAnimLooping": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkAnimationComponent::LoadAndMapAnimation": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNodeAnimationController::LoadAnimation": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAnimPoseComponent::LoadStateVectorFromExample": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
            ctypes.c_ulonglong,  # cTkAnimPoseExampleData *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cTkNodeAnimationController::MapAnimation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAnimationComponent::MatchAnimTimes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
        ]
    ),
    "cTkAnimationComponent::Play": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimLayerHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_ulonglong,  # cTkAnimLayerHandle *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eCurve
            ctypes.c_uint32,  # eCurve
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkAnimLayerHandle *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAnimationComponent::PlayAnimAndMatch": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimLayerHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_ulonglong,  # cTkAnimLayerHandle *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eCurve
            ctypes.c_uint32,  # eCurve
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAnimationComponent::PlayAnimAndOffsetByBlend": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimLayerHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_ulonglong,  # cTkAnimLayerHandle *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eCurve
            ctypes.c_uint32,  # eCurve
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAnimationComponent::PlayOnLayer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimLayerHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_ulonglong,  # cTkAnimLayerHandle *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eCurve
            ctypes.c_uint32,  # eCurve
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNodeAnimationController::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_uint32,  # eTkTimeMode
        ]
    ),
    "cTkAnimPoseComponent::Randomise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            common.GcSeed,  # cTkSeed
        ]
    ),
    "cTkAnimPoseComponent::RecomputeFromAnchors": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkEigenFinder::ReconstructMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEigenFinder *
            ctypes.c_ulonglong,  # float *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNodeAnimationController::RefreshParamMembers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
        ]
    ),
    "cTkAnimationComponent::SetAnimActive": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAnimLayerHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_ulonglong,  # cTkAnimLayerHandle *
            ctypes.c_ubyte,  # bool
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkNodeAnimationController::SetAnimFakeMoveLocatorSpeed": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNodeAnimationController::SetAnimMirrored": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNodeAnimationController::SetAnimSpeed": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNodeAnimationController::SetAnimTargetDelta": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkQuaternion *
        ]
    ),
    "cTkAnimPoseComponent::SetAnimTime": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
            ctypes.c_char * 0x10,  # TkID<128>
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkNodeAnimationController::SetAnimTime": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNodeAnimationController::SetAnimTimeWithRestart": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNodeAnimationController::SetAnimWeight": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimLayerHandle *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNodeAnimationController::SetBlendWeight": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimBlendHandle *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNodeAnimationController::SetCanBeRemovedEvenIfControl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimLayerHandle *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNodeAnimationController::SetClock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_uint32,  # eTkTimeMode
        ]
    ),
    "cTkNodeAnimationController::SetLayerClock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimLayerHandle *
            ctypes.c_ulonglong,  # cTkClock *
        ]
    ),
    "cTkNodeAnimationController::SetLayerId": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimLayerHandle *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkNodeAnimationController::SetLayerReplication": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimLayerHandle *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNodeAnimationController::SetMirrored": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNodeAnimationController::SetParamStruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const cTkMetaDataMember *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimationInstance::SetUnitTime": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationInstance *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkClock *
        ]
    ),
    "cTkAnimationLayer::SetUnitTime": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationLayer *
            ctypes.c_ulonglong,  # const cTkAnimInstanceHandle *
            ctypes.c_float,  # float
        ]
    ),
    "cTkAnimationBlendWeight::SetWeight": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationBlendWeight *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkClock *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimationLayer::Start": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationLayer *
            ctypes.c_ulonglong,  # cTkAnimationData **
            ctypes.c_ulonglong,  # cTkSmartResHandle *
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkAnimBlendTree *
            ctypes.c_ulonglong,  # std::unordered_map<TkID<128>,TkID<128>,TkIDUnorderedMap::Hash128,std::equal_to<TkID<128> >,TkSTLAllocatorShim<std::pair<TkID<128> const ,TkID<128> >,8,-1> > *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAnimationComponent::Stop": {
        "cTkAnimationComponent *, const TkID<128> *, float, eCurve": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkAnimationComponent *
                ctypes.c_char_p,  # const TkID<128> *
                ctypes.c_float,  # float
                ctypes.c_uint32,  # eCurve
            ]
        ),
        "cTkAnimationComponent *, const cTkAnimLayerHandle *, float, eCurve": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkAnimationComponent *
                ctypes.c_ulonglong,  # const cTkAnimLayerHandle *
                ctypes.c_float,  # float
                ctypes.c_uint32,  # eCurve
            ]
        ),
    },
    "cTkNodeAnimationController::Stop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ulonglong,  # const cTkAnimLayerHandle *
        ]
    ),
    "SwapMatrixCols": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # float *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNoiseUtils::TransformIndependentGaussVectorToMeanAndCovariance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cTkBehaviourTree::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTree *
            ctypes.c_float,  # float
        ]
    ),
    "cTkBehaviourTreeConcurrentSelector::Update": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeConcurrentSelector *
            ctypes.c_float,  # float
        ]
    ),
    "cTkBehaviourTreeNegateDecorator::Update": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeNegateDecorator *
            ctypes.c_float,  # float
        ]
    ),
    "cTkBehaviourTreeNeverFailDecorator::Update": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeNeverFailDecorator *
            ctypes.c_float,  # float
        ]
    ),
    "cTkBehaviourTreePrioritySelector::Update": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreePrioritySelector *
            ctypes.c_float,  # float
        ]
    ),
    "cTkBehaviourTreeSelectionDecorator::Update": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeSelectionDecorator *
            ctypes.c_float,  # float
        ]
    ),
    "cTkBehaviourTreeSequenceSelector::Update": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeSequenceSelector *
            ctypes.c_float,  # float
        ]
    ),
    "cTkBehaviourTreeSwitchDecorator::Update": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkBehaviourTreeSwitchDecorator *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNodeAnimationController::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNodeAnimationController::UpdateActiveList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNodeAnimationController *
        ]
    ),
    "cTkAnimationLayer::UpdateInput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationLayer *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cTkAnimationComponent::UpdateRemoteAnimations": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cTkAnimationLayer::UpdateStates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationLayer *
            ctypes.c_uint32,  # eAnimType
        ]
    ),
    "cTkAnimationBlend::UpdateWeights": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationBlend *
            ctypes.c_ulonglong,  # cTkAnimationBlendWeight *
            ctypes.c_ulonglong,  # cTkClock *
        ]
    ),
    "cTkAnimationInstance::WriteInstanceCommands": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationInstance *
            ctypes.c_ulonglong,  # cTkStackVector<cTkAnimCommand,16,-1> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAnimationLayer::WriteLayerCommands": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimationLayer *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkStackVector<cTkAnimCommand,16,-1> *
        ]
    ),
    "cTkAnimPoseComponent::ZeroCorrelations": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAnimPoseComponent *
        ]
    ),
    "nms_rapidjson::UTF8<char>::Encode<nms_rapidjson::GenericInsituStringStream<nms_rapidjson::UTF8<char>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericInsituStringStream<nms_rapidjson::UTF8<char> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseArray<1,nms_rapidjson::GenericInsituStringStream<nms_rapidjson::UTF8<char>>,nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericInsituStringStream<nms_rapidjson::UTF8<char> > *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseArray<0,nms_rapidjson::GenericStringStream<nms_rapidjson::UTF8<char>>,nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericStringStream<nms_rapidjson::UTF8<char> > *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseArray<0,TkJsonFileReadStream,nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # TkJsonFileReadStream *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseHex4<nms_rapidjson::GenericStringStream<nms_rapidjson::UTF8<char>>>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericStringStream<nms_rapidjson::UTF8<char> > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>::ParseInsitu<0>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseNumber<1,nms_rapidjson::GenericInsituStringStream<nms_rapidjson::UTF8<char>>,nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericInsituStringStream<nms_rapidjson::UTF8<char> > *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseNumber<0,nms_rapidjson::GenericStringStream<nms_rapidjson::UTF8<char>>,nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericStringStream<nms_rapidjson::UTF8<char> > *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseNumber<0,TkJsonFileReadStream,nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # TkJsonFileReadStream *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseObject<1,nms_rapidjson::GenericInsituStringStream<nms_rapidjson::UTF8<char>>,nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericInsituStringStream<nms_rapidjson::UTF8<char> > *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseObject<0,nms_rapidjson::GenericStringStream<nms_rapidjson::UTF8<char>>,nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericStringStream<nms_rapidjson::UTF8<char> > *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseObject<0,TkJsonFileReadStream,nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # TkJsonFileReadStream *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        ]
    ),
    "nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>::ParseStream<0,nms_rapidjson::UTF8<char>,TkJsonFileReadStream>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # TkJsonFileReadStream *
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseString<1,nms_rapidjson::GenericInsituStringStream<nms_rapidjson::UTF8<char>>,nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericInsituStringStream<nms_rapidjson::UTF8<char> > *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseString<0,nms_rapidjson::GenericStringStream<nms_rapidjson::UTF8<char>>,nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericStringStream<nms_rapidjson::UTF8<char> > *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseString<0,TkJsonFileReadStream,nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # TkJsonFileReadStream *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseValue<1,nms_rapidjson::GenericInsituStringStream<nms_rapidjson::UTF8<char>>,nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericInsituStringStream<nms_rapidjson::UTF8<char> > *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseValue<0,nms_rapidjson::GenericStringStream<nms_rapidjson::UTF8<char>>,nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericStringStream<nms_rapidjson::UTF8<char> > *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        ]
    ),
    "nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator>::ParseValue<0,TkJsonFileReadStream,nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericReader<nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator> *
            ctypes.c_ulonglong,  # TkJsonFileReadStream *
            ctypes.c_ulonglong,  # nms_rapidjson::GenericDocument<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>,nms_rapidjson::CrtAllocator> *
        ]
    ),
    "cTkClassPointer::cTkClassPointer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkDocumentReaderJSON::cTkDocumentReaderJSON": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
        ]
    ),
    "cTkDocumentStreamWriterJSON::cTkDocumentStreamWriterJSON": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
            ctypes.c_uint32,  # const cTkStoragePersistent::Slot
            ctypes.c_ulonglong,  # TkStorage::SaveResult *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cTkDocumentWriterJSON::cTkDocumentWriterJSON": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
            ctypes.c_ulonglong,  # cTkStringAssembler *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "ITkDocumentReader::~ITkDocumentReader": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ITkDocumentReader *
        ]
    ),
    "ITkDocumentWriter::~ITkDocumentWriter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ITkDocumentWriter *
        ]
    ),
    "cTkClassPointer::~cTkClassPointer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "cTkDocumentStreamWriterJSON::AddKey": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkDocumentWriterJSON::AddKey": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkDocumentWriterJSON::AddTabs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
        ]
    ),
    "cTkDocumentWriterJSON::AddTrailingComma": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
        ]
    ),
    "ITkDocumentWriter::AddValue": {
        "ITkDocumentWriter *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # ITkDocumentWriter *
                ctypes.c_int32,  # int
            ]
        ),
        "ITkDocumentWriter *, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # ITkDocumentWriter *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "ITkDocumentWriter *, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # ITkDocumentWriter *
                ctypes.c_float,  # float
            ]
        ),
    },
    "cTkDocumentStreamWriterJSON::AddValue": {
        "cTkDocumentStreamWriterJSON *, const TkID<128> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
                ctypes.c_char_p,  # const TkID<128> *
            ]
        ),
        "cTkDocumentStreamWriterJSON *, const cTkHalfVector4 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
                ctypes.c_ulonglong,  # const cTkHalfVector4 *
            ]
        ),
        "cTkDocumentStreamWriterJSON *, const cTkPhysRelVec3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ]
        ),
        "cTkDocumentStreamWriterJSON *, const cTkSeed *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
                ctypes.c_ulonglong,  # const cTkSeed *
            ]
        ),
        "cTkDocumentStreamWriterJSON *, const cTkVector2 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
                ctypes.c_ulonglong,  # const cTkVector2 *
            ]
        ),
        "cTkDocumentStreamWriterJSON *, const cTkVector3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
        "cTkDocumentStreamWriterJSON *, const cTkVector4 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
                ctypes.c_ulonglong,  # const cTkVector4 *
            ]
        ),
        "cTkDocumentStreamWriterJSON *, const long double": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
                ctypes.c_double,  # const long double
            ]
        ),
        "cTkDocumentStreamWriterJSON *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkDocumentStreamWriterJSON *, const wchar_t *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
                ctypes.c_ulonglong,  # const wchar_t *
            ]
        ),
        "cTkDocumentStreamWriterJSON *, const __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
                ctypes.c_int64,  # const __int64
            ]
        ),
        "cTkDocumentStreamWriterJSON *, const unsigned __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
                ctypes.c_uint64,  # const unsigned __int64
            ]
        ),
        "cTkDocumentStreamWriterJSON *, const bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
                ctypes.c_ubyte,  # const bool
            ]
        ),
    },
    "cTkDocumentWriterJSON::AddValue": {
        "cTkDocumentWriterJSON *, const TkID<128> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
                ctypes.c_char_p,  # const TkID<128> *
            ]
        ),
        "cTkDocumentWriterJSON *, const cTkHalfVector4 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
                ctypes.c_ulonglong,  # const cTkHalfVector4 *
            ]
        ),
        "cTkDocumentWriterJSON *, const cTkPhysRelVec3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
                ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ]
        ),
        "cTkDocumentWriterJSON *, const cTkSeed *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
                ctypes.c_ulonglong,  # const cTkSeed *
            ]
        ),
        "cTkDocumentWriterJSON *, const cTkVector2 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
                ctypes.c_ulonglong,  # const cTkVector2 *
            ]
        ),
        "cTkDocumentWriterJSON *, const cTkVector3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
        "cTkDocumentWriterJSON *, const cTkVector4 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
                ctypes.c_ulonglong,  # const cTkVector4 *
            ]
        ),
        "cTkDocumentWriterJSON *, const long double": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
                ctypes.c_double,  # const long double
            ]
        ),
        "cTkDocumentWriterJSON *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkDocumentWriterJSON *, const wchar_t *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
                ctypes.c_ulonglong,  # const wchar_t *
            ]
        ),
        "cTkDocumentWriterJSON *, const __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
                ctypes.c_int64,  # const __int64
            ]
        ),
        "cTkDocumentWriterJSON *, const unsigned __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
                ctypes.c_uint64,  # const unsigned __int64
            ]
        ),
        "cTkDocumentWriterJSON *, const bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
                ctypes.c_ubyte,  # const bool
            ]
        ),
    },
    "cTkDocumentWriterJSON::AddValueSuffix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
        ]
    ),
    "MetadataNGuiRender::Functions::BeginTree": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkDbghelpLib::Bind": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDbghelpLib *
        ]
    ),
    "nms_rapidjson::Writer<TkJsonFileWriteStream,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0>::Bool": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::Writer<TkJsonFileWriteStream,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "MetadataNGuiRender::Functions::Button": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "CheckReturnFiniteTkHalfVector4": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkHalfVector4 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkHalfVector4 *
            ctypes.c_ulonglong,  # const cTkHalfVector4 *
        ]
    ),
    "CheckReturnFiniteTkVector4": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector4 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector4 *
            ctypes.c_ulonglong,  # const cTkVector4 *
        ]
    ),
    "cTkDocumentStreamWriterJSON::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
        ]
    ),
    "cTkDocumentWriterJSON::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
        ]
    ),
    "cTkDocumentStreamWriterJSON::Close": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
        ]
    ),
    "cTkDocumentWriterJSON::Close": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
        ]
    ),
    "cTkDocumentReaderJSON::CloseParseFileStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
        ]
    ),
    "cTkDocumentReaderJSON::CommonPostParse": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
        ]
    ),
    "cTkDbghelpLib::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkDbghelpLib::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "DigitGen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const DiyFp *
            ctypes.c_ulonglong,  # const DiyFp *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "nms_rapidjson::Writer<TkJsonFileWriteStream,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0>::Double": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::Writer<TkJsonFileWriteStream,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0> *
            ctypes.c_double,  # long double
        ]
    ),
    "nms_rapidjson::Writer<TkJsonFileWriteStream,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0>::EndArray": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::Writer<TkJsonFileWriteStream,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0> *
            ctypes.c_int64,  # __int64
        ]
    ),
    "MetadataNGuiRender::Functions::EndTree": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "nms_rapidjson::GenericValue<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator>>::GetDouble": FUNCDEF(
        restype=ctypes.c_double,  # long double
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::GenericValue<nms_rapidjson::UTF8<char>,nms_rapidjson::MemoryPoolAllocator<nms_rapidjson::CrtAllocator> > *
        ]
    ),
    "cTkMetaData::GetLookup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetaDataFunctionLookup *
        argtypes=[
            ctypes.c_uint64,  # const unsigned __int64
        ]
    ),
    "cTkMetaDataXML::GetLookup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMetaDataXMLFunctionLookup *
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkDocumentStreamWriterJSON::GetWriterFormat": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "Grisu2": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_double,  # long double
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkDocumentReaderJSON::IsCursorNull": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
        ]
    ),
    "cTkDocumentReaderJSON::MoveCursor": {
        "cTkDocumentReaderJSON *, const unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
                ctypes.c_uint32,  # const unsigned int
            ]
        ),
        "cTkDocumentReaderJSON *, const char *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "cTkDocumentStreamWriterJSON::OpenArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkDocumentWriterJSON::OpenArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkDocumentWriterJSON::OpenBrace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDocumentStreamWriterJSON::OpenObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentStreamWriterJSON *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkDocumentWriterJSON::OpenObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentWriterJSON *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkDocumentReaderJSON::OpenParseFileStream": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_uint32,  # const cTkStoragePersistent::Slot
        ]
    ),
    "cTkDocumentReaderJSON::Parse": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkDocumentReaderJSON::ParseInsitu": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "cTkMetaDataXML::ParseXML": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # XMLNode *
        ]
    ),
    "cTkDocumentReaderJSON::PopContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
        ]
    ),
    "cTkDocumentReaderJSON::PopCursor": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
        ]
    ),
    "Prettify": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDocumentReaderJSON::PushContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "TkJsonFileWriteStreamBase::Put": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkJsonFileWriteStreamBase *
            ctypes.c_char,  # char
        ]
    ),
    "cTkDocumentReaderJSON::ReadArraySize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "cTkMetaDataXML::ReadClassPointer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaDataXML::ReadClassPointerArrayXML": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicArray<cTkClassPointer> *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "TkJsonFileReadStream::ReadData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # TkJsonFileReadStream *
        ]
    ),
    "cTkDocumentReaderJSON::ReadStringSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "ITkDocumentReader::ReadValue": {
        "ITkDocumentReader *, unsigned __int8 *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # ITkDocumentReader *
                ctypes.c_ulonglong,  # unsigned __int8 *
            ]
        ),
        "ITkDocumentReader *, __int16 *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # ITkDocumentReader *
                ctypes.c_ulonglong,  # __int16 *
            ]
        ),
        "ITkDocumentReader *, unsigned __int16 *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # ITkDocumentReader *
                ctypes.c_ulonglong,  # unsigned __int16 *
            ]
        ),
        "ITkDocumentReader *, int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # ITkDocumentReader *
                ctypes.c_ulonglong,  # int *
            ]
        ),
        "ITkDocumentReader *, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # ITkDocumentReader *
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
        "ITkDocumentReader *, float *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # ITkDocumentReader *
                ctypes.c_ulonglong,  # float *
            ]
        ),
        "ITkDocumentReader *, TkHandle *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # ITkDocumentReader *
                ctypes.c_ulonglong,  # TkHandle *
            ]
        ),
        "ITkDocumentReader *, cTkDynamicString *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # ITkDocumentReader *
                ctypes.c_ulonglong,  # cTkDynamicString *
            ]
        ),
        "ITkDocumentReader *, cTkDynamicWideString *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # ITkDocumentReader *
                ctypes.c_ulonglong,  # cTkDynamicWideString *
            ]
        ),
    },
    "cTkDocumentReaderJSON::ReadValue": {
        "cTkDocumentReaderJSON *, long double *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
                ctypes.c_ulonglong,  # long double *
            ]
        ),
        "cTkDocumentReaderJSON *, const char **": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
                ctypes.c_char_p,  # const char **
            ]
        ),
        "cTkDocumentReaderJSON *, const wchar_t **": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
                ctypes.c_ulonglong,  # const wchar_t **
            ]
        ),
        "cTkDocumentReaderJSON *, TkID<256> *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
                ctypes.c_ulonglong,  # TkID<256> *
            ]
        ),
        "cTkDocumentReaderJSON *, TkID<128> *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
                ctypes.c_ulonglong,  # TkID<128> *
            ]
        ),
        "cTkDocumentReaderJSON *, cTkHalfVector4 *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
                ctypes.c_ulonglong,  # cTkHalfVector4 *
            ]
        ),
        "cTkDocumentReaderJSON *, cTkPhysRelVec3 *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
                ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ]
        ),
        "cTkDocumentReaderJSON *, cTkSeed *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
                ctypes.c_ulonglong,  # cTkSeed *
            ]
        ),
        "cTkDocumentReaderJSON *, cTkVector2 *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
                ctypes.c_ulonglong,  # cTkVector2 *
            ]
        ),
        "cTkDocumentReaderJSON *, cTkVector3 *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
                ctypes.c_ulonglong,  # cTkVector3 *
            ]
        ),
        "cTkDocumentReaderJSON *, cTkVector4 *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
                ctypes.c_ulonglong,  # cTkVector4 *
            ]
        ),
        "cTkDocumentReaderJSON *, __int64 *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
                ctypes.c_ulonglong,  # __int64 *
            ]
        ),
        "cTkDocumentReaderJSON *, unsigned __int64 *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
                ctypes.c_ulonglong,  # unsigned __int64 *
            ]
        ),
        "cTkDocumentReaderJSON *, bool *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkDocumentReaderJSON *
                ctypes.c_ulonglong,  # bool *
            ]
        ),
    },
    "cTkMetaData::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMetaDataClass *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['cTkClassPointer *', 'cTkLinearMemoryPool *']}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['cTkClassPointer *', 'bool', {'type': 'unsigned', 'name': '__int64'}]}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['cTkClassPointer *']}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['cTkClassPointer *']}
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['const cTkClassPointer *', 'const cTkClassPointer *']}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['cTkClassPointer *', 'const cTkClassPointer *']}
            ctypes.c_ulonglong,  # {'return': 'cTkClassPointer *', 'args': [{'type': 'cTkClassPointer*', 'name': 'result'}]}
            ctypes.c_ulonglong,  # {'return': 'unsigned __int64', 'args': ['const cTkClassPointer *', {'type': 'unsigned', 'name': '__int64'}, 'bool']}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['cTkClassPointer *']}
        ]
    ),
    "cTkMetaDataXML::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['const cTkClassPointer *', 'XMLNode *', 'bool']}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['cTkClassPointer *', 'XMLNode *', 'cTkLinearMemoryPool *']}
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['const cTkClassPointer *', 'const char *']}
        ]
    ),
    "cTkMetaData::RenderClassPointer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "MetadataNGuiRender::RenderData": {
        "const char *, float *, const FloatEditOptions *, const FloatLimits *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # const FloatEditOptions *
                ctypes.c_ulonglong,  # const FloatLimits *
            ]
        ),
        "const char *, TkID<128> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # TkID<128> *
            ]
        ),
        "const char *, cTkSeed *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # cTkSeed *
            ]
        ),
        "const char *, bool *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # bool *
            ]
        ),
        "const char *, const cTkMetaDataEnumLookup *, int, unsigned __int8 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # unsigned __int8 *
            ]
        ),
        "const char *, const cTkMetaDataEnumLookup *, int, int *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # int *
            ]
        ),
    },
    "nms_rapidjson::Writer<TkJsonFileWriteStream,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0>::StartArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::Writer<TkJsonFileWriteStream,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0> *
        ]
    ),
    "nms_rapidjson::Writer<TkJsonFileWriteStream,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0>::String": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::Writer<TkJsonFileWriteStream,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0> *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "nms_rapidjson::Writer<TkJsonFileWriteStream,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0>::WriteBool": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::Writer<TkJsonFileWriteStream,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassPointer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::WriteClassPointerArrayXML": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkClassPointer *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMetaDataEnumLookup *
        ]
    ),
    "TkJsonFileWriteStream::WriteData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # TkJsonFileWriteStream *
        ]
    ),
    "WriteExponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "nms_rapidjson::Writer<TkJsonFileWriteStream,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0>::WriteString": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # nms_rapidjson::Writer<TkJsonFileWriteStream,nms_rapidjson::UTF8<char>,nms_rapidjson::UTF8<char>,nms_rapidjson::CrtAllocator,0> *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkMetaDataXML::WriteToXML": FUNCDEF(
        restype=ctypes.c_ulonglong,  # XMLNode *
        argtypes=[
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # XMLNode *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "xxtea_to_ubyte_array": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "xxtea_decrypt": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "cTkGraphicsManagerBase::AutoSetScreenWidthAndHeight": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManagerBase *
        ]
    ),
    "BounceEaseOut": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "BounceEaseOutIntegral": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "cTkModelResourceRenderer::CalcCameraMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelResourceRenderer *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkModelResourceRenderer::CalcFocusPoint": {
        "TkHandle, const char *, const cTkVector3 *, cTkVector3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_uint32,  # TkHandle
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # cTkVector3 *
            ]
        ),
        "TkHandle, const char *, const cTkVector3 *, cTkPhysRelVec3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_uint32,  # TkHandle
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # const cTkVector3 *
                ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ]
        ),
    },
    "cTkCurveFunction::Calculate": {
        "float, const cTkInOutCurve *": FUNCDEF(
            restype=ctypes.c_double,  # double
            argtypes=[
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # const cTkInOutCurve *
            ]
        ),
        "float, eCurve": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_float,  # float
                ctypes.c_uint32,  # eCurve
            ]
        ),
    },
    "cTkCurveFunction::CalculateIntegral": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eCurve
        ]
    ),
    "cTkLODSettings::CheckMaxObjectDistanceOverride": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkGraphicsManagerBase::ComputeScreenPosArrayImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManagerBase *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkVector4 *
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cTkGraphicsManagerBase::ComputeScreenPosArrayVRImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManagerBase *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkVector4 *
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ubyte,  # const bool
            ctypes.c_int32,  # int
        ]
    ),
    "cTkModelResourceRenderer::Construct": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelResourceRenderer *
            ctypes.c_ulonglong,  # cTkModelRendererData *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': []}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['cTkAttachment *']}
        ]
    ),
    "cTkAABB::CreateFromOOBB": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAABB *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAABB *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkModelResourceRenderer::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelResourceRenderer *
        ]
    ),
    "cTkModelDressroom::DressMaterial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelDressroom *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkModelResourceRenderer::EnqueueRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelResourceRenderer *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkModelResourceRenderer::EnqueueSetClearMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelResourceRenderer *
            ctypes.c_uint32,  # eClearMode
        ]
    ),
    "cTkModelDressroom::EnqueueSetDressedMaterialUniform": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelDressroom *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_char_p,  # const char *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
        ]
    ),
    "cTkShaderBase::FinishAsyncLoadFromFile": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkShaderAsyncLoadData *
            ctypes.c_ulonglong,  # cTkShader *
        ]
    ),
    "cTkScreenshot::FormatAsTGA": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # std::vector<char,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkLODSettings::GenerateRegionOverrideRadius": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "cTkLODSettings::GetAdjustedLODDistance": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
        ]
    ),
    "cTkLODSettings::GetAsteroidCountMultiplier": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkLODSettings::GetAsteroidDividerMultiplier": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkLODSettings::GetAsteroidFadeRangeMultiplier": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[]
    ),
    "cTkSphere::GetHorizonDisc": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkSphere *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cTkLODSettings::GetImposterOverrideDistance": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "cTkLODSettings::GetImposterResolutionMultiplier": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkModelDressroom::GetIndexOfModel": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelDressroom *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkLODSettings::GetMaxAsteroidGenerationPerFrame": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkLODSettings::GetMaxAsteroidGenerationPerFramePulseJump": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkLODSettings::GetNumberOfImposterViews": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkLODSettings::GetOctahedralImpostersEnabled": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[]
    ),
    "cTkGraphicsManagerBase::GetPickRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManagerBase *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cTkGraphicsManagerBase::GetUIScreenDimensions": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector2 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManagerBase *
            ctypes.c_ulonglong,  # cTkVector2 *
        ]
    ),
    "cTkLODSettings::GetViewImpostersFromSpace": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[]
    ),
    "cTkLODSettings::HideRegionOverride": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkModelDressroom::IsModelDressed": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelDressroom *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkAABB::IsPositionInBox": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkAABB *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkGraphicsManagerBase::IsPositionOnScreen": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManagerBase *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_float,  # float
        ]
    ),
    "cTkAABB::IsSphereOverlappingBox": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkAABB *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
        ]
    ),
    "cTkShaderBase::LoadComputeFromFile": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkShader *
        argtypes=[
            ctypes.c_ulonglong,  # cTkShader *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkShaderBase::LoadFromFile": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkShader *
        argtypes=[
            ctypes.c_ulonglong,  # cTkShader *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkModelDressroom::DressedModel::MaterialExtraction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelDressroom::DressedModel *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkModelDressroom::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelDressroom *
        ]
    ),
    "cTkSMShaderWindow::RegisterShaderMillWindows": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkScreenshot::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkScreenshot *
        ]
    ),
    "cTkSMShaderWindow::RenderEditorWindow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkModelDressroom::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelDressroom *
        ]
    ),
    "cTkModelDressroom::RenderNGuiCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkModelResourceRenderer::ResetModelMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelResourceRenderer *
        ]
    ),
    "cTkModelDressroom::Revert": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelDressroom *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkModelDressroom::RevertAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelDressroom *
        ]
    ),
    "cTkModelDressroom::DressedModel::RevertMaterial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelDressroom::DressedModel *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkModelDressroom::RevertNoDelete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelDressroom *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkScreenshot::SavePendingScreenshots": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkScreenshot *
        ]
    ),
    "cTkModelDressroom::SetDressedMaterialUniform": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelDressroom *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_char_p,  # const char *
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
        ]
    ),
    "cTkModelDressroom::SetDressedParamStr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelDressroom *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkShaderBase::StartAsyncLoadFromFile": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkShaderAsyncLoadData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkShaderAsyncLoadData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkModelResourceRenderer::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelResourceRenderer *
            ctypes.c_float,  # const float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkModelResourceRenderer::UpdateCamera": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkModelResourceRenderer *
        ]
    ),
    "cTkMetaData::ReadFromFile<cTkControllerSpecification>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkControllerSpecification *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAttachedDevice::cTkAttachedDevice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachedDevice *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkHmdBase::cTkHmdBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdBase *
        ]
    ),
    "sTkInputDeviceIcons::sTkInputDeviceIcons": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sTkInputDeviceIcons *
            ctypes.c_ulonglong,  # sTkInputDeviceIcons *
        ]
    ),
    "cTkInputPort::AddBindings": {
        "cTkInputPort *, eInputButtonType, ePadType, ITkInputDevice *, int, int, int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkInputPort *
                ctypes.c_uint32,  # eInputButtonType
                ctypes.c_uint32,  # ePadType
                ctypes.c_ulonglong,  # ITkInputDevice *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
            ]
        ),
        "cTkInputPort *, eInputButtonType, int, int, int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkInputPort *
                ctypes.c_uint32,  # eInputButtonType
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
            ]
        ),
        "cTkInputPort *, ePadType, int, int, int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkInputPort *
                ctypes.c_uint32,  # ePadType
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cTkInputPort::AddDevice": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_uint32,  # eInputButtonType
            ctypes.c_uint32,  # ePadType
            ctypes.c_ulonglong,  # ITkInputDevice *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkInputManager::AreActionsBoundToTheSameButton": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eInputPort
        ]
    ),
    "cTkInputPort::AreActionsBoundToTheSameButton": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkInputManager::BeginActionSet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputManager::BeginBindings": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "cTkInputManager::BeginBindingsActionSet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkHmdBase::BeginVRStartupSequence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdBase *
        ]
    ),
    "cTkInputDeviceManagerBase::CheckSupport": {
        "eInputButtonType, eInputAxis": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_uint32,  # eInputButtonType
                ctypes.c_uint32,  # eInputAxis
            ]
        ),
        "eInputButtonType, eInputButton": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_uint32,  # eInputButtonType
                ctypes.c_uint32,  # eInputButton
            ]
        ),
    },
    "cTkInputManager::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "cTkInputPort::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
        ]
    ),
    "cTkAttachedDevice::ClearActions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachedDevice *
        ]
    ),
    "cTkInputManager::ClearCustomisation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_uint32,  # eInputButtonType
        ]
    ),
    "cTkHmdBase::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdBase *
        ]
    ),
    "cTkInputDeviceManagerBase::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManagerBase *
        ]
    ),
    "cTkInputManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "sTkInputStringHelperFns::CountKeywords": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkInputManager::DestroyBindings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
        ]
    ),
    "CAkPlayListSequence::RemoveAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayListSequence *
        ]
    ),
    "cTkInputManager::EditBinding": {
        "cTkInputManager *, int, int, int, eInputAxis": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkInputManager *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # eInputAxis
            ]
        ),
        "cTkInputManager *, int, int, int, eInputButton": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkInputManager *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # eInputButton
            ]
        ),
    },
    "cTkInputPort::EnableDevice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_uint32,  # eInputButtonType
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputManager::EndActionSet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "cTkInputManager::EndBindings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "cTkInputManager::EndBindingsActionSet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "cTkHmdBase::EndVRStartupSequence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdBase *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cTkInputManager::FindActionName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const TryStoreMode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkInputManager::FindActionSetName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
        ]
    ),
    "sTkInputDeviceIconMap::FindAxisIcons": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sTkInputDeviceIcons *
        argtypes=[
            ctypes.c_ulonglong,  # sTkInputDeviceIconMap *
            ctypes.c_uint32,  # eInputAxis
        ]
    ),
    "cTkInputManager::FindAxisName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_uint32,  # eInputAxis
        ]
    ),
    "sTkInputDeviceIconMap::FindButtonIcons": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sTkInputDeviceIcons *
        argtypes=[
            ctypes.c_ulonglong,  # sTkInputDeviceIconMap *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkInputManager::FindButtonName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_uint32,  # eInputButton
        ]
    ),
    "cTkInputDeviceManagerBase::FindIconMap": {
        "cTkInputDeviceManagerBase *, const cTkControllerSpecification *, eInputButtonType, ePadType": FUNCDEF(
            restype=ctypes.c_ulonglong,  # sTkInputDeviceIconMap *
            argtypes=[
                ctypes.c_ulonglong,  # cTkInputDeviceManagerBase *
                ctypes.c_ulonglong,  # const cTkControllerSpecification *
                ctypes.c_uint32,  # eInputButtonType
                ctypes.c_uint32,  # ePadType
            ]
        ),
        "cTkInputDeviceManagerBase *, eInputButtonType, ePadType": FUNCDEF(
            restype=ctypes.c_ulonglong,  # sTkInputDeviceIconMap *
            argtypes=[
                ctypes.c_ulonglong,  # cTkInputDeviceManagerBase *
                ctypes.c_uint32,  # eInputButtonType
                ctypes.c_uint32,  # ePadType
            ]
        ),
    },
    "sTkInputDeviceIconMap::FindOriginButtonOrAxis": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const std::pair<enum eInputButton,enum eInputAxis> *
        argtypes=[
            ctypes.c_ulonglong,  # sTkInputDeviceIconMap *
            ctypes.c_ulonglong,  # const std::pair<enum eInputButton,enum eInputAxis> *
            ctypes.c_uint32,  # eInputHand
            ctypes.c_char_p,  # const char *
        ]
    ),
    "sTkInputDeviceIconMap::FindOriginIcons": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sTkInputDeviceIcons *
        argtypes=[
            ctypes.c_ulonglong,  # sTkInputDeviceIconMap *
            ctypes.c_uint32,  # eInputHand
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkInputManager::GetActionBasis": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkInputManager::GetActionOrientation": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkInputManager::GetActiveActionSet": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "cTkInputPort::GetActiveActionSetName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_ulonglong,  # TkID<128> *
        ]
    ),
    "cTkInputManager::GetAnalogInput": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eInputPort
        ]
    ),
    "cTkInputPort::GetAxis": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_uint32,  # eInputAxis
            ctypes.c_uint32,  # eInputValidation
            ctypes.c_ubyte,  # bool
        ]
    ),
    "sTkInputDeviceHelperFns::GetAxisDevice": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # eInputAxis
        ]
    ),
    "cTkInputManager::GetBindings": {
        "cTkInputManager *, int, int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # std::vector<sTkInputBinding,TkSTLAllocatorShim<sTkInputBinding,4,-1> > *
            argtypes=[
                ctypes.c_ulonglong,  # cTkInputManager *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
            ]
        ),
        "cTkInputManager *, int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # sTkInputBindings *
            argtypes=[
                ctypes.c_ulonglong,  # cTkInputManager *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cTkInputPort::GetButton": FUNCDEF(
        restype=ctypes.c_uint8,  # unsigned __int8
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_uint32,  # eInputButton
            ctypes.c_uint32,  # eInputValidation
            ctypes.c_ubyte,  # bool
        ]
    ),
    "sTkInputDeviceHelperFns::GetButtonDevice": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # eInputButton
        ]
    ),
    "cTkInputManager::GetButtonInput": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eInputValidation
            ctypes.c_uint32,  # eInputPort
        ]
    ),
    "cTkInputManager::GetChordTextTag": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkInputDeviceManagerBase::GetDeviceSpecMetadataFile": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManagerBase *
            ctypes.c_uint32,  # eInputButtonType
            ctypes.c_uint32,  # ePadType
        ]
    ),
    "sTkInputBinding::GetDeviceType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # sTkInputBinding *
        ]
    ),
    "cTkInputManager::GetDeviceWorldTransform": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_uint32,  # eInputDeviceTransform
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_uint32,  # eInputPort
        ]
    ),
    "cTkHmdBase::GetFramebufferViewport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdBase *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkAttachedDevice::GetIcon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const sTkInputDeviceIcons *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachedDevice *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eInputIconType
            ctypes.c_ubyte,  # bool
        ]
    ),
    "sTkInputDeviceIcons::GetIcon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sTkInputDeviceIcons *
        argtypes=[
            ctypes.c_ulonglong,  # sTkInputDeviceIcons *
            ctypes.c_uint32,  # eInputIconType
        ]
    ),
    "cTkInputPort::GetIconFromButton": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sTkInputDeviceIcons *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # eInputIconType
        ]
    ),
    "cTkAttachedDevice::GetIcons": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const sTkInputDeviceIcons *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachedDevice *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputDeviceManagerBase::GetInputAxisDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_uint32,  # eInputAxis
        ]
    ),
    "cTkInputDeviceManagerBase::GetInputButtonDescription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_uint32,  # eInputButton
        ]
    ),
    "cTkOutputPort::GetInputPort": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkInputPort *
        argtypes=[
            ctypes.c_ulonglong,  # cTkOutputPort *
        ]
    ),
    "cTkInputManager::GetLastButtonInputWasTouch": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_uint32,  # eInputPort
        ]
    ),
    "cTkInputManager::GetMousePosition": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # eInputPointerMode *
            ctypes.c_uint32,  # eInputPort
        ]
    ),
    "cTkInputPort::GetMousePosition": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # eInputPointerMode *
        ]
    ),
    "sTkInputDeviceHelperFns::GetUnboundButton": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # eInputButtonType
        ]
    ),
    "cTkInputManager::HackBinding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eInputButton
        ]
    ),
    "cTkInputManager::HasAnalogActionBound": {
        "cTkInputManager *, int, eInputPort": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkInputManager *
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # eInputPort
            ]
        ),
        "cTkInputManager *, int, int, eInputPort": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cTkInputManager *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # eInputPort
            ]
        ),
    },
    "cTkInputManager::HasDigitalActionBound": {
        "cTkInputManager *, int, eInputPort": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkInputManager *
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # eInputPort
            ]
        ),
        "cTkInputManager *, int, int, eInputPort": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cTkInputManager *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # eInputPort
            ]
        ),
    },
    "cTkInputPort::InjectAnalogueAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkInputPort::InjectDigitalAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkInputManager::IsActiveActionSet": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
        ]
    ),
    "sTkInputDeviceHelperFns::IsUnboundButton": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # eInputButton
        ]
    ),
    "cTkInputDeviceManagerBase::LoadDeviceIcons": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sTkInputDeviceIconMap *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManagerBase *
            ctypes.c_ulonglong,  # const cTkControllerSpecification *
            ctypes.c_uint32,  # eInputButtonType
            ctypes.c_uint32,  # ePadType
        ]
    ),
    "cTkInputManager::MergeBindings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eInputButtonType
        ]
    ),
    "cTkInputPort::MonitorAxes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_uint32,  # eInputButtonType
            ctypes.c_uint32,  # eInputValidation
        ]
    ),
    "cTkInputPort::MonitorButtons": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_uint32,  # eInputButtonType
            ctypes.c_uint32,  # eInputValidation
        ]
    ),
    "cTkInputManager::ParseActionName": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkInputManager::ParseActionSetName": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkInputManager::ParseAxisName": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkInputManager::ParseButtonName": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkInputManager::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "sTkInputDeviceHelperFns::PromoteAxisBasis": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # eInputAxisBasis
        ]
    ),
    "cTkInputManager::RegisterAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const TkID<256> *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkInputManager::RegisterActionSetAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkInputManager::RegisterBinding": {
        "cTkInputManager *, int, eInputAxis, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkInputManager *
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # eInputAxis
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkInputManager *, int, eInputButton, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkInputManager *
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # eInputButton
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "cTkInputManager::Release": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "cTkInputPort::RemoveDevice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_ulonglong,  # ITkInputDevice *
        ]
    ),
    "cTkHmdBase::ResetHeadPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdBase *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "sTkInputDeviceIconMap::ResolveIconSizes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sTkInputDeviceIconMap *
        ]
    ),
    "sTkInputDeviceIcons::ResolveIconSizes": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # sTkInputDeviceIcons *
        ]
    ),
    "cTkAttachedDevice::SetActions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachedDevice *
            ctypes.c_ulonglong,  # const std::vector<sTkInputBinding,TkSTLAllocatorShim<sTkInputBinding,4,-1> > *
            ctypes.c_ulonglong,  # const std::vector<sTkInputBinding,TkSTLAllocatorShim<sTkInputBinding,4,-1> > *
        ]
    ),
    "cTkInputManager::SetActiveActionSet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_int32,  # int
        ]
    ),
    "sTkActionState::SetAnalogueValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sTkActionState *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkInputPort::SetAxis": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_uint32,  # eInputAxis
            ctypes.c_float,  # float
        ]
    ),
    "cTkInputPort::SetButton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_uint32,  # eInputButton
        ]
    ),
    "cTkAttachedDevice::SetCustomisation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachedDevice *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkInputManager::SetCustomisation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_uint32,  # eInputButtonType
            ctypes.c_int32,  # int
        ]
    ),
    "cTkInputPort::SetCustomisation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_uint32,  # eInputButtonType
            ctypes.c_int32,  # int
        ]
    ),
    "cTkInputPort::SetDeviceTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_uint32,  # eInputDeviceTransform
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const cTkMatrix44 *
        ]
    ),
    "sTkActionState::SetDigitalValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sTkActionState *
            ctypes.c_int32,  # int
        ]
    ),
    "CAkPath::SetSoundUniqueID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkAttachedDevice::SetGhostActions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAttachedDevice *
            ctypes.c_ulonglong,  # const std::vector<sTkInputBinding,TkSTLAllocatorShim<sTkInputBinding,4,-1> > *
        ]
    ),
    "cTkHmdBase::SetHeadResetOrientation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdBase *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cTkInputPort::SetMotionData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_char * 0x10,  # cTkVector3
            ctypes.c_char * 0x10,  # cTkVector3
        ]
    ),
    "cTkInputManager::SetTouchInputWaitingForReset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eInputPort
        ]
    ),
    "cTkInputDeviceManagerBase::SetUseLeftHandForGyro": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManagerBase *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkInputPort::SwitchActionSet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkInputDeviceManagerBase::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManagerBase *
            ctypes.c_float,  # float
        ]
    ),
    "cTkInputManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_float,  # float
        ]
    ),
    "cTkInputPort::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_float,  # float
        ]
    ),
    "cTkInputManager::UpdateClickToPlay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
        ]
    ),
    "cTkHmdBase::UpdateHeadResetPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdBase *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkImmediateModeBase::cTkImmediateModeBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImmediateModeBase *
        ]
    ),
    "cTkGraphicsAPI::cUniformMemoryManager::Allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _BYTE *
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsAPI::cUniformMemoryManager *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # TkGpuAddress *
        ]
    ),
    "cTkImmediateMode::Begin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImmediateMode *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkDebugRenderer::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDebugRenderer *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkSmartResHandle *
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cTkImmediateMode::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImmediateMode *
        ]
    ),
    "cTkImmediateMode::End": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImmediateMode *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkGraphicsManager::EnqueueRenderToTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManager *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # EgRenderCullMode::Mode
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkCompute::GetOrLoadShader": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkShader *
        argtypes=[
            ctypes.c_ulonglong,  # cTkCompute *
            ctypes.c_uint32,  # EComputeShaders
        ]
    ),
    "cTkGraphicsManager::GetProjectionMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix44 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManager *
            ctypes.c_ulonglong,  # cTkMatrix44 *
        ]
    ),
    "cTkGraphicsManager::GetViewMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManager *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cTkGraphicsManager::InFrustum": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkImmediateMode::InitGeometry": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkImmediateMode *
        ]
    ),
    "cTkGraphicsManager::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManager *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkDebugRenderer::RegisterLogMessage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDebugRenderer *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkDebugRenderer::RegisterText3D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDebugRenderer *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_float,  # float
        ]
    ),
    "cTkGraphicsManager::Release": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManager *
        ]
    ),
    "cTkImmediateMode::RenderLines": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImmediateMode *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkImmediateMode::RenderQuads": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImmediateMode *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkImmediateMode::RenderSingleScreenQuad": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImmediateMode *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # cTkColour *
        ]
    ),
    "cTkImmediateMode::RenderTriangles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImmediateMode *
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkGraphicsManager::ResizeBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManager *
            ctypes.c_char * 0x8,  # cTkVector2
        ]
    ),
    "cTkImmediateMode::SetColour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImmediateMode *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cTkGraphicsManager::SetDimensions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManager *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkImmediateMode::SetTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkImmediateMode *
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGraphicsManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsManager *
            ctypes.c_float,  # float
        ]
    ),
    "cTkDestroyQueue<VkBuffer_T *,16384,0>::Push<void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDestroyQueue<VkBuffer_T *,16384,0> *
            ctypes.c_ulonglong,  # struct VkBuffer_T **
            ctypes.c_ulonglong,  # struct VkDeviceMemory_T *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['struct VkDevice_T *', 'struct VkBuffer_T *', 'const VkAllocationCallbacks *']}
        ]
    ),
    "cTkNGuiTextEditState::cTkNGuiTextEditState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
        ]
    ),
    "cTkNGuiTextEditState::~cTkNGuiTextEditState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
        ]
    ),
    "cTkBase2DRenderer::AddRotate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBase2DRenderer *
            ctypes.c_double,  # double
        ]
    ),
    "cTkBase2DRenderer::AddScale": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBase2DRenderer *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cTkBase2DRenderer::AddTranslate": {
        "cTkBase2DRenderer *, const cTkVector2 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkBase2DRenderer *
                ctypes.c_ulonglong,  # const cTkVector2 *
            ]
        ),
        "cTkBase2DRenderer *, const cTkVector3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkBase2DRenderer *
                ctypes.c_ulonglong,  # const cTkVector3 *
            ]
        ),
    },
    "TextureTiler::AllocAndMapPages": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # TextureTiler *
            ctypes.c_ulonglong,  # VkQueue_T *
            ctypes.c_ulonglong,  # struct VkImage_T *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # int *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkNGuiNodeGraph::AnimateActiveNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "cTkNGuiNodeGraph::AnimateOutputActive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # cTkColour *
        ]
    ),
    "cTk2dRenderer::Begin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dRenderer *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # cTkRenderStateCache *
        ]
    ),
    "cTkBase2DRenderer::Begin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBase2DRenderer *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkRenderStateCache *
        ]
    ),
    "cTkNGuiNodeGraph::Begin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eNGuiSizeType
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cTkResourceCopier::Begin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceCopier *
        ]
    ),
    "cTkNGuiNodeGraph::BeginNode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # void *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # eNGuiInputType *
        ]
    ),
    "cTkUiDataMap<void *>::Clean": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUiDataMap<void *> *
        ]
    ),
    "cTkUiDataMap<cTkNGuiNodeGraph::Connector>::Clean": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUiDataMap<cTkNGuiNodeGraph::Connector> *
        ]
    ),
    "cTkUiDataMap<cTkNGuiNodeGraph::Node>::Clean": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUiDataMap<cTkNGuiNodeGraph::Node> *
        ]
    ),
    "cTkTexture::CommitMemoryDownToMip": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkUiDataMap<cTkNGuiNodeGraph::Node>::CompareIDs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cTkUiDataMap<cTkNGuiNodeGraph::Connector>::CompareIDsBaseOnly": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cTkTexture::CompleteProbe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGnmxContext *
            ctypes.c_int32,  # int
        ]
    ),
    "TkNGuiEffectTools::ComputeCurtainReveal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_double,  # double
            ctypes.c_uint32,  # const eCurve
            ctypes.c_uint32,  # const eCurve
        ]
    ),
    "cTkTexture::ComputeEvictablePages": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkBase2DRenderer::ComputeVRQuadTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBase2DRenderer *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_ulonglong,  # cTkMatrix44 *
        ]
    ),
    "cTk2dRenderer::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dRenderer *
        ]
    ),
    "cTkTexture::CopyBufferToPixelData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # const int
            ctypes.c_uint32,  # const unsigned int
        ]
    ),
    "cTkNGuiStyles::CopyChanges": {
        "cTkNGuiGraphicStyleData *, cTkNGuiGraphicStyleData *, cTkNGuiGraphicStyleData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiGraphicStyleData *
                ctypes.c_ulonglong,  # cTkNGuiGraphicStyleData *
                ctypes.c_ulonglong,  # cTkNGuiGraphicStyleData *
            ]
        ),
        "cTkNGuiTextStyleData *, cTkNGuiTextStyleData *, cTkNGuiTextStyleData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiTextStyleData *
                ctypes.c_ulonglong,  # cTkNGuiTextStyleData *
                ctypes.c_ulonglong,  # cTkNGuiTextStyleData *
            ]
        ),
    },
    "cTkResourceCopier::CopyData": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_char_p,  # const char *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkTexture::CopyPixelDataToBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_ulonglong,  # char *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
        ]
    ),
    "cTkTexture::CouldCommitDownToMip": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkTexture::CreateEmptyTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_uint32,  # eTkTextureType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # eTextureAddressMode
            ctypes.c_uint32,  # eTextureFilterMode
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTextureBase::CreateTexture": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkTexture *
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_uint32,  # eTkTextureType
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # cTkTextureBase::eTexFormat
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eTextureAddressMode
            ctypes.c_uint32,  # eTextureFilterMode
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTexture::DecommitMemoryUpToMip": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkBase2DRenderer::Disable3dMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBase2DRenderer *
        ]
    ),
    "cTkNGuiNodeGraph::DoCurve": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
            ctypes.c_char * 0x8,  # cTkVector2
            ctypes.c_char * 0x8,  # cTkVector2
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiNodeGraph::DoInOut": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNGuiNodeGraph::DoInput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cTkNGuiNodeGraph::DoLink": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # cTkColour *
        ]
    ),
    "cTkNGuiNodeGraph::DoOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cTkResourceCopier::Download": {
        "cTkResourceCopier *, struct VkBuffer_T *, void *, int, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkResourceCopier *
                ctypes.c_ulonglong,  # struct VkBuffer_T *
                ctypes.c_ulonglong,  # void *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
            ]
        ),
        "cTkResourceCopier *, cTkTexture *, unsigned int, unsigned int, const void *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkResourceCopier *
                ctypes.c_ulonglong,  # cTkTexture *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # const void *
            ]
        ),
    },
    "cTkNGuiStyles::DrawScrollArrow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # const float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiStyles::EditGraphicStyle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyle *
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyleData *
        ]
    ),
    "cTkNGuiStyles::EditGraphicStyleData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyleData *
        ]
    ),
    "cTkNGuiStyles::EditTextStyle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextStyle *
            ctypes.c_ulonglong,  # cTkNGuiTextStyleData *
        ]
    ),
    "cTkNGuiStyles::EditTextStyleData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextStyleData *
        ]
    ),
    "cTkBase2DRenderer::Enable3dMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBase2DRenderer *
        ]
    ),
    "cTkBase2DRenderer::End": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBase2DRenderer *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkNGuiNodeGraph::End": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "cTkResourceCopier::End": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceCopier *
        ]
    ),
    "cTkTexture::EndMipStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsContext *
        ]
    ),
    "cTkNGuiNodeGraph::EndNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
        ]
    ),
    "cTkTexture::FrameEndMipStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsContext *
        ]
    ),
    "cTkTexture::FrameStartMipStats": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkNGuiTextEditState::GetCursorIndex": FUNCDEF(
        restype=ctypes.c_ulonglong,  # STB_TexteditState *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
        ]
    ),
    "cTkNGuiTextEditState::GetCursorPosition": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
        ]
    ),
    "GetLinearRowSlicePtch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # __int64 *
            ctypes.c_ulonglong,  # __int64 *
        ]
    ),
    "cTkTexture::GetMipStats": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkMipReportBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "GetResourceTilingInfo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
        ]
    ),
    "cTkNGuiNodeGraph::GetSelectedNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
            ctypes.c_ulonglong,  # std::vector<void *,TkSTLAllocatorShim<void *,8,-1> > *
        ]
    ),
    "cTkNGuiTextEditState::GetSelectionBegin": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
        ]
    ),
    "cTkNGuiTextEditState::GetSelectionEnd": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
        ]
    ),
    "GetStandardBlockSize2D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # cTkTextureBase::eTexFormat
            ctypes.c_ulonglong,  # VkExtent3D *
        ]
    ),
    "TextureTiler::GetStoreFreeSpace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # TextureTiler *
        ]
    ),
    "cTkTexture::GetStoreFreeSpace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkNGuiStyles::GetTextSize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
            ctypes.c_ulonglong,  # void *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cTkNGuiElementData *
        ]
    ),
    "cTkTexture::GetUav": FUNCDEF(
        restype=ctypes.c_ulonglong,  # struct VkImageView_T *
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # VkFormat
        ]
    ),
    "cTkBase2DRenderer::GetWorldMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix44 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkBase2DRenderer *
            ctypes.c_ulonglong,  # cTkMatrix44 *
        ]
    ),
    "cTkNGuiTextEditState::HasSelection": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
        ]
    ),
    "TextureTiler::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TextureTiler *
            ctypes.c_ulonglong,  # struct VkDevice_T *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkNGuiTextEditState::InitBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkTexture::InitMipStats": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[]
    ),
    "cTkNGuiTextEditState::InitialiseState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_float,  # float
        ]
    ),
    "cTkUiDataMap<cTkNGuiNodeGraph::Connector>::IterateElementsWithBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUiDataMap<cTkNGuiNodeGraph::Connector> *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const std::function<void __cdecl(cTkNGuiNodeGraph::Connector *)> *
        ]
    ),
    "cTkTextureBase::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkTextureBase::eTexFormat *
            ctypes.c_ulonglong,  # cTkTexture *
        ]
    ),
    "cTkTextureBase::LoadDDS": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # eTkTextureType
            ctypes.c_uint32,  # cTkTextureBase::eTexFormat
            ctypes.c_ulonglong,  # cTkTexture *
        ]
    ),
    "cTkNGuiNodeGraph::MakeNodeHash": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkNGuiElementID *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
            ctypes.c_ulonglong,  # cTkNGuiElementID *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkNGuiTextEditState::OnClick": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiTextEditState::OnDrag": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiTextEditState::OnKeyPressed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNGuiTextEditState::OnPaste": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNGuiTextEditState::OnRelease": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
        ]
    ),
    "cTkNGuiTextEditState::ReadBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_ulonglong,  # char *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkTextureBase::ReleaseTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
        ]
    ),
    "cTkNGuiStyles::RenderGraphic": {
        "cTkNGui *, cTkNGuiGraphicStyleData *, cTkBBox2d *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGui *
                ctypes.c_ulonglong,  # cTkNGuiGraphicStyleData *
                ctypes.c_ulonglong,  # cTkBBox2d *
            ]
        ),
        "cTkNGui *, void *, cTkBBox2d *, eNGuiInputType, cTkNGuiElementData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGui *
                ctypes.c_ulonglong,  # void *
                ctypes.c_ulonglong,  # cTkBBox2d *
                ctypes.c_uint32,  # eNGuiInputType
                ctypes.c_ulonglong,  # cTkNGuiElementData *
            ]
        ),
    },
    "cTk2dRenderer::RenderQuads": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dRenderer *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cTkBase2DRenderer::RenderScreenQuad": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBase2DRenderer *
            ctypes.c_ulonglong,  # cTkGnmxContext *
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkVertexAttributeMapping *
        ]
    ),
    "cTkNGuiStyles::RenderShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyleData *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTk2dRenderer::RenderTriangles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dRenderer *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # cTkVector2 *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cTkTextureBase::RestoreMipsFromDDS": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eTkTextureType
            ctypes.c_uint32,  # cTkTextureBase::eTexFormat
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "STB_TEXTEDIT_LAYOUTROW": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StbTexteditRow *
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNGuiTextEditState::SanitizeToMinimalPrintableAscii": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
        ]
    ),
    "cTkNGuiTextEditState::SelectAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
        ]
    ),
    "cTkNGuiNodeGraph::SetNodeColour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
            common.Colour,  # cTkColour
            common.Colour,  # cTkColour
        ]
    ),
    "cTkNGuiNodeGraph::SetSelectedNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
            ctypes.c_ulonglong,  # const std::vector<void *,TkSTLAllocatorShim<void *,8,-1> > *
        ]
    ),
    "cTkBase2DRenderer::SetShader": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBase2DRenderer *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTk2dRenderer::SetTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dRenderer *
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
        ]
    ),
    "cTkBase2DRenderer::SetTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBase2DRenderer *
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cTk2dRenderer::SetTextureSRT": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTk2dRenderer *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # cTkTexture *
        ]
    ),
    "cTkBase2DRenderer::SetZ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBase2DRenderer *
            ctypes.c_float,  # float
        ]
    ),
    "cTkResourceCopier::Submit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceCopier *
        ]
    ),
    "cTkResourceCopier::TransferTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceCopier *
            ctypes.c_ulonglong,  # struct VkImage_T *
            ctypes.c_ulonglong,  # struct VkImage_T *
            ctypes.c_ulonglong,  # VkImageLayout *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkTexture *
        ]
    ),
    "TextureTiler::UnmapAndFreePages": {
        "TextureTiler *, VkQueue_T *, struct VkImage_T *, unsigned __int16 *, int, int *, unsigned int, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # TextureTiler *
                ctypes.c_ulonglong,  # VkQueue_T *
                ctypes.c_ulonglong,  # struct VkImage_T *
                ctypes.c_ulonglong,  # unsigned __int16 *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # int *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "TextureTiler *, VkQueue_T *, struct VkImage_T *, unsigned __int16 *, int, unsigned __int64, unsigned int, unsigned int, unsigned int, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # TextureTiler *
                ctypes.c_ulonglong,  # VkQueue_T *
                ctypes.c_ulonglong,  # struct VkImage_T *
                ctypes.c_ulonglong,  # unsigned __int16 *
                ctypes.c_int32,  # int
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "cTkTexture::UpdateMipStats": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_ulonglong,  # TkMipReportBuffer *
            ctypes.c_ulonglong,  # int *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkTexture::UpdateTextureAfterCommandsComplete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_ulonglong,  # void **
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "cTkTextureBase::UpdateTextureSubData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # cTkTextureBase::eTexFormat
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cTkResourceCopier::Upload": {
        "cTkResourceCopier *, struct VkBuffer_T *, const void *, int, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkResourceCopier *
                ctypes.c_ulonglong,  # struct VkBuffer_T *
                ctypes.c_ulonglong,  # const void *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
            ]
        ),
        "cTkResourceCopier *, cTkTexture *, unsigned int, unsigned int, const void *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkResourceCopier *
                ctypes.c_ulonglong,  # cTkTexture *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # const void *
            ]
        ),
    },
    "cTkTextureBase::UploadTextureData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # cTkTextureBase::eTexFormat
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cTkResourceCopier::WaitForUpload": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceCopier *
        ]
    ),
    "cTkNGuiNodeGraph::WasLinkCreatedThisFrame": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph *
            ctypes.c_ulonglong,  # cTkNGuiNodeGraph::LinkInfo *
        ]
    ),
    "TkNGuiEffectTools::nvgFilledBusyRadial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_int32,  # const int
            ctypes.c_float,  # const float
            ctypes.c_uint32,  # const eCurve
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "TkNGuiEffectTools::nvgFilledProgressRadial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_uint32,  # NVGlineCap
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
        ]
    ),
    "TkNGuiEffectTools::nvgPathBusyDroplet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_uint32,  # const eCurve
        ]
    ),
    "TkNGuiEffectTools::nvgProgressSegmentedRing": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # const TkNGuiEffectTools::SegmentedRingConfig *
        ]
    ),
    "stb_text_create_undo_record": FUNCDEF(
        restype=ctypes.c_ulonglong,  # StbUndoRecord *
        argtypes=[
            ctypes.c_ulonglong,  # StbUndoState *
            ctypes.c_int32,  # int
        ]
    ),
    "stb_text_createundo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # StbUndoState *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "stb_text_locate_coord": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "stb_text_redo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_ulonglong,  # STB_TexteditState *
        ]
    ),
    "stb_text_undo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_ulonglong,  # STB_TexteditState *
        ]
    ),
    "stb_textedit_clamp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_ulonglong,  # STB_TexteditState *
        ]
    ),
    "stb_textedit_delete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_ulonglong,  # STB_TexteditState *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "stb_textedit_delete_selection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_ulonglong,  # STB_TexteditState *
        ]
    ),
    "stb_textedit_find_charpos": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StbFindState *
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "stb_textedit_key": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_ulonglong,  # STB_TexteditState *
            ctypes.c_int32,  # int
        ]
    ),
    "stb_textedit_move_to_word_next": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_ulonglong,  # STB_TexteditState *
        ]
    ),
    "stb_textedit_move_to_word_previous": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_ulonglong,  # STB_TexteditState *
        ]
    ),
    "cTkShader::AllocateUniformBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkShader *
            ctypes.c_ulonglong,  # std::array<void *,5> *
            ctypes.c_ulonglong,  # std::vector<TkShaderBufferHandle,TkSTLAllocatorShim<TkShaderBufferHandle,2,-1> > *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkRenderStateCache *
        ]
    ),
    "cTkShaderBase::AllocateUniformBuffersAndSetSRT": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # std::array<void *,5> *
            ctypes.c_ulonglong,  # std::vector<TkShaderBufferHandle,TkSTLAllocatorShim<TkShaderBufferHandle,2,-1> > *
            ctypes.c_ulonglong,  # TkShaderBufferHandle *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkShader::CleanUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkShader *
        ]
    ),
    "cTkShaderBase::GenerateUniformBufferList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkShader *
            ctypes.c_ulonglong,  # std::vector<TkShaderBufferHandle,TkSTLAllocatorShim<TkShaderBufferHandle,2,-1> > *
        ]
    ),
    "HashShaderInputs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkShader::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkShader *
        ]
    ),
    "cTkShaderBase::LoadComputeFromMemory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkShader *
        argtypes=[
            ctypes.c_ulonglong,  # cTkShader *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkShaderBase::LoadFromMemory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkShader *
        argtypes=[
            ctypes.c_ulonglong,  # cTkShader *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "ReflectionRecurseShaderTypeMembers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkShader *
            ctypes.c_ulonglong,  # const SpvReflectBlockVariable *
            ctypes.c_ulonglong,  # TkShaderBufferHandle *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # int *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkShaderBase::ResolveUniformHandle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkShaderConstHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkShaderConstHandle *
            ctypes.c_ulonglong,  # const cTkShader *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkShader::SetShader": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkShader *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkEngineUtils::ComputePhysRelMatFromNode<0>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkEngineUtils::GetNodeChildOBBs<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # std::vector<cTkAABB,TkSTLAllocatorShim<cTkAABB,16,-1> > *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkEngineUtils::GetNodeOBB<1>": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkAABB *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEngineUtils::GetNodeOBB<0>": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkAABB *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDebugHandler::OutputStyled<3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "TkSettingsReader::Read<cTkGameSettings>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkGameSettings *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "TkSettingsReader::Read<cTkGraphicsSettings>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureEffectTrigger *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadFromFile<cTkEngineSettingsMapping>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkEngineSettingsMapping *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cTkGameSettings>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkGameSettings *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadFromFile<cTkGameSettings>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkGameSettings *
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadFromFile<cTkGraphicsSettings>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcCreatureEffectTrigger *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaDataXML::ReadFromFile<cTkGraphicsSettings>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkGraphicsSettings *
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkMetaData::ReadFromFile<cTkTrophyData>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcEntitlementRewardsTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkMetaData::ReadFromFile<cTkLocalisationTable>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkLocalisationTable *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "TkSettingsReader::Write<cTkGameSettings>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGameSettings *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "TkSettingsReader::Write<cTkGraphicsSettings>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsSettings *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkEngineSettings::cTkEngineSettings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
        ]
    ),
    "cTkStringAssembler::cTkStringAssembler": {
        "cTkStringAssembler *, int, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkStringAssembler *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
            ]
        ),
        "cTkStringAssembler *, char *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkStringAssembler *
                ctypes.c_ulonglong,  # char *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cTkStringAssembler::~cTkStringAssembler": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStringAssembler *
        ]
    ),
    "cTkEngineUtils::AddNodesStep": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_ulonglong,  # TkAddNodesHandle *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkStringAssembler::AppendCharBufJsonEncoded": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkStringAssembler *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # const unsigned int
        ]
    ),
    "cTkFSM::ApplyEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFSM *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cTkFSM::ApplyEventToSpecificState": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkFSM *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cTkEngineSettings::ApplySettings": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
            ctypes.c_ulonglong,  # cTkGraphicsSettings *
        ]
    ),
    "cTkLanguageManagerBase::ChangeLanguage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguageManagerBase *
            ctypes.c_uint32,  # eLanguageRegion
        ]
    ),
    "ChangeVsync": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkEngineSettings::ChooseInitialGraphicsPreset": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
            ctypes.c_ulonglong,  # cTkGraphicsSettings *
        ]
    ),
    "cTkEngineSettings::ChooseInitialResolution": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
            ctypes.c_ulonglong,  # cTkGraphicsSettings *
        ]
    ),
    "cTkEngineSettings::ChooseInitialThreadCounts": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
        ]
    ),
    "cTkLanguageManagerBase::CompareIDs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cTkFSM::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFSM *
            ctypes.c_ulonglong,  # const cTkFSMStateOffset *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkLanguageManagerBase::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguageManagerBase *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # eLanguage
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLanguageManagerBase::Contains": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguageManagerBase *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cTkFSM::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFSM *
        ]
    ),
    "TkStringConversion::impl::DoUTF16ToUTF8": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
        ]
    ),
    "TkStringConversion::impl::DoUTF8ToUTF16": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # wchar_t *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkStringAssembler::EnsureExtraCapacity": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkStringAssembler *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkLanguageManagerBase::Find": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const wchar_t *
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguageManagerBase *
            ctypes.c_char * 0x20,  # TkID<256>
        ]
    ),
    "cTkEngineUtils::FindNode": {
        "TkHandle *, TkHandle, unsigned int, int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # TkHandle *
            argtypes=[
                ctypes.c_ulonglong,  # TkHandle *
                ctypes.c_uint32,  # TkHandle
                ctypes.c_uint32,  # unsigned int
                ctypes.c_int32,  # int
            ]
        ),
        "TkHandle *, TkHandle, const char *, int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # TkHandle *
            argtypes=[
                ctypes.c_ulonglong,  # TkHandle *
                ctypes.c_uint32,  # TkHandle
                ctypes.c_char_p,  # const char *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cTkEngineUtils::FindNodeContainingString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkLanguageManager::FindRegionFromPlatform": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguageManager *
        ]
    ),
    "cTkBasicNoiseHelper::GenerateBlob2D": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkBasicNoiseHelper::GenerateGridNoise2D": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNoiseGridData *
            ctypes.c_ulonglong,  # cTkGridNoiseData2D *
            ctypes.c_double,  # double
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkBasicNoiseHelper::NoisePositionOutput *
        ]
    ),
    "cTkBasicNoiseHelper::GenerateHeightVariance2D": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkBasicNoiseHelper::GenerateNoiseFromUberData3D_Vector": {
        "const float *, const float *, const float *, cTkNoiseUberLayerData *, int, float *, cTkVector4 *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const float *
                ctypes.c_ulonglong,  # const float *
                ctypes.c_ulonglong,  # const float *
                ctypes.c_ulonglong,  # cTkNoiseUberLayerData *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # cTkVector4 *
                ctypes.c_int32,  # int
            ]
        ),
        "const GPU::vec3 *, cTkNoiseUberLayerData *, int, float *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const GPU::vec3 *
                ctypes.c_ulonglong,  # cTkNoiseUberLayerData *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # float *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cTkBasicNoiseHelper::GenerateRegionNoise": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBasicNoiseHelper::GenerateTunnel2D": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkBasicNoiseHelper::GenerateUberNoiseFromData2D_Bucketed": {
        "cTkNoiseUberLayerData *, float *, float *, float *, int *, int, float *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkNoiseUberLayerData *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # int *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # float *
                ctypes.c_int32,  # int
            ]
        ),
        "cTkNoiseUberLayerData *, GPU::sVoronoiResults *, int *, int, float *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkNoiseUberLayerData *
                ctypes.c_ulonglong,  # GPU::sVoronoiResults *
                ctypes.c_ulonglong,  # int *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # float *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cTkStringUtilities<char>::GetDecimalCharacter": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[]
    ),
    "cTkEngineSettings::GetDefaultGraphicsSettingsForThisDevice": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkGraphicsSettings *
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
            ctypes.c_ulonglong,  # cTkGraphicsSettings *
        ]
    ),
    "cTkEngineSettings::GetDynamicResScalingSettings": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkDynamicResScalingSettings *
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
        ]
    ),
    "cTkEngineSettings::GetEngineSettingFromString": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkLanguageManagerBase::GetEntry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkLocalisationEntry *
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguageManagerBase *
            ctypes.c_char_p,  # const TkID<256> *
        ]
    ),
    "cTkEngineUtils::GetGroupNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "Http::GetKnownHeaderAsText": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
            ctypes.c_uint32,  # const Http::KnownHeader::Enum
        ]
    ),
    "cTkLanguageManagerBase::GetLanguageCode": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguageManagerBase *
        ]
    ),
    "cTkEngineUtils::GetMasterModelNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # eMasterModelNodeDistance
        ]
    ),
    "cTkEngineUtils::GetMatricesFromNode": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cTkEngineUtils::GetModelNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkStringUtilities<char>::GetNumericalSeparator": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[]
    ),
    "cTkEngineUtils::GetResHandleFromNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkResourceDescriptor *
        ]
    ),
    "cTkEngineSettings::GetScaleDetailFromEngineSetting": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
            ctypes.c_uint32,  # eEngineSetting
        ]
    ),
    "cTkEngineSettings::GetScaleDetailFromTargetEngineSetting": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
            ctypes.c_uint32,  # eEngineSetting
        ]
    ),
    "cTkEngineUtils::GetSeedFromNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSeed *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSeed *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "GetSettingBool": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cTkGraphicsSettings *
            ctypes.c_uint32,  # eEngineSetting
        ]
    ),
    "cTkEngineSettings::GetSettingBool": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
            ctypes.c_uint32,  # eEngineSetting
        ]
    ),
    "GetSettingFloat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkGraphicsSettings *
            ctypes.c_ulonglong,  # const cTkEngineSettingsMapping *
            ctypes.c_uint32,  # eEngineSetting
        ]
    ),
    "cTkEngineSettings::GetSettingFloat": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
            ctypes.c_uint32,  # eEngineSetting
        ]
    ),
    "GetSettingGraphicDetail": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkGraphicsSettings *
            ctypes.c_uint32,  # const eEngineSetting
        ]
    ),
    "cTkEngineSettings::GetSettingGraphicDetail": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
            ctypes.c_uint32,  # eEngineSetting
        ]
    ),
    "GetSettingInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const cTkGraphicsSettings *
            ctypes.c_ulonglong,  # const cTkEngineSettingsMapping *
            ctypes.c_uint32,  # const eEngineSetting
            ctypes.c_int32,  # const int
        ]
    ),
    "cTkEngineSettings::GetSettingInt": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
            ctypes.c_uint32,  # eEngineSetting
        ]
    ),
    "GetSettingSupport": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # eEngineSetting
        ]
    ),
    "cTkEngineSettings::GetSettingSupport": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
            ctypes.c_uint32,  # eEngineSetting
        ]
    ),
    "cTkLanguageManagerBase::GetStringFromEntry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const TryStoreMode *
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguageManagerBase *
            ctypes.c_ulonglong,  # cTkLocalisationEntry *
        ]
    ),
    "cTkBasicVoxelHelper::GetVoxelColourIndex": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_uint32,  # eVoxelType
        ]
    ),
    "cTkBasicVoxelHelper::GetVoxelTypeFromColourIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "cTkEngineSettings::HasSettingFloat": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
            ctypes.c_uint32,  # eEngineSetting
        ]
    ),
    "cTkEngineUtils::IsNodeImposter": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkLanguageManagerBase::Load": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLanguageManagerBase *
        ]
    ),
    "cTkBasicVoxelHelper::MakeSubstanceOnlyMask": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkEngineSettings::OutOfMemoryDropTextureDetail": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
        ]
    ),
    "cGcURL::ParseFromString": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcURL *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cTkEngineSettings::Read": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
        ]
    ),
    "cTkEngineUtils::ReadModelFilename": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "RefreshAniso": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "RefreshDLSSQuality": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "RefreshFFXSR2Quality": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "RefreshFFXSRQuality": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "RefreshLodBiasAndAA": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkFSM::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFSM *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkEngineUtils::RepositionGroupNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkFSM::RequestStateChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFSM *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEngineUtils::RequestTexturesAlwaysResident": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEngineUtils::SetNodeActive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEngineUtils::SetNodeActiveRecursive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEngineUtils::SetNodeMatrixWorld": {
        "TkHandle, const cTkMatrix34 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_uint32,  # TkHandle
                ctypes.c_ulonglong,  # const cTkMatrix34 *
            ]
        ),
        "TkHandle, const cTkPhysRelMat34 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_uint32,  # TkHandle
                ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ]
        ),
    },
    "cTkEngineUtils::SetNodeParent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkEngineUtils::SetSceneNetworkId": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFSM::StateChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFSM *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFSMState::StateChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFSM::StatePrepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFSM *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cTkFSM::StateRelease": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFSM *
            ctypes.c_ulonglong,  # cTkFSMState *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "cTkFSM::StateSupportsEvenOddRendering": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkFSM *
        ]
    ),
    "Http::KnownHeader::ToString": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_uint32,  # Http::KnownHeader::Enum
        ]
    ),
    "cGcURL::ToString": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cGcURL *
            ctypes.c_ulonglong,  # cTkFixedString<512,char> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLanguageManagerBase::Translate": {
        "cTkLanguageManagerBase *, const TkID<256> *": FUNCDEF(
            restype=ctypes.c_char_p,  # const TkID<256> *
            argtypes=[
                ctypes.c_ulonglong,  # cTkLanguageManagerBase *
                ctypes.c_char_p,  # const TkID<256> *
            ]
        ),
        "cTkLanguageManagerBase *, const TkID<256> *, const char *": FUNCDEF(
            restype=ctypes.c_char_p,  # const TkID<256> *
            argtypes=[
                ctypes.c_ulonglong,  # cTkLanguageManagerBase *
                ctypes.c_char_p,  # const TkID<256> *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "cTkLanguageManagerBase *, const char *, const char *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # TkID<256> *
            argtypes=[
                ctypes.c_ulonglong,  # cTkLanguageManagerBase *
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "cTkFSM::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFSM *
            ctypes.c_float,  # float
        ]
    ),
    "cTkEngineSettings::UpdateGraphicsSettings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
            ctypes.c_ubyte,  # const bool
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cTkEngineSettings::UpdateInitialThreadCounts": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkEngineSettings::Write": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEngineSettings *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkUserService::AllowedMultiplayer": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserService *
        ]
    ),
    "cTkClock::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClock *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkProfanityFilterTpfs::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProfanityFilterTpfs *
        ]
    ),
    "cTkTrophyManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrophyManager *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "TkFiber::ConvertFiberToThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkFiber *
        ]
    ),
    "TkFiber::ConvertThreadToFiber": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkFiber *
        ]
    ),
    "cTkJobManager::CreateFence": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkJobHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkJobHandle *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkJobManager::CreateJob": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkJobHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkJobHandle *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['void *']}
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkDependencyCounter *
        ]
    ),
    "cTkJobManager::CreateSuspendableJob": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkJobHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkJobHandle *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['void *']}
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkDependencyCounter *
        ]
    ),
    "cTkJobManager::DefaultJobSuspendedFunction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkUgcManagerBase::DeleteCachedUGCDetails": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUgcManagerBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkProfanityFilterTpfs::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProfanityFilterTpfs *
        ]
    ),
    "cTkTrophyManager::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrophyManager *
        ]
    ),
    "cTkJobManagerThreadWorker::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkJobManagerThreadWorker *
        ]
    ),
    "cTkJobManager::ExecuteJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkJob *
        ]
    ),
    "cTkUnixTimestamp::Format": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<32,char> *
            ctypes.c_int64,  # __int64
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkUserNamesManager::GetCachedName": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserNamesManager *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        ]
    ),
    "cTkJobManager::GetJob": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkJob *
            ctypes.c_ulonglong,  # cTkJobQueue **
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eTkJobGroup
        ]
    ),
    "cTkUserNamesCache::GetName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserNamesCache *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cTkClock::GetSystemFrequency": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[]
    ),
    "cTkClock::GetSystemTime": FUNCDEF(
        restype=wintypes.LARGE_INTEGER,  # LARGE_INTEGER
        argtypes=[]
    ),
    "cTkClock::GetSystemTimeAsSeconds": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[]
    ),
    "cTkClock::GetSystemTimeSinceBootAsSeconds": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[]
    ),
    "cTkUnixTimestamp::GetTimeInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # tm *
        argtypes=[
            ctypes.c_ulonglong,  # tm *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkJobManager::HasJobCompleted": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const TkJobHandle *
        ]
    ),
    "cTkJobManager::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkJobQueueDefinition *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const cTkJobQueueDefinition *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkClock::InitMinFrameTimeSyncPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClock *
        ]
    ),
    "cTkTrophyManager::IsTrophyUnlocked": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrophyManager *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkTrophyManagerBase::IsTrophyUnlocked": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrophyManagerBase *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "JobResumeWrapper": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkTrophyManager::LookupTrophyId": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrophyManager *
            ctypes.c_char * 0x10,  # TkID<128>
        ]
    ),
    "cTkClock::MarkBootTime": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkClock::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkClock *
        ]
    ),
    "cTkTrophyManager::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrophyManager *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLockFreeQueue<cTkJob>::Push": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkLockFreeQueue<cTkJob> *
            ctypes.c_ulonglong,  # const cTkJob *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "QueueSwitchFunction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkJobManager::ReAddSuspendedJob": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkJobHandle *
        argtypes=[
            ctypes.c_ulonglong,  # TkJobHandle *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkDependencyCounter *
        ]
    ),
    "cTkUserService::RequestUserDisplayName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserService *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        ]
    ),
    "cTkJobManager::RunOnEveryThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['void *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkJobManager::RunOneJobFromGroup": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "SceTpfsFree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "SceTpfsMalloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int64 *
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "SceTpfsRealloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _QWORD *
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkJobManager::Shutdown": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkJobManager::ShutdownHavokAllocators": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "SuspendFiberWrapper": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkJobManager::SuspendJob": {
        "__int64 *, void *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # {'return': 'void', 'args': ['void *', 'int', 'void *']}
                ctypes.c_ulonglong,  # void *
            ]
        ),
        "void **": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # void **
            ]
        ),
    },
    "SuspendableJobWrapper": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkJobManager::SwitchToQueue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "TkThread::ThreadCheckEnabled": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[]
    ),
    "TkThread::ThreadCheckSet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkJobManager::ThreadWorker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkTrophyManagerBase::UnlockTrophy": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrophyManagerBase *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkClock::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClock *
        ]
    ),
    "cTkProfanityFilterTpfs::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkProfanityFilterTpfs *
            ctypes.c_float,  # float
        ]
    ),
    "cTkTrophyManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTrophyManager *
        ]
    ),
    "cTkTimeManager::UpdateServerTime": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[]
    ),
    "cTkTrophyManager::UpdateTrophyUnlocksThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkTimeManager::UpdateUTCTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[]
    ),
    "cTkClock::WaitForMinFrameTime": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClock *
        ]
    ),
    "cTkJobManager::WaitForQueue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkClock::ZeroTimeUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkClock *
        ]
    ),
    "cTkAudioWwiseIO::Open": {
        "cTkAudioWwiseIO *, const wchar_t *, AkOpenMode, AkFileSystemFlags *, bool *, AkFileDesc *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkAudioWwiseIO *
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_uint32,  # AkOpenMode
                ctypes.c_ulonglong,  # AkFileSystemFlags *
                ctypes.c_ulonglong,  # bool *
                ctypes.c_ulonglong,  # AkFileDesc *
            ]
        ),
        "cTkAudioWwiseIO *, unsigned int, AkOpenMode, AkFileSystemFlags *, bool *, AkFileDesc *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkAudioWwiseIO *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # AkOpenMode
                ctypes.c_ulonglong,  # AkFileSystemFlags *
                ctypes.c_ulonglong,  # bool *
                ctypes.c_ulonglong,  # AkFileDesc *
            ]
        ),
    },
    "cTkAudioWwiseIO::Read": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioWwiseIO *
            ctypes.c_ulonglong,  # AkFileDesc *
            ctypes.c_ulonglong,  # const AkIoHeuristics *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # AkIOTransferInfo *
        ]
    ),
    "cTkAudioWwiseIO::Write": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioWwiseIO *
            ctypes.c_ulonglong,  # AkFileDesc *
            ctypes.c_ulonglong,  # const AkIoHeuristics *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # AkIOTransferInfo *
        ]
    ),
    "cTkAudioWwiseIO::Close": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioWwiseIO *
            ctypes.c_ulonglong,  # AkFileDesc *
        ]
    ),
    "AkToneGenParamBlock::AkToneGenParamBlock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkToneGenParamBlock *
        ]
    ),
    "TkAudioID::TkAudioID": {
        "TkAudioID *, const char *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # TkAudioID *
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "TkAudioID *, const char *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # TkAudioID *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "cTkAudioWwiseIO::AddOrGetHandle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioWwiseIO *
            ctypes.c_ulonglong,  # const cTkFixedString<256,char> *
        ]
    ),
    "AK::AllocHook": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAudioManager::AudioStreamExecuteCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "cTkAudioManager::AudioStreamFormatCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "cTkAudioParamCache::CacheRunTimeValue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioParamCache *
            ctypes.c_ulonglong,  # TkAudioID *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkAudioParamCache::CacheSwitch": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioParamCache *
            ctypes.c_ulonglong,  # TkAudioID *
            ctypes.c_ulonglong,  # TkAudioID *
        ]
    ),
    "cTkAudioManager::CancelCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "HgVocAlienParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # HgVocAlienParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "cTkDebugHandler::CloseLogFile": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkAudioManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDebugHandler::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "HgVaMIDIStreamDecoder::CopyMorphTargetToSynthCCs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # HgVaMIDIStreamDecoder *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # HgVaCCSetIterator::enumCCSet
        ]
    ),
    "HgVaMIDIStreamDecoder::CopySynthCCsToMorphTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # HgVaMIDIStreamDecoder *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # HgVaCCSetIterator::enumCCSet
        ]
    ),
    "CreateHgVocAlien": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateHgVocAlienParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "cTkAudioWwiseIO::DefaultOpenFs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioWwiseIO *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # AkOpenMode
            ctypes.c_ulonglong,  # AkFileSystemFlags *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # AkFileDesc *
        ]
    ),
    "cTkAudioManager::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
        ]
    ),
    "AkToneGenParamBlock::DoTargetShuffle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkToneGenParamBlock *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "HgVocAlien::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # HgVocAlien *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkAudioManager::FindAudioStream": FUNCDEF(
        restype=ctypes.c_ulonglong,  # ITkAudioStream *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::FreeHook": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "HgVaMidiFile::FreeMIDIBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # HgVaMidiFile *
        ]
    ),
    "AkToneGenParamBlock::GenerateCombinedVarianceAndJitter": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # AkToneGenParamBlock *
        ]
    ),
    "AkToneGenParamBlock::GenerateParameterisedCombinedVarianceAndJitter": {
        "AkToneGenParamBlock *, unsigned int, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # AkToneGenParamBlock *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "AkToneGenParamBlock *, float, float": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # AkToneGenParamBlock *
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
    },
    "HgVocAlien::GetDuration": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # HgVocAlien *
        ]
    ),
    "HgVocAlien::GetEnvelope": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # HgVocAlien *
        ]
    ),
    "cTkAudioManager::GetLanguageFolder": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        ]
    ),
    "AK::GetMemorySize": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "HgVocAlien::GetPluginInfo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # HgVocAlien *
            ctypes.c_ulonglong,  # AkPluginInfo *
        ]
    ),
    "AK::GetTotalReservedMemorySize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "HGOutputDebugString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "HandleAkExecutionSuspension": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "HgVocAlien::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # HgVocAlien *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkSourcePluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "HgVocAlienParams::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # HgVocAlienParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkDebugHandler::IsDebuggerAttached": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "HgVaMIDIStreamDecoder::LerpMorphTargetsToSynthCCs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # HgVaMIDIStreamDecoder *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_uint32,  # HgVaCCSetIterator::enumCCSet
        ]
    ),
    "cTkAudioManager::LoadAudioBank": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::MemAlignHook": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkDebugHandler::OutputString": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # char *
            ctypes.c_uint32,  # cTkDebugHandler::eTkOutputStyle
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAudioManager::Play": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
            ctypes.c_ulonglong,  # TkAudioID *
            ctypes.c_uint64,  # TkAudioObject
        ]
    ),
    "cTkAudioManager::PollToFinishLoadingAudioBanks": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
        ]
    ),
    "cTkAudioManager::PreDestruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
        ]
    ),
    "HgVocAlienMIDIPlayer::Process": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # HgVocAlienMIDIPlayer *
            ctypes.c_float,  # float
        ]
    ),
    "HgVaMIDIStreamDecoder::ProcessEdit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # HgVaMIDIStreamDecoder *
        ]
    ),
    "HgVaMIDIStreamDecoder::PutMIDIChar": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # HgVaMIDIStreamDecoder *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "AkToneGenParamBlock::RandomiseMorphTargetGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkToneGenParamBlock *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # HgVaCCSetIterator::enumCCSet
        ]
    ),
    "HgVaMIDIStreamDecoder::RandomiseSynthCCs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # HgVaMIDIStreamDecoder *
            ctypes.c_uint32,  # HgVaCCSetIterator::enumCCSet
        ]
    ),
    "AK::ReallocAlignHook": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::ReallocHook": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAudioManager::RegisterNamedAudioObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkAudioObject *
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
            ctypes.c_ulonglong,  # TkAudioObject *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkAudioManager::ReleaseAsyncBankStreamingMemory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
        ]
    ),
    "HgVocAlienChannelManager::ReserveChannel": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # HgVocAlienChannelManager *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "HgVocAlien::Reset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # HgVocAlien *
        ]
    ),
    "cTkAudioParamCache::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioParamCache *
        ]
    ),
    "cTkAudioManager::SetAcitveListeners": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
            ctypes.c_uint64,  # TkAudioObject
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkAudioManager::SetAttenuationFactor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
            ctypes.c_float,  # float
            ctypes.c_uint64,  # TkAudioObject
        ]
    ),
    "HgVocAlien::SetDSPFromParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # HgVocAlien *
            ctypes.c_ulonglong,  # VocAlienDSP *
            ctypes.c_ulonglong,  # HgVocAlienParams *
            ctypes.c_float,  # float
        ]
    ),
    "cTkAudioManager::SetDistanceRTPC": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
            ctypes.c_float,  # float
            ctypes.c_uint64,  # TkAudioObject
        ]
    ),
    "VocAlienDSP::SetGlottisFrequency": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VocAlienDSP *
            ctypes.c_ulonglong,  # ispc::Larynx *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkAudioManager::SetLanguageFolder": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
        ]
    ),
    "cTkDebugHandler::SetLogFile": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkAudioManager::SetMultiplePositions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # TkAudioObject
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAudioManager::SetObjectObstructionAndOcclusion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
            ctypes.c_uint64,  # TkAudioObject
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "HgVocAlienParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # HgVocAlienParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "HgVocAlienParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # HgVocAlienParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkAudioManager::SetPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint64,  # TkAudioObject
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkAudioParamCache::SetPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioParamCache *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint64,  # TkAudioObject
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "AkToneGenParamBlock::SetRealtimeParamsFromSynthCCs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkToneGenParamBlock *
            ctypes.c_ulonglong,  # HgVaMIDIStreamDecoder *
        ]
    ),
    "HgVocAlienMIDIPlayer::Start_Play": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # HgVocAlienMIDIPlayer *
            ctypes.c_ulonglong,  # HgVaMIDIStreamDecoder *
            ctypes.c_ulonglong,  # HgVocAlienParams *
        ]
    ),
    "cTkAudioManager::Stop": {
        "cTkAudioManager *, TkAudioObject": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkAudioManager *
                ctypes.c_uint64,  # TkAudioObject
            ]
        ),
        "cTkAudioManager *, TkAudioObject, ITkAudioStream *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # cTkAudioManager *
                ctypes.c_uint64,  # TkAudioObject
                ctypes.c_ulonglong,  # ITkAudioStream *
            ]
        ),
    },
    "HgVocAlien::StopLooping": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # HgVocAlien *
        ]
    ),
    "CAkGainFX::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "HgVocAlienParams::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # HgVocAlienParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "cTkAudioManager::TkAkBankLoadCallbackFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # AKRESULT
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkAudioManager::UnloadAudioBank": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
            ctypes.c_uint32,  # const unsigned int
        ]
    ),
    "cTkAudioManager::UnregisterAudioObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
            ctypes.c_ulonglong,  # TkAudioObject *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAudioManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAudioManager::UpdateAuxSendWithFalloff": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAudioManager *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint64,  # TkAudioObject
            ctypes.c_float,  # const float
        ]
    ),
    "AK::VirtualAllocHook": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "AK::VirtualFreeHook": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "HgVAMIDIFileReader::extractVLV": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # HgVAMIDIFileReader *
        ]
    ),
    "HgVAMIDIFileReader::getUInt16": FUNCDEF(
        restype=ctypes.c_uint16,  # unsigned __int16
        argtypes=[
            ctypes.c_ulonglong,  # HgVAMIDIFileReader *
        ]
    ),
    "HgVAMIDIFileReader::getUInt32": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # HgVAMIDIFileReader *
        ]
    ),
    "cTkStorageCloud::cTkStorageCloud": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStorageTemp *
        ]
    ),
    "TkStorage::Blob::~Blob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkStorage::Blob *
        ]
    ),
    "TkStorage::Blob::Allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # TkStorage::Blob *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkSystemBase::BeginPollableTaskBatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystemBase *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkSaveDataStreamBase::CloseStream": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkSaveDataStreamBase *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "TkThread::CreateEventFlag": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "TkThread::CreateMutexA": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "TkThread::CreateSemaphoreA": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void **
            ctypes.c_int32,  # int
        ]
    ),
    "TkThread::CreateThread": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['void *']}
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "TkThread::DestroyMutex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "cTkSystemBase::EndPollableTaskBatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystemBase *
        ]
    ),
    "TkStorage::Blob::Free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkStorage::Blob *
        ]
    ),
    "cTkSystemBase::HandleMoveToBackgroundExecution": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystemBase *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cTkSystemBase::HandleReturnFromBackgroundExecution": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystemBase *
        ]
    ),
    "cTkSystemBase::HandleSystemShareMenuOpened": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystemBase *
        ]
    ),
    "cTkSystemBase::HandleSystemUIOverlayActivate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystemBase *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cTkSystemBase::HandleSystemUIOverlayDeactivate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystemBase *
        ]
    ),
    "cTkSystemBase::InternalEndPollableTaskBatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystemBase *
        ]
    ),
    "TkThread::LockCriticalSection": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # _RTL_CRITICAL_SECTION *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "TkThread::LockMutex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "TkThread::MemoryReadWriteBarrier": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkThread *
        ]
    ),
    "TkThread::NumLogicalCores": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "TkThread::NumPhysicalCores": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkSaveDataStreamBase::OpenStream": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkSaveDataStreamBase *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # eFileOpenMode
            ctypes.c_ubyte,  # bool
        ]
    ),
    "TkThread::PlatformInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkThread *
        ]
    ),
    "cTkSaveDataStreamBase::ReadStream": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSaveDataStreamBase *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkSaveDataStreamBase::ReadStreamChunk": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSaveDataStreamBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkStoragePersistent::RenderActivityIconOverlay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent *
            ctypes.c_ulonglong,  # const cTkVector2 *
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_float,  # float
        ]
    ),
    "cTkStoragePersistent::ReturnSortedPlayerStateSlots": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent *
            ctypes.c_uint32,  # const cTkStoragePersistent::Slot
            ctypes.c_uint32,  # const cTkStoragePersistent::Slot
            ctypes.c_ubyte,  # const bool
            ctypes.c_ulonglong,  # cTkStackVector<enum cTkStoragePersistent::Slot,30,-1> *
            ctypes.c_ulonglong,  # cTkStackVector<int,30,-1> *
        ]
    ),
    "cTkSystemBase::RunPollableTaskQueue": {
        "cTkSystemBase *, _BOOL8": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cTkSystemBase *
                ctypes.c_ubyte,  # _BOOL8
            ]
        ),
        "cTkSystemBase *, _BOOL8, int": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cTkSystemBase *
                ctypes.c_ubyte,  # _BOOL8
                ctypes.c_int32,  # int
            ]
        ),
    },
    "TkThread::SetCurrentThreadPriority": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "TkThread::SignalEventFlag": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "TkThread::SignalSemaphore": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void **
            ctypes.c_int32,  # int
        ]
    ),
    "StorageSlotSortNewestFirst": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # const StorageSortableSlot *
            ctypes.c_ulonglong,  # const StorageSortableSlot *
        ]
    ),
    "StorageSlotSortOldestFirst": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # const StorageSortableSlot *
            ctypes.c_ulonglong,  # const StorageSortableSlot *
        ]
    ),
    "TkThread::ThreadStub": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "TkThread::TryLockMutex": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # void **
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "TkThread::UnlockMutex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "cTkSystemBase::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystemBase *
            ctypes.c_float,  # float
        ]
    ),
    "TkThread::WaitOnEventFlag": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void **
            ctypes.c_float,  # float
        ]
    ),
    "cTkSaveDataStreamBase::WriteStream": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSaveDataStreamBase *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkSaveDataStreamBase::WriteStreamChunk": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSaveDataStreamBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDestroyQueue<VkPipeline_T *,8192,0>::DestroyObjects<void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDestroyQueue<VkPipeline_T *,8192,0> *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['struct VkDevice_T *', 'struct VkPipeline_T *', 'const VkAllocationCallbacks *']}
        ]
    ),
    "cTkRenderStateCache::ApplyShader": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMiniPipelineStateDescriptor *
            ctypes.c_ulonglong,  # const cTkShader *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cTkRenderStateCache::BeginFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkCameraBehaviourInterpolate::BeginInterpolate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraBehaviourInterpolate *
            ctypes.c_ulonglong,  # cTkCameraManager *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
            ctypes.c_uint32,  # eTkCameraTimeCurve
            ctypes.c_uint32,  # eTkCameraTimeMode
            ctypes.c_float,  # float
        ]
    ),
    "cTkCameraBehaviourFly::CalcFlyMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelMat34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraBehaviourFly *
            ctypes.c_ulonglong,  # cTkPhysRelMat34 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkRenderStateCache::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # const eClearType
            ctypes.c_int32,  # const int
        ]
    ),
    "cTkCamera::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cTkCameraBehaviourFly::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraBehaviourFly *
        ]
    ),
    "cTkCameraBehaviourInterpolate::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraBehaviourInterpolate *
        ]
    ),
    "cTkCameraBehaviourOffset::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraBehaviourOffset *
        ]
    ),
    "CopyMiniStateToFullState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMiniPipelineStateDescriptor *
            ctypes.c_ulonglong,  # const VkVertexInputBindingDescription *
            ctypes.c_ulonglong,  # const VkVertexInputAttributeDescription *
            ctypes.c_ulonglong,  # PipelineStateDescriptor *
        ]
    ),
    "cTkRenderStateCache::DiscardTargetContents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_int32,  # const int
        ]
    ),
    "cTkRenderStateCache::FinaliseRenderState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_ulonglong,  # cTkGnmxContext *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cTkGnmxContext::GetCurrentRenderPass": FUNCDEF(
        restype=ctypes.c_ulonglong,  # struct VkRenderPass_T *
        argtypes=[
            ctypes.c_ulonglong,  # cTkGnmxContext *
        ]
    ),
    "cTkCameraBehaviourOffset::GetOffsetMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraBehaviourOffset *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cTkCameraBehaviourTimeBased::GetTimeCurvePosition": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraBehaviourTimeBased *
        ]
    ),
    "cTkRenderStateCache::Initialise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
        ]
    ),
    "cTkRenderStateCache::NukeFrameBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkRenderStateCache::NukePipelines": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkRenderStateCache::NukeRenderPasses": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkRenderStateCache::PrecachePipeState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMiniPipelineStateDescriptor *
            ctypes.c_ulonglong,  # const VkVertexInputBindingDescription *
            ctypes.c_ulonglong,  # const VkVertexInputAttributeDescription *
        ]
    ),
    "cTkCameraBehaviourOffset::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraBehaviourOffset *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cTkCameraBehaviourFly::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraBehaviourFly *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cTkCameraBehaviourInterpolate::Prepare": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraBehaviourInterpolate *
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_ulonglong,  # cTkCameraManager *
        ]
    ),
    "cTkCameraBehaviourInterpolate::Release": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraBehaviourInterpolate *
        ]
    ),
    "cTkRenderStateCache::RenderBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRenderStateCache::RenderInstancedArrays": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_ulonglong,  # cTkShader *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkRenderStateCache::ResetState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
        ]
    ),
    "cTkRenderStateCache::SetBlendMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_uint32,  # const eTkBlendMode
        ]
    ),
    "cTkRenderStateCache::SetBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # struct VkBuffer_T *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkRenderStateCache::SetColourMask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkRenderStateCache::SetDepthBoundsAllowed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cTkRenderStateCache::SetDepthBoundsEnable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cTkRenderStateCache::SetDepthBoundsRange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
        ]
    ),
    "cTkCamera::SetFOV": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_float,  # float
        ]
    ),
    "cTkCamera::SetFOVFixedDist": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCamera *
            ctypes.c_float,  # float
        ]
    ),
    "cTkRenderStateCache::SetFlag": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_uint32,  # const eTkRenderFlags
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRenderStateCache::SetImage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkTexture *
        ]
    ),
    "cTkRenderStateCache::SetStencilFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_uint32,  # const eTkTestFunction
            ctypes.c_int32,  # const int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const eTkSeparateFaceMask
        ]
    ),
    "cTkRenderStateCache::SetStencilMask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const eTkSeparateFaceMask
        ]
    ),
    "cTkRenderStateCache::SetStencilMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_uint32,  # const eTkStencilMode
            ctypes.c_uint8,  # const unsigned __int8
            ctypes.c_uint8,  # const unsigned __int8
            ctypes.c_uint8,  # const unsigned __int8
        ]
    ),
    "cTkRenderStateCache::SetStencilOp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_uint32,  # const eTkStencilOp
            ctypes.c_uint32,  # const eTkStencilOp
            ctypes.c_uint32,  # const eTkStencilOp
            ctypes.c_uint32,  # const eTkSeparateFaceMask
        ]
    ),
    "cTkRenderStateCache::SetTest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_uint32,  # const eTkTestType
            ctypes.c_uint32,  # const eTkTestFunction
            ctypes.c_float,  # float
        ]
    ),
    "cTkRenderStateCache::SetTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkTexture *
        ]
    ),
    "cTkRenderStateCache::UnbindTexUnitCountersMasked": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkCameraBehaviourInterpolate::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraBehaviourInterpolate *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "cTkCameraBehaviourOffset::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCameraBehaviourOffset *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkCamera *
        ]
    ),
    "TkInputDevicesGLFW::cTkDS4Input::ActivityCheck": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # TkInputDevicesGLFW::cTkDS4Input *
        ]
    ),
    "TkInputDevicesGLFW::cTkKeyboard::ActivityCheck": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # TkInputDevicesGLFW::cTkKeyboard *
        ]
    ),
    "TkInputDevicesGLFW::cTkMouse::ActivityCheck": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # TkInputDevicesGLFW::cTkMouse *
        ]
    ),
    "TkInputDevicesGLFW::cTkPadXInput::ActivityCheck": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # TkInputDevicesGLFW::cTkPadXInput *
        ]
    ),
    "cTkInputDeviceManager::CharacterCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkInputDeviceManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
        ]
    ),
    "cTkInputDeviceManager::ControllerDetectionTaskImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
        ]
    ),
    "cTkInputDeviceManager::DispatchDetectionTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
        ]
    ),
    "cTkInputDeviceManager::FindDevice": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkInputDevicesGLFW::cTkOculusTouch *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
            ctypes.c_uint32,  # eInputButtonType
            ctypes.c_uint32,  # ePadType
            ctypes.c_int32,  # int
        ]
    ),
    "cTkInputDeviceManager::GetDeviceSpecMetadataFile": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
            ctypes.c_uint32,  # eInputButtonType
            ctypes.c_uint32,  # ePadType
        ]
    ),
    "cTkInputDeviceManager::HideMouse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
        ]
    ),
    "cTkInputDeviceManager::IsMouseHidden": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
        ]
    ),
    "cTkInputDeviceManager::KeyCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkInputDeviceManager::LocaliseButton": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
            ctypes.c_uint32,  # eInputButton
        ]
    ),
    "cTkInputDeviceManager::MouseScrollCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
            ctypes.c_double,  # long double
            ctypes.c_double,  # long double
        ]
    ),
    "cTkInputDeviceManager::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
        ]
    ),
    "cTkInputDeviceManager::ProcessDS4": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_ulonglong,  # ScePadData *
            ctypes.c_ulonglong,  # const cTkOutputPort *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cTkInputDeviceManager::ProcessKeyboard": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
            ctypes.c_ulonglong,  # cTkInputPort *
        ]
    ),
    "cTkInputDeviceManager::ProcessMouse": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
            ctypes.c_ulonglong,  # cTkInputPort *
        ]
    ),
    "cTkInputDeviceManager::ProcessXInputPad": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_ulonglong,  # const cTkOutputPort *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "TkInputDevicesGLFW::cTkDS4Input::PullInput": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # TkInputDevicesGLFW::cTkDS4Input *
            ctypes.c_ulonglong,  # cTkInputPort *
        ]
    ),
    "TkInputDevicesGLFW::cTkKeyboard::PullInput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # TkInputDevicesGLFW::cTkKeyboard *
            ctypes.c_ulonglong,  # cTkInputPort *
        ]
    ),
    "TkInputDevicesGLFW::cTkMouse::PullInput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # TkInputDevicesGLFW::cTkMouse *
            ctypes.c_ulonglong,  # cTkInputPort *
        ]
    ),
    "TkInputDevicesGLFW::cTkPadXInput::PullInput": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # TkInputDevicesGLFW::cTkPadXInput *
            ctypes.c_ulonglong,  # cTkInputPort *
        ]
    ),
    "TkInputDevicesGLFW::cTkDS4Input::PushOutput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkInputDevicesGLFW::cTkDS4Input *
            ctypes.c_ulonglong,  # const cTkOutputPort *
        ]
    ),
    "TkInputDevicesGLFW::cTkPadXInput::PushOutput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkInputDevicesGLFW::cTkPadXInput *
            ctypes.c_ulonglong,  # const cTkOutputPort *
        ]
    ),
    "cTkInputDualShock4::ReadImpl": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # ScePadData *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkInputDeviceManager::Release": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
        ]
    ),
    "cTkInputDeviceManager::SetMousePosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkInputDeviceManager::ShowMouse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
        ]
    ),
    "cTkInputDeviceManager::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputDeviceManager *
            ctypes.c_float,  # float
        ]
    ),
    "cTkHmdOpenVR::cTkHmdOpenVR": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
        ]
    ),
    "cTkOpenVRDeviceBridge::ActivityCheck": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkOpenVRDeviceBridge *
        ]
    ),
    "cTkHmdOpenVR::ApplyProjectionScale": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
            ctypes.c_ulonglong,  # cTkMatrix44 *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkHmdOpenVR::BeginFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
        ]
    ),
    "cTkHmdOpenVR::BlitWindowMirror": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
            ctypes.c_ulonglong,  # cTkGnmxContext *
        ]
    ),
    "cTkHmdOpenVR::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
        ]
    ),
    "cTkInputManagerOpenVR::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManagerOpenVR *
        ]
    ),
    "cTkHmdOpenVR::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
        ]
    ),
    "cTkHmdOpenVR::EndFrame": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
        ]
    ),
    "sTkOpenVRData::FindControllerSpecPath": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # sTkOpenVRData *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkInputManager::GetActionSetName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManager *
            ctypes.c_ulonglong,  # TkID<128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkInputManagerOpenVR::GetAnalogInput": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManagerOpenVR *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eInputPort
        ]
    ),
    "cTkInputManagerOpenVR::GetButtonInput": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManagerOpenVR *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eInputValidation
            ctypes.c_uint32,  # eInputPort
        ]
    ),
    "cTkInputManagerOpenVR::GetDeviceSpecMetadataFile": FUNCDEF(
        restype=ctypes.c_ulonglong,  # {'return': 'bool', 'args': [{'type': 'cTkInputDeviceManagerBase*', 'name': 'this'}]}
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManagerOpenVR *
            ctypes.c_uint32,  # eInputButtonType
            ctypes.c_uint32,  # ePadType
        ]
    ),
    "cTkInputManagerOpenVR::GetDeviceVRViewResetLayerName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManagerOpenVR *
        ]
    ),
    "cTkHmdOpenVR::GetFramebuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkHmdOpenVR::GetFramebufferTexture": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSmartResHandle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
            ctypes.c_ulonglong,  # cTkSmartResHandle *
        ]
    ),
    "vr::custom_hidden_area_mesh::GetHmdDeviceFromTrackedDevicePropertyString": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[]
    ),
    "sTkOpenVRAction::GetInputAxisValues": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # sTkOpenVRAction *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cTkInputManagerOpenVR::GetMappedIcon": FUNCDEF(
        restype=ctypes.c_ulonglong,  # sTkInputDeviceIcons *
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManagerOpenVR *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eInputIconType
        ]
    ),
    "cTkHmdOpenVR::GetViewMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix44 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
            ctypes.c_ulonglong,  # cTkMatrix44 *
            ctypes.c_int32,  # int
            ctypes.c_char * 0x10,  # cTkVector3
        ]
    ),
    "cTkHmdOpenVR::GetVulkanDeviceExtensionsRequired": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
            ctypes.c_ulonglong,  # struct VkPhysicalDevice_T *
            ctypes.c_ulonglong,  # std::vector<char const *,TkSTLAllocatorShim<char const *,8,-1> > *
        ]
    ),
    "cTkHmdOpenVR::GetVulkanInstanceExtensionsRequired": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
            ctypes.c_ulonglong,  # std::vector<char const *,TkSTLAllocatorShim<char const *,8,-1> > *
        ]
    ),
    "cTkInputManagerOpenVR::HasAnalogActionBound": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManagerOpenVR *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eInputPort
        ]
    ),
    "cTkInputManagerOpenVR::HasDigitalActionBound": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManagerOpenVR *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eInputPort
        ]
    ),
    "cTkHmdOpenVR::HasInputFocus": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
        ]
    ),
    "cTkOpenVRDeviceBridge::InitIconLookup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkOpenVRDeviceBridge *
        ]
    ),
    "cTkOpenVRDeviceBridge::IsAvailable": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkOpenVRDeviceBridge *
        ]
    ),
    "cTkHmdOpenVR::OnEndStartupSequence": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkHmdOpenVR::OnShutdown": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
        ]
    ),
    "cTkHmdOpenVR::PostPresent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
        ]
    ),
    "cTkHmdOpenVR::PreSubmit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
        ]
    ),
    "cTkHmdOpenVR::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
        ]
    ),
    "cTkInputManagerOpenVR::Prepare": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManagerOpenVR *
        ]
    ),
    "sTkOpenVRAction::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sTkOpenVRAction *
            ctypes.c_ulonglong,  # const sTkInputAction *
            ctypes.c_ulonglong,  # sTkOpenVRActionSet *
        ]
    ),
    "sTkOpenVRActionSet::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sTkOpenVRActionSet *
            ctypes.c_ulonglong,  # const sTkInputActionSet *
            ctypes.c_ulonglong,  # const std::vector<sTkInputAction,TkSTLAllocatorShim<sTkInputAction,8,-1> > *
        ]
    ),
    "sTkOpenVRData::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sTkOpenVRData *
            ctypes.c_ulonglong,  # const std::vector<sTkInputAction,TkSTLAllocatorShim<sTkInputAction,8,-1> > *
            ctypes.c_ulonglong,  # const std::vector<sTkInputActionSet,TkSTLAllocatorShim<sTkInputActionSet,8,-1> > *
        ]
    ),
    "cTkHmdOpenVR::PrepareAttach": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
        ]
    ),
    "cTkHmdOpenVR::ProcessPoses": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
        ]
    ),
    "cTkOpenVRDeviceBridge::PullInput": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkOpenVRDeviceBridge *
            ctypes.c_ulonglong,  # cTkInputPort *
        ]
    ),
    "cTkOpenVRDeviceBridge::PushOutput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkOpenVRDeviceBridge *
            ctypes.c_ulonglong,  # const cTkOutputPort *
        ]
    ),
    "cTkHmdOpenVR::RecenterPose": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
        ]
    ),
    "cTkInputManagerOpenVR::RefreshControlMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManagerOpenVR *
        ]
    ),
    "OpenVRRumble::RegisterVibrationAction": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkFixedString<128,char> *
        ]
    ),
    "cTkHmdOpenVR::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
        ]
    ),
    "cTkHmdOpenVR::RenderHiddenAreaMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "RotateMatrix44": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix44 *
            ctypes.c_char * 0x10,  # cTkVector3
        ]
    ),
    "cTkInputManagerOpenVR::SetActiveActionSet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManagerOpenVR *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkHmdOpenVR::SetFramebuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkHmdOpenVR::SubmitTimingData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
        ]
    ),
    "cTkHmdOpenVR::SubscribeToOpenVREvents": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['vr::VREvent_t'], 'pointer_depth': 1}
        ]
    ),
    "cTkHmdOpenVR::UnsubscribeFromOpenVREvents": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "cTkHmdOpenVR::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHmdOpenVR *
            ctypes.c_float,  # float
        ]
    ),
    "cTkInputManagerOpenVR::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInputManagerOpenVR *
            ctypes.c_float,  # float
        ]
    ),
    "sTkOpenVRAction::Update": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # sTkOpenVRAction *
            ctypes.c_ulonglong,  # cTkInputPort *
            ctypes.c_int32,  # int
        ]
    ),
    "sTkOpenVRData::UpdateOpenVRActionSetState": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # sTkOpenVRData *
        ]
    ),
    "vr::VRCompositor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # vr::IVRCompositor *
        argtypes=[]
    ),
    "WaitPosesJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkFixedString<1024,wchar_t>::Format<wchar_t,0>": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<1024,wchar_t> *
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "cTkUserService::cTkUserService": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserService *
        ]
    ),
    "TkAssertDlgResourceData::ID::~ID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkAssertDlgResourceData::ID *
        ]
    ),
    "cTkUserNamesManager::~cTkUserNamesManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserNamesManager *
        ]
    ),
    "cTkUserService::~cTkUserService": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserService *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "cTkStoragePersistent::Data::AreAnySaveGamesPresent": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent::Data *
        ]
    ),
    "cTkSystem::AssertCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char * 0x40,  # cTkAssertData
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AssertDialogMsgProc": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # HWND__ *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkSystem::CalculateWindowsVersion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
        ]
    ),
    "cTkUserService::CheckUserPresence": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserService *
        ]
    ),
    "cTkStorageTemp::CloseBinary": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkStorageTemp *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "FileHandles::CloseHandles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # FileHandles *
        ]
    ),
    "cTkStoragePersistent::CloseStream": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkStoragePersistent::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent *
        ]
    ),
    "cTkStorageTemp::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStorageTemp *
        ]
    ),
    "cTkSystem::ConstructHardware": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
        ]
    ),
    "cTkSystem::ConstructVideo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_uint32,  # eWindowMode
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cTkSystem::CopyTextToClipboard": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkStorageTemp::Data::CreateFullPath": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::wstring *
        argtypes=[
            ctypes.c_ulonglong,  # cTkStorageTemp::Data *
            ctypes.c_ulonglong,  # std::wstring *
            ctypes.c_ulonglong,  # const std::string *
        ]
    ),
    "cTkStoragePersistent::DeleteData": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent *
            ctypes.c_uint32,  # const cTkStoragePersistent::Slot
            ctypes.c_ulonglong,  # std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
        ]
    ),
    "cTkSystem::DestructHardware": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
        ]
    ),
    "cTkSystem::DetectAvailableMonitors": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
        ]
    ),
    "cTkSystem::DetectAvailableResolutions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # std::vector<cTkIntTuple2<int,0>,TkSTLAllocatorShim<cTkIntTuple2<int,0>,4,-1> > *
        ]
    ),
    "cTkStoragePersistent::FetchSlotStates": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent *
            ctypes.c_ulonglong,  # std::array<cTkStoragePersistent::SlotState,32> *
        ]
    ),
    "cTkStorageTemp::Format": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkStorageTemp *
        ]
    ),
    "cTkSystem::GameHasFocus": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
            ctypes.c_int64,  # __int64
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cTkSystem::GenerateSystemSeed": FUNCDEF(
        restype=ctypes.c_ulonglong,  # SystemSeed *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
            ctypes.c_ulonglong,  # SystemSeed *
        ]
    ),
    "cTkSystem::GenerateUUIDString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
        ]
    ),
    "cTkSystem::GetAdapterFreeMemory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # wchar_t *
            ctypes.c_ulonglong,  # HDC__ *
            ctypes.c_ulonglong,  # HMONITOR__ *
            ctypes.c_ulonglong,  # __int64 *
        ]
    ),
    "cTkUserService::GetBranchName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserService *
        ]
    ),
    "GetFilenameForSlot": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const wchar_t *
        argtypes=[
            ctypes.c_uint32,  # const cTkStoragePersistent::Slot
        ]
    ),
    "cTkUserServiceGalaxy::GetPlatformId": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceGalaxy *
        ]
    ),
    "cTkUserServiceNull::GetPlatformId": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceNull *
        ]
    ),
    "cTkSystem::GetScreenPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkIntTuple2<int,0> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
            ctypes.c_ulonglong,  # cTkIntTuple2<int,0> *
        ]
    ),
    "cTkSystem::GetScreenResolution": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkIntTuple2<int,0> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
            ctypes.c_ulonglong,  # cTkIntTuple2<int,0> *
        ]
    ),
    "cTkSystem::GetTotalAdapterMemoryGiB": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[]
    ),
    "cTkSystem::GetWindowsErrorAsString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkStoragePersistent::HasAccessDeniedWarning": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent *
            ctypes.c_ulonglong,  # char **
        ]
    ),
    "cTkStoragePersistent::HasLowSpaceWarning": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent *
            ctypes.c_ulonglong,  # cTkStoragePersistent::LowSpaceWarningData *
        ]
    ),
    "cTkStoragePersistent::HasValidStreamSaves": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent *
        ]
    ),
    "cTkStoragePersistent::InitStreamMetadata": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent *
            ctypes.c_uint32,  # const cTkStoragePersistent::Slot
            ctypes.c_ulonglong,  # const TkStorage::Blob *
            ctypes.c_ulonglong,  # const int *
        ]
    ),
    "cTkSystem::IntegratedGPUActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
        ]
    ),
    "IsResolutionALessThanResolutionB": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const cTkIntTuple2<int,0> *
            ctypes.c_ulonglong,  # const cTkIntTuple2<int,0> *
        ]
    ),
    "cTkStoragePersistent::IsUsingLegacyFileNaming": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent *
        ]
    ),
    "cTkStoragePersistent::Load": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent *
            ctypes.c_uint32,  # const cTkStoragePersistent::Slot
            ctypes.c_ulonglong,  # TkStorage::Blob *
        ]
    ),
    "cTkStorageTemp::Load": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkStorageTemp *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # TkStorage::Blob *
        ]
    ),
    "cTkStoragePersistent::Data::LoadManifest": FUNCDEF(
        restype=ctypes.c_ulonglong,  # ManifestData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent::Data *
            ctypes.c_uint32,  # cTkStoragePersistent::Slot
        ]
    ),
    "FileHandles::OpenFiles": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # FileHandles *
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "cTkStoragePersistent::OpenStream": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent *
            ctypes.c_uint32,  # const cTkStoragePersistent::Slot
            ctypes.c_uint32,  # eFileOpenMode
        ]
    ),
    "cTkSystem::PasteTextFromClipboard": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkStoragePersistent::Data::cTkSaveDataStreamPC::PlatformReadFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkStorageTemp *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkStorageTemp::WriteBinary": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkStorageTemp *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkSystem::PollEventLoop": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "PostSlackAssert": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAssertData *
        ]
    ),
    "PurecallHandler": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkSystem::ReadRegistryString": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # HKEY__ *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *
        ]
    ),
    "cTkStoragePersistent::ReadStream": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkUserService::RequestAuthenticationToken": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserService *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkStoragePersistent::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent *
            ctypes.c_uint32,  # const cTkStoragePersistent::Slot
            ctypes.c_ulonglong,  # const TkStorage::Blob *
            ctypes.c_ulonglong,  # const TkStorage::Blob *
            ctypes.c_ulonglong,  # const int *
        ]
    ),
    "cTkStorageTemp::Save": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkStorageTemp *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const TkStorage::Blob *
        ]
    ),
    "cTkStoragePersistent::Data::SaveManifest": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent::Data *
            ctypes.c_uint32,  # cTkStoragePersistent::Slot
            ctypes.c_ulonglong,  # const ManifestData *
        ]
    ),
    "cTkSystem::SetGamma": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkSystem::SetScreenDimensions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
            ctypes.c_ulonglong,  # const cTkIntTuple2<int,0> *
            ctypes.c_ulonglong,  # const cTkIntTuple2<int,0> *
        ]
    ),
    "cTkSystem::SetWindowMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eWindowMode
        ]
    ),
    "cTkSystem::SetWorkingDirectory": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "ShowAssertDialog": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # HWND__ *
            ctypes.c_ulonglong,  # cTkAssertData *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cTkSystem::SwapBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkStoragePersistent::SwitchFileNaming": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "cTkSystem::SystemSetCursor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
            ctypes.c_uint32,  # eCursor
        ]
    ),
    "cTkUserService::TryGetUserIdentity": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserService *
            ctypes.c_ulonglong,  # cTkUserService::UserIdentity *
        ]
    ),
    "cTkSystem::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSystem *
            ctypes.c_float,  # float
        ]
    ),
    "cTkSystem::WindowCloseCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct GLFWwindow *
        ]
    ),
    "cTkStoragePersistent::WriteStream": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkStoragePersistent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkDebugHandler::OutputStyled<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkGraphicsAPI::DisplayBuffer::DisplayBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsAPI::DisplayBuffer *
        ]
    ),
    "VmaAllocator_T::VmaAllocator_T": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_ulonglong,  # const VmaAllocatorCreateInfo *
        ]
    ),
    "VmaPoolAllocator<VmaListItem<VmaSuballocation>>::~VmaPoolAllocator<VmaListItem<VmaSuballocation>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaPoolAllocator<VmaListItem<VmaSuballocation> > *
        ]
    ),
    "VmaBlockVector::~VmaBlockVector": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockVector *
        ]
    ),
    "VmaStringBuilder::AddNumber": {
        "VmaStringBuilder *, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # VmaStringBuilder *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "VmaStringBuilder *, unsigned __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # VmaStringBuilder *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
    },
    "VmaBlockMetadata_Buddy::AddPoolStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy *
            ctypes.c_ulonglong,  # VmaPoolStats *
        ]
    ),
    "VmaBlockMetadata_Generic::AddPoolStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Generic *
            ctypes.c_ulonglong,  # VmaPoolStats *
        ]
    ),
    "VmaBlockMetadata_Linear::AddPoolStats": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Linear *
            ctypes.c_ulonglong,  # VmaPoolStats *
        ]
    ),
    "VmaPoolAllocator<VmaListItem<VmaSuballocation>>::Alloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # VmaPoolAllocator<VmaListItem<VmaSuballocation> >::Item *
        argtypes=[
            ctypes.c_ulonglong,  # VmaPoolAllocator<VmaListItem<VmaSuballocation> > *
        ]
    ),
    "cTkGraphicsAPI::AllocBufferMemory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # cTkGraphicsAPI::eDeviceMemoryType
            ctypes.c_ulonglong,  # struct VkBuffer_T *
            ctypes.c_ulonglong,  # TkDeviceMemory *
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "cTkGraphicsAPI::AllocImageMemory": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # cTkGraphicsAPI::eDeviceMemoryType
            ctypes.c_ulonglong,  # struct VkImage_T *
            ctypes.c_ulonglong,  # TkDeviceMemory *
        ]
    ),
    "cTkResourceCopier::AllocStagingBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkResourceCopier *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # struct VkBuffer_T **
            ctypes.c_ulonglong,  # void **
            ctypes.c_int32,  # int
        ]
    ),
    "cTkGraphicsAPI::AllocateShaderResourceView": FUNCDEF(
        restype=ctypes.c_ulonglong,  # struct VkImageView_T *
        argtypes=[
            ctypes.c_uint32,  # eTkTextureType
            ctypes.c_ulonglong,  # struct VkImage_T *
            ctypes.c_ulonglong,  # VkImageCreateInfo *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkGraphicsAPI::AllocateUnorderedAccessView": FUNCDEF(
        restype=ctypes.c_ulonglong,  # struct VkImageView_T *
        argtypes=[
            ctypes.c_uint32,  # VkFormat
            ctypes.c_ulonglong,  # struct VkImage_T *
            ctypes.c_ulonglong,  # VkImageCreateInfo *
        ]
    ),
    "VmaAllocator_T::AllocateVulkanMemory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_ulonglong,  # const VkMemoryAllocateInfo *
            ctypes.c_ulonglong,  # struct VkDeviceMemory_T **
        ]
    ),
    "cTkGraphicsAPI::AllocateVulkanMemory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # struct VkDeviceMemory_T *
        argtypes=[
            ctypes.c_ulonglong,  # const VkMemoryAllocateInfo *
            ctypes.c_ulonglong,  # const VkMemoryAllocateInfo *
            ctypes.c_ulonglong,  # const VkMemoryAllocateInfo *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "cTkGraphicsAPI::ApplyDLSS": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGnmxContext *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # struct VkImage_T *
            ctypes.c_ulonglong,  # struct VkImageView_T *
            ctypes.c_uint32,  # VkFormat
            ctypes.c_ulonglong,  # struct VkImage_T *
            ctypes.c_ulonglong,  # struct VkImageView_T *
            ctypes.c_uint32,  # VkFormat
            ctypes.c_ulonglong,  # struct VkImage_T *
            ctypes.c_ulonglong,  # struct VkImageView_T *
            ctypes.c_uint32,  # VkFormat
            ctypes.c_ulonglong,  # struct VkImage_T *
            ctypes.c_ulonglong,  # struct VkImageView_T *
            ctypes.c_uint32,  # VkFormat
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkDynamicResScaling::ApplyDynamicResScale": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_float,  # const float
            ctypes.c_uint32,  # const unsigned int
        ]
    ),
    "cTkGraphicsAPI::ApplyNeededSemaphores": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VkSemaphore_T *const **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "VmaJsonWriter::BeginArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaJsonWriter *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGraphicsAPI::FrameDescriptorAllocator::BeginFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsAPI::FrameDescriptorAllocator *
        ]
    ),
    "cTkGraphicsAPI::BeginFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "VmaJsonWriter::BeginObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaJsonWriter *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGnmxContext::BeginRenderPass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGnmxContext *
        ]
    ),
    "VmaJsonWriter::BeginValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaJsonWriter *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "VmaAllocator_T::BindBufferMemory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_ulonglong,  # VmaAllocation_T *
            ctypes.c_ulonglong,  # struct VkBuffer_T *
        ]
    ),
    "VmaAllocator_T::BindImageMemory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_ulonglong,  # VmaAllocation_T *
            ctypes.c_ulonglong,  # struct VkImage_T *
        ]
    ),
    "cTkGraphicsAPI::BindSeparateDepthBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cFrameBuffer *
            ctypes.c_ulonglong,  # cTkGnmxContext *
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGraphicsAPI::BlitTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eTextureFilterMode
        ]
    ),
    "VmaBlockMetadata_Buddy::CalcAllocationStatInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy *
            ctypes.c_ulonglong,  # VmaStatInfo *
        ]
    ),
    "VmaBlockMetadata_Generic::CalcAllocationStatInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Generic *
            ctypes.c_ulonglong,  # VmaStatInfo *
        ]
    ),
    "VmaBlockMetadata_Linear::CalcAllocationStatInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Linear *
            ctypes.c_ulonglong,  # VmaStatInfo *
        ]
    ),
    "VmaBlockMetadata_Buddy::CalcAllocationStatInfoNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy *
            ctypes.c_ulonglong,  # VmaStatInfo *
            ctypes.c_ulonglong,  # const VmaBlockMetadata_Buddy::Node *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "VmaBlockMetadata_Buddy::CheckCorruption": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "VmaBlockMetadata_Linear::CleanupAfterFree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Linear *
        ]
    ),
    "cTkGraphicsAPI::ClearNeededSemaphores": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkGraphicsAPI::ClearTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGnmxContext *
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            common.Colour,  # cTkColour
        ]
    ),
    "VmaJsonWriter::ContinueString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaJsonWriter *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkGraphicsTempBuffer::Create": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsTempBuffer *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "VmaBlockVector::CreateBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockVector *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "cTkGraphicsAPI::CreateIndexBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # struct VkBuffer_T **
            ctypes.c_ulonglong,  # TkDeviceMemory *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "VmaAllocator_T::CreatePool": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_ulonglong,  # const VmaPoolCreateInfo *
            ctypes.c_ulonglong,  # VmaPool_T **
        ]
    ),
    "cTkGraphicsAPI::CreateRenderPass": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkRenderPassDesc *
        argtypes=[
            ctypes.c_ulonglong,  # TkRenderPassDesc *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # struct VkImageView_T **
            ctypes.c_ulonglong,  # VkFormat *
            ctypes.c_ulonglong,  # VkImageLayout **
            ctypes.c_ulonglong,  # struct VkImageView_T **
            ctypes.c_uint32,  # VkFormat
            ctypes.c_ulonglong,  # VkImageLayout *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkGraphicsAPI::CreateRenderTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cFrameBuffer *
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # VkFormat *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGraphicsAPI::CreateSampler": {
        "VkSamplerCreateInfo *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # struct VkSampler_T *
            argtypes=[
                ctypes.c_ulonglong,  # VkSamplerCreateInfo *
            ]
        ),
        "eTextureAddressMode, eTextureFilterMode, unsigned int, float, float": FUNCDEF(
            restype=ctypes.c_ulonglong,  # struct VkSampler_T *
            argtypes=[
                ctypes.c_uint32,  # eTextureAddressMode
                ctypes.c_uint32,  # eTextureFilterMode
                ctypes.c_uint32,  # unsigned int
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
    },
    "cTkGraphicsAPI::CreateStorageBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # struct VkBuffer_T **
            ctypes.c_ulonglong,  # TkDeviceMemory *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkGraphicsAPI::CreateVertexBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # struct VkBuffer_T **
            ctypes.c_ulonglong,  # TkDeviceMemory *
            ctypes.c_ulonglong,  # void **
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const cTkVertexLayoutRT *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGraphicsAPI::DLSS_QueryOptimalRenderSize": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # NVSDK_NGX_PerfQuality_Value
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "VmaBlockMetadata_Buddy::DeleteNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy *
            ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy::Node *
        ]
    ),
    "VmaDeviceMemoryBlock::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaDeviceMemoryBlock *
            ctypes.c_ulonglong,  # VmaAllocator_T *
        ]
    ),
    "cTkGraphicsTempBuffer::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsTempBuffer *
        ]
    ),
    "DeviceIsUsable": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # struct VkPhysicalDevice_T *
            ctypes.c_ulonglong,  # std::string *
        ]
    ),
    "cTkGraphicsAPI::DispatchWithState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRenderStateCache *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkGraphicsAPI::EndFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "FindValidAllocator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkDynamicResScaling::ForceDynamicResolutionIndexThisFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "VmaBlockVector::Free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockVector *
            ctypes.c_ulonglong,  # VmaAllocation_T *
        ]
    ),
    "VmaBlockMetadata_Buddy::FreeAtOffset": {
        "VmaBlockMetadata_Buddy *, VmaAllocation_T *, unsigned __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy *
                ctypes.c_ulonglong,  # VmaAllocation_T *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
        "VmaBlockMetadata_Buddy *, unsigned __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
    },
    "VmaBlockMetadata_Generic::FreeAtOffset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Generic *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "VmaBlockMetadata_Linear::FreeAtOffset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Linear *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "VmaAllocator_T::FreeDedicatedMemory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_ulonglong,  # VmaAllocation_T *
        ]
    ),
    "VmaAllocator_T::FreeMemory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # VmaAllocation_T *const *
        ]
    ),
    "cTkGraphicsAPI::FreeMemory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TkDeviceMemory *
        ]
    ),
    "VmaAllocator_T::FreeVulkanMemory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # struct VkDeviceMemory_T *
        ]
    ),
    "VmaBlockMetadata_Buddy::GetAllocationCount": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy *
        ]
    ),
    "VmaBlockMetadata_Generic::GetAllocationCount": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Generic *
        ]
    ),
    "VmaBlockMetadata_Linear::GetAllocationCount": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Linear *
        ]
    ),
    "VmaAllocator_T::GetAllocationInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_ulonglong,  # VmaAllocation_T *
            ctypes.c_ulonglong,  # VmaAllocationInfo *
        ]
    ),
    "cTkDynamicResScaling::GetCurrentDynamicResScale": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[]
    ),
    "cTkDynamicResScaling::GetDynamicScaleForResolution": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # const float
            ctypes.c_uint32,  # const unsigned int
        ]
    ),
    "cTkFfxFsr2::GetJitter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cTkDynamicResScaling::GetLowestPossibleSupportedScalingFactorForGraphicsSettings": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkGraphicsSettings *
        ]
    ),
    "cTkGraphicsAPI::GetPostUpscaleHeight": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkGraphicsAPI::GetPostUpscaleWidth": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkDynamicResScaling::GetSensibleDefaultForGraphicsSettings": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const cTkGraphicsSettings *
        ]
    ),
    "GetStreamingMemoryUsed": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "VmaBlockMetadata_Buddy::GetSumFreeSize": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy *
        ]
    ),
    "VmaBlockMetadata_Buddy::GetUnusedRangeSizeMax": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy *
        ]
    ),
    "VmaBlockMetadata_Generic::GetUnusedRangeSizeMax": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Generic *
        ]
    ),
    "VmaBlockMetadata_Linear::GetUnusedRangeSizeMax": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Linear *
        ]
    ),
    "cTkGraphicsAPI::GetViewMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix44 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkMatrix44 *
            ctypes.c_int64,  # __int64
        ]
    ),
    "Halton": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
        ]
    ),
    "cTkGraphicsAPI::FrameDescriptorAllocator::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsAPI::FrameDescriptorAllocator *
            ctypes.c_ulonglong,  # struct VkDevice_T *
            ctypes.c_ulonglong,  # VkAllocationCallbacks *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # VkDescriptorType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
        ]
    ),
    "VmaAllocator_T::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_ulonglong,  # const VmaAllocatorCreateInfo *
        ]
    ),
    "VmaBlockMetadata_Buddy::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "VmaBlockMetadata_Generic::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Generic *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "VmaBlockMetadata_Linear::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Linear *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "VmaDeviceMemoryBlock::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaDeviceMemoryBlock *
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # struct VkDeviceMemory_T *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkGnmxContext::Init": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkGnmxContext *
            ctypes.c_ulonglong,  # struct VkDevice_T *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGraphicsAPI::Init": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[]
    ),
    "cTkGraphicsAPI::InitBackBuffer": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkGraphicsAPI::InitVideoSubsystem": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[]
    ),
    "cTkGraphicsAPI::InitializeNGX": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "VmaBlockMetadata_Buddy::IsEmpty": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy *
        ]
    ),
    "VmaBlockMetadata_Generic::IsEmpty": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Generic *
        ]
    ),
    "VmaBlockMetadata_Linear::IsEmpty": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Linear *
        ]
    ),
    "LoadIridasLut": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # std::array<unsigned short,1048576> *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "VmaBlockMetadata_Generic::MakeAllocationsLost": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Generic *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "VmaBlockMetadata_Linear::MakeAllocationsLost": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Linear *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "VmaBlockMetadata_Buddy::MakeRequestedAllocationsLost": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # VmaAllocationRequest *
        ]
    ),
    "VmaBlockMetadata_Generic::MakeRequestedAllocationsLost": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Generic *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # VmaAllocationRequest *
        ]
    ),
    "VmaBlockMetadata_Linear::MakeRequestedAllocationsLost": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Linear *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # VmaAllocationRequest *
        ]
    ),
    "cTkGraphicsTempBuffer::MallocOnOverFlow": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cEgModelNodeTemplate *
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsTempBuffer *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "VmaAllocator_T::Map": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_ulonglong,  # VmaAllocation_T *
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "MemoryPanic": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkGraphicsAPI::PostInit": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[]
    ),
    "cTkFfxFsr2::PrepareLanczos2LutTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "VmaBlockMetadata_Buddy::PrintDetailedMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy *
            ctypes.c_ulonglong,  # VmaJsonWriter *
        ]
    ),
    "VmaBlockMetadata_Generic::PrintDetailedMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Generic *
            ctypes.c_ulonglong,  # VmaJsonWriter *
        ]
    ),
    "VmaBlockMetadata_Linear::PrintDetailedMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Linear *
            ctypes.c_ulonglong,  # VmaJsonWriter *
        ]
    ),
    "VmaBlockMetadata_Buddy::PrintDetailedMapNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy *
            ctypes.c_ulonglong,  # VmaJsonWriter *
            ctypes.c_ulonglong,  # const VmaBlockMetadata_Buddy::Node *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "VmaBlockMetadata::PrintDetailedMap_Allocation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata *
            ctypes.c_ulonglong,  # VmaJsonWriter *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # VmaAllocation_T *
        ]
    ),
    "VmaBlockMetadata::PrintDetailedMap_Begin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata *
            ctypes.c_ulonglong,  # VmaJsonWriter *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "VmaBlockMetadata::PrintDetailedMap_End": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata *
            ctypes.c_ulonglong,  # VmaJsonWriter *
        ]
    ),
    "VmaBlockMetadata::PrintDetailedMap_UnusedRange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata *
            ctypes.c_ulonglong,  # VmaJsonWriter *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "VmaAllocation_T::PrintParameters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocation_T *
            ctypes.c_ulonglong,  # VmaJsonWriter *
        ]
    ),
    "VmaRawList<VmaSuballocation>::PushBack": FUNCDEF(
        restype=ctypes.c_ulonglong,  # VmaPoolAllocator<VmaListItem<VmaSuballocation> >::Item *
        argtypes=[
            ctypes.c_ulonglong,  # VmaRawList<VmaSuballocation> *
        ]
    ),
    "cTkGraphicsAPI::ReleaseDLSSFeatures": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkGraphicsAPI::ReleaseShaderResourceView": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct VkImageView_T *
        ]
    ),
    "cTkGraphicsAPI::RenderToBufferScaleWidth": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkGnmxContext::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGnmxContext *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkGraphicsTempBuffer::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGraphicsTempBuffer *
        ]
    ),
    "cTkGraphicsAPI::ResetRenderer": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkGraphicsAPI::SetDLSSQuality": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # NVSDK_NGX_PerfQuality_Value
        ]
    ),
    "cTkGraphicsAPI::SetFFXSRQuality": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # FFXSR_PerfQuality_Value
        ]
    ),
    "cTkGraphicsAPI::SetFrameFlipRate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # eTkFlipRate
        ]
    ),
    "cTkGraphicsAPI::SetProjectionMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const cTkMatrix44 *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkGraphicsAPI::SetProjectionMatrixOrtho": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkFfxFsr2::SetQualityScaleFromLevel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # eFfxFsr2QualityLevel
        ]
    ),
    "SetShaderConsts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # FfxFsr2ContextCustom *
        ]
    ),
    "cTkGraphicsAPI::SetShaderResourceView": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct VkImageView_T **
            ctypes.c_ulonglong,  # struct VkImage_T *
            ctypes.c_ulonglong,  # VkImageCreateInfo *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkGraphicsAPI::SetTextureAddressMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # eTkTextureType
            ctypes.c_uint32,  # eTextureAddressMode
        ]
    ),
    "VmaAllocation_T::SetUserData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocation_T *
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkGraphicsAPI::SetViewMatrix": {
        "const cTkMatrix44 *, __int64, __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const cTkMatrix44 *
                ctypes.c_int64,  # __int64
                ctypes.c_int64,  # __int64
            ]
        ),
        "const cTkMatrix44 *, __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const cTkMatrix44 *
                ctypes.c_int64,  # __int64
            ]
        ),
    },
    "cTkGraphicsAPI::ShutdownNGX": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkGraphicsAPI::SwapBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "TkDebugReportCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # VkDebugUtilsMessageSeverityFlagBitsEXT
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const VkDebugUtilsMessengerCallbackDataEXT *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkGnmxContext::Transition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkGnmxContext *
            ctypes.c_ulonglong,  # struct VkImage_T *
            ctypes.c_ulonglong,  # VkImageLayout *
            ctypes.c_uint32,  # VkImageLayout
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "VmaAllocator_T::Unmap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_ulonglong,  # VmaAllocation_T *
        ]
    ),
    "cTkFfxFsr2::UpdateConfig": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkGraphicsAPI::UpdateDLSSConfig": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkFfxFsr2::UpdateShaderConsts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const sTkFfxFsr2Params *
        ]
    ),
    "VmaBlockMetadata_Buddy::Validate": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy *
        ]
    ),
    "VmaBlockMetadata_Generic::Validate": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Generic *
        ]
    ),
    "VmaBlockMetadata_Linear::Validate": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Linear *
        ]
    ),
    "VmaBlockMetadata_Generic::ValidateFreeSuballocationList": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Generic *
        ]
    ),
    "VmaBlockMetadata_Buddy::ValidateNode": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy *
            ctypes.c_ulonglong,  # VmaBlockMetadata_Buddy::ValidationContext *
            ctypes.c_ulonglong,  # const VmaBlockMetadata_Buddy::Node *
            ctypes.c_ulonglong,  # const VmaBlockMetadata_Buddy::Node *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "VmaRecorder::WriteConfiguration": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaRecorder *
            ctypes.c_ulonglong,  # const VkPhysicalDeviceProperties *
            ctypes.c_ulonglong,  # const VkPhysicalDeviceMemoryProperties *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "VmaJsonWriter::WriteIndent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaJsonWriter *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "VmaJsonWriter::WriteString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaJsonWriter *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cFrameBuffer::init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cFrameBuffer *
            ctypes.c_ulonglong,  # struct VkImage_T *
            ctypes.c_uint32,  # VkFormat
            ctypes.c_ulonglong,  # struct VkImage_T *
            ctypes.c_uint32,  # VkFormat
        ]
    ),
    "VmaVector<char,VmaStlAllocator<char>>::resize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaVector<char,VmaStlAllocator<char> > *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "VmaVector<VmaAllocation_T *,VmaStlAllocator<VmaAllocation_T *>>::resize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaVector<VmaDeviceMemoryBlock *,VmaStlAllocator<VmaDeviceMemoryBlock *> > *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "VmaVector<VmaList<VmaSuballocation,VmaStlAllocator<VmaSuballocation>>::iterator,VmaStlAllocator<VmaList<VmaSuballocation,VmaStlAllocator<VmaSuballocation>>::iterator>>::resize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaVector<VmaList<VmaSuballocation,VmaStlAllocator<VmaSuballocation> >::iterator,VmaStlAllocator<VmaList<VmaSuballocation,VmaStlAllocator<VmaSuballocation> >::iterator> > *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "VmaVector<VmaJsonWriter::StackItem,VmaStlAllocator<VmaJsonWriter::StackItem>>::resize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaVector<VmaJsonWriter::StackItem,VmaStlAllocator<VmaJsonWriter::StackItem> > *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "VmaVector<VmaSuballocation,VmaStlAllocator<VmaSuballocation>>::resize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaVector<VmaSuballocation,VmaStlAllocator<VmaSuballocation> > *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "vmaAllocateMemoryForBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_ulonglong,  # struct VkBuffer_T *
            ctypes.c_ulonglong,  # const VmaAllocationCreateInfo *
            ctypes.c_ulonglong,  # VmaAllocation_T **
            ctypes.c_ulonglong,  # VmaAllocationInfo *
        ]
    ),
    "vmaAllocateMemoryForImage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_ulonglong,  # struct VkImage_T *
            ctypes.c_ulonglong,  # const VmaAllocationCreateInfo *
            ctypes.c_ulonglong,  # VmaAllocation_T **
            ctypes.c_ulonglong,  # VmaAllocationInfo *
        ]
    ),
    "vmaCreatePool": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_ulonglong,  # const VmaPoolCreateInfo *
            ctypes.c_ulonglong,  # VmaPool_T **
        ]
    ),
    "vmaFindMemoryTypeIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const VmaAllocationCreateInfo *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "vmaMapMemory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_ulonglong,  # VmaAllocation_T *
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "vmaUnmapMemory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # VmaAllocator_T *
            ctypes.c_ulonglong,  # VmaAllocation_T *
        ]
    ),
    "cTkInstanceWindComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInstanceWindComponent *
        ]
    ),
    "cTkInstanceWindComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInstanceWindComponent *
        ]
    ),
    "cTkInstanceWindComponent::UpdateRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInstanceWindComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cTkInstanceWindComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInstanceWindComponent *
        ]
    ),
    "cTkPhysicsComponentUpdater::OnInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponentUpdater *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkPhysicsComponentUpdater::UpdateComponents": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponentUpdater *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_ulonglong,  # StackAllocator<float,256,-1>::Source *
            ctypes.c_float,  # float
        ]
    ),
    "cTkPhysicsComponentUpdater::UpdateComponentsPostPhysics": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponentUpdater *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkPhysicsComponentUpdater::OnComponentFree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponentUpdater *
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkPhysicsComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cTkPhysicsComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cTkPhysicsComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkPhysicsComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkPhysicsComponent::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkPhysicsComponent::OnOwnershipTransfer": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # const cTkUserIdBase<cTkFixedString<64,char> > *
        ]
    ),
    "cTkPhysicsComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cTkPhysicsComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cTkPhysicsComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercTkPhysicsComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercTkPhysicsComponent *
        ]
    ),
    "cTkStaticPhysicsComponentUpdater::UpdateComponents": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkStaticPhysicsComponentUpdater *
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_int32,  # int
            ctypes.c_float,  # float
        ]
    ),
    "cTkStaticPhysicsComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStaticPhysicsComponent *
        ]
    ),
    "cTkStaticPhysicsComponent::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStaticPhysicsComponent *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkStaticPhysicsComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkStaticPhysicsComponent *
        ]
    ),
    "cTkStaticPhysicsComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStaticPhysicsComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercTkStaticPhysicsComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercTkStaticPhysicsComponent *
        ]
    ),
    "cTkDynamicChainComponent::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicChainComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cTkDynamicChainComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicChainComponent *
        ]
    ),
    "cTkDynamicChainComponent::GetTypeNameHash64": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicChainComponent *
        ]
    ),
    "cTkDynamicChainComponent::InitData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicChainComponent *
            ctypes.c_ulonglong,  # cTkClassPointer *
        ]
    ),
    "ComponentInstancercTkDynamicChainComponent::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ComponentInstancercTkDynamicChainComponent *
        ]
    ),
    "cTkCreatureTailComponent::UpdatePostPhysics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailComponent *
            ctypes.c_float,  # float
        ]
    ),
    "cTkCreatureTailComponent::OnAttached": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailComponent *
        ]
    ),
    "cTkCreatureTailComponent::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailComponent *
        ]
    ),
    "cTkCreatureTailComponent::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailComponent *
        ]
    ),
    "TkClosestPointFuncs::flip<&cTkPhysicsBvMeshShape::getClosestPointsImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "cTkHavokWorld::cTkHavokWorld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokWorld *
            ctypes.c_ulonglong,  # hknpCollisionFilter *
            ctypes.c_ulonglong,  # hknpShapeTagCodec *
            ctypes.c_ulonglong,  # hkBlockStreamAllocator *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkHavokWorld::~cTkHavokWorld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokWorld *
        ]
    ),
    "cTkPhysicsComponent::~cTkPhysicsComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cTkPhysicsComponent::AddConstraint": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAsyncIOManagerImp::cTkAsyncOp *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkPhysicsComponentUpdater::AddToPostPhysQueue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponentUpdater *
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cTkPhysicsComponent::AddToWorld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cTkStaticPhysicsComponent::AddToWorld": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStaticPhysicsComponent *
        ]
    ),
    "cTkWindEigenvectors::AllocElementVectors": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkWindEigenvectors *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_int32,  # int
        ]
    ),
    "AllocWindEigenvectors": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkWindEigenvectors *
            ctypes.c_ulonglong,  # const cTkInstanceWindComponentData *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_int32,  # int
        ]
    ),
    "ComponentInstancercTkPhysicsComponent::AllocateInstances": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkComponent **
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkHavokCharacterController::ApplyForce": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkHavokCharacterController::ApplyForceLimited": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_double,  # double
        ]
    ),
    "cTkHavokCharacterController::ApplyImpulse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkCreatureTailComponent::BuildChains": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailComponent *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkHavokWorld::CastLocalRaySingle": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokWorld *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const hknpConvexShape *
        ]
    ),
    "cTkHavokWorld::CastRaySingle": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokWorld *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ulonglong,  # cTkContactPoint *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkWindEigenvectors::CompactElementVectors": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkWindEigenvectors *
            ctypes.c_ulonglong,  # cTkVector4 *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkDynamicGravityControl::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicGravityControl *
        ]
    ),
    "cTkHavokCharacterController::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_double,  # double
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_uint32,  # cTkHavokCharacterController::CollisionShape
            ctypes.c_uint32,  # eBodyOwnerType
        ]
    ),
    "cTkHavokProxyCharacterController::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokProxyCharacterController *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_float,  # float
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "ComponentInstancercTkDynamicChainComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercTkPhysicsComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "ComponentInstancercTkStaticPhysicsComponent::ConstructInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkComponent *
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkPhysicsComponent::ContinueBuildingComplexCollision": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cTkPhysicsComponent::CreateCodeDrivenSubBodiesFromNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # const std::vector<TkHandle,TkSTLAllocatorShim<TkHandle,4,-1> > *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cTkPhysicsComponent::CreateConstraintChain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkDynamicChainComponentData *
        ]
    ),
    "cTkPhysicsComponent::CreateRagdollBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkPhysicsComponent::CreateRagdollConstraints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cTkStaticPhysicsComponent::CreateRigidBody": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStaticPhysicsComponent *
        ]
    ),
    "cTkPhysicsComponent::CreateSubBodyChain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkDynamicGravityControl::DebugRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicGravityControl *
        ]
    ),
    "cTkCollisionFilter::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollisionFilter *
        ]
    ),
    "cTkPhysicsComponent::DestroyRagdollBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cTkHavokCharacterController::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
        ]
    ),
    "cTkHavokProxyCharacterController::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokProxyCharacterController *
        ]
    ),
    "ComponentInstancercTkPhysicsComponent::DestructInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkComponent *
        ]
    ),
    "cTkCollisionFilter::DisableCollisionsBetween": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollisionFilter *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkHavokCharacterController::Enable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ubyte,  # bool
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkCollisionFilter::EnableCollisionsBetween": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollisionFilter *
            ctypes.c_ulonglong,  # cTkRigidBody *
            ctypes.c_ulonglong,  # cTkRigidBody *
        ]
    ),
    "cTkPhysicsComponent::ExplodeCodeDrivenSubBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkPhysicsComponent::FindRagdollJointParentsRecursive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkStackVector<TkHandle,128,-1> *
            ctypes.c_ulonglong,  # const cTkStackVector<bool,128,-1> *
            ctypes.c_ulonglong,  # cTkStackVector<TkHandle,128,-1> *
            ctypes.c_ulonglong,  # cTkStackVector<bool,128,-1> *
            ctypes.c_ulonglong,  # cTkStackVector<int,128,-1> *
        ]
    ),
    "cTkPhysicsComponent::FindRagdollJointsRecursive": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # const cTkRagdollData *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # cTkStackVector<TkHandle,128,-1> *
            ctypes.c_ulonglong,  # cTkStackVector<bool,128,-1> *
        ]
    ),
    "cTkInstanceWindComponent::FindWindEigenvectors": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInstanceWindComponent *
        ]
    ),
    "FindWindEigenvectorsJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkHavokProxyCharacterController::ForcedUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokProxyCharacterController *
            ctypes.c_float,  # float
        ]
    ),
    "cTkPhysicsComponent::GetAverageSubBodyPos": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cTkDynamicGravityControl::GetGravity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkGravityData *
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicGravityControl *
            ctypes.c_ulonglong,  # cTkGravityData *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkPhysicsComponentUpdater::GetInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysicsComponentUpdater *
        argtypes=[]
    ),
    "cTkHavokCharacterController::GetPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cTkHavokProxyCharacterController::GetPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokProxyCharacterController *
            ctypes.c_ulonglong,  # cTkPhysRelVec3 *
        ]
    ),
    "cTkHavokCharacterController::GetRigidBody": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRigidBody *
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
        ]
    ),
    "cTkHavokCharacterController::GetVelocity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ulonglong,  # cTkVector3 *
        ]
    ),
    "cTkInstanceWindComponent::MatchInstance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkInstanceWindComponent *
        ]
    ),
    "TkBit::BitIterator::Next": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # TkBit::BitIterator *
        ]
    ),
    "cTkCollisionFilter::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollisionFilter *
        ]
    ),
    "cTkHavokCharacterController::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkHavokProxyCharacterController::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokProxyCharacterController *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkPhysicsComponent::PrepareConstraints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cTkStaticPhysicsComponentUpdater::QueueStaticPhysicsJob": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStaticPhysicsComponentUpdater *
            ctypes.c_ulonglong,  # cTkStaticPhysicsComponent *
            ctypes.c_uint32,  # cTkStaticPhysicsComponentUpdater::StaticPhysicsJobType
        ]
    ),
    "cTkPhysicsComponent::Ragdoll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "cTkHavokCharacterController::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkHavokProxyCharacterController::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokProxyCharacterController *
        ]
    ),
    "cTkHavokWorld::Render": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokWorld *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkHavokCharacterController::SetClimbableStickinessAngle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_float,  # float
        ]
    ),
    "cTkHavokWorld::SetGravity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokWorld *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkHavokProxyCharacterController::SetMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokProxyCharacterController *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cTkHavokCharacterController::SetMaxFallSpeed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_float,  # float
        ]
    ),
    "cTkHavokCharacterController::SetMaxSlopeAngle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_float,  # float
        ]
    ),
    "cTkHavokCharacterController::SetMaxWalkableAngle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_float,  # float
        ]
    ),
    "cTkPhysicsComponent::SetPhysicsToGraphics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
        ]
    ),
    "cTkHavokCharacterController::SetPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ulonglong,  # const cTkPhysRelVec3 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkHavokProxyCharacterController::SetPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokProxyCharacterController *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkHavokCharacterController::SetRelMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cTkCollisionFilter::SetVehicleBroadphaseFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCollisionFilter *
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['cTkRigidBody *', 'cTkRigidBody *']}
        ]
    ),
    "cTkPhysicsComponent::SwitchToComplexCollision": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkPhysicsComponent *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkStackVector<TkHandle,16,-1> *
        ]
    ),
    "TkDrawPhysicsSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const cTkColour *
        ]
    ),
    "cTkCreatureTailComponent::cGcChain::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkCreatureTailComponent::cGcChain *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkCreatureTailComponentData *
            ctypes.c_ulonglong,  # const cTkCreatureTailParams *
            ctypes.c_ulonglong,  # const cTkCreatureTailJoints *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkDynamicGravityControl::UpdateGravityPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkDynamicGravityControl *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkStaticPhysicsComponent::UpdatePosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkStaticPhysicsComponent *
        ]
    ),
    "VecIntrinsics::VOrthoNormalise": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "cTkHavokCharacterController::applyAction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokCharacterController *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_double,  # double
        ]
    ),
    "cTkHavokProxyCharacterController::applyAction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokProxyCharacterController *
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_float,  # float
        ]
    ),
    "cTkHavokGroupMaskFilter::filterBodyPairs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokGroupMaskFilter *
            ctypes.c_ulonglong,  # const hknpWorld *
            ctypes.c_ulonglong,  # hknpBodyIndexPair *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkHavokGroupMaskFilter::isCollisionEnabled": {
        "cTkHavokGroupMaskFilter *, hknpCollisionQueryType::Enum, const hknpQueryFilterData *, const hknpBody *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cTkHavokGroupMaskFilter *
                ctypes.c_uint32,  # hknpCollisionQueryType::Enum
                ctypes.c_ulonglong,  # const hknpQueryFilterData *
                ctypes.c_ulonglong,  # const hknpBody *
            ]
        ),
        "cTkHavokGroupMaskFilter *, hknpCollisionQueryType::Enum, bool, const hknpCollisionFilter::FilterInput *, const hknpCollisionFilter::FilterInput *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cTkHavokGroupMaskFilter *
                ctypes.c_uint32,  # hknpCollisionQueryType::Enum
                ctypes.c_ubyte,  # bool
                ctypes.c_ulonglong,  # const hknpCollisionFilter::FilterInput *
                ctypes.c_ulonglong,  # const hknpCollisionFilter::FilterInput *
            ]
        ),
    },
    "loopSubdivSphereTri": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkHavokWorld::stepSimulation": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkHavokWorld *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
        ]
    ),
    "cTkMetaData::ReadFromFile<cTkEntitlementList>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cGcWaterColourSettingList *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFileSystem::Data::Data": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem::Data *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkBankInfo::cTkBankInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBankInfo *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "sComponentEvent::sComponentEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sComponentEvent *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_char_p,  # const char *
        ]
    ),
    "sSketchCurve::sSketchCurve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sSketchCurve *
        ]
    ),
    "cTkIKFullBody::AddBaseCostConstraint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # cMatrixBuilder *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
        ]
    ),
    "cTkIKFullBody::AddJoint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_uint32,  # TkHandle
            ctypes.c_int32,  # int
        ]
    ),
    "cTkIKFullBody::AddJointAndParents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkIKFullBody::AssignStateVals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
        ]
    ),
    "cTkFileSystem::AsyncCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_int32,  # int
        ]
    ),
    "sComponentEvent::Call": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sComponentEvent *
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # cTkAttachment *
        ]
    ),
    "cTkIKFullBody::CastRayVsJointAabbs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # TkHandle *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # cTkAABB *
        ]
    ),
    "sSketchStack::CheckBufferSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sSketchStack *
            ctypes.c_uint32,  # const unsigned int
        ]
    ),
    "cTkSketchWindow::CheckStringFilter": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
        ]
    ),
    "cTkFileSystem::Close": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem *
            ctypes.c_ulonglong,  # FIOS2HANDLE *
        ]
    ),
    "cTkIKFullBody::ComputeChildrenMasses": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
        ]
    ),
    "cTkIKFullBody::ComputeEuler": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # const AnimTransform *
        ]
    ),
    "cTkFileSystem::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.POINTER(structs.cTkFileSystem),  # cTkFileSystem *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkIKFullBody::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "ConstructFIOS": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SceFiosParams *
        ]
    ),
    "cTkFileSystem::CreatePath": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkFileSystem::DestroyFileCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem *
        ]
    ),
    "sSketchCurve::DoCurveCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sSketchCurve::DoCurveEditor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sSketchCurve *
        ]
    ),
    "cTkSketchWindow::DoNodeAndChildren": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # std::vector<TkHandle,TkSTLAllocatorShim<TkHandle,4,-1> > *
        ]
    ),
    "cTkFileSystem::DoesFileExist": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "sSketchCurve::DrawCurve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sSketchCurve *
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const NVGcolor *
        ]
    ),
    "cTkFileSystem::Data::EnumerateBanks": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cTkBankInfo,TkSTLAllocatorShim<cTkBankInfo,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem::Data *
            ctypes.c_ulonglong,  # std::vector<cTkBankInfo,TkSTLAllocatorShim<cTkBankInfo,8,-1> > *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkFileSystem::EnumerateDirectory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # char **
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkSketchWindow::ExtractComponentNameOnly": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkFixedString<64,char> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkSketchWindow::FindNodeIndexForFilter": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # TkHandle
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkFixedString<64,char> *
        ]
    ),
    "cTkSketchWindow::FindSketchComponentForModel": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkSketchWindow::FindSketchComponentForSelectedNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSketchComponent *
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkFileSystem::FreeAsyncOp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem *
            ctypes.c_int32,  # int
        ]
    ),
    "sSketchStack::FreeBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "cTkEntitlementManager::GetEntitlementId": FUNCDEF(
        restype=ctypes.c_char_p,  # const TkID<128> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkEntitlementManager *
            ctypes.c_char_p,  # const TkID<128> *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkEntitlementManager::GetEntitlementServiceId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<64,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkEntitlementManager *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkIKFullBody::GetEulerFromMat": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkEulerVector *
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # cTkEulerVector *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
        ]
    ),
    "cTkFileSystem::GetFileSize": {
        "cTkFileSystem *, FIOS2HANDLE *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkFileSystem *
                ctypes.c_ulonglong,  # FIOS2HANDLE *
            ]
        ),
        "cTkFileSystem *, const char *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkFileSystem *
                ctypes.c_char_p,  # const char *
            ]
        ),
    },
    "cTkIKFullBody::GetModelSpacePosAndRotJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkEulerVector *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # cTkJacobianRow *
        ]
    ),
    "cTkIKFullBody::GetModelSpacePosJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # cTkJacobianRow *
        ]
    ),
    "cTkIKFullBody::GetModelSpaceRotJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const cTkEulerVector *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # cTkJacobianRow *
        ]
    ),
    "sSketchCurve::GetValue": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # sSketchCurve *
            ctypes.c_float,  # float
        ]
    ),
    "GetVectorFromState": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkEulerVector *
        argtypes=[
            ctypes.c_ulonglong,  # cTkEulerVector *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const cTkIKFullBody::BoneInfo *
        ]
    ),
    "cTkEntitlementManager::HasEntitlement": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkEntitlementManager *
            ctypes.c_char * 0x10,  # const TkID<128>
        ]
    ),
    "cTkFileSystem::IsModded": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFileSystem::Data *
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem *
        ]
    ),
    "cTkFileSystem::IsTampered": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFileSystem::Data *
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem *
        ]
    ),
    "cTkFileSystem::Data::LoadBankInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # std::vector<cTkBankInfo,TkSTLAllocatorShim<cTkBankInfo,8,-1> > *
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem::Data *
            ctypes.c_ulonglong,  # std::vector<cTkBankInfo,TkSTLAllocatorShim<cTkBankInfo,8,-1> > *
            ctypes.c_ulonglong,  # cTkFileSystem *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkIKFullBody::MulAddModelSpacePosJacobian": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # cTkJacobianRow *
            ctypes.c_ulonglong,  # cTkJacobianRow *
        ]
    ),
    "cTkSeed::NGuiRender": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSeed *
        ]
    ),
    "cTkIKFullBody::NormaliseInertia": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
        ]
    ),
    "cTkFileSystem::NormalisePath": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkFixedString<260,char> *
        ]
    ),
    "cTkIKFullBody::OnPostAnim": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const cTkPhysRelMat34 *
        ]
    ),
    "cTkFileSystem::Open": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FIOS2HANDLE *
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # eFileOpenMode
        ]
    ),
    "cTkSketchWindow::OpenWindow": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "TkSketchMaths::ParseNumber": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkIKFullBody::PreAdditive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # const cTkMatrix34 *
            ctypes.c_ulonglong,  # const AnimTransform *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkEntitlementManager::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkEntitlementManager *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "sSketchEvent::RaiseEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_ulonglong,  # const sSketchValue *
            ctypes.c_uint64,  # cTkAttachmentPtr
        ]
    ),
    "cTkFileSystem::Read": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # FIOS2HANDLE *
        ]
    ),
    "cTkFileSystem::ReadFromPath": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFileSystem::ReadFromPathAsync": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkSketchWindow::RefreshSelectedNode": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkFileSystem::RegisterAsyncOpCompletionCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['int *']}
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkSketchWindow::RegisterSketchWindows": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkSketchWindow::RenderEditorWindow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkSketchWindow::RenderHierarchyWindow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkBlackboard::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboard *
        ]
    ),
    "cTkBlackboardValue::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkBlackboardValue *
            ctypes.c_char_p,  # const TkID<128> *
        ]
    ),
    "sSketchCurve::RenderNGui": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # sSketchCurve *
        ]
    ),
    "cTkSketchWindow::RenderVariablesWindow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "ResolveAlgebra": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "TkSketchMaths::RunFunction": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_float,  # const float
        ]
    ),
    "cTkFileSystem::Seek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem *
            ctypes.c_ulonglong,  # FIOS2HANDLE *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # eSeekMode
        ]
    ),
    "cTkFileSystem::SetFileCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem *
            ctypes.c_ulonglong,  # std::shared_ptr<cTkFileSystemCache> *
        ]
    ),
    "cTkSketchWindow::SetSelectedNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # TkHandle
        ]
    ),
    "cTkIKChain::Solve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKChain *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkIKFullBody::Solve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkIKFullBody *
            ctypes.c_ulonglong,  # cTkMatrix34 *
            ctypes.c_ulonglong,  # AnimTransform *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkFileSystem::Tell": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem *
            ctypes.c_ulonglong,  # FIOS2HANDLE *
        ]
    ),
    "TkSketchMaths::ValidateFunction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkFileSystem::Write": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystem *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # FIOS2HANDLE *
        ]
    ),
    "cTkTransientHeap::cTkTransientHeap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTransientHeap *
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkSlotAlloc::Alloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "cTkTransientHeap::Allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkTransientHeap *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkRichPresenceData::ApplyIdle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRichPresenceData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRichPresence::Configure": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRichPresence *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkRichPresence::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRichPresence *
        ]
    ),
    "cTkSlotAlloc::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSlotAlloc *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkRichPresence::CreateInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRichPresence *
        argtypes=[]
    ),
    "cTkRichPresence::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRichPresence *
        ]
    ),
    "cTkRichPresenceData::Evaluate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkRichPresenceState *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRichPresenceData *
        ]
    ),
    "cTkSlotAlloc::Free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSlotAlloc *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkRichPresence::GetCurrentStateString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRichPresence *
        ]
    ),
    "cTkSlotAlloc::GetTotalMemSizeIncAllocator": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkSlotAlloc *
        ]
    ),
    "cTkTransientHeap::NewSubHeap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTransientHeap *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkRichPresence::NotifyState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRichPresence *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkTransientHeap::ResetForReuse": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkTransientHeap *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkRichPresence::ResetTracking": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRichPresence *
        ]
    ),
    "cTkRichPresence::Update": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkRichPresence *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGui::AddElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # eNGuiSizeType
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # cTkNGuiElementData *
            ctypes.c_ulonglong,  # cTkBBox2d *
            ctypes.c_ulonglong,  # cTkBBox2d *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGui::AddElementToLayout": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkBBox2d *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
            ctypes.c_ulonglong,  # cTkBBox2d *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::AutosaveLayout": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
        ]
    ),
    "cTkNGuiTextEditState::CalcTextSize": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiTextEditState *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cTkNGui::CalculateElementSize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # eNGuiSizeType
            ctypes.c_ulonglong,  # cTkNGuiElementData *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiInput::CentreCursor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiInput *
            ctypes.c_ulonglong,  # const cTkVector2 *
        ]
    ),
    "cTkLinearHashTable<int,int,8,8192,cTkLinearHashTableHash<int>>::ChangeNodeCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<int,int,8,8192,cTkLinearHashTableHash<int> > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkLinearHashTable<cTkTexture const *,int,8,8192,cTkLinearHashTableHash<cTkTexture const *>>::ChangeNodeCapacity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<cTkTexture const *,int,8,8192,cTkLinearHashTableHash<cTkTexture const *> > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkNGui::ClipBegin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGui::DoBegin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoButton": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # eNGuiSizeType
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # eNGuiInputType *
        ]
    ),
    "cTkNGuiEditor::DoCheckbox": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoComboBox": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char **
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoEditColour": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # cTkColour *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditor::DoEditFile": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditor::DoEditFloat": {
        "cTkNGuiEditor *, float *, float, bool, float, int, bool": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiEditor *
                ctypes.c_ulonglong,  # float *
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
                ctypes.c_float,  # float
                ctypes.c_int32,  # int
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cTkNGuiEditor *, const char *, float *, float, bool, float, int, bool": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiEditor *
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # float *
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
                ctypes.c_float,  # float
                ctypes.c_int32,  # int
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cTkNGuiEditor::DoEditHex": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkNGuiEditor::DoEditHexU64": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "cTkNGuiEditor::DoEditInt": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
        ]
    ),
    "cTkNGuiEditor::DoEditText": {
        "cTkNGuiEditor *, char *, int, int, eNGuiSizeType, float, float, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiEditor *
                ctypes.c_ulonglong,  # char *
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # eNGuiSizeType
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cTkNGuiEditor *, const char *, char *, int": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiEditor *
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # char *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "cTkNGuiEditor::DoEditVector": {
        "cTkNGuiEditor *, const char *, cTkVector2 *, const FloatEditOptions *, const FloatLimits *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiEditor *
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # cTkVector2 *
                ctypes.c_ulonglong,  # const FloatEditOptions *
                ctypes.c_ulonglong,  # const FloatLimits *
            ]
        ),
        "cTkNGuiEditor *, const char *, cTkVector3 *, const FloatEditOptions *, const FloatLimits *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiEditor *
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # cTkVector3 *
                ctypes.c_ulonglong,  # const FloatEditOptions *
                ctypes.c_ulonglong,  # const FloatLimits *
            ]
        ),
    },
    "cTkNGui::DoElement": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # eNGuiSizeType
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGui::DoElementGraphic": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eNGuiSizeType
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGui::DoElementText": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # eNGuiSizeType
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGui::DoEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
        ]
    ),
    "cTkNGuiEditor::DoErrorInline": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # eNGuiSizeType
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoIconButton": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # eNGuiSizeType
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNGuiEditor::DoIconListItem": {
        "cTkNGuiEditor *, unsigned int, const char *, bool": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiEditor *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "cTkNGuiEditor *, unsigned int, const char *, bool, float, eNGuiInputType *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiEditor *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # eNGuiInputType *
            ]
        ),
    },
    "cTkNGui::DoLayerBegin": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
            ctypes.c_uint32,  # eNGuiSizeType
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGui::DoLayerEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
        ]
    ),
    "cTkNGuiEditor::DoLeftPanelBegin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_uint32,  # eNGuiSizeType
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoLeftPanelEndRightPanelBegin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
        ]
    ),
    "cTkNGuiEditor::DoRectangle": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_ulonglong,  # const cTkColour *
            ctypes.c_uint32,  # eNGuiSizeType
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoScrollBegin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditor::DoScrollEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditor::DoSeparator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
        ]
    ),
    "cTkNGuiEditor::DoSliderFloat": {
        "cTkNGuiEditor *, float *, float, float, float, float": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiEditor *
                ctypes.c_ulonglong,  # float *
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
        "cTkNGuiEditor *, const char *, float *, float, float, float": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # cTkNGuiEditor *
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # float *
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
    },
    "cTkNGuiEditor::DoSpacing": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
        ]
    ),
    "cTkNGuiEditor::DoTextLabel": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGuiEditor::DoToggle": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditor::DoTreeNodeBegin": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditor::DoTreeNodeEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditor::DoVerticalScrollBegin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
        ]
    ),
    "cTkNGuiEditor::DoVerticalScrollEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
        ]
    ),
    "cTkNGuiEditor::DoesParentTreePassFilter": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
        ]
    ),
    "cTkNGuiEditor::GetActiveGraphicStyle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkNGuiGraphicStyle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNGuiEditor::GetComponentSize": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_uint32,  # eNGuiEditorComponentSize
        ]
    ),
    "cTkNGui::GetForcedStyle": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
        ]
    ),
    "cTkNGuiEditor::GetGraphicStyle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkNGuiGraphicStyle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNGui::GetInputState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
            ctypes.c_ulonglong,  # cTkNGuiElementID *
            ctypes.c_ulonglong,  # cTkBBox2d *
        ]
    ),
    "cTkNGuiEditor::GetLabelWidth": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
        ]
    ),
    "cTkNGuiEditor::GetLabelWidthWriteable": FUNCDEF(
        restype=ctypes.c_ulonglong,  # float *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
        ]
    ),
    "cTkNGuiEditor::GetSearchFilter": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<256,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
        ]
    ),
    "cTkNGuiEditor::GetSnappingSetting": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_uint32,  # eNGuiSnapSettings
        ]
    ),
    "cTkNGuiEditor::GetTextStyle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const cTkNGuiTextStyle *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNGuiAnimationData::GetUnitTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiAnimationData *
            ctypes.c_uint32,  # eNGuiAnimationEvent
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cTkNGuiFilter::IsFiltered": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiFilter *
        ]
    ),
    "cTkNGuiEditor::IsHighlightActive": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
        ]
    ),
    "cTkNGui::LayerBegin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkNGui::LoadFontFile": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::cTkNGuiFontHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::cTkNGuiFontHandleID> *
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkNGui::LoadImageFile": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::cTkNGuiFontHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::cTkNGuiFontHandleID> *
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "NVGRegisterTexture": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
        ]
    ),
    "NVGcreate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # NVGcontext *
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "cTkNGuiEditor::OpenFileBrowser": {
        "char *, const char *, const char *, const char *, bool, const char *": FUNCDEF(
            restype=ctypes.c_ubyte,  # _BOOL8
            argtypes=[
                ctypes.c_ulonglong,  # char *
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
                ctypes.c_char_p,  # const char *
            ]
        ),
        "char *, int, const char *, bool, bool, bool": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # char *
                ctypes.c_int32,  # int
                ctypes.c_char_p,  # const char *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "cTkNGui::PopForcedStyle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
        ]
    ),
    "cTkNGui::PrefetchImageFile": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
        argtypes=[
            ctypes.c_ulonglong,  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *
            ctypes.c_ulonglong,  # NVGcontext *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "cTkNGui::PushForcedStyle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
            ctypes.c_uint32,  # eNGuiForcedStyle
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkNGuiEditor::PushGraphicStyle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_uint32,  # eNGuiEditorGraphic
            ctypes.c_ulonglong,  # cTkNGuiGraphicStyle *
        ]
    ),
    "cTkNGuiEditor::PushTextStyle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_uint32,  # eNGuiEditorText
            ctypes.c_ulonglong,  # cTkNGuiTextStyle *
        ]
    ),
    "cTkNGuiEditor::RegisterWindow": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkNGuiElementID *
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_ulonglong,  # cTkNGuiElementID *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['void *']}
        ]
    ),
    "cTkComboBoxPopup::Render": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkComboBoxPopup *
        ]
    ),
    "cTkIconSelectPopup::Render": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkIconSelectPopup *
        ]
    ),
    "cTkNGuiInput::ResetCursorToOrigin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiInput *
        ]
    ),
    "GLNVGtexture::SetTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GLNVGtexture *
            ctypes.c_ulonglong,  # cTkTexture *
            ctypes.c_ulonglong,  # GLNVGcontext *
        ]
    ),
    "cTkNGui::SetWindowClipRegion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
        ]
    ),
    "cTkNGuiAnimationData::StartActivate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiAnimationData *
            ctypes.c_uint32,  # eNGuiAnimationEvent
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGuiEditor::TextEdit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiEditor *
            ctypes.c_ulonglong,  # char *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkNGui::TransformPointToFrameSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGui *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cTkNGui::TransformPointToLocalSpace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cTkNGuiAnimationData::UpdateEvents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkNGuiAnimationData *
            ctypes.c_uint32,  # eNGuiInputType
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkLinearHashTable<int,int,8,8192,cTkLinearHashTableHash<int>>::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<int,int,8,8192,cTkLinearHashTableHash<int> > *
        ]
    ),
    "cTkLinearHashTable<cTkTexture const *,int,8,8192,cTkLinearHashTableHash<cTkTexture const *>>::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkLinearHashTable<cTkTexture const *,int,8,8192,cTkLinearHashTableHash<cTkTexture const *> > *
        ]
    ),
    "glnvg__ActualEndFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "glnvg__allocCall": FUNCDEF(
        restype=ctypes.c_ulonglong,  # GLNVGcall *
        argtypes=[
            ctypes.c_ulonglong,  # GLNVGcontext *
        ]
    ),
    "glnvg__allocFragUniforms": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # GLNVGcontext *
            ctypes.c_int32,  # int
        ]
    ),
    "glnvg__allocPaths": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # GLNVGcontext *
            ctypes.c_int32,  # int
        ]
    ),
    "glnvg__allocTexture": FUNCDEF(
        restype=ctypes.c_ulonglong,  # GLNVGtexture *
        argtypes=[
            ctypes.c_ulonglong,  # GLNVGcontext *
        ]
    ),
    "glnvg__allocVerts": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # GLNVGcontext *
            ctypes.c_int32,  # int
        ]
    ),
    "glnvg__convertPaint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # GLNVGcontext *
            ctypes.c_ulonglong,  # GLNVGfragUniforms *
            ctypes.c_ulonglong,  # NVGpaint *
            ctypes.c_ulonglong,  # NVGscissor *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "glnvg__deleteTexture": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # GLNVGcontext *
            ctypes.c_int32,  # int
        ]
    ),
    "glnvg": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GLNVGcontext *
            ctypes.c_ulonglong,  # GLNVGcall *
            ctypes.c_ulonglong,  # cTkImmediateMode *
        ]
    ),
    "glnvg__findTexture": FUNCDEF(
        restype=ctypes.c_ulonglong,  # GLNVGtexture *
        argtypes=[
            ctypes.c_ulonglong,  # GLNVGcontext *
            ctypes.c_int32,  # int
        ]
    ),
    "glnvg__maxVertCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const NVGpath *
            ctypes.c_int32,  # int
        ]
    ),
    "glnvg__renderCancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "glnvg__renderCreate": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "glnvg__renderCreateTexture": FUNCDEF(
        restype=ctypes.c_ulonglong,  # GLNVGtexture *
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const unsigned __int8 *
        ]
    ),
    "glnvg__renderDelete": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "glnvg__renderDeleteTexture": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
        ]
    ),
    "glnvg__renderFill": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # NVGpaint *
            ctypes.c_ulonglong,  # NVGscissor *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const NVGpath *
            ctypes.c_int32,  # int
        ]
    ),
    "glnvg__renderFlush": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "glnvg__renderGetTextureSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "glnvg__renderStroke": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # NVGpaint *
            ctypes.c_ulonglong,  # NVGscissor *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const NVGpath *
            ctypes.c_int32,  # int
        ]
    ),
    "glnvg__renderTriangles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # NVGpaint *
            ctypes.c_ulonglong,  # NVGscissor *
            ctypes.c_ulonglong,  # const NVGvertex *
            ctypes.c_int32,  # int
        ]
    ),
    "glnvg__renderUpdateTexture": FUNCDEF(
        restype=ctypes.c_ulonglong,  # GLNVGtexture *
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const unsigned __int8 *
        ]
    ),
    "glnvg__renderViewport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "glnvg__setUniforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GLNVGcontext *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "glnvg__stroke": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # GLNVGcontext *
            ctypes.c_ulonglong,  # GLNVGcall *
            ctypes.c_ulonglong,  # cTkImmediateMode *
        ]
    ),
    "cTkAsyncIOManagerImp::cTkAsyncIOManagerImp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAsyncIOManagerImp *
        ]
    ),
    "cTkFileSystemCache::cTkFileSystemCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystemCache *
            ctypes.c_char_p,  # const char **
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAsyncIOManagerImp::~cTkAsyncIOManagerImp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAsyncIOManagerImp *
        ]
    ),
    "cTkFileSystemCache::~cTkFileSystemCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystemCache *
        ]
    ),
    "cTkSimpleMemoryPool::Allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkSimpleMemoryPool::cTkBlock *
        argtypes=[
            ctypes.c_ulonglong,  # cTkSimpleMemoryPool *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkAsyncIOManager::CancelOp": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkAsyncIOManager::ClearFileCache": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkAsyncIOManager::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkSimpleMemoryPool::Deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSimpleMemoryPool *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkAsyncIOManager::DeleteOpHandle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAsyncIOManager::DeleteOpHandles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAsyncIOManager::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkFileSystemCache::FindCachedFileSection": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFileSystemCache::sCachedFileSection *
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystemCache *
            ctypes.c_char_p,  # const char *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkAsyncIOManagerImp::FreeFilesystemOp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAsyncIOManagerImp *
            ctypes.c_ulonglong,  # cTkAsyncIOManagerImp::cTkAsyncOp *
        ]
    ),
    "cTkFileSystemCache::FreeOldestItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystemCache *
        ]
    ),
    "cTkAsyncIOManager::GetNumActiveOps": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkAsyncIOManager::GetNumOpsCompleted": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "cTkAsyncIOManager::GetOpData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAsyncIOManager::GetOpDataSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAsyncIOManager::GetOpPath": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkAsyncIOManagerImp::cTkAsyncOp *
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "cTkAsyncIOManager::GetOpStatus": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFileSystemCache::InvalidatePath": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystemCache *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkFileSystemCache::IsCacheablePath": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystemCache *
            ctypes.c_char_p,  # const char *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "cTkAsyncIOManager::IsOpCancelled": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAsyncIOManager::LockOpCritical": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkSimpleMemoryPool::MergeBlocksRight": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkSimpleMemoryPool *
            ctypes.c_ulonglong,  # cTkSimpleMemoryPool::cTkBlock *
        ]
    ),
    "cTkAsyncIOManagerImp::OnAsyncOpCompletion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "cTkAsyncIOManagerImp::OpThreadEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkFileSystemCache::PollToPrecache": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystemCache *
        ]
    ),
    "cTkFileSystemCache::PrecacheFile": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystemCache *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkFileSystemCache::PrecacheFromFile": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystemCache *
            ctypes.c_char_p,  # const char *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkAsyncIOManager::Read": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['void *', 'const char *', 'int', '__int64', 'void *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkAsyncIOManagerImp::Read": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkAsyncIOManagerImp *
            ctypes.c_char_p,  # const char *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['void *', 'const char *', 'int', '__int64', 'void *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkFileSystemCache::Read": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystemCache *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkAsyncIOManagerImp::ThreadUpdateOpsBlocking": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAsyncIOManagerImp *
        ]
    ),
    "cTkAsyncIOManagerImp::ThreadUpdateOpsNonBlocking": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkAsyncIOManagerImp *
        ]
    ),
    "cTkFileSystemCache::TryCachePathData": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkFileSystemCache *
            ctypes.c_char_p,  # const char *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "cTkAsyncIOManager::UnlockOpCritical": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "cTkHilbertMortonHelper::cTkHilbertMortonHelper": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkHilbertMortonHelper *
        ]
    ),
    "cTkPolygoniserCuberille::AddQuad": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkQuadElement *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_uint32,  # eVoxelType
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cTkRegion::CalculateCubeMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkMatrix34 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
            ctypes.c_ulonglong,  # cTkMatrix34 *
        ]
    ),
    "cTkPolygoniserDualContouring::CalculateMaterials": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const eVoxelType *
            ctypes.c_ulonglong,  # const eVoxelType *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const cTkVector3 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cTkPolygoniserDualContouring::DualContouring": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cCubeValue *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkRegion *
            ctypes.c_ulonglong,  # const cTkVoxel *
            ctypes.c_ulonglong,  # const cTkVoxel *
        ]
    ),
    "cTkPolygoniserDualContouring::GeneratePolygons": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cCubeValue *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkTerrainVertex *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # cTkRegion *
        ]
    ),
    "cTkPolygoniserHeightmap::GetMaterialsForTriangle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "cTkPolygoniserMarchingCubes::MarchCube": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkTerrainVertex *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "MarkNeighbor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # cTkTerrainVertex *
            ctypes.c_ulonglong,  # const cTkVector3 *
        ]
    ),
    "cTkPolygoniserCuberille::Polygonise": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
            ctypes.c_uint32,  # cTkPolygoniserCuberille::eType
            ctypes.c_ulonglong,  # cTkQuadElement *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkPolygoniserDualContouring::Polygonise": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const cTkVoxel *
            ctypes.c_ulonglong,  # const cTkVoxel *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # cTkTerrainVertex *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkPolygoniserHeightmap::Polygonise": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
            ctypes.c_ulonglong,  # cTkTerrainVertex *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkPolygoniserMarchingCubes::Polygonise": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # cTkTerrainVertex *
            ctypes.c_int32,  # int
        ]
    ),
    "cTkRegion::TryGenerateVoxels": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
        ]
    ),
    "cTkRegion::TryPolygonise": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
        ]
    ),
    "cTkPolygoniserCuberille::_IsFull": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # cTkRegion *
            ctypes.c_uint32,  # cTkPolygoniserCuberille::eType
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "cTkPolygoniserMarchingCubes::vGetNormal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkVector3 *
        argtypes=[
            ctypes.c_ulonglong,  # cTkVector3 *
            ctypes.c_ulonglong,  # cTkRegion *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CreateShaderModule": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # SpvReflectShaderModule *
        ]
    ),
    "MarkSelfAndAllMemberVarsAsUsed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectBlockVariable *
        ]
    ),
    "ParseDecorations": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectPrvParser *
        ]
    ),
    "ParseDescriptorBindings": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectPrvParser *
            ctypes.c_ulonglong,  # SpvReflectShaderModule *
        ]
    ),
    "ParseDescriptorBlockVariable": FUNCDEF(
        restype=ctypes.c_uint32,  # SpvReflectResult
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectPrvParser *
            ctypes.c_ulonglong,  # SpvReflectShaderModule *
            ctypes.c_ulonglong,  # SpvReflectTypeDescription *
            ctypes.c_ulonglong,  # SpvReflectBlockVariable *
        ]
    ),
    "ParseDescriptorBlockVariableSizes": FUNCDEF(
        restype=ctypes.c_uint32,  # SpvReflectResult
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectPrvParser *
            ctypes.c_ulonglong,  # SpvReflectShaderModule *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # SpvReflectBlockVariable *
        ]
    ),
    "ParseDescriptorBlockVariableUsage": FUNCDEF(
        restype=ctypes.c_uint32,  # SpvReflectResult
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectPrvParser *
            ctypes.c_ulonglong,  # SpvReflectShaderModule *
            ctypes.c_ulonglong,  # SpvReflectPrvAccessChain *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # SpvOp_
            ctypes.c_ulonglong,  # SpvReflectBlockVariable *
        ]
    ),
    "ParseEntryPoints": FUNCDEF(
        restype=ctypes.c_uint32,  # unsigned int
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectPrvParser *
            ctypes.c_ulonglong,  # SpvReflectShaderModule *
        ]
    ),
    "ParseEntrypointDescriptorSets": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectShaderModule *
        ]
    ),
    "ParseExecutionModes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectPrvParser *
            ctypes.c_ulonglong,  # SpvReflectShaderModule *
        ]
    ),
    "ParseFormat": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const SpvReflectTypeDescription *
            ctypes.c_ulonglong,  # SpvReflectFormat *
        ]
    ),
    "ParseFunction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectPrvParser *
            ctypes.c_ulonglong,  # SpvReflectPrvNode *
            ctypes.c_ulonglong,  # SpvReflectPrvFunction *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "ParseFunctions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectPrvParser *
        ]
    ),
    "ParseInterfaceVariable": FUNCDEF(
        restype=ctypes.c_uint32,  # unsigned int
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectPrvParser *
            ctypes.c_ulonglong,  # const SpvReflectPrvDecorations *
            ctypes.c_ulonglong,  # const SpvReflectPrvDecorations *
            ctypes.c_ulonglong,  # SpvReflectShaderModule *
            ctypes.c_ulonglong,  # SpvReflectTypeDescription *
            ctypes.c_ulonglong,  # SpvReflectInterfaceVariable *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "ParseInterfaceVariables": FUNCDEF(
        restype=ctypes.c_uint32,  # unsigned int
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectPrvParser *
            ctypes.c_ulonglong,  # SpvReflectShaderModule *
            ctypes.c_ulonglong,  # SpvReflectEntryPoint *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "ParseMemberCounts": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectPrvParser *
        ]
    ),
    "ParseNodes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectPrvParser *
        ]
    ),
    "ParsePushConstantBlocks": FUNCDEF(
        restype=ctypes.c_uint32,  # SpvReflectResult
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectPrvParser *
            ctypes.c_ulonglong,  # SpvReflectShaderModule *
        ]
    ),
    "ParseStaticallyUsedResources": FUNCDEF(
        restype=ctypes.c_uint32,  # SpvReflectResult
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectPrvParser *
            ctypes.c_ulonglong,  # SpvReflectShaderModule *
            ctypes.c_ulonglong,  # SpvReflectEntryPoint *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "ParseType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectPrvParser *
            ctypes.c_ulonglong,  # SpvReflectPrvNode *
            ctypes.c_ulonglong,  # SpvReflectPrvDecorations *
            ctypes.c_ulonglong,  # SpvReflectShaderModule *
            ctypes.c_ulonglong,  # SpvReflectTypeDescription *
        ]
    ),
    "SafeFreeBlockVariables": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectBlockVariable *
        ]
    ),
    "SafeFreeInterfaceVariable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectInterfaceVariable *
        ]
    ),
    "SafeFreeTypes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectTypeDescription *
        ]
    ),
    "SortCompareDescriptorBinding": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "AkFDNQSortCompare": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "SynchronizeDescriptorSets": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectShaderModule *
        ]
    ),
    "TraverseCallGraph": FUNCDEF(
        restype=ctypes.c_uint32,  # SpvReflectResult
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectPrvParser *
            ctypes.c_ulonglong,  # SpvReflectPrvFunction *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "spvReflectCreateShaderModule2": FUNCDEF(
        restype=ctypes.c_uint32,  # SpvReflectResult
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # SpvReflectShaderModule *
        ]
    ),
    "spvReflectDestroyShaderModule": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SpvReflectShaderModule *
        ]
    ),
    "spvReflectEnumerateDescriptorBindings": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const SpvReflectShaderModule *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # SpvReflectDescriptorBinding **
        ]
    ),
    "spvReflectEnumerateInputVariables": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const SpvReflectShaderModule *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # SpvReflectInterfaceVariable **
        ]
    ),
    "cTkUserServiceWin::AllowedMultiplayer": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceWin *
        ]
    ),
    "cTkUserServiceNull::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceNull *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkUserServiceNull::CreateUserIdentity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceNull *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "cTkUserServiceNull::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceNull *
        ]
    ),
    "cTkUserServiceWin::GetCountryCode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # cTkFixedString<32,char> *
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceWin *
        ]
    ),
    "cTkUserService::Notify_AuthenticationTokenArrived": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserService *
            ctypes.c_uint64,  # const unsigned __int64
            ctypes.c_ulonglong,  # const ITkUserServiceNotifications::AuthToken *
        ]
    ),
    "cTkUserServiceNull::RequestAuthenticationToken": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceNull *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkUserServiceNull::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceNull *
            ctypes.c_float,  # float
        ]
    ),
    "galaxy::api::SelfRegisteringListener<galaxy::api::IAuthListener,&galaxy::api::ListenerRegistrar": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # galaxy::api::SelfRegisteringListener<galaxy::api::IAuthListener,&galaxy::api::ListenerRegistrar> *
        ]
    ),
    "galaxy::api::IOperationalStateChangeListener::~IOperationalStateChangeListener": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # galaxy::api::IOperationalStateChangeListener *
        ]
    ),
    "cTkUserServiceGalaxy::AllowedMultiplayer": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceGalaxy *
        ]
    ),
    "cTkUserServiceGalaxy::Construct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceGalaxy *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "cTkUserServiceGalaxy::Destruct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceGalaxy *
        ]
    ),
    "cTkUserServiceGalaxy::GalaxyListenerHub::OnAuthFailure": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceGalaxy::GalaxyListenerHub *
        ]
    ),
    "cTkUserServiceGalaxy::GalaxyListenerHub::OnAuthSuccess": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceGalaxy::GalaxyListenerHub *
        ]
    ),
    "cTkUserServiceGalaxy::GalaxyListenerEncTicket::OnEncryptedAppTicketRetrieveSuccess": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceGalaxy::GalaxyListenerEncTicket *
        ]
    ),
    "cTkUserServiceGalaxy::GalaxyListenerHub::OnOperationalStateChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceGalaxy::GalaxyListenerHub *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "cTkUserServiceGalaxy::RequestAuthenticationToken": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceGalaxy *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "cTkUserServiceGalaxy::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cTkUserServiceGalaxy *
            ctypes.c_float,  # float
        ]
    ),
    "ProcessGainInt<int>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AK::CAkValueRamp *
        ]
    ),
    "CreateAudioInputSource": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAudioInputSourceParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkFXSrcAudioInput::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFXSrcAudioInput *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFXSrcAudioInput::GetGain": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkFXSrcAudioInput *
        ]
    ),
    "CAkFXSrcSine::GetPluginInfo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFXSrcSine *
            ctypes.c_ulonglong,  # AkPluginInfo *
        ]
    ),
    "CAkFXSrcAudioInput::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFXSrcAudioInput *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkSourcePluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "SetAudioInputCallbacks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'AkAudioBuffer *']}
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'AkAudioFormat *']}
            ctypes.c_ulonglong,  # {'return': 'float', 'args': [{'type': 'unsigned', 'name': 'int'}]}
        ]
    ),
    "CAkFxSrcAudioInputParams::CAkFxSrcAudioInputParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxSrcAudioInputParams *
        ]
    ),
    "CAkFxSrcAudioInputParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxSrcAudioInputParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkFxSrcAudioInputParams::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxSrcAudioInputParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkFxSrcAudioInputParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxSrcAudioInputParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkFxSrcAudioInputParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxSrcAudioInputParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK__DSP__ApplyGain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_double,  # double
            ctypes.c_double,  # double
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK__DSP__ApplyGain_0": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_double,  # double
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CreateAkCompressorFX": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkCompressorFX::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkCompressorFX *
            ctypes.c_ulonglong,  # const AkAudioObjects *
        ]
    ),
    "ExecuteCompression": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float **
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkCompressorSideChain *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CAkCompressorFX::GetPluginInfo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMeterFX *
            ctypes.c_ulonglong,  # AkPluginInfo *
        ]
    ),
    "CAkCompressorFX::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkCompressorFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkEffectPluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "AK__MixPower": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float **
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkCompressorFX::Process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkCompressorFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkCompressorFX::ProcessLinked": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkCompressorFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkCompressorFX::ProcessMultiObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkCompressorFX *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkAudioBuffer **
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CAkCompressorFX::Reset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkCompressorFX *
        ]
    ),
    "CAkCompressorFX::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkCompressorFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkCompressorFXParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # CAkCompressorFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkCompressorFXParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkCompressorFXParams::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkCompressorFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkCompressorFXParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkCompressorFXParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkCompressorFXParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkCompressorFXParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "ExecuteCompression_AVX2": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float **
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkCompressorSideChain *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "AkMismatchBuffer::AddZeroes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMismatchBuffer *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK__DSP__ApplyGainRamp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_double,  # double
            ctypes.c_double,  # double
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkMismatchBuffer::BufferInput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMismatchBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "ComputeSVtr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CreateAkConvolutionReverbFX": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkConvolutionReverbFX::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConvolutionReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "FillMxFBRotate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkRoomVerbFX::GetPluginInfo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
            ctypes.c_ulonglong,  # AkPluginInfo *
        ]
    ),
    "CAkConvolutionReverbFX::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkConvolutionReverbFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkEffectPluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkConvolutionReverbFX::ParametersUpdate": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkConvolutionReverbFX *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkConvolutionReverbFX::ProcessSurroundChannels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConvolutionReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkConvolutionReverbFX::RelocateMedia": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkConvolutionReverbFX *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "CAkConvolutionReverbFX::Reset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkConvolutionReverbFX *
        ]
    ),
    "CAkConvolutionReverbFX::ResetFrontBackDelay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConvolutionReverbFX *
        ]
    ),
    "CAkConvolutionReverbFX::ResetPreDelay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConvolutionReverbFX *
        ]
    ),
    "CAkConvolutionReverbFX::SetupFrontBackDelay": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkConvolutionReverbFX *
        ]
    ),
    "CAkConvolutionReverbFX::SetupPreDelay": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkConvolutionReverbFX *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkConvolutionReverbFX::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkConvolutionReverbFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkConvolutionReverbFX::TermFrontBackDelay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConvolutionReverbFX *
        ]
    ),
    "CAkConvolutionReverbFX::TermPreDelay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConvolutionReverbFX *
        ]
    ),
    "CAkGainFX::TimeSkip": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkTremoloFX *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkConvolutionReverbFX::UpMix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConvolutionReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkConvolutionReverbFXParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # CAkConvolutionReverbFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkConvolutionReverbFXParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkConvolutionReverbFXParams::GetParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConvolutionReverbFXParams *
            ctypes.c_ulonglong,  # AkConvolutionReverbFXParams *
        ]
    ),
    "CAkGuitarDistortionFXParams::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStereoDelayFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkConvolutionReverbFXParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkConvolutionReverbFXParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkConvolutionReverbFXParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkConvolutionReverbFXParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::CDelayLight::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CDelayLight *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::CDelayLight::ProcessBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CDelayLight *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::CDelayLight::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CDelayLight *
        ]
    ),
    "DSP::DelayLine::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::DelayLine *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkPartitionedConvolutionEngine::CAkPartitionedConvolutionEngine": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPartitionedConvolutionEngine *
        ]
    ),
    "CAkPartitionedConvolutionEngine::ExecuteChannels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPartitionedConvolutionEngine *
            ctypes.c_ulonglong,  # float **
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "CAkPartitionedConvolutionEngine::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPartitionedConvolutionEngine *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AK::AkSIMDProcessorSupport
        ]
    ),
    "CAkPartitionedConvolutionEngine::InitConvolutionInstance": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPartitionedConvolutionEngine *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPartitionedConvolutionEngine::RelocateMedia": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPartitionedConvolutionEngine *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "CAkPartitionedConvolutionEngine::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPartitionedConvolutionEngine *
        ]
    ),
    "CAkPartitionedConvolutionEngine::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPartitionedConvolutionEngine *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkPartitionedConvolutionEngine::TermOverlapSaveBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPartitionedConvolutionEngine *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkHalfPrecisionConvolutionEngine::CAkHalfPrecisionConvolutionEngine": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkHalfPrecisionConvolutionEngine *
        ]
    ),
    "CAkHalfPrecisionConvolutionEngine::ExecuteChannels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkHalfPrecisionConvolutionEngine *
            ctypes.c_ulonglong,  # float **
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "CAkHalfPrecisionConvolutionEngine::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkHalfPrecisionConvolutionEngine *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AK::AkSIMDProcessorSupport
        ]
    ),
    "CAkHalfPrecisionConvolutionEngine::InitConvolutionInstance": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkHalfPrecisionConvolutionEngine *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkHalfPrecisionConvolutionEngine::RelocateMedia": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkHalfPrecisionConvolutionEngine *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "CAkHalfPrecisionConvolutionEngine::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkHalfPrecisionConvolutionEngine *
        ]
    ),
    "CAkHalfPrecisionConvolutionEngine::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkHalfPrecisionConvolutionEngine *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkHalfPrecisionConvolutionEngine::TermHistoryBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkHalfPrecisionConvolutionEngine *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "DSP::Mix2": {
        "float *, float *, float *, double, float, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_double,  # double
                ctypes.c_float,  # float
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "float *, float *, double, double, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_double,  # double
                ctypes.c_double,  # double
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "DSP::Mix2Interp": {
        "float *, float *, float *, float, float, float, float, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "float *, float *, float, double, float, float, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_float,  # float
                ctypes.c_double,  # double
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "DSP::ComputeLRMixGains": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "DSP::MixStereoWidth": {
        "float *, float *, float *, float *, unsigned int, float, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
        "float *, float *, unsigned int, float, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
    },
    "DSP::AkDownMix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "AkFft::DetermineFftStages": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkFft::FftSimdStage *
            ctypes.c_uint32,  # AkFft::FftInputType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkFft::FftKernel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # AkFft::FftProcessMode
            ctypes.c_ulonglong,  # const AkFft::FftConfig *
        ]
    ),
    "AkFft::FftReal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const AkFft::FftConfig *
        ]
    ),
    "AkFft::GetFftConfigSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # AkFft::FftInputType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkFft::GetFftTempSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const AkFft::FftConfig *
        ]
    ),
    "AkFft::GetNumStages": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkFft::IfftReal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const AkFft::FftConfig *
        ]
    ),
    "AkFft::InitFftConfig": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkFft::FftConfig *
            ctypes.c_uint32,  # AkFft::FftInputType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AK::AkSIMDProcessorSupport
        ]
    ),
    "AkFft::InitFftConfigOffsets": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkFft::FftConfig *
            ctypes.c_ulonglong,  # AkFft::FftSimdStage *
        ]
    ),
    "AkFft::InitTwiddleTable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkFft::FftConfig *
            ctypes.c_ulonglong,  # AkFft::FftSimdStage *
            ctypes.c_uint32,  # AK::AkSIMDProcessorSupport
        ]
    ),
    "AkFft::InitTwiddleTablePtrs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkFft::cpx_bin **
            ctypes.c_ulonglong,  # AkFft::FftConfig *
            ctypes.c_ulonglong,  # AkFft::FftSimdStage *
        ]
    ),
    "FreqDomainBufferMult_V4F32": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "StoreAccumulatedFreq_V4F32": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "WriteHistoryPartition_V4F32": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "FreqDomainBufferMult_V8_F16C": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "StoreAccumulatedFreq_V8F32": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "WriteHistoryPartition_V8_F16C": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "FreqDomainBufferMult_V8_AVX2": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkFft::Radix3_Stage_Scalar<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix3_Stage_Scalar<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix4_Stage_Scalar<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix4_Stage_Scalar<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix5_Stage_Scalar<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix5_Stage_Scalar<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix8_Stage0_Scalar<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix8_Stage0_Scalar<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::RadixGeneric_StageFin_Scalar<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::RadixGeneric_StageFin_Scalar<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::CpxToReal_V4F32<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::CpxToReal_V4F32<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix3_Stage_V4F32<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix3_Stage_V4F32<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix4_StageFinal_V4F32<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix4_StageFinal_V4F32<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix4_Stage_V4F32<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix4_Stage_V4F32<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix5_Stage_V4F32<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix5_Stage_V4F32<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix8_Stage0_V4F32<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix8_Stage0_V4F32<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::CpxToReal_AVX2<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::CpxToReal_AVX2<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix3_Stage_AVX2<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix3_Stage_AVX2<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix4_StageFinal_AVX2<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix4_StageFinal_AVX2<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix4_Stage_AVX2<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix4_Stage_AVX2<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix5_Stage_AVX2<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix5_Stage_AVX2<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix8_Stage0_AVX2<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "AkFft::Radix8_Stage0_AVX2<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # const AkFft::cpx_bin *
            ctypes.c_ulonglong,  # AkFft::cpx_bin *
        ]
    ),
    "CreateAkDelayFX": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkDelayFXParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkDelayFX::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkDelayFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkTremoloFX::GetPluginInfo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkTremoloFX *
            ctypes.c_ulonglong,  # AkPluginInfo *
        ]
    ),
    "CAkDelayFX::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDelayFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkEffectPluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkDelayFX::Reset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDelayFX *
        ]
    ),
    "CAkDelayFX::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDelayFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkDelayFXParams::CAkDelayFXParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkDelayFXParams *
        ]
    ),
    "CAkDelayFXParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # CAkDelayFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkDelayFXParams::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDelayFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkDelayFXParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDelayFXParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkDelayFXParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDelayFXParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkDelayFXDSP::CAkDelayFXDSP": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkDelayFXDSP *
        ]
    ),
    "CAkDelayFXDSP::ComputeTailLength": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkDelayFXDSP *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "CAkDelayFXDSP::Process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkDelayFXDSP *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkDelayFXParams *
        ]
    ),
    "CAkDelayFXDSP::ResetDelay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::DSP::CAkDelayLineMemory<float> *
        ]
    ),
    "CAkDelayFXDSP::Setup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkDelayFXDSP *
            ctypes.c_ulonglong,  # AkDelayFXParams *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::DSP::CAkDelayLineMemory<float>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::DSP::CAkDelayLineMemory<float> *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkDelayFXDSP::TermDelay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkDelayFXDSP *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "DSP::MonoLFO<DSP::Unipolar,TremoloOutputPolicy>::ComputeFilterCoefsFromParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const DSP::LFO::Params *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CreateAkFlangerFX": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkFlangerFX::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFlangerFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFlangerFX::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFlangerFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkEffectPluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkFlangerFX::LiveParametersUpdate": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFlangerFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "DSP::MonoLFO<DSP::Bipolar,FlangerOutputPolicy>::ProduceBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::MonoLFO<DSP::Bipolar,FlangerOutputPolicy> *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CAkFlangerFX::Reset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFlangerFX *
        ]
    ),
    "DSP::MonoLFO<DSP::Bipolar,FlangerOutputPolicy>::SetParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::MonoLFO<DSP::Unipolar,TremoloOutputPolicy> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const DSP::LFO::Params *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CAkFlangerFX::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFlangerFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkFlangerFX::TermUniCombs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFlangerFX *
        ]
    ),
    "CAkFlangerFXParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # CAkFlangerFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkFlangerFXParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkFlangerFXParams::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFlangerFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkFlangerFXParams::SetDirty": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFlangerFXParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkFlangerFXParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFlangerFXParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkFlangerFXParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFlangerFXParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::UniComb::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::UniComb *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "DSP::UniComb::ProcessBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::UniComb *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "DSP::UniComb::ProcessBufferLFO": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::UniComb *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "DSP::UniComb::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::UniComb *
        ]
    ),
    "DSP::UniComb::SetParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::UniComb *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "DSP::UniComb::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::UniComb *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkGainFX": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkGainFX::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkGainFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkGainFX::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkGainFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkEffectPluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkGainFXParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # CAkGainFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkGainFXParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkGainFXParams::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkGainFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkGainFXParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkGainFXParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkGainFXParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkGainFXParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CreateAkGuitarDistortionFX": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkGuitarDistortionFX::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkGuitarDistortionFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkGuitarDistortionFX::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkGuitarDistortionFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkEffectPluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkGuitarDistortionFX::Reset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkGuitarDistortionFX *
        ]
    ),
    "CAkGuitarDistortionFX::SetupEQs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkGuitarDistortionFX *
            ctypes.c_ulonglong,  # AkGuitarDistortionFXParams *
        ]
    ),
    "CAkGuitarDistortionFX::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkGuitarDistortionFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "AkGuitarDistortionFXParams::AkGuitarDistortionFXParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkGuitarDistortionFXParams *
        ]
    ),
    "CAkGuitarDistortionFXParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # CAkGuitarDistortionFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkGuitarDistortionFXParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkGuitarDistortionFXParams::GetParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkGuitarDistortionFXParams *
            ctypes.c_ulonglong,  # AkGuitarDistortionFXParams *
        ]
    ),
    "CAkGuitarDistortionFXParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkGuitarDistortionFXParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkGuitarDistortionFXParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkGuitarDistortionFXParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::SIMD::ProcessBufferAll<DSP::SIMD::InPlaceBuffer,DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::SIMD::InPlaceBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadMultiCoefs *
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadMultiMemories *
        ]
    ),
    "DSP::SIMD::_SetCoefs<DSP::SIMD::BiquadMultiCoefs>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadMultiCoefs *
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "DSP::CAkMultiBandEQ::CAkMultiBandEQ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkMultiBandEQ *
        ]
    ),
    "DSP__ComputeBiquadCoefs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # DSP::FilterType
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
        ]
    ),
    "DSP::CAkMultiBandEQ::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkMultiBandEQ *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories>::ProcessBuffer": {
        "DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories> *, float *, unsigned int, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories> *
                ctypes.c_ulonglong,  # float *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories> *, float *, unsigned int, unsigned int, float *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories> *
                ctypes.c_ulonglong,  # float *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # float *
            ]
        ),
    },
    "DSP::CAkMultiBandEQ::ProcessBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkMultiBandEQ *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "DSP::CAkMultiBandEQ::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkMultiBandEQ *
        ]
    ),
    "DSP::CAkMultiBandEQ::SetBandActive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkMultiBandEQ *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "DSP::CAkMultiBandEQ::SetCoefficients": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkMultiBandEQ *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # DSP::FilterType
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "DSP::CAkMultiBandEQ::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkMultiBandEQ *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "DSP::SIMD::BiquadMultiCoefs::_Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadMultiCoefs *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "DSP::CAkDistortion::CAkDistortion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkDistortion *
        ]
    ),
    "DSP::CAkDistortion::ProcessBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkDistortion *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "DSP::CAkDistortion::ProcessClip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkDistortion *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::CAkDistortion::ProcessFuzz": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkDistortion *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::CAkDistortion::ProcessHeavy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkDistortion *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::CAkDistortion::ProcessOverdrive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkDistortion *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::CAkDistortion::SetParameters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkDistortion *
            ctypes.c_uint32,  # DSP::CAkDistortion::DistortionType
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkPathManager::CAkPathManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPathManager *
        ]
    ),
    "DSP::CAkRectifier::ProcessBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkRectifier *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "DSP::CAkRectifier::ProcessChannel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkRectifier *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::CAkRectifier::SetRectification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkRectifier *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "DSP::OnePoleZeroHPFilter::ProcessBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::OnePoleZeroHPFilter *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::OnePoleZeroHPFilter::ProcessBufferWithGain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::OnePoleZeroHPFilter *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories>::ComputeCoefs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories> *
            ctypes.c_uint32,  # DSP::FilterType
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
        ]
    ),
    "CreateAkHarmonizerFX": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkHarmonizerFX::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkHarmonizerFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories>::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories> *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkHarmonizerFX::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkHarmonizerFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkEffectPluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkHarmonizerFX::InitDryDelay": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkHarmonizerFX *
        ]
    ),
    "CAkHarmonizerFX::InitPitchVoices": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkHarmonizerFX *
            ctypes.c_uint32,  # AK::AkSIMDProcessorSupport
        ]
    ),
    "CAkHarmonizerFX::Reset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkHarmonizerFX *
        ]
    ),
    "CAkHarmonizerFX::ResetDryDelay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkHarmonizerFX *
        ]
    ),
    "CAkHarmonizerFX::ResetPitchVoices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkHarmonizerFX *
        ]
    ),
    "DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories> *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkHarmonizerFX::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkHarmonizerFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkHarmonizerFX::TermDryDelay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkHarmonizerFX *
        ]
    ),
    "CAkHarmonizerFX::TermPitchVoices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkHarmonizerFX *
        ]
    ),
    "CAkHarmonizerFXParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # CAkHarmonizerFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkHarmonizerFXParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkHarmonizerFXParams::GetParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkHarmonizerFXParams *
            ctypes.c_ulonglong,  # AkHarmonizerFXParams *
        ]
    ),
    "CAkHarmonizerFXParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkHarmonizerFXParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkHarmonizerFXParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkHarmonizerFXParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::BUTTERFLYSET_NAMESPACE::CAkPhaseVocoder::CAkPhaseVocoder": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkPhaseVocoder *
        ]
    ),
    "DSP::BUTTERFLYSET_NAMESPACE::CAkPhaseVocoder::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkPhaseVocoder *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AK::AkSIMDProcessorSupport
            ctypes.c_ubyte,  # bool
        ]
    ),
    "DSP::BUTTERFLYSET_NAMESPACE::CAkPhaseVocoder::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkPhaseVocoder *
        ]
    ),
    "DSP::BUTTERFLYSET_NAMESPACE::CAkPhaseVocoder::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkPhaseVocoder *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "DSP::BUTTERFLYSET_NAMESPACE::CAkResamplingPhaseVocoder::CAkResamplingPhaseVocoder": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkResamplingPhaseVocoder *
        ]
    ),
    "DSP::BUTTERFLYSET_NAMESPACE::CAkResamplingPhaseVocoder::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkResamplingPhaseVocoder *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AK::AkSIMDProcessorSupport
            ctypes.c_ubyte,  # bool
        ]
    ),
    "DSP::BUTTERFLYSET_NAMESPACE::CAkResamplingPhaseVocoder::ProcessPitchChannel": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkResamplingPhaseVocoder *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "DSP::BUTTERFLYSET_NAMESPACE::CAkResamplingPhaseVocoder::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkResamplingPhaseVocoder *
        ]
    ),
    "DSP::BUTTERFLYSET_NAMESPACE::CAkResamplingPhaseVocoder::ResetInputFill": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkResamplingPhaseVocoder *
        ]
    ),
    "DSP::BUTTERFLYSET_NAMESPACE::CAkResamplingPhaseVocoder::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkResamplingPhaseVocoder *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "DSP::SIMD::ProcessBufferMono_<DSP::SIMD::InPlaceBuffer,DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::SIMD::InPlaceBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadMultiCoefs *
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadMultiMemories *
        ]
    ),
    "AkHarmonizerDSPProcess": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkHarmonizerFXInfo *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "AkHarmonizerDSPProcessVoice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # AkHarmonizerFXInfo *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "DSP::CAkCircularBuffer::AdvanceFrames": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkCircularBuffer *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::CAkCircularBuffer::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkCircularBuffer *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::CAkCircularBuffer::ReadFrameBlock": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkCircularBuffer *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "DSP::CAkCircularBuffer::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkCircularBuffer *
        ]
    ),
    "DSP::CAkCircularBuffer::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkCircularBuffer *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "DSP::CAkOLACircularBuffer::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkOLACircularBuffer *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::CAkOLACircularBuffer::IsDoneTail": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkOLACircularBuffer *
        ]
    ),
    "DSP::CAkOLACircularBuffer::PopFrames": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkOLACircularBuffer *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "DSP::CAkOLACircularBuffer::PushOverlappedWindow": {
        "DSP::CAkOLACircularBuffer *, float *, unsigned int, float *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # DSP::CAkOLACircularBuffer *
                ctypes.c_ulonglong,  # float *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # float *
            ]
        ),
        "DSP::CAkOLACircularBuffer *, float *, unsigned int": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # DSP::CAkOLACircularBuffer *
                ctypes.c_ulonglong,  # float *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "DSP::CAkTimeWindow::Apply": {
        "DSP::CAkTimeWindow *, float *, unsigned int, float *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # DSP::CAkTimeWindow *
                ctypes.c_ulonglong,  # float *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # float *
            ]
        ),
        "DSP::CAkTimeWindow *, float *, unsigned int, double, float *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # DSP::CAkTimeWindow *
                ctypes.c_ulonglong,  # float *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_double,  # double
                ctypes.c_ulonglong,  # float *
            ]
        ),
        "DSP::CAkTimeWindow *, float *, unsigned int, double": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # DSP::CAkTimeWindow *
                ctypes.c_ulonglong,  # float *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_double,  # double
            ]
        ),
    },
    "DSP::CAkTimeWindow::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkTimeWindow *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # DSP::CAkTimeWindow::WindowType
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "DSP::CAkTimeWindow::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkTimeWindow *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow::Alloc": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow::CartToPol": {
        "DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow *, AkFft::cpx_bin *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow *
                ctypes.c_ulonglong,  # AkFft::cpx_bin *
            ]
        ),
        "DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow *
            ]
        ),
    },
    "DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow::Compute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkFft::FftConfig *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow::ComputeVocoderSpectrum": {
        "DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow *, DSP::AkPolar *, DSP::AkPolar *, float *, unsigned int, float, bool, DSP::AkPolar *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow *
                ctypes.c_ulonglong,  # DSP::AkPolar *
                ctypes.c_ulonglong,  # DSP::AkPolar *
                ctypes.c_ulonglong,  # float *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
                ctypes.c_ulonglong,  # DSP::AkPolar *
            ]
        ),
        "DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow *, DSP::AkPolar *, DSP::AkPolar *, float *, unsigned int, float, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow *
                ctypes.c_ulonglong,  # DSP::AkPolar *
                ctypes.c_ulonglong,  # DSP::AkPolar *
                ctypes.c_ulonglong,  # float *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow::ConvertToTimeDomain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkFft::FftConfig *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow::Free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "DSP::BUTTERFLYSET_NAMESPACE::PairedCartToPol": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow::PolToCart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::BUTTERFLYSET_NAMESPACE::CAkFreqWindow *
            ctypes.c_ulonglong,  # DSP::AkPolar *
        ]
    ),
    "DSP::BUTTERFLYSET_NAMESPACE::PolarToCart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::CAkResamplingCircularBuffer::PushFrames": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkResamplingCircularBuffer *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "DSP::CAkResamplingCircularBuffer::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::CAkResamplingCircularBuffer *
        ]
    ),
    "CreateAkMatrixReverbFX": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkFDNReverbFX::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkEffectPluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkFDNReverbFX::ProcessFivePointOne12": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessFivePointOne16": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessFivePointOne4": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessFivePointOne8": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessFivePointZero12": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessFivePointZero16": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessFivePointZero4": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessFivePointZero8": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessMono12": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessMono16": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessMono4": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessMono8": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessN12": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessN16": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessN4": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessN8": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessStereo12": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessStereo16": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessStereo4": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::ProcessStereo8": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFDNReverbFX::Reset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
        ]
    ),
    "CAkFDNReverbFX::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkFDNReverbFX::TermDelayLines": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFX *
        ]
    ),
    "CAkFDNReverbFXParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkMatrixReverbFXParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkFDNReverbFXParams::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkFDNReverbFXParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFXParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkFDNReverbFXParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFDNReverbFXParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkMemFreeDefault": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AkMemMalignDefault": FUNCDEF(
        restype=ctypes.c_ulonglong,  # span_t *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkMemMallocDefault": FUNCDEF(
        restype=ctypes.c_ulonglong,  # span_t *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AkMemReallocAlignedDefault": FUNCDEF(
        restype=ctypes.c_ulonglong,  # span_t *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkMemReallocDefault": FUNCDEF(
        restype=ctypes.c_ulonglong,  # span_t *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AkMemSizeOfMemoryDefault": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AkMemTermForThreadDefault": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "AkMemTotalReservedMemorySizeDefault": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[]
    ),
    "AKPLATFORM::AllocVM": FUNCDEF(
        restype=wintypes.LPVOID,  # LPVOID
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "AKPLATFORM::FreeVM": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::MemoryMgr::GetDefaultSettings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMemSettings *
        ]
    ),
    "AK::MemoryMgr::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkMemSettings *
        ]
    ),
    "AK::MemoryMgr::InitForThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::MemoryMgr *
        ]
    ),
    "AK::MemoryMgr::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::MemoryMgr *
        ]
    ),
    "AK::MemoryMgr::TermForThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::MemoryMgr *
        ]
    ),
    "AK::MemoryMgr::Free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AK::MemoryMgr::InitBase": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "AK::MemoryMgr::IsInitialized": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[]
    ),
    "AK::MemoryMgr::Malign": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::MemoryMgr::Malloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::MemoryMgr::Realloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::MemoryMgr::ReallocAligned": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::MemoryMgr::TermBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::MemoryMgr *
        ]
    ),
    "ak_rpaligned_alloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # span_t *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "ak_rpaligned_realloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # span_t *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "ak_rpfree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "ak_rpmalloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # span_t *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "ak_rpmalloc_finalize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "ak_rpmalloc_heap_acquire": FUNCDEF(
        restype=ctypes.c_ulonglong,  # heap_t *
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "ak_rpmalloc_heap_aligned_alloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # heap_t *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "ak_rpmalloc_heap_aligned_realloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # span_t *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # heap_t *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "ak_rpmalloc_heap_alloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # heap_t *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "ak_rpmalloc_heap_free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # heap_t *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "ak_rpmalloc_heap_realloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # span_t *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # heap_t *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "ak_rpmalloc_heap_release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # heap_t *
        ]
    ),
    "ak_rpmalloc_initialize_config": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const rpmalloc_config_t *
        ]
    ),
    "ak_rpmalloc_thread_finalize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "ak_rpmalloc_total_reserved_size": FUNCDEF(
        restype=ctypes.c_ulonglong,  # volatile void *
        argtypes=[]
    ),
    "ak_rpmalloc_usable_size": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "ak_rprealloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # span_t *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "get_thread_heap": FUNCDEF(
        restype=ctypes.c_ulonglong,  # heap_t *
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "AkMeterApplyBallistics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkMeterFXParams *
            ctypes.c_ulonglong,  # AkMeterState *
        ]
    ),
    "AkMeterGetValue": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkMeterFXParams *
            ctypes.c_ulonglong,  # float *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "AkMeterGetValueMultiObjects": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkAudioBuffer **
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # AkMeterFXParams *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # AKRESULT *
        ]
    ),
    "CreateAkMeterFX": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkMeterFX::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMeterFX *
            ctypes.c_ulonglong,  # const AkAudioObjects *
        ]
    ),
    "CAkMeterFX::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMeterFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkEffectPluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkMeterFX::Reset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMeterFX *
        ]
    ),
    "CAkMeterFX::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMeterFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkMeterFXParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMeterFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkMeterFXParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkMeterFXParams::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMeterFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMeterFXParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMeterFXParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMeterFXParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMeterFXParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkMeterFXRegisterCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkGlobalPluginContext *
            ctypes.c_uint32,  # AkGlobalCallbackLocation
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkMeterManager::BehavioralExtension": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkGlobalPluginContext *
            ctypes.c_uint32,  # AkGlobalCallbackLocation
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkMeterManager::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMeterManager *
        ]
    ),
    "CAkMeterManager::Instance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMeterManager *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkGlobalPluginContext *
        ]
    ),
    "CAkMeterManager::Register": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMeterManager *
            ctypes.c_ulonglong,  # CAkMeterFX *
        ]
    ),
    "CAkMeterManager::Unregister": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMeterManager *
            ctypes.c_ulonglong,  # CAkMeterFX *
        ]
    ),
    "AkPBIParams::AkPBIParams": {
        "AkPBIParams *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # AkPBIParams *
            ]
        ),
        "AkPBIParams *, PlayHistory *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # AkPBIParams *
                ctypes.c_ulonglong,  # PlayHistory *
            ]
        ),
        "AkPBIParams *, const AkPBIParams *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # AkPBIParams *
                ctypes.c_ulonglong,  # const AkPBIParams *
            ]
        ),
    },
    "AkInitialSoundParams::~AkInitialSoundParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkInitialSoundParams *
        ]
    ),
    "CAkMusicRenderer::AddChild": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicRenderer::CleanPendingStateChanges": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkMusicRenderer::DestroyMusicRenderer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkGlobalPluginContext *
            ctypes.c_uint32,  # AkGlobalCallbackLocation
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AK::MusicEngine::GetPlayingSegmentInfo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkSegmentInfo *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::MusicEngine::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkMusicSettings *
        ]
    ),
    "CAkMusicRenderer::MusicNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicRenderer::Pause": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # TransParams *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicRenderer::PerformDelayedStateChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkMusicRenderer::Play": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_ulonglong,  # CAkSource *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # TransParams *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const AkTrackSrc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # CAkMusicPBI **
        ]
    ),
    "CAkMusicRenderer::ProcessNextFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkGlobalPluginContext *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkMusicRenderer::RemoveChild": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
        ]
    ),
    "CAkList2<CAkMusicRenderer::AkStateChangeRecord,CAkMusicRenderer::AkStateChangeRecord const &,1,0>::RemoveFirst": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkList2<CAkMusicRenderer::AkStateChangeRecord,CAkMusicRenderer::AkStateChangeRecord const &,1,0> *
        ]
    ),
    "CAkMusicRenderer::RescheduleDelayedStateChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkMusicRenderer::Resume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # TransParams *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicRenderer::SeekPercent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicRenderer::SeekTimeAbsolute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicRenderer::SetState": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicRenderer::Stop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # TransParams *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicRenderer::StopNoteIfUsingData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::MusicEngine::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::MusicEngine *
        ]
    ),
    "CAkMusicCtx::CAkMusicCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkMusicCtx::~CAkMusicCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkMusicCtx::AddChild": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
            ctypes.c_ulonglong,  # CAkChildCtx *
        ]
    ),
    "CAkMusicCtx::HasStopTrans": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkMusicCtx::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicCtx::_EndFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkMusicCtx::OnLastFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicCtx::OnPaused": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkMusicCtx::OnPlayed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkMusicCtx::OnPropertyChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
            ctypes.c_uint32,  # AkPropID
            ctypes.c_float,  # float
        ]
    ),
    "CAkMusicCtx::OnResumed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkMusicCtx::OnStopped": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkMusicCtx::OnTriggerModulators": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkMusicCtx::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkMusicCtx::RemoveChild": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
            ctypes.c_ulonglong,  # CAkChildCtx *
        ]
    ),
    "CAkMusicCtx::SetPBIFade": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
            ctypes.c_ulonglong,  # void *
            ctypes.c_float,  # float
        ]
    ),
    "CAkMusicCtx::TransUpdateValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicCtx::VirtualAddRef": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkMusicCtx::VirtualRelease": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkMusicCtx::_Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkMusicCtx::_NotifyPropertyChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
        ]
    ),
    "CAkMusicCtx::_Pause": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
            ctypes.c_ulonglong,  # TransParams *
        ]
    ),
    "CAkMusicCtx::_Play": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
            ctypes.c_ulonglong,  # AkMusicFade *
        ]
    ),
    "CAkMusicCtx::_Resume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
            ctypes.c_ulonglong,  # TransParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicCtx::_Stop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
            ctypes.c_ulonglong,  # TransParams *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicPBI::FixStartTimeForFadeIn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicPBI *
        ]
    ),
    "CAkMusicPBI::GetAndClearStopOffset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicPBI *
        ]
    ),
    "CAkMusicPBI::GetTargetedParamsSet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkBitArray<unsigned __int64,0> *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicPBI *
            ctypes.c_ulonglong,  # CAkBitArray<unsigned __int64,0> *
        ]
    ),
    "CAkMusicPBI::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicPBI *
        ]
    ),
    "CAkPBI::NotifyParamsChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const CAkBitArray<unsigned __int64,0> *
        ]
    ),
    "CAkMusicPBI::NotifyVirtualOff": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicPBI *
            ctypes.c_uint32,  # AkVirtualQueueBehavior
        ]
    ),
    "CAkMusicPBI::OnLastFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicPBI *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "CAkMusicPBI::OnPaused": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicPBI *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkMusicPBI::OnPropertyChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicPBI *
            ctypes.c_uint32,  # AkPropID
            ctypes.c_float,  # float
        ]
    ),
    "CAkMusicPBI::OnResumed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicPBI *
        ]
    ),
    "CAkMusicPBI::RefreshParameters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicPBI *
            ctypes.c_ulonglong,  # AkInitialSoundParams *
        ]
    ),
    "CAkMusicPBI::SetAutomationValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicPBI *
            ctypes.c_uint32,  # AkClipAutomationType
            ctypes.c_float,  # float
        ]
    ),
    "CAkMusicPBI::SetPBIFade": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicPBI *
            ctypes.c_ulonglong,  # void *
            ctypes.c_float,  # float
        ]
    ),
    "CAkMusicPBI::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicPBI *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicPBI::_Stop": {
        "CAkMusicPBI *, AkPBIStopMode, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkMusicPBI *
                ctypes.c_uint32,  # AkPBIStopMode
                ctypes.c_ubyte,  # bool
            ]
        ),
        "CAkMidiClipCtx *, __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkMidiClipCtx *
                ctypes.c_int64,  # __int64
            ]
        ),
    },
    "CAkMatrixAwareCtx::CAkMatrixAwareCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkMatrixAwareCtx::~CAkMatrixAwareCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
        ]
    ),
    "CAkMatrixAwareCtx::GetAbsoluteTimeOffset": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
        ]
    ),
    "CAkMatrixAwareCtx::GetFirstSegmentNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicSegment *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
            ctypes.c_ulonglong,  # CAkMusicNode **
        ]
    ),
    "CAkMatrixAwareCtx::GetPlayingSegmentInfo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
            ctypes.c_ulonglong,  # AkSegmentInfo *
        ]
    ),
    "CAkMatrixAwareCtx::GetSilentDuration": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
        ]
    ),
    "CAkMatrixAwareCtx::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMatrixAwareCtx::OnStopped": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
        ]
    ),
    "CAkMatrixAwareCtx::ProcessEpilogue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMatrixAwareCtx::SetRegisteredNotif": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMatrixAwareCtx::SetSequencer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
            ctypes.c_ulonglong,  # CAkMatrixSequencer *
        ]
    ),
    "CAkMatrixSequencer::CAkMatrixSequencer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixSequencer *
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkScheduledItem::CAkScheduledItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduledItem *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # CAkSegmentCtx *
        ]
    ),
    "CAkMatrixSequencer::~CAkMatrixSequencer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixSequencer *
        ]
    ),
    "CAkScheduledItem::AttachStopCmd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduledItem *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
            ctypes.c_int32,  # int
        ]
    ),
    "CAkScheduledItem::CanDestroy": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduledItem *
        ]
    ),
    "CAkScheduledItem::CancelPlayback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduledItem *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkScheduledItem::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduledItem *
        ]
    ),
    "CAkMatrixSequencer::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixSequencer *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkScheduledItem::ForcePostExit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduledItem *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMatrixSequencer::GetActiveNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicNode *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixSequencer *
        ]
    ),
    "CAkMatrixSequencer::GetEarliestStateSyncTime": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixSequencer *
            ctypes.c_ulonglong,  # const CAkScheduleWindow *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # __int64 *
        ]
    ),
    "CAkScheduledItem::GetInfo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduledItem *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # AkSegmentInfo *
            ctypes.c_float,  # float
        ]
    ),
    "CAkMatrixSequencer::GetMusicSyncFlags": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixSequencer *
        ]
    ),
    "CAkMatrixSequencer::HandleTrigger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixSequencer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkScheduledItem::NotifyMusicCallbacks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduledItem *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "CAkScheduledItem::OnPaused": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduledItem *
        ]
    ),
    "CAkMatrixSequencer::OnStopped": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixSequencer *
        ]
    ),
    "CAkScheduledItem::OnStopped": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduledItem *
        ]
    ),
    "CAkScheduledItem::PopAssociatedActionsToRescheduleAfterTransitionSyncPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduledItem *
            ctypes.c_ulonglong,  # AkListBareLight<AkAssociatedAction,AkListBareLightNextItem> *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkScheduledItem::Prepare": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduledItem *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
        ]
    ),
    "CAkMatrixSequencer::Process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixSequencer *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkScheduledItem::Process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduledItem *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMatrixSequencer::ProcessDelayedStateChange": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixSequencer *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkMatrixSequencer::ProcessStingers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixSequencer *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMatrixSequencer::QueryStateChangeDelay": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixSequencer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # __int64 *
        ]
    ),
    "CAkMatrixSequencer::RemoveAllPendingStingers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixSequencer *
        ]
    ),
    "CAkMatrixSequencer::RescheduleCancelledActions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixSequencer *
            ctypes.c_ulonglong,  # AkListBareLight<AkAssociatedAction,AkListBareLightNextItem> *
        ]
    ),
    "AkTimeConv__SamplesToMilliseconds": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "CAkMatrixSequencer::ScheduleStingerForPlayback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixSequencer *
            ctypes.c_ulonglong,  # const CAkScheduleWindow *
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
            ctypes.c_ulonglong,  # const CAkStinger *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkScheduledItem::SetFadeIn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduledItem *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
            ctypes.c_int32,  # int
        ]
    ),
    "CAkMatrixSequencer::Trigger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMatrixSequencer *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkScheduledItem::_CancelPlayback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduledItem *
            ctypes.c_int32,  # int
        ]
    ),
    "AkMusicBank::LoadBankItem": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const AkBank::AKBKSubHircSection *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkMusicBank::UnloadBankSlot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkUsageSlot *
        ]
    ),
    "CAkMusicTransAware::CAkMusicTransAware": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTransAware *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicTransAware::~CAkMusicTransAware": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTransAware *
        ]
    ),
    "CAkParentNode<CAkParameterNode>::AddChildInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParentNode<CAkParameterNode> *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "AkSortedKeyArray<uint,CAkParameterNodeBase *,AkHybridAllocator<8,1,0>,AkChildIDValueGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkParameterNodeBase *>,AkDefaultSortedKeyCompare<uint>>::AddNoSetKey": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkParameterNodeBase **
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,CAkParameterNodeBase *,AkHybridAllocator<8,1,0>,AkChildIDValueGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkParameterNodeBase *>,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParameterNodeBase::BeforeEffectsUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParentNode<CAkParameterNode>::Children": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParentNode<CAkParameterNode> *
        ]
    ),
    "CAkActiveParent<CAkParameterNode>::ClearLimiters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNode> *
        ]
    ),
    "AkSortedKeyArray<uint,CAkParameterNodeBase *,AkHybridAllocator<8,1,0>,AkChildIDValueGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkParameterNodeBase *>,AkDefaultSortedKeyCompare<uint>>::Exists": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkParameterNodeBase **
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,CAkParameterNodeBase *,AkHybridAllocator<8,1,0>,AkChildIDValueGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkParameterNodeBase *>,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActiveParent<CAkParameterNode>::ForAllPBI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNode> *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['CAkPBI *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkParentNode<CAkParameterNode>::GatherSounds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParentNode<CAkParameterNode> *
            ctypes.c_ulonglong,  # AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *
            ctypes.c_ulonglong,  # AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *
            ctypes.c_ulonglong,  # AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParentNode<CAkParameterNode>::GetChildren": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParentNode<CAkParameterNode> *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # AkObjectInfo *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicTransAware::GetPanicTransitionRule": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkMusicTransitionRule *
        argtypes=[]
    ),
    "CAkParamNodeStateAware::GetStateProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkArray<AkStatePropertyInfo,AkStatePropertyInfo &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkStatePropertyInfo> > *
        argtypes=[
            ctypes.c_ulonglong,  # CAkParamNodeStateAware *
        ]
    ),
    "CAkMusicTransAware::GetTransitionRule": {
        "CAkMusicTransAware *, unsigned int, unsigned int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # AkMusicTransitionRule *
            argtypes=[
                ctypes.c_ulonglong,  # CAkMusicTransAware *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "CAkMusicTransAware *, CAkParameterNodeBase *, unsigned int, CAkParameterNodeBase *, unsigned int, CAkParameterNodeBase *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # AkMusicTransitionRule *
            argtypes=[
                ctypes.c_ulonglong,  # CAkMusicTransAware *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ]
        ),
    },
    "AkArray<uint,uint const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<uint>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<enum AkSyncType,enum AkSyncType const ,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<enum AkSyncType> > *
        ]
    ),
    "AkArray<CAkLimiter *,CAkLimiter *,AkHybridAllocator<8,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkLimiter *>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<CAkLimiter *,CAkLimiter *,AkHybridAllocator<8,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkLimiter *> > *
        ]
    ),
    "AkArray<AkMusicTransitionRule,AkMusicTransitionRule const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkMusicTransitionRule>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<PluginRTPCSub,PluginRTPCSub const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<PluginRTPCSub> > *
        ]
    ),
    "CAkActiveParent<CAkParameterNode>::MuteNotification": {
        "CAkActiveParent<CAkParameterNode> *, float, AkMutedMapItem *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNode> *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # AkMutedMapItem *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "CAkActiveParent<CAkParameterNode> *, float, CAkRegisteredObj *, AkMutedMapItem *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNode> *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # CAkRegisteredObj *
                ctypes.c_ulonglong,  # AkMutedMapItem *
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "CAkActiveParent<CAkParameterNode>::NotifyBypass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNode> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "CAkMusicNode::NotifyParamChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkMusicNode::NotifyParamsChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const CAkBitArray<unsigned __int64,0> *
        ]
    ),
    "CAkParameterNodeBase::OnPreRelease": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNode::ParamMustNotify": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_uint32,  # AkRTPC_ParameterID
        ]
    ),
    "CAkParameterNode::ParamOverriden": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_uint32,  # AkRTPC_ParameterID
        ]
    ),
    "CAkActiveParent<CAkParameterNode>::PlayToEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNode> *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParameterNodeBase::PositioningChangeNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkMusicTransAware::PrepareMusicalDependencies": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTransAware *
        ]
    ),
    "CAkActiveParent<CAkParameterNode>::PropagatePositioningNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNode> *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "CAkActiveParent<CAkParameterNode>::RecalcNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNode> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParentNode<CAkParameterNode>::RemoveChild": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParentNode<CAkParameterNode> *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNode::SetBelowThresholdBehavior": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_uint32,  # AkBelowThresholdBehavior
        ]
    ),
    "CAkMusicTransAware::SetMusicTransNodeParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTransAware *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkArray<AkMusicTransitionRule,AkMusicTransitionRule const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkMusicTransitionRule>>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkMusicTransitionRule,AkMusicTransitionRule const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkMusicTransitionRule> > *
        ]
    ),
    "CAkMusicTransAware::TermPanicTransitionRule": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkMusicTransAware::UnPrepareMusicalDependencies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTransAware *
        ]
    ),
    "CAkActiveParent<CAkParameterNode>::UpdateFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNode> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSegmentInfoRepository::CAkSegmentInfoRepository": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentInfoRepository *
        ]
    ),
    "CAkSegmentInfoRepository::~CAkSegmentInfoRepository": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentInfoRepository *
        ]
    ),
    "CAkSegmentInfoRepository::CreateEntry": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentInfoRepository *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSegmentInfoRepository::GetSegmentInfo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentInfoRepository *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkSegmentInfo *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkArray<MapStruct<uint,AkSegmentInfoRecord>,MapStruct<uint,AkSegmentInfoRecord> const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<uint,AkSegmentInfoRecord>>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<MapStruct<unsigned int,AkSegmentInfoRecord>,MapStruct<unsigned int,AkSegmentInfoRecord> const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<unsigned int,AkSegmentInfoRecord> > > *
        ]
    ),
    "CAkSegmentInfoRepository::RemoveEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentInfoRepository *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSegmentInfoRepository::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankList *
        ]
    ),
    "CAkSegmentInfoRepository::UpdateSegmentInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentInfoRepository *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkSegmentInfo *
        ]
    ),
    "CAkChildCtx::CAkChildCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkChildCtx *
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkChildCtx::~CAkChildCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkChildCtx *
        ]
    ),
    "CAkChildCtx::Connect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkChildCtx *
        ]
    ),
    "CAkChildCtx::Disconnect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkChildCtx *
        ]
    ),
    "AkMusicActionPlayClip::AttachClipAutomation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMusicActionPlayClip *
            ctypes.c_ulonglong,  # CAkMusicTrack *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkClipAutomationType
            ctypes.c_int32,  # int
        ]
    ),
    "CAkConversionTable::ConvertInternal": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkConversionTable *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkSubTrackCtx::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkSubTrackCtx *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicCtx *
            ctypes.c_ulonglong,  # CAkMusicTrack *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSubTrackCtx::ExecuteSequencerCmds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSubTrackCtx *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "CAkSubTrackCtx::GetSourceInfoForPlaybackRestart": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkSubTrackCtx *
            ctypes.c_ulonglong,  # const CAkMusicPBI *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "CAkSubTrackCtx::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSubTrackCtx *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSubTrackCtx::OnPaused": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSubTrackCtx *
        ]
    ),
    "CAkSubTrackCtx::OnPlayed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSubTrackCtx *
        ]
    ),
    "CAkSubTrackCtx::OnStopped": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSubTrackCtx *
        ]
    ),
    "CAkSubTrackCtx::OnTriggerModulators": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSubTrackCtx *
        ]
    ),
    "CAkSubTrackCtx::Prepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSubTrackCtx *
            ctypes.c_int32,  # int
        ]
    ),
    "CAkSubTrackCtx::Process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSubTrackCtx *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "CAkSubTrackCtx::RemoveReferences": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSubTrackCtx *
            ctypes.c_ulonglong,  # CAkChildCtx *
        ]
    ),
    "CAkSubTrackCtx::ScheduleSequencerCmds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSubTrackCtx *
        ]
    ),
    "CAkSequenceCtx::SetPBIFade": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequenceCtx *
            ctypes.c_ulonglong,  # void *
            ctypes.c_float,  # float
        ]
    ),
    "CAkChainCtx::CAkChainCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkChainCtx *
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkChainCtx::~CAkChainCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkChainCtx *
        ]
    ),
    "CAkChainCtx::CanRestartPlaying": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkChainCtx *
        ]
    ),
    "CAkChainCtx::CancelPlayback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkChainCtx *
            ctypes.c_int64,  # __int64
        ]
    ),
    "AkScheduledChain::SelfContainedIter::CtxTimeToSegmentPosition": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkScheduledChain::SelfContainedIter *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkChainCtx::EnqueueItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkChainCtx *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # CAkSegmentCtx *
        ]
    ),
    "CAkChainCtx::Flush": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkChainCtx *
        ]
    ),
    "CAkChainCtx::GetNextScheduleWindow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkChainCtx *
            ctypes.c_ulonglong,  # CAkScheduleWindow *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkScheduledChain::SelfContainedIter::GetScheduledItemTime": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkScheduledChain::SelfContainedIter *
        ]
    ),
    "CAkChainCtx::GetSegmentPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkChainCtx *
            ctypes.c_ulonglong,  # CAkScheduledItem *
        ]
    ),
    "CAkMusicSwitchCtx::Node": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicSwitchCntr *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
        ]
    ),
    "CAkChainCtx::OnStopped": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkChainCtx *
        ]
    ),
    "CAkChainCtx::Prepare": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkChainCtx *
            ctypes.c_ulonglong,  # const AkMusicTransDestRule *
            ctypes.c_ulonglong,  # const CAkMusicPlaybackHistory *
            ctypes.c_ulonglong,  # AkMusicSeekInfo *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkChainCtx::Process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkChainCtx *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkCutoffInfo *
        ]
    ),
    "CAkChainCtx::QueryLookAheadInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkChainCtx *
            ctypes.c_ulonglong,  # __int64 *
            ctypes.c_ulonglong,  # __int64 *
        ]
    ),
    "CAkChainCtx::RefreshWindow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkChainCtx *
            ctypes.c_ulonglong,  # CAkScheduleWindow *
        ]
    ),
    "AkScheduledChain::SelfContainedIter::SegmentPositionToCtxTime": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkScheduledChain::SelfContainedIter *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkScheduleWindow::CAkScheduleWindow": {
        "CAkScheduleWindow *, CAkMatrixAwareCtx *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkScheduleWindow *
                ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "CAkScheduleWindow *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkScheduleWindow *
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "CAkScheduleWindow::CancelActionsAfterTransitionSyncPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduleWindow *
            ctypes.c_ulonglong,  # AkListBareLight<AkAssociatedAction,AkListBareLightNextItem> *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkScheduleWindow::ChainCtxTimeRelativeToLevel": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduleWindow *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkScheduleWindow::CtxTimeToSegmentPosition": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduleWindow *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkScheduleWindow::FindSyncPoint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduleWindow *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # AkSyncType
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # __int64 *
        ]
    ),
    "CAkScheduleWindow::GetNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicSegment *
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduleWindow *
            ctypes.c_ulonglong,  # CAkMusicNode **
        ]
    ),
    "CAkScheduleWindow::IsAtExitCue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduleWindow *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CAkScheduleWindow::NotifyMusicCallbacks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduleWindow *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "CAkScheduleWindow::SetBranch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduleWindow *
            ctypes.c_ulonglong,  # const AkListBare<CAkMusicSwitchTransition,AkListBareNextItem,AkCountPolicyNoCount,AkLastPolicyWithLast>::Iterator *
        ]
    ),
    "CAkScheduleWindow::SetDuration": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduleWindow *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkScheduleWindow::SetScheduledItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduleWindow *
            ctypes.c_ulonglong,  # const AkScheduledChain::SelfContainedIter *
        ]
    ),
    "CAkScheduleWindow::StartTime": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduleWindow *
        ]
    ),
    "CAkScheduleWindow::StartTimeRelativeToCurrentLevel": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkScheduleWindow *
        ]
    ),
    "CAkSegmentCtx::CAkSegmentCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkSegmentCtx::AddSwitchTrackNotif": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # CAkMusicTrack *
        ]
    ),
    "CAkSegmentCtx::ExecuteSequencerCmds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "CAkSegmentCtx::GameObjectPtr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkRegisteredObj *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
        ]
    ),
    "CAkSegmentCtx::GetClipPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
        ]
    ),
    "CAkSegmentCtx::GetTimeWindowSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
        ]
    ),
    "CAkSegmentCtx::GetUserParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # UserParams *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
        ]
    ),
    "CAkSegmentCtx::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSegmentCtx::OnPaused": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
        ]
    ),
    "CAkSegmentCtx::OnPlayed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
        ]
    ),
    "CAkSegmentCtx::OnStopped": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
        ]
    ),
    "CAkSegmentCtx::PlayDirectly": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
        ]
    ),
    "CAkSegmentCtx::PlayingID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
        ]
    ),
    "CAkSegmentCtx::Prepare": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
            ctypes.c_int32,  # int
        ]
    ),
    "CAkSegmentCtx::Process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "CAkSegmentCtx::RemoveReferences": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
            ctypes.c_ulonglong,  # CAkSubTrackCtx *
        ]
    ),
    "CAkSegmentCtx::RescheduleSwitchTrack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
            ctypes.c_ulonglong,  # CAkSwitchTrackInfo *
        ]
    ),
    "CAkSegmentCtx::ScheduleSequencerCmds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
        ]
    ),
    "CAkSegmentCtx::ScheduleSequencerPlayCmdsOnSwitch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
            ctypes.c_ulonglong,  # CAkSwitchTrackInfo *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const AkMusicFade *
        ]
    ),
    "CAkRTPCMgr::GetRTPCConvertedValue<AccumulateAdd,CurrentValue,AkDeltaMonitorDisabled>": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkRTPCMgr::GetRTPCConvertedValue<AkDeltaMonitorDisabled>": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkRTPCMgr::GetRTPCValue<CurrentValue>": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # CAkRTPCMgr::SubscriberType
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkStateAware::GetStateParamValue<AccumulateMult,AkDeltaMonitorDisabled>": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
            ctypes.c_uint32,  # AkRTPC_ParameterID
        ]
    ),
    "CAkMusicNode::CAkMusicNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParentNode<CAkParameterNode>::~CAkParentNode<CAkParameterNode>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParentNode<CAkParameterNode> *
        ]
    ),
    "CAkMusicNode::~CAkMusicNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
        ]
    ),
    "CAkActiveParent<CAkParameterNode>::ExecuteAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNode> *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkMusicNode::ExecuteAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkActiveParent<CAkParameterNode>::ExecuteActionExcept": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNode> *
            ctypes.c_ulonglong,  # ActionParamsExcept *
        ]
    ),
    "CAkMusicNode::ExecuteActionExcept": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_ulonglong,  # ActionParamsExcept *
        ]
    ),
    "CAkActiveParent<CAkParameterNode>::ExecuteActionNoPropagate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNodeBase> *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkMusicNode::ExecuteActionNoPropagate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "AkSortedKeyArray<uchar,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>::ChildTreeType>,AkDefaultSortedKeyCompare<uchar>>::Exists": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned char,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType>,AkDefaultSortedKeyCompare<unsigned char> > *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "?Exists@?$AkSortedKeyArray@IVChildTreeType@?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkRTPCValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkRTPCValue@@@@@@@@@@@@U?$AkArrayAllocatorNoAlign@$0A@@@V12@UAkGrowByPolicy_Proportional@@U?$AkTransferMovePolicy@VChildTreeType@?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkRTPCValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkRTPCValue@@@@@@@@@@@@@@U?$AkDefaultSortedKeyCompare@I@@@@QEBAPEAVChildTreeType@?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkRTPCValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkRTPCValue@@@@@@@@@@@@I@Z": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType>,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkSortedKeyArray<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType>,AkDefaultSortedKeyCompare<CAkPBI *>>::Exists": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType>,AkDefaultSortedKeyCompare<CAkPBI *> > *
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "?Exists@?$AkSortedKeyArray@PEAVCAkRegisteredObj@@VChildTreeType@?$AkRTPCNestedSearchTree@VAkRTPCKey@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkRTPCValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkRTPCValue@@@@@@@@@@@@@@U?$AkArrayAllocatorNoAlign@$0A@@@V23@UAkGrowByPolicy_Proportional@@U?$AkTransferMovePolicy@VChildTreeType@?$AkRTPCNestedSearchTree@VAkRTPCKey@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkRTPCValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkRTPCValue@@@@@@@@@@@@@@@@U?$AkDefaultSortedKeyCompare@PEAVCAkRegisteredObj@@@@@@QEBAPEAVChildTreeType@?$AkRTPCNestedSearchTree@VAkRTPCKey@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkRTPCValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkRTPCValue@@@@@@@@@@@@@@PEAVCAkRegisteredObj@@@Z": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkRegisteredObj *,AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > >::ChildTreeType>,AkDefaultSortedKeyCompare<CAkRegisteredObj *> > *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>::FindBestMatch": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCValue *
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > *
            ctypes.c_ulonglong,  # AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>::FindBestMatch": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCValue *
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > *
            ctypes.c_ulonglong,  # AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > > *
        ]
    ),
    "CAkMusicNode::FlushStingers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
        ]
    ),
    "CAkMusicNode::GetMidiTargetNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkMusicNode::GetMidiTargetTempo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkMusicNode::GetMidiTempoSource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # AkMidiTempoSource *
        ]
    ),
    "CAkMusicNode::GetMusicGrid": FUNCDEF(
        restype=ctypes.c_ulonglong,  # ParamNodeStateAwareData **
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
        ]
    ),
    "CAkMusicNode::GetMusicParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_ulonglong,  # AkMusicParams *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkMusicNode::GetStingers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_ulonglong,  # CAkMusicNode::CAkStingers *
        ]
    ),
    "AkArray<CAkStinger,CAkStinger const &,AkArrayAllocatorNoAlign<2>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkStinger>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<CAkStinger,CAkStinger const &,AkArrayAllocatorNoAlign<2>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkStinger> > *
        ]
    ),
    "CAkMusicNode::MusicNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkActiveParent<CAkParameterNode>::ParamNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNode> *
            ctypes.c_ulonglong,  # NotifParams *
        ]
    ),
    "CAkMusicNode::ParamNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_ulonglong,  # NotifParams *
        ]
    ),
    "CAkActiveParent<CAkParameterNode>::PrepareData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNode> *
        ]
    ),
    "CAkMusicNode::PrepareData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
        ]
    ),
    "CAkMusicNode::PrepareMusicalDependencies": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
        ]
    ),
    "CAkMusicNode::PushParamUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "CAkMusicNode::RecalcNotificationWithBitArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const CAkBitArray<unsigned __int64,0> *
        ]
    ),
    "CAkMusicNode::RecalcNotificationWithID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # AkRTPC_ParameterID
        ]
    ),
    "CAkMusicNode::SetAkProp": {
        "CAkMusicNode *, AkPropID, int, int, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkMusicNode *
                ctypes.c_uint32,  # AkPropID
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
            ]
        ),
        "CAkMusicNode *, AkPropID, float, float, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkMusicNode *
                ctypes.c_uint32,  # AkPropID
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
    },
    "CAkMusicNode::SetMusicNodeParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicNode::SetStingers": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
            ctypes.c_ulonglong,  # CAkStinger *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActiveParent<CAkParameterNode>::UnPrepareData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNode> *
        ]
    ),
    "CAkMusicNode::UnPrepareData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
        ]
    ),
    "CAkMusicNode::UnPrepareMusicalDependencies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicNode *
        ]
    ),
    "CAkMusicSegment::ActiveDuration": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
        ]
    ),
    "CAkMusicSegment::CanAddChild": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkMusicSegment::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicSegment *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicSegment::CreateContext": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicCtx *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicSegment::CreateLowLevelSegmentCtxAndAddRef": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicCtx *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicSegment::CreateSegmentContext": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicCtx *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicSegment::ExecuteAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkMusicSegment::ExecuteActionExcept": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_ulonglong,  # ActionParamsExcept *
        ]
    ),
    "CAkMusicSegment::GetClosestCuePosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_int32,  # int
        ]
    ),
    "CAkMusicSegment::GetEntrySyncPos": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_ulonglong,  # const AkMusicTransDestRule *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkMusicSegment::GetExitSyncPos": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkSyncType
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkMusicSegment::GetNumCuesWithFilter": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicSegment::GetNumMusicGridInRange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkMusicSegment::GetRandomCue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkMusicSegment::GetStateSyncTypes": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkStateAware::CAkStateSyncArray *
        ]
    ),
    "CAkMusicSegment::NotifyUserCuesInRange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkSegmentInfo *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicSegment::PlayInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_ulonglong,  # AkPBIParams *
        ]
    ),
    "CAkMusicSegment::PostExitDuration": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
        ]
    ),
    "CAkMusicSegment::PreEntryDuration": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
        ]
    ),
    "CAkMusicSegment::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkArray<AkMusicMarker,AkMusicMarker const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_NoGrow,AkAssignmentMovePolicy<AkMusicMarker>>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkMusicMarker,AkMusicMarker const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_NoGrow,AkAssignmentMovePolicy<AkMusicMarker> > *
        ]
    ),
    "CAkMusicSegment::Track": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkParameterNodeBase *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkMusicRanSeqCntr::AddPlaylistChildren": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicRanSeqCntr *
            ctypes.c_ulonglong,  # CAkRSSub *
            ctypes.c_ulonglong,  # AkMusicRanSeqPlaylistItem **
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicRanSeqCntr::CanAddChild": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicRanSeqCntr *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkMusicRanSeqCntr::CloneGlobalRSInfo": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicRanSeqCntr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const CAkContainerBaseInfo *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkMusicRanSeqCntr::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicRanSeqCntr *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicRanSeqCntr::CreateContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicRanSeqCntr *
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicRanSeqCntr::CreateSequenceCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicRanSeqCntr *
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicRanSeqCntr::ExecuteAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicRanSeqCntr *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkMusicRanSeqCntr::FlushPlaylist": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicRanSeqCntr *
        ]
    ),
    "CAkMusicRanSeqCntr::GetGlobalRSInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkContainerBaseInfo_vtbl *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicRanSeqCntr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkArray<CAkModulator *,CAkModulator *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkModulator *>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<WwiseObjectIDext,WwiseObjectIDext const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<WwiseObjectIDext> > *
        ]
    ),
    "AkArray<AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<CAkMultiPlayNode::ContParam,CAkMultiPlayNode::ContParam const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkMultiPlayNode::ContParam> > *
        ]
    ),
    "CAkMusicRanSeqCntr::PlayInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicRanSeqCntr *
            ctypes.c_ulonglong,  # AkPBIParams *
        ]
    ),
    "CAkMusicRanSeqCntr::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicRanSeqCntr *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicRanSeqCntr::SetPlayList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicRanSeqCntr *
            ctypes.c_ulonglong,  # AkMusicRanSeqPlaylistItem *
        ]
    ),
    "CAkMusicSwitchCntr::CanAddChild": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCntr *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkMusicSwitchCntr::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkParameterNodeBase *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicSwitchCntr::CreateContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCntr *
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicSwitchCntr::ExecuteAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCntr *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkMusicSwitchCntr::GatherSounds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCntr *
            ctypes.c_ulonglong,  # AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *
            ctypes.c_ulonglong,  # AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *
            ctypes.c_ulonglong,  # AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicSwitchCntr::ModifyActiveState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCntr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicSwitchCntr::PlayInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCntr *
            ctypes.c_ulonglong,  # AkPBIParams *
        ]
    ),
    "CAkMusicSwitchCntr::PrepareData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCntr *
        ]
    ),
    "CAkMusicSwitchCntr::SetAkProp": {
        "CAkMusicSwitchCntr *, AkPropID, int, int, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkMusicSwitchCntr *
                ctypes.c_uint32,  # AkPropID
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
            ]
        ),
        "CAkMusicSwitchCntr *, AkPropID, float, float, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkMusicSwitchCntr *
                ctypes.c_uint32,  # AkPropID
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
    },
    "CAkMusicSwitchCntr::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCntr *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicSwitchCntr::UnPrepareData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCntr *
        ]
    ),
    "CAkMusicTrack::AddPlaylistItem": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTrack *
            ctypes.c_ulonglong,  # AkTrackSrcInfo *
        ]
    ),
    "CAkMusicTrack::ComputeMinSrcLookAhead": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTrack *
            ctypes.c_int32,  # int
        ]
    ),
    "CAkMusicTrack::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicTrack *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicTrack::GatherSounds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTrack *
            ctypes.c_ulonglong,  # AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *
            ctypes.c_ulonglong,  # AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *
            ctypes.c_ulonglong,  # AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicTrack::GetDefaultSwitch": FUNCDEF(
        restype=ctypes.c_ulonglong,  # TrackSwitchInfo *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTrack *
        ]
    ),
    "CAkMusicTrack::GetMidiTargetNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTrack *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkMusicTrack::GetMidiTempoSource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTrack *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # AkMidiTempoSource *
        ]
    ),
    "CAkMusicTrack::GetNextRS": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTrack *
        ]
    ),
    "CAkMusicTrack::GetSourcePtr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicSource **
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTrack *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicTrack::GetStateSyncTypes": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTrack *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkStateAware::CAkStateSyncArray *
        ]
    ),
    "AkArray<AkTrackSrc,AkTrackSrc const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkTrackSrc>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkTrackSrc,AkTrackSrc const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkTrackSrc> > *
        ]
    ),
    "CAkMusicTrack::HasBankSource": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTrack *
        ]
    ),
    "CAkMusicTrack::MuteNotification": {
        "CAkMusicTrack *, float, AkMutedMapItem *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkMusicTrack *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # AkMutedMapItem *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "CAkMusicTrack *, float, CAkRegisteredObj *, AkMutedMapItem *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkMusicTrack *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # CAkRegisteredObj *
                ctypes.c_ulonglong,  # AkMutedMapItem *
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "CAkMusicTrack::PrepareData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTrack *
        ]
    ),
    "CAkConversionTable::RecomputeCoefficients": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConversionTable *
        ]
    ),
    "CAkMusicTrack::RemoveAllSourcesNoCheck": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTrack *
        ]
    ),
    "CAkKeyArray<uint,CAkMusicSource *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<uint,CAkMusicSource *>>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicSource **
        argtypes=[
            ctypes.c_ulonglong,  # CAkKeyArray<unsigned int,CAkMusicSource *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<unsigned int,CAkMusicSource *> > > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicTrack::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTrack *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "TrackSwitchInfo::SetSwitchParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # TrackSwitchInfo *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkMusicTrack::ClipAutomationArray::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTrack::ClipAutomationArray *
        ]
    ),
    "CAkMusicTrack::UnPrepareData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicTrack *
        ]
    ),
    "CAkKeyArray<uint,CAkMusicSource *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<uint,CAkMusicSource *>>>::Unset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkKeyArray<unsigned int,CAkMusicSource *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<unsigned int,CAkMusicSource *> > > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicActionSequencer::CAkMusicActionSequencer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicActionSequencer *
        ]
    ),
    "CAkMusicActionSequencer::~CAkMusicActionSequencer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicActionSequencer *
        ]
    ),
    "CAkMusicSubTrackSequencer::ClearSubTrackCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicClipSequencer *
            ctypes.c_ulonglong,  # CAkChildCtx *
        ]
    ),
    "CAkMusicActionSequencer::Flush": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicActionSequencer *
        ]
    ),
    "CAkMusicActionSequencer::PopImminentAction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicActionSequencer *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # AkMusicAction **
        ]
    ),
    "CAkMusicActionSequencer::ScheduleAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicActionSequencer *
            ctypes.c_ulonglong,  # AkMusicAction *
        ]
    ),
    "CAkSwitchTrackInfo::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkSwitchTrackInfo *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSegmentCtx *
            ctypes.c_ulonglong,  # CAkMusicTrack *
        ]
    ),
    "CAkRandomInfo::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequenceInfo *
        ]
    ),
    "CAkSwitchTrackInfo::InitSwitch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchTrackInfo *
        ]
    ),
    "CAkSwitchTrackInfo::IsSubTrackActive": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchTrackInfo *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkSwitchTrackInfo::SetSwitch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchTrackInfo *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "CAkSwitchTrackInfo::UpdateActiveSubTrack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchTrackInfo *
        ]
    ),
    "CAkSequencableSegmentCtx::CAkSequencableSegmentCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequencableSegmentCtx *
            ctypes.c_ulonglong,  # CAkMusicSegment *
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkSequencableSegmentCtx::GetPlayingSegmentInfo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequencableSegmentCtx *
            ctypes.c_ulonglong,  # AkSegmentInfo *
        ]
    ),
    "CAkSequencableSegmentCtx::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequencableSegmentCtx *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSequencableSegmentCtx::Node": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicRanSeqCntr *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequenceCtx *
        ]
    ),
    "CAkSequencableSegmentCtx::Seek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequencableSegmentCtx *
            ctypes.c_int32,  # int
        ]
    ),
    "CAkSequencableSegmentCtx::SeekPercent": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequencableSegmentCtx *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSequencableSegmentCtx::SeekTimeAbsolute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequencableSegmentCtx *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSequencableSegmentCtx::SetupSegmentChain": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequencableSegmentCtx *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkRSIterator::AkRSIterator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
        ]
    ),
    "CAkRSSub::~CAkRSSub": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRSSub *
        ]
    ),
    "AkRSIterator::CreateRSInfo": {
        "CAkRSSub *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkRSSub *
            ]
        ),
        "RSType, unsigned __int16, unsigned __int16": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_uint32,  # RSType
                ctypes.c_uint16,  # unsigned __int16
                ctypes.c_uint16,  # unsigned __int16
            ]
        ),
    },
    "AkRSIterator::CreateRandomInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "RSStackItem::Deserialize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RSStackItem *
            ctypes.c_ulonglong,  # AK::IReadBytes *
            ctypes.c_ulonglong,  # CAkRSSub **
            ctypes.c_ulonglong,  # AkRSIterator *
        ]
    ),
    "AkRSIterator::DeserializeGlobalRSInfo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
            ctypes.c_ulonglong,  # AK::IReadBytes *
        ]
    ),
    "AkRSIterator::DeserializeSegmentInfo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
            ctypes.c_ulonglong,  # AK::IReadBytes *
        ]
    ),
    "AkArray<ushort,ushort,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<ushort>>::Erase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<unsigned short,unsigned short,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned short> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkRSIterator::FindAndSelect": FUNCDEF(
        restype=ctypes.c_uint32,  # AKRESULT
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
            ctypes.c_ulonglong,  # CAkRSNode *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkArray<CAkRSNode *,CAkRSNode *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkRSNode *> > *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AkRSIterator::FlushGlobalRSInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
        ]
    ),
    "AkRSIterator::FlushStack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
        ]
    ),
    "AkRSIterator::ForceSelectRandomly": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
            ctypes.c_ulonglong,  # CAkRSNode *
        ]
    ),
    "CAkRSNode::GetInitialLoopCountLoop": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRSNode *
        ]
    ),
    "AkArray<ushort,ushort,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<ushort>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<unsigned short,unsigned short,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned short> > *
        ]
    ),
    "AkArray<RSStackItem,RSStackItem const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<RSStackItem>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkKeyDataPtrStruct<AkCachePinnedFileSetKey,AkCachePinnedFileSet,AkArrayAllocatorNoAlign<0> >,AkKeyDataPtrStruct<AkCachePinnedFileSetKey,AkCachePinnedFileSet,AkArrayAllocatorNoAlign<0> > const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkKeyDataPtrStruct<AkCachePinnedFileSetKey,AkCachePinnedFileSet,AkArrayAllocatorNoAlign<0> > > > *
        ]
    ),
    "AkRSIterator::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
            ctypes.c_ulonglong,  # CAkMusicRanSeqCntr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const CAkMusicPlaybackHistory *
        ]
    ),
    "AkRSIterator::JumpNext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
        ]
    ),
    "AkRSIterator::JumpTo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const CAkMusicPlaybackHistory *
        ]
    ),
    "AkRSIterator::MoveToNextSegment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
        ]
    ),
    "AkRSIterator::NotifySegmentPlay": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicPackedHistory *
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
            ctypes.c_ulonglong,  # CAkMusicPackedHistory *
        ]
    ),
    "AkRSIterator::PackIterator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicPackedHistory *
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
            ctypes.c_ulonglong,  # CAkMusicPackedHistory *
        ]
    ),
    "AkRSIterator::PopObsoleteStackedItems": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkRSNode *
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
            ctypes.c_ulonglong,  # CAkRSNode *
        ]
    ),
    "AkRSIterator::Select": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
            ctypes.c_ulonglong,  # RSStackItem *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AkRSIterator::SelectRandomly": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
            ctypes.c_ulonglong,  # RSStackItem *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AkRSIterator::SerializeIterator": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
            ctypes.c_ulonglong,  # AK::IWriteBytes *
        ]
    ),
    "CAkKeyArray<uint,AkRSIterator::GlobalRSInfoItem,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<uint,AkRSIterator::GlobalRSInfoItem>>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRSIterator::GlobalRSInfoItem *
        argtypes=[
            ctypes.c_ulonglong,  # CAkKeyArray<unsigned int,AkRSIterator::GlobalRSInfoItem,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<unsigned int,AkRSIterator::GlobalRSInfoItem> > > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkRSIterator::GlobalRSInfoItem *
        ]
    ),
    "AkRSIterator::SkipToNextSegment": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
        ]
    ),
    "AkRSIterator::StackItem": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
            ctypes.c_ulonglong,  # CAkRSSub *
        ]
    ),
    "AkRSIterator::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
        ]
    ),
    "AkRSIterator::UnpackGlobalRSInfo": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
            ctypes.c_ulonglong,  # const CAkMusicPackedHistory *
        ]
    ),
    "AkRSIterator::UnpackIterator": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
            ctypes.c_ulonglong,  # const CAkMusicPackedHistory *
        ]
    ),
    "AkRSIterator::UpdateRandomItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRSIterator *
            ctypes.c_ulonglong,  # CAkRSSub *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # AkArray<CAkRSNode *,CAkRSNode *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkRSNode *> > *
            ctypes.c_ulonglong,  # CAkRandomInfo *
        ]
    ),
    "CAkRSSub::WasSegmentLeafFound": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRSSub *
        ]
    ),
    "CAkSequenceCtx::CAkSequenceCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequenceCtx *
            ctypes.c_ulonglong,  # CAkMusicRanSeqCntr *
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkSequenceCtx::AppendItem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkScheduledItem *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequenceCtx *
            ctypes.c_ulonglong,  # const AkMusicTransitionRule *
            ctypes.c_ulonglong,  # CAkScheduledItem *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSequenceCtx::EndInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequenceCtx *
        ]
    ),
    "CAkSequenceCtx::Grow": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequenceCtx *
        ]
    ),
    "CAkSequenceCtx::HandleFatalError": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicSwitchCntr *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequenceCtx *
        ]
    ),
    "CAkSequenceCtx::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequenceCtx *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSequenceCtx::IsValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequenceCtx *
        ]
    ),
    "CAkSequenceCtx::NotifyItemSync": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequenceCtx *
            ctypes.c_ulonglong,  # CAkScheduledItem *
        ]
    ),
    "CAkSequenceCtx::OnLastFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequenceCtx *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSequenceCtx::PlaylistJump": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequenceCtx *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkMusicTransDestRule *
            ctypes.c_ulonglong,  # const CAkMusicPlaybackHistory *
        ]
    ),
    "CAkSequenceCtx::ScheduleNextSegment": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkScheduledItem *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequenceCtx *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkSequenceCtx::UpdateChainLength": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequenceCtx *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkMusicSwitchCtx::CAkMusicSwitchCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
            ctypes.c_ulonglong,  # CAkMusicSwitchCntr *
            ctypes.c_ulonglong,  # CAkMusicCtx *
        ]
    ),
    "CAkMusicSwitchCtx::CanRestartPlaying": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
        ]
    ),
    "CAkMusicSwitchCtx::CancelPlayback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkMusicSwitchCtx::CreateDestinationContext": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkParameterNodeBase *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicSwitchCtx::CreateMusicContext": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkParameterNodeBase *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicSwitchCtx::DequeueCancellableTransitions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
            ctypes.c_ulonglong,  # CAkMusicSwitchTransition **
            ctypes.c_ulonglong,  # AkListBare<CAkMusicSwitchTransition,AkListBareNextItem,AkCountPolicyNoCount,AkLastPolicyWithLast> *
            ctypes.c_ulonglong,  # CAkScheduleWindow *
        ]
    ),
    "CAkMusicSwitchCtx::EndInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
        ]
    ),
    "CAkMusicSwitchCtx::GetNextScheduleWindow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
            ctypes.c_ulonglong,  # CAkScheduleWindow *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicSwitchCtx::GetPlaybackHistory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicPlaybackHistory *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkArray<CAkRTPCMgr::RTPCCurve,CAkRTPCMgr::RTPCCurve const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkRTPCMgr::RTPCCurve>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<MapStruct<unsigned int,AkDataReference>,MapStruct<unsigned int,AkDataReference> const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<unsigned int,AkDataReference> > > *
        ]
    ),
    "CAkMusicSwitchCtx::HasOrAscendentHasPendingTransition": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
        ]
    ),
    "CAkMusicSwitchCtx::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicSwitchCtx::OnPlayed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
        ]
    ),
    "CAkMusicSwitchCtx::OnStopped": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
        ]
    ),
    "CAkNothingCtx::OnStopped": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkNothingCtx *
        ]
    ),
    "CAkMusicSwitchCtx::PerformDelayedSwitchChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
        ]
    ),
    "CAkMusicSwitchCtx::Prepare": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
            ctypes.c_ulonglong,  # const AkMusicTransDestRule *
            ctypes.c_ulonglong,  # const CAkMusicPlaybackHistory *
            ctypes.c_ulonglong,  # AkMusicSeekInfo *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicSwitchCtx::Process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkCutoffInfo *
        ]
    ),
    "CAkMusicSwitchCtx::QueryLookAheadInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
            ctypes.c_ulonglong,  # __int64 *
            ctypes.c_ulonglong,  # __int64 *
        ]
    ),
    "CAkMusicSwitchCtx::RefreshWindow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
            ctypes.c_ulonglong,  # CAkScheduleWindow *
        ]
    ),
    "CAkMusicSwitchCtx::ResolveAudioNode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
        ]
    ),
    "CAkMusicSwitchCtx::ScheduleSwitchTransition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const CAkMusicSwitchCtx::TransitionInfo *
            ctypes.c_ulonglong,  # CAkMatrixAwareCtx **
        ]
    ),
    "CAkMusicSwitchCtx::ScheduleTransition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # CAkMusicSwitchTransition *
            ctypes.c_ulonglong,  # const AkMusicTransSrcRule *
            ctypes.c_ulonglong,  # const AkMusicTransDestRule *
            ctypes.c_ulonglong,  # CAkScheduleWindow *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # AkListBareLight<AkAssociatedAction,AkListBareLightNextItem> *
        ]
    ),
    "CAkMusicSwitchCtx::SeekPercent": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicSwitchCtx::SeekTimeAbsolute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkKeyArray<uint,CAkMusicPlaybackHistory,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<uint,CAkMusicPlaybackHistory>>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicPlaybackHistory *
        argtypes=[
            ctypes.c_ulonglong,  # CAkKeyArray<unsigned int,CAkMusicPlaybackHistory,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<unsigned int,CAkMusicPlaybackHistory> > > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMusicSwitchCtx::SetInitialSwitch": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMusicSwitchCtx::SetPlaybackHistory": {
        "CAkMusicSwitchCtx *, unsigned int, const CAkMusicPackedHistory *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # const CAkMusicPackedHistory *
            ]
        ),
        "CAkMusicSwitchCtx *, unsigned int, int, unsigned int": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "CAkMusicSwitchMonitor::SetSwitch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchMonitor *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "AkArray<CAkMusicSwitchMonitor,CAkMusicSwitchMonitor const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkMusicSwitchMonitor>>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<CAkMusicSwitchMonitor,CAkMusicSwitchMonitor const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkMusicSwitchMonitor> > *
        ]
    ),
    "CAkMusicSwitchCtx::TryPropagateDelayedSwitchChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicSwitchCtx *
        ]
    ),
    "CAkMusicPlaybackHistory::~CAkMusicPlaybackHistory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMusicPlaybackHistory *
        ]
    ),
    "CreateAkParametricEQFX": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkParametricEQFX::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParametricEQFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkParametricEQFX::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParametricEQFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkEffectPluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkParametricEQFX::Reset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParametricEQFX *
        ]
    ),
    "CAkParametricEQFX::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParametricEQFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkParameterEQFXParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterEQFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkParametricEQFXParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkParameterEQFXParams::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterEQFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParameterEQFXParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterEQFXParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParameterEQFXParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterEQFXParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CreateAkPeakLimiterFX": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkPeakLimiterFX::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPeakLimiterFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkPeakLimiterFX::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPeakLimiterFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkEffectPluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkPeakLimiterFX::InitDelayLine": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPeakLimiterFX *
        ]
    ),
    "CAkPeakLimiterFX::Process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPeakLimiterFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkPeakLimiterFX::ProcessLinked": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPeakLimiterFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkPeakLimiterFX::ProcessLinkedNoLFE": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPeakLimiterFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkPeakLimiterFX::Reset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPeakLimiterFX *
        ]
    ),
    "CAkPeakLimiterFX::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPeakLimiterFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkPeakLimiterFXParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # CAkPeakLimiterFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkPeakLimiterFXParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkPeakLimiterFXParams::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPeakLimiterFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPeakLimiterFXParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPeakLimiterFXParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPeakLimiterFXParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPeakLimiterFXParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CreateAkPitchShifterFX": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkPitchShifterFX::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPitchShifterFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkPitchShifterFX::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPitchShifterFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkEffectPluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkPitchShifterFX::InitDryDelay": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPitchShifterFX *
        ]
    ),
    "CAkPitchShifterFX::InitPitchVoice": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPitchShifterFX *
        ]
    ),
    "CAkPitchShifterFX::Reset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPitchShifterFX *
        ]
    ),
    "CAkPitchShifterFX::ResetDryDelay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPitchShifterFX *
        ]
    ),
    "CAkPitchShifterFX::ResetPitchVoice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPitchShifterFX *
        ]
    ),
    "CAkPitchShifterFX::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPitchShifterFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkPitchShifterFX::TermDryDelay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPitchShifterFX *
        ]
    ),
    "CAkPitchShifterFX::TermPitchVoice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPitchShifterFX *
        ]
    ),
    "CAkPitchShifterFXParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # CAkPitchShifterFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkPitchShifterFXParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkPitchShifterFXParams::GetParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPitchShifterFXParams *
            ctypes.c_ulonglong,  # AkPitchShifterFXParams *
        ]
    ),
    "CAkPitchShifterFXParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPitchShifterFXParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPitchShifterFXParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPitchShifterFXParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::DSP::AkDelayPitchShift::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::DSP::AkDelayPitchShift *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::DSP::AkDelayPitchShift::ProcessChannel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::DSP::AkDelayPitchShift *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::DSP::AkDelayPitchShift::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::DSP::AkDelayPitchShift *
        ]
    ),
    "AK::DSP::AkDelayPitchShift::SetPitchFactor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::DSP::AkDelayPitchShift *
            ctypes.c_float,  # float
        ]
    ),
    "AK::DSP::AkDelayPitchShift::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::DSP::AkDelayPitchShift *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "AkPitchShifterDSPProcess": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkPitchShifterFXInfo *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "DSP::SIMD::ProcessBufferMono_<DSP::SIMD::InPlaceBuffer,DSP::SIMD::BiquadMonoCoefs,DSP::SIMD::BiquadMonoMemories>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::SIMD::InPlaceBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadMonoCoefs *
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadMonoMemories *
        ]
    ),
    "DelayLengths__AkDelayQSortCompareFloat": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "DSP::SIMD::BiquadMonoCoefs::ComputeCoefs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadMonoCoefs *
            ctypes.c_uint32,  # DSP::FilterType
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
        ]
    ),
    "DelayLengths__ComputeDiffusionFiltersDelayTimes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AlgorithmTunings *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "DelayLengths__ComputeFDNDelayTimes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AlgorithmTunings *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "DelayLengths__ComputeFDNInputDelayTimes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AlgorithmTunings *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "DelayLengths__ComputePrimeDelayLengths": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkRoomVerbFX::ComputeTCCoefs1": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
        ]
    ),
    "CAkRoomVerbFX::ComputeTCCoefs2": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
        ]
    ),
    "CAkRoomVerbFX::ComputeTCCoefs3": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
        ]
    ),
    "CreateAkRoomVerbFX": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkRoomVerbFX::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "AkAudioBuffer::GetLFE": FUNCDEF(
        restype=ctypes.c_ulonglong,  # float *
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkRoomVerbFX::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkEffectPluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkRoomVerbFX::LiveParametersUpdate": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkRoomVerbFX::ProcessAmbisonics1stOrder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkRoomVerbFX::ProcessHOA": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkRoomVerbFX::ProcessSpread1Out": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkRoomVerbFX::ProcessSpread2Out": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkRoomVerbFX::ProcessSpread3Out": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkRoomVerbFX::ProcessSpread4Out": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkRoomVerbFX::RTPCParametersUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
        ]
    ),
    "CAkRoomVerbFX::Reset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
        ]
    ),
    "CAkRoomVerbFX::ResetFDNs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
        ]
    ),
    "CAkRoomVerbFX::ResetToneControlFilters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
        ]
    ),
    "CAkRoomVerbFX::ReverbPreProcess": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "SetupDecorrelationAPF": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # DSP::AllpassFilter **
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkRoomVerbFX::SetupFDNs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkRoomVerbFX::SetupReverbDelay": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkRoomVerbFX::SetupToneControlFilters": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
        ]
    ),
    "CAkRoomVerbFX::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkRoomVerbFX::TermERUnit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkRoomVerbFX::TermFDNs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkRoomVerbFX::WetPreProcess": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkRoomVerbFXParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkRoomVerbFXParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkRoomVerbFXParams::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkRoomVerbFXParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFXParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkRoomVerbFXParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRoomVerbFXParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::ERUnitDual::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::ERUnitDual *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const AkRoomVerb::TapInfo *
            ctypes.c_ulonglong,  # const AkRoomVerb::TapInfo *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::ERUnitDual::ProcessBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::ERUnitDual *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::ERUnitDual::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::ERUnitDual *
        ]
    ),
    "DSP::AllpassFilter::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::AllpassFilter *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "DSP::AllpassFilter::ProcessBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::AllpassFilter *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::AllpassFilter::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::AllpassFilter *
        ]
    ),
    "DSP::DelayLine::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::DelayLine *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::DelayLine::ProcessBuffer": {
        "DSP::DelayLine *, float *, float *, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # DSP::DelayLine *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "DSP::DelayLine *, float *, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # DSP::DelayLine *
                ctypes.c_ulonglong,  # float *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "DSP::DelayLine::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::DelayLine *
        ]
    ),
    "DSP::FDN4::ChangeDecay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::FDN4 *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "FDNLPFilter::ComputeFDNLPCoefs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # FDNLPFilter *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::FDN4::ComputeMaxStableHFRatio": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::FDN4::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # DSP::FDN4 *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::FDN4::ProcessBufferAccum": {
        "DSP::FDN4 *, float *, float *, float *, float *, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # DSP::FDN4 *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "DSP::FDN4 *, float *, float *, float *, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # DSP::FDN4 *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "DSP::FDN4 *, float *, float *, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # DSP::FDN4 *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # float *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "DSP::FDN4::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::FDN4 *
        ]
    ),
    "DSP::FDN4::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::FDN4 *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "DSP::ConstantPowerChannelMixdown": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "DSP::Mix3Interp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkFxSrcSilenceParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxSrcSilenceParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkSilenceSource": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkSilenceSourceParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkFXSrcSilence::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFXSrcSilence *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFXSrcSilence::GetDuration": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkFXSrcSilence *
        ]
    ),
    "CAkFXSrcSilence::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFXSrcSilence *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkSourcePluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkFxSrcSilenceParams::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxSrcSilenceParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkFXSrcSilence::Reset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFXSrcSilence *
        ]
    ),
    "CAkFXSrcSilence::Seek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFXSrcSilence *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkFxSrcSilenceParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxSrcSilenceParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkFxSrcSilenceParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxSrcSilenceParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkFXSrcSilence::StopLooping": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFXSrcSilence *
        ]
    ),
    "CAkFXSrcSilence::TimeSkip": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFXSrcSilence *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CreateSineSource": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateSineSourceParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkFXSrcSine::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFXSrcSine *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkFXSrcSine::GetDuration": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkFXSrcSine *
        ]
    ),
    "CAkFXSrcSine::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFXSrcSine *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkSourcePluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkFXSrcSine::Reset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFXSrcSine *
        ]
    ),
    "CAkFXSrcSine::StopLooping": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFXSrcSine *
        ]
    ),
    "CAkFXSrcSine::TimeSkip": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFXSrcSine *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkFxSrcSineParams::CAkFxSrcSineParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxSrcSineParams *
        ]
    ),
    "CAkFxSrcSineParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxSrcSineParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkFxSrcSineParams::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxSrcSineParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkFxSrcSineParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxSrcSineParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkFxSrcSineParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxSrcSineParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::DSP::AkSineOsc::Process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::DSP::AkSineOsc *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_double,  # double
            ctypes.c_float,  # float
        ]
    ),
    "AK__SoundEngine___SetMultiplePositions_AkChannelEmitter_": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const AkChannelEmitter *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint32,  # AK::SoundEngine::MultiPositionType
        ]
    ),
    "CAkAudioLibIndex::CAkAudioLibIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioLibIndex *
        ]
    ),
    "AK::SoundEngine::AddDefaultListener": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::SoundEngine::AddExternalBankHandler": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # {'return': 'AKRESULT', 'args': ['const AkBank::AKBKSubHircSection *', 'CAkUsageSlot *', {'type': 'unsigned', 'name': 'int'}]}
        ]
    ),
    "AK::SoundEngine::AddExternalStateHandler": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': [{'type': 'unsigned', 'name': 'int'}, {'type': 'unsigned', 'name': 'int'}]}
        ]
    ),
    "AK::SoundEngine::AddExternalUnloadBankHandler": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['CAkUsageSlot *']}
        ]
    ),
    "AK::SoundEngine::AddListener": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::SoundEngine::AddOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const AkOutputSettings *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::AddOutputCaptureMarker": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "AK::SoundEngine::AddRemoveOrSetDefaultListeners": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkListenerOp
        ]
    ),
    "AK::SoundEngine::AddRemoveOrSetListeners": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkListenerOp
        ]
    ),
    "AKRANDOM::AkRandomInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AKPLATFORM::AkWideCharToChar": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "AK::SoundEngine::DynamicSequence::Break": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::CancelBankCallbackCookie": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AK::SoundEngine::CancelEventCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::CancelEventCallbackCookie": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AK::SoundEngine::CancelEventCallbackGameObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::SoundEngine::ClearBanks": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "AK::SoundEngine::ClearPreparedEvents": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "AK::SoundEngine::DynamicSequence::Close": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::ComputeSphericalCoordinates": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
            ctypes.c_ulonglong,  # const AkEmitterListenerPair *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "AkExternalSourceArray::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkExternalSourceArray *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkExternalSourceInfo *
        ]
    ),
    "AK::SoundEngine::DecodeBank": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AK::SoundEngine::DefaultBankCallbackFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # AKRESULT
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AK::SoundEngine::ExecuteActionOnEvent": {
        "unsigned int, AK::SoundEngine::AkActionOnEventType, unsigned __int64, int, AkCurveInterpolation, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # AK::SoundEngine::AkActionOnEventType
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # AkCurveInterpolation
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "const char *, AK::SoundEngine::AkActionOnEventType, unsigned __int64, int, AkCurveInterpolation, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_uint32,  # AK::SoundEngine::AkActionOnEventType
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # AkCurveInterpolation
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "const wchar_t *, AK::SoundEngine::AkActionOnEventType, unsigned __int64, int, AkCurveInterpolation, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_uint32,  # AK::SoundEngine::AkActionOnEventType
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # AkCurveInterpolation
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "AK::SoundEngine::ExecuteActionOnPlayingID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # AK::SoundEngine::AkActionOnEventType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::GetAcousticTexture": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkIndexable *
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::GetAllocator": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkFXMemAlloc *
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::GetAudioSettings": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
            ctypes.c_ulonglong,  # AkAudioSettings *
        ]
    ),
    "AK::SoundEngine::GetAudioSettings": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioSettings *
        ]
    ),
    "AK::SoundEngine::GetBackgroundMusicMute": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "AK__SoundEngine__GetBankIDFromString": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "AK__SoundEngine__GetBankIDFromString_0": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "AK::SoundEngine::GetBufferStatusForPinnedEvent": {
        "unsigned int, float *, bool *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # bool *
            ]
        ),
        "const char *, float *, bool *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # bool *
            ]
        ),
        "const wchar_t *, float *, bool *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # bool *
            ]
        ),
    },
    "AK::SoundEngine::CAkGlobalPluginContext::GetBufferTick": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "AK::SoundEngine::GetContainerHistory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::IWriteBytes *
        ]
    ),
    "AK::SoundEngine::GetDefaultInitSettings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkInitSettings *
        ]
    ),
    "AK::SoundEngine::GetDefaultPlatformInitSettings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkPlatformInitSettings *
        ]
    ),
    "AK::SoundEngine::GetDeviceList": {
        "unsigned int, unsigned int *, AkDeviceDescription *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # unsigned int *
                ctypes.c_ulonglong,  # AkDeviceDescription *
            ]
        ),
        "unsigned int, unsigned int, unsigned int *, AkDeviceDescription *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # unsigned int *
                ctypes.c_ulonglong,  # AkDeviceDescription *
            ]
        ),
    },
    "AK::SoundEngine::DynamicDialogue::GetDialogueEventCustomPropertyValue": {
        "unsigned int, unsigned int, int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # int *
            ]
        ),
        "unsigned int, unsigned int, float *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # float *
            ]
        ),
    },
    "AK::SoundEngine::GetGlobalPluginContext": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
        argtypes=[]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::GetIDFromString": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "AK::SoundEngine::GetIDFromString": {
        "const char *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
            ]
        ),
        "const wchar_t *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
            ]
        ),
    },
    "AK::SoundEngine::CAkGlobalPluginContext::GetInitSettings": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkInitSettings *
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
        ]
    ),
    "CAkBusFX::GetMaxBufferLength": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::GetNumOfSamplesPerFrame": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
        ]
    ),
    "AK::SoundEngine::GetOutputDeviceConfiguration": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # AkChannelConfig *
            ctypes.c_ulonglong,  # Ak3DAudioSinkCapabilities *
        ]
    ),
    "AK::SoundEngine::GetOutputID": {
        "unsigned int, unsigned int": FUNCDEF(
            restype=ctypes.c_uint64,  # unsigned __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "const char *, unsigned int": FUNCDEF(
            restype=ctypes.c_uint64,  # unsigned __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "const wchar_t *, unsigned int": FUNCDEF(
            restype=ctypes.c_uint64,  # unsigned __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "AK::SoundEngine::GetPanningRule": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkPanningRule *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::SoundEngine::DynamicSequence::GetPauseTimes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::GetSampleRate": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::GetPlatformContext": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlatformContext *
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::GetPlatformInitSettings": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkPlatformInitSettings *
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
        ]
    ),
    "AK::SoundEngine::DynamicSequence::GetPlayingItem": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::GetPluginService": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::CAkPluginServiceMixer *
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
            ctypes.c_uint32,  # AK::AkPluginServiceType
        ]
    ),
    "AK::SoundEngine::GetSampleTick": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[]
    ),
    "AK__SoundEngine__DynamicSequence__GetSequence": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkIndexable *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::GetSourcePlayPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # int *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::SoundEngine::GetSourcePlayPositions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkSourcePosition *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::SoundEngine::GetSourceStreamBuffering": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AK::SoundEngine::GetSpeakerAngles": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::SoundEngine::GetSpeakerConfiguration": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkChannelConfig *
        argtypes=[
            ctypes.c_ulonglong,  # AkChannelConfig *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CAkEffectContextBase::GetStreamMgr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkStreamMgr *
        argtypes=[
            ctypes.c_ulonglong,  # CAkEffectContextBase *
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::GetmsPerBufferTick": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
        ]
    ),
    "AK::SoundEngine::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkInitSettings *
            ctypes.c_ulonglong,  # AkPlatformInitSettings *
        ]
    ),
    "AK::SoundEngine::IsInitialized": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::IsRenderingOffline": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # CAkEffectContextBase *
        ]
    ),
    "AK::SoundEngine::LoadBank": {
        "unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "unsigned int, __int64 *, void *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
                ctypes.c_ulonglong,  # void *
            ]
        ),
        "const char *, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
        "const char *, __int64 *, void *, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
                ctypes.c_ulonglong,  # void *
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
        "const wchar_t *, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
        "const wchar_t *, __int64 *, void *, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
                ctypes.c_ulonglong,  # void *
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
    },
    "AK::SoundEngine::LoadBankMemoryCopy": {
        "const void *, unsigned int, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const void *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
        "const void *, unsigned int, __int64 *, void *, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const void *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
                ctypes.c_ulonglong,  # void *
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
    },
    "AK::SoundEngine::LoadBankMemoryView": {
        "const void *, unsigned int, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const void *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
        "const void *, unsigned int, __int64 *, void *, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const void *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
                ctypes.c_ulonglong,  # void *
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
    },
    "AK::SoundEngine::DynamicSequence::LockPlaylist": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkIndexable *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::MuteBackgroundMusic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # cGcShipAIHailData *
        ]
    ),
    "AK::SoundEngine::DynamicSequence::Open": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkIndexable *
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkCallbackType', 'AkCallbackInfo *']}
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # AK::SoundEngine::DynamicSequence::DynamicSequenceType
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::PassSampleData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkCaptureFile *
        ]
    ),
    "AK::SoundEngine::DynamicSequence::Pause": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
        ]
    ),
    "AK::SoundEngine::PinEventInStreamCache": {
        "unsigned int, char, char": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_char,  # char
                ctypes.c_char,  # char
            ]
        ),
        "const char *, char, char": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_char,  # char
                ctypes.c_char,  # char
            ]
        ),
        "const wchar_t *, char, char": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_char,  # char
                ctypes.c_char,  # char
            ]
        ),
    },
    "AK::SoundEngine::DynamicSequence::Play": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
        ]
    ),
    "AK::SoundEngine::PlaySourcePlugin": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::SoundEngine::PostEvent": {
        "unsigned int, unsigned __int64, unsigned int, __int64 *, void *, unsigned int, AkExternalSourceInfo *, unsigned int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # AkExternalSourceArray *
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkCallbackType', 'AkCallbackInfo *']}
                ctypes.c_ulonglong,  # void *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # AkExternalSourceInfo *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "unsigned int, unsigned __int64, unsigned int, __int64 *, void *, AkCustomParamType *, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkCallbackType', 'AkCallbackInfo *']}
                ctypes.c_ulonglong,  # void *
                ctypes.c_ulonglong,  # AkCustomParamType *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "const char *, unsigned __int64, unsigned int, __int64 *, void *, unsigned int, AkExternalSourceInfo *, unsigned int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # AkExternalSourceArray *
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkCallbackType', 'AkCallbackInfo *']}
                ctypes.c_ulonglong,  # void *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # AkExternalSourceInfo *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "const wchar_t *, unsigned __int64, unsigned int, __int64 *, void *, unsigned int, AkExternalSourceInfo *, unsigned int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # AkExternalSourceArray *
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkCallbackType', 'AkCallbackInfo *']}
                ctypes.c_ulonglong,  # void *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # AkExternalSourceInfo *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "AK::SoundEngine::CAkGlobalPluginContext::PostEventSync": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkCallbackType', 'AkCallbackInfo *']}
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkExternalSourceInfo *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::PostMIDIOnEvent": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # AkMIDIPost *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkCallbackType', 'AkCallbackInfo *']}
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::PostMIDIOnEventSync": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # AkMIDIPost *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkCallbackType', 'AkCallbackInfo *']}
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::PostTrigger": {
        "StopParticles *, unsigned __int64": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # StopParticles *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
        "const char *, unsigned __int64": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
        "const wchar_t *, unsigned __int64": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
    },
    "AK__SoundEngine__PreInit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkInitSettings *
        ]
    ),
    "AK::SoundEngine::PrepareBank": {
        "AK::SoundEngine::PreparationType, unsigned int, __int64 *, void *, AK::SoundEngine::AkBankContent": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # AK::SoundEngine::PreparationType
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
                ctypes.c_ulonglong,  # void *
                ctypes.c_uint32,  # AK::SoundEngine::AkBankContent
            ]
        ),
        "AK::SoundEngine::PreparationType, unsigned int, AK::SoundEngine::AkBankContent": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # AK::SoundEngine::PreparationType
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # AK::SoundEngine::AkBankContent
            ]
        ),
        "AK::SoundEngine::PreparationType, const char *, __int64 *, void *, AK::SoundEngine::AkBankContent": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # AK::SoundEngine::PreparationType
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
                ctypes.c_ulonglong,  # void *
                ctypes.c_uint32,  # AK::SoundEngine::AkBankContent
            ]
        ),
        "AK::SoundEngine::PreparationType, const char *, AK::SoundEngine::AkBankContent": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # AK::SoundEngine::PreparationType
                ctypes.c_char_p,  # const char *
                ctypes.c_uint32,  # AK::SoundEngine::AkBankContent
            ]
        ),
        "AK::SoundEngine::PreparationType, const wchar_t *, __int64 *, void *, AK::SoundEngine::AkBankContent": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # AK::SoundEngine::PreparationType
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
                ctypes.c_ulonglong,  # void *
                ctypes.c_uint32,  # AK::SoundEngine::AkBankContent
            ]
        ),
        "AK::SoundEngine::PreparationType, const wchar_t *, AK::SoundEngine::AkBankContent": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # AK::SoundEngine::PreparationType
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_uint32,  # AK::SoundEngine::AkBankContent
            ]
        ),
    },
    "AK::SoundEngine::PrepareEvent": {
        "AK::SoundEngine::PreparationType, unsigned int *, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # AK::SoundEngine::PreparationType
                ctypes.c_ulonglong,  # unsigned int *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "AK::SoundEngine::PreparationType, unsigned int *, unsigned int, __int64 *, void *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # AK::SoundEngine::PreparationType
                ctypes.c_ulonglong,  # unsigned int *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
                ctypes.c_ulonglong,  # void *
            ]
        ),
        "AK::SoundEngine::PreparationType, const char **, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # AK::SoundEngine::PreparationType
                ctypes.c_char_p,  # const char **
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "AK::SoundEngine::PreparationType, const char **, unsigned int, __int64 *, void *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # AK::SoundEngine::PreparationType
                ctypes.c_char_p,  # const char **
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
                ctypes.c_ulonglong,  # void *
            ]
        ),
        "AK::SoundEngine::PreparationType, const wchar_t **, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # AK::SoundEngine::PreparationType
                ctypes.c_ulonglong,  # const wchar_t **
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "AK::SoundEngine::PreparationType, const wchar_t **, unsigned int, __int64 *, void *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # AK::SoundEngine::PreparationType
                ctypes.c_ulonglong,  # const wchar_t **
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
                ctypes.c_ulonglong,  # void *
            ]
        ),
    },
    "AK__SoundEngine__PrepareEventInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # AK::SoundEngine::PreparationType
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::SoundEngine::PrepareGameSyncs": {
        "AK::SoundEngine::PreparationType, AkGroupType, unsigned int, unsigned int *, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # AK::SoundEngine::PreparationType
                ctypes.c_uint32,  # AkGroupType
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # unsigned int *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "AK::SoundEngine::PreparationType, AkGroupType, unsigned int, unsigned int *, unsigned int, __int64 *, void *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # AK::SoundEngine::PreparationType
                ctypes.c_uint32,  # AkGroupType
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # unsigned int *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
                ctypes.c_ulonglong,  # void *
            ]
        ),
        "AK::SoundEngine::PreparationType, AkGroupType, const char *, const char **, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # AK::SoundEngine::PreparationType
                ctypes.c_uint32,  # AkGroupType
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char **
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "AK::SoundEngine::PreparationType, AkGroupType, const char *, const char **, unsigned int, __int64 *, void *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # AK::SoundEngine::PreparationType
                ctypes.c_uint32,  # AkGroupType
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char **
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
                ctypes.c_ulonglong,  # void *
            ]
        ),
        "AK::SoundEngine::PreparationType, AkGroupType, const wchar_t *, const wchar_t **, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # AK::SoundEngine::PreparationType
                ctypes.c_uint32,  # AkGroupType
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # const wchar_t **
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "AK::SoundEngine::PreparationType, AkGroupType, const wchar_t *, const wchar_t **, unsigned int, __int64 *, void *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # AK::SoundEngine::PreparationType
                ctypes.c_uint32,  # AkGroupType
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # const wchar_t **
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
                ctypes.c_ulonglong,  # void *
            ]
        ),
    },
    "AK__SoundEngine__PrepareGameSyncsInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # AkGroupType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::Random": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
        ]
    ),
    "AK::SoundEngine::RegisterAudioDeviceStatusCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AK::IAkGlobalPluginContext *', {'type': 'unsigned', 'name': 'int'}, {'type': 'unsigned', 'name': 'int'}, 'AK::AkAudioDeviceEvent', 'AKRESULT']}
        ]
    ),
    "AK::SoundEngine::RegisterBusMeteringCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkBusMeteringCallbackInfo *']}
            ctypes.c_uint32,  # AkMeteringFlags
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AK::SoundEngine::RegisterBusVolumeCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkSpeakerVolumeMatrixCallbackInfo *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AK::SoundEngine::RegisterCaptureCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkAudioBuffer *', {'type': 'unsigned', 'name': '__int64'}, 'void *']}
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::RegisterCodec": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # {'return': 'IAkSoftwareCodec *', 'args': ['void *']}
            ctypes.c_ulonglong,  # {'return': 'IAkSoftwareCodec *', 'args': ['void *']}
        ]
    ),
    "AK::SoundEngine::RegisterCodec": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # {'return': 'IAkSoftwareCodec *', 'args': ['void *']}
            ctypes.c_ulonglong,  # {'return': 'IAkSoftwareCodec *', 'args': ['void *']}
        ]
    ),
    "AK::SoundEngine::RegisterGameObj": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_char_p,  # const char *
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::RegisterGlobalCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
            ctypes.c_uint32,  # AkPluginType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AK::IAkGlobalPluginContext *', 'AkGlobalCallbackLocation', 'void *']}
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AK::SoundEngine::RegisterGlobalCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AK::IAkGlobalPluginContext *', 'AkGlobalCallbackLocation', 'void *']}
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # AkPluginType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::RegisterOutputDeviceMeteringCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkOutputDeviceMeteringCallbackInfo *']}
            ctypes.c_uint32,  # AkMeteringFlags
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::RegisterPlugin": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
            ctypes.c_uint32,  # AkPluginType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # {'return': 'AK::IAkPlugin *', 'args': ['AK::IAkPluginMemAlloc *']}
            ctypes.c_ulonglong,  # {'return': 'AK::IAkPluginParam *', 'args': ['AK::IAkPluginMemAlloc *']}
        ]
    ),
    "AK::SoundEngine::RegisterPlugin": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # AkPluginType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # {'return': 'AK::IAkPlugin *', 'args': ['AK::IAkPluginMemAlloc *']}
            ctypes.c_ulonglong,  # {'return': 'AK::IAkPluginParam *', 'args': ['AK::IAkPluginMemAlloc *']}
        ]
    ),
    "AkExternalSourceArray::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkExternalSourceArray *
        ]
    ),
    "AK::SoundEngine::RemoveDefaultListener": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::SoundEngine::RemoveListener": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::SoundEngine::RemoveOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::SoundEngine::RenderAudio": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::SoundEngine::ReplaceOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const AkOutputSettings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "AK::SoundEngine::ResetListenersToDefault": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::SoundEngine::ResetRTPCValue": {
        "SpawnParticles *, unsigned __int64, int, AkCurveInterpolation, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # SpawnParticles *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # AkCurveInterpolation
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const char *, unsigned __int64, int, AkCurveInterpolation, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # AkCurveInterpolation
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const wchar_t *, unsigned __int64, int, AkCurveInterpolation, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # AkCurveInterpolation
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "AK::SoundEngine::DynamicDialogue::ResolveDialogueEvent": {
        "unsigned int, unsigned int *, unsigned int, unsigned int, __int64 *, void *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # unsigned int *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # {'return': 'bool', 'args': [{'type': 'unsigned', 'name': 'int'}, {'type': 'unsigned', 'name': 'int'}, 'void *']}
                ctypes.c_ulonglong,  # void *
            ]
        ),
        "const char *, const char **, unsigned int, unsigned int, __int64 *, void *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char **
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # {'return': 'bool', 'args': [{'type': 'unsigned', 'name': 'int'}, {'type': 'unsigned', 'name': 'int'}, 'void *']}
                ctypes.c_ulonglong,  # void *
            ]
        ),
        "const wchar_t *, const wchar_t **, unsigned int, unsigned int, __int64 *, void *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # const wchar_t **
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # {'return': 'bool', 'args': [{'type': 'unsigned', 'name': 'int'}, {'type': 'unsigned', 'name': 'int'}, 'void *']}
                ctypes.c_ulonglong,  # void *
            ]
        ),
    },
    "AK::SoundEngine::DynamicSequence::Resume": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
        ]
    ),
    "AK::SoundEngine::DynamicSequence::Seek": {
        "unsigned int, int, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_int32,  # int
                ctypes.c_ubyte,  # bool
            ]
        ),
        "unsigned int, float, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "AK::SoundEngine::SeekOnEvent": {
        "unsigned int, unsigned __int64, int, bool, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_int32,  # int
                ctypes.c_ubyte,  # bool
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "unsigned int, unsigned __int64, float, bool, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "const char *, unsigned __int64, int, bool, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_int32,  # int
                ctypes.c_ubyte,  # bool
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "const char *, unsigned __int64, float, bool, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "const wchar_t *, unsigned __int64, int, bool, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_int32,  # int
                ctypes.c_ubyte,  # bool
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "const wchar_t *, unsigned __int64, float, bool, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_float,  # float
                ctypes.c_ubyte,  # bool
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "AK::SoundEngine::CAkGlobalPluginContext::SendPluginCustomGameData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # AkPluginType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::SendPluginCustomGameData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SetVariableValue *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # AkPluginType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::SetActorMixerEffect": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::SetBankLoadIOSettings": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_char,  # char
        ]
    ),
    "AK::SoundEngine::SetBusDevice": {
        "unsigned int, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "const char *, const char *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "const wchar_t *, const wchar_t *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # const wchar_t *
            ]
        ),
    },
    "AK::SoundEngine::SetBusEffect": {
        "unsigned int, unsigned int, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "const char *, unsigned int, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "const wchar_t *, unsigned int, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "AK::SoundEngine::SetContainerHistory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::IReadBytes *
        ]
    ),
    "AK::SoundEngine::SetCustomPlatformName": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "AK::SoundEngine::SetDefaultListeners": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::SetGameObjectAuxSendValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # AkAuxSendValue *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::SetGameObjectOutputBusVolume": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
        ]
    ),
    "AK::SoundEngine::SetListeners": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::SetMaxNumDangerousVirtVoicesLimitInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint32,  # AK::SoundEngine::AkCommandPriority
        ]
    ),
    "AK::SoundEngine::SetMaxNumVoicesLimit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "AK::SoundEngine::SetMaxNumVoicesLimitInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint32,  # AK::SoundEngine::AkCommandPriority
        ]
    ),
    "AK::SoundEngine::SetMedia": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkSourceSettings *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::SetMixer": {
        "unsigned int, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "const char *, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "const wchar_t *, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "AK::SoundEngine::SetMultipleObstructionAndOcclusion": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # AkObstructionOcclusionValues *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::SetMultiplePositions": {
        "unsigned __int64, const AkChannelEmitter *, unsigned __int16, AK::SoundEngine::MultiPositionType": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # const AkChannelEmitter *
                ctypes.c_uint16,  # unsigned __int16
                ctypes.c_uint32,  # AK::SoundEngine::MultiPositionType
            ]
        ),
        "unsigned __int64, const AkTransform *, unsigned __int16, AK::SoundEngine::MultiPositionType": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # const AkTransform *
                ctypes.c_uint16,  # unsigned __int16
                ctypes.c_uint32,  # AK::SoundEngine::MultiPositionType
            ]
        ),
    },
    "AK::SoundEngine::SetObjectObstructionAndOcclusion": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "AK::SoundEngine::SetOfflineRendering": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SetHazard *
        ]
    ),
    "AK::SoundEngine::SetOfflineRenderingFrameTime": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "AK::SoundEngine::SetOutputDeviceEffect": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::SetOutputVolume": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
        ]
    ),
    "AK::SoundEngine::SetPanningRule": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # StopParticles *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::SoundEngine::SetPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const AkTransform *
        ]
    ),
    "AK::SoundEngine::SetPositionInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const AkTransform *
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::SetRTPCValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::SoundEngine::SetRTPCValue": {
        "SurveyVariableValue *, float, unsigned __int64, int, AkCurveInterpolation, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # SurveyVariableValue *
                ctypes.c_float,  # float
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # AkCurveInterpolation
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const char *, float, unsigned __int64, int, AkCurveInterpolation, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_float,  # float
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # AkCurveInterpolation
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const wchar_t *, float, unsigned __int64, int, AkCurveInterpolation, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_float,  # float
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # AkCurveInterpolation
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "AK::SoundEngine::SetRTPCValueByPlayingID": {
        "unsigned int, float, unsigned int, int, AkCurveInterpolation, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_float,  # float
                ctypes.c_uint32,  # unsigned int
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # AkCurveInterpolation
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const char *, float, unsigned int, int, AkCurveInterpolation, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_float,  # float
                ctypes.c_uint32,  # unsigned int
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # AkCurveInterpolation
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const wchar_t *, float, unsigned int, int, AkCurveInterpolation, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_float,  # float
                ctypes.c_uint32,  # unsigned int
                ctypes.c_int32,  # int
                ctypes.c_uint32,  # AkCurveInterpolation
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "AK::SoundEngine::CAkGlobalPluginContext::SetRTPCValueSync": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::SoundEngine::SetRandomSeed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # RandomNumber *
        ]
    ),
    "AK::SoundEngine::SetScalingFactor": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
        ]
    ),
    "AK::SoundEngine::SetSpeakerAngles": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::SoundEngine::SetState": {
        "StopParticles *, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # StopParticles *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "StopParticles *, unsigned int, bool, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # StopParticles *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
        "const char *, const char *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "const wchar_t *, const wchar_t *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # const wchar_t *
            ]
        ),
    },
    "AK::SoundEngine::SetSwitch": {
        "unsigned int, unsigned int, unsigned __int64": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
        "const char *, const char *, unsigned __int64": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
        "const wchar_t *, const wchar_t *, unsigned __int64": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
    },
    "AK::SoundEngine::SetVolumeThreshold": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "AK::SoundEngine::SetVolumeThresholdInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_uint32,  # AK::SoundEngine::AkCommandPriority
        ]
    ),
    "AK::SoundEngine::StartOutputCapture": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "AK::SoundEngine::DynamicSequence::Stop": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
        ]
    ),
    "AK::SoundEngine::StopAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::StopCapture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
            ctypes.c_ulonglong,  # AkCaptureFile **
        ]
    ),
    "AK::SoundEngine::StopMIDIOnEvent": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::StopMIDIOnEventSync": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::StopOutputCapture": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SetHazard *
        ]
    ),
    "AK::SoundEngine::StopPlayingID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StopParticles *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
        ]
    ),
    "AK::SoundEngine::StopSourcePlugin": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::Suspend": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::SoundEngine::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine *
        ]
    ),
    "AK::SoundEngine::TryUnsetMedia": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkSourceSettings *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AKRESULT *
        ]
    ),
    "AK::SoundEngine::UnloadBank": {
        "unsigned int, const void *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # const void *
            ]
        ),
        "unsigned int, const void *, __int64 *, void *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # const void *
                ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
                ctypes.c_ulonglong,  # void *
            ]
        ),
        "const char *, const void *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # const void *
            ]
        ),
        "const char *, const void *, __int64 *, void *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # const void *
                ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
                ctypes.c_ulonglong,  # void *
            ]
        ),
        "const wchar_t *, const void *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # const void *
            ]
        ),
        "const wchar_t *, const void *, __int64 *, void *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # const void *
                ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
                ctypes.c_ulonglong,  # void *
            ]
        ),
    },
    "AK::SoundEngine::UnloadBankInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkFileNameString *
            ctypes.c_ulonglong,  # const void *
        ]
    ),
    "AK::SoundEngine::DynamicSequence::UnlockPlaylist": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::UnpinEventInStreamCache": {
        "unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "const char *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
            ]
        ),
        "const wchar_t *": FUNCDEF(
            restype=ctypes.c_uint32,  # AKRESULT
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
            ]
        ),
    },
    "AK::SoundEngine::UnregisterAllGameObj": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SetHazard *
        ]
    ),
    "AK::SoundEngine::UnregisterAudioDeviceStatusCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "AK::SoundEngine::UnregisterCaptureCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkAudioBuffer *', {'type': 'unsigned', 'name': '__int64'}, 'void *']}
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AK::SoundEngine::UnregisterGameObj": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::SoundEngine::CAkGlobalPluginContext::UnregisterGlobalCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SoundEngine::CAkGlobalPluginContext *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AK::IAkGlobalPluginContext *', 'AkGlobalCallbackLocation', 'void *']}
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::UnregisterGlobalCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AK::IAkGlobalPluginContext *', 'AkGlobalCallbackLocation', 'void *']}
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::UnsetMedia": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkSourceSettings *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::WaitSoundEngineInit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "AK::SoundEngine::WakeupFromSuspend": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "AK::SoundEngine::_SetRTPCValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SurveyVariableValue *
            ctypes.c_float,  # float
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CreateSrcMediaNodePCM": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkEffectsMgr::Alloc": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AK::IAkPlugin **
            ctypes.c_ulonglong,  # AkPluginInfo *
        ]
    ),
    "CAkEffectsMgr::AllocCodecSrc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAkSoftwareCodec *
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkEffectsMgr::AllocFileCodec": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAkFileCodec *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkEffectsMgr::AllocGrainCodec": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAkGrainCodec *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkEffectsMgr::AllocParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # AK::IAkPluginParam **
        ]
    ),
    "CreateSrcMediaCodecPCM": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const AK::SrcMedia::Header *
        ]
    ),
    "CAkEffectsMgr::FreeFileCodec": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_ulonglong,  # AkDevice::BaseCaptureCallbackStruct *
        ]
    ),
    "CAkEffectsMgr::FreeGrainCodec": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # IAkGrainCodec *
        ]
    ),
    "CAkEffectsMgr::GetEffectTypeRecord": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkEffectsMgr::EffectTypeRecord *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkEffectsMgr::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # wchar_t *
        ]
    ),
    "CAkEffectsMgr::RegisterCodec": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkCodecDescriptor *
        ]
    ),
    "CAkEffectsMgr::RegisterPlugin": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # AkPluginType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # {'return': 'AK::IAkPlugin *', 'args': ['AK::IAkPluginMemAlloc *']}
            ctypes.c_ulonglong,  # {'return': 'AK::IAkPluginParam *', 'args': ['AK::IAkPluginMemAlloc *']}
            ctypes.c_ulonglong,  # {'return': 'AKRESULT', 'args': ['unsigned int *', 'AkDeviceDescription *']}
        ]
    ),
    "CAkEffectsMgr::RegisterPluginList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::PluginRegistration *
        ]
    ),
    "CAkEffectsMgr::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "CAkEffectsMgr::ValidatePluginInfo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkPluginType
            ctypes.c_ulonglong,  # const AkPluginInfo *
        ]
    ),
    "AK::SoundEngine::Query::GetActiveGameObjects": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<unsigned __int64,unsigned __int64,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64> > *
        ]
    ),
    "CAkConnectedListeners::GetAuxAssocs": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkLstnrAssocs *
        argtypes=[
            ctypes.c_ulonglong,  # CAkConnectedListeners *
        ]
    ),
    "AK::SoundEngine::Query::GetCustomPropertyValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkConnectedListeners::GetDefault": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkConnectedListeners *
        argtypes=[]
    ),
    "AK::SoundEngine::Query::GetEventIDFromPlayingID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::Query::GetGameObjectAuxSendValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # AkAuxSendValue *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AK::SoundEngine::Query::GetGameObjectDryLevelValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "AK::SoundEngine::Query::GetGameObjectFromPlayingID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SoundEngine::Query::GetIsGameObjectActive": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::SoundEngine::Query::GetListenerPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # AkTransform *
        ]
    ),
    "AK::SoundEngine::Query::GetListenerSpatialization": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # float **
            ctypes.c_ulonglong,  # AkChannelConfig *
        ]
    ),
    "AK::SoundEngine::Query::GetListeners": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AK::SoundEngine::Query::GetMaxRadius": {
        "AkArray<AK::SoundEngine::Query::GameObjDst,AK::SoundEngine::Query::GameObjDst const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AK::SoundEngine::Query::GameObjDst> > *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # AkArray<AK::SoundEngine::Query::GameObjDst,AK::SoundEngine::Query::GameObjDst const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AK::SoundEngine::Query::GameObjDst> > *
            ]
        ),
        "unsigned __int64": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
    },
    "AK::SoundEngine::Query::GetObjectObstructionAndOcclusion": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "AK::SoundEngine::Query::GetPlayingIDsFromGameObject": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AK::SoundEngine::Query::GetPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # AkTransform *
        ]
    ),
    "AK::SoundEngine::Query::GetPositioningInfo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkPositioningInfo *
        ]
    ),
    "AK::SoundEngine::Query::GetRTPCValue": {
        "unsigned int, unsigned __int64, unsigned int, float *, AK::SoundEngine::Query::RTPCValue_type *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # AK::SoundEngine::Query::RTPCValue_type *
            ]
        ),
        "const char *, unsigned __int64, unsigned int, float *, AK::SoundEngine::Query::RTPCValue_type *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # AK::SoundEngine::Query::RTPCValue_type *
            ]
        ),
        "const wchar_t *, unsigned __int64, unsigned int, float *, AK::SoundEngine::Query::RTPCValue_type *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # AK::SoundEngine::Query::RTPCValue_type *
            ]
        ),
    },
    "AK::SoundEngine::Query::GetState": {
        "unsigned int, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
        "const char *, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
        "const wchar_t *, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
    },
    "AK::SoundEngine::Query::GetSwitch": {
        "unsigned int, unsigned __int64, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
        "const char *, unsigned __int64, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
        "const wchar_t *, unsigned __int64, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
    },
    "CAkConnectedListeners::GetUserAssocs": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkLstnrAssocs *
        argtypes=[
            ctypes.c_ulonglong,  # CAkConnectedListeners *
        ]
    ),
    "AK::SoundEngine::Query::QueryAudioObjectIDs": {
        "unsigned int, unsigned int *, AkObjectInfo *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # unsigned int *
                ctypes.c_ulonglong,  # AkObjectInfo *
            ]
        ),
        "const char *, unsigned int *, AkObjectInfo *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_char_p,  # const char *
                ctypes.c_ulonglong,  # unsigned int *
                ctypes.c_ulonglong,  # AkObjectInfo *
            ]
        ),
        "const wchar_t *, unsigned int *, AkObjectInfo *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # unsigned int *
                ctypes.c_ulonglong,  # AkObjectInfo *
            ]
        ),
    },
    "AkTlsAllocateSlot": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AkTlsFreeSlot": FUNCDEF(
        restype=ctypes.c_ubyte,  # BOOL
        argtypes=[
            ctypes.c_uint32,  # DWORD
        ]
    ),
    "AkTlsGetValue": FUNCDEF(
        restype=wintypes.LPVOID,  # LPVOID
        argtypes=[
            ctypes.c_uint32,  # DWORD
        ]
    ),
    "AkTlsSetValue": FUNCDEF(
        restype=ctypes.c_ubyte,  # BOOL
        argtypes=[
            ctypes.c_uint32,  # DWORD
            wintypes.LPVOID,  # LPVOID
        ]
    ),
    "CAkGameObject::CreateComponent<CAkVolumeRayComponent>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkVolumeRayComponent *
        argtypes=[
            ctypes.c_ulonglong,  # CAkGameObject *
        ]
    ),
    "CAkBehavioralCtx::CAkBehavioralCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkBehavioralCtx::~CAkBehavioralCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkBehavioralCtx::CacheEmitterPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkBehavioralCtx::CalculateMutedEffectiveVolume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkBehavioralCtx::ComputeEmitterListenerPairs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkBehavioralCtx::EnableObstruction": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkBehavioralCtx::EvaluateFocus": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_float,  # float
        ]
    ),
    "CAkBehavioralCtx::EvaluateSpread": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_float,  # float
        ]
    ),
    "AkSortedKeyArray<unsigned __int64,unsigned __int64,AkHybridAllocator<8,16,4>,AkSetGetKey<unsigned __int64>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64>,AkDefaultSortedKeyCompare<unsigned __int64>>::Exists": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int64 *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned __int64,unsigned __int64,AkHybridAllocator<8,16,4>,AkSetGetKey<unsigned __int64>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64>,AkDefaultSortedKeyCompare<unsigned __int64> > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CAkBehavioralCtx::FetchCustomMetadata": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ulonglong,  # AkAudioObject::CustomMetadata *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBehavioralCtx::Get3DParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkBehavioralCtx::GetAuxSendsValues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ulonglong,  # AkArray<AkAuxSendValueEx,AkAuxSendValueEx const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAuxSendValueEx> > *
        ]
    ),
    "CAkBehavioralCtx::GetControlBus": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkParameterNodeBase *
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkBehavioralCtx::GetEmitterListenerPair": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkEmitterListenerPair *
        ]
    ),
    "CAkBehavioralCtx::GetGameObjectPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkTransform *
        ]
    ),
    "AkPositionStore::GetGameRayParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkPositionStore *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkGameRayParams *
        ]
    ),
    "CAkBehavioralCtx::GetListenerData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # AkListener *
        ]
    ),
    "CAkBehavioralCtx::GetModulatorTriggerParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ulonglong,  # AkModulatorTriggerParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkBehavioralCtx::GetNumRays": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ulonglong,  # const AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
            ctypes.c_uint32,  # AkSetType
        ]
    ),
    "CAkBehavioralCtx::GetOutputBusPtr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkBus *
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "AkPositioningParams::GetRawAttenuation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkAttenuation *
        argtypes=[
            ctypes.c_ulonglong,  # AkPositioningParams *
        ]
    ),
    "CAkBehavioralCtx::GetRay": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRayVolumeData *
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ulonglong,  # const AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
            ctypes.c_uint32,  # AkSetType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBehavioralCtx::GetSoundID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkBehavioralCtx::GetTargetedParamsSet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkBitArray<unsigned __int64,0> *
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ulonglong,  # CAkBitArray<unsigned __int64,0> *
        ]
    ),
    "CAkBehavioralCtx::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkBehavioralCtx::InitObjectMetadata": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkBehavioralCtx::InitSpatialAudioVoice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkBehavioralCtx::IsAuxRoutable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkParameterNodeBase::IsInHdrHierarchy": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkBehavioralCtx::IsInitiallyUnderThreshold": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ulonglong,  # AkInitialSoundParams *
        ]
    ),
    "CAkBehavioralCtx::MultiPositionType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkBehavioralCtx::NotifyParamsChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const CAkBitArray<unsigned __int64,0> *
        ]
    ),
    "PluginRTPCSub::NotifyRtpcParamChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PluginRTPCSub *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkRtpcAccum
            ctypes.c_float,  # float
        ]
    ),
    "CAkBehavioralCtx::OnPathAdded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ulonglong,  # CAkPath *
        ]
    ),
    "CAkBehavioralCtx::ParamNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ulonglong,  # NotifParams *
        ]
    ),
    "CAkBehavioralCtx::PausePath": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkBehavioralCtx::PositioningChangeNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # AkRTPC_ParameterID
        ]
    ),
    "CAkBehavioralCtx::PreInit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # AkPathInfo *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # AkMonitorData::NotificationReason *
            ctypes.c_ulonglong,  # AkInitialSoundParams *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkBehavioralCtx::RecalcNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkBehavioralCtx::ReleaseModulators": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkBehavioralCtx::Reset3DParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "AkArray<AkRayVolumeData,AkRayVolumeData const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRayVolumeData>>::Resize": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkRayVolumeData,AkRayVolumeData const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRayVolumeData> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBehavioralCtx::Scale3DUserDefRTPCValue": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_float,  # float
        ]
    ),
    "CAkBehavioralCtx::SetParam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBehavioralCtx::SetupPath": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ulonglong,  # AkPathInfo *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkBehavioralCtx::SpatialAudioParamsUpdated": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkBehavioralCtx::StartPath": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkBehavioralCtx::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkBehavioralCtx::UpdateAttenuation": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ulonglong,  # CAkAttenuation *
        ]
    ),
    "CAkBehavioralCtx::UpdateTargetParam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "PluginRTPCSub::UpdateTargetRtpcParam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PluginRTPCSub *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkRtpcAccum
            ctypes.c_float,  # float
        ]
    ),
    "CAkBehavioralCtx::UseSpatialAudioPanningMode": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkPBI::CAkPBI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # AkPBIParams *
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_ulonglong,  # CAkSource *
            ctypes.c_ulonglong,  # const PriorityInfoCurrent *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPBI::~CAkPBI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkPBI::CalculateMutedEffectiveVolume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "AkInitialSoundParams::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkInitialSoundParams *
        ]
    ),
    "CAkPBI::ClearLimiters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkPBI::ComputePriorityWithDistance": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_float,  # float
        ]
    ),
    "CAkPBI::DecrementPlayCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkPBI::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_uint32,  # AkCtxDestroyReason
        ]
    ),
    "CAkPBI::Devirtualize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkPBI::FetchPriorityMetadata": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # AkAudioObject::CustomMetadata *
        ]
    ),
    "CAkPBI::FindAlternateMedia": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # const CAkUsageSlot *
        ]
    ),
    "CAkPBI::ForceVirtualize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_uint32,  # KickFrom
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "CAkPBI::GetAndClearStopOffset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkPBI::GetFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # AkFXDesc *
        ]
    ),
    "CAkPBI::GetFXDataID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AkPBIParams::GetMidiTargetID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkPBIParams *
        ]
    ),
    "CAkPBI::GetModulatorTriggerParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # AkModulatorTriggerParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkPBI::GetPauseResumeTransition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkTransition *
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkPBI::GetPlayStopTransition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkTransition *
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkPBI::GetVirtualBehavior": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # AkVirtualQueueBehavior *
        ]
    ),
    "CAkPBI::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkPBI::IsUsingThisSlot": {
        "CAkPBI *, const unsigned __int8 *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # CAkPBI *
                ctypes.c_ulonglong,  # const unsigned __int8 *
            ]
        ),
        "CAkPBI *, const CAkUsageSlot *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # CAkPBI *
                ctypes.c_ulonglong,  # const CAkUsageSlot *
            ]
        ),
    },
    "CAkPBI::Kick": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_uint32,  # KickFrom
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "AkMutedMap::MuteNotification": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # AkMutedMap *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # AkMutedMapItem *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkPBI::MuteNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # AkMutedMapItem *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkPBI::NotifAddedAsSA": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkPBI::NotifyBypass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPBI::OnPathAdded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # CAkPath *
        ]
    ),
    "CAkPBI::Pause": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkPBI::Play": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_float,  # float
        ]
    ),
    "CAkPBI::PlayToEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "AkPBIParams::PopulateInitialSoundParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkPBIParams *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkPBI::ProcessCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # ActionParams *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "CAkPBI::ProcessContextNotif": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_uint32,  # AkCtxState
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
        ]
    ),
    "CAkPBI::RefreshParameters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # AkInitialSoundParams *
        ]
    ),
    "AkArray<MapStruct<AkMutedMapItem,float>,MapStruct<AkMutedMapItem,float> const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<AkMutedMapItem,float>>>::Resize": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<MapStruct<AkMutedMapItem,float>,MapStruct<AkMutedMapItem,float> const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<AkMutedMapItem,float> > > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPBI::SeekPercent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkPBI::SeekTimeAbsolute": {
        "CAkPBI *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkPBI *
                ctypes.c_int32,  # int
            ]
        ),
        "CAkPBI *, int, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkPBI *
                ctypes.c_int32,  # int
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "CAkPBI::SetMuteMapEntry": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # AkMutedMapItem *
            ctypes.c_float,  # float
        ]
    ),
    "CAkPBI::ShouldChangeVirtualBehaviourForInterruption": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkPBI::Stop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkPBI::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkPBI::TransUpdateValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkPBI::TriggerModulators": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # const AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkPBI::UpdateFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPBI::UpdateLimiters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkPBI::UpdatePriority": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_float,  # float
        ]
    ),
    "CAkPBI::UpdatePriorityWithDistance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # CAkAttenuation *
            ctypes.c_float,  # float
        ]
    ),
    "CAkPBI::UpdateTargetParam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CAkPBI::VirtualPositionUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkPBI::Virtualize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkPBI::_InitPlay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkPBI::_Pause": {
        "CAkPBI *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkPBI *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "CAkPBI *, TransParams *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkPBI *
                ctypes.c_ulonglong,  # TransParams *
            ]
        ),
    },
    "CAkPBI::_Play": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # TransParams *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkPBI::_Resume": {
        "CAkPBI *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkPBI *
            ]
        ),
        "CAkPBI *, TransParams *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkPBI *
                ctypes.c_ulonglong,  # TransParams *
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "CAkPBI::_Stop": {
        "CAkPBI *, const TransParams *, _BOOL8": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkPBI *
                ctypes.c_ulonglong,  # const TransParams *
                ctypes.c_ubyte,  # _BOOL8
            ]
        ),
        "CAkPBI *, AkPBIStopMode, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkPBI *
                ctypes.c_uint32,  # AkPBIStopMode
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "CAkURenderer::AddLimiter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLimiter *
        ]
    ),
    "CAkURenderer::AddToActiveNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkURenderer::ClearPBIAndDecrement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_ulonglong,  # CAkPBI **
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkURenderer::DecrementDangerousVirtualCount": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkURenderer::DecrementVirtualCount": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkURenderer::DropAllEffectsUsingThisMedia": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int8 *
        ]
    ),
    "CAkURenderer::EnqueueContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkURenderer::EnqueueContextNotif": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_uint32,  # AkCtxState
            ctypes.c_uint32,  # AkCtxDestroyReason
            ctypes.c_float,  # float
        ]
    ),
    "CAkURenderer::GetMaxRadius": {
        "AkArray<AK::SoundEngine::Query::GameObjDst,AK::SoundEngine::Query::GameObjDst const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AK::SoundEngine::Query::GameObjDst> > *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # AkArray<AK::SoundEngine::Query::GameObjDst,AK::SoundEngine::Query::GameObjDst const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AK::SoundEngine::Query::GameObjDst> > *
            ]
        ),
        "unsigned __int64": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
    },
    "CAkURenderer::GetMinDistance": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const AkChannelEmitter *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
        ]
    ),
    "CAkURenderer::GetVirtualBehaviorAction": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_ulonglong,  # CAkPBIAware *
            ctypes.c_ulonglong,  # AkBelowThresholdBehavior *
        ]
    ),
    "CAkURenderer::IncrementDangerousVirtualCount": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkURenderer::IncrementPlayCountAndInit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # AKRESULT
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # AkMonitorData::NotificationReason *
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkURenderer::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "CAkURenderer::Kick": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkLimiter *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # CAkParameterNodeBase **
            ctypes.c_uint32,  # KickFrom
        ]
    ),
    "CAkURenderer::PerformContextNotif": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkURenderer::Play": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_ulonglong,  # CAkSource *
            ctypes.c_ulonglong,  # AkPBIParams *
        ]
    ),
    "CAkURenderer::ProcessLimiters": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkURenderer::RemoveFromActiveNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkURenderer::RemoveLimiter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLimiter *
        ]
    ),
    "AkHashList<unsigned __int64,float,AkArrayAllocatorNoAlign<0>>::Resize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkHashList<unsigned __int64,float,AkArrayAllocatorNoAlign<0> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkURenderer::StopAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ActionParams *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkURenderer::StopAllPBIs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const CAkUsageSlot *
        ]
    ),
    "CAkURenderer::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkURenderer::ValidateLimits": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # AkMonitorData::NotificationReason *
        ]
    ),
    "CAkURenderer::_CalcInitialPriority": FUNCDEF(
        restype=ctypes.c_ulonglong,  # PriorityInfoCurrent *
        argtypes=[
            ctypes.c_ulonglong,  # PriorityInfoCurrent *
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkPlayingMgr::AddItemActiveCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPlayingMgr::AddPlayingID": {
        "CAkPlayingMgr *, AkQueuedMsg_EventBase *, __int64 *, void *, unsigned int, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # CAkPlayingMgr *
                ctypes.c_ulonglong,  # AkQueuedMsg_EventBase *
                ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkCallbackType', 'AkCallbackInfo *']}
                ctypes.c_ulonglong,  # void *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "CAkPlayingMgr *, unsigned int, unsigned __int64, AkCustomParamType *, __int64 *, void *, unsigned int, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # CAkPlayingMgr *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # AkCustomParamType *
                ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkCallbackType', 'AkCallbackInfo *']}
                ctypes.c_ulonglong,  # void *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "CAkPlayingMgr *, unsigned int, unsigned __int64, __int64 *, void *, unsigned int, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # CAkPlayingMgr *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkCallbackType', 'AkCallbackInfo *']}
                ctypes.c_ulonglong,  # void *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "CAkPlayingMgr::AddedNewModulatorCtx": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPlayingMgr::AddedNewRtpcValue": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPlayingMgr::CancelCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPlayingMgr::CancelCallbackCookie": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkPlayingMgr::CancelCallbackGameObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CAkPlayingMgr::CheckRemovePlayingID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkPlayingMgr::PlayingMgrItem *
        ]
    ),
    "CAkPlayingMgr::GetCookie": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPlayingMgr::GetEventIDFromPlayingID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPlayingMgr::GetGameObjectFromPlayingID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPlayingMgr::GetPBIs": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkArray<CAkTransportAware *,CAkTransportAware *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkTransportAware *> > *
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPlayingMgr::GetPlayingIDsFromGameObject": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkPlayingMgr::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
        ]
    ),
    "CAkPlayingMgr::MusicPlaylistCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkPlayingMgr::NotifyDuration": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkPlayingMgr::NotifyEndOfDynamicSequenceItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkPlayingMgr::NotifyMarker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkAudioMarker *
        ]
    ),
    "CAkPlayingMgr::NotifyMarkers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_ulonglong,  # AkMarkerNotification *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPlayingMgr::NotifyMidiEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkMidiEventEx *
        ]
    ),
    "CAkPlayingMgr::NotifyMusic": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkCallbackType
            ctypes.c_ulonglong,  # const AkSegmentInfo *
        ]
    ),
    "CAkPlayingMgr::NotifyMusicPlayStarted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPlayingMgr::NotifyMusicUserCues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkSegmentInfo *
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "CAkPlayingMgr::NotifySpeakerVolumeMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkSpeakerVolumeMatrixCallbackInfo *
        ]
    ),
    "CAkPlayingMgr::NotifyStarvation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPlayingMgr::Remove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkTransportAware *
        ]
    ),
    "CAkPlayingMgr::RemoveItemActiveCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkHashListBare<uint,CAkPlayingMgr::PlayingMgrItem,AkArrayAllocatorNoAlign<0>,CAkPlayingMgr::PlayingMgrItemPolicy>::Resize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkHashListBare<unsigned int,CAkPlayingMgr::PlayingMgrItem,AkArrayAllocatorNoAlign<0>,CAkPlayingMgr::PlayingMgrItemPolicy> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkSortedKeyArray<uint,uint,AkArrayAllocatorNoAlign<0>,AkAkRtpcIDSetGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<uint>,AkDefaultSortedKeyCompare<uint>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned int *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,unsigned int,AkArrayAllocatorNoAlign<0>,AkAkRtpcIDSetGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned int>,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkPlayingMgr::SetPBI": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkTransportAware *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkPlayingMgr::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
        ]
    ),
    "CAkPlayingMgr::UseExistingItem": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayingMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkStateAware::GetStateParamValue<AccumulateAdd,AkDeltaMonitorDisabled>": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
            ctypes.c_uint32,  # AkRTPC_ParameterID
        ]
    ),
    "CAkLimiter::CAkLimiter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLimiter *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParameterNodeBase::CAkParameterNodeBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkDynaBlkPool<AkActivityChunk,128,AkArrayAllocatorNoAlign<0>>::PoolChunk::PoolChunk": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDynaBlkPool<AkActivityChunk,128,AkArrayAllocatorNoAlign<0> >::PoolChunk *
        ]
    ),
    "CAkParameterNodeBase::~CAkParameterNodeBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::MetadataGetArray::Accumulate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase::MetadataGetArray *
            ctypes.c_ulonglong,  # CAkParameterNodeBase::MetadataChunk *
        ]
    ),
    "CAkLimiter::Add": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkLimiter *
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkParameterNodeBase::AddChildByPtr": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::AddRef": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::AddToIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::AddToURendererActiveNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::AdjustCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkParameterNodeBase **
        ]
    ),
    "CAkParameterNodeBase::AllocateMetadata": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # SharedSetOverride
        ]
    ),
    "CAkParameterNodeBase::ApplyAllSIS": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # const CAkSIS *
            ctypes.c_ulonglong,  # AkMutedMap *
            ctypes.c_ulonglong,  # AkSoundParams *
        ]
    ),
    "CAkParameterNodeBase::ApplyTransitionsOnProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # AkSoundParams *
            ctypes.c_ulonglong,  # AkMutedMap *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkParameterNodeBase::BypassFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParameterNodeBase::CheckAndDeleteActivityChunk": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::CheckAndDeleteGameObjLimiter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkParameterNodeBase::ClearLimitersAndMakeDirty": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::CreateGameObjectLimiter": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # StructMaxInst **
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkParameterNodeBase::DecrementActivityCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkParameterNodeBase::DecrementActivityCountValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParameterNodeBase::DecrementPlayCountValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParameterNodeBase::DeleteActivityChunk": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "StructMaxInst::DisableLimiter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # StructMaxInst *
        ]
    ),
    "CAkParameterNodeBase::EnableActivityChunk": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::Ensure3DAutomationAllocated": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::FindCustomProp": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkPropValue *
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParameterNodeBase::FreePathInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::Get2DParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # BaseGenParams *
        ]
    ),
    "CAkParameterNodeBase::Get3DAutomationParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAk3DAutomationParams **
        ]
    ),
    "CAkParameterNodeBase::Get3DPanning": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # AkVector *
        ]
    ),
    "CAkParameterNodeBase::GetAttachedPropFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # AkFXDesc *
        ]
    ),
    "CAkParameterNodeBase::GetBypassFX": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkParameterNodeBase::GetControlBus": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkParameterNodeBase *
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::GetDepth": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::GetMaxNumInstances": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkParameterNodeBase::GetMaxRadius": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkParameterNodeBase::GetMetadataArray": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # AkFXDesc *
        ]
    ),
    "CAkParameterNodeBase::GetMixingBus": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkBus *
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::GetModulatorParamXfrms": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # AkArray<AkModulatorParamXfrm,AkModulatorParamXfrm const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorParamXfrm> > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const CAkRegisteredObj *
        ]
    ),
    "CAkParameterNodeBase::GetNumMetadata": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::GetPathState": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkPathState *
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::GetPositioningParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # BaseGenParams *
            ctypes.c_ulonglong,  # AkPositioningParams *
        ]
    ),
    "CAkParameterNodeBase::GetPriority": FUNCDEF(
        restype=ctypes.c_ulonglong,  # PriorityInfo *
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # PriorityInfo *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkParameterNodeBase::GetPropAndRTPCAndState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # AkSoundParams *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkParameterNodeBase::GetSIS": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkSIS *
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkParameterNodeBase::GetStateSyncTypes": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkStateAware::CAkStateSyncArray *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParameterNodeBase::GetStatic3DParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # AkPositioningInfo *
        ]
    ),
    "CAkParamTargetLimiter::GetTargetedParamsSet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkBitArray<unsigned __int64,0> *
        argtypes=[
            ctypes.c_ulonglong,  # CAkParamTargetLimiter *
            ctypes.c_ulonglong,  # CAkBitArray<unsigned __int64,0> *
        ]
    ),
    "CAkParameterNodeBase::HasListenerRelativeRouting": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::IncrementActivityCount": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkParameterNodeBase::IncrementActivityCountValue": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParameterNodeBase::IncrementPlayCountValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParamTargetLimiter::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParamTargetLimiter *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParameterNodeBase::IsException": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # AkArray<WwiseObjectIDext,WwiseObjectIDext const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<WwiseObjectIDext> > *
        ]
    ),
    "CAkParameterNodeBase::IsInfiniteLooping": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkPBIAware *
        ]
    ),
    "CAkParameterNodeBase::IsMidiBreakLoopOnNoteOff": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::IsMidiNoteTracking": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "CAkParameterNodeBase::MainBypassFX": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParameterNodeBase::Notification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "CAkParameterNodeBase::Parent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::ParentBus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::Pause": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
        ]
    ),
    "CAkParameterNodeBase::PrepareNodeData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParameterNodeBase::ProcessGameObjectLimiter": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CounterParameters *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParameterNodeBase::ProcessGlobalLimiter": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CounterParameters *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkSortedKeyArray<AkPriorityStruct,CAkPBI *,AkArrayAllocatorNoAlign<0>,AkSortedPBIGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkPBI *>,AkPBIPriorityCompare<CAkLimiter>>::ReSortArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<AkPriorityStruct,CAkPBI *,AkArrayAllocatorNoAlign<0>,AkSortedPBIGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkPBI *>,AkPBIPriorityCompare<CAkLimiter> > *
        ]
    ),
    "CAkParameterNodeBase::RegisterParameterTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkParameterTarget *
            ctypes.c_ulonglong,  # const CAkBitArray<unsigned __int64,0> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParameterNodeBase::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkLimiter::Remove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLimiter *
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkParameterNodeBase::RemoveFromURendererActiveNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNodeBase::RemoveOutputBus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParamNodeStateTargetData::RemoveTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParamNodeStateTargetData *
            ctypes.c_ulonglong,  # CAkParameterTarget *
        ]
    ),
    "CAkParameterNodeBase::ResetAkProp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # AkPropID
            ctypes.c_uint32,  # AkCurveInterpolation
            ctypes.c_int32,  # int
        ]
    ),
    "CAkParameterNodeBase::ResetBypassFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkParameterNodeBase::ResetFXBypass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkHashListBare<uint,CAkIndexable,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<uint,CAkIndexable>>::Resize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkHashListBare<unsigned int,CAkIndexable,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<unsigned int,CAkIndexable> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParameterNodeBase::Resume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
        ]
    ),
    "CAkParameterNodeBase::SafeDisconnectActivityChunk": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "AkHashListBare<uint,CAkIndexable,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<uint,CAkIndexable>>::Set": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkHashListBare<unsigned int,CAkIndexable,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<unsigned int,CAkIndexable> > *
            ctypes.c_ulonglong,  # CAkIndexable *
        ]
    ),
    "CAkParameterNodeBase::SetAkProp": {
        "CAkParameterNodeBase *, AkPropID, int, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
                ctypes.c_uint32,  # AkPropID
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
            ]
        ),
        "CAkParameterNodeBase *, AkPropID, float, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
                ctypes.c_uint32,  # AkPropID
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
    },
    "CAkParameterNodeBase::SetAuxParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkParameterNodeBase::SetMaxReachedBehavior": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParameterNodeBase::SetMetadata": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # SharedSetOverride
        ]
    ),
    "CAkParameterNodeBase::SetNodeBaseParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParameterNodeBase::SetOverLimitBehavior": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParameterNodeBase::SetPositioningParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkParameterNodeBase::SetRTPC": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkRtpcType
            ctypes.c_uint32,  # AkRtpcAccum
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkCurveScaling
            ctypes.c_ulonglong,  # AkGraphPointBase<float> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParameterNodeBase::SetVVoicesOptOverrideParent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParameterNodeBase::StartSISTransition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkSIS *
            ctypes.c_uint32,  # AkPropID
            ctypes.c_float,  # float
            ctypes.c_uint32,  # AkValueMeaning
            ctypes.c_uint32,  # AkCurveInterpolation
            ctypes.c_int32,  # int
        ]
    ),
    "CAkParameterNodeBase::StartSisMuteTransitions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkSIS *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # AkCurveInterpolation
            ctypes.c_int32,  # int
        ]
    ),
    "CAkParameterNodeBase::Stop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
        ]
    ),
    "CAkParameterNodeBase::TriggerModulators": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # const AkModulatorTriggerParams *
            ctypes.c_ulonglong,  # CAkModulatorData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParameterNodeBase::UnPrepareNodeData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParameterNodeBase::Unmute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # AkCurveInterpolation
            ctypes.c_int32,  # int
        ]
    ),
    "CAkParameterNodeBase::Unregister": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkParameterNodeBase::UnregisterParameterTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkParameterTarget *
            ctypes.c_ulonglong,  # const CAkBitArray<unsigned __int64,0> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkKeyArray<CAkRegisteredObj *,StructMaxInst,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<CAkRegisteredObj *,StructMaxInst>>>::Unset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkKeyArray<CAkRegisteredObj *,StructMaxInst,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<CAkRegisteredObj *,StructMaxInst> > > *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkParameterNodeBase::UnsetRTPC": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkLimiter::Update": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLimiter *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkParameterNodeBase::UpdateBaseParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # BaseGenParams *
            ctypes.c_ulonglong,  # CAk3DAutomationParams *
        ]
    ),
    "CAkLimiter::UpdateFlags": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLimiter *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "CAkParamTargetLimiter::UpdateTargetParam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParamTargetLimiter *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_float,  # float
        ]
    ),
    "CAkParameterNodeBase::_GetBypassAllFX": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "AkPendingAction::AkPendingAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkPendingAction *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkAudioMgr::CAkAudioMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
        ]
    ),
    "AkPendingAction::~AkPendingAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkAudioMgr::~CAkAudioMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
        ]
    ),
    "CAkAudioMgr::BreakPendingAction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAudioMgr::ClearCrossFadeOccurence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ulonglong,  # CAkContinuousPBI *
        ]
    ),
    "CAkAudioMgr::ClearPendingItems": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAudioMgr::ComputeFramesToRender": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
        ]
    ),
    "CAkAudioMgr::EnqueueOrExecuteAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ulonglong,  # AkPendingAction *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkAudioMgr::ExecuteEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkEvent *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkCustomParamType *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkPendingAction::GameObjID": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "GetListeners": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # AkQueuedMsg_ListenerIDs *
            ctypes.c_ulonglong,  # AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
        ]
    ),
    "AkArray<unsigned __int64,unsigned __int64 const &,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64>>::GrowArray": {
        "AkArray<unsigned __int64,unsigned __int64 const &,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64> > *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # AkArray<unsigned __int64,unsigned __int64 const &,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64> > *
            ]
        ),
        "AkArray<unsigned __int64,unsigned __int64 const &,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64> > *, unsigned int": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # AkArray<unsigned __int64,unsigned __int64 const &,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64> > *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "CAkAudioMgr::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
        ]
    ),
    "CAkAudioMgr::InitSinkPlugin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ulonglong,  # AkOutputSettings *
        ]
    ),
    "CAkMultiKeyList<uint,AkPendingAction *,2>::Insert": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMultiKeyList<unsigned int,AkPendingAction *,2> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkPendingAction **
        ]
    ),
    "CAkAudioMgr::InsertAsPaused": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkPendingAction *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAudioMgr::IsAnException": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ulonglong,  # CAkAction *
            ctypes.c_ulonglong,  # AkArray<WwiseObjectIDext,WwiseObjectIDext const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<WwiseObjectIDext> > *
        ]
    ),
    "CAkAudioMgr::MidiNoteOffExecuted": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ulonglong,  # CAkAction *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # MidiEventActionType
        ]
    ),
    "CAkAudioMgr::PausePending": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkAudioMgr::PausePendingAction": {
        "CAkAudioMgr *, CAkAction *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkAudioMgr *
                ctypes.c_ulonglong,  # CAkAction *
            ]
        ),
        "CAkAudioMgr *, CAkParameterNodeBase *, CAkRegisteredObj *, bool, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkAudioMgr *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
                ctypes.c_ulonglong,  # CAkRegisteredObj *
                ctypes.c_ubyte,  # bool
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "CAkAudioMgr::PausePendingActionAllExcept": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # AkArray<WwiseObjectIDext,WwiseObjectIDext const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<WwiseObjectIDext> > *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAudioMgr::PausePendingItems": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAudioMgr::Perform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkAudioMgr::ProcessAllActions": {
        "CAkAudioMgr *, CAkEvent *, AkQueuedMsg_EventAction *, CAkRegisteredObj *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkAudioMgr *
                ctypes.c_ulonglong,  # CAkEvent *
                ctypes.c_ulonglong,  # AkQueuedMsg_EventAction *
                ctypes.c_ulonglong,  # CAkRegisteredObj *
            ]
        ),
        "CAkAudioMgr *, CAkEvent *, AkQueuedMsg_EventPostMIDI *, CAkRegisteredObj *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkAudioMgr *
                ctypes.c_ulonglong,  # CAkEvent *
                ctypes.c_ulonglong,  # AkQueuedMsg_EventPostMIDI *
                ctypes.c_ulonglong,  # CAkRegisteredObj *
            ]
        ),
        "CAkAudioMgr *, CAkEvent *, AkQueuedMsg_EventStopMIDI *, CAkRegisteredObj *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkAudioMgr *
                ctypes.c_ulonglong,  # CAkEvent *
                ctypes.c_ulonglong,  # AkQueuedMsg_EventStopMIDI *
                ctypes.c_ulonglong,  # CAkRegisteredObj *
            ]
        ),
    },
    "CAkAudioMgr::ProcessMsgQueue": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkAudioMgr::ProcessPendingList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkAudioMgr::RemoveAllPreallocAndReferences": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
        ]
    ),
    "CAkAudioMgr::RemovePausedPendingAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkAudioMgr::RemovePendingAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkAudioMgr::RenderAudio": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkAudioMgr::ReserveForWrite": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkQueuedMsg *
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkAudioMgr::ReserveQueue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkQueuedMsg *
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAudioMgr::ResumePausedPendingAction": {
        "CAkAudioMgr *, CAkAction *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkAudioMgr *
                ctypes.c_ulonglong,  # CAkAction *
            ]
        ),
        "CAkAudioMgr *, CAkParameterNodeBase *, CAkRegisteredObj *, bool, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkAudioMgr *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
                ctypes.c_ulonglong,  # CAkRegisteredObj *
                ctypes.c_ubyte,  # bool
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "CAkAudioMgr::ResumePausedPendingActionAllExcept": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # AkArray<WwiseObjectIDext,WwiseObjectIDext const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<WwiseObjectIDext> > *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAudioMgr::ResumePausedPendingItems": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkQueuedMsg::Sizeof_Switch": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "AkQueuedMsg::Sizeof_PlaySourcePlugin": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "AkQueuedMsg::Sizeof_GameObjObstruction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "AkQueuedMsg::Sizeof_Seek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "CAkAudioMgr::StampTimeLastBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
        ]
    ),
    "CAkAudioMgr::Start": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
        ]
    ),
    "CAkAudioMgr::Stop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
        ]
    ),
    "CAkAudioMgr::StopAction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAudioMgr::StopPending": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkAudioMgr::StopPendingAction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAudioMgr::StopPendingActionAllExcept": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # AkArray<WwiseObjectIDext,WwiseObjectIDext const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<WwiseObjectIDext> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAudioMgr::SuspendWakeup": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkList2<MapStruct<uint,AkPendingAction *>,MapStruct<uint,AkPendingAction *> const &,2,0>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkList2<MapStruct<unsigned int,AkPendingAction *>,MapStruct<unsigned int,AkPendingAction *> const &,2,0> *
        ]
    ),
    "CAkAudioMgr::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
        ]
    ),
    "AkPendingAction::TransUpdateValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkPendingAction *
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkAudioMgr::TransferToPending": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkAudioMgr::WaitOnMsgQueuewriters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioMgr *
        ]
    ),
    "RandomizerModifier::GetMod<int>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # RANGED_MODIFIERS<int> *
        ]
    ),
    "CAkSoundBase::CAkSoundBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSoundBase::~CAkSoundBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSoundBase *
        ]
    ),
    "CAkSoundBase::ClearLimiters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSoundBase *
        ]
    ),
    "CAkSoundBase::ForAllPBI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['CAkPBI *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkSoundBase::IsInfiniteLooping": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_ulonglong,  # CAkPBIAware *
        ]
    ),
    "CAkSoundBase::Loop": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSoundBase *
        ]
    ),
    "CAkSoundBase::MuteNotification": {
        "CAkSoundBase *, float, AkMutedMapItem *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkSoundBase *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # AkMutedMapItem *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "CAkSoundBase *, float, CAkRegisteredObj *, AkMutedMapItem *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkSoundBase *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # CAkRegisteredObj *
                ctypes.c_ulonglong,  # AkMutedMapItem *
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "CAkSoundBase::NotifyBypass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "CAkSoundBase::ParamNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_ulonglong,  # NotifParams *
        ]
    ),
    "CAkSoundBase::PlayToEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSoundBase::ProcessCommand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_ulonglong,  # ActionParams *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "CAkSoundBase::PropagatePositioningNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "CAkSoundBase::RecalcNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSoundBase::UpdateFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMidiClipMgr::CAkMidiClipMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipMgr *
        ]
    ),
    "CAkMidiClipMgr::~CAkMidiClipMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipMgr *
            ctypes.c_ulonglong,  # MapStruct<CAkMidiBaseMgr::TargetKey,CAkMidiBaseMgr::TargetInfo> *
        ]
    ),
    "CAkMidiClipMgr::AddClipCtx": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipMgr *
            ctypes.c_ulonglong,  # CAkMusicCtx *
            ctypes.c_ulonglong,  # CAkMusicTrack *
            ctypes.c_ulonglong,  # CAkMusicSource *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # TransParams *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_ulonglong,  # const AkTrackSrc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkMidiClipCtx **
        ]
    ),
    "CAkMidiClipMgr::NextFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "CAkMidiClipMgr::StopNoteIfUsingData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipMgr *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMidiClipMgr::UpdateOnFirstNote": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipMgr *
            ctypes.c_ulonglong,  # AkListBare<AkMidiFrameEvent,AkListBareNextItem,AkCountPolicyNoCount,AkLastPolicyWithLast> *
            ctypes.c_ulonglong,  # AkMidiFrameEvent *
        ]
    ),
    "AkAudioLibSettings::SetAudioBufferSettings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkTransition::CAkTransition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkTransition *
        ]
    ),
    "CAkRegistryMgr::~CAkRegistryMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
        ]
    ),
    "CAkTransition::ComputeTransition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkTransition *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkTransition::InitParameters": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkTransition *
            ctypes.c_ulonglong,  # const TransitionParameters *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkInterpolation::InterpolateNoCheck": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # AkCurveInterpolation
        ]
    ),
    "CAkTransition::IsFadingOut": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkTransition *
        ]
    ),
    "CAkTransition::UpdateFilteringCoeff": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkTransition *
            ctypes.c_int32,  # int
        ]
    ),
    "CAkTransitionManager::AddTransitionToList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkTransitionManager *
            ctypes.c_ulonglong,  # const TransitionParameters *
            ctypes.c_uint32,  # CAkTransitionManager::AkTransitionCategory
        ]
    ),
    "CAkTransitionManager::AddTransitionUser": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkTransitionManager *
            ctypes.c_ulonglong,  # CAkTransition *
            ctypes.c_ulonglong,  # ITransitionable *
        ]
    ),
    "CAkTransitionManager::ChangeParameter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkTransitionManager *
            ctypes.c_ulonglong,  # CAkTransition *
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
            ctypes.c_uint32,  # AkValueMeaning
        ]
    ),
    "CAkTransitionManager::Pause": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkTransitionManager *
            ctypes.c_ulonglong,  # CAkTransition *
        ]
    ),
    "CAkTransitionManager::PauseAllStateTransitions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkTransitionManager *
        ]
    ),
    "CAkTransitionManager::ProcessTransitionsList": {
        "CAkTransitionManager *, unsigned int, AkArray<CAkTransition *,CAkTransition *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkTransition *> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkTransitionManager *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # AkArray<CAkTransition *,CAkTransition *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkTransition *> > *
            ]
        ),
        "CAkTransitionManager *, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkTransitionManager *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "CAkTransitionManager::RemoveTransitionUser": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkTransitionManager *
            ctypes.c_ulonglong,  # CAkTransition *
            ctypes.c_ulonglong,  # ITransitionable *
        ]
    ),
    "CAkTransitionManager::Resume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkTransitionManager *
            ctypes.c_ulonglong,  # CAkTransition *
        ]
    ),
    "CAkTransitionManager::ResumeAllStateTransitions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkTransitionManager *
        ]
    ),
    "CAkTransitionManager::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkTransitionManager *
        ]
    ),
    "CAkTransitionManager::TermList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkTransitionManager *
            ctypes.c_ulonglong,  # AkArray<CAkTransition *,CAkTransition *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkTransition *> > *
        ]
    ),
    "CAkParameterTarget::CAkParameterTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterTarget *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkRTPCSubscriberNode::CAkRTPCSubscriberNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCSubscriberNode *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParameterTarget::~CAkParameterTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterTarget *
        ]
    ),
    "CAkRTPCSubscriberNode::~CAkRTPCSubscriberNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCSubscriberNode *
        ]
    ),
    "AkParamTargetList::Add": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkParamTargetList *
            ctypes.c_ulonglong,  # CAkParameterTarget *
        ]
    ),
    "CAkRTPCSubscriberNode::CreateData": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCSubscriberNode *
        ]
    ),
    "CAkRTPCSubscriberNode::DestroyData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCSubscriberNode *
        ]
    ),
    "CAkRTPCSubscriberNode::EnableParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCSubscriberNode *
            ctypes.c_uint32,  # AkRTPC_ParameterID
        ]
    ),
    "CAkRTPCSubscriberNode::GetModulatorParamXfrm": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCSubscriberNode *
            ctypes.c_ulonglong,  # AkArray<AkModulatorParamXfrm,AkModulatorParamXfrm const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorParamXfrm> > *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const CAkRegisteredObj *
        ]
    ),
    "AkArray<MapStruct<uint,AkSwitchHistItem>,MapStruct<uint,AkSwitchHistItem> const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<uint,AkSwitchHistItem>>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<MapStruct<unsigned int,AkSwitchHistItem>,MapStruct<unsigned int,AkSwitchHistItem> const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<unsigned int,AkSwitchHistItem> > > *
        ]
    ),
    "CAkRTPCSubscriberNode::NotifyParamChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCSubscriberNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # AkRTPC_ParameterID
        ]
    ),
    "CAkRTPCSubscriberNode::NotifyParamsChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCSubscriberNode *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const CAkBitArray<unsigned __int64,0> *
        ]
    ),
    "CAkRTPCSubscriberNode::PushParamUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCSubscriberNode *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "CAkRTPCSubscriberNode::PushParamUpdate_All": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCSubscriberNode *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CAkRTPCSubscriberNode::PushParamUpdate_Scoped": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCSubscriberNode *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_float,  # float
        ]
    ),
    "CAkRTPCSubscriberNode::PushParamUpdate_ScopedExcept": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCSubscriberNode *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "CAkParameterTarget::RegisterParamTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterTarget *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # const CAkBitArray<unsigned __int64,0> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkParamTargetList::Remove": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkParamTargetList *
            ctypes.c_ulonglong,  # CAkParameterTarget *
        ]
    ),
    "AkSortedParamTrtArray::RemoveTarget": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedParamTrtArray *
            ctypes.c_ulonglong,  # CAkParameterTarget *
        ]
    ),
    "AkHashList<uint,AkListBare<CAkParameterTarget,AkListBareLightNextItem,AkCountPolicyNoCount,AkLastPolicyNoLast>,AkArrayAllocatorNoAlign<0>>::Resize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkHashList<unsigned int,AkListBare<CAkParameterTarget,AkListBareLightNextItem,AkCountPolicyNoCount,AkLastPolicyNoLast>,AkArrayAllocatorNoAlign<0> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkRTPCSubscriberNode::SetRTPC": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCSubscriberNode *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkRtpcType
            ctypes.c_uint32,  # AkRtpcAccum
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkCurveScaling
            ctypes.c_ulonglong,  # AkGraphPointBase<float> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParameterTarget::SetRootNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterTarget *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "AkHashList<uint,AkListBare<CAkParameterTarget,AkListBareLightNextItem,AkCountPolicyNoCount,AkLastPolicyNoLast>,AkArrayAllocatorNoAlign<0>>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkHashList<unsigned int,AkListBare<CAkParameterTarget,AkListBareLightNextItem,AkCountPolicyNoCount,AkLastPolicyNoLast>,AkArrayAllocatorNoAlign<0> > *
        ]
    ),
    "CAkParameterTarget::TermTargets": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkRTPCSubscriberNode::UnregisterAllParameterTargets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCSubscriberNode *
        ]
    ),
    "CAkParameterTarget::UnregisterParamTarget": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterTarget *
            ctypes.c_ulonglong,  # const CAkBitArray<unsigned __int64,0> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkRTPCSubscriberNode::UnsetRTPC": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCSubscriberNode *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParameterTarget::UpdateAllParameterTargets": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkParameterTarget::UpdateTargetNodeParam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterTarget *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CAkGameObject::CAkGameObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkGameObject *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CAkGameObject::~CAkGameObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkGameObject *
        ]
    ),
    "RegisterGameObjectAsActive": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkGameObject *
        ]
    ),
    "UnregisterGameObjectAsActive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkGameObject *
        ]
    ),
    "CAkAudioLibIndex::GetNodeLock": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkAudioLibIndex *
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioLibIndex *
            ctypes.c_uint32,  # AkNodeType
        ]
    ),
    "CAkAudioLibIndex::GetNodePtrAndAddRef": {
        "CAkAudioLibIndex *, WwiseObjectIDext *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # CAkParameterNodeBase *
            argtypes=[
                ctypes.c_ulonglong,  # CAkAudioLibIndex *
                ctypes.c_ulonglong,  # WwiseObjectIDext *
            ]
        ),
        "CAkAudioLibIndex *, unsigned int, AkNodeType": FUNCDEF(
            restype=ctypes.c_ulonglong,  # CAkParameterNodeBase *
            argtypes=[
                ctypes.c_ulonglong,  # CAkAudioLibIndex *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # AkNodeType
            ]
        ),
    },
    "AkHashListBare<uint,CAkIndexable,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<uint,CAkIndexable>>::Init": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkHashListBare<unsigned int,CAkIndexable,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<unsigned int,CAkIndexable> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAudioLibIndex::Init": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioLibIndex *
        ]
    ),
    "CAkAudioLibIndex::ReleaseDynamicSequences": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioLibIndex *
        ]
    ),
    "CAkAudioLibIndex::ReleaseTempObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioLibIndex *
        ]
    ),
    "CAkAudioLibIndex::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioLibIndex *
        ]
    ),
    "CAkStateMgr::CAkStateMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
        ]
    ),
    "AkStateGroupChunk::~AkStateGroupChunk": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkStateGroupChunk *
        ]
    ),
    "CAkStateMgr::~CAkStateMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
        ]
    ),
    "CAkStateMgr::AddStateGroup": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkStateMgr::AddStateGroupMember": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkStateGroupChunk *
        ]
    ),
    "CAkStateMgr::AddStateTransition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkStateMgr::GetPreparationItem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # MapStruct<unsigned int,AkStateGroupInfo *> *
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkGroupType
        ]
    ),
    "CAkStateMgr::GetState": {
        "CAkStateMgr *, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # CAkStateMgr *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "CAkStateMgr *, unsigned int, unsigned int *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # CAkStateMgr *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
    },
    "CAkStateMgr::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
        ]
    ),
    "CAkStateMgr::PrepareGameSync": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
            ctypes.c_uint32,  # AkGroupType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkStateMgr::RegisterSwitch": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
            ctypes.c_ulonglong,  # CAkSwitchAware *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkStateMgr::RegisterTrigger": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
            ctypes.c_ulonglong,  # IAkTriggerAware *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkStateMgr::RemoveAllStateGroups": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkStateMgr::RemoveStateGroupMember": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkStateGroupChunk *
        ]
    ),
    "CAkStateMgr::SetStateInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkStateMgr::SetdefaultTransitionTime": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
        ]
    ),
    "CAkStateMgr::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
        ]
    ),
    "CAkStateMgr::TermPreparationGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
            ctypes.c_ulonglong,  # AkListBare<CAkStateMgr::PreparationStateItem,AkListBareNextItem,AkCountPolicyNoCount,AkLastPolicyWithLast> *
        ]
    ),
    "CAkStateMgr::Trigger": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkStateMgr::UnregisterSwitch": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
            ctypes.c_ulonglong,  # CAkSwitchAware *
        ]
    ),
    "CAkStateMgr::UnregisterTrigger": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateMgr *
            ctypes.c_ulonglong,  # IAkTriggerAware *
        ]
    ),
    "CAkBankReader::CAkBankReader": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankReader *
        ]
    ),
    "CAkBankReader::CloseFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankReader *
        ]
    ),
    "CAkBankReader::FillData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankReader *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkBankReader::FillDataEx": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankReader *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkBankReader::FinalizeStreamInit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBankReader::GetData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankReader *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBankReader::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankReader *
        ]
    ),
    "CAkBankReader::ReleaseData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankReader *
        ]
    ),
    "CAkBankReader::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankReader *
        ]
    ),
    "CAkBankReader::SetBankLoadIOSettings": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankReader *
            ctypes.c_float,  # float
            ctypes.c_char,  # char
        ]
    ),
    "CAkBankReader::SetFile": {
        "CAkBankReader *, __int64, unsigned int, unsigned int, void *, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # CAkBankReader *
                ctypes.c_int64,  # __int64
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # void *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "CAkBankReader *, const char *, unsigned int, void *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # CAkBankReader *
                ctypes.c_char_p,  # const char *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # void *
            ]
        ),
        "CAkBankReader *, const void *, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # CAkBankReader *
                ctypes.c_ulonglong,  # const void *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "CAkBankReader::Skip": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankReader *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkBankReader::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankReader *
        ]
    ),
    "CAkBankMgr::ReadSourceParent<CAkSound>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # const AkBank::AKBKSubHircSection *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBankMgr::StdBankRead<CAkActorMixer,CAkParameterNodeBase>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # const AkBank::AKBKSubHircSection *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ulonglong,  # CAkIndexItem<CAkParameterNodeBase *> *
        ]
    ),
    "CAkBankMgr::StdBankRead<CAkAttenuation,CAkAttenuation>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # const AkBank::AKBKSubHircSection *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ulonglong,  # CAkIndexItem<CAkAttenuation *> *
        ]
    ),
    "CAkBankMgr::StdBankRead<CAkAudioDevice,CAkAudioDevice>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # const AkBank::AKBKSubHircSection *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ulonglong,  # CAkIndexItem<CAkAudioDevice *> *
        ]
    ),
    "CAkBankMgr::StdBankRead<CAkAuxBus,CAkParameterNodeBase>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # const AkBank::AKBKSubHircSection *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ulonglong,  # CAkIndexItem<CAkParameterNodeBase *> *
        ]
    ),
    "CAkBankMgr::StdBankRead<CAkDialogueEvent,CAkDialogueEvent>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # const AkBank::AKBKSubHircSection *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ulonglong,  # CAkIndexItem<CAkDialogueEvent *> *
        ]
    ),
    "CAkBankMgr::StdBankRead<CAkEnvelopeModulator,CAkModulator>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # const AkBank::AKBKSubHircSection *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ulonglong,  # CAkIndexItem<CAkModulator *> *
        ]
    ),
    "CAkBankMgr::StdBankRead<CAkFxCustom,CAkFxCustom>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # const AkBank::AKBKSubHircSection *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ulonglong,  # CAkIndexItem<CAkFxCustom *> *
        ]
    ),
    "CAkBankMgr::StdBankRead<CAkFxShareSet,CAkFxShareSet>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # const AkBank::AKBKSubHircSection *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ulonglong,  # CAkIndexItem<CAkFxShareSet *> *
        ]
    ),
    "CAkBankMgr::StdBankRead<CAkLFOModulator,CAkModulator>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # const AkBank::AKBKSubHircSection *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ulonglong,  # CAkIndexItem<CAkModulator *> *
        ]
    ),
    "CAkBankMgr::StdBankRead<CAkLayerCntr,CAkParameterNodeBase>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # const AkBank::AKBKSubHircSection *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ulonglong,  # CAkIndexItem<CAkParameterNodeBase *> *
        ]
    ),
    "CAkBankMgr::StdBankRead<CAkRanSeqCntr,CAkParameterNodeBase>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # const AkBank::AKBKSubHircSection *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ulonglong,  # CAkIndexItem<CAkParameterNodeBase *> *
        ]
    ),
    "CAkBankMgr::StdBankRead<CAkSwitchCntr,CAkParameterNodeBase>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # const AkBank::AKBKSubHircSection *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ulonglong,  # CAkIndexItem<CAkParameterNodeBase *> *
        ]
    ),
    "CAkBankMgr::StdBankRead<CAkTimeModulator,CAkModulator>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # const AkBank::AKBKSubHircSection *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ulonglong,  # CAkIndexItem<CAkModulator *> *
        ]
    ),
    "CAkBankMgr::CAkBankMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
        ]
    ),
    "CAkBankMgr::~CAkBankMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
        ]
    ),
    "AkMediaEntry::AddAlternateSource": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkMediaEntry *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkUsageSlot *
        ]
    ),
    "CAkBankMgr::AddLoadedItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ulonglong,  # CAkIndexable *
        ]
    ),
    "CAkBankMgr::AddMediaToTable": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # AkSourceSettings *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBankMgr::ClearBanksInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkBankMgr::AkBankQueueItem *
        ]
    ),
    "CAkBankMgr::ClearPreparedEvents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
        ]
    ),
    "AkFileNameString::Copy": {
        "AkFileNameString *, const char *, const char *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # AkFileNameString *
                ctypes.c_char_p,  # const char *
                ctypes.c_char_p,  # const char *
            ]
        ),
        "AkFileNameString *, const wchar_t *, const wchar_t *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # AkFileNameString *
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # const wchar_t *
            ]
        ),
    },
    "AkHashList<uint,AkMediaEntry,AkArrayAllocatorNoAlign<0>>::CreateEntry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkMediaEntry *
        argtypes=[
            ctypes.c_ulonglong,  # AkHashList<unsigned int,AkMediaEntry,AkArrayAllocatorNoAlign<0> > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CAkBankMgr::DecodeMedia": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkFileParser::FormatInfo *
        ]
    ),
    "CAkBankMgr::ExecuteCommand": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkBankMgr::AkBankQueueItem *
        ]
    ),
    "CAkBankMgr::GetBankInfoFromPtr": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkBankMgr::GetBufferSizeForDecodedMedia": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkFileParser::FormatInfo *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBus::GetDeviceShareset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
        ]
    ),
    "CAkBankMgr::GetMedia": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkMediaInfo *
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # AkMediaInfo *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkUsageSlot **
        ]
    ),
    "CAkBankMgr::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
        ]
    ),
    "CAkBankMgr::KillSlot": FUNCDEF(
        restype=ctypes.c_uint32,  # AKRESULT
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkBankMgr::KillSlotAsync": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkBankMgr::KillSlotSync": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkUsageSlot *
        ]
    ),
    "CAkBankMgr::LoadBank": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkBankMgr::AkBankQueueItem *
            ctypes.c_ulonglong,  # CAkUsageSlot **
            ctypes.c_uint32,  # CAkBankMgr::AkLoadBankDataMode
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkBankMgr::LoadMedia": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # CAkUsageSlot *
        ]
    ),
    "CAkBankMgr::LoadSingleMedia": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # AkSrcTypeInfo *
        ]
    ),
    "CAkBankMgr::LoadSource": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # AkBankSourceData *
        ]
    ),
    "LogWrongBankVersion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBankMgr::NotifyCompletion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkBankMgr::AkBankQueueItem *
            ctypes.c_uint32,  # AKRESULT
        ]
    ),
    "CAkBankMgr::PrepareBankInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkBankMgr::AkBankQueueItem *
            ctypes.c_uint32,  # CAkBankMgr::AkLoadBankDataMode
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkBankMgr::PrepareEvent": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkBankMgr::AkBankQueueItem *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBankMgr::PrepareEventInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkBankMgr::AkBankQueueItem *
            ctypes.c_ulonglong,  # CAkEvent *
        ]
    ),
    "CAkBankMgr::PrepareGameSync": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkBankMgr::AkBankQueueItem *
        ]
    ),
    "CAkBankMgr::PrepareMedia": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkBankMgr::PrepareSingleMedia": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # AkSrcTypeInfo *
        ]
    ),
    "CAkBankMgr::ProcessAccumulated": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
        ]
    ),
    "CAkBankMgr::ProcessCustomPlatformChunk": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBankMgr::ProcessGlobalSettingsChunk": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBankMgr::ProcessHircChunk": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBankMgr::ProcessPluginChunk": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBankMgr::ProcessStringMappingChunk": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkUsageSlot *
        ]
    ),
    "CAkBankMgr::QueueBankCommand": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkBankMgr::AkBankQueueItem *
        ]
    ),
    "CAkBankMgr::ReadAction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # const AkBank::AKBKSubHircSection *
            ctypes.c_ulonglong,  # CAkUsageSlot *
        ]
    ),
    "CAkBankMgr::ReadBus": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # const AkBank::AKBKSubHircSection *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkUsageSlot::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkBankMgr::ReleaseMedia": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBankMgr::ReleaseMediaHashTableEntry": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # AkMediaEntry *
        ]
    ),
    "CAkUsageSlot::ReleasePrepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkBankMgr::ReleaseSingleMedia": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkMediaEntry::RemoveAlternateSourceWithoutSlot": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkMediaEntry *
            ctypes.c_ulonglong,  # AkSourceSettings *
        ]
    ),
    "CAkUsageSlot::RemoveContent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkUsageSlot *
        ]
    ),
    "CAkBankMgr::RemoveMediaFromTable": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # AkSourceSettings *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkHashList<uint,char *,AkArrayAllocatorNoAlign<0>>::Resize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkHashList<unsigned int,AkMediaEntry,AkArrayAllocatorNoAlign<0> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkHashList<uint,AkMediaEntry,AkArrayAllocatorNoAlign<0>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkMediaEntry *
        argtypes=[
            ctypes.c_ulonglong,  # AkHashList<unsigned int,AkMediaEntry,AkArrayAllocatorNoAlign<0> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkFileNameString::Set": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkFileNameString *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "CAkBankMgr::SignalLastBankUnloaded": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "AkFileNameString::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkFileNameString *
        ]
    ),
    "CAkBankMgr::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
        ]
    ),
    "CAkBankMgr::TryRemoveMediaFromTable": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # AkSourceSettings *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AKRESULT *
        ]
    ),
    "CAkBankMgr::UnPrepareAllBank": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
        ]
    ),
    "CAkBankMgr::UnPrepareMedia": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkUsageSlot *
        ]
    ),
    "CAkBankMgr::UnloadAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
        ]
    ),
    "CAkBankMgr::UnloadBankPre": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkBankMgr::AkBankQueueItem *
        ]
    ),
    "CAkUsageSlot::UnloadCompletionNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkUsageSlot *
        ]
    ),
    "CAkBankMgr::UnloadMedia": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkUsageSlot *
        ]
    ),
    "CAkBankMgr::UnprepareEvent": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBankMgr::UnprepareEventInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # CAkEvent *
        ]
    ),
    "CAkBankMgr::UnprepareSingleMedia": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkHashList<uint,AkMediaEntry,AkArrayAllocatorNoAlign<0>>::Unset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkHashList<unsigned int,AkMediaEntry,AkArrayAllocatorNoAlign<0> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBankMgr::WaitForSyncOp": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankMgr *
            ctypes.c_ulonglong,  # AK::SoundEngine::AkSyncCaller *
            ctypes.c_uint32,  # AKRESULT
        ]
    ),
    "CAkStateAware::CAkStateAware": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
        ]
    ),
    "CAkStateAware::~CAkStateAware": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
        ]
    ),
    "AkStateGroupChunk::AddState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkStateGroupChunk *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkStateAware::AddStateGroup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkStateGroupChunk *
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkStateAware::CheckSyncTypes": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkStateAware::CAkStateSyncArray *
        ]
    ),
    "AkStateGroupChunk::FlushStateTransitions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkStateGroupChunk *
        ]
    ),
    "CAkStateAware::FlushStateTransitions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
        ]
    ),
    "AkStateGroupChunk::GetState": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkState_vtbl *
        argtypes=[
            ctypes.c_ulonglong,  # AkStateGroupChunk *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkStateAware::HasState": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkStateAware::IsStateProperty": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkStateAware::NotifyStateParametersModified": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
        ]
    ),
    "CAkStateAware::PauseTransitions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkStateAware::ReadStateChunk": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AkStateGroupChunk::RemoveState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkStateGroupChunk *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkStateAware::RemoveStateGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkStateAware::RemoveStateGroups": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkArray<AkStatePropertyInfo,AkStatePropertyInfo &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkStatePropertyInfo>>::Resize": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkStatePropertyInfo,AkStatePropertyInfo &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkStatePropertyInfo> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkStateAware::SetStateProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkStatePropertyUpdate *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkStateGroupChunk::TransUpdateValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkStateGroupChunk *
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkStateAware::UpdateStateGroups": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkStateGroupUpdate *
            ctypes.c_ulonglong,  # AkStateUpdate *
        ]
    ),
    "CAkStateAware::UseState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkPBIAware::~CAkPBIAware": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBIAware *
        ]
    ),
    "CAkPBIAware::CreatePBI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBIAware *
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_ulonglong,  # CAkSource *
            ctypes.c_ulonglong,  # AkPBIParams *
            ctypes.c_ulonglong,  # const PriorityInfoCurrent *
        ]
    ),
    "CAkParamNodeStateAware::CAkParamNodeStateAware": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParamNodeStateAware *
        ]
    ),
    "CAkParamNodeStateAware::~CAkParamNodeStateAware": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParamNodeStateAware *
        ]
    ),
    "CAkParamNodeStateAware::EnsureStateData": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkParamNodeStateAware *
        ]
    ),
    "CAkParamNodeStateAware::IsStateProperty": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkParamNodeStateAware *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParamNodeStateAware::NotifyStateParamTargets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParamNodeStateAware *
        ]
    ),
    "CAkParamNodeStateAware::PushStateParamUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParamNodeStateAware *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParamNodeStateAware::ReadStateChunk": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParamNodeStateAware *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkParamNodeStateAware::SetStateProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParamNodeStateAware *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkStatePropertyUpdate *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParamNodeStateAware::StateTransitionAdded": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParamNodeStateAware *
        ]
    ),
    "CAkParamNodeStateAware::StateTransitionRemoved": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParamNodeStateAware *
        ]
    ),
    "CAkParamNodeStateAware::UpdateStateParamTargets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParamNodeStateAware *
        ]
    ),
    "CAkParameterNode::CAkParameterNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParameterNode::~CAkParameterNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
        ]
    ),
    "CAkParameterNode::ApplyMaxNumInstances": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkParameterNode::AssociateLayer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # CAkLayer *
        ]
    ),
    "CreateDelayedAction": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
        argtypes=[
            ctypes.c_ulonglong,  # ContParams *
            ctypes.c_ulonglong,  # AkPBIParams *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CreateDelayedPendingAction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # ContParams *
            ctypes.c_ulonglong,  # AkPBIParams *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
        ]
    ),
    "CAkParameterNode::DecrementPlayCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # CounterParameters *
        ]
    ),
    "CAkParameterNode::DelayPlayback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # AkPBIParams *
        ]
    ),
    "CAkParameterNode::DissociateLayer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # CAkLayer *
        ]
    ),
    "CAkParameterNode::ExecuteActionExceptParentCheck": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # ActionParamsExcept *
        ]
    ),
    "CAkParameterNode::FilterAndTransformMidiEvent": FUNCDEF(
        restype=ctypes.c_uint32,  # AKRESULT
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # AkMidiEventEx *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParameterNode::GetAudioParameters": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # AkSoundParams *
            ctypes.c_ulonglong,  # AkMutedMap *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # AkPBIModValues *
            ctypes.c_ulonglong,  # AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParameterNode::GetFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkFXDesc *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkParameterNode::GetFXDataID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkParameterNode::GetMidiNoteOffAction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
        ]
    ),
    "CAkParameterNode::GetMidiNoteOnAction": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
        ]
    ),
    "CAkParameterNode::GetMidiPlayOnNoteType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
        ]
    ),
    "CAkParameterNode::GetVirtualBehavior": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # AkVirtualQueueBehavior *
        ]
    ),
    "CAkParameterNode::HandleInitialDelay": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # AkPBIParams *
        ]
    ),
    "CAkParameterNode::IncrementLESyncCount": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkParameterNode::IncrementPlayCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # CounterParameters *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParameterNode::IsOrIsChildOf": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParameterNode::Mute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # AkCurveInterpolation
            ctypes.c_int32,  # int
        ]
    ),
    "CAkParameterNode::PlayAndContinueAlternate": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # AkPBIParams *
        ]
    ),
    "AkTimeConv__SecondsToSamples": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_double,  # long double
        ]
    ),
    "CAkParameterNode::SetAdvSettingsParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkParameterNode::SetAkProp": {
        "CAkParameterNode *, AkPropID, int, int, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkParameterNode *
                ctypes.c_uint32,  # AkPropID
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
            ]
        ),
        "CAkParameterNode *, AkPropID, float, float, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkParameterNode *
                ctypes.c_uint32,  # AkPropID
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
        "CAkParameterNode *, AkPropID, CAkRegisteredObj *, AkValueMeaning, float, AkCurveInterpolation, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkParameterNode *
                ctypes.c_uint32,  # AkPropID
                ctypes.c_ulonglong,  # CAkRegisteredObj *
                ctypes.c_uint32,  # AkValueMeaning
                ctypes.c_float,  # float
                ctypes.c_uint32,  # AkCurveInterpolation
                ctypes.c_int32,  # int
            ]
        ),
    },
    "CAkParameterNode::SetInitialFxParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParameterNode::SetInitialMetadataParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkParameterNode::SetInitialParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkParameterNode::TriggerModulators": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # const AkModulatorTriggerParams *
            ctypes.c_ulonglong,  # CAkModulatorData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkParameterNode::UnmuteAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_uint32,  # AkCurveInterpolation
            ctypes.c_int32,  # int
        ]
    ),
    "CAkParameterNode::UnmuteAllObj": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_uint32,  # AkCurveInterpolation
            ctypes.c_int32,  # int
        ]
    ),
    "CAkModulatorData::CAkModulatorData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorData *
        ]
    ),
    "CAkModCtxRefContainer::~CAkModCtxRefContainer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModCtxRefContainer *
        ]
    ),
    "CAkModulatorData::~CAkModulatorData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorData *
        ]
    ),
    "CAkModulatorPBIData::AddModulationSource": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorPBIData *
            ctypes.c_ulonglong,  # CAkModulatorCtx *
            ctypes.c_ulonglong,  # const AkModulatorTriggerParams *
            ctypes.c_ulonglong,  # const AkModulatorSubscriberInfo *
        ]
    ),
    "CAkModulatorData::AllocData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorData *
        ]
    ),
    "CAkModulatorPBIData::ClearModulationSources": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorPBIData *
        ]
    ),
    "CAkModulatorPBIData::GetBufferList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorPBIData *
            ctypes.c_ulonglong,  # const CAkBitArray<unsigned __int64,0> *
            ctypes.c_ulonglong,  # AkModulatorXfrm *
            ctypes.c_ulonglong,  # AkModulatorGain *
            ctypes.c_ulonglong,  # float **
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkModulatorPBIData::GetCtrlRateOutput": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorPBIData *
            ctypes.c_ulonglong,  # const CAkBitArray<unsigned __int64,0> *
        ]
    ),
    "CAkModulatorPBIData::GetModulatorCtxs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorPBIData *
            ctypes.c_uint32,  # AkModulatorScope
            ctypes.c_ulonglong,  # CAkModCtxRefContainer *
        ]
    ),
    "CAkModulatorPBIData::GetNumAutomatedModulators": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorPBIData *
            ctypes.c_ulonglong,  # const CAkBitArray<unsigned __int64,0> *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkModulatorPBIData::GetPeakOutput": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorPBIData *
            ctypes.c_ulonglong,  # const CAkBitArray<unsigned __int64,0> *
        ]
    ),
    "CAkModulatorPBIData::HasModulationSource": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorPBIData *
            ctypes.c_ulonglong,  # CAkModulatorCtx *
        ]
    ),
    "CAkModCtxRefContainer::HasModulatorCtx": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkModCtxRefContainer *
            ctypes.c_ulonglong,  # CAkModulatorCtx *
        ]
    ),
    "CAkModulatorPBIData::Pause": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorPBIData *
        ]
    ),
    "CAkModCtxRefContainer::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModCtxRefContainer *
        ]
    ),
    "CAkModulatorPBIData::Resume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorPBIData *
        ]
    ),
    "CAkModulatorPBIData::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorPBIData *
        ]
    ),
    "CAkModulatorPBIData::TriggerRelease": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorPBIData *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSwitchAware::CAkSwitchAware": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchAware *
        ]
    ),
    "CAkSwitchAware::~CAkSwitchAware": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchAware *
        ]
    ),
    "CAkSwitchAware::GetSwitchToUse": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchAware *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkGroupType
        ]
    ),
    "CAkSwitchAware::SubscribeSwitch": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchAware *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkGroupType
        ]
    ),
    "CAkSwitchAware::UnsubscribeSwitches": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchAware *
        ]
    ),
    "CAkEmitter::CAkEmitter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkEmitter *
        ]
    ),
    "CAkModifiedNodes::CAkModifiedNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModifiedNodes *
        ]
    ),
    "AkPositionStore::~AkPositionStore": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkPositionStore *
        ]
    ),
    "CAkEmitter::~CAkEmitter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkEmitter *
        ]
    ),
    "CAkModifiedNodes::~CAkModifiedNodes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModifiedNodes *
        ]
    ),
    "CAkRegisteredObj::~CAkRegisteredObj": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkRegisteredObj::GetConnectedListeners": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkConnectedListeners *
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkRegisteredObj::GetListeners": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkRegisteredObj::GetNumEmitterListenerPairs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "AkArray<MapStruct<unsigned __int64,AkPerLstnrObjData>,MapStruct<unsigned __int64,AkPerLstnrObjData> const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<MapStruct<unsigned __int64,AkPerLstnrObjData>>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<MapStruct<unsigned __int64,AkPerLstnrObjData>,MapStruct<unsigned __int64,AkPerLstnrObjData> const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<MapStruct<unsigned __int64,AkPerLstnrObjData> > > *
        ]
    ),
    "AkArray<AkGameRayParams,AkGameRayParams const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameRayParams>>::Resize": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkGameRayParams,AkGameRayParams const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameRayParams> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkKeyArray<unsigned __int64,AkPerLstnrObjData,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<MapStruct<unsigned __int64,AkPerLstnrObjData>>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkPerLstnrObjData *
        argtypes=[
            ctypes.c_ulonglong,  # CAkKeyArray<unsigned __int64,AkPerLstnrObjData,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<MapStruct<unsigned __int64,AkPerLstnrObjData> > > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CAkEmitter::SetMultipleObstructionAndOcclusion": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkEmitter *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # AkObstructionOcclusionValues *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkModifiedNodes::SetNodeAsModified": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkModifiedNodes *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkEmitter::SetObjectObstructionAndOcclusion": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkEmitter *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "AkPositionStore::SetObstructionOcclusion": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkPositionStore *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # AkObstructionOcclusionValues *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkEmitter::SetPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkEmitter *
            ctypes.c_ulonglong,  # const AkChannelEmitter *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AK::SoundEngine::MultiPositionType
        ]
    ),
    "MapStruct<unsigned __int64,AkPerLstnrObjData>::Transfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # MapStruct<unsigned __int64,AkPerLstnrObjData> *
            ctypes.c_ulonglong,  # MapStruct<unsigned __int64,AkPerLstnrObjData> *
        ]
    ),
    "CAkRegisteredObj::UpdateBuiltIn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # AkBuiltInParam
        ]
    ),
    "CAkEmitter::UpdateCachedPositions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkEmitter *
        ]
    ),
    "CAkRTPCMgr::CAkRTPCMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
        ]
    ),
    "CAkRTPCMgr::AddBuiltInParamBinding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_uint32,  # AkBuiltInParam
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkRTPCMgr::AkRTPCEntry::AddRefSubcription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCEntry *
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCSubscription *
        ]
    ),
    "AkSortedKeyArray<uchar,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>::ChildTreeType>,AkDefaultSortedKeyCompare<uchar>>::BinarySearch": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned char,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType>,AkDefaultSortedKeyCompare<unsigned char> > *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "?BinarySearch@?$AkSortedKeyArray@IVChildTreeType@?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkRTPCValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkRTPCValue@@@@@@@@@@@@U?$AkArrayAllocatorNoAlign@$0A@@@V12@UAkGrowByPolicy_Proportional@@U?$AkTransferMovePolicy@VChildTreeType@?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkRTPCValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkRTPCValue@@@@@@@@@@@@@@U?$AkDefaultSortedKeyCompare@I@@@@QEBAPEAVChildTreeType@?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkRTPCValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkRTPCValue@@@@@@@@@@@@IAEA_N@Z": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType>,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AkSortedKeyArray<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>::RootValueType,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>::RootValueType>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>::RootValueType>,AkDefaultSortedKeyCompare<CAkPBI *>>::BinarySearch": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType>,AkDefaultSortedKeyCompare<CAkPBI *> > *
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "?BinarySearch@?$AkSortedKeyArray@PEAVCAkRegisteredObj@@VChildTreeType@?$AkRTPCNestedSearchTree@VAkRTPCKey@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkRTPCValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkRTPCValue@@@@@@@@@@@@@@U?$AkArrayAllocatorNoAlign@$0A@@@V23@UAkGrowByPolicy_Proportional@@U?$AkTransferMovePolicy@VChildTreeType@?$AkRTPCNestedSearchTree@VAkRTPCKey@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkRTPCValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkRTPCValue@@@@@@@@@@@@@@@@U?$AkDefaultSortedKeyCompare@PEAVCAkRegisteredObj@@@@@@QEBAPEAVChildTreeType@?$AkRTPCNestedSearchTree@VAkRTPCKey@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkRTPCValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkRTPCValue@@@@@@@@@@@@@@PEAVCAkRegisteredObj@@AEA_N@Z": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkRegisteredObj *,AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > >::ChildTreeType>,AkDefaultSortedKeyCompare<CAkRegisteredObj *> > *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkRTPCMgr::AkRTPCSubscription::ConvertCurves": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCSubscription *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "CAkConversionTable::CopyPoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConversionTable *
            ctypes.c_ulonglong,  # AkGraphPointBase<float> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkCurveScaling
        ]
    ),
    "CAkRTPCMgr::AkRTPCEntry::CreateOrModifyTransition": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCEntry *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # TransParamsBase *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkHashListBare<uint,CAkRTPCMgr::AkRTPCEntry,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<uint,CAkRTPCMgr::AkRTPCEntry>>::Exists": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCEntry *
        argtypes=[
            ctypes.c_ulonglong,  # AkHashListBare<unsigned int,CAkRTPCMgr::AkRTPCEntry,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<unsigned int,CAkRTPCMgr::AkRTPCEntry> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkRTPCMgr::GetDefaultValue": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CurrentValue::GetModulatorOutput": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "GetNumberOfRTPC": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 **
        ]
    ),
    "CAkRTPCMgr::AkRTPCSubscription::GetParamValues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCSubscription *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkRTPCMgr::GetRTPCConvertedValue": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkRTPCMgr::GetRTPCEntry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCEntry *
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkRTPCMgr::GetSubscription": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCSubscription *
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkArray<CAkRTPCMgr::AkSubsPtr,CAkRTPCMgr::AkSubsPtr const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkRTPCMgr::AkSubsPtr>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<CAkRTPCMgr::AkSubsPtr,CAkRTPCMgr::AkSubsPtr const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkRTPCMgr::AkSubsPtr> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkArray<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>::ChildTreeType,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>::ChildTreeType>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > >::ChildTreeType,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > >::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > >::ChildTreeType> > *
        ]
    ),
    "AkArray<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>::ChildTreeType,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>::ChildTreeType>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType> > *
        ]
    ),
    "AkArray<AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>::ChildTreeType,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>::ChildTreeType>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType> > *
        ]
    ),
    "AkArray<AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>>::ChildTreeType,AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>>::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>>::ChildTreeType>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > >::ChildTreeType,AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > >::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > >::ChildTreeType> > *
        ]
    ),
    "CAkRTPCMgr::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
        ]
    ),
    "AkRTPCExceptionChecker_<AkRTPCKeyTree<AkRTPCValue>>::IsException": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker_<AkRTPCKeyTree<AkRTPCValue> > *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "AkNestedKey<CAkRegisteredObj *,GetNullGameObjPtr,AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>>::MatchValidFields": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkNestedKey<CAkRegisteredObj *,GetNullGameObjPtr,AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > > > *
            ctypes.c_ulonglong,  # const AkNestedKey<CAkRegisteredObj *,GetNullGameObjPtr,AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > > > *
        ]
    ),
    "CurrentValue::ModulatorSupportsAutomatedParams": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkRTPCMgr::AkRTPCEntry::Move": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCEntry *
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCSubscription *
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkRTPCMgr::AkSubsPtr,CAkRTPCMgr::AkSubsPtr,AkArrayAllocatorNoAlign<0>,CAkRTPCMgr::AkSubsPtr,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkRTPCMgr::AkSubsPtr>,AkDefaultSortedKeyCompare<CAkRTPCMgr::AkSubsPtr> > *
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkRTPCMgr::AkSubsPtr,CAkRTPCMgr::AkSubsPtr,AkArrayAllocatorNoAlign<0>,CAkRTPCMgr::AkSubsPtr,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkRTPCMgr::AkSubsPtr>,AkDefaultSortedKeyCompare<CAkRTPCMgr::AkSubsPtr> > *
        ]
    ),
    "CAkRTPCMgr::AkRTPCEntry::NotifyRTPCChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCEntry *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkRTPCMgr::AkRTPCSubscription::PushUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCSubscription *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "CAkRTPCMgr::RegisterLayer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_ulonglong,  # CAkLayer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkRtpcType
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkRTPCMgr::RegisterSwitchGroup": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkRtpcType
            ctypes.c_ulonglong,  # AkGraphPointBase<float> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>::RemoveAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > *
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>::RemoveAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > *
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>::RemoveAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > *
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>::RemoveAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > *
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>>::RemoveAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > > *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>::RemoveAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> *
            ctypes.c_ulonglong,  # const AkRootKey<CAkPBI *,GetNullPbiPtr> *
        ]
    ),
    "CAkRTPCMgr::RemoveBuiltInParamBindings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkRTPCMgr::AkRTPCEntry::RemoveMatchingValues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCEntry *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkRTPCMgr::RemoveReferencesToSubscription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCSubscription *
        ]
    ),
    "CAkRTPCMgr::AkRTPCEntry::RemoveSubscriptionIfNoCurvesRemain": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCEntry *
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCSubscription *
        ]
    ),
    "CAkRTPCMgr::AkRTPCEntry::RemoveValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCEntry *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkRTPCMgr::RemovedScopedRtpcObj": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkRTPCMgr::ResetRTPCValue": {
        "CAkRTPCMgr *, unsigned int, const AkRTPCKey *, TransParamsBase *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkRTPCMgr *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # const AkRTPCKey *
                ctypes.c_ulonglong,  # TransParamsBase *
            ]
        ),
        "CAkRTPCMgr *, unsigned int, CAkRegisteredObj *, unsigned int, TransParamsBase *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkRTPCMgr *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # CAkRegisteredObj *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # TransParamsBase *
            ]
        ),
    },
    "AkHashListBare<uint,CAkRTPCMgr::AkRTPCEntry,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<uint,CAkRTPCMgr::AkRTPCEntry>>::Resize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkHashListBare<unsigned int,CAkRTPCMgr::AkRTPCEntry,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<unsigned int,CAkRTPCMgr::AkRTPCEntry> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkHashListBare<AkRTPCSubscriptionKey,CAkRTPCMgr::AkRTPCSubscription,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<AkRTPCSubscriptionKey,CAkRTPCMgr::AkRTPCSubscription>>::Resize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkHashListBare<AkRTPCSubscriptionKey,CAkRTPCMgr::AkRTPCSubscription,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<AkRTPCSubscriptionKey,CAkRTPCMgr::AkRTPCSubscription> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkHashListBare<uint,CAkRTPCMgr::AkRTPCEntry,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<uint,CAkRTPCMgr::AkRTPCEntry>>::Set": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkHashListBare<unsigned int,CAkRTPCMgr::AkRTPCEntry,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<unsigned int,CAkRTPCMgr::AkRTPCEntry> > *
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCEntry *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCValue *
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > *
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > *
        ]
    ),
    "AkSortedKeyArray<uchar,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>::ChildTreeType>,AkDefaultSortedKeyCompare<uchar>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned char,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > >::ChildTreeType>,AkDefaultSortedKeyCompare<unsigned char> > *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AkSortedKeyArray<uchar,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>::ChildTreeType>,AkDefaultSortedKeyCompare<uchar>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned char,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType>,AkDefaultSortedKeyCompare<unsigned char> > *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AkSortedKeyArray<uint,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>::ChildTreeType>,AkDefaultSortedKeyCompare<uint>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType>,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AkSortedKeyArray<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>::RootValueType,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>::RootValueType>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>::RootValueType>,AkDefaultSortedKeyCompare<CAkPBI *>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>::RootValueType,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>::RootValueType>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>::RootValueType>,AkDefaultSortedKeyCompare<CAkPBI *> > *
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "?Set@?$AkSortedKeyArray@PEAVCAkRegisteredObj@@VChildTreeType@?$AkRTPCNestedSearchTree@VAkRTPCKey@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkRTPCValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkRTPCValue@@@@@@@@@@@@@@U?$AkArrayAllocatorNoAlign@$0A@@@V23@UAkGrowByPolicy_Proportional@@U?$AkTransferMovePolicy@VChildTreeType@?$AkRTPCNestedSearchTree@VAkRTPCKey@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkRTPCValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkRTPCValue@@@@@@@@@@@@@@@@U?$AkDefaultSortedKeyCompare@PEAVCAkRegisteredObj@@@@@@QEAAPEAVChildTreeType@?$AkRTPCNestedSearchTree@VAkRTPCKey@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkRTPCValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkRTPCValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkRTPCValue@@@@@@@@@@@@@@PEAVCAkRegisteredObj@@AEA_N@Z": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkRegisteredObj *,AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > >::ChildTreeType>,AkDefaultSortedKeyCompare<CAkRegisteredObj *> > *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkRTPCMgr::SetBuiltInParamValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_uint32,  # AkBuiltInParam
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_float,  # float
        ]
    ),
    "CAkRTPCMgr::SetDefaultParamValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "CAkRTPCMgr::SetMidiParameterValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_uint32,  # AkAssignableMidiControl
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkRTPCMgr::AkRTPCEntry::SetRTPC": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCEntry *
            ctypes.c_ulonglong,  # AkRTPCValue *
            ctypes.c_ulonglong,  # AkRTPCValue *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # TransParamsBase *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkRTPCMgr::SetRTPCInternal": {
        "CAkRTPCMgr *, unsigned int, float, const AkRTPCKey *, TransParamsBase *, AkValueMeaning, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # CAkRTPCMgr *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # const AkRTPCKey *
                ctypes.c_ulonglong,  # TransParamsBase *
                ctypes.c_uint32,  # AkValueMeaning
                ctypes.c_ubyte,  # bool
            ]
        ),
        "CAkRTPCMgr *, unsigned int, float, CAkRegisteredObj *, unsigned int, TransParamsBase *, AkValueMeaning": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # CAkRTPCMgr *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # CAkRegisteredObj *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # TransParamsBase *
                ctypes.c_uint32,  # AkValueMeaning
            ]
        ),
    },
    "CAkRTPCMgr::SetRTPCRamping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkTransitionRampingType
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CAkRTPCMgr::SubscribeRTPC": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkRtpcType
            ctypes.c_uint32,  # AkRtpcAccum
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkCurveScaling
            ctypes.c_ulonglong,  # AkGraphPointBase<float> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_uint32,  # CAkRTPCMgr::SubscriberType
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkArray<AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>::ChildTreeType,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>::ChildTreeType>>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType> > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > > *
        ]
    ),
    "AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> *
        ]
    ),
    "CAkRTPCMgr::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
        ]
    ),
    "CAkRTPCMgr::ToggleLayerSubscription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_ulonglong,  # CAkLayer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkRTPCMgr::CAkRTPCTransition::TransUpdateValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr::CAkRTPCTransition *
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>::ChildTreeType::Transfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType *
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType *
        ]
    ),
    "CAkRTPCMgr::UnSubscribeRTPC": {
        "CAkRTPCMgr *, void *, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkRTPCMgr *
                ctypes.c_ulonglong,  # void *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "CAkRTPCMgr *, void *, unsigned int, unsigned int, bool *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkRTPCMgr *
                ctypes.c_ulonglong,  # void *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # bool *
            ]
        ),
    },
    "CAkRTPCMgr::UnregisterSwitchGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>::Unset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > *
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>::Unset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > *
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>>::Unset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > > *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "AkSortedKeyArray<uchar,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>::ChildTreeType>,AkDefaultSortedKeyCompare<uchar>>::Unset": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned char,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > >::ChildTreeType>,AkDefaultSortedKeyCompare<unsigned char> > *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "AkSortedKeyArray<uchar,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>::ChildTreeType>,AkDefaultSortedKeyCompare<uchar>>::Unset": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned char,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> >::ChildTreeType>,AkDefaultSortedKeyCompare<unsigned char> > *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "AkSortedKeyArray<uint,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>::ChildTreeType>,AkDefaultSortedKeyCompare<uint>>::Unset": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > >::ChildTreeType>,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkSortedKeyArray<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>::RootValueType,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>::RootValueType>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>::RootValueType>,AkDefaultSortedKeyCompare<CAkPBI *>>::Unset": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType>,AkDefaultSortedKeyCompare<CAkPBI *> > *
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "AkSortedKeyArray<CAkRegisteredObj *,AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>>::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>>::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>>::ChildTreeType>,AkDefaultSortedKeyCompare<CAkRegisteredObj *>>::Unset": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkRegisteredObj *,AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > >::ChildTreeType>,AkDefaultSortedKeyCompare<CAkRegisteredObj *> > *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkRTPCMgr::UpdateSubscription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRTPCMgr *
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCSubscription *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>::_CheckException": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > *
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > *
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>>::_CheckException": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > > *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkIndexable::CAkIndexable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkIndexable *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkIndexable::~CAkIndexable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkIndexable *
        ]
    ),
    "CAkPreparationAware::GetPreparedContent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # MapStruct<unsigned int,AkStateGroupInfo *> *
        argtypes=[
            ctypes.c_ulonglong,  # CAkPreparationAware *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkGroupType
        ]
    ),
    "CAkPreparationAware::SubscribePrepare": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPreparationAware *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkGroupType
        ]
    ),
    "CAkPreparationAware::UnsubscribePrepare": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPreparationAware *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkGroupType
        ]
    ),
    "AkDecisionTree::AkDecisionTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDecisionTree *
        ]
    ),
    "AkDecisionTree::~AkDecisionTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDecisionTree *
        ]
    ),
    "AkDecisionTree::AddCandidate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDecisionTree *
            ctypes.c_ulonglong,  # AkDecisionTree::Node *
            ctypes.c_ulonglong,  # AkDecisionTree::WeightedCandidates *
        ]
    ),
    "AkDecisionTree::GetAudioNodeForState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkDecisionTree *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkDecisionTree::GetSwitchNodeAssoc": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # AkDecisionTree *
            ctypes.c_ulonglong,  # CAkKeyArray<unsigned int,unsigned int,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<unsigned int,unsigned int> > > *
        ]
    ),
    "AkDecisionTree::ResolvePath": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkDecisionTree *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': [{'type': 'unsigned', 'name': 'int'}, {'type': 'unsigned', 'name': 'int'}, 'void *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AkDecisionTree::ResolvePathWeighted": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkDecisionTree::Node *
        argtypes=[
            ctypes.c_ulonglong,  # AkDecisionTree *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkDecisionTree::WeightedDecisionInfo *
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': [{'type': 'unsigned', 'name': 'int'}, {'type': 'unsigned', 'name': 'int'}, 'void *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AkDecisionTree::SetTree": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkDecisionTree *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkDecisionTree::_ResolvePath": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkDecisionTree::Node *
        argtypes=[
            ctypes.c_ulonglong,  # AkDecisionTree *
            ctypes.c_ulonglong,  # AkDecisionTree::Node *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': [{'type': 'unsigned', 'name': 'int'}, {'type': 'unsigned', 'name': 'int'}, 'void *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AkDecisionTree::_ResolvePathWeighted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDecisionTree *
            ctypes.c_ulonglong,  # AkDecisionTree::Node *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkDecisionTree::WeightedCandidates *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': [{'type': 'unsigned', 'name': 'int'}, {'type': 'unsigned', 'name': 'int'}, 'void *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkSwitchMgr::CAkSwitchMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchMgr *
        ]
    ),
    "CAkSwitchMgr::SwitchingInThisScope::SwitchingInThisScope": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchMgr::SwitchingInThisScope *
        ]
    ),
    "CAkSwitchMgr::SwitchingInThisScope::~SwitchingInThisScope": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchMgr::SwitchingInThisScope *
        ]
    ),
    "AkSortedKeyArray<CAkSwitchMgr::AkSwitchSubscription *,CAkSwitchMgr::AkSwitchSubscription *,AkArrayAllocatorNoAlign<0>,CAkSwitchMgr::AkSubsPtrGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkSwitchMgr::AkSwitchSubscription *>,AkDefaultSortedKeyCompare<CAkSwitchMgr::AkSwitchSubscription *>>::AddNoSetKey": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkSwitchMgr::AkSwitchSubscription **
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkSwitchMgr::AkSwitchSubscription *,CAkSwitchMgr::AkSwitchSubscription *,AkArrayAllocatorNoAlign<0>,CAkSwitchMgr::AkSubsPtrGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkSwitchMgr::AkSwitchSubscription *>,AkDefaultSortedKeyCompare<CAkSwitchMgr::AkSwitchSubscription *> > *
            ctypes.c_ulonglong,  # CAkSwitchMgr::AkSwitchSubscription *
        ]
    ),
    "CAkSwitchMgr::AkSwitchEntry::AddRTPC": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchMgr::AkSwitchEntry *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkRtpcType
            ctypes.c_ulonglong,  # AkGraphPointBase<unsigned int> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSwitchMgr::AddSwitchRTPC": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkRtpcType
            ctypes.c_ulonglong,  # AkGraphPointBase<unsigned int> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue>>>>>>::FindBestMatch": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCValue *
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkRTPCKey,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkRTPCValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkRTPCValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkRTPCValue> > > > > > *
            ctypes.c_ulonglong,  # AkRTPCKey *
        ]
    ),
    "CAkSwitchMgr::GetSwitch": {
        "CAkSwitchMgr *, unsigned int, const AkRTPCKey *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # CAkSwitchMgr *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # const AkRTPCKey *
            ]
        ),
        "CAkSwitchMgr *, unsigned int, const AkRTPCKey *, unsigned int *": FUNCDEF(
            restype=ctypes.c_ubyte,  # bool
            argtypes=[
                ctypes.c_ulonglong,  # CAkSwitchMgr *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # const AkRTPCKey *
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
    },
    "CAkSwitchMgr::GetSwitchEntry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkSwitchMgr::AkSwitchEntry *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkArray<uint,uint &,AkArrayAllocatorNoAlign<2>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<uint>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<unsigned int,unsigned int &,AkArrayAllocatorNoAlign<2>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned int> > *
        ]
    ),
    "AkArray<CAkSwitchMgr::SubsAction,CAkSwitchMgr::SubsAction const &,AkHybridAllocator<384,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkSwitchMgr::SubsAction>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<CAkSwitchMgr::SubsAction,CAkSwitchMgr::SubsAction const &,AkHybridAllocator<384,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkSwitchMgr::SubsAction> > *
        ]
    ),
    "CAkSwitchMgr::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchMgr *
        ]
    ),
    "CAkSwitchMgr::RemoveSubscriptionFromEntry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchMgr *
            ctypes.c_ulonglong,  # CAkSwitchMgr::AkSwitchSubscription *
        ]
    ),
    "AkHashListBare<AkSwitchSubscriptionKey,CAkSwitchMgr::AkSwitchSubscription,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<AkSwitchSubscriptionKey,CAkSwitchMgr::AkSwitchSubscription>>::Resize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkHashListBare<AkSwitchSubscriptionKey,CAkSwitchMgr::AkSwitchSubscription,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<AkSwitchSubscriptionKey,CAkSwitchMgr::AkSwitchSubscription> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSwitchMgr::AkSwitchEntry::SetSwitchFromRTPCMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchMgr::AkSwitchEntry *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "CAkSwitchMgr::AkSwitchEntry::SetSwitchInternal": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchMgr::AkSwitchEntry *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkSwitchMgr::SubscribeSwitch": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchMgr *
            ctypes.c_ulonglong,  # CAkSwitchAware *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSwitchMgr::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchMgr *
        ]
    ),
    "CAkSwitchMgr::UnSubscribeSwitch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchMgr *
            ctypes.c_ulonglong,  # CAkSwitchAware *
        ]
    ),
    "CAkSwitchMgr::UnregisterGameObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchMgr *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkSwitchMgr::_SubscribeSwitch": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchMgr *
            ctypes.c_ulonglong,  # CAkSwitchAware *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSwitchMgr::_UnSubscribeSwitch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchMgr *
            ctypes.c_ulonglong,  # CAkSwitchAware *
        ]
    ),
    "CAkSource::CAkSource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSource *
        ]
    ),
    "CAkSource::~CAkSource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSource *
        ]
    ),
    "CAkSource::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkSource *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSource *
        ]
    ),
    "CAkSource::LockDataPtr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSource *
            ctypes.c_ulonglong,  # void **
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # CAkUsageSlot **
        ]
    ),
    "CAkSource::PrepareData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSource *
        ]
    ),
    "CAkSource::SetSource": {
        "CAkSource *, unsigned int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkSource *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "CAkSource *, unsigned int, unsigned int, const wchar_t *, unsigned int, bool, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkSource *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "CAkSource::UnLockDataPtr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSource *
        ]
    ),
    "CAkSource::UnPrepareData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSource *
        ]
    ),
    "CAkMidiClipCtx::CAkMidiClipCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipCtx *
            ctypes.c_ulonglong,  # CAkMusicCtx *
            ctypes.c_ulonglong,  # CAkMidiClipMgr *
            ctypes.c_ulonglong,  # CAkMusicTrack *
            ctypes.c_ulonglong,  # CAkMusicSource *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # TransParams *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_ulonglong,  # const AkTrackSrc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMidiClipCtx::CheckIsMuted": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipCtx *
        ]
    ),
    "CAkMidiClipCtx::GetAbsoluteStop": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipCtx *
        ]
    ),
    "CAkMidiClipCtx::GetCCEvents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipCtx *
            ctypes.c_ulonglong,  # AkListBare<AkMidiFrameEvent,AkListBareNextItem,AkCountPolicyNoCount,AkLastPolicyWithLast> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMidiClipCtx::GetMidiMgr": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMidiClipMgr *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipCtx *
        ]
    ),
    "CAkMidiClipCtx::GetTargetedParamsSet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkBitArray<unsigned __int64,0> *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipCtx *
            ctypes.c_ulonglong,  # CAkBitArray<unsigned __int64,0> *
        ]
    ),
    "CAkMidiClipCtx::GetTrackID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMusicTrack *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipCtx *
        ]
    ),
    "CAkMidiClipCtx::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipCtx *
        ]
    ),
    "CAkMidiClipCtx::IsUsingThisSlot": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipCtx *
            ctypes.c_ulonglong,  # const CAkUsageSlot *
        ]
    ),
    "CAkMidiClipCtx::MuteNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipCtx *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # AkMutedMapItem *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMidiClipCtx::OnFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipCtx *
            ctypes.c_ulonglong,  # AkListBare<AkMidiFrameEvent,AkListBareNextItem,AkCountPolicyNoCount,AkLastPolicyWithLast> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMidiClipCtx::OnLastFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipCtx *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMidiClipCtx::OnPaused": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipCtx *
        ]
    ),
    "CAkMidiClipCtx::OnResumed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipCtx *
        ]
    ),
    "CAkMidiClipCtx::RelocateMedia": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipCtx *
            ctypes.c_ulonglong,  # CAkUsageSlot *
        ]
    ),
    "CAkMidiClipCtx::ResolveMidiTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipCtx *
        ]
    ),
    "CAkMidiClipCtx::VirtualAddRef": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipCtx *
        ]
    ),
    "CAkMidiClipCtx::VirtualRelease": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiClipCtx *
        ]
    ),
    "CAkBus::CAkBus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkParentNode<CAkParameterNodeBase>::~CAkParentNode<CAkParameterNodeBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParentNode<CAkParameterNodeBase> *
        ]
    ),
    "CAkBus::~CAkBus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
        ]
    ),
    "CAkParentNode<CAkParameterNodeBase>::AddChildInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParentNode<CAkParameterNodeBase> *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkBus::AddChildInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkBus::AddStateGroup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkStateGroupChunk *
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkBus::ApplyMaxNumInstances": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkBus::BackgroundMusic_Unmute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkBus::CanAddChild": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkParentNode<CAkParameterNodeBase>::Children": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkParentNode<CAkParameterNodeBase> *
        ]
    ),
    "CAkBus::Children": FUNCDEF(
        restype=ctypes.c_uint16,  # unsigned __int16
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
        ]
    ),
    "CAkBus::ClampWindowTop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkActiveParent<CAkParameterNodeBase>::ClearLimiters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNodeBase> *
        ]
    ),
    "CAkBus::ClearLimiters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
        ]
    ),
    "CAkBus::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkBus *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBus::DecrementActivityCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkBus::DecrementPlayCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # CounterParameters *
        ]
    ),
    "CAkBus::Duck": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
            ctypes.c_uint32,  # AkPropID
        ]
    ),
    "CAkBus::DuckNotif": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
        ]
    ),
    "CAkActiveParent<CAkParameterNodeBase>::ExecuteAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNodeBase> *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkBus::ExecuteAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkActiveParent<CAkParameterNodeBase>::ExecuteActionExcept": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNodeBase> *
            ctypes.c_ulonglong,  # ActionParamsExcept *
        ]
    ),
    "CAkBus::ExecuteActionExcept": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # ActionParamsExcept *
        ]
    ),
    "CAkBus::ExecuteMasterBusAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkBus::ExecuteMasterBusActionExcept": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ActionParamsExcept *
        ]
    ),
    "CAkActiveParent<CAkParameterNodeBase>::ForAllPBI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNodeBase> *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['CAkPBI *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkBus::ForAllPBI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['CAkPBI *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkParentNode<CAkParameterNodeBase>::GatherSounds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParentNode<CAkParameterNodeBase> *
            ctypes.c_ulonglong,  # AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *
            ctypes.c_ulonglong,  # AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *
            ctypes.c_ulonglong,  # AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBus::GetAudioParameters": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # AkSoundParams *
            ctypes.c_ulonglong,  # AkMutedMap *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # AkPBIModValues *
            ctypes.c_ulonglong,  # AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkBus::GetBypassAllFX": FUNCDEF(
        restype=ctypes.c_int16,  # __int16
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkParentNode<CAkParameterNodeBase>::GetChildren": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParentNode<CAkParameterNodeBase> *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # AkObjectInfo *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBus::GetControlBusParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # AkSoundParams *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkBus::GetDuckedVolume": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # AkPropID
        ]
    ),
    "CAkBus::GetFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkFXDesc *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkBus::GetFXDataID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkBus::GetGameParamID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
        ]
    ),
    "CAkBus::GetMixerPlugin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # AkFXDesc *
        ]
    ),
    "CAkBus::GetMixerPluginDataID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkBus::GetMixingBus": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkBus *
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
        ]
    ),
    "CAkBus::GetMixingBusParameters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # AkSoundParams *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkBus::GetNonMixingBusParameters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # AkSoundParams *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkBus::GetSIS": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkSIS *
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkBus::GetStateSyncTypes": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkStateAware::CAkStateSyncArray *
        ]
    ),
    "CAkBus::IncrementActivityCount": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkBus::IncrementPlayCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # CounterParameters *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkBus::IsBackgroundMusicMuted": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[]
    ),
    "CAkBus::IsMixingBus": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
        ]
    ),
    "CAkBus::IsOrIsChildOf": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkBus::IsTopBus": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
        ]
    ),
    "CAkBus::Mute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # AkCurveInterpolation
            ctypes.c_int32,  # int
        ]
    ),
    "CAkBus::MuteBackgroundMusic": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkActiveParent<CAkParameterNodeBase>::MuteNotification": {
        "CAkActiveParent<CAkParameterNodeBase> *, float, AkMutedMapItem *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNodeBase> *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # AkMutedMapItem *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "CAkActiveParent<CAkParameterNodeBase> *, float, CAkRegisteredObj *, AkMutedMapItem *, bool": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNodeBase> *
                ctypes.c_float,  # float
                ctypes.c_ulonglong,  # CAkRegisteredObj *
                ctypes.c_ulonglong,  # AkMutedMapItem *
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "CAkBus::MuteNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # AkMutedMapItem *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkActiveParent<CAkParameterNodeBase>::NotifyBypass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNodeBase> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "CAkBus::NotifyBypass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkBus::NotifyParamChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # AkRTPC_ParameterID
        ]
    ),
    "CAkBus::NotifyParamsChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const CAkBitArray<unsigned __int64,0> *
        ]
    ),
    "CAkActiveParent<CAkParameterNodeBase>::ParamNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNodeBase> *
            ctypes.c_ulonglong,  # NotifParams *
        ]
    ),
    "CAkBus::ParamNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # NotifParams *
        ]
    ),
    "CAkActiveParent<CAkParameterNodeBase>::PlayToEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNodeBase> *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBus::PlayToEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBus::PositioningChangeNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "CAkActiveParent<CAkParameterNodeBase>::PrepareData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNodeBase> *
        ]
    ),
    "CAkActiveParent<CAkParameterNodeBase>::PropagatePositioningNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNodeBase> *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "CAkBus::PropagatePositioningNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_float,  # float
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkBus::PushParamUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "CAkActiveParent<CAkParameterNodeBase>::RecalcNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNodeBase> *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkBus::RecalcNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkList2<MapStruct<uint,CAkBus::AkDuckInfo>,MapStruct<uint,CAkBus::AkDuckInfo> const &,1,0>::RemoveAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkList2<MapStruct<unsigned int,CAkBus::AkDuckInfo>,MapStruct<unsigned int,CAkBus::AkDuckInfo> const &,1,0> *
        ]
    ),
    "CAkParentNode<CAkParameterNodeBase>::RemoveChild": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkParentNode<CAkParameterNodeBase> *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkBus::RemoveChild": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkBus::RemoveStateGroup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkBus::ReplaceTopBusSharesets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBus::SetAkProp": {
        "CAkBus *, AkPropID, float, float": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkBus *
                ctypes.c_uint32,  # AkPropID
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
        "CAkBus *, AkPropID, CAkRegisteredObj *, AkValueMeaning, float, AkCurveInterpolation, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkBus *
                ctypes.c_uint32,  # AkPropID
                ctypes.c_ulonglong,  # CAkRegisteredObj *
                ctypes.c_uint32,  # AkValueMeaning
                ctypes.c_float,  # float
                ctypes.c_uint32,  # AkCurveInterpolation
                ctypes.c_int32,  # int
            ]
        ),
    },
    "CAkBus::SetBusDevice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBus::SetInitialFxParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkBus::SetInitialMetadataParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkBus::SetInitialParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkBus::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBus::SetMixerPlugin": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # SharedSetOverride
        ]
    ),
    "CAkBus::SetRTPC": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkRtpcType
            ctypes.c_uint32,  # AkRtpcAccum
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkCurveScaling
            ctypes.c_ulonglong,  # AkGraphPointBase<float> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBus::SetStateProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkStatePropertyUpdate *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkBus::StartDuckTransitions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # CAkDuckItem *
            ctypes.c_float,  # float
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # AkCurveInterpolation
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkPropID
        ]
    ),
    "CAkList2<MapStruct<uint,CAkDuckItem>,MapStruct<uint,CAkDuckItem> const &,1,0>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkList2<MapStruct<unsigned int,CAkDuckItem>,MapStruct<unsigned int,CAkDuckItem> const &,1,0> *
        ]
    ),
    "CAkBus::TriggerModulators": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # const AkModulatorTriggerParams *
            ctypes.c_ulonglong,  # CAkModulatorData *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "CAkActiveParent<CAkParameterNodeBase>::UnPrepareData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNodeBase> *
        ]
    ),
    "CAkBus::Unmute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # AkCurveInterpolation
            ctypes.c_int32,  # int
        ]
    ),
    "CAkBus::UnmuteAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # AkCurveInterpolation
            ctypes.c_int32,  # int
        ]
    ),
    "CAkBus::UnmuteBackgroundMusic": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "AkSortedKeyArray<uint,CAkParameterNodeBase *,AkHybridAllocator<8,1,0>,AkChildIDValueGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkParameterNodeBase *>,AkDefaultSortedKeyCompare<uint>>::Unset": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,CAkParameterNodeBase *,AkHybridAllocator<8,1,0>,AkChildIDValueGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkParameterNodeBase *>,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBus::UnsetRTPC": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBus::UpdateDuckedBus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
        ]
    ),
    "CAkActiveParent<CAkParameterNodeBase>::UpdateFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActiveParent<CAkParameterNodeBase> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBus::UpdateFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBus::UpdateStateGroups": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkStateGroupUpdate *
            ctypes.c_ulonglong,  # AkStateUpdate *
        ]
    ),
    "CAkRandomInfo::CAkRandomInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRandomInfo *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkSequenceInfo::CAkSequenceInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequenceInfo *
        ]
    ),
    "CAkRandomInfo::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkSequenceInfo *
        argtypes=[
            ctypes.c_ulonglong,  # CAkRandomInfo *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkSequenceInfo::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkContainerBaseInfo *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequenceInfo *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkRandomInfo::DeserializeHistory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRandomInfo *
            ctypes.c_ulonglong,  # AK::IReadBytes *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkSequenceInfo::DeserializeHistory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSequenceInfo *
            ctypes.c_ulonglong,  # AK::IWriteBytes *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkRandomInfo::FlagAsBlocked": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRandomInfo *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkRandomInfo::FlagAsUnBlocked": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRandomInfo *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkRandomInfo::FlagSetPlayed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRandomInfo *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkRandomInfo::GetRandomValue": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRandomInfo *
        ]
    ),
    "CAkRandomInfo::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRandomInfo *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkRandomInfo::IsFlagBlocked": FUNCDEF(
        restype=ctypes.c_uint8,  # unsigned __int8
        argtypes=[
            ctypes.c_ulonglong,  # CAkRandomInfo *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkRandomInfo::IsFlagSetPlayed": FUNCDEF(
        restype=ctypes.c_uint8,  # unsigned __int8
        argtypes=[
            ctypes.c_ulonglong,  # CAkRandomInfo *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkRandomInfo::ResetFlagsPlayed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRandomInfo *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CAkRandomInfo::SerializeHistory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRandomInfo *
            ctypes.c_ulonglong,  # AK::IWriteBytes *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "AK::ReadBytesMem::ReadBytesMem": {
        "AK::ReadBytesMem *, const void *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # AK::ReadBytesMem *
                ctypes.c_ulonglong,  # const void *
                ctypes.c_int32,  # int
            ]
        ),
        "AK::ReadBytesMem *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # AK::ReadBytesMem *
            ]
        ),
    },
    "AK::WriteBytesBuffer::WriteBytesBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::WriteBytesBuffer *
        ]
    ),
    "AK::WriteBytesMem::WriteBytesMem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::WriteBytesMem *
        ]
    ),
    "AK::ReadBytesMem::~ReadBytesMem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::ReadBytesMem *
        ]
    ),
    "AK::WriteBytesBuffer::~WriteBytesBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::WriteBytesBuffer *
        ]
    ),
    "AK::WriteBytesMem::~WriteBytesMem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::WriteBytesMem *
        ]
    ),
    "AK::ReadBytesMem::Attach": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::ReadBytesSkip *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int32,  # int
        ]
    ),
    "AK::WriteBytesBuffer::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::WriteBytesBuffer *
        ]
    ),
    "AK::WriteBytesMem::Clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::WriteBytesMem *
        ]
    ),
    "AK::WriteBytesMem::Detach": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int8 *
        argtypes=[
            ctypes.c_ulonglong,  # AK::WriteBytesMem *
        ]
    ),
    "AK::ReadBytesMem::ReadBytes": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AK::ReadBytesMem *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "AK::WriteBytesMem::Reserve": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # AK::WriteBytesMem *
            ctypes.c_int32,  # int
        ]
    ),
    "AK::WriteBytesBuffer::SetBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::WriteBytesBuffer *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
        ]
    ),
    "AK::WriteBytesMem::SetCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::WriteBytesMem *
            ctypes.c_int32,  # int
        ]
    ),
    "AK::WriteBytesMem::SetMemPool": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::WriteBytesMem *
            ctypes.c_int32,  # int
        ]
    ),
    "AK::WriteBytesBuffer::WriteBytes": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AK::WriteBytesBuffer *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "AK::WriteBytesMem::WriteBytes": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AK::WriteBytesMem *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "AK::ReadBytesSkip::ReadBytesSkip": {
        "AK::ReadBytesSkip *, const void *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # AK::ReadBytesSkip *
                ctypes.c_ulonglong,  # const void *
                ctypes.c_int32,  # int
            ]
        ),
        "AK::ReadBytesSkip *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # AK::ReadBytesSkip *
            ]
        ),
    },
    "AK::WriteBytesCount::WriteBytesCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::WriteBytesCount *
        ]
    ),
    "AK::ReadBytesSkip::~ReadBytesSkip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::ReadBytesSkip *
        ]
    ),
    "AK::WriteBytesCount::~WriteBytesCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::WriteBytesCount *
        ]
    ),
    "AK::ReadBytesSkip::ReadBytes": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AK::ReadBytesSkip *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "AK::WriteBytesCount::WriteBytes": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # AK::WriteBytesCount *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "CAkAudioThread::CAkAudioThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioThread *
        ]
    ),
    "AKPLATFORM::AkSetThreadName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "CAkAudioThread::EventMgrThreadFunc": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkAudioThread::Start": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioThread *
        ]
    ),
    "CAkAudioThread::Stop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioThread *
        ]
    ),
    "CAkAudioThread::WakeupEventsConsumer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioThread *
        ]
    ),
    "AkPipelineChannelMappingFunc": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const AkChannelConfig *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "AkVorbisChannelMappingFunc": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const AkChannelConfig *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "AkWaveChannelMappingFunc": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const AkChannelConfig *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "AkPipelineBufferBase::ClearFreeListBuckets": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "AkPipelineBufferBase::GetCachedBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkPipelineBufferBase *
        ]
    ),
    "AkPipelineBufferBase::InitFreeListBuckets": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "AkPipelineBufferBase::ReleaseCachedBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkPipelineBufferBase *
        ]
    ),
    "ComputeFiltering3D_Template<NoLogAttenuationDelta>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ulonglong,  # const AkArray<AkRayVolumeData,AkRayVolumeData const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRayVolumeData> > *
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "ComputeVolumeRays_Template<NoLogAttenuationDelta>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ulonglong,  # AkArray<AkRayVolumeData,AkRayVolumeData const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRayVolumeData> > *
        ]
    ),
    "CAkListener::CAkListener": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkListener *
        ]
    ),
    "SpeakerVolumeMatrixCallback::operator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SpeakerVolumeMatrixCallback *
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "CAkListener::Apply3dTransforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const Ak3dData *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkRayVolumeData **
            ctypes.c_uint32,  # Ak3DSpatializationMode
            ctypes.c_uint32,  # Ak3DSpatializationMode
            ctypes.c_ulonglong,  # const CAkListener *
            ctypes.c_ulonglong,  # CAkConversionTable *
            ctypes.c_ulonglong,  # CAkConversionTable *
            ctypes.c_ulonglong,  # Ak3dData *
        ]
    ),
    "CAkListener::ComputeFiltering3D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ulonglong,  # const AkArray<AkRayVolumeData,AkRayVolumeData const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRayVolumeData> > *
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "CAkListener::ComputeRayDistanceAndAngles": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkListener *
            ctypes.c_ulonglong,  # const CAkEmitter *
            ctypes.c_ulonglong,  # AkRayVolumeData *
        ]
    ),
    "CAkListener::ComputeRayWeightsCorrection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # AkRayVolumeData **
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkListener::ComputeSphericalCoordinates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkListener *
            ctypes.c_ulonglong,  # const AkEmitterListenerPair *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkListener::ComputeVolumeRays": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ulonglong,  # AkArray<AkRayVolumeData,AkRayVolumeData const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRayVolumeData> > *
        ]
    ),
    "CAkListener::GatherRays": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const AkMixConnection *
            ctypes.c_ulonglong,  # const AkArray<AkRayVolumeData,AkRayVolumeData const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRayVolumeData> > *
            ctypes.c_ulonglong,  # AkRayVolumeData **
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkListener::GetListenerData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkRegisteredObj *
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CAkListener::GetListenerMatrix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # float *
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CAkListener::GetListenerSpatialization": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # float **
            ctypes.c_ulonglong,  # AkChannelConfig *
        ]
    ),
    "CAkListener::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkListener *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CAkListener::MultiDirectionGainCorrectionForObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkRayVolumeData **
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # AkArrayConnectedObjects *
        ]
    ),
    "CAkListener::OnBeginFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkListener::ResetListenerData": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "AkSortedKeyArray<unsigned __int64,unsigned __int64,AkHybridAllocator<8,16,4>,AkSetGetKey<unsigned __int64>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64>,AkDefaultSortedKeyCompare<unsigned __int64>>::Set": {
        "AkSortedKeyArray<unsigned __int64,unsigned __int64,AkHybridAllocator<8,16,4>,AkSetGetKey<unsigned __int64>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64>,AkDefaultSortedKeyCompare<unsigned __int64> > *, unsigned __int64": FUNCDEF(
            restype=ctypes.c_ulonglong,  # unsigned __int64 *
            argtypes=[
                ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned __int64,unsigned __int64,AkHybridAllocator<8,16,4>,AkSetGetKey<unsigned __int64>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64>,AkDefaultSortedKeyCompare<unsigned __int64> > *
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
        "AkSortedKeyArray<unsigned __int64,unsigned __int64,AkHybridAllocator<8,16,4>,AkSetGetKey<unsigned __int64>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64>,AkDefaultSortedKeyCompare<unsigned __int64> > *, unsigned __int64, bool *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # unsigned __int64 *
            argtypes=[
                ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned __int64,unsigned __int64,AkHybridAllocator<8,16,4>,AkSetGetKey<unsigned __int64>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64>,AkDefaultSortedKeyCompare<unsigned __int64> > *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # bool *
            ]
        ),
    },
    "CAkListener::SetPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkListener *
            ctypes.c_ulonglong,  # const AkTransform *
        ]
    ),
    "CAkListener::SetScalingFactor": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkListener *
            ctypes.c_float,  # float
        ]
    ),
    "CAkListener::SetTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkListener *
            ctypes.c_ulonglong,  # const AkTransform *
        ]
    ),
    "CAkListener::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkModulatorMgr::CAkModulatorMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorMgr *
        ]
    ),
    "CAkModulator::AddSubscription": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulator *
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCSubscription *
        ]
    ),
    "CAkModulatorMgr::AddSubscription": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCSubscription *
        ]
    ),
    "CAkModulatorMgr::CleanUpFinishedCtxs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorMgr *
        ]
    ),
    "CAkModulatorMgr::GetCurrentModulatorOutput": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkConversionTable::GetLine": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConversionTable *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkModulatorMgr::GetModulators": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorMgr *
            ctypes.c_ulonglong,  # const AkModulatorSubscriberInfo *
            ctypes.c_ulonglong,  # AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *
        ]
    ),
    "CAkModulatorMgr::GetParamXfrm": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorMgr *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const CAkRegisteredObj *
            ctypes.c_ulonglong,  # AkModulatorParamXfrm *
        ]
    ),
    "AkModulatorTriggerParams::GetRTPCKey": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCKey *
        argtypes=[
            ctypes.c_ulonglong,  # AkModulatorTriggerParams *
            ctypes.c_ulonglong,  # AkRTPCKey *
        ]
    ),
    "AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *
        ]
    ),
    "CAkModulatorMgr::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorMgr *
        ]
    ),
    "CAkModulatorMgr::ProcessModulators": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorMgr *
        ]
    ),
    "CAkModulatorMgr::RemoveSubscription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorMgr *
            ctypes.c_ulonglong,  # CAkRTPCMgr::AkRTPCSubscription *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkModulatorMgr::RemovedScopedRtpcObj": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "AkHashListBare<unsigned __int64,AkModulatorArray,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<unsigned __int64,AkModulatorArray>>::Resize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkHashListBare<unsigned __int64,AkModulatorArray,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<unsigned __int64,AkModulatorArray> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkModulatorMgr::SupportsAutomatedParams": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkModulatorMgr::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorMgr *
        ]
    ),
    "CAkModulatorMgr::TermModulatorEngine": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorMgr *
        ]
    ),
    "CAkModulatorMgr::Trigger": {
        "CAkModulatorMgr *, const AkModulatorSubscriberInfo *, const AkModulatorTriggerParams *, CAkModulatorData *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # CAkModulatorMgr *
                ctypes.c_ulonglong,  # const AkModulatorSubscriberInfo *
                ctypes.c_ulonglong,  # const AkModulatorTriggerParams *
                ctypes.c_ulonglong,  # CAkModulatorData *
            ]
        ),
        "CAkModulatorMgr *, const AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, const AkModulatorTriggerParams *, CAkModulatorData *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # CAkModulatorMgr *
                ctypes.c_ulonglong,  # const AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *
                ctypes.c_ulonglong,  # const AkModulatorTriggerParams *
                ctypes.c_ulonglong,  # CAkModulatorData *
            ]
        ),
    },
    "AkFileParser::Parse": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkFileParser::FormatInfo *
            ctypes.c_ulonglong,  # CAkMarkers *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # AkFileParser::AnalysisDataChunk *
            ctypes.c_ulonglong,  # AkFileParser::SeekInfo *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSpeakerPan::ContractPoints<ContractInside>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
            ctypes.c_double,  # double
        ]
    ),
    "CAkSpeakerPan::ContractPoints<ContractOutside>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
            ctypes.c_float,  # float
        ]
    ),
    "RotateAndContractPointsSIMD_ContractInside_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "RotateAndContractPointsSIMD_ContractOutside_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkSpeakerPan::AddHeadphonePower": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkSpeakerPan::AddPowerVbap2d": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkVector *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkSpeakerPan::ComputeDecoderMatrix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkSpeakerPan::ConvertSpeakerAngles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkSpeakerPan::ConvertSpeakerAnglesToRad": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CopyRotatedCopiesOntoChannels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkSpeakerPan::CreatePanCache2D": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "CAkSpeakerPan::CreatePanCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # float *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "DownmixGridToChannels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const GridToSpeakerGain *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkSpeakerPan::EncodeToAmbisonics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSpeakerPan::EncodeVector": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "GenPointsForZeroSpread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkTransform *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "SpeakerPan::MultiDirection::GetMultiDirObjectNormalizationGains": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkVector *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkSpeakerPan::GetReWeights": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const float *
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "GetSampledHarmonics2D": FUNCDEF(
        restype=ctypes.c_ulonglong,  # float *
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "CAkSpeakerPan::GetSampledHarmonics3D": FUNCDEF(
        restype=ctypes.c_ulonglong,  # float *
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "AkDevice::GetSpreadCache": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkArray<__m128,__m128 const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_NoGrow,AkAssignmentMovePolicy<__m128> > *
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSpeakerPan::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "AkArray<__m128,__m128 const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_NoGrow,AkAssignmentMovePolicy<__m128>>::Resize": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<__m128,__m128 const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_NoGrow,AkAssignmentMovePolicy<__m128> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "RotateAndContractPointsWithRotationSIMD": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "RotateAndContractPointsWithRotationScalar": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkSpeakerPan::SetSpeakerAngles": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "SpeakerPan::Steering::Steer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkSpeakerPan::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "TransformSampledSphereNoSpread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkTransform *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "TransformSampledSphereSIMD": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkTransform *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkSpeakerPan::ValidateSpeakerAngles": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkSpeakerPan::_ComputeSN3DphericalHarmonics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "AkUnion_unsigned___int64_AkHybridAllocator_8_16_4__AkGrowByPolicy_Proportional_": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
            ctypes.c_ulonglong,  # const AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
        ]
    ),
    "AkDevice::AkDevice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_ulonglong,  # const AkOutputSettings *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkDevice::~AkDevice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
        ]
    ),
    "AkVPL::~AkVPL": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkVPL *
        ]
    ),
    "AkDevice::AddCaptureCallbackStruct": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_ulonglong,  # AkDevice::BaseCaptureCallbackStruct *
        ]
    ),
    "CAkOutputMgr::AddOutputCaptureMarker": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "CAkOutputMgr::AddOutputDevice": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const AkOutputSettings *
            ctypes.c_ulonglong,  # AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
        ]
    ),
    "AkDevice::AllocMemoryCaptureCallbackStruct": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkDevice::BaseCaptureCallbackStruct *
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkAudioBuffer *', {'type': 'unsigned', 'name': '__int64'}, 'void *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AkDevice::Allocate3DAudioResources": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_ulonglong,  # const Ak3DAudioSinkCapabilities *
        ]
    ),
    "CAkOutputMgr::CallSinkCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const AkOutputSettings *
            ctypes.c_uint32,  # AK::AkAudioDeviceEvent
            ctypes.c_uint32,  # AKRESULT
        ]
    ),
    "AkDevice::ClearEffects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkDevice::ClearSink": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
        ]
    ),
    "AkDevice::ConnectMix": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkVPL *
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional>::Contains": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AkDevice::CreateDummy": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_uint32,  # AkDevice::DeviceState
        ]
    ),
    "AkDevice::CreateSink": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
        ]
    ),
    "AkDevice::DeleteDeviceAndDisconnectMix": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
        ]
    ),
    "AkDevice::DestroyPanCaches": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
        ]
    ),
    "AkDevice::FileCaptureCallbackStruct::DoCallbackFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice::FileCaptureCallbackStruct *
        ]
    ),
    "AkDevice::CaptureImpl::MemoryCaptureCallbackStruct::DoCallbackFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice::CaptureImpl::MemoryCaptureCallbackStruct *
        ]
    ),
    "CAkOutputMgr::EnableMeteringCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # AkMeteringFlags
        ]
    ),
    "CAkOutputMgr::EndOfFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "AkDevice::FindCaptureCallbackFunc": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkAudioBuffer *', {'type': 'unsigned', 'name': '__int64'}, 'void *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkOutputMgr::FindDevice": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkDevice *
        argtypes=[
            ctypes.c_ulonglong,  # const AK::CAkBusCtx *
        ]
    ),
    "CAkOutputMgr::FindDeviceOfType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkDevice *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkDevice::ForceReset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
        ]
    ),
    "CAkOutputMgr::GetDevice": {
        "unsigned int, unsigned int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # AkDevice *
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "unsigned __int64": FUNCDEF(
            restype=ctypes.c_ulonglong,  # AkDevice *
            argtypes=[
                ctypes.c_uint64,  # unsigned __int64
            ]
        ),
    },
    "AkDevice::GetEffect": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkSmartPtr<CAkFxBase> *
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_ulonglong,  # CAkSmartPtr<CAkFxBase> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkDevice::SinkPluginContext::GetNumRefillsInVoice": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice::SinkPluginContext *
        ]
    ),
    "AkDevice::PluginContext<AK::IAkAudioDeviceEffectPluginContext>::GetOutputID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice::PluginContext<AK::IAkSinkPluginContext> *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AkDevice::GetOutputID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AkDevice::PluginContext<AK::IAkAudioDeviceEffectPluginContext>::GetParentChannelConfig": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice::PluginContext<AK::IAkSinkPluginContext> *
            ctypes.c_ulonglong,  # AkChannelConfig *
        ]
    ),
    "AkDevice::PluginContext<AK::IAkSinkPluginContext>::GetPluginMedia": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice::PluginContext<AK::IAkSinkPluginContext> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkOutputMgr::GetPrimaryDevice": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkDevice *
        argtypes=[]
    ),
    "AkDevice::PluginContext<AK::IAkAudioDeviceEffectPluginContext>::GetProcessorFeatures": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkProcessorFeatures *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "AkDevice::GetSpeakerAngles": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkInsertFXContext::GlobalContext": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkGlobalPluginContext *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "AkArray<MapStruct<CAkSpeakerPan::ConfigIn2d3d,AkArray<__m128,__m128 const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_NoGrow,AkAssignmentMovePolicy<__m128>>>,MapStruct<CAkSpeakerPan::ConfigIn2d3d,AkArray<__m128,__m128 const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_NoGrow,AkAssignmentMovePolicy<__m128>>> const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,CAkSpeakerPan::AkCacheItemMovePolicy>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<MapStruct<CAkSpeakerPan::ConfigIn2d3d,AkArray<__m128,__m128 const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_NoGrow,AkAssignmentMovePolicy<__m128> > >,MapStruct<CAkSpeakerPan::ConfigIn2d3d,AkArray<__m128,__m128 const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_NoGrow,AkAssignmentMovePolicy<__m128> > > const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,CAkSpeakerPan::AkCacheItemMovePolicy> *
        ]
    ),
    "CAkOutputMgr::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "AkDevice::InitDefaultAngles": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
        ]
    ),
    "CAkOutputMgr::InitMainDevice": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const AkOutputSettings *
        ]
    ),
    "CAkOutputMgr::IsDataNeeded": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "AkDevice::BaseCaptureCallbackStruct::IsMatch": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice::BaseCaptureCallbackStruct *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkAudioBuffer *', {'type': 'unsigned', 'name': '__int64'}, 'void *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AkDevice::SinkPluginContext::IsPrimary": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice::SinkPluginContext *
        ]
    ),
    "AkDevice::FileCaptureCallbackStruct::IsValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice::FileCaptureCallbackStruct *
        ]
    ),
    "CAkOutputMgr::ManageDevicesState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkDevice::MixForCapture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # const AkAudioObjects *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "AkDevice::RefreshSink3DAudioCapabilities": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
        ]
    ),
    "AkDevice::Release3DAudioResources": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
        ]
    ),
    "CAkOutputMgr::RemoveOutputDevice": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkOutputMgr::RenderIsActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "CAkOutputMgr::ReplaceDevice": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const AkOutputSettings *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "ReplaceIDForDefaultDevice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkOutputMgr::ResetMainDevice": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "AkDevice::SetAudioDevice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_ulonglong,  # CAkSmartPtr<CAkAudioDevice> *
        ]
    ),
    "CAkOutputMgr::SetDeviceSuspended": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkDevice::SetSpeakerAngles": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "AkDevice::SinkPluginContext::SignalAudioThread": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice::SinkPluginContext *
        ]
    ),
    "AkDevice::StartOutputCapture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "CAkOutputMgr::StartOutputCapture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "CAkOutputMgr::StartSilentMode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkDevice::StopOutputCapture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
        ]
    ),
    "CAkOutputMgr::StopOutputCapture": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkOutputMgr::SuspendedThread": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkOutputMgr::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "AkArray<__m128,__m128 const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_NoGrow,AkAssignmentMovePolicy<__m128>>::Transfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<__m128,__m128 const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_NoGrow,AkAssignmentMovePolicy<__m128> > *
            ctypes.c_ulonglong,  # AkArray<__m128,__m128 const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_NoGrow,AkAssignmentMovePolicy<__m128> > *
        ]
    ),
    "AkArray<unsigned __int64,unsigned __int64 const &,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64>>::Transfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<unsigned __int64,unsigned __int64 const &,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64> > *
            ctypes.c_ulonglong,  # AkArray<unsigned __int64,unsigned __int64 const &,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64> > *
        ]
    ),
    "CAkOutputMgr::UnsetListenersOnDevice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
        ]
    ),
    "AkDevice::UpdateEffect": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkDevice::UpdateEffects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDevice *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkOutputMgr::_AddOutputDevice": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # AkOutputSettings *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
            ctypes.c_uint32,  # AkSetType
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkWinDeviceCache::AddNotif": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_uint32,  # CAkWinDeviceCache::Notification::NotifType
        ]
    ),
    "CAkWinDeviceCache::AddRef": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
        ]
    ),
    "CAkWinDeviceCache::Count": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
        ]
    ),
    "CAkWinDeviceCache::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkWinDeviceCache *
        argtypes=[]
    ),
    "CAkWinDeviceCache::EnumDevices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
        ]
    ),
    "CAkWinDeviceCache::FindDevice": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::Win32::Device *
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
            ctypes.c_ulonglong,  # AK::Win32::Device *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkWinDeviceCache::FindDeviceIdx": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkWinDeviceCache::GetDefault": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::Win32::Device *
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
            ctypes.c_ulonglong,  # AK::Win32::Device *
            ctypes.c_int64,  # __int64
        ]
    ),
    "AK::Win32::DeviceEnumerator::GetDefaultDevice": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::Win32::Device *
        argtypes=[
            ctypes.c_ulonglong,  # AK::Win32::DeviceEnumerator *
            ctypes.c_ulonglong,  # AK::Win32::Device *
            ctypes.c_int64,  # __int64
        ]
    ),
    "AkArray<CAkVPL3dMixable::PeakPerHDRBus,CAkVPL3dMixable::PeakPerHDRBus const &,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkVPL3dMixable::PeakPerHDRBus>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<CAkVPL3dMixable::PeakPerHDRBus,CAkVPL3dMixable::PeakPerHDRBus const &,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkVPL3dMixable::PeakPerHDRBus> > *
        ]
    ),
    "CAkWinDeviceCache::IsDeviceActive": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkWinDeviceCache::Item": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::Win32::Device *
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
            ctypes.c_ulonglong,  # AK::Win32::Device *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkWinDeviceCache::Lock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
        ]
    ),
    "CAkWinDeviceCache::OnDefaultDeviceChanged": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
            ctypes.c_uint32,  # __MIDL___MIDL_itf_mmdeviceapi_0000_0000_0001
            ctypes.c_uint32,  # __MIDL___MIDL_itf_mmdeviceapi_0000_0000_0002
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "CAkWinDeviceCache::OnDeviceAdded": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "CAkWinDeviceCache::OnDeviceRemoved": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "CAkWinDeviceCache::OnDeviceStateChanged": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkWinDeviceCache::OnPropertyValueChanged": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_ulonglong,  # const _tagpropertykey *
        ]
    ),
    "CAkWinDeviceCache::ProcessDeviceNotif": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
        ]
    ),
    "CAkWinDeviceCache::QueryInterface": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
            ctypes.c_ulonglong,  # const _GUID *
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "CAkWinDeviceCache::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
        ]
    ),
    "CAkWinDeviceCache::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
        ]
    ),
    "CAkWinDeviceCache::Unlock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkWinDeviceCache *
        ]
    ),
    "CAkLEngine::ApplyGlobalSettings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkPlatformInitSettings *
        ]
    ),
    "CAkLEngine::GetDefaultOutputSettingsCommon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkOutputSettings *
        ]
    ),
    "CAkLEngine::GetDefaultPlatformThreadInitSettings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkPlatformInitSettings *
        ]
    ),
    "CAkLEngine::GetNumBufferNeededAndSubmit": FUNCDEF(
        restype=ctypes.c_uint32,  # unsigned int
        argtypes=[]
    ),
    "CAkLEngine::GetPluginDLLFullPath": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # wchar_t *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "AKPLATFORM::SafeStrCpy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # wchar_t *
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CAkLEngine::SetPanningRule": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # AkPanningRule
        ]
    ),
    "CAkLEngine::VPLDestroySource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkLEngine::AddSound": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkLECmd *
        ]
    ),
    "AddVPLToArray": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkVPL *,AkVPL *,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkVPL *> > *
            ctypes.c_ulonglong,  # AkVPL *
        ]
    ),
    "CAkLEngine::AnalyzeMixingGraph": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkLEngine::BusTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkVPL *
        ]
    ),
    "CAkLEngine::BypassBusFx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "AkHdrBus::ComputeHdrAttenuation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkHdrBus *
        ]
    ),
    "CAkLEngine::ConnectSourceToGraph": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkLEngine::DisconnectMixable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
        ]
    ),
    "CAkLEngine::DoGraphReeval": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkLEngine::DropAllEffectsUsingThisMedia": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int8 *
        ]
    ),
    "CAkLEngine::EnableMeteringCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkMeteringFlags
        ]
    ),
    "CAkLEngine::EnableVolumeCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkLEngine::EnsureVPLExists": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkLEngine::FinalizeVoiceProcessing": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkLEngine::GetAuxBus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # AkAuxSendValueEx *
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
        ]
    ),
    "CAkLEngine::GetExistingVPLMixBus": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkVPL *
        argtypes=[
            ctypes.c_ulonglong,  # const AK::CAkBusCtx *
        ]
    ),
    "CAkBus::GetHdrBallistics": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkBus::GetHdrGainComputer": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "AkArray<AkVPL *,AkVPL *,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkVPL *>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkVPL *,AkVPL *,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkVPL *> > *
        ]
    ),
    "AkArray<AkHdrBus::MaxHdrWinTopInfo,AkHdrBus::MaxHdrWinTopInfo const &,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkHdrBus::MaxHdrWinTopInfo>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkHdrBus::MaxHdrWinTopInfo,AkHdrBus::MaxHdrWinTopInfo const &,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkHdrBus::MaxHdrWinTopInfo> > *
        ]
    ),
    "AkVPL::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkVPL *
            ctypes.c_ulonglong,  # AK::CAkBusCtx *
            ctypes.c_ulonglong,  # AkVPL *
        ]
    ),
    "CAkLEngine::IsParallelProcessingAvailable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "CAkLEngine::MixBusParamNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # NotifParams *
        ]
    ),
    "MixRecordsToOutputs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const AkArray<AkAudioObjectRegistry::Record,AkAudioObjectRegistry::Record const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkAudioObjectRegistry::Record> > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AK::IAkMixerPluginContext *
        ]
    ),
    "AkHdrBus::NotifyHdrWindowTop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CAkLEngine::PositioningChangeNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_uint32,  # AkRTPC_ParameterID
        ]
    ),
    "CAkLEngine::ProcessGraph": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkLEngine::PushToDevice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkVPL *
            ctypes.c_ulonglong,  # AkDevice *
        ]
    ),
    "CAkLEngine::ReevaluateGraph": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkLEngine::ResetBusVolume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkLEngine::ResolveCommandVPL": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        argtypes=[
            ctypes.c_ulonglong,  # AkLECmd *
        ]
    ),
    "CAkLEngine::RunVPL": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkLEngine::SoftwareInit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "CAkLEngine::SoftwarePerform": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkLEngine::SoftwareTerm": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkLEngine::SortSiblingVPLs": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkVPL *,AkVPL *,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkVPL *> > *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "CAkLEngine::SortVPLs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "CAkLEngine::Stop": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkLEngine::StopMixBussesUsingThisSlot": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const CAkUsageSlot *
        ]
    ),
    "CAkLEngine::UpdateChannelConfig": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkLEngine::UpdateDeviceFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkLEngine::UpdateDeviceFXBypassMask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkLEngine::UpdateMixBusFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkLEngine::VPLRefreshDepth": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkVPL *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkLEngine::VPLTryConnectSource": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkLEngine::VoiceRangeTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "AkAudioBuffer::ZeroPadToMaxFrames": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkVPL3dMixable::_ComputeVolumeRays": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
        ]
    ),
    "DeviceEnumHelper::DeviceEnumHelper": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DeviceEnumHelper *
        ]
    ),
    "AK::DeviceEnumeratorHelper": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_ulonglong,  # IMMDevice *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkLEngine::GetDefaultOutputSettings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkOutputSettings *
        ]
    ),
    "CAkLEngine::GetDefaultPlatformInitSettings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkPlatformInitSettings *
        ]
    ),
    "CAkWindowsContext::GetDeviceEnum": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::Win32::IAkDeviceEnumerator *
        argtypes=[
            ctypes.c_ulonglong,  # CAkWindowsContext *
        ]
    ),
    "AK::GetDeviceID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # IMMDevice *
        ]
    ),
    "AK::GetDeviceIDFromName": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # wchar_t *
        ]
    ),
    "AK::SoundEngine::GetDeviceSpatialAudioSupport": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkLEngine::GetPlatformDeviceList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # AkDeviceDescription *
        ]
    ),
    "AK::GetWindowsDevice": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # IMMDevice **
            ctypes.c_uint32,  # AkAudioDeviceState
        ]
    ),
    "AK::GetWindowsDeviceCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # AkAudioDeviceState
        ]
    ),
    "AK::GetWindowsDeviceName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # AkAudioDeviceState
        ]
    ),
    "CAkLEngine::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "CAkLEngine::InitPlatformContext": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "CAkLEngine::OnThreadEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkLEngine::OnThreadStart": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkLEngine::Perform": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "AK::SoundEngine::RegisterPluginDLL": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "CAkLEngine::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkLEngine::TermPlatformContext": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkGameObject::CreateComponent<CAkConnectedListeners>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkGameObject *
        ]
    ),
    "CAkGameObject::CreateComponent<CAkEmitter>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkGameObject *
        ]
    ),
    "CAkGameObject::CreateComponent<CAkListener>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkGameObject *
        ]
    ),
    "CAkGameObject::CreateComponent<CAkSwitchHistory>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkSwitchHistory *
        argtypes=[
            ctypes.c_ulonglong,  # CAkGameObject *
        ]
    ),
    "CAkConnectedListeners::CAkConnectedListeners": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConnectedListeners *
        ]
    ),
    "CAkRegistryMgr::CAkRegistryMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
        ]
    ),
    "AkPositionStore::ClearDataForListenersNotInSet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkPositionStore *
            ctypes.c_ulonglong,  # const AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
        ]
    ),
    "CAkRegistryMgr::ClearSwitchHist": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkRegistryMgr::EnsureListenersExist": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_ulonglong,  # const AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
        ]
    ),
    "CAkRegistryMgr::GetActiveGameObjects": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_ulonglong,  # AkArray<unsigned __int64,unsigned __int64,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64> > *
        ]
    ),
    "CAkRegistryMgr::GetObj": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkRegisteredObj *
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CAkRegistryMgr::GetObjAndAddref": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkGameObject *
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CAkRegistryMgr::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
        ]
    ),
    "CAkRegistryMgr::IsGameObjectActive": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CAkRegistryMgr::NotifyListenerPosChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_ulonglong,  # const AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
        ]
    ),
    "CAkRegistryMgr::RegisterObject": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_char_p,  # const char *
        ]
    ),
    "CAkRegistryMgr::RegisterObjectAsActive": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_ulonglong,  # CAkGameObject *
        ]
    ),
    "CAkRegistryMgr::ResetListenersToDefault": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AkHashList<unsigned __int64,CAkRegisteredObj *,AkArrayAllocatorNoAlign<0>>::Resize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkHashList<unsigned __int64,CAkRegisteredObj *,AkArrayAllocatorNoAlign<0> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkKeyArray<uint,AkSwitchHistItem,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<uint,AkSwitchHistItem>>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkSwitchHistItem *
        argtypes=[
            ctypes.c_ulonglong,  # CAkKeyArray<unsigned int,AkSwitchHistItem,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<unsigned int,AkSwitchHistItem> > > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkSwitchHistItem *
        ]
    ),
    "CAkRegistryMgr::SetGameObjectAuxSendValues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # AkAuxSendValue *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkRegistryMgr::SetGameObjectOutputBusVolume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
        ]
    ),
    "CAkRegistryMgr::SetGameObjectScalingFactor": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
        ]
    ),
    "CAkRegistryMgr::SetMultipleObstructionAndOcclusion": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # AkObstructionOcclusionValues *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkRegistryMgr::SetNodeIDAsModified": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkRegistryMgr::SetObjectObstructionAndOcclusion": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CAkRegistryMgr::SetPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const AkChannelEmitter *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AK::SoundEngine::MultiPositionType
        ]
    ),
    "CAkRegistryMgr::SetSwitchDormant": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkRegistryMgr::SetSwitchHistItem": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkSwitchHistItem *
        ]
    ),
    "CAkRegistryMgr::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
        ]
    ),
    "CAkRegistryMgr::UnregisterAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_ulonglong,  # const AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
        ]
    ),
    "CAkRegistryMgr::UnregisterObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CAkRegistryMgr::UnregisterObjectAsActive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_ulonglong,  # CAkGameObject *
        ]
    ),
    "CAkKeyArray<uint,AkSwitchHistItem,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<uint,AkSwitchHistItem>>>::Unset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkKeyArray<unsigned int,AkSwitchHistItem,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<unsigned int,AkSwitchHistItem> > > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkRegistryMgr::UpdateDefaultListeners": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_ulonglong,  # const AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
            ctypes.c_uint32,  # AkListenerOp
        ]
    ),
    "CAkRegistryMgr::UpdateGameObjectPositions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
        ]
    ),
    "CAkRegistryMgr::UpdateListeners": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRegistryMgr *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
            ctypes.c_uint32,  # AkListenerOp
        ]
    ),
    "CAkDynamicSequence::AddRef": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
        ]
    ),
    "CAkDynamicSequence::AllExec": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
            ctypes.c_uint32,  # ActionParamType
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkDynamicSequence::Break": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
        ]
    ),
    "CAkDynamicSequence::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkIndexable *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AK::SoundEngine::DynamicSequence::DynamicSequenceType
        ]
    ),
    "CAkDynamicSequence::CreatePBI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_ulonglong,  # CAkSource *
            ctypes.c_ulonglong,  # AkPBIParams *
            ctypes.c_ulonglong,  # const PriorityInfoCurrent *
        ]
    ),
    "AkArray<AK::SoundEngine::DynamicSequence::PlaylistItem,AK::SoundEngine::DynamicSequence::PlaylistItem const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AK::SoundEngine::DynamicSequence::PlaylistItem>>::Erase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AK::SoundEngine::DynamicSequence::PlaylistItem,AK::SoundEngine::DynamicSequence::PlaylistItem const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AK::SoundEngine::DynamicSequence::PlaylistItem> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkDynamicSequence::GetNextToPlay": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # void **
            ctypes.c_ulonglong,  # UserParams *
        ]
    ),
    "CAkDynamicSequence::GetPauseTimes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkDynamicSequence::GetQueuedItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "CAkDynamicSequence::Pause": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
        ]
    ),
    "CAkDynamicSequence::Play": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
        ]
    ),
    "CAkDynamicSequence::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
        ]
    ),
    "CAkDynamicSequence::Resume": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
        ]
    ),
    "CAkDynamicSequence::ResumeWaiting": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
        ]
    ),
    "CAkDynamicSequence::Seek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
            ctypes.c_ulonglong,  # const AkQueuedMsg_DynamicSequenceSeek *
        ]
    ),
    "CAkDynamicSequence::SetGameObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkDynamicSequence::Stop": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
        ]
    ),
    "CAkDynamicSequence::UnlockPlaylist": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
        ]
    ),
    "CAkDynamicSequence::_GetNextToPlay": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "CAkDynamicSequence::_PlayNode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
        ]
    ),
    "CAkDynamicSequence::_Stop": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequence *
            ctypes.c_uint32,  # CAkDynamicSequence::AkDynamicSequenceStopMode
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # AkCurveInterpolation
        ]
    ),
    "CAkDialogueEvent::AddRef": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDialogueEvent *
        ]
    ),
    "CAkDialogueEvent::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkDialogueEvent *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkDialogueEvent::FindCustomProp": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkPropValue *
        argtypes=[
            ctypes.c_ulonglong,  # CAkDialogueEvent *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkDialogueEvent::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDialogueEvent *
        ]
    ),
    "CAkDialogueEvent::ResolveArgumentValueNames": {
        "CAkDialogueEvent *, const char **, unsigned int *, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # CAkDialogueEvent *
                ctypes.c_char_p,  # const char **
                ctypes.c_ulonglong,  # unsigned int *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "CAkDialogueEvent *, const wchar_t **, unsigned int *, unsigned int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # CAkDialogueEvent *
                ctypes.c_ulonglong,  # const wchar_t **
                ctypes.c_ulonglong,  # unsigned int *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
    },
    "CAkDialogueEvent::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDialogueEvent *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBusCallbackMgr::CAkBusCallbackMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusCallbackMgr *
        ]
    ),
    "CAkBusCallbackMgr::~CAkBusCallbackMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusCallbackMgr *
        ]
    ),
    "CAkBusCallbackMgr::DoVolumeCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusCallbackMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkSpeakerVolumeMatrixCallbackInfo *
        ]
    ),
    "CAkBusCallbackMgr::IsMeteringCallbackEnabled": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusCallbackMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBusCallbackMgr::IsVolumeCallbackEnabled": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusCallbackMgr *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBusCallbackMgr::SetDeviceMeteringCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusCallbackMgr *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkOutputDeviceMeteringCallbackInfo *']}
            ctypes.c_uint32,  # AkMeteringFlags
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkBusCallbackMgr::SetMeteringCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusCallbackMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkBusMeteringCallbackInfo *']}
            ctypes.c_uint32,  # AkMeteringFlags
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkBusCallbackMgr::SetVolumeCallback": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusCallbackMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkSpeakerVolumeMatrixCallbackInfo *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkPathManager::AddPathToList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPathManager *
        ]
    ),
    "CAkPathManager::AddPathUser": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPathManager *
            ctypes.c_ulonglong,  # CAkPath *
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkPathManager::AddPotentialUser": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPathManager *
            ctypes.c_ulonglong,  # CAkPath *
        ]
    ),
    "CAkPathManager::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPathManager *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPathManager::Pause": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPathManager *
            ctypes.c_ulonglong,  # CAkPath *
        ]
    ),
    "CAkPathManager::ProcessPathsList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPathManager *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPathManager::RemovePathFromList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPathManager *
            ctypes.c_ulonglong,  # CAkPath *
        ]
    ),
    "CAkPathManager::RemovePathUser": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPathManager *
            ctypes.c_ulonglong,  # CAkPath *
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkPathManager::RemovePotentialUser": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPathManager *
            ctypes.c_ulonglong,  # CAkPath *
        ]
    ),
    "CAkPathManager::Resume": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPathManager *
            ctypes.c_ulonglong,  # CAkPath *
        ]
    ),
    "CAkPathManager::SetPathsList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPathManager *
            ctypes.c_ulonglong,  # CAkPath *
            ctypes.c_ulonglong,  # AkPathListItem *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkPathMode
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # AkPathState *
        ]
    ),
    "CAkPathManager::Start": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPathManager *
            ctypes.c_ulonglong,  # CAkPath *
            ctypes.c_ulonglong,  # AkPathState *
        ]
    ),
    "CAkPathManager::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPathManager *
        ]
    ),
    "AkCaptureFile::AddMarkerData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkCaptureFile *
        ]
    ),
    "AkCaptureFile::AddOutputCaptureMarker": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkCaptureFile *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "AkCaptureMgr::Instance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkCaptureMgr *
        argtypes=[]
    ),
    "AkCaptureFile::PassSampleData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkCaptureFile *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkCaptureFile::StopCapture": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkCaptureFile *
        ]
    ),
    "CAkThreadedBankMgr::CAkThreadedBankMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkThreadedBankMgr *
        ]
    ),
    "CAkThreadedBankMgr::BankThreadFunc": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkThreadedBankMgr::CancelCookie": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkThreadedBankMgr *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkThreadedBankMgr::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkThreadedBankMgr *
        ]
    ),
    "CAkThreadedBankMgr::InitSyncOp": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkThreadedBankMgr *
            ctypes.c_ulonglong,  # AK::SoundEngine::AkSyncCaller *
        ]
    ),
    "CAkThreadedBankMgr::KillSlot": FUNCDEF(
        restype=ctypes.c_uint32,  # AKRESULT
        argtypes=[
            ctypes.c_ulonglong,  # CAkThreadedBankMgr *
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkThreadedBankMgr::QueueBankCommand": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkThreadedBankMgr *
            ctypes.c_ulonglong,  # CAkBankMgr::AkBankQueueItem *
        ]
    ),
    "CAkList2<CAkBankMgr::AkBankQueueItem,CAkBankMgr::AkBankQueueItem const &,1,0>::RemoveItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkList2<CAkBankMgr::AkBankQueueItem,CAkBankMgr::AkBankQueueItem const &,1,0> *
            ctypes.c_ulonglong,  # CAkList2<CAkBankMgr::AkBankQueueItem,CAkBankMgr::AkBankQueueItem const &,1,0>::ListItem *
            ctypes.c_ulonglong,  # CAkList2<CAkBankMgr::AkBankQueueItem,CAkBankMgr::AkBankQueueItem const &,1,0>::ListItem *
        ]
    ),
    "CAkThreadedBankMgr::StopThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkThreadedBankMgr *
        ]
    ),
    "CAkThreadedBankMgr::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkThreadedBankMgr *
        ]
    ),
    "CAkThreadedBankMgr::WaitForSyncOp": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkThreadedBankMgr *
            ctypes.c_ulonglong,  # AK::SoundEngine::AkSyncCaller *
            ctypes.c_uint32,  # AKRESULT
        ]
    ),
    "CAkGameObject::CreateComponent<CAkModifiedNodes>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkGameObject *
        ]
    ),
    "CAkRanSeqCntr::AddNewContListItem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkContinueListItem *
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # AkPBIParams *
        ]
    ),
    "CAkRanSeqCntr::CanPlayPosition": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # const CAkRandomInfo *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkRanSeqCntr::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkContainerBase *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkContainerMode
        ]
    ),
    "CAkRanSeqCntr::CreatePBI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_ulonglong,  # CAkSource *
            ctypes.c_ulonglong,  # AkPBIParams *
            ctypes.c_ulonglong,  # const PriorityInfoCurrent *
        ]
    ),
    "CAkRanSeqCntr::CreateRandomInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkRandomInfo *
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkRanSeqCntr::CreateSequenceInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
        ]
    ),
    "CAkRanSeqCntr::DeserializeHistory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # AK::IReadBytes *
        ]
    ),
    "CAkRanSeqCntr::DestroySpecificInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
        ]
    ),
    "AkSortedKeyArray<CAkContinuationList *,CAkMultiPlayNode::ContParam,AkArrayAllocatorNoAlign<0>,CAkMultiPlayNode::ContParamGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkMultiPlayNode::ContParam>,AkDefaultSortedKeyCompare<CAkContinuationList *>>::Exists": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMultiPlayNode::ContParam *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkContinuationList *,CAkMultiPlayNode::ContParam,AkArrayAllocatorNoAlign<0>,CAkMultiPlayNode::ContParamGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkMultiPlayNode::ContParam>,AkDefaultSortedKeyCompare<CAkContinuationList *> > *
            ctypes.c_ulonglong,  # CAkContinuationList *
        ]
    ),
    "CAkRanSeqCntr::GetExistingRandomInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkRandomInfo *
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkRanSeqCntr::GetExistingSequenceInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkContainerBaseInfo *
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkRanSeqCntr::GetNextToPlay": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkParameterNodeBase *
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkRanSeqCntr::GetNextToPlayContinuous": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkParameterNodeBase *
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # CAkContainerBaseInfo **
            ctypes.c_ulonglong,  # AkLoop *
        ]
    ),
    "AkArray<PluginRtpcSubscription,PluginRtpcSubscription const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<PluginRtpcSubscription>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<PluginRtpcSubscription,PluginRtpcSubscription const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<PluginRtpcSubscription> > *
        ]
    ),
    "CAkRanSeqCntr::IsInfiniteLooping": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # CAkPBIAware *
        ]
    ),
    "CAkParameterNode::Play": FUNCDEF(
        restype=ctypes.c_uint32,  # AKRESULT
        argtypes=[
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # AkPBIParams *
        ]
    ),
    "CAkRanSeqCntr::PlayInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # AkPBIParams *
        ]
    ),
    "CAkRanSeqCntr::ResetSpecificInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
        ]
    ),
    "CAkRanSeqCntr::SafeResetSpecificInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkRanSeqCntr::SelectRandomly": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # CAkRandomInfo *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # AkLoop *
        ]
    ),
    "CAkRanSeqCntr::SelectSequentially": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # CAkSequenceInfo *
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ulonglong,  # AkLoop *
        ]
    ),
    "CAkRanSeqCntr::SerializeHistory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # AK::IWriteBytes *
        ]
    ),
    "AkSortedKeyArray<CAkRegisteredObj *,CAkRanSeqCntr::CntrInfoEntry,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<CAkRegisteredObj *,CAkRanSeqCntr::CntrInfoEntry>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkRanSeqCntr::CntrInfoEntry>,AkDefaultSortedKeyCompare<CAkRegisteredObj *>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkRanSeqCntr::CntrInfoEntry *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkRegisteredObj *,CAkRanSeqCntr::CntrInfoEntry,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<CAkRegisteredObj *,CAkRanSeqCntr::CntrInfoEntry>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkRanSeqCntr::CntrInfoEntry>,AkDefaultSortedKeyCompare<CAkRegisteredObj *> > *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkRanSeqCntr::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkRanSeqCntr::SetPlaylistWithoutCheck": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkRanSeqCntr::TransitionMode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
        ]
    ),
    "CAkRanSeqCntr::TransitionTime": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkRanSeqCntr::Unregister": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "AkSortedKeyArray<CAkRegisteredObj *,CAkRanSeqCntr::CntrInfoEntry,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<CAkRegisteredObj *,CAkRanSeqCntr::CntrInfoEntry>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkRanSeqCntr::CntrInfoEntry>,AkDefaultSortedKeyCompare<CAkRegisteredObj *>>::Unset": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkRegisteredObj *,CAkRanSeqCntr::CntrInfoEntry,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<CAkRegisteredObj *,CAkRanSeqCntr::CntrInfoEntry>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkRanSeqCntr::CntrInfoEntry>,AkDefaultSortedKeyCompare<CAkRegisteredObj *> > *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkRanSeqCntr::UpdateNormalAvoidRepeat": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # CAkRandomInfo *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkRanSeqCntr::_PlayTrigger": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkRanSeqCntr *
            ctypes.c_ulonglong,  # AkPBIParams *
        ]
    ),
    "CAkVirtualAcousticsMgr::AddAcousticTexture": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkAcousticTexture *
        ]
    ),
    "CAkVirtualAcoustics::AddRef": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVirtualAcoustics *
        ]
    ),
    "CAkVirtualAcousticsMgr::GetAcousticTexture": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkIndexable *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkVirtualAcoustics::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVirtualAcoustics *
        ]
    ),
    "CAkAcousticTexture::SetAkProp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAcousticTexture *
            ctypes.c_uint32,  # AkVirtualAcousticsPropID
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CAkEnvironmentsMgr::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkEnvironmentsMgr *
        ]
    ),
    "CAkConversionTable::Linearize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConversionTable *
        ]
    ),
    "CAkEnvironmentsMgr::SetObsOccCurve": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkEnvironmentsMgr *
            ctypes.c_uint32,  # CAkEnvironmentsMgr::eCurveXType
            ctypes.c_uint32,  # CAkEnvironmentsMgr::eCurveYType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkGraphPointBase<float> *
            ctypes.c_uint32,  # AkCurveScaling
        ]
    ),
    "CAkEnvironmentsMgr::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkEnvironmentsMgr *
        ]
    ),
    "CAkPositionRepository::AddSource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPositionRepository *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
        ]
    ),
    "CAkPositionRepository::GetBuffering": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPositionRepository *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # AKRESULT *
        ]
    ),
    "CAkPositionRepository::GetCurrPositions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPositionRepository *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkSourcePosition *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkArray<MapStruct<unsigned __int64,float [9]>,MapStruct<unsigned __int64,float [9]> const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<unsigned __int64,float [9]>>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<CAkAttenuation::RTPCSubs,CAkAttenuation::RTPCSubs const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkAttenuation::RTPCSubs> > *
        ]
    ),
    "CAkPositionRepository::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPositionRepository *
        ]
    ),
    "CAkPositionRepository::RemovePlayingIDForBuffering": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPositionRepository *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPositionRepository::RemoveSource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPositionRepository *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
        ]
    ),
    "CAkPositionRepository::SetRate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPositionRepository *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
            ctypes.c_float,  # float
        ]
    ),
    "CAkPositionRepository::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPositionRepository *
        ]
    ),
    "CAkPositionRepository::UpdateBufferingInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPositionRepository *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
            ctypes.c_ulonglong,  # AkBufferingInformation *
        ]
    ),
    "CAkPositionRepository::UpdatePositionInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPositionRepository *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkBufferPosInformation *
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
        ]
    ),
    "AkCacheGameSyncMon::AddRef": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkCacheGameSyncMon *
        ]
    ),
    "AkSortedKeyArray<uint,AkKeyDataPtrStruct<uint,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0>>,AkArrayAllocatorNoAlign<0>,AkKeyDataPtrStruct<uint,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0>>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkKeyDataPtrStruct<uint,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0>>>,AkDefaultSortedKeyCompare<uint>>::BinarySearch": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkKeyDataPtrStruct<unsigned int,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0> > *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,AkKeyDataPtrStruct<unsigned int,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0> >,AkArrayAllocatorNoAlign<0>,AkKeyDataPtrStruct<unsigned int,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0> >,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkKeyDataPtrStruct<unsigned int,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0> > >,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AkSortedKeyArray<uint,uint,AkArrayAllocatorNoAlign<0>,AkSetGetKey<uint>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<uint>,AkDefaultSortedKeyCompare<uint>>::Exists": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned int *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,unsigned int,AkArrayAllocatorNoAlign<0>,AkSetGetKey<unsigned int>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned int>,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkSortedKeyArray<uint,AkKeyDataPtrStruct<uint,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0>>,AkArrayAllocatorNoAlign<0>,AkKeyDataPtrStruct<uint,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0>>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkKeyDataPtrStruct<uint,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0>>>,AkDefaultSortedKeyCompare<uint>>::Exists": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkKeyDataPtrStruct<unsigned int,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0> > *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,AkKeyDataPtrStruct<unsigned int,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0> >,AkArrayAllocatorNoAlign<0>,AkKeyDataPtrStruct<unsigned int,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0> >,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkKeyDataPtrStruct<unsigned int,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0> > >,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkKeyDataPtrStruct<AkCachePinnedFileSetKey,AkCachePinnedFileSet,AkArrayAllocatorNoAlign<0>>::FreeData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkKeyDataPtrStruct<AkCachePinnedFileSetKey,AkCachePinnedFileSet,AkArrayAllocatorNoAlign<0> > *
        ]
    ),
    "AkCachePinnedFileSet::GatherAndPinFilesInStreamCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkCachePinnedFileSet *
            ctypes.c_ulonglong,  # CAkEvent *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "GetBufferStatusForPinnedFiles": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkEvent *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AkArray<uint,uint const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<uint>>::Insert": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned int *
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<unsigned int,unsigned int const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "PinFilesInStreamCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkEvent *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_char,  # char
            ctypes.c_char,  # char
        ]
    ),
    "AkCacheGameSyncMon::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkCacheGameSyncMon *
        ]
    ),
    "AkSortedKeyArray<uint,uint,AkArrayAllocatorNoAlign<0>,AkSetGetKey<uint>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<uint>,AkDefaultSortedKeyCompare<uint>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned int *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,unsigned int,AkArrayAllocatorNoAlign<0>,AkSetGetKey<unsigned int>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned int>,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkSortedKeyArray<uint,AkKeyDataPtrStruct<uint,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0>>,AkArrayAllocatorNoAlign<0>,AkKeyDataPtrStruct<uint,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0>>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkKeyDataPtrStruct<uint,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0>>>,AkDefaultSortedKeyCompare<uint>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkKeyDataPtrStruct<unsigned int,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0> > *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,AkKeyDataPtrStruct<unsigned int,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0> >,AkArrayAllocatorNoAlign<0>,AkKeyDataPtrStruct<unsigned int,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0> >,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkKeyDataPtrStruct<unsigned int,AkCacheGameSyncMon,AkArrayAllocatorNoAlign<0> > >,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AkCacheGameSyncMon::SetSwitch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkCacheGameSyncMon *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "UnpinAllFilesInStreamCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkEvent *
        ]
    ),
    "UnpinFilesInStreamCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkEvent *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "AkCachePinnedFileSet::UnpinFilesInStreamCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkCachePinnedFileSet *
        ]
    ),
    "AkCachePinnedFileSet::UnsetAllGameSyncs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkCachePinnedFileSet *
        ]
    ),
    "CAkMidiDeviceMgr::AddCtx": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMidiBaseCtx *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiDeviceMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMidiDeviceMgr::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMidiDeviceMgr *
        argtypes=[]
    ),
    "CAkMidiDeviceMgr::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiDeviceMgr *
        ]
    ),
    "CAkMidiDeviceMgr::ExecuteAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiDeviceMgr *
            ctypes.c_uint32,  # ActionParamType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMidiDeviceMgr::GlobalCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkGlobalPluginContext *
            ctypes.c_uint32,  # AkGlobalCallbackLocation
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkMidiDeviceMgr::PostEvents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiDeviceMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkMIDIPost *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMidiDeviceMgr::StopAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiDeviceMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::GetPluginServiceMixer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::CAkPluginServiceMixer *
        argtypes=[]
    ),
    "AK::CAkPluginServiceMixer::Mix1inNChannels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::CAkPluginServiceMixer *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
        ]
    ),
    "AK::CAkPluginServiceMixer::MixChannel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::CAkPluginServiceMixer *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "AK::CAkPluginServiceMixer::MixNinNChannels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::CAkPluginServiceMixer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
        ]
    ),
    "CAkVPLFilterNodeBase::CAkVPLFilterNodeBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeBase *
        ]
    ),
    "CAkVPLFilterNodeOutOfPlace::CAkVPLFilterNodeOutOfPlace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeOutOfPlace *
        ]
    ),
    "CAkVPLSrcCbxNode::CAkVPLSrcCbxNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLSrcCbxNode::AddPipeline": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLSrcCbxNode::AddSrc": {
        "CAkVPLSrcCbxNode *, CAkPBI *, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
                ctypes.c_ulonglong,  # CAkPBI *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "CAkVPLSrcCbxNode *, CAkVPLSrcNode *, bool, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
                ctypes.c_ulonglong,  # CAkVPLSrcNode *
                ctypes.c_ubyte,  # bool
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "AreSourcesSACompatible": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
        ]
    ),
    "CAkVPLSrcCbxNode::ClearVPL": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLSrcCbxNode::ComputePriorityWithDistance": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_float,  # float
        ]
    ),
    "CAkVPLSrcCbxNode::ComputeVolumeRays": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLSrcCbxNode::ConsumeBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkMarkersQueue::Dequeue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMarkersQueue *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkVPLSrcCbxNode::FetchStreamedData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkVPLSrcCbxNode::GetAnalyzedEnvelope": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLSrcCbxNode::GetBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkVPLNode::GetChannelMappingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # {'return': 'unsigned __int8', 'args': ['const AkChannelConfig *', {'type': 'unsigned', 'name': '__int8'}]}
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLNode *
        ]
    ),
    "CAkVPL3dMixable::GetEmitterListenerPair": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkEmitterListenerPair *
        ]
    ),
    "CAkVPL3dMixable::GetGameObjectID": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
        ]
    ),
    "CAkVPL3dMixable::GetGameObjectMultiPositionType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
        ]
    ),
    "CAkVPL3dMixable::GetGameObjectPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkTransform *
        ]
    ),
    "CAkVPL3dMixable::GetGameObjectScaling": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
        ]
    ),
    "CAkVPL3dMixable::GetListenerData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # AkListener *
        ]
    ),
    "CAkVPL3dMixable::GetListeners": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkVPL3dMixable::GetNumEmitterListenerPairs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
        ]
    ),
    "CAkVPL3dMixable::GetNumGameObjectPositions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
        ]
    ),
    "CAkVPLSrcCbxNode::GetOneSourceBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkVPLNode::GetPitch": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLNode *
        ]
    ),
    "CAkVPLSrcCbxNode::GetPlayingID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLFilterNode::GetPlugin": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkOutOfPlaceEffectPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeOutOfPlace *
        ]
    ),
    "CAkVPLSrcCbxNode::GetPriority": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLSrcCbxNode::HandleSourceStarvation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLSrcCbxNode::IsUsingThisSlot": {
        "CAkVPLSrcCbxNode *, const unsigned __int8 *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
                ctypes.c_ulonglong,  # const unsigned __int8 *
            ]
        ),
        "CAkVPLSrcCbxNode *, const CAkUsageSlot *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
                ctypes.c_ulonglong,  # const CAkUsageSlot *
            ]
        ),
    },
    "CAkVPLSrcCbxNode::NotifyMarkers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_ulonglong,  # AkPipelineBuffer *
        ]
    ),
    "CAkVPLSrcCbxNode::Pause": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLNode::PopMarkers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLNode *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkVPLSrcCbxNode::RefreshBypassFx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLSrcCbxNode::ReleaseBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLSrcCbxNode::RelocateMedia": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "CAkVPLSrcCbxNode::RemovePipeline": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_int64,  # __int64
        ]
    ),
    "AkArray<AkAudioObject::CustomMetadata,AkAudioObject::CustomMetadata const &,AkPluginArrayAllocator,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAudioObject::CustomMetadata>>::Resize": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkAudioObject::CustomMetadata,AkAudioObject::CustomMetadata const &,AkPluginArrayAllocator,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAudioObject::CustomMetadata> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkVPLSrcCbxNode::RestorePreviousVolumes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_ulonglong,  # AkPipelineBuffer *
        ]
    ),
    "CAkVPLSrcCbxNode::Resume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLNode::Seek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLNode *
        ]
    ),
    "CAkVPLSrcCbxNode::SeekSource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLSrcCbxNode::SetFxBypass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkVPLSrcCbxNode::SetMixableBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLSrcCbxNode::Start": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLSrcCbxNode::StartRun": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkVPLSrcCbxNode::Stop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLSrcCbxNode::StopLooping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkVPLSrcCbxNode::SwitchToNextSrc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLSrcCbxNode::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLSrcCbxNode::TrySwitchToNextSrc": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_ulonglong,  # const AkPipelineBuffer *
        ]
    ),
    "CAkVPLSrcCbxNode::UpdateBypass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int16,  # __int16
        ]
    ),
    "CAkVPLSrcCbxNode::UpdateFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkVPLSrcCbxNode::UpdateMakeUpLinearNormalized": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
        ]
    ),
    "CAkVPLSrcCbxNode::VerifySATransition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLNode::VirtualOff": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLNode *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkVPLNode::VirtualOn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLVolAutmNode *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkVPLSrcCbxNode::_GetVoiceInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkVoicePluginInfo *
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "AkFXMemAlloc::Malloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # AkFXMemAlloc *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkFXMemAlloc::Malign": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # AkFXMemAlloc *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkFXMemAlloc::Realloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # AkFXMemAlloc *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkFXMemAlloc::ReallocAligned": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # AkFXMemAlloc *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkFXMemAlloc::Free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkFXMemAlloc *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkSrcMedia::CAkSrcMedia": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # {'return': 'IAkSrcMediaCodec *', 'args': ['const AK::SrcMedia::Header *']}
        ]
    ),
    "CAkSrcMedia::ChangeSourcePosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
        ]
    ),
    "CAkSrcMedia::GetAnalyzedEnvelope": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSrcMedia::GetBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkSrcMedia::GetChannelMappingFunc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # {'return': '__int64', 'args': [{'type': 'const AkChannelConfig*', 'name': 'config'}, {'type': 'unsigned __int8', 'name': 'idx'}]}
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
        ]
    ),
    "CAkSrcMedia::GetDuration": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
        ]
    ),
    "CAkSrcMedia::GetPitch": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
        ]
    ),
    "CAkVPLSrcNode::GetPitch": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
        ]
    ),
    "CAkSrcMedia::GetSourceOffset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned __int16 *
        ]
    ),
    "CAkSrcMedia::InitCodec": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
            ctypes.c_ulonglong,  # AK::SrcMedia::Header *
        ]
    ),
    "CAkSrcMedia::MustRelocateAnalysisDataOnMediaRelocation": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
        ]
    ),
    "CAkSrcMedia::MustRelocatePitchInputBufferOnMediaRelocation": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
        ]
    ),
    "CAkSrcMedia::PrepareNextBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSrcMedia::PrepareSourceSwitch": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
        ]
    ),
    "CAkSrcMedia::ReleaseBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
        ]
    ),
    "CAkSrcMedia::RelocateMedia": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "CAkSrcMedia::Seek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
        ]
    ),
    "CAkSrcMedia::SeekTo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkSrcMedia::SrcProcessOrder": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
        ]
    ),
    "CAkSrcMedia::StartStream": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkSrcMedia::StopLooping": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
        ]
    ),
    "CAkSrcMedia::StopStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
        ]
    ),
    "CAkSrcMedia::SupportResampling": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
        ]
    ),
    "CAkSrcMedia::TimeSkip": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkSrcMedia::UpdateBufferingInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
        ]
    ),
    "CAkSrcMedia::VirtualOff": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
            ctypes.c_uint32,  # AkVirtualQueueBehavior
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkSrcMedia::VirtualOn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMedia *
            ctypes.c_uint32,  # AkVirtualQueueBehavior
        ]
    ),
    "CAkSrcMediaCodecPCM::CAkSrcMediaCodecPCM": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecPCM *
        ]
    ),
    "CAkSrcMediaCodecPCM::FindClosestFileOffset": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecPCM *
            ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # IAkSrcMediaCodec::SeekInfo *
        ]
    ),
    "CAkSrcMediaCodecPCM::GetBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecPCM *
            ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
            ctypes.c_ulonglong,  # AK::SrcMedia::Stream::State *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # IAkSrcMediaCodec::BufferInfo *
        ]
    ),
    "CAkSrcMediaCodecPCM::Init": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecPCM *
            ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
            ctypes.c_ulonglong,  # const AK::SrcMedia::Header *
            ctypes.c_ulonglong,  # AK::SrcMedia::CodecInfo *
        ]
    ),
    "CAkSrcMediaCodecPCM::SwitchToNext": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecVorbis *
            ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
            ctypes.c_ulonglong,  # IAkSrcMediaCodec *
        ]
    ),
    "CAkSrcMediaCodecPCM::ReleaseBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecPCM *
            ctypes.c_ulonglong,  # AK::SrcMedia::Stream::State *
        ]
    ),
    "CAkSrcMediaCodecPCM::Seek": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecPCM *
            ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
            ctypes.c_ulonglong,  # AK::SrcMedia::Stream::State *
            ctypes.c_ulonglong,  # const IAkSrcMediaCodec::SeekInfo *
        ]
    ),
    "CAkSrcMediaCodecPCM::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecPCM *
        ]
    ),
    "CAkConnectedListeners::AddListeners": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConnectedListeners *
            ctypes.c_ulonglong,  # const AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
        ]
    ),
    "AkSortedKeyArray<unsigned __int64,unsigned __int64,AkHybridAllocator<8,16,4>,AkSetGetKey<unsigned __int64>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64>,AkDefaultSortedKeyCompare<unsigned __int64>>::BinarySearch": FUNCDEF(
        restype=ctypes.c_ulonglong,  # unsigned __int64 *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned __int64,unsigned __int64,AkHybridAllocator<8,16,4>,AkSetGetKey<unsigned __int64>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64>,AkDefaultSortedKeyCompare<unsigned __int64> > *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkConnectedListeners::BuildListenerSet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConnectedListeners *
        ]
    ),
    "CAkLstnrAssocs::CopyFrom": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkLstnrAssocs *
            ctypes.c_ulonglong,  # const CAkLstnrAssocs *
        ]
    ),
    "CreateListenerComponents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
        ]
    ),
    "CAkLstnrAssocs::GetGains": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLstnrAssocs *
            ctypes.c_ulonglong,  # AkArray<AkAuxSendValueEx,AkAuxSendValueEx const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAuxSendValueEx> > *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
        ]
    ),
    "AkArray<CAkLstnrAssoc,CAkLstnrAssoc const &,AkHybridAllocator<24,16,4>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkLstnrAssoc>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<CAkLstnrAssoc,CAkLstnrAssoc const &,AkHybridAllocator<24,16,4>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkLstnrAssoc> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkLstnrAssocs::PreAssocRemoved": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLstnrAssocs *
            ctypes.c_ulonglong,  # CAkLstnrAssoc *
        ]
    ),
    "CAkConnectedListeners::RemoveListeners": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConnectedListeners *
            ctypes.c_ulonglong,  # const AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
        ]
    ),
    "CAkConnectedListeners::ResetListenersToDefault": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConnectedListeners *
        ]
    ),
    "AkArray<CAkLstnrAssoc,CAkLstnrAssoc const &,AkHybridAllocator<24,16,4>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkLstnrAssoc>>::Resize": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<CAkLstnrAssoc,CAkLstnrAssoc const &,AkHybridAllocator<24,16,4>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkLstnrAssoc> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkConnectedListeners::SetAllUserGains": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConnectedListeners *
            ctypes.c_float,  # float
        ]
    ),
    "CAkConnectedListeners::SetAuxGains": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConnectedListeners *
            ctypes.c_ulonglong,  # const AkAuxSendValue *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkLstnrAssocs::SetGain": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkLstnrAssocs *
            ctypes.c_ulonglong,  # CAkLstnrAssoc::Key *
            ctypes.c_float,  # float
        ]
    ),
    "CAkLstnrAssocs::SetGains": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkLstnrAssocs *
            ctypes.c_ulonglong,  # const AkAuxSendValue *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkConnectedListeners::SetListeners": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConnectedListeners *
            ctypes.c_ulonglong,  # const AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
        ]
    ),
    "CAkLstnrAssocs::SetListeners": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkLstnrAssocs *
            ctypes.c_ulonglong,  # const AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
        ]
    ),
    "CAkConnectedListeners::SetUserGain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConnectedListeners *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
        ]
    ),
    "CAkConnectedListeners::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkConnectedListeners *
        ]
    ),
    "CAkLstnrAssocs::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLstnrAssocs *
        ]
    ),
    "CAkEvent::AddRef": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkEvent *
        ]
    ),
    "CAkEvent::AddToIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkEvent *
        ]
    ),
    "CAkEvent::CreateNoIndex": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkEvent *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkEvent::GatherSounds": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkEvent *
            ctypes.c_ulonglong,  # AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *
            ctypes.c_ulonglong,  # AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *
            ctypes.c_ulonglong,  # AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkEvent::QueryAudioObjectIDs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkEvent *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # AkObjectInfo *
        ]
    ),
    "CAkEvent::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkEvent *
        ]
    ),
    "CAkEvent::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkEvent *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPath::CAkPath": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
        ]
    ),
    "CAkPath::~CAkPath": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
        ]
    ),
    "CAkPath::GetNextPathList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
        ]
    ),
    "CAkPath::InitRotationMatricesForNoFollowMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
            ctypes.c_ulonglong,  # const AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional> *
        ]
    ),
    "CAkPath::InitRotationMatrixForListener": FUNCDEF(
        restype=ctypes.c_ulonglong,  # float *
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "CAkPath::IsIdle": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
        ]
    ),
    "CAkPath::NextVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
        ]
    ),
    "CAkPath::Pause": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPath::PickRandomList": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
        ]
    ),
    "CAkPath::RandomizePosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
            ctypes.c_ulonglong,  # AkVector *
        ]
    ),
    "CAkPath::Resume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPath::SetIsLooping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkPath::SetPathsList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
            ctypes.c_ulonglong,  # AkPathListItem *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkPathMode
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # AkPathState *
        ]
    ),
    "CAkPath::SetPlayingID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPath::Start": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPath::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
        ]
    ),
    "CAkPath::UpdatePosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPath::UpdateStartPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPath *
        ]
    ),
    "Ak3DAutomationParams::Ak3DAutomationParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Ak3DAutomationParams *
        ]
    ),
    "AkPositioningParams::AkPositioningParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkPositioningParams *
        ]
    ),
    "AkPositioningParams::ReleaseAttenuation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkPositioningParams *
        ]
    ),
    "CAk3DAutomationParamsEx::~CAk3DAutomationParamsEx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAk3DAutomationParamsEx *
        ]
    ),
    "CAk3DAutomationParamsEx::FreePathInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAk3DAutomationParamsEx *
        ]
    ),
    "CAk3DAutomationParams::PathIsDifferent": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAk3DAutomationParams *
            ctypes.c_ulonglong,  # AkPathVertex *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkPathListItemOffset *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAk3DAutomationParamsEx::SetPath": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAk3DAutomationParamsEx *
            ctypes.c_ulonglong,  # AkPathVertex *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkPathListItemOffset *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAk3DAutomationParams::SetPathPlayList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAk3DAutomationParams *
            ctypes.c_ulonglong,  # CAkPath *
            ctypes.c_ulonglong,  # AkPathState *
        ]
    ),
    "CAk3DAutomationParams::SetTransition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAk3DAutomationParams *
            ctypes.c_int32,  # int
        ]
    ),
    "CAkStateAware::GetStateParamValues<PluginRTPCSub,AccumulateAdd>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkListBareLight<AkStateGroupChunk,AkListBareLightNextInNode> *
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
            ctypes.c_ulonglong,  # PluginRTPCSub *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkStateAware::GetStateParamValues<PluginRTPCSub,AccumulateMult>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkListBareLight<AkStateGroupChunk,AkListBareLightNextInNode> *
        argtypes=[
            ctypes.c_ulonglong,  # CAkStateAware *
            ctypes.c_ulonglong,  # PluginRTPCSub *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkFxBase::CAkFxBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkFxBase::~CAkFxBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxBase *
        ]
    ),
    "CAkAudioDevice::AddRef": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioDevice *
        ]
    ),
    "CAkFxCustom::AddRef": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxCustom *
        ]
    ),
    "CAkFxShareSet::AddRef": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxShareSet *
        ]
    ),
    "PluginRTPCSub::CalculatePropertyUpdateStates": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PluginRTPCSub *
        ]
    ),
    "PluginRTPCSub::Clone": {
        "PluginRTPCSub *, CAkFxBase *, const AkRTPCKey *, CAkModulatorData *, bool": FUNCDEF(
            restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
            argtypes=[
                ctypes.c_ulonglong,  # PluginRTPCSub *
                ctypes.c_ulonglong,  # CAkFxBase *
                ctypes.c_ulonglong,  # const AkRTPCKey *
                ctypes.c_ulonglong,  # CAkModulatorData *
                ctypes.c_ubyte,  # bool
            ]
        ),
        "PluginRTPCSub *, CAkFxBase *, CAkBehavioralCtx *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
            argtypes=[
                ctypes.c_ulonglong,  # PluginRTPCSub *
                ctypes.c_ulonglong,  # CAkFxBase *
                ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ]
        ),
    },
    "CAkAudioDevice::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkAudioDevice *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkFxCustom::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkFxCustom *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkFxShareSet::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkFxShareSet *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "PluginRTPCSub::GetPluginID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkFxBase *
        argtypes=[
            ctypes.c_ulonglong,  # PluginRTPCSub *
        ]
    ),
    "CAkFxBaseStateAware::GetStateProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkArray<AkStatePropertyInfo,AkStatePropertyInfo &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkStatePropertyInfo> > *
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxBaseStateAware *
        ]
    ),
    "CAkFxBase::RecalculatePropertyValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxBase *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkFxBase::RecalculatePropertyValues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxBase *
        ]
    ),
    "CAkAudioDevice::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioDevice *
        ]
    ),
    "CAkFxCustom::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxCustom *
        ]
    ),
    "CAkFxShareSet::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxShareSet *
        ]
    ),
    "AkArray<PluginPropertyUpdate,PluginPropertyUpdate const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<PluginPropertyUpdate>>::Resize": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<PluginPropertyUpdate,PluginPropertyUpdate const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<PluginPropertyUpdate> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkFxBase::SetFX": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxBase *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAudioDevice::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioDevice *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkFxBase::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxBase *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkFxBase::SetRTPC": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxBase *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkRtpcType
            ctypes.c_uint32,  # AkRtpcAccum
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkCurveScaling
            ctypes.c_ulonglong,  # AkGraphPointBase<float> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkFxBase::SubscribeRTPC": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxBase *
            ctypes.c_ulonglong,  # IAkRTPCSubscriberPlugin *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "PluginRTPCSub::SubscribeRTPC": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PluginRTPCSub *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # CAkModulatorData *
        ]
    ),
    "AkSinkPluginParams::TakeFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkSinkPluginParams *
            ctypes.c_ulonglong,  # AkSinkPluginParams *
        ]
    ),
    "PluginRTPCSub::TakeFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PluginRTPCSub *
            ctypes.c_ulonglong,  # PluginRTPCSub *
        ]
    ),
    "AkAudioDeviceEffectParams::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkSinkPluginParams *
        ]
    ),
    "PluginRTPCSub::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PluginRTPCSub *
        ]
    ),
    "CAkAudioDevice::UpdateBypass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioDevice *
        ]
    ),
    "CAkAudioDevice::UpdateFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAudioDevice *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "PluginRTPCSub::UpdateTargetParam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PluginRTPCSub *
            ctypes.c_uint32,  # PluginPropertyUpdateType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # AkRtpcAccum
            ctypes.c_float,  # float
        ]
    ),
    "PluginRTPCSub::_Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # PluginRTPCSub *
            ctypes.c_ulonglong,  # CAkFxBase *
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # CAkModulatorData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "PluginRTPCSub::_RecalculatePropertyUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # PluginRTPCSub *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_float,  # float
        ]
    ),
    "CAkVolumeRayComponent::~CAkVolumeRayComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVolumeRayComponent *
        ]
    ),
    "AkRayBuilder::BuildApiRays": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRayBuilder *
            ctypes.c_ulonglong,  # const CAkEmitter *
            ctypes.c_ulonglong,  # const AkSet<unsigned __int64,AkHybridAllocator<32,16,4>,AkGrowByPolicy_Proportional> *
            ctypes.c_ulonglong,  # const CAkConnectedListeners *
        ]
    ),
    "CAkVolumeRayComponent::BuildCachedVolumeRays": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVolumeRayComponent *
        ]
    ),
    "CAkVolumeRayComponent::GetRayVolumeData": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkVolumeRayComponent *
            ctypes.c_ulonglong,  # AkArray<AkRayVolumeData,AkRayVolumeData const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRayVolumeData> > *
            ctypes.c_ulonglong,  # CAkSpatialAudioVoice *
        ]
    ),
    "AkArray<AkRayVolumeData,AkRayVolumeData const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRayVolumeData>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkRayVolumeData,AkRayVolumeData const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRayVolumeData> > *
        ]
    ),
    "AkArray<unsigned __int64,unsigned __int64 const &,AkHybridAllocator<32,16,4>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<unsigned __int64,unsigned __int64 const &,AkHybridAllocator<32,16,4>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned __int64> > *
        ]
    ),
    "AkRayBuilder::SubmitLast": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRayBuilder *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkVolumeRayComponent::UpdateBuiltInParamValues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVolumeRayComponent *
            ctypes.c_ulonglong,  # const AkRayVolumeData *
            ctypes.c_ulonglong,  # const CAkListener *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CAkGameObject::CreateComponent<CAkSpatialAudioComponent>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkGameObject *
        ]
    ),
    "CAkSpatialAudioReflectBusCtx::GetAuxBusID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioReflectBusCtx *
        ]
    ),
    "CAkSpatialAudioEmitterVoiceCtx::GetAuxSendsValues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioEmitterVoiceCtx *
            ctypes.c_ulonglong,  # AkArray<AkAuxSendValueEx,AkAuxSendValueEx const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAuxSendValueEx> > *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CAkSpatialAudioReflectBusCtx::GetAuxSendsValues": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioReflectBusCtx *
            ctypes.c_ulonglong,  # AkArray<AkAuxSendValueEx,AkAuxSendValueEx const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAuxSendValueEx> > *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CAkSpatialAudioEmitterVoiceCtx::GetReflectionsAuxBus": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkBehavioralCtx *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioEmitterVoiceCtx *
        ]
    ),
    "CAkSpatialAudioVoice::GetVolumeRayCategories": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioVoice *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkSpatialAudioEmitterVoiceCtx::HasTransmission": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioEmitterVoiceCtx *
        ]
    ),
    "CAkSpatialAudioEmitterVoiceCtx::HasReflections": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioEmitterVoiceCtx *
        ]
    ),
    "CAkSpatialAudioEmitterVoiceCtx::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioEmitterVoiceCtx *
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkSpatialAudioReflectBusCtx::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioReflectBusCtx *
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkSpatialAudioRoomBusCtx::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioRoomBusCtx *
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkSpatialAudioVoice::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioVoice *
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
        ]
    ),
    "CAkSpatialAudioEmitterVoiceCtx::ParamsUpdated": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioEmitterVoiceCtx *
        ]
    ),
    "CAkSpatialAudioEmitterVoiceCtx::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioEmitterVoiceCtx *
        ]
    ),
    "CAkSpatialAudioReflectBusCtx::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioReflectBusCtx *
        ]
    ),
    "CAkSpatialAudioRoomBusCtx::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioRoomBusCtx *
        ]
    ),
    "CAkScopedRtpcObj::CAkScopedRtpcObj": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScopedRtpcObj *
        ]
    ),
    "CAkScopedRtpcObj::AddedNewModulatorCtx": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkScopedRtpcObj::AddedNewRtpcValue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkScopedRtpcObj::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkScopedRtpcObj *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkLEngineCmds::DequeuePBI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkLEngineCmds::DestroyDisconnectedSources": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkLEngineCmds::EnqueueAction": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # AkLECmd::Type
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkLEngineCmds::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "CAkLEngineCmds::ProcessAllCommands": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkLEngineCmds::ProcessDisconnectedSources": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkLEngineCmds::ProcessPendingCommands": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkLEngineCmds::ProcessPlayCommands": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkLEngineCmds::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkMidiNoteEvent::CAkMidiNoteEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiNoteEvent *
            ctypes.c_ulonglong,  # CAkMidiBaseCtx *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkMidiNoteState::~CAkMidiNoteState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiNoteState *
        ]
    ),
    "CAkMidiNoteState::AddRef": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiNoteState *
        ]
    ),
    "CAkMidiNoteEvent::NoteOnToNoteOff": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiNoteEvent *
        ]
    ),
    "CAkMidiNoteEvent::PairWithExistingNote": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiNoteEvent *
            ctypes.c_ulonglong,  # CAkMidiNoteEvent *
        ]
    ),
    "CAkMidiNoteEvent::PlayNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiNoteEvent *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # const UserParams *
            ctypes.c_ulonglong,  # const AkMidiEventEx *
        ]
    ),
    "CAkMidiNoteEvent::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiNoteEvent *
        ]
    ),
    "CAkMidiNoteState::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiNoteState *
        ]
    ),
    "CAkMidiNoteEvent::ScheduleMidiEvent": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiNoteEvent *
            ctypes.c_ulonglong,  # const AkMidiEventEx *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMidiNoteEvent::StopPBIsNoFade": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiNoteEvent *
            ctypes.c_int64,  # __int64
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "CAkMidiNoteEvent::Update": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiNoteEvent *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # AkListBare<CAkMidiNoteEvent,AkListBareNextItem,AkCountPolicyNoCount,AkLastPolicyWithLast> *
        ]
    ),
    "CAkMidiNoteEvent::_ExecutePause": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiNoteEvent *
            ctypes.c_int32,  # int
        ]
    ),
    "CAkMidiNoteEvent::_ExecuteResume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiNoteEvent *
            ctypes.c_int32,  # int
        ]
    ),
    "CAkMidiNoteEvent::_ExecuteWwiseCmd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiNoteEvent *
            ctypes.c_ulonglong,  # AkListBare<CAkMidiNoteEvent,AkListBareNextItem,AkCountPolicyNoCount,AkLastPolicyWithLast> *
        ]
    ),
    "AkMath::Distance": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const AkVector *
            ctypes.c_ulonglong,  # const AkVector *
        ]
    ),
    "CAkEffectSlots::GetFX": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkSmartPtr<CAkFxBase> *
        argtypes=[
            ctypes.c_ulonglong,  # CAkEffectSlots *
            ctypes.c_ulonglong,  # CAkSmartPtr<CAkFxBase> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkEffectSlots::SetAsRendered": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkEffectSlots *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkEffectSlots::SetFX": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkEffectSlots *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # SharedSetOverride
        ]
    ),
    "CAkEffectSlots::SetFXBypass": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkEffectSlots *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkEffectSlots::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkEffectSlots *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkEffectSlots::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkEffectSlots *
        ]
    ),
    "CAkSIS::~CAkSIS": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSIS *
        ]
    ),
    "CAkSIS::GetSISValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkSISValue *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSIS *
            ctypes.c_uint32,  # AkPropID
            ctypes.c_float,  # float
        ]
    ),
    "CAkSIS::TransUpdateValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSIS *
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkAction::CAkAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAction *
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAction::~CAkAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAction *
        ]
    ),
    "CAkAction::AddRef": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAction *
        ]
    ),
    "CAkAction::AddToIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAction *
        ]
    ),
    "CAkAction::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionSetAkProp *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAction::GetAndRefTarget": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkParameterNodeBase *
        argtypes=[
            ctypes.c_ulonglong,  # CAkAction *
        ]
    ),
    "CAkAction::GetDelayTime": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAction *
        ]
    ),
    "CAkAction::GetTransitionTime": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAction *
        ]
    ),
    "CAkAction::PlayDirectly": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAction *
        ]
    ),
    "CAkAction::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAction *
        ]
    ),
    "CAkAction::SetAkProp": {
        "CAkAction *, AkPropID, int, int, int": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # CAkAction *
                ctypes.c_uint32,  # AkPropID
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
                ctypes.c_int32,  # int
            ]
        ),
        "CAkAction *, AkPropID, float, float, float": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # CAkAction *
                ctypes.c_uint32,  # AkPropID
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_float,  # float
            ]
        ),
    },
    "CAkAction::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAction *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAction::SetPlayDirectly": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAction *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSound::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkSound *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSound::ExecuteAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSound *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkSound::ExecuteActionExcept": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSound *
            ctypes.c_ulonglong,  # ActionParamsExcept *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkSound::ExecuteActionInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSound *
            ctypes.c_ulonglong,  # ActionParams *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "CAkSound::ExecuteActionNoPropagate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSound *
            ctypes.c_ulonglong,  # ActionParams *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "CAkSound::GatherSounds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSound *
            ctypes.c_ulonglong,  # AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *
            ctypes.c_ulonglong,  # AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *
            ctypes.c_ulonglong,  # AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSound::GetAudioParameters": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSound *
            ctypes.c_ulonglong,  # AkSoundParams *
            ctypes.c_ulonglong,  # AkMutedMap *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # AkPBIModValues *
            ctypes.c_ulonglong,  # AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkSound::IsPlayable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkSound *
        ]
    ),
    "CAkSound::PlayInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSound *
            ctypes.c_ulonglong,  # AkPBIParams *
        ]
    ),
    "CAkSound::PrepareData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSound *
        ]
    ),
    "CAkSound::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSound *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # CAkUsageSlot *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSound::UnPrepareData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSound *
        ]
    ),
    "CAkActionStop::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionStop *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionStop::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionStop *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkActionStop::SetActionActiveParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionStop *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkActionStop::SetActionSpecificParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionStop *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkActionPlayAndContinue::~CAkActionPlayAndContinue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
        ]
    ),
    "CAkActionPlayAndContinue::AssignMidi": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
            ctypes.c_uint32,  # MidiEventActionType
            ctypes.c_ulonglong,  # CAkMidiNoteState *
            ctypes.c_ulonglong,  # const AkMidiEventEx *
        ]
    ),
    "CAkActionPlayAndContinue::AssignModulator": {
        "CAkActionPlayAndContinue *, CAkModCtxRefContainer *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
                ctypes.c_ulonglong,  # CAkModCtxRefContainer *
            ]
        ),
        "CAkActionPlayAndContinue *, CAkModulatorData *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
                ctypes.c_ulonglong,  # CAkModulatorData *
            ]
        ),
    },
    "CAkActionPlayAndContinue::BreakToNode": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkActionPlayAndContinue::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionPlay *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkSmartPtr<CAkContinuationList> *
        ]
    ),
    "CAkActionPlayAndContinue::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkActionPlayAndContinue::GetHistArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
            ctypes.c_ulonglong,  # AkCntrHistArray *
        ]
    ),
    "CAkActionPlayAndContinue::HasModulator": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
            ctypes.c_ulonglong,  # CAkModulatorCtx *
        ]
    ),
    "CAkActionPlayAndContinue::NeedNotifyDelay": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
        ]
    ),
    "CAkActionPlayAndContinue::Resume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
        ]
    ),
    "CAkActionPlayAndContinue::SetFadeBack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
            ctypes.c_ulonglong,  # CAkContinuousPBI *
            ctypes.c_int32,  # int
        ]
    ),
    "CAkActionPlayAndContinue::SetHistory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
            ctypes.c_ulonglong,  # PlayHistory *
        ]
    ),
    "CAkActionPlayAndContinue::SetInitialPlaybackState": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
            ctypes.c_uint32,  # AkPlaybackState
        ]
    ),
    "CAkActionPlayAndContinue::SetInstigator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
            ctypes.c_ulonglong,  # CAkPBIAware *
        ]
    ),
    "CAkActionPlayAndContinue::SetIsFirstPlay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkActionPlayAndContinue::SetPathInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
            ctypes.c_ulonglong,  # AkPathInfo *
        ]
    ),
    "CAkActionPlayAndContinue::SetPauseResumeTransition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
            ctypes.c_ulonglong,  # CAkTransition *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkActionPlayAndContinue::SetPlayStopTransition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
            ctypes.c_ulonglong,  # CAkTransition *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkActionPlayAndContinue::SetSAInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionPlayAndContinue::StartAsPaused": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
        ]
    ),
    "CAkActionPlayAndContinue::UnsetFadeBack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayAndContinue *
            ctypes.c_ulonglong,  # CAkContinuousPBI *
        ]
    ),
    "AkCustomPluginDataStore::GetPluginCustomGameData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AkCustomPluginDataStore::SetPluginCustomGameData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # AkPluginType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkCustomPluginDataStore::TermPluginCustomGameData": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkMidiBaseCtx::CAkMidiBaseCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseCtx *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # TransParams *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMidiBaseCtx::~CAkMidiBaseCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseCtx *
        ]
    ),
    "CAkMidiBaseCtx::AddMidiEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseCtx *
            ctypes.c_ulonglong,  # AkListBare<AkMidiFrameEvent,AkListBareNextItem,AkCountPolicyNoCount,AkLastPolicyWithLast> *
            ctypes.c_ulonglong,  # const AkMidiEventEx *
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMidiBaseCtx::AddRef": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseCtx *
        ]
    ),
    "CAkMidiBaseCtx::CleanupActions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseCtx *
        ]
    ),
    "CAkMidiBaseCtx::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseCtx *
        ]
    ),
    "CAkMidiBaseCtx::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseCtx *
        ]
    ),
    "CAkMidiBaseMgr::CAkMidiBaseMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseMgr *
        ]
    ),
    "CAkMidiBaseMgr::~CAkMidiBaseMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseMgr *
            ctypes.c_ulonglong,  # MapStruct<CAkMidiBaseMgr::TargetKey,CAkMidiBaseMgr::TargetInfo> *
        ]
    ),
    "CAkMidiBaseMgr::AttachCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseMgr *
            ctypes.c_ulonglong,  # CAkMidiBaseCtx *
        ]
    ),
    "CAkMidiBaseMgr::CcUpdateMidiNotes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseMgr *
            ctypes.c_ulonglong,  # CAkMidiBaseMgr::TargetInfo *
            ctypes.c_ulonglong,  # CAkMidiBaseCtx *
            ctypes.c_ulonglong,  # const AkMidiEventEx *
        ]
    ),
    "CAkMidiBaseMgr::CleanupActions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseMgr *
            ctypes.c_ulonglong,  # CAkMidiBaseCtx *
        ]
    ),
    "CAkMidiBaseMgr::DetachCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseMgr *
            ctypes.c_ulonglong,  # CAkMidiBaseCtx *
        ]
    ),
    "CAkMidiBaseMgr::KillAllNotes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseMgr *
            ctypes.c_ulonglong,  # CAkMidiBaseCtx *
        ]
    ),
    "CAkMidiBaseMgr::KillNotes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseMgr *
            ctypes.c_ulonglong,  # CAkMidiBaseCtx *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "CAkMidiBaseMgr::OnNoteStateFinished": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseMgr *
            ctypes.c_ulonglong,  # CAkMidiNoteState *
        ]
    ),
    "CAkMidiBaseMgr::OnPaused": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseMgr *
            ctypes.c_ulonglong,  # CAkMidiBaseCtx *
        ]
    ),
    "CAkMidiBaseMgr::OnResumed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseMgr *
            ctypes.c_ulonglong,  # CAkMidiBaseCtx *
        ]
    ),
    "CAkMidiBaseMgr::ScheduleMidiEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseMgr *
            ctypes.c_ulonglong,  # CAkMidiBaseCtx *
            ctypes.c_ulonglong,  # const AkMidiEventEx *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMidiBaseMgr::ScheduleMidiEvents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseMgr *
            ctypes.c_ulonglong,  # AkListBare<AkMidiFrameEvent,AkListBareNextItem,AkCountPolicyNoCount,AkLastPolicyWithLast> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMidiBaseMgr::UpdateMidiNotes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiBaseMgr *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkBankCallbackMgr::CAkBankCallbackMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankCallbackMgr *
        ]
    ),
    "CAkBankCallbackMgr::~CAkBankCallbackMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankCallbackMgr *
        ]
    ),
    "CAkBankCallbackMgr::AddCookie": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankCallbackMgr *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkBankCallbackMgr::CancelCookie": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankCallbackMgr *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkBankCallbackMgr::DoCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankCallbackMgr *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'unsigned', 'name': 'int'}, 'const void *', 'AKRESULT', 'void *']}
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # AKRESULT
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkBankList::Init": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkBankList *
        ]
    ),
    "AkHashListBare<AkBankKey,CAkUsageSlot,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<AkBankKey,CAkUsageSlot>>::Resize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkHashListBare<AkBankKey,CAkUsageSlot,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<AkBankKey,CAkUsageSlot> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAttenuation::AddRef": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAttenuation *
        ]
    ),
    "CAkAttenuation::ClearRTPCs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkAttenuation *
        ]
    ),
    "CAkAttenuation::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkAttenuation *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAttenuation::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAttenuation *
        ]
    ),
    "CAkConversionTable::Set": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkConversionTable *
            ctypes.c_ulonglong,  # AkGraphPointBase<float> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkCurveScaling
        ]
    ),
    "CAkAttenuation::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAttenuation *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAttenuation::SetRTPC": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAttenuation *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkRtpcType
            ctypes.c_uint32,  # AkRtpcAccum
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkCurveScaling
            ctypes.c_ulonglong,  # AkGraphPointBase<float> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkState::AddRef": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkState *
        ]
    ),
    "CAkState::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkState *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkState::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkState *
        ]
    ),
    "AkPropBundle<float,ushort,0>::SetInitialParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkPropBundle<float,unsigned short,0> *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkState::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkState *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkState::TermNotificationSystem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkState *
        ]
    ),
    "SetInitialRTPC<CAkModulator>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # CAkModulator *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>::_ForEachEx<bool ": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > *
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['AkModTreeValue *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>::_ForEachEx<bool ": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > *
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['AkModTreeValue *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>>::_ForEachEx<bool ": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > *
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['AkModTreeValue *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > > *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>::_ForEachEx<bool ": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > *
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['AkModTreeValue *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>>>::_ForEachEx<bool ": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > > *
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['AkModTreeValue *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>::_ForEachMatching<void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkModTreeValue *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > *
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>::_ForEachMatching<void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkModTreeValue *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > *
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>>::_ForEachMatching<void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkModTreeValue *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > > *
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > > *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>::_ForEachMatching<void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkModTreeValue *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > *
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>>>::_ForEachMatching<void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > > *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkModTreeValue *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::_ForEachMatching<void ": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['AkModTreeValue *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # const AkRootKey<CAkPBI *,GetNullPbiPtr> *
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # AkRootKey<CAkPBI *,GetNullPbiPtr> *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>::_ForEachMatchingEx<bool ": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > *
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['AkModTreeValue *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > *
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>::_ForEachMatchingEx<bool ": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > *
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['AkModTreeValue *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > *
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>>::_ForEachMatchingEx<bool ": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > *
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['AkModTreeValue *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > > *
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > > *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>::_ForEachMatchingEx<bool ": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > *
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['AkModTreeValue *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > *
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>>>::_ForEachMatchingEx<bool ": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > > *
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['AkModTreeValue *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::_ForEachMatchingEx<bool ": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> *
            ctypes.c_ulonglong,  # {'return': 'bool', 'args': ['AkModTreeValue *', 'const AkRTPCKey *', 'void *']}
            ctypes.c_ulonglong,  # const AkRootKey<CAkPBI *,GetNullPbiPtr> *
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # AkRootKey<CAkPBI *,GetNullPbiPtr> *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkModulator::~CAkModulator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulator *
        ]
    ),
    "CAkModulator::AddRef": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulator *
        ]
    ),
    "AkSortedKeyArray<uchar,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>::ChildTreeType>,AkDefaultSortedKeyCompare<uchar>>::BinarySearch": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned char,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType>,AkDefaultSortedKeyCompare<unsigned char> > *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AkSortedKeyArray<uint,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>::ChildTreeType>,AkDefaultSortedKeyCompare<uint>>::BinarySearch": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > >::ChildTreeType>,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "?BinarySearch@?$AkSortedKeyArray@PEAVCAkRegisteredObj@@VChildTreeType@?$AkRTPCNestedSearchTree@VAkRTPCKey@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkModTreeValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkModTreeValue@@@@@@@@@@@@@@U?$AkArrayAllocatorNoAlign@$0A@@@V23@UAkGrowByPolicy_Proportional@@U?$AkTransferMovePolicy@VChildTreeType@?$AkRTPCNestedSearchTree@VAkRTPCKey@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkModTreeValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkModTreeValue@@@@@@@@@@@@@@@@U?$AkDefaultSortedKeyCompare@PEAVCAkRegisteredObj@@@@@@QEBAPEAVChildTreeType@?$AkRTPCNestedSearchTree@VAkRTPCKey@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkModTreeValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkModTreeValue@@@@@@@@@@@@@@PEAVCAkRegisteredObj@@AEA_N@Z": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkRegisteredObj *,AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > >::ChildTreeType>,AkDefaultSortedKeyCompare<CAkRegisteredObj *> > *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkModulator::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkIndexable *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkModulatorType
        ]
    ),
    "AkSortedKeyArray<uchar,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>::ChildTreeType>,AkDefaultSortedKeyCompare<uchar>>::Exists": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned char,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType>,AkDefaultSortedKeyCompare<unsigned char> > *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "AkSortedKeyArray<uint,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>::ChildTreeType>,AkDefaultSortedKeyCompare<uint>>::Exists": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > >::ChildTreeType>,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "?Exists@?$AkSortedKeyArray@PEAVCAkRegisteredObj@@VChildTreeType@?$AkRTPCNestedSearchTree@VAkRTPCKey@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkModTreeValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkModTreeValue@@@@@@@@@@@@@@U?$AkArrayAllocatorNoAlign@$0A@@@V23@UAkGrowByPolicy_Proportional@@U?$AkTransferMovePolicy@VChildTreeType@?$AkRTPCNestedSearchTree@VAkRTPCKey@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkModTreeValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkModTreeValue@@@@@@@@@@@@@@@@U?$AkDefaultSortedKeyCompare@PEAVCAkRegisteredObj@@@@@@QEBAPEAVChildTreeType@?$AkRTPCNestedSearchTree@VAkRTPCKey@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkModTreeValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkModTreeValue@@@@@@@@@@@@@@PEAVCAkRegisteredObj@@@Z": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkRegisteredObj *,AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > >::ChildTreeType>,AkDefaultSortedKeyCompare<CAkRegisteredObj *> > *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkModulator::GetCurrentOutput": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulator *
            ctypes.c_ulonglong,  # AkRTPCKey *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkModulatorCtx::GetCurrentOutput": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorCtx *
        ]
    ),
    "CAkEnvelopeModulator::GetInitialParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkEnvelopeModulator *
            ctypes.c_ulonglong,  # AkModulatorParams *
            ctypes.c_ulonglong,  # const CAkModulatorCtx *
        ]
    ),
    "CAkLFOModulator::GetInitialParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLFOModulator *
            ctypes.c_ulonglong,  # AkModulatorParams *
            ctypes.c_ulonglong,  # const CAkModulatorCtx *
        ]
    ),
    "CAkTimeModulator::GetInitialParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkTimeModulator *
            ctypes.c_ulonglong,  # AkModulatorParams *
            ctypes.c_ulonglong,  # const CAkModulatorCtx *
        ]
    ),
    "CAkModulatorCtx::GetLastOutput": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorCtx *
        ]
    ),
    "AkArray<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>::ChildTreeType,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>::ChildTreeType>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > >::ChildTreeType,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > >::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > >::ChildTreeType> > *
        ]
    ),
    "AkArray<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>::ChildTreeType,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>::ChildTreeType>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType> > *
        ]
    ),
    "AkArray<AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>::ChildTreeType,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>::ChildTreeType>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > >::ChildTreeType,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > >::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > >::ChildTreeType> > *
        ]
    ),
    "AkArray<AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>>>::ChildTreeType,AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>>>::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>>>::ChildTreeType>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > >::ChildTreeType,AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > >::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > >::ChildTreeType> > *
        ]
    ),
    "AkRTPCExceptionChecker_<AkRTPCKeyTree<AkModTreeValue>>::IsException": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker_<AkRTPCKeyTree<AkModTreeValue> > *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkEnvelopeModulator::IsTriggerOk": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkTimeModulator *
            ctypes.c_ulonglong,  # const AkModulatorTriggerParams *
        ]
    ),
    "CAkLFOModulator::IsTriggerOk": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkLFOModulator *
            ctypes.c_ulonglong,  # const AkModulatorTriggerParams *
        ]
    ),
    "CAkModulator::ModCtx_MarkFinishedAndRelease": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # AkModTreeValue *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkModulator::ModCtx_NotifySubscribers": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # AkModTreeValue *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkModulator::ModCtx_SetParam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkModTreeValue *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkModulator::NotifySubscribers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulator *
        ]
    ),
    "CAkModulator::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulator *
        ]
    ),
    "CAkModulatorCtx::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorCtx *
        ]
    ),
    "CAkModulator::RemoveCtxsMatchingKey": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulator *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType *
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > *
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > *
        ]
    ),
    "AkSortedKeyArray<uchar,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>::ChildTreeType>,AkDefaultSortedKeyCompare<uchar>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned char,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > >::ChildTreeType>,AkDefaultSortedKeyCompare<unsigned char> > *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AkSortedKeyArray<uchar,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>::ChildTreeType>,AkDefaultSortedKeyCompare<uchar>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned char,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType>,AkDefaultSortedKeyCompare<unsigned char> > *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "?Set@?$AkSortedKeyArray@IVChildTreeType@?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkModTreeValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkModTreeValue@@@@@@@@@@@@U?$AkArrayAllocatorNoAlign@$0A@@@V12@UAkGrowByPolicy_Proportional@@U?$AkTransferMovePolicy@VChildTreeType@?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkModTreeValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkModTreeValue@@@@@@@@@@@@@@U?$AkDefaultSortedKeyCompare@I@@@@QEAAPEAVChildTreeType@?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkModTreeValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkModTreeValue@@@@@@@@@@@@IAEA_N@Z": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > >::ChildTreeType>,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AkSortedKeyArray<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType>,AkDefaultSortedKeyCompare<CAkPBI *>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<CAkPBI *,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>::RootValueType>,AkDefaultSortedKeyCompare<CAkPBI *> > *
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "?Set@?$AkSortedKeyArray@PEAVCAkRegisteredObj@@VChildTreeType@?$AkRTPCNestedSearchTree@VAkRTPCKey@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkModTreeValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkModTreeValue@@@@@@@@@@@@@@U?$AkArrayAllocatorNoAlign@$0A@@@V23@UAkGrowByPolicy_Proportional@@U?$AkTransferMovePolicy@VChildTreeType@?$AkRTPCNestedSearchTree@VAkRTPCKey@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkModTreeValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkModTreeValue@@@@@@@@@@@@@@@@U?$AkDefaultSortedKeyCompare@PEAVCAkRegisteredObj@@@@@@QEAAPEAVChildTreeType@?$AkRTPCNestedSearchTree@VAkRTPCKey@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidPlayingID@@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@IUGetInvalidUniqueID@@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiCh@@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@@@UAkModTreeValue@@V?$AkRTPCNestedSearchTree@V?$AkNestedKey@EUGetInvalidMidiNote@@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@@@UAkModTreeValue@@V?$AkRTPCRootSearchTree@V?$AkRootKey@PEAVCAkPBI@@UGetNullPbiPtr@@@@UAkModTreeValue@@@@@@@@@@@@@@PEAVCAkRegisteredObj@@AEA_N@Z": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > >::ChildTreeType *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkRegisteredObj *,AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > >::ChildTreeType>,AkDefaultSortedKeyCompare<CAkRegisteredObj *> > *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkModulator::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulator *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkModulator::SetParamFromRTPCMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulator *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "DSP::MonoLFO<DSP::Unipolar,DSP::ModulatorLFOOutputPolicy>::SetParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::MonoLFO<DSP::Unipolar,DSP::ModulatorLFOOutputPolicy> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const DSP::LFO::Params *
        ]
    ),
    "CAkModulator::StopWhenFinished": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulator *
        ]
    ),
    "AkArray<AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>>::ChildTreeType,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>>::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>>::ChildTreeType>>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > >::ChildTreeType,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > >::ChildTreeType const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > >::ChildTreeType> > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>>>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > > *
        ]
    ),
    "CAkModulatorCtx::Tick": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorCtx *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkTimeModCtx::Tick": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkTimeModCtx *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkModulator::ToggleRTPCSubscription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulator *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>::ChildTreeType::Transfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType *
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType *
        ]
    ),
    "CAkModulator::Trigger": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulator *
            ctypes.c_ulonglong,  # const AkModulatorSubscriberInfo *
            ctypes.c_ulonglong,  # const AkModulatorTriggerParams *
            ctypes.c_ulonglong,  # CAkModulatorEngine *
            ctypes.c_ulonglong,  # CAkModulatorData *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>::Unset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > *
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>>::Unset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > *
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > > *
        ]
    ),
    "AkSortedKeyArray<uchar,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>::ChildTreeType>,AkDefaultSortedKeyCompare<uchar>>::Unset": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned char,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType,AkArrayAllocatorNoAlign<0>,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> >::ChildTreeType>,AkDefaultSortedKeyCompare<unsigned char> > *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>::_CheckException": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > *
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > *
            ctypes.c_ulonglong,  # const AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > *
        ]
    ),
    "AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidPlayingID,AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uint,GetInvalidUniqueID,AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiCh,AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>>,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<uchar,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr>>,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue>>>>>>::_CheckException": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkRTPCNestedSearchTree<AkRTPCKey,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > >,AkModTreeValue,AkRTPCNestedSearchTree<AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> >,AkModTreeValue,AkRTPCRootSearchTree<AkRootKey<CAkPBI *,GetNullPbiPtr>,AkModTreeValue> > > > > > *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkModulator::_Trigger": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkModulatorCtx *
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulator *
            ctypes.c_ulonglong,  # CAkModulatorCtx *
            ctypes.c_ulonglong,  # const AkModulatorTriggerParams *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # AkModulatorScope
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkActorMixer::CanAddChild": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActorMixer *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkActorMixer::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActorMixer *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActorMixer::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActorMixer *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAuxBus::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkAuxBus *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkAuxBus::NodeCategory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkAuxBus *
        ]
    ),
    "AkDynaBlkPool<CAkLayerCntr::ContPlaybackItem,128,AkArrayAllocatorNoAlign<0>>::PoolChunk::PoolChunk": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDynaBlkPool<CAkLayerCntr::ContPlaybackItem,128,AkArrayAllocatorNoAlign<0> >::PoolChunk *
        ]
    ),
    "CAkSwitchCntr::AddNodeInSwitch": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_ulonglong,  # CAkSwitchPackage *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSwitchCntr::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMultiPlayNode *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSwitchCntr::DecrementActivityCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkSwitchCntr::DecrementPlayCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_ulonglong,  # CounterParameters *
        ]
    ),
    "AkInitialSoundParams::DoCopy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkInitialSoundParams *
            ctypes.c_ulonglong,  # const AkInitialSoundParams *
        ]
    ),
    "CAkSwitchCntr::ExecuteAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkSwitchCntr::ExecuteActionExcept": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_ulonglong,  # ActionParamsExcept *
        ]
    ),
    "CAkSwitchCntr::ExecuteActionInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkSwitchCntr::ExecuteActionNoPropagate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkSwitchCntr::GatherSounds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_ulonglong,  # AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *
            ctypes.c_ulonglong,  # AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *
            ctypes.c_ulonglong,  # AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkArray<uint,uint,AkHybridAllocator<4,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<uint>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<unsigned int,unsigned int,AkHybridAllocator<4,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned int> > *
        ]
    ),
    "CAkSwitchCntr::IncrementActivityCount": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkSwitchCntr::IncrementPlayCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_ulonglong,  # CounterParameters *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSwitchCntr::IsContinuousPlayback": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
        ]
    ),
    "CAkSwitchCntr::IsInfiniteLooping": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_ulonglong,  # CAkPBIAware *
        ]
    ),
    "CAkSwitchCntr::ModifyActiveState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSwitchCntr::NotifyEndContinuous": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_ulonglong,  # CAkSwitchCntr::SwitchContPlaybackItem *
        ]
    ),
    "CAkSwitchCntr::OnPreRelease": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
        ]
    ),
    "CAkSwitchCntr::PerformSwitchChange": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "CAkSwitchCntr::PlayInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_ulonglong,  # AkPBIParams *
        ]
    ),
    "CAkSwitchCntr::PrepareData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
        ]
    ),
    "CAkSwitchCntr::PrepareNodeList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_ulonglong,  # const AkArray<unsigned int,unsigned int,AkHybridAllocator<4,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned int> > *
        ]
    ),
    "CAkSwitchCntr::RemoveChild": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkSwitchCntr::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSwitchCntr::SetSwitch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "CAkSwitchCntr::StopContSwitchInst": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSwitchCntr::StopPrevious": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_ulonglong,  # CAkSwitchPackage *
            ctypes.c_ulonglong,  # CAkSwitchPackage *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
        ]
    ),
    "CAkSwitchCntr::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
        ]
    ),
    "CAkSwitchCntr::TriggerModulators": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_ulonglong,  # const AkModulatorTriggerParams *
            ctypes.c_ulonglong,  # CAkModulatorData *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSwitchCntr::UnPrepareData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
        ]
    ),
    "CAkSwitchCntr::UnPrepareNodeList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSwitchCntr *
            ctypes.c_ulonglong,  # const AkArray<unsigned int,unsigned int,AkHybridAllocator<4,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned int> > *
        ]
    ),
    "CAkLayerCntr::AddChildInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayerCntr *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkLayerCntr::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMultiPlayNode *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkLayerCntr::DecrementActivityCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayerCntr *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkLayerCntr::DecrementPlayCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayerCntr *
            ctypes.c_ulonglong,  # CounterParameters *
        ]
    ),
    "CAkLayerCntr::ExecuteAction": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayerCntr *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkLayerCntr::ExecuteActionExcept": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayerCntr *
            ctypes.c_ulonglong,  # ActionParamsExcept *
        ]
    ),
    "CAkLayerCntr::ExecuteActionInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayerCntr *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkLayerCntr::ExecuteActionNoPropagate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayerCntr *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkLayerCntr::IncrementActivityCount": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayerCntr *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkLayerCntr::IncrementPlayCount": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayerCntr *
            ctypes.c_ulonglong,  # CounterParameters *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkLayerCntr::NotifyEndContinuous": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayerCntr *
            ctypes.c_ulonglong,  # CAkLayerCntr::ContPlaybackItem *
        ]
    ),
    "CAkLayerCntr::OnPreRelease": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayerCntr *
        ]
    ),
    "CAkLayerCntr::PlayInternal": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayerCntr *
            ctypes.c_ulonglong,  # AkPBIParams *
        ]
    ),
    "CAkLayerCntr::RemoveChild": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayerCntr *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "AkSortedKeyArray<uint,CAkLayerCntr::ContPlaybackChildMuteCount,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<uint,CAkLayerCntr::ContPlaybackChildMuteCount>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkLayerCntr::ContPlaybackChildMuteCount>,AkDefaultSortedKeyCompare<uint>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkLayerCntr::ContPlaybackChildMuteCount *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,CAkLayerCntr::ContPlaybackChildMuteCount,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<unsigned int,CAkLayerCntr::ContPlaybackChildMuteCount>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkLayerCntr::ContPlaybackChildMuteCount>,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkLayerCntr::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayerCntr *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkLayerCntr::StopContInst": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayerCntr *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkContinueListItem::CAkContinueListItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkContinueListItem *
        ]
    ),
    "CAkContinueListItem::~CAkContinueListItem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkContinueListItem *
        ]
    ),
    "CAkContinuationList::AddRef": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkContinuationList *
        ]
    ),
    "CAkContinuationList::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkContinuationList *
        argtypes=[]
    ),
    "CAkContinuationList::Release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkContinuationList *
        ]
    ),
    "SetInitialRTPC<CAkLayer>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # CAkLayer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkLayer::~CAkLayer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer *
        ]
    ),
    "CAkLayer::AddRef": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer *
        ]
    ),
    "CAkLayer::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkLayer *
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkLayer::DecrementPlayCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer *
            ctypes.c_ulonglong,  # CounterParameters *
        ]
    ),
    "CAkLayer::GetAudioParameters": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer *
            ctypes.c_ulonglong,  # CAkParameterNode *
            ctypes.c_ulonglong,  # AkSoundParams *
            ctypes.c_ulonglong,  # AkMutedMap *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *
        ]
    ),
    "CAkLayer::GetModulatorParamXfrms": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer *
            ctypes.c_ulonglong,  # AkArray<AkModulatorParamXfrm,AkModulatorParamXfrm const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorParamXfrm> > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const CAkRegisteredObj *
        ]
    ),
    "AkArray<MapStruct<uint,CAkLayer::CAssociatedChildData>,MapStruct<uint,CAkLayer::CAssociatedChildData> const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<uint,CAkLayer::CAssociatedChildData>>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<MapStruct<unsigned int,CAkLayer::CAssociatedChildData>,MapStruct<unsigned int,CAkLayer::CAssociatedChildData> const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<unsigned int,CAkLayer::CAssociatedChildData> > > *
        ]
    ),
    "CAkLayer::IncrementPlayCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer *
            ctypes.c_ulonglong,  # CounterParameters *
        ]
    ),
    "CAkLayer::CAssociatedChildData::IsOutsideOfActiveRange": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer::CAssociatedChildData *
            ctypes.c_float,  # float
        ]
    ),
    "CAkLayer::IsPlaying": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer *
        ]
    ),
    "CAkLayer::OnRTPCChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer *
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "PropagateMute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer::CAssociatedChildData *
            ctypes.c_ulonglong,  # _LayerMuteStruct *
            ctypes.c_ulonglong,  # const AkRTPCKey *
        ]
    ),
    "CAkLayer::RecalcNotification": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkLayer::Release": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer *
        ]
    ),
    "CAkLayer::SetChildAssoc": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkGraphPointBase<float> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkLayer::SetInitialValues": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkLayer::SetOwner": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer *
            ctypes.c_ulonglong,  # CAkLayerCntr *
        ]
    ),
    "CAkLayer::SetParamComplexFromRTPCManager": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const AkRTPCKey *
            ctypes.c_ulonglong,  # AkRTPCExceptionChecker *
        ]
    ),
    "CAkLayer::ToggleRTPCSubscription": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkLayer::TriggerModulators": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer *
            ctypes.c_ulonglong,  # const AkModulatorTriggerParams *
            ctypes.c_ulonglong,  # CAkModulatorData *
        ]
    ),
    "CAkLayer::UnsetChildAssoc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkLayer::UpdateChildPtr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkLayer::CAssociatedChildData::UpdateChildPtr": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkLayer::CAssociatedChildData *
            ctypes.c_ulonglong,  # CAkLayer *
        ]
    ),
    "AkContParamsAndPath::AkContParamsAndPath": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkContParamsAndPath *
            ctypes.c_ulonglong,  # ContParams *
        ]
    ),
    "CAkMultiPlayNode::CAkMultiPlayNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMultiPlayNode *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "SafeContinuationList::SafeContinuationList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SafeContinuationList *
            ctypes.c_ulonglong,  # AkPBIParams *
            ctypes.c_ulonglong,  # CAkMultiPlayNode *
        ]
    ),
    "AkContParamsAndPath::~AkContParamsAndPath": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkContParamsAndPath *
        ]
    ),
    "CAkMultiPlayNode::~CAkMultiPlayNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMultiPlayNode *
        ]
    ),
    "CAkMultiPlayNode::AddMultiplayItem": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMultiPlayNode *
            ctypes.c_ulonglong,  # AkContParamsAndPath *
            ctypes.c_ulonglong,  # AkPBIParams *
            ctypes.c_ulonglong,  # SafeContinuationList *
        ]
    ),
    "AkSortedKeyArray<CAkContinuationList *,CAkMultiPlayNode::ContParam,AkArrayAllocatorNoAlign<0>,CAkMultiPlayNode::ContParamGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkMultiPlayNode::ContParam>,AkDefaultSortedKeyCompare<CAkContinuationList *>>::BinarySearch": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMultiPlayNode::ContParam *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkContinuationList *,CAkMultiPlayNode::ContParam,AkArrayAllocatorNoAlign<0>,CAkMultiPlayNode::ContParamGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkMultiPlayNode::ContParam>,AkDefaultSortedKeyCompare<CAkContinuationList *> > *
            ctypes.c_ulonglong,  # CAkContinuationList *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkMultiPlayNode::ContGetList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMultiPlayNode *
            ctypes.c_ulonglong,  # CAkContinuationList *
            ctypes.c_ulonglong,  # CAkSmartPtr<CAkContinuationList> *
        ]
    ),
    "CAkMultiPlayNode::ContRefList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMultiPlayNode *
            ctypes.c_ulonglong,  # CAkContinuationList *
        ]
    ),
    "CAkMultiPlayNode::ContUnrefList": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMultiPlayNode *
            ctypes.c_ulonglong,  # CAkContinuationList *
        ]
    ),
    "CAkMultiPlayNode::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMultiPlayNode *
        ]
    ),
    "CAkMultiPlayNode::PlayAndContinueAlternateMultiPlay": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMultiPlayNode *
            ctypes.c_ulonglong,  # AkPBIParams *
        ]
    ),
    "AkSortedKeyArray<CAkContinuationList *,CAkMultiPlayNode::ContParam,AkArrayAllocatorNoAlign<0>,CAkMultiPlayNode::ContParamGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkMultiPlayNode::ContParam>,AkDefaultSortedKeyCompare<CAkContinuationList *>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkMultiPlayNode::ContParam *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<CAkContinuationList *,CAkMultiPlayNode::ContParam,AkArrayAllocatorNoAlign<0>,CAkMultiPlayNode::ContParamGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkMultiPlayNode::ContParam>,AkDefaultSortedKeyCompare<CAkContinuationList *> > *
            ctypes.c_ulonglong,  # CAkContinuationList *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkMultiPlayNode::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMultiPlayNode *
        ]
    ),
    "CAkModulatorCtx::CAkModulatorCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorCtx *
        ]
    ),
    "CAkModulatorCtx::~CAkModulatorCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorCtx *
        ]
    ),
    "CAkEnvelopeCtx::CalcBufferNeeded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkEnvelopeCtx *
        ]
    ),
    "CAkLFOCtx::CalcBufferNeeded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLFOCtx *
        ]
    ),
    "CAkTimeModCtx::CalcBufferNeeded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkTimeModCtx *
        ]
    ),
    "CAkEnvelopeCtx::InitializeOutput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkEnvelopeCtx *
        ]
    ),
    "CAkLFOCtx::InitializeOutput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLFOCtx *
        ]
    ),
    "CAkModulatorCtx::IsFinished": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorCtx *
        ]
    ),
    "DSP::MonoLFO<DSP::Unipolar,DSP::ModulatorLFOOutputPolicy>::ProduceBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::MonoLFO<DSP::Unipolar,DSP::ModulatorLFOOutputPolicy> *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CAkEnvelopeCtx::SetParam": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkEnvelopeCtx *
            ctypes.c_uint32,  # AkRTPC_ModulatorParamID
            ctypes.c_float,  # float
        ]
    ),
    "CAkLFOCtx::SetParam": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkLFOCtx *
            ctypes.c_uint32,  # AkRTPC_ModulatorParamID
            ctypes.c_float,  # float
        ]
    ),
    "CAkTimeModCtx::SetParam": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkTimeModCtx *
            ctypes.c_uint32,  # AkRTPC_ModulatorParamID
            ctypes.c_float,  # float
        ]
    ),
    "CAkModulatorCtx::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorCtx *
        ]
    ),
    "CAkModulatorCtx::Trigger": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorCtx *
            ctypes.c_ulonglong,  # CAkModulator *
            ctypes.c_ulonglong,  # const AkModulatorTriggerParams *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_uint32,  # AkModulatorScope
        ]
    ),
    "CAkModulatorCtx::TriggerRelease": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorCtx *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkMidiParseSe::AkMidiParseSe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMidiParseSe *
        ]
    ),
    "AkMidiParseSe::~AkMidiParseSe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMidiParseSe *
        ]
    ),
    "DecodeVariableLength": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AkMidiParseSe::GetCurrentPos": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMidiParseSe *
            ctypes.c_ulonglong,  # AkMidiParseSe::MidiParsePtr *
        ]
    ),
    "AkMidiParseSe::GetNextEvent": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkMidiParseSe *
            ctypes.c_ulonglong,  # AkMidiEventEx *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AkMidiParseSe::MoveEventWindow": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMidiParseSe *
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # AkListBare<AkMidiWindowEvent,AkListBareNextItem,AkCountPolicyNoCount,AkLastPolicyWithLast> *
        ]
    ),
    "AkMidiParseSe::RelocateMedia": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMidiParseSe *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "ResetSeek": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # AkMidiTrack *
            ctypes.c_uint32,  # const unsigned int
        ]
    ),
    "SeekNextEvent": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkMidiTrack *
        ]
    ),
    "AkMidiParseSe::SeekToStart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMidiParseSe *
        ]
    ),
    "AkMidiParseSe::SeekToTime": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkMidiParseSe *
            ctypes.c_double,  # double
        ]
    ),
    "AkMidiParseSe::SetBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkMidiParseSe *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkMidiParseSe::SetCurrentPos": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMidiParseSe *
            ctypes.c_ulonglong,  # const AkMidiParseSe::MidiParsePtr *
        ]
    ),
    "AkMidiParseSe::SetLoopOk": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMidiParseSe *
            ctypes.c_ubyte,  # const bool
        ]
    ),
    "AkMidiParseSe::SetTempo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMidiParseSe *
            ctypes.c_float,  # const float
        ]
    ),
    "CAkActionDuck::CAkActionDuck": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionDuck *
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionDuck::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionDuck *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkDuckItem::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDuckItem *
            ctypes.c_ulonglong,  # CAkBus *
        ]
    ),
    "CAkDuckItem::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDuckItem *
        ]
    ),
    "CAkDuckItem::TransUpdateValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkDuckItem *
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkConnectedObject::AkConnectedObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkConnectedObject *
        ]
    ),
    "AkAudioObject::~AkAudioObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioObject *
        ]
    ),
    "CAkMixConnections::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkMixConnection *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMixConnections *
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
            ctypes.c_ulonglong,  # AkVPL *
            ctypes.c_ulonglong,  # AkHdrBus *
            ctypes.c_uint32,  # AkConnectionType
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AkMixConnection::CreateObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkConnectedObject *
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkMixConnection::Get3DPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkEmitterListenerPair *
        ]
    ),
    "AkMixConnection::Get3DPositionType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "AkMixConnection::Get3DSpatializationMode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "AkMixConnection::GetCenterPerc": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "AkMixConnection::GetConnectionType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "AkMixConnection::GetFocus": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkMixConnection::GetGameObjectInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkVPL3dMixable *
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "AkMixConnection::GetInputParam": FUNCDEF(
        restype=ctypes.c_ulonglong,  # PluginRTPCSub_vtbl *
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "AkMixConnection::GetMaxAttenuationDistance": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "AkMixConnection::GetNum3DPositions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "AkMixConnection::GetOutputBus": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkVPL *
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "AkMixConnection::GetOutputBusCtx": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::CAkBusCtx *
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "AkMixConnection::GetOutputConfig": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkChannelConfig *
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
            ctypes.c_ulonglong,  # AkChannelConfig *
        ]
    ),
    "AkMixConnection::GetPannerPosition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
            ctypes.c_ulonglong,  # AkVector *
        ]
    ),
    "AkMixConnection::GetSpatializedVolumes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "AkMixConnection::GetSpeakerPanningType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "AkMixConnection::GetSpread": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkMixConnection::GetUserData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "AkMixConnection::GetVoiceInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkVoicePluginInfo *
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "AkArray<AkConnectedObject,AkConnectedObject const &,AkHybridAllocator<344,1,0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkConnectedObject>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkConnectedObject,AkConnectedObject const &,AkHybridAllocator<344,1,0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkConnectedObject> > *
        ]
    ),
    "AkMixConnection::HasListenerRelativeRouting": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "AkMixConnection::SetUserData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMixConnection *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AkAudioObject::Transfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioObject *
            ctypes.c_ulonglong,  # AkAudioObject *
        ]
    ),
    "AkConnectedObject::Transfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkConnectedObject *
            ctypes.c_ulonglong,  # AkConnectedObject *
        ]
    ),
    "CAkModulatorEngine::ProcessBatch<CAkLFOProcess,AkLFOParams,AkLFOOutput>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorBatchBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkModulatorEngine::ProcessBatch<CAkTimeModProcess,AkTimeModParams,AkTimeModOutput>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorBatchBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkEnvelopeProcess::_Process<BufferOutputPolicy>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const AkModulatorParams *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # AkModulatorOutput *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkEnvelopeProcess::_Process<SingleOutputPolicy>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const AkModulatorParams *
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # AkModulatorOutput *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkModulatorEngine::~CAkModulatorEngine": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorEngine *
        ]
    ),
    "CAkModulatorEngine::AddContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorEngine *
            ctypes.c_ulonglong,  # CAkModulatorCtx *
        ]
    ),
    "CAkModulatorEngine::CleanUpFinishedCtxs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorEngine *
        ]
    ),
    "CAkLFOCtx::FillParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkLFOCtx *
            ctypes.c_ulonglong,  # AkLFOParams *
        ]
    ),
    "AkArray<AkEnvelopeOutput,AkEnvelopeOutput const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkEnvelopeOutput>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkTimeModOutput,AkTimeModOutput const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkTimeModOutput> > *
        ]
    ),
    "AkArray<AkEnvelopeParams,AkEnvelopeParams const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkEnvelopeParams>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkEnvelopeParams,AkEnvelopeParams const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkEnvelopeParams> > *
        ]
    ),
    "AkArray<AkLFOOutput,AkLFOOutput const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkLFOOutput>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkLFOOutput,AkLFOOutput const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkLFOOutput> > *
        ]
    ),
    "AkArray<AkLFOParams,AkLFOParams const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkLFOParams>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkLFOParams,AkLFOParams const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkLFOParams> > *
        ]
    ),
    "AkArray<AkTimeModParams,AkTimeModParams const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkTimeModParams>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkTimeModParams,AkTimeModParams const &,AkArrayAllocatorAlignedSimd<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkTimeModParams> > *
        ]
    ),
    "CAkModulatorEngine::ProcessModulators": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkModulatorEngine *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "BufferOutputPolicy::Ramp": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # float **
            ctypes.c_uint32,  # unsigned int
            ctypes.c_double,  # double
            ctypes.c_double,  # double
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkMarkers::CAkMarkers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMarkers *
        ]
    ),
    "CAkMarkers::Allocate": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMarkers *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMarkersQueue::Enqueue": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMarkersQueue *
            ctypes.c_ulonglong,  # AkAudioMarker *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const CAkMarkersQueue::BufferRange *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMarkers::Free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMarkers *
        ]
    ),
    "CAkMarkers::GetClosestMarker": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkAudioMarker *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMarkers *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMarkers::NotifyRelevantMarkers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMarkers *
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMarkers::SaveMarkersForBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMarkers *
            ctypes.c_ulonglong,  # CAkMarkersQueue *
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_ulonglong,  # AkPipelineBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMarkers::SetLabel": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMarkers *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # char *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkVBAPMap::AddPointsInArea": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkVBAPMap *
            ctypes.c_ulonglong,  # const Ak2DVector *
            ctypes.c_ulonglong,  # const AkArray<AkVBAPTriplet,AkVBAPTriplet const &,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkVBAPTriplet> > *
            ctypes.c_ulonglong,  # const Ak2DVector *
            ctypes.c_ulonglong,  # AkVBAPTripletStackArray *
        ]
    ),
    "AkVBAPMap::Allocate": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkVBAPMap *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_uint32,  # const unsigned int
        ]
    ),
    "AkVBAPMap::ComputeVBAP3D": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # AkVBAPMap *
            ctypes.c_ulonglong,  # AkVBAPTriplet *
            ctypes.c_ulonglong,  # Ak3DVector *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "AkVBAPMap::ComputeVBAP": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkVBAPMap *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "AkVBAPMap::ComputeVBAPSquared": {
        "AkVBAPMap *, float, float, unsigned int, float *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # AkVBAPMap *
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # float *
            ]
        ),
        "AkVBAPMap *, float, float, double, unsigned int, float *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # AkVBAPMap *
                ctypes.c_float,  # float
                ctypes.c_float,  # float
                ctypes.c_double,  # double
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # float *
            ]
        ),
    },
    "AkVBAPMap::FindPointInTriangle": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkVBAPMap *
            ctypes.c_ulonglong,  # const Ak2DVector *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "AkVBAPMap::FindTriangle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkVBAPMap *
            ctypes.c_ulonglong,  # const Ak3DVector *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # AkVBAPTripletStackArray *
        ]
    ),
    "AkVBAPMap::NormalizePreSquaredGain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkVBAPMap *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "AkVBAPMap::PointsToVBAPMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkVBAPMap *
            ctypes.c_ulonglong,  # const AkSphericalCoord *
            ctypes.c_uint32,  # const unsigned int
        ]
    ),
    "AkVBAPMap::RunTriangulation": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkVBAPMap *
            ctypes.c_ulonglong,  # const Ak2DVector *
            ctypes.c_ulonglong,  # AkArray<AkVBAPTriplet,AkVBAPTriplet const &,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkVBAPTriplet> > *
            ctypes.c_ulonglong,  # AkArray<AkVBAPTriplet,AkVBAPTriplet const &,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkVBAPTriplet> > *
            ctypes.c_ulonglong,  # AkArray<AkVBAPTriplet,AkVBAPTriplet const &,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkVBAPTriplet> > *
            ctypes.c_ulonglong,  # AkArray<AkVBAPTriplet,AkVBAPTriplet const &,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkVBAPTriplet> > *
        ]
    ),
    "AkVBAPMap::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkVBAPMap *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "AK::CAkBusCtx::CAkBusCtx": {
        "AK::CAkBusCtx *, AK::CAkBusCtx *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # AK::CAkBusCtx *
                ctypes.c_ulonglong,  # AK::CAkBusCtx *
            ]
        ),
        "AK::CAkBusCtx *, CAkBus *, CAkRegisteredObj *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # AK::CAkBusCtx *
                ctypes.c_ulonglong,  # CAkBus *
                ctypes.c_ulonglong,  # CAkRegisteredObj *
            ]
        ),
        "AK::CAkBusCtx *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # AK::CAkBusCtx *
            ]
        ),
    },
    "AK::CAkBusCtx::~CAkBusCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::CAkBusCtx *
        ]
    ),
    "AK::CAkBusCtx::FindParentCtx": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::CAkBusCtx *
        argtypes=[
            ctypes.c_ulonglong,  # AK::CAkBusCtx *
            ctypes.c_ulonglong,  # AK::CAkBusCtx *
        ]
    ),
    "AK::CAkBusCtx::GameObjectID": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::CAkBusCtx *
        ]
    ),
    "AK::CAkBusCtx::GetChannelConfig": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkChannelConfig *
        argtypes=[
            ctypes.c_ulonglong,  # AK::CAkBusCtx *
            ctypes.c_ulonglong,  # AkChannelConfig *
        ]
    ),
    "AK::CAkBusCtx::GetFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::CAkBusCtx *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # AkFXDesc *
        ]
    ),
    "AK::CAkBusCtx::GetFXDataID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::CAkBusCtx *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AK::CAkBusCtx::GetMixerPlugin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::CAkBusCtx *
            ctypes.c_ulonglong,  # AkFXDesc *
        ]
    ),
    "AK::CAkBusCtx::GetMixerPluginDataID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::CAkBusCtx *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AK::CAkBusCtx::HasMixerPlugin": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AK::CAkBusCtx *
        ]
    ),
    "AK::CAkBusCtx::ID": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkBus *
        argtypes=[
            ctypes.c_ulonglong,  # AK::CAkBusCtx *
        ]
    ),
    "AK::CAkBusCtx::IsAuxBus": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AK::CAkBusCtx *
        ]
    ),
    "AK::CAkBusCtx::LookupGameObjectAndCreateBusCtx": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::CAkBusCtx *
        argtypes=[
            ctypes.c_ulonglong,  # AK::CAkBusCtx *
            ctypes.c_ulonglong,  # CAkBus *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AkAudioObjectRegistry::AkAudioObjectRegistry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioObjectRegistry *
        ]
    ),
    "AkAudioObjectRegistry::Record::Record": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioObjectRegistry::Record *
        ]
    ),
    "AkAudioObjectRegistry::CollectGarbage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioObjectRegistry *
        ]
    ),
    "AkArray<AkAudioObject::CustomMetadata,AkAudioObject::CustomMetadata const &,AkPluginArrayAllocator,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAudioObject::CustomMetadata>>::Copy": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkAudioObject::CustomMetadata,AkAudioObject::CustomMetadata const &,AkPluginArrayAllocator,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAudioObject::CustomMetadata> > *
            ctypes.c_ulonglong,  # const AkArray<AkAudioObject::CustomMetadata,AkAudioObject::CustomMetadata const &,AkPluginArrayAllocator,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAudioObject::CustomMetadata> > *
        ]
    ),
    "AkAudioObjectRegistry::CreateOrUpdateRecord": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkAudioObjectRegistry::Record *
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioObjectRegistry *
            ctypes.c_ulonglong,  # const AkAudioObjectRegistry::TriageResult *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioObject *
            ctypes.c_ulonglong,  # const Ak3DAudioSinkCapabilities *
        ]
    ),
    "AkArray<AkAudioObjectRegistry::Record,AkAudioObjectRegistry::Record const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkAudioObjectRegistry::Record>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkAudioObjectRegistry::Record,AkAudioObjectRegistry::Record const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkAudioObjectRegistry::Record> > *
        ]
    ),
    "AkAudioObjectRegistry::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioObjectRegistry *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkArray<AkAudioObjectRegistry::Record,AkAudioObjectRegistry::Record const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkAudioObjectRegistry::Record>>::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkAudioObjectRegistry::Record,AkAudioObjectRegistry::Record const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<AkAudioObjectRegistry::Record> > *
        ]
    ),
    "AkAudioObjectRegistry::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioObjectRegistry *
        ]
    ),
    "AkArray<AkAudioObject::CustomMetadata,AkAudioObject::CustomMetadata const &,AkPluginArrayAllocator,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAudioObject::CustomMetadata>>::Transfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AkAudioObject::CustomMetadata,AkAudioObject::CustomMetadata const &,AkPluginArrayAllocator,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAudioObject::CustomMetadata> > *
            ctypes.c_ulonglong,  # AkArray<AkAudioObject::CustomMetadata,AkAudioObject::CustomMetadata const &,AkPluginArrayAllocator,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAudioObject::CustomMetadata> > *
        ]
    ),
    "AkAudioObjectRegistry::Record::Transfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioObjectRegistry::Record *
            ctypes.c_ulonglong,  # AkAudioObjectRegistry::Record *
        ]
    ),
    "AkAudioObjectRegistry::Triage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkAudioObjectRegistry::TriageResult *
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioObjectRegistry *
            ctypes.c_ulonglong,  # AkAudioObjectRegistry::TriageResult *
            ctypes.c_ulonglong,  # const AkAudioBuffer *
            ctypes.c_ulonglong,  # const AkAudioObject *
            ctypes.c_ulonglong,  # const Ak3DAudioSinkCapabilities *
        ]
    ),
    "AkAudioObjectRegistry::WriteSystemAudioObjectPointers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioObjectRegistry *
            ctypes.c_ulonglong,  # AkAudioObjects *
        ]
    ),
    "DSP::SIMD::ProcessBufferMono_<DSP::SIMD::OutOfPlaceBuffer,DSP::SIMD::BiquadMonoCoefs,DSP::SIMD::BiquadMonoMemories>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::SIMD::OutOfPlaceBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadMonoCoefs *
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadMonoMemories *
        ]
    ),
    "CAkBusFX::CAkBusFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
        ]
    ),
    "CAkVPLMixBusNode::CAkVPLMixBusNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLMixBusNode *
        ]
    ),
    "CAkBusFX::~CAkBusFX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
        ]
    ),
    "CAkVPLMixBusNode::~CAkVPLMixBusNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLMixBusNode *
        ]
    ),
    "CAkBusFX::CanCollectProcessorObject": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBusFX::Processor::Collect": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const CAkBusFX::ArrayObjects *
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX::Processor *
            ctypes.c_ulonglong,  # const CAkBusFX *
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # const CAkBusFX::ArrayObjects *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "ComputeChannelKPower": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMonoCoefs,DSP::SIMD::BiquadMonoMemories> *
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMonoCoefs,DSP::SIMD::BiquadMonoMemories> *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkMeterCtx::ComputeKPower": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMeterCtx *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkBusFX::ComputeSphericalVBAPGains": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_ulonglong,  # void *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkVPLMixBusNode::Connect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLMixBusNode *
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "CAkVPLMixBusNode::ConsumeBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLMixBusNode *
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "ConvolvePolyPhase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "AkAudioObject::CopyContents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioObject *
            ctypes.c_ulonglong,  # const AkAudioObject *
        ]
    ),
    "AkMeterCtxFactory::CreateMeterContext": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkMeterCtx *
        argtypes=[
            ctypes.c_ulonglong,  # AkMeterCtxFactory *
        ]
    ),
    "CAkVPLMixBusNode::Disconnect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLMixBusNode *
            ctypes.c_ulonglong,  # AkMixConnection *
        ]
    ),
    "CAkBusFX::Processor::Drop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX::Processor *
        ]
    ),
    "CAkBusFX::DropFx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
        ]
    ),
    "CAkBusFX::DropFxFromIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBusFX::EnableMetering": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_uint32,  # AkMeteringFlags
        ]
    ),
    "AkArray<CAkBusFX::Object,CAkBusFX::Object const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<CAkBusFX::Object>>::EraseSwap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<CAkBusFX::Object,CAkBusFX::Object const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<CAkBusFX::Object> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBusFX::Processor::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX::Processor *
            ctypes.c_ulonglong,  # CAkBusFX::ArrayObjects **
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # CAkBusFX *
        ]
    ),
    "CAkBusFX::GetAudioNodeID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
        ]
    ),
    "CAkBusFX::GetDownstreamGain": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
        ]
    ),
    "CAkBusFX::GetGameObjectID": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
        ]
    ),
    "CAkBusFX::GetGameObjectInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkGameObjectPluginInfo *
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
        ]
    ),
    "AkMeterCtx::GetKWeightedPower": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # AkMeterCtx *
        ]
    ),
    "CAkBusFX::GetOutputConfig": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkChannelConfig *
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_ulonglong,  # AkChannelConfig *
        ]
    ),
    "CAkBusFX::GetOutputID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkBusFX::GetOutputObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkAudioObjects *
        ]
    ),
    "CAkBusFX::GetParentChannelConfig": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_ulonglong,  # AkChannelConfig *
        ]
    ),
    "CAkBusFX::GetPluginCustomGameData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_ulonglong,  # void **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkBusFX::GetPluginCustomGameDataForInsertFx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkBusFX::GetPluginMedia": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkBusFX::GetSpeakerAngles": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "AkArray<CAkBusFX::Object,CAkBusFX::Object const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<CAkBusFX::Object>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<CAkBusFX::Object,CAkBusFX::Object const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<CAkBusFX::Object> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkMeterCtx::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkMeterCtx *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkMeteringFlags
        ]
    ),
    "CAkBusFX::InitSphericalVBAP": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const AkSphericalCoord *
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "MeterBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkMeterCtx *
        ]
    ),
    "AkMeterCtx::MeterBufferPeakIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMeterCtx *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkVPLMixBusNode::PostProcessFx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLMixBusNode *
            ctypes.c_ulonglong,  # CAkBusFX::ArrayObjects *
        ]
    ),
    "CAkVPLMixBusNode::ProcessInputAndPushToConnections": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLMixBusNode *
        ]
    ),
    "CAkBusFX::RebuildFxFromIndex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_char,  # char
        ]
    ),
    "CAkBusFX::RefreshMeterWatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
        ]
    ),
    "CAkVPLMixBusNode::ReleaseBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLMixBusNode *
        ]
    ),
    "AkArray<CAkBusFX::Object,CAkBusFX::Object const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<CAkBusFX::Object>>::RemoveAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<CAkBusFX::Object,CAkBusFX::Object const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<CAkBusFX::Object> > *
        ]
    ),
    "DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMonoCoefs,DSP::SIMD::BiquadMonoMemories>::SetCoefs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMonoCoefs,DSP::SIMD::BiquadMonoMemories> *
            ctypes.c_double,  # double
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CAkBusFX::Processor::SetEffect": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX::Processor *
            ctypes.c_ulonglong,  # CAkBusFX::Object *
            ctypes.c_ulonglong,  # CAkBusFX::Object **
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkBusFX::SetInsertFxBypass": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBusFX::SetInsertFxMask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBusFX::SetMixerPlugin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
        ]
    ),
    "CAkBusFX::SetupEffectChain": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_char,  # char
            ctypes.c_char,  # char
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # CAkBusFX::Object *
        ]
    ),
    "CAkBusFX::TermSphericalVBAP": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CAkBusFX::Object::Transfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX::Object *
            ctypes.c_ulonglong,  # CAkBusFX::Object *
        ]
    ),
    "CAkBusFX::UpdateBypassFx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
        ]
    ),
    "CAkBusFX::Processor::_DropEffect": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX::Processor *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBusVolumes::_GetOutputID": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusVolumes *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkSinkDummy::CAkSinkDummy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSinkDummy *
        ]
    ),
    "AkCreateDummySink": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmMemory::GetHeuristics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmMemory *
            ctypes.c_ulonglong,  # AkAutoStmHeuristics *
        ]
    ),
    "CAkSinkCoreAudio::GetPluginInfo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSinkCoreAudio *
            ctypes.c_ulonglong,  # AkPluginInfo *
        ]
    ),
    "CAkSinkDummy::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSinkDummy *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkSinkPluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkSinkDummy::IsDataNeeded": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSinkDummy *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AK::AkRuntimeEnvironmentMgr::AkRuntimeEnvironmentMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::AkRuntimeEnvironmentMgr *
        ]
    ),
    "AK::AkRuntimeEnvironmentMgr::DisableSIMDSupport": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::AkRuntimeEnvironmentMgr *
            ctypes.c_uint32,  # AK::AkSIMDProcessorSupport
        ]
    ),
    "AK::AkRuntimeEnvironmentMgr::GetSIMDSupport": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AK::AkRuntimeEnvironmentMgr *
            ctypes.c_uint32,  # AK::AkSIMDProcessorSupport
        ]
    ),
    "AK::AkRuntimeEnvironmentMgr::Initialize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::AkRuntimeEnvironmentMgr *
        ]
    ),
    "AK::AkRuntimeEnvironmentMgr::Instance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::AkRuntimeEnvironmentMgr *
        argtypes=[]
    ),
    "AkMixer::InitMixerFunctTable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkMixer *
        ]
    ),
    "AkMixer::MixBypassNinNChannels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkRamp *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_float,  # float
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # CAkSrcLpHpFilter *
        ]
    ),
    "AkMixer::MixChannel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkMixer::MixFilterNinNChannels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkRamp *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_float,  # float
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # CAkSrcLpHpFilter *
        ]
    ),
    "AkMixer::MixNinNChannels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkRamp *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_float,  # float
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "DummySinkParams::DummySinkParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DummySinkParams *
        ]
    ),
    "AkCreateDummySinkParam": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "AkCreateSystemSinkParam": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "DummySinkParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # DummySinkParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "SystemSinkParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # SystemSinkParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "AkSinkPluginParam::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkSinkPluginParam *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "SystemSinkParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SystemSinkParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "SystemSinkParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SystemSinkParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkVPLBQFNode::ConsumeBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLBQFNode *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkVPLBQFNode::ReleaseBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLVolAutmNode *
        ]
    ),
    "CAkVPLBQFNode::VirtualOn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLBQFNode *
            ctypes.c_uint32,  # AkVirtualQueueBehavior
        ]
    ),
    "CAkResampler::CAkResampler": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkResampler *
        ]
    ),
    "CAkResampler::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkResampler *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkResampler::GetLastRate": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkResampler *
        ]
    ),
    "CAkResampler::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkResampler *
            ctypes.c_ulonglong,  # AkAudioFormat *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkResampler::InitDSPFunctTable": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkResampler::InterleaveAndSwapOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkPipelineBufferBase *
        ]
    ),
    "CAkResampler::ResetOffsets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkResampler *
        ]
    ),
    "CAkResampler::SetChannelMapping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkResampler *
            ctypes.c_ulonglong,  # const AkChannelConfig *
            ctypes.c_ulonglong,  # {'return': 'unsigned __int8', 'args': ['const AkChannelConfig *', {'type': 'unsigned', 'name': '__int8'}]}
        ]
    ),
    "CAkResampler::SetPitch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkResampler *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkResampler::SetPitchForTimeSkip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkResampler *
            ctypes.c_float,  # float
        ]
    ),
    "CAkResampler::SwitchTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkResampler *
            ctypes.c_ulonglong,  # const AkAudioFormat *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkResampler::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkResampler *
        ]
    ),
    "CAkVPLVolAutmNode::~CAkVPLVolAutmNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLVolAutmNode *
        ]
    ),
    "CAkVPLVolAutmNode::ApplyModsToVolume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLVolAutmNode *
            ctypes.c_ulonglong,  # AkVPLState *
            ctypes.c_ulonglong,  # AkModulatorXfrm *
            ctypes.c_ulonglong,  # AkModulatorGain *
            ctypes.c_ulonglong,  # float **
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkVPLVolAutmNode::ConsumeBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLVolAutmNode *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkVPLVolAutmNode::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLVolAutmNode *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkVPLVolAutmNode::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLVolAutmNode *
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkVPLSrcNode::CAkVPLSrcNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkVPLSrcNode::Create": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # AkSrcType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkVPLSrcNode::FetchStreamedData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkVPLSrcNode::Pause": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
        ]
    ),
    "CAkVPLSrcNode::Resume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
            ctypes.c_float,  # float
        ]
    ),
    "CAkVPLSrcNode::Seek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
        ]
    ),
    "CAkVPLSrcNode::Start": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
        ]
    ),
    "CAkVPLSrcNode::Stop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
        ]
    ),
    "CAkVPLSrcNode::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
            ctypes.c_uint32,  # AkCtxDestroyReason
        ]
    ),
    "CAkVPLSrcNode::TimeSkip": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkMixConnections::~CAkMixConnections": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMixConnections *
        ]
    ),
    "CAkVPL3dMixable::~CAkVPL3dMixable": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
        ]
    ),
    "CAkVPL3dMixable::AddOutputBus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
            ctypes.c_ulonglong,  # AkVPL *
            ctypes.c_uint32,  # AkConnectionType
        ]
    ),
    "CAkVPL3dMixable::DisconnectAllInputs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
        ]
    ),
    "CAkVPL3dMixable::GetFirstDirectConnection": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkMixConnection *
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
        ]
    ),
    "CAkVPL3dMixable::GetNumForwardConnectionsToDepth": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
            ctypes.c_int32,  # int
        ]
    ),
    "CAkVPL3dMixable::GetVolumes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
            ctypes.c_ulonglong,  # CAkBehavioralCtx *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # SpeakerVolumeMatrixCallback *
        ]
    ),
    "CAkVPL3dMixable::ManageAuxSends": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
        ]
    ),
    "CAkVPL3dMixable::ReleaseBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
        ]
    ),
    "CAkVPL3dMixable::RemoveOutputBus": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkListBare<AkMixConnection,AkListBareLightNextItem,AkCountPolicyNoCount,AkLastPolicyNoLast>::IteratorEx *
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
            ctypes.c_ulonglong,  # AkListBare<AkMixConnection,AkListBareLightNextItem,AkCountPolicyNoCount,AkLastPolicyNoLast>::IteratorEx *
            ctypes.c_ulonglong,  # AkListBare<AkMixConnection,AkListBareLightNextItem,AkCountPolicyNoCount,AkLastPolicyNoLast>::IteratorEx *
        ]
    ),
    "AkSortedKeyArray<AkHdrBus *,CAkVPL3dMixable::PeakPerHDRBus,AkArrayAllocatorNoAlign<5>,AkGetArrayKey<AkHdrBus *,CAkVPL3dMixable::PeakPerHDRBus>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkVPL3dMixable::PeakPerHDRBus>,AkDefaultSortedKeyCompare<AkHdrBus *>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkVPL3dMixable::PeakPerHDRBus *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<AkHdrBus *,CAkVPL3dMixable::PeakPerHDRBus,AkArrayAllocatorNoAlign<5>,AkGetArrayKey<AkHdrBus *,CAkVPL3dMixable::PeakPerHDRBus>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkVPL3dMixable::PeakPerHDRBus>,AkDefaultSortedKeyCompare<AkHdrBus *> > *
            ctypes.c_ulonglong,  # AkHdrBus *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkVPL3dMixable::SetPreviousSilentOnAllConnections": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkVPL3dMixable::UpdateConnections": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkVPL3dMixable::UpdateHDR": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPL3dMixable *
        ]
    ),
    "CAkBusFXContext::CAkBusFXContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFXContext *
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AK::CAkBusCtx *
        ]
    ),
    "CAkInsertFXContext::CAkInsertFXContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkInsertFXContext *
        ]
    ),
    "AkDataReference::~AkDataReference": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDataReference *
        ]
    ),
    "AkDataReferenceArray::~AkDataReferenceArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDataReferenceArray *
        ]
    ),
    "CAkInsertFXContext::~CAkInsertFXContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkInsertFXContext *
        ]
    ),
    "AkDataReferenceArray::AcquireData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkDataReference *
        argtypes=[
            ctypes.c_ulonglong,  # AkDataReferenceArray *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBusFXContext::ComputeSpeakerVolumesDirect": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFXContext *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "AkDataReferenceArray::FindAlternateMedia": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const CAkUsageSlot *
            ctypes.c_ulonglong,  # AkDataReference *
            ctypes.c_ulonglong,  # AK::IAkPlugin *
        ]
    ),
    "CAkBusFXContext::GetAudioNodeID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkInsertFXContext *
        ]
    ),
    "CAkBusFXContext::GetDownstreamGain": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkInsertFXContext *
        ]
    ),
    "CAkBusFXContext::GetMixerCtx": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkMixerPluginContext *
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFXContext *
        ]
    ),
    "CAkBusFXContext::GetOutputID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFXContext *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkBusFXContext::GetOutputObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFXContext *
            ctypes.c_ulonglong,  # AkAudioObjects *
        ]
    ),
    "CAkInsertFXContext::GetOutputObjects": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlay *
            ctypes.c_ulonglong,  # AkCntrHistArray *
        ]
    ),
    "CAkBusFXContext::GetParentChannelConfig": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFXContext *
            ctypes.c_ulonglong,  # AkChannelConfig *
        ]
    ),
    "CAkInsertFXContext::GetParentChannelConfig": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkInsertFXContext *
            ctypes.c_ulonglong,  # AkChannelConfig *
        ]
    ),
    "CAkBusFXContext::GetPluginCustomGameData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFXContext *
            ctypes.c_ulonglong,  # void **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkBusFXContext::GetPluginMedia": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFXContext *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkInsertFXContext::GetPluginMedia": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkInsertFXContext *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkInsertFXContext::GetVoiceInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkVoicePluginInfo *
        argtypes=[
            ctypes.c_ulonglong,  # CAkInsertFXContext *
        ]
    ),
    "CAkInsertFXContext::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkInsertFXContext *
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkBusFXContext::IsSendModeEffect": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFXContext *
        ]
    ),
    "CAkEffectContextBase::IsUsingThisSlot": {
        "CAkEffectContextBase *, const unsigned __int8 *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # CAkEffectContextBase *
                ctypes.c_ulonglong,  # const unsigned __int8 *
            ]
        ),
        "CAkEffectContextBase *, const CAkUsageSlot *, AK::IAkPlugin *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # CAkEffectContextBase *
                ctypes.c_ulonglong,  # const CAkUsageSlot *
                ctypes.c_ulonglong,  # AK::IAkPlugin *
            ]
        ),
    },
    "CAkEffectContextBase::PostMonitorMessage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkEffectContextBase *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # AK::Monitor::ErrorLevel
        ]
    ),
    "CAkMixBusCtx::CAkMixBusCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMixBusCtx *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # CAkBus *
        ]
    ),
    "CAkMixBusCtx::GetTargetedParamsSet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkBitArray<unsigned __int64,0> *
        argtypes=[
            ctypes.c_ulonglong,  # CAkMixBusCtx *
            ctypes.c_ulonglong,  # CAkBitArray<unsigned __int64,0> *
        ]
    ),
    "CAkMixBusCtx::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMixBusCtx *
        ]
    ),
    "CAkMixBusCtx::ManageAuxRoutableBusses": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkMixBusCtx::NotifyParamChanged": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMixBusCtx *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const CAkBitArray<unsigned __int64,0> *
        ]
    ),
    "CAkMixBusCtx::RefreshParameters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMixBusCtx *
            ctypes.c_ulonglong,  # AkInitialSoundParams *
        ]
    ),
    "CAkBusFX::SetFxBypassAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkBusFX *
            ctypes.c_int16,  # __int16
        ]
    ),
    "CAkMixBusCtx::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMixBusCtx *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMixBusCtx::UpdateTargetParam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMixBusCtx *
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_uint32,  # AkRTPC_ParameterID
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "AkHashIndexAdd": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkHashIndex *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
        ]
    ),
    "AkHashIndexGetEntry": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkHashIndex *
            ctypes.c_int32,  # int
        ]
    ),
    "AkHashIndexGetFirst": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkHashIndex *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkHashIndexGetNext": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkHashIndex *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
        ]
    ),
    "AkHashIndexInit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkHashIndex *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkMemID
        ]
    ),
    "AkHashIndexRemove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkHashIndex *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
        ]
    ),
    "AkHashIndexTerm": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkHashIndex *
        ]
    ),
    "Grow": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkHashIndexEntry **
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkMemID
        ]
    ),
    "CAkDynamicSequencePBI::CAkDynamicSequencePBI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequencePBI *
            ctypes.c_ulonglong,  # AkPBIParams *
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_ulonglong,  # CAkSource *
            ctypes.c_ulonglong,  # const PriorityInfoCurrent *
            ctypes.c_uint32,  # AK::SoundEngine::DynamicSequence::DynamicSequenceType
        ]
    ),
    "CAkDynamicSequencePBI::PlayNextElement": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequencePBI *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
        ]
    ),
    "CAkDynamicSequencePBI::PrepareNextToPlay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequencePBI *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkDynamicSequencePBI::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkDynamicSequencePBI *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "FileCaptureWriterFactory::CreateWriter": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAkCaptureWriter *
        argtypes=[
            ctypes.c_ulonglong,  # FileCaptureWriterFactory *
        ]
    ),
    "FileCaptureWriter::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # FileCaptureWriter *
        ]
    ),
    "FileCaptureWriterFactory::Instance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # FileCaptureWriterFactory *
        argtypes=[]
    ),
    "FileCaptureWriter::StartCapture": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # FileCaptureWriter *
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_ulonglong,  # AkFileParser::Header *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "FileCaptureWriter::StopCapture": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # FileCaptureWriter *
            ctypes.c_ulonglong,  # AkFileParser::Header *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "FileCaptureWriter::WriteGenericData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # FileCaptureWriter *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkContainerBase::CAkContainerBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkContainerBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkContainerBase::~CAkContainerBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkContainerBase *
        ]
    ),
    "CAkContainerBase::CanAddChild": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkContainerBase *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkPlayListRandom::CAkPlayListRandom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayListRandom *
        ]
    ),
    "CAkPlayListSequence::CAkPlayListSequence": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayListSequence *
        ]
    ),
    "CAkPlayListRandom::Add": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayListRandom *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPlayListSequence::Add": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayListSequence *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPlayListRandom::CalculateTotalWeight": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayListRandom *
        ]
    ),
    "CAkPlayListRandom::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayListSequence *
        ]
    ),
    "CAkPlayListRandom::Exists": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayListRandom *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPlayListSequence::Exists": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayListSequence *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPlayListRandom::GetPosition": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayListRandom *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned __int16 *
        ]
    ),
    "CAkPlayListSequence::GetPosition": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayListSequence *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned __int16 *
        ]
    ),
    "CAkPlayListRandom::GetWeight": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayListRandom *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkPlayListSequence::GetWeight": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayListSequence *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkPlayListRandom::ID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayListRandom *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkPlayListSequence::ID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayListSequence *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkPlayListRandom::Remove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayListRandom *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPlayListSequence::Remove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayListSequence *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkPlayListRandom::SetWeight": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkPlayListRandom *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkContinuousPBI::CAkContinuousPBI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkContinuousPBI *
            ctypes.c_ulonglong,  # AkPBIParams *
            ctypes.c_ulonglong,  # CAkSoundBase *
            ctypes.c_ulonglong,  # CAkSource *
            ctypes.c_ulonglong,  # const PriorityInfoCurrent *
        ]
    ),
    "CAkContinuousPBI::~CAkContinuousPBI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkContinuousPBI *
        ]
    ),
    "CAkContinuousPBI::CanSeek": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkContinuousPBI *
        ]
    ),
    "CAkContinuousPBI::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkContinuousPBI *
        ]
    ),
    "AkTimeConv__MillisecondsToSamples": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_double,  # long double
        ]
    ),
    "CAkContinuousPBI::PlayToEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkContinuousPBI *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
        ]
    ),
    "CAkContinuousPBI::PrepareNextToPlay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkContinuousPBI *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkContinuousPBI::PrepareSampleAccurateTransition": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkContinuousPBI *
            ctypes.c_ubyte,  # _BOOL8
        ]
    ),
    "CAkContinuousPBI::SeekPercent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkContinuousPBI *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkContinuousPBI::SeekTimeAbsolute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkContinuousPBI *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkContinuousPBI::SetEstimatedLength": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkContinuousPBI *
            ctypes.c_float,  # float
        ]
    ),
    "CAkContinuousPBI::SetPauseStateForContinuous": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkContinuousPBI *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkContinuousPBI::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkContinuousPBI *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkContinuousPBI::_Stop": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkContinuousPBI *
            ctypes.c_uint32,  # AkPBIStopMode
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMidiDeviceCtx::CAkMidiDeviceCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiDeviceCtx *
            ctypes.c_ulonglong,  # CAkMidiDeviceMgr *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_ulonglong,  # TransParams *
            ctypes.c_ulonglong,  # UserParams *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkMidiDeviceCtx::AddEvent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiDeviceCtx *
            ctypes.c_ulonglong,  # AkMidiEventEx *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMidiDeviceCtx::HasMoreEvents": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiDeviceCtx *
        ]
    ),
    "CAkMidiDeviceCtx::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiDeviceCtx *
        ]
    ),
    "CAkMidiDeviceCtx::IsCtxFinished": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiDeviceCtx *
        ]
    ),
    "CAkMidiDeviceCtx::OnFrame": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiDeviceCtx *
            ctypes.c_ulonglong,  # AkListBare<AkMidiFrameEvent,AkListBareNextItem,AkCountPolicyNoCount,AkLastPolicyWithLast> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkMidiDeviceCtx::Pause": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiDeviceCtx *
        ]
    ),
    "CAkMidiDeviceCtx::ResolveMidiTarget": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiDeviceCtx *
        ]
    ),
    "CAkMidiDeviceCtx::Resume": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiDeviceCtx *
        ]
    ),
    "CAkMidiDeviceCtx::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkMidiDeviceCtx *
        ]
    ),
    "DSP::SIMD::ProcessBufferAll<DSP::SIMD::OutOfPlaceBuffer,DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::SIMD::OutOfPlaceBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadMultiCoefs *
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadMultiMemories *
        ]
    ),
    "CAkSrcLpHpFilter::CAkSrcLpHpFilter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcLpHpFilter *
        ]
    ),
    "CAkSrcLpHpFilter::BypassMono": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcLpHpFilter *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSrcLpHpFilter::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcLpHpFilter *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkSrcLpHpFilter::ExecuteOutOfPlace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcLpHpFilter *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkSrcLpHpFilter::ManageBQFChange": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcLpHpFilter *
        ]
    ),
    "CAkSrcLpHpFilter::ResetRamp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcLpHpFilter *
        ]
    ),
    "DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories>::SetMemories": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CAkSrcLpHpFilter::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcLpHpFilter *
        ]
    ),
    "DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories>::Transfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories> *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadFilter<DSP::SIMD::BiquadMultiCoefs,DSP::SIMD::BiquadMultiMemories> *
        ]
    ),
    "CAkSrcLpHpFilter::Transfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcLpHpFilter *
            ctypes.c_ulonglong,  # CAkSrcLpHpFilter *
        ]
    ),
    "CAkVPLPitchNode::ConsumeBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLPitchNode *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkVPLPitchNode::GetBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLPitchNode *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkVPLPitchNode::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLPitchNode *
            ctypes.c_ulonglong,  # AkAudioFormat *
            ctypes.c_ulonglong,  # CAkPBI *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkVPLPitchNode::PipelineAdded": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLPitchNode *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkVPLPitchNode::PopMarkers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLPitchNode *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkVPLPitchNode::ReleaseBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLPitchNode *
        ]
    ),
    "CAkVPLPitchNode::ReleaseInputBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLPitchNode *
            ctypes.c_ulonglong,  # AkPipelineBuffer *
        ]
    ),
    "CAkVPLPitchNode::RelocateMedia": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLPitchNode *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "CAkVPLPitchNode::Seek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLPitchNode *
        ]
    ),
    "CAkVPLPitchNode::SwitchToNextSrc": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLPitchNode *
            ctypes.c_ulonglong,  # CAkVPLSrcNode *
        ]
    ),
    "CAkVPLPitchNode::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLPitchNode *
        ]
    ),
    "CAkVPLPitchNode::TimeSkip": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLPitchNode *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkVPLPitchNode::VirtualOff": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLPitchNode *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkVPLPitchNode::VirtualOn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLPitchNode *
            ctypes.c_uint32,  # AkVirtualQueueBehavior
        ]
    ),
    "CAkVPLFilterNodeBase::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeBase *
            ctypes.c_ulonglong,  # AK::IAkPlugin *
            ctypes.c_ulonglong,  # const AkFXDesc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
        ]
    ),
    "CAkVPLFilterNodeBase::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeBase *
        ]
    ),
    "CAkVPLFilterNodeBase::VirtualOff": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeBase *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkVPLFilterNodeBase::VirtualOn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeBase *
            ctypes.c_uint32,  # AkVirtualQueueBehavior
        ]
    ),
    "CAkVPLFilterNode::ConsumeBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNode *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkVPLFilterNode::GetBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNode *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkVPLFilterNode::GetOutputConfig": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkChannelConfig *
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNode *
            ctypes.c_ulonglong,  # AkChannelConfig *
        ]
    ),
    "CAkVPLFilterNode::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNode *
            ctypes.c_ulonglong,  # AK::IAkPlugin *
            ctypes.c_ulonglong,  # const AkFXDesc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkVPLFilterNode::ReleaseBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNode *
        ]
    ),
    "CAkVPLFilterNode::ReleaseMemory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNode *
        ]
    ),
    "CAkVPLFilterNode::Seek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNode *
        ]
    ),
    "CAkVPLFilterNode::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNode *
        ]
    ),
    "CAkVPLFilterNode::TimeSkip": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNode *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkVPLFilterNode::VirtualOn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNode *
            ctypes.c_uint32,  # AkVirtualQueueBehavior
        ]
    ),
    "CAkVPLFilterNodeObjectProc::ConsumeBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeObjectProc *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkVPLFilterNodeObjectProc::TimeSkip": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeObjectProc *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkVPLFilterNodeOutOfPlace::ConsumeBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeOutOfPlace *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkVPLFilterNodeOutOfPlace::GetBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeOutOfPlace *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkVPLFilterNodeOutOfPlace::GetOutputConfig": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkChannelConfig *
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeOutOfPlace *
            ctypes.c_ulonglong,  # AkChannelConfig *
        ]
    ),
    "CAkVPLFilterNodeOutOfPlace::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeOutOfPlace *
            ctypes.c_ulonglong,  # AK::IAkPlugin *
            ctypes.c_ulonglong,  # const AkFXDesc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # CAkVPLSrcCbxNode *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkVPLFilterNodeOutOfPlace::PopMarkers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeOutOfPlace *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkVPLFilterNodeOutOfPlace::ProcessDone": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeOutOfPlace *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkVPLFilterNodeOutOfPlace::ReleaseBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeOutOfPlace *
        ]
    ),
    "CAkVPLFilterNodeOutOfPlace::ReleaseInputBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeOutOfPlace *
        ]
    ),
    "CAkVPLFilterNodeOutOfPlace::ReleaseMemory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeOutOfPlace *
        ]
    ),
    "CAkVPLFilterNodeOutOfPlace::Seek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeOutOfPlace *
        ]
    ),
    "CAkVPLFilterNodeOutOfPlace::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeOutOfPlace *
        ]
    ),
    "CAkVPLFilterNodeOutOfPlace::TimeSkip": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeOutOfPlace *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkVPLFilterNodeOutOfPlace::VirtualOn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVPLFilterNodeOutOfPlace *
            ctypes.c_uint32,  # AkVirtualQueueBehavior
        ]
    ),
    "AK::SrcMedia::Stream::CommitStreamBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SrcMedia::Stream::State *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SrcMedia::Stream::FetchStreamBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SrcMedia::Stream::State *
        ]
    ),
    "AK::SrcMedia::Stream::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SrcMedia::Stream::State *
            ctypes.c_ulonglong,  # const AK::SrcMedia::Stream::InitParams *
        ]
    ),
    "AK::SrcMedia::Stream::ReadStreamBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SrcMedia::Stream::State *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AK::SrcMedia::Stream::RelocateMedia": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SrcMedia::Stream::State *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "AK::SrcMedia::Stream::Seek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SrcMedia::Stream::State *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "AK::SrcMedia::Stream::SetStreamPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SrcMedia::Stream::State *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SrcMedia::Stream::StopLooping": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SrcMedia::Stream::State *
        ]
    ),
    "AK::SrcMedia::Stream::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::SrcMedia::Stream::State *
        ]
    ),
    "AK::SrcMedia::AbsoluteToRelativeSourceOffset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned __int16 *
        ]
    ),
    "AK::SrcMedia::VirtualVoice::AdvanceTimeSkip": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::SrcMedia::VirtualVoice::TimeSkipResult *
        argtypes=[
            ctypes.c_ulonglong,  # AK::SrcMedia::VirtualVoice::TimeSkipResult *
            ctypes.c_ulonglong,  # AK::SrcMedia::VirtualVoice::TimeSkipState *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AK::SrcMedia::Position::State *
        ]
    ),
    "AK::SrcMedia::Position::Duration": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # const AK::SrcMedia::Position::State *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "AK::SrcMedia::Position::Forward": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::SrcMedia::Position::State *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AK::SrcMedia::EnvelopeAnalyzer::GetEnvelope": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # AK::SrcMedia::EnvelopeAnalyzer::State *
            ctypes.c_ulonglong,  # AkFileParser::AnalysisData *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::SrcMedia::Position::Init": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::SrcMedia::Position::State *
            ctypes.c_ulonglong,  # const AK::SrcMedia::Header *
            ctypes.c_ulonglong,  # const AK::SrcMedia::CodecInfo *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "AK::SrcMedia::Position::PositionInformation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkBufferPosInformation *
        argtypes=[
            ctypes.c_ulonglong,  # AkBufferPosInformation *
            ctypes.c_ulonglong,  # const AK::SrcMedia::Position::State *
            ctypes.c_float,  # float
        ]
    ),
    "AK::SrcMedia::VirtualVoice::StartTimeSkip": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::SrcMedia::VirtualVoice::TimeSkipState *
            ctypes.c_ulonglong,  # const AK::SrcMedia::Position::State *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkFxBaseStateAware::CAkFxBaseStateAware": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxBaseStateAware *
        ]
    ),
    "CAkFxBaseStateAware::~CAkFxBaseStateAware": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxBaseStateAware *
        ]
    ),
    "CAkFxBaseStateAware::EnsureStateData": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxBaseStateAware *
        ]
    ),
    "CAkFxBaseStateAware::NotifyStateParamTargets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxBaseStateAware *
        ]
    ),
    "CAkFxBaseStateAware::PushStateParamUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxBaseStateAware *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_uint32,  # AkRtpcAccum
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CAkFxBaseStateAware::SetStateProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkFxBaseStateAware *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkStatePropertyUpdate *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSpatialAudioComponent::CAkSpatialAudioComponent": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioComponent *
        ]
    ),
    "AkSortedKeyArray<uint,AkAuxBusRef,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<uint,AkAuxBusRef>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAuxBusRef>,AkDefaultSortedKeyCompare<uint>>::BinarySearch": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkAuxBusRef *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,AkAuxBusRef,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<unsigned int,AkAuxBusRef>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAuxBusRef>,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AkSortedKeyArray<uint,AkAuxBusRef,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<uint,AkAuxBusRef>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAuxBusRef>,AkDefaultSortedKeyCompare<uint>>::Exists": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkAuxBusRef *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,AkAuxBusRef,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<unsigned int,AkAuxBusRef>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAuxBusRef>,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSpatialAudioComponent::GetEarlyReflectionsAuxSends": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioComponent *
            ctypes.c_ulonglong,  # AkArray<AkAuxSendValueEx,AkAuxSendValueEx const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAuxSendValueEx> > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
        ]
    ),
    "CAkSpatialAudioComponent::GetRoomReverbAuxSends": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioComponent *
            ctypes.c_ulonglong,  # AkArray<AkAuxSendValueEx,AkAuxSendValueEx const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAuxSendValueEx> > *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "CAkSpatialAudioComponent::IsReflectionsAuxBus": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioComponent *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkSortedKeyArray<uint,AkAuxBusRef,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<uint,AkAuxBusRef>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAuxBusRef>,AkDefaultSortedKeyCompare<uint>>::Set": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkAuxBusRef *
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,AkAuxBusRef,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<unsigned int,AkAuxBusRef>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAuxBusRef>,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "CAkSpatialAudioComponent::TrackReflectAuxBusCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioComponent *
            ctypes.c_ulonglong,  # CAkSpatialAudioReflectBusCtx *
        ]
    ),
    "CAkSpatialAudioComponent::TrackVoice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioComponent *
            ctypes.c_ulonglong,  # CAkSpatialAudioEmitterVoiceCtx *
        ]
    ),
    "AkSortedKeyArray<uint,AkAuxBusRef,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<uint,AkAuxBusRef>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAuxBusRef>,AkDefaultSortedKeyCompare<uint>>::Unset": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkSortedKeyArray<unsigned int,AkAuxBusRef,AkArrayAllocatorNoAlign<0>,AkGetArrayKey<unsigned int,AkAuxBusRef>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAuxBusRef>,AkDefaultSortedKeyCompare<unsigned int> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSpatialAudioComponent::UntrackReflectAuxBusCtx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioComponent *
            ctypes.c_ulonglong,  # CAkSpatialAudioReflectBusCtx *
        ]
    ),
    "CAkSpatialAudioComponent::UntrackVoice": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioComponent *
            ctypes.c_ulonglong,  # CAkSpatialAudioEmitterVoiceCtx *
        ]
    ),
    "CAkSpatialAudioComponent::UpdateTracking": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSpatialAudioComponent *
            ctypes.c_ulonglong,  # CAkSpatialAudioEmitterVoiceCtx *
        ]
    ),
    "CAkActionPlay::CAkActionPlay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlay *
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionPlay::~CAkActionPlay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlay *
        ]
    ),
    "CAkActionPlay::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionPlay *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionPlay::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlay *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkActionPlay::SetActionParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlay *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkActionPause::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionPause *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionPause::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPause *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkActionPause::SetActionActiveParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPause *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkActionPause::SetActionSpecificParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionResume *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkActionResume::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionResume *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionResume::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionResume *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkActionResume::SetActionActiveParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionResume *
            ctypes.c_ulonglong,  # ActionParams *
        ]
    ),
    "CAkActionBreak::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionBreak *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionBreak::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionBreak *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkActionMute::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionMute *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionMute::ExecResetValue": {
        "CAkActionMute *, CAkParameterNodeBase *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActionMute *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ]
        ),
        "CAkActionMute *, CAkParameterNodeBase *, CAkRegisteredObj *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActionMute *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
                ctypes.c_ulonglong,  # CAkRegisteredObj *
            ]
        ),
    },
    "CAkActionMute::ExecResetValueExcept": {
        "CAkActionMute *, CAkParameterNodeBase *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActionMute *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ]
        ),
        "CAkActionMute *, CAkParameterNodeBase *, CAkRegisteredObj *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActionMute *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
                ctypes.c_ulonglong,  # CAkRegisteredObj *
            ]
        ),
    },
    "CAkActionMute::ExecSetValue": {
        "CAkActionMute *, CAkParameterNodeBase *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActionMute *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ]
        ),
        "CAkActionMute *, CAkParameterNodeBase *, CAkRegisteredObj *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActionMute *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
                ctypes.c_ulonglong,  # CAkRegisteredObj *
            ]
        ),
    },
    "CAkActionSetAkProp::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionSetAkProp *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkPropID
        ]
    ),
    "CAkActionSetAkProp::ExecResetValue": {
        "CAkActionSetAkProp *, CAkParameterNodeBase *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActionSetAkProp *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ]
        ),
        "CAkActionSetAkProp *, CAkParameterNodeBase *, CAkRegisteredObj *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActionSetAkProp *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
                ctypes.c_ulonglong,  # CAkRegisteredObj *
            ]
        ),
    },
    "CAkActionSetAkProp::ExecResetValueExcept": {
        "CAkActionSetAkProp *, CAkParameterNodeBase *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActionSetAkProp *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ]
        ),
        "CAkActionSetAkProp *, CAkParameterNodeBase *, CAkRegisteredObj *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActionSetAkProp *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
                ctypes.c_ulonglong,  # CAkRegisteredObj *
            ]
        ),
    },
    "CAkActionSetAkProp::ExecSetValue": {
        "CAkActionSetAkProp *, CAkParameterNodeBase *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActionSetAkProp *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ]
        ),
        "CAkActionSetAkProp *, CAkParameterNodeBase *, CAkRegisteredObj *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActionSetAkProp *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
                ctypes.c_ulonglong,  # CAkRegisteredObj *
            ]
        ),
    },
    "CAkActionSetAkProp::ExecSetValueInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionSetAkProp *
            ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # AkValueMeaning
            ctypes.c_float,  # float
        ]
    ),
    "CAkActionSetAkProp::SetActionSpecificParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionSetAkProp *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkActionSetState::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionSetState *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionSetState::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionSetState *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkActionSetSwitch::SetActionParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionSetSwitch *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkActionSetSwitch::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionSetSwitch *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionSetSwitch::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionSetSwitch *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkActionSetGameParameter::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionSetGameParameter *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionSetGameParameter::ExecResetValue": {
        "CAkActionSetGameParameter *, CAkParameterNodeBase *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActionSetGameParameter *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ]
        ),
        "CAkActionSetGameParameter *, CAkParameterNodeBase *, CAkRegisteredObj *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActionSetGameParameter *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
                ctypes.c_ulonglong,  # CAkRegisteredObj *
            ]
        ),
    },
    "CAkActionSetGameParameter::ExecSetValue": {
        "CAkActionSetGameParameter *, CAkParameterNodeBase *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActionSetGameParameter *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
            ]
        ),
        "CAkActionSetGameParameter *, CAkParameterNodeBase *, CAkRegisteredObj *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # CAkActionSetGameParameter *
                ctypes.c_ulonglong,  # CAkParameterNodeBase *
                ctypes.c_ulonglong,  # CAkRegisteredObj *
            ]
        ),
    },
    "CAkActionSetGameParameter::SetActionSpecificParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionSetGameParameter *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkActionUseState::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionUseState *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionUseState::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionUseState *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkActionBypassFX::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionBypassFX *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionBypassFX::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionBypassFX *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkActionBypassFX::ResetBypassFXAEHelper": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionBypassFX *
            ctypes.c_ulonglong,  # const CAkList2<WwiseObjectIDext,WwiseObjectIDext const &,1,0> *
        ]
    ),
    "CAkActionBypassFX::SetActionParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionBypassFX *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkActionTrigger::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionTrigger *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionTrigger::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionTrigger *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "RandomizerModifier::GetModValue<float>": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # RANGED_PARAMETER<float> *
        ]
    ),
    "CAkActionSeek::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionSeek *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionSeek::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionSeek *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkActionSeek::SetActionParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionSeek *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkActionRelease::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionRelease *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionRelease::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionRelease *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkActionPlayEvent::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionPlayEvent *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionPlayEvent::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionPlayEvent *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkActionResetPlaylist::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkActionResetPlaylist *
        argtypes=[
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionResetPlaylist::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionResetPlaylist *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkActionExcept::CAkActionExcept": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionExcept *
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionExcept::~CAkActionExcept": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionExcept *
        ]
    ),
    "CAkActionExcept::SetExceptParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionExcept *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkActionActive::CAkActionActive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionActive *
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionActive::~CAkActionActive": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionActive *
        ]
    ),
    "CAkActionActive::AllExecExcept": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionActive *
            ctypes.c_uint32,  # ActionParamType
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionActive::Exec": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionActive *
            ctypes.c_uint32,  # ActionParamType
            ctypes.c_ulonglong,  # CAkRegisteredObj *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionSetValue::SetActionParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionSetValue *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AkAtomicListAdd": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # volatile void **
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # AkMemID
        ]
    ),
    "AkAtomicListGetFirst": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkAtomicListNode *
        argtypes=[
            ctypes.c_ulonglong,  # volatile void **
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AkAtomicListInit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # volatile void **
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # AkMemID
        ]
    ),
    "AkAtomicListRemove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # volatile void **
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AkAtomicListRemoveAll": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # volatile void **
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AkAtomicListTerm": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # volatile void **
            ctypes.c_uint32,  # AkMemID
        ]
    ),
    "AkAudioMix::Allocate": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioMix *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkAudioMix::Free": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioMix *
        ]
    ),
    "AkSystemOutputMeta::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AkSystemOutputMeta *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkSystemOutputMeta": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "AkSystemOutputMeta::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkSystemOutputMeta *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkSystemOutputMeta::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkSystemOutputMeta *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AkSystemOutputMeta::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkSystemOutputMeta *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSinkCoreAudio::CAkSinkCoreAudio": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSinkCoreAudio *
        ]
    ),
    "AkCreateDefaultSink": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "AkSpatialSinkWorkCallback": FUNCDEF(
        restype=ctypes.c_uint32,  # unsigned int
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CopyWAVEFORMATEX": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const tWAVEFORMATEX *
            ctypes.c_ulonglong,  # WAVEFORMATEXTENSIBLE *
        ]
    ),
    "CAkSinkCoreAudio::Get3DAudioCapabilities": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSinkCoreAudio *
            ctypes.c_ulonglong,  # Ak3DAudioSinkCapabilities *
        ]
    ),
    "AkArray<CAkSinkCoreAudio::SpatialAudioDynamicObject,CAkSinkCoreAudio::SpatialAudioDynamicObject,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkSinkCoreAudio::SpatialAudioDynamicObject>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<CAkSinkCoreAudio::SpatialAudioDynamicObject,CAkSinkCoreAudio::SpatialAudioDynamicObject,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkSinkCoreAudio::SpatialAudioDynamicObject> > *
        ]
    ),
    "CAkSinkCoreAudio::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSinkCoreAudio *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkSinkPluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkSinkCoreAudio::InitializeSpatialStream": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSinkCoreAudio *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSinkCoreAudio::InitializeWasapiStream": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSinkCoreAudio *
            ctypes.c_ulonglong,  # const DeviceEndpointInfo *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSinkCoreAudio::IsDataNeeded": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSinkCoreAudio *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkSinkCoreAudio::IsStarved": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # CAkSinkCoreAudio *
        ]
    ),
    "CAkSinkCoreAudio::MixAndInterleave": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSinkCoreAudio *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkRamp *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "CAkSinkCoreAudio::NegotiateWasapiConfig": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSinkCoreAudio *
            ctypes.c_ulonglong,  # const DeviceEndpointInfo *
            ctypes.c_ulonglong,  # WAVEFORMATEXTENSIBLE *
            ctypes.c_ulonglong,  # AkChannelConfig *
        ]
    ),
    "CAkSinkCoreAudio::OnFrameEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSinkCoreAudio *
        ]
    ),
    "CAkSinkCoreAudio::ResetStarved": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSinkCoreAudio *
        ]
    ),
    "CAkSinkCoreAudio::SpatialSinkWorkCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSinkCoreAudio *
        ]
    ),
    "CAkSinkCoreAudio::SubmitWasapiBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSinkCoreAudio *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkSinkCoreAudio::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSinkCoreAudio *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "WaveFormatToChannelConfig": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkChannelConfig *
        argtypes=[
            ctypes.c_ulonglong,  # AkChannelConfig *
            ctypes.c_ulonglong,  # tWAVEFORMATEX *
        ]
    ),
    "MixChannel_V4F32": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_double,  # double
            ctypes.c_double,  # double
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "MixNinNChannels_V4F32": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkRamp *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_float,  # float
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "MixChannel_V8F32_AVX2": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_double,  # double
            ctypes.c_double,  # double
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "MixNinNChannels_V8F32_AVX2": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkRamp *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_float,  # float
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "Bypass_I16_1ChanV4F32": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Bypass_I16_2ChanV4F32": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Bypass_I16_NChanV4F32": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Fixed_I16_1ChanV4F32": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Fixed_I16_2ChanV4F32": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Fixed_I16_NChanV4F32": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Fixed_Native_1ChanV4F32": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Fixed_Native_2ChanV4F32": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Fixed_Native_NChanV4F32": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Interpolating_I16_1Chan": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Interpolating_I16_2Chan": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Interpolating_Native_1Chan": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Interpolating_Native_2Chan": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Bypass_I16_1ChanAVX2": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Bypass_I16_2ChanAVX2": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Bypass_I16_NChanAVX2": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Fixed_I16_1ChanAVX2": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Fixed_I16_2ChanAVX2": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Fixed_I16_NChanAVX2": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Fixed_Native_1ChanAVX2": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Fixed_Native_2ChanAVX2": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Fixed_Native_NChanAVX2": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Bypass_Native_NChan": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Interleave_Native_NChan": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "Interpolating_I16_NChan": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "Interpolating_Native_NChan": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkInternalPitchState *
        ]
    ),
    "CAkSrcPhysModel::CAkSrcPhysModel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkSrcPhysModel::AllocGrainCodec": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAkGrainCodec *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::ChangeSourcePosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::ComputePriorityWithDistance": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
            ctypes.c_float,  # float
        ]
    ),
    "CAkSrcPhysModel::FreeGrainCodec": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
            ctypes.c_ulonglong,  # IAkGrainCodec *
        ]
    ),
    "CAkSrcPhysModel::GetAnalyzedEnvelope": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkSrcPhysModel::GetAudioNodeID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::GetBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkSrcPhysModel::GetCookie": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::GetDuration": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::GetEmitterListenerPair": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkEmitterListenerPair *
        ]
    ),
    "CAkSrcPhysModel::GetGameObjectID": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::GetGameObjectInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # CAkVPLNode **
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::GetGameObjectMultiPositionType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::GetGameObjectPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkTransform *
        ]
    ),
    "CAkSrcPhysModel::GetGameObjectScaling": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::GetListenerData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # AkListener *
        ]
    ),
    "CAkSrcPhysModel::GetListeners": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkSrcPhysModel::GetMidiEvent": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkMIDIEvent *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
            ctypes.c_ulonglong,  # AkMIDIEvent *
        ]
    ),
    "CAkSrcPhysModel::GetNumEmitterListenerPairs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::GetNumGameObjectPositions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::GetNumLoops": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::GetOutputBusNode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkVPL *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::GetParentChannelConfig": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
            ctypes.c_ulonglong,  # AkChannelConfig *
        ]
    ),
    "CAkSrcPhysModel::GetPlayingID": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::GetPluginMedia": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkSrcPhysModel::GetPriority": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::GetVoiceInfo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkFileParser::AnalysisData **
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::IsUsingThisSlot": {
        "CAkSrcPhysModel *, const unsigned __int8 *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # CAkSrcPhysModel *
                ctypes.c_ulonglong,  # const unsigned __int8 *
            ]
        ),
        "CAkSrcPhysModel *, const CAkUsageSlot *": FUNCDEF(
            restype=ctypes.c_char,  # char
            argtypes=[
                ctypes.c_ulonglong,  # CAkSrcPhysModel *
                ctypes.c_ulonglong,  # const CAkUsageSlot *
            ]
        ),
    },
    "CAkSrcPhysModel::ReleaseBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::RestartSourcePlugin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::StartStream": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSrcPhysModel::StopLooping": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::StopStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
        ]
    ),
    "CAkSrcPhysModel::PluginParams::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel::PluginParams *
        ]
    ),
    "CAkSrcPhysModel::TimeSkip": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkSrcPhysModel::VirtualOn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcPhysModel *
            ctypes.c_uint32,  # AkVirtualQueueBehavior
        ]
    ),
    "CAkSrcFileADPCM::CAkSrcFileADPCM": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileADPCM *
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkSrcFileADPCM::ChangeSourcePosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileADPCM *
        ]
    ),
    "CAkSrcFileADPCM::FindClosestFileOffset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileADPCM *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkSrcFileADPCM::GetBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileADPCM *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkSrcBaseEx::OnLoopComplete": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBaseEx *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSrcFileADPCM::ParseHeader": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileADPCM *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "CAkSrcBaseEx::SetInfiniteLooping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBaseEx *
        ]
    ),
    "CAkSrcFileADPCM::StopStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileADPCM *
        ]
    ),
    "CAkSrcFileADPCM::VirtualOn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileADPCM *
            ctypes.c_uint32,  # AkVirtualQueueBehavior
        ]
    ),
    "AkHpfParamEval::EvalCutoff": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "AkLpfParamEval::EvalCutoff": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "CAkActionSetValue::CAkActionSetValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionSetValue *
            ctypes.c_uint32,  # AkActionType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkActionSetValue::~CAkActionSetValue": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionSetValue *
        ]
    ),
    "CAkActionSetValue::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionSetValue *
            ctypes.c_ulonglong,  # AkPendingAction *
        ]
    ),
    "CAkActionSetValue::ResetAEHelper": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkActionSetValue *
            ctypes.c_ulonglong,  # const CAkList2<WwiseObjectIDext,WwiseObjectIDext const &,1,0> *
        ]
    ),
    "CAkSrcBaseEx::CAkSrcBaseEx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBaseEx *
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkSrcBaseEx::~CAkSrcBaseEx": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBaseEx *
        ]
    ),
    "CAkSrcBaseEx::GetAnalyzedEnvelope": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBaseEx *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSrcBaseEx::GetDuration": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBaseEx *
        ]
    ),
    "CAkSrcBaseEx::GetSourceOffset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBaseEx *
        ]
    ),
    "CAkSrcBaseEx::GetSourceSampleRate": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBaseEx *
        ]
    ),
    "CAkSrcBaseEx::Seek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBaseEx *
        ]
    ),
    "CAkSrcBaseEx::StopLooping": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBaseEx *
        ]
    ),
    "CAkSrcBaseEx::StopStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBaseEx *
        ]
    ),
    "CAkSrcBaseEx::TimeSkip": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBaseEx *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkSrcFileBase::CAkSrcFileBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
            ctypes.c_ulonglong,  # CAkPBI *
        ]
    ),
    "CAkSrcFileBase::~CAkSrcFileBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
        ]
    ),
    "CAkSrcFileBase::ChangeSourcePosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
        ]
    ),
    "CAkSrcFileBase::CreateStream": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
            ctypes.c_ulonglong,  # AkAutoStmBufSettings *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "CAkSrcFileBase::FetchStreamBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
        ]
    ),
    "CAkSrcFileBase::IsInitialPrebufferingReady": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
        ]
    ),
    "CAkSrcFileBase::MustRelocateAnalysisDataOnMediaRelocation": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
        ]
    ),
    "CAkSrcFileBase::ProcessFirstBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
        ]
    ),
    "CAkSrcFileBase::ProcessStreamBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSrcFileBase::RelocateMedia": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "CAkSrcFileBase::SeekStream": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkSrcFileBase::SeekToSourceOffset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
        ]
    ),
    "CAkSrcFileBase::SetStreamPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSrcFileBase::StartStream": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "CAkSrcFileBase::StopLooping": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
        ]
    ),
    "CAkSrcFileBase::StopStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
        ]
    ),
    "CAkSrcFileBase::StoreAnalysisData": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
            ctypes.c_ulonglong,  # AkFileParser::AnalysisDataChunk *
        ]
    ),
    "CAkSrcFileBase::UpdateBufferingInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
        ]
    ),
    "CAkSrcFileBase::VirtualOff": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
            ctypes.c_uint32,  # AkVirtualQueueBehavior
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSrcFileBase::VirtualOn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
            ctypes.c_uint32,  # AkVirtualQueueBehavior
        ]
    ),
    "CAkSrcFileBase::_StartStream": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcFileBase *
            ctypes.c_ulonglong,  # AkAutoStmBufSettings *
        ]
    ),
    "CAkADPCMCodec::Decode": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "PtADPCM::DecodeBlocks": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # __int16 *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CreateAkStereoDelayFX": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkStereoDelayFX::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkStereoDelayFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkStereoDelayFX::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStereoDelayFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkEffectPluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "CAkStereoDelayFX::Reset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStereoDelayFX *
        ]
    ),
    "CAkStereoDelayFX::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStereoDelayFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkStereoDelayFXParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # CAkStereoDelayFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkStereoDelayFXParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkStereoDelayFXParams::GetParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkStereoDelayFXParams *
            ctypes.c_ulonglong,  # AkStereoDelayFXParams *
        ]
    ),
    "CAkStereoDelayFXParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStereoDelayFXParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkStereoDelayFXParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkStereoDelayFXParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::DSP::CAkDelayLineMemory<float>::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::DSP::CAkDelayLineMemory<float> *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::DSP::CStereoDelayLine::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::DSP::CStereoDelayLine *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::DSP::CStereoDelayLine::ProcessBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::DSP::CStereoDelayLine *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkStereoDelayChannelParams *
            ctypes.c_ulonglong,  # AkStereoDelayChannelParams *
            ctypes.c_ulonglong,  # AkStereoDelayFilterParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::DSP::CStereoDelayLine::Reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::DSP::CStereoDelayLine *
        ]
    ),
    "AK::DSP::CStereoDelayLine::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::DSP::CStereoDelayLine *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "AkStereoDelayDSPProcess": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkStereoDelayFXInfo *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "AK__DSP__ApplyGain_1": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_double,  # double
            ctypes.c_double,  # double
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "ProcessSurroundChannels": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkStereoDelayFXInfo *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "AK::StreamMgr::AddLanguageChangeObserver": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'const wchar_t*', 'name': 'const'}, 'void *']}
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AK::StreamMgr::CAkStreamMgr::CachedFileStreamData::AddRef": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStreamMgr::CachedFileStreamData *
            ctypes.c_char,  # char
        ]
    ),
    "AK::StreamMgr::AkDeferredOpenData::Create": {
        "unsigned int, AkFileSystemFlags *, AkOpenMode": FUNCDEF(
            restype=ctypes.c_ulonglong,  # AK::StreamMgr::AkDeferredOpenData *
            argtypes=[
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # AkFileSystemFlags *
                ctypes.c_uint32,  # AkOpenMode
            ]
        ),
        "const wchar_t *, AkFileSystemFlags *, AkOpenMode": FUNCDEF(
            restype=ctypes.c_ulonglong,  # AK::StreamMgr::AkDeferredOpenData *
            argtypes=[
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # AkFileSystemFlags *
                ctypes.c_uint32,  # AkOpenMode
            ]
        ),
    },
    "AK::StreamMgr::Create": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkStreamMgr *
        argtypes=[
            ctypes.c_ulonglong,  # const AkStreamMgrSettings *
        ]
    ),
    "AK::StreamMgr::CAkStreamMgr::CreateAuto": {
        "AK::StreamMgr::CAkStreamMgr *, unsigned int, AkFileSystemFlags *, const AkAutoStmHeuristics *, AkAutoStmBufSettings *, AK::IAkAutoStream **, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # AK::StreamMgr::CAkStreamMgr *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # AkFileSystemFlags *
                ctypes.c_ulonglong,  # const AkAutoStmHeuristics *
                ctypes.c_ulonglong,  # AkAutoStmBufSettings *
                ctypes.c_ulonglong,  # AK::IAkAutoStream **
                ctypes.c_ubyte,  # bool
            ]
        ),
        "AK::StreamMgr::CAkStreamMgr *, void *, unsigned __int64, const AkAutoStmHeuristics *, AK::IAkAutoStream **": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # AK::StreamMgr::CAkStreamMgr *
                ctypes.c_ulonglong,  # void *
                ctypes.c_uint64,  # unsigned __int64
                ctypes.c_ulonglong,  # const AkAutoStmHeuristics *
                ctypes.c_ulonglong,  # AK::IAkAutoStream **
            ]
        ),
        "AK::StreamMgr::CAkStreamMgr *, const wchar_t *, AkFileSystemFlags *, const AkAutoStmHeuristics *, AkAutoStmBufSettings *, AK::IAkAutoStream **, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # AK::StreamMgr::CAkStreamMgr *
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # AkFileSystemFlags *
                ctypes.c_ulonglong,  # const AkAutoStmHeuristics *
                ctypes.c_ulonglong,  # AkAutoStmBufSettings *
                ctypes.c_ulonglong,  # AK::IAkAutoStream **
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "AK::StreamMgr::CAkStreamMgr::CreateCachingStream": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStreamMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkFileSystemFlags *
            ctypes.c_char,  # char
            ctypes.c_ulonglong,  # AK::IAkAutoStream **
        ]
    ),
    "AK::StreamMgr::CAkStreamMgr::CreateDevice": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStreamMgr *
            ctypes.c_ulonglong,  # const AkDeviceSettings *
            ctypes.c_ulonglong,  # AK::StreamMgr::IAkLowLevelIOHook *
        ]
    ),
    "AK::StreamMgr::CreateDevice": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const AkDeviceSettings *
            ctypes.c_ulonglong,  # AK::StreamMgr::IAkLowLevelIOHook *
        ]
    ),
    "AK::StreamMgr::CAkStreamMgr::CreateStd": {
        "AK::StreamMgr::CAkStreamMgr *, unsigned int, AkFileSystemFlags *, AkOpenMode, AK::IAkStdStream **, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # AK::StreamMgr::CAkStreamMgr *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # AkFileSystemFlags *
                ctypes.c_uint32,  # AkOpenMode
                ctypes.c_ulonglong,  # AK::IAkStdStream **
                ctypes.c_ubyte,  # bool
            ]
        ),
        "AK::StreamMgr::CAkStreamMgr *, const wchar_t *, AkFileSystemFlags *, AkOpenMode, AK::IAkStdStream **, bool": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # AK::StreamMgr::CAkStreamMgr *
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # AkFileSystemFlags *
                ctypes.c_uint32,  # AkOpenMode
                ctypes.c_ulonglong,  # AK::IAkStdStream **
                ctypes.c_ubyte,  # bool
            ]
        ),
    },
    "AK::StreamMgr::AkDeferredOpenData::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::AkDeferredOpenData *
        ]
    ),
    "AK::StreamMgr::CAkStreamMgr::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStreamMgr *
        ]
    ),
    "AK::StreamMgr::DestroyDevice": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::AkDeferredOpenData::Execute": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::AkDeferredOpenData *
            ctypes.c_ulonglong,  # AkFileDesc *
        ]
    ),
    "AK::StreamMgr::FlushAllCaches": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr *
        ]
    ),
    "AK::StreamMgr::CAkStreamMgr::ForceCleanup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
            ctypes.c_char,  # char
        ]
    ),
    "AkKeyDataPtrStruct<uint,AK::StreamMgr::CAkStreamMgr::CachedFileStreamData,AkArrayAllocatorNoAlign<0>>::FreeData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkKeyDataPtrStruct<unsigned int,AK::StreamMgr::CAkStreamMgr::CachedFileStreamData,AkArrayAllocatorNoAlign<0> > *
        ]
    ),
    "AK::StreamMgr::CAkStreamMgr::GetBufferStatusForPinnedFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStreamMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AK::StreamMgr::GetCurrentLanguage": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[]
    ),
    "AK::StreamMgr::GetDefaultDeviceSettings": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkDeviceSettings *
        ]
    ),
    "AK::StreamMgr::GetFileLocationResolver": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::IAkFileLocationResolver *
        argtypes=[]
    ),
    "AkArray<signed char,signed char,AkHybridAllocator<1,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<signed char>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<signed char,signed char,AkHybridAllocator<1,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<signed char> > *
        ]
    ),
    "AkArray<MapStruct<uint,AkKeyDataPtrStruct<uint,AK::StreamMgr::CAkStreamMgr::CachedFileStreamData,AkArrayAllocatorNoAlign<0>>>,MapStruct<uint,AkKeyDataPtrStruct<uint,AK::StreamMgr::CAkStreamMgr::CachedFileStreamData,AkArrayAllocatorNoAlign<0>>> const &,AkArrayAllocatorNoAlign<7>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<uint,AkKeyDataPtrStruct<uint,AK::StreamMgr::CAkStreamMgr::CachedFileStreamData,AkArrayAllocatorNoAlign<0>>>>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<MapStruct<unsigned int,AkKeyDataPtrStruct<unsigned int,AK::StreamMgr::CAkStreamMgr::CachedFileStreamData,AkArrayAllocatorNoAlign<0> > >,MapStruct<unsigned int,AkKeyDataPtrStruct<unsigned int,AK::StreamMgr::CAkStreamMgr::CachedFileStreamData,AkArrayAllocatorNoAlign<0> > > const &,AkArrayAllocatorNoAlign<7>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<unsigned int,AkKeyDataPtrStruct<unsigned int,AK::StreamMgr::CAkStreamMgr::CachedFileStreamData,AkArrayAllocatorNoAlign<0> > > > > *
        ]
    ),
    "AK::StreamMgr::CAkStreamMgr::PinFileInCache": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStreamMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkFileSystemFlags *
            ctypes.c_char,  # char
        ]
    ),
    "AK::StreamMgr::CAkStreamMgr::RelocateMemoryStream": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStreamMgr *
            ctypes.c_ulonglong,  # AK::IAkAutoStream *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "AK::StreamMgr::RemoveLanguageChangeObserver": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AK::StreamMgr::SetCurrentLanguage": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "AK::StreamMgr::SetFileLocationResolver": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::IAkFileLocationResolver *
        ]
    ),
    "AK::StreamMgr::CAkStreamMgr::UnpinFileInCache": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStreamMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkKeyArray<uint,AkKeyDataPtrStruct<uint,AK::StreamMgr::CAkStreamMgr::CachedFileStreamData,AkArrayAllocatorNoAlign<0>>,AkArrayAllocatorNoAlign<7>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<uint,AkKeyDataPtrStruct<uint,AK::StreamMgr::CAkStreamMgr::CachedFileStreamData,AkArrayAllocatorNoAlign<0>>>>>::Unset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkKeyArray<unsigned int,AkKeyDataPtrStruct<unsigned int,AK::StreamMgr::CAkStreamMgr::CachedFileStreamData,AkArrayAllocatorNoAlign<0> >,AkArrayAllocatorNoAlign<7>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<unsigned int,AkKeyDataPtrStruct<unsigned int,AK::StreamMgr::CAkStreamMgr::CachedFileStreamData,AkArrayAllocatorNoAlign<0> > > > > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkStreamMgr::UpdateCachingPriority": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStreamMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_char,  # char
            ctypes.c_char,  # char
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::CAkAutoStmBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmMemory::CAkAutoStmMemory": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmMemory *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::CAkDeviceBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
            ctypes.c_ulonglong,  # AK::StreamMgr::IAkLowLevelIOHook *
        ]
    ),
    "AK::StreamMgr::CAkStdStmBase::CAkStdStmBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::~CAkAutoStmBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::~CAkDeviceBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
        ]
    ),
    "AK::StreamMgr::CAkStdStmBase::~CAkStdStmBase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
        ]
    ),
    "AK::StreamMgr::CAkStmTask::~CAkStmTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmTask *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::AddMemView": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemView *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::StreamMgr::CAkStdStmBase::AddMemView": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemView *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::ClearStreams": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::ClearTaskList": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
            ctypes.c_ulonglong,  # AkListBareLight<AK::StreamMgr::CAkStmTask,AkListBareLightNextItem> *
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::CreateAuto": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmTask *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
            ctypes.c_ulonglong,  # AkFileDesc *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const AkAutoStmHeuristics *
            ctypes.c_ulonglong,  # AkAutoStmBufSettings *
            ctypes.c_ulonglong,  # AK::IAkAutoStream **
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::CreateCachingStream": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmTask *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
            ctypes.c_ulonglong,  # AkFileDesc *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_char,  # char
            ctypes.c_ulonglong,  # AK::IAkAutoStream **
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::CreateStd": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmTask *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
            ctypes.c_ulonglong,  # AkFileDesc *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # AK::IAkStdStream **
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmMemory::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmMemory *
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::DestroyBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemView *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::EffectiveDeadline": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
        ]
    ),
    "AK::StreamMgr::CAkStdStmBase::EffectiveDeadline": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
        ]
    ),
    "AK::StreamMgr::CAkStmTask::EnsureFileIsOpen": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmTask *
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::ExecuteCachedTransfer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmTask *
        ]
    ),
    "AK::StreamMgr::CAkStdStmBase::ExecuteOp": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_char,  # char
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::Flush": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::FlushCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::ForceCleanup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
            ctypes.c_ubyte,  # bool
            ctypes.c_char,  # char
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::ForceTaskListCleanup": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
            ctypes.c_ubyte,  # bool
            ctypes.c_char,  # char
            ctypes.c_ulonglong,  # AkListBareLight<AK::StreamMgr::CAkStmTask,AkListBareLightNextItem> *
        ]
    ),
    "AK::StreamMgr::CAkStdStmBase::GetBlockSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::GetBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
            ctypes.c_ulonglong,  # void **
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::StreamMgr::CAkAutoStmMemory::GetBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmMemory *
            ctypes.c_ulonglong,  # void **
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::StreamMgr::CAkStdStmBase::GetData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::GetFileDescriptor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::GetHeuristics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
            ctypes.c_ulonglong,  # AkAutoStmHeuristics *
        ]
    ),
    "AK::StreamMgr::CAkStdStmBase::GetInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
            ctypes.c_ulonglong,  # AkStreamInfo *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmMemory::GetInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmMemory *
            ctypes.c_ulonglong,  # AkStreamInfo *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::GetNominalBuffering": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmMemory::GetNominalBuffering": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmMemory *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::GetPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # {'return': 'void', 'args': [{'type': 'AK::IAkAutoStream*', 'name': 'this'}]}
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AK::StreamMgr::CAkStdStmBase::GetPosition": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::GetReadBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::GetVirtualBufferingSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
            ctypes.c_ulonglong,  # AkFileDesc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkAutoStmHeuristics *
            ctypes.c_ulonglong,  # AkAutoStmBufSettings *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
            ctypes.c_ulonglong,  # const AkDeviceSettings *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkStdStmBase::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
            ctypes.c_ulonglong,  # AkFileDesc *
            ctypes.c_uint32,  # AkOpenMode
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::Kill": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
        ]
    ),
    "AK::StreamMgr::CAkStdStmBase::Kill": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::OnFileDeferredOpen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
        ]
    ),
    "AK::StreamMgr::CAkStmTask::OnFileDeferredOpen": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmTask *
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::OnThreadStart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::QueryBufferingStatus": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmMemory::QueryBufferingStatus": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmMemory *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AK::StreamMgr::CAkStdStmBase::Read": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_char,  # char
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::ReleaseBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmMemory::ReleaseBuffer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmMemory *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::ReleaseCachingBuffers": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkAutoStmMemory::Relocate": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmMemory *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::ScheduleStdStmOnly": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmTask *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::SchedulerFindNextCachingTask": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmTask *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::SchedulerFindNextTask": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmTask *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::SetBufferingSettings": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
            ctypes.c_ulonglong,  # const AkAutoStmBufSettings *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::SetCachingBufferSize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkStmTask::SetDeferredFileOpen": {
        "AK::StreamMgr::CAkStmTask *, AkFileDesc *, unsigned int, AkFileSystemFlags *, AkOpenMode": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmTask *
                ctypes.c_ulonglong,  # AkFileDesc *
                ctypes.c_uint32,  # unsigned int
                ctypes.c_ulonglong,  # AkFileSystemFlags *
                ctypes.c_uint32,  # AkOpenMode
            ]
        ),
        "AK::StreamMgr::CAkStmTask *, AkFileDesc *, const wchar_t *, AkFileSystemFlags *, AkOpenMode": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmTask *
                ctypes.c_ulonglong,  # AkFileDesc *
                ctypes.c_ulonglong,  # const wchar_t *
                ctypes.c_ulonglong,  # AkFileSystemFlags *
                ctypes.c_uint32,  # AkOpenMode
            ]
        ),
    },
    "AK::StreamMgr::CAkAutoStmBase::SetHeuristics": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
            ctypes.c_ulonglong,  # const AkAutoStmHeuristics *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmMemory::SetHeuristics": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmMemory *
            ctypes.c_ulonglong,  # const AkAutoStmHeuristics *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmMemory::SetMinimalBufferSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmMemory *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkAutoStmMemory::SetMinTargetBufferSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmMemory *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::SetMinimalBufferSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::SetPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # AkMoveMethod
            ctypes.c_ulonglong,  # __int64 *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmMemory::SetPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmMemory *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # AkMoveMethod
            ctypes.c_ulonglong,  # __int64 *
        ]
    ),
    "AK::StreamMgr::CAkStdStmBase::SetPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # AkMoveMethod
            ctypes.c_ulonglong,  # __int64 *
        ]
    ),
    "AK::StreamMgr::CAkStdStmBase::SetStatus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
            ctypes.c_uint32,  # AkStmStatus
        ]
    ),
    "AK::StreamMgr::CAkStdStmBase::SetStreamName": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmMemory::SetStreamName": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmMemory *
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::Start": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::StartCaching": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::Stop": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::StopCaching": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::UpdateCachingPriority": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
            ctypes.c_char,  # char
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::UpdateSchedulingStatus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBase::UpdateTaskStatus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
            ctypes.c_uint32,  # AKRESULT
        ]
    ),
    "AK::StreamMgr::CAkStdStmBase::UpdateTaskStatus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
            ctypes.c_uint32,  # AKRESULT
        ]
    ),
    "AK::StreamMgr::CAkStdStmBase::WaitForPendingOperation": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
        ]
    ),
    "AK::StreamMgr::CAkStdStmBase::Write": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_char,  # char
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "AK::StreamMgr::CAkDeviceBlocking::CAkDeviceBlocking": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBlocking *
            ctypes.c_ulonglong,  # AK::StreamMgr::IAkLowLevelIOHook *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBlocking::CanBeDestroyed": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBlocking *
        ]
    ),
    "AK::StreamMgr::CAkStdStmBlocking::CanBeDestroyed": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBlocking *
        ]
    ),
    "AK::StreamMgr::CAkStdStmBlocking::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBlocking *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBlocking::CancelAllPendingTransfers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBlocking *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBlocking::CancelCurrentTransfer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBlocking *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBlocking::CancelInconsistentPendingTransfers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBlocking *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::StreamMgr::CAkStdStmBlocking::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBlocking *
        ]
    ),
    "AK::StreamMgr::CAkDeviceBase::DestroyMemView": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBase *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemView *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBlocking::FlushSmallBuffersAndPendingTransfers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBlocking *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkAutoStmDeferredLinedUp::GetVirtualFilePosition": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmDeferredLinedUp *
        ]
    ),
    "AK::StreamMgr::CAkDeviceBlocking::Init": FUNCDEF(
        restype=ctypes.c_uint32,  # AKRESULT
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBlocking *
            ctypes.c_ulonglong,  # const AkDeviceSettings *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkDeviceBlocking::MemViewFactory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemView *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBlocking *
        ]
    ),
    "AK::StreamMgr::CAkDeviceBlocking::PerformIO": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBlocking *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBlocking::PrepareTransfer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemView *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBlocking *
            ctypes.c_ulonglong,  # AkFileDesc **
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkLowLevelTransfer **
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::StreamMgr::CAkStdStmBlocking::PrepareTransfer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemView *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBlocking *
            ctypes.c_ulonglong,  # AkFileDesc **
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkLowLevelTransfer **
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBlocking::ReleaseCachingBuffers": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBlocking *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBlocking::SetLoopEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBlocking *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkAutoStmBlocking::Update": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBlocking *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemView *
            ctypes.c_uint32,  # AKRESULT
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::StreamMgr::CAkStdStmBlocking::Update": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBlocking *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemView *
            ctypes.c_uint32,  # AKRESULT
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::StreamMgr::CAkDeviceBlocking::_CreateAuto": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBlocking *
            ctypes.c_ulonglong,  # AkFileDesc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkAutoStmHeuristics *
            ctypes.c_ulonglong,  # AkAutoStmBufSettings *
            ctypes.c_ulonglong,  # AK::IAkAutoStream **
        ]
    ),
    "AK::StreamMgr::CAkDeviceBlocking::_CreateStd": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceBlocking *
            ctypes.c_ulonglong,  # AkFileDesc *
            ctypes.c_uint32,  # AkOpenMode
            ctypes.c_ulonglong,  # AK::IAkStdStream **
        ]
    ),
    "AK::StreamMgr::CAkDeviceDeferredLinedUp::CAkDeviceDeferredLinedUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceDeferredLinedUp *
            ctypes.c_ulonglong,  # AK::StreamMgr::IAkLowLevelIOHook *
        ]
    ),
    "AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkAutoStmBase>::CanBeDestroyed": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkAutoStmBase> *
        ]
    ),
    "AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkStdStmBase>::CanBeDestroyed": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkStdStmBase> *
        ]
    ),
    "AK::StreamMgr::CAkStdStmDeferredLinedUp::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmDeferredLinedUp *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmDeferredLinedUp::CancelAllPendingTransfers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmDeferredLinedUp *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmDeferredLinedUp::CancelInconsistentPendingTransfers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmDeferredLinedUp *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkAutoStmBase>::CancelTransfers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkAutoStmBase> *
            ctypes.c_ulonglong,  # AkListBare<AK::StreamMgr::CAkStmMemView,AK::StreamMgr::AkListBareNextMemView,AkCountPolicyNoCount,AkLastPolicyWithLast> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::StreamMgr::CAkDeviceDeferredLinedUp::CreateLowLevelTransfer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkLowLevelTransferDeferred *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceDeferredLinedUp *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmTask *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkDeviceDeferredLinedUp::CreateMemViewAuto": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemViewDeferred *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceDeferredLinedUp *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmDeferredLinedUp *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkLowLevelTransferDeferred **
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AK::StreamMgr::CAkDeviceDeferredLinedUp::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceDeferredLinedUp *
        ]
    ),
    "AK::StreamMgr::CAkStdStmDeferredLinedUp::Destroy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmDeferredLinedUp *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmDeferredLinedUp::FlushSmallBuffersAndPendingTransfers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmDeferredLinedUp *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkAutoStmBase>::GetOldestCompletedTransfer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemView *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkAutoStmBase> *
        ]
    ),
    "AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkStdStmBase>::GetOldestCompletedTransfer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemView *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkStdStmBase> *
        ]
    ),
    "AK::StreamMgr::CAkStdStmDeferredLinedUp::GetStatus": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmDeferredLinedUp *
        ]
    ),
    "AK::StreamMgr::CAkDeviceDeferredLinedUp::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceDeferredLinedUp *
            ctypes.c_ulonglong,  # const AkDeviceSettings *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkDeviceDeferredLinedUp::MemViewFactory": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemView *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceDeferredLinedUp *
        ]
    ),
    "AK::StreamMgr::CAkDeviceDeferredLinedUp::PerformIO": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceDeferredLinedUp *
        ]
    ),
    "AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkAutoStmBase>::PopTransferRequest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkAutoStmBase> *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemView *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkStdStmBase>::PopTransferRequest": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkStdStmBase> *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemView *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::StreamMgr::CAkAutoStmDeferredLinedUp::PrepareTransfer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemViewDeferred *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmDeferredLinedUp *
            ctypes.c_ulonglong,  # AkFileDesc **
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkLowLevelTransfer **
            ctypes.c_ulonglong,  # bool *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::StreamMgr::CAkStdStmDeferredLinedUp::PrepareTransfer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemView *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmDeferredLinedUp *
            ctypes.c_ulonglong,  # AkFileDesc **
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkLowLevelTransfer **
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AK::StreamMgr::CAkAutoStmDeferredLinedUp::ReleaseCachingBuffers": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmDeferredLinedUp *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkAutoStmDeferredLinedUp::SetLoopEnd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmDeferredLinedUp *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkAutoStmBase>::Update": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkAutoStmBase> *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemView *
            ctypes.c_uint32,  # AKRESULT
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkStdStmBase>::Update": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkStdStmBase> *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemView *
            ctypes.c_uint32,  # AKRESULT
            ctypes.c_ubyte,  # bool
        ]
    ),
    "AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkAutoStmBase>::UpdateCompletedTransfers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkAutoStmBase> *
        ]
    ),
    "AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkStdStmBase>::UpdateCompletedTransfers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmDeferredLinedUpBase<AK::StreamMgr::CAkStdStmBase> *
        ]
    ),
    "AK::StreamMgr::CAkStdStmDeferredLinedUp::_CancelAllPendingTransfers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmDeferredLinedUp *
        ]
    ),
    "AK::StreamMgr::CAkDeviceDeferredLinedUp::_CreateAuto": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkAutoStmBase *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceDeferredLinedUp *
            ctypes.c_ulonglong,  # AkFileDesc *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const AkAutoStmHeuristics *
            ctypes.c_ulonglong,  # AkAutoStmBufSettings *
            ctypes.c_ulonglong,  # AK::IAkAutoStream **
        ]
    ),
    "AK::StreamMgr::CAkDeviceDeferredLinedUp::_CreateStd": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::StreamMgr::CAkStdStmBase *
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkDeviceDeferredLinedUp *
            ctypes.c_ulonglong,  # AkFileDesc *
            ctypes.c_uint32,  # AkOpenMode
            ctypes.c_ulonglong,  # AK::IAkStdStream **
        ]
    ),
    "AK::StreamMgr::CAkClientThreadAware::CAkClientThreadAware": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkClientThreadAware *
        ]
    ),
    "AK::StreamMgr::CAkIOThread::CAkIOThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOThread *
        ]
    ),
    "AK::StreamMgr::CAkClientThreadAware::~CAkClientThreadAware": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkClientThreadAware *
        ]
    ),
    "AK::StreamMgr::CAkIOThread::~CAkIOThread": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOThread *
        ]
    ),
    "AK::StreamMgr::CAkIOThread::AutoSemDecr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOThread *
        ]
    ),
    "AK::StreamMgr::CAkIOThread::AutoSemIncr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOThread *
        ]
    ),
    "AK::StreamMgr::CAkIOThread::DecrementIOCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOThread *
        ]
    ),
    "AK::StreamMgr::CAkIOThread::IOSchedThread": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AK::StreamMgr::CAkIOThread::IncrementIOCount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOThread *
        ]
    ),
    "AK::StreamMgr::CAkIOThread::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOThread *
            ctypes.c_ulonglong,  # const AkThreadProperties *
        ]
    ),
    "AK::StreamMgr::CAkIOThread::NotifyMemChange": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOThread *
        ]
    ),
    "AK::StreamMgr::CAkIOThread::NotifyMemIdle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOThread *
        ]
    ),
    "AK::StreamMgr::CAkClientThreadAware::SetBlockedStatus": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkClientThreadAware *
        ]
    ),
    "AK::StreamMgr::CAkIOThread::SignalIOCompleted": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOThread *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkClientThreadAware *
        ]
    ),
    "AK::StreamMgr::CAkIOThread::StdSemDecr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOThread *
        ]
    ),
    "AK::StreamMgr::CAkIOThread::StdSemIncr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOThread *
        ]
    ),
    "AK::StreamMgr::CAkIOThread::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOThread *
        ]
    ),
    "AK::StreamMgr::CAkIOThread::WaitForIOCompletion": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOThread *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkClientThreadAware *
        ]
    ),
    "AkBuddyBlockPool::AkBuddyBlockPool": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkBuddyBlockPool *
        ]
    ),
    "AK::StreamMgr::CAkIOMemMgr::CAkIOMemMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOMemMgr *
        ]
    ),
    "AK::StreamMgr::CAkIOMemMgr::~CAkIOMemMgr": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOMemMgr *
        ]
    ),
    "AK::StreamMgr::CAkIOMemMgr::CloneTempBlock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOMemMgr *
            ctypes.c_ulonglong,  # AK::StreamMgr::AkMemBlock *
            ctypes.c_ulonglong,  # AK::StreamMgr::AkMemBlock **
        ]
    ),
    "AkBuddyBlockPool::CoalesceBlock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkBuddyBlockPool *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkBuddyBlockPool::MemBlock *
        ]
    ),
    "AkBuddyBlockPool::CoalesceMem": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkBuddyBlockPool *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkIOMemMgr::DestroyTempBlock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOMemMgr *
            ctypes.c_ulonglong,  # AK::StreamMgr::AkMemBlock *
            ctypes.c_ulonglong,  # AK::StreamMgr::AkMemBlock *
        ]
    ),
    "AkArray<AK::StreamMgr::AkMemBlock *,AK::StreamMgr::AkMemBlock *,AkArrayAllocatorNoAlign<7>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AK::StreamMgr::AkMemBlock *>>::Erase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AK::StreamMgr::AkMemBlock *,AK::StreamMgr::AkMemBlock *,AkArrayAllocatorNoAlign<7>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AK::StreamMgr::AkMemBlock *> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkIOMemMgr::FlushCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOMemMgr *
        ]
    ),
    "AK::StreamMgr::CAkIOMemMgr::GetCachedBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOMemMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # AK::StreamMgr::AkMemBlock **
        ]
    ),
    "AK::StreamMgr::CAkIOMemMgr::GetOldestFreeBlock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOMemMgr *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AK::StreamMgr::AkMemBlock **
        ]
    ),
    "AkArray<AK::StreamMgr::AkMemBlock *,AK::StreamMgr::AkMemBlock *,AkArrayAllocatorNoAlign<7>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AK::StreamMgr::AkMemBlock *>>::GrowArray": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # AkArray<AK::StreamMgr::AkMemBlock *,AK::StreamMgr::AkMemBlock *,AkArrayAllocatorNoAlign<7>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AK::StreamMgr::AkMemBlock *> > *
        ]
    ),
    "AkBuddyBlockPool::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkBuddyBlockPool *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "AK::StreamMgr::CAkIOMemMgr::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOMemMgr *
            ctypes.c_ulonglong,  # const AkDeviceSettings *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOThread *
        ]
    ),
    "AkBuddyBlockPool::JoinFreeBlock": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkBuddyBlockPool::MemBlock *
        argtypes=[
            ctypes.c_ulonglong,  # AkBuddyBlockPool *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkBuddyBlockPool::MemBlock *
        ]
    ),
    "AK::StreamMgr::CAkIOMemMgr::ReleaseBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOMemMgr *
            ctypes.c_ulonglong,  # AK::StreamMgr::AkMemBlock *
        ]
    ),
    "AkBuddyBlockPool::SubdivideMem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AkBuddyBlockPool::MemBlock *
        argtypes=[
            ctypes.c_ulonglong,  # AkBuddyBlockPool *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkIOMemMgr::TagBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOMemMgr *
            ctypes.c_ulonglong,  # AK::StreamMgr::AkMemBlock *
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkLowLevelTransfer *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "AK::StreamMgr::CAkIOMemMgr::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOMemMgr *
        ]
    ),
    "AK::StreamMgr::CAkIOMemMgr::UntagBlock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkIOMemMgr *
            ctypes.c_ulonglong,  # AK::StreamMgr::AkMemBlock *
        ]
    ),
    "AK::StreamMgr::CAkStmMemViewDeferred::Cancel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::StreamMgr::CAkStmMemViewDeferred *
            ctypes.c_ulonglong,  # AK::StreamMgr::IAkIOHookDeferredBatch *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "AK::StreamMgr::CAkLowLevelTransferDeferred::LLBatchIOCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # AkAsyncIOTransferInfo **
            ctypes.c_ulonglong,  # AKRESULT *
        ]
    ),
    "AK::StreamMgr::CAkLowLevelTransferDeferred::LLIOCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkAsyncIOTransferInfo *
            ctypes.c_uint32,  # AKRESULT
        ]
    ),
    "CreateAkTremoloFX": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkTremoloFX::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkTremoloFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkTremoloFX::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkTremoloFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkEffectPluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # AkAudioFormat *
        ]
    ),
    "DSP::MonoLFO<DSP::Unipolar,TremoloOutputPolicy>::ProduceBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::MonoLFO<DSP::Unipolar,TremoloOutputPolicy> *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # TremoloOutputPolicy *
        ]
    ),
    "CAkTremoloFXParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # CAkTremoloFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateAkTremoloFXParams": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CAkTremoloFXParams::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkTremoloFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkTremoloFXParams::SetDirty": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkTremoloFXParams *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkTremoloFXParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkTremoloFXParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkTremoloFXParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkTremoloFXParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CreateVorbisSrcPlugin": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "CreateVorbisFileCodec": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAkFileCodec *
        argtypes=[]
    ),
    "CreateVorbisGrainCodec": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "CAkVorbisGrainCodec::CAkVorbisGrainCodec": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVorbisGrainCodec *
        ]
    ),
    "CAkSrcBankVorbis::ChangeSourcePosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBankVorbis *
        ]
    ),
    "CAkSrcBankVorbis::GetBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBankVorbis *
            ctypes.c_ulonglong,  # AkVPLState *
        ]
    ),
    "CAkVorbisGrainCodec::GetBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVorbisGrainCodec *
            ctypes.c_ulonglong,  # AkAudioBuffer *
        ]
    ),
    "CAkVorbisGrainCodec::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVorbisGrainCodec *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkSrcBankVorbis::LoopInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBankVorbis *
        ]
    ),
    "CAkSrcBankVorbis::OnLoopComplete": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBankVorbis *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSrcBankVorbis::ReleaseBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBankVorbis *
        ]
    ),
    "CAkVorbisGrainCodec::ReleaseBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVorbisGrainCodec *
        ]
    ),
    "CAkSrcBankVorbis::RelocateMedia": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBankVorbis *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "CAkSrcBankVorbis::Seek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBankVorbis *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkVorbisGrainCodec::SeekTo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVorbisGrainCodec *
            ctypes.c_int64,  # __int64
        ]
    ),
    "CAkSrcBankVorbis::SeekToNativeOffset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBankVorbis *
        ]
    ),
    "CAkSrcBankVorbis::StartStream": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBankVorbis *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "CAkSrcBankVorbis::StopLooping": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBankVorbis *
        ]
    ),
    "CAkSrcBankVorbis::StopStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBankVorbis *
        ]
    ),
    "CAkVorbisGrainCodec::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkVorbisGrainCodec *
        ]
    ),
    "CAkSrcBankVorbis::VirtualOff": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBankVorbis *
            ctypes.c_uint32,  # AkVirtualQueueBehavior
            ctypes.c_ubyte,  # bool
        ]
    ),
    "CAkSrcBankVorbis::VirtualOn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBankVorbis *
            ctypes.c_uint32,  # AkVirtualQueueBehavior
        ]
    ),
    "CAkSrcBankVorbis::VirtualSeek": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBankVorbis *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CAkSrcBankVorbis::VorbisDSPRestart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcBankVorbis *
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkVorbisFileCodec::DecodeFile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkVorbisFileCodec *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "CreateVorbisCodec": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAkSrcMediaCodec *
        argtypes=[
            ctypes.c_ulonglong,  # const AK::SrcMedia::Header *
        ]
    ),
    "CAkSrcMediaCodecVorbis::FindClosestFileOffset": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecVorbis *
            ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # IAkSrcMediaCodec::SeekInfo *
        ]
    ),
    "CAkSrcMediaCodecVorbis::GetBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecVorbis *
            ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
            ctypes.c_ulonglong,  # AK::SrcMedia::Stream::State *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # IAkSrcMediaCodec::BufferInfo *
        ]
    ),
    "CAkSrcMediaCodecVorbis::GetNextPacket": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecVorbis *
            ctypes.c_ulonglong,  # AK::SrcMedia::Stream::State *
        ]
    ),
    "CAkSrcMediaCodecVorbis::Init": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecVorbis *
            ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
            ctypes.c_ulonglong,  # const AK::SrcMedia::Header *
            ctypes.c_ulonglong,  # AK::SrcMedia::CodecInfo *
            ctypes.c_ulonglong,  # const IAkSrcMediaCodec::InitialCodecState *
        ]
    ),
    "CAkSrcMediaCodecVorbis::Seek": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecVorbis *
            ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
            ctypes.c_ulonglong,  # AK::SrcMedia::Stream::State *
            ctypes.c_ulonglong,  # const IAkSrcMediaCodec::SeekInfo *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "CAkSrcMediaCodecVorbis::StopLooping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecVorbis *
            ctypes.c_ulonglong,  # const AK::SrcMedia::Position::State *
        ]
    ),
    "CAkSrcMediaCodecVorbis::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecVorbis *
        ]
    ),
    "CAkSrcMediaCodecVorbis::VirtualOff": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecVorbis *
            ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
        ]
    ),
    "CAkSrcMediaCodecVorbis::VirtualOn": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecVorbis *
        ]
    ),
    "CAkSrcMediaCodecVorbis::Warmup": FUNCDEF(
        restype=ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
        argtypes=[
            ctypes.c_ulonglong,  # CAkSrcMediaCodecVorbis *
            ctypes.c_ulonglong,  # IAkSrcMediaCodec::Result *
            ctypes.c_ulonglong,  # AK::SrcMedia::Stream::State *
        ]
    ),
    "vorbis_dsp_clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # vorbis_dsp_state *
        ]
    ),
    "vorbis_dsp_init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # vorbis_dsp_state *
            ctypes.c_int32,  # int
        ]
    ),
    "vorbis_dsp_pcmout": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # vorbis_dsp_state *
            ctypes.c_ulonglong,  # float *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "vorbis_dsp_restart": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # vorbis_dsp_state *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "vorbis_dsp_synthesis": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # vorbis_dsp_state *
            ctypes.c_ulonglong,  # ogg_packet *
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
        ]
    ),
    "vorbis_shift_dct": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # vorbis_dsp_state *
        ]
    ),
    "DecodeVorbis": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkTremorInfo *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # float **
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "DecodeVorbisInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkTremorInfo *
            ctypes.c_uint16,  # unsigned __int16
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # float **
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "AkVorbisCodebookMgr::Decodebook": FUNCDEF(
        restype=ctypes.c_ulonglong,  # codec_setup_info *
        argtypes=[
            ctypes.c_ulonglong,  # AkVorbisCodebookMgr *
            ctypes.c_ulonglong,  # AkVorbisSourceState *
            ctypes.c_ulonglong,  # ogg_packet *
            ctypes.c_ulonglong,  # AK::Monitor::ErrorCode *
        ]
    ),
    "AkVorbisCodebookMgr::ReleaseCodebook": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AkVorbisCodebookMgr *
            ctypes.c_ulonglong,  # AkVorbisSourceState *
        ]
    ),
    "AkHashListBare<uint,Codebook,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<uint,Codebook>>::Resize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # AkHashListBare<unsigned int,Codebook,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<unsigned int,Codebook> > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "Codebook::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # Codebook *
        ]
    ),
    "ilog": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "mapping_info_unpack": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # vorbis_info_mapping *
            ctypes.c_ulonglong,  # codec_setup_info *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_ulonglong,  # CAkVorbisAllocator *
        ]
    ),
    "mapping_inverse": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # vorbis_dsp_state *
            ctypes.c_ulonglong,  # vorbis_info_mapping *
        ]
    ),
    "floor1_inverse2_template<floor_V4F32>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # vorbis_dsp_state *
            ctypes.c_ulonglong,  # vorbis_info_floor1 *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "InitFloorFunc": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "floor1_info_unpack": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # vorbis_info_floor1 *
            ctypes.c_ulonglong,  # codec_setup_info *
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_ulonglong,  # CAkVorbisAllocator *
        ]
    ),
    "floor1_inverse1": FUNCDEF(
        restype=ctypes.c_ulonglong,  # int *
        argtypes=[
            ctypes.c_ulonglong,  # vorbis_dsp_state *
            ctypes.c_ulonglong,  # vorbis_info_floor1 *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "mergesort": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # unsigned __int16 *
            ctypes.c_int64,  # __int64
        ]
    ),
    "mdct_backward": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_int32,  # int
        ]
    ),
    "mdct_first_2stages_SIMD_Ex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "mdct_presymmetry_SIMD_Ex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_int32,  # const int
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # float *
            ctypes.c_int32,  # const int
        ]
    ),
    "mdct_reverse_step7_step8": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "mdct_unroll_lap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "vorbis_info_init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # codec_setup_info *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "vorbis_unpack_books": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # Codebook *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # oggpack_buffer *
        ]
    ),
    "res_inverse": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # vorbis_dsp_state *
            ctypes.c_ulonglong,  # vorbis_info_residue *
            ctypes.c_ulonglong,  # int **
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "res_unpack": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # vorbis_info_residue *
            ctypes.c_ulonglong,  # codec_setup_info *
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_ulonglong,  # CAkVorbisAllocator *
        ]
    ),
    "vorbis_book_decodev_add_t<ExpandRecursive<MonoPolicy,1>,LeafByte>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_t<ExpandRecursive<MonoPolicy,2>,LeafByte>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_t<ExpandRecursive<MonoPolicy,4>,LeafByte>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_t<ExpandRecursive<MonoPolicy,8>,LeafByte>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_t<ExpandEntry1Stereo,LeafByte>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_t<ExpandRecursive<StereoPolicy,2>,LeafByte>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_t<ExpandRecursive<StereoPolicy,4>,LeafByte>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_t<ExpandRecursive<StereoPolicy,8>,LeafByte>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_t<ExpandRecursive<MonoPolicy,1>,LeafWord>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_t<ExpandRecursive<MonoPolicy,2>,LeafWord>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_t<ExpandRecursive<MonoPolicy,4>,LeafWord>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_t<ExpandRecursive<MonoPolicy,8>,LeafWord>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_t<ExpandEntry1Stereo,LeafWord>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_t<ExpandRecursive<StereoPolicy,2>,LeafWord>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_t<ExpandRecursive<StereoPolicy,4>,LeafWord>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_t<ExpandRecursive<StereoPolicy,8>,LeafWord>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_SIMD_2<LeafByte>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_SIMD_4<LeafByte>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_SIMD_4_2ch<LeafByte>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_SIMD_8_2ch<LeafByte>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_SIMD_2<LeafWord>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_SIMD_4<LeafWord>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_SIMD_4_2ch<LeafWord>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "vorbis_book_decodev_add_SIMD_8_2ch<LeafWord>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # int **
            ctypes.c_int32,  # const int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_int32,  # const int
        ]
    ),
    "BuildTree": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_ulonglong,  # CodebookBuildParam *
            ctypes.c_ulonglong,  # CAkVorbisAllocator *
            ctypes.c_int32,  # int
        ]
    ),
    "GrowableAlloc::Grow": FUNCDEF(
        restype=ctypes.c_ulonglong,  # GrowableAlloc *
        argtypes=[
            ctypes.c_ulonglong,  # GrowableAlloc *
            ctypes.c_int32,  # int
        ]
    ),
    "ak_vorbis_book_decode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # oggpack_buffer *
        ]
    ),
    "decpack": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_ulonglong,  # CodebookBuildParam *
        ]
    ),
    "pack_sub_tree": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # CAkVorbisAllocator *
        ]
    ),
    "vorbis_book_unpack": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # oggpack_buffer *
            ctypes.c_ulonglong,  # codebook *
            ctypes.c_ulonglong,  # CAkVorbisAllocator *
            ctypes.c_int32,  # int
        ]
    ),
    "floor1_inverse2_template<floor_AVX>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # vorbis_dsp_state *
            ctypes.c_ulonglong,  # vorbis_info_floor1 *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "floor1_inverse2_AVX": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # vorbis_dsp_state *
            ctypes.c_ulonglong,  # vorbis_info_floor1 *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "floor1_inverse2_template<floor_AVX2>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # vorbis_dsp_state *
            ctypes.c_ulonglong,  # vorbis_info_floor1 *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "floor1_inverse2_AVX2": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # vorbis_dsp_state *
            ctypes.c_ulonglong,  # vorbis_info_floor1 *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdSimdTree::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hkcdPlanarCsgOperand::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarCsgOperand *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkcdPlanarCsgOperand::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hkcdPlanarEntity::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarEntity *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkcdPlanarGeometry::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hkcdPlanarSolid::NodeStorage::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarSolid::NodeStorage *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkcdPlanarGeometryPlanesCollection::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hkcdPlanarGeometryPrimitives::Plane::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hkcdPlanarGeometryPrimitives::Collection<28>::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarGeometryPrimitives::Collection<28> *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkArrayBase<hkcdPlanarSolid::Node>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkcdPlanarSolid::Node> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkcdSimdTree::Dynamic::NodeData>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkcdSimdTree::Dynamic::NodeData> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkcdPlanarSolid::Node>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkcdPlanarSolid::Node> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkAabb>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkAabb> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkcdDynamicTree::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::CodecRaw<uint>>::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::CodecRaw<uint>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::CodecRaw<unsigned int> > *
            ctypes.c_ulonglong,  # const hkcdDynamicTree::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::CodecRaw<unsigned int> > *
        ]
    ),
    "hkcdDynamicTree::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::CodecRaw<unsigned __int64>>::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::CodecRaw<unsigned __int64>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::CodecRaw<unsigned __int64> > *
            ctypes.c_ulonglong,  # const hkcdDynamicTree::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::CodecRaw<unsigned __int64> > *
        ]
    ),
    "hkcdDynamicTree::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::CodecInt16>::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::CodecInt16>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::CodecInt16> *
            ctypes.c_ulonglong,  # const hkcdDynamicTree::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::CodecInt16> *
        ]
    ),
    "hkcdPlanarGeometry::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarGeometry *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkcdPlanarGeometryPlanesCollection::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarGeometryPlanesCollection *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkcdPlanarGeometryPolygonCollection::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarGeometryPolygonCollection *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkcdPlanarSolid::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarSolid *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkcdTriangleUtil::checkForFlatConvexQuad": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # hkcdTriangleUtil::CheckFlatConvexQuadMode
        ]
    ),
    "hkcdTriangleUtil::isDegenerate": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkcdObb::computeDistance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdObb *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkcdObb::DistanceResult *
        ]
    ),
    "hkcdObb::getWorldAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdObb *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hkcdObb::set": {
        "hkcdObb *, const hkStridedVertices *": FUNCDEF(
            restype=ctypes.c_double,  # double
            argtypes=[
                ctypes.c_ulonglong,  # hkcdObb *
                ctypes.c_ulonglong,  # const hkStridedVertices *
            ]
        ),
        "hkcdObb *, const hkgpConvexHull *": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # hkcdObb *
                ctypes.c_ulonglong,  # const hkgpConvexHull *
            ]
        ),
        "hkcdObb *, const hkVector4f *, int": FUNCDEF(
            restype=ctypes.c_float,  # float
            argtypes=[
                ctypes.c_ulonglong,  # hkcdObb *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_int32,  # int
            ]
        ),
        "hkcdObb *, const hkRotationImpl<float> *, const hkVector4f *, const hkVector4f *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkcdObb *
                ctypes.c_ulonglong,  # const hkRotationImpl<float> *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_ulonglong,  # const hkVector4f *
            ]
        ),
        "hkcdObb *, const hkTransformf *, const hkVector4f *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkcdObb *
                ctypes.c_ulonglong,  # const hkTransformf *
                ctypes.c_ulonglong,  # const hkVector4f *
            ]
        ),
    },
    "hkcdSimdTreeUtils::aabbOverlaps_Trampoline<0,hkcdSimdTreeUtils::LeafCollectorQuery>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdSimdTree::TransposedAabb *
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::LeafCollectorQuery *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hk::swap<hkcdSimdTree::Node>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree::Node *
            ctypes.c_ulonglong,  # hkcdSimdTree::Node *
        ]
    ),
    "hkcdSimdTreeUtils::treeOverlaps<hkVectorSort::GenericSimd,hkcdSimdTreeUtils::NullQueryMaskFilter>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # hkcdSimdTree::PairCollector *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTreeUtils::treeOverlaps<hkVectorSort::GenericSimd,hkcdSimdTreeUtils::QueryMaskFilter>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # hkcdSimdTree::PairCollector *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const hkcdSimdTreeUtils::QueryMaskFilter *
        ]
    ),
    "hkcdSimdTreeUtils::treeOverlaps<hkVectorSort::SSE31,hkcdSimdTreeUtils::NullQueryMaskFilter>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # hkcdSimdTree::PairCollector *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTreeUtils::treeOverlaps<hkVectorSort::SSE31,hkcdSimdTreeUtils::QueryMaskFilter>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # hkcdSimdTree::PairCollector *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const hkcdSimdTreeUtils::QueryMaskFilter *
        ]
    ),
    "hkcdSimdTree::aabbOverlapsNearMiss_OutOfLine": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # hkcdSimdTree::LeafCollector *
            ctypes.c_ulonglong,  # hkAabb *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::aabbOverlaps_OutOfLine": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # hkcdSimdTree::LeafCollector *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::insert": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "rebuildInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
        ]
    ),
    "recurseRefitLeaf": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # hkInplaceArray<unsigned int,32,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkcdSimdTree::Node *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkAabb *
        ]
    ),
    "hkcdSimdTree::refitLeaf": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkAabb *
        ]
    ),
    "hkcdSimdTree::sortByAabbsSize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
        ]
    ),
    "hkcdSimdTree::translate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkcdSimdTree::treeOverlaps": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # hkcdSimdTree::PairCollector *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkArrayBase<hkReflect::Detail::VtableInitializer::PreCalc>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkXmlStreamParser::SubString> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkcdSimdTree::RangeSizeAndIndex>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkSerialize::Detail::TagfileItem> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkcdSimdTree::Point>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkcdSimdTree::Point> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkcdSimdTreeUtils::Build::IRefit::_refitBranchDepthFirstMemoryLayout<hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbs>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::IRefit *
            ctypes.c_ulonglong,  # const hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbs *
            ctypes.c_ulonglong,  # hkcdSimdTree::Node *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdSimdTreeUtils::Build::IRefit::_refitBranchDepthFirstMemoryLayout<hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbsFloat3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::IRefit *
            ctypes.c_ulonglong,  # const hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbsFloat3 *
            ctypes.c_ulonglong,  # hkcdSimdTree::Node *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdSimdTreeUtils::Build::IRefit::_refitBranchGeneric<hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbs>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::IRefit *
            ctypes.c_ulonglong,  # const hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbs *
            ctypes.c_ulonglong,  # hkcdSimdTree::Node *
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdSimdTreeUtils::Build::IRefit::_refitBranchGeneric<hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbsFloat3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::IRefit *
            ctypes.c_ulonglong,  # const hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbsFloat3 *
            ctypes.c_ulonglong,  # hkcdSimdTree::Node *
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdSimdTreeUtils::Build::hkcdBuildFirstNLevels<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper>::hkcdBuildFirstNLevels<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::hkcdBuildFirstNLevels<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper> *
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # hkcdSimdTree::BuildContext *
            ctypes.c_ulonglong,  # hkcdSimdTree::TaskBuildContext *
            ctypes.c_ulonglong,  # const hkRefPtr<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper> *
            ctypes.c_ulonglong,  # hkArray<hkcdSimdTree::Point,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkcdSimdTreeUtils::Build::hkcdBuildRangesTask<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper>::hkcdBuildRangesTask<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::hkcdBuildRangesTask<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper> *
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # hkcdSimdTree::BuildContext *
            ctypes.c_ulonglong,  # hkcdSimdTree::TaskBuildContext *
            ctypes.c_ulonglong,  # const hkRefPtr<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper> *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkcdSimdTreeUtils::Build::hkcdFinalizeTreeTask<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper>::hkcdFinalizeTreeTask<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::hkcdFinalizeTreeTask<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper> *
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # hkcdSimdTree::BuildContext *
            ctypes.c_ulonglong,  # hkcdSimdTree::TaskBuildContext *
            ctypes.c_ulonglong,  # const hkRefPtr<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper> *
            ctypes.c_ulonglong,  # hkArray<hkcdSimdTree::Point,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkcdSimdTree::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
        ]
    ),
    "hkcdSimdTreeUtils::Build::build": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree::BuildContext *
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::IRefit *
        ]
    ),
    "hkcdSimdTree::buildFromAabbs": {
        "hkcdSimdTree *, hkcdSimdTree::BuildContext *, const hkAabbFloat3 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkcdSimdTree *
                ctypes.c_ulonglong,  # hkcdSimdTree::BuildContext *
                ctypes.c_ulonglong,  # const hkAabbFloat3 *
            ]
        ),
        "hkcdSimdTree *, hkcdSimdTree::BuildContext *, const hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkcdSimdTree *
                ctypes.c_ulonglong,  # hkcdSimdTree::BuildContext *
                ctypes.c_ulonglong,  # const hkAabb *
            ]
        ),
    },
    "hkcdSimdTreeUtils::Build::buildHierarchy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree::BuildContext *
            ctypes.c_ulonglong,  # const hkcdSimdTree::Range *
            ctypes.c_ulonglong,  # hkcdSimdTree::Node *
            ctypes.c_ulonglong,  # hkcdSimdTree::Node **
        ]
    ),
    "hkcdSimdTree::buildQueryMask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const bool *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdSimdTree::BuildContext::copyFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree::BuildContext *
            ctypes.c_ulonglong,  # const hkcdSimdTree::BuildContext *
        ]
    ),
    "hkcdSimdTreeUtils::Build::RefitT<hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbs>::getLeafAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::RefitT<hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbs> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hkcdSimdTreeUtils::Build::RefitT<hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbsFloat3>::getLeafAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::RefitT<hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbsFloat3> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hkcdSimdTreeUtils::Build::hkcdBuildFirstNLevels<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper>::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::hkcdBuildFirstNLevels<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper> *
        ]
    ),
    "hkcdSimdTreeUtils::Build::hkcdBuildRangesTask<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper>::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::hkcdBuildRangesTask<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper> *
        ]
    ),
    "hkcdSimdTreeUtils::Build::hkcdFinalizeTreeTask<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper>::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::hkcdFinalizeTreeTask<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper> *
        ]
    ),
    "hkcdSimdTreeUtils::Build::partitionSAH1": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree::BuildContext *
            ctypes.c_ulonglong,  # const hkcdSimdTree::Range *
            ctypes.c_ulonglong,  # hkcdSimdTree::Range *
            ctypes.c_ulonglong,  # hkcdSimdTree::Range *
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdSimdTreeUtils::Build::partitionSAH3": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree::BuildContext *
            ctypes.c_ulonglong,  # const hkcdSimdTree::Range *
            ctypes.c_ulonglong,  # hkcdSimdTree::Range *
            ctypes.c_ulonglong,  # hkcdSimdTree::Range *
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdSimdTreeUtils::Build::partitionSah3FixedSplit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree::BuildContext *
            ctypes.c_ulonglong,  # const hkcdSimdTree::Range *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkcdSimdTree::Range *
            ctypes.c_ulonglong,  # hkcdSimdTree::Range *
        ]
    ),
    "hkcdSimdTreeUtils::Build::hkcdBuildFirstNLevels<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper>::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::hkcdBuildFirstNLevels<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper> *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hkcdSimdTreeUtils::Build::hkcdBuildRangesTask<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper>::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::hkcdBuildRangesTask<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper> *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hkcdSimdTreeUtils::Build::hkcdFinalizeTreeTask<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper>::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::hkcdFinalizeTreeTask<hkcdSimdTreeUtils::Build::hkcdRefitWithAabbFloat3Wrapper> *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hkcdSimdTreeUtils::Build::RefitT<hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbs>::refitBranch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::RefitT<hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbs> *
            ctypes.c_ulonglong,  # hkcdSimdTree::Node *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkcdSimdTreeUtils::Build::RefitT<hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbsFloat3>::refitBranch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::RefitT<hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbsFloat3> *
            ctypes.c_ulonglong,  # hkcdSimdTree::Node *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkcdSimdTreeUtils::Build::RefitT<hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbs>::refitNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::RefitT<hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbs> *
            ctypes.c_ulonglong,  # const hkcdSimdTree::Node *
            ctypes.c_ulonglong,  # hkcdSimdTree::Node *
        ]
    ),
    "hkcdSimdTreeUtils::Build::RefitT<hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbsFloat3>::refitNode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::RefitT<hkcdSimdTreeUtils::Build::GetLeafAabbFromAabbsFloat3> *
            ctypes.c_ulonglong,  # const hkcdSimdTree::Node *
            ctypes.c_ulonglong,  # hkcdSimdTree::Node *
        ]
    ),
    "hkcdSimdTreeUtils::Build::Task::run": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTreeUtils::Build::Task *
        ]
    ),
    "hkcdSimdTree::BuildContext::splitRangeInFour": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree::BuildContext *
            ctypes.c_ulonglong,  # const hkcdSimdTree::Range *
            ctypes.c_ulonglong,  # hkcdSimdTree::Range *
        ]
    ),
    "hkcdSimdTreeUtils::Build::splitSmallRangeSah3": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree::BuildContext *
            ctypes.c_ulonglong,  # const hkcdSimdTree::Range *
        ]
    ),
    "hkSimplexSolverExamineActivePlanes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSimplexSolverInfo *
        ]
    ),
    "hkSimplexSolverSolve1d": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkSimplexSolverInfo *
            ctypes.c_ulonglong,  # const hkSurfaceConstraintInfo *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkSimplexSolverSolve2d": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSimplexSolverInfo *
            ctypes.c_ulonglong,  # const hkSurfaceConstraintInfo *
            ctypes.c_ulonglong,  # const hkSurfaceConstraintInfo *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkSimplexSolverSolve3d": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSimplexSolverInfo *
            ctypes.c_ulonglong,  # const hkSurfaceConstraintInfo *
            ctypes.c_ulonglong,  # const hkSurfaceConstraintInfo *
            ctypes.c_ulonglong,  # const hkSurfaceConstraintInfo *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkSimplexSolverSortInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSimplexSolverInfo *
        ]
    ),
    "hkSimplexSolverSolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkSimplexSolverInput *
            ctypes.c_ulonglong,  # hkSimplexSolverOutput *
        ]
    ),
    "hkcdDistanceSegmentQuad": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSimdFloat32 *
        argtypes=[
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkcdGsk::GetClosestPointOutput *
        ]
    ),
    "hkArrayBase<hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64>>>::SAHTask>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64> > >::SAHTask> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64>>>::SAHBin>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64> > >::SAHBin> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64>>>::copyTo<hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64> > > *
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::BuildStepUtils::DynamicTree *
        ]
    ),
    "hkgpDeprecatedConvexDecomposition::Config::Config": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpDeprecatedConvexDecomposition::Config *
        ]
    ),
    "hkgpDeprecatedConvexDecomposition::GuardGenConfig::GuardGenConfig": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpDeprecatedConvexDecomposition::GuardGenConfig *
        ]
    ),
    "hkgpDeprecatedConvexDecomposition::MeshPreProcessorConfig::MeshPreProcessorConfig": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpDeprecatedConvexDecomposition::MeshPreProcessorConfig *
        ]
    ),
    "hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64>>>::SAHTask::~SAHTask": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64> > >::SAHTask *
        ]
    ),
    "hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64>>>::compactIndices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64> > > *
        ]
    ),
    "hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64>>>::computeCost": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSimdFloat32 *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64> > > *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64>>>::rebuildBranchSAH": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64> > > *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # hknpStreamContactJacobian *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdDynamicTree::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::CodecRaw<unsigned __int64>>::reserveNodes": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::DynamicStorage<0,hkcdDynamicTree::AnisotropicMetric,hkcdDynamicTree::CodecRaw<unsigned __int64> > *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_int32,  # int
        ]
    ),
    "hkInt64Vector4::countLeadingZeros<2>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkInt64Vector4 *
            ctypes.c_ulonglong,  # hkIntVector *
        ]
    ),
    "hkcdPlanarGeometryPrimitives::Plane::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarGeometryPrimitives::Plane *
        ]
    ),
    "hkcdPlanarGeometryPolygonCollection::hkcdPlanarGeometryPolygonCollection": {
        "hkcdPlanarGeometryPolygonCollection *, const hkcdPlanarGeometryPolygonCollection *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkcdPlanarGeometryPolygonCollection *
                ctypes.c_ulonglong,  # const hkcdPlanarGeometryPolygonCollection *
            ]
        ),
        "hkcdPlanarGeometryPolygonCollection *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkcdPlanarGeometryPolygonCollection *
            ]
        ),
    },
    "hkcdPlanarGeometryPrimitives::Collection<28>::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarGeometryPrimitives::Collection<28> *
        ]
    ),
    "hkcdPlanarGeometryPrimitives::Collection<28>::copy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarGeometryPrimitives::Collection<28> *
            ctypes.c_ulonglong,  # const hkcdPlanarGeometryPrimitives::Collection<28> *
        ]
    ),
    "hkArrayBase<hkcdPlanarGeometryPrimitives::Plane>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkcdPlanarGeometryPrimitives::Plane> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkInt64Vector4::countLeadingZeros<3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkInt64Vector4 *
            ctypes.c_ulonglong,  # hkIntVector *
        ]
    ),
    "hkMath::greatestCommonDivisor<unsigned __int64>": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkcdPlanarGeometryPlanesCollection::hkcdPlanarGeometryPlanesCollection": {
        "hkcdPlanarGeometryPlanesCollection *, const hkcdPlanarGeometryPlanesCollection *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkcdPlanarGeometryPlanesCollection *
                ctypes.c_ulonglong,  # const hkcdPlanarGeometryPlanesCollection *
            ]
        ),
        "hkcdPlanarGeometryPlanesCollection *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkcdPlanarGeometryPlanesCollection *
            ]
        ),
    },
    "hkcdPlanarGeometryPlanesCollection::~hkcdPlanarGeometryPlanesCollection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarGeometryPlanesCollection *
        ]
    ),
    "hkcdPlanarGeometryPlanesCollection::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarGeometryPlanesCollection *
        ]
    ),
    "hkcdPlanarGeometryPlanesCollection::createBoundaryPlanes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarGeometryPlanesCollection *
        ]
    ),
    "hkArrayBase<hkVector4d>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkVector4d> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkVector4d>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkVector4d> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkcdPlanarGeometry::hkcdPlanarGeometry": {
        "hkcdPlanarGeometry *, const hkcdPlanarGeometry *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkcdPlanarGeometry *
                ctypes.c_ulonglong,  # const hkcdPlanarGeometry *
            ]
        ),
        "hkcdPlanarGeometry *, hkcdPlanarGeometryPlanesCollection *, int, hkcdPlanarEntityDebugger *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkcdPlanarGeometry *
                ctypes.c_ulonglong,  # hkcdPlanarGeometryPlanesCollection *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # hkcdPlanarEntityDebugger *
            ]
        ),
    },
    "hkFindUniquePositionsUtil::~hkFindUniquePositionsUtil": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkFindUniquePositionsUtil *
        ]
    ),
    "hkcdPlanarGeometry::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarGeometry *
        ]
    ),
    "hkcdPlanarGeometryPrimitives::Collection<28>::create": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarGeometryPrimitives::Collection<28> *
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdPlanarGeometry::extractGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarGeometry *
            ctypes.c_ulonglong,  # hkGeometry *
        ]
    ),
    "hkcdPlanarGeometry::VertexStorage::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarGeometry::VertexStorage *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkcdPlanarGeometry::recomputeVerticesCache": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarGeometry *
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdPlanarSolid::NodeStorage::NodeStorage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarSolid::NodeStorage *
            ctypes.c_ulonglong,  # const hkcdPlanarSolid::NodeStorage *
        ]
    ),
    "hkcdPlanarSolid::hkcdPlanarSolid": {
        "hkcdPlanarSolid *, const hkcdPlanarSolid *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkcdPlanarSolid *
                ctypes.c_ulonglong,  # const hkcdPlanarSolid *
            ]
        ),
        "hkcdPlanarSolid *, const hkcdPlanarGeometryPlanesCollection *, int, hkcdPlanarEntityDebugger *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkcdPlanarSolid *
                ctypes.c_ulonglong,  # const hkcdPlanarGeometryPlanesCollection *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # hkcdPlanarEntityDebugger *
            ]
        ),
    },
    "hkcdPlanarSolid::~hkcdPlanarSolid": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarSolid *
        ]
    ),
    "hkcdPlanarCsgOperand::hkcdPlanarCsgOperand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarCsgOperand *
        ]
    ),
    "hkcdPlanarCsgOperand::~hkcdPlanarCsgOperand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarCsgOperand *
        ]
    ),
    "hkcdPlanarCsgOperand::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdPlanarCsgOperand *
        ]
    ),
    "hkcdPlanarGeometryPredicates::computeIntersectionDeterminants": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4d *
            ctypes.c_ulonglong,  # const hkVector4d *
            ctypes.c_ulonglong,  # const hkVector4d *
            ctypes.c_ulonglong,  # hkcdMathErrorBoundsCalculator::Vector *
        ]
    ),
    "hkcdGsk::getClosestPoint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkcdGsk::GetClosestPointInput *
            ctypes.c_ulonglong,  # hkcdGsk::Cache *
            ctypes.c_ulonglong,  # hkcdGsk::GetClosestPointOutput *
        ]
    ),
    "hkcdGsk_Float3ShapeAOnlyInterface::getShapeAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdGsk_Float3ShapeInterface *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hkcdGsk_Float3ShapeAOnlyInterface::getSupportingVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdGsk_Float3ShapeAOnlyInterface *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkcdGsk::GetSupportingVertexOut *
        ]
    ),
    "hkcdGsk_Float3ShapeInterface::getSupportingVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdGsk_Float3ShapeInterface *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkcdGsk::GetSupportingVertexOut *
        ]
    ),
    "hkcdGsk::linearCast": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkcdGsk::LinearCastInput *
            ctypes.c_ulonglong,  # hkcdGsk::LinearCastOutput *
            ctypes.c_ulonglong,  # hkcdGsk::LinearCastOutput *
            ctypes.c_ulonglong,  # hkcdGsk::Cache *
        ]
    ),
    "hkcdGsk::rayCast": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkcdGsk::RayCastInput *
            ctypes.c_ulonglong,  # hkcdGsk::RayCastOutput *
        ]
    ),
    "hkcdGsk::sphereCast": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkcdGsk::LinearCastInput *
            ctypes.c_ulonglong,  # hkcdGsk::LinearCastOutput *
        ]
    ),
    "sphereCastOutOfLine": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdGsk_Float3Shape *
            ctypes.c_ulonglong,  # const hkcdGsk_Float3Shape *
            ctypes.c_ulonglong,  # const hkcdGsk::LinearCastInput *
            ctypes.c_ulonglong,  # hkcdGsk::LinearCastOutput *
        ]
    ),
    "hkcdTreeFilterUtils::computeValidBranchesMask<hkcdStaticMeshTree::Base>": {
        "const hkcdStaticMeshTree::Base *, const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb5BytesCodec>::NodeContext *, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Base *
                ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb5BytesCodec>::NodeContext *
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
        "const hkcdStaticMeshTree::Base *, unsigned int *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Base *
                ctypes.c_ulonglong,  # unsigned int *
            ]
        ),
    },
    "hkcdStaticMeshTree::Base::commitValidKeyMasks": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::Base *
            ctypes.c_ulonglong,  # hkRelArray<unsigned int> *
        ]
    ),
    "hkcdStaticMeshTree::Base::initializeValidKeyMasks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::Base *
            ctypes.c_ulonglong,  # hkRelArray<unsigned int> *
        ]
    ),
    "hkcdStaticMeshTree::Connectivity::link": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkcdStaticMeshTree::Edge *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::Connectivity *
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::Edge *
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Edge *
        ]
    ),
    "hkcdStaticMeshTree::Connectivity::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::Connectivity *
        ]
    ),
    "hkcdGskBase::handlePenetration": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkcdGskBase *
            ctypes.c_ulonglong,  # const hkcdGskBase::ShapeInterface *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # hkcdGsk::GetClosestPointOutput *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkcdConvexPenetrationUtilExact::calculatePenetrationDepth": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkcdConvexPenetrationUtilExact *
            ctypes.c_ulonglong,  # hkcdGsk::GetClosestPointOutput *
        ]
    ),
    "hkcdConvexPenetrationUtilExact::computeExactFaceNormal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdConvexPenetrationUtilExact *
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdConvexPenetrationUtilExact::computeFaceBarycentricAndTrilinear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdConvexPenetrationUtilExact *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkcdConvexPenetrationUtilExact::computeFaceNormal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdConvexPenetrationUtilExact *
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdConvexPenetrationUtilExact::computeFaceVisibilities": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkcdConvexPenetrationUtilExact *
        ]
    ),
    "hkcdConvexPenetrationUtilExact::constructOutput": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkcdConvexPenetrationUtilExact *
            ctypes.c_ulonglong,  # hkcdGsk::GetClosestPointOutput *
        ]
    ),
    "hkcdConvexPenetrationUtilExact::createInitialFaces": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdConvexPenetrationUtilExact *
        ]
    ),
    "hkcdConvexPenetrationUtilExact::defineOffset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdConvexPenetrationUtilExact *
        ]
    ),
    "hkcdConvexPenetrationUtilExact::findResultingFace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdConvexPenetrationUtilExact *
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdConvexPenetrationUtilExact::replaceVisibleFaces": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdConvexPenetrationUtilExact *
        ]
    ),
    "hkcdConvexPenetrationUtilExact::setupInitialStructure": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # hkcdConvexPenetrationUtilExact *
        ]
    ),
    "hkcdConvexPenetrationUtilExact::setupVisibleBoundaryEdgeChainSingleFace": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdConvexPenetrationUtilExact *
        ]
    ),
    "hkcdConvexPenetrationUtilExact::tryAddSupportingVertex": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkcdConvexPenetrationUtilExact *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkMemoryMeshTexture::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshTexture *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkMemoryMeshVertexBuffer::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hkSkinBinding::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkSkinBinding *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkStorageSkinnedMeshShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkStorageSkinnedMeshShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkDefaultCompoundMeshShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultCompoundMeshShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkSkinnedRefMeshShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkSkinnedRefMeshShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkDefaultCompoundMeshBody::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultCompoundMeshBody *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkDefaultCompoundMeshBody::getNumIndexedTransforms": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkIndexedTransformSet *
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultCompoundMeshBody *
        ]
    ),
    "hkDefaultCompoundMeshBody::getIndexedTransforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultCompoundMeshBody *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkMatrix4Impl<float> *
            ctypes.c_int32,  # int
        ]
    ),
    "hkDefaultCompoundMeshBody::getIndexedInverseTransforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultCompoundMeshBody *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkMatrix4Impl<float> *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMemoryMeshBody::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshBody *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkMemoryMeshBody::getVertexBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkMeshVertexBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshBody *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMemoryMeshBody::getNumIndexedTransforms": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkIndexedTransformSet *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshBody *
        ]
    ),
    "hkMemoryMeshBody::setIndexedTransforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshBody *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkMatrix4Impl<float> *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMemoryMeshBody::getIndexedTransforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshBody *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkMatrix4Impl<float> *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMemoryMeshBody::getIndexedInverseTransforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshBody *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkMatrix4Impl<float> *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMemoryMeshBody::getIndexTransformsOrder": FUNCDEF(
        restype=ctypes.c_ulonglong,  # __int16 *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshBody *
        ]
    ),
    "hkMemoryMeshBody::getIndexTransformNames": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkStringPtr *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshBody *
        ]
    ),
    "hkMemoryMeshBody::getIndexMappings": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkMeshBoneIndexMapping *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshBody *
        ]
    ),
    "hkMemoryMeshBody::getNumIndexMappings": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkIndexedTransformSet *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshBody *
        ]
    ),
    "hkMemoryMeshBody::setName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshBody *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkMemoryMeshMaterial::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkMemoryMeshMaterial::getTesselationFactor": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
        ]
    ),
    "hkMemoryMeshMaterial::setTesselationFactor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
            ctypes.c_float,  # float
        ]
    ),
    "hkMemoryMeshMaterial::getDisplacementAmount": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
        ]
    ),
    "hkMemoryMeshMaterial::setDisplacementAmount": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
            ctypes.c_float,  # float
        ]
    ),
    "hkMemoryMeshShape::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshShape *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkMemoryMeshShape::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshShape *
        ]
    ),
    "hkMemoryMeshShape::setName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshShape *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkArrayBase<short>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<short> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMeshBoneIndexMapping::hkMeshBoneIndexMapping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMeshBoneIndexMapping *
            ctypes.c_ulonglong,  # const hkMeshBoneIndexMapping *
        ]
    ),
    "hkStorageSkinnedMeshShape::~hkStorageSkinnedMeshShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkStorageSkinnedMeshShape *
        ]
    ),
    "hkIndexedTransformSet::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkIndexedTransformSet *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkMemoryMeshVertexBuffer::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshVertexBuffer *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkMeshVertexBuffer::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkMeshVertexBuffer *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkMultipleVertexBuffer::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkMultipleVertexBuffer *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkaSkeleton::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkaSkeleton *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkMultipleVertexBuffer::getNumVertices": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMultipleVertexBuffer *
        ]
    ),
    "hkMultipleVertexBuffer::getVertexFormat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshVertexBuffer *
            ctypes.c_ulonglong,  # hkVertexFormat *
        ]
    ),
    "hkMultipleVertexBuffer::isSharable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkMultipleVertexBuffer *
        ]
    ),
    "hkCompressedMassProperties::pack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkCompressedMassProperties *
            ctypes.c_ulonglong,  # const hkDiagonalizedMassProperties *
        ]
    ),
    "hkDiagonalizedMassProperties::pack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_ulonglong,  # const hkMassProperties *
        ]
    ),
    "hkDiagonalizedMassProperties::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
        ]
    ),
    "hkCompressedMassProperties::unpack": {
        "hkCompressedMassProperties *, hkMassProperties *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkCompressedMassProperties *
                ctypes.c_ulonglong,  # hkMassProperties *
            ]
        ),
        "hkCompressedMassProperties *, hkDiagonalizedMassProperties *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkCompressedMassProperties *
                ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ]
        ),
    },
    "hkDiagonalizedMassProperties::unpack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDiagonalizedMassProperties *
            ctypes.c_ulonglong,  # hkMassProperties *
        ]
    ),
    "hkArrayBase<hkMassElement>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkMassElement> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMassProperties::applyScale": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMassProperties *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkInertiaTensorComputer::combineMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkArrayBase<hkMassElement> *
        argtypes=[
            ctypes.c_ulonglong,  # const hkArrayBase<hkMassElement> *
            ctypes.c_ulonglong,  # hkMassProperties *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "InternalInertiaTensorComputer::compFaceIntegrals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # InternalInertiaTensorComputer *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "InternalInertiaTensorComputer::compProjectionIntegrals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # InternalInertiaTensorComputer *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "InternalInertiaTensorComputer::compVolumeIntegrals": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # InternalInertiaTensorComputer *
            ctypes.c_ulonglong,  # const hkGeometry *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkInertiaTensorComputer::computeBoxVolumeMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkMassProperties *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkInertiaTensorComputer::computeBoxVolumeMassPropertiesDiagonalized": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hkInertiaTensorComputer::computeCapsuleVolumeMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkMassProperties *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "hkInertiaTensorComputer::computeCylinderVolumeMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkMassProperties *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "hkInertiaTensorComputer::computeGeometryVolumeMassProperties": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkGeometry *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkMassProperties *
        ]
    ),
    "hkInertiaTensorComputer::computeSphereVolumeMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkMassProperties *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkInertiaTensorComputer::computeTriangleSurfaceMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkMassProperties *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkInertiaTensorComputer::convertInertiaTensorToPrincipleAxis": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
            ctypes.c_ulonglong,  # hkRotationImpl<float> *
        ]
    ),
    "hkMassProperties::scaleToDensity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMassProperties *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
        ]
    ),
    "hkInertiaTensorComputer::shiftInertiaFromCom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hkMatrix3Impl<float> *
        ]
    ),
    "hkGeometryUtils::_weldVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # hkBool
            ctypes.c_ulonglong,  # hkArray<int,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArray<int,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArrayBase<hk1AxisSweep::AabbInt> *
            ctypes.c_ulonglong,  # hkArrayBase<hkRadixSort::SortDataT<unsigned int,unsigned int> > *
            ctypes.c_ulonglong,  # hkArrayBase<hk1AxisSweep::AabbInt> *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkGeometryUtils::appendCapsule": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkGeometry *
        ]
    ),
    "hkGeometryUtils::appendSphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_int32,  # int
        ]
    ),
    "buildIcosahedron": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
        ]
    ),
    "hkGeometryUtils::calculateInnerSphere": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkSphere *
            ctypes.c_ulonglong,  # __m128 *
        ]
    ),
    "calculateResultingVelocity": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkInplaceArray<hkVector4f,4,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "findNextPlane": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # const hkArray<bool,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "findNextTouchingPlane": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # const hkArray<bool,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkInplaceArray<hkVector4f,4,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hkGeometryUtils::weldVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_float,  # float
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkGeometryUtils::weldVerticesVirtual": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hkGeometryUtils::IVertices *
            ctypes.c_ulonglong,  # hkArray<int,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_float,  # float
        ]
    ),
    "hkArrayBase<hkaSkeleton::LocalFrameOnBone>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkaSkeleton::LocalFrameOnBone> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkaSkeleton::Partition>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkaSkeleton::Partition> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkQsTransformf>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkQsTransformf> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkaBone>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkaBone> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkaSkeleton::hkaSkeleton": {
        "hkaSkeleton *, const hkaSkeleton *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkaSkeleton *
                ctypes.c_ulonglong,  # const hkaSkeleton *
            ]
        ),
        "hkaSkeleton *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkaSkeleton *
            ]
        ),
    },
    "hkaSkeleton::~hkaSkeleton": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkaSkeleton *
        ]
    ),
    "hkArrayBase<hknpCompressedMeshShapeInternals::GeometryProvider::PrimitiveCache *>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkSerialize::Detail::TagfileBundle::NamedType *> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkFindUniquePositionsUtil::addPosition": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkFindUniquePositionsUtil *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkMemoryMeshTexture::hkMemoryMeshTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshTexture *
        ]
    ),
    "hkMemoryMeshTexture::equals": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshTexture *
            ctypes.c_ulonglong,  # const hkMeshTexture *
        ]
    ),
    "hkMemoryMeshTexture::getData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshTexture *
            ctypes.c_ulonglong,  # unsigned __int8 **
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # hkMeshTexture::Format *
        ]
    ),
    "hkMemoryResourceContainer::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceContainer *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkMemoryMeshTexture::getFilterMode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshTexture *
        ]
    ),
    "hkMemoryMeshTexture::getFormat": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshTexture *
        ]
    ),
    "hkMemoryMeshTexture::getHasMipMaps": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshTexture *
        ]
    ),
    "hkMemoryMeshTexture::getTextureCoordChannel": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshTexture *
        ]
    ),
    "hkMemoryMeshTexture::getUsageHint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshTexture *
        ]
    ),
    "hkMemoryMeshTexture::setData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshTexture *
            ctypes.c_ulonglong,  # unsigned __int8 *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # hkMeshTexture::Format
        ]
    ),
    "hkMemoryMeshTexture::setFilename": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkMemoryMeshTexture::setFilterMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshTexture *
            ctypes.c_uint32,  # hkMeshTexture::FilterMode
        ]
    ),
    "hkMemoryMeshTexture::setHasMipMaps": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshTexture *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkMemoryMeshTexture::setTextureCoordChannel": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshTexture *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMemoryMeshTexture::setUsageHint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshTexture *
            ctypes.c_uint32,  # hkMeshTexture::TextureUsageType
        ]
    ),
    "hkArrayBase<hkMatrix4Impl<float>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkMatrix4Impl<float> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkSkinBinding::hkSkinBinding": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSkinBinding *
            ctypes.c_ulonglong,  # hkMeshShape *
            ctypes.c_ulonglong,  # const hkMatrix4Impl<float> *
            ctypes.c_ulonglong,  # const hkStringPtr *
            ctypes.c_int32,  # int
        ]
    ),
    "hkSkinBinding::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkSkinBinding *
        ]
    ),
    "hkSkinBinding::getNumSections": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkSkinBinding *
        ]
    ),
    "hkSkinBinding::lockSection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSkinBinding *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # hkMeshSection *
        ]
    ),
    "hkSkinBinding::setName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSkinBinding *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkSkinBinding::unlockSection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSkinBinding *
            ctypes.c_ulonglong,  # const hkMeshSection *
        ]
    ),
    "hkArrayBase<hkSkinnedMeshShape::Part>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkSkinnedMeshShape::Part> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkSkinnedMeshShape::BoneSection>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkSkinnedMeshShape::BoneSection> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkSkinnedMeshShape::BoneSet>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkSkinnedMeshShape::BoneSet> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkSkinnedMeshShape::BoneSection::BoneSection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSkinnedMeshShape::BoneSection *
        ]
    ),
    "hkStorageSkinnedMeshShape::hkStorageSkinnedMeshShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkStorageSkinnedMeshShape *
        ]
    ),
    "hkStorageSkinnedMeshShape::addBoneSet": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHandle<unsigned short,65535,hkSkinnedMeshShape::BoneSetIdDiscriminant> *
        argtypes=[
            ctypes.c_ulonglong,  # hkStorageSkinnedMeshShape *
            ctypes.c_ulonglong,  # hkHandle<unsigned short,65535,hkSkinnedMeshShape::BoneSetIdDiscriminant> *
            ctypes.c_ulonglong,  # const __int16 *
            ctypes.c_int32,  # int
        ]
    ),
    "hkStorageSkinnedMeshShape::addPart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkStorageSkinnedMeshShape *
            ctypes.c_ulonglong,  # const hkSkinnedMeshShape::Part *
        ]
    ),
    "hkSkinnedMeshShape::dbgOut": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSkinnedMeshShape *
        ]
    ),
    "hkStorageSkinnedMeshShape::getBoneSection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkStorageSkinnedMeshShape *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkSkinnedMeshShape::BoneSection *
        ]
    ),
    "hkStorageSkinnedMeshShape::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkStorageSkinnedMeshShape *
        ]
    ),
    "hkDefaultCompoundMeshShape::getNumSections": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceContainer *
        ]
    ),
    "hkStorageSkinnedMeshShape::getPart": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkStorageSkinnedMeshShape *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkSkinnedMeshShape::Part *
        ]
    ),
    "hkSkinnedMeshShape::Part::less": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hkSkinnedMeshShape::Part *
            ctypes.c_ulonglong,  # const hkSkinnedMeshShape::Part *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkVector4f,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkStorageSkinnedMeshShape::setName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkStorageSkinnedMeshShape *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkStorageSkinnedMeshShape::sortParts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkStorageSkinnedMeshShape *
        ]
    ),
    "hkDefaultCompoundMeshBody::~hkDefaultCompoundMeshBody": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultCompoundMeshBody *
        ]
    ),
    "hkDefaultCompoundMeshShape::~hkDefaultCompoundMeshShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultCompoundMeshShape *
        ]
    ),
    "hkDefaultCompoundMeshBody::completeUpdate": {
        "hkDefaultCompoundMeshBody *, const hkMatrix4Impl<float> *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkDefaultCompoundMeshBody *
                ctypes.c_ulonglong,  # const hkMatrix4Impl<float> *
            ]
        ),
        "hkDefaultCompoundMeshBody *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkDefaultCompoundMeshBody *
            ]
        ),
    },
    "hkDefaultCompoundMeshBody::ensureInverseTransforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultCompoundMeshBody *
        ]
    ),
    "hkDefaultCompoundMeshBody::getTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultCompoundMeshBody *
            ctypes.c_ulonglong,  # hkMatrix4Impl<float> *
        ]
    ),
    "hkDefaultCompoundMeshBody::getVertexBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkMeshVertexBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultCompoundMeshBody *
            ctypes.c_int32,  # int
        ]
    ),
    "hkDefaultCompoundMeshShape::lockSection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultCompoundMeshShape *
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # hkMeshSection *
        ]
    ),
    "hkDefaultCompoundMeshBody::setIndexedInverseTransforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultCompoundMeshBody *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkMatrix4Impl<float> *
            ctypes.c_int32,  # int
        ]
    ),
    "hkDefaultCompoundMeshBody::setIndexedTransforms": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultCompoundMeshBody *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkMatrix4Impl<float> *
            ctypes.c_int32,  # int
        ]
    ),
    "hkDefaultCompoundMeshBody::setPickingData": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultCompoundMeshBody *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkDefaultCompoundMeshBody::setTransform": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultCompoundMeshBody *
            ctypes.c_ulonglong,  # const hkMatrix4Impl<float> *
        ]
    ),
    "hkDefaultCompoundMeshShape::unlockSection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDefaultCompoundMeshShape *
            ctypes.c_ulonglong,  # const hkMeshSection *
        ]
    ),
    "hkSkinnedRefMeshShape::~hkSkinnedRefMeshShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSkinnedRefMeshShape *
        ]
    ),
    "hkSkinnedRefMeshShape::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkSkinnedRefMeshShape *
        ]
    ),
    "hkSkinnedRefMeshShape::setName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSkinnedRefMeshShape *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkArrayBase<hkRefPtr<hkMeshTexture>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkRefPtr<hkMeshTexture> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemoryMeshMaterial::hkMemoryMeshMaterial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkMemoryMeshMaterial::~hkMemoryMeshMaterial": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
        ]
    ),
    "hkMemoryMeshMaterial::addTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
            ctypes.c_ulonglong,  # hkMeshTexture *
        ]
    ),
    "hkMemoryMeshMaterial::createCompatibleVertexBuffer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkMeshVertexBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
            ctypes.c_ulonglong,  # hkMeshVertexBuffer *
        ]
    ),
    "hkMemoryMeshMaterial::createCompatibleVertexFormat": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkVertexFormat *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
            ctypes.c_ulonglong,  # const hkVertexFormat *
            ctypes.c_ulonglong,  # hkVertexFormat *
            ctypes.c_ulonglong,  # hkVertexFormat *
        ]
    ),
    "hkMemoryMeshMaterial::equals": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
            ctypes.c_ulonglong,  # const hkMeshMaterial *
        ]
    ),
    "hkMemoryMeshMaterial::getColors": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkMemoryMeshMaterial::getTexture": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkMeshTexture *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMemoryMeshMaterial::isCompatible": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer *
        ]
    ),
    "hkMemoryMeshMaterial::reserveTextureSlot": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
            ctypes.c_ulonglong,  # const hkMeshTexture *
        ]
    ),
    "hkMemoryMeshMaterial::setColors": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkMemoryMeshMaterial::setTexture": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshMaterial *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkMeshTexture *
        ]
    ),
    "hkMemoryMeshShape::~hkMemoryMeshShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshShape *
        ]
    ),
    "hkMemoryMeshShape::lockSection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshShape *
            ctypes.c_int32,  # int
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_ulonglong,  # hkMeshSection *
        ]
    ),
    "hkMemoryMeshShape::unlockSection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshShape *
            ctypes.c_ulonglong,  # const hkMeshSection *
        ]
    ),
    "hkIndexedTransformSet::~hkIndexedTransformSet": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIndexedTransformSet *
        ]
    ),
    "hkIndexedTransformSet::calculateMatrices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIndexedTransformSet *
            ctypes.c_ulonglong,  # const hkMatrix4Impl<float> *
            ctypes.c_ulonglong,  # hkArray<hkMatrix4Impl<float>,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkIndexedTransformSet::getInverseMatrices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIndexedTransformSet *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkMatrix4Impl<float> *
            ctypes.c_int32,  # int
        ]
    ),
    "hkIndexedTransformSet::getMatrices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIndexedTransformSet *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkMatrix4Impl<float> *
            ctypes.c_int32,  # int
        ]
    ),
    "hkIndexedTransformSet::setInverseMatrices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIndexedTransformSet *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkMatrix4Impl<float> *
            ctypes.c_int32,  # int
        ]
    ),
    "hkIndexedTransformSet::setMatrices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIndexedTransformSet *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkMatrix4Impl<float> *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMeshVertexBuffer::hkMeshVertexBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMeshVertexBuffer *
        ]
    ),
    "hkVertexFormat::hkVertexFormat": {
        "hkVertexFormat *, const hkVertexFormat *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkVertexFormat *
                ctypes.c_ulonglong,  # const hkVertexFormat *
            ]
        ),
        "hkVertexFormat *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkVertexFormat *
            ]
        ),
    },
    "hkVertexFormat::addElement": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkVertexFormat *
            ctypes.c_ulonglong,  # const hkVertexFormat::Element *
        ]
    ),
    "hkVertexFormat::calculateSharingType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkVertexFormat *
        ]
    ),
    "hkVertexFormat::findElementIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkVertexFormat *
            ctypes.c_uint32,  # hkVertexFormat::ComponentUsage
            ctypes.c_int32,  # int
        ]
    ),
    "hkVertexFormat_orderComponents": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hkVertexFormat::Element *
            ctypes.c_ulonglong,  # const hkVertexFormat::Element *
        ]
    ),
    "hkVertexFormat::makeCanonicalOrder": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkVertexFormat *
        ]
    ),
    "hkMemoryMeshBody::~hkMemoryMeshBody": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshBody *
        ]
    ),
    "hkMemoryMeshVertexBuffer::hkMemoryMeshVertexBuffer": {
        "hkMemoryMeshVertexBuffer *, const hkVertexFormat *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkMemoryMeshVertexBuffer *
                ctypes.c_ulonglong,  # const hkVertexFormat *
                ctypes.c_int32,  # int
            ]
        ),
        "hkMemoryMeshVertexBuffer *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkMemoryMeshVertexBuffer *
            ]
        ),
    },
    "hkMemoryMeshVertexBuffer::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshVertexBuffer *
        ]
    ),
    "hkMemoryMeshVertexBuffer::clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkMemoryMeshVertexBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshVertexBuffer *
        ]
    ),
    "hkMultipleVertexBuffer::getElementIntArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMultipleVertexBuffer *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hkMemoryMeshVertexBuffer::getElementVectorArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMultipleVertexBuffer *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "hkMemoryMeshVertexBuffer::getLockedVerticesBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshVertexBuffer *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkMeshVertexBuffer::LockedVertices::Buffer *
        ]
    ),
    "hkMemoryMeshVertexBuffer::getNumVertices": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshVertexBuffer *
        ]
    ),
    "hkMemoryMeshVertexBuffer::isSharable": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshVertexBuffer *
        ]
    ),
    "hkMemoryMeshVertexBuffer::lock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshVertexBuffer *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockInput *
            ctypes.c_ulonglong,  # hkMeshVertexBuffer::LockedVertices *
        ]
    ),
    "hkMemoryMeshVertexBuffer::partialLock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshVertexBuffer *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockInput *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::PartialLockInput *
            ctypes.c_ulonglong,  # hkMeshVertexBuffer::LockedVertices *
        ]
    ),
    "hkMemoryMeshVertexBuffer::setElementIntArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMultipleVertexBuffer *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices *
            ctypes.c_ulonglong,  # const int *
            ctypes.c_ulonglong,  # const int *
        ]
    ),
    "hkMultipleVertexBuffer::setElementVectorArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMultipleVertexBuffer *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # const float *
        ]
    ),
    "hkMemoryMeshVertexBuffer::unlock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryMeshVertexBuffer *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices *
        ]
    ),
    "hkArrayBase<hkMultipleVertexBuffer::LockedElement>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkMultipleVertexBuffer::LockedElement> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkMultipleVertexBuffer::ElementInfo>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkMultipleVertexBuffer::ElementInfo> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkMultipleVertexBuffer::VertexBufferInfo>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkMultipleVertexBuffer::VertexBufferInfo> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMultipleVertexBuffer::hkMultipleVertexBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMultipleVertexBuffer *
            ctypes.c_ulonglong,  # const hkMultipleVertexBuffer *
        ]
    ),
    "hkMultipleVertexBuffer::_less": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hkMultipleVertexBuffer::LockedElement *
            ctypes.c_ulonglong,  # const hkMultipleVertexBuffer::LockedElement *
        ]
    ),
    "hkMultipleVertexBuffer::_lockElements": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMultipleVertexBuffer *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockInput *
            ctypes.c_ulonglong,  # hkMeshVertexBuffer::LockedVertices *
        ]
    ),
    "hkMultipleVertexBuffer::_unlockVertexBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMultipleVertexBuffer *
        ]
    ),
    "hkMultipleVertexBuffer::clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkMultipleVertexBuffer *
        argtypes=[
            ctypes.c_ulonglong,  # hkMultipleVertexBuffer *
        ]
    ),
    "hkMultipleVertexBuffer::lock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMultipleVertexBuffer *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockInput *
            ctypes.c_ulonglong,  # hkMeshVertexBuffer::LockedVertices *
        ]
    ),
    "hkMultipleVertexBuffer::partialLock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkMultipleVertexBuffer *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockInput *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::PartialLockInput *
            ctypes.c_ulonglong,  # hkMeshVertexBuffer::LockedVertices *
        ]
    ),
    "hkMultipleVertexBuffer::unlock": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMultipleVertexBuffer *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices *
        ]
    ),
    "hkMeshPrimitiveUtil::calculateNumIndices": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # hkMeshSection::PrimitiveType
            ctypes.c_int32,  # int
        ]
    ),
    "hkMeshVertexBufferUtil::convert": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices::Buffer *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices::Buffer *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMeshVertexBufferUtil::getElementIntArray": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices *
        argtypes=[
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # __m128i *
        ]
    ),
    "hkMeshVertexBufferUtil::getElementVectorArray": {
        "const hkMeshVertexBuffer::LockedVertices::Buffer *, float *, char *, int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices::Buffer *
            argtypes=[
                ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices::Buffer *
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # char *
                ctypes.c_int32,  # int
            ]
        ),
        "const hkMeshVertexBuffer::LockedVertices *, __int64, float *, char *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices *
            argtypes=[
                ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices *
                ctypes.c_int64,  # __int64
                ctypes.c_ulonglong,  # float *
                ctypes.c_ulonglong,  # char *
            ]
        ),
    },
    "hkMeshVertexBufferUtil_convertArgb32ToToFloat32": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices::Buffer *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices::Buffer *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMeshVertexBufferUtil_convertFloat32ToUint8": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices::Buffer *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices::Buffer *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMeshVertexBufferUtil_convertUint8DwordToUint8": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices::Buffer *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices::Buffer *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMeshVertexBufferUtil_convertUint8ToFloat32": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices::Buffer *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices::Buffer *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMeshVertexBufferUtil_convertUint8ToUint8Dword": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices::Buffer *
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices::Buffer *
            ctypes.c_int32,  # int
        ]
    ),
    "hkMeshVertexBufferUtil::setElementIntArray": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices *
        argtypes=[
            ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const int *
            ctypes.c_ulonglong,  # const __m128i *
        ]
    ),
    "hkMeshVertexBufferUtil::setElementVectorArray": {
        "const hkMeshVertexBuffer::LockedVertices::Buffer *, const float *, __m128 *, unsigned int": FUNCDEF(
            restype=ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices::Buffer *
            argtypes=[
                ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices::Buffer *
                ctypes.c_ulonglong,  # const float *
                ctypes.c_ulonglong,  # __m128 *
                ctypes.c_uint32,  # unsigned int
            ]
        ),
        "const hkMeshVertexBuffer::LockedVertices *, __int64, const float *, __m128 *": FUNCDEF(
            restype=ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices *
            argtypes=[
                ctypes.c_ulonglong,  # const hkMeshVertexBuffer::LockedVertices *
                ctypes.c_int64,  # __int64
                ctypes.c_ulonglong,  # const float *
                ctypes.c_ulonglong,  # __m128 *
            ]
        ),
    },
    "hkMeshVertexBufferUtil::stridedCopy": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkArrayBase<hkFourTransposedPointsImpl<float>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkFourTransposedPointsImpl<float> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkGeometryProcessing::buildTransposedArray": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArray<hkFourTransposedPointsImpl<float>,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkGeometryProcessing::getSupportingVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkArrayBase<hkFourTransposedPointsImpl<float> > *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkGeometryProcessing::octahedronToNormal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkArrayBase<hkgpConvexHullImpl::Triangle *>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkResourceHandle *> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkgpConvexHullImpl::Edge>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkTuple<void const *,int,int,void,void,void,void,void> > *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase__anonymous_namespace___EdgeData____growCapacity_hkContainerAllocator_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkgpConvexHullImpl::Vertex>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkgpConvexHullImpl::Vertex> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase__anonymous_namespace___VertexInfo____growCapacity_hkContainerAllocator_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkArray<int,hkBuiltinContainerAllocator<0,0>>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkArray<int,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkSphere>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkSphere> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkgpConvexHullImpl::SortedTriangle>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkTuple<unsigned __int64,unsigned __int64,void,void,void,void,void,void> > *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkgpConvexHullImpl::buildBevelPlanes<hkgpConvexHullImpl::PositionAsTransform>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHullImpl *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkgpConvexHullImpl::buildIndices<hkgpConvexHullImpl::PositionAsTransform>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHullImpl *
        ]
    ),
    "hkgpConvexHullImpl::buildMassProperties<hkgpConvexHullImpl::PositionAsTransform>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # int *
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHullImpl *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hkgpConvexHullImpl::generateIndexedFaces<int,int>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHullImpl *
            ctypes.c_ulonglong,  # hkArray<int,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArray<int,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkgpConvexHullImpl::getCentroid<hkgpConvexHullImpl::PositionAsTransform>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHullImpl *
        ]
    ),
    "hkArrayUtil::reduce<hkVector4f>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_int32,  # int
        ]
    ),
    "hkFixedSizeBuffer<hkVector4f,hkFixedStackStorage<hkVector4f>>::hkFixedSizeBuffer<hkVector4f,hkFixedStackStorage<hkVector4f>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkFixedSizeBuffer<hkVector4f,hkFixedStackStorage<hkVector4f> > *
            ctypes.c_int32,  # int
        ]
    ),
    "hkgpConvexHull::AbsoluteScaleConfig::AbsoluteScaleConfig": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull::AbsoluteScaleConfig *
        ]
    ),
    "hkgpConvexHull::BuildConfig::BuildConfig": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull::BuildConfig *
        ]
    ),
    "hkgpConvexHull::hkgpConvexHull": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
        ]
    ),
    "hkgpConvexHullImpl::hkgpConvexHullImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHullImpl *
        ]
    ),
    "hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Triangle,32,hkBuiltinContainerAllocator<0,0>>::~PoolAllocator<hkgpConvexHullImpl::Triangle,32,hkBuiltinContainerAllocator<0,0>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Triangle,32,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0>>::~PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkFixedStackStorage__anonymous_namespace___EdgeCollapseData__QuadricErrorMetrics____hkFixedStackStorage__anonymous_namespace___EdgeCollapseData__QuadricErrorMetrics_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkFixedStackStorage<hkVector4f>::~hkFixedStackStorage<hkVector4f>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkFixedStackStorage<hkVector4f> *
        ]
    ),
    "hkgpConvexHull::~hkgpConvexHull": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
        ]
    ),
    "hkgpConvexHull::absoluteScale": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_float,  # const float
            ctypes.c_ulonglong,  # const hkgpConvexHull::AbsoluteScaleConfig *
        ]
    ),
    "hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Triangle,32,hkBuiltinContainerAllocator<0,0>>::allocateItem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Triangle,32,hkBuiltinContainerAllocator<0,0> >::Item *
        argtypes=[
            ctypes.c_ulonglong,  # hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Triangle,32,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0>>::allocateItem": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0> >::Item *
        argtypes=[
            ctypes.c_ulonglong,  # hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkgpConvexHull::build": {
        "hkgpConvexHull *, const hkStridedVertices *, const hkgpConvexHull::BuildConfig *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # hkgpConvexHull *
                ctypes.c_ulonglong,  # const hkStridedVertices *
                ctypes.c_ulonglong,  # const hkgpConvexHull::BuildConfig *
            ]
        ),
        "hkgpConvexHull *, const hkVector4f *, int, const hkgpConvexHull::BuildConfig *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # hkgpConvexHull *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # const hkgpConvexHull::BuildConfig *
            ]
        ),
    },
    "hkgpConvexHull::buildFromDiscs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSphere *
            ctypes.c_ulonglong,  # __m128 *
            ctypes.c_ulonglong,  # const hkgpConvexHull::BuildConfig *
        ]
    ),
    "hkgpConvexHull::buildFromPlanes": {
        "hkgpConvexHull *, const hkVector4f *, int, const hkgpConvexHull::BuildConfig *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # hkgpConvexHull *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # const hkgpConvexHull::BuildConfig *
            ]
        ),
        "hkgpConvexHull *, const hkVector4f *, int, const hkVector4f *, const hkgpConvexHull::BuildConfig *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # hkgpConvexHull *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_ulonglong,  # const hkgpConvexHull::BuildConfig *
            ]
        ),
    },
    "hkgpConvexHull::buildPlanar": {
        "hkgpConvexHull *, const hkStridedVertices *, const hkVector4f *, const hkgpConvexHull::BuildConfig *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # hkgpConvexHull *
                ctypes.c_ulonglong,  # const hkStridedVertices *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_ulonglong,  # const hkgpConvexHull::BuildConfig *
            ]
        ),
        "hkgpConvexHull *, const hkVector4f *, int, const hkVector4f *, const hkgpConvexHull::BuildConfig *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # hkgpConvexHull *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_ulonglong,  # const hkgpConvexHull::BuildConfig *
            ]
        ),
    },
    "hkgpConvexHull::calculatePlaneEquation": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # const hkgpConvexHull::Triangle *
        ]
    ),
    "hkgpConvexHull::clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkgpConvexHull *
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
        ]
    ),
    "hkgpConvexHullImpl::clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkVector4f **
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHullImpl *
            ctypes.c_ulonglong,  # hkScopedPtr<hkgpConvexHullImpl> *
        ]
    ),
    "hkgpConvexHull::collapseEdges": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_float,  # const float
            ctypes.c_int32,  # int
        ]
    ),
    "hkgpConvexHull::expandByPlanarMargin": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_ulonglong,  # hkBool *
            ctypes.c_float,  # const float
        ]
    ),
    "hkgpConvexHull::fetchPlanes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkgpConvexHull::fetchPositions": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_uint32,  # hkgpConvexHull::Inputs
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkgpConvexHull::generateGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_uint32,  # hkgpConvexHull::Inputs
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkgpConvexHull::generateIndexedFaces": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_uint32,  # hkgpConvexHull::Inputs
            ctypes.c_ulonglong,  # hkArray<int,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArray<int,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ubyte,  # hkBool
        ]
    ),
    "hkgpConvexHullImpl::generatePlanesPerVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHullImpl *
            ctypes.c_ulonglong,  # hkArray<int,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArray<int,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkgpConvexHull::getBestProjectionPlane": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkgpConvexHull::getCentroid": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
        ]
    ),
    "hkgpConvexHull::getDimensions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
        ]
    ),
    "hkgpConvexHull::getIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_ulonglong,  # const hkgpConvexHull::Vertex *
        ]
    ),
    "hkgpConvexHull::getLocalInertia": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_ulonglong,  # hkTransformf *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkgpConvexHull::getNumPlanes": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
        ]
    ),
    "hkgpConvexHull::getNumVertices": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
        ]
    ),
    "hkgpConvexHull::getPlane": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_int32,  # int
        ]
    ),
    "hkgpConvexHull::getPlaneIndex": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_ulonglong,  # const hkgpConvexHull::Triangle *
        ]
    ),
    "hkgpConvexHull::getProjectionPlane": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
        ]
    ),
    "hkgpConvexHull::getSupportingVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkgpConvexHull::getSurfaceArea": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSimdFloat32 *
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hkgpConvexHull::getVolume": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSimdFloat32 *
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hkgpConvexHull::getWidthBounds": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hkgpConvexHull::hasMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_ulonglong,  # hkBool *
        ]
    ),
    "hkgpConvexHull::hasValidMassProperties": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_ulonglong,  # hkBool *
        ]
    ),
    "hkgpConvexHullImpl::isPointInside": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHullImpl *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ubyte,  # bool
            ctypes.c_float,  # float
        ]
    ),
    "hkgpConvexHull::mergePlanes": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHull *
            ctypes.c_float,  # const float
        ]
    ),
    "hkgpAbstractMesh<hkgpConvexHullImpl::Edge,hkgpConvexHullImpl::Vertex,hkgpConvexHullImpl::Triangle,hkBuiltinContainerAllocator<0,0>>::EdgeIterator::next": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpAbstractMesh<hkgpConvexHullImpl::Edge,hkgpConvexHullImpl::Vertex,hkgpConvexHullImpl::Triangle,hkBuiltinContainerAllocator<0,0> >::EdgeIterator *
        ]
    ),
    "hkgpConvexHullImpl::rebuildDomainParameters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHullImpl *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_int32,  # const int
        ]
    ),
    "hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Triangle,hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Vertex,hkgpConvexHull::Vertex,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0>>>::Item,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Triangle,32,hkBuiltinContainerAllocator<0,0>>>::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Triangle,hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Vertex,hkgpConvexHull::Vertex,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0> > >::Item,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Triangle,32,hkBuiltinContainerAllocator<0,0> > > *
        ]
    ),
    "hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Vertex,hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Vertex,hkgpConvexHull::Vertex,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0>>>::Item,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0>>>::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Vertex,hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Vertex,hkgpConvexHull::Vertex,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0> > >::Item,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0> > > *
        ]
    ),
    "hkgpAbstractMesh<hkgpConvexHullImpl::Edge,hkgpConvexHullImpl::Vertex,hkgpConvexHullImpl::Triangle,hkBuiltinContainerAllocator<0,0>>::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpAbstractMesh<hkgpConvexHullImpl::Edge,hkgpConvexHullImpl::Vertex,hkgpConvexHullImpl::Triangle,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkgpConvexHullImpl::reset": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHullImpl *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkSimdFloat32,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkGeometryUtility::createMinBoundingSphere": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSphere *
        argtypes=[
            ctypes.c_ulonglong,  # hkSphere *
            ctypes.c_ulonglong,  # const hkStridedVertices *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkKeycodeManager::getErrorMessage": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_uint32,  # hkKeycodeManager::ErrorCodes
        ]
    ),
    "hkKeycodeManager::getProductName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_uint32,  # hkKeycodeManager::Products
        ]
    ),
    "hkKeycodeManager::setKeycode": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkKeycodeManager::ErrorCodes *
        argtypes=[
            ctypes.c_ulonglong,  # hkKeycodeManager::ErrorCodes *
            ctypes.c_ulonglong,  # hkStringView *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "setKeycodes": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hkgpCgo::Config::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkgpCgo::Config *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "IConvexOverlapImpl::ShapeInterface::getSupportingVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # IConvexOverlapImpl::ShapeInterface *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkcdGsk::GetSupportingVertexOut *
        ]
    ),
    "IConvexOverlapImpl::ShapeInterface::getShapeAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # IConvexOverlapImpl::ShapeInterface *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "IConvexOverlapImpl::distance": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_ulonglong,  # IConvexOverlapImpl *
            ctypes.c_ulonglong,  # const hkgpMesh::IConvexOverlap::IConvexShape *
            ctypes.c_ulonglong,  # const hkgpMesh::IConvexOverlap::IConvexShape *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "IConvexOverlapImpl::checkOverlap": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # IConvexOverlapImpl *
            ctypes.c_ulonglong,  # const hkgpMesh::IConvexOverlap::IConvexShape *
            ctypes.c_ulonglong,  # const hkgpMesh::IConvexOverlap::IConvexShape *
            ctypes.c_float,  # float
        ]
    ),
    "hkArrayBase<hkgpConvexHullImpl::Vertex>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkgpConvexHullImpl::Vertex> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkgpConvexHullImpl::Vertex>::_ensureOwnsStorage": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkgpConvexHullImpl::Vertex> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Triangle,hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Vertex,hkgpConvexHull::Vertex,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0>>>::Item,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Triangle,32,hkBuiltinContainerAllocator<0,0>>>::allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkgpConvexHullImpl::Triangle *
        argtypes=[
            ctypes.c_ulonglong,  # hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Triangle,hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Vertex,hkgpConvexHull::Vertex,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0> > >::Item,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Triangle,32,hkBuiltinContainerAllocator<0,0> > > *
        ]
    ),
    "hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Vertex,hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Vertex,hkgpConvexHull::Vertex,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0>>>::Item,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0>>>::allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkgpConvexHullImpl::Vertex *
        argtypes=[
            ctypes.c_ulonglong,  # hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Vertex,hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Vertex,hkgpConvexHull::Vertex,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0> > >::Item,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0> > > *
        ]
    ),
    "hkgpConvexHullImpl::build": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHullImpl *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_int32,  # const int
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const hkgpConvexHullImpl *
        ]
    ),
    "hkgpConvexHullImpl::clearHorizon": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHullImpl *
            ctypes.c_ulonglong,  # const hkgpConvexHullImpl::Vertex *
            ctypes.c_ulonglong,  # const hkgpConvexHullImpl::Edge *
            ctypes.c_ulonglong,  # hkgpConvexHullImpl::Edge *
        ]
    ),
    "hkgpConvexHullImpl::computeProjectionPlane": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHullImpl *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkgpConvexHullImpl::flushPending": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # hkgpConvexHullImpl *
        ]
    ),
    "hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Triangle,hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Vertex,hkgpConvexHull::Vertex,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0>>>::Item,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Triangle,32,hkBuiltinContainerAllocator<0,0>>>::release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Triangle,hkgpAbstractMeshDefinitions::List<hkgpConvexHullImpl::Vertex,hkgpConvexHull::Vertex,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Vertex,32,hkBuiltinContainerAllocator<0,0> > >::Item,hkGeometryProcessing::PoolAllocator<hkgpConvexHullImpl::Triangle,32,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_ulonglong,  # hkgpConvexHullImpl::Triangle *
        ]
    ),
    "hkSingletonUtil__initWith_hkKeycodeDataBase_hkKeycodeData_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hkKeycodeDataBase> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hkKeycodeDataBase> *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hkVersionUtils::getHavokTime": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[]
    ),
    "hkVersionUtils::getSystemTime": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[]
    ),
    "hkVersionUtils::linkProduct": FUNCDEF(
        restype=ctypes.c_uint32,  # unsigned int
        argtypes=[
            ctypes.c_uint32,  # hkKeycodeManager::Products
        ]
    ),
    "hknpCompressedMeshShapeData::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeData *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hknpCompressedMeshShapeData::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hknpCompressedMeshShapeInternals::KeyMask::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::KeyMask *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkArrayBase<hkcdStaticMeshTree::Connectivity::BuildEdge>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkcdStaticMeshTree::Connectivity::BuildEdge> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkcdStaticMeshTree::BuildStepUtils::Quad>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkcdStaticMeshTree::BuildStepUtils::Quad> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase__anonymous_namespace___InternalEdgeSortingStruct____growCapacity_hkBuiltinContainerAllocatorBase_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkcdStaticMeshTree::BuildStepUtils::SubTree>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkcdStaticMeshTree::BuildStepUtils::SubTree> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::SlotPair<hkcdStaticMeshTree::Section,hkcdStaticMeshTree::Section>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::SlotPair<hknpCompressedMeshShapeTree,hknpCompressedMeshShapeTree> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase__anonymous_namespace___InternalEdgeSortingStruct____growCapacity_hkContainerAllocator_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkPointerMap<int,int,hkBuiltinContainerAllocator<0,0>>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkPointerMap<int,int,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkcdStaticMeshTree::BuildStepUtils::SubTree::LeafCollector::append<hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::BuildStepUtils::SubTree::LeafCollector *
            ctypes.c_ulonglong,  # const hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64> > > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::binary<hkcdStaticMeshTree::Section,hkcdStaticMeshTree::Section,hkcdTreeQueriesStacks::Dynamic<128,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::SlotPair<hknpCompressedMeshShapeTree,hknpCompressedMeshShapeTree>>,hknpCompressedMeshShapeInternals::GetClosestPointsToMeshQueryScaled>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Section *
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Section *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::SlotPair<hknpCompressedMeshShapeTree,hknpCompressedMeshShapeTree> > *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GetClosestPointsToMeshQueryScaled *
            ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb4BytesCodec>::NodeContext *
            ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb4BytesCodec>::NodeContext *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::binary<hkcdStaticMeshTree::Section,hkcdStaticMeshTree::Section,hkcdTreeQueriesStacks::Dynamic<128,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::SlotPair<hknpCompressedMeshShapeTree,hknpCompressedMeshShapeTree>>,hknpCompressedMeshShapeInternals::GetClosestPointsToMeshQueryUnscaled>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Section *
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Section *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::SlotPair<hknpCompressedMeshShapeTree,hknpCompressedMeshShapeTree> > *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GetClosestPointsToMeshQueryUnscaled *
            ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb4BytesCodec>::NodeContext *
            ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb4BytesCodec>::NodeContext *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::binary<hknpCompressedMeshShapeTree,hknpCompressedMeshShapeTree,hkcdTreeQueriesStacks::Dynamic<128,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::SlotPair<hknpCompressedMeshShapeTree,hknpCompressedMeshShapeTree>>,hknpCompressedMeshShapeInternals::GetClosestPointsToMeshQueryScaled>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::SlotPair<hknpCompressedMeshShapeTree,hknpCompressedMeshShapeTree> > *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GetClosestPointsToMeshQueryScaled *
            ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb5BytesCodec>::NodeContext *
            ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb5BytesCodec>::NodeContext *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::binary<hknpCompressedMeshShapeTree,hknpCompressedMeshShapeTree,hkcdTreeQueriesStacks::Dynamic<128,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::SlotPair<hknpCompressedMeshShapeTree,hknpCompressedMeshShapeTree>>,hknpCompressedMeshShapeInternals::GetClosestPointsToMeshQueryUnscaled>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::SlotPair<hknpCompressedMeshShapeTree,hknpCompressedMeshShapeTree> > *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GetClosestPointsToMeshQueryUnscaled *
            ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb5BytesCodec>::NodeContext *
            ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb5BytesCodec>::NodeContext *
        ]
    ),
    "hkcdStaticMeshTree::BuildStepUtils::buildStep12<hkcdDefaultStaticMeshTree>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::BuildConfig *
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::GeometryProvider *
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::BuildMappings *
            ctypes.c_ulonglong,  # const hkArrayView<hkBool> *
            ctypes.c_ulonglong,  # hkArray<hkcdStaticMeshTree::BuildStepUtils::Quad,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArray<hkcdStaticMeshTree::BuildStepUtils::SubTree,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::BuildStepUtils::DynamicTree *
        ]
    ),
    "hkcdStaticMeshTree::BuildStepUtils::buildStep3<hkcdDefaultStaticMeshTree>": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::BuildConfig *
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::BuildStepUtils::QuantizersInfos *
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::GeometryProvider *
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::BuildMappings *
            ctypes.c_ulonglong,  # const hkArrayView<hkBool> *
            ctypes.c_ulonglong,  # hkArray<hkcdStaticMeshTree::BuildStepUtils::Quad,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArray<hkcdStaticMeshTree::BuildStepUtils::SubTree,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::BuildStepUtils::DynamicTree *
            ctypes.c_ulonglong,  # hkcdDefaultStaticMeshTree *
        ]
    ),
    "hkcdSimdTree::closestFromAabb<hknpCompressedMeshShapeInternals::GetClosestPointsToConvexQuery>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GetClosestPointsToConvexQuery *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromAabbOrdered<hknpCompressedMeshShapeInternals::GetClosestPointsToConvexQuery>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GetClosestPointsToConvexQuery *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdStaticMeshTree::BuildStepUtils::computeConnectivity<hkcdDefaultStaticMeshTree>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdDefaultStaticMeshTree *
            ctypes.c_ulonglong,  # hkArray<hkcdStaticMeshTree::BuildStepUtils::Links,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkcdStaticTree::convertDynamicTreeToStaticTree<hkcdStaticMeshTree::Section,hkcdStaticMeshTree::BuildStepUtils::DynamicTree>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::BuildStepUtils::DynamicTree *
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::Section *
        ]
    ),
    "hkcdStaticTree::convertDynamicTreeToStaticTree<hkcdDefaultStaticMeshTree,hkcdStaticMeshTree::BuildStepUtils::DynamicTree>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::BuildStepUtils::DynamicTree *
            ctypes.c_ulonglong,  # hkcdDefaultStaticMeshTree *
        ]
    ),
    "hknpCompressedMeshShapeInternals::GetClosestPointsToHeightFieldQueryBase::processLeaf<hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb4BytesCodec>::NodeContext>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GetClosestPointsToHeightFieldQueryBase *
            ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb4BytesCodec>::NodeContext *
        ]
    ),
    "hknpCompressedMeshShapeInternals::GetClosestPointsToMeshQueryBase::processLeaves<hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb4BytesCodec>::NodeContext>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GetClosestPointsToMeshQueryBase *
            ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb4BytesCodec>::NodeContext *
            ctypes.c_ulonglong,  # const hkcdStaticTree::AabbTree<hkcdCompressedAabbCodecs::Aabb4BytesCodec>::NodeContext *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::rayCastExternal<hknpCompressedMeshShapeTree,hknpCompressedMeshShapeInternals::RayCastQuery<0>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # const hkcdRay *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::RayCastQuery<0> *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticMeshTree::Section,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromAabbWrapper<hknpCompressedMeshShapeInternals::GetClosestPointsToConvexQuery>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Section *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromAabbWrapper<hknpCompressedMeshShapeInternals::GetClosestPointsToConvexQuery> *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticMeshTree::Section,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::RayCastWrapperExternal<hknpCompressedMeshShapeInternals::RayCastQuery<0>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Section *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::RayCastWrapperExternal<hknpCompressedMeshShapeInternals::RayCastQuery<0> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticMeshTree::Section,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::VisibilityWrapper<hknpCompressedMeshShapeInternals::RayCastQuery<1>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Section *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::VisibilityWrapper<hknpCompressedMeshShapeInternals::RayCastQuery<1> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticMeshTree::Section,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hknpCompressedMeshShapeInternals::GetClosestPointsToConvexQueryScaled>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Section *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GetClosestPointsToConvexQueryScaled *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticMeshTree::Section,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hknpCompressedMeshShapeInternals::GetClosestPointsToHeightFieldQueryScaled0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Section *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GetClosestPointsToHeightFieldQueryScaled0 *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticMeshTree::Section,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hknpCompressedMeshShapeInternals::GetClosestPointsToHeightFieldQueryScaled1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Section *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GetClosestPointsToHeightFieldQueryScaled1 *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticMeshTree::Section,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hknpCompressedMeshShapeInternals::GetClosestPointsToHeightFieldQueryScaled2>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Section *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GetClosestPointsToHeightFieldQueryScaled2 *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hkcdStaticMeshTree::Section,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hknpCompressedMeshShapeInternals::GetClosestPointsToHeightFieldQueryUnscaled>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::Section *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GetClosestPointsToHeightFieldQueryUnscaled *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompressedMeshShapeTree,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromAabbWrapper<hknpCompressedMeshShapeInternals::GetClosestPointsToConvexQuery>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::ClosestFromAabbWrapper<hknpCompressedMeshShapeInternals::GetClosestPointsToConvexQuery> *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompressedMeshShapeTree,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::RayCastWrapperExternal<hknpCompressedMeshShapeInternals::RayCastQuery<0>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::RayCastWrapperExternal<hknpCompressedMeshShapeInternals::RayCastQuery<0> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompressedMeshShapeTree,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::VisibilityWrapper<hknpCompressedMeshShapeInternals::RayCastQuery<1>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::VisibilityWrapper<hknpCompressedMeshShapeInternals::RayCastQuery<1> > *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompressedMeshShapeTree,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hknpCompressedMeshShapeInternals::GetClosestPointsToConvexQueryScaled>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GetClosestPointsToConvexQueryScaled *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompressedMeshShapeTree,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hknpCompressedMeshShapeInternals::GetClosestPointsToHeightFieldQueryScaled0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GetClosestPointsToHeightFieldQueryScaled0 *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompressedMeshShapeTree,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hknpCompressedMeshShapeInternals::GetClosestPointsToHeightFieldQueryScaled1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GetClosestPointsToHeightFieldQueryScaled1 *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompressedMeshShapeTree,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hknpCompressedMeshShapeInternals::GetClosestPointsToHeightFieldQueryScaled2>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GetClosestPointsToHeightFieldQueryScaled2 *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::unary<hknpCompressedMeshShapeTree,hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot>,hknpCompressedMeshShapeInternals::GetClosestPointsToHeightFieldQueryUnscaled>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # hkcdTreeQueriesStacks::Dynamic<128,hkcdStaticTree::AabbTreeBase::Slot> *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GetClosestPointsToHeightFieldQueryUnscaled *
        ]
    ),
    "hkcdTreeQueries<hkcdTreeQueriesStacks::Dynamic,128,0>::visible<hknpCompressedMeshShapeTree,hknpCompressedMeshShapeInternals::RayCastQuery<1>>": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeTree *
            ctypes.c_ulonglong,  # const hkcdRay *
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::RayCastQuery<1> *
        ]
    ),
    "hkcdStaticMeshTree::BuildStepUtils::DynamicTree::DynamicTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::BuildStepUtils::DynamicTree *
        ]
    ),
    "hknpCompressedMeshShapeData::hknpCompressedMeshShapeData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeData *
        ]
    ),
    "hknpRayCastQuery::hknpRayCastQuery": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
        ]
    ),
    "hkFixedStackStorage<short>::~hkFixedStackStorage<short>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkFixedStackStorage<short> *
        ]
    ),
    "hknpCompressedMeshShapeData::_getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeData *
        ]
    ),
    "hknpCompressedMeshShapeData::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeData *
        ]
    ),
    "hkcdStaticMeshTree::SectionDecoder<hkcdDefaultStaticMeshTree>::CustomGeometryConverterBase::appendCustomPrimitiveToGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::SectionDecoder<hkcdDefaultStaticMeshTree>::CustomGeometryConverterBase *
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::SectionDecoder<hkcdDefaultStaticMeshTree> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkGeometry *
        ]
    ),
    "hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64>>>::balance": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64> > > *
            ctypes.c_int32,  # int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkcdStaticMeshTree::Builder<hkcdDefaultStaticMeshTree>::build": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::BuildConfig *
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::GeometryProvider *
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::BuildMappings *
            ctypes.c_ulonglong,  # hkcdDefaultStaticMeshTree *
        ]
    ),
    "hkcdStaticMeshTree::Builder<hkcdDefaultStaticMeshTree>::buildConnectivity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdDefaultStaticMeshTree *
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::Connectivity *
        ]
    ),
    "hknpCompressedMeshShape::buildConnectivity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShape *
        ]
    ),
    "hknpCompressedMeshShape::buildInternalData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShape *
            ctypes.c_ulonglong,  # const hknpCompressedMeshShapeCinfo *
        ]
    ),
    "hkcdStaticMeshTree::Builder<hkcdDefaultStaticMeshTree>::buildSimdTree": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdDefaultStaticMeshTree *
            ctypes.c_ulonglong,  # hkcdSimdTree *
        ]
    ),
    "hknpCompressedMeshShapeFunctions::buildSurfaceGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpShape *
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hknpShape::BuildSurfaceGeometryConfig *
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hknpCompressedMeshShapeFunctions::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hkcdStaticMeshTree::SectionDecoder<hkcdDefaultStaticMeshTree>::convertToGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdDefaultStaticMeshTree *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkGeometry *
            ctypes.c_ulonglong,  # hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::SectionDecoder<hkcdDefaultStaticMeshTree>::CustomGeometryConverterBase *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # hkcdStaticMeshTree::TriangleMaterial
        ]
    ),
    "hknpCompressedMeshShape::countPrimitives": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShape *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64>>>::detachBranch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64> > > *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64> > > *
        ]
    ),
    "hknpCompressedMeshShape::flagInteriorTriangles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShape *
            ctypes.c_double,  # double
        ]
    ),
    "hknpCompressedMeshShape::getAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShape *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpCompressedMeshShapeUtil::getClosestPointsToCompressedMesh": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpCompressedMeshShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompressedMeshShapeUtil::getClosestPointsToConvex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpCompressedMeshShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompressedMeshShapeUtil::getClosestPointsToHeightfield": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hknpCompressedMeshShape *
            ctypes.c_ulonglong,  # const hknpQueryFilterData *
            ctypes.c_ulonglong,  # const hknpShapeQueryInfo *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpCompressedMeshShapeInternals::GeometryProvider::getCustomPrimitiveDataImpl": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GeometryProvider *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpCompressedMeshShapeInternals::GeometryProvider::getCustomPrimitiveInfos": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GeometryProvider *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::GeometryProvider::CustomPrimitiveInfos *
        ]
    ),
    "hknpCompressedMeshShapeInternals::GeometryProvider::getCustomPrimitiveVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GeometryProvider *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hknpCompressedMeshShapeInternals::GeometryProvider::getIndicesImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GeometryProvider *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hknpCompressedMeshShapeFunctions::getLeafShapes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hknpShape *
            ctypes.c_ulonglong,  # const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hknpShapeCollector *
        ]
    ),
    "hknpCompressedMeshShapeData::getMemoryFootprint": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeData *
        ]
    ),
    "hknpCompressedMeshShapeInternals::GeometryProvider::getNumCustomPrimitives": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GeometryProvider *
        ]
    ),
    "hknpCompressedMeshShapeInternals::GeometryProvider::getNumTriangles": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GeometryProvider *
        ]
    ),
    "hknpCompressedMeshShapeInternals::GeometryProvider::getNumVertices": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GeometryProvider *
        ]
    ),
    "hkcdStaticMeshTree::SectionDecoder<hkcdDefaultStaticMeshTree>::getQuadVertices": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdDefaultStaticMeshTree *
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::ExpandedPrimitiveKey *
            ctypes.c_ulonglong,  # hkFloat3 *
        ]
    ),
    "hkcdStaticMeshTree::SectionDecoder<hkcdDefaultStaticMeshTree>::CustomGeometryConverterBase::getScale": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkVector4f *
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::SectionDecoder<hkcdDefaultStaticMeshTree>::CustomGeometryConverterBase *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpCompressedMeshShapeInternals::GeometryProvider::getTriangleDataImpl": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeInternals::GeometryProvider *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hknpCompressedMeshShapeFunctions::makeShapeKeyMask": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hknpShapeKeyMask> *
            ctypes.c_ulonglong,  # const hknpShape *
        ]
    ),
    "hknpCompressedMeshShape::optimizeForSpeed": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShape *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64>>>::rebuildGlobalSAH": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64> > > *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkSet<unsigned __int64,hkBuiltinContainerAllocator<2,0>,hkMapOperations<unsigned __int64>>::remove": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSet<unsigned __int64,hkBuiltinContainerAllocator<2,0>,hkMapOperations<unsigned __int64> > *
            ctypes.c_ulonglong,  # Dummy *
        ]
    ),
    "hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64>>>::rotateLeft": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64> > > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64>>>::rotateRight": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkcdDynamicTree::Tree<hkcdDynamicTree::DefaultDynamicStorage<hkcdDynamicTree::CodecRaw<unsigned __int64> > > *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "calcFanNMP": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSimdFloat32 *
        argtypes=[
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hknpConvexHull *
            ctypes.c_ulonglong,  # const hkFloat3 *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkcdPointConvexVoronoiImpl": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hknpConvexHull *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkcdPointConvexConfig *
            ctypes.c_ulonglong,  # hkcdPointConvexVoronoiCache *
            ctypes.c_ulonglong,  # hkcdPointQueryOut *
        ]
    ),
    "hkcdPointVsConvexHull": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # const hknpConvexHull *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hkcdPointQueryCache *
            ctypes.c_ulonglong,  # hkcdPointQueryOut *
        ]
    ),
    "hknpGsk2D::getClosestPoints": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpGsk2D *
            ctypes.c_ulonglong,  # const hknpGsk2DShapeInterface *
            ctypes.c_ulonglong,  # const hknpConvexVertexShape *
            ctypes.c_ulonglong,  # const hknpConvexVertexShape *
            ctypes.c_ulonglong,  # const hkcdGsk::GetClosestPointInput *
            ctypes.c_ulonglong,  # hkcdGsk::Cache *
            ctypes.c_ulonglong,  # hkcdGsk::GetClosestPointOutput *
        ]
    ),
    "hknpConvexConvexShapeBaseInterface::getPlanes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexConvexShapeBaseInterface *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkVector4f **
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # const hkVector4f **
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hknpConvexConvexShapeBaseInterface::getShapeAabb": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexConvexShapeBaseInterface *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpConvexConvexShapeBaseInterface::getSupportingVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpConvexConvexShapeBaseInterface *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hkcdGsk::GetSupportingVertexOut *
        ]
    ),
    "hknpProcessFlyingColors": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkBool *
        ]
    ),
    "hknpCollision2DFastProcessUtil::buildFaceManifold": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSolverInfo *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_ulonglong,  # hkVector4f *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # hknpConvexConvexManifoldCollisionCache *
            ctypes.c_ulonglong,  # hknpManifold *
        ]
    ),
    "hknpFaceClipConvexConvexManifoldGenerator::collide": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpFaceClipConvexConvexManifoldGenerator *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpConvexConvexManifoldGenerator::Input *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpGskCollisionCache *
            ctypes.c_ulonglong,  # hknpManifold *
            ctypes.c_ulonglong,  # hkMonitorStream::MultiTimerCommand **
        ]
    ),
    "hknpCollision2DFastProcessUtil::collideWithInteriorTriangle": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hknpModifierSharedData *
            ctypes.c_ulonglong,  # const hknpConvexConvexManifoldGenerator::Input *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # const hknpCdBody *
            ctypes.c_ulonglong,  # hknpGskCollisionCache *
            ctypes.c_ulonglong,  # hknpManifold *
        ]
    ),
    "hkSerialize::Bundle::Bundle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Bundle *
        ]
    ),
    "hkSerialize::ReadFormat::ReadFormat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::ReadFormat *
        ]
    ),
    "hkSerialize::TypeWriter::TypeWriter": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TypeWriter *
        ]
    ),
    "hkSerialize::WriteFormat::WriteFormat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::WriteFormat *
        ]
    ),
    "hkSerialize::Bundle::~Bundle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Bundle *
        ]
    ),
    "hkSerialize::WriteFormat::useTypeCompendium": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkIo::Detail::WriteBufferAdapter *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::WriteFormat *
            ctypes.c_ulonglong,  # const hkIo::Detail::WriteBufferAdapter *
            ctypes.c_ulonglong,  # const hkIo::Detail::ReadBufferAdapter *
        ]
    ),
    "hkArrayBase<hkPair<hkReflect::TypeName,hkTuple<int,hk::optional<uint>,void,void,void,void,void,void>>>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<hkReflect::TypeName,hkTuple<int,hk::optional<unsigned int>,void,void,void,void,void,void> > > *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkPair<hkReflect::Type const *,int>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<hkReflect::Type const *,int> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkPair<hkReflect::TypeName,hkTuple<int,hk::optional<uint>,void,void,void,void,void,void>>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<hkReflect::TypeName,hkTuple<int,hk::optional<unsigned int>,void,void,void,void,void,void> > > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkQueue<hkSerialize::TypeAndId,hkBuiltinContainerAllocator<0,0>>::enqueue<hkSerialize::TypeAndId>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkQueue<hkSerialize::TypeAndId,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkSerialize::TypeAndId *
        ]
    ),
    "hkQueue<hkSerialize::TypeAndId,hkBuiltinContainerAllocator<0,0>>::~hkQueue<hkSerialize::TypeAndId,hkBuiltinContainerAllocator<0,0>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkQueue<hkSerialize::TypeAndId,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkSerialize::Detail::TypeWriterMap::~TypeWriterMap": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::TypeWriterMap *
        ]
    ),
    "detectBinaryPackfile2014": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "detectBinaryTagfile2014": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "detectBinaryTagfile2015": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "detectBinaryTagfile": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "detectBinaryTagfileNT6": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "detectCompressedTagfile": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "detectXmlPackfile2014": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "detectXmlTagfile2014": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "detectXmlTagfile": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "detectYamlFile": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkSerialize::Detail::TypeWriterMap::enqueueForWrite": {
        "hkSerialize::Detail::TypeWriterMap *, const hkReflect::Type *": FUNCDEF(
            restype=ctypes.c_int64,  # __int64
            argtypes=[
                ctypes.c_ulonglong,  # hkSerialize::Detail::TypeWriterMap *
                ctypes.c_ulonglong,  # const hkReflect::Type *
            ]
        ),
        "hkSerialize::Detail::TypeWriterMap *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkSerialize::Detail::TypeWriterMap *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "hkHashMap<hkReflect::TypeName,hkTuple<int,hk::optional<uint>,void,void,void,void,void,void>,hkBuiltinContainerAllocator<0,0>>::insert": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # hkHashMap<hkReflect::TypeName,hkTuple<int,hk::optional<unsigned int>,void,void,void,void,void,void>,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # const hkReflect::TypeName *
            ctypes.c_ulonglong,  # hkTuple<int,hk::optional<unsigned int>,void,void,void,void,void,void> *
        ]
    ),
    "hkSerialize::Detail::TypeWriterMap::isWritingQueueEmpty": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::TypeWriterMap *
        ]
    ),
    "hkSerialize::Detail::TypeWriterMap::lookupType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::TypeWriterMap *
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "hkSerialize::Detail::TypeWriterMap::nextToWrite": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSerialize::TypeAndId *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::TypeWriterMap *
            ctypes.c_ulonglong,  # hkSerialize::TypeAndId *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__handleCompendiumReference": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # _BYTE *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__handlePropertyDecls": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # _BYTE *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__inplaceTypeHashes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__inplacePropertyHashes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkSerialize::TagfileReadFormat::Impl::handleDataStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat::Impl *
            ctypes.c_ulonglong,  # hkSerialize::HffStreamReader *
        ]
    ),
    "hkSerialize::TagfileReadFormat::Impl::handleItemsStream": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat::Impl *
            ctypes.c_ulonglong,  # hkSerialize::HffStreamReader *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__handleItems": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # _BYTE *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__handlePatchData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # _BYTE *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__handleNativePointers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # _BYTE *
        ]
    ),
    "hkSerialize::TagfileReadFormat::Impl::readStringsHff<104,1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat::Impl *
            ctypes.c_ulonglong,  # hkSerialize::HffStreamReader *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__readStringsRbv_104_1_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # _BYTE *
        ]
    ),
    "hkSerialize::TagfileReadFormat::Impl::readStringsHff<120,1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat::Impl *
            ctypes.c_ulonglong,  # hkSerialize::HffStreamReader *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__readStringsRbv_120_1_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # _BYTE *
        ]
    ),
    "hkSerialize::TagfileReadFormat::Impl::readStringsHff<136,1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat::Impl *
            ctypes.c_ulonglong,  # hkSerialize::HffStreamReader *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__readStringsRbv_136_1_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # _BYTE *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__handleTypeIdentity_1_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # __int128 *
        ]
    ),
    "hkSerialize::TagfileReadFormat::Impl::handleTypeBodyStream<1>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat::Impl *
            ctypes.c_ulonglong,  # hkSerialize::HffStreamReader *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__handleTypeBody_1_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # __int128 *
        ]
    ),
    "hkSerialize::TagfileReadFormat::Impl::readStringsHff<104,0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat::Impl *
            ctypes.c_ulonglong,  # hkSerialize::HffStreamReader *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__readStringsRbv_104_0_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # _BYTE *
        ]
    ),
    "hkSerialize::TagfileReadFormat::Impl::readStringsHff<120,0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat::Impl *
            ctypes.c_ulonglong,  # hkSerialize::HffStreamReader *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__readStringsRbv_120_0_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # _BYTE *
        ]
    ),
    "hkSerialize::TagfileReadFormat::Impl::readStringsHff<136,0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat::Impl *
            ctypes.c_ulonglong,  # hkSerialize::HffStreamReader *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__readStringsRbv_136_0_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # _BYTE *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__handleTypeIdentity_0_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # __int128 *
        ]
    ),
    "hkSerialize::TagfileReadFormat::Impl::handleTypeBodyStream<0>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat::Impl *
            ctypes.c_ulonglong,  # hkSerialize::HffStreamReader *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__handleTypeBody_0_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # __int128 *
        ]
    ),
    "hkSerialize::TagfileReadFormat::Impl::parseSubsection<17,&TagfileReadFormat::Impl::s_types::HandlerEntry const near * const hkSerialize::TagfileReadFormat::Impl::s_types>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat::Impl *
            ctypes.c_ulonglong,  # hkSerialize::HffStreamReader *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__parseSubsection_17__hkSerialize__TagfileReadFormat__Impl__s_types__0": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # __m128d *
        ]
    ),
    "hkSerialize::TagfileReadFormat::Impl::parseSubsection<2,&TagfileReadFormat::Impl::s_index::HandlerEntry const near * const hkSerialize::TagfileReadFormat::Impl::s_index>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat::Impl *
            ctypes.c_ulonglong,  # hkSerialize::HffStreamReader *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__parseSubsection_2__hkSerialize__TagfileReadFormat__Impl__s_index__0": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # __m128d *
        ]
    ),
    "hkArrayBase<hkTuple<int,uint,void,void,void,void,void,void>>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkSerialize::TagfileWriteFormat::Impl::Parts::Index::Inplace> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkSerialize::HffReaderBase<hkIo::ReadBufferView>::ScopeState>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkTuple<unsigned int,hkArrayView<hkSerialize::TagfileWriteFormat::Impl::Parts::Index::Inplace const >,void,void,void,void,void,void> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkSerialize::Detail::TypeIdentity>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkSerialize::Detail::TypeIdentity> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkSerialize::Detail::TypeIdentity>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkSerialize::Detail::TypeIdentity> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__inplaceTypeHashesImpl__anonymous_namespace___LocalTypeHasher_": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # LocalTypeHasher *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkSerialize::TagfileReadFormat::Impl::Impl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat::Impl *
        ]
    ),
    "hkSerialize::TagfileReadFormat::TagfileReadFormat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat *
        ]
    ),
    "hkSerialize::TagfileReadFormat::Impl::Compendium::~Compendium": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat::Impl::Compendium *
        ]
    ),
    "hkSerialize::Detail::TagfileBundle::~TagfileBundle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::TagfileBundle *
        ]
    ),
    "hkSerialize::TagfileReadFormat::~TagfileReadFormat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat *
        ]
    ),
    "hkSerialize::Detail::TypeIdentity::~TypeIdentity": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::TypeIdentity *
        ]
    ),
    "binaryTagfileCreateRead": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hkSerialize::ReadFormat> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hkSerialize::ReadFormat> *
        ]
    ),
    "hkSerialize::HffReaderBase<hkIo::ReadBuffer &>::enter": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::HffReaderBase<hkIo::ReadBuffer &> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkSerialize::HffReaderBase<hkIo::ReadBufferView>::enter": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::HffReaderBase<hkIo::ReadBufferView> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkSerialize::Detail::TagfileBundle::getAddrAndType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::TagfileBundle *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
            ctypes.c_ulonglong,  # const hkSerialize::Detail::TagfileItem *
        ]
    ),
    "hkSerialize::TagfileReadFormat::getCompendiumState": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSerialize::Detail::CompendiumState *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat *
            ctypes.c_ulonglong,  # hkSerialize::Detail::CompendiumState *
        ]
    ),
    "hkSerialize::Detail::TagfileBundle::getContentsVar": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Var *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::TagfileBundle *
            ctypes.c_ulonglong,  # hkReflect::Var *
        ]
    ),
    "hkSerialize::Detail::TagfileBundle::getItems": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::TagfileBundle *
            ctypes.c_ulonglong,  # hkArray<hkSerialize::Bundle::Item,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkSerialize::Detail::TagfileBundle::getNoteOnPointer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Var *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::TagfileBundle *
            ctypes.c_ulonglong,  # hkReflect::Var *
            ctypes.c_ulonglong,  # const hkReflect::PointerVar *
        ]
    ),
    "hkSerialize::Detail::TagfileArrayImpl::getValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::TagfileArrayImpl *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkReflect::ArrayType *
            ctypes.c_ulonglong,  # hkReflect::ArrayValue *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkSerialize::Detail::TagfilePointerImpl::getValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::TagfilePointerImpl *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkReflect::PointerType *
            ctypes.c_ulonglong,  # hkReflect::Var *
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "hkSerialize::Detail::TagfileStringImpl::getValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::TagfileStringImpl *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkReflect::StringType *
            ctypes.c_ulonglong,  # hkReflect::StringValue *
            ctypes.c_ulonglong,  # const hkReflect::StringType **
        ]
    ),
    "hkSerialize::Detail::TagfileBundle::getVar": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Var *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::TagfileBundle *
            ctypes.c_ulonglong,  # hkReflect::Var *
            ctypes.c_ulonglong,  # const hkSerialize::Detail::TagfileItem *
        ]
    ),
    "hkSerialize__Detail__PatchDataHandlerBase__hkSerialize__TagfileReadFormat__Impl__handlePatchData____34___PatchDataHandler___handleSection": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # _DWORD *
            ctypes.c_ulonglong,  # __int64 *
        ]
    ),
    "hkSerialize__Detail__PatchDataHandlerBase__hkSerialize__TagfileReadFormat__Impl__handlePatchData____11___PatchDataHandler___handleSection": FUNCDEF(
        restype=ctypes.c_ulonglong,  # int *
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # __int64 *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__handleTypeBodyImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # _BYTE *
            ctypes.c_int32,  # int
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__handleTypeIdentityImpl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkSerialize::TagfileReadFormat::inplace": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkViewPtr<hkSerialize::Bundle> *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat *
            ctypes.c_ulonglong,  # hkViewPtr<hkSerialize::Bundle> *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # const hkReflect::TypeReg *
        ]
    ),
    "hkSerialize::TagfileReadFormat::Impl::read": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkViewPtr<hkSerialize::Bundle> *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat::Impl *
            ctypes.c_ulonglong,  # hkViewPtr<hkSerialize::Bundle> *
            ctypes.c_ulonglong,  # hkIo::ReadBuffer *
        ]
    ),
    "hkSerialize::TagfileReadFormat::read": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkViewPtr<hkSerialize::Bundle> *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat *
            ctypes.c_ulonglong,  # hkViewPtr<hkSerialize::Bundle> *
            ctypes.c_ulonglong,  # hkIo::ReadBuffer *
        ]
    ),
    "hkSerialize::TagfileReadFormat::Impl::readType": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat::Impl *
            ctypes.c_ulonglong,  # hkIo::ReadBufferView *
            ctypes.c_ulonglong,  # hkReflect::Type **
            ctypes.c_ulonglong,  # bool *
        ]
    ),
    "hkSerialize::Detail::TagfileBundle::release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::TagfileBundle *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<char,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkPrintableArgs::ToStringFunctionFromType<hkArray<unsigned __int64,hkBuiltinContainerAllocator<0,0>>,void>::s_function": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkSerialize__TagfileReadFormat__Impl__view": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkViewPtr<hkSerialize::Bundle> *
        argtypes=[
            ctypes.c_ulonglong,  # hkViewPtr<hkSerialize::Bundle> *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_ulonglong,  # ImplBase::ViewConfig *
            ctypes.c_ulonglong,  # __m128d *
        ]
    ),
    "hkSerialize::TagfileReadFormat::view": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkViewPtr<hkSerialize::Bundle> *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileReadFormat *
            ctypes.c_ulonglong,  # hkViewPtr<hkSerialize::Bundle> *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "hkArrayBase<hkTuple<hkReflect::Var,int,void,void,void,void,void,void>>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkTuple<hkReflect::Var,int,void,void,void,void,void,void> > *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkPair<hkStringPtr,int>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<hkStringPtr,int> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkTuple<hkReflect::TypeName,hk::optional<uint>,void,void,void,void,void,void>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkTuple<hkReflect::TypeName,hk::optional<unsigned int>,void,void,void,void,void,void> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkSerialize::TagfileWriteFormat::Impl::Parts::Index::ObjectReference>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkTuple<void const *,int,int,void,void,void,void,void> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkHashMap<hkTuple<int,int,void,void,void,void,void,void>,int,hkBuiltinContainerAllocator<0,0>>::findOrInsertKey<hkTuple<int,int,void,void,void,void,void,void> &>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHashBase<hkPair<hkTuple<int,int,void,void,void,void,void,void>,int>,hkBuiltinContainerAllocator<0,0> >::Iterator *
        argtypes=[
            ctypes.c_ulonglong,  # hkHashMap<hkTuple<int,int,void,void,void,void,void,void>,int,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkHashBase<hkPair<hkTuple<int,int,void,void,void,void,void,void>,int>,hkBuiltinContainerAllocator<0,0> >::Iterator *
            ctypes.c_ulonglong,  # hkTuple<int,int,void,void,void,void,void,void> *
            ctypes.c_ulonglong,  # const int *
        ]
    ),
    "hkHashSet<char const *,hkBuiltinContainerAllocator<0,0>>::tryInsert<char const * const &>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHashSet<char const *,hkBuiltinContainerAllocator<0,0> >::TryInsertResult *
        argtypes=[
            ctypes.c_ulonglong,  # hkHashSet<char const *,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkHashSet<char const *,hkBuiltinContainerAllocator<0,0> >::TryInsertResult *
            ctypes.c_char_p,  # const char *const *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::Impl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl *
            ctypes.c_ulonglong,  # hkReflect::TypeCopier *
            ctypes.c_ubyte,  # bool
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # hkSerialize::TagfileWriteFormat::MetaDataOptions
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::Parts::Index::Index": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl::Parts::Index *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::TagfileWriteFormat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::Parts::Type::Type": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl::Parts::Type *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::Compendium::~Compendium": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl::Compendium *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::~Impl": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::Parts::Index::~Index": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl::Parts::Index *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::Parts::Type::InternTab::~InternTab": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl::Parts::Type::InternTab *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::Parts::~Parts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl::Parts *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::Parts::Type::~Type": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl::Parts::Type *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::Parts::Type::addHash": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl::Parts::Type *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::Parts::Index::addVarN": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl::Parts::Index *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "hkSerialize::TagfileWriteFormat::beginBundle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat *
            ctypes.c_ulonglong,  # hkIo::WriteBuffer *
        ]
    ),
    "binaryTagfileCreateWrite": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hkSerialize::WriteFormat> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hkSerialize::WriteFormat> *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::Parts::Type::calcCompendiumSignatureImpl": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl::Parts::Type *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::enableMultiBundle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat *
            ctypes.c_ulonglong,  # _DWORD *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::endBundle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::endBundle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::getCompendiumSignature": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::TagfileTypeWriter::lookupType": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl::TagfileTypeWriter *
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "padUp": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArray<char,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
            ctypes.c_char,  # char
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::useTypeCompendium": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkIo::Detail::WriteBufferAdapter *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl *
            ctypes.c_ulonglong,  # const hkIo::Detail::WriteBufferAdapter *
            ctypes.c_ulonglong,  # const hkIo::Detail::ReadBufferAdapter *
            ctypes.c_ulonglong,  # const hkIo::Detail::ReadBufferAdapter *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::useTypeCompendium": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkIo::Detail::WriteBufferAdapter *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat *
            ctypes.c_ulonglong,  # const hkIo::Detail::WriteBufferAdapter *
            ctypes.c_ulonglong,  # const hkIo::Detail::ReadBufferAdapter *
            ctypes.c_ulonglong,  # const hkIo::Detail::ReadBufferAdapter *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::write": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Var *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat *
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # hkSerialize::IdFromVar *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::writeInplacePatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkReflect::Type *
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::writeInternal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Var *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl *
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # hkReflect::QualifiedType<hkReflect::Type> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkSerialize::IdFromVar *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkSerialize::TagfileWriteFormat::writeNote": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Var *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat *
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # hkSerialize::IdFromVar *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::writeObjectRef": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl *
            ctypes.c_ulonglong,  # hkEndian::EndianType<unsigned int,1> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::Parts::Type::InternTab::writeTo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl::Parts::Type::InternTab *
            ctypes.c_ulonglong,  # hkIo::WriteBuffer *
        ]
    ),
    "hkSerialize::TagfileWriteFormat::Impl::writeTypeSection": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::TagfileWriteFormat::Impl *
            ctypes.c_ulonglong,  # hkSerialize::HffWriter *
            ctypes.c_ulonglong,  # hkIo::WriteBuffer *
        ]
    ),
    "hkSerialize::XmlReadFormat::XmlReadFormat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlReadFormat *
        ]
    ),
    "hkSerialize::Detail::IndexedBundle::~IndexedBundle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::IndexedBundle *
        ]
    ),
    "hkSerialize::Detail::TypeRelocs::~TypeRelocs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::TypeRelocs *
        ]
    ),
    "hkSerialize::Detail::TypeRelocs::addTemplateParam": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::TypeRelocs *
            ctypes.c_int32,  # int
        ]
    ),
    "hkSerialize::XmlReadFormat::Impl::advanceUntilEndBlock": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkXmlStreamParser *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlReadFormat::Impl *
            ctypes.c_ulonglong,  # hkXmlStreamParser *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkSerialize::XmlReadFormat::Impl::AllocationImpl::allocate": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Var *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlReadFormat::Impl::AllocationImpl *
            ctypes.c_ulonglong,  # hkReflect::Var *
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "hkSerialize::XmlReadFormat::Impl::AllocationImpl::allocateForClone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[
            ctypes.c_ulonglong,  # hkReflect::Var *
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "hkSerialize::XmlReadFormat::Impl::AllocationImpl::deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlReadFormat::Impl::AllocationImpl *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "hkSerialize::XmlReadFormat::Impl::ArrayImpl::getValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlReadFormat::Impl::ArrayImpl *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkReflect::ArrayType *
            ctypes.c_ulonglong,  # hkReflect::ArrayValue *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkSerialize::XmlReadFormat::Impl::StringImpl::getValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlReadFormat::Impl::StringImpl *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkReflect::StringType *
            ctypes.c_ulonglong,  # hkReflect::StringValue *
            ctypes.c_ulonglong,  # _QWORD *
        ]
    ),
    "hkSerialize::XmlReadFormat::Impl::parseObject": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkXmlStreamParser *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlReadFormat::Impl *
            ctypes.c_ulonglong,  # hkXmlStreamParser *
            ctypes.c_ulonglong,  # hkXmlStreamParser *
            ctypes.c_char,  # char
        ]
    ),
    "hkSerialize::XmlReadFormat::Impl::parseType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkXmlStreamParser *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlReadFormat::Impl *
            ctypes.c_ulonglong,  # hkXmlStreamParser *
            ctypes.c_ulonglong,  # hkXmlStreamParser *
        ]
    ),
    "hkSerialize::XmlReadFormat::Impl::read": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkViewPtr<hkSerialize::Bundle> *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlReadFormat::Impl *
            ctypes.c_ulonglong,  # hkViewPtr<hkSerialize::Bundle> *
            ctypes.c_ulonglong,  # hkIo::ReadBuffer *
        ]
    ),
    "hkSerialize::XmlReadFormat::read": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkViewPtr<hkSerialize::Bundle> *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlReadFormat *
            ctypes.c_ulonglong,  # hkViewPtr<hkSerialize::Bundle> *
            ctypes.c_ulonglong,  # hkIo::ReadBuffer *
        ]
    ),
    "hkSerialize::XmlReadFormat::Impl::recalcOutstandingTypes": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlReadFormat::Impl *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkSerialize::XmlReadFormat::Impl::recursivelyLoad": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkXmlStreamParser *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlReadFormat::Impl *
            ctypes.c_ulonglong,  # hkXmlStreamParser *
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "hkSerialize::XmlReadFormat::Impl::BundleXml::release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlReadFormat::Impl::BundleXml *
        ]
    ),
    "hkSerialize::XmlReadFormat::view": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkViewPtr<hkSerialize::Bundle> *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlReadFormat *
            ctypes.c_ulonglong,  # hkViewPtr<hkSerialize::Bundle> *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "xmlTagfileCreateRead": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hkSerialize::ReadFormat> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hkSerialize::ReadFormat> *
        ]
    ),
    "hkSerialize::XmlWriteFormat::XmlWriteFormat": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlWriteFormat *
            ctypes.c_ulonglong,  # hkSerialize::TypeWriter *
        ]
    ),
    "hkSerialize::XmlWriteFormat::beginBundle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlWriteFormat *
            ctypes.c_ulonglong,  # hkIo::WriteBuffer *
        ]
    ),
    "hkSerialize::XmlWriteFormat::endBundle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlWriteFormat *
        ]
    ),
    "hkSerialize::XmlWriteFormat::hasVariantArrays": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const hkReflect::Type *
            ctypes.c_ulonglong,  # const hkReflect::RecordType *
        ]
    ),
    "hkSerialize::XmlWriteFormat::XmlFormatter::write": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlWriteFormat::XmlFormatter *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkSerialize::XmlWriteFormat::write": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Var *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlWriteFormat *
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # hkSerialize::IdFromVar *
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "hkSerialize::XmlWriteFormat::TypeWriter::writeAllOutstandingTypes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlWriteFormat::TypeWriter *
            ctypes.c_ulonglong,  # hkSerialize::XmlWriteFormat::XmlFormatter *
        ]
    ),
    "hkSerialize::XmlWriteFormat::writeInternal": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Var *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlWriteFormat *
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkSerialize::IdFromVar *
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "hkSerialize::XmlWriteFormat::writeNote": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Var *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlWriteFormat *
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # hkSerialize::IdFromVar *
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "hkSerialize::XmlWriteFormat::TypeWriter::writeType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkSerialize::TypeAndId *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlWriteFormat::TypeWriter *
            ctypes.c_ulonglong,  # hkSerialize::TypeAndId *
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "hkSerialize::XmlWriteFormat::TypeWriter::writeTypeTag": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlWriteFormat::TypeWriter *
            ctypes.c_ulonglong,  # hkSerialize::XmlWriteFormat::XmlFormatter *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const hkReflect::Type *
            ctypes.c_int32,  # int
        ]
    ),
    "hkSerialize::XmlWriteFormat::writeVariantArraySubtypes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::XmlWriteFormat *
            ctypes.c_ulonglong,  # const hkReflect::Var *
        ]
    ),
    "xmlTagfileCreateWrite": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hkSerialize::WriteFormat> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hkSerialize::WriteFormat> *
        ]
    ),
    "hctClothSetup20151Options::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hctClothSetup20151Options *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkMemoryResourceContainer::_Auto::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "hctClothSetupObjectData20151::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hctClothSetupObjectData20151 *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hctClothSetupClothData20151Options::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hctClothSetupClothData20151Options *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkArrayBase<hctClothSetupObjectData20151 *>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hctClothSetupObjectData20151 *> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkRootLevelContainer::NamedVariant>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkRootLevelContainer::NamedVariant> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hctClothSetupClothData20151Options::hctClothSetupClothData20151Options": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hctClothSetupClothData20151Options *
            ctypes.c_ulonglong,  # const hctClothSetupClothData20151Options *
        ]
    ),
    "hctClothSetupClothData20151Options::~hctClothSetupClothData20151Options": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hctClothSetupClothData20151Options *
        ]
    ),
    "hkMemoryResourceContainer::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceContainer *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkMemoryResourceHandle::getExactType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceHandle *
            ctypes.c_ulonglong,  # hkReflect::Detail::AddrAndType *
        ]
    ),
    "hkXmlStreamParser::hkXmlStreamParser": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkXmlStreamParser *
            ctypes.c_ulonglong,  # hkIo::ReadBuffer *
        ]
    ),
    "hkXmlStreamParser::~hkXmlStreamParser": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkXmlStreamParser *
        ]
    ),
    "hkXmlStreamParser::_advance": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkXmlStreamParser *
        ]
    ),
    "hkXmlStreamParser::_extractAttributes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkXmlStreamParser *
        ]
    ),
    "hkXmlStreamParser::_pushLexeme": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkXmlStreamParser *
        ]
    ),
    "hkXmlStreamParser::advance": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkXmlStreamParser *
        ]
    ),
    "hkXmlStreamParser::decodeString": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkStringView *
        argtypes=[
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkXmlStreamParser::getBlockName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkStringView *
        argtypes=[
            ctypes.c_ulonglong,  # hkXmlStreamParser *
            ctypes.c_ulonglong,  # hkStringView *
        ]
    ),
    "hkXmlStreamParser::getIntAttribute": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkXmlStreamParser *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # _DWORD *
        ]
    ),
    "hkXmlStreamParser::getValue": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkXmlStreamParser *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # hkStringView *
            ctypes.c_ulonglong,  # _OWORD *
        ]
    ),
    "hkXmlStreamParser::hasAttribute": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkXmlStreamParser *
            ctypes.c_ulonglong,  # hkBool *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkXmlStreamParser::parseInt": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkStringView *
        argtypes=[
            ctypes.c_ulonglong,  # const hkStringView *
            ctypes.c_ulonglong,  # __int64 *
            ctypes.c_ulonglong,  # _QWORD *
        ]
    ),
    "hkArrayBase<hkPair<int,hkArray<hkReflect::Type const * *,hkBuiltinContainerAllocator<0,0>>>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkPair<int,hkArray<hkReflect::Type const * *,hkBuiltinContainerAllocator<0,0> > > > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkSerialize::Bundle::Item>::_growCapacityExactly<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkSerialize::Bundle::Item> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkPair<int,hkArray<hkReflect::Type const * *,hkBuiltinContainerAllocator<0,0>>>::hkPair<int,hkArray<hkReflect::Type const * *,hkBuiltinContainerAllocator<0,0>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkPair<int,hkArray<hkReflect::Type const * *,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_ulonglong,  # hkPair<int,hkArray<hkReflect::Type const * *,hkBuiltinContainerAllocator<0,0> > > *
        ]
    ),
    "hkSerialize::Detail::IndexedBundle::IndexedBundle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::IndexedBundle *
        ]
    ),
    "hkSerialize::Detail::FlatTypeRelocs::~FlatTypeRelocs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::FlatTypeRelocs *
        ]
    ),
    "hkSerialize::Detail::IndexedBundle::getContentsVar": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Var *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::IndexedBundle *
            ctypes.c_ulonglong,  # hkReflect::Var *
        ]
    ),
    "hkSerialize::Detail::IndexedBundle::getItems": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::IndexedBundle *
            ctypes.c_ulonglong,  # hkArray<hkSerialize::Bundle::Item,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkSerialize::Detail::IndexedBundle::getNoteOnPointer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkReflect::Var *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::IndexedBundle *
            ctypes.c_ulonglong,  # hkReflect::Var *
            ctypes.c_ulonglong,  # const hkReflect::PointerVar *
        ]
    ),
    "hkSerialize::Detail::IndexedArrayImpl::getValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::IndexedArrayImpl *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkReflect::ArrayType *
            ctypes.c_ulonglong,  # hkReflect::ArrayValue *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkSerialize::Detail::IndexedPointerImpl::getValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::IndexedPointerImpl *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkReflect::PointerType *
            ctypes.c_ulonglong,  # hkReflect::Var *
            ctypes.c_ulonglong,  # _QWORD *
        ]
    ),
    "hkSerialize::Detail::IndexedStringImpl::getValue": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::IndexedStringImpl *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkReflect::StringType *
            ctypes.c_ulonglong,  # hkReflect::StringValue *
            ctypes.c_ulonglong,  # _QWORD *
        ]
    ),
    "hkSerialize::Detail::IndexedPointerImpl::getValueAddress": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::IndexedPointerImpl *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkReflect::PointerType *
        ]
    ),
    "hkSerialize::Detail::IndexedPointerImpl::getValueAs": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const void *
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::IndexedPointerImpl *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const hkReflect::PointerType *
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "hkSerialize::Detail::IndexedBundle::release": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::IndexedBundle *
        ]
    ),
    "hkSerialize::Detail::FlatTypeRelocs::resolve": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::Detail::FlatTypeRelocs *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "hkArrayBase<hkResourceHandle::Link>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkResourceHandle::Link> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkRefPtr<hkMemoryResourceHandle>>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkRefPtr<hkMemoryResourceHandle> > *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkMemoryResourceHandle::ExternalLink>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkMemoryResourceHandle::ExternalLink> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkMemoryResourceContainer::hkMemoryResourceContainer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceContainer *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkMemoryResourceHandle::hkMemoryResourceHandle": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceHandle *
        ]
    ),
    "hkMemoryResourceHandle::addExternalLink": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceHandle *
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkMemoryResourceContainer::afterReflectNew": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceContainer *
        ]
    ),
    "hkMemoryResourceHandle::clearExternalLinks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceHandle *
        ]
    ),
    "hkMemoryResourceContainer::createContainer": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkMemoryResourceContainer *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceContainer *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkMemoryResourceContainer::createResource": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkMemoryResourceHandle *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceContainer *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "hkMemoryResourceContainer::destroyContainer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceContainer *
            ctypes.c_ulonglong,  # hkResourceContainer *
        ]
    ),
    "hkMemoryResourceContainer::destroyResource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceContainer *
            ctypes.c_ulonglong,  # hkResourceHandle *
        ]
    ),
    "hkResourceContainer::findAllResourceRecursively": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkResourceContainer *
            ctypes.c_ulonglong,  # hkArray<hkResourceHandle *,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkMemoryResourceContainer::findContainerByName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkMemoryResourceContainer *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceContainer *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const hkResourceContainer *
        ]
    ),
    "hkMemoryResourceContainer::findResourceByName": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkMemoryResourceHandle *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceContainer *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # const hkReflect::Type *
            ctypes.c_ulonglong,  # const hkResourceHandle *
        ]
    ),
    "hkMemoryResourceContainer::findResourceByType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkMemoryResourceHandle *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceContainer *
            ctypes.c_ulonglong,  # const hkReflect::Type *
            ctypes.c_ulonglong,  # const hkResourceHandle *
        ]
    ),
    "hkMemoryResourceHandle::getExternalLinks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceHandle *
            ctypes.c_ulonglong,  # hkArray<hkResourceHandle::Link,hkBuiltinContainerAllocator<0,0> > *
        ]
    ),
    "hkMemoryResourceHandle::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceHandle *
            ctypes.c_ulonglong,  # hkStringBuf *
        ]
    ),
    "hkMemoryResourceHandle::getObjectType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceHandle *
        ]
    ),
    "hkMemoryResourceContainer::parentTo": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkResourceContainer *
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceContainer *
            ctypes.c_ulonglong,  # hkResourceContainer *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkMemoryResourceHandle::removeExternalLink": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceHandle *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkMemoryResourceHandle::setName": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceHandle *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkMemoryResourceHandle::setObject": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMemoryResourceHandle *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "hkResourceContainer::tryToResolveLinks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkResourceContainer *
            ctypes.c_ulonglong,  # hkResourceMap *
        ]
    ),
    "hkResourceHandle::tryToResolveLinks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkResourceHandle *
            ctypes.c_ulonglong,  # hkResourceMap *
        ]
    ),
    "hkParserBuffer::hkParserBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkParserBuffer *
            ctypes.c_ulonglong,  # hkIo::ReadBuffer *
        ]
    ),
    "hkParserBuffer::~hkParserBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkParserBuffer *
        ]
    ),
    "hkParserBuffer::bufferCommit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkParserBuffer *
        ]
    ),
    "hkParserBuffer::lexemeCommit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkParserBuffer *
        ]
    ),
    "hkParserBuffer::match": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkParserBuffer *
            ctypes.c_ulonglong,  # hkBool *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "hkParserBuffer::matchAndConsume": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkBool *
        argtypes=[
            ctypes.c_ulonglong,  # hkParserBuffer *
            ctypes.c_ulonglong,  # hkBool *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "hkParserBuffer::read": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkParserBuffer *
            ctypes.c_int32,  # int
        ]
    ),
    "hkXmlLexAnalyzer::hkXmlLexAnalyzer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkXmlLexAnalyzer *
            ctypes.c_ulonglong,  # hkIo::ReadBuffer *
        ]
    ),
    "hkXmlLexAnalyzer::~hkXmlLexAnalyzer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkXmlLexAnalyzer *
        ]
    ),
    "hkXmlLexAnalyzer::_lexWhiteSpace": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkXmlLexAnalyzer *
        ]
    ),
    "hkXmlLexAnalyzer::_matchInBrackets": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkXmlLexAnalyzer *
        ]
    ),
    "hkXmlLexAnalyzer::_matchOutsideBrackets": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkXmlLexAnalyzer *
        ]
    ),
    "hkXmlLexAnalyzer::advance": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkXmlLexAnalyzer *
        ]
    ),
    "hkVdb::Detail::ToString::ToString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkVdb::Detail::ToString *
        ]
    ),
    "hkVdbReflect::Var::end": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkVdbReflect::Var::_Iterator *
        argtypes=[
            ctypes.c_ulonglong,  # hkVdbReflect::Var *
            ctypes.c_ulonglong,  # hkVdbReflect::Var::_Iterator *
        ]
    ),
    "hkVdb::Detail::ToString::toString": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkReflect::Var *
            ctypes.c_ulonglong,  # hkStringBuf *
            ctypes.c_ulonglong,  # const hkStringView *
        ]
    ),
    "hkSingletonUtil__init_hkProcessFactory_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hkProcessFactory> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hkProcessFactory> *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hkDisplayGeometry::~hkDisplayGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDisplayGeometry *
        ]
    ),
    "hkDisplayGeometry::getWireframeGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDisplayGeometry *
            ctypes.c_ulonglong,  # hkArrayBase<hkVector4f> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hkDisplaySphere::hkDisplaySphere": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDisplaySphere *
            ctypes.c_ulonglong,  # const hkSphere *
            ctypes.c_int32,  # int
        ]
    ),
    "hkDisplaySphere::buildGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDisplaySphere *
        ]
    ),
    "hkDisplaySphere::getWireframeGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDisplaySphere *
            ctypes.c_ulonglong,  # hkArrayBase<hkVector4f> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hkDisplayCapsule::hkDisplayCapsule": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDisplayCapsule *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkDisplayCapsule::buildGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDisplayCapsule *
        ]
    ),
    "hkDisplayCapsule::getWireframeGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDisplayCapsule *
            ctypes.c_ulonglong,  # hkArrayBase<hkVector4f> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hkDisplayAABB::hkDisplayAABB": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDisplayAABB *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkDisplayAABB::~hkDisplayAABB": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDisplayAABB *
        ]
    ),
    "hkDisplayAABB::buildGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDisplayAABB *
        ]
    ),
    "hkDisplayAABB::getWireframeGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDisplayAABB *
            ctypes.c_ulonglong,  # hkArrayBase<hkVector4f> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hkDisplayWireframe::hkDisplayWireframe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDisplayWireframe *
        ]
    ),
    "hkDisplayWireframe::getWireframeGeometry": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDisplayWireframe *
            ctypes.c_ulonglong,  # hkArrayBase<hkVector4f> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
        ]
    ),
    "hkDebugCommandProcessor::exec": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugCommandProcessor *
            ctypes.c_ulonglong,  # const hkCommand *
        ]
    ),
    "hkDebugCommandProcessor::print": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugCommandProcessor *
            ctypes.c_ulonglong,  # const hkCommand *
            ctypes.c_ulonglong,  # hkOstream *
        ]
    ),
    "hkDebugDisplayHandler::hkDebugDisplayHandler": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplayHandler *
        ]
    ),
    "hkDebugDisplayHandler::addGeometry": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplayHandler *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const hkArrayBase<hkDisplayGeometry *> *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "hkDebugDisplayHandler::addGeometryInstance": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplayHandler *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "hkDebugDisplayHandler::displayFrame": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplayHandler *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const hkQsTransformf *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_ulonglong,  # int *
            ctypes.c_int32,  # int
        ]
    ),
    "hkDebugDisplayHandler::updateGeometryTransform": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplayHandler *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkSingletonUtil__init_hkDebugDisplay_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hkDebugDisplay> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hkDebugDisplay> *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hkDebugDisplay::addGeometry": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplay *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const hkArrayBase<hkDisplayGeometry *> *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "hkDebugDisplay::addGeometryInstance": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplay *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "hkDebugDisplay::addGeometryInstanceEx": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplay *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const struct hkArrayBase<hkGeometryDecorations *> *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "hkDebugDisplay::clearGeometryFlagBits": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplay *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int64,  # __int64
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "hkDebugDisplay::displayFrame": FUNCDEF(
        restype=ctypes.c_ulonglong,  # _DWORD *
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplay *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const hkQsTransformf *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "hkDebugDisplay::flushDisplay": FUNCDEF(
        restype=ctypes.c_ulonglong,  # int *
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplay *
            ctypes.c_ulonglong,  # int *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "hkDebugDisplay::removeCamera": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplay *
            ctypes.c_char_p,  # const char *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkDebugDisplay::removeGeometry": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplay *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkDebugDisplay::setDisplayOptions": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkDebugDisplayHandler::Options *
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplay *
            ctypes.c_ulonglong,  # hkDebugDisplayHandler::Options *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkDebugDisplay::setGeometryAlpha": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplay *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
        ]
    ),
    "hkDebugDisplay::setGeometryColor": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplay *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int64,  # __int64
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkDebugDisplay::setGeometryFlagBits": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplay *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_int64,  # __int64
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "hkDebugDisplay::updateCamera": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplay *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # hkDisplayCamera *
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkDebugDisplay::updateGeometryMaterialData": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplay *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const struct hkArray<hkDisplayMaterialDatas *,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkDebugDisplay::updateGeometryTransform": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplay *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkDebugDisplay::updateGeometryTransformEx": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplay *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const hkMatrix4Impl<float> *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkDebugDisplay::updateGeometryVerts": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDebugDisplay *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const hkArray<hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> >,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_int64,  # __int64
        ]
    ),
    "jediIsInitialized": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[]
    ),
    "scePadGetControllerInformation": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int64,  # __int64
        ]
    ),
    "scePadInit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "scePadOpen": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "scePadRead": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # struct ScePadData *
            ctypes.c_int32,  # int
        ]
    ),
    "scePadSetAngularVelocityDeadbandState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_char,  # char
        ]
    ),
    "scePadSetTiltCorrectionState": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_char,  # char
        ]
    ),
    "scePadSetVibration": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "scePadTerminate": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[]
    ),
    "sub_1427C7220": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "sub_1427C73B0": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int32,  # int
        ]
    ),
    "sub_1427C7470": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char,  # char
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # _BYTE *
            ctypes.c_uint8,  # unsigned __int8
        ]
    ),
    "sub_1427C77D0": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_ulonglong,  # _BYTE *
        ]
    ),
    "sub_1427C84E0": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "sub_1427C8850": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "sub_1427C8A70": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_char,  # char
            ctypes.c_int64,  # __int64
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_char,  # char
        ]
    ),
    "getDeviceByDeviceId": FUNCDEF(
        restype=ctypes.c_ulonglong,  # struct Device *
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "getDeviceByHandle": FUNCDEF(
        restype=ctypes.c_ulonglong,  # struct Device *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_char,  # char
        ]
    ),
    "isBond": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_int16,  # __int16
        ]
    ),
    "isClone": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_int16,  # __int16
        ]
    ),
    "isHunt": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_int16,  # __int16
        ]
    ),
    "isJedi": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_int16,  # __int16
            ctypes.c_int32,  # int
        ]
    ),
    "isPs5GenController": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_int16,  # __int16
            ctypes.c_int16,  # __int16
        ]
    ),
    "sub_1427C9CB0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sub_1427C9D90": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_char,  # char
        ]
    ),
    "sub_1427CA670": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int64,  # __int64
        ]
    ),
    "sub_1427CA6B0": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_uint16,  # unsigned __int16
        ]
    ),
    "sub_1427CA830": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sub_1427CB620": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sub_1427CB8D0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_int32,  # int
        ]
    ),
    "myallocator<std::_Tree_node<std::basic_string<wchar_t,std::char_traits<wchar_t>,myallocator<wchar_t>>,void *>>::deallocate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sub_1427CBC50": FUNCDEF(
        restype=None,  # void
        argtypes=[]
    ),
    "sub_1427CBD70": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sub_1427CBDA0": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int64,  # __int64
        ]
    ),
    "sub_1427CBDE0": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sub_1427CC820": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_int64,  # __int64
        ]
    ),
    "sub_1427CCA70": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # _BYTE *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_char,  # char
        ]
    ),
    "StartAddress": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sub_1427CDE60": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[]
    ),
    "sub_1427CDE90": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_int64,  # __int64
        ]
    ),
    "sub_1427CE130": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
            ctypes.c_uint8,  # unsigned __int8
            ctypes.c_int64,  # __int64
        ]
    ),
    "sub_1427CE440": FUNCDEF(
        restype=ctypes.c_int16,  # __int16
        argtypes=[
            ctypes.c_ulonglong,  # _WORD *
            ctypes.c_int64,  # __int64
            ctypes.c_char,  # char
        ]
    ),
    "sub_1427CE5F0": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "setTimerResolution": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_uint32,  # UINT
        ]
    ),
    "sub_1427CE7D0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # wchar_t *
            ctypes.c_int64,  # __int64
        ]
    ),
    "sub_1427CEC80": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "getCrc32OnBtHid": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char,  # char
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
        ]
    ),
    "hasCrcErrorOnBtHid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_char,  # char
            ctypes.c_ulonglong,  # char *
            ctypes.c_int32,  # int
        ]
    ),
    "sce::Vectormath::Libpad::Aos::Quat::operator*": FUNCDEF(
        restype=ctypes.c_ulonglong,  # float *
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "sub_1427CF170": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_float,  # float
        ]
    ),
    "sce::Vectormath::Libpad::Aos::inverse": FUNCDEF(
        restype=ctypes.c_ulonglong,  # float *
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "sub_1427D0150": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "gbeInit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct GyroBiasEstimatePrm *
        ]
    ),
    "gbeSetup": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # struct GyroBiasEstimatePrm *
            ctypes.c_ulonglong,  # struct SceHidMotionCalibData *
        ]
    ),
    "gbeUpdate": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # struct GyroBiasEstimatePrm *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # struct GyroValF *
            ctypes.c_ulonglong,  # struct GyroValF *
        ]
    ),
    "ChannelManager::BiquadFilters::BiquadFilters": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ChannelManager::BiquadFilters *
        ]
    ),
    "ChannelManagerUpdate::ChannelManagerUpdate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ChannelManagerUpdate *
        ]
    ),
    "SceAudioMasterWork::SceAudioMasterWork": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SceAudioMasterWork *
        ]
    ),
    "MasteringSuiteFX::CreateChannelManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFX *
        ]
    ),
    "CreateMasteringSuiteFX": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPlugin *
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "CreateMasteringSuiteFXParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "MasteringSuiteFX::Execute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # const AkAudioObjects *
        ]
    ),
    "MasteringSuiteFX::GetBufferTick": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFX *
        ]
    ),
    "MasteringSuiteFX::GetPluginInfo": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFX *
            ctypes.c_ulonglong,  # AkPluginInfo *
        ]
    ),
    "MasteringSuiteFX::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # AK::IAkAudioDeviceEffectPluginContext *
            ctypes.c_ulonglong,  # AK::IAkPluginParam *
            ctypes.c_ulonglong,  # const AkAudioFormat *
            ctypes.c_ulonglong,  # const Ak3DAudioSinkCapabilities *
        ]
    ),
    "MasteringSuiteFX::Reset": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFX *
        ]
    ),
    "MasteringSuiteFX::SoftwareDirtyParams": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFX *
            ctypes.c_ubyte,  # bool
        ]
    ),
    "MasteringSuiteFX::SoftwareExecute": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # const AkAudioObjects *
        ]
    ),
    "MasteringSuiteFX::SoftwareFinalizeOutput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # const AkAudioObjects *
        ]
    ),
    "MasteringSuiteFX::SoftwareInit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFX *
            ctypes.c_ulonglong,  # const AkAudioFormat *
        ]
    ),
    "MasteringSuiteFX::SoftwarePrepareInput": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFX *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # const AkAudioObjects *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "MasteringSuiteFX::Term": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFX *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "MasteringSuiteFX::UpdateChannelManager": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFX *
        ]
    ),
    "SceAudioOut2MasteringModuleWork::AllocateMemory": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SceAudioOut2MasteringModuleWork *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const AkBufferAllocateInfo *
        ]
    ),
    "SceAudioOut2MasteringModuleCheckParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const SceAudioOut2MasteringParamsHeader *
            ctypes.c_ulonglong,  # SceAudioOut2MasteringParamsHeader *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "SceAudioOut2MasteringModuleCoreProcess": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # const SceAudioOut2MasteringParamsHeader *
            ctypes.c_ulonglong,  # SceAudioOut2MasteringModuleWork *
            ctypes.c_ulonglong,  # SceAudioOut2MasteringStatesHeader *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "SceAudioOut2MasteringModuleTerm": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # SceAudioOut2MasteringModuleWork *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "MasteringSuiteFXParams::MasteringSuiteFXParams": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFXParams *
        ]
    ),
    "MasteringSuiteFXParams::Clone": FUNCDEF(
        restype=ctypes.c_ulonglong,  # AK::IAkPluginParam *
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "MasteringSuiteFXParams::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFXParams *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "MasteringSuiteFXParams::SetParam": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFXParams *
            ctypes.c_int16,  # __int16
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "MasteringSuiteFXParams::SetParamsBlock": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFXParams *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "MasteringSuiteFXParams::UpdateEqBandFilterMode": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # MasteringSuiteFXParams *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::SIMD::ProcessParallelBuffers<DSP::SIMD::OutOfPlaceBuffer,DSP::SIMD::BiquadMultiCoefs>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::SIMD::OutOfPlaceBuffer *
            ctypes.c_ulonglong,  # DSP::Memories **
            ctypes.c_ulonglong,  # DSP::SIMD::BiquadMultiCoefs *
            ctypes.c_uint32,  # const unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "ChannelManager::FindFreeObjectContext": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # ChannelManager *
        ]
    ),
    "ChannelManager::Init": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # ChannelManager *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
            ctypes.c_ulonglong,  # const ChannelManagerConfig *
        ]
    ),
    "ChannelManager::PostProcessBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ChannelManager *
            ctypes.c_ulonglong,  # const AkAudioObjects *
        ]
    ),
    "ChannelManager::PreProcessBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ChannelManager *
            ctypes.c_ulonglong,  # const AkAudioObjects *
        ]
    ),
    "ChannelManager::ProcessBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ChannelManager *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # AkAudioBuffer *
            ctypes.c_ulonglong,  # const AkAudioObjects *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "DSP::SIMD::ParallelBiquad::ProcessBuffers": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # DSP::SIMD::ParallelBiquad *
            ctypes.c_ulonglong,  # float **
            ctypes.c_ulonglong,  # float **
            ctypes.c_ulonglong,  # DSP::Memories **
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "ChannelManager::ProcessCompressor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ChannelManager *
            ctypes.c_ulonglong,  # ChannelManager::ProcessInfo *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "ChannelManager::ProcessContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ChannelManager *
            ctypes.c_ulonglong,  # ChannelManager::ProcessInfo *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "ChannelManager::ResetContexts": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ChannelManager *
        ]
    ),
    "ChannelManager::Term": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ChannelManager *
            ctypes.c_ulonglong,  # AK::IAkPluginMemAlloc *
        ]
    ),
    "ChannelManager::UpdateConfig": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ChannelManager *
            ctypes.c_ulonglong,  # const ChannelManagerUpdate *
        ]
    ),
    "ChannelManager::UpdateContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # ChannelManager *
            ctypes.c_ulonglong,  # ChannelManager::ChannelContext *
        ]
    ),
    "GetChannelMapping": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "InitializeSystemModuleFunctionEntryPoints": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SceAudioMasterModuleDef *
        ]
    ),
    "sceAudioMasterCheckParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SceAudioMasterParams *
            ctypes.c_ulonglong,  # SceAudioMasterParams *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "sceAudioMasterGetStates": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SceAudioMasterWork *
            ctypes.c_ulonglong,  # SceAudioOut2MasteringStatesHeader *
        ]
    ),
    "sceAudioMasterInit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SceAudioMasterWork *
            ctypes.c_ulonglong,  # const SceAudioMasterInitParams *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sceAudioMasterProcessing": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SceAudioMasterWork *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # SceAudioMasterScratch *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "sceAudioMasterSetModuleBypass": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SceAudioMasterWork *
            ctypes.c_int32,  # const int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "sceAudioMasterSetParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SceAudioMasterWork *
            ctypes.c_ulonglong,  # const SceAudioMasterParams *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sceAudioMasterSystemInit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SceAudioMasterWork *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sceAudioMasterSystemReconfig": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SceAudioMasterWork *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sceAudioMasterTerm": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SceAudioMasterWork *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sceAudioMasterParamEqCheckParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const SceAudioOut2MasteringParamsDescriptor *
            ctypes.c_ulonglong,  # SceAudioOut2MasteringParamsDescriptor *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "sceAudioMasterParamEqInit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const SceAudioMasterModuleConfig *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sceAudioMasterParamEqProcess": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # SceAudioMasterModuleScratch *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "sceAudioMasterParamEqSetParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const SceAudioOut2MasteringParamsDescriptor *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sceAudioMasterParamEqTerm": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sceAudioMasterCompressorCheckParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const SceAudioOut2MasteringParamsDescriptor *
            ctypes.c_ulonglong,  # SceAudioOut2MasteringParamsDescriptor *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "sceAudioMasterCompressorGetStates": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # SceAudioOut2MasteringStatesDescriptor *
        ]
    ),
    "sceAudioMasterCompressorInit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const SceAudioMasterModuleConfig *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sceAudioMasterCompressorProcess": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # SceAudioMasterModuleScratch *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "sceAudioMasterCompressorSetParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const SceAudioOut2MasteringParamsDescriptor *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sceAudioMasterCompressorTerm": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sceAudioMasterLimiterCheckParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const SceAudioOut2MasteringParamsDescriptor *
            ctypes.c_ulonglong,  # SceAudioOut2MasteringParamsDescriptor *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "sceAudioMasterLimiterGetStates": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # SceAudioOut2MasteringStatesDescriptor *
        ]
    ),
    "sceAudioMasterLimiterInit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const SceAudioMasterModuleConfig *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sceAudioMasterLimiterProcess": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # SceAudioMasterModuleScratch *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "sceAudioMasterLimiterSetParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const SceAudioOut2MasteringParamsDescriptor *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sceAudioMasterMasterVolumeCheckParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const SceAudioOut2MasteringParamsDescriptor *
            ctypes.c_ulonglong,  # SceAudioOut2MasteringParamsDescriptor *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "sceAudioMasterMasterVolumeInit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const SceAudioMasterModuleConfig *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sceAudioMasterMasterVolumeProcess": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # SceAudioMasterModuleScratch *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "sceAudioMasterMasterVolumeSetParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const SceAudioOut2MasteringParamsDescriptor *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "FilterProcess": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "FilterSetParams": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # void *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "MultichannelCalcBiQuadTransferCoEffs": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # SceAudioMasterFilterTransferCoEff *
            ctypes.c_int64,  # __int64
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
        ]
    ),
    "paramValidationWithClampFloat": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # const float
            ctypes.c_float,  # const float
        ]
    ),
    "akCompressorAccumulate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "akMultichannelCompressorProcess": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CompressorBandWork *
            ctypes.c_ulonglong,  # SceAudioMasterModuleScratch *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "akMultichannelXChCompressorProcess": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # CompressorBandWork *
            ctypes.c_ulonglong,  # SceAudioMasterModuleScratch *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_float,  # float
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "GetSavedGainsPtrs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # LimiterChannelWork *
            ctypes.c_ulonglong,  # SceAudioMasterModuleScratch *
            ctypes.c_ulonglong,  # float **
        ]
    ),
    "akMultichannelAdvancedLimiterProcess": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # LimiterChannelWork *
            ctypes.c_ulonglong,  # SceAudioMasterModuleScratch *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "akMultichannelHardLimiterProcess": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # LimiterChannelWork *
            ctypes.c_ulonglong,  # SceAudioMasterModuleScratch *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "akMultichannelSoftLimiterProcess": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # LimiterChannelWork *
            ctypes.c_ulonglong,  # SceAudioMasterModuleScratch *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "akMultichannelXChHardLimiterProcess": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # LimiterChannelWork *
            ctypes.c_ulonglong,  # SceAudioMasterModuleScratch *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "akMultichannelXChSoftLimiterProcess": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # LimiterChannelWork *
            ctypes.c_ulonglong,  # SceAudioMasterModuleScratch *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "akMultichannelMasterVolumeProcess": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # MasterVolumeWork *
            ctypes.c_ulonglong,  # const float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "akFilterProcess": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # FilterWork *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_ulonglong,  # float *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "NVSDK_NGX_Parameter_GetD": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # long double *
        ]
    ),
    "NVSDK_NGX_Parameter_GetD3d11Resource": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # struct ID3D11Resource **
        ]
    ),
    "NVSDK_NGX_Parameter_GetD3d12Resource": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # struct ID3D12Resource **
        ]
    ),
    "NVSDK_NGX_Parameter_GetF": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # float *
        ]
    ),
    "NVSDK_NGX_Parameter_GetI": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "NVSDK_NGX_Parameter_GetUI": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "NVSDK_NGX_Parameter_GetULL": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "NVSDK_NGX_Parameter_GetVoidPointer": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # void **
        ]
    ),
    "NVSDK_NGX_Parameter_SetD": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter *
            ctypes.c_char_p,  # const char *
            ctypes.c_double,  # long double
        ]
    ),
    "NVSDK_NGX_Parameter_SetD3d11Resource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # struct ID3D11Resource *
        ]
    ),
    "NVSDK_NGX_Parameter_SetD3d12Resource": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # struct ID3D12Resource *
        ]
    ),
    "NVSDK_NGX_Parameter_SetF": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter *
            ctypes.c_char_p,  # const char *
            ctypes.c_float,  # float
        ]
    ),
    "NVSDK_NGX_Parameter_SetI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "NVSDK_NGX_Parameter_SetUI": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "NVSDK_NGX_Parameter_SetULL": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "NVSDK_NGX_Parameter_SetVoidPointer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter *
            ctypes.c_char_p,  # const char *
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "GetLUIDFromVkPhysicalDevice": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # VkInstance_T *
            ctypes.c_ulonglong,  # struct VkPhysicalDevice_T *
            ctypes.c_ulonglong,  # struct VkDevice_T *
            ctypes.c_ulonglong,  # _LUID *
        ]
    ),
    "NVSDK_NGX_VULKAN_Init_Common": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_ulonglong,  # VkInstance_T *
            ctypes.c_ulonglong,  # struct VkPhysicalDevice_T *
            ctypes.c_ulonglong,  # struct VkDevice_T *
            ctypes.c_ulonglong,  # const NVSDK_NGX_FeatureCommonInfo *
            ctypes.c_uint32,  # NVSDK_NGX_Version
        ]
    ),
    "NVSDK_NGX_VULKAN_ShutdownCommon": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # struct VkDevice_T *
        ]
    ),
    "NVSDK_NGX_VULKAN_AllocateParameters": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter **
        ]
    ),
    "NVSDK_NGX_VULKAN_CreateFeature": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # struct VkCommandBuffer_T *
            ctypes.c_uint32,  # NVSDK_NGX_Feature
            ctypes.c_ulonglong,  # const NVSDK_NGX_Parameter *
            ctypes.c_ulonglong,  # NVSDK_NGX_Handle **
        ]
    ),
    "NVSDK_NGX_VULKAN_CreateFeature1": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # struct VkDevice_T *
            ctypes.c_ulonglong,  # struct VkCommandBuffer_T *
            ctypes.c_uint32,  # NVSDK_NGX_Feature
            ctypes.c_ulonglong,  # const NVSDK_NGX_Parameter *
            ctypes.c_ulonglong,  # NVSDK_NGX_Handle **
        ]
    ),
    "NVSDK_NGX_VULKAN_DestroyParameters": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter *
        ]
    ),
    "NVSDK_NGX_VULKAN_EvaluateFeature": FUNCDEF(
        restype=ctypes.c_uint32,  # NVSDK_NGX_Result
        argtypes=[
            ctypes.c_ulonglong,  # struct VkCommandBuffer_T *
            ctypes.c_ulonglong,  # const NVSDK_NGX_Handle *
            ctypes.c_ulonglong,  # const NVSDK_NGX_Parameter *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['float', 'bool *']}
        ]
    ),
    "NVSDK_NGX_VULKAN_EvaluateFeature_C": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # struct VkCommandBuffer_T *
            ctypes.c_ulonglong,  # const NVSDK_NGX_Handle *
            ctypes.c_ulonglong,  # const NVSDK_NGX_Parameter *
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['float', 'bool *']}
        ]
    ),
    "NVSDK_NGX_VULKAN_GetCapabilityParameters": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter **
        ]
    ),
    "NVSDK_NGX_VULKAN_GetParameters": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Parameter **
        ]
    ),
    "NVSDK_NGX_VULKAN_GetScratchBufferSize": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # NVSDK_NGX_Feature
            ctypes.c_ulonglong,  # const NVSDK_NGX_Parameter *
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "NVSDK_NGX_VULKAN_Init": FUNCDEF(
        restype=ctypes.c_uint32,  # NVSDK_NGX_Result
        argtypes=[
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_ulonglong,  # VkInstance_T *
            ctypes.c_ulonglong,  # struct VkPhysicalDevice_T *
            ctypes.c_ulonglong,  # struct VkDevice_T *
            ctypes.c_ulonglong,  # const NVSDK_NGX_FeatureCommonInfo *
            ctypes.c_uint32,  # NVSDK_NGX_Version
        ]
    ),
    "NVSDK_NGX_VULKAN_ReleaseFeature": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # NVSDK_NGX_Handle *
        ]
    ),
    "NVSDK_NGX_VULKAN_RequiredExtensions": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_char_p,  # const char ***
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_char_p,  # const char ***
        ]
    ),
    "NVSDK_NGX_VULKAN_Shutdown": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "NVSDK_NGX_VULKAN_Shutdown1": FUNCDEF(
        restype=ctypes.c_uint32,  # NVSDK_NGX_Result
        argtypes=[
            ctypes.c_ulonglong,  # struct VkDevice_T *
        ]
    ),
    "ngx__CreateSHFolderFilePath": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "ngx__CreateSystemFilePath": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "ngx__GetSignerInfoTimeStamp": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # _CMSG_SIGNER_INFO *
            ctypes.c_ulonglong,  # _FILETIME *
        ]
    ),
    "ngx__IsCryptSvcPausedOrRunning": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[]
    ),
    "ngx__IsNvidiaSubordinatePublicKey": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # const _CRYPT_BIT_BLOB *
        ]
    ),
    "ngx__IsPeNvidiaSigned": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const _CERT_CONTEXT *
        ]
    ),
    "ngx__IsTrustedLocation": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "ngx__IsWindowsBuildOrGreater": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "ngx__OverrideSignatureVerificationFailure": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "ngx__ParseASN1Timestamp": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "ngx__QueryRegistryValueFromKey": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # HKEY__ *
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned __int8 **
        ]
    ),
    "ngx__TestCertificateChain": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const _CERT_CONTEXT *
            ctypes.c_char_p,  # const char *const *
        ]
    ),
    "ngx__VerifySingleSignature": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "ngx__VerifyTimeStampRFC3161": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # _CMSG_SIGNER_INFO *
            ctypes.c_ulonglong,  # _FILETIME *
        ]
    ),
    "ngx__VerifyTimeStampSignerInfo": FUNCDEF(
        restype=ctypes.c_ubyte,  # _BOOL8
        argtypes=[
            ctypes.c_ulonglong,  # _CMSG_SIGNER_INFO *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # _FILETIME *
        ]
    ),
    "ngx___preloadSystemDlls": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "ngx::nvLoadSignedLibraryExW": FUNCDEF(
        restype=wintypes.HMODULE,  # HMODULE
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # const int
        ]
    ),
    "ngx::nvVerifyFileSignatureW": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "ngx::util::log": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ngx::util::logva": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
            ctypes.c_uint32,  # NVSDK_NGX_Logging_Level
            ctypes.c_ulonglong,  # char *
        ]
    ),
    "ngx::util::logverbose": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "nvFPrintf": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # _iobuf *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "nvSWprintf": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # wchar_t *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "nvSprintf": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # char *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_char_p,  # const char *
        ]
    ),
    "ngx::util::setLogFilename": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_ubyte,  # bool
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # {'return': 'void', 'args': ['const char *', 'NVSDK_NGX_Logging_Level', 'NVSDK_NGX_Feature']}
            ctypes.c_ubyte,  # bool
        ]
    ),
    "ngx__util__DefaultErrorCallback": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_char_p,  # const char *
        ]
    ),
    "XInputGetState_0": FUNCDEF(
        restype=ctypes.c_uint32,  # DWORD
        argtypes=[
            ctypes.c_uint32,  # DWORD
            ctypes.c_ulonglong,  # XINPUT_STATE *
        ]
    ),
    "XInputSetState_0": FUNCDEF(
        restype=ctypes.c_uint32,  # DWORD
        argtypes=[
            ctypes.c_uint32,  # DWORD
            ctypes.c_ulonglong,  # XINPUT_VIBRATION *
        ]
    ),
    "pre_c_initialization": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[]
    ),
    "post_pgo_initialization": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[]
    ),
    "mainCRTStartup": FUNCDEF(
        restype=ctypes.c_uint32,  # unsigned int
        argtypes=[
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "atexit": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # {'return': 'void', 'args': []}
        ]
    ),
    "capture_previous_context": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # _CONTEXT *
        ]
    ),
    "strstr_0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "strchr_0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "strrchr_0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # char *
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
        ]
    ),
    "wcsstr_0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_ulonglong,  # const wchar_t *
        ]
    ),
    "memcpy_0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int32,  # size_t
        ]
    ),
    "memmove_0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int32,  # size_t
        ]
    ),
    "memset_0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # size_t
        ]
    ),
    "memcmp_0": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int32,  # size_t
        ]
    ),
    "memchr_0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # size_t
        ]
    ),
    "wcsrchr_0": FUNCDEF(
        restype=ctypes.c_ulonglong,  # wchar_t *
        argtypes=[
            ctypes.c_ulonglong,  # const wchar_t *
            ctypes.c_wchar,  # wchar_t
        ]
    ),
    "exit_0": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "roundf_0": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "strcmp_0": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_char_p,  # const char *
            ctypes.c_char_p,  # const char *
        ]
    ),
    "fabs_0": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_double,  # double
        ]
    ),
    "sqrt_0": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_double,  # double
        ]
    ),
    "floor_0": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_double,  # double
        ]
    ),
    "ceilf": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "fmodf_0": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "isalpha_0": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "floorf_0": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "sqrtf_0": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "isdigit_0": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_int32,  # int
        ]
    ),
    "atan2f_0": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "cosf_0": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "expf_0": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "logf_0": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "powf_0": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "sinf_0": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "acosf_0": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "atanf_0": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "log10_0": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_double,  # double
        ]
    ),
    "log10f_0": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "cos_0": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_double,  # double
        ]
    ),
    "exp_0": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_double,  # double
        ]
    ),
    "log_0": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_double,  # double
        ]
    ),
    "tanf_0": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "pow_0": FUNCDEF(
        restype=ctypes.c_double,  # double
        argtypes=[
            ctypes.c_double,  # double
            ctypes.c_double,  # double
        ]
    ),
    "hkConcurrent::FixedCapacityWaitFreeMap64::FixedCapacityWaitFreeMap64": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkConcurrent::FixedCapacityWaitFreeMap64 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
        ]
    ),
    "hkConcurrent::FixedCapacityWaitFreeMap64::~FixedCapacityWaitFreeMap64": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkConcurrent::FixedCapacityWaitFreeMap64 *
        ]
    ),
    "hkConcurrent::FixedCapacityWaitFreeMap64::allocateBuckets": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkConcurrent::FixedCapacityWaitFreeMap64 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
        ]
    ),
    "hkConcurrent::FixedCapacityWaitFreeMap64::editKey": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkConcurrent::FixedCapacityWaitFreeMap64 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # const hkFunc<hk::optional<unsigned __int64> __cdecl(hk::optional<unsigned __int64>)> *
        ]
    ),
    "hkConcurrent::FixedCapacityWaitFreeMap64::erase": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # hkConcurrent::FixedCapacityWaitFreeMap64 *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkConcurrent::FixedCapacityWaitFreeMap64::Bucket::lockOutOfLine": FUNCDEF(
        restype=ctypes.c_int64,  # signed __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkConcurrent::FixedCapacityWaitFreeMap64::Bucket *
        ]
    ),
    "hkConcurrent::FixedCapacityWaitFreeMap64::resizeThreadUnsafe": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkConcurrent::FixedCapacityWaitFreeMap64 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_float,  # float
        ]
    ),
    "hkConcurrent::FixedCapacityWaitFreeMap64::tryGet": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # hkConcurrent::FixedCapacityWaitFreeMap64 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_ulonglong,  # unsigned __int64 *
        ]
    ),
    "hkConcurrent::FixedCapacityWaitFreeMap64::tryInsert": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkConcurrent::FixedCapacityWaitFreeMap64 *
            ctypes.c_uint64,  # unsigned __int64
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkAabb::includePoints": {
        "hkAabb *, const hkFloat3 *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkAabb *
                ctypes.c_ulonglong,  # const hkFloat3 *
                ctypes.c_int32,  # int
            ]
        ),
        "hkAabb *, const hkVector4f *, int": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkAabb *
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_int32,  # int
            ]
        ),
    },
    "hkAabb::isValid": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hkAabbUtil::calcAabb": {
        "const hkTransformf *, const hkAabb *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hkTransformf *
                ctypes.c_ulonglong,  # const hkAabb *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "const hkTransformf *, const hkAabb *, const hkSimdFloat32 *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hkTransformf *
                ctypes.c_ulonglong,  # const hkAabb *
                ctypes.c_ulonglong,  # const hkSimdFloat32 *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "const hkFloat3 *, int, const hkTransformf *, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hkFloat3 *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # const hkTransformf *
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
        "const hkVector4f *, int, hkAabb *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # const hkVector4f *
                ctypes.c_int32,  # int
                ctypes.c_ulonglong,  # hkAabb *
            ]
        ),
    },
    "hkStepInfo::set": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkStepInfo *
            ctypes.c_float,  # float
            ctypes.c_float,  # float
        ]
    ),
    "hkIo::ReadBuffer::ReadBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBuffer *
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkIo::ReadBuffer::~ReadBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBuffer *
        ]
    ),
    "hkIo::ReadBuffer::attach": {
        "hkIo::ReadBuffer *, const hkIo::Detail::ReadBufferAdapter *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkIo::ReadBuffer *
                ctypes.c_ulonglong,  # const hkIo::Detail::ReadBufferAdapter *
            ]
        ),
        "hkIo::ReadBuffer *, const void *, __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkIo::ReadBuffer *
                ctypes.c_ulonglong,  # const void *
                ctypes.c_int64,  # __int64
            ]
        ),
    },
    "hkIo::ReadBuffer::getReadLimit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBuffer *
        ]
    ),
    "hkIo::ReadBuffer::implPrefetch": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBuffer *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkIo::ReadBuffer::implRead": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBuffer *
            ctypes.c_ulonglong,  # void *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkIo::ReadBuffer::implSkip": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBuffer *
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkIo::ReadBuffer::setReadLimit": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkIo::ReadBuffer *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkLineSegmentUtil::capsuleCapsuleManifold": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_float,  # float
            ctypes.c_ulonglong,  # hkContactPoint *
        ]
    ),
    "hkUnionFind::addEdge": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkUnionFind *
            ctypes.c_int64,  # __int64
            ctypes.c_int64,  # __int64
        ]
    ),
    "hkUnionFind::collapseTree": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkUnionFind *
        ]
    ),
    "hkUnionFind::findRootOfNode": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkUnionFind *
            ctypes.c_int32,  # int
        ]
    ),
    "hkUnionFind::sortByGroupIdUnchecked": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkArrayView<int const > *
        argtypes=[
            ctypes.c_ulonglong,  # hkUnionFind *
            ctypes.c_ulonglong,  # hkArrayView<int const > *
            ctypes.c_ulonglong,  # hkArrayView<int> *
            ctypes.c_ulonglong,  # __int64 *
        ]
    ),
    "hkPrimaryCommandDispatcher::hkPrimaryCommandDispatcher": {
        "hkPrimaryCommandDispatcher *, hkPrimaryCommandDispatcher *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkPrimaryCommandDispatcher *
                ctypes.c_ulonglong,  # hkPrimaryCommandDispatcher *
            ]
        ),
        "hkPrimaryCommandDispatcher *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkPrimaryCommandDispatcher *
            ]
        ),
    },
    "hkSecondaryCommandDispatcher::hkSecondaryCommandDispatcher": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSecondaryCommandDispatcher *
        ]
    ),
    "hkPrimaryCommandDispatcher::~hkPrimaryCommandDispatcher": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkPrimaryCommandDispatcher *
        ]
    ),
    "hkSecondaryCommandDispatcher::~hkSecondaryCommandDispatcher": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSecondaryCommandDispatcher *
        ]
    ),
    "hkPrimaryCommandDispatcher::exec": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkPrimaryCommandDispatcher *
            ctypes.c_ulonglong,  # const hkCommand *
        ]
    ),
    "hkPrimaryCommandDispatcher::registerDispatcher": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkPrimaryCommandDispatcher *
            ctypes.c_uint32,  # hkCommand::PrimaryType
            ctypes.c_ulonglong,  # hkSecondaryCommandDispatcher *
        ]
    ),
    "hkPreciseTransform::setMul": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkPreciseTransform *
            ctypes.c_ulonglong,  # const hkPreciseTransform *
            ctypes.c_ulonglong,  # const hkTransformf *
        ]
    ),
    "hkArrayBase<hkParallel::IteratorContext::Queue>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkParallel::IteratorContext::Queue> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkParallel::CoordinatorContext::CoordinatorContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkParallel::CoordinatorContext *
            ctypes.c_ulonglong,  # hkSemaphore *
            ctypes.c_ulonglong,  # hkSemaphore *
            ctypes.c_int32,  # int
        ]
    ),
    "hkParallel::IteratorContext::addMoreItems": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkParallel::IteratorContext *
            ctypes.c_int32,  # int
        ]
    ),
    "hkParallel::Coordinator::barrier": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkParallel::Coordinator *
        ]
    ),
    "hkParallel::IteratorContext::getItems": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkParallel::Iterator *
        argtypes=[
            ctypes.c_ulonglong,  # hkParallel::IteratorContext *
            ctypes.c_ulonglong,  # hkParallel::Iterator *
            ctypes.c_ulonglong,  # _DWORD *
        ]
    ),
    "hkParallel::IteratorContext::initThreadInfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkParallel::IteratorContext *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkParallel::Coordinator::isLastSingle": FUNCDEF(
        restype=ctypes.c_ubyte,  # bool
        argtypes=[
            ctypes.c_ulonglong,  # hkParallel::Coordinator *
        ]
    ),
    "hkParallel::CoordinatorWithIterator::parallelFor": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkParallel::Iterator *
        argtypes=[
            ctypes.c_ulonglong,  # hkParallel::CoordinatorWithIterator *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkParallel::IteratorContext::refillLocalQueueAndGetItems": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkParallel::IteratorContext *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # unsigned int *
        ]
    ),
    "hkParallel::IteratorContext::resetIterator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkParallel::IteratorContext *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkSingletonUtil__init_hkConcurrency_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<hkConcurrency> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<hkConcurrency> *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "hkConcurrency::getMaxNumThreads": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkConcurrency *
        ]
    ),
    "hkFourTransposedPointsImpl<float>::getOriginalVertices<hkFloat3>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkFourTransposedPointsImpl<float> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkFloat3 *
        ]
    ),
    "hkFourTransposedPointsImpl<float>::getOriginalVertices<hkVector4f>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkFourTransposedPointsImpl<float> *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkPackedVector3::pack": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkPackedVector3 *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hkDynamicBlockStreamAllocator::hkDynamicBlockStreamAllocator": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDynamicBlockStreamAllocator *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # hkDynamicBlockStreamAllocator::FreePolicy
        ]
    ),
    "hkDynamicBlockStreamAllocator::blockAllocBatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDynamicBlockStreamAllocator *
            ctypes.c_ulonglong,  # hkBlockStream::Block **
            ctypes.c_int32,  # int
        ]
    ),
    "hkDynamicBlockStreamAllocator::blockFreeBatch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDynamicBlockStreamAllocator *
            ctypes.c_ulonglong,  # hkBlockStream::Block **
            ctypes.c_int32,  # int
        ]
    ),
    "hkDynamicBlockStreamAllocator::expand": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDynamicBlockStreamAllocator *
            ctypes.c_int32,  # int
        ]
    ),
    "hkDynamicBlockStreamAllocator::freeAllRemainingAllocations": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDynamicBlockStreamAllocator *
        ]
    ),
    "hkDynamicBlockStreamAllocator::getBytesUsed": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDynamicBlockStreamAllocator *
        ]
    ),
    "hkDynamicBlockStreamAllocator::getMaxBytesUsed": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkDynamicBlockStreamAllocator *
        ]
    ),
    "hkDynamicBlockStreamAllocator::getMemoryStatistics": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDynamicBlockStreamAllocator *
            ctypes.c_ulonglong,  # hkMemory::AllocatorStats *
        ]
    ),
    "hkDynamicBlockStreamAllocator::releaseUnusedBlocks": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkDynamicBlockStreamAllocator *
        ]
    ),
    "hkOptionalComponent::addToList": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkOptionalComponent *
        ]
    ),
    "hkOptionalComponent::request": FUNCDEF(
        restype=ctypes.c_ulonglong,  # int *
        argtypes=[
            ctypes.c_ulonglong,  # hkOptionalComponent *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hkMatrix4Impl<float>::setInverse": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkMatrix4Impl<float> *
        argtypes=[
            ctypes.c_ulonglong,  # hkMatrix4Impl<float> *
            ctypes.c_ulonglong,  # const hkMatrix4Impl<float> *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # _OWORD *
        ]
    ),
    "hkMatrix4Impl<float>::sub": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkMatrix4Impl<float> *
            ctypes.c_ulonglong,  # const hkMatrix4Impl<float> *
        ]
    ),
    "hkLargeIntComputeDiv32<256,hkInt256>": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const hkInt256 *
            ctypes.c_ulonglong,  # const hkInt256 *
        ]
    ),
    "hkInt256::addMul": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkInt256 *
            ctypes.c_ulonglong,  # const hkInt128 *
            ctypes.c_ulonglong,  # const hkInt128 *
        ]
    ),
    "hkInt256::computeDiv32": FUNCDEF(
        restype=ctypes.c_int32,  # int
        argtypes=[
            ctypes.c_ulonglong,  # const hkInt256 *
            ctypes.c_ulonglong,  # const hkInt256 *
        ]
    ),
    "hkLargeIntImpl::countLeadingZeros": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkLargeIntImpl::countTrailingZeros": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkLargeIntDivideUnsigned": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_ulonglong,  # unsigned int *
            ctypes.c_int32,  # int
        ]
    ),
    "hkLargeIntImpl::multiplyUnsigned": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_ulonglong,  # const unsigned __int64 *
            ctypes.c_ulonglong,  # unsigned __int64 *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkInt256::setAdd": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkInt256 *
            ctypes.c_ulonglong,  # const hkInt256 *
            ctypes.c_ulonglong,  # const hkInt256 *
        ]
    ),
    "hkInt128::setGreatestCommonDivisor": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkInt128 *
            ctypes.c_ulonglong,  # const hkInt128 *
            ctypes.c_ulonglong,  # const hkInt128 *
        ]
    ),
    "hkInt256::setMul": {
        "hkInt256 *, const hkInt128 *, const hkInt128 *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkInt256 *
                ctypes.c_ulonglong,  # const hkInt128 *
                ctypes.c_ulonglong,  # const hkInt128 *
            ]
        ),
        "hkInt256 *, const hkInt128 *, __int64": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hkInt256 *
                ctypes.c_ulonglong,  # const hkInt128 *
                ctypes.c_int64,  # __int64
            ]
        ),
    },
    "hkInt256::setNeg": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkInt256 *
            ctypes.c_ulonglong,  # const hkInt256 *
        ]
    ),
    "hkInt128::setUnsignedDiv": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkInt128 *
            ctypes.c_ulonglong,  # const hkInt128 *
            ctypes.c_ulonglong,  # const hkInt128 *
        ]
    ),
    "hk1AxisSweep::AabbInt::set": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hk1AxisSweep::AabbInt *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_int32,  # int
        ]
    ),
    "hk1AxisSweep::sortAabbs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hk1AxisSweep::AabbInt *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkArrayBase<hkRadixSort::SortDataT<unsigned int,unsigned int> > *
            ctypes.c_ulonglong,  # hkArrayBase<hk1AxisSweep::AabbInt> *
        ]
    ),
    "hkUintVle::read": FUNCDEF(
        restype=ctypes.c_uint64,  # unsigned __int64
        argtypes=[
            ctypes.c_ulonglong,  # const unsigned __int8 *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hkUintVle::write": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkUintVle *
            ctypes.c_ulonglong,  # unsigned __int8 *
        ]
    ),
    "hkHashMap<hkReflect::Type const *,hkReflect::Type *,hkBuiltinContainerAllocator<0,0>>::findOrInsertKey<hkReflect::Type const * &>": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkHashBase<hkPair<hkReflect::Type const *,hkReflect::Type *>,hkBuiltinContainerAllocator<0,0> >::Iterator *
        argtypes=[
            ctypes.c_ulonglong,  # hkHashMap<hkReflect::Type const *,hkReflect::Type *,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_ulonglong,  # hkHashBase<hkPair<hkReflect::Type const *,hkReflect::Type *>,hkBuiltinContainerAllocator<0,0> >::Iterator *
            ctypes.c_ulonglong,  # const hkReflect::Type **
            ctypes.c_ulonglong,  # hkReflect::Type *const *
        ]
    ),
    "hkHashMap<char const *,hkReflect::Type *,hkBuiltinContainerAllocator<0,0>>::insert": FUNCDEF(
        restype=ctypes.c_char,  # char
        argtypes=[
            ctypes.c_ulonglong,  # hkHashMap<char const *,hkReflect::Type *,hkBuiltinContainerAllocator<0,0> > *
            ctypes.c_char_p,  # const char *const *
            ctypes.c_ulonglong,  # hkReflect::Type **
        ]
    ),
    "hkSerialize::HffWriter::closeScope": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::HffWriter *
        ]
    ),
    "hkSerialize::HffWriter::openScope": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::HffWriter *
            ctypes.c_uint32,  # hkSerialize::HffBase::ScopeType
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkSerialize::HffWriter::writeLeaf": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkSerialize::HffWriter *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_ulonglong,  # const void *
            ctypes.c_int32,  # int
        ]
    ),
    "hkSingletonUtil__init_ReflectStrings_": FUNCDEF(
        restype=ctypes.c_ulonglong,  # hkRefPtr<ReflectStrings> *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefPtr<ReflectStrings> *
            ctypes.c_ulonglong,  # hkReferencedObject **
        ]
    ),
    "ReflectStrings::intern": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # ReflectStrings *
            ctypes.c_char_p,  # const char *
            ctypes.c_int32,  # int
            ctypes.c_ubyte,  # bool
        ]
    ),
    "hkRefVariant::getType": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hkReflect::Type *
        argtypes=[
            ctypes.c_ulonglong,  # hkRefVariant *
        ]
    ),
    "hkRefVariant::set": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkRefVariant *
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # const hkReflect::Type *
        ]
    ),
    "hkArrayBase<hkAttrData::FieldData>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkAttrData::FieldData> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkLinearBuffer::blockAlloc": FUNCDEF(
        restype=ctypes.c_ulonglong,  # void *
        argtypes=[
            ctypes.c_ulonglong,  # hkLinearBuffer *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hkRadixSortDetail::Histogram<256>>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hkRadixSortDetail::Histogram<256> > *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkRadixSortDetail::SortSharedData::SortSharedData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkRadixSortDetail::SortSharedData *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkRadixSortDetail::SortSharedData::~SortSharedData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkRadixSortDetail::SortSharedData *
        ]
    ),
    "hknpCollisionQueryContext::hknpCollisionQueryContext": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
        ]
    ),
    "hknpAnyHitCollector::addHit": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpAnyHitCollector *
            ctypes.c_ulonglong,  # const hknpCollisionResult *
        ]
    ),
    "hkArrayBase<hknpSimdTreeBroadPhaseTasks::CollideTrees>::_growCapacity<hkBuiltinContainerAllocatorBase>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpSimdTreeBroadPhaseTasks::CollideTrees> *
            ctypes.c_ulonglong,  # const hkBuiltinContainerAllocatorBase *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpSimdTreeBroadPhaseTasks::CollideTrees>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpSimdTreeBroadPhaseTasks::CollideTrees> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkArrayBase<hknpSimdTreeBroadPhaseInternal::ThreadCollisionPairs>::_growCapacity<hkContainerAllocator>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkArrayBase<hknpSimdTreeBroadPhaseInternal::ThreadCollisionPairs> *
            ctypes.c_ulonglong,  # const hkContainerAllocator *
            ctypes.c_uint64,  # unsigned __int64
        ]
    ),
    "hkcdSimdTree::aabbCast<QueryNarrowPhaseProcessing::CastAabbImpl<hknpBroadPhaseQueryCollector *>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdRay *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # QueryNarrowPhaseProcessing::CastAabbImpl<hknpBroadPhaseQueryCollector *> *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::aabbCast<QueryNarrowPhaseProcessing::CastAabbImpl<hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdRay *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # QueryNarrowPhaseProcessing::CastAabbImpl<hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::aabbCast<QueryNarrowPhaseProcessing::CastShapeImpl>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdRay *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # QueryNarrowPhaseProcessing::CastShapeImpl *
            ctypes.c_ubyte,  # bool
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTreeUtils::aabbOverlaps_Trampoline<0,QueryNarrowPhaseProcessing::QueryAabbImpl<hknpBroadPhaseQueryCollector *>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdSimdTree::TransposedAabb *
            ctypes.c_ulonglong,  # QueryNarrowPhaseProcessing::QueryAabbImpl<hknpBroadPhaseQueryCollector *> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTreeUtils::aabbOverlaps_Trampoline<0,QueryNarrowPhaseProcessing::QueryAabbImpl<hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdSimdTree::TransposedAabb *
            ctypes.c_ulonglong,  # QueryNarrowPhaseProcessing::QueryAabbImpl<hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTreeUtils::aabbOverlaps_Trampoline<0,QueryNarrowPhaseProcessing::QueryAabbImpl<hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # const hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkcdSimdTree::TransposedAabb *
            ctypes.c_ulonglong,  # QueryNarrowPhaseProcessing::QueryAabbImpl<hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromAabb<QueryNarrowPhaseProcessing::GetClosestPointsImplWithQueryContext>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # QueryNarrowPhaseProcessing::GetClosestPointsImplWithQueryContext *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromAabbOrdered<QueryNarrowPhaseProcessing::GetClosestPointsImplWithQueryContext>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # QueryNarrowPhaseProcessing::GetClosestPointsImplWithQueryContext *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPoint<QueryNarrowPhaseProcessing::GetClosestPointsImpl>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # QueryNarrowPhaseProcessing::GetClosestPointsImpl *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPoint<QueryNarrowPhaseProcessing::QueryPointsImpl>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # QueryNarrowPhaseProcessing::QueryPointsImpl *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPointOrdered<QueryNarrowPhaseProcessing::QueryPointsImpl>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # QueryNarrowPhaseProcessing::QueryPointsImpl *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::closestFromPointParticles<QueryNarrowPhaseProcessing::GetClosestPointsImpl>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkVector4f *
            ctypes.c_ulonglong,  # const hkSimdFloat32 *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
            ctypes.c_ulonglong,  # QueryNarrowPhaseProcessing::GetClosestPointsImpl *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hknpSimdTreeBroadPhaseInternal::collideTreesBranches<TreeOverlapsBufferedPairCollector<1>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhaseInternal *
            ctypes.c_ulonglong,  # TreeOverlapsBufferedPairCollector<1> *
            ctypes.c_uint32,  # unsigned int
            ctypes.c_uint32,  # unsigned int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdSimdTree::genericProcess<DisableBodyFunctor &>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # DisableBodyFunctor *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::outsideOfAabb<QueryNarrowPhaseProcessing::QueryAabbImpl<hknpBroadPhaseQueryCollector *>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # QueryNarrowPhaseProcessing::QueryAabbImpl<hknpBroadPhaseQueryCollector *> *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hkcdSimdTree::outsideOfAabb<QueryNarrowPhaseProcessing::QueryAabbImpl<hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0>>>>": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # QueryNarrowPhaseProcessing::QueryAabbImpl<hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_uint32,  # unsigned int
        ]
    ),
    "hknpSimdTreeBroadPhase::Layer::Layer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase::Layer *
        ]
    ),
    "hknpSimdTreeBroadPhase::hknpSimdTreeBroadPhase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
            ctypes.c_ulonglong,  # hknpBodyManager *
            ctypes.c_ulonglong,  # const hknpBroadPhaseConfig *
        ]
    ),
    "hknpSimdTreeBroadPhase::~hknpSimdTreeBroadPhase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
        ]
    ),
    "hknpSimdTreeBroadPhaseInternal::~hknpSimdTreeBroadPhaseInternal": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhaseInternal *
        ]
    ),
    "hknpSimdTreeBroadPhase::addBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # hknpBroadPhase::AdditionModeHint
        ]
    ),
    "TreeOverlapsBufferedPairCollector<1>::addPairs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TreeOverlapsBufferedPairCollector<1> *
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_int32,  # int
        ]
    ),
    "TreeOverlapsBufferedPairCollector<0>::addPairs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # TreeOverlapsBufferedPairCollector<0> *
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_ulonglong,  # const unsigned int *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpSimdTreeBroadPhase::castAabb": {
        "hknpSimdTreeBroadPhase *, const hknpAabbCastQuery *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbCastQuery *
                ctypes.c_ulonglong,  # hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *
            ]
        ),
        "hknpSimdTreeBroadPhase *, const hknpAabbCastQuery *, hknpBroadPhaseQueryCollector *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbCastQuery *
                ctypes.c_ulonglong,  # hknpBroadPhaseQueryCollector *
            ]
        ),
    },
    "hknpSimdTreeBroadPhase::castRay": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
            ctypes.c_ulonglong,  # const hknpRayCastQuery *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
        ]
    ),
    "hknpSimdTreeBroadPhase::castShape": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
            ctypes.c_ulonglong,  # const hknpShapeCastQuery *
            ctypes.c_ulonglong,  # const hkRotationImpl<float> *
            ctypes.c_ulonglong,  # const hknpCollisionQueryDispatcher *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
        ]
    ),
    "hknpSimdTreeBroadPhase::Layer::clear": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase::Layer *
        ]
    ),
    "hknpSimdTreeBroadPhaseInternal::collideBranch": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # void *
            ctypes.c_ulonglong,  # hkcdSimdTree *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpSimdTreeBroadPhase::copyFrom": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
            ctypes.c_ulonglong,  # const hknpBroadPhase *
        ]
    ),
    "createSimdTreeBroadPhase": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpWorld *
            ctypes.c_ulonglong,  # const hknpBroadPhaseConfig *
        ]
    ),
    "hknpSimdTreeBroadPhase::findAllPairs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # hkTypedBlockStream<hknpBodyIndexPair>::Writer *
        ]
    ),
    "hknpSimdTreeBroadPhase::findNewPairs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
            ctypes.c_ulonglong,  # const hknpSimulationThreadContext *
            ctypes.c_ulonglong,  # const hkAabbFloat3 *
            ctypes.c_ulonglong,  # hkTypedBlockStream<hknpBodyIndexPair>::Writer *
        ]
    ),
    "hknpSimdTreeBroadPhase::getClosestPoints": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # const hknpCollisionQueryDispatcher *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpSimdTreeBroadPhase::getClosestPointsAdvanced": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpSimdTreeBroadPhase::getClosestPointsParticles": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
            ctypes.c_ulonglong,  # hknpCollisionQueryContext *
            ctypes.c_ulonglong,  # const hknpClosestPointsQuery *
            ctypes.c_ulonglong,  # const hkTransformf *
            ctypes.c_ulonglong,  # hknpParticlesStaticCollector *
        ]
    ),
    "hknpSimdTreeBroadPhase::getConfig": FUNCDEF(
        restype=ctypes.c_ulonglong,  # const hknpBroadPhaseConfig *
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
        ]
    ),
    "hknpSimdTreeBroadPhaseTasks::CollideTrees::getDebugAttributes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhaseTasks::CollideTrees *
            ctypes.c_ulonglong,  # hkTask::DebugAttributes *
        ]
    ),
    "hknpSimdTreeBroadPhaseTasks::PrepareTreeBuildData::getDebugAttributes": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhaseTasks::PrepareTreeBuildData *
            ctypes.c_ulonglong,  # hkTask::DebugAttributes *
        ]
    ),
    "hknpSimdTreeBroadPhase::getExtents": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
            ctypes.c_ulonglong,  # hkAabb *
        ]
    ),
    "hknpSimdTreeBroadPhaseTasks::CollideTrees::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhaseTasks::CollideTrees *
        ]
    ),
    "hknpSimdTreeBroadPhaseTasks::CopyAabbsAndMarkActiveBodiesDirty::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhaseTasks::CopyAabbsAndMarkActiveBodiesDirty *
        ]
    ),
    "hknpSimdTreeBroadPhaseTasks::Finalize::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhaseTasks::Finalize *
        ]
    ),
    "hknpSimdTreeBroadPhaseTasks::PrepareTreeBuildData::getName": FUNCDEF(
        restype=ctypes.c_char_p,  # const char *
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhaseTasks::PrepareTreeBuildData *
        ]
    ),
    "hknpSimdTreeBroadPhase::markBodiesDirty": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_uint32,  # hknpBroadPhase::UpdateModeHint
        ]
    ),
    "hknpSimdTreeBroadPhaseInternal::prepareTreeBuildData": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhaseInternal *
            ctypes.c_ulonglong,  # const hkTask::Input *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpSimdTreeBroadPhaseTasks::CollideTrees::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhaseTasks::CollideTrees *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpSimdTreeBroadPhaseTasks::CopyAabbsAndMarkActiveBodiesDirty::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhaseTasks::CopyAabbsAndMarkActiveBodiesDirty *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpSimdTreeBroadPhaseTasks::Finalize::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhaseTasks::Finalize *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "hknpSimdTreeBroadPhaseTasks::PrepareTreeBuildData::process": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhaseTasks::PrepareTreeBuildData *
            ctypes.c_ulonglong,  # const hkTask::Input *
        ]
    ),
    "QueryNarrowPhaseProcessing::QueryAabbImpl<hknpBroadPhaseQueryCollector *>::processSimdTreeLeaves": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # QueryNarrowPhaseProcessing::QueryAabbImpl<hknpBroadPhaseQueryCollector *> *
            ctypes.c_ulonglong,  # const hkIntVector *
            ctypes.c_int32,  # int
        ]
    ),
    "QueryNarrowPhaseProcessing::QueryAabbImpl<hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0>>>::processSimdTreeLeaves": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # QueryNarrowPhaseProcessing::QueryAabbImpl<hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_ulonglong,  # const hkIntVector *
            ctypes.c_int32,  # int
        ]
    ),
    "QueryNarrowPhaseProcessing::QueryAabbImpl<hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0>>>::processSimdTreeLeaves": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # QueryNarrowPhaseProcessing::QueryAabbImpl<hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > > *
            ctypes.c_ulonglong,  # const hkIntVector *
            ctypes.c_int32,  # int
        ]
    ),
    "QueryNarrowPhaseProcessing::GetClosestPointsImpl::processSimdTreeLeaves": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # QueryNarrowPhaseProcessing::GetClosestPointsImpl *
            ctypes.c_ulonglong,  # const hkIntVector *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # const hkAabb *
            ctypes.c_ulonglong,  # hkSimdFloat32 *
        ]
    ),
    "hknpSimdTreeBroadPhase::queryAabb": {
        "hknpSimdTreeBroadPhase *, const hknpAabbQuery *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *
            ]
        ),
        "hknpSimdTreeBroadPhase *, const hknpAabbQuery *, hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *
            ]
        ),
        "hknpSimdTreeBroadPhase *, const hknpAabbQuery *, hknpBroadPhaseQueryCollector *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # hknpBroadPhaseQueryCollector *
            ]
        ),
    },
    "hknpSimdTreeBroadPhase::queryAabbNmp": {
        "hknpSimdTreeBroadPhase *, const hknpAabbQuery *, const hkAabb *, hkAabb *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hkAabb *
                ctypes.c_ulonglong,  # hkAabb *
                ctypes.c_ulonglong,  # hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *
            ]
        ),
        "hknpSimdTreeBroadPhase *, const hknpAabbQuery *, const hkAabb *, hkAabb *, hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hkAabb *
                ctypes.c_ulonglong,  # hkAabb *
                ctypes.c_ulonglong,  # hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *
            ]
        ),
        "hknpSimdTreeBroadPhase *, const hknpAabbQuery *, const hkAabb *, hkAabb *, hknpBroadPhaseQueryCollector *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # const hkAabb *
                ctypes.c_ulonglong,  # hkAabb *
                ctypes.c_ulonglong,  # hknpBroadPhaseQueryCollector *
            ]
        ),
    },
    "hknpSimdTreeBroadPhase::queryOutsideOfAabb": {
        "hknpSimdTreeBroadPhase *, const hknpAabbQuery *, hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *
            ]
        ),
        "hknpSimdTreeBroadPhase *, const hknpAabbQuery *, hknpBroadPhaseQueryCollector *": FUNCDEF(
            restype=None,  # void
            argtypes=[
                ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
                ctypes.c_ulonglong,  # const hknpAabbQuery *
                ctypes.c_ulonglong,  # hknpBroadPhaseQueryCollector *
            ]
        ),
    },
    "hknpSimdTreeBroadPhase::queryPoint": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
            ctypes.c_ulonglong,  # const hknpPointQuery *
            ctypes.c_ulonglong,  # const hknpCollisionQueryDispatcher *
            ctypes.c_ulonglong,  # hknpCollisionQueryCollector *
        ]
    ),
    "hknpSimdTreeBroadPhase::removeBodies": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpSimdTreeBroadPhase::setBodyBuffer": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
            ctypes.c_ulonglong,  # hknpBody *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpSimdTreeBroadPhase::translate": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
            ctypes.c_ulonglong,  # const hkVector4f *
        ]
    ),
    "hknpSimdTreeBroadPhase::updateBodyAabbs": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhase *
            ctypes.c_ulonglong,  # const hknpBodyIndex *
            ctypes.c_int32,  # int
        ]
    ),
    "hknpSimdTreeBroadPhaseInternal::updatePointIdMapping": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpSimdTreeBroadPhaseInternal *
        ]
    ),
    "hknpCompressedMeshShapeCinfo::hknpCompressedMeshShapeCinfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeCinfo *
        ]
    ),
    "hknpCompressedMeshShapeCinfo::~hknpCompressedMeshShapeCinfo": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeCinfo *
        ]
    ),
    "hknpCompressedMeshShapeCinfo::getConvexShapeTag": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hknpCompressedMeshShapeCinfo *
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdStaticMeshTree::GeometryProvider::GeometryProvider": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::GeometryProvider *
        ]
    ),
    "hkcdStaticMeshTree::GeometryProvider::~GeometryProvider": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::GeometryProvider *
        ]
    ),
    "hkcdStaticMeshTree::GeometryProvider::finalize": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::GeometryProvider *
        ]
    ),
    "hkcdStaticMeshTree::GeometryProvider::getCustomPrimitiveDataImpl": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::GeometryProvider *
            ctypes.c_int32,  # int
        ]
    ),
    "hkcdStaticMeshTree::GeometryProvider::getCustomPrimitiveInfos": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::GeometryProvider *
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::GeometryProvider::CustomPrimitiveInfos *
        ]
    ),
    "hkcdStaticMeshTree::GeometryProvider::getCustomPrimitiveVertex": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::GeometryProvider *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkcdStaticMeshTree::GeometryProvider::getIndices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::GeometryProvider *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hkcdStaticMeshTree::GeometryProvider::getVertices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::GeometryProvider *
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # hkVector4f *
        ]
    ),
    "hkcdStaticMeshTree::BuildStepUtils::QuantizersInfos::QuantizersInfos": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::BuildStepUtils::QuantizersInfos *
        ]
    ),
    "hkcdStaticMeshTree::BuildStepUtils::HalfEdge::compare": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::BuildStepUtils::HalfEdge *
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::BuildStepUtils::HalfEdge *
        ]
    ),
    "hkcdStaticMeshTree::BuildStepUtils::fetchPrimitiveIndices": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::GeometryProvider *
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::BuildStepUtils::RootToSectionData *
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::BuildStepUtils::Quad *
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_int32,  # int
            ctypes.c_ulonglong,  # int *
            ctypes.c_ulonglong,  # int *
        ]
    ),
    "hkcdStaticMeshTree::BuildStepUtils::Quad::index": FUNCDEF(
        restype=ctypes.c_int64,  # __int64
        argtypes=[
            ctypes.c_ulonglong,  # hkcdStaticMeshTree::BuildStepUtils::Quad *
            ctypes.c_ulonglong,  # const hkcdStaticMeshTree::GeometryProvider *
            ctypes.c_int32,  # int
        ]
    ),
    "asinf_0": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "log2f_0": FUNCDEF(
        restype=ctypes.c_float,  # float
        argtypes=[
            ctypes.c_float,  # float
        ]
    ),
    "sub_14280EB25": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sub_14280EB7B": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # void *
        ]
    ),
    "sub_14280EBAB": FUNCDEF(
        restype=None,  # void
        argtypes=[
            ctypes.c_int64,  # __int64
            ctypes.c_ulonglong,  # void *
        ]
    ),
}
